# pylint:disable=line-too-long
from __future__ import annotations
from collections import OrderedDict

from angr.procedures.definitions import SimTypeCollection
from angr.sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeArray,     SimTypeBottom,     SimUnion,     SimTypeBool,     SimTypeRef

typelib = SimTypeCollection()
typelib.set_names(*['win32'])
typelib.types = {
    "AAAccountingData": SimStruct(OrderedDict((("userName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("clientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("authType", SimTypeInt(signed=False, label="AAAuthSchemes")), ("resourceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("portNumber", SimTypeInt(signed=True, label="Int32")), ("protocolName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("numberOfBytesReceived", SimTypeInt(signed=True, label="Int32")), ("numberOfBytesTransfered", SimTypeInt(signed=True, label="Int32")), ("reasonForDisconnect", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("mainSessionId", SimTypeBottom(label="Guid")), ("subSessionId", SimTypeInt(signed=True, label="Int32")),)), name="AAAccountingData", pack=False, align=None),
    "AAAccountingDataType": SimTypeInt(signed=False, label="AAAccountingDataType"),
    "AAAuthSchemes": SimTypeInt(signed=False, label="AAAuthSchemes"),
    "AAL5_PARAMETERS": SimStruct(OrderedDict((("ForwardMaxCPCSSDUSize", SimTypeInt(signed=False, label="UInt32")), ("BackwardMaxCPCSSDUSize", SimTypeInt(signed=False, label="UInt32")), ("Mode", SimTypeChar(label="Byte")), ("SSCSType", SimTypeChar(label="Byte")),)), name="AAL5_PARAMETERS", pack=False, align=None),
    "AALUSER_PARAMETERS": SimStruct(OrderedDict((("UserDefined", SimTypeInt(signed=False, label="UInt32")),)), name="AALUSER_PARAMETERS", pack=False, align=None),
    "AAL_PARAMETERS_IE": SimStruct(OrderedDict((("AALType", SimTypeInt(signed=False, label="AAL_TYPE")), ("AALSpecificParameters", SimUnion({"AAL5Parameters": SimTypeRef("AAL5_PARAMETERS", SimStruct), "AALUserParameters": SimTypeRef("AALUSER_PARAMETERS", SimStruct)}, name="<anon>", label="None")),)), name="AAL_PARAMETERS_IE", pack=False, align=None),
    "AAL_TYPE": SimTypeInt(signed=False, label="AAL_TYPE"),
    "AASHELLMENUFILENAME": SimStruct(OrderedDict((("cbTotal", SimTypeShort(signed=True, label="Int16")), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("szFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AASHELLMENUFILENAME", pack=False, align=None),
    "AASHELLMENUITEM": SimStruct(OrderedDict((("lpReserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("iReserved", SimTypeInt(signed=True, label="Int32")), ("uiReserved", SimTypeInt(signed=False, label="UInt32")), ("lpName", SimTypePointer(SimTypeRef("AASHELLMENUFILENAME", SimStruct), offset=0)), ("psz", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AASHELLMENUITEM", pack=False, align=None),
    "AATrustClassID": SimTypeInt(signed=False, label="AATrustClassID"),
    "ABC": SimStruct(OrderedDict((("abcA", SimTypeInt(signed=True, label="Int32")), ("abcB", SimTypeInt(signed=False, label="UInt32")), ("abcC", SimTypeInt(signed=True, label="Int32")),)), name="ABC", pack=False, align=None),
    "ABCFLOAT": SimStruct(OrderedDict((("abcfA", SimTypeFloat(size=32)), ("abcfB", SimTypeFloat(size=32)), ("abcfC", SimTypeFloat(size=32)),)), name="ABCFLOAT", pack=False, align=None),
    "ABORTPATH": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)),)), name="ABORTPATH", pack=False, align=None),
    "ABORTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "ACCEL": SimStruct(OrderedDict((("fVirt", SimTypeInt(signed=False, label="ACCEL_VIRT_FLAGS")), ("key", SimTypeShort(signed=False, label="UInt16")), ("cmd", SimTypeShort(signed=False, label="UInt16")),)), name="ACCEL", pack=False, align=None),
    "ACCEL_VIRT_FLAGS": SimTypeInt(signed=False, label="ACCEL_VIRT_FLAGS"),
    "ACCEPT_SECURITY_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "ACCESSRECTLIST": SimStruct(OrderedDict((("lpLink", SimTypePointer(SimTypeRef("ACCESSRECTLIST", SimStruct), offset=0)), ("rDest", SimTypeRef("RECT", SimStruct)), ("lpOwner", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpSurfaceData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpHeapAliasInfo", SimTypePointer(SimTypeRef("HEAPALIASINFO", SimStruct), offset=0)),)), name="ACCESSRECTLIST", pack=False, align=None),
    "ACCESSTIMEOUT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iTimeOutMSec", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESSTIMEOUT", pack=False, align=None),
    "ACCESS_ALLOWED_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_ALLOWED_ACE", pack=False, align=None),
    "ACCESS_ALLOWED_CALLBACK_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_ALLOWED_CALLBACK_ACE", pack=False, align=None),
    "ACCESS_ALLOWED_CALLBACK_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_ALLOWED_CALLBACK_OBJECT_ACE", pack=False, align=None),
    "ACCESS_ALLOWED_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_ALLOWED_OBJECT_ACE", pack=False, align=None),
    "ACCESS_DENIED_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_DENIED_ACE", pack=False, align=None),
    "ACCESS_DENIED_CALLBACK_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_DENIED_CALLBACK_ACE", pack=False, align=None),
    "ACCESS_DENIED_CALLBACK_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_DENIED_CALLBACK_OBJECT_ACE", pack=False, align=None),
    "ACCESS_DENIED_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_DENIED_OBJECT_ACE", pack=False, align=None),
    "ACCESS_INFO_0": SimStruct(OrderedDict((("acc0_resource_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ACCESS_INFO_0", pack=False, align=None),
    "ACCESS_INFO_1": SimStruct(OrderedDict((("acc1_resource_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("acc1_attr", SimTypeInt(signed=False, label="UInt32")), ("acc1_count", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_INFO_1", pack=False, align=None),
    "ACCESS_INFO_1002": SimStruct(OrderedDict((("acc1002_attr", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_INFO_1002", pack=False, align=None),
    "ACCESS_LIST": SimStruct(OrderedDict((("acl_ugname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("acl_access", SimTypeInt(signed=False, label="UInt32")),)), name="ACCESS_LIST", pack=False, align=None),
    "ACCESS_MASKENUM": SimTypeInt(signed=False, label="ACCESS_MASKENUM"),
    "ACCESS_MODE": SimTypeInt(signed=False, label="ACCESS_MODE"),
    "ACCESS_REASONS": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)),)), name="ACCESS_REASONS", pack=False, align=None),
    "ACCESS_REASON_TYPE": SimTypeInt(signed=False, label="ACCESS_REASON_TYPE"),
    "ACCESS_STATE": SimStruct(OrderedDict((("OperationID", SimTypeRef("LUID", SimStruct)), ("SecurityEvaluated", SimTypeChar(label="Byte")), ("GenerateAudit", SimTypeChar(label="Byte")), ("GenerateOnClose", SimTypeChar(label="Byte")), ("PrivilegesAllocated", SimTypeChar(label="Byte")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("RemainingDesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("PreviouslyGrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("OriginalDesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("SubjectSecurityContext", SimTypeRef("SECURITY_SUBJECT_CONTEXT", SimStruct)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AuxData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Privileges", SimUnion({"InitialPrivilegeSet": SimTypeRef("INITIAL_PRIVILEGE_SET", SimStruct), "PrivilegeSet": SimTypeRef("PRIVILEGE_SET", SimStruct)}, name="<anon>", label="None")), ("AuditPrivileges", SimTypeChar(label="Byte")), ("ObjectName", SimTypeRef("UNICODE_STRING", SimStruct)), ("ObjectTypeName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="ACCESS_STATE", pack=False, align=None),
    "ACCOUNTINGPROPERTIES": SimTypeInt(signed=False, label="ACCOUNTINGPROPERTIES"),
    "ACCOUNT_STATE": SimTypeInt(signed=False, label="ACCOUNT_STATE"),
    "ACC_UTILITY_STATE_FLAGS": SimTypeInt(signed=False, label="ACC_UTILITY_STATE_FLAGS"),
    "ACDGROUP_EVENT": SimTypeInt(signed=False, label="ACDGROUP_EVENT"),
    "ACDQUEUE_EVENT": SimTypeInt(signed=False, label="ACDQUEUE_EVENT"),
    "ACENUMOPTION": SimTypeInt(signed=False, label="ACENUMOPTION"),
    "ACE_FLAGS": SimTypeInt(signed=False, label="ACE_FLAGS"),
    "ACE_HEADER": SimStruct(OrderedDict((("AceType", SimTypeChar(label="Byte")), ("AceFlags", SimTypeChar(label="Byte")), ("AceSize", SimTypeShort(signed=False, label="UInt16")),)), name="ACE_HEADER", pack=False, align=None),
    "ACE_REVISION": SimTypeInt(signed=False, label="ACE_REVISION"),
    "ACL": SimStruct(OrderedDict((("AclRevision", SimTypeChar(label="Byte")), ("Sbz1", SimTypeChar(label="Byte")), ("AclSize", SimTypeShort(signed=False, label="UInt16")), ("AceCount", SimTypeShort(signed=False, label="UInt16")), ("Sbz2", SimTypeShort(signed=False, label="UInt16")),)), name="ACL", pack=False, align=None),
    "ACL_INFORMATION_CLASS": SimTypeInt(signed=False, label="ACL_INFORMATION_CLASS"),
    "ACL_REVISION_INFORMATION": SimStruct(OrderedDict((("AclRevision", SimTypeInt(signed=False, label="UInt32")),)), name="ACL_REVISION_INFORMATION", pack=False, align=None),
    "ACL_SIZE_INFORMATION": SimStruct(OrderedDict((("AceCount", SimTypeInt(signed=False, label="UInt32")), ("AclBytesInUse", SimTypeInt(signed=False, label="UInt32")), ("AclBytesFree", SimTypeInt(signed=False, label="UInt32")),)), name="ACL_SIZE_INFORMATION", pack=False, align=None),
    "ACMDRIVERDETAILSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccComp", SimTypeInt(signed=False, label="UInt32")), ("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vdwACM", SimTypeInt(signed=False, label="UInt32")), ("vdwDriver", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cFormatTags", SimTypeInt(signed=False, label="UInt32")), ("cFilterTags", SimTypeInt(signed=False, label="UInt32")), ("hicon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szShortName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szLongName", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("szCopyright", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("szLicensing", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("szFeatures", SimTypeArray(SimTypeChar(label="SByte"), 512)),)), name="ACMDRIVERDETAILSA", pack=False, align=None),
    "ACMDRIVERDETAILSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccComp", SimTypeInt(signed=False, label="UInt32")), ("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vdwACM", SimTypeInt(signed=False, label="UInt32")), ("vdwDriver", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cFormatTags", SimTypeInt(signed=False, label="UInt32")), ("cFilterTags", SimTypeInt(signed=False, label="UInt32")), ("hicon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szShortName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szLongName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("szCopyright", SimTypeArray(SimTypeChar(label="Char"), 80)), ("szLicensing", SimTypeArray(SimTypeChar(label="Char"), 128)), ("szFeatures", SimTypeArray(SimTypeChar(label="Char"), 512)),)), name="ACMDRIVERDETAILSW", pack=False, align=None),
    "ACMDRIVERENUMCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "dwInstance", "fdwSupport"]), offset=0),
    "ACMDRVFORMATSUGGEST": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwSuggest", SimTypeInt(signed=False, label="UInt32")), ("pwfxSrc", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("cbwfxSrc", SimTypeInt(signed=False, label="UInt32")), ("pwfxDst", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("cbwfxDst", SimTypeInt(signed=False, label="UInt32")),)), name="ACMDRVFORMATSUGGEST", pack=False, align=None),
    "ACMDRVOPENDESCA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccComp", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("pszSectionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszAliasName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dnDevNode", SimTypeInt(signed=False, label="UInt32")),)), name="ACMDRVOPENDESCA", pack=False, align=None),
    "ACMDRVOPENDESCW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccComp", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("pszSectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszAliasName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dnDevNode", SimTypeInt(signed=False, label="UInt32")),)), name="ACMDRVOPENDESCW", pack=False, align=None),
    "ACMDRVSTREAMHEADER": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwStatus", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pbSrc", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbSrcLength", SimTypeInt(signed=False, label="UInt32")), ("cbSrcLengthUsed", SimTypeInt(signed=False, label="UInt32")), ("dwSrcUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pbDst", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbDstLength", SimTypeInt(signed=False, label="UInt32")), ("cbDstLengthUsed", SimTypeInt(signed=False, label="UInt32")), ("dwDstUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fdwConvert", SimTypeInt(signed=False, label="UInt32")), ("padshNext", SimTypePointer(SimTypeRef("ACMDRVSTREAMHEADER", SimStruct), offset=0)), ("fdwDriver", SimTypeInt(signed=False, label="UInt32")), ("dwDriver", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fdwPrepared", SimTypeInt(signed=False, label="UInt32")), ("dwPrepared", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pbPreparedSrc", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPreparedSrcLength", SimTypeInt(signed=False, label="UInt32")), ("pbPreparedDst", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPreparedDstLength", SimTypeInt(signed=False, label="UInt32")),)), name="ACMDRVSTREAMHEADER", pack=False, align=None),
    "ACMDRVSTREAMINSTANCE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pwfxSrc", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("pwfxDst", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("pwfltr", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fdwOpen", SimTypeInt(signed=False, label="UInt32")), ("fdwDriver", SimTypeInt(signed=False, label="UInt32")), ("dwDriver", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("has", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ACMDRVSTREAMINSTANCE", pack=False, align=None),
    "ACMDRVSTREAMSIZE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwSize", SimTypeInt(signed=False, label="UInt32")), ("cbSrcLength", SimTypeInt(signed=False, label="UInt32")), ("cbDstLength", SimTypeInt(signed=False, label="UInt32")),)), name="ACMDRVSTREAMSIZE", pack=False, align=None),
    "ACMFILTERCHOOSEA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwStyle", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwfltr", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("cbwfltr", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szFilterTag", SimTypeArray(SimTypeChar(label="SByte"), 48)), ("szFilter", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("pszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchName", SimTypeInt(signed=False, label="UInt32")), ("fdwEnum", SimTypeInt(signed=False, label="UInt32")), ("pwfltrEnum", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)),)), name="ACMFILTERCHOOSEA", pack=False, align=None),
    "ACMFILTERCHOOSEHOOKPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0),
    "ACMFILTERCHOOSEHOOKPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0),
    "ACMFILTERCHOOSEW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwStyle", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwfltr", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("cbwfltr", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szFilterTag", SimTypeArray(SimTypeChar(label="Char"), 48)), ("szFilter", SimTypeArray(SimTypeChar(label="Char"), 128)), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchName", SimTypeInt(signed=False, label="UInt32")), ("fdwEnum", SimTypeInt(signed=False, label="UInt32")), ("pwfltrEnum", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)),)), name="ACMFILTERCHOOSEW", pack=False, align=None),
    "ACMFILTERDETAILSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFilterIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTag", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("pwfltr", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("cbwfltr", SimTypeInt(signed=False, label="UInt32")), ("szFilter", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="ACMFILTERDETAILSA", pack=False, align=None),
    "ACMFILTERDETAILSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFilterIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTag", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("pwfltr", SimTypePointer(SimTypeRef("WAVEFILTER", SimStruct), offset=0)), ("cbwfltr", SimTypeInt(signed=False, label="UInt32")), ("szFilter", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="ACMFILTERDETAILSW", pack=False, align=None),
    "ACMFILTERENUMCBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFILTERDETAILSA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFILTERENUMCBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFILTERDETAILSW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFILTERTAGDETAILSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTagIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTag", SimTypeInt(signed=False, label="UInt32")), ("cbFilterSize", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cStandardFilters", SimTypeInt(signed=False, label="UInt32")), ("szFilterTag", SimTypeArray(SimTypeChar(label="SByte"), 48)),)), name="ACMFILTERTAGDETAILSA", pack=False, align=None),
    "ACMFILTERTAGDETAILSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTagIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTag", SimTypeInt(signed=False, label="UInt32")), ("cbFilterSize", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cStandardFilters", SimTypeInt(signed=False, label="UInt32")), ("szFilterTag", SimTypeArray(SimTypeChar(label="Char"), 48)),)), name="ACMFILTERTAGDETAILSW", pack=False, align=None),
    "ACMFILTERTAGENUMCBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFILTERTAGDETAILSA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFILTERTAGENUMCBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFILTERTAGDETAILSW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFORMATCHOOSEA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwStyle", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwfx", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("cbwfx", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szFormatTag", SimTypeArray(SimTypeChar(label="SByte"), 48)), ("szFormat", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("pszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchName", SimTypeInt(signed=False, label="UInt32")), ("fdwEnum", SimTypeInt(signed=False, label="UInt32")), ("pwfxEnum", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)),)), name="ACMFORMATCHOOSEA", pack=False, align=None),
    "ACMFORMATCHOOSEHOOKPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0),
    "ACMFORMATCHOOSEHOOKPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0),
    "ACMFORMATCHOOSEW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwStyle", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwfx", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("cbwfx", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szFormatTag", SimTypeArray(SimTypeChar(label="Char"), 48)), ("szFormat", SimTypeArray(SimTypeChar(label="Char"), 128)), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchName", SimTypeInt(signed=False, label="UInt32")), ("fdwEnum", SimTypeInt(signed=False, label="UInt32")), ("pwfxEnum", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)),)), name="ACMFORMATCHOOSEW", pack=False, align=None),
    "ACMFORMATDETAILSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFormatIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFormatTag", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("pwfx", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("cbwfx", SimTypeInt(signed=False, label="UInt32")), ("szFormat", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="ACMFORMATDETAILSA", pack=False, align=None),
    "ACMFORMATENUMCBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFORMATDETAILSA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFORMATENUMCBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("tACMFORMATDETAILSW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFORMATTAGDETAILSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFormatTagIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFormatTag", SimTypeInt(signed=False, label="UInt32")), ("cbFormatSize", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cStandardFormats", SimTypeInt(signed=False, label="UInt32")), ("szFormatTag", SimTypeArray(SimTypeChar(label="SByte"), 48)),)), name="ACMFORMATTAGDETAILSA", pack=False, align=None),
    "ACMFORMATTAGDETAILSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFormatTagIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFormatTag", SimTypeInt(signed=False, label="UInt32")), ("cbFormatSize", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cStandardFormats", SimTypeInt(signed=False, label="UInt32")), ("szFormatTag", SimTypeArray(SimTypeChar(label="Char"), 48)),)), name="ACMFORMATTAGDETAILSW", pack=False, align=None),
    "ACMFORMATTAGENUMCBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFORMATTAGDETAILSA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMFORMATTAGENUMCBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACMFORMATTAGDETAILSW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0),
    "ACMSTREAMHEADER": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fdwStatus", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pbSrc", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbSrcLength", SimTypeInt(signed=False, label="UInt32")), ("cbSrcLengthUsed", SimTypeInt(signed=False, label="UInt32")), ("dwSrcUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pbDst", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbDstLength", SimTypeInt(signed=False, label="UInt32")), ("cbDstLengthUsed", SimTypeInt(signed=False, label="UInt32")), ("dwDstUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReservedDriver", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 10)),)), name="ACMSTREAMHEADER", pack=False, align=None),
    "ACPI_DEBUGGING_DEVICE_IN_USE": SimStruct(OrderedDict((("NameSpacePathLength", SimTypeInt(signed=False, label="UInt32")), ("NameSpacePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ACPI_DEBUGGING_DEVICE_IN_USE", pack=False, align=None),
    "ACPI_INTERFACE_STANDARD": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("GpeConnectVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="KINTERRUPT_MODE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0)), ("GpeDisconnectVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("GpeEnableEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("GpeDisableEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("GpeClearStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("RegisterForDeviceNotifications", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("UnregisterForDeviceNotifications", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)),)), name="ACPI_INTERFACE_STANDARD", pack=False, align=None),
    "ACPI_INTERFACE_STANDARD2": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("GpeConnectVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="KINTERRUPT_MODE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "GpeNumber", "Mode", "Shareable", "ServiceRoutine", "ServiceContext", "ObjectContext"]), offset=0)), ("GpeDisconnectVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0)), ("GpeEnableEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0)), ("GpeDisableEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0)), ("GpeClearStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0)), ("RegisterForDeviceNotifications", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["NotificationContext", "NotifyCode"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "NotificationHandler", "NotificationContext"]), offset=0)), ("UnregisterForDeviceNotifications", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)),)), name="ACPI_INTERFACE_STANDARD2", pack=False, align=None),
    "ACPI_REAL_TIME": SimStruct(OrderedDict((("Year", SimTypeShort(signed=False, label="UInt16")), ("Month", SimTypeChar(label="Byte")), ("Day", SimTypeChar(label="Byte")), ("Hour", SimTypeChar(label="Byte")), ("Minute", SimTypeChar(label="Byte")), ("Second", SimTypeChar(label="Byte")), ("Valid", SimTypeChar(label="Byte")), ("Milliseconds", SimTypeShort(signed=False, label="UInt16")), ("TimeZone", SimTypeShort(signed=True, label="Int16")), ("DayLight", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="ACPI_REAL_TIME", pack=False, align=None),
    "ACPI_TIME_AND_ALARM_CAPABILITIES": SimStruct(OrderedDict((("AcWakeSupported", SimTypeChar(label="Byte")), ("DcWakeSupported", SimTypeChar(label="Byte")), ("S4AcWakeSupported", SimTypeChar(label="Byte")), ("S4DcWakeSupported", SimTypeChar(label="Byte")), ("S5AcWakeSupported", SimTypeChar(label="Byte")), ("S5DcWakeSupported", SimTypeChar(label="Byte")), ("S4S5WakeStatusSupported", SimTypeChar(label="Byte")), ("DeepestWakeSystemState", SimTypeInt(signed=False, label="UInt32")), ("RealTimeFeaturesSupported", SimTypeChar(label="Byte")), ("RealTimeResolution", SimTypeInt(signed=False, label="ACPI_TIME_RESOLUTION")),)), name="ACPI_TIME_AND_ALARM_CAPABILITIES", pack=False, align=None),
    "ACPI_TIME_RESOLUTION": SimTypeInt(signed=False, label="ACPI_TIME_RESOLUTION"),
    "ACQUIRE_CREDENTIALS_HANDLE_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "ACQUIRE_CREDENTIALS_HANDLE_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "ACTCTXA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpSource", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("wLangId", SimTypeShort(signed=False, label="UInt16")), ("lpAssemblyDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpResourceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpApplicationName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ACTCTXA", pack=False, align=None),
    "ACTCTXW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("wLangId", SimTypeShort(signed=False, label="UInt16")), ("lpAssemblyDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpResourceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ACTCTXW", pack=False, align=None),
    "ACTCTX_COMPATIBILITY_ELEMENT_TYPE": SimTypeInt(signed=False, label="ACTCTX_COMPATIBILITY_ELEMENT_TYPE"),
    "ACTCTX_REQUESTED_RUN_LEVEL": SimTypeInt(signed=False, label="ACTCTX_REQUESTED_RUN_LEVEL"),
    "ACTCTX_SECTION_KEYED_DATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulDataFormatVersion", SimTypeInt(signed=False, label="UInt32")), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulLength", SimTypeInt(signed=False, label="UInt32")), ("lpSectionGlobalData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSectionGlobalDataLength", SimTypeInt(signed=False, label="UInt32")), ("lpSectionBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSectionTotalLength", SimTypeInt(signed=False, label="UInt32")), ("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ulAssemblyRosterIndex", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("AssemblyMetadata", SimTypeRef("ACTCTX_SECTION_KEYED_DATA_ASSEMBLY_METADATA", SimStruct)),)), name="ACTCTX_SECTION_KEYED_DATA", pack=False, align=None),
    "ACTCTX_SECTION_KEYED_DATA_2600": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulDataFormatVersion", SimTypeInt(signed=False, label="UInt32")), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulLength", SimTypeInt(signed=False, label="UInt32")), ("lpSectionGlobalData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSectionGlobalDataLength", SimTypeInt(signed=False, label="UInt32")), ("lpSectionBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSectionTotalLength", SimTypeInt(signed=False, label="UInt32")), ("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ulAssemblyRosterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="ACTCTX_SECTION_KEYED_DATA_2600", pack=False, align=None),
    "ACTCTX_SECTION_KEYED_DATA_ASSEMBLY_METADATA": SimStruct(OrderedDict((("lpInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpSectionBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSectionLength", SimTypeInt(signed=False, label="UInt32")), ("lpSectionGlobalDataBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSectionGlobalDataLength", SimTypeInt(signed=False, label="UInt32")),)), name="ACTCTX_SECTION_KEYED_DATA_ASSEMBLY_METADATA", pack=False, align=None),
    "ACTION_HEADER": SimStruct(OrderedDict((("transport_id", SimTypeInt(signed=False, label="UInt32")), ("action_code", SimTypeShort(signed=False, label="UInt16")), ("reserved", SimTypeShort(signed=False, label="UInt16")),)), name="ACTION_HEADER", pack=False, align=None),
    "ACTIVATEFLAGS": SimTypeInt(signed=False, label="ACTIVATEFLAGS"),
    "ACTIVATEOPTIONS": SimTypeInt(signed=False, label="ACTIVATEOPTIONS"),
    "ACTIVATE_KEYBOARD_LAYOUT_FLAGS": SimTypeInt(signed=False, label="ACTIVATE_KEYBOARD_LAYOUT_FLAGS"),
    "ACTIVATION_CONTEXT_ASSEMBLY_DETAILED_INFORMATION": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulEncodedAssemblyIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("ulManifestPathType", SimTypeInt(signed=False, label="UInt32")), ("ulManifestPathLength", SimTypeInt(signed=False, label="UInt32")), ("liManifestLastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ulPolicyPathType", SimTypeInt(signed=False, label="UInt32")), ("ulPolicyPathLength", SimTypeInt(signed=False, label="UInt32")), ("liPolicyLastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ulMetadataSatelliteRosterIndex", SimTypeInt(signed=False, label="UInt32")), ("ulManifestVersionMajor", SimTypeInt(signed=False, label="UInt32")), ("ulManifestVersionMinor", SimTypeInt(signed=False, label="UInt32")), ("ulPolicyVersionMajor", SimTypeInt(signed=False, label="UInt32")), ("ulPolicyVersionMinor", SimTypeInt(signed=False, label="UInt32")), ("ulAssemblyDirectoryNameLength", SimTypeInt(signed=False, label="UInt32")), ("lpAssemblyEncodedAssemblyIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpAssemblyManifestPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpAssemblyPolicyPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpAssemblyDirectoryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFileCount", SimTypeInt(signed=False, label="UInt32")),)), name="ACTIVATION_CONTEXT_ASSEMBLY_DETAILED_INFORMATION", pack=False, align=None),
    "ACTIVATION_CONTEXT_BASIC_INFORMATION": SimStruct(OrderedDict((("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="ACTIVATION_CONTEXT_BASIC_INFORMATION", pack=False, align=None),
    "ACTIVATION_CONTEXT_COMPATIBILITY_INFORMATION": SimStruct(OrderedDict((("ElementCount", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("COMPATIBILITY_CONTEXT_ELEMENT", SimStruct), offset=0)),)), name="ACTIVATION_CONTEXT_COMPATIBILITY_INFORMATION", pack=False, align=None),
    "ACTIVATION_CONTEXT_DETAILED_INFORMATION": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ulFormatVersion", SimTypeInt(signed=False, label="UInt32")), ("ulAssemblyCount", SimTypeInt(signed=False, label="UInt32")), ("ulRootManifestPathType", SimTypeInt(signed=False, label="UInt32")), ("ulRootManifestPathChars", SimTypeInt(signed=False, label="UInt32")), ("ulRootConfigurationPathType", SimTypeInt(signed=False, label="UInt32")), ("ulRootConfigurationPathChars", SimTypeInt(signed=False, label="UInt32")), ("ulAppDirPathType", SimTypeInt(signed=False, label="UInt32")), ("ulAppDirPathChars", SimTypeInt(signed=False, label="UInt32")), ("lpRootManifestPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpRootConfigurationPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpAppDirPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ACTIVATION_CONTEXT_DETAILED_INFORMATION", pack=False, align=None),
    "ACTIVATION_CONTEXT_INFO_CLASS": SimTypeInt(signed=False, label="ACTIVATION_CONTEXT_INFO_CLASS"),
    "ACTIVATION_CONTEXT_QUERY_INDEX": SimStruct(OrderedDict((("ulAssemblyIndex", SimTypeInt(signed=False, label="UInt32")), ("ulFileIndexInAssembly", SimTypeInt(signed=False, label="UInt32")),)), name="ACTIVATION_CONTEXT_QUERY_INDEX", pack=False, align=None),
    "ACTIVATION_CONTEXT_RUN_LEVEL_INFORMATION": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("RunLevel", SimTypeInt(signed=False, label="ACTCTX_REQUESTED_RUN_LEVEL")), ("UiAccess", SimTypeInt(signed=False, label="UInt32")),)), name="ACTIVATION_CONTEXT_RUN_LEVEL_INFORMATION", pack=False, align=None),
    "ACTIVEOBJECT_FLAGS": SimTypeInt(signed=False, label="ACTIVEOBJECT_FLAGS"),
    "ACTIVE_LATENCY_CONFIGURATION": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="ACTIVE_LATENCY_CONFIGURATION", pack=False, align=None),
    "ACTIVITY_STATE": SimTypeInt(signed=False, label="ACTIVITY_STATE"),
    "ACTIVITY_STATE_COUNT": SimTypeInt(signed=False, label="ACTIVITY_STATE_COUNT"),
    "ACTRL_ACCESSA": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pPropertyAccessList", SimTypePointer(SimTypeRef("ACTRL_PROPERTY_ENTRYA", SimStruct), offset=0)),)), name="ACTRL_ACCESSA", pack=False, align=None),
    "ACTRL_ACCESSW": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pPropertyAccessList", SimTypePointer(SimTypeRef("ACTRL_PROPERTY_ENTRYW", SimStruct), offset=0)),)), name="ACTRL_ACCESSW", pack=False, align=None),
    "ACTRL_ACCESS_ENTRYA": SimStruct(OrderedDict((("Trustee", SimTypeRef("TRUSTEE_A", SimStruct)), ("fAccessFlags", SimTypeInt(signed=False, label="ACTRL_ACCESS_ENTRY_ACCESS_FLAGS")), ("Access", SimTypeInt(signed=False, label="UInt32")), ("ProvSpecificAccess", SimTypeInt(signed=False, label="UInt32")), ("Inheritance", SimTypeInt(signed=False, label="ACE_FLAGS")), ("lpInheritProperty", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ACTRL_ACCESS_ENTRYA", pack=False, align=None),
    "ACTRL_ACCESS_ENTRYW": SimStruct(OrderedDict((("Trustee", SimTypeRef("TRUSTEE_W", SimStruct)), ("fAccessFlags", SimTypeInt(signed=False, label="ACTRL_ACCESS_ENTRY_ACCESS_FLAGS")), ("Access", SimTypeInt(signed=False, label="UInt32")), ("ProvSpecificAccess", SimTypeInt(signed=False, label="UInt32")), ("Inheritance", SimTypeInt(signed=False, label="ACE_FLAGS")), ("lpInheritProperty", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ACTRL_ACCESS_ENTRYW", pack=False, align=None),
    "ACTRL_ACCESS_ENTRY_ACCESS_FLAGS": SimTypeInt(signed=False, label="ACTRL_ACCESS_ENTRY_ACCESS_FLAGS"),
    "ACTRL_ACCESS_ENTRY_LISTA": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pAccessList", SimTypePointer(SimTypeRef("ACTRL_ACCESS_ENTRYA", SimStruct), offset=0)),)), name="ACTRL_ACCESS_ENTRY_LISTA", pack=False, align=None),
    "ACTRL_ACCESS_ENTRY_LISTW": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pAccessList", SimTypePointer(SimTypeRef("ACTRL_ACCESS_ENTRYW", SimStruct), offset=0)),)), name="ACTRL_ACCESS_ENTRY_LISTW", pack=False, align=None),
    "ACTRL_ACCESS_INFOA": SimStruct(OrderedDict((("fAccessPermission", SimTypeInt(signed=False, label="UInt32")), ("lpAccessPermissionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ACTRL_ACCESS_INFOA", pack=False, align=None),
    "ACTRL_ACCESS_INFOW": SimStruct(OrderedDict((("fAccessPermission", SimTypeInt(signed=False, label="UInt32")), ("lpAccessPermissionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ACTRL_ACCESS_INFOW", pack=False, align=None),
    "ACTRL_CONTROL_INFOA": SimStruct(OrderedDict((("lpControlId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpControlName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ACTRL_CONTROL_INFOA", pack=False, align=None),
    "ACTRL_CONTROL_INFOW": SimStruct(OrderedDict((("lpControlId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpControlName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ACTRL_CONTROL_INFOW", pack=False, align=None),
    "ACTRL_OVERLAPPED": SimStruct(OrderedDict((("Anonymous", SimUnion({"Provider": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Reserved1": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("hEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ACTRL_OVERLAPPED", pack=False, align=None),
    "ACTRL_PROPERTY_ENTRYA": SimStruct(OrderedDict((("lpProperty", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pAccessEntryList", SimTypePointer(SimTypeRef("ACTRL_ACCESS_ENTRY_LISTA", SimStruct), offset=0)), ("fListFlags", SimTypeInt(signed=False, label="UInt32")),)), name="ACTRL_PROPERTY_ENTRYA", pack=False, align=None),
    "ACTRL_PROPERTY_ENTRYW": SimStruct(OrderedDict((("lpProperty", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAccessEntryList", SimTypePointer(SimTypeRef("ACTRL_ACCESS_ENTRY_LISTW", SimStruct), offset=0)), ("fListFlags", SimTypeInt(signed=False, label="UInt32")),)), name="ACTRL_PROPERTY_ENTRYW", pack=False, align=None),
    "ACT_AUTHORIZATION_STATE": SimStruct(OrderedDict((("ulState", SimTypeInt(signed=False, label="UInt32")),)), name="ACT_AUTHORIZATION_STATE", pack=False, align=None),
    "ACT_AUTHORIZATION_STATE_VALUE": SimTypeInt(signed=False, label="ACT_AUTHORIZATION_STATE_VALUE"),
    "ADAPTER_STATUS": SimStruct(OrderedDict((("adapter_address", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("rev_major", SimTypeChar(label="Byte")), ("reserved0", SimTypeChar(label="Byte")), ("adapter_type", SimTypeChar(label="Byte")), ("rev_minor", SimTypeChar(label="Byte")), ("duration", SimTypeShort(signed=False, label="UInt16")), ("frmr_recv", SimTypeShort(signed=False, label="UInt16")), ("frmr_xmit", SimTypeShort(signed=False, label="UInt16")), ("iframe_recv_err", SimTypeShort(signed=False, label="UInt16")), ("xmit_aborts", SimTypeShort(signed=False, label="UInt16")), ("xmit_success", SimTypeInt(signed=False, label="UInt32")), ("recv_success", SimTypeInt(signed=False, label="UInt32")), ("iframe_xmit_err", SimTypeShort(signed=False, label="UInt16")), ("recv_buff_unavail", SimTypeShort(signed=False, label="UInt16")), ("t1_timeouts", SimTypeShort(signed=False, label="UInt16")), ("ti_timeouts", SimTypeShort(signed=False, label="UInt16")), ("reserved1", SimTypeInt(signed=False, label="UInt32")), ("free_ncbs", SimTypeShort(signed=False, label="UInt16")), ("max_cfg_ncbs", SimTypeShort(signed=False, label="UInt16")), ("max_ncbs", SimTypeShort(signed=False, label="UInt16")), ("xmit_buf_unavail", SimTypeShort(signed=False, label="UInt16")), ("max_dgram_size", SimTypeShort(signed=False, label="UInt16")), ("pending_sess", SimTypeShort(signed=False, label="UInt16")), ("max_cfg_sess", SimTypeShort(signed=False, label="UInt16")), ("max_sess", SimTypeShort(signed=False, label="UInt16")), ("max_sess_pkt_size", SimTypeShort(signed=False, label="UInt16")), ("name_count", SimTypeShort(signed=False, label="UInt16")),)), name="ADAPTER_STATUS", pack=False, align=None),
    "ADDED_CERT_TYPE": SimTypeInt(signed=False, label="ADDED_CERT_TYPE"),
    "ADDJOB_INFO_1A": SimStruct(OrderedDict((("Path", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("JobId", SimTypeInt(signed=False, label="UInt32")),)), name="ADDJOB_INFO_1A", pack=False, align=None),
    "ADDJOB_INFO_1W": SimStruct(OrderedDict((("Path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("JobId", SimTypeInt(signed=False, label="UInt32")),)), name="ADDJOB_INFO_1W", pack=False, align=None),
    "ADDRALIAS": SimStruct(OrderedDict((("rgchName", SimTypeArray(SimTypeChar(label="SByte"), 41)), ("rgchEName", SimTypeArray(SimTypeChar(label="SByte"), 11)), ("rgchSrvr", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("dibDetail", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeShort(signed=False, label="UInt16")),)), name="ADDRALIAS", pack=False, align=None),
    "ADDRESS": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("Segment", SimTypeShort(signed=False, label="UInt16")), ("Mode", SimTypeInt(signed=False, label="ADDRESS_MODE")),)), name="ADDRESS", pack=False, align=None),
    "ADDRESS64": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Segment", SimTypeShort(signed=False, label="UInt16")), ("Mode", SimTypeInt(signed=False, label="ADDRESS_MODE")),)), name="ADDRESS64", pack=False, align=None),
    "ADDRESS_CAPABILITY": SimTypeInt(signed=False, label="ADDRESS_CAPABILITY"),
    "ADDRESS_CAPABILITY_STRING": SimTypeInt(signed=False, label="ADDRESS_CAPABILITY_STRING"),
    "ADDRESS_EVENT": SimTypeInt(signed=False, label="ADDRESS_EVENT"),
    "ADDRESS_FAMILY": SimTypeInt(signed=False, label="ADDRESS_FAMILY"),
    "ADDRESS_LIST_DESCRIPTOR": SimStruct(OrderedDict((("MediaType", SimTypeInt(signed=False, label="UInt32")), ("AddressList", SimTypeRef("NETWORK_ADDRESS_LIST", SimStruct)),)), name="ADDRESS_LIST_DESCRIPTOR", pack=False, align=None),
    "ADDRESS_MODE": SimTypeInt(signed=False, label="ADDRESS_MODE"),
    "ADDRESS_STATE": SimTypeInt(signed=False, label="ADDRESS_STATE"),
    "ADDRINFOA": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOA", SimStruct), offset=0)),)), name="ADDRINFOA", pack=False, align=None),
    "ADDRINFOEX2A": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEX2A", SimStruct), offset=0)), ("ai_version", SimTypeInt(signed=True, label="Int32")), ("ai_fqdn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADDRINFOEX2A", pack=False, align=None),
    "ADDRINFOEX2W": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEX2W", SimStruct), offset=0)), ("ai_version", SimTypeInt(signed=True, label="Int32")), ("ai_fqdn", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADDRINFOEX2W", pack=False, align=None),
    "ADDRINFOEX3": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEX3", SimStruct), offset=0)), ("ai_version", SimTypeInt(signed=True, label="Int32")), ("ai_fqdn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_interfaceindex", SimTypeInt(signed=True, label="Int32")),)), name="ADDRINFOEX3", pack=False, align=None),
    "ADDRINFOEX4": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEX4", SimStruct), offset=0)), ("ai_version", SimTypeInt(signed=True, label="Int32")), ("ai_fqdn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_interfaceindex", SimTypeInt(signed=True, label="Int32")), ("ai_resolutionhandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ADDRINFOEX4", pack=False, align=None),
    "ADDRINFOEX5": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEX5", SimStruct), offset=0)), ("ai_version", SimTypeInt(signed=True, label="Int32")), ("ai_fqdn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_interfaceindex", SimTypeInt(signed=True, label="Int32")), ("ai_resolutionhandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ai_ttl", SimTypeInt(signed=False, label="UInt32")),)), name="ADDRINFOEX5", pack=False, align=None),
    "ADDRINFOEX6": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEX5", SimStruct), offset=0)), ("ai_version", SimTypeInt(signed=True, label="Int32")), ("ai_fqdn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_interfaceindex", SimTypeInt(signed=True, label="Int32")), ("ai_resolutionhandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ai_ttl", SimTypeInt(signed=False, label="UInt32")), ("ai_numservers", SimTypeInt(signed=False, label="UInt32")), ("ai_servers", SimTypePointer(SimTypeRef("ADDRINFO_DNS_SERVER", SimStruct), offset=0)), ("ai_responseflags", SimTypeLongLong(signed=False, label="UInt64")),)), name="ADDRINFOEX6", pack=False, align=None),
    "ADDRINFOEXA": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEXA", SimStruct), offset=0)),)), name="ADDRINFOEXA", pack=False, align=None),
    "ADDRINFOEXW": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_blob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ai_bloblen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_provider", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOEXW", SimStruct), offset=0)),)), name="ADDRINFOEXW", pack=False, align=None),
    "ADDRINFOW": SimStruct(OrderedDict((("ai_flags", SimTypeInt(signed=True, label="Int32")), ("ai_family", SimTypeInt(signed=True, label="Int32")), ("ai_socktype", SimTypeInt(signed=True, label="Int32")), ("ai_protocol", SimTypeInt(signed=True, label="Int32")), ("ai_addrlen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ai_canonname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("ai_next", SimTypePointer(SimTypeRef("ADDRINFOW", SimStruct), offset=0)),)), name="ADDRINFOW", pack=False, align=None),
    "ADDRINFO_DNS_SERVER": SimStruct(OrderedDict((("ai_servertype", SimTypeInt(signed=False, label="UInt32")), ("ai_flags", SimTypeLongLong(signed=False, label="UInt64")), ("ai_addrlen", SimTypeInt(signed=False, label="UInt32")), ("ai_addr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("Anonymous", SimUnion({"ai_template": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="ADDRINFO_DNS_SERVER", pack=False, align=None),
    "ADDURL_FLAG": SimTypeInt(signed=False, label="ADDURL_FLAG"),
    "ADD_CREDENTIALS_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "ADD_CREDENTIALS_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "ADJACENT_DISPLAY_EDGES": SimTypeInt(signed=False, label="ADJACENT_DISPLAY_EDGES"),
    "ADMINISTRATOR_POWER_POLICY": SimStruct(OrderedDict((("MinSleep", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("MaxSleep", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("MinVideoTimeout", SimTypeInt(signed=False, label="UInt32")), ("MaxVideoTimeout", SimTypeInt(signed=False, label="UInt32")), ("MinSpindownTimeout", SimTypeInt(signed=False, label="UInt32")), ("MaxSpindownTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="ADMINISTRATOR_POWER_POLICY", pack=False, align=None),
    "ADMIN_OTHER_INFO": SimStruct(OrderedDict((("alrtad_errcode", SimTypeInt(signed=False, label="UInt32")), ("alrtad_numstrings", SimTypeInt(signed=False, label="UInt32")),)), name="ADMIN_OTHER_INFO", pack=False, align=None),
    "ADPCMCOEFSET": SimStruct(OrderedDict((("iCoef1", SimTypeShort(signed=True, label="Int16")), ("iCoef2", SimTypeShort(signed=True, label="Int16")),)), name="ADPCMCOEFSET", pack=False, align=None),
    "ADPCMEWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="ADPCMEWAVEFORMAT", pack=False, align=None),
    "ADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")), ("wNumCoef", SimTypeShort(signed=False, label="UInt16")), ("aCoef", SimTypePointer(SimTypeRef("ADPCMCOEFSET", SimStruct), offset=0)),)), name="ADPCMWAVEFORMAT", pack=False, align=None),
    "ADRENTRY": SimStruct(OrderedDict((("ulReserved1", SimTypeInt(signed=False, label="UInt32")), ("cValues", SimTypeInt(signed=False, label="UInt32")), ("rgPropVals", SimTypePointer(SimTypeRef("SPropValue", SimStruct), offset=0)),)), name="ADRENTRY", pack=False, align=None),
    "ADRLIST": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("aEntries", SimTypePointer(SimTypeRef("ADRENTRY", SimStruct), offset=0)),)), name="ADRLIST", pack=False, align=None),
    "ADRPARM": SimStruct(OrderedDict((("cbABContEntryID", SimTypeInt(signed=False, label="UInt32")), ("lpABContEntryID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulHelpContext", SimTypeInt(signed=False, label="UInt32")), ("lpszHelpFileName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpfnABSDI", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ulUIParam", "lpvmsg"]), offset=0)), ("lpfnDismiss", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ulUIParam", "lpvContext"]), offset=0)), ("lpvDismissContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszNewEntryTitle", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszDestWellsTitle", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("cDestFields", SimTypeInt(signed=False, label="UInt32")), ("nDestFieldFocus", SimTypeInt(signed=False, label="UInt32")), ("lppszDestTitles", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("lpulDestComps", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpContRestriction", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)), ("lpHierRestriction", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)),)), name="ADRPARM", pack=False, align=None),
    "ADSI_DIALECT_ENUM": SimTypeInt(signed=False, label="ADSI_DIALECT_ENUM"),
    "ADSPEC": SimStruct(OrderedDict((("adspec_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("adspec_body", SimTypeRef("IS_ADSPEC_BODY", SimStruct)),)), name="ADSPEC", pack=False, align=None),
    "ADSPROPERROR": SimStruct(OrderedDict((("hwndPage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszPageTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hr", SimTypeInt(signed=True, label="Int32")), ("pszError", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADSPROPERROR", pack=False, align=None),
    "ADSPROPINITPARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hr", SimTypeInt(signed=True, label="Int32")), ("pDsObj", SimTypeBottom(label="IDirectoryObject")), ("pwzCN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pWritableAttrs", SimTypePointer(SimTypeRef("ADS_ATTR_INFO", SimStruct), offset=0)),)), name="ADSPROPINITPARAMS", pack=False, align=None),
    "ADSTYPE": SimTypeInt(signed=False, label="ADSTYPE"),
    "ADSVALUE": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="ADSTYPE")), ("Anonymous", SimUnion({"DNString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseExactString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseIgnoreString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PrintableString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "NumericString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "Boolean": SimTypeInt(signed=False, label="UInt32"), "Integer": SimTypeInt(signed=False, label="UInt32"), "OctetString": SimTypeRef("ADS_OCTET_STRING", SimStruct), "UTCTime": SimTypeRef("SYSTEMTIME", SimStruct), "LargeInteger": SimTypeLongLong(signed=True, label="Int64"), "ClassName": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ProviderSpecific": SimTypeRef("ADS_PROV_SPECIFIC", SimStruct), "pCaseIgnoreList": SimTypePointer(SimTypeRef("ADS_CASEIGNORE_LIST", SimStruct), offset=0), "pOctetList": SimTypePointer(SimTypeRef("ADS_OCTET_LIST", SimStruct), offset=0), "pPath": SimTypePointer(SimTypeRef("ADS_PATH", SimStruct), offset=0), "pPostalAddress": SimTypePointer(SimTypeRef("ADS_POSTALADDRESS", SimStruct), offset=0), "Timestamp": SimTypeRef("ADS_TIMESTAMP", SimStruct), "BackLink": SimTypeRef("ADS_BACKLINK", SimStruct), "pTypedName": SimTypePointer(SimTypeRef("ADS_TYPEDNAME", SimStruct), offset=0), "Hold": SimTypeRef("ADS_HOLD", SimStruct), "pNetAddress": SimTypePointer(SimTypeRef("ADS_NETADDRESS", SimStruct), offset=0), "pReplicaPointer": SimTypePointer(SimTypeRef("ADS_REPLICAPOINTER", SimStruct), offset=0), "pFaxNumber": SimTypePointer(SimTypeRef("ADS_FAXNUMBER", SimStruct), offset=0), "Email": SimTypeRef("ADS_EMAIL", SimStruct), "SecurityDescriptor": SimTypeRef("ADS_NT_SECURITY_DESCRIPTOR", SimStruct), "pDNWithBinary": SimTypePointer(SimTypeRef("ADS_DN_WITH_BINARY", SimStruct), offset=0), "pDNWithString": SimTypePointer(SimTypeRef("ADS_DN_WITH_STRING", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="ADSVALUE", pack=False, align=None),
    "ADS_ACEFLAG_ENUM": SimTypeInt(signed=False, label="ADS_ACEFLAG_ENUM"),
    "ADS_ACETYPE_ENUM": SimTypeInt(signed=False, label="ADS_ACETYPE_ENUM"),
    "ADS_ATTR_DEF": SimStruct(OrderedDict((("pszAttrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwADsType", SimTypeInt(signed=False, label="ADSTYPE")), ("dwMinRange", SimTypeInt(signed=False, label="UInt32")), ("dwMaxRange", SimTypeInt(signed=False, label="UInt32")), ("fMultiValued", SimTypeInt(signed=True, label="Int32")),)), name="ADS_ATTR_DEF", pack=False, align=None),
    "ADS_ATTR_INFO": SimStruct(OrderedDict((("pszAttrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwControlCode", SimTypeInt(signed=False, label="UInt32")), ("dwADsType", SimTypeInt(signed=False, label="ADSTYPE")), ("pADsValues", SimTypePointer(SimTypeRef("ADSVALUE", SimStruct), offset=0)), ("dwNumValues", SimTypeInt(signed=False, label="UInt32")),)), name="ADS_ATTR_INFO", pack=False, align=None),
    "ADS_AUTHENTICATION_ENUM": SimTypeInt(signed=False, label="ADS_AUTHENTICATION_ENUM"),
    "ADS_BACKLINK": SimStruct(OrderedDict((("RemoteID", SimTypeInt(signed=False, label="UInt32")), ("ObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADS_BACKLINK", pack=False, align=None),
    "ADS_CASEIGNORE_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("ADS_CASEIGNORE_LIST", SimStruct), offset=0)), ("String", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADS_CASEIGNORE_LIST", pack=False, align=None),
    "ADS_CHASE_REFERRALS_ENUM": SimTypeInt(signed=False, label="ADS_CHASE_REFERRALS_ENUM"),
    "ADS_CLASS_DEF": SimStruct(OrderedDict((("pszClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMandatoryAttrs", SimTypeInt(signed=False, label="UInt32")), ("ppszMandatoryAttrs", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("optionalAttrs", SimTypeInt(signed=False, label="UInt32")), ("ppszOptionalAttrs", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), offset=0)), ("dwNamingAttrs", SimTypeInt(signed=False, label="UInt32")), ("ppszNamingAttrs", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), offset=0)), ("dwSuperClasses", SimTypeInt(signed=False, label="UInt32")), ("ppszSuperClasses", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), offset=0)), ("fIsContainer", SimTypeInt(signed=True, label="Int32")),)), name="ADS_CLASS_DEF", pack=False, align=None),
    "ADS_DEREFENUM": SimTypeInt(signed=False, label="ADS_DEREFENUM"),
    "ADS_DISPLAY_ENUM": SimTypeInt(signed=False, label="ADS_DISPLAY_ENUM"),
    "ADS_DN_WITH_BINARY": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("lpBinaryValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszDNString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADS_DN_WITH_BINARY", pack=False, align=None),
    "ADS_DN_WITH_STRING": SimStruct(OrderedDict((("pszStringValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDNString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADS_DN_WITH_STRING", pack=False, align=None),
    "ADS_EMAIL": SimStruct(OrderedDict((("Address", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="ADS_EMAIL", pack=False, align=None),
    "ADS_ESCAPE_MODE_ENUM": SimTypeInt(signed=False, label="ADS_ESCAPE_MODE_ENUM"),
    "ADS_FAXNUMBER": SimStruct(OrderedDict((("TelephoneNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NumberOfBits", SimTypeInt(signed=False, label="UInt32")), ("Parameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_FAXNUMBER", pack=False, align=None),
    "ADS_FLAGTYPE_ENUM": SimTypeInt(signed=False, label="ADS_FLAGTYPE_ENUM"),
    "ADS_FORMAT_ENUM": SimTypeInt(signed=False, label="ADS_FORMAT_ENUM"),
    "ADS_GROUP_TYPE_ENUM": SimTypeInt(signed=False, label="ADS_GROUP_TYPE_ENUM"),
    "ADS_HOLD": SimStruct(OrderedDict((("ObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Amount", SimTypeInt(signed=False, label="UInt32")),)), name="ADS_HOLD", pack=False, align=None),
    "ADS_NAME_INITTYPE_ENUM": SimTypeInt(signed=False, label="ADS_NAME_INITTYPE_ENUM"),
    "ADS_NAME_TYPE_ENUM": SimTypeInt(signed=False, label="ADS_NAME_TYPE_ENUM"),
    "ADS_NETADDRESS": SimStruct(OrderedDict((("AddressType", SimTypeInt(signed=False, label="UInt32")), ("AddressLength", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_NETADDRESS", pack=False, align=None),
    "ADS_NT_SECURITY_DESCRIPTOR": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("lpValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_NT_SECURITY_DESCRIPTOR", pack=False, align=None),
    "ADS_OBJECT_INFO": SimStruct(OrderedDict((("pszRDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszParentDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSchemaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADS_OBJECT_INFO", pack=False, align=None),
    "ADS_OCTET_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("ADS_OCTET_LIST", SimStruct), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_OCTET_LIST", pack=False, align=None),
    "ADS_OCTET_STRING": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("lpValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_OCTET_STRING", pack=False, align=None),
    "ADS_OPTION_ENUM": SimTypeInt(signed=False, label="ADS_OPTION_ENUM"),
    "ADS_PASSWORD_ENCODING_ENUM": SimTypeInt(signed=False, label="ADS_PASSWORD_ENCODING_ENUM"),
    "ADS_PATH": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("VolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Path", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ADS_PATH", pack=False, align=None),
    "ADS_PATHTYPE_ENUM": SimTypeInt(signed=False, label="ADS_PATHTYPE_ENUM"),
    "ADS_POSTALADDRESS": SimStruct(OrderedDict((("PostalAddress", SimTypeArray(SimTypePointer(SimTypeChar(label="Char"), offset=0), 6)),)), name="ADS_POSTALADDRESS", pack=False, align=None),
    "ADS_PREFERENCES_ENUM": SimTypeInt(signed=False, label="ADS_PREFERENCES_ENUM"),
    "ADS_PROPERTY_OPERATION_ENUM": SimTypeInt(signed=False, label="ADS_PROPERTY_OPERATION_ENUM"),
    "ADS_PROV_SPECIFIC": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("lpValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_PROV_SPECIFIC", pack=False, align=None),
    "ADS_REPLICAPOINTER": SimStruct(OrderedDict((("ServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReplicaType", SimTypeInt(signed=False, label="UInt32")), ("ReplicaNumber", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("ReplicaAddressHints", SimTypePointer(SimTypeRef("ADS_NETADDRESS", SimStruct), offset=0)),)), name="ADS_REPLICAPOINTER", pack=False, align=None),
    "ADS_RIGHTS_ENUM": SimTypeInt(signed=False, label="ADS_RIGHTS_ENUM"),
    "ADS_SCOPEENUM": SimTypeInt(signed=False, label="ADS_SCOPEENUM"),
    "ADS_SD_CONTROL_ENUM": SimTypeInt(signed=False, label="ADS_SD_CONTROL_ENUM"),
    "ADS_SD_FORMAT_ENUM": SimTypeInt(signed=False, label="ADS_SD_FORMAT_ENUM"),
    "ADS_SD_REVISION_ENUM": SimTypeInt(signed=False, label="ADS_SD_REVISION_ENUM"),
    "ADS_SEARCHPREF_ENUM": SimTypeInt(signed=False, label="ADS_SEARCHPREF_ENUM"),
    "ADS_SEARCHPREF_INFO": SimStruct(OrderedDict((("dwSearchPref", SimTypeInt(signed=False, label="ADS_SEARCHPREF_ENUM")), ("vValue", SimTypeRef("ADSVALUE", SimStruct)), ("dwStatus", SimTypeInt(signed=False, label="ADS_STATUSENUM")),)), name="ADS_SEARCHPREF_INFO", pack=False, align=None),
    "ADS_SEARCH_COLUMN": SimStruct(OrderedDict((("pszAttrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwADsType", SimTypeInt(signed=False, label="ADSTYPE")), ("pADsValues", SimTypePointer(SimTypeRef("ADSVALUE", SimStruct), offset=0)), ("dwNumValues", SimTypeInt(signed=False, label="UInt32")), ("hReserved", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ADS_SEARCH_COLUMN", pack=False, align=None),
    "ADS_SEARCH_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ADS_SECURITY_INFO_ENUM": SimTypeInt(signed=False, label="ADS_SECURITY_INFO_ENUM"),
    "ADS_SETTYPE_ENUM": SimTypeInt(signed=False, label="ADS_SETTYPE_ENUM"),
    "ADS_SORTKEY": SimStruct(OrderedDict((("pszAttrType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszReserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fReverseorder", SimTypeChar(label="Byte")),)), name="ADS_SORTKEY", pack=False, align=None),
    "ADS_STATUSENUM": SimTypeInt(signed=False, label="ADS_STATUSENUM"),
    "ADS_SYSTEMFLAG_ENUM": SimTypeInt(signed=False, label="ADS_SYSTEMFLAG_ENUM"),
    "ADS_TIMESTAMP": SimStruct(OrderedDict((("WholeSeconds", SimTypeInt(signed=False, label="UInt32")), ("EventID", SimTypeInt(signed=False, label="UInt32")),)), name="ADS_TIMESTAMP", pack=False, align=None),
    "ADS_TYPEDNAME": SimStruct(OrderedDict((("ObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Level", SimTypeInt(signed=False, label="UInt32")), ("Interval", SimTypeInt(signed=False, label="UInt32")),)), name="ADS_TYPEDNAME", pack=False, align=None),
    "ADS_USER_FLAG_ENUM": SimTypeInt(signed=False, label="ADS_USER_FLAG_ENUM"),
    "ADS_VLV": SimStruct(OrderedDict((("dwBeforeCount", SimTypeInt(signed=False, label="UInt32")), ("dwAfterCount", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwContentCount", SimTypeInt(signed=False, label="UInt32")), ("pszTarget", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwContextIDLength", SimTypeInt(signed=False, label="UInt32")), ("lpContextID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ADS_VLV", pack=False, align=None),
    "ADVANCED_FEATURE_FLAGS": SimTypeInt(signed=False, label="ADVANCED_FEATURE_FLAGS"),
    "ADVERTISEFLAGS": SimTypeInt(signed=False, label="ADVERTISEFLAGS"),
    "ADVF": SimTypeInt(signed=False, label="ADVF"),
    "ADVISE_TYPE": SimTypeInt(signed=False, label="ADVISE_TYPE"),
    "AD_GENERAL_PARAMS": SimStruct(OrderedDict((("IntServAwareHopCount", SimTypeInt(signed=False, label="UInt32")), ("PathBandwidthEstimate", SimTypeInt(signed=False, label="UInt32")), ("MinimumLatency", SimTypeInt(signed=False, label="UInt32")), ("PathMTU", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="AD_GENERAL_PARAMS", pack=False, align=None),
    "AD_GUARANTEED": SimStruct(OrderedDict((("CTotal", SimTypeInt(signed=False, label="UInt32")), ("DTotal", SimTypeInt(signed=False, label="UInt32")), ("CSum", SimTypeInt(signed=False, label="UInt32")), ("DSum", SimTypeInt(signed=False, label="UInt32")),)), name="AD_GUARANTEED", pack=False, align=None),
    "AEC_INPUT_STREAM": SimTypeInt(signed=False, label="AEC_INPUT_STREAM"),
    "AEC_SYSTEM_MODE": SimTypeInt(signed=False, label="AEC_SYSTEM_MODE"),
    "AEC_VAD_MODE": SimTypeInt(signed=False, label="AEC_VAD_MODE"),
    "AEROWIZARDPARTS": SimTypeInt(signed=False, label="AEROWIZARDPARTS"),
    "AER_BRIDGE_DESCRIPTOR_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "AER_ENDPOINT_DESCRIPTOR_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "AER_ROOTPORT_DESCRIPTOR_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "AE_ACCLIM": SimStruct(OrderedDict((("ae_al_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_al_username", SimTypeInt(signed=False, label="UInt32")), ("ae_al_resname", SimTypeInt(signed=False, label="UInt32")), ("ae_al_limit", SimTypeInt(signed=False, label="UInt32")),)), name="AE_ACCLIM", pack=False, align=None),
    "AE_ACLMOD": SimStruct(OrderedDict((("ae_am_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_am_username", SimTypeInt(signed=False, label="UInt32")), ("ae_am_resname", SimTypeInt(signed=False, label="UInt32")), ("ae_am_action", SimTypeInt(signed=False, label="UInt32")), ("ae_am_datalen", SimTypeInt(signed=False, label="UInt32")),)), name="AE_ACLMOD", pack=False, align=None),
    "AE_CLOSEFILE": SimStruct(OrderedDict((("ae_cf_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_cf_username", SimTypeInt(signed=False, label="UInt32")), ("ae_cf_resname", SimTypeInt(signed=False, label="UInt32")), ("ae_cf_fileid", SimTypeInt(signed=False, label="UInt32")), ("ae_cf_duration", SimTypeInt(signed=False, label="UInt32")), ("ae_cf_reason", SimTypeInt(signed=False, label="UInt32")),)), name="AE_CLOSEFILE", pack=False, align=None),
    "AE_CONNREJ": SimStruct(OrderedDict((("ae_cr_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_cr_username", SimTypeInt(signed=False, label="UInt32")), ("ae_cr_netname", SimTypeInt(signed=False, label="UInt32")), ("ae_cr_reason", SimTypeInt(signed=False, label="UInt32")),)), name="AE_CONNREJ", pack=False, align=None),
    "AE_CONNSTART": SimStruct(OrderedDict((("ae_ct_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_ct_username", SimTypeInt(signed=False, label="UInt32")), ("ae_ct_netname", SimTypeInt(signed=False, label="UInt32")), ("ae_ct_connid", SimTypeInt(signed=False, label="UInt32")),)), name="AE_CONNSTART", pack=False, align=None),
    "AE_CONNSTOP": SimStruct(OrderedDict((("ae_cp_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_cp_username", SimTypeInt(signed=False, label="UInt32")), ("ae_cp_netname", SimTypeInt(signed=False, label="UInt32")), ("ae_cp_connid", SimTypeInt(signed=False, label="UInt32")), ("ae_cp_reason", SimTypeInt(signed=False, label="UInt32")),)), name="AE_CONNSTOP", pack=False, align=None),
    "AE_CURRENT_POSITION": SimStruct(OrderedDict((("u64DevicePosition", SimTypeLongLong(signed=False, label="UInt64")), ("u64StreamPosition", SimTypeLongLong(signed=False, label="UInt64")), ("u64PaddingFrames", SimTypeLongLong(signed=False, label="UInt64")), ("hnsQPCPosition", SimTypeLongLong(signed=True, label="Int64")), ("f32FramesPerSecond", SimTypeFloat(size=32)), ("Flag", SimTypeInt(signed=False, label="AE_POSITION_FLAGS")),)), name="AE_CURRENT_POSITION", pack=False, align=None),
    "AE_GENERIC": SimStruct(OrderedDict((("ae_ge_msgfile", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_msgnum", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_params", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param1", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param2", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param3", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param4", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param5", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param6", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param7", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param8", SimTypeInt(signed=False, label="UInt32")), ("ae_ge_param9", SimTypeInt(signed=False, label="UInt32")),)), name="AE_GENERIC", pack=False, align=None),
    "AE_LOCKOUT": SimStruct(OrderedDict((("ae_lk_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_lk_username", SimTypeInt(signed=False, label="UInt32")), ("ae_lk_action", SimTypeInt(signed=False, label="UInt32")), ("ae_lk_bad_pw_count", SimTypeInt(signed=False, label="UInt32")),)), name="AE_LOCKOUT", pack=False, align=None),
    "AE_NETLOGOFF": SimStruct(OrderedDict((("ae_nf_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_nf_username", SimTypeInt(signed=False, label="UInt32")), ("ae_nf_reserved1", SimTypeInt(signed=False, label="UInt32")), ("ae_nf_reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="AE_NETLOGOFF", pack=False, align=None),
    "AE_NETLOGON": SimStruct(OrderedDict((("ae_no_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_no_username", SimTypeInt(signed=False, label="UInt32")), ("ae_no_privilege", SimTypeInt(signed=False, label="UInt32")), ("ae_no_authflags", SimTypeInt(signed=False, label="UInt32")),)), name="AE_NETLOGON", pack=False, align=None),
    "AE_POSITION_FLAGS": SimTypeInt(signed=False, label="AE_POSITION_FLAGS"),
    "AE_RESACCESS": SimStruct(OrderedDict((("ae_ra_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_ra_username", SimTypeInt(signed=False, label="UInt32")), ("ae_ra_resname", SimTypeInt(signed=False, label="UInt32")), ("ae_ra_operation", SimTypeInt(signed=False, label="UInt32")), ("ae_ra_returncode", SimTypeInt(signed=False, label="UInt32")), ("ae_ra_restype", SimTypeInt(signed=False, label="UInt32")), ("ae_ra_fileid", SimTypeInt(signed=False, label="UInt32")),)), name="AE_RESACCESS", pack=False, align=None),
    "AE_RESACCESSREJ": SimStruct(OrderedDict((("ae_rr_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_rr_username", SimTypeInt(signed=False, label="UInt32")), ("ae_rr_resname", SimTypeInt(signed=False, label="UInt32")), ("ae_rr_operation", SimTypeInt(signed=False, label="UInt32")),)), name="AE_RESACCESSREJ", pack=False, align=None),
    "AE_SERVICESTAT": SimStruct(OrderedDict((("ae_ss_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_ss_username", SimTypeInt(signed=False, label="UInt32")), ("ae_ss_svcname", SimTypeInt(signed=False, label="UInt32")), ("ae_ss_status", SimTypeInt(signed=False, label="UInt32")), ("ae_ss_code", SimTypeInt(signed=False, label="UInt32")), ("ae_ss_text", SimTypeInt(signed=False, label="UInt32")), ("ae_ss_returnval", SimTypeInt(signed=False, label="UInt32")),)), name="AE_SERVICESTAT", pack=False, align=None),
    "AE_SESSLOGOFF": SimStruct(OrderedDict((("ae_sf_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_sf_username", SimTypeInt(signed=False, label="UInt32")), ("ae_sf_reason", SimTypeInt(signed=False, label="UInt32")),)), name="AE_SESSLOGOFF", pack=False, align=None),
    "AE_SESSLOGON": SimStruct(OrderedDict((("ae_so_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_so_username", SimTypeInt(signed=False, label="UInt32")), ("ae_so_privilege", SimTypeInt(signed=False, label="UInt32")),)), name="AE_SESSLOGON", pack=False, align=None),
    "AE_SESSPWERR": SimStruct(OrderedDict((("ae_sp_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_sp_username", SimTypeInt(signed=False, label="UInt32")),)), name="AE_SESSPWERR", pack=False, align=None),
    "AE_SRVSTATUS": SimStruct(OrderedDict((("ae_sv_status", SimTypeInt(signed=False, label="UInt32")),)), name="AE_SRVSTATUS", pack=False, align=None),
    "AE_UASMOD": SimStruct(OrderedDict((("ae_um_compname", SimTypeInt(signed=False, label="UInt32")), ("ae_um_username", SimTypeInt(signed=False, label="UInt32")), ("ae_um_resname", SimTypeInt(signed=False, label="UInt32")), ("ae_um_rectype", SimTypeInt(signed=False, label="UInt32")), ("ae_um_action", SimTypeInt(signed=False, label="UInt32")), ("ae_um_datalen", SimTypeInt(signed=False, label="UInt32")),)), name="AE_UASMOD", pack=False, align=None),
    "AFPROTOCOLS": SimStruct(OrderedDict((("iAddressFamily", SimTypeInt(signed=True, label="Int32")), ("iProtocol", SimTypeInt(signed=True, label="Int32")),)), name="AFPROTOCOLS", pack=False, align=None),
    "AF_OP": SimTypeInt(signed=False, label="AF_OP"),
    "AGENTHANDLER_EVENT": SimTypeInt(signed=False, label="AGENTHANDLER_EVENT"),
    "AGENT_EVENT": SimTypeInt(signed=False, label="AGENT_EVENT"),
    "AGENT_SESSION_EVENT": SimTypeInt(signed=False, label="AGENT_SESSION_EVENT"),
    "AGENT_SESSION_STATE": SimTypeInt(signed=False, label="AGENT_SESSION_STATE"),
    "AGENT_STATE": SimTypeInt(signed=False, label="AGENT_STATE"),
    "AGP_TARGET_BUS_INTERFACE_STANDARD": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetBusData", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("GetBusData", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("CapabilityID", SimTypeChar(label="Byte")),)), name="AGP_TARGET_BUS_INTERFACE_STANDARD", pack=False, align=None),
    "AHE_TYPE": SimTypeInt(signed=False, label="AHE_TYPE"),
    "AHTYPE": SimTypeInt(signed=False, label="AHTYPE"),
    "ALERT_SYSTEM_SEV": SimTypeInt(signed=False, label="ALERT_SYSTEM_SEV"),
    "ALG_ID": SimTypeInt(signed=False, label="ALG_ID"),
    "ALLOCATE_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="POOL_TYPE"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PoolType", "NumberOfBytes", "Tag"]), offset=0),
    "ALLOCATE_VIRTUAL_MEMORY_EX_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MEM_EXTENDED_PARAMETER", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackContext", "ProcessHandle", "BaseAddress", "RegionSize", "AllocationType", "PageProtection", "ExtendedParameters", "ExtendedParameterCount"]), offset=0),
    "ALLOCATOR_PROPERTIES": SimStruct(OrderedDict((("cBuffers", SimTypeInt(signed=True, label="Int32")), ("cbBuffer", SimTypeInt(signed=True, label="Int32")), ("cbAlign", SimTypeInt(signed=True, label="Int32")), ("cbPrefix", SimTypeInt(signed=True, label="Int32")),)), name="ALLOCATOR_PROPERTIES", pack=False, align=None),
    "ALLOCATOR_PROPERTIES_EX": SimStruct(OrderedDict((("cBuffers", SimTypeInt(signed=True, label="Int32")), ("cbBuffer", SimTypeInt(signed=True, label="Int32")), ("cbAlign", SimTypeInt(signed=True, label="Int32")), ("cbPrefix", SimTypeInt(signed=True, label="Int32")), ("MemoryType", SimTypeBottom(label="Guid")), ("BusType", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="PIPE_STATE")), ("Input", SimTypeRef("PIPE_TERMINATION", SimStruct)), ("Output", SimTypeRef("PIPE_TERMINATION", SimStruct)), ("Strategy", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Weight", SimTypeInt(signed=False, label="UInt32")), ("LogicalMemoryType", SimTypeInt(signed=False, label="KS_LogicalMemoryType")), ("AllocatorPlace", SimTypeInt(signed=False, label="PIPE_ALLOCATOR_PLACE")), ("Dimensions", SimTypeRef("PIPE_DIMENSIONS", SimStruct)), ("PhysicalRange", SimTypeRef("KS_FRAMING_RANGE", SimStruct)), ("PrevSegment", SimTypeBottom(label="IKsAllocatorEx")), ("CountNextSegments", SimTypeInt(signed=False, label="UInt32")), ("NextSegments", SimTypePointer(SimTypeBottom(label="IKsAllocatorEx"), offset=0)), ("InsideFactors", SimTypeInt(signed=False, label="UInt32")), ("NumberPins", SimTypeInt(signed=False, label="UInt32")),)), name="ALLOCATOR_PROPERTIES_EX", pack=False, align=None),
    "ALTERNATE_INTERFACE": SimStruct(OrderedDict((("InterfaceNumber", SimTypeShort(signed=False, label="UInt16")), ("AlternateInterfaceNumber", SimTypeShort(signed=False, label="UInt16")),)), name="ALTERNATE_INTERFACE", pack=False, align=None),
    "ALTERNATIVE_ARCHITECTURE_TYPE": SimTypeInt(signed=False, label="ALTERNATIVE_ARCHITECTURE_TYPE"),
    "ALTTABINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cItems", SimTypeInt(signed=True, label="Int32")), ("cColumns", SimTypeInt(signed=True, label="Int32")), ("cRows", SimTypeInt(signed=True, label="Int32")), ("iColFocus", SimTypeInt(signed=True, label="Int32")), ("iRowFocus", SimTypeInt(signed=True, label="Int32")), ("cxItem", SimTypeInt(signed=True, label="Int32")), ("cyItem", SimTypeInt(signed=True, label="Int32")), ("ptStart", SimTypeRef("POINT", SimStruct)),)), name="ALTTABINFO", pack=False, align=None),
    "ALT_BREAKS": SimTypeInt(signed=False, label="ALT_BREAKS"),
    "AMBISONICS_CHANNEL_ORDERING": SimTypeInt(signed=False, label="AMBISONICS_CHANNEL_ORDERING"),
    "AMBISONICS_NORMALIZATION": SimTypeInt(signed=False, label="AMBISONICS_NORMALIZATION"),
    "AMBISONICS_PARAMS": SimStruct(OrderedDict((("u32Size", SimTypeInt(signed=False, label="UInt32")), ("u32Version", SimTypeInt(signed=False, label="UInt32")), ("u32Type", SimTypeInt(signed=False, label="AMBISONICS_TYPE")), ("u32ChannelOrdering", SimTypeInt(signed=False, label="AMBISONICS_CHANNEL_ORDERING")), ("u32Normalization", SimTypeInt(signed=False, label="AMBISONICS_NORMALIZATION")), ("u32Order", SimTypeInt(signed=False, label="UInt32")), ("u32NumChannels", SimTypeInt(signed=False, label="UInt32")), ("pu32ChannelMap", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="AMBISONICS_PARAMS", pack=False, align=None),
    "AMBISONICS_TYPE": SimTypeInt(signed=False, label="AMBISONICS_TYPE"),
    "AMCOPPCommand": SimStruct(OrderedDict((("macKDI", SimTypeBottom(label="Guid")), ("guidCommandID", SimTypeBottom(label="Guid")), ("dwSequence", SimTypeInt(signed=False, label="UInt32")), ("cbSizeData", SimTypeInt(signed=False, label="UInt32")), ("CommandData", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="AMCOPPCommand", pack=False, align=None),
    "AMCOPPSignature": SimStruct(OrderedDict((("Signature", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="AMCOPPSignature", pack=False, align=None),
    "AMCOPPStatusInput": SimStruct(OrderedDict((("rApp", SimTypeBottom(label="Guid")), ("guidStatusRequestID", SimTypeBottom(label="Guid")), ("dwSequence", SimTypeInt(signed=False, label="UInt32")), ("cbSizeData", SimTypeInt(signed=False, label="UInt32")), ("StatusData", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="AMCOPPStatusInput", pack=False, align=None),
    "AMCOPPStatusOutput": SimStruct(OrderedDict((("macKDI", SimTypeBottom(label="Guid")), ("cbSizeData", SimTypeInt(signed=False, label="UInt32")), ("COPPStatus", SimTypeArray(SimTypeChar(label="Byte"), 4076)),)), name="AMCOPPStatusOutput", pack=False, align=None),
    "AMD_L1_CACHE_INFO": SimUnion({"Ulong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("LineSize", SimTypeChar(label="Byte")), ("LinesPerTag", SimTypeChar(label="Byte")), ("Associativity", SimTypeChar(label="Byte")), ("Size", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "AMD_L2_CACHE_INFO": SimUnion({"Ulong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("LineSize", SimTypeChar(label="Byte")), ("_bitfield", SimTypeChar(label="Byte")), ("Size", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "AMD_L3_CACHE_INFO": SimUnion({"Ulong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("LineSize", SimTypeChar(label="Byte")), ("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "AMExtendedSeekingCapabilities": SimTypeInt(signed=False, label="AMExtendedSeekingCapabilities"),
    "AMGETERRORTEXTPROCA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "AMGETERRORTEXTPROCW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "AMMSF_MMS_INIT_FLAGS": SimTypeInt(signed=False, label="AMMSF_MMS_INIT_FLAGS"),
    "AMMSF_MS_FLAGS": SimTypeInt(signed=False, label="AMMSF_MS_FLAGS"),
    "AMMSF_RENDER_FLAGS": SimTypeInt(signed=False, label="AMMSF_RENDER_FLAGS"),
    "AMOVERLAYFX": SimTypeInt(signed=False, label="AMOVERLAYFX"),
    "AMPROPERTY_PIN": SimTypeInt(signed=False, label="AMPROPERTY_PIN"),
    "AMPlayListEventFlags": SimTypeInt(signed=False, label="AMPlayListEventFlags"),
    "AMPlayListFlags": SimTypeInt(signed=False, label="AMPlayListFlags"),
    "AMPlayListItemFlags": SimTypeInt(signed=False, label="AMPlayListItemFlags"),
    "AMSI_ATTRIBUTE": SimTypeInt(signed=False, label="AMSI_ATTRIBUTE"),
    "AMSI_RESULT": SimTypeInt(signed=False, label="AMSI_RESULT"),
    "AMSI_UAC_MSI_ACTION": SimTypeInt(signed=False, label="AMSI_UAC_MSI_ACTION"),
    "AMSI_UAC_REQUEST_AX_INFO": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("lpwszLocalInstallPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszSourceURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AMSI_UAC_REQUEST_AX_INFO", pack=False, align=None),
    "AMSI_UAC_REQUEST_COM_INFO": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("lpwszServerBinary", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszRequestor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Clsid", SimTypeBottom(label="Guid")),)), name="AMSI_UAC_REQUEST_COM_INFO", pack=False, align=None),
    "AMSI_UAC_REQUEST_CONTEXT": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("ulRequestorProcessId", SimTypeInt(signed=False, label="UInt32")), ("UACTrustState", SimTypeInt(signed=False, label="AMSI_UAC_TRUST_STATE")), ("Type", SimTypeInt(signed=False, label="AMSI_UAC_REQUEST_TYPE")), ("RequestType", SimUnion({"ExeInfo": SimTypeRef("AMSI_UAC_REQUEST_EXE_INFO", SimStruct), "ComInfo": SimTypeRef("AMSI_UAC_REQUEST_COM_INFO", SimStruct), "MsiInfo": SimTypeRef("AMSI_UAC_REQUEST_MSI_INFO", SimStruct), "ActiveXInfo": SimTypeRef("AMSI_UAC_REQUEST_AX_INFO", SimStruct), "PackagedAppInfo": SimTypeRef("AMSI_UAC_REQUEST_PACKAGED_APP_INFO", SimStruct)}, name="<anon>", label="None")), ("bAutoElevateRequest", SimTypeInt(signed=True, label="Int32")),)), name="AMSI_UAC_REQUEST_CONTEXT", pack=False, align=None),
    "AMSI_UAC_REQUEST_EXE_INFO": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("lpwszApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszCommandLine", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszDLLParameter", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AMSI_UAC_REQUEST_EXE_INFO", pack=False, align=None),
    "AMSI_UAC_REQUEST_MSI_INFO": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("MsiAction", SimTypeInt(signed=False, label="AMSI_UAC_MSI_ACTION")), ("lpwszProductName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszManufacturer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszPackagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszPackageSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulUpdates", SimTypeInt(signed=False, label="UInt32")), ("ppwszUpdates", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("ppwszUpdateSources", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="AMSI_UAC_REQUEST_MSI_INFO", pack=False, align=None),
    "AMSI_UAC_REQUEST_PACKAGED_APP_INFO": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("lpwszApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszCommandLine", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpPackageFamilyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpApplicationId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AMSI_UAC_REQUEST_PACKAGED_APP_INFO", pack=False, align=None),
    "AMSI_UAC_REQUEST_TYPE": SimTypeInt(signed=False, label="AMSI_UAC_REQUEST_TYPE"),
    "AMSI_UAC_TRUST_STATE": SimTypeInt(signed=False, label="AMSI_UAC_TRUST_STATE"),
    "AMTVAudioEventType": SimTypeInt(signed=False, label="AMTVAudioEventType"),
    "AMTunerEventType": SimTypeInt(signed=False, label="AMTunerEventType"),
    "AMTunerModeType": SimTypeInt(signed=False, label="AMTunerModeType"),
    "AMTunerSignalStrength": SimTypeInt(signed=False, label="AMTunerSignalStrength"),
    "AMTunerSubChannel": SimTypeInt(signed=False, label="AMTunerSubChannel"),
    "AMVABUFFERINFO": SimStruct(OrderedDict((("dwTypeIndex", SimTypeInt(signed=False, label="UInt32")), ("dwBufferIndex", SimTypeInt(signed=False, label="UInt32")), ("dwDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="AMVABUFFERINFO", pack=False, align=None),
    "AMVABeginFrameInfo": SimStruct(OrderedDict((("dwDestSurfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwSizeInputData", SimTypeInt(signed=False, label="UInt32")), ("pOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwSizeOutputData", SimTypeInt(signed=False, label="UInt32")),)), name="AMVABeginFrameInfo", pack=False, align=None),
    "AMVACompBufferInfo": SimStruct(OrderedDict((("dwNumCompBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwWidthToCreate", SimTypeInt(signed=False, label="UInt32")), ("dwHeightToCreate", SimTypeInt(signed=False, label="UInt32")), ("dwBytesToAllocate", SimTypeInt(signed=False, label="UInt32")), ("ddCompCaps", SimTypeRef("DDSCAPS2", SimStruct)), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="AMVACompBufferInfo", pack=False, align=None),
    "AMVAEndFrameInfo": SimStruct(OrderedDict((("dwSizeMiscData", SimTypeInt(signed=False, label="UInt32")), ("pMiscData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="AMVAEndFrameInfo", pack=False, align=None),
    "AMVAInternalMemInfo": SimStruct(OrderedDict((("dwScratchMemAlloc", SimTypeInt(signed=False, label="UInt32")),)), name="AMVAInternalMemInfo", pack=False, align=None),
    "AMVAUncompBufferInfo": SimStruct(OrderedDict((("dwMinNumSurfaces", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumSurfaces", SimTypeInt(signed=False, label="UInt32")), ("ddUncompPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="AMVAUncompBufferInfo", pack=False, align=None),
    "AMVAUncompDataInfo": SimStruct(OrderedDict((("dwUncompWidth", SimTypeInt(signed=False, label="UInt32")), ("dwUncompHeight", SimTypeInt(signed=False, label="UInt32")), ("ddUncompPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="AMVAUncompDataInfo", pack=False, align=None),
    "AMVPDATAINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMicrosecondsPerField", SimTypeInt(signed=False, label="UInt32")), ("amvpDimInfo", SimTypeRef("AMVPDIMINFO", SimStruct)), ("dwPictAspectRatioX", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioY", SimTypeInt(signed=False, label="UInt32")), ("bEnableDoubleClock", SimTypeInt(signed=True, label="Int32")), ("bEnableVACT", SimTypeInt(signed=True, label="Int32")), ("bDataIsInterlaced", SimTypeInt(signed=True, label="Int32")), ("lHalfLinesOdd", SimTypeInt(signed=True, label="Int32")), ("bFieldPolarityInverted", SimTypeInt(signed=True, label="Int32")), ("dwNumLinesInVREF", SimTypeInt(signed=False, label="UInt32")), ("lHalfLinesEven", SimTypeInt(signed=True, label="Int32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")),)), name="AMVPDATAINFO", pack=False, align=None),
    "AMVPDIMINFO": SimStruct(OrderedDict((("dwFieldWidth", SimTypeInt(signed=False, label="UInt32")), ("dwFieldHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVBIWidth", SimTypeInt(signed=False, label="UInt32")), ("dwVBIHeight", SimTypeInt(signed=False, label="UInt32")), ("rcValidRegion", SimTypeRef("RECT", SimStruct)),)), name="AMVPDIMINFO", pack=False, align=None),
    "AMVPSIZE": SimStruct(OrderedDict((("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")),)), name="AMVPSIZE", pack=False, align=None),
    "AMVP_MODE": SimTypeInt(signed=False, label="AMVP_MODE"),
    "AMVP_SELECT_FORMAT_BY": SimTypeInt(signed=False, label="AMVP_SELECT_FORMAT_BY"),
    "AM_AC3_ALTERNATE_AUDIO": SimStruct(OrderedDict((("fStereo", SimTypeInt(signed=True, label="Int32")), ("DualMode", SimTypeInt(signed=False, label="UInt32")),)), name="AM_AC3_ALTERNATE_AUDIO", pack=False, align=None),
    "AM_AC3_BIT_STREAM_MODE": SimStruct(OrderedDict((("BitStreamMode", SimTypeInt(signed=True, label="Int32")),)), name="AM_AC3_BIT_STREAM_MODE", pack=False, align=None),
    "AM_AC3_DIALOGUE_LEVEL": SimStruct(OrderedDict((("DialogueLevel", SimTypeInt(signed=False, label="UInt32")),)), name="AM_AC3_DIALOGUE_LEVEL", pack=False, align=None),
    "AM_AC3_DOWNMIX": SimStruct(OrderedDict((("fDownMix", SimTypeInt(signed=True, label="Int32")), ("fDolbySurround", SimTypeInt(signed=True, label="Int32")),)), name="AM_AC3_DOWNMIX", pack=False, align=None),
    "AM_AC3_ERROR_CONCEALMENT": SimStruct(OrderedDict((("fRepeatPreviousBlock", SimTypeInt(signed=True, label="Int32")), ("fErrorInCurrentBlock", SimTypeInt(signed=True, label="Int32")),)), name="AM_AC3_ERROR_CONCEALMENT", pack=False, align=None),
    "AM_AC3_ROOM_TYPE": SimStruct(OrderedDict((("fLargeRoom", SimTypeInt(signed=True, label="Int32")),)), name="AM_AC3_ROOM_TYPE", pack=False, align=None),
    "AM_ASPECT_RATIO_MODE": SimTypeInt(signed=False, label="AM_ASPECT_RATIO_MODE"),
    "AM_COLCON": SimStruct(OrderedDict((("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("_bitfield3", SimTypeChar(label="Byte")), ("_bitfield4", SimTypeChar(label="Byte")),)), name="AM_COLCON", pack=False, align=None),
    "AM_COPY_MACROVISION": SimStruct(OrderedDict((("MACROVISIONLevel", SimTypeInt(signed=False, label="UInt32")),)), name="AM_COPY_MACROVISION", pack=False, align=None),
    "AM_COPY_MACROVISION_LEVEL": SimTypeInt(signed=False, label="AM_COPY_MACROVISION_LEVEL"),
    "AM_DIGITAL_CP": SimTypeInt(signed=False, label="AM_DIGITAL_CP"),
    "AM_DVDCOPYSTATE": SimTypeInt(signed=False, label="AM_DVDCOPYSTATE"),
    "AM_DVDCOPY_BUSKEY": SimStruct(OrderedDict((("BusKey", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("Reserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="AM_DVDCOPY_BUSKEY", pack=False, align=None),
    "AM_DVDCOPY_CHLGKEY": SimStruct(OrderedDict((("ChlgKey", SimTypeArray(SimTypeChar(label="Byte"), 10)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="AM_DVDCOPY_CHLGKEY", pack=False, align=None),
    "AM_DVDCOPY_DISCKEY": SimStruct(OrderedDict((("DiscKey", SimTypeArray(SimTypeChar(label="Byte"), 2048)),)), name="AM_DVDCOPY_DISCKEY", pack=False, align=None),
    "AM_DVDCOPY_SET_COPY_STATE": SimStruct(OrderedDict((("DVDCopyState", SimTypeInt(signed=False, label="UInt32")),)), name="AM_DVDCOPY_SET_COPY_STATE", pack=False, align=None),
    "AM_DVDCOPY_TITLEKEY": SimStruct(OrderedDict((("KeyFlags", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("TitleKey", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="AM_DVDCOPY_TITLEKEY", pack=False, align=None),
    "AM_DVD_ChangeRate": SimStruct(OrderedDict((("StartInTime", SimTypeLongLong(signed=True, label="Int64")), ("StartOutTime", SimTypeLongLong(signed=True, label="Int64")), ("Rate", SimTypeInt(signed=True, label="Int32")),)), name="AM_DVD_ChangeRate", pack=False, align=None),
    "AM_DVD_GRAPH_FLAGS": SimTypeInt(signed=False, label="AM_DVD_GRAPH_FLAGS"),
    "AM_DVD_RENDERSTATUS": SimStruct(OrderedDict((("hrVPEStatus", SimTypeInt(signed=True, label="Int32")), ("bDvdVolInvalid", SimTypeInt(signed=True, label="Int32")), ("bDvdVolUnknown", SimTypeInt(signed=True, label="Int32")), ("bNoLine21In", SimTypeInt(signed=True, label="Int32")), ("bNoLine21Out", SimTypeInt(signed=True, label="Int32")), ("iNumStreams", SimTypeInt(signed=True, label="Int32")), ("iNumStreamsFailed", SimTypeInt(signed=True, label="Int32")), ("dwFailedStreamsFlag", SimTypeInt(signed=False, label="UInt32")),)), name="AM_DVD_RENDERSTATUS", pack=False, align=None),
    "AM_DVD_STREAM_FLAGS": SimTypeInt(signed=False, label="AM_DVD_STREAM_FLAGS"),
    "AM_DVD_YUV": SimStruct(OrderedDict((("Reserved", SimTypeChar(label="Byte")), ("Y", SimTypeChar(label="Byte")), ("U", SimTypeChar(label="Byte")), ("V", SimTypeChar(label="Byte")),)), name="AM_DVD_YUV", pack=False, align=None),
    "AM_DvdKaraokeData": SimStruct(OrderedDict((("dwDownmix", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerAssignment", SimTypeInt(signed=False, label="UInt32")),)), name="AM_DvdKaraokeData", pack=False, align=None),
    "AM_ExactRateChange": SimStruct(OrderedDict((("OutputZeroTime", SimTypeLongLong(signed=True, label="Int64")), ("Rate", SimTypeInt(signed=True, label="Int32")),)), name="AM_ExactRateChange", pack=False, align=None),
    "AM_FILESINK_FLAGS": SimTypeInt(signed=False, label="AM_FILESINK_FLAGS"),
    "AM_FILTER_FLAGS": SimTypeInt(signed=False, label="AM_FILTER_FLAGS"),
    "AM_FRAMESTEP_STEP": SimStruct(OrderedDict((("dwFramesToStep", SimTypeInt(signed=False, label="UInt32")),)), name="AM_FRAMESTEP_STEP", pack=False, align=None),
    "AM_GRAPH_CONFIG_RECONNECT_FLAGS": SimTypeInt(signed=False, label="AM_GRAPH_CONFIG_RECONNECT_FLAGS"),
    "AM_LINE21_CCLEVEL": SimTypeInt(signed=False, label="AM_LINE21_CCLEVEL"),
    "AM_LINE21_CCSERVICE": SimTypeInt(signed=False, label="AM_LINE21_CCSERVICE"),
    "AM_LINE21_CCSTATE": SimTypeInt(signed=False, label="AM_LINE21_CCSTATE"),
    "AM_LINE21_CCSTYLE": SimTypeInt(signed=False, label="AM_LINE21_CCSTYLE"),
    "AM_LINE21_DRAWBGMODE": SimTypeInt(signed=False, label="AM_LINE21_DRAWBGMODE"),
    "AM_MEDIAEVENT_FLAGS": SimTypeInt(signed=False, label="AM_MEDIAEVENT_FLAGS"),
    "AM_MEDIA_TYPE": SimStruct(OrderedDict((("majortype", SimTypeBottom(label="Guid")), ("subtype", SimTypeBottom(label="Guid")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("lSampleSize", SimTypeInt(signed=False, label="UInt32")), ("formattype", SimTypeBottom(label="Guid")), ("pUnk", SimTypeBottom(label="IUnknown")), ("cbFormat", SimTypeInt(signed=False, label="UInt32")), ("pbFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="AM_MEDIA_TYPE", pack=False, align=None),
    "AM_MPEG2Level": SimTypeInt(signed=False, label="AM_MPEG2Level"),
    "AM_MPEG2Profile": SimTypeInt(signed=False, label="AM_MPEG2Profile"),
    "AM_MPEGSTREAMTYPE": SimStruct(OrderedDict((("dwStreamId", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("mt", SimTypeRef("AM_MEDIA_TYPE", SimStruct)), ("bFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="AM_MPEGSTREAMTYPE", pack=False, align=None),
    "AM_MPEGSYSTEMTYPE": SimStruct(OrderedDict((("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("cStreams", SimTypeInt(signed=False, label="UInt32")), ("Streams", SimTypePointer(SimTypeRef("AM_MPEGSTREAMTYPE", SimStruct), offset=0)),)), name="AM_MPEGSYSTEMTYPE", pack=False, align=None),
    "AM_PROPERTY_AC3": SimTypeInt(signed=False, label="AM_PROPERTY_AC3"),
    "AM_PROPERTY_DVDCOPYPROT": SimTypeInt(signed=False, label="AM_PROPERTY_DVDCOPYPROT"),
    "AM_PROPERTY_DVDKARAOKE": SimTypeInt(signed=False, label="AM_PROPERTY_DVDKARAOKE"),
    "AM_PROPERTY_DVDSUBPIC": SimTypeInt(signed=False, label="AM_PROPERTY_DVDSUBPIC"),
    "AM_PROPERTY_DVD_RATE_CHANGE": SimTypeInt(signed=False, label="AM_PROPERTY_DVD_RATE_CHANGE"),
    "AM_PROPERTY_FRAMESTEP": SimTypeInt(signed=False, label="AM_PROPERTY_FRAMESTEP"),
    "AM_PROPERTY_SPHLI": SimStruct(OrderedDict((("HLISS", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("StartPTM", SimTypeInt(signed=False, label="UInt32")), ("EndPTM", SimTypeInt(signed=False, label="UInt32")), ("StartX", SimTypeShort(signed=False, label="UInt16")), ("StartY", SimTypeShort(signed=False, label="UInt16")), ("StopX", SimTypeShort(signed=False, label="UInt16")), ("StopY", SimTypeShort(signed=False, label="UInt16")), ("ColCon", SimTypeRef("AM_COLCON", SimStruct)),)), name="AM_PROPERTY_SPHLI", pack=False, align=None),
    "AM_PROPERTY_SPPAL": SimStruct(OrderedDict((("sppal", SimTypeArray(SimTypeRef("AM_DVD_YUV", SimStruct), 16)),)), name="AM_PROPERTY_SPPAL", pack=False, align=None),
    "AM_PROPERTY_TS_RATE_CHANGE": SimTypeInt(signed=False, label="AM_PROPERTY_TS_RATE_CHANGE"),
    "AM_QueryRate": SimStruct(OrderedDict((("lMaxForwardFullFrame", SimTypeInt(signed=True, label="Int32")), ("lMaxReverseFullFrame", SimTypeInt(signed=True, label="Int32")),)), name="AM_QueryRate", pack=False, align=None),
    "AM_SAMPLE2_PROPERTIES": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("dwTypeSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSampleFlags", SimTypeInt(signed=False, label="UInt32")), ("lActual", SimTypeInt(signed=True, label="Int32")), ("tStart", SimTypeLongLong(signed=True, label="Int64")), ("tStop", SimTypeLongLong(signed=True, label="Int64")), ("dwStreamId", SimTypeInt(signed=False, label="UInt32")), ("pMediaType", SimTypePointer(SimTypeRef("AM_MEDIA_TYPE", SimStruct), offset=0)), ("pbBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbBuffer", SimTypeInt(signed=True, label="Int32")),)), name="AM_SAMPLE2_PROPERTIES", pack=False, align=None),
    "AM_SAMPLE_PROPERTY_FLAGS": SimTypeInt(signed=False, label="AM_SAMPLE_PROPERTY_FLAGS"),
    "AM_SEEKING_SEEKING_CAPABILITIES": SimTypeInt(signed=False, label="AM_SEEKING_SEEKING_CAPABILITIES"),
    "AM_SEEKING_SEEKING_FLAGS": SimTypeInt(signed=False, label="AM_SEEKING_SEEKING_FLAGS"),
    "AM_STREAM_INFO": SimStruct(OrderedDict((("tStart", SimTypeLongLong(signed=True, label="Int64")), ("tStop", SimTypeLongLong(signed=True, label="Int64")), ("dwStartCookie", SimTypeInt(signed=False, label="UInt32")), ("dwStopCookie", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="AM_STREAM_INFO", pack=False, align=None),
    "AM_STREAM_INFO_FLAGS": SimTypeInt(signed=False, label="AM_STREAM_INFO_FLAGS"),
    "AM_SimpleRateChange": SimStruct(OrderedDict((("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("Rate", SimTypeInt(signed=True, label="Int32")),)), name="AM_SimpleRateChange", pack=False, align=None),
    "AM_WMT_EVENT_DATA": SimStruct(OrderedDict((("hrStatus", SimTypeInt(signed=True, label="Int32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="AM_WMT_EVENT_DATA", pack=False, align=None),
    "AM_WST_DRAWBGMODE": SimTypeInt(signed=False, label="AM_WST_DRAWBGMODE"),
    "AM_WST_LEVEL": SimTypeInt(signed=False, label="AM_WST_LEVEL"),
    "AM_WST_PAGE": SimStruct(OrderedDict((("dwPageNr", SimTypeInt(signed=False, label="UInt32")), ("dwSubPageNr", SimTypeInt(signed=False, label="UInt32")), ("pucPageData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="AM_WST_PAGE", pack=False, align=None),
    "AM_WST_SERVICE": SimTypeInt(signed=False, label="AM_WST_SERVICE"),
    "AM_WST_STATE": SimTypeInt(signed=False, label="AM_WST_STATE"),
    "AM_WST_STYLE": SimTypeInt(signed=False, label="AM_WST_STYLE"),
    "ANALOGVIDEOINFO": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwActiveWidth", SimTypeInt(signed=False, label="UInt32")), ("dwActiveHeight", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")),)), name="ANALOGVIDEOINFO", pack=False, align=None),
    "ANCHOR_CHANGE_HISTORY_FLAGS": SimTypeInt(signed=False, label="ANCHOR_CHANGE_HISTORY_FLAGS"),
    "ANIMATE_WINDOW_FLAGS": SimTypeInt(signed=False, label="ANIMATE_WINDOW_FLAGS"),
    "ANIMATIONINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iMinAnimate", SimTypeInt(signed=True, label="Int32")),)), name="ANIMATIONINFO", pack=False, align=None),
    "ANON_OBJECT_HEADER": SimStruct(OrderedDict((("Sig1", SimTypeShort(signed=False, label="UInt16")), ("Sig2", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Machine", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ClassID", SimTypeBottom(label="Guid")), ("SizeOfData", SimTypeInt(signed=False, label="UInt32")),)), name="ANON_OBJECT_HEADER", pack=False, align=None),
    "ANON_OBJECT_HEADER_BIGOBJ": SimStruct(OrderedDict((("Sig1", SimTypeShort(signed=False, label="UInt16")), ("Sig2", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Machine", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ClassID", SimTypeBottom(label="Guid")), ("SizeOfData", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MetaDataSize", SimTypeInt(signed=False, label="UInt32")), ("MetaDataOffset", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSections", SimTypeInt(signed=False, label="UInt32")), ("PointerToSymbolTable", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSymbols", SimTypeInt(signed=False, label="UInt32")),)), name="ANON_OBJECT_HEADER_BIGOBJ", pack=False, align=None),
    "ANON_OBJECT_HEADER_V2": SimStruct(OrderedDict((("Sig1", SimTypeShort(signed=False, label="UInt16")), ("Sig2", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Machine", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ClassID", SimTypeBottom(label="Guid")), ("SizeOfData", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MetaDataSize", SimTypeInt(signed=False, label="UInt32")), ("MetaDataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="ANON_OBJECT_HEADER_V2", pack=False, align=None),
    "APC_CALLBACK_DATA": SimStruct(OrderedDict((("Parameter", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ContextRecord", SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0)), ("Reserved0", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="APC_CALLBACK_DATA", pack=False, align=None),
    "APC_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "API_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="API_VERSION", pack=False, align=None),
    "APOInitBaseStruct": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")),)), name="APOInitBaseStruct", pack=False, align=None),
    "APOInitSystemEffects": SimStruct(OrderedDict((("APOInit", SimTypeRef("APOInitBaseStruct", SimStruct)), ("pAPOEndpointProperties", SimTypeBottom(label="IPropertyStore")), ("pAPOSystemEffectsProperties", SimTypeBottom(label="IPropertyStore")), ("pReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pDeviceCollection", SimTypeBottom(label="IMMDeviceCollection")),)), name="APOInitSystemEffects", pack=False, align=None),
    "APOInitSystemEffects2": SimStruct(OrderedDict((("APOInit", SimTypeRef("APOInitBaseStruct", SimStruct)), ("pAPOEndpointProperties", SimTypeBottom(label="IPropertyStore")), ("pAPOSystemEffectsProperties", SimTypeBottom(label="IPropertyStore")), ("pReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pDeviceCollection", SimTypeBottom(label="IMMDeviceCollection")), ("nSoftwareIoDeviceInCollection", SimTypeInt(signed=False, label="UInt32")), ("nSoftwareIoConnectorIndex", SimTypeInt(signed=False, label="UInt32")), ("AudioProcessingMode", SimTypeBottom(label="Guid")), ("InitializeForDiscoveryOnly", SimTypeInt(signed=True, label="Int32")),)), name="APOInitSystemEffects2", pack=False, align=None),
    "APOInitSystemEffects3": SimStruct(OrderedDict((("APOInit", SimTypeRef("APOInitBaseStruct", SimStruct)), ("pAPOEndpointProperties", SimTypeBottom(label="IPropertyStore")), ("pServiceProvider", SimTypeBottom(label="IServiceProvider")), ("pDeviceCollection", SimTypeBottom(label="IMMDeviceCollection")), ("nSoftwareIoDeviceInCollection", SimTypeInt(signed=False, label="UInt32")), ("nSoftwareIoConnectorIndex", SimTypeInt(signed=False, label="UInt32")), ("AudioProcessingMode", SimTypeBottom(label="Guid")), ("InitializeForDiscoveryOnly", SimTypeInt(signed=True, label="Int32")),)), name="APOInitSystemEffects3", pack=False, align=None),
    "APO_BUFFER_FLAGS": SimTypeInt(signed=False, label="APO_BUFFER_FLAGS"),
    "APO_CONNECTION_BUFFER_TYPE": SimTypeInt(signed=False, label="APO_CONNECTION_BUFFER_TYPE"),
    "APO_CONNECTION_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="APO_CONNECTION_BUFFER_TYPE")), ("pBuffer", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("u32MaxFrameCount", SimTypeInt(signed=False, label="UInt32")), ("pFormat", SimTypeBottom(label="IAudioMediaType")), ("u32Signature", SimTypeInt(signed=False, label="UInt32")),)), name="APO_CONNECTION_DESCRIPTOR", pack=False, align=None),
    "APO_CONNECTION_PROPERTY": SimStruct(OrderedDict((("pBuffer", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("u32ValidFrameCount", SimTypeInt(signed=False, label="UInt32")), ("u32BufferFlags", SimTypeInt(signed=False, label="APO_BUFFER_FLAGS")), ("u32Signature", SimTypeInt(signed=False, label="UInt32")),)), name="APO_CONNECTION_PROPERTY", pack=False, align=None),
    "APO_CONNECTION_PROPERTY_V2": SimStruct(OrderedDict((("property", SimTypeRef("APO_CONNECTION_PROPERTY", SimStruct)), ("u64QPCTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="APO_CONNECTION_PROPERTY_V2", pack=False, align=None),
    "APO_FLAG": SimTypeInt(signed=False, label="APO_FLAG"),
    "APO_LOG_LEVEL": SimTypeInt(signed=False, label="APO_LOG_LEVEL"),
    "APO_NOTIFICATION": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="APO_NOTIFICATION_TYPE")), ("Anonymous", SimUnion({"audioEndpointVolumeChange": SimTypeRef("AUDIO_ENDPOINT_VOLUME_CHANGE_NOTIFICATION", SimStruct), "audioEndpointPropertyChange": SimTypeRef("AUDIO_ENDPOINT_PROPERTY_CHANGE_NOTIFICATION", SimStruct), "audioSystemEffectsPropertyChange": SimTypeRef("AUDIO_SYSTEMEFFECTS_PROPERTY_CHANGE_NOTIFICATION", SimStruct), "audioEndpointVolumeChange2": SimTypeRef("AUDIO_ENDPOINT_VOLUME_CHANGE_NOTIFICATION2", SimStruct), "deviceOrientation": SimTypeInt(signed=False, label="DEVICE_ORIENTATION_TYPE"), "audioMicrophoneBoostChange": SimTypeRef("AUDIO_MICROPHONE_BOOST_NOTIFICATION", SimStruct)}, name="<anon>", label="None")),)), name="APO_NOTIFICATION", pack=False, align=None),
    "APO_NOTIFICATION_DESCRIPTOR": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="APO_NOTIFICATION_TYPE")), ("Anonymous", SimUnion({"audioEndpointVolume": SimTypeRef("AUDIO_ENDPOINT_VOLUME_APO_NOTIFICATION_DESCRIPTOR", SimStruct), "audioEndpointPropertyChange": SimTypeRef("AUDIO_ENDPOINT_PROPERTY_CHANGE_APO_NOTIFICATION_DESCRIPTOR", SimStruct), "audioSystemEffectsPropertyChange": SimTypeRef("AUDIO_SYSTEMEFFECTS_PROPERTY_CHANGE_APO_NOTIFICATION_DESCRIPTOR", SimStruct), "audioMicrophoneBoost": SimTypeRef("AUDIO_MICROPHONE_BOOST_APO_NOTIFICATION_DESCRIPTOR", SimStruct)}, name="<anon>", label="None")),)), name="APO_NOTIFICATION_DESCRIPTOR", pack=False, align=None),
    "APO_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="APO_NOTIFICATION_TYPE"),
    "APO_REG_PROPERTIES": SimStruct(OrderedDict((("clsid", SimTypeBottom(label="Guid")), ("Flags", SimTypeInt(signed=False, label="APO_FLAG")), ("szFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("szCopyrightInfo", SimTypeArray(SimTypeChar(label="Char"), 256)), ("u32MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("u32MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("u32MinInputConnections", SimTypeInt(signed=False, label="UInt32")), ("u32MaxInputConnections", SimTypeInt(signed=False, label="UInt32")), ("u32MinOutputConnections", SimTypeInt(signed=False, label="UInt32")), ("u32MaxOutputConnections", SimTypeInt(signed=False, label="UInt32")), ("u32MaxInstances", SimTypeInt(signed=False, label="UInt32")), ("u32NumAPOInterfaces", SimTypeInt(signed=False, label="UInt32")), ("iidAPOInterfaceList", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="APO_REG_PROPERTIES", pack=False, align=None),
    "APPACTIONFLAGS": SimTypeInt(signed=False, label="APPACTIONFLAGS"),
    "APPBARDATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uCallbackMessage", SimTypeInt(signed=False, label="UInt32")), ("uEdge", SimTypeInt(signed=False, label="UInt32")), ("rc", SimTypeRef("RECT", SimStruct)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="APPBARDATA", pack=False, align=None),
    "APPCATEGORYINFO": SimStruct(OrderedDict((("Locale", SimTypeInt(signed=False, label="UInt32")), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("AppCategoryId", SimTypeBottom(label="Guid")),)), name="APPCATEGORYINFO", pack=False, align=None),
    "APPCATEGORYINFOLIST": SimStruct(OrderedDict((("cCategory", SimTypeInt(signed=False, label="UInt32")), ("pCategoryInfo", SimTypePointer(SimTypeRef("APPCATEGORYINFO", SimStruct), offset=0)),)), name="APPCATEGORYINFOLIST", pack=False, align=None),
    "APPCOMMAND_ID": SimTypeInt(signed=False, label="APPCOMMAND_ID"),
    "APPDATA": SimStruct(OrderedDict((("m_idApp", SimTypeInt(signed=False, label="UInt32")), ("m_szAppGuid", SimTypeArray(SimTypeChar(label="Char"), 40)), ("m_dwAppProcessId", SimTypeInt(signed=False, label="UInt32")), ("m_AppStatistics", SimTypeRef("APPSTATISTICS", SimStruct)),)), name="APPDATA", pack=False, align=None),
    "APPDOCLISTTYPE": SimTypeInt(signed=False, label="APPDOCLISTTYPE"),
    "APPDOMAIN_SECURITY_FLAGS": SimTypeInt(signed=False, label="APPDOMAIN_SECURITY_FLAGS"),
    "APPINFODATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("pszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPublisher", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszProductID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRegisteredOwner", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRegisteredCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSupportUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSupportTelephone", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHelpLink", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInstallLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInstallSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInstallDate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszContact", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszComments", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszImage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszReadmeUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUpdateInfoUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="APPINFODATA", pack=False, align=None),
    "APPINFODATAFLAGS": SimTypeInt(signed=False, label="APPINFODATAFLAGS"),
    "APPLETIDLIST": SimStruct(OrderedDict((("count", SimTypeInt(signed=True, label="Int32")), ("pIIDList", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="APPLETIDLIST", pack=False, align=None),
    "APPLET_PROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndCpl", "msg", "lParam1", "lParam2"]), offset=0),
    "APPLICATIONLAUNCH_SETTING_VALUE": SimStruct(OrderedDict((("ActivationTime", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ButtonInstanceID", SimTypeInt(signed=False, label="UInt32")),)), name="APPLICATIONLAUNCH_SETTING_VALUE", pack=False, align=None),
    "APPLICATIONTYPE": SimTypeInt(signed=False, label="APPLICATIONTYPE"),
    "APPLICATION_EVENT_DATA": SimStruct(OrderedDict((("cbApplicationEventData", SimTypeInt(signed=False, label="UInt32")), ("ApplicationId", SimTypeBottom(label="Guid")), ("EndpointId", SimTypeBottom(label="Guid")), ("dwEventId", SimTypeInt(signed=False, label="UInt32")), ("cbEventData", SimTypeInt(signed=False, label="UInt32")), ("bEventData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="APPLICATION_EVENT_DATA", pack=False, align=None),
    "APPLICATION_NODE_EVENT_FILTER": SimTypeInt(signed=False, label="APPLICATION_NODE_EVENT_FILTER"),
    "APPLICATION_RECOVERY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pvParameter"]), offset=0),
    "APPLICATION_VIEW_MIN_WIDTH": SimTypeInt(signed=False, label="APPLICATION_VIEW_MIN_WIDTH"),
    "APPLICATION_VIEW_ORIENTATION": SimTypeInt(signed=False, label="APPLICATION_VIEW_ORIENTATION"),
    "APPLICATION_VIEW_SIZE_PREFERENCE": SimTypeInt(signed=False, label="APPLICATION_VIEW_SIZE_PREFERENCE"),
    "APPLICATION_VIEW_STATE": SimTypeInt(signed=False, label="APPLICATION_VIEW_STATE"),
    "APPLYCANDEXPARAM": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpwstrDisplay", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwstrReading", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="APPLYCANDEXPARAM", pack=False, align=None),
    "APPLY_CONTROL_TOKEN_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "APPLY_SNAPSHOT_VHDSET_FLAG": SimTypeInt(signed=False, label="APPLY_SNAPSHOT_VHDSET_FLAG"),
    "APPLY_SNAPSHOT_VHDSET_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="APPLY_SNAPSHOT_VHDSET_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="APPLY_SNAPSHOT_VHDSET_PARAMETERS", pack=False, align=None),
    "APPLY_SNAPSHOT_VHDSET_VERSION": SimTypeInt(signed=False, label="APPLY_SNAPSHOT_VHDSET_VERSION"),
    "APPSTATE": SimTypeInt(signed=False, label="APPSTATE"),
    "APPSTATISTICS": SimStruct(OrderedDict((("m_cTotalCalls", SimTypeInt(signed=False, label="UInt32")), ("m_cTotalInstances", SimTypeInt(signed=False, label="UInt32")), ("m_cTotalClasses", SimTypeInt(signed=False, label="UInt32")), ("m_cCallsPerSecond", SimTypeInt(signed=False, label="UInt32")),)), name="APPSTATISTICS", pack=False, align=None),
    "APPX_BUNDLE_FOOTPRINT_FILE_TYPE": SimTypeInt(signed=False, label="APPX_BUNDLE_FOOTPRINT_FILE_TYPE"),
    "APPX_BUNDLE_PAYLOAD_PACKAGE_TYPE": SimTypeInt(signed=False, label="APPX_BUNDLE_PAYLOAD_PACKAGE_TYPE"),
    "APPX_CAPABILITIES": SimTypeInt(signed=False, label="APPX_CAPABILITIES"),
    "APPX_CAPABILITY_CLASS_TYPE": SimTypeInt(signed=False, label="APPX_CAPABILITY_CLASS_TYPE"),
    "APPX_COMPRESSION_OPTION": SimTypeInt(signed=False, label="APPX_COMPRESSION_OPTION"),
    "APPX_ENCRYPTED_EXEMPTIONS": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("plainTextFiles", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="APPX_ENCRYPTED_EXEMPTIONS", pack=False, align=None),
    "APPX_ENCRYPTED_PACKAGE_OPTIONS": SimTypeInt(signed=False, label="APPX_ENCRYPTED_PACKAGE_OPTIONS"),
    "APPX_ENCRYPTED_PACKAGE_SETTINGS": SimStruct(OrderedDict((("keyLength", SimTypeInt(signed=False, label="UInt32")), ("encryptionAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("useDiffusion", SimTypeInt(signed=True, label="Int32")), ("blockMapHashAlgorithm", SimTypeBottom(label="IUri")),)), name="APPX_ENCRYPTED_PACKAGE_SETTINGS", pack=False, align=None),
    "APPX_ENCRYPTED_PACKAGE_SETTINGS2": SimStruct(OrderedDict((("keyLength", SimTypeInt(signed=False, label="UInt32")), ("encryptionAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("blockMapHashAlgorithm", SimTypeBottom(label="IUri")), ("options", SimTypeInt(signed=False, label="UInt32")),)), name="APPX_ENCRYPTED_PACKAGE_SETTINGS2", pack=False, align=None),
    "APPX_FOOTPRINT_FILE_TYPE": SimTypeInt(signed=False, label="APPX_FOOTPRINT_FILE_TYPE"),
    "APPX_KEY_INFO": SimStruct(OrderedDict((("keyLength", SimTypeInt(signed=False, label="UInt32")), ("keyIdLength", SimTypeInt(signed=False, label="UInt32")), ("key", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("keyId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="APPX_KEY_INFO", pack=False, align=None),
    "APPX_PACKAGE_ARCHITECTURE": SimTypeInt(signed=False, label="APPX_PACKAGE_ARCHITECTURE"),
    "APPX_PACKAGE_ARCHITECTURE2": SimTypeInt(signed=False, label="APPX_PACKAGE_ARCHITECTURE2"),
    "APPX_PACKAGE_EDITOR_UPDATE_PACKAGE_MANIFEST_OPTIONS": SimTypeInt(signed=False, label="APPX_PACKAGE_EDITOR_UPDATE_PACKAGE_MANIFEST_OPTIONS"),
    "APPX_PACKAGE_EDITOR_UPDATE_PACKAGE_OPTION": SimTypeInt(signed=False, label="APPX_PACKAGE_EDITOR_UPDATE_PACKAGE_OPTION"),
    "APPX_PACKAGE_SETTINGS": SimStruct(OrderedDict((("forceZip32", SimTypeInt(signed=True, label="Int32")), ("hashMethod", SimTypeBottom(label="IUri")),)), name="APPX_PACKAGE_SETTINGS", pack=False, align=None),
    "APPX_PACKAGE_WRITER_PAYLOAD_STREAM": SimStruct(OrderedDict((("inputStream", SimTypeBottom(label="IStream")), ("fileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("contentType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("compressionOption", SimTypeInt(signed=False, label="APPX_COMPRESSION_OPTION")),)), name="APPX_PACKAGE_WRITER_PAYLOAD_STREAM", pack=False, align=None),
    "APPX_PACKAGING_CONTEXT_CHANGE_TYPE": SimTypeInt(signed=False, label="APPX_PACKAGING_CONTEXT_CHANGE_TYPE"),
    "APP_CACHE_DOWNLOAD_ENTRY": SimStruct(OrderedDict((("pwszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwEntryType", SimTypeInt(signed=False, label="UInt32")),)), name="APP_CACHE_DOWNLOAD_ENTRY", pack=False, align=None),
    "APP_CACHE_DOWNLOAD_LIST": SimStruct(OrderedDict((("dwEntryCount", SimTypeInt(signed=False, label="UInt32")), ("pEntries", SimTypePointer(SimTypeRef("APP_CACHE_DOWNLOAD_ENTRY", SimStruct), offset=0)),)), name="APP_CACHE_DOWNLOAD_LIST", pack=False, align=None),
    "APP_CACHE_FINALIZE_STATE": SimTypeInt(signed=False, label="APP_CACHE_FINALIZE_STATE"),
    "APP_CACHE_GROUP_INFO": SimStruct(OrderedDict((("pwszManifestUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="APP_CACHE_GROUP_INFO", pack=False, align=None),
    "APP_CACHE_GROUP_LIST": SimStruct(OrderedDict((("dwAppCacheGroupCount", SimTypeInt(signed=False, label="UInt32")), ("pAppCacheGroups", SimTypePointer(SimTypeRef("APP_CACHE_GROUP_INFO", SimStruct), offset=0)),)), name="APP_CACHE_GROUP_LIST", pack=False, align=None),
    "APP_CACHE_STATE": SimTypeInt(signed=False, label="APP_CACHE_STATE"),
    "APP_LOCAL_DEVICE_ID": SimStruct(OrderedDict((("value", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="APP_LOCAL_DEVICE_ID", pack=False, align=None),
    "APP_MEMORY_INFORMATION": SimStruct(OrderedDict((("AvailableCommit", SimTypeLongLong(signed=False, label="UInt64")), ("PrivateCommitUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PeakPrivateCommitUsage", SimTypeLongLong(signed=False, label="UInt64")), ("TotalCommitUsage", SimTypeLongLong(signed=False, label="UInt64")),)), name="APP_MEMORY_INFORMATION", pack=False, align=None),
    "APTTYPE": SimTypeInt(signed=False, label="APTTYPE"),
    "APTTYPEQUALIFIER": SimTypeInt(signed=False, label="APTTYPEQUALIFIER"),
    "APTXWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="APTXWAVEFORMAT", pack=False, align=None),
    "ARBITER_ACTION": SimTypeInt(signed=False, label="ARBITER_ACTION"),
    "ARBITER_ADD_RESERVED_PARAMETERS": SimStruct(OrderedDict((("ReserveDevice", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)),)), name="ARBITER_ADD_RESERVED_PARAMETERS", pack=False, align=None),
    "ARBITER_BOOT_ALLOCATION_PARAMETERS": SimStruct(OrderedDict((("ArbitrationList", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)),)), name="ARBITER_BOOT_ALLOCATION_PARAMETERS", pack=False, align=None),
    "ARBITER_CONFLICT_INFO": SimStruct(OrderedDict((("OwningObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("Start", SimTypeLongLong(signed=False, label="UInt64")), ("End", SimTypeLongLong(signed=False, label="UInt64")),)), name="ARBITER_CONFLICT_INFO", pack=False, align=None),
    "ARBITER_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("ArbiterHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="ARBITER_ACTION"), SimTypePointer(SimTypeRef("ARBITER_PARAMETERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Action", "Parameters"]), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="ARBITER_INTERFACE", pack=False, align=None),
    "ARBITER_LIST_ENTRY": SimStruct(OrderedDict((("ListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("AlternativeCount", SimTypeInt(signed=False, label="UInt32")), ("Alternatives", SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0)), ("PhysicalDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("RequestSource", SimTypeInt(signed=False, label="ARBITER_REQUEST_SOURCE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("WorkSpace", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InterfaceType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("SlotNumber", SimTypeInt(signed=False, label="UInt32")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("Assignment", SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)), ("SelectedAlternative", SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0)), ("Result", SimTypeInt(signed=False, label="ARBITER_RESULT")),)), name="ARBITER_LIST_ENTRY", pack=False, align=None),
    "ARBITER_PARAMETERS": SimStruct(OrderedDict((("Parameters", SimUnion({"TestAllocation": SimTypeRef("ARBITER_TEST_ALLOCATION_PARAMETERS", SimStruct), "RetestAllocation": SimTypeRef("ARBITER_RETEST_ALLOCATION_PARAMETERS", SimStruct), "BootAllocation": SimTypeRef("ARBITER_BOOT_ALLOCATION_PARAMETERS", SimStruct), "QueryAllocatedResources": SimTypeRef("ARBITER_QUERY_ALLOCATED_RESOURCES_PARAMETERS", SimStruct), "QueryConflict": SimTypeRef("ARBITER_QUERY_CONFLICT_PARAMETERS", SimStruct), "QueryArbitrate": SimTypeRef("ARBITER_QUERY_ARBITRATE_PARAMETERS", SimStruct), "AddReserved": SimTypeRef("ARBITER_ADD_RESERVED_PARAMETERS", SimStruct)}, name="<anon>", label="None")),)), name="ARBITER_PARAMETERS", pack=False, align=None),
    "ARBITER_QUERY_ALLOCATED_RESOURCES_PARAMETERS": SimStruct(OrderedDict((("AllocatedResources", SimTypePointer(SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_LIST", SimStruct), offset=0), offset=0)),)), name="ARBITER_QUERY_ALLOCATED_RESOURCES_PARAMETERS", pack=False, align=None),
    "ARBITER_QUERY_ARBITRATE_PARAMETERS": SimStruct(OrderedDict((("ArbitrationList", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)),)), name="ARBITER_QUERY_ARBITRATE_PARAMETERS", pack=False, align=None),
    "ARBITER_QUERY_CONFLICT_PARAMETERS": SimStruct(OrderedDict((("PhysicalDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("ConflictingResource", SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0)), ("ConflictCount", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Conflicts", SimTypePointer(SimTypePointer(SimTypeRef("ARBITER_CONFLICT_INFO", SimStruct), offset=0), offset=0)),)), name="ARBITER_QUERY_CONFLICT_PARAMETERS", pack=False, align=None),
    "ARBITER_REQUEST_SOURCE": SimTypeInt(signed=False, label="ARBITER_REQUEST_SOURCE"),
    "ARBITER_RESULT": SimTypeInt(signed=False, label="ARBITER_RESULT"),
    "ARBITER_RETEST_ALLOCATION_PARAMETERS": SimStruct(OrderedDict((("ArbitrationList", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("AllocateFromCount", SimTypeInt(signed=False, label="UInt32")), ("AllocateFrom", SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)),)), name="ARBITER_RETEST_ALLOCATION_PARAMETERS", pack=False, align=None),
    "ARBITER_TEST_ALLOCATION_PARAMETERS": SimStruct(OrderedDict((("ArbitrationList", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("AllocateFromCount", SimTypeInt(signed=False, label="UInt32")), ("AllocateFrom", SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)),)), name="ARBITER_TEST_ALLOCATION_PARAMETERS", pack=False, align=None),
    "ARC_DIRECTION": SimTypeInt(signed=False, label="ARC_DIRECTION"),
    "ARM64_FNPDATA_CR": SimTypeInt(signed=False, label="ARM64_FNPDATA_CR"),
    "ARM64_FNPDATA_FLAGS": SimTypeInt(signed=False, label="ARM64_FNPDATA_FLAGS"),
    "ARM64_NT_CONTEXT": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=False, label="UInt32")), ("Cpsr", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("X0", SimTypeLongLong(signed=False, label="UInt64")), ("X1", SimTypeLongLong(signed=False, label="UInt64")), ("X2", SimTypeLongLong(signed=False, label="UInt64")), ("X3", SimTypeLongLong(signed=False, label="UInt64")), ("X4", SimTypeLongLong(signed=False, label="UInt64")), ("X5", SimTypeLongLong(signed=False, label="UInt64")), ("X6", SimTypeLongLong(signed=False, label="UInt64")), ("X7", SimTypeLongLong(signed=False, label="UInt64")), ("X8", SimTypeLongLong(signed=False, label="UInt64")), ("X9", SimTypeLongLong(signed=False, label="UInt64")), ("X10", SimTypeLongLong(signed=False, label="UInt64")), ("X11", SimTypeLongLong(signed=False, label="UInt64")), ("X12", SimTypeLongLong(signed=False, label="UInt64")), ("X13", SimTypeLongLong(signed=False, label="UInt64")), ("X14", SimTypeLongLong(signed=False, label="UInt64")), ("X15", SimTypeLongLong(signed=False, label="UInt64")), ("X16", SimTypeLongLong(signed=False, label="UInt64")), ("X17", SimTypeLongLong(signed=False, label="UInt64")), ("X18", SimTypeLongLong(signed=False, label="UInt64")), ("X19", SimTypeLongLong(signed=False, label="UInt64")), ("X20", SimTypeLongLong(signed=False, label="UInt64")), ("X21", SimTypeLongLong(signed=False, label="UInt64")), ("X22", SimTypeLongLong(signed=False, label="UInt64")), ("X23", SimTypeLongLong(signed=False, label="UInt64")), ("X24", SimTypeLongLong(signed=False, label="UInt64")), ("X25", SimTypeLongLong(signed=False, label="UInt64")), ("X26", SimTypeLongLong(signed=False, label="UInt64")), ("X27", SimTypeLongLong(signed=False, label="UInt64")), ("X28", SimTypeLongLong(signed=False, label="UInt64")), ("Fp", SimTypeLongLong(signed=False, label="UInt64")), ("Lr", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "X": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 31)}, name="<anon>", label="None")), ("Sp", SimTypeLongLong(signed=False, label="UInt64")), ("Pc", SimTypeLongLong(signed=False, label="UInt64")), ("V", SimTypeArray(SimUnion({"Anonymous": SimStruct(OrderedDict((("Low", SimTypeLongLong(signed=False, label="UInt64")), ("High", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "D": SimTypeArray(SimTypeFloat(size=64), 2), "S": SimTypeArray(SimTypeFloat(size=32), 4), "H": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "B": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), 32)), ("Fpcr", SimTypeInt(signed=False, label="UInt32")), ("Fpsr", SimTypeInt(signed=False, label="UInt32")), ("Bcr", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("Bvr", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)), ("Wcr", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("Wvr", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="ARM64_NT_CONTEXT", pack=False, align=None),
    "ARM64_NT_NEON128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low", SimTypeLongLong(signed=False, label="UInt64")), ("High", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "D": SimTypeArray(SimTypeFloat(size=64), 2), "S": SimTypeArray(SimTypeFloat(size=32), 4), "H": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "B": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"),
    "ARP_HARDWARE_TYPE": SimTypeInt(signed=False, label="ARP_HARDWARE_TYPE"),
    "ARP_HEADER": SimStruct(OrderedDict((("HardwareAddressSpace", SimTypeShort(signed=False, label="UInt16")), ("ProtocolAddressSpace", SimTypeShort(signed=False, label="UInt16")), ("HardwareAddressLength", SimTypeChar(label="Byte")), ("ProtocolAddressLength", SimTypeChar(label="Byte")), ("Opcode", SimTypeShort(signed=False, label="UInt16")), ("SenderHardwareAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ARP_HEADER", pack=False, align=None),
    "ARP_OPCODE": SimTypeInt(signed=False, label="ARP_OPCODE"),
    "ARP_SEND_REPLY": SimStruct(OrderedDict((("DestAddress", SimTypeInt(signed=False, label="UInt32")), ("SrcAddress", SimTypeInt(signed=False, label="UInt32")),)), name="ARP_SEND_REPLY", pack=False, align=None),
    "ARRAYDESC": SimStruct(OrderedDict((("tdescElem", SimTypeRef("TYPEDESC", SimStruct)), ("cDims", SimTypeShort(signed=False, label="UInt16")), ("rgbounds", SimTypePointer(SimTypeRef("SAFEARRAYBOUND", SimStruct), offset=0)),)), name="ARRAYDESC", pack=False, align=None),
    "ARRAY_INFO": SimStruct(OrderedDict((("Dimension", SimTypeInt(signed=True, label="Int32")), ("BufferConformanceMark", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("BufferVarianceMark", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("MaxCountArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("OffsetArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ActualCountArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="ARRAY_INFO", pack=False, align=None),
    "ARROWBTNSTATES": SimTypeInt(signed=False, label="ARROWBTNSTATES"),
    "AR_STATE": SimTypeInt(signed=False, label="AR_STATE"),
    "ASC_REQ_FLAGS": SimTypeInt(signed=False, label="ASC_REQ_FLAGS"),
    "ASC_REQ_HIGH_FLAGS": SimTypeInt(signed=False, label="ASC_REQ_HIGH_FLAGS"),
    "ASF_FLAT_PICTURE": SimStruct(OrderedDict((("bPictureType", SimTypeChar(label="Byte")), ("dwDataLen", SimTypeInt(signed=False, label="UInt32")),)), name="ASF_FLAT_PICTURE", pack=False, align=None),
    "ASF_FLAT_SYNCHRONISED_LYRICS": SimStruct(OrderedDict((("bTimeStampFormat", SimTypeChar(label="Byte")), ("bContentType", SimTypeChar(label="Byte")), ("dwLyricsLen", SimTypeInt(signed=False, label="UInt32")),)), name="ASF_FLAT_SYNCHRONISED_LYRICS", pack=False, align=None),
    "ASF_INDEX_DESCRIPTOR": SimStruct(OrderedDict((("Identifier", SimTypeRef("ASF_INDEX_IDENTIFIER", SimStruct)), ("cPerEntryBytes", SimTypeShort(signed=False, label="UInt16")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwInterval", SimTypeInt(signed=False, label="UInt32")),)), name="ASF_INDEX_DESCRIPTOR", pack=False, align=None),
    "ASF_INDEX_IDENTIFIER": SimStruct(OrderedDict((("guidIndexType", SimTypeBottom(label="Guid")), ("wStreamNumber", SimTypeShort(signed=False, label="UInt16")),)), name="ASF_INDEX_IDENTIFIER", pack=False, align=None),
    "ASF_MUX_STATISTICS": SimStruct(OrderedDict((("cFramesWritten", SimTypeInt(signed=False, label="UInt32")), ("cFramesDropped", SimTypeInt(signed=False, label="UInt32")),)), name="ASF_MUX_STATISTICS", pack=False, align=None),
    "ASF_SELECTION_STATUS": SimTypeInt(signed=False, label="ASF_SELECTION_STATUS"),
    "ASF_STATUSFLAGS": SimTypeInt(signed=False, label="ASF_STATUSFLAGS"),
    "ASM_BIND_FLAGS": SimTypeInt(signed=False, label="ASM_BIND_FLAGS"),
    "ASM_CMP_FLAGS": SimTypeInt(signed=False, label="ASM_CMP_FLAGS"),
    "ASM_DISPLAY_FLAGS": SimTypeInt(signed=False, label="ASM_DISPLAY_FLAGS"),
    "ASM_NAME": SimTypeInt(signed=False, label="ASM_NAME"),
    "ASSEMBLY_FILE_DETAILED_INFORMATION": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulFilenameLength", SimTypeInt(signed=False, label="UInt32")), ("ulPathLength", SimTypeInt(signed=False, label="UInt32")), ("lpFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ASSEMBLY_FILE_DETAILED_INFORMATION", pack=False, align=None),
    "ASSEMBLY_INFO": SimStruct(OrderedDict((("cbAssemblyInfo", SimTypeInt(signed=False, label="UInt32")), ("dwAssemblyFlags", SimTypeInt(signed=False, label="UInt32")), ("uliAssemblySizeInKB", SimTypeLongLong(signed=False, label="UInt64")), ("pszCurrentAssemblyPathBuf", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchBuf", SimTypeInt(signed=False, label="UInt32")),)), name="ASSEMBLY_INFO", pack=False, align=None),
    "ASSOCCLASS": SimTypeInt(signed=False, label="ASSOCCLASS"),
    "ASSOCDATA": SimTypeInt(signed=False, label="ASSOCDATA"),
    "ASSOCENUM": SimTypeInt(signed=False, label="ASSOCENUM"),
    "ASSOCF": SimTypeInt(signed=False, label="ASSOCF"),
    "ASSOCIATE_NAMERES_CONTEXT_INPUT": SimStruct(OrderedDict((("TransportSettingId", SimTypeRef("TRANSPORT_SETTING_ID", SimStruct)), ("Handle", SimTypeLongLong(signed=False, label="UInt64")),)), name="ASSOCIATE_NAMERES_CONTEXT_INPUT", pack=False, align=None),
    "ASSOCIATIONELEMENT": SimStruct(OrderedDict((("ac", SimTypeInt(signed=False, label="ASSOCCLASS")), ("hkClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ASSOCIATIONELEMENT", pack=False, align=None),
    "ASSOCIATIONLEVEL": SimTypeInt(signed=False, label="ASSOCIATIONLEVEL"),
    "ASSOCIATIONTYPE": SimTypeInt(signed=False, label="ASSOCIATIONTYPE"),
    "ASSOCKEY": SimTypeInt(signed=False, label="ASSOCKEY"),
    "ASSOCSTR": SimTypeInt(signed=False, label="ASSOCSTR"),
    "ASSOC_FILTER": SimTypeInt(signed=False, label="ASSOC_FILTER"),
    "ASYNC_COMPLETION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["dwRequestID", "lResult"]), offset=0),
    "ASYNC_DUPLICATE_EXTENTS_STATUS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="DUPLICATE_EXTENTS_STATE")), ("SourceFileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("TargetFileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("BytesDuplicated", SimTypeLongLong(signed=False, label="UInt64")),)), name="ASYNC_DUPLICATE_EXTENTS_STATUS", pack=False, align=None),
    "ATA_PASS_THROUGH_DIRECT": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("AtaFlags", SimTypeShort(signed=False, label="UInt16")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("ReservedAsUchar", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("ReservedAsUlong", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PreviousTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("CurrentTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="ATA_PASS_THROUGH_DIRECT", pack=False, align=None),
    "ATA_PASS_THROUGH_DIRECT32": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("AtaFlags", SimTypeShort(signed=False, label="UInt16")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("ReservedAsUchar", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("ReservedAsUlong", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PreviousTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("CurrentTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="ATA_PASS_THROUGH_DIRECT32", pack=False, align=None),
    "ATA_PASS_THROUGH_EX": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("AtaFlags", SimTypeShort(signed=False, label="UInt16")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("ReservedAsUchar", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("ReservedAsUlong", SimTypeInt(signed=False, label="UInt32")), ("DataBufferOffset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PreviousTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("CurrentTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="ATA_PASS_THROUGH_EX", pack=False, align=None),
    "ATA_PASS_THROUGH_EX32": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("AtaFlags", SimTypeShort(signed=False, label="UInt16")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("ReservedAsUchar", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("ReservedAsUlong", SimTypeInt(signed=False, label="UInt32")), ("DataBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("PreviousTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("CurrentTaskFile", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="ATA_PASS_THROUGH_EX32", pack=False, align=None),
    "ATF_FLAGS": SimTypeInt(signed=False, label="ATF_FLAGS"),
    "ATM_ADDRESS": SimStruct(OrderedDict((("AddressType", SimTypeInt(signed=False, label="UInt32")), ("NumofDigits", SimTypeInt(signed=False, label="UInt32")), ("Addr", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="ATM_ADDRESS", pack=False, align=None),
    "ATM_BHLI": SimStruct(OrderedDict((("HighLayerInfoType", SimTypeInt(signed=False, label="UInt32")), ("HighLayerInfoLength", SimTypeInt(signed=False, label="UInt32")), ("HighLayerInfo", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="ATM_BHLI", pack=False, align=None),
    "ATM_BLLI": SimStruct(OrderedDict((("Layer2Protocol", SimTypeInt(signed=False, label="UInt32")), ("Layer2UserSpecifiedProtocol", SimTypeInt(signed=False, label="UInt32")), ("Layer3Protocol", SimTypeInt(signed=False, label="UInt32")), ("Layer3UserSpecifiedProtocol", SimTypeInt(signed=False, label="UInt32")), ("Layer3IPI", SimTypeInt(signed=False, label="UInt32")), ("SnapID", SimTypeArray(SimTypeChar(label="Byte"), 5)),)), name="ATM_BLLI", pack=False, align=None),
    "ATM_BLLI_IE": SimStruct(OrderedDict((("Layer2Protocol", SimTypeInt(signed=False, label="UInt32")), ("Layer2Mode", SimTypeChar(label="Byte")), ("Layer2WindowSize", SimTypeChar(label="Byte")), ("Layer2UserSpecifiedProtocol", SimTypeInt(signed=False, label="UInt32")), ("Layer3Protocol", SimTypeInt(signed=False, label="UInt32")), ("Layer3Mode", SimTypeChar(label="Byte")), ("Layer3DefaultPacketSize", SimTypeChar(label="Byte")), ("Layer3PacketWindowSize", SimTypeChar(label="Byte")), ("Layer3UserSpecifiedProtocol", SimTypeInt(signed=False, label="UInt32")), ("Layer3IPI", SimTypeInt(signed=False, label="UInt32")), ("SnapID", SimTypeArray(SimTypeChar(label="Byte"), 5)),)), name="ATM_BLLI_IE", pack=False, align=None),
    "ATM_BROADBAND_BEARER_CAPABILITY_IE": SimStruct(OrderedDict((("BearerClass", SimTypeChar(label="Byte")), ("TrafficType", SimTypeChar(label="Byte")), ("TimingRequirements", SimTypeChar(label="Byte")), ("ClippingSusceptability", SimTypeChar(label="Byte")), ("UserPlaneConnectionConfig", SimTypeChar(label="Byte")),)), name="ATM_BROADBAND_BEARER_CAPABILITY_IE", pack=False, align=None),
    "ATM_CALLING_PARTY_NUMBER_IE": SimStruct(OrderedDict((("ATM_Number", SimTypeRef("ATM_ADDRESS", SimStruct)), ("Presentation_Indication", SimTypeChar(label="Byte")), ("Screening_Indicator", SimTypeChar(label="Byte")),)), name="ATM_CALLING_PARTY_NUMBER_IE", pack=False, align=None),
    "ATM_CAUSE_IE": SimStruct(OrderedDict((("Location", SimTypeChar(label="Byte")), ("Cause", SimTypeChar(label="Byte")), ("DiagnosticsLength", SimTypeChar(label="Byte")), ("Diagnostics", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="ATM_CAUSE_IE", pack=False, align=None),
    "ATM_CONNECTION_ID": SimStruct(OrderedDict((("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("VPI", SimTypeInt(signed=False, label="UInt32")), ("VCI", SimTypeInt(signed=False, label="UInt32")),)), name="ATM_CONNECTION_ID", pack=False, align=None),
    "ATM_PVC_PARAMS": SimStruct(OrderedDict((("PvcConnectionId", SimTypeRef("ATM_CONNECTION_ID", SimStruct)), ("PvcQos", SimTypeRef("QOS", SimStruct)),)), name="ATM_PVC_PARAMS", pack=False, align=None),
    "ATM_QOS_CLASS_IE": SimStruct(OrderedDict((("QOSClassForward", SimTypeChar(label="Byte")), ("QOSClassBackward", SimTypeChar(label="Byte")),)), name="ATM_QOS_CLASS_IE", pack=False, align=None),
    "ATM_TD": SimStruct(OrderedDict((("PeakCellRate_CLP0", SimTypeInt(signed=False, label="UInt32")), ("PeakCellRate_CLP01", SimTypeInt(signed=False, label="UInt32")), ("SustainableCellRate_CLP0", SimTypeInt(signed=False, label="UInt32")), ("SustainableCellRate_CLP01", SimTypeInt(signed=False, label="UInt32")), ("MaxBurstSize_CLP0", SimTypeInt(signed=False, label="UInt32")), ("MaxBurstSize_CLP01", SimTypeInt(signed=False, label="UInt32")), ("Tagging", SimTypeInt(signed=True, label="Int32")),)), name="ATM_TD", pack=False, align=None),
    "ATM_TRAFFIC_DESCRIPTOR_IE": SimStruct(OrderedDict((("Forward", SimTypeRef("ATM_TD", SimStruct)), ("Backward", SimTypeRef("ATM_TD", SimStruct)), ("BestEffort", SimTypeInt(signed=True, label="Int32")),)), name="ATM_TRAFFIC_DESCRIPTOR_IE", pack=False, align=None),
    "ATM_TRANSIT_NETWORK_SELECTION_IE": SimStruct(OrderedDict((("TypeOfNetworkId", SimTypeChar(label="Byte")), ("NetworkIdPlan", SimTypeChar(label="Byte")), ("NetworkIdLength", SimTypeChar(label="Byte")), ("NetworkId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ATM_TRANSIT_NETWORK_SELECTION_IE", pack=False, align=None),
    "ATOMIC_CREATE_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("InFlags", SimTypeShort(signed=False, label="UInt16")), ("OutFlags", SimTypeShort(signed=False, label="UInt16")), ("ReparseBufferLength", SimTypeShort(signed=False, label="UInt16")), ("ReparseBuffer", SimTypePointer(SimTypeRef("REPARSE_DATA_BUFFER", SimStruct), offset=0)), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("ValidDataLength", SimTypeLongLong(signed=True, label="Int64")), ("FileTimestamps", SimTypePointer(SimTypeRef("FILE_TIMESTAMPS", SimStruct), offset=0)), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("UsnSourceInfo", SimTypeInt(signed=False, label="UInt32")), ("Usn", SimTypeLongLong(signed=True, label="Int64")), ("SuppressFileAttributeInheritanceMask", SimTypeInt(signed=False, label="UInt32")), ("InOpFlags", SimTypeInt(signed=False, label="UInt32")), ("OutOpFlags", SimTypeInt(signed=False, label="UInt32")), ("InGenFlags", SimTypeInt(signed=False, label="UInt32")), ("OutGenFlags", SimTypeInt(signed=False, label="UInt32")), ("CaseSensitiveFlagsMask", SimTypeInt(signed=False, label="UInt32")), ("InCaseSensitiveFlags", SimTypeInt(signed=False, label="UInt32")), ("OutCaseSensitiveFlags", SimTypeInt(signed=False, label="UInt32")),)), name="ATOMIC_CREATE_ECP_CONTEXT", pack=False, align=None),
    "ATSCComponentTypeFlags": SimTypeInt(signed=False, label="ATSCComponentTypeFlags"),
    "ATSC_FILTER_OPTIONS": SimStruct(OrderedDict((("fSpecifyEtmId", SimTypeInt(signed=True, label="Int32")), ("EtmId", SimTypeInt(signed=False, label="UInt32")),)), name="ATSC_FILTER_OPTIONS", pack=False, align=None),
    "ATTACHLIST": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpLink", SimTypePointer(SimTypeRef("ATTACHLIST", SimStruct), offset=0)), ("lpAttached", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpIAttached", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)),)), name="ATTACHLIST", pack=False, align=None),
    "ATTACHMENT_ACTION": SimTypeInt(signed=False, label="ATTACHMENT_ACTION"),
    "ATTACHMENT_PROMPT": SimTypeInt(signed=False, label="ATTACHMENT_PROMPT"),
    "ATTACH_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="ATTACH_VIRTUAL_DISK_FLAG"),
    "ATTACH_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="ATTACH_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct), "Version2": SimTypeRef("_Version2_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="ATTACH_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "ATTACH_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="ATTACH_VIRTUAL_DISK_VERSION"),
    "ATTENDEE_DISCONNECT_REASON": SimTypeInt(signed=False, label="ATTENDEE_DISCONNECT_REASON"),
    "ATTRIBUTEFILTER": SimTypeInt(signed=False, label="ATTRIBUTEFILTER"),
    "ATTRIBUTEID": SimTypeInt(signed=False, label="ATTRIBUTEID"),
    "ATTRIBUTEINFO": SimTypeInt(signed=False, label="ATTRIBUTEINFO"),
    "ATTRIBUTEPROPERTIES": SimTypeInt(signed=False, label="ATTRIBUTEPROPERTIES"),
    "ATTRIBUTERESTRICTIONS": SimTypeInt(signed=False, label="ATTRIBUTERESTRICTIONS"),
    "ATTRIBUTESYNTAX": SimTypeInt(signed=False, label="ATTRIBUTESYNTAX"),
    "ATTRIBUTE_INFO_1": SimStruct(OrderedDict((("dwJobNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwNupBorderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDrvPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfCopies", SimTypeInt(signed=False, label="UInt32")),)), name="ATTRIBUTE_INFO_1", pack=False, align=None),
    "ATTRIBUTE_INFO_2": SimStruct(OrderedDict((("dwJobNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwNupBorderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDrvPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwColorOptimization", SimTypeInt(signed=False, label="UInt32")),)), name="ATTRIBUTE_INFO_2", pack=False, align=None),
    "ATTRIBUTE_INFO_3": SimStruct(OrderedDict((("dwJobNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwNupBorderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDrvPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwColorOptimization", SimTypeInt(signed=False, label="UInt32")), ("dmPrintQuality", SimTypeShort(signed=True, label="Int16")), ("dmYResolution", SimTypeShort(signed=True, label="Int16")),)), name="ATTRIBUTE_INFO_3", pack=False, align=None),
    "ATTRIBUTE_INFO_4": SimStruct(OrderedDict((("dwJobNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfPagesPerSide", SimTypeInt(signed=False, label="UInt32")), ("dwNupBorderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDrvPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwJobNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwDrvNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwColorOptimization", SimTypeInt(signed=False, label="UInt32")), ("dmPrintQuality", SimTypeShort(signed=True, label="Int16")), ("dmYResolution", SimTypeShort(signed=True, label="Int16")), ("dwDuplexFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNupDirection", SimTypeInt(signed=False, label="UInt32")), ("dwBookletFlags", SimTypeInt(signed=False, label="UInt32")), ("dwScalingPercentX", SimTypeInt(signed=False, label="UInt32")), ("dwScalingPercentY", SimTypeInt(signed=False, label="UInt32")),)), name="ATTRIBUTE_INFO_4", pack=False, align=None),
    "ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="ATTRIBUTE_TYPE"),
    "AT_ENUM": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("JobTime", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DaysOfMonth", SimTypeInt(signed=False, label="UInt32")), ("DaysOfWeek", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Command", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AT_ENUM", pack=False, align=None),
    "AT_INFO": SimStruct(OrderedDict((("JobTime", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DaysOfMonth", SimTypeInt(signed=False, label="UInt32")), ("DaysOfWeek", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Command", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AT_INFO", pack=False, align=None),
    "AUDCLNT_SHAREMODE": SimTypeInt(signed=False, label="AUDCLNT_SHAREMODE"),
    "AUDCLNT_STREAMOPTIONS": SimTypeInt(signed=False, label="AUDCLNT_STREAMOPTIONS"),
    "AUDIOCLIENT_ACTIVATION_PARAMS": SimStruct(OrderedDict((("ActivationType", SimTypeInt(signed=False, label="AUDIOCLIENT_ACTIVATION_TYPE")), ("Anonymous", SimUnion({"ProcessLoopbackParams": SimTypeRef("AUDIOCLIENT_PROCESS_LOOPBACK_PARAMS", SimStruct)}, name="<anon>", label="None")),)), name="AUDIOCLIENT_ACTIVATION_PARAMS", pack=False, align=None),
    "AUDIOCLIENT_ACTIVATION_TYPE": SimTypeInt(signed=False, label="AUDIOCLIENT_ACTIVATION_TYPE"),
    "AUDIOCLIENT_PROCESS_LOOPBACK_PARAMS": SimStruct(OrderedDict((("TargetProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessLoopbackMode", SimTypeInt(signed=False, label="PROCESS_LOOPBACK_MODE")),)), name="AUDIOCLIENT_PROCESS_LOOPBACK_PARAMS", pack=False, align=None),
    "AUDIODESCRIPTION": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("Locale", SimTypeInt(signed=False, label="UInt32")),)), name="AUDIODESCRIPTION", pack=False, align=None),
    "AUDIOFILE_AF10WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="AUDIOFILE_AF10WAVEFORMAT", pack=False, align=None),
    "AUDIOFILE_AF36WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="AUDIOFILE_AF36WAVEFORMAT", pack=False, align=None),
    "AUDIOPOSTURE_ORIENTATION": SimTypeInt(signed=False, label="AUDIOPOSTURE_ORIENTATION"),
    "AUDIORESOURCEMANAGEMENT_RESOURCEGROUP": SimStruct(OrderedDict((("ResourceGroupAcquired", SimTypeInt(signed=True, label="Int32")), ("ResourceGroupName", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="AUDIORESOURCEMANAGEMENT_RESOURCEGROUP", pack=False, align=None),
    "AUDIO_CURVE_TYPE": SimTypeInt(signed=False, label="AUDIO_CURVE_TYPE"),
    "AUDIO_DUCKING_OPTIONS": SimTypeInt(signed=False, label="AUDIO_DUCKING_OPTIONS"),
    "AUDIO_EFFECT": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("canSetState", SimTypeInt(signed=True, label="Int32")), ("state", SimTypeInt(signed=False, label="AUDIO_EFFECT_STATE")),)), name="AUDIO_EFFECT", pack=False, align=None),
    "AUDIO_EFFECT_STATE": SimTypeInt(signed=False, label="AUDIO_EFFECT_STATE"),
    "AUDIO_ENDPOINT_PROPERTY_CHANGE_APO_NOTIFICATION_DESCRIPTOR": SimStruct(OrderedDict((("device", SimTypeBottom(label="IMMDevice")),)), name="AUDIO_ENDPOINT_PROPERTY_CHANGE_APO_NOTIFICATION_DESCRIPTOR", pack=False, align=None),
    "AUDIO_ENDPOINT_PROPERTY_CHANGE_NOTIFICATION": SimStruct(OrderedDict((("endpoint", SimTypeBottom(label="IMMDevice")), ("propertyStore", SimTypeBottom(label="IPropertyStore")), ("propertyKey", SimTypeRef("PROPERTYKEY", SimStruct)),)), name="AUDIO_ENDPOINT_PROPERTY_CHANGE_NOTIFICATION", pack=False, align=None),
    "AUDIO_ENDPOINT_SHARED_CREATE_PARAMS": SimStruct(OrderedDict((("u32Size", SimTypeInt(signed=False, label="UInt32")), ("u32TSSessionId", SimTypeInt(signed=False, label="UInt32")), ("targetEndpointConnectorType", SimTypeInt(signed=False, label="EndpointConnectorType")), ("wfxDeviceFormat", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="AUDIO_ENDPOINT_SHARED_CREATE_PARAMS", pack=False, align=None),
    "AUDIO_ENDPOINT_VOLUME_APO_NOTIFICATION_DESCRIPTOR": SimStruct(OrderedDict((("device", SimTypeBottom(label="IMMDevice")),)), name="AUDIO_ENDPOINT_VOLUME_APO_NOTIFICATION_DESCRIPTOR", pack=False, align=None),
    "AUDIO_ENDPOINT_VOLUME_CHANGE_NOTIFICATION": SimStruct(OrderedDict((("endpoint", SimTypeBottom(label="IMMDevice")), ("volume", SimTypePointer(SimTypeRef("AUDIO_VOLUME_NOTIFICATION_DATA", SimStruct), offset=0)),)), name="AUDIO_ENDPOINT_VOLUME_CHANGE_NOTIFICATION", pack=False, align=None),
    "AUDIO_ENDPOINT_VOLUME_CHANGE_NOTIFICATION2": SimStruct(OrderedDict((("endpoint", SimTypeBottom(label="IMMDevice")), ("volume", SimTypePointer(SimTypeRef("AUDIO_VOLUME_NOTIFICATION_DATA2", SimStruct), offset=0)),)), name="AUDIO_ENDPOINT_VOLUME_CHANGE_NOTIFICATION2", pack=False, align=None),
    "AUDIO_FLOW_TYPE": SimTypeInt(signed=False, label="AUDIO_FLOW_TYPE"),
    "AUDIO_MICROPHONE_BOOST_APO_NOTIFICATION_DESCRIPTOR": SimStruct(OrderedDict((("device", SimTypeBottom(label="IMMDevice")),)), name="AUDIO_MICROPHONE_BOOST_APO_NOTIFICATION_DESCRIPTOR", pack=False, align=None),
    "AUDIO_MICROPHONE_BOOST_NOTIFICATION": SimStruct(OrderedDict((("endpoint", SimTypeBottom(label="IMMDevice")), ("eventContext", SimTypeBottom(label="Guid")), ("microphoneBoostEnabled", SimTypeInt(signed=True, label="Int32")), ("levelInDb", SimTypeFloat(size=32)), ("levelMinInDb", SimTypeFloat(size=32)), ("levelMaxInDb", SimTypeFloat(size=32)), ("levelStepInDb", SimTypeFloat(size=32)), ("muteSupported", SimTypeInt(signed=True, label="Int32")), ("mute", SimTypeInt(signed=True, label="Int32")),)), name="AUDIO_MICROPHONE_BOOST_NOTIFICATION", pack=False, align=None),
    "AUDIO_STREAM_CATEGORY": SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY"),
    "AUDIO_STREAM_CONFIG_CAPS": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("MinimumChannels", SimTypeInt(signed=False, label="UInt32")), ("MaximumChannels", SimTypeInt(signed=False, label="UInt32")), ("ChannelsGranularity", SimTypeInt(signed=False, label="UInt32")), ("MinimumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MaximumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("BitsPerSampleGranularity", SimTypeInt(signed=False, label="UInt32")), ("MinimumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaximumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("SampleFrequencyGranularity", SimTypeInt(signed=False, label="UInt32")),)), name="AUDIO_STREAM_CONFIG_CAPS", pack=False, align=None),
    "AUDIO_SYSTEMEFFECT": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("canSetState", SimTypeInt(signed=True, label="Int32")), ("state", SimTypeInt(signed=False, label="AUDIO_SYSTEMEFFECT_STATE")),)), name="AUDIO_SYSTEMEFFECT", pack=False, align=None),
    "AUDIO_SYSTEMEFFECTS_PROPERTYSTORE_TYPE": SimTypeInt(signed=False, label="AUDIO_SYSTEMEFFECTS_PROPERTYSTORE_TYPE"),
    "AUDIO_SYSTEMEFFECTS_PROPERTY_CHANGE_APO_NOTIFICATION_DESCRIPTOR": SimStruct(OrderedDict((("device", SimTypeBottom(label="IMMDevice")), ("propertyStoreContext", SimTypeBottom(label="Guid")),)), name="AUDIO_SYSTEMEFFECTS_PROPERTY_CHANGE_APO_NOTIFICATION_DESCRIPTOR", pack=False, align=None),
    "AUDIO_SYSTEMEFFECTS_PROPERTY_CHANGE_NOTIFICATION": SimStruct(OrderedDict((("endpoint", SimTypeBottom(label="IMMDevice")), ("propertyStoreContext", SimTypeBottom(label="Guid")), ("propertyStoreType", SimTypeInt(signed=False, label="AUDIO_SYSTEMEFFECTS_PROPERTYSTORE_TYPE")), ("propertyStore", SimTypeBottom(label="IPropertyStore")), ("propertyKey", SimTypeRef("PROPERTYKEY", SimStruct)),)), name="AUDIO_SYSTEMEFFECTS_PROPERTY_CHANGE_NOTIFICATION", pack=False, align=None),
    "AUDIO_SYSTEMEFFECT_STATE": SimTypeInt(signed=False, label="AUDIO_SYSTEMEFFECT_STATE"),
    "AUDIO_VOLUME_NOTIFICATION_DATA": SimStruct(OrderedDict((("guidEventContext", SimTypeBottom(label="Guid")), ("bMuted", SimTypeInt(signed=True, label="Int32")), ("fMasterVolume", SimTypeFloat(size=32)), ("nChannels", SimTypeInt(signed=False, label="UInt32")), ("afChannelVolumes", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="AUDIO_VOLUME_NOTIFICATION_DATA", pack=False, align=None),
    "AUDIO_VOLUME_NOTIFICATION_DATA2": SimStruct(OrderedDict((("notificationData", SimTypePointer(SimTypeRef("AUDIO_VOLUME_NOTIFICATION_DATA", SimStruct), offset=0)), ("masterVolumeInDb", SimTypeFloat(size=32)), ("volumeMinInDb", SimTypeFloat(size=32)), ("volumeMaxInDb", SimTypeFloat(size=32)), ("volumeIncrementInDb", SimTypeFloat(size=32)), ("step", SimTypeInt(signed=False, label="UInt32")), ("stepCount", SimTypeInt(signed=False, label="UInt32")), ("channelVolumesInDb", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="AUDIO_VOLUME_NOTIFICATION_DATA2", pack=False, align=None),
    "AUDIT_ENTRY": SimStruct(OrderedDict((("ae_len", SimTypeInt(signed=False, label="UInt32")), ("ae_reserved", SimTypeInt(signed=False, label="UInt32")), ("ae_time", SimTypeInt(signed=False, label="UInt32")), ("ae_type", SimTypeInt(signed=False, label="UInt32")), ("ae_data_offset", SimTypeInt(signed=False, label="UInt32")), ("ae_data_size", SimTypeInt(signed=False, label="UInt32")),)), name="AUDIT_ENTRY", pack=False, align=None),
    "AUDIT_EVENT_TYPE": SimTypeInt(signed=False, label="AUDIT_EVENT_TYPE"),
    "AUDIT_IP_ADDRESS": SimStruct(OrderedDict((("pIpAddress", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="AUDIT_IP_ADDRESS", pack=False, align=None),
    "AUDIT_OBJECT_TYPE": SimStruct(OrderedDict((("ObjectType", SimTypeBottom(label="Guid")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Level", SimTypeShort(signed=False, label="UInt16")), ("AccessMask", SimTypeInt(signed=False, label="UInt32")),)), name="AUDIT_OBJECT_TYPE", pack=False, align=None),
    "AUDIT_OBJECT_TYPES": SimStruct(OrderedDict((("Count", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("pObjectTypes", SimTypePointer(SimTypeRef("AUDIT_OBJECT_TYPE", SimStruct), offset=0)),)), name="AUDIT_OBJECT_TYPES", pack=False, align=None),
    "AUDIT_PARAM": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="AUDIT_PARAM_TYPE")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"Data0": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "String": SimTypePointer(SimTypeChar(label="Char"), offset=0), "u": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "psid": SimTypePointer(SimTypeRef("SID", SimStruct), offset=0), "pguid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "LogonId_LowPart": SimTypeInt(signed=False, label="UInt32"), "pObjectTypes": SimTypePointer(SimTypeRef("AUDIT_OBJECT_TYPES", SimStruct), offset=0), "pIpAddress": SimTypePointer(SimTypeRef("AUDIT_IP_ADDRESS", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Data1": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "LogonId_HighPart": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="AUDIT_PARAM", pack=False, align=None),
    "AUDIT_PARAMS": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeShort(signed=False, label="UInt16")), ("Parameters", SimTypePointer(SimTypeRef("AUDIT_PARAM", SimStruct), offset=0)),)), name="AUDIT_PARAMS", pack=False, align=None),
    "AUDIT_PARAM_TYPE": SimTypeInt(signed=False, label="AUDIT_PARAM_TYPE"),
    "AUDIT_POLICY_INFORMATION": SimStruct(OrderedDict((("AuditSubCategoryGuid", SimTypeBottom(label="Guid")), ("AuditingInformation", SimTypeInt(signed=False, label="UInt32")), ("AuditCategoryGuid", SimTypeBottom(label="Guid")),)), name="AUDIT_POLICY_INFORMATION", pack=False, align=None),
    "AUTHENTICATEF": SimTypeInt(signed=False, label="AUTHENTICATEF"),
    "AUTHENTICATEINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="AUTHENTICATEINFO", pack=False, align=None),
    "AUTHENTICATION_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("atAuthenticationType", SimTypeInt(signed=False, label="AUTH_TYPE")), ("pcwszUser", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AUTHENTICATION_INFO", pack=False, align=None),
    "AUTHENTICATION_LEVEL": SimTypeInt(signed=False, label="AUTHENTICATION_LEVEL"),
    "AUTHENTICATION_REQUIREMENTS": SimTypeInt(signed=False, label="AUTHENTICATION_REQUIREMENTS"),
    "AUTHENTICATION_TYPE": SimTypeInt(signed=False, label="AUTHENTICATION_TYPE"),
    "AUTHENTICODE_EXTRA_CERT_CHAIN_POLICY_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRegPolicySettings", SimTypeInt(signed=False, label="UInt32")), ("pSignerInfo", SimTypePointer(SimTypeRef("CMSG_SIGNER_INFO", SimStruct), offset=0)),)), name="AUTHENTICODE_EXTRA_CERT_CHAIN_POLICY_PARA", pack=False, align=None),
    "AUTHENTICODE_EXTRA_CERT_CHAIN_POLICY_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fCommercial", SimTypeInt(signed=True, label="Int32")),)), name="AUTHENTICODE_EXTRA_CERT_CHAIN_POLICY_STATUS", pack=False, align=None),
    "AUTHENTICODE_TS_EXTRA_CERT_CHAIN_POLICY_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRegPolicySettings", SimTypeInt(signed=False, label="UInt32")), ("fCommercial", SimTypeInt(signed=True, label="Int32")),)), name="AUTHENTICODE_TS_EXTRA_CERT_CHAIN_POLICY_PARA", pack=False, align=None),
    "AUTHNEXTSTEP": SimTypeInt(signed=False, label="AUTHNEXTSTEP"),
    "AUTHZ_ACCESS_CHECK_FLAGS": SimTypeInt(signed=False, label="AUTHZ_ACCESS_CHECK_FLAGS"),
    "AUTHZ_ACCESS_CHECK_RESULTS_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_ACCESS_REPLY": SimStruct(OrderedDict((("ResultListLength", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccessMask", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("SaclEvaluationResults", SimTypePointer(SimTypeInt(signed=False, label="AUTHZ_GENERATE_RESULTS"), offset=0)), ("Error", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="AUTHZ_ACCESS_REPLY", pack=False, align=None),
    "AUTHZ_ACCESS_REQUEST": SimStruct(OrderedDict((("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("PrincipalSelfSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectTypeList", SimTypePointer(SimTypeRef("OBJECT_TYPE_LIST", SimStruct), offset=0)), ("ObjectTypeListLength", SimTypeInt(signed=False, label="UInt32")), ("OptionalArguments", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="AUTHZ_ACCESS_REQUEST", pack=False, align=None),
    "AUTHZ_AUDIT_EVENT_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_AUDIT_EVENT_INFORMATION_CLASS": SimTypeInt(signed=False, label="AUTHZ_AUDIT_EVENT_INFORMATION_CLASS"),
    "AUTHZ_AUDIT_EVENT_TYPE_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_AUDIT_EVENT_TYPE_LEGACY": SimStruct(OrderedDict((("CategoryId", SimTypeShort(signed=False, label="UInt16")), ("AuditId", SimTypeShort(signed=False, label="UInt16")), ("ParameterCount", SimTypeShort(signed=False, label="UInt16")),)), name="AUTHZ_AUDIT_EVENT_TYPE_LEGACY", pack=False, align=None),
    "AUTHZ_AUDIT_EVENT_TYPE_OLD": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("RefCount", SimTypeInt(signed=True, label="Int32")), ("hAudit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("LinkId", SimTypeRef("LUID", SimStruct)), ("u", SimUnion({"Legacy": SimTypeRef("AUTHZ_AUDIT_EVENT_TYPE_LEGACY", SimStruct)}, name="<anon>", label="None")),)), name="AUTHZ_AUDIT_EVENT_TYPE_OLD", pack=False, align=None),
    "AUTHZ_AUDIT_EVENT_TYPE_UNION": SimUnion({"Legacy": SimTypeRef("AUTHZ_AUDIT_EVENT_TYPE_LEGACY", SimStruct)}, name="<anon>", label="None"),
    "AUTHZ_CAP_CHANGE_SUBSCRIPTION_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_CLIENT_CONTEXT_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_CONTEXT_INFORMATION_CLASS": SimTypeInt(signed=False, label="AUTHZ_CONTEXT_INFORMATION_CLASS"),
    "AUTHZ_GENERATE_RESULTS": SimTypeInt(signed=False, label="AUTHZ_GENERATE_RESULTS"),
    "AUTHZ_INITIALIZE_OBJECT_ACCESS_AUDIT_EVENT_FLAGS": SimTypeInt(signed=False, label="AUTHZ_INITIALIZE_OBJECT_ACCESS_AUDIT_EVENT_FLAGS"),
    "AUTHZ_INIT_INFO": SimStruct(OrderedDict((("version", SimTypeShort(signed=False, label="UInt16")), ("szResourceManagerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnDynamicAccessCheck", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACE_HEADER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAuthzClientContext", "pAce", "pArgs", "pbAceApplicable"]), offset=0)), ("pfnComputeDynamicGroups", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAuthzClientContext", "Args", "pSidAttrArray", "pSidCount", "pRestrictedSidAttrArray", "pRestrictedSidCount"]), offset=0)), ("pfnFreeDynamicGroups", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pSidAttrArray"]), offset=0)), ("pfnGetCentralAccessPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAuthzClientContext", "capid", "pArgs", "pCentralAccessPolicyApplicable", "ppCentralAccessPolicy"]), offset=0)), ("pfnFreeCentralAccessPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pCentralAccessPolicy"]), offset=0)),)), name="AUTHZ_INIT_INFO", pack=False, align=None),
    "AUTHZ_REGISTRATION_OBJECT_TYPE_NAME_OFFSET": SimStruct(OrderedDict((("szObjectTypeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="AUTHZ_REGISTRATION_OBJECT_TYPE_NAME_OFFSET", pack=False, align=None),
    "AUTHZ_RESOURCE_MANAGER_FLAGS": SimTypeInt(signed=False, label="AUTHZ_RESOURCE_MANAGER_FLAGS"),
    "AUTHZ_RESOURCE_MANAGER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_RPC_INIT_INFO_CLIENT": SimStruct(OrderedDict((("version", SimTypeShort(signed=False, label="UInt16")), ("ObjectUuid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProtSeq", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NetworkAddr", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Endpoint", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Options", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServerSpn", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AUTHZ_RPC_INIT_INFO_CLIENT", pack=False, align=None),
    "AUTHZ_SECURITY_ATTRIBUTES_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("AttributeCount", SimTypeInt(signed=False, label="UInt32")), ("Attribute", SimUnion({"pAttributeV1": SimTypePointer(SimTypeRef("AUTHZ_SECURITY_ATTRIBUTE_V1", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="AUTHZ_SECURITY_ATTRIBUTES_INFORMATION", pack=False, align=None),
    "AUTHZ_SECURITY_ATTRIBUTE_FLAGS": SimTypeInt(signed=False, label="AUTHZ_SECURITY_ATTRIBUTE_FLAGS"),
    "AUTHZ_SECURITY_ATTRIBUTE_FQBN_VALUE": SimStruct(OrderedDict((("Version", SimTypeLongLong(signed=False, label="UInt64")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="AUTHZ_SECURITY_ATTRIBUTE_FQBN_VALUE", pack=False, align=None),
    "AUTHZ_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE": SimStruct(OrderedDict((("pValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValueLength", SimTypeInt(signed=False, label="UInt32")),)), name="AUTHZ_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE", pack=False, align=None),
    "AUTHZ_SECURITY_ATTRIBUTE_OPERATION": SimTypeInt(signed=False, label="AUTHZ_SECURITY_ATTRIBUTE_OPERATION"),
    "AUTHZ_SECURITY_ATTRIBUTE_V1": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ValueType", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="AUTHZ_SECURITY_ATTRIBUTE_FLAGS")), ("ValueCount", SimTypeInt(signed=False, label="UInt32")), ("Values", SimUnion({"pInt64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pUint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "ppString": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "pFqbn": SimTypePointer(SimTypeRef("AUTHZ_SECURITY_ATTRIBUTE_FQBN_VALUE", SimStruct), offset=0), "pOctetString": SimTypePointer(SimTypeRef("AUTHZ_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="AUTHZ_SECURITY_ATTRIBUTE_V1", pack=False, align=None),
    "AUTHZ_SECURITY_EVENT_PROVIDER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AUTHZ_SID_OPERATION": SimTypeInt(signed=False, label="AUTHZ_SID_OPERATION"),
    "AUTHZ_SOURCE_SCHEMA_REGISTRATION": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szEventSourceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szEventMessageFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szEventSourceXmlSchemaFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szEventAccessStringsFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szExecutableImagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"pReserved": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pProviderGuid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="<anon>", label="None")), ("dwObjectTypeNameCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectTypeNames", SimTypePointer(SimTypeRef("AUTHZ_REGISTRATION_OBJECT_TYPE_NAME_OFFSET", SimStruct), offset=0)),)), name="AUTHZ_SOURCE_SCHEMA_REGISTRATION", pack=False, align=None),
    "AUTH_TYPE": SimTypeInt(signed=False, label="AUTH_TYPE"),
    "AUTH_VALIDATION_EX": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("hRasConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszLogonDomain", SimTypeArray(SimTypeChar(label="Char"), 16)), ("AuthInfoSize", SimTypeInt(signed=False, label="UInt32")), ("AuthInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="AUTH_VALIDATION_EX", pack=False, align=None),
    "AUTOCOMPLETELISTOPTIONS": SimTypeInt(signed=False, label="AUTOCOMPLETELISTOPTIONS"),
    "AUTOCOMPLETEOPTIONS": SimTypeInt(signed=False, label="AUTOCOMPLETEOPTIONS"),
    "AUTO_PROXY_SCRIPT_BUFFER": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszScriptBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwScriptBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="AUTO_PROXY_SCRIPT_BUFFER", pack=False, align=None),
    "AUTO_SCROLL_DATA": SimStruct(OrderedDict((("iNextSample", SimTypeInt(signed=True, label="Int32")), ("dwLastScroll", SimTypeInt(signed=False, label="UInt32")), ("bFull", SimTypeInt(signed=True, label="Int32")), ("pts", SimTypeArray(SimTypeRef("POINT", SimStruct), 3)), ("dwTimes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="AUTO_SCROLL_DATA", pack=False, align=None),
    "AUXCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="AUXCAPS2A", pack=False, align=None),
    "AUXCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="AUXCAPS2W", pack=False, align=None),
    "AUXCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="AUXCAPSA", pack=False, align=None),
    "AUXCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="AUXCAPSW", pack=False, align=None),
    "AVICOMPRESSOPTIONS": SimStruct(OrderedDict((("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwKeyFrameEvery", SimTypeInt(signed=False, label="UInt32")), ("dwQuality", SimTypeInt(signed=False, label="UInt32")), ("dwBytesPerSecond", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpFormat", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbFormat", SimTypeInt(signed=False, label="UInt32")), ("lpParms", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbParms", SimTypeInt(signed=False, label="UInt32")), ("dwInterleaveEvery", SimTypeInt(signed=False, label="UInt32")),)), name="AVICOMPRESSOPTIONS", pack=False, align=None),
    "AVIEXTHEADER": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("dwGrandFrames", SimTypeInt(signed=False, label="UInt32")), ("dwFuture", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 61)),)), name="AVIEXTHEADER", pack=False, align=None),
    "AVIFIELDINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("qwBaseOffset", SimTypeLongLong(signed=False, label="UInt64")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("aIndex", SimTypePointer(SimTypeRef("_avifieldindex_entry", SimStruct), offset=0)),)), name="AVIFIELDINDEX", pack=False, align=None),
    "AVIFILEINFOA": SimStruct(OrderedDict((("dwMaxBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwStreams", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwEditCount", SimTypeInt(signed=False, label="UInt32")), ("szFileType", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="AVIFILEINFOA", pack=False, align=None),
    "AVIFILEINFOW": SimStruct(OrderedDict((("dwMaxBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwStreams", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwEditCount", SimTypeInt(signed=False, label="UInt32")), ("szFileType", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="AVIFILEINFOW", pack=False, align=None),
    "AVIINDEXENTRY": SimStruct(OrderedDict((("ckid", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwChunkOffset", SimTypeInt(signed=False, label="UInt32")), ("dwChunkLength", SimTypeInt(signed=False, label="UInt32")),)), name="AVIINDEXENTRY", pack=False, align=None),
    "AVIMAINHEADER": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("dwMicroSecPerFrame", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("dwPaddingGranularity", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTotalFrames", SimTypeInt(signed=False, label="UInt32")), ("dwInitialFrames", SimTypeInt(signed=False, label="UInt32")), ("dwStreams", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="AVIMAINHEADER", pack=False, align=None),
    "AVIMETAINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("adwIndex", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="AVIMETAINDEX", pack=False, align=None),
    "AVIOLDINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("aIndex", SimTypePointer(SimTypeRef("_avioldindex_entry", SimStruct), offset=0)),)), name="AVIOLDINDEX", pack=False, align=None),
    "AVIPALCHANGE": SimStruct(OrderedDict((("bFirstEntry", SimTypeChar(label="Byte")), ("bNumEntries", SimTypeChar(label="Byte")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("peNew", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)),)), name="AVIPALCHANGE", pack=False, align=None),
    "AVISAVECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "AVISTDINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("qwBaseOffset", SimTypeLongLong(signed=False, label="UInt64")), ("dwReserved_3", SimTypeInt(signed=False, label="UInt32")), ("aIndex", SimTypeArray(SimTypeRef("AVISTDINDEX_ENTRY", SimStruct), 2044)),)), name="AVISTDINDEX", pack=False, align=None),
    "AVISTDINDEX_ENTRY": SimStruct(OrderedDict((("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="AVISTDINDEX_ENTRY", pack=False, align=None),
    "AVISTREAMHEADER": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wLanguage", SimTypeShort(signed=False, label="UInt16")), ("dwInitialFrames", SimTypeInt(signed=False, label="UInt32")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwStart", SimTypeInt(signed=False, label="UInt32")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwQuality", SimTypeInt(signed=False, label="UInt32")), ("dwSampleSize", SimTypeInt(signed=False, label="UInt32")), ("rcFrame", SimTypeRef("_rcFrame_e__Struct", SimStruct)),)), name="AVISTREAMHEADER", pack=False, align=None),
    "AVISTREAMINFOA": SimStruct(OrderedDict((("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wLanguage", SimTypeShort(signed=False, label="UInt16")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwStart", SimTypeInt(signed=False, label="UInt32")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwInitialFrames", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwQuality", SimTypeInt(signed=False, label="UInt32")), ("dwSampleSize", SimTypeInt(signed=False, label="UInt32")), ("rcFrame", SimTypeRef("RECT", SimStruct)), ("dwEditCount", SimTypeInt(signed=False, label="UInt32")), ("dwFormatChangeCount", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="AVISTREAMINFOA", pack=False, align=None),
    "AVISTREAMINFOW": SimStruct(OrderedDict((("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wLanguage", SimTypeShort(signed=False, label="UInt16")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwStart", SimTypeInt(signed=False, label="UInt32")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwInitialFrames", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwQuality", SimTypeInt(signed=False, label="UInt32")), ("dwSampleSize", SimTypeInt(signed=False, label="UInt32")), ("rcFrame", SimTypeRef("RECT", SimStruct)), ("dwEditCount", SimTypeInt(signed=False, label="UInt32")), ("dwFormatChangeCount", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="AVISTREAMINFOW", pack=False, align=None),
    "AVISUPERINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("aIndex", SimTypeArray(SimTypeRef("_avisuperindex_entry", SimStruct), 1022)),)), name="AVISUPERINDEX", pack=False, align=None),
    "AVIStreamHeader": SimStruct(OrderedDict((("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wLanguage", SimTypeShort(signed=False, label="UInt16")), ("dwInitialFrames", SimTypeInt(signed=False, label="UInt32")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwStart", SimTypeInt(signed=False, label="UInt32")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwQuality", SimTypeInt(signed=False, label="UInt32")), ("dwSampleSize", SimTypeInt(signed=False, label="UInt32")), ("rcFrame", SimTypeRef("RECT", SimStruct)),)), name="AVIStreamHeader", pack=False, align=None),
    "AVITCDLINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("aIndex", SimTypeArray(SimTypeRef("AVITCDLINDEX_ENTRY", SimStruct), 584)), ("adwTrailingFill", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3512)),)), name="AVITCDLINDEX", pack=False, align=None),
    "AVITCDLINDEX_ENTRY": SimStruct(OrderedDict((("dwTick", SimTypeInt(signed=False, label="UInt32")), ("time", SimUnion({"Anonymous": SimStruct(OrderedDict((("wFrameRate", SimTypeShort(signed=False, label="UInt16")), ("wFrameFract", SimTypeShort(signed=False, label="UInt16")), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "qw": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("dwSMPTEflags", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypeInt(signed=False, label="UInt32")), ("szReelId", SimTypeArray(SimTypeChar(label="SByte"), 12)),)), name="AVITCDLINDEX_ENTRY", pack=False, align=None),
    "AVITIMECODEINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("aIndex", SimTypeArray(SimTypeRef("TIMECODEDATA", SimStruct), 1022)),)), name="AVITIMECODEINDEX", pack=False, align=None),
    "AVITIMEDINDEX": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("wLongsPerEntry", SimTypeShort(signed=False, label="UInt16")), ("bIndexSubType", SimTypeChar(label="Byte")), ("bIndexType", SimTypeChar(label="Byte")), ("nEntriesInUse", SimTypeInt(signed=False, label="UInt32")), ("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("qwBaseOffset", SimTypeLongLong(signed=False, label="UInt64")), ("dwReserved_3", SimTypeInt(signed=False, label="UInt32")), ("aIndex", SimTypeArray(SimTypeRef("AVITIMEDINDEX_ENTRY", SimStruct), 1362)), ("adwTrailingFill", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2734)),)), name="AVITIMEDINDEX", pack=False, align=None),
    "AVITIMEDINDEX_ENTRY": SimStruct(OrderedDict((("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")),)), name="AVITIMEDINDEX_ENTRY", pack=False, align=None),
    "AVRF_BACKTRACE_INFORMATION": SimStruct(OrderedDict((("Depth", SimTypeInt(signed=False, label="UInt32")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("ReturnAddresses", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 32)),)), name="AVRF_BACKTRACE_INFORMATION", pack=False, align=None),
    "AVRF_HANDLEOPERATION_ENUMERATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("AVRF_HANDLE_OPERATION", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["HandleOperation", "EnumerationContext", "EnumerationLevel"]), offset=0),
    "AVRF_HANDLE_OPERATION": SimStruct(OrderedDict((("Handle", SimTypeLongLong(signed=False, label="UInt64")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("OperationType", SimTypeInt(signed=False, label="UInt32")), ("Spare0", SimTypeInt(signed=False, label="UInt32")), ("BackTraceInformation", SimTypeRef("AVRF_BACKTRACE_INFORMATION", SimStruct)),)), name="AVRF_HANDLE_OPERATION", pack=False, align=None),
    "AVRF_HEAPALLOCATION_ENUMERATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("AVRF_HEAP_ALLOCATION", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["HeapAllocation", "EnumerationContext", "EnumerationLevel"]), offset=0),
    "AVRF_HEAP_ALLOCATION": SimStruct(OrderedDict((("HeapHandle", SimTypeLongLong(signed=False, label="UInt64")), ("UserAllocation", SimTypeLongLong(signed=False, label="UInt64")), ("UserAllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("Allocation", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("UserAllocationState", SimTypeInt(signed=False, label="UInt32")), ("HeapState", SimTypeInt(signed=False, label="UInt32")), ("HeapContext", SimTypeLongLong(signed=False, label="UInt64")), ("BackTraceInformation", SimTypePointer(SimTypeRef("AVRF_BACKTRACE_INFORMATION", SimStruct), offset=0)),)), name="AVRF_HEAP_ALLOCATION", pack=False, align=None),
    "AVRF_RESOURCE_ENUMERATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceDescription", "EnumerationContext", "EnumerationLevel"]), offset=0),
    "AVRT_PRIORITY": SimTypeInt(signed=False, label="AVRT_PRIORITY"),
    "AXESLISTA": SimStruct(OrderedDict((("axlReserved", SimTypeInt(signed=False, label="UInt32")), ("axlNumAxes", SimTypeInt(signed=False, label="UInt32")), ("axlAxisInfo", SimTypeArray(SimTypeRef("AXISINFOA", SimStruct), 16)),)), name="AXESLISTA", pack=False, align=None),
    "AXESLISTW": SimStruct(OrderedDict((("axlReserved", SimTypeInt(signed=False, label="UInt32")), ("axlNumAxes", SimTypeInt(signed=False, label="UInt32")), ("axlAxisInfo", SimTypeArray(SimTypeRef("AXISINFOW", SimStruct), 16)),)), name="AXESLISTW", pack=False, align=None),
    "AXIS": SimTypeInt(signed=False, label="AXIS"),
    "AXISINFOA": SimStruct(OrderedDict((("axMinValue", SimTypeInt(signed=True, label="Int32")), ("axMaxValue", SimTypeInt(signed=True, label="Int32")), ("axAxisName", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="AXISINFOA", pack=False, align=None),
    "AXISINFOW": SimStruct(OrderedDict((("axMinValue", SimTypeInt(signed=True, label="Int32")), ("axMaxValue", SimTypeInt(signed=True, label="Int32")), ("axAxisName", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="AXISINFOW", pack=False, align=None),
    "AZ_PROP_CONSTANTS": SimTypeInt(signed=False, label="AZ_PROP_CONSTANTS"),
    "ActiveEnd": SimTypeInt(signed=False, label="ActiveEnd"),
    "Adapter": SimStruct(OrderedDict((("AdapterName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("numSources", SimTypeInt(signed=True, label="Int32")), ("sources", SimTypePointer(SimTypeRef("Sources", SimStruct), offset=0)),)), name="Adapter", pack=False, align=None),
    "Adapters": SimStruct(OrderedDict((("numAdapters", SimTypeInt(signed=True, label="Int32")), ("adapter", SimTypePointer(SimTypeRef("Adapter", SimStruct), offset=0)),)), name="Adapters", pack=False, align=None),
    "AddPackageDependencyOptions": SimTypeInt(signed=False, label="AddPackageDependencyOptions"),
    "AddServiceFlag": SimTypeInt(signed=False, label="AddServiceFlag"),
    "AdrClientDisplayFlags": SimTypeInt(signed=False, label="AdrClientDisplayFlags"),
    "AdrClientErrorType": SimTypeInt(signed=False, label="AdrClientErrorType"),
    "AdrClientFlags": SimTypeInt(signed=False, label="AdrClientFlags"),
    "AdrEmailFlags": SimTypeInt(signed=False, label="AdrEmailFlags"),
    "AecQualityMetrics_Struct": SimStruct(OrderedDict((("i64Timestamp", SimTypeLongLong(signed=True, label="Int64")), ("ConvergenceFlag", SimTypeChar(label="Byte")), ("MicClippedFlag", SimTypeChar(label="Byte")), ("MicSilenceFlag", SimTypeChar(label="Byte")), ("PstvFeadbackFlag", SimTypeChar(label="Byte")), ("SpkClippedFlag", SimTypeChar(label="Byte")), ("SpkMuteFlag", SimTypeChar(label="Byte")), ("GlitchFlag", SimTypeChar(label="Byte")), ("DoubleTalkFlag", SimTypeChar(label="Byte")), ("uGlitchCount", SimTypeInt(signed=False, label="UInt32")), ("uMicClipCount", SimTypeInt(signed=False, label="UInt32")), ("fDuration", SimTypeFloat(size=32)), ("fTSVariance", SimTypeFloat(size=32)), ("fTSDriftRate", SimTypeFloat(size=32)), ("fVoiceLevel", SimTypeFloat(size=32)), ("fNoiseLevel", SimTypeFloat(size=32)), ("fERLE", SimTypeFloat(size=32)), ("fAvgERLE", SimTypeFloat(size=32)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="AecQualityMetrics_Struct", pack=False, align=None),
    "AlgorithmFlags": SimTypeInt(signed=False, label="AlgorithmFlags"),
    "AlgorithmOperationFlags": SimTypeInt(signed=False, label="AlgorithmOperationFlags"),
    "AlgorithmType": SimTypeInt(signed=False, label="AlgorithmType"),
    "AlternativeNameType": SimTypeInt(signed=False, label="AlternativeNameType"),
    "AnalogVideoStandard": SimTypeInt(signed=False, label="AnalogVideoStandard"),
    "AnimationStyle": SimTypeInt(signed=False, label="AnimationStyle"),
    "AnnoScope": SimTypeInt(signed=False, label="AnnoScope"),
    "AppEvents": SimTypeBottom(label="AppEvents"),
    "AppPolicyClrCompat": SimTypeInt(signed=False, label="AppPolicyClrCompat"),
    "AppPolicyCreateFileAccess": SimTypeInt(signed=False, label="AppPolicyCreateFileAccess"),
    "AppPolicyLifecycleManagement": SimTypeInt(signed=False, label="AppPolicyLifecycleManagement"),
    "AppPolicyMediaFoundationCodecLoading": SimTypeInt(signed=False, label="AppPolicyMediaFoundationCodecLoading"),
    "AppPolicyProcessTerminationMethod": SimTypeInt(signed=False, label="AppPolicyProcessTerminationMethod"),
    "AppPolicyShowDeveloperDiagnostic": SimTypeInt(signed=False, label="AppPolicyShowDeveloperDiagnostic"),
    "AppPolicyThreadInitializationType": SimTypeInt(signed=False, label="AppPolicyThreadInitializationType"),
    "AppPolicyWindowingModel": SimTypeInt(signed=False, label="AppPolicyWindowingModel"),
    "AppearanceConstants": SimTypeInt(signed=False, label="AppearanceConstants"),
    "ApplicationProcessRecycleInfo": SimStruct(OrderedDict((("IsRecyclable", SimTypeInt(signed=True, label="Int32")), ("IsRecycled", SimTypeInt(signed=True, label="Int32")), ("TimeRecycled", SimTypeRef("FILETIME", SimStruct)), ("TimeToTerminate", SimTypeRef("FILETIME", SimStruct)), ("RecycleReasonCode", SimTypeInt(signed=True, label="Int32")), ("IsPendingRecycle", SimTypeInt(signed=True, label="Int32")), ("HasAutomaticLifetimeRecycling", SimTypeInt(signed=True, label="Int32")), ("TimeForAutomaticRecycling", SimTypeRef("FILETIME", SimStruct)), ("MemoryLimitInKB", SimTypeInt(signed=False, label="UInt32")), ("MemoryUsageInKBLastCheck", SimTypeInt(signed=False, label="UInt32")), ("ActivationLimit", SimTypeInt(signed=False, label="UInt32")), ("NumActivationsLastReported", SimTypeInt(signed=False, label="UInt32")), ("CallLimit", SimTypeInt(signed=False, label="UInt32")), ("NumCallsLastReported", SimTypeInt(signed=False, label="UInt32")),)), name="ApplicationProcessRecycleInfo", pack=False, align=None),
    "ApplicationProcessStatistics": SimStruct(OrderedDict((("NumCallsOutstanding", SimTypeInt(signed=False, label="UInt32")), ("NumTrackedComponents", SimTypeInt(signed=False, label="UInt32")), ("NumComponentInstances", SimTypeInt(signed=False, label="UInt32")), ("AvgCallsPerSecond", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("Reserved4", SimTypeInt(signed=False, label="UInt32")),)), name="ApplicationProcessStatistics", pack=False, align=None),
    "ApplicationProcessSummary": SimStruct(OrderedDict((("PartitionIdPrimaryApplication", SimTypeBottom(label="Guid")), ("ApplicationIdPrimaryApplication", SimTypeBottom(label="Guid")), ("ApplicationInstanceId", SimTypeBottom(label="Guid")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="COMPLUS_APPTYPE")), ("ProcessExeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsService", SimTypeInt(signed=True, label="Int32")), ("IsPaused", SimTypeInt(signed=True, label="Int32")), ("IsRecycled", SimTypeInt(signed=True, label="Int32")),)), name="ApplicationProcessSummary", pack=False, align=None),
    "ApplicationSummary": SimStruct(OrderedDict((("ApplicationInstanceId", SimTypeBottom(label="Guid")), ("PartitionId", SimTypeBottom(label="Guid")), ("ApplicationId", SimTypeBottom(label="Guid")), ("Type", SimTypeInt(signed=False, label="COMPLUS_APPTYPE")), ("ApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NumTrackedComponents", SimTypeInt(signed=False, label="UInt32")), ("NumComponentInstances", SimTypeInt(signed=False, label="UInt32")),)), name="ApplicationSummary", pack=False, align=None),
    "ApplicationType": SimTypeInt(signed=False, label="ApplicationType"),
    "ApplicationTypeType": SimTypeInt(signed=False, label="ApplicationTypeType"),
    "ArrayDimension": SimStruct(OrderedDict((("LowerBound", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")), ("Stride", SimTypeLongLong(signed=False, label="UInt64")),)), name="ArrayDimension", pack=False, align=None),
    "AsnAny": SimStruct(OrderedDict((("asnType", SimTypeChar(label="Byte")), ("asnValue", SimUnion({"number": SimTypeInt(signed=True, label="Int32"), "unsigned32": SimTypeInt(signed=False, label="UInt32"), "counter64": SimTypeLongLong(signed=False, label="UInt64"), "string": SimTypeRef("AsnOctetString", SimStruct), "bits": SimTypeRef("AsnOctetString", SimStruct), "object": SimTypeRef("AsnObjectIdentifier", SimStruct), "sequence": SimTypeRef("AsnOctetString", SimStruct), "address": SimTypeRef("AsnOctetString", SimStruct), "counter": SimTypeInt(signed=False, label="UInt32"), "gauge": SimTypeInt(signed=False, label="UInt32"), "ticks": SimTypeInt(signed=False, label="UInt32"), "arbitrary": SimTypeRef("AsnOctetString", SimStruct)}, name="<anon>", label="None")),)), name="AsnAny", pack=False, align=None),
    "AsnObjectIdentifier": SimStruct(OrderedDict((("idLength", SimTypeInt(signed=False, label="UInt32")), ("ids", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="AsnObjectIdentifier", pack=False, align=None),
    "AsnOctetString": SimStruct(OrderedDict((("stream", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")), ("dynamic", SimTypeInt(signed=True, label="Int32")),)), name="AsnOctetString", pack=False, align=None),
    "AssemblyBindInfo": SimStruct(OrderedDict((("dwAppDomainId", SimTypeInt(signed=False, label="UInt32")), ("lpReferencedIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpPostPolicyIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ePolicyLevel", SimTypeInt(signed=False, label="UInt32")),)), name="AssemblyBindInfo", pack=False, align=None),
    "AsyncContentLoadedState": SimTypeInt(signed=False, label="AsyncContentLoadedState"),
    "AsyncIAdviseSink": SimTypeBottom(label="AsyncIAdviseSink"),
    "AsyncIAdviseSink2": SimTypeBottom(label="AsyncIAdviseSink2"),
    "AsyncIAssociatedIdentityProvider": SimTypeBottom(label="AsyncIAssociatedIdentityProvider"),
    "AsyncIBackgroundCopyCallback": SimTypeBottom(label="AsyncIBackgroundCopyCallback"),
    "AsyncIConnectedIdentityProvider": SimTypeBottom(label="AsyncIConnectedIdentityProvider"),
    "AsyncIDebugApplicationNodeEvents": SimTypeBottom(label="AsyncIDebugApplicationNodeEvents"),
    "AsyncIFtpAuthenticationProvider": SimTypeBottom(label="AsyncIFtpAuthenticationProvider"),
    "AsyncIFtpAuthorizationProvider": SimTypeBottom(label="AsyncIFtpAuthorizationProvider"),
    "AsyncIFtpHomeDirectoryProvider": SimTypeBottom(label="AsyncIFtpHomeDirectoryProvider"),
    "AsyncIFtpLogProvider": SimTypeBottom(label="AsyncIFtpLogProvider"),
    "AsyncIFtpPostprocessProvider": SimTypeBottom(label="AsyncIFtpPostprocessProvider"),
    "AsyncIFtpPreprocessProvider": SimTypeBottom(label="AsyncIFtpPreprocessProvider"),
    "AsyncIFtpRoleProvider": SimTypeBottom(label="AsyncIFtpRoleProvider"),
    "AsyncIIdentityAdvise": SimTypeBottom(label="AsyncIIdentityAdvise"),
    "AsyncIIdentityAuthentication": SimTypeBottom(label="AsyncIIdentityAuthentication"),
    "AsyncIIdentityProvider": SimTypeBottom(label="AsyncIIdentityProvider"),
    "AsyncIIdentityStore": SimTypeBottom(label="AsyncIIdentityStore"),
    "AsyncIIdentityStoreEx": SimTypeBottom(label="AsyncIIdentityStoreEx"),
    "AsyncIMSAdminBaseSinkW": SimTypeBottom(label="AsyncIMSAdminBaseSinkW"),
    "AsyncIMultiQI": SimTypeBottom(label="AsyncIMultiQI"),
    "AsyncIPipeByte": SimTypeBottom(label="AsyncIPipeByte"),
    "AsyncIPipeDouble": SimTypeBottom(label="AsyncIPipeDouble"),
    "AsyncIPipeLong": SimTypeBottom(label="AsyncIPipeLong"),
    "AsyncIUnknown": SimTypeBottom(label="AsyncIUnknown"),
    "AtlThunkData_t": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "AudioClient3ActivationParams": SimStruct(OrderedDict((("tracingContextId", SimTypeBottom(label="Guid")),)), name="AudioClient3ActivationParams", pack=False, align=None),
    "AudioClientProperties": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("bIsOffload", SimTypeInt(signed=True, label="Int32")), ("eCategory", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("Options", SimTypeInt(signed=False, label="AUDCLNT_STREAMOPTIONS")),)), name="AudioClientProperties", pack=False, align=None),
    "AudioExtensionParams": SimStruct(OrderedDict((("AddPageParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pEndpoint", SimTypeBottom(label="IMMDevice")), ("pPnpInterface", SimTypeBottom(label="IMMDevice")), ("pPnpDevnode", SimTypeBottom(label="IMMDevice")),)), name="AudioExtensionParams", pack=False, align=None),
    "AudioFXExtensionParams": SimStruct(OrderedDict((("AddPageParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwstrEndpointID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pFxProperties", SimTypeBottom(label="IPropertyStore")),)), name="AudioFXExtensionParams", pack=False, align=None),
    "AudioObjectType": SimTypeInt(signed=False, label="AudioObjectType"),
    "AudioSessionDisconnectReason": SimTypeInt(signed=False, label="AudioSessionDisconnectReason"),
    "AudioSessionState": SimTypeInt(signed=False, label="AudioSessionState"),
    "AudioStateMonitorSoundLevel": SimTypeInt(signed=False, label="AudioStateMonitorSoundLevel"),
    "AutoCorrectProc": SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["langid", "pszBefore", "pszAfter", "cchAfter", "pcchReplaced"]), offset=0),
    "AutoDownloadMode": SimTypeInt(signed=False, label="AutoDownloadMode"),
    "AutoPathFormat": SimTypeInt(signed=False, label="AutoPathFormat"),
    "AutoProxyHelperFunctions": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeRef("AutoProxyHelperVtbl", SimStruct), offset=0)),)), name="AutoProxyHelperFunctions", pack=False, align=None),
    "AutoProxyHelperVtbl": SimStruct(OrderedDict((("IsResolvable", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetIPAddress", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ResolveHostName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IsInNet", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IsResolvableEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetIPAddressEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ResolveHostNameEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IsInNetEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SortIpList", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="AutoProxyHelperVtbl", pack=False, align=None),
    "AutoSelectionMode": SimTypeInt(signed=False, label="AutoSelectionMode"),
    "AutoSvcs_Error_Constants": SimTypeInt(signed=False, label="AutoSvcs_Error_Constants"),
    "AutomaticUpdatesNotificationLevel": SimTypeInt(signed=False, label="AutomaticUpdatesNotificationLevel"),
    "AutomaticUpdatesPermissionType": SimTypeInt(signed=False, label="AutomaticUpdatesPermissionType"),
    "AutomaticUpdatesScheduledInstallationDay": SimTypeInt(signed=False, label="AutomaticUpdatesScheduledInstallationDay"),
    "AutomaticUpdatesUserType": SimTypeInt(signed=False, label="AutomaticUpdatesUserType"),
    "AutomationElementMode": SimTypeInt(signed=False, label="AutomationElementMode"),
    "AutomationIdentifierType": SimTypeInt(signed=False, label="AutomationIdentifierType"),
    "BACKGROUNDSTATES": SimTypeInt(signed=False, label="BACKGROUNDSTATES"),
    "BACKGROUNDWITHBORDERSTATES": SimTypeInt(signed=False, label="BACKGROUNDWITHBORDERSTATES"),
    "BACKGROUND_MODE": SimTypeInt(signed=False, label="BACKGROUND_MODE"),
    "BACKLIGHT_OPTIMIZATION_LEVEL": SimTypeInt(signed=False, label="BACKLIGHT_OPTIMIZATION_LEVEL"),
    "BACKLIGHT_REDUCTION_GAMMA_RAMP": SimStruct(OrderedDict((("R", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("G", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("B", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="BACKLIGHT_REDUCTION_GAMMA_RAMP", pack=False, align=None),
    "BALLOONSTATES": SimTypeInt(signed=False, label="BALLOONSTATES"),
    "BALLOONSTEMSTATES": SimTypeInt(signed=False, label="BALLOONSTEMSTATES"),
    "BANDINFOSFB": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwStateMask", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("crBkgnd", SimTypeInt(signed=False, label="UInt32")), ("crBtnLt", SimTypeInt(signed=False, label="UInt32")), ("crBtnDk", SimTypeInt(signed=False, label="UInt32")), ("wViewMode", SimTypeShort(signed=False, label="UInt16")), ("wAlign", SimTypeShort(signed=False, label="UInt16")), ("psf", SimTypeBottom(label="IShellFolder")), ("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)),)), name="BANDINFOSFB", pack=False, align=None),
    "BANDSITECID": SimTypeInt(signed=False, label="BANDSITECID"),
    "BANDSITEINFO": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwStyle", SimTypeInt(signed=False, label="UInt32")),)), name="BANDSITEINFO", pack=False, align=None),
    "BANK_POSITION": SimStruct(OrderedDict((("ReadBankPosition", SimTypeInt(signed=False, label="UInt32")), ("WriteBankPosition", SimTypeInt(signed=False, label="UInt32")),)), name="BANK_POSITION", pack=False, align=None),
    "BANNER_NOTIFICATION": SimStruct(OrderedDict((("event", SimTypeInt(signed=False, label="BANNER_NOTIFICATION_EVENT")), ("providerIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("contentId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BANNER_NOTIFICATION", pack=False, align=None),
    "BANNER_NOTIFICATION_EVENT": SimTypeInt(signed=False, label="BANNER_NOTIFICATION_EVENT"),
    "BARBACKGROUNDSTATES": SimTypeInt(signed=False, label="BARBACKGROUNDSTATES"),
    "BARITEMSTATES": SimTypeInt(signed=False, label="BARITEMSTATES"),
    "BASEBROWSERDATALH": SimStruct(OrderedDict((("_hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_ptl", SimTypeBottom(label="ITravelLog")), ("_phlf", SimTypeBottom(label="IHlinkFrame")), ("_pautoWB2", SimTypeBottom(label="IWebBrowser2")), ("_pautoEDS", SimTypeBottom(label="IExpDispSupport")), ("_pautoSS", SimTypeBottom(label="IShellService")), ("_eSecureLockIcon", SimTypeInt(signed=True, label="Int32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("_uActivateState", SimTypeInt(signed=False, label="UInt32")), ("_pidlViewState", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("_pctView", SimTypeBottom(label="IOleCommandTarget")), ("_pidlCur", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("_psv", SimTypeBottom(label="IShellView")), ("_psf", SimTypeBottom(label="IShellFolder")), ("_hwndView", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_pszTitleCur", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_pidlPending", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("_psvPending", SimTypeBottom(label="IShellView")), ("_psfPending", SimTypeBottom(label="IShellFolder")), ("_hwndViewPending", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_pszTitlePending", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_fIsViewMSHTML", SimTypeInt(signed=True, label="Int32")), ("_fPrivacyImpacted", SimTypeInt(signed=True, label="Int32")), ("_clsidView", SimTypeBottom(label="Guid")), ("_clsidViewPending", SimTypeBottom(label="Guid")), ("_hwndFrame", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_lPhishingFilterStatus", SimTypeInt(signed=True, label="Int32")),)), name="BASEBROWSERDATALH", pack=False, align=None),
    "BASEBROWSERDATAXP": SimStruct(OrderedDict((("_hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_ptl", SimTypeBottom(label="ITravelLog")), ("_phlf", SimTypeBottom(label="IHlinkFrame")), ("_pautoWB2", SimTypeBottom(label="IWebBrowser2")), ("_pautoEDS", SimTypeBottom(label="IExpDispSupportXP")), ("_pautoSS", SimTypeBottom(label="IShellService")), ("_eSecureLockIcon", SimTypeInt(signed=True, label="Int32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("_uActivateState", SimTypeInt(signed=False, label="UInt32")), ("_pidlViewState", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("_pctView", SimTypeBottom(label="IOleCommandTarget")), ("_pidlCur", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("_psv", SimTypeBottom(label="IShellView")), ("_psf", SimTypeBottom(label="IShellFolder")), ("_hwndView", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_pszTitleCur", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_pidlPending", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("_psvPending", SimTypeBottom(label="IShellView")), ("_psfPending", SimTypeBottom(label="IShellFolder")), ("_hwndViewPending", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_pszTitlePending", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_fIsViewMSHTML", SimTypeInt(signed=True, label="Int32")), ("_fPrivacyImpacted", SimTypeInt(signed=True, label="Int32")), ("_clsidView", SimTypeBottom(label="Guid")), ("_clsidViewPending", SimTypeBottom(label="Guid")), ("_hwndFrame", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="BASEBROWSERDATAXP", pack=False, align=None),
    "BASE_MCB": SimStruct(OrderedDict((("MaximumPairCount", SimTypeInt(signed=False, label="UInt32")), ("PairCount", SimTypeInt(signed=False, label="UInt32")), ("PoolType", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Mapping", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="BASE_MCB", pack=False, align=None),
    "BATTERY_CHARGER_STATUS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="BATTERY_CHARGING_SOURCE_TYPE")), ("VaData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BATTERY_CHARGER_STATUS", pack=False, align=None),
    "BATTERY_CHARGING_SOURCE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="BATTERY_CHARGING_SOURCE_TYPE")), ("MaxCurrent", SimTypeInt(signed=False, label="UInt32")),)), name="BATTERY_CHARGING_SOURCE", pack=False, align=None),
    "BATTERY_CHARGING_SOURCE_INFORMATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="BATTERY_CHARGING_SOURCE_TYPE")), ("SourceOnline", SimTypeChar(label="Byte")),)), name="BATTERY_CHARGING_SOURCE_INFORMATION", pack=False, align=None),
    "BATTERY_CHARGING_SOURCE_TYPE": SimTypeInt(signed=False, label="BATTERY_CHARGING_SOURCE_TYPE"),
    "BATTERY_DEVTYPE": SimTypeInt(signed=False, label="BATTERY_DEVTYPE"),
    "BATTERY_INFORMATION": SimStruct(OrderedDict((("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("Technology", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Chemistry", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("DesignedCapacity", SimTypeInt(signed=False, label="UInt32")), ("FullChargedCapacity", SimTypeInt(signed=False, label="UInt32")), ("DefaultAlert1", SimTypeInt(signed=False, label="UInt32")), ("DefaultAlert2", SimTypeInt(signed=False, label="UInt32")), ("CriticalBias", SimTypeInt(signed=False, label="UInt32")), ("CycleCount", SimTypeInt(signed=False, label="UInt32")),)), name="BATTERY_INFORMATION", pack=False, align=None),
    "BATTERY_LEVEL": SimTypeInt(signed=False, label="BATTERY_LEVEL"),
    "BATTERY_MANUFACTURE_DATE": SimStruct(OrderedDict((("Day", SimTypeChar(label="Byte")), ("Month", SimTypeChar(label="Byte")), ("Year", SimTypeShort(signed=False, label="UInt16")),)), name="BATTERY_MANUFACTURE_DATE", pack=False, align=None),
    "BATTERY_QUERY_INFORMATION": SimStruct(OrderedDict((("BatteryTag", SimTypeInt(signed=False, label="UInt32")), ("InformationLevel", SimTypeInt(signed=False, label="BATTERY_QUERY_INFORMATION_LEVEL")), ("AtRate", SimTypeInt(signed=False, label="UInt32")),)), name="BATTERY_QUERY_INFORMATION", pack=False, align=None),
    "BATTERY_QUERY_INFORMATION_LEVEL": SimTypeInt(signed=False, label="BATTERY_QUERY_INFORMATION_LEVEL"),
    "BATTERY_REPORTING_SCALE": SimStruct(OrderedDict((("Granularity", SimTypeInt(signed=False, label="UInt32")), ("Capacity", SimTypeInt(signed=False, label="UInt32")),)), name="BATTERY_REPORTING_SCALE", pack=False, align=None),
    "BATTERY_SET_INFORMATION": SimStruct(OrderedDict((("BatteryTag", SimTypeInt(signed=False, label="UInt32")), ("InformationLevel", SimTypeInt(signed=False, label="BATTERY_SET_INFORMATION_LEVEL")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BATTERY_SET_INFORMATION", pack=False, align=None),
    "BATTERY_SET_INFORMATION_LEVEL": SimTypeInt(signed=False, label="BATTERY_SET_INFORMATION_LEVEL"),
    "BATTERY_STATUS": SimStruct(OrderedDict((("PowerState", SimTypeInt(signed=False, label="UInt32")), ("Capacity", SimTypeInt(signed=False, label="UInt32")), ("Voltage", SimTypeInt(signed=False, label="UInt32")), ("Rate", SimTypeInt(signed=True, label="Int32")),)), name="BATTERY_STATUS", pack=False, align=None),
    "BATTERY_TYPE": SimTypeInt(signed=False, label="BATTERY_TYPE"),
    "BATTERY_USB_CHARGER_STATUS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="BATTERY_CHARGING_SOURCE_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxCurrent", SimTypeInt(signed=False, label="UInt32")), ("Voltage", SimTypeInt(signed=False, label="UInt32")), ("PortType", SimTypeInt(signed=False, label="USB_CHARGER_PORT")), ("PortId", SimTypeLongLong(signed=False, label="UInt64")), ("PowerSourceInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OemCharger", SimTypeBottom(label="Guid")),)), name="BATTERY_USB_CHARGER_STATUS", pack=False, align=None),
    "BATTERY_WAIT_STATUS": SimStruct(OrderedDict((("BatteryTag", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("PowerState", SimTypeInt(signed=False, label="UInt32")), ("LowCapacity", SimTypeInt(signed=False, label="UInt32")), ("HighCapacity", SimTypeInt(signed=False, label="UInt32")),)), name="BATTERY_WAIT_STATUS", pack=False, align=None),
    "BCRYPTGENRANDOM_FLAGS": SimTypeInt(signed=False, label="BCRYPTGENRANDOM_FLAGS"),
    "BCRYPT_ALGORITHM_IDENTIFIER": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwClass", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_ALGORITHM_IDENTIFIER", pack=False, align=None),
    "BCRYPT_ALG_HANDLE": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "BCRYPT_AUTHENTICATED_CIPHER_MODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwInfoVersion", SimTypeInt(signed=False, label="UInt32")), ("pbNonce", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbNonce", SimTypeInt(signed=False, label="UInt32")), ("pbAuthData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbAuthData", SimTypeInt(signed=False, label="UInt32")), ("pbTag", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbTag", SimTypeInt(signed=False, label="UInt32")), ("pbMacContext", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbMacContext", SimTypeInt(signed=False, label="UInt32")), ("cbAAD", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeLongLong(signed=False, label="UInt64")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_AUTHENTICATED_CIPHER_MODE_INFO", pack=False, align=None),
    "BCRYPT_DH_KEY_BLOB": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="BCRYPT_DH_KEY_BLOB_MAGIC")), ("cbKey", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_DH_KEY_BLOB", pack=False, align=None),
    "BCRYPT_DH_KEY_BLOB_MAGIC": SimTypeInt(signed=False, label="BCRYPT_DH_KEY_BLOB_MAGIC"),
    "BCRYPT_DH_PARAMETER_HEADER": SimStruct(OrderedDict((("cbLength", SimTypeInt(signed=False, label="UInt32")), ("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("cbKeyLength", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_DH_PARAMETER_HEADER", pack=False, align=None),
    "BCRYPT_DSA_KEY_BLOB": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="BCRYPT_DSA_MAGIC")), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Seed", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("q", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="BCRYPT_DSA_KEY_BLOB", pack=False, align=None),
    "BCRYPT_DSA_KEY_BLOB_V2": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="BCRYPT_DSA_MAGIC")), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("hashAlgorithm", SimTypeInt(signed=False, label="HASHALGORITHM_ENUM")), ("standardVersion", SimTypeInt(signed=False, label="DSAFIPSVERSION_ENUM")), ("cbSeedLength", SimTypeInt(signed=False, label="UInt32")), ("cbGroupSize", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="BCRYPT_DSA_KEY_BLOB_V2", pack=False, align=None),
    "BCRYPT_DSA_MAGIC": SimTypeInt(signed=False, label="BCRYPT_DSA_MAGIC"),
    "BCRYPT_DSA_PARAMETER_HEADER": SimStruct(OrderedDict((("cbLength", SimTypeInt(signed=False, label="UInt32")), ("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("cbKeyLength", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Seed", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("q", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="BCRYPT_DSA_PARAMETER_HEADER", pack=False, align=None),
    "BCRYPT_DSA_PARAMETER_HEADER_V2": SimStruct(OrderedDict((("cbLength", SimTypeInt(signed=False, label="UInt32")), ("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("cbKeyLength", SimTypeInt(signed=False, label="UInt32")), ("hashAlgorithm", SimTypeInt(signed=False, label="HASHALGORITHM_ENUM")), ("standardVersion", SimTypeInt(signed=False, label="DSAFIPSVERSION_ENUM")), ("cbSeedLength", SimTypeInt(signed=False, label="UInt32")), ("cbGroupSize", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="BCRYPT_DSA_PARAMETER_HEADER_V2", pack=False, align=None),
    "BCRYPT_ECCFULLKEY_BLOB": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwCurveType", SimTypeInt(signed=False, label="ECC_CURVE_TYPE_ENUM")), ("dwCurveGenerationAlgId", SimTypeInt(signed=False, label="ECC_CURVE_ALG_ID_ENUM")), ("cbFieldLength", SimTypeInt(signed=False, label="UInt32")), ("cbSubgroupOrder", SimTypeInt(signed=False, label="UInt32")), ("cbCofactor", SimTypeInt(signed=False, label="UInt32")), ("cbSeed", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_ECCFULLKEY_BLOB", pack=False, align=None),
    "BCRYPT_ECCKEY_BLOB": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("cbKey", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_ECCKEY_BLOB", pack=False, align=None),
    "BCRYPT_ECC_CURVE_NAMES": SimStruct(OrderedDict((("dwEccCurveNames", SimTypeInt(signed=False, label="UInt32")), ("pEccCurveNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="BCRYPT_ECC_CURVE_NAMES", pack=False, align=None),
    "BCRYPT_FLAGS": SimTypeInt(signed=False, label="BCRYPT_FLAGS"),
    "BCRYPT_HANDLE": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "BCRYPT_HASH_HANDLE": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "BCRYPT_HASH_OPERATION_TYPE": SimTypeInt(signed=False, label="BCRYPT_HASH_OPERATION_TYPE"),
    "BCRYPT_INTERFACE": SimTypeInt(signed=False, label="BCRYPT_INTERFACE"),
    "BCRYPT_INTERFACE_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="BCRYPT_INTERFACE_VERSION", pack=False, align=None),
    "BCRYPT_KEY_BLOB": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_KEY_BLOB", pack=False, align=None),
    "BCRYPT_KEY_DATA_BLOB_HEADER": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbKeyData", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_KEY_DATA_BLOB_HEADER", pack=False, align=None),
    "BCRYPT_KEY_HANDLE": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "BCRYPT_KEY_LENGTHS_STRUCT": SimStruct(OrderedDict((("dwMinLength", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLength", SimTypeInt(signed=False, label="UInt32")), ("dwIncrement", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_KEY_LENGTHS_STRUCT", pack=False, align=None),
    "BCRYPT_MULTI_HASH_OPERATION": SimStruct(OrderedDict((("iHash", SimTypeInt(signed=False, label="UInt32")), ("hashOperation", SimTypeInt(signed=False, label="BCRYPT_HASH_OPERATION_TYPE")), ("pbBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_MULTI_HASH_OPERATION", pack=False, align=None),
    "BCRYPT_MULTI_OBJECT_LENGTH_STRUCT": SimStruct(OrderedDict((("cbPerObject", SimTypeInt(signed=False, label="UInt32")), ("cbPerElement", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_MULTI_OBJECT_LENGTH_STRUCT", pack=False, align=None),
    "BCRYPT_MULTI_OPERATION_TYPE": SimTypeInt(signed=False, label="BCRYPT_MULTI_OPERATION_TYPE"),
    "BCRYPT_OAEP_PADDING_INFO": SimStruct(OrderedDict((("pszAlgId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pbLabel", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLabel", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_OAEP_PADDING_INFO", pack=False, align=None),
    "BCRYPT_OID": SimStruct(OrderedDict((("cbOID", SimTypeInt(signed=False, label="UInt32")), ("pbOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BCRYPT_OID", pack=False, align=None),
    "BCRYPT_OID_LIST": SimStruct(OrderedDict((("dwOIDCount", SimTypeInt(signed=False, label="UInt32")), ("pOIDs", SimTypePointer(SimTypeRef("BCRYPT_OID", SimStruct), offset=0)),)), name="BCRYPT_OID_LIST", pack=False, align=None),
    "BCRYPT_OPEN_ALGORITHM_PROVIDER_FLAGS": SimTypeInt(signed=False, label="BCRYPT_OPEN_ALGORITHM_PROVIDER_FLAGS"),
    "BCRYPT_OPERATION": SimTypeInt(signed=False, label="BCRYPT_OPERATION"),
    "BCRYPT_PKCS1_PADDING_INFO": SimStruct(OrderedDict((("pszAlgId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BCRYPT_PKCS1_PADDING_INFO", pack=False, align=None),
    "BCRYPT_PROVIDER_NAME": SimStruct(OrderedDict((("pszProviderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BCRYPT_PROVIDER_NAME", pack=False, align=None),
    "BCRYPT_PSS_PADDING_INFO": SimStruct(OrderedDict((("pszAlgId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbSalt", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_PSS_PADDING_INFO", pack=False, align=None),
    "BCRYPT_QUERY_PROVIDER_MODE": SimTypeInt(signed=False, label="BCRYPT_QUERY_PROVIDER_MODE"),
    "BCRYPT_RESOLVE_PROVIDERS_FLAGS": SimTypeInt(signed=False, label="BCRYPT_RESOLVE_PROVIDERS_FLAGS"),
    "BCRYPT_RSAKEY_BLOB": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="BCRYPT_RSAKEY_BLOB_MAGIC")), ("BitLength", SimTypeInt(signed=False, label="UInt32")), ("cbPublicExp", SimTypeInt(signed=False, label="UInt32")), ("cbModulus", SimTypeInt(signed=False, label="UInt32")), ("cbPrime1", SimTypeInt(signed=False, label="UInt32")), ("cbPrime2", SimTypeInt(signed=False, label="UInt32")),)), name="BCRYPT_RSAKEY_BLOB", pack=False, align=None),
    "BCRYPT_RSAKEY_BLOB_MAGIC": SimTypeInt(signed=False, label="BCRYPT_RSAKEY_BLOB_MAGIC"),
    "BCRYPT_SECRET_HANDLE": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "BCRYPT_TABLE": SimTypeInt(signed=False, label="BCRYPT_TABLE"),
    "BCryptBuffer": SimStruct(OrderedDict((("cbBuffer", SimTypeInt(signed=False, label="UInt32")), ("BufferType", SimTypeInt(signed=False, label="UInt32")), ("pvBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="BCryptBuffer", pack=False, align=None),
    "BCryptBufferDesc": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("cBuffers", SimTypeInt(signed=False, label="UInt32")), ("pBuffers", SimTypePointer(SimTypeRef("BCryptBuffer", SimStruct), offset=0)),)), name="BCryptBufferDesc", pack=False, align=None),
    "BDANODE_DESCRIPTOR": SimStruct(OrderedDict((("ulBdaNodeType", SimTypeInt(signed=False, label="UInt32")), ("guidFunction", SimTypeBottom(label="Guid")), ("guidName", SimTypeBottom(label="Guid")),)), name="BDANODE_DESCRIPTOR", pack=False, align=None),
    "BDA_BUFFER": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulBufferSize", SimTypeInt(signed=False, label="UInt32")), ("argbBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_BUFFER", pack=False, align=None),
    "BDA_CAS_CHECK_ENTITLEMENTTOKEN": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulDescrambleStatus", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_CAS_CHECK_ENTITLEMENTTOKEN", pack=False, align=None),
    "BDA_CAS_CLOSEMMIDATA": SimStruct(OrderedDict((("ulDialogNumber", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_CAS_CLOSEMMIDATA", pack=False, align=None),
    "BDA_CAS_CLOSE_MMIDIALOG": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("SessionResult", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_CAS_CLOSE_MMIDIALOG", pack=False, align=None),
    "BDA_CAS_OPENMMIDATA": SimStruct(OrderedDict((("ulDialogNumber", SimTypeInt(signed=False, label="UInt32")), ("ulDialogRequest", SimTypeInt(signed=False, label="UInt32")), ("uuidDialogType", SimTypeBottom(label="Guid")), ("usDialogDataLength", SimTypeShort(signed=False, label="UInt16")), ("argbDialogData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_CAS_OPENMMIDATA", pack=False, align=None),
    "BDA_CAS_REQUESTTUNERDATA": SimStruct(OrderedDict((("ucRequestPriority", SimTypeChar(label="Byte")), ("ucRequestReason", SimTypeChar(label="Byte")), ("ucRequestConsequences", SimTypeChar(label="Byte")), ("ulEstimatedTime", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_CAS_REQUESTTUNERDATA", pack=False, align=None),
    "BDA_CA_MODULE_UI": SimStruct(OrderedDict((("ulFormat", SimTypeInt(signed=False, label="UInt32")), ("ulbcDesc", SimTypeInt(signed=False, label="UInt32")), ("ulDesc", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BDA_CA_MODULE_UI", pack=False, align=None),
    "BDA_CHANGE_STATE": SimTypeInt(signed=False, label="BDA_CHANGE_STATE"),
    "BDA_CONDITIONALACCESS_MMICLOSEREASON": SimTypeInt(signed=False, label="BDA_CONDITIONALACCESS_MMICLOSEREASON"),
    "BDA_CONDITIONALACCESS_REQUESTTYPE": SimTypeInt(signed=False, label="BDA_CONDITIONALACCESS_REQUESTTYPE"),
    "BDA_CONDITIONALACCESS_SESSION_RESULT": SimTypeInt(signed=False, label="BDA_CONDITIONALACCESS_SESSION_RESULT"),
    "BDA_Channel": SimTypeInt(signed=False, label="BDA_Channel"),
    "BDA_Channel_Bandwidth": SimTypeInt(signed=False, label="BDA_Channel_Bandwidth"),
    "BDA_Comp_Flags": SimTypeInt(signed=False, label="BDA_Comp_Flags"),
    "BDA_DEBUG_DATA": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("uuidDebugDataType", SimTypeBottom(label="Guid")), ("ulDataSize", SimTypeInt(signed=False, label="UInt32")), ("argbDebugData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_DEBUG_DATA", pack=False, align=None),
    "BDA_DISCOVERY_STATE": SimTypeInt(signed=False, label="BDA_DISCOVERY_STATE"),
    "BDA_DISEQC_RESPONSE": SimStruct(OrderedDict((("ulRequestId", SimTypeInt(signed=False, label="UInt32")), ("ulPacketLength", SimTypeInt(signed=False, label="UInt32")), ("argbPacketData", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="BDA_DISEQC_RESPONSE", pack=False, align=None),
    "BDA_DISEQC_SEND": SimStruct(OrderedDict((("ulRequestId", SimTypeInt(signed=False, label="UInt32")), ("ulPacketLength", SimTypeInt(signed=False, label="UInt32")), ("argbPacketData", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="BDA_DISEQC_SEND", pack=False, align=None),
    "BDA_DRM_DRMSTATUS": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("DRMuuid", SimTypeBottom(label="Guid")), ("ulDrmUuidListStringSize", SimTypeInt(signed=False, label="UInt32")), ("argbDrmUuidListString", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="BDA_DRM_DRMSTATUS", pack=False, align=None),
    "BDA_DVBT2_L1_SIGNALLING_DATA": SimStruct(OrderedDict((("L1Pre_TYPE", SimTypeChar(label="Byte")), ("L1Pre_BWT_S1_S2", SimTypeChar(label="Byte")), ("L1Pre_REPETITION_GUARD_PAPR", SimTypeChar(label="Byte")), ("L1Pre_MOD_COD_FEC", SimTypeChar(label="Byte")), ("L1Pre_POSTSIZE_INFO_PILOT", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("L1Pre_TX_ID_AVAIL", SimTypeChar(label="Byte")), ("L1Pre_CELL_ID", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("L1Pre_NETWORK_ID", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("L1Pre_T2SYSTEM_ID", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("L1Pre_NUM_T2_FRAMES", SimTypeChar(label="Byte")), ("L1Pre_NUM_DATA_REGENFLAG_L1POSTEXT", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("L1Pre_NUMRF_CURRENTRF_RESERVED", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("L1Pre_CRC32", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("L1PostData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_DVBT2_L1_SIGNALLING_DATA", pack=False, align=None),
    "BDA_DigitalSignalStandard": SimTypeInt(signed=False, label="BDA_DigitalSignalStandard"),
    "BDA_DrmPairingError": SimTypeInt(signed=False, label="BDA_DrmPairingError"),
    "BDA_ETHERNET_ADDRESS": SimStruct(OrderedDict((("rgbAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="BDA_ETHERNET_ADDRESS", pack=False, align=None),
    "BDA_ETHERNET_ADDRESS_LIST": SimStruct(OrderedDict((("ulcAddresses", SimTypeInt(signed=False, label="UInt32")), ("rgAddressl", SimTypePointer(SimTypeRef("BDA_ETHERNET_ADDRESS", SimStruct), offset=0)),)), name="BDA_ETHERNET_ADDRESS_LIST", pack=False, align=None),
    "BDA_EVENT_DATA": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulEventID", SimTypeInt(signed=False, label="UInt32")), ("uuidEventType", SimTypeBottom(label="Guid")), ("ulEventDataLength", SimTypeInt(signed=False, label="UInt32")), ("argbEventData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_EVENT_DATA", pack=False, align=None),
    "BDA_EVENT_ID": SimTypeInt(signed=False, label="BDA_EVENT_ID"),
    "BDA_Frequency": SimTypeInt(signed=False, label="BDA_Frequency"),
    "BDA_Frequency_Multiplier": SimTypeInt(signed=False, label="BDA_Frequency_Multiplier"),
    "BDA_GDDS_DATA": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulDataLength", SimTypeInt(signed=False, label="UInt32")), ("ulPercentageProgress", SimTypeInt(signed=False, label="UInt32")), ("argbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_GDDS_DATA", pack=False, align=None),
    "BDA_GDDS_DATATYPE": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("uuidDataType", SimTypeBottom(label="Guid")),)), name="BDA_GDDS_DATATYPE", pack=False, align=None),
    "BDA_IPv4_ADDRESS": SimStruct(OrderedDict((("rgbAddress", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="BDA_IPv4_ADDRESS", pack=False, align=None),
    "BDA_IPv4_ADDRESS_LIST": SimStruct(OrderedDict((("ulcAddresses", SimTypeInt(signed=False, label="UInt32")), ("rgAddressl", SimTypePointer(SimTypeRef("BDA_IPv4_ADDRESS", SimStruct), offset=0)),)), name="BDA_IPv4_ADDRESS_LIST", pack=False, align=None),
    "BDA_IPv6_ADDRESS": SimStruct(OrderedDict((("rgbAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="BDA_IPv6_ADDRESS", pack=False, align=None),
    "BDA_IPv6_ADDRESS_LIST": SimStruct(OrderedDict((("ulcAddresses", SimTypeInt(signed=False, label="UInt32")), ("rgAddressl", SimTypePointer(SimTypeRef("BDA_IPv6_ADDRESS", SimStruct), offset=0)),)), name="BDA_IPv6_ADDRESS_LIST", pack=False, align=None),
    "BDA_ISDBCAS_EMG_REQ": SimStruct(OrderedDict((("bCLA", SimTypeChar(label="Byte")), ("bINS", SimTypeChar(label="Byte")), ("bP1", SimTypeChar(label="Byte")), ("bP2", SimTypeChar(label="Byte")), ("bLC", SimTypeChar(label="Byte")), ("bCardId", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bProtocol", SimTypeChar(label="Byte")), ("bCABroadcasterGroupId", SimTypeChar(label="Byte")), ("bMessageControl", SimTypeChar(label="Byte")), ("bMessageCode", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_ISDBCAS_EMG_REQ", pack=False, align=None),
    "BDA_ISDBCAS_REQUESTHEADER": SimStruct(OrderedDict((("bInstruction", SimTypeChar(label="Byte")), ("bReserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ulDataLength", SimTypeInt(signed=False, label="UInt32")), ("argbIsdbCommand", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_ISDBCAS_REQUESTHEADER", pack=False, align=None),
    "BDA_ISDBCAS_RESPONSEDATA": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulRequestID", SimTypeInt(signed=False, label="UInt32")), ("ulIsdbStatus", SimTypeInt(signed=False, label="UInt32")), ("ulIsdbDataSize", SimTypeInt(signed=False, label="UInt32")), ("argbIsdbCommandData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_ISDBCAS_RESPONSEDATA", pack=False, align=None),
    "BDA_LockType": SimTypeInt(signed=False, label="BDA_LockType"),
    "BDA_MULTICAST_MODE": SimTypeInt(signed=False, label="BDA_MULTICAST_MODE"),
    "BDA_MUX_PIDLISTITEM": SimStruct(OrderedDict((("usPIDNumber", SimTypeShort(signed=False, label="UInt16")), ("usProgramNumber", SimTypeShort(signed=False, label="UInt16")), ("ePIDType", SimTypeInt(signed=False, label="MUX_PID_TYPE")),)), name="BDA_MUX_PIDLISTITEM", pack=False, align=None),
    "BDA_PID_MAP": SimStruct(OrderedDict((("MediaSampleContent", SimTypeInt(signed=False, label="MEDIA_SAMPLE_CONTENT")), ("ulcPIDs", SimTypeInt(signed=False, label="UInt32")), ("aulPIDs", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BDA_PID_MAP", pack=False, align=None),
    "BDA_PID_UNMAP": SimStruct(OrderedDict((("ulcPIDs", SimTypeInt(signed=False, label="UInt32")), ("aulPIDs", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BDA_PID_UNMAP", pack=False, align=None),
    "BDA_PROGRAM_PID_LIST": SimStruct(OrderedDict((("ulProgramNumber", SimTypeInt(signed=False, label="UInt32")), ("ulcPIDs", SimTypeInt(signed=False, label="UInt32")), ("ulPID", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BDA_PROGRAM_PID_LIST", pack=False, align=None),
    "BDA_RATING_PINRESET": SimStruct(OrderedDict((("bPinLength", SimTypeChar(label="Byte")), ("argbNewPin", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_RATING_PINRESET", pack=False, align=None),
    "BDA_Range": SimTypeInt(signed=False, label="BDA_Range"),
    "BDA_SCAN_CAPABILTIES": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ul64AnalogStandardsSupported", SimTypeLongLong(signed=False, label="UInt64")),)), name="BDA_SCAN_CAPABILTIES", pack=False, align=None),
    "BDA_SCAN_START": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("LowerFrequency", SimTypeInt(signed=False, label="UInt32")), ("HigerFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_SCAN_START", pack=False, align=None),
    "BDA_SCAN_STATE": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulSignalLock", SimTypeInt(signed=False, label="UInt32")), ("ulSecondsLeft", SimTypeInt(signed=False, label="UInt32")), ("ulCurrentFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_SCAN_STATE", pack=False, align=None),
    "BDA_SIGNAL_STATE": SimTypeInt(signed=False, label="BDA_SIGNAL_STATE"),
    "BDA_SIGNAL_TIMEOUTS": SimStruct(OrderedDict((("ulCarrierTimeoutMs", SimTypeInt(signed=False, label="UInt32")), ("ulScanningTimeoutMs", SimTypeInt(signed=False, label="UInt32")), ("ulTuningTimeoutMs", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_SIGNAL_TIMEOUTS", pack=False, align=None),
    "BDA_STRING": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulStringSize", SimTypeInt(signed=False, label="UInt32")), ("argbString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_STRING", pack=False, align=None),
    "BDA_SignalType": SimTypeInt(signed=False, label="BDA_SignalType"),
    "BDA_TABLE_SECTION": SimStruct(OrderedDict((("ulPrimarySectionId", SimTypeInt(signed=False, label="UInt32")), ("ulSecondarySectionId", SimTypeInt(signed=False, label="UInt32")), ("ulcbSectionLength", SimTypeInt(signed=False, label="UInt32")), ("argbSectionData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BDA_TABLE_SECTION", pack=False, align=None),
    "BDA_TEMPLATE_CONNECTION": SimStruct(OrderedDict((("FromNodeType", SimTypeInt(signed=False, label="UInt32")), ("FromNodePinType", SimTypeInt(signed=False, label="UInt32")), ("ToNodeType", SimTypeInt(signed=False, label="UInt32")), ("ToNodePinType", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_TEMPLATE_CONNECTION", pack=False, align=None),
    "BDA_TEMPLATE_PIN_JOINT": SimStruct(OrderedDict((("uliTemplateConnection", SimTypeInt(signed=False, label="UInt32")), ("ulcInstancesMax", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_TEMPLATE_PIN_JOINT", pack=False, align=None),
    "BDA_TRANSPORT_INFO": SimStruct(OrderedDict((("ulcbPhyiscalPacket", SimTypeInt(signed=False, label="UInt32")), ("ulcbPhyiscalFrame", SimTypeInt(signed=False, label="UInt32")), ("ulcbPhyiscalFrameAlignment", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")),)), name="BDA_TRANSPORT_INFO", pack=False, align=None),
    "BDA_TS_SELECTORINFO": SimStruct(OrderedDict((("bTSInfolength", SimTypeChar(label="Byte")), ("bReserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("guidNetworkType", SimTypeBottom(label="Guid")), ("bTSIDCount", SimTypeChar(label="Byte")), ("usTSID", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="BDA_TS_SELECTORINFO", pack=False, align=None),
    "BDA_TS_SELECTORINFO_ISDBS_EXT": SimStruct(OrderedDict((("bTMCC", SimTypeArray(SimTypeChar(label="Byte"), 48)),)), name="BDA_TS_SELECTORINFO_ISDBS_EXT", pack=False, align=None),
    "BDA_TUNER_DIAGNOSTICS": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulSignalLevel", SimTypeInt(signed=False, label="UInt32")), ("ulSignalLevelQuality", SimTypeInt(signed=False, label="UInt32")), ("ulSignalNoiseRatio", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_TUNER_DIAGNOSTICS", pack=False, align=None),
    "BDA_TUNER_TUNERSTATE": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulTuneLength", SimTypeInt(signed=False, label="UInt32")), ("argbTuneData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_TUNER_TUNERSTATE", pack=False, align=None),
    "BDA_USERACTIVITY_INTERVAL": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulActivityInterval", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_USERACTIVITY_INTERVAL", pack=False, align=None),
    "BDA_WMDRMTUNER_PIDPROTECTION": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("uuidKeyID", SimTypeBottom(label="Guid")),)), name="BDA_WMDRMTUNER_PIDPROTECTION", pack=False, align=None),
    "BDA_WMDRMTUNER_PURCHASEENTITLEMENT": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulDescrambleStatus", SimTypeInt(signed=False, label="UInt32")), ("ulCaptureTokenLength", SimTypeInt(signed=False, label="UInt32")), ("argbCaptureTokenBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_WMDRMTUNER_PURCHASEENTITLEMENT", pack=False, align=None),
    "BDA_WMDRM_KEYINFOLIST": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulKeyuuidBufferLen", SimTypeInt(signed=False, label="UInt32")), ("argKeyuuidBuffer", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="BDA_WMDRM_KEYINFOLIST", pack=False, align=None),
    "BDA_WMDRM_RENEWLICENSE": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulDescrambleStatus", SimTypeInt(signed=False, label="UInt32")), ("ulXmrLicenseOutputLength", SimTypeInt(signed=False, label="UInt32")), ("argbXmrLicenceOutputBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BDA_WMDRM_RENEWLICENSE", pack=False, align=None),
    "BDA_WMDRM_STATUS": SimStruct(OrderedDict((("lResult", SimTypeInt(signed=True, label="Int32")), ("ulMaxCaptureTokenSize", SimTypeInt(signed=False, label="UInt32")), ("uMaxStreamingPid", SimTypeInt(signed=False, label="UInt32")), ("ulMaxLicense", SimTypeInt(signed=False, label="UInt32")), ("ulMinSecurityLevel", SimTypeInt(signed=False, label="UInt32")), ("ulRevInfoSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("ulRevInfoIssuedTime", SimTypeLongLong(signed=False, label="UInt64")), ("ulRevListVersion", SimTypeInt(signed=False, label="UInt32")), ("ulRevInfoTTL", SimTypeInt(signed=False, label="UInt32")), ("ulState", SimTypeInt(signed=False, label="UInt32")),)), name="BDA_WMDRM_STATUS", pack=False, align=None),
    "BDCB_CALLBACK_TYPE": SimTypeInt(signed=False, label="BDCB_CALLBACK_TYPE"),
    "BDCB_CLASSIFICATION": SimTypeInt(signed=False, label="BDCB_CLASSIFICATION"),
    "BDCB_IMAGE_INFORMATION": SimStruct(OrderedDict((("Classification", SimTypeInt(signed=False, label="BDCB_CLASSIFICATION")), ("ImageFlags", SimTypeInt(signed=False, label="UInt32")), ("ImageName", SimTypeRef("UNICODE_STRING", SimStruct)), ("RegistryPath", SimTypeRef("UNICODE_STRING", SimStruct)), ("CertificatePublisher", SimTypeRef("UNICODE_STRING", SimStruct)), ("CertificateIssuer", SimTypeRef("UNICODE_STRING", SimStruct)), ("ImageHash", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CertificateThumbprint", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ImageHashAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("ThumbprintHashAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("ImageHashLength", SimTypeInt(signed=False, label="UInt32")), ("CertificateThumbprintLength", SimTypeInt(signed=False, label="UInt32")),)), name="BDCB_IMAGE_INFORMATION", pack=False, align=None),
    "BDCB_STATUS_UPDATE_CONTEXT": SimStruct(OrderedDict((("StatusType", SimTypeInt(signed=False, label="BDCB_STATUS_UPDATE_TYPE")),)), name="BDCB_STATUS_UPDATE_CONTEXT", pack=False, align=None),
    "BDCB_STATUS_UPDATE_TYPE": SimTypeInt(signed=False, label="BDCB_STATUS_UPDATE_TYPE"),
    "BEHAVIOR_EVENT": SimTypeInt(signed=False, label="BEHAVIOR_EVENT"),
    "BEHAVIOR_EVENT_FLAGS": SimTypeInt(signed=False, label="BEHAVIOR_EVENT_FLAGS"),
    "BEHAVIOR_LAYOUT_INFO": SimTypeInt(signed=False, label="BEHAVIOR_LAYOUT_INFO"),
    "BEHAVIOR_LAYOUT_MODE": SimTypeInt(signed=False, label="BEHAVIOR_LAYOUT_MODE"),
    "BEHAVIOR_RELATION": SimTypeInt(signed=False, label="BEHAVIOR_RELATION"),
    "BEHAVIOR_RENDER_INFO": SimTypeInt(signed=False, label="BEHAVIOR_RENDER_INFO"),
    "BFFCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "uMsg", "lParam", "lpData"]), offset=0),
    "BGTYPE": SimTypeInt(signed=False, label="BGTYPE"),
    "BG_AUTH_CREDENTIALS": SimStruct(OrderedDict((("Target", SimTypeInt(signed=False, label="BG_AUTH_TARGET")), ("Scheme", SimTypeInt(signed=False, label="BG_AUTH_SCHEME")), ("Credentials", SimUnion({"Basic": SimTypeRef("BG_BASIC_CREDENTIALS", SimStruct)}, name="<anon>", label="None")),)), name="BG_AUTH_CREDENTIALS", pack=False, align=None),
    "BG_AUTH_CREDENTIALS_UNION": SimUnion({"Basic": SimTypeRef("BG_BASIC_CREDENTIALS", SimStruct)}, name="<anon>", label="None"),
    "BG_AUTH_SCHEME": SimTypeInt(signed=False, label="BG_AUTH_SCHEME"),
    "BG_AUTH_TARGET": SimTypeInt(signed=False, label="BG_AUTH_TARGET"),
    "BG_BASIC_CREDENTIALS": SimStruct(OrderedDict((("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Password", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BG_BASIC_CREDENTIALS", pack=False, align=None),
    "BG_CERT_STORE_LOCATION": SimTypeInt(signed=False, label="BG_CERT_STORE_LOCATION"),
    "BG_ERROR_CONTEXT": SimTypeInt(signed=False, label="BG_ERROR_CONTEXT"),
    "BG_FILE_INFO": SimStruct(OrderedDict((("RemoteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BG_FILE_INFO", pack=False, align=None),
    "BG_FILE_PROGRESS": SimStruct(OrderedDict((("BytesTotal", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferred", SimTypeLongLong(signed=False, label="UInt64")), ("Completed", SimTypeInt(signed=True, label="Int32")),)), name="BG_FILE_PROGRESS", pack=False, align=None),
    "BG_FILE_RANGE": SimStruct(OrderedDict((("InitialOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="BG_FILE_RANGE", pack=False, align=None),
    "BG_JOB_PRIORITY": SimTypeInt(signed=False, label="BG_JOB_PRIORITY"),
    "BG_JOB_PROGRESS": SimStruct(OrderedDict((("BytesTotal", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferred", SimTypeLongLong(signed=False, label="UInt64")), ("FilesTotal", SimTypeInt(signed=False, label="UInt32")), ("FilesTransferred", SimTypeInt(signed=False, label="UInt32")),)), name="BG_JOB_PROGRESS", pack=False, align=None),
    "BG_JOB_PROXY_USAGE": SimTypeInt(signed=False, label="BG_JOB_PROXY_USAGE"),
    "BG_JOB_REPLY_PROGRESS": SimStruct(OrderedDict((("BytesTotal", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferred", SimTypeLongLong(signed=False, label="UInt64")),)), name="BG_JOB_REPLY_PROGRESS", pack=False, align=None),
    "BG_JOB_STATE": SimTypeInt(signed=False, label="BG_JOB_STATE"),
    "BG_JOB_TIMES": SimStruct(OrderedDict((("CreationTime", SimTypeRef("FILETIME", SimStruct)), ("ModificationTime", SimTypeRef("FILETIME", SimStruct)), ("TransferCompletionTime", SimTypeRef("FILETIME", SimStruct)),)), name="BG_JOB_TIMES", pack=False, align=None),
    "BG_JOB_TYPE": SimTypeInt(signed=False, label="BG_JOB_TYPE"),
    "BG_TOKEN": SimTypeInt(signed=False, label="BG_TOKEN"),
    "BIDIOPTIONS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wMask", SimTypeShort(signed=False, label="UInt16")), ("wEffects", SimTypeShort(signed=False, label="UInt16")),)), name="BIDIOPTIONS", pack=False, align=None),
    "BIDI_DATA": SimStruct(OrderedDict((("dwBidiType", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"bData": SimTypeInt(signed=True, label="Int32"), "iData": SimTypeInt(signed=True, label="Int32"), "sData": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fData": SimTypeFloat(size=32), "biData": SimTypeRef("BINARY_CONTAINER", SimStruct)}, name="<anon>", label="None")),)), name="BIDI_DATA", pack=False, align=None),
    "BIDI_REQUEST_CONTAINER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("aData", SimTypePointer(SimTypeRef("BIDI_REQUEST_DATA", SimStruct), offset=0)),)), name="BIDI_REQUEST_CONTAINER", pack=False, align=None),
    "BIDI_REQUEST_DATA": SimStruct(OrderedDict((("dwReqNumber", SimTypeInt(signed=False, label="UInt32")), ("pSchema", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("data", SimTypeRef("BIDI_DATA", SimStruct)),)), name="BIDI_REQUEST_DATA", pack=False, align=None),
    "BIDI_RESPONSE_CONTAINER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("aData", SimTypePointer(SimTypeRef("BIDI_RESPONSE_DATA", SimStruct), offset=0)),)), name="BIDI_RESPONSE_CONTAINER", pack=False, align=None),
    "BIDI_RESPONSE_DATA": SimStruct(OrderedDict((("dwResult", SimTypeInt(signed=False, label="UInt32")), ("dwReqNumber", SimTypeInt(signed=False, label="UInt32")), ("pSchema", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("data", SimTypeRef("BIDI_DATA", SimStruct)),)), name="BIDI_RESPONSE_DATA", pack=False, align=None),
    "BIDI_TYPE": SimTypeInt(signed=False, label="BIDI_TYPE"),
    "BINARY_BLOB_CREDENTIAL_INFO": SimStruct(OrderedDict((("cbBlob", SimTypeInt(signed=False, label="UInt32")), ("pbBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BINARY_BLOB_CREDENTIAL_INFO", pack=False, align=None),
    "BINARY_CONTAINER": SimStruct(OrderedDict((("cbBuf", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BINARY_CONTAINER", pack=False, align=None),
    "BINARY_DATA": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="BINARY_DATA", pack=False, align=None),
    "BINDF": SimTypeInt(signed=False, label="BINDF"),
    "BINDF2": SimTypeInt(signed=False, label="BINDF2"),
    "BINDHANDLETYPES": SimTypeInt(signed=False, label="BINDHANDLETYPES"),
    "BINDINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("szExtraInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("stgmedData", SimTypeRef("STGMEDIUM", SimStruct)), ("grfBindInfoF", SimTypeInt(signed=False, label="UInt32")), ("dwBindVerb", SimTypeInt(signed=False, label="UInt32")), ("szCustomVerb", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbstgmedData", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwOptionsFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCodePage", SimTypeInt(signed=False, label="UInt32")), ("securityAttributes", SimTypeRef("SECURITY_ATTRIBUTES", SimStruct)), ("iid", SimTypeBottom(label="Guid")), ("pUnk", SimTypeBottom(label="IUnknown")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="BINDINFO", pack=False, align=None),
    "BINDINFOF": SimTypeInt(signed=False, label="BINDINFOF"),
    "BINDINFO_OPTIONS": SimTypeInt(signed=False, label="BINDINFO_OPTIONS"),
    "BINDPTR": SimUnion({"lpfuncdesc": SimTypePointer(SimTypeRef("FUNCDESC", SimStruct), offset=0), "lpvardesc": SimTypePointer(SimTypeRef("VARDESC", SimStruct), offset=0), "lptcomp": SimTypeBottom(label="ITypeComp")}, name="<anon>", label="None"),
    "BINDSPEED": SimTypeInt(signed=False, label="BINDSPEED"),
    "BINDSTATUS": SimTypeInt(signed=False, label="BINDSTATUS"),
    "BINDSTRING": SimTypeInt(signed=False, label="BINDSTRING"),
    "BINDVERB": SimTypeInt(signed=False, label="BINDVERB"),
    "BIND_FLAGS": SimTypeInt(signed=False, label="BIND_FLAGS"),
    "BIND_FLAGS1": SimTypeInt(signed=False, label="BIND_FLAGS1"),
    "BIND_OPTS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("grfFlags", SimTypeInt(signed=False, label="UInt32")), ("grfMode", SimTypeInt(signed=False, label="UInt32")), ("dwTickCountDeadline", SimTypeInt(signed=False, label="UInt32")),)), name="BIND_OPTS", pack=False, align=None),
    "BIND_OPTS2": SimStruct(OrderedDict((("Base", SimTypeRef("BIND_OPTS", SimStruct)), ("dwTrackFlags", SimTypeInt(signed=False, label="UInt32")), ("dwClassContext", SimTypeInt(signed=False, label="UInt32")), ("locale", SimTypeInt(signed=False, label="UInt32")), ("pServerInfo", SimTypePointer(SimTypeRef("COSERVERINFO", SimStruct), offset=0)),)), name="BIND_OPTS2", pack=False, align=None),
    "BIND_OPTS3": SimStruct(OrderedDict((("Base", SimTypeRef("BIND_OPTS2", SimStruct)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="BIND_OPTS3", pack=False, align=None),
    "BIN_COUNT": SimStruct(OrderedDict((("BinRange", SimTypeRef("BIN_RANGE", SimStruct)), ("BinCount", SimTypeInt(signed=False, label="UInt32")),)), name="BIN_COUNT", pack=False, align=None),
    "BIN_RANGE": SimStruct(OrderedDict((("StartValue", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="BIN_RANGE", pack=False, align=None),
    "BIN_RESULTS": SimStruct(OrderedDict((("NumberOfBins", SimTypeInt(signed=False, label="UInt32")), ("BinCounts", SimTypePointer(SimTypeRef("BIN_COUNT", SimStruct), offset=0)),)), name="BIN_RESULTS", pack=False, align=None),
    "BIN_TYPES": SimTypeInt(signed=False, label="BIN_TYPES"),
    "BITMAP": SimStruct(OrderedDict((("bmType", SimTypeInt(signed=True, label="Int32")), ("bmWidth", SimTypeInt(signed=True, label="Int32")), ("bmHeight", SimTypeInt(signed=True, label="Int32")), ("bmWidthBytes", SimTypeInt(signed=True, label="Int32")), ("bmPlanes", SimTypeShort(signed=False, label="UInt16")), ("bmBitsPixel", SimTypeShort(signed=False, label="UInt16")), ("bmBits", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="BITMAP", pack=False, align=None),
    "BITMAPCOREHEADER": SimStruct(OrderedDict((("bcSize", SimTypeInt(signed=False, label="UInt32")), ("bcWidth", SimTypeShort(signed=False, label="UInt16")), ("bcHeight", SimTypeShort(signed=False, label="UInt16")), ("bcPlanes", SimTypeShort(signed=False, label="UInt16")), ("bcBitCount", SimTypeShort(signed=False, label="UInt16")),)), name="BITMAPCOREHEADER", pack=False, align=None),
    "BITMAPCOREINFO": SimStruct(OrderedDict((("bmciHeader", SimTypeRef("BITMAPCOREHEADER", SimStruct)), ("bmciColors", SimTypePointer(SimTypeRef("RGBTRIPLE", SimStruct), offset=0)),)), name="BITMAPCOREINFO", pack=False, align=None),
    "BITMAPFILEHEADER": SimStruct(OrderedDict((("bfType", SimTypeShort(signed=False, label="UInt16")), ("bfSize", SimTypeInt(signed=False, label="UInt32")), ("bfReserved1", SimTypeShort(signed=False, label="UInt16")), ("bfReserved2", SimTypeShort(signed=False, label="UInt16")), ("bfOffBits", SimTypeInt(signed=False, label="UInt32")),)), name="BITMAPFILEHEADER", pack=False, align=None),
    "BITMAPINFO": SimStruct(OrderedDict((("bmiHeader", SimTypeRef("BITMAPINFOHEADER", SimStruct)), ("bmiColors", SimTypePointer(SimTypeRef("RGBQUAD", SimStruct), offset=0)),)), name="BITMAPINFO", pack=False, align=None),
    "BITMAPINFOHEADER": SimStruct(OrderedDict((("biSize", SimTypeInt(signed=False, label="UInt32")), ("biWidth", SimTypeInt(signed=True, label="Int32")), ("biHeight", SimTypeInt(signed=True, label="Int32")), ("biPlanes", SimTypeShort(signed=False, label="UInt16")), ("biBitCount", SimTypeShort(signed=False, label="UInt16")), ("biCompression", SimTypeInt(signed=False, label="UInt32")), ("biSizeImage", SimTypeInt(signed=False, label="UInt32")), ("biXPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("biYPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("biClrUsed", SimTypeInt(signed=False, label="UInt32")), ("biClrImportant", SimTypeInt(signed=False, label="UInt32")),)), name="BITMAPINFOHEADER", pack=False, align=None),
    "BITMAPV4HEADER": SimStruct(OrderedDict((("bV4Size", SimTypeInt(signed=False, label="UInt32")), ("bV4Width", SimTypeInt(signed=True, label="Int32")), ("bV4Height", SimTypeInt(signed=True, label="Int32")), ("bV4Planes", SimTypeShort(signed=False, label="UInt16")), ("bV4BitCount", SimTypeShort(signed=False, label="UInt16")), ("bV4V4Compression", SimTypeInt(signed=False, label="BI_COMPRESSION")), ("bV4SizeImage", SimTypeInt(signed=False, label="UInt32")), ("bV4XPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("bV4YPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("bV4ClrUsed", SimTypeInt(signed=False, label="UInt32")), ("bV4ClrImportant", SimTypeInt(signed=False, label="UInt32")), ("bV4RedMask", SimTypeInt(signed=False, label="UInt32")), ("bV4GreenMask", SimTypeInt(signed=False, label="UInt32")), ("bV4BlueMask", SimTypeInt(signed=False, label="UInt32")), ("bV4AlphaMask", SimTypeInt(signed=False, label="UInt32")), ("bV4CSType", SimTypeInt(signed=False, label="UInt32")), ("bV4Endpoints", SimTypeRef("CIEXYZTRIPLE", SimStruct)), ("bV4GammaRed", SimTypeInt(signed=False, label="UInt32")), ("bV4GammaGreen", SimTypeInt(signed=False, label="UInt32")), ("bV4GammaBlue", SimTypeInt(signed=False, label="UInt32")),)), name="BITMAPV4HEADER", pack=False, align=None),
    "BITMAPV5HEADER": SimStruct(OrderedDict((("bV5Size", SimTypeInt(signed=False, label="UInt32")), ("bV5Width", SimTypeInt(signed=True, label="Int32")), ("bV5Height", SimTypeInt(signed=True, label="Int32")), ("bV5Planes", SimTypeShort(signed=False, label="UInt16")), ("bV5BitCount", SimTypeShort(signed=False, label="UInt16")), ("bV5Compression", SimTypeInt(signed=False, label="BI_COMPRESSION")), ("bV5SizeImage", SimTypeInt(signed=False, label="UInt32")), ("bV5XPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("bV5YPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("bV5ClrUsed", SimTypeInt(signed=False, label="UInt32")), ("bV5ClrImportant", SimTypeInt(signed=False, label="UInt32")), ("bV5RedMask", SimTypeInt(signed=False, label="UInt32")), ("bV5GreenMask", SimTypeInt(signed=False, label="UInt32")), ("bV5BlueMask", SimTypeInt(signed=False, label="UInt32")), ("bV5AlphaMask", SimTypeInt(signed=False, label="UInt32")), ("bV5CSType", SimTypeInt(signed=False, label="UInt32")), ("bV5Endpoints", SimTypeRef("CIEXYZTRIPLE", SimStruct)), ("bV5GammaRed", SimTypeInt(signed=False, label="UInt32")), ("bV5GammaGreen", SimTypeInt(signed=False, label="UInt32")), ("bV5GammaBlue", SimTypeInt(signed=False, label="UInt32")), ("bV5Intent", SimTypeInt(signed=False, label="UInt32")), ("bV5ProfileData", SimTypeInt(signed=False, label="UInt32")), ("bV5ProfileSize", SimTypeInt(signed=False, label="UInt32")), ("bV5Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="BITMAPV5HEADER", pack=False, align=None),
    "BITMAP_RENDERER_STATISTICS": SimStruct(OrderedDict((("dwFramesDelivered", SimTypeInt(signed=False, label="UInt32")), ("dwFramesDropped", SimTypeInt(signed=False, label="UInt32")),)), name="BITMAP_RENDERER_STATISTICS", pack=False, align=None),
    "BITS_FILE_PROPERTY_ID": SimTypeInt(signed=False, label="BITS_FILE_PROPERTY_ID"),
    "BITS_FILE_PROPERTY_VALUE": SimUnion({"String": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "BITS_JOB_PROPERTY_ID": SimTypeInt(signed=False, label="BITS_JOB_PROPERTY_ID"),
    "BITS_JOB_PROPERTY_VALUE": SimUnion({"Dword": SimTypeInt(signed=False, label="UInt32"), "ClsID": SimTypeBottom(label="Guid"), "Enable": SimTypeInt(signed=True, label="Int32"), "Uint64": SimTypeLongLong(signed=False, label="UInt64"), "Target": SimTypeInt(signed=False, label="BG_AUTH_TARGET")}, name="<anon>", label="None"),
    "BITS_JOB_TRANSFER_POLICY": SimTypeInt(signed=False, label="BITS_JOB_TRANSFER_POLICY"),
    "BI_COMPRESSION": SimTypeInt(signed=False, label="BI_COMPRESSION"),
    "BLENDFUNCTION": SimStruct(OrderedDict((("BlendOp", SimTypeChar(label="Byte")), ("BlendFlags", SimTypeChar(label="Byte")), ("SourceConstantAlpha", SimTypeChar(label="Byte")), ("AlphaFormat", SimTypeChar(label="Byte")),)), name="BLENDFUNCTION", pack=False, align=None),
    "BLENDOBJ": SimStruct(OrderedDict((("BlendFunction", SimTypeRef("BLENDFUNCTION", SimStruct)),)), name="BLENDOBJ", pack=False, align=None),
    "BLOB": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pBlobData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BLOB", pack=False, align=None),
    "BLUETOOTH_ADDRESS": SimStruct(OrderedDict((("Anonymous", SimUnion({"ullLong": SimTypeLongLong(signed=False, label="UInt64"), "rgBytes": SimTypeArray(SimTypeChar(label="Byte"), 6)}, name="<anon>", label="None")),)), name="BLUETOOTH_ADDRESS", pack=False, align=None),
    "BLUETOOTH_AUTHENTICATE_RESPONSE": SimStruct(OrderedDict((("bthAddressRemote", SimTypeRef("BLUETOOTH_ADDRESS", SimStruct)), ("authMethod", SimTypeInt(signed=False, label="BLUETOOTH_AUTHENTICATION_METHOD")), ("Anonymous", SimUnion({"pinInfo": SimTypeRef("BLUETOOTH_PIN_INFO", SimStruct), "oobInfo": SimTypeRef("BLUETOOTH_OOB_DATA_INFO", SimStruct), "numericCompInfo": SimTypeRef("BLUETOOTH_NUMERIC_COMPARISON_INFO", SimStruct), "passkeyInfo": SimTypeRef("BLUETOOTH_PASSKEY_INFO", SimStruct)}, name="<anon>", label="None")), ("negativeResponse", SimTypeChar(label="Byte")),)), name="BLUETOOTH_AUTHENTICATE_RESPONSE", pack=False, align=None),
    "BLUETOOTH_AUTHENTICATION_CALLBACK_PARAMS": SimStruct(OrderedDict((("deviceInfo", SimTypeRef("BLUETOOTH_DEVICE_INFO", SimStruct)), ("authenticationMethod", SimTypeInt(signed=False, label="BLUETOOTH_AUTHENTICATION_METHOD")), ("ioCapability", SimTypeInt(signed=False, label="BLUETOOTH_IO_CAPABILITY")), ("authenticationRequirements", SimTypeInt(signed=False, label="BLUETOOTH_AUTHENTICATION_REQUIREMENTS")), ("Anonymous", SimUnion({"Numeric_Value": SimTypeInt(signed=False, label="UInt32"), "Passkey": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="BLUETOOTH_AUTHENTICATION_CALLBACK_PARAMS", pack=False, align=None),
    "BLUETOOTH_AUTHENTICATION_METHOD": SimTypeInt(signed=False, label="BLUETOOTH_AUTHENTICATION_METHOD"),
    "BLUETOOTH_AUTHENTICATION_REQUIREMENTS": SimTypeInt(signed=False, label="BLUETOOTH_AUTHENTICATION_REQUIREMENTS"),
    "BLUETOOTH_COD_PAIRS": SimStruct(OrderedDict((("ulCODMask", SimTypeInt(signed=False, label="UInt32")), ("pcszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BLUETOOTH_COD_PAIRS", pack=False, align=None),
    "BLUETOOTH_DEVICE_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeRef("BLUETOOTH_ADDRESS", SimStruct)), ("ulClassofDevice", SimTypeInt(signed=False, label="UInt32")), ("fConnected", SimTypeInt(signed=True, label="Int32")), ("fRemembered", SimTypeInt(signed=True, label="Int32")), ("fAuthenticated", SimTypeInt(signed=True, label="Int32")), ("stLastSeen", SimTypeRef("SYSTEMTIME", SimStruct)), ("stLastUsed", SimTypeRef("SYSTEMTIME", SimStruct)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 248)),)), name="BLUETOOTH_DEVICE_INFO", pack=False, align=None),
    "BLUETOOTH_DEVICE_SEARCH_PARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fReturnAuthenticated", SimTypeInt(signed=True, label="Int32")), ("fReturnRemembered", SimTypeInt(signed=True, label="Int32")), ("fReturnUnknown", SimTypeInt(signed=True, label="Int32")), ("fReturnConnected", SimTypeInt(signed=True, label="Int32")), ("fIssueInquiry", SimTypeInt(signed=True, label="Int32")), ("cTimeoutMultiplier", SimTypeChar(label="Byte")), ("hRadio", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="BLUETOOTH_DEVICE_SEARCH_PARAMS", pack=False, align=None),
    "BLUETOOTH_FIND_RADIO_PARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="BLUETOOTH_FIND_RADIO_PARAMS", pack=False, align=None),
    "BLUETOOTH_GATT_VALUE_CHANGED_EVENT": SimStruct(OrderedDict((("ChangedAttributeHandle", SimTypeShort(signed=False, label="UInt16")), ("CharacteristicValueDataSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CharacteristicValue", SimTypePointer(SimTypeRef("BTH_LE_GATT_CHARACTERISTIC_VALUE", SimStruct), offset=0)),)), name="BLUETOOTH_GATT_VALUE_CHANGED_EVENT", pack=False, align=None),
    "BLUETOOTH_GATT_VALUE_CHANGED_EVENT_REGISTRATION": SimStruct(OrderedDict((("NumCharacteristics", SimTypeShort(signed=False, label="UInt16")), ("Characteristics", SimTypePointer(SimTypeRef("BTH_LE_GATT_CHARACTERISTIC", SimStruct), offset=0)),)), name="BLUETOOTH_GATT_VALUE_CHANGED_EVENT_REGISTRATION", pack=False, align=None),
    "BLUETOOTH_IO_CAPABILITY": SimTypeInt(signed=False, label="BLUETOOTH_IO_CAPABILITY"),
    "BLUETOOTH_LOCAL_SERVICE_INFO": SimStruct(OrderedDict((("Enabled", SimTypeInt(signed=True, label="Int32")), ("btAddr", SimTypeRef("BLUETOOTH_ADDRESS", SimStruct)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("szDeviceString", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="BLUETOOTH_LOCAL_SERVICE_INFO", pack=False, align=None),
    "BLUETOOTH_NUMERIC_COMPARISON_INFO": SimStruct(OrderedDict((("NumericValue", SimTypeInt(signed=False, label="UInt32")),)), name="BLUETOOTH_NUMERIC_COMPARISON_INFO", pack=False, align=None),
    "BLUETOOTH_OOB_DATA_INFO": SimStruct(OrderedDict((("C", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("R", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="BLUETOOTH_OOB_DATA_INFO", pack=False, align=None),
    "BLUETOOTH_PASSKEY_INFO": SimStruct(OrderedDict((("passkey", SimTypeInt(signed=False, label="UInt32")),)), name="BLUETOOTH_PASSKEY_INFO", pack=False, align=None),
    "BLUETOOTH_PIN_INFO": SimStruct(OrderedDict((("pin", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("pinLength", SimTypeChar(label="Byte")),)), name="BLUETOOTH_PIN_INFO", pack=False, align=None),
    "BLUETOOTH_RADIO_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("address", SimTypeRef("BLUETOOTH_ADDRESS", SimStruct)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 248)), ("ulClassofDevice", SimTypeInt(signed=False, label="UInt32")), ("lmpSubversion", SimTypeShort(signed=False, label="UInt16")), ("manufacturer", SimTypeShort(signed=False, label="UInt16")),)), name="BLUETOOTH_RADIO_INFO", pack=False, align=None),
    "BLUETOOTH_SELECT_DEVICE_PARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cNumOfClasses", SimTypeInt(signed=False, label="UInt32")), ("prgClassOfDevices", SimTypePointer(SimTypeRef("BLUETOOTH_COD_PAIRS", SimStruct), offset=0)), ("pszInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fForceAuthentication", SimTypeInt(signed=True, label="Int32")), ("fShowAuthenticated", SimTypeInt(signed=True, label="Int32")), ("fShowRemembered", SimTypeInt(signed=True, label="Int32")), ("fShowUnknown", SimTypeInt(signed=True, label="Int32")), ("fAddNewDeviceWizard", SimTypeInt(signed=True, label="Int32")), ("fSkipServicesPage", SimTypeInt(signed=True, label="Int32")), ("pfnDeviceCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("BLUETOOTH_DEVICE_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvParam", "pDevice"]), offset=0)), ("pvParam", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cNumDevices", SimTypeInt(signed=False, label="UInt32")), ("pDevices", SimTypePointer(SimTypeRef("BLUETOOTH_DEVICE_INFO", SimStruct), offset=0)),)), name="BLUETOOTH_SELECT_DEVICE_PARAMS", pack=False, align=None),
    "BMFORMAT": SimTypeInt(signed=False, label="BMFORMAT"),
    "BM_REQUEST_TYPE": SimUnion({"s": SimTypeRef("_BM", SimStruct), "B": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "BNSTATE": SimTypeInt(signed=False, label="BNSTATE"),
    "BODYSTATES": SimTypeInt(signed=False, label="BODYSTATES"),
    "BOID": SimStruct(OrderedDict((("rgb", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="BOID", pack=False, align=None),
    "BOOL": SimTypeInt(signed=True, label="Int32"),
    "BOOLEAN": SimTypeChar(label="Byte"),
    "BOOTDISK_INFORMATION": SimStruct(OrderedDict((("BootPartitionOffset", SimTypeLongLong(signed=True, label="Int64")), ("SystemPartitionOffset", SimTypeLongLong(signed=True, label="Int64")), ("BootDeviceSignature", SimTypeInt(signed=False, label="UInt32")), ("SystemDeviceSignature", SimTypeInt(signed=False, label="UInt32")),)), name="BOOTDISK_INFORMATION", pack=False, align=None),
    "BOOTDISK_INFORMATION_EX": SimStruct(OrderedDict((("BootPartitionOffset", SimTypeLongLong(signed=True, label="Int64")), ("SystemPartitionOffset", SimTypeLongLong(signed=True, label="Int64")), ("BootDeviceSignature", SimTypeInt(signed=False, label="UInt32")), ("SystemDeviceSignature", SimTypeInt(signed=False, label="UInt32")), ("BootDeviceGuid", SimTypeBottom(label="Guid")), ("SystemDeviceGuid", SimTypeBottom(label="Guid")), ("BootDeviceIsGpt", SimTypeChar(label="Byte")), ("SystemDeviceIsGpt", SimTypeChar(label="Byte")),)), name="BOOTDISK_INFORMATION_EX", pack=False, align=None),
    "BOOTDISK_INFORMATION_LITE": SimStruct(OrderedDict((("NumberEntries", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeRef("LOADER_PARTITION_INFORMATION_EX", SimStruct), offset=0)),)), name="BOOTDISK_INFORMATION_LITE", pack=False, align=None),
    "BOOT_AREA_INFO": SimStruct(OrderedDict((("BootSectorCount", SimTypeInt(signed=False, label="UInt32")), ("BootSectors", SimTypeArray(SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous_e__Struct", pack=False, align=None), 2)),)), name="BOOT_AREA_INFO", pack=False, align=None),
    "BOOT_DRIVER_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="BDCB_CALLBACK_TYPE"), SimTypePointer(SimTypeRef("BDCB_IMAGE_INFORMATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackContext", "Classification", "ImageInformation"]), offset=0),
    "BORDERSTATES": SimTypeInt(signed=False, label="BORDERSTATES"),
    "BORDERTYPE": SimTypeInt(signed=False, label="BORDERTYPE"),
    "BORDER_HSCROLLSTATES": SimTypeInt(signed=False, label="BORDER_HSCROLLSTATES"),
    "BORDER_HVSCROLLSTATES": SimTypeInt(signed=False, label="BORDER_HVSCROLLSTATES"),
    "BORDER_NOSCROLLSTATES": SimTypeInt(signed=False, label="BORDER_NOSCROLLSTATES"),
    "BORDER_VSCROLLSTATES": SimTypeInt(signed=False, label="BORDER_VSCROLLSTATES"),
    "BOUND_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="BOUND_CALLBACK_STATUS")), offset=0),
    "BOUND_CALLBACK_STATUS": SimTypeInt(signed=False, label="BOUND_CALLBACK_STATUS"),
    "BP_ANIMATIONPARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("style", SimTypeInt(signed=False, label="BP_ANIMATIONSTYLE")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")),)), name="BP_ANIMATIONPARAMS", pack=False, align=None),
    "BP_ANIMATIONSTYLE": SimTypeInt(signed=False, label="BP_ANIMATIONSTYLE"),
    "BP_BUFFERFORMAT": SimTypeInt(signed=False, label="BP_BUFFERFORMAT"),
    "BP_PAINTPARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="BP_PAINTPARAMS_FLAGS")), ("prcExclude", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("pBlendFunction", SimTypePointer(SimTypeRef("BLENDFUNCTION", SimStruct), offset=0)),)), name="BP_PAINTPARAMS", pack=False, align=None),
    "BP_PAINTPARAMS_FLAGS": SimTypeInt(signed=False, label="BP_PAINTPARAMS_FLAGS"),
    "BREAKPOINT_STATE": SimTypeInt(signed=False, label="BREAKPOINT_STATE"),
    "BREAKREASON": SimTypeInt(signed=False, label="BREAKREASON"),
    "BREAKRESUMEACTION": SimTypeInt(signed=False, label="BREAKRESUMEACTION"),
    "BRIGHTNESS_INTERFACE_VERSION": SimTypeInt(signed=False, label="BRIGHTNESS_INTERFACE_VERSION"),
    "BRIGHTNESS_LEVEL": SimStruct(OrderedDict((("Count", SimTypeChar(label="Byte")), ("Level", SimTypeArray(SimTypeChar(label="Byte"), 103)),)), name="BRIGHTNESS_LEVEL", pack=False, align=None),
    "BRIGHTNESS_NIT_RANGE": SimStruct(OrderedDict((("MinLevelInMillinit", SimTypeInt(signed=False, label="UInt32")), ("MaxLevelInMillinit", SimTypeInt(signed=False, label="UInt32")), ("StepSizeInMillinit", SimTypeInt(signed=False, label="UInt32")),)), name="BRIGHTNESS_NIT_RANGE", pack=False, align=None),
    "BRIGHTNESS_NIT_RANGES": SimStruct(OrderedDict((("NormalRangeCount", SimTypeInt(signed=False, label="UInt32")), ("RangeCount", SimTypeInt(signed=False, label="UInt32")), ("PreferredMaximumBrightness", SimTypeInt(signed=False, label="UInt32")), ("SupportedRanges", SimTypeArray(SimTypeRef("BRIGHTNESS_NIT_RANGE", SimStruct), 16)),)), name="BRIGHTNESS_NIT_RANGES", pack=False, align=None),
    "BROADCAST_SYSTEM_MESSAGE_FLAGS": SimTypeInt(signed=False, label="BROADCAST_SYSTEM_MESSAGE_FLAGS"),
    "BROADCAST_SYSTEM_MESSAGE_INFO": SimTypeInt(signed=False, label="BROADCAST_SYSTEM_MESSAGE_INFO"),
    "BROWSEINFOA": SimStruct(OrderedDict((("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pidlRoot", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("pszDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpfn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "uMsg", "lParam", "lpData"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")),)), name="BROWSEINFOA", pack=False, align=None),
    "BROWSEINFOW": SimStruct(OrderedDict((("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pidlRoot", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("pszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpfn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "uMsg", "lParam", "lpData"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")),)), name="BROWSEINFOW", pack=False, align=None),
    "BRUSHOBJ": SimStruct(OrderedDict((("iSolidColor", SimTypeInt(signed=False, label="UInt32")), ("pvRbrush", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("flColorType", SimTypeInt(signed=False, label="UInt32")),)), name="BRUSHOBJ", pack=False, align=None),
    "BRUSH_STYLE": SimTypeInt(signed=False, label="BRUSH_STYLE"),
    "BSCF": SimTypeInt(signed=False, label="BSCF"),
    "BSMINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hdesk", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("luid", SimTypeRef("LUID", SimStruct)),)), name="BSMINFO", pack=False, align=None),
    "BSSID_INFO": SimStruct(OrderedDict((("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("PMKID", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="BSSID_INFO", pack=False, align=None),
    "BSTR": SimTypePointer(SimTypeChar(label="Char"), offset=0),
    "BSTRBLOB": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BSTRBLOB", pack=False, align=None),
    "BTH_DEVICE_INFO": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("address", SimTypeLongLong(signed=False, label="UInt64")), ("classOfDevice", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypeArray(SimTypeChar(label="SByte"), 248)),)), name="BTH_DEVICE_INFO", pack=False, align=None),
    "BTH_HCI_EVENT_INFO": SimStruct(OrderedDict((("bthAddress", SimTypeLongLong(signed=False, label="UInt64")), ("connectionType", SimTypeChar(label="Byte")), ("connected", SimTypeChar(label="Byte")),)), name="BTH_HCI_EVENT_INFO", pack=False, align=None),
    "BTH_INFO_REQ": SimStruct(OrderedDict((("btAddr", SimTypeLongLong(signed=False, label="UInt64")), ("infoType", SimTypeShort(signed=False, label="UInt16")),)), name="BTH_INFO_REQ", pack=False, align=None),
    "BTH_INFO_RSP": SimStruct(OrderedDict((("result", SimTypeShort(signed=False, label="UInt16")), ("dataLen", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"connectionlessMTU": SimTypeShort(signed=False, label="UInt16"), "data": SimTypeArray(SimTypeChar(label="Byte"), 44)}, name="<anon>", label="None")),)), name="BTH_INFO_RSP", pack=False, align=None),
    "BTH_L2CAP_EVENT_INFO": SimStruct(OrderedDict((("bthAddress", SimTypeLongLong(signed=False, label="UInt64")), ("psm", SimTypeShort(signed=False, label="UInt16")), ("connected", SimTypeChar(label="Byte")), ("initiated", SimTypeChar(label="Byte")),)), name="BTH_L2CAP_EVENT_INFO", pack=False, align=None),
    "BTH_LE_GATT_CHARACTERISTIC": SimStruct(OrderedDict((("ServiceHandle", SimTypeShort(signed=False, label="UInt16")), ("CharacteristicUuid", SimTypeRef("BTH_LE_UUID", SimStruct)), ("AttributeHandle", SimTypeShort(signed=False, label="UInt16")), ("CharacteristicValueHandle", SimTypeShort(signed=False, label="UInt16")), ("IsBroadcastable", SimTypeChar(label="Byte")), ("IsReadable", SimTypeChar(label="Byte")), ("IsWritable", SimTypeChar(label="Byte")), ("IsWritableWithoutResponse", SimTypeChar(label="Byte")), ("IsSignedWritable", SimTypeChar(label="Byte")), ("IsNotifiable", SimTypeChar(label="Byte")), ("IsIndicatable", SimTypeChar(label="Byte")), ("HasExtendedProperties", SimTypeChar(label="Byte")),)), name="BTH_LE_GATT_CHARACTERISTIC", pack=False, align=None),
    "BTH_LE_GATT_CHARACTERISTIC_VALUE": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BTH_LE_GATT_CHARACTERISTIC_VALUE", pack=False, align=None),
    "BTH_LE_GATT_DESCRIPTOR": SimStruct(OrderedDict((("ServiceHandle", SimTypeShort(signed=False, label="UInt16")), ("CharacteristicHandle", SimTypeShort(signed=False, label="UInt16")), ("DescriptorType", SimTypeInt(signed=False, label="BTH_LE_GATT_DESCRIPTOR_TYPE")), ("DescriptorUuid", SimTypeRef("BTH_LE_UUID", SimStruct)), ("AttributeHandle", SimTypeShort(signed=False, label="UInt16")),)), name="BTH_LE_GATT_DESCRIPTOR", pack=False, align=None),
    "BTH_LE_GATT_DESCRIPTOR_TYPE": SimTypeInt(signed=False, label="BTH_LE_GATT_DESCRIPTOR_TYPE"),
    "BTH_LE_GATT_DESCRIPTOR_VALUE": SimStruct(OrderedDict((("DescriptorType", SimTypeInt(signed=False, label="BTH_LE_GATT_DESCRIPTOR_TYPE")), ("DescriptorUuid", SimTypeRef("BTH_LE_UUID", SimStruct)), ("Anonymous", SimUnion({"CharacteristicExtendedProperties": SimTypeRef("_CharacteristicExtendedProperties_e__Struct", SimStruct), "ClientCharacteristicConfiguration": SimTypeRef("_ClientCharacteristicConfiguration_e__Struct", SimStruct), "ServerCharacteristicConfiguration": SimTypeRef("_ServerCharacteristicConfiguration_e__Struct", SimStruct), "CharacteristicFormat": SimTypeRef("_CharacteristicFormat_e__Struct", SimStruct)}, name="<anon>", label="None")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BTH_LE_GATT_DESCRIPTOR_VALUE", pack=False, align=None),
    "BTH_LE_GATT_EVENT_TYPE": SimTypeInt(signed=False, label="BTH_LE_GATT_EVENT_TYPE"),
    "BTH_LE_GATT_SERVICE": SimStruct(OrderedDict((("ServiceUuid", SimTypeRef("BTH_LE_UUID", SimStruct)), ("AttributeHandle", SimTypeShort(signed=False, label="UInt16")),)), name="BTH_LE_GATT_SERVICE", pack=False, align=None),
    "BTH_LE_UUID": SimStruct(OrderedDict((("IsShortUuid", SimTypeChar(label="Byte")), ("Value", SimUnion({"ShortUuid": SimTypeShort(signed=False, label="UInt16"), "LongUuid": SimTypeBottom(label="Guid")}, name="<anon>", label="None")),)), name="BTH_LE_UUID", pack=False, align=None),
    "BTH_PING_REQ": SimStruct(OrderedDict((("btAddr", SimTypeLongLong(signed=False, label="UInt64")), ("dataLen", SimTypeChar(label="Byte")), ("data", SimTypeArray(SimTypeChar(label="Byte"), 44)),)), name="BTH_PING_REQ", pack=False, align=None),
    "BTH_PING_RSP": SimStruct(OrderedDict((("dataLen", SimTypeChar(label="Byte")), ("data", SimTypeArray(SimTypeChar(label="Byte"), 44)),)), name="BTH_PING_RSP", pack=False, align=None),
    "BTH_QUERY_DEVICE": SimStruct(OrderedDict((("LAP", SimTypeInt(signed=False, label="UInt32")), ("length", SimTypeChar(label="Byte")),)), name="BTH_QUERY_DEVICE", pack=False, align=None),
    "BTH_QUERY_SERVICE": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="UInt32")), ("serviceHandle", SimTypeInt(signed=False, label="UInt32")), ("uuids", SimTypeArray(SimTypeRef("SdpQueryUuid", SimStruct), 12)), ("numRange", SimTypeInt(signed=False, label="UInt32")), ("pRange", SimTypePointer(SimTypeRef("SdpAttributeRange", SimStruct), offset=0)),)), name="BTH_QUERY_SERVICE", pack=False, align=None),
    "BTH_RADIO_IN_RANGE": SimStruct(OrderedDict((("deviceInfo", SimTypeRef("BTH_DEVICE_INFO", SimStruct)), ("previousDeviceFlags", SimTypeInt(signed=False, label="UInt32")),)), name="BTH_RADIO_IN_RANGE", pack=False, align=None),
    "BTH_SET_SERVICE": SimStruct(OrderedDict((("pSdpVersion", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pRecordHandle", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("fCodService", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)), ("ulRecordLength", SimTypeInt(signed=False, label="UInt32")), ("pRecord", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BTH_SET_SERVICE", pack=False, align=None),
    "BUCKETCATEGORIZE": SimStruct(OrderedDict((("cBuckets", SimTypeInt(signed=False, label="UInt32")), ("Distribution", SimTypeInt(signed=False, label="UInt32")),)), name="BUCKETCATEGORIZE", pack=False, align=None),
    "BUCKET_COUNTER": SimStruct(OrderedDict((("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Trim", SimTypeInt(signed=False, label="UInt32")), ("Write", SimTypeInt(signed=False, label="UInt32")), ("Read", SimTypeInt(signed=False, label="UInt32")),)), name="BUCKET_COUNTER", pack=False, align=None),
    "BUGCHECK_ERROR": SimTypeInt(signed=False, label="BUGCHECK_ERROR"),
    "BULK_SECURITY_TEST_DATA": SimStruct(OrderedDict((("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("SecurityIds", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="BULK_SECURITY_TEST_DATA", pack=False, align=None),
    "BUSDATA": SimStruct(OrderedDict((("BusDataType", SimTypeInt(signed=False, label="UInt32")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("SlotNumber", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="BUSDATA", pack=False, align=None),
    "BUSNUMBER_DES": SimStruct(OrderedDict((("BUSD_Count", SimTypeInt(signed=False, label="UInt32")), ("BUSD_Type", SimTypeInt(signed=False, label="UInt32")), ("BUSD_Flags", SimTypeInt(signed=False, label="UInt32")), ("BUSD_Alloc_Base", SimTypeInt(signed=False, label="UInt32")), ("BUSD_Alloc_End", SimTypeInt(signed=False, label="UInt32")),)), name="BUSNUMBER_DES", pack=False, align=None),
    "BUSNUMBER_RANGE": SimStruct(OrderedDict((("BUSR_Min", SimTypeInt(signed=False, label="UInt32")), ("BUSR_Max", SimTypeInt(signed=False, label="UInt32")), ("BUSR_nBusNumbers", SimTypeInt(signed=False, label="UInt32")), ("BUSR_Flags", SimTypeInt(signed=False, label="UInt32")),)), name="BUSNUMBER_RANGE", pack=False, align=None),
    "BUSNUMBER_RESOURCE": SimStruct(OrderedDict((("BusNumber_Header", SimTypeRef("BUSNUMBER_DES", SimStruct)), ("BusNumber_Data", SimTypePointer(SimTypeRef("BUSNUMBER_RANGE", SimStruct), offset=0)),)), name="BUSNUMBER_RESOURCE", pack=False, align=None),
    "BUSY_DIALOG_FLAGS": SimTypeInt(signed=False, label="BUSY_DIALOG_FLAGS"),
    "BUS_DATA_TYPE": SimTypeInt(signed=False, label="BUS_DATA_TYPE"),
    "BUS_INTERFACE_STANDARD": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("TranslateBusAddress", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)), ("GetDmaAdapter", SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)), offset=0)), ("SetBusData", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("GetBusData", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)),)), name="BUS_INTERFACE_STANDARD", pack=False, align=None),
    "BUS_QUERY_ID_TYPE": SimTypeInt(signed=False, label="BUS_QUERY_ID_TYPE"),
    "BUS_RESOURCE_UPDATE_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("GetUpdatedBusResource", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="BUS_RESOURCE_UPDATE_INTERFACE", pack=False, align=None),
    "BUS_SPECIFIC_RESET_FLAGS": SimUnion({"u": SimTypeRef("_u_e__Struct", SimStruct), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "BUTTONPARTS": SimTypeInt(signed=False, label="BUTTONPARTS"),
    "BUTTON_IMAGELIST": SimStruct(OrderedDict((("himl", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("margin", SimTypeRef("RECT", SimStruct)), ("uAlign", SimTypeInt(signed=False, label="BUTTON_IMAGELIST_ALIGN")),)), name="BUTTON_IMAGELIST", pack=False, align=None),
    "BUTTON_IMAGELIST_ALIGN": SimTypeInt(signed=False, label="BUTTON_IMAGELIST_ALIGN"),
    "BUTTON_SPLITINFO": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="UInt32")), ("himlGlyph", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uSplitStyle", SimTypeInt(signed=False, label="UInt32")), ("size", SimTypeRef("SIZE", SimStruct)),)), name="BUTTON_SPLITINFO", pack=False, align=None),
    "BYTE_BLOB": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("abData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BYTE_BLOB", pack=False, align=None),
    "BYTE_SIZEDARR": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BYTE_SIZEDARR", pack=False, align=None),
    "BY_HANDLE_FILE_INFORMATION": SimStruct(OrderedDict((("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ftCreationTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastWriteTime", SimTypeRef("FILETIME", SimStruct)), ("dwVolumeSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeLow", SimTypeInt(signed=False, label="UInt32")), ("nNumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("nFileIndexHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileIndexLow", SimTypeInt(signed=False, label="UInt32")),)), name="BY_HANDLE_FILE_INFORMATION", pack=False, align=None),
    "BadSampleInfo": SimStruct(OrderedDict((("hrReason", SimTypeInt(signed=True, label="Int32")),)), name="BadSampleInfo", pack=False, align=None),
    "BaseValueSource": SimTypeInt(signed=False, label="BaseValueSource"),
    "BerElement": SimStruct(OrderedDict((("opaque", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="BerElement", pack=False, align=None),
    "BfEnTvRat_Attributes_CAE_TV": SimTypeInt(signed=False, label="BfEnTvRat_Attributes_CAE_TV"),
    "BfEnTvRat_Attributes_CAF_TV": SimTypeInt(signed=False, label="BfEnTvRat_Attributes_CAF_TV"),
    "BfEnTvRat_Attributes_MPAA": SimTypeInt(signed=False, label="BfEnTvRat_Attributes_MPAA"),
    "BfEnTvRat_Attributes_US_TV": SimTypeInt(signed=False, label="BfEnTvRat_Attributes_US_TV"),
    "BfEnTvRat_GenericAttributes": SimTypeInt(signed=False, label="BfEnTvRat_GenericAttributes"),
    "BinaryConvolutionCodeRate": SimTypeInt(signed=False, label="BinaryConvolutionCodeRate"),
    "BinaryParam": SimStruct(OrderedDict((("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypeShort(signed=True, label="Int16")),)), name="BinaryParam", pack=False, align=None),
    "Bitmap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "BitmapData": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Stride", SimTypeInt(signed=True, label="Int32")), ("PixelFormat", SimTypeInt(signed=True, label="Int32")), ("Scan0", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="BitmapData", pack=False, align=None),
    "BitmapDescription": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("AlphaMode", SimTypeInt(signed=False, label="DXGI_ALPHA_MODE")),)), name="BitmapDescription", pack=False, align=None),
    "BlackInformation": SimStruct(OrderedDict((("fBlackOnly", SimTypeInt(signed=True, label="Int32")), ("blackWeight", SimTypeFloat(size=32)),)), name="BlackInformation", pack=False, align=None),
    "BlackScreenDiagnosticsCalloutParam": SimTypeInt(signed=False, label="BlackScreenDiagnosticsCalloutParam"),
    "Blur": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="Blur", pack=False, align=None),
    "BlurParams": SimStruct(OrderedDict((("radius", SimTypeFloat(size=32)), ("expandEdge", SimTypeInt(signed=True, label="Int32")),)), name="BlurParams", pack=False, align=None),
    "BorderStyleConstants": SimTypeInt(signed=False, label="BorderStyleConstants"),
    "BranchOfficeJobData": SimStruct(OrderedDict((("eEventType", SimTypeInt(signed=False, label="EBranchOfficeJobEventType")), ("JobId", SimTypeInt(signed=False, label="UInt32")), ("JobInfo", SimUnion({"LogJobPrinted": SimTypeRef("BranchOfficeJobDataPrinted", SimStruct), "LogJobRendered": SimTypeRef("BranchOfficeJobDataRendered", SimStruct), "LogJobError": SimTypeRef("BranchOfficeJobDataError", SimStruct), "LogPipelineFailed": SimTypeRef("BranchOfficeJobDataPipelineFailed", SimStruct), "LogOfflineFileFull": SimTypeRef("BranchOfficeLogOfflineFileFull", SimStruct)}, name="<anon>", label="None")),)), name="BranchOfficeJobData", pack=False, align=None),
    "BranchOfficeJobDataContainer": SimStruct(OrderedDict((("cJobDataEntries", SimTypeInt(signed=False, label="UInt32")), ("JobData", SimTypePointer(SimTypeRef("BranchOfficeJobData", SimStruct), offset=0)),)), name="BranchOfficeJobDataContainer", pack=False, align=None),
    "BranchOfficeJobDataError": SimStruct(OrderedDict((("LastError", SimTypeInt(signed=False, label="UInt32")), ("pDocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TotalSize", SimTypeLongLong(signed=True, label="Int64")), ("PrintedSize", SimTypeLongLong(signed=True, label="Int64")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("PrintedPages", SimTypeInt(signed=False, label="UInt32")), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pJobError", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pErrorDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BranchOfficeJobDataError", pack=False, align=None),
    "BranchOfficeJobDataPipelineFailed": SimStruct(OrderedDict((("pDocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pExtraErrorInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BranchOfficeJobDataPipelineFailed", pack=False, align=None),
    "BranchOfficeJobDataPrinted": SimStruct(OrderedDict((("Status", SimTypeInt(signed=False, label="UInt32")), ("pDocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPortName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Size", SimTypeLongLong(signed=True, label="Int64")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")),)), name="BranchOfficeJobDataPrinted", pack=False, align=None),
    "BranchOfficeJobDataRendered": SimStruct(OrderedDict((("Size", SimTypeLongLong(signed=True, label="Int64")), ("ICMMethod", SimTypeInt(signed=False, label="UInt32")), ("Color", SimTypeShort(signed=True, label="Int16")), ("PrintQuality", SimTypeShort(signed=True, label="Int16")), ("YResolution", SimTypeShort(signed=True, label="Int16")), ("Copies", SimTypeShort(signed=True, label="Int16")), ("TTOption", SimTypeShort(signed=True, label="Int16")),)), name="BranchOfficeJobDataRendered", pack=False, align=None),
    "BranchOfficeLogOfflineFileFull": SimStruct(OrderedDict((("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="BranchOfficeLogOfflineFileFull", pack=False, align=None),
    "BrightnessContrast": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="BrightnessContrast", pack=False, align=None),
    "BrightnessContrastParams": SimStruct(OrderedDict((("brightnessLevel", SimTypeInt(signed=True, label="Int32")), ("contrastLevel", SimTypeInt(signed=True, label="Int32")),)), name="BrightnessContrastParams", pack=False, align=None),
    "BrowserNavConstants": SimTypeInt(signed=False, label="BrowserNavConstants"),
    "BrushType": SimTypeInt(signed=False, label="BrushType"),
    "BucketParameterIndex": SimTypeInt(signed=False, label="BucketParameterIndex"),
    "BucketParameters": SimStruct(OrderedDict((("fInited", SimTypeInt(signed=True, label="Int32")), ("pszEventTypeName", SimTypeArray(SimTypeChar(label="Char"), 255)), ("pszParams", SimTypeArray(SimTypeChar(label="Char"), 2550)),)), name="BucketParameters", pack=False, align=None),
    "BufferUnion": SimUnion({"Address": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "IndexAndOffset": SimTypeRef("IORING_REGISTERED_BUFFER", SimStruct)}, name="<anon>", label="None"),
    "BulletStyle": SimTypeInt(signed=False, label="BulletStyle"),
    "CABINETSTATE": SimStruct(OrderedDict((("cLength", SimTypeShort(signed=False, label="UInt16")), ("nVersion", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeInt(signed=True, label="Int32")), ("fMenuEnumFilter", SimTypeInt(signed=False, label="UInt32")),)), name="CABINETSTATE", pack=False, align=None),
    "CABINET_INFO_A": SimStruct(OrderedDict((("CabinetPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CabinetFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DiskName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SetId", SimTypeShort(signed=False, label="UInt16")), ("CabinetNumber", SimTypeShort(signed=False, label="UInt16")),)), name="CABINET_INFO_A", pack=False, align=None),
    "CABINET_INFO_W": SimStruct(OrderedDict((("CabinetPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CabinetFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DiskName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SetId", SimTypeShort(signed=False, label="UInt16")), ("CabinetNumber", SimTypeShort(signed=False, label="UInt16")),)), name="CABINET_INFO_W", pack=False, align=None),
    "CABINFOA": SimStruct(OrderedDict((("pszCab", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszInf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszSection", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szSrcPath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CABINFOA", pack=False, align=None),
    "CABINFOW": SimStruct(OrderedDict((("pszCab", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInf", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSection", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szSrcPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CABINFOW", pack=False, align=None),
    "CABOOL": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)),)), name="CABOOL", pack=False, align=None),
    "CABSTR": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CABSTR", pack=False, align=None),
    "CABSTRBLOB": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeRef("BSTRBLOB", SimStruct), offset=0)),)), name="CABSTRBLOB", pack=False, align=None),
    "CAC": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CAC", pack=False, align=None),
    "CACHE_ACCESS_CHECK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("GENERIC_MAPPING", SimStruct), offset=0), SimTypePointer(SimTypeRef("PRIVILEGE_SET", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSecurityDescriptor", "hClientToken", "dwDesiredAccess", "GenericMapping", "PrivilegeSet", "PrivilegeSetLength", "GrantedAccess", "AccessStatus"]), offset=0),
    "CACHE_CONFIG": SimTypeInt(signed=False, label="CACHE_CONFIG"),
    "CACHE_DESCRIPTOR": SimStruct(OrderedDict((("Level", SimTypeChar(label="Byte")), ("Associativity", SimTypeChar(label="Byte")), ("LineSize", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="PROCESSOR_CACHE_TYPE")),)), name="CACHE_DESCRIPTOR", pack=False, align=None),
    "CACHE_DESTROY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["cb", "lpb"]), offset=0),
    "CACHE_KEY_COMPARE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbKey1", "lpbKey1", "cbKey2", "lpbKey2"]), offset=0),
    "CACHE_KEY_HASH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpbKey", "cbKey"]), offset=0),
    "CACHE_MANAGER_CALLBACKS": SimStruct(OrderedDict((("AcquireForLazyWrite", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["Context", "Wait"]), offset=0)), ("ReleaseFromLazyWrite", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("AcquireForReadAhead", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["Context", "Wait"]), offset=0)), ("ReleaseFromReadAhead", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)),)), name="CACHE_MANAGER_CALLBACKS", pack=False, align=None),
    "CACHE_MANAGER_CALLBACKS_EX": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Functions", SimTypeRef("CACHE_MANAGER_CALLBACK_FUNCTIONS", SimStruct)),)), name="CACHE_MANAGER_CALLBACKS_EX", pack=False, align=None),
    "CACHE_MANAGER_CALLBACK_FUNCTIONS": SimStruct(OrderedDict((("AcquireForLazyWriteEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Context", "InFlags", "OutFlags"]), offset=0)), ("ReleaseFromLazyWrite", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("AcquireForReadAhead", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["Context", "Wait"]), offset=0)), ("ReleaseFromReadAhead", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)),)), name="CACHE_MANAGER_CALLBACK_FUNCTIONS", pack=False, align=None),
    "CACHE_OPERATOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("INTERNET_CACHE_ENTRY_INFOA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcei", "pcbcei", "pOpData"]), offset=0),
    "CACHE_READ_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cb", "lpb", "lpvContext"]), offset=0),
    "CACHE_RELATIONSHIP": SimStruct(OrderedDict((("Level", SimTypeChar(label="Byte")), ("Associativity", SimTypeChar(label="Byte")), ("LineSize", SimTypeShort(signed=False, label="UInt16")), ("CacheSize", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="PROCESSOR_CACHE_TYPE")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 18)), ("GroupCount", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"GroupMask": SimTypeRef("GROUP_AFFINITY", SimStruct), "GroupMasks": SimTypePointer(SimTypeRef("GROUP_AFFINITY", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CACHE_RELATIONSHIP", pack=False, align=None),
    "CACHE_UNINITIALIZE_EVENT": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("CACHE_UNINITIALIZE_EVENT", SimStruct), offset=0)), ("Event", SimTypeRef("KEVENT", SimStruct)),)), name="CACHE_UNINITIALIZE_EVENT", pack=False, align=None),
    "CACLIPDATA": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeRef("CLIPDATA", SimStruct), offset=0)),)), name="CACLIPDATA", pack=False, align=None),
    "CACLSID": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="CACLSID", pack=False, align=None),
    "CACY": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), offset=0)),)), name="CACY", pack=False, align=None),
    "CADATE": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeFloat(size=64), offset=0)),)), name="CADATE", pack=False, align=None),
    "CADBL": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeFloat(size=64), offset=0)),)), name="CADBL", pack=False, align=None),
    "CADWORD": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CADWORD", pack=False, align=None),
    "CAFILETIME": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)),)), name="CAFILETIME", pack=False, align=None),
    "CAFLT": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="CAFLT", pack=False, align=None),
    "CAH": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="CAH", pack=False, align=None),
    "CAI": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)),)), name="CAI", pack=False, align=None),
    "CAINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("CAType", SimTypeInt(signed=False, label="ENUM_CATYPES")), ("cCASignatureCerts", SimTypeInt(signed=False, label="UInt32")), ("cCAExchangeCerts", SimTypeInt(signed=False, label="UInt32")), ("cExitModules", SimTypeInt(signed=False, label="UInt32")), ("lPropIdMax", SimTypeInt(signed=True, label="Int32")), ("lRoleSeparationEnabled", SimTypeInt(signed=True, label="Int32")), ("cKRACertUsedCount", SimTypeInt(signed=False, label="UInt32")), ("cKRACertCount", SimTypeInt(signed=False, label="UInt32")), ("fAdvancedServer", SimTypeInt(signed=False, label="UInt32")),)), name="CAINFO", pack=False, align=None),
    "CAL": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="CAL", pack=False, align=None),
    "CALDATETIME": SimStruct(OrderedDict((("CalId", SimTypeInt(signed=False, label="UInt32")), ("Era", SimTypeInt(signed=False, label="UInt32")), ("Year", SimTypeInt(signed=False, label="UInt32")), ("Month", SimTypeInt(signed=False, label="UInt32")), ("Day", SimTypeInt(signed=False, label="UInt32")), ("DayOfWeek", SimTypeInt(signed=False, label="UInt32")), ("Hour", SimTypeInt(signed=False, label="UInt32")), ("Minute", SimTypeInt(signed=False, label="UInt32")), ("Second", SimTypeInt(signed=False, label="UInt32")), ("Tick", SimTypeInt(signed=False, label="UInt32")),)), name="CALDATETIME", pack=False, align=None),
    "CALDATETIME_DATEUNIT": SimTypeInt(signed=False, label="CALDATETIME_DATEUNIT"),
    "CALINFO_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "CALINFO_ENUMPROCEXA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "CALINFO_ENUMPROCEXEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "CALINFO_ENUMPROCEXW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "CALINFO_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "CALLCONV": SimTypeInt(signed=False, label="CALLCONV"),
    "CALLERRELEASE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="ITableData"), SimTypeBottom(label="IMAPITable")], SimTypeBottom(label="Void"), arg_names=["ulCallerData", "lpTblData", "lpVue"]), offset=0),
    "CALLFRAMEINFO": SimStruct(OrderedDict((("iMethod", SimTypeInt(signed=False, label="UInt32")), ("fHasInValues", SimTypeInt(signed=True, label="Int32")), ("fHasInOutValues", SimTypeInt(signed=True, label="Int32")), ("fHasOutValues", SimTypeInt(signed=True, label="Int32")), ("fDerivesFromIDispatch", SimTypeInt(signed=True, label="Int32")), ("cInInterfacesMax", SimTypeInt(signed=True, label="Int32")), ("cInOutInterfacesMax", SimTypeInt(signed=True, label="Int32")), ("cOutInterfacesMax", SimTypeInt(signed=True, label="Int32")), ("cTopLevelInInterfaces", SimTypeInt(signed=True, label="Int32")), ("iid", SimTypeBottom(label="Guid")), ("cMethod", SimTypeInt(signed=False, label="UInt32")), ("cParams", SimTypeInt(signed=False, label="UInt32")),)), name="CALLFRAMEINFO", pack=False, align=None),
    "CALLFRAMEPARAMINFO": SimStruct(OrderedDict((("fIn", SimTypeChar(label="Byte")), ("fOut", SimTypeChar(label="Byte")), ("stackOffset", SimTypeInt(signed=False, label="UInt32")), ("cbParam", SimTypeInt(signed=False, label="UInt32")),)), name="CALLFRAMEPARAMINFO", pack=False, align=None),
    "CALLFRAME_COPY": SimTypeInt(signed=False, label="CALLFRAME_COPY"),
    "CALLFRAME_FREE": SimTypeInt(signed=False, label="CALLFRAME_FREE"),
    "CALLFRAME_MARSHALCONTEXT": SimStruct(OrderedDict((("fIn", SimTypeChar(label="Byte")), ("dwDestContext", SimTypeInt(signed=False, label="UInt32")), ("pvDestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("punkReserved", SimTypeBottom(label="IUnknown")), ("guidTransferSyntax", SimTypeBottom(label="Guid")),)), name="CALLFRAME_MARSHALCONTEXT", pack=False, align=None),
    "CALLFRAME_NULL": SimTypeInt(signed=False, label="CALLFRAME_NULL"),
    "CALLFRAME_WALK": SimTypeInt(signed=False, label="CALLFRAME_WALK"),
    "CALLHUB_EVENT": SimTypeInt(signed=False, label="CALLHUB_EVENT"),
    "CALLHUB_STATE": SimTypeInt(signed=False, label="CALLHUB_STATE"),
    "CALLINFOCHANGE_CAUSE": SimTypeInt(signed=False, label="CALLINFOCHANGE_CAUSE"),
    "CALLINFO_BUFFER": SimTypeInt(signed=False, label="CALLINFO_BUFFER"),
    "CALLINFO_LONG": SimTypeInt(signed=False, label="CALLINFO_LONG"),
    "CALLINFO_STRING": SimTypeInt(signed=False, label="CALLINFO_STRING"),
    "CALLTYPE": SimTypeInt(signed=False, label="CALLTYPE"),
    "CALL_MEDIA_EVENT": SimTypeInt(signed=False, label="CALL_MEDIA_EVENT"),
    "CALL_MEDIA_EVENT_CAUSE": SimTypeInt(signed=False, label="CALL_MEDIA_EVENT_CAUSE"),
    "CALL_NOTIFICATION_EVENT": SimTypeInt(signed=False, label="CALL_NOTIFICATION_EVENT"),
    "CALL_PRIVILEGE": SimTypeInt(signed=False, label="CALL_PRIVILEGE"),
    "CALL_STATE": SimTypeInt(signed=False, label="CALL_STATE"),
    "CALL_STATE_EVENT_CAUSE": SimTypeInt(signed=False, label="CALL_STATE_EVENT_CAUSE"),
    "CALPOLESTR": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CALPOLESTR", pack=False, align=None),
    "CALPSTR": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="CALPSTR", pack=False, align=None),
    "CALPWSTR": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CALPWSTR", pack=False, align=None),
    "CANDIDATEFORM": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("ptCurrentPos", SimTypeRef("POINT", SimStruct)), ("rcArea", SimTypeRef("RECT", SimStruct)),)), name="CANDIDATEFORM", pack=False, align=None),
    "CANDIDATEINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("dwPrivateSize", SimTypeInt(signed=False, label="UInt32")), ("dwPrivateOffset", SimTypeInt(signed=False, label="UInt32")),)), name="CANDIDATEINFO", pack=False, align=None),
    "CANDIDATELIST": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("dwSelection", SimTypeInt(signed=False, label="UInt32")), ("dwPageStart", SimTypeInt(signed=False, label="UInt32")), ("dwPageSize", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CANDIDATELIST", pack=False, align=None),
    "CAPCONTROLCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "nState"]), offset=0),
    "CAPDRIVERCAPS": SimStruct(OrderedDict((("wDeviceIndex", SimTypeInt(signed=False, label="UInt32")), ("fHasOverlay", SimTypeInt(signed=True, label="Int32")), ("fHasDlgVideoSource", SimTypeInt(signed=True, label="Int32")), ("fHasDlgVideoFormat", SimTypeInt(signed=True, label="Int32")), ("fHasDlgVideoDisplay", SimTypeInt(signed=True, label="Int32")), ("fCaptureInitialized", SimTypeInt(signed=True, label="Int32")), ("fDriverSuppliesPalettes", SimTypeInt(signed=True, label="Int32")), ("hVideoIn", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hVideoOut", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hVideoExtIn", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hVideoExtOut", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CAPDRIVERCAPS", pack=False, align=None),
    "CAPERRORCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "nID", "lpsz"]), offset=0),
    "CAPERRORCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "nID", "lpsz"]), offset=0),
    "CAPINFOCHUNK": SimStruct(OrderedDict((("fccInfoID", SimTypeInt(signed=False, label="UInt32")), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=True, label="Int32")),)), name="CAPINFOCHUNK", pack=False, align=None),
    "CAPROPVARIANT": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)),)), name="CAPROPVARIANT", pack=False, align=None),
    "CAPSTATUS": SimStruct(OrderedDict((("uiImageWidth", SimTypeInt(signed=False, label="UInt32")), ("uiImageHeight", SimTypeInt(signed=False, label="UInt32")), ("fLiveWindow", SimTypeInt(signed=True, label="Int32")), ("fOverlayWindow", SimTypeInt(signed=True, label="Int32")), ("fScale", SimTypeInt(signed=True, label="Int32")), ("ptScroll", SimTypeRef("POINT", SimStruct)), ("fUsingDefaultPalette", SimTypeInt(signed=True, label="Int32")), ("fAudioHardware", SimTypeInt(signed=True, label="Int32")), ("fCapFileExists", SimTypeInt(signed=True, label="Int32")), ("dwCurrentVideoFrame", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentVideoFramesDropped", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentWaveSamples", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentTimeElapsedMS", SimTypeInt(signed=False, label="UInt32")), ("hPalCurrent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fCapturingNow", SimTypeInt(signed=True, label="Int32")), ("dwReturn", SimTypeInt(signed=False, label="UInt32")), ("wNumVideoAllocated", SimTypeInt(signed=False, label="UInt32")), ("wNumAudioAllocated", SimTypeInt(signed=False, label="UInt32")),)), name="CAPSTATUS", pack=False, align=None),
    "CAPSTATUSCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "nID", "lpsz"]), offset=0),
    "CAPSTATUSCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "nID", "lpsz"]), offset=0),
    "CAPTIONSTATES": SimTypeInt(signed=False, label="CAPTIONSTATES"),
    "CAPTUREPARMS": SimStruct(OrderedDict((("dwRequestMicroSecPerFrame", SimTypeInt(signed=False, label="UInt32")), ("fMakeUserHitOKToCapture", SimTypeInt(signed=True, label="Int32")), ("wPercentDropForError", SimTypeInt(signed=False, label="UInt32")), ("fYield", SimTypeInt(signed=True, label="Int32")), ("dwIndexSize", SimTypeInt(signed=False, label="UInt32")), ("wChunkGranularity", SimTypeInt(signed=False, label="UInt32")), ("fUsingDOSMemory", SimTypeInt(signed=True, label="Int32")), ("wNumVideoRequested", SimTypeInt(signed=False, label="UInt32")), ("fCaptureAudio", SimTypeInt(signed=True, label="Int32")), ("wNumAudioRequested", SimTypeInt(signed=False, label="UInt32")), ("vKeyAbort", SimTypeInt(signed=False, label="UInt32")), ("fAbortLeftMouse", SimTypeInt(signed=True, label="Int32")), ("fAbortRightMouse", SimTypeInt(signed=True, label="Int32")), ("fLimitEnabled", SimTypeInt(signed=True, label="Int32")), ("wTimeLimit", SimTypeInt(signed=False, label="UInt32")), ("fMCIControl", SimTypeInt(signed=True, label="Int32")), ("fStepMCIDevice", SimTypeInt(signed=True, label="Int32")), ("dwMCIStartTime", SimTypeInt(signed=False, label="UInt32")), ("dwMCIStopTime", SimTypeInt(signed=False, label="UInt32")), ("fStepCaptureAt2x", SimTypeInt(signed=True, label="Int32")), ("wStepCaptureAverageFrames", SimTypeInt(signed=False, label="UInt32")), ("dwAudioBufferSize", SimTypeInt(signed=False, label="UInt32")), ("fDisableWriteCache", SimTypeInt(signed=True, label="Int32")), ("AVStreamMaster", SimTypeInt(signed=False, label="UInt32")),)), name="CAPTUREPARMS", pack=False, align=None),
    "CAPTURE_MEMORY_ALLOCATION_FLAGS": SimTypeInt(signed=False, label="CAPTURE_MEMORY_ALLOCATION_FLAGS"),
    "CAPTURE_STREAMTIME": SimStruct(OrderedDict((("StreamTime", SimTypeLongLong(signed=True, label="Int64")),)), name="CAPTURE_STREAMTIME", pack=False, align=None),
    "CAPVIDEOCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("VIDEOHDR", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "lpVHdr"]), offset=0),
    "CAPWAVECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WAVEHDR", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "lpWHdr"]), offset=0),
    "CAPYIELDCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd"]), offset=0),
    "CARET_DIRECTION": SimTypeInt(signed=False, label="CARET_DIRECTION"),
    "CARET_FLAGS": SimTypeInt(signed=False, label="CARET_FLAGS"),
    "CARET_INFO": SimUnion({"hbitmap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "caretFlags": SimTypeInt(signed=False, label="CARET_FLAGS")}, name="<anon>", label="None"),
    "CASCADE_WINDOWS_HOW": SimTypeInt(signed=False, label="CASCADE_WINDOWS_HOW"),
    "CASCODE": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="CASCODE", pack=False, align=None),
    "CASE_REQUIREMENT": SimTypeInt(signed=False, label="CASE_REQUIREMENT"),
    "CASetupProperty": SimTypeInt(signed=False, label="CASetupProperty"),
    "CATALOG_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("wszCatalogFile", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CATALOG_INFO", pack=False, align=None),
    "CATEGORIZATION": SimStruct(OrderedDict((("ulCatType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"cClusters": SimTypeInt(signed=False, label="UInt32"), "bucket": SimTypeRef("BUCKETCATEGORIZE", SimStruct), "range": SimTypeRef("RANGECATEGORIZE", SimStruct)}, name="<anon>", label="None")), ("csColumns", SimTypeRef("COLUMNSET", SimStruct)),)), name="CATEGORIZATION", pack=False, align=None),
    "CATEGORIZATIONSET": SimStruct(OrderedDict((("cCat", SimTypeInt(signed=False, label="UInt32")), ("aCat", SimTypePointer(SimTypeRef("CATEGORIZATION", SimStruct), offset=0)),)), name="CATEGORIZATIONSET", pack=False, align=None),
    "CATEGORYINFO": SimStruct(OrderedDict((("catid", SimTypeBottom(label="Guid")), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="CATEGORYINFO", pack=False, align=None),
    "CATEGORYINFO_FLAGS": SimTypeInt(signed=False, label="CATEGORYINFO_FLAGS"),
    "CATEGORY_INFO": SimStruct(OrderedDict((("cif", SimTypeInt(signed=False, label="CATEGORYINFO_FLAGS")), ("wszName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CATEGORY_INFO", pack=False, align=None),
    "CATSORT_FLAGS": SimTypeInt(signed=False, label="CATSORT_FLAGS"),
    "CAT_MEMBERINFO": SimStruct(OrderedDict((("pwszSubjGuid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwCertVersion", SimTypeInt(signed=False, label="UInt32")),)), name="CAT_MEMBERINFO", pack=False, align=None),
    "CAT_MEMBERINFO2": SimStruct(OrderedDict((("SubjectGuid", SimTypeBottom(label="Guid")), ("dwCertVersion", SimTypeInt(signed=False, label="UInt32")),)), name="CAT_MEMBERINFO2", pack=False, align=None),
    "CAT_NAMEVALUE": SimStruct(OrderedDict((("pwszTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fdwFlags", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CAT_NAMEVALUE", pack=False, align=None),
    "CAUB": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CAUB", pack=False, align=None),
    "CAUH": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="CAUH", pack=False, align=None),
    "CAUI": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="CAUI", pack=False, align=None),
    "CAUL": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CAUL", pack=False, align=None),
    "CAUUID": SimStruct(OrderedDict((("cElems", SimTypeInt(signed=False, label="UInt32")), ("pElems", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="CAUUID", pack=False, align=None),
    "CBADMITRESULT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("POLICY_DECISION", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), arg_names=["LpmHandle", "RequestHandle", "ulPcmActionFlags", "LpmError", "PolicyDecisionsCount", "pPolicyDecisions"]), offset=0),
    "CBGETRSVPOBJECTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeRef("RsvpObjHdr", SimStruct), offset=0), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), arg_names=["LpmHandle", "RequestHandle", "LpmError", "RsvpObjectsCount", "ppRsvpObjects"]), offset=0),
    "CBTACTIVATESTRUCT": SimStruct(OrderedDict((("fMouse", SimTypeInt(signed=True, label="Int32")), ("hWndActive", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CBTACTIVATESTRUCT", pack=False, align=None),
    "CBT_CREATEWNDA": SimStruct(OrderedDict((("lpcs", SimTypePointer(SimTypeRef("CREATESTRUCTA", SimStruct), offset=0)), ("hwndInsertAfter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CBT_CREATEWNDA", pack=False, align=None),
    "CBT_CREATEWNDW": SimStruct(OrderedDict((("lpcs", SimTypePointer(SimTypeRef("CREATESTRUCTW", SimStruct), offset=0)), ("hwndInsertAfter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CBT_CREATEWNDW", pack=False, align=None),
    "CCAB": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("cbFolderThresh", SimTypeInt(signed=False, label="UInt32")), ("cbReserveCFHeader", SimTypeInt(signed=False, label="UInt32")), ("cbReserveCFFolder", SimTypeInt(signed=False, label="UInt32")), ("cbReserveCFData", SimTypeInt(signed=False, label="UInt32")), ("iCab", SimTypeInt(signed=True, label="Int32")), ("iDisk", SimTypeInt(signed=True, label="Int32")), ("fFailOnIncompressible", SimTypeInt(signed=True, label="Int32")), ("setID", SimTypeShort(signed=False, label="UInt16")), ("szDisk", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("szCab", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("szCabPath", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="CCAB", pack=False, align=None),
    "CCINFOA": SimStruct(OrderedDict((("szClass", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("flOptions", SimTypeInt(signed=False, label="UInt32")), ("szDesc", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("cxDefault", SimTypeInt(signed=False, label="UInt32")), ("cyDefault", SimTypeInt(signed=False, label="UInt32")), ("flStyleDefault", SimTypeInt(signed=False, label="UInt32")), ("flExtStyleDefault", SimTypeInt(signed=False, label="UInt32")), ("flCtrlTypeMask", SimTypeInt(signed=False, label="UInt32")), ("szTextDefault", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("cStyleFlags", SimTypeInt(signed=True, label="Int32")), ("aStyleFlags", SimTypePointer(SimTypeRef("CCSTYLEFLAGA", SimStruct), offset=0)), ("lpfnStyle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CCSTYLEA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pccs"]), offset=0)), ("lpfnSizeToText", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["flStyle", "flExtStyle", "hfont", "pszText"]), offset=0)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="CCINFOA", pack=False, align=None),
    "CCINFOW": SimStruct(OrderedDict((("szClass", SimTypeArray(SimTypeChar(label="Char"), 32)), ("flOptions", SimTypeInt(signed=False, label="UInt32")), ("szDesc", SimTypeArray(SimTypeChar(label="Char"), 32)), ("cxDefault", SimTypeInt(signed=False, label="UInt32")), ("cyDefault", SimTypeInt(signed=False, label="UInt32")), ("flStyleDefault", SimTypeInt(signed=False, label="UInt32")), ("flExtStyleDefault", SimTypeInt(signed=False, label="UInt32")), ("flCtrlTypeMask", SimTypeInt(signed=False, label="UInt32")), ("cStyleFlags", SimTypeInt(signed=True, label="Int32")), ("aStyleFlags", SimTypePointer(SimTypeRef("CCSTYLEFLAGW", SimStruct), offset=0)), ("szTextDefault", SimTypeArray(SimTypeChar(label="Char"), 256)), ("lpfnStyle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CCSTYLEW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pccs"]), offset=0)), ("lpfnSizeToText", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["flStyle", "flExtStyle", "hfont", "pszText"]), offset=0)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="CCINFOW", pack=False, align=None),
    "CCM_COMMANDID_MASK_CONSTANTS": SimTypeInt(signed=False, label="CCM_COMMANDID_MASK_CONSTANTS"),
    "CCM_INSERTIONALLOWED": SimTypeInt(signed=False, label="CCM_INSERTIONALLOWED"),
    "CCM_INSERTIONPOINTID": SimTypeInt(signed=False, label="CCM_INSERTIONPOINTID"),
    "CCM_SPECIAL": SimTypeInt(signed=False, label="CCM_SPECIAL"),
    "CCSTYLEA": SimStruct(OrderedDict((("flStyle", SimTypeInt(signed=False, label="UInt32")), ("flExtStyle", SimTypeInt(signed=False, label="UInt32")), ("szText", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("lgid", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")),)), name="CCSTYLEA", pack=False, align=None),
    "CCSTYLEFLAGA": SimStruct(OrderedDict((("flStyle", SimTypeInt(signed=False, label="UInt32")), ("flStyleMask", SimTypeInt(signed=False, label="UInt32")), ("pszStyle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CCSTYLEFLAGA", pack=False, align=None),
    "CCSTYLEFLAGW": SimStruct(OrderedDict((("flStyle", SimTypeInt(signed=False, label="UInt32")), ("flStyleMask", SimTypeInt(signed=False, label="UInt32")), ("pszStyle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CCSTYLEFLAGW", pack=False, align=None),
    "CCSTYLEW": SimStruct(OrderedDict((("flStyle", SimTypeInt(signed=False, label="UInt32")), ("flExtStyle", SimTypeInt(signed=False, label="UInt32")), ("szText", SimTypeArray(SimTypeChar(label="Char"), 256)), ("lgid", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")),)), name="CCSTYLEW", pack=False, align=None),
    "CC_ASYNC_READ_CONTEXT": SimStruct(OrderedDict((("CompletionRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Context"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Mdl", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)), ("RequestorMode", SimTypeChar(label="SByte")), ("NestingLevel", SimTypeInt(signed=False, label="UInt32")),)), name="CC_ASYNC_READ_CONTEXT", pack=False, align=None),
    "CC_BYTE_PAIR": SimStruct(OrderedDict((("Decoded", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="CC_BYTE_PAIR", pack=False, align=None),
    "CC_ERROR_CALLBACK_CONTEXT": SimStruct(OrderedDict((("NodeByteSize", SimTypeShort(signed=True, label="Int16")), ("ErrorCode", SimTypeInt(signed=True, label="Int32")),)), name="CC_ERROR_CALLBACK_CONTEXT", pack=False, align=None),
    "CC_FILE_SIZES": SimStruct(OrderedDict((("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("ValidDataLength", SimTypeLongLong(signed=True, label="Int64")),)), name="CC_FILE_SIZES", pack=False, align=None),
    "CC_HW_FIELD": SimStruct(OrderedDict((("ScanlinesRequested", SimTypeRef("VBICODECFILTERING_SCANLINES", SimStruct)), ("fieldFlags", SimTypeInt(signed=False, label="UInt32")), ("PictureNumber", SimTypeLongLong(signed=True, label="Int64")), ("Lines", SimTypeArray(SimTypeRef("CC_BYTE_PAIR", SimStruct), 12)),)), name="CC_HW_FIELD", pack=False, align=None),
    "CDBURNINGEXTENSIONRET": SimTypeInt(signed=False, label="CDBURNINGEXTENSIONRET"),
    "CDCONTROLSTATEF": SimTypeInt(signed=False, label="CDCONTROLSTATEF"),
    "CDDDXGK_REDIRBITMAPPRESENTINFO": SimStruct(OrderedDict((("NumDirtyRects", SimTypeInt(signed=False, label="UInt32")), ("DirtyRect", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("NumContexts", SimTypeInt(signed=False, label="UInt32")), ("hContext", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 65)), ("bDoNotSynchronizeWithDxContent", SimTypeChar(label="Byte")),)), name="CDDDXGK_REDIRBITMAPPRESENTINFO", pack=False, align=None),
    "CDS_TYPE": SimTypeInt(signed=False, label="CDS_TYPE"),
    "CENTRAL_ACCESS_POLICY": SimStruct(OrderedDict((("CAPID", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Description", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ChangeId", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CAPECount", SimTypeInt(signed=False, label="UInt32")), ("CAPEs", SimTypePointer(SimTypePointer(SimTypeRef("CENTRAL_ACCESS_POLICY_ENTRY", SimStruct), offset=0), offset=0)),)), name="CENTRAL_ACCESS_POLICY", pack=False, align=None),
    "CENTRAL_ACCESS_POLICY_ENTRY": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Description", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ChangeId", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LengthAppliesTo", SimTypeInt(signed=False, label="UInt32")), ("AppliesTo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("LengthSD", SimTypeInt(signed=False, label="UInt32")), ("SD", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LengthStagedSD", SimTypeInt(signed=False, label="UInt32")), ("StagedSD", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="CENTRAL_ACCESS_POLICY_ENTRY", pack=False, align=None),
    "CEPSetupProperty": SimTypeInt(signed=False, label="CEPSetupProperty"),
    "CERTADMIN_GET_ROLES_FLAGS": SimTypeInt(signed=False, label="CERTADMIN_GET_ROLES_FLAGS"),
    "CERTENROLL_OBJECTID": SimTypeInt(signed=False, label="CERTENROLL_OBJECTID"),
    "CERTENROLL_PROPERTYID": SimTypeInt(signed=False, label="CERTENROLL_PROPERTYID"),
    "CERTIFICATE_CHAIN_BLOB": SimStruct(OrderedDict((("certCount", SimTypeInt(signed=False, label="UInt32")), ("rawCertificates", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CERTIFICATE_CHAIN_BLOB", pack=False, align=None),
    "CERTTRANSBLOB": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("pb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CERTTRANSBLOB", pack=False, align=None),
    "CERTVIEWRESTRICTION": SimStruct(OrderedDict((("ColumnIndex", SimTypeInt(signed=False, label="UInt32")), ("SeekOperator", SimTypeInt(signed=True, label="Int32")), ("SortOrder", SimTypeInt(signed=True, label="Int32")), ("pbValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbValue", SimTypeInt(signed=False, label="UInt32")),)), name="CERTVIEWRESTRICTION", pack=False, align=None),
    "CERT_ACCESS_DESCRIPTION": SimStruct(OrderedDict((("pszAccessMethod", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("AccessLocation", SimTypeRef("CERT_ALT_NAME_ENTRY", SimStruct)),)), name="CERT_ACCESS_DESCRIPTION", pack=False, align=None),
    "CERT_ALT_NAME": SimTypeInt(signed=False, label="CERT_ALT_NAME"),
    "CERT_ALT_NAME_ENTRY": SimStruct(OrderedDict((("dwAltNameChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pOtherName": SimTypePointer(SimTypeRef("CERT_OTHER_NAME", SimStruct), offset=0), "pwszRfc822Name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszDNSName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "DirectoryName": SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), "pwszURL": SimTypePointer(SimTypeChar(label="Char"), offset=0), "IPAddress": SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), "pszRegisteredID": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="CERT_ALT_NAME_ENTRY", pack=False, align=None),
    "CERT_ALT_NAME_INFO": SimStruct(OrderedDict((("cAltEntry", SimTypeInt(signed=False, label="UInt32")), ("rgAltEntry", SimTypePointer(SimTypeRef("CERT_ALT_NAME_ENTRY", SimStruct), offset=0)),)), name="CERT_ALT_NAME_INFO", pack=False, align=None),
    "CERT_AUTHORITY_INFO_ACCESS": SimStruct(OrderedDict((("cAccDescr", SimTypeInt(signed=False, label="UInt32")), ("rgAccDescr", SimTypePointer(SimTypeRef("CERT_ACCESS_DESCRIPTION", SimStruct), offset=0)),)), name="CERT_AUTHORITY_INFO_ACCESS", pack=False, align=None),
    "CERT_AUTHORITY_KEY_ID2_INFO": SimStruct(OrderedDict((("KeyId", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("AuthorityCertIssuer", SimTypeRef("CERT_ALT_NAME_INFO", SimStruct)), ("AuthorityCertSerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_AUTHORITY_KEY_ID2_INFO", pack=False, align=None),
    "CERT_AUTHORITY_KEY_ID_INFO": SimStruct(OrderedDict((("KeyId", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("CertIssuer", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("CertSerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_AUTHORITY_KEY_ID_INFO", pack=False, align=None),
    "CERT_BASIC_CONSTRAINTS2_INFO": SimStruct(OrderedDict((("fCA", SimTypeInt(signed=True, label="Int32")), ("fPathLenConstraint", SimTypeInt(signed=True, label="Int32")), ("dwPathLenConstraint", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_BASIC_CONSTRAINTS2_INFO", pack=False, align=None),
    "CERT_BASIC_CONSTRAINTS_INFO": SimStruct(OrderedDict((("SubjectType", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("fPathLenConstraint", SimTypeInt(signed=True, label="Int32")), ("dwPathLenConstraint", SimTypeInt(signed=False, label="UInt32")), ("cSubtreesConstraint", SimTypeInt(signed=False, label="UInt32")), ("rgSubtreesConstraint", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CERT_BASIC_CONSTRAINTS_INFO", pack=False, align=None),
    "CERT_BIOMETRIC_DATA": SimStruct(OrderedDict((("dwTypeOfBiometricDataChoice", SimTypeInt(signed=False, label="CERT_BIOMETRIC_DATA_TYPE")), ("Anonymous", SimUnion({"dwPredefined": SimTypeInt(signed=False, label="UInt32"), "pszObjId": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("HashedUrl", SimTypeRef("CERT_HASHED_URL", SimStruct)),)), name="CERT_BIOMETRIC_DATA", pack=False, align=None),
    "CERT_BIOMETRIC_DATA_TYPE": SimTypeInt(signed=False, label="CERT_BIOMETRIC_DATA_TYPE"),
    "CERT_BIOMETRIC_EXT_INFO": SimStruct(OrderedDict((("cBiometricData", SimTypeInt(signed=False, label="UInt32")), ("rgBiometricData", SimTypePointer(SimTypeRef("CERT_BIOMETRIC_DATA", SimStruct), offset=0)),)), name="CERT_BIOMETRIC_EXT_INFO", pack=False, align=None),
    "CERT_CHAIN": SimStruct(OrderedDict((("cCerts", SimTypeInt(signed=False, label="UInt32")), ("certs", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("keyLocatorInfo", SimTypeRef("CRYPT_KEY_PROV_INFO", SimStruct)),)), name="CERT_CHAIN", pack=False, align=None),
    "CERT_CHAIN_CONTEXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("TrustStatus", SimTypeRef("CERT_TRUST_STATUS", SimStruct)), ("cChain", SimTypeInt(signed=False, label="UInt32")), ("rgpChain", SimTypePointer(SimTypePointer(SimTypeRef("CERT_SIMPLE_CHAIN", SimStruct), offset=0), offset=0)), ("cLowerQualityChainContext", SimTypeInt(signed=False, label="UInt32")), ("rgpLowerQualityChainContext", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0), offset=0)), ("fHasRevocationFreshnessTime", SimTypeInt(signed=True, label="Int32")), ("dwRevocationFreshnessTime", SimTypeInt(signed=False, label="UInt32")), ("dwCreateFlags", SimTypeInt(signed=False, label="UInt32")), ("ChainId", SimTypeBottom(label="Guid")),)), name="CERT_CHAIN_CONTEXT", pack=False, align=None),
    "CERT_CHAIN_ELEMENT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("TrustStatus", SimTypeRef("CERT_TRUST_STATUS", SimStruct)), ("pRevocationInfo", SimTypePointer(SimTypeRef("CERT_REVOCATION_INFO", SimStruct), offset=0)), ("pIssuanceUsage", SimTypePointer(SimTypeRef("CTL_USAGE", SimStruct), offset=0)), ("pApplicationUsage", SimTypePointer(SimTypeRef("CTL_USAGE", SimStruct), offset=0)), ("pwszExtendedErrorInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_CHAIN_ELEMENT", pack=False, align=None),
    "CERT_CHAIN_ENGINE_CONFIG": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hRestrictedRoot", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hRestrictedTrust", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hRestrictedOther", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cAdditionalStore", SimTypeInt(signed=False, label="UInt32")), ("rghAdditionalStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwUrlRetrievalTimeout", SimTypeInt(signed=False, label="UInt32")), ("MaximumCachedCertificates", SimTypeInt(signed=False, label="UInt32")), ("CycleDetectionModulus", SimTypeInt(signed=False, label="UInt32")), ("hExclusiveRoot", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hExclusiveTrustedPeople", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwExclusiveFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_CHAIN_ENGINE_CONFIG", pack=False, align=None),
    "CERT_CHAIN_FIND_BY_ISSUER_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszUsageIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("dwAcquirePrivateKeyFlags", SimTypeInt(signed=False, label="UInt32")), ("cIssuer", SimTypeInt(signed=False, label="UInt32")), ("rgIssuer", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("pfnFindCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCert", "pvFindArg"]), offset=0)), ("pvFindArg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_CHAIN_FIND_BY_ISSUER_PARA", pack=False, align=None),
    "CERT_CHAIN_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("RequestedUsage", SimTypeRef("CERT_USAGE_MATCH", SimStruct)),)), name="CERT_CHAIN_PARA", pack=False, align=None),
    "CERT_CHAIN_POLICY_FLAGS": SimTypeInt(signed=False, label="CERT_CHAIN_POLICY_FLAGS"),
    "CERT_CHAIN_POLICY_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CERT_CHAIN_POLICY_FLAGS")), ("pvExtraPolicyPara", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_CHAIN_POLICY_PARA", pack=False, align=None),
    "CERT_CHAIN_POLICY_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("lChainIndex", SimTypeInt(signed=True, label="Int32")), ("lElementIndex", SimTypeInt(signed=True, label="Int32")), ("pvExtraPolicyStatus", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_CHAIN_POLICY_STATUS", pack=False, align=None),
    "CERT_CONTEXT": SimStruct(OrderedDict((("dwCertEncodingType", SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE")), ("pbCertEncoded", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCertEncoded", SimTypeInt(signed=False, label="UInt32")), ("pCertInfo", SimTypePointer(SimTypeRef("CERT_INFO", SimStruct), offset=0)), ("hCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_CONTEXT", pack=False, align=None),
    "CERT_CONTEXT_EX": SimStruct(OrderedDict((("CertContext", SimTypeRef("CERT_CONTEXT", SimStruct)), ("cbAllocated", SimTypeInt(signed=False, label="UInt32")), ("dwCertificateFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_CONTEXT_EX", pack=False, align=None),
    "CERT_CONTROL_STORE_FLAGS": SimTypeInt(signed=False, label="CERT_CONTROL_STORE_FLAGS"),
    "CERT_CREATE_CONTEXT_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)), ("pvFree", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfnSort", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbTotalEncoded", "cbRemainEncoded", "cEntry", "pvSort"]), offset=0)), ("pvSort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_CREATE_CONTEXT_PARA", pack=False, align=None),
    "CERT_CREATE_REQUEST_FLAGS": SimTypeInt(signed=False, label="CERT_CREATE_REQUEST_FLAGS"),
    "CERT_CREATE_SELFSIGN_FLAGS": SimTypeInt(signed=False, label="CERT_CREATE_SELFSIGN_FLAGS"),
    "CERT_CREDENTIAL_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rgbHashOfCert", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="CERT_CREDENTIAL_INFO", pack=False, align=None),
    "CERT_CRL_CONTEXT_PAIR": SimStruct(OrderedDict((("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("pCrlContext", SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0)),)), name="CERT_CRL_CONTEXT_PAIR", pack=False, align=None),
    "CERT_DELETE_ROW_FLAGS": SimTypeInt(signed=False, label="CERT_DELETE_ROW_FLAGS"),
    "CERT_DH_PARAMETERS": SimStruct(OrderedDict((("p", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("g", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_DH_PARAMETERS", pack=False, align=None),
    "CERT_DSS_PARAMETERS": SimStruct(OrderedDict((("p", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("q", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("g", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_DSS_PARAMETERS", pack=False, align=None),
    "CERT_ECC_SIGNATURE": SimStruct(OrderedDict((("r", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("s", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_ECC_SIGNATURE", pack=False, align=None),
    "CERT_EXIT_EVENT_MASK": SimTypeInt(signed=False, label="CERT_EXIT_EVENT_MASK"),
    "CERT_EXTENSION": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fCritical", SimTypeInt(signed=True, label="Int32")), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_EXTENSION", pack=False, align=None),
    "CERT_EXTENSIONS": SimStruct(OrderedDict((("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CERT_EXTENSIONS", pack=False, align=None),
    "CERT_FILTER_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cExtensionChecks", SimTypeInt(signed=False, label="UInt32")), ("arrayExtensionChecks", SimTypePointer(SimTypeRef("CERT_FILTER_EXTENSION_MATCH", SimStruct), offset=0)), ("dwCheckingFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_FILTER_DATA", pack=False, align=None),
    "CERT_FILTER_EXTENSION_MATCH": SimStruct(OrderedDict((("szExtensionOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwTestOperation", SimTypeInt(signed=False, label="UInt32")), ("pbTestData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbTestData", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_FILTER_EXTENSION_MATCH", pack=False, align=None),
    "CERT_FIND_CHAIN_IN_STORE_FLAGS": SimTypeInt(signed=False, label="CERT_FIND_CHAIN_IN_STORE_FLAGS"),
    "CERT_FIND_FLAGS": SimTypeInt(signed=False, label="CERT_FIND_FLAGS"),
    "CERT_FIND_TYPE": SimTypeInt(signed=False, label="CERT_FIND_TYPE"),
    "CERT_FORTEZZA_DATA_PROP": SimStruct(OrderedDict((("SerialNumber", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("CertIndex", SimTypeInt(signed=True, label="Int32")), ("CertLabel", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="CERT_FORTEZZA_DATA_PROP", pack=False, align=None),
    "CERT_GENERAL_SUBTREE": SimStruct(OrderedDict((("Base", SimTypeRef("CERT_ALT_NAME_ENTRY", SimStruct)), ("dwMinimum", SimTypeInt(signed=False, label="UInt32")), ("fMaximum", SimTypeInt(signed=True, label="Int32")), ("dwMaximum", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_GENERAL_SUBTREE", pack=False, align=None),
    "CERT_GET_CONFIG_FLAGS": SimTypeInt(signed=False, label="CERT_GET_CONFIG_FLAGS"),
    "CERT_HASHED_URL": SimStruct(OrderedDict((("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Hash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pwszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_HASHED_URL", pack=False, align=None),
    "CERT_ID": SimStruct(OrderedDict((("dwIdChoice", SimTypeInt(signed=False, label="CERT_ID_OPTION")), ("Anonymous", SimUnion({"IssuerSerialNumber": SimTypeRef("CERT_ISSUER_SERIAL_NUMBER", SimStruct), "KeyId": SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), "HashId": SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)}, name="<anon>", label="None")),)), name="CERT_ID", pack=False, align=None),
    "CERT_ID_OPTION": SimTypeInt(signed=False, label="CERT_ID_OPTION"),
    "CERT_IMPORT_FLAGS": SimTypeInt(signed=False, label="CERT_IMPORT_FLAGS"),
    "CERT_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("SerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SignatureAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Issuer", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("NotBefore", SimTypeRef("FILETIME", SimStruct)), ("NotAfter", SimTypeRef("FILETIME", SimStruct)), ("Subject", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SubjectPublicKeyInfo", SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct)), ("IssuerUniqueId", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("SubjectUniqueId", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), label="LPArray", offset=0)),)), name="CERT_INFO", pack=False, align=None),
    "CERT_ISSUER_SERIAL_NUMBER": SimStruct(OrderedDict((("Issuer", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_ISSUER_SERIAL_NUMBER", pack=False, align=None),
    "CERT_KEYGEN_REQUEST_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("SubjectPublicKeyInfo", SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct)), ("pwszChallengeString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_KEYGEN_REQUEST_INFO", pack=False, align=None),
    "CERT_KEY_ATTRIBUTES_INFO": SimStruct(OrderedDict((("KeyId", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("IntendedKeyUsage", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("pPrivateKeyUsagePeriod", SimTypePointer(SimTypeRef("CERT_PRIVATE_KEY_VALIDITY", SimStruct), offset=0)),)), name="CERT_KEY_ATTRIBUTES_INFO", pack=False, align=None),
    "CERT_KEY_CONTEXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hCryptProv": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hNCryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_KEY_CONTEXT", pack=False, align=None),
    "CERT_KEY_SPEC": SimTypeInt(signed=False, label="CERT_KEY_SPEC"),
    "CERT_KEY_USAGE_RESTRICTION_INFO": SimStruct(OrderedDict((("cCertPolicyId", SimTypeInt(signed=False, label="UInt32")), ("rgCertPolicyId", SimTypePointer(SimTypeRef("CERT_POLICY_ID", SimStruct), offset=0)), ("RestrictedKeyUsage", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)),)), name="CERT_KEY_USAGE_RESTRICTION_INFO", pack=False, align=None),
    "CERT_LDAP_STORE_OPENED_PARA": SimStruct(OrderedDict((("pvLdapSessionHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pwszLdapUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_LDAP_STORE_OPENED_PARA", pack=False, align=None),
    "CERT_LOGOTYPE_AUDIO": SimStruct(OrderedDict((("LogotypeDetails", SimTypeRef("CERT_LOGOTYPE_DETAILS", SimStruct)), ("pLogotypeAudioInfo", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_AUDIO_INFO", SimStruct), offset=0)),)), name="CERT_LOGOTYPE_AUDIO", pack=False, align=None),
    "CERT_LOGOTYPE_AUDIO_INFO": SimStruct(OrderedDict((("dwFileSize", SimTypeInt(signed=False, label="UInt32")), ("dwPlayTime", SimTypeInt(signed=False, label="UInt32")), ("dwChannels", SimTypeInt(signed=False, label="UInt32")), ("dwSampleRate", SimTypeInt(signed=False, label="UInt32")), ("pwszLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_LOGOTYPE_AUDIO_INFO", pack=False, align=None),
    "CERT_LOGOTYPE_CHOICE": SimTypeInt(signed=False, label="CERT_LOGOTYPE_CHOICE"),
    "CERT_LOGOTYPE_DATA": SimStruct(OrderedDict((("cLogotypeImage", SimTypeInt(signed=False, label="UInt32")), ("rgLogotypeImage", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_IMAGE", SimStruct), offset=0)), ("cLogotypeAudio", SimTypeInt(signed=False, label="UInt32")), ("rgLogotypeAudio", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_AUDIO", SimStruct), offset=0)),)), name="CERT_LOGOTYPE_DATA", pack=False, align=None),
    "CERT_LOGOTYPE_DETAILS": SimStruct(OrderedDict((("pwszMimeType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cHashedUrl", SimTypeInt(signed=False, label="UInt32")), ("rgHashedUrl", SimTypePointer(SimTypeRef("CERT_HASHED_URL", SimStruct), offset=0)),)), name="CERT_LOGOTYPE_DETAILS", pack=False, align=None),
    "CERT_LOGOTYPE_EXT_INFO": SimStruct(OrderedDict((("cCommunityLogo", SimTypeInt(signed=False, label="UInt32")), ("rgCommunityLogo", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_INFO", SimStruct), offset=0)), ("pIssuerLogo", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_INFO", SimStruct), offset=0)), ("pSubjectLogo", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_INFO", SimStruct), offset=0)), ("cOtherLogo", SimTypeInt(signed=False, label="UInt32")), ("rgOtherLogo", SimTypePointer(SimTypeRef("CERT_OTHER_LOGOTYPE_INFO", SimStruct), offset=0)),)), name="CERT_LOGOTYPE_EXT_INFO", pack=False, align=None),
    "CERT_LOGOTYPE_IMAGE": SimStruct(OrderedDict((("LogotypeDetails", SimTypeRef("CERT_LOGOTYPE_DETAILS", SimStruct)), ("pLogotypeImageInfo", SimTypePointer(SimTypeRef("CERT_LOGOTYPE_IMAGE_INFO", SimStruct), offset=0)),)), name="CERT_LOGOTYPE_IMAGE", pack=False, align=None),
    "CERT_LOGOTYPE_IMAGE_INFO": SimStruct(OrderedDict((("dwLogotypeImageInfoChoice", SimTypeInt(signed=False, label="CERT_LOGOTYPE_IMAGE_INFO_TYPE")), ("dwFileSize", SimTypeInt(signed=False, label="UInt32")), ("dwXSize", SimTypeInt(signed=False, label="UInt32")), ("dwYSize", SimTypeInt(signed=False, label="UInt32")), ("dwLogotypeImageResolutionChoice", SimTypeInt(signed=False, label="CERT_LOGOTYPE_CHOICE")), ("Anonymous", SimUnion({"dwNumBits": SimTypeInt(signed=False, label="UInt32"), "dwTableSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("pwszLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_LOGOTYPE_IMAGE_INFO", pack=False, align=None),
    "CERT_LOGOTYPE_IMAGE_INFO_TYPE": SimTypeInt(signed=False, label="CERT_LOGOTYPE_IMAGE_INFO_TYPE"),
    "CERT_LOGOTYPE_INFO": SimStruct(OrderedDict((("dwLogotypeInfoChoice", SimTypeInt(signed=False, label="CERT_LOGOTYPE_OPTION")), ("Anonymous", SimUnion({"pLogotypeDirectInfo": SimTypePointer(SimTypeRef("CERT_LOGOTYPE_DATA", SimStruct), offset=0), "pLogotypeIndirectInfo": SimTypePointer(SimTypeRef("CERT_LOGOTYPE_REFERENCE", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CERT_LOGOTYPE_INFO", pack=False, align=None),
    "CERT_LOGOTYPE_OPTION": SimTypeInt(signed=False, label="CERT_LOGOTYPE_OPTION"),
    "CERT_LOGOTYPE_REFERENCE": SimStruct(OrderedDict((("cHashedUrl", SimTypeInt(signed=False, label="UInt32")), ("rgHashedUrl", SimTypePointer(SimTypeRef("CERT_HASHED_URL", SimStruct), offset=0)),)), name="CERT_LOGOTYPE_REFERENCE", pack=False, align=None),
    "CERT_NAME_CONSTRAINTS_INFO": SimStruct(OrderedDict((("cPermittedSubtree", SimTypeInt(signed=False, label="UInt32")), ("rgPermittedSubtree", SimTypePointer(SimTypeRef("CERT_GENERAL_SUBTREE", SimStruct), offset=0)), ("cExcludedSubtree", SimTypeInt(signed=False, label="UInt32")), ("rgExcludedSubtree", SimTypePointer(SimTypeRef("CERT_GENERAL_SUBTREE", SimStruct), offset=0)),)), name="CERT_NAME_CONSTRAINTS_INFO", pack=False, align=None),
    "CERT_NAME_INFO": SimStruct(OrderedDict((("cRDN", SimTypeInt(signed=False, label="UInt32")), ("rgRDN", SimTypePointer(SimTypeRef("CERT_RDN", SimStruct), offset=0)),)), name="CERT_NAME_INFO", pack=False, align=None),
    "CERT_NAME_VALUE": SimStruct(OrderedDict((("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_NAME_VALUE", pack=False, align=None),
    "CERT_OPEN_STORE_FLAGS": SimTypeInt(signed=False, label="CERT_OPEN_STORE_FLAGS"),
    "CERT_OR_CRL_BLOB": SimStruct(OrderedDict((("dwChoice", SimTypeInt(signed=False, label="UInt32")), ("cbEncoded", SimTypeInt(signed=False, label="UInt32")), ("pbEncoded", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CERT_OR_CRL_BLOB", pack=False, align=None),
    "CERT_OR_CRL_BUNDLE": SimStruct(OrderedDict((("cItem", SimTypeInt(signed=False, label="UInt32")), ("rgItem", SimTypePointer(SimTypeRef("CERT_OR_CRL_BLOB", SimStruct), offset=0)),)), name="CERT_OR_CRL_BUNDLE", pack=False, align=None),
    "CERT_OTHER_LOGOTYPE_INFO": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("LogotypeInfo", SimTypeRef("CERT_LOGOTYPE_INFO", SimStruct)),)), name="CERT_OTHER_LOGOTYPE_INFO", pack=False, align=None),
    "CERT_OTHER_NAME": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_OTHER_NAME", pack=False, align=None),
    "CERT_PAIR": SimStruct(OrderedDict((("Forward", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("Reverse", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_PAIR", pack=False, align=None),
    "CERT_PHYSICAL_STORE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszOpenStoreProvider", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwOpenEncodingType", SimTypeInt(signed=False, label="UInt32")), ("dwOpenFlags", SimTypeInt(signed=False, label="UInt32")), ("OpenParameters", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPriority", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_PHYSICAL_STORE_INFO", pack=False, align=None),
    "CERT_POLICIES_INFO": SimStruct(OrderedDict((("cPolicyInfo", SimTypeInt(signed=False, label="UInt32")), ("rgPolicyInfo", SimTypePointer(SimTypeRef("CERT_POLICY_INFO", SimStruct), offset=0)),)), name="CERT_POLICIES_INFO", pack=False, align=None),
    "CERT_POLICY95_QUALIFIER1": SimStruct(OrderedDict((("pszPracticesReference", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszNoticeIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszNSINoticeIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cCPSURLs", SimTypeInt(signed=False, label="UInt32")), ("rgCPSURLs", SimTypePointer(SimTypeRef("CPS_URLS", SimStruct), offset=0)),)), name="CERT_POLICY95_QUALIFIER1", pack=False, align=None),
    "CERT_POLICY_CONSTRAINTS_INFO": SimStruct(OrderedDict((("fRequireExplicitPolicy", SimTypeInt(signed=True, label="Int32")), ("dwRequireExplicitPolicySkipCerts", SimTypeInt(signed=False, label="UInt32")), ("fInhibitPolicyMapping", SimTypeInt(signed=True, label="Int32")), ("dwInhibitPolicyMappingSkipCerts", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_POLICY_CONSTRAINTS_INFO", pack=False, align=None),
    "CERT_POLICY_ID": SimStruct(OrderedDict((("cCertPolicyElementId", SimTypeInt(signed=False, label="UInt32")), ("rgpszCertPolicyElementId", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="CERT_POLICY_ID", pack=False, align=None),
    "CERT_POLICY_INFO": SimStruct(OrderedDict((("pszPolicyIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cPolicyQualifier", SimTypeInt(signed=False, label="UInt32")), ("rgPolicyQualifier", SimTypePointer(SimTypeRef("CERT_POLICY_QUALIFIER_INFO", SimStruct), offset=0)),)), name="CERT_POLICY_INFO", pack=False, align=None),
    "CERT_POLICY_MAPPING": SimStruct(OrderedDict((("pszIssuerDomainPolicy", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszSubjectDomainPolicy", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CERT_POLICY_MAPPING", pack=False, align=None),
    "CERT_POLICY_MAPPINGS_INFO": SimStruct(OrderedDict((("cPolicyMapping", SimTypeInt(signed=False, label="UInt32")), ("rgPolicyMapping", SimTypePointer(SimTypeRef("CERT_POLICY_MAPPING", SimStruct), offset=0)),)), name="CERT_POLICY_MAPPINGS_INFO", pack=False, align=None),
    "CERT_POLICY_QUALIFIER_INFO": SimStruct(OrderedDict((("pszPolicyQualifierId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Qualifier", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_POLICY_QUALIFIER_INFO", pack=False, align=None),
    "CERT_POLICY_QUALIFIER_NOTICE_REFERENCE": SimStruct(OrderedDict((("pszOrganization", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cNoticeNumbers", SimTypeInt(signed=False, label="UInt32")), ("rgNoticeNumbers", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="CERT_POLICY_QUALIFIER_NOTICE_REFERENCE", pack=False, align=None),
    "CERT_POLICY_QUALIFIER_USER_NOTICE": SimStruct(OrderedDict((("pNoticeReference", SimTypePointer(SimTypeRef("CERT_POLICY_QUALIFIER_NOTICE_REFERENCE", SimStruct), offset=0)), ("pszDisplayText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_POLICY_QUALIFIER_USER_NOTICE", pack=False, align=None),
    "CERT_PRIVATE_KEY_VALIDITY": SimStruct(OrderedDict((("NotBefore", SimTypeRef("FILETIME", SimStruct)), ("NotAfter", SimTypeRef("FILETIME", SimStruct)),)), name="CERT_PRIVATE_KEY_VALIDITY", pack=False, align=None),
    "CERT_PROPERTY_TYPE": SimTypeInt(signed=False, label="CERT_PROPERTY_TYPE"),
    "CERT_PUBLIC_KEY_INFO": SimStruct(OrderedDict((("Algorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("PublicKey", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)),)), name="CERT_PUBLIC_KEY_INFO", pack=False, align=None),
    "CERT_QC_STATEMENT": SimStruct(OrderedDict((("pszStatementId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StatementInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_QC_STATEMENT", pack=False, align=None),
    "CERT_QC_STATEMENTS_EXT_INFO": SimStruct(OrderedDict((("cStatement", SimTypeInt(signed=False, label="UInt32")), ("rgStatement", SimTypePointer(SimTypeRef("CERT_QC_STATEMENT", SimStruct), offset=0)),)), name="CERT_QC_STATEMENTS_EXT_INFO", pack=False, align=None),
    "CERT_QUERY_CONTENT_TYPE": SimTypeInt(signed=False, label="CERT_QUERY_CONTENT_TYPE"),
    "CERT_QUERY_CONTENT_TYPE_FLAGS": SimTypeInt(signed=False, label="CERT_QUERY_CONTENT_TYPE_FLAGS"),
    "CERT_QUERY_ENCODING_TYPE": SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"),
    "CERT_QUERY_FORMAT_TYPE": SimTypeInt(signed=False, label="CERT_QUERY_FORMAT_TYPE"),
    "CERT_QUERY_FORMAT_TYPE_FLAGS": SimTypeInt(signed=False, label="CERT_QUERY_FORMAT_TYPE_FLAGS"),
    "CERT_QUERY_OBJECT_TYPE": SimTypeInt(signed=False, label="CERT_QUERY_OBJECT_TYPE"),
    "CERT_RDN": SimStruct(OrderedDict((("cRDNAttr", SimTypeInt(signed=False, label="UInt32")), ("rgRDNAttr", SimTypePointer(SimTypeRef("CERT_RDN_ATTR", SimStruct), offset=0)),)), name="CERT_RDN", pack=False, align=None),
    "CERT_RDN_ATTR": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CERT_RDN_ATTR", pack=False, align=None),
    "CERT_RDN_ATTR_VALUE_TYPE": SimTypeInt(signed=False, label="CERT_RDN_ATTR_VALUE_TYPE"),
    "CERT_REGISTRY_STORE_CLIENT_GPT_PARA": SimStruct(OrderedDict((("hKeyBase", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwszRegPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_REGISTRY_STORE_CLIENT_GPT_PARA", pack=False, align=None),
    "CERT_REGISTRY_STORE_ROAMING_PARA": SimStruct(OrderedDict((("hKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwszStoreDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_REGISTRY_STORE_ROAMING_PARA", pack=False, align=None),
    "CERT_REQUEST_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Subject", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SubjectPublicKeyInfo", SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct)), ("cAttribute", SimTypeInt(signed=False, label="UInt32")), ("rgAttribute", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)),)), name="CERT_REQUEST_INFO", pack=False, align=None),
    "CERT_REQUEST_OUT_TYPE": SimTypeInt(signed=False, label="CERT_REQUEST_OUT_TYPE"),
    "CERT_REVOCATION_CHAIN_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hChainEngine", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAdditionalStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwChainFlags", SimTypeInt(signed=False, label="UInt32")), ("dwUrlRetrievalTimeout", SimTypeInt(signed=False, label="UInt32")), ("pftCurrentTime", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("pftCacheResync", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("cbMaxUrlRetrievalByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_REVOCATION_CHAIN_PARA", pack=False, align=None),
    "CERT_REVOCATION_CRL_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pBaseCrlContext", SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0)), ("pDeltaCrlContext", SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0)), ("pCrlEntry", SimTypePointer(SimTypeRef("CRL_ENTRY", SimStruct), offset=0)), ("fDeltaCrlEntry", SimTypeInt(signed=True, label="Int32")),)), name="CERT_REVOCATION_CRL_INFO", pack=False, align=None),
    "CERT_REVOCATION_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRevocationResult", SimTypeInt(signed=False, label="UInt32")), ("pszRevocationOid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvOidSpecificInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("fHasFreshnessTime", SimTypeInt(signed=True, label="Int32")), ("dwFreshnessTime", SimTypeInt(signed=False, label="UInt32")), ("pCrlInfo", SimTypePointer(SimTypeRef("CERT_REVOCATION_CRL_INFO", SimStruct), offset=0)),)), name="CERT_REVOCATION_INFO", pack=False, align=None),
    "CERT_REVOCATION_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pIssuerCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("cCertStore", SimTypeInt(signed=False, label="UInt32")), ("rgCertStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("hCrlStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pftTimeToUse", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)),)), name="CERT_REVOCATION_PARA", pack=False, align=None),
    "CERT_REVOCATION_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwReason", SimTypeInt(signed=False, label="CERT_REVOCATION_STATUS_REASON")), ("fHasFreshnessTime", SimTypeInt(signed=True, label="Int32")), ("dwFreshnessTime", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_REVOCATION_STATUS", pack=False, align=None),
    "CERT_REVOCATION_STATUS_REASON": SimTypeInt(signed=False, label="CERT_REVOCATION_STATUS_REASON"),
    "CERT_ROOT_PROGRAM_FLAGS": SimTypeInt(signed=False, label="CERT_ROOT_PROGRAM_FLAGS"),
    "CERT_SELECTUI_INPUT": SimStruct(OrderedDict((("hStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("prgpChain", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0), offset=0)), ("cChain", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_SELECTUI_INPUT", pack=False, align=None),
    "CERT_SELECT_CHAIN_PARA": SimStruct(OrderedDict((("hChainEngine", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pTime", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("hAdditionalStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pChainPara", SimTypePointer(SimTypeRef("CERT_CHAIN_PARA", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_SELECT_CHAIN_PARA", pack=False, align=None),
    "CERT_SELECT_CRITERIA": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="CERT_SELECT_CRITERIA_TYPE")), ("cPara", SimTypeInt(signed=False, label="UInt32")), ("ppPara", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="CERT_SELECT_CRITERIA", pack=False, align=None),
    "CERT_SELECT_CRITERIA_TYPE": SimTypeInt(signed=False, label="CERT_SELECT_CRITERIA_TYPE"),
    "CERT_SELECT_STRUCT_A": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CERT_SELECT_STRUCT_FLAGS")), ("szTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cCertStore", SimTypeInt(signed=False, label="UInt32")), ("arrayCertStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("szPurposeOid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cCertContext", SimTypeInt(signed=False, label="UInt32")), ("arrayCertContext", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndDialog", "message", "wParam", "lParam"]), offset=0)), ("pfnFilter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "param1", "param2", "param3"]), offset=0)), ("szHelpFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwHelpId", SimTypeInt(signed=False, label="UInt32")), ("hprov", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CERT_SELECT_STRUCT_A", pack=False, align=None),
    "CERT_SELECT_STRUCT_FLAGS": SimTypeInt(signed=False, label="CERT_SELECT_STRUCT_FLAGS"),
    "CERT_SELECT_STRUCT_W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CERT_SELECT_STRUCT_FLAGS")), ("szTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cCertStore", SimTypeInt(signed=False, label="UInt32")), ("arrayCertStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("szPurposeOid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cCertContext", SimTypeInt(signed=False, label="UInt32")), ("arrayCertContext", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndDialog", "message", "wParam", "lParam"]), offset=0)), ("pfnFilter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "param1", "param2", "param3"]), offset=0)), ("szHelpFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHelpId", SimTypeInt(signed=False, label="UInt32")), ("hprov", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CERT_SELECT_STRUCT_W", pack=False, align=None),
    "CERT_SERVER_OCSP_RESPONSE_CONTEXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pbEncodedOcspResponse", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbEncodedOcspResponse", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_SERVER_OCSP_RESPONSE_CONTEXT", pack=False, align=None),
    "CERT_SERVER_OCSP_RESPONSE_OPEN_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pcbUsedSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pwszOcspDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnUpdateCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CERT_SERVER_OCSP_RESPONSE_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pChainContext", "pServerOcspResponseContext", "pNewCrlContext", "pPrevCrlContext", "pvArg", "dwWriteOcspFileError"]), offset=0)), ("pvUpdateCallbackArg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_SERVER_OCSP_RESPONSE_OPEN_PARA", pack=False, align=None),
    "CERT_SIGNED_CONTENT_INFO": SimStruct(OrderedDict((("ToBeSigned", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SignatureAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Signature", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)),)), name="CERT_SIGNED_CONTENT_INFO", pack=False, align=None),
    "CERT_SIMPLE_CHAIN": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("TrustStatus", SimTypeRef("CERT_TRUST_STATUS", SimStruct)), ("cElement", SimTypeInt(signed=False, label="UInt32")), ("rgpElement", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CHAIN_ELEMENT", SimStruct), offset=0), offset=0)), ("pTrustListInfo", SimTypePointer(SimTypeRef("CERT_TRUST_LIST_INFO", SimStruct), offset=0)), ("fHasRevocationFreshnessTime", SimTypeInt(signed=True, label="Int32")), ("dwRevocationFreshnessTime", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_SIMPLE_CHAIN", pack=False, align=None),
    "CERT_STORE_PROV_FIND_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgAndCertEncodingType", SimTypeInt(signed=False, label="UInt32")), ("dwFindFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFindType", SimTypeInt(signed=False, label="UInt32")), ("pvFindPara", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_STORE_PROV_FIND_INFO", pack=False, align=None),
    "CERT_STORE_PROV_FLAGS": SimTypeInt(signed=False, label="CERT_STORE_PROV_FLAGS"),
    "CERT_STORE_PROV_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cStoreProvFunc", SimTypeInt(signed=False, label="UInt32")), ("rgpvStoreProvFunc", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("hStoreProv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwStoreProvFlags", SimTypeInt(signed=False, label="CERT_STORE_PROV_FLAGS")), ("hStoreProvFuncAddr2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CERT_STORE_PROV_INFO", pack=False, align=None),
    "CERT_STORE_SAVE_AS": SimTypeInt(signed=False, label="CERT_STORE_SAVE_AS"),
    "CERT_STORE_SAVE_TO": SimTypeInt(signed=False, label="CERT_STORE_SAVE_TO"),
    "CERT_STRING_TYPE": SimTypeInt(signed=False, label="CERT_STRING_TYPE"),
    "CERT_STRONG_SIGN_FLAGS": SimTypeInt(signed=False, label="CERT_STRONG_SIGN_FLAGS"),
    "CERT_STRONG_SIGN_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwInfoChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pvInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pSerializedInfo": SimTypePointer(SimTypeRef("CERT_STRONG_SIGN_SERIALIZED_INFO", SimStruct), offset=0), "pszOID": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="CERT_STRONG_SIGN_PARA", pack=False, align=None),
    "CERT_STRONG_SIGN_SERIALIZED_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="CERT_STRONG_SIGN_FLAGS")), ("pwszCNGSignHashAlgids", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszCNGPubKeyMinBitLengths", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CERT_STRONG_SIGN_SERIALIZED_INFO", pack=False, align=None),
    "CERT_SUPPORTED_ALGORITHM_INFO": SimStruct(OrderedDict((("Algorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("IntendedKeyUsage", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("IntendedCertPolicies", SimTypeRef("CERT_POLICIES_INFO", SimStruct)),)), name="CERT_SUPPORTED_ALGORITHM_INFO", pack=False, align=None),
    "CERT_SYSTEM_STORE_FLAGS": SimTypeInt(signed=False, label="CERT_SYSTEM_STORE_FLAGS"),
    "CERT_SYSTEM_STORE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_SYSTEM_STORE_INFO", pack=False, align=None),
    "CERT_SYSTEM_STORE_RELOCATE_PARA": SimStruct(OrderedDict((("Anonymous1", SimUnion({"hKeyBase": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pvBase": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"pvSystemStore": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pszSystemStore": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszSystemStore": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="CERT_SYSTEM_STORE_RELOCATE_PARA", pack=False, align=None),
    "CERT_TEMPLATE_EXT": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("fMinorVersion", SimTypeInt(signed=True, label="Int32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_TEMPLATE_EXT", pack=False, align=None),
    "CERT_TPM_SPECIFICATION_INFO": SimStruct(OrderedDict((("pwszFamily", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwRevision", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_TPM_SPECIFICATION_INFO", pack=False, align=None),
    "CERT_TRUST_LIST_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pCtlEntry", SimTypePointer(SimTypeRef("CTL_ENTRY", SimStruct), offset=0)), ("pCtlContext", SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0)),)), name="CERT_TRUST_LIST_INFO", pack=False, align=None),
    "CERT_TRUST_STATUS": SimStruct(OrderedDict((("dwErrorStatus", SimTypeInt(signed=False, label="UInt32")), ("dwInfoStatus", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_TRUST_STATUS", pack=False, align=None),
    "CERT_USAGE_MATCH": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("Usage", SimTypeRef("CTL_USAGE", SimStruct)),)), name="CERT_USAGE_MATCH", pack=False, align=None),
    "CERT_VERIFY_CERTIFICATE_TRUST": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pccert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwIgnoreErr", SimTypeInt(signed=False, label="UInt32")), ("pdwErrors", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszUsageOid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hprov", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cRootStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreRoots", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreCAs", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cTrustStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreTrust", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnTrustHelper", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "lCustData", "fLeafCertificate", "pbTrustBlob"]), offset=0)), ("pcChain", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("prgChain", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), offset=0)), ("prgdwErrors", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), offset=0)), ("prgpbTrustInfo", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), offset=0)),)), name="CERT_VERIFY_CERTIFICATE_TRUST", pack=False, align=None),
    "CERT_VIEWPROPERTIES_STRUCT_A": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CERT_VIEWPROPERTIES_STRUCT_FLAGS")), ("szTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("arrayPurposes", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)), ("cArrayPurposes", SimTypeInt(signed=False, label="UInt32")), ("cRootStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreRoots", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreCAs", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cTrustStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreTrust", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("hprov", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwPad", SimTypeInt(signed=False, label="UInt32")), ("szHelpFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwHelpId", SimTypeInt(signed=False, label="UInt32")), ("nStartPage", SimTypeInt(signed=False, label="UInt32")), ("cArrayPropSheetPages", SimTypeInt(signed=False, label="UInt32")), ("arrayPropSheetPages", SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)),)), name="CERT_VIEWPROPERTIES_STRUCT_A", pack=False, align=None),
    "CERT_VIEWPROPERTIES_STRUCT_FLAGS": SimTypeInt(signed=False, label="CERT_VIEWPROPERTIES_STRUCT_FLAGS"),
    "CERT_VIEWPROPERTIES_STRUCT_W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CERT_VIEWPROPERTIES_STRUCT_FLAGS")), ("szTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("arrayPurposes", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)), ("cArrayPurposes", SimTypeInt(signed=False, label="UInt32")), ("cRootStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreRoots", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreCAs", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cTrustStores", SimTypeInt(signed=False, label="UInt32")), ("rghstoreTrust", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("hprov", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwPad", SimTypeInt(signed=False, label="UInt32")), ("szHelpFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHelpId", SimTypeInt(signed=False, label="UInt32")), ("nStartPage", SimTypeInt(signed=False, label="UInt32")), ("cArrayPropSheetPages", SimTypeInt(signed=False, label="UInt32")), ("arrayPropSheetPages", SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)),)), name="CERT_VIEWPROPERTIES_STRUCT_W", pack=False, align=None),
    "CERT_VIEW_COLUMN_INDEX": SimTypeInt(signed=False, label="CERT_VIEW_COLUMN_INDEX"),
    "CERT_VIEW_SEEK_OPERATOR_FLAGS": SimTypeInt(signed=False, label="CERT_VIEW_SEEK_OPERATOR_FLAGS"),
    "CERT_X942_DH_PARAMETERS": SimStruct(OrderedDict((("p", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("g", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("q", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("j", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pValidationParams", SimTypePointer(SimTypeRef("CERT_X942_DH_VALIDATION_PARAMS", SimStruct), offset=0)),)), name="CERT_X942_DH_PARAMETERS", pack=False, align=None),
    "CERT_X942_DH_VALIDATION_PARAMS": SimStruct(OrderedDict((("seed", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("pgenCounter", SimTypeInt(signed=False, label="UInt32")),)), name="CERT_X942_DH_VALIDATION_PARAMS", pack=False, align=None),
    "CESSetupProperty": SimTypeInt(signed=False, label="CESSetupProperty"),
    "CFE_EFFECTS": SimTypeInt(signed=False, label="CFE_EFFECTS"),
    "CFE_UNDERLINE": SimTypeInt(signed=False, label="CFE_UNDERLINE"),
    "CFG_CALL_TARGET_INFO": SimStruct(OrderedDict((("Offset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Flags", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CFG_CALL_TARGET_INFO", pack=False, align=None),
    "CFM_MASK": SimTypeInt(signed=False, label="CFM_MASK"),
    "CFP_ALLOCPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0),
    "CFP_FREEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "CFP_REALLOCPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0", "param1"]), offset=0),
    "CF_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CF_CALLBACK_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CF_CALLBACK_PARAMETERS", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackInfo", "CallbackParameters"]), offset=0),
    "CF_CALLBACK_CANCEL_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_CANCEL_FLAGS"),
    "CF_CALLBACK_CLOSE_COMPLETION_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_CLOSE_COMPLETION_FLAGS"),
    "CF_CALLBACK_DEHYDRATE_COMPLETION_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATE_COMPLETION_FLAGS"),
    "CF_CALLBACK_DEHYDRATE_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATE_FLAGS"),
    "CF_CALLBACK_DEHYDRATION_REASON": SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATION_REASON"),
    "CF_CALLBACK_DELETE_COMPLETION_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_DELETE_COMPLETION_FLAGS"),
    "CF_CALLBACK_DELETE_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_DELETE_FLAGS"),
    "CF_CALLBACK_FETCH_DATA_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_FETCH_DATA_FLAGS"),
    "CF_CALLBACK_FETCH_PLACEHOLDERS_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_FETCH_PLACEHOLDERS_FLAGS"),
    "CF_CALLBACK_INFO": SimStruct(OrderedDict((("StructSize", SimTypeInt(signed=False, label="UInt32")), ("ConnectionKey", SimTypeLongLong(signed=True, label="Int64")), ("CallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("VolumeGuidName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("VolumeDosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("VolumeSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("SyncRootFileId", SimTypeLongLong(signed=True, label="Int64")), ("SyncRootIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SyncRootIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("FileIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("NormalizedPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TransferKey", SimTypeLongLong(signed=True, label="Int64")), ("PriorityHint", SimTypeChar(label="Byte")), ("CorrelationVector", SimTypePointer(SimTypeRef("CORRELATION_VECTOR", SimStruct), offset=0)), ("ProcessInfo", SimTypePointer(SimTypeRef("CF_PROCESS_INFO", SimStruct), offset=0)), ("RequestKey", SimTypeLongLong(signed=True, label="Int64")),)), name="CF_CALLBACK_INFO", pack=False, align=None),
    "CF_CALLBACK_OPEN_COMPLETION_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_OPEN_COMPLETION_FLAGS"),
    "CF_CALLBACK_PARAMETERS": SimStruct(OrderedDict((("ParamSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Cancel": SimTypeRef("_Cancel_e__Struct", SimStruct), "FetchData": SimTypeRef("_FetchData_e__Struct", SimStruct), "ValidateData": SimTypeRef("_ValidateData_e__Struct", SimStruct), "FetchPlaceholders": SimTypeRef("_FetchPlaceholders_e__Struct", SimStruct), "OpenCompletion": SimTypeRef("_OpenCompletion_e__Struct", SimStruct), "CloseCompletion": SimTypeRef("_CloseCompletion_e__Struct", SimStruct), "Dehydrate": SimTypeRef("_Dehydrate_e__Struct", SimStruct), "DehydrateCompletion": SimTypeRef("_DehydrateCompletion_e__Struct", SimStruct), "Delete": SimTypeRef("_Delete_e__Struct", SimStruct), "DeleteCompletion": SimTypeRef("_DeleteCompletion_e__Struct", SimStruct), "Rename": SimTypeRef("_Rename_e__Struct", SimStruct), "RenameCompletion": SimTypeRef("_RenameCompletion_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CF_CALLBACK_PARAMETERS", pack=False, align=None),
    "CF_CALLBACK_REGISTRATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="CF_CALLBACK_TYPE")), ("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CF_CALLBACK_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CF_CALLBACK_PARAMETERS", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackInfo", "CallbackParameters"]), offset=0)),)), name="CF_CALLBACK_REGISTRATION", pack=False, align=None),
    "CF_CALLBACK_RENAME_COMPLETION_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_RENAME_COMPLETION_FLAGS"),
    "CF_CALLBACK_RENAME_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_RENAME_FLAGS"),
    "CF_CALLBACK_TYPE": SimTypeInt(signed=False, label="CF_CALLBACK_TYPE"),
    "CF_CALLBACK_VALIDATE_DATA_FLAGS": SimTypeInt(signed=False, label="CF_CALLBACK_VALIDATE_DATA_FLAGS"),
    "CF_CONNECTION_KEY": SimTypeLongLong(signed=True, label="Int64"),
    "CF_CONNECT_FLAGS": SimTypeInt(signed=False, label="CF_CONNECT_FLAGS"),
    "CF_CONVERT_FLAGS": SimTypeInt(signed=False, label="CF_CONVERT_FLAGS"),
    "CF_CREATE_FLAGS": SimTypeInt(signed=False, label="CF_CREATE_FLAGS"),
    "CF_DEHYDRATE_FLAGS": SimTypeInt(signed=False, label="CF_DEHYDRATE_FLAGS"),
    "CF_FILE_RANGE": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="CF_FILE_RANGE", pack=False, align=None),
    "CF_FS_METADATA": SimStruct(OrderedDict((("BasicInfo", SimTypeRef("FILE_BASIC_INFO", SimStruct)), ("FileSize", SimTypeLongLong(signed=True, label="Int64")),)), name="CF_FS_METADATA", pack=False, align=None),
    "CF_HARDLINK_POLICY": SimTypeInt(signed=False, label="CF_HARDLINK_POLICY"),
    "CF_HYDRATE_FLAGS": SimTypeInt(signed=False, label="CF_HYDRATE_FLAGS"),
    "CF_HYDRATION_POLICY": SimStruct(OrderedDict((("Primary", SimTypeInt(signed=False, label="CF_HYDRATION_POLICY_PRIMARY")), ("Modifier", SimTypeInt(signed=False, label="CF_HYDRATION_POLICY_MODIFIER")),)), name="CF_HYDRATION_POLICY", pack=False, align=None),
    "CF_HYDRATION_POLICY_MODIFIER": SimTypeInt(signed=False, label="CF_HYDRATION_POLICY_MODIFIER"),
    "CF_HYDRATION_POLICY_PRIMARY": SimTypeInt(signed=False, label="CF_HYDRATION_POLICY_PRIMARY"),
    "CF_INSYNC_POLICY": SimTypeInt(signed=False, label="CF_INSYNC_POLICY"),
    "CF_IN_SYNC_STATE": SimTypeInt(signed=False, label="CF_IN_SYNC_STATE"),
    "CF_OPEN_FILE_FLAGS": SimTypeInt(signed=False, label="CF_OPEN_FILE_FLAGS"),
    "CF_OPERATION_ACK_DATA_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_ACK_DATA_FLAGS"),
    "CF_OPERATION_ACK_DEHYDRATE_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_ACK_DEHYDRATE_FLAGS"),
    "CF_OPERATION_ACK_DELETE_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_ACK_DELETE_FLAGS"),
    "CF_OPERATION_ACK_RENAME_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_ACK_RENAME_FLAGS"),
    "CF_OPERATION_INFO": SimStruct(OrderedDict((("StructSize", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="CF_OPERATION_TYPE")), ("ConnectionKey", SimTypeLongLong(signed=True, label="Int64")), ("TransferKey", SimTypeLongLong(signed=True, label="Int64")), ("CorrelationVector", SimTypePointer(SimTypeRef("CORRELATION_VECTOR", SimStruct), offset=0)), ("SyncStatus", SimTypePointer(SimTypeRef("CF_SYNC_STATUS", SimStruct), offset=0)), ("RequestKey", SimTypeLongLong(signed=True, label="Int64")),)), name="CF_OPERATION_INFO", pack=False, align=None),
    "CF_OPERATION_PARAMETERS": SimStruct(OrderedDict((("ParamSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"TransferData": SimTypeRef("_TransferData_e__Struct", SimStruct), "RetrieveData": SimTypeRef("_RetrieveData_e__Struct", SimStruct), "AckData": SimTypeRef("_AckData_e__Struct", SimStruct), "RestartHydration": SimTypeRef("_RestartHydration_e__Struct", SimStruct), "TransferPlaceholders": SimTypeRef("_TransferPlaceholders_e__Struct", SimStruct), "AckDehydrate": SimTypeRef("_AckDehydrate_e__Struct", SimStruct), "AckRename": SimTypeRef("_AckRename_e__Struct", SimStruct), "AckDelete": SimTypeRef("_AckDelete_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CF_OPERATION_PARAMETERS", pack=False, align=None),
    "CF_OPERATION_RESTART_HYDRATION_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_RESTART_HYDRATION_FLAGS"),
    "CF_OPERATION_RETRIEVE_DATA_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_RETRIEVE_DATA_FLAGS"),
    "CF_OPERATION_TRANSFER_DATA_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_TRANSFER_DATA_FLAGS"),
    "CF_OPERATION_TRANSFER_PLACEHOLDERS_FLAGS": SimTypeInt(signed=False, label="CF_OPERATION_TRANSFER_PLACEHOLDERS_FLAGS"),
    "CF_OPERATION_TYPE": SimTypeInt(signed=False, label="CF_OPERATION_TYPE"),
    "CF_PIN_STATE": SimTypeInt(signed=False, label="CF_PIN_STATE"),
    "CF_PLACEHOLDER_BASIC_INFO": SimStruct(OrderedDict((("PinState", SimTypeInt(signed=False, label="CF_PIN_STATE")), ("InSyncState", SimTypeInt(signed=False, label="CF_IN_SYNC_STATE")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("SyncRootFileId", SimTypeLongLong(signed=True, label="Int64")), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("FileIdentity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CF_PLACEHOLDER_BASIC_INFO", pack=False, align=None),
    "CF_PLACEHOLDER_CREATE_FLAGS": SimTypeInt(signed=False, label="CF_PLACEHOLDER_CREATE_FLAGS"),
    "CF_PLACEHOLDER_CREATE_INFO": SimStruct(OrderedDict((("RelativeFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FsMetadata", SimTypeRef("CF_FS_METADATA", SimStruct)), ("FileIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="CF_PLACEHOLDER_CREATE_FLAGS")), ("Result", SimTypeInt(signed=True, label="Int32")), ("CreateUsn", SimTypeLongLong(signed=True, label="Int64")),)), name="CF_PLACEHOLDER_CREATE_INFO", pack=False, align=None),
    "CF_PLACEHOLDER_INFO_CLASS": SimTypeInt(signed=False, label="CF_PLACEHOLDER_INFO_CLASS"),
    "CF_PLACEHOLDER_MANAGEMENT_POLICY": SimTypeInt(signed=False, label="CF_PLACEHOLDER_MANAGEMENT_POLICY"),
    "CF_PLACEHOLDER_RANGE_INFO_CLASS": SimTypeInt(signed=False, label="CF_PLACEHOLDER_RANGE_INFO_CLASS"),
    "CF_PLACEHOLDER_STANDARD_INFO": SimStruct(OrderedDict((("OnDiskDataSize", SimTypeLongLong(signed=True, label="Int64")), ("ValidatedDataSize", SimTypeLongLong(signed=True, label="Int64")), ("ModifiedDataSize", SimTypeLongLong(signed=True, label="Int64")), ("PropertiesSize", SimTypeLongLong(signed=True, label="Int64")), ("PinState", SimTypeInt(signed=False, label="CF_PIN_STATE")), ("InSyncState", SimTypeInt(signed=False, label="CF_IN_SYNC_STATE")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("SyncRootFileId", SimTypeLongLong(signed=True, label="Int64")), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("FileIdentity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CF_PLACEHOLDER_STANDARD_INFO", pack=False, align=None),
    "CF_PLACEHOLDER_STATE": SimTypeInt(signed=False, label="CF_PLACEHOLDER_STATE"),
    "CF_PLATFORM_INFO": SimStruct(OrderedDict((("BuildNumber", SimTypeInt(signed=False, label="UInt32")), ("RevisionNumber", SimTypeInt(signed=False, label="UInt32")), ("IntegrationNumber", SimTypeInt(signed=False, label="UInt32")),)), name="CF_PLATFORM_INFO", pack=False, align=None),
    "CF_POPULATION_POLICY": SimStruct(OrderedDict((("Primary", SimTypeInt(signed=False, label="CF_POPULATION_POLICY_PRIMARY")), ("Modifier", SimTypeInt(signed=False, label="CF_POPULATION_POLICY_MODIFIER")),)), name="CF_POPULATION_POLICY", pack=False, align=None),
    "CF_POPULATION_POLICY_MODIFIER": SimTypeInt(signed=False, label="CF_POPULATION_POLICY_MODIFIER"),
    "CF_POPULATION_POLICY_PRIMARY": SimTypeInt(signed=False, label="CF_POPULATION_POLICY_PRIMARY"),
    "CF_PROCESS_INFO": SimStruct(OrderedDict((("StructSize", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ImagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ApplicationId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CommandLine", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SessionId", SimTypeInt(signed=False, label="UInt32")),)), name="CF_PROCESS_INFO", pack=False, align=None),
    "CF_REGISTER_FLAGS": SimTypeInt(signed=False, label="CF_REGISTER_FLAGS"),
    "CF_REVERT_FLAGS": SimTypeInt(signed=False, label="CF_REVERT_FLAGS"),
    "CF_SET_IN_SYNC_FLAGS": SimTypeInt(signed=False, label="CF_SET_IN_SYNC_FLAGS"),
    "CF_SET_PIN_FLAGS": SimTypeInt(signed=False, label="CF_SET_PIN_FLAGS"),
    "CF_SYNC_POLICIES": SimStruct(OrderedDict((("StructSize", SimTypeInt(signed=False, label="UInt32")), ("Hydration", SimTypeRef("CF_HYDRATION_POLICY", SimStruct)), ("Population", SimTypeRef("CF_POPULATION_POLICY", SimStruct)), ("InSync", SimTypeInt(signed=False, label="CF_INSYNC_POLICY")), ("HardLink", SimTypeInt(signed=False, label="CF_HARDLINK_POLICY")), ("PlaceholderManagement", SimTypeInt(signed=False, label="CF_PLACEHOLDER_MANAGEMENT_POLICY")),)), name="CF_SYNC_POLICIES", pack=False, align=None),
    "CF_SYNC_PROVIDER_STATUS": SimTypeInt(signed=False, label="CF_SYNC_PROVIDER_STATUS"),
    "CF_SYNC_REGISTRATION": SimStruct(OrderedDict((("StructSize", SimTypeInt(signed=False, label="UInt32")), ("ProviderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProviderVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SyncRootIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SyncRootIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("FileIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("ProviderId", SimTypeBottom(label="Guid")),)), name="CF_SYNC_REGISTRATION", pack=False, align=None),
    "CF_SYNC_ROOT_BASIC_INFO": SimStruct(OrderedDict((("SyncRootFileId", SimTypeLongLong(signed=True, label="Int64")),)), name="CF_SYNC_ROOT_BASIC_INFO", pack=False, align=None),
    "CF_SYNC_ROOT_INFO_CLASS": SimTypeInt(signed=False, label="CF_SYNC_ROOT_INFO_CLASS"),
    "CF_SYNC_ROOT_PROVIDER_INFO": SimStruct(OrderedDict((("ProviderStatus", SimTypeInt(signed=False, label="CF_SYNC_PROVIDER_STATUS")), ("ProviderName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ProviderVersion", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="CF_SYNC_ROOT_PROVIDER_INFO", pack=False, align=None),
    "CF_SYNC_ROOT_STANDARD_INFO": SimStruct(OrderedDict((("SyncRootFileId", SimTypeLongLong(signed=True, label="Int64")), ("HydrationPolicy", SimTypeRef("CF_HYDRATION_POLICY", SimStruct)), ("PopulationPolicy", SimTypeRef("CF_POPULATION_POLICY", SimStruct)), ("InSyncPolicy", SimTypeInt(signed=False, label="CF_INSYNC_POLICY")), ("HardLinkPolicy", SimTypeInt(signed=False, label="CF_HARDLINK_POLICY")), ("ProviderStatus", SimTypeInt(signed=False, label="CF_SYNC_PROVIDER_STATUS")), ("ProviderName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ProviderVersion", SimTypeArray(SimTypeChar(label="Char"), 256)), ("SyncRootIdentityLength", SimTypeInt(signed=False, label="UInt32")), ("SyncRootIdentity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CF_SYNC_ROOT_STANDARD_INFO", pack=False, align=None),
    "CF_SYNC_STATUS": SimStruct(OrderedDict((("StructSize", SimTypeInt(signed=False, label="UInt32")), ("Code", SimTypeInt(signed=False, label="UInt32")), ("DescriptionOffset", SimTypeInt(signed=False, label="UInt32")), ("DescriptionLength", SimTypeInt(signed=False, label="UInt32")), ("DeviceIdOffset", SimTypeInt(signed=False, label="UInt32")), ("DeviceIdLength", SimTypeInt(signed=False, label="UInt32")),)), name="CF_SYNC_STATUS", pack=False, align=None),
    "CF_UPDATE_FLAGS": SimTypeInt(signed=False, label="CF_UPDATE_FLAGS"),
    "CGpEffect": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CHANGEFILTERSTRUCT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ExtStatus", SimTypeInt(signed=False, label="MSGFLTINFO_STATUS")),)), name="CHANGEFILTERSTRUCT", pack=False, align=None),
    "CHANGEKIND": SimTypeInt(signed=False, label="CHANGEKIND"),
    "CHANGENOTIFY": SimStruct(OrderedDict((("dwChangeType", SimTypeInt(signed=False, label="UInt32")), ("pvCookieData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CHANGENOTIFY", pack=False, align=None),
    "CHANGER_DEVICE_PROBLEM_TYPE": SimTypeInt(signed=False, label="CHANGER_DEVICE_PROBLEM_TYPE"),
    "CHANGER_ELEMENT": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="ELEMENT_TYPE")), ("ElementAddress", SimTypeInt(signed=False, label="UInt32")),)), name="CHANGER_ELEMENT", pack=False, align=None),
    "CHANGER_ELEMENT_LIST": SimStruct(OrderedDict((("Element", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("NumberOfElements", SimTypeInt(signed=False, label="UInt32")),)), name="CHANGER_ELEMENT_LIST", pack=False, align=None),
    "CHANGER_ELEMENT_STATUS": SimStruct(OrderedDict((("Element", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("SrcElementAddress", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Flags", SimTypeInt(signed=False, label="CHANGER_ELEMENT_STATUS_FLAGS")), ("ExceptionCode", SimTypeInt(signed=False, label="UInt32")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("PrimaryVolumeID", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("AlternateVolumeID", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="CHANGER_ELEMENT_STATUS", pack=False, align=None),
    "CHANGER_ELEMENT_STATUS_EX": SimStruct(OrderedDict((("Element", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("SrcElementAddress", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Flags", SimTypeInt(signed=False, label="CHANGER_ELEMENT_STATUS_FLAGS")), ("ExceptionCode", SimTypeInt(signed=False, label="UInt32")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("PrimaryVolumeID", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("AlternateVolumeID", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("VendorIdentification", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("ProductIdentification", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("SerialNumber", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="CHANGER_ELEMENT_STATUS_EX", pack=False, align=None),
    "CHANGER_ELEMENT_STATUS_FLAGS": SimTypeInt(signed=False, label="CHANGER_ELEMENT_STATUS_FLAGS"),
    "CHANGER_EXCHANGE_MEDIUM": SimStruct(OrderedDict((("Transport", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Source", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Destination1", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Destination2", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Flip1", SimTypeChar(label="Byte")), ("Flip2", SimTypeChar(label="Byte")),)), name="CHANGER_EXCHANGE_MEDIUM", pack=False, align=None),
    "CHANGER_FEATURES": SimTypeInt(signed=False, label="CHANGER_FEATURES"),
    "CHANGER_INITIALIZE_ELEMENT_STATUS": SimStruct(OrderedDict((("ElementList", SimTypeRef("CHANGER_ELEMENT_LIST", SimStruct)), ("BarCodeScan", SimTypeChar(label="Byte")),)), name="CHANGER_INITIALIZE_ELEMENT_STATUS", pack=False, align=None),
    "CHANGER_MOVE_MEDIUM": SimStruct(OrderedDict((("Transport", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Source", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Destination", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Flip", SimTypeChar(label="Byte")),)), name="CHANGER_MOVE_MEDIUM", pack=False, align=None),
    "CHANGER_PRODUCT_DATA": SimStruct(OrderedDict((("VendorId", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("ProductId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Revision", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("SerialNumber", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("DeviceType", SimTypeChar(label="Byte")),)), name="CHANGER_PRODUCT_DATA", pack=False, align=None),
    "CHANGER_READ_ELEMENT_STATUS": SimStruct(OrderedDict((("ElementList", SimTypeRef("CHANGER_ELEMENT_LIST", SimStruct)), ("VolumeTagInfo", SimTypeChar(label="Byte")),)), name="CHANGER_READ_ELEMENT_STATUS", pack=False, align=None),
    "CHANGER_SEND_VOLUME_TAG_INFORMATION": SimStruct(OrderedDict((("StartingElement", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("ActionCode", SimTypeInt(signed=False, label="UInt32")), ("VolumeIDTemplate", SimTypeArray(SimTypeChar(label="Byte"), 40)),)), name="CHANGER_SEND_VOLUME_TAG_INFORMATION", pack=False, align=None),
    "CHANGER_SET_ACCESS": SimStruct(OrderedDict((("Element", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Control", SimTypeInt(signed=False, label="UInt32")),)), name="CHANGER_SET_ACCESS", pack=False, align=None),
    "CHANGER_SET_POSITION": SimStruct(OrderedDict((("Transport", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Destination", SimTypeRef("CHANGER_ELEMENT", SimStruct)), ("Flip", SimTypeChar(label="Byte")),)), name="CHANGER_SET_POSITION", pack=False, align=None),
    "CHANGETYPE": SimTypeInt(signed=False, label="CHANGETYPE"),
    "CHANGE_ATTRIBUTES_PARAMETERS": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("Anonymous", SimUnion({"MbrPartInfo": SimTypeRef("_MbrPartInfo_e__Struct", SimStruct), "GptPartInfo": SimTypeRef("_GptPartInfo_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CHANGE_ATTRIBUTES_PARAMETERS", pack=False, align=None),
    "CHANGE_ICON_FLAGS": SimTypeInt(signed=False, label="CHANGE_ICON_FLAGS"),
    "CHANGE_PARTITION_TYPE_PARAMETERS": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("Anonymous", SimUnion({"MbrPartInfo": SimTypeRef("_MbrPartInfo_e__Struct", SimStruct), "GptPartInfo": SimTypeRef("_GptPartInfo_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CHANGE_PARTITION_TYPE_PARAMETERS", pack=False, align=None),
    "CHANGE_PASSWORD_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "CHANGE_PASSWORD_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "CHANGE_SOURCE_FLAGS": SimTypeInt(signed=False, label="CHANGE_SOURCE_FLAGS"),
    "CHANGE_WINDOW_MESSAGE_FILTER_FLAGS": SimTypeInt(signed=False, label="CHANGE_WINDOW_MESSAGE_FILTER_FLAGS"),
    "CHANNEL_ACCESS_ENUM": SimTypeInt(signed=False, label="CHANNEL_ACCESS_ENUM"),
    "CHANNEL_AGENT_FLAGS": SimTypeInt(signed=False, label="CHANNEL_AGENT_FLAGS"),
    "CHANNEL_CAPS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSrcRectXMod", SimTypeInt(signed=False, label="UInt32")), ("dwSrcRectYMod", SimTypeInt(signed=False, label="UInt32")), ("dwSrcRectWidthMod", SimTypeInt(signed=False, label="UInt32")), ("dwSrcRectHeightMod", SimTypeInt(signed=False, label="UInt32")), ("dwDstRectXMod", SimTypeInt(signed=False, label="UInt32")), ("dwDstRectYMod", SimTypeInt(signed=False, label="UInt32")), ("dwDstRectWidthMod", SimTypeInt(signed=False, label="UInt32")), ("dwDstRectHeightMod", SimTypeInt(signed=False, label="UInt32")),)), name="CHANNEL_CAPS", pack=False, align=None),
    "CHANNEL_DEF": SimStruct(OrderedDict((("name", SimTypeArray(SimTypeChar(label="SByte"), 8)), ("options", SimTypeInt(signed=False, label="UInt32")),)), name="CHANNEL_DEF", pack=False, align=None),
    "CHANNEL_ENTRY_POINTS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("protocolVersion", SimTypeInt(signed=False, label="UInt32")), ("pVirtualChannelInit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeRef("CHANNEL_DEF", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pInitHandle", "event", "pData", "dataLength"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppInitHandle", "pChannel", "channelCount", "versionRequested", "pChannelInitEventProc"]), offset=0)), ("pVirtualChannelOpen", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["openHandle", "event", "pData", "dataLength", "totalLength", "dataFlags"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pInitHandle", "pOpenHandle", "pChannelName", "pChannelOpenEventProc"]), offset=0)), ("pVirtualChannelClose", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["openHandle"]), offset=0)), ("pVirtualChannelWrite", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["openHandle", "pData", "dataLength", "pUserData"]), offset=0)),)), name="CHANNEL_ENTRY_POINTS", pack=False, align=None),
    "CHANNEL_FLAGS": SimTypeInt(signed=False, label="CHANNEL_FLAGS"),
    "CHANNEL_INFO": SimStruct(OrderedDict((("EventChannelSize", SimTypeInt(signed=False, label="UInt32")), ("uReadDataAlignment", SimTypeInt(signed=False, label="UInt32")), ("uWriteDataAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="CHANNEL_INFO", pack=False, align=None),
    "CHANNEL_PDU_HEADER": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="CHANNEL_PDU_HEADER", pack=False, align=None),
    "CHANNEL_PRIORITY": SimTypeInt(signed=False, label="CHANNEL_PRIORITY"),
    "CHAR": SimTypeChar(label="SByte"),
    "CHARACTER_RANGE": SimStruct(OrderedDict((("wcLow", SimTypeChar(label="Char")), ("cChars", SimTypeShort(signed=False, label="UInt16")),)), name="CHARACTER_RANGE", pack=False, align=None),
    "CHARFORMAT2A": SimStruct(OrderedDict((("Base", SimTypeRef("CHARFORMATA", SimStruct)), ("wWeight", SimTypeShort(signed=False, label="UInt16")), ("sSpacing", SimTypeShort(signed=True, label="Int16")), ("crBackColor", SimTypeInt(signed=False, label="UInt32")), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwReserved": SimTypeInt(signed=False, label="UInt32"), "dwCookie": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("sStyle", SimTypeShort(signed=True, label="Int16")), ("wKerning", SimTypeShort(signed=False, label="UInt16")), ("bUnderlineType", SimTypeChar(label="Byte")), ("bAnimation", SimTypeChar(label="Byte")), ("bRevAuthor", SimTypeChar(label="Byte")), ("bUnderlineColor", SimTypeChar(label="Byte")),)), name="CHARFORMAT2A", pack=False, align=None),
    "CHARFORMAT2W": SimStruct(OrderedDict((("Base", SimTypeRef("CHARFORMATW", SimStruct)), ("wWeight", SimTypeShort(signed=False, label="UInt16")), ("sSpacing", SimTypeShort(signed=True, label="Int16")), ("crBackColor", SimTypeInt(signed=False, label="UInt32")), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwReserved": SimTypeInt(signed=False, label="UInt32"), "dwCookie": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("sStyle", SimTypeShort(signed=True, label="Int16")), ("wKerning", SimTypeShort(signed=False, label="UInt16")), ("bUnderlineType", SimTypeChar(label="Byte")), ("bAnimation", SimTypeChar(label="Byte")), ("bRevAuthor", SimTypeChar(label="Byte")), ("bUnderlineColor", SimTypeChar(label="Byte")),)), name="CHARFORMAT2W", pack=False, align=None),
    "CHARFORMATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="CFM_MASK")), ("dwEffects", SimTypeInt(signed=False, label="CFE_EFFECTS")), ("yHeight", SimTypeInt(signed=True, label="Int32")), ("yOffset", SimTypeInt(signed=True, label="Int32")), ("crTextColor", SimTypeInt(signed=False, label="UInt32")), ("bCharSet", SimTypeInt(signed=False, label="FONT_CHARSET")), ("bPitchAndFamily", SimTypeChar(label="Byte")), ("szFaceName", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="CHARFORMATA", pack=False, align=None),
    "CHARFORMATW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="CFM_MASK")), ("dwEffects", SimTypeInt(signed=False, label="CFE_EFFECTS")), ("yHeight", SimTypeInt(signed=True, label="Int32")), ("yOffset", SimTypeInt(signed=True, label="Int32")), ("crTextColor", SimTypeInt(signed=False, label="UInt32")), ("bCharSet", SimTypeInt(signed=False, label="FONT_CHARSET")), ("bPitchAndFamily", SimTypeChar(label="Byte")), ("szFaceName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="CHARFORMATW", pack=False, align=None),
    "CHARRANGE": SimStruct(OrderedDict((("cpMin", SimTypeInt(signed=True, label="Int32")), ("cpMax", SimTypeInt(signed=True, label="Int32")),)), name="CHARRANGE", pack=False, align=None),
    "CHARSETINFO": SimStruct(OrderedDict((("ciCharset", SimTypeInt(signed=False, label="UInt32")), ("ciACP", SimTypeInt(signed=False, label="UInt32")), ("fs", SimTypeRef("FONTSIGNATURE", SimStruct)),)), name="CHARSETINFO", pack=False, align=None),
    "CHAR_IMAGE_INFO": SimStruct(OrderedDict((("CharInfo", SimTypeRef("CHAR_INFO", SimStruct)), ("FontImageInfo", SimTypeRef("FONT_IMAGE_INFO", SimStruct)),)), name="CHAR_IMAGE_INFO", pack=False, align=None),
    "CHAR_INFO": SimStruct(OrderedDict((("Char", SimUnion({"UnicodeChar": SimTypeChar(label="Char"), "AsciiChar": SimTypeChar(label="SByte")}, name="<anon>", label="None")), ("Attributes", SimTypeShort(signed=False, label="UInt16")),)), name="CHAR_INFO", pack=False, align=None),
    "CHECKBOXSTATES": SimTypeInt(signed=False, label="CHECKBOXSTATES"),
    "CHEVRONSTATES": SimTypeInt(signed=False, label="CHEVRONSTATES"),
    "CHEVRONVERTSTATES": SimTypeInt(signed=False, label="CHEVRONVERTSTATES"),
    "CHOOSECOLORA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rgbResult", SimTypeInt(signed=False, label="UInt32")), ("lpCustColors", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Flags", SimTypeInt(signed=False, label="CHOOSECOLOR_FLAGS")), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CHOOSECOLORA", pack=False, align=None),
    "CHOOSECOLORW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rgbResult", SimTypeInt(signed=False, label="UInt32")), ("lpCustColors", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Flags", SimTypeInt(signed=False, label="CHOOSECOLOR_FLAGS")), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CHOOSECOLORW", pack=False, align=None),
    "CHOOSECOLOR_FLAGS": SimTypeInt(signed=False, label="CHOOSECOLOR_FLAGS"),
    "CHOOSEFONTA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpLogFont", SimTypePointer(SimTypeRef("LOGFONTA", SimStruct), offset=0)), ("iPointSize", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="CHOOSEFONT_FLAGS")), ("rgbColors", SimTypeInt(signed=False, label="UInt32")), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszStyle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nFontType", SimTypeInt(signed=False, label="CHOOSEFONT_FONT_TYPE")), ("___MISSING_ALIGNMENT__", SimTypeShort(signed=False, label="UInt16")), ("nSizeMin", SimTypeInt(signed=True, label="Int32")), ("nSizeMax", SimTypeInt(signed=True, label="Int32")),)), name="CHOOSEFONTA", pack=False, align=None),
    "CHOOSEFONTW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpLogFont", SimTypePointer(SimTypeRef("LOGFONTW", SimStruct), offset=0)), ("iPointSize", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="CHOOSEFONT_FLAGS")), ("rgbColors", SimTypeInt(signed=False, label="UInt32")), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszStyle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nFontType", SimTypeInt(signed=False, label="CHOOSEFONT_FONT_TYPE")), ("___MISSING_ALIGNMENT__", SimTypeShort(signed=False, label="UInt16")), ("nSizeMin", SimTypeInt(signed=True, label="Int32")), ("nSizeMax", SimTypeInt(signed=True, label="Int32")),)), name="CHOOSEFONTW", pack=False, align=None),
    "CHOOSEFONT_FLAGS": SimTypeInt(signed=False, label="CHOOSEFONT_FLAGS"),
    "CHOOSEFONT_FONT_TYPE": SimTypeInt(signed=False, label="CHOOSEFONT_FONT_TYPE"),
    "CHROMATICITY_COORDINATE": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="CHROMATICITY_COORDINATE", pack=False, align=None),
    "CHUNKSTATE": SimTypeInt(signed=False, label="CHUNKSTATE"),
    "CHUNK_BREAKTYPE": SimTypeInt(signed=False, label="CHUNK_BREAKTYPE"),
    "CH_DESCRIPTION_TYPE": SimTypeInt(signed=False, label="CH_DESCRIPTION_TYPE"),
    "CIDA": SimStruct(OrderedDict((("cidl", SimTypeInt(signed=False, label="UInt32")), ("aoffset", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CIDA", pack=False, align=None),
    "CIE4ConnectionPoint": SimTypeBottom(label="CIE4ConnectionPoint"),
    "CIECHROMA": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("Y", SimTypeInt(signed=True, label="Int32")),)), name="CIECHROMA", pack=False, align=None),
    "CIEXYZ": SimStruct(OrderedDict((("ciexyzX", SimTypeInt(signed=True, label="Int32")), ("ciexyzY", SimTypeInt(signed=True, label="Int32")), ("ciexyzZ", SimTypeInt(signed=True, label="Int32")),)), name="CIEXYZ", pack=False, align=None),
    "CIEXYZTRIPLE": SimStruct(OrderedDict((("ciexyzRed", SimTypeRef("CIEXYZ", SimStruct)), ("ciexyzGreen", SimTypeRef("CIEXYZ", SimStruct)), ("ciexyzBlue", SimTypeRef("CIEXYZ", SimStruct)),)), name="CIEXYZTRIPLE", pack=False, align=None),
    "CIMTYPE_ENUMERATION": SimTypeInt(signed=False, label="CIMTYPE_ENUMERATION"),
    "CIP_STATUS": SimTypeInt(signed=False, label="CIP_STATUS"),
    "CI_STATE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cWordList", SimTypeInt(signed=False, label="UInt32")), ("cPersistentIndex", SimTypeInt(signed=False, label="UInt32")), ("cQueries", SimTypeInt(signed=False, label="UInt32")), ("cDocuments", SimTypeInt(signed=False, label="UInt32")), ("cFreshTest", SimTypeInt(signed=False, label="UInt32")), ("dwMergeProgress", SimTypeInt(signed=False, label="UInt32")), ("eState", SimTypeInt(signed=False, label="UInt32")), ("cFilteredDocuments", SimTypeInt(signed=False, label="UInt32")), ("cTotalDocuments", SimTypeInt(signed=False, label="UInt32")), ("cPendingScans", SimTypeInt(signed=False, label="UInt32")), ("dwIndexSize", SimTypeInt(signed=False, label="UInt32")), ("cUniqueKeys", SimTypeInt(signed=False, label="UInt32")), ("cSecQDocuments", SimTypeInt(signed=False, label="UInt32")), ("dwPropCacheSize", SimTypeInt(signed=False, label="UInt32")),)), name="CI_STATE", pack=False, align=None),
    "CKCL_DATA": SimStruct(OrderedDict((("NextLogEvent", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TAnalyzeString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TAnalyzeReturnType", SimTypeInt(signed=False, label="TANALYZE_RETURN")),)), name="CKCL_DATA", pack=False, align=None),
    "CKCL_LISTHEAD": SimStruct(OrderedDict((("LogEventListHead", SimTypePointer(SimTypeRef("CKCL_DATA", SimStruct), offset=0)), ("Heap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CKCL_LISTHEAD", pack=False, align=None),
    "CLAIMLIST": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("claims", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CLAIMLIST", pack=False, align=None),
    "CLAIMMEDIALABEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MediaLabelInfo", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pBuffer", "nBufferSize", "pLabelInfo"]), offset=0),
    "CLAIMMEDIALABELEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MediaLabelInfo", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pBuffer", "nBufferSize", "pLabelInfo", "LabelGuid"]), offset=0),
    "CLAIM_SECURITY_ATTRIBUTES_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("AttributeCount", SimTypeInt(signed=False, label="UInt32")), ("Attribute", SimUnion({"pAttributeV1": SimTypePointer(SimTypeRef("CLAIM_SECURITY_ATTRIBUTE_V1", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CLAIM_SECURITY_ATTRIBUTES_INFORMATION", pack=False, align=None),
    "CLAIM_SECURITY_ATTRIBUTE_FLAGS": SimTypeInt(signed=False, label="CLAIM_SECURITY_ATTRIBUTE_FLAGS"),
    "CLAIM_SECURITY_ATTRIBUTE_FQBN_VALUE": SimStruct(OrderedDict((("Version", SimTypeLongLong(signed=False, label="UInt64")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLAIM_SECURITY_ATTRIBUTE_FQBN_VALUE", pack=False, align=None),
    "CLAIM_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE": SimStruct(OrderedDict((("pValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValueLength", SimTypeInt(signed=False, label="UInt32")),)), name="CLAIM_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE", pack=False, align=None),
    "CLAIM_SECURITY_ATTRIBUTE_RELATIVE_V1": SimStruct(OrderedDict((("Name", SimTypeInt(signed=False, label="UInt32")), ("ValueType", SimTypeInt(signed=False, label="CLAIM_SECURITY_ATTRIBUTE_VALUE_TYPE")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="CLAIM_SECURITY_ATTRIBUTE_FLAGS")), ("ValueCount", SimTypeInt(signed=False, label="UInt32")), ("Values", SimUnion({"pInt64": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pUint64": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "ppString": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pFqbn": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pOctetString": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="<anon>", label="None")),)), name="CLAIM_SECURITY_ATTRIBUTE_RELATIVE_V1", pack=False, align=None),
    "CLAIM_SECURITY_ATTRIBUTE_V1": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ValueType", SimTypeInt(signed=False, label="CLAIM_SECURITY_ATTRIBUTE_VALUE_TYPE")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ValueCount", SimTypeInt(signed=False, label="UInt32")), ("Values", SimUnion({"pInt64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pUint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "ppString": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "pFqbn": SimTypePointer(SimTypeRef("CLAIM_SECURITY_ATTRIBUTE_FQBN_VALUE", SimStruct), offset=0), "pOctetString": SimTypePointer(SimTypeRef("CLAIM_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CLAIM_SECURITY_ATTRIBUTE_V1", pack=False, align=None),
    "CLAIM_SECURITY_ATTRIBUTE_VALUE_TYPE": SimTypeInt(signed=False, label="CLAIM_SECURITY_ATTRIBUTE_VALUE_TYPE"),
    "CLASSIC_EVENT_ID": SimStruct(OrderedDict((("EventGuid", SimTypeBottom(label="Guid")), ("Type", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 7)),)), name="CLASSIC_EVENT_ID", pack=False, align=None),
    "CLASS_MEDIA_CHANGE_CONTEXT": SimStruct(OrderedDict((("MediaChangeCount", SimTypeInt(signed=False, label="UInt32")), ("NewState", SimTypeInt(signed=False, label="UInt32")),)), name="CLASS_MEDIA_CHANGE_CONTEXT", pack=False, align=None),
    "CLCTL_CODES": SimTypeInt(signed=False, label="CLCTL_CODES"),
    "CLEANLOCALSTORAGE": SimStruct(OrderedDict((("pInterface", SimTypeBottom(label="IUnknown")), ("pStorage", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="CLEANLOCALSTORAGE", pack=False, align=None),
    "CLEAR_BLOCK": SimStruct(OrderedDict((("data", SimTypeArray(SimTypeChar(label="SByte"), 8)),)), name="CLEAR_BLOCK", pack=False, align=None),
    "CLEAR_COMM_ERROR_FLAGS": SimTypeInt(signed=False, label="CLEAR_COMM_ERROR_FLAGS"),
    "CLFS_BLOCK_ALLOCATION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbBufferLength", "pvUserContext"]), offset=0),
    "CLFS_BLOCK_DEALLOCATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvBuffer", "pvUserContext"]), offset=0),
    "CLFS_CONTEXT_MODE": SimTypeInt(signed=False, label="CLFS_CONTEXT_MODE"),
    "CLFS_FLAG": SimTypeInt(signed=False, label="CLFS_FLAG"),
    "CLFS_IOSTATS_CLASS": SimTypeInt(signed=False, label="CLFS_IOSTATS_CLASS"),
    "CLFS_LOG_ARCHIVE_MODE": SimTypeInt(signed=False, label="CLFS_LOG_ARCHIVE_MODE"),
    "CLFS_LOG_NAME_INFORMATION": SimStruct(OrderedDict((("NameLengthInBytes", SimTypeShort(signed=False, label="UInt16")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLFS_LOG_NAME_INFORMATION", pack=False, align=None),
    "CLFS_MGMT_CLIENT_REGISTRATION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("AdvanceTailCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLS_LSN", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogFile", "TargetLsn", "ClientData"]), offset=0)), ("AdvanceTailCallbackData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LogGrowthCompleteCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogFile", "OperationStatus", "LogIsPinned", "ClientData"]), offset=0)), ("LogGrowthCompleteCallbackData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LogUnpinnedCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogFile", "ClientData"]), offset=0)), ("LogUnpinnedCallbackData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CLFS_MGMT_CLIENT_REGISTRATION", pack=False, align=None),
    "CLFS_MGMT_NOTIFICATION": SimStruct(OrderedDict((("Notification", SimTypeInt(signed=False, label="CLFS_MGMT_NOTIFICATION_TYPE")), ("Lsn", SimTypeRef("CLS_LSN", SimStruct)), ("LogIsPinned", SimTypeShort(signed=False, label="UInt16")),)), name="CLFS_MGMT_NOTIFICATION", pack=False, align=None),
    "CLFS_MGMT_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="CLFS_MGMT_NOTIFICATION_TYPE"),
    "CLFS_MGMT_POLICY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("LengthInBytes", SimTypeInt(signed=False, label="UInt32")), ("PolicyFlags", SimTypeInt(signed=False, label="UInt32")), ("PolicyType", SimTypeInt(signed=False, label="CLFS_MGMT_POLICY_TYPE")), ("PolicyParameters", SimUnion({"MaximumSize": SimTypeRef("_MaximumSize_e__Struct", SimStruct), "MinimumSize": SimTypeRef("_MinimumSize_e__Struct", SimStruct), "NewContainerSize": SimTypeRef("_NewContainerSize_e__Struct", SimStruct), "GrowthRate": SimTypeRef("_GrowthRate_e__Struct", SimStruct), "LogTail": SimTypeRef("_LogTail_e__Struct", SimStruct), "AutoShrink": SimTypeRef("_AutoShrink_e__Struct", SimStruct), "AutoGrow": SimTypeRef("_AutoGrow_e__Struct", SimStruct), "NewContainerPrefix": SimTypeRef("_NewContainerPrefix_e__Struct", SimStruct), "NewContainerSuffix": SimTypeRef("_NewContainerSuffix_e__Struct", SimStruct), "NewContainerExtension": SimTypeRef("_NewContainerExtension_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CLFS_MGMT_POLICY", pack=False, align=None),
    "CLFS_MGMT_POLICY_TYPE": SimTypeInt(signed=False, label="CLFS_MGMT_POLICY_TYPE"),
    "CLFS_NODE_ID": SimStruct(OrderedDict((("cType", SimTypeInt(signed=False, label="UInt32")), ("cbNode", SimTypeInt(signed=False, label="UInt32")),)), name="CLFS_NODE_ID", pack=False, align=None),
    "CLFS_PHYSICAL_LSN_INFORMATION": SimStruct(OrderedDict((("StreamIdentifier", SimTypeChar(label="Byte")), ("VirtualLsn", SimTypeRef("CLS_LSN", SimStruct)), ("PhysicalLsn", SimTypeRef("CLS_LSN", SimStruct)),)), name="CLFS_PHYSICAL_LSN_INFORMATION", pack=False, align=None),
    "CLFS_STREAM_ID_INFORMATION": SimStruct(OrderedDict((("StreamIdentifier", SimTypeChar(label="Byte")),)), name="CLFS_STREAM_ID_INFORMATION", pack=False, align=None),
    "CLIENTCREATESTRUCT": SimStruct(OrderedDict((("hWindowMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("idFirstChild", SimTypeInt(signed=False, label="UInt32")),)), name="CLIENTCREATESTRUCT", pack=False, align=None),
    "CLIENTPROPERTIES": SimTypeInt(signed=False, label="CLIENTPROPERTIES"),
    "CLIENT_CALL_RETURN": SimUnion({"Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Simple": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"),
    "CLIENT_DISPLAY": SimStruct(OrderedDict((("HorizontalResolution", SimTypeInt(signed=False, label="UInt32")), ("VerticalResolution", SimTypeInt(signed=False, label="UInt32")), ("ColorDepth", SimTypeInt(signed=False, label="UInt32")),)), name="CLIENT_DISPLAY", pack=False, align=None),
    "CLIENT_ID": SimStruct(OrderedDict((("UniqueProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UniqueThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CLIENT_ID", pack=False, align=None),
    "CLIENT_MESSAGE_TYPE": SimTypeInt(signed=False, label="CLIENT_MESSAGE_TYPE"),
    "CLIPBOARDFORMAT": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("cf", SimTypeShort(signed=False, label="UInt16")),)), name="CLIPBOARDFORMAT", pack=False, align=None),
    "CLIPBOARD_FORMAT": SimTypeInt(signed=False, label="CLIPBOARD_FORMAT"),
    "CLIPDATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulClipFmt", SimTypeInt(signed=True, label="Int32")), ("pClipData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CLIPDATA", pack=False, align=None),
    "CLIPLINE": SimStruct(OrderedDict((("ptfxA", SimTypeRef("POINTFIX", SimStruct)), ("ptfxB", SimTypeRef("POINTFIX", SimStruct)), ("lStyleState", SimTypeInt(signed=True, label="Int32")), ("c", SimTypeInt(signed=False, label="UInt32")), ("arun", SimTypePointer(SimTypeRef("RUN", SimStruct), offset=0)),)), name="CLIPLINE", pack=False, align=None),
    "CLIPOBJ": SimStruct(OrderedDict((("iUniq", SimTypeInt(signed=False, label="UInt32")), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("iDComplexity", SimTypeChar(label="Byte")), ("iFComplexity", SimTypeChar(label="Byte")), ("iMode", SimTypeChar(label="Byte")), ("fjOptions", SimTypeChar(label="Byte")),)), name="CLIPOBJ", pack=False, align=None),
    "CLOCKPARTS": SimTypeInt(signed=False, label="CLOCKPARTS"),
    "CLOCKSTATES": SimTypeInt(signed=False, label="CLOCKSTATES"),
    "CLOSEBUTTONSTATES": SimTypeInt(signed=False, label="CLOSEBUTTONSTATES"),
    "CLOSESTATES": SimTypeInt(signed=False, label="CLOSESTATES"),
    "CLRCreateInstanceFnPtr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["clsid", "riid", "ppInterface"]), offset=0),
    "CLRES_CALLBACK_FUNCTION_TABLE": SimStruct(OrderedDict((("LogEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0)), ("SetResourceStatusEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESOURCE_STATUS_EX", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "ResourceStatus"]), offset=0)), ("SetResourceLockedMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "LockedModeEnabled", "LockedModeReason"]), offset=0)), ("SignalFailure", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="FAILURE_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "FailureType", "ApplicationSpecificErrorCode"]), offset=0)), ("SetResourceInMemoryNodeLocalProperties", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "propertyListBuffer", "propertyListBufferSize"]), offset=0)), ("EndControlCall", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "status"]), offset=0)), ("EndTypeControlCall", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "status"]), offset=0)), ("ExtendControlCall", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "newTimeoutInMs"]), offset=0)), ("ExtendTypeControlCall", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "newTimeoutInMs"]), offset=0)), ("RaiseResTypeNotification", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceType", "pPayload", "payloadSize"]), offset=0)), ("ChangeResourceProcessForDumps", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["resource", "processName", "processId", "isAdd"]), offset=0)), ("ChangeResTypeProcessForDumps", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["resourceTypeName", "processName", "processId", "isAdd"]), offset=0)), ("SetInternalState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CLUSTER_RESOURCE_APPLICATION_STATE"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "stateType", "active"]), offset=0)), ("SetResourceLockedModeEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "LockedModeEnabled", "LockedModeReason", "LockedModeFlags"]), offset=0)), ("RequestDump", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "DumpDueToCallInProgress", "DumpDelayInMs"]), offset=0)), ("SetResourceWprPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "WprPolicyFlags"]), offset=0)), ("ArmWprWatchdogForCurrentResourceCall", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "TimeoutInMs"]), offset=0)),)), name="CLRES_CALLBACK_FUNCTION_TABLE", pack=False, align=None),
    "CLRES_FUNCTION_TABLE": SimStruct(OrderedDict((("TableSize", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"V1Functions": SimTypeRef("CLRES_V1_FUNCTIONS", SimStruct), "V2Functions": SimTypeRef("CLRES_V2_FUNCTIONS", SimStruct), "V3Functions": SimTypeRef("CLRES_V3_FUNCTIONS", SimStruct), "V4Functions": SimTypeRef("CLRES_V4_FUNCTIONS", SimStruct)}, name="<anon>", label="None")),)), name="CLRES_FUNCTION_TABLE", pack=False, align=None),
    "CLRES_V1_FUNCTIONS": SimStruct(OrderedDict((("Open", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ResourceName", "ResourceKey", "ResourceHandle"]), offset=0)), ("Close", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("Online", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "EventHandle"]), offset=0)), ("Offline", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0)), ("Terminate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("LooksAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("IsAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("Arbitrate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "LostQuorumResource"]), offset=0)), ("Release", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0)), ("ResourceControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0)), ("ResourceTypeControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0)),)), name="CLRES_V1_FUNCTIONS", pack=False, align=None),
    "CLRES_V2_FUNCTIONS": SimStruct(OrderedDict((("Open", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ResourceName", "ResourceKey", "ResourceHandle", "OpenFlags"]), offset=0)), ("Close", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("Online", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "EventHandle", "OnlineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0)), ("Offline", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "DestinationNodeName", "OfflineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0)), ("Terminate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("LooksAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("IsAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("Arbitrate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "LostQuorumResource"]), offset=0)), ("Release", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0)), ("ResourceControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0)), ("ResourceTypeControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0)), ("Cancel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "CancelFlags_RESERVED"]), offset=0)),)), name="CLRES_V2_FUNCTIONS", pack=False, align=None),
    "CLRES_V3_FUNCTIONS": SimStruct(OrderedDict((("Open", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ResourceName", "ResourceKey", "ResourceHandle", "OpenFlags"]), offset=0)), ("Close", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("Online", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "EventHandle", "OnlineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0)), ("Offline", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "DestinationNodeName", "OfflineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0)), ("Terminate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("LooksAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("IsAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("Arbitrate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "LostQuorumResource"]), offset=0)), ("Release", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0)), ("BeginResourceControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0)), ("BeginResourceTypeControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0)), ("Cancel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "CancelFlags_RESERVED"]), offset=0)),)), name="CLRES_V3_FUNCTIONS", pack=False, align=None),
    "CLRES_V4_FUNCTIONS": SimStruct(OrderedDict((("Open", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ResourceName", "ResourceKey", "ResourceHandle", "OpenFlags"]), offset=0)), ("Close", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("Online", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "EventHandle", "OnlineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0)), ("Offline", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "DestinationNodeName", "OfflineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0)), ("Terminate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)), ("LooksAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("IsAlive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0)), ("Arbitrate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "LostQuorumResource"]), offset=0)), ("Release", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0)), ("BeginResourceControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0)), ("BeginResourceTypeControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0)), ("Cancel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "CancelFlags_RESERVED"]), offset=0)), ("BeginResourceControlAsUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "TokenHandle", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0)), ("BeginResourceTypeControlAsUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "TokenHandle", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0)),)), name="CLRES_V4_FUNCTIONS", pack=False, align=None),
    "CLR_DEBUGGING_PROCESS_FLAGS": SimTypeInt(signed=False, label="CLR_DEBUGGING_PROCESS_FLAGS"),
    "CLR_DEBUGGING_VERSION": SimStruct(OrderedDict((("wStructVersion", SimTypeShort(signed=False, label="UInt16")), ("wMajor", SimTypeShort(signed=False, label="UInt16")), ("wMinor", SimTypeShort(signed=False, label="UInt16")), ("wBuild", SimTypeShort(signed=False, label="UInt16")), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="CLR_DEBUGGING_VERSION", pack=False, align=None),
    "CLSCTX": SimTypeInt(signed=False, label="CLSCTX"),
    "CLSIDDATA": SimStruct(OrderedDict((("m_clsid", SimTypeBottom(label="Guid")), ("m_cReferences", SimTypeInt(signed=False, label="UInt32")), ("m_cBound", SimTypeInt(signed=False, label="UInt32")), ("m_cPooled", SimTypeInt(signed=False, label="UInt32")), ("m_cInCall", SimTypeInt(signed=False, label="UInt32")), ("m_dwRespTime", SimTypeInt(signed=False, label="UInt32")), ("m_cCallsCompleted", SimTypeInt(signed=False, label="UInt32")), ("m_cCallsFailed", SimTypeInt(signed=False, label="UInt32")),)), name="CLSIDDATA", pack=False, align=None),
    "CLSIDDATA2": SimStruct(OrderedDict((("m_clsid", SimTypeBottom(label="Guid")), ("m_appid", SimTypeBottom(label="Guid")), ("m_partid", SimTypeBottom(label="Guid")), ("m_pwszAppName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pwszCtxName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_eAppType", SimTypeInt(signed=False, label="COMPLUS_APPTYPE")), ("m_cReferences", SimTypeInt(signed=False, label="UInt32")), ("m_cBound", SimTypeInt(signed=False, label="UInt32")), ("m_cPooled", SimTypeInt(signed=False, label="UInt32")), ("m_cInCall", SimTypeInt(signed=False, label="UInt32")), ("m_dwRespTime", SimTypeInt(signed=False, label="UInt32")), ("m_cCallsCompleted", SimTypeInt(signed=False, label="UInt32")), ("m_cCallsFailed", SimTypeInt(signed=False, label="UInt32")),)), name="CLSIDDATA2", pack=False, align=None),
    "CLSID_RESOLUTION_FLAGS": SimTypeInt(signed=False, label="CLSID_RESOLUTION_FLAGS"),
    "CLS_ARCHIVE_DESCRIPTOR": SimStruct(OrderedDict((("coffLow", SimTypeLongLong(signed=False, label="UInt64")), ("coffHigh", SimTypeLongLong(signed=False, label="UInt64")), ("infoContainer", SimTypeRef("CLS_CONTAINER_INFORMATION", SimStruct)),)), name="CLS_ARCHIVE_DESCRIPTOR", pack=False, align=None),
    "CLS_CONTAINER_INFORMATION": SimStruct(OrderedDict((("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=False, label="UInt64")), ("LastAccessTime", SimTypeLongLong(signed=False, label="UInt64")), ("LastWriteTime", SimTypeLongLong(signed=False, label="UInt64")), ("ContainerSize", SimTypeLongLong(signed=True, label="Int64")), ("FileNameActualLength", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("State", SimTypeInt(signed=False, label="UInt32")), ("PhysicalContainerId", SimTypeInt(signed=False, label="UInt32")), ("LogicalContainerId", SimTypeInt(signed=False, label="UInt32")),)), name="CLS_CONTAINER_INFORMATION", pack=False, align=None),
    "CLS_CONTEXT_MODE": SimTypeInt(signed=False, label="CLS_CONTEXT_MODE"),
    "CLS_INFORMATION": SimStruct(OrderedDict((("TotalAvailable", SimTypeLongLong(signed=True, label="Int64")), ("CurrentAvailable", SimTypeLongLong(signed=True, label="Int64")), ("TotalReservation", SimTypeLongLong(signed=True, label="Int64")), ("BaseFileSize", SimTypeLongLong(signed=False, label="UInt64")), ("ContainerSize", SimTypeLongLong(signed=False, label="UInt64")), ("TotalContainers", SimTypeInt(signed=False, label="UInt32")), ("FreeContainers", SimTypeInt(signed=False, label="UInt32")), ("TotalClients", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("FlushThreshold", SimTypeInt(signed=False, label="UInt32")), ("SectorSize", SimTypeInt(signed=False, label="UInt32")), ("MinArchiveTailLsn", SimTypeRef("CLS_LSN", SimStruct)), ("BaseLsn", SimTypeRef("CLS_LSN", SimStruct)), ("LastFlushedLsn", SimTypeRef("CLS_LSN", SimStruct)), ("LastLsn", SimTypeRef("CLS_LSN", SimStruct)), ("RestartLsn", SimTypeRef("CLS_LSN", SimStruct)), ("Identity", SimTypeBottom(label="Guid")),)), name="CLS_INFORMATION", pack=False, align=None),
    "CLS_IOSTATS_CLASS": SimTypeInt(signed=False, label="CLS_IOSTATS_CLASS"),
    "CLS_IO_STATISTICS": SimStruct(OrderedDict((("hdrIoStats", SimTypeRef("CLS_IO_STATISTICS_HEADER", SimStruct)), ("cFlush", SimTypeLongLong(signed=False, label="UInt64")), ("cbFlush", SimTypeLongLong(signed=False, label="UInt64")), ("cMetaFlush", SimTypeLongLong(signed=False, label="UInt64")), ("cbMetaFlush", SimTypeLongLong(signed=False, label="UInt64")),)), name="CLS_IO_STATISTICS", pack=False, align=None),
    "CLS_IO_STATISTICS_HEADER": SimStruct(OrderedDict((("ubMajorVersion", SimTypeChar(label="Byte")), ("ubMinorVersion", SimTypeChar(label="Byte")), ("eStatsClass", SimTypeInt(signed=False, label="CLFS_IOSTATS_CLASS")), ("cbLength", SimTypeShort(signed=False, label="UInt16")), ("coffData", SimTypeInt(signed=False, label="UInt32")),)), name="CLS_IO_STATISTICS_HEADER", pack=False, align=None),
    "CLS_LOG_INFORMATION_CLASS": SimTypeInt(signed=False, label="CLS_LOG_INFORMATION_CLASS"),
    "CLS_LSN": SimStruct(OrderedDict((("Internal", SimTypeLongLong(signed=False, label="UInt64")),)), name="CLS_LSN", pack=False, align=None),
    "CLS_SCAN_CONTEXT": SimStruct(OrderedDict((("cidNode", SimTypeRef("CLFS_NODE_ID", SimStruct)), ("hLog", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cIndex", SimTypeInt(signed=False, label="UInt32")), ("cContainers", SimTypeInt(signed=False, label="UInt32")), ("cContainersReturned", SimTypeInt(signed=False, label="UInt32")), ("eScanMode", SimTypeChar(label="Byte")), ("pinfoContainer", SimTypePointer(SimTypeRef("CLS_CONTAINER_INFORMATION", SimStruct), offset=0)),)), name="CLS_SCAN_CONTEXT", pack=False, align=None),
    "CLS_WRITE_ENTRY": SimStruct(OrderedDict((("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ByteLength", SimTypeInt(signed=False, label="UInt32")),)), name="CLS_WRITE_ENTRY", pack=False, align=None),
    "CLUADMEX_OBJECT_TYPE": SimTypeInt(signed=False, label="CLUADMEX_OBJECT_TYPE"),
    "CLUSAPI_REASON_HANDLER": SimStruct(OrderedDict((("lpParameter", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfnHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpParameter", "hCluster", "szReason", "lpSize"]), offset=0)),)), name="CLUSAPI_REASON_HANDLER", pack=False, align=None),
    "CLUSCTL_AFFINITYRULE_CODES": SimTypeInt(signed=False, label="CLUSCTL_AFFINITYRULE_CODES"),
    "CLUSCTL_CLUSTER_CODES": SimTypeInt(signed=False, label="CLUSCTL_CLUSTER_CODES"),
    "CLUSCTL_GROUPSET_CODES": SimTypeInt(signed=False, label="CLUSCTL_GROUPSET_CODES"),
    "CLUSCTL_GROUP_CODES": SimTypeInt(signed=False, label="CLUSCTL_GROUP_CODES"),
    "CLUSCTL_GROUP_GET_LAST_MOVE_TIME_OUTPUT": SimStruct(OrderedDict((("GetTickCount64", SimTypeLongLong(signed=False, label="UInt64")), ("GetSystemTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("NodeId", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSCTL_GROUP_GET_LAST_MOVE_TIME_OUTPUT", pack=False, align=None),
    "CLUSCTL_NETINTERFACE_CODES": SimTypeInt(signed=False, label="CLUSCTL_NETINTERFACE_CODES"),
    "CLUSCTL_NETWORK_CODES": SimTypeInt(signed=False, label="CLUSCTL_NETWORK_CODES"),
    "CLUSCTL_NODE_CODES": SimTypeInt(signed=False, label="CLUSCTL_NODE_CODES"),
    "CLUSCTL_RESOURCE_CODES": SimTypeInt(signed=False, label="CLUSCTL_RESOURCE_CODES"),
    "CLUSCTL_RESOURCE_STATE_CHANGE_REASON_STRUCT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("eReason", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE_CHANGE_REASON")),)), name="CLUSCTL_RESOURCE_STATE_CHANGE_REASON_STRUCT", pack=False, align=None),
    "CLUSCTL_RESOURCE_TYPE_CODES": SimTypeInt(signed=False, label="CLUSCTL_RESOURCE_TYPE_CODES"),
    "CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_INPUT": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidPoolFilter", SimTypeBottom(label="Guid")),)), name="CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS_EX2_INPUT", pack=False, align=None),
    "CLUSGROUP_TYPE": SimTypeInt(signed=False, label="CLUSGROUP_TYPE"),
    "CLUSION_REASON": SimTypeInt(signed=False, label="CLUSION_REASON"),
    "CLUSPROP_BINARY": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("rgb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CLUSPROP_BINARY", pack=False, align=None),
    "CLUSPROP_BUFFER_HELPER": SimUnion({"pb": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pw": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pdw": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pl": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "psz": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pList": SimTypePointer(SimTypeRef("CLUSPROP_LIST", SimStruct), offset=0), "pSyntax": SimTypePointer(SimUnion({"dw": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("wFormat", SimTypeShort(signed=False, label="UInt16")), ("wType", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), offset=0), "pName": SimTypePointer(SimTypeRef("CLUSPROP_SZ", SimStruct), offset=0), "pValue": SimTypePointer(SimTypeRef("CLUSPROP_VALUE", SimStruct), offset=0), "pBinaryValue": SimTypePointer(SimTypeRef("CLUSPROP_BINARY", SimStruct), offset=0), "pWordValue": SimTypePointer(SimTypeRef("CLUSPROP_WORD", SimStruct), offset=0), "pDwordValue": SimTypePointer(SimTypeRef("CLUSPROP_DWORD", SimStruct), offset=0), "pLongValue": SimTypePointer(SimTypeRef("CLUSPROP_LONG", SimStruct), offset=0), "pULargeIntegerValue": SimTypePointer(SimTypeRef("CLUSPROP_ULARGE_INTEGER", SimStruct), offset=0), "pLargeIntegerValue": SimTypePointer(SimTypeRef("CLUSPROP_LARGE_INTEGER", SimStruct), offset=0), "pStringValue": SimTypePointer(SimTypeRef("CLUSPROP_SZ", SimStruct), offset=0), "pMultiSzValue": SimTypePointer(SimTypeRef("CLUSPROP_SZ", SimStruct), offset=0), "pSecurityDescriptor": SimTypePointer(SimTypeRef("CLUSPROP_SECURITY_DESCRIPTOR", SimStruct), offset=0), "pResourceClassValue": SimTypePointer(SimTypeRef("CLUSPROP_RESOURCE_CLASS", SimStruct), offset=0), "pResourceClassInfoValue": SimTypePointer(SimTypeRef("CLUSPROP_RESOURCE_CLASS_INFO", SimStruct), offset=0), "pDiskSignatureValue": SimTypePointer(SimTypeRef("CLUSPROP_DWORD", SimStruct), offset=0), "pScsiAddressValue": SimTypePointer(SimTypeRef("CLUSPROP_SCSI_ADDRESS", SimStruct), offset=0), "pDiskNumberValue": SimTypePointer(SimTypeRef("CLUSPROP_DWORD", SimStruct), offset=0), "pPartitionInfoValue": SimTypePointer(SimTypeRef("CLUSPROP_PARTITION_INFO", SimStruct), offset=0), "pRequiredDependencyValue": SimTypePointer(SimUnion({"Value": SimTypeRef("CLUSPROP_VALUE", SimStruct), "ResClass": SimTypeRef("CLUSPROP_RESOURCE_CLASS", SimStruct), "ResTypeName": SimTypeRef("CLUSPROP_SZ", SimStruct)}, name="<anon>", label="None"), offset=0), "pPartitionInfoValueEx": SimTypePointer(SimTypeRef("CLUSPROP_PARTITION_INFO_EX", SimStruct), offset=0), "pPartitionInfoValueEx2": SimTypePointer(SimTypeRef("CLUSPROP_PARTITION_INFO_EX2", SimStruct), offset=0), "pFileTimeValue": SimTypePointer(SimTypeRef("CLUSPROP_FILETIME", SimStruct), offset=0)}, name="<anon>", label="None"),
    "CLUSPROP_DWORD": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("dw", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSPROP_DWORD", pack=False, align=None),
    "CLUSPROP_FILETIME": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("ft", SimTypeRef("FILETIME", SimStruct)),)), name="CLUSPROP_FILETIME", pack=False, align=None),
    "CLUSPROP_FTSET_INFO": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("Base2", SimTypeRef("CLUS_FTSET_INFO", SimStruct)),)), name="CLUSPROP_FTSET_INFO", pack=False, align=None),
    "CLUSPROP_IPADDR_ENABLENETBIOS": SimTypeInt(signed=False, label="CLUSPROP_IPADDR_ENABLENETBIOS"),
    "CLUSPROP_LARGE_INTEGER": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("li", SimTypeLongLong(signed=True, label="Int64")),)), name="CLUSPROP_LARGE_INTEGER", pack=False, align=None),
    "CLUSPROP_LIST": SimStruct(OrderedDict((("nPropertyCount", SimTypeInt(signed=False, label="UInt32")), ("PropertyName", SimTypeRef("CLUSPROP_SZ", SimStruct)),)), name="CLUSPROP_LIST", pack=False, align=None),
    "CLUSPROP_LONG": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("l", SimTypeInt(signed=True, label="Int32")),)), name="CLUSPROP_LONG", pack=False, align=None),
    "CLUSPROP_PARTITION_INFO": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("Base2", SimTypeRef("CLUS_PARTITION_INFO", SimStruct)),)), name="CLUSPROP_PARTITION_INFO", pack=False, align=None),
    "CLUSPROP_PARTITION_INFO_EX": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("Base2", SimTypeRef("CLUS_PARTITION_INFO_EX", SimStruct)),)), name="CLUSPROP_PARTITION_INFO_EX", pack=False, align=None),
    "CLUSPROP_PARTITION_INFO_EX2": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_PARTITION_INFO_EX", SimStruct)), ("Base2", SimTypeRef("CLUS_PARTITION_INFO_EX2", SimStruct)),)), name="CLUSPROP_PARTITION_INFO_EX2", pack=False, align=None),
    "CLUSPROP_PIFLAGS": SimTypeInt(signed=False, label="CLUSPROP_PIFLAGS"),
    "CLUSPROP_REQUIRED_DEPENDENCY": SimUnion({"Value": SimTypeRef("CLUSPROP_VALUE", SimStruct), "ResClass": SimTypeRef("CLUSPROP_RESOURCE_CLASS", SimStruct), "ResTypeName": SimTypeRef("CLUSPROP_SZ", SimStruct)}, name="<anon>", label="None"),
    "CLUSPROP_RESOURCE_CLASS": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("rc", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_CLASS")),)), name="CLUSPROP_RESOURCE_CLASS", pack=False, align=None),
    "CLUSPROP_RESOURCE_CLASS_INFO": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("Base2", SimTypeRef("CLUS_RESOURCE_CLASS_INFO", SimStruct)),)), name="CLUSPROP_RESOURCE_CLASS_INFO", pack=False, align=None),
    "CLUSPROP_SCSI_ADDRESS": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("Base2", SimTypeRef("CLUS_SCSI_ADDRESS", SimStruct)),)), name="CLUSPROP_SCSI_ADDRESS", pack=False, align=None),
    "CLUSPROP_SECURITY_DESCRIPTOR": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("Anonymous", SimUnion({"sd": SimTypeRef("SECURITY_DESCRIPTOR_RELATIVE", SimStruct), "rgbSecurityDescriptor": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="CLUSPROP_SECURITY_DESCRIPTOR", pack=False, align=None),
    "CLUSPROP_SYNTAX": SimUnion({"dw": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("wFormat", SimTypeShort(signed=False, label="UInt16")), ("wType", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "CLUSPROP_SZ": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("sz", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUSPROP_SZ", pack=False, align=None),
    "CLUSPROP_ULARGE_INTEGER": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("li", SimTypeLongLong(signed=False, label="UInt64")),)), name="CLUSPROP_ULARGE_INTEGER", pack=False, align=None),
    "CLUSPROP_VALUE": SimStruct(OrderedDict((("Syntax", SimUnion({"dw": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("wFormat", SimTypeShort(signed=False, label="UInt16")), ("wType", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("cbLength", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSPROP_VALUE", pack=False, align=None),
    "CLUSPROP_WORD": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSPROP_VALUE", SimStruct)), ("w", SimTypeShort(signed=False, label="UInt16")),)), name="CLUSPROP_WORD", pack=False, align=None),
    "CLUSTERSET_OBJECT_TYPE": SimTypeInt(signed=False, label="CLUSTERSET_OBJECT_TYPE"),
    "CLUSTERVERSIONINFO": SimStruct(OrderedDict((("dwVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("BuildNumber", SimTypeShort(signed=False, label="UInt16")), ("szVendorId", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szCSDVersion", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwClusterHighestVersion", SimTypeInt(signed=False, label="UInt32")), ("dwClusterLowestVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSTERVERSIONINFO", pack=False, align=None),
    "CLUSTERVERSIONINFO_NT4": SimStruct(OrderedDict((("dwVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("BuildNumber", SimTypeShort(signed=False, label="UInt16")), ("szVendorId", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szCSDVersion", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="CLUSTERVERSIONINFO_NT4", pack=False, align=None),
    "CLUSTER_AVAILABILITY_SET_CONFIG": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwUpdateDomains", SimTypeInt(signed=False, label="UInt32")), ("dwFaultDomains", SimTypeInt(signed=False, label="UInt32")), ("bReserveSpareNode", SimTypeInt(signed=True, label="Int32")),)), name="CLUSTER_AVAILABILITY_SET_CONFIG", pack=False, align=None),
    "CLUSTER_BATCH_COMMAND": SimStruct(OrderedDict((("Command", SimTypeInt(signed=False, label="CLUSTER_REG_COMMAND")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("wzName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSTER_BATCH_COMMAND", pack=False, align=None),
    "CLUSTER_CHANGE": SimTypeInt(signed=False, label="CLUSTER_CHANGE"),
    "CLUSTER_CHANGE_CLUSTER_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_CLUSTER_V2"),
    "CLUSTER_CHANGE_GROUPSET_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_GROUPSET_V2"),
    "CLUSTER_CHANGE_GROUP_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_GROUP_V2"),
    "CLUSTER_CHANGE_NETINTERFACE_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_NETINTERFACE_V2"),
    "CLUSTER_CHANGE_NETWORK_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_NETWORK_V2"),
    "CLUSTER_CHANGE_NODE_UPGRADE_PHASE_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_NODE_UPGRADE_PHASE_V2"),
    "CLUSTER_CHANGE_NODE_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_NODE_V2"),
    "CLUSTER_CHANGE_QUORUM_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_QUORUM_V2"),
    "CLUSTER_CHANGE_REGISTRY_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_REGISTRY_V2"),
    "CLUSTER_CHANGE_RESOURCE_TYPE_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_RESOURCE_TYPE_V2"),
    "CLUSTER_CHANGE_RESOURCE_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_RESOURCE_V2"),
    "CLUSTER_CHANGE_SHARED_VOLUME_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_SHARED_VOLUME_V2"),
    "CLUSTER_CHANGE_SPACEPORT_V2": SimTypeInt(signed=False, label="CLUSTER_CHANGE_SPACEPORT_V2"),
    "CLUSTER_CLOUD_TYPE": SimTypeInt(signed=False, label="CLUSTER_CLOUD_TYPE"),
    "CLUSTER_CONTROL_OBJECT": SimTypeInt(signed=False, label="CLUSTER_CONTROL_OBJECT"),
    "CLUSTER_CREATE_GROUP_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("groupType", SimTypeInt(signed=False, label="CLUSGROUP_TYPE")),)), name="CLUSTER_CREATE_GROUP_INFO", pack=False, align=None),
    "CLUSTER_CSV_VOLUME_FAULT_STATE": SimTypeInt(signed=False, label="CLUSTER_CSV_VOLUME_FAULT_STATE"),
    "CLUSTER_ENUM": SimTypeInt(signed=False, label="CLUSTER_ENUM"),
    "CLUSTER_ENUM_ITEM": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("cbId", SimTypeInt(signed=False, label="UInt32")), ("lpszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbName", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUSTER_ENUM_ITEM", pack=False, align=None),
    "CLUSTER_GROUP_AUTOFAILBACK_TYPE": SimTypeInt(signed=False, label="CLUSTER_GROUP_AUTOFAILBACK_TYPE"),
    "CLUSTER_GROUP_ENUM": SimTypeInt(signed=False, label="CLUSTER_GROUP_ENUM"),
    "CLUSTER_GROUP_ENUM_ITEM": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbId", SimTypeInt(signed=False, label="UInt32")), ("lpszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbName", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("state", SimTypeInt(signed=False, label="CLUSTER_GROUP_STATE")), ("cbOwnerNode", SimTypeInt(signed=False, label="UInt32")), ("lpszOwnerNode", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbProperties", SimTypeInt(signed=False, label="UInt32")), ("pProperties", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbRoProperties", SimTypeInt(signed=False, label="UInt32")), ("pRoProperties", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CLUSTER_GROUP_ENUM_ITEM", pack=False, align=None),
    "CLUSTER_GROUP_PRIORITY": SimTypeInt(signed=False, label="CLUSTER_GROUP_PRIORITY"),
    "CLUSTER_GROUP_STATE": SimTypeInt(signed=False, label="CLUSTER_GROUP_STATE"),
    "CLUSTER_HEALTH_FAULT": SimStruct(OrderedDict((("Id", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ErrorType", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Provider", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSTER_HEALTH_FAULT", pack=False, align=None),
    "CLUSTER_HEALTH_FAULT_ARRAY": SimStruct(OrderedDict((("numFaults", SimTypeInt(signed=False, label="UInt32")), ("faults", SimTypePointer(SimTypeRef("CLUSTER_HEALTH_FAULT", SimStruct), offset=0)),)), name="CLUSTER_HEALTH_FAULT_ARRAY", pack=False, align=None),
    "CLUSTER_IP_ENTRY": SimStruct(OrderedDict((("lpszIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSTER_IP_ENTRY", pack=False, align=None),
    "CLUSTER_MEMBERSHIP_INFO": SimStruct(OrderedDict((("HasQuorum", SimTypeInt(signed=True, label="Int32")), ("UpnodesSize", SimTypeInt(signed=False, label="UInt32")), ("Upnodes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CLUSTER_MEMBERSHIP_INFO", pack=False, align=None),
    "CLUSTER_MGMT_POINT_RESTYPE": SimTypeInt(signed=False, label="CLUSTER_MGMT_POINT_RESTYPE"),
    "CLUSTER_MGMT_POINT_TYPE": SimTypeInt(signed=False, label="CLUSTER_MGMT_POINT_TYPE"),
    "CLUSTER_NETINTERFACE_STATE": SimTypeInt(signed=False, label="CLUSTER_NETINTERFACE_STATE"),
    "CLUSTER_NETWORK_ENUM": SimTypeInt(signed=False, label="CLUSTER_NETWORK_ENUM"),
    "CLUSTER_NETWORK_ROLE": SimTypeInt(signed=False, label="CLUSTER_NETWORK_ROLE"),
    "CLUSTER_NETWORK_STATE": SimTypeInt(signed=False, label="CLUSTER_NETWORK_STATE"),
    "CLUSTER_NODE_DRAIN_STATUS": SimTypeInt(signed=False, label="CLUSTER_NODE_DRAIN_STATUS"),
    "CLUSTER_NODE_ENUM": SimTypeInt(signed=False, label="CLUSTER_NODE_ENUM"),
    "CLUSTER_NODE_RESUME_FAILBACK_TYPE": SimTypeInt(signed=False, label="CLUSTER_NODE_RESUME_FAILBACK_TYPE"),
    "CLUSTER_NODE_STATE": SimTypeInt(signed=False, label="CLUSTER_NODE_STATE"),
    "CLUSTER_NODE_STATUS": SimTypeInt(signed=False, label="CLUSTER_NODE_STATUS"),
    "CLUSTER_NOTIFICATIONS_VERSION": SimTypeInt(signed=False, label="CLUSTER_NOTIFICATIONS_VERSION"),
    "CLUSTER_OBJECT_TYPE": SimTypeInt(signed=False, label="CLUSTER_OBJECT_TYPE"),
    "CLUSTER_PROPERTY_FORMAT": SimTypeInt(signed=False, label="CLUSTER_PROPERTY_FORMAT"),
    "CLUSTER_PROPERTY_SYNTAX": SimTypeInt(signed=False, label="CLUSTER_PROPERTY_SYNTAX"),
    "CLUSTER_PROPERTY_TYPE": SimTypeInt(signed=False, label="CLUSTER_PROPERTY_TYPE"),
    "CLUSTER_QUORUM_TYPE": SimTypeInt(signed=False, label="CLUSTER_QUORUM_TYPE"),
    "CLUSTER_QUORUM_VALUE": SimTypeInt(signed=False, label="CLUSTER_QUORUM_VALUE"),
    "CLUSTER_RANGE": SimStruct(OrderedDict((("StartingCluster", SimTypeLongLong(signed=True, label="Int64")), ("ClusterCount", SimTypeLongLong(signed=True, label="Int64")),)), name="CLUSTER_RANGE", pack=False, align=None),
    "CLUSTER_READ_BATCH_COMMAND": SimStruct(OrderedDict((("Command", SimTypeInt(signed=False, label="CLUSTER_REG_COMMAND")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("wzSubkeyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wzValueName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSTER_READ_BATCH_COMMAND", pack=False, align=None),
    "CLUSTER_REG_COMMAND": SimTypeInt(signed=False, label="CLUSTER_REG_COMMAND"),
    "CLUSTER_RESOURCE_APPLICATION_STATE": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_APPLICATION_STATE"),
    "CLUSTER_RESOURCE_CLASS": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_CLASS"),
    "CLUSTER_RESOURCE_CREATE_FLAGS": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_CREATE_FLAGS"),
    "CLUSTER_RESOURCE_EMBEDDED_FAILURE_ACTION": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_EMBEDDED_FAILURE_ACTION"),
    "CLUSTER_RESOURCE_ENUM": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_ENUM"),
    "CLUSTER_RESOURCE_ENUM_ITEM": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbId", SimTypeInt(signed=False, label="UInt32")), ("lpszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbName", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbOwnerGroupName", SimTypeInt(signed=False, label="UInt32")), ("lpszOwnerGroupName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbOwnerGroupId", SimTypeInt(signed=False, label="UInt32")), ("lpszOwnerGroupId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbProperties", SimTypeInt(signed=False, label="UInt32")), ("pProperties", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbRoProperties", SimTypeInt(signed=False, label="UInt32")), ("pRoProperties", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CLUSTER_RESOURCE_ENUM_ITEM", pack=False, align=None),
    "CLUSTER_RESOURCE_RESTART_ACTION": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_RESTART_ACTION"),
    "CLUSTER_RESOURCE_STATE": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE"),
    "CLUSTER_RESOURCE_STATE_CHANGE_REASON": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE_CHANGE_REASON"),
    "CLUSTER_RESOURCE_TYPE_ENUM": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_TYPE_ENUM"),
    "CLUSTER_ROLE": SimTypeInt(signed=False, label="CLUSTER_ROLE"),
    "CLUSTER_ROLE_STATE": SimTypeInt(signed=False, label="CLUSTER_ROLE_STATE"),
    "CLUSTER_SETUP_PHASE": SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"),
    "CLUSTER_SETUP_PHASE_SEVERITY": SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"),
    "CLUSTER_SETUP_PHASE_TYPE": SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"),
    "CLUSTER_SET_PASSWORD_STATUS": SimStruct(OrderedDict((("NodeId", SimTypeInt(signed=False, label="UInt32")), ("SetAttempted", SimTypeChar(label="Byte")), ("ReturnStatus", SimTypeInt(signed=False, label="UInt32")),)), name="CLUSTER_SET_PASSWORD_STATUS", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_BACKUP_STATE": SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_BACKUP_STATE"),
    "CLUSTER_SHARED_VOLUME_RENAME_GUID_INPUT": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSTER_SHARED_VOLUME_RENAME_INPUT_VOLUME", SimStruct)), ("Base2", SimTypeRef("CLUSTER_SHARED_VOLUME_RENAME_INPUT_GUID_NAME", SimStruct)),)), name="CLUSTER_SHARED_VOLUME_RENAME_GUID_INPUT", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_RENAME_INPUT": SimStruct(OrderedDict((("Base", SimTypeRef("CLUSTER_SHARED_VOLUME_RENAME_INPUT_VOLUME", SimStruct)), ("Base2", SimTypeRef("CLUSTER_SHARED_VOLUME_RENAME_INPUT_NAME", SimStruct)),)), name="CLUSTER_SHARED_VOLUME_RENAME_INPUT", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_RENAME_INPUT_GUID_NAME": SimStruct(OrderedDict((("NewVolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("NewVolumeGuid", SimTypeArray(SimTypeChar(label="Char"), 50)),)), name="CLUSTER_SHARED_VOLUME_RENAME_INPUT_GUID_NAME", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_RENAME_INPUT_NAME": SimStruct(OrderedDict((("NewVolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CLUSTER_SHARED_VOLUME_RENAME_INPUT_NAME", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_RENAME_INPUT_TYPE": SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_RENAME_INPUT_TYPE"),
    "CLUSTER_SHARED_VOLUME_RENAME_INPUT_VOLUME": SimStruct(OrderedDict((("InputType", SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_RENAME_INPUT_TYPE")), ("Anonymous", SimUnion({"VolumeOffset": SimTypeLongLong(signed=False, label="UInt64"), "VolumeId": SimTypeArray(SimTypeChar(label="Char"), 260), "VolumeName": SimTypeArray(SimTypeChar(label="Char"), 260), "VolumeGuid": SimTypeArray(SimTypeChar(label="Char"), 50)}, name="<anon>", label="None")),)), name="CLUSTER_SHARED_VOLUME_RENAME_INPUT_VOLUME", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE": SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE"),
    "CLUSTER_SHARED_VOLUME_STATE": SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_STATE"),
    "CLUSTER_SHARED_VOLUME_STATE_INFO": SimStruct(OrderedDict((("szVolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szNodeName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("VolumeState", SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_STATE")),)), name="CLUSTER_SHARED_VOLUME_STATE_INFO", pack=False, align=None),
    "CLUSTER_SHARED_VOLUME_STATE_INFO_EX": SimStruct(OrderedDict((("szVolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szNodeName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("VolumeState", SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_STATE")), ("szVolumeFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("RedirectedIOReason", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeRedirectedIOReason", SimTypeLongLong(signed=False, label="UInt64")),)), name="CLUSTER_SHARED_VOLUME_STATE_INFO_EX", pack=False, align=None),
    "CLUSTER_STORAGENODE_STATE": SimTypeInt(signed=False, label="CLUSTER_STORAGENODE_STATE"),
    "CLUSTER_UPGRADE_PHASE": SimTypeInt(signed=False, label="CLUSTER_UPGRADE_PHASE"),
    "CLUSTER_VALIDATE_CSV_FILENAME": SimStruct(OrderedDict((("szFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUSTER_VALIDATE_CSV_FILENAME", pack=False, align=None),
    "CLUSTER_VALIDATE_DIRECTORY": SimStruct(OrderedDict((("szPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUSTER_VALIDATE_DIRECTORY", pack=False, align=None),
    "CLUSTER_VALIDATE_NETNAME": SimStruct(OrderedDict((("szNetworkName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUSTER_VALIDATE_NETNAME", pack=False, align=None),
    "CLUSTER_VALIDATE_PATH": SimStruct(OrderedDict((("szPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUSTER_VALIDATE_PATH", pack=False, align=None),
    "CLUS_AFFINITY_RULE_TYPE": SimTypeInt(signed=False, label="CLUS_AFFINITY_RULE_TYPE"),
    "CLUS_CHARACTERISTICS": SimTypeInt(signed=False, label="CLUS_CHARACTERISTICS"),
    "CLUS_CHKDSK_INFO": SimStruct(OrderedDict((("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("ChkdskState", SimTypeInt(signed=False, label="UInt32")), ("FileIdCount", SimTypeInt(signed=False, label="UInt32")), ("FileIdList", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="CLUS_CHKDSK_INFO", pack=False, align=None),
    "CLUS_CREATE_INFRASTRUCTURE_FILESERVER_INPUT": SimStruct(OrderedDict((("FileServerName", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="CLUS_CREATE_INFRASTRUCTURE_FILESERVER_INPUT", pack=False, align=None),
    "CLUS_CREATE_INFRASTRUCTURE_FILESERVER_OUTPUT": SimStruct(OrderedDict((("FileServerName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CLUS_CREATE_INFRASTRUCTURE_FILESERVER_OUTPUT", pack=False, align=None),
    "CLUS_CSV_MAINTENANCE_MODE_INFO": SimStruct(OrderedDict((("InMaintenance", SimTypeInt(signed=True, label="Int32")), ("VolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CLUS_CSV_MAINTENANCE_MODE_INFO", pack=False, align=None),
    "CLUS_CSV_VOLUME_INFO": SimStruct(OrderedDict((("VolumeOffset", SimTypeLongLong(signed=False, label="UInt64")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("FaultState", SimTypeInt(signed=False, label="CLUSTER_CSV_VOLUME_FAULT_STATE")), ("BackupState", SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_BACKUP_STATE")), ("szVolumeFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szVolumeName", SimTypeArray(SimTypeChar(label="Char"), 50)),)), name="CLUS_CSV_VOLUME_INFO", pack=False, align=None),
    "CLUS_CSV_VOLUME_NAME": SimStruct(OrderedDict((("VolumeOffset", SimTypeLongLong(signed=True, label="Int64")), ("szVolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szRootPath", SimTypeArray(SimTypeChar(label="Char"), 263)),)), name="CLUS_CSV_VOLUME_NAME", pack=False, align=None),
    "CLUS_DISK_NUMBER_INFO": SimStruct(OrderedDict((("DiskNumber", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_DISK_NUMBER_INFO", pack=False, align=None),
    "CLUS_DNN_LEADER_STATUS": SimStruct(OrderedDict((("IsOnline", SimTypeInt(signed=True, label="Int32")), ("IsFileServerPresent", SimTypeInt(signed=True, label="Int32")),)), name="CLUS_DNN_LEADER_STATUS", pack=False, align=None),
    "CLUS_DNN_SODAFS_CLONE_STATUS": SimStruct(OrderedDict((("NodeId", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE")),)), name="CLUS_DNN_SODAFS_CLONE_STATUS", pack=False, align=None),
    "CLUS_FLAGS": SimTypeInt(signed=False, label="CLUS_FLAGS"),
    "CLUS_FORCE_QUORUM_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNodeBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumberofNodes", SimTypeInt(signed=False, label="UInt32")), ("multiszNodeList", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUS_FORCE_QUORUM_INFO", pack=False, align=None),
    "CLUS_FTSET_INFO": SimStruct(OrderedDict((("dwRootSignature", SimTypeInt(signed=False, label="UInt32")), ("dwFtType", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_FTSET_INFO", pack=False, align=None),
    "CLUS_GROUP_START_SETTING": SimTypeInt(signed=False, label="CLUS_GROUP_START_SETTING"),
    "CLUS_MAINTENANCE_MODE_INFO": SimStruct(OrderedDict((("InMaintenance", SimTypeInt(signed=True, label="Int32")),)), name="CLUS_MAINTENANCE_MODE_INFO", pack=False, align=None),
    "CLUS_MAINTENANCE_MODE_INFOEX": SimStruct(OrderedDict((("InMaintenance", SimTypeInt(signed=True, label="Int32")), ("MaintainenceModeType", SimTypeInt(signed=False, label="MAINTENANCE_MODE_TYPE_ENUM")), ("InternalState", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE")), ("Signature", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_MAINTENANCE_MODE_INFOEX", pack=False, align=None),
    "CLUS_NETNAME_IP_INFO_ENTRY": SimStruct(OrderedDict((("NodeId", SimTypeInt(signed=False, label="UInt32")), ("AddressSize", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CLUS_NETNAME_IP_INFO_ENTRY", pack=False, align=None),
    "CLUS_NETNAME_IP_INFO_FOR_MULTICHANNEL": SimStruct(OrderedDict((("szName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("IpInfo", SimTypePointer(SimTypeRef("CLUS_NETNAME_IP_INFO_ENTRY", SimStruct), offset=0)),)), name="CLUS_NETNAME_IP_INFO_FOR_MULTICHANNEL", pack=False, align=None),
    "CLUS_NETNAME_PWD_INFO": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypeArray(SimTypeChar(label="Char"), 16)), ("CreatingDC", SimTypeArray(SimTypeChar(label="Char"), 258)), ("ObjectGuid", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="CLUS_NETNAME_PWD_INFO", pack=False, align=None),
    "CLUS_NETNAME_PWD_INFOEX": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypeArray(SimTypeChar(label="Char"), 128)), ("CreatingDC", SimTypeArray(SimTypeChar(label="Char"), 258)), ("ObjectGuid", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="CLUS_NETNAME_PWD_INFOEX", pack=False, align=None),
    "CLUS_NETNAME_VS_TOKEN_INFO": SimStruct(OrderedDict((("ProcessID", SimTypeInt(signed=False, label="UInt32")), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("InheritHandle", SimTypeInt(signed=True, label="Int32")),)), name="CLUS_NETNAME_VS_TOKEN_INFO", pack=False, align=None),
    "CLUS_PARTITION_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szVolumeLabel", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("rgdwMaximumComponentLength", SimTypeInt(signed=False, label="UInt32")), ("dwFileSystemFlags", SimTypeInt(signed=False, label="UInt32")), ("szFileSystem", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="CLUS_PARTITION_INFO", pack=False, align=None),
    "CLUS_PARTITION_INFO_EX": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szVolumeLabel", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("rgdwMaximumComponentLength", SimTypeInt(signed=False, label="UInt32")), ("dwFileSystemFlags", SimTypeInt(signed=False, label="UInt32")), ("szFileSystem", SimTypeArray(SimTypeChar(label="Char"), 32)), ("TotalSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("FreeSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("VolumeGuid", SimTypeBottom(label="Guid")),)), name="CLUS_PARTITION_INFO_EX", pack=False, align=None),
    "CLUS_PARTITION_INFO_EX2": SimStruct(OrderedDict((("GptPartitionId", SimTypeBottom(label="Guid")), ("szPartitionName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("EncryptionFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_PARTITION_INFO_EX2", pack=False, align=None),
    "CLUS_PROVIDER_STATE_CHANGE_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("resourceState", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE")), ("szProviderId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CLUS_PROVIDER_STATE_CHANGE_INFO", pack=False, align=None),
    "CLUS_RESOURCE_CLASS_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"dw": SimTypeInt(signed=False, label="UInt32"), "rc": SimTypeInt(signed=False, label="CLUSTER_RESOURCE_CLASS")}, name="<anon>", label="None")), ("SubClass", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "li": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="CLUS_RESOURCE_CLASS_INFO", pack=False, align=None),
    "CLUS_RESSUBCLASS": SimTypeInt(signed=False, label="CLUS_RESSUBCLASS"),
    "CLUS_RESSUBCLASS_NETWORK": SimTypeInt(signed=False, label="CLUS_RESSUBCLASS_NETWORK"),
    "CLUS_RESSUBCLASS_STORAGE": SimTypeInt(signed=False, label="CLUS_RESSUBCLASS_STORAGE"),
    "CLUS_SCSI_ADDRESS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("PortNumber", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "dw": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="CLUS_SCSI_ADDRESS", pack=False, align=None),
    "CLUS_SET_MAINTENANCE_MODE_INPUT": SimStruct(OrderedDict((("InMaintenance", SimTypeInt(signed=True, label="Int32")), ("ExtraParameterSize", SimTypeInt(signed=False, label="UInt32")), ("ExtraParameter", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CLUS_SET_MAINTENANCE_MODE_INPUT", pack=False, align=None),
    "CLUS_SHARED_VOLUME_BACKUP_MODE": SimStruct(OrderedDict((("BackupState", SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_BACKUP_STATE")), ("DelayTimerInSecs", SimTypeInt(signed=False, label="UInt32")), ("VolumeName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CLUS_SHARED_VOLUME_BACKUP_MODE", pack=False, align=None),
    "CLUS_STARTING_PARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("bForm", SimTypeInt(signed=True, label="Int32")), ("bFirst", SimTypeInt(signed=True, label="Int32")),)), name="CLUS_STARTING_PARAMS", pack=False, align=None),
    "CLUS_STORAGE_GET_AVAILABLE_DRIVELETTERS": SimStruct(OrderedDict((("AvailDrivelettersMask", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_STORAGE_GET_AVAILABLE_DRIVELETTERS", pack=False, align=None),
    "CLUS_STORAGE_REMAP_DRIVELETTER": SimStruct(OrderedDict((("CurrentDriveLetterMask", SimTypeInt(signed=False, label="UInt32")), ("TargetDriveLetterMask", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_STORAGE_REMAP_DRIVELETTER", pack=False, align=None),
    "CLUS_STORAGE_SET_DRIVELETTER": SimStruct(OrderedDict((("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("DriveLetterMask", SimTypeInt(signed=False, label="UInt32")),)), name="CLUS_STORAGE_SET_DRIVELETTER", pack=False, align=None),
    "CLUS_WORKER": SimStruct(OrderedDict((("hThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Terminate", SimTypeInt(signed=True, label="Int32")),)), name="CLUS_WORKER", pack=False, align=None),
    "CL_ADD_PARTY_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_CALL_CONNECTED_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_CLOSE_AF_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_CLOSE_CALL_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_DEREG_SAP_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_DROP_PARTY_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_INCOMING_CALL_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CL_INCOMING_CALL_QOS_CHANGE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_INCOMING_CLOSE_CALL_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_INCOMING_DROP_PARTY_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_MAKE_CALL_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_MODIFY_CALL_QOS_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_OPEN_AF_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CL_REG_SAP_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CMC_ADD_ATTRIBUTES_INFO": SimStruct(OrderedDict((("dwCmcDataReference", SimTypeInt(signed=False, label="UInt32")), ("cCertReference", SimTypeInt(signed=False, label="UInt32")), ("rgdwCertReference", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cAttribute", SimTypeInt(signed=False, label="UInt32")), ("rgAttribute", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)),)), name="CMC_ADD_ATTRIBUTES_INFO", pack=False, align=None),
    "CMC_ADD_EXTENSIONS_INFO": SimStruct(OrderedDict((("dwCmcDataReference", SimTypeInt(signed=False, label="UInt32")), ("cCertReference", SimTypeInt(signed=False, label="UInt32")), ("rgdwCertReference", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CMC_ADD_EXTENSIONS_INFO", pack=False, align=None),
    "CMC_DATA_INFO": SimStruct(OrderedDict((("cTaggedAttribute", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedAttribute", SimTypePointer(SimTypeRef("CMC_TAGGED_ATTRIBUTE", SimStruct), offset=0)), ("cTaggedRequest", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedRequest", SimTypePointer(SimTypeRef("CMC_TAGGED_REQUEST", SimStruct), offset=0)), ("cTaggedContentInfo", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedContentInfo", SimTypePointer(SimTypeRef("CMC_TAGGED_CONTENT_INFO", SimStruct), offset=0)), ("cTaggedOtherMsg", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedOtherMsg", SimTypePointer(SimTypeRef("CMC_TAGGED_OTHER_MSG", SimStruct), offset=0)),)), name="CMC_DATA_INFO", pack=False, align=None),
    "CMC_DRIVER_INFO": SimStruct(OrderedDict((("ExceptionCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MCA_EXCEPTION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "CmcLog"]), offset=0)), ("DpcCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CMC_DRIVER_INFO", pack=False, align=None),
    "CMC_PEND_INFO": SimStruct(OrderedDict((("PendToken", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("PendTime", SimTypeRef("FILETIME", SimStruct)),)), name="CMC_PEND_INFO", pack=False, align=None),
    "CMC_RESPONSE_INFO": SimStruct(OrderedDict((("cTaggedAttribute", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedAttribute", SimTypePointer(SimTypeRef("CMC_TAGGED_ATTRIBUTE", SimStruct), offset=0)), ("cTaggedContentInfo", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedContentInfo", SimTypePointer(SimTypeRef("CMC_TAGGED_CONTENT_INFO", SimStruct), offset=0)), ("cTaggedOtherMsg", SimTypeInt(signed=False, label="UInt32")), ("rgTaggedOtherMsg", SimTypePointer(SimTypeRef("CMC_TAGGED_OTHER_MSG", SimStruct), offset=0)),)), name="CMC_RESPONSE_INFO", pack=False, align=None),
    "CMC_STATUS_INFO": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("cBodyList", SimTypeInt(signed=False, label="UInt32")), ("rgdwBodyList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pwszStatusString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwOtherInfoChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwFailInfo": SimTypeInt(signed=False, label="UInt32"), "pPendInfo": SimTypePointer(SimTypeRef("CMC_PEND_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CMC_STATUS_INFO", pack=False, align=None),
    "CMC_TAGGED_ATTRIBUTE": SimStruct(OrderedDict((("dwBodyPartID", SimTypeInt(signed=False, label="UInt32")), ("Attribute", SimTypeRef("CRYPT_ATTRIBUTE", SimStruct)),)), name="CMC_TAGGED_ATTRIBUTE", pack=False, align=None),
    "CMC_TAGGED_CERT_REQUEST": SimStruct(OrderedDict((("dwBodyPartID", SimTypeInt(signed=False, label="UInt32")), ("SignedCertRequest", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CMC_TAGGED_CERT_REQUEST", pack=False, align=None),
    "CMC_TAGGED_CONTENT_INFO": SimStruct(OrderedDict((("dwBodyPartID", SimTypeInt(signed=False, label="UInt32")), ("EncodedContentInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CMC_TAGGED_CONTENT_INFO", pack=False, align=None),
    "CMC_TAGGED_OTHER_MSG": SimStruct(OrderedDict((("dwBodyPartID", SimTypeInt(signed=False, label="UInt32")), ("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CMC_TAGGED_OTHER_MSG", pack=False, align=None),
    "CMC_TAGGED_REQUEST": SimStruct(OrderedDict((("dwTaggedRequestChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pTaggedCertRequest": SimTypePointer(SimTypeRef("CMC_TAGGED_CERT_REQUEST", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CMC_TAGGED_REQUEST", pack=False, align=None),
    "CMD_ENTRY": SimStruct(OrderedDict((("pwszCmdToken", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnCmdHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pwszMachine", "ppwcArguments", "dwCurrentIndex", "dwArgCount", "dwFlags", "pvData", "pbDone"]), offset=0)), ("dwShortCmdHelpToken", SimTypeInt(signed=False, label="UInt32")), ("dwCmdHlpToken", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pOsVersionCheck", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CIMOSType", "CIMOSProductSuite", "CIMOSVersion", "CIMOSBuildNumber", "CIMServicePackMajorVersion", "CIMServicePackMinorVersion", "uiReserved", "dwReserved"]), offset=0)), ("pfnCustomHelpFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hModule", "pwszCmdToken"]), offset=0)),)), name="CMD_ENTRY", pack=False, align=None),
    "CMD_GROUP_ENTRY": SimStruct(OrderedDict((("pwszCmdGroupToken", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwShortCmdHelpToken", SimTypeInt(signed=False, label="UInt32")), ("ulCmdGroupSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pCmdGroup", SimTypePointer(SimTypeRef("CMD_ENTRY", SimStruct), offset=0)), ("pOsVersionCheck", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CIMOSType", "CIMOSProductSuite", "CIMOSVersion", "CIMOSBuildNumber", "CIMServicePackMajorVersion", "CIMServicePackMinorVersion", "uiReserved", "dwReserved"]), offset=0)),)), name="CMD_GROUP_ENTRY", pack=False, align=None),
    "CMINVOKECOMMANDINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpVerb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nShow", SimTypeInt(signed=True, label="Int32")), ("dwHotKey", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CMINVOKECOMMANDINFO", pack=False, align=None),
    "CMINVOKECOMMANDINFOEX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpVerb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nShow", SimTypeInt(signed=True, label="Int32")), ("dwHotKey", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpVerbW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpParametersW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDirectoryW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpTitleW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ptInvoke", SimTypeRef("POINT", SimStruct)),)), name="CMINVOKECOMMANDINFOEX", pack=False, align=None),
    "CMINVOKECOMMANDINFOEX_REMOTE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpVerbString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nShow", SimTypeInt(signed=True, label="Int32")), ("dwHotKey", SimTypeInt(signed=False, label="UInt32")), ("lpTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpVerbWString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpParametersW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDirectoryW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpTitleW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ptInvoke", SimTypeRef("POINT", SimStruct)), ("lpVerbInt", SimTypeInt(signed=False, label="UInt32")), ("lpVerbWInt", SimTypeInt(signed=False, label="UInt32")),)), name="CMINVOKECOMMANDINFOEX_REMOTE", pack=False, align=None),
    "CMSGHDR": SimStruct(OrderedDict((("cmsg_len", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cmsg_level", SimTypeInt(signed=True, label="Int32")), ("cmsg_type", SimTypeInt(signed=True, label="Int32")),)), name="CMSGHDR", pack=False, align=None),
    "CMSG_CMS_RECIPIENT_INFO": SimStruct(OrderedDict((("dwRecipientChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pKeyTrans": SimTypePointer(SimTypeRef("CMSG_KEY_TRANS_RECIPIENT_INFO", SimStruct), offset=0), "pKeyAgree": SimTypePointer(SimTypeRef("CMSG_KEY_AGREE_RECIPIENT_INFO", SimStruct), offset=0), "pMailList": SimTypePointer(SimTypeRef("CMSG_MAIL_LIST_RECIPIENT_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CMSG_CMS_RECIPIENT_INFO", pack=False, align=None),
    "CMSG_CMS_SIGNER_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("SignerId", SimTypeRef("CERT_ID", SimStruct)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("HashEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("AuthAttrs", SimTypeRef("CRYPT_ATTRIBUTES", SimStruct)), ("UnauthAttrs", SimTypeRef("CRYPT_ATTRIBUTES", SimStruct)),)), name="CMSG_CMS_SIGNER_INFO", pack=False, align=None),
    "CMSG_CNG_CONTENT_DECRYPT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ContentEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pfnAlloc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)), ("hNCryptKey", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pbContentEncryptKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbContentEncryptKey", SimTypeInt(signed=False, label="UInt32")), ("hCNGContentEncryptKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pbCNGContentEncryptKeyObject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CMSG_CNG_CONTENT_DECRYPT_INFO", pack=False, align=None),
    "CMSG_CONTENT_ENCRYPT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ContentEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cRecipients", SimTypeInt(signed=False, label="UInt32")), ("rgCmsRecipients", SimTypePointer(SimTypeRef("CMSG_RECIPIENT_ENCODE_INFO", SimStruct), offset=0)), ("pfnAlloc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)), ("dwEncryptFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hContentEncryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hCNGContentEncryptKey": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fCNG", SimTypeInt(signed=True, label="Int32")), ("pbCNGContentEncryptKeyObject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pbContentEncryptKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbContentEncryptKey", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_CONTENT_ENCRYPT_INFO", pack=False, align=None),
    "CMSG_CTRL_ADD_SIGNER_UNAUTH_ATTR_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignerIndex", SimTypeInt(signed=False, label="UInt32")), ("blob", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CMSG_CTRL_ADD_SIGNER_UNAUTH_ATTR_PARA", pack=False, align=None),
    "CMSG_CTRL_DECRYPT_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hCryptProv": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hNCryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_CTRL_DECRYPT_PARA", pack=False, align=None),
    "CMSG_CTRL_DEL_SIGNER_UNAUTH_ATTR_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignerIndex", SimTypeInt(signed=False, label="UInt32")), ("dwUnauthAttrIndex", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_CTRL_DEL_SIGNER_UNAUTH_ATTR_PARA", pack=False, align=None),
    "CMSG_CTRL_KEY_AGREE_DECRYPT_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hCryptProv": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hNCryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("pKeyAgree", SimTypePointer(SimTypeRef("CMSG_KEY_AGREE_RECIPIENT_INFO", SimStruct), offset=0)), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")), ("dwRecipientEncryptedKeyIndex", SimTypeInt(signed=False, label="UInt32")), ("OriginatorPublicKey", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)),)), name="CMSG_CTRL_KEY_AGREE_DECRYPT_PARA", pack=False, align=None),
    "CMSG_CTRL_KEY_TRANS_DECRYPT_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hCryptProv": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hNCryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("pKeyTrans", SimTypePointer(SimTypeRef("CMSG_KEY_TRANS_RECIPIENT_INFO", SimStruct), offset=0)), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_CTRL_KEY_TRANS_DECRYPT_PARA", pack=False, align=None),
    "CMSG_CTRL_MAIL_LIST_DECRYPT_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pMailList", SimTypePointer(SimTypeRef("CMSG_MAIL_LIST_RECIPIENT_INFO", SimStruct), offset=0)), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")), ("dwKeyChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hKeyEncryptionKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pvKeyEncryptionKey": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="CMSG_CTRL_MAIL_LIST_DECRYPT_PARA", pack=False, align=None),
    "CMSG_CTRL_VERIFY_SIGNATURE_EX_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwSignerIndex", SimTypeInt(signed=False, label="UInt32")), ("dwSignerType", SimTypeInt(signed=False, label="UInt32")), ("pvSigner", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CMSG_CTRL_VERIFY_SIGNATURE_EX_PARA", pack=False, align=None),
    "CMSG_ENCRYPTED_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ContentEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CMSG_ENCRYPTED_ENCODE_INFO", pack=False, align=None),
    "CMSG_ENVELOPED_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ContentEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cRecipients", SimTypeInt(signed=False, label="UInt32")), ("rgpRecipients", SimTypePointer(SimTypePointer(SimTypeRef("CERT_INFO", SimStruct), offset=0), offset=0)),)), name="CMSG_ENVELOPED_ENCODE_INFO", pack=False, align=None),
    "CMSG_HASHED_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvHashAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CMSG_HASHED_ENCODE_INFO", pack=False, align=None),
    "CMSG_KEY_AGREE_ENCRYPT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("UserKeyingMaterial", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwOriginatorChoice", SimTypeInt(signed=False, label="CMSG_KEY_AGREE_ORIGINATOR")), ("Anonymous", SimUnion({"OriginatorCertId": SimTypeRef("CERT_ID", SimStruct), "OriginatorPublicKeyInfo": SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct)}, name="<anon>", label="None")), ("cKeyAgreeKeyEncryptInfo", SimTypeInt(signed=False, label="UInt32")), ("rgpKeyAgreeKeyEncryptInfo", SimTypePointer(SimTypePointer(SimTypeRef("CMSG_KEY_AGREE_KEY_ENCRYPT_INFO", SimStruct), offset=0), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_KEY_AGREE_ENCRYPT_INFO", pack=False, align=None),
    "CMSG_KEY_AGREE_KEY_ENCRYPT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("EncryptedKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CMSG_KEY_AGREE_KEY_ENCRYPT_INFO", pack=False, align=None),
    "CMSG_KEY_AGREE_OPTION": SimTypeInt(signed=False, label="CMSG_KEY_AGREE_OPTION"),
    "CMSG_KEY_AGREE_ORIGINATOR": SimTypeInt(signed=False, label="CMSG_KEY_AGREE_ORIGINATOR"),
    "CMSG_KEY_AGREE_RECIPIENT_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvKeyEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeyWrapAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvKeyWrapAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("dwKeyChoice", SimTypeInt(signed=False, label="CMSG_KEY_AGREE_OPTION")), ("Anonymous", SimUnion({"pEphemeralAlgorithm": SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), "pSenderId": SimTypePointer(SimTypeRef("CERT_ID", SimStruct), offset=0)}, name="<anon>", label="None")), ("UserKeyingMaterial", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cRecipientEncryptedKeys", SimTypeInt(signed=False, label="UInt32")), ("rgpRecipientEncryptedKeys", SimTypePointer(SimTypePointer(SimTypeRef("CMSG_RECIPIENT_ENCRYPTED_KEY_ENCODE_INFO", SimStruct), offset=0), offset=0)),)), name="CMSG_KEY_AGREE_RECIPIENT_ENCODE_INFO", pack=False, align=None),
    "CMSG_KEY_AGREE_RECIPIENT_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwOriginatorChoice", SimTypeInt(signed=False, label="CMSG_KEY_AGREE_ORIGINATOR")), ("Anonymous", SimUnion({"OriginatorCertId": SimTypeRef("CERT_ID", SimStruct), "OriginatorPublicKeyInfo": SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct)}, name="<anon>", label="None")), ("UserKeyingMaterial", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("cRecipientEncryptedKeys", SimTypeInt(signed=False, label="UInt32")), ("rgpRecipientEncryptedKeys", SimTypePointer(SimTypePointer(SimTypeRef("CMSG_RECIPIENT_ENCRYPTED_KEY_INFO", SimStruct), offset=0), offset=0)),)), name="CMSG_KEY_AGREE_RECIPIENT_INFO", pack=False, align=None),
    "CMSG_KEY_TRANS_ENCRYPT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_KEY_TRANS_ENCRYPT_INFO", pack=False, align=None),
    "CMSG_KEY_TRANS_RECIPIENT_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvKeyEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("RecipientPublicKey", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("RecipientId", SimTypeRef("CERT_ID", SimStruct)),)), name="CMSG_KEY_TRANS_RECIPIENT_ENCODE_INFO", pack=False, align=None),
    "CMSG_KEY_TRANS_RECIPIENT_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("RecipientId", SimTypeRef("CERT_ID", SimStruct)), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CMSG_KEY_TRANS_RECIPIENT_INFO", pack=False, align=None),
    "CMSG_MAIL_LIST_ENCRYPT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRecipientIndex", SimTypeInt(signed=False, label="UInt32")), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_MAIL_LIST_ENCRYPT_INFO", pack=False, align=None),
    "CMSG_MAIL_LIST_RECIPIENT_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvKeyEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwKeyChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hKeyEncryptionKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pvKeyEncryptionKey": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("KeyId", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("Date", SimTypeRef("FILETIME", SimStruct)), ("pOtherAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE_TYPE_VALUE", SimStruct), offset=0)),)), name="CMSG_MAIL_LIST_RECIPIENT_ENCODE_INFO", pack=False, align=None),
    "CMSG_MAIL_LIST_RECIPIENT_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("KeyId", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("KeyEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("Date", SimTypeRef("FILETIME", SimStruct)), ("pOtherAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE_TYPE_VALUE", SimStruct), offset=0)),)), name="CMSG_MAIL_LIST_RECIPIENT_INFO", pack=False, align=None),
    "CMSG_RC2_AUX_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwBitLen", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_RC2_AUX_INFO", pack=False, align=None),
    "CMSG_RC4_AUX_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwBitLen", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_RC4_AUX_INFO", pack=False, align=None),
    "CMSG_RECIPIENT_ENCODE_INFO": SimStruct(OrderedDict((("dwRecipientChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pKeyTrans": SimTypePointer(SimTypeRef("CMSG_KEY_TRANS_RECIPIENT_ENCODE_INFO", SimStruct), offset=0), "pKeyAgree": SimTypePointer(SimTypeRef("CMSG_KEY_AGREE_RECIPIENT_ENCODE_INFO", SimStruct), offset=0), "pMailList": SimTypePointer(SimTypeRef("CMSG_MAIL_LIST_RECIPIENT_ENCODE_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CMSG_RECIPIENT_ENCODE_INFO", pack=False, align=None),
    "CMSG_RECIPIENT_ENCRYPTED_KEY_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("RecipientPublicKey", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("RecipientId", SimTypeRef("CERT_ID", SimStruct)), ("Date", SimTypeRef("FILETIME", SimStruct)), ("pOtherAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE_TYPE_VALUE", SimStruct), offset=0)),)), name="CMSG_RECIPIENT_ENCRYPTED_KEY_ENCODE_INFO", pack=False, align=None),
    "CMSG_RECIPIENT_ENCRYPTED_KEY_INFO": SimStruct(OrderedDict((("RecipientId", SimTypeRef("CERT_ID", SimStruct)), ("EncryptedKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("Date", SimTypeRef("FILETIME", SimStruct)), ("pOtherAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE_TYPE_VALUE", SimStruct), offset=0)),)), name="CMSG_RECIPIENT_ENCRYPTED_KEY_INFO", pack=False, align=None),
    "CMSG_SIGNED_AND_ENVELOPED_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("SignedInfo", SimTypeRef("CMSG_SIGNED_ENCODE_INFO", SimStruct)), ("EnvelopedInfo", SimTypeRef("CMSG_ENVELOPED_ENCODE_INFO", SimStruct)),)), name="CMSG_SIGNED_AND_ENVELOPED_ENCODE_INFO", pack=False, align=None),
    "CMSG_SIGNED_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cSigners", SimTypeInt(signed=False, label="UInt32")), ("rgSigners", SimTypePointer(SimTypeRef("CMSG_SIGNER_ENCODE_INFO", SimStruct), offset=0)), ("cCertEncoded", SimTypeInt(signed=False, label="UInt32")), ("rgCertEncoded", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("cCrlEncoded", SimTypeInt(signed=False, label="UInt32")), ("rgCrlEncoded", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CMSG_SIGNED_ENCODE_INFO", pack=False, align=None),
    "CMSG_SIGNER_ENCODE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pCertInfo", SimTypePointer(SimTypeRef("CERT_INFO", SimStruct), offset=0)), ("Anonymous", SimUnion({"hCryptProv": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hNCryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvHashAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cAuthAttr", SimTypeInt(signed=False, label="UInt32")), ("rgAuthAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)), ("cUnauthAttr", SimTypeInt(signed=False, label="UInt32")), ("rgUnauthAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)),)), name="CMSG_SIGNER_ENCODE_INFO", pack=False, align=None),
    "CMSG_SIGNER_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Issuer", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("HashEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("AuthAttrs", SimTypeRef("CRYPT_ATTRIBUTES", SimStruct)), ("UnauthAttrs", SimTypeRef("CRYPT_ATTRIBUTES", SimStruct)),)), name="CMSG_SIGNER_INFO", pack=False, align=None),
    "CMSG_SP3_COMPATIBLE_AUX_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CMSG_SP3_COMPATIBLE_AUX_INFO", pack=False, align=None),
    "CMSG_STREAM_INFO": SimStruct(OrderedDict((("cbContent", SimTypeInt(signed=False, label="UInt32")), ("pfnStreamOutput", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvArg", "pbData", "cbData", "fFinal"]), offset=0)), ("pvArg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CMSG_STREAM_INFO", pack=False, align=None),
    "CMS_DH_KEY_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Algid", SimTypeInt(signed=False, label="ALG_ID")), ("pszContentEncObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PubInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CMS_DH_KEY_INFO", pack=False, align=None),
    "CMS_KEY_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Algid", SimTypeInt(signed=False, label="ALG_ID")), ("pbOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbOID", SimTypeInt(signed=False, label="UInt32")),)), name="CMS_KEY_INFO", pack=False, align=None),
    "CMYKCOLOR": SimStruct(OrderedDict((("cyan", SimTypeShort(signed=False, label="UInt16")), ("magenta", SimTypeShort(signed=False, label="UInt16")), ("yellow", SimTypeShort(signed=False, label="UInt16")), ("black", SimTypeShort(signed=False, label="UInt16")),)), name="CMYKCOLOR", pack=False, align=None),
    "CM_ACTIVATE_VC_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CM_ADD_PARTY_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_CDFLAGS": SimTypeInt(signed=False, label="CM_CDFLAGS"),
    "CM_CDMASK": SimTypeInt(signed=False, label="CM_CDMASK"),
    "CM_CLOSE_AF_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_CLOSE_CALL_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_COLUMNINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("uWidth", SimTypeInt(signed=False, label="UInt32")), ("uDefaultWidth", SimTypeInt(signed=False, label="UInt32")), ("uIdealWidth", SimTypeInt(signed=False, label="UInt32")), ("wszName", SimTypeArray(SimTypeChar(label="Char"), 80)),)), name="CM_COLUMNINFO", pack=False, align=None),
    "CM_COMPONENT_INFORMATION": SimStruct(OrderedDict((("Flags", SimTypeRef("DEVICE_FLAGS", SimStruct)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeInt(signed=False, label="UInt32")), ("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CM_COMPONENT_INFORMATION", pack=False, align=None),
    "CM_DEACTIVATE_VC_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CM_DEREG_SAP_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_DEVCAP": SimTypeInt(signed=False, label="CM_DEVCAP"),
    "CM_DEVNODE_STATUS_FLAGS": SimTypeInt(signed=False, label="CM_DEVNODE_STATUS_FLAGS"),
    "CM_DISK_GEOMETRY_DEVICE_DATA": SimStruct(OrderedDict((("BytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("NumberOfCylinders", SimTypeInt(signed=False, label="UInt32")), ("SectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("NumberOfHeads", SimTypeInt(signed=False, label="UInt32")),)), name="CM_DISK_GEOMETRY_DEVICE_DATA", pack=False, align=None),
    "CM_DROP_PARTY_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_EISA_FUNCTION_INFORMATION": SimStruct(OrderedDict((("CompressedId", SimTypeInt(signed=False, label="UInt32")), ("IdSlotFlags1", SimTypeChar(label="Byte")), ("IdSlotFlags2", SimTypeChar(label="Byte")), ("MinorRevision", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")), ("Selections", SimTypeArray(SimTypeChar(label="Byte"), 26)), ("FunctionFlags", SimTypeChar(label="Byte")), ("TypeString", SimTypeArray(SimTypeChar(label="Byte"), 80)), ("EisaMemory", SimTypeArray(SimTypeRef("EISA_MEMORY_CONFIGURATION", SimStruct), 9)), ("EisaIrq", SimTypeArray(SimTypeRef("EISA_IRQ_CONFIGURATION", SimStruct), 7)), ("EisaDma", SimTypeArray(SimTypeRef("EISA_DMA_CONFIGURATION", SimStruct), 4)), ("EisaPort", SimTypeArray(SimTypeRef("EISA_PORT_CONFIGURATION", SimStruct), 20)), ("InitializationData", SimTypeArray(SimTypeChar(label="Byte"), 60)),)), name="CM_EISA_FUNCTION_INFORMATION", pack=False, align=None),
    "CM_EISA_SLOT_INFORMATION": SimStruct(OrderedDict((("ReturnCode", SimTypeChar(label="Byte")), ("ReturnFlags", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")), ("MinorRevision", SimTypeChar(label="Byte")), ("Checksum", SimTypeShort(signed=False, label="UInt16")), ("NumberFunctions", SimTypeChar(label="Byte")), ("FunctionInformation", SimTypeChar(label="Byte")), ("CompressedId", SimTypeInt(signed=False, label="UInt32")),)), name="CM_EISA_SLOT_INFORMATION", pack=False, align=None),
    "CM_ENUMERATE_FLAGS": SimTypeInt(signed=False, label="CM_ENUMERATE_FLAGS"),
    "CM_ENUM_FLAGS": SimTypeInt(signed=False, label="CM_ENUM_FLAGS"),
    "CM_FLOPPY_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeArray(SimTypeChar(label="SByte"), 8)), ("MaxDensity", SimTypeInt(signed=False, label="UInt32")), ("MountDensity", SimTypeInt(signed=False, label="UInt32")), ("StepRateHeadUnloadTime", SimTypeChar(label="Byte")), ("HeadLoadTime", SimTypeChar(label="Byte")), ("MotorOffTime", SimTypeChar(label="Byte")), ("SectorLengthCode", SimTypeChar(label="Byte")), ("SectorPerTrack", SimTypeChar(label="Byte")), ("ReadWriteGapLength", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeChar(label="Byte")), ("FormatGapLength", SimTypeChar(label="Byte")), ("FormatFillCharacter", SimTypeChar(label="Byte")), ("HeadSettleTime", SimTypeChar(label="Byte")), ("MotorSettleTime", SimTypeChar(label="Byte")), ("MaximumTrackValue", SimTypeChar(label="Byte")), ("DataTransferRate", SimTypeChar(label="Byte")),)), name="CM_FLOPPY_DEVICE_DATA", pack=False, align=None),
    "CM_FULL_RESOURCE_DESCRIPTOR": SimStruct(OrderedDict((("InterfaceType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("PartialResourceList", SimTypeRef("CM_PARTIAL_RESOURCE_LIST", SimStruct)),)), name="CM_FULL_RESOURCE_DESCRIPTOR", pack=False, align=None),
    "CM_GET_DEVICE_INTERFACE_LIST_FLAGS": SimTypeInt(signed=False, label="CM_GET_DEVICE_INTERFACE_LIST_FLAGS"),
    "CM_INCOMING_CALL_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CM_INSTALL_STATE": SimTypeInt(signed=False, label="CM_INSTALL_STATE"),
    "CM_INT13_DRIVE_PARAMETER": SimStruct(OrderedDict((("DriveSelect", SimTypeShort(signed=False, label="UInt16")), ("MaxCylinders", SimTypeInt(signed=False, label="UInt32")), ("SectorsPerTrack", SimTypeShort(signed=False, label="UInt16")), ("MaxHeads", SimTypeShort(signed=False, label="UInt16")), ("NumberDrives", SimTypeShort(signed=False, label="UInt16")),)), name="CM_INT13_DRIVE_PARAMETER", pack=False, align=None),
    "CM_KEYBOARD_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeChar(label="Byte")), ("Subtype", SimTypeChar(label="Byte")), ("KeyboardFlags", SimTypeShort(signed=False, label="UInt16")),)), name="CM_KEYBOARD_DEVICE_DATA", pack=False, align=None),
    "CM_LOCATE_DEVNODE_FLAGS": SimTypeInt(signed=False, label="CM_LOCATE_DEVNODE_FLAGS"),
    "CM_LOG_CONF": SimTypeInt(signed=False, label="CM_LOG_CONF"),
    "CM_MAKE_CALL_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_MASK": SimTypeInt(signed=False, label="CM_MASK"),
    "CM_MCA_POS_DATA": SimStruct(OrderedDict((("AdapterId", SimTypeShort(signed=False, label="UInt16")), ("PosData1", SimTypeChar(label="Byte")), ("PosData2", SimTypeChar(label="Byte")), ("PosData3", SimTypeChar(label="Byte")), ("PosData4", SimTypeChar(label="Byte")),)), name="CM_MCA_POS_DATA", pack=False, align=None),
    "CM_MODIFY_CALL_QOS_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_MONITOR_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("HorizontalScreenSize", SimTypeShort(signed=False, label="UInt16")), ("VerticalScreenSize", SimTypeShort(signed=False, label="UInt16")), ("HorizontalResolution", SimTypeShort(signed=False, label="UInt16")), ("VerticalResolution", SimTypeShort(signed=False, label="UInt16")), ("HorizontalDisplayTimeLow", SimTypeShort(signed=False, label="UInt16")), ("HorizontalDisplayTime", SimTypeShort(signed=False, label="UInt16")), ("HorizontalDisplayTimeHigh", SimTypeShort(signed=False, label="UInt16")), ("HorizontalBackPorchLow", SimTypeShort(signed=False, label="UInt16")), ("HorizontalBackPorch", SimTypeShort(signed=False, label="UInt16")), ("HorizontalBackPorchHigh", SimTypeShort(signed=False, label="UInt16")), ("HorizontalFrontPorchLow", SimTypeShort(signed=False, label="UInt16")), ("HorizontalFrontPorch", SimTypeShort(signed=False, label="UInt16")), ("HorizontalFrontPorchHigh", SimTypeShort(signed=False, label="UInt16")), ("HorizontalSyncLow", SimTypeShort(signed=False, label="UInt16")), ("HorizontalSync", SimTypeShort(signed=False, label="UInt16")), ("HorizontalSyncHigh", SimTypeShort(signed=False, label="UInt16")), ("VerticalBackPorchLow", SimTypeShort(signed=False, label="UInt16")), ("VerticalBackPorch", SimTypeShort(signed=False, label="UInt16")), ("VerticalBackPorchHigh", SimTypeShort(signed=False, label="UInt16")), ("VerticalFrontPorchLow", SimTypeShort(signed=False, label="UInt16")), ("VerticalFrontPorch", SimTypeShort(signed=False, label="UInt16")), ("VerticalFrontPorchHigh", SimTypeShort(signed=False, label="UInt16")), ("VerticalSyncLow", SimTypeShort(signed=False, label="UInt16")), ("VerticalSync", SimTypeShort(signed=False, label="UInt16")), ("VerticalSyncHigh", SimTypeShort(signed=False, label="UInt16")),)), name="CM_MONITOR_DEVICE_DATA", pack=False, align=None),
    "CM_NOTIFY_ACTION": SimTypeInt(signed=False, label="CM_NOTIFY_ACTION"),
    "CM_NOTIFY_EVENT_DATA": SimStruct(OrderedDict((("FilterType", SimTypeInt(signed=False, label="CM_NOTIFY_FILTER_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"DeviceInterface": SimTypeRef("_DeviceInterface_e__Struct", SimStruct), "DeviceHandle": SimTypeRef("_DeviceHandle_e__Struct", SimStruct), "DeviceInstance": SimTypeRef("_DeviceInstance_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CM_NOTIFY_EVENT_DATA", pack=False, align=None),
    "CM_NOTIFY_FILTER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FilterType", SimTypeInt(signed=False, label="CM_NOTIFY_FILTER_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"DeviceInterface": SimTypeRef("_DeviceInterface_e__Struct", SimStruct), "DeviceHandle": SimTypeRef("_DeviceHandle_e__Struct", SimStruct), "DeviceInstance": SimTypeRef("_DeviceInstance_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CM_NOTIFY_FILTER", pack=False, align=None),
    "CM_NOTIFY_FILTER_TYPE": SimTypeInt(signed=False, label="CM_NOTIFY_FILTER_TYPE"),
    "CM_OPEN_AF_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_PARTIAL_RESOURCE_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("ShareDisposition", SimTypeChar(label="Byte")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("u", SimUnion({"Generic": SimTypeRef("_Generic_e__Struct", SimStruct), "Port": SimTypeRef("_Port_e__Struct", SimStruct), "Interrupt": SimTypeRef("_Interrupt_e__Struct", SimStruct), "MessageInterrupt": SimTypeRef("_MessageInterrupt_e__Struct", SimStruct), "Memory": SimTypeRef("_Memory_e__Struct", SimStruct), "Dma": SimTypeRef("_Dma_e__Struct", SimStruct), "DmaV3": SimTypeRef("_DmaV3_e__Struct", SimStruct), "DevicePrivate": SimTypeRef("_DevicePrivate_e__Struct", SimStruct), "BusNumber": SimTypeRef("_BusNumber_e__Struct", SimStruct), "DeviceSpecificData": SimTypeRef("_DeviceSpecificData_e__Struct", SimStruct), "Memory40": SimTypeRef("_Memory40_e__Struct", SimStruct), "Memory48": SimTypeRef("_Memory48_e__Struct", SimStruct), "Memory64": SimTypeRef("_Memory64_e__Struct", SimStruct), "Connection": SimTypeRef("_Connection_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CM_PARTIAL_RESOURCE_DESCRIPTOR", pack=False, align=None),
    "CM_PARTIAL_RESOURCE_LIST": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("PartialDescriptors", SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)),)), name="CM_PARTIAL_RESOURCE_LIST", pack=False, align=None),
    "CM_PCCARD_DEVICE_DATA": SimStruct(OrderedDict((("Flags", SimTypeChar(label="Byte")), ("ErrorCode", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("BusData", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("LegacyBaseAddress", SimTypeInt(signed=False, label="UInt32")), ("IRQMap", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="CM_PCCARD_DEVICE_DATA", pack=False, align=None),
    "CM_PNP_BIOS_DEVICE_NODE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Node", SimTypeChar(label="Byte")), ("ProductId", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("DeviceAttributes", SimTypeShort(signed=False, label="UInt16")),)), name="CM_PNP_BIOS_DEVICE_NODE", pack=False, align=None),
    "CM_PNP_BIOS_INSTALLATION_CHECK": SimStruct(OrderedDict((("Signature", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Revision", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("ControlField", SimTypeShort(signed=False, label="UInt16")), ("Checksum", SimTypeChar(label="Byte")), ("EventFlagAddress", SimTypeInt(signed=False, label="UInt32")), ("RealModeEntryOffset", SimTypeShort(signed=False, label="UInt16")), ("RealModeEntrySegment", SimTypeShort(signed=False, label="UInt16")), ("ProtectedModeEntryOffset", SimTypeShort(signed=False, label="UInt16")), ("ProtectedModeCodeBaseAddress", SimTypeInt(signed=False, label="UInt32")), ("OemDeviceId", SimTypeInt(signed=False, label="UInt32")), ("RealModeDataBaseAddress", SimTypeShort(signed=False, label="UInt16")), ("ProtectedModeDataBaseAddress", SimTypeInt(signed=False, label="UInt32")),)), name="CM_PNP_BIOS_INSTALLATION_CHECK", pack=False, align=None),
    "CM_POWER_DATA": SimStruct(OrderedDict((("PD_Size", SimTypeInt(signed=False, label="UInt32")), ("PD_MostRecentPowerState", SimTypeInt(signed=False, label="DEVICE_POWER_STATE")), ("PD_Capabilities", SimTypeInt(signed=False, label="UInt32")), ("PD_D1Latency", SimTypeInt(signed=False, label="UInt32")), ("PD_D2Latency", SimTypeInt(signed=False, label="UInt32")), ("PD_D3Latency", SimTypeInt(signed=False, label="UInt32")), ("PD_PowerStateMapping", SimTypeArray(SimTypeInt(signed=False, label="DEVICE_POWER_STATE"), 7)), ("PD_DeepestSystemWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")),)), name="CM_POWER_DATA", pack=False, align=None),
    "CM_PROB": SimTypeInt(signed=False, label="CM_PROB"),
    "CM_REENUMERATE_FLAGS": SimTypeInt(signed=False, label="CM_REENUMERATE_FLAGS"),
    "CM_REG_SAP_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CM_REMOVAL_POLICY": SimTypeInt(signed=False, label="CM_REMOVAL_POLICY"),
    "CM_RESOURCE_LIST": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("List", SimTypePointer(SimTypeRef("CM_FULL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)),)), name="CM_RESOURCE_LIST", pack=False, align=None),
    "CM_RESTYPE": SimTypeInt(signed=False, label="CM_RESTYPE"),
    "CM_ROM_BLOCK": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="CM_ROM_BLOCK", pack=False, align=None),
    "CM_SCSI_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("HostIdentifier", SimTypeChar(label="Byte")),)), name="CM_SCSI_DEVICE_DATA", pack=False, align=None),
    "CM_SERIAL_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("BaudClock", SimTypeInt(signed=False, label="UInt32")),)), name="CM_SERIAL_DEVICE_DATA", pack=False, align=None),
    "CM_SET_WIDTH_VALUE": SimTypeInt(signed=False, label="CM_SET_WIDTH_VALUE"),
    "CM_SHARE_DISPOSITION": SimTypeInt(signed=False, label="CM_SHARE_DISPOSITION"),
    "CM_SONIC_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("DataConfigurationRegister", SimTypeShort(signed=False, label="UInt16")), ("EthernetAddress", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="CM_SONIC_DEVICE_DATA", pack=False, align=None),
    "CM_STATE": SimTypeInt(signed=False, label="CM_STATE"),
    "CM_VIDEO_DEVICE_DATA": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("VideoClock", SimTypeInt(signed=False, label="UInt32")),)), name="CM_VIDEO_DEVICE_DATA", pack=False, align=None),
    "COAUTHIDENTITY": SimStruct(OrderedDict((("User", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="COAUTHIDENTITY", pack=False, align=None),
    "COAUTHINFO": SimStruct(OrderedDict((("dwAuthnSvc", SimTypeInt(signed=False, label="UInt32")), ("dwAuthzSvc", SimTypeInt(signed=False, label="UInt32")), ("pwszServerPrincName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAuthnLevel", SimTypeInt(signed=False, label="UInt32")), ("dwImpersonationLevel", SimTypeInt(signed=False, label="UInt32")), ("pAuthIdentityData", SimTypePointer(SimTypeRef("COAUTHIDENTITY", SimStruct), offset=0)), ("dwCapabilities", SimTypeInt(signed=False, label="UInt32")),)), name="COAUTHINFO", pack=False, align=None),
    "CODEBASEHOLD": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("szDistUnit", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szCodeBase", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwVersionLS", SimTypeInt(signed=False, label="UInt32")), ("dwStyle", SimTypeInt(signed=False, label="UInt32")),)), name="CODEBASEHOLD", pack=False, align=None),
    "CODEPAGE_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "CODEPAGE_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "COINIT": SimTypeInt(signed=False, label="COINIT"),
    "COINITBASE": SimTypeInt(signed=False, label="COINITBASE"),
    "COINSTALLER_CONTEXT_DATA": SimStruct(OrderedDict((("PostProcessing", SimTypeInt(signed=True, label="Int32")), ("InstallResult", SimTypeInt(signed=False, label="UInt32")), ("PrivateData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="COINSTALLER_CONTEXT_DATA", pack=False, align=None),
    "COLLAPSEBUTTONSTATES": SimTypeInt(signed=False, label="COLLAPSEBUTTONSTATES"),
    "COLOR": SimUnion({"gray": SimTypeRef("GRAYCOLOR", SimStruct), "rgb": SimTypeRef("RGBCOLOR", SimStruct), "cmyk": SimTypeRef("CMYKCOLOR", SimStruct), "XYZ": SimTypeRef("XYZCOLOR", SimStruct), "Yxy": SimTypeRef("YxyCOLOR", SimStruct), "Lab": SimTypeRef("LabCOLOR", SimStruct), "gen3ch": SimTypeRef("GENERIC3CHANNEL", SimStruct), "named": SimTypeRef("NAMEDCOLOR", SimStruct), "hifi": SimTypeRef("HiFiCOLOR", SimStruct), "Anonymous": SimStruct(OrderedDict((("reserved1", SimTypeInt(signed=False, label="UInt32")), ("reserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "COLORADJUSTMENT": SimStruct(OrderedDict((("caSize", SimTypeShort(signed=False, label="UInt16")), ("caFlags", SimTypeShort(signed=False, label="UInt16")), ("caIlluminantIndex", SimTypeShort(signed=False, label="UInt16")), ("caRedGamma", SimTypeShort(signed=False, label="UInt16")), ("caGreenGamma", SimTypeShort(signed=False, label="UInt16")), ("caBlueGamma", SimTypeShort(signed=False, label="UInt16")), ("caReferenceBlack", SimTypeShort(signed=False, label="UInt16")), ("caReferenceWhite", SimTypeShort(signed=False, label="UInt16")), ("caContrast", SimTypeShort(signed=True, label="Int16")), ("caBrightness", SimTypeShort(signed=True, label="Int16")), ("caColorfulness", SimTypeShort(signed=True, label="Int16")), ("caRedGreenTint", SimTypeShort(signed=True, label="Int16")),)), name="COLORADJUSTMENT", pack=False, align=None),
    "COLORDATATYPE": SimTypeInt(signed=False, label="COLORDATATYPE"),
    "COLORINFO": SimStruct(OrderedDict((("Red", SimTypeRef("CIECHROMA", SimStruct)), ("Green", SimTypeRef("CIECHROMA", SimStruct)), ("Blue", SimTypeRef("CIECHROMA", SimStruct)), ("Cyan", SimTypeRef("CIECHROMA", SimStruct)), ("Magenta", SimTypeRef("CIECHROMA", SimStruct)), ("Yellow", SimTypeRef("CIECHROMA", SimStruct)), ("AlignmentWhite", SimTypeRef("CIECHROMA", SimStruct)), ("RedGamma", SimTypeInt(signed=True, label="Int32")), ("GreenGamma", SimTypeInt(signed=True, label="Int32")), ("BlueGamma", SimTypeInt(signed=True, label="Int32")), ("MagentaInCyanDye", SimTypeInt(signed=True, label="Int32")), ("YellowInCyanDye", SimTypeInt(signed=True, label="Int32")), ("CyanInMagentaDye", SimTypeInt(signed=True, label="Int32")), ("YellowInMagentaDye", SimTypeInt(signed=True, label="Int32")), ("CyanInYellowDye", SimTypeInt(signed=True, label="Int32")), ("MagentaInYellowDye", SimTypeInt(signed=True, label="Int32")),)), name="COLORINFO", pack=False, align=None),
    "COLORKEY": SimStruct(OrderedDict((("KeyType", SimTypeInt(signed=False, label="UInt32")), ("PaletteIndex", SimTypeInt(signed=False, label="UInt32")), ("LowColorValue", SimTypeInt(signed=False, label="UInt32")), ("HighColorValue", SimTypeInt(signed=False, label="UInt32")),)), name="COLORKEY", pack=False, align=None),
    "COLORKEY_TYPE": SimTypeInt(signed=False, label="COLORKEY_TYPE"),
    "COLORMAP": SimStruct(OrderedDict((("from", SimTypeInt(signed=False, label="UInt32")), ("to", SimTypeInt(signed=False, label="UInt32")),)), name="COLORMAP", pack=False, align=None),
    "COLORMATCHSETUPA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pSourceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwRenderIntent", SimTypeInt(signed=False, label="UInt32")), ("dwProofingIntent", SimTypeInt(signed=False, label="UInt32")), ("pMonitorProfile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ccMonitorProfile", SimTypeInt(signed=False, label="UInt32")), ("pPrinterProfile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ccPrinterProfile", SimTypeInt(signed=False, label="UInt32")), ("pTargetProfile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ccTargetProfile", SimTypeInt(signed=False, label="UInt32")), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnApplyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COLORMATCHSETUPA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("lParamApplyCallback", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="COLORMATCHSETUPA", pack=False, align=None),
    "COLORMATCHSETUPW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pSourceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRenderIntent", SimTypeInt(signed=False, label="UInt32")), ("dwProofingIntent", SimTypeInt(signed=False, label="UInt32")), ("pMonitorProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ccMonitorProfile", SimTypeInt(signed=False, label="UInt32")), ("pPrinterProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ccPrinterProfile", SimTypeInt(signed=False, label="UInt32")), ("pTargetProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ccTargetProfile", SimTypeInt(signed=False, label="UInt32")), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnApplyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COLORMATCHSETUPW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("lParamApplyCallback", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="COLORMATCHSETUPW", pack=False, align=None),
    "COLORPROFILESUBTYPE": SimTypeInt(signed=False, label="COLORPROFILESUBTYPE"),
    "COLORPROFILETYPE": SimTypeInt(signed=False, label="COLORPROFILETYPE"),
    "COLORREF": SimTypeInt(signed=False, label="UInt32"),
    "COLORSCHEME": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("clrBtnHighlight", SimTypeInt(signed=False, label="UInt32")), ("clrBtnShadow", SimTypeInt(signed=False, label="UInt32")),)), name="COLORSCHEME", pack=False, align=None),
    "COLORSPACE_TRANSFORM": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_TYPE")), ("Data", SimUnion({"Rgb256x3x16": SimTypeRef("GAMMA_RAMP_RGB256x3x16", SimStruct), "Dxgi1": SimTypeRef("GAMMA_RAMP_DXGI_1", SimStruct), "T3x4": SimTypeRef("COLORSPACE_TRANSFORM_3x4", SimStruct), "MatrixV2": SimTypeRef("COLORSPACE_TRANSFORM_MATRIX_V2", SimStruct)}, name="<anon>", label="None")),)), name="COLORSPACE_TRANSFORM", pack=False, align=None),
    "COLORSPACE_TRANSFORM_1DLUT_CAP": SimStruct(OrderedDict((("NumberOfLUTEntries", SimTypeInt(signed=False, label="UInt32")), ("DataCap", SimTypeRef("COLORSPACE_TRANSFORM_DATA_CAP", SimStruct)),)), name="COLORSPACE_TRANSFORM_1DLUT_CAP", pack=False, align=None),
    "COLORSPACE_TRANSFORM_3x4": SimStruct(OrderedDict((("ColorMatrix3x4", SimTypeArray(SimTypeFloat(size=32), 12)), ("ScalarMultiplier", SimTypeFloat(size=32)), ("LookupTable1D", SimTypeArray(SimTypeRef("GAMMA_RAMP_RGB", SimStruct), 4096)),)), name="COLORSPACE_TRANSFORM_3x4", pack=False, align=None),
    "COLORSPACE_TRANSFORM_DATA_CAP": SimStruct(OrderedDict((("DataType", SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_DATA_TYPE")), ("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("NumericRangeMin", SimTypeFloat(size=32)), ("NumericRangeMax", SimTypeFloat(size=32)),)), name="COLORSPACE_TRANSFORM_DATA_CAP", pack=False, align=None),
    "COLORSPACE_TRANSFORM_DATA_TYPE": SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_DATA_TYPE"),
    "COLORSPACE_TRANSFORM_MATRIX_CAP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DataCap", SimTypeRef("COLORSPACE_TRANSFORM_DATA_CAP", SimStruct)),)), name="COLORSPACE_TRANSFORM_MATRIX_CAP", pack=False, align=None),
    "COLORSPACE_TRANSFORM_MATRIX_V2": SimStruct(OrderedDict((("StageControlLookupTable1DDegamma", SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_STAGE_CONTROL")), ("LookupTable1DDegamma", SimTypeArray(SimTypeRef("GAMMA_RAMP_RGB", SimStruct), 4096)), ("StageControlColorMatrix3x3", SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_STAGE_CONTROL")), ("ColorMatrix3x3", SimTypeArray(SimTypeFloat(size=32), 9)), ("StageControlLookupTable1DRegamma", SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_STAGE_CONTROL")), ("LookupTable1DRegamma", SimTypeArray(SimTypeRef("GAMMA_RAMP_RGB", SimStruct), 4096)),)), name="COLORSPACE_TRANSFORM_MATRIX_V2", pack=False, align=None),
    "COLORSPACE_TRANSFORM_SET_INPUT": SimStruct(OrderedDict((("OutputWireColorSpaceExpected", SimTypeInt(signed=False, label="OUTPUT_WIRE_COLOR_SPACE_TYPE")), ("OutputWireFormatExpected", SimTypeRef("OUTPUT_WIRE_FORMAT", SimStruct)), ("ColorSpaceTransform", SimTypeRef("COLORSPACE_TRANSFORM", SimStruct)),)), name="COLORSPACE_TRANSFORM_SET_INPUT", pack=False, align=None),
    "COLORSPACE_TRANSFORM_STAGE_CONTROL": SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_STAGE_CONTROL"),
    "COLORSPACE_TRANSFORM_TARGET_CAPS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_TARGET_CAPS_VERSION")), ("LookupTable1DDegammaCap", SimTypeRef("COLORSPACE_TRANSFORM_1DLUT_CAP", SimStruct)), ("ColorMatrix3x3Cap", SimTypeRef("COLORSPACE_TRANSFORM_MATRIX_CAP", SimStruct)), ("LookupTable1DRegammaCap", SimTypeRef("COLORSPACE_TRANSFORM_1DLUT_CAP", SimStruct)),)), name="COLORSPACE_TRANSFORM_TARGET_CAPS", pack=False, align=None),
    "COLORSPACE_TRANSFORM_TARGET_CAPS_VERSION": SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_TARGET_CAPS_VERSION"),
    "COLORSPACE_TRANSFORM_TYPE": SimTypeInt(signed=False, label="COLORSPACE_TRANSFORM_TYPE"),
    "COLORTYPE": SimTypeInt(signed=False, label="COLORTYPE"),
    "COLOR_MATCH_TO_TARGET_ACTION": SimTypeInt(signed=False, label="COLOR_MATCH_TO_TARGET_ACTION"),
    "COLUMNSET": SimStruct(OrderedDict((("cCol", SimTypeInt(signed=False, label="UInt32")), ("aCol", SimTypePointer(SimTypeRef("FULLPROPSPEC", SimStruct), offset=0)),)), name="COLUMNSET", pack=False, align=None),
    "COLUMNSTATUS": SimStruct(OrderedDict((("cPropCount", SimTypeInt(signed=True, label="Int32")), ("cPropsLoaded", SimTypeInt(signed=True, label="Int32")),)), name="COLUMNSTATUS", pack=False, align=None),
    "COMAdminAccessChecksLevelOptions": SimTypeInt(signed=False, label="COMAdminAccessChecksLevelOptions"),
    "COMAdminActivationOptions": SimTypeInt(signed=False, label="COMAdminActivationOptions"),
    "COMAdminApplicationExportOptions": SimTypeInt(signed=False, label="COMAdminApplicationExportOptions"),
    "COMAdminApplicationInstallOptions": SimTypeInt(signed=False, label="COMAdminApplicationInstallOptions"),
    "COMAdminAuthenticationCapabilitiesOptions": SimTypeInt(signed=False, label="COMAdminAuthenticationCapabilitiesOptions"),
    "COMAdminAuthenticationLevelOptions": SimTypeInt(signed=False, label="COMAdminAuthenticationLevelOptions"),
    "COMAdminComponentFlags": SimTypeInt(signed=False, label="COMAdminComponentFlags"),
    "COMAdminComponentType": SimTypeInt(signed=False, label="COMAdminComponentType"),
    "COMAdminErrorCodes": SimTypeInt(signed=False, label="COMAdminErrorCodes"),
    "COMAdminFileFlags": SimTypeInt(signed=False, label="COMAdminFileFlags"),
    "COMAdminImpersonationLevelOptions": SimTypeInt(signed=False, label="COMAdminImpersonationLevelOptions"),
    "COMAdminInUse": SimTypeInt(signed=False, label="COMAdminInUse"),
    "COMAdminOS": SimTypeInt(signed=False, label="COMAdminOS"),
    "COMAdminQCMessageAuthenticateOptions": SimTypeInt(signed=False, label="COMAdminQCMessageAuthenticateOptions"),
    "COMAdminServiceOptions": SimTypeInt(signed=False, label="COMAdminServiceOptions"),
    "COMAdminServiceStatusOptions": SimTypeInt(signed=False, label="COMAdminServiceStatusOptions"),
    "COMAdminSynchronizationOptions": SimTypeInt(signed=False, label="COMAdminSynchronizationOptions"),
    "COMAdminThreadingModels": SimTypeInt(signed=False, label="COMAdminThreadingModels"),
    "COMAdminTransactionOptions": SimTypeInt(signed=False, label="COMAdminTransactionOptions"),
    "COMAdminTxIsolationLevelOptions": SimTypeInt(signed=False, label="COMAdminTxIsolationLevelOptions"),
    "COMBOBOXEXITEMA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="COMBOBOX_EX_ITEM_FLAGS")), ("iItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iSelectedImage", SimTypeInt(signed=True, label="Int32")), ("iOverlay", SimTypeInt(signed=True, label="Int32")), ("iIndent", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="COMBOBOXEXITEMA", pack=False, align=None),
    "COMBOBOXEXITEMW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="COMBOBOX_EX_ITEM_FLAGS")), ("iItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iSelectedImage", SimTypeInt(signed=True, label="Int32")), ("iOverlay", SimTypeInt(signed=True, label="Int32")), ("iIndent", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="COMBOBOXEXITEMW", pack=False, align=None),
    "COMBOBOXINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcItem", SimTypeRef("RECT", SimStruct)), ("rcButton", SimTypeRef("RECT", SimStruct)), ("stateButton", SimTypeInt(signed=False, label="COMBOBOXINFO_BUTTON_STATE")), ("hwndCombo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndList", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="COMBOBOXINFO", pack=False, align=None),
    "COMBOBOXINFO_BUTTON_STATE": SimTypeInt(signed=False, label="COMBOBOXINFO_BUTTON_STATE"),
    "COMBOBOXPARTS": SimTypeInt(signed=False, label="COMBOBOXPARTS"),
    "COMBOBOXSTYLESTATES": SimTypeInt(signed=False, label="COMBOBOXSTYLESTATES"),
    "COMBOBOX_EX_ITEM_FLAGS": SimTypeInt(signed=False, label="COMBOBOX_EX_ITEM_FLAGS"),
    "COMDLG_FILTERSPEC": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSpec", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="COMDLG_FILTERSPEC", pack=False, align=None),
    "COMEVENTSYSCHANGEINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("changeType", SimTypeInt(signed=False, label="EOC_ChangeType")), ("objectId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("partitionId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("applicationId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("reserved", SimTypeArray(SimTypeBottom(label="Guid"), 10)),)), name="COMEVENTSYSCHANGEINFO", pack=False, align=None),
    "COMMANDLINKGLYPHSTATES": SimTypeInt(signed=False, label="COMMANDLINKGLYPHSTATES"),
    "COMMANDLINKSTATES": SimTypeInt(signed=False, label="COMMANDLINKSTATES"),
    "COMMCONFIG": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("dcb", SimTypeRef("DCB", SimStruct)), ("dwProviderSubType", SimTypeInt(signed=False, label="UInt32")), ("dwProviderOffset", SimTypeInt(signed=False, label="UInt32")), ("dwProviderSize", SimTypeInt(signed=False, label="UInt32")), ("wcProviderData", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="COMMCONFIG", pack=False, align=None),
    "COMMON_DLG_ERRORS": SimTypeInt(signed=False, label="COMMON_DLG_ERRORS"),
    "COMMPROP": SimStruct(OrderedDict((("wPacketLength", SimTypeShort(signed=False, label="UInt16")), ("wPacketVersion", SimTypeShort(signed=False, label="UInt16")), ("dwServiceMask", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTxQueue", SimTypeInt(signed=False, label="UInt32")), ("dwMaxRxQueue", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBaud", SimTypeInt(signed=False, label="UInt32")), ("dwProvSubType", SimTypeInt(signed=False, label="UInt32")), ("dwProvCapabilities", SimTypeInt(signed=False, label="UInt32")), ("dwSettableParams", SimTypeInt(signed=False, label="UInt32")), ("dwSettableBaud", SimTypeInt(signed=False, label="UInt32")), ("wSettableData", SimTypeShort(signed=False, label="UInt16")), ("wSettableStopParity", SimTypeInt(signed=False, label="COMMPROP_STOP_PARITY")), ("dwCurrentTxQueue", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentRxQueue", SimTypeInt(signed=False, label="UInt32")), ("dwProvSpec1", SimTypeInt(signed=False, label="UInt32")), ("dwProvSpec2", SimTypeInt(signed=False, label="UInt32")), ("wcProvChar", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="COMMPROP", pack=False, align=None),
    "COMMPROP_STOP_PARITY": SimTypeInt(signed=False, label="COMMPROP_STOP_PARITY"),
    "COMMTIMEOUTS": SimStruct(OrderedDict((("ReadIntervalTimeout", SimTypeInt(signed=False, label="UInt32")), ("ReadTotalTimeoutMultiplier", SimTypeInt(signed=False, label="UInt32")), ("ReadTotalTimeoutConstant", SimTypeInt(signed=False, label="UInt32")), ("WriteTotalTimeoutMultiplier", SimTypeInt(signed=False, label="UInt32")), ("WriteTotalTimeoutConstant", SimTypeInt(signed=False, label="UInt32")),)), name="COMMTIMEOUTS", pack=False, align=None),
    "COMMUNICATIONSPARTS": SimTypeInt(signed=False, label="COMMUNICATIONSPARTS"),
    "COMM_EVENT_MASK": SimTypeInt(signed=False, label="COMM_EVENT_MASK"),
    "COMM_FAULT_OFFSETS": SimStruct(OrderedDict((("CommOffset", SimTypeShort(signed=True, label="Int16")), ("FaultOffset", SimTypeShort(signed=True, label="Int16")),)), name="COMM_FAULT_OFFSETS", pack=False, align=None),
    "COMPACT_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="COMPACT_VIRTUAL_DISK_FLAG"),
    "COMPACT_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="COMPACT_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="COMPACT_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "COMPACT_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="COMPACT_VIRTUAL_DISK_VERSION"),
    "COMPAREITEMSTRUCT": SimStruct(OrderedDict((("CtlType", SimTypeInt(signed=False, label="DRAWITEMSTRUCT_CTL_TYPE")), ("CtlID", SimTypeInt(signed=False, label="UInt32")), ("hwndItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("itemID1", SimTypeInt(signed=False, label="UInt32")), ("itemData1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("itemID2", SimTypeInt(signed=False, label="UInt32")), ("itemData2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwLocaleId", SimTypeInt(signed=False, label="UInt32")),)), name="COMPAREITEMSTRUCT", pack=False, align=None),
    "COMPARESTRING_RESULT": SimTypeInt(signed=False, label="COMPARESTRING_RESULT"),
    "COMPARE_STRING_FLAGS": SimTypeInt(signed=False, label="COMPARE_STRING_FLAGS"),
    "COMPARTMENT_ID": SimTypeInt(signed=False, label="COMPARTMENT_ID"),
    "COMPATIBILITY_CONTEXT_ELEMENT": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("Type", SimTypeInt(signed=False, label="ACTCTX_COMPATIBILITY_ELEMENT_TYPE")), ("MaxVersionTested", SimTypeLongLong(signed=False, label="UInt64")),)), name="COMPATIBILITY_CONTEXT_ELEMENT", pack=False, align=None),
    "COMPCOLOR": SimStruct(OrderedDict((("crText", SimTypeInt(signed=False, label="UInt32")), ("crBackground", SimTypeInt(signed=False, label="UInt32")), ("dwEffects", SimTypeInt(signed=False, label="UInt32")),)), name="COMPCOLOR", pack=False, align=None),
    "COMPLETE_AUTH_TOKEN_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "COMPLETION_STATUS_FLAGS": SimTypeInt(signed=False, label="COMPLETION_STATUS_FLAGS"),
    "COMPLUS_APPTYPE": SimTypeInt(signed=False, label="COMPLUS_APPTYPE"),
    "COMPONENT_CHARACTERISTICS": SimTypeInt(signed=False, label="COMPONENT_CHARACTERISTICS"),
    "COMPONENT_FILTER": SimStruct(OrderedDict((("ComponentFlags", SimTypeInt(signed=False, label="UInt32")),)), name="COMPONENT_FILTER", pack=False, align=None),
    "COMPOSITIONFORM": SimStruct(OrderedDict((("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("ptCurrentPos", SimTypeRef("POINT", SimStruct)), ("rcArea", SimTypeRef("RECT", SimStruct)),)), name="COMPOSITIONFORM", pack=False, align=None),
    "COMPOSITIONSTRING": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCompReadAttrLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompReadAttrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCompReadClauseLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompReadClauseOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCompReadStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompReadStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCompAttrLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompAttrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCompClauseLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompClauseOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCompStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCursorPos", SimTypeInt(signed=False, label="UInt32")), ("dwDeltaStart", SimTypeInt(signed=False, label="UInt32")), ("dwResultReadClauseLen", SimTypeInt(signed=False, label="UInt32")), ("dwResultReadClauseOffset", SimTypeInt(signed=False, label="UInt32")), ("dwResultReadStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwResultReadStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwResultClauseLen", SimTypeInt(signed=False, label="UInt32")), ("dwResultClauseOffset", SimTypeInt(signed=False, label="UInt32")), ("dwResultStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwResultStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPrivateSize", SimTypeInt(signed=False, label="UInt32")), ("dwPrivateOffset", SimTypeInt(signed=False, label="UInt32")),)), name="COMPOSITIONSTRING", pack=False, align=None),
    "COMPOSITION_FRAME_ID_TYPE": SimTypeInt(signed=False, label="COMPOSITION_FRAME_ID_TYPE"),
    "COMPOSITION_FRAME_STATS": SimStruct(OrderedDict((("startTime", SimTypeLongLong(signed=False, label="UInt64")), ("targetTime", SimTypeLongLong(signed=False, label="UInt64")), ("framePeriod", SimTypeLongLong(signed=False, label="UInt64")),)), name="COMPOSITION_FRAME_STATS", pack=False, align=None),
    "COMPOSITION_STATS": SimStruct(OrderedDict((("presentCount", SimTypeInt(signed=False, label="UInt32")), ("refreshCount", SimTypeInt(signed=False, label="UInt32")), ("virtualRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("time", SimTypeLongLong(signed=False, label="UInt64")),)), name="COMPOSITION_STATS", pack=False, align=None),
    "COMPOSITION_TARGET_ID": SimStruct(OrderedDict((("displayAdapterLuid", SimTypeRef("LUID", SimStruct)), ("renderAdapterLuid", SimTypeRef("LUID", SimStruct)), ("vidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("vidPnTargetId", SimTypeInt(signed=False, label="UInt32")), ("uniqueId", SimTypeInt(signed=False, label="UInt32")),)), name="COMPOSITION_TARGET_ID", pack=False, align=None),
    "COMPOSITION_TARGET_STATS": SimStruct(OrderedDict((("outstandingPresents", SimTypeInt(signed=False, label="UInt32")), ("presentTime", SimTypeLongLong(signed=False, label="UInt64")), ("vblankDuration", SimTypeLongLong(signed=False, label="UInt64")), ("presentedStats", SimTypeRef("COMPOSITION_STATS", SimStruct)), ("completedStats", SimTypeRef("COMPOSITION_STATS", SimStruct)),)), name="COMPOSITION_TARGET_STATS", pack=False, align=None),
    "COMPRESSED_DATA_INFO": SimStruct(OrderedDict((("CompressionFormatAndEngine", SimTypeShort(signed=False, label="UInt16")), ("CompressionUnitShift", SimTypeChar(label="Byte")), ("ChunkShift", SimTypeChar(label="Byte")), ("ClusterShift", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("NumberOfChunks", SimTypeShort(signed=False, label="UInt16")), ("CompressedChunkSizes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="COMPRESSED_DATA_INFO", pack=False, align=None),
    "COMPRESSION_FORMAT": SimTypeInt(signed=False, label="COMPRESSION_FORMAT"),
    "COMPRESSOR_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "COMPRESS_ALGORITHM": SimTypeInt(signed=False, label="COMPRESS_ALGORITHM"),
    "COMPRESS_ALLOCATION_ROUTINES": SimStruct(OrderedDict((("Allocate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["UserContext", "Size"]), offset=0)), ("Free", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["UserContext", "Memory"]), offset=0)), ("UserContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="COMPRESS_ALLOCATION_ROUTINES", pack=False, align=None),
    "COMPRESS_INFORMATION_CLASS": SimTypeInt(signed=False, label="COMPRESS_INFORMATION_CLASS"),
    "COMPROPSHEETUI": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("hInstCaller", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pCallerName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("UserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pfnCallBack", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CPSUICBPARAM", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCPSUICBParam"]), offset=0)), ("pOptItem", SimTypePointer(SimTypeRef("OPTITEM", SimStruct), offset=0)), ("pDlgPage", SimTypePointer(SimTypeRef("DLGPAGE", SimStruct), offset=0)), ("cOptItem", SimTypeShort(signed=False, label="UInt16")), ("cDlgPage", SimTypeShort(signed=False, label="UInt16")), ("IconID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pOptItemName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("CallerVersion", SimTypeShort(signed=False, label="UInt16")), ("OptItemVersion", SimTypeShort(signed=False, label="UInt16")), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 4)),)), name="COMPROPSHEETUI", pack=False, align=None),
    "COMPUTER_NAME_FORMAT": SimTypeInt(signed=False, label="COMPUTER_NAME_FORMAT"),
    "COMPVARS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hic", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("lpbiIn", SimTypePointer(SimTypeRef("BITMAPINFO", SimStruct), offset=0)), ("lpbiOut", SimTypePointer(SimTypeRef("BITMAPINFO", SimStruct), offset=0)), ("lpBitsOut", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpBitsPrev", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lFrame", SimTypeInt(signed=True, label="Int32")), ("lKey", SimTypeInt(signed=True, label="Int32")), ("lDataRate", SimTypeInt(signed=True, label="Int32")), ("lQ", SimTypeInt(signed=True, label="Int32")), ("lKeyCount", SimTypeInt(signed=True, label="Int32")), ("lpState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbState", SimTypeInt(signed=True, label="Int32")),)), name="COMPVARS", pack=False, align=None),
    "COMSD": SimTypeInt(signed=False, label="COMSD"),
    "COMSTAT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("cbInQue", SimTypeInt(signed=False, label="UInt32")), ("cbOutQue", SimTypeInt(signed=False, label="UInt32")),)), name="COMSTAT", pack=False, align=None),
    "COMSVCSEVENTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwPid", SimTypeInt(signed=False, label="UInt32")), ("lTime", SimTypeLongLong(signed=True, label="Int64")), ("lMicroTime", SimTypeInt(signed=True, label="Int32")), ("perfCount", SimTypeLongLong(signed=True, label="Int64")), ("guidApp", SimTypeBottom(label="Guid")), ("sMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="COMSVCSEVENTINFO", pack=False, align=None),
    "CONDITIONPROPERTIES": SimTypeInt(signed=False, label="CONDITIONPROPERTIES"),
    "CONDITION_CREATION_OPTIONS": SimTypeInt(signed=False, label="CONDITION_CREATION_OPTIONS"),
    "CONDITION_OPERATION": SimTypeInt(signed=False, label="CONDITION_OPERATION"),
    "CONDITION_TYPE": SimTypeInt(signed=False, label="CONDITION_TYPE"),
    "CONDITION_VARIABLE": SimStruct(OrderedDict((("Ptr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CONDITION_VARIABLE", pack=False, align=None),
    "CONFIDENCE_LEVEL": SimTypeInt(signed=False, label="CONFIDENCE_LEVEL"),
    "CONFIGRET": SimTypeInt(signed=False, label="CONFIGRET"),
    "CONFIGURATION_ENTRY": SimStruct(OrderedDict((("bstrKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CONFIGURATION_ENTRY", pack=False, align=None),
    "CONFIGURATION_INFORMATION": SimStruct(OrderedDict((("DiskCount", SimTypeInt(signed=False, label="UInt32")), ("FloppyCount", SimTypeInt(signed=False, label="UInt32")), ("CdRomCount", SimTypeInt(signed=False, label="UInt32")), ("TapeCount", SimTypeInt(signed=False, label="UInt32")), ("ScsiPortCount", SimTypeInt(signed=False, label="UInt32")), ("SerialCount", SimTypeInt(signed=False, label="UInt32")), ("ParallelCount", SimTypeInt(signed=False, label="UInt32")), ("AtDiskPrimaryAddressClaimed", SimTypeChar(label="Byte")), ("AtDiskSecondaryAddressClaimed", SimTypeChar(label="Byte")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("MediumChangerCount", SimTypeInt(signed=False, label="UInt32")),)), name="CONFIGURATION_INFORMATION", pack=False, align=None),
    "CONFIGURATION_TYPE": SimTypeInt(signed=False, label="CONFIGURATION_TYPE"),
    "CONFIG_CI_PROV_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwPolicies", SimTypeInt(signed=False, label="UInt32")), ("pPolicies", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("result", SimTypeRef("CONFIG_CI_PROV_INFO_RESULT", SimStruct)), ("dwScenario", SimTypeInt(signed=False, label="UInt32")), ("result2", SimTypePointer(SimTypeRef("CONFIG_CI_PROV_INFO_RESULT2", SimStruct), offset=0)),)), name="CONFIG_CI_PROV_INFO", pack=False, align=None),
    "CONFIG_CI_PROV_INFO_RESULT": SimStruct(OrderedDict((("hr", SimTypeInt(signed=True, label="Int32")), ("dwResult", SimTypeInt(signed=False, label="UInt32")), ("dwPolicyIndex", SimTypeInt(signed=False, label="UInt32")), ("fIsExplicitDeny", SimTypeChar(label="Byte")),)), name="CONFIG_CI_PROV_INFO_RESULT", pack=False, align=None),
    "CONFIG_CI_PROV_INFO_RESULT2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hr", SimTypeInt(signed=True, label="Int32")), ("dwResult", SimTypeInt(signed=False, label="UInt32")), ("dwPolicyIndex", SimTypeInt(signed=False, label="UInt32")), ("fIsExplicitDeny", SimTypeChar(label="Byte")), ("cbCalculatedFileHash", SimTypeInt(signed=False, label="UInt32")), ("pbCalculatedFileHash", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CONFIG_CI_PROV_INFO_RESULT2", pack=False, align=None),
    "CONFIG_INFO_0": SimStruct(OrderedDict((("cfgi0_key", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cfgi0_data", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CONFIG_INFO_0", pack=False, align=None),
    "CONFIG_INFO_DATA_1": SimStruct(OrderedDict((("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")),)), name="CONFIG_INFO_DATA_1", pack=False, align=None),
    "CONFIRMSAFETY": SimStruct(OrderedDict((("clsid", SimTypeBottom(label="Guid")), ("pUnk", SimTypeBottom(label="IUnknown")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CONFIRMSAFETY", pack=False, align=None),
    "CONFIRM_CONFLICT_ITEM": SimStruct(OrderedDict((("pShellItem", SimTypeBottom(label="IShellItem2")), ("pszOriginalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszAlternateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLocationShort", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLocationFull", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nType", SimTypeInt(signed=False, label="SYNCMGR_CONFLICT_ITEM_TYPE")),)), name="CONFIRM_CONFLICT_ITEM", pack=False, align=None),
    "CONFIRM_CONFLICT_RESULT_INFO": SimStruct(OrderedDict((("pszNewName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("iItemIndex", SimTypeInt(signed=False, label="UInt32")),)), name="CONFIRM_CONFLICT_RESULT_INFO", pack=False, align=None),
    "CONFLICT_DETAILS_A": SimStruct(OrderedDict((("CD_ulSize", SimTypeInt(signed=False, label="UInt32")), ("CD_ulMask", SimTypeInt(signed=False, label="CM_CDMASK")), ("CD_dnDevInst", SimTypeInt(signed=False, label="UInt32")), ("CD_rdResDes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CD_ulFlags", SimTypeInt(signed=False, label="CM_CDFLAGS")), ("CD_szDescription", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="CONFLICT_DETAILS_A", pack=False, align=None),
    "CONFLICT_DETAILS_W": SimStruct(OrderedDict((("CD_ulSize", SimTypeInt(signed=False, label="UInt32")), ("CD_ulMask", SimTypeInt(signed=False, label="CM_CDMASK")), ("CD_dnDevInst", SimTypeInt(signed=False, label="UInt32")), ("CD_rdResDes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CD_ulFlags", SimTypeInt(signed=False, label="CM_CDFLAGS")), ("CD_szDescription", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CONFLICT_DETAILS_W", pack=False, align=None),
    "CONFLICT_RESOLUTION_POLICY": SimTypeInt(signed=False, label="CONFLICT_RESOLUTION_POLICY"),
    "CONNECTDATA": SimStruct(OrderedDict((("pUnk", SimTypeBottom(label="IUnknown")), ("dwCookie", SimTypeInt(signed=False, label="UInt32")),)), name="CONNECTDATA", pack=False, align=None),
    "CONNECTDLGSTRUCTA": SimStruct(OrderedDict((("cbStructure", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpConnRes", SimTypePointer(SimTypeRef("NETRESOURCEA", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CONNECTDLGSTRUCT_FLAGS")), ("dwDevNum", SimTypeInt(signed=False, label="UInt32")),)), name="CONNECTDLGSTRUCTA", pack=False, align=None),
    "CONNECTDLGSTRUCTW": SimStruct(OrderedDict((("cbStructure", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpConnRes", SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CONNECTDLGSTRUCT_FLAGS")), ("dwDevNum", SimTypeInt(signed=False, label="UInt32")),)), name="CONNECTDLGSTRUCTW", pack=False, align=None),
    "CONNECTDLGSTRUCT_FLAGS": SimTypeInt(signed=False, label="CONNECTDLGSTRUCT_FLAGS"),
    "CONNECTION": SimStruct(OrderedDict((("usSource", SimTypeShort(signed=False, label="UInt16")), ("usControl", SimTypeShort(signed=False, label="UInt16")), ("usDestination", SimTypeShort(signed=False, label="UInt16")), ("usTransform", SimTypeShort(signed=False, label="UInt16")), ("lScale", SimTypeInt(signed=True, label="Int32")),)), name="CONNECTION", pack=False, align=None),
    "CONNECTIONLIST": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cConnections", SimTypeInt(signed=False, label="UInt32")),)), name="CONNECTIONLIST", pack=False, align=None),
    "CONNECTION_CHANGE_NOTIFICATION": SimTypeInt(signed=False, label="CONNECTION_CHANGE_NOTIFICATION"),
    "CONNECTION_DES": SimStruct(OrderedDict((("COND_Type", SimTypeInt(signed=False, label="UInt32")), ("COND_Flags", SimTypeInt(signed=False, label="UInt32")), ("COND_Class", SimTypeChar(label="Byte")), ("COND_ClassType", SimTypeChar(label="Byte")), ("COND_Reserved1", SimTypeChar(label="Byte")), ("COND_Reserved2", SimTypeChar(label="Byte")), ("COND_Id", SimTypeLongLong(signed=True, label="Int64")),)), name="CONNECTION_DES", pack=False, align=None),
    "CONNECTION_INFO_0": SimStruct(OrderedDict((("coni0_id", SimTypeInt(signed=False, label="UInt32")),)), name="CONNECTION_INFO_0", pack=False, align=None),
    "CONNECTION_INFO_1": SimStruct(OrderedDict((("coni1_id", SimTypeInt(signed=False, label="UInt32")), ("coni1_type", SimTypeInt(signed=False, label="SHARE_TYPE")), ("coni1_num_opens", SimTypeInt(signed=False, label="UInt32")), ("coni1_num_users", SimTypeInt(signed=False, label="UInt32")), ("coni1_time", SimTypeInt(signed=False, label="UInt32")), ("coni1_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("coni1_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CONNECTION_INFO_1", pack=False, align=None),
    "CONNECTION_RESOURCE": SimStruct(OrderedDict((("Connection_Header", SimTypeRef("CONNECTION_DES", SimStruct)),)), name="CONNECTION_RESOURCE", pack=False, align=None),
    "CONSOLECONTROL": SimTypeInt(signed=False, label="CONSOLECONTROL"),
    "CONSOLEENDTASK": SimStruct(OrderedDict((("ProcessId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ConsoleEventCode", SimTypeInt(signed=False, label="UInt32")), ("ConsoleFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CONSOLEENDTASK", pack=False, align=None),
    "CONSOLESETFOREGROUND": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("bForeground", SimTypeInt(signed=True, label="Int32")),)), name="CONSOLESETFOREGROUND", pack=False, align=None),
    "CONSOLEWINDOWOWNER": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")),)), name="CONSOLEWINDOWOWNER", pack=False, align=None),
    "CONSOLE_CARET_INFO": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="CONSOLE_CARET_INFO", pack=False, align=None),
    "CONSOLE_CHARACTER_ATTRIBUTES": SimTypeInt(signed=False, label="CONSOLE_CHARACTER_ATTRIBUTES"),
    "CONSOLE_CURSOR_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("bVisible", SimTypeInt(signed=True, label="Int32")),)), name="CONSOLE_CURSOR_INFO", pack=False, align=None),
    "CONSOLE_FONT_INFO": SimStruct(OrderedDict((("nFont", SimTypeInt(signed=False, label="UInt32")), ("dwFontSize", SimTypeRef("COORD", SimStruct)),)), name="CONSOLE_FONT_INFO", pack=False, align=None),
    "CONSOLE_FONT_INFOEX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("nFont", SimTypeInt(signed=False, label="UInt32")), ("dwFontSize", SimTypeRef("COORD", SimStruct)), ("FontFamily", SimTypeInt(signed=False, label="UInt32")), ("FontWeight", SimTypeInt(signed=False, label="UInt32")), ("FaceName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="CONSOLE_FONT_INFOEX", pack=False, align=None),
    "CONSOLE_HISTORY_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("HistoryBufferSize", SimTypeInt(signed=False, label="UInt32")), ("NumberOfHistoryBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CONSOLE_HISTORY_INFO", pack=False, align=None),
    "CONSOLE_MODE": SimTypeInt(signed=False, label="CONSOLE_MODE"),
    "CONSOLE_PROCESS_INFO": SimStruct(OrderedDict((("dwProcessID", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CONSOLE_PROCESS_INFO", pack=False, align=None),
    "CONSOLE_READCONSOLE_CONTROL": SimStruct(OrderedDict((("nLength", SimTypeInt(signed=False, label="UInt32")), ("nInitialChars", SimTypeInt(signed=False, label="UInt32")), ("dwCtrlWakeupMask", SimTypeInt(signed=False, label="UInt32")), ("dwControlKeyState", SimTypeInt(signed=False, label="UInt32")),)), name="CONSOLE_READCONSOLE_CONTROL", pack=False, align=None),
    "CONSOLE_SCREEN_BUFFER_INFO": SimStruct(OrderedDict((("dwSize", SimTypeRef("COORD", SimStruct)), ("dwCursorPosition", SimTypeRef("COORD", SimStruct)), ("wAttributes", SimTypeInt(signed=False, label="CONSOLE_CHARACTER_ATTRIBUTES")), ("srWindow", SimTypeRef("SMALL_RECT", SimStruct)), ("dwMaximumWindowSize", SimTypeRef("COORD", SimStruct)),)), name="CONSOLE_SCREEN_BUFFER_INFO", pack=False, align=None),
    "CONSOLE_SCREEN_BUFFER_INFOEX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeRef("COORD", SimStruct)), ("dwCursorPosition", SimTypeRef("COORD", SimStruct)), ("wAttributes", SimTypeInt(signed=False, label="CONSOLE_CHARACTER_ATTRIBUTES")), ("srWindow", SimTypeRef("SMALL_RECT", SimStruct)), ("dwMaximumWindowSize", SimTypeRef("COORD", SimStruct)), ("wPopupAttributes", SimTypeShort(signed=False, label="UInt16")), ("bFullscreenSupported", SimTypeInt(signed=True, label="Int32")), ("ColorTable", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)),)), name="CONSOLE_SCREEN_BUFFER_INFOEX", pack=False, align=None),
    "CONSOLE_SELECTION_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSelectionAnchor", SimTypeRef("COORD", SimStruct)), ("srSelection", SimTypeRef("SMALL_RECT", SimStruct)),)), name="CONSOLE_SELECTION_INFO", pack=False, align=None),
    "CONSTRAINT_CONFLICT_REASON": SimTypeInt(signed=False, label="CONSTRAINT_CONFLICT_REASON"),
    "CONSTRICTOR_OPTION": SimTypeInt(signed=False, label="CONSTRICTOR_OPTION"),
    "CONTACT_AGGREGATION_BLOB": SimStruct(OrderedDict((("dwCount", SimTypeInt(signed=False, label="UInt32")), ("lpb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CONTACT_AGGREGATION_BLOB", pack=False, align=None),
    "CONTACT_AGGREGATION_COLLECTION_OPTIONS": SimTypeInt(signed=False, label="CONTACT_AGGREGATION_COLLECTION_OPTIONS"),
    "CONTACT_AGGREGATION_CREATE_OR_OPEN_OPTIONS": SimTypeInt(signed=False, label="CONTACT_AGGREGATION_CREATE_OR_OPEN_OPTIONS"),
    "CONTAINER_ROOT_INFO_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="CONTAINER_ROOT_INFO_INPUT", pack=False, align=None),
    "CONTAINER_ROOT_INFO_OUTPUT": SimStruct(OrderedDict((("ContainerRootIdLength", SimTypeShort(signed=False, label="UInt16")), ("ContainerRootId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CONTAINER_ROOT_INFO_OUTPUT", pack=False, align=None),
    "CONTAINER_VOLUME_STATE": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="CONTAINER_VOLUME_STATE", pack=False, align=None),
    "CONTENTALIGNMENT": SimTypeInt(signed=False, label="CONTENTALIGNMENT"),
    "CONTENTAREASTATES": SimTypeInt(signed=False, label="CONTENTAREASTATES"),
    "CONTENTLINKSTATES": SimTypeInt(signed=False, label="CONTENTLINKSTATES"),
    "CONTENTPANESTATES": SimTypeInt(signed=False, label="CONTENTPANESTATES"),
    "CONTENTRESTRICTION": SimStruct(OrderedDict((("prop", SimTypeRef("FULLPROPSPEC", SimStruct)), ("pwcsPhrase", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("ulGenerateMethod", SimTypeInt(signed=False, label="UInt32")),)), name="CONTENTRESTRICTION", pack=False, align=None),
    "CONTENT_MISSING_EVENT_DATA": SimStruct(OrderedDict((("cbContentMissingEventData", SimTypeInt(signed=False, label="UInt32")), ("ApplicationId", SimTypeBottom(label="Guid")), ("EndpointId", SimTypeBottom(label="Guid")), ("ContentId", SimTypeInt(signed=False, label="UInt32")),)), name="CONTENT_MISSING_EVENT_DATA", pack=False, align=None),
    "CONTEXT": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=False, label="CONTEXT_FLAGS")), ("Dr0", SimTypeInt(signed=False, label="UInt32")), ("Dr1", SimTypeInt(signed=False, label="UInt32")), ("Dr2", SimTypeInt(signed=False, label="UInt32")), ("Dr3", SimTypeInt(signed=False, label="UInt32")), ("Dr6", SimTypeInt(signed=False, label="UInt32")), ("Dr7", SimTypeInt(signed=False, label="UInt32")), ("FloatSave", SimTypeRef("FLOATING_SAVE_AREA", SimStruct)), ("SegGs", SimTypeInt(signed=False, label="UInt32")), ("SegFs", SimTypeInt(signed=False, label="UInt32")), ("SegEs", SimTypeInt(signed=False, label="UInt32")), ("SegDs", SimTypeInt(signed=False, label="UInt32")), ("Edi", SimTypeInt(signed=False, label="UInt32")), ("Esi", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebp", SimTypeInt(signed=False, label="UInt32")), ("Eip", SimTypeInt(signed=False, label="UInt32")), ("SegCs", SimTypeInt(signed=False, label="UInt32")), ("EFlags", SimTypeInt(signed=False, label="UInt32")), ("Esp", SimTypeInt(signed=False, label="UInt32")), ("SegSs", SimTypeInt(signed=False, label="UInt32")), ("ExtendedRegisters", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="CONTEXT", pack=False, align=None),
    "CONTEXTMENUITEM": SimStruct(OrderedDict((("strName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("strStatusBarText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lCommandID", SimTypeInt(signed=True, label="Int32")), ("lInsertionPointID", SimTypeInt(signed=True, label="Int32")), ("fFlags", SimTypeInt(signed=True, label="Int32")), ("fSpecialFlags", SimTypeInt(signed=True, label="Int32")),)), name="CONTEXTMENUITEM", pack=False, align=None),
    "CONTEXTMENUITEM2": SimStruct(OrderedDict((("strName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("strStatusBarText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lCommandID", SimTypeInt(signed=True, label="Int32")), ("lInsertionPointID", SimTypeInt(signed=True, label="Int32")), ("fFlags", SimTypeInt(signed=True, label="Int32")), ("fSpecialFlags", SimTypeInt(signed=True, label="Int32")), ("strLanguageIndependentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CONTEXTMENUITEM2", pack=False, align=None),
    "CONTEXT_FLAGS": SimTypeInt(signed=False, label="CONTEXT_FLAGS"),
    "CONTRESCR10WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="CONTRESCR10WAVEFORMAT", pack=False, align=None),
    "CONTRESVQLPCWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="CONTRESVQLPCWAVEFORMAT", pack=False, align=None),
    "CONTROLINFO": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("hAccel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cAccel", SimTypeShort(signed=False, label="UInt16")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CONTROLINFO", pack=False, align=None),
    "CONTROLLABELSTATES": SimTypeInt(signed=False, label="CONTROLLABELSTATES"),
    "CONTROLLER_OBJECT": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=True, label="Int16")), ("ControllerExtension", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DeviceWaitQueue", SimTypeRef("KDEVICE_QUEUE", SimStruct)), ("Spare1", SimTypeInt(signed=False, label="UInt32")), ("Spare2", SimTypeLongLong(signed=True, label="Int64")),)), name="CONTROLLER_OBJECT", pack=False, align=None),
    "CONTROLPANELPARTS": SimTypeInt(signed=False, label="CONTROLPANELPARTS"),
    "CONTROLTRACE_HANDLE": SimStruct(OrderedDict((("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="CONTROLTRACE_HANDLE", pack=False, align=None),
    "CONTROL_CHANNEL_TRIGGER_STATUS": SimTypeInt(signed=False, label="CONTROL_CHANNEL_TRIGGER_STATUS"),
    "CONTROL_SERVICE": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Service", SimTypeInt(signed=False, label="UInt32")), ("Overrides", SimTypeRef("AD_GENERAL_PARAMS", SimStruct)), ("Anonymous", SimUnion({"Guaranteed": SimTypeRef("AD_GUARANTEED", SimStruct), "ParamBuffer": SimTypePointer(SimTypeRef("PARAM_BUFFER", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CONTROL_SERVICE", pack=False, align=None),
    "CONVCONTEXT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("wFlags", SimTypeInt(signed=False, label="UInt32")), ("wCountryID", SimTypeInt(signed=False, label="UInt32")), ("iCodePage", SimTypeInt(signed=True, label="Int32")), ("dwLangID", SimTypeInt(signed=False, label="UInt32")), ("dwSecurity", SimTypeInt(signed=False, label="UInt32")), ("qos", SimTypeRef("SECURITY_QUALITY_OF_SERVICE", SimStruct)),)), name="CONVCONTEXT", pack=False, align=None),
    "CONVINFO": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("hUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hConvPartner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszSvcPartner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszServiceReq", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszTopic", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wFmt", SimTypeInt(signed=False, label="UInt32")), ("wType", SimTypeInt(signed=False, label="DDE_CLIENT_TRANSACTION_TYPE")), ("wStatus", SimTypeInt(signed=False, label="CONVINFO_STATUS")), ("wConvst", SimTypeInt(signed=False, label="CONVINFO_CONVERSATION_STATE")), ("wLastError", SimTypeInt(signed=False, label="UInt32")), ("hConvList", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ConvCtxt", SimTypeRef("CONVCONTEXT", SimStruct)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndPartner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CONVINFO", pack=False, align=None),
    "CONVINFO_CONVERSATION_STATE": SimTypeInt(signed=False, label="CONVINFO_CONVERSATION_STATE"),
    "CONVINFO_STATUS": SimTypeInt(signed=False, label="CONVINFO_STATUS"),
    "COOKIE_DLG_INFO": SimStruct(OrderedDict((("pszServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pic", SimTypePointer(SimTypeRef("INTERNET_COOKIE", SimStruct), offset=0)), ("dwStopWarning", SimTypeInt(signed=False, label="UInt32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")), ("pszHeader", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwOperation", SimTypeInt(signed=False, label="UInt32")),)), name="COOKIE_DLG_INFO", pack=False, align=None),
    "COORD": SimStruct(OrderedDict((("X", SimTypeShort(signed=True, label="Int16")), ("Y", SimTypeShort(signed=True, label="Int16")),)), name="COORD", pack=False, align=None),
    "COORD_SYSTEM": SimTypeInt(signed=False, label="COORD_SYSTEM"),
    "COPPEventBlockReason": SimTypeInt(signed=False, label="COPPEventBlockReason"),
    "COPP_ACP_Protection_Level": SimTypeInt(signed=False, label="COPP_ACP_Protection_Level"),
    "COPP_BusType": SimTypeInt(signed=False, label="COPP_BusType"),
    "COPP_CGMSA_Protection_Level": SimTypeInt(signed=False, label="COPP_CGMSA_Protection_Level"),
    "COPP_ConnectorType": SimTypeInt(signed=False, label="COPP_ConnectorType"),
    "COPP_HDCP_Protection_Level": SimTypeInt(signed=False, label="COPP_HDCP_Protection_Level"),
    "COPP_ImageAspectRatio_EN300294": SimTypeInt(signed=False, label="COPP_ImageAspectRatio_EN300294"),
    "COPP_StatusFlags": SimTypeInt(signed=False, label="COPP_StatusFlags"),
    "COPP_StatusHDCPFlags": SimTypeInt(signed=False, label="COPP_StatusHDCPFlags"),
    "COPP_TVProtectionStandard": SimTypeInt(signed=False, label="COPP_TVProtectionStandard"),
    "COPYDATASTRUCT": SimStruct(OrderedDict((("dwData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="COPYDATASTRUCT", pack=False, align=None),
    "COPYFILE2_COPY_PHASE": SimTypeInt(signed=False, label="COPYFILE2_COPY_PHASE"),
    "COPYFILE2_EXTENDED_PARAMETERS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCopyFlags", SimTypeInt(signed=False, label="UInt32")), ("pfCancel", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("pProgressRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COPYFILE2_MESSAGE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="COPYFILE2_MESSAGE_ACTION"), arg_names=["pMessage", "pvCallbackContext"]), offset=0)), ("pvCallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="COPYFILE2_EXTENDED_PARAMETERS", pack=False, align=None),
    "COPYFILE2_EXTENDED_PARAMETERS_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCopyFlags", SimTypeInt(signed=False, label="UInt32")), ("pfCancel", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("pProgressRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COPYFILE2_MESSAGE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="COPYFILE2_MESSAGE_ACTION"), arg_names=["pMessage", "pvCallbackContext"]), offset=0)), ("pvCallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCopyFlagsV2", SimTypeInt(signed=False, label="UInt32")), ("ioDesiredSize", SimTypeInt(signed=False, label="UInt32")), ("ioDesiredRate", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 8)),)), name="COPYFILE2_EXTENDED_PARAMETERS_V2", pack=False, align=None),
    "COPYFILE2_MESSAGE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="COPYFILE2_MESSAGE_TYPE")), ("dwPadding", SimTypeInt(signed=False, label="UInt32")), ("Info", SimUnion({"ChunkStarted": SimTypeRef("_ChunkStarted_e__Struct", SimStruct), "ChunkFinished": SimTypeRef("_ChunkFinished_e__Struct", SimStruct), "StreamStarted": SimTypeRef("_StreamStarted_e__Struct", SimStruct), "StreamFinished": SimTypeRef("_StreamFinished_e__Struct", SimStruct), "PollContinue": SimTypeRef("_PollContinue_e__Struct", SimStruct), "Error": SimTypeRef("_Error_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="COPYFILE2_MESSAGE", pack=False, align=None),
    "COPYFILE2_MESSAGE_ACTION": SimTypeInt(signed=False, label="COPYFILE2_MESSAGE_ACTION"),
    "COPYFILE2_MESSAGE_TYPE": SimTypeInt(signed=False, label="COPYFILE2_MESSAGE_TYPE"),
    "COPYSTATES": SimTypeInt(signed=False, label="COPYSTATES"),
    "COPY_INFORMATION": SimStruct(OrderedDict((("SourceFileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("SourceFileOffset", SimTypeLongLong(signed=True, label="Int64")),)), name="COPY_INFORMATION", pack=False, align=None),
    "CORE_PRINTER_DRIVERA": SimStruct(OrderedDict((("CoreDriverGUID", SimTypeBottom(label="Guid")), ("ftDriverDate", SimTypeRef("FILETIME", SimStruct)), ("dwlDriverVersion", SimTypeLongLong(signed=False, label="UInt64")), ("szPackageID", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="CORE_PRINTER_DRIVERA", pack=False, align=None),
    "CORE_PRINTER_DRIVERW": SimStruct(OrderedDict((("CoreDriverGUID", SimTypeBottom(label="Guid")), ("ftDriverDate", SimTypeRef("FILETIME", SimStruct)), ("dwlDriverVersion", SimTypeLongLong(signed=False, label="UInt64")), ("szPackageID", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CORE_PRINTER_DRIVERW", pack=False, align=None),
    "CORRECTIVE_ACTION": SimTypeInt(signed=False, label="CORRECTIVE_ACTION"),
    "CORRELATION_VECTOR": SimStruct(OrderedDict((("Version", SimTypeChar(label="SByte")), ("Vector", SimTypeArray(SimTypeChar(label="SByte"), 129)),)), name="CORRELATION_VECTOR", pack=False, align=None),
    "COR_DEBUG_IL_TO_NATIVE_MAP": SimStruct(OrderedDict((("ilOffset", SimTypeInt(signed=False, label="UInt32")), ("nativeStartOffset", SimTypeInt(signed=False, label="UInt32")), ("nativeEndOffset", SimTypeInt(signed=False, label="UInt32")),)), name="COR_DEBUG_IL_TO_NATIVE_MAP", pack=False, align=None),
    "COR_GC_STATS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ExplicitGCCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("GenCollectionsTaken", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)), ("CommittedKBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ReservedKBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Gen0HeapSizeKBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Gen1HeapSizeKBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Gen2HeapSizeKBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("LargeObjectHeapSizeKBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("KBytesPromotedFromGen0", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("KBytesPromotedFromGen1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="COR_GC_STATS", pack=False, align=None),
    "COR_GC_STAT_TYPES": SimTypeInt(signed=False, label="COR_GC_STAT_TYPES"),
    "COR_GC_THREAD_STATS": SimStruct(OrderedDict((("PerThreadAllocation", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="COR_GC_THREAD_STATS", pack=False, align=None),
    "COR_GC_THREAD_STATS_TYPES": SimTypeInt(signed=False, label="COR_GC_THREAD_STATS_TYPES"),
    "COR_IL_MAP": SimStruct(OrderedDict((("oldOffset", SimTypeInt(signed=False, label="UInt32")), ("newOffset", SimTypeInt(signed=False, label="UInt32")), ("fAccurate", SimTypeInt(signed=True, label="Int32")),)), name="COR_IL_MAP", pack=False, align=None),
    "COR_PRF_CLAUSE_TYPE": SimTypeInt(signed=False, label="COR_PRF_CLAUSE_TYPE"),
    "COR_PRF_CODEGEN_FLAGS": SimTypeInt(signed=False, label="COR_PRF_CODEGEN_FLAGS"),
    "COR_PRF_CODE_INFO": SimStruct(OrderedDict((("startAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="COR_PRF_CODE_INFO", pack=False, align=None),
    "COR_PRF_EVENTPIPE_LEVEL": SimTypeInt(signed=False, label="COR_PRF_EVENTPIPE_LEVEL"),
    "COR_PRF_EVENTPIPE_PARAM_DESC": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="UInt32")), ("elementType", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="COR_PRF_EVENTPIPE_PARAM_DESC", pack=False, align=None),
    "COR_PRF_EVENTPIPE_PARAM_TYPE": SimTypeInt(signed=False, label="COR_PRF_EVENTPIPE_PARAM_TYPE"),
    "COR_PRF_EVENTPIPE_PROVIDER_CONFIG": SimStruct(OrderedDict((("providerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("keywords", SimTypeLongLong(signed=False, label="UInt64")), ("loggingLevel", SimTypeInt(signed=False, label="UInt32")), ("filterData", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="COR_PRF_EVENTPIPE_PROVIDER_CONFIG", pack=False, align=None),
    "COR_PRF_EVENT_DATA": SimStruct(OrderedDict((("ptr", SimTypeLongLong(signed=False, label="UInt64")), ("size", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypeInt(signed=False, label="UInt32")),)), name="COR_PRF_EVENT_DATA", pack=False, align=None),
    "COR_PRF_EX_CLAUSE_INFO": SimStruct(OrderedDict((("clauseType", SimTypeInt(signed=False, label="COR_PRF_CLAUSE_TYPE")), ("programCounter", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("framePointer", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("shadowStackPointer", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="COR_PRF_EX_CLAUSE_INFO", pack=False, align=None),
    "COR_PRF_FILTER_DATA": SimStruct(OrderedDict((("Ptr", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="COR_PRF_FILTER_DATA", pack=False, align=None),
    "COR_PRF_FINALIZER_FLAGS": SimTypeInt(signed=False, label="COR_PRF_FINALIZER_FLAGS"),
    "COR_PRF_FUNCTION": SimStruct(OrderedDict((("functionId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reJitId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="COR_PRF_FUNCTION", pack=False, align=None),
    "COR_PRF_FUNCTION_ARGUMENT_INFO": SimStruct(OrderedDict((("numRanges", SimTypeInt(signed=False, label="UInt32")), ("totalArgumentSize", SimTypeInt(signed=False, label="UInt32")), ("ranges", SimTypePointer(SimTypeRef("COR_PRF_FUNCTION_ARGUMENT_RANGE", SimStruct), offset=0)),)), name="COR_PRF_FUNCTION_ARGUMENT_INFO", pack=False, align=None),
    "COR_PRF_FUNCTION_ARGUMENT_RANGE": SimStruct(OrderedDict((("startAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="COR_PRF_FUNCTION_ARGUMENT_RANGE", pack=False, align=None),
    "COR_PRF_GC_GENERATION": SimTypeInt(signed=False, label="COR_PRF_GC_GENERATION"),
    "COR_PRF_GC_GENERATION_RANGE": SimStruct(OrderedDict((("generation", SimTypeInt(signed=False, label="COR_PRF_GC_GENERATION")), ("rangeStart", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rangeLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rangeLengthReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="COR_PRF_GC_GENERATION_RANGE", pack=False, align=None),
    "COR_PRF_GC_REASON": SimTypeInt(signed=False, label="COR_PRF_GC_REASON"),
    "COR_PRF_GC_ROOT_FLAGS": SimTypeInt(signed=False, label="COR_PRF_GC_ROOT_FLAGS"),
    "COR_PRF_GC_ROOT_KIND": SimTypeInt(signed=False, label="COR_PRF_GC_ROOT_KIND"),
    "COR_PRF_HANDLE_TYPE": SimTypeInt(signed=False, label="COR_PRF_HANDLE_TYPE"),
    "COR_PRF_HIGH_MONITOR": SimTypeInt(signed=False, label="COR_PRF_HIGH_MONITOR"),
    "COR_PRF_JIT_CACHE": SimTypeInt(signed=False, label="COR_PRF_JIT_CACHE"),
    "COR_PRF_METHOD": SimStruct(OrderedDict((("moduleId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("methodId", SimTypeInt(signed=False, label="UInt32")),)), name="COR_PRF_METHOD", pack=False, align=None),
    "COR_PRF_MISC": SimTypeInt(signed=False, label="COR_PRF_MISC"),
    "COR_PRF_MODULE_FLAGS": SimTypeInt(signed=False, label="COR_PRF_MODULE_FLAGS"),
    "COR_PRF_MONITOR": SimTypeInt(signed=False, label="COR_PRF_MONITOR"),
    "COR_PRF_NONGC_HEAP_RANGE": SimStruct(OrderedDict((("rangeStart", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rangeLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rangeLengthReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="COR_PRF_NONGC_HEAP_RANGE", pack=False, align=None),
    "COR_PRF_REJIT_FLAGS": SimTypeInt(signed=False, label="COR_PRF_REJIT_FLAGS"),
    "COR_PRF_RUNTIME_TYPE": SimTypeInt(signed=False, label="COR_PRF_RUNTIME_TYPE"),
    "COR_PRF_SNAPSHOT_INFO": SimTypeInt(signed=False, label="COR_PRF_SNAPSHOT_INFO"),
    "COR_PRF_STATIC_TYPE": SimTypeInt(signed=False, label="COR_PRF_STATIC_TYPE"),
    "COR_PRF_SUSPEND_REASON": SimTypeInt(signed=False, label="COR_PRF_SUSPEND_REASON"),
    "COR_PRF_TRANSITION_REASON": SimTypeInt(signed=False, label="COR_PRF_TRANSITION_REASON"),
    "COSERVERINFO": SimStruct(OrderedDict((("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAuthInfo", SimTypePointer(SimTypeRef("COAUTHINFO", SimStruct), offset=0)), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="COSERVERINFO", pack=False, align=None),
    "COUNTED_REASON_CONTEXT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("ResourceFileName", SimTypeRef("UNICODE_STRING", SimStruct)), ("ResourceReasonId", SimTypeShort(signed=False, label="UInt16")), ("StringCount", SimTypeInt(signed=False, label="UInt32")), ("ReasonStrings", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "SimpleString": SimTypeRef("UNICODE_STRING", SimStruct)}, name="<anon>", label="None")),)), name="COUNTED_REASON_CONTEXT", pack=False, align=None),
    "COWAIT_FLAGS": SimTypeInt(signed=False, label="COWAIT_FLAGS"),
    "CO_ADDRESS": SimStruct(OrderedDict((("AddressSize", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CO_ADDRESS", pack=False, align=None),
    "CO_ADDRESS_FAMILY": SimStruct(OrderedDict((("AddressFamily", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="CO_ADDRESS_FAMILY", pack=False, align=None),
    "CO_ADDRESS_LIST": SimStruct(OrderedDict((("NumberOfAddressesAvailable", SimTypeInt(signed=False, label="UInt32")), ("NumberOfAddresses", SimTypeInt(signed=False, label="UInt32")), ("AddressList", SimTypeRef("CO_ADDRESS", SimStruct)),)), name="CO_ADDRESS_LIST", pack=False, align=None),
    "CO_AF_REGISTER_NOTIFY_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "CO_CALL_MANAGER_PARAMETERS": SimStruct(OrderedDict((("Transmit", SimTypeRef("FLOWSPEC", SimStruct)), ("Receive", SimTypeRef("FLOWSPEC", SimStruct)), ("CallMgrSpecific", SimTypeRef("CO_SPECIFIC_PARAMETERS", SimStruct)),)), name="CO_CALL_MANAGER_PARAMETERS", pack=False, align=None),
    "CO_CALL_PARAMETERS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CO_CREATE_VC_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CO_DELETE_VC_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CO_DEVICE_CATALOG_COOKIE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CO_MARSHALING_CONTEXT_ATTRIBUTES": SimTypeInt(signed=False, label="CO_MARSHALING_CONTEXT_ATTRIBUTES"),
    "CO_MEDIA_PARAMETERS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CO_MTA_USAGE_COOKIE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CO_PVC": SimStruct(OrderedDict((("NdisAfHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PvcParameters", SimTypeRef("CO_SPECIFIC_PARAMETERS", SimStruct)),)), name="CO_PVC", pack=False, align=None),
    "CO_SAP": SimStruct(OrderedDict((("SapType", SimTypeInt(signed=False, label="UInt32")), ("SapLength", SimTypeInt(signed=False, label="UInt32")), ("Sap", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CO_SAP", pack=False, align=None),
    "CO_SPECIFIC_PARAMETERS": SimStruct(OrderedDict((("ParamType", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Parameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CO_SPECIFIC_PARAMETERS", pack=False, align=None),
    "CPE_DRIVER_INFO": SimStruct(OrderedDict((("ExceptionCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MCA_EXCEPTION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "CmcLog"]), offset=0)), ("DpcCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CPE_DRIVER_INFO", pack=False, align=None),
    "CPEventBitShift": SimTypeInt(signed=False, label="CPEventBitShift"),
    "CPEvents": SimTypeInt(signed=False, label="CPEvents"),
    "CPINFO": SimStruct(OrderedDict((("MaxCharSize", SimTypeInt(signed=False, label="UInt32")), ("DefaultChar", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("LeadByte", SimTypeArray(SimTypeChar(label="Byte"), 12)),)), name="CPINFO", pack=False, align=None),
    "CPINFOEXA": SimStruct(OrderedDict((("MaxCharSize", SimTypeInt(signed=False, label="UInt32")), ("DefaultChar", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("LeadByte", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("UnicodeDefaultChar", SimTypeChar(label="Char")), ("CodePage", SimTypeInt(signed=False, label="UInt32")), ("CodePageName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="CPINFOEXA", pack=False, align=None),
    "CPINFOEXW": SimStruct(OrderedDict((("MaxCharSize", SimTypeInt(signed=False, label="UInt32")), ("DefaultChar", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("LeadByte", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("UnicodeDefaultChar", SimTypeChar(label="Char")), ("CodePage", SimTypeInt(signed=False, label="UInt32")), ("CodePageName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="CPINFOEXW", pack=False, align=None),
    "CPLINFO": SimStruct(OrderedDict((("idIcon", SimTypeInt(signed=True, label="Int32")), ("idName", SimTypeInt(signed=True, label="Int32")), ("idInfo", SimTypeInt(signed=True, label="Int32")), ("lData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CPLINFO", pack=False, align=None),
    "CPOINT": SimStruct(OrderedDict((("lP", SimTypeInt(signed=True, label="Int32")), ("dwLog", SimTypeInt(signed=False, label="UInt32")),)), name="CPOINT", pack=False, align=None),
    "CPRecordingStatus": SimTypeInt(signed=False, label="CPRecordingStatus"),
    "CPSUICBPARAM": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Reason", SimTypeShort(signed=False, label="UInt16")), ("hDlg", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pOptItem", SimTypePointer(SimTypeRef("OPTITEM", SimStruct), offset=0)), ("cOptItem", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("pCurItem", SimTypePointer(SimTypeRef("OPTITEM", SimStruct), offset=0)), ("Anonymous", SimUnion({"OldSel": SimTypeInt(signed=True, label="Int32"), "pOldSel": SimTypePointer(SimTypeChar(label="SByte"), offset=0)}, name="<anon>", label="None")), ("UserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Result", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CPSUICBPARAM", pack=False, align=None),
    "CPSUIDATABLOCK": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CPSUIDATABLOCK", pack=False, align=None),
    "CPS_URLS": SimStruct(OrderedDict((("pszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAlgorithm", SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0)), ("pDigest", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CPS_URLS", pack=False, align=None),
    "CPTABLEINFO": SimStruct(OrderedDict((("CodePage", SimTypeShort(signed=False, label="UInt16")), ("MaximumCharacterSize", SimTypeShort(signed=False, label="UInt16")), ("DefaultChar", SimTypeShort(signed=False, label="UInt16")), ("UniDefaultChar", SimTypeShort(signed=False, label="UInt16")), ("TransDefaultChar", SimTypeShort(signed=False, label="UInt16")), ("TransUniDefaultChar", SimTypeShort(signed=False, label="UInt16")), ("DBCSCodePage", SimTypeShort(signed=False, label="UInt16")), ("LeadByte", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("MultiByteTable", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("WideCharTable", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DBCSRanges", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("DBCSOffsets", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="CPTABLEINFO", pack=False, align=None),
    "CPU_ARCHITECTURE": SimTypeInt(signed=False, label="CPU_ARCHITECTURE"),
    "CPU_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("NumCPUs", SimTypeInt(signed=False, label="UInt32")), ("CurrentProc", SimTypeInt(signed=False, label="UInt32")), ("ProcInfo", SimTypeArray(SimUnion({"Alpha": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ALPHA", SimStruct), "Amd64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_AMD64", SimStruct), "Ia64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_IA64", SimStruct), "X86": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_X86", SimStruct), "Arm": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM", SimStruct), "Arm64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM64", SimStruct)}, name="<anon>", label="None"), 2048)), ("Mhz", SimTypeInt(signed=False, label="UInt32")),)), name="CPU_INFO", pack=False, align=None),
    "CPU_INFORMATION": SimUnion({"X86CpuInfo": SimTypeRef("_X86CpuInfo_e__Struct", SimStruct), "OtherCpuInfo": SimTypeRef("_OtherCpuInfo_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "CPU_INFO_v1": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("NumCPUs", SimTypeInt(signed=False, label="UInt32")), ("CurrentProc", SimTypeInt(signed=False, label="UInt32")), ("ProcInfo", SimTypeArray(SimUnion({"Alpha": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ALPHA", SimStruct), "Amd64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_AMD64", SimStruct), "Ia64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_IA64", SimStruct), "X86": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_X86", SimStruct), "Arm": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM", SimStruct), "Arm64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM64", SimStruct)}, name="<anon>", label="None"), 32)), ("Mhz", SimTypeInt(signed=False, label="UInt32")),)), name="CPU_INFO_v1", pack=False, align=None),
    "CPU_INFO_v2": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("NumCPUs", SimTypeInt(signed=False, label="UInt32")), ("CurrentProc", SimTypeInt(signed=False, label="UInt32")), ("ProcInfo", SimTypeArray(SimUnion({"Alpha": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ALPHA", SimStruct), "Amd64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_AMD64", SimStruct), "Ia64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_IA64", SimStruct), "X86": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_X86", SimStruct), "Arm": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM", SimStruct), "Arm64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM64", SimStruct)}, name="<anon>", label="None"), 1280)), ("Mhz", SimTypeInt(signed=False, label="UInt32")),)), name="CPU_INFO_v2", pack=False, align=None),
    "CPU_SET_INFORMATION_TYPE": SimTypeInt(signed=False, label="CPU_SET_INFORMATION_TYPE"),
    "CPVIEW": SimTypeInt(signed=False, label="CPVIEW"),
    "CProperty": SimStruct(OrderedDict((("dwPropID", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"lpszwData": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lpvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "dwValue": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("fPersist", SimTypeInt(signed=True, label="Int32")),)), name="CProperty", pack=False, align=None),
    "CQFORM": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CQFORM", pack=False, align=None),
    "CQPAGE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pPageProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CQPAGE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPage", "hwnd", "uMsg", "wParam", "lParam"]), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("idPageName", SimTypeInt(signed=True, label="Int32")), ("idPageTemplate", SimTypeInt(signed=True, label="Int32")), ("pDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CQPAGE", pack=False, align=None),
    "CRASHDUMP_FUNCTIONS_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("PowerOn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0)),)), name="CRASHDUMP_FUNCTIONS_INTERFACE", pack=False, align=None),
    "CREATEFILE2_EXTENDED_PARAMETERS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwFileFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSecurityQosFlags", SimTypeInt(signed=False, label="UInt32")), ("lpSecurityAttributes", SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0)), ("hTemplateFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CREATEFILE2_EXTENDED_PARAMETERS", pack=False, align=None),
    "CREATELINKSTATES": SimTypeInt(signed=False, label="CREATELINKSTATES"),
    "CREATESTRUCTA": SimStruct(OrderedDict((("lpCreateParams", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cy", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("x", SimTypeInt(signed=True, label="Int32")), ("style", SimTypeInt(signed=True, label="Int32")), ("lpszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszClass", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwExStyle", SimTypeInt(signed=False, label="UInt32")),)), name="CREATESTRUCTA", pack=False, align=None),
    "CREATESTRUCTW": SimStruct(OrderedDict((("lpCreateParams", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cy", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("x", SimTypeInt(signed=True, label="Int32")), ("style", SimTypeInt(signed=True, label="Int32")), ("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwExStyle", SimTypeInt(signed=False, label="UInt32")),)), name="CREATESTRUCTW", pack=False, align=None),
    "CREATESUBSCRIPTIONFLAGS": SimTypeInt(signed=False, label="CREATESUBSCRIPTIONFLAGS"),
    "CREATE_ASM_NAME_OBJ_FLAGS": SimTypeInt(signed=False, label="CREATE_ASM_NAME_OBJ_FLAGS"),
    "CREATE_CLUSTER_CONFIG": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszClusterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cNodes", SimTypeInt(signed=False, label="UInt32")), ("ppszNodeNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("cIpEntries", SimTypeInt(signed=False, label="UInt32")), ("pIpEntries", SimTypePointer(SimTypeRef("CLUSTER_IP_ENTRY", SimStruct), offset=0)), ("fEmptyCluster", SimTypeChar(label="Byte")), ("managementPointType", SimTypeInt(signed=False, label="CLUSTER_MGMT_POINT_TYPE")), ("managementPointResType", SimTypeInt(signed=False, label="CLUSTER_MGMT_POINT_RESTYPE")),)), name="CREATE_CLUSTER_CONFIG", pack=False, align=None),
    "CREATE_CLUSTER_NAME_ACCOUNT": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszClusterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("managementPointType", SimTypeInt(signed=False, label="CLUSTER_MGMT_POINT_TYPE")), ("managementPointResType", SimTypeInt(signed=False, label="CLUSTER_MGMT_POINT_RESTYPE")), ("bUpgradeVCOs", SimTypeChar(label="Byte")),)), name="CREATE_CLUSTER_NAME_ACCOUNT", pack=False, align=None),
    "CREATE_DISK": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="PARTITION_STYLE")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("CREATE_DISK_MBR", SimStruct), "Gpt": SimTypeRef("CREATE_DISK_GPT", SimStruct)}, name="<anon>", label="None")),)), name="CREATE_DISK", pack=False, align=None),
    "CREATE_DISK_GPT": SimStruct(OrderedDict((("DiskId", SimTypeBottom(label="Guid")), ("MaxPartitionCount", SimTypeInt(signed=False, label="UInt32")),)), name="CREATE_DISK_GPT", pack=False, align=None),
    "CREATE_DISK_MBR": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")),)), name="CREATE_DISK_MBR", pack=False, align=None),
    "CREATE_EVENT": SimTypeInt(signed=False, label="CREATE_EVENT"),
    "CREATE_FILE_TYPE": SimTypeInt(signed=False, label="CREATE_FILE_TYPE"),
    "CREATE_FONT_PACKAGE_SUBSET_ENCODING": SimTypeInt(signed=False, label="CREATE_FONT_PACKAGE_SUBSET_ENCODING"),
    "CREATE_FONT_PACKAGE_SUBSET_PLATFORM": SimTypeInt(signed=False, label="CREATE_FONT_PACKAGE_SUBSET_PLATFORM"),
    "CREATE_PARTITION_PARAMETERS": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("Anonymous", SimUnion({"MbrPartInfo": SimTypeRef("_MbrPartInfo_e__Struct", SimStruct), "GptPartInfo": SimTypeRef("_GptPartInfo_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CREATE_PARTITION_PARAMETERS", pack=False, align=None),
    "CREATE_POLYGON_RGN_MODE": SimTypeInt(signed=False, label="CREATE_POLYGON_RGN_MODE"),
    "CREATE_PROCESS_DEBUG_INFO": SimStruct(OrderedDict((("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpBaseOfImage", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDebugInfoFileOffset", SimTypeInt(signed=False, label="UInt32")), ("nDebugInfoSize", SimTypeInt(signed=False, label="UInt32")), ("lpThreadLocalBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpStartAddress", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0)), ("lpImageName", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("fUnicode", SimTypeShort(signed=False, label="UInt16")),)), name="CREATE_PROCESS_DEBUG_INFO", pack=False, align=None),
    "CREATE_PROCESS_LOGON_FLAGS": SimTypeInt(signed=False, label="CREATE_PROCESS_LOGON_FLAGS"),
    "CREATE_REDIRECTION_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("FileId", SimTypeRef("FILE_ID_128", SimStruct)), ("VolumeGuid", SimTypeBottom(label="Guid")),)), name="CREATE_REDIRECTION_ECP_CONTEXT", pack=False, align=None),
    "CREATE_RESTRICTED_TOKEN_FLAGS": SimTypeInt(signed=False, label="CREATE_RESTRICTED_TOKEN_FLAGS"),
    "CREATE_TAPE_PARTITION_METHOD": SimTypeInt(signed=False, label="CREATE_TAPE_PARTITION_METHOD"),
    "CREATE_THREAD_DEBUG_INFO": SimStruct(OrderedDict((("hThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpThreadLocalBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpStartAddress", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0)),)), name="CREATE_THREAD_DEBUG_INFO", pack=False, align=None),
    "CREATE_TOOLHELP_SNAPSHOT_FLAGS": SimTypeInt(signed=False, label="CREATE_TOOLHELP_SNAPSHOT_FLAGS"),
    "CREATE_USER_PROCESS_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("AccessToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CREATE_USER_PROCESS_ECP_CONTEXT", pack=False, align=None),
    "CREATE_USN_JOURNAL_DATA": SimStruct(OrderedDict((("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationDelta", SimTypeLongLong(signed=False, label="UInt64")),)), name="CREATE_USN_JOURNAL_DATA", pack=False, align=None),
    "CREATE_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="CREATE_VIRTUAL_DISK_FLAG"),
    "CREATE_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="CREATE_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct), "Version2": SimTypeRef("_Version2_e__Struct", SimStruct), "Version3": SimTypeRef("_Version3_e__Struct", SimStruct), "Version4": SimTypeRef("_Version4_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="CREATE_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "CREATE_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="CREATE_VIRTUAL_DISK_VERSION"),
    "CREATIVEADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="CREATIVEADPCMWAVEFORMAT", pack=False, align=None),
    "CREATIVEFASTSPEECH10WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="CREATIVEFASTSPEECH10WAVEFORMAT", pack=False, align=None),
    "CREATIVEFASTSPEECH8WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="CREATIVEFASTSPEECH8WAVEFORMAT", pack=False, align=None),
    "CREDENTIALA": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CRED_FLAGS")), ("Type", SimTypeInt(signed=False, label="CRED_TYPE")), ("TargetName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("LastWritten", SimTypeRef("FILETIME", SimStruct)), ("CredentialBlobSize", SimTypeInt(signed=False, label="UInt32")), ("CredentialBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Persist", SimTypeInt(signed=False, label="CRED_PERSIST")), ("AttributeCount", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypePointer(SimTypeRef("CREDENTIAL_ATTRIBUTEA", SimStruct), offset=0)), ("TargetAlias", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CREDENTIALA", pack=False, align=None),
    "CREDENTIALW": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CRED_FLAGS")), ("Type", SimTypeInt(signed=False, label="CRED_TYPE")), ("TargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LastWritten", SimTypeRef("FILETIME", SimStruct)), ("CredentialBlobSize", SimTypeInt(signed=False, label="UInt32")), ("CredentialBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Persist", SimTypeInt(signed=False, label="CRED_PERSIST")), ("AttributeCount", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypePointer(SimTypeRef("CREDENTIAL_ATTRIBUTEW", SimStruct), offset=0)), ("TargetAlias", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CREDENTIALW", pack=False, align=None),
    "CREDENTIAL_ATTRIBUTEA": SimStruct(OrderedDict((("Keyword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ValueSize", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CREDENTIAL_ATTRIBUTEA", pack=False, align=None),
    "CREDENTIAL_ATTRIBUTEW": SimStruct(OrderedDict((("Keyword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ValueSize", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CREDENTIAL_ATTRIBUTEW", pack=False, align=None),
    "CREDENTIAL_PROVIDER_ACCOUNT_OPTIONS": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_ACCOUNT_OPTIONS"),
    "CREDENTIAL_PROVIDER_CREDENTIAL_FIELD_OPTIONS": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_CREDENTIAL_FIELD_OPTIONS"),
    "CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION": SimStruct(OrderedDict((("ulAuthenticationPackage", SimTypeInt(signed=False, label="UInt32")), ("clsidCredentialProvider", SimTypeBottom(label="Guid")), ("cbSerialization", SimTypeInt(signed=False, label="UInt32")), ("rgbSerialization", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION", pack=False, align=None),
    "CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR": SimStruct(OrderedDict((("dwFieldID", SimTypeInt(signed=False, label="UInt32")), ("cpft", SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_FIELD_TYPE")), ("pszLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("guidFieldType", SimTypeBottom(label="Guid")),)), name="CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR", pack=False, align=None),
    "CREDENTIAL_PROVIDER_FIELD_INTERACTIVE_STATE": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_FIELD_INTERACTIVE_STATE"),
    "CREDENTIAL_PROVIDER_FIELD_STATE": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_FIELD_STATE"),
    "CREDENTIAL_PROVIDER_FIELD_TYPE": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_FIELD_TYPE"),
    "CREDENTIAL_PROVIDER_GET_SERIALIZATION_RESPONSE": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_GET_SERIALIZATION_RESPONSE"),
    "CREDENTIAL_PROVIDER_STATUS_ICON": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_STATUS_ICON"),
    "CREDENTIAL_PROVIDER_USAGE_SCENARIO": SimTypeInt(signed=False, label="CREDENTIAL_PROVIDER_USAGE_SCENARIO"),
    "CREDENTIAL_TARGET_INFORMATIONA": SimStruct(OrderedDict((("TargetName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NetbiosServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NetbiosDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsTreeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PackageName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CredTypeCount", SimTypeInt(signed=False, label="UInt32")), ("CredTypes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CREDENTIAL_TARGET_INFORMATIONA", pack=False, align=None),
    "CREDENTIAL_TARGET_INFORMATIONW": SimStruct(OrderedDict((("TargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NetbiosServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NetbiosDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsTreeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CredTypeCount", SimTypeInt(signed=False, label="UInt32")), ("CredTypes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CREDENTIAL_TARGET_INFORMATIONW", pack=False, align=None),
    "CREDSPP_SUBMIT_TYPE": SimTypeInt(signed=False, label="CREDSPP_SUBMIT_TYPE"),
    "CREDSSP_CRED": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="CREDSPP_SUBMIT_TYPE")), ("pSchannelCred", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pSpnegoCred", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CREDSSP_CRED", pack=False, align=None),
    "CREDSSP_CRED_EX": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="CREDSPP_SUBMIT_TYPE")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Cred", SimTypeRef("CREDSSP_CRED", SimStruct)),)), name="CREDSSP_CRED_EX", pack=False, align=None),
    "CREDUIWIN_FLAGS": SimTypeInt(signed=False, label="CREDUIWIN_FLAGS"),
    "CREDUI_FLAGS": SimTypeInt(signed=False, label="CREDUI_FLAGS"),
    "CREDUI_INFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszMessageText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszCaptionText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hbmBanner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CREDUI_INFOA", pack=False, align=None),
    "CREDUI_INFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszMessageText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCaptionText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hbmBanner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CREDUI_INFOW", pack=False, align=None),
    "CRED_ENUMERATE_FLAGS": SimTypeInt(signed=False, label="CRED_ENUMERATE_FLAGS"),
    "CRED_FETCH": SimTypeInt(signed=False, label="CRED_FETCH"),
    "CRED_FLAGS": SimTypeInt(signed=False, label="CRED_FLAGS"),
    "CRED_MARSHAL_TYPE": SimTypeInt(signed=False, label="CRED_MARSHAL_TYPE"),
    "CRED_PACK_FLAGS": SimTypeInt(signed=False, label="CRED_PACK_FLAGS"),
    "CRED_PERSIST": SimTypeInt(signed=False, label="CRED_PERSIST"),
    "CRED_PROTECTION_TYPE": SimTypeInt(signed=False, label="CRED_PROTECTION_TYPE"),
    "CRED_TYPE": SimTypeInt(signed=False, label="CRED_TYPE"),
    "CRID_LOCATION": SimTypeInt(signed=False, label="CRID_LOCATION"),
    "CRITICAL_SECTION": SimStruct(OrderedDict((("DebugInfo", SimTypePointer(SimTypeRef("CRITICAL_SECTION_DEBUG", SimStruct), offset=0)), ("LockCount", SimTypeInt(signed=True, label="Int32")), ("RecursionCount", SimTypeInt(signed=True, label="Int32")), ("OwningThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("LockSemaphore", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SpinCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CRITICAL_SECTION", pack=False, align=None),
    "CRITICAL_SECTION_DEBUG": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("CreatorBackTraceIndex", SimTypeShort(signed=False, label="UInt16")), ("CriticalSection", SimTypePointer(SimTypeRef("CRITICAL_SECTION", SimStruct), offset=0)), ("ProcessLocksList", SimTypeRef("LIST_ENTRY", SimStruct)), ("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("ContentionCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CreatorBackTraceIndexHigh", SimTypeShort(signed=False, label="UInt16")), ("Identifier", SimTypeShort(signed=False, label="UInt16")),)), name="CRITICAL_SECTION_DEBUG", pack=False, align=None),
    "CRLRevocationReason": SimTypeInt(signed=False, label="CRLRevocationReason"),
    "CRL_CONTEXT": SimStruct(OrderedDict((("dwCertEncodingType", SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE")), ("pbCrlEncoded", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCrlEncoded", SimTypeInt(signed=False, label="UInt32")), ("pCrlInfo", SimTypePointer(SimTypeRef("CRL_INFO", SimStruct), offset=0)), ("hCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRL_CONTEXT", pack=False, align=None),
    "CRL_DIST_POINT": SimStruct(OrderedDict((("DistPointName", SimTypeRef("CRL_DIST_POINT_NAME", SimStruct)), ("ReasonFlags", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("CRLIssuer", SimTypeRef("CERT_ALT_NAME_INFO", SimStruct)),)), name="CRL_DIST_POINT", pack=False, align=None),
    "CRL_DIST_POINTS_INFO": SimStruct(OrderedDict((("cDistPoint", SimTypeInt(signed=False, label="UInt32")), ("rgDistPoint", SimTypePointer(SimTypeRef("CRL_DIST_POINT", SimStruct), offset=0)),)), name="CRL_DIST_POINTS_INFO", pack=False, align=None),
    "CRL_DIST_POINT_NAME": SimStruct(OrderedDict((("dwDistPointNameChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"FullName": SimTypeRef("CERT_ALT_NAME_INFO", SimStruct)}, name="<anon>", label="None")),)), name="CRL_DIST_POINT_NAME", pack=False, align=None),
    "CRL_ENTRY": SimStruct(OrderedDict((("SerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("RevocationDate", SimTypeRef("FILETIME", SimStruct)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CRL_ENTRY", pack=False, align=None),
    "CRL_FIND_ISSUED_FOR_PARA": SimStruct(OrderedDict((("pSubjectCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("pIssuerCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)),)), name="CRL_FIND_ISSUED_FOR_PARA", pack=False, align=None),
    "CRL_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("SignatureAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Issuer", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("ThisUpdate", SimTypeRef("FILETIME", SimStruct)), ("NextUpdate", SimTypeRef("FILETIME", SimStruct)), ("cCRLEntry", SimTypeInt(signed=False, label="UInt32")), ("rgCRLEntry", SimTypePointer(SimTypeRef("CRL_ENTRY", SimStruct), offset=0)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CRL_INFO", pack=False, align=None),
    "CRL_ISSUING_DIST_POINT": SimStruct(OrderedDict((("DistPointName", SimTypeRef("CRL_DIST_POINT_NAME", SimStruct)), ("fOnlyContainsUserCerts", SimTypeInt(signed=True, label="Int32")), ("fOnlyContainsCACerts", SimTypeInt(signed=True, label="Int32")), ("OnlySomeReasonFlags", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("fIndirectCRL", SimTypeInt(signed=True, label="Int32")),)), name="CRL_ISSUING_DIST_POINT", pack=False, align=None),
    "CRL_REVOCATION_INFO": SimStruct(OrderedDict((("pCrlEntry", SimTypePointer(SimTypeRef("CRL_ENTRY", SimStruct), offset=0)), ("pCrlContext", SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0)), ("pCrlIssuerChain", SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0)),)), name="CRL_REVOCATION_INFO", pack=False, align=None),
    "CRMFLAGS": SimTypeInt(signed=False, label="CRMFLAGS"),
    "CRMREGFLAGS": SimTypeInt(signed=False, label="CRMREGFLAGS"),
    "CROSSBAR_DEFAULT_FLAGS": SimTypeInt(signed=False, label="CROSSBAR_DEFAULT_FLAGS"),
    "CROSS_CERT_DIST_POINTS_INFO": SimStruct(OrderedDict((("dwSyncDeltaTime", SimTypeInt(signed=False, label="UInt32")), ("cDistPoint", SimTypeInt(signed=False, label="UInt32")), ("rgDistPoint", SimTypePointer(SimTypeRef("CERT_ALT_NAME_INFO", SimStruct), offset=0)),)), name="CROSS_CERT_DIST_POINTS_INFO", pack=False, align=None),
    "CROSS_SLIDE_FLAGS": SimTypeInt(signed=False, label="CROSS_SLIDE_FLAGS"),
    "CROSS_SLIDE_PARAMETER": SimStruct(OrderedDict((("threshold", SimTypeInt(signed=False, label="CROSS_SLIDE_THRESHOLD")), ("distance", SimTypeFloat(size=32)),)), name="CROSS_SLIDE_PARAMETER", pack=False, align=None),
    "CROSS_SLIDE_THRESHOLD": SimTypeInt(signed=False, label="CROSS_SLIDE_THRESHOLD"),
    "CRYPTCATATTRIBUTE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pwszReferenceTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAttrTypeAndAction", SimTypeInt(signed=False, label="UInt32")), ("cbValue", SimTypeInt(signed=False, label="UInt32")), ("pbValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPTCATATTRIBUTE", pack=False, align=None),
    "CRYPTCATCDF": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwCurFilePos", SimTypeInt(signed=False, label="UInt32")), ("dwLastMemberOffset", SimTypeInt(signed=False, label="UInt32")), ("fEOF", SimTypeInt(signed=True, label="Int32")), ("pwszResultDir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hCATStore", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CRYPTCATCDF", pack=False, align=None),
    "CRYPTCATMEMBER": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pwszReferenceTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("gSubjectType", SimTypeBottom(label="Guid")), ("fdwMemberFlags", SimTypeInt(signed=False, label="UInt32")), ("pIndirectData", SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)), ("dwCertVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("hReserved", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("sEncodedIndirectData", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("sEncodedMemberInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPTCATMEMBER", pack=False, align=None),
    "CRYPTCATSTORE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwPublicVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszP7File", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwEncodingType", SimTypeInt(signed=False, label="UInt32")), ("fdwStoreFlags", SimTypeInt(signed=False, label="CRYPTCAT_OPEN_FLAGS")), ("hReserved", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAttrs", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCryptMsg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hSorted", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CRYPTCATSTORE", pack=False, align=None),
    "CRYPTCAT_OPEN_FLAGS": SimTypeInt(signed=False, label="CRYPTCAT_OPEN_FLAGS"),
    "CRYPTCAT_VERSION": SimTypeInt(signed=False, label="CRYPTCAT_VERSION"),
    "CRYPTNET_URL_CACHE_FLUSH_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwExemptSeconds", SimTypeInt(signed=False, label="UInt32")), ("ExpireTime", SimTypeRef("FILETIME", SimStruct)),)), name="CRYPTNET_URL_CACHE_FLUSH_INFO", pack=False, align=None),
    "CRYPTNET_URL_CACHE_PRE_FETCH_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwObjectType", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ThisUpdateTime", SimTypeRef("FILETIME", SimStruct)), ("NextUpdateTime", SimTypeRef("FILETIME", SimStruct)), ("PublishTime", SimTypeRef("FILETIME", SimStruct)),)), name="CRYPTNET_URL_CACHE_PRE_FETCH_INFO", pack=False, align=None),
    "CRYPTNET_URL_CACHE_RESPONSE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wResponseType", SimTypeShort(signed=False, label="UInt16")), ("wResponseFlags", SimTypeShort(signed=False, label="UInt16")), ("LastModifiedTime", SimTypeRef("FILETIME", SimStruct)), ("dwMaxAge", SimTypeInt(signed=False, label="UInt32")), ("pwszETag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwProxyId", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPTNET_URL_CACHE_RESPONSE_INFO", pack=False, align=None),
    "CRYPTO_SETTINGS": SimStruct(OrderedDict((("eAlgorithmUsage", SimTypeInt(signed=False, label="eTlsAlgorithmUsage")), ("strCngAlgId", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("cChainingModes", SimTypeInt(signed=False, label="UInt32")), ("rgstrChainingModes", SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)), ("dwMinBitLength", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBitLength", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPTO_SETTINGS", pack=False, align=None),
    "CRYPTPROTECT_PROMPTSTRUCT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwPromptFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndApp", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szPrompt", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPTPROTECT_PROMPTSTRUCT", pack=False, align=None),
    "CRYPTUI_CERT_MGR_STRUCT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInitUsageOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPTUI_CERT_MGR_STRUCT", pack=False, align=None),
    "CRYPTUI_INITDIALOG_STRUCT": SimStruct(OrderedDict((("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)),)), name="CRYPTUI_INITDIALOG_STRUCT", pack=False, align=None),
    "CRYPTUI_VIEWCERTIFICATE_FLAGS": SimTypeInt(signed=False, label="CRYPTUI_VIEWCERTIFICATE_FLAGS"),
    "CRYPTUI_VIEWCERTIFICATE_STRUCTA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CRYPTUI_VIEWCERTIFICATE_FLAGS")), ("szTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("rgszPurposes", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)), ("cPurposes", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pCryptProviderData": SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), "hWVTStateData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("fpCryptProviderDataTrustedUsage", SimTypeInt(signed=True, label="Int32")), ("idxSigner", SimTypeInt(signed=False, label="UInt32")), ("idxCert", SimTypeInt(signed=False, label="UInt32")), ("fCounterSigner", SimTypeInt(signed=True, label="Int32")), ("idxCounterSigner", SimTypeInt(signed=False, label="UInt32")), ("cStores", SimTypeInt(signed=False, label="UInt32")), ("rghStores", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cPropSheetPages", SimTypeInt(signed=False, label="UInt32")), ("rgPropSheetPages", SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)), ("nStartPage", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPTUI_VIEWCERTIFICATE_STRUCTA", pack=False, align=None),
    "CRYPTUI_VIEWCERTIFICATE_STRUCTW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CRYPTUI_VIEWCERTIFICATE_FLAGS")), ("szTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("rgszPurposes", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)), ("cPurposes", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pCryptProviderData": SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), "hWVTStateData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("fpCryptProviderDataTrustedUsage", SimTypeInt(signed=True, label="Int32")), ("idxSigner", SimTypeInt(signed=False, label="UInt32")), ("idxCert", SimTypeInt(signed=False, label="UInt32")), ("fCounterSigner", SimTypeInt(signed=True, label="Int32")), ("idxCounterSigner", SimTypeInt(signed=False, label="UInt32")), ("cStores", SimTypeInt(signed=False, label="UInt32")), ("rghStores", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cPropSheetPages", SimTypeInt(signed=False, label="UInt32")), ("rgPropSheetPages", SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0)), ("nStartPage", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPTUI_VIEWCERTIFICATE_STRUCTW", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_ADDITIONAL_CERT_CHOICE": SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_ADDITIONAL_CERT_CHOICE"),
    "CRYPTUI_WIZ_DIGITAL_SIGN": SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN"),
    "CRYPTUI_WIZ_DIGITAL_SIGN_BLOB_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pGuidSubject", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cbBlob", SimTypeInt(signed=False, label="UInt32")), ("pbBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_BLOB_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_CERT_PVK_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwszSigningCertFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPvkChoice", SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN_PVK_OPTION")), ("Anonymous", SimUnion({"pPvkFileInfo": SimTypePointer(SimTypeRef("CRYPTUI_WIZ_DIGITAL_SIGN_PVK_FILE_INFO", SimStruct), offset=0), "pPvkProvInfo": SimTypePointer(SimTypeRef("CRYPT_KEY_PROV_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_CERT_PVK_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_CONTEXT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cbBlob", SimTypeInt(signed=False, label="UInt32")), ("pbBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_CONTEXT", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_EXTENDED_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwAttrFlags", SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN_SIG_TYPE")), ("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszMoreInfoLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHashAlg", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszSigningCertDisplayString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hAdditionalCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("psAuthenticated", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTES", SimStruct), offset=0)), ("psUnauthenticated", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTES", SimStruct), offset=0)),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_EXTENDED_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSubjectChoice", SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN_SUBJECT")), ("Anonymous1", SimUnion({"pwszFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pSignBlobInfo": SimTypePointer(SimTypeRef("CRYPTUI_WIZ_DIGITAL_SIGN_BLOB_INFO", SimStruct), offset=0)}, name="<anon>", label="None")), ("dwSigningCertChoice", SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN")), ("Anonymous2", SimUnion({"pSigningCertContext": SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), "pSigningCertStore": SimTypePointer(SimTypeRef("CRYPTUI_WIZ_DIGITAL_SIGN_STORE_INFO", SimStruct), offset=0), "pSigningCertPvkInfo": SimTypePointer(SimTypeRef("CRYPTUI_WIZ_DIGITAL_SIGN_CERT_PVK_INFO", SimStruct), offset=0)}, name="<anon>", label="None")), ("pwszTimestampURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAdditionalCertChoice", SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_ADDITIONAL_CERT_CHOICE")), ("pSignExtInfo", SimTypePointer(SimTypeRef("CRYPTUI_WIZ_DIGITAL_SIGN_EXTENDED_INFO", SimStruct), offset=0)),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_PVK_FILE_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwszPvkFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszProvName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwProvType", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_PVK_FILE_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_PVK_OPTION": SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN_PVK_OPTION"),
    "CRYPTUI_WIZ_DIGITAL_SIGN_SIG_TYPE": SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN_SIG_TYPE"),
    "CRYPTUI_WIZ_DIGITAL_SIGN_STORE_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cCertStore", SimTypeInt(signed=False, label="UInt32")), ("rghCertStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("pFilterCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "pfInitialSelectedCert", "pvCallbackData"]), offset=0)), ("pvCallbackData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPTUI_WIZ_DIGITAL_SIGN_STORE_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_DIGITAL_SIGN_SUBJECT": SimTypeInt(signed=False, label="CRYPTUI_WIZ_DIGITAL_SIGN_SUBJECT"),
    "CRYPTUI_WIZ_EXPORT_CERTCONTEXT_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwExportFormat", SimTypeInt(signed=False, label="CRYPTUI_WIZ_EXPORT_FORMAT")), ("fExportChain", SimTypeInt(signed=True, label="Int32")), ("fExportPrivateKeys", SimTypeInt(signed=True, label="Int32")), ("pwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fStrongEncryption", SimTypeInt(signed=True, label="Int32")),)), name="CRYPTUI_WIZ_EXPORT_CERTCONTEXT_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_EXPORT_FORMAT": SimTypeInt(signed=False, label="CRYPTUI_WIZ_EXPORT_FORMAT"),
    "CRYPTUI_WIZ_EXPORT_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwszExportFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSubjectChoice", SimTypeInt(signed=False, label="CRYPTUI_WIZ_EXPORT_SUBJECT")), ("Anonymous", SimUnion({"pCertContext": SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), "pCTLContext": SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), "pCRLContext": SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), "hCertStore": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("cStores", SimTypeInt(signed=False, label="UInt32")), ("rghStores", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="CRYPTUI_WIZ_EXPORT_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_EXPORT_SUBJECT": SimTypeInt(signed=False, label="CRYPTUI_WIZ_EXPORT_SUBJECT"),
    "CRYPTUI_WIZ_FLAGS": SimTypeInt(signed=False, label="CRYPTUI_WIZ_FLAGS"),
    "CRYPTUI_WIZ_IMPORT_SRC_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSubjectChoice", SimTypeInt(signed=False, label="CRYPTUI_WIZ_IMPORT_SUBJECT_OPTION")), ("Anonymous", SimUnion({"pwszFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pCertContext": SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), "pCTLContext": SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), "pCRLContext": SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), "hCertStore": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("dwFlags", SimTypeInt(signed=False, label="CRYPT_KEY_FLAGS")), ("pwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPTUI_WIZ_IMPORT_SRC_INFO", pack=False, align=None),
    "CRYPTUI_WIZ_IMPORT_SUBJECT_OPTION": SimTypeInt(signed=False, label="CRYPTUI_WIZ_IMPORT_SUBJECT_OPTION"),
    "CRYPT_3DES_KEY_STATE": SimStruct(OrderedDict((("Key", SimTypeArray(SimTypeChar(label="Byte"), 24)), ("IV", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Feedback", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="CRYPT_3DES_KEY_STATE", pack=False, align=None),
    "CRYPT_ACQUIRE_FLAGS": SimTypeInt(signed=False, label="CRYPT_ACQUIRE_FLAGS"),
    "CRYPT_AES_128_KEY_STATE": SimStruct(OrderedDict((("Key", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("IV", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("EncryptionState", SimTypeArray(SimTypeChar(label="Byte"), 176)), ("DecryptionState", SimTypeArray(SimTypeChar(label="Byte"), 176)), ("Feedback", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="CRYPT_AES_128_KEY_STATE", pack=False, align=None),
    "CRYPT_AES_256_KEY_STATE": SimStruct(OrderedDict((("Key", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("IV", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("EncryptionState", SimTypeArray(SimTypeChar(label="Byte"), 240)), ("DecryptionState", SimTypeArray(SimTypeChar(label="Byte"), 240)), ("Feedback", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="CRYPT_AES_256_KEY_STATE", pack=False, align=None),
    "CRYPT_ALGORITHM_IDENTIFIER": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Parameters", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_ALGORITHM_IDENTIFIER", pack=False, align=None),
    "CRYPT_ASYNC_RETRIEVAL_COMPLETION": SimStruct(OrderedDict((("pfnCompletion", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvCompletion", "dwCompletionCode", "pszUrl", "pszObjectOid", "pvObject"]), offset=0)), ("pvCompletion", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_ASYNC_RETRIEVAL_COMPLETION", pack=False, align=None),
    "CRYPT_ATTRIBUTE": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cValue", SimTypeInt(signed=False, label="UInt32")), ("rgValue", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CRYPT_ATTRIBUTE", pack=False, align=None),
    "CRYPT_ATTRIBUTES": SimStruct(OrderedDict((("cAttr", SimTypeInt(signed=False, label="UInt32")), ("rgAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)),)), name="CRYPT_ATTRIBUTES", pack=False, align=None),
    "CRYPT_ATTRIBUTE_TYPE_VALUE": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_ATTRIBUTE_TYPE_VALUE", pack=False, align=None),
    "CRYPT_BIT_BLOB": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cUnusedBits", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_BIT_BLOB", pack=False, align=None),
    "CRYPT_BLOB_ARRAY": SimStruct(OrderedDict((("cBlob", SimTypeInt(signed=False, label="UInt32")), ("rgBlob", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CRYPT_BLOB_ARRAY", pack=False, align=None),
    "CRYPT_CONTENT_INFO": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Content", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_CONTENT_INFO", pack=False, align=None),
    "CRYPT_CONTENT_INFO_SEQUENCE_OF_ANY": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cValue", SimTypeInt(signed=False, label="UInt32")), ("rgValue", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CRYPT_CONTENT_INFO_SEQUENCE_OF_ANY", pack=False, align=None),
    "CRYPT_CONTEXTS": SimStruct(OrderedDict((("cContexts", SimTypeInt(signed=False, label="UInt32")), ("rgpszContexts", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CRYPT_CONTEXTS", pack=False, align=None),
    "CRYPT_CONTEXT_CONFIG": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="CRYPT_CONTEXT_CONFIG_FLAGS")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_CONTEXT_CONFIG", pack=False, align=None),
    "CRYPT_CONTEXT_CONFIG_FLAGS": SimTypeInt(signed=False, label="CRYPT_CONTEXT_CONFIG_FLAGS"),
    "CRYPT_CONTEXT_FUNCTIONS": SimStruct(OrderedDict((("cFunctions", SimTypeInt(signed=False, label="UInt32")), ("rgpszFunctions", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CRYPT_CONTEXT_FUNCTIONS", pack=False, align=None),
    "CRYPT_CONTEXT_FUNCTION_CONFIG": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_CONTEXT_FUNCTION_CONFIG", pack=False, align=None),
    "CRYPT_CONTEXT_FUNCTION_PROVIDERS": SimStruct(OrderedDict((("cProviders", SimTypeInt(signed=False, label="UInt32")), ("rgpszProviders", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CRYPT_CONTEXT_FUNCTION_PROVIDERS", pack=False, align=None),
    "CRYPT_CREDENTIALS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszCredentialsOid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvCredentials", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_CREDENTIALS", pack=False, align=None),
    "CRYPT_CSP_PROVIDER": SimStruct(OrderedDict((("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("pwszProviderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Signature", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)),)), name="CRYPT_CSP_PROVIDER", pack=False, align=None),
    "CRYPT_DECODE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnAlloc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)),)), name="CRYPT_DECODE_PARA", pack=False, align=None),
    "CRYPT_DECRYPT_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgAndCertEncodingType", SimTypeInt(signed=False, label="UInt32")), ("cCertStore", SimTypeInt(signed=False, label="UInt32")), ("rghCertStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="CRYPT_DECRYPT_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_DEFAULT_CONTEXT_FLAGS": SimTypeInt(signed=False, label="CRYPT_DEFAULT_CONTEXT_FLAGS"),
    "CRYPT_DEFAULT_CONTEXT_MULTI_OID_PARA": SimStruct(OrderedDict((("cOID", SimTypeInt(signed=False, label="UInt32")), ("rgpszOID", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="CRYPT_DEFAULT_CONTEXT_MULTI_OID_PARA", pack=False, align=None),
    "CRYPT_DEFAULT_CONTEXT_TYPE": SimTypeInt(signed=False, label="CRYPT_DEFAULT_CONTEXT_TYPE"),
    "CRYPT_DES_KEY_STATE": SimStruct(OrderedDict((("Key", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("IV", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Feedback", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="CRYPT_DES_KEY_STATE", pack=False, align=None),
    "CRYPT_ECC_CMS_SHARED_INFO": SimStruct(OrderedDict((("Algorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EntityUInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("rgbSuppPubInfo", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="CRYPT_ECC_CMS_SHARED_INFO", pack=False, align=None),
    "CRYPT_ECC_PRIVATE_KEY_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("PrivateKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("szCurveOid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PublicKey", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)),)), name="CRYPT_ECC_PRIVATE_KEY_INFO", pack=False, align=None),
    "CRYPT_ENCODE_OBJECT_FLAGS": SimTypeInt(signed=False, label="CRYPT_ENCODE_OBJECT_FLAGS"),
    "CRYPT_ENCODE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnAlloc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)),)), name="CRYPT_ENCODE_PARA", pack=False, align=None),
    "CRYPT_ENCRYPTED_PRIVATE_KEY_INFO": SimStruct(OrderedDict((("EncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("EncryptedPrivateKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_ENCRYPTED_PRIVATE_KEY_INFO", pack=False, align=None),
    "CRYPT_ENCRYPT_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgEncodingType", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ContentEncryptionAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvEncryptionAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwInnerContentType", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_ENCRYPT_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_ENROLLMENT_NAME_VALUE_PAIR": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPT_ENROLLMENT_NAME_VALUE_PAIR", pack=False, align=None),
    "CRYPT_FIND_FLAGS": SimTypeInt(signed=False, label="CRYPT_FIND_FLAGS"),
    "CRYPT_GET_TIME_VALID_OBJECT_EXTRA_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iDeltaCrlIndicator", SimTypeInt(signed=True, label="Int32")), ("pftCacheResync", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("pLastSyncTime", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("pMaxAgeTime", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("pChainPara", SimTypePointer(SimTypeRef("CERT_REVOCATION_CHAIN_PARA", SimStruct), offset=0)), ("pDeltaCrlIndicator", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CRYPT_GET_TIME_VALID_OBJECT_EXTRA_INFO", pack=False, align=None),
    "CRYPT_GET_URL_FLAGS": SimTypeInt(signed=False, label="CRYPT_GET_URL_FLAGS"),
    "CRYPT_HASH_INFO": SimStruct(OrderedDict((("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Hash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_HASH_INFO", pack=False, align=None),
    "CRYPT_HASH_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgEncodingType", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvHashAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_HASH_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_IMAGE_REF": SimStruct(OrderedDict((("pszImage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="CRYPT_IMAGE_REF_FLAGS")),)), name="CRYPT_IMAGE_REF", pack=False, align=None),
    "CRYPT_IMAGE_REF_FLAGS": SimTypeInt(signed=False, label="CRYPT_IMAGE_REF_FLAGS"),
    "CRYPT_IMAGE_REG": SimStruct(OrderedDict((("pszImage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cInterfaces", SimTypeInt(signed=False, label="UInt32")), ("rgpInterfaces", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTERFACE_REG", SimStruct), offset=0), offset=0)),)), name="CRYPT_IMAGE_REG", pack=False, align=None),
    "CRYPT_IMPORT_PUBLIC_KEY_FLAGS": SimTypeInt(signed=False, label="CRYPT_IMPORT_PUBLIC_KEY_FLAGS"),
    "CRYPT_INTEGER_BLOB": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_INTEGER_BLOB", pack=False, align=None),
    "CRYPT_INTERFACE_REG": SimStruct(OrderedDict((("dwInterface", SimTypeInt(signed=False, label="BCRYPT_INTERFACE")), ("dwFlags", SimTypeInt(signed=False, label="BCRYPT_TABLE")), ("cFunctions", SimTypeInt(signed=False, label="UInt32")), ("rgpszFunctions", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CRYPT_INTERFACE_REG", pack=False, align=None),
    "CRYPT_KEY_FLAGS": SimTypeInt(signed=False, label="CRYPT_KEY_FLAGS"),
    "CRYPT_KEY_PARAM_ID": SimTypeInt(signed=False, label="CRYPT_KEY_PARAM_ID"),
    "CRYPT_KEY_PROV_INFO": SimStruct(OrderedDict((("pwszContainerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszProvName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwProvType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CRYPT_KEY_FLAGS")), ("cProvParam", SimTypeInt(signed=False, label="UInt32")), ("rgProvParam", SimTypePointer(SimTypeRef("CRYPT_KEY_PROV_PARAM", SimStruct), offset=0)), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_KEY_PROV_INFO", pack=False, align=None),
    "CRYPT_KEY_PROV_PARAM": SimStruct(OrderedDict((("dwParam", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_KEY_PROV_PARAM", pack=False, align=None),
    "CRYPT_KEY_SIGN_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgAndCertEncodingType", SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE")), ("Anonymous", SimUnion({"hCryptProv": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hNCryptKey": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwKeySpec", SimTypeInt(signed=False, label="CERT_KEY_SPEC")), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvHashAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PubKeyAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)),)), name="CRYPT_KEY_SIGN_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_KEY_VERIFY_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgEncodingType", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="CRYPT_KEY_VERIFY_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_MASK_GEN_ALGORITHM": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)),)), name="CRYPT_MASK_GEN_ALGORITHM", pack=False, align=None),
    "CRYPT_MSG_TYPE": SimTypeInt(signed=False, label="CRYPT_MSG_TYPE"),
    "CRYPT_OBJECT_LOCATOR_PROVIDER_TABLE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnGet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPluginContext", "pIdentifier", "dwNameType", "pNameBlob", "ppbContent", "pcbContent", "ppwszPassword", "ppIdentifier"]), offset=0)), ("pfnRelease", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="CRYPT_OBJECT_LOCATOR_RELEASE_REASON"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwReason", "pPluginContext"]), offset=0)), ("pfnFreePassword", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPluginContext", "pwszPassword"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPluginContext", "pbData"]), offset=0)), ("pfnFreeIdentifier", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPluginContext", "pIdentifier"]), offset=0)),)), name="CRYPT_OBJECT_LOCATOR_PROVIDER_TABLE", pack=False, align=None),
    "CRYPT_OBJECT_LOCATOR_RELEASE_REASON": SimTypeInt(signed=False, label="CRYPT_OBJECT_LOCATOR_RELEASE_REASON"),
    "CRYPT_OBJID_TABLE": SimStruct(OrderedDict((("dwAlgId", SimTypeInt(signed=False, label="UInt32")), ("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_OBJID_TABLE", pack=False, align=None),
    "CRYPT_OID_FUNC_ENTRY": SimStruct(OrderedDict((("pszOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvFuncAddr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_OID_FUNC_ENTRY", pack=False, align=None),
    "CRYPT_OID_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwGroupId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwValue": SimTypeInt(signed=False, label="UInt32"), "Algid": SimTypeInt(signed=False, label="ALG_ID"), "dwLength": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ExtraInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_OID_INFO", pack=False, align=None),
    "CRYPT_PASSWORD_CREDENTIALSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszUsername", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_PASSWORD_CREDENTIALSA", pack=False, align=None),
    "CRYPT_PASSWORD_CREDENTIALSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszUsername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPT_PASSWORD_CREDENTIALSW", pack=False, align=None),
    "CRYPT_PKCS12_PBE_PARAMS": SimStruct(OrderedDict((("iIterations", SimTypeInt(signed=True, label="Int32")), ("cbSalt", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_PKCS12_PBE_PARAMS", pack=False, align=None),
    "CRYPT_PKCS8_EXPORT_PARAMS": SimStruct(OrderedDict((("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("pszPrivateKeyObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEncryptPrivateKeyFunc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAlgorithm", "pClearTextPrivateKey", "pbEncryptedKey", "pcbEncryptedKey", "pVoidEncryptFunc"]), offset=0)), ("pVoidEncryptFunc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_PKCS8_EXPORT_PARAMS", pack=False, align=None),
    "CRYPT_PKCS8_IMPORT_PARAMS": SimStruct(OrderedDict((("PrivateKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pResolvehCryptProvFunc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PRIVATE_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPrivateKeyInfo", "phCryptProv", "pVoidResolveFunc"]), offset=0)), ("pVoidResolveFunc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pDecryptPrivateKeyFunc", SimTypePointer(SimTypeFunction([SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Algorithm", "EncryptedPrivateKey", "pbClearTextKey", "pcbClearTextKey", "pVoidDecryptFunc"]), offset=0)), ("pVoidDecryptFunc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_PKCS8_IMPORT_PARAMS", pack=False, align=None),
    "CRYPT_PRIVATE_KEY_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Algorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("PrivateKey", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pAttributes", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTES", SimStruct), offset=0)),)), name="CRYPT_PRIVATE_KEY_INFO", pack=False, align=None),
    "CRYPT_PROPERTY_REF": SimStruct(OrderedDict((("pszProperty", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbValue", SimTypeInt(signed=False, label="UInt32")), ("pbValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_PROPERTY_REF", pack=False, align=None),
    "CRYPT_PROVIDERS": SimStruct(OrderedDict((("cProviders", SimTypeInt(signed=False, label="UInt32")), ("rgpszProviders", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CRYPT_PROVIDERS", pack=False, align=None),
    "CRYPT_PROVIDER_CERT": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("fCommercial", SimTypeInt(signed=True, label="Int32")), ("fTrustedRoot", SimTypeInt(signed=True, label="Int32")), ("fSelfSigned", SimTypeInt(signed=True, label="Int32")), ("fTestCert", SimTypeInt(signed=True, label="Int32")), ("dwRevokedReason", SimTypeInt(signed=False, label="UInt32")), ("dwConfidence", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("pTrustListContext", SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0)), ("fTrustListSignerCert", SimTypeInt(signed=True, label="Int32")), ("pCtlContext", SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0)), ("dwCtlError", SimTypeInt(signed=False, label="UInt32")), ("fIsCyclic", SimTypeInt(signed=True, label="Int32")), ("pChainElement", SimTypePointer(SimTypeRef("CERT_CHAIN_ELEMENT", SimStruct), offset=0)),)), name="CRYPT_PROVIDER_CERT", pack=False, align=None),
    "CRYPT_PROVIDER_DATA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pWintrustData", SimTypePointer(SimTypeRef("WINTRUST_DATA", SimStruct), offset=0)), ("fOpenedFile", SimTypeInt(signed=True, label="Int32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pgActionID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("hProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwRegSecuritySettings", SimTypeInt(signed=False, label="UInt32")), ("dwRegPolicySettings", SimTypeInt(signed=False, label="UInt32")), ("psPfns", SimTypePointer(SimTypeRef("CRYPT_PROVIDER_FUNCTIONS", SimStruct), offset=0)), ("cdwTrustStepErrors", SimTypeInt(signed=False, label="UInt32")), ("padwTrustStepErrors", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("chStores", SimTypeInt(signed=False, label="UInt32")), ("pahStores", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("dwEncoding", SimTypeInt(signed=False, label="UInt32")), ("hMsg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("csSigners", SimTypeInt(signed=False, label="UInt32")), ("pasSigners", SimTypePointer(SimTypeRef("CRYPT_PROVIDER_SGNR", SimStruct), offset=0)), ("csProvPrivData", SimTypeInt(signed=False, label="UInt32")), ("pasProvPrivData", SimTypePointer(SimTypeRef("CRYPT_PROVIDER_PRIVDATA", SimStruct), offset=0)), ("dwSubjectChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pPDSip": SimTypePointer(SimTypeRef("PROVDATA_SIP", SimStruct), offset=0)}, name="<anon>", label="None")), ("pszUsageOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fRecallWithState", SimTypeInt(signed=True, label="Int32")), ("sftSystemTime", SimTypeRef("FILETIME", SimStruct)), ("pszCTLSignerUsageOID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwProvFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFinalError", SimTypeInt(signed=False, label="UInt32")), ("pRequestUsage", SimTypePointer(SimTypeRef("CERT_USAGE_MATCH", SimStruct), offset=0)), ("dwTrustPubSettings", SimTypeInt(signed=False, label="UInt32")), ("dwUIStateFlags", SimTypeInt(signed=False, label="UInt32")), ("pSigState", SimTypePointer(SimTypeRef("CRYPT_PROVIDER_SIGSTATE", SimStruct), offset=0)), ("pSigSettings", SimTypePointer(SimTypeRef("WINTRUST_SIGNATURE_SETTINGS", SimStruct), offset=0)),)), name="CRYPT_PROVIDER_DATA", pack=False, align=None),
    "CRYPT_PROVIDER_DEFUSAGE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("gActionID", SimTypeBottom(label="Guid")), ("pDefPolicyCallbackData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pDefSIPClientData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_PROVIDER_DEFUSAGE", pack=False, align=None),
    "CRYPT_PROVIDER_FUNCTIONS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pfnAlloc", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvMem2Free"]), offset=0)), ("pfnAddStore2Chain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "hStore2Add"]), offset=0)), ("pfnAddSgnr2Chain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_SGNR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "fCounterSigner", "idxSigner", "pSgnr2Add"]), offset=0)), ("pfnAddCert2Chain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "idxSigner", "fCounterSigner", "idxCounterSigner", "pCert2Add"]), offset=0)), ("pfnAddPrivData2Chain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_PRIVDATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "pPrivData2Add"]), offset=0)), ("pfnInitialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)), ("pfnObjectTrust", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)), ("pfnSignatureTrust", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)), ("pfnCertificateTrust", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)), ("pfnFinalPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)), ("pfnCertCheckPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "idxSigner", "fCounterSignerChain", "idxCounterSigner"]), offset=0)), ("pfnTestFinalPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)), ("psUIpfns", SimTypePointer(SimTypeRef("CRYPT_PROVUI_FUNCS", SimStruct), offset=0)), ("pfnCleanupPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0)),)), name="CRYPT_PROVIDER_FUNCTIONS", pack=False, align=None),
    "CRYPT_PROVIDER_PRIVDATA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("gProviderID", SimTypeBottom(label="Guid")), ("cbProvData", SimTypeInt(signed=False, label="UInt32")), ("pvProvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_PROVIDER_PRIVDATA", pack=False, align=None),
    "CRYPT_PROVIDER_REF": SimStruct(OrderedDict((("dwInterface", SimTypeInt(signed=False, label="UInt32")), ("pszFunction", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cProperties", SimTypeInt(signed=False, label="UInt32")), ("rgpProperties", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_PROPERTY_REF", SimStruct), offset=0), offset=0)), ("pUM", SimTypePointer(SimTypeRef("CRYPT_IMAGE_REF", SimStruct), offset=0)), ("pKM", SimTypePointer(SimTypeRef("CRYPT_IMAGE_REF", SimStruct), offset=0)),)), name="CRYPT_PROVIDER_REF", pack=False, align=None),
    "CRYPT_PROVIDER_REFS": SimStruct(OrderedDict((("cProviders", SimTypeInt(signed=False, label="UInt32")), ("rgpProviders", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_PROVIDER_REF", SimStruct), offset=0), offset=0)),)), name="CRYPT_PROVIDER_REFS", pack=False, align=None),
    "CRYPT_PROVIDER_REG": SimStruct(OrderedDict((("cAliases", SimTypeInt(signed=False, label="UInt32")), ("rgpszAliases", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("pUM", SimTypePointer(SimTypeRef("CRYPT_IMAGE_REG", SimStruct), offset=0)), ("pKM", SimTypePointer(SimTypeRef("CRYPT_IMAGE_REG", SimStruct), offset=0)),)), name="CRYPT_PROVIDER_REG", pack=False, align=None),
    "CRYPT_PROVIDER_REGDEFUSAGE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pgActionID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pwszDllName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszLoadCallbackDataFunctionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszFreeCallbackDataFunctionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_PROVIDER_REGDEFUSAGE", pack=False, align=None),
    "CRYPT_PROVIDER_SGNR": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("sftVerifyAsOf", SimTypeRef("FILETIME", SimStruct)), ("csCertChain", SimTypeInt(signed=False, label="UInt32")), ("pasCertChain", SimTypePointer(SimTypeRef("CRYPT_PROVIDER_CERT", SimStruct), offset=0)), ("dwSignerType", SimTypeInt(signed=False, label="UInt32")), ("psSigner", SimTypePointer(SimTypeRef("CMSG_SIGNER_INFO", SimStruct), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("csCounterSigners", SimTypeInt(signed=False, label="UInt32")), ("pasCounterSigners", SimTypePointer(SimTypeRef("CRYPT_PROVIDER_SGNR", SimStruct), offset=0)), ("pChainContext", SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0)),)), name="CRYPT_PROVIDER_SGNR", pack=False, align=None),
    "CRYPT_PROVIDER_SIGSTATE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("rhSecondarySigs", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("hPrimarySig", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("fFirstAttemptMade", SimTypeInt(signed=True, label="Int32")), ("fNoMoreSigs", SimTypeInt(signed=True, label="Int32")), ("cSecondarySigs", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentIndex", SimTypeInt(signed=False, label="UInt32")), ("fSupportMultiSig", SimTypeInt(signed=True, label="Int32")), ("dwCryptoPolicySupport", SimTypeInt(signed=False, label="UInt32")), ("iAttemptCount", SimTypeInt(signed=False, label="UInt32")), ("fCheckedSealing", SimTypeInt(signed=True, label="Int32")), ("pSealingSignature", SimTypePointer(SimTypeRef("SEALING_SIGNATURE_ATTRIBUTE", SimStruct), offset=0)),)), name="CRYPT_PROVIDER_SIGSTATE", pack=False, align=None),
    "CRYPT_PROVUI_DATA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFinalError", SimTypeInt(signed=False, label="UInt32")), ("pYesButtonText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pNoButtonText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMoreInfoButtonText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAdvancedLinkText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCopyActionText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCopyActionTextNoTS", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCopyActionTextNotSigned", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPT_PROVUI_DATA", pack=False, align=None),
    "CRYPT_PROVUI_FUNCS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("psUIData", SimTypePointer(SimTypeRef("CRYPT_PROVUI_DATA", SimStruct), offset=0)), ("pfnOnMoreInfoClick", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndSecurityDialog", "pProvData"]), offset=0)), ("pfnOnMoreInfoClickDefault", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndSecurityDialog", "pProvData"]), offset=0)), ("pfnOnAdvancedClick", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndSecurityDialog", "pProvData"]), offset=0)), ("pfnOnAdvancedClickDefault", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndSecurityDialog", "pProvData"]), offset=0)),)), name="CRYPT_PROVUI_FUNCS", pack=False, align=None),
    "CRYPT_PSOURCE_ALGORITHM": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("EncodingParameters", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_PSOURCE_ALGORITHM", pack=False, align=None),
    "CRYPT_RC2_CBC_PARAMETERS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("fIV", SimTypeInt(signed=True, label="Int32")), ("rgbIV", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="CRYPT_RC2_CBC_PARAMETERS", pack=False, align=None),
    "CRYPT_RC4_KEY_STATE": SimStruct(OrderedDict((("Key", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("SBox", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("i", SimTypeChar(label="Byte")), ("j", SimTypeChar(label="Byte")),)), name="CRYPT_RC4_KEY_STATE", pack=False, align=None),
    "CRYPT_REGISTER_ACTIONID": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("sInitProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sObjectProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sSignatureProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sCertificateProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sCertificatePolicyProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sFinalPolicyProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sTestPolicyProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)), ("sCleanupProvider", SimTypeRef("CRYPT_TRUST_REG_ENTRY", SimStruct)),)), name="CRYPT_REGISTER_ACTIONID", pack=False, align=None),
    "CRYPT_RETRIEVE_AUX_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pLastSyncTime", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("dwMaxUrlRetrievalByteCount", SimTypeInt(signed=False, label="UInt32")), ("pPreFetchInfo", SimTypePointer(SimTypeRef("CRYPTNET_URL_CACHE_PRE_FETCH_INFO", SimStruct), offset=0)), ("pFlushInfo", SimTypePointer(SimTypeRef("CRYPTNET_URL_CACHE_FLUSH_INFO", SimStruct), offset=0)), ("ppResponseInfo", SimTypePointer(SimTypePointer(SimTypeRef("CRYPTNET_URL_CACHE_RESPONSE_INFO", SimStruct), offset=0), offset=0)), ("pwszCacheFileNamePrefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pftCacheResync", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("fProxyCacheRetrieval", SimTypeInt(signed=True, label="Int32")), ("dwHttpStatusCode", SimTypeInt(signed=False, label="UInt32")), ("ppwszErrorResponseHeaders", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("ppErrorContentBlob", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), offset=0)),)), name="CRYPT_RETRIEVE_AUX_INFO", pack=False, align=None),
    "CRYPT_RSAES_OAEP_PARAMETERS": SimStruct(OrderedDict((("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("MaskGenAlgorithm", SimTypeRef("CRYPT_MASK_GEN_ALGORITHM", SimStruct)), ("PSourceAlgorithm", SimTypeRef("CRYPT_PSOURCE_ALGORITHM", SimStruct)),)), name="CRYPT_RSAES_OAEP_PARAMETERS", pack=False, align=None),
    "CRYPT_RSA_SSA_PSS_PARAMETERS": SimStruct(OrderedDict((("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("MaskGenAlgorithm", SimTypeRef("CRYPT_MASK_GEN_ALGORITHM", SimStruct)), ("dwSaltLength", SimTypeInt(signed=False, label="UInt32")), ("dwTrailerField", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_RSA_SSA_PSS_PARAMETERS", pack=False, align=None),
    "CRYPT_SEQUENCE_OF_ANY": SimStruct(OrderedDict((("cValue", SimTypeInt(signed=False, label="UInt32")), ("rgValue", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CRYPT_SEQUENCE_OF_ANY", pack=False, align=None),
    "CRYPT_SET_HASH_PARAM": SimTypeInt(signed=False, label="CRYPT_SET_HASH_PARAM"),
    "CRYPT_SET_PROV_PARAM_ID": SimTypeInt(signed=False, label="CRYPT_SET_PROV_PARAM_ID"),
    "CRYPT_SIGN_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgEncodingType", SimTypeInt(signed=False, label="UInt32")), ("pSigningCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("pvHashAuxInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cMsgCert", SimTypeInt(signed=False, label="UInt32")), ("rgpMsgCert", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("cMsgCrl", SimTypeInt(signed=False, label="UInt32")), ("rgpMsgCrl", SimTypePointer(SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), offset=0)), ("cAuthAttr", SimTypeInt(signed=False, label="UInt32")), ("rgAuthAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)), ("cUnauthAttr", SimTypeInt(signed=False, label="UInt32")), ("rgUnauthAttr", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwInnerContentType", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_SIGN_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_SMART_CARD_ROOT_INFO": SimStruct(OrderedDict((("rgbCardID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("luid", SimTypeRef("ROOT_INFO_LUID", SimStruct)),)), name="CRYPT_SMART_CARD_ROOT_INFO", pack=False, align=None),
    "CRYPT_SMIME_CAPABILITIES": SimStruct(OrderedDict((("cCapability", SimTypeInt(signed=False, label="UInt32")), ("rgCapability", SimTypePointer(SimTypeRef("CRYPT_SMIME_CAPABILITY", SimStruct), offset=0)),)), name="CRYPT_SMIME_CAPABILITIES", pack=False, align=None),
    "CRYPT_SMIME_CAPABILITY": SimStruct(OrderedDict((("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Parameters", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_SMIME_CAPABILITY", pack=False, align=None),
    "CRYPT_STRING": SimTypeInt(signed=False, label="CRYPT_STRING"),
    "CRYPT_TIMESTAMP_ACCURACY": SimStruct(OrderedDict((("dwSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwMillis", SimTypeInt(signed=False, label="UInt32")), ("dwMicros", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_TIMESTAMP_ACCURACY", pack=False, align=None),
    "CRYPT_TIMESTAMP_CONTEXT": SimStruct(OrderedDict((("cbEncoded", SimTypeInt(signed=False, label="UInt32")), ("pbEncoded", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pTimeStamp", SimTypePointer(SimTypeRef("CRYPT_TIMESTAMP_INFO", SimStruct), offset=0)),)), name="CRYPT_TIMESTAMP_CONTEXT", pack=False, align=None),
    "CRYPT_TIMESTAMP_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pszTSAPolicyId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("HashedMessage", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("ftTime", SimTypeRef("FILETIME", SimStruct)), ("pvAccuracy", SimTypePointer(SimTypeRef("CRYPT_TIMESTAMP_ACCURACY", SimStruct), offset=0)), ("fOrdering", SimTypeInt(signed=True, label="Int32")), ("Nonce", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("Tsa", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CRYPT_TIMESTAMP_INFO", pack=False, align=None),
    "CRYPT_TIMESTAMP_PARA": SimStruct(OrderedDict((("pszTSAPolicyId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fRequestCerts", SimTypeInt(signed=True, label="Int32")), ("Nonce", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CRYPT_TIMESTAMP_PARA", pack=False, align=None),
    "CRYPT_TIMESTAMP_REQUEST": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="CRYPT_TIMESTAMP_VERSION")), ("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("HashedMessage", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pszTSAPolicyId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Nonce", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("fCertReq", SimTypeInt(signed=True, label="Int32")), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CRYPT_TIMESTAMP_REQUEST", pack=False, align=None),
    "CRYPT_TIMESTAMP_RESPONSE": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="CRYPT_TIMESTAMP_RESPONSE_STATUS")), ("cFreeText", SimTypeInt(signed=False, label="UInt32")), ("rgFreeText", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("FailureInfo", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("ContentInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_TIMESTAMP_RESPONSE", pack=False, align=None),
    "CRYPT_TIMESTAMP_RESPONSE_STATUS": SimTypeInt(signed=False, label="CRYPT_TIMESTAMP_RESPONSE_STATUS"),
    "CRYPT_TIMESTAMP_VERSION": SimTypeInt(signed=False, label="CRYPT_TIMESTAMP_VERSION"),
    "CRYPT_TIME_STAMP_REQUEST_INFO": SimStruct(OrderedDict((("pszTimeStampAlgorithm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszContentType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Content", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cAttribute", SimTypeInt(signed=False, label="UInt32")), ("rgAttribute", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)),)), name="CRYPT_TIME_STAMP_REQUEST_INFO", pack=False, align=None),
    "CRYPT_TRUST_REG_ENTRY": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pwszDLLName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFunctionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPT_TRUST_REG_ENTRY", pack=False, align=None),
    "CRYPT_URL_ARRAY": SimStruct(OrderedDict((("cUrl", SimTypeInt(signed=False, label="UInt32")), ("rgwszUrl", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="CRYPT_URL_ARRAY", pack=False, align=None),
    "CRYPT_URL_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSyncDeltaTime", SimTypeInt(signed=False, label="UInt32")), ("cGroup", SimTypeInt(signed=False, label="UInt32")), ("rgcGroupEntry", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="CRYPT_URL_INFO", pack=False, align=None),
    "CRYPT_VERIFY_CERT_FLAGS": SimTypeInt(signed=False, label="CRYPT_VERIFY_CERT_FLAGS"),
    "CRYPT_VERIFY_CERT_SIGN_STRONG_PROPERTIES_INFO": SimStruct(OrderedDict((("CertSignHashCNGAlgPropData", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("CertIssuerPubKeyBitLengthPropData", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_VERIFY_CERT_SIGN_STRONG_PROPERTIES_INFO", pack=False, align=None),
    "CRYPT_VERIFY_CERT_SIGN_WEAK_HASH_INFO": SimStruct(OrderedDict((("cCNGHashAlgid", SimTypeInt(signed=False, label="UInt32")), ("rgpwszCNGHashAlgid", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwWeakIndex", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_VERIFY_CERT_SIGN_WEAK_HASH_INFO", pack=False, align=None),
    "CRYPT_VERIFY_MESSAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMsgAndCertEncodingType", SimTypeInt(signed=False, label="UInt32")), ("hCryptProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pfnGetSignerCertificate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeRef("CERT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), arg_names=["pvGetArg", "dwCertEncodingType", "pSignerId", "hMsgCertStore"]), offset=0)), ("pvGetArg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_VERIFY_MESSAGE_PARA", pack=False, align=None),
    "CRYPT_X942_OTHER_INFO": SimStruct(OrderedDict((("pszContentEncryptionObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rgbCounter", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("rgbKeyLength", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("PubInfo", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CRYPT_X942_OTHER_INFO", pack=False, align=None),
    "CRYPT_XML_ALGORITHM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wszAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Encoded", SimTypeRef("CRYPT_XML_BLOB", SimStruct)),)), name="CRYPT_XML_ALGORITHM", pack=False, align=None),
    "CRYPT_XML_ALGORITHM_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wszAlgorithmURI", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwGroupId", SimTypeInt(signed=False, label="CRYPT_XML_GROUP_ID")), ("wszCNGAlgid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszCNGExtraAlgid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSignFlags", SimTypeInt(signed=False, label="UInt32")), ("dwVerifyFlags", SimTypeInt(signed=False, label="UInt32")), ("pvPaddingInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pvExtraInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_XML_ALGORITHM_INFO", pack=False, align=None),
    "CRYPT_XML_BLOB": SimStruct(OrderedDict((("dwCharset", SimTypeInt(signed=False, label="CRYPT_XML_CHARSET")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_XML_BLOB", pack=False, align=None),
    "CRYPT_XML_CHARSET": SimTypeInt(signed=False, label="CRYPT_XML_CHARSET"),
    "CRYPT_XML_CRYPTOGRAPHIC_INTERFACE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fpCryptXmlEncodeAlgorithm", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="CRYPT_XML_CHARSET"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState", "pbData", "cbData"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAlgInfo", "dwCharset", "pvCallbackState", "pfnWrite"]), offset=0)), ("fpCryptXmlCreateDigest", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDigestMethod", "pcbSize", "phDigest"]), offset=0)), ("fpCryptXmlDigestData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDigest", "pbData", "cbData"]), offset=0)), ("fpCryptXmlFinalizeDigest", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDigest", "pbDigest", "cbDigest"]), offset=0)), ("fpCryptXmlCloseDigest", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDigest"]), offset=0)), ("fpCryptXmlSignData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSignatureMethod", "hCryptProvOrNCryptKey", "dwKeySpec", "pbInput", "cbInput", "pbOutput", "cbOutput", "pcbResult"]), offset=0)), ("fpCryptXmlVerifySignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pSignatureMethod", "hKey", "pbInput", "cbInput", "pbSignature", "cbSignature"]), offset=0)), ("fpCryptXmlGetAlgorithmInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM_INFO", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pXmlAlgorithm", "ppAlgInfo"]), offset=0)),)), name="CRYPT_XML_CRYPTOGRAPHIC_INTERFACE", pack=False, align=None),
    "CRYPT_XML_DATA_BLOB": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CRYPT_XML_DATA_BLOB", pack=False, align=None),
    "CRYPT_XML_DATA_PROVIDER": SimStruct(OrderedDict((("pvCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbBufferSize", SimTypeInt(signed=False, label="UInt32")), ("pfnRead", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState", "pbData", "cbData", "pcbRead"]), offset=0)), ("pfnClose", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState"]), offset=0)),)), name="CRYPT_XML_DATA_PROVIDER", pack=False, align=None),
    "CRYPT_XML_DOC_CTXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hDocCtxt", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pTransformsConfig", SimTypePointer(SimTypeRef("CRYPT_XML_TRANSFORM_CHAIN_CONFIG", SimStruct), offset=0)), ("cSignature", SimTypeInt(signed=False, label="UInt32")), ("rgpSignature", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_SIGNATURE", SimStruct), offset=0), offset=0)),)), name="CRYPT_XML_DOC_CTXT", pack=False, align=None),
    "CRYPT_XML_FLAGS": SimTypeInt(signed=False, label="CRYPT_XML_FLAGS"),
    "CRYPT_XML_GROUP_ID": SimTypeInt(signed=False, label="CRYPT_XML_GROUP_ID"),
    "CRYPT_XML_ISSUER_SERIAL": SimStruct(OrderedDict((("wszIssuer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszSerial", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CRYPT_XML_ISSUER_SERIAL", pack=False, align=None),
    "CRYPT_XML_KEYINFO_PARAM": SimStruct(OrderedDict((("wszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszKeyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SKI", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("wszSubjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cCertificate", SimTypeInt(signed=False, label="UInt32")), ("rgCertificate", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("cCRL", SimTypeInt(signed=False, label="UInt32")), ("rgCRL", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="CRYPT_XML_KEYINFO_PARAM", pack=False, align=None),
    "CRYPT_XML_KEYINFO_SPEC": SimTypeInt(signed=False, label="CRYPT_XML_KEYINFO_SPEC"),
    "CRYPT_XML_KEYINFO_TYPE": SimTypeInt(signed=False, label="CRYPT_XML_KEYINFO_TYPE"),
    "CRYPT_XML_KEY_DSA_KEY_VALUE": SimStruct(OrderedDict((("P", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("Q", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("G", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("Y", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("J", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("Seed", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("Counter", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)),)), name="CRYPT_XML_KEY_DSA_KEY_VALUE", pack=False, align=None),
    "CRYPT_XML_KEY_ECDSA_KEY_VALUE": SimStruct(OrderedDict((("wszNamedCurve", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("X", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("Y", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("ExplicitPara", SimTypeRef("CRYPT_XML_BLOB", SimStruct)),)), name="CRYPT_XML_KEY_ECDSA_KEY_VALUE", pack=False, align=None),
    "CRYPT_XML_KEY_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cKeyInfo", SimTypeInt(signed=False, label="UInt32")), ("rgKeyInfo", SimTypePointer(SimTypeRef("CRYPT_XML_KEY_INFO_ITEM", SimStruct), offset=0)), ("hVerifyKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CRYPT_XML_KEY_INFO", pack=False, align=None),
    "CRYPT_XML_KEY_INFO_ITEM": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="CRYPT_XML_KEYINFO_TYPE")), ("Anonymous", SimUnion({"wszKeyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "KeyValue": SimTypeRef("CRYPT_XML_KEY_VALUE", SimStruct), "RetrievalMethod": SimTypeRef("CRYPT_XML_BLOB", SimStruct), "X509Data": SimTypeRef("CRYPT_XML_X509DATA", SimStruct), "Custom": SimTypeRef("CRYPT_XML_BLOB", SimStruct)}, name="<anon>", label="None")),)), name="CRYPT_XML_KEY_INFO_ITEM", pack=False, align=None),
    "CRYPT_XML_KEY_RSA_KEY_VALUE": SimStruct(OrderedDict((("Modulus", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)), ("Exponent", SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct)),)), name="CRYPT_XML_KEY_RSA_KEY_VALUE", pack=False, align=None),
    "CRYPT_XML_KEY_VALUE": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="CRYPT_XML_KEY_VALUE_TYPE")), ("Anonymous", SimUnion({"DSAKeyValue": SimTypeRef("CRYPT_XML_KEY_DSA_KEY_VALUE", SimStruct), "RSAKeyValue": SimTypeRef("CRYPT_XML_KEY_RSA_KEY_VALUE", SimStruct), "ECDSAKeyValue": SimTypeRef("CRYPT_XML_KEY_ECDSA_KEY_VALUE", SimStruct), "Custom": SimTypeRef("CRYPT_XML_BLOB", SimStruct)}, name="<anon>", label="None")),)), name="CRYPT_XML_KEY_VALUE", pack=False, align=None),
    "CRYPT_XML_KEY_VALUE_TYPE": SimTypeInt(signed=False, label="CRYPT_XML_KEY_VALUE_TYPE"),
    "CRYPT_XML_OBJECT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("wszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszMimeType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszEncoding", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Manifest", SimTypeRef("CRYPT_XML_REFERENCES", SimStruct)), ("Encoded", SimTypeRef("CRYPT_XML_BLOB", SimStruct)),)), name="CRYPT_XML_OBJECT", pack=False, align=None),
    "CRYPT_XML_PROPERTY": SimStruct(OrderedDict((("dwPropId", SimTypeInt(signed=False, label="CRYPT_XML_PROPERTY_ID")), ("pvValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbValue", SimTypeInt(signed=False, label="UInt32")),)), name="CRYPT_XML_PROPERTY", pack=False, align=None),
    "CRYPT_XML_PROPERTY_ID": SimTypeInt(signed=False, label="CRYPT_XML_PROPERTY_ID"),
    "CRYPT_XML_REFERENCE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hReference", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("wszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszUri", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DigestMethod", SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct)), ("DigestValue", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cTransform", SimTypeInt(signed=False, label="UInt32")), ("rgTransform", SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0)),)), name="CRYPT_XML_REFERENCE", pack=False, align=None),
    "CRYPT_XML_REFERENCES": SimStruct(OrderedDict((("cReference", SimTypeInt(signed=False, label="UInt32")), ("rgpReference", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_REFERENCE", SimStruct), offset=0), offset=0)),)), name="CRYPT_XML_REFERENCES", pack=False, align=None),
    "CRYPT_XML_SIGNATURE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hSignature", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("wszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SignedInfo", SimTypeRef("CRYPT_XML_SIGNED_INFO", SimStruct)), ("SignatureValue", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pKeyInfo", SimTypePointer(SimTypeRef("CRYPT_XML_KEY_INFO", SimStruct), offset=0)), ("cObject", SimTypeInt(signed=False, label="UInt32")), ("rgpObject", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_OBJECT", SimStruct), offset=0), offset=0)),)), name="CRYPT_XML_SIGNATURE", pack=False, align=None),
    "CRYPT_XML_SIGNED_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Canonicalization", SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct)), ("SignatureMethod", SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct)), ("cReference", SimTypeInt(signed=False, label="UInt32")), ("rgpReference", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_REFERENCE", SimStruct), offset=0), offset=0)), ("Encoded", SimTypeRef("CRYPT_XML_BLOB", SimStruct)),)), name="CRYPT_XML_SIGNED_INFO", pack=False, align=None),
    "CRYPT_XML_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwErrorStatus", SimTypeInt(signed=False, label="CRYPT_XML_STATUS_ERROR_STATUS")), ("dwInfoStatus", SimTypeInt(signed=False, label="CRYPT_XML_STATUS_INFO_STATUS")),)), name="CRYPT_XML_STATUS", pack=False, align=None),
    "CRYPT_XML_STATUS_ERROR_STATUS": SimTypeInt(signed=False, label="CRYPT_XML_STATUS_ERROR_STATUS"),
    "CRYPT_XML_STATUS_INFO_STATUS": SimTypeInt(signed=False, label="CRYPT_XML_STATUS_INFO_STATUS"),
    "CRYPT_XML_TRANSFORM_CHAIN_CONFIG": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cTransformInfo", SimTypeInt(signed=False, label="UInt32")), ("rgpTransformInfo", SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_TRANSFORM_INFO", SimStruct), offset=0), offset=0)),)), name="CRYPT_XML_TRANSFORM_CHAIN_CONFIG", pack=False, align=None),
    "CRYPT_XML_TRANSFORM_FLAGS": SimTypeInt(signed=False, label="CRYPT_XML_TRANSFORM_FLAGS"),
    "CRYPT_XML_TRANSFORM_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wszAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CRYPT_XML_TRANSFORM_FLAGS")), ("pfnCreateTransform", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_XML_DATA_PROVIDER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_XML_DATA_PROVIDER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTransform", "pProviderIn", "pProviderOut"]), offset=0)),)), name="CRYPT_XML_TRANSFORM_INFO", pack=False, align=None),
    "CRYPT_XML_X509DATA": SimStruct(OrderedDict((("cX509Data", SimTypeInt(signed=False, label="UInt32")), ("rgX509Data", SimTypePointer(SimTypeRef("CRYPT_XML_X509DATA_ITEM", SimStruct), offset=0)),)), name="CRYPT_XML_X509DATA", pack=False, align=None),
    "CRYPT_XML_X509DATA_ITEM": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="CRYPT_XML_X509DATA_TYPE")), ("Anonymous", SimUnion({"IssuerSerial": SimTypeRef("CRYPT_XML_ISSUER_SERIAL", SimStruct), "SKI": SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct), "wszSubjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Certificate": SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct), "CRL": SimTypeRef("CRYPT_XML_DATA_BLOB", SimStruct), "Custom": SimTypeRef("CRYPT_XML_BLOB", SimStruct)}, name="<anon>", label="None")),)), name="CRYPT_XML_X509DATA_ITEM", pack=False, align=None),
    "CRYPT_XML_X509DATA_TYPE": SimTypeInt(signed=False, label="CRYPT_XML_X509DATA_TYPE"),
    "CR_DISP": SimTypeInt(signed=False, label="CR_DISP"),
    "CSADDR_INFO": SimStruct(OrderedDict((("LocalAddr", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("RemoteAddr", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("iSocketType", SimTypeInt(signed=True, label="Int32")), ("iProtocol", SimTypeInt(signed=True, label="Int32")),)), name="CSADDR_INFO", pack=False, align=None),
    "CSBACKUP_TYPE": SimTypeInt(signed=False, label="CSBACKUP_TYPE"),
    "CSC_Binding": SimTypeInt(signed=False, label="CSC_Binding"),
    "CSC_COMTIIntrinsicsConfig": SimTypeInt(signed=False, label="CSC_COMTIIntrinsicsConfig"),
    "CSC_IISIntrinsicsConfig": SimTypeInt(signed=False, label="CSC_IISIntrinsicsConfig"),
    "CSC_InheritanceConfig": SimTypeInt(signed=False, label="CSC_InheritanceConfig"),
    "CSC_PartitionConfig": SimTypeInt(signed=False, label="CSC_PartitionConfig"),
    "CSC_SxsConfig": SimTypeInt(signed=False, label="CSC_SxsConfig"),
    "CSC_SynchronizationConfig": SimTypeInt(signed=False, label="CSC_SynchronizationConfig"),
    "CSC_ThreadPool": SimTypeInt(signed=False, label="CSC_ThreadPool"),
    "CSC_TrackerConfig": SimTypeInt(signed=False, label="CSC_TrackerConfig"),
    "CSC_TransactionConfig": SimTypeInt(signed=False, label="CSC_TransactionConfig"),
    "CSEDB_RSTMAPW": SimStruct(OrderedDict((("pwszDatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszNewDatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CSEDB_RSTMAPW", pack=False, align=None),
    "CSFV": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pshf", SimTypeBottom(label="IShellFolder")), ("psvOuter", SimTypeBottom(label="IShellView")), ("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("lEvents", SimTypeInt(signed=True, label="Int32")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IShellView"), SimTypeBottom(label="IShellFolder"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["psvOuter", "psf", "hwndMain", "uMsg", "wParam", "lParam"]), offset=0)), ("fvm", SimTypeInt(signed=False, label="FOLDERVIEWMODE")),)), name="CSFV", pack=False, align=None),
    "CSIMAADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="CSIMAADPCMWAVEFORMAT", pack=False, align=None),
    "CSPLATFORM": SimStruct(OrderedDict((("dwPlatformId", SimTypeInt(signed=False, label="UInt32")), ("dwVersionHi", SimTypeInt(signed=False, label="UInt32")), ("dwVersionLo", SimTypeInt(signed=False, label="UInt32")), ("dwProcessorArch", SimTypeInt(signed=False, label="UInt32")),)), name="CSPLATFORM", pack=False, align=None),
    "CSTRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CSTRING", pack=False, align=None),
    "CSVFS_DISK_CONNECTIVITY": SimTypeInt(signed=False, label="CSVFS_DISK_CONNECTIVITY"),
    "CSV_CONTROL_OP": SimTypeInt(signed=False, label="CSV_CONTROL_OP"),
    "CSV_CONTROL_PARAM": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="CSV_CONTROL_OP")), ("Unused", SimTypeLongLong(signed=True, label="Int64")),)), name="CSV_CONTROL_PARAM", pack=False, align=None),
    "CSV_DOWN_LEVEL_FILE_TYPE": SimTypeInt(signed=False, label="CSV_DOWN_LEVEL_FILE_TYPE"),
    "CSV_DOWN_LEVEL_OPEN_ECP_CONTEXT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("IsResume", SimTypeChar(label="Byte")), ("FileType", SimTypeInt(signed=False, label="CSV_DOWN_LEVEL_FILE_TYPE")), ("SourceNodeId", SimTypeInt(signed=False, label="UInt32")), ("DestinationNodeId", SimTypeInt(signed=False, label="UInt32")),)), name="CSV_DOWN_LEVEL_OPEN_ECP_CONTEXT", pack=False, align=None),
    "CSV_IS_OWNED_BY_CSVFS": SimStruct(OrderedDict((("OwnedByCSVFS", SimTypeChar(label="Byte")),)), name="CSV_IS_OWNED_BY_CSVFS", pack=False, align=None),
    "CSV_MGMT_LOCK": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="CSV_MGMT_LOCK", pack=False, align=None),
    "CSV_NAMESPACE_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("SectorSize", SimTypeInt(signed=False, label="UInt32")),)), name="CSV_NAMESPACE_INFO", pack=False, align=None),
    "CSV_QUERY_FILE_REVISION": SimStruct(OrderedDict((("FileId", SimTypeLongLong(signed=True, label="Int64")), ("FileRevision", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 3)),)), name="CSV_QUERY_FILE_REVISION", pack=False, align=None),
    "CSV_QUERY_FILE_REVISION_ECP_CONTEXT": SimStruct(OrderedDict((("FileId", SimTypeLongLong(signed=True, label="Int64")), ("FileRevision", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 3)),)), name="CSV_QUERY_FILE_REVISION_ECP_CONTEXT", pack=False, align=None),
    "CSV_QUERY_FILE_REVISION_ECP_CONTEXT_FILE_ID_128": SimStruct(OrderedDict((("FileId", SimTypeRef("FILE_ID_128", SimStruct)), ("FileRevision", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 3)),)), name="CSV_QUERY_FILE_REVISION_ECP_CONTEXT_FILE_ID_128", pack=False, align=None),
    "CSV_QUERY_FILE_REVISION_FILE_ID_128": SimStruct(OrderedDict((("FileId", SimTypeRef("FILE_ID_128", SimStruct)), ("FileRevision", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 3)),)), name="CSV_QUERY_FILE_REVISION_FILE_ID_128", pack=False, align=None),
    "CSV_QUERY_MDS_PATH": SimStruct(OrderedDict((("MdsNodeId", SimTypeInt(signed=False, label="UInt32")), ("DsNodeId", SimTypeInt(signed=False, label="UInt32")), ("PathLength", SimTypeInt(signed=False, label="UInt32")), ("Path", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CSV_QUERY_MDS_PATH", pack=False, align=None),
    "CSV_QUERY_MDS_PATH_V2": SimStruct(OrderedDict((("Version", SimTypeLongLong(signed=True, label="Int64")), ("RequiredSize", SimTypeInt(signed=False, label="UInt32")), ("MdsNodeId", SimTypeInt(signed=False, label="UInt32")), ("DsNodeId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DiskConnectivity", SimTypeInt(signed=False, label="CSVFS_DISK_CONNECTIVITY")), ("VolumeId", SimTypeBottom(label="Guid")), ("IpAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("IpAddressLength", SimTypeInt(signed=False, label="UInt32")), ("PathOffset", SimTypeInt(signed=False, label="UInt32")), ("PathLength", SimTypeInt(signed=False, label="UInt32")),)), name="CSV_QUERY_MDS_PATH_V2", pack=False, align=None),
    "CSV_QUERY_REDIRECT_STATE": SimStruct(OrderedDict((("MdsNodeId", SimTypeInt(signed=False, label="UInt32")), ("DsNodeId", SimTypeInt(signed=False, label="UInt32")), ("FileRedirected", SimTypeChar(label="Byte")),)), name="CSV_QUERY_REDIRECT_STATE", pack=False, align=None),
    "CSV_QUERY_VETO_FILE_DIRECT_IO_OUTPUT": SimStruct(OrderedDict((("VetoedFromAltitudeIntegral", SimTypeLongLong(signed=False, label="UInt64")), ("VetoedFromAltitudeDecimal", SimTypeLongLong(signed=False, label="UInt64")), ("Reason", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="CSV_QUERY_VETO_FILE_DIRECT_IO_OUTPUT", pack=False, align=None),
    "CSV_QUERY_VOLUME_ID": SimStruct(OrderedDict((("VolumeId", SimTypeBottom(label="Guid")),)), name="CSV_QUERY_VOLUME_ID", pack=False, align=None),
    "CSV_QUERY_VOLUME_REDIRECT_STATE": SimStruct(OrderedDict((("MdsNodeId", SimTypeInt(signed=False, label="UInt32")), ("DsNodeId", SimTypeInt(signed=False, label="UInt32")), ("IsDiskConnected", SimTypeChar(label="Byte")), ("ClusterEnableDirectIo", SimTypeChar(label="Byte")), ("DiskConnectivity", SimTypeInt(signed=False, label="CSVFS_DISK_CONNECTIVITY")),)), name="CSV_QUERY_VOLUME_REDIRECT_STATE", pack=False, align=None),
    "CSV_SET_HANDLE_PROPERTIES_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PauseTimeoutInSeconds", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="CSV_SET_HANDLE_PROPERTIES_ECP_CONTEXT", pack=False, align=None),
    "CSV_SET_VOLUME_ID": SimStruct(OrderedDict((("VolumeId", SimTypeBottom(label="Guid")),)), name="CSV_SET_VOLUME_ID", pack=False, align=None),
    "CS_DES": SimStruct(OrderedDict((("CSD_SignatureLength", SimTypeInt(signed=False, label="UInt32")), ("CSD_LegacyDataOffset", SimTypeInt(signed=False, label="UInt32")), ("CSD_LegacyDataSize", SimTypeInt(signed=False, label="UInt32")), ("CSD_Flags", SimTypeInt(signed=False, label="UInt32")), ("CSD_ClassGuid", SimTypeBottom(label="Guid")), ("CSD_Signature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CS_DES", pack=False, align=None),
    "CS_RESOURCE": SimStruct(OrderedDict((("CS_Header", SimTypeRef("CS_DES", SimStruct)),)), name="CS_RESOURCE", pack=False, align=None),
    "CS_TAG_GETTING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "fServerSide", "pulSendingTag", "pulDesiredReceivingTag", "pulReceivingTag", "pStatus"]), offset=0),
    "CS_TYPE_FROM_NETCS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "pNetworkData", "ulNetworkDataLength", "ulLocalBufferSize", "pLocalData", "pulLocalDataLength", "pStatus"]), offset=0),
    "CS_TYPE_LOCAL_SIZE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="IDL_CS_CONVERT"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "ulNetworkBufferSize", "conversionType", "pulLocalBufferSize", "pStatus"]), offset=0),
    "CS_TYPE_NET_SIZE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="IDL_CS_CONVERT"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "ulLocalBufferSize", "conversionType", "pulNetworkBufferSize", "pStatus"]), offset=0),
    "CS_TYPE_TO_NETCS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "pLocalData", "ulLocalDataLength", "pNetworkData", "pulNetworkDataLength", "pStatus"]), offset=0),
    "CTAPCBOR_HYBRID_STORAGE_LINKED_DATA": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbContactId", SimTypeInt(signed=False, label="UInt32")), ("pbContactId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLinkId", SimTypeInt(signed=False, label="UInt32")), ("pbLinkId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLinkSecret", SimTypeInt(signed=False, label="UInt32")), ("pbLinkSecret", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPublicKey", SimTypeInt(signed=False, label="UInt32")), ("pbPublicKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszAuthenticatorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wEncodedTunnelServerDomain", SimTypeShort(signed=False, label="UInt16")),)), name="CTAPCBOR_HYBRID_STORAGE_LINKED_DATA", pack=False, align=None),
    "CTL_ANY_SUBJECT_INFO": SimStruct(OrderedDict((("SubjectAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("SubjectIdentifier", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="CTL_ANY_SUBJECT_INFO", pack=False, align=None),
    "CTL_CONTEXT": SimStruct(OrderedDict((("dwMsgAndCertEncodingType", SimTypeInt(signed=False, label="UInt32")), ("pbCtlEncoded", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCtlEncoded", SimTypeInt(signed=False, label="UInt32")), ("pCtlInfo", SimTypePointer(SimTypeRef("CTL_INFO", SimStruct), offset=0)), ("hCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCryptMsg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pbCtlContent", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCtlContent", SimTypeInt(signed=False, label="UInt32")),)), name="CTL_CONTEXT", pack=False, align=None),
    "CTL_ENTRY": SimStruct(OrderedDict((("SubjectIdentifier", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cAttribute", SimTypeInt(signed=False, label="UInt32")), ("rgAttribute", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTE", SimStruct), offset=0)),)), name="CTL_ENTRY", pack=False, align=None),
    "CTL_FIND_SUBJECT_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pUsagePara", SimTypePointer(SimTypeRef("CTL_FIND_USAGE_PARA", SimStruct), offset=0)), ("dwSubjectType", SimTypeInt(signed=False, label="UInt32")), ("pvSubject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="CTL_FIND_SUBJECT_PARA", pack=False, align=None),
    "CTL_FIND_USAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("SubjectUsage", SimTypeRef("CTL_USAGE", SimStruct)), ("ListIdentifier", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pSigner", SimTypePointer(SimTypeRef("CERT_INFO", SimStruct), offset=0)),)), name="CTL_FIND_USAGE_PARA", pack=False, align=None),
    "CTL_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("SubjectUsage", SimTypeRef("CTL_USAGE", SimStruct)), ("ListIdentifier", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SequenceNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("ThisUpdate", SimTypeRef("FILETIME", SimStruct)), ("NextUpdate", SimTypeRef("FILETIME", SimStruct)), ("SubjectAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("cCTLEntry", SimTypeInt(signed=False, label="UInt32")), ("rgCTLEntry", SimTypePointer(SimTypeRef("CTL_ENTRY", SimStruct), offset=0)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="CTL_INFO", pack=False, align=None),
    "CTL_MODIFY_REQUEST": SimStruct(OrderedDict((("pccert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("dwOperation", SimTypeInt(signed=False, label="CTL_MODIFY_REQUEST_OPERATION")), ("dwError", SimTypeInt(signed=False, label="UInt32")),)), name="CTL_MODIFY_REQUEST", pack=False, align=None),
    "CTL_MODIFY_REQUEST_OPERATION": SimTypeInt(signed=False, label="CTL_MODIFY_REQUEST_OPERATION"),
    "CTL_USAGE": SimStruct(OrderedDict((("cUsageIdentifier", SimTypeInt(signed=False, label="UInt32")), ("rgpszUsageIdentifier", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="CTL_USAGE", pack=False, align=None),
    "CTL_USAGE_MATCH": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("Usage", SimTypeRef("CTL_USAGE", SimStruct)),)), name="CTL_USAGE_MATCH", pack=False, align=None),
    "CTL_VERIFY_USAGE_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ListIdentifier", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("cCtlStore", SimTypeInt(signed=False, label="UInt32")), ("rghCtlStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cSignerStore", SimTypeInt(signed=False, label="UInt32")), ("rghSignerStore", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="CTL_VERIFY_USAGE_PARA", pack=False, align=None),
    "CTL_VERIFY_USAGE_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ppCtl", SimTypePointer(SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), offset=0)), ("dwCtlEntryIndex", SimTypeInt(signed=False, label="UInt32")), ("ppSigner", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("dwSignerIndex", SimTypeInt(signed=False, label="UInt32")),)), name="CTL_VERIFY_USAGE_STATUS", pack=False, align=None),
    "CTRLINFO": SimTypeInt(signed=False, label="CTRLINFO"),
    "CTRL_LEVEL": SimTypeInt(signed=False, label="CTRL_LEVEL"),
    "CUEBANNERSTATES": SimTypeInt(signed=False, label="CUEBANNERSTATES"),
    "CURRENCYFMTA": SimStruct(OrderedDict((("NumDigits", SimTypeInt(signed=False, label="UInt32")), ("LeadingZero", SimTypeInt(signed=False, label="UInt32")), ("Grouping", SimTypeInt(signed=False, label="UInt32")), ("lpDecimalSep", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpThousandSep", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NegativeOrder", SimTypeInt(signed=False, label="UInt32")), ("PositiveOrder", SimTypeInt(signed=False, label="UInt32")), ("lpCurrencySymbol", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="CURRENCYFMTA", pack=False, align=None),
    "CURRENCYFMTW": SimStruct(OrderedDict((("NumDigits", SimTypeInt(signed=False, label="UInt32")), ("LeadingZero", SimTypeInt(signed=False, label="UInt32")), ("Grouping", SimTypeInt(signed=False, label="UInt32")), ("lpDecimalSep", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpThousandSep", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NegativeOrder", SimTypeInt(signed=False, label="UInt32")), ("PositiveOrder", SimTypeInt(signed=False, label="UInt32")), ("lpCurrencySymbol", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CURRENCYFMTW", pack=False, align=None),
    "CURSORINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="CURSORINFO_FLAGS")), ("hCursor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ptScreenPos", SimTypeRef("POINT", SimStruct)),)), name="CURSORINFO", pack=False, align=None),
    "CURSORINFO_FLAGS": SimTypeInt(signed=False, label="CURSORINFO_FLAGS"),
    "CURSORSHAPE": SimStruct(OrderedDict((("xHotSpot", SimTypeInt(signed=True, label="Int32")), ("yHotSpot", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")), ("cbWidth", SimTypeInt(signed=True, label="Int32")), ("Planes", SimTypeChar(label="Byte")), ("BitsPixel", SimTypeChar(label="Byte")),)), name="CURSORSHAPE", pack=False, align=None),
    "CUSTDATA": SimStruct(OrderedDict((("cCustData", SimTypeInt(signed=False, label="UInt32")), ("prgCustData", SimTypePointer(SimTypeRef("CUSTDATAITEM", SimStruct), offset=0)),)), name="CUSTDATA", pack=False, align=None),
    "CUSTDATAITEM": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("varValue", SimTypeRef("VARIANT", SimStruct)),)), name="CUSTDATAITEM", pack=False, align=None),
    "CUSTOMSIZEPARAM": SimStruct(OrderedDict((("dwOrder", SimTypeInt(signed=True, label="Int32")), ("lMinVal", SimTypeInt(signed=True, label="Int32")), ("lMaxVal", SimTypeInt(signed=True, label="Int32")),)), name="CUSTOMSIZEPARAM", pack=False, align=None),
    "CUSTOM_SYSTEM_EVENT_TRIGGER_CONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("TriggerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CUSTOM_SYSTEM_EVENT_TRIGGER_CONFIG", pack=False, align=None),
    "CVRC_COLUMN": SimTypeInt(signed=False, label="CVRC_COLUMN"),
    "CVRC_TABLE": SimTypeInt(signed=False, label="CVRC_TABLE"),
    "CWMO_FLAGS": SimTypeInt(signed=False, label="CWMO_FLAGS"),
    "CWPRETSTRUCT": SimStruct(OrderedDict((("lResult", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("message", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CWPRETSTRUCT", pack=False, align=None),
    "CWPSTRUCT": SimStruct(OrderedDict((("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("message", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="CWPSTRUCT", pack=False, align=None),
    "CWP_FLAGS": SimTypeInt(signed=False, label="CWP_FLAGS"),
    "CY": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"),
    "CYPHER_BLOCK": SimStruct(OrderedDict((("data", SimTypeArray(SimTypeChar(label="SByte"), 8)),)), name="CYPHER_BLOCK", pack=False, align=None),
    "CachedBitmap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CallbackThreadSetFnPtr": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CallbackThreadUnsetFnPtr": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "CallingConventionKind": SimTypeInt(signed=False, label="CallingConventionKind"),
    "CameraControlFlags": SimTypeInt(signed=False, label="CameraControlFlags"),
    "CameraControlProperty": SimTypeInt(signed=False, label="CameraControlProperty"),
    "CameraUIControlCaptureMode": SimTypeInt(signed=False, label="CameraUIControlCaptureMode"),
    "CameraUIControlLinearSelectionMode": SimTypeInt(signed=False, label="CameraUIControlLinearSelectionMode"),
    "CameraUIControlMode": SimTypeInt(signed=False, label="CameraUIControlMode"),
    "CameraUIControlPhotoFormat": SimTypeInt(signed=False, label="CameraUIControlPhotoFormat"),
    "CameraUIControlVideoFormat": SimTypeInt(signed=False, label="CameraUIControlVideoFormat"),
    "CameraUIControlViewType": SimTypeInt(signed=False, label="CameraUIControlViewType"),
    "CapStyle": SimTypeInt(signed=False, label="CapStyle"),
    "CapturedMetadataExposureCompensation": SimStruct(OrderedDict((("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Value", SimTypeInt(signed=True, label="Int32")),)), name="CapturedMetadataExposureCompensation", pack=False, align=None),
    "CapturedMetadataISOGains": SimStruct(OrderedDict((("AnalogGain", SimTypeFloat(size=32)), ("DigitalGain", SimTypeFloat(size=32)),)), name="CapturedMetadataISOGains", pack=False, align=None),
    "CapturedMetadataWhiteBalanceGains": SimStruct(OrderedDict((("R", SimTypeFloat(size=32)), ("G", SimTypeFloat(size=32)), ("B", SimTypeFloat(size=32)),)), name="CapturedMetadataWhiteBalanceGains", pack=False, align=None),
    "CaretBidiMode": SimTypeInt(signed=False, label="CaretBidiMode"),
    "CaretPosition": SimTypeInt(signed=False, label="CaretPosition"),
    "CatalogPausedReason": SimTypeInt(signed=False, label="CatalogPausedReason"),
    "CatalogStatus": SimTypeInt(signed=False, label="CatalogStatus"),
    "CertKeyType": SimTypeInt(signed=False, label="CertKeyType"),
    "ChannelChangeInfo": SimStruct(OrderedDict((("state", SimTypeInt(signed=False, label="ChannelChangeSpanningEvent_State")), ("TimeStamp", SimTypeLongLong(signed=False, label="UInt64")),)), name="ChannelChangeInfo", pack=False, align=None),
    "ChannelChangeSpanningEvent_State": SimTypeInt(signed=False, label="ChannelChangeSpanningEvent_State"),
    "ChannelInfo": SimStruct(OrderedDict((("lFrequency", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"DVB": SimTypeRef("_DVB_e__Struct", SimStruct), "DC": SimTypeRef("_DC_e__Struct", SimStruct), "ATSC": SimTypeRef("_ATSC_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="ChannelInfo", pack=False, align=None),
    "ChannelType": SimTypeInt(signed=False, label="ChannelType"),
    "ChannelTypeInfo": SimStruct(OrderedDict((("channelType", SimTypeInt(signed=False, label="ChannelType")), ("timeStamp", SimTypeLongLong(signed=False, label="UInt64")),)), name="ChannelTypeInfo", pack=False, align=None),
    "CharacterRange": SimStruct(OrderedDict((("First", SimTypeInt(signed=True, label="Int32")), ("Length", SimTypeInt(signed=True, label="Int32")),)), name="CharacterRange", pack=False, align=None),
    "ClockType": SimTypeInt(signed=False, label="ClockType"),
    "CoalesceEventsOptions": SimTypeInt(signed=False, label="CoalesceEventsOptions"),
    "CodecAPIEventData": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("dataLength", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="CodecAPIEventData", pack=False, align=None),
    "CollectionElementValue": SimStruct(OrderedDict((("Index", SimTypeInt(signed=False, label="UInt32")), ("ValueType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MetadataBits", SimTypeLongLong(signed=True, label="Int64")),)), name="CollectionElementValue", pack=False, align=None),
    "Color": SimStruct(OrderedDict((("Argb", SimTypeInt(signed=False, label="UInt32")),)), name="Color", pack=False, align=None),
    "ColorAdjustType": SimTypeInt(signed=False, label="ColorAdjustType"),
    "ColorBalance": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="ColorBalance", pack=False, align=None),
    "ColorBalanceParams": SimStruct(OrderedDict((("cyanRed", SimTypeInt(signed=True, label="Int32")), ("magentaGreen", SimTypeInt(signed=True, label="Int32")), ("yellowBlue", SimTypeInt(signed=True, label="Int32")),)), name="ColorBalanceParams", pack=False, align=None),
    "ColorChannelFlags": SimTypeInt(signed=False, label="ColorChannelFlags"),
    "ColorCurve": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="ColorCurve", pack=False, align=None),
    "ColorCurveParams": SimStruct(OrderedDict((("adjustment", SimTypeInt(signed=False, label="CurveAdjustments")), ("channel", SimTypeInt(signed=False, label="CurveChannel")), ("adjustValue", SimTypeInt(signed=True, label="Int32")),)), name="ColorCurveParams", pack=False, align=None),
    "ColorLUT": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="ColorLUT", pack=False, align=None),
    "ColorLUTParams": SimStruct(OrderedDict((("lutB", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("lutG", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("lutR", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("lutA", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="ColorLUTParams", pack=False, align=None),
    "ColorMap": SimStruct(OrderedDict((("oldColor", SimTypeRef("Color", SimStruct)), ("newColor", SimTypeRef("Color", SimStruct)),)), name="ColorMap", pack=False, align=None),
    "ColorMatrix": SimStruct(OrderedDict((("m", SimTypeArray(SimTypeFloat(size=32), 25)),)), name="ColorMatrix", pack=False, align=None),
    "ColorMatrixEffect": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="ColorMatrixEffect", pack=False, align=None),
    "ColorMatrixFlags": SimTypeInt(signed=False, label="ColorMatrixFlags"),
    "ColorMode": SimTypeInt(signed=False, label="ColorMode"),
    "ColorPalette": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="ColorPalette", pack=False, align=None),
    "Column": SimTypeBottom(label="Column"),
    "Columns": SimTypeBottom(label="Columns"),
    "ComCallData": SimStruct(OrderedDict((("dwDispid", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pUserDefined", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="ComCallData", pack=False, align=None),
    "CombineMode": SimTypeInt(signed=False, label="CombineMode"),
    "CommandStateChangeConstants": SimTypeInt(signed=False, label="CommandStateChangeConstants"),
    "CommitMode": SimTypeInt(signed=False, label="CommitMode"),
    "CommitTemplateFlags": SimTypeInt(signed=False, label="CommitTemplateFlags"),
    "ComponentCategory": SimTypeInt(signed=False, label="ComponentCategory"),
    "ComponentHangMonitorInfo": SimStruct(OrderedDict((("IsMonitored", SimTypeInt(signed=True, label="Int32")), ("TerminateOnHang", SimTypeInt(signed=True, label="Int32")), ("AvgCallThresholdInMs", SimTypeInt(signed=False, label="UInt32")),)), name="ComponentHangMonitorInfo", pack=False, align=None),
    "ComponentStatistics": SimStruct(OrderedDict((("NumInstances", SimTypeInt(signed=False, label="UInt32")), ("NumBoundReferences", SimTypeInt(signed=False, label="UInt32")), ("NumPooledObjects", SimTypeInt(signed=False, label="UInt32")), ("NumObjectsInCall", SimTypeInt(signed=False, label="UInt32")), ("AvgResponseTimeInMs", SimTypeInt(signed=False, label="UInt32")), ("NumCallsCompletedRecent", SimTypeInt(signed=False, label="UInt32")), ("NumCallsFailedRecent", SimTypeInt(signed=False, label="UInt32")), ("NumCallsCompletedTotal", SimTypeInt(signed=False, label="UInt32")), ("NumCallsFailedTotal", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("Reserved4", SimTypeInt(signed=False, label="UInt32")),)), name="ComponentStatistics", pack=False, align=None),
    "ComponentStatus": SimTypeInt(signed=False, label="ComponentStatus"),
    "ComponentSummary": SimStruct(OrderedDict((("ApplicationInstanceId", SimTypeBottom(label="Guid")), ("PartitionId", SimTypeBottom(label="Guid")), ("ApplicationId", SimTypeBottom(label="Guid")), ("Clsid", SimTypeBottom(label="Guid")), ("ClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ComponentSummary", pack=False, align=None),
    "CompositingMode": SimTypeInt(signed=False, label="CompositingMode"),
    "CompositingQuality": SimTypeInt(signed=False, label="CompositingQuality"),
    "CompositionFrameDisplayInstance": SimStruct(OrderedDict((("displayAdapterLUID", SimTypeRef("LUID", SimStruct)), ("displayVidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("displayUniqueId", SimTypeInt(signed=False, label="UInt32")), ("renderAdapterLUID", SimTypeRef("LUID", SimStruct)), ("instanceKind", SimTypeInt(signed=False, label="CompositionFrameInstanceKind")), ("finalTransform", SimTypeRef("PresentationTransform", SimStruct)), ("requiredCrossAdapterCopy", SimTypeChar(label="Byte")), ("colorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")),)), name="CompositionFrameDisplayInstance", pack=False, align=None),
    "CompositionFrameInstanceKind": SimTypeInt(signed=False, label="CompositionFrameInstanceKind"),
    "CompressionCaps": SimTypeInt(signed=False, label="CompressionCaps"),
    "ConditionType": SimTypeInt(signed=False, label="ConditionType"),
    "ConnectionRecoveryBehaviorOptions": SimTypeInt(signed=False, label="ConnectionRecoveryBehaviorOptions"),
    "ConnectorType": SimTypeInt(signed=False, label="ConnectorType"),
    "ContextInfo": SimTypeBottom(label="ContextInfo"),
    "ContextInfo2": SimTypeBottom(label="ContextInfo2"),
    "ContextMenu": SimTypeBottom(label="ContextMenu"),
    "ContextProperty": SimStruct(OrderedDict((("policyId", SimTypeBottom(label="Guid")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("pUnk", SimTypeBottom(label="IUnknown")),)), name="ContextProperty", pack=False, align=None),
    "ConvertToEmfPlusFlags": SimTypeInt(signed=False, label="ConvertToEmfPlusFlags"),
    "CookieDecision": SimStruct(OrderedDict((("dwCookieState", SimTypeInt(signed=False, label="UInt32")), ("fAllowSession", SimTypeInt(signed=True, label="Int32")),)), name="CookieDecision", pack=False, align=None),
    "CoordinateSpace": SimTypeInt(signed=False, label="CoordinateSpace"),
    "CorDebugIlToNativeMappingTypes": SimTypeInt(signed=False, label="CorDebugIlToNativeMappingTypes"),
    "CorrectionMode": SimTypeInt(signed=False, label="CorrectionMode"),
    "CorrectionPosition": SimTypeInt(signed=False, label="CorrectionPosition"),
    "CorrelationId": SimStruct(OrderedDict((("connId", SimTypeBottom(label="Guid")), ("timeStamp", SimTypeRef("FILETIME", SimStruct)),)), name="CorrelationId", pack=False, align=None),
    "CountedString": SimStruct(OrderedDict((("length", SimTypeShort(signed=False, label="UInt16")), ("string", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="CountedString", pack=False, align=None),
    "CounterPathCallBack": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "CreateInterfaceFnPtr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["clsid", "riid", "ppInterface"]), offset=0),
    "CreatePackageDependencyOptions": SimTypeInt(signed=False, label="CreatePackageDependencyOptions"),
    "CredFreeCredentialsFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), label="LPArray", offset=0)], SimTypeBottom(label="Void"), arg_names=["Count", "Credentials"]), offset=0),
    "CredReadDomainCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CREDENTIAL_TARGET_INFORMATIONW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "CredFlags", "TargetInfo", "Flags", "Count", "Credential"]), offset=0),
    "CredReadFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "CredFlags", "TargetName", "Type", "Flags", "Credential"]), offset=0),
    "CredWriteFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "CredFlags", "Credential", "Flags"]), offset=0),
    "CrediUnmarshalandDecodeStringFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MarshaledString", "Blob", "BlobSize", "IsFailureFatal"]), offset=0),
    "CrmLogRecordRead": SimStruct(OrderedDict((("dwCrmFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("blobUserData", SimTypeRef("BLOB", SimStruct)),)), name="CrmLogRecordRead", pack=False, align=None),
    "CrmTransactionState": SimTypeInt(signed=False, label="CrmTransactionState"),
    "CryptXmlDllCloseDigest": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDigest"]), offset=0),
    "CryptXmlDllCreateDigest": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDigestMethod", "pcbSize", "phDigest"]), offset=0),
    "CryptXmlDllCreateKey": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pEncoded", "phKey"]), offset=0),
    "CryptXmlDllDigestData": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDigest", "pbData", "cbData"]), offset=0),
    "CryptXmlDllEncodeAlgorithm": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="CRYPT_XML_CHARSET"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState", "pbData", "cbData"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAlgInfo", "dwCharset", "pvCallbackState", "pfnWrite"]), offset=0),
    "CryptXmlDllEncodeKeyValue": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="CRYPT_XML_CHARSET"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState", "pbData", "cbData"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "dwCharset", "pvCallbackState", "pfnWrite"]), offset=0),
    "CryptXmlDllFinalizeDigest": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDigest", "pbDigest", "cbDigest"]), offset=0),
    "CryptXmlDllGetAlgorithmInfo": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM_INFO", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pXmlAlgorithm", "ppAlgInfo"]), offset=0),
    "CryptXmlDllGetInterface": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_XML_CRYPTOGRAPHIC_INTERFACE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwFlags", "pMethod", "pInterface"]), offset=0),
    "CryptXmlDllSignData": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSignatureMethod", "hCryptProvOrNCryptKey", "dwKeySpec", "pbInput", "cbInput", "pbOutput", "cbOutput", "pcbResult"]), offset=0),
    "CryptXmlDllVerifySignature": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pSignatureMethod", "hKey", "pbInput", "cbInput", "pbSignature", "cbSignature"]), offset=0),
    "CtrlLoadFlowspec": SimStruct(OrderedDict((("CL_spec_serv_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("CL_spec_parm_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("CL_spec_parms", SimTypeRef("GenTspecParms", SimStruct)),)), name="CtrlLoadFlowspec", pack=False, align=None),
    "CurveAdjustments": SimTypeInt(signed=False, label="CurveAdjustments"),
    "CurveChannel": SimTypeInt(signed=False, label="CurveChannel"),
    "CustomDumpItem": SimStruct(OrderedDict((("itemKind", SimTypeInt(signed=False, label="ECustomDumpItemKind")), ("Anonymous", SimUnion({"pReserved": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")),)), name="CustomDumpItem", pack=False, align=None),
    "CustomLineCap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "CustomLineCapType": SimTypeInt(signed=False, label="CustomLineCapType"),
    "D2D1_2DAFFINETRANSFORM_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_2DAFFINETRANSFORM_INTERPOLATION_MODE"),
    "D2D1_2DAFFINETRANSFORM_PROP": SimTypeInt(signed=False, label="D2D1_2DAFFINETRANSFORM_PROP"),
    "D2D1_3DPERSPECTIVETRANSFORM_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_3DPERSPECTIVETRANSFORM_INTERPOLATION_MODE"),
    "D2D1_3DPERSPECTIVETRANSFORM_PROP": SimTypeInt(signed=False, label="D2D1_3DPERSPECTIVETRANSFORM_PROP"),
    "D2D1_3DTRANSFORM_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_3DTRANSFORM_INTERPOLATION_MODE"),
    "D2D1_3DTRANSFORM_PROP": SimTypeInt(signed=False, label="D2D1_3DTRANSFORM_PROP"),
    "D2D1_ALPHA_MODE": SimTypeInt(signed=False, label="D2D1_ALPHA_MODE"),
    "D2D1_ANTIALIAS_MODE": SimTypeInt(signed=False, label="D2D1_ANTIALIAS_MODE"),
    "D2D1_ARC_SEGMENT": SimStruct(OrderedDict((("point", SimTypeRef("D2D_POINT_2F", SimStruct)), ("size", SimTypeRef("D2D_SIZE_F", SimStruct)), ("rotationAngle", SimTypeFloat(size=32)), ("sweepDirection", SimTypeInt(signed=False, label="D2D1_SWEEP_DIRECTION")), ("arcSize", SimTypeInt(signed=False, label="D2D1_ARC_SIZE")),)), name="D2D1_ARC_SEGMENT", pack=False, align=None),
    "D2D1_ARC_SIZE": SimTypeInt(signed=False, label="D2D1_ARC_SIZE"),
    "D2D1_ARITHMETICCOMPOSITE_PROP": SimTypeInt(signed=False, label="D2D1_ARITHMETICCOMPOSITE_PROP"),
    "D2D1_ATLAS_PROP": SimTypeInt(signed=False, label="D2D1_ATLAS_PROP"),
    "D2D1_BEZIER_SEGMENT": SimStruct(OrderedDict((("point1", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point2", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point3", SimTypeRef("D2D_POINT_2F", SimStruct)),)), name="D2D1_BEZIER_SEGMENT", pack=False, align=None),
    "D2D1_BITMAPSOURCE_ALPHA_MODE": SimTypeInt(signed=False, label="D2D1_BITMAPSOURCE_ALPHA_MODE"),
    "D2D1_BITMAPSOURCE_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_BITMAPSOURCE_INTERPOLATION_MODE"),
    "D2D1_BITMAPSOURCE_ORIENTATION": SimTypeInt(signed=False, label="D2D1_BITMAPSOURCE_ORIENTATION"),
    "D2D1_BITMAPSOURCE_PROP": SimTypeInt(signed=False, label="D2D1_BITMAPSOURCE_PROP"),
    "D2D1_BITMAP_BRUSH_PROPERTIES": SimStruct(OrderedDict((("extendModeX", SimTypeInt(signed=False, label="D2D1_EXTEND_MODE")), ("extendModeY", SimTypeInt(signed=False, label="D2D1_EXTEND_MODE")), ("interpolationMode", SimTypeInt(signed=False, label="D2D1_BITMAP_INTERPOLATION_MODE")),)), name="D2D1_BITMAP_BRUSH_PROPERTIES", pack=False, align=None),
    "D2D1_BITMAP_BRUSH_PROPERTIES1": SimStruct(OrderedDict((("extendModeX", SimTypeInt(signed=False, label="D2D1_EXTEND_MODE")), ("extendModeY", SimTypeInt(signed=False, label="D2D1_EXTEND_MODE")), ("interpolationMode", SimTypeInt(signed=False, label="D2D1_INTERPOLATION_MODE")),)), name="D2D1_BITMAP_BRUSH_PROPERTIES1", pack=False, align=None),
    "D2D1_BITMAP_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_BITMAP_INTERPOLATION_MODE"),
    "D2D1_BITMAP_OPTIONS": SimTypeInt(signed=False, label="D2D1_BITMAP_OPTIONS"),
    "D2D1_BITMAP_PROPERTIES": SimStruct(OrderedDict((("pixelFormat", SimTypeRef("D2D1_PIXEL_FORMAT", SimStruct)), ("dpiX", SimTypeFloat(size=32)), ("dpiY", SimTypeFloat(size=32)),)), name="D2D1_BITMAP_PROPERTIES", pack=False, align=None),
    "D2D1_BITMAP_PROPERTIES1": SimStruct(OrderedDict((("pixelFormat", SimTypeRef("D2D1_PIXEL_FORMAT", SimStruct)), ("dpiX", SimTypeFloat(size=32)), ("dpiY", SimTypeFloat(size=32)), ("bitmapOptions", SimTypeInt(signed=False, label="D2D1_BITMAP_OPTIONS")), ("colorContext", SimTypeBottom(label="ID2D1ColorContext")),)), name="D2D1_BITMAP_PROPERTIES1", pack=False, align=None),
    "D2D1_BLEND": SimTypeInt(signed=False, label="D2D1_BLEND"),
    "D2D1_BLEND_DESCRIPTION": SimStruct(OrderedDict((("sourceBlend", SimTypeInt(signed=False, label="D2D1_BLEND")), ("destinationBlend", SimTypeInt(signed=False, label="D2D1_BLEND")), ("blendOperation", SimTypeInt(signed=False, label="D2D1_BLEND_OPERATION")), ("sourceBlendAlpha", SimTypeInt(signed=False, label="D2D1_BLEND")), ("destinationBlendAlpha", SimTypeInt(signed=False, label="D2D1_BLEND")), ("blendOperationAlpha", SimTypeInt(signed=False, label="D2D1_BLEND_OPERATION")), ("blendFactor", SimTypeArray(SimTypeFloat(size=32), 4)),)), name="D2D1_BLEND_DESCRIPTION", pack=False, align=None),
    "D2D1_BLEND_MODE": SimTypeInt(signed=False, label="D2D1_BLEND_MODE"),
    "D2D1_BLEND_OPERATION": SimTypeInt(signed=False, label="D2D1_BLEND_OPERATION"),
    "D2D1_BLEND_PROP": SimTypeInt(signed=False, label="D2D1_BLEND_PROP"),
    "D2D1_BORDER_EDGE_MODE": SimTypeInt(signed=False, label="D2D1_BORDER_EDGE_MODE"),
    "D2D1_BORDER_MODE": SimTypeInt(signed=False, label="D2D1_BORDER_MODE"),
    "D2D1_BORDER_PROP": SimTypeInt(signed=False, label="D2D1_BORDER_PROP"),
    "D2D1_BRIGHTNESS_PROP": SimTypeInt(signed=False, label="D2D1_BRIGHTNESS_PROP"),
    "D2D1_BRUSH_PROPERTIES": SimStruct(OrderedDict((("opacity", SimTypeFloat(size=32)), ("transform", SimTypeRef("D2D_MATRIX_3X2_F", SimStruct)),)), name="D2D1_BRUSH_PROPERTIES", pack=False, align=None),
    "D2D1_BUFFER_PRECISION": SimTypeInt(signed=False, label="D2D1_BUFFER_PRECISION"),
    "D2D1_CAP_STYLE": SimTypeInt(signed=False, label="D2D1_CAP_STYLE"),
    "D2D1_CHANGE_TYPE": SimTypeInt(signed=False, label="D2D1_CHANGE_TYPE"),
    "D2D1_CHANNEL_DEPTH": SimTypeInt(signed=False, label="D2D1_CHANNEL_DEPTH"),
    "D2D1_CHANNEL_SELECTOR": SimTypeInt(signed=False, label="D2D1_CHANNEL_SELECTOR"),
    "D2D1_CHROMAKEY_PROP": SimTypeInt(signed=False, label="D2D1_CHROMAKEY_PROP"),
    "D2D1_COLORMANAGEMENT_ALPHA_MODE": SimTypeInt(signed=False, label="D2D1_COLORMANAGEMENT_ALPHA_MODE"),
    "D2D1_COLORMANAGEMENT_PROP": SimTypeInt(signed=False, label="D2D1_COLORMANAGEMENT_PROP"),
    "D2D1_COLORMANAGEMENT_QUALITY": SimTypeInt(signed=False, label="D2D1_COLORMANAGEMENT_QUALITY"),
    "D2D1_COLORMANAGEMENT_RENDERING_INTENT": SimTypeInt(signed=False, label="D2D1_COLORMANAGEMENT_RENDERING_INTENT"),
    "D2D1_COLORMATRIX_ALPHA_MODE": SimTypeInt(signed=False, label="D2D1_COLORMATRIX_ALPHA_MODE"),
    "D2D1_COLORMATRIX_PROP": SimTypeInt(signed=False, label="D2D1_COLORMATRIX_PROP"),
    "D2D1_COLOR_BITMAP_GLYPH_SNAP_OPTION": SimTypeInt(signed=False, label="D2D1_COLOR_BITMAP_GLYPH_SNAP_OPTION"),
    "D2D1_COLOR_CONTEXT_TYPE": SimTypeInt(signed=False, label="D2D1_COLOR_CONTEXT_TYPE"),
    "D2D1_COLOR_F": SimStruct(OrderedDict((("r", SimTypeFloat(size=32)), ("g", SimTypeFloat(size=32)), ("b", SimTypeFloat(size=32)), ("a", SimTypeFloat(size=32)),)), name="D2D1_COLOR_F", pack=False, align=None),
    "D2D1_COLOR_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_COLOR_INTERPOLATION_MODE"),
    "D2D1_COLOR_SPACE": SimTypeInt(signed=False, label="D2D1_COLOR_SPACE"),
    "D2D1_COMBINE_MODE": SimTypeInt(signed=False, label="D2D1_COMBINE_MODE"),
    "D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS": SimTypeInt(signed=False, label="D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS"),
    "D2D1_COMPOSITE_MODE": SimTypeInt(signed=False, label="D2D1_COMPOSITE_MODE"),
    "D2D1_COMPOSITE_PROP": SimTypeInt(signed=False, label="D2D1_COMPOSITE_PROP"),
    "D2D1_CONTRAST_PROP": SimTypeInt(signed=False, label="D2D1_CONTRAST_PROP"),
    "D2D1_CONVOLVEMATRIX_PROP": SimTypeInt(signed=False, label="D2D1_CONVOLVEMATRIX_PROP"),
    "D2D1_CONVOLVEMATRIX_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_CONVOLVEMATRIX_SCALE_MODE"),
    "D2D1_CREATION_PROPERTIES": SimStruct(OrderedDict((("threadingMode", SimTypeInt(signed=False, label="D2D1_THREADING_MODE")), ("debugLevel", SimTypeInt(signed=False, label="D2D1_DEBUG_LEVEL")), ("options", SimTypeInt(signed=False, label="D2D1_DEVICE_CONTEXT_OPTIONS")),)), name="D2D1_CREATION_PROPERTIES", pack=False, align=None),
    "D2D1_CROP_PROP": SimTypeInt(signed=False, label="D2D1_CROP_PROP"),
    "D2D1_CROSSFADE_PROP": SimTypeInt(signed=False, label="D2D1_CROSSFADE_PROP"),
    "D2D1_CUSTOM_VERTEX_BUFFER_PROPERTIES": SimStruct(OrderedDict((("shaderBufferWithInputSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("shaderBufferSize", SimTypeInt(signed=False, label="UInt32")), ("inputElements", SimTypePointer(SimTypeRef("D2D1_INPUT_ELEMENT_DESC", SimStruct), offset=0)), ("elementCount", SimTypeInt(signed=False, label="UInt32")), ("stride", SimTypeInt(signed=False, label="UInt32")),)), name="D2D1_CUSTOM_VERTEX_BUFFER_PROPERTIES", pack=False, align=None),
    "D2D1_DASH_STYLE": SimTypeInt(signed=False, label="D2D1_DASH_STYLE"),
    "D2D1_DC_INITIALIZE_MODE": SimTypeInt(signed=False, label="D2D1_DC_INITIALIZE_MODE"),
    "D2D1_DEBUG_LEVEL": SimTypeInt(signed=False, label="D2D1_DEBUG_LEVEL"),
    "D2D1_DEVICE_CONTEXT_OPTIONS": SimTypeInt(signed=False, label="D2D1_DEVICE_CONTEXT_OPTIONS"),
    "D2D1_DIRECTIONALBLUR_OPTIMIZATION": SimTypeInt(signed=False, label="D2D1_DIRECTIONALBLUR_OPTIMIZATION"),
    "D2D1_DIRECTIONALBLUR_PROP": SimTypeInt(signed=False, label="D2D1_DIRECTIONALBLUR_PROP"),
    "D2D1_DISCRETETRANSFER_PROP": SimTypeInt(signed=False, label="D2D1_DISCRETETRANSFER_PROP"),
    "D2D1_DISPLACEMENTMAP_PROP": SimTypeInt(signed=False, label="D2D1_DISPLACEMENTMAP_PROP"),
    "D2D1_DISTANTDIFFUSE_PROP": SimTypeInt(signed=False, label="D2D1_DISTANTDIFFUSE_PROP"),
    "D2D1_DISTANTDIFFUSE_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_DISTANTDIFFUSE_SCALE_MODE"),
    "D2D1_DISTANTSPECULAR_PROP": SimTypeInt(signed=False, label="D2D1_DISTANTSPECULAR_PROP"),
    "D2D1_DISTANTSPECULAR_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_DISTANTSPECULAR_SCALE_MODE"),
    "D2D1_DPICOMPENSATION_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_DPICOMPENSATION_INTERPOLATION_MODE"),
    "D2D1_DPICOMPENSATION_PROP": SimTypeInt(signed=False, label="D2D1_DPICOMPENSATION_PROP"),
    "D2D1_DRAWING_STATE_DESCRIPTION": SimStruct(OrderedDict((("antialiasMode", SimTypeInt(signed=False, label="D2D1_ANTIALIAS_MODE")), ("textAntialiasMode", SimTypeInt(signed=False, label="D2D1_TEXT_ANTIALIAS_MODE")), ("tag1", SimTypeLongLong(signed=False, label="UInt64")), ("tag2", SimTypeLongLong(signed=False, label="UInt64")), ("transform", SimTypeRef("D2D_MATRIX_3X2_F", SimStruct)),)), name="D2D1_DRAWING_STATE_DESCRIPTION", pack=False, align=None),
    "D2D1_DRAWING_STATE_DESCRIPTION1": SimStruct(OrderedDict((("antialiasMode", SimTypeInt(signed=False, label="D2D1_ANTIALIAS_MODE")), ("textAntialiasMode", SimTypeInt(signed=False, label="D2D1_TEXT_ANTIALIAS_MODE")), ("tag1", SimTypeLongLong(signed=False, label="UInt64")), ("tag2", SimTypeLongLong(signed=False, label="UInt64")), ("transform", SimTypeRef("D2D_MATRIX_3X2_F", SimStruct)), ("primitiveBlend", SimTypeInt(signed=False, label="D2D1_PRIMITIVE_BLEND")), ("unitMode", SimTypeInt(signed=False, label="D2D1_UNIT_MODE")),)), name="D2D1_DRAWING_STATE_DESCRIPTION1", pack=False, align=None),
    "D2D1_DRAW_TEXT_OPTIONS": SimTypeInt(signed=False, label="D2D1_DRAW_TEXT_OPTIONS"),
    "D2D1_EDGEDETECTION_MODE": SimTypeInt(signed=False, label="D2D1_EDGEDETECTION_MODE"),
    "D2D1_EDGEDETECTION_PROP": SimTypeInt(signed=False, label="D2D1_EDGEDETECTION_PROP"),
    "D2D1_EFFECT_INPUT_DESCRIPTION": SimStruct(OrderedDict((("effect", SimTypeBottom(label="ID2D1Effect")), ("inputIndex", SimTypeInt(signed=False, label="UInt32")), ("inputRectangle", SimTypeRef("D2D_RECT_F", SimStruct)),)), name="D2D1_EFFECT_INPUT_DESCRIPTION", pack=False, align=None),
    "D2D1_ELLIPSE": SimStruct(OrderedDict((("point", SimTypeRef("D2D_POINT_2F", SimStruct)), ("radiusX", SimTypeFloat(size=32)), ("radiusY", SimTypeFloat(size=32)),)), name="D2D1_ELLIPSE", pack=False, align=None),
    "D2D1_EMBOSS_PROP": SimTypeInt(signed=False, label="D2D1_EMBOSS_PROP"),
    "D2D1_EXPOSURE_PROP": SimTypeInt(signed=False, label="D2D1_EXPOSURE_PROP"),
    "D2D1_EXTEND_MODE": SimTypeInt(signed=False, label="D2D1_EXTEND_MODE"),
    "D2D1_FACTORY_OPTIONS": SimStruct(OrderedDict((("debugLevel", SimTypeInt(signed=False, label="D2D1_DEBUG_LEVEL")),)), name="D2D1_FACTORY_OPTIONS", pack=False, align=None),
    "D2D1_FACTORY_TYPE": SimTypeInt(signed=False, label="D2D1_FACTORY_TYPE"),
    "D2D1_FEATURE": SimTypeInt(signed=False, label="D2D1_FEATURE"),
    "D2D1_FEATURE_DATA_D3D10_X_HARDWARE_OPTIONS": SimStruct(OrderedDict((("computeShaders_Plus_RawAndStructuredBuffers_Via_Shader_4_x", SimTypeInt(signed=True, label="Int32")),)), name="D2D1_FEATURE_DATA_D3D10_X_HARDWARE_OPTIONS", pack=False, align=None),
    "D2D1_FEATURE_DATA_DOUBLES": SimStruct(OrderedDict((("doublePrecisionFloatShaderOps", SimTypeInt(signed=True, label="Int32")),)), name="D2D1_FEATURE_DATA_DOUBLES", pack=False, align=None),
    "D2D1_FEATURE_LEVEL": SimTypeInt(signed=False, label="D2D1_FEATURE_LEVEL"),
    "D2D1_FIGURE_BEGIN": SimTypeInt(signed=False, label="D2D1_FIGURE_BEGIN"),
    "D2D1_FIGURE_END": SimTypeInt(signed=False, label="D2D1_FIGURE_END"),
    "D2D1_FILL_MODE": SimTypeInt(signed=False, label="D2D1_FILL_MODE"),
    "D2D1_FILTER": SimTypeInt(signed=False, label="D2D1_FILTER"),
    "D2D1_FLOOD_PROP": SimTypeInt(signed=False, label="D2D1_FLOOD_PROP"),
    "D2D1_GAMMA": SimTypeInt(signed=False, label="D2D1_GAMMA"),
    "D2D1_GAMMA1": SimTypeInt(signed=False, label="D2D1_GAMMA1"),
    "D2D1_GAMMATRANSFER_PROP": SimTypeInt(signed=False, label="D2D1_GAMMATRANSFER_PROP"),
    "D2D1_GAUSSIANBLUR_OPTIMIZATION": SimTypeInt(signed=False, label="D2D1_GAUSSIANBLUR_OPTIMIZATION"),
    "D2D1_GAUSSIANBLUR_PROP": SimTypeInt(signed=False, label="D2D1_GAUSSIANBLUR_PROP"),
    "D2D1_GEOMETRY_RELATION": SimTypeInt(signed=False, label="D2D1_GEOMETRY_RELATION"),
    "D2D1_GEOMETRY_SIMPLIFICATION_OPTION": SimTypeInt(signed=False, label="D2D1_GEOMETRY_SIMPLIFICATION_OPTION"),
    "D2D1_GRADIENT_MESH_PATCH": SimStruct(OrderedDict((("point00", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point01", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point02", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point03", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point10", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point11", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point12", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point13", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point20", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point21", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point22", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point23", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point30", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point31", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point32", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point33", SimTypeRef("D2D_POINT_2F", SimStruct)), ("color00", SimTypeRef("D2D1_COLOR_F", SimStruct)), ("color03", SimTypeRef("D2D1_COLOR_F", SimStruct)), ("color30", SimTypeRef("D2D1_COLOR_F", SimStruct)), ("color33", SimTypeRef("D2D1_COLOR_F", SimStruct)), ("topEdgeMode", SimTypeInt(signed=False, label="D2D1_PATCH_EDGE_MODE")), ("leftEdgeMode", SimTypeInt(signed=False, label="D2D1_PATCH_EDGE_MODE")), ("bottomEdgeMode", SimTypeInt(signed=False, label="D2D1_PATCH_EDGE_MODE")), ("rightEdgeMode", SimTypeInt(signed=False, label="D2D1_PATCH_EDGE_MODE")),)), name="D2D1_GRADIENT_MESH_PATCH", pack=False, align=None),
    "D2D1_GRADIENT_STOP": SimStruct(OrderedDict((("position", SimTypeFloat(size=32)), ("color", SimTypeRef("D2D1_COLOR_F", SimStruct)),)), name="D2D1_GRADIENT_STOP", pack=False, align=None),
    "D2D1_HDRTONEMAP_DISPLAY_MODE": SimTypeInt(signed=False, label="D2D1_HDRTONEMAP_DISPLAY_MODE"),
    "D2D1_HDRTONEMAP_PROP": SimTypeInt(signed=False, label="D2D1_HDRTONEMAP_PROP"),
    "D2D1_HIGHLIGHTSANDSHADOWS_INPUT_GAMMA": SimTypeInt(signed=False, label="D2D1_HIGHLIGHTSANDSHADOWS_INPUT_GAMMA"),
    "D2D1_HIGHLIGHTSANDSHADOWS_PROP": SimTypeInt(signed=False, label="D2D1_HIGHLIGHTSANDSHADOWS_PROP"),
    "D2D1_HISTOGRAM_PROP": SimTypeInt(signed=False, label="D2D1_HISTOGRAM_PROP"),
    "D2D1_HUEROTATION_PROP": SimTypeInt(signed=False, label="D2D1_HUEROTATION_PROP"),
    "D2D1_HUETORGB_INPUT_COLOR_SPACE": SimTypeInt(signed=False, label="D2D1_HUETORGB_INPUT_COLOR_SPACE"),
    "D2D1_HUETORGB_PROP": SimTypeInt(signed=False, label="D2D1_HUETORGB_PROP"),
    "D2D1_HWND_RENDER_TARGET_PROPERTIES": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pixelSize", SimTypeRef("D2D_SIZE_U", SimStruct)), ("presentOptions", SimTypeInt(signed=False, label="D2D1_PRESENT_OPTIONS")),)), name="D2D1_HWND_RENDER_TARGET_PROPERTIES", pack=False, align=None),
    "D2D1_IMAGE_BRUSH_PROPERTIES": SimStruct(OrderedDict((("sourceRectangle", SimTypeRef("D2D_RECT_F", SimStruct)), ("extendModeX", SimTypeInt(signed=False, label="D2D1_EXTEND_MODE")), ("extendModeY", SimTypeInt(signed=False, label="D2D1_EXTEND_MODE")), ("interpolationMode", SimTypeInt(signed=False, label="D2D1_INTERPOLATION_MODE")),)), name="D2D1_IMAGE_BRUSH_PROPERTIES", pack=False, align=None),
    "D2D1_IMAGE_SOURCE_FROM_DXGI_OPTIONS": SimTypeInt(signed=False, label="D2D1_IMAGE_SOURCE_FROM_DXGI_OPTIONS"),
    "D2D1_IMAGE_SOURCE_LOADING_OPTIONS": SimTypeInt(signed=False, label="D2D1_IMAGE_SOURCE_LOADING_OPTIONS"),
    "D2D1_INK_BEZIER_SEGMENT": SimStruct(OrderedDict((("point1", SimTypeRef("D2D1_INK_POINT", SimStruct)), ("point2", SimTypeRef("D2D1_INK_POINT", SimStruct)), ("point3", SimTypeRef("D2D1_INK_POINT", SimStruct)),)), name="D2D1_INK_BEZIER_SEGMENT", pack=False, align=None),
    "D2D1_INK_NIB_SHAPE": SimTypeInt(signed=False, label="D2D1_INK_NIB_SHAPE"),
    "D2D1_INK_POINT": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("radius", SimTypeFloat(size=32)),)), name="D2D1_INK_POINT", pack=False, align=None),
    "D2D1_INK_STYLE_PROPERTIES": SimStruct(OrderedDict((("nibShape", SimTypeInt(signed=False, label="D2D1_INK_NIB_SHAPE")), ("nibTransform", SimTypeRef("D2D_MATRIX_3X2_F", SimStruct)),)), name="D2D1_INK_STYLE_PROPERTIES", pack=False, align=None),
    "D2D1_INPUT_DESCRIPTION": SimStruct(OrderedDict((("filter", SimTypeInt(signed=False, label="D2D1_FILTER")), ("levelOfDetailCount", SimTypeInt(signed=False, label="UInt32")),)), name="D2D1_INPUT_DESCRIPTION", pack=False, align=None),
    "D2D1_INPUT_ELEMENT_DESC": SimStruct(OrderedDict((("semanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("semanticIndex", SimTypeInt(signed=False, label="UInt32")), ("format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("inputSlot", SimTypeInt(signed=False, label="UInt32")), ("alignedByteOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D2D1_INPUT_ELEMENT_DESC", pack=False, align=None),
    "D2D1_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_INTERPOLATION_MODE"),
    "D2D1_INTERPOLATION_MODE_DEFINITION": SimTypeInt(signed=False, label="D2D1_INTERPOLATION_MODE_DEFINITION"),
    "D2D1_LAYER_OPTIONS": SimTypeInt(signed=False, label="D2D1_LAYER_OPTIONS"),
    "D2D1_LAYER_OPTIONS1": SimTypeInt(signed=False, label="D2D1_LAYER_OPTIONS1"),
    "D2D1_LAYER_PARAMETERS": SimStruct(OrderedDict((("contentBounds", SimTypeRef("D2D_RECT_F", SimStruct)), ("geometricMask", SimTypeBottom(label="ID2D1Geometry")), ("maskAntialiasMode", SimTypeInt(signed=False, label="D2D1_ANTIALIAS_MODE")), ("maskTransform", SimTypeRef("D2D_MATRIX_3X2_F", SimStruct)), ("opacity", SimTypeFloat(size=32)), ("opacityBrush", SimTypeBottom(label="ID2D1Brush")), ("layerOptions", SimTypeInt(signed=False, label="D2D1_LAYER_OPTIONS")),)), name="D2D1_LAYER_PARAMETERS", pack=False, align=None),
    "D2D1_LAYER_PARAMETERS1": SimStruct(OrderedDict((("contentBounds", SimTypeRef("D2D_RECT_F", SimStruct)), ("geometricMask", SimTypeBottom(label="ID2D1Geometry")), ("maskAntialiasMode", SimTypeInt(signed=False, label="D2D1_ANTIALIAS_MODE")), ("maskTransform", SimTypeRef("D2D_MATRIX_3X2_F", SimStruct)), ("opacity", SimTypeFloat(size=32)), ("opacityBrush", SimTypeBottom(label="ID2D1Brush")), ("layerOptions", SimTypeInt(signed=False, label="D2D1_LAYER_OPTIONS1")),)), name="D2D1_LAYER_PARAMETERS1", pack=False, align=None),
    "D2D1_LINEARTRANSFER_PROP": SimTypeInt(signed=False, label="D2D1_LINEARTRANSFER_PROP"),
    "D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES": SimStruct(OrderedDict((("startPoint", SimTypeRef("D2D_POINT_2F", SimStruct)), ("endPoint", SimTypeRef("D2D_POINT_2F", SimStruct)),)), name="D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES", pack=False, align=None),
    "D2D1_LINE_JOIN": SimTypeInt(signed=False, label="D2D1_LINE_JOIN"),
    "D2D1_LOOKUPTABLE3D_PROP": SimTypeInt(signed=False, label="D2D1_LOOKUPTABLE3D_PROP"),
    "D2D1_MAPPED_RECT": SimStruct(OrderedDict((("pitch", SimTypeInt(signed=False, label="UInt32")), ("bits", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D2D1_MAPPED_RECT", pack=False, align=None),
    "D2D1_MAP_OPTIONS": SimTypeInt(signed=False, label="D2D1_MAP_OPTIONS"),
    "D2D1_MORPHOLOGY_MODE": SimTypeInt(signed=False, label="D2D1_MORPHOLOGY_MODE"),
    "D2D1_MORPHOLOGY_PROP": SimTypeInt(signed=False, label="D2D1_MORPHOLOGY_PROP"),
    "D2D1_OPACITYMETADATA_PROP": SimTypeInt(signed=False, label="D2D1_OPACITYMETADATA_PROP"),
    "D2D1_OPACITY_MASK_CONTENT": SimTypeInt(signed=False, label="D2D1_OPACITY_MASK_CONTENT"),
    "D2D1_OPACITY_PROP": SimTypeInt(signed=False, label="D2D1_OPACITY_PROP"),
    "D2D1_ORIENTATION": SimTypeInt(signed=False, label="D2D1_ORIENTATION"),
    "D2D1_PATCH_EDGE_MODE": SimTypeInt(signed=False, label="D2D1_PATCH_EDGE_MODE"),
    "D2D1_PATH_SEGMENT": SimTypeInt(signed=False, label="D2D1_PATH_SEGMENT"),
    "D2D1_PIXEL_FORMAT": SimStruct(OrderedDict((("format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("alphaMode", SimTypeInt(signed=False, label="D2D1_ALPHA_MODE")),)), name="D2D1_PIXEL_FORMAT", pack=False, align=None),
    "D2D1_PIXEL_OPTIONS": SimTypeInt(signed=False, label="D2D1_PIXEL_OPTIONS"),
    "D2D1_POINTDIFFUSE_PROP": SimTypeInt(signed=False, label="D2D1_POINTDIFFUSE_PROP"),
    "D2D1_POINTDIFFUSE_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_POINTDIFFUSE_SCALE_MODE"),
    "D2D1_POINTSPECULAR_PROP": SimTypeInt(signed=False, label="D2D1_POINTSPECULAR_PROP"),
    "D2D1_POINTSPECULAR_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_POINTSPECULAR_SCALE_MODE"),
    "D2D1_POINT_DESCRIPTION": SimStruct(OrderedDict((("point", SimTypeRef("D2D_POINT_2F", SimStruct)), ("unitTangentVector", SimTypeRef("D2D_POINT_2F", SimStruct)), ("endSegment", SimTypeInt(signed=False, label="UInt32")), ("endFigure", SimTypeInt(signed=False, label="UInt32")), ("lengthToEndSegment", SimTypeFloat(size=32)),)), name="D2D1_POINT_DESCRIPTION", pack=False, align=None),
    "D2D1_POSTERIZE_PROP": SimTypeInt(signed=False, label="D2D1_POSTERIZE_PROP"),
    "D2D1_PRESENT_OPTIONS": SimTypeInt(signed=False, label="D2D1_PRESENT_OPTIONS"),
    "D2D1_PRIMITIVE_BLEND": SimTypeInt(signed=False, label="D2D1_PRIMITIVE_BLEND"),
    "D2D1_PRINT_CONTROL_PROPERTIES": SimStruct(OrderedDict((("fontSubset", SimTypeInt(signed=False, label="D2D1_PRINT_FONT_SUBSET_MODE")), ("rasterDPI", SimTypeFloat(size=32)), ("colorSpace", SimTypeInt(signed=False, label="D2D1_COLOR_SPACE")),)), name="D2D1_PRINT_CONTROL_PROPERTIES", pack=False, align=None),
    "D2D1_PRINT_FONT_SUBSET_MODE": SimTypeInt(signed=False, label="D2D1_PRINT_FONT_SUBSET_MODE"),
    "D2D1_PROPERTY": SimTypeInt(signed=False, label="D2D1_PROPERTY"),
    "D2D1_PROPERTY_BINDING": SimStruct(OrderedDict((("propertyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("setFunction", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["effect", "data", "dataSize"]), offset=0)), ("getFunction", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["effect", "data", "dataSize", "actualSize"]), offset=0)),)), name="D2D1_PROPERTY_BINDING", pack=False, align=None),
    "D2D1_PROPERTY_TYPE": SimTypeInt(signed=False, label="D2D1_PROPERTY_TYPE"),
    "D2D1_QUADRATIC_BEZIER_SEGMENT": SimStruct(OrderedDict((("point1", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point2", SimTypeRef("D2D_POINT_2F", SimStruct)),)), name="D2D1_QUADRATIC_BEZIER_SEGMENT", pack=False, align=None),
    "D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES": SimStruct(OrderedDict((("center", SimTypeRef("D2D_POINT_2F", SimStruct)), ("gradientOriginOffset", SimTypeRef("D2D_POINT_2F", SimStruct)), ("radiusX", SimTypeFloat(size=32)), ("radiusY", SimTypeFloat(size=32)),)), name="D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES", pack=False, align=None),
    "D2D1_RENDERING_CONTROLS": SimStruct(OrderedDict((("bufferPrecision", SimTypeInt(signed=False, label="D2D1_BUFFER_PRECISION")), ("tileSize", SimTypeRef("D2D_SIZE_U", SimStruct)),)), name="D2D1_RENDERING_CONTROLS", pack=False, align=None),
    "D2D1_RENDERING_PRIORITY": SimTypeInt(signed=False, label="D2D1_RENDERING_PRIORITY"),
    "D2D1_RENDER_TARGET_PROPERTIES": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="D2D1_RENDER_TARGET_TYPE")), ("pixelFormat", SimTypeRef("D2D1_PIXEL_FORMAT", SimStruct)), ("dpiX", SimTypeFloat(size=32)), ("dpiY", SimTypeFloat(size=32)), ("usage", SimTypeInt(signed=False, label="D2D1_RENDER_TARGET_USAGE")), ("minLevel", SimTypeInt(signed=False, label="D2D1_FEATURE_LEVEL")),)), name="D2D1_RENDER_TARGET_PROPERTIES", pack=False, align=None),
    "D2D1_RENDER_TARGET_TYPE": SimTypeInt(signed=False, label="D2D1_RENDER_TARGET_TYPE"),
    "D2D1_RENDER_TARGET_USAGE": SimTypeInt(signed=False, label="D2D1_RENDER_TARGET_USAGE"),
    "D2D1_RESOURCE_TEXTURE_PROPERTIES": SimStruct(OrderedDict((("extents", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("dimensions", SimTypeInt(signed=False, label="UInt32")), ("bufferPrecision", SimTypeInt(signed=False, label="D2D1_BUFFER_PRECISION")), ("channelDepth", SimTypeInt(signed=False, label="D2D1_CHANNEL_DEPTH")), ("filter", SimTypeInt(signed=False, label="D2D1_FILTER")), ("extendModes", SimTypePointer(SimTypeInt(signed=False, label="D2D1_EXTEND_MODE"), offset=0)),)), name="D2D1_RESOURCE_TEXTURE_PROPERTIES", pack=False, align=None),
    "D2D1_RGBTOHUE_OUTPUT_COLOR_SPACE": SimTypeInt(signed=False, label="D2D1_RGBTOHUE_OUTPUT_COLOR_SPACE"),
    "D2D1_RGBTOHUE_PROP": SimTypeInt(signed=False, label="D2D1_RGBTOHUE_PROP"),
    "D2D1_ROUNDED_RECT": SimStruct(OrderedDict((("rect", SimTypeRef("D2D_RECT_F", SimStruct)), ("radiusX", SimTypeFloat(size=32)), ("radiusY", SimTypeFloat(size=32)),)), name="D2D1_ROUNDED_RECT", pack=False, align=None),
    "D2D1_SATURATION_PROP": SimTypeInt(signed=False, label="D2D1_SATURATION_PROP"),
    "D2D1_SCALE_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_SCALE_INTERPOLATION_MODE"),
    "D2D1_SCALE_PROP": SimTypeInt(signed=False, label="D2D1_SCALE_PROP"),
    "D2D1_SEPIA_PROP": SimTypeInt(signed=False, label="D2D1_SEPIA_PROP"),
    "D2D1_SHADOW_OPTIMIZATION": SimTypeInt(signed=False, label="D2D1_SHADOW_OPTIMIZATION"),
    "D2D1_SHADOW_PROP": SimTypeInt(signed=False, label="D2D1_SHADOW_PROP"),
    "D2D1_SHARPEN_PROP": SimTypeInt(signed=False, label="D2D1_SHARPEN_PROP"),
    "D2D1_SIMPLE_COLOR_PROFILE": SimStruct(OrderedDict((("redPrimary", SimTypeRef("D2D_POINT_2F", SimStruct)), ("greenPrimary", SimTypeRef("D2D_POINT_2F", SimStruct)), ("bluePrimary", SimTypeRef("D2D_POINT_2F", SimStruct)), ("whitePointXZ", SimTypeRef("D2D_POINT_2F", SimStruct)), ("gamma", SimTypeInt(signed=False, label="D2D1_GAMMA1")),)), name="D2D1_SIMPLE_COLOR_PROFILE", pack=False, align=None),
    "D2D1_SPOTDIFFUSE_PROP": SimTypeInt(signed=False, label="D2D1_SPOTDIFFUSE_PROP"),
    "D2D1_SPOTDIFFUSE_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_SPOTDIFFUSE_SCALE_MODE"),
    "D2D1_SPOTSPECULAR_PROP": SimTypeInt(signed=False, label="D2D1_SPOTSPECULAR_PROP"),
    "D2D1_SPOTSPECULAR_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_SPOTSPECULAR_SCALE_MODE"),
    "D2D1_SPRITE_OPTIONS": SimTypeInt(signed=False, label="D2D1_SPRITE_OPTIONS"),
    "D2D1_STRAIGHTEN_PROP": SimTypeInt(signed=False, label="D2D1_STRAIGHTEN_PROP"),
    "D2D1_STRAIGHTEN_SCALE_MODE": SimTypeInt(signed=False, label="D2D1_STRAIGHTEN_SCALE_MODE"),
    "D2D1_STROKE_STYLE_PROPERTIES": SimStruct(OrderedDict((("startCap", SimTypeInt(signed=False, label="D2D1_CAP_STYLE")), ("endCap", SimTypeInt(signed=False, label="D2D1_CAP_STYLE")), ("dashCap", SimTypeInt(signed=False, label="D2D1_CAP_STYLE")), ("lineJoin", SimTypeInt(signed=False, label="D2D1_LINE_JOIN")), ("miterLimit", SimTypeFloat(size=32)), ("dashStyle", SimTypeInt(signed=False, label="D2D1_DASH_STYLE")), ("dashOffset", SimTypeFloat(size=32)),)), name="D2D1_STROKE_STYLE_PROPERTIES", pack=False, align=None),
    "D2D1_STROKE_STYLE_PROPERTIES1": SimStruct(OrderedDict((("startCap", SimTypeInt(signed=False, label="D2D1_CAP_STYLE")), ("endCap", SimTypeInt(signed=False, label="D2D1_CAP_STYLE")), ("dashCap", SimTypeInt(signed=False, label="D2D1_CAP_STYLE")), ("lineJoin", SimTypeInt(signed=False, label="D2D1_LINE_JOIN")), ("miterLimit", SimTypeFloat(size=32)), ("dashStyle", SimTypeInt(signed=False, label="D2D1_DASH_STYLE")), ("dashOffset", SimTypeFloat(size=32)), ("transformType", SimTypeInt(signed=False, label="D2D1_STROKE_TRANSFORM_TYPE")),)), name="D2D1_STROKE_STYLE_PROPERTIES1", pack=False, align=None),
    "D2D1_STROKE_TRANSFORM_TYPE": SimTypeInt(signed=False, label="D2D1_STROKE_TRANSFORM_TYPE"),
    "D2D1_SUBPROPERTY": SimTypeInt(signed=False, label="D2D1_SUBPROPERTY"),
    "D2D1_SVG_ASPECT_ALIGN": SimTypeInt(signed=False, label="D2D1_SVG_ASPECT_ALIGN"),
    "D2D1_SVG_ASPECT_SCALING": SimTypeInt(signed=False, label="D2D1_SVG_ASPECT_SCALING"),
    "D2D1_SVG_ATTRIBUTE_POD_TYPE": SimTypeInt(signed=False, label="D2D1_SVG_ATTRIBUTE_POD_TYPE"),
    "D2D1_SVG_ATTRIBUTE_STRING_TYPE": SimTypeInt(signed=False, label="D2D1_SVG_ATTRIBUTE_STRING_TYPE"),
    "D2D1_SVG_DISPLAY": SimTypeInt(signed=False, label="D2D1_SVG_DISPLAY"),
    "D2D1_SVG_LENGTH": SimStruct(OrderedDict((("value", SimTypeFloat(size=32)), ("units", SimTypeInt(signed=False, label="D2D1_SVG_LENGTH_UNITS")),)), name="D2D1_SVG_LENGTH", pack=False, align=None),
    "D2D1_SVG_LENGTH_UNITS": SimTypeInt(signed=False, label="D2D1_SVG_LENGTH_UNITS"),
    "D2D1_SVG_LINE_CAP": SimTypeInt(signed=False, label="D2D1_SVG_LINE_CAP"),
    "D2D1_SVG_LINE_JOIN": SimTypeInt(signed=False, label="D2D1_SVG_LINE_JOIN"),
    "D2D1_SVG_OVERFLOW": SimTypeInt(signed=False, label="D2D1_SVG_OVERFLOW"),
    "D2D1_SVG_PAINT_TYPE": SimTypeInt(signed=False, label="D2D1_SVG_PAINT_TYPE"),
    "D2D1_SVG_PATH_COMMAND": SimTypeInt(signed=False, label="D2D1_SVG_PATH_COMMAND"),
    "D2D1_SVG_PRESERVE_ASPECT_RATIO": SimStruct(OrderedDict((("defer", SimTypeInt(signed=True, label="Int32")), ("align", SimTypeInt(signed=False, label="D2D1_SVG_ASPECT_ALIGN")), ("meetOrSlice", SimTypeInt(signed=False, label="D2D1_SVG_ASPECT_SCALING")),)), name="D2D1_SVG_PRESERVE_ASPECT_RATIO", pack=False, align=None),
    "D2D1_SVG_UNIT_TYPE": SimTypeInt(signed=False, label="D2D1_SVG_UNIT_TYPE"),
    "D2D1_SVG_VIEWBOX": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("width", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)),)), name="D2D1_SVG_VIEWBOX", pack=False, align=None),
    "D2D1_SVG_VISIBILITY": SimTypeInt(signed=False, label="D2D1_SVG_VISIBILITY"),
    "D2D1_SWEEP_DIRECTION": SimTypeInt(signed=False, label="D2D1_SWEEP_DIRECTION"),
    "D2D1_TABLETRANSFER_PROP": SimTypeInt(signed=False, label="D2D1_TABLETRANSFER_PROP"),
    "D2D1_TEMPERATUREANDTINT_PROP": SimTypeInt(signed=False, label="D2D1_TEMPERATUREANDTINT_PROP"),
    "D2D1_TEXT_ANTIALIAS_MODE": SimTypeInt(signed=False, label="D2D1_TEXT_ANTIALIAS_MODE"),
    "D2D1_THREADING_MODE": SimTypeInt(signed=False, label="D2D1_THREADING_MODE"),
    "D2D1_TILE_PROP": SimTypeInt(signed=False, label="D2D1_TILE_PROP"),
    "D2D1_TINT_PROP": SimTypeInt(signed=False, label="D2D1_TINT_PROP"),
    "D2D1_TRANSFORMED_IMAGE_SOURCE_OPTIONS": SimTypeInt(signed=False, label="D2D1_TRANSFORMED_IMAGE_SOURCE_OPTIONS"),
    "D2D1_TRANSFORMED_IMAGE_SOURCE_PROPERTIES": SimStruct(OrderedDict((("orientation", SimTypeInt(signed=False, label="D2D1_ORIENTATION")), ("scaleX", SimTypeFloat(size=32)), ("scaleY", SimTypeFloat(size=32)), ("interpolationMode", SimTypeInt(signed=False, label="D2D1_INTERPOLATION_MODE")), ("options", SimTypeInt(signed=False, label="D2D1_TRANSFORMED_IMAGE_SOURCE_OPTIONS")),)), name="D2D1_TRANSFORMED_IMAGE_SOURCE_PROPERTIES", pack=False, align=None),
    "D2D1_TRIANGLE": SimStruct(OrderedDict((("point1", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point2", SimTypeRef("D2D_POINT_2F", SimStruct)), ("point3", SimTypeRef("D2D_POINT_2F", SimStruct)),)), name="D2D1_TRIANGLE", pack=False, align=None),
    "D2D1_TURBULENCE_NOISE": SimTypeInt(signed=False, label="D2D1_TURBULENCE_NOISE"),
    "D2D1_TURBULENCE_PROP": SimTypeInt(signed=False, label="D2D1_TURBULENCE_PROP"),
    "D2D1_UNIT_MODE": SimTypeInt(signed=False, label="D2D1_UNIT_MODE"),
    "D2D1_VERTEX_BUFFER_PROPERTIES": SimStruct(OrderedDict((("inputCount", SimTypeInt(signed=False, label="UInt32")), ("usage", SimTypeInt(signed=False, label="D2D1_VERTEX_USAGE")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("byteWidth", SimTypeInt(signed=False, label="UInt32")),)), name="D2D1_VERTEX_BUFFER_PROPERTIES", pack=False, align=None),
    "D2D1_VERTEX_OPTIONS": SimTypeInt(signed=False, label="D2D1_VERTEX_OPTIONS"),
    "D2D1_VERTEX_RANGE": SimStruct(OrderedDict((("startVertex", SimTypeInt(signed=False, label="UInt32")), ("vertexCount", SimTypeInt(signed=False, label="UInt32")),)), name="D2D1_VERTEX_RANGE", pack=False, align=None),
    "D2D1_VERTEX_USAGE": SimTypeInt(signed=False, label="D2D1_VERTEX_USAGE"),
    "D2D1_VIGNETTE_PROP": SimTypeInt(signed=False, label="D2D1_VIGNETTE_PROP"),
    "D2D1_WHITELEVELADJUSTMENT_PROP": SimTypeInt(signed=False, label="D2D1_WHITELEVELADJUSTMENT_PROP"),
    "D2D1_WINDOW_STATE": SimTypeInt(signed=False, label="D2D1_WINDOW_STATE"),
    "D2D1_YCBCR_CHROMA_SUBSAMPLING": SimTypeInt(signed=False, label="D2D1_YCBCR_CHROMA_SUBSAMPLING"),
    "D2D1_YCBCR_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D2D1_YCBCR_INTERPOLATION_MODE"),
    "D2D1_YCBCR_PROP": SimTypeInt(signed=False, label="D2D1_YCBCR_PROP"),
    "D2D_COLOR_F": SimStruct(OrderedDict((("r", SimTypeFloat(size=32)), ("g", SimTypeFloat(size=32)), ("b", SimTypeFloat(size=32)), ("a", SimTypeFloat(size=32)),)), name="D2D_COLOR_F", pack=False, align=None),
    "D2D_MATRIX_3X2_F": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("m11", SimTypeFloat(size=32)), ("m12", SimTypeFloat(size=32)), ("m21", SimTypeFloat(size=32)), ("m22", SimTypeFloat(size=32)), ("dx", SimTypeFloat(size=32)), ("dy", SimTypeFloat(size=32)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)),)), name="_Anonymous2_e__Struct", pack=False, align=None), "m": SimTypeArray(SimTypeFloat(size=32), 6)}, name="<anon>", label="None")),)), name="D2D_MATRIX_3X2_F", pack=False, align=None),
    "D2D_MATRIX_4X3_F": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_13", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_23", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)), ("_33", SimTypeFloat(size=32)), ("_41", SimTypeFloat(size=32)), ("_42", SimTypeFloat(size=32)), ("_43", SimTypeFloat(size=32)),)), name="_Anonymous_e__Struct", pack=False, align=None), "m": SimTypeArray(SimTypeFloat(size=32), 12)}, name="<anon>", label="None")),)), name="D2D_MATRIX_4X3_F", pack=False, align=None),
    "D2D_MATRIX_4X4_F": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_13", SimTypeFloat(size=32)), ("_14", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_23", SimTypeFloat(size=32)), ("_24", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)), ("_33", SimTypeFloat(size=32)), ("_34", SimTypeFloat(size=32)), ("_41", SimTypeFloat(size=32)), ("_42", SimTypeFloat(size=32)), ("_43", SimTypeFloat(size=32)), ("_44", SimTypeFloat(size=32)),)), name="_Anonymous_e__Struct", pack=False, align=None), "m": SimTypeArray(SimTypeFloat(size=32), 16)}, name="<anon>", label="None")),)), name="D2D_MATRIX_4X4_F", pack=False, align=None),
    "D2D_MATRIX_5X4_F": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_13", SimTypeFloat(size=32)), ("_14", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_23", SimTypeFloat(size=32)), ("_24", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)), ("_33", SimTypeFloat(size=32)), ("_34", SimTypeFloat(size=32)), ("_41", SimTypeFloat(size=32)), ("_42", SimTypeFloat(size=32)), ("_43", SimTypeFloat(size=32)), ("_44", SimTypeFloat(size=32)), ("_51", SimTypeFloat(size=32)), ("_52", SimTypeFloat(size=32)), ("_53", SimTypeFloat(size=32)), ("_54", SimTypeFloat(size=32)),)), name="_Anonymous_e__Struct", pack=False, align=None), "m": SimTypeArray(SimTypeFloat(size=32), 20)}, name="<anon>", label="None")),)), name="D2D_MATRIX_5X4_F", pack=False, align=None),
    "D2D_POINT_2F": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="D2D_POINT_2F", pack=False, align=None),
    "D2D_POINT_2U": SimStruct(OrderedDict((("x", SimTypeInt(signed=False, label="UInt32")), ("y", SimTypeInt(signed=False, label="UInt32")),)), name="D2D_POINT_2U", pack=False, align=None),
    "D2D_RECT_F": SimStruct(OrderedDict((("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("right", SimTypeFloat(size=32)), ("bottom", SimTypeFloat(size=32)),)), name="D2D_RECT_F", pack=False, align=None),
    "D2D_RECT_U": SimStruct(OrderedDict((("left", SimTypeInt(signed=False, label="UInt32")), ("top", SimTypeInt(signed=False, label="UInt32")), ("right", SimTypeInt(signed=False, label="UInt32")), ("bottom", SimTypeInt(signed=False, label="UInt32")),)), name="D2D_RECT_U", pack=False, align=None),
    "D2D_SIZE_F": SimStruct(OrderedDict((("width", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)),)), name="D2D_SIZE_F", pack=False, align=None),
    "D2D_SIZE_U": SimStruct(OrderedDict((("width", SimTypeInt(signed=False, label="UInt32")), ("height", SimTypeInt(signed=False, label="UInt32")),)), name="D2D_SIZE_U", pack=False, align=None),
    "D2D_VECTOR_2F": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="D2D_VECTOR_2F", pack=False, align=None),
    "D2D_VECTOR_3F": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("z", SimTypeFloat(size=32)),)), name="D2D_VECTOR_3F", pack=False, align=None),
    "D2D_VECTOR_4F": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("z", SimTypeFloat(size=32)), ("w", SimTypeFloat(size=32)),)), name="D2D_VECTOR_4F", pack=False, align=None),
    "D3COLD_AUX_POWER_AND_TIMING_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("RequestCorePowerRail", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("RequestAuxPower", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("RequestPerstDelay", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="D3COLD_AUX_POWER_AND_TIMING_INTERFACE", pack=False, align=None),
    "D3COLD_LAST_TRANSITION_STATUS": SimTypeInt(signed=False, label="D3COLD_LAST_TRANSITION_STATUS"),
    "D3COLD_REQUEST_AUX_POWER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "AuxPowerInMilliWatts", "RetryInSeconds"]), offset=0),
    "D3COLD_REQUEST_CORE_POWER_RAIL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Context", "CorePowerRailNeeded"]), offset=0),
    "D3COLD_REQUEST_PERST_DELAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "DelayInMicroSeconds"]), offset=0),
    "D3COLD_SUPPORT_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetD3ColdSupport", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("GetIdleWakeInfo", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetD3ColdCapability", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetBusDriverD3ColdSupport", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetLastTransitionStatus", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)),)), name="D3COLD_SUPPORT_INTERFACE", pack=False, align=None),
    "D3D10_ASYNC_GETDATA_FLAG": SimTypeInt(signed=False, label="D3D10_ASYNC_GETDATA_FLAG"),
    "D3D10_BIND_FLAG": SimTypeInt(signed=False, label="D3D10_BIND_FLAG"),
    "D3D10_BLEND": SimTypeInt(signed=False, label="D3D10_BLEND"),
    "D3D10_BLEND_DESC": SimStruct(OrderedDict((("AlphaToCoverageEnable", SimTypeInt(signed=True, label="Int32")), ("BlendEnable", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 8)), ("SrcBlend", SimTypeInt(signed=False, label="D3D10_BLEND")), ("DestBlend", SimTypeInt(signed=False, label="D3D10_BLEND")), ("BlendOp", SimTypeInt(signed=False, label="D3D10_BLEND_OP")), ("SrcBlendAlpha", SimTypeInt(signed=False, label="D3D10_BLEND")), ("DestBlendAlpha", SimTypeInt(signed=False, label="D3D10_BLEND")), ("BlendOpAlpha", SimTypeInt(signed=False, label="D3D10_BLEND_OP")), ("RenderTargetWriteMask", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="D3D10_BLEND_DESC", pack=False, align=None),
    "D3D10_BLEND_DESC1": SimStruct(OrderedDict((("AlphaToCoverageEnable", SimTypeInt(signed=True, label="Int32")), ("IndependentBlendEnable", SimTypeInt(signed=True, label="Int32")), ("RenderTarget", SimTypeArray(SimTypeRef("D3D10_RENDER_TARGET_BLEND_DESC1", SimStruct), 8)),)), name="D3D10_BLEND_DESC1", pack=False, align=None),
    "D3D10_BLEND_OP": SimTypeInt(signed=False, label="D3D10_BLEND_OP"),
    "D3D10_BOX": SimStruct(OrderedDict((("left", SimTypeInt(signed=False, label="UInt32")), ("top", SimTypeInt(signed=False, label="UInt32")), ("front", SimTypeInt(signed=False, label="UInt32")), ("right", SimTypeInt(signed=False, label="UInt32")), ("bottom", SimTypeInt(signed=False, label="UInt32")), ("back", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_BOX", pack=False, align=None),
    "D3D10_BUFFER_DESC": SimStruct(OrderedDict((("ByteWidth", SimTypeInt(signed=False, label="UInt32")), ("Usage", SimTypeInt(signed=False, label="D3D10_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_BUFFER_DESC", pack=False, align=None),
    "D3D10_BUFFER_RTV": SimStruct(OrderedDict((("Anonymous1", SimUnion({"FirstElement": SimTypeInt(signed=False, label="UInt32"), "ElementOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"NumElements": SimTypeInt(signed=False, label="UInt32"), "ElementWidth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D10_BUFFER_RTV", pack=False, align=None),
    "D3D10_BUFFER_SRV": SimStruct(OrderedDict((("Anonymous1", SimUnion({"FirstElement": SimTypeInt(signed=False, label="UInt32"), "ElementOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"NumElements": SimTypeInt(signed=False, label="UInt32"), "ElementWidth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D10_BUFFER_SRV", pack=False, align=None),
    "D3D10_CLEAR_FLAG": SimTypeInt(signed=False, label="D3D10_CLEAR_FLAG"),
    "D3D10_COLOR_WRITE_ENABLE": SimTypeInt(signed=False, label="D3D10_COLOR_WRITE_ENABLE"),
    "D3D10_COMPARISON_FUNC": SimTypeInt(signed=False, label="D3D10_COMPARISON_FUNC"),
    "D3D10_COUNTER": SimTypeInt(signed=False, label="D3D10_COUNTER"),
    "D3D10_COUNTER_DESC": SimStruct(OrderedDict((("Counter", SimTypeInt(signed=False, label="D3D10_COUNTER")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_COUNTER_DESC", pack=False, align=None),
    "D3D10_COUNTER_INFO": SimStruct(OrderedDict((("LastDeviceDependentCounter", SimTypeInt(signed=False, label="D3D10_COUNTER")), ("NumSimultaneousCounters", SimTypeInt(signed=False, label="UInt32")), ("NumDetectableParallelUnits", SimTypeChar(label="Byte")),)), name="D3D10_COUNTER_INFO", pack=False, align=None),
    "D3D10_COUNTER_TYPE": SimTypeInt(signed=False, label="D3D10_COUNTER_TYPE"),
    "D3D10_CPU_ACCESS_FLAG": SimTypeInt(signed=False, label="D3D10_CPU_ACCESS_FLAG"),
    "D3D10_CREATE_DEVICE_FLAG": SimTypeInt(signed=False, label="D3D10_CREATE_DEVICE_FLAG"),
    "D3D10_CULL_MODE": SimTypeInt(signed=False, label="D3D10_CULL_MODE"),
    "D3D10_DEPTH_STENCILOP_DESC": SimStruct(OrderedDict((("StencilFailOp", SimTypeInt(signed=False, label="D3D10_STENCIL_OP")), ("StencilDepthFailOp", SimTypeInt(signed=False, label="D3D10_STENCIL_OP")), ("StencilPassOp", SimTypeInt(signed=False, label="D3D10_STENCIL_OP")), ("StencilFunc", SimTypeInt(signed=False, label="D3D10_COMPARISON_FUNC")),)), name="D3D10_DEPTH_STENCILOP_DESC", pack=False, align=None),
    "D3D10_DEPTH_STENCIL_DESC": SimStruct(OrderedDict((("DepthEnable", SimTypeInt(signed=True, label="Int32")), ("DepthWriteMask", SimTypeInt(signed=False, label="D3D10_DEPTH_WRITE_MASK")), ("DepthFunc", SimTypeInt(signed=False, label="D3D10_COMPARISON_FUNC")), ("StencilEnable", SimTypeInt(signed=True, label="Int32")), ("StencilReadMask", SimTypeChar(label="Byte")), ("StencilWriteMask", SimTypeChar(label="Byte")), ("FrontFace", SimTypeRef("D3D10_DEPTH_STENCILOP_DESC", SimStruct)), ("BackFace", SimTypeRef("D3D10_DEPTH_STENCILOP_DESC", SimStruct)),)), name="D3D10_DEPTH_STENCIL_DESC", pack=False, align=None),
    "D3D10_DEPTH_STENCIL_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D10_DSV_DIMENSION")), ("Anonymous", SimUnion({"Texture1D": SimTypeRef("D3D10_TEX1D_DSV", SimStruct), "Texture1DArray": SimTypeRef("D3D10_TEX1D_ARRAY_DSV", SimStruct), "Texture2D": SimTypeRef("D3D10_TEX2D_DSV", SimStruct), "Texture2DArray": SimTypeRef("D3D10_TEX2D_ARRAY_DSV", SimStruct), "Texture2DMS": SimTypeRef("D3D10_TEX2DMS_DSV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D10_TEX2DMS_ARRAY_DSV", SimStruct)}, name="<anon>", label="None")),)), name="D3D10_DEPTH_STENCIL_VIEW_DESC", pack=False, align=None),
    "D3D10_DEPTH_WRITE_MASK": SimTypeInt(signed=False, label="D3D10_DEPTH_WRITE_MASK"),
    "D3D10_DEVICE_STATE_TYPES": SimTypeInt(signed=False, label="D3D10_DEVICE_STATE_TYPES"),
    "D3D10_DRIVER_TYPE": SimTypeInt(signed=False, label="D3D10_DRIVER_TYPE"),
    "D3D10_DSV_DIMENSION": SimTypeInt(signed=False, label="D3D10_DSV_DIMENSION"),
    "D3D10_EFFECT_DESC": SimStruct(OrderedDict((("IsChildEffect", SimTypeInt(signed=True, label="Int32")), ("ConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("SharedConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("GlobalVariables", SimTypeInt(signed=False, label="UInt32")), ("SharedGlobalVariables", SimTypeInt(signed=False, label="UInt32")), ("Techniques", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_EFFECT_DESC", pack=False, align=None),
    "D3D10_EFFECT_SHADER_DESC": SimStruct(OrderedDict((("pInputSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("IsInline", SimTypeInt(signed=True, label="Int32")), ("pBytecode", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BytecodeLength", SimTypeInt(signed=False, label="UInt32")), ("SODecl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NumInputSignatureEntries", SimTypeInt(signed=False, label="UInt32")), ("NumOutputSignatureEntries", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_EFFECT_SHADER_DESC", pack=False, align=None),
    "D3D10_EFFECT_TYPE_DESC": SimStruct(OrderedDict((("TypeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Elements", SimTypeInt(signed=False, label="UInt32")), ("Members", SimTypeInt(signed=False, label="UInt32")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("PackedSize", SimTypeInt(signed=False, label="UInt32")), ("UnpackedSize", SimTypeInt(signed=False, label="UInt32")), ("Stride", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_EFFECT_TYPE_DESC", pack=False, align=None),
    "D3D10_EFFECT_VARIABLE_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Semantic", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Annotations", SimTypeInt(signed=False, label="UInt32")), ("BufferOffset", SimTypeInt(signed=False, label="UInt32")), ("ExplicitBindPoint", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_EFFECT_VARIABLE_DESC", pack=False, align=None),
    "D3D10_FEATURE_LEVEL1": SimTypeInt(signed=False, label="D3D10_FEATURE_LEVEL1"),
    "D3D10_FILL_MODE": SimTypeInt(signed=False, label="D3D10_FILL_MODE"),
    "D3D10_FILTER": SimTypeInt(signed=False, label="D3D10_FILTER"),
    "D3D10_FILTER_TYPE": SimTypeInt(signed=False, label="D3D10_FILTER_TYPE"),
    "D3D10_FORMAT_SUPPORT": SimTypeInt(signed=False, label="D3D10_FORMAT_SUPPORT"),
    "D3D10_INFO_QUEUE_FILTER": SimStruct(OrderedDict((("AllowList", SimTypeRef("D3D10_INFO_QUEUE_FILTER_DESC", SimStruct)), ("DenyList", SimTypeRef("D3D10_INFO_QUEUE_FILTER_DESC", SimStruct)),)), name="D3D10_INFO_QUEUE_FILTER", pack=False, align=None),
    "D3D10_INFO_QUEUE_FILTER_DESC": SimStruct(OrderedDict((("NumCategories", SimTypeInt(signed=False, label="UInt32")), ("pCategoryList", SimTypePointer(SimTypeInt(signed=False, label="D3D10_MESSAGE_CATEGORY"), offset=0)), ("NumSeverities", SimTypeInt(signed=False, label="UInt32")), ("pSeverityList", SimTypePointer(SimTypeInt(signed=False, label="D3D10_MESSAGE_SEVERITY"), offset=0)), ("NumIDs", SimTypeInt(signed=False, label="UInt32")), ("pIDList", SimTypePointer(SimTypeInt(signed=False, label="D3D10_MESSAGE_ID"), offset=0)),)), name="D3D10_INFO_QUEUE_FILTER_DESC", pack=False, align=None),
    "D3D10_INPUT_CLASSIFICATION": SimTypeInt(signed=False, label="D3D10_INPUT_CLASSIFICATION"),
    "D3D10_INPUT_ELEMENT_DESC": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("InputSlot", SimTypeInt(signed=False, label="UInt32")), ("AlignedByteOffset", SimTypeInt(signed=False, label="UInt32")), ("InputSlotClass", SimTypeInt(signed=False, label="D3D10_INPUT_CLASSIFICATION")), ("InstanceDataStepRate", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_INPUT_ELEMENT_DESC", pack=False, align=None),
    "D3D10_MAP": SimTypeInt(signed=False, label="D3D10_MAP"),
    "D3D10_MAPPED_TEXTURE2D": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RowPitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_MAPPED_TEXTURE2D", pack=False, align=None),
    "D3D10_MAPPED_TEXTURE3D": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RowPitch", SimTypeInt(signed=False, label="UInt32")), ("DepthPitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_MAPPED_TEXTURE3D", pack=False, align=None),
    "D3D10_MAP_FLAG": SimTypeInt(signed=False, label="D3D10_MAP_FLAG"),
    "D3D10_MESSAGE": SimStruct(OrderedDict((("Category", SimTypeInt(signed=False, label="D3D10_MESSAGE_CATEGORY")), ("Severity", SimTypeInt(signed=False, label="D3D10_MESSAGE_SEVERITY")), ("ID", SimTypeInt(signed=False, label="D3D10_MESSAGE_ID")), ("pDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DescriptionByteLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D10_MESSAGE", pack=False, align=None),
    "D3D10_MESSAGE_CATEGORY": SimTypeInt(signed=False, label="D3D10_MESSAGE_CATEGORY"),
    "D3D10_MESSAGE_ID": SimTypeInt(signed=False, label="D3D10_MESSAGE_ID"),
    "D3D10_MESSAGE_SEVERITY": SimTypeInt(signed=False, label="D3D10_MESSAGE_SEVERITY"),
    "D3D10_PASS_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Annotations", SimTypeInt(signed=False, label="UInt32")), ("pIAInputSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("IAInputSignatureSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("StencilRef", SimTypeInt(signed=False, label="UInt32")), ("SampleMask", SimTypeInt(signed=False, label="UInt32")), ("BlendFactor", SimTypeArray(SimTypeFloat(size=32), 4)),)), name="D3D10_PASS_DESC", pack=False, align=None),
    "D3D10_PASS_SHADER_DESC": SimStruct(OrderedDict((("pShaderVariable", SimTypeBottom(label="ID3D10EffectShaderVariable")), ("ShaderIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_PASS_SHADER_DESC", pack=False, align=None),
    "D3D10_QUERY": SimTypeInt(signed=False, label="D3D10_QUERY"),
    "D3D10_QUERY_DATA_PIPELINE_STATISTICS": SimStruct(OrderedDict((("IAVertices", SimTypeLongLong(signed=False, label="UInt64")), ("IAPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("VSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("CInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("PSInvocations", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D10_QUERY_DATA_PIPELINE_STATISTICS", pack=False, align=None),
    "D3D10_QUERY_DATA_SO_STATISTICS": SimStruct(OrderedDict((("NumPrimitivesWritten", SimTypeLongLong(signed=False, label="UInt64")), ("PrimitivesStorageNeeded", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D10_QUERY_DATA_SO_STATISTICS", pack=False, align=None),
    "D3D10_QUERY_DATA_TIMESTAMP_DISJOINT": SimStruct(OrderedDict((("Frequency", SimTypeLongLong(signed=False, label="UInt64")), ("Disjoint", SimTypeInt(signed=True, label="Int32")),)), name="D3D10_QUERY_DATA_TIMESTAMP_DISJOINT", pack=False, align=None),
    "D3D10_QUERY_DESC": SimStruct(OrderedDict((("Query", SimTypeInt(signed=False, label="D3D10_QUERY")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_QUERY_DESC", pack=False, align=None),
    "D3D10_QUERY_MISC_FLAG": SimTypeInt(signed=False, label="D3D10_QUERY_MISC_FLAG"),
    "D3D10_RAISE_FLAG": SimTypeInt(signed=False, label="D3D10_RAISE_FLAG"),
    "D3D10_RASTERIZER_DESC": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D10_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D10_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeInt(signed=True, label="Int32")), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("ScissorEnable", SimTypeInt(signed=True, label="Int32")), ("MultisampleEnable", SimTypeInt(signed=True, label="Int32")), ("AntialiasedLineEnable", SimTypeInt(signed=True, label="Int32")),)), name="D3D10_RASTERIZER_DESC", pack=False, align=None),
    "D3D10_RENDER_TARGET_BLEND_DESC1": SimStruct(OrderedDict((("BlendEnable", SimTypeInt(signed=True, label="Int32")), ("SrcBlend", SimTypeInt(signed=False, label="D3D10_BLEND")), ("DestBlend", SimTypeInt(signed=False, label="D3D10_BLEND")), ("BlendOp", SimTypeInt(signed=False, label="D3D10_BLEND_OP")), ("SrcBlendAlpha", SimTypeInt(signed=False, label="D3D10_BLEND")), ("DestBlendAlpha", SimTypeInt(signed=False, label="D3D10_BLEND")), ("BlendOpAlpha", SimTypeInt(signed=False, label="D3D10_BLEND_OP")), ("RenderTargetWriteMask", SimTypeChar(label="Byte")),)), name="D3D10_RENDER_TARGET_BLEND_DESC1", pack=False, align=None),
    "D3D10_RENDER_TARGET_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D10_RTV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D10_BUFFER_RTV", SimStruct), "Texture1D": SimTypeRef("D3D10_TEX1D_RTV", SimStruct), "Texture1DArray": SimTypeRef("D3D10_TEX1D_ARRAY_RTV", SimStruct), "Texture2D": SimTypeRef("D3D10_TEX2D_RTV", SimStruct), "Texture2DArray": SimTypeRef("D3D10_TEX2D_ARRAY_RTV", SimStruct), "Texture2DMS": SimTypeRef("D3D10_TEX2DMS_RTV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D10_TEX2DMS_ARRAY_RTV", SimStruct), "Texture3D": SimTypeRef("D3D10_TEX3D_RTV", SimStruct)}, name="<anon>", label="None")),)), name="D3D10_RENDER_TARGET_VIEW_DESC", pack=False, align=None),
    "D3D10_RESOURCE_DIMENSION": SimTypeInt(signed=False, label="D3D10_RESOURCE_DIMENSION"),
    "D3D10_RESOURCE_MISC_FLAG": SimTypeInt(signed=False, label="D3D10_RESOURCE_MISC_FLAG"),
    "D3D10_RTV_DIMENSION": SimTypeInt(signed=False, label="D3D10_RTV_DIMENSION"),
    "D3D10_SAMPLER_DESC": SimStruct(OrderedDict((("Filter", SimTypeInt(signed=False, label="D3D10_FILTER")), ("AddressU", SimTypeInt(signed=False, label="D3D10_TEXTURE_ADDRESS_MODE")), ("AddressV", SimTypeInt(signed=False, label="D3D10_TEXTURE_ADDRESS_MODE")), ("AddressW", SimTypeInt(signed=False, label="D3D10_TEXTURE_ADDRESS_MODE")), ("MipLODBias", SimTypeFloat(size=32)), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("ComparisonFunc", SimTypeInt(signed=False, label="D3D10_COMPARISON_FUNC")), ("BorderColor", SimTypeArray(SimTypeFloat(size=32), 4)), ("MinLOD", SimTypeFloat(size=32)), ("MaxLOD", SimTypeFloat(size=32)),)), name="D3D10_SAMPLER_DESC", pack=False, align=None),
    "D3D10_SHADER_BUFFER_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_CBUFFER_TYPE")), ("Variables", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_BUFFER_DESC", pack=False, align=None),
    "D3D10_SHADER_DEBUG_FILE_INFO": SimStruct(OrderedDict((("FileName", SimTypeInt(signed=False, label="UInt32")), ("FileNameLen", SimTypeInt(signed=False, label="UInt32")), ("FileData", SimTypeInt(signed=False, label="UInt32")), ("FileLen", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_FILE_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Creator", SimTypeInt(signed=False, label="UInt32")), ("EntrypointName", SimTypeInt(signed=False, label="UInt32")), ("ShaderTarget", SimTypeInt(signed=False, label="UInt32")), ("CompileFlags", SimTypeInt(signed=False, label="UInt32")), ("Files", SimTypeInt(signed=False, label="UInt32")), ("FileInfo", SimTypeInt(signed=False, label="UInt32")), ("Instructions", SimTypeInt(signed=False, label="UInt32")), ("InstructionInfo", SimTypeInt(signed=False, label="UInt32")), ("Variables", SimTypeInt(signed=False, label="UInt32")), ("VariableInfo", SimTypeInt(signed=False, label="UInt32")), ("InputVariables", SimTypeInt(signed=False, label="UInt32")), ("InputVariableInfo", SimTypeInt(signed=False, label="UInt32")), ("Tokens", SimTypeInt(signed=False, label="UInt32")), ("TokenInfo", SimTypeInt(signed=False, label="UInt32")), ("Scopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeInfo", SimTypeInt(signed=False, label="UInt32")), ("ScopeVariables", SimTypeInt(signed=False, label="UInt32")), ("ScopeVariableInfo", SimTypeInt(signed=False, label="UInt32")), ("UintOffset", SimTypeInt(signed=False, label="UInt32")), ("StringOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_INPUT_INFO": SimStruct(OrderedDict((("Var", SimTypeInt(signed=False, label="UInt32")), ("InitialRegisterSet", SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_REGTYPE")), ("InitialBank", SimTypeInt(signed=False, label="UInt32")), ("InitialRegister", SimTypeInt(signed=False, label="UInt32")), ("InitialComponent", SimTypeInt(signed=False, label="UInt32")), ("InitialValue", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_INPUT_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_INST_INFO": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("Opcode", SimTypeInt(signed=False, label="UInt32")), ("uOutputs", SimTypeInt(signed=False, label="UInt32")), ("pOutputs", SimTypeArray(SimTypeRef("D3D10_SHADER_DEBUG_OUTPUTREG_INFO", SimStruct), 2)), ("TokenId", SimTypeInt(signed=False, label="UInt32")), ("NestingLevel", SimTypeInt(signed=False, label="UInt32")), ("Scopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeInfo", SimTypeInt(signed=False, label="UInt32")), ("AccessedVars", SimTypeInt(signed=False, label="UInt32")), ("AccessedVarsInfo", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_INST_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_OUTPUTREG_INFO": SimStruct(OrderedDict((("OutputRegisterSet", SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_REGTYPE")), ("OutputReg", SimTypeInt(signed=False, label="UInt32")), ("TempArrayReg", SimTypeInt(signed=False, label="UInt32")), ("OutputComponents", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("OutputVars", SimTypeArray(SimTypeRef("D3D10_SHADER_DEBUG_OUTPUTVAR", SimStruct), 4)), ("IndexReg", SimTypeInt(signed=False, label="UInt32")), ("IndexComp", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_OUTPUTREG_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_OUTPUTVAR": SimStruct(OrderedDict((("Var", SimTypeInt(signed=False, label="UInt32")), ("uValueMin", SimTypeInt(signed=False, label="UInt32")), ("uValueMax", SimTypeInt(signed=False, label="UInt32")), ("iValueMin", SimTypeInt(signed=True, label="Int32")), ("iValueMax", SimTypeInt(signed=True, label="Int32")), ("fValueMin", SimTypeFloat(size=32)), ("fValueMax", SimTypeFloat(size=32)), ("bNaNPossible", SimTypeInt(signed=True, label="Int32")), ("bInfPossible", SimTypeInt(signed=True, label="Int32")),)), name="D3D10_SHADER_DEBUG_OUTPUTVAR", pack=False, align=None),
    "D3D10_SHADER_DEBUG_REGTYPE": SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_REGTYPE"),
    "D3D10_SHADER_DEBUG_SCOPETYPE": SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_SCOPETYPE"),
    "D3D10_SHADER_DEBUG_SCOPEVAR_INFO": SimStruct(OrderedDict((("TokenId", SimTypeInt(signed=False, label="UInt32")), ("VarType", SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_VARTYPE")), ("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("StructMemberScope", SimTypeInt(signed=False, label="UInt32")), ("uArrayIndices", SimTypeInt(signed=False, label="UInt32")), ("ArrayElements", SimTypeInt(signed=False, label="UInt32")), ("ArrayStrides", SimTypeInt(signed=False, label="UInt32")), ("uVariables", SimTypeInt(signed=False, label="UInt32")), ("uFirstVariable", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_SCOPEVAR_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_SCOPE_INFO": SimStruct(OrderedDict((("ScopeType", SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_SCOPETYPE")), ("Name", SimTypeInt(signed=False, label="UInt32")), ("uNameLen", SimTypeInt(signed=False, label="UInt32")), ("uVariables", SimTypeInt(signed=False, label="UInt32")), ("VariableData", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_SCOPE_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_TOKEN_INFO": SimStruct(OrderedDict((("File", SimTypeInt(signed=False, label="UInt32")), ("Line", SimTypeInt(signed=False, label="UInt32")), ("Column", SimTypeInt(signed=False, label="UInt32")), ("TokenLength", SimTypeInt(signed=False, label="UInt32")), ("TokenId", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_TOKEN_INFO", pack=False, align=None),
    "D3D10_SHADER_DEBUG_VARTYPE": SimTypeInt(signed=False, label="D3D10_SHADER_DEBUG_VARTYPE"),
    "D3D10_SHADER_DEBUG_VAR_INFO": SimStruct(OrderedDict((("TokenId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("Component", SimTypeInt(signed=False, label="UInt32")), ("ScopeVar", SimTypeInt(signed=False, label="UInt32")), ("ScopeVarOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DEBUG_VAR_INFO", pack=False, align=None),
    "D3D10_SHADER_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("BoundResources", SimTypeInt(signed=False, label="UInt32")), ("InputParameters", SimTypeInt(signed=False, label="UInt32")), ("OutputParameters", SimTypeInt(signed=False, label="UInt32")), ("InstructionCount", SimTypeInt(signed=False, label="UInt32")), ("TempRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("TempArrayCount", SimTypeInt(signed=False, label="UInt32")), ("DefCount", SimTypeInt(signed=False, label="UInt32")), ("DclCount", SimTypeInt(signed=False, label="UInt32")), ("TextureNormalInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureLoadInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureCompInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureBiasInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureGradientInstructions", SimTypeInt(signed=False, label="UInt32")), ("FloatInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("IntInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("UintInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("StaticFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("MacroInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ArrayInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("CutInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("EmitInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GSOutputTopology", SimTypeInt(signed=False, label="D3D_PRIMITIVE_TOPOLOGY")), ("GSMaxOutputVertexCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_DESC", pack=False, align=None),
    "D3D10_SHADER_INPUT_BIND_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_INPUT_TYPE")), ("BindPoint", SimTypeInt(signed=False, label="UInt32")), ("BindCount", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("ReturnType", SimTypeInt(signed=False, label="D3D_RESOURCE_RETURN_TYPE")), ("Dimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("NumSamples", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_INPUT_BIND_DESC", pack=False, align=None),
    "D3D10_SHADER_RESOURCE_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D10_BUFFER_SRV", SimStruct), "Texture1D": SimTypeRef("D3D10_TEX1D_SRV", SimStruct), "Texture1DArray": SimTypeRef("D3D10_TEX1D_ARRAY_SRV", SimStruct), "Texture2D": SimTypeRef("D3D10_TEX2D_SRV", SimStruct), "Texture2DArray": SimTypeRef("D3D10_TEX2D_ARRAY_SRV", SimStruct), "Texture2DMS": SimTypeRef("D3D10_TEX2DMS_SRV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D10_TEX2DMS_ARRAY_SRV", SimStruct), "Texture3D": SimTypeRef("D3D10_TEX3D_SRV", SimStruct), "TextureCube": SimTypeRef("D3D10_TEXCUBE_SRV", SimStruct)}, name="<anon>", label="None")),)), name="D3D10_SHADER_RESOURCE_VIEW_DESC", pack=False, align=None),
    "D3D10_SHADER_RESOURCE_VIEW_DESC1": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D10_BUFFER_SRV", SimStruct), "Texture1D": SimTypeRef("D3D10_TEX1D_SRV", SimStruct), "Texture1DArray": SimTypeRef("D3D10_TEX1D_ARRAY_SRV", SimStruct), "Texture2D": SimTypeRef("D3D10_TEX2D_SRV", SimStruct), "Texture2DArray": SimTypeRef("D3D10_TEX2D_ARRAY_SRV", SimStruct), "Texture2DMS": SimTypeRef("D3D10_TEX2DMS_SRV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D10_TEX2DMS_ARRAY_SRV", SimStruct), "Texture3D": SimTypeRef("D3D10_TEX3D_SRV", SimStruct), "TextureCube": SimTypeRef("D3D10_TEXCUBE_SRV", SimStruct), "TextureCubeArray": SimTypeRef("D3D10_TEXCUBE_ARRAY_SRV1", SimStruct)}, name="<anon>", label="None")),)), name="D3D10_SHADER_RESOURCE_VIEW_DESC1", pack=False, align=None),
    "D3D10_SHADER_TYPE_DESC": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypeInt(signed=False, label="UInt32")), ("Members", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SHADER_TYPE_DESC", pack=False, align=None),
    "D3D10_SHADER_VARIABLE_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StartOffset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("DefaultValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3D10_SHADER_VARIABLE_DESC", pack=False, align=None),
    "D3D10_SIGNATURE_PARAMETER_DESC": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("SystemValueType", SimTypeInt(signed=False, label="D3D_NAME")), ("ComponentType", SimTypeInt(signed=False, label="D3D_REGISTER_COMPONENT_TYPE")), ("Mask", SimTypeChar(label="Byte")), ("ReadWriteMask", SimTypeChar(label="Byte")),)), name="D3D10_SIGNATURE_PARAMETER_DESC", pack=False, align=None),
    "D3D10_SO_DECLARATION_ENTRY": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("StartComponent", SimTypeChar(label="Byte")), ("ComponentCount", SimTypeChar(label="Byte")), ("OutputSlot", SimTypeChar(label="Byte")),)), name="D3D10_SO_DECLARATION_ENTRY", pack=False, align=None),
    "D3D10_STANDARD_MULTISAMPLE_QUALITY_LEVELS": SimTypeInt(signed=False, label="D3D10_STANDARD_MULTISAMPLE_QUALITY_LEVELS"),
    "D3D10_STATE_BLOCK_MASK": SimStruct(OrderedDict((("VS", SimTypeChar(label="Byte")), ("VSSamplers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("VSShaderResources", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("VSConstantBuffers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("GS", SimTypeChar(label="Byte")), ("GSSamplers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("GSShaderResources", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("GSConstantBuffers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("PS", SimTypeChar(label="Byte")), ("PSSamplers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("PSShaderResources", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PSConstantBuffers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("IAVertexBuffers", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("IAIndexBuffer", SimTypeChar(label="Byte")), ("IAInputLayout", SimTypeChar(label="Byte")), ("IAPrimitiveTopology", SimTypeChar(label="Byte")), ("OMRenderTargets", SimTypeChar(label="Byte")), ("OMDepthStencilState", SimTypeChar(label="Byte")), ("OMBlendState", SimTypeChar(label="Byte")), ("RSViewports", SimTypeChar(label="Byte")), ("RSScissorRects", SimTypeChar(label="Byte")), ("RSRasterizerState", SimTypeChar(label="Byte")), ("SOBuffers", SimTypeChar(label="Byte")), ("Predication", SimTypeChar(label="Byte")),)), name="D3D10_STATE_BLOCK_MASK", pack=False, align=None),
    "D3D10_STENCIL_OP": SimTypeInt(signed=False, label="D3D10_STENCIL_OP"),
    "D3D10_SUBRESOURCE_DATA": SimStruct(OrderedDict((("pSysMem", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SysMemPitch", SimTypeInt(signed=False, label="UInt32")), ("SysMemSlicePitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_SUBRESOURCE_DATA", pack=False, align=None),
    "D3D10_TECHNIQUE_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Passes", SimTypeInt(signed=False, label="UInt32")), ("Annotations", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TECHNIQUE_DESC", pack=False, align=None),
    "D3D10_TEX1D_ARRAY_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX1D_ARRAY_DSV", pack=False, align=None),
    "D3D10_TEX1D_ARRAY_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX1D_ARRAY_RTV", pack=False, align=None),
    "D3D10_TEX1D_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX1D_ARRAY_SRV", pack=False, align=None),
    "D3D10_TEX1D_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX1D_DSV", pack=False, align=None),
    "D3D10_TEX1D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX1D_RTV", pack=False, align=None),
    "D3D10_TEX1D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX1D_SRV", pack=False, align=None),
    "D3D10_TEX2DMS_ARRAY_DSV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2DMS_ARRAY_DSV", pack=False, align=None),
    "D3D10_TEX2DMS_ARRAY_RTV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2DMS_ARRAY_RTV", pack=False, align=None),
    "D3D10_TEX2DMS_ARRAY_SRV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2DMS_ARRAY_SRV", pack=False, align=None),
    "D3D10_TEX2DMS_DSV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2DMS_DSV", pack=False, align=None),
    "D3D10_TEX2DMS_RTV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2DMS_RTV", pack=False, align=None),
    "D3D10_TEX2DMS_SRV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2DMS_SRV", pack=False, align=None),
    "D3D10_TEX2D_ARRAY_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2D_ARRAY_DSV", pack=False, align=None),
    "D3D10_TEX2D_ARRAY_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2D_ARRAY_RTV", pack=False, align=None),
    "D3D10_TEX2D_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2D_ARRAY_SRV", pack=False, align=None),
    "D3D10_TEX2D_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2D_DSV", pack=False, align=None),
    "D3D10_TEX2D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2D_RTV", pack=False, align=None),
    "D3D10_TEX2D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX2D_SRV", pack=False, align=None),
    "D3D10_TEX3D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstWSlice", SimTypeInt(signed=False, label="UInt32")), ("WSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX3D_RTV", pack=False, align=None),
    "D3D10_TEX3D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEX3D_SRV", pack=False, align=None),
    "D3D10_TEXCUBE_ARRAY_SRV1": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("First2DArrayFace", SimTypeInt(signed=False, label="UInt32")), ("NumCubes", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEXCUBE_ARRAY_SRV1", pack=False, align=None),
    "D3D10_TEXCUBE_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEXCUBE_SRV", pack=False, align=None),
    "D3D10_TEXTURE1D_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Usage", SimTypeInt(signed=False, label="D3D10_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEXTURE1D_DESC", pack=False, align=None),
    "D3D10_TEXTURE2D_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("Usage", SimTypeInt(signed=False, label="D3D10_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEXTURE2D_DESC", pack=False, align=None),
    "D3D10_TEXTURE3D_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Usage", SimTypeInt(signed=False, label="D3D10_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D10_TEXTURE3D_DESC", pack=False, align=None),
    "D3D10_TEXTURECUBE_FACE": SimTypeInt(signed=False, label="D3D10_TEXTURECUBE_FACE"),
    "D3D10_TEXTURE_ADDRESS_MODE": SimTypeInt(signed=False, label="D3D10_TEXTURE_ADDRESS_MODE"),
    "D3D10_USAGE": SimTypeInt(signed=False, label="D3D10_USAGE"),
    "D3D10_VIEWPORT": SimStruct(OrderedDict((("TopLeftX", SimTypeInt(signed=True, label="Int32")), ("TopLeftY", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("MinDepth", SimTypeFloat(size=32)), ("MaxDepth", SimTypeFloat(size=32)),)), name="D3D10_VIEWPORT", pack=False, align=None),
    "D3D11_1_CREATE_DEVICE_CONTEXT_STATE_FLAG": SimTypeInt(signed=False, label="D3D11_1_CREATE_DEVICE_CONTEXT_STATE_FLAG"),
    "D3D11_AES_CTR_IV": SimStruct(OrderedDict((("IV", SimTypeLongLong(signed=False, label="UInt64")), ("Count", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_AES_CTR_IV", pack=False, align=None),
    "D3D11_ASYNC_GETDATA_FLAG": SimTypeInt(signed=False, label="D3D11_ASYNC_GETDATA_FLAG"),
    "D3D11_AUTHENTICATED_CHANNEL_TYPE": SimTypeInt(signed=False, label="D3D11_AUTHENTICATED_CHANNEL_TYPE"),
    "D3D11_AUTHENTICATED_CONFIGURE_ACCESSIBLE_ENCRYPTION_INPUT": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3D11_AUTHENTICATED_CONFIGURE_INPUT", SimStruct)), ("EncryptionGuid", SimTypeBottom(label="Guid")),)), name="D3D11_AUTHENTICATED_CONFIGURE_ACCESSIBLE_ENCRYPTION_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_CONFIGURE_CRYPTO_SESSION_INPUT": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3D11_AUTHENTICATED_CONFIGURE_INPUT", SimStruct)), ("DecoderHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D11_AUTHENTICATED_CONFIGURE_CRYPTO_SESSION_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_CONFIGURE_INITIALIZE_INPUT": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3D11_AUTHENTICATED_CONFIGURE_INPUT", SimStruct)), ("StartSequenceQuery", SimTypeInt(signed=False, label="UInt32")), ("StartSequenceConfigure", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_CONFIGURE_INITIALIZE_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_CONFIGURE_INPUT": SimStruct(OrderedDict((("omac", SimTypeRef("D3D11_OMAC", SimStruct)), ("ConfigureType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_CONFIGURE_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_CONFIGURE_OUTPUT": SimStruct(OrderedDict((("omac", SimTypeRef("D3D11_OMAC", SimStruct)), ("ConfigureType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("ReturnCode", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_AUTHENTICATED_CONFIGURE_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_CONFIGURE_PROTECTION_INPUT": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3D11_AUTHENTICATED_CONFIGURE_INPUT", SimStruct)), ("Protections", SimUnion({"Flags": SimTypeRef("_Flags_e__Struct", SimStruct), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D11_AUTHENTICATED_CONFIGURE_PROTECTION_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_CONFIGURE_SHARED_RESOURCE_INPUT": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3D11_AUTHENTICATED_CONFIGURE_INPUT", SimStruct)), ("ProcessType", SimTypeInt(signed=False, label="D3D11_AUTHENTICATED_PROCESS_IDENTIFIER_TYPE")), ("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AllowAccess", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_AUTHENTICATED_CONFIGURE_SHARED_RESOURCE_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_PROCESS_IDENTIFIER_TYPE": SimTypeInt(signed=False, label="D3D11_AUTHENTICATED_PROCESS_IDENTIFIER_TYPE"),
    "D3D11_AUTHENTICATED_PROTECTION_FLAGS": SimUnion({"Flags": SimTypeRef("_Flags_e__Struct", SimStruct), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_ENCRYPTION_GUID_COUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("EncryptionGuidCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_ENCRYPTION_GUID_COUNT_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_ENCRYPTION_GUID_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3D11_AUTHENTICATED_QUERY_INPUT", SimStruct)), ("EncryptionGuidIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_ENCRYPTION_GUID_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_ENCRYPTION_GUID_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("EncryptionGuidIndex", SimTypeInt(signed=False, label="UInt32")), ("EncryptionGuid", SimTypeBottom(label="Guid")),)), name="D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_ENCRYPTION_GUID_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("BusType", SimTypeInt(signed=False, label="D3D11_BUS_TYPE")), ("AccessibleInContiguousBlocks", SimTypeInt(signed=True, label="Int32")), ("AccessibleInNonContiguousBlocks", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_AUTHENTICATED_QUERY_ACCESSIBILITY_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_CHANNEL_TYPE_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("ChannelType", SimTypeInt(signed=False, label="D3D11_AUTHENTICATED_CHANNEL_TYPE")),)), name="D3D11_AUTHENTICATED_QUERY_CHANNEL_TYPE_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_CRYPTO_SESSION_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3D11_AUTHENTICATED_QUERY_INPUT", SimStruct)), ("DecoderHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D11_AUTHENTICATED_QUERY_CRYPTO_SESSION_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_CRYPTO_SESSION_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("DecoderHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D11_AUTHENTICATED_QUERY_CRYPTO_SESSION_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_CURRENT_ACCESSIBILITY_ENCRYPTION_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("EncryptionGuid", SimTypeBottom(label="Guid")),)), name="D3D11_AUTHENTICATED_QUERY_CURRENT_ACCESSIBILITY_ENCRYPTION_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_DEVICE_HANDLE_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D11_AUTHENTICATED_QUERY_DEVICE_HANDLE_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_INPUT": SimStruct(OrderedDict((("QueryType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_OUTPUT": SimStruct(OrderedDict((("omac", SimTypeRef("D3D11_OMAC", SimStruct)), ("QueryType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("ReturnCode", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_AUTHENTICATED_QUERY_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_COUNT_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3D11_AUTHENTICATED_QUERY_INPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_COUNT_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_COUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("OutputIDCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_COUNT_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3D11_AUTHENTICATED_QUERY_INPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("OutputIDIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("OutputIDIndex", SimTypeInt(signed=False, label="UInt32")), ("OutputID", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_AUTHENTICATED_QUERY_OUTPUT_ID_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_PROTECTION_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("ProtectionFlags", SimUnion({"Flags": SimTypeRef("_Flags_e__Struct", SimStruct), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D11_AUTHENTICATED_QUERY_PROTECTION_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_RESTRICTED_SHARED_RESOURCE_PROCESS_COUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("RestrictedSharedResourceProcessCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_RESTRICTED_SHARED_RESOURCE_PROCESS_COUNT_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_RESTRICTED_SHARED_RESOURCE_PROCESS_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3D11_AUTHENTICATED_QUERY_INPUT", SimStruct)), ("ProcessIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_RESTRICTED_SHARED_RESOURCE_PROCESS_INPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_RESTRICTED_SHARED_RESOURCE_PROCESS_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("ProcessIndex", SimTypeInt(signed=False, label="UInt32")), ("ProcessIdentifier", SimTypeInt(signed=False, label="D3D11_AUTHENTICATED_PROCESS_IDENTIFIER_TYPE")), ("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D11_AUTHENTICATED_QUERY_RESTRICTED_SHARED_RESOURCE_PROCESS_OUTPUT", pack=False, align=None),
    "D3D11_AUTHENTICATED_QUERY_UNRESTRICTED_PROTECTED_SHARED_RESOURCE_COUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3D11_AUTHENTICATED_QUERY_OUTPUT", SimStruct)), ("UnrestrictedProtectedSharedResourceCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_AUTHENTICATED_QUERY_UNRESTRICTED_PROTECTED_SHARED_RESOURCE_COUNT_OUTPUT", pack=False, align=None),
    "D3D11_BIND_FLAG": SimTypeInt(signed=False, label="D3D11_BIND_FLAG"),
    "D3D11_BLEND": SimTypeInt(signed=False, label="D3D11_BLEND"),
    "D3D11_BLEND_DESC": SimStruct(OrderedDict((("AlphaToCoverageEnable", SimTypeInt(signed=True, label="Int32")), ("IndependentBlendEnable", SimTypeInt(signed=True, label="Int32")), ("RenderTarget", SimTypeArray(SimTypeRef("D3D11_RENDER_TARGET_BLEND_DESC", SimStruct), 8)),)), name="D3D11_BLEND_DESC", pack=False, align=None),
    "D3D11_BLEND_DESC1": SimStruct(OrderedDict((("AlphaToCoverageEnable", SimTypeInt(signed=True, label="Int32")), ("IndependentBlendEnable", SimTypeInt(signed=True, label="Int32")), ("RenderTarget", SimTypeArray(SimTypeRef("D3D11_RENDER_TARGET_BLEND_DESC1", SimStruct), 8)),)), name="D3D11_BLEND_DESC1", pack=False, align=None),
    "D3D11_BLEND_OP": SimTypeInt(signed=False, label="D3D11_BLEND_OP"),
    "D3D11_BOX": SimStruct(OrderedDict((("left", SimTypeInt(signed=False, label="UInt32")), ("top", SimTypeInt(signed=False, label="UInt32")), ("front", SimTypeInt(signed=False, label="UInt32")), ("right", SimTypeInt(signed=False, label="UInt32")), ("bottom", SimTypeInt(signed=False, label="UInt32")), ("back", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_BOX", pack=False, align=None),
    "D3D11_BUFFEREX_SRV": SimStruct(OrderedDict((("FirstElement", SimTypeInt(signed=False, label="UInt32")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_BUFFEREX_SRV", pack=False, align=None),
    "D3D11_BUFFEREX_SRV_FLAG": SimTypeInt(signed=False, label="D3D11_BUFFEREX_SRV_FLAG"),
    "D3D11_BUFFER_DESC": SimStruct(OrderedDict((("ByteWidth", SimTypeInt(signed=False, label="UInt32")), ("Usage", SimTypeInt(signed=False, label="D3D11_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")), ("StructureByteStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_BUFFER_DESC", pack=False, align=None),
    "D3D11_BUFFER_RTV": SimStruct(OrderedDict((("Anonymous1", SimUnion({"FirstElement": SimTypeInt(signed=False, label="UInt32"), "ElementOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"NumElements": SimTypeInt(signed=False, label="UInt32"), "ElementWidth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D11_BUFFER_RTV", pack=False, align=None),
    "D3D11_BUFFER_SRV": SimStruct(OrderedDict((("Anonymous1", SimUnion({"FirstElement": SimTypeInt(signed=False, label="UInt32"), "ElementOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"NumElements": SimTypeInt(signed=False, label="UInt32"), "ElementWidth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D11_BUFFER_SRV", pack=False, align=None),
    "D3D11_BUFFER_UAV": SimStruct(OrderedDict((("FirstElement", SimTypeInt(signed=False, label="UInt32")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_BUFFER_UAV", pack=False, align=None),
    "D3D11_BUFFER_UAV_FLAG": SimTypeInt(signed=False, label="D3D11_BUFFER_UAV_FLAG"),
    "D3D11_BUS_TYPE": SimTypeInt(signed=False, label="D3D11_BUS_TYPE"),
    "D3D11_CHECK_MULTISAMPLE_QUALITY_LEVELS_FLAG": SimTypeInt(signed=False, label="D3D11_CHECK_MULTISAMPLE_QUALITY_LEVELS_FLAG"),
    "D3D11_CLASS_INSTANCE_DESC": SimStruct(OrderedDict((("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("InstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("TypeId", SimTypeInt(signed=False, label="UInt32")), ("ConstantBuffer", SimTypeInt(signed=False, label="UInt32")), ("BaseConstantBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("BaseTexture", SimTypeInt(signed=False, label="UInt32")), ("BaseSampler", SimTypeInt(signed=False, label="UInt32")), ("Created", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_CLASS_INSTANCE_DESC", pack=False, align=None),
    "D3D11_CLEAR_FLAG": SimTypeInt(signed=False, label="D3D11_CLEAR_FLAG"),
    "D3D11_COLOR_WRITE_ENABLE": SimTypeInt(signed=False, label="D3D11_COLOR_WRITE_ENABLE"),
    "D3D11_COMPARISON_FUNC": SimTypeInt(signed=False, label="D3D11_COMPARISON_FUNC"),
    "D3D11_COMPUTE_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Invocation", SimTypeLongLong(signed=False, label="UInt64")), ("ThreadIDInGroup", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("ThreadGroupID", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="D3D11_COMPUTE_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_CONSERVATIVE_RASTERIZATION_MODE": SimTypeInt(signed=False, label="D3D11_CONSERVATIVE_RASTERIZATION_MODE"),
    "D3D11_CONSERVATIVE_RASTERIZATION_TIER": SimTypeInt(signed=False, label="D3D11_CONSERVATIVE_RASTERIZATION_TIER"),
    "D3D11_CONTENT_PROTECTION_CAPS": SimTypeInt(signed=False, label="D3D11_CONTENT_PROTECTION_CAPS"),
    "D3D11_CONTEXT_TYPE": SimTypeInt(signed=False, label="D3D11_CONTEXT_TYPE"),
    "D3D11_COPY_FLAGS": SimTypeInt(signed=False, label="D3D11_COPY_FLAGS"),
    "D3D11_COUNTER": SimTypeInt(signed=False, label="D3D11_COUNTER"),
    "D3D11_COUNTER_DESC": SimStruct(OrderedDict((("Counter", SimTypeInt(signed=False, label="D3D11_COUNTER")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_COUNTER_DESC", pack=False, align=None),
    "D3D11_COUNTER_INFO": SimStruct(OrderedDict((("LastDeviceDependentCounter", SimTypeInt(signed=False, label="D3D11_COUNTER")), ("NumSimultaneousCounters", SimTypeInt(signed=False, label="UInt32")), ("NumDetectableParallelUnits", SimTypeChar(label="Byte")),)), name="D3D11_COUNTER_INFO", pack=False, align=None),
    "D3D11_COUNTER_TYPE": SimTypeInt(signed=False, label="D3D11_COUNTER_TYPE"),
    "D3D11_CPU_ACCESS_FLAG": SimTypeInt(signed=False, label="D3D11_CPU_ACCESS_FLAG"),
    "D3D11_CREATE_DEVICE_FLAG": SimTypeInt(signed=False, label="D3D11_CREATE_DEVICE_FLAG"),
    "D3D11_CRYPTO_SESSION_KEY_EXCHANGE_FLAGS": SimTypeInt(signed=False, label="D3D11_CRYPTO_SESSION_KEY_EXCHANGE_FLAGS"),
    "D3D11_CRYPTO_SESSION_STATUS": SimTypeInt(signed=False, label="D3D11_CRYPTO_SESSION_STATUS"),
    "D3D11_CULL_MODE": SimTypeInt(signed=False, label="D3D11_CULL_MODE"),
    "D3D11_DEPTH_STENCILOP_DESC": SimStruct(OrderedDict((("StencilFailOp", SimTypeInt(signed=False, label="D3D11_STENCIL_OP")), ("StencilDepthFailOp", SimTypeInt(signed=False, label="D3D11_STENCIL_OP")), ("StencilPassOp", SimTypeInt(signed=False, label="D3D11_STENCIL_OP")), ("StencilFunc", SimTypeInt(signed=False, label="D3D11_COMPARISON_FUNC")),)), name="D3D11_DEPTH_STENCILOP_DESC", pack=False, align=None),
    "D3D11_DEPTH_STENCIL_DESC": SimStruct(OrderedDict((("DepthEnable", SimTypeInt(signed=True, label="Int32")), ("DepthWriteMask", SimTypeInt(signed=False, label="D3D11_DEPTH_WRITE_MASK")), ("DepthFunc", SimTypeInt(signed=False, label="D3D11_COMPARISON_FUNC")), ("StencilEnable", SimTypeInt(signed=True, label="Int32")), ("StencilReadMask", SimTypeChar(label="Byte")), ("StencilWriteMask", SimTypeChar(label="Byte")), ("FrontFace", SimTypeRef("D3D11_DEPTH_STENCILOP_DESC", SimStruct)), ("BackFace", SimTypeRef("D3D11_DEPTH_STENCILOP_DESC", SimStruct)),)), name="D3D11_DEPTH_STENCIL_DESC", pack=False, align=None),
    "D3D11_DEPTH_STENCIL_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_DSV_DIMENSION")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Texture1D": SimTypeRef("D3D11_TEX1D_DSV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_DSV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_DSV", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_DSV", SimStruct), "Texture2DMS": SimTypeRef("D3D11_TEX2DMS_DSV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D11_TEX2DMS_ARRAY_DSV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_DEPTH_STENCIL_VIEW_DESC", pack=False, align=None),
    "D3D11_DEPTH_WRITE_MASK": SimTypeInt(signed=False, label="D3D11_DEPTH_WRITE_MASK"),
    "D3D11_DEVICE_CONTEXT_TYPE": SimTypeInt(signed=False, label="D3D11_DEVICE_CONTEXT_TYPE"),
    "D3D11_DOMAIN_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Invocation", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_DOMAIN_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_DRAW_INDEXED_INSTANCED_INDIRECT_ARGS": SimStruct(OrderedDict((("IndexCountPerInstance", SimTypeInt(signed=False, label="UInt32")), ("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("StartIndexLocation", SimTypeInt(signed=False, label="UInt32")), ("BaseVertexLocation", SimTypeInt(signed=True, label="Int32")), ("StartInstanceLocation", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_DRAW_INDEXED_INSTANCED_INDIRECT_ARGS", pack=False, align=None),
    "D3D11_DRAW_INSTANCED_INDIRECT_ARGS": SimStruct(OrderedDict((("VertexCountPerInstance", SimTypeInt(signed=False, label="UInt32")), ("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("StartVertexLocation", SimTypeInt(signed=False, label="UInt32")), ("StartInstanceLocation", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_DRAW_INSTANCED_INDIRECT_ARGS", pack=False, align=None),
    "D3D11_DSV_DIMENSION": SimTypeInt(signed=False, label="D3D11_DSV_DIMENSION"),
    "D3D11_DSV_FLAG": SimTypeInt(signed=False, label="D3D11_DSV_FLAG"),
    "D3D11_ENCRYPTED_BLOCK_INFO": SimStruct(OrderedDict((("NumEncryptedBytesAtBeginning", SimTypeInt(signed=False, label="UInt32")), ("NumBytesInSkipPattern", SimTypeInt(signed=False, label="UInt32")), ("NumBytesInEncryptPattern", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_ENCRYPTED_BLOCK_INFO", pack=False, align=None),
    "D3D11_FEATURE": SimTypeInt(signed=False, label="D3D11_FEATURE"),
    "D3D11_FEATURE_DATA_ARCHITECTURE_INFO": SimStruct(OrderedDict((("TileBasedDeferredRenderer", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_ARCHITECTURE_INFO", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D10_X_HARDWARE_OPTIONS": SimStruct(OrderedDict((("ComputeShaders_Plus_RawAndStructuredBuffers_Via_Shader_4_x", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D10_X_HARDWARE_OPTIONS", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D11_OPTIONS": SimStruct(OrderedDict((("OutputMergerLogicOp", SimTypeInt(signed=True, label="Int32")), ("UAVOnlyRenderingForcedSampleCount", SimTypeInt(signed=True, label="Int32")), ("DiscardAPIsSeenByDriver", SimTypeInt(signed=True, label="Int32")), ("FlagsForUpdateAndCopySeenByDriver", SimTypeInt(signed=True, label="Int32")), ("ClearView", SimTypeInt(signed=True, label="Int32")), ("CopyWithOverlap", SimTypeInt(signed=True, label="Int32")), ("ConstantBufferPartialUpdate", SimTypeInt(signed=True, label="Int32")), ("ConstantBufferOffsetting", SimTypeInt(signed=True, label="Int32")), ("MapNoOverwriteOnDynamicConstantBuffer", SimTypeInt(signed=True, label="Int32")), ("MapNoOverwriteOnDynamicBufferSRV", SimTypeInt(signed=True, label="Int32")), ("MultisampleRTVWithForcedSampleCountOne", SimTypeInt(signed=True, label="Int32")), ("SAD4ShaderInstructions", SimTypeInt(signed=True, label="Int32")), ("ExtendedDoublesShaderInstructions", SimTypeInt(signed=True, label="Int32")), ("ExtendedResourceSharing", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D11_OPTIONS", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D11_OPTIONS1": SimStruct(OrderedDict((("TiledResourcesTier", SimTypeInt(signed=False, label="D3D11_TILED_RESOURCES_TIER")), ("MinMaxFiltering", SimTypeInt(signed=True, label="Int32")), ("ClearViewAlsoSupportsDepthOnlyFormats", SimTypeInt(signed=True, label="Int32")), ("MapOnDefaultBuffers", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D11_OPTIONS1", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D11_OPTIONS2": SimStruct(OrderedDict((("PSSpecifiedStencilRefSupported", SimTypeInt(signed=True, label="Int32")), ("TypedUAVLoadAdditionalFormats", SimTypeInt(signed=True, label="Int32")), ("ROVsSupported", SimTypeInt(signed=True, label="Int32")), ("ConservativeRasterizationTier", SimTypeInt(signed=False, label="D3D11_CONSERVATIVE_RASTERIZATION_TIER")), ("TiledResourcesTier", SimTypeInt(signed=False, label="D3D11_TILED_RESOURCES_TIER")), ("MapOnDefaultTextures", SimTypeInt(signed=True, label="Int32")), ("StandardSwizzle", SimTypeInt(signed=True, label="Int32")), ("UnifiedMemoryArchitecture", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D11_OPTIONS2", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D11_OPTIONS3": SimStruct(OrderedDict((("VPAndRTArrayIndexFromAnyShaderFeedingRasterizer", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D11_OPTIONS3", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D11_OPTIONS4": SimStruct(OrderedDict((("ExtendedNV12SharedTextureSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D11_OPTIONS4", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D11_OPTIONS5": SimStruct(OrderedDict((("SharedResourceTier", SimTypeInt(signed=False, label="D3D11_SHARED_RESOURCE_TIER")),)), name="D3D11_FEATURE_DATA_D3D11_OPTIONS5", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D9_OPTIONS": SimStruct(OrderedDict((("FullNonPow2TextureSupport", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D9_OPTIONS", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D9_OPTIONS1": SimStruct(OrderedDict((("FullNonPow2TextureSupported", SimTypeInt(signed=True, label="Int32")), ("DepthAsTextureWithLessEqualComparisonFilterSupported", SimTypeInt(signed=True, label="Int32")), ("SimpleInstancingSupported", SimTypeInt(signed=True, label="Int32")), ("TextureCubeFaceRenderTargetWithNonCubeDepthStencilSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D9_OPTIONS1", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D9_SHADOW_SUPPORT": SimStruct(OrderedDict((("SupportsDepthAsTextureWithLessEqualComparisonFilter", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D9_SHADOW_SUPPORT", pack=False, align=None),
    "D3D11_FEATURE_DATA_D3D9_SIMPLE_INSTANCING_SUPPORT": SimStruct(OrderedDict((("SimpleInstancingSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_D3D9_SIMPLE_INSTANCING_SUPPORT", pack=False, align=None),
    "D3D11_FEATURE_DATA_DISPLAYABLE": SimStruct(OrderedDict((("DisplayableTexture", SimTypeInt(signed=True, label="Int32")), ("SharedResourceTier", SimTypeInt(signed=False, label="D3D11_SHARED_RESOURCE_TIER")),)), name="D3D11_FEATURE_DATA_DISPLAYABLE", pack=False, align=None),
    "D3D11_FEATURE_DATA_DOUBLES": SimStruct(OrderedDict((("DoublePrecisionFloatShaderOps", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_DOUBLES", pack=False, align=None),
    "D3D11_FEATURE_DATA_FORMAT_SUPPORT": SimStruct(OrderedDict((("InFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("OutFormatSupport", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_FEATURE_DATA_FORMAT_SUPPORT", pack=False, align=None),
    "D3D11_FEATURE_DATA_FORMAT_SUPPORT2": SimStruct(OrderedDict((("InFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("OutFormatSupport2", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_FEATURE_DATA_FORMAT_SUPPORT2", pack=False, align=None),
    "D3D11_FEATURE_DATA_GPU_VIRTUAL_ADDRESS_SUPPORT": SimStruct(OrderedDict((("MaxGPUVirtualAddressBitsPerResource", SimTypeInt(signed=False, label="UInt32")), ("MaxGPUVirtualAddressBitsPerProcess", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_FEATURE_DATA_GPU_VIRTUAL_ADDRESS_SUPPORT", pack=False, align=None),
    "D3D11_FEATURE_DATA_MARKER_SUPPORT": SimStruct(OrderedDict((("Profile", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_MARKER_SUPPORT", pack=False, align=None),
    "D3D11_FEATURE_DATA_SHADER_CACHE": SimStruct(OrderedDict((("SupportFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_FEATURE_DATA_SHADER_CACHE", pack=False, align=None),
    "D3D11_FEATURE_DATA_SHADER_MIN_PRECISION_SUPPORT": SimStruct(OrderedDict((("PixelShaderMinPrecision", SimTypeInt(signed=False, label="UInt32")), ("AllOtherShaderStagesMinPrecision", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_FEATURE_DATA_SHADER_MIN_PRECISION_SUPPORT", pack=False, align=None),
    "D3D11_FEATURE_DATA_THREADING": SimStruct(OrderedDict((("DriverConcurrentCreates", SimTypeInt(signed=True, label="Int32")), ("DriverCommandLists", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FEATURE_DATA_THREADING", pack=False, align=None),
    "D3D11_FEATURE_DATA_VIDEO_DECODER_HISTOGRAM": SimStruct(OrderedDict((("DecoderDesc", SimTypeRef("D3D11_VIDEO_DECODER_DESC", SimStruct)), ("Components", SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_FLAGS")), ("BinCount", SimTypeInt(signed=False, label="UInt32")), ("CounterBitDepth", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_FEATURE_DATA_VIDEO_DECODER_HISTOGRAM", pack=False, align=None),
    "D3D11_FEATURE_VIDEO": SimTypeInt(signed=False, label="D3D11_FEATURE_VIDEO"),
    "D3D11_FENCE_FLAG": SimTypeInt(signed=False, label="D3D11_FENCE_FLAG"),
    "D3D11_FILL_MODE": SimTypeInt(signed=False, label="D3D11_FILL_MODE"),
    "D3D11_FILTER": SimTypeInt(signed=False, label="D3D11_FILTER"),
    "D3D11_FILTER_REDUCTION_TYPE": SimTypeInt(signed=False, label="D3D11_FILTER_REDUCTION_TYPE"),
    "D3D11_FILTER_TYPE": SimTypeInt(signed=False, label="D3D11_FILTER_TYPE"),
    "D3D11_FORMAT_SUPPORT": SimTypeInt(signed=False, label="D3D11_FORMAT_SUPPORT"),
    "D3D11_FORMAT_SUPPORT2": SimTypeInt(signed=False, label="D3D11_FORMAT_SUPPORT2"),
    "D3D11_FUNCTION_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("BoundResources", SimTypeInt(signed=False, label="UInt32")), ("InstructionCount", SimTypeInt(signed=False, label="UInt32")), ("TempRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("TempArrayCount", SimTypeInt(signed=False, label="UInt32")), ("DefCount", SimTypeInt(signed=False, label="UInt32")), ("DclCount", SimTypeInt(signed=False, label="UInt32")), ("TextureNormalInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureLoadInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureCompInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureBiasInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureGradientInstructions", SimTypeInt(signed=False, label="UInt32")), ("FloatInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("IntInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("UintInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("StaticFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("MacroInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ArrayInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("MovInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("MovcInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ConversionInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("BitwiseInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("MinFeatureLevel", SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL")), ("RequiredFeatureFlags", SimTypeLongLong(signed=False, label="UInt64")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FunctionParameterCount", SimTypeInt(signed=True, label="Int32")), ("HasReturn", SimTypeInt(signed=True, label="Int32")), ("Has10Level9VertexShader", SimTypeInt(signed=True, label="Int32")), ("Has10Level9PixelShader", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_FUNCTION_DESC", pack=False, align=None),
    "D3D11_GEOMETRY_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Invocation", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_GEOMETRY_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_HULL_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Invocation", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_HULL_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_INFO_QUEUE_FILTER": SimStruct(OrderedDict((("AllowList", SimTypeRef("D3D11_INFO_QUEUE_FILTER_DESC", SimStruct)), ("DenyList", SimTypeRef("D3D11_INFO_QUEUE_FILTER_DESC", SimStruct)),)), name="D3D11_INFO_QUEUE_FILTER", pack=False, align=None),
    "D3D11_INFO_QUEUE_FILTER_DESC": SimStruct(OrderedDict((("NumCategories", SimTypeInt(signed=False, label="UInt32")), ("pCategoryList", SimTypePointer(SimTypeInt(signed=False, label="D3D11_MESSAGE_CATEGORY"), offset=0)), ("NumSeverities", SimTypeInt(signed=False, label="UInt32")), ("pSeverityList", SimTypePointer(SimTypeInt(signed=False, label="D3D11_MESSAGE_SEVERITY"), offset=0)), ("NumIDs", SimTypeInt(signed=False, label="UInt32")), ("pIDList", SimTypePointer(SimTypeInt(signed=False, label="D3D11_MESSAGE_ID"), offset=0)),)), name="D3D11_INFO_QUEUE_FILTER_DESC", pack=False, align=None),
    "D3D11_INPUT_CLASSIFICATION": SimTypeInt(signed=False, label="D3D11_INPUT_CLASSIFICATION"),
    "D3D11_INPUT_ELEMENT_DESC": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("InputSlot", SimTypeInt(signed=False, label="UInt32")), ("AlignedByteOffset", SimTypeInt(signed=False, label="UInt32")), ("InputSlotClass", SimTypeInt(signed=False, label="D3D11_INPUT_CLASSIFICATION")), ("InstanceDataStepRate", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_INPUT_ELEMENT_DESC", pack=False, align=None),
    "D3D11_KEY_EXCHANGE_HW_PROTECTION_DATA": SimStruct(OrderedDict((("HWProtectionFunctionID", SimTypeInt(signed=False, label="UInt32")), ("pInputData", SimTypePointer(SimTypeRef("D3D11_KEY_EXCHANGE_HW_PROTECTION_INPUT_DATA", SimStruct), offset=0)), ("pOutputData", SimTypePointer(SimTypeRef("D3D11_KEY_EXCHANGE_HW_PROTECTION_OUTPUT_DATA", SimStruct), offset=0)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_KEY_EXCHANGE_HW_PROTECTION_DATA", pack=False, align=None),
    "D3D11_KEY_EXCHANGE_HW_PROTECTION_INPUT_DATA": SimStruct(OrderedDict((("PrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("HWProtectionDataSize", SimTypeInt(signed=False, label="UInt32")), ("pbInput", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="D3D11_KEY_EXCHANGE_HW_PROTECTION_INPUT_DATA", pack=False, align=None),
    "D3D11_KEY_EXCHANGE_HW_PROTECTION_OUTPUT_DATA": SimStruct(OrderedDict((("PrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("MaxHWProtectionDataSize", SimTypeInt(signed=False, label="UInt32")), ("HWProtectionDataSize", SimTypeInt(signed=False, label="UInt32")), ("TransportTime", SimTypeLongLong(signed=False, label="UInt64")), ("ExecutionTime", SimTypeLongLong(signed=False, label="UInt64")), ("pbOutput", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="D3D11_KEY_EXCHANGE_HW_PROTECTION_OUTPUT_DATA", pack=False, align=None),
    "D3D11_LIBRARY_DESC": SimStruct(OrderedDict((("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FunctionCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_LIBRARY_DESC", pack=False, align=None),
    "D3D11_LOGIC_OP": SimTypeInt(signed=False, label="D3D11_LOGIC_OP"),
    "D3D11_MAP": SimTypeInt(signed=False, label="D3D11_MAP"),
    "D3D11_MAPPED_SUBRESOURCE": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RowPitch", SimTypeInt(signed=False, label="UInt32")), ("DepthPitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_MAPPED_SUBRESOURCE", pack=False, align=None),
    "D3D11_MAP_FLAG": SimTypeInt(signed=False, label="D3D11_MAP_FLAG"),
    "D3D11_MESSAGE": SimStruct(OrderedDict((("Category", SimTypeInt(signed=False, label="D3D11_MESSAGE_CATEGORY")), ("Severity", SimTypeInt(signed=False, label="D3D11_MESSAGE_SEVERITY")), ("ID", SimTypeInt(signed=False, label="D3D11_MESSAGE_ID")), ("pDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DescriptionByteLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D11_MESSAGE", pack=False, align=None),
    "D3D11_MESSAGE_CATEGORY": SimTypeInt(signed=False, label="D3D11_MESSAGE_CATEGORY"),
    "D3D11_MESSAGE_ID": SimTypeInt(signed=False, label="D3D11_MESSAGE_ID"),
    "D3D11_MESSAGE_SEVERITY": SimTypeInt(signed=False, label="D3D11_MESSAGE_SEVERITY"),
    "D3D11_OMAC": SimStruct(OrderedDict((("Omac", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="D3D11_OMAC", pack=False, align=None),
    "D3D11_PACKED_MIP_DESC": SimStruct(OrderedDict((("NumStandardMips", SimTypeChar(label="Byte")), ("NumPackedMips", SimTypeChar(label="Byte")), ("NumTilesForPackedMips", SimTypeInt(signed=False, label="UInt32")), ("StartTileIndexInOverallResource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_PACKED_MIP_DESC", pack=False, align=None),
    "D3D11_PARAMETER_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("InterpolationMode", SimTypeInt(signed=False, label="D3D_INTERPOLATION_MODE")), ("Flags", SimTypeInt(signed=False, label="D3D_PARAMETER_FLAGS")), ("FirstInRegister", SimTypeInt(signed=False, label="UInt32")), ("FirstInComponent", SimTypeInt(signed=False, label="UInt32")), ("FirstOutRegister", SimTypeInt(signed=False, label="UInt32")), ("FirstOutComponent", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_PARAMETER_DESC", pack=False, align=None),
    "D3D11_PIXEL_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Invocation", SimTypeLongLong(signed=False, label="UInt64")), ("X", SimTypeInt(signed=True, label="Int32")), ("Y", SimTypeInt(signed=True, label="Int32")), ("SampleMask", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_PIXEL_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_QUERY": SimTypeInt(signed=False, label="D3D11_QUERY"),
    "D3D11_QUERY_DATA_PIPELINE_STATISTICS": SimStruct(OrderedDict((("IAVertices", SimTypeLongLong(signed=False, label="UInt64")), ("IAPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("VSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("CInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("PSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("HSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("DSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CSInvocations", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_QUERY_DATA_PIPELINE_STATISTICS", pack=False, align=None),
    "D3D11_QUERY_DATA_SO_STATISTICS": SimStruct(OrderedDict((("NumPrimitivesWritten", SimTypeLongLong(signed=False, label="UInt64")), ("PrimitivesStorageNeeded", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_QUERY_DATA_SO_STATISTICS", pack=False, align=None),
    "D3D11_QUERY_DATA_TIMESTAMP_DISJOINT": SimStruct(OrderedDict((("Frequency", SimTypeLongLong(signed=False, label="UInt64")), ("Disjoint", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_QUERY_DATA_TIMESTAMP_DISJOINT", pack=False, align=None),
    "D3D11_QUERY_DESC": SimStruct(OrderedDict((("Query", SimTypeInt(signed=False, label="D3D11_QUERY")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_QUERY_DESC", pack=False, align=None),
    "D3D11_QUERY_DESC1": SimStruct(OrderedDict((("Query", SimTypeInt(signed=False, label="D3D11_QUERY")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")), ("ContextType", SimTypeInt(signed=False, label="D3D11_CONTEXT_TYPE")),)), name="D3D11_QUERY_DESC1", pack=False, align=None),
    "D3D11_QUERY_MISC_FLAG": SimTypeInt(signed=False, label="D3D11_QUERY_MISC_FLAG"),
    "D3D11_RAISE_FLAG": SimTypeInt(signed=False, label="D3D11_RAISE_FLAG"),
    "D3D11_RASTERIZER_DESC": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D11_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D11_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeInt(signed=True, label="Int32")), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("ScissorEnable", SimTypeInt(signed=True, label="Int32")), ("MultisampleEnable", SimTypeInt(signed=True, label="Int32")), ("AntialiasedLineEnable", SimTypeInt(signed=True, label="Int32")),)), name="D3D11_RASTERIZER_DESC", pack=False, align=None),
    "D3D11_RASTERIZER_DESC1": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D11_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D11_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeInt(signed=True, label="Int32")), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("ScissorEnable", SimTypeInt(signed=True, label="Int32")), ("MultisampleEnable", SimTypeInt(signed=True, label="Int32")), ("AntialiasedLineEnable", SimTypeInt(signed=True, label="Int32")), ("ForcedSampleCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_RASTERIZER_DESC1", pack=False, align=None),
    "D3D11_RASTERIZER_DESC2": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D11_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D11_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeInt(signed=True, label="Int32")), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("ScissorEnable", SimTypeInt(signed=True, label="Int32")), ("MultisampleEnable", SimTypeInt(signed=True, label="Int32")), ("AntialiasedLineEnable", SimTypeInt(signed=True, label="Int32")), ("ForcedSampleCount", SimTypeInt(signed=False, label="UInt32")), ("ConservativeRaster", SimTypeInt(signed=False, label="D3D11_CONSERVATIVE_RASTERIZATION_MODE")),)), name="D3D11_RASTERIZER_DESC2", pack=False, align=None),
    "D3D11_RENDER_TARGET_BLEND_DESC": SimStruct(OrderedDict((("BlendEnable", SimTypeInt(signed=True, label="Int32")), ("SrcBlend", SimTypeInt(signed=False, label="D3D11_BLEND")), ("DestBlend", SimTypeInt(signed=False, label="D3D11_BLEND")), ("BlendOp", SimTypeInt(signed=False, label="D3D11_BLEND_OP")), ("SrcBlendAlpha", SimTypeInt(signed=False, label="D3D11_BLEND")), ("DestBlendAlpha", SimTypeInt(signed=False, label="D3D11_BLEND")), ("BlendOpAlpha", SimTypeInt(signed=False, label="D3D11_BLEND_OP")), ("RenderTargetWriteMask", SimTypeChar(label="Byte")),)), name="D3D11_RENDER_TARGET_BLEND_DESC", pack=False, align=None),
    "D3D11_RENDER_TARGET_BLEND_DESC1": SimStruct(OrderedDict((("BlendEnable", SimTypeInt(signed=True, label="Int32")), ("LogicOpEnable", SimTypeInt(signed=True, label="Int32")), ("SrcBlend", SimTypeInt(signed=False, label="D3D11_BLEND")), ("DestBlend", SimTypeInt(signed=False, label="D3D11_BLEND")), ("BlendOp", SimTypeInt(signed=False, label="D3D11_BLEND_OP")), ("SrcBlendAlpha", SimTypeInt(signed=False, label="D3D11_BLEND")), ("DestBlendAlpha", SimTypeInt(signed=False, label="D3D11_BLEND")), ("BlendOpAlpha", SimTypeInt(signed=False, label="D3D11_BLEND_OP")), ("LogicOp", SimTypeInt(signed=False, label="D3D11_LOGIC_OP")), ("RenderTargetWriteMask", SimTypeChar(label="Byte")),)), name="D3D11_RENDER_TARGET_BLEND_DESC1", pack=False, align=None),
    "D3D11_RENDER_TARGET_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_RTV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D11_BUFFER_RTV", SimStruct), "Texture1D": SimTypeRef("D3D11_TEX1D_RTV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_RTV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_RTV", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_RTV", SimStruct), "Texture2DMS": SimTypeRef("D3D11_TEX2DMS_RTV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D11_TEX2DMS_ARRAY_RTV", SimStruct), "Texture3D": SimTypeRef("D3D11_TEX3D_RTV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_RENDER_TARGET_VIEW_DESC", pack=False, align=None),
    "D3D11_RENDER_TARGET_VIEW_DESC1": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_RTV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D11_BUFFER_RTV", SimStruct), "Texture1D": SimTypeRef("D3D11_TEX1D_RTV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_RTV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_RTV1", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_RTV1", SimStruct), "Texture2DMS": SimTypeRef("D3D11_TEX2DMS_RTV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D11_TEX2DMS_ARRAY_RTV", SimStruct), "Texture3D": SimTypeRef("D3D11_TEX3D_RTV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_RENDER_TARGET_VIEW_DESC1", pack=False, align=None),
    "D3D11_RESOURCE_DIMENSION": SimTypeInt(signed=False, label="D3D11_RESOURCE_DIMENSION"),
    "D3D11_RESOURCE_FLAGS": SimStruct(OrderedDict((("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("StructureByteStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_RESOURCE_FLAGS", pack=False, align=None),
    "D3D11_RESOURCE_MISC_FLAG": SimTypeInt(signed=False, label="D3D11_RESOURCE_MISC_FLAG"),
    "D3D11_RLDO_FLAGS": SimTypeInt(signed=False, label="D3D11_RLDO_FLAGS"),
    "D3D11_RTV_DIMENSION": SimTypeInt(signed=False, label="D3D11_RTV_DIMENSION"),
    "D3D11_SAMPLER_DESC": SimStruct(OrderedDict((("Filter", SimTypeInt(signed=False, label="D3D11_FILTER")), ("AddressU", SimTypeInt(signed=False, label="D3D11_TEXTURE_ADDRESS_MODE")), ("AddressV", SimTypeInt(signed=False, label="D3D11_TEXTURE_ADDRESS_MODE")), ("AddressW", SimTypeInt(signed=False, label="D3D11_TEXTURE_ADDRESS_MODE")), ("MipLODBias", SimTypeFloat(size=32)), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("ComparisonFunc", SimTypeInt(signed=False, label="D3D11_COMPARISON_FUNC")), ("BorderColor", SimTypeArray(SimTypeFloat(size=32), 4)), ("MinLOD", SimTypeFloat(size=32)), ("MaxLOD", SimTypeFloat(size=32)),)), name="D3D11_SAMPLER_DESC", pack=False, align=None),
    "D3D11_SHADER_BUFFER_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_CBUFFER_TYPE")), ("Variables", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_SHADER_BUFFER_DESC", pack=False, align=None),
    "D3D11_SHADER_CACHE_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D11_SHADER_CACHE_SUPPORT_FLAGS"),
    "D3D11_SHADER_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("BoundResources", SimTypeInt(signed=False, label="UInt32")), ("InputParameters", SimTypeInt(signed=False, label="UInt32")), ("OutputParameters", SimTypeInt(signed=False, label="UInt32")), ("InstructionCount", SimTypeInt(signed=False, label="UInt32")), ("TempRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("TempArrayCount", SimTypeInt(signed=False, label="UInt32")), ("DefCount", SimTypeInt(signed=False, label="UInt32")), ("DclCount", SimTypeInt(signed=False, label="UInt32")), ("TextureNormalInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureLoadInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureCompInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureBiasInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureGradientInstructions", SimTypeInt(signed=False, label="UInt32")), ("FloatInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("IntInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("UintInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("StaticFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("MacroInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ArrayInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("CutInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("EmitInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GSOutputTopology", SimTypeInt(signed=False, label="D3D_PRIMITIVE_TOPOLOGY")), ("GSMaxOutputVertexCount", SimTypeInt(signed=False, label="UInt32")), ("InputPrimitive", SimTypeInt(signed=False, label="D3D_PRIMITIVE")), ("PatchConstantParameters", SimTypeInt(signed=False, label="UInt32")), ("cGSInstanceCount", SimTypeInt(signed=False, label="UInt32")), ("cControlPoints", SimTypeInt(signed=False, label="UInt32")), ("HSOutputPrimitive", SimTypeInt(signed=False, label="D3D_TESSELLATOR_OUTPUT_PRIMITIVE")), ("HSPartitioning", SimTypeInt(signed=False, label="D3D_TESSELLATOR_PARTITIONING")), ("TessellatorDomain", SimTypeInt(signed=False, label="D3D_TESSELLATOR_DOMAIN")), ("cBarrierInstructions", SimTypeInt(signed=False, label="UInt32")), ("cInterlockedInstructions", SimTypeInt(signed=False, label="UInt32")), ("cTextureStoreInstructions", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_SHADER_DESC", pack=False, align=None),
    "D3D11_SHADER_INPUT_BIND_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_INPUT_TYPE")), ("BindPoint", SimTypeInt(signed=False, label="UInt32")), ("BindCount", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("ReturnType", SimTypeInt(signed=False, label="D3D_RESOURCE_RETURN_TYPE")), ("Dimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("NumSamples", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_SHADER_INPUT_BIND_DESC", pack=False, align=None),
    "D3D11_SHADER_MIN_PRECISION_SUPPORT": SimTypeInt(signed=False, label="D3D11_SHADER_MIN_PRECISION_SUPPORT"),
    "D3D11_SHADER_RESOURCE_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D11_BUFFER_SRV", SimStruct), "Texture1D": SimTypeRef("D3D11_TEX1D_SRV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_SRV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_SRV", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_SRV", SimStruct), "Texture2DMS": SimTypeRef("D3D11_TEX2DMS_SRV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D11_TEX2DMS_ARRAY_SRV", SimStruct), "Texture3D": SimTypeRef("D3D11_TEX3D_SRV", SimStruct), "TextureCube": SimTypeRef("D3D11_TEXCUBE_SRV", SimStruct), "TextureCubeArray": SimTypeRef("D3D11_TEXCUBE_ARRAY_SRV", SimStruct), "BufferEx": SimTypeRef("D3D11_BUFFEREX_SRV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_SHADER_RESOURCE_VIEW_DESC", pack=False, align=None),
    "D3D11_SHADER_RESOURCE_VIEW_DESC1": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D11_BUFFER_SRV", SimStruct), "Texture1D": SimTypeRef("D3D11_TEX1D_SRV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_SRV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_SRV1", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_SRV1", SimStruct), "Texture2DMS": SimTypeRef("D3D11_TEX2DMS_SRV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D11_TEX2DMS_ARRAY_SRV", SimStruct), "Texture3D": SimTypeRef("D3D11_TEX3D_SRV", SimStruct), "TextureCube": SimTypeRef("D3D11_TEXCUBE_SRV", SimStruct), "TextureCubeArray": SimTypeRef("D3D11_TEXCUBE_ARRAY_SRV", SimStruct), "BufferEx": SimTypeRef("D3D11_BUFFEREX_SRV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_SHADER_RESOURCE_VIEW_DESC1", pack=False, align=None),
    "D3D11_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D11_SHADER_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"VertexShaderTraceDesc": SimTypeRef("D3D11_VERTEX_SHADER_TRACE_DESC", SimStruct), "HullShaderTraceDesc": SimTypeRef("D3D11_HULL_SHADER_TRACE_DESC", SimStruct), "DomainShaderTraceDesc": SimTypeRef("D3D11_DOMAIN_SHADER_TRACE_DESC", SimStruct), "GeometryShaderTraceDesc": SimTypeRef("D3D11_GEOMETRY_SHADER_TRACE_DESC", SimStruct), "PixelShaderTraceDesc": SimTypeRef("D3D11_PIXEL_SHADER_TRACE_DESC", SimStruct), "ComputeShaderTraceDesc": SimTypeRef("D3D11_COMPUTE_SHADER_TRACE_DESC", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_SHADER_TRACKING_OPTIONS": SimTypeInt(signed=False, label="D3D11_SHADER_TRACKING_OPTIONS"),
    "D3D11_SHADER_TRACKING_RESOURCE_TYPE": SimTypeInt(signed=False, label="D3D11_SHADER_TRACKING_RESOURCE_TYPE"),
    "D3D11_SHADER_TYPE": SimTypeInt(signed=False, label="D3D11_SHADER_TYPE"),
    "D3D11_SHADER_TYPE_DESC": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypeInt(signed=False, label="UInt32")), ("Members", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3D11_SHADER_TYPE_DESC", pack=False, align=None),
    "D3D11_SHADER_VARIABLE_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StartOffset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("DefaultValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StartTexture", SimTypeInt(signed=False, label="UInt32")), ("TextureSize", SimTypeInt(signed=False, label="UInt32")), ("StartSampler", SimTypeInt(signed=False, label="UInt32")), ("SamplerSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_SHADER_VARIABLE_DESC", pack=False, align=None),
    "D3D11_SHADER_VERSION_TYPE": SimTypeInt(signed=False, label="D3D11_SHADER_VERSION_TYPE"),
    "D3D11_SHARED_RESOURCE_TIER": SimTypeInt(signed=False, label="D3D11_SHARED_RESOURCE_TIER"),
    "D3D11_SIGNATURE_PARAMETER_DESC": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("SystemValueType", SimTypeInt(signed=False, label="D3D_NAME")), ("ComponentType", SimTypeInt(signed=False, label="D3D_REGISTER_COMPONENT_TYPE")), ("Mask", SimTypeChar(label="Byte")), ("ReadWriteMask", SimTypeChar(label="Byte")), ("Stream", SimTypeInt(signed=False, label="UInt32")), ("MinPrecision", SimTypeInt(signed=False, label="D3D_MIN_PRECISION")),)), name="D3D11_SIGNATURE_PARAMETER_DESC", pack=False, align=None),
    "D3D11_SO_DECLARATION_ENTRY": SimStruct(OrderedDict((("Stream", SimTypeInt(signed=False, label="UInt32")), ("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("StartComponent", SimTypeChar(label="Byte")), ("ComponentCount", SimTypeChar(label="Byte")), ("OutputSlot", SimTypeChar(label="Byte")),)), name="D3D11_SO_DECLARATION_ENTRY", pack=False, align=None),
    "D3D11_STANDARD_MULTISAMPLE_QUALITY_LEVELS": SimTypeInt(signed=False, label="D3D11_STANDARD_MULTISAMPLE_QUALITY_LEVELS"),
    "D3D11_STENCIL_OP": SimTypeInt(signed=False, label="D3D11_STENCIL_OP"),
    "D3D11_SUBRESOURCE_DATA": SimStruct(OrderedDict((("pSysMem", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SysMemPitch", SimTypeInt(signed=False, label="UInt32")), ("SysMemSlicePitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_SUBRESOURCE_DATA", pack=False, align=None),
    "D3D11_SUBRESOURCE_TILING": SimStruct(OrderedDict((("WidthInTiles", SimTypeInt(signed=False, label="UInt32")), ("HeightInTiles", SimTypeShort(signed=False, label="UInt16")), ("DepthInTiles", SimTypeShort(signed=False, label="UInt16")), ("StartTileIndexInOverallResource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_SUBRESOURCE_TILING", pack=False, align=None),
    "D3D11_TEX1D_ARRAY_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_ARRAY_DSV", pack=False, align=None),
    "D3D11_TEX1D_ARRAY_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_ARRAY_RTV", pack=False, align=None),
    "D3D11_TEX1D_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_ARRAY_SRV", pack=False, align=None),
    "D3D11_TEX1D_ARRAY_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_ARRAY_UAV", pack=False, align=None),
    "D3D11_TEX1D_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_DSV", pack=False, align=None),
    "D3D11_TEX1D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_RTV", pack=False, align=None),
    "D3D11_TEX1D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_SRV", pack=False, align=None),
    "D3D11_TEX1D_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX1D_UAV", pack=False, align=None),
    "D3D11_TEX2DMS_ARRAY_DSV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2DMS_ARRAY_DSV", pack=False, align=None),
    "D3D11_TEX2DMS_ARRAY_RTV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2DMS_ARRAY_RTV", pack=False, align=None),
    "D3D11_TEX2DMS_ARRAY_SRV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2DMS_ARRAY_SRV", pack=False, align=None),
    "D3D11_TEX2DMS_DSV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2DMS_DSV", pack=False, align=None),
    "D3D11_TEX2DMS_RTV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2DMS_RTV", pack=False, align=None),
    "D3D11_TEX2DMS_SRV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2DMS_SRV", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_DSV", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_RTV", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_RTV1": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_RTV1", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_SRV", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_SRV1": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_SRV1", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_UAV", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_UAV1": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_UAV1", pack=False, align=None),
    "D3D11_TEX2D_ARRAY_VPOV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_ARRAY_VPOV", pack=False, align=None),
    "D3D11_TEX2D_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_DSV", pack=False, align=None),
    "D3D11_TEX2D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_RTV", pack=False, align=None),
    "D3D11_TEX2D_RTV1": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_RTV1", pack=False, align=None),
    "D3D11_TEX2D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_SRV", pack=False, align=None),
    "D3D11_TEX2D_SRV1": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_SRV1", pack=False, align=None),
    "D3D11_TEX2D_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_UAV", pack=False, align=None),
    "D3D11_TEX2D_UAV1": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_UAV1", pack=False, align=None),
    "D3D11_TEX2D_VDOV": SimStruct(OrderedDict((("ArraySlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_VDOV", pack=False, align=None),
    "D3D11_TEX2D_VPIV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_VPIV", pack=False, align=None),
    "D3D11_TEX2D_VPOV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX2D_VPOV", pack=False, align=None),
    "D3D11_TEX3D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstWSlice", SimTypeInt(signed=False, label="UInt32")), ("WSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX3D_RTV", pack=False, align=None),
    "D3D11_TEX3D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX3D_SRV", pack=False, align=None),
    "D3D11_TEX3D_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstWSlice", SimTypeInt(signed=False, label="UInt32")), ("WSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEX3D_UAV", pack=False, align=None),
    "D3D11_TEXCUBE_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("First2DArrayFace", SimTypeInt(signed=False, label="UInt32")), ("NumCubes", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEXCUBE_ARRAY_SRV", pack=False, align=None),
    "D3D11_TEXCUBE_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEXCUBE_SRV", pack=False, align=None),
    "D3D11_TEXTURE1D_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Usage", SimTypeInt(signed=False, label="D3D11_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEXTURE1D_DESC", pack=False, align=None),
    "D3D11_TEXTURE2D_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("Usage", SimTypeInt(signed=False, label="D3D11_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEXTURE2D_DESC", pack=False, align=None),
    "D3D11_TEXTURE2D_DESC1": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("Usage", SimTypeInt(signed=False, label="D3D11_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")), ("TextureLayout", SimTypeInt(signed=False, label="D3D11_TEXTURE_LAYOUT")),)), name="D3D11_TEXTURE2D_DESC1", pack=False, align=None),
    "D3D11_TEXTURE3D_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Usage", SimTypeInt(signed=False, label="D3D11_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TEXTURE3D_DESC", pack=False, align=None),
    "D3D11_TEXTURE3D_DESC1": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Usage", SimTypeInt(signed=False, label="D3D11_USAGE")), ("BindFlags", SimTypeInt(signed=False, label="UInt32")), ("CPUAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("MiscFlags", SimTypeInt(signed=False, label="UInt32")), ("TextureLayout", SimTypeInt(signed=False, label="D3D11_TEXTURE_LAYOUT")),)), name="D3D11_TEXTURE3D_DESC1", pack=False, align=None),
    "D3D11_TEXTURECUBE_FACE": SimTypeInt(signed=False, label="D3D11_TEXTURECUBE_FACE"),
    "D3D11_TEXTURE_ADDRESS_MODE": SimTypeInt(signed=False, label="D3D11_TEXTURE_ADDRESS_MODE"),
    "D3D11_TEXTURE_LAYOUT": SimTypeInt(signed=False, label="D3D11_TEXTURE_LAYOUT"),
    "D3D11_TILED_RESOURCES_TIER": SimTypeInt(signed=False, label="D3D11_TILED_RESOURCES_TIER"),
    "D3D11_TILED_RESOURCE_COORDINATE": SimStruct(OrderedDict((("X", SimTypeInt(signed=False, label="UInt32")), ("Y", SimTypeInt(signed=False, label="UInt32")), ("Z", SimTypeInt(signed=False, label="UInt32")), ("Subresource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TILED_RESOURCE_COORDINATE", pack=False, align=None),
    "D3D11_TILE_COPY_FLAG": SimTypeInt(signed=False, label="D3D11_TILE_COPY_FLAG"),
    "D3D11_TILE_MAPPING_FLAG": SimTypeInt(signed=False, label="D3D11_TILE_MAPPING_FLAG"),
    "D3D11_TILE_RANGE_FLAG": SimTypeInt(signed=False, label="D3D11_TILE_RANGE_FLAG"),
    "D3D11_TILE_REGION_SIZE": SimStruct(OrderedDict((("NumTiles", SimTypeInt(signed=False, label="UInt32")), ("bUseBox", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeShort(signed=False, label="UInt16")), ("Depth", SimTypeShort(signed=False, label="UInt16")),)), name="D3D11_TILE_REGION_SIZE", pack=False, align=None),
    "D3D11_TILE_SHAPE": SimStruct(OrderedDict((("WidthInTexels", SimTypeInt(signed=False, label="UInt32")), ("HeightInTexels", SimTypeInt(signed=False, label="UInt32")), ("DepthInTexels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_TILE_SHAPE", pack=False, align=None),
    "D3D11_TRACE_GS_INPUT_PRIMITIVE": SimTypeInt(signed=False, label="D3D11_TRACE_GS_INPUT_PRIMITIVE"),
    "D3D11_TRACE_REGISTER": SimStruct(OrderedDict((("RegType", SimTypeInt(signed=False, label="D3D11_TRACE_REGISTER_TYPE")), ("Anonymous", SimUnion({"Index1D": SimTypeShort(signed=False, label="UInt16"), "Index2D": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)}, name="<anon>", label="None")), ("OperandIndex", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")),)), name="D3D11_TRACE_REGISTER", pack=False, align=None),
    "D3D11_TRACE_REGISTER_TYPE": SimTypeInt(signed=False, label="D3D11_TRACE_REGISTER_TYPE"),
    "D3D11_TRACE_STATS": SimStruct(OrderedDict((("TraceDesc", SimTypeRef("D3D11_SHADER_TRACE_DESC", SimStruct)), ("NumInvocationsInStamp", SimTypeChar(label="Byte")), ("TargetStampIndex", SimTypeChar(label="Byte")), ("NumTraceSteps", SimTypeInt(signed=False, label="UInt32")), ("InputMask", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("OutputMask", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("NumTemps", SimTypeShort(signed=False, label="UInt16")), ("MaxIndexableTempIndex", SimTypeShort(signed=False, label="UInt16")), ("IndexableTempSize", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 4096)), ("ImmediateConstantBufferSize", SimTypeShort(signed=False, label="UInt16")), ("PixelPosition", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("PixelCoverageMask", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PixelDiscardedMask", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PixelCoverageMaskAfterShader", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PixelCoverageMaskAfterA2CSampleMask", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PixelCoverageMaskAfterA2CSampleMaskDepth", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PixelCoverageMaskAfterA2CSampleMaskDepthStencil", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PSOutputsDepth", SimTypeInt(signed=True, label="Int32")), ("PSOutputsMask", SimTypeInt(signed=True, label="Int32")), ("GSInputPrimitive", SimTypeInt(signed=False, label="D3D11_TRACE_GS_INPUT_PRIMITIVE")), ("GSInputsPrimitiveID", SimTypeInt(signed=True, label="Int32")), ("HSOutputPatchConstantMask", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("DSInputPatchConstantMask", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="D3D11_TRACE_STATS", pack=False, align=None),
    "D3D11_TRACE_STEP": SimStruct(OrderedDict((("ID", SimTypeInt(signed=False, label="UInt32")), ("InstructionActive", SimTypeInt(signed=True, label="Int32")), ("NumRegistersWritten", SimTypeChar(label="Byte")), ("NumRegistersRead", SimTypeChar(label="Byte")), ("MiscOperations", SimTypeShort(signed=False, label="UInt16")), ("OpcodeType", SimTypeInt(signed=False, label="UInt32")), ("CurrentGlobalCycle", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_TRACE_STEP", pack=False, align=None),
    "D3D11_TRACE_VALUE": SimStruct(OrderedDict((("Bits", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ValidMask", SimTypeChar(label="Byte")),)), name="D3D11_TRACE_VALUE", pack=False, align=None),
    "D3D11_UAV_DIMENSION": SimTypeInt(signed=False, label="D3D11_UAV_DIMENSION"),
    "D3D11_UNORDERED_ACCESS_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_UAV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D11_BUFFER_UAV", SimStruct), "Texture1D": SimTypeRef("D3D11_TEX1D_UAV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_UAV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_UAV", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_UAV", SimStruct), "Texture3D": SimTypeRef("D3D11_TEX3D_UAV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_UNORDERED_ACCESS_VIEW_DESC", pack=False, align=None),
    "D3D11_UNORDERED_ACCESS_VIEW_DESC1": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_UAV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D11_BUFFER_UAV", SimStruct), "Texture1D": SimTypeRef("D3D11_TEX1D_UAV", SimStruct), "Texture1DArray": SimTypeRef("D3D11_TEX1D_ARRAY_UAV", SimStruct), "Texture2D": SimTypeRef("D3D11_TEX2D_UAV1", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_UAV1", SimStruct), "Texture3D": SimTypeRef("D3D11_TEX3D_UAV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_UNORDERED_ACCESS_VIEW_DESC1", pack=False, align=None),
    "D3D11_USAGE": SimTypeInt(signed=False, label="D3D11_USAGE"),
    "D3D11_VDOV_DIMENSION": SimTypeInt(signed=False, label="D3D11_VDOV_DIMENSION"),
    "D3D11_VERTEX_SHADER_TRACE_DESC": SimStruct(OrderedDict((("Invocation", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_VERTEX_SHADER_TRACE_DESC", pack=False, align=None),
    "D3D11_VIDEO_COLOR": SimStruct(OrderedDict((("Anonymous", SimUnion({"YCbCr": SimTypeRef("D3D11_VIDEO_COLOR_YCbCrA", SimStruct), "RGBA": SimTypeRef("D3D11_VIDEO_COLOR_RGBA", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_VIDEO_COLOR", pack=False, align=None),
    "D3D11_VIDEO_COLOR_RGBA": SimStruct(OrderedDict((("R", SimTypeFloat(size=32)), ("G", SimTypeFloat(size=32)), ("B", SimTypeFloat(size=32)), ("A", SimTypeFloat(size=32)),)), name="D3D11_VIDEO_COLOR_RGBA", pack=False, align=None),
    "D3D11_VIDEO_COLOR_YCbCrA": SimStruct(OrderedDict((("Y", SimTypeFloat(size=32)), ("Cb", SimTypeFloat(size=32)), ("Cr", SimTypeFloat(size=32)), ("A", SimTypeFloat(size=32)),)), name="D3D11_VIDEO_COLOR_YCbCrA", pack=False, align=None),
    "D3D11_VIDEO_CONTENT_PROTECTION_CAPS": SimStruct(OrderedDict((("Caps", SimTypeInt(signed=False, label="UInt32")), ("KeyExchangeTypeCount", SimTypeInt(signed=False, label="UInt32")), ("BlockAlignmentSize", SimTypeInt(signed=False, label="UInt32")), ("ProtectedMemorySize", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D11_VIDEO_CONTENT_PROTECTION_CAPS", pack=False, align=None),
    "D3D11_VIDEO_DECODER_BEGIN_FRAME_CRYPTO_SESSION": SimStruct(OrderedDict((("pCryptoSession", SimTypeBottom(label="ID3D11CryptoSession")), ("BlobSize", SimTypeInt(signed=False, label="UInt32")), ("pBlob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pKeyInfoId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("PrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3D11_VIDEO_DECODER_BEGIN_FRAME_CRYPTO_SESSION", pack=False, align=None),
    "D3D11_VIDEO_DECODER_BUFFER_DESC": SimStruct(OrderedDict((("BufferType", SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_BUFFER_TYPE")), ("BufferIndex", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("FirstMBaddress", SimTypeInt(signed=False, label="UInt32")), ("NumMBsInBuffer", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Stride", SimTypeInt(signed=False, label="UInt32")), ("ReservedBits", SimTypeInt(signed=False, label="UInt32")), ("pIV", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("IVSize", SimTypeInt(signed=False, label="UInt32")), ("PartialEncryption", SimTypeInt(signed=True, label="Int32")), ("EncryptedBlockInfo", SimTypeRef("D3D11_ENCRYPTED_BLOCK_INFO", SimStruct)),)), name="D3D11_VIDEO_DECODER_BUFFER_DESC", pack=False, align=None),
    "D3D11_VIDEO_DECODER_BUFFER_DESC1": SimStruct(OrderedDict((("BufferType", SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_BUFFER_TYPE")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("pIV", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("IVSize", SimTypeInt(signed=False, label="UInt32")), ("pSubSampleMappingBlock", SimTypePointer(SimTypeRef("D3D11_VIDEO_DECODER_SUB_SAMPLE_MAPPING_BLOCK", SimStruct), offset=0)), ("SubSampleMappingCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_DECODER_BUFFER_DESC1", pack=False, align=None),
    "D3D11_VIDEO_DECODER_BUFFER_DESC2": SimStruct(OrderedDict((("BufferType", SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_BUFFER_TYPE")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("pIV", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("IVSize", SimTypeInt(signed=False, label="UInt32")), ("pSubSampleMappingBlock", SimTypePointer(SimTypeRef("D3D11_VIDEO_DECODER_SUB_SAMPLE_MAPPING_BLOCK", SimStruct), offset=0)), ("SubSampleMappingCount", SimTypeInt(signed=False, label="UInt32")), ("cBlocksStripeEncrypted", SimTypeInt(signed=False, label="UInt32")), ("cBlocksStripeClear", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_DECODER_BUFFER_DESC2", pack=False, align=None),
    "D3D11_VIDEO_DECODER_BUFFER_TYPE": SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_BUFFER_TYPE"),
    "D3D11_VIDEO_DECODER_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_CAPS"),
    "D3D11_VIDEO_DECODER_CONFIG": SimStruct(OrderedDict((("guidConfigBitstreamEncryption", SimTypeBottom(label="Guid")), ("guidConfigMBcontrolEncryption", SimTypeBottom(label="Guid")), ("guidConfigResidDiffEncryption", SimTypeBottom(label="Guid")), ("ConfigBitstreamRaw", SimTypeInt(signed=False, label="UInt32")), ("ConfigMBcontrolRasterOrder", SimTypeInt(signed=False, label="UInt32")), ("ConfigResidDiffHost", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpatialResid8", SimTypeInt(signed=False, label="UInt32")), ("ConfigResid8Subtraction", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpatialHost8or9Clipping", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpatialResidInterleaved", SimTypeInt(signed=False, label="UInt32")), ("ConfigIntraResidUnsigned", SimTypeInt(signed=False, label="UInt32")), ("ConfigResidDiffAccelerator", SimTypeInt(signed=False, label="UInt32")), ("ConfigHostInverseScan", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpecificIDCT", SimTypeInt(signed=False, label="UInt32")), ("Config4GroupedCoefs", SimTypeInt(signed=False, label="UInt32")), ("ConfigMinRenderTargetBuffCount", SimTypeShort(signed=False, label="UInt16")), ("ConfigDecoderSpecific", SimTypeShort(signed=False, label="UInt16")),)), name="D3D11_VIDEO_DECODER_CONFIG", pack=False, align=None),
    "D3D11_VIDEO_DECODER_DESC": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("SampleWidth", SimTypeInt(signed=False, label="UInt32")), ("SampleHeight", SimTypeInt(signed=False, label="UInt32")), ("OutputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")),)), name="D3D11_VIDEO_DECODER_DESC", pack=False, align=None),
    "D3D11_VIDEO_DECODER_EXTENSION": SimStruct(OrderedDict((("Function", SimTypeInt(signed=False, label="UInt32")), ("pPrivateInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateOutputDataSize", SimTypeInt(signed=False, label="UInt32")), ("ResourceCount", SimTypeInt(signed=False, label="UInt32")), ("ppResourceList", SimTypePointer(SimTypeBottom(label="ID3D11Resource"), offset=0)),)), name="D3D11_VIDEO_DECODER_EXTENSION", pack=False, align=None),
    "D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT": SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT"),
    "D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_FLAGS": SimTypeInt(signed=False, label="D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_FLAGS"),
    "D3D11_VIDEO_DECODER_OUTPUT_VIEW_DESC": SimStruct(OrderedDict((("DecodeProfile", SimTypeBottom(label="Guid")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_VDOV_DIMENSION")), ("Anonymous", SimUnion({"Texture2D": SimTypeRef("D3D11_TEX2D_VDOV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_VIDEO_DECODER_OUTPUT_VIEW_DESC", pack=False, align=None),
    "D3D11_VIDEO_DECODER_SUB_SAMPLE_MAPPING_BLOCK": SimStruct(OrderedDict((("ClearSize", SimTypeInt(signed=False, label="UInt32")), ("EncryptedSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_DECODER_SUB_SAMPLE_MAPPING_BLOCK", pack=False, align=None),
    "D3D11_VIDEO_FRAME_FORMAT": SimTypeInt(signed=False, label="D3D11_VIDEO_FRAME_FORMAT"),
    "D3D11_VIDEO_PROCESSOR_ALPHA_FILL_MODE": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_ALPHA_FILL_MODE"),
    "D3D11_VIDEO_PROCESSOR_AUTO_STREAM_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_AUTO_STREAM_CAPS"),
    "D3D11_VIDEO_PROCESSOR_BEHAVIOR_HINTS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_BEHAVIOR_HINTS"),
    "D3D11_VIDEO_PROCESSOR_CAPS": SimStruct(OrderedDict((("DeviceCaps", SimTypeInt(signed=False, label="UInt32")), ("FeatureCaps", SimTypeInt(signed=False, label="UInt32")), ("FilterCaps", SimTypeInt(signed=False, label="UInt32")), ("InputFormatCaps", SimTypeInt(signed=False, label="UInt32")), ("AutoStreamCaps", SimTypeInt(signed=False, label="UInt32")), ("StereoCaps", SimTypeInt(signed=False, label="UInt32")), ("RateConversionCapsCount", SimTypeInt(signed=False, label="UInt32")), ("MaxInputStreams", SimTypeInt(signed=False, label="UInt32")), ("MaxStreamStates", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_PROCESSOR_CAPS", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_COLOR_SPACE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_PROCESSOR_COLOR_SPACE", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_CONTENT_DESC": SimStruct(OrderedDict((("InputFrameFormat", SimTypeInt(signed=False, label="D3D11_VIDEO_FRAME_FORMAT")), ("InputFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("InputWidth", SimTypeInt(signed=False, label="UInt32")), ("InputHeight", SimTypeInt(signed=False, label="UInt32")), ("OutputFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("OutputWidth", SimTypeInt(signed=False, label="UInt32")), ("OutputHeight", SimTypeInt(signed=False, label="UInt32")), ("Usage", SimTypeInt(signed=False, label="D3D11_VIDEO_USAGE")),)), name="D3D11_VIDEO_PROCESSOR_CONTENT_DESC", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_CUSTOM_RATE": SimStruct(OrderedDict((("CustomRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("OutputFrames", SimTypeInt(signed=False, label="UInt32")), ("InputInterlaced", SimTypeInt(signed=True, label="Int32")), ("InputFramesOrFields", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_PROCESSOR_CUSTOM_RATE", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_DEVICE_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_DEVICE_CAPS"),
    "D3D11_VIDEO_PROCESSOR_FEATURE_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_FEATURE_CAPS"),
    "D3D11_VIDEO_PROCESSOR_FILTER": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_FILTER"),
    "D3D11_VIDEO_PROCESSOR_FILTER_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_FILTER_CAPS"),
    "D3D11_VIDEO_PROCESSOR_FILTER_RANGE": SimStruct(OrderedDict((("Minimum", SimTypeInt(signed=True, label="Int32")), ("Maximum", SimTypeInt(signed=True, label="Int32")), ("Default", SimTypeInt(signed=True, label="Int32")), ("Multiplier", SimTypeFloat(size=32)),)), name="D3D11_VIDEO_PROCESSOR_FILTER_RANGE", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_FORMAT_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_FORMAT_CAPS"),
    "D3D11_VIDEO_PROCESSOR_FORMAT_SUPPORT": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_FORMAT_SUPPORT"),
    "D3D11_VIDEO_PROCESSOR_INPUT_VIEW_DESC": SimStruct(OrderedDict((("FourCC", SimTypeInt(signed=False, label="UInt32")), ("ViewDimension", SimTypeInt(signed=False, label="D3D11_VPIV_DIMENSION")), ("Anonymous", SimUnion({"Texture2D": SimTypeRef("D3D11_TEX2D_VPIV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_VIDEO_PROCESSOR_INPUT_VIEW_DESC", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_ITELECINE_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_ITELECINE_CAPS"),
    "D3D11_VIDEO_PROCESSOR_NOMINAL_RANGE": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_NOMINAL_RANGE"),
    "D3D11_VIDEO_PROCESSOR_OUTPUT_RATE": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_OUTPUT_RATE"),
    "D3D11_VIDEO_PROCESSOR_OUTPUT_VIEW_DESC": SimStruct(OrderedDict((("ViewDimension", SimTypeInt(signed=False, label="D3D11_VPOV_DIMENSION")), ("Anonymous", SimUnion({"Texture2D": SimTypeRef("D3D11_TEX2D_VPOV", SimStruct), "Texture2DArray": SimTypeRef("D3D11_TEX2D_ARRAY_VPOV", SimStruct)}, name="<anon>", label="None")),)), name="D3D11_VIDEO_PROCESSOR_OUTPUT_VIEW_DESC", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_PROCESSOR_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_PROCESSOR_CAPS"),
    "D3D11_VIDEO_PROCESSOR_RATE_CONVERSION_CAPS": SimStruct(OrderedDict((("PastFrames", SimTypeInt(signed=False, label="UInt32")), ("FutureFrames", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCaps", SimTypeInt(signed=False, label="UInt32")), ("ITelecineCaps", SimTypeInt(signed=False, label="UInt32")), ("CustomRateCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D11_VIDEO_PROCESSOR_RATE_CONVERSION_CAPS", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_ROTATION": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_ROTATION"),
    "D3D11_VIDEO_PROCESSOR_STEREO_CAPS": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_STEREO_CAPS"),
    "D3D11_VIDEO_PROCESSOR_STEREO_FLIP_MODE": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_STEREO_FLIP_MODE"),
    "D3D11_VIDEO_PROCESSOR_STEREO_FORMAT": SimTypeInt(signed=False, label="D3D11_VIDEO_PROCESSOR_STEREO_FORMAT"),
    "D3D11_VIDEO_PROCESSOR_STREAM": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("OutputIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFrameOrField", SimTypeInt(signed=False, label="UInt32")), ("PastFrames", SimTypeInt(signed=False, label="UInt32")), ("FutureFrames", SimTypeInt(signed=False, label="UInt32")), ("ppPastSurfaces", SimTypePointer(SimTypeBottom(label="ID3D11VideoProcessorInputView"), offset=0)), ("pInputSurface", SimTypeBottom(label="ID3D11VideoProcessorInputView")), ("ppFutureSurfaces", SimTypePointer(SimTypeBottom(label="ID3D11VideoProcessorInputView"), offset=0)), ("ppPastSurfacesRight", SimTypePointer(SimTypeBottom(label="ID3D11VideoProcessorInputView"), offset=0)), ("pInputSurfaceRight", SimTypeBottom(label="ID3D11VideoProcessorInputView")), ("ppFutureSurfacesRight", SimTypePointer(SimTypeBottom(label="ID3D11VideoProcessorInputView"), offset=0)),)), name="D3D11_VIDEO_PROCESSOR_STREAM", pack=False, align=None),
    "D3D11_VIDEO_PROCESSOR_STREAM_BEHAVIOR_HINT": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")),)), name="D3D11_VIDEO_PROCESSOR_STREAM_BEHAVIOR_HINT", pack=False, align=None),
    "D3D11_VIDEO_SAMPLE_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")),)), name="D3D11_VIDEO_SAMPLE_DESC", pack=False, align=None),
    "D3D11_VIDEO_USAGE": SimTypeInt(signed=False, label="D3D11_VIDEO_USAGE"),
    "D3D11_VIEWPORT": SimStruct(OrderedDict((("TopLeftX", SimTypeFloat(size=32)), ("TopLeftY", SimTypeFloat(size=32)), ("Width", SimTypeFloat(size=32)), ("Height", SimTypeFloat(size=32)), ("MinDepth", SimTypeFloat(size=32)), ("MaxDepth", SimTypeFloat(size=32)),)), name="D3D11_VIEWPORT", pack=False, align=None),
    "D3D11_VPIV_DIMENSION": SimTypeInt(signed=False, label="D3D11_VPIV_DIMENSION"),
    "D3D11_VPOV_DIMENSION": SimTypeInt(signed=False, label="D3D11_VPOV_DIMENSION"),
    "D3D12MessageFunc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="D3D12_MESSAGE_CATEGORY"), SimTypeInt(signed=False, label="D3D12_MESSAGE_SEVERITY"), SimTypeInt(signed=False, label="D3D12_MESSAGE_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Category", "Severity", "ID", "pDescription", "pContext"]), offset=0),
    "D3D12_AUTO_BREADCRUMB_NODE": SimStruct(OrderedDict((("pCommandListDebugNameA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pCommandListDebugNameW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCommandQueueDebugNameA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pCommandQueueDebugNameW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCommandList", SimTypeBottom(label="ID3D12GraphicsCommandList")), ("pCommandQueue", SimTypeBottom(label="ID3D12CommandQueue")), ("BreadcrumbCount", SimTypeInt(signed=False, label="UInt32")), ("pLastBreadcrumbValue", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pCommandHistory", SimTypePointer(SimTypeInt(signed=False, label="D3D12_AUTO_BREADCRUMB_OP"), offset=0)), ("pNext", SimTypePointer(SimTypeRef("D3D12_AUTO_BREADCRUMB_NODE", SimStruct), offset=0)),)), name="D3D12_AUTO_BREADCRUMB_NODE", pack=False, align=None),
    "D3D12_AUTO_BREADCRUMB_NODE1": SimStruct(OrderedDict((("pCommandListDebugNameA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pCommandListDebugNameW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCommandQueueDebugNameA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pCommandQueueDebugNameW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCommandList", SimTypeBottom(label="ID3D12GraphicsCommandList")), ("pCommandQueue", SimTypeBottom(label="ID3D12CommandQueue")), ("BreadcrumbCount", SimTypeInt(signed=False, label="UInt32")), ("pLastBreadcrumbValue", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pCommandHistory", SimTypePointer(SimTypeInt(signed=False, label="D3D12_AUTO_BREADCRUMB_OP"), offset=0)), ("pNext", SimTypePointer(SimTypeRef("D3D12_AUTO_BREADCRUMB_NODE1", SimStruct), offset=0)), ("BreadcrumbContextsCount", SimTypeInt(signed=False, label="UInt32")), ("pBreadcrumbContexts", SimTypePointer(SimTypeRef("D3D12_DRED_BREADCRUMB_CONTEXT", SimStruct), offset=0)),)), name="D3D12_AUTO_BREADCRUMB_NODE1", pack=False, align=None),
    "D3D12_AUTO_BREADCRUMB_OP": SimTypeInt(signed=False, label="D3D12_AUTO_BREADCRUMB_OP"),
    "D3D12_AXIS_SHADING_RATE": SimTypeInt(signed=False, label="D3D12_AXIS_SHADING_RATE"),
    "D3D12_BACKGROUND_PROCESSING_MODE": SimTypeInt(signed=False, label="D3D12_BACKGROUND_PROCESSING_MODE"),
    "D3D12_BARRIER_ACCESS": SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS"),
    "D3D12_BARRIER_GROUP": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_BARRIER_TYPE")), ("NumBarriers", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pGlobalBarriers": SimTypePointer(SimTypeRef("D3D12_GLOBAL_BARRIER", SimStruct), offset=0), "pTextureBarriers": SimTypePointer(SimTypeRef("D3D12_TEXTURE_BARRIER", SimStruct), offset=0), "pBufferBarriers": SimTypePointer(SimTypeRef("D3D12_BUFFER_BARRIER", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_BARRIER_GROUP", pack=False, align=None),
    "D3D12_BARRIER_LAYOUT": SimTypeInt(signed=False, label="D3D12_BARRIER_LAYOUT"),
    "D3D12_BARRIER_SUBRESOURCE_RANGE": SimStruct(OrderedDict((("IndexOrFirstMipLevel", SimTypeInt(signed=False, label="UInt32")), ("NumMipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("NumArraySlices", SimTypeInt(signed=False, label="UInt32")), ("FirstPlane", SimTypeInt(signed=False, label="UInt32")), ("NumPlanes", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_BARRIER_SUBRESOURCE_RANGE", pack=False, align=None),
    "D3D12_BARRIER_SYNC": SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC"),
    "D3D12_BARRIER_TYPE": SimTypeInt(signed=False, label="D3D12_BARRIER_TYPE"),
    "D3D12_BITSTREAM_ENCRYPTION_TYPE": SimTypeInt(signed=False, label="D3D12_BITSTREAM_ENCRYPTION_TYPE"),
    "D3D12_BLEND": SimTypeInt(signed=False, label="D3D12_BLEND"),
    "D3D12_BLEND_DESC": SimStruct(OrderedDict((("AlphaToCoverageEnable", SimTypeInt(signed=True, label="Int32")), ("IndependentBlendEnable", SimTypeInt(signed=True, label="Int32")), ("RenderTarget", SimTypeArray(SimTypeRef("D3D12_RENDER_TARGET_BLEND_DESC", SimStruct), 8)),)), name="D3D12_BLEND_DESC", pack=False, align=None),
    "D3D12_BLEND_OP": SimTypeInt(signed=False, label="D3D12_BLEND_OP"),
    "D3D12_BOX": SimStruct(OrderedDict((("left", SimTypeInt(signed=False, label="UInt32")), ("top", SimTypeInt(signed=False, label="UInt32")), ("front", SimTypeInt(signed=False, label="UInt32")), ("right", SimTypeInt(signed=False, label="UInt32")), ("bottom", SimTypeInt(signed=False, label="UInt32")), ("back", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_BOX", pack=False, align=None),
    "D3D12_BUFFER_BARRIER": SimStruct(OrderedDict((("SyncBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC")), ("SyncAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC")), ("AccessBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS")), ("AccessAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS")), ("pResource", SimTypeBottom(label="ID3D12Resource")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_BUFFER_BARRIER", pack=False, align=None),
    "D3D12_BUFFER_RTV": SimStruct(OrderedDict((("FirstElement", SimTypeLongLong(signed=False, label="UInt64")), ("NumElements", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_BUFFER_RTV", pack=False, align=None),
    "D3D12_BUFFER_SRV": SimStruct(OrderedDict((("FirstElement", SimTypeLongLong(signed=False, label="UInt64")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("StructureByteStride", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_BUFFER_SRV_FLAGS")),)), name="D3D12_BUFFER_SRV", pack=False, align=None),
    "D3D12_BUFFER_SRV_FLAGS": SimTypeInt(signed=False, label="D3D12_BUFFER_SRV_FLAGS"),
    "D3D12_BUFFER_UAV": SimStruct(OrderedDict((("FirstElement", SimTypeLongLong(signed=False, label="UInt64")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("StructureByteStride", SimTypeInt(signed=False, label="UInt32")), ("CounterOffsetInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="D3D12_BUFFER_UAV_FLAGS")),)), name="D3D12_BUFFER_UAV", pack=False, align=None),
    "D3D12_BUFFER_UAV_FLAGS": SimTypeInt(signed=False, label="D3D12_BUFFER_UAV_FLAGS"),
    "D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_DESC": SimStruct(OrderedDict((("DestAccelerationStructureData", SimTypeLongLong(signed=False, label="UInt64")), ("Inputs", SimTypeRef("D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_INPUTS", SimStruct)), ("SourceAccelerationStructureData", SimTypeLongLong(signed=False, label="UInt64")), ("ScratchAccelerationStructureData", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_DESC", pack=False, align=None),
    "D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_INPUTS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_TYPE")), ("Flags", SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_BUILD_FLAGS")), ("NumDescs", SimTypeInt(signed=False, label="UInt32")), ("DescsLayout", SimTypeInt(signed=False, label="D3D12_ELEMENTS_LAYOUT")), ("Anonymous", SimUnion({"InstanceDescs": SimTypeLongLong(signed=False, label="UInt64"), "pGeometryDescs": SimTypePointer(SimTypeRef("D3D12_RAYTRACING_GEOMETRY_DESC", SimStruct), offset=0), "ppGeometryDescs": SimTypePointer(SimTypePointer(SimTypeRef("D3D12_RAYTRACING_GEOMETRY_DESC", SimStruct), offset=0), offset=0)}, name="<anon>", label="None")),)), name="D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_INPUTS", pack=False, align=None),
    "D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_TOOLS_VISUALIZATION_HEADER": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_TYPE")), ("NumDescs", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_TOOLS_VISUALIZATION_HEADER", pack=False, align=None),
    "D3D12_CACHED_PIPELINE_STATE": SimStruct(OrderedDict((("pCachedBlob", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CachedBlobSizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_CACHED_PIPELINE_STATE", pack=False, align=None),
    "D3D12_CLEAR_FLAGS": SimTypeInt(signed=False, label="D3D12_CLEAR_FLAGS"),
    "D3D12_CLEAR_VALUE": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Anonymous", SimUnion({"Color": SimTypeArray(SimTypeFloat(size=32), 4), "DepthStencil": SimTypeRef("D3D12_DEPTH_STENCIL_VALUE", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_CLEAR_VALUE", pack=False, align=None),
    "D3D12_COLOR_WRITE_ENABLE": SimTypeInt(signed=False, label="D3D12_COLOR_WRITE_ENABLE"),
    "D3D12_COMMAND_LIST_FLAGS": SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_FLAGS"),
    "D3D12_COMMAND_LIST_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_SUPPORT_FLAGS"),
    "D3D12_COMMAND_LIST_TYPE": SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_TYPE"),
    "D3D12_COMMAND_POOL_FLAGS": SimTypeInt(signed=False, label="D3D12_COMMAND_POOL_FLAGS"),
    "D3D12_COMMAND_QUEUE_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_TYPE")), ("Priority", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="D3D12_COMMAND_QUEUE_FLAGS")), ("NodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_COMMAND_QUEUE_DESC", pack=False, align=None),
    "D3D12_COMMAND_QUEUE_FLAGS": SimTypeInt(signed=False, label="D3D12_COMMAND_QUEUE_FLAGS"),
    "D3D12_COMMAND_QUEUE_PRIORITY": SimTypeInt(signed=False, label="D3D12_COMMAND_QUEUE_PRIORITY"),
    "D3D12_COMMAND_RECORDER_FLAGS": SimTypeInt(signed=False, label="D3D12_COMMAND_RECORDER_FLAGS"),
    "D3D12_COMMAND_SIGNATURE_DESC": SimStruct(OrderedDict((("ByteStride", SimTypeInt(signed=False, label="UInt32")), ("NumArgumentDescs", SimTypeInt(signed=False, label="UInt32")), ("pArgumentDescs", SimTypePointer(SimTypeRef("D3D12_INDIRECT_ARGUMENT_DESC", SimStruct), label="LPArray", offset=0)), ("NodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_COMMAND_SIGNATURE_DESC", pack=False, align=None),
    "D3D12_COMPARISON_FUNC": SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC"),
    "D3D12_COMPUTE_PIPELINE_STATE_DESC": SimStruct(OrderedDict((("pRootSignature", SimTypeBottom(label="ID3D12RootSignature")), ("CS", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("CachedPSO", SimTypeRef("D3D12_CACHED_PIPELINE_STATE", SimStruct)), ("Flags", SimTypeInt(signed=False, label="D3D12_PIPELINE_STATE_FLAGS")),)), name="D3D12_COMPUTE_PIPELINE_STATE_DESC", pack=False, align=None),
    "D3D12_CONSERVATIVE_RASTERIZATION_MODE": SimTypeInt(signed=False, label="D3D12_CONSERVATIVE_RASTERIZATION_MODE"),
    "D3D12_CONSERVATIVE_RASTERIZATION_TIER": SimTypeInt(signed=False, label="D3D12_CONSERVATIVE_RASTERIZATION_TIER"),
    "D3D12_CONSTANT_BUFFER_VIEW_DESC": SimStruct(OrderedDict((("BufferLocation", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_CONSTANT_BUFFER_VIEW_DESC", pack=False, align=None),
    "D3D12_CPU_DESCRIPTOR_HANDLE": SimStruct(OrderedDict((("ptr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_CPU_DESCRIPTOR_HANDLE", pack=False, align=None),
    "D3D12_CPU_PAGE_PROPERTY": SimTypeInt(signed=False, label="D3D12_CPU_PAGE_PROPERTY"),
    "D3D12_CROSS_NODE_SHARING_TIER": SimTypeInt(signed=False, label="D3D12_CROSS_NODE_SHARING_TIER"),
    "D3D12_CULL_MODE": SimTypeInt(signed=False, label="D3D12_CULL_MODE"),
    "D3D12_DEBUG_COMMAND_LIST_GPU_BASED_VALIDATION_SETTINGS": SimStruct(OrderedDict((("ShaderPatchMode", SimTypeInt(signed=False, label="D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE")),)), name="D3D12_DEBUG_COMMAND_LIST_GPU_BASED_VALIDATION_SETTINGS", pack=False, align=None),
    "D3D12_DEBUG_COMMAND_LIST_PARAMETER_TYPE": SimTypeInt(signed=False, label="D3D12_DEBUG_COMMAND_LIST_PARAMETER_TYPE"),
    "D3D12_DEBUG_DEVICE_GPU_BASED_VALIDATION_SETTINGS": SimStruct(OrderedDict((("MaxMessagesPerCommandList", SimTypeInt(signed=False, label="UInt32")), ("DefaultShaderPatchMode", SimTypeInt(signed=False, label="D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE")), ("PipelineStateCreateFlags", SimTypeInt(signed=False, label="D3D12_GPU_BASED_VALIDATION_PIPELINE_STATE_CREATE_FLAGS")),)), name="D3D12_DEBUG_DEVICE_GPU_BASED_VALIDATION_SETTINGS", pack=False, align=None),
    "D3D12_DEBUG_DEVICE_GPU_SLOWDOWN_PERFORMANCE_FACTOR": SimStruct(OrderedDict((("SlowdownFactor", SimTypeFloat(size=32)),)), name="D3D12_DEBUG_DEVICE_GPU_SLOWDOWN_PERFORMANCE_FACTOR", pack=False, align=None),
    "D3D12_DEBUG_DEVICE_PARAMETER_TYPE": SimTypeInt(signed=False, label="D3D12_DEBUG_DEVICE_PARAMETER_TYPE"),
    "D3D12_DEBUG_FEATURE": SimTypeInt(signed=False, label="D3D12_DEBUG_FEATURE"),
    "D3D12_DEPTH_STENCILOP_DESC": SimStruct(OrderedDict((("StencilFailOp", SimTypeInt(signed=False, label="D3D12_STENCIL_OP")), ("StencilDepthFailOp", SimTypeInt(signed=False, label="D3D12_STENCIL_OP")), ("StencilPassOp", SimTypeInt(signed=False, label="D3D12_STENCIL_OP")), ("StencilFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")),)), name="D3D12_DEPTH_STENCILOP_DESC", pack=False, align=None),
    "D3D12_DEPTH_STENCILOP_DESC1": SimStruct(OrderedDict((("StencilFailOp", SimTypeInt(signed=False, label="D3D12_STENCIL_OP")), ("StencilDepthFailOp", SimTypeInt(signed=False, label="D3D12_STENCIL_OP")), ("StencilPassOp", SimTypeInt(signed=False, label="D3D12_STENCIL_OP")), ("StencilFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("StencilReadMask", SimTypeChar(label="Byte")), ("StencilWriteMask", SimTypeChar(label="Byte")),)), name="D3D12_DEPTH_STENCILOP_DESC1", pack=False, align=None),
    "D3D12_DEPTH_STENCIL_DESC": SimStruct(OrderedDict((("DepthEnable", SimTypeInt(signed=True, label="Int32")), ("DepthWriteMask", SimTypeInt(signed=False, label="D3D12_DEPTH_WRITE_MASK")), ("DepthFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("StencilEnable", SimTypeInt(signed=True, label="Int32")), ("StencilReadMask", SimTypeChar(label="Byte")), ("StencilWriteMask", SimTypeChar(label="Byte")), ("FrontFace", SimTypeRef("D3D12_DEPTH_STENCILOP_DESC", SimStruct)), ("BackFace", SimTypeRef("D3D12_DEPTH_STENCILOP_DESC", SimStruct)),)), name="D3D12_DEPTH_STENCIL_DESC", pack=False, align=None),
    "D3D12_DEPTH_STENCIL_DESC1": SimStruct(OrderedDict((("DepthEnable", SimTypeInt(signed=True, label="Int32")), ("DepthWriteMask", SimTypeInt(signed=False, label="D3D12_DEPTH_WRITE_MASK")), ("DepthFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("StencilEnable", SimTypeInt(signed=True, label="Int32")), ("StencilReadMask", SimTypeChar(label="Byte")), ("StencilWriteMask", SimTypeChar(label="Byte")), ("FrontFace", SimTypeRef("D3D12_DEPTH_STENCILOP_DESC", SimStruct)), ("BackFace", SimTypeRef("D3D12_DEPTH_STENCILOP_DESC", SimStruct)), ("DepthBoundsTestEnable", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_DEPTH_STENCIL_DESC1", pack=False, align=None),
    "D3D12_DEPTH_STENCIL_DESC2": SimStruct(OrderedDict((("DepthEnable", SimTypeInt(signed=True, label="Int32")), ("DepthWriteMask", SimTypeInt(signed=False, label="D3D12_DEPTH_WRITE_MASK")), ("DepthFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("StencilEnable", SimTypeInt(signed=True, label="Int32")), ("FrontFace", SimTypeRef("D3D12_DEPTH_STENCILOP_DESC1", SimStruct)), ("BackFace", SimTypeRef("D3D12_DEPTH_STENCILOP_DESC1", SimStruct)), ("DepthBoundsTestEnable", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_DEPTH_STENCIL_DESC2", pack=False, align=None),
    "D3D12_DEPTH_STENCIL_VALUE": SimStruct(OrderedDict((("Depth", SimTypeFloat(size=32)), ("Stencil", SimTypeChar(label="Byte")),)), name="D3D12_DEPTH_STENCIL_VALUE", pack=False, align=None),
    "D3D12_DEPTH_STENCIL_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D12_DSV_DIMENSION")), ("Flags", SimTypeInt(signed=False, label="D3D12_DSV_FLAGS")), ("Anonymous", SimUnion({"Texture1D": SimTypeRef("D3D12_TEX1D_DSV", SimStruct), "Texture1DArray": SimTypeRef("D3D12_TEX1D_ARRAY_DSV", SimStruct), "Texture2D": SimTypeRef("D3D12_TEX2D_DSV", SimStruct), "Texture2DArray": SimTypeRef("D3D12_TEX2D_ARRAY_DSV", SimStruct), "Texture2DMS": SimTypeRef("D3D12_TEX2DMS_DSV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D12_TEX2DMS_ARRAY_DSV", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_DEPTH_STENCIL_VIEW_DESC", pack=False, align=None),
    "D3D12_DEPTH_WRITE_MASK": SimTypeInt(signed=False, label="D3D12_DEPTH_WRITE_MASK"),
    "D3D12_DESCRIPTOR_HEAP_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_HEAP_TYPE")), ("NumDescriptors", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_HEAP_FLAGS")), ("NodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DESCRIPTOR_HEAP_DESC", pack=False, align=None),
    "D3D12_DESCRIPTOR_HEAP_FLAGS": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_HEAP_FLAGS"),
    "D3D12_DESCRIPTOR_HEAP_TYPE": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_HEAP_TYPE"),
    "D3D12_DESCRIPTOR_RANGE": SimStruct(OrderedDict((("RangeType", SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_TYPE")), ("NumDescriptors", SimTypeInt(signed=False, label="UInt32")), ("BaseShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")), ("OffsetInDescriptorsFromTableStart", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DESCRIPTOR_RANGE", pack=False, align=None),
    "D3D12_DESCRIPTOR_RANGE1": SimStruct(OrderedDict((("RangeType", SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_TYPE")), ("NumDescriptors", SimTypeInt(signed=False, label="UInt32")), ("BaseShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_FLAGS")), ("OffsetInDescriptorsFromTableStart", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DESCRIPTOR_RANGE1", pack=False, align=None),
    "D3D12_DESCRIPTOR_RANGE_FLAGS": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_FLAGS"),
    "D3D12_DESCRIPTOR_RANGE_TYPE": SimTypeInt(signed=False, label="D3D12_DESCRIPTOR_RANGE_TYPE"),
    "D3D12_DEVICE_CONFIGURATION_DESC": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_DEVICE_FLAGS")), ("GpuBasedValidationFlags", SimTypeInt(signed=False, label="UInt32")), ("SDKVersion", SimTypeInt(signed=False, label="UInt32")), ("NumEnabledExperimentalFeatures", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DEVICE_CONFIGURATION_DESC", pack=False, align=None),
    "D3D12_DEVICE_FACTORY_FLAGS": SimTypeInt(signed=False, label="D3D12_DEVICE_FACTORY_FLAGS"),
    "D3D12_DEVICE_FLAGS": SimTypeInt(signed=False, label="D3D12_DEVICE_FLAGS"),
    "D3D12_DEVICE_REMOVED_EXTENDED_DATA": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_DRED_FLAGS")), ("pHeadAutoBreadcrumbNode", SimTypePointer(SimTypeRef("D3D12_AUTO_BREADCRUMB_NODE", SimStruct), offset=0)),)), name="D3D12_DEVICE_REMOVED_EXTENDED_DATA", pack=False, align=None),
    "D3D12_DEVICE_REMOVED_EXTENDED_DATA1": SimStruct(OrderedDict((("DeviceRemovedReason", SimTypeInt(signed=True, label="Int32")), ("AutoBreadcrumbsOutput", SimTypeRef("D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT", SimStruct)), ("PageFaultOutput", SimTypeRef("D3D12_DRED_PAGE_FAULT_OUTPUT", SimStruct)),)), name="D3D12_DEVICE_REMOVED_EXTENDED_DATA1", pack=False, align=None),
    "D3D12_DEVICE_REMOVED_EXTENDED_DATA2": SimStruct(OrderedDict((("DeviceRemovedReason", SimTypeInt(signed=True, label="Int32")), ("AutoBreadcrumbsOutput", SimTypeRef("D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT1", SimStruct)), ("PageFaultOutput", SimTypeRef("D3D12_DRED_PAGE_FAULT_OUTPUT1", SimStruct)),)), name="D3D12_DEVICE_REMOVED_EXTENDED_DATA2", pack=False, align=None),
    "D3D12_DEVICE_REMOVED_EXTENDED_DATA3": SimStruct(OrderedDict((("DeviceRemovedReason", SimTypeInt(signed=True, label="Int32")), ("AutoBreadcrumbsOutput", SimTypeRef("D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT1", SimStruct)), ("PageFaultOutput", SimTypeRef("D3D12_DRED_PAGE_FAULT_OUTPUT2", SimStruct)), ("DeviceState", SimTypeInt(signed=False, label="D3D12_DRED_DEVICE_STATE")),)), name="D3D12_DEVICE_REMOVED_EXTENDED_DATA3", pack=False, align=None),
    "D3D12_DISCARD_REGION": SimStruct(OrderedDict((("NumRects", SimTypeInt(signed=False, label="UInt32")), ("pRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("FirstSubresource", SimTypeInt(signed=False, label="UInt32")), ("NumSubresources", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DISCARD_REGION", pack=False, align=None),
    "D3D12_DISPATCH_ARGUMENTS": SimStruct(OrderedDict((("ThreadGroupCountX", SimTypeInt(signed=False, label="UInt32")), ("ThreadGroupCountY", SimTypeInt(signed=False, label="UInt32")), ("ThreadGroupCountZ", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DISPATCH_ARGUMENTS", pack=False, align=None),
    "D3D12_DISPATCH_MESH_ARGUMENTS": SimStruct(OrderedDict((("ThreadGroupCountX", SimTypeInt(signed=False, label="UInt32")), ("ThreadGroupCountY", SimTypeInt(signed=False, label="UInt32")), ("ThreadGroupCountZ", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DISPATCH_MESH_ARGUMENTS", pack=False, align=None),
    "D3D12_DISPATCH_RAYS_DESC": SimStruct(OrderedDict((("RayGenerationShaderRecord", SimTypeRef("D3D12_GPU_VIRTUAL_ADDRESS_RANGE", SimStruct)), ("MissShaderTable", SimTypeRef("D3D12_GPU_VIRTUAL_ADDRESS_RANGE_AND_STRIDE", SimStruct)), ("HitGroupTable", SimTypeRef("D3D12_GPU_VIRTUAL_ADDRESS_RANGE_AND_STRIDE", SimStruct)), ("CallableShaderTable", SimTypeRef("D3D12_GPU_VIRTUAL_ADDRESS_RANGE_AND_STRIDE", SimStruct)), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DISPATCH_RAYS_DESC", pack=False, align=None),
    "D3D12_DRAW_ARGUMENTS": SimStruct(OrderedDict((("VertexCountPerInstance", SimTypeInt(signed=False, label="UInt32")), ("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("StartVertexLocation", SimTypeInt(signed=False, label="UInt32")), ("StartInstanceLocation", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DRAW_ARGUMENTS", pack=False, align=None),
    "D3D12_DRAW_INDEXED_ARGUMENTS": SimStruct(OrderedDict((("IndexCountPerInstance", SimTypeInt(signed=False, label="UInt32")), ("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("StartIndexLocation", SimTypeInt(signed=False, label="UInt32")), ("BaseVertexLocation", SimTypeInt(signed=True, label="Int32")), ("StartInstanceLocation", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_DRAW_INDEXED_ARGUMENTS", pack=False, align=None),
    "D3D12_DRED_ALLOCATION_NODE": SimStruct(OrderedDict((("ObjectNameA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ObjectNameW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("AllocationType", SimTypeInt(signed=False, label="D3D12_DRED_ALLOCATION_TYPE")), ("pNext", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE", SimStruct), offset=0)),)), name="D3D12_DRED_ALLOCATION_NODE", pack=False, align=None),
    "D3D12_DRED_ALLOCATION_NODE1": SimStruct(OrderedDict((("ObjectNameA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ObjectNameW", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("AllocationType", SimTypeInt(signed=False, label="D3D12_DRED_ALLOCATION_TYPE")), ("pNext", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE1", SimStruct), offset=0)), ("pObject", SimTypeBottom(label="IUnknown")),)), name="D3D12_DRED_ALLOCATION_NODE1", pack=False, align=None),
    "D3D12_DRED_ALLOCATION_TYPE": SimTypeInt(signed=False, label="D3D12_DRED_ALLOCATION_TYPE"),
    "D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT": SimStruct(OrderedDict((("pHeadAutoBreadcrumbNode", SimTypePointer(SimTypeRef("D3D12_AUTO_BREADCRUMB_NODE", SimStruct), offset=0)),)), name="D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT", pack=False, align=None),
    "D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT1": SimStruct(OrderedDict((("pHeadAutoBreadcrumbNode", SimTypePointer(SimTypeRef("D3D12_AUTO_BREADCRUMB_NODE1", SimStruct), offset=0)),)), name="D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT1", pack=False, align=None),
    "D3D12_DRED_BREADCRUMB_CONTEXT": SimStruct(OrderedDict((("BreadcrumbIndex", SimTypeInt(signed=False, label="UInt32")), ("pContextString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="D3D12_DRED_BREADCRUMB_CONTEXT", pack=False, align=None),
    "D3D12_DRED_DEVICE_STATE": SimTypeInt(signed=False, label="D3D12_DRED_DEVICE_STATE"),
    "D3D12_DRED_ENABLEMENT": SimTypeInt(signed=False, label="D3D12_DRED_ENABLEMENT"),
    "D3D12_DRED_FLAGS": SimTypeInt(signed=False, label="D3D12_DRED_FLAGS"),
    "D3D12_DRED_PAGE_FAULT_FLAGS": SimTypeInt(signed=False, label="D3D12_DRED_PAGE_FAULT_FLAGS"),
    "D3D12_DRED_PAGE_FAULT_OUTPUT": SimStruct(OrderedDict((("PageFaultVA", SimTypeLongLong(signed=False, label="UInt64")), ("pHeadExistingAllocationNode", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE", SimStruct), offset=0)), ("pHeadRecentFreedAllocationNode", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE", SimStruct), offset=0)),)), name="D3D12_DRED_PAGE_FAULT_OUTPUT", pack=False, align=None),
    "D3D12_DRED_PAGE_FAULT_OUTPUT1": SimStruct(OrderedDict((("PageFaultVA", SimTypeLongLong(signed=False, label="UInt64")), ("pHeadExistingAllocationNode", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE1", SimStruct), offset=0)), ("pHeadRecentFreedAllocationNode", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE1", SimStruct), offset=0)),)), name="D3D12_DRED_PAGE_FAULT_OUTPUT1", pack=False, align=None),
    "D3D12_DRED_PAGE_FAULT_OUTPUT2": SimStruct(OrderedDict((("PageFaultVA", SimTypeLongLong(signed=False, label="UInt64")), ("pHeadExistingAllocationNode", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE1", SimStruct), offset=0)), ("pHeadRecentFreedAllocationNode", SimTypePointer(SimTypeRef("D3D12_DRED_ALLOCATION_NODE1", SimStruct), offset=0)), ("PageFaultFlags", SimTypeInt(signed=False, label="D3D12_DRED_PAGE_FAULT_FLAGS")),)), name="D3D12_DRED_PAGE_FAULT_OUTPUT2", pack=False, align=None),
    "D3D12_DRED_VERSION": SimTypeInt(signed=False, label="D3D12_DRED_VERSION"),
    "D3D12_DRIVER_MATCHING_IDENTIFIER_STATUS": SimTypeInt(signed=False, label="D3D12_DRIVER_MATCHING_IDENTIFIER_STATUS"),
    "D3D12_DSV_DIMENSION": SimTypeInt(signed=False, label="D3D12_DSV_DIMENSION"),
    "D3D12_DSV_FLAGS": SimTypeInt(signed=False, label="D3D12_DSV_FLAGS"),
    "D3D12_DXIL_LIBRARY_DESC": SimStruct(OrderedDict((("DXILLibrary", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("NumExports", SimTypeInt(signed=False, label="UInt32")), ("pExports", SimTypePointer(SimTypeRef("D3D12_EXPORT_DESC", SimStruct), offset=0)),)), name="D3D12_DXIL_LIBRARY_DESC", pack=False, align=None),
    "D3D12_DXIL_SUBOBJECT_TO_EXPORTS_ASSOCIATION": SimStruct(OrderedDict((("SubobjectToAssociate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NumExports", SimTypeInt(signed=False, label="UInt32")), ("pExports", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="D3D12_DXIL_SUBOBJECT_TO_EXPORTS_ASSOCIATION", pack=False, align=None),
    "D3D12_ELEMENTS_LAYOUT": SimTypeInt(signed=False, label="D3D12_ELEMENTS_LAYOUT"),
    "D3D12_EXISTING_COLLECTION_DESC": SimStruct(OrderedDict((("pExistingCollection", SimTypeBottom(label="ID3D12StateObject")), ("NumExports", SimTypeInt(signed=False, label="UInt32")), ("pExports", SimTypePointer(SimTypeRef("D3D12_EXPORT_DESC", SimStruct), offset=0)),)), name="D3D12_EXISTING_COLLECTION_DESC", pack=False, align=None),
    "D3D12_EXPORT_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ExportToRename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="D3D12_EXPORT_FLAGS")),)), name="D3D12_EXPORT_DESC", pack=False, align=None),
    "D3D12_EXPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_EXPORT_FLAGS"),
    "D3D12_FEATURE": SimTypeInt(signed=False, label="D3D12_FEATURE"),
    "D3D12_FEATURE_DATA_ARCHITECTURE": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("TileBasedRenderer", SimTypeInt(signed=True, label="Int32")), ("UMA", SimTypeInt(signed=True, label="Int32")), ("CacheCoherentUMA", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_ARCHITECTURE", pack=False, align=None),
    "D3D12_FEATURE_DATA_ARCHITECTURE1": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("TileBasedRenderer", SimTypeInt(signed=True, label="Int32")), ("UMA", SimTypeInt(signed=True, label="Int32")), ("CacheCoherentUMA", SimTypeInt(signed=True, label="Int32")), ("IsolatedMMU", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_ARCHITECTURE1", pack=False, align=None),
    "D3D12_FEATURE_DATA_COMMAND_QUEUE_PRIORITY": SimStruct(OrderedDict((("CommandListType", SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_TYPE")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("PriorityForTypeIsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_COMMAND_QUEUE_PRIORITY", pack=False, align=None),
    "D3D12_FEATURE_DATA_CROSS_NODE": SimStruct(OrderedDict((("SharingTier", SimTypeInt(signed=False, label="D3D12_CROSS_NODE_SHARING_TIER")), ("AtomicShaderInstructions", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_CROSS_NODE", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS": SimStruct(OrderedDict((("DoublePrecisionFloatShaderOps", SimTypeInt(signed=True, label="Int32")), ("OutputMergerLogicOp", SimTypeInt(signed=True, label="Int32")), ("MinPrecisionSupport", SimTypeInt(signed=False, label="D3D12_SHADER_MIN_PRECISION_SUPPORT")), ("TiledResourcesTier", SimTypeInt(signed=False, label="D3D12_TILED_RESOURCES_TIER")), ("ResourceBindingTier", SimTypeInt(signed=False, label="D3D12_RESOURCE_BINDING_TIER")), ("PSSpecifiedStencilRefSupported", SimTypeInt(signed=True, label="Int32")), ("TypedUAVLoadAdditionalFormats", SimTypeInt(signed=True, label="Int32")), ("ROVsSupported", SimTypeInt(signed=True, label="Int32")), ("ConservativeRasterizationTier", SimTypeInt(signed=False, label="D3D12_CONSERVATIVE_RASTERIZATION_TIER")), ("MaxGPUVirtualAddressBitsPerResource", SimTypeInt(signed=False, label="UInt32")), ("StandardSwizzle64KBSupported", SimTypeInt(signed=True, label="Int32")), ("CrossNodeSharingTier", SimTypeInt(signed=False, label="D3D12_CROSS_NODE_SHARING_TIER")), ("CrossAdapterRowMajorTextureSupported", SimTypeInt(signed=True, label="Int32")), ("VPAndRTArrayIndexFromAnyShaderFeedingRasterizerSupportedWithoutGSEmulation", SimTypeInt(signed=True, label="Int32")), ("ResourceHeapTier", SimTypeInt(signed=False, label="D3D12_RESOURCE_HEAP_TIER")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS1": SimStruct(OrderedDict((("WaveOps", SimTypeInt(signed=True, label="Int32")), ("WaveLaneCountMin", SimTypeInt(signed=False, label="UInt32")), ("WaveLaneCountMax", SimTypeInt(signed=False, label="UInt32")), ("TotalLaneCount", SimTypeInt(signed=False, label="UInt32")), ("ExpandedComputeResourceStates", SimTypeInt(signed=True, label="Int32")), ("Int64ShaderOps", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS1", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS10": SimStruct(OrderedDict((("VariableRateShadingSumCombinerSupported", SimTypeInt(signed=True, label="Int32")), ("MeshShaderPerPrimitiveShadingRateSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS10", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS11": SimStruct(OrderedDict((("AtomicInt64OnDescriptorHeapResourceSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS11", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS12": SimStruct(OrderedDict((("MSPrimitivesPipelineStatisticIncludesCulledPrimitives", SimTypeInt(signed=False, label="D3D12_TRI_STATE")), ("EnhancedBarriersSupported", SimTypeInt(signed=True, label="Int32")), ("RelaxedFormatCastingSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS12", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS13": SimStruct(OrderedDict((("UnrestrictedBufferTextureCopyPitchSupported", SimTypeInt(signed=True, label="Int32")), ("UnrestrictedVertexElementAlignmentSupported", SimTypeInt(signed=True, label="Int32")), ("InvertedViewportHeightFlipsYSupported", SimTypeInt(signed=True, label="Int32")), ("InvertedViewportDepthFlipsZSupported", SimTypeInt(signed=True, label="Int32")), ("TextureCopyBetweenDimensionsSupported", SimTypeInt(signed=True, label="Int32")), ("AlphaBlendFactorSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS13", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS14": SimStruct(OrderedDict((("AdvancedTextureOpsSupported", SimTypeInt(signed=True, label="Int32")), ("WriteableMSAATexturesSupported", SimTypeInt(signed=True, label="Int32")), ("IndependentFrontAndBackStencilRefMaskSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS14", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS15": SimStruct(OrderedDict((("TriangleFanSupported", SimTypeInt(signed=True, label="Int32")), ("DynamicIndexBufferStripCutSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS15", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS16": SimStruct(OrderedDict((("DynamicDepthBiasSupported", SimTypeInt(signed=True, label="Int32")), ("GPUUploadHeapSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS16", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS17": SimStruct(OrderedDict((("NonNormalizedCoordinateSamplersSupported", SimTypeInt(signed=True, label="Int32")), ("ManualWriteTrackingResourceSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS17", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS18": SimStruct(OrderedDict((("RenderPassesValid", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS18", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS19": SimStruct(OrderedDict((("MismatchingOutputDimensionsSupported", SimTypeInt(signed=True, label="Int32")), ("SupportedSampleCountsWithNoOutputs", SimTypeInt(signed=False, label="UInt32")), ("PointSamplingAddressesNeverRoundUp", SimTypeInt(signed=True, label="Int32")), ("RasterizerDesc2Supported", SimTypeInt(signed=True, label="Int32")), ("NarrowQuadrilateralLinesSupported", SimTypeInt(signed=True, label="Int32")), ("AnisoFilterWithPointMipSupported", SimTypeInt(signed=True, label="Int32")), ("MaxSamplerDescriptorHeapSize", SimTypeInt(signed=False, label="UInt32")), ("MaxSamplerDescriptorHeapSizeWithStaticSamplers", SimTypeInt(signed=False, label="UInt32")), ("MaxViewDescriptorHeapSize", SimTypeInt(signed=False, label="UInt32")), ("ComputeOnlyCustomHeapSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS19", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS2": SimStruct(OrderedDict((("DepthBoundsTestSupported", SimTypeInt(signed=True, label="Int32")), ("ProgrammableSamplePositionsTier", SimTypeInt(signed=False, label="D3D12_PROGRAMMABLE_SAMPLE_POSITIONS_TIER")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS2", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS20": SimStruct(OrderedDict((("ComputeOnlyWriteWatchSupported", SimTypeInt(signed=True, label="Int32")), ("RecreateAtTier", SimTypeInt(signed=False, label="D3D12_RECREATE_AT_TIER")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS20", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS3": SimStruct(OrderedDict((("CopyQueueTimestampQueriesSupported", SimTypeInt(signed=True, label="Int32")), ("CastingFullyTypedFormatSupported", SimTypeInt(signed=True, label="Int32")), ("WriteBufferImmediateSupportFlags", SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_SUPPORT_FLAGS")), ("ViewInstancingTier", SimTypeInt(signed=False, label="D3D12_VIEW_INSTANCING_TIER")), ("BarycentricsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS3", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS4": SimStruct(OrderedDict((("MSAA64KBAlignedTextureSupported", SimTypeInt(signed=True, label="Int32")), ("SharedResourceCompatibilityTier", SimTypeInt(signed=False, label="D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER")), ("Native16BitShaderOpsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS4", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS5": SimStruct(OrderedDict((("SRVOnlyTiledResourceTier3", SimTypeInt(signed=True, label="Int32")), ("RenderPassesTier", SimTypeInt(signed=False, label="D3D12_RENDER_PASS_TIER")), ("RaytracingTier", SimTypeInt(signed=False, label="D3D12_RAYTRACING_TIER")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS5", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS6": SimStruct(OrderedDict((("AdditionalShadingRatesSupported", SimTypeInt(signed=True, label="Int32")), ("PerPrimitiveShadingRateSupportedWithViewportIndexing", SimTypeInt(signed=True, label="Int32")), ("VariableShadingRateTier", SimTypeInt(signed=False, label="D3D12_VARIABLE_SHADING_RATE_TIER")), ("ShadingRateImageTileSize", SimTypeInt(signed=False, label="UInt32")), ("BackgroundProcessingSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS6", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS7": SimStruct(OrderedDict((("MeshShaderTier", SimTypeInt(signed=False, label="D3D12_MESH_SHADER_TIER")), ("SamplerFeedbackTier", SimTypeInt(signed=False, label="D3D12_SAMPLER_FEEDBACK_TIER")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS7", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS8": SimStruct(OrderedDict((("UnalignedBlockTexturesSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS8", pack=False, align=None),
    "D3D12_FEATURE_DATA_D3D12_OPTIONS9": SimStruct(OrderedDict((("MeshShaderPipelineStatsSupported", SimTypeInt(signed=True, label="Int32")), ("MeshShaderSupportsFullRangeRenderTargetArrayIndex", SimTypeInt(signed=True, label="Int32")), ("AtomicInt64OnTypedResourceSupported", SimTypeInt(signed=True, label="Int32")), ("AtomicInt64OnGroupSharedSupported", SimTypeInt(signed=True, label="Int32")), ("DerivativesInMeshAndAmplificationShadersSupported", SimTypeInt(signed=True, label="Int32")), ("WaveMMATier", SimTypeInt(signed=False, label="D3D12_WAVE_MMA_TIER")),)), name="D3D12_FEATURE_DATA_D3D12_OPTIONS9", pack=False, align=None),
    "D3D12_FEATURE_DATA_DISPLAYABLE": SimStruct(OrderedDict((("DisplayableTexture", SimTypeInt(signed=True, label="Int32")), ("SharedResourceCompatibilityTier", SimTypeInt(signed=False, label="D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER")),)), name="D3D12_FEATURE_DATA_DISPLAYABLE", pack=False, align=None),
    "D3D12_FEATURE_DATA_EXISTING_HEAPS": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_EXISTING_HEAPS", pack=False, align=None),
    "D3D12_FEATURE_DATA_FEATURE_LEVELS": SimStruct(OrderedDict((("NumFeatureLevels", SimTypeInt(signed=False, label="UInt32")), ("pFeatureLevelsRequested", SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), offset=0)), ("MaxSupportedFeatureLevel", SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL")),)), name="D3D12_FEATURE_DATA_FEATURE_LEVELS", pack=False, align=None),
    "D3D12_FEATURE_DATA_FORMAT_INFO": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("PlaneCount", SimTypeChar(label="Byte")),)), name="D3D12_FEATURE_DATA_FORMAT_INFO", pack=False, align=None),
    "D3D12_FEATURE_DATA_FORMAT_SUPPORT": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Support1", SimTypeInt(signed=False, label="D3D12_FORMAT_SUPPORT1")), ("Support2", SimTypeInt(signed=False, label="D3D12_FORMAT_SUPPORT2")),)), name="D3D12_FEATURE_DATA_FORMAT_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_GPU_VIRTUAL_ADDRESS_SUPPORT": SimStruct(OrderedDict((("MaxGPUVirtualAddressBitsPerResource", SimTypeInt(signed=False, label="UInt32")), ("MaxGPUVirtualAddressBitsPerProcess", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_GPU_VIRTUAL_ADDRESS_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_HARDWARE_COPY": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_HARDWARE_COPY", pack=False, align=None),
    "D3D12_FEATURE_DATA_MULTISAMPLE_QUALITY_LEVELS": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_MULTISAMPLE_QUALITY_LEVEL_FLAGS")), ("NumQualityLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_MULTISAMPLE_QUALITY_LEVELS", pack=False, align=None),
    "D3D12_FEATURE_DATA_PLACED_RESOURCE_SUPPORT_INFO": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Dimension", SimTypeInt(signed=False, label="D3D12_RESOURCE_DIMENSION")), ("DestHeapProperties", SimTypeRef("D3D12_HEAP_PROPERTIES", SimStruct)), ("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_PLACED_RESOURCE_SUPPORT_INFO", pack=False, align=None),
    "D3D12_FEATURE_DATA_PREDICATION": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_PREDICATION", pack=False, align=None),
    "D3D12_FEATURE_DATA_PROTECTED_RESOURCE_SESSION_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Support", SimTypeInt(signed=False, label="D3D12_PROTECTED_RESOURCE_SESSION_SUPPORT_FLAGS")),)), name="D3D12_FEATURE_DATA_PROTECTED_RESOURCE_SESSION_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_PROTECTED_RESOURCE_SESSION_TYPES": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("pTypes", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="D3D12_FEATURE_DATA_PROTECTED_RESOURCE_SESSION_TYPES", pack=False, align=None),
    "D3D12_FEATURE_DATA_PROTECTED_RESOURCE_SESSION_TYPE_COUNT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_PROTECTED_RESOURCE_SESSION_TYPE_COUNT", pack=False, align=None),
    "D3D12_FEATURE_DATA_QUERY_META_COMMAND": SimStruct(OrderedDict((("CommandId", SimTypeBottom(label="Guid")), ("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("pQueryInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QueryInputDataSizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pQueryOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QueryOutputDataSizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_FEATURE_DATA_QUERY_META_COMMAND", pack=False, align=None),
    "D3D12_FEATURE_DATA_ROOT_SIGNATURE": SimStruct(OrderedDict((("HighestVersion", SimTypeInt(signed=False, label="D3D_ROOT_SIGNATURE_VERSION")),)), name="D3D12_FEATURE_DATA_ROOT_SIGNATURE", pack=False, align=None),
    "D3D12_FEATURE_DATA_SERIALIZATION": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("HeapSerializationTier", SimTypeInt(signed=False, label="D3D12_HEAP_SERIALIZATION_TIER")),)), name="D3D12_FEATURE_DATA_SERIALIZATION", pack=False, align=None),
    "D3D12_FEATURE_DATA_SHADER_CACHE": SimStruct(OrderedDict((("SupportFlags", SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_SUPPORT_FLAGS")),)), name="D3D12_FEATURE_DATA_SHADER_CACHE", pack=False, align=None),
    "D3D12_FEATURE_DATA_SHADER_MODEL": SimStruct(OrderedDict((("HighestShaderModel", SimTypeInt(signed=False, label="D3D_SHADER_MODEL")),)), name="D3D12_FEATURE_DATA_SHADER_MODEL", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ARCHITECTURE": SimStruct(OrderedDict((("IOCoherent", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ARCHITECTURE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODER_HEAP_SIZE": SimStruct(OrderedDict((("VideoDecoderHeapDesc", SimTypeRef("D3D12_VIDEO_DECODER_HEAP_DESC", SimStruct)), ("MemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODER_HEAP_SIZE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODER_HEAP_SIZE1": SimStruct(OrderedDict((("VideoDecoderHeapDesc", SimTypeRef("D3D12_VIDEO_DECODER_HEAP_DESC", SimStruct)), ("Protected", SimTypeInt(signed=True, label="Int32")), ("MemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODER_HEAP_SIZE1", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_CONVERSION_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)), ("DecodeSample", SimTypeRef("D3D12_VIDEO_SAMPLE", SimStruct)), ("OutputFormat", SimTypeRef("D3D12_VIDEO_FORMAT", SimStruct)), ("FrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("BitRate", SimTypeInt(signed=False, label="UInt32")), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_CONVERSION_SUPPORT_FLAGS")), ("ScaleSupport", SimTypeRef("D3D12_VIDEO_SCALE_SUPPORT", SimStruct)),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_CONVERSION_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_FORMATS": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)), ("FormatCount", SimTypeInt(signed=False, label="UInt32")), ("pOutputFormats", SimTypePointer(SimTypeInt(signed=False, label="DXGI_FORMAT"), offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_FORMATS", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_FORMAT_COUNT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)), ("FormatCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_FORMAT_COUNT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_HISTOGRAM": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("DecodeProfile", SimTypeBottom(label="Guid")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("DecodeFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Components", SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_FLAGS")), ("BinCount", SimTypeInt(signed=False, label="UInt32")), ("CounterBitDepth", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_HISTOGRAM", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_PROFILES": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("ProfileCount", SimTypeInt(signed=False, label="UInt32")), ("pProfiles", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_PROFILES", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_PROFILE_COUNT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("ProfileCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_PROFILE_COUNT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_PROTECTED_RESOURCES": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROTECTED_RESOURCE_SUPPORT_FLAGS")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_PROTECTED_RESOURCES", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_DECODE_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("DecodeFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("FrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("BitRate", SimTypeInt(signed=False, label="UInt32")), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_SUPPORT_FLAGS")), ("ConfigurationFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_CONFIGURATION_FLAGS")), ("DecodeTier", SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_TIER")),)), name="D3D12_FEATURE_DATA_VIDEO_DECODE_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_CODEC": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_CODEC", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("IsSupported", SimTypeInt(signed=True, label="Int32")), ("CodecSupportLimits", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT", SimStruct)),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("IsSupported", SimTypeInt(signed=True, label="Int32")), ("PictureSupport", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT", SimStruct)),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_CONFIG": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("Level", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("SubregionMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE")), ("FrameResolution", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct)), ("CodecSupport", SimTypeRef("D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_CONFIG_SUPPORT", SimStruct)), ("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_CONFIG", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("Level", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("SubregionMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE")), ("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_HEAP_SIZE": SimStruct(OrderedDict((("HeapDesc", SimTypeRef("D3D12_VIDEO_ENCODER_HEAP_DESC", SimStruct)), ("IsSupported", SimTypeInt(signed=True, label="Int32")), ("MemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_HEAP_SIZE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_INPUT_FORMAT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_INPUT_FORMAT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_INTRA_REFRESH_MODE": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("Level", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("IntraRefreshMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_INTRA_REFRESH_MODE")), ("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_INTRA_REFRESH_MODE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_OUTPUT_RESOLUTION": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("ResolutionRatiosCount", SimTypeInt(signed=False, label="UInt32")), ("IsSupported", SimTypeInt(signed=True, label="Int32")), ("MinResolutionSupported", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct)), ("MaxResolutionSupported", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct)), ("ResolutionWidthMultipleRequirement", SimTypeInt(signed=False, label="UInt32")), ("ResolutionHeightMultipleRequirement", SimTypeInt(signed=False, label="UInt32")), ("pResolutionRatios", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_RATIO_DESC", SimStruct), offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_OUTPUT_RESOLUTION", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_OUTPUT_RESOLUTION_RATIOS_COUNT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("ResolutionRatiosCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_OUTPUT_RESOLUTION_RATIOS_COUNT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_PROFILE_LEVEL": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("IsSupported", SimTypeInt(signed=True, label="Int32")), ("MinSupportedLevel", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("MaxSupportedLevel", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_PROFILE_LEVEL", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_RATE_CONTROL_MODE": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("RateControlMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_RATE_CONTROL_MODE")), ("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_RATE_CONTROL_MODE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLUTION_SUPPORT_LIMITS": SimStruct(OrderedDict((("MaxSubregionsNumber", SimTypeInt(signed=False, label="UInt32")), ("MaxIntraRefreshFrameDuration", SimTypeInt(signed=False, label="UInt32")), ("SubregionBlockPixelsSize", SimTypeInt(signed=False, label="UInt32")), ("QPMapRegionPixelsSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLUTION_SUPPORT_LIMITS", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOURCE_REQUIREMENTS": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("Profile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("PictureTargetResolution", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct)), ("IsSupported", SimTypeInt(signed=True, label="Int32")), ("CompressedBitstreamBufferAccessAlignment", SimTypeInt(signed=False, label="UInt32")), ("EncoderMetadataBufferAccessAlignment", SimTypeInt(signed=False, label="UInt32")), ("MaxEncoderOutputMetadataBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOURCE_REQUIREMENTS", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("CodecConfiguration", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION", SimStruct)), ("CodecGopSequence", SimTypeRef("D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE", SimStruct)), ("RateControl", SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL", SimStruct)), ("IntraRefresh", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_INTRA_REFRESH_MODE")), ("SubregionFrameEncoding", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE")), ("ResolutionsListCount", SimTypeInt(signed=False, label="UInt32")), ("pResolutionList", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct), offset=0)), ("MaxReferenceFramesInDPB", SimTypeInt(signed=False, label="UInt32")), ("ValidationFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_VALIDATION_FLAGS")), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_SUPPORT_FLAGS")), ("SuggestedProfile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("SuggestedLevel", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("pResolutionDependentSupport", SimTypePointer(SimTypeRef("D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLUTION_SUPPORT_LIMITS", SimStruct), offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT1": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("Codec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("CodecConfiguration", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION", SimStruct)), ("CodecGopSequence", SimTypeRef("D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE", SimStruct)), ("RateControl", SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL", SimStruct)), ("IntraRefresh", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_INTRA_REFRESH_MODE")), ("SubregionFrameEncoding", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE")), ("ResolutionsListCount", SimTypeInt(signed=False, label="UInt32")), ("pResolutionList", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct), offset=0)), ("MaxReferenceFramesInDPB", SimTypeInt(signed=False, label="UInt32")), ("ValidationFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_VALIDATION_FLAGS")), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_SUPPORT_FLAGS")), ("SuggestedProfile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("SuggestedLevel", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("pResolutionDependentSupport", SimTypePointer(SimTypeRef("D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLUTION_SUPPORT_LIMITS", SimStruct), offset=0)), ("SubregionFrameEncodingData", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA", SimStruct)), ("MaxQualityVsSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT1", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMANDS": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("CommandCount", SimTypeInt(signed=False, label="UInt32")), ("pCommandInfos", SimTypePointer(SimTypeRef("D3D12_VIDEO_EXTENSION_COMMAND_INFO", SimStruct), offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMANDS", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_COUNT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("CommandCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_COUNT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_PARAMETERS": SimStruct(OrderedDict((("CommandId", SimTypeBottom(label="Guid")), ("Stage", SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_STAGE")), ("ParameterCount", SimTypeInt(signed=False, label="UInt32")), ("pParameterInfos", SimTypePointer(SimTypeRef("D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_INFO", SimStruct), offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_PARAMETERS", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_PARAMETER_COUNT": SimStruct(OrderedDict((("CommandId", SimTypeBottom(label="Guid")), ("Stage", SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_STAGE")), ("ParameterCount", SimTypeInt(signed=False, label="UInt32")), ("ParameterPacking", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_PARAMETER_COUNT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_SIZE": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("CommandId", SimTypeBottom(label="Guid")), ("pCreationParameters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CreationParametersSizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_SIZE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("CommandId", SimTypeBottom(label="Guid")), ("pInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InputDataSizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputDataSizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_FEATURE_DATA_VIDEO_EXTENSION_COMMAND_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_FEATURE_AREA_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("VideoDecodeSupport", SimTypeInt(signed=True, label="Int32")), ("VideoProcessSupport", SimTypeInt(signed=True, label="Int32")), ("VideoEncodeSupport", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FEATURE_DATA_VIDEO_FEATURE_AREA_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_MOTION_ESTIMATOR": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("BlockSizeFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_FLAGS")), ("PrecisionFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION_FLAGS")), ("SizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)),)), name="D3D12_FEATURE_DATA_VIDEO_MOTION_ESTIMATOR", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_MOTION_ESTIMATOR_PROTECTED_RESOURCES": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROTECTED_RESOURCE_SUPPORT_FLAGS")),)), name="D3D12_FEATURE_DATA_VIDEO_MOTION_ESTIMATOR_PROTECTED_RESOURCES", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_MOTION_ESTIMATOR_SIZE": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("BlockSize", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE")), ("Precision", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION")), ("SizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)), ("Protected", SimTypeInt(signed=True, label="Int32")), ("MotionVectorHeapMemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MotionVectorHeapMemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")), ("MotionEstimatorMemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MotionEstimatorMemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_MOTION_ESTIMATOR_SIZE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_PROCESSOR_SIZE": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("pOutputStreamDesc", SimTypePointer(SimTypeRef("D3D12_VIDEO_PROCESS_OUTPUT_STREAM_DESC", SimStruct), offset=0)), ("NumInputStreamDescs", SimTypeInt(signed=False, label="UInt32")), ("pInputStreamDescs", SimTypePointer(SimTypeRef("D3D12_VIDEO_PROCESS_INPUT_STREAM_DESC", SimStruct), offset=0)), ("MemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_PROCESSOR_SIZE", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_PROCESSOR_SIZE1": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("pOutputStreamDesc", SimTypePointer(SimTypeRef("D3D12_VIDEO_PROCESS_OUTPUT_STREAM_DESC", SimStruct), offset=0)), ("NumInputStreamDescs", SimTypeInt(signed=False, label="UInt32")), ("pInputStreamDescs", SimTypePointer(SimTypeRef("D3D12_VIDEO_PROCESS_INPUT_STREAM_DESC", SimStruct), offset=0)), ("Protected", SimTypeInt(signed=True, label="Int32")), ("MemoryPoolL0Size", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryPoolL1Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_FEATURE_DATA_VIDEO_PROCESSOR_SIZE1", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_PROCESS_MAX_INPUT_STREAMS": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxInputStreams", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_PROCESS_MAX_INPUT_STREAMS", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_PROCESS_PROTECTED_RESOURCES": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROTECTED_RESOURCE_SUPPORT_FLAGS")),)), name="D3D12_FEATURE_DATA_VIDEO_PROCESS_PROTECTED_RESOURCES", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_PROCESS_REFERENCE_INFO": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("DeinterlaceMode", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_DEINTERLACE_FLAGS")), ("Filters", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FILTER_FLAGS")), ("FeatureSupport", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FEATURE_FLAGS")), ("InputFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("OutputFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("EnableAutoProcessing", SimTypeInt(signed=True, label="Int32")), ("PastFrames", SimTypeInt(signed=False, label="UInt32")), ("FutureFrames", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_FEATURE_DATA_VIDEO_PROCESS_REFERENCE_INFO", pack=False, align=None),
    "D3D12_FEATURE_DATA_VIDEO_PROCESS_SUPPORT": SimStruct(OrderedDict((("NodeIndex", SimTypeInt(signed=False, label="UInt32")), ("InputSample", SimTypeRef("D3D12_VIDEO_SAMPLE", SimStruct)), ("InputFieldType", SimTypeInt(signed=False, label="D3D12_VIDEO_FIELD_TYPE")), ("InputStereoFormat", SimTypeInt(signed=False, label="D3D12_VIDEO_FRAME_STEREO_FORMAT")), ("InputFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("OutputFormat", SimTypeRef("D3D12_VIDEO_FORMAT", SimStruct)), ("OutputStereoFormat", SimTypeInt(signed=False, label="D3D12_VIDEO_FRAME_STEREO_FORMAT")), ("OutputFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_SUPPORT_FLAGS")), ("ScaleSupport", SimTypeRef("D3D12_VIDEO_SCALE_SUPPORT", SimStruct)), ("FeatureSupport", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FEATURE_FLAGS")), ("DeinterlaceSupport", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_DEINTERLACE_FLAGS")), ("AutoProcessingSupport", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_AUTO_PROCESSING_FLAGS")), ("FilterSupport", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FILTER_FLAGS")), ("FilterRangeSupport", SimTypeArray(SimTypeRef("D3D12_VIDEO_PROCESS_FILTER_RANGE", SimStruct), 32)),)), name="D3D12_FEATURE_DATA_VIDEO_PROCESS_SUPPORT", pack=False, align=None),
    "D3D12_FEATURE_VIDEO": SimTypeInt(signed=False, label="D3D12_FEATURE_VIDEO"),
    "D3D12_FENCE_FLAGS": SimTypeInt(signed=False, label="D3D12_FENCE_FLAGS"),
    "D3D12_FILL_MODE": SimTypeInt(signed=False, label="D3D12_FILL_MODE"),
    "D3D12_FILTER": SimTypeInt(signed=False, label="D3D12_FILTER"),
    "D3D12_FILTER_REDUCTION_TYPE": SimTypeInt(signed=False, label="D3D12_FILTER_REDUCTION_TYPE"),
    "D3D12_FILTER_TYPE": SimTypeInt(signed=False, label="D3D12_FILTER_TYPE"),
    "D3D12_FORMAT_SUPPORT1": SimTypeInt(signed=False, label="D3D12_FORMAT_SUPPORT1"),
    "D3D12_FORMAT_SUPPORT2": SimTypeInt(signed=False, label="D3D12_FORMAT_SUPPORT2"),
    "D3D12_FUNCTION_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("BoundResources", SimTypeInt(signed=False, label="UInt32")), ("InstructionCount", SimTypeInt(signed=False, label="UInt32")), ("TempRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("TempArrayCount", SimTypeInt(signed=False, label="UInt32")), ("DefCount", SimTypeInt(signed=False, label="UInt32")), ("DclCount", SimTypeInt(signed=False, label="UInt32")), ("TextureNormalInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureLoadInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureCompInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureBiasInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureGradientInstructions", SimTypeInt(signed=False, label="UInt32")), ("FloatInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("IntInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("UintInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("StaticFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("MacroInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ArrayInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("MovInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("MovcInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ConversionInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("BitwiseInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("MinFeatureLevel", SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL")), ("RequiredFeatureFlags", SimTypeLongLong(signed=False, label="UInt64")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FunctionParameterCount", SimTypeInt(signed=True, label="Int32")), ("HasReturn", SimTypeInt(signed=True, label="Int32")), ("Has10Level9VertexShader", SimTypeInt(signed=True, label="Int32")), ("Has10Level9PixelShader", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_FUNCTION_DESC", pack=False, align=None),
    "D3D12_GLOBAL_BARRIER": SimStruct(OrderedDict((("SyncBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC")), ("SyncAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC")), ("AccessBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS")), ("AccessAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS")),)), name="D3D12_GLOBAL_BARRIER", pack=False, align=None),
    "D3D12_GLOBAL_ROOT_SIGNATURE": SimStruct(OrderedDict((("pGlobalRootSignature", SimTypeBottom(label="ID3D12RootSignature")),)), name="D3D12_GLOBAL_ROOT_SIGNATURE", pack=False, align=None),
    "D3D12_GPU_BASED_VALIDATION_FLAGS": SimTypeInt(signed=False, label="D3D12_GPU_BASED_VALIDATION_FLAGS"),
    "D3D12_GPU_BASED_VALIDATION_PIPELINE_STATE_CREATE_FLAGS": SimTypeInt(signed=False, label="D3D12_GPU_BASED_VALIDATION_PIPELINE_STATE_CREATE_FLAGS"),
    "D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE": SimTypeInt(signed=False, label="D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE"),
    "D3D12_GPU_DESCRIPTOR_HANDLE": SimStruct(OrderedDict((("ptr", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_GPU_DESCRIPTOR_HANDLE", pack=False, align=None),
    "D3D12_GPU_VIRTUAL_ADDRESS_AND_STRIDE": SimStruct(OrderedDict((("StartAddress", SimTypeLongLong(signed=False, label="UInt64")), ("StrideInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_GPU_VIRTUAL_ADDRESS_AND_STRIDE", pack=False, align=None),
    "D3D12_GPU_VIRTUAL_ADDRESS_RANGE": SimStruct(OrderedDict((("StartAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_GPU_VIRTUAL_ADDRESS_RANGE", pack=False, align=None),
    "D3D12_GPU_VIRTUAL_ADDRESS_RANGE_AND_STRIDE": SimStruct(OrderedDict((("StartAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("StrideInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_GPU_VIRTUAL_ADDRESS_RANGE_AND_STRIDE", pack=False, align=None),
    "D3D12_GRAPHICS_PIPELINE_STATE_DESC": SimStruct(OrderedDict((("pRootSignature", SimTypeBottom(label="ID3D12RootSignature")), ("VS", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("PS", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("DS", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("HS", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("GS", SimTypeRef("D3D12_SHADER_BYTECODE", SimStruct)), ("StreamOutput", SimTypeRef("D3D12_STREAM_OUTPUT_DESC", SimStruct)), ("BlendState", SimTypeRef("D3D12_BLEND_DESC", SimStruct)), ("SampleMask", SimTypeInt(signed=False, label="UInt32")), ("RasterizerState", SimTypeRef("D3D12_RASTERIZER_DESC", SimStruct)), ("DepthStencilState", SimTypeRef("D3D12_DEPTH_STENCIL_DESC", SimStruct)), ("InputLayout", SimTypeRef("D3D12_INPUT_LAYOUT_DESC", SimStruct)), ("IBStripCutValue", SimTypeInt(signed=False, label="D3D12_INDEX_BUFFER_STRIP_CUT_VALUE")), ("PrimitiveTopologyType", SimTypeInt(signed=False, label="D3D12_PRIMITIVE_TOPOLOGY_TYPE")), ("NumRenderTargets", SimTypeInt(signed=False, label="UInt32")), ("RTVFormats", SimTypeArray(SimTypeInt(signed=False, label="DXGI_FORMAT"), 8)), ("DSVFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("CachedPSO", SimTypeRef("D3D12_CACHED_PIPELINE_STATE", SimStruct)), ("Flags", SimTypeInt(signed=False, label="D3D12_PIPELINE_STATE_FLAGS")),)), name="D3D12_GRAPHICS_PIPELINE_STATE_DESC", pack=False, align=None),
    "D3D12_GRAPHICS_STATES": SimTypeInt(signed=False, label="D3D12_GRAPHICS_STATES"),
    "D3D12_HEAP_DESC": SimStruct(OrderedDict((("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Properties", SimTypeRef("D3D12_HEAP_PROPERTIES", SimStruct)), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="D3D12_HEAP_FLAGS")),)), name="D3D12_HEAP_DESC", pack=False, align=None),
    "D3D12_HEAP_FLAGS": SimTypeInt(signed=False, label="D3D12_HEAP_FLAGS"),
    "D3D12_HEAP_PROPERTIES": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_HEAP_TYPE")), ("CPUPageProperty", SimTypeInt(signed=False, label="D3D12_CPU_PAGE_PROPERTY")), ("MemoryPoolPreference", SimTypeInt(signed=False, label="D3D12_MEMORY_POOL")), ("CreationNodeMask", SimTypeInt(signed=False, label="UInt32")), ("VisibleNodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_HEAP_PROPERTIES", pack=False, align=None),
    "D3D12_HEAP_SERIALIZATION_TIER": SimTypeInt(signed=False, label="D3D12_HEAP_SERIALIZATION_TIER"),
    "D3D12_HEAP_TYPE": SimTypeInt(signed=False, label="D3D12_HEAP_TYPE"),
    "D3D12_HIT_GROUP_DESC": SimStruct(OrderedDict((("HitGroupExport", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D12_HIT_GROUP_TYPE")), ("AnyHitShaderImport", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClosestHitShaderImport", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IntersectionShaderImport", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="D3D12_HIT_GROUP_DESC", pack=False, align=None),
    "D3D12_HIT_GROUP_TYPE": SimTypeInt(signed=False, label="D3D12_HIT_GROUP_TYPE"),
    "D3D12_HIT_KIND": SimTypeInt(signed=False, label="D3D12_HIT_KIND"),
    "D3D12_INDEX_BUFFER_STRIP_CUT_VALUE": SimTypeInt(signed=False, label="D3D12_INDEX_BUFFER_STRIP_CUT_VALUE"),
    "D3D12_INDEX_BUFFER_VIEW": SimStruct(OrderedDict((("BufferLocation", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")),)), name="D3D12_INDEX_BUFFER_VIEW", pack=False, align=None),
    "D3D12_INDIRECT_ARGUMENT_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_INDIRECT_ARGUMENT_TYPE")), ("Anonymous", SimUnion({"VertexBuffer": SimTypeRef("_VertexBuffer_e__Struct", SimStruct), "Constant": SimTypeRef("_Constant_e__Struct", SimStruct), "ConstantBufferView": SimTypeRef("_ConstantBufferView_e__Struct", SimStruct), "ShaderResourceView": SimTypeRef("_ShaderResourceView_e__Struct", SimStruct), "UnorderedAccessView": SimTypeRef("_UnorderedAccessView_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_INDIRECT_ARGUMENT_DESC", pack=False, align=None),
    "D3D12_INDIRECT_ARGUMENT_TYPE": SimTypeInt(signed=False, label="D3D12_INDIRECT_ARGUMENT_TYPE"),
    "D3D12_INFO_QUEUE_FILTER": SimStruct(OrderedDict((("AllowList", SimTypeRef("D3D12_INFO_QUEUE_FILTER_DESC", SimStruct)), ("DenyList", SimTypeRef("D3D12_INFO_QUEUE_FILTER_DESC", SimStruct)),)), name="D3D12_INFO_QUEUE_FILTER", pack=False, align=None),
    "D3D12_INFO_QUEUE_FILTER_DESC": SimStruct(OrderedDict((("NumCategories", SimTypeInt(signed=False, label="UInt32")), ("pCategoryList", SimTypePointer(SimTypeInt(signed=False, label="D3D12_MESSAGE_CATEGORY"), offset=0)), ("NumSeverities", SimTypeInt(signed=False, label="UInt32")), ("pSeverityList", SimTypePointer(SimTypeInt(signed=False, label="D3D12_MESSAGE_SEVERITY"), offset=0)), ("NumIDs", SimTypeInt(signed=False, label="UInt32")), ("pIDList", SimTypePointer(SimTypeInt(signed=False, label="D3D12_MESSAGE_ID"), offset=0)),)), name="D3D12_INFO_QUEUE_FILTER_DESC", pack=False, align=None),
    "D3D12_INPUT_CLASSIFICATION": SimTypeInt(signed=False, label="D3D12_INPUT_CLASSIFICATION"),
    "D3D12_INPUT_ELEMENT_DESC": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("InputSlot", SimTypeInt(signed=False, label="UInt32")), ("AlignedByteOffset", SimTypeInt(signed=False, label="UInt32")), ("InputSlotClass", SimTypeInt(signed=False, label="D3D12_INPUT_CLASSIFICATION")), ("InstanceDataStepRate", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_INPUT_ELEMENT_DESC", pack=False, align=None),
    "D3D12_INPUT_LAYOUT_DESC": SimStruct(OrderedDict((("pInputElementDescs", SimTypePointer(SimTypeRef("D3D12_INPUT_ELEMENT_DESC", SimStruct), label="LPArray", offset=0)), ("NumElements", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_INPUT_LAYOUT_DESC", pack=False, align=None),
    "D3D12_LIBRARY_DESC": SimStruct(OrderedDict((("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FunctionCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_LIBRARY_DESC", pack=False, align=None),
    "D3D12_LIFETIME_STATE": SimTypeInt(signed=False, label="D3D12_LIFETIME_STATE"),
    "D3D12_LINE_RASTERIZATION_MODE": SimTypeInt(signed=False, label="D3D12_LINE_RASTERIZATION_MODE"),
    "D3D12_LOCAL_ROOT_SIGNATURE": SimStruct(OrderedDict((("pLocalRootSignature", SimTypeBottom(label="ID3D12RootSignature")),)), name="D3D12_LOCAL_ROOT_SIGNATURE", pack=False, align=None),
    "D3D12_LOGIC_OP": SimTypeInt(signed=False, label="D3D12_LOGIC_OP"),
    "D3D12_MEASUREMENTS_ACTION": SimTypeInt(signed=False, label="D3D12_MEASUREMENTS_ACTION"),
    "D3D12_MEMCPY_DEST": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RowPitch", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SlicePitch", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_MEMCPY_DEST", pack=False, align=None),
    "D3D12_MEMORY_POOL": SimTypeInt(signed=False, label="D3D12_MEMORY_POOL"),
    "D3D12_MESH_SHADER_TIER": SimTypeInt(signed=False, label="D3D12_MESH_SHADER_TIER"),
    "D3D12_MESSAGE": SimStruct(OrderedDict((("Category", SimTypeInt(signed=False, label="D3D12_MESSAGE_CATEGORY")), ("Severity", SimTypeInt(signed=False, label="D3D12_MESSAGE_SEVERITY")), ("ID", SimTypeInt(signed=False, label="D3D12_MESSAGE_ID")), ("pDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DescriptionByteLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_MESSAGE", pack=False, align=None),
    "D3D12_MESSAGE_CALLBACK_FLAGS": SimTypeInt(signed=False, label="D3D12_MESSAGE_CALLBACK_FLAGS"),
    "D3D12_MESSAGE_CATEGORY": SimTypeInt(signed=False, label="D3D12_MESSAGE_CATEGORY"),
    "D3D12_MESSAGE_ID": SimTypeInt(signed=False, label="D3D12_MESSAGE_ID"),
    "D3D12_MESSAGE_SEVERITY": SimTypeInt(signed=False, label="D3D12_MESSAGE_SEVERITY"),
    "D3D12_META_COMMAND_DESC": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InitializationDirtyState", SimTypeInt(signed=False, label="D3D12_GRAPHICS_STATES")), ("ExecutionDirtyState", SimTypeInt(signed=False, label="D3D12_GRAPHICS_STATES")),)), name="D3D12_META_COMMAND_DESC", pack=False, align=None),
    "D3D12_META_COMMAND_PARAMETER_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D12_META_COMMAND_PARAMETER_TYPE")), ("Flags", SimTypeInt(signed=False, label="D3D12_META_COMMAND_PARAMETER_FLAGS")), ("RequiredResourceState", SimTypeInt(signed=False, label="D3D12_RESOURCE_STATES")), ("StructureOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_META_COMMAND_PARAMETER_DESC", pack=False, align=None),
    "D3D12_META_COMMAND_PARAMETER_FLAGS": SimTypeInt(signed=False, label="D3D12_META_COMMAND_PARAMETER_FLAGS"),
    "D3D12_META_COMMAND_PARAMETER_STAGE": SimTypeInt(signed=False, label="D3D12_META_COMMAND_PARAMETER_STAGE"),
    "D3D12_META_COMMAND_PARAMETER_TYPE": SimTypeInt(signed=False, label="D3D12_META_COMMAND_PARAMETER_TYPE"),
    "D3D12_MIP_REGION": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_MIP_REGION", pack=False, align=None),
    "D3D12_MULTIPLE_FENCE_WAIT_FLAGS": SimTypeInt(signed=False, label="D3D12_MULTIPLE_FENCE_WAIT_FLAGS"),
    "D3D12_MULTISAMPLE_QUALITY_LEVEL_FLAGS": SimTypeInt(signed=False, label="D3D12_MULTISAMPLE_QUALITY_LEVEL_FLAGS"),
    "D3D12_NODE_MASK": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_NODE_MASK", pack=False, align=None),
    "D3D12_PACKED_MIP_INFO": SimStruct(OrderedDict((("NumStandardMips", SimTypeChar(label="Byte")), ("NumPackedMips", SimTypeChar(label="Byte")), ("NumTilesForPackedMips", SimTypeInt(signed=False, label="UInt32")), ("StartTileIndexInOverallResource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_PACKED_MIP_INFO", pack=False, align=None),
    "D3D12_PARAMETER_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("InterpolationMode", SimTypeInt(signed=False, label="D3D_INTERPOLATION_MODE")), ("Flags", SimTypeInt(signed=False, label="D3D_PARAMETER_FLAGS")), ("FirstInRegister", SimTypeInt(signed=False, label="UInt32")), ("FirstInComponent", SimTypeInt(signed=False, label="UInt32")), ("FirstOutRegister", SimTypeInt(signed=False, label="UInt32")), ("FirstOutComponent", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_PARAMETER_DESC", pack=False, align=None),
    "D3D12_PIPELINE_STATE_FLAGS": SimTypeInt(signed=False, label="D3D12_PIPELINE_STATE_FLAGS"),
    "D3D12_PIPELINE_STATE_STREAM_DESC": SimStruct(OrderedDict((("SizeInBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pPipelineStateSubobjectStream", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3D12_PIPELINE_STATE_STREAM_DESC", pack=False, align=None),
    "D3D12_PIPELINE_STATE_SUBOBJECT_TYPE": SimTypeInt(signed=False, label="D3D12_PIPELINE_STATE_SUBOBJECT_TYPE"),
    "D3D12_PLACED_SUBRESOURCE_FOOTPRINT": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Footprint", SimTypeRef("D3D12_SUBRESOURCE_FOOTPRINT", SimStruct)),)), name="D3D12_PLACED_SUBRESOURCE_FOOTPRINT", pack=False, align=None),
    "D3D12_PREDICATION_OP": SimTypeInt(signed=False, label="D3D12_PREDICATION_OP"),
    "D3D12_PRIMITIVE_TOPOLOGY_TYPE": SimTypeInt(signed=False, label="D3D12_PRIMITIVE_TOPOLOGY_TYPE"),
    "D3D12_PROGRAMMABLE_SAMPLE_POSITIONS_TIER": SimTypeInt(signed=False, label="D3D12_PROGRAMMABLE_SAMPLE_POSITIONS_TIER"),
    "D3D12_PROTECTED_RESOURCE_SESSION_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_PROTECTED_RESOURCE_SESSION_FLAGS")),)), name="D3D12_PROTECTED_RESOURCE_SESSION_DESC", pack=False, align=None),
    "D3D12_PROTECTED_RESOURCE_SESSION_DESC1": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_PROTECTED_RESOURCE_SESSION_FLAGS")), ("ProtectionType", SimTypeBottom(label="Guid")),)), name="D3D12_PROTECTED_RESOURCE_SESSION_DESC1", pack=False, align=None),
    "D3D12_PROTECTED_RESOURCE_SESSION_FLAGS": SimTypeInt(signed=False, label="D3D12_PROTECTED_RESOURCE_SESSION_FLAGS"),
    "D3D12_PROTECTED_RESOURCE_SESSION_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_PROTECTED_RESOURCE_SESSION_SUPPORT_FLAGS"),
    "D3D12_PROTECTED_SESSION_STATUS": SimTypeInt(signed=False, label="D3D12_PROTECTED_SESSION_STATUS"),
    "D3D12_QUERY_DATA_PIPELINE_STATISTICS": SimStruct(OrderedDict((("IAVertices", SimTypeLongLong(signed=False, label="UInt64")), ("IAPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("VSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("CInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("PSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("HSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("DSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CSInvocations", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_QUERY_DATA_PIPELINE_STATISTICS", pack=False, align=None),
    "D3D12_QUERY_DATA_PIPELINE_STATISTICS1": SimStruct(OrderedDict((("IAVertices", SimTypeLongLong(signed=False, label="UInt64")), ("IAPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("VSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("GSPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("CInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CPrimitives", SimTypeLongLong(signed=False, label="UInt64")), ("PSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("HSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("DSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("CSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("ASInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("MSInvocations", SimTypeLongLong(signed=False, label="UInt64")), ("MSPrimitives", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_QUERY_DATA_PIPELINE_STATISTICS1", pack=False, align=None),
    "D3D12_QUERY_DATA_SO_STATISTICS": SimStruct(OrderedDict((("NumPrimitivesWritten", SimTypeLongLong(signed=False, label="UInt64")), ("PrimitivesStorageNeeded", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_QUERY_DATA_SO_STATISTICS", pack=False, align=None),
    "D3D12_QUERY_DATA_VIDEO_DECODE_STATISTICS": SimStruct(OrderedDict((("Status", SimTypeLongLong(signed=False, label="UInt64")), ("NumMacroblocksAffected", SimTypeLongLong(signed=False, label="UInt64")), ("FrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("BitRate", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_QUERY_DATA_VIDEO_DECODE_STATISTICS", pack=False, align=None),
    "D3D12_QUERY_HEAP_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_QUERY_HEAP_TYPE")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("NodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_QUERY_HEAP_DESC", pack=False, align=None),
    "D3D12_QUERY_HEAP_TYPE": SimTypeInt(signed=False, label="D3D12_QUERY_HEAP_TYPE"),
    "D3D12_QUERY_TYPE": SimTypeInt(signed=False, label="D3D12_QUERY_TYPE"),
    "D3D12_RANGE": SimStruct(OrderedDict((("Begin", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("End", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_RANGE", pack=False, align=None),
    "D3D12_RANGE_UINT64": SimStruct(OrderedDict((("Begin", SimTypeLongLong(signed=False, label="UInt64")), ("End", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RANGE_UINT64", pack=False, align=None),
    "D3D12_RASTERIZER_DESC": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D12_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D12_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeInt(signed=True, label="Int32")), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("MultisampleEnable", SimTypeInt(signed=True, label="Int32")), ("AntialiasedLineEnable", SimTypeInt(signed=True, label="Int32")), ("ForcedSampleCount", SimTypeInt(signed=False, label="UInt32")), ("ConservativeRaster", SimTypeInt(signed=False, label="D3D12_CONSERVATIVE_RASTERIZATION_MODE")),)), name="D3D12_RASTERIZER_DESC", pack=False, align=None),
    "D3D12_RASTERIZER_DESC1": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D12_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D12_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeFloat(size=32)), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("MultisampleEnable", SimTypeInt(signed=True, label="Int32")), ("AntialiasedLineEnable", SimTypeInt(signed=True, label="Int32")), ("ForcedSampleCount", SimTypeInt(signed=False, label="UInt32")), ("ConservativeRaster", SimTypeInt(signed=False, label="D3D12_CONSERVATIVE_RASTERIZATION_MODE")),)), name="D3D12_RASTERIZER_DESC1", pack=False, align=None),
    "D3D12_RASTERIZER_DESC2": SimStruct(OrderedDict((("FillMode", SimTypeInt(signed=False, label="D3D12_FILL_MODE")), ("CullMode", SimTypeInt(signed=False, label="D3D12_CULL_MODE")), ("FrontCounterClockwise", SimTypeInt(signed=True, label="Int32")), ("DepthBias", SimTypeFloat(size=32)), ("DepthBiasClamp", SimTypeFloat(size=32)), ("SlopeScaledDepthBias", SimTypeFloat(size=32)), ("DepthClipEnable", SimTypeInt(signed=True, label="Int32")), ("LineRasterizationMode", SimTypeInt(signed=False, label="D3D12_LINE_RASTERIZATION_MODE")), ("ForcedSampleCount", SimTypeInt(signed=False, label="UInt32")), ("ConservativeRaster", SimTypeInt(signed=False, label="D3D12_CONSERVATIVE_RASTERIZATION_MODE")),)), name="D3D12_RASTERIZER_DESC2", pack=False, align=None),
    "D3D12_RAYTRACING_AABB": SimStruct(OrderedDict((("MinX", SimTypeFloat(size=32)), ("MinY", SimTypeFloat(size=32)), ("MinZ", SimTypeFloat(size=32)), ("MaxX", SimTypeFloat(size=32)), ("MaxY", SimTypeFloat(size=32)), ("MaxZ", SimTypeFloat(size=32)),)), name="D3D12_RAYTRACING_AABB", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_BUILD_FLAGS": SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_BUILD_FLAGS"),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_COPY_MODE": SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_COPY_MODE"),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_COMPACTED_SIZE_DESC": SimStruct(OrderedDict((("CompactedSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_COMPACTED_SIZE_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_CURRENT_SIZE_DESC": SimStruct(OrderedDict((("CurrentSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_CURRENT_SIZE_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_DESC": SimStruct(OrderedDict((("DestBuffer", SimTypeLongLong(signed=False, label="UInt64")), ("InfoType", SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_TYPE")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_SERIALIZATION_DESC": SimStruct(OrderedDict((("SerializedSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("NumBottomLevelAccelerationStructurePointers", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_SERIALIZATION_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_TOOLS_VISUALIZATION_DESC": SimStruct(OrderedDict((("DecodedSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_TOOLS_VISUALIZATION_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_TYPE": SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_TYPE"),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_PREBUILD_INFO": SimStruct(OrderedDict((("ResultDataMaxSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("ScratchDataSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("UpdateScratchDataSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_PREBUILD_INFO", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_SRV": SimStruct(OrderedDict((("Location", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_SRV", pack=False, align=None),
    "D3D12_RAYTRACING_ACCELERATION_STRUCTURE_TYPE": SimTypeInt(signed=False, label="D3D12_RAYTRACING_ACCELERATION_STRUCTURE_TYPE"),
    "D3D12_RAYTRACING_GEOMETRY_AABBS_DESC": SimStruct(OrderedDict((("AABBCount", SimTypeLongLong(signed=False, label="UInt64")), ("AABBs", SimTypeRef("D3D12_GPU_VIRTUAL_ADDRESS_AND_STRIDE", SimStruct)),)), name="D3D12_RAYTRACING_GEOMETRY_AABBS_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_GEOMETRY_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_RAYTRACING_GEOMETRY_TYPE")), ("Flags", SimTypeInt(signed=False, label="D3D12_RAYTRACING_GEOMETRY_FLAGS")), ("Anonymous", SimUnion({"Triangles": SimTypeRef("D3D12_RAYTRACING_GEOMETRY_TRIANGLES_DESC", SimStruct), "AABBs": SimTypeRef("D3D12_RAYTRACING_GEOMETRY_AABBS_DESC", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_RAYTRACING_GEOMETRY_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_GEOMETRY_FLAGS": SimTypeInt(signed=False, label="D3D12_RAYTRACING_GEOMETRY_FLAGS"),
    "D3D12_RAYTRACING_GEOMETRY_TRIANGLES_DESC": SimStruct(OrderedDict((("Transform3x4", SimTypeLongLong(signed=False, label="UInt64")), ("IndexFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("VertexFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("IndexCount", SimTypeInt(signed=False, label="UInt32")), ("VertexCount", SimTypeInt(signed=False, label="UInt32")), ("IndexBuffer", SimTypeLongLong(signed=False, label="UInt64")), ("VertexBuffer", SimTypeRef("D3D12_GPU_VIRTUAL_ADDRESS_AND_STRIDE", SimStruct)),)), name="D3D12_RAYTRACING_GEOMETRY_TRIANGLES_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_GEOMETRY_TYPE": SimTypeInt(signed=False, label="D3D12_RAYTRACING_GEOMETRY_TYPE"),
    "D3D12_RAYTRACING_INSTANCE_DESC": SimStruct(OrderedDict((("Transform", SimTypeArray(SimTypeFloat(size=32), 12)), ("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")), ("AccelerationStructure", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RAYTRACING_INSTANCE_DESC", pack=False, align=None),
    "D3D12_RAYTRACING_INSTANCE_FLAGS": SimTypeInt(signed=False, label="D3D12_RAYTRACING_INSTANCE_FLAGS"),
    "D3D12_RAYTRACING_PIPELINE_CONFIG": SimStruct(OrderedDict((("MaxTraceRecursionDepth", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RAYTRACING_PIPELINE_CONFIG", pack=False, align=None),
    "D3D12_RAYTRACING_PIPELINE_CONFIG1": SimStruct(OrderedDict((("MaxTraceRecursionDepth", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_RAYTRACING_PIPELINE_FLAGS")),)), name="D3D12_RAYTRACING_PIPELINE_CONFIG1", pack=False, align=None),
    "D3D12_RAYTRACING_PIPELINE_FLAGS": SimTypeInt(signed=False, label="D3D12_RAYTRACING_PIPELINE_FLAGS"),
    "D3D12_RAYTRACING_SHADER_CONFIG": SimStruct(OrderedDict((("MaxPayloadSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("MaxAttributeSizeInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RAYTRACING_SHADER_CONFIG", pack=False, align=None),
    "D3D12_RAYTRACING_TIER": SimTypeInt(signed=False, label="D3D12_RAYTRACING_TIER"),
    "D3D12_RAY_FLAGS": SimTypeInt(signed=False, label="D3D12_RAY_FLAGS"),
    "D3D12_RECREATE_AT_TIER": SimTypeInt(signed=False, label="D3D12_RECREATE_AT_TIER"),
    "D3D12_RENDER_PASS_BEGINNING_ACCESS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE")), ("Anonymous", SimUnion({"Clear": SimTypeRef("D3D12_RENDER_PASS_BEGINNING_ACCESS_CLEAR_PARAMETERS", SimStruct), "PreserveLocal": SimTypeRef("D3D12_RENDER_PASS_BEGINNING_ACCESS_PRESERVE_LOCAL_PARAMETERS", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_RENDER_PASS_BEGINNING_ACCESS", pack=False, align=None),
    "D3D12_RENDER_PASS_BEGINNING_ACCESS_CLEAR_PARAMETERS": SimStruct(OrderedDict((("ClearValue", SimTypeRef("D3D12_CLEAR_VALUE", SimStruct)),)), name="D3D12_RENDER_PASS_BEGINNING_ACCESS_CLEAR_PARAMETERS", pack=False, align=None),
    "D3D12_RENDER_PASS_BEGINNING_ACCESS_PRESERVE_LOCAL_PARAMETERS": SimStruct(OrderedDict((("AdditionalWidth", SimTypeInt(signed=False, label="UInt32")), ("AdditionalHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RENDER_PASS_BEGINNING_ACCESS_PRESERVE_LOCAL_PARAMETERS", pack=False, align=None),
    "D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE": SimTypeInt(signed=False, label="D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE"),
    "D3D12_RENDER_PASS_DEPTH_STENCIL_DESC": SimStruct(OrderedDict((("cpuDescriptor", SimTypeRef("D3D12_CPU_DESCRIPTOR_HANDLE", SimStruct)), ("DepthBeginningAccess", SimTypeRef("D3D12_RENDER_PASS_BEGINNING_ACCESS", SimStruct)), ("StencilBeginningAccess", SimTypeRef("D3D12_RENDER_PASS_BEGINNING_ACCESS", SimStruct)), ("DepthEndingAccess", SimTypeRef("D3D12_RENDER_PASS_ENDING_ACCESS", SimStruct)), ("StencilEndingAccess", SimTypeRef("D3D12_RENDER_PASS_ENDING_ACCESS", SimStruct)),)), name="D3D12_RENDER_PASS_DEPTH_STENCIL_DESC", pack=False, align=None),
    "D3D12_RENDER_PASS_ENDING_ACCESS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_RENDER_PASS_ENDING_ACCESS_TYPE")), ("Anonymous", SimUnion({"Resolve": SimTypeRef("D3D12_RENDER_PASS_ENDING_ACCESS_RESOLVE_PARAMETERS", SimStruct), "PreserveLocal": SimTypeRef("D3D12_RENDER_PASS_ENDING_ACCESS_PRESERVE_LOCAL_PARAMETERS", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_RENDER_PASS_ENDING_ACCESS", pack=False, align=None),
    "D3D12_RENDER_PASS_ENDING_ACCESS_PRESERVE_LOCAL_PARAMETERS": SimStruct(OrderedDict((("AdditionalWidth", SimTypeInt(signed=False, label="UInt32")), ("AdditionalHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RENDER_PASS_ENDING_ACCESS_PRESERVE_LOCAL_PARAMETERS", pack=False, align=None),
    "D3D12_RENDER_PASS_ENDING_ACCESS_RESOLVE_PARAMETERS": SimStruct(OrderedDict((("pSrcResource", SimTypeBottom(label="ID3D12Resource")), ("pDstResource", SimTypeBottom(label="ID3D12Resource")), ("SubresourceCount", SimTypeInt(signed=False, label="UInt32")), ("pSubresourceParameters", SimTypePointer(SimTypeRef("D3D12_RENDER_PASS_ENDING_ACCESS_RESOLVE_SUBRESOURCE_PARAMETERS", SimStruct), label="LPArray", offset=0)), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ResolveMode", SimTypeInt(signed=False, label="D3D12_RESOLVE_MODE")), ("PreserveResolveSource", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_RENDER_PASS_ENDING_ACCESS_RESOLVE_PARAMETERS", pack=False, align=None),
    "D3D12_RENDER_PASS_ENDING_ACCESS_RESOLVE_SUBRESOURCE_PARAMETERS": SimStruct(OrderedDict((("SrcSubresource", SimTypeInt(signed=False, label="UInt32")), ("DstSubresource", SimTypeInt(signed=False, label="UInt32")), ("DstX", SimTypeInt(signed=False, label="UInt32")), ("DstY", SimTypeInt(signed=False, label="UInt32")), ("SrcRect", SimTypeRef("RECT", SimStruct)),)), name="D3D12_RENDER_PASS_ENDING_ACCESS_RESOLVE_SUBRESOURCE_PARAMETERS", pack=False, align=None),
    "D3D12_RENDER_PASS_ENDING_ACCESS_TYPE": SimTypeInt(signed=False, label="D3D12_RENDER_PASS_ENDING_ACCESS_TYPE"),
    "D3D12_RENDER_PASS_FLAGS": SimTypeInt(signed=False, label="D3D12_RENDER_PASS_FLAGS"),
    "D3D12_RENDER_PASS_RENDER_TARGET_DESC": SimStruct(OrderedDict((("cpuDescriptor", SimTypeRef("D3D12_CPU_DESCRIPTOR_HANDLE", SimStruct)), ("BeginningAccess", SimTypeRef("D3D12_RENDER_PASS_BEGINNING_ACCESS", SimStruct)), ("EndingAccess", SimTypeRef("D3D12_RENDER_PASS_ENDING_ACCESS", SimStruct)),)), name="D3D12_RENDER_PASS_RENDER_TARGET_DESC", pack=False, align=None),
    "D3D12_RENDER_PASS_TIER": SimTypeInt(signed=False, label="D3D12_RENDER_PASS_TIER"),
    "D3D12_RENDER_TARGET_BLEND_DESC": SimStruct(OrderedDict((("BlendEnable", SimTypeInt(signed=True, label="Int32")), ("LogicOpEnable", SimTypeInt(signed=True, label="Int32")), ("SrcBlend", SimTypeInt(signed=False, label="D3D12_BLEND")), ("DestBlend", SimTypeInt(signed=False, label="D3D12_BLEND")), ("BlendOp", SimTypeInt(signed=False, label="D3D12_BLEND_OP")), ("SrcBlendAlpha", SimTypeInt(signed=False, label="D3D12_BLEND")), ("DestBlendAlpha", SimTypeInt(signed=False, label="D3D12_BLEND")), ("BlendOpAlpha", SimTypeInt(signed=False, label="D3D12_BLEND_OP")), ("LogicOp", SimTypeInt(signed=False, label="D3D12_LOGIC_OP")), ("RenderTargetWriteMask", SimTypeChar(label="Byte")),)), name="D3D12_RENDER_TARGET_BLEND_DESC", pack=False, align=None),
    "D3D12_RENDER_TARGET_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D12_RTV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D12_BUFFER_RTV", SimStruct), "Texture1D": SimTypeRef("D3D12_TEX1D_RTV", SimStruct), "Texture1DArray": SimTypeRef("D3D12_TEX1D_ARRAY_RTV", SimStruct), "Texture2D": SimTypeRef("D3D12_TEX2D_RTV", SimStruct), "Texture2DArray": SimTypeRef("D3D12_TEX2D_ARRAY_RTV", SimStruct), "Texture2DMS": SimTypeRef("D3D12_TEX2DMS_RTV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D12_TEX2DMS_ARRAY_RTV", SimStruct), "Texture3D": SimTypeRef("D3D12_TEX3D_RTV", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_RENDER_TARGET_VIEW_DESC", pack=False, align=None),
    "D3D12_RESIDENCY_FLAGS": SimTypeInt(signed=False, label="D3D12_RESIDENCY_FLAGS"),
    "D3D12_RESIDENCY_PRIORITY": SimTypeInt(signed=False, label="D3D12_RESIDENCY_PRIORITY"),
    "D3D12_RESOLVE_MODE": SimTypeInt(signed=False, label="D3D12_RESOLVE_MODE"),
    "D3D12_RESOLVE_VIDEO_MOTION_VECTOR_HEAP_INPUT": SimStruct(OrderedDict((("pMotionVectorHeap", SimTypeBottom(label="ID3D12VideoMotionVectorHeap")), ("PixelWidth", SimTypeInt(signed=False, label="UInt32")), ("PixelHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RESOLVE_VIDEO_MOTION_VECTOR_HEAP_INPUT", pack=False, align=None),
    "D3D12_RESOLVE_VIDEO_MOTION_VECTOR_HEAP_OUTPUT": SimStruct(OrderedDict((("pMotionVectorTexture2D", SimTypeBottom(label="ID3D12Resource")), ("MotionVectorCoordinate", SimTypeRef("D3D12_RESOURCE_COORDINATE", SimStruct)),)), name="D3D12_RESOLVE_VIDEO_MOTION_VECTOR_HEAP_OUTPUT", pack=False, align=None),
    "D3D12_RESOURCE_ALIASING_BARRIER": SimStruct(OrderedDict((("pResourceBefore", SimTypeBottom(label="ID3D12Resource")), ("pResourceAfter", SimTypeBottom(label="ID3D12Resource")),)), name="D3D12_RESOURCE_ALIASING_BARRIER", pack=False, align=None),
    "D3D12_RESOURCE_ALLOCATION_INFO": SimStruct(OrderedDict((("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RESOURCE_ALLOCATION_INFO", pack=False, align=None),
    "D3D12_RESOURCE_ALLOCATION_INFO1": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_RESOURCE_ALLOCATION_INFO1", pack=False, align=None),
    "D3D12_RESOURCE_BARRIER": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_RESOURCE_BARRIER_TYPE")), ("Flags", SimTypeInt(signed=False, label="D3D12_RESOURCE_BARRIER_FLAGS")), ("Anonymous", SimUnion({"Transition": SimTypeRef("D3D12_RESOURCE_TRANSITION_BARRIER", SimStruct), "Aliasing": SimTypeRef("D3D12_RESOURCE_ALIASING_BARRIER", SimStruct), "UAV": SimTypeRef("D3D12_RESOURCE_UAV_BARRIER", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_RESOURCE_BARRIER", pack=False, align=None),
    "D3D12_RESOURCE_BARRIER_FLAGS": SimTypeInt(signed=False, label="D3D12_RESOURCE_BARRIER_FLAGS"),
    "D3D12_RESOURCE_BARRIER_TYPE": SimTypeInt(signed=False, label="D3D12_RESOURCE_BARRIER_TYPE"),
    "D3D12_RESOURCE_BINDING_TIER": SimTypeInt(signed=False, label="D3D12_RESOURCE_BINDING_TIER"),
    "D3D12_RESOURCE_COORDINATE": SimStruct(OrderedDict((("X", SimTypeLongLong(signed=False, label="UInt64")), ("Y", SimTypeInt(signed=False, label="UInt32")), ("Z", SimTypeInt(signed=False, label="UInt32")), ("SubresourceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RESOURCE_COORDINATE", pack=False, align=None),
    "D3D12_RESOURCE_DESC": SimStruct(OrderedDict((("Dimension", SimTypeInt(signed=False, label="D3D12_RESOURCE_DIMENSION")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("Width", SimTypeLongLong(signed=False, label="UInt64")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("DepthOrArraySize", SimTypeShort(signed=False, label="UInt16")), ("MipLevels", SimTypeShort(signed=False, label="UInt16")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("Layout", SimTypeInt(signed=False, label="D3D12_TEXTURE_LAYOUT")), ("Flags", SimTypeInt(signed=False, label="D3D12_RESOURCE_FLAGS")),)), name="D3D12_RESOURCE_DESC", pack=False, align=None),
    "D3D12_RESOURCE_DESC1": SimStruct(OrderedDict((("Dimension", SimTypeInt(signed=False, label="D3D12_RESOURCE_DIMENSION")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("Width", SimTypeLongLong(signed=False, label="UInt64")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("DepthOrArraySize", SimTypeShort(signed=False, label="UInt16")), ("MipLevels", SimTypeShort(signed=False, label="UInt16")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("Layout", SimTypeInt(signed=False, label="D3D12_TEXTURE_LAYOUT")), ("Flags", SimTypeInt(signed=False, label="D3D12_RESOURCE_FLAGS")), ("SamplerFeedbackMipRegion", SimTypeRef("D3D12_MIP_REGION", SimStruct)),)), name="D3D12_RESOURCE_DESC1", pack=False, align=None),
    "D3D12_RESOURCE_DIMENSION": SimTypeInt(signed=False, label="D3D12_RESOURCE_DIMENSION"),
    "D3D12_RESOURCE_FLAGS": SimTypeInt(signed=False, label="D3D12_RESOURCE_FLAGS"),
    "D3D12_RESOURCE_HEAP_TIER": SimTypeInt(signed=False, label="D3D12_RESOURCE_HEAP_TIER"),
    "D3D12_RESOURCE_STATES": SimTypeInt(signed=False, label="D3D12_RESOURCE_STATES"),
    "D3D12_RESOURCE_TRANSITION_BARRIER": SimStruct(OrderedDict((("pResource", SimTypeBottom(label="ID3D12Resource")), ("Subresource", SimTypeInt(signed=False, label="UInt32")), ("StateBefore", SimTypeInt(signed=False, label="D3D12_RESOURCE_STATES")), ("StateAfter", SimTypeInt(signed=False, label="D3D12_RESOURCE_STATES")),)), name="D3D12_RESOURCE_TRANSITION_BARRIER", pack=False, align=None),
    "D3D12_RESOURCE_UAV_BARRIER": SimStruct(OrderedDict((("pResource", SimTypeBottom(label="ID3D12Resource")),)), name="D3D12_RESOURCE_UAV_BARRIER", pack=False, align=None),
    "D3D12_RLDO_FLAGS": SimTypeInt(signed=False, label="D3D12_RLDO_FLAGS"),
    "D3D12_ROOT_CONSTANTS": SimStruct(OrderedDict((("ShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")), ("Num32BitValues", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_ROOT_CONSTANTS", pack=False, align=None),
    "D3D12_ROOT_DESCRIPTOR": SimStruct(OrderedDict((("ShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_ROOT_DESCRIPTOR", pack=False, align=None),
    "D3D12_ROOT_DESCRIPTOR1": SimStruct(OrderedDict((("ShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_ROOT_DESCRIPTOR_FLAGS")),)), name="D3D12_ROOT_DESCRIPTOR1", pack=False, align=None),
    "D3D12_ROOT_DESCRIPTOR_FLAGS": SimTypeInt(signed=False, label="D3D12_ROOT_DESCRIPTOR_FLAGS"),
    "D3D12_ROOT_DESCRIPTOR_TABLE": SimStruct(OrderedDict((("NumDescriptorRanges", SimTypeInt(signed=False, label="UInt32")), ("pDescriptorRanges", SimTypePointer(SimTypeRef("D3D12_DESCRIPTOR_RANGE", SimStruct), label="LPArray", offset=0)),)), name="D3D12_ROOT_DESCRIPTOR_TABLE", pack=False, align=None),
    "D3D12_ROOT_DESCRIPTOR_TABLE1": SimStruct(OrderedDict((("NumDescriptorRanges", SimTypeInt(signed=False, label="UInt32")), ("pDescriptorRanges", SimTypePointer(SimTypeRef("D3D12_DESCRIPTOR_RANGE1", SimStruct), label="LPArray", offset=0)),)), name="D3D12_ROOT_DESCRIPTOR_TABLE1", pack=False, align=None),
    "D3D12_ROOT_PARAMETER": SimStruct(OrderedDict((("ParameterType", SimTypeInt(signed=False, label="D3D12_ROOT_PARAMETER_TYPE")), ("Anonymous", SimUnion({"DescriptorTable": SimTypeRef("D3D12_ROOT_DESCRIPTOR_TABLE", SimStruct), "Constants": SimTypeRef("D3D12_ROOT_CONSTANTS", SimStruct), "Descriptor": SimTypeRef("D3D12_ROOT_DESCRIPTOR", SimStruct)}, name="<anon>", label="None")), ("ShaderVisibility", SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")),)), name="D3D12_ROOT_PARAMETER", pack=False, align=None),
    "D3D12_ROOT_PARAMETER1": SimStruct(OrderedDict((("ParameterType", SimTypeInt(signed=False, label="D3D12_ROOT_PARAMETER_TYPE")), ("Anonymous", SimUnion({"DescriptorTable": SimTypeRef("D3D12_ROOT_DESCRIPTOR_TABLE1", SimStruct), "Constants": SimTypeRef("D3D12_ROOT_CONSTANTS", SimStruct), "Descriptor": SimTypeRef("D3D12_ROOT_DESCRIPTOR1", SimStruct)}, name="<anon>", label="None")), ("ShaderVisibility", SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")),)), name="D3D12_ROOT_PARAMETER1", pack=False, align=None),
    "D3D12_ROOT_PARAMETER_TYPE": SimTypeInt(signed=False, label="D3D12_ROOT_PARAMETER_TYPE"),
    "D3D12_ROOT_SIGNATURE_DESC": SimStruct(OrderedDict((("NumParameters", SimTypeInt(signed=False, label="UInt32")), ("pParameters", SimTypePointer(SimTypeRef("D3D12_ROOT_PARAMETER", SimStruct), label="LPArray", offset=0)), ("NumStaticSamplers", SimTypeInt(signed=False, label="UInt32")), ("pStaticSamplers", SimTypePointer(SimTypeRef("D3D12_STATIC_SAMPLER_DESC", SimStruct), label="LPArray", offset=0)), ("Flags", SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS")),)), name="D3D12_ROOT_SIGNATURE_DESC", pack=False, align=None),
    "D3D12_ROOT_SIGNATURE_DESC1": SimStruct(OrderedDict((("NumParameters", SimTypeInt(signed=False, label="UInt32")), ("pParameters", SimTypePointer(SimTypeRef("D3D12_ROOT_PARAMETER1", SimStruct), label="LPArray", offset=0)), ("NumStaticSamplers", SimTypeInt(signed=False, label="UInt32")), ("pStaticSamplers", SimTypePointer(SimTypeRef("D3D12_STATIC_SAMPLER_DESC", SimStruct), label="LPArray", offset=0)), ("Flags", SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS")),)), name="D3D12_ROOT_SIGNATURE_DESC1", pack=False, align=None),
    "D3D12_ROOT_SIGNATURE_DESC2": SimStruct(OrderedDict((("NumParameters", SimTypeInt(signed=False, label="UInt32")), ("pParameters", SimTypePointer(SimTypeRef("D3D12_ROOT_PARAMETER1", SimStruct), offset=0)), ("NumStaticSamplers", SimTypeInt(signed=False, label="UInt32")), ("pStaticSamplers", SimTypePointer(SimTypeRef("D3D12_STATIC_SAMPLER_DESC1", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS")),)), name="D3D12_ROOT_SIGNATURE_DESC2", pack=False, align=None),
    "D3D12_ROOT_SIGNATURE_FLAGS": SimTypeInt(signed=False, label="D3D12_ROOT_SIGNATURE_FLAGS"),
    "D3D12_RTV_DIMENSION": SimTypeInt(signed=False, label="D3D12_RTV_DIMENSION"),
    "D3D12_RT_FORMAT_ARRAY": SimStruct(OrderedDict((("RTFormats", SimTypeArray(SimTypeInt(signed=False, label="DXGI_FORMAT"), 8)), ("NumRenderTargets", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_RT_FORMAT_ARRAY", pack=False, align=None),
    "D3D12_SAMPLER_DESC": SimStruct(OrderedDict((("Filter", SimTypeInt(signed=False, label="D3D12_FILTER")), ("AddressU", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressV", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressW", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("MipLODBias", SimTypeFloat(size=32)), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("ComparisonFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("BorderColor", SimTypeArray(SimTypeFloat(size=32), 4)), ("MinLOD", SimTypeFloat(size=32)), ("MaxLOD", SimTypeFloat(size=32)),)), name="D3D12_SAMPLER_DESC", pack=False, align=None),
    "D3D12_SAMPLER_DESC2": SimStruct(OrderedDict((("Filter", SimTypeInt(signed=False, label="D3D12_FILTER")), ("AddressU", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressV", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressW", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("MipLODBias", SimTypeFloat(size=32)), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("ComparisonFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("Anonymous", SimUnion({"FloatBorderColor": SimTypeArray(SimTypeFloat(size=32), 4), "UintBorderColor": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")), ("MinLOD", SimTypeFloat(size=32)), ("MaxLOD", SimTypeFloat(size=32)), ("Flags", SimTypeInt(signed=False, label="D3D12_SAMPLER_FLAGS")),)), name="D3D12_SAMPLER_DESC2", pack=False, align=None),
    "D3D12_SAMPLER_FEEDBACK_TIER": SimTypeInt(signed=False, label="D3D12_SAMPLER_FEEDBACK_TIER"),
    "D3D12_SAMPLER_FLAGS": SimTypeInt(signed=False, label="D3D12_SAMPLER_FLAGS"),
    "D3D12_SAMPLE_POSITION": SimStruct(OrderedDict((("X", SimTypeChar(label="SByte")), ("Y", SimTypeChar(label="SByte")),)), name="D3D12_SAMPLE_POSITION", pack=False, align=None),
    "D3D12_SERIALIZED_DATA_DRIVER_MATCHING_IDENTIFIER": SimStruct(OrderedDict((("DriverOpaqueGUID", SimTypeBottom(label="Guid")), ("DriverOpaqueVersioningData", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="D3D12_SERIALIZED_DATA_DRIVER_MATCHING_IDENTIFIER", pack=False, align=None),
    "D3D12_SERIALIZED_DATA_TYPE": SimTypeInt(signed=False, label="D3D12_SERIALIZED_DATA_TYPE"),
    "D3D12_SERIALIZED_RAYTRACING_ACCELERATION_STRUCTURE_HEADER": SimStruct(OrderedDict((("DriverMatchingIdentifier", SimTypeRef("D3D12_SERIALIZED_DATA_DRIVER_MATCHING_IDENTIFIER", SimStruct)), ("SerializedSizeInBytesIncludingHeader", SimTypeLongLong(signed=False, label="UInt64")), ("DeserializedSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("NumBottomLevelAccelerationStructurePointersAfterHeader", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_SERIALIZED_RAYTRACING_ACCELERATION_STRUCTURE_HEADER", pack=False, align=None),
    "D3D12_SHADER_BUFFER_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_CBUFFER_TYPE")), ("Variables", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SHADER_BUFFER_DESC", pack=False, align=None),
    "D3D12_SHADER_BYTECODE": SimStruct(OrderedDict((("pShaderBytecode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BytecodeLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D12_SHADER_BYTECODE", pack=False, align=None),
    "D3D12_SHADER_CACHE_CONTROL_FLAGS": SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_CONTROL_FLAGS"),
    "D3D12_SHADER_CACHE_FLAGS": SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_FLAGS"),
    "D3D12_SHADER_CACHE_KIND_FLAGS": SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_KIND_FLAGS"),
    "D3D12_SHADER_CACHE_MODE": SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_MODE"),
    "D3D12_SHADER_CACHE_SESSION_DESC": SimStruct(OrderedDict((("Identifier", SimTypeBottom(label="Guid")), ("Mode", SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_MODE")), ("Flags", SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_FLAGS")), ("MaximumInMemoryCacheSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("MaximumInMemoryCacheEntries", SimTypeInt(signed=False, label="UInt32")), ("MaximumValueFileSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_SHADER_CACHE_SESSION_DESC", pack=False, align=None),
    "D3D12_SHADER_CACHE_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_SHADER_CACHE_SUPPORT_FLAGS"),
    "D3D12_SHADER_COMPONENT_MAPPING": SimTypeInt(signed=False, label="D3D12_SHADER_COMPONENT_MAPPING"),
    "D3D12_SHADER_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Creator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ConstantBuffers", SimTypeInt(signed=False, label="UInt32")), ("BoundResources", SimTypeInt(signed=False, label="UInt32")), ("InputParameters", SimTypeInt(signed=False, label="UInt32")), ("OutputParameters", SimTypeInt(signed=False, label="UInt32")), ("InstructionCount", SimTypeInt(signed=False, label="UInt32")), ("TempRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("TempArrayCount", SimTypeInt(signed=False, label="UInt32")), ("DefCount", SimTypeInt(signed=False, label="UInt32")), ("DclCount", SimTypeInt(signed=False, label="UInt32")), ("TextureNormalInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureLoadInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureCompInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureBiasInstructions", SimTypeInt(signed=False, label="UInt32")), ("TextureGradientInstructions", SimTypeInt(signed=False, label="UInt32")), ("FloatInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("IntInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("UintInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("StaticFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlCount", SimTypeInt(signed=False, label="UInt32")), ("MacroInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ArrayInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("CutInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("EmitInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GSOutputTopology", SimTypeInt(signed=False, label="D3D_PRIMITIVE_TOPOLOGY")), ("GSMaxOutputVertexCount", SimTypeInt(signed=False, label="UInt32")), ("InputPrimitive", SimTypeInt(signed=False, label="D3D_PRIMITIVE")), ("PatchConstantParameters", SimTypeInt(signed=False, label="UInt32")), ("cGSInstanceCount", SimTypeInt(signed=False, label="UInt32")), ("cControlPoints", SimTypeInt(signed=False, label="UInt32")), ("HSOutputPrimitive", SimTypeInt(signed=False, label="D3D_TESSELLATOR_OUTPUT_PRIMITIVE")), ("HSPartitioning", SimTypeInt(signed=False, label="D3D_TESSELLATOR_PARTITIONING")), ("TessellatorDomain", SimTypeInt(signed=False, label="D3D_TESSELLATOR_DOMAIN")), ("cBarrierInstructions", SimTypeInt(signed=False, label="UInt32")), ("cInterlockedInstructions", SimTypeInt(signed=False, label="UInt32")), ("cTextureStoreInstructions", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SHADER_DESC", pack=False, align=None),
    "D3D12_SHADER_INPUT_BIND_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_INPUT_TYPE")), ("BindPoint", SimTypeInt(signed=False, label="UInt32")), ("BindCount", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("ReturnType", SimTypeInt(signed=False, label="D3D_RESOURCE_RETURN_TYPE")), ("Dimension", SimTypeInt(signed=False, label="D3D_SRV_DIMENSION")), ("NumSamples", SimTypeInt(signed=False, label="UInt32")), ("Space", SimTypeInt(signed=False, label="UInt32")), ("uID", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SHADER_INPUT_BIND_DESC", pack=False, align=None),
    "D3D12_SHADER_MIN_PRECISION_SUPPORT": SimTypeInt(signed=False, label="D3D12_SHADER_MIN_PRECISION_SUPPORT"),
    "D3D12_SHADER_RESOURCE_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D12_SRV_DIMENSION")), ("Shader4ComponentMapping", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D12_BUFFER_SRV", SimStruct), "Texture1D": SimTypeRef("D3D12_TEX1D_SRV", SimStruct), "Texture1DArray": SimTypeRef("D3D12_TEX1D_ARRAY_SRV", SimStruct), "Texture2D": SimTypeRef("D3D12_TEX2D_SRV", SimStruct), "Texture2DArray": SimTypeRef("D3D12_TEX2D_ARRAY_SRV", SimStruct), "Texture2DMS": SimTypeRef("D3D12_TEX2DMS_SRV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D12_TEX2DMS_ARRAY_SRV", SimStruct), "Texture3D": SimTypeRef("D3D12_TEX3D_SRV", SimStruct), "TextureCube": SimTypeRef("D3D12_TEXCUBE_SRV", SimStruct), "TextureCubeArray": SimTypeRef("D3D12_TEXCUBE_ARRAY_SRV", SimStruct), "RaytracingAccelerationStructure": SimTypeRef("D3D12_RAYTRACING_ACCELERATION_STRUCTURE_SRV", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_SHADER_RESOURCE_VIEW_DESC", pack=False, align=None),
    "D3D12_SHADER_TYPE_DESC": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS")), ("Type", SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE")), ("Rows", SimTypeInt(signed=False, label="UInt32")), ("Columns", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypeInt(signed=False, label="UInt32")), ("Members", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3D12_SHADER_TYPE_DESC", pack=False, align=None),
    "D3D12_SHADER_VARIABLE_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StartOffset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("DefaultValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StartTexture", SimTypeInt(signed=False, label="UInt32")), ("TextureSize", SimTypeInt(signed=False, label="UInt32")), ("StartSampler", SimTypeInt(signed=False, label="UInt32")), ("SamplerSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SHADER_VARIABLE_DESC", pack=False, align=None),
    "D3D12_SHADER_VERSION_TYPE": SimTypeInt(signed=False, label="D3D12_SHADER_VERSION_TYPE"),
    "D3D12_SHADER_VISIBILITY": SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY"),
    "D3D12_SHADING_RATE": SimTypeInt(signed=False, label="D3D12_SHADING_RATE"),
    "D3D12_SHADING_RATE_COMBINER": SimTypeInt(signed=False, label="D3D12_SHADING_RATE_COMBINER"),
    "D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER": SimTypeInt(signed=False, label="D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER"),
    "D3D12_SIGNATURE_PARAMETER_DESC": SimStruct(OrderedDict((("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("SystemValueType", SimTypeInt(signed=False, label="D3D_NAME")), ("ComponentType", SimTypeInt(signed=False, label="D3D_REGISTER_COMPONENT_TYPE")), ("Mask", SimTypeChar(label="Byte")), ("ReadWriteMask", SimTypeChar(label="Byte")), ("Stream", SimTypeInt(signed=False, label="UInt32")), ("MinPrecision", SimTypeInt(signed=False, label="D3D_MIN_PRECISION")),)), name="D3D12_SIGNATURE_PARAMETER_DESC", pack=False, align=None),
    "D3D12_SO_DECLARATION_ENTRY": SimStruct(OrderedDict((("Stream", SimTypeInt(signed=False, label="UInt32")), ("SemanticName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SemanticIndex", SimTypeInt(signed=False, label="UInt32")), ("StartComponent", SimTypeChar(label="Byte")), ("ComponentCount", SimTypeChar(label="Byte")), ("OutputSlot", SimTypeChar(label="Byte")),)), name="D3D12_SO_DECLARATION_ENTRY", pack=False, align=None),
    "D3D12_SRV_DIMENSION": SimTypeInt(signed=False, label="D3D12_SRV_DIMENSION"),
    "D3D12_STATE_OBJECT_CONFIG": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_STATE_OBJECT_FLAGS")),)), name="D3D12_STATE_OBJECT_CONFIG", pack=False, align=None),
    "D3D12_STATE_OBJECT_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_STATE_OBJECT_TYPE")), ("NumSubobjects", SimTypeInt(signed=False, label="UInt32")), ("pSubobjects", SimTypePointer(SimTypeRef("D3D12_STATE_SUBOBJECT", SimStruct), offset=0)),)), name="D3D12_STATE_OBJECT_DESC", pack=False, align=None),
    "D3D12_STATE_OBJECT_FLAGS": SimTypeInt(signed=False, label="D3D12_STATE_OBJECT_FLAGS"),
    "D3D12_STATE_OBJECT_TYPE": SimTypeInt(signed=False, label="D3D12_STATE_OBJECT_TYPE"),
    "D3D12_STATE_SUBOBJECT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_STATE_SUBOBJECT_TYPE")), ("pDesc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3D12_STATE_SUBOBJECT", pack=False, align=None),
    "D3D12_STATE_SUBOBJECT_TYPE": SimTypeInt(signed=False, label="D3D12_STATE_SUBOBJECT_TYPE"),
    "D3D12_STATIC_BORDER_COLOR": SimTypeInt(signed=False, label="D3D12_STATIC_BORDER_COLOR"),
    "D3D12_STATIC_SAMPLER_DESC": SimStruct(OrderedDict((("Filter", SimTypeInt(signed=False, label="D3D12_FILTER")), ("AddressU", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressV", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressW", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("MipLODBias", SimTypeFloat(size=32)), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("ComparisonFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("BorderColor", SimTypeInt(signed=False, label="D3D12_STATIC_BORDER_COLOR")), ("MinLOD", SimTypeFloat(size=32)), ("MaxLOD", SimTypeFloat(size=32)), ("ShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")), ("ShaderVisibility", SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")),)), name="D3D12_STATIC_SAMPLER_DESC", pack=False, align=None),
    "D3D12_STATIC_SAMPLER_DESC1": SimStruct(OrderedDict((("Filter", SimTypeInt(signed=False, label="D3D12_FILTER")), ("AddressU", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressV", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("AddressW", SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE")), ("MipLODBias", SimTypeFloat(size=32)), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("ComparisonFunc", SimTypeInt(signed=False, label="D3D12_COMPARISON_FUNC")), ("BorderColor", SimTypeInt(signed=False, label="D3D12_STATIC_BORDER_COLOR")), ("MinLOD", SimTypeFloat(size=32)), ("MaxLOD", SimTypeFloat(size=32)), ("ShaderRegister", SimTypeInt(signed=False, label="UInt32")), ("RegisterSpace", SimTypeInt(signed=False, label="UInt32")), ("ShaderVisibility", SimTypeInt(signed=False, label="D3D12_SHADER_VISIBILITY")), ("Flags", SimTypeInt(signed=False, label="D3D12_SAMPLER_FLAGS")),)), name="D3D12_STATIC_SAMPLER_DESC1", pack=False, align=None),
    "D3D12_STENCIL_OP": SimTypeInt(signed=False, label="D3D12_STENCIL_OP"),
    "D3D12_STREAM_OUTPUT_BUFFER_VIEW": SimStruct(OrderedDict((("BufferLocation", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("BufferFilledSizeLocation", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_STREAM_OUTPUT_BUFFER_VIEW", pack=False, align=None),
    "D3D12_STREAM_OUTPUT_DESC": SimStruct(OrderedDict((("pSODeclaration", SimTypePointer(SimTypeRef("D3D12_SO_DECLARATION_ENTRY", SimStruct), label="LPArray", offset=0)), ("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("pBufferStrides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)), ("NumStrides", SimTypeInt(signed=False, label="UInt32")), ("RasterizedStream", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_STREAM_OUTPUT_DESC", pack=False, align=None),
    "D3D12_SUBOBJECT_TO_EXPORTS_ASSOCIATION": SimStruct(OrderedDict((("pSubobjectToAssociate", SimTypePointer(SimTypeRef("D3D12_STATE_SUBOBJECT", SimStruct), offset=0)), ("NumExports", SimTypeInt(signed=False, label="UInt32")), ("pExports", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="D3D12_SUBOBJECT_TO_EXPORTS_ASSOCIATION", pack=False, align=None),
    "D3D12_SUBRESOURCE_DATA": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RowPitch", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SlicePitch", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3D12_SUBRESOURCE_DATA", pack=False, align=None),
    "D3D12_SUBRESOURCE_FOOTPRINT": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")), ("RowPitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SUBRESOURCE_FOOTPRINT", pack=False, align=None),
    "D3D12_SUBRESOURCE_INFO": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("RowPitch", SimTypeInt(signed=False, label="UInt32")), ("DepthPitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SUBRESOURCE_INFO", pack=False, align=None),
    "D3D12_SUBRESOURCE_RANGE_UINT64": SimStruct(OrderedDict((("Subresource", SimTypeInt(signed=False, label="UInt32")), ("Range", SimTypeRef("D3D12_RANGE_UINT64", SimStruct)),)), name="D3D12_SUBRESOURCE_RANGE_UINT64", pack=False, align=None),
    "D3D12_SUBRESOURCE_TILING": SimStruct(OrderedDict((("WidthInTiles", SimTypeInt(signed=False, label="UInt32")), ("HeightInTiles", SimTypeShort(signed=False, label="UInt16")), ("DepthInTiles", SimTypeShort(signed=False, label="UInt16")), ("StartTileIndexInOverallResource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_SUBRESOURCE_TILING", pack=False, align=None),
    "D3D12_TEX1D_ARRAY_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX1D_ARRAY_DSV", pack=False, align=None),
    "D3D12_TEX1D_ARRAY_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX1D_ARRAY_RTV", pack=False, align=None),
    "D3D12_TEX1D_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEX1D_ARRAY_SRV", pack=False, align=None),
    "D3D12_TEX1D_ARRAY_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX1D_ARRAY_UAV", pack=False, align=None),
    "D3D12_TEX1D_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX1D_DSV", pack=False, align=None),
    "D3D12_TEX1D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX1D_RTV", pack=False, align=None),
    "D3D12_TEX1D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEX1D_SRV", pack=False, align=None),
    "D3D12_TEX1D_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX1D_UAV", pack=False, align=None),
    "D3D12_TEX2DMS_ARRAY_DSV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_ARRAY_DSV", pack=False, align=None),
    "D3D12_TEX2DMS_ARRAY_RTV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_ARRAY_RTV", pack=False, align=None),
    "D3D12_TEX2DMS_ARRAY_SRV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_ARRAY_SRV", pack=False, align=None),
    "D3D12_TEX2DMS_ARRAY_UAV": SimStruct(OrderedDict((("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_ARRAY_UAV", pack=False, align=None),
    "D3D12_TEX2DMS_DSV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_DSV", pack=False, align=None),
    "D3D12_TEX2DMS_RTV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_RTV", pack=False, align=None),
    "D3D12_TEX2DMS_SRV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_SRV", pack=False, align=None),
    "D3D12_TEX2DMS_UAV": SimStruct(OrderedDict((("UnusedField_NothingToDefine", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2DMS_UAV", pack=False, align=None),
    "D3D12_TEX2D_ARRAY_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2D_ARRAY_DSV", pack=False, align=None),
    "D3D12_TEX2D_ARRAY_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2D_ARRAY_RTV", pack=False, align=None),
    "D3D12_TEX2D_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEX2D_ARRAY_SRV", pack=False, align=None),
    "D3D12_TEX2D_ARRAY_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstArraySlice", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2D_ARRAY_UAV", pack=False, align=None),
    "D3D12_TEX2D_DSV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2D_DSV", pack=False, align=None),
    "D3D12_TEX2D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2D_RTV", pack=False, align=None),
    "D3D12_TEX2D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEX2D_SRV", pack=False, align=None),
    "D3D12_TEX2D_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("PlaneSlice", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX2D_UAV", pack=False, align=None),
    "D3D12_TEX3D_RTV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstWSlice", SimTypeInt(signed=False, label="UInt32")), ("WSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX3D_RTV", pack=False, align=None),
    "D3D12_TEX3D_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEX3D_SRV", pack=False, align=None),
    "D3D12_TEX3D_UAV": SimStruct(OrderedDict((("MipSlice", SimTypeInt(signed=False, label="UInt32")), ("FirstWSlice", SimTypeInt(signed=False, label="UInt32")), ("WSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TEX3D_UAV", pack=False, align=None),
    "D3D12_TEXCUBE_ARRAY_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("First2DArrayFace", SimTypeInt(signed=False, label="UInt32")), ("NumCubes", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEXCUBE_ARRAY_SRV", pack=False, align=None),
    "D3D12_TEXCUBE_SRV": SimStruct(OrderedDict((("MostDetailedMip", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ResourceMinLODClamp", SimTypeFloat(size=32)),)), name="D3D12_TEXCUBE_SRV", pack=False, align=None),
    "D3D12_TEXTURE_ADDRESS_MODE": SimTypeInt(signed=False, label="D3D12_TEXTURE_ADDRESS_MODE"),
    "D3D12_TEXTURE_BARRIER": SimStruct(OrderedDict((("SyncBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC")), ("SyncAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_SYNC")), ("AccessBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS")), ("AccessAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_ACCESS")), ("LayoutBefore", SimTypeInt(signed=False, label="D3D12_BARRIER_LAYOUT")), ("LayoutAfter", SimTypeInt(signed=False, label="D3D12_BARRIER_LAYOUT")), ("pResource", SimTypeBottom(label="ID3D12Resource")), ("Subresources", SimTypeRef("D3D12_BARRIER_SUBRESOURCE_RANGE", SimStruct)), ("Flags", SimTypeInt(signed=False, label="D3D12_TEXTURE_BARRIER_FLAGS")),)), name="D3D12_TEXTURE_BARRIER", pack=False, align=None),
    "D3D12_TEXTURE_BARRIER_FLAGS": SimTypeInt(signed=False, label="D3D12_TEXTURE_BARRIER_FLAGS"),
    "D3D12_TEXTURE_COPY_LOCATION": SimStruct(OrderedDict((("pResource", SimTypeBottom(label="ID3D12Resource")), ("Type", SimTypeInt(signed=False, label="D3D12_TEXTURE_COPY_TYPE")), ("Anonymous", SimUnion({"PlacedFootprint": SimTypeRef("D3D12_PLACED_SUBRESOURCE_FOOTPRINT", SimStruct), "SubresourceIndex": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D12_TEXTURE_COPY_LOCATION", pack=False, align=None),
    "D3D12_TEXTURE_COPY_TYPE": SimTypeInt(signed=False, label="D3D12_TEXTURE_COPY_TYPE"),
    "D3D12_TEXTURE_LAYOUT": SimTypeInt(signed=False, label="D3D12_TEXTURE_LAYOUT"),
    "D3D12_TILED_RESOURCES_TIER": SimTypeInt(signed=False, label="D3D12_TILED_RESOURCES_TIER"),
    "D3D12_TILED_RESOURCE_COORDINATE": SimStruct(OrderedDict((("X", SimTypeInt(signed=False, label="UInt32")), ("Y", SimTypeInt(signed=False, label="UInt32")), ("Z", SimTypeInt(signed=False, label="UInt32")), ("Subresource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TILED_RESOURCE_COORDINATE", pack=False, align=None),
    "D3D12_TILE_COPY_FLAGS": SimTypeInt(signed=False, label="D3D12_TILE_COPY_FLAGS"),
    "D3D12_TILE_MAPPING_FLAGS": SimTypeInt(signed=False, label="D3D12_TILE_MAPPING_FLAGS"),
    "D3D12_TILE_RANGE_FLAGS": SimTypeInt(signed=False, label="D3D12_TILE_RANGE_FLAGS"),
    "D3D12_TILE_REGION_SIZE": SimStruct(OrderedDict((("NumTiles", SimTypeInt(signed=False, label="UInt32")), ("UseBox", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeShort(signed=False, label="UInt16")), ("Depth", SimTypeShort(signed=False, label="UInt16")),)), name="D3D12_TILE_REGION_SIZE", pack=False, align=None),
    "D3D12_TILE_SHAPE": SimStruct(OrderedDict((("WidthInTexels", SimTypeInt(signed=False, label="UInt32")), ("HeightInTexels", SimTypeInt(signed=False, label="UInt32")), ("DepthInTexels", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_TILE_SHAPE", pack=False, align=None),
    "D3D12_TRI_STATE": SimTypeInt(signed=False, label="D3D12_TRI_STATE"),
    "D3D12_UAV_DIMENSION": SimTypeInt(signed=False, label="D3D12_UAV_DIMENSION"),
    "D3D12_UNORDERED_ACCESS_VIEW_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ViewDimension", SimTypeInt(signed=False, label="D3D12_UAV_DIMENSION")), ("Anonymous", SimUnion({"Buffer": SimTypeRef("D3D12_BUFFER_UAV", SimStruct), "Texture1D": SimTypeRef("D3D12_TEX1D_UAV", SimStruct), "Texture1DArray": SimTypeRef("D3D12_TEX1D_ARRAY_UAV", SimStruct), "Texture2D": SimTypeRef("D3D12_TEX2D_UAV", SimStruct), "Texture2DArray": SimTypeRef("D3D12_TEX2D_ARRAY_UAV", SimStruct), "Texture2DMS": SimTypeRef("D3D12_TEX2DMS_UAV", SimStruct), "Texture2DMSArray": SimTypeRef("D3D12_TEX2DMS_ARRAY_UAV", SimStruct), "Texture3D": SimTypeRef("D3D12_TEX3D_UAV", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_UNORDERED_ACCESS_VIEW_DESC", pack=False, align=None),
    "D3D12_VARIABLE_SHADING_RATE_TIER": SimTypeInt(signed=False, label="D3D12_VARIABLE_SHADING_RATE_TIER"),
    "D3D12_VERSIONED_DEVICE_REMOVED_EXTENDED_DATA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="D3D12_DRED_VERSION")), ("Anonymous", SimUnion({"Dred_1_0": SimTypeRef("D3D12_DEVICE_REMOVED_EXTENDED_DATA", SimStruct), "Dred_1_1": SimTypeRef("D3D12_DEVICE_REMOVED_EXTENDED_DATA1", SimStruct), "Dred_1_2": SimTypeRef("D3D12_DEVICE_REMOVED_EXTENDED_DATA2", SimStruct), "Dred_1_3": SimTypeRef("D3D12_DEVICE_REMOVED_EXTENDED_DATA3", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_VERSIONED_DEVICE_REMOVED_EXTENDED_DATA", pack=False, align=None),
    "D3D12_VERSIONED_ROOT_SIGNATURE_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="D3D_ROOT_SIGNATURE_VERSION")), ("Anonymous", SimUnion({"Desc_1_0": SimTypeRef("D3D12_ROOT_SIGNATURE_DESC", SimStruct), "Desc_1_1": SimTypeRef("D3D12_ROOT_SIGNATURE_DESC1", SimStruct), "Desc_1_2": SimTypeRef("D3D12_ROOT_SIGNATURE_DESC2", SimStruct)}, name="<anon>", label="None")),)), name="D3D12_VERSIONED_ROOT_SIGNATURE_DESC", pack=False, align=None),
    "D3D12_VERTEX_BUFFER_VIEW": SimStruct(OrderedDict((("BufferLocation", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("StrideInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VERTEX_BUFFER_VIEW", pack=False, align=None),
    "D3D12_VIDEO_DECODER_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)),)), name="D3D12_VIDEO_DECODER_DESC", pack=False, align=None),
    "D3D12_VIDEO_DECODER_HEAP_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("Configuration", SimTypeRef("D3D12_VIDEO_DECODE_CONFIGURATION", SimStruct)), ("DecodeWidth", SimTypeInt(signed=False, label="UInt32")), ("DecodeHeight", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("FrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("BitRate", SimTypeInt(signed=False, label="UInt32")), ("MaxDecodePictureBufferCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_DECODER_HEAP_DESC", pack=False, align=None),
    "D3D12_VIDEO_DECODE_ARGUMENT_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_ARGUMENT_TYPE"),
    "D3D12_VIDEO_DECODE_COMPRESSED_BITSTREAM": SimStruct(OrderedDict((("pBuffer", SimTypeBottom(label="ID3D12Resource")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_DECODE_COMPRESSED_BITSTREAM", pack=False, align=None),
    "D3D12_VIDEO_DECODE_CONFIGURATION": SimStruct(OrderedDict((("DecodeProfile", SimTypeBottom(label="Guid")), ("BitstreamEncryption", SimTypeInt(signed=False, label="D3D12_BITSTREAM_ENCRYPTION_TYPE")), ("InterlaceType", SimTypeInt(signed=False, label="D3D12_VIDEO_FRAME_CODED_INTERLACE_TYPE")),)), name="D3D12_VIDEO_DECODE_CONFIGURATION", pack=False, align=None),
    "D3D12_VIDEO_DECODE_CONFIGURATION_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_CONFIGURATION_FLAGS"),
    "D3D12_VIDEO_DECODE_CONVERSION_ARGUMENTS": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("pReferenceTexture2D", SimTypeBottom(label="ID3D12Resource")), ("ReferenceSubresource", SimTypeInt(signed=False, label="UInt32")), ("OutputColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")), ("DecodeColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")),)), name="D3D12_VIDEO_DECODE_CONVERSION_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_DECODE_CONVERSION_ARGUMENTS1": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("pReferenceTexture2D", SimTypeBottom(label="ID3D12Resource")), ("ReferenceSubresource", SimTypeInt(signed=False, label="UInt32")), ("OutputColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")), ("DecodeColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")), ("OutputWidth", SimTypeInt(signed=False, label="UInt32")), ("OutputHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_DECODE_CONVERSION_ARGUMENTS1", pack=False, align=None),
    "D3D12_VIDEO_DECODE_CONVERSION_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_CONVERSION_SUPPORT_FLAGS"),
    "D3D12_VIDEO_DECODE_FRAME_ARGUMENT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_ARGUMENT_TYPE")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3D12_VIDEO_DECODE_FRAME_ARGUMENT", pack=False, align=None),
    "D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT"),
    "D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_FLAGS"),
    "D3D12_VIDEO_DECODE_INPUT_STREAM_ARGUMENTS": SimStruct(OrderedDict((("NumFrameArguments", SimTypeInt(signed=False, label="UInt32")), ("FrameArguments", SimTypeArray(SimTypeRef("D3D12_VIDEO_DECODE_FRAME_ARGUMENT", SimStruct), 10)), ("ReferenceFrames", SimTypeRef("D3D12_VIDEO_DECODE_REFERENCE_FRAMES", SimStruct)), ("CompressedBitstream", SimTypeRef("D3D12_VIDEO_DECODE_COMPRESSED_BITSTREAM", SimStruct)), ("pHeap", SimTypeBottom(label="ID3D12VideoDecoderHeap")),)), name="D3D12_VIDEO_DECODE_INPUT_STREAM_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_DECODE_OUTPUT_HISTOGRAM": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("pBuffer", SimTypeBottom(label="ID3D12Resource")),)), name="D3D12_VIDEO_DECODE_OUTPUT_HISTOGRAM", pack=False, align=None),
    "D3D12_VIDEO_DECODE_OUTPUT_STREAM_ARGUMENTS": SimStruct(OrderedDict((("pOutputTexture2D", SimTypeBottom(label="ID3D12Resource")), ("OutputSubresource", SimTypeInt(signed=False, label="UInt32")), ("ConversionArguments", SimTypeRef("D3D12_VIDEO_DECODE_CONVERSION_ARGUMENTS", SimStruct)),)), name="D3D12_VIDEO_DECODE_OUTPUT_STREAM_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_DECODE_OUTPUT_STREAM_ARGUMENTS1": SimStruct(OrderedDict((("pOutputTexture2D", SimTypeBottom(label="ID3D12Resource")), ("OutputSubresource", SimTypeInt(signed=False, label="UInt32")), ("ConversionArguments", SimTypeRef("D3D12_VIDEO_DECODE_CONVERSION_ARGUMENTS1", SimStruct)), ("Histograms", SimTypeArray(SimTypeRef("D3D12_VIDEO_DECODE_OUTPUT_HISTOGRAM", SimStruct), 4)),)), name="D3D12_VIDEO_DECODE_OUTPUT_STREAM_ARGUMENTS1", pack=False, align=None),
    "D3D12_VIDEO_DECODE_REFERENCE_FRAMES": SimStruct(OrderedDict((("NumTexture2Ds", SimTypeInt(signed=False, label="UInt32")), ("ppTexture2Ds", SimTypePointer(SimTypeBottom(label="ID3D12Resource"), offset=0)), ("pSubresources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ppHeaps", SimTypePointer(SimTypeBottom(label="ID3D12VideoDecoderHeap"), offset=0)),)), name="D3D12_VIDEO_DECODE_REFERENCE_FRAMES", pack=False, align=None),
    "D3D12_VIDEO_DECODE_STATUS": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_STATUS"),
    "D3D12_VIDEO_DECODE_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_SUPPORT_FLAGS"),
    "D3D12_VIDEO_DECODE_TIER": SimTypeInt(signed=False, label="D3D12_VIDEO_DECODE_TIER"),
    "D3D12_VIDEO_ENCODER_AV1_CDEF_CONFIG": SimStruct(OrderedDict((("CdefBits", SimTypeLongLong(signed=False, label="UInt64")), ("CdefDampingMinus3", SimTypeLongLong(signed=False, label="UInt64")), ("CdefYPriStrength", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)), ("CdefUVPriStrength", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)), ("CdefYSecStrength", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)), ("CdefUVSecStrength", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="D3D12_VIDEO_ENCODER_AV1_CDEF_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_CODEC_CONFIGURATION": SimStruct(OrderedDict((("FeatureFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FEATURE_FLAGS")), ("OrderHintBitsMinus1", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_AV1_CODEC_CONFIGURATION", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_CODEC_CONFIGURATION_SUPPORT": SimStruct(OrderedDict((("SupportedFeatureFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FEATURE_FLAGS")), ("RequiredFeatureFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FEATURE_FLAGS")), ("SupportedInterpolationFilters", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_INTERPOLATION_FILTERS_FLAGS")), ("SupportedRestorationParams", SimTypeArray(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_RESTORATION_SUPPORT_FLAGS"), 9)), ("SupportedSegmentationModes", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MODE_FLAGS")), ("SupportedTxModes", SimTypeArray(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_TX_MODE_FLAGS"), 4)), ("SegmentationBlockSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_BLOCK_SIZE")), ("PostEncodeValuesFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_POST_ENCODE_VALUES_FLAGS")), ("MaxTemporalLayers", SimTypeInt(signed=False, label="UInt32")), ("MaxSpatialLayers", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_AV1_CODEC_CONFIGURATION_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_COMP_PREDICTION_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_COMP_PREDICTION_TYPE"),
    "D3D12_VIDEO_ENCODER_AV1_FEATURE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FEATURE_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_FRAME_SUBREGION_LAYOUT_CONFIG_SUPPORT": SimStruct(OrderedDict((("Use128SuperBlocks", SimTypeInt(signed=True, label="Int32")), ("TilesConfiguration", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_TILES", SimStruct)), ("ValidationFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_SUBREGION_LAYOUT_CONFIG_VALIDATION_FLAGS")), ("MinTileRows", SimTypeInt(signed=False, label="UInt32")), ("MaxTileRows", SimTypeInt(signed=False, label="UInt32")), ("MinTileCols", SimTypeInt(signed=False, label="UInt32")), ("MaxTileCols", SimTypeInt(signed=False, label="UInt32")), ("MinTileWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxTileWidth", SimTypeInt(signed=False, label="UInt32")), ("MinTileArea", SimTypeInt(signed=False, label="UInt32")), ("MaxTileArea", SimTypeInt(signed=False, label="UInt32")), ("TileSizeBytesMinus1", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_AV1_FRAME_SUBREGION_LAYOUT_CONFIG_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_FRAME_SUBREGION_LAYOUT_CONFIG_VALIDATION_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_SUBREGION_LAYOUT_CONFIG_VALIDATION_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE"),
    "D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_INTERPOLATION_FILTERS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_INTERPOLATION_FILTERS"),
    "D3D12_VIDEO_ENCODER_AV1_INTERPOLATION_FILTERS_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_INTERPOLATION_FILTERS_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_LEVELS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_LEVELS"),
    "D3D12_VIDEO_ENCODER_AV1_LEVEL_TIER_CONSTRAINTS": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_LEVELS")), ("Tier", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_TIER")),)), name="D3D12_VIDEO_ENCODER_AV1_LEVEL_TIER_CONSTRAINTS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_CODEC_DATA": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_FLAGS")), ("FrameType", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE")), ("CompoundPredictionType", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_COMP_PREDICTION_TYPE")), ("InterpolationFilter", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_INTERPOLATION_FILTERS")), ("FrameRestorationConfig", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_RESTORATION_CONFIG", SimStruct)), ("TxMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_TX_MODE")), ("SuperResDenominator", SimTypeInt(signed=False, label="UInt32")), ("OrderHint", SimTypeInt(signed=False, label="UInt32")), ("PictureIndex", SimTypeInt(signed=False, label="UInt32")), ("TemporalLayerIndexPlus1", SimTypeInt(signed=False, label="UInt32")), ("SpatialLayerIndexPlus1", SimTypeInt(signed=False, label="UInt32")), ("ReferenceFramesReconPictureDescriptors", SimTypeArray(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_REFERENCE_PICTURE_DESCRIPTOR", SimStruct), 8)), ("ReferenceIndices", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 7)), ("PrimaryRefFrame", SimTypeInt(signed=False, label="UInt32")), ("RefreshFrameFlags", SimTypeInt(signed=False, label="UInt32")), ("LoopFilter", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_CONFIG", SimStruct)), ("LoopFilterDelta", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_DELTA_CONFIG", SimStruct)), ("Quantization", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_CONFIG", SimStruct)), ("QuantizationDelta", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_DELTA_CONFIG", SimStruct)), ("CDEF", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_CDEF_CONFIG", SimStruct)), ("QPMapValuesCount", SimTypeInt(signed=False, label="UInt32")), ("pRateControlQPMap", SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)), ("CustomSegmentation", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_CONFIG", SimStruct)), ("CustomSegmentsMap", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MAP", SimStruct)),)), name="D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_CODEC_DATA", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_TILES": SimStruct(OrderedDict((("RowCount", SimTypeLongLong(signed=False, label="UInt64")), ("ColCount", SimTypeLongLong(signed=False, label="UInt64")), ("RowHeights", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 64)), ("ColWidths", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 64)), ("ContextUpdateTileId", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_TILES", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_POST_ENCODE_VALUES": SimStruct(OrderedDict((("CompoundPredictionType", SimTypeLongLong(signed=False, label="UInt64")), ("LoopFilter", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_CONFIG", SimStruct)), ("LoopFilterDelta", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_DELTA_CONFIG", SimStruct)), ("Quantization", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_CONFIG", SimStruct)), ("QuantizationDelta", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_DELTA_CONFIG", SimStruct)), ("CDEF", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_CDEF_CONFIG", SimStruct)), ("SegmentationConfig", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_CONFIG", SimStruct)), ("PrimaryRefFrame", SimTypeLongLong(signed=False, label="UInt64")), ("ReferenceIndices", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 7)),)), name="D3D12_VIDEO_ENCODER_AV1_POST_ENCODE_VALUES", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_POST_ENCODE_VALUES_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_POST_ENCODE_VALUES_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_PROFILE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_PROFILE"),
    "D3D12_VIDEO_ENCODER_AV1_REFERENCE_PICTURE_DESCRIPTOR": SimStruct(OrderedDict((("ReconstructedPictureResourceIndex", SimTypeInt(signed=False, label="UInt32")), ("TemporalLayerIndexPlus1", SimTypeInt(signed=False, label="UInt32")), ("SpatialLayerIndexPlus1", SimTypeInt(signed=False, label="UInt32")), ("FrameType", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE")), ("WarpedMotionInfo", SimTypeRef("D3D12_VIDEO_ENCODER_AV1_REFERENCE_PICTURE_WARPED_MOTION_INFO", SimStruct)), ("OrderHint", SimTypeInt(signed=False, label="UInt32")), ("PictureIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_AV1_REFERENCE_PICTURE_DESCRIPTOR", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_REFERENCE_PICTURE_WARPED_MOTION_INFO": SimStruct(OrderedDict((("TransformationType", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_REFERENCE_WARPED_MOTION_TRANSFORMATION")), ("TransformationMatrix", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 8)), ("InvalidAffineSet", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_VIDEO_ENCODER_AV1_REFERENCE_PICTURE_WARPED_MOTION_INFO", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_REFERENCE_WARPED_MOTION_TRANSFORMATION": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_REFERENCE_WARPED_MOTION_TRANSFORMATION"),
    "D3D12_VIDEO_ENCODER_AV1_REFERENCE_WARPED_MOTION_TRANSFORMATION_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_REFERENCE_WARPED_MOTION_TRANSFORMATION_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_RESTORATION_CONFIG": SimStruct(OrderedDict((("FrameRestorationType", SimTypeArray(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_RESTORATION_TYPE"), 3)), ("LoopRestorationPixelSize", SimTypeArray(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_RESTORATION_TILESIZE"), 3)),)), name="D3D12_VIDEO_ENCODER_AV1_RESTORATION_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_RESTORATION_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_RESTORATION_SUPPORT_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_RESTORATION_TILESIZE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_RESTORATION_TILESIZE"),
    "D3D12_VIDEO_ENCODER_AV1_RESTORATION_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_RESTORATION_TYPE"),
    "D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_BLOCK_SIZE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_BLOCK_SIZE"),
    "D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_CONFIG": SimStruct(OrderedDict((("UpdateMap", SimTypeLongLong(signed=False, label="UInt64")), ("TemporalUpdate", SimTypeLongLong(signed=False, label="UInt64")), ("UpdateData", SimTypeLongLong(signed=False, label="UInt64")), ("NumSegments", SimTypeLongLong(signed=False, label="UInt64")), ("SegmentsData", SimTypeArray(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_SEGMENT_DATA", SimStruct), 8)),)), name="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MAP": SimStruct(OrderedDict((("SegmentsMapByteSize", SimTypeInt(signed=False, label="UInt32")), ("pSegmentsMap", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MAP", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MODE"),
    "D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MODE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_SEGMENTATION_MODE_FLAGS"),
    "D3D12_VIDEO_ENCODER_AV1_SEGMENT_DATA": SimStruct(OrderedDict((("EnabledFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("FeatureValue", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 8)),)), name="D3D12_VIDEO_ENCODER_AV1_SEGMENT_DATA", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_SEQUENCE_STRUCTURE": SimStruct(OrderedDict((("IntraDistance", SimTypeInt(signed=False, label="UInt32")), ("InterFramePeriod", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_AV1_SEQUENCE_STRUCTURE", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_AV1_TIER": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_TIER"),
    "D3D12_VIDEO_ENCODER_AV1_TX_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_TX_MODE"),
    "D3D12_VIDEO_ENCODER_AV1_TX_MODE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_TX_MODE_FLAGS"),
    "D3D12_VIDEO_ENCODER_CODEC": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC"),
    "D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_CONFIG": SimStruct(OrderedDict((("LoopFilterLevel", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)), ("LoopFilterLevelU", SimTypeLongLong(signed=False, label="UInt64")), ("LoopFilterLevelV", SimTypeLongLong(signed=False, label="UInt64")), ("LoopFilterSharpnessLevel", SimTypeLongLong(signed=False, label="UInt64")), ("LoopFilterDeltaEnabled", SimTypeLongLong(signed=False, label="UInt64")), ("UpdateRefDelta", SimTypeLongLong(signed=False, label="UInt64")), ("RefDeltas", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 8)), ("UpdateModeDelta", SimTypeLongLong(signed=False, label="UInt64")), ("ModeDeltas", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 2)),)), name="D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_DELTA_CONFIG": SimStruct(OrderedDict((("DeltaLFPresent", SimTypeLongLong(signed=False, label="UInt64")), ("DeltaLFMulti", SimTypeLongLong(signed=False, label="UInt64")), ("DeltaLFRes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_CODEC_AV1_LOOP_FILTER_DELTA_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_AV1_PICTURE_CONTROL_SUPPORT": SimStruct(OrderedDict((("PredictionMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_COMP_PREDICTION_TYPE")), ("MaxUniqueReferencesPerFrame", SimTypeInt(signed=False, label="UInt32")), ("SupportedFrameTypes", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_FRAME_TYPE_FLAGS")), ("SupportedReferenceWarpedMotionFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_REFERENCE_WARPED_MOTION_TRANSFORMATION_FLAGS")),)), name="D3D12_VIDEO_ENCODER_CODEC_AV1_PICTURE_CONTROL_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_CONFIG": SimStruct(OrderedDict((("BaseQIndex", SimTypeLongLong(signed=False, label="UInt64")), ("YDCDeltaQ", SimTypeLongLong(signed=True, label="Int64")), ("UDCDeltaQ", SimTypeLongLong(signed=True, label="Int64")), ("UACDeltaQ", SimTypeLongLong(signed=True, label="Int64")), ("VDCDeltaQ", SimTypeLongLong(signed=True, label="Int64")), ("VACDeltaQ", SimTypeLongLong(signed=True, label="Int64")), ("UsingQMatrix", SimTypeLongLong(signed=False, label="UInt64")), ("QMY", SimTypeLongLong(signed=False, label="UInt64")), ("QMU", SimTypeLongLong(signed=False, label="UInt64")), ("QMV", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_DELTA_CONFIG": SimStruct(OrderedDict((("DeltaQPresent", SimTypeLongLong(signed=False, label="UInt64")), ("DeltaQRes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_CODEC_AV1_QUANTIZATION_DELTA_CONFIG", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264Config": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264", SimStruct), offset=0), "pHEVCConfig": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC", SimStruct), offset=0), "pAV1Config": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_CODEC_CONFIGURATION", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264": SimStruct(OrderedDict((("ConfigurationFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_FLAGS")), ("DirectModeConfig", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_DIRECT_MODES")), ("DisableDeblockingFilterConfig", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_SLICES_DEBLOCKING_MODES")),)), name="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_DIRECT_MODES": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_DIRECT_MODES"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_FLAGS"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_SLICES_DEBLOCKING_MODES": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_SLICES_DEBLOCKING_MODES"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_SLICES_DEBLOCKING_MODE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_SLICES_DEBLOCKING_MODE_FLAGS"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC": SimStruct(OrderedDict((("ConfigurationFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_FLAGS")), ("MinLumaCodingUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_CUSIZE")), ("MaxLumaCodingUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_CUSIZE")), ("MinLumaTransformUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_TUSIZE")), ("MaxLumaTransformUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_TUSIZE")), ("max_transform_hierarchy_depth_inter", SimTypeChar(label="Byte")), ("max_transform_hierarchy_depth_intra", SimTypeChar(label="Byte")),)), name="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_CUSIZE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_CUSIZE"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_FLAGS"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_TUSIZE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_TUSIZE"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264Support": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_H264", SimStruct), offset=0), "pHEVCSupport": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_HEVC", SimStruct), offset=0), "pAV1Support": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_CODEC_CONFIGURATION_SUPPORT", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_H264": SimStruct(OrderedDict((("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_H264_FLAGS")), ("DisableDeblockingFilterSupportedModes", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_H264_SLICES_DEBLOCKING_MODE_FLAGS")),)), name="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_H264", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_H264_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_H264_FLAGS"),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_HEVC": SimStruct(OrderedDict((("SupportFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_HEVC_FLAGS")), ("MinLumaCodingUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_CUSIZE")), ("MaxLumaCodingUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_CUSIZE")), ("MinLumaTransformUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_TUSIZE")), ("MaxLumaTransformUnitSize", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_HEVC_TUSIZE")), ("max_transform_hierarchy_depth_inter", SimTypeChar(label="Byte")), ("max_transform_hierarchy_depth_intra", SimTypeChar(label="Byte")),)), name="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_HEVC_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION_SUPPORT_HEVC_FLAGS"),
    "D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264Support": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT_H264", SimStruct), offset=0), "pHEVCSupport": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT_HEVC", SimStruct), offset=0), "pAV1Support": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_AV1_PICTURE_CONTROL_SUPPORT", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT_H264": SimStruct(OrderedDict((("MaxL0ReferencesForP", SimTypeInt(signed=False, label="UInt32")), ("MaxL0ReferencesForB", SimTypeInt(signed=False, label="UInt32")), ("MaxL1ReferencesForB", SimTypeInt(signed=False, label="UInt32")), ("MaxLongTermReferences", SimTypeInt(signed=False, label="UInt32")), ("MaxDPBCapacity", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT_H264", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT_HEVC": SimStruct(OrderedDict((("MaxL0ReferencesForP", SimTypeInt(signed=False, label="UInt32")), ("MaxL0ReferencesForB", SimTypeInt(signed=False, label="UInt32")), ("MaxL1ReferencesForB", SimTypeInt(signed=False, label="UInt32")), ("MaxLongTermReferences", SimTypeInt(signed=False, label="UInt32")), ("MaxDPBCapacity", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_CODEC_PICTURE_CONTROL_SUPPORT_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM": SimStruct(OrderedDict((("pBuffer", SimTypeBottom(label="ID3D12Resource")), ("FrameStartOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FLAGS")), ("EncodeCodec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("EncodeProfile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("CodecConfiguration", SimTypeRef("D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION", SimStruct)), ("MaxMotionEstimationPrecision", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE")),)), name="D3D12_VIDEO_ENCODER_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_ENCODEFRAME_INPUT_ARGUMENTS": SimStruct(OrderedDict((("SequenceControlDesc", SimTypeRef("D3D12_VIDEO_ENCODER_SEQUENCE_CONTROL_DESC", SimStruct)), ("PictureControlDesc", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_DESC", SimStruct)), ("pInputFrame", SimTypeBottom(label="ID3D12Resource")), ("InputFrameSubresource", SimTypeInt(signed=False, label="UInt32")), ("CurrentFrameBitstreamMetadataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_ENCODEFRAME_INPUT_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_ENCODEFRAME_OUTPUT_ARGUMENTS": SimStruct(OrderedDict((("Bitstream", SimTypeRef("D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM", SimStruct)), ("ReconstructedPicture", SimTypeRef("D3D12_VIDEO_ENCODER_RECONSTRUCTED_PICTURE", SimStruct)), ("EncoderOutputMetadata", SimTypeRef("D3D12_VIDEO_ENCODER_ENCODE_OPERATION_METADATA_BUFFER", SimStruct)),)), name="D3D12_VIDEO_ENCODER_ENCODEFRAME_OUTPUT_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_ENCODE_ERROR_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_ENCODE_ERROR_FLAGS"),
    "D3D12_VIDEO_ENCODER_ENCODE_OPERATION_METADATA_BUFFER": SimStruct(OrderedDict((("pBuffer", SimTypeBottom(label="ID3D12Resource")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_ENCODE_OPERATION_METADATA_BUFFER", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FLAGS"),
    "D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_CONFIG_SUPPORT": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pAV1Support": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_FRAME_SUBREGION_LAYOUT_CONFIG_SUPPORT", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_CONFIG_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE"),
    "D3D12_VIDEO_ENCODER_FRAME_SUBREGION_METADATA": SimStruct(OrderedDict((("bSize", SimTypeLongLong(signed=False, label="UInt64")), ("bStartOffset", SimTypeLongLong(signed=False, label="UInt64")), ("bHeaderSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_METADATA", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_FRAME_TYPE_H264": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_TYPE_H264"),
    "D3D12_VIDEO_ENCODER_FRAME_TYPE_HEVC": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_TYPE_HEVC"),
    "D3D12_VIDEO_ENCODER_HEAP_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_HEAP_FLAGS")), ("EncodeCodec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("EncodeProfile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("EncodeLevel", SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_SETTING", SimStruct)), ("ResolutionsListCount", SimTypeInt(signed=False, label="UInt32")), ("pResolutionList", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct), offset=0)),)), name="D3D12_VIDEO_ENCODER_HEAP_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_HEAP_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_HEAP_FLAGS"),
    "D3D12_VIDEO_ENCODER_INTRA_REFRESH": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_INTRA_REFRESH_MODE")), ("IntraRefreshDuration", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_INTRA_REFRESH", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_INTRA_REFRESH_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_INTRA_REFRESH_MODE"),
    "D3D12_VIDEO_ENCODER_LEVELS_H264": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_LEVELS_H264"),
    "D3D12_VIDEO_ENCODER_LEVELS_HEVC": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_LEVELS_HEVC"),
    "D3D12_VIDEO_ENCODER_LEVEL_SETTING": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264LevelSetting": SimTypePointer(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_LEVELS_H264"), offset=0), "pHEVCLevelSetting": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_LEVEL_TIER_CONSTRAINTS_HEVC", SimStruct), offset=0), "pAV1LevelSetting": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_LEVEL_TIER_CONSTRAINTS", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_LEVEL_SETTING", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_LEVEL_TIER_CONSTRAINTS_HEVC": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_LEVELS_HEVC")), ("Tier", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_TIER_HEVC")),)), name="D3D12_VIDEO_ENCODER_LEVEL_TIER_CONSTRAINTS_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE"),
    "D3D12_VIDEO_ENCODER_OUTPUT_METADATA": SimStruct(OrderedDict((("EncodeErrorFlags", SimTypeLongLong(signed=False, label="UInt64")), ("EncodeStats", SimTypeRef("D3D12_VIDEO_ENCODER_OUTPUT_METADATA_STATISTICS", SimStruct)), ("EncodedBitstreamWrittenBytesCount", SimTypeLongLong(signed=False, label="UInt64")), ("WrittenSubregionsCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_OUTPUT_METADATA", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_OUTPUT_METADATA_STATISTICS": SimStruct(OrderedDict((("AverageQP", SimTypeLongLong(signed=False, label="UInt64")), ("IntraCodingUnitsCount", SimTypeLongLong(signed=False, label="UInt64")), ("InterCodingUnitsCount", SimTypeLongLong(signed=False, label="UInt64")), ("SkipCodingUnitsCount", SimTypeLongLong(signed=False, label="UInt64")), ("AverageMotionEstimationXDirection", SimTypeLongLong(signed=False, label="UInt64")), ("AverageMotionEstimationYDirection", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_OUTPUT_METADATA_STATISTICS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264PicData": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264", SimStruct), offset=0), "pHEVCPicData": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC", SimStruct), offset=0), "pAV1PicData": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_CODEC_DATA", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_FLAGS")), ("FrameType", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_TYPE_H264")), ("pic_parameter_set_id", SimTypeInt(signed=False, label="UInt32")), ("idr_pic_id", SimTypeInt(signed=False, label="UInt32")), ("PictureOrderCountNumber", SimTypeInt(signed=False, label="UInt32")), ("FrameDecodingOrderNumber", SimTypeInt(signed=False, label="UInt32")), ("TemporalLayerIndex", SimTypeInt(signed=False, label="UInt32")), ("List0ReferenceFramesCount", SimTypeInt(signed=False, label="UInt32")), ("pList0ReferenceFrames", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("List1ReferenceFramesCount", SimTypeInt(signed=False, label="UInt32")), ("pList1ReferenceFrames", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ReferenceFramesReconPictureDescriptorsCount", SimTypeInt(signed=False, label="UInt32")), ("pReferenceFramesReconPictureDescriptors", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_REFERENCE_PICTURE_DESCRIPTOR_H264", SimStruct), offset=0)), ("adaptive_ref_pic_marking_mode_flag", SimTypeChar(label="Byte")), ("RefPicMarkingOperationsCommandsCount", SimTypeInt(signed=False, label="UInt32")), ("pRefPicMarkingOperationsCommands", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_MARKING_OPERATION", SimStruct), offset=0)), ("List0RefPicModificationsCount", SimTypeInt(signed=False, label="UInt32")), ("pList0RefPicModifications", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_LIST_MODIFICATION_OPERATION", SimStruct), offset=0)), ("List1RefPicModificationsCount", SimTypeInt(signed=False, label="UInt32")), ("pList1RefPicModifications", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_LIST_MODIFICATION_OPERATION", SimStruct), offset=0)), ("QPMapValuesCount", SimTypeInt(signed=False, label="UInt32")), ("pRateControlQPMap", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_FLAGS"),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_LIST_MODIFICATION_OPERATION": SimStruct(OrderedDict((("modification_of_pic_nums_idc", SimTypeChar(label="Byte")), ("abs_diff_pic_num_minus1", SimTypeInt(signed=False, label="UInt32")), ("long_term_pic_num", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_LIST_MODIFICATION_OPERATION", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_MARKING_OPERATION": SimStruct(OrderedDict((("memory_management_control_operation", SimTypeChar(label="Byte")), ("difference_of_pic_nums_minus1", SimTypeInt(signed=False, label="UInt32")), ("long_term_pic_num", SimTypeInt(signed=False, label="UInt32")), ("long_term_frame_idx", SimTypeInt(signed=False, label="UInt32")), ("max_long_term_frame_idx_plus1", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264_REFERENCE_PICTURE_MARKING_OPERATION", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC_FLAGS")), ("FrameType", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_TYPE_HEVC")), ("slice_pic_parameter_set_id", SimTypeInt(signed=False, label="UInt32")), ("PictureOrderCountNumber", SimTypeInt(signed=False, label="UInt32")), ("TemporalLayerIndex", SimTypeInt(signed=False, label="UInt32")), ("List0ReferenceFramesCount", SimTypeInt(signed=False, label="UInt32")), ("pList0ReferenceFrames", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("List1ReferenceFramesCount", SimTypeInt(signed=False, label="UInt32")), ("pList1ReferenceFrames", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ReferenceFramesReconPictureDescriptorsCount", SimTypeInt(signed=False, label="UInt32")), ("pReferenceFramesReconPictureDescriptors", SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_REFERENCE_PICTURE_DESCRIPTOR_HEVC", SimStruct), offset=0)), ("List0RefPicModificationsCount", SimTypeInt(signed=False, label="UInt32")), ("pList0RefPicModifications", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("List1RefPicModificationsCount", SimTypeInt(signed=False, label="UInt32")), ("pList1RefPicModifications", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("QPMapValuesCount", SimTypeInt(signed=False, label="UInt32")), ("pRateControlQPMap", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC_FLAGS"),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_DESC": SimStruct(OrderedDict((("IntraRefreshFrameIndex", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_FLAGS")), ("PictureControlCodecData", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA", SimStruct)), ("ReferenceFrames", SimTypeRef("D3D12_VIDEO_ENCODE_REFERENCE_FRAMES", SimStruct)),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_FLAGS"),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pSlicesPartition_H264": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_SLICES", SimStruct), offset=0), "pSlicesPartition_HEVC": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_SLICES", SimStruct), offset=0), "pTilesPartition_AV1": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_TILES", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_SLICES": SimStruct(OrderedDict((("Anonymous", SimUnion({"MaxBytesPerSlice": SimTypeInt(signed=False, label="UInt32"), "NumberOfCodingUnitsPerSlice": SimTypeInt(signed=False, label="UInt32"), "NumberOfRowsPerSlice": SimTypeInt(signed=False, label="UInt32"), "NumberOfSlicesPerFrame": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA_SLICES", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_RATIO_DESC": SimStruct(OrderedDict((("WidthRatio", SimTypeInt(signed=False, label="UInt32")), ("HeightRatio", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_RATIO_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PROFILE_DESC": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264Profile": SimTypePointer(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PROFILE_H264"), offset=0), "pHEVCProfile": SimTypePointer(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PROFILE_HEVC"), offset=0), "pAV1Profile": SimTypePointer(SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_AV1_PROFILE"), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_PROFILE_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_PROFILE_H264": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PROFILE_H264"),
    "D3D12_VIDEO_ENCODER_PROFILE_HEVC": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_PROFILE_HEVC"),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_RATE_CONTROL_MODE")), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_RATE_CONTROL_FLAGS")), ("ConfigParams", SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_CONFIGURATION_PARAMS", SimStruct)), ("TargetFrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_ABSOLUTE_QP_MAP": SimStruct(OrderedDict((("QualityVsSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_ABSOLUTE_QP_MAP", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_CBR": SimStruct(OrderedDict((("InitialQP", SimTypeInt(signed=False, label="UInt32")), ("MinQP", SimTypeInt(signed=False, label="UInt32")), ("MaxQP", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameBitSize", SimTypeLongLong(signed=False, label="UInt64")), ("TargetBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("VBVCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("InitialVBVFullness", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_CBR", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_CBR1": SimStruct(OrderedDict((("InitialQP", SimTypeInt(signed=False, label="UInt32")), ("MinQP", SimTypeInt(signed=False, label="UInt32")), ("MaxQP", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameBitSize", SimTypeLongLong(signed=False, label="UInt64")), ("TargetBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("VBVCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("InitialVBVFullness", SimTypeLongLong(signed=False, label="UInt64")), ("QualityVsSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_CBR1", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_CONFIGURATION_PARAMS": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pConfiguration_CQP": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_CQP", SimStruct), offset=0), "pConfiguration_CBR": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_CBR", SimStruct), offset=0), "pConfiguration_VBR": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR", SimStruct), offset=0), "pConfiguration_QVBR": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR", SimStruct), offset=0), "pConfiguration_CQP1": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_CQP1", SimStruct), offset=0), "pConfiguration_CBR1": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_CBR1", SimStruct), offset=0), "pConfiguration_VBR1": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR1", SimStruct), offset=0), "pConfiguration_QVBR1": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR1", SimStruct), offset=0), "pConfiguration_AbsoluteQPMap": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL_ABSOLUTE_QP_MAP", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_CONFIGURATION_PARAMS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_CQP": SimStruct(OrderedDict((("ConstantQP_FullIntracodedFrame", SimTypeInt(signed=False, label="UInt32")), ("ConstantQP_InterPredictedFrame_PrevRefOnly", SimTypeInt(signed=False, label="UInt32")), ("ConstantQP_InterPredictedFrame_BiDirectionalRef", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_CQP", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_CQP1": SimStruct(OrderedDict((("ConstantQP_FullIntracodedFrame", SimTypeInt(signed=False, label="UInt32")), ("ConstantQP_InterPredictedFrame_PrevRefOnly", SimTypeInt(signed=False, label="UInt32")), ("ConstantQP_InterPredictedFrame_BiDirectionalRef", SimTypeInt(signed=False, label="UInt32")), ("QualityVsSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_CQP1", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_RATE_CONTROL_FLAGS"),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_RATE_CONTROL_MODE"),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR": SimStruct(OrderedDict((("InitialQP", SimTypeInt(signed=False, label="UInt32")), ("MinQP", SimTypeInt(signed=False, label="UInt32")), ("MaxQP", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameBitSize", SimTypeLongLong(signed=False, label="UInt64")), ("TargetAvgBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("PeakBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("ConstantQualityTarget", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR1": SimStruct(OrderedDict((("InitialQP", SimTypeInt(signed=False, label="UInt32")), ("MinQP", SimTypeInt(signed=False, label="UInt32")), ("MaxQP", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameBitSize", SimTypeLongLong(signed=False, label="UInt64")), ("TargetAvgBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("PeakBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("ConstantQualityTarget", SimTypeInt(signed=False, label="UInt32")), ("VBVCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("InitialVBVFullness", SimTypeLongLong(signed=False, label="UInt64")), ("QualityVsSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR1", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR": SimStruct(OrderedDict((("InitialQP", SimTypeInt(signed=False, label="UInt32")), ("MinQP", SimTypeInt(signed=False, label="UInt32")), ("MaxQP", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameBitSize", SimTypeLongLong(signed=False, label="UInt64")), ("TargetAvgBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("PeakBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("VBVCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("InitialVBVFullness", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR1": SimStruct(OrderedDict((("InitialQP", SimTypeInt(signed=False, label="UInt32")), ("MinQP", SimTypeInt(signed=False, label="UInt32")), ("MaxQP", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameBitSize", SimTypeLongLong(signed=False, label="UInt64")), ("TargetAvgBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("PeakBitRate", SimTypeLongLong(signed=False, label="UInt64")), ("VBVCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("InitialVBVFullness", SimTypeLongLong(signed=False, label="UInt64")), ("QualityVsSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR1", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RECONSTRUCTED_PICTURE": SimStruct(OrderedDict((("pReconstructedPicture", SimTypeBottom(label="ID3D12Resource")), ("ReconstructedPictureSubresource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_RECONSTRUCTED_PICTURE", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_REFERENCE_PICTURE_DESCRIPTOR_H264": SimStruct(OrderedDict((("ReconstructedPictureResourceIndex", SimTypeInt(signed=False, label="UInt32")), ("IsLongTermReference", SimTypeInt(signed=True, label="Int32")), ("LongTermPictureIdx", SimTypeInt(signed=False, label="UInt32")), ("PictureOrderCountNumber", SimTypeInt(signed=False, label="UInt32")), ("FrameDecodingOrderNumber", SimTypeInt(signed=False, label="UInt32")), ("TemporalLayerIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_REFERENCE_PICTURE_DESCRIPTOR_H264", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_REFERENCE_PICTURE_DESCRIPTOR_HEVC": SimStruct(OrderedDict((("ReconstructedPictureResourceIndex", SimTypeInt(signed=False, label="UInt32")), ("IsRefUsedByCurrentPic", SimTypeInt(signed=True, label="Int32")), ("IsLongTermReference", SimTypeInt(signed=True, label="Int32")), ("PictureOrderCountNumber", SimTypeInt(signed=False, label="UInt32")), ("TemporalLayerIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_ENCODER_REFERENCE_PICTURE_DESCRIPTOR_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RESOLVE_METADATA_INPUT_ARGUMENTS": SimStruct(OrderedDict((("EncoderCodec", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_CODEC")), ("EncoderProfile", SimTypeRef("D3D12_VIDEO_ENCODER_PROFILE_DESC", SimStruct)), ("EncoderInputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("EncodedPictureEffectiveResolution", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct)), ("HWLayoutMetadata", SimTypeRef("D3D12_VIDEO_ENCODER_ENCODE_OPERATION_METADATA_BUFFER", SimStruct)),)), name="D3D12_VIDEO_ENCODER_RESOLVE_METADATA_INPUT_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_RESOLVE_METADATA_OUTPUT_ARGUMENTS": SimStruct(OrderedDict((("ResolvedLayoutMetadata", SimTypeRef("D3D12_VIDEO_ENCODER_ENCODE_OPERATION_METADATA_BUFFER", SimStruct)),)), name="D3D12_VIDEO_ENCODER_RESOLVE_METADATA_OUTPUT_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_SEQUENCE_CONTROL_DESC": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_SEQUENCE_CONTROL_FLAGS")), ("IntraRefreshConfig", SimTypeRef("D3D12_VIDEO_ENCODER_INTRA_REFRESH", SimStruct)), ("RateControl", SimTypeRef("D3D12_VIDEO_ENCODER_RATE_CONTROL", SimStruct)), ("PictureTargetResolution", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_RESOLUTION_DESC", SimStruct)), ("SelectedLayoutMode", SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE")), ("FrameSubregionsLayoutData", SimTypeRef("D3D12_VIDEO_ENCODER_PICTURE_CONTROL_SUBREGIONS_LAYOUT_DATA", SimStruct)), ("CodecGopSequence", SimTypeRef("D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE", SimStruct)),)), name="D3D12_VIDEO_ENCODER_SEQUENCE_CONTROL_DESC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_SEQUENCE_CONTROL_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_SEQUENCE_CONTROL_FLAGS"),
    "D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pH264GroupOfPictures": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE_H264", SimStruct), offset=0), "pHEVCGroupOfPictures": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE_HEVC", SimStruct), offset=0), "pAV1SequenceStructure": SimTypePointer(SimTypeRef("D3D12_VIDEO_ENCODER_AV1_SEQUENCE_STRUCTURE", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE_H264": SimStruct(OrderedDict((("GOPLength", SimTypeInt(signed=False, label="UInt32")), ("PPicturePeriod", SimTypeInt(signed=False, label="UInt32")), ("pic_order_cnt_type", SimTypeChar(label="Byte")), ("log2_max_frame_num_minus4", SimTypeChar(label="Byte")), ("log2_max_pic_order_cnt_lsb_minus4", SimTypeChar(label="Byte")),)), name="D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE_H264", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE_HEVC": SimStruct(OrderedDict((("GOPLength", SimTypeInt(signed=False, label="UInt32")), ("PPicturePeriod", SimTypeInt(signed=False, label="UInt32")), ("log2_max_pic_order_cnt_lsb_minus4", SimTypeChar(label="Byte")),)), name="D3D12_VIDEO_ENCODER_SEQUENCE_GOP_STRUCTURE_HEVC", pack=False, align=None),
    "D3D12_VIDEO_ENCODER_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_SUPPORT_FLAGS"),
    "D3D12_VIDEO_ENCODER_TIER_HEVC": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_TIER_HEVC"),
    "D3D12_VIDEO_ENCODER_VALIDATION_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_ENCODER_VALIDATION_FLAGS"),
    "D3D12_VIDEO_ENCODE_REFERENCE_FRAMES": SimStruct(OrderedDict((("NumTexture2Ds", SimTypeInt(signed=False, label="UInt32")), ("ppTexture2Ds", SimTypePointer(SimTypeBottom(label="ID3D12Resource"), offset=0)), ("pSubresources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3D12_VIDEO_ENCODE_REFERENCE_FRAMES", pack=False, align=None),
    "D3D12_VIDEO_EXTENSION_COMMAND_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("CommandId", SimTypeBottom(label="Guid")),)), name="D3D12_VIDEO_EXTENSION_COMMAND_DESC", pack=False, align=None),
    "D3D12_VIDEO_EXTENSION_COMMAND_INFO": SimStruct(OrderedDict((("CommandId", SimTypeBottom(label="Guid")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CommandListSupportFlags", SimTypeInt(signed=False, label="D3D12_COMMAND_LIST_SUPPORT_FLAGS")),)), name="D3D12_VIDEO_EXTENSION_COMMAND_INFO", pack=False, align=None),
    "D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_FLAGS"),
    "D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_INFO": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Type", SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_TYPE")), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_FLAGS")),)), name="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_INFO", pack=False, align=None),
    "D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_STAGE": SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_STAGE"),
    "D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_EXTENSION_COMMAND_PARAMETER_TYPE"),
    "D3D12_VIDEO_FIELD_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_FIELD_TYPE"),
    "D3D12_VIDEO_FORMAT": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")),)), name="D3D12_VIDEO_FORMAT", pack=False, align=None),
    "D3D12_VIDEO_FRAME_CODED_INTERLACE_TYPE": SimTypeInt(signed=False, label="D3D12_VIDEO_FRAME_CODED_INTERLACE_TYPE"),
    "D3D12_VIDEO_FRAME_STEREO_FORMAT": SimTypeInt(signed=False, label="D3D12_VIDEO_FRAME_STEREO_FORMAT"),
    "D3D12_VIDEO_MOTION_ESTIMATOR_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("BlockSize", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE")), ("Precision", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION")), ("SizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)),)), name="D3D12_VIDEO_MOTION_ESTIMATOR_DESC", pack=False, align=None),
    "D3D12_VIDEO_MOTION_ESTIMATOR_INPUT": SimStruct(OrderedDict((("pInputTexture2D", SimTypeBottom(label="ID3D12Resource")), ("InputSubresourceIndex", SimTypeInt(signed=False, label="UInt32")), ("pReferenceTexture2D", SimTypeBottom(label="ID3D12Resource")), ("ReferenceSubresourceIndex", SimTypeInt(signed=False, label="UInt32")), ("pHintMotionVectorHeap", SimTypeBottom(label="ID3D12VideoMotionVectorHeap")),)), name="D3D12_VIDEO_MOTION_ESTIMATOR_INPUT", pack=False, align=None),
    "D3D12_VIDEO_MOTION_ESTIMATOR_OUTPUT": SimStruct(OrderedDict((("pMotionVectorHeap", SimTypeBottom(label="ID3D12VideoMotionVectorHeap")),)), name="D3D12_VIDEO_MOTION_ESTIMATOR_OUTPUT", pack=False, align=None),
    "D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE": SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE"),
    "D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_FLAGS"),
    "D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION": SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION"),
    "D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION_FLAGS"),
    "D3D12_VIDEO_MOTION_VECTOR_HEAP_DESC": SimStruct(OrderedDict((("NodeMask", SimTypeInt(signed=False, label="UInt32")), ("InputFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("BlockSize", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE")), ("Precision", SimTypeInt(signed=False, label="D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION")), ("SizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)),)), name="D3D12_VIDEO_MOTION_VECTOR_HEAP_DESC", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_ALPHA_BLENDING": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Alpha", SimTypeFloat(size=32)),)), name="D3D12_VIDEO_PROCESS_ALPHA_BLENDING", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_ALPHA_FILL_MODE": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_ALPHA_FILL_MODE"),
    "D3D12_VIDEO_PROCESS_AUTO_PROCESSING_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_AUTO_PROCESSING_FLAGS"),
    "D3D12_VIDEO_PROCESS_DEINTERLACE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_DEINTERLACE_FLAGS"),
    "D3D12_VIDEO_PROCESS_FEATURE_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FEATURE_FLAGS"),
    "D3D12_VIDEO_PROCESS_FILTER": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FILTER"),
    "D3D12_VIDEO_PROCESS_FILTER_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FILTER_FLAGS"),
    "D3D12_VIDEO_PROCESS_FILTER_RANGE": SimStruct(OrderedDict((("Minimum", SimTypeInt(signed=True, label="Int32")), ("Maximum", SimTypeInt(signed=True, label="Int32")), ("Default", SimTypeInt(signed=True, label="Int32")), ("Multiplier", SimTypeFloat(size=32)),)), name="D3D12_VIDEO_PROCESS_FILTER_RANGE", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_INPUT_STREAM": SimStruct(OrderedDict((("pTexture2D", SimTypeBottom(label="ID3D12Resource")), ("Subresource", SimTypeInt(signed=False, label="UInt32")), ("ReferenceSet", SimTypeRef("D3D12_VIDEO_PROCESS_REFERENCE_SET", SimStruct)),)), name="D3D12_VIDEO_PROCESS_INPUT_STREAM", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_INPUT_STREAM_ARGUMENTS": SimStruct(OrderedDict((("InputStream", SimTypeArray(SimTypeRef("D3D12_VIDEO_PROCESS_INPUT_STREAM", SimStruct), 2)), ("Transform", SimTypeRef("D3D12_VIDEO_PROCESS_TRANSFORM", SimStruct)), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_INPUT_STREAM_FLAGS")), ("RateInfo", SimTypeRef("D3D12_VIDEO_PROCESS_INPUT_STREAM_RATE", SimStruct)), ("FilterLevels", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 32)), ("AlphaBlending", SimTypeRef("D3D12_VIDEO_PROCESS_ALPHA_BLENDING", SimStruct)),)), name="D3D12_VIDEO_PROCESS_INPUT_STREAM_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_INPUT_STREAM_ARGUMENTS1": SimStruct(OrderedDict((("InputStream", SimTypeArray(SimTypeRef("D3D12_VIDEO_PROCESS_INPUT_STREAM", SimStruct), 2)), ("Transform", SimTypeRef("D3D12_VIDEO_PROCESS_TRANSFORM", SimStruct)), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_INPUT_STREAM_FLAGS")), ("RateInfo", SimTypeRef("D3D12_VIDEO_PROCESS_INPUT_STREAM_RATE", SimStruct)), ("FilterLevels", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 32)), ("AlphaBlending", SimTypeRef("D3D12_VIDEO_PROCESS_ALPHA_BLENDING", SimStruct)), ("FieldType", SimTypeInt(signed=False, label="D3D12_VIDEO_FIELD_TYPE")),)), name="D3D12_VIDEO_PROCESS_INPUT_STREAM_ARGUMENTS1", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_INPUT_STREAM_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")), ("SourceAspectRatio", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("DestinationAspectRatio", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("FrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("SourceSizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)), ("DestinationSizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)), ("EnableOrientation", SimTypeInt(signed=True, label="Int32")), ("FilterFlags", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_FILTER_FLAGS")), ("StereoFormat", SimTypeInt(signed=False, label="D3D12_VIDEO_FRAME_STEREO_FORMAT")), ("FieldType", SimTypeInt(signed=False, label="D3D12_VIDEO_FIELD_TYPE")), ("DeinterlaceMode", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_DEINTERLACE_FLAGS")), ("EnableAlphaBlending", SimTypeInt(signed=True, label="Int32")), ("LumaKey", SimTypeRef("D3D12_VIDEO_PROCESS_LUMA_KEY", SimStruct)), ("NumPastFrames", SimTypeInt(signed=False, label="UInt32")), ("NumFutureFrames", SimTypeInt(signed=False, label="UInt32")), ("EnableAutoProcessing", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_VIDEO_PROCESS_INPUT_STREAM_DESC", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_INPUT_STREAM_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_INPUT_STREAM_FLAGS"),
    "D3D12_VIDEO_PROCESS_INPUT_STREAM_RATE": SimStruct(OrderedDict((("OutputIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFrameOrField", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_PROCESS_INPUT_STREAM_RATE", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_LUMA_KEY": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Lower", SimTypeFloat(size=32)), ("Upper", SimTypeFloat(size=32)),)), name="D3D12_VIDEO_PROCESS_LUMA_KEY", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_ORIENTATION": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_ORIENTATION"),
    "D3D12_VIDEO_PROCESS_OUTPUT_STREAM": SimStruct(OrderedDict((("pTexture2D", SimTypeBottom(label="ID3D12Resource")), ("Subresource", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_PROCESS_OUTPUT_STREAM", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_OUTPUT_STREAM_ARGUMENTS": SimStruct(OrderedDict((("OutputStream", SimTypeArray(SimTypeRef("D3D12_VIDEO_PROCESS_OUTPUT_STREAM", SimStruct), 2)), ("TargetRectangle", SimTypeRef("RECT", SimStruct)),)), name="D3D12_VIDEO_PROCESS_OUTPUT_STREAM_ARGUMENTS", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_OUTPUT_STREAM_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")), ("AlphaFillMode", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_ALPHA_FILL_MODE")), ("AlphaFillModeSourceStreamIndex", SimTypeInt(signed=False, label="UInt32")), ("BackgroundColor", SimTypeArray(SimTypeFloat(size=32), 4)), ("FrameRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("EnableStereo", SimTypeInt(signed=True, label="Int32")),)), name="D3D12_VIDEO_PROCESS_OUTPUT_STREAM_DESC", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_REFERENCE_SET": SimStruct(OrderedDict((("NumPastFrames", SimTypeInt(signed=False, label="UInt32")), ("ppPastFrames", SimTypePointer(SimTypeBottom(label="ID3D12Resource"), offset=0)), ("pPastSubresources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("NumFutureFrames", SimTypeInt(signed=False, label="UInt32")), ("ppFutureFrames", SimTypePointer(SimTypeBottom(label="ID3D12Resource"), offset=0)), ("pFutureSubresources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3D12_VIDEO_PROCESS_REFERENCE_SET", pack=False, align=None),
    "D3D12_VIDEO_PROCESS_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_SUPPORT_FLAGS"),
    "D3D12_VIDEO_PROCESS_TRANSFORM": SimStruct(OrderedDict((("SourceRectangle", SimTypeRef("RECT", SimStruct)), ("DestinationRectangle", SimTypeRef("RECT", SimStruct)), ("Orientation", SimTypeInt(signed=False, label="D3D12_VIDEO_PROCESS_ORIENTATION")),)), name="D3D12_VIDEO_PROCESS_TRANSFORM", pack=False, align=None),
    "D3D12_VIDEO_PROTECTED_RESOURCE_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_PROTECTED_RESOURCE_SUPPORT_FLAGS"),
    "D3D12_VIDEO_SAMPLE": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeRef("D3D12_VIDEO_FORMAT", SimStruct)),)), name="D3D12_VIDEO_SAMPLE", pack=False, align=None),
    "D3D12_VIDEO_SCALE_SUPPORT": SimStruct(OrderedDict((("OutputSizeRange", SimTypeRef("D3D12_VIDEO_SIZE_RANGE", SimStruct)), ("Flags", SimTypeInt(signed=False, label="D3D12_VIDEO_SCALE_SUPPORT_FLAGS")),)), name="D3D12_VIDEO_SCALE_SUPPORT", pack=False, align=None),
    "D3D12_VIDEO_SCALE_SUPPORT_FLAGS": SimTypeInt(signed=False, label="D3D12_VIDEO_SCALE_SUPPORT_FLAGS"),
    "D3D12_VIDEO_SIZE_RANGE": SimStruct(OrderedDict((("MaxWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxHeight", SimTypeInt(signed=False, label="UInt32")), ("MinWidth", SimTypeInt(signed=False, label="UInt32")), ("MinHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIDEO_SIZE_RANGE", pack=False, align=None),
    "D3D12_VIEWPORT": SimStruct(OrderedDict((("TopLeftX", SimTypeFloat(size=32)), ("TopLeftY", SimTypeFloat(size=32)), ("Width", SimTypeFloat(size=32)), ("Height", SimTypeFloat(size=32)), ("MinDepth", SimTypeFloat(size=32)), ("MaxDepth", SimTypeFloat(size=32)),)), name="D3D12_VIEWPORT", pack=False, align=None),
    "D3D12_VIEW_INSTANCE_LOCATION": SimStruct(OrderedDict((("ViewportArrayIndex", SimTypeInt(signed=False, label="UInt32")), ("RenderTargetArrayIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_VIEW_INSTANCE_LOCATION", pack=False, align=None),
    "D3D12_VIEW_INSTANCING_DESC": SimStruct(OrderedDict((("ViewInstanceCount", SimTypeInt(signed=False, label="UInt32")), ("pViewInstanceLocations", SimTypePointer(SimTypeRef("D3D12_VIEW_INSTANCE_LOCATION", SimStruct), label="LPArray", offset=0)), ("Flags", SimTypeInt(signed=False, label="D3D12_VIEW_INSTANCING_FLAGS")),)), name="D3D12_VIEW_INSTANCING_DESC", pack=False, align=None),
    "D3D12_VIEW_INSTANCING_FLAGS": SimTypeInt(signed=False, label="D3D12_VIEW_INSTANCING_FLAGS"),
    "D3D12_VIEW_INSTANCING_TIER": SimTypeInt(signed=False, label="D3D12_VIEW_INSTANCING_TIER"),
    "D3D12_WAVE_MMA_TIER": SimTypeInt(signed=False, label="D3D12_WAVE_MMA_TIER"),
    "D3D12_WRITEBUFFERIMMEDIATE_MODE": SimTypeInt(signed=False, label="D3D12_WRITEBUFFERIMMEDIATE_MODE"),
    "D3D12_WRITEBUFFERIMMEDIATE_PARAMETER": SimStruct(OrderedDict((("Dest", SimTypeLongLong(signed=False, label="UInt64")), ("Value", SimTypeInt(signed=False, label="UInt32")),)), name="D3D12_WRITEBUFFERIMMEDIATE_PARAMETER", pack=False, align=None),
    "D3D9ON12_ARGS": SimStruct(OrderedDict((("Enable9On12", SimTypeInt(signed=True, label="Int32")), ("pD3D12Device", SimTypeBottom(label="IUnknown")), ("ppD3D12Queues", SimTypeArray(SimTypeBottom(label="IUnknown"), 2)), ("NumQueues", SimTypeInt(signed=False, label="UInt32")), ("NodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="D3D9ON12_ARGS", pack=False, align=None),
    "D3DADAPTER_IDENTIFIER9": SimStruct(OrderedDict((("Driver", SimTypeArray(SimTypeChar(label="SByte"), 512)), ("Description", SimTypeArray(SimTypeChar(label="SByte"), 512)), ("DeviceName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("DriverVersion", SimTypeLongLong(signed=True, label="Int64")), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("SubSysId", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("DeviceIdentifier", SimTypeBottom(label="Guid")), ("WHQLLevel", SimTypeInt(signed=False, label="UInt32")),)), name="D3DADAPTER_IDENTIFIER9", pack=False, align=None),
    "D3DAES_CTR_IV": SimStruct(OrderedDict((("IV", SimTypeLongLong(signed=False, label="UInt64")), ("Count", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DAES_CTR_IV", pack=False, align=None),
    "D3DANTIALIASMODE": SimTypeInt(signed=False, label="D3DANTIALIASMODE"),
    "D3DAUTHENTICATEDCHANNELTYPE": SimTypeInt(signed=False, label="D3DAUTHENTICATEDCHANNELTYPE"),
    "D3DAUTHENTICATEDCHANNEL_CONFIGURECRYPTOSESSION": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT", SimStruct)), ("DXVA2DecodeHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGURECRYPTOSESSION", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_CONFIGUREINITIALIZE": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT", SimStruct)), ("StartSequenceQuery", SimTypeInt(signed=False, label="UInt32")), ("StartSequenceConfigure", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGUREINITIALIZE", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_CONFIGUREPROTECTION": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT", SimStruct)), ("Protections", SimTypeRef("D3DAUTHENTICATEDCHANNEL_PROTECTION_FLAGS", SimStruct)),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGUREPROTECTION", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_CONFIGURESHAREDRESOURCE": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT", SimStruct)), ("ProcessIdentiferType", SimTypeInt(signed=False, label="D3DAUTHENTICATEDCHANNEL_PROCESSIDENTIFIERTYPE")), ("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AllowAccess", SimTypeInt(signed=True, label="Int32")),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGURESHAREDRESOURCE", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_CONFIGUREUNCOMPRESSEDENCRYPTION": SimStruct(OrderedDict((("Parameters", SimTypeRef("D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT", SimStruct)), ("EncryptionGuid", SimTypeBottom(label="Guid")),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGUREUNCOMPRESSEDENCRYPTION", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT": SimStruct(OrderedDict((("omac", SimTypeRef("D3D_OMAC", SimStruct)), ("ConfigureType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGURE_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_CONFIGURE_OUTPUT": SimStruct(OrderedDict((("omac", SimTypeRef("D3D_OMAC", SimStruct)), ("ConfigureType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("ReturnCode", SimTypeInt(signed=True, label="Int32")),)), name="D3DAUTHENTICATEDCHANNEL_CONFIGURE_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_PROCESSIDENTIFIERTYPE": SimTypeInt(signed=False, label="D3DAUTHENTICATEDCHANNEL_PROCESSIDENTIFIERTYPE"),
    "D3DAUTHENTICATEDCHANNEL_PROTECTION_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DAUTHENTICATEDCHANNEL_PROTECTION_FLAGS", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYCHANNELTYPE_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("ChannelType", SimTypeInt(signed=False, label="D3DAUTHENTICATEDCHANNELTYPE")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYCHANNELTYPE_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYCRYPTOSESSION_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_INPUT", SimStruct)), ("DXVA2DecodeHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DAUTHENTICATEDCHANNEL_QUERYCRYPTOSESSION_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYCRYPTOSESSION_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("DXVA2DecodeHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DAUTHENTICATEDCHANNEL_QUERYCRYPTOSESSION_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYDEVICEHANDLE_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DAUTHENTICATEDCHANNEL_QUERYDEVICEHANDLE_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYEVICTIONENCRYPTIONGUIDCOUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("NumEncryptionGuids", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYEVICTIONENCRYPTIONGUIDCOUNT_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYEVICTIONENCRYPTIONGUID_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_INPUT", SimStruct)), ("EncryptionGuidIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYEVICTIONENCRYPTIONGUID_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYEVICTIONENCRYPTIONGUID_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("EncryptionGuidIndex", SimTypeInt(signed=False, label="UInt32")), ("EncryptionGuid", SimTypeBottom(label="Guid")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYEVICTIONENCRYPTIONGUID_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYINFOBUSTYPE_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("BusType", SimTypeInt(signed=False, label="D3DBUSTYPE")), ("bAccessibleInContiguousBlocks", SimTypeInt(signed=True, label="Int32")), ("bAccessibleInNonContiguousBlocks", SimTypeInt(signed=True, label="Int32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYINFOBUSTYPE_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTIDCOUNT_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_INPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTIDCOUNT_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTIDCOUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NumOutputIDs", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTIDCOUNT_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTID_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_INPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("OutputIDIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTID_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTID_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("DeviceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CryptoSessionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("OutputIDIndex", SimTypeInt(signed=False, label="UInt32")), ("OutputID", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYOUTPUTID_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYPROTECTION_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("ProtectionFlags", SimTypeRef("D3DAUTHENTICATEDCHANNEL_PROTECTION_FLAGS", SimStruct)),)), name="D3DAUTHENTICATEDCHANNEL_QUERYPROTECTION_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYRESTRICTEDSHAREDRESOURCEPROCESSCOUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("NumRestrictedSharedResourceProcesses", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYRESTRICTEDSHAREDRESOURCEPROCESSCOUNT_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYRESTRICTEDSHAREDRESOURCEPROCESS_INPUT": SimStruct(OrderedDict((("Input", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_INPUT", SimStruct)), ("ProcessIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYRESTRICTEDSHAREDRESOURCEPROCESS_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYRESTRICTEDSHAREDRESOURCEPROCESS_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("ProcessIndex", SimTypeInt(signed=False, label="UInt32")), ("ProcessIdentifer", SimTypeInt(signed=False, label="D3DAUTHENTICATEDCHANNEL_PROCESSIDENTIFIERTYPE")), ("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DAUTHENTICATEDCHANNEL_QUERYRESTRICTEDSHAREDRESOURCEPROCESS_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYUNCOMPRESSEDENCRYPTIONLEVEL_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("EncryptionGuid", SimTypeBottom(label="Guid")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYUNCOMPRESSEDENCRYPTIONLEVEL_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERYUNRESTRICTEDPROTECTEDSHAREDRESOURCECOUNT_OUTPUT": SimStruct(OrderedDict((("Output", SimTypeRef("D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", SimStruct)), ("NumUnrestrictedProtectedSharedResources", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERYUNRESTRICTEDPROTECTEDSHAREDRESOURCECOUNT_OUTPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERY_INPUT": SimStruct(OrderedDict((("QueryType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERY_INPUT", pack=False, align=None),
    "D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT": SimStruct(OrderedDict((("omac", SimTypeRef("D3D_OMAC", SimStruct)), ("QueryType", SimTypeBottom(label="Guid")), ("hChannel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("ReturnCode", SimTypeInt(signed=True, label="Int32")),)), name="D3DAUTHENTICATEDCHANNEL_QUERY_OUTPUT", pack=False, align=None),
    "D3DBACKBUFFER_TYPE": SimTypeInt(signed=False, label="D3DBACKBUFFER_TYPE"),
    "D3DBASISTYPE": SimTypeInt(signed=False, label="D3DBASISTYPE"),
    "D3DBLEND": SimTypeInt(signed=False, label="D3DBLEND"),
    "D3DBLENDOP": SimTypeInt(signed=False, label="D3DBLENDOP"),
    "D3DBOX": SimStruct(OrderedDict((("Left", SimTypeInt(signed=False, label="UInt32")), ("Top", SimTypeInt(signed=False, label="UInt32")), ("Right", SimTypeInt(signed=False, label="UInt32")), ("Bottom", SimTypeInt(signed=False, label="UInt32")), ("Front", SimTypeInt(signed=False, label="UInt32")), ("Back", SimTypeInt(signed=False, label="UInt32")),)), name="D3DBOX", pack=False, align=None),
    "D3DBRANCH": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwValue", SimTypeInt(signed=False, label="UInt32")), ("bNegate", SimTypeInt(signed=True, label="Int32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3DBRANCH", pack=False, align=None),
    "D3DBUSTYPE": SimTypeInt(signed=False, label="D3DBUSTYPE"),
    "D3DCAPS8": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="D3DDEVTYPE")), ("AdapterOrdinal", SimTypeInt(signed=False, label="UInt32")), ("Caps", SimTypeInt(signed=False, label="UInt32")), ("Caps2", SimTypeInt(signed=False, label="UInt32")), ("Caps3", SimTypeInt(signed=False, label="UInt32")), ("PresentationIntervals", SimTypeInt(signed=False, label="UInt32")), ("CursorCaps", SimTypeInt(signed=False, label="UInt32")), ("DevCaps", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveMiscCaps", SimTypeInt(signed=False, label="UInt32")), ("RasterCaps", SimTypeInt(signed=False, label="UInt32")), ("ZCmpCaps", SimTypeInt(signed=False, label="UInt32")), ("SrcBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("DestBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("AlphaCmpCaps", SimTypeInt(signed=False, label="UInt32")), ("ShadeCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("CubeTextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("VolumeTextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureAddressCaps", SimTypeInt(signed=False, label="UInt32")), ("VolumeTextureAddressCaps", SimTypeInt(signed=False, label="UInt32")), ("LineCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("MaxVolumeExtent", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexW", SimTypeFloat(size=32)), ("GuardBandLeft", SimTypeFloat(size=32)), ("GuardBandTop", SimTypeFloat(size=32)), ("GuardBandRight", SimTypeFloat(size=32)), ("GuardBandBottom", SimTypeFloat(size=32)), ("ExtentsAdjust", SimTypeFloat(size=32)), ("StencilCaps", SimTypeInt(signed=False, label="UInt32")), ("FVFCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureBlendStages", SimTypeInt(signed=False, label="UInt32")), ("MaxSimultaneousTextures", SimTypeInt(signed=False, label="UInt32")), ("VertexProcessingCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxActiveLights", SimTypeInt(signed=False, label="UInt32")), ("MaxUserClipPlanes", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexBlendMatrices", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexBlendMatrixIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxPointSize", SimTypeFloat(size=32)), ("MaxPrimitiveCount", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxStreams", SimTypeInt(signed=False, label="UInt32")), ("MaxStreamStride", SimTypeInt(signed=False, label="UInt32")), ("VertexShaderVersion", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexShaderConst", SimTypeInt(signed=False, label="UInt32")), ("PixelShaderVersion", SimTypeInt(signed=False, label="UInt32")), ("MaxPixelShaderValue", SimTypeFloat(size=32)),)), name="D3DCAPS8", pack=False, align=None),
    "D3DCAPS9": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="D3DDEVTYPE")), ("AdapterOrdinal", SimTypeInt(signed=False, label="UInt32")), ("Caps", SimTypeInt(signed=False, label="UInt32")), ("Caps2", SimTypeInt(signed=False, label="UInt32")), ("Caps3", SimTypeInt(signed=False, label="UInt32")), ("PresentationIntervals", SimTypeInt(signed=False, label="UInt32")), ("CursorCaps", SimTypeInt(signed=False, label="UInt32")), ("DevCaps", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveMiscCaps", SimTypeInt(signed=False, label="UInt32")), ("RasterCaps", SimTypeInt(signed=False, label="UInt32")), ("ZCmpCaps", SimTypeInt(signed=False, label="UInt32")), ("SrcBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("DestBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("AlphaCmpCaps", SimTypeInt(signed=False, label="UInt32")), ("ShadeCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("CubeTextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("VolumeTextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureAddressCaps", SimTypeInt(signed=False, label="UInt32")), ("VolumeTextureAddressCaps", SimTypeInt(signed=False, label="UInt32")), ("LineCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("MaxVolumeExtent", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("MaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexW", SimTypeFloat(size=32)), ("GuardBandLeft", SimTypeFloat(size=32)), ("GuardBandTop", SimTypeFloat(size=32)), ("GuardBandRight", SimTypeFloat(size=32)), ("GuardBandBottom", SimTypeFloat(size=32)), ("ExtentsAdjust", SimTypeFloat(size=32)), ("StencilCaps", SimTypeInt(signed=False, label="UInt32")), ("FVFCaps", SimTypeInt(signed=False, label="UInt32")), ("TextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxTextureBlendStages", SimTypeInt(signed=False, label="UInt32")), ("MaxSimultaneousTextures", SimTypeInt(signed=False, label="UInt32")), ("VertexProcessingCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxActiveLights", SimTypeInt(signed=False, label="UInt32")), ("MaxUserClipPlanes", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexBlendMatrices", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexBlendMatrixIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxPointSize", SimTypeFloat(size=32)), ("MaxPrimitiveCount", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxStreams", SimTypeInt(signed=False, label="UInt32")), ("MaxStreamStride", SimTypeInt(signed=False, label="UInt32")), ("VertexShaderVersion", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexShaderConst", SimTypeInt(signed=False, label="UInt32")), ("PixelShaderVersion", SimTypeInt(signed=False, label="UInt32")), ("PixelShader1xMaxValue", SimTypeFloat(size=32)), ("DevCaps2", SimTypeInt(signed=False, label="UInt32")), ("MaxNpatchTessellationLevel", SimTypeFloat(size=32)), ("Reserved5", SimTypeInt(signed=False, label="UInt32")), ("MasterAdapterOrdinal", SimTypeInt(signed=False, label="UInt32")), ("AdapterOrdinalInGroup", SimTypeInt(signed=False, label="UInt32")), ("NumberOfAdaptersInGroup", SimTypeInt(signed=False, label="UInt32")), ("DeclTypes", SimTypeInt(signed=False, label="UInt32")), ("NumSimultaneousRTs", SimTypeInt(signed=False, label="UInt32")), ("StretchRectFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("VS20Caps", SimTypeRef("D3DVSHADERCAPS2_0", SimStruct)), ("PS20Caps", SimTypeRef("D3DPSHADERCAPS2_0", SimStruct)), ("VertexTextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("MaxVShaderInstructionsExecuted", SimTypeInt(signed=False, label="UInt32")), ("MaxPShaderInstructionsExecuted", SimTypeInt(signed=False, label="UInt32")), ("MaxVertexShader30InstructionSlots", SimTypeInt(signed=False, label="UInt32")), ("MaxPixelShader30InstructionSlots", SimTypeInt(signed=False, label="UInt32")),)), name="D3DCAPS9", pack=False, align=None),
    "D3DCLIPSTATUS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("minx", SimTypeFloat(size=32)), ("maxx", SimTypeFloat(size=32)), ("miny", SimTypeFloat(size=32)), ("maxy", SimTypeFloat(size=32)), ("minz", SimTypeFloat(size=32)), ("maxz", SimTypeFloat(size=32)),)), name="D3DCLIPSTATUS", pack=False, align=None),
    "D3DCLIPSTATUS9": SimStruct(OrderedDict((("ClipUnion", SimTypeInt(signed=False, label="UInt32")), ("ClipIntersection", SimTypeInt(signed=False, label="UInt32")),)), name="D3DCLIPSTATUS9", pack=False, align=None),
    "D3DCMPFUNC": SimTypeInt(signed=False, label="D3DCMPFUNC"),
    "D3DCOLORVALUE": SimStruct(OrderedDict((("r", SimTypeFloat(size=32)), ("g", SimTypeFloat(size=32)), ("b", SimTypeFloat(size=32)), ("a", SimTypeFloat(size=32)),)), name="D3DCOLORVALUE", pack=False, align=None),
    "D3DCOMPILER_STRIP_FLAGS": SimTypeInt(signed=False, label="D3DCOMPILER_STRIP_FLAGS"),
    "D3DCOMPOSERECTDESC": SimStruct(OrderedDict((("X", SimTypeShort(signed=False, label="UInt16")), ("Y", SimTypeShort(signed=False, label="UInt16")), ("Width", SimTypeShort(signed=False, label="UInt16")), ("Height", SimTypeShort(signed=False, label="UInt16")),)), name="D3DCOMPOSERECTDESC", pack=False, align=None),
    "D3DCOMPOSERECTDESTINATION": SimStruct(OrderedDict((("SrcRectIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("X", SimTypeShort(signed=True, label="Int16")), ("Y", SimTypeShort(signed=True, label="Int16")),)), name="D3DCOMPOSERECTDESTINATION", pack=False, align=None),
    "D3DCOMPOSERECTSOP": SimTypeInt(signed=False, label="D3DCOMPOSERECTSOP"),
    "D3DCONTENTPROTECTIONCAPS": SimStruct(OrderedDict((("Caps", SimTypeInt(signed=False, label="UInt32")), ("KeyExchangeType", SimTypeBottom(label="Guid")), ("BufferAlignmentStart", SimTypeInt(signed=False, label="UInt32")), ("BlockAlignmentSize", SimTypeInt(signed=False, label="UInt32")), ("ProtectedMemorySize", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DCONTENTPROTECTIONCAPS", pack=False, align=None),
    "D3DCUBEMAP_FACES": SimTypeInt(signed=False, label="D3DCUBEMAP_FACES"),
    "D3DCULL": SimTypeInt(signed=False, label="D3DCULL"),
    "D3DDDIARG_CREATERESOURCE": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("Pool", SimTypeInt(signed=False, label="D3DDDI_POOL")), ("MultisampleType", SimTypeInt(signed=False, label="D3DDDIMULTISAMPLE_TYPE")), ("MultisampleQuality", SimTypeInt(signed=False, label="UInt32")), ("pSurfList", SimTypePointer(SimTypeRef("D3DDDI_SURFACEINFO", SimStruct), offset=0)), ("SurfCount", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("Fvf", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeRef("D3DDDI_RESOURCEFLAGS", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")),)), name="D3DDDIARG_CREATERESOURCE", pack=False, align=None),
    "D3DDDIARG_CREATERESOURCE2": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("Pool", SimTypeInt(signed=False, label="D3DDDI_POOL")), ("MultisampleType", SimTypeInt(signed=False, label="D3DDDIMULTISAMPLE_TYPE")), ("MultisampleQuality", SimTypeInt(signed=False, label="UInt32")), ("pSurfList", SimTypePointer(SimTypeRef("D3DDDI_SURFACEINFO", SimStruct), offset=0)), ("SurfCount", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("Fvf", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeRef("D3DDDI_RESOURCEFLAGS", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("Flags2", SimTypeRef("D3DDDI_RESOURCEFLAGS2", SimStruct)),)), name="D3DDDIARG_CREATERESOURCE2", pack=False, align=None),
    "D3DDDICB_DESTROYALLOCATION2FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDICB_DESTROYALLOCATION2FLAGS", pack=False, align=None),
    "D3DDDICB_LOCK2FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDICB_LOCK2FLAGS", pack=False, align=None),
    "D3DDDICB_LOCKFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDICB_LOCKFLAGS", pack=False, align=None),
    "D3DDDICB_SIGNALFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDICB_SIGNALFLAGS", pack=False, align=None),
    "D3DDDIFORMAT": SimTypeInt(signed=False, label="D3DDDIFORMAT"),
    "D3DDDIGPUVIRTUALADDRESS_PROTECTION_TYPE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="D3DDDIGPUVIRTUALADDRESS_PROTECTION_TYPE", pack=False, align=None),
    "D3DDDIGPUVIRTUALADDRESS_RESERVATION_TYPE": SimTypeInt(signed=False, label="D3DDDIGPUVIRTUALADDRESS_RESERVATION_TYPE"),
    "D3DDDIMULTISAMPLE_TYPE": SimTypeInt(signed=False, label="D3DDDIMULTISAMPLE_TYPE"),
    "D3DDDIRECT": SimStruct(OrderedDict((("left", SimTypeInt(signed=True, label="Int32")), ("top", SimTypeInt(signed=True, label="Int32")), ("right", SimTypeInt(signed=True, label="Int32")), ("bottom", SimTypeInt(signed=True, label="Int32")),)), name="D3DDDIRECT", pack=False, align=None),
    "D3DDDI_ALLOCATIONINFO": SimStruct(OrderedDict((("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("pSystemMem", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_ALLOCATIONINFO", pack=False, align=None),
    "D3DDDI_ALLOCATIONINFO2": SimStruct(OrderedDict((("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"hSection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pSystemMem": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("GpuVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous2", SimUnion({"Priority": SimTypeInt(signed=False, label="UInt32"), "Unused": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 5)),)), name="D3DDDI_ALLOCATIONINFO2", pack=False, align=None),
    "D3DDDI_ALLOCATIONLIST": SimStruct(OrderedDict((("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_ALLOCATIONLIST", pack=False, align=None),
    "D3DDDI_COLOR_SPACE_TYPE": SimTypeInt(signed=False, label="D3DDDI_COLOR_SPACE_TYPE"),
    "D3DDDI_CREATECONTEXTFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_CREATECONTEXTFLAGS", pack=False, align=None),
    "D3DDDI_CREATEHWCONTEXTFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_CREATEHWCONTEXTFLAGS", pack=False, align=None),
    "D3DDDI_CREATEHWQUEUEFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_CREATEHWQUEUEFLAGS", pack=False, align=None),
    "D3DDDI_CREATENATIVEFENCEINFO": SimStruct(OrderedDict((("InitialFenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS", SimStruct)), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("NativeFenceMapping", SimTypeRef("D3DDDI_NATIVEFENCEMAPPING", SimStruct)),)), name="D3DDDI_CREATENATIVEFENCEINFO", pack=False, align=None),
    "D3DDDI_DESTROYPAGINGQUEUE": SimStruct(OrderedDict((("hPagingQueue", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_DESTROYPAGINGQUEUE", pack=False, align=None),
    "D3DDDI_DOORBELLSTATUS": SimTypeInt(signed=False, label="D3DDDI_DOORBELLSTATUS"),
    "D3DDDI_DRIVERESCAPETYPE": SimTypeInt(signed=False, label="D3DDDI_DRIVERESCAPETYPE"),
    "D3DDDI_DRIVERESCAPE_CPUEVENTUSAGE": SimStruct(OrderedDict((("EscapeType", SimTypeInt(signed=False, label="D3DDDI_DRIVERESCAPETYPE")), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("hKmdCpuEvent", SimTypeLongLong(signed=False, label="UInt64")), ("Usage", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="D3DDDI_DRIVERESCAPE_CPUEVENTUSAGE", pack=False, align=None),
    "D3DDDI_DRIVERESCAPE_TRANSLATEALLOCATIONEHANDLE": SimStruct(OrderedDict((("EscapeType", SimTypeInt(signed=False, label="D3DDDI_DRIVERESCAPETYPE")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_DRIVERESCAPE_TRANSLATEALLOCATIONEHANDLE", pack=False, align=None),
    "D3DDDI_DRIVERESCAPE_TRANSLATERESOURCEHANDLE": SimStruct(OrderedDict((("EscapeType", SimTypeInt(signed=False, label="D3DDDI_DRIVERESCAPETYPE")), ("hResource", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_DRIVERESCAPE_TRANSLATERESOURCEHANDLE", pack=False, align=None),
    "D3DDDI_DXGI_RGB": SimStruct(OrderedDict((("Red", SimTypeFloat(size=32)), ("Green", SimTypeFloat(size=32)), ("Blue", SimTypeFloat(size=32)),)), name="D3DDDI_DXGI_RGB", pack=False, align=None),
    "D3DDDI_ESCAPEFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_ESCAPEFLAGS", pack=False, align=None),
    "D3DDDI_EVICT_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_EVICT_FLAGS", pack=False, align=None),
    "D3DDDI_FLIPINTERVAL_TYPE": SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE"),
    "D3DDDI_GAMMARAMP_TYPE": SimTypeInt(signed=False, label="D3DDDI_GAMMARAMP_TYPE"),
    "D3DDDI_GAMMA_RAMP_DXGI_1": SimStruct(OrderedDict((("Scale", SimTypeRef("D3DDDI_DXGI_RGB", SimStruct)), ("Offset", SimTypeRef("D3DDDI_DXGI_RGB", SimStruct)), ("GammaCurve", SimTypeArray(SimTypeRef("D3DDDI_DXGI_RGB", SimStruct), 1025)),)), name="D3DDDI_GAMMA_RAMP_DXGI_1", pack=False, align=None),
    "D3DDDI_GAMMA_RAMP_RGB256x3x16": SimStruct(OrderedDict((("Red", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Green", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Blue", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="D3DDDI_GAMMA_RAMP_RGB256x3x16", pack=False, align=None),
    "D3DDDI_GETRESOURCEPRESENTPRIVATEDRIVERDATA": SimStruct(OrderedDict((("hResource", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DDDI_GETRESOURCEPRESENTPRIVATEDRIVERDATA", pack=False, align=None),
    "D3DDDI_HDR_METADATA_HDR10": SimStruct(OrderedDict((("RedPrimary", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("GreenPrimary", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("BluePrimary", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("WhitePoint", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("MaxMasteringLuminance", SimTypeInt(signed=False, label="UInt32")), ("MinMasteringLuminance", SimTypeInt(signed=False, label="UInt32")), ("MaxContentLightLevel", SimTypeShort(signed=False, label="UInt16")), ("MaxFrameAverageLightLevel", SimTypeShort(signed=False, label="UInt16")),)), name="D3DDDI_HDR_METADATA_HDR10", pack=False, align=None),
    "D3DDDI_HDR_METADATA_HDR10PLUS": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeChar(label="Byte"), 72)),)), name="D3DDDI_HDR_METADATA_HDR10PLUS", pack=False, align=None),
    "D3DDDI_HDR_METADATA_TYPE": SimTypeInt(signed=False, label="D3DDDI_HDR_METADATA_TYPE"),
    "D3DDDI_KERNELOVERLAYINFO": SimStruct(OrderedDict((("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("DstRect", SimTypeRef("D3DDDIRECT", SimStruct)), ("SrcRect", SimTypeRef("D3DDDIRECT", SimStruct)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_KERNELOVERLAYINFO", pack=False, align=None),
    "D3DDDI_MAKERESIDENT": SimStruct(OrderedDict((("hPagingQueue", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("AllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("PriorityList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Flags", SimTypeRef("D3DDDI_MAKERESIDENT_FLAGS", SimStruct)), ("PagingFenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("NumBytesToTrim", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DDDI_MAKERESIDENT", pack=False, align=None),
    "D3DDDI_MAKERESIDENT_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_MAKERESIDENT_FLAGS", pack=False, align=None),
    "D3DDDI_MAPGPUVIRTUALADDRESS": SimStruct(OrderedDict((("hPagingQueue", SimTypeInt(signed=False, label="UInt32")), ("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MinimumAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumAddress", SimTypeLongLong(signed=False, label="UInt64")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("OffsetInPages", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInPages", SimTypeLongLong(signed=False, label="UInt64")), ("Protection", SimTypeRef("D3DDDIGPUVIRTUALADDRESS_PROTECTION_TYPE", SimStruct)), ("DriverProtection", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PagingFenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DDDI_MAPGPUVIRTUALADDRESS", pack=False, align=None),
    "D3DDDI_MULTISAMPLINGMETHOD": SimStruct(OrderedDict((("NumSamples", SimTypeInt(signed=False, label="UInt32")), ("NumQualityLevels", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_MULTISAMPLINGMETHOD", pack=False, align=None),
    "D3DDDI_NATIVEFENCEMAPPING": SimStruct(OrderedDict((("CurrentValueCpuVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CurrentValueGpuVa", SimTypeLongLong(signed=False, label="UInt64")), ("MonitoredValueGpuVa", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DDDI_NATIVEFENCEMAPPING", pack=False, align=None),
    "D3DDDI_OFFER_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_OFFER_FLAGS", pack=False, align=None),
    "D3DDDI_OFFER_PRIORITY": SimTypeInt(signed=False, label="D3DDDI_OFFER_PRIORITY"),
    "D3DDDI_OPENALLOCATIONINFO": SimStruct(OrderedDict((("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_OPENALLOCATIONINFO", pack=False, align=None),
    "D3DDDI_OPENALLOCATIONINFO2": SimStruct(OrderedDict((("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("GpuVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 6)),)), name="D3DDDI_OPENALLOCATIONINFO2", pack=False, align=None),
    "D3DDDI_OUTPUT_WIRE_COLOR_SPACE_TYPE": SimTypeInt(signed=False, label="D3DDDI_OUTPUT_WIRE_COLOR_SPACE_TYPE"),
    "D3DDDI_PAGINGQUEUE_PRIORITY": SimTypeInt(signed=False, label="D3DDDI_PAGINGQUEUE_PRIORITY"),
    "D3DDDI_PATCHLOCATIONLIST": SimStruct(OrderedDict((("AllocationIndex", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DriverId", SimTypeInt(signed=False, label="UInt32")), ("AllocationOffset", SimTypeInt(signed=False, label="UInt32")), ("PatchOffset", SimTypeInt(signed=False, label="UInt32")), ("SplitOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_PATCHLOCATIONLIST", pack=False, align=None),
    "D3DDDI_POOL": SimTypeInt(signed=False, label="D3DDDI_POOL"),
    "D3DDDI_QUERYREGISTRY_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_QUERYREGISTRY_FLAGS", pack=False, align=None),
    "D3DDDI_QUERYREGISTRY_INFO": SimStruct(OrderedDict((("QueryType", SimTypeInt(signed=False, label="D3DDDI_QUERYREGISTRY_TYPE")), ("QueryFlags", SimTypeRef("D3DDDI_QUERYREGISTRY_FLAGS", SimStruct)), ("ValueName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("ValueType", SimTypeInt(signed=False, label="UInt32")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("OutputValueSize", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="D3DDDI_QUERYREGISTRY_STATUS")), ("Anonymous", SimUnion({"OutputDword": SimTypeInt(signed=False, label="UInt32"), "OutputQword": SimTypeLongLong(signed=False, label="UInt64"), "OutputString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "OutputBinary": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="D3DDDI_QUERYREGISTRY_INFO", pack=False, align=None),
    "D3DDDI_QUERYREGISTRY_STATUS": SimTypeInt(signed=False, label="D3DDDI_QUERYREGISTRY_STATUS"),
    "D3DDDI_QUERYREGISTRY_TYPE": SimTypeInt(signed=False, label="D3DDDI_QUERYREGISTRY_TYPE"),
    "D3DDDI_RATIONAL": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_RATIONAL", pack=False, align=None),
    "D3DDDI_RECLAIM_RESULT": SimTypeInt(signed=False, label="D3DDDI_RECLAIM_RESULT"),
    "D3DDDI_RESERVEGPUVIRTUALADDRESS": SimStruct(OrderedDict((("Anonymous1", SimUnion({"hPagingQueue": SimTypeInt(signed=False, label="UInt32"), "hAdapter": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MinimumAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous2", SimUnion({"ReservationType": SimTypeInt(signed=False, label="D3DDDIGPUVIRTUALADDRESS_RESERVATION_TYPE"), "Reserved0": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"DriverProtection": SimTypeLongLong(signed=False, label="UInt64"), "Reserved1": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("VirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous4", SimUnion({"PagingFenceValue": SimTypeLongLong(signed=False, label="UInt64"), "Reserved2": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="D3DDDI_RESERVEGPUVIRTUALADDRESS", pack=False, align=None),
    "D3DDDI_RESOURCEFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_RESOURCEFLAGS", pack=False, align=None),
    "D3DDDI_RESOURCEFLAGS2": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_RESOURCEFLAGS2", pack=False, align=None),
    "D3DDDI_ROTATION": SimTypeInt(signed=False, label="D3DDDI_ROTATION"),
    "D3DDDI_SCANLINEORDERING": SimTypeInt(signed=False, label="D3DDDI_SCANLINEORDERING"),
    "D3DDDI_SEGMENTPREFERENCE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_SEGMENTPREFERENCE", pack=False, align=None),
    "D3DDDI_SURFACEINFO": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")), ("pSysMem", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SysMemPitch", SimTypeInt(signed=False, label="UInt32")), ("SysMemSlicePitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_SURFACEINFO", pack=False, align=None),
    "D3DDDI_SYNCHRONIZATIONOBJECTINFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DDDI_SYNCHRONIZATIONOBJECT_TYPE")), ("Anonymous", SimUnion({"SynchronizationMutex": SimTypeRef("_SynchronizationMutex_e__Struct", SimStruct), "Semaphore": SimTypeRef("_Semaphore_e__Struct", SimStruct), "Reserved": SimTypeRef("_Reserved_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="D3DDDI_SYNCHRONIZATIONOBJECTINFO", pack=False, align=None),
    "D3DDDI_SYNCHRONIZATIONOBJECTINFO2": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DDDI_SYNCHRONIZATIONOBJECT_TYPE")), ("Flags", SimTypeRef("D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS", SimStruct)), ("Anonymous", SimUnion({"SynchronizationMutex": SimTypeRef("_SynchronizationMutex_e__Struct", SimStruct), "Semaphore": SimTypeRef("_Semaphore_e__Struct", SimStruct), "Fence": SimTypeRef("_Fence_e__Struct", SimStruct), "CPUNotification": SimTypeRef("_CPUNotification_e__Struct", SimStruct), "MonitoredFence": SimTypeRef("_MonitoredFence_e__Struct", SimStruct), "PeriodicMonitoredFence": SimTypeRef("_PeriodicMonitoredFence_e__Struct", SimStruct), "Reserved": SimTypeRef("_Reserved_e__Struct", SimStruct)}, name="<anon>", label="None")), ("SharedHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDDI_SYNCHRONIZATIONOBJECTINFO2", pack=False, align=None),
    "D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS", pack=False, align=None),
    "D3DDDI_SYNCHRONIZATIONOBJECT_TYPE": SimTypeInt(signed=False, label="D3DDDI_SYNCHRONIZATIONOBJECT_TYPE"),
    "D3DDDI_TRIMRESIDENCYSET_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_TRIMRESIDENCYSET_FLAGS", pack=False, align=None),
    "D3DDDI_UPDATEALLOCPROPERTY": SimStruct(OrderedDict((("hPagingQueue", SimTypeInt(signed=False, label="UInt32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("SupportedSegmentSet", SimTypeInt(signed=False, label="UInt32")), ("PreferredSegment", SimTypeRef("D3DDDI_SEGMENTPREFERENCE", SimStruct)), ("Flags", SimTypeRef("D3DDDI_UPDATEALLOCPROPERTY_FLAGS", SimStruct)), ("PagingFenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "PropertyMaskValue": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_UPDATEALLOCPROPERTY", pack=False, align=None),
    "D3DDDI_UPDATEALLOCPROPERTY_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_UPDATEALLOCPROPERTY_FLAGS", pack=False, align=None),
    "D3DDDI_UPDATEGPUVIRTUALADDRESS_OPERATION": SimStruct(OrderedDict((("OperationType", SimTypeInt(signed=False, label="D3DDDI_UPDATEGPUVIRTUALADDRESS_OPERATION_TYPE")), ("Anonymous", SimUnion({"Map": SimTypeRef("_Map_e__Struct", SimStruct), "MapProtect": SimTypeRef("_MapProtect_e__Struct", SimStruct), "Unmap": SimTypeRef("_Unmap_e__Struct", SimStruct), "Copy": SimTypeRef("_Copy_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="D3DDDI_UPDATEGPUVIRTUALADDRESS_OPERATION", pack=False, align=None),
    "D3DDDI_UPDATEGPUVIRTUALADDRESS_OPERATION_TYPE": SimTypeInt(signed=False, label="D3DDDI_UPDATEGPUVIRTUALADDRESS_OPERATION_TYPE"),
    "D3DDDI_VIDEO_SIGNAL_SCANLINE_ORDERING": SimTypeInt(signed=False, label="D3DDDI_VIDEO_SIGNAL_SCANLINE_ORDERING"),
    "D3DDDI_WAITFORSYNCHRONIZATIONOBJECTFROMCPU_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DDDI_WAITFORSYNCHRONIZATIONOBJECTFROMCPU_FLAGS", pack=False, align=None),
    "D3DDEBUGMONITORTOKENS": SimTypeInt(signed=False, label="D3DDEBUGMONITORTOKENS"),
    "D3DDECLMETHOD": SimTypeInt(signed=False, label="D3DDECLMETHOD"),
    "D3DDECLTYPE": SimTypeInt(signed=False, label="D3DDECLTYPE"),
    "D3DDECLUSAGE": SimTypeInt(signed=False, label="D3DDECLUSAGE"),
    "D3DDEGREETYPE": SimTypeInt(signed=False, label="D3DDEGREETYPE"),
    "D3DDEVICEDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")),)), name="D3DDEVICEDESC", pack=False, align=None),
    "D3DDEVICEDESC7": SimStruct(OrderedDict((("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")), ("dwMaxActiveLights", SimTypeInt(signed=False, label="UInt32")), ("dvMaxVertexW", SimTypeFloat(size=32)), ("deviceGUID", SimTypeBottom(label="Guid")), ("wMaxUserClipPlanes", SimTypeShort(signed=False, label="UInt16")), ("wMaxVertexBlendMatrices", SimTypeShort(signed=False, label="UInt16")), ("dwVertexProcessingCaps", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDEVICEDESC7", pack=False, align=None),
    "D3DDEVICEDESC_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDEVICEDESC_V1", pack=False, align=None),
    "D3DDEVICEDESC_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDEVICEDESC_V2", pack=False, align=None),
    "D3DDEVICEDESC_V3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")),)), name="D3DDEVICEDESC_V3", pack=False, align=None),
    "D3DDEVICE_CREATION_PARAMETERS": SimStruct(OrderedDict((("AdapterOrdinal", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="D3DDEVTYPE")), ("hFocusWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("BehaviorFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDEVICE_CREATION_PARAMETERS", pack=False, align=None),
    "D3DDEVINFO_D3D9BANDWIDTHTIMINGS": SimStruct(OrderedDict((("MaxBandwidthUtilized", SimTypeFloat(size=32)), ("FrontEndUploadMemoryUtilizedPercent", SimTypeFloat(size=32)), ("VertexRateUtilizedPercent", SimTypeFloat(size=32)), ("TriangleSetupRateUtilizedPercent", SimTypeFloat(size=32)), ("FillRateUtilizedPercent", SimTypeFloat(size=32)),)), name="D3DDEVINFO_D3D9BANDWIDTHTIMINGS", pack=False, align=None),
    "D3DDEVINFO_D3D9CACHEUTILIZATION": SimStruct(OrderedDict((("TextureCacheHitRate", SimTypeFloat(size=32)), ("PostTransformVertexCacheHitRate", SimTypeFloat(size=32)),)), name="D3DDEVINFO_D3D9CACHEUTILIZATION", pack=False, align=None),
    "D3DDEVINFO_D3D9INTERFACETIMINGS": SimStruct(OrderedDict((("WaitingForGPUToUseApplicationResourceTimePercent", SimTypeFloat(size=32)), ("WaitingForGPUToAcceptMoreCommandsTimePercent", SimTypeFloat(size=32)), ("WaitingForGPUToStayWithinLatencyTimePercent", SimTypeFloat(size=32)), ("WaitingForGPUExclusiveResourceTimePercent", SimTypeFloat(size=32)), ("WaitingForGPUOtherTimePercent", SimTypeFloat(size=32)),)), name="D3DDEVINFO_D3D9INTERFACETIMINGS", pack=False, align=None),
    "D3DDEVINFO_D3D9PIPELINETIMINGS": SimStruct(OrderedDict((("VertexProcessingTimePercent", SimTypeFloat(size=32)), ("PixelProcessingTimePercent", SimTypeFloat(size=32)), ("OtherGPUProcessingTimePercent", SimTypeFloat(size=32)), ("GPUIdleTimePercent", SimTypeFloat(size=32)),)), name="D3DDEVINFO_D3D9PIPELINETIMINGS", pack=False, align=None),
    "D3DDEVINFO_D3D9STAGETIMINGS": SimStruct(OrderedDict((("MemoryProcessingPercent", SimTypeFloat(size=32)), ("ComputationProcessingPercent", SimTypeFloat(size=32)),)), name="D3DDEVINFO_D3D9STAGETIMINGS", pack=False, align=None),
    "D3DDEVINFO_D3DVERTEXSTATS": SimStruct(OrderedDict((("NumRenderedTriangles", SimTypeInt(signed=False, label="UInt32")), ("NumExtraClippingTriangles", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDEVINFO_D3DVERTEXSTATS", pack=False, align=None),
    "D3DDEVINFO_RESOURCEMANAGER": SimStruct(OrderedDict((("stats", SimTypeArray(SimTypeRef("D3DRESOURCESTATS", SimStruct), 8)),)), name="D3DDEVINFO_RESOURCEMANAGER", pack=False, align=None),
    "D3DDEVINFO_VCACHE": SimStruct(OrderedDict((("Pattern", SimTypeInt(signed=False, label="UInt32")), ("OptMethod", SimTypeInt(signed=False, label="UInt32")), ("CacheSize", SimTypeInt(signed=False, label="UInt32")), ("MagicNumber", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDEVINFO_VCACHE", pack=False, align=None),
    "D3DDEVTYPE": SimTypeInt(signed=False, label="D3DDEVTYPE"),
    "D3DDISPLAYMODE": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")),)), name="D3DDISPLAYMODE", pack=False, align=None),
    "D3DDISPLAYMODEEX": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("ScanLineOrdering", SimTypeInt(signed=False, label="D3DSCANLINEORDERING")),)), name="D3DDISPLAYMODEEX", pack=False, align=None),
    "D3DDISPLAYMODEFILTER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("ScanLineOrdering", SimTypeInt(signed=False, label="D3DSCANLINEORDERING")),)), name="D3DDISPLAYMODEFILTER", pack=False, align=None),
    "D3DDISPLAYROTATION": SimTypeInt(signed=False, label="D3DDISPLAYROTATION"),
    "D3DDP_PTRSTRIDE": SimStruct(OrderedDict((("lpvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DDP_PTRSTRIDE", pack=False, align=None),
    "D3DDRAWPRIMITIVESTRIDEDDATA": SimStruct(OrderedDict((("position", SimTypeRef("D3DDP_PTRSTRIDE", SimStruct)), ("normal", SimTypeRef("D3DDP_PTRSTRIDE", SimStruct)), ("diffuse", SimTypeRef("D3DDP_PTRSTRIDE", SimStruct)), ("specular", SimTypeRef("D3DDP_PTRSTRIDE", SimStruct)), ("textureCoords", SimTypeArray(SimTypeRef("D3DDP_PTRSTRIDE", SimStruct), 8)),)), name="D3DDRAWPRIMITIVESTRIDEDDATA", pack=False, align=None),
    "D3DENCRYPTED_BLOCK_INFO": SimStruct(OrderedDict((("NumEncryptedBytesAtBeginning", SimTypeInt(signed=False, label="UInt32")), ("NumBytesInSkipPattern", SimTypeInt(signed=False, label="UInt32")), ("NumBytesInEncryptPattern", SimTypeInt(signed=False, label="UInt32")),)), name="D3DENCRYPTED_BLOCK_INFO", pack=False, align=None),
    "D3DEXECUTEBUFFERDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DEXECUTEBUFFERDESC", pack=False, align=None),
    "D3DEXECUTEDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("dwVertexCount", SimTypeInt(signed=False, label="UInt32")), ("dwInstructionOffset", SimTypeInt(signed=False, label="UInt32")), ("dwInstructionLength", SimTypeInt(signed=False, label="UInt32")), ("dwHVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("dsStatus", SimTypeRef("D3DSTATUS", SimStruct)),)), name="D3DEXECUTEDATA", pack=False, align=None),
    "D3DFILLMODE": SimTypeInt(signed=False, label="D3DFILLMODE"),
    "D3DFINDDEVICERESULT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guid", SimTypeBottom(label="Guid")), ("ddHwDesc", SimTypeRef("D3DDEVICEDESC", SimStruct)), ("ddSwDesc", SimTypeRef("D3DDEVICEDESC", SimStruct)),)), name="D3DFINDDEVICERESULT", pack=False, align=None),
    "D3DFINDDEVICESEARCH": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bHardware", SimTypeInt(signed=True, label="Int32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("guid", SimTypeBottom(label="Guid")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dpcPrimCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)),)), name="D3DFINDDEVICESEARCH", pack=False, align=None),
    "D3DFOGMODE": SimTypeInt(signed=False, label="D3DFOGMODE"),
    "D3DFORMAT": SimTypeInt(signed=False, label="D3DFORMAT"),
    "D3DGAMMARAMP": SimStruct(OrderedDict((("red", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("green", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("blue", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="D3DGAMMARAMP", pack=False, align=None),
    "D3DGPU_PHYSICAL_ADDRESS": SimStruct(OrderedDict((("SegmentId", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeInt(signed=False, label="UInt32")), ("SegmentOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DGPU_PHYSICAL_ADDRESS", pack=False, align=None),
    "D3DHAL_CALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ContextCreate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CONTEXTCREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ContextDestroy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CONTEXTDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ContextDestroyAll", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CONTEXTDESTROYALLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SceneCapture", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_SCENECAPTUREDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpReserved10", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved11", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RenderState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_RENDERSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("RenderPrimitive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_RENDERPRIMITIVEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("TextureCreate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTURECREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("TextureDestroy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTUREDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("TextureSwap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTURESWAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("TextureGetSurf", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTUREGETSURFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpReserved12", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved13", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved14", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved15", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved16", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved17", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved18", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved19", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved20", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpReserved21", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("GetState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_GETSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwReserved0", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")), ("dwReserved5", SimTypeInt(signed=False, label="UInt32")), ("dwReserved6", SimTypeInt(signed=False, label="UInt32")), ("dwReserved7", SimTypeInt(signed=False, label="UInt32")), ("dwReserved8", SimTypeInt(signed=False, label="UInt32")), ("dwReserved9", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_CALLBACKS", pack=False, align=None),
    "D3DHAL_CALLBACKS2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("SetRenderTarget", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_SETRENDERTARGETDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Clear", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CLEARDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DrawOnePrimitive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWONEPRIMITIVEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DrawOneIndexedPrimitive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWONEINDEXEDPRIMITIVEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DrawPrimitives", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWPRIMITIVESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="D3DHAL_CALLBACKS2", pack=False, align=None),
    "D3DHAL_CALLBACKS3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Clear2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CLEAR2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValidateTextureStageState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_VALIDATETEXTURESTAGESTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DrawPrimitives2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWPRIMITIVES2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="D3DHAL_CALLBACKS3", pack=False, align=None),
    "D3DHAL_CLEAR2DATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFillColor", SimTypeInt(signed=False, label="UInt32")), ("dvFillDepth", SimTypeFloat(size=32)), ("dwFillStencil", SimTypeInt(signed=False, label="UInt32")), ("lpRects", SimTypePointer(SimTypeRef("D3DRECT", SimStruct), offset=0)), ("dwNumRects", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_CLEAR2DATA", pack=False, align=None),
    "D3DHAL_CLEARDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFillColor", SimTypeInt(signed=False, label="UInt32")), ("dwFillDepth", SimTypeInt(signed=False, label="UInt32")), ("lpRects", SimTypePointer(SimTypeRef("D3DRECT", SimStruct), offset=0)), ("dwNumRects", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_CLEARDATA", pack=False, align=None),
    "D3DHAL_CLIPPEDTRIANGLEFAN": SimStruct(OrderedDict((("FirstVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("dwEdgeFlags", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_CLIPPEDTRIANGLEFAN", pack=False, align=None),
    "D3DHAL_CONTEXTCREATEDATA": SimStruct(OrderedDict((("Anonymous1", SimUnion({"lpDDGbl": SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), "lpDDLcl": SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"lpDDS": SimTypeBottom(label="IDirectDrawSurface"), "lpDDSLcl": SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"lpDDSZ": SimTypeBottom(label="IDirectDrawSurface"), "lpDDSZLcl": SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"dwPID": SimTypeInt(signed=False, label="UInt32"), "dwrstates": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_CONTEXTCREATEDATA", pack=False, align=None),
    "D3DHAL_CONTEXTDESTROYALLDATA": SimStruct(OrderedDict((("dwPID", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_CONTEXTDESTROYALLDATA", pack=False, align=None),
    "D3DHAL_CONTEXTDESTROYDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_CONTEXTDESTROYDATA", pack=False, align=None),
    "D3DHAL_D3DDX6EXTENDEDCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_D3DDX6EXTENDEDCAPS", pack=False, align=None),
    "D3DHAL_D3DEXTENDEDCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")), ("dwMaxActiveLights", SimTypeInt(signed=False, label="UInt32")), ("dvMaxVertexW", SimTypeFloat(size=32)), ("wMaxUserClipPlanes", SimTypeShort(signed=False, label="UInt16")), ("wMaxVertexBlendMatrices", SimTypeShort(signed=False, label="UInt16")), ("dwVertexProcessingCaps", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_D3DEXTENDEDCAPS", pack=False, align=None),
    "D3DHAL_DP2ADDDIRTYBOX": SimStruct(OrderedDict((("dwSurface", SimTypeInt(signed=False, label="UInt32")), ("DirtyBox", SimTypeRef("D3DBOX", SimStruct)),)), name="D3DHAL_DP2ADDDIRTYBOX", pack=False, align=None),
    "D3DHAL_DP2ADDDIRTYRECT": SimStruct(OrderedDict((("dwSurface", SimTypeInt(signed=False, label="UInt32")), ("rDirtyArea", SimTypeRef("RECTL", SimStruct)),)), name="D3DHAL_DP2ADDDIRTYRECT", pack=False, align=None),
    "D3DHAL_DP2BLT": SimStruct(OrderedDict((("dwSource", SimTypeInt(signed=False, label="UInt32")), ("rSource", SimTypeRef("RECTL", SimStruct)), ("dwSourceMipLevel", SimTypeInt(signed=False, label="UInt32")), ("dwDest", SimTypeInt(signed=False, label="UInt32")), ("rDest", SimTypeRef("RECTL", SimStruct)), ("dwDestMipLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2BLT", pack=False, align=None),
    "D3DHAL_DP2BUFFERBLT": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDDSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("rSrc", SimTypeRef("D3DRANGE", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2BUFFERBLT", pack=False, align=None),
    "D3DHAL_DP2CLEAR": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFillColor", SimTypeInt(signed=False, label="UInt32")), ("dvFillDepth", SimTypeFloat(size=32)), ("dwFillStencil", SimTypeInt(signed=False, label="UInt32")), ("Rects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)),)), name="D3DHAL_DP2CLEAR", pack=False, align=None),
    "D3DHAL_DP2COLORFILL": SimStruct(OrderedDict((("dwSurface", SimTypeInt(signed=False, label="UInt32")), ("rRect", SimTypeRef("RECTL", SimStruct)), ("Color", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2COLORFILL", pack=False, align=None),
    "D3DHAL_DP2COMMAND": SimStruct(OrderedDict((("bCommand", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"wPrimitiveCount": SimTypeShort(signed=False, label="UInt16"), "wStateCount": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="D3DHAL_DP2COMMAND", pack=False, align=None),
    "D3DHAL_DP2COMPOSERECTS": SimStruct(OrderedDict((("SrcSurfaceHandle", SimTypeInt(signed=False, label="UInt32")), ("DstSurfaceHandle", SimTypeInt(signed=False, label="UInt32")), ("SrcRectDescsVBHandle", SimTypeInt(signed=False, label="UInt32")), ("NumRects", SimTypeInt(signed=False, label="UInt32")), ("DstRectDescsVBHandle", SimTypeInt(signed=False, label="UInt32")), ("Operation", SimTypeInt(signed=False, label="D3DCOMPOSERECTSOP")), ("XOffset", SimTypeInt(signed=True, label="Int32")), ("YOffset", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_DP2COMPOSERECTS", pack=False, align=None),
    "D3DHAL_DP2CREATELIGHT": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2CREATELIGHT", pack=False, align=None),
    "D3DHAL_DP2CREATEPIXELSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwCodeSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2CREATEPIXELSHADER", pack=False, align=None),
    "D3DHAL_DP2CREATEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")), ("QueryType", SimTypeInt(signed=False, label="D3DQUERYTYPE")),)), name="D3DHAL_DP2CREATEQUERY", pack=False, align=None),
    "D3DHAL_DP2CREATEVERTEXSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwDeclSize", SimTypeInt(signed=False, label="UInt32")), ("dwCodeSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2CREATEVERTEXSHADER", pack=False, align=None),
    "D3DHAL_DP2CREATEVERTEXSHADERDECL": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwNumVertexElements", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2CREATEVERTEXSHADERDECL", pack=False, align=None),
    "D3DHAL_DP2CREATEVERTEXSHADERFUNC": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2CREATEVERTEXSHADERFUNC", pack=False, align=None),
    "D3DHAL_DP2DELETEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DELETEQUERY", pack=False, align=None),
    "D3DHAL_DP2DRAWINDEXEDPRIMITIVE": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("BaseVertexIndex", SimTypeInt(signed=True, label="Int32")), ("MinIndex", SimTypeInt(signed=False, label="UInt32")), ("NumVertices", SimTypeInt(signed=False, label="UInt32")), ("StartIndex", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DRAWINDEXEDPRIMITIVE", pack=False, align=None),
    "D3DHAL_DP2DRAWINDEXEDPRIMITIVE2": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("BaseVertexOffset", SimTypeInt(signed=True, label="Int32")), ("MinIndex", SimTypeInt(signed=False, label="UInt32")), ("NumVertices", SimTypeInt(signed=False, label="UInt32")), ("StartIndexOffset", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DRAWINDEXEDPRIMITIVE2", pack=False, align=None),
    "D3DHAL_DP2DRAWPRIMITIVE": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("VStart", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DRAWPRIMITIVE", pack=False, align=None),
    "D3DHAL_DP2DRAWPRIMITIVE2": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("FirstVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DRAWPRIMITIVE2", pack=False, align=None),
    "D3DHAL_DP2DRAWRECTPATCH": SimStruct(OrderedDict((("Handle", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DRAWRECTPATCH", pack=False, align=None),
    "D3DHAL_DP2DRAWTRIPATCH": SimStruct(OrderedDict((("Handle", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2DRAWTRIPATCH", pack=False, align=None),
    "D3DHAL_DP2EXT": SimStruct(OrderedDict((("dwExtToken", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2EXT", pack=False, align=None),
    "D3DHAL_DP2GENERATEMIPSUBLEVELS": SimStruct(OrderedDict((("hSurface", SimTypeInt(signed=False, label="UInt32")), ("Filter", SimTypeInt(signed=False, label="D3DTEXTUREFILTERTYPE")),)), name="D3DHAL_DP2GENERATEMIPSUBLEVELS", pack=False, align=None),
    "D3DHAL_DP2INDEXEDLINELIST": SimStruct(OrderedDict((("wV1", SimTypeShort(signed=False, label="UInt16")), ("wV2", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2INDEXEDLINELIST", pack=False, align=None),
    "D3DHAL_DP2INDEXEDLINESTRIP": SimStruct(OrderedDict((("wV", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)),)), name="D3DHAL_DP2INDEXEDLINESTRIP", pack=False, align=None),
    "D3DHAL_DP2INDEXEDTRIANGLEFAN": SimStruct(OrderedDict((("wV", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)),)), name="D3DHAL_DP2INDEXEDTRIANGLEFAN", pack=False, align=None),
    "D3DHAL_DP2INDEXEDTRIANGLELIST": SimStruct(OrderedDict((("wV1", SimTypeShort(signed=False, label="UInt16")), ("wV2", SimTypeShort(signed=False, label="UInt16")), ("wV3", SimTypeShort(signed=False, label="UInt16")), ("wFlags", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2INDEXEDTRIANGLELIST", pack=False, align=None),
    "D3DHAL_DP2INDEXEDTRIANGLELIST2": SimStruct(OrderedDict((("wV1", SimTypeShort(signed=False, label="UInt16")), ("wV2", SimTypeShort(signed=False, label="UInt16")), ("wV3", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2INDEXEDTRIANGLELIST2", pack=False, align=None),
    "D3DHAL_DP2INDEXEDTRIANGLESTRIP": SimStruct(OrderedDict((("wV", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)),)), name="D3DHAL_DP2INDEXEDTRIANGLESTRIP", pack=False, align=None),
    "D3DHAL_DP2ISSUEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2ISSUEQUERY", pack=False, align=None),
    "D3DHAL_DP2LINELIST": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2LINELIST", pack=False, align=None),
    "D3DHAL_DP2LINESTRIP": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2LINESTRIP", pack=False, align=None),
    "D3DHAL_DP2MULTIPLYTRANSFORM": SimStruct(OrderedDict((("xfrmType", SimTypeInt(signed=False, label="D3DTRANSFORMSTATETYPE")), ("matrix", SimTypeRef("D3DMATRIX", SimStruct)),)), name="D3DHAL_DP2MULTIPLYTRANSFORM", pack=False, align=None),
    "D3DHAL_DP2OPERATION": SimTypeInt(signed=False, label="D3DHAL_DP2OPERATION"),
    "D3DHAL_DP2PIXELSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2PIXELSHADER", pack=False, align=None),
    "D3DHAL_DP2POINTS": SimStruct(OrderedDict((("wCount", SimTypeShort(signed=False, label="UInt16")), ("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2POINTS", pack=False, align=None),
    "D3DHAL_DP2RENDERSTATE": SimStruct(OrderedDict((("RenderState", SimTypeInt(signed=False, label="D3DRENDERSTATETYPE")), ("Anonymous", SimUnion({"dvState": SimTypeFloat(size=32), "dwState": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DHAL_DP2RENDERSTATE", pack=False, align=None),
    "D3DHAL_DP2RESPONSE": SimStruct(OrderedDict((("bCommand", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("wStateCount", SimTypeShort(signed=False, label="UInt16")), ("dwTotalSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2RESPONSE", pack=False, align=None),
    "D3DHAL_DP2RESPONSEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2RESPONSEQUERY", pack=False, align=None),
    "D3DHAL_DP2SETCLIPPLANE": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("plane", SimTypeArray(SimTypeFloat(size=32), 4)),)), name="D3DHAL_DP2SETCLIPPLANE", pack=False, align=None),
    "D3DHAL_DP2SETCONVOLUTIONKERNELMONO": SimStruct(OrderedDict((("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETCONVOLUTIONKERNELMONO", pack=False, align=None),
    "D3DHAL_DP2SETDEPTHSTENCIL": SimStruct(OrderedDict((("hZBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETDEPTHSTENCIL", pack=False, align=None),
    "D3DHAL_DP2SETINDICES": SimStruct(OrderedDict((("dwVBHandle", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETINDICES", pack=False, align=None),
    "D3DHAL_DP2SETLIGHT": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwDataType", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETLIGHT", pack=False, align=None),
    "D3DHAL_DP2SETPALETTE": SimStruct(OrderedDict((("dwPaletteHandle", SimTypeInt(signed=False, label="UInt32")), ("dwPaletteFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSurfaceHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETPALETTE", pack=False, align=None),
    "D3DHAL_DP2SETPIXELSHADERCONST": SimStruct(OrderedDict((("dwRegister", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETPIXELSHADERCONST", pack=False, align=None),
    "D3DHAL_DP2SETPRIORITY": SimStruct(OrderedDict((("dwDDSurface", SimTypeInt(signed=False, label="UInt32")), ("dwPriority", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETPRIORITY", pack=False, align=None),
    "D3DHAL_DP2SETRENDERTARGET": SimStruct(OrderedDict((("hRenderTarget", SimTypeInt(signed=False, label="UInt32")), ("hZBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETRENDERTARGET", pack=False, align=None),
    "D3DHAL_DP2SETRENDERTARGET2": SimStruct(OrderedDict((("RTIndex", SimTypeInt(signed=False, label="UInt32")), ("hRenderTarget", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETRENDERTARGET2", pack=False, align=None),
    "D3DHAL_DP2SETSTREAMSOURCE": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwVBHandle", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETSTREAMSOURCE", pack=False, align=None),
    "D3DHAL_DP2SETSTREAMSOURCE2": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwVBHandle", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETSTREAMSOURCE2", pack=False, align=None),
    "D3DHAL_DP2SETSTREAMSOURCEFREQ": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwDivider", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETSTREAMSOURCEFREQ", pack=False, align=None),
    "D3DHAL_DP2SETSTREAMSOURCEUM": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETSTREAMSOURCEUM", pack=False, align=None),
    "D3DHAL_DP2SETTEXLOD": SimStruct(OrderedDict((("dwDDSurface", SimTypeInt(signed=False, label="UInt32")), ("dwLOD", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETTEXLOD", pack=False, align=None),
    "D3DHAL_DP2SETTRANSFORM": SimStruct(OrderedDict((("xfrmType", SimTypeInt(signed=False, label="D3DTRANSFORMSTATETYPE")), ("matrix", SimTypeRef("D3DMATRIX", SimStruct)),)), name="D3DHAL_DP2SETTRANSFORM", pack=False, align=None),
    "D3DHAL_DP2SETVERTEXSHADERCONST": SimStruct(OrderedDict((("dwRegister", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SETVERTEXSHADERCONST", pack=False, align=None),
    "D3DHAL_DP2STARTVERTEX": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2STARTVERTEX", pack=False, align=None),
    "D3DHAL_DP2STATESET": SimStruct(OrderedDict((("dwOperation", SimTypeInt(signed=False, label="UInt32")), ("dwParam", SimTypeInt(signed=False, label="UInt32")), ("sbType", SimTypeInt(signed=False, label="D3DSTATEBLOCKTYPE")),)), name="D3DHAL_DP2STATESET", pack=False, align=None),
    "D3DHAL_DP2SURFACEBLT": SimStruct(OrderedDict((("dwSource", SimTypeInt(signed=False, label="UInt32")), ("rSource", SimTypeRef("RECTL", SimStruct)), ("dwSourceMipLevel", SimTypeInt(signed=False, label="UInt32")), ("dwDest", SimTypeInt(signed=False, label="UInt32")), ("rDest", SimTypeRef("RECTL", SimStruct)), ("dwDestMipLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2SURFACEBLT", pack=False, align=None),
    "D3DHAL_DP2TEXBLT": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDDSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("pDest", SimTypeRef("POINT", SimStruct)), ("rSrc", SimTypeRef("RECTL", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2TEXBLT", pack=False, align=None),
    "D3DHAL_DP2TEXTURESTAGESTATE": SimStruct(OrderedDict((("wStage", SimTypeShort(signed=False, label="UInt16")), ("TSState", SimTypeShort(signed=False, label="UInt16")), ("dwValue", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2TEXTURESTAGESTATE", pack=False, align=None),
    "D3DHAL_DP2TRIANGLEFAN": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2TRIANGLEFAN", pack=False, align=None),
    "D3DHAL_DP2TRIANGLEFAN_IMM": SimStruct(OrderedDict((("dwEdgeFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2TRIANGLEFAN_IMM", pack=False, align=None),
    "D3DHAL_DP2TRIANGLELIST": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2TRIANGLELIST", pack=False, align=None),
    "D3DHAL_DP2TRIANGLESTRIP": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2TRIANGLESTRIP", pack=False, align=None),
    "D3DHAL_DP2UPDATEPALETTE": SimStruct(OrderedDict((("dwPaletteHandle", SimTypeInt(signed=False, label="UInt32")), ("wStartIndex", SimTypeShort(signed=False, label="UInt16")), ("wNumEntries", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DP2UPDATEPALETTE", pack=False, align=None),
    "D3DHAL_DP2VERTEXSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2VERTEXSHADER", pack=False, align=None),
    "D3DHAL_DP2VIEWPORTINFO": SimStruct(OrderedDict((("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2VIEWPORTINFO", pack=False, align=None),
    "D3DHAL_DP2VOLUMEBLT": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDDSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDestX", SimTypeInt(signed=False, label="UInt32")), ("dwDestY", SimTypeInt(signed=False, label="UInt32")), ("dwDestZ", SimTypeInt(signed=False, label="UInt32")), ("srcBox", SimTypeRef("D3DBOX", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DP2VOLUMEBLT", pack=False, align=None),
    "D3DHAL_DP2WINFO": SimStruct(OrderedDict((("dvWNear", SimTypeFloat(size=32)), ("dvWFar", SimTypeFloat(size=32)),)), name="D3DHAL_DP2WINFO", pack=False, align=None),
    "D3DHAL_DP2ZRANGE": SimStruct(OrderedDict((("dvMinZ", SimTypeFloat(size=32)), ("dvMaxZ", SimTypeFloat(size=32)),)), name="D3DHAL_DP2ZRANGE", pack=False, align=None),
    "D3DHAL_DRAWONEINDEXEDPRIMITIVEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("Anonymous", SimUnion({"VertexType": SimTypeInt(signed=False, label="D3DVERTEXTYPE"), "dwFVFControl": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("lpvVertices", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwNumVertices", SimTypeInt(signed=False, label="UInt32")), ("lpwIndices", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("dwNumIndices", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_DRAWONEINDEXEDPRIMITIVEDATA", pack=False, align=None),
    "D3DHAL_DRAWONEPRIMITIVEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("Anonymous", SimUnion({"VertexType": SimTypeInt(signed=False, label="D3DVERTEXTYPE"), "dwFVFControl": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("lpvVertices", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwNumVertices", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_DRAWONEPRIMITIVEDATA", pack=False, align=None),
    "D3DHAL_DRAWPRIMCOUNTS": SimStruct(OrderedDict((("wNumStateChanges", SimTypeShort(signed=False, label="UInt16")), ("wPrimitiveType", SimTypeShort(signed=False, label="UInt16")), ("wVertexType", SimTypeShort(signed=False, label="UInt16")), ("wNumVertices", SimTypeShort(signed=False, label="UInt16")),)), name="D3DHAL_DRAWPRIMCOUNTS", pack=False, align=None),
    "D3DHAL_DRAWPRIMITIVES2DATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwVertexType", SimTypeInt(signed=False, label="UInt32")), ("lpDDCommands", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwCommandOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCommandLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lpDDVertex": SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0), "lpVertices": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("dwVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("dwVertexLength", SimTypeInt(signed=False, label="UInt32")), ("dwReqVertexBufSize", SimTypeInt(signed=False, label="UInt32")), ("dwReqCommandBufSize", SimTypeInt(signed=False, label="UInt32")), ("lpdwRStates", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Anonymous2", SimUnion({"dwVertexSize": SimTypeInt(signed=False, label="UInt32"), "ddrval": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("dwErrorOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3DHAL_DRAWPRIMITIVES2DATA", pack=False, align=None),
    "D3DHAL_DRAWPRIMITIVESDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFVFControl", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_DRAWPRIMITIVESDATA", pack=False, align=None),
    "D3DHAL_GETSTATEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwWhich", SimTypeInt(signed=False, label="UInt32")), ("ddState", SimTypeRef("D3DSTATE", SimStruct)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_GETSTATEDATA", pack=False, align=None),
    "D3DHAL_GLOBALDRIVERDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwCaps", SimTypeRef("D3DDEVICEDESC_V1", SimStruct)), ("dwNumVertices", SimTypeInt(signed=False, label="UInt32")), ("dwNumClipVertices", SimTypeInt(signed=False, label="UInt32")), ("dwNumTextureFormats", SimTypeInt(signed=False, label="UInt32")), ("lpTextureFormats", SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0)),)), name="D3DHAL_GLOBALDRIVERDATA", pack=False, align=None),
    "D3DHAL_RENDERPRIMITIVEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("lpExeBuf", SimTypeBottom(label="IDirectDrawSurface")), ("dwTLOffset", SimTypeInt(signed=False, label="UInt32")), ("lpTLBuf", SimTypeBottom(label="IDirectDrawSurface")), ("diInstruction", SimTypeRef("D3DINSTRUCTION", SimStruct)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_RENDERPRIMITIVEDATA", pack=False, align=None),
    "D3DHAL_RENDERSTATEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("lpExeBuf", SimTypeBottom(label="IDirectDrawSurface")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_RENDERSTATEDATA", pack=False, align=None),
    "D3DHAL_SCENECAPTUREDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlag", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_SCENECAPTUREDATA", pack=False, align=None),
    "D3DHAL_SETRENDERTARGETDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous1", SimUnion({"lpDDS": SimTypeBottom(label="IDirectDrawSurface"), "lpDDSLcl": SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"lpDDSZ": SimTypeBottom(label="IDirectDrawSurface"), "lpDDSZLcl": SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)}, name="<anon>", label="None")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_SETRENDERTARGETDATA", pack=False, align=None),
    "D3DHAL_TEXTURECREATEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpDDS", SimTypeBottom(label="IDirectDrawSurface")), ("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_TEXTURECREATEDATA", pack=False, align=None),
    "D3DHAL_TEXTUREDESTROYDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_TEXTUREDESTROYDATA", pack=False, align=None),
    "D3DHAL_TEXTUREGETSURFDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpDDS", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_TEXTUREGETSURFDATA", pack=False, align=None),
    "D3DHAL_TEXTURESWAPDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHandle1", SimTypeInt(signed=False, label="UInt32")), ("dwHandle2", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_TEXTURESWAPDATA", pack=False, align=None),
    "D3DHAL_VALIDATETEXTURESTAGESTATEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwNumPasses", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DHAL_VALIDATETEXTURESTAGESTATEDATA", pack=False, align=None),
    "D3DHVERTEX": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"hx": SimTypeFloat(size=32), "dvHX": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hy": SimTypeFloat(size=32), "dvHY": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"hz": SimTypeFloat(size=32), "dvHZ": SimTypeFloat(size=32)}, name="<anon>", label="None")),)), name="D3DHVERTEX", pack=False, align=None),
    "D3DINDEXBUFFER_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("Type", SimTypeInt(signed=False, label="D3DRESOURCETYPE")), ("Usage", SimTypeInt(signed=False, label="UInt32")), ("Pool", SimTypeInt(signed=False, label="D3DPOOL")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="D3DINDEXBUFFER_DESC", pack=False, align=None),
    "D3DINSTRUCTION": SimStruct(OrderedDict((("bOpcode", SimTypeChar(label="Byte")), ("bSize", SimTypeChar(label="Byte")), ("wCount", SimTypeShort(signed=False, label="UInt16")),)), name="D3DINSTRUCTION", pack=False, align=None),
    "D3DKMDT_2DREGION": SimStruct(OrderedDict((("cx", SimTypeInt(signed=False, label="UInt32")), ("cy", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_2DREGION", pack=False, align=None),
    "D3DKMDT_3x4_COLORSPACE_TRANSFORM": SimStruct(OrderedDict((("ColorMatrix3x4", SimTypeArray(SimTypeFloat(size=32), 12)), ("ScalarMultiplier", SimTypeFloat(size=32)), ("LookupTable1D", SimTypeArray(SimTypeRef("D3DDDI_DXGI_RGB", SimStruct), 4096)),)), name="D3DKMDT_3x4_COLORSPACE_TRANSFORM", pack=False, align=None),
    "D3DKMDT_COLORSPACE_TRANSFORM_MATRIX_V2": SimStruct(OrderedDict((("StageControlLookupTable1DDegamma", SimTypeInt(signed=False, label="D3DKMDT_COLORSPACE_TRANSFORM_STAGE_CONTROL")), ("LookupTable1DDegamma", SimTypeArray(SimTypeRef("D3DDDI_DXGI_RGB", SimStruct), 4096)), ("StageControlColorMatrix3x3", SimTypeInt(signed=False, label="D3DKMDT_COLORSPACE_TRANSFORM_STAGE_CONTROL")), ("ColorMatrix3x3", SimTypeArray(SimTypeFloat(size=32), 9)), ("StageControlLookupTable1DRegamma", SimTypeInt(signed=False, label="D3DKMDT_COLORSPACE_TRANSFORM_STAGE_CONTROL")), ("LookupTable1DRegamma", SimTypeArray(SimTypeRef("D3DDDI_DXGI_RGB", SimStruct), 4096)),)), name="D3DKMDT_COLORSPACE_TRANSFORM_MATRIX_V2", pack=False, align=None),
    "D3DKMDT_COLORSPACE_TRANSFORM_STAGE_CONTROL": SimTypeInt(signed=False, label="D3DKMDT_COLORSPACE_TRANSFORM_STAGE_CONTROL"),
    "D3DKMDT_COLOR_BASIS": SimTypeInt(signed=False, label="D3DKMDT_COLOR_BASIS"),
    "D3DKMDT_COLOR_COEFF_DYNAMIC_RANGES": SimStruct(OrderedDict((("FirstChannel", SimTypeInt(signed=False, label="UInt32")), ("SecondChannel", SimTypeInt(signed=False, label="UInt32")), ("ThirdChannel", SimTypeInt(signed=False, label="UInt32")), ("FourthChannel", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_COLOR_COEFF_DYNAMIC_RANGES", pack=False, align=None),
    "D3DKMDT_COMPUTE_PREEMPTION_GRANULARITY": SimTypeInt(signed=False, label="D3DKMDT_COMPUTE_PREEMPTION_GRANULARITY"),
    "D3DKMDT_DISPLAYMODE_FLAGS": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_DISPLAYMODE_FLAGS", pack=False, align=None),
    "D3DKMDT_ENUMCOFUNCMODALITY_PIVOT_TYPE": SimTypeInt(signed=False, label="D3DKMDT_ENUMCOFUNCMODALITY_PIVOT_TYPE"),
    "D3DKMDT_FREQUENCY_RANGE": SimStruct(OrderedDict((("MinVSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("MaxVSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("MinHSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("MaxHSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)),)), name="D3DKMDT_FREQUENCY_RANGE", pack=False, align=None),
    "D3DKMDT_GAMMA_RAMP": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DDDI_GAMMARAMP_TYPE")), ("DataSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Data", SimUnion({"pRgb256x3x16": SimTypePointer(SimTypeRef("D3DDDI_GAMMA_RAMP_RGB256x3x16", SimStruct), offset=0), "pDxgi1": SimTypePointer(SimTypeRef("D3DDDI_GAMMA_RAMP_DXGI_1", SimStruct), offset=0), "p3x4": SimTypePointer(SimTypeRef("D3DKMDT_3x4_COLORSPACE_TRANSFORM", SimStruct), offset=0), "pMatrixV2": SimTypePointer(SimTypeRef("D3DKMDT_COLORSPACE_TRANSFORM_MATRIX_V2", SimStruct), offset=0), "pRaw": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="D3DKMDT_GAMMA_RAMP", pack=False, align=None),
    "D3DKMDT_GDISURFACEDATA": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("Type", SimTypeInt(signed=False, label="D3DKMDT_GDISURFACETYPE")), ("Flags", SimTypeRef("D3DKMDT_GDISURFACEFLAGS", SimStruct)), ("Pitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_GDISURFACEDATA", pack=False, align=None),
    "D3DKMDT_GDISURFACEFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMDT_GDISURFACEFLAGS", pack=False, align=None),
    "D3DKMDT_GDISURFACETYPE": SimTypeInt(signed=False, label="D3DKMDT_GDISURFACETYPE"),
    "D3DKMDT_GRAPHICS_PREEMPTION_GRANULARITY": SimTypeInt(signed=False, label="D3DKMDT_GRAPHICS_PREEMPTION_GRANULARITY"),
    "D3DKMDT_GRAPHICS_RENDERING_FORMAT": SimStruct(OrderedDict((("PrimSurfSize", SimTypeRef("D3DKMDT_2DREGION", SimStruct)), ("VisibleRegionSize", SimTypeRef("D3DKMDT_2DREGION", SimStruct)), ("Stride", SimTypeInt(signed=False, label="UInt32")), ("PixelFormat", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("ColorBasis", SimTypeInt(signed=False, label="D3DKMDT_COLOR_BASIS")), ("PixelValueAccessMode", SimTypeInt(signed=False, label="D3DKMDT_PIXEL_VALUE_ACCESS_MODE")),)), name="D3DKMDT_GRAPHICS_RENDERING_FORMAT", pack=False, align=None),
    "D3DKMDT_GTFCOMPLIANCE": SimTypeInt(signed=False, label="D3DKMDT_GTFCOMPLIANCE"),
    "D3DKMDT_MODE_PREFERENCE": SimTypeInt(signed=False, label="D3DKMDT_MODE_PREFERENCE"),
    "D3DKMDT_MODE_PRUNING_REASON": SimTypeInt(signed=False, label="D3DKMDT_MODE_PRUNING_REASON"),
    "D3DKMDT_MONITOR_CAPABILITIES_ORIGIN": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_CAPABILITIES_ORIGIN"),
    "D3DKMDT_MONITOR_CONNECTIVITY_CHECKS": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_CONNECTIVITY_CHECKS"),
    "D3DKMDT_MONITOR_DESCRIPTOR": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DKMDT_MONITOR_DESCRIPTOR_TYPE")), ("DataSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Origin", SimTypeInt(signed=False, label="D3DKMDT_MONITOR_CAPABILITIES_ORIGIN")),)), name="D3DKMDT_MONITOR_DESCRIPTOR", pack=False, align=None),
    "D3DKMDT_MONITOR_DESCRIPTOR_TYPE": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_DESCRIPTOR_TYPE"),
    "D3DKMDT_MONITOR_FREQUENCY_RANGE": SimStruct(OrderedDict((("Origin", SimTypeInt(signed=False, label="D3DKMDT_MONITOR_CAPABILITIES_ORIGIN")), ("RangeLimits", SimTypeRef("D3DKMDT_FREQUENCY_RANGE", SimStruct)), ("ConstraintType", SimTypeInt(signed=False, label="D3DKMDT_MONITOR_FREQUENCY_RANGE_CONSTRAINT")), ("Constraint", SimUnion({"ActiveSize": SimTypeRef("D3DKMDT_2DREGION", SimStruct), "MaxPixelRate": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")),)), name="D3DKMDT_MONITOR_FREQUENCY_RANGE", pack=False, align=None),
    "D3DKMDT_MONITOR_FREQUENCY_RANGE_CONSTRAINT": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_FREQUENCY_RANGE_CONSTRAINT"),
    "D3DKMDT_MONITOR_ORIENTATION": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_ORIENTATION"),
    "D3DKMDT_MONITOR_ORIENTATION_AWARENESS": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_ORIENTATION_AWARENESS"),
    "D3DKMDT_MONITOR_SOURCE_MODE": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("VideoSignalInfo", SimTypeRef("D3DKMDT_VIDEO_SIGNAL_INFO", SimStruct)), ("ColorBasis", SimTypeInt(signed=False, label="D3DKMDT_COLOR_BASIS")), ("ColorCoeffDynamicRanges", SimTypeRef("D3DKMDT_COLOR_COEFF_DYNAMIC_RANGES", SimStruct)), ("Origin", SimTypeInt(signed=False, label="D3DKMDT_MONITOR_CAPABILITIES_ORIGIN")), ("Preference", SimTypeInt(signed=False, label="D3DKMDT_MODE_PREFERENCE")),)), name="D3DKMDT_MONITOR_SOURCE_MODE", pack=False, align=None),
    "D3DKMDT_MONITOR_TIMING_TYPE": SimTypeInt(signed=False, label="D3DKMDT_MONITOR_TIMING_TYPE"),
    "D3DKMDT_PALETTEDATA": SimStruct(OrderedDict((("Red", SimTypeChar(label="Byte")), ("Green", SimTypeChar(label="Byte")), ("Blue", SimTypeChar(label="Byte")), ("Unused", SimTypeChar(label="Byte")),)), name="D3DKMDT_PALETTEDATA", pack=False, align=None),
    "D3DKMDT_PIXEL_VALUE_ACCESS_MODE": SimTypeInt(signed=False, label="D3DKMDT_PIXEL_VALUE_ACCESS_MODE"),
    "D3DKMDT_PREEMPTION_CAPS": SimStruct(OrderedDict((("GraphicsPreemptionGranularity", SimTypeInt(signed=False, label="D3DKMDT_GRAPHICS_PREEMPTION_GRANULARITY")), ("ComputePreemptionGranularity", SimTypeInt(signed=False, label="D3DKMDT_COMPUTE_PREEMPTION_GRANULARITY")),)), name="D3DKMDT_PREEMPTION_CAPS", pack=False, align=None),
    "D3DKMDT_SHADOWSURFACEDATA": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("Pitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_SHADOWSURFACEDATA", pack=False, align=None),
    "D3DKMDT_SHAREDPRIMARYSURFACEDATA": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("RefreshRate", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_SHAREDPRIMARYSURFACEDATA", pack=False, align=None),
    "D3DKMDT_STAGINGSURFACEDATA": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Pitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_STAGINGSURFACEDATA", pack=False, align=None),
    "D3DKMDT_STANDARDALLOCATION_TYPE": SimTypeInt(signed=False, label="D3DKMDT_STANDARDALLOCATION_TYPE"),
    "D3DKMDT_TEXT_RENDERING_FORMAT": SimTypeInt(signed=False, label="D3DKMDT_TEXT_RENDERING_FORMAT"),
    "D3DKMDT_VIDEO_OUTPUT_TECHNOLOGY": SimTypeInt(signed=False, label="D3DKMDT_VIDEO_OUTPUT_TECHNOLOGY"),
    "D3DKMDT_VIDEO_PRESENT_SOURCE": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_VIDEO_PRESENT_SOURCE", pack=False, align=None),
    "D3DKMDT_VIDEO_PRESENT_TARGET": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("VideoOutputTechnology", SimTypeInt(signed=False, label="D3DKMDT_VIDEO_OUTPUT_TECHNOLOGY")), ("VideoOutputHpdAwareness", SimTypeInt(signed=False, label="DXGK_CHILD_DEVICE_HPD_AWARENESS")), ("MonitorOrientationAwareness", SimTypeInt(signed=False, label="D3DKMDT_MONITOR_ORIENTATION_AWARENESS")), ("SupportsSdtvModes", SimTypeChar(label="Byte")),)), name="D3DKMDT_VIDEO_PRESENT_TARGET", pack=False, align=None),
    "D3DKMDT_VIDEO_SIGNAL_INFO": SimStruct(OrderedDict((("VideoStandard", SimTypeInt(signed=False, label="D3DKMDT_VIDEO_SIGNAL_STANDARD")), ("TotalSize", SimTypeRef("D3DKMDT_2DREGION", SimStruct)), ("ActiveSize", SimTypeRef("D3DKMDT_2DREGION", SimStruct)), ("VSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("HSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("PixelRate", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous", SimUnion({"AdditionalSignalInfo": SimTypeRef("_AdditionalSignalInfo_e__Struct", SimStruct), "ScanLineOrdering": SimTypeInt(signed=False, label="D3DDDI_VIDEO_SIGNAL_SCANLINE_ORDERING")}, name="<anon>", label="None")),)), name="D3DKMDT_VIDEO_SIGNAL_INFO", pack=False, align=None),
    "D3DKMDT_VIDEO_SIGNAL_STANDARD": SimTypeInt(signed=False, label="D3DKMDT_VIDEO_SIGNAL_STANDARD"),
    "D3DKMDT_VIDPN_HW_CAPABILITY": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_VIDPN_HW_CAPABILITY", pack=False, align=None),
    "D3DKMDT_VIDPN_PRESENT_PATH": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("VidPnTargetId", SimTypeInt(signed=False, label="UInt32")), ("ImportanceOrdinal", SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_IMPORTANCE")), ("ContentTransformation", SimTypeRef("D3DKMDT_VIDPN_PRESENT_PATH_TRANSFORMATION", SimStruct)), ("VisibleFromActiveTLOffset", SimTypeRef("D3DKMDT_2DREGION", SimStruct)), ("VisibleFromActiveBROffset", SimTypeRef("D3DKMDT_2DREGION", SimStruct)), ("VidPnTargetColorBasis", SimTypeInt(signed=False, label="D3DKMDT_COLOR_BASIS")), ("VidPnTargetColorCoeffDynamicRanges", SimTypeRef("D3DKMDT_COLOR_COEFF_DYNAMIC_RANGES", SimStruct)), ("Content", SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_CONTENT")), ("CopyProtection", SimTypeRef("D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION", SimStruct)), ("GammaRamp", SimTypeRef("D3DKMDT_GAMMA_RAMP", SimStruct)),)), name="D3DKMDT_VIDPN_PRESENT_PATH", pack=False, align=None),
    "D3DKMDT_VIDPN_PRESENT_PATH_CONTENT": SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_CONTENT"),
    "D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION": SimStruct(OrderedDict((("CopyProtectionType", SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION_TYPE")), ("APSTriggerBits", SimTypeInt(signed=False, label="UInt32")), ("OEMCopyProtection", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("CopyProtectionSupport", SimTypeRef("D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION_SUPPORT", SimStruct)),)), name="D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION", pack=False, align=None),
    "D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION_SUPPORT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION_SUPPORT", pack=False, align=None),
    "D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION_TYPE": SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_COPYPROTECTION_TYPE"),
    "D3DKMDT_VIDPN_PRESENT_PATH_IMPORTANCE": SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_IMPORTANCE"),
    "D3DKMDT_VIDPN_PRESENT_PATH_ROTATION": SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_ROTATION"),
    "D3DKMDT_VIDPN_PRESENT_PATH_ROTATION_SUPPORT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_VIDPN_PRESENT_PATH_ROTATION_SUPPORT", pack=False, align=None),
    "D3DKMDT_VIDPN_PRESENT_PATH_SCALING": SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_SCALING"),
    "D3DKMDT_VIDPN_PRESENT_PATH_SCALING_SUPPORT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_VIDPN_PRESENT_PATH_SCALING_SUPPORT", pack=False, align=None),
    "D3DKMDT_VIDPN_PRESENT_PATH_TRANSFORMATION": SimStruct(OrderedDict((("Scaling", SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_SCALING")), ("ScalingSupport", SimTypeRef("D3DKMDT_VIDPN_PRESENT_PATH_SCALING_SUPPORT", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DKMDT_VIDPN_PRESENT_PATH_ROTATION")), ("RotationSupport", SimTypeRef("D3DKMDT_VIDPN_PRESENT_PATH_ROTATION_SUPPORT", SimStruct)),)), name="D3DKMDT_VIDPN_PRESENT_PATH_TRANSFORMATION", pack=False, align=None),
    "D3DKMDT_VIDPN_SOURCE_MODE": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DKMDT_VIDPN_SOURCE_MODE_TYPE")), ("Format", SimUnion({"Graphics": SimTypeRef("D3DKMDT_GRAPHICS_RENDERING_FORMAT", SimStruct), "Text": SimTypeInt(signed=False, label="D3DKMDT_TEXT_RENDERING_FORMAT")}, name="<anon>", label="None")),)), name="D3DKMDT_VIDPN_SOURCE_MODE", pack=False, align=None),
    "D3DKMDT_VIDPN_SOURCE_MODE_TYPE": SimTypeInt(signed=False, label="D3DKMDT_VIDPN_SOURCE_MODE_TYPE"),
    "D3DKMDT_VIDPN_TARGET_MODE": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("VideoSignalInfo", SimTypeRef("D3DKMDT_VIDEO_SIGNAL_INFO", SimStruct)), ("Anonymous", SimUnion({"WireFormatAndPreference": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("MinimumVSyncFreq", SimTypeRef("D3DDDI_RATIONAL", SimStruct)),)), name="D3DKMDT_VIDPN_TARGET_MODE", pack=False, align=None),
    "D3DKMDT_VIRTUALGPUSURFACEDATA": SimStruct(OrderedDict((("Size", SimTypeLongLong(signed=False, label="UInt64")), ("Alignment", SimTypeInt(signed=False, label="UInt32")), ("DriverSegmentId", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverData", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMDT_VIRTUALGPUSURFACEDATA", pack=False, align=None),
    "D3DKMDT_WIRE_FORMAT_AND_PREFERENCE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3DKMT_ACQUIREKEYEDMUTEX": SimStruct(OrderedDict((("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeLongLong(signed=False, label="UInt64")), ("pTimeout", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("FenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_ACQUIREKEYEDMUTEX", pack=False, align=None),
    "D3DKMT_ACQUIREKEYEDMUTEX2": SimStruct(OrderedDict((("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeLongLong(signed=False, label="UInt64")), ("pTimeout", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("FenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_ACQUIREKEYEDMUTEX2", pack=False, align=None),
    "D3DKMT_ACTIVATE_SPECIFIC_DIAG_ESCAPE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_ACTIVATE_SPECIFIC_DIAG_TYPE")), ("Activate", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_ACTIVATE_SPECIFIC_DIAG_ESCAPE", pack=False, align=None),
    "D3DKMT_ACTIVATE_SPECIFIC_DIAG_TYPE": SimTypeInt(signed=False, label="D3DKMT_ACTIVATE_SPECIFIC_DIAG_TYPE"),
    "D3DKMT_ADAPTERADDRESS": SimStruct(OrderedDict((("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("FunctionNumber", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_ADAPTERADDRESS", pack=False, align=None),
    "D3DKMT_ADAPTERINFO": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("NumOfSources", SimTypeInt(signed=False, label="UInt32")), ("bPrecisePresentRegionsPreferred", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_ADAPTERINFO", pack=False, align=None),
    "D3DKMT_ADAPTERREGISTRYINFO": SimStruct(OrderedDict((("AdapterString", SimTypeArray(SimTypeChar(label="Char"), 260)), ("BiosString", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DacType", SimTypeArray(SimTypeChar(label="Char"), 260)), ("ChipType", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_ADAPTERREGISTRYINFO", pack=False, align=None),
    "D3DKMT_ADAPTERTYPE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_ADAPTERTYPE", pack=False, align=None),
    "D3DKMT_ADAPTER_PERFDATA": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("MemoryFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxMemoryFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxMemoryFrequencyOC", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("PCIEBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("FanRPM", SimTypeInt(signed=False, label="UInt32")), ("Power", SimTypeInt(signed=False, label="UInt32")), ("Temperature", SimTypeInt(signed=False, label="UInt32")), ("PowerStateOverride", SimTypeChar(label="Byte")),)), name="D3DKMT_ADAPTER_PERFDATA", pack=False, align=None),
    "D3DKMT_ADAPTER_PERFDATACAPS": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxMemoryBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("MaxPCIEBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("MaxFanRPM", SimTypeInt(signed=False, label="UInt32")), ("TemperatureMax", SimTypeInt(signed=False, label="UInt32")), ("TemperatureWarning", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_ADAPTER_PERFDATACAPS", pack=False, align=None),
    "D3DKMT_ADAPTER_VERIFIER_OPTION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_ADAPTER_VERIFIER_OPTION_TYPE")), ("Mode", SimTypeInt(signed=False, label="D3DKMT_VERIFIER_OPTION_MODE")), ("Data", SimUnion({"VidMmFlags": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VidMmTrimInterval": SimTypeRef("D3DKMT_ADAPTER_VERIFIER_VIDMM_TRIM_INTERVAL", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_ADAPTER_VERIFIER_OPTION", pack=False, align=None),
    "D3DKMT_ADAPTER_VERIFIER_OPTION_DATA": SimUnion({"VidMmFlags": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VidMmTrimInterval": SimTypeRef("D3DKMT_ADAPTER_VERIFIER_VIDMM_TRIM_INTERVAL", SimStruct)}, name="<anon>", label="None"),
    "D3DKMT_ADAPTER_VERIFIER_OPTION_TYPE": SimTypeInt(signed=False, label="D3DKMT_ADAPTER_VERIFIER_OPTION_TYPE"),
    "D3DKMT_ADAPTER_VERIFIER_VIDMM_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3DKMT_ADAPTER_VERIFIER_VIDMM_TRIM_INTERVAL": SimStruct(OrderedDict((("MinimumTrimInterval", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumTrimInterval", SimTypeLongLong(signed=False, label="UInt64")), ("IdleTrimInterval", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_ADAPTER_VERIFIER_VIDMM_TRIM_INTERVAL", pack=False, align=None),
    "D3DKMT_ADJUSTFULLSCREENGAMMA": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Scale", SimTypeRef("D3DDDI_DXGI_RGB", SimStruct)), ("Offset", SimTypeRef("D3DDDI_DXGI_RGB", SimStruct)),)), name="D3DKMT_ADJUSTFULLSCREENGAMMA", pack=False, align=None),
    "D3DKMT_ALLOCATIONRESIDENCYSTATUS": SimTypeInt(signed=False, label="D3DKMT_ALLOCATIONRESIDENCYSTATUS"),
    "D3DKMT_AUXILIARYPRESENTINFO": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="D3DKMT_AUXILIARYPRESENTINFO_TYPE")),)), name="D3DKMT_AUXILIARYPRESENTINFO", pack=False, align=None),
    "D3DKMT_AUXILIARYPRESENTINFO_TYPE": SimTypeInt(signed=False, label="D3DKMT_AUXILIARYPRESENTINFO_TYPE"),
    "D3DKMT_BDDFALLBACK_CTL": SimStruct(OrderedDict((("ForceBddHeadlessNextFallback", SimTypeChar(label="Byte")),)), name="D3DKMT_BDDFALLBACK_CTL", pack=False, align=None),
    "D3DKMT_BLOCKLIST_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("BlockList", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="D3DKMT_BLOCKLIST_INFO", pack=False, align=None),
    "D3DKMT_BLTMODEL_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("hLogicalSurface", SimTypeLongLong(signed=False, label="UInt64")), ("hPhysicalSurface", SimTypeLongLong(signed=False, label="UInt64")), ("EventId", SimTypeLongLong(signed=False, label="UInt64")), ("DirtyRegions", SimTypeRef("D3DKMT_DIRTYREGIONS", SimStruct)),)), name="D3DKMT_BLTMODEL_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_BRIGHTNESS_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_BRIGHTNESS_INFO_TYPE")), ("ChildUid", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"PossibleLevels": SimTypeRef("D3DKMT_BRIGHTNESS_POSSIBLE_LEVELS", SimStruct), "Brightness": SimTypeChar(label="Byte"), "BrightnessCaps": SimTypeRef("DXGK_BRIGHTNESS_CAPS", SimStruct), "BrightnessState": SimTypeRef("DXGK_BRIGHTNESS_STATE", SimStruct), "OptimizationLevel": SimTypeInt(signed=False, label="DXGK_BACKLIGHT_OPTIMIZATION_LEVEL"), "ReductionInfo": SimTypeRef("DXGK_BACKLIGHT_INFO", SimStruct), "VerboseLogging": SimTypeChar(label="Byte"), "NitRanges": SimTypeRef("DXGK_BRIGHTNESS_GET_NIT_RANGES_OUT", SimStruct), "GetBrightnessMillinits": SimTypeRef("DXGK_BRIGHTNESS_GET_OUT", SimStruct), "SetBrightnessMillinits": SimTypeRef("DXGK_BRIGHTNESS_SET_IN", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_BRIGHTNESS_INFO", pack=False, align=None),
    "D3DKMT_BRIGHTNESS_INFO_TYPE": SimTypeInt(signed=False, label="D3DKMT_BRIGHTNESS_INFO_TYPE"),
    "D3DKMT_BRIGHTNESS_POSSIBLE_LEVELS": SimStruct(OrderedDict((("LevelCount", SimTypeChar(label="Byte")), ("BrightnessLevels", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="D3DKMT_BRIGHTNESS_POSSIBLE_LEVELS", pack=False, align=None),
    "D3DKMT_BUDGETCHANGENOTIFICATION": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Budget", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_BUDGETCHANGENOTIFICATION", pack=False, align=None),
    "D3DKMT_CANCEL_PRESENTS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_CANCEL_PRESENTS_FLAGS", SimStruct)), ("Operation", SimTypeInt(signed=False, label="D3DKMT_CANCEL_PRESENTS_OPERATION")), ("CancelFromPresentId", SimTypeLongLong(signed=False, label="UInt64")), ("CompSurfaceLuid", SimTypeRef("LUID", SimStruct)), ("BindId", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_CANCEL_PRESENTS", pack=False, align=None),
    "D3DKMT_CANCEL_PRESENTS_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"ReprogramInterrupt": SimTypeRef("_ReprogramInterrupt_e__Struct", SimStruct), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_CANCEL_PRESENTS_FLAGS", pack=False, align=None),
    "D3DKMT_CANCEL_PRESENTS_OPERATION": SimTypeInt(signed=False, label="D3DKMT_CANCEL_PRESENTS_OPERATION"),
    "D3DKMT_CHANGESURFACEPOINTER": SimStruct(OrderedDict((("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hBitmap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pSurfacePointer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Pitch", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CHANGESURFACEPOINTER", pack=False, align=None),
    "D3DKMT_CHANGEVIDEOMEMORYRESERVATION": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("MemorySegmentGroup", SimTypeInt(signed=False, label="D3DKMT_MEMORY_SEGMENT_GROUP")), ("Reservation", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CHANGEVIDEOMEMORYRESERVATION", pack=False, align=None),
    "D3DKMT_CHECKMONITORPOWERSTATE": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CHECKMONITORPOWERSTATE", pack=False, align=None),
    "D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("PlaneCount", SimTypeInt(signed=False, label="UInt32")), ("pOverlayPlanes", SimTypePointer(SimTypeRef("D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE", SimStruct), offset=0)), ("Supported", SimTypeInt(signed=True, label="Int32")), ("ReturnInfo", SimTypeRef("D3DKMT_CHECK_MULTIPLANE_OVERLAY_SUPPORT_RETURN_INFO", SimStruct)),)), name="D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT", pack=False, align=None),
    "D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT2": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("PlaneCount", SimTypeInt(signed=False, label="UInt32")), ("pOverlayPlanes", SimTypePointer(SimTypeRef("D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE2", SimStruct), offset=0)), ("Supported", SimTypeInt(signed=True, label="Int32")), ("ReturnInfo", SimTypeRef("D3DKMT_CHECK_MULTIPLANE_OVERLAY_SUPPORT_RETURN_INFO", SimStruct)),)), name="D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT2", pack=False, align=None),
    "D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT3": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("PlaneCount", SimTypeInt(signed=False, label="UInt32")), ("ppOverlayPlanes", SimTypePointer(SimTypePointer(SimTypeRef("D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE3", SimStruct), offset=0), offset=0)), ("PostCompositionCount", SimTypeInt(signed=False, label="UInt32")), ("ppPostComposition", SimTypePointer(SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION_WITH_SOURCE", SimStruct), offset=0), offset=0)), ("Supported", SimTypeInt(signed=True, label="Int32")), ("ReturnInfo", SimTypeRef("D3DKMT_CHECK_MULTIPLANE_OVERLAY_SUPPORT_RETURN_INFO", SimStruct)),)), name="D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT3", pack=False, align=None),
    "D3DKMT_CHECKOCCLUSION": SimStruct(OrderedDict((("hWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_CHECKOCCLUSION", pack=False, align=None),
    "D3DKMT_CHECKSHAREDRESOURCEACCESS": SimStruct(OrderedDict((("hResource", SimTypeInt(signed=False, label="UInt32")), ("ClientPid", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CHECKSHAREDRESOURCEACCESS", pack=False, align=None),
    "D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP", pack=False, align=None),
    "D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE": SimStruct(OrderedDict((("hResource", SimTypeInt(signed=False, label="UInt32")), ("CompSurfaceLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PlaneAttributes", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES", SimStruct)),)), name="D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE", pack=False, align=None),
    "D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE2": SimStruct(OrderedDict((("LayerIndex", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("CompSurfaceLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PlaneAttributes", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES2", SimStruct)),)), name="D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE2", pack=False, align=None),
    "D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE3": SimStruct(OrderedDict((("LayerIndex", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("CompSurfaceLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("pPlaneAttributes", SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES3", SimStruct), offset=0)),)), name="D3DKMT_CHECK_MULTIPLANE_OVERLAY_PLANE3", pack=False, align=None),
    "D3DKMT_CHECK_MULTIPLANE_OVERLAY_SUPPORT_RETURN_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_CHECK_MULTIPLANE_OVERLAY_SUPPORT_RETURN_INFO", pack=False, align=None),
    "D3DKMT_CLIENTHINT": SimTypeInt(signed=False, label="D3DKMT_CLIENTHINT"),
    "D3DKMT_CLOSEADAPTER": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CLOSEADAPTER", pack=False, align=None),
    "D3DKMT_COMPOSITION_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("hPrivateData", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_COMPOSITION_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_CONFIGURESHAREDRESOURCE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("IsDwm", SimTypeChar(label="Byte")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AllowAccess", SimTypeChar(label="Byte")),)), name="D3DKMT_CONFIGURESHAREDRESOURCE", pack=False, align=None),
    "D3DKMT_CONNECT_DOORBELL": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_CONNECT_DOORBELL_FLAGS", SimStruct)),)), name="D3DKMT_CONNECT_DOORBELL", pack=False, align=None),
    "D3DKMT_CONNECT_DOORBELL_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_CONNECT_DOORBELL_FLAGS", pack=False, align=None),
    "D3DKMT_CPDRIVERNAME": SimStruct(OrderedDict((("ContentProtectionFileName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_CPDRIVERNAME", pack=False, align=None),
    "D3DKMT_CREATEALLOCATION": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("hGlobalShare", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"pStandardAllocation": SimTypePointer(SimTypeRef("D3DKMT_CREATESTANDARDALLOCATION", SimStruct), offset=0), "pPrivateDriverData": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"pAllocationInfo": SimTypePointer(SimTypeRef("D3DDDI_ALLOCATIONINFO", SimStruct), offset=0), "pAllocationInfo2": SimTypePointer(SimTypeRef("D3DDDI_ALLOCATIONINFO2", SimStruct), offset=0)}, name="<anon>", label="None")), ("Flags", SimTypeRef("D3DKMT_CREATEALLOCATIONFLAGS", SimStruct)), ("hPrivateRuntimeResourceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_CREATEALLOCATION", pack=False, align=None),
    "D3DKMT_CREATEALLOCATIONFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEALLOCATIONFLAGS", pack=False, align=None),
    "D3DKMT_CREATECONTEXT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("NodeOrdinal", SimTypeInt(signed=False, label="UInt32")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_CREATECONTEXTFLAGS", SimStruct)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("ClientHint", SimTypeInt(signed=False, label="D3DKMT_CLIENTHINT")), ("hContext", SimTypeInt(signed=False, label="UInt32")), ("pCommandBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CommandBufferSize", SimTypeInt(signed=False, label="UInt32")), ("pAllocationList", SimTypePointer(SimTypeRef("D3DDDI_ALLOCATIONLIST", SimStruct), offset=0)), ("AllocationListSize", SimTypeInt(signed=False, label="UInt32")), ("pPatchLocationList", SimTypePointer(SimTypeRef("D3DDDI_PATCHLOCATIONLIST", SimStruct), offset=0)), ("PatchLocationListSize", SimTypeInt(signed=False, label="UInt32")), ("CommandBuffer", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_CREATECONTEXT", pack=False, align=None),
    "D3DKMT_CREATECONTEXTVIRTUAL": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("NodeOrdinal", SimTypeInt(signed=False, label="UInt32")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_CREATECONTEXTFLAGS", SimStruct)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("ClientHint", SimTypeInt(signed=False, label="D3DKMT_CLIENTHINT")), ("hContext", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATECONTEXTVIRTUAL", pack=False, align=None),
    "D3DKMT_CREATEDCFROMMEMORY": SimStruct(OrderedDict((("pMemory", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Pitch", SimTypeInt(signed=False, label="UInt32")), ("hDeviceDc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pColorTable", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)), ("hDc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hBitmap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_CREATEDCFROMMEMORY", pack=False, align=None),
    "D3DKMT_CREATEDEVICE": SimStruct(OrderedDict((("Anonymous", SimUnion({"hAdapter": SimTypeInt(signed=False, label="UInt32"), "pAdapter": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("Flags", SimTypeRef("D3DKMT_CREATEDEVICEFLAGS", SimStruct)), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("pCommandBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CommandBufferSize", SimTypeInt(signed=False, label="UInt32")), ("pAllocationList", SimTypePointer(SimTypeRef("D3DDDI_ALLOCATIONLIST", SimStruct), offset=0)), ("AllocationListSize", SimTypeInt(signed=False, label="UInt32")), ("pPatchLocationList", SimTypePointer(SimTypeRef("D3DDDI_PATCHLOCATIONLIST", SimStruct), offset=0)), ("PatchLocationListSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEDEVICE", pack=False, align=None),
    "D3DKMT_CREATEDEVICEFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEDEVICEFLAGS", pack=False, align=None),
    "D3DKMT_CREATEHWCONTEXT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("NodeOrdinal", SimTypeInt(signed=False, label="UInt32")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_CREATEHWCONTEXTFLAGS", SimStruct)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hHwContext", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEHWCONTEXT", pack=False, align=None),
    "D3DKMT_CREATEHWQUEUE": SimStruct(OrderedDict((("hHwContext", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_CREATEHWQUEUEFLAGS", SimStruct)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("hHwQueueProgressFence", SimTypeInt(signed=False, label="UInt32")), ("HwQueueProgressFenceCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HwQueueProgressFenceGPUVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_CREATEHWQUEUE", pack=False, align=None),
    "D3DKMT_CREATEKEYEDMUTEX": SimStruct(OrderedDict((("InitialValue", SimTypeLongLong(signed=False, label="UInt64")), ("hSharedHandle", SimTypeInt(signed=False, label="UInt32")), ("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEKEYEDMUTEX", pack=False, align=None),
    "D3DKMT_CREATEKEYEDMUTEX2": SimStruct(OrderedDict((("InitialValue", SimTypeLongLong(signed=False, label="UInt64")), ("hSharedHandle", SimTypeInt(signed=False, label="UInt32")), ("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_CREATEKEYEDMUTEX2_FLAGS", SimStruct)),)), name="D3DKMT_CREATEKEYEDMUTEX2", pack=False, align=None),
    "D3DKMT_CREATEKEYEDMUTEX2_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_CREATEKEYEDMUTEX2_FLAGS", pack=False, align=None),
    "D3DKMT_CREATENATIVEFENCE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Info", SimTypeRef("D3DDDI_CREATENATIVEFENCEINFO", SimStruct)),)), name="D3DKMT_CREATENATIVEFENCE", pack=False, align=None),
    "D3DKMT_CREATEOVERLAY": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("OverlayInfo", SimTypeRef("D3DDDI_KERNELOVERLAYINFO", SimStruct)), ("hOverlay", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEOVERLAY", pack=False, align=None),
    "D3DKMT_CREATEPAGINGQUEUE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="D3DDDI_PAGINGQUEUE_PRIORITY")), ("hPagingQueue", SimTypeInt(signed=False, label="UInt32")), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("FenceValueCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEPAGINGQUEUE", pack=False, align=None),
    "D3DKMT_CREATEPROTECTEDSESSION": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("hHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATEPROTECTEDSESSION", pack=False, align=None),
    "D3DKMT_CREATESTANDARDALLOCATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_STANDARDALLOCATIONTYPE")), ("Anonymous", SimUnion({"ExistingHeapData": SimTypeRef("D3DKMT_STANDARDALLOCATION_EXISTINGHEAP", SimStruct)}, name="<anon>", label="None")), ("Flags", SimTypeRef("D3DKMT_CREATESTANDARDALLOCATIONFLAGS", SimStruct)),)), name="D3DKMT_CREATESTANDARDALLOCATION", pack=False, align=None),
    "D3DKMT_CREATESTANDARDALLOCATIONFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_CREATESTANDARDALLOCATIONFLAGS", pack=False, align=None),
    "D3DKMT_CREATESYNCFILE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hMonitoredFence", SimTypeInt(signed=False, label="UInt32")), ("FenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("hSyncFile", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_CREATESYNCFILE", pack=False, align=None),
    "D3DKMT_CREATESYNCHRONIZATIONOBJECT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Info", SimTypeRef("D3DDDI_SYNCHRONIZATIONOBJECTINFO", SimStruct)), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATESYNCHRONIZATIONOBJECT", pack=False, align=None),
    "D3DKMT_CREATESYNCHRONIZATIONOBJECT2": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Info", SimTypeRef("D3DDDI_SYNCHRONIZATIONOBJECTINFO2", SimStruct)), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_CREATESYNCHRONIZATIONOBJECT2", pack=False, align=None),
    "D3DKMT_CREATE_DOORBELL": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("hRingBuffer", SimTypeInt(signed=False, label="UInt32")), ("hRingBufferControl", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_CREATE_DOORBELL_FLAGS", SimStruct)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DoorbellCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DoorbellSecondaryCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DoorbellStatusCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DKMT_CREATE_DOORBELL", pack=False, align=None),
    "D3DKMT_CREATE_DOORBELL_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_CREATE_DOORBELL_FLAGS", pack=False, align=None),
    "D3DKMT_CREATE_OUTPUTDUPL": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("KeyedMutexCount", SimTypeInt(signed=False, label="UInt32")), ("RequiredKeyedMutexCount", SimTypeInt(signed=False, label="UInt32")), ("KeyedMutexs", SimTypeArray(SimTypeRef("D3DKMT_OUTPUTDUPL_KEYEDMUTEX", SimStruct), 3)), ("Flags", SimTypeRef("D3DKMT_OUTPUTDUPLCREATIONFLAGS", SimStruct)),)), name="D3DKMT_CREATE_OUTPUTDUPL", pack=False, align=None),
    "D3DKMT_CROSSADAPTERRESOURCE_SUPPORT": SimStruct(OrderedDict((("SupportTier", SimTypeInt(signed=False, label="D3DKMT_CROSSADAPTERRESOURCE_SUPPORT_TIER")),)), name="D3DKMT_CROSSADAPTERRESOURCE_SUPPORT", pack=False, align=None),
    "D3DKMT_CROSSADAPTERRESOURCE_SUPPORT_TIER": SimTypeInt(signed=False, label="D3DKMT_CROSSADAPTERRESOURCE_SUPPORT_TIER"),
    "D3DKMT_CURRENTDISPLAYMODE": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("DisplayMode", SimTypeRef("D3DKMT_DISPLAYMODE", SimStruct)),)), name="D3DKMT_CURRENTDISPLAYMODE", pack=False, align=None),
    "D3DKMT_DEBUG_SNAPSHOT_ESCAPE": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3DKMT_DEBUG_SNAPSHOT_ESCAPE", pack=False, align=None),
    "D3DKMT_DEFRAG_ESCAPE_OPERATION": SimTypeInt(signed=False, label="D3DKMT_DEFRAG_ESCAPE_OPERATION"),
    "D3DKMT_DESTROYALLOCATION": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("phAllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYALLOCATION", pack=False, align=None),
    "D3DKMT_DESTROYALLOCATION2": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("phAllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDICB_DESTROYALLOCATION2FLAGS", SimStruct)),)), name="D3DKMT_DESTROYALLOCATION2", pack=False, align=None),
    "D3DKMT_DESTROYCONTEXT": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYCONTEXT", pack=False, align=None),
    "D3DKMT_DESTROYDCFROMMEMORY": SimStruct(OrderedDict((("hDc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hBitmap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_DESTROYDCFROMMEMORY", pack=False, align=None),
    "D3DKMT_DESTROYDEVICE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYDEVICE", pack=False, align=None),
    "D3DKMT_DESTROYHWCONTEXT": SimStruct(OrderedDict((("hHwContext", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYHWCONTEXT", pack=False, align=None),
    "D3DKMT_DESTROYHWQUEUE": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYHWQUEUE", pack=False, align=None),
    "D3DKMT_DESTROYKEYEDMUTEX": SimStruct(OrderedDict((("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYKEYEDMUTEX", pack=False, align=None),
    "D3DKMT_DESTROYOVERLAY": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hOverlay", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYOVERLAY", pack=False, align=None),
    "D3DKMT_DESTROYPROTECTEDSESSION": SimStruct(OrderedDict((("hHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYPROTECTEDSESSION", pack=False, align=None),
    "D3DKMT_DESTROYSYNCHRONIZATIONOBJECT": SimStruct(OrderedDict((("hSyncObject", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROYSYNCHRONIZATIONOBJECT", pack=False, align=None),
    "D3DKMT_DESTROY_DOORBELL": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DESTROY_DOORBELL", pack=False, align=None),
    "D3DKMT_DESTROY_OUTPUTDUPL": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("bDestroyAllContexts", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_DESTROY_OUTPUTDUPL", pack=False, align=None),
    "D3DKMT_DEVICEESCAPE_TYPE": SimTypeInt(signed=False, label="D3DKMT_DEVICEESCAPE_TYPE"),
    "D3DKMT_DEVICEEXECUTION_STATE": SimTypeInt(signed=False, label="D3DKMT_DEVICEEXECUTION_STATE"),
    "D3DKMT_DEVICEPAGEFAULT_STATE": SimStruct(OrderedDict((("FaultedPrimitiveAPISequenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("FaultedPipelineStage", SimTypeInt(signed=False, label="DXGK_RENDER_PIPELINE_STAGE")), ("FaultedBindTableEntry", SimTypeInt(signed=False, label="UInt32")), ("PageFaultFlags", SimTypeInt(signed=False, label="DXGK_PAGE_FAULT_FLAGS")), ("FaultErrorCode", SimTypeRef("DXGK_FAULT_ERROR_CODE", SimStruct)), ("FaultedVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_DEVICEPAGEFAULT_STATE", pack=False, align=None),
    "D3DKMT_DEVICEPRESENT_QUEUE_STATE": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("bQueuedPresentLimitReached", SimTypeChar(label="Byte")),)), name="D3DKMT_DEVICEPRESENT_QUEUE_STATE", pack=False, align=None),
    "D3DKMT_DEVICEPRESENT_STATE": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PresentStats", SimTypeRef("D3DKMT_PRESENT_STATS", SimStruct)),)), name="D3DKMT_DEVICEPRESENT_STATE", pack=False, align=None),
    "D3DKMT_DEVICEPRESENT_STATE_DWM": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PresentStatsDWM", SimTypeRef("D3DKMT_PRESENT_STATS_DWM", SimStruct)),)), name="D3DKMT_DEVICEPRESENT_STATE_DWM", pack=False, align=None),
    "D3DKMT_DEVICERESET_STATE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_DEVICERESET_STATE", pack=False, align=None),
    "D3DKMT_DEVICESTATE_TYPE": SimTypeInt(signed=False, label="D3DKMT_DEVICESTATE_TYPE"),
    "D3DKMT_DEVICE_ERROR_REASON": SimTypeInt(signed=False, label="D3DKMT_DEVICE_ERROR_REASON"),
    "D3DKMT_DEVICE_ESCAPE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_DEVICEESCAPE_TYPE")), ("Anonymous", SimUnion({"VidPnFromAllocation": SimTypeRef("_VidPnFromAllocation_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_DEVICE_ESCAPE", pack=False, align=None),
    "D3DKMT_DEVICE_IDS": SimStruct(OrderedDict((("VendorID", SimTypeInt(signed=False, label="UInt32")), ("DeviceID", SimTypeInt(signed=False, label="UInt32")), ("SubVendorID", SimTypeInt(signed=False, label="UInt32")), ("SubSystemID", SimTypeInt(signed=False, label="UInt32")), ("RevisionID", SimTypeInt(signed=False, label="UInt32")), ("BusType", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_DEVICE_IDS", pack=False, align=None),
    "D3DKMT_DIRECTFLIP_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_DIRECTFLIP_SUPPORT", pack=False, align=None),
    "D3DKMT_DIRTYREGIONS": SimStruct(OrderedDict((("NumRects", SimTypeInt(signed=False, label="UInt32")), ("Rects", SimTypeArray(SimTypeRef("RECT", SimStruct), 16)),)), name="D3DKMT_DIRTYREGIONS", pack=False, align=None),
    "D3DKMT_DISPLAYMODE": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("IntegerRefreshRate", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeRef("D3DDDI_RATIONAL", SimStruct)), ("ScanLineOrdering", SimTypeInt(signed=False, label="D3DDDI_VIDEO_SIGNAL_SCANLINE_ORDERING")), ("DisplayOrientation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("DisplayFixedOutput", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMDT_DISPLAYMODE_FLAGS", SimStruct)),)), name="D3DKMT_DISPLAYMODE", pack=False, align=None),
    "D3DKMT_DISPLAYMODELIST": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("ModeCount", SimTypeInt(signed=False, label="UInt32")), ("pModeList", SimTypePointer(SimTypeRef("D3DKMT_DISPLAYMODE", SimStruct), offset=0)),)), name="D3DKMT_DISPLAYMODELIST", pack=False, align=None),
    "D3DKMT_DISPLAY_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="D3DKMT_DISPLAY_CAPS", pack=False, align=None),
    "D3DKMT_DISPLAY_UMD_FILENAMEINFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="KMT_DISPLAY_UMD_VERSION")), ("UmdFileName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_DISPLAY_UMD_FILENAMEINFO", pack=False, align=None),
    "D3DKMT_DLIST_DRIVER_NAME": SimStruct(OrderedDict((("DListFileName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_DLIST_DRIVER_NAME", pack=False, align=None),
    "D3DKMT_DMMESCAPETYPE": SimTypeInt(signed=False, label="D3DKMT_DMMESCAPETYPE"),
    "D3DKMT_DMM_ESCAPE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_DMMESCAPETYPE")), ("ProvidedBufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MinRequiredBufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3DKMT_DMM_ESCAPE", pack=False, align=None),
    "D3DKMT_DOD_SET_DIRTYRECT_MODE": SimStruct(OrderedDict((("bForceFullScreenDirty", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_DOD_SET_DIRTYRECT_MODE", pack=False, align=None),
    "D3DKMT_DRIVERCAPS_EXT": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_DRIVERCAPS_EXT", pack=False, align=None),
    "D3DKMT_DRIVERVERSION": SimTypeInt(signed=False, label="D3DKMT_DRIVERVERSION"),
    "D3DKMT_DRIVER_DESCRIPTION": SimStruct(OrderedDict((("DriverDescription", SimTypeArray(SimTypeChar(label="Char"), 4096)),)), name="D3DKMT_DRIVER_DESCRIPTION", pack=False, align=None),
    "D3DKMT_ENUMADAPTERS": SimStruct(OrderedDict((("NumAdapters", SimTypeInt(signed=False, label="UInt32")), ("Adapters", SimTypeArray(SimTypeRef("D3DKMT_ADAPTERINFO", SimStruct), 16)),)), name="D3DKMT_ENUMADAPTERS", pack=False, align=None),
    "D3DKMT_ENUMADAPTERS2": SimStruct(OrderedDict((("NumAdapters", SimTypeInt(signed=False, label="UInt32")), ("pAdapters", SimTypePointer(SimTypeRef("D3DKMT_ADAPTERINFO", SimStruct), offset=0)),)), name="D3DKMT_ENUMADAPTERS2", pack=False, align=None),
    "D3DKMT_ENUMADAPTERS3": SimStruct(OrderedDict((("Filter", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("NumAdapters", SimTypeInt(signed=False, label="UInt32")), ("pAdapters", SimTypePointer(SimTypeRef("D3DKMT_ADAPTERINFO", SimStruct), offset=0)),)), name="D3DKMT_ENUMADAPTERS3", pack=False, align=None),
    "D3DKMT_ENUMADAPTERS_FILTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "D3DKMT_ESCAPE": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DKMT_ESCAPETYPE")), ("Flags", SimTypeRef("D3DDDI_ESCAPEFLAGS", SimStruct)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("hContext", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_ESCAPE", pack=False, align=None),
    "D3DKMT_ESCAPETYPE": SimTypeInt(signed=False, label="D3DKMT_ESCAPETYPE"),
    "D3DKMT_ESCAPE_PFN_CONTROL_COMMAND": SimTypeInt(signed=False, label="D3DKMT_ESCAPE_PFN_CONTROL_COMMAND"),
    "D3DKMT_ESCAPE_VIRTUAL_REFRESH_RATE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_ESCAPE_VIRTUAL_REFRESH_RATE_TYPE")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("ProcessBoostEligible", SimTypeChar(label="Byte")), ("VSyncMultiplier", SimTypeInt(signed=False, label="UInt32")), ("BaseDesktopDuration", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="D3DKMT_ESCAPE_VIRTUAL_REFRESH_RATE", pack=False, align=None),
    "D3DKMT_ESCAPE_VIRTUAL_REFRESH_RATE_TYPE": SimTypeInt(signed=False, label="D3DKMT_ESCAPE_VIRTUAL_REFRESH_RATE_TYPE"),
    "D3DKMT_EVICT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("AllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Flags", SimTypeRef("D3DDDI_EVICT_FLAGS", SimStruct)), ("NumBytesToTrim", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_EVICT", pack=False, align=None),
    "D3DKMT_EVICTION_CRITERIA": SimStruct(OrderedDict((("MinimumSize", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeRef("_Flags_e__Struct", SimStruct), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)),)), name="D3DKMT_EVICTION_CRITERIA", pack=False, align=None),
    "D3DKMT_FENCE_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("Key", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_FENCE_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_FLIPINFOFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_FLIPINFOFLAGS", pack=False, align=None),
    "D3DKMT_FLIPMANAGER_AUXILIARYPRESENTINFO": SimStruct(OrderedDict((("auxiliaryPresentInfo", SimTypeRef("D3DKMT_AUXILIARYPRESENTINFO", SimStruct)), ("flipManagerTracingId", SimTypeInt(signed=False, label="UInt32")), ("customDurationChanged", SimTypeInt(signed=True, label="Int32")), ("FlipAdapterLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("independentFlipStage", SimTypeInt(signed=False, label="D3DKMT_FLIPMODEL_INDEPENDENT_FLIP_STAGE")), ("FlipCompletedQpc", SimTypeLongLong(signed=False, label="UInt64")), ("HwPresentDurationQpc", SimTypeInt(signed=False, label="UInt32")), ("WasCanceled", SimTypeInt(signed=True, label="Int32")), ("ConvertedToNonIFlip", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_FLIPMANAGER_AUXILIARYPRESENTINFO", pack=False, align=None),
    "D3DKMT_FLIPMANAGER_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("hPrivateData", SimTypeLongLong(signed=False, label="UInt64")), ("PresentAtQpc", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_FLIPMANAGER_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_FLIPMODEL_INDEPENDENT_FLIP_STAGE": SimTypeInt(signed=False, label="D3DKMT_FLIPMODEL_INDEPENDENT_FLIP_STAGE"),
    "D3DKMT_FLIPMODEL_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("FenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("hLogicalSurface", SimTypeLongLong(signed=False, label="UInt64")), ("dxgContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("SwapChainIndex", SimTypeInt(signed=False, label="UInt32")), ("PresentLimitSemaphoreId", SimTypeLongLong(signed=False, label="UInt64")), ("FlipInterval", SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE")), ("Flags", SimTypeRef("D3DKMT_FLIPMODEL_PRESENTHISTORYTOKENFLAGS", SimStruct)), ("hCompSurf", SimTypeLongLong(signed=True, label="Int64")), ("compSurfLuid", SimTypeRef("LUID", SimStruct)), ("confirmationCookie", SimTypeLongLong(signed=False, label="UInt64")), ("CompositionSyncKey", SimTypeLongLong(signed=False, label="UInt64")), ("RemainingTokens", SimTypeInt(signed=False, label="UInt32")), ("ScrollRect", SimTypeRef("RECT", SimStruct)), ("ScrollOffset", SimTypeRef("POINT", SimStruct)), ("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("RevealColor", SimTypeArray(SimTypeFloat(size=32), 4)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("Anonymous", SimUnion({"ScatterBlts": SimTypeRef("D3DKMT_SCATTERBLTS", SimStruct), "Anonymous": SimStruct(OrderedDict((("hSyncObject", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("HDRMetaDataType", SimTypeInt(signed=False, label="D3DDDI_HDR_METADATA_TYPE")), ("Anonymous", SimUnion({"HDRMetaDataHDR10": SimTypeRef("D3DDDI_HDR_METADATA_HDR10", SimStruct), "HDRMetaDataHDR10Plus": SimTypeRef("D3DDDI_HDR_METADATA_HDR10PLUS", SimStruct)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("InkCookie", SimTypeInt(signed=False, label="UInt32")), ("SourceRect", SimTypeRef("RECT", SimStruct)), ("DestWidth", SimTypeInt(signed=False, label="UInt32")), ("DestHeight", SimTypeInt(signed=False, label="UInt32")), ("TargetRect", SimTypeRef("RECT", SimStruct)), ("Transform", SimTypeArray(SimTypeFloat(size=32), 6)), ("CustomDuration", SimTypeInt(signed=False, label="UInt32")), ("CustomDurationFlipInterval", SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE")), ("PlaneIndex", SimTypeInt(signed=False, label="UInt32")), ("ColorSpace", SimTypeInt(signed=False, label="D3DDDI_COLOR_SPACE_TYPE")), ("DirtyRegions", SimTypeRef("D3DKMT_DIRTYREGIONS", SimStruct)),)), name="D3DKMT_FLIPMODEL_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_FLIPMODEL_PRESENTHISTORYTOKENFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_FLIPMODEL_PRESENTHISTORYTOKENFLAGS", pack=False, align=None),
    "D3DKMT_FLIPOVERLAY": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hOverlay", SimTypeInt(signed=False, label="UInt32")), ("hSource", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_FLIPOVERLAY", pack=False, align=None),
    "D3DKMT_FLIPQUEUEINFO": SimStruct(OrderedDict((("MaxHardwareFlipQueueLength", SimTypeInt(signed=False, label="UInt32")), ("MaxSoftwareFlipQueueLength", SimTypeInt(signed=False, label="UInt32")), ("FlipFlags", SimTypeRef("D3DKMT_FLIPINFOFLAGS", SimStruct)),)), name="D3DKMT_FLIPQUEUEINFO", pack=False, align=None),
    "D3DKMT_FLUSHHEAPTRANSITIONS": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_FLUSHHEAPTRANSITIONS", pack=False, align=None),
    "D3DKMT_FREEGPUVIRTUALADDRESS": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_FREEGPUVIRTUALADDRESS", pack=False, align=None),
    "D3DKMT_GDIMODEL_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("hLogicalSurface", SimTypeLongLong(signed=False, label="UInt64")), ("hPhysicalSurface", SimTypeLongLong(signed=False, label="UInt64")), ("ScrollRect", SimTypeRef("RECT", SimStruct)), ("ScrollOffset", SimTypeRef("POINT", SimStruct)), ("DirtyRegions", SimTypeRef("D3DKMT_DIRTYREGIONS", SimStruct)),)), name="D3DKMT_GDIMODEL_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_GDIMODEL_SYSMEM_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("hlsurf", SimTypeLongLong(signed=False, label="UInt64")), ("dwDirtyFlags", SimTypeInt(signed=False, label="UInt32")), ("uiCookie", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_GDIMODEL_SYSMEM_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_GETALLOCATIONPRIORITY": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("phAllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")), ("pPriorities", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_GETALLOCATIONPRIORITY", pack=False, align=None),
    "D3DKMT_GETCONTEXTINPROCESSSCHEDULINGPRIORITY": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_GETCONTEXTINPROCESSSCHEDULINGPRIORITY", pack=False, align=None),
    "D3DKMT_GETCONTEXTSCHEDULINGPRIORITY": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_GETCONTEXTSCHEDULINGPRIORITY", pack=False, align=None),
    "D3DKMT_GETDEVICESTATE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("StateType", SimTypeInt(signed=False, label="D3DKMT_DEVICESTATE_TYPE")), ("Anonymous", SimUnion({"ExecutionState": SimTypeInt(signed=False, label="D3DKMT_DEVICEEXECUTION_STATE"), "PresentState": SimTypeRef("D3DKMT_DEVICEPRESENT_STATE", SimStruct), "ResetState": SimTypeRef("D3DKMT_DEVICERESET_STATE", SimStruct), "PresentStateDWM": SimTypeRef("D3DKMT_DEVICEPRESENT_STATE_DWM", SimStruct), "PageFaultState": SimTypeRef("D3DKMT_DEVICEPAGEFAULT_STATE", SimStruct), "PresentQueueState": SimTypeRef("D3DKMT_DEVICEPRESENT_QUEUE_STATE", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_GETDEVICESTATE", pack=False, align=None),
    "D3DKMT_GETDISPLAYMODELIST": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("pModeList", SimTypePointer(SimTypeRef("D3DKMT_DISPLAYMODE", SimStruct), offset=0)), ("ModeCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GETDISPLAYMODELIST", pack=False, align=None),
    "D3DKMT_GETMULTISAMPLEMETHODLIST": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("pMethodList", SimTypePointer(SimTypeRef("D3DKMT_MULTISAMPLEMETHOD", SimStruct), offset=0)), ("MethodCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GETMULTISAMPLEMETHODLIST", pack=False, align=None),
    "D3DKMT_GETOVERLAYSTATE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hOverlay", SimTypeInt(signed=False, label="UInt32")), ("OverlayEnabled", SimTypeChar(label="Byte")),)), name="D3DKMT_GETOVERLAYSTATE", pack=False, align=None),
    "D3DKMT_GETPRESENTHISTORY": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("ProvidedSize", SimTypeInt(signed=False, label="UInt32")), ("WrittenSize", SimTypeInt(signed=False, label="UInt32")), ("pTokens", SimTypePointer(SimTypeRef("D3DKMT_PRESENTHISTORYTOKEN", SimStruct), offset=0)), ("NumTokens", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GETPRESENTHISTORY", pack=False, align=None),
    "D3DKMT_GETPROCESSDEVICEREMOVALSUPPORT": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("Support", SimTypeChar(label="Byte")),)), name="D3DKMT_GETPROCESSDEVICEREMOVALSUPPORT", pack=False, align=None),
    "D3DKMT_GETRUNTIMEDATA": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hGlobalShare", SimTypeInt(signed=False, label="UInt32")), ("pRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RuntimeDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GETRUNTIMEDATA", pack=False, align=None),
    "D3DKMT_GETSCANLINE": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("InVerticalBlank", SimTypeChar(label="Byte")), ("ScanLine", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GETSCANLINE", pack=False, align=None),
    "D3DKMT_GETSHAREDPRIMARYHANDLE": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("hSharedPrimary", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GETSHAREDPRIMARYHANDLE", pack=False, align=None),
    "D3DKMT_GETSHAREDRESOURCEADAPTERLUID": SimStruct(OrderedDict((("hGlobalShare", SimTypeInt(signed=False, label="UInt32")), ("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AdapterLuid", SimTypeRef("LUID", SimStruct)),)), name="D3DKMT_GETSHAREDRESOURCEADAPTERLUID", pack=False, align=None),
    "D3DKMT_GETVERTICALBLANKEVENT": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("phEvent", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="D3DKMT_GETVERTICALBLANKEVENT", pack=False, align=None),
    "D3DKMT_GET_DEVICE_VIDPN_OWNERSHIP_INFO": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("bFailedDwmAcquireVidPn", SimTypeChar(label="Byte")),)), name="D3DKMT_GET_DEVICE_VIDPN_OWNERSHIP_INFO", pack=False, align=None),
    "D3DKMT_GET_GPUMMU_CAPS": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("GpuMmuCaps", SimTypeRef("DXGK_ESCAPE_GPUMMUCAPS", SimStruct)),)), name="D3DKMT_GET_GPUMMU_CAPS", pack=False, align=None),
    "D3DKMT_GET_MULTIPLANE_OVERLAY_CAPS": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("MaxPlanes", SimTypeInt(signed=False, label="UInt32")), ("MaxRGBPlanes", SimTypeInt(signed=False, label="UInt32")), ("MaxYUVPlanes", SimTypeInt(signed=False, label="UInt32")), ("OverlayCaps", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_CAPS", SimStruct)), ("MaxStretchFactor", SimTypeFloat(size=32)), ("MaxShrinkFactor", SimTypeFloat(size=32)),)), name="D3DKMT_GET_MULTIPLANE_OVERLAY_CAPS", pack=False, align=None),
    "D3DKMT_GET_POST_COMPOSITION_CAPS": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("MaxStretchFactor", SimTypeFloat(size=32)), ("MaxShrinkFactor", SimTypeFloat(size=32)),)), name="D3DKMT_GET_POST_COMPOSITION_CAPS", pack=False, align=None),
    "D3DKMT_GET_PTE": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("PageTableLevel", SimTypeInt(signed=False, label="UInt32")), ("PageTableIndex", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)), ("b64KBPte", SimTypeChar(label="Byte")), ("NumPtes", SimTypeInt(signed=False, label="UInt32")), ("Pte", SimTypeArray(SimTypeRef("DXGK_PTE", SimStruct), 64)), ("NumValidEntries", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GET_PTE", pack=False, align=None),
    "D3DKMT_GET_SEGMENT_CAPS": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("NumSegments", SimTypeInt(signed=False, label="UInt32")), ("SegmentCaps", SimTypeArray(SimTypeRef("D3DKMT_SEGMENT_CAPS", SimStruct), 32)),)), name="D3DKMT_GET_SEGMENT_CAPS", pack=False, align=None),
    "D3DKMT_GPUMMU_CAPS": SimStruct(OrderedDict((("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("VirtualAddressBitCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_GPUMMU_CAPS", pack=False, align=None),
    "D3DKMT_GPUVERSION": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("BiosVersion", SimTypeArray(SimTypeChar(label="Char"), 32)), ("GpuArchitecture", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="D3DKMT_GPUVERSION", pack=False, align=None),
    "D3DKMT_GPU_PREFERENCE_QUERY_STATE": SimTypeInt(signed=False, label="D3DKMT_GPU_PREFERENCE_QUERY_STATE"),
    "D3DKMT_GPU_PREFERENCE_QUERY_TYPE": SimTypeInt(signed=False, label="D3DKMT_GPU_PREFERENCE_QUERY_TYPE"),
    "D3DKMT_HISTORY_BUFFER_STATUS": SimStruct(OrderedDict((("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_HISTORY_BUFFER_STATUS", pack=False, align=None),
    "D3DKMT_HWDRM_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeChar(label="Byte")),)), name="D3DKMT_HWDRM_SUPPORT", pack=False, align=None),
    "D3DKMT_HYBRID_DLIST_DLL_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_HYBRID_DLIST_DLL_SUPPORT", pack=False, align=None),
    "D3DKMT_HYBRID_LIST": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="D3DKMT_GPU_PREFERENCE_QUERY_STATE")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("bUserPreferenceQuery", SimTypeInt(signed=True, label="Int32")), ("QueryType", SimTypeInt(signed=False, label="D3DKMT_GPU_PREFERENCE_QUERY_TYPE")),)), name="D3DKMT_HYBRID_LIST", pack=False, align=None),
    "D3DKMT_INDEPENDENTFLIP_SECONDARY_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_INDEPENDENTFLIP_SECONDARY_SUPPORT", pack=False, align=None),
    "D3DKMT_INDEPENDENTFLIP_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_INDEPENDENTFLIP_SUPPORT", pack=False, align=None),
    "D3DKMT_INVALIDATEACTIVEVIDPN": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_INVALIDATEACTIVEVIDPN", pack=False, align=None),
    "D3DKMT_INVALIDATECACHE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Length", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3DKMT_INVALIDATECACHE", pack=False, align=None),
    "D3DKMT_ISBADDRIVERFORHWPROTECTIONDISABLED": SimStruct(OrderedDict((("Disabled", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_ISBADDRIVERFORHWPROTECTIONDISABLED", pack=False, align=None),
    "D3DKMT_KMD_DRIVER_VERSION": SimStruct(OrderedDict((("DriverVersion", SimTypeLongLong(signed=True, label="Int64")),)), name="D3DKMT_KMD_DRIVER_VERSION", pack=False, align=None),
    "D3DKMT_LOCK": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverData", SimTypeInt(signed=False, label="UInt32")), ("NumPages", SimTypeInt(signed=False, label="UInt32")), ("pPages", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeRef("D3DDDICB_LOCKFLAGS", SimStruct)), ("GpuVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_LOCK", pack=False, align=None),
    "D3DKMT_LOCK2": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDICB_LOCK2FLAGS", SimStruct)), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DKMT_LOCK2", pack=False, align=None),
    "D3DKMT_MARKDEVICEASERROR": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Reason", SimTypeInt(signed=False, label="D3DKMT_DEVICE_ERROR_REASON")),)), name="D3DKMT_MARKDEVICEASERROR", pack=False, align=None),
    "D3DKMT_MEMORY_SEGMENT_GROUP": SimTypeInt(signed=False, label="D3DKMT_MEMORY_SEGMENT_GROUP"),
    "D3DKMT_MIRACASTCOMPANIONDRIVERNAME": SimStruct(OrderedDict((("MiracastCompanionDriverName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_MIRACASTCOMPANIONDRIVERNAME", pack=False, align=None),
    "D3DKMT_MIRACAST_CHUNK_DATA": SimStruct(OrderedDict((("ChunkInfo", SimTypeRef("DXGK_MIRACAST_CHUNK_INFO", SimStruct)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3DKMT_MIRACAST_CHUNK_DATA", pack=False, align=None),
    "D3DKMT_MIRACAST_DEVICE_STATUS": SimTypeInt(signed=False, label="D3DKMT_MIRACAST_DEVICE_STATUS"),
    "D3DKMT_MIRACAST_DISPLAY_DEVICE_CAPS": SimStruct(OrderedDict((("HdcpSupported", SimTypeChar(label="Byte")), ("DefaultControlPort", SimTypeInt(signed=False, label="UInt32")), ("UsesIhvSolution", SimTypeChar(label="Byte")),)), name="D3DKMT_MIRACAST_DISPLAY_DEVICE_CAPS", pack=False, align=None),
    "D3DKMT_MIRACAST_DISPLAY_DEVICE_STATE": SimTypeInt(signed=False, label="D3DKMT_MIRACAST_DISPLAY_DEVICE_STATE"),
    "D3DKMT_MIRACAST_DISPLAY_DEVICE_STATUS": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="D3DKMT_MIRACAST_DISPLAY_DEVICE_STATE")),)), name="D3DKMT_MIRACAST_DISPLAY_DEVICE_STATUS", pack=False, align=None),
    "D3DKMT_MIRACAST_DISPLAY_STOP_SESSIONS": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("TargetId", SimTypeInt(signed=False, label="UInt32")), ("StopReason", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_MIRACAST_DISPLAY_STOP_SESSIONS", pack=False, align=None),
    "D3DKMT_MIRACAST_DRIVER_TYPE": SimTypeInt(signed=False, label="D3DKMT_MIRACAST_DRIVER_TYPE"),
    "D3DKMT_MOVE_RECT": SimStruct(OrderedDict((("SourcePoint", SimTypeRef("POINT", SimStruct)), ("DestRect", SimTypeRef("RECT", SimStruct)),)), name="D3DKMT_MOVE_RECT", pack=False, align=None),
    "D3DKMT_MPO3DDI_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MPO3DDI_SUPPORT", pack=False, align=None),
    "D3DKMT_MPOKERNELCAPS_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MPOKERNELCAPS_SUPPORT", pack=False, align=None),
    "D3DKMT_MULTIPLANEOVERLAY_DECODE_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MULTIPLANEOVERLAY_DECODE_SUPPORT", pack=False, align=None),
    "D3DKMT_MULTIPLANEOVERLAY_HUD_SUPPORT": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Update", SimTypeInt(signed=True, label="Int32")), ("KernelSupported", SimTypeInt(signed=True, label="Int32")), ("HudSupported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MULTIPLANEOVERLAY_HUD_SUPPORT", pack=False, align=None),
    "D3DKMT_MULTIPLANEOVERLAY_SECONDARY_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MULTIPLANEOVERLAY_SECONDARY_SUPPORT", pack=False, align=None),
    "D3DKMT_MULTIPLANEOVERLAY_STRETCH_SUPPORT": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Update", SimTypeInt(signed=True, label="Int32")), ("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MULTIPLANEOVERLAY_STRETCH_SUPPORT", pack=False, align=None),
    "D3DKMT_MULTIPLANEOVERLAY_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_MULTIPLANEOVERLAY_SUPPORT", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY": SimStruct(OrderedDict((("LayerIndex", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("PlaneAttributes", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES", SimStruct)),)), name="D3DKMT_MULTIPLANE_OVERLAY", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY2": SimStruct(OrderedDict((("LayerIndex", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("PlaneAttributes", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES2", SimStruct)),)), name="D3DKMT_MULTIPLANE_OVERLAY2", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY3": SimStruct(OrderedDict((("LayerIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFlags", SimTypeRef("D3DKMT_PLANE_SPECIFIC_INPUT_FLAGS", SimStruct)), ("FlipInterval", SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE")), ("MaxImmediateFlipLine", SimTypeInt(signed=False, label="UInt32")), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")), ("pAllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("DriverPrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("pDriverPrivateData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pPlaneAttributes", SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES3", SimStruct), offset=0)), ("hFlipToFence", SimTypeInt(signed=False, label="UInt32")), ("hFlipAwayFence", SimTypeInt(signed=False, label="UInt32")), ("FlipToFenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("FlipAwayFenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_MULTIPLANE_OVERLAY3", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("DstRect", SimTypeRef("RECT", SimStruct)), ("ClipRect", SimTypeRef("RECT", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("Blend", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_BLEND")), ("DirtyRectCount", SimTypeInt(signed=False, label="UInt32")), ("pDirtyRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("VideoFrameFormat", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_VIDEO_FRAME_FORMAT")), ("YCbCrFlags", SimTypeInt(signed=False, label="UInt32")), ("StereoFormat", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_STEREO_FORMAT")), ("StereoLeftViewFrame0", SimTypeInt(signed=True, label="Int32")), ("StereoBaseViewFrame0", SimTypeInt(signed=True, label="Int32")), ("StereoFlipMode", SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STEREO_FLIP_MODE")), ("StretchQuality", SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STRETCH_QUALITY")),)), name="D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES2": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("DstRect", SimTypeRef("RECT", SimStruct)), ("ClipRect", SimTypeRef("RECT", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("Blend", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_BLEND")), ("DirtyRectCount", SimTypeInt(signed=False, label="UInt32")), ("pDirtyRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("VideoFrameFormat", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_VIDEO_FRAME_FORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="D3DDDI_COLOR_SPACE_TYPE")), ("StereoFormat", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_STEREO_FORMAT")), ("StereoLeftViewFrame0", SimTypeInt(signed=True, label="Int32")), ("StereoBaseViewFrame0", SimTypeInt(signed=True, label="Int32")), ("StereoFlipMode", SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STEREO_FLIP_MODE")), ("StretchQuality", SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STRETCH_QUALITY")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES2", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES3": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("DstRect", SimTypeRef("RECT", SimStruct)), ("ClipRect", SimTypeRef("RECT", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("Blend", SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_BLEND")), ("DirtyRectCount", SimTypeInt(signed=False, label="UInt32")), ("pDirtyRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("ColorSpace", SimTypeInt(signed=False, label="D3DDDI_COLOR_SPACE_TYPE")), ("StretchQuality", SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STRETCH_QUALITY")), ("SDRWhiteLevel", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_MULTIPLANE_OVERLAY_ATTRIBUTES3", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_BLEND": SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_BLEND"),
    "D3DKMT_MULTIPLANE_OVERLAY_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_MULTIPLANE_OVERLAY_CAPS", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_FLAGS": SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_FLAGS"),
    "D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION": SimStruct(OrderedDict((("Flags", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION_FLAGS", SimStruct)), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("DstRect", SimTypeRef("RECT", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")),)), name="D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION_FLAGS", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION_WITH_SOURCE": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PostComposition", SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION", SimStruct)),)), name="D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION_WITH_SOURCE", pack=False, align=None),
    "D3DKMT_MULTIPLANE_OVERLAY_STEREO_FORMAT": SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_STEREO_FORMAT"),
    "D3DKMT_MULTIPLANE_OVERLAY_VIDEO_FRAME_FORMAT": SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_VIDEO_FRAME_FORMAT"),
    "D3DKMT_MULTIPLANE_OVERLAY_YCbCr_FLAGS": SimTypeInt(signed=False, label="D3DKMT_MULTIPLANE_OVERLAY_YCbCr_FLAGS"),
    "D3DKMT_MULTISAMPLEMETHOD": SimStruct(OrderedDict((("NumSamples", SimTypeInt(signed=False, label="UInt32")), ("NumQualityLevels", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_MULTISAMPLEMETHOD", pack=False, align=None),
    "D3DKMT_NODEMETADATA": SimStruct(OrderedDict((("NodeOrdinalAndAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("NodeData", SimTypeRef("DXGK_NODEMETADATA", SimStruct)),)), name="D3DKMT_NODEMETADATA", pack=False, align=None),
    "D3DKMT_NODE_PERFDATA": SimStruct(OrderedDict((("NodeOrdinal", SimTypeInt(signed=False, label="UInt32")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("Frequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxFrequencyOC", SimTypeLongLong(signed=False, label="UInt64")), ("Voltage", SimTypeInt(signed=False, label="UInt32")), ("VoltageMax", SimTypeInt(signed=False, label="UInt32")), ("VoltageMaxOC", SimTypeInt(signed=False, label="UInt32")), ("MaxTransitionLatency", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_NODE_PERFDATA", pack=False, align=None),
    "D3DKMT_NOTIFY_WORK_SUBMISSION": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_NOTIFY_WORK_SUBMISSION_FLAGS", SimStruct)),)), name="D3DKMT_NOTIFY_WORK_SUBMISSION", pack=False, align=None),
    "D3DKMT_NOTIFY_WORK_SUBMISSION_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_NOTIFY_WORK_SUBMISSION_FLAGS", pack=False, align=None),
    "D3DKMT_OFFERALLOCATIONS": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("pResources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("HandleList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="D3DKMT_OFFER_PRIORITY")), ("Flags", SimTypeRef("D3DKMT_OFFER_FLAGS", SimStruct)),)), name="D3DKMT_OFFERALLOCATIONS", pack=False, align=None),
    "D3DKMT_OFFER_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_OFFER_FLAGS", pack=False, align=None),
    "D3DKMT_OFFER_PRIORITY": SimTypeInt(signed=False, label="D3DKMT_OFFER_PRIORITY"),
    "D3DKMT_OPENADAPTERFROMDEVICENAME": SimStruct(OrderedDict((("pDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)),)), name="D3DKMT_OPENADAPTERFROMDEVICENAME", pack=False, align=None),
    "D3DKMT_OPENADAPTERFROMGDIDISPLAYNAME": SimStruct(OrderedDict((("DeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENADAPTERFROMGDIDISPLAYNAME", pack=False, align=None),
    "D3DKMT_OPENADAPTERFROMHDC": SimStruct(OrderedDict((("hDc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENADAPTERFROMHDC", pack=False, align=None),
    "D3DKMT_OPENADAPTERFROMLUID": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENADAPTERFROMLUID", pack=False, align=None),
    "D3DKMT_OPENGLINFO": SimStruct(OrderedDict((("UmdOpenGlIcdFileName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENGLINFO", pack=False, align=None),
    "D3DKMT_OPENKEYEDMUTEX": SimStruct(OrderedDict((("hSharedHandle", SimTypeInt(signed=False, label="UInt32")), ("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENKEYEDMUTEX", pack=False, align=None),
    "D3DKMT_OPENKEYEDMUTEX2": SimStruct(OrderedDict((("hSharedHandle", SimTypeInt(signed=False, label="UInt32")), ("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENKEYEDMUTEX2", pack=False, align=None),
    "D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE": SimStruct(OrderedDict((("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_OPENNATIVEFENCEFROMNTHANDLE": SimStruct(OrderedDict((("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS", SimStruct)), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("NativeFenceMapping", SimTypeRef("D3DDDI_NATIVEFENCEMAPPING", SimStruct)),)), name="D3DKMT_OPENNATIVEFENCEFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_OPENNTHANDLEFROMNAME": SimStruct(OrderedDict((("dwDesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("pObjAttrib", SimTypePointer(SimTypeRef("OBJECT_ATTRIBUTES", SimStruct), offset=0)), ("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_OPENNTHANDLEFROMNAME", pack=False, align=None),
    "D3DKMT_OPENPROTECTEDSESSIONFROMNTHANDLE": SimStruct(OrderedDict((("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENPROTECTEDSESSIONFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_OPENRESOURCE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hGlobalShare", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pOpenAllocationInfo": SimTypePointer(SimTypeRef("D3DDDI_OPENALLOCATIONINFO", SimStruct), offset=0), "pOpenAllocationInfo2": SimTypePointer(SimTypeRef("D3DDDI_OPENALLOCATIONINFO2", SimStruct), offset=0)}, name="<anon>", label="None")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("pResourcePrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ResourcePrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pTotalPrivateDriverDataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TotalPrivateDriverDataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENRESOURCE", pack=False, align=None),
    "D3DKMT_OPENRESOURCEFROMNTHANDLE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("pOpenAllocationInfo2", SimTypePointer(SimTypeRef("D3DDDI_OPENALLOCATIONINFO2", SimStruct), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ResourcePrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pResourcePrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TotalPrivateDriverDataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("pTotalPrivateDriverDataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("pKeyedMutexPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeyedMutexPrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENRESOURCEFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_OPENSYNCHRONIZATIONOBJECT": SimStruct(OrderedDict((("hSharedHandle", SimTypeInt(signed=False, label="UInt32")), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="D3DKMT_OPENSYNCHRONIZATIONOBJECT", pack=False, align=None),
    "D3DKMT_OPENSYNCOBJECTFROMNTHANDLE": SimStruct(OrderedDict((("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OPENSYNCOBJECTFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_OPENSYNCOBJECTFROMNTHANDLE2": SimStruct(OrderedDict((("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS", SimStruct)), ("hSyncObject", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"MonitoredFence": SimTypeRef("_MonitoredFence_e__Struct", SimStruct), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)}, name="<anon>", label="None")),)), name="D3DKMT_OPENSYNCOBJECTFROMNTHANDLE2", pack=False, align=None),
    "D3DKMT_OPENSYNCOBJECTNTHANDLEFROMNAME": SimStruct(OrderedDict((("dwDesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("pObjAttrib", SimTypePointer(SimTypeRef("OBJECT_ATTRIBUTES", SimStruct), offset=0)), ("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_OPENSYNCOBJECTNTHANDLEFROMNAME", pack=False, align=None),
    "D3DKMT_OUTDUPL_POINTER_SHAPE_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_OUTDUPL_POINTER_SHAPE_TYPE")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Pitch", SimTypeInt(signed=False, label="UInt32")), ("HotSpot", SimTypeRef("POINT", SimStruct)),)), name="D3DKMT_OUTDUPL_POINTER_SHAPE_INFO", pack=False, align=None),
    "D3DKMT_OUTDUPL_POINTER_SHAPE_TYPE": SimTypeInt(signed=False, label="D3DKMT_OUTDUPL_POINTER_SHAPE_TYPE"),
    "D3DKMT_OUTPUTDUPLCONTEXTSCOUNT": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("OutputDuplicationCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OUTPUTDUPLCONTEXTSCOUNT", pack=False, align=None),
    "D3DKMT_OUTPUTDUPLCREATIONFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_OUTPUTDUPLCREATIONFLAGS", pack=False, align=None),
    "D3DKMT_OUTPUTDUPLPRESENT": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("hSource", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("PresentRegions", SimTypeRef("D3DKMT_PRESENT_RGNS", SimStruct)), ("Flags", SimTypeRef("D3DKMT_OUTPUTDUPLPRESENTFLAGS", SimStruct)), ("hIndirectContext", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OUTPUTDUPLPRESENT", pack=False, align=None),
    "D3DKMT_OUTPUTDUPLPRESENTFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_OUTPUTDUPLPRESENTFLAGS", pack=False, align=None),
    "D3DKMT_OUTPUTDUPLPRESENTTOHWQUEUE": SimStruct(OrderedDict((("hSource", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("BroadcastHwQueueCount", SimTypeInt(signed=False, label="UInt32")), ("hHwQueues", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("PresentRegions", SimTypeRef("D3DKMT_PRESENT_RGNS", SimStruct)), ("Flags", SimTypeRef("D3DKMT_OUTPUTDUPLPRESENTFLAGS", SimStruct)), ("hIndirectHwQueue", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OUTPUTDUPLPRESENTTOHWQUEUE", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_FRAMEINFO": SimStruct(OrderedDict((("LastPresentTime", SimTypeLongLong(signed=True, label="Int64")), ("LastMouseUpdateTime", SimTypeLongLong(signed=True, label="Int64")), ("AccumulatedFrames", SimTypeInt(signed=False, label="UInt32")), ("RectsCoalesced", SimTypeInt(signed=True, label="Int32")), ("ProtectedContentMaskedOut", SimTypeInt(signed=True, label="Int32")), ("PointerPosition", SimTypeRef("D3DKMT_OUTPUTDUPL_POINTER_POSITION", SimStruct)), ("TotalMetadataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("PointerShapeBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OUTPUTDUPL_FRAMEINFO", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_GET_FRAMEINFO": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("FrameInfo", SimTypeRef("D3DKMT_OUTPUTDUPL_FRAMEINFO", SimStruct)),)), name="D3DKMT_OUTPUTDUPL_GET_FRAMEINFO", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_GET_POINTER_SHAPE_DATA": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("BufferSizeSupplied", SimTypeInt(signed=False, label="UInt32")), ("pShapeBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSizeRequired", SimTypeInt(signed=False, label="UInt32")), ("ShapeInfo", SimTypeRef("D3DKMT_OUTDUPL_POINTER_SHAPE_INFO", SimStruct)),)), name="D3DKMT_OUTPUTDUPL_GET_POINTER_SHAPE_DATA", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_KEYEDMUTEX": SimStruct(OrderedDict((("hSharedSurfaceNt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="D3DKMT_OUTPUTDUPL_KEYEDMUTEX", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_METADATA": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DKMT_OUTPUTDUPL_METADATATYPE")), ("BufferSizeSupplied", SimTypeInt(signed=False, label="UInt32")), ("pBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSizeRequired", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OUTPUTDUPL_METADATA", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_METADATATYPE": SimTypeInt(signed=False, label="D3DKMT_OUTPUTDUPL_METADATATYPE"),
    "D3DKMT_OUTPUTDUPL_POINTER_POSITION": SimStruct(OrderedDict((("Position", SimTypeRef("POINT", SimStruct)), ("Visible", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_OUTPUTDUPL_POINTER_POSITION", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_RELEASE_FRAME": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("NextKeyMutexIdx", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_OUTPUTDUPL_RELEASE_FRAME", pack=False, align=None),
    "D3DKMT_OUTPUTDUPL_SNAPSHOT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("SessionProcessCount", SimTypeInt(signed=False, label="UInt32")), ("SessionActiveConnectionsCount", SimTypeInt(signed=False, label="UInt32")), ("NumVidPnSources", SimTypeInt(signed=False, label="UInt32")), ("NumOutputDuplContexts", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeInt(signed=False, label="UInt32")), ("OutputDuplDebugInfos", SimTypePointer(SimTypeRef("OUTPUTDUPL_CONTEXT_DEBUG_INFO", SimStruct), offset=0)),)), name="D3DKMT_OUTPUTDUPL_SNAPSHOT", pack=False, align=None),
    "D3DKMT_PAGE_TABLE_LEVEL_DESC": SimStruct(OrderedDict((("IndexBitCount", SimTypeInt(signed=False, label="UInt32")), ("IndexMask", SimTypeLongLong(signed=False, label="UInt64")), ("IndexShift", SimTypeLongLong(signed=False, label="UInt64")), ("LowerLevelsMask", SimTypeLongLong(signed=False, label="UInt64")), ("EntryCoverageInPages", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_PAGE_TABLE_LEVEL_DESC", pack=False, align=None),
    "D3DKMT_PANELFITTER_SUPPORT": SimStruct(OrderedDict((("Supported", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_PANELFITTER_SUPPORT", pack=False, align=None),
    "D3DKMT_PARAVIRTUALIZATION": SimStruct(OrderedDict((("SecureContainer", SimTypeChar(label="Byte")),)), name="D3DKMT_PARAVIRTUALIZATION", pack=False, align=None),
    "D3DKMT_PHYSICAL_ADAPTER_COUNT": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_PHYSICAL_ADAPTER_COUNT", pack=False, align=None),
    "D3DKMT_PINDIRECTFLIPRESOURCES": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("ResourceCount", SimTypeInt(signed=False, label="UInt32")), ("pResourceList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_PINDIRECTFLIPRESOURCES", pack=False, align=None),
    "D3DKMT_PLANE_SPECIFIC_INPUT_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_PLANE_SPECIFIC_INPUT_FLAGS", pack=False, align=None),
    "D3DKMT_PLANE_SPECIFIC_OUTPUT_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_PLANE_SPECIFIC_OUTPUT_FLAGS", pack=False, align=None),
    "D3DKMT_PNP_KEY_TYPE": SimTypeInt(signed=False, label="D3DKMT_PNP_KEY_TYPE"),
    "D3DKMT_POLLDISPLAYCHILDREN": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_POLLDISPLAYCHILDREN", pack=False, align=None),
    "D3DKMT_PRESENT": SimStruct(OrderedDict((("Anonymous1", SimUnion({"hDevice": SimTypeInt(signed=False, label="UInt32"), "hContext": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("hWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("hSource", SimTypeInt(signed=False, label="UInt32")), ("hDestination", SimTypeInt(signed=False, label="UInt32")), ("Color", SimTypeInt(signed=False, label="UInt32")), ("DstRect", SimTypeRef("RECT", SimStruct)), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("SubRectCnt", SimTypeInt(signed=False, label="UInt32")), ("pSrcSubRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("FlipInterval", SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE")), ("Flags", SimTypeRef("D3DKMT_PRESENTFLAGS", SimStruct)), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("PresentLimitSemaphore", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PresentHistoryToken", SimTypeRef("D3DKMT_PRESENTHISTORYTOKEN", SimStruct)), ("pPresentRegions", SimTypePointer(SimTypeRef("D3DKMT_PRESENT_RGNS", SimStruct), offset=0)), ("Anonymous2", SimUnion({"hAdapter": SimTypeInt(signed=False, label="UInt32"), "hIndirectContext": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Duration", SimTypeInt(signed=False, label="UInt32")), ("BroadcastSrcAllocation", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("BroadcastDstAllocation", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("bOptimizeForComposition", SimTypeChar(label="Byte")),)), name="D3DKMT_PRESENT", pack=False, align=None),
    "D3DKMT_PRESENTFLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_PRESENTFLAGS", pack=False, align=None),
    "D3DKMT_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("Model", SimTypeInt(signed=False, label="D3DKMT_PRESENT_MODEL")), ("TokenSize", SimTypeInt(signed=False, label="UInt32")), ("CompositionBindingId", SimTypeLongLong(signed=False, label="UInt64")), ("Token", SimUnion({"Flip": SimTypeRef("D3DKMT_FLIPMODEL_PRESENTHISTORYTOKEN", SimStruct), "Blt": SimTypeRef("D3DKMT_BLTMODEL_PRESENTHISTORYTOKEN", SimStruct), "VistaBlt": SimTypeLongLong(signed=False, label="UInt64"), "Gdi": SimTypeRef("D3DKMT_GDIMODEL_PRESENTHISTORYTOKEN", SimStruct), "Fence": SimTypeRef("D3DKMT_FENCE_PRESENTHISTORYTOKEN", SimStruct), "GdiSysMem": SimTypeRef("D3DKMT_GDIMODEL_SYSMEM_PRESENTHISTORYTOKEN", SimStruct), "Composition": SimTypeRef("D3DKMT_COMPOSITION_PRESENTHISTORYTOKEN", SimStruct), "FlipManager": SimTypeRef("D3DKMT_FLIPMANAGER_PRESENTHISTORYTOKEN", SimStruct), "SurfaceComplete": SimTypeRef("D3DKMT_SURFACECOMPLETE_PRESENTHISTORYTOKEN", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_PRESENT_MODEL": SimTypeInt(signed=False, label="D3DKMT_PRESENT_MODEL"),
    "D3DKMT_PRESENT_MULTIPLANE_OVERLAY": SimStruct(OrderedDict((("Anonymous", SimUnion({"hDevice": SimTypeInt(signed=False, label="UInt32"), "hContext": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("FlipInterval", SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE")), ("Flags", SimTypeRef("D3DKMT_PRESENTFLAGS", SimStruct)), ("PresentPlaneCount", SimTypeInt(signed=False, label="UInt32")), ("pPresentPlanes", SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY", SimStruct), offset=0)), ("Duration", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_PRESENT_MULTIPLANE_OVERLAY", pack=False, align=None),
    "D3DKMT_PRESENT_MULTIPLANE_OVERLAY2": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hDevice": SimTypeInt(signed=False, label="UInt32"), "hContext": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("FlipInterval", SimTypeInt(signed=False, label="D3DDDI_FLIPINTERVAL_TYPE")), ("Flags", SimTypeRef("D3DKMT_PRESENTFLAGS", SimStruct)), ("PresentPlaneCount", SimTypeInt(signed=False, label="UInt32")), ("pPresentPlanes", SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY2", SimStruct), offset=0)), ("Duration", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_PRESENT_MULTIPLANE_OVERLAY2", pack=False, align=None),
    "D3DKMT_PRESENT_MULTIPLANE_OVERLAY3": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("ContextCount", SimTypeInt(signed=False, label="UInt32")), ("pContextList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_PRESENT_MULTIPLANE_OVERLAY_FLAGS", SimStruct)), ("PresentPlaneCount", SimTypeInt(signed=False, label="UInt32")), ("ppPresentPlanes", SimTypePointer(SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY3", SimStruct), offset=0), offset=0)), ("pPostComposition", SimTypePointer(SimTypeRef("D3DKMT_MULTIPLANE_OVERLAY_POST_COMPOSITION", SimStruct), offset=0)), ("Duration", SimTypeInt(signed=False, label="UInt32")), ("HDRMetaDataType", SimTypeInt(signed=False, label="D3DDDI_HDR_METADATA_TYPE")), ("HDRMetaDataSize", SimTypeInt(signed=False, label="UInt32")), ("pHDRMetaData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BoostRefreshRateMultiplier", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_PRESENT_MULTIPLANE_OVERLAY3", pack=False, align=None),
    "D3DKMT_PRESENT_MULTIPLANE_OVERLAY_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_PRESENT_MULTIPLANE_OVERLAY_FLAGS", pack=False, align=None),
    "D3DKMT_PRESENT_REDIRECTED": SimStruct(OrderedDict((("hSyncObj", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("WaitedFenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("PresentHistoryToken", SimTypeRef("D3DKMT_PRESENTHISTORYTOKEN", SimStruct)), ("Flags", SimTypeRef("D3DKMT_PRESENT_REDIRECTED_FLAGS", SimStruct)), ("hSource", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DKMT_PRESENT_REDIRECTED", pack=False, align=None),
    "D3DKMT_PRESENT_REDIRECTED_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_PRESENT_REDIRECTED_FLAGS", pack=False, align=None),
    "D3DKMT_PRESENT_RGNS": SimStruct(OrderedDict((("DirtyRectCount", SimTypeInt(signed=False, label="UInt32")), ("pDirtyRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("MoveRectCount", SimTypeInt(signed=False, label="UInt32")), ("pMoveRects", SimTypePointer(SimTypeRef("D3DKMT_MOVE_RECT", SimStruct), offset=0)),)), name="D3DKMT_PRESENT_RGNS", pack=False, align=None),
    "D3DKMT_PRESENT_STATS": SimStruct(OrderedDict((("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("PresentRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("SyncGPUTime", SimTypeLongLong(signed=True, label="Int64")),)), name="D3DKMT_PRESENT_STATS", pack=False, align=None),
    "D3DKMT_PRESENT_STATS_DWM": SimStruct(OrderedDict((("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("PresentRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("PresentQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("SyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("CustomPresentDuration", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_PRESENT_STATS_DWM", pack=False, align=None),
    "D3DKMT_PRESENT_STATS_DWM2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("PresentRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("PresentQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("SyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("CustomPresentDuration", SimTypeInt(signed=False, label="UInt32")), ("VirtualSyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("VirtualSyncQPCTime", SimTypeLongLong(signed=True, label="Int64")),)), name="D3DKMT_PRESENT_STATS_DWM2", pack=False, align=None),
    "D3DKMT_PROCESS_VERIFIER_OPTION": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Type", SimTypeInt(signed=False, label="D3DKMT_PROCESS_VERIFIER_OPTION_TYPE")), ("Mode", SimTypeInt(signed=False, label="D3DKMT_VERIFIER_OPTION_MODE")), ("Data", SimUnion({"VidMmFlags": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VidMmRestrictBudget": SimTypeRef("D3DKMT_PROCESS_VERIFIER_VIDMM_RESTRICT_BUDGET", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_PROCESS_VERIFIER_OPTION", pack=False, align=None),
    "D3DKMT_PROCESS_VERIFIER_OPTION_DATA": SimUnion({"VidMmFlags": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VidMmRestrictBudget": SimTypeRef("D3DKMT_PROCESS_VERIFIER_VIDMM_RESTRICT_BUDGET", SimStruct)}, name="<anon>", label="None"),
    "D3DKMT_PROCESS_VERIFIER_OPTION_TYPE": SimTypeInt(signed=False, label="D3DKMT_PROCESS_VERIFIER_OPTION_TYPE"),
    "D3DKMT_PROCESS_VERIFIER_VIDMM_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3DKMT_PROCESS_VERIFIER_VIDMM_RESTRICT_BUDGET": SimStruct(OrderedDict((("LocalBudget", SimTypeLongLong(signed=False, label="UInt64")), ("NonLocalBudget", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_PROCESS_VERIFIER_VIDMM_RESTRICT_BUDGET", pack=False, align=None),
    "D3DKMT_PROTECTED_SESSION_STATUS": SimTypeInt(signed=False, label="D3DKMT_PROTECTED_SESSION_STATUS"),
    "D3DKMT_QUERYADAPTERINFO": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="KMTQUERYADAPTERINFOTYPE")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYADAPTERINFO", pack=False, align=None),
    "D3DKMT_QUERYALLOCATIONRESIDENCY": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("phAllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")), ("pResidencyStatus", SimTypePointer(SimTypeInt(signed=False, label="D3DKMT_ALLOCATIONRESIDENCYSTATUS"), offset=0)),)), name="D3DKMT_QUERYALLOCATIONRESIDENCY", pack=False, align=None),
    "D3DKMT_QUERYCLOCKCALIBRATION": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("NodeOrdinal", SimTypeInt(signed=False, label="UInt32")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("ClockData", SimTypeRef("DXGK_GPUCLOCKDATA", SimStruct)),)), name="D3DKMT_QUERYCLOCKCALIBRATION", pack=False, align=None),
    "D3DKMT_QUERYFSEBLOCK": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_QUERYFSEBLOCK", pack=False, align=None),
    "D3DKMT_QUERYFSEBLOCKFLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3DKMT_QUERYPROCESSOFFERINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DecommitUniqueness", SimTypeLongLong(signed=False, label="UInt64")), ("DecommittableBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYPROCESSOFFERINFO", pack=False, align=None),
    "D3DKMT_QUERYPROTECTEDSESSIONINFOFROMNTHANDLE": SimStruct(OrderedDict((("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYPROTECTEDSESSIONINFOFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_QUERYPROTECTEDSESSIONSTATUS": SimStruct(OrderedDict((("hHandle", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="D3DKMT_PROTECTED_SESSION_STATUS")),)), name="D3DKMT_QUERYPROTECTEDSESSIONSTATUS", pack=False, align=None),
    "D3DKMT_QUERYREMOTEVIDPNSOURCEFROMGDIDISPLAYNAME": SimStruct(OrderedDict((("DeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYREMOTEVIDPNSOURCEFROMGDIDISPLAYNAME", pack=False, align=None),
    "D3DKMT_QUERYRESOURCEINFO": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hGlobalShare", SimTypeInt(signed=False, label="UInt32")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("TotalPrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("ResourcePrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYRESOURCEINFO", pack=False, align=None),
    "D3DKMT_QUERYRESOURCEINFOFROMNTHANDLE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hNtHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")), ("TotalPrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("ResourcePrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYRESOURCEINFOFROMNTHANDLE", pack=False, align=None),
    "D3DKMT_QUERYRESULT_PREEMPTION_ATTEMPT_RESULT": SimTypeInt(signed=False, label="D3DKMT_QUERYRESULT_PREEMPTION_ATTEMPT_RESULT"),
    "D3DKMT_QUERYSTATISTICS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_QUERYSTATISTICS_TYPE")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("QueryResult", SimUnion({"AdapterInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION", SimStruct), "PhysAdapterInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PHYSICAL_ADAPTER_INFORMATION", SimStruct), "SegmentInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_SEGMENT_INFORMATION", SimStruct), "NodeInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_NODE_INFORMATION", SimStruct), "VidPnSourceInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_VIDPNSOURCE_INFORMATION", SimStruct), "ProcessInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_INFORMATION", SimStruct), "ProcessAdapterInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_ADAPTER_INFORMATION", SimStruct), "ProcessSegmentInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_INFORMATION", SimStruct), "ProcessNodeInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_NODE_INFORMATION", SimStruct), "ProcessVidPnSourceInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_VIDPNSOURCE_INFORMATION", SimStruct), "ProcessSegmentGroupInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_GROUP_INFORMATION", SimStruct), "SegmentUsageInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_MEMORY_USAGE", SimStruct), "SegmentGroupUsageInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_MEMORY_USAGE", SimStruct)}, name="<anon>", label="None")), ("Anonymous", SimUnion({"QuerySegment": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT", SimStruct), "QueryProcessSegment": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT", SimStruct), "QueryProcessSegmentGroup": SimTypeInt(signed=False, label="D3DKMT_MEMORY_SEGMENT_GROUP"), "QueryNode": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_NODE", SimStruct), "QueryProcessNode": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_NODE", SimStruct), "QueryVidPnSource": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_VIDPNSOURCE", SimStruct), "QueryProcessVidPnSource": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_VIDPNSOURCE", SimStruct), "QueryPhysAdapter": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_PHYSICAL_ADAPTER", SimStruct), "QueryAdapter2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_ADAPTER2", SimStruct), "QuerySegment2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT2", SimStruct), "QueryProcessAdapter2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_ADAPTER2", SimStruct), "QueryProcessSegment2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT2", SimStruct), "QueryProcessSegmentGroup2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_PROCESS_SEGMENT_GROUP2", SimStruct), "QuerySegmentUsage": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT_USAGE", SimStruct), "QuerySegmentGroupUsage": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT_GROUP_USAGE", SimStruct), "QueryNode2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_NODE2", SimStruct), "QueryProcessNode2": SimTypeRef("D3DKMT_QUERYSTATISTICS_QUERY_NODE2", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_QUERYSTATISTICS", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION": SimStruct(OrderedDict((("NbSegments", SimTypeInt(signed=False, label="UInt32")), ("NodeCount", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceCount", SimTypeInt(signed=False, label="UInt32")), ("VSyncEnabled", SimTypeInt(signed=False, label="UInt32")), ("TdrDetectedCount", SimTypeInt(signed=False, label="UInt32")), ("ZeroLengthDmaBuffers", SimTypeLongLong(signed=True, label="Int64")), ("RestartedPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("ReferenceDmaBuffer", SimTypeRef("D3DKMT_QUERYSTATSTICS_REFERENCE_DMA_BUFFER", SimStruct)), ("Renaming", SimTypeRef("D3DKMT_QUERYSTATSTICS_RENAMING", SimStruct)), ("Preparation", SimTypeRef("D3DKMT_QUERYSTATSTICS_PREPRATION", SimStruct)), ("PagingFault", SimTypeRef("D3DKMT_QUERYSTATSTICS_PAGING_FAULT", SimStruct)), ("PagingTransfer", SimTypeRef("D3DKMT_QUERYSTATSTICS_PAGING_TRANSFER", SimStruct)), ("SwizzlingRange", SimTypeRef("D3DKMT_QUERYSTATSTICS_SWIZZLING_RANGE", SimStruct)), ("Locks", SimTypeRef("D3DKMT_QUERYSTATSTICS_LOCKS", SimStruct)), ("Allocations", SimTypeRef("D3DKMT_QUERYSTATSTICS_ALLOCATIONS", SimStruct)), ("Terminations", SimTypeRef("D3DKMT_QUERYSTATSTICS_TERMINATIONS", SimStruct)), ("Flags", SimTypeRef("D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION_FLAGS", SimStruct)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 7)),)), name="D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION_FLAGS", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_ALLOCATION_PRIORITY_CLASS": SimTypeInt(signed=False, label="D3DKMT_QUERYSTATISTICS_ALLOCATION_PRIORITY_CLASS"),
    "D3DKMT_QUERYSTATISTICS_COMMITMENT_DATA": SimStruct(OrderedDict((("TotalBytesEvictedFromProcess", SimTypeLongLong(signed=False, label="UInt64")), ("BytesBySegmentPreference", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)),)), name="D3DKMT_QUERYSTATISTICS_COMMITMENT_DATA", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_COUNTER": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Bytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATISTICS_COUNTER", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_DMA_BUFFER": SimStruct(OrderedDict((("Size", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("AllocationListBytes", SimTypeInt(signed=False, label="UInt32")), ("PatchLocationListBytes", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_DMA_BUFFER", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_DMA_PACKET_TYPE": SimTypeInt(signed=False, label="D3DKMT_QUERYSTATISTICS_DMA_PACKET_TYPE"),
    "D3DKMT_QUERYSTATISTICS_DMA_PACKET_TYPE_INFORMATION": SimStruct(OrderedDict((("PacketSubmited", SimTypeInt(signed=False, label="UInt32")), ("PacketCompleted", SimTypeInt(signed=False, label="UInt32")), ("PacketPreempted", SimTypeInt(signed=False, label="UInt32")), ("PacketFaulted", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_DMA_PACKET_TYPE_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_MEMORY": SimStruct(OrderedDict((("TotalBytesEvicted", SimTypeLongLong(signed=False, label="UInt64")), ("AllocsCommitted", SimTypeInt(signed=False, label="UInt32")), ("AllocsResident", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_MEMORY", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_MEMORY_USAGE": SimStruct(OrderedDict((("AllocatedBytes", SimTypeLongLong(signed=False, label="UInt64")), ("FreeBytes", SimTypeLongLong(signed=False, label="UInt64")), ("ZeroBytes", SimTypeLongLong(signed=False, label="UInt64")), ("ModifiedBytes", SimTypeLongLong(signed=False, label="UInt64")), ("StandbyBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATISTICS_MEMORY_USAGE", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_NODE_INFORMATION": SimStruct(OrderedDict((("GlobalInformation", SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_NODE_INFORMATION", SimStruct)), ("SystemInformation", SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_NODE_INFORMATION", SimStruct)), ("NodePerfData", SimTypeRef("D3DKMT_NODE_PERFDATA", SimStruct)), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="D3DKMT_QUERYSTATISTICS_NODE_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PACKET_INFORMATION": SimStruct(OrderedDict((("QueuePacket", SimTypeArray(SimTypeRef("D3DKMT_QUERYSTATISTICS_QUEUE_PACKET_TYPE_INFORMATION", SimStruct), 8)), ("DmaPacket", SimTypeArray(SimTypeRef("D3DKMT_QUERYSTATISTICS_DMA_PACKET_TYPE_INFORMATION", SimStruct), 4)),)), name="D3DKMT_QUERYSTATISTICS_PACKET_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PHYSICAL_ADAPTER_INFORMATION": SimStruct(OrderedDict((("AdapterPerfData", SimTypeRef("D3DKMT_ADAPTER_PERFDATA", SimStruct)), ("AdapterPerfDataCaps", SimTypeRef("D3DKMT_ADAPTER_PERFDATACAPS", SimStruct)), ("GpuVersion", SimTypeRef("D3DKMT_GPUVERSION", SimStruct)),)), name="D3DKMT_QUERYSTATISTICS_PHYSICAL_ADAPTER_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_POLICY": SimStruct(OrderedDict((("PreferApertureForRead", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)), ("PreferAperture", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)), ("MemResetOnPaging", SimTypeLongLong(signed=False, label="UInt64")), ("RemovePagesFromWorkingSetOnPaging", SimTypeLongLong(signed=False, label="UInt64")), ("MigrationEnabled", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATISTICS_POLICY", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PREEMPTION_INFORMATION": SimStruct(OrderedDict((("PreemptionCounter", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)),)), name="D3DKMT_QUERYSTATISTICS_PREEMPTION_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_ADAPTER_INFORMATION": SimStruct(OrderedDict((("NbSegments", SimTypeInt(signed=False, label="UInt32")), ("NodeCount", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceCount", SimTypeInt(signed=False, label="UInt32")), ("VirtualMemoryUsage", SimTypeInt(signed=False, label="UInt32")), ("DmaBuffer", SimTypeRef("D3DKMT_QUERYSTATISTICS_DMA_BUFFER", SimStruct)), ("CommitmentData", SimTypeRef("D3DKMT_QUERYSTATISTICS_COMMITMENT_DATA", SimStruct)), ("_Policy", SimTypeRef("D3DKMT_QUERYSTATISTICS_POLICY", SimStruct)), ("ProcessInterferenceCounters", SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_INTERFERENCE_COUNTERS", SimStruct)), ("ClientHint", SimTypeInt(signed=False, label="D3DKMT_CLIENTHINT")),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_ADAPTER_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_INFORMATION": SimStruct(OrderedDict((("NodeCount", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceCount", SimTypeInt(signed=False, label="UInt32")), ("SystemMemory", SimTypeRef("D3DKMT_QUERYSTATISTICS_SYSTEM_MEMORY", SimStruct)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 7)),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_INTERFERENCE_COUNTERS": SimStruct(OrderedDict((("InterferenceCount", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 9)),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_INTERFERENCE_COUNTERS", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_NODE_INFORMATION": SimStruct(OrderedDict((("RunningTime", SimTypeLongLong(signed=True, label="Int64")), ("ContextSwitch", SimTypeInt(signed=False, label="UInt32")), ("PreemptionStatistics", SimTypeRef("D3DKMT_QUERYSTATISTICS_PREEMPTION_INFORMATION", SimStruct)), ("PacketStatistics", SimTypeRef("D3DKMT_QUERYSTATISTICS_PACKET_INFORMATION", SimStruct)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_NODE_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_GROUP_INFORMATION": SimStruct(OrderedDict((("Budget", SimTypeLongLong(signed=False, label="UInt64")), ("Requested", SimTypeLongLong(signed=False, label="UInt64")), ("Usage", SimTypeLongLong(signed=False, label="UInt64")), ("Demoted", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_GROUP_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_INFORMATION": SimStruct(OrderedDict((("BytesCommitted", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumWorkingSet", SimTypeLongLong(signed=False, label="UInt64")), ("MinimumWorkingSet", SimTypeLongLong(signed=False, label="UInt64")), ("NbReferencedAllocationEvictedInPeriod", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeInt(signed=False, label="UInt32")), ("VideoMemory", SimTypeRef("D3DKMT_QUERYSTATISTICS_VIDEO_MEMORY", SimStruct)), ("_Policy", SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_POLICY", SimStruct)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_POLICY": SimStruct(OrderedDict((("UseMRU", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_POLICY", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_PROCESS_VIDPNSOURCE_INFORMATION": SimStruct(OrderedDict((("Frame", SimTypeInt(signed=False, label="UInt32")), ("CancelledFrame", SimTypeInt(signed=False, label="UInt32")), ("QueuedPresent", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeInt(signed=False, label="UInt32")), ("IsVSyncEnabled", SimTypeLongLong(signed=False, label="UInt64")), ("VSyncOnTotalTimeMs", SimTypeLongLong(signed=False, label="UInt64")), ("VSyncOffKeepPhaseTotalTimeMs", SimTypeLongLong(signed=False, label="UInt64")), ("VSyncOffNoPhaseTotalTimeMs", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)),)), name="D3DKMT_QUERYSTATISTICS_PROCESS_VIDPNSOURCE_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_ADAPTER2": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_ADAPTER2", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_ADAPTER_INFORMATION2": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_ADAPTER_INFORMATION2", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_NODE": SimStruct(OrderedDict((("NodeId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_NODE", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_NODE2": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")), ("NodeOrdinal", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_NODE2", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_PHYSICAL_ADAPTER": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_PHYSICAL_ADAPTER", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_PROCESS_SEGMENT_GROUP2": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")), ("SegmentGroup", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_PROCESS_SEGMENT_GROUP2", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT": SimStruct(OrderedDict((("SegmentId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT2": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")), ("SegmentId", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT2", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT_GROUP_USAGE": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")), ("SegmentGroup", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT_GROUP_USAGE", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT_USAGE": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeShort(signed=False, label="UInt16")), ("SegmentId", SimTypeShort(signed=False, label="UInt16")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_SEGMENT_USAGE", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUERY_VIDPNSOURCE": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_QUERY_VIDPNSOURCE", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_QUEUE_PACKET_TYPE": SimTypeInt(signed=False, label="D3DKMT_QUERYSTATISTICS_QUEUE_PACKET_TYPE"),
    "D3DKMT_QUERYSTATISTICS_QUEUE_PACKET_TYPE_INFORMATION": SimStruct(OrderedDict((("PacketSubmited", SimTypeInt(signed=False, label="UInt32")), ("PacketCompleted", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATISTICS_QUEUE_PACKET_TYPE_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_RESULT": SimUnion({"AdapterInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_ADAPTER_INFORMATION", SimStruct), "PhysAdapterInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PHYSICAL_ADAPTER_INFORMATION", SimStruct), "SegmentInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_SEGMENT_INFORMATION", SimStruct), "NodeInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_NODE_INFORMATION", SimStruct), "VidPnSourceInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_VIDPNSOURCE_INFORMATION", SimStruct), "ProcessInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_INFORMATION", SimStruct), "ProcessAdapterInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_ADAPTER_INFORMATION", SimStruct), "ProcessSegmentInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_INFORMATION", SimStruct), "ProcessNodeInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_NODE_INFORMATION", SimStruct), "ProcessVidPnSourceInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_VIDPNSOURCE_INFORMATION", SimStruct), "ProcessSegmentGroupInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_SEGMENT_GROUP_INFORMATION", SimStruct), "SegmentUsageInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_MEMORY_USAGE", SimStruct), "SegmentGroupUsageInformation": SimTypeRef("D3DKMT_QUERYSTATISTICS_MEMORY_USAGE", SimStruct)}, name="<anon>", label="None"),
    "D3DKMT_QUERYSTATISTICS_SEGMENT_INFORMATION": SimStruct(OrderedDict((("CommitLimit", SimTypeLongLong(signed=False, label="UInt64")), ("BytesCommitted", SimTypeLongLong(signed=False, label="UInt64")), ("BytesResident", SimTypeLongLong(signed=False, label="UInt64")), ("Memory", SimTypeRef("D3DKMT_QUERYSTATISTICS_MEMORY", SimStruct)), ("Aperture", SimTypeInt(signed=False, label="UInt32")), ("TotalBytesEvictedByPriority", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)), ("SystemMemoryEndAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PowerFlags", SimTypeRef("_PowerFlags_e__Struct", SimStruct)), ("SegmentProperties", SimTypeRef("_SegmentProperties_e__Struct", SimStruct)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)),)), name="D3DKMT_QUERYSTATISTICS_SEGMENT_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_SEGMENT_TYPE": SimTypeInt(signed=False, label="D3DKMT_QUERYSTATISTICS_SEGMENT_TYPE"),
    "D3DKMT_QUERYSTATISTICS_SYSTEM_MEMORY": SimStruct(OrderedDict((("BytesAllocated", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReserved", SimTypeLongLong(signed=False, label="UInt64")), ("SmallAllocationBlocks", SimTypeInt(signed=False, label="UInt32")), ("LargeAllocationBlocks", SimTypeInt(signed=False, label="UInt32")), ("WriteCombinedBytesAllocated", SimTypeLongLong(signed=False, label="UInt64")), ("WriteCombinedBytesReserved", SimTypeLongLong(signed=False, label="UInt64")), ("CachedBytesAllocated", SimTypeLongLong(signed=False, label="UInt64")), ("CachedBytesReserved", SimTypeLongLong(signed=False, label="UInt64")), ("SectionBytesAllocated", SimTypeLongLong(signed=False, label="UInt64")), ("SectionBytesReserved", SimTypeLongLong(signed=False, label="UInt64")), ("BytesZeroed", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATISTICS_SYSTEM_MEMORY", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_TYPE": SimTypeInt(signed=False, label="D3DKMT_QUERYSTATISTICS_TYPE"),
    "D3DKMT_QUERYSTATISTICS_VIDEO_MEMORY": SimStruct(OrderedDict((("AllocsCommitted", SimTypeInt(signed=False, label="UInt32")), ("AllocsResidentInP", SimTypeArray(SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct), 5)), ("AllocsResidentInNonPreferred", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("TotalBytesEvictedDueToPreparation", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATISTICS_VIDEO_MEMORY", pack=False, align=None),
    "D3DKMT_QUERYSTATISTICS_VIDPNSOURCE_INFORMATION": SimStruct(OrderedDict((("GlobalInformation", SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_VIDPNSOURCE_INFORMATION", SimStruct)), ("SystemInformation", SimTypeRef("D3DKMT_QUERYSTATISTICS_PROCESS_VIDPNSOURCE_INFORMATION", SimStruct)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="D3DKMT_QUERYSTATISTICS_VIDPNSOURCE_INFORMATION", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_ALLOCATIONS": SimStruct(OrderedDict((("Created", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("Destroyed", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("Opened", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("Closed", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("MigratedSuccess", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("MigratedFail", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("MigratedAbandoned", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)),)), name="D3DKMT_QUERYSTATSTICS_ALLOCATIONS", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_LOCKS": SimStruct(OrderedDict((("NbLocks", SimTypeInt(signed=False, label="UInt32")), ("NbLocksWaitFlag", SimTypeInt(signed=False, label="UInt32")), ("NbLocksDiscardFlag", SimTypeInt(signed=False, label="UInt32")), ("NbLocksNoOverwrite", SimTypeInt(signed=False, label="UInt32")), ("NbLocksNoReadSync", SimTypeInt(signed=False, label="UInt32")), ("NbLocksLinearization", SimTypeInt(signed=False, label="UInt32")), ("NbComplexLocks", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATSTICS_LOCKS", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_PAGING_FAULT": SimStruct(OrderedDict((("Faults", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("FaultsFirstTimeAccess", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("FaultsReclaimed", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("FaultsMigration", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("FaultsIncorrectResource", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("FaultsLostContent", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("FaultsEvicted", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("AllocationsMEM_RESET", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("AllocationsUnresetSuccess", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("AllocationsUnresetFail", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("AllocationsUnresetSuccessRead", SimTypeInt(signed=False, label="UInt32")), ("AllocationsUnresetFailRead", SimTypeInt(signed=False, label="UInt32")), ("Evictions", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("EvictionsDueToPreparation", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("EvictionsDueToLock", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("EvictionsDueToClose", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("EvictionsDueToPurge", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("EvictionsDueToSuspendCPUAccess", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)),)), name="D3DKMT_QUERYSTATSTICS_PAGING_FAULT", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_PAGING_TRANSFER": SimStruct(OrderedDict((("BytesFilled", SimTypeLongLong(signed=False, label="UInt64")), ("BytesDiscarded", SimTypeLongLong(signed=False, label="UInt64")), ("BytesMappedIntoAperture", SimTypeLongLong(signed=False, label="UInt64")), ("BytesUnmappedFromAperture", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferredFromMdlToMemory", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferredFromMemoryToMdl", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferredFromApertureToMemory", SimTypeLongLong(signed=False, label="UInt64")), ("BytesTransferredFromMemoryToAperture", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_QUERYSTATSTICS_PAGING_TRANSFER", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_PREPRATION": SimStruct(OrderedDict((("BroadcastStall", SimTypeInt(signed=False, label="UInt32")), ("NbDMAPrepared", SimTypeInt(signed=False, label="UInt32")), ("NbDMAPreparedLongPath", SimTypeInt(signed=False, label="UInt32")), ("ImmediateHighestPreparationPass", SimTypeInt(signed=False, label="UInt32")), ("AllocationsTrimmed", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)),)), name="D3DKMT_QUERYSTATSTICS_PREPRATION", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_REFERENCE_DMA_BUFFER": SimStruct(OrderedDict((("NbCall", SimTypeInt(signed=False, label="UInt32")), ("NbAllocationsReferenced", SimTypeInt(signed=False, label="UInt32")), ("MaxNbAllocationsReferenced", SimTypeInt(signed=False, label="UInt32")), ("NbNULLReference", SimTypeInt(signed=False, label="UInt32")), ("NbWriteReference", SimTypeInt(signed=False, label="UInt32")), ("NbRenamedAllocationsReferenced", SimTypeInt(signed=False, label="UInt32")), ("NbIterationSearchingRenamedAllocation", SimTypeInt(signed=False, label="UInt32")), ("NbLockedAllocationReferenced", SimTypeInt(signed=False, label="UInt32")), ("NbAllocationWithValidPrepatchingInfoReferenced", SimTypeInt(signed=False, label="UInt32")), ("NbAllocationWithInvalidPrepatchingInfoReferenced", SimTypeInt(signed=False, label="UInt32")), ("NbDMABufferSuccessfullyPrePatched", SimTypeInt(signed=False, label="UInt32")), ("NbPrimariesReferencesOverflow", SimTypeInt(signed=False, label="UInt32")), ("NbAllocationWithNonPreferredResources", SimTypeInt(signed=False, label="UInt32")), ("NbAllocationInsertedInMigrationTable", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATSTICS_REFERENCE_DMA_BUFFER", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_RENAMING": SimStruct(OrderedDict((("NbAllocationsRenamed", SimTypeInt(signed=False, label="UInt32")), ("NbAllocationsShrinked", SimTypeInt(signed=False, label="UInt32")), ("NbRenamedBuffer", SimTypeInt(signed=False, label="UInt32")), ("MaxRenamingListLength", SimTypeInt(signed=False, label="UInt32")), ("NbFailuresDueToRenamingLimit", SimTypeInt(signed=False, label="UInt32")), ("NbFailuresDueToCreateAllocation", SimTypeInt(signed=False, label="UInt32")), ("NbFailuresDueToOpenAllocation", SimTypeInt(signed=False, label="UInt32")), ("NbFailuresDueToLowResource", SimTypeInt(signed=False, label="UInt32")), ("NbFailuresDueToNonRetiredLimit", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATSTICS_RENAMING", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_SWIZZLING_RANGE": SimStruct(OrderedDict((("NbRangesAcquired", SimTypeInt(signed=False, label="UInt32")), ("NbRangesReleased", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYSTATSTICS_SWIZZLING_RANGE", pack=False, align=None),
    "D3DKMT_QUERYSTATSTICS_TERMINATIONS": SimStruct(OrderedDict((("TerminatedShared", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("TerminatedNonShared", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("DestroyedShared", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)), ("DestroyedNonShared", SimTypeRef("D3DKMT_QUERYSTATISTICS_COUNTER", SimStruct)),)), name="D3DKMT_QUERYSTATSTICS_TERMINATIONS", pack=False, align=None),
    "D3DKMT_QUERYVIDEOMEMORYINFO": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("MemorySegmentGroup", SimTypeInt(signed=False, label="D3DKMT_MEMORY_SEGMENT_GROUP")), ("Budget", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentUsage", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentReservation", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableForReservation", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERYVIDEOMEMORYINFO", pack=False, align=None),
    "D3DKMT_QUERYVIDPNEXCLUSIVEOWNERSHIP": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("OwnerType", SimTypeInt(signed=False, label="D3DKMT_VIDPNSOURCEOWNER_TYPE")),)), name="D3DKMT_QUERYVIDPNEXCLUSIVEOWNERSHIP", pack=False, align=None),
    "D3DKMT_QUERY_ADAPTER_UNIQUE_GUID": SimStruct(OrderedDict((("AdapterUniqueGUID", SimTypeArray(SimTypeChar(label="Char"), 40)),)), name="D3DKMT_QUERY_ADAPTER_UNIQUE_GUID", pack=False, align=None),
    "D3DKMT_QUERY_DEVICE_IDS": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("DeviceIds", SimTypeRef("D3DKMT_DEVICE_IDS", SimStruct)),)), name="D3DKMT_QUERY_DEVICE_IDS", pack=False, align=None),
    "D3DKMT_QUERY_GPUMMU_CAPS": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("Caps", SimTypeRef("D3DKMT_GPUMMU_CAPS", SimStruct)),)), name="D3DKMT_QUERY_GPUMMU_CAPS", pack=False, align=None),
    "D3DKMT_QUERY_MIRACAST_DRIVER_TYPE": SimStruct(OrderedDict((("MiracastDriverType", SimTypeInt(signed=False, label="D3DKMT_MIRACAST_DRIVER_TYPE")),)), name="D3DKMT_QUERY_MIRACAST_DRIVER_TYPE", pack=False, align=None),
    "D3DKMT_QUERY_PHYSICAL_ADAPTER_PNP_KEY": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("PnPKeyType", SimTypeInt(signed=False, label="D3DKMT_PNP_KEY_TYPE")), ("pDest", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pCchDest", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_QUERY_PHYSICAL_ADAPTER_PNP_KEY", pack=False, align=None),
    "D3DKMT_QUERY_SCANOUT_CAPS": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Caps", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_QUERY_SCANOUT_CAPS", pack=False, align=None),
    "D3DKMT_QUEUEDLIMIT_TYPE": SimTypeInt(signed=False, label="D3DKMT_QUEUEDLIMIT_TYPE"),
    "D3DKMT_RECLAIMALLOCATIONS": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("pResources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("HandleList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pDiscarded", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_RECLAIMALLOCATIONS", pack=False, align=None),
    "D3DKMT_RECLAIMALLOCATIONS2": SimStruct(OrderedDict((("hPagingQueue", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("pResources", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("HandleList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Anonymous", SimUnion({"pDiscarded": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pResults": SimTypePointer(SimTypeInt(signed=False, label="D3DDDI_RECLAIM_RESULT"), offset=0)}, name="<anon>", label="None")), ("PagingFenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_RECLAIMALLOCATIONS2", pack=False, align=None),
    "D3DKMT_REGISTERBUDGETCHANGENOTIFICATION": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_BUDGETCHANGENOTIFICATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DKMT_REGISTERBUDGETCHANGENOTIFICATION", pack=False, align=None),
    "D3DKMT_REGISTERTRIMNOTIFICATION": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_TRIMNOTIFICATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DKMT_REGISTERTRIMNOTIFICATION", pack=False, align=None),
    "D3DKMT_RELEASEKEYEDMUTEX": SimStruct(OrderedDict((("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeLongLong(signed=False, label="UInt64")), ("FenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_RELEASEKEYEDMUTEX", pack=False, align=None),
    "D3DKMT_RELEASEKEYEDMUTEX2": SimStruct(OrderedDict((("hKeyedMutex", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeLongLong(signed=False, label="UInt64")), ("FenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("pPrivateRuntimeData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateRuntimeDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_RELEASEKEYEDMUTEX2", pack=False, align=None),
    "D3DKMT_RENDER": SimStruct(OrderedDict((("Anonymous", SimUnion({"hDevice": SimTypeInt(signed=False, label="UInt32"), "hContext": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CommandOffset", SimTypeInt(signed=False, label="UInt32")), ("CommandLength", SimTypeInt(signed=False, label="UInt32")), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")), ("PatchLocationCount", SimTypeInt(signed=False, label="UInt32")), ("pNewCommandBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NewCommandBufferSize", SimTypeInt(signed=False, label="UInt32")), ("pNewAllocationList", SimTypePointer(SimTypeRef("D3DDDI_ALLOCATIONLIST", SimStruct), offset=0)), ("NewAllocationListSize", SimTypeInt(signed=False, label="UInt32")), ("pNewPatchLocationList", SimTypePointer(SimTypeRef("D3DDDI_PATCHLOCATIONLIST", SimStruct), offset=0)), ("NewPatchLocationListSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_RENDERFLAGS", SimStruct)), ("PresentHistoryToken", SimTypeLongLong(signed=False, label="UInt64")), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("QueuedBufferCount", SimTypeInt(signed=False, label="UInt32")), ("NewCommandBuffer", SimTypeLongLong(signed=False, label="UInt64")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_RENDER", pack=False, align=None),
    "D3DKMT_RENDERFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_RENDERFLAGS", pack=False, align=None),
    "D3DKMT_REQUEST_MACHINE_CRASH_ESCAPE": SimStruct(OrderedDict((("Param1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Param2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Param3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3DKMT_REQUEST_MACHINE_CRASH_ESCAPE", pack=False, align=None),
    "D3DKMT_SCATTERBLT": SimStruct(OrderedDict((("hLogicalSurfaceDestination", SimTypeLongLong(signed=False, label="UInt64")), ("hDestinationCompSurfDWM", SimTypeLongLong(signed=True, label="Int64")), ("DestinationCompositionBindingId", SimTypeLongLong(signed=False, label="UInt64")), ("SourceRect", SimTypeRef("RECT", SimStruct)), ("DestinationOffset", SimTypeRef("POINT", SimStruct)),)), name="D3DKMT_SCATTERBLT", pack=False, align=None),
    "D3DKMT_SCATTERBLTS": SimStruct(OrderedDict((("NumBlts", SimTypeInt(signed=False, label="UInt32")), ("Blts", SimTypeArray(SimTypeRef("D3DKMT_SCATTERBLT", SimStruct), 12)),)), name="D3DKMT_SCATTERBLTS", pack=False, align=None),
    "D3DKMT_SCHEDULINGPRIORITYCLASS": SimTypeInt(signed=False, label="D3DKMT_SCHEDULINGPRIORITYCLASS"),
    "D3DKMT_SEGMENTGROUPSIZEINFO": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("LegacyInfo", SimTypeRef("D3DKMT_SEGMENTSIZEINFO", SimStruct)), ("LocalMemory", SimTypeLongLong(signed=False, label="UInt64")), ("NonLocalMemory", SimTypeLongLong(signed=False, label="UInt64")), ("NonBudgetMemory", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_SEGMENTGROUPSIZEINFO", pack=False, align=None),
    "D3DKMT_SEGMENTSIZEINFO": SimStruct(OrderedDict((("DedicatedVideoMemorySize", SimTypeLongLong(signed=False, label="UInt64")), ("DedicatedSystemMemorySize", SimTypeLongLong(signed=False, label="UInt64")), ("SharedSystemMemorySize", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_SEGMENTSIZEINFO", pack=False, align=None),
    "D3DKMT_SEGMENT_CAPS": SimStruct(OrderedDict((("Size", SimTypeLongLong(signed=False, label="UInt64")), ("PageSize", SimTypeInt(signed=False, label="UInt32")), ("SegmentId", SimTypeInt(signed=False, label="UInt32")), ("bAperture", SimTypeChar(label="Byte")), ("bReservedSysMem", SimTypeChar(label="Byte")), ("BudgetGroup", SimTypeInt(signed=False, label="D3DKMT_MEMORY_SEGMENT_GROUP")),)), name="D3DKMT_SEGMENT_CAPS", pack=False, align=None),
    "D3DKMT_SETALLOCATIONPRIORITY": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hResource", SimTypeInt(signed=False, label="UInt32")), ("phAllocationList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AllocationCount", SimTypeInt(signed=False, label="UInt32")), ("pPriorities", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_SETALLOCATIONPRIORITY", pack=False, align=None),
    "D3DKMT_SETCONTEXTINPROCESSSCHEDULINGPRIORITY": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_SETCONTEXTINPROCESSSCHEDULINGPRIORITY", pack=False, align=None),
    "D3DKMT_SETCONTEXTSCHEDULINGPRIORITY": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_SETCONTEXTSCHEDULINGPRIORITY", pack=False, align=None),
    "D3DKMT_SETDISPLAYMODE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hPrimaryAllocation", SimTypeInt(signed=False, label="UInt32")), ("ScanLineOrdering", SimTypeInt(signed=False, label="D3DDDI_VIDEO_SIGNAL_SCANLINE_ORDERING")), ("DisplayOrientation", SimTypeInt(signed=False, label="D3DDDI_ROTATION")), ("PrivateDriverFormatAttribute", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_SETDISPLAYMODE_FLAGS", SimStruct)),)), name="D3DKMT_SETDISPLAYMODE", pack=False, align=None),
    "D3DKMT_SETDISPLAYMODE_FLAGS": SimStruct(OrderedDict((("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SETDISPLAYMODE_FLAGS", pack=False, align=None),
    "D3DKMT_SETDISPLAYPRIVATEDRIVERFORMAT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverFormatAttribute", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SETDISPLAYPRIVATEDRIVERFORMAT", pack=False, align=None),
    "D3DKMT_SETFSEBLOCK": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_SETFSEBLOCK", pack=False, align=None),
    "D3DKMT_SETFSEBLOCKFLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3DKMT_SETGAMMARAMP": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DDDI_GAMMARAMP_TYPE")), ("Anonymous", SimUnion({"pGammaRampRgb256x3x16": SimTypePointer(SimTypeRef("D3DDDI_GAMMA_RAMP_RGB256x3x16", SimStruct), offset=0), "pGammaRampDXGI1": SimTypePointer(SimTypeRef("D3DDDI_GAMMA_RAMP_DXGI_1", SimStruct), offset=0)}, name="<anon>", label="None")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SETGAMMARAMP", pack=False, align=None),
    "D3DKMT_SETHWPROTECTIONTEARDOWNRECOVERY": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("Recovered", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_SETHWPROTECTIONTEARDOWNRECOVERY", pack=False, align=None),
    "D3DKMT_SETQUEUEDLIMIT": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DKMT_QUEUEDLIMIT_TYPE")), ("Anonymous", SimUnion({"QueuedPresentLimit": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("QueuedPendingFlipLimit", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="D3DKMT_SETQUEUEDLIMIT", pack=False, align=None),
    "D3DKMT_SETSTABLEPOWERSTATE": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_SETSTABLEPOWERSTATE", pack=False, align=None),
    "D3DKMT_SETSYNCREFRESHCOUNTWAITTARGET": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("TargetSyncRefreshCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SETSYNCREFRESHCOUNTWAITTARGET", pack=False, align=None),
    "D3DKMT_SETVIDPNSOURCEHWPROTECTION": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("HwProtected", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_SETVIDPNSOURCEHWPROTECTION", pack=False, align=None),
    "D3DKMT_SETVIDPNSOURCEOWNER": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("pType", SimTypePointer(SimTypeInt(signed=False, label="D3DKMT_VIDPNSOURCEOWNER_TYPE"), offset=0)), ("pVidPnSourceId", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("VidPnSourceCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SETVIDPNSOURCEOWNER", pack=False, align=None),
    "D3DKMT_SETVIDPNSOURCEOWNER1": SimStruct(OrderedDict((("Version0", SimTypeRef("D3DKMT_SETVIDPNSOURCEOWNER", SimStruct)), ("Flags", SimTypeRef("D3DKMT_VIDPNSOURCEOWNER_FLAGS", SimStruct)),)), name="D3DKMT_SETVIDPNSOURCEOWNER1", pack=False, align=None),
    "D3DKMT_SETVIDPNSOURCEOWNER2": SimStruct(OrderedDict((("Version1", SimTypeRef("D3DKMT_SETVIDPNSOURCEOWNER1", SimStruct)), ("pVidPnSourceNtHandles", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="D3DKMT_SETVIDPNSOURCEOWNER2", pack=False, align=None),
    "D3DKMT_SET_COLORSPACE_TRANSFORM": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("VidPnTargetId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DDDI_GAMMARAMP_TYPE")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pColorSpaceTransform": SimTypePointer(SimTypeRef("D3DKMDT_3x4_COLORSPACE_TRANSFORM", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="D3DKMT_SET_COLORSPACE_TRANSFORM", pack=False, align=None),
    "D3DKMT_SHAREDPRIMARYLOCKNOTIFICATION": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("LockRect", SimTypeRef("RECTL", SimStruct)),)), name="D3DKMT_SHAREDPRIMARYLOCKNOTIFICATION", pack=False, align=None),
    "D3DKMT_SHAREDPRIMARYUNLOCKNOTIFICATION": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SHAREDPRIMARYUNLOCKNOTIFICATION", pack=False, align=None),
    "D3DKMT_SHAREOBJECTWITHHOST": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hObject", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")), ("hVailProcessNtHandle", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_SHAREOBJECTWITHHOST", pack=False, align=None),
    "D3DKMT_SIGNALSYNCHRONIZATIONOBJECT": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("Flags", SimTypeRef("D3DDDICB_SIGNALFLAGS", SimStruct)),)), name="D3DKMT_SIGNALSYNCHRONIZATIONOBJECT", pack=False, align=None),
    "D3DKMT_SIGNALSYNCHRONIZATIONOBJECT2": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("Flags", SimTypeRef("D3DDDICB_SIGNALFLAGS", SimStruct)), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("Anonymous", SimUnion({"Fence": SimTypeRef("_Fence_e__Struct", SimStruct), "CpuEventHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)}, name="<anon>", label="None")),)), name="D3DKMT_SIGNALSYNCHRONIZATIONOBJECT2", pack=False, align=None),
    "D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("FenceValueArray", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("Flags", SimTypeRef("D3DDDICB_SIGNALFLAGS", SimStruct)),)), name="D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU", pack=False, align=None),
    "D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Anonymous", SimUnion({"MonitoredFenceValueArray": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)}, name="<anon>", label="None")),)), name="D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU", pack=False, align=None),
    "D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU2": SimStruct(OrderedDict((("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Flags", SimTypeRef("D3DDDICB_SIGNALFLAGS", SimStruct)), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContextArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Anonymous", SimUnion({"FenceValue": SimTypeLongLong(signed=False, label="UInt64"), "CpuEventHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "MonitoredFenceValueArray": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)}, name="<anon>", label="None")),)), name="D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU2", pack=False, align=None),
    "D3DKMT_STANDARDALLOCATIONTYPE": SimTypeInt(signed=False, label="D3DKMT_STANDARDALLOCATIONTYPE"),
    "D3DKMT_STANDARDALLOCATION_EXISTINGHEAP": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3DKMT_STANDARDALLOCATION_EXISTINGHEAP", pack=False, align=None),
    "D3DKMT_SUBMITCOMMAND": SimStruct(OrderedDict((("Commands", SimTypeLongLong(signed=False, label="UInt64")), ("CommandLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("D3DKMT_SUBMITCOMMANDFLAGS", SimStruct)), ("PresentHistoryToken", SimTypeLongLong(signed=False, label="UInt64")), ("BroadcastContextCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("NumPrimaries", SimTypeInt(signed=False, label="UInt32")), ("WrittenPrimaries", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)), ("NumHistoryBuffers", SimTypeInt(signed=False, label="UInt32")), ("HistoryBufferArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_SUBMITCOMMAND", pack=False, align=None),
    "D3DKMT_SUBMITCOMMANDFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_SUBMITCOMMANDFLAGS", pack=False, align=None),
    "D3DKMT_SUBMITCOMMANDTOHWQUEUE": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("HwQueueProgressFenceId", SimTypeLongLong(signed=False, label="UInt64")), ("CommandBuffer", SimTypeLongLong(signed=False, label="UInt64")), ("CommandLength", SimTypeInt(signed=False, label="UInt32")), ("PrivateDriverDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumPrimaries", SimTypeInt(signed=False, label="UInt32")), ("WrittenPrimaries", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_SUBMITCOMMANDTOHWQUEUE", pack=False, align=None),
    "D3DKMT_SUBMITPRESENTBLTTOHWQUEUE": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("HwQueueProgressFenceId", SimTypeLongLong(signed=False, label="UInt64")), ("PrivatePresentData", SimTypeRef("D3DKMT_PRESENT", SimStruct)),)), name="D3DKMT_SUBMITPRESENTBLTTOHWQUEUE", pack=False, align=None),
    "D3DKMT_SUBMITPRESENTTOHWQUEUE": SimStruct(OrderedDict((("hHwQueues", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("PrivatePresentData", SimTypeRef("D3DKMT_PRESENT", SimStruct)),)), name="D3DKMT_SUBMITPRESENTTOHWQUEUE", pack=False, align=None),
    "D3DKMT_SUBMITSIGNALSYNCOBJECTSTOHWQUEUE": SimStruct(OrderedDict((("Flags", SimTypeRef("D3DDDICB_SIGNALFLAGS", SimStruct)), ("BroadcastHwQueueCount", SimTypeInt(signed=False, label="UInt32")), ("BroadcastHwQueueArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("FenceValueArray", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="D3DKMT_SUBMITSIGNALSYNCOBJECTSTOHWQUEUE", pack=False, align=None),
    "D3DKMT_SUBMITWAITFORSYNCOBJECTSTOHWQUEUE": SimStruct(OrderedDict((("hHwQueue", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("FenceValueArray", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="D3DKMT_SUBMITWAITFORSYNCOBJECTSTOHWQUEUE", pack=False, align=None),
    "D3DKMT_SURFACECOMPLETE_PRESENTHISTORYTOKEN": SimStruct(OrderedDict((("hLogicalSurface", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_SURFACECOMPLETE_PRESENTHISTORYTOKEN", pack=False, align=None),
    "D3DKMT_TDRDBGCTRLTYPE": SimTypeInt(signed=False, label="D3DKMT_TDRDBGCTRLTYPE"),
    "D3DKMT_TDRDBGCTRL_ESCAPE": SimStruct(OrderedDict((("TdrControl", SimTypeInt(signed=False, label="D3DKMT_TDRDBGCTRLTYPE")), ("Anonymous", SimUnion({"NodeOrdinal": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_TDRDBGCTRL_ESCAPE", pack=False, align=None),
    "D3DKMT_TRACKEDWORKLOAD_SUPPORT": SimStruct(OrderedDict((("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("EngineType", SimTypeInt(signed=False, label="DXGK_ENGINE_TYPE")), ("Support", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_TRACKEDWORKLOAD_SUPPORT", pack=False, align=None),
    "D3DKMT_TRIMNOTIFICATION": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeRef("D3DDDI_TRIMRESIDENCYSET_FLAGS", SimStruct)), ("NumBytesToTrim", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_TRIMNOTIFICATION", pack=False, align=None),
    "D3DKMT_TRIMPROCESSCOMMITMENT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DecommitRequested", SimTypeLongLong(signed=False, label="UInt64")), ("NumBytesDecommitted", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_TRIMPROCESSCOMMITMENT", pack=False, align=None),
    "D3DKMT_TRIMPROCESSCOMMITMENT_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "D3DKMT_UMDFILENAMEINFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="KMTUMDVERSION")), ("UmdFileName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_UMDFILENAMEINFO", pack=False, align=None),
    "D3DKMT_UMD_DRIVER_VERSION": SimStruct(OrderedDict((("DriverVersion", SimTypeLongLong(signed=True, label="Int64")),)), name="D3DKMT_UMD_DRIVER_VERSION", pack=False, align=None),
    "D3DKMT_UNLOCK": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("NumAllocations", SimTypeInt(signed=False, label="UInt32")), ("phAllocations", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_UNLOCK", pack=False, align=None),
    "D3DKMT_UNLOCK2": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_UNLOCK2", pack=False, align=None),
    "D3DKMT_UNPINDIRECTFLIPRESOURCES": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("ResourceCount", SimTypeInt(signed=False, label="UInt32")), ("pResourceList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="D3DKMT_UNPINDIRECTFLIPRESOURCES", pack=False, align=None),
    "D3DKMT_UNREGISTERBUDGETCHANGENOTIFICATION": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DKMT_UNREGISTERBUDGETCHANGENOTIFICATION", pack=False, align=None),
    "D3DKMT_UNREGISTERTRIMNOTIFICATION": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_TRIMNOTIFICATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)),)), name="D3DKMT_UNREGISTERTRIMNOTIFICATION", pack=False, align=None),
    "D3DKMT_UPDATEGPUVIRTUALADDRESS": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hContext", SimTypeInt(signed=False, label="UInt32")), ("hFenceObject", SimTypeInt(signed=False, label="UInt32")), ("NumOperations", SimTypeInt(signed=False, label="UInt32")), ("Operations", SimTypePointer(SimTypeRef("D3DDDI_UPDATEGPUVIRTUALADDRESS_OPERATION", SimStruct), offset=0)), ("Reserved0", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("FenceValue", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_UPDATEGPUVIRTUALADDRESS", pack=False, align=None),
    "D3DKMT_UPDATEOVERLAY": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("hOverlay", SimTypeInt(signed=False, label="UInt32")), ("OverlayInfo", SimTypeRef("D3DDDI_KERNELOVERLAYINFO", SimStruct)),)), name="D3DKMT_UPDATEOVERLAY", pack=False, align=None),
    "D3DKMT_VAD_DESC": SimStruct(OrderedDict((("VadIndex", SimTypeInt(signed=False, label="UInt32")), ("VadAddress", SimTypeLongLong(signed=False, label="UInt64")), ("NumMappedRanges", SimTypeInt(signed=False, label="UInt32")), ("VadType", SimTypeInt(signed=False, label="UInt32")), ("StartAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EndAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="D3DKMT_VAD_DESC", pack=False, align=None),
    "D3DKMT_VAD_ESCAPE_COMMAND": SimTypeInt(signed=False, label="D3DKMT_VAD_ESCAPE_COMMAND"),
    "D3DKMT_VA_RANGE_DESC": SimStruct(OrderedDict((("VadAddress", SimTypeLongLong(signed=False, label="UInt64")), ("VaRangeIndex", SimTypeInt(signed=False, label="UInt32")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("StartAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EndAddress", SimTypeLongLong(signed=False, label="UInt64")), ("DriverProtection", SimTypeLongLong(signed=False, label="UInt64")), ("OwnerType", SimTypeInt(signed=False, label="UInt32")), ("pOwner", SimTypeLongLong(signed=False, label="UInt64")), ("OwnerOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Protection", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_VA_RANGE_DESC", pack=False, align=None),
    "D3DKMT_VERIFIER_OPTION_MODE": SimTypeInt(signed=False, label="D3DKMT_VERIFIER_OPTION_MODE"),
    "D3DKMT_VGPUINTERFACEID": SimStruct(OrderedDict((("VirtualGpuIntefaceId", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_VGPUINTERFACEID", pack=False, align=None),
    "D3DKMT_VIDMMESCAPETYPE": SimTypeInt(signed=False, label="D3DKMT_VIDMMESCAPETYPE"),
    "D3DKMT_VIDMM_ESCAPE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_VIDMMESCAPETYPE")), ("Anonymous", SimUnion({"SetFault": SimTypeRef("_SetFault_e__Struct", SimStruct), "Evict": SimTypeRef("_Evict_e__Struct", SimStruct), "EvictByNtHandle": SimTypeRef("_EvictByNtHandle_e__Struct", SimStruct), "GetVads": SimTypeRef("_GetVads_e__Struct", SimStruct), "SetBudget": SimTypeRef("_SetBudget_e__Struct", SimStruct), "SuspendProcess": SimTypeRef("_SuspendProcess_e__Struct", SimStruct), "ResumeProcess": SimTypeRef("_ResumeProcess_e__Struct", SimStruct), "GetBudget": SimTypeRef("_GetBudget_e__Struct", SimStruct), "SetTrimIntervals": SimTypeRef("_SetTrimIntervals_e__Struct", SimStruct), "EvictByCriteria": SimTypeRef("D3DKMT_EVICTION_CRITERIA", SimStruct), "Wake": SimTypeRef("_Wake_e__Struct", SimStruct), "Defrag": SimTypeRef("_Defrag_e__Struct", SimStruct), "DelayExecution": SimTypeRef("_DelayExecution_e__Struct", SimStruct), "VerifyIntegrity": SimTypeRef("_VerifyIntegrity_e__Struct", SimStruct), "DelayedEvictionConfig": SimTypeRef("_DelayedEvictionConfig_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="D3DKMT_VIDMM_ESCAPE", pack=False, align=None),
    "D3DKMT_VIDPNSOURCEOWNER_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_VIDPNSOURCEOWNER_FLAGS", pack=False, align=None),
    "D3DKMT_VIDPNSOURCEOWNER_TYPE": SimTypeInt(signed=False, label="D3DKMT_VIDPNSOURCEOWNER_TYPE"),
    "D3DKMT_VIDSCHESCAPETYPE": SimTypeInt(signed=False, label="D3DKMT_VIDSCHESCAPETYPE"),
    "D3DKMT_VIDSCH_ESCAPE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DKMT_VIDSCHESCAPETYPE")), ("Anonymous", SimUnion({"PreemptionControl": SimTypeInt(signed=True, label="Int32"), "EnableContextDelay": SimTypeInt(signed=True, label="Int32"), "TdrControl2": SimTypeRef("_TdrControl2_e__Struct", SimStruct), "SuspendScheduler": SimTypeInt(signed=True, label="Int32"), "TdrControl": SimTypeInt(signed=False, label="UInt32"), "SuspendTime": SimTypeInt(signed=False, label="UInt32"), "TdrLimit": SimTypeRef("_TdrLimit_e__Struct", SimStruct), "PfnControl": SimTypeInt(signed=False, label="D3DKMT_ESCAPE_PFN_CONTROL_COMMAND")}, name="<anon>", label="None")), ("VirtualRefreshRateControl", SimTypeRef("D3DKMT_ESCAPE_VIRTUAL_REFRESH_RATE", SimStruct)),)), name="D3DKMT_VIDSCH_ESCAPE", pack=False, align=None),
    "D3DKMT_VIRTUALADDRESSFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_VIRTUALADDRESSFLAGS", pack=False, align=None),
    "D3DKMT_VIRTUALADDRESSINFO": SimStruct(OrderedDict((("VirtualAddressFlags", SimTypeRef("D3DKMT_VIRTUALADDRESSFLAGS", SimStruct)),)), name="D3DKMT_VIRTUALADDRESSINFO", pack=False, align=None),
    "D3DKMT_WAITFORIDLE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_WAITFORIDLE", pack=False, align=None),
    "D3DKMT_WAITFORSYNCHRONIZATIONOBJECT": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)),)), name="D3DKMT_WAITFORSYNCHRONIZATIONOBJECT", pack=False, align=None),
    "D3DKMT_WAITFORSYNCHRONIZATIONOBJECT2": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("Anonymous", SimUnion({"Fence": SimTypeRef("_Fence_e__Struct", SimStruct), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)}, name="<anon>", label="None")),)), name="D3DKMT_WAITFORSYNCHRONIZATIONOBJECT2", pack=False, align=None),
    "D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("FenceValueArray", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("hAsyncEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeRef("D3DDDI_WAITFORSYNCHRONIZATIONOBJECTFROMCPU_FLAGS", SimStruct)),)), name="D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU", pack=False, align=None),
    "D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMGPU": SimStruct(OrderedDict((("hContext", SimTypeInt(signed=False, label="UInt32")), ("ObjectCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Anonymous", SimUnion({"MonitoredFenceValueArray": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "FenceValue": SimTypeLongLong(signed=False, label="UInt64"), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)}, name="<anon>", label="None")),)), name="D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMGPU", pack=False, align=None),
    "D3DKMT_WAITFORVERTICALBLANKEVENT": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_WAITFORVERTICALBLANKEVENT", pack=False, align=None),
    "D3DKMT_WAITFORVERTICALBLANKEVENT2": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("NumObjects", SimTypeInt(signed=False, label="UInt32")), ("ObjectHandleArray", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 8)),)), name="D3DKMT_WAITFORVERTICALBLANKEVENT2", pack=False, align=None),
    "D3DKMT_WDDM_1_2_CAPS": SimStruct(OrderedDict((("PreemptionCaps", SimTypeRef("D3DKMDT_PREEMPTION_CAPS", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_1_2_CAPS", pack=False, align=None),
    "D3DKMT_WDDM_1_3_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_1_3_CAPS", pack=False, align=None),
    "D3DKMT_WDDM_2_0_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_2_0_CAPS", pack=False, align=None),
    "D3DKMT_WDDM_2_7_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_2_7_CAPS", pack=False, align=None),
    "D3DKMT_WDDM_2_9_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_2_9_CAPS", pack=False, align=None),
    "D3DKMT_WDDM_3_0_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_3_0_CAPS", pack=False, align=None),
    "D3DKMT_WDDM_3_1_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DKMT_WDDM_3_1_CAPS", pack=False, align=None),
    "D3DKMT_WORKINGSETFLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_WORKINGSETFLAGS", pack=False, align=None),
    "D3DKMT_WORKINGSETINFO": SimStruct(OrderedDict((("Flags", SimTypeRef("D3DKMT_WORKINGSETFLAGS", SimStruct)), ("MinimumWorkingSetPercentile", SimTypeInt(signed=False, label="UInt32")), ("MaximumWorkingSetPercentile", SimTypeInt(signed=False, label="UInt32")),)), name="D3DKMT_WORKINGSETINFO", pack=False, align=None),
    "D3DKMT_WSAUMDIMAGENAME": SimStruct(OrderedDict((("WsaUmdImageName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="D3DKMT_WSAUMDIMAGENAME", pack=False, align=None),
    "D3DKMT_XBOX": SimStruct(OrderedDict((("IsXBOX", SimTypeInt(signed=True, label="Int32")),)), name="D3DKMT_XBOX", pack=False, align=None),
    "D3DLIGHT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dltType", SimTypeInt(signed=False, label="D3DLIGHTTYPE")), ("dcvColor", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("dvPosition", SimTypeRef("D3DVECTOR", SimStruct)), ("dvDirection", SimTypeRef("D3DVECTOR", SimStruct)), ("dvRange", SimTypeFloat(size=32)), ("dvFalloff", SimTypeFloat(size=32)), ("dvAttenuation0", SimTypeFloat(size=32)), ("dvAttenuation1", SimTypeFloat(size=32)), ("dvAttenuation2", SimTypeFloat(size=32)), ("dvTheta", SimTypeFloat(size=32)), ("dvPhi", SimTypeFloat(size=32)),)), name="D3DLIGHT", pack=False, align=None),
    "D3DLIGHT2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dltType", SimTypeInt(signed=False, label="D3DLIGHTTYPE")), ("dcvColor", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("dvPosition", SimTypeRef("D3DVECTOR", SimStruct)), ("dvDirection", SimTypeRef("D3DVECTOR", SimStruct)), ("dvRange", SimTypeFloat(size=32)), ("dvFalloff", SimTypeFloat(size=32)), ("dvAttenuation0", SimTypeFloat(size=32)), ("dvAttenuation1", SimTypeFloat(size=32)), ("dvAttenuation2", SimTypeFloat(size=32)), ("dvTheta", SimTypeFloat(size=32)), ("dvPhi", SimTypeFloat(size=32)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DLIGHT2", pack=False, align=None),
    "D3DLIGHT7": SimStruct(OrderedDict((("dltType", SimTypeInt(signed=False, label="D3DLIGHTTYPE")), ("dcvDiffuse", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("dcvSpecular", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("dcvAmbient", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("dvPosition", SimTypeRef("D3DVECTOR", SimStruct)), ("dvDirection", SimTypeRef("D3DVECTOR", SimStruct)), ("dvRange", SimTypeFloat(size=32)), ("dvFalloff", SimTypeFloat(size=32)), ("dvAttenuation0", SimTypeFloat(size=32)), ("dvAttenuation1", SimTypeFloat(size=32)), ("dvAttenuation2", SimTypeFloat(size=32)), ("dvTheta", SimTypeFloat(size=32)), ("dvPhi", SimTypeFloat(size=32)),)), name="D3DLIGHT7", pack=False, align=None),
    "D3DLIGHT9": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="D3DLIGHTTYPE")), ("Diffuse", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Specular", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Ambient", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Position", SimTypeRef("D3DVECTOR", SimStruct)), ("Direction", SimTypeRef("D3DVECTOR", SimStruct)), ("Range", SimTypeFloat(size=32)), ("Falloff", SimTypeFloat(size=32)), ("Attenuation0", SimTypeFloat(size=32)), ("Attenuation1", SimTypeFloat(size=32)), ("Attenuation2", SimTypeFloat(size=32)), ("Theta", SimTypeFloat(size=32)), ("Phi", SimTypeFloat(size=32)),)), name="D3DLIGHT9", pack=False, align=None),
    "D3DLIGHTDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpIn", SimTypePointer(SimTypeRef("D3DLIGHTINGELEMENT", SimStruct), offset=0)), ("dwInSize", SimTypeInt(signed=False, label="UInt32")), ("lpOut", SimTypePointer(SimTypeRef("D3DTLVERTEX", SimStruct), offset=0)), ("dwOutSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DLIGHTDATA", pack=False, align=None),
    "D3DLIGHTINGCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwLightingModel", SimTypeInt(signed=False, label="UInt32")), ("dwNumLights", SimTypeInt(signed=False, label="UInt32")),)), name="D3DLIGHTINGCAPS", pack=False, align=None),
    "D3DLIGHTINGELEMENT": SimStruct(OrderedDict((("dvPosition", SimTypeRef("D3DVECTOR", SimStruct)), ("dvNormal", SimTypeRef("D3DVECTOR", SimStruct)),)), name="D3DLIGHTINGELEMENT", pack=False, align=None),
    "D3DLIGHTSTATETYPE": SimTypeInt(signed=False, label="D3DLIGHTSTATETYPE"),
    "D3DLIGHTTYPE": SimTypeInt(signed=False, label="D3DLIGHTTYPE"),
    "D3DLINE": SimStruct(OrderedDict((("Anonymous1", SimUnion({"v1": SimTypeShort(signed=False, label="UInt16"), "wV1": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"v2": SimTypeShort(signed=False, label="UInt16"), "wV2": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="D3DLINE", pack=False, align=None),
    "D3DLINEPATTERN": SimStruct(OrderedDict((("wRepeatFactor", SimTypeShort(signed=False, label="UInt16")), ("wLinePattern", SimTypeShort(signed=False, label="UInt16")),)), name="D3DLINEPATTERN", pack=False, align=None),
    "D3DLOCKED_BOX": SimStruct(OrderedDict((("RowPitch", SimTypeInt(signed=True, label="Int32")), ("SlicePitch", SimTypeInt(signed=True, label="Int32")), ("pBits", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DLOCKED_BOX", pack=False, align=None),
    "D3DLOCKED_RECT": SimStruct(OrderedDict((("Pitch", SimTypeInt(signed=True, label="Int32")), ("pBits", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DLOCKED_RECT", pack=False, align=None),
    "D3DLVERTEX": SimStruct(OrderedDict((("Anonymous1", SimUnion({"x": SimTypeFloat(size=32), "dvX": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"y": SimTypeFloat(size=32), "dvY": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"z": SimTypeFloat(size=32), "dvZ": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("Anonymous4", SimUnion({"color": SimTypeInt(signed=False, label="UInt32"), "dcColor": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"specular": SimTypeInt(signed=False, label="UInt32"), "dcSpecular": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous6", SimUnion({"tu": SimTypeFloat(size=32), "dvTU": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous7", SimUnion({"tv": SimTypeFloat(size=32), "dvTV": SimTypeFloat(size=32)}, name="<anon>", label="None")),)), name="D3DLVERTEX", pack=False, align=None),
    "D3DMATERIAL": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"diffuse": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvDiffuse": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"ambient": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvAmbient": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"specular": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvSpecular": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"emissive": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvEmissive": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"power": SimTypeFloat(size=32), "dvPower": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("hTexture", SimTypeInt(signed=False, label="UInt32")), ("dwRampSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DMATERIAL", pack=False, align=None),
    "D3DMATERIAL7": SimStruct(OrderedDict((("Anonymous1", SimUnion({"diffuse": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvDiffuse": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"ambient": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvAmbient": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"specular": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvSpecular": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"emissive": SimTypeRef("D3DCOLORVALUE", SimStruct), "dcvEmissive": SimTypeRef("D3DCOLORVALUE", SimStruct)}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"power": SimTypeFloat(size=32), "dvPower": SimTypeFloat(size=32)}, name="<anon>", label="None")),)), name="D3DMATERIAL7", pack=False, align=None),
    "D3DMATERIAL9": SimStruct(OrderedDict((("Diffuse", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Ambient", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Specular", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Emissive", SimTypeRef("D3DCOLORVALUE", SimStruct)), ("Power", SimTypeFloat(size=32)),)), name="D3DMATERIAL9", pack=False, align=None),
    "D3DMATERIALCOLORSOURCE": SimTypeInt(signed=False, label="D3DMATERIALCOLORSOURCE"),
    "D3DMATRIX": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_13", SimTypeFloat(size=32)), ("_14", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_23", SimTypeFloat(size=32)), ("_24", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)), ("_33", SimTypeFloat(size=32)), ("_34", SimTypeFloat(size=32)), ("_41", SimTypeFloat(size=32)), ("_42", SimTypeFloat(size=32)), ("_43", SimTypeFloat(size=32)), ("_44", SimTypeFloat(size=32)),)), name="_Anonymous_e__Struct", pack=False, align=None), "m": SimTypeArray(SimTypeFloat(size=32), 16)}, name="<anon>", label="None")),)), name="D3DMATRIX", pack=False, align=None),
    "D3DMATRIXLOAD": SimStruct(OrderedDict((("hDestMatrix", SimTypeInt(signed=False, label="UInt32")), ("hSrcMatrix", SimTypeInt(signed=False, label="UInt32")),)), name="D3DMATRIXLOAD", pack=False, align=None),
    "D3DMATRIXMULTIPLY": SimStruct(OrderedDict((("hDestMatrix", SimTypeInt(signed=False, label="UInt32")), ("hSrcMatrix1", SimTypeInt(signed=False, label="UInt32")), ("hSrcMatrix2", SimTypeInt(signed=False, label="UInt32")),)), name="D3DMATRIXMULTIPLY", pack=False, align=None),
    "D3DMEMORYPRESSURE": SimStruct(OrderedDict((("BytesEvictedFromProcess", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfInefficientAllocation", SimTypeLongLong(signed=False, label="UInt64")), ("LevelOfEfficiency", SimTypeInt(signed=False, label="UInt32")),)), name="D3DMEMORYPRESSURE", pack=False, align=None),
    "D3DMULTISAMPLE_TYPE": SimTypeInt(signed=False, label="D3DMULTISAMPLE_TYPE"),
    "D3DNTDEVICEDESC_V3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTDEVICEDESC_V3", pack=False, align=None),
    "D3DNTHALDEVICEDESC_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHALDEVICEDESC_V1", pack=False, align=None),
    "D3DNTHALDEVICEDESC_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dcmColorModel", SimTypeInt(signed=False, label="UInt32")), ("dwDevCaps", SimTypeInt(signed=False, label="UInt32")), ("dtcTransformCaps", SimTypeRef("D3DTRANSFORMCAPS", SimStruct)), ("bClipping", SimTypeInt(signed=True, label="Int32")), ("dlcLightingCaps", SimTypeRef("D3DLIGHTINGCAPS", SimStruct)), ("dpcLineCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dpcTriCaps", SimTypeRef("D3DPRIMCAPS", SimStruct)), ("dwDeviceRenderBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceZBufferBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVertexCount", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHALDEVICEDESC_V2", pack=False, align=None),
    "D3DNTHAL_CALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ContextCreate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CONTEXTCREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ContextDestroy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CONTEXTDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ContextDestroyAll", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CONTEXTDESTROYALLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SceneCapture", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_SCENECAPTUREDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwReserved10", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved11", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved22", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved23", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TextureCreate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTURECREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("TextureDestroy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTUREDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("TextureSwap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTURESWAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("TextureGetSurf", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTUREGETSURFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwReserved12", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved13", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved14", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved15", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved16", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved17", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved18", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved19", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved20", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved21", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved24", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved0", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved4", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved5", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved6", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved7", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved8", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved9", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3DNTHAL_CALLBACKS", pack=False, align=None),
    "D3DNTHAL_CALLBACKS2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("SetRenderTarget", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_SETRENDERTARGETDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwReserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="D3DNTHAL_CALLBACKS2", pack=False, align=None),
    "D3DNTHAL_CALLBACKS3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Clear2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CLEAR2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValidateTextureStageState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_VALIDATETEXTURESTAGESTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DrawPrimitives2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_DRAWPRIMITIVES2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="D3DNTHAL_CALLBACKS3", pack=False, align=None),
    "D3DNTHAL_CLEAR2DATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFillColor", SimTypeInt(signed=False, label="UInt32")), ("dvFillDepth", SimTypeFloat(size=32)), ("dwFillStencil", SimTypeInt(signed=False, label="UInt32")), ("lpRects", SimTypePointer(SimTypeRef("D3DRECT", SimStruct), offset=0)), ("dwNumRects", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_CLEAR2DATA", pack=False, align=None),
    "D3DNTHAL_CLIPPEDTRIANGLEFAN": SimStruct(OrderedDict((("FirstVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("dwEdgeFlags", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_CLIPPEDTRIANGLEFAN", pack=False, align=None),
    "D3DNTHAL_CONTEXTCREATEDATA": SimStruct(OrderedDict((("Anonymous1", SimUnion({"lpDDGbl": SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0), "lpDDLcl": SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"lpDDS": SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0), "lpDDSLcl": SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"lpDDSZ": SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0), "lpDDSZLcl": SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)}, name="<anon>", label="None")), ("dwPID", SimTypeInt(signed=False, label="UInt32")), ("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_CONTEXTCREATEDATA", pack=False, align=None),
    "D3DNTHAL_CONTEXTDESTROYALLDATA": SimStruct(OrderedDict((("dwPID", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_CONTEXTDESTROYALLDATA", pack=False, align=None),
    "D3DNTHAL_CONTEXTDESTROYDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_CONTEXTDESTROYDATA", pack=False, align=None),
    "D3DNTHAL_D3DDX6EXTENDEDCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_D3DDX6EXTENDEDCAPS", pack=False, align=None),
    "D3DNTHAL_D3DEXTENDEDCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMinStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxStippleHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureRepeat", SimTypeInt(signed=False, label="UInt32")), ("dwMaxTextureAspectRatio", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAnisotropy", SimTypeInt(signed=False, label="UInt32")), ("dvGuardBandLeft", SimTypeFloat(size=32)), ("dvGuardBandTop", SimTypeFloat(size=32)), ("dvGuardBandRight", SimTypeFloat(size=32)), ("dvGuardBandBottom", SimTypeFloat(size=32)), ("dvExtentsAdjust", SimTypeFloat(size=32)), ("dwStencilCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVFCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureOpCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxTextureBlendStages", SimTypeShort(signed=False, label="UInt16")), ("wMaxSimultaneousTextures", SimTypeShort(signed=False, label="UInt16")), ("dwMaxActiveLights", SimTypeInt(signed=False, label="UInt32")), ("dvMaxVertexW", SimTypeFloat(size=32)), ("wMaxUserClipPlanes", SimTypeShort(signed=False, label="UInt16")), ("wMaxVertexBlendMatrices", SimTypeShort(signed=False, label="UInt16")), ("dwVertexProcessingCaps", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_D3DEXTENDEDCAPS", pack=False, align=None),
    "D3DNTHAL_DP2ADDDIRTYBOX": SimStruct(OrderedDict((("dwSurface", SimTypeInt(signed=False, label="UInt32")), ("DirtyBox", SimTypeRef("D3DBOX", SimStruct)),)), name="D3DNTHAL_DP2ADDDIRTYBOX", pack=False, align=None),
    "D3DNTHAL_DP2ADDDIRTYRECT": SimStruct(OrderedDict((("dwSurface", SimTypeInt(signed=False, label="UInt32")), ("rDirtyArea", SimTypeRef("RECTL", SimStruct)),)), name="D3DNTHAL_DP2ADDDIRTYRECT", pack=False, align=None),
    "D3DNTHAL_DP2BLT": SimStruct(OrderedDict((("dwSource", SimTypeInt(signed=False, label="UInt32")), ("rSource", SimTypeRef("RECTL", SimStruct)), ("dwSourceMipLevel", SimTypeInt(signed=False, label="UInt32")), ("dwDest", SimTypeInt(signed=False, label="UInt32")), ("rDest", SimTypeRef("RECTL", SimStruct)), ("dwDestMipLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2BLT", pack=False, align=None),
    "D3DNTHAL_DP2BUFFERBLT": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDDSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("rSrc", SimTypeRef("D3DRANGE", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2BUFFERBLT", pack=False, align=None),
    "D3DNTHAL_DP2CLEAR": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFillColor", SimTypeInt(signed=False, label="UInt32")), ("dvFillDepth", SimTypeFloat(size=32)), ("dwFillStencil", SimTypeInt(signed=False, label="UInt32")), ("Rects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)),)), name="D3DNTHAL_DP2CLEAR", pack=False, align=None),
    "D3DNTHAL_DP2COLORFILL": SimStruct(OrderedDict((("dwSurface", SimTypeInt(signed=False, label="UInt32")), ("rRect", SimTypeRef("RECTL", SimStruct)), ("Color", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2COLORFILL", pack=False, align=None),
    "D3DNTHAL_DP2COMMAND": SimStruct(OrderedDict((("bCommand", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"wPrimitiveCount": SimTypeShort(signed=False, label="UInt16"), "wStateCount": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="D3DNTHAL_DP2COMMAND", pack=False, align=None),
    "D3DNTHAL_DP2COMPOSERECTS": SimStruct(OrderedDict((("SrcSurfaceHandle", SimTypeInt(signed=False, label="UInt32")), ("DstSurfaceHandle", SimTypeInt(signed=False, label="UInt32")), ("SrcRectDescsVBHandle", SimTypeInt(signed=False, label="UInt32")), ("NumRects", SimTypeInt(signed=False, label="UInt32")), ("DstRectDescsVBHandle", SimTypeInt(signed=False, label="UInt32")), ("Operation", SimTypeInt(signed=False, label="D3DCOMPOSERECTSOP")), ("XOffset", SimTypeInt(signed=True, label="Int32")), ("YOffset", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_DP2COMPOSERECTS", pack=False, align=None),
    "D3DNTHAL_DP2CREATELIGHT": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2CREATELIGHT", pack=False, align=None),
    "D3DNTHAL_DP2CREATEPIXELSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwCodeSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2CREATEPIXELSHADER", pack=False, align=None),
    "D3DNTHAL_DP2CREATEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")), ("QueryType", SimTypeInt(signed=False, label="D3DQUERYTYPE")),)), name="D3DNTHAL_DP2CREATEQUERY", pack=False, align=None),
    "D3DNTHAL_DP2CREATEVERTEXSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwDeclSize", SimTypeInt(signed=False, label="UInt32")), ("dwCodeSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2CREATEVERTEXSHADER", pack=False, align=None),
    "D3DNTHAL_DP2CREATEVERTEXSHADERDECL": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwNumVertexElements", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2CREATEVERTEXSHADERDECL", pack=False, align=None),
    "D3DNTHAL_DP2CREATEVERTEXSHADERFUNC": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2CREATEVERTEXSHADERFUNC", pack=False, align=None),
    "D3DNTHAL_DP2DELETEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DELETEQUERY", pack=False, align=None),
    "D3DNTHAL_DP2DRAWINDEXEDPRIMITIVE": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("BaseVertexIndex", SimTypeInt(signed=True, label="Int32")), ("MinIndex", SimTypeInt(signed=False, label="UInt32")), ("NumVertices", SimTypeInt(signed=False, label="UInt32")), ("StartIndex", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DRAWINDEXEDPRIMITIVE", pack=False, align=None),
    "D3DNTHAL_DP2DRAWINDEXEDPRIMITIVE2": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("BaseVertexOffset", SimTypeInt(signed=True, label="Int32")), ("MinIndex", SimTypeInt(signed=False, label="UInt32")), ("NumVertices", SimTypeInt(signed=False, label="UInt32")), ("StartIndexOffset", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DRAWINDEXEDPRIMITIVE2", pack=False, align=None),
    "D3DNTHAL_DP2DRAWPRIMITIVE": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("VStart", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DRAWPRIMITIVE", pack=False, align=None),
    "D3DNTHAL_DP2DRAWPRIMITIVE2": SimStruct(OrderedDict((("primType", SimTypeInt(signed=False, label="D3DPRIMITIVETYPE")), ("FirstVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("PrimitiveCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DRAWPRIMITIVE2", pack=False, align=None),
    "D3DNTHAL_DP2DRAWRECTPATCH": SimStruct(OrderedDict((("Handle", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DRAWRECTPATCH", pack=False, align=None),
    "D3DNTHAL_DP2DRAWTRIPATCH": SimStruct(OrderedDict((("Handle", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2DRAWTRIPATCH", pack=False, align=None),
    "D3DNTHAL_DP2EXT": SimStruct(OrderedDict((("dwExtToken", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2EXT", pack=False, align=None),
    "D3DNTHAL_DP2GENERATEMIPSUBLEVELS": SimStruct(OrderedDict((("hSurface", SimTypeInt(signed=False, label="UInt32")), ("Filter", SimTypeInt(signed=False, label="D3DTEXTUREFILTERTYPE")),)), name="D3DNTHAL_DP2GENERATEMIPSUBLEVELS", pack=False, align=None),
    "D3DNTHAL_DP2INDEXEDLINELIST": SimStruct(OrderedDict((("wV1", SimTypeShort(signed=False, label="UInt16")), ("wV2", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2INDEXEDLINELIST", pack=False, align=None),
    "D3DNTHAL_DP2INDEXEDLINESTRIP": SimStruct(OrderedDict((("wV", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)),)), name="D3DNTHAL_DP2INDEXEDLINESTRIP", pack=False, align=None),
    "D3DNTHAL_DP2INDEXEDTRIANGLEFAN": SimStruct(OrderedDict((("wV", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)),)), name="D3DNTHAL_DP2INDEXEDTRIANGLEFAN", pack=False, align=None),
    "D3DNTHAL_DP2INDEXEDTRIANGLELIST": SimStruct(OrderedDict((("wV1", SimTypeShort(signed=False, label="UInt16")), ("wV2", SimTypeShort(signed=False, label="UInt16")), ("wV3", SimTypeShort(signed=False, label="UInt16")), ("wFlags", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2INDEXEDTRIANGLELIST", pack=False, align=None),
    "D3DNTHAL_DP2INDEXEDTRIANGLELIST2": SimStruct(OrderedDict((("wV1", SimTypeShort(signed=False, label="UInt16")), ("wV2", SimTypeShort(signed=False, label="UInt16")), ("wV3", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2INDEXEDTRIANGLELIST2", pack=False, align=None),
    "D3DNTHAL_DP2INDEXEDTRIANGLESTRIP": SimStruct(OrderedDict((("wV", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)),)), name="D3DNTHAL_DP2INDEXEDTRIANGLESTRIP", pack=False, align=None),
    "D3DNTHAL_DP2ISSUEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2ISSUEQUERY", pack=False, align=None),
    "D3DNTHAL_DP2LINELIST": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2LINELIST", pack=False, align=None),
    "D3DNTHAL_DP2LINESTRIP": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2LINESTRIP", pack=False, align=None),
    "D3DNTHAL_DP2MULTIPLYTRANSFORM": SimStruct(OrderedDict((("xfrmType", SimTypeInt(signed=False, label="D3DTRANSFORMSTATETYPE")), ("matrix", SimTypeRef("D3DMATRIX", SimStruct)),)), name="D3DNTHAL_DP2MULTIPLYTRANSFORM", pack=False, align=None),
    "D3DNTHAL_DP2OPERATION": SimTypeInt(signed=False, label="D3DNTHAL_DP2OPERATION"),
    "D3DNTHAL_DP2PIXELSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2PIXELSHADER", pack=False, align=None),
    "D3DNTHAL_DP2POINTS": SimStruct(OrderedDict((("wCount", SimTypeShort(signed=False, label="UInt16")), ("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2POINTS", pack=False, align=None),
    "D3DNTHAL_DP2RENDERSTATE": SimStruct(OrderedDict((("RenderState", SimTypeInt(signed=False, label="D3DRENDERSTATETYPE")), ("Anonymous", SimUnion({"fState": SimTypeFloat(size=32), "dwState": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DNTHAL_DP2RENDERSTATE", pack=False, align=None),
    "D3DNTHAL_DP2RESPONSE": SimStruct(OrderedDict((("bCommand", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("wStateCount", SimTypeShort(signed=False, label="UInt16")), ("dwTotalSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2RESPONSE", pack=False, align=None),
    "D3DNTHAL_DP2RESPONSEQUERY": SimStruct(OrderedDict((("dwQueryID", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2RESPONSEQUERY", pack=False, align=None),
    "D3DNTHAL_DP2SETCLIPPLANE": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("plane", SimTypeArray(SimTypeFloat(size=32), 4)),)), name="D3DNTHAL_DP2SETCLIPPLANE", pack=False, align=None),
    "D3DNTHAL_DP2SETCONVOLUTIONKERNELMONO": SimStruct(OrderedDict((("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETCONVOLUTIONKERNELMONO", pack=False, align=None),
    "D3DNTHAL_DP2SETDEPTHSTENCIL": SimStruct(OrderedDict((("hZBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETDEPTHSTENCIL", pack=False, align=None),
    "D3DNTHAL_DP2SETINDICES": SimStruct(OrderedDict((("dwVBHandle", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETINDICES", pack=False, align=None),
    "D3DNTHAL_DP2SETLIGHT": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"lightData": SimTypeInt(signed=False, label="UInt32"), "dwDataType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="D3DNTHAL_DP2SETLIGHT", pack=False, align=None),
    "D3DNTHAL_DP2SETPALETTE": SimStruct(OrderedDict((("dwPaletteHandle", SimTypeInt(signed=False, label="UInt32")), ("dwPaletteFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSurfaceHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETPALETTE", pack=False, align=None),
    "D3DNTHAL_DP2SETPIXELSHADERCONST": SimStruct(OrderedDict((("dwRegister", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETPIXELSHADERCONST", pack=False, align=None),
    "D3DNTHAL_DP2SETPRIORITY": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwPriority", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETPRIORITY", pack=False, align=None),
    "D3DNTHAL_DP2SETRENDERTARGET": SimStruct(OrderedDict((("hRenderTarget", SimTypeInt(signed=False, label="UInt32")), ("hZBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETRENDERTARGET", pack=False, align=None),
    "D3DNTHAL_DP2SETRENDERTARGET2": SimStruct(OrderedDict((("RTIndex", SimTypeInt(signed=False, label="UInt32")), ("hRenderTarget", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETRENDERTARGET2", pack=False, align=None),
    "D3DNTHAL_DP2SETSTREAMSOURCE": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwVBHandle", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETSTREAMSOURCE", pack=False, align=None),
    "D3DNTHAL_DP2SETSTREAMSOURCE2": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwVBHandle", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETSTREAMSOURCE2", pack=False, align=None),
    "D3DNTHAL_DP2SETSTREAMSOURCEFREQ": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwDivider", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETSTREAMSOURCEFREQ", pack=False, align=None),
    "D3DNTHAL_DP2SETSTREAMSOURCEUM": SimStruct(OrderedDict((("dwStream", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETSTREAMSOURCEUM", pack=False, align=None),
    "D3DNTHAL_DP2SETTEXLOD": SimStruct(OrderedDict((("dwDDSurface", SimTypeInt(signed=False, label="UInt32")), ("dwLOD", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETTEXLOD", pack=False, align=None),
    "D3DNTHAL_DP2SETTRANSFORM": SimStruct(OrderedDict((("xfrmType", SimTypeInt(signed=False, label="D3DTRANSFORMSTATETYPE")), ("matrix", SimTypeRef("D3DMATRIX", SimStruct)),)), name="D3DNTHAL_DP2SETTRANSFORM", pack=False, align=None),
    "D3DNTHAL_DP2SETVERTEXSHADERCONST": SimStruct(OrderedDict((("dwRegister", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SETVERTEXSHADERCONST", pack=False, align=None),
    "D3DNTHAL_DP2STARTVERTEX": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2STARTVERTEX", pack=False, align=None),
    "D3DNTHAL_DP2STATESET": SimStruct(OrderedDict((("dwOperation", SimTypeInt(signed=False, label="UInt32")), ("dwParam", SimTypeInt(signed=False, label="UInt32")), ("sbType", SimTypeInt(signed=False, label="D3DSTATEBLOCKTYPE")),)), name="D3DNTHAL_DP2STATESET", pack=False, align=None),
    "D3DNTHAL_DP2SURFACEBLT": SimStruct(OrderedDict((("dwSource", SimTypeInt(signed=False, label="UInt32")), ("rSource", SimTypeRef("RECTL", SimStruct)), ("dwSourceMipLevel", SimTypeInt(signed=False, label="UInt32")), ("dwDest", SimTypeInt(signed=False, label="UInt32")), ("rDest", SimTypeRef("RECTL", SimStruct)), ("dwDestMipLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2SURFACEBLT", pack=False, align=None),
    "D3DNTHAL_DP2TEXBLT": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDDSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("pDest", SimTypeRef("POINT", SimStruct)), ("rSrc", SimTypeRef("RECTL", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2TEXBLT", pack=False, align=None),
    "D3DNTHAL_DP2TEXTURESTAGESTATE": SimStruct(OrderedDict((("wStage", SimTypeShort(signed=False, label="UInt16")), ("TSState", SimTypeShort(signed=False, label="UInt16")), ("dwValue", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2TEXTURESTAGESTATE", pack=False, align=None),
    "D3DNTHAL_DP2TRIANGLEFAN": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2TRIANGLEFAN", pack=False, align=None),
    "D3DNTHAL_DP2TRIANGLEFAN_IMM": SimStruct(OrderedDict((("dwEdgeFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2TRIANGLEFAN_IMM", pack=False, align=None),
    "D3DNTHAL_DP2TRIANGLELIST": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2TRIANGLELIST", pack=False, align=None),
    "D3DNTHAL_DP2TRIANGLESTRIP": SimStruct(OrderedDict((("wVStart", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2TRIANGLESTRIP", pack=False, align=None),
    "D3DNTHAL_DP2UPDATEPALETTE": SimStruct(OrderedDict((("dwPaletteHandle", SimTypeInt(signed=False, label="UInt32")), ("wStartIndex", SimTypeShort(signed=False, label="UInt16")), ("wNumEntries", SimTypeShort(signed=False, label="UInt16")),)), name="D3DNTHAL_DP2UPDATEPALETTE", pack=False, align=None),
    "D3DNTHAL_DP2VERTEXSHADER": SimStruct(OrderedDict((("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2VERTEXSHADER", pack=False, align=None),
    "D3DNTHAL_DP2VIEWPORTINFO": SimStruct(OrderedDict((("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2VIEWPORTINFO", pack=False, align=None),
    "D3DNTHAL_DP2VOLUMEBLT": SimStruct(OrderedDict((("dwDDDestSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDDSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("dwDestX", SimTypeInt(signed=False, label="UInt32")), ("dwDestY", SimTypeInt(signed=False, label="UInt32")), ("dwDestZ", SimTypeInt(signed=False, label="UInt32")), ("srcBox", SimTypeRef("D3DBOX", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DP2VOLUMEBLT", pack=False, align=None),
    "D3DNTHAL_DP2WINFO": SimStruct(OrderedDict((("dvWNear", SimTypeFloat(size=32)), ("dvWFar", SimTypeFloat(size=32)),)), name="D3DNTHAL_DP2WINFO", pack=False, align=None),
    "D3DNTHAL_DP2ZRANGE": SimStruct(OrderedDict((("dvMinZ", SimTypeFloat(size=32)), ("dvMaxZ", SimTypeFloat(size=32)),)), name="D3DNTHAL_DP2ZRANGE", pack=False, align=None),
    "D3DNTHAL_DRAWPRIMITIVES2DATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwVertexType", SimTypeInt(signed=False, label="UInt32")), ("lpDDCommands", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwCommandOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCommandLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lpDDVertex": SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0), "lpVertices": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("dwVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("dwVertexLength", SimTypeInt(signed=False, label="UInt32")), ("dwReqVertexBufSize", SimTypeInt(signed=False, label="UInt32")), ("dwReqCommandBufSize", SimTypeInt(signed=False, label="UInt32")), ("lpdwRStates", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Anonymous2", SimUnion({"dwVertexSize": SimTypeInt(signed=False, label="UInt32"), "ddrval": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("dwErrorOffset", SimTypeInt(signed=False, label="UInt32")),)), name="D3DNTHAL_DRAWPRIMITIVES2DATA", pack=False, align=None),
    "D3DNTHAL_GLOBALDRIVERDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwCaps", SimTypeRef("D3DNTHALDEVICEDESC_V1", SimStruct)), ("dwNumVertices", SimTypeInt(signed=False, label="UInt32")), ("dwNumClipVertices", SimTypeInt(signed=False, label="UInt32")), ("dwNumTextureFormats", SimTypeInt(signed=False, label="UInt32")), ("lpTextureFormats", SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0)),)), name="D3DNTHAL_GLOBALDRIVERDATA", pack=False, align=None),
    "D3DNTHAL_SCENECAPTUREDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlag", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_SCENECAPTUREDATA", pack=False, align=None),
    "D3DNTHAL_SETRENDERTARGETDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpDDS", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpDDSZ", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_SETRENDERTARGETDATA", pack=False, align=None),
    "D3DNTHAL_TEXTURECREATEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hDDS", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_TEXTURECREATEDATA", pack=False, align=None),
    "D3DNTHAL_TEXTUREDESTROYDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_TEXTUREDESTROYDATA", pack=False, align=None),
    "D3DNTHAL_TEXTUREGETSURFDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hDDS", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_TEXTUREGETSURFDATA", pack=False, align=None),
    "D3DNTHAL_TEXTURESWAPDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHandle1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHandle2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_TEXTURESWAPDATA", pack=False, align=None),
    "D3DNTHAL_VALIDATETEXTURESTAGESTATEDATA": SimStruct(OrderedDict((("dwhContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwNumPasses", SimTypeInt(signed=False, label="UInt32")), ("ddrval", SimTypeInt(signed=True, label="Int32")),)), name="D3DNTHAL_VALIDATETEXTURESTAGESTATEDATA", pack=False, align=None),
    "D3DOPCODE": SimTypeInt(signed=False, label="D3DOPCODE"),
    "D3DOVERLAYCAPS": SimStruct(OrderedDict((("Caps", SimTypeInt(signed=False, label="UInt32")), ("MaxOverlayDisplayWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxOverlayDisplayHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3DOVERLAYCAPS", pack=False, align=None),
    "D3DPATCHEDGESTYLE": SimTypeInt(signed=False, label="D3DPATCHEDGESTYLE"),
    "D3DPICKRECORD": SimStruct(OrderedDict((("bOpcode", SimTypeChar(label="Byte")), ("bPad", SimTypeChar(label="Byte")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dvZ", SimTypeFloat(size=32)),)), name="D3DPICKRECORD", pack=False, align=None),
    "D3DPOINT": SimStruct(OrderedDict((("wCount", SimTypeShort(signed=False, label="UInt16")), ("wFirst", SimTypeShort(signed=False, label="UInt16")),)), name="D3DPOINT", pack=False, align=None),
    "D3DPOOL": SimTypeInt(signed=False, label="D3DPOOL"),
    "D3DPRESENTSTATS": SimStruct(OrderedDict((("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("PresentRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("SyncGPUTime", SimTypeLongLong(signed=True, label="Int64")),)), name="D3DPRESENTSTATS", pack=False, align=None),
    "D3DPRESENT_PARAMETERS": SimStruct(OrderedDict((("BackBufferWidth", SimTypeInt(signed=False, label="UInt32")), ("BackBufferHeight", SimTypeInt(signed=False, label="UInt32")), ("BackBufferFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("BackBufferCount", SimTypeInt(signed=False, label="UInt32")), ("MultiSampleType", SimTypeInt(signed=False, label="D3DMULTISAMPLE_TYPE")), ("MultiSampleQuality", SimTypeInt(signed=False, label="UInt32")), ("SwapEffect", SimTypeInt(signed=False, label="D3DSWAPEFFECT")), ("hDeviceWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Windowed", SimTypeInt(signed=True, label="Int32")), ("EnableAutoDepthStencil", SimTypeInt(signed=True, label="Int32")), ("AutoDepthStencilFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FullScreen_RefreshRateInHz", SimTypeInt(signed=False, label="UInt32")), ("PresentationInterval", SimTypeInt(signed=False, label="UInt32")),)), name="D3DPRESENT_PARAMETERS", pack=False, align=None),
    "D3DPRIMCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMiscCaps", SimTypeInt(signed=False, label="UInt32")), ("dwRasterCaps", SimTypeInt(signed=False, label="UInt32")), ("dwZCmpCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSrcBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("dwDestBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaCmpCaps", SimTypeInt(signed=False, label="UInt32")), ("dwShadeCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureBlendCaps", SimTypeInt(signed=False, label="UInt32")), ("dwTextureAddressCaps", SimTypeInt(signed=False, label="UInt32")), ("dwStippleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwStippleHeight", SimTypeInt(signed=False, label="UInt32")),)), name="D3DPRIMCAPS", pack=False, align=None),
    "D3DPRIMITIVETYPE": SimTypeInt(signed=False, label="D3DPRIMITIVETYPE"),
    "D3DPROCESSVERTICES": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("wStart", SimTypeShort(signed=False, label="UInt16")), ("wDest", SimTypeShort(signed=False, label="UInt16")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="D3DPROCESSVERTICES", pack=False, align=None),
    "D3DPSHADERCAPS2_0": SimStruct(OrderedDict((("Caps", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlDepth", SimTypeInt(signed=True, label="Int32")), ("NumTemps", SimTypeInt(signed=True, label="Int32")), ("StaticFlowControlDepth", SimTypeInt(signed=True, label="Int32")), ("NumInstructionSlots", SimTypeInt(signed=True, label="Int32")),)), name="D3DPSHADERCAPS2_0", pack=False, align=None),
    "D3DQUERYTYPE": SimTypeInt(signed=False, label="D3DQUERYTYPE"),
    "D3DRANGE": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="D3DRANGE", pack=False, align=None),
    "D3DRASTER_STATUS": SimStruct(OrderedDict((("InVBlank", SimTypeInt(signed=True, label="Int32")), ("ScanLine", SimTypeInt(signed=False, label="UInt32")),)), name="D3DRASTER_STATUS", pack=False, align=None),
    "D3DRECT": SimStruct(OrderedDict((("x1", SimTypeInt(signed=True, label="Int32")), ("y1", SimTypeInt(signed=True, label="Int32")), ("x2", SimTypeInt(signed=True, label="Int32")), ("y2", SimTypeInt(signed=True, label="Int32")),)), name="D3DRECT", pack=False, align=None),
    "D3DRECTPATCH_INFO": SimStruct(OrderedDict((("StartVertexOffsetWidth", SimTypeInt(signed=False, label="UInt32")), ("StartVertexOffsetHeight", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Stride", SimTypeInt(signed=False, label="UInt32")), ("Basis", SimTypeInt(signed=False, label="D3DBASISTYPE")), ("Degree", SimTypeInt(signed=False, label="D3DDEGREETYPE")),)), name="D3DRECTPATCH_INFO", pack=False, align=None),
    "D3DRENDERSTATETYPE": SimTypeInt(signed=False, label="D3DRENDERSTATETYPE"),
    "D3DRESOURCESTATS": SimStruct(OrderedDict((("bThrashing", SimTypeInt(signed=True, label="Int32")), ("ApproxBytesDownloaded", SimTypeInt(signed=False, label="UInt32")), ("NumEvicts", SimTypeInt(signed=False, label="UInt32")), ("NumVidCreates", SimTypeInt(signed=False, label="UInt32")), ("LastPri", SimTypeInt(signed=False, label="UInt32")), ("NumUsed", SimTypeInt(signed=False, label="UInt32")), ("NumUsedInVidMem", SimTypeInt(signed=False, label="UInt32")), ("WorkingSet", SimTypeInt(signed=False, label="UInt32")), ("WorkingSetBytes", SimTypeInt(signed=False, label="UInt32")), ("TotalManaged", SimTypeInt(signed=False, label="UInt32")), ("TotalBytes", SimTypeInt(signed=False, label="UInt32")),)), name="D3DRESOURCESTATS", pack=False, align=None),
    "D3DRESOURCETYPE": SimTypeInt(signed=False, label="D3DRESOURCETYPE"),
    "D3DSAMPLERSTATETYPE": SimTypeInt(signed=False, label="D3DSAMPLERSTATETYPE"),
    "D3DSAMPLER_TEXTURE_TYPE": SimTypeInt(signed=False, label="D3DSAMPLER_TEXTURE_TYPE"),
    "D3DSCANLINEORDERING": SimTypeInt(signed=False, label="D3DSCANLINEORDERING"),
    "D3DSHADEMODE": SimTypeInt(signed=False, label="D3DSHADEMODE"),
    "D3DSHADER_ADDRESSMODE_TYPE": SimTypeInt(signed=False, label="D3DSHADER_ADDRESSMODE_TYPE"),
    "D3DSHADER_COMPARISON": SimTypeInt(signed=False, label="D3DSHADER_COMPARISON"),
    "D3DSHADER_INSTRUCTION_OPCODE_TYPE": SimTypeInt(signed=False, label="D3DSHADER_INSTRUCTION_OPCODE_TYPE"),
    "D3DSHADER_MIN_PRECISION": SimTypeInt(signed=False, label="D3DSHADER_MIN_PRECISION"),
    "D3DSHADER_MISCTYPE_OFFSETS": SimTypeInt(signed=False, label="D3DSHADER_MISCTYPE_OFFSETS"),
    "D3DSHADER_PARAM_REGISTER_TYPE": SimTypeInt(signed=False, label="D3DSHADER_PARAM_REGISTER_TYPE"),
    "D3DSHADER_PARAM_SRCMOD_TYPE": SimTypeInt(signed=False, label="D3DSHADER_PARAM_SRCMOD_TYPE"),
    "D3DSPAN": SimStruct(OrderedDict((("wCount", SimTypeShort(signed=False, label="UInt16")), ("wFirst", SimTypeShort(signed=False, label="UInt16")),)), name="D3DSPAN", pack=False, align=None),
    "D3DSTATE": SimStruct(OrderedDict((("Anonymous1", SimUnion({"dlstLightStateType": SimTypeInt(signed=False, label="D3DLIGHTSTATETYPE"), "drstRenderStateType": SimTypeInt(signed=False, label="D3DRENDERSTATETYPE")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"dwArg": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "dvArg": SimTypePointer(SimTypeFloat(size=32), offset=0)}, name="<anon>", label="None")),)), name="D3DSTATE", pack=False, align=None),
    "D3DSTATEBLOCKTYPE": SimTypeInt(signed=False, label="D3DSTATEBLOCKTYPE"),
    "D3DSTATS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwTrianglesDrawn", SimTypeInt(signed=False, label="UInt32")), ("dwLinesDrawn", SimTypeInt(signed=False, label="UInt32")), ("dwPointsDrawn", SimTypeInt(signed=False, label="UInt32")), ("dwSpansDrawn", SimTypeInt(signed=False, label="UInt32")), ("dwVerticesProcessed", SimTypeInt(signed=False, label="UInt32")),)), name="D3DSTATS", pack=False, align=None),
    "D3DSTATUS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("drExtent", SimTypeRef("D3DRECT", SimStruct)),)), name="D3DSTATUS", pack=False, align=None),
    "D3DSTENCILOP": SimTypeInt(signed=False, label="D3DSTENCILOP"),
    "D3DSURFACE_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("Type", SimTypeInt(signed=False, label="D3DRESOURCETYPE")), ("Usage", SimTypeInt(signed=False, label="UInt32")), ("Pool", SimTypeInt(signed=False, label="D3DPOOL")), ("MultiSampleType", SimTypeInt(signed=False, label="D3DMULTISAMPLE_TYPE")), ("MultiSampleQuality", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")),)), name="D3DSURFACE_DESC", pack=False, align=None),
    "D3DSWAPEFFECT": SimTypeInt(signed=False, label="D3DSWAPEFFECT"),
    "D3DTEXTUREADDRESS": SimTypeInt(signed=False, label="D3DTEXTUREADDRESS"),
    "D3DTEXTUREBLEND": SimTypeInt(signed=False, label="D3DTEXTUREBLEND"),
    "D3DTEXTUREFILTER": SimTypeInt(signed=False, label="D3DTEXTUREFILTER"),
    "D3DTEXTUREFILTERTYPE": SimTypeInt(signed=False, label="D3DTEXTUREFILTERTYPE"),
    "D3DTEXTURELOAD": SimStruct(OrderedDict((("hDestTexture", SimTypeInt(signed=False, label="UInt32")), ("hSrcTexture", SimTypeInt(signed=False, label="UInt32")),)), name="D3DTEXTURELOAD", pack=False, align=None),
    "D3DTEXTUREMAGFILTER": SimTypeInt(signed=False, label="D3DTEXTUREMAGFILTER"),
    "D3DTEXTUREMINFILTER": SimTypeInt(signed=False, label="D3DTEXTUREMINFILTER"),
    "D3DTEXTUREMIPFILTER": SimTypeInt(signed=False, label="D3DTEXTUREMIPFILTER"),
    "D3DTEXTUREOP": SimTypeInt(signed=False, label="D3DTEXTUREOP"),
    "D3DTEXTURESTAGESTATETYPE": SimTypeInt(signed=False, label="D3DTEXTURESTAGESTATETYPE"),
    "D3DTEXTURETRANSFORMFLAGS": SimTypeInt(signed=False, label="D3DTEXTURETRANSFORMFLAGS"),
    "D3DTLVERTEX": SimStruct(OrderedDict((("Anonymous1", SimUnion({"sx": SimTypeFloat(size=32), "dvSX": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"sy": SimTypeFloat(size=32), "dvSY": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"sz": SimTypeFloat(size=32), "dvSZ": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"rhw": SimTypeFloat(size=32), "dvRHW": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"color": SimTypeInt(signed=False, label="UInt32"), "dcColor": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous6", SimUnion({"specular": SimTypeInt(signed=False, label="UInt32"), "dcSpecular": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous7", SimUnion({"tu": SimTypeFloat(size=32), "dvTU": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous8", SimUnion({"tv": SimTypeFloat(size=32), "dvTV": SimTypeFloat(size=32)}, name="<anon>", label="None")),)), name="D3DTLVERTEX", pack=False, align=None),
    "D3DTRANSFORMCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")),)), name="D3DTRANSFORMCAPS", pack=False, align=None),
    "D3DTRANSFORMDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpIn", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwInSize", SimTypeInt(signed=False, label="UInt32")), ("lpOut", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwOutSize", SimTypeInt(signed=False, label="UInt32")), ("lpHOut", SimTypePointer(SimTypeRef("D3DHVERTEX", SimStruct), offset=0)), ("dwClip", SimTypeInt(signed=False, label="UInt32")), ("dwClipIntersection", SimTypeInt(signed=False, label="UInt32")), ("dwClipUnion", SimTypeInt(signed=False, label="UInt32")), ("drExtent", SimTypeRef("D3DRECT", SimStruct)),)), name="D3DTRANSFORMDATA", pack=False, align=None),
    "D3DTRANSFORMSTATETYPE": SimTypeInt(signed=False, label="D3DTRANSFORMSTATETYPE"),
    "D3DTRIANGLE": SimStruct(OrderedDict((("Anonymous1", SimUnion({"v1": SimTypeShort(signed=False, label="UInt16"), "wV1": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"v2": SimTypeShort(signed=False, label="UInt16"), "wV2": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"v3": SimTypeShort(signed=False, label="UInt16"), "wV3": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("wFlags", SimTypeShort(signed=False, label="UInt16")),)), name="D3DTRIANGLE", pack=False, align=None),
    "D3DTRIPATCH_INFO": SimStruct(OrderedDict((("StartVertexOffset", SimTypeInt(signed=False, label="UInt32")), ("NumVertices", SimTypeInt(signed=False, label="UInt32")), ("Basis", SimTypeInt(signed=False, label="D3DBASISTYPE")), ("Degree", SimTypeInt(signed=False, label="D3DDEGREETYPE")),)), name="D3DTRIPATCH_INFO", pack=False, align=None),
    "D3DVECTOR": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("z", SimTypeFloat(size=32)),)), name="D3DVECTOR", pack=False, align=None),
    "D3DVERTEX": SimStruct(OrderedDict((("Anonymous1", SimUnion({"x": SimTypeFloat(size=32), "dvX": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"y": SimTypeFloat(size=32), "dvY": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"z": SimTypeFloat(size=32), "dvZ": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"nx": SimTypeFloat(size=32), "dvNX": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"ny": SimTypeFloat(size=32), "dvNY": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous6", SimUnion({"nz": SimTypeFloat(size=32), "dvNZ": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous7", SimUnion({"tu": SimTypeFloat(size=32), "dvTU": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous8", SimUnion({"tv": SimTypeFloat(size=32), "dvTV": SimTypeFloat(size=32)}, name="<anon>", label="None")),)), name="D3DVERTEX", pack=False, align=None),
    "D3DVERTEXBLENDFLAGS": SimTypeInt(signed=False, label="D3DVERTEXBLENDFLAGS"),
    "D3DVERTEXBUFFERDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFVF", SimTypeInt(signed=False, label="UInt32")), ("dwNumVertices", SimTypeInt(signed=False, label="UInt32")),)), name="D3DVERTEXBUFFERDESC", pack=False, align=None),
    "D3DVERTEXBUFFER_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("Type", SimTypeInt(signed=False, label="D3DRESOURCETYPE")), ("Usage", SimTypeInt(signed=False, label="UInt32")), ("Pool", SimTypeInt(signed=False, label="D3DPOOL")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("FVF", SimTypeInt(signed=False, label="UInt32")),)), name="D3DVERTEXBUFFER_DESC", pack=False, align=None),
    "D3DVERTEXELEMENT9": SimStruct(OrderedDict((("Stream", SimTypeShort(signed=False, label="UInt16")), ("Offset", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeChar(label="Byte")), ("Method", SimTypeChar(label="Byte")), ("Usage", SimTypeChar(label="Byte")), ("UsageIndex", SimTypeChar(label="Byte")),)), name="D3DVERTEXELEMENT9", pack=False, align=None),
    "D3DVERTEXTYPE": SimTypeInt(signed=False, label="D3DVERTEXTYPE"),
    "D3DVIEWPORT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dvScaleX", SimTypeFloat(size=32)), ("dvScaleY", SimTypeFloat(size=32)), ("dvMaxX", SimTypeFloat(size=32)), ("dvMaxY", SimTypeFloat(size=32)), ("dvMinZ", SimTypeFloat(size=32)), ("dvMaxZ", SimTypeFloat(size=32)),)), name="D3DVIEWPORT", pack=False, align=None),
    "D3DVIEWPORT2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dvClipX", SimTypeFloat(size=32)), ("dvClipY", SimTypeFloat(size=32)), ("dvClipWidth", SimTypeFloat(size=32)), ("dvClipHeight", SimTypeFloat(size=32)), ("dvMinZ", SimTypeFloat(size=32)), ("dvMaxZ", SimTypeFloat(size=32)),)), name="D3DVIEWPORT2", pack=False, align=None),
    "D3DVIEWPORT7": SimStruct(OrderedDict((("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dvMinZ", SimTypeFloat(size=32)), ("dvMaxZ", SimTypeFloat(size=32)),)), name="D3DVIEWPORT7", pack=False, align=None),
    "D3DVIEWPORT9": SimStruct(OrderedDict((("X", SimTypeInt(signed=False, label="UInt32")), ("Y", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("MinZ", SimTypeFloat(size=32)), ("MaxZ", SimTypeFloat(size=32)),)), name="D3DVIEWPORT9", pack=False, align=None),
    "D3DVOLUME_DESC": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("Type", SimTypeInt(signed=False, label="D3DRESOURCETYPE")), ("Usage", SimTypeInt(signed=False, label="UInt32")), ("Pool", SimTypeInt(signed=False, label="D3DPOOL")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")),)), name="D3DVOLUME_DESC", pack=False, align=None),
    "D3DVSD_TOKENTYPE": SimTypeInt(signed=False, label="D3DVSD_TOKENTYPE"),
    "D3DVSHADERCAPS2_0": SimStruct(OrderedDict((("Caps", SimTypeInt(signed=False, label="UInt32")), ("DynamicFlowControlDepth", SimTypeInt(signed=True, label="Int32")), ("NumTemps", SimTypeInt(signed=True, label="Int32")), ("StaticFlowControlDepth", SimTypeInt(signed=True, label="Int32")),)), name="D3DVSHADERCAPS2_0", pack=False, align=None),
    "D3DVS_ADDRESSMODE_TYPE": SimTypeInt(signed=False, label="D3DVS_ADDRESSMODE_TYPE"),
    "D3DVS_RASTOUT_OFFSETS": SimTypeInt(signed=False, label="D3DVS_RASTOUT_OFFSETS"),
    "D3DX11_FFT_BUFFER_INFO": SimStruct(OrderedDict((("NumTempBufferSizes", SimTypeInt(signed=False, label="UInt32")), ("TempBufferFloatSizes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("NumPrecomputeBufferSizes", SimTypeInt(signed=False, label="UInt32")), ("PrecomputeBufferFloatSizes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="D3DX11_FFT_BUFFER_INFO", pack=False, align=None),
    "D3DX11_FFT_CREATE_FLAG": SimTypeInt(signed=False, label="D3DX11_FFT_CREATE_FLAG"),
    "D3DX11_FFT_DATA_TYPE": SimTypeInt(signed=False, label="D3DX11_FFT_DATA_TYPE"),
    "D3DX11_FFT_DESC": SimStruct(OrderedDict((("NumDimensions", SimTypeInt(signed=False, label="UInt32")), ("ElementLengths", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("DimensionMask", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="D3DX11_FFT_DATA_TYPE")),)), name="D3DX11_FFT_DESC", pack=False, align=None),
    "D3DX11_FFT_DIM_MASK": SimTypeInt(signed=False, label="D3DX11_FFT_DIM_MASK"),
    "D3DX11_SCAN_DATA_TYPE": SimTypeInt(signed=False, label="D3DX11_SCAN_DATA_TYPE"),
    "D3DX11_SCAN_DIRECTION": SimTypeInt(signed=False, label="D3DX11_SCAN_DIRECTION"),
    "D3DX11_SCAN_OPCODE": SimTypeInt(signed=False, label="D3DX11_SCAN_OPCODE"),
    "D3DZBUFFERTYPE": SimTypeInt(signed=False, label="D3DZBUFFERTYPE"),
    "D3D_BLOB_PART": SimTypeInt(signed=False, label="D3D_BLOB_PART"),
    "D3D_CBUFFER_TYPE": SimTypeInt(signed=False, label="D3D_CBUFFER_TYPE"),
    "D3D_DRIVER_TYPE": SimTypeInt(signed=False, label="D3D_DRIVER_TYPE"),
    "D3D_FEATURE_LEVEL": SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"),
    "D3D_FORMAT_COMPONENT_INTERPRETATION": SimTypeInt(signed=False, label="D3D_FORMAT_COMPONENT_INTERPRETATION"),
    "D3D_FORMAT_COMPONENT_NAME": SimTypeInt(signed=False, label="D3D_FORMAT_COMPONENT_NAME"),
    "D3D_FORMAT_LAYOUT": SimTypeInt(signed=False, label="D3D_FORMAT_LAYOUT"),
    "D3D_FORMAT_TYPE_LEVEL": SimTypeInt(signed=False, label="D3D_FORMAT_TYPE_LEVEL"),
    "D3D_INCLUDE_TYPE": SimTypeInt(signed=False, label="D3D_INCLUDE_TYPE"),
    "D3D_INTERPOLATION_MODE": SimTypeInt(signed=False, label="D3D_INTERPOLATION_MODE"),
    "D3D_MIN_PRECISION": SimTypeInt(signed=False, label="D3D_MIN_PRECISION"),
    "D3D_NAME": SimTypeInt(signed=False, label="D3D_NAME"),
    "D3D_OMAC": SimStruct(OrderedDict((("Omac", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="D3D_OMAC", pack=False, align=None),
    "D3D_PARAMETER_FLAGS": SimTypeInt(signed=False, label="D3D_PARAMETER_FLAGS"),
    "D3D_PRIMITIVE": SimTypeInt(signed=False, label="D3D_PRIMITIVE"),
    "D3D_PRIMITIVE_TOPOLOGY": SimTypeInt(signed=False, label="D3D_PRIMITIVE_TOPOLOGY"),
    "D3D_REGISTER_COMPONENT_TYPE": SimTypeInt(signed=False, label="D3D_REGISTER_COMPONENT_TYPE"),
    "D3D_RESOURCE_RETURN_TYPE": SimTypeInt(signed=False, label="D3D_RESOURCE_RETURN_TYPE"),
    "D3D_ROOT_SIGNATURE_VERSION": SimTypeInt(signed=False, label="D3D_ROOT_SIGNATURE_VERSION"),
    "D3D_SHADER_CBUFFER_FLAGS": SimTypeInt(signed=False, label="D3D_SHADER_CBUFFER_FLAGS"),
    "D3D_SHADER_DATA": SimStruct(OrderedDict((("pBytecode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BytecodeLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="D3D_SHADER_DATA", pack=False, align=None),
    "D3D_SHADER_INPUT_FLAGS": SimTypeInt(signed=False, label="D3D_SHADER_INPUT_FLAGS"),
    "D3D_SHADER_INPUT_TYPE": SimTypeInt(signed=False, label="D3D_SHADER_INPUT_TYPE"),
    "D3D_SHADER_MACRO": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Definition", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="D3D_SHADER_MACRO", pack=False, align=None),
    "D3D_SHADER_MODEL": SimTypeInt(signed=False, label="D3D_SHADER_MODEL"),
    "D3D_SHADER_VARIABLE_CLASS": SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_CLASS"),
    "D3D_SHADER_VARIABLE_FLAGS": SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_FLAGS"),
    "D3D_SHADER_VARIABLE_TYPE": SimTypeInt(signed=False, label="D3D_SHADER_VARIABLE_TYPE"),
    "D3D_SRV_DIMENSION": SimTypeInt(signed=False, label="D3D_SRV_DIMENSION"),
    "D3D_TESSELLATOR_DOMAIN": SimTypeInt(signed=False, label="D3D_TESSELLATOR_DOMAIN"),
    "D3D_TESSELLATOR_OUTPUT_PRIMITIVE": SimTypeInt(signed=False, label="D3D_TESSELLATOR_OUTPUT_PRIMITIVE"),
    "D3D_TESSELLATOR_PARTITIONING": SimTypeInt(signed=False, label="D3D_TESSELLATOR_PARTITIONING"),
    "DAILY": SimStruct(OrderedDict((("DaysInterval", SimTypeShort(signed=False, label="UInt16")),)), name="DAILY", pack=False, align=None),
    "DATABLOCK_HEADER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")),)), name="DATABLOCK_HEADER", pack=False, align=None),
    "DATADIR": SimTypeInt(signed=False, label="DATADIR"),
    "DATAINFO": SimStruct(OrderedDict((("ulTotalSize", SimTypeInt(signed=False, label="UInt32")), ("ulavrPacketSize", SimTypeInt(signed=False, label="UInt32")), ("ulConnectSpeed", SimTypeInt(signed=False, label="UInt32")), ("ulProcessorSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="DATAINFO", pack=False, align=None),
    "DATAOBJ_GET_ITEM_FLAGS": SimTypeInt(signed=False, label="DATAOBJ_GET_ITEM_FLAGS"),
    "DATATYPES_INFO_1A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DATATYPES_INFO_1A", pack=False, align=None),
    "DATATYPES_INFO_1W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DATATYPES_INFO_1W", pack=False, align=None),
    "DATA_HEADER": SimStruct(OrderedDict((("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("wSize", SimTypeShort(signed=False, label="UInt16")), ("wDataID", SimTypeShort(signed=False, label="UInt16")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DATA_HEADER", pack=False, align=None),
    "DATA_OBJECT_TYPES": SimTypeInt(signed=False, label="DATA_OBJECT_TYPES"),
    "DATEBORDERSTATES": SimTypeInt(signed=False, label="DATEBORDERSTATES"),
    "DATEFMT_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "DATEFMT_ENUMPROCEXA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "DATEFMT_ENUMPROCEXEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "DATEFMT_ENUMPROCEXW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "DATEFMT_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "DATEPICKERPARTS": SimTypeInt(signed=False, label="DATEPICKERPARTS"),
    "DATETEXTSTATES": SimTypeInt(signed=False, label="DATETEXTSTATES"),
    "DATETIME": SimStruct(OrderedDict((("year", SimTypeShort(signed=False, label="UInt16")), ("month", SimTypeShort(signed=False, label="UInt16")), ("day", SimTypeShort(signed=False, label="UInt16")), ("hour", SimTypeShort(signed=False, label="UInt16")), ("min", SimTypeShort(signed=False, label="UInt16")), ("sec", SimTypeShort(signed=False, label="UInt16")),)), name="DATETIME", pack=False, align=None),
    "DATETIMEPICKERINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcCheck", SimTypeRef("RECT", SimStruct)), ("stateCheck", SimTypeInt(signed=False, label="UInt32")), ("rcButton", SimTypeRef("RECT", SimStruct)), ("stateButton", SimTypeInt(signed=False, label="UInt32")), ("hwndEdit", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndUD", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndDropDown", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DATETIMEPICKERINFO", pack=False, align=None),
    "DATE_STRUCT": SimStruct(OrderedDict((("year", SimTypeShort(signed=True, label="Int16")), ("month", SimTypeShort(signed=False, label="UInt16")), ("day", SimTypeShort(signed=False, label="UInt16")),)), name="DATE_STRUCT", pack=False, align=None),
    "DATE_TIME": SimStruct(OrderedDict((("dwLowDateTime", SimTypeInt(signed=False, label="UInt32")), ("dwHighDateTime", SimTypeInt(signed=False, label="UInt32")),)), name="DATE_TIME", pack=False, align=None),
    "DAV_CALLBACK_AUTH_BLOB": SimStruct(OrderedDict((("pBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulSize", SimTypeInt(signed=False, label="UInt32")), ("ulType", SimTypeInt(signed=False, label="UInt32")),)), name="DAV_CALLBACK_AUTH_BLOB", pack=False, align=None),
    "DAV_CALLBACK_AUTH_UNP": SimStruct(OrderedDict((("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulUserNameLength", SimTypeInt(signed=False, label="UInt32")), ("pszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulPasswordLength", SimTypeInt(signed=False, label="UInt32")),)), name="DAV_CALLBACK_AUTH_UNP", pack=False, align=None),
    "DAV_CALLBACK_CRED": SimStruct(OrderedDict((("AuthBlob", SimTypeRef("DAV_CALLBACK_AUTH_BLOB", SimStruct)), ("UNPBlob", SimTypeRef("DAV_CALLBACK_AUTH_UNP", SimStruct)), ("bAuthBlobValid", SimTypeInt(signed=True, label="Int32")), ("bSave", SimTypeInt(signed=True, label="Int32")),)), name="DAV_CALLBACK_CRED", pack=False, align=None),
    "DBACCESSORFLAGSENUM": SimTypeInt(signed=False, label="DBACCESSORFLAGSENUM"),
    "DBASYNCHOPENUM": SimTypeInt(signed=False, label="DBASYNCHOPENUM"),
    "DBASYNCHPHASEENUM": SimTypeInt(signed=False, label="DBASYNCHPHASEENUM"),
    "DBBINDEXT": SimStruct(OrderedDict((("pExtension", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulExtension", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DBBINDEXT", pack=False, align=None),
    "DBBINDFLAGENUM": SimTypeInt(signed=False, label="DBBINDFLAGENUM"),
    "DBBINDING": SimStruct(OrderedDict((("iOrdinal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("obValue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("obLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("obStatus", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pTypeInfo", SimTypeBottom(label="ITypeInfo")), ("pObject", SimTypePointer(SimTypeRef("DBOBJECT", SimStruct), offset=0)), ("pBindExt", SimTypePointer(SimTypeRef("DBBINDEXT", SimStruct), offset=0)), ("dwPart", SimTypeInt(signed=False, label="UInt32")), ("dwMemOwner", SimTypeInt(signed=False, label="UInt32")), ("eParamIO", SimTypeInt(signed=False, label="UInt32")), ("cbMaxLen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("wType", SimTypeShort(signed=False, label="UInt16")), ("bPrecision", SimTypeChar(label="Byte")), ("bScale", SimTypeChar(label="Byte")),)), name="DBBINDING", pack=False, align=None),
    "DBBINDSTATUSENUM": SimTypeInt(signed=False, label="DBBINDSTATUSENUM"),
    "DBBINDURLFLAGENUM": SimTypeInt(signed=False, label="DBBINDURLFLAGENUM"),
    "DBBINDURLSTATUSENUM": SimTypeInt(signed=False, label="DBBINDURLSTATUSENUM"),
    "DBBOOKMARK": SimTypeInt(signed=False, label="DBBOOKMARK"),
    "DBCOLUMNACCESS": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("columnid", SimTypeRef("DBID", SimStruct)), ("cbDataLen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("cbMaxLen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("bPrecision", SimTypeChar(label="Byte")), ("bScale", SimTypeChar(label="Byte")),)), name="DBCOLUMNACCESS", pack=False, align=None),
    "DBCOLUMNDESC": SimStruct(OrderedDict((("pwszTypeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pTypeInfo", SimTypeBottom(label="ITypeInfo")), ("rgPropertySets", SimTypePointer(SimTypeRef("DBPROPSET", SimStruct), offset=0)), ("pclsid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cPropertySets", SimTypeInt(signed=False, label="UInt32")), ("ulColumnSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dbcid", SimTypeRef("DBID", SimStruct)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("bPrecision", SimTypeChar(label="Byte")), ("bScale", SimTypeChar(label="Byte")),)), name="DBCOLUMNDESC", pack=False, align=None),
    "DBCOLUMNDESCFLAGSENUM": SimTypeInt(signed=False, label="DBCOLUMNDESCFLAGSENUM"),
    "DBCOLUMNFLAGS15ENUM": SimTypeInt(signed=False, label="DBCOLUMNFLAGS15ENUM"),
    "DBCOLUMNFLAGSDEPRECATED": SimTypeInt(signed=False, label="DBCOLUMNFLAGSDEPRECATED"),
    "DBCOLUMNFLAGSENUM": SimTypeInt(signed=False, label="DBCOLUMNFLAGSENUM"),
    "DBCOLUMNFLAGSENUM20": SimTypeInt(signed=False, label="DBCOLUMNFLAGSENUM20"),
    "DBCOLUMNFLAGSENUM21": SimTypeInt(signed=False, label="DBCOLUMNFLAGSENUM21"),
    "DBCOLUMNFLAGSENUM26": SimTypeInt(signed=False, label="DBCOLUMNFLAGSENUM26"),
    "DBCOLUMNINFO": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pTypeInfo", SimTypeBottom(label="ITypeInfo")), ("iOrdinal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ulColumnSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("bPrecision", SimTypeChar(label="Byte")), ("bScale", SimTypeChar(label="Byte")), ("columnid", SimTypeRef("DBID", SimStruct)),)), name="DBCOLUMNINFO", pack=False, align=None),
    "DBCOMMANDPERSISTFLAGENUM": SimTypeInt(signed=False, label="DBCOMMANDPERSISTFLAGENUM"),
    "DBCOMMANDPERSISTFLAGENUM21": SimTypeInt(signed=False, label="DBCOMMANDPERSISTFLAGENUM21"),
    "DBCOMPAREENUM": SimTypeInt(signed=False, label="DBCOMPAREENUM"),
    "DBCOMPAREOPSENUM": SimTypeInt(signed=False, label="DBCOMPAREOPSENUM"),
    "DBCOMPAREOPSENUM20": SimTypeInt(signed=False, label="DBCOMPAREOPSENUM20"),
    "DBCONSTRAINTDESC": SimStruct(OrderedDict((("pConstraintID", SimTypePointer(SimTypeRef("DBID", SimStruct), offset=0)), ("ConstraintType", SimTypeInt(signed=False, label="UInt32")), ("cColumns", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rgColumnList", SimTypePointer(SimTypeRef("DBID", SimStruct), offset=0)), ("pReferencedTableID", SimTypePointer(SimTypeRef("DBID", SimStruct), offset=0)), ("cForeignKeyColumns", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rgForeignKeyColumnList", SimTypePointer(SimTypeRef("DBID", SimStruct), offset=0)), ("pwszConstraintText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UpdateRule", SimTypeInt(signed=False, label="UInt32")), ("DeleteRule", SimTypeInt(signed=False, label="UInt32")), ("MatchType", SimTypeInt(signed=False, label="UInt32")), ("Deferrability", SimTypeInt(signed=False, label="UInt32")), ("cReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rgReserved", SimTypePointer(SimTypeRef("DBPROPSET", SimStruct), offset=0)),)), name="DBCONSTRAINTDESC", pack=False, align=None),
    "DBCONSTRAINTTYPEENUM": SimTypeInt(signed=False, label="DBCONSTRAINTTYPEENUM"),
    "DBCONVERTFLAGSENUM": SimTypeInt(signed=False, label="DBCONVERTFLAGSENUM"),
    "DBCONVERTFLAGSENUM20": SimTypeInt(signed=False, label="DBCONVERTFLAGSENUM20"),
    "DBCOPYFLAGSENUM": SimTypeInt(signed=False, label="DBCOPYFLAGSENUM"),
    "DBCOST": SimStruct(OrderedDict((("eKind", SimTypeInt(signed=False, label="UInt32")), ("dwUnits", SimTypeInt(signed=False, label="UInt32")), ("lValue", SimTypeInt(signed=True, label="Int32")),)), name="DBCOST", pack=False, align=None),
    "DBCOSTUNITENUM": SimTypeInt(signed=False, label="DBCOSTUNITENUM"),
    "DBDATACONVERTENUM": SimTypeInt(signed=False, label="DBDATACONVERTENUM"),
    "DBDATE": SimStruct(OrderedDict((("year", SimTypeShort(signed=True, label="Int16")), ("month", SimTypeShort(signed=False, label="UInt16")), ("day", SimTypeShort(signed=False, label="UInt16")),)), name="DBDATE", pack=False, align=None),
    "DBDATETIM4": SimStruct(OrderedDict((("numdays", SimTypeShort(signed=False, label="UInt16")), ("nummins", SimTypeShort(signed=False, label="UInt16")),)), name="DBDATETIM4", pack=False, align=None),
    "DBDATETIME": SimStruct(OrderedDict((("dtdays", SimTypeInt(signed=True, label="Int32")), ("dttime", SimTypeInt(signed=False, label="UInt32")),)), name="DBDATETIME", pack=False, align=None),
    "DBDEFERRABILITYENUM": SimTypeInt(signed=False, label="DBDEFERRABILITYENUM"),
    "DBDELETEFLAGSENUM": SimTypeInt(signed=False, label="DBDELETEFLAGSENUM"),
    "DBEVENTPHASEENUM": SimTypeInt(signed=False, label="DBEVENTPHASEENUM"),
    "DBEXECLIMITSENUM": SimTypeInt(signed=False, label="DBEXECLIMITSENUM"),
    "DBFAILUREINFO": SimStruct(OrderedDict((("hRow", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iColumn", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("failure", SimTypeInt(signed=True, label="Int32")),)), name="DBFAILUREINFO", pack=False, align=None),
    "DBGHELP_DATA_REPORT_STRUCT": SimStruct(OrderedDict((("pBinPathNonExist", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSymbolPathNonExist", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DBGHELP_DATA_REPORT_STRUCT", pack=False, align=None),
    "DBGKD_DEBUG_DATA_HEADER32": SimStruct(OrderedDict((("List", SimTypeRef("LIST_ENTRY32", SimStruct)), ("OwnerTag", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="DBGKD_DEBUG_DATA_HEADER32", pack=False, align=None),
    "DBGKD_DEBUG_DATA_HEADER64": SimStruct(OrderedDict((("List", SimTypeRef("LIST_ENTRY64", SimStruct)), ("OwnerTag", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="DBGKD_DEBUG_DATA_HEADER64", pack=False, align=None),
    "DBGKD_GET_VERSION32": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("ProtocolVersion", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("KernBase", SimTypeInt(signed=False, label="UInt32")), ("PsLoadedModuleList", SimTypeInt(signed=False, label="UInt32")), ("MachineType", SimTypeShort(signed=False, label="UInt16")), ("ThCallbackStack", SimTypeShort(signed=False, label="UInt16")), ("NextCallback", SimTypeShort(signed=False, label="UInt16")), ("FramePointer", SimTypeShort(signed=False, label="UInt16")), ("KiCallUserMode", SimTypeInt(signed=False, label="UInt32")), ("KeUserCallbackDispatcher", SimTypeInt(signed=False, label="UInt32")), ("BreakpointWithStatus", SimTypeInt(signed=False, label="UInt32")), ("DebuggerDataList", SimTypeInt(signed=False, label="UInt32")),)), name="DBGKD_GET_VERSION32", pack=False, align=None),
    "DBGKD_GET_VERSION64": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("ProtocolVersion", SimTypeChar(label="Byte")), ("KdSecondaryVersion", SimTypeChar(label="Byte")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("MachineType", SimTypeShort(signed=False, label="UInt16")), ("MaxPacketType", SimTypeChar(label="Byte")), ("MaxStateChange", SimTypeChar(label="Byte")), ("MaxManipulate", SimTypeChar(label="Byte")), ("Simulation", SimTypeChar(label="Byte")), ("Unused", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("KernBase", SimTypeLongLong(signed=False, label="UInt64")), ("PsLoadedModuleList", SimTypeLongLong(signed=False, label="UInt64")), ("DebuggerDataList", SimTypeLongLong(signed=False, label="UInt64")),)), name="DBGKD_GET_VERSION64", pack=False, align=None),
    "DBGKD_MAJOR_TYPES": SimTypeInt(signed=False, label="DBGKD_MAJOR_TYPES"),
    "DBGPRINT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Format"]), offset=0),
    "DBGPROP_ATTRIB_FLAGS": SimTypeInt(signed=False, label="DBGPROP_ATTRIB_FLAGS"),
    "DBGPROP_INFO": SimTypeInt(signed=False, label="DBGPROP_INFO"),
    "DBG_THREAD_ATTRIBUTES": SimStruct(OrderedDict((("ThreadIndex", SimTypeInt(signed=False, label="UInt32")), ("ProcessID", SimTypeLongLong(signed=False, label="UInt64")), ("ThreadID", SimTypeLongLong(signed=False, label="UInt64")), ("AttributeBits", SimTypeLongLong(signed=False, label="UInt64")), ("BoolBits", SimTypeInt(signed=False, label="UInt32")), ("BlockedOnPID", SimTypeLongLong(signed=False, label="UInt64")), ("BlockedOnTID", SimTypeLongLong(signed=False, label="UInt64")), ("CritSecAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Timeout_msec", SimTypeInt(signed=False, label="UInt32")), ("StringData", SimTypeArray(SimTypeChar(label="SByte"), 100)), ("SymName", SimTypeArray(SimTypeChar(label="SByte"), 100)),)), name="DBG_THREAD_ATTRIBUTES", pack=False, align=None),
    "DBID": SimStruct(OrderedDict((("uGuid", SimUnion({"guid": SimTypeBottom(label="Guid"), "pguid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="<anon>", label="None")), ("eKind", SimTypeInt(signed=False, label="UInt32")), ("uName", SimUnion({"pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ulPropid": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DBID", pack=False, align=None),
    "DBIMPLICITSESSION": SimStruct(OrderedDict((("pUnkOuter", SimTypeBottom(label="IUnknown")), ("piid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pSession", SimTypeBottom(label="IUnknown")),)), name="DBIMPLICITSESSION", pack=False, align=None),
    "DBINDEXCOLUMNDESC": SimStruct(OrderedDict((("pColumnID", SimTypePointer(SimTypeRef("DBID", SimStruct), offset=0)), ("eIndexColOrder", SimTypeInt(signed=False, label="UInt32")),)), name="DBINDEXCOLUMNDESC", pack=False, align=None),
    "DBINDEX_COL_ORDERENUM": SimTypeInt(signed=False, label="DBINDEX_COL_ORDERENUM"),
    "DBKINDENUM": SimTypeInt(signed=False, label="DBKINDENUM"),
    "DBLITERALENUM": SimTypeInt(signed=False, label="DBLITERALENUM"),
    "DBLITERALENUM20": SimTypeInt(signed=False, label="DBLITERALENUM20"),
    "DBLITERALENUM21": SimTypeInt(signed=False, label="DBLITERALENUM21"),
    "DBLITERALINFO": SimStruct(OrderedDict((("pwszLiteralValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszInvalidChars", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszInvalidStartingChars", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lt", SimTypeInt(signed=False, label="UInt32")), ("fSupported", SimTypeInt(signed=True, label="Int32")), ("cchMaxLen", SimTypeInt(signed=False, label="UInt32")),)), name="DBLITERALINFO", pack=False, align=None),
    "DBLNODE": SimStruct(OrderedDict((("next", SimTypePointer(SimTypeRef("DBLNODE", SimStruct), offset=0)), ("prev", SimTypePointer(SimTypeRef("DBLNODE", SimStruct), offset=0)), ("object", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("object_int", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)),)), name="DBLNODE", pack=False, align=None),
    "DBMATCHTYPEENUM": SimTypeInt(signed=False, label="DBMATCHTYPEENUM"),
    "DBMEMOWNERENUM": SimTypeInt(signed=False, label="DBMEMOWNERENUM"),
    "DBMONEY": SimStruct(OrderedDict((("mnyhigh", SimTypeInt(signed=True, label="Int32")), ("mnylow", SimTypeInt(signed=False, label="UInt32")),)), name="DBMONEY", pack=False, align=None),
    "DBMOVEFLAGSENUM": SimTypeInt(signed=False, label="DBMOVEFLAGSENUM"),
    "DBOBJECT": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iid", SimTypeBottom(label="Guid")),)), name="DBOBJECT", pack=False, align=None),
    "DBPARAMBINDINFO": SimStruct(OrderedDict((("pwszDataSourceType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulParamSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bPrecision", SimTypeChar(label="Byte")), ("bScale", SimTypeChar(label="Byte")),)), name="DBPARAMBINDINFO", pack=False, align=None),
    "DBPARAMFLAGSENUM": SimTypeInt(signed=False, label="DBPARAMFLAGSENUM"),
    "DBPARAMFLAGSENUM20": SimTypeInt(signed=False, label="DBPARAMFLAGSENUM20"),
    "DBPARAMINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iOrdinal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pTypeInfo", SimTypeBottom(label="ITypeInfo")), ("ulParamSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("bPrecision", SimTypeChar(label="Byte")), ("bScale", SimTypeChar(label="Byte")),)), name="DBPARAMINFO", pack=False, align=None),
    "DBPARAMIOENUM": SimTypeInt(signed=False, label="DBPARAMIOENUM"),
    "DBPARAMS": SimStruct(OrderedDict((("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cParamSets", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hAccessor", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DBPARAMS", pack=False, align=None),
    "DBPARTENUM": SimTypeInt(signed=False, label="DBPARTENUM"),
    "DBPENDINGSTATUSENUM": SimTypeInt(signed=False, label="DBPENDINGSTATUSENUM"),
    "DBPOSITIONFLAGSENUM": SimTypeInt(signed=False, label="DBPOSITIONFLAGSENUM"),
    "DBPROMPTOPTIONSENUM": SimTypeInt(signed=False, label="DBPROMPTOPTIONSENUM"),
    "DBPROP": SimStruct(OrderedDict((("dwPropertyID", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("colid", SimTypeRef("DBID", SimStruct)), ("vValue", SimTypeRef("VARIANT", SimStruct)),)), name="DBPROP", pack=False, align=None),
    "DBPROPENUM": SimTypeInt(signed=False, label="DBPROPENUM"),
    "DBPROPENUM15": SimTypeInt(signed=False, label="DBPROPENUM15"),
    "DBPROPENUM20": SimTypeInt(signed=False, label="DBPROPENUM20"),
    "DBPROPENUM21": SimTypeInt(signed=False, label="DBPROPENUM21"),
    "DBPROPENUM25": SimTypeInt(signed=False, label="DBPROPENUM25"),
    "DBPROPENUM25_DEPRECATED": SimTypeInt(signed=False, label="DBPROPENUM25_DEPRECATED"),
    "DBPROPENUM26": SimTypeInt(signed=False, label="DBPROPENUM26"),
    "DBPROPENUMDEPRECATED": SimTypeInt(signed=False, label="DBPROPENUMDEPRECATED"),
    "DBPROPFLAGSENUM": SimTypeInt(signed=False, label="DBPROPFLAGSENUM"),
    "DBPROPFLAGSENUM21": SimTypeInt(signed=False, label="DBPROPFLAGSENUM21"),
    "DBPROPFLAGSENUM25": SimTypeInt(signed=False, label="DBPROPFLAGSENUM25"),
    "DBPROPFLAGSENUM26": SimTypeInt(signed=False, label="DBPROPFLAGSENUM26"),
    "DBPROPIDSET": SimStruct(OrderedDict((("rgPropertyIDs", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cPropertyIDs", SimTypeInt(signed=False, label="UInt32")), ("guidPropertySet", SimTypeBottom(label="Guid")),)), name="DBPROPIDSET", pack=False, align=None),
    "DBPROPINFO": SimStruct(OrderedDict((("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPropertyID", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("vtType", SimTypeInt(signed=False, label="VARENUM")), ("vValues", SimTypeRef("VARIANT", SimStruct)),)), name="DBPROPINFO", pack=False, align=None),
    "DBPROPINFOSET": SimStruct(OrderedDict((("rgPropertyInfos", SimTypePointer(SimTypeRef("DBPROPINFO", SimStruct), offset=0)), ("cPropertyInfos", SimTypeInt(signed=False, label="UInt32")), ("guidPropertySet", SimTypeBottom(label="Guid")),)), name="DBPROPINFOSET", pack=False, align=None),
    "DBPROPOPTIONSENUM": SimTypeInt(signed=False, label="DBPROPOPTIONSENUM"),
    "DBPROPSET": SimStruct(OrderedDict((("rgProperties", SimTypePointer(SimTypeRef("DBPROP", SimStruct), offset=0)), ("cProperties", SimTypeInt(signed=False, label="UInt32")), ("guidPropertySet", SimTypeBottom(label="Guid")),)), name="DBPROPSET", pack=False, align=None),
    "DBPROPSTATUSENUM": SimTypeInt(signed=False, label="DBPROPSTATUSENUM"),
    "DBPROPSTATUSENUM21": SimTypeInt(signed=False, label="DBPROPSTATUSENUM21"),
    "DBRANGEENUM": SimTypeInt(signed=False, label="DBRANGEENUM"),
    "DBRANGEENUM20": SimTypeInt(signed=False, label="DBRANGEENUM20"),
    "DBREASONENUM": SimTypeInt(signed=False, label="DBREASONENUM"),
    "DBREASONENUM15": SimTypeInt(signed=False, label="DBREASONENUM15"),
    "DBREASONENUM25": SimTypeInt(signed=False, label="DBREASONENUM25"),
    "DBRESOURCEKINDENUM": SimTypeInt(signed=False, label="DBRESOURCEKINDENUM"),
    "DBRESULTFLAGENUM": SimTypeInt(signed=False, label="DBRESULTFLAGENUM"),
    "DBROWCHANGEKINDENUM": SimTypeInt(signed=False, label="DBROWCHANGEKINDENUM"),
    "DBROWSTATUSENUM": SimTypeInt(signed=False, label="DBROWSTATUSENUM"),
    "DBROWSTATUSENUM20": SimTypeInt(signed=False, label="DBROWSTATUSENUM20"),
    "DBROWWATCHCHANGE": SimStruct(OrderedDict((("hRegion", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("eChangeKind", SimTypeInt(signed=False, label="UInt32")), ("hRow", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iRow", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DBROWWATCHCHANGE", pack=False, align=None),
    "DBSEEKENUM": SimTypeInt(signed=False, label="DBSEEKENUM"),
    "DBSORTENUM": SimTypeInt(signed=False, label="DBSORTENUM"),
    "DBSOURCETYPEENUM": SimTypeInt(signed=False, label="DBSOURCETYPEENUM"),
    "DBSOURCETYPEENUM20": SimTypeInt(signed=False, label="DBSOURCETYPEENUM20"),
    "DBSOURCETYPEENUM25": SimTypeInt(signed=False, label="DBSOURCETYPEENUM25"),
    "DBSTATUSENUM": SimTypeInt(signed=False, label="DBSTATUSENUM"),
    "DBSTATUSENUM20": SimTypeInt(signed=False, label="DBSTATUSENUM20"),
    "DBSTATUSENUM21": SimTypeInt(signed=False, label="DBSTATUSENUM21"),
    "DBSTATUSENUM25": SimTypeInt(signed=False, label="DBSTATUSENUM25"),
    "DBSTATUSENUM26": SimTypeInt(signed=False, label="DBSTATUSENUM26"),
    "DBTABLESTATISTICSTYPE26": SimTypeInt(signed=False, label="DBTABLESTATISTICSTYPE26"),
    "DBTIME": SimStruct(OrderedDict((("hour", SimTypeShort(signed=False, label="UInt16")), ("minute", SimTypeShort(signed=False, label="UInt16")), ("second", SimTypeShort(signed=False, label="UInt16")),)), name="DBTIME", pack=False, align=None),
    "DBTIMESTAMP": SimStruct(OrderedDict((("year", SimTypeShort(signed=True, label="Int16")), ("month", SimTypeShort(signed=False, label="UInt16")), ("day", SimTypeShort(signed=False, label="UInt16")), ("hour", SimTypeShort(signed=False, label="UInt16")), ("minute", SimTypeShort(signed=False, label="UInt16")), ("second", SimTypeShort(signed=False, label="UInt16")), ("fraction", SimTypeInt(signed=False, label="UInt32")),)), name="DBTIMESTAMP", pack=False, align=None),
    "DBTYPEENUM": SimTypeInt(signed=False, label="DBTYPEENUM"),
    "DBTYPEENUM15": SimTypeInt(signed=False, label="DBTYPEENUM15"),
    "DBTYPEENUM20": SimTypeInt(signed=False, label="DBTYPEENUM20"),
    "DBUPDELRULEENUM": SimTypeInt(signed=False, label="DBUPDELRULEENUM"),
    "DBVARYBIN": SimStruct(OrderedDict((("len", SimTypeShort(signed=True, label="Int16")), ("array", SimTypeArray(SimTypeChar(label="Byte"), 8001)),)), name="DBVARYBIN", pack=False, align=None),
    "DBVARYCHAR": SimStruct(OrderedDict((("len", SimTypeShort(signed=True, label="Int16")), ("str", SimTypeArray(SimTypeChar(label="SByte"), 8001)),)), name="DBVARYCHAR", pack=False, align=None),
    "DBVECTOR": SimStruct(OrderedDict((("size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ptr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DBVECTOR", pack=False, align=None),
    "DBWATCHMODEENUM": SimTypeInt(signed=False, label="DBWATCHMODEENUM"),
    "DBWATCHNOTIFYENUM": SimTypeInt(signed=False, label="DBWATCHNOTIFYENUM"),
    "DB_NUMERIC": SimStruct(OrderedDict((("precision", SimTypeChar(label="Byte")), ("scale", SimTypeChar(label="Byte")), ("sign", SimTypeChar(label="Byte")), ("val", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DB_NUMERIC", pack=False, align=None),
    "DB_VARNUMERIC": SimStruct(OrderedDict((("precision", SimTypeChar(label="Byte")), ("scale", SimTypeChar(label="SByte")), ("sign", SimTypeChar(label="Byte")), ("val", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DB_VARNUMERIC", pack=False, align=None),
    "DCB": SimStruct(OrderedDict((("DCBlength", SimTypeInt(signed=False, label="UInt32")), ("BaudRate", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("XonLim", SimTypeShort(signed=False, label="UInt16")), ("XoffLim", SimTypeShort(signed=False, label="UInt16")), ("ByteSize", SimTypeChar(label="Byte")), ("Parity", SimTypeInt(signed=False, label="DCB_PARITY")), ("StopBits", SimTypeInt(signed=False, label="DCB_STOP_BITS")), ("XonChar", SimTypeChar(label="SByte")), ("XoffChar", SimTypeChar(label="SByte")), ("ErrorChar", SimTypeChar(label="SByte")), ("EofChar", SimTypeChar(label="SByte")), ("EvtChar", SimTypeChar(label="SByte")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")),)), name="DCB", pack=False, align=None),
    "DCB_PARITY": SimTypeInt(signed=False, label="DCB_PARITY"),
    "DCB_STOP_BITS": SimTypeInt(signed=False, label="DCB_STOP_BITS"),
    "DCICMD": SimStruct(OrderedDict((("dwCommand", SimTypeInt(signed=False, label="UInt32")), ("dwParam1", SimTypeInt(signed=False, label="UInt32")), ("dwParam2", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DCICMD", pack=False, align=None),
    "DCICREATEINPUT": SimStruct(OrderedDict((("cmd", SimTypeRef("DCICMD", SimStruct)), ("dwCompression", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwDCICaps", SimTypeInt(signed=False, label="UInt32")), ("dwBitCount", SimTypeInt(signed=False, label="UInt32")), ("lpSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DCICREATEINPUT", pack=False, align=None),
    "DCIENUMINPUT": SimStruct(OrderedDict((("cmd", SimTypeRef("DCICMD", SimStruct)), ("rSrc", SimTypeRef("RECT", SimStruct)), ("rDst", SimTypeRef("RECT", SimStruct)), ("EnumCallback", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DCIENUMINPUT", pack=False, align=None),
    "DCINFO": SimStruct(OrderedDict((("eInfoType", SimTypeInt(signed=False, label="UInt32")), ("vData", SimTypeRef("VARIANT", SimStruct)),)), name="DCINFO", pack=False, align=None),
    "DCINFOTYPEENUM": SimTypeInt(signed=False, label="DCINFOTYPEENUM"),
    "DCIOFFSCREEN": SimStruct(OrderedDict((("dciInfo", SimTypeRef("DCISURFACEINFO", SimStruct)), ("Draw", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetClipList", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetDestination", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DCIOFFSCREEN", pack=False, align=None),
    "DCIOVERLAY": SimStruct(OrderedDict((("dciInfo", SimTypeRef("DCISURFACEINFO", SimStruct)), ("dwChromakeyValue", SimTypeInt(signed=False, label="UInt32")), ("dwChromakeyMask", SimTypeInt(signed=False, label="UInt32")),)), name="DCIOVERLAY", pack=False, align=None),
    "DCISURFACEINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwDCICaps", SimTypeInt(signed=False, label="UInt32")), ("dwCompression", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("lStride", SimTypeInt(signed=True, label="Int32")), ("dwBitCount", SimTypeInt(signed=False, label="UInt32")), ("dwOffSurface", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wSelSurface", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("BeginAccess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EndAccess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DestroySurface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DCISURFACEINFO", pack=False, align=None),
    "DCOMPOSITION_BACKFACE_VISIBILITY": SimTypeInt(signed=False, label="DCOMPOSITION_BACKFACE_VISIBILITY"),
    "DCOMPOSITION_BITMAP_INTERPOLATION_MODE": SimTypeInt(signed=False, label="DCOMPOSITION_BITMAP_INTERPOLATION_MODE"),
    "DCOMPOSITION_BORDER_MODE": SimTypeInt(signed=False, label="DCOMPOSITION_BORDER_MODE"),
    "DCOMPOSITION_COMPOSITE_MODE": SimTypeInt(signed=False, label="DCOMPOSITION_COMPOSITE_MODE"),
    "DCOMPOSITION_DEPTH_MODE": SimTypeInt(signed=False, label="DCOMPOSITION_DEPTH_MODE"),
    "DCOMPOSITION_FRAME_STATISTICS": SimStruct(OrderedDict((("lastFrameTime", SimTypeLongLong(signed=True, label="Int64")), ("currentCompositionRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("currentTime", SimTypeLongLong(signed=True, label="Int64")), ("timeFrequency", SimTypeLongLong(signed=True, label="Int64")), ("nextEstimatedFrameTime", SimTypeLongLong(signed=True, label="Int64")),)), name="DCOMPOSITION_FRAME_STATISTICS", pack=False, align=None),
    "DCOMPOSITION_OPACITY_MODE": SimTypeInt(signed=False, label="DCOMPOSITION_OPACITY_MODE"),
    "DCOM_CALL_STATE": SimTypeInt(signed=False, label="DCOM_CALL_STATE"),
    "DC_LAYOUT": SimTypeInt(signed=False, label="DC_LAYOUT"),
    "DCompositionInkTrailPoint": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("radius", SimTypeFloat(size=32)),)), name="DCompositionInkTrailPoint", pack=False, align=None),
    "DD32BITDRIVERDATA": SimStruct(OrderedDict((("szName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szEntryPoint", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("dwContext", SimTypeInt(signed=False, label="UInt32")),)), name="DD32BITDRIVERDATA", pack=False, align=None),
    "DDARGB": SimStruct(OrderedDict((("blue", SimTypeChar(label="Byte")), ("green", SimTypeChar(label="Byte")), ("red", SimTypeChar(label="Byte")), ("alpha", SimTypeChar(label="Byte")),)), name="DDARGB", pack=False, align=None),
    "DDBLTBATCH": SimStruct(OrderedDict((("lprDest", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("lpDDSSrc", SimTypeBottom(label="IDirectDrawSurface")), ("lprSrc", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDDBltFx", SimTypePointer(SimTypeRef("DDBLTFX", SimStruct), offset=0)),)), name="DDBLTBATCH", pack=False, align=None),
    "DDBLTFX": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwDDFX", SimTypeInt(signed=False, label="UInt32")), ("dwROP", SimTypeInt(signed=False, label="UInt32")), ("dwDDROP", SimTypeInt(signed=False, label="UInt32")), ("dwRotationAngle", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferOpCode", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferLow", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferHigh", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBaseDest", SimTypeInt(signed=False, label="UInt32")), ("dwZDestConstBitDepth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"dwZDestConst": SimTypeInt(signed=False, label="UInt32"), "lpDDSZBufferDest": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("dwZSrcConstBitDepth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"dwZSrcConst": SimTypeInt(signed=False, label="UInt32"), "lpDDSZBufferSrc": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("dwAlphaEdgeBlendBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaEdgeBlend", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaDestConstBitDepth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous3", SimUnion({"dwAlphaDestConst": SimTypeInt(signed=False, label="UInt32"), "lpDDSAlphaDest": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("dwAlphaSrcConstBitDepth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous4", SimUnion({"dwAlphaSrcConst": SimTypeInt(signed=False, label="UInt32"), "lpDDSAlphaSrc": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"dwFillColor": SimTypeInt(signed=False, label="UInt32"), "dwFillDepth": SimTypeInt(signed=False, label="UInt32"), "dwFillPixel": SimTypeInt(signed=False, label="UInt32"), "lpDDSPattern": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("ddckDestColorkey", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckSrcColorkey", SimTypeRef("DDCOLORKEY", SimStruct)),)), name="DDBLTFX", pack=False, align=None),
    "DDBOBNEXTFIELDINFO": SimStruct(OrderedDict((("lpSurface", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)),)), name="DDBOBNEXTFIELDINFO", pack=False, align=None),
    "DDCAPS_DX1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="DDCAPS_DX1", pack=False, align=None),
    "DDCAPS_DX3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwVSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwSSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")), ("dwReserved5", SimTypeInt(signed=False, label="UInt32")), ("dwReserved6", SimTypeInt(signed=False, label="UInt32")),)), name="DDCAPS_DX3", pack=False, align=None),
    "DDCAPS_DX5": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwVSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwSSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwMaxVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="DDCAPS_DX5", pack=False, align=None),
    "DDCAPS_DX6": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsOldCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwVSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwSSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwMaxVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS2", SimStruct)),)), name="DDCAPS_DX6", pack=False, align=None),
    "DDCAPS_DX7": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsOldCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwVSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwSSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwMaxVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS2", SimStruct)),)), name="DDCAPS_DX7", pack=False, align=None),
    "DDCOLORCONTROL": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lBrightness", SimTypeInt(signed=True, label="Int32")), ("lContrast", SimTypeInt(signed=True, label="Int32")), ("lHue", SimTypeInt(signed=True, label="Int32")), ("lSaturation", SimTypeInt(signed=True, label="Int32")), ("lSharpness", SimTypeInt(signed=True, label="Int32")), ("lGamma", SimTypeInt(signed=True, label="Int32")), ("lColorEnable", SimTypeInt(signed=True, label="Int32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")),)), name="DDCOLORCONTROL", pack=False, align=None),
    "DDCOLORKEY": SimStruct(OrderedDict((("dwColorSpaceLowValue", SimTypeInt(signed=False, label="UInt32")), ("dwColorSpaceHighValue", SimTypeInt(signed=False, label="UInt32")),)), name="DDCOLORKEY", pack=False, align=None),
    "DDCOMPBUFFERINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumCompBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwWidthToCreate", SimTypeInt(signed=False, label="UInt32")), ("dwHeightToCreate", SimTypeInt(signed=False, label="UInt32")), ("dwBytesToAllocate", SimTypeInt(signed=False, label="UInt32")), ("ddCompCaps", SimTypeRef("DDSCAPS2", SimStruct)), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="DDCOMPBUFFERINFO", pack=False, align=None),
    "DDCORECAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwVSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwSSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwMaxVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps2", SimTypeInt(signed=False, label="UInt32")),)), name="DDCORECAPS", pack=False, align=None),
    "DDDEVICEIDENTIFIER": SimStruct(OrderedDict((("szDriver", SimTypeArray(SimTypeChar(label="SByte"), 512)), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 512)), ("liDriverVersion", SimTypeLongLong(signed=True, label="Int64")), ("dwVendorId", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceId", SimTypeInt(signed=False, label="UInt32")), ("dwSubSysId", SimTypeInt(signed=False, label="UInt32")), ("dwRevision", SimTypeInt(signed=False, label="UInt32")), ("guidDeviceIdentifier", SimTypeBottom(label="Guid")),)), name="DDDEVICEIDENTIFIER", pack=False, align=None),
    "DDDEVICEIDENTIFIER2": SimStruct(OrderedDict((("szDriver", SimTypeArray(SimTypeChar(label="SByte"), 512)), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 512)), ("liDriverVersion", SimTypeLongLong(signed=True, label="Int64")), ("dwVendorId", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceId", SimTypeInt(signed=False, label="UInt32")), ("dwSubSysId", SimTypeInt(signed=False, label="UInt32")), ("dwRevision", SimTypeInt(signed=False, label="UInt32")), ("guidDeviceIdentifier", SimTypeBottom(label="Guid")), ("dwWHQLLevel", SimTypeInt(signed=False, label="UInt32")),)), name="DDDEVICEIDENTIFIER2", pack=False, align=None),
    "DDEACK": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="DDEACK", pack=False, align=None),
    "DDEADVISE": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("cfFormat", SimTypeShort(signed=True, label="Int16")),)), name="DDEADVISE", pack=False, align=None),
    "DDEDATA": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("cfFormat", SimTypeShort(signed=True, label="Int16")), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DDEDATA", pack=False, align=None),
    "DDELN": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("cfFormat", SimTypeShort(signed=True, label="Int16")),)), name="DDELN", pack=False, align=None),
    "DDEML_MSG_HOOK_DATA": SimStruct(OrderedDict((("uiLo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("uiHi", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="DDEML_MSG_HOOK_DATA", pack=False, align=None),
    "DDENABLEIRQINFO": SimStruct(OrderedDict((("dwIRQSources", SimTypeInt(signed=False, label="UInt32")), ("dwLine", SimTypeInt(signed=False, label="UInt32")), ("IRQCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DX_IRQDATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pIrqData"]), offset=0)), ("lpIRQData", SimTypePointer(SimTypeRef("DX_IRQDATA", SimStruct), offset=0)),)), name="DDENABLEIRQINFO", pack=False, align=None),
    "DDEPOKE": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("cfFormat", SimTypeShort(signed=True, label="Int16")), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DDEPOKE", pack=False, align=None),
    "DDEUP": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("cfFormat", SimTypeShort(signed=True, label="Int16")), ("rgb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DDEUP", pack=False, align=None),
    "DDE_CLIENT_TRANSACTION_TYPE": SimTypeInt(signed=False, label="DDE_CLIENT_TRANSACTION_TYPE"),
    "DDE_ENABLE_CALLBACK_CMD": SimTypeInt(signed=False, label="DDE_ENABLE_CALLBACK_CMD"),
    "DDE_INITIALIZE_COMMAND": SimTypeInt(signed=False, label="DDE_INITIALIZE_COMMAND"),
    "DDE_NAME_SERVICE_CMD": SimTypeInt(signed=False, label="DDE_NAME_SERVICE_CMD"),
    "DDFLIPOVERLAYINFO": SimStruct(OrderedDict((("lpCurrentSurface", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)), ("lpTargetSurface", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDFLIPOVERLAYINFO", pack=False, align=None),
    "DDFLIPVIDEOPORTINFO": SimStruct(OrderedDict((("lpVideoPortData", SimTypePointer(SimTypeRef("DDVIDEOPORTDATA", SimStruct), offset=0)), ("lpCurrentSurface", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)), ("lpTargetSurface", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)), ("dwFlipVPFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDFLIPVIDEOPORTINFO", pack=False, align=None),
    "DDGAMMARAMP": SimStruct(OrderedDict((("red", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("green", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("blue", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="DDGAMMARAMP", pack=False, align=None),
    "DDGETCURRENTAUTOFLIPININFO": SimStruct(OrderedDict((("lpVideoPortData", SimTypePointer(SimTypeRef("DDVIDEOPORTDATA", SimStruct), offset=0)),)), name="DDGETCURRENTAUTOFLIPININFO", pack=False, align=None),
    "DDGETCURRENTAUTOFLIPOUTINFO": SimStruct(OrderedDict((("dwSurfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("dwVBISurfaceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="DDGETCURRENTAUTOFLIPOUTINFO", pack=False, align=None),
    "DDGETIRQINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDGETIRQINFO", pack=False, align=None),
    "DDGETPOLARITYININFO": SimStruct(OrderedDict((("lpVideoPortData", SimTypePointer(SimTypeRef("DDVIDEOPORTDATA", SimStruct), offset=0)),)), name="DDGETPOLARITYININFO", pack=False, align=None),
    "DDGETPOLARITYOUTINFO": SimStruct(OrderedDict((("bPolarity", SimTypeInt(signed=False, label="UInt32")),)), name="DDGETPOLARITYOUTINFO", pack=False, align=None),
    "DDGETPREVIOUSAUTOFLIPININFO": SimStruct(OrderedDict((("lpVideoPortData", SimTypePointer(SimTypeRef("DDVIDEOPORTDATA", SimStruct), offset=0)),)), name="DDGETPREVIOUSAUTOFLIPININFO", pack=False, align=None),
    "DDGETPREVIOUSAUTOFLIPOUTINFO": SimStruct(OrderedDict((("dwSurfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("dwVBISurfaceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="DDGETPREVIOUSAUTOFLIPOUTINFO", pack=False, align=None),
    "DDGETTRANSFERSTATUSOUTINFO": SimStruct(OrderedDict((("dwTransferID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDGETTRANSFERSTATUSOUTINFO", pack=False, align=None),
    "DDHALDDRAWFNS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpSetInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHALINFO", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpDDHalInfo", "reset"]), offset=0)), ("lpVidMemAlloc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["lpDD", "heap", "dwWidth", "dwHeight"]), offset=0)), ("lpVidMemFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["lpDD", "heap", "fpMem"]), offset=0)),)), name="DDHALDDRAWFNS", pack=False, align=None),
    "DDHALINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpDDCallbacks", SimTypePointer(SimTypeRef("DDHAL_DDCALLBACKS", SimStruct), offset=0)), ("lpDDSurfaceCallbacks", SimTypePointer(SimTypeRef("DDHAL_DDSURFACECALLBACKS", SimStruct), offset=0)), ("lpDDPaletteCallbacks", SimTypePointer(SimTypeRef("DDHAL_DDPALETTECALLBACKS", SimStruct), offset=0)), ("vmiData", SimTypeRef("VIDMEMINFO", SimStruct)), ("ddCaps", SimTypeRef("DDCORECAPS", SimStruct)), ("dwMonitorFrequency", SimTypeInt(signed=False, label="UInt32")), ("GetDriverInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETDRIVERINFODATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwModeIndex", SimTypeInt(signed=False, label="UInt32")), ("lpdwFourCC", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("dwNumModes", SimTypeInt(signed=False, label="UInt32")), ("lpModeInfo", SimTypePointer(SimTypeRef("DDHALMODEINFO", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpPDevice", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hInstance", SimTypeInt(signed=False, label="UInt32")), ("lpD3DGlobalDriverData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpD3DHALCallbacks", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpDDExeBufCallbacks", SimTypePointer(SimTypeRef("DDHAL_DDEXEBUFCALLBACKS", SimStruct), offset=0)),)), name="DDHALINFO", pack=False, align=None),
    "DDHALMODEINFO": SimStruct(OrderedDict((("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("lPitch", SimTypeInt(signed=True, label="Int32")), ("dwBPP", SimTypeInt(signed=False, label="UInt32")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("wRefreshRate", SimTypeShort(signed=False, label="UInt16")), ("dwRBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwGBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwBBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBitMask", SimTypeInt(signed=False, label="UInt32")),)), name="DDHALMODEINFO", pack=False, align=None),
    "DDHAL_ADDATTACHEDSURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpSurfAttached", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("AddAttachedSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_ADDATTACHEDSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_ADDATTACHEDSURFACEDATA", pack=False, align=None),
    "DDHAL_BEGINMOCOMPFRAMEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("lpDestSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwOutputDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("BeginMoCompFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_BEGINMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_BEGINMOCOMPFRAMEDATA", pack=False, align=None),
    "DDHAL_BLTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDDestSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("rDest", SimTypeRef("RECTL", SimStruct)), ("lpDDSrcSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("rSrc", SimTypeRef("RECTL", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwROPFlags", SimTypeInt(signed=False, label="UInt32")), ("bltFX", SimTypeRef("DDBLTFX", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Blt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("IsClipped", SimTypeInt(signed=True, label="Int32")), ("rOrigDest", SimTypeRef("RECTL", SimStruct)), ("rOrigSrc", SimTypeRef("RECTL", SimStruct)), ("dwRectCnt", SimTypeInt(signed=False, label="UInt32")), ("prDestRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)),)), name="DDHAL_BLTDATA", pack=False, align=None),
    "DDHAL_CALLBACKS": SimStruct(OrderedDict((("cbDDCallbacks", SimTypeRef("DDHAL_DDCALLBACKS", SimStruct)), ("cbDDSurfaceCallbacks", SimTypeRef("DDHAL_DDSURFACECALLBACKS", SimStruct)), ("cbDDPaletteCallbacks", SimTypeRef("DDHAL_DDPALETTECALLBACKS", SimStruct)), ("HALDD", SimTypeRef("DDHAL_DDCALLBACKS", SimStruct)), ("HALDDSurface", SimTypeRef("DDHAL_DDSURFACECALLBACKS", SimStruct)), ("HALDDPalette", SimTypeRef("DDHAL_DDPALETTECALLBACKS", SimStruct)), ("HELDD", SimTypeRef("DDHAL_DDCALLBACKS", SimStruct)), ("HELDDSurface", SimTypeRef("DDHAL_DDSURFACECALLBACKS", SimStruct)), ("HELDDPalette", SimTypeRef("DDHAL_DDPALETTECALLBACKS", SimStruct)), ("cbDDExeBufCallbacks", SimTypeRef("DDHAL_DDEXEBUFCALLBACKS", SimStruct)), ("HALDDExeBuf", SimTypeRef("DDHAL_DDEXEBUFCALLBACKS", SimStruct)), ("HELDDExeBuf", SimTypeRef("DDHAL_DDEXEBUFCALLBACKS", SimStruct)), ("cbDDVideoPortCallbacks", SimTypeRef("DDHAL_DDVIDEOPORTCALLBACKS", SimStruct)), ("HALDDVideoPort", SimTypeRef("DDHAL_DDVIDEOPORTCALLBACKS", SimStruct)), ("cbDDColorControlCallbacks", SimTypeRef("DDHAL_DDCOLORCONTROLCALLBACKS", SimStruct)), ("HALDDColorControl", SimTypeRef("DDHAL_DDCOLORCONTROLCALLBACKS", SimStruct)), ("cbDDMiscellaneousCallbacks", SimTypeRef("DDHAL_DDMISCELLANEOUSCALLBACKS", SimStruct)), ("HALDDMiscellaneous", SimTypeRef("DDHAL_DDMISCELLANEOUSCALLBACKS", SimStruct)), ("cbDDKernelCallbacks", SimTypeRef("DDHAL_DDKERNELCALLBACKS", SimStruct)), ("HALDDKernel", SimTypeRef("DDHAL_DDKERNELCALLBACKS", SimStruct)), ("cbDDMotionCompCallbacks", SimTypeRef("DDHAL_DDMOTIONCOMPCALLBACKS", SimStruct)), ("HALDDMotionComp", SimTypeRef("DDHAL_DDMOTIONCOMPCALLBACKS", SimStruct)),)), name="DDHAL_CALLBACKS", pack=False, align=None),
    "DDHAL_CANCREATESURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurfaceDesc", SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0)), ("bIsDifferentPixelFormat", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CanCreateSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_CANCREATESURFACEDATA", pack=False, align=None),
    "DDHAL_CANCREATEVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpDDVideoPortDesc", SimTypePointer(SimTypeRef("DDVIDEOPORTDESC", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CanCreateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_CANCREATEVPORTDATA", pack=False, align=None),
    "DDHAL_COLORCONTROLDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpColorData", SimTypePointer(SimTypeRef("DDCOLORCONTROL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("ColorControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_COLORCONTROLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_COLORCONTROLDATA", pack=False, align=None),
    "DDHAL_CREATEMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwUncompWidth", SimTypeInt(signed=False, label="UInt32")), ("dwUncompHeight", SimTypeInt(signed=False, label="UInt32")), ("ddUncompPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreateMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_CREATEMOCOMPDATA", pack=False, align=None),
    "DDHAL_CREATEPALETTEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_GBL", SimStruct), offset=0)), ("lpColorTable", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreatePalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("is_excl", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_CREATEPALETTEDATA", pack=False, align=None),
    "DDHAL_CREATESURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurfaceDesc", SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0)), ("lplpSList", SimTypePointer(SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0), offset=0)), ("dwSCnt", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreateSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_CREATESURFACEDATA", pack=False, align=None),
    "DDHAL_CREATESURFACEEXDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDDLcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpDDSLcl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_CREATESURFACEEXDATA", pack=False, align=None),
    "DDHAL_CREATEVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpDDVideoPortDesc", SimTypePointer(SimTypeRef("DDVIDEOPORTDESC", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_CREATEVPORTDATA", pack=False, align=None),
    "DDHAL_DDCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DestroyDriver", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYDRIVERDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetColorKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DRVSETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("WaitForVerticalBlank", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_WAITFORVERTICALBLANKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CanCreateSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreatePalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetScanLine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETSCANLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetExclusiveMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETEXCLUSIVEMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("FlipToGDISurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPTOGDISURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDCALLBACKS", pack=False, align=None),
    "DDHAL_DDCOLORCONTROLCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ColorControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_COLORCONTROLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDCOLORCONTROLCALLBACKS", pack=False, align=None),
    "DDHAL_DDEXEBUFCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("CanCreateExecuteBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateExecuteBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyExecuteBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("LockExecuteBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UnlockExecuteBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDEXEBUFCALLBACKS", pack=False, align=None),
    "DDHAL_DDKERNELCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("SyncSurfaceData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SYNCSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SyncVideoPortData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SYNCVIDEOPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDKERNELCALLBACKS", pack=False, align=None),
    "DDHAL_DDMISCELLANEOUS2CALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CreateSurfaceEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEEXDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetDriverState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETDRIVERSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyDDLocal", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYDDLOCALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDMISCELLANEOUS2CALLBACKS", pack=False, align=None),
    "DDHAL_DDMISCELLANEOUSCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("GetAvailDriverMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETAVAILDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UpdateNonLocalHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATENONLOCALHEAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetHeapAlignment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETHEAPALIGNMENTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetSysmemBltStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETBLTSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDMISCELLANEOUSCALLBACKS", pack=False, align=None),
    "DDHAL_DDMOTIONCOMPCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("GetMoCompGuids", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPGUIDSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetMoCompFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPFORMATSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetMoCompBuffInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPCOMPBUFFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetInternalMoCompInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETINTERNALMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("BeginMoCompFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_BEGINMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("EndMoCompFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_ENDMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("RenderMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_RENDERMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("QueryMoCompStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_QUERYMOCOMPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDMOTIONCOMPCALLBACKS", pack=False, align=None),
    "DDHAL_DDPALETTECALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DestroyPalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetEntries", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETENTRIESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDPALETTECALLBACKS", pack=False, align=None),
    "DDHAL_DDSURFACECALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DestroySurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Flip", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetClipList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETCLIPLISTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Lock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Unlock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Blt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetColorKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("AddAttachedSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_ADDATTACHEDSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetBltStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETBLTSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetFlipStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UpdateOverlay", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATEOVERLAYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetOverlayPosition", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETOVERLAYPOSITIONDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("reserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SetPalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDSURFACECALLBACKS", pack=False, align=None),
    "DDHAL_DDVIDEOPORTCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("CanCreateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("FlipVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortBandwidth", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTBANDWIDTHDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortInputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTINPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortOutputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTOUTPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpReserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("GetVideoPortField", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTFIELDDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortLine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortConnectInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTCONNECTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortFlipStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UpdateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("WaitForVideoPortSync", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_WAITFORVPORTSYNCDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoSignalStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTSIGNALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ColorControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_VPORTCOLORDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DDVIDEOPORTCALLBACKS", pack=False, align=None),
    "DDHAL_DESTROYDDLOCALDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pDDLcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_DESTROYDDLOCALDATA", pack=False, align=None),
    "DDHAL_DESTROYDRIVERDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroyDriver", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYDRIVERDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DESTROYDRIVERDATA", pack=False, align=None),
    "DDHAL_DESTROYMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroyMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DESTROYMOCOMPDATA", pack=False, align=None),
    "DDHAL_DESTROYPALETTEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_GBL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroyPalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DESTROYPALETTEDATA", pack=False, align=None),
    "DDHAL_DESTROYSURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroySurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DESTROYSURFACEDATA", pack=False, align=None),
    "DDHAL_DESTROYVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroyVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DESTROYVPORTDATA", pack=False, align=None),
    "DDHAL_DRVSETCOLORKEYDATA": SimStruct(OrderedDict((("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ckNew", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetColorKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DRVSETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_DRVSETCOLORKEYDATA", pack=False, align=None),
    "DDHAL_ENDMOCOMPFRAMEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("lpInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("EndMoCompFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_ENDMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_ENDMOCOMPFRAMEDATA", pack=False, align=None),
    "DDHAL_FLIPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpSurfCurr", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpSurfTarg", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Flip", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpSurfCurrLeft", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpSurfTargLeft", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)),)), name="DDHAL_FLIPDATA", pack=False, align=None),
    "DDHAL_FLIPTOGDISURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwToGDI", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("FlipToGDISurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPTOGDISURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_FLIPTOGDISURFACEDATA", pack=False, align=None),
    "DDHAL_FLIPVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("lpSurfCurr", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpSurfTarg", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("FlipVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_FLIPVPORTDATA", pack=False, align=None),
    "DDHAL_GETAVAILDRIVERMEMORYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("DDSCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwTotal", SimTypeInt(signed=False, label="UInt32")), ("dwFree", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetAvailDriverMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETAVAILDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ddsCapsEx", SimTypeRef("DDSCAPSEX", SimStruct)),)), name="DDHAL_GETAVAILDRIVERMEMORYDATA", pack=False, align=None),
    "DDHAL_GETBLTSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetBltStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETBLTSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETBLTSTATUSDATA", pack=False, align=None),
    "DDHAL_GETDRIVERINFODATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidInfo", SimTypeBottom(label="Guid")), ("dwExpectedSize", SimTypeInt(signed=False, label="UInt32")), ("lpvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwActualSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("dwContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDHAL_GETDRIVERINFODATA", pack=False, align=None),
    "DDHAL_GETDRIVERSTATEDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwhContext": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("lpdwStates", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_GETDRIVERSTATEDATA", pack=False, align=None),
    "DDHAL_GETFLIPSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetFlipStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETFLIPSTATUSDATA", pack=False, align=None),
    "DDHAL_GETHEAPALIGNMENTDATA": SimStruct(OrderedDict((("dwInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHeap", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetHeapAlignment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETHEAPALIGNMENTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Alignment", SimTypeRef("HEAPALIGNMENT", SimStruct)),)), name="DDHAL_GETHEAPALIGNMENTDATA", pack=False, align=None),
    "DDHAL_GETINTERNALMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwScratchMemAlloc", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetInternalMoCompInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETINTERNALMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETINTERNALMOCOMPDATA", pack=False, align=None),
    "DDHAL_GETMOCOMPCOMPBUFFDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwNumTypesCompBuffs", SimTypeInt(signed=False, label="UInt32")), ("lpCompBuffInfo", SimTypePointer(SimTypeRef("DDMCCOMPBUFFERINFO", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetMoCompBuffInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPCOMPBUFFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETMOCOMPCOMPBUFFDATA", pack=False, align=None),
    "DDHAL_GETMOCOMPFORMATSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwNumFormats", SimTypeInt(signed=False, label="UInt32")), ("lpFormats", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetMoCompFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPFORMATSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETMOCOMPFORMATSDATA", pack=False, align=None),
    "DDHAL_GETMOCOMPGUIDSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwNumGuids", SimTypeInt(signed=False, label="UInt32")), ("lpGuids", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetMoCompGuids", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPGUIDSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETMOCOMPGUIDSDATA", pack=False, align=None),
    "DDHAL_GETSCANLINEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwScanLine", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetScanLine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETSCANLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETSCANLINEDATA", pack=False, align=None),
    "DDHAL_GETVPORTBANDWIDTHDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("lpddpfFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpBandwidth", SimTypePointer(SimTypeRef("DDVIDEOPORTBANDWIDTH", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortBandwidth", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTBANDWIDTHDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTBANDWIDTHDATA", pack=False, align=None),
    "DDHAL_GETVPORTCONNECTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwPortId", SimTypeInt(signed=False, label="UInt32")), ("lpConnect", SimTypePointer(SimTypeRef("DDVIDEOPORTCONNECT", SimStruct), offset=0)), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortConnectInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTCONNECTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTCONNECTDATA", pack=False, align=None),
    "DDHAL_GETVPORTFIELDDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("bField", SimTypeInt(signed=True, label="Int32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortField", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTFIELDDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTFIELDDATA", pack=False, align=None),
    "DDHAL_GETVPORTFLIPSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("fpSurface", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortFlipStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTFLIPSTATUSDATA", pack=False, align=None),
    "DDHAL_GETVPORTINPUTFORMATDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpddpfFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwNumFormats", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortInputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTINPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTINPUTFORMATDATA", pack=False, align=None),
    "DDHAL_GETVPORTLINEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwLine", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortLine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTLINEDATA", pack=False, align=None),
    "DDHAL_GETVPORTOUTPUTFORMATDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpddpfInputFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("lpddpfOutputFormats", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwNumFormats", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortOutputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTOUTPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTOUTPUTFORMATDATA", pack=False, align=None),
    "DDHAL_GETVPORTSIGNALDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoSignalStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTSIGNALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_GETVPORTSIGNALDATA", pack=False, align=None),
    "DDHAL_LOCKDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("bHasRect", SimTypeInt(signed=False, label="UInt32")), ("rArea", SimTypeRef("RECTL", SimStruct)), ("lpSurfData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Lock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDHAL_LOCKDATA", pack=False, align=None),
    "DDHAL_QUERYMOCOMPSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("lpSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("QueryMoCompStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_QUERYMOCOMPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_QUERYMOCOMPSTATUSDATA", pack=False, align=None),
    "DDHAL_RENDERMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("dwNumBuffers", SimTypeInt(signed=False, label="UInt32")), ("lpBufferInfo", SimTypePointer(SimTypeRef("DDMCBUFFERINFO", SimStruct), offset=0)), ("dwFunction", SimTypeInt(signed=False, label="UInt32")), ("lpInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwOutputDataSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("RenderMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_RENDERMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_RENDERMOCOMPDATA", pack=False, align=None),
    "DDHAL_SETCLIPLISTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetClipList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETCLIPLISTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_SETCLIPLISTDATA", pack=False, align=None),
    "DDHAL_SETCOLORKEYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ckNew", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetColorKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_SETCOLORKEYDATA", pack=False, align=None),
    "DDHAL_SETENTRIESDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_GBL", SimStruct), offset=0)), ("dwBase", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("lpEntries", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetEntries", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETENTRIESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_SETENTRIESDATA", pack=False, align=None),
    "DDHAL_SETEXCLUSIVEMODEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwEnterExcl", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetExclusiveMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETEXCLUSIVEMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_SETEXCLUSIVEMODEDATA", pack=False, align=None),
    "DDHAL_SETMODEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwModeIndex", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("inexcl", SimTypeInt(signed=True, label="Int32")), ("useRefreshRate", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_SETMODEDATA", pack=False, align=None),
    "DDHAL_SETOVERLAYPOSITIONDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSrcSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpDDDestSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lXPos", SimTypeInt(signed=True, label="Int32")), ("lYPos", SimTypeInt(signed=True, label="Int32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetOverlayPosition", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETOVERLAYPOSITIONDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_SETOVERLAYPOSITIONDATA", pack=False, align=None),
    "DDHAL_SETPALETTEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_GBL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetPalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Attach", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_SETPALETTEDATA", pack=False, align=None),
    "DDHAL_SYNCSURFACEDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwSurfaceOffset", SimTypeInt(signed=False, label="UInt32")), ("fpLockPtr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lPitch", SimTypeInt(signed=True, label="Int32")), ("dwOverlayOffset", SimTypeInt(signed=False, label="UInt32")), ("dwOverlaySrcWidth", SimTypeInt(signed=False, label="UInt32")), ("dwOverlaySrcHeight", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayDestWidth", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayDestHeight", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDriverReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDriverReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_SYNCSURFACEDATA", pack=False, align=None),
    "DDHAL_SYNCVIDEOPORTDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwOriginOffset", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVBIHeight", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDriverReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDriverReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DDHAL_SYNCVIDEOPORTDATA", pack=False, align=None),
    "DDHAL_UNLOCKDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Unlock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_UNLOCKDATA", pack=False, align=None),
    "DDHAL_UPDATENONLOCALHEAPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwHeap", SimTypeInt(signed=False, label="UInt32")), ("fpGARTLin", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fpGARTDev", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ulPolicyMaxBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateNonLocalHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATENONLOCALHEAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_UPDATENONLOCALHEAPDATA", pack=False, align=None),
    "DDHAL_UPDATEOVERLAYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("lpDDDestSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("rDest", SimTypeRef("RECTL", SimStruct)), ("lpDDSrcSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("rSrc", SimTypeRef("RECTL", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("overlayFX", SimTypeRef("DDOVERLAYFX", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateOverlay", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATEOVERLAYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_UPDATEOVERLAYDATA", pack=False, align=None),
    "DDHAL_UPDATEVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("lplpDDSurface", SimTypePointer(SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0), offset=0)), ("lplpDDVBISurface", SimTypePointer(SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0), offset=0)), ("lpVideoInfo", SimTypePointer(SimTypeRef("DDVIDEOPORTINFO", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumAutoflip", SimTypeInt(signed=False, label="UInt32")), ("dwNumVBIAutoflip", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_UPDATEVPORTDATA", pack=False, align=None),
    "DDHAL_VPORTCOLORDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpColorData", SimTypePointer(SimTypeRef("DDCOLORCONTROL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("ColorControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_VPORTCOLORDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_VPORTCOLORDATA", pack=False, align=None),
    "DDHAL_WAITFORVERTICALBLANKDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bIsInVB", SimTypeInt(signed=False, label="UInt32")), ("hEvent", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("WaitForVerticalBlank", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_WAITFORVERTICALBLANKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_WAITFORVERTICALBLANKDATA", pack=False, align=None),
    "DDHAL_WAITFORVPORTSYNCDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLine", SimTypeInt(signed=False, label="UInt32")), ("dwTimeOut", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("WaitForVideoPortSync", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_WAITFORVPORTSYNCDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DDHAL_WAITFORVPORTSYNCDATA", pack=False, align=None),
    "DDKERNELCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwIRQCaps", SimTypeInt(signed=False, label="UInt32")),)), name="DDKERNELCAPS", pack=False, align=None),
    "DDLOCKININFO": SimStruct(OrderedDict((("lpSurfaceData", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)),)), name="DDLOCKININFO", pack=False, align=None),
    "DDLOCKOUTINFO": SimStruct(OrderedDict((("dwSurfacePtr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDLOCKOUTINFO", pack=False, align=None),
    "DDMCBUFFERINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpCompSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("dwDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpPrivate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DDMCBUFFERINFO", pack=False, align=None),
    "DDMCCOMPBUFFERINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumCompBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwWidthToCreate", SimTypeInt(signed=False, label="UInt32")), ("dwHeightToCreate", SimTypeInt(signed=False, label="UInt32")), ("dwBytesToAllocate", SimTypeInt(signed=False, label="UInt32")), ("ddCompCaps", SimTypeRef("DDSCAPS2", SimStruct)), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="DDMCCOMPBUFFERINFO", pack=False, align=None),
    "DDMDL": SimStruct(OrderedDict((("MdlNext", SimTypePointer(SimTypeRef("DDMDL", SimStruct), offset=0)), ("MdlSize", SimTypeShort(signed=True, label="Int16")), ("MdlFlags", SimTypeShort(signed=True, label="Int16")), ("Process", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpMappedSystemVa", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpStartVa", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ByteCount", SimTypeInt(signed=False, label="UInt32")), ("ByteOffset", SimTypeInt(signed=False, label="UInt32")),)), name="DDMDL", pack=False, align=None),
    "DDMOCOMPBUFFERINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpCompSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpPrivate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DDMOCOMPBUFFERINFO", pack=False, align=None),
    "DDMONITORINFO": SimStruct(OrderedDict((("Manufacturer", SimTypeShort(signed=False, label="UInt16")), ("Product", SimTypeShort(signed=False, label="UInt16")), ("SerialNumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceIdentifier", SimTypeBottom(label="Guid")), ("Mode640x480", SimTypeInt(signed=True, label="Int32")), ("Mode800x600", SimTypeInt(signed=True, label="Int32")), ("Mode1024x768", SimTypeInt(signed=True, label="Int32")), ("Mode1280x1024", SimTypeInt(signed=True, label="Int32")), ("Mode1600x1200", SimTypeInt(signed=True, label="Int32")), ("ModeReserved1", SimTypeInt(signed=True, label="Int32")), ("ModeReserved2", SimTypeInt(signed=True, label="Int32")), ("ModeReserved3", SimTypeInt(signed=True, label="Int32")),)), name="DDMONITORINFO", pack=False, align=None),
    "DDMORESURFACECAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ddsCapsMore", SimTypeRef("DDSCAPSEX", SimStruct)), ("ddsExtendedHeapRestrictions", SimTypePointer(SimTypeRef("ExtendedHeapRestrictions", SimStruct), offset=0)),)), name="DDMORESURFACECAPS", pack=False, align=None),
    "DDNONLOCALVIDMEMCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="DDNONLOCALVIDMEMCAPS", pack=False, align=None),
    "DDNTCORECAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFXAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwPalCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVCaps", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaBltSurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayConstBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlayPixelBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaOverlaySurfaceBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferBitDepths", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemTotal", SimTypeInt(signed=False, label="UInt32")), ("dwVidMemFree", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVisibleOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCCCodes", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundarySrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeSrc", SimTypeInt(signed=False, label="UInt32")), ("dwAlignBoundaryDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignSizeDest", SimTypeInt(signed=False, label="UInt32")), ("dwAlignStrideAlign", SimTypeInt(signed=False, label="UInt32")), ("dwRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwMinOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxOverlayStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLiveVideoStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMinHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwCodecStretch", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwVSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwSSBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("dwMaxVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwCurrVideoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwSVBCaps2", SimTypeInt(signed=False, label="UInt32")),)), name="DDNTCORECAPS", pack=False, align=None),
    "DDNT_DEFERRED_AGP_AWARE_DATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)),)), name="DDNT_DEFERRED_AGP_AWARE_DATA", pack=False, align=None),
    "DDNT_DXVERSION": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwDXVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_DXVERSION", pack=False, align=None),
    "DDNT_FREE_DEFERRED_AGP_DATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_FREE_DEFERRED_AGP_DATA", pack=False, align=None),
    "DDNT_GETADAPTERGROUPDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("ulUniqueAdapterGroupId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_GETADAPTERGROUPDATA", pack=False, align=None),
    "DDNT_GETD3DQUERYCOUNTDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwNumQueries", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_GETD3DQUERYCOUNTDATA", pack=False, align=None),
    "DDNT_GETD3DQUERYDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("Anonymous", SimUnion({"dwQueryIndex": SimTypeInt(signed=False, label="UInt32"), "QueryType": SimTypeInt(signed=False, label="D3DQUERYTYPE")}, name="<anon>", label="None")),)), name="DDNT_GETD3DQUERYDATA", pack=False, align=None),
    "DDNT_GETDDIVERSIONDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwDXVersion", SimTypeInt(signed=False, label="UInt32")), ("dwDDIVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_GETDDIVERSIONDATA", pack=False, align=None),
    "DDNT_GETDRIVERINFO2DATA": SimStruct(OrderedDict((("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwExpectedSize", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_GETDRIVERINFO2DATA", pack=False, align=None),
    "DDNT_GETEXTENDEDMODECOUNTDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwModeCount", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_GETEXTENDEDMODECOUNTDATA", pack=False, align=None),
    "DDNT_GETEXTENDEDMODEDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwModeIndex", SimTypeInt(signed=False, label="UInt32")), ("mode", SimTypeRef("D3DDISPLAYMODE", SimStruct)),)), name="DDNT_GETEXTENDEDMODEDATA", pack=False, align=None),
    "DDNT_GETFORMATCOUNTDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwFormatCount", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_GETFORMATCOUNTDATA", pack=False, align=None),
    "DDNT_GETFORMATDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("dwFormatIndex", SimTypeInt(signed=False, label="UInt32")), ("format", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="DDNT_GETFORMATDATA", pack=False, align=None),
    "DDNT_MULTISAMPLEQUALITYLEVELSDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DDNT_GETDRIVERINFO2DATA", SimStruct)), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("_bitfield", SimTypeInt(signed=True, label="Int32")), ("QualityLevels", SimTypeInt(signed=False, label="UInt32")),)), name="DDNT_MULTISAMPLEQUALITYLEVELSDATA", pack=False, align=None),
    "DDOPTSURFACEDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddSCaps", SimTypeRef("DDSCAPS2", SimStruct)), ("ddOSCaps", SimTypeRef("DDOSCAPS", SimStruct)), ("guid", SimTypeBottom(label="Guid")), ("dwCompressionRatio", SimTypeInt(signed=False, label="UInt32")),)), name="DDOPTSURFACEDESC", pack=False, align=None),
    "DDOSCAPS": SimStruct(OrderedDict((("dwCaps", SimTypeInt(signed=False, label="UInt32")),)), name="DDOSCAPS", pack=False, align=None),
    "DDOVERLAYFX": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaEdgeBlendBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaEdgeBlend", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaDestConstBitDepth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"dwAlphaDestConst": SimTypeInt(signed=False, label="UInt32"), "lpDDSAlphaDest": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("dwAlphaSrcConstBitDepth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"dwAlphaSrcConst": SimTypeInt(signed=False, label="UInt32"), "lpDDSAlphaSrc": SimTypeBottom(label="IDirectDrawSurface")}, name="<anon>", label="None")), ("dckDestColorkey", SimTypeRef("DDCOLORKEY", SimStruct)), ("dckSrcColorkey", SimTypeRef("DDCOLORKEY", SimStruct)), ("dwDDFX", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDOVERLAYFX", pack=False, align=None),
    "DDPIXELFORMAT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFourCC", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"dwRGBBitCount": SimTypeInt(signed=False, label="UInt32"), "dwYUVBitCount": SimTypeInt(signed=False, label="UInt32"), "dwZBufferBitDepth": SimTypeInt(signed=False, label="UInt32"), "dwAlphaBitDepth": SimTypeInt(signed=False, label="UInt32"), "dwLuminanceBitCount": SimTypeInt(signed=False, label="UInt32"), "dwBumpBitCount": SimTypeInt(signed=False, label="UInt32"), "dwPrivateFormatBitCount": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"dwRBitMask": SimTypeInt(signed=False, label="UInt32"), "dwYBitMask": SimTypeInt(signed=False, label="UInt32"), "dwStencilBitDepth": SimTypeInt(signed=False, label="UInt32"), "dwLuminanceBitMask": SimTypeInt(signed=False, label="UInt32"), "dwBumpDuBitMask": SimTypeInt(signed=False, label="UInt32"), "dwOperations": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"dwGBitMask": SimTypeInt(signed=False, label="UInt32"), "dwUBitMask": SimTypeInt(signed=False, label="UInt32"), "dwZBitMask": SimTypeInt(signed=False, label="UInt32"), "dwBumpDvBitMask": SimTypeInt(signed=False, label="UInt32"), "MultiSampleCaps": SimTypeRef("_MultiSampleCaps_e__Struct", SimStruct)}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"dwBBitMask": SimTypeInt(signed=False, label="UInt32"), "dwVBitMask": SimTypeInt(signed=False, label="UInt32"), "dwStencilBitMask": SimTypeInt(signed=False, label="UInt32"), "dwBumpLuminanceBitMask": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous5", SimUnion({"dwRGBAlphaBitMask": SimTypeInt(signed=False, label="UInt32"), "dwYUVAlphaBitMask": SimTypeInt(signed=False, label="UInt32"), "dwLuminanceAlphaBitMask": SimTypeInt(signed=False, label="UInt32"), "dwRGBZBitMask": SimTypeInt(signed=False, label="UInt32"), "dwYUVZBitMask": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DDPIXELFORMAT", pack=False, align=None),
    "DDP_FILE_EXTENT": SimStruct(OrderedDict((("Length", SimTypeLongLong(signed=True, label="Int64")), ("Offset", SimTypeLongLong(signed=True, label="Int64")),)), name="DDP_FILE_EXTENT", pack=False, align=None),
    "DDRAWI_DDMOTIONCOMP_INT": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLcl", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_LCL", SimStruct), offset=0)), ("lpLink", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_INT", SimStruct), offset=0)), ("dwIntRefCnt", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DDMOTIONCOMP_INT", pack=False, align=None),
    "DDRAWI_DDMOTIONCOMP_LCL": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("guid", SimTypeBottom(label="Guid")), ("dwUncompWidth", SimTypeInt(signed=False, label="UInt32")), ("dwUncompHeight", SimTypeInt(signed=False, label="UInt32")), ("ddUncompPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwInternalFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("hMoComp", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwDriverReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved3", SimTypeInt(signed=False, label="UInt32")), ("lpDriverReserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDriverReserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDriverReserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DDRAWI_DDMOTIONCOMP_LCL", pack=False, align=None),
    "DDRAWI_DDRAWCLIPPER_GBL": SimStruct(OrderedDict((("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpStaticClipList", SimTypePointer(SimTypeRef("RGNDATA", SimStruct), offset=0)),)), name="DDRAWI_DDRAWCLIPPER_GBL", pack=False, align=None),
    "DDRAWI_DDRAWCLIPPER_INT": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLcl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_LCL", SimStruct), offset=0)), ("lpLink", SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_INT", SimStruct), offset=0)), ("dwIntRefCnt", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DDRAWCLIPPER_INT", pack=False, align=None),
    "DDRAWI_DDRAWCLIPPER_LCL": SimStruct(OrderedDict((("lpClipMore", SimTypeInt(signed=False, label="UInt32")), ("lpGbl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_GBL", SimStruct), offset=0)), ("lpDD_lcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwLocalRefCnt", SimTypeInt(signed=False, label="UInt32")), ("pUnkOuter", SimTypeBottom(label="IUnknown")), ("lpDD_int", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_INT", SimStruct), offset=0)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pAddrefedThisOwner", SimTypeBottom(label="IUnknown")),)), name="DDRAWI_DDRAWCLIPPER_LCL", pack=False, align=None),
    "DDRAWI_DDRAWPALETTE_GBL": SimStruct(OrderedDict((("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDD_lcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("lpColorTable", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)), ("Anonymous", SimUnion({"dwReserved1": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "hHELGDIPalette": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("dwDriverReserved", SimTypeInt(signed=False, label="UInt32")), ("dwContentsStamp", SimTypeInt(signed=False, label="UInt32")), ("dwSaveStamp", SimTypeInt(signed=False, label="UInt32")), ("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DDRAWPALETTE_GBL", pack=False, align=None),
    "DDRAWI_DDRAWPALETTE_INT": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLcl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_LCL", SimStruct), offset=0)), ("lpLink", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_INT", SimStruct), offset=0)), ("dwIntRefCnt", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DDRAWPALETTE_INT", pack=False, align=None),
    "DDRAWI_DDRAWPALETTE_LCL": SimStruct(OrderedDict((("lpPalMore", SimTypeInt(signed=False, label="UInt32")), ("lpGbl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_GBL", SimStruct), offset=0)), ("dwUnused0", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwLocalRefCnt", SimTypeInt(signed=False, label="UInt32")), ("pUnkOuter", SimTypeBottom(label="IUnknown")), ("lpDD_lcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDDRAWReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDDRAWReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDDRAWReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDRAWI_DDRAWPALETTE_LCL", pack=False, align=None),
    "DDRAWI_DDRAWSURFACE_GBL": SimStruct(OrderedDict((("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwGlobalFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lpRectList": SimTypePointer(SimTypeRef("ACCESSRECTLIST", SimStruct), offset=0), "dwBlockSizeY": SimTypeInt(signed=False, label="UInt32"), "lSlicePitch": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"lpVidMemHeap": SimTypePointer(SimTypeRef("VMEMHEAP", SimStruct), offset=0), "dwBlockSizeX": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"lpDD": SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), "lpDDHandle": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("fpVidMem", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous4", SimUnion({"lPitch": SimTypeInt(signed=True, label="Int32"), "dwLinearSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("wHeight", SimTypeShort(signed=False, label="UInt16")), ("wWidth", SimTypeShort(signed=False, label="UInt16")), ("dwUsageCount", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddpfSurface", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="DDRAWI_DDRAWSURFACE_GBL", pack=False, align=None),
    "DDRAWI_DDRAWSURFACE_GBL_MORE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwPhysicalPageTable": SimTypeInt(signed=False, label="UInt32"), "fpPhysicalVidMem": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("pPageTable", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cPages", SimTypeInt(signed=False, label="UInt32")), ("dwSavedDCContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fpAliasedVidMem", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDriverReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHELReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cPageUnlocks", SimTypeInt(signed=False, label="UInt32")), ("hKernelSurface", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwKernelRefCnt", SimTypeInt(signed=False, label="UInt32")), ("lpColorInfo", SimTypePointer(SimTypeRef("DDCOLORCONTROL", SimStruct), offset=0)), ("fpNTAlias", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwContentsStamp", SimTypeInt(signed=False, label="UInt32")), ("lpvUnswappedDriverReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDDRAWReserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDDRAWReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDDRAWReserved2", SimTypeInt(signed=False, label="UInt32")), ("fpAliasOfVidMem", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDRAWI_DDRAWSURFACE_GBL_MORE", pack=False, align=None),
    "DDRAWI_DDRAWSURFACE_INT": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLcl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0)), ("lpLink", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("dwIntRefCnt", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DDRAWSURFACE_INT", pack=False, align=None),
    "DDRAWI_DDRAWSURFACE_LCL": SimStruct(OrderedDict((("lpSurfMore", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_MORE", SimStruct), offset=0)), ("lpGbl", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_GBL", SimStruct), offset=0)), ("hDDSurface", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpAttachList", SimTypePointer(SimTypeRef("ATTACHLIST", SimStruct), offset=0)), ("lpAttachListFrom", SimTypePointer(SimTypeRef("ATTACHLIST", SimStruct), offset=0)), ("dwLocalRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("Anonymous1", SimUnion({"lpDDPalette": SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_INT", SimStruct), offset=0), "lp16DDPalette": SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_INT", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"lpDDClipper": SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_LCL", SimStruct), offset=0), "lp16DDClipper": SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_INT", SimStruct), offset=0)}, name="<anon>", label="None")), ("dwModeCreatedIn", SimTypeInt(signed=False, label="UInt32")), ("dwBackBufferCount", SimTypeInt(signed=False, label="UInt32")), ("ddckCKDestBlt", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKSrcBlt", SimTypeRef("DDCOLORKEY", SimStruct)), ("hDC", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddckCKSrcOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKDestOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("lpSurfaceOverlaying", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("dbnOverlayNode", SimTypeRef("DBLNODE", SimStruct)), ("rcOverlaySrc", SimTypeRef("RECT", SimStruct)), ("rcOverlayDest", SimTypeRef("RECT", SimStruct)), ("dwClrXparent", SimTypeInt(signed=False, label="UInt32")), ("dwAlpha", SimTypeInt(signed=False, label="UInt32")), ("lOverlayX", SimTypeInt(signed=True, label="Int32")), ("lOverlayY", SimTypeInt(signed=True, label="Int32")),)), name="DDRAWI_DDRAWSURFACE_LCL", pack=False, align=None),
    "DDRAWI_DDRAWSURFACE_MORE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpIUnknowns", SimTypePointer(SimTypeRef("IUNKNOWN_LIST", SimStruct), offset=0)), ("lpDD_lcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwPageLockCount", SimTypeInt(signed=False, label="UInt32")), ("dwBytesAllocated", SimTypeInt(signed=False, label="UInt32")), ("lpDD_int", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_INT", SimStruct), offset=0)), ("dwMipMapCount", SimTypeInt(signed=False, label="UInt32")), ("lpDDIClipper", SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_INT", SimStruct), offset=0)), ("lpHeapAliasInfo", SimTypePointer(SimTypeRef("HEAPALIASINFO", SimStruct), offset=0)), ("dwOverlayFlags", SimTypeInt(signed=False, label="UInt32")), ("rgjunc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("lpddOverlayFX", SimTypePointer(SimTypeRef("DDOVERLAYFX", SimStruct), offset=0)), ("ddsCapsEx", SimTypeRef("DDSCAPSEX", SimStruct)), ("dwTextureStage", SimTypeInt(signed=False, label="UInt32")), ("lpDDRAWReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDDRAWReserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDDrawReserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDDrawReserved4", SimTypeInt(signed=False, label="UInt32")), ("lpDDrawReserved5", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpGammaRamp", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpOriginalGammaRamp", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpDDrawReserved6", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwSurfaceHandle", SimTypeInt(signed=False, label="UInt32")), ("qwDDrawReserved8", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpDDrawReserved9", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cSurfaces", SimTypeInt(signed=False, label="UInt32")), ("pCreatedDDSurfaceDesc2", SimTypePointer(SimTypeRef("DDSURFACEDESC2", SimStruct), offset=0)), ("slist", SimTypePointer(SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_LCL", SimStruct), offset=0), offset=0)), ("dwFVF", SimTypeInt(signed=False, label="UInt32")), ("lpVB", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DDRAWI_DDRAWSURFACE_MORE", pack=False, align=None),
    "DDRAWI_DDVIDEOPORT_INT": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLcl", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_LCL", SimStruct), offset=0)), ("lpLink", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_INT", SimStruct), offset=0)), ("dwIntRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DDVIDEOPORT_INT", pack=False, align=None),
    "DDRAWI_DDVIDEOPORT_LCL": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("ddvpDesc", SimTypeRef("DDVIDEOPORTDESC", SimStruct)), ("ddvpInfo", SimTypeRef("DDVIDEOPORTINFO", SimStruct)), ("lpSurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("lpVBISurface", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("lpFlipInts", SimTypePointer(SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0), offset=0)), ("dwNumAutoflip", SimTypeInt(signed=False, label="UInt32")), ("dwProcessID", SimTypeInt(signed=False, label="UInt32")), ("dwStateFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("fpLastFlip", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hDDVideoPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwNumVBIAutoflip", SimTypeInt(signed=False, label="UInt32")), ("lpVBIDesc", SimTypePointer(SimTypeRef("DDVIDEOPORTDESC", SimStruct), offset=0)), ("lpVideoDesc", SimTypePointer(SimTypeRef("DDVIDEOPORTDESC", SimStruct), offset=0)), ("lpVBIInfo", SimTypePointer(SimTypeRef("DDVIDEOPORTINFO", SimStruct), offset=0)), ("lpVideoInfo", SimTypePointer(SimTypeRef("DDVIDEOPORTINFO", SimStruct), offset=0)), ("dwVBIProcessID", SimTypeInt(signed=False, label="UInt32")), ("lpVPNotify", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_INT", SimStruct), offset=0)),)), name="DDRAWI_DDVIDEOPORT_LCL", pack=False, align=None),
    "DDRAWI_DIRECTDRAW_GBL": SimStruct(OrderedDict((("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fpPrimaryOrig", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddCaps", SimTypeRef("DDCORECAPS", SimStruct)), ("dwInternal1", SimTypeInt(signed=False, label="UInt32")), ("dwUnused1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 9)), ("lpDDCBtmp", SimTypePointer(SimTypeRef("DDHAL_CALLBACKS", SimStruct), offset=0)), ("dsList", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("palList", SimTypePointer(SimTypeRef("DDRAWI_DDRAWPALETTE_INT", SimStruct), offset=0)), ("clipperList", SimTypePointer(SimTypeRef("DDRAWI_DDRAWCLIPPER_INT", SimStruct), offset=0)), ("lp16DD", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwMaxOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwCurrOverlays", SimTypeInt(signed=False, label="UInt32")), ("dwMonitorFrequency", SimTypeInt(signed=False, label="UInt32")), ("ddHELCaps", SimTypeRef("DDCORECAPS", SimStruct)), ("dwUnused2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 50)), ("ddckCKDestOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKSrcOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("vmiData", SimTypeRef("VIDMEMINFO", SimStruct)), ("lpDriverHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpExclusiveOwner", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("dwModeIndex", SimTypeInt(signed=False, label="UInt32")), ("dwModeIndexOrig", SimTypeInt(signed=False, label="UInt32")), ("dwNumFourCC", SimTypeInt(signed=False, label="UInt32")), ("lpdwFourCC", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("dwNumModes", SimTypeInt(signed=False, label="UInt32")), ("lpModeInfo", SimTypePointer(SimTypeRef("DDHALMODEINFO", SimStruct), offset=0)), ("plProcessList", SimTypeRef("PROCESS_LIST", SimStruct)), ("dwSurfaceLockCount", SimTypeInt(signed=False, label="UInt32")), ("dwAliasedLockCnt", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hDD", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cObsolete", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dbnOverlayRoot", SimTypeRef("DBLNODE", SimStruct)), ("lpwPDeviceFlags", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("dwPDevice", SimTypeInt(signed=False, label="UInt32")), ("dwWin16LockCnt", SimTypeInt(signed=False, label="UInt32")), ("dwUnused3", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypeInt(signed=False, label="UInt32")), ("dwEvent16", SimTypeInt(signed=False, label="UInt32")), ("dwSaveNumModes", SimTypeInt(signed=False, label="UInt32")), ("lpD3DGlobalDriverData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpD3DHALCallbacks", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddBothCaps", SimTypeRef("DDCORECAPS", SimStruct)), ("lpDDVideoPortCaps", SimTypePointer(SimTypeRef("DDVIDEOPORTCAPS", SimStruct), offset=0)), ("dvpList", SimTypePointer(SimTypeRef("DDRAWI_DDVIDEOPORT_INT", SimStruct), offset=0)), ("lpD3DHALCallbacks2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rectDevice", SimTypeRef("RECT", SimStruct)), ("cMonitors", SimTypeInt(signed=False, label="UInt32")), ("gpbmiSrc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("gpbmiDest", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("phaiHeapAliases", SimTypePointer(SimTypeRef("HEAPALIASINFO", SimStruct), offset=0)), ("hKernelHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pfnNotifyProc", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpDDKernelCaps", SimTypePointer(SimTypeRef("DDKERNELCAPS", SimStruct), offset=0)), ("lpddNLVCaps", SimTypePointer(SimTypeRef("DDNONLOCALVIDMEMCAPS", SimStruct), offset=0)), ("lpddNLVHELCaps", SimTypePointer(SimTypeRef("DDNONLOCALVIDMEMCAPS", SimStruct), offset=0)), ("lpddNLVBothCaps", SimTypePointer(SimTypeRef("DDNONLOCALVIDMEMCAPS", SimStruct), offset=0)), ("lpD3DExtendedCaps", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwDOSBoxEvent", SimTypeInt(signed=False, label="UInt32")), ("rectDesktop", SimTypeRef("RECT", SimStruct)), ("cDriverName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("lpD3DHALCallbacks3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwNumZPixelFormats", SimTypeInt(signed=False, label="UInt32")), ("lpZPixelFormats", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("mcList", SimTypePointer(SimTypeRef("DDRAWI_DDMOTIONCOMP_INT", SimStruct), offset=0)), ("hDDVxd", SimTypeInt(signed=False, label="UInt32")), ("ddsCapsMore", SimTypeRef("DDSCAPSEX", SimStruct)),)), name="DDRAWI_DIRECTDRAW_GBL", pack=False, align=None),
    "DDRAWI_DIRECTDRAW_INT": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLcl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_LCL", SimStruct), offset=0)), ("lpLink", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_INT", SimStruct), offset=0)), ("dwIntRefCnt", SimTypeInt(signed=False, label="UInt32")),)), name="DDRAWI_DIRECTDRAW_INT", pack=False, align=None),
    "DDRAWI_DIRECTDRAW_LCL": SimStruct(OrderedDict((("lpDDMore", SimTypeInt(signed=False, label="UInt32")), ("lpGbl", SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0)), ("dwUnused0", SimTypeInt(signed=False, label="UInt32")), ("dwLocalFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLocalRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("pUnkOuter", SimTypeBottom(label="IUnknown")), ("dwObsolete1", SimTypeInt(signed=False, label="UInt32")), ("hWnd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwErrorMode", SimTypeInt(signed=False, label="UInt32")), ("lpPrimary", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("lpCB", SimTypePointer(SimTypeRef("DDRAWI_DDRAWSURFACE_INT", SimStruct), offset=0)), ("dwPreferredMode", SimTypeInt(signed=False, label="UInt32")), ("hD3DInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pD3DIUnknown", SimTypeBottom(label="IUnknown")), ("lpDDCB", SimTypePointer(SimTypeRef("DDHAL_CALLBACKS", SimStruct), offset=0)), ("hDDVxd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwAppHackFlags", SimTypeInt(signed=False, label="UInt32")), ("hFocusWnd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHotTracking", SimTypeInt(signed=False, label="UInt32")), ("dwIMEState", SimTypeInt(signed=False, label="UInt32")), ("hWndPopup", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hDD", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hGammaCalibrator", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpGammaCalibrator", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDGAMMARAMP", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)),)), name="DDRAWI_DIRECTDRAW_LCL", pack=False, align=None),
    "DDRGBA": SimStruct(OrderedDict((("red", SimTypeChar(label="Byte")), ("green", SimTypeChar(label="Byte")), ("blue", SimTypeChar(label="Byte")), ("alpha", SimTypeChar(label="Byte")),)), name="DDRGBA", pack=False, align=None),
    "DDSCAPS": SimStruct(OrderedDict((("dwCaps", SimTypeInt(signed=False, label="UInt32")),)), name="DDSCAPS", pack=False, align=None),
    "DDSCAPS2": SimStruct(OrderedDict((("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCaps3", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwCaps4": SimTypeInt(signed=False, label="UInt32"), "dwVolumeDepth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DDSCAPS2", pack=False, align=None),
    "DDSCAPSEX": SimStruct(OrderedDict((("dwCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwCaps3", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwCaps4": SimTypeInt(signed=False, label="UInt32"), "dwVolumeDepth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DDSCAPSEX", pack=False, align=None),
    "DDSETSTATEININFO": SimStruct(OrderedDict((("lpSurfaceData", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)), ("lpVideoPortData", SimTypePointer(SimTypeRef("DDVIDEOPORTDATA", SimStruct), offset=0)),)), name="DDSETSTATEININFO", pack=False, align=None),
    "DDSETSTATEOUTINFO": SimStruct(OrderedDict((("bSoftwareAutoflip", SimTypeInt(signed=True, label="Int32")), ("dwSurfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("dwVBISurfaceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="DDSETSTATEOUTINFO", pack=False, align=None),
    "DDSFF_FLAGS": SimTypeInt(signed=False, label="DDSFF_FLAGS"),
    "DDSKIPNEXTFIELDINFO": SimStruct(OrderedDict((("lpVideoPortData", SimTypePointer(SimTypeRef("DDVIDEOPORTDATA", SimStruct), offset=0)), ("dwSkipFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DDSKIPNEXTFIELDINFO", pack=False, align=None),
    "DDSTEREOMODE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwBpp", SimTypeInt(signed=False, label="UInt32")), ("dwRefreshRate", SimTypeInt(signed=False, label="UInt32")), ("bSupported", SimTypeInt(signed=True, label="Int32")),)), name="DDSTEREOMODE", pack=False, align=None),
    "DDSURFACEDATA": SimStruct(OrderedDict((("ddsCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSurfaceOffset", SimTypeInt(signed=False, label="UInt32")), ("fpLockPtr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("lPitch", SimTypeInt(signed=True, label="Int32")), ("dwOverlayFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayOffset", SimTypeInt(signed=False, label="UInt32")), ("dwOverlaySrcWidth", SimTypeInt(signed=False, label="UInt32")), ("dwOverlaySrcHeight", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayDestWidth", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayDestHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVideoPortId", SimTypeInt(signed=False, label="UInt32")), ("dwFormatFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFormatFourCC", SimTypeInt(signed=False, label="UInt32")), ("dwFormatBitCount", SimTypeInt(signed=False, label="UInt32")), ("dwRBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwGBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwBBitMask", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved4", SimTypeInt(signed=False, label="UInt32")),)), name="DDSURFACEDATA", pack=False, align=None),
    "DDSURFACEDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lPitch": SimTypeInt(signed=True, label="Int32"), "dwLinearSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dwBackBufferCount", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"dwMipMapCount": SimTypeInt(signed=False, label="UInt32"), "dwZBufferBitDepth": SimTypeInt(signed=False, label="UInt32"), "dwRefreshRate": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dwAlphaBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("lpSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ddckCKDestOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKDestBlt", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKSrcOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKSrcBlt", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddpfPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)),)), name="DDSURFACEDESC", pack=False, align=None),
    "DDSURFACEDESC2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lPitch": SimTypeInt(signed=True, label="Int32"), "dwLinearSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"dwBackBufferCount": SimTypeInt(signed=False, label="UInt32"), "dwDepth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"dwMipMapCount": SimTypeInt(signed=False, label="UInt32"), "dwRefreshRate": SimTypeInt(signed=False, label="UInt32"), "dwSrcVBHandle": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dwAlphaBitDepth", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("lpSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Anonymous4", SimUnion({"ddckCKDestOverlay": SimTypeRef("DDCOLORKEY", SimStruct), "dwEmptyFaceColor": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ddckCKDestBlt", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKSrcOverlay", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddckCKSrcBlt", SimTypeRef("DDCOLORKEY", SimStruct)), ("Anonymous5", SimUnion({"ddpfPixelFormat": SimTypeRef("DDPIXELFORMAT", SimStruct), "dwFVF": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ddsCaps", SimTypeRef("DDSCAPS2", SimStruct)), ("dwTextureStage", SimTypeInt(signed=False, label="UInt32")),)), name="DDSURFACEDESC2", pack=False, align=None),
    "DDTRANSFERININFO": SimStruct(OrderedDict((("lpSurfaceData", SimTypePointer(SimTypeRef("DDSURFACEDATA", SimStruct), offset=0)), ("dwStartLine", SimTypeInt(signed=False, label="UInt32")), ("dwEndLine", SimTypeInt(signed=False, label="UInt32")), ("dwTransferID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTransferFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDestMDL", SimTypePointer(SimTypeRef("DDMDL", SimStruct), offset=0)),)), name="DDTRANSFERININFO", pack=False, align=None),
    "DDTRANSFEROUTINFO": SimStruct(OrderedDict((("dwBufferPolarity", SimTypeInt(signed=False, label="UInt32")),)), name="DDTRANSFEROUTINFO", pack=False, align=None),
    "DDVERSIONDATA": SimStruct(OrderedDict((("dwHALVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDVERSIONDATA", pack=False, align=None),
    "DDVIDEOPORTBANDWIDTH": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwOverlay", SimTypeInt(signed=False, label="UInt32")), ("dwColorkey", SimTypeInt(signed=False, label="UInt32")), ("dwYInterpolate", SimTypeInt(signed=False, label="UInt32")), ("dwYInterpAndColorkey", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDVIDEOPORTBANDWIDTH", pack=False, align=None),
    "DDVIDEOPORTCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwMaxWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVBIWidth", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVideoPortID", SimTypeInt(signed=False, label="UInt32")), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFX", SimTypeInt(signed=False, label="UInt32")), ("dwNumAutoFlipSurfaces", SimTypeInt(signed=False, label="UInt32")), ("dwAlignVideoPortBoundary", SimTypeInt(signed=False, label="UInt32")), ("dwAlignVideoPortPrescaleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwAlignVideoPortCropBoundary", SimTypeInt(signed=False, label="UInt32")), ("dwAlignVideoPortCropWidth", SimTypeInt(signed=False, label="UInt32")), ("dwPreshrinkXStep", SimTypeInt(signed=False, label="UInt32")), ("dwPreshrinkYStep", SimTypeInt(signed=False, label="UInt32")), ("dwNumVBIAutoFlipSurfaces", SimTypeInt(signed=False, label="UInt32")), ("dwNumPreferredAutoflip", SimTypeInt(signed=False, label="UInt32")), ("wNumFilterTapsX", SimTypeShort(signed=False, label="UInt16")), ("wNumFilterTapsY", SimTypeShort(signed=False, label="UInt16")),)), name="DDVIDEOPORTCAPS", pack=False, align=None),
    "DDVIDEOPORTCONNECT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwPortWidth", SimTypeInt(signed=False, label="UInt32")), ("guidTypeID", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDVIDEOPORTCONNECT", pack=False, align=None),
    "DDVIDEOPORTDATA": SimStruct(OrderedDict((("dwVideoPortId", SimTypeInt(signed=False, label="UInt32")), ("dwVPFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOriginOffset", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVBIHeight", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="DDVIDEOPORTDATA", pack=False, align=None),
    "DDVIDEOPORTDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFieldWidth", SimTypeInt(signed=False, label="UInt32")), ("dwVBIWidth", SimTypeInt(signed=False, label="UInt32")), ("dwFieldHeight", SimTypeInt(signed=False, label="UInt32")), ("dwMicrosecondsPerField", SimTypeInt(signed=False, label="UInt32")), ("dwMaxPixelsPerSecond", SimTypeInt(signed=False, label="UInt32")), ("dwVideoPortID", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("VideoPortType", SimTypeRef("DDVIDEOPORTCONNECT", SimStruct)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDVIDEOPORTDESC", pack=False, align=None),
    "DDVIDEOPORTINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwOriginX", SimTypeInt(signed=False, label="UInt32")), ("dwOriginY", SimTypeInt(signed=False, label="UInt32")), ("dwVPFlags", SimTypeInt(signed=False, label="UInt32")), ("rCrop", SimTypeRef("RECT", SimStruct)), ("dwPrescaleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwPrescaleHeight", SimTypeInt(signed=False, label="UInt32")), ("lpddpfInputFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("lpddpfVBIInputFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("lpddpfVBIOutputFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwVBIHeight", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDVIDEOPORTINFO", pack=False, align=None),
    "DDVIDEOPORTNOTIFY": SimStruct(OrderedDict((("ApproximateTimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("lField", SimTypeInt(signed=True, label="Int32")), ("dwSurfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("lDone", SimTypeInt(signed=True, label="Int32")),)), name="DDVIDEOPORTNOTIFY", pack=False, align=None),
    "DDVIDEOPORTSTATUS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("bInUse", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("VideoPortType", SimTypeRef("DDVIDEOPORTCONNECT", SimStruct)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DDVIDEOPORTSTATUS", pack=False, align=None),
    "DD_ADDATTACHEDSURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpSurfAttached", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("AddAttachedSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_ADDATTACHEDSURFACEDATA", pack=False, align=None),
    "DD_ATTACHLIST": SimStruct(OrderedDict((("lpLink", SimTypePointer(SimTypeRef("DD_ATTACHLIST", SimStruct), offset=0)), ("lpAttached", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)),)), name="DD_ATTACHLIST", pack=False, align=None),
    "DD_BEGINMOCOMPFRAMEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DD_MOTIONCOMP_LOCAL", SimStruct), offset=0)), ("lpDestSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwOutputDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_BEGINMOCOMPFRAMEDATA", pack=False, align=None),
    "DD_BLTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDDestSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("rDest", SimTypeRef("RECTL", SimStruct)), ("lpDDSrcSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("rSrc", SimTypeRef("RECTL", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwROPFlags", SimTypeInt(signed=False, label="UInt32")), ("bltFX", SimTypeRef("DDBLTFX", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Blt", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("IsClipped", SimTypeInt(signed=True, label="Int32")), ("rOrigDest", SimTypeRef("RECTL", SimStruct)), ("rOrigSrc", SimTypeRef("RECTL", SimStruct)), ("dwRectCnt", SimTypeInt(signed=False, label="UInt32")), ("prDestRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("dwAFlags", SimTypeInt(signed=False, label="UInt32")), ("ddargbScaleFactors", SimTypeRef("DDARGB", SimStruct)),)), name="DD_BLTDATA", pack=False, align=None),
    "DD_CALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DestroyDriver", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetColorKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DRVSETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("WaitForVerticalBlank", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_WAITFORVERTICALBLANKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CanCreateSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreatePalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATEPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetScanLine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETSCANLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("MapMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_MAPMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_CALLBACKS", pack=False, align=None),
    "DD_CANCREATESURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurfaceDesc", SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0)), ("bIsDifferentPixelFormat", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CanCreateSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_CANCREATESURFACEDATA", pack=False, align=None),
    "DD_CANCREATEVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpDDVideoPortDesc", SimTypePointer(SimTypeRef("DDVIDEOPORTDESC", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CanCreateVideoPort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_CANCREATEVPORTDATA", pack=False, align=None),
    "DD_CLIPPER_GLOBAL": SimStruct(OrderedDict((("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DD_CLIPPER_GLOBAL", pack=False, align=None),
    "DD_CLIPPER_LOCAL": SimStruct(OrderedDict((("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DD_CLIPPER_LOCAL", pack=False, align=None),
    "DD_COLORCONTROLCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ColorControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_COLORCONTROLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_COLORCONTROLCALLBACKS", pack=False, align=None),
    "DD_COLORCONTROLDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpColorData", SimTypePointer(SimTypeRef("DDCOLORCONTROL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("ColorControl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_COLORCONTROLDATA", pack=False, align=None),
    "DD_CREATEMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DD_MOTIONCOMP_LOCAL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwUncompWidth", SimTypeInt(signed=False, label="UInt32")), ("dwUncompHeight", SimTypeInt(signed=False, label="UInt32")), ("ddUncompPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_CREATEMOCOMPDATA", pack=False, align=None),
    "DD_CREATEPALETTEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DD_PALETTE_GLOBAL", SimStruct), offset=0)), ("lpColorTable", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreatePalette", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("is_excl", SimTypeInt(signed=True, label="Int32")),)), name="DD_CREATEPALETTEDATA", pack=False, align=None),
    "DD_CREATESURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurfaceDesc", SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0)), ("lplpSList", SimTypePointer(SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0), offset=0)), ("dwSCnt", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreateSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_CREATESURFACEDATA", pack=False, align=None),
    "DD_CREATESURFACEEXDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDDLcl", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpDDSLcl", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_CREATESURFACEEXDATA", pack=False, align=None),
    "DD_CREATEVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpDDVideoPortDesc", SimTypePointer(SimTypeRef("DDVIDEOPORTDESC", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("CreateVideoPort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_CREATEVPORTDATA", pack=False, align=None),
    "DD_D3DBUFCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("CanCreateD3DBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateD3DBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyD3DBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("LockD3DBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UnlockD3DBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_D3DBUFCALLBACKS", pack=False, align=None),
    "DD_DEFERRED_AGP_AWARE_DATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)),)), name="DD_DEFERRED_AGP_AWARE_DATA", pack=False, align=None),
    "DD_DESTROYDDLOCALDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pDDLcl", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_DESTROYDDLOCALDATA", pack=False, align=None),
    "DD_DESTROYMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DD_MOTIONCOMP_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_DESTROYMOCOMPDATA", pack=False, align=None),
    "DD_DESTROYPALETTEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DD_PALETTE_GLOBAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroyPalette", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_DESTROYPALETTEDATA", pack=False, align=None),
    "DD_DESTROYSURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroySurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_DESTROYSURFACEDATA", pack=False, align=None),
    "DD_DESTROYVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("DestroyVideoPort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_DESTROYVPORTDATA", pack=False, align=None),
    "DD_DIRECTDRAW_GLOBAL": SimStruct(OrderedDict((("dhpdev", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpDDVideoPortCaps", SimTypePointer(SimTypeRef("DDVIDEOPORTCAPS", SimStruct), offset=0)),)), name="DD_DIRECTDRAW_GLOBAL", pack=False, align=None),
    "DD_DIRECTDRAW_LOCAL": SimStruct(OrderedDict((("lpGbl", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)),)), name="DD_DIRECTDRAW_LOCAL", pack=False, align=None),
    "DD_DRVSETCOLORKEYDATA": SimStruct(OrderedDict((("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ckNew", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetColorKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_DRVSETCOLORKEYDATA", pack=False, align=None),
    "DD_DXVERSION": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwDXVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DD_DXVERSION", pack=False, align=None),
    "DD_ENDMOCOMPFRAMEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DD_MOTIONCOMP_LOCAL", SimStruct), offset=0)), ("lpInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_ENDMOCOMPFRAMEDATA", pack=False, align=None),
    "DD_FLAGS": SimTypeInt(signed=False, label="DD_FLAGS"),
    "DD_FLIPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpSurfCurr", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpSurfTarg", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Flip", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpSurfCurrLeft", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpSurfTargLeft", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)),)), name="DD_FLIPDATA", pack=False, align=None),
    "DD_FLIPTOGDISURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("dwToGDI", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("FlipToGDISurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_FLIPTOGDISURFACEDATA", pack=False, align=None),
    "DD_FLIPVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("lpSurfCurr", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpSurfTarg", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("FlipVideoPort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_FLIPVPORTDATA", pack=False, align=None),
    "DD_FREEDRIVERMEMORYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("FreeDriverMemory", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_FREEDRIVERMEMORYDATA", pack=False, align=None),
    "DD_FREE_DEFERRED_AGP_DATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")),)), name="DD_FREE_DEFERRED_AGP_DATA", pack=False, align=None),
    "DD_GETADAPTERGROUPDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("ulUniqueAdapterGroupId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="DD_GETADAPTERGROUPDATA", pack=False, align=None),
    "DD_GETAVAILDRIVERMEMORYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("DDSCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwTotal", SimTypeInt(signed=False, label="UInt32")), ("dwFree", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetAvailDriverMemory", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETAVAILDRIVERMEMORYDATA", pack=False, align=None),
    "DD_GETBLTSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetBltStatus", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETBLTSTATUSDATA", pack=False, align=None),
    "DD_GETD3DQUERYCOUNTDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwNumQueries", SimTypeInt(signed=False, label="UInt32")),)), name="DD_GETD3DQUERYCOUNTDATA", pack=False, align=None),
    "DD_GETD3DQUERYDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("Anonymous", SimUnion({"dwQueryIndex": SimTypeInt(signed=False, label="UInt32"), "QueryType": SimTypeInt(signed=False, label="D3DQUERYTYPE")}, name="<anon>", label="None")),)), name="DD_GETD3DQUERYDATA", pack=False, align=None),
    "DD_GETDDIVERSIONDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwDXVersion", SimTypeInt(signed=False, label="UInt32")), ("dwDDIVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DD_GETDDIVERSIONDATA", pack=False, align=None),
    "DD_GETDRIVERINFO2DATA": SimStruct(OrderedDict((("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwExpectedSize", SimTypeInt(signed=False, label="UInt32")),)), name="DD_GETDRIVERINFO2DATA", pack=False, align=None),
    "DD_GETDRIVERINFODATA": SimStruct(OrderedDict((("dhpdev", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidInfo", SimTypeBottom(label="Guid")), ("dwExpectedSize", SimTypeInt(signed=False, label="UInt32")), ("lpvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwActualSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_GETDRIVERINFODATA", pack=False, align=None),
    "DD_GETDRIVERSTATEDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"lpDD": SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0), "dwhContext": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("lpdwStates", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_GETDRIVERSTATEDATA", pack=False, align=None),
    "DD_GETEXTENDEDMODECOUNTDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwModeCount", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DD_GETEXTENDEDMODECOUNTDATA", pack=False, align=None),
    "DD_GETEXTENDEDMODEDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwModeIndex", SimTypeInt(signed=False, label="UInt32")), ("mode", SimTypeRef("D3DDISPLAYMODE", SimStruct)),)), name="DD_GETEXTENDEDMODEDATA", pack=False, align=None),
    "DD_GETFLIPSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetFlipStatus", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETFLIPSTATUSDATA", pack=False, align=None),
    "DD_GETFORMATCOUNTDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwFormatCount", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DD_GETFORMATCOUNTDATA", pack=False, align=None),
    "DD_GETFORMATDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("dwFormatIndex", SimTypeInt(signed=False, label="UInt32")), ("format", SimTypeRef("DDPIXELFORMAT", SimStruct)),)), name="DD_GETFORMATDATA", pack=False, align=None),
    "DD_GETHEAPALIGNMENTDATA": SimStruct(OrderedDict((("dwInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwHeap", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetHeapAlignment", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Alignment", SimTypeRef("HEAPALIGNMENT", SimStruct)),)), name="DD_GETHEAPALIGNMENTDATA", pack=False, align=None),
    "DD_GETINTERNALMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwScratchMemAlloc", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_GETINTERNALMOCOMPDATA", pack=False, align=None),
    "DD_GETMOCOMPCOMPBUFFDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("ddPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwNumTypesCompBuffs", SimTypeInt(signed=False, label="UInt32")), ("lpCompBuffInfo", SimTypePointer(SimTypeRef("DDCOMPBUFFERINFO", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_GETMOCOMPCOMPBUFFDATA", pack=False, align=None),
    "DD_GETMOCOMPFORMATSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwNumFormats", SimTypeInt(signed=False, label="UInt32")), ("lpFormats", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_GETMOCOMPFORMATSDATA", pack=False, align=None),
    "DD_GETMOCOMPGUIDSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("dwNumGuids", SimTypeInt(signed=False, label="UInt32")), ("lpGuids", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_GETMOCOMPGUIDSDATA", pack=False, align=None),
    "DD_GETSCANLINEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("dwScanLine", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetScanLine", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETSCANLINEDATA", pack=False, align=None),
    "DD_GETVPORTBANDWIDTHDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("lpddpfFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpBandwidth", SimTypePointer(SimTypeRef("DDVIDEOPORTBANDWIDTH", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortBandwidth", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTBANDWIDTHDATA", pack=False, align=None),
    "DD_GETVPORTCONNECTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("dwPortId", SimTypeInt(signed=False, label="UInt32")), ("lpConnect", SimTypePointer(SimTypeRef("DDVIDEOPORTCONNECT", SimStruct), offset=0)), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortConnectInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTCONNECTDATA", pack=False, align=None),
    "DD_GETVPORTFIELDDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("bField", SimTypeInt(signed=True, label="Int32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortField", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTFIELDDATA", pack=False, align=None),
    "DD_GETVPORTFLIPSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("fpSurface", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortFlipStatus", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTFLIPSTATUSDATA", pack=False, align=None),
    "DD_GETVPORTINPUTFORMATDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpddpfFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwNumFormats", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortInputFormats", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTINPUTFORMATDATA", pack=False, align=None),
    "DD_GETVPORTLINEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwLine", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortLine", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTLINEDATA", pack=False, align=None),
    "DD_GETVPORTOUTPUTFORMATDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpddpfInputFormat", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("lpddpfOutputFormats", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("dwNumFormats", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoPortInputFormats", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTOUTPUTFORMATDATA", pack=False, align=None),
    "DD_GETVPORTSIGNALDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("GetVideoSignalStatus", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_GETVPORTSIGNALDATA", pack=False, align=None),
    "DD_HALINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("vmiData", SimTypeRef("VIDEOMEMORYINFO", SimStruct)), ("ddCaps", SimTypeRef("DDNTCORECAPS", SimStruct)), ("GetDriverInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETDRIVERINFODATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpD3DGlobalDriverData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpD3DHALCallbacks", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpD3DBufCallbacks", SimTypePointer(SimTypeRef("DD_D3DBUFCALLBACKS", SimStruct), offset=0)),)), name="DD_HALINFO", pack=False, align=None),
    "DD_HALINFO_V4": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("vmiData", SimTypeRef("VIDEOMEMORYINFO", SimStruct)), ("ddCaps", SimTypeRef("DDNTCORECAPS", SimStruct)), ("GetDriverInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETDRIVERINFODATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DD_HALINFO_V4", pack=False, align=None),
    "DD_KERNELCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("SyncSurfaceData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SYNCSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SyncVideoPortData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SYNCVIDEOPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_KERNELCALLBACKS", pack=False, align=None),
    "DD_LOCKDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("bHasRect", SimTypeInt(signed=False, label="UInt32")), ("rArea", SimTypeRef("RECTL", SimStruct)), ("lpSurfData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Lock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fpProcess", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DD_LOCKDATA", pack=False, align=None),
    "DD_MAPMEMORYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("bMap", SimTypeInt(signed=True, label="Int32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpProcess", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_MAPMEMORYDATA", pack=False, align=None),
    "DD_MISCELLANEOUS2CALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("AlphaBlt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateSurfaceEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATESURFACEEXDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetDriverState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETDRIVERSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyDDLocal", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYDDLOCALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_MISCELLANEOUS2CALLBACKS", pack=False, align=None),
    "DD_MISCELLANEOUSCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("GetAvailDriverMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETAVAILDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_MISCELLANEOUSCALLBACKS", pack=False, align=None),
    "DD_MORECAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBAlphaCaps", SimTypeInt(signed=False, label="UInt32")), ("dwFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSVBFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("dwVSBFilterCaps", SimTypeInt(signed=False, label="UInt32")), ("dwSSBFilterCaps", SimTypeInt(signed=False, label="UInt32")),)), name="DD_MORECAPS", pack=False, align=None),
    "DD_MORESURFACECAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ddsCapsMore", SimTypeRef("DDSCAPSEX", SimStruct)), ("ddsExtendedHeapRestrictions", SimTypePointer(SimTypeRef("NTExtendedHeapRestrictions", SimStruct), offset=0)),)), name="DD_MORESURFACECAPS", pack=False, align=None),
    "DD_MOTIONCOMPCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("GetMoCompGuids", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETMOCOMPGUIDSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetMoCompFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETMOCOMPFORMATSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATEMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetMoCompBuffInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETMOCOMPCOMPBUFFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetInternalMoCompInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETINTERNALMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("BeginMoCompFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_BEGINMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("EndMoCompFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_ENDMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("RenderMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_RENDERMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("QueryMoCompStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_QUERYMOCOMPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyMoComp", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_MOTIONCOMPCALLBACKS", pack=False, align=None),
    "DD_MOTIONCOMP_LOCAL": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("guid", SimTypeBottom(label="Guid")), ("dwUncompWidth", SimTypeInt(signed=False, label="UInt32")), ("dwUncompHeight", SimTypeInt(signed=False, label="UInt32")), ("ddUncompPixelFormat", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwDriverReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved3", SimTypeInt(signed=False, label="UInt32")), ("lpDriverReserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDriverReserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpDriverReserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_MOTIONCOMP_LOCAL", pack=False, align=None),
    "DD_MULTISAMPLEQUALITYLEVELSDATA": SimStruct(OrderedDict((("gdi2", SimTypeRef("DD_GETDRIVERINFO2DATA", SimStruct)), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("_bitfield", SimTypeInt(signed=True, label="Int32")), ("QualityLevels", SimTypeInt(signed=False, label="UInt32")),)), name="DD_MULTISAMPLEQUALITYLEVELSDATA", pack=False, align=None),
    "DD_NONLOCALVIDMEMCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCaps2", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBCKeyCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBFXCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNLVBRops", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="DD_NONLOCALVIDMEMCAPS", pack=False, align=None),
    "DD_NTCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("FreeDriverMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FREEDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetExclusiveMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETEXCLUSIVEMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("FlipToGDISurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FLIPTOGDISURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_NTCALLBACKS", pack=False, align=None),
    "DD_NTPRIVATEDRIVERCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwPrivateCaps", SimTypeInt(signed=False, label="UInt32")),)), name="DD_NTPRIVATEDRIVERCAPS", pack=False, align=None),
    "DD_PALETTECALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DestroyPalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetEntries", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETENTRIESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_PALETTECALLBACKS", pack=False, align=None),
    "DD_PALETTE_GLOBAL": SimStruct(OrderedDict((("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DD_PALETTE_GLOBAL", pack=False, align=None),
    "DD_PALETTE_LOCAL": SimStruct(OrderedDict((("dwReserved0", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DD_PALETTE_LOCAL", pack=False, align=None),
    "DD_QUERYMOCOMPSTATUSDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DD_MOTIONCOMP_LOCAL", SimStruct), offset=0)), ("lpSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_QUERYMOCOMPSTATUSDATA", pack=False, align=None),
    "DD_RENDERMOCOMPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpMoComp", SimTypePointer(SimTypeRef("DD_MOTIONCOMP_LOCAL", SimStruct), offset=0)), ("dwNumBuffers", SimTypeInt(signed=False, label="UInt32")), ("lpBufferInfo", SimTypePointer(SimTypeRef("DDMOCOMPBUFFERINFO", SimStruct), offset=0)), ("dwFunction", SimTypeInt(signed=False, label="UInt32")), ("lpInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwOutputDataSize", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_RENDERMOCOMPDATA", pack=False, align=None),
    "DD_SETCLIPLISTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetClipList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_SETCLIPLISTDATA", pack=False, align=None),
    "DD_SETCOLORKEYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ckNew", SimTypeRef("DDCOLORKEY", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetColorKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_SETCOLORKEYDATA", pack=False, align=None),
    "DD_SETENTRIESDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DD_PALETTE_GLOBAL", SimStruct), offset=0)), ("dwBase", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("lpEntries", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetEntries", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_SETENTRIESDATA", pack=False, align=None),
    "DD_SETEXCLUSIVEMODEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("dwEnterExcl", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetExclusiveMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_SETEXCLUSIVEMODEDATA", pack=False, align=None),
    "DD_SETOVERLAYPOSITIONDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSrcSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpDDDestSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lXPos", SimTypeInt(signed=True, label="Int32")), ("lYPos", SimTypeInt(signed=True, label="Int32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetOverlayPosition", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_SETOVERLAYPOSITIONDATA", pack=False, align=None),
    "DD_SETPALETTEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("lpDDPalette", SimTypePointer(SimTypeRef("DD_PALETTE_GLOBAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("SetPalette", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attach", SimTypeInt(signed=True, label="Int32")),)), name="DD_SETPALETTEDATA", pack=False, align=None),
    "DD_STEREOMODE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwBpp", SimTypeInt(signed=False, label="UInt32")), ("dwRefreshRate", SimTypeInt(signed=False, label="UInt32")), ("bSupported", SimTypeInt(signed=True, label="Int32")),)), name="DD_STEREOMODE", pack=False, align=None),
    "DD_SURFACECALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DestroySurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Flip", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FLIPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetClipList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETCLIPLISTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Lock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Unlock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("Blt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetColorKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("AddAttachedSurface", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_ADDATTACHEDSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetBltStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETBLTSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetFlipStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UpdateOverlay", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UPDATEOVERLAYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("SetOverlayPosition", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETOVERLAYPOSITIONDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("reserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SetPalette", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_SURFACECALLBACKS", pack=False, align=None),
    "DD_SURFACE_GLOBAL": SimStruct(OrderedDict((("Anonymous1", SimUnion({"dwBlockSizeY": SimTypeInt(signed=False, label="UInt32"), "lSlicePitch": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"lpVidMemHeap": SimTypePointer(SimTypeRef("VIDEOMEMORY", SimStruct), offset=0), "dwBlockSizeX": SimTypeInt(signed=False, label="UInt32"), "dwUserMemSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("fpVidMem", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous3", SimUnion({"lPitch": SimTypeInt(signed=True, label="Int32"), "dwLinearSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("yHint", SimTypeInt(signed=True, label="Int32")), ("xHint", SimTypeInt(signed=True, label="Int32")), ("wHeight", SimTypeInt(signed=False, label="UInt32")), ("wWidth", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddpfSurface", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("fpHeapOffset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hCreatorProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DD_SURFACE_GLOBAL", pack=False, align=None),
    "DD_SURFACE_INT": SimStruct(OrderedDict((("lpLcl", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)),)), name="DD_SURFACE_INT", pack=False, align=None),
    "DD_SURFACE_LOCAL": SimStruct(OrderedDict((("lpGbl", SimTypePointer(SimTypeRef("DD_SURFACE_GLOBAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous1", SimUnion({"ddckCKSrcOverlay": SimTypeRef("DDCOLORKEY", SimStruct), "ddckCKSrcBlt": SimTypeRef("DDCOLORKEY", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"ddckCKDestOverlay": SimTypeRef("DDCOLORKEY", SimStruct), "ddckCKDestBlt": SimTypeRef("DDCOLORKEY", SimStruct)}, name="<anon>", label="None")), ("lpSurfMore", SimTypePointer(SimTypeRef("DD_SURFACE_MORE", SimStruct), offset=0)), ("lpAttachList", SimTypePointer(SimTypeRef("DD_ATTACHLIST", SimStruct), offset=0)), ("lpAttachListFrom", SimTypePointer(SimTypeRef("DD_ATTACHLIST", SimStruct), offset=0)), ("rcOverlaySrc", SimTypeRef("RECT", SimStruct)),)), name="DD_SURFACE_LOCAL", pack=False, align=None),
    "DD_SURFACE_MORE": SimStruct(OrderedDict((("dwMipMapCount", SimTypeInt(signed=False, label="UInt32")), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwOverlayFlags", SimTypeInt(signed=False, label="UInt32")), ("ddsCapsEx", SimTypeRef("DDSCAPSEX", SimStruct)), ("dwSurfaceHandle", SimTypeInt(signed=False, label="UInt32")),)), name="DD_SURFACE_MORE", pack=False, align=None),
    "DD_SYNCSURFACEDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("dwSurfaceOffset", SimTypeInt(signed=False, label="UInt32")), ("fpLockPtr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lPitch", SimTypeInt(signed=True, label="Int32")), ("dwOverlayOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved4", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_SYNCSURFACEDATA", pack=False, align=None),
    "DD_SYNCVIDEOPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwOriginOffset", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVBIHeight", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwDriverReserved3", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")),)), name="DD_SYNCVIDEOPORTDATA", pack=False, align=None),
    "DD_UNLOCKDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("Unlock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_UNLOCKDATA", pack=False, align=None),
    "DD_UPDATENONLOCALHEAPDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("dwHeap", SimTypeInt(signed=False, label="UInt32")), ("fpGARTLin", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fpGARTDev", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ulPolicyMaxBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateNonLocalHeap", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_UPDATENONLOCALHEAPDATA", pack=False, align=None),
    "DD_UPDATEOVERLAYDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("lpDDDestSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("rDest", SimTypeRef("RECTL", SimStruct)), ("lpDDSrcSurface", SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)), ("rSrc", SimTypeRef("RECTL", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("overlayFX", SimTypeRef("DDOVERLAYFX", SimStruct)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateOverlay", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_UPDATEOVERLAYDATA", pack=False, align=None),
    "DD_UPDATEVPORTDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("lplpDDSurface", SimTypePointer(SimTypePointer(SimTypeRef("DD_SURFACE_INT", SimStruct), offset=0), offset=0)), ("lplpDDVBISurface", SimTypePointer(SimTypePointer(SimTypeRef("DD_SURFACE_INT", SimStruct), offset=0), offset=0)), ("lpVideoInfo", SimTypePointer(SimTypeRef("DDVIDEOPORTINFO", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumAutoflip", SimTypeInt(signed=False, label="UInt32")), ("dwNumVBIAutoflip", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateVideoPort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_UPDATEVPORTDATA", pack=False, align=None),
    "DD_VIDEOPORTCALLBACKS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("CanCreateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CANCREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("CreateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("FlipVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FLIPVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortBandwidth", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTBANDWIDTHDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortInputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTINPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortOutputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTOUTPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("lpReserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("GetVideoPortField", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTFIELDDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortLine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortConnectInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTCONNECTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("DestroyVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoPortFlipStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("UpdateVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UPDATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("WaitForVideoPortSync", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_WAITFORVPORTSYNCDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("GetVideoSignalStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTSIGNALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)), ("ColorControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_VPORTCOLORDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="DD_VIDEOPORTCALLBACKS", pack=False, align=None),
    "DD_VIDEOPORT_LOCAL": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("ddvpDesc", SimTypeRef("DDVIDEOPORTDESC", SimStruct)), ("ddvpInfo", SimTypeRef("DDVIDEOPORTINFO", SimStruct)), ("lpSurface", SimTypePointer(SimTypeRef("DD_SURFACE_INT", SimStruct), offset=0)), ("lpVBISurface", SimTypePointer(SimTypeRef("DD_SURFACE_INT", SimStruct), offset=0)), ("dwNumAutoflip", SimTypeInt(signed=False, label="UInt32")), ("dwNumVBIAutoflip", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DD_VIDEOPORT_LOCAL", pack=False, align=None),
    "DD_VPORTCOLORDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpColorData", SimTypePointer(SimTypeRef("DDCOLORCONTROL", SimStruct), offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("ColorControl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_VPORTCOLORDATA", pack=False, align=None),
    "DD_WAITFORVERTICALBLANKDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bIsInVB", SimTypeInt(signed=False, label="UInt32")), ("hEvent", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("WaitForVerticalBlank", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_WAITFORVERTICALBLANKDATA", pack=False, align=None),
    "DD_WAITFORVPORTSYNCDATA": SimStruct(OrderedDict((("lpDD", SimTypePointer(SimTypeRef("DD_DIRECTDRAW_LOCAL", SimStruct), offset=0)), ("lpVideoPort", SimTypePointer(SimTypeRef("DD_VIDEOPORT_LOCAL", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLine", SimTypeInt(signed=False, label="UInt32")), ("dwTimeOut", SimTypeInt(signed=False, label="UInt32")), ("ddRVal", SimTypeInt(signed=True, label="Int32")), ("UpdateVideoPort", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DD_WAITFORVPORTSYNCDATA", pack=False, align=None),
    "DDiscFormat2DataEvents": SimTypeBottom(label="DDiscFormat2DataEvents"),
    "DDiscFormat2EraseEvents": SimTypeBottom(label="DDiscFormat2EraseEvents"),
    "DDiscFormat2RawCDEvents": SimTypeBottom(label="DDiscFormat2RawCDEvents"),
    "DDiscFormat2TrackAtOnceEvents": SimTypeBottom(label="DDiscFormat2TrackAtOnceEvents"),
    "DDiscMaster2Events": SimTypeBottom(label="DDiscMaster2Events"),
    "DEADKEY": SimStruct(OrderedDict((("dwBoth", SimTypeInt(signed=False, label="UInt32")), ("wchComposed", SimTypeChar(label="Char")), ("uFlags", SimTypeShort(signed=False, label="UInt16")),)), name="DEADKEY", pack=False, align=None),
    "DEBUGEVENTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0),
    "DEBUGGING_DEVICE_IN_USE": SimStruct(OrderedDict((("NameSpace", SimTypeInt(signed=False, label="KD_NAMESPACE_ENUM")), ("StructureLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"AcpiDevice": SimTypeRef("ACPI_DEBUGGING_DEVICE_IN_USE", SimStruct), "PciDevice": SimTypeRef("PCI_DEBUGGING_DEVICE_IN_USE", SimStruct)}, name="<anon>", label="None")),)), name="DEBUGGING_DEVICE_IN_USE", pack=False, align=None),
    "DEBUGGING_DEVICE_IN_USE_INFORMATION": SimStruct(OrderedDict((("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypePointer(SimTypeRef("DEBUGGING_DEVICE_IN_USE", SimStruct), offset=0)),)), name="DEBUGGING_DEVICE_IN_USE_INFORMATION", pack=False, align=None),
    "DEBUGHOOKINFO": SimStruct(OrderedDict((("idThread", SimTypeInt(signed=False, label="UInt32")), ("idThreadInstaller", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("code", SimTypeInt(signed=True, label="Int32")),)), name="DEBUGHOOKINFO", pack=False, align=None),
    "DEBUG_ANALYSIS_PROCESSOR_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Model", SimTypeInt(signed=False, label="UInt32")), ("Family", SimTypeInt(signed=False, label="UInt32")), ("Stepping", SimTypeInt(signed=False, label="UInt32")), ("Architecture", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("CurrentClockSpeed", SimTypeInt(signed=False, label="UInt32")), ("CurrentVoltage", SimTypeInt(signed=False, label="UInt32")), ("MaxClockSpeed", SimTypeInt(signed=False, label="UInt32")), ("ProcessorType", SimTypeInt(signed=False, label="UInt32")), ("DeviceID", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("Manufacturer", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Name", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Version", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Description", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="DEBUG_ANALYSIS_PROCESSOR_INFO", pack=False, align=None),
    "DEBUG_BIT_FIELD": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="DEBUG_BIT_FIELD", pack=False, align=None),
    "DEBUG_BREAKPOINT_PARAMETERS": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Id", SimTypeInt(signed=False, label="UInt32")), ("BreakType", SimTypeInt(signed=False, label="UInt32")), ("ProcType", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("DataAccessType", SimTypeInt(signed=False, label="UInt32")), ("PassCount", SimTypeInt(signed=False, label="UInt32")), ("CurrentPassCount", SimTypeInt(signed=False, label="UInt32")), ("MatchThread", SimTypeInt(signed=False, label="UInt32")), ("CommandSize", SimTypeInt(signed=False, label="UInt32")), ("OffsetExpressionSize", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_BREAKPOINT_PARAMETERS", pack=False, align=None),
    "DEBUG_CACHED_SYMBOL_INFO": SimStruct(OrderedDict((("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("Arg1", SimTypeLongLong(signed=False, label="UInt64")), ("Arg2", SimTypeLongLong(signed=False, label="UInt64")), ("Id", SimTypeInt(signed=False, label="UInt32")), ("Arg3", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_CACHED_SYMBOL_INFO", pack=False, align=None),
    "DEBUG_CLIENT_CONTEXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("eClient", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_CLIENT_CONTEXT", pack=False, align=None),
    "DEBUG_CPU_MICROCODE_VERSION": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("CachedSignature", SimTypeLongLong(signed=True, label="Int64")), ("InitialSignature", SimTypeLongLong(signed=True, label="Int64")), ("ProcessorModel", SimTypeInt(signed=False, label="UInt32")), ("ProcessorFamily", SimTypeInt(signed=False, label="UInt32")), ("ProcessorStepping", SimTypeInt(signed=False, label="UInt32")), ("ProcessorArchRev", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_CPU_MICROCODE_VERSION", pack=False, align=None),
    "DEBUG_CPU_SPEED_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("CurrentSpeed", SimTypeInt(signed=False, label="UInt32")), ("RatedSpeed", SimTypeInt(signed=False, label="UInt32")), ("NameString", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="DEBUG_CPU_SPEED_INFO", pack=False, align=None),
    "DEBUG_CREATE_PROCESS_OPTIONS": SimStruct(OrderedDict((("CreateFlags", SimTypeInt(signed=False, label="UInt32")), ("EngCreateFlags", SimTypeInt(signed=False, label="UInt32")), ("VerifierFlags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_CREATE_PROCESS_OPTIONS", pack=False, align=None),
    "DEBUG_DECODE_ERROR": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Code", SimTypeInt(signed=False, label="UInt32")), ("TreatAsStatus", SimTypeInt(signed=True, label="Int32")), ("Source", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Message", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="DEBUG_DECODE_ERROR", pack=False, align=None),
    "DEBUG_DEVICE_ADDRESS": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Valid", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Reserved": SimTypeArray(SimTypeChar(label="Byte"), 2), "Anonymous": SimStruct(OrderedDict((("BitWidth", SimTypeChar(label="Byte")), ("AccessSize", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("TranslatedAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_DEVICE_ADDRESS", pack=False, align=None),
    "DEBUG_DEVICE_DESCRIPTOR": SimStruct(OrderedDict((("Bus", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeInt(signed=False, label="UInt32")), ("Segment", SimTypeShort(signed=False, label="UInt16")), ("VendorID", SimTypeShort(signed=False, label="UInt16")), ("DeviceID", SimTypeShort(signed=False, label="UInt16")), ("BaseClass", SimTypeChar(label="Byte")), ("SubClass", SimTypeChar(label="Byte")), ("ProgIf", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Flags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Initialized", SimTypeChar(label="Byte")), ("Configured", SimTypeChar(label="Byte")), ("BaseAddress", SimTypeArray(SimTypeRef("DEBUG_DEVICE_ADDRESS", SimStruct), 6)), ("Memory", SimTypeRef("DEBUG_MEMORY_REQUIREMENTS", SimStruct)), ("Dbg2TableIndex", SimTypeInt(signed=False, label="UInt32")), ("PortType", SimTypeShort(signed=False, label="UInt16")), ("PortSubtype", SimTypeShort(signed=False, label="UInt16")), ("OemData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OemDataLength", SimTypeInt(signed=False, label="UInt32")), ("NameSpace", SimTypeInt(signed=False, label="KD_NAMESPACE_ENUM")), ("NameSpacePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NameSpacePathLength", SimTypeInt(signed=False, label="UInt32")), ("TransportType", SimTypeInt(signed=False, label="UInt32")), ("TransportData", SimTypeRef("DEBUG_TRANSPORT_DATA", SimStruct)), ("EfiIoMmuData", SimTypeRef("DEBUG_EFI_IOMMU_DATA", SimStruct)),)), name="DEBUG_DEVICE_DESCRIPTOR", pack=False, align=None),
    "DEBUG_DEVICE_OBJECT_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DevObjAddress", SimTypeLongLong(signed=False, label="UInt64")), ("ReferenceCount", SimTypeInt(signed=False, label="UInt32")), ("QBusy", SimTypeInt(signed=True, label="Int32")), ("DriverObject", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentIrp", SimTypeLongLong(signed=False, label="UInt64")), ("DevExtension", SimTypeLongLong(signed=False, label="UInt64")), ("DevObjExtension", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_DEVICE_OBJECT_INFO", pack=False, align=None),
    "DEBUG_DRIVER_OBJECT_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DriverSize", SimTypeInt(signed=False, label="UInt32")), ("DriverObjAddress", SimTypeLongLong(signed=False, label="UInt64")), ("DriverStart", SimTypeLongLong(signed=False, label="UInt64")), ("DriverExtension", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceObject", SimTypeLongLong(signed=False, label="UInt64")), ("DriverName", SimTypeRef("_DriverName_e__Struct", SimStruct)),)), name="DEBUG_DRIVER_OBJECT_INFO", pack=False, align=None),
    "DEBUG_EFI_IOMMU_DATA": SimStruct(OrderedDict((("PciIoProtocolHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Mapping", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEBUG_EFI_IOMMU_DATA", pack=False, align=None),
    "DEBUG_EVENT": SimStruct(OrderedDict((("dwDebugEventCode", SimTypeInt(signed=False, label="DEBUG_EVENT_CODE")), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwThreadId", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"Exception": SimTypeRef("EXCEPTION_DEBUG_INFO", SimStruct), "CreateThread": SimTypeRef("CREATE_THREAD_DEBUG_INFO", SimStruct), "CreateProcessInfo": SimTypeRef("CREATE_PROCESS_DEBUG_INFO", SimStruct), "ExitThread": SimTypeRef("EXIT_THREAD_DEBUG_INFO", SimStruct), "ExitProcess": SimTypeRef("EXIT_PROCESS_DEBUG_INFO", SimStruct), "LoadDll": SimTypeRef("LOAD_DLL_DEBUG_INFO", SimStruct), "UnloadDll": SimTypeRef("UNLOAD_DLL_DEBUG_INFO", SimStruct), "DebugString": SimTypeRef("OUTPUT_DEBUG_STRING_INFO", SimStruct), "RipInfo": SimTypeRef("RIP_INFO", SimStruct)}, name="<anon>", label="None")),)), name="DEBUG_EVENT", pack=False, align=None),
    "DEBUG_EVENT_CODE": SimTypeInt(signed=False, label="DEBUG_EVENT_CODE"),
    "DEBUG_EVENT_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ProcessEngineId", SimTypeInt(signed=False, label="UInt32")), ("ThreadEngineId", SimTypeInt(signed=False, label="UInt32")), ("FrameEngineId", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_EVENT_CONTEXT", pack=False, align=None),
    "DEBUG_EVENT_INFO_TYPE": SimTypeInt(signed=False, label="DEBUG_EVENT_INFO_TYPE"),
    "DEBUG_EXCEPTION_FILTER_PARAMETERS": SimStruct(OrderedDict((("ExecutionOption", SimTypeInt(signed=False, label="UInt32")), ("ContinueOption", SimTypeInt(signed=False, label="UInt32")), ("TextSize", SimTypeInt(signed=False, label="UInt32")), ("CommandSize", SimTypeInt(signed=False, label="UInt32")), ("SecondCommandSize", SimTypeInt(signed=False, label="UInt32")), ("ExceptionCode", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_EXCEPTION_FILTER_PARAMETERS", pack=False, align=None),
    "DEBUG_FAILURE_TYPE": SimTypeInt(signed=False, label="DEBUG_FAILURE_TYPE"),
    "DEBUG_FLR_PARAM_TYPE": SimTypeInt(signed=False, label="DEBUG_FLR_PARAM_TYPE"),
    "DEBUG_GET_TEXT_COMPLETIONS_IN": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("MatchCountLimit", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 3)),)), name="DEBUG_GET_TEXT_COMPLETIONS_IN", pack=False, align=None),
    "DEBUG_GET_TEXT_COMPLETIONS_OUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ReplaceIndex", SimTypeInt(signed=False, label="UInt32")), ("MatchCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="DEBUG_GET_TEXT_COMPLETIONS_OUT", pack=False, align=None),
    "DEBUG_HANDLE_DATA_BASIC": SimStruct(OrderedDict((("TypeNameSize", SimTypeInt(signed=False, label="UInt32")), ("ObjectNameSize", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PointerCount", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_HANDLE_DATA_BASIC", pack=False, align=None),
    "DEBUG_IRP_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("IrpAddress", SimTypeLongLong(signed=False, label="UInt64")), ("IoStatus", SimTypeInt(signed=False, label="UInt32")), ("StackCount", SimTypeInt(signed=False, label="UInt32")), ("CurrentLocation", SimTypeInt(signed=False, label="UInt32")), ("MdlAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Thread", SimTypeLongLong(signed=False, label="UInt64")), ("CancelRoutine", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentStack", SimTypeRef("DEBUG_IRP_STACK_INFO", SimStruct)), ("Stack", SimTypeArray(SimTypeRef("DEBUG_IRP_STACK_INFO", SimStruct), 10)),)), name="DEBUG_IRP_INFO", pack=False, align=None),
    "DEBUG_IRP_STACK_INFO": SimStruct(OrderedDict((("Major", SimTypeChar(label="Byte")), ("Minor", SimTypeChar(label="Byte")), ("DeviceObject", SimTypeLongLong(signed=False, label="UInt64")), ("FileObject", SimTypeLongLong(signed=False, label="UInt64")), ("CompletionRoutine", SimTypeLongLong(signed=False, label="UInt64")), ("StackAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_IRP_STACK_INFO", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_BREAKPOINT": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_LAST_EVENT_INFO_BREAKPOINT", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_EXCEPTION": SimStruct(OrderedDict((("ExceptionRecord", SimTypeRef("EXCEPTION_RECORD64", SimStruct)), ("FirstChance", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_LAST_EVENT_INFO_EXCEPTION", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_EXIT_PROCESS": SimStruct(OrderedDict((("ExitCode", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_LAST_EVENT_INFO_EXIT_PROCESS", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_EXIT_THREAD": SimStruct(OrderedDict((("ExitCode", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_LAST_EVENT_INFO_EXIT_THREAD", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_LOAD_MODULE": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_LAST_EVENT_INFO_LOAD_MODULE", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_SERVICE_EXCEPTION": SimStruct(OrderedDict((("Kind", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_LAST_EVENT_INFO_SERVICE_EXCEPTION", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_SYSTEM_ERROR": SimStruct(OrderedDict((("Error", SimTypeInt(signed=False, label="UInt32")), ("Level", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_LAST_EVENT_INFO_SYSTEM_ERROR", pack=False, align=None),
    "DEBUG_LAST_EVENT_INFO_UNLOAD_MODULE": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_LAST_EVENT_INFO_UNLOAD_MODULE", pack=False, align=None),
    "DEBUG_MEMORY_REQUIREMENTS": SimStruct(OrderedDict((("Start", SimTypeLongLong(signed=True, label="Int64")), ("MaxEnd", SimTypeLongLong(signed=True, label="Int64")), ("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Cached", SimTypeChar(label="Byte")), ("Aligned", SimTypeChar(label="Byte")),)), name="DEBUG_MEMORY_REQUIREMENTS", pack=False, align=None),
    "DEBUG_MODULE_AND_ID": SimStruct(OrderedDict((("ModuleBase", SimTypeLongLong(signed=False, label="UInt64")), ("Id", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_MODULE_AND_ID", pack=False, align=None),
    "DEBUG_MODULE_PARAMETERS": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("Checksum", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SymbolType", SimTypeInt(signed=False, label="UInt32")), ("ImageNameSize", SimTypeInt(signed=False, label="UInt32")), ("ModuleNameSize", SimTypeInt(signed=False, label="UInt32")), ("LoadedImageNameSize", SimTypeInt(signed=False, label="UInt32")), ("SymbolFileNameSize", SimTypeInt(signed=False, label="UInt32")), ("MappedImageNameSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="DEBUG_MODULE_PARAMETERS", pack=False, align=None),
    "DEBUG_OFFSET_REGION": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_OFFSET_REGION", pack=False, align=None),
    "DEBUG_PNP_TRIAGE_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Lock_Address", SimTypeLongLong(signed=False, label="UInt64")), ("Lock_ActiveCount", SimTypeInt(signed=True, label="Int32")), ("Lock_ContentionCount", SimTypeInt(signed=False, label="UInt32")), ("Lock_NumberOfExclusiveWaiters", SimTypeInt(signed=False, label="UInt32")), ("Lock_NumberOfSharedWaiters", SimTypeInt(signed=False, label="UInt32")), ("Lock_Flag", SimTypeShort(signed=False, label="UInt16")), ("TriagedThread", SimTypeLongLong(signed=False, label="UInt64")), ("ThreadCount", SimTypeInt(signed=True, label="Int32")), ("TriagedThread_WaitTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_PNP_TRIAGE_INFO", pack=False, align=None),
    "DEBUG_POOLTAG_DESCRIPTION": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("PoolTag", SimTypeInt(signed=False, label="UInt32")), ("Description", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Binary", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("Owner", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="DEBUG_POOLTAG_DESCRIPTION", pack=False, align=None),
    "DEBUG_POOL_DATA": SimStruct(OrderedDict((("SizeofStruct", SimTypeInt(signed=False, label="UInt32")), ("PoolBlock", SimTypeLongLong(signed=False, label="UInt64")), ("Pool", SimTypeLongLong(signed=False, label="UInt64")), ("PreviousSize", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PoolTag", SimTypeInt(signed=False, label="UInt32")), ("ProcessBilled", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved2", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("PoolTagDescription", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="DEBUG_POOL_DATA", pack=False, align=None),
    "DEBUG_POOL_REGION": SimTypeInt(signed=False, label="DEBUG_POOL_REGION"),
    "DEBUG_PROCESSOR_IDENTIFICATION_ALL": SimUnion({"Alpha": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ALPHA", SimStruct), "Amd64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_AMD64", SimStruct), "Ia64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_IA64", SimStruct), "X86": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_X86", SimStruct), "Arm": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM", SimStruct), "Arm64": SimTypeRef("DEBUG_PROCESSOR_IDENTIFICATION_ARM64", SimStruct)}, name="<anon>", label="None"),
    "DEBUG_PROCESSOR_IDENTIFICATION_ALPHA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_PROCESSOR_IDENTIFICATION_ALPHA", pack=False, align=None),
    "DEBUG_PROCESSOR_IDENTIFICATION_AMD64": SimStruct(OrderedDict((("Family", SimTypeInt(signed=False, label="UInt32")), ("Model", SimTypeInt(signed=False, label="UInt32")), ("Stepping", SimTypeInt(signed=False, label="UInt32")), ("VendorString", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="DEBUG_PROCESSOR_IDENTIFICATION_AMD64", pack=False, align=None),
    "DEBUG_PROCESSOR_IDENTIFICATION_ARM": SimStruct(OrderedDict((("Model", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("VendorString", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="DEBUG_PROCESSOR_IDENTIFICATION_ARM", pack=False, align=None),
    "DEBUG_PROCESSOR_IDENTIFICATION_ARM64": SimStruct(OrderedDict((("Model", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("VendorString", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="DEBUG_PROCESSOR_IDENTIFICATION_ARM64", pack=False, align=None),
    "DEBUG_PROCESSOR_IDENTIFICATION_IA64": SimStruct(OrderedDict((("Model", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("Family", SimTypeInt(signed=False, label="UInt32")), ("ArchRev", SimTypeInt(signed=False, label="UInt32")), ("VendorString", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="DEBUG_PROCESSOR_IDENTIFICATION_IA64", pack=False, align=None),
    "DEBUG_PROCESSOR_IDENTIFICATION_X86": SimStruct(OrderedDict((("Family", SimTypeInt(signed=False, label="UInt32")), ("Model", SimTypeInt(signed=False, label="UInt32")), ("Stepping", SimTypeInt(signed=False, label="UInt32")), ("VendorString", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="DEBUG_PROCESSOR_IDENTIFICATION_X86", pack=False, align=None),
    "DEBUG_READ_USER_MINIDUMP_STREAM": SimStruct(OrderedDict((("StreamType", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("BufferUsed", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_READ_USER_MINIDUMP_STREAM", pack=False, align=None),
    "DEBUG_REGISTER_DESCRIPTION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SubregMaster", SimTypeInt(signed=False, label="UInt32")), ("SubregLength", SimTypeInt(signed=False, label="UInt32")), ("SubregMask", SimTypeLongLong(signed=False, label="UInt64")), ("SubregShift", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_REGISTER_DESCRIPTION", pack=False, align=None),
    "DEBUG_SMBIOS_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("SmbiosMajorVersion", SimTypeChar(label="Byte")), ("SmbiosMinorVersion", SimTypeChar(label="Byte")), ("DMIVersion", SimTypeChar(label="Byte")), ("TableSize", SimTypeInt(signed=False, label="UInt32")), ("BiosMajorRelease", SimTypeChar(label="Byte")), ("BiosMinorRelease", SimTypeChar(label="Byte")), ("FirmwareMajorRelease", SimTypeChar(label="Byte")), ("FirmwareMinorRelease", SimTypeChar(label="Byte")), ("BaseBoardManufacturer", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BaseBoardProduct", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BaseBoardVersion", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BiosReleaseDate", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BiosVendor", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BiosVersion", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("SystemFamily", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("SystemManufacturer", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("SystemProductName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("SystemSKU", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("SystemVersion", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="DEBUG_SMBIOS_INFO", pack=False, align=None),
    "DEBUG_SPECIFIC_FILTER_PARAMETERS": SimStruct(OrderedDict((("ExecutionOption", SimTypeInt(signed=False, label="UInt32")), ("ContinueOption", SimTypeInt(signed=False, label="UInt32")), ("TextSize", SimTypeInt(signed=False, label="UInt32")), ("CommandSize", SimTypeInt(signed=False, label="UInt32")), ("ArgumentSize", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_SPECIFIC_FILTER_PARAMETERS", pack=False, align=None),
    "DEBUG_STACKFRAME_TYPE": SimTypeInt(signed=False, label="DEBUG_STACKFRAME_TYPE"),
    "DEBUG_STACK_FRAME": SimStruct(OrderedDict((("InstructionOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ReturnOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FrameOffset", SimTypeLongLong(signed=False, label="UInt64")), ("StackOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FuncTableEntry", SimTypeLongLong(signed=False, label="UInt64")), ("Params", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 6)), ("Virtual", SimTypeInt(signed=True, label="Int32")), ("FrameNumber", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_STACK_FRAME", pack=False, align=None),
    "DEBUG_STACK_FRAME_EX": SimStruct(OrderedDict((("InstructionOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ReturnOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FrameOffset", SimTypeLongLong(signed=False, label="UInt64")), ("StackOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FuncTableEntry", SimTypeLongLong(signed=False, label="UInt64")), ("Params", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 6)), ("Virtual", SimTypeInt(signed=True, label="Int32")), ("FrameNumber", SimTypeInt(signed=False, label="UInt32")), ("InlineFrameContext", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_STACK_FRAME_EX", pack=False, align=None),
    "DEBUG_SYMBOL_ENTRY": SimStruct(OrderedDict((("ModuleBase", SimTypeLongLong(signed=False, label="UInt64")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Id", SimTypeLongLong(signed=False, label="UInt64")), ("Arg64", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TypeId", SimTypeInt(signed=False, label="UInt32")), ("NameSize", SimTypeInt(signed=False, label="UInt32")), ("Token", SimTypeInt(signed=False, label="UInt32")), ("Tag", SimTypeInt(signed=False, label="UInt32")), ("Arg32", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_SYMBOL_ENTRY", pack=False, align=None),
    "DEBUG_SYMBOL_PARAMETERS": SimStruct(OrderedDict((("Module", SimTypeLongLong(signed=False, label="UInt64")), ("TypeId", SimTypeInt(signed=False, label="UInt32")), ("ParentSymbol", SimTypeInt(signed=False, label="UInt32")), ("SubElements", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_SYMBOL_PARAMETERS", pack=False, align=None),
    "DEBUG_SYMBOL_SOURCE_ENTRY": SimStruct(OrderedDict((("ModuleBase", SimTypeLongLong(signed=False, label="UInt64")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("FileNameId", SimTypeLongLong(signed=False, label="UInt64")), ("EngineInternal", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileNameSize", SimTypeInt(signed=False, label="UInt32")), ("StartLine", SimTypeInt(signed=False, label="UInt32")), ("EndLine", SimTypeInt(signed=False, label="UInt32")), ("StartColumn", SimTypeInt(signed=False, label="UInt32")), ("EndColumn", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_SYMBOL_SOURCE_ENTRY", pack=False, align=None),
    "DEBUG_THREAD_BASIC_INFORMATION": SimStruct(OrderedDict((("Valid", SimTypeInt(signed=False, label="UInt32")), ("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("CreateTime", SimTypeLongLong(signed=False, label="UInt64")), ("ExitTime", SimTypeLongLong(signed=False, label="UInt64")), ("KernelTime", SimTypeLongLong(signed=False, label="UInt64")), ("UserTime", SimTypeLongLong(signed=False, label="UInt64")), ("StartOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Affinity", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEBUG_THREAD_BASIC_INFORMATION", pack=False, align=None),
    "DEBUG_TRANSPORT_DATA": SimStruct(OrderedDict((("HwContextSize", SimTypeInt(signed=False, label="UInt32")), ("SharedVisibleDataSize", SimTypeInt(signed=False, label="UInt32")), ("UseSerialFraming", SimTypeChar(label="Byte")), ("ValidUSBCoreId", SimTypeChar(label="Byte")), ("USBCoreId", SimTypeChar(label="Byte")),)), name="DEBUG_TRANSPORT_DATA", pack=False, align=None),
    "DEBUG_TRIAGE_FOLLOWUP_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("OwnerNameSize", SimTypeInt(signed=False, label="UInt32")), ("OwnerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEBUG_TRIAGE_FOLLOWUP_INFO", pack=False, align=None),
    "DEBUG_TRIAGE_FOLLOWUP_INFO_2": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("OwnerNameSize", SimTypeInt(signed=False, label="UInt32")), ("OwnerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FeaturePathSize", SimTypeInt(signed=False, label="UInt32")), ("FeaturePath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEBUG_TRIAGE_FOLLOWUP_INFO_2", pack=False, align=None),
    "DEBUG_TYPED_DATA": SimStruct(OrderedDict((("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("EngineHandle", SimTypeLongLong(signed=False, label="UInt64")), ("Data", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TypeId", SimTypeInt(signed=False, label="UInt32")), ("BaseTypeId", SimTypeInt(signed=False, label="UInt32")), ("Tag", SimTypeInt(signed=False, label="UInt32")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("Internal", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 9)),)), name="DEBUG_TYPED_DATA", pack=False, align=None),
    "DEBUG_VALUE": SimStruct(OrderedDict((("Anonymous", SimUnion({"I8": SimTypeChar(label="Byte"), "I16": SimTypeShort(signed=False, label="UInt16"), "I32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("I64", SimTypeLongLong(signed=False, label="UInt64")), ("Nat", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "F32": SimTypeFloat(size=32), "F64": SimTypeFloat(size=64), "F80Bytes": SimTypeArray(SimTypeChar(label="Byte"), 10), "F82Bytes": SimTypeArray(SimTypeChar(label="Byte"), 11), "F128Bytes": SimTypeArray(SimTypeChar(label="Byte"), 16), "VI8": SimTypeArray(SimTypeChar(label="Byte"), 16), "VI16": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "VI32": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4), "VI64": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2), "VF32": SimTypeArray(SimTypeFloat(size=32), 4), "VF64": SimTypeArray(SimTypeFloat(size=64), 2), "I64Parts32": SimTypeRef("_I64Parts32_e__Struct", SimStruct), "F128Parts64": SimTypeRef("_F128Parts64_e__Struct", SimStruct), "RawBytes": SimTypeArray(SimTypeChar(label="Byte"), 24)}, name="<anon>", label="None")), ("TailOfRawBytes", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="DEBUG_VALUE", pack=False, align=None),
    "DECIMAL": SimStruct(OrderedDict((("wReserved", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("scale", SimTypeChar(label="Byte")), ("sign", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "signscale": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Hi32", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo32", SimTypeInt(signed=False, label="UInt32")), ("Mid32", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Lo64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="DECIMAL", pack=False, align=None),
    "DECIMATION_USAGE": SimTypeInt(signed=False, label="DECIMATION_USAGE"),
    "DECISION_LOCATION": SimTypeInt(signed=False, label="DECISION_LOCATION"),
    "DECODING_SOURCE": SimTypeInt(signed=False, label="DECODING_SOURCE"),
    "DECRYPTION_STATUS_BUFFER": SimStruct(OrderedDict((("NoEncryptedStreams", SimTypeChar(label="Byte")),)), name="DECRYPTION_STATUS_BUFFER", pack=False, align=None),
    "DECRYPT_MESSAGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "DEDUP_BACKUP_SUPPORT_PARAM_TYPE": SimTypeInt(signed=False, label="DEDUP_BACKUP_SUPPORT_PARAM_TYPE"),
    "DEDUP_CHUNK_INFO_HASH32": SimStruct(OrderedDict((("ChunkFlags", SimTypeInt(signed=False, label="UInt32")), ("ChunkOffsetInStream", SimTypeLongLong(signed=False, label="UInt64")), ("ChunkSize", SimTypeLongLong(signed=False, label="UInt64")), ("HashVal", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="DEDUP_CHUNK_INFO_HASH32", pack=False, align=None),
    "DEDUP_CONTAINER_EXTENT": SimStruct(OrderedDict((("ContainerIndex", SimTypeInt(signed=False, label="UInt32")), ("StartOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="DEDUP_CONTAINER_EXTENT", pack=False, align=None),
    "DEDUP_SET_PARAM_TYPE": SimTypeInt(signed=False, label="DEDUP_SET_PARAM_TYPE"),
    "DEFAULTSAVEFOLDERTYPE": SimTypeInt(signed=False, label="DEFAULTSAVEFOLDERTYPE"),
    "DEFAULT_FOLDER_MENU_RESTRICTIONS": SimTypeInt(signed=False, label="DEFAULT_FOLDER_MENU_RESTRICTIONS"),
    "DEFAULT_PAGES": SimTypeInt(signed=False, label="DEFAULT_PAGES"),
    "DEFCONTEXTMENU": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pcmcb", SimTypeBottom(label="IContextMenuCB")), ("pidlFolder", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("psf", SimTypeBottom(label="IShellFolder")), ("cidl", SimTypeInt(signed=False, label="UInt32")), ("apidl", SimTypePointer(SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0), offset=0)), ("punkAssociationInfo", SimTypeBottom(label="IUnknown")), ("cKeys", SimTypeInt(signed=False, label="UInt32")), ("aKeys", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="DEFCONTEXTMENU", pack=False, align=None),
    "DEFINE_DOS_DEVICE_FLAGS": SimTypeInt(signed=False, label="DEFINE_DOS_DEVICE_FLAGS"),
    "DEF_SHARE_ID": SimTypeInt(signed=False, label="DEF_SHARE_ID"),
    "DELAYLOAD_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("DelayloadDescriptor", SimTypePointer(SimTypeRef("IMAGE_DELAYLOAD_DESCRIPTOR", SimStruct), offset=0)), ("ThunkAddress", SimTypePointer(SimTypeRef("IMAGE_THUNK_DATA32", SimStruct), offset=0)), ("TargetDllName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TargetApiDescriptor", SimTypeRef("DELAYLOAD_PROC_DESCRIPTOR", SimStruct)), ("TargetModuleBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Unused", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LastError", SimTypeInt(signed=False, label="UInt32")),)), name="DELAYLOAD_INFO", pack=False, align=None),
    "DELAYLOAD_PROC_DESCRIPTOR": SimStruct(OrderedDict((("ImportDescribedByName", SimTypeInt(signed=False, label="UInt32")), ("Description", SimUnion({"Name": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "Ordinal": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DELAYLOAD_PROC_DESCRIPTOR", pack=False, align=None),
    "DELEGATEITEMID": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("wOuter", SimTypeShort(signed=False, label="UInt16")), ("cbInner", SimTypeShort(signed=False, label="UInt16")), ("rgb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DELEGATEITEMID", pack=False, align=None),
    "DELETEITEMSTRUCT": SimStruct(OrderedDict((("CtlType", SimTypeInt(signed=False, label="DRAWITEMSTRUCT_CTL_TYPE")), ("CtlID", SimTypeInt(signed=False, label="UInt32")), ("itemID", SimTypeInt(signed=False, label="UInt32")), ("hwndItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("itemData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DELETEITEMSTRUCT", pack=False, align=None),
    "DELETE_OBJECT_OPTIONS": SimTypeInt(signed=False, label="DELETE_OBJECT_OPTIONS"),
    "DELETE_PORT_DATA_1": SimStruct(OrderedDict((("psztPortName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 98)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DELETE_PORT_DATA_1", pack=False, align=None),
    "DELETE_SECURITY_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "DELETE_SNAPSHOT_VHDSET_FLAG": SimTypeInt(signed=False, label="DELETE_SNAPSHOT_VHDSET_FLAG"),
    "DELETE_SNAPSHOT_VHDSET_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="DELETE_SNAPSHOT_VHDSET_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DELETE_SNAPSHOT_VHDSET_PARAMETERS", pack=False, align=None),
    "DELETE_SNAPSHOT_VHDSET_VERSION": SimTypeInt(signed=False, label="DELETE_SNAPSHOT_VHDSET_VERSION"),
    "DELETE_USN_JOURNAL_DATA": SimStruct(OrderedDict((("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("DeleteFlags", SimTypeInt(signed=False, label="USN_DELETE_FLAGS")),)), name="DELETE_USN_JOURNAL_DATA", pack=False, align=None),
    "DELIVERY_AGENT_FLAGS": SimTypeInt(signed=False, label="DELIVERY_AGENT_FLAGS"),
    "DELTA_HASH": SimStruct(OrderedDict((("HashSize", SimTypeInt(signed=False, label="UInt32")), ("HashValue", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="DELTA_HASH", pack=False, align=None),
    "DELTA_HEADER_INFO": SimStruct(OrderedDict((("FileTypeSet", SimTypeLongLong(signed=True, label="Int64")), ("FileType", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeLongLong(signed=True, label="Int64")), ("TargetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TargetFileTime", SimTypeRef("FILETIME", SimStruct)), ("TargetHashAlgId", SimTypeInt(signed=False, label="ALG_ID")), ("TargetHash", SimTypeRef("DELTA_HASH", SimStruct)),)), name="DELTA_HEADER_INFO", pack=False, align=None),
    "DELTA_INPUT": SimStruct(OrderedDict((("Anonymous", SimUnion({"lpcStart": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "lpStart": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("uSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Editable", SimTypeInt(signed=True, label="Int32")),)), name="DELTA_INPUT", pack=False, align=None),
    "DELTA_OUTPUT": SimStruct(OrderedDict((("lpStart", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DELTA_OUTPUT", pack=False, align=None),
    "DEPENDENT_DISK_FLAG": SimTypeInt(signed=False, label="DEPENDENT_DISK_FLAG"),
    "DEP_SYSTEM_POLICY_TYPE": SimTypeInt(signed=False, label="DEP_SYSTEM_POLICY_TYPE"),
    "DEREFERENCECONNECTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PrimaryConnection", "ConnectionToDereference"]), offset=0),
    "DESCKIND": SimTypeInt(signed=False, label="DESCKIND"),
    "DESC_LINKAGE_TYPE": SimTypeInt(signed=False, label="DESC_LINKAGE_TYPE"),
    "DESIGNVECTOR": SimStruct(OrderedDict((("dvReserved", SimTypeInt(signed=False, label="UInt32")), ("dvNumAxes", SimTypeInt(signed=False, label="UInt32")), ("dvValues", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 16)),)), name="DESIGNVECTOR", pack=False, align=None),
    "DESKBANDCID": SimTypeInt(signed=False, label="DESKBANDCID"),
    "DESKBANDINFO": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="UInt32")), ("ptMinSize", SimTypeRef("POINTL", SimStruct)), ("ptMaxSize", SimTypeRef("POINTL", SimStruct)), ("ptIntegral", SimTypeRef("POINTL", SimStruct)), ("ptActual", SimTypeRef("POINTL", SimStruct)), ("wszTitle", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwModeFlags", SimTypeInt(signed=False, label="UInt32")), ("crBkgnd", SimTypeInt(signed=False, label="UInt32")),)), name="DESKBANDINFO", pack=False, align=None),
    "DESKTOPENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "DESKTOPENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "DESKTOP_ACCESS_FLAGS": SimTypeInt(signed=False, label="DESKTOP_ACCESS_FLAGS"),
    "DESKTOP_CONTROL_FLAGS": SimTypeInt(signed=False, label="DESKTOP_CONTROL_FLAGS"),
    "DESKTOP_SLIDESHOW_DIRECTION": SimTypeInt(signed=False, label="DESKTOP_SLIDESHOW_DIRECTION"),
    "DESKTOP_SLIDESHOW_OPTIONS": SimTypeInt(signed=False, label="DESKTOP_SLIDESHOW_OPTIONS"),
    "DESKTOP_SLIDESHOW_STATE": SimTypeInt(signed=False, label="DESKTOP_SLIDESHOW_STATE"),
    "DESKTOP_WALLPAPER_POSITION": SimTypeInt(signed=False, label="DESKTOP_WALLPAPER_POSITION"),
    "DETACH_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="DETACH_VIRTUAL_DISK_FLAG"),
    "DETAILSINFO": SimStruct(OrderedDict((("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("fmt", SimTypeInt(signed=True, label="Int32")), ("cxChar", SimTypeInt(signed=True, label="Int32")), ("str", SimTypeRef("STRRET", SimStruct)), ("iImage", SimTypeInt(signed=True, label="Int32")),)), name="DETAILSINFO", pack=False, align=None),
    "DETECTION_TYPE": SimTypeInt(signed=False, label="DETECTION_TYPE"),
    "DEVCAPS": SimStruct(OrderedDict((("CanRecord", SimTypeInt(signed=True, label="Int32")), ("CanRecordStrobe", SimTypeInt(signed=True, label="Int32")), ("HasAudio", SimTypeInt(signed=True, label="Int32")), ("HasVideo", SimTypeInt(signed=True, label="Int32")), ("UsesFiles", SimTypeInt(signed=True, label="Int32")), ("CanSave", SimTypeInt(signed=True, label="Int32")), ("DeviceType", SimTypeInt(signed=True, label="Int32")), ("TCRead", SimTypeInt(signed=True, label="Int32")), ("TCWrite", SimTypeInt(signed=True, label="Int32")), ("CTLRead", SimTypeInt(signed=True, label="Int32")), ("IndexRead", SimTypeInt(signed=True, label="Int32")), ("Preroll", SimTypeInt(signed=True, label="Int32")), ("Postroll", SimTypeInt(signed=True, label="Int32")), ("SyncAcc", SimTypeInt(signed=True, label="Int32")), ("NormRate", SimTypeInt(signed=True, label="Int32")), ("CanPreview", SimTypeInt(signed=True, label="Int32")), ("CanMonitorSrc", SimTypeInt(signed=True, label="Int32")), ("CanTest", SimTypeInt(signed=True, label="Int32")), ("VideoIn", SimTypeInt(signed=True, label="Int32")), ("AudioIn", SimTypeInt(signed=True, label="Int32")), ("Calibrate", SimTypeInt(signed=True, label="Int32")), ("SeekType", SimTypeInt(signed=True, label="Int32")), ("SimulatedHardware", SimTypeInt(signed=True, label="Int32")),)), name="DEVCAPS", pack=False, align=None),
    "DEVELOPER_DRIVE_ENABLEMENT_STATE": SimTypeInt(signed=False, label="DEVELOPER_DRIVE_ENABLEMENT_STATE"),
    "DEVHTADJDATA": SimStruct(OrderedDict((("DeviceFlags", SimTypeInt(signed=False, label="UInt32")), ("DeviceXDPI", SimTypeInt(signed=False, label="UInt32")), ("DeviceYDPI", SimTypeInt(signed=False, label="UInt32")), ("pDefHTInfo", SimTypePointer(SimTypeRef("DEVHTINFO", SimStruct), offset=0)), ("pAdjHTInfo", SimTypePointer(SimTypeRef("DEVHTINFO", SimStruct), offset=0)),)), name="DEVHTADJDATA", pack=False, align=None),
    "DEVHTINFO": SimStruct(OrderedDict((("HTFlags", SimTypeInt(signed=False, label="UInt32")), ("HTPatternSize", SimTypeInt(signed=False, label="UInt32")), ("DevPelsDPI", SimTypeInt(signed=False, label="UInt32")), ("ColorInfo", SimTypeRef("COLORINFO", SimStruct)),)), name="DEVHTINFO", pack=False, align=None),
    "DEVICEDIALOGDATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pIWiaItemRoot", SimTypeBottom(label="IWiaItem")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lIntent", SimTypeInt(signed=True, label="Int32")), ("lItemCount", SimTypeInt(signed=True, label="Int32")), ("ppWiaItems", SimTypePointer(SimTypeBottom(label="IWiaItem"), offset=0)),)), name="DEVICEDIALOGDATA", pack=False, align=None),
    "DEVICEDIALOGDATA2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pIWiaItemRoot", SimTypeBottom(label="IWiaItem2")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("bstrFolderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lNumFiles", SimTypeInt(signed=True, label="Int32")), ("pbstrFilePaths", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("pWiaItem", SimTypeBottom(label="IWiaItem2")),)), name="DEVICEDIALOGDATA2", pack=False, align=None),
    "DEVICEDUMP_COLLECTION_TYPEIDE_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="DEVICEDUMP_COLLECTION_TYPEIDE_NOTIFICATION_TYPE"),
    "DEVICEDUMP_PRIVATE_SUBSECTION": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("GPLogId", SimTypeRef("GP_LOG_PAGE_DESCRIPTOR", SimStruct)), ("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEVICEDUMP_PRIVATE_SUBSECTION", pack=False, align=None),
    "DEVICEDUMP_PUBLIC_SUBSECTION": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("GPLogTable", SimTypeArray(SimTypeRef("GP_LOG_PAGE_DESCRIPTOR", SimStruct), 16)), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 16)), ("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEVICEDUMP_PUBLIC_SUBSECTION", pack=False, align=None),
    "DEVICEDUMP_RESTRICTED_SUBSECTION": SimStruct(OrderedDict((("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEVICEDUMP_RESTRICTED_SUBSECTION", pack=False, align=None),
    "DEVICEDUMP_SECTION_HEADER": SimStruct(OrderedDict((("guidDeviceDataId", SimTypeBottom(label="Guid")), ("sOrganizationID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwFirmwareRevision", SimTypeInt(signed=False, label="UInt32")), ("sModelNumber", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("szDeviceManufacturingID", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bRestrictedPrivateDataVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFirmwareIssueId", SimTypeInt(signed=False, label="UInt32")), ("szIssueDescriptionString", SimTypeArray(SimTypeChar(label="Byte"), 132)),)), name="DEVICEDUMP_SECTION_HEADER", pack=False, align=None),
    "DEVICEDUMP_STORAGEDEVICE_DATA": SimStruct(OrderedDict((("Descriptor", SimTypeRef("DEVICEDUMP_STRUCTURE_VERSION", SimStruct)), ("SectionHeader", SimTypeRef("DEVICEDUMP_SECTION_HEADER", SimStruct)), ("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwReasonForCollection", SimTypeInt(signed=False, label="UInt32")), ("PublicData", SimTypeRef("DEVICEDUMP_SUBSECTION_POINTER", SimStruct)), ("RestrictedData", SimTypeRef("DEVICEDUMP_SUBSECTION_POINTER", SimStruct)), ("PrivateData", SimTypeRef("DEVICEDUMP_SUBSECTION_POINTER", SimStruct)),)), name="DEVICEDUMP_STORAGEDEVICE_DATA", pack=False, align=None),
    "DEVICEDUMP_STORAGESTACK_PUBLIC_DUMP": SimStruct(OrderedDict((("Descriptor", SimTypeRef("DEVICEDUMP_STRUCTURE_VERSION", SimStruct)), ("dwReasonForCollection", SimTypeInt(signed=False, label="UInt32")), ("cDriverName", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("uiNumRecords", SimTypeInt(signed=False, label="UInt32")), ("RecordArray", SimTypePointer(SimTypeRef("DEVICEDUMP_STORAGESTACK_PUBLIC_STATE_RECORD", SimStruct), offset=0)),)), name="DEVICEDUMP_STORAGESTACK_PUBLIC_DUMP", pack=False, align=None),
    "DEVICEDUMP_STORAGESTACK_PUBLIC_STATE_RECORD": SimStruct(OrderedDict((("Cdb", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Command", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("StartTime", SimTypeLongLong(signed=False, label="UInt64")), ("EndTime", SimTypeLongLong(signed=False, label="UInt64")), ("OperationStatus", SimTypeInt(signed=False, label="UInt32")), ("OperationError", SimTypeInt(signed=False, label="UInt32")), ("StackSpecific", SimUnion({"ExternalStack": SimTypeRef("_ExternalStack_e__Struct", SimStruct), "AtaPort": SimTypeRef("_AtaPort_e__Struct", SimStruct), "StorPort": SimTypeRef("_StorPort_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DEVICEDUMP_STORAGESTACK_PUBLIC_STATE_RECORD", pack=False, align=None),
    "DEVICEDUMP_STRUCTURE_VERSION": SimStruct(OrderedDict((("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICEDUMP_STRUCTURE_VERSION", pack=False, align=None),
    "DEVICEDUMP_SUBSECTION_POINTER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICEDUMP_SUBSECTION_POINTER", pack=False, align=None),
    "DEVICEFAMILYDEVICEFORM": SimTypeInt(signed=False, label="DEVICEFAMILYDEVICEFORM"),
    "DEVICEFAMILYINFOENUM": SimTypeInt(signed=False, label="DEVICEFAMILYINFOENUM"),
    "DEVICEPROPERTYHEADER": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszPrinterName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="DEVICEPROPERTYHEADER", pack=False, align=None),
    "DEVICE_BUS_SPECIFIC_RESET_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimUnion({"Pci": SimTypeRef("_Pci_e__Struct", SimStruct), "Acpi": SimTypeRef("_Acpi_e__Struct", SimStruct), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), SimTypePointer(SimUnion({"u": SimTypeRef("_u_e__Struct", SimStruct), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InterfaceContext", "BusType", "ResetTypeSelected", "Flags", "ResetParameters"]), offset=0),
    "DEVICE_BUS_SPECIFIC_RESET_INFO": SimStruct(OrderedDict((("BusTypeGuid", SimTypeBottom(label="Guid")), ("ResetTypeSupported", SimUnion({"Pci": SimTypeRef("_Pci_e__Struct", SimStruct), "Acpi": SimTypeRef("_Acpi_e__Struct", SimStruct), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="DEVICE_BUS_SPECIFIC_RESET_INFO", pack=False, align=None),
    "DEVICE_BUS_SPECIFIC_RESET_TYPE": SimUnion({"Pci": SimTypeRef("_Pci_e__Struct", SimStruct), "Acpi": SimTypeRef("_Acpi_e__Struct", SimStruct), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "DEVICE_CAPABILITIES": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeInt(signed=False, label="UInt32")), ("UINumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceState", SimTypeArray(SimTypeInt(signed=False, label="DEVICE_POWER_STATE"), 7)), ("SystemWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("DeviceWake", SimTypeInt(signed=False, label="DEVICE_POWER_STATE")), ("D1Latency", SimTypeInt(signed=False, label="UInt32")), ("D2Latency", SimTypeInt(signed=False, label="UInt32")), ("D3Latency", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_CAPABILITIES", pack=False, align=None),
    "DEVICE_CHANGE_COMPLETE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "DEVICE_COPY_OFFLOAD_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("MaximumTokenLifetime", SimTypeInt(signed=False, label="UInt32")), ("DefaultTokenLifetime", SimTypeInt(signed=False, label="UInt32")), ("MaximumTransferSize", SimTypeLongLong(signed=False, label="UInt64")), ("OptimalTransferCount", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumDataDescriptors", SimTypeInt(signed=False, label="UInt32")), ("MaximumTransferLengthPerDescriptor", SimTypeInt(signed=False, label="UInt32")), ("OptimalTransferLengthPerDescriptor", SimTypeInt(signed=False, label="UInt32")), ("OptimalTransferLengthGranularity", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="DEVICE_COPY_OFFLOAD_DESCRIPTOR", pack=False, align=None),
    "DEVICE_DATA_SET_LBP_STATE_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("OutputVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_DATA_SET_LBP_STATE_PARAMETERS", pack=False, align=None),
    "DEVICE_DATA_SET_LB_PROVISIONING_STATE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SlabSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("SlabOffsetDeltaInBytes", SimTypeInt(signed=False, label="UInt32")), ("SlabAllocationBitMapBitCount", SimTypeInt(signed=False, label="UInt32")), ("SlabAllocationBitMapLength", SimTypeInt(signed=False, label="UInt32")), ("SlabAllocationBitMap", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DEVICE_DATA_SET_LB_PROVISIONING_STATE", pack=False, align=None),
    "DEVICE_DATA_SET_LB_PROVISIONING_STATE_V2": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SlabSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("SlabOffsetDeltaInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("SlabAllocationBitMapBitCount", SimTypeInt(signed=False, label="UInt32")), ("SlabAllocationBitMapLength", SimTypeInt(signed=False, label="UInt32")), ("SlabAllocationBitMap", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DEVICE_DATA_SET_LB_PROVISIONING_STATE_V2", pack=False, align=None),
    "DEVICE_DATA_SET_RANGE": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("LengthInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEVICE_DATA_SET_RANGE", pack=False, align=None),
    "DEVICE_DATA_SET_REPAIR_OUTPUT": SimStruct(OrderedDict((("ParityExtent", SimTypeRef("DEVICE_DATA_SET_RANGE", SimStruct)),)), name="DEVICE_DATA_SET_REPAIR_OUTPUT", pack=False, align=None),
    "DEVICE_DATA_SET_REPAIR_PARAMETERS": SimStruct(OrderedDict((("NumberOfRepairCopies", SimTypeInt(signed=False, label="UInt32")), ("SourceCopy", SimTypeInt(signed=False, label="UInt32")), ("RepairCopies", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DEVICE_DATA_SET_REPAIR_PARAMETERS", pack=False, align=None),
    "DEVICE_DATA_SET_SCRUB_EX_OUTPUT": SimStruct(OrderedDict((("BytesProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("BytesRepaired", SimTypeLongLong(signed=False, label="UInt64")), ("BytesFailed", SimTypeLongLong(signed=False, label="UInt64")), ("ParityExtent", SimTypeRef("DEVICE_DATA_SET_RANGE", SimStruct)), ("BytesScrubbed", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEVICE_DATA_SET_SCRUB_EX_OUTPUT", pack=False, align=None),
    "DEVICE_DATA_SET_SCRUB_OUTPUT": SimStruct(OrderedDict((("BytesProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("BytesRepaired", SimTypeLongLong(signed=False, label="UInt64")), ("BytesFailed", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEVICE_DATA_SET_SCRUB_OUTPUT", pack=False, align=None),
    "DEVICE_DATA_SET_TOPOLOGY_ID_QUERY_OUTPUT": SimStruct(OrderedDict((("TopologyRangeBytes", SimTypeLongLong(signed=False, label="UInt64")), ("TopologyId", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DEVICE_DATA_SET_TOPOLOGY_ID_QUERY_OUTPUT", pack=False, align=None),
    "DEVICE_DESCRIPTION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Master", SimTypeChar(label="Byte")), ("ScatterGather", SimTypeChar(label="Byte")), ("DemandMode", SimTypeChar(label="Byte")), ("AutoInitialize", SimTypeChar(label="Byte")), ("Dma32BitAddresses", SimTypeChar(label="Byte")), ("IgnoreCount", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("Dma64BitAddresses", SimTypeChar(label="Byte")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("DmaChannel", SimTypeInt(signed=False, label="UInt32")), ("InterfaceType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("DmaWidth", SimTypeInt(signed=False, label="DMA_WIDTH")), ("DmaSpeed", SimTypeInt(signed=False, label="DMA_SPEED")), ("MaximumLength", SimTypeInt(signed=False, label="UInt32")), ("DmaPort", SimTypeInt(signed=False, label="UInt32")), ("DmaAddressWidth", SimTypeInt(signed=False, label="UInt32")), ("DmaControllerInstance", SimTypeInt(signed=False, label="UInt32")), ("DmaRequestLine", SimTypeInt(signed=False, label="UInt32")), ("DeviceAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="DEVICE_DESCRIPTION", pack=False, align=None),
    "DEVICE_DESCRIPTOR": SimStruct(OrderedDict((("usVendorId", SimTypeShort(signed=False, label="UInt16")), ("usProductId", SimTypeShort(signed=False, label="UInt16")), ("usBcdDevice", SimTypeShort(signed=False, label="UInt16")), ("usLanguageId", SimTypeShort(signed=False, label="UInt16")),)), name="DEVICE_DESCRIPTOR", pack=False, align=None),
    "DEVICE_DIRECTORY_TYPE": SimTypeInt(signed=False, label="DEVICE_DIRECTORY_TYPE"),
    "DEVICE_DSM_CONVERSION_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Source", SimTypeBottom(label="Guid")),)), name="DEVICE_DSM_CONVERSION_OUTPUT", pack=False, align=None),
    "DEVICE_DSM_DEFINITION": SimStruct(OrderedDict((("Action", SimTypeInt(signed=False, label="UInt32")), ("SingleRange", SimTypeChar(label="Byte")), ("ParameterBlockAlignment", SimTypeInt(signed=False, label="UInt32")), ("ParameterBlockLength", SimTypeInt(signed=False, label="UInt32")), ("HasOutput", SimTypeChar(label="Byte")), ("OutputBlockAlignment", SimTypeInt(signed=False, label="UInt32")), ("OutputBlockLength", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_DSM_DEFINITION", pack=False, align=None),
    "DEVICE_DSM_FREE_SPACE_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("FreeSpace", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEVICE_DSM_FREE_SPACE_OUTPUT", pack=False, align=None),
    "DEVICE_DSM_LOST_QUERY_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("NumberOfBits", SimTypeInt(signed=False, label="UInt32")), ("BitMap", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DEVICE_DSM_LOST_QUERY_OUTPUT", pack=False, align=None),
    "DEVICE_DSM_LOST_QUERY_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Granularity", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEVICE_DSM_LOST_QUERY_PARAMETERS", pack=False, align=None),
    "DEVICE_DSM_NOTIFICATION_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumFileTypeIDs", SimTypeInt(signed=False, label="UInt32")), ("FileTypeID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="DEVICE_DSM_NOTIFICATION_PARAMETERS", pack=False, align=None),
    "DEVICE_DSM_NVCACHE_CHANGE_PRIORITY_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("TargetPriority", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="DEVICE_DSM_NVCACHE_CHANGE_PRIORITY_PARAMETERS", pack=False, align=None),
    "DEVICE_DSM_OFFLOAD_READ_PARAMETERS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("TimeToLive", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="DEVICE_DSM_OFFLOAD_READ_PARAMETERS", pack=False, align=None),
    "DEVICE_DSM_OFFLOAD_WRITE_PARAMETERS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("TokenOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Token", SimTypeRef("STORAGE_OFFLOAD_TOKEN", SimStruct)),)), name="DEVICE_DSM_OFFLOAD_WRITE_PARAMETERS", pack=False, align=None),
    "DEVICE_DSM_PHYSICAL_ADDRESSES_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TotalNumberOfRanges", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRangesReturned", SimTypeInt(signed=False, label="UInt32")), ("Ranges", SimTypePointer(SimTypeRef("DEVICE_STORAGE_ADDRESS_RANGE", SimStruct), offset=0)),)), name="DEVICE_DSM_PHYSICAL_ADDRESSES_OUTPUT", pack=False, align=None),
    "DEVICE_DSM_RANGE_ERROR_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TotalNumberOfRanges", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRangesReturned", SimTypeInt(signed=False, label="UInt32")), ("Ranges", SimTypePointer(SimTypeRef("DEVICE_STORAGE_RANGE_ATTRIBUTES", SimStruct), offset=0)),)), name="DEVICE_DSM_RANGE_ERROR_INFO", pack=False, align=None),
    "DEVICE_DSM_REPORT_ZONES_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ZoneCount", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="STORAGE_ZONES_ATTRIBUTES")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("ZoneDescriptors", SimTypePointer(SimTypeRef("STORAGE_ZONE_DESCRIPTOR", SimStruct), offset=0)),)), name="DEVICE_DSM_REPORT_ZONES_DATA", pack=False, align=None),
    "DEVICE_DSM_REPORT_ZONES_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ReportOption", SimTypeChar(label="Byte")), ("Partial", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="DEVICE_DSM_REPORT_ZONES_PARAMETERS", pack=False, align=None),
    "DEVICE_DSM_TIERING_QUERY_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumberOfTierIds", SimTypeInt(signed=False, label="UInt32")), ("TierIds", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="DEVICE_DSM_TIERING_QUERY_INPUT", pack=False, align=None),
    "DEVICE_DSM_TIERING_QUERY_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("TotalNumberOfRegions", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRegionsReturned", SimTypeInt(signed=False, label="UInt32")), ("Regions", SimTypePointer(SimTypeRef("STORAGE_TIER_REGION", SimStruct), offset=0)),)), name="DEVICE_DSM_TIERING_QUERY_OUTPUT", pack=False, align=None),
    "DEVICE_EVENT_BECOMING_READY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Reason", SimTypeInt(signed=False, label="UInt32")), ("Estimated100msToReady", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_EVENT_BECOMING_READY", pack=False, align=None),
    "DEVICE_EVENT_EXTERNAL_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("DeviceClass", SimTypeInt(signed=False, label="UInt32")), ("ButtonStatus", SimTypeShort(signed=False, label="UInt16")), ("Request", SimTypeShort(signed=False, label="UInt16")), ("SystemTime", SimTypeLongLong(signed=True, label="Int64")),)), name="DEVICE_EVENT_EXTERNAL_REQUEST", pack=False, align=None),
    "DEVICE_EVENT_GENERIC_DATA": SimStruct(OrderedDict((("EventNumber", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_EVENT_GENERIC_DATA", pack=False, align=None),
    "DEVICE_EVENT_MOUNT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileSystemNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileSystemNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_EVENT_MOUNT", pack=False, align=None),
    "DEVICE_EVENT_RBC_DATA": SimStruct(OrderedDict((("EventNumber", SimTypeInt(signed=False, label="UInt32")), ("SenseQualifier", SimTypeChar(label="Byte")), ("SenseCode", SimTypeChar(label="Byte")), ("SenseKey", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Information", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_EVENT_RBC_DATA", pack=False, align=None),
    "DEVICE_FAULT_CONFIGURATION": SimStruct(OrderedDict((("FaultHandler", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("FaultContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEVICE_FAULT_CONFIGURATION", pack=False, align=None),
    "DEVICE_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_FLAGS", pack=False, align=None),
    "DEVICE_INFO": SimStruct(OrderedDict((("pFriendlyDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUniqueDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pManufacturerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pModelName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pIconURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DEVICE_INFO", pack=False, align=None),
    "DEVICE_INSTALL_STATE": SimTypeInt(signed=False, label="DEVICE_INSTALL_STATE"),
    "DEVICE_INTERFACE_CHANGE_NOTIFICATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Event", SimTypeBottom(label="Guid")), ("InterfaceClassGuid", SimTypeBottom(label="Guid")), ("SymbolicLinkName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)),)), name="DEVICE_INTERFACE_CHANGE_NOTIFICATION", pack=False, align=None),
    "DEVICE_INTERNAL_STATUS_DATA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("T10VendorId", SimTypeLongLong(signed=False, label="UInt64")), ("DataSet1Length", SimTypeInt(signed=False, label="UInt32")), ("DataSet2Length", SimTypeInt(signed=False, label="UInt32")), ("DataSet3Length", SimTypeInt(signed=False, label="UInt32")), ("DataSet4Length", SimTypeInt(signed=False, label="UInt32")), ("StatusDataVersion", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ReasonIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("StatusDataLength", SimTypeInt(signed=False, label="UInt32")), ("StatusData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEVICE_INTERNAL_STATUS_DATA", pack=False, align=None),
    "DEVICE_INTERNAL_STATUS_DATA_REQUEST_TYPE": SimTypeInt(signed=False, label="DEVICE_INTERNAL_STATUS_DATA_REQUEST_TYPE"),
    "DEVICE_INTERNAL_STATUS_DATA_SET": SimTypeInt(signed=False, label="DEVICE_INTERNAL_STATUS_DATA_SET"),
    "DEVICE_LB_PROVISIONING_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("OptimalUnmapGranularity", SimTypeLongLong(signed=False, label="UInt64")), ("UnmapGranularityAlignment", SimTypeLongLong(signed=False, label="UInt64")), ("MaxUnmapLbaCount", SimTypeInt(signed=False, label="UInt32")), ("MaxUnmapBlockDescriptorCount", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_LB_PROVISIONING_DESCRIPTOR", pack=False, align=None),
    "DEVICE_LOCATION": SimStruct(OrderedDict((("Socket", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeInt(signed=False, label="UInt32")), ("Adapter", SimTypeInt(signed=False, label="UInt32")), ("Port", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("Channel", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("Target", SimTypeInt(signed=False, label="UInt32")), ("Lun", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="DEVICE_LOCATION", pack=False, align=None),
    "DEVICE_MANAGE_DATA_SET_ATTRIBUTES": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ParameterBlockOffset", SimTypeInt(signed=False, label="UInt32")), ("ParameterBlockLength", SimTypeInt(signed=False, label="UInt32")), ("DataSetRangesOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSetRangesLength", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_MANAGE_DATA_SET_ATTRIBUTES", pack=False, align=None),
    "DEVICE_MANAGE_DATA_SET_ATTRIBUTES_OUTPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("OperationStatus", SimTypeInt(signed=False, label="UInt32")), ("ExtendedError", SimTypeInt(signed=False, label="UInt32")), ("TargetDetailedError", SimTypeInt(signed=False, label="UInt32")), ("ReservedStatus", SimTypeInt(signed=False, label="UInt32")), ("OutputBlockOffset", SimTypeInt(signed=False, label="UInt32")), ("OutputBlockLength", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_MANAGE_DATA_SET_ATTRIBUTES_OUTPUT", pack=False, align=None),
    "DEVICE_MEDIA_INFO": SimStruct(OrderedDict((("DeviceSpecific", SimUnion({"DiskInfo": SimTypeRef("_DiskInfo_e__Struct", SimStruct), "RemovableDiskInfo": SimTypeRef("_RemovableDiskInfo_e__Struct", SimStruct), "TapeInfo": SimTypeRef("_TapeInfo_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DEVICE_MEDIA_INFO", pack=False, align=None),
    "DEVICE_NOTIFY_SUBSCRIBE_PARAMETERS": SimStruct(OrderedDict((("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Type", "Setting"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEVICE_NOTIFY_SUBSCRIBE_PARAMETERS", pack=False, align=None),
    "DEVICE_OBJECT": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("ReferenceCount", SimTypeInt(signed=True, label="Int32")), ("DriverObject", SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0)), ("NextDevice", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("AttachedDevice", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("CurrentIrp", SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), ("Timer", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Characteristics", SimTypeInt(signed=False, label="UInt32")), ("Vpb", SimTypePointer(SimTypeRef("VPB", SimStruct), offset=0)), ("DeviceExtension", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("StackSize", SimTypeChar(label="SByte")), ("Queue", SimUnion({"ListEntry": SimTypeRef("LIST_ENTRY", SimStruct), "Wcb": SimTypeRef("WAIT_CONTEXT_BLOCK", SimStruct)}, name="<anon>", label="None")), ("AlignmentRequirement", SimTypeInt(signed=False, label="UInt32")), ("DeviceQueue", SimTypeRef("KDEVICE_QUEUE", SimStruct)), ("Dpc", SimTypeRef("KDPC", SimStruct)), ("ActiveThreadCount", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DeviceLock", SimTypeRef("KEVENT", SimStruct)), ("SectorSize", SimTypeShort(signed=False, label="UInt16")), ("Spare1", SimTypeShort(signed=False, label="UInt16")), ("DeviceObjectExtension", SimTypePointer(SimTypeRef("DEVOBJ_EXTENSION", SimStruct), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEVICE_OBJECT", pack=False, align=None),
    "DEVICE_ORIENTATION_TYPE": SimTypeInt(signed=False, label="DEVICE_ORIENTATION_TYPE"),
    "DEVICE_POWER_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceAttentionSupported", SimTypeChar(label="Byte")), ("AsynchronousNotificationSupported", SimTypeChar(label="Byte")), ("IdlePowerManagementEnabled", SimTypeChar(label="Byte")), ("D3ColdEnabled", SimTypeChar(label="Byte")), ("D3ColdSupported", SimTypeChar(label="Byte")), ("NoVerifyDuringIdlePower", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("IdleTimeoutInMS", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_POWER_DESCRIPTOR", pack=False, align=None),
    "DEVICE_POWER_STATE": SimTypeInt(signed=False, label="DEVICE_POWER_STATE"),
    "DEVICE_QUERY_BUS_SPECIFIC_RESET_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("DEVICE_BUS_SPECIFIC_RESET_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InterfaceContext", "ResetInfoCount", "ResetInfoSupported"]), offset=0),
    "DEVICE_RADIO_STATE": SimTypeInt(signed=False, label="DEVICE_RADIO_STATE"),
    "DEVICE_REGISTRY_PROPERTY": SimTypeInt(signed=False, label="DEVICE_REGISTRY_PROPERTY"),
    "DEVICE_RELATIONS": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Objects", SimTypePointer(SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), offset=0)),)), name="DEVICE_RELATIONS", pack=False, align=None),
    "DEVICE_RELATION_TYPE": SimTypeInt(signed=False, label="DEVICE_RELATION_TYPE"),
    "DEVICE_REMOVAL_POLICY": SimTypeInt(signed=False, label="DEVICE_REMOVAL_POLICY"),
    "DEVICE_RESET_COMPLETION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "Context"]), offset=0),
    "DEVICE_RESET_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="DEVICE_RESET_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InterfaceContext", "ResetType", "Flags", "ResetParameters"]), offset=0),
    "DEVICE_RESET_INTERFACE_STANDARD": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("DeviceReset", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("SupportedResetTypes", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QueryBusSpecificResetInfo", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeviceBusSpecificReset", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetDeviceResetStatus", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="DEVICE_RESET_INTERFACE_STANDARD", pack=False, align=None),
    "DEVICE_RESET_STATUS_FLAGS": SimUnion({"u": SimTypeRef("_u_e__Struct", SimStruct), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "DEVICE_RESET_TYPE": SimTypeInt(signed=False, label="DEVICE_RESET_TYPE"),
    "DEVICE_SCALE_FACTOR": SimTypeInt(signed=False, label="DEVICE_SCALE_FACTOR"),
    "DEVICE_SEEK_PENALTY_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("IncursSeekPenalty", SimTypeChar(label="Byte")),)), name="DEVICE_SEEK_PENALTY_DESCRIPTOR", pack=False, align=None),
    "DEVICE_SELECTION_DEVICE_TYPE": SimTypeInt(signed=False, label="DEVICE_SELECTION_DEVICE_TYPE"),
    "DEVICE_STATE": SimTypeInt(signed=False, label="DEVICE_STATE"),
    "DEVICE_STORAGE_ADDRESS_RANGE": SimStruct(OrderedDict((("StartAddress", SimTypeLongLong(signed=True, label="Int64")), ("LengthInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="DEVICE_STORAGE_ADDRESS_RANGE", pack=False, align=None),
    "DEVICE_STORAGE_RANGE_ATTRIBUTES": SimStruct(OrderedDict((("LengthInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"AllFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DEVICE_STORAGE_RANGE_ATTRIBUTES", pack=False, align=None),
    "DEVICE_TEXT_TYPE": SimTypeInt(signed=False, label="DEVICE_TEXT_TYPE"),
    "DEVICE_TRIM_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("TrimEnabled", SimTypeChar(label="Byte")),)), name="DEVICE_TRIM_DESCRIPTOR", pack=False, align=None),
    "DEVICE_USAGE_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="DEVICE_USAGE_NOTIFICATION_TYPE"),
    "DEVICE_USER_CHANGE_EVENT_DATA": SimStruct(OrderedDict((("cbDeviceUserChangeEventData", SimTypeInt(signed=False, label="UInt32")), ("wszUser", SimTypeChar(label="Char")),)), name="DEVICE_USER_CHANGE_EVENT_DATA", pack=False, align=None),
    "DEVICE_WAKE_DEPTH": SimTypeInt(signed=False, label="DEVICE_WAKE_DEPTH"),
    "DEVICE_WRITE_AGGREGATION_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("BenefitsFromWriteAggregation", SimTypeChar(label="Byte")),)), name="DEVICE_WRITE_AGGREGATION_DESCRIPTOR", pack=False, align=None),
    "DEVINFO": SimStruct(OrderedDict((("flGraphicsCaps", SimTypeInt(signed=False, label="UInt32")), ("lfDefaultFont", SimTypeRef("LOGFONTW", SimStruct)), ("lfAnsiVarFont", SimTypeRef("LOGFONTW", SimStruct)), ("lfAnsiFixFont", SimTypeRef("LOGFONTW", SimStruct)), ("cFonts", SimTypeInt(signed=False, label="UInt32")), ("iDitherFormat", SimTypeInt(signed=False, label="UInt32")), ("cxDither", SimTypeShort(signed=False, label="UInt16")), ("cyDither", SimTypeShort(signed=False, label="UInt16")), ("hpalDefault", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("flGraphicsCaps2", SimTypeInt(signed=False, label="UInt32")),)), name="DEVINFO", pack=False, align=None),
    "DEVMODEA": SimStruct(OrderedDict((("dmDeviceName", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("dmSpecVersion", SimTypeShort(signed=False, label="UInt16")), ("dmDriverVersion", SimTypeShort(signed=False, label="UInt16")), ("dmSize", SimTypeShort(signed=False, label="UInt16")), ("dmDriverExtra", SimTypeShort(signed=False, label="UInt16")), ("dmFields", SimTypeInt(signed=False, label="DEVMODE_FIELD_FLAGS")), ("Anonymous1", SimUnion({"Anonymous1": SimStruct(OrderedDict((("dmOrientation", SimTypeShort(signed=True, label="Int16")), ("dmPaperSize", SimTypeShort(signed=True, label="Int16")), ("dmPaperLength", SimTypeShort(signed=True, label="Int16")), ("dmPaperWidth", SimTypeShort(signed=True, label="Int16")), ("dmScale", SimTypeShort(signed=True, label="Int16")), ("dmCopies", SimTypeShort(signed=True, label="Int16")), ("dmDefaultSource", SimTypeShort(signed=True, label="Int16")), ("dmPrintQuality", SimTypeShort(signed=True, label="Int16")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("dmPosition", SimTypeRef("POINTL", SimStruct)), ("dmDisplayOrientation", SimTypeInt(signed=False, label="DEVMODE_DISPLAY_ORIENTATION")), ("dmDisplayFixedOutput", SimTypeInt(signed=False, label="DEVMODE_DISPLAY_FIXED_OUTPUT")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("dmColor", SimTypeInt(signed=False, label="DEVMODE_COLOR")), ("dmDuplex", SimTypeInt(signed=False, label="DEVMODE_DUPLEX")), ("dmYResolution", SimTypeShort(signed=True, label="Int16")), ("dmTTOption", SimTypeInt(signed=False, label="DEVMODE_TRUETYPE_OPTION")), ("dmCollate", SimTypeInt(signed=False, label="DEVMODE_COLLATE")), ("dmFormName", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("dmLogPixels", SimTypeShort(signed=False, label="UInt16")), ("dmBitsPerPel", SimTypeInt(signed=False, label="UInt32")), ("dmPelsWidth", SimTypeInt(signed=False, label="UInt32")), ("dmPelsHeight", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"dmDisplayFlags": SimTypeInt(signed=False, label="UInt32"), "dmNup": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dmDisplayFrequency", SimTypeInt(signed=False, label="UInt32")), ("dmICMMethod", SimTypeInt(signed=False, label="UInt32")), ("dmICMIntent", SimTypeInt(signed=False, label="UInt32")), ("dmMediaType", SimTypeInt(signed=False, label="UInt32")), ("dmDitherType", SimTypeInt(signed=False, label="UInt32")), ("dmReserved1", SimTypeInt(signed=False, label="UInt32")), ("dmReserved2", SimTypeInt(signed=False, label="UInt32")), ("dmPanningWidth", SimTypeInt(signed=False, label="UInt32")), ("dmPanningHeight", SimTypeInt(signed=False, label="UInt32")),)), name="DEVMODEA", pack=False, align=None),
    "DEVMODEW": SimStruct(OrderedDict((("dmDeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dmSpecVersion", SimTypeShort(signed=False, label="UInt16")), ("dmDriverVersion", SimTypeShort(signed=False, label="UInt16")), ("dmSize", SimTypeShort(signed=False, label="UInt16")), ("dmDriverExtra", SimTypeShort(signed=False, label="UInt16")), ("dmFields", SimTypeInt(signed=False, label="DEVMODE_FIELD_FLAGS")), ("Anonymous1", SimUnion({"Anonymous1": SimStruct(OrderedDict((("dmOrientation", SimTypeShort(signed=True, label="Int16")), ("dmPaperSize", SimTypeShort(signed=True, label="Int16")), ("dmPaperLength", SimTypeShort(signed=True, label="Int16")), ("dmPaperWidth", SimTypeShort(signed=True, label="Int16")), ("dmScale", SimTypeShort(signed=True, label="Int16")), ("dmCopies", SimTypeShort(signed=True, label="Int16")), ("dmDefaultSource", SimTypeShort(signed=True, label="Int16")), ("dmPrintQuality", SimTypeShort(signed=True, label="Int16")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("dmPosition", SimTypeRef("POINTL", SimStruct)), ("dmDisplayOrientation", SimTypeInt(signed=False, label="DEVMODE_DISPLAY_ORIENTATION")), ("dmDisplayFixedOutput", SimTypeInt(signed=False, label="DEVMODE_DISPLAY_FIXED_OUTPUT")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("dmColor", SimTypeInt(signed=False, label="DEVMODE_COLOR")), ("dmDuplex", SimTypeInt(signed=False, label="DEVMODE_DUPLEX")), ("dmYResolution", SimTypeShort(signed=True, label="Int16")), ("dmTTOption", SimTypeInt(signed=False, label="DEVMODE_TRUETYPE_OPTION")), ("dmCollate", SimTypeInt(signed=False, label="DEVMODE_COLLATE")), ("dmFormName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dmLogPixels", SimTypeShort(signed=False, label="UInt16")), ("dmBitsPerPel", SimTypeInt(signed=False, label="UInt32")), ("dmPelsWidth", SimTypeInt(signed=False, label="UInt32")), ("dmPelsHeight", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"dmDisplayFlags": SimTypeInt(signed=False, label="UInt32"), "dmNup": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dmDisplayFrequency", SimTypeInt(signed=False, label="UInt32")), ("dmICMMethod", SimTypeInt(signed=False, label="UInt32")), ("dmICMIntent", SimTypeInt(signed=False, label="UInt32")), ("dmMediaType", SimTypeInt(signed=False, label="UInt32")), ("dmDitherType", SimTypeInt(signed=False, label="UInt32")), ("dmReserved1", SimTypeInt(signed=False, label="UInt32")), ("dmReserved2", SimTypeInt(signed=False, label="UInt32")), ("dmPanningWidth", SimTypeInt(signed=False, label="UInt32")), ("dmPanningHeight", SimTypeInt(signed=False, label="UInt32")),)), name="DEVMODEW", pack=False, align=None),
    "DEVMODE_COLLATE": SimTypeInt(signed=False, label="DEVMODE_COLLATE"),
    "DEVMODE_COLOR": SimTypeInt(signed=False, label="DEVMODE_COLOR"),
    "DEVMODE_DISPLAY_FIXED_OUTPUT": SimTypeInt(signed=False, label="DEVMODE_DISPLAY_FIXED_OUTPUT"),
    "DEVMODE_DISPLAY_ORIENTATION": SimTypeInt(signed=False, label="DEVMODE_DISPLAY_ORIENTATION"),
    "DEVMODE_DUPLEX": SimTypeInt(signed=False, label="DEVMODE_DUPLEX"),
    "DEVMODE_FIELD_FLAGS": SimTypeInt(signed=False, label="DEVMODE_FIELD_FLAGS"),
    "DEVMODE_TRUETYPE_OPTION": SimTypeInt(signed=False, label="DEVMODE_TRUETYPE_OPTION"),
    "DEVNAMES": SimStruct(OrderedDict((("wDriverOffset", SimTypeShort(signed=False, label="UInt16")), ("wDeviceOffset", SimTypeShort(signed=False, label="UInt16")), ("wOutputOffset", SimTypeShort(signed=False, label="UInt16")), ("wDefault", SimTypeShort(signed=False, label="UInt16")),)), name="DEVNAMES", pack=False, align=None),
    "DEVOBJ_EXTENSION": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("PowerFlags", SimTypeInt(signed=False, label="UInt32")), ("Dope", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("ExtensionFlags", SimTypeInt(signed=False, label="UInt32")), ("DeviceNode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AttachedTo", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("StartIoCount", SimTypeInt(signed=True, label="Int32")), ("StartIoKey", SimTypeInt(signed=True, label="Int32")), ("StartIoFlags", SimTypeInt(signed=False, label="UInt32")), ("Vpb", SimTypePointer(SimTypeRef("VPB", SimStruct), offset=0)), ("DependencyNode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterruptContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterruptCount", SimTypeInt(signed=True, label="Int32")), ("VerifierContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEVOBJ_EXTENSION", pack=False, align=None),
    "DEVPRIVATE_DES": SimStruct(OrderedDict((("PD_Count", SimTypeInt(signed=False, label="UInt32")), ("PD_Type", SimTypeInt(signed=False, label="UInt32")), ("PD_Data1", SimTypeInt(signed=False, label="UInt32")), ("PD_Data2", SimTypeInt(signed=False, label="UInt32")), ("PD_Data3", SimTypeInt(signed=False, label="UInt32")), ("PD_Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DEVPRIVATE_DES", pack=False, align=None),
    "DEVPRIVATE_RANGE": SimStruct(OrderedDict((("PR_Data1", SimTypeInt(signed=False, label="UInt32")), ("PR_Data2", SimTypeInt(signed=False, label="UInt32")), ("PR_Data3", SimTypeInt(signed=False, label="UInt32")),)), name="DEVPRIVATE_RANGE", pack=False, align=None),
    "DEVPRIVATE_RESOURCE": SimStruct(OrderedDict((("PRV_Header", SimTypeRef("DEVPRIVATE_DES", SimStruct)), ("PRV_Data", SimTypePointer(SimTypeRef("DEVPRIVATE_RANGE", SimStruct), offset=0)),)), name="DEVPRIVATE_RESOURCE", pack=False, align=None),
    "DEVPROPCOMPKEY": SimStruct(OrderedDict((("Key", SimTypeRef("DEVPROPKEY", SimStruct)), ("Store", SimTypeInt(signed=False, label="DEVPROPSTORE")), ("LocaleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DEVPROPCOMPKEY", pack=False, align=None),
    "DEVPROPERTY": SimStruct(OrderedDict((("CompKey", SimTypeRef("DEVPROPCOMPKEY", SimStruct)), ("Type", SimTypeInt(signed=False, label="DEVPROPTYPE")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEVPROPERTY", pack=False, align=None),
    "DEVPROPKEY": SimStruct(OrderedDict((("fmtid", SimTypeBottom(label="Guid")), ("pid", SimTypeInt(signed=False, label="UInt32")),)), name="DEVPROPKEY", pack=False, align=None),
    "DEVPROPSTORE": SimTypeInt(signed=False, label="DEVPROPSTORE"),
    "DEVPROPTYPE": SimTypeInt(signed=False, label="DEVPROPTYPE"),
    "DEVPROP_BOOLEAN": SimTypeChar(label="Byte"),
    "DEVPROP_FILTER_EXPRESSION": SimStruct(OrderedDict((("Operator", SimTypeInt(signed=False, label="DEVPROP_OPERATOR")), ("Property", SimTypeRef("DEVPROPERTY", SimStruct)),)), name="DEVPROP_FILTER_EXPRESSION", pack=False, align=None),
    "DEVPROP_OPERATOR": SimTypeInt(signed=False, label="DEVPROP_OPERATOR"),
    "DEVPROP_PCIDEVICE_ACSCOMPATIBLEUPHIERARCHY": SimTypeInt(signed=False, label="DEVPROP_PCIDEVICE_ACSCOMPATIBLEUPHIERARCHY"),
    "DEVPROP_PCIDEVICE_ACSSUPPORT": SimTypeInt(signed=False, label="DEVPROP_PCIDEVICE_ACSSUPPORT"),
    "DEVPROP_PCIDEVICE_CURRENTSPEEDANDMODE": SimTypeInt(signed=False, label="DEVPROP_PCIDEVICE_CURRENTSPEEDANDMODE"),
    "DEVPROP_PCIDEVICE_DEVICEBRIDGETYPE": SimTypeInt(signed=False, label="DEVPROP_PCIDEVICE_DEVICEBRIDGETYPE"),
    "DEVPROP_PCIDEVICE_INTERRUPTTYPE": SimTypeInt(signed=False, label="DEVPROP_PCIDEVICE_INTERRUPTTYPE"),
    "DEVPROP_PCIDEVICE_SRIOVSUPPORT": SimTypeInt(signed=False, label="DEVPROP_PCIDEVICE_SRIOVSUPPORT"),
    "DEVPROP_PCIEXPRESSDEVICE_LINKSPEED": SimTypeInt(signed=False, label="DEVPROP_PCIEXPRESSDEVICE_LINKSPEED"),
    "DEVPROP_PCIEXPRESSDEVICE_LINKWIDTH": SimTypeInt(signed=False, label="DEVPROP_PCIEXPRESSDEVICE_LINKWIDTH"),
    "DEVPROP_PCIEXPRESSDEVICE_PAYLOADORREQUESTSIZE": SimTypeInt(signed=False, label="DEVPROP_PCIEXPRESSDEVICE_PAYLOADORREQUESTSIZE"),
    "DEVPROP_PCIEXPRESSDEVICE_SPEC_VERSION": SimTypeInt(signed=False, label="DEVPROP_PCIEXPRESSDEVICE_SPEC_VERSION"),
    "DEVPROP_PCIROOTBUS_BUSWIDTH": SimTypeInt(signed=False, label="DEVPROP_PCIROOTBUS_BUSWIDTH"),
    "DEVPROP_PCIROOTBUS_CURRENTSPEEDANDMODE": SimTypeInt(signed=False, label="DEVPROP_PCIROOTBUS_CURRENTSPEEDANDMODE"),
    "DEVPROP_PCIROOTBUS_SECONDARYINTERFACE": SimTypeInt(signed=False, label="DEVPROP_PCIROOTBUS_SECONDARYINTERFACE"),
    "DEVPROP_PCIROOTBUS_SUPPORTEDSPEEDSANDMODES": SimTypeInt(signed=False, label="DEVPROP_PCIROOTBUS_SUPPORTEDSPEEDSANDMODES"),
    "DEVQUERYPRINT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Level", SimTypeShort(signed=False, label="UInt16")), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pszErrorStr", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchErrorStr", SimTypeInt(signed=False, label="UInt32")), ("cchNeeded", SimTypeInt(signed=False, label="UInt32")),)), name="DEVQUERYPRINT_INFO", pack=False, align=None),
    "DEV_BROADCAST_DEVICEINTERFACE_A": SimStruct(OrderedDict((("dbcc_size", SimTypeInt(signed=False, label="UInt32")), ("dbcc_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcc_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcc_classguid", SimTypeBottom(label="Guid")), ("dbcc_name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="DEV_BROADCAST_DEVICEINTERFACE_A", pack=False, align=None),
    "DEV_BROADCAST_DEVICEINTERFACE_W": SimStruct(OrderedDict((("dbcc_size", SimTypeInt(signed=False, label="UInt32")), ("dbcc_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcc_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcc_classguid", SimTypeBottom(label="Guid")), ("dbcc_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DEV_BROADCAST_DEVICEINTERFACE_W", pack=False, align=None),
    "DEV_BROADCAST_DEVNODE": SimStruct(OrderedDict((("dbcd_size", SimTypeInt(signed=False, label="UInt32")), ("dbcd_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcd_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcd_devnode", SimTypeInt(signed=False, label="UInt32")),)), name="DEV_BROADCAST_DEVNODE", pack=False, align=None),
    "DEV_BROADCAST_HANDLE": SimStruct(OrderedDict((("dbch_size", SimTypeInt(signed=False, label="UInt32")), ("dbch_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbch_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbch_handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dbch_hdevnotify", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dbch_eventguid", SimTypeBottom(label="Guid")), ("dbch_nameoffset", SimTypeInt(signed=True, label="Int32")), ("dbch_data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEV_BROADCAST_HANDLE", pack=False, align=None),
    "DEV_BROADCAST_HANDLE32": SimStruct(OrderedDict((("dbch_size", SimTypeInt(signed=False, label="UInt32")), ("dbch_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbch_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbch_handle", SimTypeInt(signed=False, label="UInt32")), ("dbch_hdevnotify", SimTypeInt(signed=False, label="UInt32")), ("dbch_eventguid", SimTypeBottom(label="Guid")), ("dbch_nameoffset", SimTypeInt(signed=True, label="Int32")), ("dbch_data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEV_BROADCAST_HANDLE32", pack=False, align=None),
    "DEV_BROADCAST_HANDLE64": SimStruct(OrderedDict((("dbch_size", SimTypeInt(signed=False, label="UInt32")), ("dbch_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbch_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbch_handle", SimTypeLongLong(signed=False, label="UInt64")), ("dbch_hdevnotify", SimTypeLongLong(signed=False, label="UInt64")), ("dbch_eventguid", SimTypeBottom(label="Guid")), ("dbch_nameoffset", SimTypeInt(signed=True, label="Int32")), ("dbch_data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DEV_BROADCAST_HANDLE64", pack=False, align=None),
    "DEV_BROADCAST_HDR": SimStruct(OrderedDict((("dbch_size", SimTypeInt(signed=False, label="UInt32")), ("dbch_devicetype", SimTypeInt(signed=False, label="DEV_BROADCAST_HDR_DEVICE_TYPE")), ("dbch_reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DEV_BROADCAST_HDR", pack=False, align=None),
    "DEV_BROADCAST_HDR_DEVICE_TYPE": SimTypeInt(signed=False, label="DEV_BROADCAST_HDR_DEVICE_TYPE"),
    "DEV_BROADCAST_NET": SimStruct(OrderedDict((("dbcn_size", SimTypeInt(signed=False, label="UInt32")), ("dbcn_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcn_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcn_resource", SimTypeInt(signed=False, label="UInt32")), ("dbcn_flags", SimTypeInt(signed=False, label="UInt32")),)), name="DEV_BROADCAST_NET", pack=False, align=None),
    "DEV_BROADCAST_OEM": SimStruct(OrderedDict((("dbco_size", SimTypeInt(signed=False, label="UInt32")), ("dbco_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbco_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbco_identifier", SimTypeInt(signed=False, label="UInt32")), ("dbco_suppfunc", SimTypeInt(signed=False, label="UInt32")),)), name="DEV_BROADCAST_OEM", pack=False, align=None),
    "DEV_BROADCAST_PORT_A": SimStruct(OrderedDict((("dbcp_size", SimTypeInt(signed=False, label="UInt32")), ("dbcp_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcp_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcp_name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="DEV_BROADCAST_PORT_A", pack=False, align=None),
    "DEV_BROADCAST_PORT_W": SimStruct(OrderedDict((("dbcp_size", SimTypeInt(signed=False, label="UInt32")), ("dbcp_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcp_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcp_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DEV_BROADCAST_PORT_W", pack=False, align=None),
    "DEV_BROADCAST_VOLUME": SimStruct(OrderedDict((("dbcv_size", SimTypeInt(signed=False, label="UInt32")), ("dbcv_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcv_reserved", SimTypeInt(signed=False, label="UInt32")), ("dbcv_unitmask", SimTypeInt(signed=False, label="UInt32")), ("dbcv_flags", SimTypeInt(signed=False, label="DEV_BROADCAST_VOLUME_FLAGS")),)), name="DEV_BROADCAST_VOLUME", pack=False, align=None),
    "DEV_BROADCAST_VOLUME_FLAGS": SimTypeInt(signed=False, label="DEV_BROADCAST_VOLUME_FLAGS"),
    "DEV_CONSOLE_MESSAGE_LEVEL": SimTypeInt(signed=False, label="DEV_CONSOLE_MESSAGE_LEVEL"),
    "DEV_OBJECT": SimStruct(OrderedDict((("ObjectType", SimTypeInt(signed=False, label="DEV_OBJECT_TYPE")), ("pszObjectId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cPropertyCount", SimTypeInt(signed=False, label="UInt32")), ("pProperties", SimTypePointer(SimTypeRef("DEVPROPERTY", SimStruct), offset=0)),)), name="DEV_OBJECT", pack=False, align=None),
    "DEV_OBJECT_TYPE": SimTypeInt(signed=False, label="DEV_OBJECT_TYPE"),
    "DEV_QUERY_FLAGS": SimTypeInt(signed=False, label="DEV_QUERY_FLAGS"),
    "DEV_QUERY_PARAMETER": SimStruct(OrderedDict((("Key", SimTypeRef("DEVPROPKEY", SimStruct)), ("Type", SimTypeInt(signed=False, label="DEVPROPTYPE")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DEV_QUERY_PARAMETER", pack=False, align=None),
    "DEV_QUERY_RESULT_ACTION": SimTypeInt(signed=False, label="DEV_QUERY_RESULT_ACTION"),
    "DEV_QUERY_RESULT_ACTION_DATA": SimStruct(OrderedDict((("Action", SimTypeInt(signed=False, label="DEV_QUERY_RESULT_ACTION")), ("Data", SimUnion({"State": SimTypeInt(signed=False, label="DEV_QUERY_STATE"), "DeviceObject": SimTypeRef("DEV_OBJECT", SimStruct)}, name="<anon>", label="None")),)), name="DEV_QUERY_RESULT_ACTION_DATA", pack=False, align=None),
    "DEV_QUERY_STATE": SimTypeInt(signed=False, label="DEV_QUERY_STATE"),
    "DFCS_STATE": SimTypeInt(signed=False, label="DFCS_STATE"),
    "DFC_TYPE": SimTypeInt(signed=False, label="DFC_TYPE"),
    "DFConstraint": SimTypeBottom(label="DFConstraint"),
    "DFMICS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("idCmdFirst", SimTypeInt(signed=False, label="UInt32")), ("idDefMax", SimTypeInt(signed=False, label="UInt32")), ("pici", SimTypePointer(SimTypeRef("CMINVOKECOMMANDINFO", SimStruct), offset=0)), ("punkSite", SimTypeBottom(label="IUnknown")),)), name="DFMICS", pack=False, align=None),
    "DFM_CMD": SimTypeInt(signed=False, label="DFM_CMD"),
    "DFM_MESSAGE_ID": SimTypeInt(signed=False, label="DFM_MESSAGE_ID"),
    "DFS_GET_PKT_ENTRY_STATE_ARG": SimStruct(OrderedDict((("DfsEntryPathLen", SimTypeShort(signed=False, label="UInt16")), ("ServerNameLen", SimTypeShort(signed=False, label="UInt16")), ("ShareNameLen", SimTypeShort(signed=False, label="UInt16")), ("Level", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_GET_PKT_ENTRY_STATE_ARG", pack=False, align=None),
    "DFS_INFO_1": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_INFO_1", pack=False, align=None),
    "DFS_INFO_100": SimStruct(OrderedDict((("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_INFO_100", pack=False, align=None),
    "DFS_INFO_101": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_101", pack=False, align=None),
    "DFS_INFO_102": SimStruct(OrderedDict((("Timeout", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_102", pack=False, align=None),
    "DFS_INFO_103": SimStruct(OrderedDict((("PropertyFlagMask", SimTypeInt(signed=False, label="UInt32")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_103", pack=False, align=None),
    "DFS_INFO_104": SimStruct(OrderedDict((("TargetPriority", SimTypeRef("DFS_TARGET_PRIORITY", SimStruct)),)), name="DFS_INFO_104", pack=False, align=None),
    "DFS_INFO_105": SimStruct(OrderedDict((("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("PropertyFlagMask", SimTypeInt(signed=False, label="UInt32")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_105", pack=False, align=None),
    "DFS_INFO_106": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")), ("TargetPriority", SimTypeRef("DFS_TARGET_PRIORITY", SimStruct)),)), name="DFS_INFO_106", pack=False, align=None),
    "DFS_INFO_107": SimStruct(OrderedDict((("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("PropertyFlagMask", SimTypeInt(signed=False, label="UInt32")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")), ("SdLengthReserved", SimTypeInt(signed=False, label="UInt32")), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DFS_INFO_107", pack=False, align=None),
    "DFS_INFO_150": SimStruct(OrderedDict((("SdLengthReserved", SimTypeInt(signed=False, label="UInt32")), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DFS_INFO_150", pack=False, align=None),
    "DFS_INFO_1_32": SimStruct(OrderedDict((("EntryPath", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_1_32", pack=False, align=None),
    "DFS_INFO_2": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_2", pack=False, align=None),
    "DFS_INFO_200": SimStruct(OrderedDict((("FtDfsName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_INFO_200", pack=False, align=None),
    "DFS_INFO_2_32": SimStruct(OrderedDict((("EntryPath", SimTypeInt(signed=False, label="UInt32")), ("Comment", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_2_32", pack=False, align=None),
    "DFS_INFO_3": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")), ("Storage", SimTypePointer(SimTypeRef("DFS_STORAGE_INFO", SimStruct), offset=0)),)), name="DFS_INFO_3", pack=False, align=None),
    "DFS_INFO_300": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("DfsName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_INFO_300", pack=False, align=None),
    "DFS_INFO_3_32": SimStruct(OrderedDict((("EntryPath", SimTypeInt(signed=False, label="UInt32")), ("Comment", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")), ("Storage", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_3_32", pack=False, align=None),
    "DFS_INFO_4": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")), ("Storage", SimTypePointer(SimTypeRef("DFS_STORAGE_INFO", SimStruct), offset=0)),)), name="DFS_INFO_4", pack=False, align=None),
    "DFS_INFO_4_32": SimStruct(OrderedDict((("EntryPath", SimTypeInt(signed=False, label="UInt32")), ("Comment", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")), ("Storage", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_4_32", pack=False, align=None),
    "DFS_INFO_5": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")), ("MetadataSize", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_5", pack=False, align=None),
    "DFS_INFO_50": SimStruct(OrderedDict((("NamespaceMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("NamespaceMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("NamespaceCapabilities", SimTypeLongLong(signed=False, label="UInt64")),)), name="DFS_INFO_50", pack=False, align=None),
    "DFS_INFO_6": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")), ("MetadataSize", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")), ("Storage", SimTypePointer(SimTypeRef("DFS_STORAGE_INFO_1", SimStruct), offset=0)),)), name="DFS_INFO_6", pack=False, align=None),
    "DFS_INFO_7": SimStruct(OrderedDict((("GenerationGuid", SimTypeBottom(label="Guid")),)), name="DFS_INFO_7", pack=False, align=None),
    "DFS_INFO_8": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")), ("MetadataSize", SimTypeInt(signed=False, label="UInt32")), ("SdLengthReserved", SimTypeInt(signed=False, label="UInt32")), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_INFO_8", pack=False, align=None),
    "DFS_INFO_9": SimStruct(OrderedDict((("EntryPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("PropertyFlags", SimTypeInt(signed=False, label="UInt32")), ("MetadataSize", SimTypeInt(signed=False, label="UInt32")), ("SdLengthReserved", SimTypeInt(signed=False, label="UInt32")), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfStorages", SimTypeInt(signed=False, label="UInt32")), ("Storage", SimTypePointer(SimTypeRef("DFS_STORAGE_INFO_1", SimStruct), offset=0)),)), name="DFS_INFO_9", pack=False, align=None),
    "DFS_NAMESPACE_VERSION_ORIGIN": SimTypeInt(signed=False, label="DFS_NAMESPACE_VERSION_ORIGIN"),
    "DFS_SITELIST_INFO": SimStruct(OrderedDict((("cSites", SimTypeInt(signed=False, label="UInt32")), ("Site", SimTypePointer(SimTypeRef("DFS_SITENAME_INFO", SimStruct), offset=0)),)), name="DFS_SITELIST_INFO", pack=False, align=None),
    "DFS_SITENAME_INFO": SimStruct(OrderedDict((("SiteFlags", SimTypeInt(signed=False, label="UInt32")), ("SiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_SITENAME_INFO", pack=False, align=None),
    "DFS_STORAGE_INFO": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")), ("ServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ShareName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DFS_STORAGE_INFO", pack=False, align=None),
    "DFS_STORAGE_INFO_0_32": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")), ("ServerName", SimTypeInt(signed=False, label="UInt32")), ("ShareName", SimTypeInt(signed=False, label="UInt32")),)), name="DFS_STORAGE_INFO_0_32", pack=False, align=None),
    "DFS_STORAGE_INFO_1": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")), ("ServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ShareName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TargetPriority", SimTypeRef("DFS_TARGET_PRIORITY", SimStruct)),)), name="DFS_STORAGE_INFO_1", pack=False, align=None),
    "DFS_SUPPORTED_NAMESPACE_VERSION_INFO": SimStruct(OrderedDict((("DomainDfsMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("DomainDfsMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("DomainDfsCapabilities", SimTypeLongLong(signed=False, label="UInt64")), ("StandaloneDfsMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("StandaloneDfsMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("StandaloneDfsCapabilities", SimTypeLongLong(signed=False, label="UInt64")),)), name="DFS_SUPPORTED_NAMESPACE_VERSION_INFO", pack=False, align=None),
    "DFS_TARGET_PRIORITY": SimStruct(OrderedDict((("TargetPriorityClass", SimTypeInt(signed=False, label="DFS_TARGET_PRIORITY_CLASS")), ("TargetPriorityRank", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="DFS_TARGET_PRIORITY", pack=False, align=None),
    "DFS_TARGET_PRIORITY_CLASS": SimTypeInt(signed=False, label="DFS_TARGET_PRIORITY_CLASS"),
    "DFileSystemImageEvents": SimTypeBottom(label="DFileSystemImageEvents"),
    "DFileSystemImageImportEvents": SimTypeBottom(label="DFileSystemImageImportEvents"),
    "DHCPAPI_PARAMS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("OptionId", SimTypeInt(signed=False, label="UInt32")), ("IsVendor", SimTypeInt(signed=True, label="Int32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nBytesData", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPAPI_PARAMS", pack=False, align=None),
    "DHCPCAPI_CLASSID": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nBytesData", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPCAPI_CLASSID", pack=False, align=None),
    "DHCPCAPI_PARAMS_ARRAY": SimStruct(OrderedDict((("nParams", SimTypeInt(signed=False, label="UInt32")), ("Params", SimTypePointer(SimTypeRef("DHCPAPI_PARAMS", SimStruct), offset=0)),)), name="DHCPCAPI_PARAMS_ARRAY", pack=False, align=None),
    "DHCPDS_SERVER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServerAddress", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("DsLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DsLocType", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPDS_SERVER", pack=False, align=None),
    "DHCPDS_SERVERS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Servers", SimTypePointer(SimTypeRef("DHCPDS_SERVER", SimStruct), offset=0)),)), name="DHCPDS_SERVERS", pack=False, align=None),
    "DHCPV4_FAILOVER_CLIENT_INFO": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="QuarantineStatus")), ("ProbationEnds", SimTypeRef("DATE_TIME", SimStruct)), ("QuarantineCapable", SimTypeInt(signed=True, label="Int32")), ("SentPotExpTime", SimTypeInt(signed=False, label="UInt32")), ("AckPotExpTime", SimTypeInt(signed=False, label="UInt32")), ("RecvPotExpTime", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("CltLastTransTime", SimTypeInt(signed=False, label="UInt32")), ("LastBndUpdTime", SimTypeInt(signed=False, label="UInt32")), ("BndMsgStatus", SimTypeInt(signed=False, label="UInt32")), ("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeChar(label="Byte")),)), name="DHCPV4_FAILOVER_CLIENT_INFO", pack=False, align=None),
    "DHCPV4_FAILOVER_CLIENT_INFO_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCPV4_FAILOVER_CLIENT_INFO", SimStruct), offset=0), offset=0)),)), name="DHCPV4_FAILOVER_CLIENT_INFO_ARRAY", pack=False, align=None),
    "DHCPV4_FAILOVER_CLIENT_INFO_EX": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="QuarantineStatus")), ("ProbationEnds", SimTypeRef("DATE_TIME", SimStruct)), ("QuarantineCapable", SimTypeInt(signed=True, label="Int32")), ("SentPotExpTime", SimTypeInt(signed=False, label="UInt32")), ("AckPotExpTime", SimTypeInt(signed=False, label="UInt32")), ("RecvPotExpTime", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("CltLastTransTime", SimTypeInt(signed=False, label="UInt32")), ("LastBndUpdTime", SimTypeInt(signed=False, label="UInt32")), ("BndMsgStatus", SimTypeInt(signed=False, label="UInt32")), ("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeChar(label="Byte")), ("AddressStateEx", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPV4_FAILOVER_CLIENT_INFO_EX", pack=False, align=None),
    "DHCPV6CAPI_CLASSID": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nBytesData", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPV6CAPI_CLASSID", pack=False, align=None),
    "DHCPV6CAPI_PARAMS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("OptionId", SimTypeInt(signed=False, label="UInt32")), ("IsVendor", SimTypeInt(signed=True, label="Int32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nBytesData", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPV6CAPI_PARAMS", pack=False, align=None),
    "DHCPV6CAPI_PARAMS_ARRAY": SimStruct(OrderedDict((("nParams", SimTypeInt(signed=False, label="UInt32")), ("Params", SimTypePointer(SimTypeRef("DHCPV6CAPI_PARAMS", SimStruct), offset=0)),)), name="DHCPV6CAPI_PARAMS_ARRAY", pack=False, align=None),
    "DHCPV6Prefix": SimStruct(OrderedDict((("prefix", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("prefixLength", SimTypeInt(signed=False, label="UInt32")), ("preferredLifeTime", SimTypeInt(signed=False, label="UInt32")), ("validLifeTime", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="StatusCode")),)), name="DHCPV6Prefix", pack=False, align=None),
    "DHCPV6PrefixLeaseInformation": SimStruct(OrderedDict((("nPrefixes", SimTypeInt(signed=False, label="UInt32")), ("prefixArray", SimTypePointer(SimTypeRef("DHCPV6Prefix", SimStruct), offset=0)), ("iaid", SimTypeInt(signed=False, label="UInt32")), ("T1", SimTypeLongLong(signed=True, label="Int64")), ("T2", SimTypeLongLong(signed=True, label="Int64")), ("MaxLeaseExpirationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastRenewalTime", SimTypeLongLong(signed=True, label="Int64")), ("status", SimTypeInt(signed=False, label="StatusCode")), ("ServerId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerIdLen", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPV6PrefixLeaseInformation", pack=False, align=None),
    "DHCPV6_BIND_ELEMENT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("fBoundToDHCPServer", SimTypeInt(signed=True, label="Int32")), ("AdapterPrimaryAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("AdapterSubnetAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("IfDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IpV6IfIndex", SimTypeInt(signed=False, label="UInt32")), ("IfIdSize", SimTypeInt(signed=False, label="UInt32")), ("IfId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DHCPV6_BIND_ELEMENT", pack=False, align=None),
    "DHCPV6_BIND_ELEMENT_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCPV6_BIND_ELEMENT", SimStruct), offset=0)),)), name="DHCPV6_BIND_ELEMENT_ARRAY", pack=False, align=None),
    "DHCPV6_IP_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct), offset=0)),)), name="DHCPV6_IP_ARRAY", pack=False, align=None),
    "DHCPV6_STATELESS_PARAMS": SimStruct(OrderedDict((("Status", SimTypeInt(signed=True, label="Int32")), ("PurgeInterval", SimTypeInt(signed=False, label="UInt32")),)), name="DHCPV6_STATELESS_PARAMS", pack=False, align=None),
    "DHCPV6_STATELESS_PARAM_TYPE": SimTypeInt(signed=False, label="DHCPV6_STATELESS_PARAM_TYPE"),
    "DHCPV6_STATELESS_SCOPE_STATS": SimStruct(OrderedDict((("SubnetAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("NumStatelessClientsAdded", SimTypeLongLong(signed=False, label="UInt64")), ("NumStatelessClientsRemoved", SimTypeLongLong(signed=False, label="UInt64")),)), name="DHCPV6_STATELESS_SCOPE_STATS", pack=False, align=None),
    "DHCPV6_STATELESS_STATS": SimStruct(OrderedDict((("NumScopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeStats", SimTypePointer(SimTypeRef("DHCPV6_STATELESS_SCOPE_STATS", SimStruct), offset=0)),)), name="DHCPV6_STATELESS_STATS", pack=False, align=None),
    "DHCP_ADDR_PATTERN": SimStruct(OrderedDict((("MatchHWType", SimTypeInt(signed=True, label="Int32")), ("HWType", SimTypeChar(label="Byte")), ("IsWildcard", SimTypeInt(signed=True, label="Int32")), ("Length", SimTypeChar(label="Byte")), ("Pattern", SimTypeArray(SimTypeChar(label="Byte"), 255)),)), name="DHCP_ADDR_PATTERN", pack=False, align=None),
    "DHCP_ALL_OPTIONS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NonVendorOptions", SimTypePointer(SimTypeRef("DHCP_OPTION_ARRAY", SimStruct), offset=0)), ("NumVendorOptions", SimTypeInt(signed=False, label="UInt32")), ("VendorOptions", SimTypePointer(SimStruct(OrderedDict((("Option", SimTypeRef("DHCP_OPTION", SimStruct)), ("VendorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="DHCP_ALL_OPTIONS", pack=False, align=None),
    "DHCP_ALL_OPTION_VALUES": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Options", SimTypePointer(SimStruct(OrderedDict((("ClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("VendorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsVendor", SimTypeInt(signed=True, label="Int32")), ("OptionsArray", SimTypePointer(SimTypeRef("DHCP_OPTION_VALUE_ARRAY", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="DHCP_ALL_OPTION_VALUES", pack=False, align=None),
    "DHCP_ALL_OPTION_VALUES_PB": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Options", SimTypePointer(SimStruct(OrderedDict((("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("VendorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsVendor", SimTypeInt(signed=True, label="Int32")), ("OptionsArray", SimTypePointer(SimTypeRef("DHCP_OPTION_VALUE_ARRAY", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="DHCP_ALL_OPTION_VALUES_PB", pack=False, align=None),
    "DHCP_ATTRIB": SimStruct(OrderedDict((("DhcpAttribId", SimTypeInt(signed=False, label="UInt32")), ("DhcpAttribType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"DhcpAttribBool": SimTypeInt(signed=True, label="Int32"), "DhcpAttribUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DHCP_ATTRIB", pack=False, align=None),
    "DHCP_ATTRIB_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("DhcpAttribs", SimTypePointer(SimTypeRef("DHCP_ATTRIB", SimStruct), offset=0)),)), name="DHCP_ATTRIB_ARRAY", pack=False, align=None),
    "DHCP_BINARY_DATA": SimStruct(OrderedDict((("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DHCP_BINARY_DATA", pack=False, align=None),
    "DHCP_BIND_ELEMENT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("fBoundToDHCPServer", SimTypeInt(signed=True, label="Int32")), ("AdapterPrimaryAddress", SimTypeInt(signed=False, label="UInt32")), ("AdapterSubnetAddress", SimTypeInt(signed=False, label="UInt32")), ("IfDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IfIdSize", SimTypeInt(signed=False, label="UInt32")), ("IfId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DHCP_BIND_ELEMENT", pack=False, align=None),
    "DHCP_BIND_ELEMENT_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_BIND_ELEMENT", SimStruct), offset=0)),)), name="DHCP_BIND_ELEMENT_ARRAY", pack=False, align=None),
    "DHCP_BOOTP_IP_RANGE": SimStruct(OrderedDict((("StartAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")), ("BootpAllocated", SimTypeInt(signed=False, label="UInt32")), ("MaxBootpAllowed", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_BOOTP_IP_RANGE", pack=False, align=None),
    "DHCP_CALLOUT_TABLE": SimStruct(OrderedDict((("DhcpControlHook", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwControlCode", "lpReserved"]), offset=0)), ("DhcpNewPktHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "IpAddress", "Reserved", "PktContext", "ProcessIt"]), offset=0)), ("DhcpPktDropHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "Reserved", "PktContext"]), offset=0)), ("DhcpPktSendHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "Reserved", "PktContext"]), offset=0)), ("DhcpAddressDelHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "AltAddress", "Reserved", "PktContext"]), offset=0)), ("DhcpAddressOfferHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "AltAddress", "AddrType", "LeaseTime", "Reserved", "PktContext"]), offset=0)), ("DhcpHandleOptionsHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DHCP_SERVER_OPTIONS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "Reserved", "PktContext", "ServerOptions"]), offset=0)), ("DhcpDeleteClientHook", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["IpAddress", "HwAddress", "HwAddressLength", "Reserved", "ClientType"]), offset=0)), ("DhcpExtensionHook", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DhcpReservedHook", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DHCP_CALLOUT_TABLE", pack=False, align=None),
    "DHCP_CLASS_INFO": SimStruct(OrderedDict((("ClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClassComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClassDataLength", SimTypeInt(signed=False, label="UInt32")), ("IsVendor", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ClassData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DHCP_CLASS_INFO", pack=False, align=None),
    "DHCP_CLASS_INFO_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Classes", SimTypePointer(SimTypeRef("DHCP_CLASS_INFO", SimStruct), offset=0)),)), name="DHCP_CLASS_INFO_ARRAY", pack=False, align=None),
    "DHCP_CLASS_INFO_ARRAY_V6": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Classes", SimTypePointer(SimTypeRef("DHCP_CLASS_INFO_V6", SimStruct), offset=0)),)), name="DHCP_CLASS_INFO_ARRAY_V6", pack=False, align=None),
    "DHCP_CLASS_INFO_V6": SimStruct(OrderedDict((("ClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClassComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClassDataLength", SimTypeInt(signed=False, label="UInt32")), ("IsVendor", SimTypeInt(signed=True, label="Int32")), ("EnterpriseNumber", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ClassData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DHCP_CLASS_INFO_V6", pack=False, align=None),
    "DHCP_CLIENT_FILTER_STATUS_INFO": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="QuarantineStatus")), ("ProbationEnds", SimTypeRef("DATE_TIME", SimStruct)), ("QuarantineCapable", SimTypeInt(signed=True, label="Int32")), ("FilterStatus", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_CLIENT_FILTER_STATUS_INFO", pack=False, align=None),
    "DHCP_CLIENT_FILTER_STATUS_INFO_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_FILTER_STATUS_INFO", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_FILTER_STATUS_INFO_ARRAY", pack=False, align=None),
    "DHCP_CLIENT_INFO": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)),)), name="DHCP_CLIENT_INFO", pack=False, align=None),
    "DHCP_CLIENT_INFO_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_ARRAY", pack=False, align=None),
    "DHCP_CLIENT_INFO_ARRAY_V4": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO_V4", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_ARRAY_V4", pack=False, align=None),
    "DHCP_CLIENT_INFO_ARRAY_V5": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO_V5", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_ARRAY_V5", pack=False, align=None),
    "DHCP_CLIENT_INFO_ARRAY_V6": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO_V6", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_ARRAY_V6", pack=False, align=None),
    "DHCP_CLIENT_INFO_ARRAY_VQ": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO_VQ", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_ARRAY_VQ", pack=False, align=None),
    "DHCP_CLIENT_INFO_EX": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="QuarantineStatus")), ("ProbationEnds", SimTypeRef("DATE_TIME", SimStruct)), ("QuarantineCapable", SimTypeInt(signed=True, label="Int32")), ("FilterStatus", SimTypeInt(signed=False, label="UInt32")), ("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Properties", SimTypePointer(SimTypeRef("DHCP_PROPERTY_ARRAY", SimStruct), offset=0)),)), name="DHCP_CLIENT_INFO_EX", pack=False, align=None),
    "DHCP_CLIENT_INFO_EX_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO_EX", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_EX_ARRAY", pack=False, align=None),
    "DHCP_CLIENT_INFO_PB": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="QuarantineStatus")), ("ProbationEnds", SimTypeRef("DATE_TIME", SimStruct)), ("QuarantineCapable", SimTypeInt(signed=True, label="Int32")), ("FilterStatus", SimTypeInt(signed=False, label="UInt32")), ("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_CLIENT_INFO_PB", pack=False, align=None),
    "DHCP_CLIENT_INFO_PB_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Clients", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_CLIENT_INFO_PB", SimStruct), offset=0), offset=0)),)), name="DHCP_CLIENT_INFO_PB_ARRAY", pack=False, align=None),
    "DHCP_CLIENT_INFO_V4": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")),)), name="DHCP_CLIENT_INFO_V4", pack=False, align=None),
    "DHCP_CLIENT_INFO_V5": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")),)), name="DHCP_CLIENT_INFO_V5", pack=False, align=None),
    "DHCP_CLIENT_INFO_V6": SimStruct(OrderedDict((("ClientIpAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("ClientDUID", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("AddressType", SimTypeInt(signed=False, label="UInt32")), ("IAID", SimTypeInt(signed=False, label="UInt32")), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientValidLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("ClientPrefLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO_V6", SimStruct)),)), name="DHCP_CLIENT_INFO_V6", pack=False, align=None),
    "DHCP_CLIENT_INFO_VQ": SimStruct(OrderedDict((("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddress", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientLeaseExpires", SimTypeRef("DATE_TIME", SimStruct)), ("OwnerHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("bClientType", SimTypeChar(label="Byte")), ("AddressState", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="QuarantineStatus")), ("ProbationEnds", SimTypeRef("DATE_TIME", SimStruct)), ("QuarantineCapable", SimTypeInt(signed=True, label="Int32")),)), name="DHCP_CLIENT_INFO_VQ", pack=False, align=None),
    "DHCP_CLIENT_SEARCH_UNION": SimUnion({"ClientIpAddress": SimTypeInt(signed=False, label="UInt32"), "ClientHardwareAddress": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "ClientName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "DHCP_FAILOVER_MODE": SimTypeInt(signed=False, label="DHCP_FAILOVER_MODE"),
    "DHCP_FAILOVER_RELATIONSHIP": SimStruct(OrderedDict((("PrimaryServer", SimTypeInt(signed=False, label="UInt32")), ("SecondaryServer", SimTypeInt(signed=False, label="UInt32")), ("Mode", SimTypeInt(signed=False, label="DHCP_FAILOVER_MODE")), ("ServerType", SimTypeInt(signed=False, label="DHCP_FAILOVER_SERVER")), ("State", SimTypeInt(signed=False, label="FSM_STATE")), ("PrevState", SimTypeInt(signed=False, label="FSM_STATE")), ("Mclt", SimTypeInt(signed=False, label="UInt32")), ("SafePeriod", SimTypeInt(signed=False, label="UInt32")), ("RelationshipName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PrimaryServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SecondaryServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pScopes", SimTypePointer(SimTypeRef("DHCP_IP_ARRAY", SimStruct), offset=0)), ("Percentage", SimTypeChar(label="Byte")), ("SharedSecret", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_FAILOVER_RELATIONSHIP", pack=False, align=None),
    "DHCP_FAILOVER_RELATIONSHIP_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("pRelationships", SimTypePointer(SimTypeRef("DHCP_FAILOVER_RELATIONSHIP", SimStruct), offset=0)),)), name="DHCP_FAILOVER_RELATIONSHIP_ARRAY", pack=False, align=None),
    "DHCP_FAILOVER_SERVER": SimTypeInt(signed=False, label="DHCP_FAILOVER_SERVER"),
    "DHCP_FAILOVER_STATISTICS": SimStruct(OrderedDict((("NumAddr", SimTypeInt(signed=False, label="UInt32")), ("AddrFree", SimTypeInt(signed=False, label="UInt32")), ("AddrInUse", SimTypeInt(signed=False, label="UInt32")), ("PartnerAddrFree", SimTypeInt(signed=False, label="UInt32")), ("ThisAddrFree", SimTypeInt(signed=False, label="UInt32")), ("PartnerAddrInUse", SimTypeInt(signed=False, label="UInt32")), ("ThisAddrInUse", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_FAILOVER_STATISTICS", pack=False, align=None),
    "DHCP_FILTER_ADD_INFO": SimStruct(OrderedDict((("AddrPatt", SimTypeRef("DHCP_ADDR_PATTERN", SimStruct)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ListType", SimTypeInt(signed=False, label="DHCP_FILTER_LIST_TYPE")),)), name="DHCP_FILTER_ADD_INFO", pack=False, align=None),
    "DHCP_FILTER_ENUM_INFO": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("pEnumRecords", SimTypePointer(SimTypeRef("DHCP_FILTER_RECORD", SimStruct), offset=0)),)), name="DHCP_FILTER_ENUM_INFO", pack=False, align=None),
    "DHCP_FILTER_GLOBAL_INFO": SimStruct(OrderedDict((("EnforceAllowList", SimTypeInt(signed=True, label="Int32")), ("EnforceDenyList", SimTypeInt(signed=True, label="Int32")),)), name="DHCP_FILTER_GLOBAL_INFO", pack=False, align=None),
    "DHCP_FILTER_LIST_TYPE": SimTypeInt(signed=False, label="DHCP_FILTER_LIST_TYPE"),
    "DHCP_FILTER_RECORD": SimStruct(OrderedDict((("AddrPatt", SimTypeRef("DHCP_ADDR_PATTERN", SimStruct)), ("Comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_FILTER_RECORD", pack=False, align=None),
    "DHCP_FORCE_FLAG": SimTypeInt(signed=False, label="DHCP_FORCE_FLAG"),
    "DHCP_HOST_INFO": SimStruct(OrderedDict((("IpAddress", SimTypeInt(signed=False, label="UInt32")), ("NetBiosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_HOST_INFO", pack=False, align=None),
    "DHCP_HOST_INFO_V6": SimStruct(OrderedDict((("IpAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("NetBiosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_HOST_INFO_V6", pack=False, align=None),
    "DHCP_IPV6_ADDRESS": SimStruct(OrderedDict((("HighOrderBits", SimTypeLongLong(signed=False, label="UInt64")), ("LowOrderBits", SimTypeLongLong(signed=False, label="UInt64")),)), name="DHCP_IPV6_ADDRESS", pack=False, align=None),
    "DHCP_IP_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)),)), name="DHCP_IP_ARRAY", pack=False, align=None),
    "DHCP_IP_CLUSTER": SimStruct(OrderedDict((("ClusterAddress", SimTypeInt(signed=False, label="UInt32")), ("ClusterMask", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_IP_CLUSTER", pack=False, align=None),
    "DHCP_IP_RANGE": SimStruct(OrderedDict((("StartAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_IP_RANGE", pack=False, align=None),
    "DHCP_IP_RANGE_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0)),)), name="DHCP_IP_RANGE_ARRAY", pack=False, align=None),
    "DHCP_IP_RANGE_V6": SimStruct(OrderedDict((("StartAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("EndAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)),)), name="DHCP_IP_RANGE_V6", pack=False, align=None),
    "DHCP_IP_RESERVATION": SimStruct(OrderedDict((("ReservedIpAddress", SimTypeInt(signed=False, label="UInt32")), ("ReservedForClient", SimTypePointer(SimTypeRef("DHCP_BINARY_DATA", SimStruct), offset=0)),)), name="DHCP_IP_RESERVATION", pack=False, align=None),
    "DHCP_IP_RESERVATION_INFO": SimStruct(OrderedDict((("ReservedIpAddress", SimTypeInt(signed=False, label="UInt32")), ("ReservedForClient", SimTypeRef("DHCP_BINARY_DATA", SimStruct)), ("ReservedClientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReservedClientDesc", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bAllowedClientTypes", SimTypeChar(label="Byte")), ("fOptionsPresent", SimTypeChar(label="Byte")),)), name="DHCP_IP_RESERVATION_INFO", pack=False, align=None),
    "DHCP_IP_RESERVATION_V4": SimStruct(OrderedDict((("ReservedIpAddress", SimTypeInt(signed=False, label="UInt32")), ("ReservedForClient", SimTypePointer(SimTypeRef("DHCP_BINARY_DATA", SimStruct), offset=0)), ("bAllowedClientTypes", SimTypeChar(label="Byte")),)), name="DHCP_IP_RESERVATION_V4", pack=False, align=None),
    "DHCP_IP_RESERVATION_V6": SimStruct(OrderedDict((("ReservedIpAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("ReservedForClient", SimTypePointer(SimTypeRef("DHCP_BINARY_DATA", SimStruct), offset=0)), ("InterfaceId", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_IP_RESERVATION_V6", pack=False, align=None),
    "DHCP_MIB_INFO": SimStruct(OrderedDict((("Discovers", SimTypeInt(signed=False, label="UInt32")), ("Offers", SimTypeInt(signed=False, label="UInt32")), ("Requests", SimTypeInt(signed=False, label="UInt32")), ("Acks", SimTypeInt(signed=False, label="UInt32")), ("Naks", SimTypeInt(signed=False, label="UInt32")), ("Declines", SimTypeInt(signed=False, label="UInt32")), ("Releases", SimTypeInt(signed=False, label="UInt32")), ("ServerStartTime", SimTypeRef("DATE_TIME", SimStruct)), ("Scopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeInfo", SimTypePointer(SimTypeRef("SCOPE_MIB_INFO", SimStruct), offset=0)),)), name="DHCP_MIB_INFO", pack=False, align=None),
    "DHCP_MIB_INFO_V5": SimStruct(OrderedDict((("Discovers", SimTypeInt(signed=False, label="UInt32")), ("Offers", SimTypeInt(signed=False, label="UInt32")), ("Requests", SimTypeInt(signed=False, label="UInt32")), ("Acks", SimTypeInt(signed=False, label="UInt32")), ("Naks", SimTypeInt(signed=False, label="UInt32")), ("Declines", SimTypeInt(signed=False, label="UInt32")), ("Releases", SimTypeInt(signed=False, label="UInt32")), ("ServerStartTime", SimTypeRef("DATE_TIME", SimStruct)), ("QtnNumLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnPctQtnLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnProbationLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnNonQtnLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnExemptLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnCapableClients", SimTypeInt(signed=False, label="UInt32")), ("QtnIASErrors", SimTypeInt(signed=False, label="UInt32")), ("DelayedOffers", SimTypeInt(signed=False, label="UInt32")), ("ScopesWithDelayedOffers", SimTypeInt(signed=False, label="UInt32")), ("Scopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeInfo", SimTypePointer(SimTypeRef("SCOPE_MIB_INFO_V5", SimStruct), offset=0)),)), name="DHCP_MIB_INFO_V5", pack=False, align=None),
    "DHCP_MIB_INFO_V6": SimStruct(OrderedDict((("Solicits", SimTypeInt(signed=False, label="UInt32")), ("Advertises", SimTypeInt(signed=False, label="UInt32")), ("Requests", SimTypeInt(signed=False, label="UInt32")), ("Renews", SimTypeInt(signed=False, label="UInt32")), ("Rebinds", SimTypeInt(signed=False, label="UInt32")), ("Replies", SimTypeInt(signed=False, label="UInt32")), ("Confirms", SimTypeInt(signed=False, label="UInt32")), ("Declines", SimTypeInt(signed=False, label="UInt32")), ("Releases", SimTypeInt(signed=False, label="UInt32")), ("Informs", SimTypeInt(signed=False, label="UInt32")), ("ServerStartTime", SimTypeRef("DATE_TIME", SimStruct)), ("Scopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeInfo", SimTypePointer(SimTypeRef("SCOPE_MIB_INFO_V6", SimStruct), offset=0)),)), name="DHCP_MIB_INFO_V6", pack=False, align=None),
    "DHCP_MIB_INFO_VQ": SimStruct(OrderedDict((("Discovers", SimTypeInt(signed=False, label="UInt32")), ("Offers", SimTypeInt(signed=False, label="UInt32")), ("Requests", SimTypeInt(signed=False, label="UInt32")), ("Acks", SimTypeInt(signed=False, label="UInt32")), ("Naks", SimTypeInt(signed=False, label="UInt32")), ("Declines", SimTypeInt(signed=False, label="UInt32")), ("Releases", SimTypeInt(signed=False, label="UInt32")), ("ServerStartTime", SimTypeRef("DATE_TIME", SimStruct)), ("QtnNumLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnPctQtnLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnProbationLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnNonQtnLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnExemptLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnCapableClients", SimTypeInt(signed=False, label="UInt32")), ("QtnIASErrors", SimTypeInt(signed=False, label="UInt32")), ("Scopes", SimTypeInt(signed=False, label="UInt32")), ("ScopeInfo", SimTypePointer(SimTypeRef("SCOPE_MIB_INFO_VQ", SimStruct), offset=0)),)), name="DHCP_MIB_INFO_VQ", pack=False, align=None),
    "DHCP_OPTION": SimStruct(OrderedDict((("OptionID", SimTypeInt(signed=False, label="UInt32")), ("OptionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("OptionComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DefaultValue", SimTypeRef("DHCP_OPTION_DATA", SimStruct)), ("OptionType", SimTypeInt(signed=False, label="DHCP_OPTION_TYPE")),)), name="DHCP_OPTION", pack=False, align=None),
    "DHCP_OPTION_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Options", SimTypePointer(SimTypeRef("DHCP_OPTION", SimStruct), offset=0)),)), name="DHCP_OPTION_ARRAY", pack=False, align=None),
    "DHCP_OPTION_DATA": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_OPTION_DATA_ELEMENT", SimStruct), offset=0)),)), name="DHCP_OPTION_DATA", pack=False, align=None),
    "DHCP_OPTION_DATA_ELEMENT": SimStruct(OrderedDict((("OptionType", SimTypeInt(signed=False, label="DHCP_OPTION_DATA_TYPE")), ("Element", SimUnion({"ByteOption": SimTypeChar(label="Byte"), "WordOption": SimTypeShort(signed=False, label="UInt16"), "DWordOption": SimTypeInt(signed=False, label="UInt32"), "DWordDWordOption": SimTypeRef("DWORD_DWORD", SimStruct), "IpAddressOption": SimTypeInt(signed=False, label="UInt32"), "StringDataOption": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryDataOption": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "EncapsulatedDataOption": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "Ipv6AddressDataOption": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="DHCP_OPTION_DATA_ELEMENT", pack=False, align=None),
    "DHCP_OPTION_DATA_TYPE": SimTypeInt(signed=False, label="DHCP_OPTION_DATA_TYPE"),
    "DHCP_OPTION_ELEMENT_UNION": SimUnion({"ByteOption": SimTypeChar(label="Byte"), "WordOption": SimTypeShort(signed=False, label="UInt16"), "DWordOption": SimTypeInt(signed=False, label="UInt32"), "DWordDWordOption": SimTypeRef("DWORD_DWORD", SimStruct), "IpAddressOption": SimTypeInt(signed=False, label="UInt32"), "StringDataOption": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryDataOption": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "EncapsulatedDataOption": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "Ipv6AddressDataOption": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "DHCP_OPTION_LIST": SimStruct(OrderedDict((("NumOptions", SimTypeInt(signed=False, label="UInt32")), ("Options", SimTypePointer(SimTypeRef("DHCP_OPTION_VALUE", SimStruct), offset=0)),)), name="DHCP_OPTION_LIST", pack=False, align=None),
    "DHCP_OPTION_SCOPE_INFO": SimStruct(OrderedDict((("ScopeType", SimTypeInt(signed=False, label="DHCP_OPTION_SCOPE_TYPE")), ("ScopeInfo", SimUnion({"DefaultScopeInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "GlobalScopeInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "SubnetScopeInfo": SimTypeInt(signed=False, label="UInt32"), "ReservedScopeInfo": SimTypeRef("DHCP_RESERVED_SCOPE", SimStruct), "MScopeInfo": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="DHCP_OPTION_SCOPE_INFO", pack=False, align=None),
    "DHCP_OPTION_SCOPE_INFO6": SimStruct(OrderedDict((("ScopeType", SimTypeInt(signed=False, label="DHCP_OPTION_SCOPE_TYPE6")), ("ScopeInfo", SimUnion({"DefaultScopeInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "SubnetScopeInfo": SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct), "ReservedScopeInfo": SimTypeRef("DHCP_RESERVED_SCOPE6", SimStruct)}, name="<anon>", label="None")),)), name="DHCP_OPTION_SCOPE_INFO6", pack=False, align=None),
    "DHCP_OPTION_SCOPE_TYPE": SimTypeInt(signed=False, label="DHCP_OPTION_SCOPE_TYPE"),
    "DHCP_OPTION_SCOPE_TYPE6": SimTypeInt(signed=False, label="DHCP_OPTION_SCOPE_TYPE6"),
    "DHCP_OPTION_SCOPE_UNION6": SimUnion({"DefaultScopeInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "SubnetScopeInfo": SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct), "ReservedScopeInfo": SimTypeRef("DHCP_RESERVED_SCOPE6", SimStruct)}, name="<anon>", label="None"),
    "DHCP_OPTION_TYPE": SimTypeInt(signed=False, label="DHCP_OPTION_TYPE"),
    "DHCP_OPTION_VALUE": SimStruct(OrderedDict((("OptionID", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeRef("DHCP_OPTION_DATA", SimStruct)),)), name="DHCP_OPTION_VALUE", pack=False, align=None),
    "DHCP_OPTION_VALUE_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Values", SimTypePointer(SimTypeRef("DHCP_OPTION_VALUE", SimStruct), offset=0)),)), name="DHCP_OPTION_VALUE_ARRAY", pack=False, align=None),
    "DHCP_PERF_STATS": SimStruct(OrderedDict((("dwNumPacketsReceived", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsDuplicate", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsExpired", SimTypeInt(signed=False, label="UInt32")), ("dwNumMilliSecondsProcessed", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsInActiveQueue", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsInPingQueue", SimTypeInt(signed=False, label="UInt32")), ("dwNumDiscoversReceived", SimTypeInt(signed=False, label="UInt32")), ("dwNumOffersSent", SimTypeInt(signed=False, label="UInt32")), ("dwNumRequestsReceived", SimTypeInt(signed=False, label="UInt32")), ("dwNumInformsReceived", SimTypeInt(signed=False, label="UInt32")), ("dwNumAcksSent", SimTypeInt(signed=False, label="UInt32")), ("dwNumNacksSent", SimTypeInt(signed=False, label="UInt32")), ("dwNumDeclinesReceived", SimTypeInt(signed=False, label="UInt32")), ("dwNumReleasesReceived", SimTypeInt(signed=False, label="UInt32")), ("dwNumDelayedOfferInQueue", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsProcessed", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsInQuarWaitingQueue", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsInQuarReadyQueue", SimTypeInt(signed=False, label="UInt32")), ("dwNumPacketsInQuarDecisionQueue", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_PERF_STATS", pack=False, align=None),
    "DHCP_POLICY": SimStruct(OrderedDict((("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsGlobalPolicy", SimTypeInt(signed=True, label="Int32")), ("Subnet", SimTypeInt(signed=False, label="UInt32")), ("ProcessingOrder", SimTypeInt(signed=False, label="UInt32")), ("Conditions", SimTypePointer(SimTypeRef("DHCP_POL_COND_ARRAY", SimStruct), offset=0)), ("Expressions", SimTypePointer(SimTypeRef("DHCP_POL_EXPR_ARRAY", SimStruct), offset=0)), ("Ranges", SimTypePointer(SimTypeRef("DHCP_IP_RANGE_ARRAY", SimStruct), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Enabled", SimTypeInt(signed=True, label="Int32")),)), name="DHCP_POLICY", pack=False, align=None),
    "DHCP_POLICY_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_POLICY", SimStruct), offset=0)),)), name="DHCP_POLICY_ARRAY", pack=False, align=None),
    "DHCP_POLICY_EX": SimStruct(OrderedDict((("PolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsGlobalPolicy", SimTypeInt(signed=True, label="Int32")), ("Subnet", SimTypeInt(signed=False, label="UInt32")), ("ProcessingOrder", SimTypeInt(signed=False, label="UInt32")), ("Conditions", SimTypePointer(SimTypeRef("DHCP_POL_COND_ARRAY", SimStruct), offset=0)), ("Expressions", SimTypePointer(SimTypeRef("DHCP_POL_EXPR_ARRAY", SimStruct), offset=0)), ("Ranges", SimTypePointer(SimTypeRef("DHCP_IP_RANGE_ARRAY", SimStruct), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("Properties", SimTypePointer(SimTypeRef("DHCP_PROPERTY_ARRAY", SimStruct), offset=0)),)), name="DHCP_POLICY_EX", pack=False, align=None),
    "DHCP_POLICY_EX_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_POLICY_EX", SimStruct), offset=0)),)), name="DHCP_POLICY_EX_ARRAY", pack=False, align=None),
    "DHCP_POLICY_FIELDS_TO_UPDATE": SimTypeInt(signed=False, label="DHCP_POLICY_FIELDS_TO_UPDATE"),
    "DHCP_POL_ATTR_TYPE": SimTypeInt(signed=False, label="DHCP_POL_ATTR_TYPE"),
    "DHCP_POL_COMPARATOR": SimTypeInt(signed=False, label="DHCP_POL_COMPARATOR"),
    "DHCP_POL_COND": SimStruct(OrderedDict((("ParentExpr", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="DHCP_POL_ATTR_TYPE")), ("OptionID", SimTypeInt(signed=False, label="UInt32")), ("SubOptionID", SimTypeInt(signed=False, label="UInt32")), ("VendorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Operator", SimTypeInt(signed=False, label="DHCP_POL_COMPARATOR")), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ValueLength", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_POL_COND", pack=False, align=None),
    "DHCP_POL_COND_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_POL_COND", SimStruct), offset=0)),)), name="DHCP_POL_COND_ARRAY", pack=False, align=None),
    "DHCP_POL_EXPR": SimStruct(OrderedDict((("ParentExpr", SimTypeInt(signed=False, label="UInt32")), ("Operator", SimTypeInt(signed=False, label="DHCP_POL_LOGIC_OPER")),)), name="DHCP_POL_EXPR", pack=False, align=None),
    "DHCP_POL_EXPR_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_POL_EXPR", SimStruct), offset=0)),)), name="DHCP_POL_EXPR_ARRAY", pack=False, align=None),
    "DHCP_POL_LOGIC_OPER": SimTypeInt(signed=False, label="DHCP_POL_LOGIC_OPER"),
    "DHCP_PROPERTY": SimStruct(OrderedDict((("ID", SimTypeInt(signed=False, label="DHCP_PROPERTY_ID")), ("Type", SimTypeInt(signed=False, label="DHCP_PROPERTY_TYPE")), ("Value", SimUnion({"ByteValue": SimTypeChar(label="Byte"), "WordValue": SimTypeShort(signed=False, label="UInt16"), "DWordValue": SimTypeInt(signed=False, label="UInt32"), "StringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryValue": SimTypeRef("DHCP_BINARY_DATA", SimStruct)}, name="<anon>", label="None")),)), name="DHCP_PROPERTY", pack=False, align=None),
    "DHCP_PROPERTY_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_PROPERTY", SimStruct), offset=0)),)), name="DHCP_PROPERTY_ARRAY", pack=False, align=None),
    "DHCP_PROPERTY_ID": SimTypeInt(signed=False, label="DHCP_PROPERTY_ID"),
    "DHCP_PROPERTY_TYPE": SimTypeInt(signed=False, label="DHCP_PROPERTY_TYPE"),
    "DHCP_RESERVATION_INFO_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_INFO", SimStruct), offset=0), offset=0)),)), name="DHCP_RESERVATION_INFO_ARRAY", pack=False, align=None),
    "DHCP_RESERVED_SCOPE": SimStruct(OrderedDict((("ReservedIpAddress", SimTypeInt(signed=False, label="UInt32")), ("ReservedIpSubnetAddress", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_RESERVED_SCOPE", pack=False, align=None),
    "DHCP_RESERVED_SCOPE6": SimStruct(OrderedDict((("ReservedIpAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("ReservedIpSubnetAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)),)), name="DHCP_RESERVED_SCOPE6", pack=False, align=None),
    "DHCP_SCAN_FLAG": SimTypeInt(signed=False, label="DHCP_SCAN_FLAG"),
    "DHCP_SCAN_ITEM": SimStruct(OrderedDict((("IpAddress", SimTypeInt(signed=False, label="UInt32")), ("ScanFlag", SimTypeInt(signed=False, label="DHCP_SCAN_FLAG")),)), name="DHCP_SCAN_ITEM", pack=False, align=None),
    "DHCP_SCAN_LIST": SimStruct(OrderedDict((("NumScanItems", SimTypeInt(signed=False, label="UInt32")), ("ScanItems", SimTypePointer(SimTypeRef("DHCP_SCAN_ITEM", SimStruct), offset=0)),)), name="DHCP_SCAN_LIST", pack=False, align=None),
    "DHCP_SEARCH_INFO": SimStruct(OrderedDict((("SearchType", SimTypeInt(signed=False, label="DHCP_SEARCH_INFO_TYPE")), ("SearchInfo", SimUnion({"ClientIpAddress": SimTypeInt(signed=False, label="UInt32"), "ClientHardwareAddress": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "ClientName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="DHCP_SEARCH_INFO", pack=False, align=None),
    "DHCP_SEARCH_INFO_TYPE": SimTypeInt(signed=False, label="DHCP_SEARCH_INFO_TYPE"),
    "DHCP_SEARCH_INFO_TYPE_V6": SimTypeInt(signed=False, label="DHCP_SEARCH_INFO_TYPE_V6"),
    "DHCP_SEARCH_INFO_V6": SimStruct(OrderedDict((("SearchType", SimTypeInt(signed=False, label="DHCP_SEARCH_INFO_TYPE_V6")), ("SearchInfo", SimUnion({"ClientIpAddress": SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct), "ClientDUID": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "ClientName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="DHCP_SEARCH_INFO_V6", pack=False, align=None),
    "DHCP_SERVER_CONFIG_INFO": SimStruct(OrderedDict((("APIProtocolSupport", SimTypeInt(signed=False, label="UInt32")), ("DatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DatabasePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackupPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackupInterval", SimTypeInt(signed=False, label="UInt32")), ("DatabaseLoggingFlag", SimTypeInt(signed=False, label="UInt32")), ("RestoreFlag", SimTypeInt(signed=False, label="UInt32")), ("DatabaseCleanupInterval", SimTypeInt(signed=False, label="UInt32")), ("DebugFlag", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_SERVER_CONFIG_INFO", pack=False, align=None),
    "DHCP_SERVER_CONFIG_INFO_V4": SimStruct(OrderedDict((("APIProtocolSupport", SimTypeInt(signed=False, label="UInt32")), ("DatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DatabasePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackupPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackupInterval", SimTypeInt(signed=False, label="UInt32")), ("DatabaseLoggingFlag", SimTypeInt(signed=False, label="UInt32")), ("RestoreFlag", SimTypeInt(signed=False, label="UInt32")), ("DatabaseCleanupInterval", SimTypeInt(signed=False, label="UInt32")), ("DebugFlag", SimTypeInt(signed=False, label="UInt32")), ("dwPingRetries", SimTypeInt(signed=False, label="UInt32")), ("cbBootTableString", SimTypeInt(signed=False, label="UInt32")), ("wszBootTableString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fAuditLog", SimTypeInt(signed=True, label="Int32")),)), name="DHCP_SERVER_CONFIG_INFO_V4", pack=False, align=None),
    "DHCP_SERVER_CONFIG_INFO_V6": SimStruct(OrderedDict((("UnicastFlag", SimTypeInt(signed=True, label="Int32")), ("RapidCommitFlag", SimTypeInt(signed=True, label="Int32")), ("PreferredLifetime", SimTypeInt(signed=False, label="UInt32")), ("ValidLifetime", SimTypeInt(signed=False, label="UInt32")), ("T1", SimTypeInt(signed=False, label="UInt32")), ("T2", SimTypeInt(signed=False, label="UInt32")), ("PreferredLifetimeIATA", SimTypeInt(signed=False, label="UInt32")), ("ValidLifetimeIATA", SimTypeInt(signed=False, label="UInt32")), ("fAuditLog", SimTypeInt(signed=True, label="Int32")),)), name="DHCP_SERVER_CONFIG_INFO_V6", pack=False, align=None),
    "DHCP_SERVER_CONFIG_INFO_VQ": SimStruct(OrderedDict((("APIProtocolSupport", SimTypeInt(signed=False, label="UInt32")), ("DatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DatabasePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackupPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackupInterval", SimTypeInt(signed=False, label="UInt32")), ("DatabaseLoggingFlag", SimTypeInt(signed=False, label="UInt32")), ("RestoreFlag", SimTypeInt(signed=False, label="UInt32")), ("DatabaseCleanupInterval", SimTypeInt(signed=False, label="UInt32")), ("DebugFlag", SimTypeInt(signed=False, label="UInt32")), ("dwPingRetries", SimTypeInt(signed=False, label="UInt32")), ("cbBootTableString", SimTypeInt(signed=False, label="UInt32")), ("wszBootTableString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fAuditLog", SimTypeInt(signed=True, label="Int32")), ("QuarantineOn", SimTypeInt(signed=True, label="Int32")), ("QuarDefFail", SimTypeInt(signed=False, label="UInt32")), ("QuarRuntimeStatus", SimTypeInt(signed=True, label="Int32")),)), name="DHCP_SERVER_CONFIG_INFO_VQ", pack=False, align=None),
    "DHCP_SERVER_OPTIONS": SimStruct(OrderedDict((("MessageType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SubnetMask", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("RequestedAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("RequestLeaseTime", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("OverlayFields", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RouterAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Server", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ParameterRequestList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ParameterRequestListLength", SimTypeInt(signed=False, label="UInt32")), ("MachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("MachineNameLength", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareAddressType", SimTypeChar(label="Byte")), ("ClientHardwareAddressLength", SimTypeChar(label="Byte")), ("ClientHardwareAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClassIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClassIdentifierLength", SimTypeInt(signed=False, label="UInt32")), ("VendorClass", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("VendorClassLength", SimTypeInt(signed=False, label="UInt32")), ("DNSFlags", SimTypeInt(signed=False, label="UInt32")), ("DNSNameLength", SimTypeInt(signed=False, label="UInt32")), ("DNSName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DSDomainNameRequested", SimTypeChar(label="Byte")), ("DSDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DSDomainNameLen", SimTypeInt(signed=False, label="UInt32")), ("ScopeId", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DHCP_SERVER_OPTIONS", pack=False, align=None),
    "DHCP_SERVER_SPECIFIC_STRINGS": SimStruct(OrderedDict((("DefaultVendorClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DefaultUserClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_SERVER_SPECIFIC_STRINGS", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_DATA": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="DHCP_SUBNET_ELEMENT_TYPE")), ("Element", SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "SecondaryHost": SimTypePointer(SimTypeRef("DHCP_HOST_INFO", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "IpUsedCluster": SimTypePointer(SimTypeRef("DHCP_IP_CLUSTER", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="DHCP_SUBNET_ELEMENT_DATA", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_DATA_V4": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="DHCP_SUBNET_ELEMENT_TYPE")), ("Element", SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "SecondaryHost": SimTypePointer(SimTypeRef("DHCP_HOST_INFO", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_V4", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "IpUsedCluster": SimTypePointer(SimTypeRef("DHCP_IP_CLUSTER", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="DHCP_SUBNET_ELEMENT_DATA_V4", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_DATA_V5": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="DHCP_SUBNET_ELEMENT_TYPE")), ("Element", SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_BOOTP_IP_RANGE", SimStruct), offset=0), "SecondaryHost": SimTypePointer(SimTypeRef("DHCP_HOST_INFO", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_V4", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "IpUsedCluster": SimTypePointer(SimTypeRef("DHCP_IP_CLUSTER", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="DHCP_SUBNET_ELEMENT_DATA_V5", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_DATA_V6": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="DHCP_SUBNET_ELEMENT_TYPE_V6")), ("Element", SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE_V6", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_V6", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE_V6", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="DHCP_SUBNET_ELEMENT_DATA_V6", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_INFO_ARRAY": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_SUBNET_ELEMENT_DATA", SimStruct), offset=0)),)), name="DHCP_SUBNET_ELEMENT_INFO_ARRAY", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_INFO_ARRAY_V4": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_SUBNET_ELEMENT_DATA_V4", SimStruct), offset=0)),)), name="DHCP_SUBNET_ELEMENT_INFO_ARRAY_V4", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_INFO_ARRAY_V5": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_SUBNET_ELEMENT_DATA_V5", SimStruct), offset=0)),)), name="DHCP_SUBNET_ELEMENT_INFO_ARRAY_V5", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_INFO_ARRAY_V6": SimStruct(OrderedDict((("NumElements", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("DHCP_SUBNET_ELEMENT_DATA_V6", SimStruct), offset=0)),)), name="DHCP_SUBNET_ELEMENT_INFO_ARRAY_V6", pack=False, align=None),
    "DHCP_SUBNET_ELEMENT_TYPE": SimTypeInt(signed=False, label="DHCP_SUBNET_ELEMENT_TYPE"),
    "DHCP_SUBNET_ELEMENT_TYPE_V6": SimTypeInt(signed=False, label="DHCP_SUBNET_ELEMENT_TYPE_V6"),
    "DHCP_SUBNET_ELEMENT_UNION": SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "SecondaryHost": SimTypePointer(SimTypeRef("DHCP_HOST_INFO", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "IpUsedCluster": SimTypePointer(SimTypeRef("DHCP_IP_CLUSTER", SimStruct), offset=0)}, name="<anon>", label="None"),
    "DHCP_SUBNET_ELEMENT_UNION_V4": SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "SecondaryHost": SimTypePointer(SimTypeRef("DHCP_HOST_INFO", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_V4", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "IpUsedCluster": SimTypePointer(SimTypeRef("DHCP_IP_CLUSTER", SimStruct), offset=0)}, name="<anon>", label="None"),
    "DHCP_SUBNET_ELEMENT_UNION_V6": SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE_V6", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_V6", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE_V6", SimStruct), offset=0)}, name="<anon>", label="None"),
    "DHCP_SUBNET_INFO": SimStruct(OrderedDict((("SubnetAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("SubnetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SubnetComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PrimaryHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("SubnetState", SimTypeInt(signed=False, label="DHCP_SUBNET_STATE")),)), name="DHCP_SUBNET_INFO", pack=False, align=None),
    "DHCP_SUBNET_INFO_V6": SimStruct(OrderedDict((("SubnetAddress", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("Prefix", SimTypeInt(signed=False, label="UInt32")), ("Preference", SimTypeShort(signed=False, label="UInt16")), ("SubnetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SubnetComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("ScopeId", SimTypeInt(signed=False, label="UInt32")),)), name="DHCP_SUBNET_INFO_V6", pack=False, align=None),
    "DHCP_SUBNET_INFO_VQ": SimStruct(OrderedDict((("SubnetAddress", SimTypeInt(signed=False, label="UInt32")), ("SubnetMask", SimTypeInt(signed=False, label="UInt32")), ("SubnetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SubnetComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PrimaryHost", SimTypeRef("DHCP_HOST_INFO", SimStruct)), ("SubnetState", SimTypeInt(signed=False, label="DHCP_SUBNET_STATE")), ("QuarantineOn", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeLongLong(signed=True, label="Int64")), ("Reserved4", SimTypeLongLong(signed=True, label="Int64")),)), name="DHCP_SUBNET_INFO_VQ", pack=False, align=None),
    "DHCP_SUBNET_STATE": SimTypeInt(signed=False, label="DHCP_SUBNET_STATE"),
    "DHCP_SUPER_SCOPE_TABLE": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pEntries", SimTypePointer(SimTypeRef("DHCP_SUPER_SCOPE_TABLE_ENTRY", SimStruct), offset=0)),)), name="DHCP_SUPER_SCOPE_TABLE", pack=False, align=None),
    "DHCP_SUPER_SCOPE_TABLE_ENTRY": SimStruct(OrderedDict((("SubnetAddress", SimTypeInt(signed=False, label="UInt32")), ("SuperScopeNumber", SimTypeInt(signed=False, label="UInt32")), ("NextInSuperScope", SimTypeInt(signed=False, label="UInt32")), ("SuperScopeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DHCP_SUPER_SCOPE_TABLE_ENTRY", pack=False, align=None),
    "DHPDEV": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "DHSURF": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "DIACTIONA": SimStruct(OrderedDict((("uAppData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwSemantic", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"lptszActionName": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "uResIdString": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("guidInstance", SimTypeBottom(label="Guid")), ("dwObjID", SimTypeInt(signed=False, label="UInt32")), ("dwHow", SimTypeInt(signed=False, label="UInt32")),)), name="DIACTIONA", pack=False, align=None),
    "DIACTIONFORMATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwActionSize", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumActions", SimTypeInt(signed=False, label="UInt32")), ("rgoAction", SimTypePointer(SimTypeRef("DIACTIONA", SimStruct), offset=0)), ("guidActionMap", SimTypeBottom(label="Guid")), ("dwGenre", SimTypeInt(signed=False, label="UInt32")), ("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("lAxisMin", SimTypeInt(signed=True, label="Int32")), ("lAxisMax", SimTypeInt(signed=True, label="Int32")), ("hInstString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ftTimeStamp", SimTypeRef("FILETIME", SimStruct)), ("dwCRC", SimTypeInt(signed=False, label="UInt32")), ("tszActionMap", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="DIACTIONFORMATA", pack=False, align=None),
    "DIACTIONFORMATW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwActionSize", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumActions", SimTypeInt(signed=False, label="UInt32")), ("rgoAction", SimTypePointer(SimTypeRef("DIACTIONW", SimStruct), offset=0)), ("guidActionMap", SimTypeBottom(label="Guid")), ("dwGenre", SimTypeInt(signed=False, label="UInt32")), ("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("lAxisMin", SimTypeInt(signed=True, label="Int32")), ("lAxisMax", SimTypeInt(signed=True, label="Int32")), ("hInstString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ftTimeStamp", SimTypeRef("FILETIME", SimStruct)), ("dwCRC", SimTypeInt(signed=False, label="UInt32")), ("tszActionMap", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIACTIONFORMATW", pack=False, align=None),
    "DIACTIONW": SimStruct(OrderedDict((("uAppData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwSemantic", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"lptszActionName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "uResIdString": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("guidInstance", SimTypeBottom(label="Guid")), ("dwObjID", SimTypeInt(signed=False, label="UInt32")), ("dwHow", SimTypeInt(signed=False, label="UInt32")),)), name="DIACTIONW", pack=False, align=None),
    "DIAGNOSIS_STATUS": SimTypeInt(signed=False, label="DIAGNOSIS_STATUS"),
    "DIAGNOSTIC_DATA_EVENT_BINARY_STATS": SimStruct(OrderedDict((("moduleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("friendlyModuleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("eventCount", SimTypeInt(signed=False, label="UInt32")), ("uploadSizeBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="DIAGNOSTIC_DATA_EVENT_BINARY_STATS", pack=False, align=None),
    "DIAGNOSTIC_DATA_EVENT_CATEGORY_DESCRIPTION": SimStruct(OrderedDict((("id", SimTypeInt(signed=True, label="Int32")), ("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DIAGNOSTIC_DATA_EVENT_CATEGORY_DESCRIPTION", pack=False, align=None),
    "DIAGNOSTIC_DATA_EVENT_PRODUCER_DESCRIPTION": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DIAGNOSTIC_DATA_EVENT_PRODUCER_DESCRIPTION", pack=False, align=None),
    "DIAGNOSTIC_DATA_EVENT_TAG_DESCRIPTION": SimStruct(OrderedDict((("privacyTag", SimTypeInt(signed=True, label="Int32")), ("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("description", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DIAGNOSTIC_DATA_EVENT_TAG_DESCRIPTION", pack=False, align=None),
    "DIAGNOSTIC_DATA_EVENT_TAG_STATS": SimStruct(OrderedDict((("privacyTag", SimTypeInt(signed=True, label="Int32")), ("eventCount", SimTypeInt(signed=False, label="UInt32")),)), name="DIAGNOSTIC_DATA_EVENT_TAG_STATS", pack=False, align=None),
    "DIAGNOSTIC_DATA_EVENT_TRANSCRIPT_CONFIGURATION": SimStruct(OrderedDict((("hoursOfHistoryToKeep", SimTypeInt(signed=False, label="UInt32")), ("maxStoreMegabytes", SimTypeInt(signed=False, label="UInt32")), ("requestedMaxStoreMegabytes", SimTypeInt(signed=False, label="UInt32")),)), name="DIAGNOSTIC_DATA_EVENT_TRANSCRIPT_CONFIGURATION", pack=False, align=None),
    "DIAGNOSTIC_DATA_GENERAL_STATS": SimStruct(OrderedDict((("optInLevel", SimTypeInt(signed=False, label="UInt32")), ("transcriptSizeBytes", SimTypeLongLong(signed=False, label="UInt64")), ("oldestEventTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("totalEventCountLast24Hours", SimTypeInt(signed=False, label="UInt32")), ("averageDailyEvents", SimTypeFloat(size=32)),)), name="DIAGNOSTIC_DATA_GENERAL_STATS", pack=False, align=None),
    "DIAGNOSTIC_DATA_RECORD": SimStruct(OrderedDict((("rowId", SimTypeLongLong(signed=True, label="Int64")), ("timestamp", SimTypeLongLong(signed=False, label="UInt64")), ("eventKeywords", SimTypeLongLong(signed=False, label="UInt64")), ("fullEventName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("providerGroupGuid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("producerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("privacyTags", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("privacyTagCount", SimTypeInt(signed=False, label="UInt32")), ("categoryIds", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("categoryIdCount", SimTypeInt(signed=False, label="UInt32")), ("isCoreData", SimTypeInt(signed=True, label="Int32")), ("extra1", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("extra2", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("extra3", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DIAGNOSTIC_DATA_RECORD", pack=False, align=None),
    "DIAGNOSTIC_DATA_SEARCH_CRITERIA": SimStruct(OrderedDict((("producerNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("producerNameCount", SimTypeInt(signed=False, label="UInt32")), ("textToMatch", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("categoryIds", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("categoryIdCount", SimTypeInt(signed=False, label="UInt32")), ("privacyTags", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("privacyTagCount", SimTypeInt(signed=False, label="UInt32")), ("coreDataOnly", SimTypeInt(signed=True, label="Int32")),)), name="DIAGNOSTIC_DATA_SEARCH_CRITERIA", pack=False, align=None),
    "DIAGNOSTIC_REPORT_DATA": SimStruct(OrderedDict((("signature", SimTypeRef("DIAGNOSTIC_REPORT_SIGNATURE", SimStruct)), ("bucketId", SimTypeBottom(label="Guid")), ("reportId", SimTypeBottom(label="Guid")), ("creationTime", SimTypeRef("FILETIME", SimStruct)), ("sizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("cabId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("reportStatus", SimTypeInt(signed=False, label="UInt32")), ("reportIntegratorId", SimTypeBottom(label="Guid")), ("fileNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("fileCount", SimTypeInt(signed=False, label="UInt32")), ("friendlyEventName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("applicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("applicationPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bucketIdString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("legacyBucketId", SimTypeLongLong(signed=False, label="UInt64")), ("reportKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DIAGNOSTIC_REPORT_DATA", pack=False, align=None),
    "DIAGNOSTIC_REPORT_PARAMETER": SimStruct(OrderedDict((("name", SimTypeArray(SimTypeChar(label="Char"), 129)), ("value", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIAGNOSTIC_REPORT_PARAMETER", pack=False, align=None),
    "DIAGNOSTIC_REPORT_SIGNATURE": SimStruct(OrderedDict((("eventName", SimTypeArray(SimTypeChar(label="Char"), 65)), ("parameters", SimTypeArray(SimTypeRef("DIAGNOSTIC_REPORT_PARAMETER", SimStruct), 10)),)), name="DIAGNOSTIC_REPORT_SIGNATURE", pack=False, align=None),
    "DIAG_SOCKADDR": SimStruct(OrderedDict((("family", SimTypeShort(signed=False, label="UInt16")), ("data", SimTypeArray(SimTypeChar(label="SByte"), 126)),)), name="DIAG_SOCKADDR", pack=False, align=None),
    "DIALOGICOKIADPCMWAVEFORMAT": SimStruct(OrderedDict((("ewf", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="DIALOGICOKIADPCMWAVEFORMAT", pack=False, align=None),
    "DIALOG_CONTROL_DPI_CHANGE_BEHAVIORS": SimTypeInt(signed=False, label="DIALOG_CONTROL_DPI_CHANGE_BEHAVIORS"),
    "DIALOG_DPI_CHANGE_BEHAVIORS": SimTypeInt(signed=False, label="DIALOG_DPI_CHANGE_BEHAVIORS"),
    "DIBSECTION": SimStruct(OrderedDict((("dsBm", SimTypeRef("BITMAP", SimStruct)), ("dsBmih", SimTypeRef("BITMAPINFOHEADER", SimStruct)), ("dsBitfields", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("dshSection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dsOffset", SimTypeInt(signed=False, label="UInt32")),)), name="DIBSECTION", pack=False, align=None),
    "DIB_USAGE": SimTypeInt(signed=False, label="DIB_USAGE"),
    "DICOLORSET": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cTextFore", SimTypeInt(signed=False, label="UInt32")), ("cTextHighlight", SimTypeInt(signed=False, label="UInt32")), ("cCalloutLine", SimTypeInt(signed=False, label="UInt32")), ("cCalloutHighlight", SimTypeInt(signed=False, label="UInt32")), ("cBorder", SimTypeInt(signed=False, label="UInt32")), ("cControlFill", SimTypeInt(signed=False, label="UInt32")), ("cHighlightFill", SimTypeInt(signed=False, label="UInt32")), ("cAreaFill", SimTypeInt(signed=False, label="UInt32")),)), name="DICOLORSET", pack=False, align=None),
    "DICONDITION": SimStruct(OrderedDict((("lOffset", SimTypeInt(signed=True, label="Int32")), ("lPositiveCoefficient", SimTypeInt(signed=True, label="Int32")), ("lNegativeCoefficient", SimTypeInt(signed=True, label="Int32")), ("dwPositiveSaturation", SimTypeInt(signed=False, label="UInt32")), ("dwNegativeSaturation", SimTypeInt(signed=False, label="UInt32")), ("lDeadBand", SimTypeInt(signed=True, label="Int32")),)), name="DICONDITION", pack=False, align=None),
    "DICONFIGUREDEVICESPARAMSA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwcUsers", SimTypeInt(signed=False, label="UInt32")), ("lptszUserNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwcFormats", SimTypeInt(signed=False, label="UInt32")), ("lprgFormats", SimTypePointer(SimTypeRef("DIACTIONFORMATA", SimStruct), offset=0)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dics", SimTypeRef("DICOLORSET", SimStruct)), ("lpUnkDDSTarget", SimTypeBottom(label="IUnknown")),)), name="DICONFIGUREDEVICESPARAMSA", pack=False, align=None),
    "DICONFIGUREDEVICESPARAMSW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwcUsers", SimTypeInt(signed=False, label="UInt32")), ("lptszUserNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwcFormats", SimTypeInt(signed=False, label="UInt32")), ("lprgFormats", SimTypePointer(SimTypeRef("DIACTIONFORMATW", SimStruct), offset=0)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dics", SimTypeRef("DICOLORSET", SimStruct)), ("lpUnkDDSTarget", SimTypeBottom(label="IUnknown")),)), name="DICONFIGUREDEVICESPARAMSW", pack=False, align=None),
    "DICONSTANTFORCE": SimStruct(OrderedDict((("lMagnitude", SimTypeInt(signed=True, label="Int32")),)), name="DICONSTANTFORCE", pack=False, align=None),
    "DICTIONARYPROPERTIES": SimTypeInt(signed=False, label="DICTIONARYPROPERTIES"),
    "DICUSTOMFORCE": SimStruct(OrderedDict((("cChannels", SimTypeInt(signed=False, label="UInt32")), ("dwSamplePeriod", SimTypeInt(signed=False, label="UInt32")), ("cSamples", SimTypeInt(signed=False, label="UInt32")), ("rglForceData", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="DICUSTOMFORCE", pack=False, align=None),
    "DICounterItem": SimTypeBottom(label="DICounterItem"),
    "DIDATAFORMAT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwObjSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumObjs", SimTypeInt(signed=False, label="UInt32")), ("rgodf", SimTypePointer(SimTypeRef("DIOBJECTDATAFORMAT", SimStruct), offset=0)),)), name="DIDATAFORMAT", pack=False, align=None),
    "DIDEVCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDevType", SimTypeInt(signed=False, label="UInt32")), ("dwAxes", SimTypeInt(signed=False, label="UInt32")), ("dwButtons", SimTypeInt(signed=False, label="UInt32")), ("dwPOVs", SimTypeInt(signed=False, label="UInt32")), ("dwFFSamplePeriod", SimTypeInt(signed=False, label="UInt32")), ("dwFFMinTimeResolution", SimTypeInt(signed=False, label="UInt32")), ("dwFirmwareRevision", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareRevision", SimTypeInt(signed=False, label="UInt32")), ("dwFFDriverVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DIDEVCAPS", pack=False, align=None),
    "DIDEVCAPS_DX3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDevType", SimTypeInt(signed=False, label="UInt32")), ("dwAxes", SimTypeInt(signed=False, label="UInt32")), ("dwButtons", SimTypeInt(signed=False, label="UInt32")), ("dwPOVs", SimTypeInt(signed=False, label="UInt32")),)), name="DIDEVCAPS_DX3", pack=False, align=None),
    "DIDEVICEIMAGEINFOA": SimStruct(OrderedDict((("tszImagePath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwViewID", SimTypeInt(signed=False, label="UInt32")), ("rcOverlay", SimTypeRef("RECT", SimStruct)), ("dwObjID", SimTypeInt(signed=False, label="UInt32")), ("dwcValidPts", SimTypeInt(signed=False, label="UInt32")), ("rgptCalloutLine", SimTypeArray(SimTypeRef("POINT", SimStruct), 5)), ("rcCalloutRect", SimTypeRef("RECT", SimStruct)), ("dwTextAlign", SimTypeInt(signed=False, label="UInt32")),)), name="DIDEVICEIMAGEINFOA", pack=False, align=None),
    "DIDEVICEIMAGEINFOHEADERA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSizeImageInfo", SimTypeInt(signed=False, label="UInt32")), ("dwcViews", SimTypeInt(signed=False, label="UInt32")), ("dwcButtons", SimTypeInt(signed=False, label="UInt32")), ("dwcAxes", SimTypeInt(signed=False, label="UInt32")), ("dwcPOVs", SimTypeInt(signed=False, label="UInt32")), ("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwBufferUsed", SimTypeInt(signed=False, label="UInt32")), ("lprgImageInfoArray", SimTypePointer(SimTypeRef("DIDEVICEIMAGEINFOA", SimStruct), offset=0)),)), name="DIDEVICEIMAGEINFOHEADERA", pack=False, align=None),
    "DIDEVICEIMAGEINFOHEADERW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSizeImageInfo", SimTypeInt(signed=False, label="UInt32")), ("dwcViews", SimTypeInt(signed=False, label="UInt32")), ("dwcButtons", SimTypeInt(signed=False, label="UInt32")), ("dwcAxes", SimTypeInt(signed=False, label="UInt32")), ("dwcPOVs", SimTypeInt(signed=False, label="UInt32")), ("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwBufferUsed", SimTypeInt(signed=False, label="UInt32")), ("lprgImageInfoArray", SimTypePointer(SimTypeRef("DIDEVICEIMAGEINFOW", SimStruct), offset=0)),)), name="DIDEVICEIMAGEINFOHEADERW", pack=False, align=None),
    "DIDEVICEIMAGEINFOW": SimStruct(OrderedDict((("tszImagePath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwViewID", SimTypeInt(signed=False, label="UInt32")), ("rcOverlay", SimTypeRef("RECT", SimStruct)), ("dwObjID", SimTypeInt(signed=False, label="UInt32")), ("dwcValidPts", SimTypeInt(signed=False, label="UInt32")), ("rgptCalloutLine", SimTypeArray(SimTypeRef("POINT", SimStruct), 5)), ("rcCalloutRect", SimTypeRef("RECT", SimStruct)), ("dwTextAlign", SimTypeInt(signed=False, label="UInt32")),)), name="DIDEVICEIMAGEINFOW", pack=False, align=None),
    "DIDEVICEINSTANCEA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidInstance", SimTypeBottom(label="Guid")), ("guidProduct", SimTypeBottom(label="Guid")), ("dwDevType", SimTypeInt(signed=False, label="UInt32")), ("tszInstanceName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("tszProductName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("guidFFDriver", SimTypeBottom(label="Guid")), ("wUsagePage", SimTypeShort(signed=False, label="UInt16")), ("wUsage", SimTypeShort(signed=False, label="UInt16")),)), name="DIDEVICEINSTANCEA", pack=False, align=None),
    "DIDEVICEINSTANCEW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidInstance", SimTypeBottom(label="Guid")), ("guidProduct", SimTypeBottom(label="Guid")), ("dwDevType", SimTypeInt(signed=False, label="UInt32")), ("tszInstanceName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("tszProductName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("guidFFDriver", SimTypeBottom(label="Guid")), ("wUsagePage", SimTypeShort(signed=False, label="UInt16")), ("wUsage", SimTypeShort(signed=False, label="UInt16")),)), name="DIDEVICEINSTANCEW", pack=False, align=None),
    "DIDEVICEINSTANCE_DX3A": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidInstance", SimTypeBottom(label="Guid")), ("guidProduct", SimTypeBottom(label="Guid")), ("dwDevType", SimTypeInt(signed=False, label="UInt32")), ("tszInstanceName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("tszProductName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="DIDEVICEINSTANCE_DX3A", pack=False, align=None),
    "DIDEVICEINSTANCE_DX3W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidInstance", SimTypeBottom(label="Guid")), ("guidProduct", SimTypeBottom(label="Guid")), ("dwDevType", SimTypeInt(signed=False, label="UInt32")), ("tszInstanceName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("tszProductName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIDEVICEINSTANCE_DX3W", pack=False, align=None),
    "DIDEVICEOBJECTDATA": SimStruct(OrderedDict((("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwData", SimTypeInt(signed=False, label="UInt32")), ("dwTimeStamp", SimTypeInt(signed=False, label="UInt32")), ("dwSequence", SimTypeInt(signed=False, label="UInt32")), ("uAppData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DIDEVICEOBJECTDATA", pack=False, align=None),
    "DIDEVICEOBJECTDATA_DX3": SimStruct(OrderedDict((("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwData", SimTypeInt(signed=False, label="UInt32")), ("dwTimeStamp", SimTypeInt(signed=False, label="UInt32")), ("dwSequence", SimTypeInt(signed=False, label="UInt32")),)), name="DIDEVICEOBJECTDATA_DX3", pack=False, align=None),
    "DIDEVICEOBJECTINSTANCEA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidType", SimTypeBottom(label="Guid")), ("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("tszName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwFFMaxForce", SimTypeInt(signed=False, label="UInt32")), ("dwFFForceResolution", SimTypeInt(signed=False, label="UInt32")), ("wCollectionNumber", SimTypeShort(signed=False, label="UInt16")), ("wDesignatorIndex", SimTypeShort(signed=False, label="UInt16")), ("wUsagePage", SimTypeShort(signed=False, label="UInt16")), ("wUsage", SimTypeShort(signed=False, label="UInt16")), ("dwDimension", SimTypeInt(signed=False, label="UInt32")), ("wExponent", SimTypeShort(signed=False, label="UInt16")), ("wReportId", SimTypeShort(signed=False, label="UInt16")),)), name="DIDEVICEOBJECTINSTANCEA", pack=False, align=None),
    "DIDEVICEOBJECTINSTANCEW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidType", SimTypeBottom(label="Guid")), ("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("tszName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwFFMaxForce", SimTypeInt(signed=False, label="UInt32")), ("dwFFForceResolution", SimTypeInt(signed=False, label="UInt32")), ("wCollectionNumber", SimTypeShort(signed=False, label="UInt16")), ("wDesignatorIndex", SimTypeShort(signed=False, label="UInt16")), ("wUsagePage", SimTypeShort(signed=False, label="UInt16")), ("wUsage", SimTypeShort(signed=False, label="UInt16")), ("dwDimension", SimTypeInt(signed=False, label="UInt32")), ("wExponent", SimTypeShort(signed=False, label="UInt16")), ("wReportId", SimTypeShort(signed=False, label="UInt16")),)), name="DIDEVICEOBJECTINSTANCEW", pack=False, align=None),
    "DIDEVICEOBJECTINSTANCE_DX3A": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidType", SimTypeBottom(label="Guid")), ("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("tszName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="DIDEVICEOBJECTINSTANCE_DX3A", pack=False, align=None),
    "DIDEVICEOBJECTINSTANCE_DX3W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidType", SimTypeBottom(label="Guid")), ("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("tszName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIDEVICEOBJECTINSTANCE_DX3W", pack=False, align=None),
    "DIDEVICESTATE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwLoad", SimTypeInt(signed=False, label="UInt32")),)), name="DIDEVICESTATE", pack=False, align=None),
    "DIDRIVERVERSIONS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFirmwareRevision", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareRevision", SimTypeInt(signed=False, label="UInt32")), ("dwFFDriverVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DIDRIVERVERSIONS", pack=False, align=None),
    "DIEFFECT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")), ("dwSamplePeriod", SimTypeInt(signed=False, label="UInt32")), ("dwGain", SimTypeInt(signed=False, label="UInt32")), ("dwTriggerButton", SimTypeInt(signed=False, label="UInt32")), ("dwTriggerRepeatInterval", SimTypeInt(signed=False, label="UInt32")), ("cAxes", SimTypeInt(signed=False, label="UInt32")), ("rgdwAxes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("rglDirection", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lpEnvelope", SimTypePointer(SimTypeRef("DIENVELOPE", SimStruct), offset=0)), ("cbTypeSpecificParams", SimTypeInt(signed=False, label="UInt32")), ("lpvTypeSpecificParams", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwStartDelay", SimTypeInt(signed=False, label="UInt32")),)), name="DIEFFECT", pack=False, align=None),
    "DIEFFECTATTRIBUTES": SimStruct(OrderedDict((("dwEffectId", SimTypeInt(signed=False, label="UInt32")), ("dwEffType", SimTypeInt(signed=False, label="UInt32")), ("dwStaticParams", SimTypeInt(signed=False, label="UInt32")), ("dwDynamicParams", SimTypeInt(signed=False, label="UInt32")), ("dwCoords", SimTypeInt(signed=False, label="UInt32")),)), name="DIEFFECTATTRIBUTES", pack=False, align=None),
    "DIEFFECTINFOA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guid", SimTypeBottom(label="Guid")), ("dwEffType", SimTypeInt(signed=False, label="UInt32")), ("dwStaticParams", SimTypeInt(signed=False, label="UInt32")), ("dwDynamicParams", SimTypeInt(signed=False, label="UInt32")), ("tszName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="DIEFFECTINFOA", pack=False, align=None),
    "DIEFFECTINFOW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guid", SimTypeBottom(label="Guid")), ("dwEffType", SimTypeInt(signed=False, label="UInt32")), ("dwStaticParams", SimTypeInt(signed=False, label="UInt32")), ("dwDynamicParams", SimTypeInt(signed=False, label="UInt32")), ("tszName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIEFFECTINFOW", pack=False, align=None),
    "DIEFFECT_DX5": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")), ("dwSamplePeriod", SimTypeInt(signed=False, label="UInt32")), ("dwGain", SimTypeInt(signed=False, label="UInt32")), ("dwTriggerButton", SimTypeInt(signed=False, label="UInt32")), ("dwTriggerRepeatInterval", SimTypeInt(signed=False, label="UInt32")), ("cAxes", SimTypeInt(signed=False, label="UInt32")), ("rgdwAxes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("rglDirection", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lpEnvelope", SimTypePointer(SimTypeRef("DIENVELOPE", SimStruct), offset=0)), ("cbTypeSpecificParams", SimTypeInt(signed=False, label="UInt32")), ("lpvTypeSpecificParams", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DIEFFECT_DX5", pack=False, align=None),
    "DIEFFESCAPE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCommand", SimTypeInt(signed=False, label="UInt32")), ("lpvInBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbInBuffer", SimTypeInt(signed=False, label="UInt32")), ("lpvOutBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbOutBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="DIEFFESCAPE", pack=False, align=None),
    "DIENVELOPE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwAttackLevel", SimTypeInt(signed=False, label="UInt32")), ("dwAttackTime", SimTypeInt(signed=False, label="UInt32")), ("dwFadeLevel", SimTypeInt(signed=False, label="UInt32")), ("dwFadeTime", SimTypeInt(signed=False, label="UInt32")),)), name="DIENVELOPE", pack=False, align=None),
    "DIFFDEVICEATTRIBUTES": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFFSamplePeriod", SimTypeInt(signed=False, label="UInt32")), ("dwFFMinTimeResolution", SimTypeInt(signed=False, label="UInt32")),)), name="DIFFDEVICEATTRIBUTES", pack=False, align=None),
    "DIFFOBJECTATTRIBUTES": SimStruct(OrderedDict((("dwFFMaxForce", SimTypeInt(signed=False, label="UInt32")), ("dwFFForceResolution", SimTypeInt(signed=False, label="UInt32")),)), name="DIFFOBJECTATTRIBUTES", pack=False, align=None),
    "DIFILEEFFECT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("GuidEffect", SimTypeBottom(label="Guid")), ("lpDiEffect", SimTypePointer(SimTypeRef("DIEFFECT", SimStruct), offset=0)), ("szFriendlyName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="DIFILEEFFECT", pack=False, align=None),
    "DIFsrmClassificationEvents": SimTypeBottom(label="DIFsrmClassificationEvents"),
    "DIGEST_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["refdata", "pData", "dwLength"]), offset=0),
    "DIGIADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="DIGIADPCMWAVEFORMAT", pack=False, align=None),
    "DIGIFIXWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="DIGIFIXWAVEFORMAT", pack=False, align=None),
    "DIGIREALWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="DIGIREALWAVEFORMAT", pack=False, align=None),
    "DIGISTDWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="DIGISTDWAVEFORMAT", pack=False, align=None),
    "DIHIDFFINITINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwszDeviceInterface", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("GuidInstance", SimTypeBottom(label="Guid")),)), name="DIHIDFFINITINFO", pack=False, align=None),
    "DIINSTALLDEVICE_FLAGS": SimTypeInt(signed=False, label="DIINSTALLDEVICE_FLAGS"),
    "DIINSTALLDRIVER_FLAGS": SimTypeInt(signed=False, label="DIINSTALLDRIVER_FLAGS"),
    "DIJOYCONFIG": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidInstance", SimTypeBottom(label="Guid")), ("hwc", SimTypeRef("JOYREGHWCONFIG", SimStruct)), ("dwGain", SimTypeInt(signed=False, label="UInt32")), ("wszType", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszCallout", SimTypeArray(SimTypeChar(label="Char"), 256)), ("guidGameport", SimTypeBottom(label="Guid")),)), name="DIJOYCONFIG", pack=False, align=None),
    "DIJOYCONFIG_DX5": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidInstance", SimTypeBottom(label="Guid")), ("hwc", SimTypeRef("JOYREGHWCONFIG", SimStruct)), ("dwGain", SimTypeInt(signed=False, label="UInt32")), ("wszType", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszCallout", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="DIJOYCONFIG_DX5", pack=False, align=None),
    "DIJOYSTATE": SimStruct(OrderedDict((("lX", SimTypeInt(signed=True, label="Int32")), ("lY", SimTypeInt(signed=True, label="Int32")), ("lZ", SimTypeInt(signed=True, label="Int32")), ("lRx", SimTypeInt(signed=True, label="Int32")), ("lRy", SimTypeInt(signed=True, label="Int32")), ("lRz", SimTypeInt(signed=True, label="Int32")), ("rglSlider", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 2)), ("rgdwPOV", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("rgbButtons", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="DIJOYSTATE", pack=False, align=None),
    "DIJOYSTATE2": SimStruct(OrderedDict((("lX", SimTypeInt(signed=True, label="Int32")), ("lY", SimTypeInt(signed=True, label="Int32")), ("lZ", SimTypeInt(signed=True, label="Int32")), ("lRx", SimTypeInt(signed=True, label="Int32")), ("lRy", SimTypeInt(signed=True, label="Int32")), ("lRz", SimTypeInt(signed=True, label="Int32")), ("rglSlider", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 2)), ("rgdwPOV", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("rgbButtons", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("lVX", SimTypeInt(signed=True, label="Int32")), ("lVY", SimTypeInt(signed=True, label="Int32")), ("lVZ", SimTypeInt(signed=True, label="Int32")), ("lVRx", SimTypeInt(signed=True, label="Int32")), ("lVRy", SimTypeInt(signed=True, label="Int32")), ("lVRz", SimTypeInt(signed=True, label="Int32")), ("rglVSlider", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 2)), ("lAX", SimTypeInt(signed=True, label="Int32")), ("lAY", SimTypeInt(signed=True, label="Int32")), ("lAZ", SimTypeInt(signed=True, label="Int32")), ("lARx", SimTypeInt(signed=True, label="Int32")), ("lARy", SimTypeInt(signed=True, label="Int32")), ("lARz", SimTypeInt(signed=True, label="Int32")), ("rglASlider", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 2)), ("lFX", SimTypeInt(signed=True, label="Int32")), ("lFY", SimTypeInt(signed=True, label="Int32")), ("lFZ", SimTypeInt(signed=True, label="Int32")), ("lFRx", SimTypeInt(signed=True, label="Int32")), ("lFRy", SimTypeInt(signed=True, label="Int32")), ("lFRz", SimTypeInt(signed=True, label="Int32")), ("rglFSlider", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 2)),)), name="DIJOYSTATE2", pack=False, align=None),
    "DIJOYTYPEINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hws", SimTypeRef("JOYREGHWSETTINGS", SimStruct)), ("clsidConfig", SimTypeBottom(label="Guid")), ("wszDisplayName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszCallout", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wszHardwareId", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwFlags1", SimTypeInt(signed=False, label="UInt32")), ("dwFlags2", SimTypeInt(signed=False, label="UInt32")), ("wszMapFile", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="DIJOYTYPEINFO", pack=False, align=None),
    "DIJOYTYPEINFO_DX5": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hws", SimTypeRef("JOYREGHWSETTINGS", SimStruct)), ("clsidConfig", SimTypeBottom(label="Guid")), ("wszDisplayName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszCallout", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIJOYTYPEINFO_DX5", pack=False, align=None),
    "DIJOYTYPEINFO_DX6": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hws", SimTypeRef("JOYREGHWSETTINGS", SimStruct)), ("clsidConfig", SimTypeBottom(label="Guid")), ("wszDisplayName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszCallout", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wszHardwareId", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwFlags1", SimTypeInt(signed=False, label="UInt32")),)), name="DIJOYTYPEINFO_DX6", pack=False, align=None),
    "DIJOYUSERVALUES": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ruv", SimTypeRef("JOYREGUSERVALUES", SimStruct)), ("wszGlobalDriver", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszGameportEmulator", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="DIJOYUSERVALUES", pack=False, align=None),
    "DILogFileItem": SimTypeBottom(label="DILogFileItem"),
    "DIMOUSESTATE": SimStruct(OrderedDict((("lX", SimTypeInt(signed=True, label="Int32")), ("lY", SimTypeInt(signed=True, label="Int32")), ("lZ", SimTypeInt(signed=True, label="Int32")), ("rgbButtons", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="DIMOUSESTATE", pack=False, align=None),
    "DIMOUSESTATE2": SimStruct(OrderedDict((("lX", SimTypeInt(signed=True, label="Int32")), ("lY", SimTypeInt(signed=True, label="Int32")), ("lZ", SimTypeInt(signed=True, label="Int32")), ("rgbButtons", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="DIMOUSESTATE2", pack=False, align=None),
    "DIOBJECTATTRIBUTES": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("wUsagePage", SimTypeShort(signed=False, label="UInt16")), ("wUsage", SimTypeShort(signed=False, label="UInt16")),)), name="DIOBJECTATTRIBUTES", pack=False, align=None),
    "DIOBJECTCALIBRATION": SimStruct(OrderedDict((("lMin", SimTypeInt(signed=True, label="Int32")), ("lCenter", SimTypeInt(signed=True, label="Int32")), ("lMax", SimTypeInt(signed=True, label="Int32")),)), name="DIOBJECTCALIBRATION", pack=False, align=None),
    "DIOBJECTDATAFORMAT": SimStruct(OrderedDict((("pguid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwOfs", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DIOBJECTDATAFORMAT", pack=False, align=None),
    "DIPERIODIC": SimStruct(OrderedDict((("dwMagnitude", SimTypeInt(signed=False, label="UInt32")), ("lOffset", SimTypeInt(signed=True, label="Int32")), ("dwPhase", SimTypeInt(signed=False, label="UInt32")), ("dwPeriod", SimTypeInt(signed=False, label="UInt32")),)), name="DIPERIODIC", pack=False, align=None),
    "DIPOVCALIBRATION": SimStruct(OrderedDict((("lMin", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 5)), ("lMax", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 5)),)), name="DIPOVCALIBRATION", pack=False, align=None),
    "DIPROPCAL": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("lMin", SimTypeInt(signed=True, label="Int32")), ("lCenter", SimTypeInt(signed=True, label="Int32")), ("lMax", SimTypeInt(signed=True, label="Int32")),)), name="DIPROPCAL", pack=False, align=None),
    "DIPROPCALPOV": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("lMin", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 5)), ("lMax", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 5)),)), name="DIPROPCALPOV", pack=False, align=None),
    "DIPROPCPOINTS": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("dwCPointsNum", SimTypeInt(signed=False, label="UInt32")), ("cp", SimTypeArray(SimTypeRef("CPOINT", SimStruct), 8)),)), name="DIPROPCPOINTS", pack=False, align=None),
    "DIPROPDWORD": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("dwData", SimTypeInt(signed=False, label="UInt32")),)), name="DIPROPDWORD", pack=False, align=None),
    "DIPROPGUIDANDPATH": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("guidClass", SimTypeBottom(label="Guid")), ("wszPath", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIPROPGUIDANDPATH", pack=False, align=None),
    "DIPROPHEADER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwHeaderSize", SimTypeInt(signed=False, label="UInt32")), ("dwObj", SimTypeInt(signed=False, label="UInt32")), ("dwHow", SimTypeInt(signed=False, label="UInt32")),)), name="DIPROPHEADER", pack=False, align=None),
    "DIPROPPOINTER": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("uData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DIPROPPOINTER", pack=False, align=None),
    "DIPROPRANGE": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("lMin", SimTypeInt(signed=True, label="Int32")), ("lMax", SimTypeInt(signed=True, label="Int32")),)), name="DIPROPRANGE", pack=False, align=None),
    "DIPROPSTRING": SimStruct(OrderedDict((("diph", SimTypeRef("DIPROPHEADER", SimStruct)), ("wsz", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DIPROPSTRING", pack=False, align=None),
    "DIRAMPFORCE": SimStruct(OrderedDict((("lStart", SimTypeInt(signed=True, label="Int32")), ("lEnd", SimTypeInt(signed=True, label="Int32")),)), name="DIRAMPFORCE", pack=False, align=None),
    "DIRECTMANIPULATION_AUTOSCROLL_CONFIGURATION": SimTypeInt(signed=False, label="DIRECTMANIPULATION_AUTOSCROLL_CONFIGURATION"),
    "DIRECTMANIPULATION_CONFIGURATION": SimTypeInt(signed=False, label="DIRECTMANIPULATION_CONFIGURATION"),
    "DIRECTMANIPULATION_DRAG_DROP_CONFIGURATION": SimTypeInt(signed=False, label="DIRECTMANIPULATION_DRAG_DROP_CONFIGURATION"),
    "DIRECTMANIPULATION_DRAG_DROP_STATUS": SimTypeInt(signed=False, label="DIRECTMANIPULATION_DRAG_DROP_STATUS"),
    "DIRECTMANIPULATION_GESTURE_CONFIGURATION": SimTypeInt(signed=False, label="DIRECTMANIPULATION_GESTURE_CONFIGURATION"),
    "DIRECTMANIPULATION_HITTEST_TYPE": SimTypeInt(signed=False, label="DIRECTMANIPULATION_HITTEST_TYPE"),
    "DIRECTMANIPULATION_HORIZONTALALIGNMENT": SimTypeInt(signed=False, label="DIRECTMANIPULATION_HORIZONTALALIGNMENT"),
    "DIRECTMANIPULATION_INPUT_MODE": SimTypeInt(signed=False, label="DIRECTMANIPULATION_INPUT_MODE"),
    "DIRECTMANIPULATION_INTERACTION_TYPE": SimTypeInt(signed=False, label="DIRECTMANIPULATION_INTERACTION_TYPE"),
    "DIRECTMANIPULATION_MOTION_TYPES": SimTypeInt(signed=False, label="DIRECTMANIPULATION_MOTION_TYPES"),
    "DIRECTMANIPULATION_SNAPPOINT_COORDINATE": SimTypeInt(signed=False, label="DIRECTMANIPULATION_SNAPPOINT_COORDINATE"),
    "DIRECTMANIPULATION_SNAPPOINT_TYPE": SimTypeInt(signed=False, label="DIRECTMANIPULATION_SNAPPOINT_TYPE"),
    "DIRECTMANIPULATION_STATUS": SimTypeInt(signed=False, label="DIRECTMANIPULATION_STATUS"),
    "DIRECTMANIPULATION_VERTICALALIGNMENT": SimTypeInt(signed=False, label="DIRECTMANIPULATION_VERTICALALIGNMENT"),
    "DIRECTMANIPULATION_VIEWPORT_OPTIONS": SimTypeInt(signed=False, label="DIRECTMANIPULATION_VIEWPORT_OPTIONS"),
    "DIRECTORY_NOTIFY_INFORMATION_CLASS": SimTypeInt(signed=False, label="DIRECTORY_NOTIFY_INFORMATION_CLASS"),
    "DIRECTORY_OBJECT_TYPE": SimTypeInt(signed=False, label="DIRECTORY_OBJECT_TYPE"),
    "DIRECTORY_TYPE": SimTypeInt(signed=False, label="DIRECTORY_TYPE"),
    "DIRECTSOUNDDEVICE_DATAFLOW": SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_DATAFLOW"),
    "DIRECTSOUNDDEVICE_TYPE": SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_TYPE"),
    "DIRECTX_AUDIO_ACTIVATION_PARAMS": SimStruct(OrderedDict((("cbDirectXAudioActivationParams", SimTypeInt(signed=False, label="UInt32")), ("guidAudioSession", SimTypeBottom(label="Guid")), ("dwAudioStreamFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DIRECTX_AUDIO_ACTIVATION_PARAMS", pack=False, align=None),
    "DIROLLBACKDRIVER_FLAGS": SimTypeInt(signed=False, label="DIROLLBACKDRIVER_FLAGS"),
    "DIRTYRECT_INFO": SimStruct(OrderedDict((("FrameNumber", SimTypeInt(signed=False, label="UInt32")), ("NumDirtyRects", SimTypeInt(signed=False, label="UInt32")), ("DirtyRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)),)), name="DIRTYRECT_INFO", pack=False, align=None),
    "DISCARDCACHE": SimTypeInt(signed=False, label="DISCARDCACHE"),
    "DISCDLGSTRUCTA": SimStruct(OrderedDict((("cbStructure", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpLocalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpRemoteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="DISCDLGSTRUCT_FLAGS")),)), name="DISCDLGSTRUCTA", pack=False, align=None),
    "DISCDLGSTRUCTW": SimStruct(OrderedDict((("cbStructure", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpLocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpRemoteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="DISCDLGSTRUCT_FLAGS")),)), name="DISCDLGSTRUCTW", pack=False, align=None),
    "DISCDLGSTRUCT_FLAGS": SimTypeInt(signed=False, label="DISCDLGSTRUCT_FLAGS"),
    "DISCONNECT_CODE": SimTypeInt(signed=False, label="DISCONNECT_CODE"),
    "DISC_RECORDER_STATE_FLAGS": SimTypeInt(signed=False, label="DISC_RECORDER_STATE_FLAGS"),
    "DISKQUOTA_USERNAME_RESOLVE": SimTypeInt(signed=False, label="DISKQUOTA_USERNAME_RESOLVE"),
    "DISKQUOTA_USER_INFORMATION": SimStruct(OrderedDict((("QuotaUsed", SimTypeLongLong(signed=True, label="Int64")), ("QuotaThreshold", SimTypeLongLong(signed=True, label="Int64")), ("QuotaLimit", SimTypeLongLong(signed=True, label="Int64")),)), name="DISKQUOTA_USER_INFORMATION", pack=False, align=None),
    "DISK_CACHE_INFORMATION": SimStruct(OrderedDict((("ParametersSavable", SimTypeChar(label="Byte")), ("ReadCacheEnabled", SimTypeChar(label="Byte")), ("WriteCacheEnabled", SimTypeChar(label="Byte")), ("ReadRetentionPriority", SimTypeInt(signed=False, label="DISK_CACHE_RETENTION_PRIORITY")), ("WriteRetentionPriority", SimTypeInt(signed=False, label="DISK_CACHE_RETENTION_PRIORITY")), ("DisablePrefetchTransferLength", SimTypeShort(signed=False, label="UInt16")), ("PrefetchScalar", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"ScalarPrefetch": SimTypeRef("_ScalarPrefetch_e__Struct", SimStruct), "BlockPrefetch": SimTypeRef("_BlockPrefetch_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DISK_CACHE_INFORMATION", pack=False, align=None),
    "DISK_CACHE_RETENTION_PRIORITY": SimTypeInt(signed=False, label="DISK_CACHE_RETENTION_PRIORITY"),
    "DISK_CONTROLLER_NUMBER": SimStruct(OrderedDict((("ControllerNumber", SimTypeInt(signed=False, label="UInt32")), ("DiskNumber", SimTypeInt(signed=False, label="UInt32")),)), name="DISK_CONTROLLER_NUMBER", pack=False, align=None),
    "DISK_DETECTION_INFO": SimStruct(OrderedDict((("SizeOfDetectInfo", SimTypeInt(signed=False, label="UInt32")), ("DetectionType", SimTypeInt(signed=False, label="DETECTION_TYPE")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Int13", SimTypeRef("DISK_INT13_INFO", SimStruct)), ("ExInt13", SimTypeRef("DISK_EX_INT13_INFO", SimStruct)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="DISK_DETECTION_INFO", pack=False, align=None),
    "DISK_EXTENT": SimStruct(OrderedDict((("DiskNumber", SimTypeInt(signed=False, label="UInt32")), ("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("ExtentLength", SimTypeLongLong(signed=True, label="Int64")),)), name="DISK_EXTENT", pack=False, align=None),
    "DISK_EX_INT13_INFO": SimStruct(OrderedDict((("ExBufferSize", SimTypeShort(signed=False, label="UInt16")), ("ExFlags", SimTypeShort(signed=False, label="UInt16")), ("ExCylinders", SimTypeInt(signed=False, label="UInt32")), ("ExHeads", SimTypeInt(signed=False, label="UInt32")), ("ExSectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("ExSectorsPerDrive", SimTypeLongLong(signed=False, label="UInt64")), ("ExSectorSize", SimTypeShort(signed=False, label="UInt16")), ("ExReserved", SimTypeShort(signed=False, label="UInt16")),)), name="DISK_EX_INT13_INFO", pack=False, align=None),
    "DISK_GEOMETRY": SimStruct(OrderedDict((("Cylinders", SimTypeLongLong(signed=True, label="Int64")), ("MediaType", SimTypeInt(signed=False, label="MEDIA_TYPE")), ("TracksPerCylinder", SimTypeInt(signed=False, label="UInt32")), ("SectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="DISK_GEOMETRY", pack=False, align=None),
    "DISK_GEOMETRY_EX": SimStruct(OrderedDict((("Geometry", SimTypeRef("DISK_GEOMETRY", SimStruct)), ("DiskSize", SimTypeLongLong(signed=True, label="Int64")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DISK_GEOMETRY_EX", pack=False, align=None),
    "DISK_GROW_PARTITION": SimStruct(OrderedDict((("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("BytesToGrow", SimTypeLongLong(signed=True, label="Int64")),)), name="DISK_GROW_PARTITION", pack=False, align=None),
    "DISK_HEALTH_NOTIFICATION_DATA": SimStruct(OrderedDict((("DeviceGuid", SimTypeBottom(label="Guid")),)), name="DISK_HEALTH_NOTIFICATION_DATA", pack=False, align=None),
    "DISK_HISTOGRAM": SimStruct(OrderedDict((("DiskSize", SimTypeLongLong(signed=True, label="Int64")), ("Start", SimTypeLongLong(signed=True, label="Int64")), ("End", SimTypeLongLong(signed=True, label="Int64")), ("Average", SimTypeLongLong(signed=True, label="Int64")), ("AverageRead", SimTypeLongLong(signed=True, label="Int64")), ("AverageWrite", SimTypeLongLong(signed=True, label="Int64")), ("Granularity", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ReadCount", SimTypeInt(signed=False, label="UInt32")), ("WriteCount", SimTypeInt(signed=False, label="UInt32")), ("Histogram", SimTypePointer(SimTypeRef("HISTOGRAM_BUCKET", SimStruct), offset=0)),)), name="DISK_HISTOGRAM", pack=False, align=None),
    "DISK_INT13_INFO": SimStruct(OrderedDict((("DriveSelect", SimTypeShort(signed=False, label="UInt16")), ("MaxCylinders", SimTypeInt(signed=False, label="UInt32")), ("SectorsPerTrack", SimTypeShort(signed=False, label="UInt16")), ("MaxHeads", SimTypeShort(signed=False, label="UInt16")), ("NumberDrives", SimTypeShort(signed=False, label="UInt16")),)), name="DISK_INT13_INFO", pack=False, align=None),
    "DISK_LOGGING": SimStruct(OrderedDict((("Function", SimTypeChar(label="Byte")), ("BufferAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="DISK_LOGGING", pack=False, align=None),
    "DISK_PARTITION_INFO": SimStruct(OrderedDict((("SizeOfPartitionInfo", SimTypeInt(signed=False, label="UInt32")), ("PartitionStyle", SimTypeInt(signed=False, label="PARTITION_STYLE")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("_Mbr_e__Struct", SimStruct), "Gpt": SimTypeRef("_Gpt_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DISK_PARTITION_INFO", pack=False, align=None),
    "DISK_PERFORMANCE": SimStruct(OrderedDict((("BytesRead", SimTypeLongLong(signed=True, label="Int64")), ("BytesWritten", SimTypeLongLong(signed=True, label="Int64")), ("ReadTime", SimTypeLongLong(signed=True, label="Int64")), ("WriteTime", SimTypeLongLong(signed=True, label="Int64")), ("IdleTime", SimTypeLongLong(signed=True, label="Int64")), ("ReadCount", SimTypeInt(signed=False, label="UInt32")), ("WriteCount", SimTypeInt(signed=False, label="UInt32")), ("QueueDepth", SimTypeInt(signed=False, label="UInt32")), ("SplitCount", SimTypeInt(signed=False, label="UInt32")), ("QueryTime", SimTypeLongLong(signed=True, label="Int64")), ("StorageDeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("StorageManagerName", SimTypeArray(SimTypeChar(label="Char"), 8)),)), name="DISK_PERFORMANCE", pack=False, align=None),
    "DISK_RECORD": SimStruct(OrderedDict((("ByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfBytes", SimTypeInt(signed=False, label="UInt32")), ("DeviceNumber", SimTypeChar(label="Byte")), ("ReadRequest", SimTypeChar(label="Byte")),)), name="DISK_RECORD", pack=False, align=None),
    "DISK_SIGNATURE": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("_Mbr_e__Struct", SimStruct), "Gpt": SimTypeRef("_Gpt_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DISK_SIGNATURE", pack=False, align=None),
    "DISK_SPACE_INFORMATION": SimStruct(OrderedDict((("ActualTotalAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("ActualAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("ActualPoolUnavailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("CallerTotalAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("CallerAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("CallerPoolUnavailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("UsedAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("TotalReservedAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeStorageReserveAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableCommittedAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("PoolAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("SectorsPerAllocationUnit", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="DISK_SPACE_INFORMATION", pack=False, align=None),
    "DISPATCHER_CONTEXT": SimStruct(OrderedDict((("ControlPc", SimTypeLongLong(signed=False, label="UInt64")), ("ImageBase", SimTypeLongLong(signed=False, label="UInt64")), ("FunctionEntry", SimTypePointer(SimTypeRef("IMAGE_RUNTIME_FUNCTION_ENTRY", SimStruct), offset=0)), ("EstablisherFrame", SimTypeLongLong(signed=False, label="UInt64")), ("TargetIp", SimTypeLongLong(signed=False, label="UInt64")), ("ContextRecord", SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0)), ("LanguageHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_RECORD", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="EXCEPTION_DISPOSITION"), arg_names=["ExceptionRecord", "EstablisherFrame", "ContextRecord", "DispatcherContext"]), offset=0)), ("HandlerData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HistoryTable", SimTypePointer(SimTypeRef("UNWIND_HISTORY_TABLE", SimStruct), offset=0)), ("ScopeIndex", SimTypeInt(signed=False, label="UInt32")), ("Fill0", SimTypeInt(signed=False, label="UInt32")),)), name="DISPATCHER_CONTEXT", pack=False, align=None),
    "DISPATCHER_CONTEXT_NONVOLREG_ARM64": SimUnion({"Buffer": SimTypeArray(SimTypeChar(label="Byte"), 152), "Anonymous": SimStruct(OrderedDict((("GpNvRegs", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 11)), ("FpNvRegs", SimTypeArray(SimTypeFloat(size=64), 8)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "DISPATCHER_HEADER": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous1": SimUnion({"Lock": SimTypeInt(signed=True, label="Int32"), "LockNV": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"), "Anonymous2": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Signalling", SimTypeChar(label="Byte")), ("Size", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct(OrderedDict((("TimerType", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"TimerControlFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Hand", SimTypeChar(label="Byte")), ("Anonymous2", SimUnion({"TimerMiscFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous3_e__Struct", pack=False, align=None), "Anonymous4": SimStruct(OrderedDict((("Timer2Type", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Timer2Flags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Timer2ComponentId", SimTypeChar(label="Byte")), ("Timer2RelativeId", SimTypeChar(label="Byte")),)), name="_Anonymous4_e__Struct", pack=False, align=None), "Anonymous5": SimStruct(OrderedDict((("QueueType", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"QueueControlFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("QueueSize", SimTypeChar(label="Byte")), ("QueueReserved", SimTypeChar(label="Byte")),)), name="_Anonymous5_e__Struct", pack=False, align=None), "Anonymous6": SimStruct(OrderedDict((("ThreadType", SimTypeChar(label="Byte")), ("ThreadReserved", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"ThreadControlFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"DebugActive": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="_Anonymous6_e__Struct", pack=False, align=None), "Anonymous7": SimStruct(OrderedDict((("MutantType", SimTypeChar(label="Byte")), ("MutantSize", SimTypeChar(label="Byte")), ("DpcActive", SimTypeChar(label="Byte")), ("MutantReserved", SimTypeChar(label="Byte")),)), name="_Anonymous7_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("SignalState", SimTypeInt(signed=True, label="Int32")), ("WaitListHead", SimTypeRef("LIST_ENTRY", SimStruct)),)), name="DISPATCHER_HEADER", pack=False, align=None),
    "DISPATCH_FLAGS": SimTypeInt(signed=False, label="DISPATCH_FLAGS"),
    "DISPIDSPRG": SimTypeInt(signed=False, label="DISPIDSPRG"),
    "DISPIDSPTSI": SimTypeInt(signed=False, label="DISPIDSPTSI"),
    "DISPID_Ink": SimTypeInt(signed=False, label="DISPID_Ink"),
    "DISPID_InkCollector": SimTypeInt(signed=False, label="DISPID_InkCollector"),
    "DISPID_InkCollectorEvent": SimTypeInt(signed=False, label="DISPID_InkCollectorEvent"),
    "DISPID_InkCursor": SimTypeInt(signed=False, label="DISPID_InkCursor"),
    "DISPID_InkCursorButton": SimTypeInt(signed=False, label="DISPID_InkCursorButton"),
    "DISPID_InkCursorButtons": SimTypeInt(signed=False, label="DISPID_InkCursorButtons"),
    "DISPID_InkCursors": SimTypeInt(signed=False, label="DISPID_InkCursors"),
    "DISPID_InkCustomStrokes": SimTypeInt(signed=False, label="DISPID_InkCustomStrokes"),
    "DISPID_InkDivider": SimTypeInt(signed=False, label="DISPID_InkDivider"),
    "DISPID_InkDivisionResult": SimTypeInt(signed=False, label="DISPID_InkDivisionResult"),
    "DISPID_InkDivisionUnit": SimTypeInt(signed=False, label="DISPID_InkDivisionUnit"),
    "DISPID_InkDivisionUnits": SimTypeInt(signed=False, label="DISPID_InkDivisionUnits"),
    "DISPID_InkDrawingAttributes": SimTypeInt(signed=False, label="DISPID_InkDrawingAttributes"),
    "DISPID_InkEdit": SimTypeInt(signed=False, label="DISPID_InkEdit"),
    "DISPID_InkEditEvents": SimTypeInt(signed=False, label="DISPID_InkEditEvents"),
    "DISPID_InkEvent": SimTypeInt(signed=False, label="DISPID_InkEvent"),
    "DISPID_InkExtendedProperties": SimTypeInt(signed=False, label="DISPID_InkExtendedProperties"),
    "DISPID_InkExtendedProperty": SimTypeInt(signed=False, label="DISPID_InkExtendedProperty"),
    "DISPID_InkGesture": SimTypeInt(signed=False, label="DISPID_InkGesture"),
    "DISPID_InkRecoAlternate": SimTypeInt(signed=False, label="DISPID_InkRecoAlternate"),
    "DISPID_InkRecoContext": SimTypeInt(signed=False, label="DISPID_InkRecoContext"),
    "DISPID_InkRecoContext2": SimTypeInt(signed=False, label="DISPID_InkRecoContext2"),
    "DISPID_InkRecognitionAlternates": SimTypeInt(signed=False, label="DISPID_InkRecognitionAlternates"),
    "DISPID_InkRecognitionEvent": SimTypeInt(signed=False, label="DISPID_InkRecognitionEvent"),
    "DISPID_InkRecognitionResult": SimTypeInt(signed=False, label="DISPID_InkRecognitionResult"),
    "DISPID_InkRecognizer": SimTypeInt(signed=False, label="DISPID_InkRecognizer"),
    "DISPID_InkRecognizer2": SimTypeInt(signed=False, label="DISPID_InkRecognizer2"),
    "DISPID_InkRecognizerGuide": SimTypeInt(signed=False, label="DISPID_InkRecognizerGuide"),
    "DISPID_InkRecognizers": SimTypeInt(signed=False, label="DISPID_InkRecognizers"),
    "DISPID_InkRectangle": SimTypeInt(signed=False, label="DISPID_InkRectangle"),
    "DISPID_InkRenderer": SimTypeInt(signed=False, label="DISPID_InkRenderer"),
    "DISPID_InkStrokeDisp": SimTypeInt(signed=False, label="DISPID_InkStrokeDisp"),
    "DISPID_InkStrokes": SimTypeInt(signed=False, label="DISPID_InkStrokes"),
    "DISPID_InkTablet": SimTypeInt(signed=False, label="DISPID_InkTablet"),
    "DISPID_InkTablet2": SimTypeInt(signed=False, label="DISPID_InkTablet2"),
    "DISPID_InkTablet3": SimTypeInt(signed=False, label="DISPID_InkTablet3"),
    "DISPID_InkTablets": SimTypeInt(signed=False, label="DISPID_InkTablets"),
    "DISPID_InkTransform": SimTypeInt(signed=False, label="DISPID_InkTransform"),
    "DISPID_InkWordList": SimTypeInt(signed=False, label="DISPID_InkWordList"),
    "DISPID_InkWordList2": SimTypeInt(signed=False, label="DISPID_InkWordList2"),
    "DISPID_MathInputControlEvents": SimTypeInt(signed=False, label="DISPID_MathInputControlEvents"),
    "DISPID_PenInputPanel": SimTypeInt(signed=False, label="DISPID_PenInputPanel"),
    "DISPID_PenInputPanelEvents": SimTypeInt(signed=False, label="DISPID_PenInputPanelEvents"),
    "DISPID_SpeechAudio": SimTypeInt(signed=False, label="DISPID_SpeechAudio"),
    "DISPID_SpeechAudioBufferInfo": SimTypeInt(signed=False, label="DISPID_SpeechAudioBufferInfo"),
    "DISPID_SpeechAudioFormat": SimTypeInt(signed=False, label="DISPID_SpeechAudioFormat"),
    "DISPID_SpeechAudioStatus": SimTypeInt(signed=False, label="DISPID_SpeechAudioStatus"),
    "DISPID_SpeechBaseStream": SimTypeInt(signed=False, label="DISPID_SpeechBaseStream"),
    "DISPID_SpeechCustomStream": SimTypeInt(signed=False, label="DISPID_SpeechCustomStream"),
    "DISPID_SpeechDataKey": SimTypeInt(signed=False, label="DISPID_SpeechDataKey"),
    "DISPID_SpeechFileStream": SimTypeInt(signed=False, label="DISPID_SpeechFileStream"),
    "DISPID_SpeechGrammarRule": SimTypeInt(signed=False, label="DISPID_SpeechGrammarRule"),
    "DISPID_SpeechGrammarRuleState": SimTypeInt(signed=False, label="DISPID_SpeechGrammarRuleState"),
    "DISPID_SpeechGrammarRuleStateTransition": SimTypeInt(signed=False, label="DISPID_SpeechGrammarRuleStateTransition"),
    "DISPID_SpeechGrammarRuleStateTransitions": SimTypeInt(signed=False, label="DISPID_SpeechGrammarRuleStateTransitions"),
    "DISPID_SpeechGrammarRules": SimTypeInt(signed=False, label="DISPID_SpeechGrammarRules"),
    "DISPID_SpeechLexicon": SimTypeInt(signed=False, label="DISPID_SpeechLexicon"),
    "DISPID_SpeechLexiconProns": SimTypeInt(signed=False, label="DISPID_SpeechLexiconProns"),
    "DISPID_SpeechLexiconPronunciation": SimTypeInt(signed=False, label="DISPID_SpeechLexiconPronunciation"),
    "DISPID_SpeechLexiconWord": SimTypeInt(signed=False, label="DISPID_SpeechLexiconWord"),
    "DISPID_SpeechLexiconWords": SimTypeInt(signed=False, label="DISPID_SpeechLexiconWords"),
    "DISPID_SpeechMMSysAudio": SimTypeInt(signed=False, label="DISPID_SpeechMMSysAudio"),
    "DISPID_SpeechMemoryStream": SimTypeInt(signed=False, label="DISPID_SpeechMemoryStream"),
    "DISPID_SpeechObjectToken": SimTypeInt(signed=False, label="DISPID_SpeechObjectToken"),
    "DISPID_SpeechObjectTokenCategory": SimTypeInt(signed=False, label="DISPID_SpeechObjectTokenCategory"),
    "DISPID_SpeechObjectTokens": SimTypeInt(signed=False, label="DISPID_SpeechObjectTokens"),
    "DISPID_SpeechPhoneConverter": SimTypeInt(signed=False, label="DISPID_SpeechPhoneConverter"),
    "DISPID_SpeechPhraseAlternate": SimTypeInt(signed=False, label="DISPID_SpeechPhraseAlternate"),
    "DISPID_SpeechPhraseAlternates": SimTypeInt(signed=False, label="DISPID_SpeechPhraseAlternates"),
    "DISPID_SpeechPhraseBuilder": SimTypeInt(signed=False, label="DISPID_SpeechPhraseBuilder"),
    "DISPID_SpeechPhraseElement": SimTypeInt(signed=False, label="DISPID_SpeechPhraseElement"),
    "DISPID_SpeechPhraseElements": SimTypeInt(signed=False, label="DISPID_SpeechPhraseElements"),
    "DISPID_SpeechPhraseInfo": SimTypeInt(signed=False, label="DISPID_SpeechPhraseInfo"),
    "DISPID_SpeechPhraseProperties": SimTypeInt(signed=False, label="DISPID_SpeechPhraseProperties"),
    "DISPID_SpeechPhraseProperty": SimTypeInt(signed=False, label="DISPID_SpeechPhraseProperty"),
    "DISPID_SpeechPhraseReplacement": SimTypeInt(signed=False, label="DISPID_SpeechPhraseReplacement"),
    "DISPID_SpeechPhraseReplacements": SimTypeInt(signed=False, label="DISPID_SpeechPhraseReplacements"),
    "DISPID_SpeechPhraseRule": SimTypeInt(signed=False, label="DISPID_SpeechPhraseRule"),
    "DISPID_SpeechPhraseRules": SimTypeInt(signed=False, label="DISPID_SpeechPhraseRules"),
    "DISPID_SpeechRecoContext": SimTypeInt(signed=False, label="DISPID_SpeechRecoContext"),
    "DISPID_SpeechRecoContextEvents": SimTypeInt(signed=False, label="DISPID_SpeechRecoContextEvents"),
    "DISPID_SpeechRecoResult": SimTypeInt(signed=False, label="DISPID_SpeechRecoResult"),
    "DISPID_SpeechRecoResult2": SimTypeInt(signed=False, label="DISPID_SpeechRecoResult2"),
    "DISPID_SpeechRecoResultTimes": SimTypeInt(signed=False, label="DISPID_SpeechRecoResultTimes"),
    "DISPID_SpeechRecognizer": SimTypeInt(signed=False, label="DISPID_SpeechRecognizer"),
    "DISPID_SpeechRecognizerStatus": SimTypeInt(signed=False, label="DISPID_SpeechRecognizerStatus"),
    "DISPID_SpeechVoice": SimTypeInt(signed=False, label="DISPID_SpeechVoice"),
    "DISPID_SpeechVoiceEvent": SimTypeInt(signed=False, label="DISPID_SpeechVoiceEvent"),
    "DISPID_SpeechVoiceStatus": SimTypeInt(signed=False, label="DISPID_SpeechVoiceStatus"),
    "DISPID_SpeechWaveFormatEx": SimTypeInt(signed=False, label="DISPID_SpeechWaveFormatEx"),
    "DISPID_SpeechXMLRecoResult": SimTypeInt(signed=False, label="DISPID_SpeechXMLRecoResult"),
    "DISPID_StrokeEvent": SimTypeInt(signed=False, label="DISPID_StrokeEvent"),
    "DISPID_TUNER": SimTypeInt(signed=False, label="DISPID_TUNER"),
    "DISPLAYCONFIG_2DREGION": SimStruct(OrderedDict((("cx", SimTypeInt(signed=False, label="UInt32")), ("cy", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_2DREGION", pack=False, align=None),
    "DISPLAYCONFIG_ADAPTER_NAME": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("adapterDevicePath", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="DISPLAYCONFIG_ADAPTER_NAME", pack=False, align=None),
    "DISPLAYCONFIG_COLOR_ENCODING": SimTypeInt(signed=False, label="DISPLAYCONFIG_COLOR_ENCODING"),
    "DISPLAYCONFIG_DESKTOP_IMAGE_INFO": SimStruct(OrderedDict((("PathSourceSize", SimTypeRef("POINTL", SimStruct)), ("DesktopImageRegion", SimTypeRef("RECTL", SimStruct)), ("DesktopImageClip", SimTypeRef("RECTL", SimStruct)),)), name="DISPLAYCONFIG_DESKTOP_IMAGE_INFO", pack=False, align=None),
    "DISPLAYCONFIG_DEVICE_INFO_HEADER": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="DISPLAYCONFIG_DEVICE_INFO_TYPE")), ("size", SimTypeInt(signed=False, label="UInt32")), ("adapterId", SimTypeRef("LUID", SimStruct)), ("id", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_DEVICE_INFO_HEADER", pack=False, align=None),
    "DISPLAYCONFIG_DEVICE_INFO_TYPE": SimTypeInt(signed=False, label="DISPLAYCONFIG_DEVICE_INFO_TYPE"),
    "DISPLAYCONFIG_GET_ADVANCED_COLOR_INFO": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("colorEncoding", SimTypeInt(signed=False, label="DISPLAYCONFIG_COLOR_ENCODING")), ("bitsPerColorChannel", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_GET_ADVANCED_COLOR_INFO", pack=False, align=None),
    "DISPLAYCONFIG_GET_MONITOR_SPECIALIZATION": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DISPLAYCONFIG_GET_MONITOR_SPECIALIZATION", pack=False, align=None),
    "DISPLAYCONFIG_MODE_INFO": SimStruct(OrderedDict((("infoType", SimTypeInt(signed=False, label="DISPLAYCONFIG_MODE_INFO_TYPE")), ("id", SimTypeInt(signed=False, label="UInt32")), ("adapterId", SimTypeRef("LUID", SimStruct)), ("Anonymous", SimUnion({"targetMode": SimTypeRef("DISPLAYCONFIG_TARGET_MODE", SimStruct), "sourceMode": SimTypeRef("DISPLAYCONFIG_SOURCE_MODE", SimStruct), "desktopImageInfo": SimTypeRef("DISPLAYCONFIG_DESKTOP_IMAGE_INFO", SimStruct)}, name="<anon>", label="None")),)), name="DISPLAYCONFIG_MODE_INFO", pack=False, align=None),
    "DISPLAYCONFIG_MODE_INFO_TYPE": SimTypeInt(signed=False, label="DISPLAYCONFIG_MODE_INFO_TYPE"),
    "DISPLAYCONFIG_PATH_INFO": SimStruct(OrderedDict((("sourceInfo", SimTypeRef("DISPLAYCONFIG_PATH_SOURCE_INFO", SimStruct)), ("targetInfo", SimTypeRef("DISPLAYCONFIG_PATH_TARGET_INFO", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_PATH_INFO", pack=False, align=None),
    "DISPLAYCONFIG_PATH_SOURCE_INFO": SimStruct(OrderedDict((("adapterId", SimTypeRef("LUID", SimStruct)), ("id", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"modeInfoIdx": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("statusFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_PATH_SOURCE_INFO", pack=False, align=None),
    "DISPLAYCONFIG_PATH_TARGET_INFO": SimStruct(OrderedDict((("adapterId", SimTypeRef("LUID", SimStruct)), ("id", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"modeInfoIdx": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("outputTechnology", SimTypeInt(signed=False, label="DISPLAYCONFIG_VIDEO_OUTPUT_TECHNOLOGY")), ("rotation", SimTypeInt(signed=False, label="DISPLAYCONFIG_ROTATION")), ("scaling", SimTypeInt(signed=False, label="DISPLAYCONFIG_SCALING")), ("refreshRate", SimTypeRef("DISPLAYCONFIG_RATIONAL", SimStruct)), ("scanLineOrdering", SimTypeInt(signed=False, label="DISPLAYCONFIG_SCANLINE_ORDERING")), ("targetAvailable", SimTypeInt(signed=True, label="Int32")), ("statusFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_PATH_TARGET_INFO", pack=False, align=None),
    "DISPLAYCONFIG_PIXELFORMAT": SimTypeInt(signed=False, label="DISPLAYCONFIG_PIXELFORMAT"),
    "DISPLAYCONFIG_RATIONAL": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_RATIONAL", pack=False, align=None),
    "DISPLAYCONFIG_ROTATION": SimTypeInt(signed=False, label="DISPLAYCONFIG_ROTATION"),
    "DISPLAYCONFIG_SCALING": SimTypeInt(signed=False, label="DISPLAYCONFIG_SCALING"),
    "DISPLAYCONFIG_SCANLINE_ORDERING": SimTypeInt(signed=False, label="DISPLAYCONFIG_SCANLINE_ORDERING"),
    "DISPLAYCONFIG_SDR_WHITE_LEVEL": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("SDRWhiteLevel", SimTypeInt(signed=False, label="UInt32")),)), name="DISPLAYCONFIG_SDR_WHITE_LEVEL", pack=False, align=None),
    "DISPLAYCONFIG_SET_ADVANCED_COLOR_STATE": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DISPLAYCONFIG_SET_ADVANCED_COLOR_STATE", pack=False, align=None),
    "DISPLAYCONFIG_SET_MONITOR_SPECIALIZATION": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("specializationType", SimTypeBottom(label="Guid")), ("specializationSubType", SimTypeBottom(label="Guid")), ("specializationApplicationName", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="DISPLAYCONFIG_SET_MONITOR_SPECIALIZATION", pack=False, align=None),
    "DISPLAYCONFIG_SET_TARGET_PERSISTENCE": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DISPLAYCONFIG_SET_TARGET_PERSISTENCE", pack=False, align=None),
    "DISPLAYCONFIG_SOURCE_DEVICE_NAME": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("viewGdiDeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="DISPLAYCONFIG_SOURCE_DEVICE_NAME", pack=False, align=None),
    "DISPLAYCONFIG_SOURCE_MODE": SimStruct(OrderedDict((("width", SimTypeInt(signed=False, label="UInt32")), ("height", SimTypeInt(signed=False, label="UInt32")), ("pixelFormat", SimTypeInt(signed=False, label="DISPLAYCONFIG_PIXELFORMAT")), ("position", SimTypeRef("POINTL", SimStruct)),)), name="DISPLAYCONFIG_SOURCE_MODE", pack=False, align=None),
    "DISPLAYCONFIG_SUPPORT_VIRTUAL_RESOLUTION": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DISPLAYCONFIG_SUPPORT_VIRTUAL_RESOLUTION", pack=False, align=None),
    "DISPLAYCONFIG_TARGET_BASE_TYPE": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("baseOutputTechnology", SimTypeInt(signed=False, label="DISPLAYCONFIG_VIDEO_OUTPUT_TECHNOLOGY")),)), name="DISPLAYCONFIG_TARGET_BASE_TYPE", pack=False, align=None),
    "DISPLAYCONFIG_TARGET_DEVICE_NAME": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("flags", SimTypeRef("DISPLAYCONFIG_TARGET_DEVICE_NAME_FLAGS", SimStruct)), ("outputTechnology", SimTypeInt(signed=False, label="DISPLAYCONFIG_VIDEO_OUTPUT_TECHNOLOGY")), ("edidManufactureId", SimTypeShort(signed=False, label="UInt16")), ("edidProductCodeId", SimTypeShort(signed=False, label="UInt16")), ("connectorInstance", SimTypeInt(signed=False, label="UInt32")), ("monitorFriendlyDeviceName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("monitorDevicePath", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="DISPLAYCONFIG_TARGET_DEVICE_NAME", pack=False, align=None),
    "DISPLAYCONFIG_TARGET_DEVICE_NAME_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DISPLAYCONFIG_TARGET_DEVICE_NAME_FLAGS", pack=False, align=None),
    "DISPLAYCONFIG_TARGET_MODE": SimStruct(OrderedDict((("targetVideoSignalInfo", SimTypeRef("DISPLAYCONFIG_VIDEO_SIGNAL_INFO", SimStruct)),)), name="DISPLAYCONFIG_TARGET_MODE", pack=False, align=None),
    "DISPLAYCONFIG_TARGET_PREFERRED_MODE": SimStruct(OrderedDict((("header", SimTypeRef("DISPLAYCONFIG_DEVICE_INFO_HEADER", SimStruct)), ("width", SimTypeInt(signed=False, label="UInt32")), ("height", SimTypeInt(signed=False, label="UInt32")), ("targetMode", SimTypeRef("DISPLAYCONFIG_TARGET_MODE", SimStruct)),)), name="DISPLAYCONFIG_TARGET_PREFERRED_MODE", pack=False, align=None),
    "DISPLAYCONFIG_TOPOLOGY_ID": SimTypeInt(signed=False, label="DISPLAYCONFIG_TOPOLOGY_ID"),
    "DISPLAYCONFIG_VIDEO_OUTPUT_TECHNOLOGY": SimTypeInt(signed=False, label="DISPLAYCONFIG_VIDEO_OUTPUT_TECHNOLOGY"),
    "DISPLAYCONFIG_VIDEO_SIGNAL_INFO": SimStruct(OrderedDict((("pixelRate", SimTypeLongLong(signed=False, label="UInt64")), ("hSyncFreq", SimTypeRef("DISPLAYCONFIG_RATIONAL", SimStruct)), ("vSyncFreq", SimTypeRef("DISPLAYCONFIG_RATIONAL", SimStruct)), ("activeSize", SimTypeRef("DISPLAYCONFIG_2DREGION", SimStruct)), ("totalSize", SimTypeRef("DISPLAYCONFIG_2DREGION", SimStruct)), ("Anonymous", SimUnion({"AdditionalSignalInfo": SimTypeRef("_AdditionalSignalInfo_e__Struct", SimStruct), "videoStandard": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("scanLineOrdering", SimTypeInt(signed=False, label="DISPLAYCONFIG_SCANLINE_ORDERING")),)), name="DISPLAYCONFIG_VIDEO_SIGNAL_INFO", pack=False, align=None),
    "DISPLAYID_DETAILED_TIMING_TYPE_I": SimStruct(OrderedDict((("Anonymous1", SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None)), ("HorizontalActivePixels", SimTypeShort(signed=False, label="UInt16")), ("HorizontalBlankPixels", SimTypeShort(signed=False, label="UInt16")), ("Anonymous2", SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous2_e__Struct", pack=False, align=None)), ("HorizontalSyncWidth", SimTypeShort(signed=False, label="UInt16")), ("VerticalActiveLines", SimTypeShort(signed=False, label="UInt16")), ("VerticalBlankLines", SimTypeShort(signed=False, label="UInt16")), ("Anonymous3", SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous3_e__Struct", pack=False, align=None)), ("VerticalSyncWidth", SimTypeShort(signed=False, label="UInt16")),)), name="DISPLAYID_DETAILED_TIMING_TYPE_I", pack=False, align=None),
    "DISPLAYID_DETAILED_TIMING_TYPE_I_ASPECT_RATIO": SimTypeInt(signed=False, label="DISPLAYID_DETAILED_TIMING_TYPE_I_ASPECT_RATIO"),
    "DISPLAYID_DETAILED_TIMING_TYPE_I_SCANNING_MODE": SimTypeInt(signed=False, label="DISPLAYID_DETAILED_TIMING_TYPE_I_SCANNING_MODE"),
    "DISPLAYID_DETAILED_TIMING_TYPE_I_STEREO_MODE": SimTypeInt(signed=False, label="DISPLAYID_DETAILED_TIMING_TYPE_I_STEREO_MODE"),
    "DISPLAYID_DETAILED_TIMING_TYPE_I_SYNC_POLARITY": SimTypeInt(signed=False, label="DISPLAYID_DETAILED_TIMING_TYPE_I_SYNC_POLARITY"),
    "DISPLAY_BREAK": SimTypeInt(signed=False, label="DISPLAY_BREAK"),
    "DISPLAY_BRIGHTNESS": SimStruct(OrderedDict((("ucDisplayPolicy", SimTypeChar(label="Byte")), ("ucACBrightness", SimTypeChar(label="Byte")), ("ucDCBrightness", SimTypeChar(label="Byte")),)), name="DISPLAY_BRIGHTNESS", pack=False, align=None),
    "DISPLAY_DEVICEA": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("DeviceString", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("StateFlags", SimTypeInt(signed=False, label="UInt32")), ("DeviceID", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("DeviceKey", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="DISPLAY_DEVICEA", pack=False, align=None),
    "DISPLAY_DEVICEW": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DeviceString", SimTypeArray(SimTypeChar(label="Char"), 128)), ("StateFlags", SimTypeInt(signed=False, label="UInt32")), ("DeviceID", SimTypeArray(SimTypeChar(label="Char"), 128)), ("DeviceKey", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="DISPLAY_DEVICEW", pack=False, align=None),
    "DISPLAY_DEVICE_TYPE": SimTypeInt(signed=False, label="DISPLAY_DEVICE_TYPE"),
    "DISPLAY_GRAVITY": SimTypeInt(signed=False, label="DISPLAY_GRAVITY"),
    "DISPLAY_MOVEUNIT": SimTypeInt(signed=False, label="DISPLAY_MOVEUNIT"),
    "DISPPARAMS": SimStruct(OrderedDict((("rgvarg", SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)), ("rgdispidNamedArgs", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("cArgs", SimTypeInt(signed=False, label="UInt32")), ("cNamedArgs", SimTypeInt(signed=False, label="UInt32")),)), name="DISPPARAMS", pack=False, align=None),
    "DISP_CHANGE": SimTypeInt(signed=False, label="DISP_CHANGE"),
    "DISystemMonitor": SimTypeBottom(label="DISystemMonitor"),
    "DISystemMonitorEvents": SimTypeBottom(label="DISystemMonitorEvents"),
    "DISystemMonitorInternal": SimTypeBottom(label="DISystemMonitorInternal"),
    "DIUNINSTALLDRIVER_FLAGS": SimTypeInt(signed=False, label="DIUNINSTALLDRIVER_FLAGS"),
    "DI_FLAGS": SimTypeInt(signed=False, label="DI_FLAGS"),
    "DI_FUNCTION": SimTypeInt(signed=False, label="DI_FUNCTION"),
    "DLGITEMTEMPLATE": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="UInt32")), ("dwExtendedStyle", SimTypeInt(signed=False, label="UInt32")), ("x", SimTypeShort(signed=True, label="Int16")), ("y", SimTypeShort(signed=True, label="Int16")), ("cx", SimTypeShort(signed=True, label="Int16")), ("cy", SimTypeShort(signed=True, label="Int16")), ("id", SimTypeShort(signed=False, label="UInt16")),)), name="DLGITEMTEMPLATE", pack=False, align=None),
    "DLGPAGE": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("DlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pTabName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("IconID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous", SimUnion({"DlgTemplateID": SimTypeShort(signed=False, label="UInt16"), "hDlgTemplate": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")),)), name="DLGPAGE", pack=False, align=None),
    "DLGPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "DLGTEMPLATE": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="UInt32")), ("dwExtendedStyle", SimTypeInt(signed=False, label="UInt32")), ("cdit", SimTypeShort(signed=False, label="UInt16")), ("x", SimTypeShort(signed=True, label="Int16")), ("y", SimTypeShort(signed=True, label="Int16")), ("cx", SimTypeShort(signed=True, label="Int16")), ("cy", SimTypeShort(signed=True, label="Int16")),)), name="DLGTEMPLATE", pack=False, align=None),
    "DLG_BUTTON_CHECK_STATE": SimTypeInt(signed=False, label="DLG_BUTTON_CHECK_STATE"),
    "DLG_DIR_LIST_FILE_TYPE": SimTypeInt(signed=False, label="DLG_DIR_LIST_FILE_TYPE"),
    "DLLGETVERSIONPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DLLVERSIONINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "DLLVERSIONINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("dwPlatformID", SimTypeInt(signed=False, label="UInt32")),)), name="DLLVERSIONINFO", pack=False, align=None),
    "DLLVERSIONINFO2": SimStruct(OrderedDict((("info1", SimTypeRef("DLLVERSIONINFO", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ullVersion", SimTypeLongLong(signed=False, label="UInt64")),)), name="DLLVERSIONINFO2", pack=False, align=None),
    "DLSHEADER": SimStruct(OrderedDict((("cInstruments", SimTypeInt(signed=False, label="UInt32")),)), name="DLSHEADER", pack=False, align=None),
    "DLSID": SimStruct(OrderedDict((("ulData1", SimTypeInt(signed=False, label="UInt32")), ("usData2", SimTypeShort(signed=False, label="UInt16")), ("usData3", SimTypeShort(signed=False, label="UInt16")), ("abData4", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="DLSID", pack=False, align=None),
    "DLSVERSION": SimStruct(OrderedDict((("dwVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwVersionLS", SimTypeInt(signed=False, label="UInt32")),)), name="DLSVERSION", pack=False, align=None),
    "DL_ADDRESS_TYPE": SimTypeInt(signed=False, label="DL_ADDRESS_TYPE"),
    "DL_EI48": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None"),
    "DL_EI64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 5)}, name="<anon>", label="None"),
    "DL_EUI48": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 6), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "DL_EUI64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 8), "Value": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous", SimUnion({"Ei64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 5)}, name="<anon>", label="None"), "Anonymous": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Tse", SimTypeChar(label="Byte")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "DL_OUI": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "DL_TEREDO_ADDRESS": SimStruct(OrderedDict((("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Anonymous", SimUnion({"Eui64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 8), "Value": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous", SimUnion({"Ei64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 5)}, name="<anon>", label="None"), "Anonymous": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Tse", SimTypeChar(label="Byte")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous": SimStruct(OrderedDict((("Flags", SimTypeShort(signed=False, label="UInt16")), ("MappedPort", SimTypeShort(signed=False, label="UInt16")), ("MappedAddress", SimTypeRef("IN_ADDR", SimStruct)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="DL_TEREDO_ADDRESS", pack=False, align=None),
    "DL_TEREDO_ADDRESS_PRV": SimStruct(OrderedDict((("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Anonymous", SimUnion({"Eui64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 8), "Value": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous", SimUnion({"Ei64": SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 5)}, name="<anon>", label="None"), "Anonymous": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Tse", SimTypeChar(label="Byte")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous": SimStruct(OrderedDict((("Flags", SimTypeShort(signed=False, label="UInt16")), ("MappedPort", SimTypeShort(signed=False, label="UInt16")), ("MappedAddress", SimTypeRef("IN_ADDR", SimStruct)), ("LocalAddress", SimTypeRef("IN_ADDR", SimStruct)), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("LocalPort", SimTypeShort(signed=False, label="UInt16")), ("DlDestination", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 6), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="DL_TEREDO_ADDRESS_PRV", pack=False, align=None),
    "DL_TUNNEL_ADDRESS": SimStruct(OrderedDict((("CompartmentId", SimTypeInt(signed=False, label="COMPARTMENT_ID")), ("ScopeId", SimTypeRef("SCOPE_ID", SimStruct)), ("IpAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DL_TUNNEL_ADDRESS", pack=False, align=None),
    "DMA_ADAPTER": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("DmaOperations", SimTypePointer(SimTypeRef("DMA_OPERATIONS", SimStruct), offset=0)),)), name="DMA_ADAPTER", pack=False, align=None),
    "DMA_ADAPTER_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"V1": SimTypeRef("DMA_ADAPTER_INFO_V1", SimStruct), "Crashdump": SimTypeRef("DMA_ADAPTER_INFO_CRASHDUMP", SimStruct)}, name="<anon>", label="None")),)), name="DMA_ADAPTER_INFO", pack=False, align=None),
    "DMA_ADAPTER_INFO_CRASHDUMP": SimStruct(OrderedDict((("DeviceDescription", SimTypeRef("DEVICE_DESCRIPTION", SimStruct)), ("DeviceIdSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DeviceId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DMA_ADAPTER_INFO_CRASHDUMP", pack=False, align=None),
    "DMA_ADAPTER_INFO_V1": SimStruct(OrderedDict((("ReadDmaCounterAvailable", SimTypeInt(signed=False, label="UInt32")), ("ScatterGatherLimit", SimTypeInt(signed=False, label="UInt32")), ("DmaAddressWidth", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MinimumTransferUnit", SimTypeInt(signed=False, label="UInt32")),)), name="DMA_ADAPTER_INFO_V1", pack=False, align=None),
    "DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION": SimStruct(OrderedDict((("ConfigType", SimTypeInt(signed=False, label="DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION_TYPE")), ("Anonymous", SimUnion({"LogicalAddressLimits": SimTypeRef("_LogicalAddressLimits_e__Struct", SimStruct), "SubSection": SimTypeRef("_SubSection_e__Struct", SimStruct), "HardwareAccessType": SimTypeInt(signed=False, label="DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION_ACCESS_TYPE"), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)}, name="<anon>", label="None")),)), name="DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION", pack=False, align=None),
    "DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION_ACCESS_TYPE": SimTypeInt(signed=False, label="DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION_ACCESS_TYPE"),
    "DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION_TYPE": SimTypeInt(signed=False, label="DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION_TYPE"),
    "DMA_COMMON_BUFFER_VECTOR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "DMA_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="DMA_COMPLETION_STATUS")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "DeviceObject", "CompletionContext", "Status"]), offset=0),
    "DMA_COMPLETION_STATUS": SimTypeInt(signed=False, label="DMA_COMPLETION_STATUS"),
    "DMA_CONFIGURATION_BYTE0": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="DMA_CONFIGURATION_BYTE0", pack=False, align=None),
    "DMA_CONFIGURATION_BYTE1": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="DMA_CONFIGURATION_BYTE1", pack=False, align=None),
    "DMA_DES": SimStruct(OrderedDict((("DD_Count", SimTypeInt(signed=False, label="UInt32")), ("DD_Type", SimTypeInt(signed=False, label="UInt32")), ("DD_Flags", SimTypeInt(signed=False, label="DD_FLAGS")), ("DD_Alloc_Chan", SimTypeInt(signed=False, label="UInt32")),)), name="DMA_DES", pack=False, align=None),
    "DMA_IOMMU_INTERFACE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("CreateDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeleteDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("AttachDevice", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DetachDevice", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FlushDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FlushDomainByVaList", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryInputMappings", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapIdentityRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapIdentityRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("SetDeviceFaultReporting", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("ConfigureDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="DMA_IOMMU_INTERFACE", pack=False, align=None),
    "DMA_IOMMU_INTERFACE_EX": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"V1": SimTypeRef("DMA_IOMMU_INTERFACE_V1", SimStruct), "V2": SimTypeRef("DMA_IOMMU_INTERFACE_V2", SimStruct)}, name="<anon>", label="None")),)), name="DMA_IOMMU_INTERFACE_EX", pack=False, align=None),
    "DMA_IOMMU_INTERFACE_V1": SimStruct(OrderedDict((("CreateDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeleteDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("AttachDevice", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DetachDevice", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FlushDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FlushDomainByVaList", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryInputMappings", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapIdentityRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapIdentityRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("SetDeviceFaultReporting", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("ConfigureDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="DMA_IOMMU_INTERFACE_V1", pack=False, align=None),
    "DMA_IOMMU_INTERFACE_V2": SimStruct(OrderedDict((("CreateDomainEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeleteDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("AttachDeviceEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DetachDeviceEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FlushDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FlushDomainByVaList", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryInputMappings", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapLogicalRangeEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapIdentityRangeEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapIdentityRangeEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("SetDeviceFaultReportingEx", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("ConfigureDomain", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryAvailableDomainTypes", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("RegisterInterfaceStateChangeCallback", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnregisterInterfaceStateChangeCallback", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("ReserveLogicalAddressRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FreeReservedLogicalAddressRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MapReservedLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmapReservedLogicalRange", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("CreateDevice", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeleteDevice", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="DMA_IOMMU_INTERFACE_V2", pack=False, align=None),
    "DMA_OPERATIONS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("PutDmaAdapter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter"]), offset=0)), ("AllocateCommonBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeChar(label="Byte")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["DmaAdapter", "Length", "LogicalAddress", "CacheEnabled"]), offset=0)), ("FreeCommonBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Length", "LogicalAddress", "VirtualAddress", "CacheEnabled"]), offset=0)), ("AllocateAdapterChannel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"), arg_names=["DeviceObject", "Irp", "MapRegisterBase", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "NumberOfMapRegisters", "ExecutionRoutine", "Context"]), offset=0)), ("FlushAdapterBuffers", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "CurrentVa", "Length", "WriteToDevice"]), offset=0)), ("FreeAdapterChannel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter"]), offset=0)), ("FreeMapRegisters", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "MapRegisterBase", "NumberOfMapRegisters"]), offset=0)), ("MapTransfer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte")], SimTypeLongLong(signed=True, label="Int64"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "CurrentVa", "Length", "WriteToDevice"]), offset=0)), ("GetDmaAlignment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["DmaAdapter"]), offset=0)), ("ReadDmaCounter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["DmaAdapter"]), offset=0)), ("GetScatterGatherList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "Mdl", "CurrentVa", "Length", "ExecutionRoutine", "Context", "WriteToDevice"]), offset=0)), ("PutScatterGatherList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "ScatterGather", "WriteToDevice"]), offset=0)), ("CalculateScatterGatherList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "CurrentVa", "Length", "ScatterGatherListSize", "pNumberOfMapRegisters"]), offset=0)), ("BuildScatterGatherList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "Mdl", "CurrentVa", "Length", "ExecutionRoutine", "Context", "WriteToDevice", "ScatterGatherBuffer", "ScatterGatherLength"]), offset=0)), ("BuildMdlFromScatterGatherList", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "ScatterGather", "OriginalMdl", "TargetMdl"]), offset=0)), ("GetDmaAdapterInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DMA_ADAPTER_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "AdapterInfo"]), offset=0)), ("GetDmaTransferInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("DMA_TRANSFER_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "Offset", "Length", "WriteOnly", "TransferInfo"]), offset=0)), ("InitializeDmaTransferContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DmaTransferContext"]), offset=0)), ("AllocateCommonBufferEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["DmaAdapter", "MaximumAddress", "Length", "LogicalAddress", "CacheEnabled", "PreferredNode"]), offset=0)), ("AllocateAdapterChannelEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"), arg_names=["DeviceObject", "Irp", "MapRegisterBase", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext", "NumberOfMapRegisters", "Flags", "ExecutionRoutine", "ExecutionContext", "MapRegisterBase"]), offset=0)), ("ConfigureAdapterChannel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "FunctionNumber", "Context"]), offset=0)), ("CancelAdapterChannel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext"]), offset=0)), ("MapTransferEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "Offset", "DeviceOffset", "Length", "WriteToDevice", "ScatterGatherBuffer", "ScatterGatherBufferLength", "DmaCompletionRoutine", "CompletionContext"]), offset=0)), ("GetScatterGatherListEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext", "Mdl", "Offset", "Length", "Flags", "ExecutionRoutine", "Context", "WriteToDevice", "DmaCompletionRoutine", "CompletionContext", "ScatterGatherList"]), offset=0)), ("BuildScatterGatherListEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext", "Mdl", "Offset", "Length", "Flags", "ExecutionRoutine", "Context", "WriteToDevice", "ScatterGatherBuffer", "ScatterGatherLength", "DmaCompletionRoutine", "CompletionContext", "ScatterGatherList"]), offset=0)), ("FlushAdapterBuffersEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "Offset", "Length", "WriteToDevice"]), offset=0)), ("FreeAdapterObject", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "AllocationAction"]), offset=0)), ("CancelMappedTransfer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DmaTransferContext"]), offset=0)), ("AllocateDomainCommonBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DomainHandle", "MaximumAddress", "Length", "Flags", "CacheType", "PreferredNode", "LogicalAddress", "VirtualAddress"]), offset=0)), ("FlushDmaBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "ReadOperation"]), offset=0)), ("JoinDmaDomain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DomainHandle"]), offset=0)), ("LeaveDmaDomain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter"]), offset=0)), ("GetDmaDomain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["DmaAdapter"]), offset=0)), ("AllocateCommonBufferWithBounds", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["DmaAdapter", "MinimumAddress", "MaximumAddress", "Length", "Flags", "CacheType", "PreferredNode", "LogicalAddress"]), offset=0)), ("AllocateCommonBufferVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "LowAddress", "HighAddress", "CacheType", "IdealNode", "Flags", "NumberOfElements", "SizeOfElements", "VectorOut"]), offset=0)), ("GetCommonBufferFromVectorByIndex", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Vector", "Index", "VirtualAddressOut", "LogicalAddressOut"]), offset=0)), ("FreeCommonBufferFromVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Vector", "Index"]), offset=0)), ("FreeCommonBufferVector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Vector"]), offset=0)), ("CreateCommonBufferFromMdl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "ExtendedConfigs", "ExtendedConfigsCount", "LogicalAddress"]), offset=0)),)), name="DMA_OPERATIONS", pack=False, align=None),
    "DMA_RANGE": SimStruct(OrderedDict((("DR_Min", SimTypeInt(signed=False, label="UInt32")), ("DR_Max", SimTypeInt(signed=False, label="UInt32")), ("DR_Flags", SimTypeInt(signed=False, label="DD_FLAGS")),)), name="DMA_RANGE", pack=False, align=None),
    "DMA_RESOURCE": SimStruct(OrderedDict((("DMA_Header", SimTypeRef("DMA_DES", SimStruct)), ("DMA_Data", SimTypePointer(SimTypeRef("DMA_RANGE", SimStruct), offset=0)),)), name="DMA_RESOURCE", pack=False, align=None),
    "DMA_SPEED": SimTypeInt(signed=False, label="DMA_SPEED"),
    "DMA_TRANSFER_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"V1": SimTypeRef("DMA_TRANSFER_INFO_V1", SimStruct), "V2": SimTypeRef("DMA_TRANSFER_INFO_V2", SimStruct)}, name="<anon>", label="None")),)), name="DMA_TRANSFER_INFO", pack=False, align=None),
    "DMA_TRANSFER_INFO_V1": SimStruct(OrderedDict((("MapRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("ScatterGatherElementCount", SimTypeInt(signed=False, label="UInt32")), ("ScatterGatherListSize", SimTypeInt(signed=False, label="UInt32")),)), name="DMA_TRANSFER_INFO_V1", pack=False, align=None),
    "DMA_TRANSFER_INFO_V2": SimStruct(OrderedDict((("MapRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("ScatterGatherElementCount", SimTypeInt(signed=False, label="UInt32")), ("ScatterGatherListSize", SimTypeInt(signed=False, label="UInt32")), ("LogicalPageCount", SimTypeInt(signed=False, label="UInt32")),)), name="DMA_TRANSFER_INFO_V2", pack=False, align=None),
    "DMA_WIDTH": SimTypeInt(signed=False, label="DMA_WIDTH"),
    "DML_ACTIVATION_CELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_CELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_ELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_ELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_HARDMAX_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_HARDMAX_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_HARD_SIGMOID_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_HARD_SIGMOID_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_IDENTITY_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_IDENTITY_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_LEAKY_RELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_LEAKY_RELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_LINEAR_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_LINEAR_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_LOG_SOFTMAX_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_LOG_SOFTMAX_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_PARAMETERIZED_RELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("SlopeTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_PARAMETERIZED_RELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_PARAMETRIC_SOFTPLUS_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_PARAMETRIC_SOFTPLUS_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_RELU_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_RELU_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_RELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_RELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SCALED_ELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)), ("Gamma", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_SCALED_ELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SCALED_TANH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_SCALED_TANH_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SHRINK_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Bias", SimTypeFloat(size=32)), ("Threshold", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_SHRINK_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SIGMOID_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_SIGMOID_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SOFTMAX_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_SOFTMAX_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SOFTPLUS_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Steepness", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_SOFTPLUS_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_SOFTSIGN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_SOFTSIGN_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_TANH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ACTIVATION_TANH_OPERATOR_DESC", pack=False, align=None),
    "DML_ACTIVATION_THRESHOLDED_RELU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Alpha", SimTypeFloat(size=32)),)), name="DML_ACTIVATION_THRESHOLDED_RELU_OPERATOR_DESC", pack=False, align=None),
    "DML_ADAM_OPTIMIZER_OPERATOR_DESC": SimStruct(OrderedDict((("InputParametersTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputFirstMomentTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputSecondMomentTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("GradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("TrainingStepTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputParametersTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputFirstMomentTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSecondMomentTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("LearningRate", SimTypeFloat(size=32)), ("Beta1", SimTypeFloat(size=32)), ("Beta2", SimTypeFloat(size=32)), ("Epsilon", SimTypeFloat(size=32)),)), name="DML_ADAM_OPTIMIZER_OPERATOR_DESC", pack=False, align=None),
    "DML_ARGMAX_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AxisCount", SimTypeInt(signed=False, label="UInt32")), ("Axes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AxisDirection", SimTypeInt(signed=False, label="DML_AXIS_DIRECTION")),)), name="DML_ARGMAX_OPERATOR_DESC", pack=False, align=None),
    "DML_ARGMIN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AxisCount", SimTypeInt(signed=False, label="UInt32")), ("Axes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AxisDirection", SimTypeInt(signed=False, label="DML_AXIS_DIRECTION")),)), name="DML_ARGMIN_OPERATOR_DESC", pack=False, align=None),
    "DML_AVERAGE_POOLING_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("IncludePadding", SimTypeInt(signed=True, label="Int32")),)), name="DML_AVERAGE_POOLING_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_AVERAGE_POOLING_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("IncludePadding", SimTypeInt(signed=True, label="Int32")),)), name="DML_AVERAGE_POOLING_OPERATOR_DESC", pack=False, align=None),
    "DML_AXIS_DIRECTION": SimTypeInt(signed=False, label="DML_AXIS_DIRECTION"),
    "DML_BATCH_NORMALIZATION_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("MeanTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("VarianceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputScaleGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputBiasGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Epsilon", SimTypeFloat(size=32)),)), name="DML_BATCH_NORMALIZATION_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_BATCH_NORMALIZATION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("MeanTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("VarianceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Spatial", SimTypeInt(signed=True, label="Int32")), ("Epsilon", SimTypeFloat(size=32)), ("FusedActivation", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)),)), name="DML_BATCH_NORMALIZATION_OPERATOR_DESC", pack=False, align=None),
    "DML_BINDING_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DML_BINDING_TYPE")), ("Desc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DML_BINDING_DESC", pack=False, align=None),
    "DML_BINDING_PROPERTIES": SimStruct(OrderedDict((("RequiredDescriptorCount", SimTypeInt(signed=False, label="UInt32")), ("TemporaryResourceSize", SimTypeLongLong(signed=False, label="UInt64")), ("PersistentResourceSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="DML_BINDING_PROPERTIES", pack=False, align=None),
    "DML_BINDING_TABLE_DESC": SimStruct(OrderedDict((("Dispatchable", SimTypeBottom(label="IDMLDispatchable")), ("CPUDescriptorHandle", SimTypeRef("D3D12_CPU_DESCRIPTOR_HANDLE", SimStruct)), ("GPUDescriptorHandle", SimTypeRef("D3D12_GPU_DESCRIPTOR_HANDLE", SimStruct)), ("SizeInDescriptors", SimTypeInt(signed=False, label="UInt32")),)), name="DML_BINDING_TABLE_DESC", pack=False, align=None),
    "DML_BINDING_TYPE": SimTypeInt(signed=False, label="DML_BINDING_TYPE"),
    "DML_BUFFER_ARRAY_BINDING": SimStruct(OrderedDict((("BindingCount", SimTypeInt(signed=False, label="UInt32")), ("Bindings", SimTypePointer(SimTypeRef("DML_BUFFER_BINDING", SimStruct), offset=0)),)), name="DML_BUFFER_ARRAY_BINDING", pack=False, align=None),
    "DML_BUFFER_BINDING": SimStruct(OrderedDict((("Buffer", SimTypeBottom(label="ID3D12Resource")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="DML_BUFFER_BINDING", pack=False, align=None),
    "DML_BUFFER_TENSOR_DESC": SimStruct(OrderedDict((("DataType", SimTypeInt(signed=False, label="DML_TENSOR_DATA_TYPE")), ("Flags", SimTypeInt(signed=False, label="DML_TENSOR_FLAGS")), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Sizes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("TotalTensorSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("GuaranteedBaseOffsetAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="DML_BUFFER_TENSOR_DESC", pack=False, align=None),
    "DML_CAST_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_CAST_OPERATOR_DESC", pack=False, align=None),
    "DML_CONVOLUTION_DIRECTION": SimTypeInt(signed=False, label="DML_CONVOLUTION_DIRECTION"),
    "DML_CONVOLUTION_INTEGER_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FilterTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FilterZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Dilations", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("GroupCount", SimTypeInt(signed=False, label="UInt32")),)), name="DML_CONVOLUTION_INTEGER_OPERATOR_DESC", pack=False, align=None),
    "DML_CONVOLUTION_MODE": SimTypeInt(signed=False, label="DML_CONVOLUTION_MODE"),
    "DML_CONVOLUTION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FilterTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Mode", SimTypeInt(signed=False, label="DML_CONVOLUTION_MODE")), ("Direction", SimTypeInt(signed=False, label="DML_CONVOLUTION_DIRECTION")), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Dilations", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("OutputPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("GroupCount", SimTypeInt(signed=False, label="UInt32")), ("FusedActivation", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)),)), name="DML_CONVOLUTION_OPERATOR_DESC", pack=False, align=None),
    "DML_CREATE_DEVICE_FLAGS": SimTypeInt(signed=False, label="DML_CREATE_DEVICE_FLAGS"),
    "DML_CUMULATIVE_PRODUCT_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")), ("AxisDirection", SimTypeInt(signed=False, label="DML_AXIS_DIRECTION")), ("HasExclusiveProduct", SimTypeInt(signed=True, label="Int32")),)), name="DML_CUMULATIVE_PRODUCT_OPERATOR_DESC", pack=False, align=None),
    "DML_CUMULATIVE_SUMMATION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")), ("AxisDirection", SimTypeInt(signed=False, label="DML_AXIS_DIRECTION")), ("HasExclusiveSum", SimTypeInt(signed=True, label="Int32")),)), name="DML_CUMULATIVE_SUMMATION_OPERATOR_DESC", pack=False, align=None),
    "DML_DEPTH_SPACE_ORDER": SimTypeInt(signed=False, label="DML_DEPTH_SPACE_ORDER"),
    "DML_DEPTH_TO_SPACE1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BlockSize", SimTypeInt(signed=False, label="UInt32")), ("Order", SimTypeInt(signed=False, label="DML_DEPTH_SPACE_ORDER")),)), name="DML_DEPTH_TO_SPACE1_OPERATOR_DESC", pack=False, align=None),
    "DML_DEPTH_TO_SPACE_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BlockSize", SimTypeInt(signed=False, label="UInt32")),)), name="DML_DEPTH_TO_SPACE_OPERATOR_DESC", pack=False, align=None),
    "DML_DIAGONAL_MATRIX_OPERATOR_DESC": SimStruct(OrderedDict((("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Offset", SimTypeInt(signed=True, label="Int32")), ("Value", SimTypeFloat(size=32)),)), name="DML_DIAGONAL_MATRIX_OPERATOR_DESC", pack=False, align=None),
    "DML_DYNAMIC_QUANTIZE_LINEAR_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_DYNAMIC_QUANTIZE_LINEAR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ABS_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ABS_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ACOSH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ACOSH_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ACOS_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ACOS_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ADD1_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FusedActivation", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ADD1_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ADD_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ADD_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ASINH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ASINH_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ASIN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ASIN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ATANH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ATANH_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ATAN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ATAN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ATAN_YX_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ATAN_YX_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_AND_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_AND_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_COUNT_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_COUNT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_NOT_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_NOT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_OR_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_OR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_SHIFT_LEFT_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_SHIFT_LEFT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_SHIFT_RIGHT_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_SHIFT_RIGHT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_BIT_XOR_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_BIT_XOR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_CEIL_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_CEIL_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_CLIP_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Min", SimTypeFloat(size=32)), ("Max", SimTypeFloat(size=32)),)), name="DML_ELEMENT_WISE_CLIP_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_CLIP_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)), ("Min", SimTypeFloat(size=32)), ("Max", SimTypeFloat(size=32)),)), name="DML_ELEMENT_WISE_CLIP_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_CONSTANT_POW_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)), ("Exponent", SimTypeFloat(size=32)),)), name="DML_ELEMENT_WISE_CONSTANT_POW_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_COSH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_COSH_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_COS_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_COS_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_DEQUANTIZE_LINEAR_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_DEQUANTIZE_LINEAR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_DIFFERENCE_SQUARE_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_DIFFERENCE_SQUARE_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_DIVIDE_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_DIVIDE_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ERF_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_ERF_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_EXP_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_EXP_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_FLOOR_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_FLOOR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_IDENTITY_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_IDENTITY_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_IF_OPERATOR_DESC": SimStruct(OrderedDict((("ConditionTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_IF_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_IS_INFINITY_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InfinityMode", SimTypeInt(signed=False, label="DML_IS_INFINITY_MODE")),)), name="DML_ELEMENT_WISE_IS_INFINITY_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_IS_NAN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_IS_NAN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_AND_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_AND_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_EQUALS_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_EQUALS_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_GREATER_THAN_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_GREATER_THAN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_GREATER_THAN_OR_EQUAL_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_GREATER_THAN_OR_EQUAL_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_LESS_THAN_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_LESS_THAN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_LESS_THAN_OR_EQUAL_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_LESS_THAN_OR_EQUAL_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_NOT_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_NOT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_OR_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_OR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOGICAL_XOR_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOGICAL_XOR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_LOG_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_LOG_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_MAX_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_MAX_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_MEAN_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_MEAN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_MIN_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_MIN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_MODULUS_FLOOR_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_MODULUS_FLOOR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_MODULUS_TRUNCATE_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_MODULUS_TRUNCATE_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_MULTIPLY_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_MULTIPLY_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_POW_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ExponentTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_POW_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_QUANTIZED_LINEAR_ADD_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_QUANTIZED_LINEAR_ADD_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_QUANTIZE_LINEAR_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_QUANTIZE_LINEAR_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_RECIP_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_RECIP_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_ROUND_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("RoundingMode", SimTypeInt(signed=False, label="DML_ROUNDING_MODE")),)), name="DML_ELEMENT_WISE_ROUND_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_SIGN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_SIGN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_SINH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_SINH_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_SIN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_SIN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_SQRT_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_SQRT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_SUBTRACT_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_SUBTRACT_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_TANH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_TANH_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_TAN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)),)), name="DML_ELEMENT_WISE_TAN_OPERATOR_DESC", pack=False, align=None),
    "DML_ELEMENT_WISE_THRESHOLD_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleBias", SimTypePointer(SimTypeRef("DML_SCALE_BIAS", SimStruct), offset=0)), ("Min", SimTypeFloat(size=32)),)), name="DML_ELEMENT_WISE_THRESHOLD_OPERATOR_DESC", pack=False, align=None),
    "DML_EXECUTION_FLAGS": SimTypeInt(signed=False, label="DML_EXECUTION_FLAGS"),
    "DML_FEATURE": SimTypeInt(signed=False, label="DML_FEATURE"),
    "DML_FEATURE_DATA_FEATURE_LEVELS": SimStruct(OrderedDict((("MaxSupportedFeatureLevel", SimTypeInt(signed=False, label="DML_FEATURE_LEVEL")),)), name="DML_FEATURE_DATA_FEATURE_LEVELS", pack=False, align=None),
    "DML_FEATURE_DATA_TENSOR_DATA_TYPE_SUPPORT": SimStruct(OrderedDict((("IsSupported", SimTypeInt(signed=True, label="Int32")),)), name="DML_FEATURE_DATA_TENSOR_DATA_TYPE_SUPPORT", pack=False, align=None),
    "DML_FEATURE_LEVEL": SimTypeInt(signed=False, label="DML_FEATURE_LEVEL"),
    "DML_FEATURE_QUERY_FEATURE_LEVELS": SimStruct(OrderedDict((("RequestedFeatureLevelCount", SimTypeInt(signed=False, label="UInt32")), ("RequestedFeatureLevels", SimTypePointer(SimTypeInt(signed=False, label="DML_FEATURE_LEVEL"), offset=0)),)), name="DML_FEATURE_QUERY_FEATURE_LEVELS", pack=False, align=None),
    "DML_FEATURE_QUERY_TENSOR_DATA_TYPE_SUPPORT": SimStruct(OrderedDict((("DataType", SimTypeInt(signed=False, label="DML_TENSOR_DATA_TYPE")),)), name="DML_FEATURE_QUERY_TENSOR_DATA_TYPE_SUPPORT", pack=False, align=None),
    "DML_FILL_VALUE_CONSTANT_OPERATOR_DESC": SimStruct(OrderedDict((("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ValueDataType", SimTypeInt(signed=False, label="DML_TENSOR_DATA_TYPE")), ("Value", SimUnion({"Bytes": SimTypeArray(SimTypeChar(label="Byte"), 8), "Int8": SimTypeChar(label="SByte"), "UInt8": SimTypeChar(label="Byte"), "Int16": SimTypeShort(signed=True, label="Int16"), "UInt16": SimTypeShort(signed=False, label="UInt16"), "Int32": SimTypeInt(signed=True, label="Int32"), "UInt32": SimTypeInt(signed=False, label="UInt32"), "Int64": SimTypeLongLong(signed=True, label="Int64"), "UInt64": SimTypeLongLong(signed=False, label="UInt64"), "Float32": SimTypeFloat(size=32), "Float64": SimTypeFloat(size=64)}, name="<anon>", label="None")),)), name="DML_FILL_VALUE_CONSTANT_OPERATOR_DESC", pack=False, align=None),
    "DML_FILL_VALUE_SEQUENCE_OPERATOR_DESC": SimStruct(OrderedDict((("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ValueDataType", SimTypeInt(signed=False, label="DML_TENSOR_DATA_TYPE")), ("ValueStart", SimUnion({"Bytes": SimTypeArray(SimTypeChar(label="Byte"), 8), "Int8": SimTypeChar(label="SByte"), "UInt8": SimTypeChar(label="Byte"), "Int16": SimTypeShort(signed=True, label="Int16"), "UInt16": SimTypeShort(signed=False, label="UInt16"), "Int32": SimTypeInt(signed=True, label="Int32"), "UInt32": SimTypeInt(signed=False, label="UInt32"), "Int64": SimTypeLongLong(signed=True, label="Int64"), "UInt64": SimTypeLongLong(signed=False, label="UInt64"), "Float32": SimTypeFloat(size=32), "Float64": SimTypeFloat(size=64)}, name="<anon>", label="None")), ("ValueDelta", SimUnion({"Bytes": SimTypeArray(SimTypeChar(label="Byte"), 8), "Int8": SimTypeChar(label="SByte"), "UInt8": SimTypeChar(label="Byte"), "Int16": SimTypeShort(signed=True, label="Int16"), "UInt16": SimTypeShort(signed=False, label="UInt16"), "Int32": SimTypeInt(signed=True, label="Int32"), "UInt32": SimTypeInt(signed=False, label="UInt32"), "Int64": SimTypeLongLong(signed=True, label="Int64"), "UInt64": SimTypeLongLong(signed=False, label="UInt64"), "Float32": SimTypeFloat(size=32), "Float64": SimTypeFloat(size=64)}, name="<anon>", label="None")),)), name="DML_FILL_VALUE_SEQUENCE_OPERATOR_DESC", pack=False, align=None),
    "DML_GATHER_ELEMENTS_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")),)), name="DML_GATHER_ELEMENTS_OPERATOR_DESC", pack=False, align=None),
    "DML_GATHER_ND1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputDimensionCount", SimTypeInt(signed=False, label="UInt32")), ("IndicesDimensionCount", SimTypeInt(signed=False, label="UInt32")), ("BatchDimensionCount", SimTypeInt(signed=False, label="UInt32")),)), name="DML_GATHER_ND1_OPERATOR_DESC", pack=False, align=None),
    "DML_GATHER_ND_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputDimensionCount", SimTypeInt(signed=False, label="UInt32")), ("IndicesDimensionCount", SimTypeInt(signed=False, label="UInt32")),)), name="DML_GATHER_ND_OPERATOR_DESC", pack=False, align=None),
    "DML_GATHER_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")), ("IndexDimensions", SimTypeInt(signed=False, label="UInt32")),)), name="DML_GATHER_OPERATOR_DESC", pack=False, align=None),
    "DML_GEMM_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("CTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("TransA", SimTypeInt(signed=False, label="DML_MATRIX_TRANSFORM")), ("TransB", SimTypeInt(signed=False, label="DML_MATRIX_TRANSFORM")), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)), ("FusedActivation", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)),)), name="DML_GEMM_OPERATOR_DESC", pack=False, align=None),
    "DML_GRAPH_DESC": SimStruct(OrderedDict((("InputCount", SimTypeInt(signed=False, label="UInt32")), ("OutputCount", SimTypeInt(signed=False, label="UInt32")), ("NodeCount", SimTypeInt(signed=False, label="UInt32")), ("Nodes", SimTypePointer(SimTypeRef("DML_GRAPH_NODE_DESC", SimStruct), offset=0)), ("InputEdgeCount", SimTypeInt(signed=False, label="UInt32")), ("InputEdges", SimTypePointer(SimTypeRef("DML_GRAPH_EDGE_DESC", SimStruct), offset=0)), ("OutputEdgeCount", SimTypeInt(signed=False, label="UInt32")), ("OutputEdges", SimTypePointer(SimTypeRef("DML_GRAPH_EDGE_DESC", SimStruct), offset=0)), ("IntermediateEdgeCount", SimTypeInt(signed=False, label="UInt32")), ("IntermediateEdges", SimTypePointer(SimTypeRef("DML_GRAPH_EDGE_DESC", SimStruct), offset=0)),)), name="DML_GRAPH_DESC", pack=False, align=None),
    "DML_GRAPH_EDGE_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DML_GRAPH_EDGE_TYPE")), ("Desc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DML_GRAPH_EDGE_DESC", pack=False, align=None),
    "DML_GRAPH_EDGE_TYPE": SimTypeInt(signed=False, label="DML_GRAPH_EDGE_TYPE"),
    "DML_GRAPH_NODE_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DML_GRAPH_NODE_TYPE")), ("Desc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DML_GRAPH_NODE_DESC", pack=False, align=None),
    "DML_GRAPH_NODE_TYPE": SimTypeInt(signed=False, label="DML_GRAPH_NODE_TYPE"),
    "DML_GRU_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("WeightTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("RecurrenceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("HiddenInitTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("SequenceLengthsTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSequenceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSingleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ActivationDescCount", SimTypeInt(signed=False, label="UInt32")), ("ActivationDescs", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)), ("Direction", SimTypeInt(signed=False, label="DML_RECURRENT_NETWORK_DIRECTION")), ("LinearBeforeReset", SimTypeInt(signed=True, label="Int32")),)), name="DML_GRU_OPERATOR_DESC", pack=False, align=None),
    "DML_INPUT_GRAPH_EDGE_DESC": SimStruct(OrderedDict((("GraphInputIndex", SimTypeInt(signed=False, label="UInt32")), ("ToNodeIndex", SimTypeInt(signed=False, label="UInt32")), ("ToNodeInputIndex", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DML_INPUT_GRAPH_EDGE_DESC", pack=False, align=None),
    "DML_INTERMEDIATE_GRAPH_EDGE_DESC": SimStruct(OrderedDict((("FromNodeIndex", SimTypeInt(signed=False, label="UInt32")), ("FromNodeOutputIndex", SimTypeInt(signed=False, label="UInt32")), ("ToNodeIndex", SimTypeInt(signed=False, label="UInt32")), ("ToNodeInputIndex", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DML_INTERMEDIATE_GRAPH_EDGE_DESC", pack=False, align=None),
    "DML_INTERPOLATION_MODE": SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE"),
    "DML_IS_INFINITY_MODE": SimTypeInt(signed=False, label="DML_IS_INFINITY_MODE"),
    "DML_JOIN_OPERATOR_DESC": SimStruct(OrderedDict((("InputCount", SimTypeInt(signed=False, label="UInt32")), ("InputTensors", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")),)), name="DML_JOIN_OPERATOR_DESC", pack=False, align=None),
    "DML_LOCAL_RESPONSE_NORMALIZATION_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("CrossChannel", SimTypeInt(signed=True, label="Int32")), ("LocalSize", SimTypeInt(signed=False, label="UInt32")), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)), ("Bias", SimTypeFloat(size=32)),)), name="DML_LOCAL_RESPONSE_NORMALIZATION_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_LOCAL_RESPONSE_NORMALIZATION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("CrossChannel", SimTypeInt(signed=True, label="Int32")), ("LocalSize", SimTypeInt(signed=False, label="UInt32")), ("Alpha", SimTypeFloat(size=32)), ("Beta", SimTypeFloat(size=32)), ("Bias", SimTypeFloat(size=32)),)), name="DML_LOCAL_RESPONSE_NORMALIZATION_OPERATOR_DESC", pack=False, align=None),
    "DML_LP_NORMALIZATION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")), ("Epsilon", SimTypeFloat(size=32)), ("P", SimTypeInt(signed=False, label="UInt32")),)), name="DML_LP_NORMALIZATION_OPERATOR_DESC", pack=False, align=None),
    "DML_LP_POOLING_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("P", SimTypeInt(signed=False, label="UInt32")),)), name="DML_LP_POOLING_OPERATOR_DESC", pack=False, align=None),
    "DML_LSTM_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("WeightTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("RecurrenceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("HiddenInitTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("CellMemInitTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("SequenceLengthsTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("PeepholeTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSequenceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSingleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputCellSingleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ActivationDescCount", SimTypeInt(signed=False, label="UInt32")), ("ActivationDescs", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)), ("Direction", SimTypeInt(signed=False, label="DML_RECURRENT_NETWORK_DIRECTION")), ("ClipThreshold", SimTypeFloat(size=32)), ("UseClipThreshold", SimTypeInt(signed=True, label="Int32")), ("CoupleInputForget", SimTypeInt(signed=True, label="Int32")),)), name="DML_LSTM_OPERATOR_DESC", pack=False, align=None),
    "DML_MATRIX_MULTIPLY_INTEGER_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_MATRIX_MULTIPLY_INTEGER_OPERATOR_DESC", pack=False, align=None),
    "DML_MATRIX_TRANSFORM": SimTypeInt(signed=False, label="DML_MATRIX_TRANSFORM"),
    "DML_MAX_POOLING1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputIndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_MAX_POOLING1_OPERATOR_DESC", pack=False, align=None),
    "DML_MAX_POOLING2_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputIndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Dilations", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_MAX_POOLING2_OPERATOR_DESC", pack=False, align=None),
    "DML_MAX_POOLING_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Dilations", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_MAX_POOLING_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_MAX_POOLING_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("WindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_MAX_POOLING_OPERATOR_DESC", pack=False, align=None),
    "DML_MAX_UNPOOLING_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_MAX_UNPOOLING_OPERATOR_DESC", pack=False, align=None),
    "DML_MEAN_VARIANCE_NORMALIZATION1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AxisCount", SimTypeInt(signed=False, label="UInt32")), ("Axes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("NormalizeVariance", SimTypeInt(signed=True, label="Int32")), ("Epsilon", SimTypeFloat(size=32)), ("FusedActivation", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)),)), name="DML_MEAN_VARIANCE_NORMALIZATION1_OPERATOR_DESC", pack=False, align=None),
    "DML_MEAN_VARIANCE_NORMALIZATION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("CrossChannel", SimTypeInt(signed=True, label="Int32")), ("NormalizeVariance", SimTypeInt(signed=True, label="Int32")), ("Epsilon", SimTypeFloat(size=32)), ("FusedActivation", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)),)), name="DML_MEAN_VARIANCE_NORMALIZATION_OPERATOR_DESC", pack=False, align=None),
    "DML_NONZERO_COORDINATES_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputCountTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputCoordinatesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_NONZERO_COORDINATES_OPERATOR_DESC", pack=False, align=None),
    "DML_ONE_HOT_OPERATOR_DESC": SimStruct(OrderedDict((("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ValuesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")),)), name="DML_ONE_HOT_OPERATOR_DESC", pack=False, align=None),
    "DML_OPERATOR_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DML_OPERATOR_TYPE")), ("Desc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DML_OPERATOR_DESC", pack=False, align=None),
    "DML_OPERATOR_GRAPH_NODE_DESC": SimStruct(OrderedDict((("Operator", SimTypeBottom(label="IDMLOperator")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DML_OPERATOR_GRAPH_NODE_DESC", pack=False, align=None),
    "DML_OPERATOR_TYPE": SimTypeInt(signed=False, label="DML_OPERATOR_TYPE"),
    "DML_OUTPUT_GRAPH_EDGE_DESC": SimStruct(OrderedDict((("FromNodeIndex", SimTypeInt(signed=False, label="UInt32")), ("FromNodeOutputIndex", SimTypeInt(signed=False, label="UInt32")), ("GraphOutputIndex", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DML_OUTPUT_GRAPH_EDGE_DESC", pack=False, align=None),
    "DML_PADDING_MODE": SimTypeInt(signed=False, label="DML_PADDING_MODE"),
    "DML_PADDING_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("PaddingMode", SimTypeInt(signed=False, label="DML_PADDING_MODE")), ("PaddingValue", SimTypeFloat(size=32)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_PADDING_OPERATOR_DESC", pack=False, align=None),
    "DML_QUANTIZED_LINEAR_CONVOLUTION_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FilterTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FilterScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("FilterZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Dilations", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("StartPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("EndPadding", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("GroupCount", SimTypeInt(signed=False, label="UInt32")),)), name="DML_QUANTIZED_LINEAR_CONVOLUTION_OPERATOR_DESC", pack=False, align=None),
    "DML_QUANTIZED_LINEAR_MATRIX_MULTIPLY_OPERATOR_DESC": SimStruct(OrderedDict((("ATensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputScaleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputZeroPointTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)),)), name="DML_QUANTIZED_LINEAR_MATRIX_MULTIPLY_OPERATOR_DESC", pack=False, align=None),
    "DML_RANDOM_GENERATOR_OPERATOR_DESC": SimStruct(OrderedDict((("InputStateTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputStateTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Type", SimTypeInt(signed=False, label="DML_RANDOM_GENERATOR_TYPE")),)), name="DML_RANDOM_GENERATOR_OPERATOR_DESC", pack=False, align=None),
    "DML_RANDOM_GENERATOR_TYPE": SimTypeInt(signed=False, label="DML_RANDOM_GENERATOR_TYPE"),
    "DML_RECURRENT_NETWORK_DIRECTION": SimTypeInt(signed=False, label="DML_RECURRENT_NETWORK_DIRECTION"),
    "DML_REDUCE_FUNCTION": SimTypeInt(signed=False, label="DML_REDUCE_FUNCTION"),
    "DML_REDUCE_OPERATOR_DESC": SimStruct(OrderedDict((("Function", SimTypeInt(signed=False, label="DML_REDUCE_FUNCTION")), ("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("AxisCount", SimTypeInt(signed=False, label="UInt32")), ("Axes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_REDUCE_OPERATOR_DESC", pack=False, align=None),
    "DML_RESAMPLE1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InterpolationMode", SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE")), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Scales", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("InputPixelOffsets", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("OutputPixelOffsets", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="DML_RESAMPLE1_OPERATOR_DESC", pack=False, align=None),
    "DML_RESAMPLE_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InterpolationMode", SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE")), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Scales", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("InputPixelOffsets", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("OutputPixelOffsets", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="DML_RESAMPLE_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_RESAMPLE_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InterpolationMode", SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE")), ("ScaleCount", SimTypeInt(signed=False, label="UInt32")), ("Scales", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="DML_RESAMPLE_OPERATOR_DESC", pack=False, align=None),
    "DML_REVERSE_SUBSEQUENCES_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("SequenceLengthsTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")),)), name="DML_REVERSE_SUBSEQUENCES_OPERATOR_DESC", pack=False, align=None),
    "DML_RNN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("WeightTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("RecurrenceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BiasTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("HiddenInitTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("SequenceLengthsTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSequenceTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputSingleTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ActivationDescCount", SimTypeInt(signed=False, label="UInt32")), ("ActivationDescs", SimTypePointer(SimTypeRef("DML_OPERATOR_DESC", SimStruct), offset=0)), ("Direction", SimTypeInt(signed=False, label="DML_RECURRENT_NETWORK_DIRECTION")),)), name="DML_RNN_OPERATOR_DESC", pack=False, align=None),
    "DML_ROI_ALIGN1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ROITensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BatchIndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ReductionFunction", SimTypeInt(signed=False, label="DML_REDUCE_FUNCTION")), ("InterpolationMode", SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE")), ("SpatialScaleX", SimTypeFloat(size=32)), ("SpatialScaleY", SimTypeFloat(size=32)), ("InputPixelOffset", SimTypeFloat(size=32)), ("OutputPixelOffset", SimTypeFloat(size=32)), ("OutOfBoundsInputValue", SimTypeFloat(size=32)), ("MinimumSamplesPerOutput", SimTypeInt(signed=False, label="UInt32")), ("MaximumSamplesPerOutput", SimTypeInt(signed=False, label="UInt32")), ("AlignRegionsToCorners", SimTypeInt(signed=True, label="Int32")),)), name="DML_ROI_ALIGN1_OPERATOR_DESC", pack=False, align=None),
    "DML_ROI_ALIGN_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ROITensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BatchIndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ReductionFunction", SimTypeInt(signed=False, label="DML_REDUCE_FUNCTION")), ("InterpolationMode", SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE")), ("SpatialScaleX", SimTypeFloat(size=32)), ("SpatialScaleY", SimTypeFloat(size=32)), ("OutOfBoundsInputValue", SimTypeFloat(size=32)), ("MinimumSamplesPerOutput", SimTypeInt(signed=False, label="UInt32")), ("MaximumSamplesPerOutput", SimTypeInt(signed=False, label="UInt32")),)), name="DML_ROI_ALIGN_OPERATOR_DESC", pack=False, align=None),
    "DML_ROI_POOLING_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ROITensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("SpatialScale", SimTypeFloat(size=32)), ("PooledSize", SimTypeRef("DML_SIZE_2D", SimStruct)),)), name="DML_ROI_POOLING_OPERATOR_DESC", pack=False, align=None),
    "DML_ROUNDING_MODE": SimTypeInt(signed=False, label="DML_ROUNDING_MODE"),
    "DML_SCALAR_UNION": SimUnion({"Bytes": SimTypeArray(SimTypeChar(label="Byte"), 8), "Int8": SimTypeChar(label="SByte"), "UInt8": SimTypeChar(label="Byte"), "Int16": SimTypeShort(signed=True, label="Int16"), "UInt16": SimTypeShort(signed=False, label="UInt16"), "Int32": SimTypeInt(signed=True, label="Int32"), "UInt32": SimTypeInt(signed=False, label="UInt32"), "Int64": SimTypeLongLong(signed=True, label="Int64"), "UInt64": SimTypeLongLong(signed=False, label="UInt64"), "Float32": SimTypeFloat(size=32), "Float64": SimTypeFloat(size=64)}, name="<anon>", label="None"),
    "DML_SCALE_BIAS": SimStruct(OrderedDict((("Scale", SimTypeFloat(size=32)), ("Bias", SimTypeFloat(size=32)),)), name="DML_SCALE_BIAS", pack=False, align=None),
    "DML_SCATTER_ND_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("UpdatesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("InputDimensionCount", SimTypeInt(signed=False, label="UInt32")), ("IndicesDimensionCount", SimTypeInt(signed=False, label="UInt32")),)), name="DML_SCATTER_ND_OPERATOR_DESC", pack=False, align=None),
    "DML_SCATTER_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("IndicesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("UpdatesTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")),)), name="DML_SCATTER_OPERATOR_DESC", pack=False, align=None),
    "DML_SIZE_2D": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")),)), name="DML_SIZE_2D", pack=False, align=None),
    "DML_SLICE1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("InputWindowOffsets", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("InputWindowSizes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("InputWindowStrides", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="DML_SLICE1_OPERATOR_DESC", pack=False, align=None),
    "DML_SLICE_GRAD_OPERATOR_DESC": SimStruct(OrderedDict((("InputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputGradientTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("InputWindowOffsets", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("InputWindowSizes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("InputWindowStrides", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="DML_SLICE_GRAD_OPERATOR_DESC", pack=False, align=None),
    "DML_SLICE_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("DimensionCount", SimTypeInt(signed=False, label="UInt32")), ("Offsets", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Sizes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Strides", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_SLICE_OPERATOR_DESC", pack=False, align=None),
    "DML_SPACE_TO_DEPTH1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BlockSize", SimTypeInt(signed=False, label="UInt32")), ("Order", SimTypeInt(signed=False, label="DML_DEPTH_SPACE_ORDER")),)), name="DML_SPACE_TO_DEPTH1_OPERATOR_DESC", pack=False, align=None),
    "DML_SPACE_TO_DEPTH_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("BlockSize", SimTypeInt(signed=False, label="UInt32")),)), name="DML_SPACE_TO_DEPTH_OPERATOR_DESC", pack=False, align=None),
    "DML_SPLIT_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputCount", SimTypeInt(signed=False, label="UInt32")), ("OutputTensors", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")),)), name="DML_SPLIT_OPERATOR_DESC", pack=False, align=None),
    "DML_TENSOR_DATA_TYPE": SimTypeInt(signed=False, label="DML_TENSOR_DATA_TYPE"),
    "DML_TENSOR_DESC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DML_TENSOR_TYPE")), ("Desc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DML_TENSOR_DESC", pack=False, align=None),
    "DML_TENSOR_FLAGS": SimTypeInt(signed=False, label="DML_TENSOR_FLAGS"),
    "DML_TENSOR_TYPE": SimTypeInt(signed=False, label="DML_TENSOR_TYPE"),
    "DML_TILE_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("RepeatsCount", SimTypeInt(signed=False, label="UInt32")), ("Repeats", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DML_TILE_OPERATOR_DESC", pack=False, align=None),
    "DML_TOP_K1_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputValueTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputIndexTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")), ("K", SimTypeInt(signed=False, label="UInt32")), ("AxisDirection", SimTypeInt(signed=False, label="DML_AXIS_DIRECTION")),)), name="DML_TOP_K1_OPERATOR_DESC", pack=False, align=None),
    "DML_TOP_K_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputValueTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputIndexTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Axis", SimTypeInt(signed=False, label="UInt32")), ("K", SimTypeInt(signed=False, label="UInt32")),)), name="DML_TOP_K_OPERATOR_DESC", pack=False, align=None),
    "DML_UPSAMPLE_2D_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("ScaleSize", SimTypeRef("DML_SIZE_2D", SimStruct)), ("InterpolationMode", SimTypeInt(signed=False, label="DML_INTERPOLATION_MODE")),)), name="DML_UPSAMPLE_2D_OPERATOR_DESC", pack=False, align=None),
    "DML_VALUE_SCALE_2D_OPERATOR_DESC": SimStruct(OrderedDict((("InputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("OutputTensor", SimTypePointer(SimTypeRef("DML_TENSOR_DESC", SimStruct), offset=0)), ("Scale", SimTypeFloat(size=32)), ("ChannelCount", SimTypeInt(signed=False, label="UInt32")), ("Bias", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="DML_VALUE_SCALE_2D_OPERATOR_DESC", pack=False, align=None),
    "DMO_ENUM_FLAGS": SimTypeInt(signed=False, label="DMO_ENUM_FLAGS"),
    "DMO_MEDIA_TYPE": SimStruct(OrderedDict((("majortype", SimTypeBottom(label="Guid")), ("subtype", SimTypeBottom(label="Guid")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("lSampleSize", SimTypeInt(signed=False, label="UInt32")), ("formattype", SimTypeBottom(label="Guid")), ("pUnk", SimTypeBottom(label="IUnknown")), ("cbFormat", SimTypeInt(signed=False, label="UInt32")), ("pbFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DMO_MEDIA_TYPE", pack=False, align=None),
    "DMO_OUTPUT_DATA_BUFFER": SimStruct(OrderedDict((("pBuffer", SimTypeBottom(label="IMediaBuffer")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("rtTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("rtTimelength", SimTypeLongLong(signed=True, label="Int64")),)), name="DMO_OUTPUT_DATA_BUFFER", pack=False, align=None),
    "DMO_PARTIAL_MEDIATYPE": SimStruct(OrderedDict((("type", SimTypeBottom(label="Guid")), ("subtype", SimTypeBottom(label="Guid")),)), name="DMO_PARTIAL_MEDIATYPE", pack=False, align=None),
    "DMO_REGISTER_FLAGS": SimTypeInt(signed=False, label="DMO_REGISTER_FLAGS"),
    "DMUS_ARTICPARAMS": SimStruct(OrderedDict((("LFO", SimTypeRef("DMUS_LFOPARAMS", SimStruct)), ("VolEG", SimTypeRef("DMUS_VEGPARAMS", SimStruct)), ("PitchEG", SimTypeRef("DMUS_PEGPARAMS", SimStruct)), ("Misc", SimTypeRef("DMUS_MSCPARAMS", SimStruct)),)), name="DMUS_ARTICPARAMS", pack=False, align=None),
    "DMUS_ARTICULATION": SimStruct(OrderedDict((("ulArt1Idx", SimTypeInt(signed=False, label="UInt32")), ("ulFirstExtCkIdx", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_ARTICULATION", pack=False, align=None),
    "DMUS_ARTICULATION2": SimStruct(OrderedDict((("ulArtIdx", SimTypeInt(signed=False, label="UInt32")), ("ulFirstExtCkIdx", SimTypeInt(signed=False, label="UInt32")), ("ulNextArtIdx", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_ARTICULATION2", pack=False, align=None),
    "DMUS_BUFFERDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidBufferFormat", SimTypeBottom(label="Guid")), ("cbBuffer", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_BUFFERDESC", pack=False, align=None),
    "DMUS_CLOCKINFO7": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ctType", SimTypeInt(signed=False, label="DMUS_CLOCKTYPE")), ("guidClock", SimTypeBottom(label="Guid")), ("wszDescription", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="DMUS_CLOCKINFO7", pack=False, align=None),
    "DMUS_CLOCKINFO8": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ctType", SimTypeInt(signed=False, label="DMUS_CLOCKTYPE")), ("guidClock", SimTypeBottom(label="Guid")), ("wszDescription", SimTypeArray(SimTypeChar(label="Char"), 128)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_CLOCKINFO8", pack=False, align=None),
    "DMUS_CLOCKTYPE": SimTypeInt(signed=False, label="DMUS_CLOCKTYPE"),
    "DMUS_COPYRIGHT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("byCopyright", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="DMUS_COPYRIGHT", pack=False, align=None),
    "DMUS_DOWNLOADINFO": SimStruct(OrderedDict((("dwDLType", SimTypeInt(signed=False, label="UInt32")), ("dwDLId", SimTypeInt(signed=False, label="UInt32")), ("dwNumOffsetTableEntries", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_DOWNLOADINFO", pack=False, align=None),
    "DMUS_EVENTHEADER": SimStruct(OrderedDict((("cbEvent", SimTypeInt(signed=False, label="UInt32")), ("dwChannelGroup", SimTypeInt(signed=False, label="UInt32")), ("rtDelta", SimTypeLongLong(signed=True, label="Int64")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_EVENTHEADER", pack=False, align=None),
    "DMUS_EXTENSIONCHUNK": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulNextExtCkIdx", SimTypeInt(signed=False, label="UInt32")), ("ExtCkID", SimTypeInt(signed=False, label="UInt32")), ("byExtCk", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="DMUS_EXTENSIONCHUNK", pack=False, align=None),
    "DMUS_INSTRUMENT": SimStruct(OrderedDict((("ulPatch", SimTypeInt(signed=False, label="UInt32")), ("ulFirstRegionIdx", SimTypeInt(signed=False, label="UInt32")), ("ulGlobalArtIdx", SimTypeInt(signed=False, label="UInt32")), ("ulFirstExtCkIdx", SimTypeInt(signed=False, label="UInt32")), ("ulCopyrightIdx", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_INSTRUMENT", pack=False, align=None),
    "DMUS_LFOPARAMS": SimStruct(OrderedDict((("pcFrequency", SimTypeInt(signed=True, label="Int32")), ("tcDelay", SimTypeInt(signed=True, label="Int32")), ("gcVolumeScale", SimTypeInt(signed=True, label="Int32")), ("pcPitchScale", SimTypeInt(signed=True, label="Int32")), ("gcMWToVolume", SimTypeInt(signed=True, label="Int32")), ("pcMWToPitch", SimTypeInt(signed=True, label="Int32")),)), name="DMUS_LFOPARAMS", pack=False, align=None),
    "DMUS_MSCPARAMS": SimStruct(OrderedDict((("ptDefaultPan", SimTypeInt(signed=True, label="Int32")),)), name="DMUS_MSCPARAMS", pack=False, align=None),
    "DMUS_NOTERANGE": SimStruct(OrderedDict((("dwLowNote", SimTypeInt(signed=False, label="UInt32")), ("dwHighNote", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_NOTERANGE", pack=False, align=None),
    "DMUS_OFFSETTABLE": SimStruct(OrderedDict((("ulOffsetTable", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DMUS_OFFSETTABLE", pack=False, align=None),
    "DMUS_PEGPARAMS": SimStruct(OrderedDict((("tcAttack", SimTypeInt(signed=True, label="Int32")), ("tcDecay", SimTypeInt(signed=True, label="Int32")), ("ptSustain", SimTypeInt(signed=True, label="Int32")), ("tcRelease", SimTypeInt(signed=True, label="Int32")), ("tcVel2Attack", SimTypeInt(signed=True, label="Int32")), ("tcKey2Decay", SimTypeInt(signed=True, label="Int32")), ("pcRange", SimTypeInt(signed=True, label="Int32")),)), name="DMUS_PEGPARAMS", pack=False, align=None),
    "DMUS_PORTCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidPort", SimTypeBottom(label="Guid")), ("dwClass", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwMemorySize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxChannelGroups", SimTypeInt(signed=False, label="UInt32")), ("dwMaxVoices", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAudioChannels", SimTypeInt(signed=False, label="UInt32")), ("dwEffectFlags", SimTypeInt(signed=False, label="UInt32")), ("wszDescription", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="DMUS_PORTCAPS", pack=False, align=None),
    "DMUS_PORTPARAMS7": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwValidParams", SimTypeInt(signed=False, label="UInt32")), ("dwVoices", SimTypeInt(signed=False, label="UInt32")), ("dwChannelGroups", SimTypeInt(signed=False, label="UInt32")), ("dwAudioChannels", SimTypeInt(signed=False, label="UInt32")), ("dwSampleRate", SimTypeInt(signed=False, label="UInt32")), ("dwEffectFlags", SimTypeInt(signed=False, label="UInt32")), ("fShare", SimTypeInt(signed=True, label="Int32")),)), name="DMUS_PORTPARAMS7", pack=False, align=None),
    "DMUS_PORTPARAMS8": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwValidParams", SimTypeInt(signed=False, label="UInt32")), ("dwVoices", SimTypeInt(signed=False, label="UInt32")), ("dwChannelGroups", SimTypeInt(signed=False, label="UInt32")), ("dwAudioChannels", SimTypeInt(signed=False, label="UInt32")), ("dwSampleRate", SimTypeInt(signed=False, label="UInt32")), ("dwEffectFlags", SimTypeInt(signed=False, label="UInt32")), ("fShare", SimTypeInt(signed=True, label="Int32")), ("dwFeatures", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_PORTPARAMS8", pack=False, align=None),
    "DMUS_REGION": SimStruct(OrderedDict((("RangeKey", SimTypeRef("RGNRANGE", SimStruct)), ("RangeVelocity", SimTypeRef("RGNRANGE", SimStruct)), ("fusOptions", SimTypeShort(signed=False, label="UInt16")), ("usKeyGroup", SimTypeShort(signed=False, label="UInt16")), ("ulRegionArtIdx", SimTypeInt(signed=False, label="UInt32")), ("ulNextRegionIdx", SimTypeInt(signed=False, label="UInt32")), ("ulFirstExtCkIdx", SimTypeInt(signed=False, label="UInt32")), ("WaveLink", SimTypeRef("WAVELINK", SimStruct)), ("WSMP", SimTypeRef("WSMPL", SimStruct)), ("WLOOP", SimTypePointer(SimTypeRef("WLOOP", SimStruct), offset=0)),)), name="DMUS_REGION", pack=False, align=None),
    "DMUS_SYNTHSTATS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwValidStats", SimTypeInt(signed=False, label="UInt32")), ("dwVoices", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCPU", SimTypeInt(signed=False, label="UInt32")), ("dwCPUPerVoice", SimTypeInt(signed=False, label="UInt32")), ("dwLostNotes", SimTypeInt(signed=False, label="UInt32")), ("dwFreeMemory", SimTypeInt(signed=False, label="UInt32")), ("lPeakVolume", SimTypeInt(signed=True, label="Int32")),)), name="DMUS_SYNTHSTATS", pack=False, align=None),
    "DMUS_SYNTHSTATS8": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwValidStats", SimTypeInt(signed=False, label="UInt32")), ("dwVoices", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCPU", SimTypeInt(signed=False, label="UInt32")), ("dwCPUPerVoice", SimTypeInt(signed=False, label="UInt32")), ("dwLostNotes", SimTypeInt(signed=False, label="UInt32")), ("dwFreeMemory", SimTypeInt(signed=False, label="UInt32")), ("lPeakVolume", SimTypeInt(signed=True, label="Int32")), ("dwSynthMemUse", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_SYNTHSTATS8", pack=False, align=None),
    "DMUS_VEGPARAMS": SimStruct(OrderedDict((("tcAttack", SimTypeInt(signed=True, label="Int32")), ("tcDecay", SimTypeInt(signed=True, label="Int32")), ("ptSustain", SimTypeInt(signed=True, label="Int32")), ("tcRelease", SimTypeInt(signed=True, label="Int32")), ("tcVel2Attack", SimTypeInt(signed=True, label="Int32")), ("tcKey2Decay", SimTypeInt(signed=True, label="Int32")),)), name="DMUS_VEGPARAMS", pack=False, align=None),
    "DMUS_VOICE_STATE": SimStruct(OrderedDict((("bExists", SimTypeInt(signed=True, label="Int32")), ("spPosition", SimTypeLongLong(signed=False, label="UInt64")),)), name="DMUS_VOICE_STATE", pack=False, align=None),
    "DMUS_WAVE": SimStruct(OrderedDict((("ulFirstExtCkIdx", SimTypeInt(signed=False, label="UInt32")), ("ulCopyrightIdx", SimTypeInt(signed=False, label="UInt32")), ("ulWaveDataIdx", SimTypeInt(signed=False, label="UInt32")), ("WaveformatEx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="DMUS_WAVE", pack=False, align=None),
    "DMUS_WAVEARTDL": SimStruct(OrderedDict((("ulDownloadIdIdx", SimTypeInt(signed=False, label="UInt32")), ("ulBus", SimTypeInt(signed=False, label="UInt32")), ("ulBuffers", SimTypeInt(signed=False, label="UInt32")), ("ulMasterDLId", SimTypeInt(signed=False, label="UInt32")), ("usOptions", SimTypeShort(signed=False, label="UInt16")),)), name="DMUS_WAVEARTDL", pack=False, align=None),
    "DMUS_WAVEDATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("byData", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="DMUS_WAVEDATA", pack=False, align=None),
    "DMUS_WAVEDL": SimStruct(OrderedDict((("cbWaveData", SimTypeInt(signed=False, label="UInt32")),)), name="DMUS_WAVEDL", pack=False, align=None),
    "DMUS_WAVES_REVERB_PARAMS": SimStruct(OrderedDict((("fInGain", SimTypeFloat(size=32)), ("fReverbMix", SimTypeFloat(size=32)), ("fReverbTime", SimTypeFloat(size=32)), ("fHighFreqRTRatio", SimTypeFloat(size=32)),)), name="DMUS_WAVES_REVERB_PARAMS", pack=False, align=None),
    "DNS_AAAA_DATA": SimStruct(OrderedDict((("Ip6Address", SimUnion({"IP6Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4), "IP6Word": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "IP6Byte": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")),)), name="DNS_AAAA_DATA", pack=False, align=None),
    "DNS_ADDR": SimStruct(OrderedDict((("MaxSa", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("Data", SimUnion({"DnsAddrUserDword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)}, name="<anon>", label="None")),)), name="DNS_ADDR", pack=False, align=None),
    "DNS_ADDR_ARRAY": SimStruct(OrderedDict((("MaxCount", SimTypeInt(signed=False, label="UInt32")), ("AddrCount", SimTypeInt(signed=False, label="UInt32")), ("Tag", SimTypeInt(signed=False, label="UInt32")), ("Family", SimTypeShort(signed=False, label="UInt16")), ("WordReserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MatchFlag", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("AddrArray", SimTypePointer(SimTypeRef("DNS_ADDR", SimStruct), offset=0)),)), name="DNS_ADDR_ARRAY", pack=False, align=None),
    "DNS_APPLICATION_SETTINGS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="DNS_APPLICATION_SETTINGS", pack=False, align=None),
    "DNS_ATMA_DATA": SimStruct(OrderedDict((("AddressType", SimTypeChar(label="Byte")), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="DNS_ATMA_DATA", pack=False, align=None),
    "DNS_A_DATA": SimStruct(OrderedDict((("IpAddress", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_A_DATA", pack=False, align=None),
    "DNS_CHARSET": SimTypeInt(signed=False, label="DNS_CHARSET"),
    "DNS_CONFIG_TYPE": SimTypeInt(signed=False, label="DNS_CONFIG_TYPE"),
    "DNS_CONNECTION_IFINDEX_ENTRY": SimStruct(OrderedDict((("pwszConnectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_CONNECTION_IFINDEX_ENTRY", pack=False, align=None),
    "DNS_CONNECTION_IFINDEX_LIST": SimStruct(OrderedDict((("pConnectionIfIndexEntries", SimTypePointer(SimTypeRef("DNS_CONNECTION_IFINDEX_ENTRY", SimStruct), offset=0)), ("nEntries", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_CONNECTION_IFINDEX_LIST", pack=False, align=None),
    "DNS_CONNECTION_NAME": SimStruct(OrderedDict((("wszName", SimTypeArray(SimTypeChar(label="Char"), 65)),)), name="DNS_CONNECTION_NAME", pack=False, align=None),
    "DNS_CONNECTION_NAME_LIST": SimStruct(OrderedDict((("cNames", SimTypeInt(signed=False, label="UInt32")), ("pNames", SimTypePointer(SimTypeRef("DNS_CONNECTION_NAME", SimStruct), offset=0)),)), name="DNS_CONNECTION_NAME_LIST", pack=False, align=None),
    "DNS_CONNECTION_POLICY_ENTRY": SimStruct(OrderedDict((("pwszHost", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszAppId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbAppSid", SimTypeInt(signed=False, label="UInt32")), ("pbAppSid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nConnections", SimTypeInt(signed=False, label="UInt32")), ("ppwszConnections", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwPolicyEntryFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_CONNECTION_POLICY_ENTRY", pack=False, align=None),
    "DNS_CONNECTION_POLICY_ENTRY_LIST": SimStruct(OrderedDict((("pPolicyEntries", SimTypePointer(SimTypeRef("DNS_CONNECTION_POLICY_ENTRY", SimStruct), offset=0)), ("nEntries", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_CONNECTION_POLICY_ENTRY_LIST", pack=False, align=None),
    "DNS_CONNECTION_POLICY_TAG": SimTypeInt(signed=False, label="DNS_CONNECTION_POLICY_TAG"),
    "DNS_CONNECTION_PROXY_ELEMENT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DNS_CONNECTION_PROXY_TYPE")), ("Info", SimTypeRef("DNS_CONNECTION_PROXY_INFO", SimStruct)),)), name="DNS_CONNECTION_PROXY_ELEMENT", pack=False, align=None),
    "DNS_CONNECTION_PROXY_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Switch", SimTypeInt(signed=False, label="DNS_CONNECTION_PROXY_INFO_SWITCH")), ("Anonymous", SimUnion({"Config": SimTypeRef("_DNS_CONNECTION_PROXY_INFO_CONFIG", SimStruct), "Script": SimTypeRef("_DNS_CONNECTION_PROXY_INFO_SCRIPT", SimStruct)}, name="<anon>", label="None")),)), name="DNS_CONNECTION_PROXY_INFO", pack=False, align=None),
    "DNS_CONNECTION_PROXY_INFO_EX": SimStruct(OrderedDict((("ProxyInfo", SimTypeRef("DNS_CONNECTION_PROXY_INFO", SimStruct)), ("dwInterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pwszConnectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fDirectConfiguration", SimTypeInt(signed=True, label="Int32")), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DNS_CONNECTION_PROXY_INFO_EX", pack=False, align=None),
    "DNS_CONNECTION_PROXY_INFO_SWITCH": SimTypeInt(signed=False, label="DNS_CONNECTION_PROXY_INFO_SWITCH"),
    "DNS_CONNECTION_PROXY_LIST": SimStruct(OrderedDict((("cProxies", SimTypeInt(signed=False, label="UInt32")), ("pProxies", SimTypePointer(SimTypeRef("DNS_CONNECTION_PROXY_ELEMENT", SimStruct), offset=0)),)), name="DNS_CONNECTION_PROXY_LIST", pack=False, align=None),
    "DNS_CONNECTION_PROXY_TYPE": SimTypeInt(signed=False, label="DNS_CONNECTION_PROXY_TYPE"),
    "DNS_CUSTOM_SERVER": SimStruct(OrderedDict((("dwServerType", SimTypeInt(signed=False, label="UInt32")), ("ullFlags", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous1", SimUnion({"pwszTemplate": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"MaxSa": SimTypeArray(SimTypeChar(label="SByte"), 32)}, name="<anon>", label="None")),)), name="DNS_CUSTOM_SERVER", pack=False, align=None),
    "DNS_DHCID_DATA": SimStruct(OrderedDict((("dwByteCount", SimTypeInt(signed=False, label="UInt32")), ("DHCID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_DHCID_DATA", pack=False, align=None),
    "DNS_DOH_SERVER_SETTINGS": SimStruct(OrderedDict((("Template", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="DNS_DOH_SERVER_SETTINGS", pack=False, align=None),
    "DNS_DS_DATA": SimStruct(OrderedDict((("wKeyTag", SimTypeShort(signed=False, label="UInt16")), ("chAlgorithm", SimTypeChar(label="Byte")), ("chDigestType", SimTypeChar(label="Byte")), ("wDigestLength", SimTypeShort(signed=False, label="UInt16")), ("wPad", SimTypeShort(signed=False, label="UInt16")), ("Digest", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_DS_DATA", pack=False, align=None),
    "DNS_FREE_TYPE": SimTypeInt(signed=False, label="DNS_FREE_TYPE"),
    "DNS_HEADER": SimStruct(OrderedDict((("Xid", SimTypeShort(signed=False, label="UInt16")), ("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("QuestionCount", SimTypeShort(signed=False, label="UInt16")), ("AnswerCount", SimTypeShort(signed=False, label="UInt16")), ("NameServerCount", SimTypeShort(signed=False, label="UInt16")), ("AdditionalCount", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_HEADER", pack=False, align=None),
    "DNS_HEADER_EXT": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("chRcode", SimTypeChar(label="Byte")), ("chVersion", SimTypeChar(label="Byte")),)), name="DNS_HEADER_EXT", pack=False, align=None),
    "DNS_INTERFACE_SETTINGS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NameServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RegistrationEnabled", SimTypeInt(signed=False, label="UInt32")), ("RegisterAdapterName", SimTypeInt(signed=False, label="UInt32")), ("EnableLLMNR", SimTypeInt(signed=False, label="UInt32")), ("QueryAdapterName", SimTypeInt(signed=False, label="UInt32")), ("ProfileNameServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_INTERFACE_SETTINGS", pack=False, align=None),
    "DNS_INTERFACE_SETTINGS3": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NameServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RegistrationEnabled", SimTypeInt(signed=False, label="UInt32")), ("RegisterAdapterName", SimTypeInt(signed=False, label="UInt32")), ("EnableLLMNR", SimTypeInt(signed=False, label="UInt32")), ("QueryAdapterName", SimTypeInt(signed=False, label="UInt32")), ("ProfileNameServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DisableUnconstrainedQueries", SimTypeInt(signed=False, label="UInt32")), ("SupplementalSearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cServerProperties", SimTypeInt(signed=False, label="UInt32")), ("ServerProperties", SimTypePointer(SimTypeRef("DNS_SERVER_PROPERTY", SimStruct), offset=0)), ("cProfileServerProperties", SimTypeInt(signed=False, label="UInt32")), ("ProfileServerProperties", SimTypePointer(SimTypeRef("DNS_SERVER_PROPERTY", SimStruct), offset=0)),)), name="DNS_INTERFACE_SETTINGS3", pack=False, align=None),
    "DNS_INTERFACE_SETTINGS4": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NameServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RegistrationEnabled", SimTypeInt(signed=False, label="UInt32")), ("RegisterAdapterName", SimTypeInt(signed=False, label="UInt32")), ("EnableLLMNR", SimTypeInt(signed=False, label="UInt32")), ("QueryAdapterName", SimTypeInt(signed=False, label="UInt32")), ("ProfileNameServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DisableUnconstrainedQueries", SimTypeInt(signed=False, label="UInt32")), ("SupplementalSearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cServerProperties", SimTypeInt(signed=False, label="UInt32")), ("ServerProperties", SimTypePointer(SimTypeRef("DNS_SERVER_PROPERTY", SimStruct), offset=0)), ("cProfileServerProperties", SimTypeInt(signed=False, label="UInt32")), ("ProfileServerProperties", SimTypePointer(SimTypeRef("DNS_SERVER_PROPERTY", SimStruct), offset=0)), ("EncryptedDnsAdapterFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_INTERFACE_SETTINGS4", pack=False, align=None),
    "DNS_INTERFACE_SETTINGS_EX": SimStruct(OrderedDict((("SettingsV1", SimTypeRef("DNS_INTERFACE_SETTINGS", SimStruct)), ("DisableUnconstrainedQueries", SimTypeInt(signed=False, label="UInt32")), ("SupplementalSearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_INTERFACE_SETTINGS_EX", pack=False, align=None),
    "DNS_KEY_DATA": SimStruct(OrderedDict((("wFlags", SimTypeShort(signed=False, label="UInt16")), ("chProtocol", SimTypeChar(label="Byte")), ("chAlgorithm", SimTypeChar(label="Byte")), ("wKeyLength", SimTypeShort(signed=False, label="UInt16")), ("wPad", SimTypeShort(signed=False, label="UInt16")), ("Key", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_KEY_DATA", pack=False, align=None),
    "DNS_LOC_DATA": SimStruct(OrderedDict((("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wSize", SimTypeShort(signed=False, label="UInt16")), ("wHorPrec", SimTypeShort(signed=False, label="UInt16")), ("wVerPrec", SimTypeShort(signed=False, label="UInt16")), ("dwLatitude", SimTypeInt(signed=False, label="UInt32")), ("dwLongitude", SimTypeInt(signed=False, label="UInt32")), ("dwAltitude", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_LOC_DATA", pack=False, align=None),
    "DNS_MESSAGE_BUFFER": SimStruct(OrderedDict((("MessageHead", SimTypeRef("DNS_HEADER", SimStruct)), ("MessageBody", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="DNS_MESSAGE_BUFFER", pack=False, align=None),
    "DNS_MINFO_DATAA": SimStruct(OrderedDict((("pNameMailbox", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pNameErrorsMailbox", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_MINFO_DATAA", pack=False, align=None),
    "DNS_MINFO_DATAW": SimStruct(OrderedDict((("pNameMailbox", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pNameErrorsMailbox", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_MINFO_DATAW", pack=False, align=None),
    "DNS_MX_DATAA": SimStruct(OrderedDict((("pNameExchange", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wPreference", SimTypeShort(signed=False, label="UInt16")), ("Pad", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_MX_DATAA", pack=False, align=None),
    "DNS_MX_DATAW": SimStruct(OrderedDict((("pNameExchange", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wPreference", SimTypeShort(signed=False, label="UInt16")), ("Pad", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_MX_DATAW", pack=False, align=None),
    "DNS_NAME_FORMAT": SimTypeInt(signed=False, label="DNS_NAME_FORMAT"),
    "DNS_NAPTR_DATAA": SimStruct(OrderedDict((("wOrder", SimTypeShort(signed=False, label="UInt16")), ("wPreference", SimTypeShort(signed=False, label="UInt16")), ("pFlags", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pService", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pRegularExpression", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pReplacement", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_NAPTR_DATAA", pack=False, align=None),
    "DNS_NAPTR_DATAW": SimStruct(OrderedDict((("wOrder", SimTypeShort(signed=False, label="UInt16")), ("wPreference", SimTypeShort(signed=False, label="UInt16")), ("pFlags", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pService", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pRegularExpression", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pReplacement", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_NAPTR_DATAW", pack=False, align=None),
    "DNS_NSEC3PARAM_DATA": SimStruct(OrderedDict((("chAlgorithm", SimTypeChar(label="Byte")), ("bFlags", SimTypeChar(label="Byte")), ("wIterations", SimTypeShort(signed=False, label="UInt16")), ("bSaltLength", SimTypeChar(label="Byte")), ("bPad", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("pbSalt", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_NSEC3PARAM_DATA", pack=False, align=None),
    "DNS_NSEC3_DATA": SimStruct(OrderedDict((("chAlgorithm", SimTypeChar(label="Byte")), ("bFlags", SimTypeChar(label="Byte")), ("wIterations", SimTypeShort(signed=False, label="UInt16")), ("bSaltLength", SimTypeChar(label="Byte")), ("bHashLength", SimTypeChar(label="Byte")), ("wTypeBitMapsLength", SimTypeShort(signed=False, label="UInt16")), ("chData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_NSEC3_DATA", pack=False, align=None),
    "DNS_NSEC_DATAA": SimStruct(OrderedDict((("pNextDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wTypeBitMapsLength", SimTypeShort(signed=False, label="UInt16")), ("wPad", SimTypeShort(signed=False, label="UInt16")), ("TypeBitMaps", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_NSEC_DATAA", pack=False, align=None),
    "DNS_NSEC_DATAW": SimStruct(OrderedDict((("pNextDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wTypeBitMapsLength", SimTypeShort(signed=False, label="UInt16")), ("wPad", SimTypeShort(signed=False, label="UInt16")), ("TypeBitMaps", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_NSEC_DATAW", pack=False, align=None),
    "DNS_NULL_DATA": SimStruct(OrderedDict((("dwByteCount", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_NULL_DATA", pack=False, align=None),
    "DNS_NXT_DATAA": SimStruct(OrderedDict((("pNameNext", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wNumTypes", SimTypeShort(signed=False, label="UInt16")), ("wTypes", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="DNS_NXT_DATAA", pack=False, align=None),
    "DNS_NXT_DATAW": SimStruct(OrderedDict((("pNameNext", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wNumTypes", SimTypeShort(signed=False, label="UInt16")), ("wTypes", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="DNS_NXT_DATAW", pack=False, align=None),
    "DNS_OPT_DATA": SimStruct(OrderedDict((("wDataLength", SimTypeShort(signed=False, label="UInt16")), ("wPad", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_OPT_DATA", pack=False, align=None),
    "DNS_PROXY_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["completionContext", "status"]), offset=0),
    "DNS_PROXY_INFORMATION": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("proxyInformationType", SimTypeInt(signed=False, label="DNS_PROXY_INFORMATION_TYPE")), ("proxyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_PROXY_INFORMATION", pack=False, align=None),
    "DNS_PROXY_INFORMATION_TYPE": SimTypeInt(signed=False, label="DNS_PROXY_INFORMATION_TYPE"),
    "DNS_PTR_DATAA": SimStruct(OrderedDict((("pNameHost", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_PTR_DATAA", pack=False, align=None),
    "DNS_PTR_DATAW": SimStruct(OrderedDict((("pNameHost", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_PTR_DATAW", pack=False, align=None),
    "DNS_QUERY_CANCEL": SimStruct(OrderedDict((("Reserved", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="DNS_QUERY_CANCEL", pack=False, align=None),
    "DNS_QUERY_OPTIONS": SimTypeInt(signed=False, label="DNS_QUERY_OPTIONS"),
    "DNS_QUERY_RAW_CANCEL": SimStruct(OrderedDict((("reserved", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="DNS_QUERY_RAW_CANCEL", pack=False, align=None),
    "DNS_QUERY_RAW_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RAW_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["queryContext", "queryResults"]), offset=0),
    "DNS_QUERY_RAW_REQUEST": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("resultsVersion", SimTypeInt(signed=False, label="UInt32")), ("dnsQueryRawSize", SimTypeInt(signed=False, label="UInt32")), ("dnsQueryRaw", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dnsQueryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dnsQueryType", SimTypeShort(signed=False, label="UInt16")), ("queryOptions", SimTypeLongLong(signed=False, label="UInt64")), ("interfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("queryCompletionCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RAW_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["queryContext", "queryResults"]), offset=0)), ("queryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("queryRawOptions", SimTypeLongLong(signed=False, label="UInt64")), ("customServersSize", SimTypeInt(signed=False, label="UInt32")), ("customServers", SimTypePointer(SimTypeRef("DNS_CUSTOM_SERVER", SimStruct), offset=0)), ("protocol", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"maxSa": SimTypeArray(SimTypeChar(label="SByte"), 32)}, name="<anon>", label="None")),)), name="DNS_QUERY_RAW_REQUEST", pack=False, align=None),
    "DNS_QUERY_RAW_RESULT": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("queryStatus", SimTypeInt(signed=True, label="Int32")), ("queryOptions", SimTypeLongLong(signed=False, label="UInt64")), ("queryRawOptions", SimTypeLongLong(signed=False, label="UInt64")), ("responseFlags", SimTypeLongLong(signed=False, label="UInt64")), ("queryRawResponseSize", SimTypeInt(signed=False, label="UInt32")), ("queryRawResponse", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("queryRecords", SimTypePointer(SimTypeRef("DNS_RECORDA", SimStruct), offset=0)), ("protocol", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"maxSa": SimTypeArray(SimTypeChar(label="SByte"), 32)}, name="<anon>", label="None")),)), name="DNS_QUERY_RAW_RESULT", pack=False, align=None),
    "DNS_QUERY_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("QueryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("QueryType", SimTypeShort(signed=False, label="UInt16")), ("QueryOptions", SimTypeLongLong(signed=False, label="UInt64")), ("pDnsServerList", SimTypePointer(SimTypeRef("DNS_ADDR_ARRAY", SimStruct), offset=0)), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pQueryCompletionCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pQueryContext", "pQueryResults"]), offset=0)), ("pQueryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DNS_QUERY_REQUEST", pack=False, align=None),
    "DNS_QUERY_REQUEST3": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("QueryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("QueryType", SimTypeShort(signed=False, label="UInt16")), ("QueryOptions", SimTypeLongLong(signed=False, label="UInt64")), ("pDnsServerList", SimTypePointer(SimTypeRef("DNS_ADDR_ARRAY", SimStruct), offset=0)), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pQueryCompletionCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pQueryContext", "pQueryResults"]), offset=0)), ("pQueryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("IsNetworkQueryRequired", SimTypeInt(signed=True, label="Int32")), ("RequiredNetworkIndex", SimTypeInt(signed=False, label="UInt32")), ("cCustomServers", SimTypeInt(signed=False, label="UInt32")), ("pCustomServers", SimTypePointer(SimTypeRef("DNS_CUSTOM_SERVER", SimStruct), offset=0)),)), name="DNS_QUERY_REQUEST3", pack=False, align=None),
    "DNS_QUERY_RESULT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("QueryStatus", SimTypeInt(signed=True, label="Int32")), ("QueryOptions", SimTypeLongLong(signed=False, label="UInt64")), ("pQueryRecords", SimTypePointer(SimTypeRef("DNS_RECORDA", SimStruct), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DNS_QUERY_RESULT", pack=False, align=None),
    "DNS_RECORDA": SimStruct(OrderedDict((("pNext", SimTypePointer(SimTypeRef("DNS_RECORDA", SimStruct), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("wDataLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"DW": SimTypeInt(signed=False, label="UInt32"), "S": SimTypeRef("DNS_RECORD_FLAGS", SimStruct)}, name="<anon>", label="None")), ("dwTtl", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("Data", SimUnion({"A": SimTypeRef("DNS_A_DATA", SimStruct), "SOA": SimTypeRef("DNS_SOA_DATAA", SimStruct), "Soa": SimTypeRef("DNS_SOA_DATAA", SimStruct), "PTR": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Ptr": SimTypeRef("DNS_PTR_DATAA", SimStruct), "NS": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Ns": SimTypeRef("DNS_PTR_DATAA", SimStruct), "CNAME": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Cname": SimTypeRef("DNS_PTR_DATAA", SimStruct), "DNAME": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Dname": SimTypeRef("DNS_PTR_DATAA", SimStruct), "MB": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Mb": SimTypeRef("DNS_PTR_DATAA", SimStruct), "MD": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Md": SimTypeRef("DNS_PTR_DATAA", SimStruct), "MF": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Mf": SimTypeRef("DNS_PTR_DATAA", SimStruct), "MG": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Mg": SimTypeRef("DNS_PTR_DATAA", SimStruct), "MR": SimTypeRef("DNS_PTR_DATAA", SimStruct), "Mr": SimTypeRef("DNS_PTR_DATAA", SimStruct), "MINFO": SimTypeRef("DNS_MINFO_DATAA", SimStruct), "Minfo": SimTypeRef("DNS_MINFO_DATAA", SimStruct), "RP": SimTypeRef("DNS_MINFO_DATAA", SimStruct), "Rp": SimTypeRef("DNS_MINFO_DATAA", SimStruct), "MX": SimTypeRef("DNS_MX_DATAA", SimStruct), "Mx": SimTypeRef("DNS_MX_DATAA", SimStruct), "AFSDB": SimTypeRef("DNS_MX_DATAA", SimStruct), "Afsdb": SimTypeRef("DNS_MX_DATAA", SimStruct), "RT": SimTypeRef("DNS_MX_DATAA", SimStruct), "Rt": SimTypeRef("DNS_MX_DATAA", SimStruct), "HINFO": SimTypeRef("DNS_TXT_DATAA", SimStruct), "Hinfo": SimTypeRef("DNS_TXT_DATAA", SimStruct), "ISDN": SimTypeRef("DNS_TXT_DATAA", SimStruct), "Isdn": SimTypeRef("DNS_TXT_DATAA", SimStruct), "TXT": SimTypeRef("DNS_TXT_DATAA", SimStruct), "Txt": SimTypeRef("DNS_TXT_DATAA", SimStruct), "X25": SimTypeRef("DNS_TXT_DATAA", SimStruct), "Null": SimTypeRef("DNS_NULL_DATA", SimStruct), "WKS": SimTypeRef("DNS_WKS_DATA", SimStruct), "Wks": SimTypeRef("DNS_WKS_DATA", SimStruct), "AAAA": SimTypeRef("DNS_AAAA_DATA", SimStruct), "KEY": SimTypeRef("DNS_KEY_DATA", SimStruct), "Key": SimTypeRef("DNS_KEY_DATA", SimStruct), "SIG": SimTypeRef("DNS_SIG_DATAA", SimStruct), "Sig": SimTypeRef("DNS_SIG_DATAA", SimStruct), "ATMA": SimTypeRef("DNS_ATMA_DATA", SimStruct), "Atma": SimTypeRef("DNS_ATMA_DATA", SimStruct), "NXT": SimTypeRef("DNS_NXT_DATAA", SimStruct), "Nxt": SimTypeRef("DNS_NXT_DATAA", SimStruct), "SRV": SimTypeRef("DNS_SRV_DATAA", SimStruct), "Srv": SimTypeRef("DNS_SRV_DATAA", SimStruct), "NAPTR": SimTypeRef("DNS_NAPTR_DATAA", SimStruct), "Naptr": SimTypeRef("DNS_NAPTR_DATAA", SimStruct), "OPT": SimTypeRef("DNS_OPT_DATA", SimStruct), "Opt": SimTypeRef("DNS_OPT_DATA", SimStruct), "DS": SimTypeRef("DNS_DS_DATA", SimStruct), "Ds": SimTypeRef("DNS_DS_DATA", SimStruct), "RRSIG": SimTypeRef("DNS_SIG_DATAA", SimStruct), "Rrsig": SimTypeRef("DNS_SIG_DATAA", SimStruct), "NSEC": SimTypeRef("DNS_NSEC_DATAA", SimStruct), "Nsec": SimTypeRef("DNS_NSEC_DATAA", SimStruct), "DNSKEY": SimTypeRef("DNS_KEY_DATA", SimStruct), "Dnskey": SimTypeRef("DNS_KEY_DATA", SimStruct), "TKEY": SimTypeRef("DNS_TKEY_DATAA", SimStruct), "Tkey": SimTypeRef("DNS_TKEY_DATAA", SimStruct), "TSIG": SimTypeRef("DNS_TSIG_DATAA", SimStruct), "Tsig": SimTypeRef("DNS_TSIG_DATAA", SimStruct), "WINS": SimTypeRef("DNS_WINS_DATA", SimStruct), "Wins": SimTypeRef("DNS_WINS_DATA", SimStruct), "WINSR": SimTypeRef("DNS_WINSR_DATAA", SimStruct), "WinsR": SimTypeRef("DNS_WINSR_DATAA", SimStruct), "NBSTAT": SimTypeRef("DNS_WINSR_DATAA", SimStruct), "Nbstat": SimTypeRef("DNS_WINSR_DATAA", SimStruct), "DHCID": SimTypeRef("DNS_DHCID_DATA", SimStruct), "NSEC3": SimTypeRef("DNS_NSEC3_DATA", SimStruct), "Nsec3": SimTypeRef("DNS_NSEC3_DATA", SimStruct), "NSEC3PARAM": SimTypeRef("DNS_NSEC3PARAM_DATA", SimStruct), "Nsec3Param": SimTypeRef("DNS_NSEC3PARAM_DATA", SimStruct), "TLSA": SimTypeRef("DNS_TLSA_DATA", SimStruct), "Tlsa": SimTypeRef("DNS_TLSA_DATA", SimStruct), "SVCB": SimTypeRef("DNS_SVCB_DATA", SimStruct), "Svcb": SimTypeRef("DNS_SVCB_DATA", SimStruct), "UNKNOWN": SimTypeRef("DNS_UNKNOWN_DATA", SimStruct), "Unknown": SimTypeRef("DNS_UNKNOWN_DATA", SimStruct), "pDataPtr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="DNS_RECORDA", pack=False, align=None),
    "DNS_RECORDW": SimStruct(OrderedDict((("pNext", SimTypePointer(SimTypeRef("DNS_RECORDW", SimStruct), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("wDataLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"DW": SimTypeInt(signed=False, label="UInt32"), "S": SimTypeRef("DNS_RECORD_FLAGS", SimStruct)}, name="<anon>", label="None")), ("dwTtl", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("Data", SimUnion({"A": SimTypeRef("DNS_A_DATA", SimStruct), "SOA": SimTypeRef("DNS_SOA_DATAW", SimStruct), "Soa": SimTypeRef("DNS_SOA_DATAW", SimStruct), "PTR": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Ptr": SimTypeRef("DNS_PTR_DATAW", SimStruct), "NS": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Ns": SimTypeRef("DNS_PTR_DATAW", SimStruct), "CNAME": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Cname": SimTypeRef("DNS_PTR_DATAW", SimStruct), "DNAME": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Dname": SimTypeRef("DNS_PTR_DATAW", SimStruct), "MB": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Mb": SimTypeRef("DNS_PTR_DATAW", SimStruct), "MD": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Md": SimTypeRef("DNS_PTR_DATAW", SimStruct), "MF": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Mf": SimTypeRef("DNS_PTR_DATAW", SimStruct), "MG": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Mg": SimTypeRef("DNS_PTR_DATAW", SimStruct), "MR": SimTypeRef("DNS_PTR_DATAW", SimStruct), "Mr": SimTypeRef("DNS_PTR_DATAW", SimStruct), "MINFO": SimTypeRef("DNS_MINFO_DATAW", SimStruct), "Minfo": SimTypeRef("DNS_MINFO_DATAW", SimStruct), "RP": SimTypeRef("DNS_MINFO_DATAW", SimStruct), "Rp": SimTypeRef("DNS_MINFO_DATAW", SimStruct), "MX": SimTypeRef("DNS_MX_DATAW", SimStruct), "Mx": SimTypeRef("DNS_MX_DATAW", SimStruct), "AFSDB": SimTypeRef("DNS_MX_DATAW", SimStruct), "Afsdb": SimTypeRef("DNS_MX_DATAW", SimStruct), "RT": SimTypeRef("DNS_MX_DATAW", SimStruct), "Rt": SimTypeRef("DNS_MX_DATAW", SimStruct), "HINFO": SimTypeRef("DNS_TXT_DATAW", SimStruct), "Hinfo": SimTypeRef("DNS_TXT_DATAW", SimStruct), "ISDN": SimTypeRef("DNS_TXT_DATAW", SimStruct), "Isdn": SimTypeRef("DNS_TXT_DATAW", SimStruct), "TXT": SimTypeRef("DNS_TXT_DATAW", SimStruct), "Txt": SimTypeRef("DNS_TXT_DATAW", SimStruct), "X25": SimTypeRef("DNS_TXT_DATAW", SimStruct), "Null": SimTypeRef("DNS_NULL_DATA", SimStruct), "WKS": SimTypeRef("DNS_WKS_DATA", SimStruct), "Wks": SimTypeRef("DNS_WKS_DATA", SimStruct), "AAAA": SimTypeRef("DNS_AAAA_DATA", SimStruct), "KEY": SimTypeRef("DNS_KEY_DATA", SimStruct), "Key": SimTypeRef("DNS_KEY_DATA", SimStruct), "SIG": SimTypeRef("DNS_SIG_DATAW", SimStruct), "Sig": SimTypeRef("DNS_SIG_DATAW", SimStruct), "ATMA": SimTypeRef("DNS_ATMA_DATA", SimStruct), "Atma": SimTypeRef("DNS_ATMA_DATA", SimStruct), "NXT": SimTypeRef("DNS_NXT_DATAW", SimStruct), "Nxt": SimTypeRef("DNS_NXT_DATAW", SimStruct), "SRV": SimTypeRef("DNS_SRV_DATAW", SimStruct), "Srv": SimTypeRef("DNS_SRV_DATAW", SimStruct), "NAPTR": SimTypeRef("DNS_NAPTR_DATAW", SimStruct), "Naptr": SimTypeRef("DNS_NAPTR_DATAW", SimStruct), "OPT": SimTypeRef("DNS_OPT_DATA", SimStruct), "Opt": SimTypeRef("DNS_OPT_DATA", SimStruct), "DS": SimTypeRef("DNS_DS_DATA", SimStruct), "Ds": SimTypeRef("DNS_DS_DATA", SimStruct), "RRSIG": SimTypeRef("DNS_SIG_DATAW", SimStruct), "Rrsig": SimTypeRef("DNS_SIG_DATAW", SimStruct), "NSEC": SimTypeRef("DNS_NSEC_DATAW", SimStruct), "Nsec": SimTypeRef("DNS_NSEC_DATAW", SimStruct), "DNSKEY": SimTypeRef("DNS_KEY_DATA", SimStruct), "Dnskey": SimTypeRef("DNS_KEY_DATA", SimStruct), "TKEY": SimTypeRef("DNS_TKEY_DATAW", SimStruct), "Tkey": SimTypeRef("DNS_TKEY_DATAW", SimStruct), "TSIG": SimTypeRef("DNS_TSIG_DATAW", SimStruct), "Tsig": SimTypeRef("DNS_TSIG_DATAW", SimStruct), "WINS": SimTypeRef("DNS_WINS_DATA", SimStruct), "Wins": SimTypeRef("DNS_WINS_DATA", SimStruct), "WINSR": SimTypeRef("DNS_WINSR_DATAW", SimStruct), "WinsR": SimTypeRef("DNS_WINSR_DATAW", SimStruct), "NBSTAT": SimTypeRef("DNS_WINSR_DATAW", SimStruct), "Nbstat": SimTypeRef("DNS_WINSR_DATAW", SimStruct), "DHCID": SimTypeRef("DNS_DHCID_DATA", SimStruct), "NSEC3": SimTypeRef("DNS_NSEC3_DATA", SimStruct), "Nsec3": SimTypeRef("DNS_NSEC3_DATA", SimStruct), "NSEC3PARAM": SimTypeRef("DNS_NSEC3PARAM_DATA", SimStruct), "Nsec3Param": SimTypeRef("DNS_NSEC3PARAM_DATA", SimStruct), "TLSA": SimTypeRef("DNS_TLSA_DATA", SimStruct), "Tlsa": SimTypeRef("DNS_TLSA_DATA", SimStruct), "SVCB": SimTypeRef("DNS_SVCB_DATA", SimStruct), "Svcb": SimTypeRef("DNS_SVCB_DATA", SimStruct), "UNKNOWN": SimTypeRef("DNS_UNKNOWN_DATA", SimStruct), "Unknown": SimTypeRef("DNS_UNKNOWN_DATA", SimStruct), "pDataPtr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="DNS_RECORDW", pack=False, align=None),
    "DNS_RECORD_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_RECORD_FLAGS", pack=False, align=None),
    "DNS_RECORD_OPTW": SimStruct(OrderedDict((("pNext", SimTypePointer(SimTypeRef("DNS_RECORDW", SimStruct), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("wDataLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"DW": SimTypeInt(signed=False, label="UInt32"), "S": SimTypeRef("DNS_RECORD_FLAGS", SimStruct)}, name="<anon>", label="None")), ("ExtHeader", SimTypeRef("DNS_HEADER_EXT", SimStruct)), ("wPayloadSize", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("Data", SimUnion({"OPT": SimTypeRef("DNS_OPT_DATA", SimStruct), "Opt": SimTypeRef("DNS_OPT_DATA", SimStruct)}, name="<anon>", label="None")),)), name="DNS_RECORD_OPTW", pack=False, align=None),
    "DNS_RRSET": SimStruct(OrderedDict((("pFirstRR", SimTypePointer(SimTypeRef("DNS_RECORDA", SimStruct), offset=0)), ("pLastRR", SimTypePointer(SimTypeRef("DNS_RECORDA", SimStruct), offset=0)),)), name="DNS_RRSET", pack=False, align=None),
    "DNS_SECTION": SimTypeInt(signed=False, label="DNS_SECTION"),
    "DNS_SERVER_PROPERTY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ServerIndex", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="DNS_SERVER_PROPERTY_TYPE")), ("Property", SimUnion({"DohSettings": SimTypePointer(SimTypeRef("DNS_DOH_SERVER_SETTINGS", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="DNS_SERVER_PROPERTY", pack=False, align=None),
    "DNS_SERVER_PROPERTY_TYPE": SimTypeInt(signed=False, label="DNS_SERVER_PROPERTY_TYPE"),
    "DNS_SERVER_PROPERTY_TYPES": SimUnion({"DohSettings": SimTypePointer(SimTypeRef("DNS_DOH_SERVER_SETTINGS", SimStruct), offset=0)}, name="<anon>", label="None"),
    "DNS_SERVICE_BROWSE_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("QueryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"pBrowseCallback": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_RECORDW", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "pQueryContext", "pDnsRecord"]), offset=0), "pBrowseCallbackV2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pQueryContext", "pQueryResults"]), offset=0)}, name="<anon>", label="None")), ("pQueryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DNS_SERVICE_BROWSE_REQUEST", pack=False, align=None),
    "DNS_SERVICE_CANCEL": SimStruct(OrderedDict((("reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DNS_SERVICE_CANCEL", pack=False, align=None),
    "DNS_SERVICE_INSTANCE": SimStruct(OrderedDict((("pszInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ip4Address", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ip6Address", SimTypePointer(SimUnion({"IP6Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4), "IP6Word": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "IP6Byte": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), offset=0)), ("wPort", SimTypeShort(signed=False, label="UInt16")), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wWeight", SimTypeShort(signed=False, label="UInt16")), ("dwPropertyCount", SimTypeInt(signed=False, label="UInt32")), ("keys", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("values", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwInterfaceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_SERVICE_INSTANCE", pack=False, align=None),
    "DNS_SERVICE_REGISTER_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pServiceInstance", SimTypePointer(SimTypeRef("DNS_SERVICE_INSTANCE", SimStruct), offset=0)), ("pRegisterCompletionCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_SERVICE_INSTANCE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "pQueryContext", "pInstance"]), offset=0)), ("pQueryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCredentials", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("unicastEnabled", SimTypeInt(signed=True, label="Int32")),)), name="DNS_SERVICE_REGISTER_REQUEST", pack=False, align=None),
    "DNS_SERVICE_RESOLVE_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("QueryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pResolveCompletionCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_SERVICE_INSTANCE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "pQueryContext", "pInstance"]), offset=0)), ("pQueryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DNS_SERVICE_RESOLVE_REQUEST", pack=False, align=None),
    "DNS_SETTINGS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Hostname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_SETTINGS", pack=False, align=None),
    "DNS_SETTINGS2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Hostname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SearchList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SettingFlags", SimTypeLongLong(signed=False, label="UInt64")),)), name="DNS_SETTINGS2", pack=False, align=None),
    "DNS_SIG_DATAA": SimStruct(OrderedDict((("wTypeCovered", SimTypeShort(signed=False, label="UInt16")), ("chAlgorithm", SimTypeChar(label="Byte")), ("chLabelCount", SimTypeChar(label="Byte")), ("dwOriginalTtl", SimTypeInt(signed=False, label="UInt32")), ("dwExpiration", SimTypeInt(signed=False, label="UInt32")), ("dwTimeSigned", SimTypeInt(signed=False, label="UInt32")), ("wKeyTag", SimTypeShort(signed=False, label="UInt16")), ("wSignatureLength", SimTypeShort(signed=False, label="UInt16")), ("pNameSigner", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Signature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_SIG_DATAA", pack=False, align=None),
    "DNS_SIG_DATAW": SimStruct(OrderedDict((("wTypeCovered", SimTypeShort(signed=False, label="UInt16")), ("chAlgorithm", SimTypeChar(label="Byte")), ("chLabelCount", SimTypeChar(label="Byte")), ("dwOriginalTtl", SimTypeInt(signed=False, label="UInt32")), ("dwExpiration", SimTypeInt(signed=False, label="UInt32")), ("dwTimeSigned", SimTypeInt(signed=False, label="UInt32")), ("wKeyTag", SimTypeShort(signed=False, label="UInt16")), ("wSignatureLength", SimTypeShort(signed=False, label="UInt16")), ("pNameSigner", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Signature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_SIG_DATAW", pack=False, align=None),
    "DNS_SOA_DATAA": SimStruct(OrderedDict((("pNamePrimaryServer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pNameAdministrator", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSerialNo", SimTypeInt(signed=False, label="UInt32")), ("dwRefresh", SimTypeInt(signed=False, label="UInt32")), ("dwRetry", SimTypeInt(signed=False, label="UInt32")), ("dwExpire", SimTypeInt(signed=False, label="UInt32")), ("dwDefaultTtl", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_SOA_DATAA", pack=False, align=None),
    "DNS_SOA_DATAW": SimStruct(OrderedDict((("pNamePrimaryServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pNameAdministrator", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSerialNo", SimTypeInt(signed=False, label="UInt32")), ("dwRefresh", SimTypeInt(signed=False, label="UInt32")), ("dwRetry", SimTypeInt(signed=False, label="UInt32")), ("dwExpire", SimTypeInt(signed=False, label="UInt32")), ("dwDefaultTtl", SimTypeInt(signed=False, label="UInt32")),)), name="DNS_SOA_DATAW", pack=False, align=None),
    "DNS_SRV_DATAA": SimStruct(OrderedDict((("pNameTarget", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wWeight", SimTypeShort(signed=False, label="UInt16")), ("wPort", SimTypeShort(signed=False, label="UInt16")), ("Pad", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_SRV_DATAA", pack=False, align=None),
    "DNS_SRV_DATAW": SimStruct(OrderedDict((("pNameTarget", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wPriority", SimTypeShort(signed=False, label="UInt16")), ("wWeight", SimTypeShort(signed=False, label="UInt16")), ("wPort", SimTypeShort(signed=False, label="UInt16")), ("Pad", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_SRV_DATAW", pack=False, align=None),
    "DNS_SVCB_DATA": SimStruct(OrderedDict((("wSvcPriority", SimTypeShort(signed=False, label="UInt16")), ("pszTargetName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cSvcParams", SimTypeShort(signed=False, label="UInt16")), ("pSvcParams", SimTypePointer(SimTypeRef("DNS_SVCB_PARAM", SimStruct), offset=0)),)), name="DNS_SVCB_DATA", pack=False, align=None),
    "DNS_SVCB_PARAM": SimStruct(OrderedDict((("wSvcParamKey", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"pIpv4Hints": SimTypePointer(SimTypeRef("DNS_SVCB_PARAM_IPV4", SimStruct), offset=0), "pIpv6Hints": SimTypePointer(SimTypeRef("DNS_SVCB_PARAM_IPV6", SimStruct), offset=0), "pMandatory": SimTypePointer(SimTypeRef("DNS_SVCB_PARAM_MANDATORY", SimStruct), offset=0), "pAlpn": SimTypePointer(SimTypeRef("DNS_SVCB_PARAM_ALPN", SimStruct), offset=0), "wPort": SimTypeShort(signed=False, label="UInt16"), "pUnknown": SimTypePointer(SimTypeRef("DNS_SVCB_PARAM_UNKNOWN", SimStruct), offset=0), "pszDohPath": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pReserved": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="DNS_SVCB_PARAM", pack=False, align=None),
    "DNS_SVCB_PARAM_ALPN": SimStruct(OrderedDict((("cIds", SimTypeShort(signed=False, label="UInt16")), ("rgIds", SimTypePointer(SimTypeRef("DNS_SVCB_PARAM_ALPN_ID", SimStruct), offset=0)),)), name="DNS_SVCB_PARAM_ALPN", pack=False, align=None),
    "DNS_SVCB_PARAM_ALPN_ID": SimStruct(OrderedDict((("cBytes", SimTypeChar(label="Byte")), ("pbId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_SVCB_PARAM_ALPN_ID", pack=False, align=None),
    "DNS_SVCB_PARAM_IPV4": SimStruct(OrderedDict((("cIps", SimTypeShort(signed=False, label="UInt16")), ("rgIps", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DNS_SVCB_PARAM_IPV4", pack=False, align=None),
    "DNS_SVCB_PARAM_IPV6": SimStruct(OrderedDict((("cIps", SimTypeShort(signed=False, label="UInt16")), ("rgIps", SimTypePointer(SimUnion({"IP6Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4), "IP6Word": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "IP6Byte": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), offset=0)),)), name="DNS_SVCB_PARAM_IPV6", pack=False, align=None),
    "DNS_SVCB_PARAM_MANDATORY": SimStruct(OrderedDict((("cMandatoryKeys", SimTypeShort(signed=False, label="UInt16")), ("rgwMandatoryKeys", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="DNS_SVCB_PARAM_MANDATORY", pack=False, align=None),
    "DNS_SVCB_PARAM_TYPE": SimTypeInt(signed=False, label="DNS_SVCB_PARAM_TYPE"),
    "DNS_SVCB_PARAM_UNKNOWN": SimStruct(OrderedDict((("cBytes", SimTypeShort(signed=False, label="UInt16")), ("pbSvcParamValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_SVCB_PARAM_UNKNOWN", pack=False, align=None),
    "DNS_TKEY_DATAA": SimStruct(OrderedDict((("pNameAlgorithm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pAlgorithmPacket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOtherData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwCreateTime", SimTypeInt(signed=False, label="UInt32")), ("dwExpireTime", SimTypeInt(signed=False, label="UInt32")), ("wMode", SimTypeShort(signed=False, label="UInt16")), ("wError", SimTypeShort(signed=False, label="UInt16")), ("wKeyLength", SimTypeShort(signed=False, label="UInt16")), ("wOtherLength", SimTypeShort(signed=False, label="UInt16")), ("cAlgNameLength", SimTypeChar(label="Byte")), ("bPacketPointers", SimTypeInt(signed=True, label="Int32")),)), name="DNS_TKEY_DATAA", pack=False, align=None),
    "DNS_TKEY_DATAW": SimStruct(OrderedDict((("pNameAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAlgorithmPacket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOtherData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwCreateTime", SimTypeInt(signed=False, label="UInt32")), ("dwExpireTime", SimTypeInt(signed=False, label="UInt32")), ("wMode", SimTypeShort(signed=False, label="UInt16")), ("wError", SimTypeShort(signed=False, label="UInt16")), ("wKeyLength", SimTypeShort(signed=False, label="UInt16")), ("wOtherLength", SimTypeShort(signed=False, label="UInt16")), ("cAlgNameLength", SimTypeChar(label="Byte")), ("bPacketPointers", SimTypeInt(signed=True, label="Int32")),)), name="DNS_TKEY_DATAW", pack=False, align=None),
    "DNS_TLSA_DATA": SimStruct(OrderedDict((("bCertUsage", SimTypeChar(label="Byte")), ("bSelector", SimTypeChar(label="Byte")), ("bMatchingType", SimTypeChar(label="Byte")), ("bCertificateAssociationDataLength", SimTypeShort(signed=False, label="UInt16")), ("bPad", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("bCertificateAssociationData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_TLSA_DATA", pack=False, align=None),
    "DNS_TSIG_DATAA": SimStruct(OrderedDict((("pNameAlgorithm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pAlgorithmPacket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOtherData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("i64CreateTime", SimTypeLongLong(signed=True, label="Int64")), ("wFudgeTime", SimTypeShort(signed=False, label="UInt16")), ("wOriginalXid", SimTypeShort(signed=False, label="UInt16")), ("wError", SimTypeShort(signed=False, label="UInt16")), ("wSigLength", SimTypeShort(signed=False, label="UInt16")), ("wOtherLength", SimTypeShort(signed=False, label="UInt16")), ("cAlgNameLength", SimTypeChar(label="Byte")), ("bPacketPointers", SimTypeInt(signed=True, label="Int32")),)), name="DNS_TSIG_DATAA", pack=False, align=None),
    "DNS_TSIG_DATAW": SimStruct(OrderedDict((("pNameAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAlgorithmPacket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOtherData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("i64CreateTime", SimTypeLongLong(signed=True, label="Int64")), ("wFudgeTime", SimTypeShort(signed=False, label="UInt16")), ("wOriginalXid", SimTypeShort(signed=False, label="UInt16")), ("wError", SimTypeShort(signed=False, label="UInt16")), ("wSigLength", SimTypeShort(signed=False, label="UInt16")), ("wOtherLength", SimTypeShort(signed=False, label="UInt16")), ("cAlgNameLength", SimTypeChar(label="Byte")), ("bPacketPointers", SimTypeInt(signed=True, label="Int32")),)), name="DNS_TSIG_DATAW", pack=False, align=None),
    "DNS_TXT_DATAA": SimStruct(OrderedDict((("dwStringCount", SimTypeInt(signed=False, label="UInt32")), ("pStringArray", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="DNS_TXT_DATAA", pack=False, align=None),
    "DNS_TXT_DATAW": SimStruct(OrderedDict((("dwStringCount", SimTypeInt(signed=False, label="UInt32")), ("pStringArray", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="DNS_TXT_DATAW", pack=False, align=None),
    "DNS_TYPE": SimTypeInt(signed=False, label="DNS_TYPE"),
    "DNS_UNKNOWN_DATA": SimStruct(OrderedDict((("dwByteCount", SimTypeInt(signed=False, label="UInt32")), ("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_UNKNOWN_DATA", pack=False, align=None),
    "DNS_WINSR_DATAA": SimStruct(OrderedDict((("dwMappingFlag", SimTypeInt(signed=False, label="UInt32")), ("dwLookupTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwCacheTimeout", SimTypeInt(signed=False, label="UInt32")), ("pNameResultDomain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_WINSR_DATAA", pack=False, align=None),
    "DNS_WINSR_DATAW": SimStruct(OrderedDict((("dwMappingFlag", SimTypeInt(signed=False, label="UInt32")), ("dwLookupTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwCacheTimeout", SimTypeInt(signed=False, label="UInt32")), ("pNameResultDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DNS_WINSR_DATAW", pack=False, align=None),
    "DNS_WINS_DATA": SimStruct(OrderedDict((("dwMappingFlag", SimTypeInt(signed=False, label="UInt32")), ("dwLookupTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwCacheTimeout", SimTypeInt(signed=False, label="UInt32")), ("cWinsServerCount", SimTypeInt(signed=False, label="UInt32")), ("WinsServers", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DNS_WINS_DATA", pack=False, align=None),
    "DNS_WIRE_QUESTION": SimStruct(OrderedDict((("QuestionType", SimTypeShort(signed=False, label="UInt16")), ("QuestionClass", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_WIRE_QUESTION", pack=False, align=None),
    "DNS_WIRE_RECORD": SimStruct(OrderedDict((("RecordType", SimTypeShort(signed=False, label="UInt16")), ("RecordClass", SimTypeShort(signed=False, label="UInt16")), ("TimeToLive", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeShort(signed=False, label="UInt16")),)), name="DNS_WIRE_RECORD", pack=False, align=None),
    "DNS_WKS_DATA": SimStruct(OrderedDict((("IpAddress", SimTypeInt(signed=False, label="UInt32")), ("chProtocol", SimTypeChar(label="Byte")), ("BitMask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DNS_WKS_DATA", pack=False, align=None),
    "DOCEVENT_CREATEDCPRE": SimStruct(OrderedDict((("pszDriver", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDevice", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pdm", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)), ("bIC", SimTypeInt(signed=True, label="Int32")),)), name="DOCEVENT_CREATEDCPRE", pack=False, align=None),
    "DOCEVENT_ESCAPE": SimStruct(OrderedDict((("iEscape", SimTypeInt(signed=True, label="Int32")), ("cjInput", SimTypeInt(signed=True, label="Int32")), ("pvInData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DOCEVENT_ESCAPE", pack=False, align=None),
    "DOCEVENT_FILTER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cElementsAllocated", SimTypeInt(signed=False, label="UInt32")), ("cElementsNeeded", SimTypeInt(signed=False, label="UInt32")), ("cElementsReturned", SimTypeInt(signed=False, label="UInt32")), ("aDocEventCall", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DOCEVENT_FILTER", pack=False, align=None),
    "DOCHOSTUIDBLCLK": SimTypeInt(signed=False, label="DOCHOSTUIDBLCLK"),
    "DOCHOSTUIFLAG": SimTypeInt(signed=False, label="DOCHOSTUIFLAG"),
    "DOCHOSTUIINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDoubleClick", SimTypeInt(signed=False, label="UInt32")), ("pchHostCss", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pchHostNS", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DOCHOSTUIINFO", pack=False, align=None),
    "DOCHOSTUITYPE": SimTypeInt(signed=False, label="DOCHOSTUITYPE"),
    "DOCINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("lpszDocName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszOutput", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fwType", SimTypeInt(signed=False, label="UInt32")),)), name="DOCINFOA", pack=False, align=None),
    "DOCINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("lpszDocName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszOutput", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fwType", SimTypeInt(signed=False, label="UInt32")),)), name="DOCINFOW", pack=False, align=None),
    "DOCMISC": SimTypeInt(signed=False, label="DOCMISC"),
    "DOCUMENTNAMETYPE": SimTypeInt(signed=False, label="DOCUMENTNAMETYPE"),
    "DOCUMENTPROPERTYHEADER": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszPrinterName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pdmIn", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pdmOut", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("cbOut", SimTypeInt(signed=False, label="UInt32")), ("fMode", SimTypeInt(signed=False, label="UInt32")),)), name="DOCUMENTPROPERTYHEADER", pack=False, align=None),
    "DOC_INFO_1A": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOC_INFO_1A", pack=False, align=None),
    "DOC_INFO_1W": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DOC_INFO_1W", pack=False, align=None),
    "DOC_INFO_2A": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwMode", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")),)), name="DOC_INFO_2A", pack=False, align=None),
    "DOC_INFO_2W": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMode", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")),)), name="DOC_INFO_2W", pack=False, align=None),
    "DOC_INFO_3A": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DOC_INFO_3A", pack=False, align=None),
    "DOC_INFO_3W": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DOC_INFO_3W", pack=False, align=None),
    "DOC_INFO_INTERNAL": SimStruct(OrderedDict((("pDocName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pOutputFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("bLowILJob", SimTypeInt(signed=True, label="Int32")), ("hTokenLowIL", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DOC_INFO_INTERNAL", pack=False, align=None),
    "DOLBYAC2WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("nAuxBitsCode", SimTypeShort(signed=False, label="UInt16")),)), name="DOLBYAC2WAVEFORMAT", pack=False, align=None),
    "DOMAINDESC": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszNCName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszTrustParent", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("fDownLevel", SimTypeInt(signed=True, label="Int32")), ("pdChildList", SimTypePointer(SimTypeRef("DOMAINDESC", SimStruct), offset=0)), ("pdNextSibling", SimTypePointer(SimTypeRef("DOMAINDESC", SimStruct), offset=0)),)), name="DOMAINDESC", pack=False, align=None),
    "DOMAIN_CONFIGURATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DOMAIN_CONFIGURATION_ARCH")), ("Anonymous", SimUnion({"Arm64": SimTypeRef("DOMAIN_CONFIGURATION_ARM64", SimStruct), "X64": SimTypeRef("DOMAIN_CONFIGURATION_X64", SimStruct)}, name="<anon>", label="None")),)), name="DOMAIN_CONFIGURATION", pack=False, align=None),
    "DOMAIN_CONFIGURATION_ARCH": SimTypeInt(signed=False, label="DOMAIN_CONFIGURATION_ARCH"),
    "DOMAIN_CONFIGURATION_ARM64": SimStruct(OrderedDict((("Ttbr0", SimTypeLongLong(signed=True, label="Int64")), ("Ttbr1", SimTypeLongLong(signed=True, label="Int64")), ("Mair0", SimTypeInt(signed=False, label="UInt32")), ("Mair1", SimTypeInt(signed=False, label="UInt32")), ("InputSize0", SimTypeChar(label="Byte")), ("InputSize1", SimTypeChar(label="Byte")), ("CoherentTableWalks", SimTypeChar(label="Byte")), ("TranslationEnabled", SimTypeChar(label="Byte")),)), name="DOMAIN_CONFIGURATION_ARM64", pack=False, align=None),
    "DOMAIN_CONFIGURATION_X64": SimStruct(OrderedDict((("FirstLevelPageTableRoot", SimTypeLongLong(signed=True, label="Int64")), ("TranslationEnabled", SimTypeChar(label="Byte")),)), name="DOMAIN_CONFIGURATION_X64", pack=False, align=None),
    "DOMAIN_CONTROLLER_INFOA": SimStruct(OrderedDict((("DomainControllerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DomainControllerAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DomainControllerAddressType", SimTypeInt(signed=False, label="UInt32")), ("DomainGuid", SimTypeBottom(label="Guid")), ("DomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsForestName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DcSiteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClientSiteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOMAIN_CONTROLLER_INFOA", pack=False, align=None),
    "DOMAIN_CONTROLLER_INFOW": SimStruct(OrderedDict((("DomainControllerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DomainControllerAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DomainControllerAddressType", SimTypeInt(signed=False, label="UInt32")), ("DomainGuid", SimTypeBottom(label="Guid")), ("DomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsForestName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DcSiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientSiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DOMAIN_CONTROLLER_INFOW", pack=False, align=None),
    "DOMAIN_PASSWORD_INFORMATION": SimStruct(OrderedDict((("MinPasswordLength", SimTypeShort(signed=False, label="UInt16")), ("PasswordHistoryLength", SimTypeShort(signed=False, label="UInt16")), ("PasswordProperties", SimTypeInt(signed=False, label="DOMAIN_PASSWORD_PROPERTIES")), ("MaxPasswordAge", SimTypeLongLong(signed=True, label="Int64")), ("MinPasswordAge", SimTypeLongLong(signed=True, label="Int64")),)), name="DOMAIN_PASSWORD_INFORMATION", pack=False, align=None),
    "DOMAIN_PASSWORD_PROPERTIES": SimTypeInt(signed=False, label="DOMAIN_PASSWORD_PROPERTIES"),
    "DOMAIN_TREE": SimStruct(OrderedDict((("dsSize", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("aDomains", SimTypePointer(SimTypeRef("DOMAINDESC", SimStruct), offset=0)),)), name="DOMAIN_TREE", pack=False, align=None),
    "DOMNodeType": SimTypeInt(signed=False, label="DOMNodeType"),
    "DOM_EVENT_PHASE": SimTypeInt(signed=False, label="DOM_EVENT_PHASE"),
    "DOS_IMAGE_INFO": SimStruct(OrderedDict((("PdbName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ImageBaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("Timestamp", SimTypeInt(signed=False, label="UInt32")),)), name="DOS_IMAGE_INFO", pack=False, align=None),
    "DOT11EXTIHV_ADAPTER_RESET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter"]), offset=0),
    "DOT11EXTIHV_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "dwInBufferSize", "pInBuffer", "dwOutBufferSize", "pOutBuffer", "pdwBytesReturned"]), offset=0),
    "DOT11EXTIHV_CREATE_DISCOVERY_PROFILES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11_BSS_LIST", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_DISCOVERY_PROFILE_LIST", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "bInsecure", "pIhvProfileParams", "pConnectableBssid", "pIhvDiscoveryProfileList", "pdwReasonCode"]), offset=0),
    "DOT11EXTIHV_DEINIT_ADAPTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hIhvExtAdapter"]), offset=0),
    "DOT11EXTIHV_DEINIT_SERVICE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "DOT11EXTIHV_GET_VERSION_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DOT11_IHV_VERSION_INFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pDot11IHVVersionInfo"]), offset=0),
    "DOT11EXTIHV_INIT_ADAPTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DOT11_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pDot11Adapter", "hDot11SvcHandle", "phIhvExtAdapter"]), offset=0),
    "DOT11EXTIHV_INIT_SERVICE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11EXT_APIS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_HANDLERS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwVerNumUsed", "pDot11ExtAPI", "pvReserved", "pDot11IHVHandlers"]), offset=0),
    "DOT11EXTIHV_INIT_VIRTUAL_STATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DOT11EXT_VIRTUAL_STATION_APIS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pDot11ExtVSAPI", "pvReserved"]), offset=0),
    "DOT11EXTIHV_IS_UI_REQUEST_PENDING": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["guidUIRequest", "pbIsRequestPending"]), offset=0),
    "DOT11EXTIHV_ONEX_INDICATE_RESULT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DOT11_MSONEX_RESULT"), SimTypePointer(SimTypeRef("DOT11_MSONEX_RESULT_PARAMS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "msOneXResult", "pDot11MsOneXResultParams"]), offset=0),
    "DOT11EXTIHV_PERFORM_CAPABILITY_MATCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11_BSS_LIST", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "pIhvProfileParams", "pIhvConnProfile", "pIhvSecProfile", "pConnectableBssid", "pdwReasonCode"]), offset=0),
    "DOT11EXTIHV_PERFORM_POST_ASSOCIATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11_PORT_STATE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11_ASSOCIATION_COMPLETION_PARAMETERS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "hSecuritySessionID", "pPortState", "uDot11AssocParamsBytes", "pDot11AssocParams"]), offset=0),
    "DOT11EXTIHV_PERFORM_PRE_ASSOCIATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11_BSS_LIST", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "hConnectSession", "pIhvProfileParams", "pIhvConnProfile", "pIhvSecProfile", "pConnectableBssid", "pdwReasonCode"]), offset=0),
    "DOT11EXTIHV_PROCESS_SESSION_CHANGE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WTSSESSION_NOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["uEventType", "pSessionNotification"]), offset=0),
    "DOT11EXTIHV_PROCESS_UI_RESPONSE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["guidUIRequest", "dwByteCount", "pvResponseBuffer"]), offset=0),
    "DOT11EXTIHV_QUERY_UI_REQUEST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DOT11EXT_IHV_CONNECTION_PHASE"), SimTypePointer(SimTypePointer(SimTypeRef("DOT11EXT_IHV_UI_REQUEST", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "connectionPhase", "ppIhvUIRequest"]), offset=0),
    "DOT11EXTIHV_RECEIVE_INDICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DOT11EXT_IHV_INDICATION_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "indicationType", "uBufferLength", "pvBuffer"]), offset=0),
    "DOT11EXTIHV_RECEIVE_PACKET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "dwInBufferSize", "pvInBuffer"]), offset=0),
    "DOT11EXTIHV_SEND_PACKET_COMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hSendCompletion"]), offset=0),
    "DOT11EXTIHV_STOP_POST_ASSOCIATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "pPeer", "dot11AssocStatus"]), offset=0),
    "DOT11EXTIHV_VALIDATE_PROFILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "pIhvProfileParams", "pIhvConnProfile", "pIhvSecProfile", "pdwReasonCode"]), offset=0),
    "DOT11EXT_ALLOCATE_BUFFER": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwByteCount", "ppvBuffer"]), offset=0),
    "DOT11EXT_APIS": SimStruct(OrderedDict((("Dot11ExtAllocateBuffer", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwByteCount", "ppvBuffer"]), offset=0)), ("Dot11ExtFreeBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvMemory"]), offset=0)), ("Dot11ExtSetProfileCustomUserData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwSessionID", "dwDataSize", "pvData"]), offset=0)), ("Dot11ExtGetProfileCustomUserData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwSessionID", "pdwDataSize", "ppvData"]), offset=0)), ("Dot11ExtSetCurrentProfile", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "pIhvConnProfile", "pIhvSecProfile"]), offset=0)), ("Dot11ExtSendUIRequest", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_UI_REQUEST", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pIhvUIRequest"]), offset=0)), ("Dot11ExtPreAssociateCompletion", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwReasonCode", "dwWin32Error"]), offset=0)), ("Dot11ExtPostAssociateCompletion", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hSecuritySessionID", "pPeer", "dwReasonCode", "dwWin32Error"]), offset=0)), ("Dot11ExtSendNotification", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("L2_NOTIFICATION_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pNotificationData"]), offset=0)), ("Dot11ExtSendPacket", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "uPacketLen", "pvPacket", "hSendCompletion"]), offset=0)), ("Dot11ExtSetEtherTypeHandling", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11_PRIVACY_EXEMPTION", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "uMaxBackLog", "uNumOfExemption", "pExemption", "uNumOfRegistration", "pusRegistration"]), offset=0)), ("Dot11ExtSetAuthAlgorithm", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwAuthAlgo"]), offset=0)), ("Dot11ExtSetUnicastCipherAlgorithm", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwUnicastCipherAlgo"]), offset=0)), ("Dot11ExtSetMulticastCipherAlgorithm", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwMulticastCipherAlgo"]), offset=0)), ("Dot11ExtSetDefaultKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11_CIPHER_DEFAULT_KEY_VALUE", SimStruct), offset=0), SimTypeInt(signed=False, label="DOT11_DIRECTION")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pKey", "dot11Direction"]), offset=0)), ("Dot11ExtSetKeyMappingKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11_CIPHER_KEY_MAPPING_KEY_VALUE", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pKey"]), offset=0)), ("Dot11ExtSetDefaultKeyId", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "uDefaultKeyId"]), offset=0)), ("Dot11ExtNicSpecificExtension", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwInBufferSize", "pvInBuffer", "pdwOutBufferSize", "pvOutBuffer"]), offset=0)), ("Dot11ExtSetExcludeUnencrypted", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "bExcludeUnencrypted"]), offset=0)), ("Dot11ExtStartOneX", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("EAP_ATTRIBUTES", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pEapAttributes"]), offset=0)), ("Dot11ExtStopOneX", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle"]), offset=0)), ("Dot11ExtProcessSecurityPacket", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwInPacketSize", "pvInPacket"]), offset=0)),)), name="DOT11EXT_APIS", pack=False, align=None),
    "DOT11EXT_FREE_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvMemory"]), offset=0),
    "DOT11EXT_GET_PROFILE_CUSTOM_USER_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwSessionID", "pdwDataSize", "ppvData"]), offset=0),
    "DOT11EXT_IHV_CONNECTION_PHASE": SimTypeInt(signed=False, label="DOT11EXT_IHV_CONNECTION_PHASE"),
    "DOT11EXT_IHV_CONNECTIVITY_PROFILE": SimStruct(OrderedDict((("pszXmlFragmentIhvConnectivity", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DOT11EXT_IHV_CONNECTIVITY_PROFILE", pack=False, align=None),
    "DOT11EXT_IHV_DISCOVERY_PROFILE": SimStruct(OrderedDict((("IhvConnectivityProfile", SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct)), ("IhvSecurityProfile", SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct)),)), name="DOT11EXT_IHV_DISCOVERY_PROFILE", pack=False, align=None),
    "DOT11EXT_IHV_DISCOVERY_PROFILE_LIST": SimStruct(OrderedDict((("dwCount", SimTypeInt(signed=False, label="UInt32")), ("pIhvDiscoveryProfiles", SimTypePointer(SimTypeRef("DOT11EXT_IHV_DISCOVERY_PROFILE", SimStruct), offset=0)),)), name="DOT11EXT_IHV_DISCOVERY_PROFILE_LIST", pack=False, align=None),
    "DOT11EXT_IHV_HANDLERS": SimStruct(OrderedDict((("Dot11ExtIhvDeinitService", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Dot11ExtIhvInitAdapter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DOT11_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pDot11Adapter", "hDot11SvcHandle", "phIhvExtAdapter"]), offset=0)), ("Dot11ExtIhvDeinitAdapter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hIhvExtAdapter"]), offset=0)), ("Dot11ExtIhvPerformPreAssociate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11_BSS_LIST", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "hConnectSession", "pIhvProfileParams", "pIhvConnProfile", "pIhvSecProfile", "pConnectableBssid", "pdwReasonCode"]), offset=0)), ("Dot11ExtIhvAdapterReset", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter"]), offset=0)), ("Dot11ExtIhvPerformPostAssociate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11_PORT_STATE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11_ASSOCIATION_COMPLETION_PARAMETERS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "hSecuritySessionID", "pPortState", "uDot11AssocParamsBytes", "pDot11AssocParams"]), offset=0)), ("Dot11ExtIhvStopPostAssociate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "pPeer", "dot11AssocStatus"]), offset=0)), ("Dot11ExtIhvValidateProfile", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "pIhvProfileParams", "pIhvConnProfile", "pIhvSecProfile", "pdwReasonCode"]), offset=0)), ("Dot11ExtIhvPerformCapabilityMatch", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11_BSS_LIST", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "pIhvProfileParams", "pIhvConnProfile", "pIhvSecProfile", "pConnectableBssid", "pdwReasonCode"]), offset=0)), ("Dot11ExtIhvCreateDiscoveryProfiles", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11_BSS_LIST", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_DISCOVERY_PROFILE_LIST", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "bInsecure", "pIhvProfileParams", "pConnectableBssid", "pIhvDiscoveryProfileList", "pdwReasonCode"]), offset=0)), ("Dot11ExtIhvProcessSessionChange", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WTSSESSION_NOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["uEventType", "pSessionNotification"]), offset=0)), ("Dot11ExtIhvReceiveIndication", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DOT11EXT_IHV_INDICATION_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "indicationType", "uBufferLength", "pvBuffer"]), offset=0)), ("Dot11ExtIhvReceivePacket", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "dwInBufferSize", "pvInBuffer"]), offset=0)), ("Dot11ExtIhvSendPacketCompletion", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hSendCompletion"]), offset=0)), ("Dot11ExtIhvIsUIRequestPending", SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["guidUIRequest", "pbIsRequestPending"]), offset=0)), ("Dot11ExtIhvProcessUIResponse", SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["guidUIRequest", "dwByteCount", "pvResponseBuffer"]), offset=0)), ("Dot11ExtIhvQueryUIRequest", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DOT11EXT_IHV_CONNECTION_PHASE"), SimTypePointer(SimTypePointer(SimTypeRef("DOT11EXT_IHV_UI_REQUEST", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "connectionPhase", "ppIhvUIRequest"]), offset=0)), ("Dot11ExtIhvOnexIndicateResult", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DOT11_MSONEX_RESULT"), SimTypePointer(SimTypeRef("DOT11_MSONEX_RESULT_PARAMS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "msOneXResult", "pDot11MsOneXResultParams"]), offset=0)), ("Dot11ExtIhvControl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hIhvExtAdapter", "dwInBufferSize", "pInBuffer", "dwOutBufferSize", "pOutBuffer", "pdwBytesReturned"]), offset=0)),)), name="DOT11EXT_IHV_HANDLERS", pack=False, align=None),
    "DOT11EXT_IHV_INDICATION_TYPE": SimTypeInt(signed=False, label="DOT11EXT_IHV_INDICATION_TYPE"),
    "DOT11EXT_IHV_PARAMS": SimStruct(OrderedDict((("dot11ExtIhvProfileParams", SimTypeRef("DOT11EXT_IHV_PROFILE_PARAMS", SimStruct)), ("wstrProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwProfileTypeFlags", SimTypeInt(signed=False, label="UInt32")), ("interfaceGuid", SimTypeBottom(label="Guid")),)), name="DOT11EXT_IHV_PARAMS", pack=False, align=None),
    "DOT11EXT_IHV_PROFILE_PARAMS": SimStruct(OrderedDict((("pSsidList", SimTypePointer(SimTypeRef("DOT11EXT_IHV_SSID_LIST", SimStruct), offset=0)), ("BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("pMSSecuritySettings", SimTypePointer(SimTypeRef("DOT11_MSSECURITY_SETTINGS", SimStruct), offset=0)),)), name="DOT11EXT_IHV_PROFILE_PARAMS", pack=False, align=None),
    "DOT11EXT_IHV_SECURITY_PROFILE": SimStruct(OrderedDict((("pszXmlFragmentIhvSecurity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bUseMSOnex", SimTypeInt(signed=True, label="Int32")),)), name="DOT11EXT_IHV_SECURITY_PROFILE", pack=False, align=None),
    "DOT11EXT_IHV_SSID_LIST": SimStruct(OrderedDict((("ulCount", SimTypeInt(signed=False, label="UInt32")), ("SSIDs", SimTypePointer(SimTypeRef("DOT11_SSID", SimStruct), offset=0)),)), name="DOT11EXT_IHV_SSID_LIST", pack=False, align=None),
    "DOT11EXT_IHV_UI_REQUEST": SimStruct(OrderedDict((("dwSessionId", SimTypeInt(signed=False, label="UInt32")), ("guidUIRequest", SimTypeBottom(label="Guid")), ("UIPageClsid", SimTypeBottom(label="Guid")), ("dwByteCount", SimTypeInt(signed=False, label="UInt32")), ("pvUIRequest", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11EXT_IHV_UI_REQUEST", pack=False, align=None),
    "DOT11EXT_NIC_SPECIFIC_EXTENSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwInBufferSize", "pvInBuffer", "pdwOutBufferSize", "pvOutBuffer"]), offset=0),
    "DOT11EXT_ONEX_START": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("EAP_ATTRIBUTES", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pEapAttributes"]), offset=0),
    "DOT11EXT_ONEX_STOP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle"]), offset=0),
    "DOT11EXT_POST_ASSOCIATE_COMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hSecuritySessionID", "pPeer", "dwReasonCode", "dwWin32Error"]), offset=0),
    "DOT11EXT_PRE_ASSOCIATE_COMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwReasonCode", "dwWin32Error"]), offset=0),
    "DOT11EXT_PROCESS_ONEX_PACKET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwInPacketSize", "pvInPacket"]), offset=0),
    "DOT11EXT_QUERY_VIRTUAL_STATION_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pbIsVirtualStation", "pgPrimary", "pvReserved"]), offset=0),
    "DOT11EXT_RELEASE_VIRTUAL_STATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11PrimaryHandle", "pvReserved"]), offset=0),
    "DOT11EXT_REQUEST_VIRTUAL_STATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11PrimaryHandle", "pvReserved"]), offset=0),
    "DOT11EXT_SEND_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("L2_NOTIFICATION_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pNotificationData"]), offset=0),
    "DOT11EXT_SEND_PACKET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "uPacketLen", "pvPacket", "hSendCompletion"]), offset=0),
    "DOT11EXT_SEND_UI_REQUEST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_UI_REQUEST", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pIhvUIRequest"]), offset=0),
    "DOT11EXT_SET_AUTH_ALGORITHM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwAuthAlgo"]), offset=0),
    "DOT11EXT_SET_CURRENT_PROFILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_CONNECTIVITY_PROFILE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DOT11EXT_IHV_SECURITY_PROFILE", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "pIhvConnProfile", "pIhvSecProfile"]), offset=0),
    "DOT11EXT_SET_DEFAULT_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11_CIPHER_DEFAULT_KEY_VALUE", SimStruct), offset=0), SimTypeInt(signed=False, label="DOT11_DIRECTION")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pKey", "dot11Direction"]), offset=0),
    "DOT11EXT_SET_DEFAULT_KEY_ID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "uDefaultKeyId"]), offset=0),
    "DOT11EXT_SET_ETHERTYPE_HANDLING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11_PRIVACY_EXEMPTION", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "uMaxBackLog", "uNumOfExemption", "pExemption", "uNumOfRegistration", "pusRegistration"]), offset=0),
    "DOT11EXT_SET_EXCLUDE_UNENCRYPTED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "bExcludeUnencrypted"]), offset=0),
    "DOT11EXT_SET_KEY_MAPPING_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DOT11_CIPHER_KEY_MAPPING_KEY_VALUE", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pKey"]), offset=0),
    "DOT11EXT_SET_MULTICAST_CIPHER_ALGORITHM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwMulticastCipherAlgo"]), offset=0),
    "DOT11EXT_SET_PROFILE_CUSTOM_USER_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwSessionID", "dwDataSize", "pvData"]), offset=0),
    "DOT11EXT_SET_UNICAST_CIPHER_ALGORITHM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "dwUnicastCipherAlgo"]), offset=0),
    "DOT11EXT_SET_VIRTUAL_STATION_AP_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11EXT_VIRTUAL_STATION_AP_PROPERTY", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwNumProperties", "pProperties", "pvReserved"]), offset=0),
    "DOT11EXT_VIRTUAL_STATION_APIS": SimStruct(OrderedDict((("Dot11ExtRequestVirtualStation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11PrimaryHandle", "pvReserved"]), offset=0)), ("Dot11ExtReleaseVirtualStation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11PrimaryHandle", "pvReserved"]), offset=0)), ("Dot11ExtQueryVirtualStationProperties", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "pbIsVirtualStation", "pgPrimary", "pvReserved"]), offset=0)), ("Dot11ExtSetVirtualStationAPProperties", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DOT11EXT_VIRTUAL_STATION_AP_PROPERTY", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDot11SvcHandle", "hConnectSession", "dwNumProperties", "pProperties", "pvReserved"]), offset=0)),)), name="DOT11EXT_VIRTUAL_STATION_APIS", pack=False, align=None),
    "DOT11EXT_VIRTUAL_STATION_AP_PROPERTY": SimStruct(OrderedDict((("dot11SSID", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11AuthAlgo", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("dot11CipherAlgo", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("bIsPassPhrase", SimTypeInt(signed=True, label="Int32")), ("dwKeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucKeyData", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="DOT11EXT_VIRTUAL_STATION_AP_PROPERTY", pack=False, align=None),
    "DOT11_ACCESSNETWORKOPTIONS": SimStruct(OrderedDict((("AccessNetworkType", SimTypeChar(label="Byte")), ("Internet", SimTypeChar(label="Byte")), ("ASRA", SimTypeChar(label="Byte")), ("ESR", SimTypeChar(label="Byte")), ("UESA", SimTypeChar(label="Byte")),)), name="DOT11_ACCESSNETWORKOPTIONS", pack=False, align=None),
    "DOT11_AC_PARAM": SimTypeInt(signed=False, label="DOT11_AC_PARAM"),
    "DOT11_ADAPTER": SimStruct(OrderedDict((("gAdapterId", SimTypeBottom(label="Guid")), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Dot11CurrentOpMode", SimTypeRef("DOT11_CURRENT_OPERATION_MODE", SimStruct)),)), name="DOT11_ADAPTER", pack=False, align=None),
    "DOT11_ADDITIONAL_IE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uBeaconIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uBeaconIEsLength", SimTypeInt(signed=False, label="UInt32")), ("uResponseIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uResponseIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ADDITIONAL_IE", pack=False, align=None),
    "DOT11_ADHOC_AUTH_ALGORITHM": SimTypeInt(signed=False, label="DOT11_ADHOC_AUTH_ALGORITHM"),
    "DOT11_ADHOC_CIPHER_ALGORITHM": SimTypeInt(signed=False, label="DOT11_ADHOC_CIPHER_ALGORITHM"),
    "DOT11_ADHOC_CONNECT_FAIL_REASON": SimTypeInt(signed=False, label="DOT11_ADHOC_CONNECT_FAIL_REASON"),
    "DOT11_ADHOC_NETWORK_CONNECTION_STATUS": SimTypeInt(signed=False, label="DOT11_ADHOC_NETWORK_CONNECTION_STATUS"),
    "DOT11_ANQP_QUERY_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Status", SimTypeInt(signed=False, label="DOT11_ANQP_QUERY_RESULT")), ("hContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uResponseLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ANQP_QUERY_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_ANQP_QUERY_RESULT": SimTypeInt(signed=False, label="DOT11_ANQP_QUERY_RESULT"),
    "DOT11_AP_JOIN_REQUEST": SimStruct(OrderedDict((("uJoinFailureTimeout", SimTypeInt(signed=False, label="UInt32")), ("OperationalRateSet", SimTypeRef("DOT11_RATE_SET", SimStruct)), ("uChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("dot11BSSDescription", SimTypeRef("DOT11_BSS_DESCRIPTION", SimStruct)),)), name="DOT11_AP_JOIN_REQUEST", pack=False, align=None),
    "DOT11_ASSOCIATION_COMPLETION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("MacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uStatus", SimTypeInt(signed=False, label="UInt32")), ("bReAssocReq", SimTypeChar(label="Byte")), ("bReAssocResp", SimTypeChar(label="Byte")), ("uAssocReqOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocReqSize", SimTypeInt(signed=False, label="UInt32")), ("uAssocRespOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocRespSize", SimTypeInt(signed=False, label="UInt32")), ("uBeaconOffset", SimTypeInt(signed=False, label="UInt32")), ("uBeaconSize", SimTypeInt(signed=False, label="UInt32")), ("uIHVDataOffset", SimTypeInt(signed=False, label="UInt32")), ("uIHVDataSize", SimTypeInt(signed=False, label="UInt32")), ("AuthAlgo", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("UnicastCipher", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("MulticastCipher", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("uActivePhyListOffset", SimTypeInt(signed=False, label="UInt32")), ("uActivePhyListSize", SimTypeInt(signed=False, label="UInt32")), ("bFourAddressSupported", SimTypeChar(label="Byte")), ("bPortAuthorized", SimTypeChar(label="Byte")), ("ucActiveQoSProtocol", SimTypeChar(label="Byte")), ("DSInfo", SimTypeInt(signed=False, label="DOT11_DS_INFO")), ("uEncapTableOffset", SimTypeInt(signed=False, label="UInt32")), ("uEncapTableSize", SimTypeInt(signed=False, label="UInt32")), ("MulticastMgmtCipher", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("uAssocComebackTime", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ASSOCIATION_COMPLETION_PARAMETERS", pack=False, align=None),
    "DOT11_ASSOCIATION_INFO_EX": SimStruct(OrderedDict((("PeerMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("usCapabilityInformation", SimTypeShort(signed=False, label="UInt16")), ("usListenInterval", SimTypeShort(signed=False, label="UInt16")), ("ucPeerSupportedRates", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("usAssociationID", SimTypeShort(signed=False, label="UInt16")), ("dot11AssociationState", SimTypeInt(signed=False, label="DOT11_ASSOCIATION_STATE")), ("dot11PowerMode", SimTypeInt(signed=False, label="DOT11_POWER_MODE")), ("liAssociationUpTime", SimTypeLongLong(signed=True, label="Int64")), ("ullNumOfTxPacketSuccesses", SimTypeLongLong(signed=False, label="UInt64")), ("ullNumOfTxPacketFailures", SimTypeLongLong(signed=False, label="UInt64")), ("ullNumOfRxPacketSuccesses", SimTypeLongLong(signed=False, label="UInt64")), ("ullNumOfRxPacketFailures", SimTypeLongLong(signed=False, label="UInt64")),)), name="DOT11_ASSOCIATION_INFO_EX", pack=False, align=None),
    "DOT11_ASSOCIATION_INFO_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11AssocInfo", SimTypePointer(SimTypeRef("DOT11_ASSOCIATION_INFO_EX", SimStruct), offset=0)),)), name="DOT11_ASSOCIATION_INFO_LIST", pack=False, align=None),
    "DOT11_ASSOCIATION_PARAMS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uAssocRequestIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocRequestIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ASSOCIATION_PARAMS", pack=False, align=None),
    "DOT11_ASSOCIATION_START_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("MacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("SSID", SimTypeRef("DOT11_SSID", SimStruct)), ("uIHVDataOffset", SimTypeInt(signed=False, label="UInt32")), ("uIHVDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ASSOCIATION_START_PARAMETERS", pack=False, align=None),
    "DOT11_ASSOCIATION_STATE": SimTypeInt(signed=False, label="DOT11_ASSOCIATION_STATE"),
    "DOT11_AUTH_ALGORITHM": SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM"),
    "DOT11_AUTH_ALGORITHM_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("AlgorithmIds", SimTypePointer(SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM"), offset=0)),)), name="DOT11_AUTH_ALGORITHM_LIST", pack=False, align=None),
    "DOT11_AUTH_CIPHER_PAIR": SimStruct(OrderedDict((("AuthAlgoId", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("CipherAlgoId", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")),)), name="DOT11_AUTH_CIPHER_PAIR", pack=False, align=None),
    "DOT11_AUTH_CIPHER_PAIR_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("AuthCipherPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)),)), name="DOT11_AUTH_CIPHER_PAIR_LIST", pack=False, align=None),
    "DOT11_AVAILABLE_CHANNEL_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uChannelNumber", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DOT11_AVAILABLE_CHANNEL_LIST", pack=False, align=None),
    "DOT11_AVAILABLE_FREQUENCY_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uFrequencyValue", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DOT11_AVAILABLE_FREQUENCY_LIST", pack=False, align=None),
    "DOT11_BAND": SimTypeInt(signed=False, label="DOT11_BAND"),
    "DOT11_BSSID_CANDIDATE": SimStruct(OrderedDict((("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_BSSID_CANDIDATE", pack=False, align=None),
    "DOT11_BSSID_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("BSSIDs", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_BSSID_LIST", pack=False, align=None),
    "DOT11_BSS_DESCRIPTION": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("dot11BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11BSSType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("usBeaconPeriod", SimTypeShort(signed=False, label="UInt16")), ("ullTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("usCapabilityInformation", SimTypeShort(signed=False, label="UInt16")), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_BSS_DESCRIPTION", pack=False, align=None),
    "DOT11_BSS_ENTRY": SimStruct(OrderedDict((("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("PhySpecificInfo", SimUnion({"uChCenterFrequency": SimTypeInt(signed=False, label="UInt32"), "FHSS": SimTypeRef("_FHSS_e__Struct", SimStruct)}, name="<anon>", label="None")), ("dot11BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11BSSType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("lRSSI", SimTypeInt(signed=True, label="Int32")), ("uLinkQuality", SimTypeInt(signed=False, label="UInt32")), ("bInRegDomain", SimTypeChar(label="Byte")), ("usBeaconPeriod", SimTypeShort(signed=False, label="UInt16")), ("ullTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("ullHostTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("usCapabilityInformation", SimTypeShort(signed=False, label="UInt16")), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_BSS_ENTRY", pack=False, align=None),
    "DOT11_BSS_ENTRY_PHY_SPECIFIC_INFO": SimUnion({"uChCenterFrequency": SimTypeInt(signed=False, label="UInt32"), "FHSS": SimTypeRef("_FHSS_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "DOT11_BSS_LIST": SimStruct(OrderedDict((("uNumOfBytes", SimTypeInt(signed=False, label="UInt32")), ("pucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_BSS_LIST", pack=False, align=None),
    "DOT11_BSS_TYPE": SimTypeInt(signed=False, label="DOT11_BSS_TYPE"),
    "DOT11_BYTE_ARRAY": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfBytes", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfBytes", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_BYTE_ARRAY", pack=False, align=None),
    "DOT11_CAN_SUSTAIN_AP_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ulReason", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_CAN_SUSTAIN_AP_PARAMETERS", pack=False, align=None),
    "DOT11_CHANNEL_HINT": SimStruct(OrderedDict((("Dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("uChannelNumber", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_CHANNEL_HINT", pack=False, align=None),
    "DOT11_CIPHER_ALGORITHM": SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM"),
    "DOT11_CIPHER_ALGORITHM_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("AlgorithmIds", SimTypePointer(SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM"), offset=0)),)), name="DOT11_CIPHER_ALGORITHM_LIST", pack=False, align=None),
    "DOT11_CIPHER_DEFAULT_KEY_VALUE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uKeyIndex", SimTypeInt(signed=False, label="UInt32")), ("AlgorithmId", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("MacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bDelete", SimTypeChar(label="Byte")), ("bStatic", SimTypeChar(label="Byte")), ("usKeyLength", SimTypeShort(signed=False, label="UInt16")), ("ucKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_CIPHER_DEFAULT_KEY_VALUE", pack=False, align=None),
    "DOT11_CIPHER_KEY_MAPPING_KEY_VALUE": SimStruct(OrderedDict((("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("AlgorithmId", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("Direction", SimTypeInt(signed=False, label="DOT11_DIRECTION")), ("bDelete", SimTypeChar(label="Byte")), ("bStatic", SimTypeChar(label="Byte")), ("usKeyLength", SimTypeShort(signed=False, label="UInt16")), ("ucKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_CIPHER_KEY_MAPPING_KEY_VALUE", pack=False, align=None),
    "DOT11_CONNECTION_COMPLETION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uStatus", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_CONNECTION_COMPLETION_PARAMETERS", pack=False, align=None),
    "DOT11_CONNECTION_START_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("BSSType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("AdhocBSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("AdhocSSID", SimTypeRef("DOT11_SSID", SimStruct)),)), name="DOT11_CONNECTION_START_PARAMETERS", pack=False, align=None),
    "DOT11_COUNTERS_ENTRY": SimStruct(OrderedDict((("uTransmittedFragmentCount", SimTypeInt(signed=False, label="UInt32")), ("uMulticastTransmittedFrameCount", SimTypeInt(signed=False, label="UInt32")), ("uFailedCount", SimTypeInt(signed=False, label="UInt32")), ("uRetryCount", SimTypeInt(signed=False, label="UInt32")), ("uMultipleRetryCount", SimTypeInt(signed=False, label="UInt32")), ("uFrameDuplicateCount", SimTypeInt(signed=False, label="UInt32")), ("uRTSSuccessCount", SimTypeInt(signed=False, label="UInt32")), ("uRTSFailureCount", SimTypeInt(signed=False, label="UInt32")), ("uACKFailureCount", SimTypeInt(signed=False, label="UInt32")), ("uReceivedFragmentCount", SimTypeInt(signed=False, label="UInt32")), ("uMulticastReceivedFrameCount", SimTypeInt(signed=False, label="UInt32")), ("uFCSErrorCount", SimTypeInt(signed=False, label="UInt32")), ("uTransmittedFrameCount", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_COUNTERS_ENTRY", pack=False, align=None),
    "DOT11_COUNTRY_OR_REGION_STRING_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("CountryOrRegionStrings", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="DOT11_COUNTRY_OR_REGION_STRING_LIST", pack=False, align=None),
    "DOT11_CURRENT_OFFLOAD_CAPABILITY": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_CURRENT_OFFLOAD_CAPABILITY", pack=False, align=None),
    "DOT11_CURRENT_OPERATION_MODE": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("uCurrentOpMode", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_CURRENT_OPERATION_MODE", pack=False, align=None),
    "DOT11_CURRENT_OPTIONAL_CAPABILITY": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("bDot11CFPollable", SimTypeChar(label="Byte")), ("bDot11PCF", SimTypeChar(label="Byte")), ("bDot11PCFMPDUTransferToPC", SimTypeChar(label="Byte")), ("bStrictlyOrderedServiceClass", SimTypeChar(label="Byte")),)), name="DOT11_CURRENT_OPTIONAL_CAPABILITY", pack=False, align=None),
    "DOT11_DATA_RATE_MAPPING_ENTRY": SimStruct(OrderedDict((("ucDataRateIndex", SimTypeChar(label="Byte")), ("ucDataRateFlag", SimTypeChar(label="Byte")), ("usDataRateValue", SimTypeShort(signed=False, label="UInt16")),)), name="DOT11_DATA_RATE_MAPPING_ENTRY", pack=False, align=None),
    "DOT11_DATA_RATE_MAPPING_TABLE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uDataRateMappingLength", SimTypeInt(signed=False, label="UInt32")), ("DataRateMappingEntries", SimTypeArray(SimTypeRef("DOT11_DATA_RATE_MAPPING_ENTRY", SimStruct), 126)),)), name="DOT11_DATA_RATE_MAPPING_TABLE", pack=False, align=None),
    "DOT11_DEFAULT_WEP_OFFLOAD": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("hOffloadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hOffload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dot11OffloadType", SimTypeInt(signed=False, label="DOT11_OFFLOAD_TYPE")), ("dwAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("dot11KeyDirection", SimTypeInt(signed=False, label="DOT11_KEY_DIRECTION")), ("ucMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uNumOfRWsOnMe", SimTypeInt(signed=False, label="UInt32")), ("dot11IV48Counters", SimTypeArray(SimTypeRef("DOT11_IV48_COUNTER", SimStruct), 16)), ("usDot11RWBitMaps", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)), ("usKeyLength", SimTypeShort(signed=False, label="UInt16")), ("ucKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_DEFAULT_WEP_OFFLOAD", pack=False, align=None),
    "DOT11_DEFAULT_WEP_UPLOAD": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("dot11OffloadType", SimTypeInt(signed=False, label="DOT11_OFFLOAD_TYPE")), ("hOffload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uNumOfRWsUsed", SimTypeInt(signed=False, label="UInt32")), ("dot11IV48Counters", SimTypeArray(SimTypeRef("DOT11_IV48_COUNTER", SimStruct), 16)), ("usDot11RWBitMaps", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)),)), name="DOT11_DEFAULT_WEP_UPLOAD", pack=False, align=None),
    "DOT11_DIRECTION": SimTypeInt(signed=False, label="DOT11_DIRECTION"),
    "DOT11_DISASSOCIATE_PEER_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("usReason", SimTypeShort(signed=False, label="UInt16")),)), name="DOT11_DISASSOCIATE_PEER_REQUEST", pack=False, align=None),
    "DOT11_DISASSOCIATION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("MacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uReason", SimTypeInt(signed=False, label="UInt32")), ("uIHVDataOffset", SimTypeInt(signed=False, label="UInt32")), ("uIHVDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_DISASSOCIATION_PARAMETERS", pack=False, align=None),
    "DOT11_DIVERSITY_SELECTION_RX": SimStruct(OrderedDict((("uAntennaListIndex", SimTypeInt(signed=False, label="UInt32")), ("bDiversitySelectionRX", SimTypeChar(label="Byte")),)), name="DOT11_DIVERSITY_SELECTION_RX", pack=False, align=None),
    "DOT11_DIVERSITY_SELECTION_RX_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11DiversitySelectionRx", SimTypePointer(SimTypeRef("DOT11_DIVERSITY_SELECTION_RX", SimStruct), offset=0)),)), name="DOT11_DIVERSITY_SELECTION_RX_LIST", pack=False, align=None),
    "DOT11_DIVERSITY_SUPPORT": SimTypeInt(signed=False, label="DOT11_DIVERSITY_SUPPORT"),
    "DOT11_DS_INFO": SimTypeInt(signed=False, label="DOT11_DS_INFO"),
    "DOT11_EAP_RESULT": SimStruct(OrderedDict((("dwFailureReasonCode", SimTypeInt(signed=False, label="UInt32")), ("pAttribArray", SimTypePointer(SimTypeRef("EAP_ATTRIBUTES", SimStruct), offset=0)),)), name="DOT11_EAP_RESULT", pack=False, align=None),
    "DOT11_ENCAP_ENTRY": SimStruct(OrderedDict((("usEtherType", SimTypeShort(signed=False, label="UInt16")), ("usEncapType", SimTypeShort(signed=False, label="UInt16")),)), name="DOT11_ENCAP_ENTRY", pack=False, align=None),
    "DOT11_ERP_PHY_ATTRIBUTES": SimStruct(OrderedDict((("HRDSSSAttributes", SimTypeRef("DOT11_HRDSSS_PHY_ATTRIBUTES", SimStruct)), ("bERPPBCCOptionImplemented", SimTypeChar(label="Byte")), ("bDSSSOFDMOptionImplemented", SimTypeChar(label="Byte")), ("bShortSlotTimeOptionImplemented", SimTypeChar(label="Byte")),)), name="DOT11_ERP_PHY_ATTRIBUTES", pack=False, align=None),
    "DOT11_EXTAP_ATTRIBUTES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uScanSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uDesiredSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uPrivacyExemptionListSize", SimTypeInt(signed=False, label="UInt32")), ("uAssociationTableSize", SimTypeInt(signed=False, label="UInt32")), ("uDefaultKeyTableSize", SimTypeInt(signed=False, label="UInt32")), ("uWEPKeyValueMaxLength", SimTypeInt(signed=False, label="UInt32")), ("bStrictlyOrderedServiceClassImplemented", SimTypeChar(label="Byte")), ("uNumSupportedCountryOrRegionStrings", SimTypeInt(signed=False, label="UInt32")), ("pSupportedCountryOrRegionStrings", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uInfraNumSupportedUcastAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pInfraSupportedUcastAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)), ("uInfraNumSupportedMcastAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pInfraSupportedMcastAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)),)), name="DOT11_EXTAP_ATTRIBUTES", pack=False, align=None),
    "DOT11_EXTSTA_ATTRIBUTES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uScanSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uDesiredBSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uDesiredSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uExcludedMacAddressListSize", SimTypeInt(signed=False, label="UInt32")), ("uPrivacyExemptionListSize", SimTypeInt(signed=False, label="UInt32")), ("uKeyMappingTableSize", SimTypeInt(signed=False, label="UInt32")), ("uDefaultKeyTableSize", SimTypeInt(signed=False, label="UInt32")), ("uWEPKeyValueMaxLength", SimTypeInt(signed=False, label="UInt32")), ("uPMKIDCacheSize", SimTypeInt(signed=False, label="UInt32")), ("uMaxNumPerSTADefaultKeyTables", SimTypeInt(signed=False, label="UInt32")), ("bStrictlyOrderedServiceClassImplemented", SimTypeChar(label="Byte")), ("ucSupportedQoSProtocolFlags", SimTypeChar(label="Byte")), ("bSafeModeImplemented", SimTypeChar(label="Byte")), ("uNumSupportedCountryOrRegionStrings", SimTypeInt(signed=False, label="UInt32")), ("pSupportedCountryOrRegionStrings", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uInfraNumSupportedUcastAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pInfraSupportedUcastAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)), ("uInfraNumSupportedMcastAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pInfraSupportedMcastAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)), ("uAdhocNumSupportedUcastAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pAdhocSupportedUcastAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)), ("uAdhocNumSupportedMcastAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pAdhocSupportedMcastAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)), ("bAutoPowerSaveMode", SimTypeChar(label="Byte")), ("uMaxNetworkOffloadListSize", SimTypeInt(signed=False, label="UInt32")), ("bMFPCapable", SimTypeChar(label="Byte")), ("uInfraNumSupportedMcastMgmtAlgoPairs", SimTypeInt(signed=False, label="UInt32")), ("pInfraSupportedMcastMgmtAlgoPairs", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)), ("bNeighborReportSupported", SimTypeChar(label="Byte")), ("bAPChannelReportSupported", SimTypeChar(label="Byte")), ("bActionFramesSupported", SimTypeChar(label="Byte")), ("bANQPQueryOffloadSupported", SimTypeChar(label="Byte")), ("bHESSIDConnectionSupported", SimTypeChar(label="Byte")),)), name="DOT11_EXTSTA_ATTRIBUTES", pack=False, align=None),
    "DOT11_EXTSTA_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uScanSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uDesiredBSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uDesiredSSIDListSize", SimTypeInt(signed=False, label="UInt32")), ("uExcludedMacAddressListSize", SimTypeInt(signed=False, label="UInt32")), ("uPrivacyExemptionListSize", SimTypeInt(signed=False, label="UInt32")), ("uKeyMappingTableSize", SimTypeInt(signed=False, label="UInt32")), ("uDefaultKeyTableSize", SimTypeInt(signed=False, label="UInt32")), ("uWEPKeyValueMaxLength", SimTypeInt(signed=False, label="UInt32")), ("uPMKIDCacheSize", SimTypeInt(signed=False, label="UInt32")), ("uMaxNumPerSTADefaultKeyTables", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_EXTSTA_CAPABILITY", pack=False, align=None),
    "DOT11_EXTSTA_RECV_CONTEXT": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uReceiveFlags", SimTypeInt(signed=False, label="UInt32")), ("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("uChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("usNumberOfMPDUsReceived", SimTypeShort(signed=False, label="UInt16")), ("lRSSI", SimTypeInt(signed=True, label="Int32")), ("ucDataRate", SimTypeChar(label="Byte")), ("uSizeMediaSpecificInfo", SimTypeInt(signed=False, label="UInt32")), ("pvMediaSpecificInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ullTimestamp", SimTypeLongLong(signed=False, label="UInt64")),)), name="DOT11_EXTSTA_RECV_CONTEXT", pack=False, align=None),
    "DOT11_EXTSTA_SEND_CONTEXT": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("usExemptionActionType", SimTypeShort(signed=False, label="UInt16")), ("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("uDelayedSleepValue", SimTypeInt(signed=False, label="UInt32")), ("pvMediaSpecificInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uSendFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_EXTSTA_SEND_CONTEXT", pack=False, align=None),
    "DOT11_FRAGMENT_DESCRIPTOR": SimStruct(OrderedDict((("uOffset", SimTypeInt(signed=False, label="UInt32")), ("uLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_FRAGMENT_DESCRIPTOR", pack=False, align=None),
    "DOT11_GO_NEGOTIATION_CONFIRMATION_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_GO_NEGOTIATION_CONFIRMATION_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_GO_NEGOTIATION_REQUEST_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_GO_NEGOTIATION_REQUEST_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_GO_NEGOTIATION_RESPONSE_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_GO_NEGOTIATION_RESPONSE_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_HOPPING_PATTERN_ENTRY": SimStruct(OrderedDict((("uHoppingPatternIndex", SimTypeInt(signed=False, label="UInt32")), ("uRandomTableFieldNumber", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_HOPPING_PATTERN_ENTRY", pack=False, align=None),
    "DOT11_HOPPING_PATTERN_ENTRY_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11HoppingPatternEntry", SimTypePointer(SimTypeRef("DOT11_HOPPING_PATTERN_ENTRY", SimStruct), offset=0)),)), name="DOT11_HOPPING_PATTERN_ENTRY_LIST", pack=False, align=None),
    "DOT11_HOP_ALGO_ADOPTED": SimTypeInt(signed=False, label="DOT11_HOP_ALGO_ADOPTED"),
    "DOT11_HRDSSS_PHY_ATTRIBUTES": SimStruct(OrderedDict((("bShortPreambleOptionImplemented", SimTypeChar(label="Byte")), ("bPBCCOptionImplemented", SimTypeChar(label="Byte")), ("bChannelAgilityPresent", SimTypeChar(label="Byte")), ("uHRCCAModeSupported", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_HRDSSS_PHY_ATTRIBUTES", pack=False, align=None),
    "DOT11_IBSS_PARAMS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("bJoinOnly", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_IBSS_PARAMS", pack=False, align=None),
    "DOT11_IHV_VERSION_INFO": SimStruct(OrderedDict((("dwVerMin", SimTypeInt(signed=False, label="UInt32")), ("dwVerMax", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_IHV_VERSION_INFO", pack=False, align=None),
    "DOT11_INCOMING_ASSOC_COMPLETION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uStatus", SimTypeInt(signed=False, label="UInt32")), ("ucErrorSource", SimTypeChar(label="Byte")), ("bReAssocReq", SimTypeChar(label="Byte")), ("bReAssocResp", SimTypeChar(label="Byte")), ("uAssocReqOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocReqSize", SimTypeInt(signed=False, label="UInt32")), ("uAssocRespOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocRespSize", SimTypeInt(signed=False, label="UInt32")), ("AuthAlgo", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("UnicastCipher", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("MulticastCipher", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("uActivePhyListOffset", SimTypeInt(signed=False, label="UInt32")), ("uActivePhyListSize", SimTypeInt(signed=False, label="UInt32")), ("uBeaconOffset", SimTypeInt(signed=False, label="UInt32")), ("uBeaconSize", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_INCOMING_ASSOC_COMPLETION_PARAMETERS", pack=False, align=None),
    "DOT11_INCOMING_ASSOC_DECISION": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bAccept", SimTypeChar(label="Byte")), ("usReasonCode", SimTypeShort(signed=False, label="UInt16")), ("uAssocResponseIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocResponseIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_INCOMING_ASSOC_DECISION", pack=False, align=None),
    "DOT11_INCOMING_ASSOC_DECISION_V2": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bAccept", SimTypeChar(label="Byte")), ("usReasonCode", SimTypeShort(signed=False, label="UInt16")), ("uAssocResponseIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocResponseIEsLength", SimTypeInt(signed=False, label="UInt32")), ("WFDStatus", SimTypeChar(label="Byte")),)), name="DOT11_INCOMING_ASSOC_DECISION_V2", pack=False, align=None),
    "DOT11_INCOMING_ASSOC_REQUEST_RECEIVED_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bReAssocReq", SimTypeChar(label="Byte")), ("uAssocReqOffset", SimTypeInt(signed=False, label="UInt32")), ("uAssocReqSize", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_INCOMING_ASSOC_REQUEST_RECEIVED_PARAMETERS", pack=False, align=None),
    "DOT11_INCOMING_ASSOC_STARTED_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_INCOMING_ASSOC_STARTED_PARAMETERS", pack=False, align=None),
    "DOT11_INVITATION_REQUEST_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ReceiverAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_INVITATION_REQUEST_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_INVITATION_RESPONSE_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ReceiverDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_INVITATION_RESPONSE_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_IV48_COUNTER": SimStruct(OrderedDict((("uIV32Counter", SimTypeInt(signed=False, label="UInt32")), ("usIV16Counter", SimTypeShort(signed=False, label="UInt16")),)), name="DOT11_IV48_COUNTER", pack=False, align=None),
    "DOT11_JOIN_REQUEST": SimStruct(OrderedDict((("uJoinFailureTimeout", SimTypeInt(signed=False, label="UInt32")), ("OperationalRateSet", SimTypeRef("DOT11_RATE_SET", SimStruct)), ("uChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("dot11BSSDescription", SimTypeRef("DOT11_BSS_DESCRIPTION", SimStruct)),)), name="DOT11_JOIN_REQUEST", pack=False, align=None),
    "DOT11_KEY_ALGO_BIP": SimStruct(OrderedDict((("ucIPN", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ulBIPKeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucBIPKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_KEY_ALGO_BIP", pack=False, align=None),
    "DOT11_KEY_ALGO_BIP_GMAC_256": SimStruct(OrderedDict((("ucIPN", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ulBIPGmac256KeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucBIPGmac256Key", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_KEY_ALGO_BIP_GMAC_256", pack=False, align=None),
    "DOT11_KEY_ALGO_CCMP": SimStruct(OrderedDict((("ucIV48Counter", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ulCCMPKeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucCCMPKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_KEY_ALGO_CCMP", pack=False, align=None),
    "DOT11_KEY_ALGO_GCMP": SimStruct(OrderedDict((("ucIV48Counter", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ulGCMPKeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucGCMPKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_KEY_ALGO_GCMP", pack=False, align=None),
    "DOT11_KEY_ALGO_GCMP_256": SimStruct(OrderedDict((("ucIV48Counter", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ulGCMP256KeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucGCMP256Key", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_KEY_ALGO_GCMP_256", pack=False, align=None),
    "DOT11_KEY_ALGO_TKIP_MIC": SimStruct(OrderedDict((("ucIV48Counter", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ulTKIPKeyLength", SimTypeInt(signed=False, label="UInt32")), ("ulMICKeyLength", SimTypeInt(signed=False, label="UInt32")), ("ucTKIPMICKeys", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_KEY_ALGO_TKIP_MIC", pack=False, align=None),
    "DOT11_KEY_DIRECTION": SimTypeInt(signed=False, label="DOT11_KEY_DIRECTION"),
    "DOT11_LINK_QUALITY_ENTRY": SimStruct(OrderedDict((("PeerMacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ucLinkQuality", SimTypeChar(label="Byte")),)), name="DOT11_LINK_QUALITY_ENTRY", pack=False, align=None),
    "DOT11_LINK_QUALITY_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uLinkQualityListSize", SimTypeInt(signed=False, label="UInt32")), ("uLinkQualityListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_LINK_QUALITY_PARAMETERS", pack=False, align=None),
    "DOT11_MAC_ADDRESS_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("MacAddrs", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_MAC_ADDRESS_LIST", pack=False, align=None),
    "DOT11_MAC_FRAME_STATISTICS": SimStruct(OrderedDict((("ullTransmittedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTransmittedFailureFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFailureFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullWEPExcludedCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPLocalMICFailures", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPReplays", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPICVErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullCCMPReplays", SimTypeLongLong(signed=False, label="UInt64")), ("ullCCMPDecryptErrors", SimTypeLongLong(signed=False, label="UInt64")), ("ullWEPUndecryptableCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullWEPICVErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullDecryptSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullDecryptFailureCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="DOT11_MAC_FRAME_STATISTICS", pack=False, align=None),
    "DOT11_MAC_INFO": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("uNdisPortNumber", SimTypeInt(signed=False, label="UInt32")), ("MacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_MAC_INFO", pack=False, align=None),
    "DOT11_MAC_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uOpmodeMask", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_MAC_PARAMETERS", pack=False, align=None),
    "DOT11_MANUFACTURING_CALLBACK_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("dot11ManufacturingCallbackType", SimTypeInt(signed=False, label="DOT11_MANUFACTURING_CALLBACK_TYPE")), ("uStatus", SimTypeInt(signed=False, label="UInt32")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DOT11_MANUFACTURING_CALLBACK_PARAMETERS", pack=False, align=None),
    "DOT11_MANUFACTURING_CALLBACK_TYPE": SimTypeInt(signed=False, label="DOT11_MANUFACTURING_CALLBACK_TYPE"),
    "DOT11_MANUFACTURING_FUNCTIONAL_TEST_QUERY_ADC": SimStruct(OrderedDict((("Dot11Band", SimTypeInt(signed=False, label="DOT11_BAND")), ("uChannel", SimTypeInt(signed=False, label="UInt32")), ("ADCPowerLevel", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_MANUFACTURING_FUNCTIONAL_TEST_QUERY_ADC", pack=False, align=None),
    "DOT11_MANUFACTURING_FUNCTIONAL_TEST_RX": SimStruct(OrderedDict((("bEnabled", SimTypeChar(label="Byte")), ("Dot11Band", SimTypeInt(signed=False, label="DOT11_BAND")), ("uChannel", SimTypeInt(signed=False, label="UInt32")), ("PowerLevel", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_MANUFACTURING_FUNCTIONAL_TEST_RX", pack=False, align=None),
    "DOT11_MANUFACTURING_FUNCTIONAL_TEST_TX": SimStruct(OrderedDict((("bEnable", SimTypeChar(label="Byte")), ("bOpenLoop", SimTypeChar(label="Byte")), ("Dot11Band", SimTypeInt(signed=False, label="DOT11_BAND")), ("uChannel", SimTypeInt(signed=False, label="UInt32")), ("uSetPowerLevel", SimTypeInt(signed=False, label="UInt32")), ("ADCPowerLevel", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_MANUFACTURING_FUNCTIONAL_TEST_TX", pack=False, align=None),
    "DOT11_MANUFACTURING_SELF_TEST_QUERY_RESULTS": SimStruct(OrderedDict((("SelfTestType", SimTypeInt(signed=False, label="DOT11_MANUFACTURING_SELF_TEST_TYPE")), ("uTestID", SimTypeInt(signed=False, label="UInt32")), ("bResult", SimTypeChar(label="Byte")), ("uPinFailedBitMask", SimTypeInt(signed=False, label="UInt32")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uBytesWrittenOut", SimTypeInt(signed=False, label="UInt32")), ("ucBufferOut", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_MANUFACTURING_SELF_TEST_QUERY_RESULTS", pack=False, align=None),
    "DOT11_MANUFACTURING_SELF_TEST_SET_PARAMS": SimStruct(OrderedDict((("SelfTestType", SimTypeInt(signed=False, label="DOT11_MANUFACTURING_SELF_TEST_TYPE")), ("uTestID", SimTypeInt(signed=False, label="UInt32")), ("uPinBitMask", SimTypeInt(signed=False, label="UInt32")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBufferIn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_MANUFACTURING_SELF_TEST_SET_PARAMS", pack=False, align=None),
    "DOT11_MANUFACTURING_SELF_TEST_TYPE": SimTypeInt(signed=False, label="DOT11_MANUFACTURING_SELF_TEST_TYPE"),
    "DOT11_MANUFACTURING_TEST": SimStruct(OrderedDict((("dot11ManufacturingTestType", SimTypeInt(signed=False, label="DOT11_MANUFACTURING_TEST_TYPE")), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_MANUFACTURING_TEST", pack=False, align=None),
    "DOT11_MANUFACTURING_TEST_QUERY_DATA": SimStruct(OrderedDict((("uKey", SimTypeInt(signed=False, label="UInt32")), ("uOffset", SimTypeInt(signed=False, label="UInt32")), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("uBytesRead", SimTypeInt(signed=False, label="UInt32")), ("ucBufferOut", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_MANUFACTURING_TEST_QUERY_DATA", pack=False, align=None),
    "DOT11_MANUFACTURING_TEST_SET_DATA": SimStruct(OrderedDict((("uKey", SimTypeInt(signed=False, label="UInt32")), ("uOffset", SimTypeInt(signed=False, label="UInt32")), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBufferIn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_MANUFACTURING_TEST_SET_DATA", pack=False, align=None),
    "DOT11_MANUFACTURING_TEST_SLEEP": SimStruct(OrderedDict((("uSleepTime", SimTypeInt(signed=False, label="UInt32")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DOT11_MANUFACTURING_TEST_SLEEP", pack=False, align=None),
    "DOT11_MANUFACTURING_TEST_TYPE": SimTypeInt(signed=False, label="DOT11_MANUFACTURING_TEST_TYPE"),
    "DOT11_MD_CAPABILITY_ENTRY_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11MDCapabilityEntry", SimTypePointer(SimTypeRef("DOT11_MULTI_DOMAIN_CAPABILITY_ENTRY", SimStruct), offset=0)),)), name="DOT11_MD_CAPABILITY_ENTRY_LIST", pack=False, align=None),
    "DOT11_MPDU_MAX_LENGTH_INDICATION": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("uMPDUMaxLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_MPDU_MAX_LENGTH_INDICATION", pack=False, align=None),
    "DOT11_MSONEX_RESULT": SimTypeInt(signed=False, label="DOT11_MSONEX_RESULT"),
    "DOT11_MSONEX_RESULT_PARAMS": SimStruct(OrderedDict((("Dot11OnexAuthStatus", SimTypeInt(signed=False, label="ONEX_AUTH_STATUS")), ("Dot11OneXReasonCode", SimTypeInt(signed=False, label="ONEX_REASON_CODE")), ("pbMPPESendKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwMPPESendKeyLen", SimTypeInt(signed=False, label="UInt32")), ("pbMPPERecvKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwMPPERecvKeyLen", SimTypeInt(signed=False, label="UInt32")), ("pDot11EapResult", SimTypePointer(SimTypeRef("DOT11_EAP_RESULT", SimStruct), offset=0)),)), name="DOT11_MSONEX_RESULT_PARAMS", pack=False, align=None),
    "DOT11_MSSECURITY_SETTINGS": SimStruct(OrderedDict((("dot11AuthAlgorithm", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("dot11CipherAlgorithm", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("fOneXEnabled", SimTypeInt(signed=True, label="Int32")), ("eapMethodType", SimTypeRef("EAP_METHOD_TYPE", SimStruct)), ("dwEapConnectionDataLen", SimTypeInt(signed=False, label="UInt32")), ("pEapConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_MSSECURITY_SETTINGS", pack=False, align=None),
    "DOT11_MULTI_DOMAIN_CAPABILITY_ENTRY": SimStruct(OrderedDict((("uMultiDomainCapabilityIndex", SimTypeInt(signed=False, label="UInt32")), ("uFirstChannelNumber", SimTypeInt(signed=False, label="UInt32")), ("uNumberOfChannels", SimTypeInt(signed=False, label="UInt32")), ("lMaximumTransmitPowerLevel", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_MULTI_DOMAIN_CAPABILITY_ENTRY", pack=False, align=None),
    "DOT11_NETWORK": SimStruct(OrderedDict((("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")),)), name="DOT11_NETWORK", pack=False, align=None),
    "DOT11_NETWORK_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("Network", SimTypePointer(SimTypeRef("DOT11_NETWORK", SimStruct), offset=0)),)), name="DOT11_NETWORK_LIST", pack=False, align=None),
    "DOT11_NIC_SPECIFIC_EXTENSION": SimStruct(OrderedDict((("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("uTotalBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_NIC_SPECIFIC_EXTENSION", pack=False, align=None),
    "DOT11_OFDM_PHY_ATTRIBUTES": SimStruct(OrderedDict((("uFrequencyBandsSupported", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_OFDM_PHY_ATTRIBUTES", pack=False, align=None),
    "DOT11_OFFLOAD_CAPABILITY": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("uSupportedWEPAlgorithms", SimTypeInt(signed=False, label="UInt32")), ("uNumOfReplayWindows", SimTypeInt(signed=False, label="UInt32")), ("uMaxWEPKeyMappingLength", SimTypeInt(signed=False, label="UInt32")), ("uSupportedAuthAlgorithms", SimTypeInt(signed=False, label="UInt32")), ("uMaxAuthKeyMappingLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_OFFLOAD_CAPABILITY", pack=False, align=None),
    "DOT11_OFFLOAD_NETWORK": SimStruct(OrderedDict((("Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("UnicastCipher", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("AuthAlgo", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("Dot11ChannelHints", SimTypeArray(SimTypeRef("DOT11_CHANNEL_HINT", SimStruct), 4)),)), name="DOT11_OFFLOAD_NETWORK", pack=False, align=None),
    "DOT11_OFFLOAD_NETWORK_LIST_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("FastScanPeriod", SimTypeInt(signed=False, label="UInt32")), ("FastScanIterations", SimTypeInt(signed=False, label="UInt32")), ("SlowScanPeriod", SimTypeInt(signed=False, label="UInt32")), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("offloadNetworkList", SimTypePointer(SimTypeRef("DOT11_OFFLOAD_NETWORK", SimStruct), offset=0)),)), name="DOT11_OFFLOAD_NETWORK_LIST_INFO", pack=False, align=None),
    "DOT11_OFFLOAD_NETWORK_STATUS_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_OFFLOAD_NETWORK_STATUS_PARAMETERS", pack=False, align=None),
    "DOT11_OFFLOAD_TYPE": SimTypeInt(signed=False, label="DOT11_OFFLOAD_TYPE"),
    "DOT11_OI": SimStruct(OrderedDict((("OILength", SimTypeShort(signed=False, label="UInt16")), ("OI", SimTypeArray(SimTypeChar(label="Byte"), 5)),)), name="DOT11_OI", pack=False, align=None),
    "DOT11_OPERATION_MODE_CAPABILITY": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("uMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("uMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("uNumOfTXBuffers", SimTypeInt(signed=False, label="UInt32")), ("uNumOfRXBuffers", SimTypeInt(signed=False, label="UInt32")), ("uOpModeCapability", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_OPERATION_MODE_CAPABILITY", pack=False, align=None),
    "DOT11_OPTIONAL_CAPABILITY": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("bDot11PCF", SimTypeChar(label="Byte")), ("bDot11PCFMPDUTransferToPC", SimTypeChar(label="Byte")), ("bStrictlyOrderedServiceClass", SimTypeChar(label="Byte")),)), name="DOT11_OPTIONAL_CAPABILITY", pack=False, align=None),
    "DOT11_PEER_INFO": SimStruct(OrderedDict((("MacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("usCapabilityInformation", SimTypeShort(signed=False, label="UInt16")), ("AuthAlgo", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("UnicastCipherAlgo", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("MulticastCipherAlgo", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("bWpsEnabled", SimTypeChar(label="Byte")), ("usListenInterval", SimTypeShort(signed=False, label="UInt16")), ("ucSupportedRates", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("usAssociationID", SimTypeShort(signed=False, label="UInt16")), ("AssociationState", SimTypeInt(signed=False, label="DOT11_ASSOCIATION_STATE")), ("PowerMode", SimTypeInt(signed=False, label="DOT11_POWER_MODE")), ("liAssociationUpTime", SimTypeLongLong(signed=True, label="Int64")), ("Statistics", SimTypeRef("DOT11_PEER_STATISTICS", SimStruct)),)), name="DOT11_PEER_INFO", pack=False, align=None),
    "DOT11_PEER_INFO_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("PeerInfo", SimTypePointer(SimTypeRef("DOT11_PEER_INFO", SimStruct), offset=0)),)), name="DOT11_PEER_INFO_LIST", pack=False, align=None),
    "DOT11_PEER_STATISTICS": SimStruct(OrderedDict((("ullDecryptSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullDecryptFailureCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTxPacketSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTxPacketFailureCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRxPacketSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRxPacketFailureCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="DOT11_PEER_STATISTICS", pack=False, align=None),
    "DOT11_PER_MSDU_COUNTERS": SimStruct(OrderedDict((("uTransmittedFragmentCount", SimTypeInt(signed=False, label="UInt32")), ("uRetryCount", SimTypeInt(signed=False, label="UInt32")), ("uRTSSuccessCount", SimTypeInt(signed=False, label="UInt32")), ("uRTSFailureCount", SimTypeInt(signed=False, label="UInt32")), ("uACKFailureCount", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_PER_MSDU_COUNTERS", pack=False, align=None),
    "DOT11_PHY_ATTRIBUTES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("bHardwarePhyState", SimTypeChar(label="Byte")), ("bSoftwarePhyState", SimTypeChar(label="Byte")), ("bCFPollable", SimTypeChar(label="Byte")), ("uMPDUMaxLength", SimTypeInt(signed=False, label="UInt32")), ("TempType", SimTypeInt(signed=False, label="DOT11_TEMP_TYPE")), ("DiversitySupport", SimTypeInt(signed=False, label="DOT11_DIVERSITY_SUPPORT")), ("PhySpecificAttributes", SimUnion({"HRDSSSAttributes": SimTypeRef("DOT11_HRDSSS_PHY_ATTRIBUTES", SimStruct), "OFDMAttributes": SimTypeRef("DOT11_OFDM_PHY_ATTRIBUTES", SimStruct), "ERPAttributes": SimTypeRef("DOT11_ERP_PHY_ATTRIBUTES", SimStruct)}, name="<anon>", label="None")), ("uNumberSupportedPowerLevels", SimTypeInt(signed=False, label="UInt32")), ("TxPowerLevels", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)), ("uNumDataRateMappingEntries", SimTypeInt(signed=False, label="UInt32")), ("DataRateMappingEntries", SimTypeArray(SimTypeRef("DOT11_DATA_RATE_MAPPING_ENTRY", SimStruct), 126)), ("SupportedDataRatesValue", SimTypeRef("DOT11_SUPPORTED_DATA_RATES_VALUE_V2", SimStruct)),)), name="DOT11_PHY_ATTRIBUTES", pack=False, align=None),
    "DOT11_PHY_FRAME_STATISTICS": SimStruct(OrderedDict((("ullTransmittedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMulticastTransmittedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullFailedCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRetryCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMultipleRetryCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMaxTXLifetimeExceededCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTransmittedFragmentCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRTSSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRTSFailureCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullACKFailureCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMulticastReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullPromiscuousReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMaxRXLifetimeExceededCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullFrameDuplicateCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFragmentCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullPromiscuousReceivedFragmentCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullFCSErrorCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="DOT11_PHY_FRAME_STATISTICS", pack=False, align=None),
    "DOT11_PHY_FREQUENCY_ADOPTED_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ulPhyId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ulChannel": SimTypeInt(signed=False, label="UInt32"), "ulFrequency": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DOT11_PHY_FREQUENCY_ADOPTED_PARAMETERS", pack=False, align=None),
    "DOT11_PHY_ID_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11PhyId", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DOT11_PHY_ID_LIST", pack=False, align=None),
    "DOT11_PHY_STATE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("bHardwarePhyState", SimTypeChar(label="Byte")), ("bSoftwarePhyState", SimTypeChar(label="Byte")),)), name="DOT11_PHY_STATE_PARAMETERS", pack=False, align=None),
    "DOT11_PHY_TYPE": SimTypeInt(signed=False, label="DOT11_PHY_TYPE"),
    "DOT11_PHY_TYPE_INFO": SimStruct(OrderedDict((("dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("bUseParameters", SimTypeChar(label="Byte")), ("uProbeDelay", SimTypeInt(signed=False, label="UInt32")), ("uMinChannelTime", SimTypeInt(signed=False, label="UInt32")), ("uMaxChannelTime", SimTypeInt(signed=False, label="UInt32")), ("ChDescriptionType", SimTypeInt(signed=False, label="CH_DESCRIPTION_TYPE")), ("uChannelListSize", SimTypeInt(signed=False, label="UInt32")), ("ucChannelListBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_PHY_TYPE_INFO", pack=False, align=None),
    "DOT11_PHY_TYPE_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11PhyType", SimTypePointer(SimTypeInt(signed=False, label="DOT11_PHY_TYPE"), offset=0)),)), name="DOT11_PHY_TYPE_LIST", pack=False, align=None),
    "DOT11_PMKID_CANDIDATE_LIST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uCandidateListSize", SimTypeInt(signed=False, label="UInt32")), ("uCandidateListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_PMKID_CANDIDATE_LIST_PARAMETERS", pack=False, align=None),
    "DOT11_PMKID_ENTRY": SimStruct(OrderedDict((("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("PMKID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_PMKID_ENTRY", pack=False, align=None),
    "DOT11_PMKID_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("PMKIDs", SimTypePointer(SimTypeRef("DOT11_PMKID_ENTRY", SimStruct), offset=0)),)), name="DOT11_PMKID_LIST", pack=False, align=None),
    "DOT11_PORT_STATE": SimStruct(OrderedDict((("PeerMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uSessionId", SimTypeInt(signed=False, label="UInt32")), ("bPortControlled", SimTypeInt(signed=True, label="Int32")), ("bPortAuthorized", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_PORT_STATE", pack=False, align=None),
    "DOT11_PORT_STATE_NOTIFICATION": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerMac", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bOpen", SimTypeChar(label="Byte")),)), name="DOT11_PORT_STATE_NOTIFICATION", pack=False, align=None),
    "DOT11_POWER_MGMT_AUTO_MODE_ENABLED_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("bEnabled", SimTypeChar(label="Byte")),)), name="DOT11_POWER_MGMT_AUTO_MODE_ENABLED_INFO", pack=False, align=None),
    "DOT11_POWER_MGMT_MODE": SimStruct(OrderedDict((("dot11PowerMode", SimTypeInt(signed=False, label="DOT11_POWER_MODE")), ("uPowerSaveLevel", SimTypeInt(signed=False, label="UInt32")), ("usListenInterval", SimTypeShort(signed=False, label="UInt16")), ("usAID", SimTypeShort(signed=False, label="UInt16")), ("bReceiveDTIMs", SimTypeChar(label="Byte")),)), name="DOT11_POWER_MGMT_MODE", pack=False, align=None),
    "DOT11_POWER_MGMT_MODE_STATUS_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PowerSaveMode", SimTypeInt(signed=False, label="DOT11_POWER_MODE")), ("uPowerSaveLevel", SimTypeInt(signed=False, label="UInt32")), ("Reason", SimTypeInt(signed=False, label="DOT11_POWER_MODE_REASON")),)), name="DOT11_POWER_MGMT_MODE_STATUS_INFO", pack=False, align=None),
    "DOT11_POWER_MODE": SimTypeInt(signed=False, label="DOT11_POWER_MODE"),
    "DOT11_POWER_MODE_REASON": SimTypeInt(signed=False, label="DOT11_POWER_MODE_REASON"),
    "DOT11_PRIVACY_EXEMPTION": SimStruct(OrderedDict((("usEtherType", SimTypeShort(signed=False, label="UInt16")), ("usExemptionActionType", SimTypeShort(signed=False, label="UInt16")), ("usExemptionPacketType", SimTypeShort(signed=False, label="UInt16")),)), name="DOT11_PRIVACY_EXEMPTION", pack=False, align=None),
    "DOT11_PRIVACY_EXEMPTION_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("PrivacyExemptionEntries", SimTypePointer(SimTypeRef("DOT11_PRIVACY_EXEMPTION", SimStruct), offset=0)),)), name="DOT11_PRIVACY_EXEMPTION_LIST", pack=False, align=None),
    "DOT11_PROVISION_DISCOVERY_REQUEST_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ReceiverAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_PROVISION_DISCOVERY_REQUEST_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_PROVISION_DISCOVERY_RESPONSE_SEND_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ReceiverDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_PROVISION_DISCOVERY_RESPONSE_SEND_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_QOS_PARAMS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ucEnabledQoSProtocolFlags", SimTypeChar(label="Byte")),)), name="DOT11_QOS_PARAMS", pack=False, align=None),
    "DOT11_QOS_TX_DURATION": SimStruct(OrderedDict((("uNominalMSDUSize", SimTypeInt(signed=False, label="UInt32")), ("uMinPHYRate", SimTypeInt(signed=False, label="UInt32")), ("uDuration", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_QOS_TX_DURATION", pack=False, align=None),
    "DOT11_QOS_TX_MEDIUM_TIME": SimStruct(OrderedDict((("dot11PeerAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ucQoSPriority", SimTypeChar(label="Byte")), ("uMediumTimeAdmited", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_QOS_TX_MEDIUM_TIME", pack=False, align=None),
    "DOT11_RADIO_STATE": SimTypeInt(signed=False, label="DOT11_RADIO_STATE"),
    "DOT11_RATE_SET": SimStruct(OrderedDict((("uRateSetLength", SimTypeInt(signed=False, label="UInt32")), ("ucRateSet", SimTypeArray(SimTypeChar(label="Byte"), 126)),)), name="DOT11_RATE_SET", pack=False, align=None),
    "DOT11_RECEIVED_GO_NEGOTIATION_CONFIRMATION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_GO_NEGOTIATION_CONFIRMATION_PARAMETERS", pack=False, align=None),
    "DOT11_RECEIVED_GO_NEGOTIATION_REQUEST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("RequestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_GO_NEGOTIATION_REQUEST_PARAMETERS", pack=False, align=None),
    "DOT11_RECEIVED_GO_NEGOTIATION_RESPONSE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("ResponseContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_GO_NEGOTIATION_RESPONSE_PARAMETERS", pack=False, align=None),
    "DOT11_RECEIVED_INVITATION_REQUEST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("TransmitterDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("RequestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_INVITATION_REQUEST_PARAMETERS", pack=False, align=None),
    "DOT11_RECEIVED_INVITATION_RESPONSE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("TransmitterDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_INVITATION_RESPONSE_PARAMETERS", pack=False, align=None),
    "DOT11_RECEIVED_PROVISION_DISCOVERY_REQUEST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("TransmitterDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("RequestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_PROVISION_DISCOVERY_REQUEST_PARAMETERS", pack=False, align=None),
    "DOT11_RECEIVED_PROVISION_DISCOVERY_RESPONSE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("TransmitterDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RECEIVED_PROVISION_DISCOVERY_RESPONSE_PARAMETERS", pack=False, align=None),
    "DOT11_RECV_EXTENSION_INFO": SimStruct(OrderedDict((("uVersion", SimTypeInt(signed=False, label="UInt32")), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("uChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("lRSSI", SimTypeInt(signed=True, label="Int32")), ("lRSSIMin", SimTypeInt(signed=True, label="Int32")), ("lRSSIMax", SimTypeInt(signed=True, label="Int32")), ("uRSSI", SimTypeInt(signed=False, label="UInt32")), ("ucPriority", SimTypeChar(label="Byte")), ("ucDataRate", SimTypeChar(label="Byte")), ("ucPeerMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dwExtendedStatus", SimTypeInt(signed=False, label="UInt32")), ("hWEPOffloadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAuthOffloadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("usWEPAppliedMask", SimTypeShort(signed=False, label="UInt16")), ("usWPAMSDUPriority", SimTypeShort(signed=False, label="UInt16")), ("dot11LowestIV48Counter", SimTypeRef("DOT11_IV48_COUNTER", SimStruct)), ("usDot11LeftRWBitMap", SimTypeShort(signed=False, label="UInt16")), ("dot11HighestIV48Counter", SimTypeRef("DOT11_IV48_COUNTER", SimStruct)), ("usDot11RightRWBitMap", SimTypeShort(signed=False, label="UInt16")), ("usNumberOfMPDUsReceived", SimTypeShort(signed=False, label="UInt16")), ("usNumberOfFragments", SimTypeShort(signed=False, label="UInt16")), ("pNdisPackets", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="DOT11_RECV_EXTENSION_INFO", pack=False, align=None),
    "DOT11_RECV_EXTENSION_INFO_V2": SimStruct(OrderedDict((("uVersion", SimTypeInt(signed=False, label="UInt32")), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("uChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("lRSSI", SimTypeInt(signed=True, label="Int32")), ("uRSSI", SimTypeInt(signed=False, label="UInt32")), ("ucPriority", SimTypeChar(label="Byte")), ("ucDataRate", SimTypeChar(label="Byte")), ("ucPeerMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dwExtendedStatus", SimTypeInt(signed=False, label="UInt32")), ("hWEPOffloadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAuthOffloadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("usWEPAppliedMask", SimTypeShort(signed=False, label="UInt16")), ("usWPAMSDUPriority", SimTypeShort(signed=False, label="UInt16")), ("dot11LowestIV48Counter", SimTypeRef("DOT11_IV48_COUNTER", SimStruct)), ("usDot11LeftRWBitMap", SimTypeShort(signed=False, label="UInt16")), ("dot11HighestIV48Counter", SimTypeRef("DOT11_IV48_COUNTER", SimStruct)), ("usDot11RightRWBitMap", SimTypeShort(signed=False, label="UInt16")), ("usNumberOfMPDUsReceived", SimTypeShort(signed=False, label="UInt16")), ("usNumberOfFragments", SimTypeShort(signed=False, label="UInt16")), ("pNdisPackets", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="DOT11_RECV_EXTENSION_INFO_V2", pack=False, align=None),
    "DOT11_RECV_SENSITIVITY": SimStruct(OrderedDict((("ucDataRate", SimTypeChar(label="Byte")), ("lRSSIMin", SimTypeInt(signed=True, label="Int32")), ("lRSSIMax", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_RECV_SENSITIVITY", pack=False, align=None),
    "DOT11_RECV_SENSITIVITY_LIST": SimStruct(OrderedDict((("Anonymous", SimUnion({"dot11PhyType": SimTypeInt(signed=False, label="DOT11_PHY_TYPE"), "uPhyId": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11RecvSensitivity", SimTypePointer(SimTypeRef("DOT11_RECV_SENSITIVITY", SimStruct), offset=0)),)), name="DOT11_RECV_SENSITIVITY_LIST", pack=False, align=None),
    "DOT11_REG_DOMAINS_SUPPORT_VALUE": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11RegDomainValue", SimTypePointer(SimTypeRef("DOT11_REG_DOMAIN_VALUE", SimStruct), offset=0)),)), name="DOT11_REG_DOMAINS_SUPPORT_VALUE", pack=False, align=None),
    "DOT11_REG_DOMAIN_VALUE": SimStruct(OrderedDict((("uRegDomainsSupportIndex", SimTypeInt(signed=False, label="UInt32")), ("uRegDomainsSupportValue", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_REG_DOMAIN_VALUE", pack=False, align=None),
    "DOT11_RESET_REQUEST": SimStruct(OrderedDict((("dot11ResetType", SimTypeInt(signed=False, label="DOT11_RESET_TYPE")), ("dot11MacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bSetDefaultMIB", SimTypeChar(label="Byte")),)), name="DOT11_RESET_REQUEST", pack=False, align=None),
    "DOT11_RESET_TYPE": SimTypeInt(signed=False, label="DOT11_RESET_TYPE"),
    "DOT11_ROAMING_COMPLETION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uStatus", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ROAMING_COMPLETION_PARAMETERS", pack=False, align=None),
    "DOT11_ROAMING_START_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("AdhocBSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("AdhocSSID", SimTypeRef("DOT11_SSID", SimStruct)), ("uRoamingReason", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_ROAMING_START_PARAMETERS", pack=False, align=None),
    "DOT11_RSSI_RANGE": SimStruct(OrderedDict((("dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("uRSSIMin", SimTypeInt(signed=False, label="UInt32")), ("uRSSIMax", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_RSSI_RANGE", pack=False, align=None),
    "DOT11_SCAN_REQUEST": SimStruct(OrderedDict((("dot11BSSType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dot11BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11SSID", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11ScanType", SimTypeInt(signed=False, label="DOT11_SCAN_TYPE")), ("bRestrictedScan", SimTypeChar(label="Byte")), ("bUseRequestIE", SimTypeChar(label="Byte")), ("uRequestIDsOffset", SimTypeInt(signed=False, label="UInt32")), ("uNumOfRequestIDs", SimTypeInt(signed=False, label="UInt32")), ("uPhyTypesOffset", SimTypeInt(signed=False, label="UInt32")), ("uNumOfPhyTypes", SimTypeInt(signed=False, label="UInt32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_SCAN_REQUEST", pack=False, align=None),
    "DOT11_SCAN_REQUEST_V2": SimStruct(OrderedDict((("dot11BSSType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dot11BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11ScanType", SimTypeInt(signed=False, label="DOT11_SCAN_TYPE")), ("bRestrictedScan", SimTypeChar(label="Byte")), ("udot11SSIDsOffset", SimTypeInt(signed=False, label="UInt32")), ("uNumOfdot11SSIDs", SimTypeInt(signed=False, label="UInt32")), ("bUseRequestIE", SimTypeChar(label="Byte")), ("uRequestIDsOffset", SimTypeInt(signed=False, label="UInt32")), ("uNumOfRequestIDs", SimTypeInt(signed=False, label="UInt32")), ("uPhyTypeInfosOffset", SimTypeInt(signed=False, label="UInt32")), ("uNumOfPhyTypeInfos", SimTypeInt(signed=False, label="UInt32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_SCAN_REQUEST_V2", pack=False, align=None),
    "DOT11_SCAN_TYPE": SimTypeInt(signed=False, label="DOT11_SCAN_TYPE"),
    "DOT11_SECURITY_PACKET_HEADER": SimStruct(OrderedDict((("PeerMac", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("usEtherType", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_SECURITY_PACKET_HEADER", pack=False, align=None),
    "DOT11_SEND_GO_NEGOTIATION_CONFIRMATION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("ResponseContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeChar(label="Byte")), ("GroupCapability", SimTypeChar(label="Byte")), ("GroupID", SimTypeRef("DOT11_WFD_GROUP_ID", SimStruct)), ("bUseGroupID", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_GO_NEGOTIATION_CONFIRMATION_PARAMETERS", pack=False, align=None),
    "DOT11_SEND_GO_NEGOTIATION_REQUEST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("GroupOwnerIntent", SimTypeRef("DOT11_WFD_GO_INTENT", SimStruct)), ("MinimumConfigTimeout", SimTypeRef("DOT11_WFD_CONFIGURATION_TIMEOUT", SimStruct)), ("IntendedInterfaceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("GroupCapability", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_GO_NEGOTIATION_REQUEST_PARAMETERS", pack=False, align=None),
    "DOT11_SEND_GO_NEGOTIATION_RESPONSE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("RequestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeChar(label="Byte")), ("GroupOwnerIntent", SimTypeRef("DOT11_WFD_GO_INTENT", SimStruct)), ("MinimumConfigTimeout", SimTypeRef("DOT11_WFD_CONFIGURATION_TIMEOUT", SimStruct)), ("IntendedInterfaceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("GroupCapability", SimTypeChar(label="Byte")), ("GroupID", SimTypeRef("DOT11_WFD_GROUP_ID", SimStruct)), ("bUseGroupID", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_GO_NEGOTIATION_RESPONSE_PARAMETERS", pack=False, align=None),
    "DOT11_SEND_INVITATION_REQUEST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("DialogToken", SimTypeChar(label="Byte")), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("MinimumConfigTimeout", SimTypeRef("DOT11_WFD_CONFIGURATION_TIMEOUT", SimStruct)), ("InvitationFlags", SimTypeRef("DOT11_WFD_INVITATION_FLAGS", SimStruct)), ("GroupBSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bUseGroupBSSID", SimTypeChar(label="Byte")), ("OperatingChannel", SimTypeRef("DOT11_WFD_CHANNEL", SimStruct)), ("bUseSpecifiedOperatingChannel", SimTypeChar(label="Byte")), ("GroupID", SimTypeRef("DOT11_WFD_GROUP_ID", SimStruct)), ("bLocalGO", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_INVITATION_REQUEST_PARAMETERS", pack=False, align=None),
    "DOT11_SEND_INVITATION_RESPONSE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ReceiverDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("RequestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeChar(label="Byte")), ("MinimumConfigTimeout", SimTypeRef("DOT11_WFD_CONFIGURATION_TIMEOUT", SimStruct)), ("GroupBSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bUseGroupBSSID", SimTypeChar(label="Byte")), ("OperatingChannel", SimTypeRef("DOT11_WFD_CHANNEL", SimStruct)), ("bUseSpecifiedOperatingChannel", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_INVITATION_RESPONSE_PARAMETERS", pack=False, align=None),
    "DOT11_SEND_PROVISION_DISCOVERY_REQUEST_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("DialogToken", SimTypeChar(label="Byte")), ("PeerDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("GroupCapability", SimTypeChar(label="Byte")), ("GroupID", SimTypeRef("DOT11_WFD_GROUP_ID", SimStruct)), ("bUseGroupID", SimTypeChar(label="Byte")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_PROVISION_DISCOVERY_REQUEST_PARAMETERS", pack=False, align=None),
    "DOT11_SEND_PROVISION_DISCOVERY_RESPONSE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ReceiverDeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("DialogToken", SimTypeChar(label="Byte")), ("RequestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("uSendTimeout", SimTypeInt(signed=False, label="UInt32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SEND_PROVISION_DISCOVERY_RESPONSE_PARAMETERS", pack=False, align=None),
    "DOT11_SSID": SimStruct(OrderedDict((("uSSIDLength", SimTypeInt(signed=False, label="UInt32")), ("ucSSID", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="DOT11_SSID", pack=False, align=None),
    "DOT11_SSID_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("SSIDs", SimTypePointer(SimTypeRef("DOT11_SSID", SimStruct), offset=0)),)), name="DOT11_SSID_LIST", pack=False, align=None),
    "DOT11_START_REQUEST": SimStruct(OrderedDict((("uStartFailureTimeout", SimTypeInt(signed=False, label="UInt32")), ("OperationalRateSet", SimTypeRef("DOT11_RATE_SET", SimStruct)), ("uChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("dot11BSSDescription", SimTypeRef("DOT11_BSS_DESCRIPTION", SimStruct)),)), name="DOT11_START_REQUEST", pack=False, align=None),
    "DOT11_STATISTICS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ullFourWayHandshakeFailures", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPCounterMeasuresInvoked", SimTypeLongLong(signed=False, label="UInt64")), ("ullReserved", SimTypeLongLong(signed=False, label="UInt64")), ("MacUcastCounters", SimTypeRef("DOT11_MAC_FRAME_STATISTICS", SimStruct)), ("MacMcastCounters", SimTypeRef("DOT11_MAC_FRAME_STATISTICS", SimStruct)), ("PhyCounters", SimTypePointer(SimTypeRef("DOT11_PHY_FRAME_STATISTICS", SimStruct), offset=0)),)), name="DOT11_STATISTICS", pack=False, align=None),
    "DOT11_STATUS_INDICATION": SimStruct(OrderedDict((("uStatusType", SimTypeInt(signed=False, label="UInt32")), ("ndisStatus", SimTypeInt(signed=True, label="Int32")),)), name="DOT11_STATUS_INDICATION", pack=False, align=None),
    "DOT11_STOP_AP_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ulReason", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_STOP_AP_PARAMETERS", pack=False, align=None),
    "DOT11_SUPPORTED_ANTENNA": SimStruct(OrderedDict((("uAntennaListIndex", SimTypeInt(signed=False, label="UInt32")), ("bSupportedAntenna", SimTypeChar(label="Byte")),)), name="DOT11_SUPPORTED_ANTENNA", pack=False, align=None),
    "DOT11_SUPPORTED_ANTENNA_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11SupportedAntenna", SimTypePointer(SimTypeRef("DOT11_SUPPORTED_ANTENNA", SimStruct), offset=0)),)), name="DOT11_SUPPORTED_ANTENNA_LIST", pack=False, align=None),
    "DOT11_SUPPORTED_DATA_RATES_VALUE": SimStruct(OrderedDict((("ucSupportedTxDataRatesValue", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("ucSupportedRxDataRatesValue", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="DOT11_SUPPORTED_DATA_RATES_VALUE", pack=False, align=None),
    "DOT11_SUPPORTED_DATA_RATES_VALUE_V2": SimStruct(OrderedDict((("ucSupportedTxDataRatesValue", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("ucSupportedRxDataRatesValue", SimTypeArray(SimTypeChar(label="Byte"), 255)),)), name="DOT11_SUPPORTED_DATA_RATES_VALUE_V2", pack=False, align=None),
    "DOT11_SUPPORTED_DSSS_CHANNEL": SimStruct(OrderedDict((("uChannel", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SUPPORTED_DSSS_CHANNEL", pack=False, align=None),
    "DOT11_SUPPORTED_DSSS_CHANNEL_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11SupportedDSSSChannel", SimTypePointer(SimTypeRef("DOT11_SUPPORTED_DSSS_CHANNEL", SimStruct), offset=0)),)), name="DOT11_SUPPORTED_DSSS_CHANNEL_LIST", pack=False, align=None),
    "DOT11_SUPPORTED_OFDM_FREQUENCY": SimStruct(OrderedDict((("uCenterFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_SUPPORTED_OFDM_FREQUENCY", pack=False, align=None),
    "DOT11_SUPPORTED_OFDM_FREQUENCY_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11SupportedOFDMFrequency", SimTypePointer(SimTypeRef("DOT11_SUPPORTED_OFDM_FREQUENCY", SimStruct), offset=0)),)), name="DOT11_SUPPORTED_OFDM_FREQUENCY_LIST", pack=False, align=None),
    "DOT11_SUPPORTED_PHY_TYPES": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11PHYType", SimTypePointer(SimTypeInt(signed=False, label="DOT11_PHY_TYPE"), offset=0)),)), name="DOT11_SUPPORTED_PHY_TYPES", pack=False, align=None),
    "DOT11_SUPPORTED_POWER_LEVELS": SimStruct(OrderedDict((("uNumOfSupportedPowerLevels", SimTypeInt(signed=False, label="UInt32")), ("uTxPowerLevelValues", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="DOT11_SUPPORTED_POWER_LEVELS", pack=False, align=None),
    "DOT11_TEMP_TYPE": SimTypeInt(signed=False, label="DOT11_TEMP_TYPE"),
    "DOT11_TKIPMIC_FAILURE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("bDefaultKeyFailure", SimTypeChar(label="Byte")), ("uKeyIndex", SimTypeInt(signed=False, label="UInt32")), ("PeerMac", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_TKIPMIC_FAILURE_PARAMETERS", pack=False, align=None),
    "DOT11_UPDATE_IE": SimStruct(OrderedDict((("dot11UpdateIEOp", SimTypeInt(signed=False, label="DOT11_UPDATE_IE_OP")), ("uBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_UPDATE_IE", pack=False, align=None),
    "DOT11_UPDATE_IE_OP": SimTypeInt(signed=False, label="DOT11_UPDATE_IE_OP"),
    "DOT11_VENUEINFO": SimStruct(OrderedDict((("VenueGroup", SimTypeChar(label="Byte")), ("VenueType", SimTypeChar(label="Byte")),)), name="DOT11_VENUEINFO", pack=False, align=None),
    "DOT11_VWIFI_ATTRIBUTES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("Combinations", SimTypePointer(SimTypeRef("DOT11_VWIFI_COMBINATION", SimStruct), offset=0)),)), name="DOT11_VWIFI_ATTRIBUTES", pack=False, align=None),
    "DOT11_VWIFI_COMBINATION": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumInfrastructure", SimTypeInt(signed=False, label="UInt32")), ("uNumAdhoc", SimTypeInt(signed=False, label="UInt32")), ("uNumSoftAP", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_VWIFI_COMBINATION", pack=False, align=None),
    "DOT11_VWIFI_COMBINATION_V2": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumInfrastructure", SimTypeInt(signed=False, label="UInt32")), ("uNumAdhoc", SimTypeInt(signed=False, label="UInt32")), ("uNumSoftAP", SimTypeInt(signed=False, label="UInt32")), ("uNumVirtualStation", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_VWIFI_COMBINATION_V2", pack=False, align=None),
    "DOT11_VWIFI_COMBINATION_V3": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumInfrastructure", SimTypeInt(signed=False, label="UInt32")), ("uNumAdhoc", SimTypeInt(signed=False, label="UInt32")), ("uNumSoftAP", SimTypeInt(signed=False, label="UInt32")), ("uNumVirtualStation", SimTypeInt(signed=False, label="UInt32")), ("uNumWFDGroup", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_VWIFI_COMBINATION_V3", pack=False, align=None),
    "DOT11_WEP_OFFLOAD": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("hOffloadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hOffload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dot11OffloadType", SimTypeInt(signed=False, label="DOT11_OFFLOAD_TYPE")), ("dwAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("bRowIsOutbound", SimTypeChar(label="Byte")), ("bUseDefault", SimTypeChar(label="Byte")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("ucMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uNumOfRWsOnPeer", SimTypeInt(signed=False, label="UInt32")), ("uNumOfRWsOnMe", SimTypeInt(signed=False, label="UInt32")), ("dot11IV48Counters", SimTypeArray(SimTypeRef("DOT11_IV48_COUNTER", SimStruct), 16)), ("usDot11RWBitMaps", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)), ("usKeyLength", SimTypeShort(signed=False, label="UInt16")), ("ucKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_WEP_OFFLOAD", pack=False, align=None),
    "DOT11_WEP_UPLOAD": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("dot11OffloadType", SimTypeInt(signed=False, label="DOT11_OFFLOAD_TYPE")), ("hOffload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uNumOfRWsUsed", SimTypeInt(signed=False, label="UInt32")), ("dot11IV48Counters", SimTypeArray(SimTypeRef("DOT11_IV48_COUNTER", SimStruct), 16)), ("usDot11RWBitMaps", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)),)), name="DOT11_WEP_UPLOAD", pack=False, align=None),
    "DOT11_WFD_ADDITIONAL_IE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uBeaconIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uBeaconIEsLength", SimTypeInt(signed=False, label="UInt32")), ("uProbeResponseIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uProbeResponseIEsLength", SimTypeInt(signed=False, label="UInt32")), ("uDefaultRequestIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uDefaultRequestIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_WFD_ADDITIONAL_IE", pack=False, align=None),
    "DOT11_WFD_ADVERTISED_SERVICE_DESCRIPTOR": SimStruct(OrderedDict((("AdvertisementID", SimTypeInt(signed=False, label="UInt32")), ("ConfigMethods", SimTypeShort(signed=False, label="UInt16")), ("ServiceNameLength", SimTypeChar(label="Byte")), ("ServiceName", SimTypeArray(SimTypeChar(label="Byte"), 255)),)), name="DOT11_WFD_ADVERTISED_SERVICE_DESCRIPTOR", pack=False, align=None),
    "DOT11_WFD_ADVERTISED_SERVICE_LIST": SimStruct(OrderedDict((("ServiceCount", SimTypeShort(signed=False, label="UInt16")), ("AdvertisedService", SimTypePointer(SimTypeRef("DOT11_WFD_ADVERTISED_SERVICE_DESCRIPTOR", SimStruct), offset=0)),)), name="DOT11_WFD_ADVERTISED_SERVICE_LIST", pack=False, align=None),
    "DOT11_WFD_ADVERTISEMENT_ID": SimStruct(OrderedDict((("AdvertisementID", SimTypeInt(signed=False, label="UInt32")), ("ServiceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_WFD_ADVERTISEMENT_ID", pack=False, align=None),
    "DOT11_WFD_ATTRIBUTES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumConcurrentGORole", SimTypeInt(signed=False, label="UInt32")), ("uNumConcurrentClientRole", SimTypeInt(signed=False, label="UInt32")), ("WPSVersionsSupported", SimTypeInt(signed=False, label="UInt32")), ("bServiceDiscoverySupported", SimTypeChar(label="Byte")), ("bClientDiscoverabilitySupported", SimTypeChar(label="Byte")), ("bInfrastructureManagementSupported", SimTypeChar(label="Byte")), ("uMaxSecondaryDeviceTypeListSize", SimTypeInt(signed=False, label="UInt32")), ("DeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("uInterfaceAddressListCount", SimTypeInt(signed=False, label="UInt32")), ("pInterfaceAddressList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uNumSupportedCountryOrRegionStrings", SimTypeInt(signed=False, label="UInt32")), ("pSupportedCountryOrRegionStrings", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uDiscoveryFilterListSize", SimTypeInt(signed=False, label="UInt32")), ("uGORoleClientTableSize", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_WFD_ATTRIBUTES", pack=False, align=None),
    "DOT11_WFD_CHANNEL": SimStruct(OrderedDict((("CountryRegionString", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("OperatingClass", SimTypeChar(label="Byte")), ("ChannelNumber", SimTypeChar(label="Byte")),)), name="DOT11_WFD_CHANNEL", pack=False, align=None),
    "DOT11_WFD_CONFIGURATION_TIMEOUT": SimStruct(OrderedDict((("GOTimeout", SimTypeChar(label="Byte")), ("ClientTimeout", SimTypeChar(label="Byte")),)), name="DOT11_WFD_CONFIGURATION_TIMEOUT", pack=False, align=None),
    "DOT11_WFD_DEVICE_CAPABILITY_CONFIG": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("bServiceDiscoveryEnabled", SimTypeChar(label="Byte")), ("bClientDiscoverabilityEnabled", SimTypeChar(label="Byte")), ("bConcurrentOperationSupported", SimTypeChar(label="Byte")), ("bInfrastructureManagementEnabled", SimTypeChar(label="Byte")), ("bDeviceLimitReached", SimTypeChar(label="Byte")), ("bInvitationProcedureEnabled", SimTypeChar(label="Byte")), ("WPSVersionsEnabled", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_WFD_DEVICE_CAPABILITY_CONFIG", pack=False, align=None),
    "DOT11_WFD_DEVICE_ENTRY": SimStruct(OrderedDict((("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("PhySpecificInfo", SimUnion({"uChCenterFrequency": SimTypeInt(signed=False, label="UInt32"), "FHSS": SimTypeRef("_FHSS_e__Struct", SimStruct)}, name="<anon>", label="None")), ("dot11BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11BSSType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("TransmitterAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("lRSSI", SimTypeInt(signed=True, label="Int32")), ("uLinkQuality", SimTypeInt(signed=False, label="UInt32")), ("usBeaconPeriod", SimTypeShort(signed=False, label="UInt16")), ("ullTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("ullBeaconHostTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("ullProbeResponseHostTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("usCapabilityInformation", SimTypeShort(signed=False, label="UInt16")), ("uBeaconIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uBeaconIEsLength", SimTypeInt(signed=False, label="UInt32")), ("uProbeResponseIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uProbeResponseIEsLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_WFD_DEVICE_ENTRY", pack=False, align=None),
    "DOT11_WFD_DEVICE_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("DeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ConfigMethods", SimTypeShort(signed=False, label="UInt16")), ("PrimaryDeviceType", SimTypeRef("DOT11_WFD_DEVICE_TYPE", SimStruct)), ("DeviceName", SimTypeRef("DOT11_WPS_DEVICE_NAME", SimStruct)),)), name="DOT11_WFD_DEVICE_INFO", pack=False, align=None),
    "DOT11_WFD_DEVICE_LISTEN_CHANNEL": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("ChannelNumber", SimTypeChar(label="Byte")),)), name="DOT11_WFD_DEVICE_LISTEN_CHANNEL", pack=False, align=None),
    "DOT11_WFD_DEVICE_TYPE": SimStruct(OrderedDict((("CategoryID", SimTypeShort(signed=False, label="UInt16")), ("SubCategoryID", SimTypeShort(signed=False, label="UInt16")), ("OUI", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="DOT11_WFD_DEVICE_TYPE", pack=False, align=None),
    "DOT11_WFD_DISCOVER_COMPLETE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Status", SimTypeInt(signed=True, label="Int32")), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uListOffset", SimTypeInt(signed=False, label="UInt32")), ("uListLength", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_WFD_DISCOVER_COMPLETE_PARAMETERS", pack=False, align=None),
    "DOT11_WFD_DISCOVER_DEVICE_FILTER": SimStruct(OrderedDict((("DeviceID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ucBitmask", SimTypeChar(label="Byte")), ("GroupSSID", SimTypeRef("DOT11_SSID", SimStruct)),)), name="DOT11_WFD_DISCOVER_DEVICE_FILTER", pack=False, align=None),
    "DOT11_WFD_DISCOVER_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("DiscoverType", SimTypeInt(signed=False, label="DOT11_WFD_DISCOVER_TYPE")), ("ScanType", SimTypeInt(signed=False, label="DOT11_WFD_SCAN_TYPE")), ("uDiscoverTimeout", SimTypeInt(signed=False, label="UInt32")), ("uDeviceFilterListOffset", SimTypeInt(signed=False, label="UInt32")), ("uNumDeviceFilters", SimTypeInt(signed=False, label="UInt32")), ("uIEsOffset", SimTypeInt(signed=False, label="UInt32")), ("uIEsLength", SimTypeInt(signed=False, label="UInt32")), ("bForceScanLegacyNetworks", SimTypeChar(label="Byte")),)), name="DOT11_WFD_DISCOVER_REQUEST", pack=False, align=None),
    "DOT11_WFD_DISCOVER_TYPE": SimTypeInt(signed=False, label="DOT11_WFD_DISCOVER_TYPE"),
    "DOT11_WFD_GO_INTENT": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="DOT11_WFD_GO_INTENT", pack=False, align=None),
    "DOT11_WFD_GROUP_ID": SimStruct(OrderedDict((("DeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("SSID", SimTypeRef("DOT11_SSID", SimStruct)),)), name="DOT11_WFD_GROUP_ID", pack=False, align=None),
    "DOT11_WFD_GROUP_JOIN_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("GOOperatingChannel", SimTypeRef("DOT11_WFD_CHANNEL", SimStruct)), ("GOConfigTime", SimTypeInt(signed=False, label="UInt32")), ("bInGroupFormation", SimTypeChar(label="Byte")), ("bWaitForWPSReady", SimTypeChar(label="Byte")),)), name="DOT11_WFD_GROUP_JOIN_PARAMETERS", pack=False, align=None),
    "DOT11_WFD_GROUP_OWNER_CAPABILITY_CONFIG": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("bPersistentGroupEnabled", SimTypeChar(label="Byte")), ("bIntraBSSDistributionSupported", SimTypeChar(label="Byte")), ("bCrossConnectionSupported", SimTypeChar(label="Byte")), ("bPersistentReconnectSupported", SimTypeChar(label="Byte")), ("bGroupFormationEnabled", SimTypeChar(label="Byte")), ("uMaximumGroupLimit", SimTypeInt(signed=False, label="UInt32")),)), name="DOT11_WFD_GROUP_OWNER_CAPABILITY_CONFIG", pack=False, align=None),
    "DOT11_WFD_GROUP_OWNER_CAPABILITY_CONFIG_V2": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("bPersistentGroupEnabled", SimTypeChar(label="Byte")), ("bIntraBSSDistributionSupported", SimTypeChar(label="Byte")), ("bCrossConnectionSupported", SimTypeChar(label="Byte")), ("bPersistentReconnectSupported", SimTypeChar(label="Byte")), ("bGroupFormationEnabled", SimTypeChar(label="Byte")), ("uMaximumGroupLimit", SimTypeInt(signed=False, label="UInt32")), ("bEapolKeyIpAddressAllocationSupported", SimTypeChar(label="Byte")),)), name="DOT11_WFD_GROUP_OWNER_CAPABILITY_CONFIG_V2", pack=False, align=None),
    "DOT11_WFD_GROUP_START_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("AdvertisedOperatingChannel", SimTypeRef("DOT11_WFD_CHANNEL", SimStruct)),)), name="DOT11_WFD_GROUP_START_PARAMETERS", pack=False, align=None),
    "DOT11_WFD_INVITATION_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="DOT11_WFD_INVITATION_FLAGS", pack=False, align=None),
    "DOT11_WFD_SCAN_TYPE": SimTypeInt(signed=False, label="DOT11_WFD_SCAN_TYPE"),
    "DOT11_WFD_SECONDARY_DEVICE_TYPE_LIST": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("SecondaryDeviceTypes", SimTypePointer(SimTypeRef("DOT11_WFD_DEVICE_TYPE", SimStruct), offset=0)),)), name="DOT11_WFD_SECONDARY_DEVICE_TYPE_LIST", pack=False, align=None),
    "DOT11_WFD_SERVICE_HASH_LIST": SimStruct(OrderedDict((("ServiceHashCount", SimTypeShort(signed=False, label="UInt16")), ("ServiceHash", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_WFD_SERVICE_HASH_LIST", pack=False, align=None),
    "DOT11_WFD_SESSION_ID": SimStruct(OrderedDict((("SessionID", SimTypeInt(signed=False, label="UInt32")), ("SessionAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="DOT11_WFD_SESSION_ID", pack=False, align=None),
    "DOT11_WFD_SESSION_INFO": SimStruct(OrderedDict((("uSessionInfoLength", SimTypeShort(signed=False, label="UInt16")), ("ucSessionInfo", SimTypeArray(SimTypeChar(label="Byte"), 144)),)), name="DOT11_WFD_SESSION_INFO", pack=False, align=None),
    "DOT11_WME_AC_PARAMETERS": SimStruct(OrderedDict((("ucAccessCategoryIndex", SimTypeChar(label="Byte")), ("ucAIFSN", SimTypeChar(label="Byte")), ("ucECWmin", SimTypeChar(label="Byte")), ("ucECWmax", SimTypeChar(label="Byte")), ("usTXOPLimit", SimTypeShort(signed=False, label="UInt16")),)), name="DOT11_WME_AC_PARAMETERS", pack=False, align=None),
    "DOT11_WME_AC_PARAMETERS_LIST": SimStruct(OrderedDict((("uNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("uTotalNumOfEntries", SimTypeInt(signed=False, label="UInt32")), ("dot11WMEACParameters", SimTypePointer(SimTypeRef("DOT11_WME_AC_PARAMETERS", SimStruct), offset=0)),)), name="DOT11_WME_AC_PARAMETERS_LIST", pack=False, align=None),
    "DOT11_WME_UPDATE_IE": SimStruct(OrderedDict((("uParamElemMinBeaconIntervals", SimTypeInt(signed=False, label="UInt32")), ("uWMEInfoElemOffset", SimTypeInt(signed=False, label="UInt32")), ("uWMEInfoElemLength", SimTypeInt(signed=False, label="UInt32")), ("uWMEParamElemOffset", SimTypeInt(signed=False, label="UInt32")), ("uWMEParamElemLength", SimTypeInt(signed=False, label="UInt32")), ("ucBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DOT11_WME_UPDATE_IE", pack=False, align=None),
    "DOT11_WPA_TSC": SimStruct(OrderedDict((("uReserved", SimTypeInt(signed=False, label="UInt32")), ("dot11OffloadType", SimTypeInt(signed=False, label="DOT11_OFFLOAD_TYPE")), ("hOffload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dot11IV48Counter", SimTypeRef("DOT11_IV48_COUNTER", SimStruct)),)), name="DOT11_WPA_TSC", pack=False, align=None),
    "DOT11_WPS_CONFIG_METHOD": SimTypeInt(signed=False, label="DOT11_WPS_CONFIG_METHOD"),
    "DOT11_WPS_DEVICE_NAME": SimStruct(OrderedDict((("uDeviceNameLength", SimTypeInt(signed=False, label="UInt32")), ("ucDeviceName", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="DOT11_WPS_DEVICE_NAME", pack=False, align=None),
    "DOT11_WPS_DEVICE_PASSWORD_ID": SimTypeInt(signed=False, label="DOT11_WPS_DEVICE_PASSWORD_ID"),
    "DOWNHORZSTATES": SimTypeInt(signed=False, label="DOWNHORZSTATES"),
    "DOWNSTATES": SimTypeInt(signed=False, label="DOWNSTATES"),
    "DPAMM_MESSAGE": SimTypeInt(signed=False, label="DPAMM_MESSAGE"),
    "DPASTREAMINFO": SimStruct(OrderedDict((("iPos", SimTypeInt(signed=True, label="Int32")), ("pvItem", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DPASTREAMINFO", pack=False, align=None),
    "DPC_WATCHDOG_GLOBAL_TRIAGE_BLOCK": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("DpcWatchdogProfileOffset", SimTypeShort(signed=False, label="UInt16")), ("DpcWatchdogProfileLength", SimTypeInt(signed=False, label="UInt32")),)), name="DPC_WATCHDOG_GLOBAL_TRIAGE_BLOCK", pack=False, align=None),
    "DPI_AWARENESS": SimTypeInt(signed=False, label="DPI_AWARENESS"),
    "DPI_AWARENESS_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "DPI_HOSTING_BEHAVIOR": SimTypeInt(signed=False, label="DPI_HOSTING_BEHAVIOR"),
    "DRAGDROPPARTS": SimTypeInt(signed=False, label="DRAGDROPPARTS"),
    "DRAGINFOA": SimStruct(OrderedDict((("uSize", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)), ("fNC", SimTypeInt(signed=True, label="Int32")), ("lpFileList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("grfKeyState", SimTypeInt(signed=False, label="UInt32")),)), name="DRAGINFOA", pack=False, align=None),
    "DRAGINFOW": SimStruct(OrderedDict((("uSize", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)), ("fNC", SimTypeInt(signed=True, label="Int32")), ("lpFileList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grfKeyState", SimTypeInt(signed=False, label="UInt32")),)), name="DRAGINFOW", pack=False, align=None),
    "DRAGLISTINFO": SimStruct(OrderedDict((("uNotification", SimTypeInt(signed=False, label="DRAGLISTINFO_NOTIFICATION_FLAGS")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ptCursor", SimTypeRef("POINT", SimStruct)),)), name="DRAGLISTINFO", pack=False, align=None),
    "DRAGLISTINFO_NOTIFICATION_FLAGS": SimTypeInt(signed=False, label="DRAGLISTINFO_NOTIFICATION_FLAGS"),
    "DRAWDIBTIME": SimStruct(OrderedDict((("timeCount", SimTypeInt(signed=True, label="Int32")), ("timeDraw", SimTypeInt(signed=True, label="Int32")), ("timeDecompress", SimTypeInt(signed=True, label="Int32")), ("timeDither", SimTypeInt(signed=True, label="Int32")), ("timeStretch", SimTypeInt(signed=True, label="Int32")), ("timeBlt", SimTypeInt(signed=True, label="Int32")), ("timeSetDIBits", SimTypeInt(signed=True, label="Int32")),)), name="DRAWDIBTIME", pack=False, align=None),
    "DRAWEDGE_FLAGS": SimTypeInt(signed=False, label="DRAWEDGE_FLAGS"),
    "DRAWITEMSTRUCT": SimStruct(OrderedDict((("CtlType", SimTypeInt(signed=False, label="DRAWITEMSTRUCT_CTL_TYPE")), ("CtlID", SimTypeInt(signed=False, label="UInt32")), ("itemID", SimTypeInt(signed=False, label="UInt32")), ("itemAction", SimTypeInt(signed=False, label="ODA_FLAGS")), ("itemState", SimTypeInt(signed=False, label="ODS_FLAGS")), ("hwndItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rcItem", SimTypeRef("RECT", SimStruct)), ("itemData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DRAWITEMSTRUCT", pack=False, align=None),
    "DRAWITEMSTRUCT_CTL_TYPE": SimTypeInt(signed=False, label="DRAWITEMSTRUCT_CTL_TYPE"),
    "DRAWPATRECT": SimStruct(OrderedDict((("ptPosition", SimTypeRef("POINT", SimStruct)), ("ptSize", SimTypeRef("POINT", SimStruct)), ("wStyle", SimTypeShort(signed=False, label="UInt16")), ("wPattern", SimTypeShort(signed=False, label="UInt16")),)), name="DRAWPATRECT", pack=False, align=None),
    "DRAWPROGRESSFLAGS": SimTypeInt(signed=False, label="DRAWPROGRESSFLAGS"),
    "DRAWSTATEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hdc", "lData", "wData", "cx", "cy"]), offset=0),
    "DRAWSTATE_FLAGS": SimTypeInt(signed=False, label="DRAWSTATE_FLAGS"),
    "DRAWTEXTPARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iTabLength", SimTypeInt(signed=True, label="Int32")), ("iLeftMargin", SimTypeInt(signed=True, label="Int32")), ("iRightMargin", SimTypeInt(signed=True, label="Int32")), ("uiLengthDrawn", SimTypeInt(signed=False, label="UInt32")),)), name="DRAWTEXTPARAMS", pack=False, align=None),
    "DRAW_CAPTION_FLAGS": SimTypeInt(signed=False, label="DRAW_CAPTION_FLAGS"),
    "DRAW_EDGE_FLAGS": SimTypeInt(signed=False, label="DRAW_EDGE_FLAGS"),
    "DRAW_TEXT_FORMAT": SimTypeInt(signed=False, label="DRAW_TEXT_FORMAT"),
    "DRAW_THEME_PARENT_BACKGROUND_FLAGS": SimTypeInt(signed=False, label="DRAW_THEME_PARENT_BACKGROUND_FLAGS"),
    "DRH_APIBITMAPDATA": SimStruct(OrderedDict((("pso", SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0)), ("b", SimTypeInt(signed=True, label="Int32")),)), name="DRH_APIBITMAPDATA", pack=False, align=None),
    "DRIVERMSGPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "DRIVEROBJ": SimStruct(OrderedDict((("pvObj", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pFreeProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVEROBJ", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDriverObj"]), offset=0)), ("hdev", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dhpdev", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DRIVEROBJ", pack=False, align=None),
    "DRIVERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "DRIVERSTATUS": SimStruct(OrderedDict((("bDriverError", SimTypeChar(label="Byte")), ("bIDEError", SimTypeChar(label="Byte")), ("bReserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="DRIVERSTATUS", pack=False, align=None),
    "DRIVER_ADD_DEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DriverObject", "PhysicalDeviceObject"]), offset=0),
    "DRIVER_CANCEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp"]), offset=0),
    "DRIVER_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"), arg_names=["DeviceObject", "Irp", "MapRegisterBase", "Context"]), offset=0),
    "DRIVER_DIRECTORY_TYPE": SimTypeInt(signed=False, label="DRIVER_DIRECTORY_TYPE"),
    "DRIVER_DISPATCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "Irp"]), offset=0),
    "DRIVER_DISPATCH_PAGED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "Irp"]), offset=0),
    "DRIVER_EXTENSION": SimStruct(OrderedDict((("DriverObject", SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0)), ("AddDevice", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DriverObject", "PhysicalDeviceObject"]), offset=0), offset=0)), ("Count", SimTypeInt(signed=False, label="UInt32")), ("ServiceKeyName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="DRIVER_EXTENSION", pack=False, align=None),
    "DRIVER_FS_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "FsActive"]), offset=0),
    "DRIVER_INFO_1A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DRIVER_INFO_1A", pack=False, align=None),
    "DRIVER_INFO_1W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRIVER_INFO_1W", pack=False, align=None),
    "DRIVER_INFO_2A": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DRIVER_INFO_2A", pack=False, align=None),
    "DRIVER_INFO_2W": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRIVER_INFO_2W", pack=False, align=None),
    "DRIVER_INFO_3A": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DRIVER_INFO_3A", pack=False, align=None),
    "DRIVER_INFO_3W": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRIVER_INFO_3W", pack=False, align=None),
    "DRIVER_INFO_4A": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DRIVER_INFO_4A", pack=False, align=None),
    "DRIVER_INFO_4W": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRIVER_INFO_4W", pack=False, align=None),
    "DRIVER_INFO_5A": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwDriverAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwConfigVersion", SimTypeInt(signed=False, label="UInt32")), ("dwDriverVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DRIVER_INFO_5A", pack=False, align=None),
    "DRIVER_INFO_5W": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwDriverAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwConfigVersion", SimTypeInt(signed=False, label="UInt32")), ("dwDriverVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DRIVER_INFO_5W", pack=False, align=None),
    "DRIVER_INFO_6A": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ftDriverDate", SimTypeRef("FILETIME", SimStruct)), ("dwlDriverVersion", SimTypeLongLong(signed=False, label="UInt64")), ("pszMfgName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszOEMUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHardwareID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszProvider", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DRIVER_INFO_6A", pack=False, align=None),
    "DRIVER_INFO_6W": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftDriverDate", SimTypeRef("FILETIME", SimStruct)), ("dwlDriverVersion", SimTypeLongLong(signed=False, label="UInt64")), ("pszMfgName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOEMUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHardwareID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRIVER_INFO_6W", pack=False, align=None),
    "DRIVER_INFO_8A": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ftDriverDate", SimTypeRef("FILETIME", SimStruct)), ("dwlDriverVersion", SimTypeLongLong(signed=False, label="UInt64")), ("pszMfgName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszOEMUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHardwareID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszProvider", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPrintProcessor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszVendorSetup", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszzColorProfiles", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszInfPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwPrinterDriverAttributes", SimTypeInt(signed=False, label="UInt32")), ("pszzCoreDriverDependencies", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ftMinInboxDriverVerDate", SimTypeRef("FILETIME", SimStruct)), ("dwlMinInboxDriverVerVersion", SimTypeLongLong(signed=False, label="UInt64")),)), name="DRIVER_INFO_8A", pack=False, align=None),
    "DRIVER_INFO_8W": SimStruct(OrderedDict((("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftDriverDate", SimTypeRef("FILETIME", SimStruct)), ("dwlDriverVersion", SimTypeLongLong(signed=False, label="UInt64")), ("pszMfgName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOEMUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHardwareID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPrintProcessor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszVendorSetup", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszzColorProfiles", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInfPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPrinterDriverAttributes", SimTypeInt(signed=False, label="UInt32")), ("pszzCoreDriverDependencies", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftMinInboxDriverVerDate", SimTypeRef("FILETIME", SimStruct)), ("dwlMinInboxDriverVerVersion", SimTypeLongLong(signed=False, label="UInt64")),)), name="DRIVER_INFO_8W", pack=False, align=None),
    "DRIVER_INITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DriverObject", "RegistryPath"]), offset=0),
    "DRIVER_LIST_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0),
    "DRIVER_NOTIFICATION_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["NotificationStructure", "Context"]), offset=0),
    "DRIVER_OBJECT": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=True, label="Int16")), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DriverStart", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DriverSize", SimTypeInt(signed=False, label="UInt32")), ("DriverSection", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DriverExtension", SimTypePointer(SimTypeRef("DRIVER_EXTENSION", SimStruct), offset=0)), ("DriverName", SimTypeRef("UNICODE_STRING", SimStruct)), ("HardwareDatabase", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("FastIoDispatch", SimTypePointer(SimTypeRef("FAST_IO_DISPATCH", SimStruct), offset=0)), ("DriverInit", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DriverObject", "RegistryPath"]), offset=0), offset=0)), ("DriverStartIo", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp"]), offset=0), offset=0)), ("DriverUnload", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DriverObject"]), offset=0), offset=0)), ("MajorFunction", SimTypeArray(SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "Irp"]), offset=0), offset=0), 28)),)), name="DRIVER_OBJECT", pack=False, align=None),
    "DRIVER_REGKEY_TYPE": SimTypeInt(signed=False, label="DRIVER_REGKEY_TYPE"),
    "DRIVER_REINITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["DriverObject", "Context", "Count"]), offset=0),
    "DRIVER_RUNTIME_INIT_FLAGS": SimTypeInt(signed=False, label="DRIVER_RUNTIME_INIT_FLAGS"),
    "DRIVER_STARTIO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp"]), offset=0),
    "DRIVER_UNLOAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DriverObject"]), offset=0),
    "DRIVER_UPGRADE_INFO_1": SimStruct(OrderedDict((("pPrinterName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pOldDriverDirectory", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="DRIVER_UPGRADE_INFO_1", pack=False, align=None),
    "DRIVER_UPGRADE_INFO_2": SimStruct(OrderedDict((("pPrinterName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pOldDriverDirectory", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("cVersion", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pDriverPath", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pDataFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pConfigFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pDependentFiles", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pDefaultDataType", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszzPreviousNames", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="DRIVER_UPGRADE_INFO_2", pack=False, align=None),
    "DRIVER_VERIFIER_THUNK_PAIRS": SimStruct(OrderedDict((("PristineRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["Context"]), offset=0)), ("NewRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["Context"]), offset=0)),)), name="DRIVER_VERIFIER_THUNK_PAIRS", pack=False, align=None),
    "DRIVER_VER_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwPlatform", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("wszVersion", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wszSignedBy", SimTypeArray(SimTypeChar(label="Char"), 260)), ("pcSignerCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("sOSVersionLow", SimTypeRef("DRIVER_VER_MAJORMINOR", SimStruct)), ("sOSVersionHigh", SimTypeRef("DRIVER_VER_MAJORMINOR", SimStruct)), ("dwBuildNumberLow", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumberHigh", SimTypeInt(signed=False, label="UInt32")),)), name="DRIVER_VER_INFO", pack=False, align=None),
    "DRIVER_VER_MAJORMINOR": SimStruct(OrderedDict((("dwMajor", SimTypeInt(signed=False, label="UInt32")), ("dwMinor", SimTypeInt(signed=False, label="UInt32")),)), name="DRIVER_VER_MAJORMINOR", pack=False, align=None),
    "DRIVE_LAYOUT_INFORMATION": SimStruct(OrderedDict((("PartitionCount", SimTypeInt(signed=False, label="UInt32")), ("Signature", SimTypeInt(signed=False, label="UInt32")), ("PartitionEntry", SimTypePointer(SimTypeRef("PARTITION_INFORMATION", SimStruct), offset=0)),)), name="DRIVE_LAYOUT_INFORMATION", pack=False, align=None),
    "DRIVE_LAYOUT_INFORMATION_EX": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="UInt32")), ("PartitionCount", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("DRIVE_LAYOUT_INFORMATION_MBR", SimStruct), "Gpt": SimTypeRef("DRIVE_LAYOUT_INFORMATION_GPT", SimStruct)}, name="<anon>", label="None")), ("PartitionEntry", SimTypePointer(SimTypeRef("PARTITION_INFORMATION_EX", SimStruct), offset=0)),)), name="DRIVE_LAYOUT_INFORMATION_EX", pack=False, align=None),
    "DRIVE_LAYOUT_INFORMATION_GPT": SimStruct(OrderedDict((("DiskId", SimTypeBottom(label="Guid")), ("StartingUsableOffset", SimTypeLongLong(signed=True, label="Int64")), ("UsableLength", SimTypeLongLong(signed=True, label="Int64")), ("MaxPartitionCount", SimTypeInt(signed=False, label="UInt32")),)), name="DRIVE_LAYOUT_INFORMATION_GPT", pack=False, align=None),
    "DRIVE_LAYOUT_INFORMATION_MBR": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")),)), name="DRIVE_LAYOUT_INFORMATION_MBR", pack=False, align=None),
    "DRMATTESTTYPE": SimTypeInt(signed=False, label="DRMATTESTTYPE"),
    "DRMBOUNDLICENSEPARAMS": SimStruct(OrderedDict((("uVersion", SimTypeInt(signed=False, label="UInt32")), ("hEnablingPrincipal", SimTypeInt(signed=False, label="UInt32")), ("hSecureStore", SimTypeInt(signed=False, label="UInt32")), ("wszRightsRequested", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszRightsGroup", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("idResource", SimTypeRef("DRMID", SimStruct)), ("cAuthenticatorCount", SimTypeInt(signed=False, label="UInt32")), ("rghAuthenticators", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("wszDefaultEnablingPrincipalCredentials", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DRMBOUNDLICENSEPARAMS", pack=False, align=None),
    "DRMCALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="DRM_STATUS_MSG"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "DRMENCODINGTYPE": SimTypeInt(signed=False, label="DRMENCODINGTYPE"),
    "DRMGLOBALOPTIONS": SimTypeInt(signed=False, label="DRMGLOBALOPTIONS"),
    "DRMID": SimStruct(OrderedDict((("uVersion", SimTypeInt(signed=False, label="UInt32")), ("wszIDType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRMID", pack=False, align=None),
    "DRMSECURITYPROVIDERTYPE": SimTypeInt(signed=False, label="DRMSECURITYPROVIDERTYPE"),
    "DRMSPECTYPE": SimTypeInt(signed=False, label="DRMSPECTYPE"),
    "DRMTIMETYPE": SimTypeInt(signed=False, label="DRMTIMETYPE"),
    "DRMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("ulContentId", SimTypeInt(signed=False, label="UInt32")), ("wfxSecure", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="DRMWAVEFORMAT", pack=False, align=None),
    "DRM_ACTSERV_INFO": SimStruct(OrderedDict((("uVersion", SimTypeInt(signed=False, label="UInt32")), ("wszPubKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRM_ACTSERV_INFO", pack=False, align=None),
    "DRM_CLIENT_VERSION_INFO": SimStruct(OrderedDict((("uStructVersion", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("wszHierarchy", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszProductId", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wszProductDescription", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="DRM_CLIENT_VERSION_INFO", pack=False, align=None),
    "DRM_COPY_OPL": SimStruct(OrderedDict((("wMinimumCopyLevel", SimTypeShort(signed=False, label="UInt16")), ("oplIdIncludes", SimTypeRef("DRM_OPL_OUTPUT_IDS", SimStruct)), ("oplIdExcludes", SimTypeRef("DRM_OPL_OUTPUT_IDS", SimStruct)),)), name="DRM_COPY_OPL", pack=False, align=None),
    "DRM_DISTRIBUTION_POINT_INFO": SimTypeInt(signed=False, label="DRM_DISTRIBUTION_POINT_INFO"),
    "DRM_LICENSE_ACQ_DATA": SimStruct(OrderedDict((("uVersion", SimTypeInt(signed=False, label="UInt32")), ("wszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszLocalFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pbPostData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwPostDataSize", SimTypeInt(signed=False, label="UInt32")), ("wszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRM_LICENSE_ACQ_DATA", pack=False, align=None),
    "DRM_MINIMUM_OUTPUT_PROTECTION_LEVELS": SimStruct(OrderedDict((("wCompressedDigitalVideo", SimTypeShort(signed=False, label="UInt16")), ("wUncompressedDigitalVideo", SimTypeShort(signed=False, label="UInt16")), ("wAnalogVideo", SimTypeShort(signed=False, label="UInt16")), ("wCompressedDigitalAudio", SimTypeShort(signed=False, label="UInt16")), ("wUncompressedDigitalAudio", SimTypeShort(signed=False, label="UInt16")),)), name="DRM_MINIMUM_OUTPUT_PROTECTION_LEVELS", pack=False, align=None),
    "DRM_OPL_OUTPUT_IDS": SimStruct(OrderedDict((("cIds", SimTypeShort(signed=False, label="UInt16")), ("rgIds", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="DRM_OPL_OUTPUT_IDS", pack=False, align=None),
    "DRM_OUTPUT_PROTECTION": SimStruct(OrderedDict((("guidId", SimTypeBottom(label="Guid")), ("bConfigData", SimTypeChar(label="Byte")),)), name="DRM_OUTPUT_PROTECTION", pack=False, align=None),
    "DRM_PLAY_OPL": SimStruct(OrderedDict((("minOPL", SimTypeRef("DRM_MINIMUM_OUTPUT_PROTECTION_LEVELS", SimStruct)), ("oplIdReserved", SimTypeRef("DRM_OPL_OUTPUT_IDS", SimStruct)), ("vopi", SimTypeRef("DRM_VIDEO_OUTPUT_PROTECTION_IDS", SimStruct)),)), name="DRM_PLAY_OPL", pack=False, align=None),
    "DRM_STATUS_MSG": SimTypeInt(signed=False, label="DRM_STATUS_MSG"),
    "DRM_USAGEPOLICY_TYPE": SimTypeInt(signed=False, label="DRM_USAGEPOLICY_TYPE"),
    "DRM_VAL16": SimStruct(OrderedDict((("val", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DRM_VAL16", pack=False, align=None),
    "DRM_VIDEO_OUTPUT_PROTECTION_IDS": SimStruct(OrderedDict((("cEntries", SimTypeShort(signed=False, label="UInt16")), ("rgVop", SimTypePointer(SimTypeRef("DRM_OUTPUT_PROTECTION", SimStruct), offset=0)),)), name="DRM_VIDEO_OUTPUT_PROTECTION_IDS", pack=False, align=None),
    "DROPDESCRIPTION": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="DROPIMAGETYPE")), ("szMessage", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szInsert", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="DROPDESCRIPTION", pack=False, align=None),
    "DROPDOWNBUTTONLEFTSTATES": SimTypeInt(signed=False, label="DROPDOWNBUTTONLEFTSTATES"),
    "DROPDOWNBUTTONRIGHTSTATES": SimTypeInt(signed=False, label="DROPDOWNBUTTONRIGHTSTATES"),
    "DROPDOWNITEMSTATES": SimTypeInt(signed=False, label="DROPDOWNITEMSTATES"),
    "DROPEFFECT": SimTypeInt(signed=False, label="DROPEFFECT"),
    "DROPFILES": SimStruct(OrderedDict((("pFiles", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)), ("fNC", SimTypeInt(signed=True, label="Int32")), ("fWide", SimTypeInt(signed=True, label="Int32")),)), name="DROPFILES", pack=False, align=None),
    "DROPIMAGETYPE": SimTypeInt(signed=False, label="DROPIMAGETYPE"),
    "DROPSTRUCT": SimStruct(OrderedDict((("hwndSource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndSink", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wFmt", SimTypeInt(signed=False, label="UInt32")), ("dwData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ptDrop", SimTypeRef("POINT", SimStruct)), ("dwControlData", SimTypeInt(signed=False, label="UInt32")),)), name="DROPSTRUCT", pack=False, align=None),
    "DRT_ADDRESS": SimStruct(OrderedDict((("socketAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("nearness", SimTypeInt(signed=True, label="Int32")), ("latency", SimTypeInt(signed=False, label="UInt32")),)), name="DRT_ADDRESS", pack=False, align=None),
    "DRT_ADDRESS_FLAGS": SimTypeInt(signed=False, label="DRT_ADDRESS_FLAGS"),
    "DRT_ADDRESS_LIST": SimStruct(OrderedDict((("AddressCount", SimTypeInt(signed=False, label="UInt32")), ("AddressList", SimTypePointer(SimTypeRef("DRT_ADDRESS", SimStruct), offset=0)),)), name="DRT_ADDRESS_LIST", pack=False, align=None),
    "DRT_BOOTSTRAP_PROVIDER": SimStruct(OrderedDict((("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attach", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Detach", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InitResolve", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IssueResolve", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EndResolve", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Register", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Unregister", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DRT_BOOTSTRAP_PROVIDER", pack=False, align=None),
    "DRT_BOOTSTRAP_RESOLVE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SOCKET_ADDRESS_LIST", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["hr", "pvContext", "pAddresses", "fFatalError"]), offset=0),
    "DRT_DATA": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("pb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DRT_DATA", pack=False, align=None),
    "DRT_EVENT_DATA": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="DRT_EVENT_TYPE")), ("hr", SimTypeInt(signed=True, label="Int32")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Anonymous", SimUnion({"leafsetKeyChange": SimTypeRef("_leafsetKeyChange_e__Struct", SimStruct), "registrationStateChange": SimTypeRef("_registrationStateChange_e__Struct", SimStruct), "statusChange": SimTypeRef("_statusChange_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="DRT_EVENT_DATA", pack=False, align=None),
    "DRT_EVENT_TYPE": SimTypeInt(signed=False, label="DRT_EVENT_TYPE"),
    "DRT_LEAFSET_KEY_CHANGE_TYPE": SimTypeInt(signed=False, label="DRT_LEAFSET_KEY_CHANGE_TYPE"),
    "DRT_MATCH_TYPE": SimTypeInt(signed=False, label="DRT_MATCH_TYPE"),
    "DRT_REGISTRATION": SimStruct(OrderedDict((("key", SimTypeRef("DRT_DATA", SimStruct)), ("appData", SimTypeRef("DRT_DATA", SimStruct)),)), name="DRT_REGISTRATION", pack=False, align=None),
    "DRT_REGISTRATION_STATE": SimTypeInt(signed=False, label="DRT_REGISTRATION_STATE"),
    "DRT_SCOPE": SimTypeInt(signed=False, label="DRT_SCOPE"),
    "DRT_SEARCH_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fIterative", SimTypeInt(signed=True, label="Int32")), ("fAllowCurrentInstanceMatch", SimTypeInt(signed=True, label="Int32")), ("fAnyMatchInRange", SimTypeInt(signed=True, label="Int32")), ("cMaxEndpoints", SimTypeInt(signed=False, label="UInt32")), ("pMaximumKey", SimTypePointer(SimTypeRef("DRT_DATA", SimStruct), offset=0)), ("pMinimumKey", SimTypePointer(SimTypeRef("DRT_DATA", SimStruct), offset=0)),)), name="DRT_SEARCH_INFO", pack=False, align=None),
    "DRT_SEARCH_RESULT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="DRT_MATCH_TYPE")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("registration", SimTypeRef("DRT_REGISTRATION", SimStruct)),)), name="DRT_SEARCH_RESULT", pack=False, align=None),
    "DRT_SECURITY_MODE": SimTypeInt(signed=False, label="DRT_SECURITY_MODE"),
    "DRT_SECURITY_PROVIDER": SimStruct(OrderedDict((("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attach", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Detach", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RegisterKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UnregisterKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ValidateAndUnpackPayload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SecureAndPackPayload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FreeData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EncryptData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DecryptData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetSerializedCredential", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ValidateRemoteCredential", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SignData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("VerifyData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DRT_SECURITY_PROVIDER", pack=False, align=None),
    "DRT_SETTINGS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("bProtocolMajorVersion", SimTypeChar(label="Byte")), ("bProtocolMinorVersion", SimTypeChar(label="Byte")), ("ulMaxRoutingAddresses", SimTypeInt(signed=False, label="UInt32")), ("pwzDrtInstancePrefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hTransport", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pSecurityProvider", SimTypePointer(SimTypeRef("DRT_SECURITY_PROVIDER", SimStruct), offset=0)), ("pBootstrapProvider", SimTypePointer(SimTypeRef("DRT_BOOTSTRAP_PROVIDER", SimStruct), offset=0)), ("eSecurityMode", SimTypeInt(signed=False, label="DRT_SECURITY_MODE")),)), name="DRT_SETTINGS", pack=False, align=None),
    "DRT_STATUS": SimTypeInt(signed=False, label="DRT_STATUS"),
    "DRVCONFIGINFO": SimStruct(OrderedDict((("dwDCISize", SimTypeInt(signed=False, label="UInt32")), ("lpszDCISectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszDCIAliasName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DRVCONFIGINFO", pack=False, align=None),
    "DRVCONFIGINFOEX": SimStruct(OrderedDict((("dwDCISize", SimTypeInt(signed=False, label="UInt32")), ("lpszDCISectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszDCIAliasName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dnDevNode", SimTypeInt(signed=False, label="UInt32")),)), name="DRVCONFIGINFOEX", pack=False, align=None),
    "DRVENABLEDATA": SimStruct(OrderedDict((("iDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("c", SimTypeInt(signed=False, label="UInt32")), ("pdrvfn", SimTypePointer(SimTypeRef("DRVFN", SimStruct), offset=0)),)), name="DRVENABLEDATA", pack=False, align=None),
    "DRVFN": SimStruct(OrderedDict((("iFunc", SimTypeInt(signed=False, label="UInt32")), ("pfn", SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0)),)), name="DRVFN", pack=False, align=None),
    "DRVM_IOCTL_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCmd", SimTypeInt(signed=False, label="UInt32")),)), name="DRVM_IOCTL_DATA", pack=False, align=None),
    "DRV_VERSION": SimStruct(OrderedDict((("major", SimTypeInt(signed=False, label="UInt32")), ("minor", SimTypeInt(signed=False, label="UInt32")), ("internal", SimTypeInt(signed=False, label="UInt32")),)), name="DRV_VERSION", pack=False, align=None),
    "DRendezvousSessionEvents": SimTypeBottom(label="DRendezvousSessionEvents"),
    "DS3DBUFFER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("vPosition", SimTypeRef("D3DVECTOR", SimStruct)), ("vVelocity", SimTypeRef("D3DVECTOR", SimStruct)), ("dwInsideConeAngle", SimTypeInt(signed=False, label="UInt32")), ("dwOutsideConeAngle", SimTypeInt(signed=False, label="UInt32")), ("vConeOrientation", SimTypeRef("D3DVECTOR", SimStruct)), ("lConeOutsideVolume", SimTypeInt(signed=True, label="Int32")), ("flMinDistance", SimTypeFloat(size=32)), ("flMaxDistance", SimTypeFloat(size=32)), ("dwMode", SimTypeInt(signed=False, label="UInt32")),)), name="DS3DBUFFER", pack=False, align=None),
    "DS3DLISTENER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("vPosition", SimTypeRef("D3DVECTOR", SimStruct)), ("vVelocity", SimTypeRef("D3DVECTOR", SimStruct)), ("vOrientFront", SimTypeRef("D3DVECTOR", SimStruct)), ("vOrientTop", SimTypeRef("D3DVECTOR", SimStruct)), ("flDistanceFactor", SimTypeFloat(size=32)), ("flRolloffFactor", SimTypeFloat(size=32)), ("flDopplerFactor", SimTypeFloat(size=32)),)), name="DS3DLISTENER", pack=False, align=None),
    "DS3DVECTOR": SimStruct(OrderedDict((("Anonymous1", SimUnion({"x": SimTypeFloat(size=32), "dvX": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"y": SimTypeFloat(size=32), "dvY": SimTypeFloat(size=32)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"z": SimTypeFloat(size=32), "dvZ": SimTypeFloat(size=32)}, name="<anon>", label="None")),)), name="DS3DVECTOR", pack=False, align=None),
    "DSAFIPSVERSION_ENUM": SimTypeInt(signed=False, label="DSAFIPSVERSION_ENUM"),
    "DSA_NEWOBJ_DISPINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hObjClassIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszWizTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszContDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DSA_NEWOBJ_DISPINFO", pack=False, align=None),
    "DSBCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("dwUnlockTransferRate", SimTypeInt(signed=False, label="UInt32")), ("dwPlayCpuOverhead", SimTypeInt(signed=False, label="UInt32")),)), name="DSBCAPS", pack=False, align=None),
    "DSBITEMA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pszADsPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwStateMask", SimTypeInt(signed=False, label="UInt32")), ("szDisplayName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szIconLocation", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("iIconResID", SimTypeInt(signed=True, label="Int32")),)), name="DSBITEMA", pack=False, align=None),
    "DSBITEMW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pszADsPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwStateMask", SimTypeInt(signed=False, label="UInt32")), ("szDisplayName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szIconLocation", SimTypeArray(SimTypeChar(label="Char"), 260)), ("iIconResID", SimTypeInt(signed=True, label="Int32")),)), name="DSBITEMW", pack=False, align=None),
    "DSBPOSITIONNOTIFY": SimStruct(OrderedDict((("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("hEventNotify", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DSBPOSITIONNOTIFY", pack=False, align=None),
    "DSBROWSEINFOA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszRoot", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchPath", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "uMsg", "lParam", "lpData"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReturnFormat", SimTypeInt(signed=False, label="UInt32")), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchObjectClass", SimTypeInt(signed=False, label="UInt32")),)), name="DSBROWSEINFOA", pack=False, align=None),
    "DSBROWSEINFOW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRoot", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchPath", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "uMsg", "lParam", "lpData"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReturnFormat", SimTypeInt(signed=False, label="UInt32")), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchObjectClass", SimTypeInt(signed=False, label="UInt32")),)), name="DSBROWSEINFOW", pack=False, align=None),
    "DSBUFFERDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("lpwfxFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("guid3DAlgorithm", SimTypeBottom(label="Guid")),)), name="DSBUFFERDESC", pack=False, align=None),
    "DSBUFFERDESC1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("lpwfxFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)),)), name="DSBUFFERDESC1", pack=False, align=None),
    "DSCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwMinSecondarySampleRate", SimTypeInt(signed=False, label="UInt32")), ("dwMaxSecondarySampleRate", SimTypeInt(signed=False, label="UInt32")), ("dwPrimaryBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwMixingAllBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwMixingStaticBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHwMixingStreamingBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHwMixingAllBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHwMixingStaticBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHwMixingStreamingBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHw3DAllBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHw3DStaticBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxHw3DStreamingBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHw3DAllBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHw3DStaticBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHw3DStreamingBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwTotalHwMemBytes", SimTypeInt(signed=False, label="UInt32")), ("dwFreeHwMemBytes", SimTypeInt(signed=False, label="UInt32")), ("dwMaxContigFreeHwMemBytes", SimTypeInt(signed=False, label="UInt32")), ("dwUnlockTransferRateHwBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwPlayCpuOverheadSwBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="DSCAPS", pack=False, align=None),
    "DSCBCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DSCBCAPS", pack=False, align=None),
    "DSCBUFFERDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("lpwfxFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("dwFXCount", SimTypeInt(signed=False, label="UInt32")), ("lpDSCFXDesc", SimTypePointer(SimTypeRef("DSCEFFECTDESC", SimStruct), offset=0)),)), name="DSCBUFFERDESC", pack=False, align=None),
    "DSCBUFFERDESC1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("lpwfxFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)),)), name="DSCBUFFERDESC1", pack=False, align=None),
    "DSCCAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("dwChannels", SimTypeInt(signed=False, label="UInt32")),)), name="DSCCAPS", pack=False, align=None),
    "DSCEFFECTDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidDSCFXClass", SimTypeBottom(label="Guid")), ("guidDSCFXInstance", SimTypeBottom(label="Guid")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="DSCEFFECTDESC", pack=False, align=None),
    "DSCFXAec": SimStruct(OrderedDict((("fEnable", SimTypeInt(signed=True, label="Int32")), ("fNoiseFill", SimTypeInt(signed=True, label="Int32")), ("dwMode", SimTypeInt(signed=False, label="UInt32")),)), name="DSCFXAec", pack=False, align=None),
    "DSCFXNoiseSuppress": SimStruct(OrderedDict((("fEnable", SimTypeInt(signed=True, label="Int32")),)), name="DSCFXNoiseSuppress", pack=False, align=None),
    "DSCLASSCREATIONINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("clsidWizardDialog", SimTypeBottom(label="Guid")), ("clsidWizardPrimaryPage", SimTypeBottom(label="Guid")), ("cWizardExtensions", SimTypeInt(signed=False, label="UInt32")), ("aWizardExtensions", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="DSCLASSCREATIONINFO", pack=False, align=None),
    "DSCOLUMN": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fmt", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("idsName", SimTypeInt(signed=True, label="Int32")), ("offsetProperty", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="DSCOLUMN", pack=False, align=None),
    "DSDISPLAYSPECOPTIONS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("offsetAttribPrefix", SimTypeInt(signed=False, label="UInt32")), ("offsetUserName", SimTypeInt(signed=False, label="UInt32")), ("offsetPassword", SimTypeInt(signed=False, label="UInt32")), ("offsetServer", SimTypeInt(signed=False, label="UInt32")), ("offsetServerConfigPath", SimTypeInt(signed=False, label="UInt32")),)), name="DSDISPLAYSPECOPTIONS", pack=False, align=None),
    "DSEFFECTDESC": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidDSFXClass", SimTypeBottom(label="Guid")), ("dwReserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DSEFFECTDESC", pack=False, align=None),
    "DSFXChorus": SimStruct(OrderedDict((("fWetDryMix", SimTypeFloat(size=32)), ("fDepth", SimTypeFloat(size=32)), ("fFeedback", SimTypeFloat(size=32)), ("fFrequency", SimTypeFloat(size=32)), ("lWaveform", SimTypeInt(signed=True, label="Int32")), ("fDelay", SimTypeFloat(size=32)), ("lPhase", SimTypeInt(signed=True, label="Int32")),)), name="DSFXChorus", pack=False, align=None),
    "DSFXCompressor": SimStruct(OrderedDict((("fGain", SimTypeFloat(size=32)), ("fAttack", SimTypeFloat(size=32)), ("fRelease", SimTypeFloat(size=32)), ("fThreshold", SimTypeFloat(size=32)), ("fRatio", SimTypeFloat(size=32)), ("fPredelay", SimTypeFloat(size=32)),)), name="DSFXCompressor", pack=False, align=None),
    "DSFXDistortion": SimStruct(OrderedDict((("fGain", SimTypeFloat(size=32)), ("fEdge", SimTypeFloat(size=32)), ("fPostEQCenterFrequency", SimTypeFloat(size=32)), ("fPostEQBandwidth", SimTypeFloat(size=32)), ("fPreLowpassCutoff", SimTypeFloat(size=32)),)), name="DSFXDistortion", pack=False, align=None),
    "DSFXEcho": SimStruct(OrderedDict((("fWetDryMix", SimTypeFloat(size=32)), ("fFeedback", SimTypeFloat(size=32)), ("fLeftDelay", SimTypeFloat(size=32)), ("fRightDelay", SimTypeFloat(size=32)), ("lPanDelay", SimTypeInt(signed=True, label="Int32")),)), name="DSFXEcho", pack=False, align=None),
    "DSFXFlanger": SimStruct(OrderedDict((("fWetDryMix", SimTypeFloat(size=32)), ("fDepth", SimTypeFloat(size=32)), ("fFeedback", SimTypeFloat(size=32)), ("fFrequency", SimTypeFloat(size=32)), ("lWaveform", SimTypeInt(signed=True, label="Int32")), ("fDelay", SimTypeFloat(size=32)), ("lPhase", SimTypeInt(signed=True, label="Int32")),)), name="DSFXFlanger", pack=False, align=None),
    "DSFXGargle": SimStruct(OrderedDict((("dwRateHz", SimTypeInt(signed=False, label="UInt32")), ("dwWaveShape", SimTypeInt(signed=False, label="UInt32")),)), name="DSFXGargle", pack=False, align=None),
    "DSFXI3DL2Reverb": SimStruct(OrderedDict((("lRoom", SimTypeInt(signed=True, label="Int32")), ("lRoomHF", SimTypeInt(signed=True, label="Int32")), ("flRoomRolloffFactor", SimTypeFloat(size=32)), ("flDecayTime", SimTypeFloat(size=32)), ("flDecayHFRatio", SimTypeFloat(size=32)), ("lReflections", SimTypeInt(signed=True, label="Int32")), ("flReflectionsDelay", SimTypeFloat(size=32)), ("lReverb", SimTypeInt(signed=True, label="Int32")), ("flReverbDelay", SimTypeFloat(size=32)), ("flDiffusion", SimTypeFloat(size=32)), ("flDensity", SimTypeFloat(size=32)), ("flHFReference", SimTypeFloat(size=32)),)), name="DSFXI3DL2Reverb", pack=False, align=None),
    "DSFXParamEq": SimStruct(OrderedDict((("fCenter", SimTypeFloat(size=32)), ("fBandwidth", SimTypeFloat(size=32)), ("fGain", SimTypeFloat(size=32)),)), name="DSFXParamEq", pack=False, align=None),
    "DSFXWavesReverb": SimStruct(OrderedDict((("fInGain", SimTypeFloat(size=32)), ("fReverbMix", SimTypeFloat(size=32)), ("fReverbTime", SimTypeFloat(size=32)), ("fHighFreqRTRatio", SimTypeFloat(size=32)),)), name="DSFXWavesReverb", pack=False, align=None),
    "DSHOW_STREAM_DESC": SimStruct(OrderedDict((("VersionNo", SimTypeInt(signed=False, label="UInt32")), ("StreamId", SimTypeInt(signed=False, label="UInt32")), ("Default", SimTypeInt(signed=True, label="Int32")), ("Creation", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DSHOW_STREAM_DESC", pack=False, align=None),
    "DSH_FLAGS": SimTypeInt(signed=False, label="DSH_FLAGS"),
    "DSI_CONTROL_TRANSMISSION_MODE": SimTypeInt(signed=False, label="DSI_CONTROL_TRANSMISSION_MODE"),
    "DSKTLSYSTEMTIME": SimStruct(OrderedDict((("wYear", SimTypeShort(signed=False, label="UInt16")), ("wMonth", SimTypeShort(signed=False, label="UInt16")), ("wDayOfWeek", SimTypeShort(signed=False, label="UInt16")), ("wDay", SimTypeShort(signed=False, label="UInt16")), ("wHour", SimTypeShort(signed=False, label="UInt16")), ("wMinute", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")), ("wMilliseconds", SimTypeShort(signed=False, label="UInt16")), ("wResult", SimTypeShort(signed=False, label="UInt16")),)), name="DSKTLSYSTEMTIME", pack=False, align=None),
    "DSMCC_ELEMENT": SimStruct(OrderedDict((("pid", SimTypeShort(signed=False, label="UInt16")), ("bComponentTag", SimTypeChar(label="Byte")), ("dwCarouselId", SimTypeInt(signed=False, label="UInt32")), ("dwTransactionId", SimTypeInt(signed=False, label="UInt32")), ("pNext", SimTypePointer(SimTypeRef("DSMCC_ELEMENT", SimStruct), offset=0)),)), name="DSMCC_ELEMENT", pack=False, align=None),
    "DSMCC_FILTER_OPTIONS": SimStruct(OrderedDict((("fSpecifyProtocol", SimTypeInt(signed=True, label="Int32")), ("Protocol", SimTypeChar(label="Byte")), ("fSpecifyType", SimTypeInt(signed=True, label="Int32")), ("Type", SimTypeChar(label="Byte")), ("fSpecifyMessageId", SimTypeInt(signed=True, label="Int32")), ("MessageId", SimTypeShort(signed=False, label="UInt16")), ("fSpecifyTransactionId", SimTypeInt(signed=True, label="Int32")), ("fUseTrxIdMessageIdMask", SimTypeInt(signed=True, label="Int32")), ("TransactionId", SimTypeInt(signed=False, label="UInt32")), ("fSpecifyModuleVersion", SimTypeInt(signed=True, label="Int32")), ("ModuleVersion", SimTypeChar(label="Byte")), ("fSpecifyBlockNumber", SimTypeInt(signed=True, label="Int32")), ("BlockNumber", SimTypeShort(signed=False, label="UInt16")), ("fGetModuleCall", SimTypeInt(signed=True, label="Int32")), ("NumberOfBlocksInModule", SimTypeShort(signed=False, label="UInt16")),)), name="DSMCC_FILTER_OPTIONS", pack=False, align=None),
    "DSMCC_SECTION": SimStruct(OrderedDict((("TableId", SimTypeChar(label="Byte")), ("Header", SimUnion({"S": SimTypeRef("MPEG_HEADER_BITS_MIDL", SimStruct), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("TableIdExtension", SimTypeShort(signed=False, label="UInt16")), ("Version", SimUnion({"S": SimTypeRef("MPEG_HEADER_VERSION_BITS_MIDL", SimStruct), "B": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("SectionNumber", SimTypeChar(label="Byte")), ("LastSectionNumber", SimTypeChar(label="Byte")), ("ProtocolDiscriminator", SimTypeChar(label="Byte")), ("DsmccType", SimTypeChar(label="Byte")), ("MessageId", SimTypeShort(signed=False, label="UInt16")), ("TransactionId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeChar(label="Byte")), ("AdaptationLength", SimTypeChar(label="Byte")), ("MessageLength", SimTypeShort(signed=False, label="UInt16")), ("RemainingData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DSMCC_SECTION", pack=False, align=None),
    "DSM_NOTIFICATION_REQUEST_BLOCK": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("NotifyFlags", SimTypeInt(signed=False, label="UInt32")), ("DataSetProfile", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("DataSetRangesCount", SimTypeInt(signed=False, label="UInt32")), ("DataSetRanges", SimTypePointer(SimTypeRef("MP_DEVICE_DATA_SET_RANGE", SimStruct), offset=0)),)), name="DSM_NOTIFICATION_REQUEST_BLOCK", pack=False, align=None),
    "DSOBJECT": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwProviderFlags", SimTypeInt(signed=False, label="UInt32")), ("offsetName", SimTypeInt(signed=False, label="UInt32")), ("offsetClass", SimTypeInt(signed=False, label="UInt32")),)), name="DSOBJECT", pack=False, align=None),
    "DSOBJECTNAMES": SimStruct(OrderedDict((("clsidNamespace", SimTypeBottom(label="Guid")), ("cItems", SimTypeInt(signed=False, label="UInt32")), ("aObjects", SimTypePointer(SimTypeRef("DSOBJECT", SimStruct), offset=0)),)), name="DSOBJECTNAMES", pack=False, align=None),
    "DSOP_FILTER_FLAGS": SimStruct(OrderedDict((("Uplevel", SimTypeRef("DSOP_UPLEVEL_FILTER_FLAGS", SimStruct)), ("flDownlevel", SimTypeInt(signed=False, label="UInt32")),)), name="DSOP_FILTER_FLAGS", pack=False, align=None),
    "DSOP_INIT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pwzTargetComputer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cDsScopeInfos", SimTypeInt(signed=False, label="UInt32")), ("aDsScopeInfos", SimTypePointer(SimTypeRef("DSOP_SCOPE_INIT_INFO", SimStruct), offset=0)), ("flOptions", SimTypeInt(signed=False, label="UInt32")), ("cAttributesToFetch", SimTypeInt(signed=False, label="UInt32")), ("apwzAttributeNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="DSOP_INIT_INFO", pack=False, align=None),
    "DSOP_SCOPE_INIT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("flType", SimTypeInt(signed=False, label="UInt32")), ("flScope", SimTypeInt(signed=False, label="UInt32")), ("FilterFlags", SimTypeRef("DSOP_FILTER_FLAGS", SimStruct)), ("pwzDcName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzADsPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hr", SimTypeInt(signed=True, label="Int32")),)), name="DSOP_SCOPE_INIT_INFO", pack=False, align=None),
    "DSOP_UPLEVEL_FILTER_FLAGS": SimStruct(OrderedDict((("flBothModes", SimTypeInt(signed=False, label="UInt32")), ("flMixedModeOnly", SimTypeInt(signed=False, label="UInt32")), ("flNativeModeOnly", SimTypeInt(signed=False, label="UInt32")),)), name="DSOP_UPLEVEL_FILTER_FLAGS", pack=False, align=None),
    "DSPROPERTYPAGEINFO": SimStruct(OrderedDict((("offsetString", SimTypeInt(signed=False, label="UInt32")),)), name="DSPROPERTYPAGEINFO", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE": SimTypeInt(signed=False, label="DSPROPERTY_DIRECTSOUNDDEVICE"),
    "DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_1_DATA": SimStruct(OrderedDict((("DeviceId", SimTypeBottom(label="Guid")), ("DescriptionA", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("DescriptionW", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ModuleA", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("ModuleW", SimTypeArray(SimTypeChar(label="Char"), 260)), ("Type", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_TYPE")), ("DataFlow", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_DATAFLOW")), ("WaveDeviceId", SimTypeInt(signed=False, label="UInt32")), ("Devnode", SimTypeInt(signed=False, label="UInt32")),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_1_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_A_DATA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_TYPE")), ("DataFlow", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_DATAFLOW")), ("DeviceId", SimTypeBottom(label="Guid")), ("Description", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Module", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Interface", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("WaveDeviceId", SimTypeInt(signed=False, label="UInt32")),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_A_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_W_DATA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_TYPE")), ("DataFlow", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_DATAFLOW")), ("DeviceId", SimTypeBottom(label="Guid")), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Module", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Interface", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("WaveDeviceId", SimTypeInt(signed=False, label="UInt32")),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_W_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_1_DATA": SimStruct(OrderedDict((("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_1_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_1_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_A_DATA": SimStruct(OrderedDict((("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_A_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_A_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_W_DATA": SimStruct(OrderedDict((("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_W_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_W_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_WAVEDEVICEMAPPING_A_DATA": SimStruct(OrderedDict((("DeviceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DataFlow", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_DATAFLOW")), ("DeviceId", SimTypeBottom(label="Guid")),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_WAVEDEVICEMAPPING_A_DATA", pack=False, align=None),
    "DSPROPERTY_DIRECTSOUNDDEVICE_WAVEDEVICEMAPPING_W_DATA": SimStruct(OrderedDict((("DeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DataFlow", SimTypeInt(signed=False, label="DIRECTSOUNDDEVICE_DATAFLOW")), ("DeviceId", SimTypeBottom(label="Guid")),)), name="DSPROPERTY_DIRECTSOUNDDEVICE_WAVEDEVICEMAPPING_W_DATA", pack=False, align=None),
    "DSQUERYCLASSLIST": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cClasses", SimTypeInt(signed=True, label="Int32")), ("offsetClass", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DSQUERYCLASSLIST", pack=False, align=None),
    "DSQUERYINITPARAMS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pDefaultScope", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDefaultSaveLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DSQUERYINITPARAMS", pack=False, align=None),
    "DSQUERYPARAMS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("offsetQuery", SimTypeInt(signed=True, label="Int32")), ("iColumns", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("aColumns", SimTypePointer(SimTypeRef("DSCOLUMN", SimStruct), offset=0)),)), name="DSQUERYPARAMS", pack=False, align=None),
    "DSREG_JOIN_INFO": SimStruct(OrderedDict((("joinType", SimTypeInt(signed=False, label="DSREG_JOIN_TYPE")), ("pJoinCertificate", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("pszDeviceId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszIdpDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszTenantId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszJoinUserEmail", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszTenantDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszMdmEnrollmentUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszMdmTermsOfUseUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszMdmComplianceUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUserSettingSyncUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserInfo", SimTypePointer(SimTypeRef("DSREG_USER_INFO", SimStruct), offset=0)),)), name="DSREG_JOIN_INFO", pack=False, align=None),
    "DSREG_JOIN_TYPE": SimTypeInt(signed=False, label="DSREG_JOIN_TYPE"),
    "DSREG_USER_INFO": SimStruct(OrderedDict((("pszUserEmail", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUserKeyId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUserKeyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DSREG_USER_INFO", pack=False, align=None),
    "DSROLE_MACHINE_ROLE": SimTypeInt(signed=False, label="DSROLE_MACHINE_ROLE"),
    "DSROLE_OPERATION_STATE": SimTypeInt(signed=False, label="DSROLE_OPERATION_STATE"),
    "DSROLE_OPERATION_STATE_INFO": SimStruct(OrderedDict((("OperationState", SimTypeInt(signed=False, label="DSROLE_OPERATION_STATE")),)), name="DSROLE_OPERATION_STATE_INFO", pack=False, align=None),
    "DSROLE_PRIMARY_DOMAIN_INFO_BASIC": SimStruct(OrderedDict((("MachineRole", SimTypeInt(signed=False, label="DSROLE_MACHINE_ROLE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DomainNameFlat", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DomainNameDns", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DomainForestName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DomainGuid", SimTypeBottom(label="Guid")),)), name="DSROLE_PRIMARY_DOMAIN_INFO_BASIC", pack=False, align=None),
    "DSROLE_PRIMARY_DOMAIN_INFO_LEVEL": SimTypeInt(signed=False, label="DSROLE_PRIMARY_DOMAIN_INFO_LEVEL"),
    "DSROLE_SERVER_STATE": SimTypeInt(signed=False, label="DSROLE_SERVER_STATE"),
    "DSROLE_UPGRADE_STATUS_INFO": SimStruct(OrderedDict((("OperationState", SimTypeInt(signed=False, label="UInt32")), ("PreviousServerState", SimTypeInt(signed=False, label="DSROLE_SERVER_STATE")),)), name="DSROLE_UPGRADE_STATUS_INFO", pack=False, align=None),
    "DSSD_POWER_STATE_DESCRIPTOR": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="DSSD_POWER_STATE_DESCRIPTOR", pack=False, align=None),
    "DSSSEED": SimStruct(OrderedDict((("counter", SimTypeInt(signed=False, label="UInt32")), ("seed", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="DSSSEED", pack=False, align=None),
    "DS_DOMAIN_CONTROLLER_INFO_1A": SimStruct(OrderedDict((("NetbiosName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SiteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ComputerObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fIsPdc", SimTypeInt(signed=True, label="Int32")), ("fDsEnabled", SimTypeInt(signed=True, label="Int32")),)), name="DS_DOMAIN_CONTROLLER_INFO_1A", pack=False, align=None),
    "DS_DOMAIN_CONTROLLER_INFO_1W": SimStruct(OrderedDict((("NetbiosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ComputerObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServerObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fIsPdc", SimTypeInt(signed=True, label="Int32")), ("fDsEnabled", SimTypeInt(signed=True, label="Int32")),)), name="DS_DOMAIN_CONTROLLER_INFO_1W", pack=False, align=None),
    "DS_DOMAIN_CONTROLLER_INFO_2A": SimStruct(OrderedDict((("NetbiosName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SiteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SiteObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ComputerObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NtdsDsaObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fIsPdc", SimTypeInt(signed=True, label="Int32")), ("fDsEnabled", SimTypeInt(signed=True, label="Int32")), ("fIsGc", SimTypeInt(signed=True, label="Int32")), ("SiteObjectGuid", SimTypeBottom(label="Guid")), ("ComputerObjectGuid", SimTypeBottom(label="Guid")), ("ServerObjectGuid", SimTypeBottom(label="Guid")), ("NtdsDsaObjectGuid", SimTypeBottom(label="Guid")),)), name="DS_DOMAIN_CONTROLLER_INFO_2A", pack=False, align=None),
    "DS_DOMAIN_CONTROLLER_INFO_2W": SimStruct(OrderedDict((("NetbiosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SiteObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ComputerObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServerObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NtdsDsaObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fIsPdc", SimTypeInt(signed=True, label="Int32")), ("fDsEnabled", SimTypeInt(signed=True, label="Int32")), ("fIsGc", SimTypeInt(signed=True, label="Int32")), ("SiteObjectGuid", SimTypeBottom(label="Guid")), ("ComputerObjectGuid", SimTypeBottom(label="Guid")), ("ServerObjectGuid", SimTypeBottom(label="Guid")), ("NtdsDsaObjectGuid", SimTypeBottom(label="Guid")),)), name="DS_DOMAIN_CONTROLLER_INFO_2W", pack=False, align=None),
    "DS_DOMAIN_CONTROLLER_INFO_3A": SimStruct(OrderedDict((("NetbiosName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SiteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SiteObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ComputerObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NtdsDsaObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fIsPdc", SimTypeInt(signed=True, label="Int32")), ("fDsEnabled", SimTypeInt(signed=True, label="Int32")), ("fIsGc", SimTypeInt(signed=True, label="Int32")), ("fIsRodc", SimTypeInt(signed=True, label="Int32")), ("SiteObjectGuid", SimTypeBottom(label="Guid")), ("ComputerObjectGuid", SimTypeBottom(label="Guid")), ("ServerObjectGuid", SimTypeBottom(label="Guid")), ("NtdsDsaObjectGuid", SimTypeBottom(label="Guid")),)), name="DS_DOMAIN_CONTROLLER_INFO_3A", pack=False, align=None),
    "DS_DOMAIN_CONTROLLER_INFO_3W": SimStruct(OrderedDict((("NetbiosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SiteObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ComputerObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServerObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NtdsDsaObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fIsPdc", SimTypeInt(signed=True, label="Int32")), ("fDsEnabled", SimTypeInt(signed=True, label="Int32")), ("fIsGc", SimTypeInt(signed=True, label="Int32")), ("fIsRodc", SimTypeInt(signed=True, label="Int32")), ("SiteObjectGuid", SimTypeBottom(label="Guid")), ("ComputerObjectGuid", SimTypeBottom(label="Guid")), ("ServerObjectGuid", SimTypeBottom(label="Guid")), ("NtdsDsaObjectGuid", SimTypeBottom(label="Guid")),)), name="DS_DOMAIN_CONTROLLER_INFO_3W", pack=False, align=None),
    "DS_DOMAIN_TRUSTSA": SimStruct(OrderedDict((("NetbiosDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DnsDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ParentIndex", SimTypeInt(signed=False, label="UInt32")), ("TrustType", SimTypeInt(signed=False, label="UInt32")), ("TrustAttributes", SimTypeInt(signed=False, label="UInt32")), ("DomainSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DomainGuid", SimTypeBottom(label="Guid")),)), name="DS_DOMAIN_TRUSTSA", pack=False, align=None),
    "DS_DOMAIN_TRUSTSW": SimStruct(OrderedDict((("NetbiosDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DnsDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ParentIndex", SimTypeInt(signed=False, label="UInt32")), ("TrustType", SimTypeInt(signed=False, label="UInt32")), ("TrustAttributes", SimTypeInt(signed=False, label="UInt32")), ("DomainSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DomainGuid", SimTypeBottom(label="Guid")),)), name="DS_DOMAIN_TRUSTSW", pack=False, align=None),
    "DS_KCC_TASKID": SimTypeInt(signed=False, label="DS_KCC_TASKID"),
    "DS_MANGLE_FOR": SimTypeInt(signed=False, label="DS_MANGLE_FOR"),
    "DS_NAME_ERROR": SimTypeInt(signed=False, label="DS_NAME_ERROR"),
    "DS_NAME_FLAGS": SimTypeInt(signed=False, label="DS_NAME_FLAGS"),
    "DS_NAME_FORMAT": SimTypeInt(signed=False, label="DS_NAME_FORMAT"),
    "DS_NAME_RESULTA": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("rItems", SimTypePointer(SimTypeRef("DS_NAME_RESULT_ITEMA", SimStruct), offset=0)),)), name="DS_NAME_RESULTA", pack=False, align=None),
    "DS_NAME_RESULTW": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("rItems", SimTypePointer(SimTypeRef("DS_NAME_RESULT_ITEMW", SimStruct), offset=0)),)), name="DS_NAME_RESULTW", pack=False, align=None),
    "DS_NAME_RESULT_ITEMA": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="UInt32")), ("pDomain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DS_NAME_RESULT_ITEMA", pack=False, align=None),
    "DS_NAME_RESULT_ITEMW": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="UInt32")), ("pDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DS_NAME_RESULT_ITEMW", pack=False, align=None),
    "DS_REPL_ATTR_META_DATA": SimStruct(OrderedDict((("pszAttributeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")),)), name="DS_REPL_ATTR_META_DATA", pack=False, align=None),
    "DS_REPL_ATTR_META_DATA_2": SimStruct(OrderedDict((("pszAttributeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")), ("pszLastOriginatingDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DS_REPL_ATTR_META_DATA_2", pack=False, align=None),
    "DS_REPL_ATTR_META_DATA_BLOB": SimStruct(OrderedDict((("oszAttributeName", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")), ("oszLastOriginatingDsaDN", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_ATTR_META_DATA_BLOB", pack=False, align=None),
    "DS_REPL_ATTR_VALUE_META_DATA": SimStruct(OrderedDict((("cNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwEnumerationContext", SimTypeInt(signed=False, label="UInt32")), ("rgMetaData", SimTypePointer(SimTypeRef("DS_REPL_VALUE_META_DATA", SimStruct), offset=0)),)), name="DS_REPL_ATTR_VALUE_META_DATA", pack=False, align=None),
    "DS_REPL_ATTR_VALUE_META_DATA_2": SimStruct(OrderedDict((("cNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwEnumerationContext", SimTypeInt(signed=False, label="UInt32")), ("rgMetaData", SimTypePointer(SimTypeRef("DS_REPL_VALUE_META_DATA_2", SimStruct), offset=0)),)), name="DS_REPL_ATTR_VALUE_META_DATA_2", pack=False, align=None),
    "DS_REPL_ATTR_VALUE_META_DATA_EXT": SimStruct(OrderedDict((("cNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwEnumerationContext", SimTypeInt(signed=False, label="UInt32")), ("rgMetaData", SimTypePointer(SimTypeRef("DS_REPL_VALUE_META_DATA_EXT", SimStruct), offset=0)),)), name="DS_REPL_ATTR_VALUE_META_DATA_EXT", pack=False, align=None),
    "DS_REPL_CURSOR": SimStruct(OrderedDict((("uuidSourceDsaInvocationID", SimTypeBottom(label="Guid")), ("usnAttributeFilter", SimTypeLongLong(signed=True, label="Int64")),)), name="DS_REPL_CURSOR", pack=False, align=None),
    "DS_REPL_CURSORS": SimStruct(OrderedDict((("cNumCursors", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("rgCursor", SimTypePointer(SimTypeRef("DS_REPL_CURSOR", SimStruct), offset=0)),)), name="DS_REPL_CURSORS", pack=False, align=None),
    "DS_REPL_CURSORS_2": SimStruct(OrderedDict((("cNumCursors", SimTypeInt(signed=False, label="UInt32")), ("dwEnumerationContext", SimTypeInt(signed=False, label="UInt32")), ("rgCursor", SimTypePointer(SimTypeRef("DS_REPL_CURSOR_2", SimStruct), offset=0)),)), name="DS_REPL_CURSORS_2", pack=False, align=None),
    "DS_REPL_CURSORS_3W": SimStruct(OrderedDict((("cNumCursors", SimTypeInt(signed=False, label="UInt32")), ("dwEnumerationContext", SimTypeInt(signed=False, label="UInt32")), ("rgCursor", SimTypePointer(SimTypeRef("DS_REPL_CURSOR_3W", SimStruct), offset=0)),)), name="DS_REPL_CURSORS_3W", pack=False, align=None),
    "DS_REPL_CURSOR_2": SimStruct(OrderedDict((("uuidSourceDsaInvocationID", SimTypeBottom(label="Guid")), ("usnAttributeFilter", SimTypeLongLong(signed=True, label="Int64")), ("ftimeLastSyncSuccess", SimTypeRef("FILETIME", SimStruct)),)), name="DS_REPL_CURSOR_2", pack=False, align=None),
    "DS_REPL_CURSOR_3W": SimStruct(OrderedDict((("uuidSourceDsaInvocationID", SimTypeBottom(label="Guid")), ("usnAttributeFilter", SimTypeLongLong(signed=True, label="Int64")), ("ftimeLastSyncSuccess", SimTypeRef("FILETIME", SimStruct)), ("pszSourceDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DS_REPL_CURSOR_3W", pack=False, align=None),
    "DS_REPL_CURSOR_BLOB": SimStruct(OrderedDict((("uuidSourceDsaInvocationID", SimTypeBottom(label="Guid")), ("usnAttributeFilter", SimTypeLongLong(signed=True, label="Int64")), ("ftimeLastSyncSuccess", SimTypeRef("FILETIME", SimStruct)), ("oszSourceDsaDN", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_CURSOR_BLOB", pack=False, align=None),
    "DS_REPL_INFO_TYPE": SimTypeInt(signed=False, label="DS_REPL_INFO_TYPE"),
    "DS_REPL_KCC_DSA_FAILURESW": SimStruct(OrderedDict((("cNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("rgDsaFailure", SimTypePointer(SimTypeRef("DS_REPL_KCC_DSA_FAILUREW", SimStruct), offset=0)),)), name="DS_REPL_KCC_DSA_FAILURESW", pack=False, align=None),
    "DS_REPL_KCC_DSA_FAILUREW": SimStruct(OrderedDict((("pszDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("uuidDsaObjGuid", SimTypeBottom(label="Guid")), ("ftimeFirstFailure", SimTypeRef("FILETIME", SimStruct)), ("cNumFailures", SimTypeInt(signed=False, label="UInt32")), ("dwLastResult", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_KCC_DSA_FAILUREW", pack=False, align=None),
    "DS_REPL_KCC_DSA_FAILUREW_BLOB": SimStruct(OrderedDict((("oszDsaDN", SimTypeInt(signed=False, label="UInt32")), ("uuidDsaObjGuid", SimTypeBottom(label="Guid")), ("ftimeFirstFailure", SimTypeRef("FILETIME", SimStruct)), ("cNumFailures", SimTypeInt(signed=False, label="UInt32")), ("dwLastResult", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_KCC_DSA_FAILUREW_BLOB", pack=False, align=None),
    "DS_REPL_NEIGHBORSW": SimStruct(OrderedDict((("cNumNeighbors", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("rgNeighbor", SimTypePointer(SimTypeRef("DS_REPL_NEIGHBORW", SimStruct), offset=0)),)), name="DS_REPL_NEIGHBORSW", pack=False, align=None),
    "DS_REPL_NEIGHBORW": SimStruct(OrderedDict((("pszNamingContext", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSourceDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSourceDsaAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszAsyncIntersiteTransportDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwReplicaFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("uuidNamingContextObjGuid", SimTypeBottom(label="Guid")), ("uuidSourceDsaObjGuid", SimTypeBottom(label="Guid")), ("uuidSourceDsaInvocationID", SimTypeBottom(label="Guid")), ("uuidAsyncIntersiteTransportObjGuid", SimTypeBottom(label="Guid")), ("usnLastObjChangeSynced", SimTypeLongLong(signed=True, label="Int64")), ("usnAttributeFilter", SimTypeLongLong(signed=True, label="Int64")), ("ftimeLastSyncSuccess", SimTypeRef("FILETIME", SimStruct)), ("ftimeLastSyncAttempt", SimTypeRef("FILETIME", SimStruct)), ("dwLastSyncResult", SimTypeInt(signed=False, label="UInt32")), ("cNumConsecutiveSyncFailures", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_NEIGHBORW", pack=False, align=None),
    "DS_REPL_NEIGHBORW_BLOB": SimStruct(OrderedDict((("oszNamingContext", SimTypeInt(signed=False, label="UInt32")), ("oszSourceDsaDN", SimTypeInt(signed=False, label="UInt32")), ("oszSourceDsaAddress", SimTypeInt(signed=False, label="UInt32")), ("oszAsyncIntersiteTransportDN", SimTypeInt(signed=False, label="UInt32")), ("dwReplicaFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("uuidNamingContextObjGuid", SimTypeBottom(label="Guid")), ("uuidSourceDsaObjGuid", SimTypeBottom(label="Guid")), ("uuidSourceDsaInvocationID", SimTypeBottom(label="Guid")), ("uuidAsyncIntersiteTransportObjGuid", SimTypeBottom(label="Guid")), ("usnLastObjChangeSynced", SimTypeLongLong(signed=True, label="Int64")), ("usnAttributeFilter", SimTypeLongLong(signed=True, label="Int64")), ("ftimeLastSyncSuccess", SimTypeRef("FILETIME", SimStruct)), ("ftimeLastSyncAttempt", SimTypeRef("FILETIME", SimStruct)), ("dwLastSyncResult", SimTypeInt(signed=False, label="UInt32")), ("cNumConsecutiveSyncFailures", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_NEIGHBORW_BLOB", pack=False, align=None),
    "DS_REPL_OBJ_META_DATA": SimStruct(OrderedDict((("cNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("rgMetaData", SimTypePointer(SimTypeRef("DS_REPL_ATTR_META_DATA", SimStruct), offset=0)),)), name="DS_REPL_OBJ_META_DATA", pack=False, align=None),
    "DS_REPL_OBJ_META_DATA_2": SimStruct(OrderedDict((("cNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("rgMetaData", SimTypePointer(SimTypeRef("DS_REPL_ATTR_META_DATA_2", SimStruct), offset=0)),)), name="DS_REPL_OBJ_META_DATA_2", pack=False, align=None),
    "DS_REPL_OPW": SimStruct(OrderedDict((("ftimeEnqueued", SimTypeRef("FILETIME", SimStruct)), ("ulSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("ulPriority", SimTypeInt(signed=False, label="UInt32")), ("OpType", SimTypeInt(signed=False, label="DS_REPL_OP_TYPE")), ("ulOptions", SimTypeInt(signed=False, label="UInt32")), ("pszNamingContext", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDsaAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("uuidNamingContextObjGuid", SimTypeBottom(label="Guid")), ("uuidDsaObjGuid", SimTypeBottom(label="Guid")),)), name="DS_REPL_OPW", pack=False, align=None),
    "DS_REPL_OPW_BLOB": SimStruct(OrderedDict((("ftimeEnqueued", SimTypeRef("FILETIME", SimStruct)), ("ulSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("ulPriority", SimTypeInt(signed=False, label="UInt32")), ("OpType", SimTypeInt(signed=False, label="DS_REPL_OP_TYPE")), ("ulOptions", SimTypeInt(signed=False, label="UInt32")), ("oszNamingContext", SimTypeInt(signed=False, label="UInt32")), ("oszDsaDN", SimTypeInt(signed=False, label="UInt32")), ("oszDsaAddress", SimTypeInt(signed=False, label="UInt32")), ("uuidNamingContextObjGuid", SimTypeBottom(label="Guid")), ("uuidDsaObjGuid", SimTypeBottom(label="Guid")),)), name="DS_REPL_OPW_BLOB", pack=False, align=None),
    "DS_REPL_OP_TYPE": SimTypeInt(signed=False, label="DS_REPL_OP_TYPE"),
    "DS_REPL_PENDING_OPSW": SimStruct(OrderedDict((("ftimeCurrentOpStarted", SimTypeRef("FILETIME", SimStruct)), ("cNumPendingOps", SimTypeInt(signed=False, label="UInt32")), ("rgPendingOp", SimTypePointer(SimTypeRef("DS_REPL_OPW", SimStruct), offset=0)),)), name="DS_REPL_PENDING_OPSW", pack=False, align=None),
    "DS_REPL_QUEUE_STATISTICSW": SimStruct(OrderedDict((("ftimeCurrentOpStarted", SimTypeRef("FILETIME", SimStruct)), ("cNumPendingOps", SimTypeInt(signed=False, label="UInt32")), ("ftimeOldestSync", SimTypeRef("FILETIME", SimStruct)), ("ftimeOldestAdd", SimTypeRef("FILETIME", SimStruct)), ("ftimeOldestMod", SimTypeRef("FILETIME", SimStruct)), ("ftimeOldestDel", SimTypeRef("FILETIME", SimStruct)), ("ftimeOldestUpdRefs", SimTypeRef("FILETIME", SimStruct)),)), name="DS_REPL_QUEUE_STATISTICSW", pack=False, align=None),
    "DS_REPL_VALUE_META_DATA": SimStruct(OrderedDict((("pszAttributeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectDn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ftimeDeleted", SimTypeRef("FILETIME", SimStruct)), ("ftimeCreated", SimTypeRef("FILETIME", SimStruct)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")),)), name="DS_REPL_VALUE_META_DATA", pack=False, align=None),
    "DS_REPL_VALUE_META_DATA_2": SimStruct(OrderedDict((("pszAttributeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectDn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ftimeDeleted", SimTypeRef("FILETIME", SimStruct)), ("ftimeCreated", SimTypeRef("FILETIME", SimStruct)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")), ("pszLastOriginatingDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DS_REPL_VALUE_META_DATA_2", pack=False, align=None),
    "DS_REPL_VALUE_META_DATA_BLOB": SimStruct(OrderedDict((("oszAttributeName", SimTypeInt(signed=False, label="UInt32")), ("oszObjectDn", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("obData", SimTypeInt(signed=False, label="UInt32")), ("ftimeDeleted", SimTypeRef("FILETIME", SimStruct)), ("ftimeCreated", SimTypeRef("FILETIME", SimStruct)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")), ("oszLastOriginatingDsaDN", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_VALUE_META_DATA_BLOB", pack=False, align=None),
    "DS_REPL_VALUE_META_DATA_BLOB_EXT": SimStruct(OrderedDict((("oszAttributeName", SimTypeInt(signed=False, label="UInt32")), ("oszObjectDn", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("obData", SimTypeInt(signed=False, label="UInt32")), ("ftimeDeleted", SimTypeRef("FILETIME", SimStruct)), ("ftimeCreated", SimTypeRef("FILETIME", SimStruct)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")), ("oszLastOriginatingDsaDN", SimTypeInt(signed=False, label="UInt32")), ("dwUserIdentifier", SimTypeInt(signed=False, label="UInt32")), ("dwPriorLinkState", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentLinkState", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_VALUE_META_DATA_BLOB_EXT", pack=False, align=None),
    "DS_REPL_VALUE_META_DATA_EXT": SimStruct(OrderedDict((("pszAttributeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectDn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ftimeDeleted", SimTypeRef("FILETIME", SimStruct)), ("ftimeCreated", SimTypeRef("FILETIME", SimStruct)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ftimeLastOriginatingChange", SimTypeRef("FILETIME", SimStruct)), ("uuidLastOriginatingDsaInvocationID", SimTypeBottom(label="Guid")), ("usnOriginatingChange", SimTypeLongLong(signed=True, label="Int64")), ("usnLocalChange", SimTypeLongLong(signed=True, label="Int64")), ("pszLastOriginatingDsaDN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwUserIdentifier", SimTypeInt(signed=False, label="UInt32")), ("dwPriorLinkState", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentLinkState", SimTypeInt(signed=False, label="UInt32")),)), name="DS_REPL_VALUE_META_DATA_EXT", pack=False, align=None),
    "DS_REPSYNCALL_ERRINFOA": SimStruct(OrderedDict((("pszSvrId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("error", SimTypeInt(signed=False, label="DS_REPSYNCALL_ERROR")), ("dwWin32Err", SimTypeInt(signed=False, label="UInt32")), ("pszSrcId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DS_REPSYNCALL_ERRINFOA", pack=False, align=None),
    "DS_REPSYNCALL_ERRINFOW": SimStruct(OrderedDict((("pszSvrId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("error", SimTypeInt(signed=False, label="DS_REPSYNCALL_ERROR")), ("dwWin32Err", SimTypeInt(signed=False, label="UInt32")), ("pszSrcId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DS_REPSYNCALL_ERRINFOW", pack=False, align=None),
    "DS_REPSYNCALL_ERROR": SimTypeInt(signed=False, label="DS_REPSYNCALL_ERROR"),
    "DS_REPSYNCALL_EVENT": SimTypeInt(signed=False, label="DS_REPSYNCALL_EVENT"),
    "DS_REPSYNCALL_SYNCA": SimStruct(OrderedDict((("pszSrcId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszDstId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszNC", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pguidSrc", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pguidDst", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="DS_REPSYNCALL_SYNCA", pack=False, align=None),
    "DS_REPSYNCALL_SYNCW": SimStruct(OrderedDict((("pszSrcId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDstId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszNC", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pguidSrc", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pguidDst", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="DS_REPSYNCALL_SYNCW", pack=False, align=None),
    "DS_REPSYNCALL_UPDATEA": SimStruct(OrderedDict((("event", SimTypeInt(signed=False, label="DS_REPSYNCALL_EVENT")), ("pErrInfo", SimTypePointer(SimTypeRef("DS_REPSYNCALL_ERRINFOA", SimStruct), offset=0)), ("pSync", SimTypePointer(SimTypeRef("DS_REPSYNCALL_SYNCA", SimStruct), offset=0)),)), name="DS_REPSYNCALL_UPDATEA", pack=False, align=None),
    "DS_REPSYNCALL_UPDATEW": SimStruct(OrderedDict((("event", SimTypeInt(signed=False, label="DS_REPSYNCALL_EVENT")), ("pErrInfo", SimTypePointer(SimTypeRef("DS_REPSYNCALL_ERRINFOW", SimStruct), offset=0)), ("pSync", SimTypePointer(SimTypeRef("DS_REPSYNCALL_SYNCW", SimStruct), offset=0)),)), name="DS_REPSYNCALL_UPDATEW", pack=False, align=None),
    "DS_SCHEMA_GUID_MAPA": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("guidType", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DS_SCHEMA_GUID_MAPA", pack=False, align=None),
    "DS_SCHEMA_GUID_MAPW": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("guidType", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DS_SCHEMA_GUID_MAPW", pack=False, align=None),
    "DS_SELECTION": SimStruct(OrderedDict((("pwzName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzADsPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzUPN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pvarFetchedAttributes", SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)), ("flScopeType", SimTypeInt(signed=False, label="UInt32")),)), name="DS_SELECTION", pack=False, align=None),
    "DS_SELECTION_LIST": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("cFetchedAttributes", SimTypeInt(signed=False, label="UInt32")), ("aDsSelection", SimTypePointer(SimTypeRef("DS_SELECTION", SimStruct), offset=0)),)), name="DS_SELECTION_LIST", pack=False, align=None),
    "DS_SITE_COST_INFO": SimStruct(OrderedDict((("errorCode", SimTypeInt(signed=False, label="UInt32")), ("cost", SimTypeInt(signed=False, label="UInt32")),)), name="DS_SITE_COST_INFO", pack=False, align=None),
    "DS_SPN_NAME_TYPE": SimTypeInt(signed=False, label="DS_SPN_NAME_TYPE"),
    "DS_SPN_WRITE_OP": SimTypeInt(signed=False, label="DS_SPN_WRITE_OP"),
    "DShellFolderViewEvents": SimTypeBottom(label="DShellFolderViewEvents"),
    "DShellNameSpaceEvents": SimTypeBottom(label="DShellNameSpaceEvents"),
    "DShellWindowsEvents": SimTypeBottom(label="DShellWindowsEvents"),
    "DTBGOPTS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("rcClip", SimTypeRef("RECT", SimStruct)),)), name="DTBGOPTS", pack=False, align=None),
    "DTBLBUTTON": SimStruct(OrderedDict((("ulbLpszLabel", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulPRControl", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLBUTTON", pack=False, align=None),
    "DTBLCHECKBOX": SimStruct(OrderedDict((("ulbLpszLabel", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulPRPropertyName", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLCHECKBOX", pack=False, align=None),
    "DTBLCOMBOBOX": SimStruct(OrderedDict((("ulbLpszCharsAllowed", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulNumCharsAllowed", SimTypeInt(signed=False, label="UInt32")), ("ulPRPropertyName", SimTypeInt(signed=False, label="UInt32")), ("ulPRTableName", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLCOMBOBOX", pack=False, align=None),
    "DTBLDDLBX": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulPRDisplayProperty", SimTypeInt(signed=False, label="UInt32")), ("ulPRSetProperty", SimTypeInt(signed=False, label="UInt32")), ("ulPRTableName", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLDDLBX", pack=False, align=None),
    "DTBLEDIT": SimStruct(OrderedDict((("ulbLpszCharsAllowed", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulNumCharsAllowed", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLEDIT", pack=False, align=None),
    "DTBLGROUPBOX": SimStruct(OrderedDict((("ulbLpszLabel", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLGROUPBOX", pack=False, align=None),
    "DTBLLABEL": SimStruct(OrderedDict((("ulbLpszLabelName", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLLABEL", pack=False, align=None),
    "DTBLLBX": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulPRSetProperty", SimTypeInt(signed=False, label="UInt32")), ("ulPRTableName", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLLBX", pack=False, align=None),
    "DTBLMVDDLBX": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulMVPropTag", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLMVDDLBX", pack=False, align=None),
    "DTBLMVLISTBOX": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulMVPropTag", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLMVLISTBOX", pack=False, align=None),
    "DTBLPAGE": SimStruct(OrderedDict((("ulbLpszLabel", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulbLpszComponent", SimTypeInt(signed=False, label="UInt32")), ("ulContext", SimTypeInt(signed=False, label="UInt32")),)), name="DTBLPAGE", pack=False, align=None),
    "DTBLRADIOBUTTON": SimStruct(OrderedDict((("ulbLpszLabel", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulcButtons", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("lReturnValue", SimTypeInt(signed=True, label="Int32")),)), name="DTBLRADIOBUTTON", pack=False, align=None),
    "DTCINITIATEDRECOVERYWORK": SimTypeInt(signed=False, label="DTCINITIATEDRECOVERYWORK"),
    "DTCLUCOMPARESTATE": SimTypeInt(signed=False, label="DTCLUCOMPARESTATE"),
    "DTCLUCOMPARESTATESCONFIRMATION": SimTypeInt(signed=False, label="DTCLUCOMPARESTATESCONFIRMATION"),
    "DTCLUCOMPARESTATESERROR": SimTypeInt(signed=False, label="DTCLUCOMPARESTATESERROR"),
    "DTCLUCOMPARESTATESRESPONSE": SimTypeInt(signed=False, label="DTCLUCOMPARESTATESRESPONSE"),
    "DTCLUXLN": SimTypeInt(signed=False, label="DTCLUXLN"),
    "DTCLUXLNCONFIRMATION": SimTypeInt(signed=False, label="DTCLUXLNCONFIRMATION"),
    "DTCLUXLNERROR": SimTypeInt(signed=False, label="DTCLUXLNERROR"),
    "DTCLUXLNRESPONSE": SimTypeInt(signed=False, label="DTCLUXLNRESPONSE"),
    "DTCTL": SimStruct(OrderedDict((("ulCtlType", SimTypeInt(signed=False, label="UInt32")), ("ulCtlFlags", SimTypeInt(signed=False, label="UInt32")), ("lpbNotif", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbNotif", SimTypeInt(signed=False, label="UInt32")), ("lpszFilter", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("ulItemID", SimTypeInt(signed=False, label="UInt32")), ("ctl", SimUnion({"lpv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "lplabel": SimTypePointer(SimTypeRef("DTBLLABEL", SimStruct), offset=0), "lpedit": SimTypePointer(SimTypeRef("DTBLEDIT", SimStruct), offset=0), "lplbx": SimTypePointer(SimTypeRef("DTBLLBX", SimStruct), offset=0), "lpcombobox": SimTypePointer(SimTypeRef("DTBLCOMBOBOX", SimStruct), offset=0), "lpddlbx": SimTypePointer(SimTypeRef("DTBLDDLBX", SimStruct), offset=0), "lpcheckbox": SimTypePointer(SimTypeRef("DTBLCHECKBOX", SimStruct), offset=0), "lpgroupbox": SimTypePointer(SimTypeRef("DTBLGROUPBOX", SimStruct), offset=0), "lpbutton": SimTypePointer(SimTypeRef("DTBLBUTTON", SimStruct), offset=0), "lpradiobutton": SimTypePointer(SimTypeRef("DTBLRADIOBUTTON", SimStruct), offset=0), "lpmvlbx": SimTypePointer(SimTypeRef("DTBLMVLISTBOX", SimStruct), offset=0), "lpmvddlbx": SimTypePointer(SimTypeRef("DTBLMVDDLBX", SimStruct), offset=0), "lppage": SimTypePointer(SimTypeRef("DTBLPAGE", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="DTCTL", pack=False, align=None),
    "DTC_GET_TRANSACTION_MANAGER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszHost", "pszTmName", "rid", "dwReserved1", "wcbReserved2", "pvReserved2", "ppvObject"]), offset=0),
    "DTC_GET_TRANSACTION_MANAGER_EX_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["i_pszHost", "i_pszTmName", "i_riid", "i_grfOptions", "i_pvConfigParams", "o_ppvObject"]), offset=0),
    "DTC_GET_TRANSACTION_MANAGER_EX_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["i_pwszHost", "i_pwszTmName", "i_riid", "i_grfOptions", "i_pvConfigParams", "o_ppvObject"]), offset=0),
    "DTC_INSTALL_CLIENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["i_pszRemoteTmHostName", "i_dwProtocol", "i_dwOverwrite"]), offset=0),
    "DTC_STATUS_": SimTypeInt(signed=False, label="DTC_STATUS_"),
    "DTPAGE": SimStruct(OrderedDict((("cctl", SimTypeInt(signed=False, label="UInt32")), ("lpszResourceName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Anonymous", SimUnion({"lpszComponent": SimTypePointer(SimTypeChar(label="SByte"), offset=0), "ulItemID": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("lpctl", SimTypePointer(SimTypeRef("DTCTL", SimStruct), offset=0)),)), name="DTPAGE", pack=False, align=None),
    "DTR": SimStruct(OrderedDict((("wYear", SimTypeShort(signed=False, label="UInt16")), ("wMonth", SimTypeShort(signed=False, label="UInt16")), ("wDay", SimTypeShort(signed=False, label="UInt16")), ("wHour", SimTypeShort(signed=False, label="UInt16")), ("wMinute", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")), ("wDayOfWeek", SimTypeShort(signed=False, label="UInt16")),)), name="DTR", pack=False, align=None),
    "DTTOPTS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="DTTOPTS_FLAGS")), ("crText", SimTypeInt(signed=False, label="UInt32")), ("crBorder", SimTypeInt(signed=False, label="UInt32")), ("crShadow", SimTypeInt(signed=False, label="UInt32")), ("iTextShadowType", SimTypeInt(signed=True, label="Int32")), ("ptShadowOffset", SimTypeRef("POINT", SimStruct)), ("iBorderSize", SimTypeInt(signed=True, label="Int32")), ("iFontPropId", SimTypeInt(signed=True, label="Int32")), ("iColorPropId", SimTypeInt(signed=True, label="Int32")), ("iStateId", SimTypeInt(signed=True, label="Int32")), ("fApplyOverlay", SimTypeInt(signed=True, label="Int32")), ("iGlowSize", SimTypeInt(signed=True, label="Int32")), ("pfnDrawTextCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hdc", "pszText", "cchText", "prc", "dwFlags", "lParam"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DTTOPTS", pack=False, align=None),
    "DTTOPTS_FLAGS": SimTypeInt(signed=False, label="DTTOPTS_FLAGS"),
    "DTT_CALLBACK_PROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hdc", "pszText", "cchText", "prc", "dwFlags", "lParam"]), offset=0),
    "DUAL_OPLOCK_KEY_ECP_CONTEXT": SimStruct(OrderedDict((("ParentOplockKey", SimTypeBottom(label="Guid")), ("TargetOplockKey", SimTypeBottom(label="Guid")), ("ParentOplockKeySet", SimTypeChar(label="Byte")), ("TargetOplockKeySet", SimTypeChar(label="Byte")),)), name="DUAL_OPLOCK_KEY_ECP_CONTEXT", pack=False, align=None),
    "DUMPTYPE": SimTypeInt(signed=False, label="DUMPTYPE"),
    "DUMP_DRIVER": SimStruct(OrderedDict((("DumpDriverList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DriverName", SimTypeArray(SimTypeChar(label="Char"), 15)), ("BaseName", SimTypeArray(SimTypeChar(label="Char"), 15)),)), name="DUMP_DRIVER", pack=False, align=None),
    "DUMP_DRIVER_EX": SimStruct(OrderedDict((("DumpDriverList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DriverName", SimTypeArray(SimTypeChar(label="Char"), 15)), ("BaseName", SimTypeArray(SimTypeChar(label="Char"), 15)), ("DriverFullPath", SimTypeRef("NTSCSI_UNICODE_STRING", SimStruct)),)), name="DUMP_DRIVER_EX", pack=False, align=None),
    "DUMP_FILE_ATTRIBUTES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Attributes": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "DUMP_HEADER32": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("ValidDump", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("DirectoryTableBase", SimTypeInt(signed=False, label="UInt32")), ("PfnDataBase", SimTypeInt(signed=False, label="UInt32")), ("PsLoadedModuleList", SimTypeInt(signed=False, label="UInt32")), ("PsActiveProcessHead", SimTypeInt(signed=False, label="UInt32")), ("MachineImageType", SimTypeInt(signed=False, label="UInt32")), ("NumberProcessors", SimTypeInt(signed=False, label="UInt32")), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter1", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter2", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter3", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter4", SimTypeInt(signed=False, label="UInt32")), ("VersionUser", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("PaeEnabled", SimTypeChar(label="Byte")), ("KdSecondaryVersion", SimTypeChar(label="Byte")), ("Spare3", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("KdDebuggerDataBlock", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"PhysicalMemoryBlock": SimTypeRef("PHYSICAL_MEMORY_DESCRIPTOR32", SimStruct), "PhysicalMemoryBlockBuffer": SimTypeArray(SimTypeChar(label="Byte"), 700)}, name="<anon>", label="None")), ("ContextRecord", SimTypeArray(SimTypeChar(label="Byte"), 1200)), ("Exception", SimTypeRef("EXCEPTION_RECORD32", SimStruct)), ("Comment", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("Attributes", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Attributes": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("BootId", SimTypeInt(signed=False, label="UInt32")), ("_reserved0", SimTypeArray(SimTypeChar(label="Byte"), 1760)), ("DumpType", SimTypeInt(signed=False, label="UInt32")), ("MiniDumpFields", SimTypeInt(signed=False, label="UInt32")), ("SecondaryDataState", SimTypeInt(signed=False, label="UInt32")), ("ProductType", SimTypeInt(signed=False, label="UInt32")), ("SuiteMask", SimTypeInt(signed=False, label="UInt32")), ("WriterStatus", SimTypeInt(signed=False, label="UInt32")), ("RequiredDumpSpace", SimTypeLongLong(signed=True, label="Int64")), ("_reserved2", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("SystemUpTime", SimTypeLongLong(signed=True, label="Int64")), ("SystemTime", SimTypeLongLong(signed=True, label="Int64")), ("_reserved3", SimTypeArray(SimTypeChar(label="Byte"), 56)),)), name="DUMP_HEADER32", pack=False, align=None),
    "DUMP_HEADER64": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("ValidDump", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("DirectoryTableBase", SimTypeLongLong(signed=False, label="UInt64")), ("PfnDataBase", SimTypeLongLong(signed=False, label="UInt64")), ("PsLoadedModuleList", SimTypeLongLong(signed=False, label="UInt64")), ("PsActiveProcessHead", SimTypeLongLong(signed=False, label="UInt64")), ("MachineImageType", SimTypeInt(signed=False, label="UInt32")), ("NumberProcessors", SimTypeInt(signed=False, label="UInt32")), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter1", SimTypeLongLong(signed=False, label="UInt64")), ("BugCheckParameter2", SimTypeLongLong(signed=False, label="UInt64")), ("BugCheckParameter3", SimTypeLongLong(signed=False, label="UInt64")), ("BugCheckParameter4", SimTypeLongLong(signed=False, label="UInt64")), ("VersionUser", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("KdDebuggerDataBlock", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"PhysicalMemoryBlock": SimTypeRef("PHYSICAL_MEMORY_DESCRIPTOR64", SimStruct), "PhysicalMemoryBlockBuffer": SimTypeArray(SimTypeChar(label="Byte"), 700)}, name="<anon>", label="None")), ("ContextRecord", SimTypeArray(SimTypeChar(label="Byte"), 3000)), ("Exception", SimTypeRef("EXCEPTION_RECORD64", SimStruct)), ("DumpType", SimTypeInt(signed=False, label="UInt32")), ("RequiredDumpSpace", SimTypeLongLong(signed=True, label="Int64")), ("SystemTime", SimTypeLongLong(signed=True, label="Int64")), ("Comment", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("SystemUpTime", SimTypeLongLong(signed=True, label="Int64")), ("MiniDumpFields", SimTypeInt(signed=False, label="UInt32")), ("SecondaryDataState", SimTypeInt(signed=False, label="UInt32")), ("ProductType", SimTypeInt(signed=False, label="UInt32")), ("SuiteMask", SimTypeInt(signed=False, label="UInt32")), ("WriterStatus", SimTypeInt(signed=False, label="UInt32")), ("Unused1", SimTypeChar(label="Byte")), ("KdSecondaryVersion", SimTypeChar(label="Byte")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Attributes", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Attributes": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("BootId", SimTypeInt(signed=False, label="UInt32")), ("_reserved0", SimTypeArray(SimTypeChar(label="Byte"), 4008)),)), name="DUMP_HEADER64", pack=False, align=None),
    "DUMP_POINTERS": SimStruct(OrderedDict((("AdapterObject", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("MappedRegisterBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DumpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CommonBufferVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CommonBufferPa", SimTypeLongLong(signed=True, label="Int64")), ("CommonBufferSize", SimTypeInt(signed=False, label="UInt32")), ("AllocateCommonBuffers", SimTypeChar(label="Byte")), ("UseDiskDump", SimTypeChar(label="Byte")), ("Spare1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("DeviceObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DUMP_POINTERS", pack=False, align=None),
    "DUMP_POINTERS_EX": SimStruct(OrderedDict((("Header", SimTypeRef("DUMP_POINTERS_VERSION", SimStruct)), ("DumpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CommonBufferVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CommonBufferSize", SimTypeInt(signed=False, label="UInt32")), ("AllocateCommonBuffers", SimTypeChar(label="Byte")), ("DeviceObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DriverList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")), ("MaxDeviceDumpSectionSize", SimTypeInt(signed=False, label="UInt32")), ("MaxDeviceDumpLevel", SimTypeInt(signed=False, label="UInt32")), ("MaxTransferSize", SimTypeInt(signed=False, label="UInt32")), ("AdapterObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MappedRegisterBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DeviceReady", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DumpDevicePowerOn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0)), ("DumpDevicePowerOnContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DUMP_POINTERS_EX", pack=False, align=None),
    "DUMP_POINTERS_VERSION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="DUMP_POINTERS_VERSION", pack=False, align=None),
    "DUMP_TYPE": SimTypeInt(signed=False, label="DUMP_TYPE"),
    "DUPLICATE_CLUSTER_DATA": SimStruct(OrderedDict((("SourceLcn", SimTypeLongLong(signed=True, label="Int64")), ("TargetFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("DuplicationLimit", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DUPLICATE_CLUSTER_DATA", pack=False, align=None),
    "DUPLICATE_EXTENTS_DATA": SimStruct(OrderedDict((("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SourceFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("TargetFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")),)), name="DUPLICATE_EXTENTS_DATA", pack=False, align=None),
    "DUPLICATE_EXTENTS_DATA32": SimStruct(OrderedDict((("FileHandle", SimTypeInt(signed=False, label="UInt32")), ("SourceFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("TargetFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")),)), name="DUPLICATE_EXTENTS_DATA32", pack=False, align=None),
    "DUPLICATE_EXTENTS_DATA_EX": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SourceFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("TargetFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DUPLICATE_EXTENTS_DATA_EX", pack=False, align=None),
    "DUPLICATE_EXTENTS_DATA_EX32": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("FileHandle", SimTypeInt(signed=False, label="UInt32")), ("SourceFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("TargetFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DUPLICATE_EXTENTS_DATA_EX32", pack=False, align=None),
    "DUPLICATE_EXTENTS_STATE": SimTypeInt(signed=False, label="DUPLICATE_EXTENTS_STATE"),
    "DUPLICATE_HANDLE_OPTIONS": SimTypeInt(signed=False, label="DUPLICATE_HANDLE_OPTIONS"),
    "DVASPECT": SimTypeInt(signed=False, label="DVASPECT"),
    "DVASPECTINFO": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DVASPECTINFO", pack=False, align=None),
    "DVASPECTINFOFLAG": SimTypeInt(signed=False, label="DVASPECTINFOFLAG"),
    "DVAudInfo": SimStruct(OrderedDict((("bAudStyle", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("bAudQu", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("bNumAudPin", SimTypeChar(label="Byte")), ("wAvgSamplesPerPinPerFrm", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("wBlkMode", SimTypeShort(signed=False, label="UInt16")), ("wDIFMode", SimTypeShort(signed=False, label="UInt16")), ("wBlkDiv", SimTypeShort(signed=False, label="UInt16")),)), name="DVAudInfo", pack=False, align=None),
    "DVBScramblingControlSpanningEvent": SimStruct(OrderedDict((("ulPID", SimTypeInt(signed=False, label="UInt32")), ("fScrambled", SimTypeInt(signed=True, label="Int32")),)), name="DVBScramblingControlSpanningEvent", pack=False, align=None),
    "DVBSystemType": SimTypeInt(signed=False, label="DVBSystemType"),
    "DVB_EIT_FILTER_OPTIONS": SimStruct(OrderedDict((("fSpecifySegment", SimTypeInt(signed=True, label="Int32")), ("bSegment", SimTypeChar(label="Byte")),)), name="DVB_EIT_FILTER_OPTIONS", pack=False, align=None),
    "DVB_STRCONV_MODE": SimTypeInt(signed=False, label="DVB_STRCONV_MODE"),
    "DVDFilterState": SimTypeInt(signed=False, label="DVDFilterState"),
    "DVDMenuIDConstants": SimTypeInt(signed=False, label="DVDMenuIDConstants"),
    "DVDSPExt": SimTypeInt(signed=False, label="DVDSPExt"),
    "DVDTextStringType": SimTypeInt(signed=False, label="DVDTextStringType"),
    "DVD_ATR": SimStruct(OrderedDict((("ulCAT", SimTypeInt(signed=False, label="UInt32")), ("pbATRI", SimTypeArray(SimTypeChar(label="Byte"), 768)),)), name="DVD_ATR", pack=False, align=None),
    "DVD_AUDIO_APPMODE": SimTypeInt(signed=False, label="DVD_AUDIO_APPMODE"),
    "DVD_AUDIO_FORMAT": SimTypeInt(signed=False, label="DVD_AUDIO_FORMAT"),
    "DVD_AUDIO_LANG_EXT": SimTypeInt(signed=False, label="DVD_AUDIO_LANG_EXT"),
    "DVD_AudioAttributes": SimStruct(OrderedDict((("AppMode", SimTypeInt(signed=False, label="DVD_AUDIO_APPMODE")), ("AppModeData", SimTypeChar(label="Byte")), ("AudioFormat", SimTypeInt(signed=False, label="DVD_AUDIO_FORMAT")), ("Language", SimTypeInt(signed=False, label="UInt32")), ("LanguageExtension", SimTypeInt(signed=False, label="DVD_AUDIO_LANG_EXT")), ("fHasMultichannelInfo", SimTypeInt(signed=True, label="Int32")), ("dwFrequency", SimTypeInt(signed=False, label="UInt32")), ("bQuantization", SimTypeChar(label="Byte")), ("bNumberOfChannels", SimTypeChar(label="Byte")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="DVD_AudioAttributes", pack=False, align=None),
    "DVD_CMD_FLAGS": SimTypeInt(signed=False, label="DVD_CMD_FLAGS"),
    "DVD_DECODER_CAPS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwAudioCaps", SimTypeInt(signed=False, label="UInt32")), ("dFwdMaxRateVideo", SimTypeFloat(size=64)), ("dFwdMaxRateAudio", SimTypeFloat(size=64)), ("dFwdMaxRateSP", SimTypeFloat(size=64)), ("dBwdMaxRateVideo", SimTypeFloat(size=64)), ("dBwdMaxRateAudio", SimTypeFloat(size=64)), ("dBwdMaxRateSP", SimTypeFloat(size=64)), ("dwRes1", SimTypeInt(signed=False, label="UInt32")), ("dwRes2", SimTypeInt(signed=False, label="UInt32")), ("dwRes3", SimTypeInt(signed=False, label="UInt32")), ("dwRes4", SimTypeInt(signed=False, label="UInt32")),)), name="DVD_DECODER_CAPS", pack=False, align=None),
    "DVD_DISC_SIDE": SimTypeInt(signed=False, label="DVD_DISC_SIDE"),
    "DVD_DOMAIN": SimTypeInt(signed=False, label="DVD_DOMAIN"),
    "DVD_ERROR": SimTypeInt(signed=False, label="DVD_ERROR"),
    "DVD_FRAMERATE": SimTypeInt(signed=False, label="DVD_FRAMERATE"),
    "DVD_HMSF_TIMECODE": SimStruct(OrderedDict((("bHours", SimTypeChar(label="Byte")), ("bMinutes", SimTypeChar(label="Byte")), ("bSeconds", SimTypeChar(label="Byte")), ("bFrames", SimTypeChar(label="Byte")),)), name="DVD_HMSF_TIMECODE", pack=False, align=None),
    "DVD_KARAOKE_ASSIGNMENT": SimTypeInt(signed=False, label="DVD_KARAOKE_ASSIGNMENT"),
    "DVD_KARAOKE_CONTENTS": SimTypeInt(signed=False, label="DVD_KARAOKE_CONTENTS"),
    "DVD_KARAOKE_DOWNMIX": SimTypeInt(signed=False, label="DVD_KARAOKE_DOWNMIX"),
    "DVD_KaraokeAttributes": SimStruct(OrderedDict((("bVersion", SimTypeChar(label="Byte")), ("fMasterOfCeremoniesInGuideVocal1", SimTypeInt(signed=True, label="Int32")), ("fDuet", SimTypeInt(signed=True, label="Int32")), ("ChannelAssignment", SimTypeInt(signed=False, label="DVD_KARAOKE_ASSIGNMENT")), ("wChannelContents", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)),)), name="DVD_KaraokeAttributes", pack=False, align=None),
    "DVD_MENU_ID": SimTypeInt(signed=False, label="DVD_MENU_ID"),
    "DVD_MUA_Coeff": SimStruct(OrderedDict((("log2_alpha", SimTypeFloat(size=64)), ("log2_beta", SimTypeFloat(size=64)),)), name="DVD_MUA_Coeff", pack=False, align=None),
    "DVD_MUA_MixingInfo": SimStruct(OrderedDict((("fMixTo0", SimTypeInt(signed=True, label="Int32")), ("fMixTo1", SimTypeInt(signed=True, label="Int32")), ("fMix0InPhase", SimTypeInt(signed=True, label="Int32")), ("fMix1InPhase", SimTypeInt(signed=True, label="Int32")), ("dwSpeakerPosition", SimTypeInt(signed=False, label="UInt32")),)), name="DVD_MUA_MixingInfo", pack=False, align=None),
    "DVD_MenuAttributes": SimStruct(OrderedDict((("fCompatibleRegion", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 8)), ("VideoAttributes", SimTypeRef("DVD_VideoAttributes", SimStruct)), ("fAudioPresent", SimTypeInt(signed=True, label="Int32")), ("AudioAttributes", SimTypeRef("DVD_AudioAttributes", SimStruct)), ("fSubpicturePresent", SimTypeInt(signed=True, label="Int32")), ("SubpictureAttributes", SimTypeRef("DVD_SubpictureAttributes", SimStruct)),)), name="DVD_MenuAttributes", pack=False, align=None),
    "DVD_MultichannelAudioAttributes": SimStruct(OrderedDict((("Info", SimTypeArray(SimTypeRef("DVD_MUA_MixingInfo", SimStruct), 8)), ("Coeff", SimTypeArray(SimTypeRef("DVD_MUA_Coeff", SimStruct), 8)),)), name="DVD_MultichannelAudioAttributes", pack=False, align=None),
    "DVD_NavCmdType": SimTypeInt(signed=False, label="DVD_NavCmdType"),
    "DVD_OPTION_FLAG": SimTypeInt(signed=False, label="DVD_OPTION_FLAG"),
    "DVD_PARENTAL_LEVEL": SimTypeInt(signed=False, label="DVD_PARENTAL_LEVEL"),
    "DVD_PB_STOPPED": SimTypeInt(signed=False, label="DVD_PB_STOPPED"),
    "DVD_PLAYBACK_LOCATION": SimStruct(OrderedDict((("TitleNum", SimTypeInt(signed=False, label="UInt32")), ("ChapterNum", SimTypeInt(signed=False, label="UInt32")), ("TimeCode", SimTypeInt(signed=False, label="UInt32")),)), name="DVD_PLAYBACK_LOCATION", pack=False, align=None),
    "DVD_PLAYBACK_LOCATION2": SimStruct(OrderedDict((("TitleNum", SimTypeInt(signed=False, label="UInt32")), ("ChapterNum", SimTypeInt(signed=False, label="UInt32")), ("TimeCode", SimTypeRef("DVD_HMSF_TIMECODE", SimStruct)), ("TimeCodeFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DVD_PLAYBACK_LOCATION2", pack=False, align=None),
    "DVD_PLAY_DIRECTION": SimTypeInt(signed=False, label="DVD_PLAY_DIRECTION"),
    "DVD_PREFERRED_DISPLAY_MODE": SimTypeInt(signed=False, label="DVD_PREFERRED_DISPLAY_MODE"),
    "DVD_REGION": SimStruct(OrderedDict((("CopySystem", SimTypeChar(label="Byte")), ("RegionData", SimTypeChar(label="Byte")), ("SystemRegion", SimTypeChar(label="Byte")), ("ResetCount", SimTypeChar(label="Byte")),)), name="DVD_REGION", pack=False, align=None),
    "DVD_RELATIVE_BUTTON": SimTypeInt(signed=False, label="DVD_RELATIVE_BUTTON"),
    "DVD_SUBPICTURE_CODING": SimTypeInt(signed=False, label="DVD_SUBPICTURE_CODING"),
    "DVD_SUBPICTURE_LANG_EXT": SimTypeInt(signed=False, label="DVD_SUBPICTURE_LANG_EXT"),
    "DVD_SUBPICTURE_TYPE": SimTypeInt(signed=False, label="DVD_SUBPICTURE_TYPE"),
    "DVD_SubpictureAttributes": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DVD_SUBPICTURE_TYPE")), ("CodingMode", SimTypeInt(signed=False, label="DVD_SUBPICTURE_CODING")), ("Language", SimTypeInt(signed=False, label="UInt32")), ("LanguageExtension", SimTypeInt(signed=False, label="DVD_SUBPICTURE_LANG_EXT")),)), name="DVD_SubpictureAttributes", pack=False, align=None),
    "DVD_TIMECODE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="DVD_TIMECODE", pack=False, align=None),
    "DVD_TIMECODE_FLAGS": SimTypeInt(signed=False, label="DVD_TIMECODE_FLAGS"),
    "DVD_TITLE_APPMODE": SimTypeInt(signed=False, label="DVD_TITLE_APPMODE"),
    "DVD_TextCharSet": SimTypeInt(signed=False, label="DVD_TextCharSet"),
    "DVD_TextStringType": SimTypeInt(signed=False, label="DVD_TextStringType"),
    "DVD_TitleAttributes": SimStruct(OrderedDict((("Anonymous", SimUnion({"AppMode": SimTypeInt(signed=False, label="DVD_TITLE_APPMODE"), "TitleLength": SimTypeRef("DVD_HMSF_TIMECODE", SimStruct)}, name="<anon>", label="None")), ("VideoAttributes", SimTypeRef("DVD_VideoAttributes", SimStruct)), ("ulNumberOfAudioStreams", SimTypeInt(signed=False, label="UInt32")), ("AudioAttributes", SimTypeArray(SimTypeRef("DVD_AudioAttributes", SimStruct), 8)), ("MultichannelAudioAttributes", SimTypeArray(SimTypeRef("DVD_MultichannelAudioAttributes", SimStruct), 8)), ("ulNumberOfSubpictureStreams", SimTypeInt(signed=False, label="UInt32")), ("SubpictureAttributes", SimTypeArray(SimTypeRef("DVD_SubpictureAttributes", SimStruct), 32)),)), name="DVD_TitleAttributes", pack=False, align=None),
    "DVD_VIDEO_COMPRESSION": SimTypeInt(signed=False, label="DVD_VIDEO_COMPRESSION"),
    "DVD_VideoAttributes": SimStruct(OrderedDict((("fPanscanPermitted", SimTypeInt(signed=True, label="Int32")), ("fLetterboxPermitted", SimTypeInt(signed=True, label="Int32")), ("ulAspectX", SimTypeInt(signed=False, label="UInt32")), ("ulAspectY", SimTypeInt(signed=False, label="UInt32")), ("ulFrameRate", SimTypeInt(signed=False, label="UInt32")), ("ulFrameHeight", SimTypeInt(signed=False, label="UInt32")), ("Compression", SimTypeInt(signed=False, label="DVD_VIDEO_COMPRESSION")), ("fLine21Field1InGOP", SimTypeInt(signed=True, label="Int32")), ("fLine21Field2InGOP", SimTypeInt(signed=True, label="Int32")), ("ulSourceResolutionX", SimTypeInt(signed=False, label="UInt32")), ("ulSourceResolutionY", SimTypeInt(signed=False, label="UInt32")), ("fIsSourceLetterboxed", SimTypeInt(signed=True, label="Int32")), ("fIsFilmMode", SimTypeInt(signed=True, label="Int32")),)), name="DVD_VideoAttributes", pack=False, align=None),
    "DVD_WARNING": SimTypeInt(signed=False, label="DVD_WARNING"),
    "DVEXTENTINFO": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("dwExtentMode", SimTypeInt(signed=False, label="UInt32")), ("sizelProposed", SimTypeRef("SIZE", SimStruct)),)), name="DVEXTENTINFO", pack=False, align=None),
    "DVEXTENTMODE": SimTypeInt(signed=False, label="DVEXTENTMODE"),
    "DVIADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="DVIADPCMWAVEFORMAT", pack=False, align=None),
    "DVINFO": SimStruct(OrderedDict((("dwDVAAuxSrc", SimTypeInt(signed=False, label="UInt32")), ("dwDVAAuxCtl", SimTypeInt(signed=False, label="UInt32")), ("dwDVAAuxSrc1", SimTypeInt(signed=False, label="UInt32")), ("dwDVAAuxCtl1", SimTypeInt(signed=False, label="UInt32")), ("dwDVVAuxSrc", SimTypeInt(signed=False, label="UInt32")), ("dwDVVAuxCtl", SimTypeInt(signed=False, label="UInt32")), ("dwDVReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="DVINFO", pack=False, align=None),
    "DVR_STREAM_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("StreamId", SimTypeInt(signed=False, label="UInt32")), ("Default", SimTypeInt(signed=True, label="Int32")), ("Creation", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("guidSubMediaType", SimTypeBottom(label="Guid")), ("guidFormatType", SimTypeBottom(label="Guid")), ("MediaType", SimTypeRef("AM_MEDIA_TYPE", SimStruct)),)), name="DVR_STREAM_DESC", pack=False, align=None),
    "DVTARGETDEVICE": SimStruct(OrderedDict((("tdSize", SimTypeInt(signed=False, label="UInt32")), ("tdDriverNameOffset", SimTypeShort(signed=False, label="UInt16")), ("tdDeviceNameOffset", SimTypeShort(signed=False, label="UInt16")), ("tdPortNameOffset", SimTypeShort(signed=False, label="UInt16")), ("tdExtDevmodeOffset", SimTypeShort(signed=False, label="UInt16")), ("tdData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DVTARGETDEVICE", pack=False, align=None),
    "DWMFLIP3DWINDOWPOLICY": SimTypeInt(signed=False, label="DWMFLIP3DWINDOWPOLICY"),
    "DWMNCRENDERINGPOLICY": SimTypeInt(signed=False, label="DWMNCRENDERINGPOLICY"),
    "DWMTRANSITION_OWNEDWINDOW_TARGET": SimTypeInt(signed=False, label="DWMTRANSITION_OWNEDWINDOW_TARGET"),
    "DWMWINDOWATTRIBUTE": SimTypeInt(signed=False, label="DWMWINDOWATTRIBUTE"),
    "DWM_BLURBEHIND": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fEnable", SimTypeInt(signed=True, label="Int32")), ("hRgnBlur", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fTransitionOnMaximized", SimTypeInt(signed=True, label="Int32")),)), name="DWM_BLURBEHIND", pack=False, align=None),
    "DWM_PRESENT_PARAMETERS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fQueue", SimTypeInt(signed=True, label="Int32")), ("cRefreshStart", SimTypeLongLong(signed=False, label="UInt64")), ("cBuffer", SimTypeInt(signed=False, label="UInt32")), ("fUseSourceRate", SimTypeInt(signed=True, label="Int32")), ("rateSource", SimTypeRef("UNSIGNED_RATIO", SimStruct)), ("cRefreshesPerFrame", SimTypeInt(signed=False, label="UInt32")), ("eSampling", SimTypeInt(signed=False, label="DWM_SOURCE_FRAME_SAMPLING")),)), name="DWM_PRESENT_PARAMETERS", pack=False, align=None),
    "DWM_SHOWCONTACT": SimTypeInt(signed=False, label="DWM_SHOWCONTACT"),
    "DWM_SOURCE_FRAME_SAMPLING": SimTypeInt(signed=False, label="DWM_SOURCE_FRAME_SAMPLING"),
    "DWM_SYSTEMBACKDROP_TYPE": SimTypeInt(signed=False, label="DWM_SYSTEMBACKDROP_TYPE"),
    "DWM_TAB_WINDOW_REQUIREMENTS": SimTypeInt(signed=False, label="DWM_TAB_WINDOW_REQUIREMENTS"),
    "DWM_THUMBNAIL_PROPERTIES": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("rcDestination", SimTypeRef("RECT", SimStruct)), ("rcSource", SimTypeRef("RECT", SimStruct)), ("opacity", SimTypeChar(label="Byte")), ("fVisible", SimTypeInt(signed=True, label="Int32")), ("fSourceClientAreaOnly", SimTypeInt(signed=True, label="Int32")),)), name="DWM_THUMBNAIL_PROPERTIES", pack=False, align=None),
    "DWM_TIMING_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rateRefresh", SimTypeRef("UNSIGNED_RATIO", SimStruct)), ("qpcRefreshPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("rateCompose", SimTypeRef("UNSIGNED_RATIO", SimStruct)), ("qpcVBlank", SimTypeLongLong(signed=False, label="UInt64")), ("cRefresh", SimTypeLongLong(signed=False, label="UInt64")), ("cDXRefresh", SimTypeInt(signed=False, label="UInt32")), ("qpcCompose", SimTypeLongLong(signed=False, label="UInt64")), ("cFrame", SimTypeLongLong(signed=False, label="UInt64")), ("cDXPresent", SimTypeInt(signed=False, label="UInt32")), ("cRefreshFrame", SimTypeLongLong(signed=False, label="UInt64")), ("cFrameSubmitted", SimTypeLongLong(signed=False, label="UInt64")), ("cDXPresentSubmitted", SimTypeInt(signed=False, label="UInt32")), ("cFrameConfirmed", SimTypeLongLong(signed=False, label="UInt64")), ("cDXPresentConfirmed", SimTypeInt(signed=False, label="UInt32")), ("cRefreshConfirmed", SimTypeLongLong(signed=False, label="UInt64")), ("cDXRefreshConfirmed", SimTypeInt(signed=False, label="UInt32")), ("cFramesLate", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesOutstanding", SimTypeInt(signed=False, label="UInt32")), ("cFrameDisplayed", SimTypeLongLong(signed=False, label="UInt64")), ("qpcFrameDisplayed", SimTypeLongLong(signed=False, label="UInt64")), ("cRefreshFrameDisplayed", SimTypeLongLong(signed=False, label="UInt64")), ("cFrameComplete", SimTypeLongLong(signed=False, label="UInt64")), ("qpcFrameComplete", SimTypeLongLong(signed=False, label="UInt64")), ("cFramePending", SimTypeLongLong(signed=False, label="UInt64")), ("qpcFramePending", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesDisplayed", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesComplete", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesPending", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesAvailable", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesDropped", SimTypeLongLong(signed=False, label="UInt64")), ("cFramesMissed", SimTypeLongLong(signed=False, label="UInt64")), ("cRefreshNextDisplayed", SimTypeLongLong(signed=False, label="UInt64")), ("cRefreshNextPresented", SimTypeLongLong(signed=False, label="UInt64")), ("cRefreshesDisplayed", SimTypeLongLong(signed=False, label="UInt64")), ("cRefreshesPresented", SimTypeLongLong(signed=False, label="UInt64")), ("cRefreshStarted", SimTypeLongLong(signed=False, label="UInt64")), ("cPixelsReceived", SimTypeLongLong(signed=False, label="UInt64")), ("cPixelsDrawn", SimTypeLongLong(signed=False, label="UInt64")), ("cBuffersEmpty", SimTypeLongLong(signed=False, label="UInt64")),)), name="DWM_TIMING_INFO", pack=False, align=None),
    "DWM_WINDOW_CORNER_PREFERENCE": SimTypeInt(signed=False, label="DWM_WINDOW_CORNER_PREFERENCE"),
    "DWORD_BLOB": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("alData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DWORD_BLOB", pack=False, align=None),
    "DWORD_DWORD": SimStruct(OrderedDict((("DWord1", SimTypeInt(signed=False, label="UInt32")), ("DWord2", SimTypeInt(signed=False, label="UInt32")),)), name="DWORD_DWORD", pack=False, align=None),
    "DWORD_SIZEDARR": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DWORD_SIZEDARR", pack=False, align=None),
    "DWRITE_AUTOMATIC_FONT_AXES": SimTypeInt(signed=False, label="DWRITE_AUTOMATIC_FONT_AXES"),
    "DWRITE_BASELINE": SimTypeInt(signed=False, label="DWRITE_BASELINE"),
    "DWRITE_BITMAP_DATA_BGRA32": SimStruct(OrderedDict((("width", SimTypeInt(signed=False, label="UInt32")), ("height", SimTypeInt(signed=False, label="UInt32")), ("pixels", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DWRITE_BITMAP_DATA_BGRA32", pack=False, align=None),
    "DWRITE_BREAK_CONDITION": SimTypeInt(signed=False, label="DWRITE_BREAK_CONDITION"),
    "DWRITE_CARET_METRICS": SimStruct(OrderedDict((("slopeRise", SimTypeShort(signed=True, label="Int16")), ("slopeRun", SimTypeShort(signed=True, label="Int16")), ("offset", SimTypeShort(signed=True, label="Int16")),)), name="DWRITE_CARET_METRICS", pack=False, align=None),
    "DWRITE_CLUSTER_METRICS": SimStruct(OrderedDict((("width", SimTypeFloat(size=32)), ("length", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="DWRITE_CLUSTER_METRICS", pack=False, align=None),
    "DWRITE_COLOR_COMPOSITE_MODE": SimTypeInt(signed=False, label="DWRITE_COLOR_COMPOSITE_MODE"),
    "DWRITE_COLOR_F": SimStruct(OrderedDict((("r", SimTypeFloat(size=32)), ("g", SimTypeFloat(size=32)), ("b", SimTypeFloat(size=32)), ("a", SimTypeFloat(size=32)),)), name="DWRITE_COLOR_F", pack=False, align=None),
    "DWRITE_COLOR_GLYPH_RUN": SimStruct(OrderedDict((("glyphRun", SimTypeRef("DWRITE_GLYPH_RUN", SimStruct)), ("glyphRunDescription", SimTypePointer(SimTypeRef("DWRITE_GLYPH_RUN_DESCRIPTION", SimStruct), offset=0)), ("baselineOriginX", SimTypeFloat(size=32)), ("baselineOriginY", SimTypeFloat(size=32)), ("runColor", SimTypeRef("DWRITE_COLOR_F", SimStruct)), ("paletteIndex", SimTypeShort(signed=False, label="UInt16")),)), name="DWRITE_COLOR_GLYPH_RUN", pack=False, align=None),
    "DWRITE_COLOR_GLYPH_RUN1": SimStruct(OrderedDict((("Base", SimTypeRef("DWRITE_COLOR_GLYPH_RUN", SimStruct)), ("glyphImageFormat", SimTypeInt(signed=False, label="DWRITE_GLYPH_IMAGE_FORMATS")), ("measuringMode", SimTypeInt(signed=False, label="DWRITE_MEASURING_MODE")),)), name="DWRITE_COLOR_GLYPH_RUN1", pack=False, align=None),
    "DWRITE_CONTAINER_TYPE": SimTypeInt(signed=False, label="DWRITE_CONTAINER_TYPE"),
    "DWRITE_FACTORY_TYPE": SimTypeInt(signed=False, label="DWRITE_FACTORY_TYPE"),
    "DWRITE_FILE_FRAGMENT": SimStruct(OrderedDict((("fileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("fragmentSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="DWRITE_FILE_FRAGMENT", pack=False, align=None),
    "DWRITE_FLOW_DIRECTION": SimTypeInt(signed=False, label="DWRITE_FLOW_DIRECTION"),
    "DWRITE_FONT_AXIS_ATTRIBUTES": SimTypeInt(signed=False, label="DWRITE_FONT_AXIS_ATTRIBUTES"),
    "DWRITE_FONT_AXIS_RANGE": SimStruct(OrderedDict((("axisTag", SimTypeInt(signed=False, label="DWRITE_FONT_AXIS_TAG")), ("minValue", SimTypeFloat(size=32)), ("maxValue", SimTypeFloat(size=32)),)), name="DWRITE_FONT_AXIS_RANGE", pack=False, align=None),
    "DWRITE_FONT_AXIS_TAG": SimTypeInt(signed=False, label="DWRITE_FONT_AXIS_TAG"),
    "DWRITE_FONT_AXIS_VALUE": SimStruct(OrderedDict((("axisTag", SimTypeInt(signed=False, label="DWRITE_FONT_AXIS_TAG")), ("value", SimTypeFloat(size=32)),)), name="DWRITE_FONT_AXIS_VALUE", pack=False, align=None),
    "DWRITE_FONT_FACE_TYPE": SimTypeInt(signed=False, label="DWRITE_FONT_FACE_TYPE"),
    "DWRITE_FONT_FAMILY_MODEL": SimTypeInt(signed=False, label="DWRITE_FONT_FAMILY_MODEL"),
    "DWRITE_FONT_FEATURE": SimStruct(OrderedDict((("nameTag", SimTypeInt(signed=False, label="DWRITE_FONT_FEATURE_TAG")), ("parameter", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_FONT_FEATURE", pack=False, align=None),
    "DWRITE_FONT_FEATURE_TAG": SimTypeInt(signed=False, label="DWRITE_FONT_FEATURE_TAG"),
    "DWRITE_FONT_FILE_TYPE": SimTypeInt(signed=False, label="DWRITE_FONT_FILE_TYPE"),
    "DWRITE_FONT_LINE_GAP_USAGE": SimTypeInt(signed=False, label="DWRITE_FONT_LINE_GAP_USAGE"),
    "DWRITE_FONT_METRICS": SimStruct(OrderedDict((("designUnitsPerEm", SimTypeShort(signed=False, label="UInt16")), ("ascent", SimTypeShort(signed=False, label="UInt16")), ("descent", SimTypeShort(signed=False, label="UInt16")), ("lineGap", SimTypeShort(signed=True, label="Int16")), ("capHeight", SimTypeShort(signed=False, label="UInt16")), ("xHeight", SimTypeShort(signed=False, label="UInt16")), ("underlinePosition", SimTypeShort(signed=True, label="Int16")), ("underlineThickness", SimTypeShort(signed=False, label="UInt16")), ("strikethroughPosition", SimTypeShort(signed=True, label="Int16")), ("strikethroughThickness", SimTypeShort(signed=False, label="UInt16")),)), name="DWRITE_FONT_METRICS", pack=False, align=None),
    "DWRITE_FONT_METRICS1": SimStruct(OrderedDict((("Base", SimTypeRef("DWRITE_FONT_METRICS", SimStruct)), ("glyphBoxLeft", SimTypeShort(signed=True, label="Int16")), ("glyphBoxTop", SimTypeShort(signed=True, label="Int16")), ("glyphBoxRight", SimTypeShort(signed=True, label="Int16")), ("glyphBoxBottom", SimTypeShort(signed=True, label="Int16")), ("subscriptPositionX", SimTypeShort(signed=True, label="Int16")), ("subscriptPositionY", SimTypeShort(signed=True, label="Int16")), ("subscriptSizeX", SimTypeShort(signed=True, label="Int16")), ("subscriptSizeY", SimTypeShort(signed=True, label="Int16")), ("superscriptPositionX", SimTypeShort(signed=True, label="Int16")), ("superscriptPositionY", SimTypeShort(signed=True, label="Int16")), ("superscriptSizeX", SimTypeShort(signed=True, label="Int16")), ("superscriptSizeY", SimTypeShort(signed=True, label="Int16")), ("hasTypographicMetrics", SimTypeInt(signed=True, label="Int32")),)), name="DWRITE_FONT_METRICS1", pack=False, align=None),
    "DWRITE_FONT_PROPERTY": SimStruct(OrderedDict((("propertyId", SimTypeInt(signed=False, label="DWRITE_FONT_PROPERTY_ID")), ("propertyValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("localeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DWRITE_FONT_PROPERTY", pack=False, align=None),
    "DWRITE_FONT_PROPERTY_ID": SimTypeInt(signed=False, label="DWRITE_FONT_PROPERTY_ID"),
    "DWRITE_FONT_SIMULATIONS": SimTypeInt(signed=False, label="DWRITE_FONT_SIMULATIONS"),
    "DWRITE_FONT_SOURCE_TYPE": SimTypeInt(signed=False, label="DWRITE_FONT_SOURCE_TYPE"),
    "DWRITE_FONT_STRETCH": SimTypeInt(signed=False, label="DWRITE_FONT_STRETCH"),
    "DWRITE_FONT_STYLE": SimTypeInt(signed=False, label="DWRITE_FONT_STYLE"),
    "DWRITE_FONT_WEIGHT": SimTypeInt(signed=False, label="DWRITE_FONT_WEIGHT"),
    "DWRITE_GLYPH_IMAGE_DATA": SimStruct(OrderedDict((("imageData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("imageDataSize", SimTypeInt(signed=False, label="UInt32")), ("uniqueDataId", SimTypeInt(signed=False, label="UInt32")), ("pixelsPerEm", SimTypeInt(signed=False, label="UInt32")), ("pixelSize", SimTypeRef("D2D_SIZE_U", SimStruct)), ("horizontalLeftOrigin", SimTypeRef("POINT", SimStruct)), ("horizontalRightOrigin", SimTypeRef("POINT", SimStruct)), ("verticalTopOrigin", SimTypeRef("POINT", SimStruct)), ("verticalBottomOrigin", SimTypeRef("POINT", SimStruct)),)), name="DWRITE_GLYPH_IMAGE_DATA", pack=False, align=None),
    "DWRITE_GLYPH_IMAGE_FORMATS": SimTypeInt(signed=False, label="DWRITE_GLYPH_IMAGE_FORMATS"),
    "DWRITE_GLYPH_METRICS": SimStruct(OrderedDict((("leftSideBearing", SimTypeInt(signed=True, label="Int32")), ("advanceWidth", SimTypeInt(signed=False, label="UInt32")), ("rightSideBearing", SimTypeInt(signed=True, label="Int32")), ("topSideBearing", SimTypeInt(signed=True, label="Int32")), ("advanceHeight", SimTypeInt(signed=False, label="UInt32")), ("bottomSideBearing", SimTypeInt(signed=True, label="Int32")), ("verticalOriginY", SimTypeInt(signed=True, label="Int32")),)), name="DWRITE_GLYPH_METRICS", pack=False, align=None),
    "DWRITE_GLYPH_OFFSET": SimStruct(OrderedDict((("advanceOffset", SimTypeFloat(size=32)), ("ascenderOffset", SimTypeFloat(size=32)),)), name="DWRITE_GLYPH_OFFSET", pack=False, align=None),
    "DWRITE_GLYPH_ORIENTATION_ANGLE": SimTypeInt(signed=False, label="DWRITE_GLYPH_ORIENTATION_ANGLE"),
    "DWRITE_GLYPH_RUN": SimStruct(OrderedDict((("fontFace", SimTypeBottom(label="IDWriteFontFace")), ("fontEmSize", SimTypeFloat(size=32)), ("glyphCount", SimTypeInt(signed=False, label="UInt32")), ("glyphIndices", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("glyphAdvances", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("glyphOffsets", SimTypePointer(SimTypeRef("DWRITE_GLYPH_OFFSET", SimStruct), offset=0)), ("isSideways", SimTypeInt(signed=True, label="Int32")), ("bidiLevel", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_GLYPH_RUN", pack=False, align=None),
    "DWRITE_GLYPH_RUN_DESCRIPTION": SimStruct(OrderedDict((("localeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("string", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("stringLength", SimTypeInt(signed=False, label="UInt32")), ("clusterMap", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("textPosition", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_GLYPH_RUN_DESCRIPTION", pack=False, align=None),
    "DWRITE_GRID_FIT_MODE": SimTypeInt(signed=False, label="DWRITE_GRID_FIT_MODE"),
    "DWRITE_HIT_TEST_METRICS": SimStruct(OrderedDict((("textPosition", SimTypeInt(signed=False, label="UInt32")), ("length", SimTypeInt(signed=False, label="UInt32")), ("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("width", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)), ("bidiLevel", SimTypeInt(signed=False, label="UInt32")), ("isText", SimTypeInt(signed=True, label="Int32")), ("isTrimmed", SimTypeInt(signed=True, label="Int32")),)), name="DWRITE_HIT_TEST_METRICS", pack=False, align=None),
    "DWRITE_INFORMATIONAL_STRING_ID": SimTypeInt(signed=False, label="DWRITE_INFORMATIONAL_STRING_ID"),
    "DWRITE_INLINE_OBJECT_METRICS": SimStruct(OrderedDict((("width", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)), ("baseline", SimTypeFloat(size=32)), ("supportsSideways", SimTypeInt(signed=True, label="Int32")),)), name="DWRITE_INLINE_OBJECT_METRICS", pack=False, align=None),
    "DWRITE_JUSTIFICATION_OPPORTUNITY": SimStruct(OrderedDict((("expansionMinimum", SimTypeFloat(size=32)), ("expansionMaximum", SimTypeFloat(size=32)), ("compressionMaximum", SimTypeFloat(size=32)), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_JUSTIFICATION_OPPORTUNITY", pack=False, align=None),
    "DWRITE_LINE_BREAKPOINT": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="DWRITE_LINE_BREAKPOINT", pack=False, align=None),
    "DWRITE_LINE_METRICS": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("trailingWhitespaceLength", SimTypeInt(signed=False, label="UInt32")), ("newlineLength", SimTypeInt(signed=False, label="UInt32")), ("height", SimTypeFloat(size=32)), ("baseline", SimTypeFloat(size=32)), ("isTrimmed", SimTypeInt(signed=True, label="Int32")),)), name="DWRITE_LINE_METRICS", pack=False, align=None),
    "DWRITE_LINE_METRICS1": SimStruct(OrderedDict((("Base", SimTypeRef("DWRITE_LINE_METRICS", SimStruct)), ("leadingBefore", SimTypeFloat(size=32)), ("leadingAfter", SimTypeFloat(size=32)),)), name="DWRITE_LINE_METRICS1", pack=False, align=None),
    "DWRITE_LINE_SPACING": SimStruct(OrderedDict((("method", SimTypeInt(signed=False, label="DWRITE_LINE_SPACING_METHOD")), ("height", SimTypeFloat(size=32)), ("baseline", SimTypeFloat(size=32)), ("leadingBefore", SimTypeFloat(size=32)), ("fontLineGapUsage", SimTypeInt(signed=False, label="DWRITE_FONT_LINE_GAP_USAGE")),)), name="DWRITE_LINE_SPACING", pack=False, align=None),
    "DWRITE_LINE_SPACING_METHOD": SimTypeInt(signed=False, label="DWRITE_LINE_SPACING_METHOD"),
    "DWRITE_LOCALITY": SimTypeInt(signed=False, label="DWRITE_LOCALITY"),
    "DWRITE_MATRIX": SimStruct(OrderedDict((("m11", SimTypeFloat(size=32)), ("m12", SimTypeFloat(size=32)), ("m21", SimTypeFloat(size=32)), ("m22", SimTypeFloat(size=32)), ("dx", SimTypeFloat(size=32)), ("dy", SimTypeFloat(size=32)),)), name="DWRITE_MATRIX", pack=False, align=None),
    "DWRITE_MEASURING_MODE": SimTypeInt(signed=False, label="DWRITE_MEASURING_MODE"),
    "DWRITE_NUMBER_SUBSTITUTION_METHOD": SimTypeInt(signed=False, label="DWRITE_NUMBER_SUBSTITUTION_METHOD"),
    "DWRITE_OPTICAL_ALIGNMENT": SimTypeInt(signed=False, label="DWRITE_OPTICAL_ALIGNMENT"),
    "DWRITE_OUTLINE_THRESHOLD": SimTypeInt(signed=False, label="DWRITE_OUTLINE_THRESHOLD"),
    "DWRITE_OVERHANG_METRICS": SimStruct(OrderedDict((("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("right", SimTypeFloat(size=32)), ("bottom", SimTypeFloat(size=32)),)), name="DWRITE_OVERHANG_METRICS", pack=False, align=None),
    "DWRITE_PAINT_ATTRIBUTES": SimTypeInt(signed=False, label="DWRITE_PAINT_ATTRIBUTES"),
    "DWRITE_PAINT_COLOR": SimStruct(OrderedDict((("value", SimTypeRef("DWRITE_COLOR_F", SimStruct)), ("paletteEntryIndex", SimTypeShort(signed=False, label="UInt16")), ("alphaMultiplier", SimTypeFloat(size=32)), ("colorAttributes", SimTypeInt(signed=False, label="DWRITE_PAINT_ATTRIBUTES")),)), name="DWRITE_PAINT_COLOR", pack=False, align=None),
    "DWRITE_PAINT_ELEMENT": SimStruct(OrderedDict((("paintType", SimTypeInt(signed=False, label="DWRITE_PAINT_TYPE")), ("paint", SimUnion({"layers": SimTypeRef("PAINT_LAYERS", SimStruct), "solidGlyph": SimTypeRef("PAINT_SOLID_GLYPH", SimStruct), "solid": SimTypeRef("DWRITE_PAINT_COLOR", SimStruct), "linearGradient": SimTypeRef("PAINT_LINEAR_GRADIENT", SimStruct), "radialGradient": SimTypeRef("PAINT_RADIAL_GRADIENT", SimStruct), "sweepGradient": SimTypeRef("PAINT_SWEEP_GRADIENT", SimStruct), "glyph": SimTypeRef("PAINT_GLYPH", SimStruct), "colorGlyph": SimTypeRef("PAINT_COLOR_GLYPH", SimStruct), "transform": SimTypeRef("DWRITE_MATRIX", SimStruct), "composite": SimTypeRef("PAINT_COMPOSITE", SimStruct)}, name="<anon>", label="None")),)), name="DWRITE_PAINT_ELEMENT", pack=False, align=None),
    "DWRITE_PAINT_FEATURE_LEVEL": SimTypeInt(signed=False, label="DWRITE_PAINT_FEATURE_LEVEL"),
    "DWRITE_PAINT_TYPE": SimTypeInt(signed=False, label="DWRITE_PAINT_TYPE"),
    "DWRITE_PANOSE": SimUnion({"values": SimTypeArray(SimTypeChar(label="Byte"), 10), "familyKind": SimTypeChar(label="Byte"), "text": SimTypeRef("_text_e__Struct", SimStruct), "script": SimTypeRef("_script_e__Struct", SimStruct), "decorative": SimTypeRef("_decorative_e__Struct", SimStruct), "symbol": SimTypeRef("_symbol_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "DWRITE_PANOSE_ARM_STYLE": SimTypeInt(signed=False, label="DWRITE_PANOSE_ARM_STYLE"),
    "DWRITE_PANOSE_ASPECT": SimTypeInt(signed=False, label="DWRITE_PANOSE_ASPECT"),
    "DWRITE_PANOSE_ASPECT_RATIO": SimTypeInt(signed=False, label="DWRITE_PANOSE_ASPECT_RATIO"),
    "DWRITE_PANOSE_CHARACTER_RANGES": SimTypeInt(signed=False, label="DWRITE_PANOSE_CHARACTER_RANGES"),
    "DWRITE_PANOSE_CONTRAST": SimTypeInt(signed=False, label="DWRITE_PANOSE_CONTRAST"),
    "DWRITE_PANOSE_DECORATIVE_CLASS": SimTypeInt(signed=False, label="DWRITE_PANOSE_DECORATIVE_CLASS"),
    "DWRITE_PANOSE_DECORATIVE_TOPOLOGY": SimTypeInt(signed=False, label="DWRITE_PANOSE_DECORATIVE_TOPOLOGY"),
    "DWRITE_PANOSE_FAMILY": SimTypeInt(signed=False, label="DWRITE_PANOSE_FAMILY"),
    "DWRITE_PANOSE_FILL": SimTypeInt(signed=False, label="DWRITE_PANOSE_FILL"),
    "DWRITE_PANOSE_FINIALS": SimTypeInt(signed=False, label="DWRITE_PANOSE_FINIALS"),
    "DWRITE_PANOSE_LETTERFORM": SimTypeInt(signed=False, label="DWRITE_PANOSE_LETTERFORM"),
    "DWRITE_PANOSE_LINING": SimTypeInt(signed=False, label="DWRITE_PANOSE_LINING"),
    "DWRITE_PANOSE_MIDLINE": SimTypeInt(signed=False, label="DWRITE_PANOSE_MIDLINE"),
    "DWRITE_PANOSE_PROPORTION": SimTypeInt(signed=False, label="DWRITE_PANOSE_PROPORTION"),
    "DWRITE_PANOSE_SCRIPT_FORM": SimTypeInt(signed=False, label="DWRITE_PANOSE_SCRIPT_FORM"),
    "DWRITE_PANOSE_SCRIPT_TOPOLOGY": SimTypeInt(signed=False, label="DWRITE_PANOSE_SCRIPT_TOPOLOGY"),
    "DWRITE_PANOSE_SERIF_STYLE": SimTypeInt(signed=False, label="DWRITE_PANOSE_SERIF_STYLE"),
    "DWRITE_PANOSE_SPACING": SimTypeInt(signed=False, label="DWRITE_PANOSE_SPACING"),
    "DWRITE_PANOSE_STROKE_VARIATION": SimTypeInt(signed=False, label="DWRITE_PANOSE_STROKE_VARIATION"),
    "DWRITE_PANOSE_SYMBOL_ASPECT_RATIO": SimTypeInt(signed=False, label="DWRITE_PANOSE_SYMBOL_ASPECT_RATIO"),
    "DWRITE_PANOSE_SYMBOL_KIND": SimTypeInt(signed=False, label="DWRITE_PANOSE_SYMBOL_KIND"),
    "DWRITE_PANOSE_TOOL_KIND": SimTypeInt(signed=False, label="DWRITE_PANOSE_TOOL_KIND"),
    "DWRITE_PANOSE_WEIGHT": SimTypeInt(signed=False, label="DWRITE_PANOSE_WEIGHT"),
    "DWRITE_PANOSE_XASCENT": SimTypeInt(signed=False, label="DWRITE_PANOSE_XASCENT"),
    "DWRITE_PANOSE_XHEIGHT": SimTypeInt(signed=False, label="DWRITE_PANOSE_XHEIGHT"),
    "DWRITE_PARAGRAPH_ALIGNMENT": SimTypeInt(signed=False, label="DWRITE_PARAGRAPH_ALIGNMENT"),
    "DWRITE_PIXEL_GEOMETRY": SimTypeInt(signed=False, label="DWRITE_PIXEL_GEOMETRY"),
    "DWRITE_READING_DIRECTION": SimTypeInt(signed=False, label="DWRITE_READING_DIRECTION"),
    "DWRITE_RENDERING_MODE": SimTypeInt(signed=False, label="DWRITE_RENDERING_MODE"),
    "DWRITE_RENDERING_MODE1": SimTypeInt(signed=False, label="DWRITE_RENDERING_MODE1"),
    "DWRITE_SCRIPT_ANALYSIS": SimStruct(OrderedDict((("script", SimTypeShort(signed=False, label="UInt16")), ("shapes", SimTypeInt(signed=False, label="DWRITE_SCRIPT_SHAPES")),)), name="DWRITE_SCRIPT_ANALYSIS", pack=False, align=None),
    "DWRITE_SCRIPT_PROPERTIES": SimStruct(OrderedDict((("isoScriptCode", SimTypeInt(signed=False, label="UInt32")), ("isoScriptNumber", SimTypeInt(signed=False, label="UInt32")), ("clusterLookahead", SimTypeInt(signed=False, label="UInt32")), ("justificationCharacter", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_SCRIPT_PROPERTIES", pack=False, align=None),
    "DWRITE_SCRIPT_SHAPES": SimTypeInt(signed=False, label="DWRITE_SCRIPT_SHAPES"),
    "DWRITE_SHAPING_GLYPH_PROPERTIES": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="DWRITE_SHAPING_GLYPH_PROPERTIES", pack=False, align=None),
    "DWRITE_SHAPING_TEXT_PROPERTIES": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="DWRITE_SHAPING_TEXT_PROPERTIES", pack=False, align=None),
    "DWRITE_STRIKETHROUGH": SimStruct(OrderedDict((("width", SimTypeFloat(size=32)), ("thickness", SimTypeFloat(size=32)), ("offset", SimTypeFloat(size=32)), ("readingDirection", SimTypeInt(signed=False, label="DWRITE_READING_DIRECTION")), ("flowDirection", SimTypeInt(signed=False, label="DWRITE_FLOW_DIRECTION")), ("localeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("measuringMode", SimTypeInt(signed=False, label="DWRITE_MEASURING_MODE")),)), name="DWRITE_STRIKETHROUGH", pack=False, align=None),
    "DWRITE_TEXTURE_TYPE": SimTypeInt(signed=False, label="DWRITE_TEXTURE_TYPE"),
    "DWRITE_TEXT_ALIGNMENT": SimTypeInt(signed=False, label="DWRITE_TEXT_ALIGNMENT"),
    "DWRITE_TEXT_ANTIALIAS_MODE": SimTypeInt(signed=False, label="DWRITE_TEXT_ANTIALIAS_MODE"),
    "DWRITE_TEXT_METRICS": SimStruct(OrderedDict((("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("width", SimTypeFloat(size=32)), ("widthIncludingTrailingWhitespace", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)), ("layoutWidth", SimTypeFloat(size=32)), ("layoutHeight", SimTypeFloat(size=32)), ("maxBidiReorderingDepth", SimTypeInt(signed=False, label="UInt32")), ("lineCount", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_TEXT_METRICS", pack=False, align=None),
    "DWRITE_TEXT_METRICS1": SimStruct(OrderedDict((("Base", SimTypeRef("DWRITE_TEXT_METRICS", SimStruct)), ("heightIncludingTrailingWhitespace", SimTypeFloat(size=32)),)), name="DWRITE_TEXT_METRICS1", pack=False, align=None),
    "DWRITE_TEXT_RANGE": SimStruct(OrderedDict((("startPosition", SimTypeInt(signed=False, label="UInt32")), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_TEXT_RANGE", pack=False, align=None),
    "DWRITE_TRIMMING": SimStruct(OrderedDict((("granularity", SimTypeInt(signed=False, label="DWRITE_TRIMMING_GRANULARITY")), ("delimiter", SimTypeInt(signed=False, label="UInt32")), ("delimiterCount", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_TRIMMING", pack=False, align=None),
    "DWRITE_TRIMMING_GRANULARITY": SimTypeInt(signed=False, label="DWRITE_TRIMMING_GRANULARITY"),
    "DWRITE_TYPOGRAPHIC_FEATURES": SimStruct(OrderedDict((("features", SimTypePointer(SimTypeRef("DWRITE_FONT_FEATURE", SimStruct), offset=0)), ("featureCount", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_TYPOGRAPHIC_FEATURES", pack=False, align=None),
    "DWRITE_UNDERLINE": SimStruct(OrderedDict((("width", SimTypeFloat(size=32)), ("thickness", SimTypeFloat(size=32)), ("offset", SimTypeFloat(size=32)), ("runHeight", SimTypeFloat(size=32)), ("readingDirection", SimTypeInt(signed=False, label="DWRITE_READING_DIRECTION")), ("flowDirection", SimTypeInt(signed=False, label="DWRITE_FLOW_DIRECTION")), ("localeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("measuringMode", SimTypeInt(signed=False, label="DWRITE_MEASURING_MODE")),)), name="DWRITE_UNDERLINE", pack=False, align=None),
    "DWRITE_UNICODE_RANGE": SimStruct(OrderedDict((("first", SimTypeInt(signed=False, label="UInt32")), ("last", SimTypeInt(signed=False, label="UInt32")),)), name="DWRITE_UNICODE_RANGE", pack=False, align=None),
    "DWRITE_VERTICAL_GLYPH_ORIENTATION": SimTypeInt(signed=False, label="DWRITE_VERTICAL_GLYPH_ORIENTATION"),
    "DWRITE_WORD_WRAPPING": SimTypeInt(signed=False, label="DWRITE_WORD_WRAPPING"),
    "DWebBridgeEvents": SimTypeBottom(label="DWebBridgeEvents"),
    "DWebBrowserEvents": SimTypeBottom(label="DWebBrowserEvents"),
    "DWebBrowserEvents2": SimTypeBottom(label="DWebBrowserEvents2"),
    "DWriteEngine2Events": SimTypeBottom(label="DWriteEngine2Events"),
    "DXAPI_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DxGetIrqInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETIRQINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxEnableIrq", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDENABLEIRQINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxSkipNextField", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDSKIPNEXTFIELDINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxBobNextField", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDBOBNEXTFIELDINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxSetState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDSETSTATEININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDSETSTATEOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxLock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDLOCKININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDLOCKOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxFlipOverlay", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDFLIPOVERLAYINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxFlipVideoPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDFLIPVIDEOPORTINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxGetPolarity", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETPOLARITYININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDGETPOLARITYOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxGetCurrentAutoflip", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETCURRENTAUTOFLIPININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDGETCURRENTAUTOFLIPOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxGetPreviousAutoflip", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETPREVIOUSAUTOFLIPININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDGETPREVIOUSAUTOFLIPOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxTransfer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDTRANSFERININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDTRANSFEROUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("DxGetTransferStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETTRANSFERSTATUSOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)),)), name="DXAPI_INTERFACE", pack=False, align=None),
    "DXC_CP": SimTypeInt(signed=False, label="DXC_CP"),
    "DXC_OUT_KIND": SimTypeInt(signed=False, label="DXC_OUT_KIND"),
    "DXCoreAdapterMemoryBudget": SimStruct(OrderedDict((("budget", SimTypeLongLong(signed=False, label="UInt64")), ("currentUsage", SimTypeLongLong(signed=False, label="UInt64")), ("availableForReservation", SimTypeLongLong(signed=False, label="UInt64")), ("currentReservation", SimTypeLongLong(signed=False, label="UInt64")),)), name="DXCoreAdapterMemoryBudget", pack=False, align=None),
    "DXCoreAdapterMemoryBudgetNodeSegmentGroup": SimStruct(OrderedDict((("nodeIndex", SimTypeInt(signed=False, label="UInt32")), ("segmentGroup", SimTypeInt(signed=False, label="DXCoreSegmentGroup")),)), name="DXCoreAdapterMemoryBudgetNodeSegmentGroup", pack=False, align=None),
    "DXCoreAdapterPreference": SimTypeInt(signed=False, label="DXCoreAdapterPreference"),
    "DXCoreAdapterProperty": SimTypeInt(signed=False, label="DXCoreAdapterProperty"),
    "DXCoreAdapterState": SimTypeInt(signed=False, label="DXCoreAdapterState"),
    "DXCoreHardwareID": SimStruct(OrderedDict((("vendorID", SimTypeInt(signed=False, label="UInt32")), ("deviceID", SimTypeInt(signed=False, label="UInt32")), ("subSysID", SimTypeInt(signed=False, label="UInt32")), ("revision", SimTypeInt(signed=False, label="UInt32")),)), name="DXCoreHardwareID", pack=False, align=None),
    "DXCoreHardwareIDParts": SimStruct(OrderedDict((("vendorID", SimTypeInt(signed=False, label="UInt32")), ("deviceID", SimTypeInt(signed=False, label="UInt32")), ("subSystemID", SimTypeInt(signed=False, label="UInt32")), ("subVendorID", SimTypeInt(signed=False, label="UInt32")), ("revisionID", SimTypeInt(signed=False, label="UInt32")),)), name="DXCoreHardwareIDParts", pack=False, align=None),
    "DXCoreNotificationType": SimTypeInt(signed=False, label="DXCoreNotificationType"),
    "DXCoreSegmentGroup": SimTypeInt(signed=False, label="DXCoreSegmentGroup"),
    "DXGI_ADAPTER_DESC": SimStruct(OrderedDict((("Description", SimTypeArray(SimTypeChar(label="Char"), 128)), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("SubSysId", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("DedicatedVideoMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DedicatedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterLuid", SimTypeRef("LUID", SimStruct)),)), name="DXGI_ADAPTER_DESC", pack=False, align=None),
    "DXGI_ADAPTER_DESC1": SimStruct(OrderedDict((("Description", SimTypeArray(SimTypeChar(label="Char"), 128)), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("SubSysId", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("DedicatedVideoMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DedicatedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_ADAPTER_DESC1", pack=False, align=None),
    "DXGI_ADAPTER_DESC2": SimStruct(OrderedDict((("Description", SimTypeArray(SimTypeChar(label="Char"), 128)), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("SubSysId", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("DedicatedVideoMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DedicatedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("GraphicsPreemptionGranularity", SimTypeInt(signed=False, label="DXGI_GRAPHICS_PREEMPTION_GRANULARITY")), ("ComputePreemptionGranularity", SimTypeInt(signed=False, label="DXGI_COMPUTE_PREEMPTION_GRANULARITY")),)), name="DXGI_ADAPTER_DESC2", pack=False, align=None),
    "DXGI_ADAPTER_DESC3": SimStruct(OrderedDict((("Description", SimTypeArray(SimTypeChar(label="Char"), 128)), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("SubSysId", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("DedicatedVideoMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DedicatedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedSystemMemory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="DXGI_ADAPTER_FLAG3")), ("GraphicsPreemptionGranularity", SimTypeInt(signed=False, label="DXGI_GRAPHICS_PREEMPTION_GRANULARITY")), ("ComputePreemptionGranularity", SimTypeInt(signed=False, label="DXGI_COMPUTE_PREEMPTION_GRANULARITY")),)), name="DXGI_ADAPTER_DESC3", pack=False, align=None),
    "DXGI_ADAPTER_FLAG": SimTypeInt(signed=False, label="DXGI_ADAPTER_FLAG"),
    "DXGI_ADAPTER_FLAG3": SimTypeInt(signed=False, label="DXGI_ADAPTER_FLAG3"),
    "DXGI_ALPHA_MODE": SimTypeInt(signed=False, label="DXGI_ALPHA_MODE"),
    "DXGI_COLOR_SPACE_TYPE": SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE"),
    "DXGI_COMPUTE_PREEMPTION_GRANULARITY": SimTypeInt(signed=False, label="DXGI_COMPUTE_PREEMPTION_GRANULARITY"),
    "DXGI_DEBUG_RLO_FLAGS": SimTypeInt(signed=False, label="DXGI_DEBUG_RLO_FLAGS"),
    "DXGI_DECODE_SWAP_CHAIN_DESC": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_DECODE_SWAP_CHAIN_DESC", pack=False, align=None),
    "DXGI_DISPLAY_COLOR_SPACE": SimStruct(OrderedDict((("PrimaryCoordinates", SimTypeArray(SimTypeFloat(size=32), 16)), ("WhitePoints", SimTypeArray(SimTypeFloat(size=32), 32)),)), name="DXGI_DISPLAY_COLOR_SPACE", pack=False, align=None),
    "DXGI_FEATURE": SimTypeInt(signed=False, label="DXGI_FEATURE"),
    "DXGI_FORMAT": SimTypeInt(signed=False, label="DXGI_FORMAT"),
    "DXGI_FRAME_PRESENTATION_MODE": SimTypeInt(signed=False, label="DXGI_FRAME_PRESENTATION_MODE"),
    "DXGI_FRAME_STATISTICS": SimStruct(OrderedDict((("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("PresentRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("SyncGPUTime", SimTypeLongLong(signed=True, label="Int64")),)), name="DXGI_FRAME_STATISTICS", pack=False, align=None),
    "DXGI_FRAME_STATISTICS_MEDIA": SimStruct(OrderedDict((("PresentCount", SimTypeInt(signed=False, label="UInt32")), ("PresentRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncRefreshCount", SimTypeInt(signed=False, label="UInt32")), ("SyncQPCTime", SimTypeLongLong(signed=True, label="Int64")), ("SyncGPUTime", SimTypeLongLong(signed=True, label="Int64")), ("CompositionMode", SimTypeInt(signed=False, label="DXGI_FRAME_PRESENTATION_MODE")), ("ApprovedPresentDuration", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_FRAME_STATISTICS_MEDIA", pack=False, align=None),
    "DXGI_GAMMA_CONTROL": SimStruct(OrderedDict((("Scale", SimTypeRef("DXGI_RGB", SimStruct)), ("Offset", SimTypeRef("DXGI_RGB", SimStruct)), ("GammaCurve", SimTypeArray(SimTypeRef("DXGI_RGB", SimStruct), 1025)),)), name="DXGI_GAMMA_CONTROL", pack=False, align=None),
    "DXGI_GAMMA_CONTROL_CAPABILITIES": SimStruct(OrderedDict((("ScaleAndOffsetSupported", SimTypeInt(signed=True, label="Int32")), ("MaxConvertedValue", SimTypeFloat(size=32)), ("MinConvertedValue", SimTypeFloat(size=32)), ("NumGammaControlPoints", SimTypeInt(signed=False, label="UInt32")), ("ControlPointPositions", SimTypeArray(SimTypeFloat(size=32), 1025)),)), name="DXGI_GAMMA_CONTROL_CAPABILITIES", pack=False, align=None),
    "DXGI_GPU_PREFERENCE": SimTypeInt(signed=False, label="DXGI_GPU_PREFERENCE"),
    "DXGI_GRAPHICS_PREEMPTION_GRANULARITY": SimTypeInt(signed=False, label="DXGI_GRAPHICS_PREEMPTION_GRANULARITY"),
    "DXGI_HARDWARE_COMPOSITION_SUPPORT_FLAGS": SimTypeInt(signed=False, label="DXGI_HARDWARE_COMPOSITION_SUPPORT_FLAGS"),
    "DXGI_HDR_METADATA_HDR10": SimStruct(OrderedDict((("RedPrimary", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("GreenPrimary", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("BluePrimary", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("WhitePoint", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("MaxMasteringLuminance", SimTypeInt(signed=False, label="UInt32")), ("MinMasteringLuminance", SimTypeInt(signed=False, label="UInt32")), ("MaxContentLightLevel", SimTypeShort(signed=False, label="UInt16")), ("MaxFrameAverageLightLevel", SimTypeShort(signed=False, label="UInt16")),)), name="DXGI_HDR_METADATA_HDR10", pack=False, align=None),
    "DXGI_HDR_METADATA_HDR10PLUS": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeChar(label="Byte"), 72)),)), name="DXGI_HDR_METADATA_HDR10PLUS", pack=False, align=None),
    "DXGI_HDR_METADATA_TYPE": SimTypeInt(signed=False, label="DXGI_HDR_METADATA_TYPE"),
    "DXGI_INFO_QUEUE_FILTER": SimStruct(OrderedDict((("AllowList", SimTypeRef("DXGI_INFO_QUEUE_FILTER_DESC", SimStruct)), ("DenyList", SimTypeRef("DXGI_INFO_QUEUE_FILTER_DESC", SimStruct)),)), name="DXGI_INFO_QUEUE_FILTER", pack=False, align=None),
    "DXGI_INFO_QUEUE_FILTER_DESC": SimStruct(OrderedDict((("NumCategories", SimTypeInt(signed=False, label="UInt32")), ("pCategoryList", SimTypePointer(SimTypeInt(signed=False, label="DXGI_INFO_QUEUE_MESSAGE_CATEGORY"), offset=0)), ("NumSeverities", SimTypeInt(signed=False, label="UInt32")), ("pSeverityList", SimTypePointer(SimTypeInt(signed=False, label="DXGI_INFO_QUEUE_MESSAGE_SEVERITY"), offset=0)), ("NumIDs", SimTypeInt(signed=False, label="UInt32")), ("pIDList", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="DXGI_INFO_QUEUE_FILTER_DESC", pack=False, align=None),
    "DXGI_INFO_QUEUE_MESSAGE": SimStruct(OrderedDict((("Producer", SimTypeBottom(label="Guid")), ("Category", SimTypeInt(signed=False, label="DXGI_INFO_QUEUE_MESSAGE_CATEGORY")), ("Severity", SimTypeInt(signed=False, label="DXGI_INFO_QUEUE_MESSAGE_SEVERITY")), ("ID", SimTypeInt(signed=True, label="Int32")), ("pDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DescriptionByteLength", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="DXGI_INFO_QUEUE_MESSAGE", pack=False, align=None),
    "DXGI_INFO_QUEUE_MESSAGE_CATEGORY": SimTypeInt(signed=False, label="DXGI_INFO_QUEUE_MESSAGE_CATEGORY"),
    "DXGI_INFO_QUEUE_MESSAGE_SEVERITY": SimTypeInt(signed=False, label="DXGI_INFO_QUEUE_MESSAGE_SEVERITY"),
    "DXGI_JPEG_AC_HUFFMAN_TABLE": SimStruct(OrderedDict((("CodeCounts", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("CodeValues", SimTypeArray(SimTypeChar(label="Byte"), 162)),)), name="DXGI_JPEG_AC_HUFFMAN_TABLE", pack=False, align=None),
    "DXGI_JPEG_DC_HUFFMAN_TABLE": SimStruct(OrderedDict((("CodeCounts", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("CodeValues", SimTypeArray(SimTypeChar(label="Byte"), 12)),)), name="DXGI_JPEG_DC_HUFFMAN_TABLE", pack=False, align=None),
    "DXGI_JPEG_QUANTIZATION_TABLE": SimStruct(OrderedDict((("Elements", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="DXGI_JPEG_QUANTIZATION_TABLE", pack=False, align=None),
    "DXGI_MAPPED_RECT": SimStruct(OrderedDict((("Pitch", SimTypeInt(signed=True, label="Int32")), ("pBits", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="DXGI_MAPPED_RECT", pack=False, align=None),
    "DXGI_MATRIX_3X2_F": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)),)), name="DXGI_MATRIX_3X2_F", pack=False, align=None),
    "DXGI_MEMORY_SEGMENT_GROUP": SimTypeInt(signed=False, label="DXGI_MEMORY_SEGMENT_GROUP"),
    "DXGI_MODE_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ScanlineOrdering", SimTypeInt(signed=False, label="DXGI_MODE_SCANLINE_ORDER")), ("Scaling", SimTypeInt(signed=False, label="DXGI_MODE_SCALING")),)), name="DXGI_MODE_DESC", pack=False, align=None),
    "DXGI_MODE_DESC1": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("RefreshRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("ScanlineOrdering", SimTypeInt(signed=False, label="DXGI_MODE_SCANLINE_ORDER")), ("Scaling", SimTypeInt(signed=False, label="DXGI_MODE_SCALING")), ("Stereo", SimTypeInt(signed=True, label="Int32")),)), name="DXGI_MODE_DESC1", pack=False, align=None),
    "DXGI_MODE_ROTATION": SimTypeInt(signed=False, label="DXGI_MODE_ROTATION"),
    "DXGI_MODE_SCALING": SimTypeInt(signed=False, label="DXGI_MODE_SCALING"),
    "DXGI_MODE_SCANLINE_ORDER": SimTypeInt(signed=False, label="DXGI_MODE_SCANLINE_ORDER"),
    "DXGI_MULTIPLANE_OVERLAY_YCbCr_FLAGS": SimTypeInt(signed=False, label="DXGI_MULTIPLANE_OVERLAY_YCbCr_FLAGS"),
    "DXGI_Message_Id": SimTypeInt(signed=False, label="DXGI_Message_Id"),
    "DXGI_OFFER_RESOURCE_FLAGS": SimTypeInt(signed=False, label="DXGI_OFFER_RESOURCE_FLAGS"),
    "DXGI_OFFER_RESOURCE_PRIORITY": SimTypeInt(signed=False, label="DXGI_OFFER_RESOURCE_PRIORITY"),
    "DXGI_OUTDUPL_DESC": SimStruct(OrderedDict((("ModeDesc", SimTypeRef("DXGI_MODE_DESC", SimStruct)), ("Rotation", SimTypeInt(signed=False, label="DXGI_MODE_ROTATION")), ("DesktopImageInSystemMemory", SimTypeInt(signed=True, label="Int32")),)), name="DXGI_OUTDUPL_DESC", pack=False, align=None),
    "DXGI_OUTDUPL_FLAG": SimTypeInt(signed=False, label="DXGI_OUTDUPL_FLAG"),
    "DXGI_OUTDUPL_FRAME_INFO": SimStruct(OrderedDict((("LastPresentTime", SimTypeLongLong(signed=True, label="Int64")), ("LastMouseUpdateTime", SimTypeLongLong(signed=True, label="Int64")), ("AccumulatedFrames", SimTypeInt(signed=False, label="UInt32")), ("RectsCoalesced", SimTypeInt(signed=True, label="Int32")), ("ProtectedContentMaskedOut", SimTypeInt(signed=True, label="Int32")), ("PointerPosition", SimTypeRef("DXGI_OUTDUPL_POINTER_POSITION", SimStruct)), ("TotalMetadataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("PointerShapeBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_OUTDUPL_FRAME_INFO", pack=False, align=None),
    "DXGI_OUTDUPL_MOVE_RECT": SimStruct(OrderedDict((("SourcePoint", SimTypeRef("POINT", SimStruct)), ("DestinationRect", SimTypeRef("RECT", SimStruct)),)), name="DXGI_OUTDUPL_MOVE_RECT", pack=False, align=None),
    "DXGI_OUTDUPL_POINTER_POSITION": SimStruct(OrderedDict((("Position", SimTypeRef("POINT", SimStruct)), ("Visible", SimTypeInt(signed=True, label="Int32")),)), name="DXGI_OUTDUPL_POINTER_POSITION", pack=False, align=None),
    "DXGI_OUTDUPL_POINTER_SHAPE_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Pitch", SimTypeInt(signed=False, label="UInt32")), ("HotSpot", SimTypeRef("POINT", SimStruct)),)), name="DXGI_OUTDUPL_POINTER_SHAPE_INFO", pack=False, align=None),
    "DXGI_OUTDUPL_POINTER_SHAPE_TYPE": SimTypeInt(signed=False, label="DXGI_OUTDUPL_POINTER_SHAPE_TYPE"),
    "DXGI_OUTPUT_DESC": SimStruct(OrderedDict((("DeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DesktopCoordinates", SimTypeRef("RECT", SimStruct)), ("AttachedToDesktop", SimTypeInt(signed=True, label="Int32")), ("Rotation", SimTypeInt(signed=False, label="DXGI_MODE_ROTATION")), ("Monitor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DXGI_OUTPUT_DESC", pack=False, align=None),
    "DXGI_OUTPUT_DESC1": SimStruct(OrderedDict((("DeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DesktopCoordinates", SimTypeRef("RECT", SimStruct)), ("AttachedToDesktop", SimTypeInt(signed=True, label="Int32")), ("Rotation", SimTypeInt(signed=False, label="DXGI_MODE_ROTATION")), ("Monitor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("BitsPerColor", SimTypeInt(signed=False, label="UInt32")), ("ColorSpace", SimTypeInt(signed=False, label="DXGI_COLOR_SPACE_TYPE")), ("RedPrimary", SimTypeArray(SimTypeFloat(size=32), 2)), ("GreenPrimary", SimTypeArray(SimTypeFloat(size=32), 2)), ("BluePrimary", SimTypeArray(SimTypeFloat(size=32), 2)), ("WhitePoint", SimTypeArray(SimTypeFloat(size=32), 2)), ("MinLuminance", SimTypeFloat(size=32)), ("MaxLuminance", SimTypeFloat(size=32)), ("MaxFullFrameLuminance", SimTypeFloat(size=32)),)), name="DXGI_OUTPUT_DESC1", pack=False, align=None),
    "DXGI_OVERLAY_COLOR_SPACE_SUPPORT_FLAG": SimTypeInt(signed=False, label="DXGI_OVERLAY_COLOR_SPACE_SUPPORT_FLAG"),
    "DXGI_OVERLAY_SUPPORT_FLAG": SimTypeInt(signed=False, label="DXGI_OVERLAY_SUPPORT_FLAG"),
    "DXGI_PRESENT_PARAMETERS": SimStruct(OrderedDict((("DirtyRectsCount", SimTypeInt(signed=False, label="UInt32")), ("pDirtyRects", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("pScrollRect", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("pScrollOffset", SimTypePointer(SimTypeRef("POINT", SimStruct), offset=0)),)), name="DXGI_PRESENT_PARAMETERS", pack=False, align=None),
    "DXGI_QUERY_VIDEO_MEMORY_INFO": SimStruct(OrderedDict((("Budget", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentUsage", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableForReservation", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentReservation", SimTypeLongLong(signed=False, label="UInt64")),)), name="DXGI_QUERY_VIDEO_MEMORY_INFO", pack=False, align=None),
    "DXGI_RATIONAL": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_RATIONAL", pack=False, align=None),
    "DXGI_RECLAIM_RESOURCE_RESULTS": SimTypeInt(signed=False, label="DXGI_RECLAIM_RESOURCE_RESULTS"),
    "DXGI_RESIDENCY": SimTypeInt(signed=False, label="DXGI_RESIDENCY"),
    "DXGI_RGB": SimStruct(OrderedDict((("Red", SimTypeFloat(size=32)), ("Green", SimTypeFloat(size=32)), ("Blue", SimTypeFloat(size=32)),)), name="DXGI_RGB", pack=False, align=None),
    "DXGI_RGBA": SimStruct(OrderedDict((("r", SimTypeFloat(size=32)), ("g", SimTypeFloat(size=32)), ("b", SimTypeFloat(size=32)), ("a", SimTypeFloat(size=32)),)), name="DXGI_RGBA", pack=False, align=None),
    "DXGI_SAMPLE_DESC": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Quality", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_SAMPLE_DESC", pack=False, align=None),
    "DXGI_SCALING": SimTypeInt(signed=False, label="DXGI_SCALING"),
    "DXGI_SHARED_RESOURCE": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="DXGI_SHARED_RESOURCE", pack=False, align=None),
    "DXGI_SURFACE_DESC": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)),)), name="DXGI_SURFACE_DESC", pack=False, align=None),
    "DXGI_SWAP_CHAIN_COLOR_SPACE_SUPPORT_FLAG": SimTypeInt(signed=False, label="DXGI_SWAP_CHAIN_COLOR_SPACE_SUPPORT_FLAG"),
    "DXGI_SWAP_CHAIN_DESC": SimStruct(OrderedDict((("BufferDesc", SimTypeRef("DXGI_MODE_DESC", SimStruct)), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("BufferUsage", SimTypeInt(signed=False, label="DXGI_USAGE")), ("BufferCount", SimTypeInt(signed=False, label="UInt32")), ("OutputWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Windowed", SimTypeInt(signed=True, label="Int32")), ("SwapEffect", SimTypeInt(signed=False, label="DXGI_SWAP_EFFECT")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_SWAP_CHAIN_DESC", pack=False, align=None),
    "DXGI_SWAP_CHAIN_DESC1": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Stereo", SimTypeInt(signed=True, label="Int32")), ("SampleDesc", SimTypeRef("DXGI_SAMPLE_DESC", SimStruct)), ("BufferUsage", SimTypeInt(signed=False, label="DXGI_USAGE")), ("BufferCount", SimTypeInt(signed=False, label="UInt32")), ("Scaling", SimTypeInt(signed=False, label="DXGI_SCALING")), ("SwapEffect", SimTypeInt(signed=False, label="DXGI_SWAP_EFFECT")), ("AlphaMode", SimTypeInt(signed=False, label="DXGI_ALPHA_MODE")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DXGI_SWAP_CHAIN_DESC1", pack=False, align=None),
    "DXGI_SWAP_CHAIN_FLAG": SimTypeInt(signed=False, label="DXGI_SWAP_CHAIN_FLAG"),
    "DXGI_SWAP_CHAIN_FULLSCREEN_DESC": SimStruct(OrderedDict((("RefreshRate", SimTypeRef("DXGI_RATIONAL", SimStruct)), ("ScanlineOrdering", SimTypeInt(signed=False, label="DXGI_MODE_SCANLINE_ORDER")), ("Scaling", SimTypeInt(signed=False, label="DXGI_MODE_SCALING")), ("Windowed", SimTypeInt(signed=True, label="Int32")),)), name="DXGI_SWAP_CHAIN_FULLSCREEN_DESC", pack=False, align=None),
    "DXGI_SWAP_EFFECT": SimTypeInt(signed=False, label="DXGI_SWAP_EFFECT"),
    "DXGI_USAGE": SimTypeInt(signed=False, label="DXGI_USAGE"),
    "DXGKARG_SETPALETTE": SimStruct(OrderedDict((("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")), ("FirstEntry", SimTypeInt(signed=False, label="UInt32")), ("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("pLookupTable", SimTypePointer(SimTypeRef("D3DKMDT_PALETTEDATA", SimStruct), offset=0)),)), name="DXGKARG_SETPALETTE", pack=False, align=None),
    "DXGKDT_OPM_DVI_CHARACTERISTICS": SimTypeInt(signed=False, label="DXGKDT_OPM_DVI_CHARACTERISTICS"),
    "DXGKMDT_CERTIFICATE_TYPE": SimTypeInt(signed=False, label="DXGKMDT_CERTIFICATE_TYPE"),
    "DXGKMDT_OPM_ACP_AND_CGMSA_SIGNALING": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulAvailableTVProtectionStandards", SimTypeInt(signed=False, label="UInt32")), ("ulActiveTVProtectionStandard", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioValidMask1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioValidMask2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioValidMask3", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData3", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ulReserved3", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="DXGKMDT_OPM_ACP_AND_CGMSA_SIGNALING", pack=False, align=None),
    "DXGKMDT_OPM_ACP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="DXGKMDT_OPM_ACP_PROTECTION_LEVEL"),
    "DXGKMDT_OPM_ACTUAL_OUTPUT_FORMAT": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulDisplayWidth", SimTypeInt(signed=False, label="UInt32")), ("ulDisplayHeight", SimTypeInt(signed=False, label="UInt32")), ("ifInterleaveFormat", SimTypeInt(signed=False, label="DXGKMDT_OPM_INTERLEAVE_FORMAT")), ("d3dFormat", SimTypeInt(signed=False, label="UInt32")), ("ulFrequencyNumerator", SimTypeInt(signed=False, label="UInt32")), ("ulFrequencyDenominator", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKMDT_OPM_ACTUAL_OUTPUT_FORMAT", pack=False, align=None),
    "DXGKMDT_OPM_BUS_TYPE_AND_IMPLEMENTATION": SimTypeInt(signed=False, label="DXGKMDT_OPM_BUS_TYPE_AND_IMPLEMENTATION"),
    "DXGKMDT_OPM_CGMSA": SimTypeInt(signed=False, label="DXGKMDT_OPM_CGMSA"),
    "DXGKMDT_OPM_CONFIGURE_PARAMETERS": SimStruct(OrderedDict((("omac", SimTypeRef("DXGKMDT_OPM_OMAC", SimStruct)), ("guidSetting", SimTypeBottom(label="Guid")), ("ulSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("cbParametersSize", SimTypeInt(signed=False, label="UInt32")), ("abParameters", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="DXGKMDT_OPM_CONFIGURE_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_CONNECTED_HDCP_DEVICE_INFORMATION": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulHDCPFlags", SimTypeInt(signed=False, label="UInt32")), ("ksvB", SimTypeRef("DXGKMDT_OPM_HDCP_KEY_SELECTION_VECTOR", SimStruct)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 11)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DXGKMDT_OPM_CONNECTED_HDCP_DEVICE_INFORMATION", pack=False, align=None),
    "DXGKMDT_OPM_CONNECTOR_TYPE": SimTypeInt(signed=False, label="DXGKMDT_OPM_CONNECTOR_TYPE"),
    "DXGKMDT_OPM_COPP_COMPATIBLE_GET_INFO_PARAMETERS": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("guidInformation", SimTypeBottom(label="Guid")), ("ulSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("cbParametersSize", SimTypeInt(signed=False, label="UInt32")), ("abParameters", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="DXGKMDT_OPM_COPP_COMPATIBLE_GET_INFO_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_CREATE_VIDEO_OUTPUT_FOR_TARGET_PARAMETERS": SimStruct(OrderedDict((("AdapterLuid", SimTypeRef("LUID", SimStruct)), ("TargetId", SimTypeInt(signed=False, label="UInt32")), ("Vos", SimTypeInt(signed=False, label="DXGKMDT_OPM_VIDEO_OUTPUT_SEMANTICS")),)), name="DXGKMDT_OPM_CREATE_VIDEO_OUTPUT_FOR_TARGET_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_DPCP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="DXGKMDT_OPM_DPCP_PROTECTION_LEVEL"),
    "DXGKMDT_OPM_ENCRYPTED_PARAMETERS": SimStruct(OrderedDict((("abEncryptedParameters", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="DXGKMDT_OPM_ENCRYPTED_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_GET_INFO_PARAMETERS": SimStruct(OrderedDict((("omac", SimTypeRef("DXGKMDT_OPM_OMAC", SimStruct)), ("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("guidInformation", SimTypeBottom(label="Guid")), ("ulSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("cbParametersSize", SimTypeInt(signed=False, label="UInt32")), ("abParameters", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="DXGKMDT_OPM_GET_INFO_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_HDCP_FLAG": SimTypeInt(signed=False, label="DXGKMDT_OPM_HDCP_FLAG"),
    "DXGKMDT_OPM_HDCP_KEY_SELECTION_VECTOR": SimStruct(OrderedDict((("abKeySelectionVector", SimTypeArray(SimTypeChar(label="Byte"), 5)),)), name="DXGKMDT_OPM_HDCP_KEY_SELECTION_VECTOR", pack=False, align=None),
    "DXGKMDT_OPM_HDCP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="DXGKMDT_OPM_HDCP_PROTECTION_LEVEL"),
    "DXGKMDT_OPM_IMAGE_ASPECT_RATIO_EN300294": SimTypeInt(signed=False, label="DXGKMDT_OPM_IMAGE_ASPECT_RATIO_EN300294"),
    "DXGKMDT_OPM_INTERLEAVE_FORMAT": SimTypeInt(signed=False, label="DXGKMDT_OPM_INTERLEAVE_FORMAT"),
    "DXGKMDT_OPM_OMAC": SimStruct(OrderedDict((("abOMAC", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DXGKMDT_OPM_OMAC", pack=False, align=None),
    "DXGKMDT_OPM_OUTPUT_HARDWARE_PROTECTION": SimTypeInt(signed=False, label="DXGKMDT_OPM_OUTPUT_HARDWARE_PROTECTION"),
    "DXGKMDT_OPM_OUTPUT_ID": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("OutputId", SimTypeLongLong(signed=False, label="UInt64")),)), name="DXGKMDT_OPM_OUTPUT_ID", pack=False, align=None),
    "DXGKMDT_OPM_PROTECTION_STANDARD": SimTypeInt(signed=False, label="DXGKMDT_OPM_PROTECTION_STANDARD"),
    "DXGKMDT_OPM_PROTECTION_TYPE": SimTypeInt(signed=False, label="DXGKMDT_OPM_PROTECTION_TYPE"),
    "DXGKMDT_OPM_RANDOM_NUMBER": SimStruct(OrderedDict((("abRandomNumber", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DXGKMDT_OPM_RANDOM_NUMBER", pack=False, align=None),
    "DXGKMDT_OPM_REQUESTED_INFORMATION": SimStruct(OrderedDict((("omac", SimTypeRef("DXGKMDT_OPM_OMAC", SimStruct)), ("cbRequestedInformationSize", SimTypeInt(signed=False, label="UInt32")), ("abRequestedInformation", SimTypeArray(SimTypeChar(label="Byte"), 4076)),)), name="DXGKMDT_OPM_REQUESTED_INFORMATION", pack=False, align=None),
    "DXGKMDT_OPM_SET_ACP_AND_CGMSA_SIGNALING_PARAMETERS": SimStruct(OrderedDict((("ulNewTVProtectionStandard", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioChangeMask1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioChangeMask2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioChangeMask3", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData3", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ulReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ulReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKMDT_OPM_SET_ACP_AND_CGMSA_SIGNALING_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_SET_HDCP_SRM_PARAMETERS": SimStruct(OrderedDict((("ulSRMVersion", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKMDT_OPM_SET_HDCP_SRM_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_SET_PROTECTION_LEVEL_PARAMETERS": SimStruct(OrderedDict((("ulProtectionType", SimTypeInt(signed=False, label="UInt32")), ("ulProtectionLevel", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKMDT_OPM_SET_PROTECTION_LEVEL_PARAMETERS", pack=False, align=None),
    "DXGKMDT_OPM_STANDARD_INFORMATION": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("DXGKMDT_OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulInformation", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKMDT_OPM_STANDARD_INFORMATION", pack=False, align=None),
    "DXGKMDT_OPM_STATUS": SimTypeInt(signed=False, label="DXGKMDT_OPM_STATUS"),
    "DXGKMDT_OPM_TYPE_ENFORCEMENT_HDCP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="DXGKMDT_OPM_TYPE_ENFORCEMENT_HDCP_PROTECTION_LEVEL"),
    "DXGKMDT_OPM_VIDEO_OUTPUT_SEMANTICS": SimTypeInt(signed=False, label="DXGKMDT_OPM_VIDEO_OUTPUT_SEMANTICS"),
    "DXGKMT_MULTIPLANE_OVERLAY_STEREO_FLIP_MODE": SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STEREO_FLIP_MODE"),
    "DXGKMT_MULTIPLANE_OVERLAY_STRETCH_QUALITY": SimTypeInt(signed=False, label="DXGKMT_MULTIPLANE_OVERLAY_STRETCH_QUALITY"),
    "DXGKMT_POWER_SHARED_TYPE": SimTypeInt(signed=False, label="DXGKMT_POWER_SHARED_TYPE"),
    "DXGKVGPU_ESCAPE_HEAD": SimStruct(OrderedDict((("Luid", SimTypeRef("GPUP_DRIVER_ESCAPE_INPUT", SimStruct)), ("Type", SimTypeInt(signed=False, label="DXGKVGPU_ESCAPE_TYPE")),)), name="DXGKVGPU_ESCAPE_HEAD", pack=False, align=None),
    "DXGKVGPU_ESCAPE_INITIALIZE": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)), ("VmGuid", SimTypeBottom(label="Guid")),)), name="DXGKVGPU_ESCAPE_INITIALIZE", pack=False, align=None),
    "DXGKVGPU_ESCAPE_PAUSE": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)), ("DeviceLuid", SimTypeRef("LUID", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXGKVGPU_ESCAPE_PAUSE", pack=False, align=None),
    "DXGKVGPU_ESCAPE_POWERTRANSITIONCOMPLETE": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)), ("PowerState", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKVGPU_ESCAPE_POWERTRANSITIONCOMPLETE", pack=False, align=None),
    "DXGKVGPU_ESCAPE_READ_PCI_CONFIG": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKVGPU_ESCAPE_READ_PCI_CONFIG", pack=False, align=None),
    "DXGKVGPU_ESCAPE_READ_VGPU_TYPE": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)),)), name="DXGKVGPU_ESCAPE_READ_VGPU_TYPE", pack=False, align=None),
    "DXGKVGPU_ESCAPE_RELEASE": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)),)), name="DXGKVGPU_ESCAPE_RELEASE", pack=False, align=None),
    "DXGKVGPU_ESCAPE_RESUME": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)), ("DeviceLuid", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKVGPU_ESCAPE_RESUME", pack=False, align=None),
    "DXGKVGPU_ESCAPE_TYPE": SimTypeInt(signed=False, label="DXGKVGPU_ESCAPE_TYPE"),
    "DXGKVGPU_ESCAPE_WRITE_PCI_CONFIG": SimStruct(OrderedDict((("Header", SimTypeRef("DXGKVGPU_ESCAPE_HEAD", SimStruct)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="DXGKVGPU_ESCAPE_WRITE_PCI_CONFIG", pack=False, align=None),
    "DXGK_ADAPTER_PERFDATA": SimStruct(OrderedDict((("MemoryFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxMemoryFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxMemoryFrequencyOC", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("PCIEBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("FanRPM", SimTypeInt(signed=False, label="UInt32")), ("Power", SimTypeInt(signed=False, label="UInt32")), ("Temperature", SimTypeInt(signed=False, label="UInt32")), ("PowerStateOverride", SimTypeChar(label="Byte")),)), name="DXGK_ADAPTER_PERFDATA", pack=False, align=None),
    "DXGK_ADAPTER_PERFDATACAPS": SimStruct(OrderedDict((("MaxMemoryBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("MaxPCIEBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("MaxFanRPM", SimTypeInt(signed=False, label="UInt32")), ("TemperatureMax", SimTypeInt(signed=False, label="UInt32")), ("TemperatureWarning", SimTypeInt(signed=False, label="UInt32")),)), name="DXGK_ADAPTER_PERFDATACAPS", pack=False, align=None),
    "DXGK_BACKLIGHT_INFO": SimStruct(OrderedDict((("BacklightUsersetting", SimTypeShort(signed=False, label="UInt16")), ("BacklightEffective", SimTypeShort(signed=False, label="UInt16")), ("GammaRamp", SimTypeRef("D3DDDI_GAMMA_RAMP_RGB256x3x16", SimStruct)),)), name="DXGK_BACKLIGHT_INFO", pack=False, align=None),
    "DXGK_BACKLIGHT_OPTIMIZATION_LEVEL": SimTypeInt(signed=False, label="DXGK_BACKLIGHT_OPTIMIZATION_LEVEL"),
    "DXGK_BRIGHTNESS_CAPS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXGK_BRIGHTNESS_CAPS", pack=False, align=None),
    "DXGK_BRIGHTNESS_GET_NIT_RANGES_OUT": SimStruct(OrderedDict((("NormalRangeCount", SimTypeInt(signed=False, label="UInt32")), ("RangeCount", SimTypeInt(signed=False, label="UInt32")), ("PreferredMaximumBrightness", SimTypeInt(signed=False, label="UInt32")), ("SupportedRanges", SimTypeArray(SimTypeRef("DXGK_BRIGHTNESS_NIT_RANGE", SimStruct), 16)),)), name="DXGK_BRIGHTNESS_GET_NIT_RANGES_OUT", pack=False, align=None),
    "DXGK_BRIGHTNESS_GET_OUT": SimStruct(OrderedDict((("CurrentBrightnessMillinits", SimTypeInt(signed=False, label="UInt32")), ("TargetBrightnessMillinits", SimTypeInt(signed=False, label="UInt32")),)), name="DXGK_BRIGHTNESS_GET_OUT", pack=False, align=None),
    "DXGK_BRIGHTNESS_NIT_RANGE": SimStruct(OrderedDict((("MinimumLevelMillinit", SimTypeInt(signed=False, label="UInt32")), ("MaximumLevelMillinit", SimTypeInt(signed=False, label="UInt32")), ("StepSizeMillinit", SimTypeInt(signed=False, label="UInt32")),)), name="DXGK_BRIGHTNESS_NIT_RANGE", pack=False, align=None),
    "DXGK_BRIGHTNESS_SENSOR_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Flags": SimTypeRef("_Flags_e__Struct", SimStruct), "ValidSensorValues": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("AlsReading", SimTypeFloat(size=32)), ("Chromaticity", SimTypeRef("DXGK_BRIGHTNESS_SENSOR_DATA_CHROMATICITY", SimStruct)), ("ColorTemperature", SimTypeFloat(size=32)),)), name="DXGK_BRIGHTNESS_SENSOR_DATA", pack=False, align=None),
    "DXGK_BRIGHTNESS_SENSOR_DATA_CHROMATICITY": SimStruct(OrderedDict((("ChromaticityX", SimTypeFloat(size=32)), ("ChromaticityY", SimTypeFloat(size=32)),)), name="DXGK_BRIGHTNESS_SENSOR_DATA_CHROMATICITY", pack=False, align=None),
    "DXGK_BRIGHTNESS_SET_IN": SimStruct(OrderedDict((("BrightnessMillinits", SimTypeInt(signed=False, label="UInt32")), ("TransitionTimeMs", SimTypeInt(signed=False, label="UInt32")), ("SensorReadings", SimTypeRef("DXGK_BRIGHTNESS_SENSOR_DATA", SimStruct)),)), name="DXGK_BRIGHTNESS_SET_IN", pack=False, align=None),
    "DXGK_BRIGHTNESS_STATE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXGK_BRIGHTNESS_STATE", pack=False, align=None),
    "DXGK_CHILD_DEVICE_HPD_AWARENESS": SimTypeInt(signed=False, label="DXGK_CHILD_DEVICE_HPD_AWARENESS"),
    "DXGK_DISPLAY_DESCRIPTOR_TYPE": SimTypeInt(signed=False, label="DXGK_DISPLAY_DESCRIPTOR_TYPE"),
    "DXGK_DISPLAY_INFORMATION": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Pitch", SimTypeInt(signed=False, label="UInt32")), ("ColorFormat", SimTypeInt(signed=False, label="D3DDDIFORMAT")), ("PhysicAddress", SimTypeLongLong(signed=True, label="Int64")), ("TargetId", SimTypeInt(signed=False, label="UInt32")), ("AcpiId", SimTypeInt(signed=False, label="UInt32")),)), name="DXGK_DISPLAY_INFORMATION", pack=False, align=None),
    "DXGK_DISPLAY_TECHNOLOGY": SimTypeInt(signed=False, label="DXGK_DISPLAY_TECHNOLOGY"),
    "DXGK_DISPLAY_USAGE": SimTypeInt(signed=False, label="DXGK_DISPLAY_USAGE"),
    "DXGK_ENGINE_TYPE": SimTypeInt(signed=False, label="DXGK_ENGINE_TYPE"),
    "DXGK_ESCAPE_GPUMMUCAPS": SimStruct(OrderedDict((("ReadOnlyMemorySupported", SimTypeChar(label="Byte")), ("NoExecuteMemorySupported", SimTypeChar(label="Byte")), ("ZeroInPteSupported", SimTypeChar(label="Byte")), ("CacheCoherentMemorySupported", SimTypeChar(label="Byte")), ("LargePageSupported", SimTypeChar(label="Byte")), ("DualPteSupported", SimTypeChar(label="Byte")), ("AllowNonAlignedLargePageAddress", SimTypeChar(label="Byte")), ("VirtualAddressBitCount", SimTypeInt(signed=False, label="UInt32")), ("PageTableLevelCount", SimTypeInt(signed=False, label="UInt32")), ("PageTableLevelDesk", SimTypeArray(SimTypeRef("D3DKMT_PAGE_TABLE_LEVEL_DESC", SimStruct), 6)),)), name="DXGK_ESCAPE_GPUMMUCAPS", pack=False, align=None),
    "DXGK_FAULT_ERROR_CODE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="DXGK_FAULT_ERROR_CODE", pack=False, align=None),
    "DXGK_GENERAL_ERROR_CODE": SimTypeInt(signed=False, label="DXGK_GENERAL_ERROR_CODE"),
    "DXGK_GPUCLOCKDATA": SimStruct(OrderedDict((("GpuFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("GpuClockCounter", SimTypeLongLong(signed=False, label="UInt64")), ("CpuClockCounter", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeRef("DXGK_GPUCLOCKDATA_FLAGS", SimStruct)),)), name="DXGK_GPUCLOCKDATA", pack=False, align=None),
    "DXGK_GPUCLOCKDATA_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXGK_GPUCLOCKDATA_FLAGS", pack=False, align=None),
    "DXGK_GPUVERSION": SimStruct(OrderedDict((("BiosVersion", SimTypeArray(SimTypeChar(label="Char"), 32)), ("GpuArchitecture", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="DXGK_GPUVERSION", pack=False, align=None),
    "DXGK_GRAPHICSPOWER_REGISTER_INPUT_V_1_2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("PrivateHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PowerNotificationCb", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="DEVICE_POWER_STATE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "NewGrfxPowerState", "PreNotification", "PrivateHandle"]), offset=0)), ("RemovalNotificationCb", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "PrivateHandle"]), offset=0)), ("FStateNotificationCb", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "ComponentIndex", "NewFState", "PreNotification", "PrivateHandle"]), offset=0)), ("InitialComponentStateCb", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="Guid"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "PrivateHandle", "ComponentIndex", "IsBlockingType", "InitialFState", "ComponentGuid", "PowerComponentMappingFlag"]), offset=0)),)), name="DXGK_GRAPHICSPOWER_REGISTER_INPUT_V_1_2", pack=False, align=None),
    "DXGK_GRAPHICSPOWER_REGISTER_OUTPUT": SimStruct(OrderedDict((("DeviceHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InitialGrfxPowerState", SimTypeInt(signed=False, label="DEVICE_POWER_STATE")), ("SetSharedPowerComponentStateCb", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceHandle", "PrivateHandle", "ComponentIndex", "Active"]), offset=0)), ("UnregisterCb", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceHandle", "PrivateHandle"]), offset=0)),)), name="DXGK_GRAPHICSPOWER_REGISTER_OUTPUT", pack=False, align=None),
    "DXGK_MIRACAST_CHUNK_ID": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "DXGK_MIRACAST_CHUNK_INFO": SimStruct(OrderedDict((("ChunkType", SimTypeInt(signed=False, label="DXGK_MIRACAST_CHUNK_TYPE")), ("ChunkId", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ProcessingTime", SimTypeInt(signed=False, label="UInt32")), ("EncodeRate", SimTypeInt(signed=False, label="UInt32")),)), name="DXGK_MIRACAST_CHUNK_INFO", pack=False, align=None),
    "DXGK_MIRACAST_CHUNK_TYPE": SimTypeInt(signed=False, label="DXGK_MIRACAST_CHUNK_TYPE"),
    "DXGK_MONITORLINKINFO_CAPABILITIES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "DXGK_MONITORLINKINFO_USAGEHINTS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "DXGK_NODEMETADATA": SimStruct(OrderedDict((("EngineType", SimTypeInt(signed=False, label="DXGK_ENGINE_TYPE")), ("FriendlyName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("Flags", SimTypeRef("DXGK_NODEMETADATA_FLAGS", SimStruct)), ("GpuMmuSupported", SimTypeChar(label="Byte")), ("IoMmuSupported", SimTypeChar(label="Byte")),)), name="DXGK_NODEMETADATA", pack=False, align=None),
    "DXGK_NODEMETADATA_FLAGS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXGK_NODEMETADATA_FLAGS", pack=False, align=None),
    "DXGK_NODE_PERFDATA": SimStruct(OrderedDict((("Frequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("MaxFrequencyOC", SimTypeLongLong(signed=False, label="UInt64")), ("Voltage", SimTypeInt(signed=False, label="UInt32")), ("VoltageMax", SimTypeInt(signed=False, label="UInt32")), ("VoltageMaxOC", SimTypeInt(signed=False, label="UInt32")), ("MaxTransitionLatency", SimTypeLongLong(signed=False, label="UInt64")),)), name="DXGK_NODE_PERFDATA", pack=False, align=None),
    "DXGK_PAGE_FAULT_FLAGS": SimTypeInt(signed=False, label="DXGK_PAGE_FAULT_FLAGS"),
    "DXGK_PTE": SimStruct(OrderedDict((("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Flags": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"PageAddress": SimTypeLongLong(signed=False, label="UInt64"), "PageTableAddress": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="DXGK_PTE", pack=False, align=None),
    "DXGK_PTE_PAGE_SIZE": SimTypeInt(signed=False, label="DXGK_PTE_PAGE_SIZE"),
    "DXGK_RENDER_PIPELINE_STAGE": SimTypeInt(signed=False, label="DXGK_RENDER_PIPELINE_STAGE"),
    "DXGK_TARGETMODE_DETAIL_TIMING": SimStruct(OrderedDict((("VideoStandard", SimTypeInt(signed=False, label="D3DKMDT_VIDEO_SIGNAL_STANDARD")), ("TimingId", SimTypeInt(signed=False, label="UInt32")), ("DetailTiming", SimTypeRef("DISPLAYID_DETAILED_TIMING_TYPE_I", SimStruct)),)), name="DXGK_TARGETMODE_DETAIL_TIMING", pack=False, align=None),
    "DXGK_WIN32K_PARAM_DATA": SimStruct(OrderedDict((("PathsArray", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ModesArray", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumPathArrayElements", SimTypeInt(signed=False, label="UInt32")), ("NumModeArrayElements", SimTypeInt(signed=False, label="UInt32")), ("SDCFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DXGK_WIN32K_PARAM_DATA", pack=False, align=None),
    "DXVA2SW_CALLBACKS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("GetVideoProcessorRenderTargetCount", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "pCount"]), offset=0)), ("GetVideoProcessorRenderTargets", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "Count", "pFormats"]), offset=0)), ("GetVideoProcessorCaps", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypePointer(SimTypeRef("DXVA2_VideoProcessorCaps", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "pCaps"]), offset=0)), ("GetVideoProcessorSubStreamFormatCount", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "pCount"]), offset=0)), ("GetVideoProcessorSubStreamFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "Count", "pFormats"]), offset=0)), ("GetProcAmpRange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVA2_ValueRange", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "ProcAmpCap", "pRange"]), offset=0)), ("GetFilterPropertyRange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVA2_ValueRange", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "FilterSetting", "pRange"]), offset=0)), ("CreateVideoProcessDevice", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirect3DDevice9"), SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pD3DD9", "pVideoDesc", "RenderTargetFormat", "MaxSubStreams", "phDevice"]), offset=0)), ("DestroyVideoProcessDevice", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice"]), offset=0)), ("VideoProcessBeginFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice"]), offset=0)), ("VideoProcessEndFrame", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pHandleComplete"]), offset=0)), ("VideoProcessSetRenderTarget", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IDirect3DSurface9")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pRenderTarget"]), offset=0)), ("VideoProcessBlt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVA2_VIDEOPROCESSBLT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pBlt"]), offset=0)),)), name="DXVA2SW_CALLBACKS", pack=False, align=None),
    "DXVA2TraceVideoProcessBltData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pRenderTarget", SimTypeLongLong(signed=False, label="UInt64")), ("TargetFrameTime", SimTypeLongLong(signed=False, label="UInt64")), ("TargetRect", SimTypeRef("RECT", SimStruct)), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2TraceVideoProcessBltData", pack=False, align=None),
    "DXVA2Trace_DecodeDevBeginFrameData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pRenderTarget", SimTypeLongLong(signed=False, label="UInt64")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2Trace_DecodeDevBeginFrameData", pack=False, align=None),
    "DXVA2Trace_DecodeDevCreatedData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pD3DDevice", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2Trace_DecodeDevCreatedData", pack=False, align=None),
    "DXVA2Trace_DecodeDevGetBufferData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("BufferType", SimTypeInt(signed=False, label="UInt32")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2Trace_DecodeDevGetBufferData", pack=False, align=None),
    "DXVA2Trace_DecodeDeviceData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2Trace_DecodeDeviceData", pack=False, align=None),
    "DXVA2Trace_VideoProcessDevCreatedData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pD3DDevice", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("RTFourCC", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2Trace_VideoProcessDevCreatedData", pack=False, align=None),
    "DXVA2Trace_VideoProcessDeviceData": SimStruct(OrderedDict((("wmiHeader", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVA2Trace_VideoProcessDeviceData", pack=False, align=None),
    "DXVA2_AES_CTR_IV": SimStruct(OrderedDict((("IV", SimTypeLongLong(signed=False, label="UInt64")), ("Count", SimTypeLongLong(signed=False, label="UInt64")),)), name="DXVA2_AES_CTR_IV", pack=False, align=None),
    "DXVA2_AYUVSample16": SimStruct(OrderedDict((("Cr", SimTypeShort(signed=False, label="UInt16")), ("Cb", SimTypeShort(signed=False, label="UInt16")), ("Y", SimTypeShort(signed=False, label="UInt16")), ("Alpha", SimTypeShort(signed=False, label="UInt16")),)), name="DXVA2_AYUVSample16", pack=False, align=None),
    "DXVA2_AYUVSample8": SimStruct(OrderedDict((("Cr", SimTypeChar(label="Byte")), ("Cb", SimTypeChar(label="Byte")), ("Y", SimTypeChar(label="Byte")), ("Alpha", SimTypeChar(label="Byte")),)), name="DXVA2_AYUVSample8", pack=False, align=None),
    "DXVA2_BufferfType": SimTypeInt(signed=False, label="DXVA2_BufferfType"),
    "DXVA2_ConfigPictureDecode": SimStruct(OrderedDict((("guidConfigBitstreamEncryption", SimTypeBottom(label="Guid")), ("guidConfigMBcontrolEncryption", SimTypeBottom(label="Guid")), ("guidConfigResidDiffEncryption", SimTypeBottom(label="Guid")), ("ConfigBitstreamRaw", SimTypeInt(signed=False, label="UInt32")), ("ConfigMBcontrolRasterOrder", SimTypeInt(signed=False, label="UInt32")), ("ConfigResidDiffHost", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpatialResid8", SimTypeInt(signed=False, label="UInt32")), ("ConfigResid8Subtraction", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpatialHost8or9Clipping", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpatialResidInterleaved", SimTypeInt(signed=False, label="UInt32")), ("ConfigIntraResidUnsigned", SimTypeInt(signed=False, label="UInt32")), ("ConfigResidDiffAccelerator", SimTypeInt(signed=False, label="UInt32")), ("ConfigHostInverseScan", SimTypeInt(signed=False, label="UInt32")), ("ConfigSpecificIDCT", SimTypeInt(signed=False, label="UInt32")), ("Config4GroupedCoefs", SimTypeInt(signed=False, label="UInt32")), ("ConfigMinRenderTargetBuffCount", SimTypeShort(signed=False, label="UInt16")), ("ConfigDecoderSpecific", SimTypeShort(signed=False, label="UInt16")),)), name="DXVA2_ConfigPictureDecode", pack=False, align=None),
    "DXVA2_DecodeBufferDesc": SimStruct(OrderedDict((("CompressedBufferType", SimTypeInt(signed=False, label="UInt32")), ("BufferIndex", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("FirstMBaddress", SimTypeInt(signed=False, label="UInt32")), ("NumMBsInBuffer", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Stride", SimTypeInt(signed=False, label="UInt32")), ("ReservedBits", SimTypeInt(signed=False, label="UInt32")), ("pvPVPState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DXVA2_DecodeBufferDesc", pack=False, align=None),
    "DXVA2_DecodeExecuteParams": SimStruct(OrderedDict((("NumCompBuffers", SimTypeInt(signed=False, label="UInt32")), ("pCompressedBuffers", SimTypePointer(SimTypeRef("DXVA2_DecodeBufferDesc", SimStruct), offset=0)), ("pExtensionData", SimTypePointer(SimTypeRef("DXVA2_DecodeExtensionData", SimStruct), offset=0)),)), name="DXVA2_DecodeExecuteParams", pack=False, align=None),
    "DXVA2_DecodeExtensionData": SimStruct(OrderedDict((("Function", SimTypeInt(signed=False, label="UInt32")), ("pPrivateInputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateInputDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateOutputData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrivateOutputDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_DecodeExtensionData", pack=False, align=None),
    "DXVA2_DeinterlaceTech": SimTypeInt(signed=False, label="DXVA2_DeinterlaceTech"),
    "DXVA2_DestData": SimTypeInt(signed=False, label="DXVA2_DestData"),
    "DXVA2_DestinationFlags": SimTypeInt(signed=False, label="DXVA2_DestinationFlags"),
    "DXVA2_DetailFilterTech": SimTypeInt(signed=False, label="DXVA2_DetailFilterTech"),
    "DXVA2_ExtendedFormat": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXVA2_ExtendedFormat", pack=False, align=None),
    "DXVA2_FilterType": SimTypeInt(signed=False, label="DXVA2_FilterType"),
    "DXVA2_FilterValues": SimStruct(OrderedDict((("Level", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("Threshold", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("Radius", SimTypeRef("DXVA2_Fixed32", SimStruct)),)), name="DXVA2_FilterValues", pack=False, align=None),
    "DXVA2_Fixed32": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Fraction", SimTypeShort(signed=False, label="UInt16")), ("Value", SimTypeShort(signed=True, label="Int16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ll": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="DXVA2_Fixed32", pack=False, align=None),
    "DXVA2_Frequency": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_Frequency", pack=False, align=None),
    "DXVA2_NoiseFilterTech": SimTypeInt(signed=False, label="DXVA2_NoiseFilterTech"),
    "DXVA2_NominalRange": SimTypeInt(signed=False, label="DXVA2_NominalRange"),
    "DXVA2_ProcAmp": SimTypeInt(signed=False, label="DXVA2_ProcAmp"),
    "DXVA2_ProcAmpValues": SimStruct(OrderedDict((("Brightness", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("Contrast", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("Hue", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("Saturation", SimTypeRef("DXVA2_Fixed32", SimStruct)),)), name="DXVA2_ProcAmpValues", pack=False, align=None),
    "DXVA2_SampleData": SimTypeInt(signed=False, label="DXVA2_SampleData"),
    "DXVA2_SampleFlags": SimTypeInt(signed=False, label="DXVA2_SampleFlags"),
    "DXVA2_SampleFormat": SimTypeInt(signed=False, label="DXVA2_SampleFormat"),
    "DXVA2_SurfaceType": SimTypeInt(signed=False, label="DXVA2_SurfaceType"),
    "DXVA2_VIDEOPROCESSBLT": SimStruct(OrderedDict((("TargetFrame", SimTypeLongLong(signed=True, label="Int64")), ("TargetRect", SimTypeRef("RECT", SimStruct)), ("ConstrictionSize", SimTypeRef("SIZE", SimStruct)), ("StreamingFlags", SimTypeInt(signed=False, label="UInt32")), ("BackgroundColor", SimTypeRef("DXVA2_AYUVSample16", SimStruct)), ("DestFormat", SimTypeRef("DXVA2_ExtendedFormat", SimStruct)), ("DestFlags", SimTypeInt(signed=False, label="UInt32")), ("ProcAmpValues", SimTypeRef("DXVA2_ProcAmpValues", SimStruct)), ("Alpha", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("NoiseFilterLuma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("NoiseFilterChroma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("DetailFilterLuma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("DetailFilterChroma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("pSrcSurfaces", SimTypePointer(SimTypeRef("DXVA2_VIDEOSAMPLE", SimStruct), offset=0)), ("NumSrcSurfaces", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_VIDEOPROCESSBLT", pack=False, align=None),
    "DXVA2_VIDEOSAMPLE": SimStruct(OrderedDict((("Start", SimTypeLongLong(signed=True, label="Int64")), ("End", SimTypeLongLong(signed=True, label="Int64")), ("SampleFormat", SimTypeRef("DXVA2_ExtendedFormat", SimStruct)), ("SampleFlags", SimTypeInt(signed=False, label="UInt32")), ("SrcResource", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("DstRect", SimTypeRef("RECT", SimStruct)), ("Pal", SimTypeArray(SimTypeRef("DXVA2_AYUVSample8", SimStruct), 16)), ("PlanarAlpha", SimTypeRef("DXVA2_Fixed32", SimStruct)),)), name="DXVA2_VIDEOSAMPLE", pack=False, align=None),
    "DXVA2_VPDev": SimTypeInt(signed=False, label="DXVA2_VPDev"),
    "DXVA2_ValueRange": SimStruct(OrderedDict((("MinValue", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("MaxValue", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("DefaultValue", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("StepSize", SimTypeRef("DXVA2_Fixed32", SimStruct)),)), name="DXVA2_ValueRange", pack=False, align=None),
    "DXVA2_VideoChromaSubSampling": SimTypeInt(signed=False, label="DXVA2_VideoChromaSubSampling"),
    "DXVA2_VideoDesc": SimStruct(OrderedDict((("SampleWidth", SimTypeInt(signed=False, label="UInt32")), ("SampleHeight", SimTypeInt(signed=False, label="UInt32")), ("SampleFormat", SimTypeRef("DXVA2_ExtendedFormat", SimStruct)), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("InputSampleFreq", SimTypeRef("DXVA2_Frequency", SimStruct)), ("OutputFrameFreq", SimTypeRef("DXVA2_Frequency", SimStruct)), ("UABProtectionLevel", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_VideoDesc", pack=False, align=None),
    "DXVA2_VideoLighting": SimTypeInt(signed=False, label="DXVA2_VideoLighting"),
    "DXVA2_VideoPrimaries": SimTypeInt(signed=False, label="DXVA2_VideoPrimaries"),
    "DXVA2_VideoProcess": SimTypeInt(signed=False, label="DXVA2_VideoProcess"),
    "DXVA2_VideoProcessBltParams": SimStruct(OrderedDict((("TargetFrame", SimTypeLongLong(signed=True, label="Int64")), ("TargetRect", SimTypeRef("RECT", SimStruct)), ("ConstrictionSize", SimTypeRef("SIZE", SimStruct)), ("StreamingFlags", SimTypeInt(signed=False, label="UInt32")), ("BackgroundColor", SimTypeRef("DXVA2_AYUVSample16", SimStruct)), ("DestFormat", SimTypeRef("DXVA2_ExtendedFormat", SimStruct)), ("ProcAmpValues", SimTypeRef("DXVA2_ProcAmpValues", SimStruct)), ("Alpha", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("NoiseFilterLuma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("NoiseFilterChroma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("DetailFilterLuma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("DetailFilterChroma", SimTypeRef("DXVA2_FilterValues", SimStruct)), ("DestData", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_VideoProcessBltParams", pack=False, align=None),
    "DXVA2_VideoProcessorCaps": SimStruct(OrderedDict((("DeviceCaps", SimTypeInt(signed=False, label="UInt32")), ("InputPool", SimTypeInt(signed=False, label="D3DPOOL")), ("NumForwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("NumBackwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DeinterlaceTechnology", SimTypeInt(signed=False, label="UInt32")), ("ProcAmpControlCaps", SimTypeInt(signed=False, label="UInt32")), ("VideoProcessorOperations", SimTypeInt(signed=False, label="UInt32")), ("NoiseFilterTechnology", SimTypeInt(signed=False, label="UInt32")), ("DetailFilterTechnology", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_VideoProcessorCaps", pack=False, align=None),
    "DXVA2_VideoRenderTargetType": SimTypeInt(signed=False, label="DXVA2_VideoRenderTargetType"),
    "DXVA2_VideoSample": SimStruct(OrderedDict((("Start", SimTypeLongLong(signed=True, label="Int64")), ("End", SimTypeLongLong(signed=True, label="Int64")), ("SampleFormat", SimTypeRef("DXVA2_ExtendedFormat", SimStruct)), ("SrcSurface", SimTypeBottom(label="IDirect3DSurface9")), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("DstRect", SimTypeRef("RECT", SimStruct)), ("Pal", SimTypeArray(SimTypeRef("DXVA2_AYUVSample8", SimStruct), 16)), ("PlanarAlpha", SimTypeRef("DXVA2_Fixed32", SimStruct)), ("SampleData", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA2_VideoSample", pack=False, align=None),
    "DXVA2_VideoTransferFunction": SimTypeInt(signed=False, label="DXVA2_VideoTransferFunction"),
    "DXVA2_VideoTransferMatrix": SimTypeInt(signed=False, label="DXVA2_VideoTransferMatrix"),
    "DXVABufferInfo": SimStruct(OrderedDict((("pCompSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")),)), name="DXVABufferInfo", pack=False, align=None),
    "DXVACompBufferInfo": SimStruct(OrderedDict((("NumCompBuffers", SimTypeInt(signed=False, label="UInt32")), ("WidthToCreate", SimTypeInt(signed=False, label="UInt32")), ("HeightToCreate", SimTypeInt(signed=False, label="UInt32")), ("BytesToAllocate", SimTypeInt(signed=False, label="UInt32")), ("Usage", SimTypeInt(signed=False, label="UInt32")), ("Pool", SimTypeInt(signed=False, label="D3DPOOL")), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")),)), name="DXVACompBufferInfo", pack=False, align=None),
    "DXVAHDETW_CREATEVIDEOPROCESSOR": SimStruct(OrderedDict((("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pD3D9Ex", SimTypeLongLong(signed=False, label="UInt64")), ("VPGuid", SimTypeBottom(label="Guid")),)), name="DXVAHDETW_CREATEVIDEOPROCESSOR", pack=False, align=None),
    "DXVAHDETW_DESTROYVIDEOPROCESSOR": SimStruct(OrderedDict((("pObject", SimTypeLongLong(signed=False, label="UInt64")),)), name="DXVAHDETW_DESTROYVIDEOPROCESSOR", pack=False, align=None),
    "DXVAHDETW_VIDEOPROCESSBLTHD": SimStruct(OrderedDict((("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pOutputSurface", SimTypeLongLong(signed=False, label="UInt64")), ("TargetRect", SimTypeRef("RECT", SimStruct)), ("OutputFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="UInt32")), ("OutputFrame", SimTypeInt(signed=False, label="UInt32")), ("StreamCount", SimTypeInt(signed=False, label="UInt32")), ("Enter", SimTypeInt(signed=True, label="Int32")),)), name="DXVAHDETW_VIDEOPROCESSBLTHD", pack=False, align=None),
    "DXVAHDETW_VIDEOPROCESSBLTHD_STREAM": SimStruct(OrderedDict((("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("pInputSurface", SimTypeLongLong(signed=False, label="UInt64")), ("SourceRect", SimTypeRef("RECT", SimStruct)), ("DestinationRect", SimTypeRef("RECT", SimStruct)), ("InputFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("FrameFormat", SimTypeInt(signed=False, label="DXVAHD_FRAME_FORMAT")), ("ColorSpace", SimTypeInt(signed=False, label="UInt32")), ("StreamNumber", SimTypeInt(signed=False, label="UInt32")), ("OutputIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFrameOrField", SimTypeInt(signed=False, label="UInt32")), ("PastFrames", SimTypeInt(signed=False, label="UInt32")), ("FutureFrames", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHDETW_VIDEOPROCESSBLTHD_STREAM", pack=False, align=None),
    "DXVAHDETW_VIDEOPROCESSBLTSTATE": SimStruct(OrderedDict((("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypeInt(signed=False, label="DXVAHD_BLT_STATE")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("SetState", SimTypeInt(signed=True, label="Int32")),)), name="DXVAHDETW_VIDEOPROCESSBLTSTATE", pack=False, align=None),
    "DXVAHDETW_VIDEOPROCESSSTREAMSTATE": SimStruct(OrderedDict((("pObject", SimTypeLongLong(signed=False, label="UInt64")), ("StreamNumber", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="DXVAHD_STREAM_STATE")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("SetState", SimTypeInt(signed=True, label="Int32")),)), name="DXVAHDETW_VIDEOPROCESSSTREAMSTATE", pack=False, align=None),
    "DXVAHDSW_CALLBACKS": SimStruct(OrderedDict((("CreateDevice", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirect3DDevice9Ex"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pD3DDevice", "phDevice"]), offset=0)), ("ProposeVideoPrivateFormat", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pFormat"]), offset=0)), ("GetVideoProcessorDeviceCaps", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypePointer(SimTypeRef("DXVAHD_VPDEVCAPS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "pCaps"]), offset=0)), ("GetVideoProcessorOutputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "Count", "pFormats"]), offset=0)), ("GetVideoProcessorInputFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "Count", "pFormats"]), offset=0)), ("GetVideoProcessorCaps", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_VPCAPS", SimStruct), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "Count", "pCaps"]), offset=0)), ("GetVideoProcessorCustomRates", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_CUSTOM_RATE_DATA", SimStruct), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pVPGuid", "Count", "pRates"]), offset=0)), ("GetVideoProcessorFilterRange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DXVAHD_FILTER"), SimTypePointer(SimTypeRef("DXVAHD_FILTER_RANGE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "Filter", "pRange"]), offset=0)), ("DestroyDevice", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice"]), offset=0)), ("CreateVideoProcessor", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pVPGuid", "phVideoProcessor"]), offset=0)), ("SetVideoProcessBltState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DXVAHD_BLT_STATE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "State", "DataSize", "pData"]), offset=0)), ("GetVideoProcessBltStatePrivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_BLT_STATE_PRIVATE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "pData"]), offset=0)), ("SetVideoProcessStreamState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="DXVAHD_STREAM_STATE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "StreamNumber", "State", "DataSize", "pData"]), offset=0)), ("GetVideoProcessStreamStatePrivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_STREAM_STATE_PRIVATE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "StreamNumber", "pData"]), offset=0)), ("VideoProcessBltHD", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IDirect3DSurface9"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_STREAM_DATA", SimStruct), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "pOutputSurface", "OutputFrame", "StreamCount", "pStreams"]), offset=0)), ("DestroyVideoProcessor", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor"]), offset=0)),)), name="DXVAHDSW_CALLBACKS", pack=False, align=None),
    "DXVAHD_ALPHA_FILL_MODE": SimTypeInt(signed=False, label="DXVAHD_ALPHA_FILL_MODE"),
    "DXVAHD_BLT_STATE": SimTypeInt(signed=False, label="DXVAHD_BLT_STATE"),
    "DXVAHD_BLT_STATE_ALPHA_FILL_DATA": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="DXVAHD_ALPHA_FILL_MODE")), ("StreamNumber", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_BLT_STATE_ALPHA_FILL_DATA", pack=False, align=None),
    "DXVAHD_BLT_STATE_BACKGROUND_COLOR_DATA": SimStruct(OrderedDict((("YCbCr", SimTypeInt(signed=True, label="Int32")), ("BackgroundColor", SimUnion({"RGB": SimTypeRef("DXVAHD_COLOR_RGBA", SimStruct), "YCbCr": SimTypeRef("DXVAHD_COLOR_YCbCrA", SimStruct)}, name="<anon>", label="None")),)), name="DXVAHD_BLT_STATE_BACKGROUND_COLOR_DATA", pack=False, align=None),
    "DXVAHD_BLT_STATE_CONSTRICTION_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Size", SimTypeRef("SIZE", SimStruct)),)), name="DXVAHD_BLT_STATE_CONSTRICTION_DATA", pack=False, align=None),
    "DXVAHD_BLT_STATE_OUTPUT_COLOR_SPACE_DATA": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXVAHD_BLT_STATE_OUTPUT_COLOR_SPACE_DATA", pack=False, align=None),
    "DXVAHD_BLT_STATE_PRIVATE_DATA": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DXVAHD_BLT_STATE_PRIVATE_DATA", pack=False, align=None),
    "DXVAHD_BLT_STATE_TARGET_RECT_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("TargetRect", SimTypeRef("RECT", SimStruct)),)), name="DXVAHD_BLT_STATE_TARGET_RECT_DATA", pack=False, align=None),
    "DXVAHD_COLOR": SimUnion({"RGB": SimTypeRef("DXVAHD_COLOR_RGBA", SimStruct), "YCbCr": SimTypeRef("DXVAHD_COLOR_YCbCrA", SimStruct)}, name="<anon>", label="None"),
    "DXVAHD_COLOR_RGBA": SimStruct(OrderedDict((("R", SimTypeFloat(size=32)), ("G", SimTypeFloat(size=32)), ("B", SimTypeFloat(size=32)), ("A", SimTypeFloat(size=32)),)), name="DXVAHD_COLOR_RGBA", pack=False, align=None),
    "DXVAHD_COLOR_YCbCrA": SimStruct(OrderedDict((("Y", SimTypeFloat(size=32)), ("Cb", SimTypeFloat(size=32)), ("Cr", SimTypeFloat(size=32)), ("A", SimTypeFloat(size=32)),)), name="DXVAHD_COLOR_YCbCrA", pack=False, align=None),
    "DXVAHD_CONTENT_DESC": SimStruct(OrderedDict((("InputFrameFormat", SimTypeInt(signed=False, label="DXVAHD_FRAME_FORMAT")), ("InputFrameRate", SimTypeRef("DXVAHD_RATIONAL", SimStruct)), ("InputWidth", SimTypeInt(signed=False, label="UInt32")), ("InputHeight", SimTypeInt(signed=False, label="UInt32")), ("OutputFrameRate", SimTypeRef("DXVAHD_RATIONAL", SimStruct)), ("OutputWidth", SimTypeInt(signed=False, label="UInt32")), ("OutputHeight", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_CONTENT_DESC", pack=False, align=None),
    "DXVAHD_CUSTOM_RATE_DATA": SimStruct(OrderedDict((("CustomRate", SimTypeRef("DXVAHD_RATIONAL", SimStruct)), ("OutputFrames", SimTypeInt(signed=False, label="UInt32")), ("InputInterlaced", SimTypeInt(signed=True, label="Int32")), ("InputFramesOrFields", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_CUSTOM_RATE_DATA", pack=False, align=None),
    "DXVAHD_DEVICE_CAPS": SimTypeInt(signed=False, label="DXVAHD_DEVICE_CAPS"),
    "DXVAHD_DEVICE_TYPE": SimTypeInt(signed=False, label="DXVAHD_DEVICE_TYPE"),
    "DXVAHD_DEVICE_USAGE": SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"),
    "DXVAHD_FEATURE_CAPS": SimTypeInt(signed=False, label="DXVAHD_FEATURE_CAPS"),
    "DXVAHD_FILTER": SimTypeInt(signed=False, label="DXVAHD_FILTER"),
    "DXVAHD_FILTER_CAPS": SimTypeInt(signed=False, label="DXVAHD_FILTER_CAPS"),
    "DXVAHD_FILTER_RANGE_DATA": SimStruct(OrderedDict((("Minimum", SimTypeInt(signed=True, label="Int32")), ("Maximum", SimTypeInt(signed=True, label="Int32")), ("Default", SimTypeInt(signed=True, label="Int32")), ("Multiplier", SimTypeFloat(size=32)),)), name="DXVAHD_FILTER_RANGE_DATA", pack=False, align=None),
    "DXVAHD_FRAME_FORMAT": SimTypeInt(signed=False, label="DXVAHD_FRAME_FORMAT"),
    "DXVAHD_INPUT_FORMAT_CAPS": SimTypeInt(signed=False, label="DXVAHD_INPUT_FORMAT_CAPS"),
    "DXVAHD_ITELECINE_CAPS": SimTypeInt(signed=False, label="DXVAHD_ITELECINE_CAPS"),
    "DXVAHD_OUTPUT_RATE": SimTypeInt(signed=False, label="DXVAHD_OUTPUT_RATE"),
    "DXVAHD_PROCESSOR_CAPS": SimTypeInt(signed=False, label="DXVAHD_PROCESSOR_CAPS"),
    "DXVAHD_RATIONAL": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_RATIONAL", pack=False, align=None),
    "DXVAHD_STREAM_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("OutputIndex", SimTypeInt(signed=False, label="UInt32")), ("InputFrameOrField", SimTypeInt(signed=False, label="UInt32")), ("PastFrames", SimTypeInt(signed=False, label="UInt32")), ("FutureFrames", SimTypeInt(signed=False, label="UInt32")), ("ppPastSurfaces", SimTypePointer(SimTypeBottom(label="IDirect3DSurface9"), label="LPArray", offset=0)), ("pInputSurface", SimTypeBottom(label="IDirect3DSurface9")), ("ppFutureSurfaces", SimTypePointer(SimTypeBottom(label="IDirect3DSurface9"), label="LPArray", offset=0)),)), name="DXVAHD_STREAM_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE": SimTypeInt(signed=False, label="DXVAHD_STREAM_STATE"),
    "DXVAHD_STREAM_STATE_ALPHA_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Alpha", SimTypeFloat(size=32)),)), name="DXVAHD_STREAM_STATE_ALPHA_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_ASPECT_RATIO_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("SourceAspectRatio", SimTypeRef("DXVAHD_RATIONAL", SimStruct)), ("DestinationAspectRatio", SimTypeRef("DXVAHD_RATIONAL", SimStruct)),)), name="DXVAHD_STREAM_STATE_ASPECT_RATIO_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_D3DFORMAT_DATA": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="D3DFORMAT")),)), name="DXVAHD_STREAM_STATE_D3DFORMAT_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_DESTINATION_RECT_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("DestinationRect", SimTypeRef("RECT", SimStruct)),)), name="DXVAHD_STREAM_STATE_DESTINATION_RECT_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_FILTER_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Level", SimTypeInt(signed=True, label="Int32")),)), name="DXVAHD_STREAM_STATE_FILTER_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_FRAME_FORMAT_DATA": SimStruct(OrderedDict((("FrameFormat", SimTypeInt(signed=False, label="DXVAHD_FRAME_FORMAT")),)), name="DXVAHD_STREAM_STATE_FRAME_FORMAT_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_INPUT_COLOR_SPACE_DATA": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="DXVAHD_STREAM_STATE_INPUT_COLOR_SPACE_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_LUMA_KEY_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("Lower", SimTypeFloat(size=32)), ("Upper", SimTypeFloat(size=32)),)), name="DXVAHD_STREAM_STATE_LUMA_KEY_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_OUTPUT_RATE_DATA": SimStruct(OrderedDict((("RepeatFrame", SimTypeInt(signed=True, label="Int32")), ("OutputRate", SimTypeInt(signed=False, label="DXVAHD_OUTPUT_RATE")), ("CustomRate", SimTypeRef("DXVAHD_RATIONAL", SimStruct)),)), name="DXVAHD_STREAM_STATE_OUTPUT_RATE_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_PALETTE_DATA": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("pEntries", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="DXVAHD_STREAM_STATE_PALETTE_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_PRIVATE_DATA": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DXVAHD_STREAM_STATE_PRIVATE_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_PRIVATE_IVTC_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("ITelecineFlags", SimTypeInt(signed=False, label="UInt32")), ("Frames", SimTypeInt(signed=False, label="UInt32")), ("InputField", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_STREAM_STATE_PRIVATE_IVTC_DATA", pack=False, align=None),
    "DXVAHD_STREAM_STATE_SOURCE_RECT_DATA": SimStruct(OrderedDict((("Enable", SimTypeInt(signed=True, label="Int32")), ("SourceRect", SimTypeRef("RECT", SimStruct)),)), name="DXVAHD_STREAM_STATE_SOURCE_RECT_DATA", pack=False, align=None),
    "DXVAHD_SURFACE_TYPE": SimTypeInt(signed=False, label="DXVAHD_SURFACE_TYPE"),
    "DXVAHD_VPCAPS": SimStruct(OrderedDict((("VPGuid", SimTypeBottom(label="Guid")), ("PastFrames", SimTypeInt(signed=False, label="UInt32")), ("FutureFrames", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCaps", SimTypeInt(signed=False, label="UInt32")), ("ITelecineCaps", SimTypeInt(signed=False, label="UInt32")), ("CustomRateCount", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_VPCAPS", pack=False, align=None),
    "DXVAHD_VPDEVCAPS": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="DXVAHD_DEVICE_TYPE")), ("DeviceCaps", SimTypeInt(signed=False, label="UInt32")), ("FeatureCaps", SimTypeInt(signed=False, label="UInt32")), ("FilterCaps", SimTypeInt(signed=False, label="UInt32")), ("InputFormatCaps", SimTypeInt(signed=False, label="UInt32")), ("InputPool", SimTypeInt(signed=False, label="D3DPOOL")), ("OutputFormatCount", SimTypeInt(signed=False, label="UInt32")), ("InputFormatCount", SimTypeInt(signed=False, label="UInt32")), ("VideoProcessorCount", SimTypeInt(signed=False, label="UInt32")), ("MaxInputStreams", SimTypeInt(signed=False, label="UInt32")), ("MaxStreamStates", SimTypeInt(signed=False, label="UInt32")),)), name="DXVAHD_VPDEVCAPS", pack=False, align=None),
    "DXVAUncompDataInfo": SimStruct(OrderedDict((("UncompWidth", SimTypeInt(signed=False, label="UInt32")), ("UncompHeight", SimTypeInt(signed=False, label="UInt32")), ("UncompFormat", SimTypeInt(signed=False, label="D3DFORMAT")),)), name="DXVAUncompDataInfo", pack=False, align=None),
    "DXVA_AYUVsample2": SimStruct(OrderedDict((("bCrValue", SimTypeChar(label="Byte")), ("bCbValue", SimTypeChar(label="Byte")), ("bY_Value", SimTypeChar(label="Byte")), ("bSampleAlpha8", SimTypeChar(label="Byte")),)), name="DXVA_AYUVsample2", pack=False, align=None),
    "DXVA_BufferDescription": SimStruct(OrderedDict((("dwTypeIndex", SimTypeInt(signed=False, label="UInt32")), ("dwBufferIndex", SimTypeInt(signed=False, label="UInt32")), ("dwDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwFirstMBaddress", SimTypeInt(signed=False, label="UInt32")), ("dwNumMBsInBuffer", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")), ("dwReservedBits", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_BufferDescription", pack=False, align=None),
    "DXVA_COPPCommand": SimStruct(OrderedDict((("macKDI", SimTypeBottom(label="Guid")), ("guidCommandID", SimTypeBottom(label="Guid")), ("dwSequence", SimTypeInt(signed=False, label="UInt32")), ("cbSizeData", SimTypeInt(signed=False, label="UInt32")), ("CommandData", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="DXVA_COPPCommand", pack=False, align=None),
    "DXVA_COPPSetProtectionLevelCmdData": SimStruct(OrderedDict((("ProtType", SimTypeInt(signed=False, label="UInt32")), ("ProtLevel", SimTypeInt(signed=False, label="UInt32")), ("ExtendedInfoChangeMask", SimTypeInt(signed=False, label="UInt32")), ("ExtendedInfoData", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_COPPSetProtectionLevelCmdData", pack=False, align=None),
    "DXVA_COPPSetSignalingCmdData": SimStruct(OrderedDict((("ActiveTVProtectionStandard", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioChangeMask1", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioData1", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioChangeMask2", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioData2", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioChangeMask3", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioData3", SimTypeInt(signed=False, label="UInt32")), ("ExtendedInfoChangeMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ExtendedInfoData", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_COPPSetSignalingCmdData", pack=False, align=None),
    "DXVA_COPPSignature": SimStruct(OrderedDict((("Signature", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="DXVA_COPPSignature", pack=False, align=None),
    "DXVA_COPPStatusData": SimStruct(OrderedDict((("rApp", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwData", SimTypeInt(signed=False, label="UInt32")), ("ExtendedInfoValidMask", SimTypeInt(signed=False, label="UInt32")), ("ExtendedInfoData", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_COPPStatusData", pack=False, align=None),
    "DXVA_COPPStatusDisplayData": SimStruct(OrderedDict((("rApp", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("DisplayWidth", SimTypeInt(signed=False, label="UInt32")), ("DisplayHeight", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="UInt32")), ("d3dFormat", SimTypeInt(signed=False, label="UInt32")), ("FreqNumerator", SimTypeInt(signed=False, label="UInt32")), ("FreqDenominator", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_COPPStatusDisplayData", pack=False, align=None),
    "DXVA_COPPStatusHDCPKeyData": SimStruct(OrderedDict((("rApp", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwHDCPFlags", SimTypeInt(signed=False, label="UInt32")), ("BKey", SimTypeBottom(label="Guid")), ("Reserved1", SimTypeBottom(label="Guid")), ("Reserved2", SimTypeBottom(label="Guid")),)), name="DXVA_COPPStatusHDCPKeyData", pack=False, align=None),
    "DXVA_COPPStatusInput": SimStruct(OrderedDict((("rApp", SimTypeBottom(label="Guid")), ("guidStatusRequestID", SimTypeBottom(label="Guid")), ("dwSequence", SimTypeInt(signed=False, label="UInt32")), ("cbSizeData", SimTypeInt(signed=False, label="UInt32")), ("StatusData", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="DXVA_COPPStatusInput", pack=False, align=None),
    "DXVA_COPPStatusOutput": SimStruct(OrderedDict((("macKDI", SimTypeBottom(label="Guid")), ("cbSizeData", SimTypeInt(signed=False, label="UInt32")), ("COPPStatus", SimTypeArray(SimTypeChar(label="Byte"), 4076)),)), name="DXVA_COPPStatusOutput", pack=False, align=None),
    "DXVA_COPPStatusSignalingCmdData": SimStruct(OrderedDict((("rApp", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("AvailableTVProtectionStandards", SimTypeInt(signed=False, label="UInt32")), ("ActiveTVProtectionStandard", SimTypeInt(signed=False, label="UInt32")), ("TVType", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioValidMask1", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioData1", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioValidMask2", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioData2", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioValidMask3", SimTypeInt(signed=False, label="UInt32")), ("AspectRatioData3", SimTypeInt(signed=False, label="UInt32")), ("ExtendedInfoValidMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ExtendedInfoData", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="DXVA_COPPStatusSignalingCmdData", pack=False, align=None),
    "DXVA_ConfigPictureDecode": SimStruct(OrderedDict((("dwFunction", SimTypeInt(signed=False, label="UInt32")), ("dwReservedBits", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("guidConfigBitstreamEncryption", SimTypeBottom(label="Guid")), ("guidConfigMBcontrolEncryption", SimTypeBottom(label="Guid")), ("guidConfigResidDiffEncryption", SimTypeBottom(label="Guid")), ("bConfigBitstreamRaw", SimTypeChar(label="Byte")), ("bConfigMBcontrolRasterOrder", SimTypeChar(label="Byte")), ("bConfigResidDiffHost", SimTypeChar(label="Byte")), ("bConfigSpatialResid8", SimTypeChar(label="Byte")), ("bConfigResid8Subtraction", SimTypeChar(label="Byte")), ("bConfigSpatialHost8or9Clipping", SimTypeChar(label="Byte")), ("bConfigSpatialResidInterleaved", SimTypeChar(label="Byte")), ("bConfigIntraResidUnsigned", SimTypeChar(label="Byte")), ("bConfigResidDiffAccelerator", SimTypeChar(label="Byte")), ("bConfigHostInverseScan", SimTypeChar(label="Byte")), ("bConfigSpecificIDCT", SimTypeChar(label="Byte")), ("bConfig4GroupedCoefs", SimTypeChar(label="Byte")),)), name="DXVA_ConfigPictureDecode", pack=False, align=None),
    "DXVA_DeinterlaceBlt": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("rtTarget", SimTypeLongLong(signed=True, label="Int64")), ("DstRect", SimTypeRef("RECT", SimStruct)), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("NumSourceSurfaces", SimTypeInt(signed=False, label="UInt32")), ("Alpha", SimTypeFloat(size=32)), ("Source", SimTypeArray(SimTypeRef("DXVA_VideoSample", SimStruct), 32)),)), name="DXVA_DeinterlaceBlt", pack=False, align=None),
    "DXVA_DeinterlaceBltEx": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("BackgroundColor", SimTypeRef("DXVA_AYUVsample2", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("rtTarget", SimTypeLongLong(signed=True, label="Int64")), ("NumSourceSurfaces", SimTypeInt(signed=False, label="UInt32")), ("Alpha", SimTypeFloat(size=32)), ("Source", SimTypeArray(SimTypeRef("DXVA_VideoSample2", SimStruct), 32)), ("DestinationFormat", SimTypeInt(signed=False, label="UInt32")), ("DestinationFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_DeinterlaceBltEx", pack=False, align=None),
    "DXVA_DeinterlaceBltEx32": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("BackgroundColor", SimTypeRef("DXVA_AYUVsample2", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("rtTarget", SimTypeLongLong(signed=True, label="Int64")), ("NumSourceSurfaces", SimTypeInt(signed=False, label="UInt32")), ("Alpha", SimTypeFloat(size=32)), ("Source", SimTypeArray(SimTypeRef("DXVA_VideoSample32", SimStruct), 32)), ("DestinationFormat", SimTypeInt(signed=False, label="UInt32")), ("DestinationFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_DeinterlaceBltEx32", pack=False, align=None),
    "DXVA_DeinterlaceCaps": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("NumPreviousOutputFrames", SimTypeInt(signed=False, label="UInt32")), ("InputPool", SimTypeInt(signed=False, label="UInt32")), ("NumForwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("NumBackwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("d3dOutputFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("VideoProcessingCaps", SimTypeInt(signed=False, label="DXVA_VideoProcessCaps")), ("DeinterlaceTechnology", SimTypeInt(signed=False, label="DXVA_DeinterlaceTech")),)), name="DXVA_DeinterlaceCaps", pack=False, align=None),
    "DXVA_DeinterlaceQueryAvailableModes": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("NumGuids", SimTypeInt(signed=False, label="UInt32")), ("Guids", SimTypeArray(SimTypeBottom(label="Guid"), 32)),)), name="DXVA_DeinterlaceQueryAvailableModes", pack=False, align=None),
    "DXVA_DeinterlaceQueryModeCaps": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("VideoDesc", SimTypeRef("DXVA_VideoDesc", SimStruct)),)), name="DXVA_DeinterlaceQueryModeCaps", pack=False, align=None),
    "DXVA_DeinterlaceTech": SimTypeInt(signed=False, label="DXVA_DeinterlaceTech"),
    "DXVA_DestinationFlags": SimTypeInt(signed=False, label="DXVA_DestinationFlags"),
    "DXVA_ExtendedFormat": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_ExtendedFormat", pack=False, align=None),
    "DXVA_Frequency": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_Frequency", pack=False, align=None),
    "DXVA_NominalRange": SimTypeInt(signed=False, label="DXVA_NominalRange"),
    "DXVA_PictureParameters": SimStruct(OrderedDict((("wDecodedPictureIndex", SimTypeShort(signed=False, label="UInt16")), ("wDeblockedPictureIndex", SimTypeShort(signed=False, label="UInt16")), ("wForwardRefPictureIndex", SimTypeShort(signed=False, label="UInt16")), ("wBackwardRefPictureIndex", SimTypeShort(signed=False, label="UInt16")), ("wPicWidthInMBminus1", SimTypeShort(signed=False, label="UInt16")), ("wPicHeightInMBminus1", SimTypeShort(signed=False, label="UInt16")), ("bMacroblockWidthMinus1", SimTypeChar(label="Byte")), ("bMacroblockHeightMinus1", SimTypeChar(label="Byte")), ("bBlockWidthMinus1", SimTypeChar(label="Byte")), ("bBlockHeightMinus1", SimTypeChar(label="Byte")), ("bBPPminus1", SimTypeChar(label="Byte")), ("bPicStructure", SimTypeChar(label="Byte")), ("bSecondField", SimTypeChar(label="Byte")), ("bPicIntra", SimTypeChar(label="Byte")), ("bPicBackwardPrediction", SimTypeChar(label="Byte")), ("bBidirectionalAveragingMode", SimTypeChar(label="Byte")), ("bMVprecisionAndChromaRelation", SimTypeChar(label="Byte")), ("bChromaFormat", SimTypeChar(label="Byte")), ("bPicScanFixed", SimTypeChar(label="Byte")), ("bPicScanMethod", SimTypeChar(label="Byte")), ("bPicReadbackRequests", SimTypeChar(label="Byte")), ("bRcontrol", SimTypeChar(label="Byte")), ("bPicSpatialResid8", SimTypeChar(label="Byte")), ("bPicOverflowBlocks", SimTypeChar(label="Byte")), ("bPicExtrapolation", SimTypeChar(label="Byte")), ("bPicDeblocked", SimTypeChar(label="Byte")), ("bPicDeblockConfined", SimTypeChar(label="Byte")), ("bPic4MVallowed", SimTypeChar(label="Byte")), ("bPicOBMC", SimTypeChar(label="Byte")), ("bPicBinPB", SimTypeChar(label="Byte")), ("bMV_RPS", SimTypeChar(label="Byte")), ("bReservedBits", SimTypeChar(label="Byte")), ("wBitstreamFcodes", SimTypeShort(signed=False, label="UInt16")), ("wBitstreamPCEelements", SimTypeShort(signed=False, label="UInt16")), ("bBitstreamConcealmentNeed", SimTypeChar(label="Byte")), ("bBitstreamConcealmentMethod", SimTypeChar(label="Byte")),)), name="DXVA_PictureParameters", pack=False, align=None),
    "DXVA_ProcAmpControlBlt": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("DstRect", SimTypeRef("RECT", SimStruct)), ("SrcRect", SimTypeRef("RECT", SimStruct)), ("Alpha", SimTypeFloat(size=32)), ("Brightness", SimTypeFloat(size=32)), ("Contrast", SimTypeFloat(size=32)), ("Hue", SimTypeFloat(size=32)), ("Saturation", SimTypeFloat(size=32)),)), name="DXVA_ProcAmpControlBlt", pack=False, align=None),
    "DXVA_ProcAmpControlCaps": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("InputPool", SimTypeInt(signed=False, label="UInt32")), ("d3dOutputFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("ProcAmpControlProps", SimTypeInt(signed=False, label="UInt32")), ("VideoProcessingCaps", SimTypeInt(signed=False, label="UInt32")),)), name="DXVA_ProcAmpControlCaps", pack=False, align=None),
    "DXVA_ProcAmpControlProp": SimTypeInt(signed=False, label="DXVA_ProcAmpControlProp"),
    "DXVA_ProcAmpControlQueryRange": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ProcAmpControlProp", SimTypeInt(signed=False, label="DXVA_ProcAmpControlProp")), ("VideoDesc", SimTypeRef("DXVA_VideoDesc", SimStruct)),)), name="DXVA_ProcAmpControlQueryRange", pack=False, align=None),
    "DXVA_SampleFlags": SimTypeInt(signed=False, label="DXVA_SampleFlags"),
    "DXVA_SampleFormat": SimTypeInt(signed=False, label="DXVA_SampleFormat"),
    "DXVA_VideoChromaSubsampling": SimTypeInt(signed=False, label="DXVA_VideoChromaSubsampling"),
    "DXVA_VideoDesc": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("SampleWidth", SimTypeInt(signed=False, label="UInt32")), ("SampleHeight", SimTypeInt(signed=False, label="UInt32")), ("SampleFormat", SimTypeInt(signed=False, label="UInt32")), ("d3dFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("InputSampleFreq", SimTypeRef("DXVA_Frequency", SimStruct)), ("OutputFrameFreq", SimTypeRef("DXVA_Frequency", SimStruct)),)), name="DXVA_VideoDesc", pack=False, align=None),
    "DXVA_VideoLighting": SimTypeInt(signed=False, label="DXVA_VideoLighting"),
    "DXVA_VideoPrimaries": SimTypeInt(signed=False, label="DXVA_VideoPrimaries"),
    "DXVA_VideoProcessCaps": SimTypeInt(signed=False, label="DXVA_VideoProcessCaps"),
    "DXVA_VideoPropertyRange": SimStruct(OrderedDict((("MinValue", SimTypeFloat(size=32)), ("MaxValue", SimTypeFloat(size=32)), ("DefaultValue", SimTypeFloat(size=32)), ("StepSize", SimTypeFloat(size=32)),)), name="DXVA_VideoPropertyRange", pack=False, align=None),
    "DXVA_VideoSample": SimStruct(OrderedDict((("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("SampleFormat", SimTypeInt(signed=False, label="DXVA_SampleFormat")), ("lpDDSSrcSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="DXVA_VideoSample", pack=False, align=None),
    "DXVA_VideoSample2": SimStruct(OrderedDict((("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("SampleFormat", SimTypeInt(signed=False, label="UInt32")), ("SampleFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDDSSrcSurface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("rcSrc", SimTypeRef("RECT", SimStruct)), ("rcDst", SimTypeRef("RECT", SimStruct)), ("Palette", SimTypeArray(SimTypeRef("DXVA_AYUVsample2", SimStruct), 16)),)), name="DXVA_VideoSample2", pack=False, align=None),
    "DXVA_VideoSample32": SimStruct(OrderedDict((("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("SampleFormat", SimTypeInt(signed=False, label="UInt32")), ("SampleFlags", SimTypeInt(signed=False, label="UInt32")), ("lpDDSSrcSurface", SimTypeInt(signed=False, label="UInt32")), ("rcSrc", SimTypeRef("RECT", SimStruct)), ("rcDst", SimTypeRef("RECT", SimStruct)), ("Palette", SimTypeArray(SimTypeRef("DXVA_AYUVsample2", SimStruct), 16)),)), name="DXVA_VideoSample32", pack=False, align=None),
    "DXVA_VideoTransferFunction": SimTypeInt(signed=False, label="DXVA_VideoTransferFunction"),
    "DXVA_VideoTransferMatrix": SimTypeInt(signed=False, label="DXVA_VideoTransferMatrix"),
    "DX_FEATURE_LEVEL": SimTypeInt(signed=False, label="DX_FEATURE_LEVEL"),
    "DX_IRQDATA": SimStruct(OrderedDict((("dwIrqFlags", SimTypeInt(signed=False, label="UInt32")),)), name="DX_IRQDATA", pack=False, align=None),
    "DYNAMIC_RENDERER_CACHED_DATA": SimStruct(OrderedDict((("strokeId", SimTypeInt(signed=True, label="Int32")), ("dynamicRenderer", SimTypeBottom(label="IDynamicRenderer")),)), name="DYNAMIC_RENDERER_CACHED_DATA", pack=False, align=None),
    "DYNAMIC_TIME_ZONE_INFORMATION": SimStruct(OrderedDict((("Bias", SimTypeInt(signed=True, label="Int32")), ("StandardName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("StandardDate", SimTypeRef("SYSTEMTIME", SimStruct)), ("StandardBias", SimTypeInt(signed=True, label="Int32")), ("DaylightName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DaylightDate", SimTypeRef("SYSTEMTIME", SimStruct)), ("DaylightBias", SimTypeInt(signed=True, label="Int32")), ("TimeZoneKeyName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("DynamicDaylightTimeDisabled", SimTypeChar(label="Byte")),)), name="DYNAMIC_TIME_ZONE_INFORMATION", pack=False, align=None),
    "DashCap": SimTypeInt(signed=False, label="DashCap"),
    "DashStyle": SimTypeInt(signed=False, label="DashStyle"),
    "DataCollectorSetStatus": SimTypeInt(signed=False, label="DataCollectorSetStatus"),
    "DataCollectorType": SimTypeInt(signed=False, label="DataCollectorType"),
    "DataFlow": SimTypeInt(signed=False, label="DataFlow"),
    "DataManagerSteps": SimTypeInt(signed=False, label="DataManagerSteps"),
    "DataSource": SimTypeBottom(label="DataSource"),
    "DataSourceListener": SimTypeBottom(label="DataSourceListener"),
    "DataSourceObject": SimTypeBottom(label="DataSourceObject"),
    "DataSourceTypeConstants": SimTypeInt(signed=False, label="DataSourceTypeConstants"),
    "DdqAccessLevel": SimTypeInt(signed=False, label="DdqAccessLevel"),
    "DebugBaseEventCallbacks": SimTypeBottom(label="DebugBaseEventCallbacks"),
    "DebugBaseEventCallbacksWide": SimTypeBottom(label="DebugBaseEventCallbacksWide"),
    "DebugEventLevel": SimTypeInt(signed=False, label="DebugEventLevel"),
    "DebugEventProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="DebugEventLevel"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["level", "message"]), offset=0),
    "DebugPropertyInfo": SimStruct(OrderedDict((("m_dwValidFields", SimTypeInt(signed=False, label="UInt32")), ("m_bstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_bstrType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_bstrValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_bstrFullName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_dwAttrib", SimTypeInt(signed=False, label="UInt32")), ("m_pDebugProp", SimTypeBottom(label="IDebugProperty")),)), name="DebugPropertyInfo", pack=False, align=None),
    "DebugStackFrameDescriptor": SimStruct(OrderedDict((("pdsf", SimTypeBottom(label="IDebugStackFrame")), ("dwMin", SimTypeInt(signed=False, label="UInt32")), ("dwLim", SimTypeInt(signed=False, label="UInt32")), ("fFinal", SimTypeInt(signed=True, label="Int32")), ("punkFinal", SimTypeBottom(label="IUnknown")),)), name="DebugStackFrameDescriptor", pack=False, align=None),
    "DebugStackFrameDescriptor64": SimStruct(OrderedDict((("pdsf", SimTypeBottom(label="IDebugStackFrame")), ("dwMin", SimTypeLongLong(signed=False, label="UInt64")), ("dwLim", SimTypeLongLong(signed=False, label="UInt64")), ("fFinal", SimTypeInt(signed=True, label="Int32")), ("punkFinal", SimTypeBottom(label="IUnknown")),)), name="DebugStackFrameDescriptor64", pack=False, align=None),
    "DedupChunk": SimStruct(OrderedDict((("Hash", SimTypeRef("DedupHash", SimStruct)), ("Flags", SimTypeInt(signed=False, label="DedupChunkFlags")), ("LogicalSize", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")),)), name="DedupChunk", pack=False, align=None),
    "DedupChunkFlags": SimTypeInt(signed=False, label="DedupChunkFlags"),
    "DedupChunkingAlgorithm": SimTypeInt(signed=False, label="DedupChunkingAlgorithm"),
    "DedupCompressionAlgorithm": SimTypeInt(signed=False, label="DedupCompressionAlgorithm"),
    "DedupDataPortManagerOption": SimTypeInt(signed=False, label="DedupDataPortManagerOption"),
    "DedupDataPortRequestStatus": SimTypeInt(signed=False, label="DedupDataPortRequestStatus"),
    "DedupDataPortVolumeStatus": SimTypeInt(signed=False, label="DedupDataPortVolumeStatus"),
    "DedupHash": SimStruct(OrderedDict((("Hash", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="DedupHash", pack=False, align=None),
    "DedupHashingAlgorithm": SimTypeInt(signed=False, label="DedupHashingAlgorithm"),
    "DedupStream": SimStruct(OrderedDict((("Path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")), ("ChunkCount", SimTypeInt(signed=False, label="UInt32")),)), name="DedupStream", pack=False, align=None),
    "DedupStreamEntry": SimStruct(OrderedDict((("Hash", SimTypeRef("DedupHash", SimStruct)), ("LogicalSize", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")),)), name="DedupStreamEntry", pack=False, align=None),
    "DelayRetryAction": SimTypeInt(signed=False, label="DelayRetryAction"),
    "DeploymentAction": SimTypeInt(signed=False, label="DeploymentAction"),
    "DetectEncodingInfo": SimStruct(OrderedDict((("nLangID", SimTypeInt(signed=False, label="UInt32")), ("nCodePage", SimTypeInt(signed=False, label="UInt32")), ("nDocPercent", SimTypeInt(signed=True, label="Int32")), ("nConfidence", SimTypeInt(signed=True, label="Int32")),)), name="DetectEncodingInfo", pack=False, align=None),
    "DeviceDialogFunction": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICEDIALOGDATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "DeviceDiscoveryMechanism": SimTypeInt(signed=False, label="DeviceDiscoveryMechanism"),
    "DeviceStreamState": SimTypeInt(signed=False, label="DeviceStreamState"),
    "DiagnosticsInfo": SimStruct(OrderedDict((("cost", SimTypeInt(signed=True, label="Int32")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="DiagnosticsInfo", pack=False, align=None),
    "DigitalWindowSetting": SimStruct(OrderedDict((("OriginX", SimTypeFloat(size=64)), ("OriginY", SimTypeFloat(size=64)), ("WindowSize", SimTypeFloat(size=64)),)), name="DigitalWindowSetting", pack=False, align=None),
    "Direction": SimTypeInt(signed=False, label="Direction"),
    "DispApplicationCache": SimTypeBottom(label="DispApplicationCache"),
    "DispCEventObj": SimTypeBottom(label="DispCEventObj"),
    "DispCPlugins": SimTypeBottom(label="DispCPlugins"),
    "DispCPrintManagerTemplatePrinter": SimTypeBottom(label="DispCPrintManagerTemplatePrinter"),
    "DispCanvasGradient": SimTypeBottom(label="DispCanvasGradient"),
    "DispCanvasImageData": SimTypeBottom(label="DispCanvasImageData"),
    "DispCanvasPattern": SimTypeBottom(label="DispCanvasPattern"),
    "DispCanvasRenderingContext2D": SimTypeBottom(label="DispCanvasRenderingContext2D"),
    "DispCanvasTextMetrics": SimTypeBottom(label="DispCanvasTextMetrics"),
    "DispDOMBeforeUnloadEvent": SimTypeBottom(label="DispDOMBeforeUnloadEvent"),
    "DispDOMChildrenCollection": SimTypeBottom(label="DispDOMChildrenCollection"),
    "DispDOMCloseEvent": SimTypeBottom(label="DispDOMCloseEvent"),
    "DispDOMCompositionEvent": SimTypeBottom(label="DispDOMCompositionEvent"),
    "DispDOMCustomEvent": SimTypeBottom(label="DispDOMCustomEvent"),
    "DispDOMDocumentType": SimTypeBottom(label="DispDOMDocumentType"),
    "DispDOMDragEvent": SimTypeBottom(label="DispDOMDragEvent"),
    "DispDOMEvent": SimTypeBottom(label="DispDOMEvent"),
    "DispDOMException": SimTypeBottom(label="DispDOMException"),
    "DispDOMFocusEvent": SimTypeBottom(label="DispDOMFocusEvent"),
    "DispDOMKeyboardEvent": SimTypeBottom(label="DispDOMKeyboardEvent"),
    "DispDOMMSAnimationEvent": SimTypeBottom(label="DispDOMMSAnimationEvent"),
    "DispDOMMSManipulationEvent": SimTypeBottom(label="DispDOMMSManipulationEvent"),
    "DispDOMMSTransitionEvent": SimTypeBottom(label="DispDOMMSTransitionEvent"),
    "DispDOMMessageEvent": SimTypeBottom(label="DispDOMMessageEvent"),
    "DispDOMMouseEvent": SimTypeBottom(label="DispDOMMouseEvent"),
    "DispDOMMouseWheelEvent": SimTypeBottom(label="DispDOMMouseWheelEvent"),
    "DispDOMMutationEvent": SimTypeBottom(label="DispDOMMutationEvent"),
    "DispDOMParser": SimTypeBottom(label="DispDOMParser"),
    "DispDOMProcessingInstruction": SimTypeBottom(label="DispDOMProcessingInstruction"),
    "DispDOMProgressEvent": SimTypeBottom(label="DispDOMProgressEvent"),
    "DispDOMSiteModeEvent": SimTypeBottom(label="DispDOMSiteModeEvent"),
    "DispDOMStorageEvent": SimTypeBottom(label="DispDOMStorageEvent"),
    "DispDOMTextEvent": SimTypeBottom(label="DispDOMTextEvent"),
    "DispDOMUIEvent": SimTypeBottom(label="DispDOMUIEvent"),
    "DispDOMWheelEvent": SimTypeBottom(label="DispDOMWheelEvent"),
    "DispEventException": SimTypeBottom(label="DispEventException"),
    "DispHTCAttachBehavior": SimTypeBottom(label="DispHTCAttachBehavior"),
    "DispHTCDefaultDispatch": SimTypeBottom(label="DispHTCDefaultDispatch"),
    "DispHTCDescBehavior": SimTypeBottom(label="DispHTCDescBehavior"),
    "DispHTCEventBehavior": SimTypeBottom(label="DispHTCEventBehavior"),
    "DispHTCMethodBehavior": SimTypeBottom(label="DispHTCMethodBehavior"),
    "DispHTCPropertyBehavior": SimTypeBottom(label="DispHTCPropertyBehavior"),
    "DispHTMLAnchorElement": SimTypeBottom(label="DispHTMLAnchorElement"),
    "DispHTMLAppBehavior": SimTypeBottom(label="DispHTMLAppBehavior"),
    "DispHTMLAreaElement": SimTypeBottom(label="DispHTMLAreaElement"),
    "DispHTMLAreasCollection": SimTypeBottom(label="DispHTMLAreasCollection"),
    "DispHTMLAttributeCollection": SimTypeBottom(label="DispHTMLAttributeCollection"),
    "DispHTMLAudioElement": SimTypeBottom(label="DispHTMLAudioElement"),
    "DispHTMLBGsound": SimTypeBottom(label="DispHTMLBGsound"),
    "DispHTMLBRElement": SimTypeBottom(label="DispHTMLBRElement"),
    "DispHTMLBaseElement": SimTypeBottom(label="DispHTMLBaseElement"),
    "DispHTMLBaseFontElement": SimTypeBottom(label="DispHTMLBaseFontElement"),
    "DispHTMLBlockElement": SimTypeBottom(label="DispHTMLBlockElement"),
    "DispHTMLBody": SimTypeBottom(label="DispHTMLBody"),
    "DispHTMLButtonElement": SimTypeBottom(label="DispHTMLButtonElement"),
    "DispHTMLCSSImportRule": SimTypeBottom(label="DispHTMLCSSImportRule"),
    "DispHTMLCSSMediaList": SimTypeBottom(label="DispHTMLCSSMediaList"),
    "DispHTMLCSSMediaRule": SimTypeBottom(label="DispHTMLCSSMediaRule"),
    "DispHTMLCSSNamespaceRule": SimTypeBottom(label="DispHTMLCSSNamespaceRule"),
    "DispHTMLCSSRule": SimTypeBottom(label="DispHTMLCSSRule"),
    "DispHTMLCSSStyleDeclaration": SimTypeBottom(label="DispHTMLCSSStyleDeclaration"),
    "DispHTMLCanvasElement": SimTypeBottom(label="DispHTMLCanvasElement"),
    "DispHTMLCommentElement": SimTypeBottom(label="DispHTMLCommentElement"),
    "DispHTMLCurrentStyle": SimTypeBottom(label="DispHTMLCurrentStyle"),
    "DispHTMLDDElement": SimTypeBottom(label="DispHTMLDDElement"),
    "DispHTMLDListElement": SimTypeBottom(label="DispHTMLDListElement"),
    "DispHTMLDOMAttribute": SimTypeBottom(label="DispHTMLDOMAttribute"),
    "DispHTMLDOMImplementation": SimTypeBottom(label="DispHTMLDOMImplementation"),
    "DispHTMLDOMRange": SimTypeBottom(label="DispHTMLDOMRange"),
    "DispHTMLDOMTextNode": SimTypeBottom(label="DispHTMLDOMTextNode"),
    "DispHTMLDTElement": SimTypeBottom(label="DispHTMLDTElement"),
    "DispHTMLDefaults": SimTypeBottom(label="DispHTMLDefaults"),
    "DispHTMLDivElement": SimTypeBottom(label="DispHTMLDivElement"),
    "DispHTMLDivPosition": SimTypeBottom(label="DispHTMLDivPosition"),
    "DispHTMLDocument": SimTypeBottom(label="DispHTMLDocument"),
    "DispHTMLDocumentCompatibleInfo": SimTypeBottom(label="DispHTMLDocumentCompatibleInfo"),
    "DispHTMLDocumentCompatibleInfoCollection": SimTypeBottom(label="DispHTMLDocumentCompatibleInfoCollection"),
    "DispHTMLElementCollection": SimTypeBottom(label="DispHTMLElementCollection"),
    "DispHTMLEmbed": SimTypeBottom(label="DispHTMLEmbed"),
    "DispHTMLFieldSetElement": SimTypeBottom(label="DispHTMLFieldSetElement"),
    "DispHTMLFontElement": SimTypeBottom(label="DispHTMLFontElement"),
    "DispHTMLFormElement": SimTypeBottom(label="DispHTMLFormElement"),
    "DispHTMLFrameBase": SimTypeBottom(label="DispHTMLFrameBase"),
    "DispHTMLFrameElement": SimTypeBottom(label="DispHTMLFrameElement"),
    "DispHTMLFrameSetSite": SimTypeBottom(label="DispHTMLFrameSetSite"),
    "DispHTMLGenericElement": SimTypeBottom(label="DispHTMLGenericElement"),
    "DispHTMLHRElement": SimTypeBottom(label="DispHTMLHRElement"),
    "DispHTMLHeadElement": SimTypeBottom(label="DispHTMLHeadElement"),
    "DispHTMLHeaderElement": SimTypeBottom(label="DispHTMLHeaderElement"),
    "DispHTMLHistory": SimTypeBottom(label="DispHTMLHistory"),
    "DispHTMLHtmlElement": SimTypeBottom(label="DispHTMLHtmlElement"),
    "DispHTMLIFrame": SimTypeBottom(label="DispHTMLIFrame"),
    "DispHTMLImg": SimTypeBottom(label="DispHTMLImg"),
    "DispHTMLInputElement": SimTypeBottom(label="DispHTMLInputElement"),
    "DispHTMLIsIndexElement": SimTypeBottom(label="DispHTMLIsIndexElement"),
    "DispHTMLLIElement": SimTypeBottom(label="DispHTMLLIElement"),
    "DispHTMLLabelElement": SimTypeBottom(label="DispHTMLLabelElement"),
    "DispHTMLLegendElement": SimTypeBottom(label="DispHTMLLegendElement"),
    "DispHTMLLinkElement": SimTypeBottom(label="DispHTMLLinkElement"),
    "DispHTMLListElement": SimTypeBottom(label="DispHTMLListElement"),
    "DispHTMLLocation": SimTypeBottom(label="DispHTMLLocation"),
    "DispHTMLMSCSSKeyframeRule": SimTypeBottom(label="DispHTMLMSCSSKeyframeRule"),
    "DispHTMLMSCSSKeyframesRule": SimTypeBottom(label="DispHTMLMSCSSKeyframesRule"),
    "DispHTMLMapElement": SimTypeBottom(label="DispHTMLMapElement"),
    "DispHTMLMarqueeElement": SimTypeBottom(label="DispHTMLMarqueeElement"),
    "DispHTMLMediaElement": SimTypeBottom(label="DispHTMLMediaElement"),
    "DispHTMLMediaError": SimTypeBottom(label="DispHTMLMediaError"),
    "DispHTMLMetaElement": SimTypeBottom(label="DispHTMLMetaElement"),
    "DispHTMLNamespace": SimTypeBottom(label="DispHTMLNamespace"),
    "DispHTMLNamespaceCollection": SimTypeBottom(label="DispHTMLNamespaceCollection"),
    "DispHTMLNavigator": SimTypeBottom(label="DispHTMLNavigator"),
    "DispHTMLNextIdElement": SimTypeBottom(label="DispHTMLNextIdElement"),
    "DispHTMLNoShowElement": SimTypeBottom(label="DispHTMLNoShowElement"),
    "DispHTMLOListElement": SimTypeBottom(label="DispHTMLOListElement"),
    "DispHTMLObjectElement": SimTypeBottom(label="DispHTMLObjectElement"),
    "DispHTMLOptionElement": SimTypeBottom(label="DispHTMLOptionElement"),
    "DispHTMLParaElement": SimTypeBottom(label="DispHTMLParaElement"),
    "DispHTMLParamElement": SimTypeBottom(label="DispHTMLParamElement"),
    "DispHTMLPerformance": SimTypeBottom(label="DispHTMLPerformance"),
    "DispHTMLPerformanceNavigation": SimTypeBottom(label="DispHTMLPerformanceNavigation"),
    "DispHTMLPerformanceTiming": SimTypeBottom(label="DispHTMLPerformanceTiming"),
    "DispHTMLPhraseElement": SimTypeBottom(label="DispHTMLPhraseElement"),
    "DispHTMLPopup": SimTypeBottom(label="DispHTMLPopup"),
    "DispHTMLProgressElement": SimTypeBottom(label="DispHTMLProgressElement"),
    "DispHTMLRenderStyle": SimTypeBottom(label="DispHTMLRenderStyle"),
    "DispHTMLRichtextElement": SimTypeBottom(label="DispHTMLRichtextElement"),
    "DispHTMLRuleStyle": SimTypeBottom(label="DispHTMLRuleStyle"),
    "DispHTMLScreen": SimTypeBottom(label="DispHTMLScreen"),
    "DispHTMLScriptElement": SimTypeBottom(label="DispHTMLScriptElement"),
    "DispHTMLSelectElement": SimTypeBottom(label="DispHTMLSelectElement"),
    "DispHTMLSemanticElement": SimTypeBottom(label="DispHTMLSemanticElement"),
    "DispHTMLSourceElement": SimTypeBottom(label="DispHTMLSourceElement"),
    "DispHTMLSpanElement": SimTypeBottom(label="DispHTMLSpanElement"),
    "DispHTMLSpanFlow": SimTypeBottom(label="DispHTMLSpanFlow"),
    "DispHTMLStorage": SimTypeBottom(label="DispHTMLStorage"),
    "DispHTMLStyle": SimTypeBottom(label="DispHTMLStyle"),
    "DispHTMLStyleElement": SimTypeBottom(label="DispHTMLStyleElement"),
    "DispHTMLStyleFontFace": SimTypeBottom(label="DispHTMLStyleFontFace"),
    "DispHTMLStyleMedia": SimTypeBottom(label="DispHTMLStyleMedia"),
    "DispHTMLStyleSheet": SimTypeBottom(label="DispHTMLStyleSheet"),
    "DispHTMLStyleSheetPage": SimTypeBottom(label="DispHTMLStyleSheetPage"),
    "DispHTMLStyleSheetPagesCollection": SimTypeBottom(label="DispHTMLStyleSheetPagesCollection"),
    "DispHTMLStyleSheetRule": SimTypeBottom(label="DispHTMLStyleSheetRule"),
    "DispHTMLStyleSheetRulesAppliedCollection": SimTypeBottom(label="DispHTMLStyleSheetRulesAppliedCollection"),
    "DispHTMLStyleSheetRulesCollection": SimTypeBottom(label="DispHTMLStyleSheetRulesCollection"),
    "DispHTMLStyleSheetsCollection": SimTypeBottom(label="DispHTMLStyleSheetsCollection"),
    "DispHTMLTable": SimTypeBottom(label="DispHTMLTable"),
    "DispHTMLTableCaption": SimTypeBottom(label="DispHTMLTableCaption"),
    "DispHTMLTableCell": SimTypeBottom(label="DispHTMLTableCell"),
    "DispHTMLTableCol": SimTypeBottom(label="DispHTMLTableCol"),
    "DispHTMLTableRow": SimTypeBottom(label="DispHTMLTableRow"),
    "DispHTMLTableSection": SimTypeBottom(label="DispHTMLTableSection"),
    "DispHTMLTextAreaElement": SimTypeBottom(label="DispHTMLTextAreaElement"),
    "DispHTMLTextElement": SimTypeBottom(label="DispHTMLTextElement"),
    "DispHTMLTimeRanges": SimTypeBottom(label="DispHTMLTimeRanges"),
    "DispHTMLTitleElement": SimTypeBottom(label="DispHTMLTitleElement"),
    "DispHTMLUListElement": SimTypeBottom(label="DispHTMLUListElement"),
    "DispHTMLUnknownElement": SimTypeBottom(label="DispHTMLUnknownElement"),
    "DispHTMLUrnCollection": SimTypeBottom(label="DispHTMLUrnCollection"),
    "DispHTMLVideoElement": SimTypeBottom(label="DispHTMLVideoElement"),
    "DispHTMLW3CComputedStyle": SimTypeBottom(label="DispHTMLW3CComputedStyle"),
    "DispHTMLWindow2": SimTypeBottom(label="DispHTMLWindow2"),
    "DispHTMLWindowProxy": SimTypeBottom(label="DispHTMLWindowProxy"),
    "DispHTMLWndOptionElement": SimTypeBottom(label="DispHTMLWndOptionElement"),
    "DispHTMLWndSelectElement": SimTypeBottom(label="DispHTMLWndSelectElement"),
    "DispHTMLXMLHttpRequest": SimTypeBottom(label="DispHTMLXMLHttpRequest"),
    "DispIHTMLInputButtonElement": SimTypeBottom(label="DispIHTMLInputButtonElement"),
    "DispIHTMLInputFileElement": SimTypeBottom(label="DispIHTMLInputFileElement"),
    "DispIHTMLInputImage": SimTypeBottom(label="DispIHTMLInputImage"),
    "DispIHTMLInputTextElement": SimTypeBottom(label="DispIHTMLInputTextElement"),
    "DispIHTMLOptionButtonElement": SimTypeBottom(label="DispIHTMLOptionButtonElement"),
    "DispNodeIterator": SimTypeBottom(label="DispNodeIterator"),
    "DispRangeException": SimTypeBottom(label="DispRangeException"),
    "DispRulesApplied": SimTypeBottom(label="DispRulesApplied"),
    "DispRulesAppliedCollection": SimTypeBottom(label="DispRulesAppliedCollection"),
    "DispSVGAElement": SimTypeBottom(label="DispSVGAElement"),
    "DispSVGCircleElement": SimTypeBottom(label="DispSVGCircleElement"),
    "DispSVGClipPathElement": SimTypeBottom(label="DispSVGClipPathElement"),
    "DispSVGDefsElement": SimTypeBottom(label="DispSVGDefsElement"),
    "DispSVGDescElement": SimTypeBottom(label="DispSVGDescElement"),
    "DispSVGElement": SimTypeBottom(label="DispSVGElement"),
    "DispSVGElementInstance": SimTypeBottom(label="DispSVGElementInstance"),
    "DispSVGElementInstanceList": SimTypeBottom(label="DispSVGElementInstanceList"),
    "DispSVGEllipseElement": SimTypeBottom(label="DispSVGEllipseElement"),
    "DispSVGException": SimTypeBottom(label="DispSVGException"),
    "DispSVGGElement": SimTypeBottom(label="DispSVGGElement"),
    "DispSVGGradientElement": SimTypeBottom(label="DispSVGGradientElement"),
    "DispSVGImageElement": SimTypeBottom(label="DispSVGImageElement"),
    "DispSVGLineElement": SimTypeBottom(label="DispSVGLineElement"),
    "DispSVGLinearGradientElement": SimTypeBottom(label="DispSVGLinearGradientElement"),
    "DispSVGMarkerElement": SimTypeBottom(label="DispSVGMarkerElement"),
    "DispSVGMaskElement": SimTypeBottom(label="DispSVGMaskElement"),
    "DispSVGMetadataElement": SimTypeBottom(label="DispSVGMetadataElement"),
    "DispSVGPathElement": SimTypeBottom(label="DispSVGPathElement"),
    "DispSVGPathSegArcAbs": SimTypeBottom(label="DispSVGPathSegArcAbs"),
    "DispSVGPathSegArcRel": SimTypeBottom(label="DispSVGPathSegArcRel"),
    "DispSVGPathSegClosePath": SimTypeBottom(label="DispSVGPathSegClosePath"),
    "DispSVGPathSegCurvetoCubicAbs": SimTypeBottom(label="DispSVGPathSegCurvetoCubicAbs"),
    "DispSVGPathSegCurvetoCubicRel": SimTypeBottom(label="DispSVGPathSegCurvetoCubicRel"),
    "DispSVGPathSegCurvetoCubicSmoothAbs": SimTypeBottom(label="DispSVGPathSegCurvetoCubicSmoothAbs"),
    "DispSVGPathSegCurvetoCubicSmoothRel": SimTypeBottom(label="DispSVGPathSegCurvetoCubicSmoothRel"),
    "DispSVGPathSegCurvetoQuadraticAbs": SimTypeBottom(label="DispSVGPathSegCurvetoQuadraticAbs"),
    "DispSVGPathSegCurvetoQuadraticRel": SimTypeBottom(label="DispSVGPathSegCurvetoQuadraticRel"),
    "DispSVGPathSegCurvetoQuadraticSmoothAbs": SimTypeBottom(label="DispSVGPathSegCurvetoQuadraticSmoothAbs"),
    "DispSVGPathSegCurvetoQuadraticSmoothRel": SimTypeBottom(label="DispSVGPathSegCurvetoQuadraticSmoothRel"),
    "DispSVGPathSegLinetoAbs": SimTypeBottom(label="DispSVGPathSegLinetoAbs"),
    "DispSVGPathSegLinetoHorizontalAbs": SimTypeBottom(label="DispSVGPathSegLinetoHorizontalAbs"),
    "DispSVGPathSegLinetoHorizontalRel": SimTypeBottom(label="DispSVGPathSegLinetoHorizontalRel"),
    "DispSVGPathSegLinetoRel": SimTypeBottom(label="DispSVGPathSegLinetoRel"),
    "DispSVGPathSegLinetoVerticalAbs": SimTypeBottom(label="DispSVGPathSegLinetoVerticalAbs"),
    "DispSVGPathSegLinetoVerticalRel": SimTypeBottom(label="DispSVGPathSegLinetoVerticalRel"),
    "DispSVGPathSegMovetoAbs": SimTypeBottom(label="DispSVGPathSegMovetoAbs"),
    "DispSVGPathSegMovetoRel": SimTypeBottom(label="DispSVGPathSegMovetoRel"),
    "DispSVGPatternElement": SimTypeBottom(label="DispSVGPatternElement"),
    "DispSVGPolygonElement": SimTypeBottom(label="DispSVGPolygonElement"),
    "DispSVGPolylineElement": SimTypeBottom(label="DispSVGPolylineElement"),
    "DispSVGRadialGradientElement": SimTypeBottom(label="DispSVGRadialGradientElement"),
    "DispSVGRectElement": SimTypeBottom(label="DispSVGRectElement"),
    "DispSVGSVGElement": SimTypeBottom(label="DispSVGSVGElement"),
    "DispSVGScriptElement": SimTypeBottom(label="DispSVGScriptElement"),
    "DispSVGStopElement": SimTypeBottom(label="DispSVGStopElement"),
    "DispSVGStyleElement": SimTypeBottom(label="DispSVGStyleElement"),
    "DispSVGSwitchElement": SimTypeBottom(label="DispSVGSwitchElement"),
    "DispSVGSymbolElement": SimTypeBottom(label="DispSVGSymbolElement"),
    "DispSVGTSpanElement": SimTypeBottom(label="DispSVGTSpanElement"),
    "DispSVGTextContentElement": SimTypeBottom(label="DispSVGTextContentElement"),
    "DispSVGTextElement": SimTypeBottom(label="DispSVGTextElement"),
    "DispSVGTextPathElement": SimTypeBottom(label="DispSVGTextPathElement"),
    "DispSVGTextPositioningElement": SimTypeBottom(label="DispSVGTextPositioningElement"),
    "DispSVGTitleElement": SimTypeBottom(label="DispSVGTitleElement"),
    "DispSVGUseElement": SimTypeBottom(label="DispSVGUseElement"),
    "DispSVGViewElement": SimTypeBottom(label="DispSVGViewElement"),
    "DispSVGZoomEvent": SimTypeBottom(label="DispSVGZoomEvent"),
    "DispStaticNodeList": SimTypeBottom(label="DispStaticNodeList"),
    "DispTreeWalker": SimTypeBottom(label="DispTreeWalker"),
    "DispWebGeocoordinates": SimTypeBottom(label="DispWebGeocoordinates"),
    "DispWebGeolocation": SimTypeBottom(label="DispWebGeolocation"),
    "DispWebGeoposition": SimTypeBottom(label="DispWebGeoposition"),
    "DispWebGeopositionError": SimTypeBottom(label="DispWebGeopositionError"),
    "DispXDomainRequest": SimTypeBottom(label="DispXDomainRequest"),
    "DispXMLHttpRequestEventTarget": SimTypeBottom(label="DispXMLHttpRequestEventTarget"),
    "DispXMLSerializer": SimTypeBottom(label="DispXMLSerializer"),
    "DisplayMode": SimStruct(OrderedDict((("DeviceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("devMode", SimTypeRef("DEVMODEW", SimStruct)),)), name="DisplayMode", pack=False, align=None),
    "DisplayModes": SimStruct(OrderedDict((("numDisplayModes", SimTypeInt(signed=True, label="Int32")), ("displayMode", SimTypePointer(SimTypeRef("DisplayMode", SimStruct), offset=0)),)), name="DisplayModes", pack=False, align=None),
    "DisplaySizeList": SimTypeInt(signed=False, label="DisplaySizeList"),
    "DisplayTypeConstants": SimTypeInt(signed=False, label="DisplayTypeConstants"),
    "DitherType": SimTypeInt(signed=False, label="DitherType"),
    "DockPosition": SimTypeInt(signed=False, label="DockPosition"),
    "Document": SimTypeBottom(label="Document"),
    "DomConstructor": SimTypeInt(signed=False, label="DomConstructor"),
    "DownResEventParam": SimTypeInt(signed=False, label="DownResEventParam"),
    "DownloadPhase": SimTypeInt(signed=False, label="DownloadPhase"),
    "DownloadPriority": SimTypeInt(signed=False, label="DownloadPriority"),
    "DrawImageAbort": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "DriverStringOptions": SimTypeInt(signed=False, label="DriverStringOptions"),
    "DtdProcessing": SimTypeInt(signed=False, label="DtdProcessing"),
    "DualMonoInfo": SimStruct(OrderedDict((("LangID1", SimTypeShort(signed=False, label="UInt16")), ("LangID2", SimTypeShort(signed=False, label="UInt16")), ("lISOLangCode1", SimTypeInt(signed=True, label="Int32")), ("lISOLangCode2", SimTypeInt(signed=True, label="Int32")),)), name="DualMonoInfo", pack=False, align=None),
    "DvbParentalRatingDescriptor": SimStruct(OrderedDict((("ulNumParams", SimTypeInt(signed=False, label="UInt32")), ("pParams", SimTypePointer(SimTypeRef("DvbParentalRatingParam", SimStruct), offset=0)),)), name="DvbParentalRatingDescriptor", pack=False, align=None),
    "DvbParentalRatingParam": SimStruct(OrderedDict((("szCountryCode", SimTypeArray(SimTypeChar(label="SByte"), 4)), ("bRating", SimTypeChar(label="Byte")),)), name="DvbParentalRatingParam", pack=False, align=None),
    "DxcArgPair": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DxcArgPair", pack=False, align=None),
    "DxcBuffer": SimStruct(OrderedDict((("Ptr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Encoding", SimTypeInt(signed=False, label="UInt32")),)), name="DxcBuffer", pack=False, align=None),
    "DxcCreateInstance2Proc": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IMalloc"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMalloc", "rclsid", "riid", "ppv"]), offset=0),
    "DxcCreateInstanceProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["rclsid", "riid", "ppv"]), offset=0),
    "DxcDefine": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="DxcDefine", pack=False, align=None),
    "DxcShaderHash": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("HashDigest", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="DxcShaderHash", pack=False, align=None),
    "EALocationCodeType": SimStruct(OrderedDict((("LocationCodeScheme", SimTypeInt(signed=False, label="LocationCodeSchemeType")), ("state_code", SimTypeChar(label="Byte")), ("county_subdivision", SimTypeChar(label="Byte")), ("county_code", SimTypeShort(signed=False, label="UInt16")),)), name="EALocationCodeType", pack=False, align=None),
    "EAPHOST_AUTH_INFO": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="EAPHOST_AUTH_STATUS")), ("dwErrorCode", SimTypeInt(signed=False, label="UInt32")), ("dwReasonCode", SimTypeInt(signed=False, label="UInt32")),)), name="EAPHOST_AUTH_INFO", pack=False, align=None),
    "EAPHOST_AUTH_STATUS": SimTypeInt(signed=False, label="EAPHOST_AUTH_STATUS"),
    "EAPHOST_IDENTITY_UI_PARAMS": SimStruct(OrderedDict((("eapMethodType", SimTypeRef("EAP_METHOD_TYPE", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSizeofConnectionData", SimTypeInt(signed=False, label="UInt32")), ("pConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeofUserData", SimTypeInt(signed=False, label="UInt32")), ("pUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeofUserDataOut", SimTypeInt(signed=False, label="UInt32")), ("pUserDataOut", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("pEapError", SimTypePointer(SimTypeRef("EAP_ERROR", SimStruct), offset=0)),)), name="EAPHOST_IDENTITY_UI_PARAMS", pack=False, align=None),
    "EAPHOST_INTERACTIVE_UI_PARAMS": SimStruct(OrderedDict((("dwSizeofContextData", SimTypeInt(signed=False, label="UInt32")), ("pContextData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeofInteractiveUIData", SimTypeInt(signed=False, label="UInt32")), ("pInteractiveUIData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("pEapError", SimTypePointer(SimTypeRef("EAP_ERROR", SimStruct), offset=0)),)), name="EAPHOST_INTERACTIVE_UI_PARAMS", pack=False, align=None),
    "EAP_ATTRIBUTE": SimStruct(OrderedDict((("eaType", SimTypeInt(signed=False, label="EAP_ATTRIBUTE_TYPE")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("pValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EAP_ATTRIBUTE", pack=False, align=None),
    "EAP_ATTRIBUTES": SimStruct(OrderedDict((("dwNumberOfAttributes", SimTypeInt(signed=False, label="UInt32")), ("pAttribs", SimTypePointer(SimTypeRef("EAP_ATTRIBUTE", SimStruct), offset=0)),)), name="EAP_ATTRIBUTES", pack=False, align=None),
    "EAP_ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="EAP_ATTRIBUTE_TYPE"),
    "EAP_AUTHENTICATOR_METHOD_ROUTINES": SimStruct(OrderedDict((("dwSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("pEapType", SimTypePointer(SimTypeRef("EAP_METHOD_TYPE", SimStruct), offset=0)), ("EapMethodAuthenticatorInitialize", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorBeginSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorUpdateInnerMethodParams", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorReceivePacket", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorSendPacket", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorGetAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorSetAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorGetResult", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorEndSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapMethodAuthenticatorShutdown", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="EAP_AUTHENTICATOR_METHOD_ROUTINES", pack=False, align=None),
    "EAP_AUTHENTICATOR_SEND_TIMEOUT": SimTypeInt(signed=False, label="EAP_AUTHENTICATOR_SEND_TIMEOUT"),
    "EAP_CONFIG_INPUT_FIELD_ARRAY": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfFields", SimTypeInt(signed=False, label="UInt32")), ("pFields", SimTypePointer(SimTypeRef("EAP_CONFIG_INPUT_FIELD_DATA", SimStruct), offset=0)),)), name="EAP_CONFIG_INPUT_FIELD_ARRAY", pack=False, align=None),
    "EAP_CONFIG_INPUT_FIELD_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="EAP_CONFIG_INPUT_FIELD_TYPE")), ("dwFlagProps", SimTypeInt(signed=False, label="UInt32")), ("pwszLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszData", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMinDataLength", SimTypeInt(signed=False, label="UInt32")), ("dwMaxDataLength", SimTypeInt(signed=False, label="UInt32")),)), name="EAP_CONFIG_INPUT_FIELD_DATA", pack=False, align=None),
    "EAP_CONFIG_INPUT_FIELD_TYPE": SimTypeInt(signed=False, label="EAP_CONFIG_INPUT_FIELD_TYPE"),
    "EAP_CRED_EXPIRY_REQ": SimStruct(OrderedDict((("curCreds", SimTypeRef("EAP_CONFIG_INPUT_FIELD_ARRAY", SimStruct)), ("newCreds", SimTypeRef("EAP_CONFIG_INPUT_FIELD_ARRAY", SimStruct)),)), name="EAP_CRED_EXPIRY_REQ", pack=False, align=None),
    "EAP_ERROR": SimStruct(OrderedDict((("dwWinError", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeRef("EAP_METHOD_TYPE", SimStruct)), ("dwReasonCode", SimTypeInt(signed=False, label="UInt32")), ("rootCauseGuid", SimTypeBottom(label="Guid")), ("repairGuid", SimTypeBottom(label="Guid")), ("helpLinkGuid", SimTypeBottom(label="Guid")), ("pRootCauseString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pRepairString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="EAP_ERROR", pack=False, align=None),
    "EAP_INTERACTIVE_UI_DATA": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwDataType", SimTypeInt(signed=False, label="EAP_INTERACTIVE_UI_DATA_TYPE")), ("cbUiData", SimTypeInt(signed=False, label="UInt32")), ("pbUiData", SimUnion({"credData": SimTypePointer(SimTypeRef("EAP_CONFIG_INPUT_FIELD_ARRAY", SimStruct), offset=0), "credExpiryData": SimTypePointer(SimTypeRef("EAP_CRED_EXPIRY_REQ", SimStruct), offset=0), "credLogonData": SimTypePointer(SimTypeRef("EAP_CONFIG_INPUT_FIELD_ARRAY", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="EAP_INTERACTIVE_UI_DATA", pack=False, align=None),
    "EAP_INTERACTIVE_UI_DATA_TYPE": SimTypeInt(signed=False, label="EAP_INTERACTIVE_UI_DATA_TYPE"),
    "EAP_METHOD_AUTHENTICATOR_RESPONSE_ACTION": SimTypeInt(signed=False, label="EAP_METHOD_AUTHENTICATOR_RESPONSE_ACTION"),
    "EAP_METHOD_AUTHENTICATOR_RESULT": SimStruct(OrderedDict((("fIsSuccess", SimTypeInt(signed=True, label="Int32")), ("dwFailureReason", SimTypeInt(signed=False, label="UInt32")), ("pAuthAttribs", SimTypePointer(SimTypeRef("EAP_ATTRIBUTES", SimStruct), offset=0)),)), name="EAP_METHOD_AUTHENTICATOR_RESULT", pack=False, align=None),
    "EAP_METHOD_INFO": SimStruct(OrderedDict((("eaptype", SimTypeRef("EAP_METHOD_TYPE", SimStruct)), ("pwszAuthorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("eapProperties", SimTypeInt(signed=False, label="UInt32")), ("pInnerMethodInfo", SimTypePointer(SimTypeRef("EAP_METHOD_INFO", SimStruct), offset=0)),)), name="EAP_METHOD_INFO", pack=False, align=None),
    "EAP_METHOD_INFO_ARRAY": SimStruct(OrderedDict((("dwNumberOfMethods", SimTypeInt(signed=False, label="UInt32")), ("pEapMethods", SimTypePointer(SimTypeRef("EAP_METHOD_INFO", SimStruct), offset=0)),)), name="EAP_METHOD_INFO_ARRAY", pack=False, align=None),
    "EAP_METHOD_INFO_ARRAY_EX": SimStruct(OrderedDict((("dwNumberOfMethods", SimTypeInt(signed=False, label="UInt32")), ("pEapMethods", SimTypePointer(SimTypeRef("EAP_METHOD_INFO_EX", SimStruct), offset=0)),)), name="EAP_METHOD_INFO_ARRAY_EX", pack=False, align=None),
    "EAP_METHOD_INFO_EX": SimStruct(OrderedDict((("eaptype", SimTypeRef("EAP_METHOD_TYPE", SimStruct)), ("pwszAuthorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("eapProperties", SimTypeInt(signed=False, label="UInt32")), ("pInnerMethodInfoArray", SimTypePointer(SimTypeRef("EAP_METHOD_INFO_ARRAY_EX", SimStruct), offset=0)),)), name="EAP_METHOD_INFO_EX", pack=False, align=None),
    "EAP_METHOD_PROPERTY": SimStruct(OrderedDict((("eapMethodPropertyType", SimTypeInt(signed=False, label="EAP_METHOD_PROPERTY_TYPE")), ("eapMethodPropertyValueType", SimTypeInt(signed=False, label="EAP_METHOD_PROPERTY_VALUE_TYPE")), ("eapMethodPropertyValue", SimUnion({"empvBool": SimTypeRef("EAP_METHOD_PROPERTY_VALUE_BOOL", SimStruct), "empvDword": SimTypeRef("EAP_METHOD_PROPERTY_VALUE_DWORD", SimStruct), "empvString": SimTypeRef("EAP_METHOD_PROPERTY_VALUE_STRING", SimStruct)}, name="<anon>", label="None")),)), name="EAP_METHOD_PROPERTY", pack=False, align=None),
    "EAP_METHOD_PROPERTY_ARRAY": SimStruct(OrderedDict((("dwNumberOfProperties", SimTypeInt(signed=False, label="UInt32")), ("pMethodProperty", SimTypePointer(SimTypeRef("EAP_METHOD_PROPERTY", SimStruct), offset=0)),)), name="EAP_METHOD_PROPERTY_ARRAY", pack=False, align=None),
    "EAP_METHOD_PROPERTY_TYPE": SimTypeInt(signed=False, label="EAP_METHOD_PROPERTY_TYPE"),
    "EAP_METHOD_PROPERTY_VALUE": SimUnion({"empvBool": SimTypeRef("EAP_METHOD_PROPERTY_VALUE_BOOL", SimStruct), "empvDword": SimTypeRef("EAP_METHOD_PROPERTY_VALUE_DWORD", SimStruct), "empvString": SimTypeRef("EAP_METHOD_PROPERTY_VALUE_STRING", SimStruct)}, name="<anon>", label="None"),
    "EAP_METHOD_PROPERTY_VALUE_BOOL": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("value", SimTypeInt(signed=True, label="Int32")),)), name="EAP_METHOD_PROPERTY_VALUE_BOOL", pack=False, align=None),
    "EAP_METHOD_PROPERTY_VALUE_DWORD": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("value", SimTypeInt(signed=False, label="UInt32")),)), name="EAP_METHOD_PROPERTY_VALUE_DWORD", pack=False, align=None),
    "EAP_METHOD_PROPERTY_VALUE_STRING": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EAP_METHOD_PROPERTY_VALUE_STRING", pack=False, align=None),
    "EAP_METHOD_PROPERTY_VALUE_TYPE": SimTypeInt(signed=False, label="EAP_METHOD_PROPERTY_VALUE_TYPE"),
    "EAP_METHOD_TYPE": SimStruct(OrderedDict((("eapType", SimTypeRef("EAP_TYPE", SimStruct)), ("dwAuthorId", SimTypeInt(signed=False, label="UInt32")),)), name="EAP_METHOD_TYPE", pack=False, align=None),
    "EAP_PEER_METHOD_ROUTINES": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pEapType", SimTypePointer(SimTypeRef("EAP_TYPE", SimStruct), offset=0)), ("EapPeerInitialize", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerGetIdentity", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerBeginSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerSetCredentials", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerProcessRequestPacket", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerGetResponsePacket", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerGetResult", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerGetUIContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerSetUIContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerGetResponseAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerSetResponseAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerEndSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EapPeerShutdown", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="EAP_PEER_METHOD_ROUTINES", pack=False, align=None),
    "EAP_TYPE": SimStruct(OrderedDict((("type", SimTypeChar(label="Byte")), ("dwVendorId", SimTypeInt(signed=False, label="UInt32")), ("dwVendorType", SimTypeInt(signed=False, label="UInt32")),)), name="EAP_TYPE", pack=False, align=None),
    "EAP_UI_DATA_FORMAT": SimUnion({"credData": SimTypePointer(SimTypeRef("EAP_CONFIG_INPUT_FIELD_ARRAY", SimStruct), offset=0), "credExpiryData": SimTypePointer(SimTypeRef("EAP_CRED_EXPIRY_REQ", SimStruct), offset=0), "credLogonData": SimTypePointer(SimTypeRef("EAP_CONFIG_INPUT_FIELD_ARRAY", SimStruct), offset=0)}, name="<anon>", label="None"),
    "EATTRIBUTE_DATATYPE": SimTypeInt(signed=False, label="EATTRIBUTE_DATATYPE"),
    "EAllocationType": SimTypeInt(signed=False, label="EAllocationType"),
    "EApiCategories": SimTypeInt(signed=False, label="EApiCategories"),
    "EAudioConstriction": SimTypeInt(signed=False, label="EAudioConstriction"),
    "EBindInfoOptions": SimTypeInt(signed=False, label="EBindInfoOptions"),
    "EBindPolicyLevels": SimTypeInt(signed=False, label="EBindPolicyLevels"),
    "EBranchOfficeJobEventType": SimTypeInt(signed=False, label="EBranchOfficeJobEventType"),
    "ECC_CURVE_ALG_ID_ENUM": SimTypeInt(signed=False, label="ECC_CURVE_ALG_ID_ENUM"),
    "ECC_CURVE_TYPE_ENUM": SimTypeInt(signed=False, label="ECC_CURVE_TYPE_ENUM"),
    "ECHOSC1WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="ECHOSC1WAVEFORMAT", pack=False, align=None),
    "ECHOWAVEFILTER": SimStruct(OrderedDict((("wfltr", SimTypeRef("WAVEFILTER", SimStruct)), ("dwVolume", SimTypeInt(signed=False, label="UInt32")), ("dwDelay", SimTypeInt(signed=False, label="UInt32")),)), name="ECHOWAVEFILTER", pack=False, align=None),
    "ECLRAssemblyIdentityFlags": SimTypeInt(signed=False, label="ECLRAssemblyIdentityFlags"),
    "ECP_HEADER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ECP_LIST": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ECP_OPEN_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="ECP_OPEN_PARAMETERS", pack=False, align=None),
    "EC_ENDOFLINE": SimTypeInt(signed=False, label="EC_ENDOFLINE"),
    "EC_HOST_UI_MODE": SimTypeInt(signed=False, label="EC_HOST_UI_MODE"),
    "EC_SEARCHWEB_ENTRYPOINT": SimTypeInt(signed=False, label="EC_SEARCHWEB_ENTRYPOINT"),
    "EC_SUBSCRIPTION_CONFIGURATION_MODE": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_CONFIGURATION_MODE"),
    "EC_SUBSCRIPTION_CONTENT_FORMAT": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_CONTENT_FORMAT"),
    "EC_SUBSCRIPTION_CREDENTIALS_TYPE": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_CREDENTIALS_TYPE"),
    "EC_SUBSCRIPTION_DELIVERY_MODE": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_DELIVERY_MODE"),
    "EC_SUBSCRIPTION_PROPERTY_ID": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_PROPERTY_ID"),
    "EC_SUBSCRIPTION_RUNTIME_STATUS_ACTIVE_STATUS": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_RUNTIME_STATUS_ACTIVE_STATUS"),
    "EC_SUBSCRIPTION_RUNTIME_STATUS_INFO_ID": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_RUNTIME_STATUS_INFO_ID"),
    "EC_SUBSCRIPTION_TYPE": SimTypeInt(signed=False, label="EC_SUBSCRIPTION_TYPE"),
    "EC_VARIANT": SimStruct(OrderedDict((("Anonymous", SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "DateTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "PropertyHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="EC_VARIANT", pack=False, align=None),
    "EC_VARIANT_TYPE": SimTypeInt(signed=False, label="EC_VARIANT_TYPE"),
    "EClrEvent": SimTypeInt(signed=False, label="EClrEvent"),
    "EClrFailure": SimTypeInt(signed=False, label="EClrFailure"),
    "EClrOperation": SimTypeInt(signed=False, label="EClrOperation"),
    "EClrUnhandledException": SimTypeInt(signed=False, label="EClrUnhandledException"),
    "EContextType": SimTypeInt(signed=False, label="EContextType"),
    "ECustomDumpFlavor": SimTypeInt(signed=False, label="ECustomDumpFlavor"),
    "ECustomDumpItemKind": SimTypeInt(signed=False, label="ECustomDumpItemKind"),
    "EDGE_GESTURE_KIND": SimTypeInt(signed=False, label="EDGE_GESTURE_KIND"),
    "EDITBALLOONTIP": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ttiIcon", SimTypeInt(signed=False, label="EDITBALLOONTIP_ICON")),)), name="EDITBALLOONTIP", pack=False, align=None),
    "EDITBALLOONTIP_ICON": SimTypeInt(signed=False, label="EDITBALLOONTIP_ICON"),
    "EDITBORDER_HSCROLLSTATES": SimTypeInt(signed=False, label="EDITBORDER_HSCROLLSTATES"),
    "EDITBORDER_HVSCROLLSTATES": SimTypeInt(signed=False, label="EDITBORDER_HVSCROLLSTATES"),
    "EDITBORDER_NOSCROLLSTATES": SimTypeInt(signed=False, label="EDITBORDER_NOSCROLLSTATES"),
    "EDITBORDER_VSCROLLSTATES": SimTypeInt(signed=False, label="EDITBORDER_VSCROLLSTATES"),
    "EDITPARTS": SimTypeInt(signed=False, label="EDITPARTS"),
    "EDITSTREAM": SimStruct(OrderedDict((("dwCookie", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwCookie", "pbBuff", "cb", "pcb"]), offset=0)),)), name="EDITSTREAM", pack=False, align=None),
    "EDITSTREAMCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwCookie", "pbBuff", "cb", "pcb"]), offset=0),
    "EDITTEXTSTATES": SimTypeInt(signed=False, label="EDITTEXTSTATES"),
    "EDITWORDBREAKPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="WORD_BREAK_ACTION")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpch", "ichCurrent", "cch", "code"]), offset=0),
    "EDITWORDBREAKPROCEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeChar(label="Byte"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pchText", "cchText", "bCharSet", "action"]), offset=0),
    "EDITWORDBREAKPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="WORD_BREAK_ACTION")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpch", "ichCurrent", "cch", "code"]), offset=0),
    "EDIT_CONTROL_FEATURE": SimTypeInt(signed=False, label="EDIT_CONTROL_FEATURE"),
    "EDIT_LINKS_FLAGS": SimTypeInt(signed=False, label="EDIT_LINKS_FLAGS"),
    "EDataFlow": SimTypeInt(signed=False, label="EDataFlow"),
    "EDefaultDevmodeType": SimTypeInt(signed=False, label="EDefaultDevmodeType"),
    "EDeviceControlUseType": SimTypeInt(signed=False, label="EDeviceControlUseType"),
    "EFFECTIVE_POWER_MODE": SimTypeInt(signed=False, label="EFFECTIVE_POWER_MODE"),
    "EFFECTIVE_POWER_MODE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="EFFECTIVE_POWER_MODE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Mode", "Context"]), offset=0),
    "EFFPERM_RESULT_LIST": SimStruct(OrderedDict((("fEvaluated", SimTypeChar(label="Byte")), ("cObjectTypeListLength", SimTypeInt(signed=False, label="UInt32")), ("pObjectTypeList", SimTypePointer(SimTypeRef("OBJECT_TYPE_LIST", SimStruct), offset=0)), ("pGrantedAccessList", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="EFFPERM_RESULT_LIST", pack=False, align=None),
    "EFI_ACPI_RAS_SIGNAL_TABLE": SimStruct(OrderedDict((("Header", SimTypeRef("WHEA_ACPI_HEADER", SimStruct)), ("NumberRecord", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeRef("SIGNAL_REG_VALUE", SimStruct), offset=0)),)), name="EFI_ACPI_RAS_SIGNAL_TABLE", pack=False, align=None),
    "EFS_CERTIFICATE_BLOB": SimStruct(OrderedDict((("dwCertEncodingType", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EFS_CERTIFICATE_BLOB", pack=False, align=None),
    "EFS_COMPATIBILITY_INFO": SimStruct(OrderedDict((("EfsVersion", SimTypeInt(signed=False, label="UInt32")),)), name="EFS_COMPATIBILITY_INFO", pack=False, align=None),
    "EFS_DECRYPTION_STATUS_INFO": SimStruct(OrderedDict((("dwDecryptionError", SimTypeInt(signed=False, label="UInt32")), ("dwHashOffset", SimTypeInt(signed=False, label="UInt32")), ("cbHash", SimTypeInt(signed=False, label="UInt32")),)), name="EFS_DECRYPTION_STATUS_INFO", pack=False, align=None),
    "EFS_ENCRYPTION_STATUS_INFO": SimStruct(OrderedDict((("bHasCurrentKey", SimTypeInt(signed=True, label="Int32")), ("dwEncryptionError", SimTypeInt(signed=False, label="UInt32")),)), name="EFS_ENCRYPTION_STATUS_INFO", pack=False, align=None),
    "EFS_HASH_BLOB": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EFS_HASH_BLOB", pack=False, align=None),
    "EFS_KEY_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Entropy", SimTypeInt(signed=False, label="UInt32")), ("Algorithm", SimTypeInt(signed=False, label="ALG_ID")), ("KeyLength", SimTypeInt(signed=False, label="UInt32")),)), name="EFS_KEY_INFO", pack=False, align=None),
    "EFS_PIN_BLOB": SimStruct(OrderedDict((("cbPadding", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EFS_PIN_BLOB", pack=False, align=None),
    "EFS_RPC_BLOB": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EFS_RPC_BLOB", pack=False, align=None),
    "EFS_VERSION_INFO": SimStruct(OrderedDict((("EfsVersion", SimTypeInt(signed=False, label="UInt32")), ("SubVersion", SimTypeInt(signed=False, label="UInt32")),)), name="EFS_VERSION_INFO", pack=False, align=None),
    "EFaultRepRetVal": SimTypeInt(signed=False, label="EFaultRepRetVal"),
    "EHostApplicationPolicy": SimTypeInt(signed=False, label="EHostApplicationPolicy"),
    "EHostBindingPolicyModifyFlags": SimTypeInt(signed=False, label="EHostBindingPolicyModifyFlags"),
    "EISA_DMA_CONFIGURATION": SimStruct(OrderedDict((("ConfigurationByte0", SimTypeRef("DMA_CONFIGURATION_BYTE0", SimStruct)), ("ConfigurationByte1", SimTypeRef("DMA_CONFIGURATION_BYTE1", SimStruct)),)), name="EISA_DMA_CONFIGURATION", pack=False, align=None),
    "EISA_IRQ_CONFIGURATION": SimStruct(OrderedDict((("ConfigurationByte", SimTypeRef("EISA_IRQ_DESCRIPTOR", SimStruct)), ("Reserved", SimTypeChar(label="Byte")),)), name="EISA_IRQ_CONFIGURATION", pack=False, align=None),
    "EISA_IRQ_DESCRIPTOR": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="EISA_IRQ_DESCRIPTOR", pack=False, align=None),
    "EISA_MEMORY_CONFIGURATION": SimStruct(OrderedDict((("ConfigurationByte", SimTypeRef("EISA_MEMORY_TYPE", SimStruct)), ("DataSize", SimTypeChar(label="Byte")), ("AddressLowWord", SimTypeShort(signed=False, label="UInt16")), ("AddressHighByte", SimTypeChar(label="Byte")), ("MemorySize", SimTypeShort(signed=False, label="UInt16")),)), name="EISA_MEMORY_CONFIGURATION", pack=False, align=None),
    "EISA_MEMORY_TYPE": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="EISA_MEMORY_TYPE", pack=False, align=None),
    "EISA_PORT_CONFIGURATION": SimStruct(OrderedDict((("Configuration", SimTypeRef("EISA_PORT_DESCRIPTOR", SimStruct)), ("PortAddress", SimTypeShort(signed=False, label="UInt16")),)), name="EISA_PORT_CONFIGURATION", pack=False, align=None),
    "EISA_PORT_DESCRIPTOR": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="EISA_PORT_DESCRIPTOR", pack=False, align=None),
    "EInitializeNewDomainFlags": SimTypeInt(signed=False, label="EInitializeNewDomainFlags"),
    "ELEMDESC": SimStruct(OrderedDict((("tdesc", SimTypeRef("TYPEDESC", SimStruct)), ("Anonymous", SimUnion({"idldesc": SimTypeRef("IDLDESC", SimStruct), "paramdesc": SimTypeRef("PARAMDESC", SimStruct)}, name="<anon>", label="None")),)), name="ELEMDESC", pack=False, align=None),
    "ELEMENTDESCRIPTOR_FLAGS": SimTypeInt(signed=False, label="ELEMENTDESCRIPTOR_FLAGS"),
    "ELEMENTNAMESPACE_FLAGS": SimTypeInt(signed=False, label="ELEMENTNAMESPACE_FLAGS"),
    "ELEMENT_ADJACENCY": SimTypeInt(signed=False, label="ELEMENT_ADJACENCY"),
    "ELEMENT_CORNER": SimTypeInt(signed=False, label="ELEMENT_CORNER"),
    "ELEMENT_TAG_ID": SimTypeInt(signed=False, label="ELEMENT_TAG_ID"),
    "ELEMENT_TYPE": SimTypeInt(signed=False, label="ELEMENT_TYPE"),
    "ELEVATION_CHANGE_MODE": SimTypeInt(signed=False, label="ELEVATION_CHANGE_MODE"),
    "EMBDHLP_FLAGS": SimTypeInt(signed=False, label="EMBDHLP_FLAGS"),
    "EMBEDDED_FONT_PRIV_STATUS": SimTypeInt(signed=False, label="EMBEDDED_FONT_PRIV_STATUS"),
    "EMBED_FONT_CHARSET": SimTypeInt(signed=False, label="EMBED_FONT_CHARSET"),
    "EMFINFO": SimStruct(OrderedDict((("nSize", SimTypeInt(signed=False, label="UInt32")), ("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pvEMF", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvCurrentRecord", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMFINFO", pack=False, align=None),
    "EMFPLAYPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "EMI_CHANNEL_MEASUREMENT_DATA": SimStruct(OrderedDict((("AbsoluteEnergy", SimTypeLongLong(signed=False, label="UInt64")), ("AbsoluteTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="EMI_CHANNEL_MEASUREMENT_DATA", pack=False, align=None),
    "EMI_CHANNEL_V2": SimStruct(OrderedDict((("MeasurementUnit", SimTypeInt(signed=False, label="EMI_MEASUREMENT_UNIT")), ("ChannelNameSize", SimTypeShort(signed=False, label="UInt16")), ("ChannelName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="EMI_CHANNEL_V2", pack=False, align=None),
    "EMI_MEASUREMENT_DATA_V2": SimStruct(OrderedDict((("ChannelData", SimTypePointer(SimTypeRef("EMI_CHANNEL_MEASUREMENT_DATA", SimStruct), offset=0)),)), name="EMI_MEASUREMENT_DATA_V2", pack=False, align=None),
    "EMI_MEASUREMENT_UNIT": SimTypeInt(signed=False, label="EMI_MEASUREMENT_UNIT"),
    "EMI_METADATA_SIZE": SimStruct(OrderedDict((("MetadataSize", SimTypeInt(signed=False, label="UInt32")),)), name="EMI_METADATA_SIZE", pack=False, align=None),
    "EMI_METADATA_V1": SimStruct(OrderedDict((("MeasurementUnit", SimTypeInt(signed=False, label="EMI_MEASUREMENT_UNIT")), ("HardwareOEM", SimTypeArray(SimTypeChar(label="Char"), 16)), ("HardwareModel", SimTypeArray(SimTypeChar(label="Char"), 16)), ("HardwareRevision", SimTypeShort(signed=False, label="UInt16")), ("MeteredHardwareNameSize", SimTypeShort(signed=False, label="UInt16")), ("MeteredHardwareName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="EMI_METADATA_V1", pack=False, align=None),
    "EMI_METADATA_V2": SimStruct(OrderedDict((("HardwareOEM", SimTypeArray(SimTypeChar(label="Char"), 16)), ("HardwareModel", SimTypeArray(SimTypeChar(label="Char"), 16)), ("HardwareRevision", SimTypeShort(signed=False, label="UInt16")), ("ChannelCount", SimTypeShort(signed=False, label="UInt16")), ("Channels", SimTypePointer(SimTypeRef("EMI_CHANNEL_V2", SimStruct), offset=0)),)), name="EMI_METADATA_V2", pack=False, align=None),
    "EMI_VERSION": SimStruct(OrderedDict((("EmiVersion", SimTypeShort(signed=False, label="UInt16")),)), name="EMI_VERSION", pack=False, align=None),
    "EMPTYMARKUPPARTS": SimTypeInt(signed=False, label="EMPTYMARKUPPARTS"),
    "EMPTY_VOLUME_CACHE_FLAGS": SimTypeInt(signed=False, label="EMPTY_VOLUME_CACHE_FLAGS"),
    "EMR": SimStruct(OrderedDict((("iType", SimTypeInt(signed=False, label="ENHANCED_METAFILE_RECORD_TYPE")), ("nSize", SimTypeInt(signed=False, label="UInt32")),)), name="EMR", pack=False, align=None),
    "EMRALPHABLEND": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("cxDest", SimTypeInt(signed=True, label="Int32")), ("cyDest", SimTypeInt(signed=True, label="Int32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("xformSrc", SimTypeRef("XFORM", SimStruct)), ("crBkColorSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cxSrc", SimTypeInt(signed=True, label="Int32")), ("cySrc", SimTypeInt(signed=True, label="Int32")),)), name="EMRALPHABLEND", pack=False, align=None),
    "EMRANGLEARC": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ptlCenter", SimTypeRef("POINTL", SimStruct)), ("nRadius", SimTypeInt(signed=False, label="UInt32")), ("eStartAngle", SimTypeFloat(size=32)), ("eSweepAngle", SimTypeFloat(size=32)),)), name="EMRANGLEARC", pack=False, align=None),
    "EMRARC": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBox", SimTypeRef("RECTL", SimStruct)), ("ptlStart", SimTypeRef("POINTL", SimStruct)), ("ptlEnd", SimTypeRef("POINTL", SimStruct)),)), name="EMRARC", pack=False, align=None),
    "EMRBITBLT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("cxDest", SimTypeInt(signed=True, label="Int32")), ("cyDest", SimTypeInt(signed=True, label="Int32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("xformSrc", SimTypeRef("XFORM", SimStruct)), ("crBkColorSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")),)), name="EMRBITBLT", pack=False, align=None),
    "EMRCOLORCORRECTPALETTE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPalette", SimTypeInt(signed=False, label="UInt32")), ("nFirstEntry", SimTypeInt(signed=False, label="UInt32")), ("nPalEntries", SimTypeInt(signed=False, label="UInt32")), ("nReserved", SimTypeInt(signed=False, label="UInt32")),)), name="EMRCOLORCORRECTPALETTE", pack=False, align=None),
    "EMRCOLORMATCHTOTARGET": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("dwAction", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbName", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRCOLORMATCHTOTARGET", pack=False, align=None),
    "EMRCREATEBRUSHINDIRECT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihBrush", SimTypeInt(signed=False, label="UInt32")), ("lb", SimTypeRef("LOGBRUSH32", SimStruct)),)), name="EMRCREATEBRUSHINDIRECT", pack=False, align=None),
    "EMRCREATECOLORSPACE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihCS", SimTypeInt(signed=False, label="UInt32")), ("lcs", SimTypeRef("LOGCOLORSPACEA", SimStruct)),)), name="EMRCREATECOLORSPACE", pack=False, align=None),
    "EMRCREATECOLORSPACEW": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihCS", SimTypeInt(signed=False, label="UInt32")), ("lcs", SimTypeRef("LOGCOLORSPACEW", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRCREATECOLORSPACEW", pack=False, align=None),
    "EMRCREATEDIBPATTERNBRUSHPT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihBrush", SimTypeInt(signed=False, label="UInt32")), ("iUsage", SimTypeInt(signed=False, label="UInt32")), ("offBmi", SimTypeInt(signed=False, label="UInt32")), ("cbBmi", SimTypeInt(signed=False, label="UInt32")), ("offBits", SimTypeInt(signed=False, label="UInt32")), ("cbBits", SimTypeInt(signed=False, label="UInt32")),)), name="EMRCREATEDIBPATTERNBRUSHPT", pack=False, align=None),
    "EMRCREATEMONOBRUSH": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihBrush", SimTypeInt(signed=False, label="UInt32")), ("iUsage", SimTypeInt(signed=False, label="UInt32")), ("offBmi", SimTypeInt(signed=False, label="UInt32")), ("cbBmi", SimTypeInt(signed=False, label="UInt32")), ("offBits", SimTypeInt(signed=False, label="UInt32")), ("cbBits", SimTypeInt(signed=False, label="UInt32")),)), name="EMRCREATEMONOBRUSH", pack=False, align=None),
    "EMRCREATEPALETTE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPal", SimTypeInt(signed=False, label="UInt32")), ("lgpl", SimTypeRef("LOGPALETTE", SimStruct)),)), name="EMRCREATEPALETTE", pack=False, align=None),
    "EMRCREATEPEN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPen", SimTypeInt(signed=False, label="UInt32")), ("lopn", SimTypeRef("LOGPEN", SimStruct)),)), name="EMRCREATEPEN", pack=False, align=None),
    "EMRELLIPSE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBox", SimTypeRef("RECTL", SimStruct)),)), name="EMRELLIPSE", pack=False, align=None),
    "EMREOF": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("nPalEntries", SimTypeInt(signed=False, label="UInt32")), ("offPalEntries", SimTypeInt(signed=False, label="UInt32")), ("nSizeLast", SimTypeInt(signed=False, label="UInt32")),)), name="EMREOF", pack=False, align=None),
    "EMREXCLUDECLIPRECT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclClip", SimTypeRef("RECTL", SimStruct)),)), name="EMREXCLUDECLIPRECT", pack=False, align=None),
    "EMREXTCREATEFONTINDIRECTW": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihFont", SimTypeInt(signed=False, label="UInt32")), ("elfw", SimTypeRef("EXTLOGFONTW", SimStruct)),)), name="EMREXTCREATEFONTINDIRECTW", pack=False, align=None),
    "EMREXTCREATEPEN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPen", SimTypeInt(signed=False, label="UInt32")), ("offBmi", SimTypeInt(signed=False, label="UInt32")), ("cbBmi", SimTypeInt(signed=False, label="UInt32")), ("offBits", SimTypeInt(signed=False, label="UInt32")), ("cbBits", SimTypeInt(signed=False, label="UInt32")), ("elp", SimTypeRef("EXTLOGPEN32", SimStruct)),)), name="EMREXTCREATEPEN", pack=False, align=None),
    "EMREXTESCAPE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("iEscape", SimTypeInt(signed=True, label="Int32")), ("cbEscData", SimTypeInt(signed=True, label="Int32")), ("EscData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMREXTESCAPE", pack=False, align=None),
    "EMREXTFLOODFILL": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ptlStart", SimTypeRef("POINTL", SimStruct)), ("crColor", SimTypeInt(signed=False, label="UInt32")), ("iMode", SimTypeInt(signed=False, label="UInt32")),)), name="EMREXTFLOODFILL", pack=False, align=None),
    "EMREXTSELECTCLIPRGN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("cbRgnData", SimTypeInt(signed=False, label="UInt32")), ("iMode", SimTypeInt(signed=False, label="UInt32")), ("RgnData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMREXTSELECTCLIPRGN", pack=False, align=None),
    "EMREXTTEXTOUTA": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("iGraphicsMode", SimTypeInt(signed=False, label="UInt32")), ("exScale", SimTypeFloat(size=32)), ("eyScale", SimTypeFloat(size=32)), ("emrtext", SimTypeRef("EMRTEXT", SimStruct)),)), name="EMREXTTEXTOUTA", pack=False, align=None),
    "EMRFILLPATH": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)),)), name="EMRFILLPATH", pack=False, align=None),
    "EMRFILLRGN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cbRgnData", SimTypeInt(signed=False, label="UInt32")), ("ihBrush", SimTypeInt(signed=False, label="UInt32")), ("RgnData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRFILLRGN", pack=False, align=None),
    "EMRFORMAT": SimStruct(OrderedDict((("dSignature", SimTypeInt(signed=False, label="UInt32")), ("nVersion", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("offData", SimTypeInt(signed=False, label="UInt32")),)), name="EMRFORMAT", pack=False, align=None),
    "EMRFRAMERGN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cbRgnData", SimTypeInt(signed=False, label="UInt32")), ("ihBrush", SimTypeInt(signed=False, label="UInt32")), ("szlStroke", SimTypeRef("SIZE", SimStruct)), ("RgnData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRFRAMERGN", pack=False, align=None),
    "EMRGDICOMMENT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRGDICOMMENT", pack=False, align=None),
    "EMRGLSBOUNDEDRECORD": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRGLSBOUNDEDRECORD", pack=False, align=None),
    "EMRGLSRECORD": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRGLSRECORD", pack=False, align=None),
    "EMRGRADIENTFILL": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("nVer", SimTypeInt(signed=False, label="UInt32")), ("nTri", SimTypeInt(signed=False, label="UInt32")), ("ulMode", SimTypeInt(signed=False, label="GRADIENT_FILL")), ("Ver", SimTypePointer(SimTypeRef("TRIVERTEX", SimStruct), offset=0)),)), name="EMRGRADIENTFILL", pack=False, align=None),
    "EMRINVERTRGN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cbRgnData", SimTypeInt(signed=False, label="UInt32")), ("RgnData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRINVERTRGN", pack=False, align=None),
    "EMRLINETO": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ptl", SimTypeRef("POINTL", SimStruct)),)), name="EMRLINETO", pack=False, align=None),
    "EMRMASKBLT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("cxDest", SimTypeInt(signed=True, label="Int32")), ("cyDest", SimTypeInt(signed=True, label="Int32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("xformSrc", SimTypeRef("XFORM", SimStruct)), ("crBkColorSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("xMask", SimTypeInt(signed=True, label="Int32")), ("yMask", SimTypeInt(signed=True, label="Int32")), ("iUsageMask", SimTypeInt(signed=False, label="UInt32")), ("offBmiMask", SimTypeInt(signed=False, label="UInt32")), ("cbBmiMask", SimTypeInt(signed=False, label="UInt32")), ("offBitsMask", SimTypeInt(signed=False, label="UInt32")), ("cbBitsMask", SimTypeInt(signed=False, label="UInt32")),)), name="EMRMASKBLT", pack=False, align=None),
    "EMRMODIFYWORLDTRANSFORM": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("xform", SimTypeRef("XFORM", SimStruct)), ("iMode", SimTypeInt(signed=False, label="MODIFY_WORLD_TRANSFORM_MODE")),)), name="EMRMODIFYWORLDTRANSFORM", pack=False, align=None),
    "EMRNAMEDESCAPE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("iEscape", SimTypeInt(signed=True, label="Int32")), ("cbDriver", SimTypeInt(signed=True, label="Int32")), ("cbEscData", SimTypeInt(signed=True, label="Int32")), ("EscData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRNAMEDESCAPE", pack=False, align=None),
    "EMROFFSETCLIPRGN": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ptlOffset", SimTypeRef("POINTL", SimStruct)),)), name="EMROFFSETCLIPRGN", pack=False, align=None),
    "EMRPIXELFORMAT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("pfd", SimTypeRef("PIXELFORMATDESCRIPTOR", SimStruct)),)), name="EMRPIXELFORMAT", pack=False, align=None),
    "EMRPLGBLT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("aptlDest", SimTypeArray(SimTypeRef("POINTL", SimStruct), 3)), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("cxSrc", SimTypeInt(signed=True, label="Int32")), ("cySrc", SimTypeInt(signed=True, label="Int32")), ("xformSrc", SimTypeRef("XFORM", SimStruct)), ("crBkColorSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("xMask", SimTypeInt(signed=True, label="Int32")), ("yMask", SimTypeInt(signed=True, label="Int32")), ("iUsageMask", SimTypeInt(signed=False, label="UInt32")), ("offBmiMask", SimTypeInt(signed=False, label="UInt32")), ("cbBmiMask", SimTypeInt(signed=False, label="UInt32")), ("offBitsMask", SimTypeInt(signed=False, label="UInt32")), ("cbBitsMask", SimTypeInt(signed=False, label="UInt32")),)), name="EMRPLGBLT", pack=False, align=None),
    "EMRPOLYDRAW": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cptl", SimTypeInt(signed=False, label="UInt32")), ("aptl", SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0)), ("abTypes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRPOLYDRAW", pack=False, align=None),
    "EMRPOLYDRAW16": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cpts", SimTypeInt(signed=False, label="UInt32")), ("apts", SimTypePointer(SimTypeRef("POINTS", SimStruct), offset=0)), ("abTypes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRPOLYDRAW16", pack=False, align=None),
    "EMRPOLYLINE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cptl", SimTypeInt(signed=False, label="UInt32")), ("aptl", SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0)),)), name="EMRPOLYLINE", pack=False, align=None),
    "EMRPOLYLINE16": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("cpts", SimTypeInt(signed=False, label="UInt32")), ("apts", SimTypePointer(SimTypeRef("POINTS", SimStruct), offset=0)),)), name="EMRPOLYLINE16", pack=False, align=None),
    "EMRPOLYPOLYLINE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("nPolys", SimTypeInt(signed=False, label="UInt32")), ("cptl", SimTypeInt(signed=False, label="UInt32")), ("aPolyCounts", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("aptl", SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0)),)), name="EMRPOLYPOLYLINE", pack=False, align=None),
    "EMRPOLYPOLYLINE16": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("nPolys", SimTypeInt(signed=False, label="UInt32")), ("cpts", SimTypeInt(signed=False, label="UInt32")), ("aPolyCounts", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("apts", SimTypePointer(SimTypeRef("POINTS", SimStruct), offset=0)),)), name="EMRPOLYPOLYLINE16", pack=False, align=None),
    "EMRPOLYTEXTOUTA": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("iGraphicsMode", SimTypeInt(signed=False, label="UInt32")), ("exScale", SimTypeFloat(size=32)), ("eyScale", SimTypeFloat(size=32)), ("cStrings", SimTypeInt(signed=True, label="Int32")), ("aemrtext", SimTypePointer(SimTypeRef("EMRTEXT", SimStruct), offset=0)),)), name="EMRPOLYTEXTOUTA", pack=False, align=None),
    "EMRRESIZEPALETTE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPal", SimTypeInt(signed=False, label="UInt32")), ("cEntries", SimTypeInt(signed=False, label="UInt32")),)), name="EMRRESIZEPALETTE", pack=False, align=None),
    "EMRRESTOREDC": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("iRelative", SimTypeInt(signed=True, label="Int32")),)), name="EMRRESTOREDC", pack=False, align=None),
    "EMRROUNDRECT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBox", SimTypeRef("RECTL", SimStruct)), ("szlCorner", SimTypeRef("SIZE", SimStruct)),)), name="EMRROUNDRECT", pack=False, align=None),
    "EMRSCALEVIEWPORTEXTEX": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("xNum", SimTypeInt(signed=True, label="Int32")), ("xDenom", SimTypeInt(signed=True, label="Int32")), ("yNum", SimTypeInt(signed=True, label="Int32")), ("yDenom", SimTypeInt(signed=True, label="Int32")),)), name="EMRSCALEVIEWPORTEXTEX", pack=False, align=None),
    "EMRSELECTCLIPPATH": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("iMode", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSELECTCLIPPATH", pack=False, align=None),
    "EMRSELECTOBJECT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihObject", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSELECTOBJECT", pack=False, align=None),
    "EMRSELECTPALETTE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPal", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSELECTPALETTE", pack=False, align=None),
    "EMRSETARCDIRECTION": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("iArcDirection", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSETARCDIRECTION", pack=False, align=None),
    "EMRSETCOLORADJUSTMENT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ColorAdjustment", SimTypeRef("COLORADJUSTMENT", SimStruct)),)), name="EMRSETCOLORADJUSTMENT", pack=False, align=None),
    "EMRSETCOLORSPACE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihCS", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSETCOLORSPACE", pack=False, align=None),
    "EMRSETDIBITSTODEVICE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("cxSrc", SimTypeInt(signed=True, label="Int32")), ("cySrc", SimTypeInt(signed=True, label="Int32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("iStartScan", SimTypeInt(signed=False, label="UInt32")), ("cScans", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSETDIBITSTODEVICE", pack=False, align=None),
    "EMRSETICMPROFILE": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbName", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EMRSETICMPROFILE", pack=False, align=None),
    "EMRSETMAPPERFLAGS": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSETMAPPERFLAGS", pack=False, align=None),
    "EMRSETMITERLIMIT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("eMiterLimit", SimTypeFloat(size=32)),)), name="EMRSETMITERLIMIT", pack=False, align=None),
    "EMRSETPALETTEENTRIES": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ihPal", SimTypeInt(signed=False, label="UInt32")), ("iStart", SimTypeInt(signed=False, label="UInt32")), ("cEntries", SimTypeInt(signed=False, label="UInt32")), ("aPalEntries", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)),)), name="EMRSETPALETTEENTRIES", pack=False, align=None),
    "EMRSETPIXELV": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ptlPixel", SimTypeRef("POINTL", SimStruct)), ("crColor", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSETPIXELV", pack=False, align=None),
    "EMRSETTEXTCOLOR": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("crColor", SimTypeInt(signed=False, label="UInt32")),)), name="EMRSETTEXTCOLOR", pack=False, align=None),
    "EMRSETVIEWPORTEXTEX": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("szlExtent", SimTypeRef("SIZE", SimStruct)),)), name="EMRSETVIEWPORTEXTEX", pack=False, align=None),
    "EMRSETVIEWPORTORGEX": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("ptlOrigin", SimTypeRef("POINTL", SimStruct)),)), name="EMRSETVIEWPORTORGEX", pack=False, align=None),
    "EMRSETWORLDTRANSFORM": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("xform", SimTypeRef("XFORM", SimStruct)),)), name="EMRSETWORLDTRANSFORM", pack=False, align=None),
    "EMRSTRETCHBLT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("cxDest", SimTypeInt(signed=True, label="Int32")), ("cyDest", SimTypeInt(signed=True, label="Int32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("xformSrc", SimTypeRef("XFORM", SimStruct)), ("crBkColorSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cxSrc", SimTypeInt(signed=True, label="Int32")), ("cySrc", SimTypeInt(signed=True, label="Int32")),)), name="EMRSTRETCHBLT", pack=False, align=None),
    "EMRSTRETCHDIBITS": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("cxSrc", SimTypeInt(signed=True, label="Int32")), ("cySrc", SimTypeInt(signed=True, label="Int32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("cxDest", SimTypeInt(signed=True, label="Int32")), ("cyDest", SimTypeInt(signed=True, label="Int32")),)), name="EMRSTRETCHDIBITS", pack=False, align=None),
    "EMRTEXT": SimStruct(OrderedDict((("ptlReference", SimTypeRef("POINTL", SimStruct)), ("nChars", SimTypeInt(signed=False, label="UInt32")), ("offString", SimTypeInt(signed=False, label="UInt32")), ("fOptions", SimTypeInt(signed=False, label="UInt32")), ("rcl", SimTypeRef("RECTL", SimStruct)), ("offDx", SimTypeInt(signed=False, label="UInt32")),)), name="EMRTEXT", pack=False, align=None),
    "EMRTRANSPARENTBLT": SimStruct(OrderedDict((("emr", SimTypeRef("EMR", SimStruct)), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("xDest", SimTypeInt(signed=True, label="Int32")), ("yDest", SimTypeInt(signed=True, label="Int32")), ("cxDest", SimTypeInt(signed=True, label="Int32")), ("cyDest", SimTypeInt(signed=True, label="Int32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("xformSrc", SimTypeRef("XFORM", SimStruct)), ("crBkColorSrc", SimTypeInt(signed=False, label="UInt32")), ("iUsageSrc", SimTypeInt(signed=False, label="UInt32")), ("offBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBmiSrc", SimTypeInt(signed=False, label="UInt32")), ("offBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cbBitsSrc", SimTypeInt(signed=False, label="UInt32")), ("cxSrc", SimTypeInt(signed=True, label="Int32")), ("cySrc", SimTypeInt(signed=True, label="Int32")),)), name="EMRTRANSPARENTBLT", pack=False, align=None),
    "EMemoryAvailable": SimTypeInt(signed=False, label="EMemoryAvailable"),
    "EMemoryCriticalLevel": SimTypeInt(signed=False, label="EMemoryCriticalLevel"),
    "ENABLECALLBACK_ENABLED_STATE": SimTypeInt(signed=False, label="ENABLECALLBACK_ENABLED_STATE"),
    "ENABLE_SCROLL_BAR_ARROWS": SimTypeInt(signed=False, label="ENABLE_SCROLL_BAR_ARROWS"),
    "ENABLE_TRACE_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("EnableProperty", SimTypeInt(signed=False, label="UInt32")), ("ControlFlags", SimTypeInt(signed=False, label="UInt32")), ("SourceId", SimTypeBottom(label="Guid")), ("EnableFilterDesc", SimTypePointer(SimTypeRef("EVENT_FILTER_DESCRIPTOR", SimStruct), offset=0)), ("FilterDescCount", SimTypeInt(signed=False, label="UInt32")),)), name="ENABLE_TRACE_PARAMETERS", pack=False, align=None),
    "ENABLE_TRACE_PARAMETERS_V1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("EnableProperty", SimTypeInt(signed=False, label="UInt32")), ("ControlFlags", SimTypeInt(signed=False, label="UInt32")), ("SourceId", SimTypeBottom(label="Guid")), ("EnableFilterDesc", SimTypePointer(SimTypeRef("EVENT_FILTER_DESCRIPTOR", SimStruct), offset=0)),)), name="ENABLE_TRACE_PARAMETERS_V1", pack=False, align=None),
    "ENABLE_VIRTUALIZATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "NumVFs", "EnableVfMigration", "EnableMigrationInterrupt", "EnableVirtualization"]), offset=0),
    "ENCLAVE_IDENTITY": SimStruct(OrderedDict((("OwnerId", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("UniqueId", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("AuthorId", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("FamilyId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ImageId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("EnclaveSvn", SimTypeInt(signed=False, label="UInt32")), ("SecureKernelSvn", SimTypeInt(signed=False, label="UInt32")), ("PlatformSvn", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SigningLevel", SimTypeInt(signed=False, label="UInt32")), ("EnclaveType", SimTypeInt(signed=False, label="UInt32")),)), name="ENCLAVE_IDENTITY", pack=False, align=None),
    "ENCLAVE_INFORMATION": SimStruct(OrderedDict((("EnclaveType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Identity", SimTypeRef("ENCLAVE_IDENTITY", SimStruct)),)), name="ENCLAVE_INFORMATION", pack=False, align=None),
    "ENCLAVE_SEALING_IDENTITY_POLICY": SimTypeInt(signed=False, label="ENCLAVE_SEALING_IDENTITY_POLICY"),
    "ENCLAVE_VBS_BASIC_KEY_REQUEST": SimStruct(OrderedDict((("RequestSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("EnclaveSVN", SimTypeInt(signed=False, label="UInt32")), ("SystemKeyID", SimTypeInt(signed=False, label="UInt32")), ("CurrentSystemKeyID", SimTypeInt(signed=False, label="UInt32")),)), name="ENCLAVE_VBS_BASIC_KEY_REQUEST", pack=False, align=None),
    "ENCORRECTTEXT": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("seltyp", SimTypeInt(signed=False, label="RICH_EDIT_GET_CONTEXT_MENU_SEL_TYPE")),)), name="ENCORRECTTEXT", pack=False, align=None),
    "ENCRYPTED_CREDENTIALW": SimStruct(OrderedDict((("Cred", SimTypeRef("CREDENTIALW", SimStruct)), ("ClearCredentialBlobSize", SimTypeInt(signed=False, label="UInt32")),)), name="ENCRYPTED_CREDENTIALW", pack=False, align=None),
    "ENCRYPTED_DATA_INFO": SimStruct(OrderedDict((("StartingFileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("OutputBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("BytesWithinFileSize", SimTypeInt(signed=False, label="UInt32")), ("BytesWithinValidDataLength", SimTypeInt(signed=False, label="UInt32")), ("CompressionFormat", SimTypeShort(signed=False, label="UInt16")), ("DataUnitShift", SimTypeChar(label="Byte")), ("ChunkShift", SimTypeChar(label="Byte")), ("ClusterShift", SimTypeChar(label="Byte")), ("EncryptionFormat", SimTypeChar(label="Byte")), ("NumberOfDataBlocks", SimTypeShort(signed=False, label="UInt16")), ("DataBlockSize", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="ENCRYPTED_DATA_INFO", pack=False, align=None),
    "ENCRYPTED_FILE_METADATA_SIGNATURE": SimStruct(OrderedDict((("dwEfsAccessType", SimTypeInt(signed=False, label="UInt32")), ("pCertificatesAdded", SimTypePointer(SimTypeRef("ENCRYPTION_CERTIFICATE_HASH_LIST", SimStruct), offset=0)), ("pEncryptionCertificate", SimTypePointer(SimTypeRef("ENCRYPTION_CERTIFICATE", SimStruct), offset=0)), ("pEfsStreamSignature", SimTypePointer(SimTypeRef("EFS_RPC_BLOB", SimStruct), offset=0)),)), name="ENCRYPTED_FILE_METADATA_SIGNATURE", pack=False, align=None),
    "ENCRYPTED_LM_OWF_PASSWORD": SimStruct(OrderedDict((("data", SimTypeArray(SimTypeRef("CYPHER_BLOCK", SimStruct), 2)),)), name="ENCRYPTED_LM_OWF_PASSWORD", pack=False, align=None),
    "ENCRYPTION_BUFFER": SimStruct(OrderedDict((("EncryptionOperation", SimTypeInt(signed=False, label="UInt32")), ("Private", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ENCRYPTION_BUFFER", pack=False, align=None),
    "ENCRYPTION_CERTIFICATE": SimStruct(OrderedDict((("cbTotalLength", SimTypeInt(signed=False, label="UInt32")), ("pUserSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("pCertBlob", SimTypePointer(SimTypeRef("EFS_CERTIFICATE_BLOB", SimStruct), offset=0)),)), name="ENCRYPTION_CERTIFICATE", pack=False, align=None),
    "ENCRYPTION_CERTIFICATE_HASH": SimStruct(OrderedDict((("cbTotalLength", SimTypeInt(signed=False, label="UInt32")), ("pUserSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("pHash", SimTypePointer(SimTypeRef("EFS_HASH_BLOB", SimStruct), offset=0)), ("lpDisplayInformation", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ENCRYPTION_CERTIFICATE_HASH", pack=False, align=None),
    "ENCRYPTION_CERTIFICATE_HASH_LIST": SimStruct(OrderedDict((("nCert_Hash", SimTypeInt(signed=False, label="UInt32")), ("pUsers", SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTION_CERTIFICATE_HASH", SimStruct), offset=0), offset=0)),)), name="ENCRYPTION_CERTIFICATE_HASH_LIST", pack=False, align=None),
    "ENCRYPTION_CERTIFICATE_LIST": SimStruct(OrderedDict((("nUsers", SimTypeInt(signed=False, label="UInt32")), ("pUsers", SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTION_CERTIFICATE", SimStruct), offset=0), offset=0)),)), name="ENCRYPTION_CERTIFICATE_LIST", pack=False, align=None),
    "ENCRYPTION_KEY_CTRL_INPUT": SimStruct(OrderedDict((("HeaderSize", SimTypeInt(signed=False, label="UInt32")), ("StructureSize", SimTypeInt(signed=False, label="UInt32")), ("KeyOffset", SimTypeShort(signed=False, label="UInt16")), ("KeySize", SimTypeShort(signed=False, label="UInt16")), ("DplLock", SimTypeInt(signed=False, label="UInt32")), ("DplUserId", SimTypeLongLong(signed=False, label="UInt64")), ("DplCredentialId", SimTypeLongLong(signed=False, label="UInt64")),)), name="ENCRYPTION_KEY_CTRL_INPUT", pack=False, align=None),
    "ENCRYPTION_PROTECTOR": SimStruct(OrderedDict((("cbTotalLength", SimTypeInt(signed=False, label="UInt32")), ("pUserSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("lpProtectorDescriptor", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ENCRYPTION_PROTECTOR", pack=False, align=None),
    "ENCRYPTION_PROTECTOR_LIST": SimStruct(OrderedDict((("nProtectors", SimTypeInt(signed=False, label="UInt32")), ("pProtectors", SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTION_PROTECTOR", SimStruct), offset=0), offset=0)),)), name="ENCRYPTION_PROTECTOR_LIST", pack=False, align=None),
    "ENCRYPT_MESSAGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "ENDCOMPOSITIONNOTIFY": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("dwCode", SimTypeInt(signed=False, label="ENDCOMPOSITIONNOTIFY_CODE")),)), name="ENDCOMPOSITIONNOTIFY", pack=False, align=None),
    "ENDCOMPOSITIONNOTIFY_CODE": SimTypeInt(signed=False, label="ENDCOMPOSITIONNOTIFY_CODE"),
    "ENDPOINTADDRESS": SimStruct(OrderedDict((("serviceUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("policyUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rawCertificate", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="ENDPOINTADDRESS", pack=False, align=None),
    "ENDPOINTADDRESS2": SimStruct(OrderedDict((("serviceUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("policyUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("identityType", SimTypeInt(signed=False, label="UInt32")), ("identityBytes", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="ENDPOINTADDRESS2", pack=False, align=None),
    "ENDROPFILES": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("hDrop", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cp", SimTypeInt(signed=True, label="Int32")), ("fProtected", SimTypeInt(signed=True, label="Int32")),)), name="ENDROPFILES", pack=False, align=None),
    "ENGSAFESEMAPHORE": SimStruct(OrderedDict((("hsem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lCount", SimTypeInt(signed=True, label="Int32")),)), name="ENGSAFESEMAPHORE", pack=False, align=None),
    "ENG_DEVICE_ATTRIBUTE": SimTypeInt(signed=False, label="ENG_DEVICE_ATTRIBUTE"),
    "ENG_EVENT": SimStruct(OrderedDict((("pKEvent", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("fFlags", SimTypeInt(signed=False, label="UInt32")),)), name="ENG_EVENT", pack=False, align=None),
    "ENG_SYSTEM_ATTRIBUTE": SimTypeInt(signed=False, label="ENG_SYSTEM_ATTRIBUTE"),
    "ENG_TIME_FIELDS": SimStruct(OrderedDict((("usYear", SimTypeShort(signed=False, label="UInt16")), ("usMonth", SimTypeShort(signed=False, label="UInt16")), ("usDay", SimTypeShort(signed=False, label="UInt16")), ("usHour", SimTypeShort(signed=False, label="UInt16")), ("usMinute", SimTypeShort(signed=False, label="UInt16")), ("usSecond", SimTypeShort(signed=False, label="UInt16")), ("usMilliseconds", SimTypeShort(signed=False, label="UInt16")), ("usWeekday", SimTypeShort(signed=False, label="UInt16")),)), name="ENG_TIME_FIELDS", pack=False, align=None),
    "ENHANCED_METAFILE_RECORD_TYPE": SimTypeInt(signed=False, label="ENHANCED_METAFILE_RECORD_TYPE"),
    "ENHANCED_STORAGE_PASSWORD_SILO_INFORMATION": SimStruct(OrderedDict((("CurrentAdminFailures", SimTypeChar(label="Byte")), ("CurrentUserFailures", SimTypeChar(label="Byte")), ("TotalUserAuthenticationCount", SimTypeInt(signed=False, label="UInt32")), ("TotalAdminAuthenticationCount", SimTypeInt(signed=False, label="UInt32")), ("FipsCompliant", SimTypeInt(signed=True, label="Int32")), ("SecurityIDAvailable", SimTypeInt(signed=True, label="Int32")), ("InitializeInProgress", SimTypeInt(signed=True, label="Int32")), ("ITMSArmed", SimTypeInt(signed=True, label="Int32")), ("ITMSArmable", SimTypeInt(signed=True, label="Int32")), ("UserCreated", SimTypeInt(signed=True, label="Int32")), ("ResetOnPORDefault", SimTypeInt(signed=True, label="Int32")), ("ResetOnPORCurrent", SimTypeInt(signed=True, label="Int32")), ("MaxAdminFailures", SimTypeChar(label="Byte")), ("MaxUserFailures", SimTypeChar(label="Byte")), ("TimeToCompleteInitialization", SimTypeInt(signed=False, label="UInt32")), ("TimeRemainingToCompleteInitialization", SimTypeInt(signed=False, label="UInt32")), ("MinTimeToAuthenticate", SimTypeInt(signed=False, label="UInt32")), ("MaxAdminPasswordSize", SimTypeChar(label="Byte")), ("MinAdminPasswordSize", SimTypeChar(label="Byte")), ("MaxAdminHintSize", SimTypeChar(label="Byte")), ("MaxUserPasswordSize", SimTypeChar(label="Byte")), ("MinUserPasswordSize", SimTypeChar(label="Byte")), ("MaxUserHintSize", SimTypeChar(label="Byte")), ("MaxUserNameSize", SimTypeChar(label="Byte")), ("MaxSiloNameSize", SimTypeChar(label="Byte")), ("MaxChallengeSize", SimTypeShort(signed=False, label="UInt16")),)), name="ENHANCED_STORAGE_PASSWORD_SILO_INFORMATION", pack=False, align=None),
    "ENHMETAHEADER": SimStruct(OrderedDict((("iType", SimTypeInt(signed=False, label="UInt32")), ("nSize", SimTypeInt(signed=False, label="UInt32")), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("rclFrame", SimTypeRef("RECTL", SimStruct)), ("dSignature", SimTypeInt(signed=False, label="UInt32")), ("nVersion", SimTypeInt(signed=False, label="UInt32")), ("nBytes", SimTypeInt(signed=False, label="UInt32")), ("nRecords", SimTypeInt(signed=False, label="UInt32")), ("nHandles", SimTypeShort(signed=False, label="UInt16")), ("sReserved", SimTypeShort(signed=False, label="UInt16")), ("nDescription", SimTypeInt(signed=False, label="UInt32")), ("offDescription", SimTypeInt(signed=False, label="UInt32")), ("nPalEntries", SimTypeInt(signed=False, label="UInt32")), ("szlDevice", SimTypeRef("SIZE", SimStruct)), ("szlMillimeters", SimTypeRef("SIZE", SimStruct)), ("cbPixelFormat", SimTypeInt(signed=False, label="UInt32")), ("offPixelFormat", SimTypeInt(signed=False, label="UInt32")), ("bOpenGL", SimTypeInt(signed=False, label="UInt32")), ("szlMicrometers", SimTypeRef("SIZE", SimStruct)),)), name="ENHMETAHEADER", pack=False, align=None),
    "ENHMETAHEADER3": SimStruct(OrderedDict((("iType", SimTypeInt(signed=False, label="UInt32")), ("nSize", SimTypeInt(signed=False, label="UInt32")), ("rclBounds", SimTypeRef("RECTL", SimStruct)), ("rclFrame", SimTypeRef("RECTL", SimStruct)), ("dSignature", SimTypeInt(signed=False, label="UInt32")), ("nVersion", SimTypeInt(signed=False, label="UInt32")), ("nBytes", SimTypeInt(signed=False, label="UInt32")), ("nRecords", SimTypeInt(signed=False, label="UInt32")), ("nHandles", SimTypeShort(signed=False, label="UInt16")), ("sReserved", SimTypeShort(signed=False, label="UInt16")), ("nDescription", SimTypeInt(signed=False, label="UInt32")), ("offDescription", SimTypeInt(signed=False, label="UInt32")), ("nPalEntries", SimTypeInt(signed=False, label="UInt32")), ("szlDevice", SimTypeRef("SIZE", SimStruct)), ("szlMillimeters", SimTypeRef("SIZE", SimStruct)),)), name="ENHMETAHEADER3", pack=False, align=None),
    "ENHMETARECORD": SimStruct(OrderedDict((("iType", SimTypeInt(signed=False, label="ENHANCED_METAFILE_RECORD_TYPE")), ("nSize", SimTypeInt(signed=False, label="UInt32")), ("dParm", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="ENHMETARECORD", pack=False, align=None),
    "ENHMFENUMPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("HANDLETABLE", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeRef("ENHMETARECORD", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hdc", "lpht", "lpmr", "nHandles", "data"]), offset=0),
    "ENLINK": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("msg", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("chrg", SimTypeRef("CHARRANGE", SimStruct)),)), name="ENLINK", pack=False, align=None),
    "ENLISTMENT_BASIC_INFORMATION": SimStruct(OrderedDict((("EnlistmentId", SimTypeBottom(label="Guid")), ("TransactionId", SimTypeBottom(label="Guid")), ("ResourceManagerId", SimTypeBottom(label="Guid")),)), name="ENLISTMENT_BASIC_INFORMATION", pack=False, align=None),
    "ENLISTMENT_CRM_INFORMATION": SimStruct(OrderedDict((("CrmTransactionManagerId", SimTypeBottom(label="Guid")), ("CrmResourceManagerId", SimTypeBottom(label="Guid")), ("CrmEnlistmentId", SimTypeBottom(label="Guid")),)), name="ENLISTMENT_CRM_INFORMATION", pack=False, align=None),
    "ENLISTMENT_INFORMATION_CLASS": SimTypeInt(signed=False, label="ENLISTMENT_INFORMATION_CLASS"),
    "ENLOWFIRTF": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("szControl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ENLOWFIRTF", pack=False, align=None),
    "ENOLEOPFAILED": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("iob", SimTypeInt(signed=True, label="Int32")), ("lOper", SimTypeInt(signed=True, label="Int32")), ("hr", SimTypeInt(signed=True, label="Int32")),)), name="ENOLEOPFAILED", pack=False, align=None),
    "ENPROTECTED": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("msg", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("chrg", SimTypeRef("CHARRANGE", SimStruct)),)), name="ENPROTECTED", pack=False, align=None),
    "ENSAVECLIPBOARD": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("cObjectCount", SimTypeInt(signed=True, label="Int32")), ("cch", SimTypeInt(signed=True, label="Int32")),)), name="ENSAVECLIPBOARD", pack=False, align=None),
    "ENTERPRISE_DATA_POLICIES": SimTypeInt(signed=False, label="ENTERPRISE_DATA_POLICIES"),
    "ENTRYID": SimStruct(OrderedDict((("abFlags", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("ab", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ENTRYID", pack=False, align=None),
    "ENTRY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EntryAddress", "Context"]), offset=0),
    "ENUMERATE_SECURITY_PACKAGES_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "ENUMERATE_SECURITY_PACKAGES_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "ENUMERATION_BUFFER": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("OwnerProcessId", SimTypeInt(signed=False, label="UInt32")), ("FlowNameLength", SimTypeShort(signed=False, label="UInt16")), ("FlowName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("pFlow", SimTypePointer(SimTypeRef("TC_GEN_FLOW", SimStruct), offset=0)), ("NumberOfFilters", SimTypeInt(signed=False, label="UInt32")), ("GenericFilter", SimTypePointer(SimTypeRef("TC_GEN_FILTER", SimStruct), offset=0)),)), name="ENUMERATION_BUFFER", pack=False, align=None),
    "ENUMLOGFONTA": SimStruct(OrderedDict((("elfLogFont", SimTypeRef("LOGFONTA", SimStruct)), ("elfFullName", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("elfStyle", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="ENUMLOGFONTA", pack=False, align=None),
    "ENUMLOGFONTEXA": SimStruct(OrderedDict((("elfLogFont", SimTypeRef("LOGFONTA", SimStruct)), ("elfFullName", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("elfStyle", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("elfScript", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="ENUMLOGFONTEXA", pack=False, align=None),
    "ENUMLOGFONTEXDVA": SimStruct(OrderedDict((("elfEnumLogfontEx", SimTypeRef("ENUMLOGFONTEXA", SimStruct)), ("elfDesignVector", SimTypeRef("DESIGNVECTOR", SimStruct)),)), name="ENUMLOGFONTEXDVA", pack=False, align=None),
    "ENUMLOGFONTEXDVW": SimStruct(OrderedDict((("elfEnumLogfontEx", SimTypeRef("ENUMLOGFONTEXW", SimStruct)), ("elfDesignVector", SimTypeRef("DESIGNVECTOR", SimStruct)),)), name="ENUMLOGFONTEXDVW", pack=False, align=None),
    "ENUMLOGFONTEXW": SimStruct(OrderedDict((("elfLogFont", SimTypeRef("LOGFONTW", SimStruct)), ("elfFullName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("elfStyle", SimTypeArray(SimTypeChar(label="Char"), 32)), ("elfScript", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="ENUMLOGFONTEXW", pack=False, align=None),
    "ENUMLOGFONTW": SimStruct(OrderedDict((("elfLogFont", SimTypeRef("LOGFONTW", SimStruct)), ("elfFullName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("elfStyle", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="ENUMLOGFONTW", pack=False, align=None),
    "ENUMRECTS": SimStruct(OrderedDict((("c", SimTypeInt(signed=False, label="UInt32")), ("arcl", SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0)),)), name="ENUMRECTS", pack=False, align=None),
    "ENUMRESLANGPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hModule", "lpType", "lpName", "wLanguage", "lParam"]), offset=0),
    "ENUMRESLANGPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hModule", "lpType", "lpName", "wLanguage", "lParam"]), offset=0),
    "ENUMRESNAMEPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hModule", "lpType", "lpName", "lParam"]), offset=0),
    "ENUMRESNAMEPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hModule", "lpType", "lpName", "lParam"]), offset=0),
    "ENUMRESTYPEPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hModule", "lpType", "lParam"]), offset=0),
    "ENUMRESTYPEPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hModule", "lpType", "lParam"]), offset=0),
    "ENUMTEXTMETRICA": SimStruct(OrderedDict((("etmNewTextMetricEx", SimTypeRef("NEWTEXTMETRICEXA", SimStruct)), ("etmAxesList", SimTypeRef("AXESLISTA", SimStruct)),)), name="ENUMTEXTMETRICA", pack=False, align=None),
    "ENUMTEXTMETRICW": SimStruct(OrderedDict((("etmNewTextMetricEx", SimTypeRef("NEWTEXTMETRICEXW", SimStruct)), ("etmAxesList", SimTypeRef("AXESLISTW", SimStruct)),)), name="ENUMTEXTMETRICW", pack=False, align=None),
    "ENUMTYPEA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFields", SimTypeInt(signed=False, label="UInt32")), ("pDeviceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwMediaType", SimTypeInt(signed=False, label="UInt32")), ("dwDitheringMode", SimTypeInt(signed=False, label="UInt32")), ("dwResolution", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("dwCMMType", SimTypeInt(signed=False, label="UInt32")), ("dwClass", SimTypeInt(signed=False, label="UInt32")), ("dwDataColorSpace", SimTypeInt(signed=False, label="UInt32")), ("dwConnectionSpace", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("dwPlatform", SimTypeInt(signed=False, label="UInt32")), ("dwProfileFlags", SimTypeInt(signed=False, label="UInt32")), ("dwManufacturer", SimTypeInt(signed=False, label="UInt32")), ("dwModel", SimTypeInt(signed=False, label="UInt32")), ("dwAttributes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("dwRenderingIntent", SimTypeInt(signed=False, label="UInt32")), ("dwCreator", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClass", SimTypeInt(signed=False, label="UInt32")),)), name="ENUMTYPEA", pack=False, align=None),
    "ENUMTYPEW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFields", SimTypeInt(signed=False, label="UInt32")), ("pDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMediaType", SimTypeInt(signed=False, label="UInt32")), ("dwDitheringMode", SimTypeInt(signed=False, label="UInt32")), ("dwResolution", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("dwCMMType", SimTypeInt(signed=False, label="UInt32")), ("dwClass", SimTypeInt(signed=False, label="UInt32")), ("dwDataColorSpace", SimTypeInt(signed=False, label="UInt32")), ("dwConnectionSpace", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("dwPlatform", SimTypeInt(signed=False, label="UInt32")), ("dwProfileFlags", SimTypeInt(signed=False, label="UInt32")), ("dwManufacturer", SimTypeInt(signed=False, label="UInt32")), ("dwModel", SimTypeInt(signed=False, label="UInt32")), ("dwAttributes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("dwRenderingIntent", SimTypeInt(signed=False, label="UInt32")), ("dwCreator", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClass", SimTypeInt(signed=False, label="UInt32")),)), name="ENUMTYPEW", pack=False, align=None),
    "ENUMUILANG": SimStruct(OrderedDict((("NumOfEnumUILang", SimTypeInt(signed=False, label="UInt32")), ("SizeOfEnumUIBuffer", SimTypeInt(signed=False, label="UInt32")), ("pEnumUIBuffer", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="ENUMUILANG", pack=False, align=None),
    "ENUM_BINDING_PATHS_FLAGS": SimTypeInt(signed=False, label="ENUM_BINDING_PATHS_FLAGS"),
    "ENUM_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DCISURFACEINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpSurfaceInfo", "lpContext"]), offset=0),
    "ENUM_CATYPES": SimTypeInt(signed=False, label="ENUM_CATYPES"),
    "ENUM_CERT_COLUMN_VALUE_FLAGS": SimTypeInt(signed=False, label="ENUM_CERT_COLUMN_VALUE_FLAGS"),
    "ENUM_CONTROLS_WHICH_FLAGS": SimTypeInt(signed=False, label="ENUM_CONTROLS_WHICH_FLAGS"),
    "ENUM_DATE_FORMATS_FLAGS": SimTypeInt(signed=False, label="ENUM_DATE_FORMATS_FLAGS"),
    "ENUM_DISPLAY_SETTINGS_FLAGS": SimTypeInt(signed=False, label="ENUM_DISPLAY_SETTINGS_FLAGS"),
    "ENUM_DISPLAY_SETTINGS_MODE": SimTypeInt(signed=False, label="ENUM_DISPLAY_SETTINGS_MODE"),
    "ENUM_PAGE_FILE_INFORMATION": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("TotalSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TotalInUse", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="ENUM_PAGE_FILE_INFORMATION", pack=False, align=None),
    "ENUM_PERIOD": SimTypeInt(signed=False, label="ENUM_PERIOD"),
    "ENUM_PROCESS_MODULES_EX_FLAGS": SimTypeInt(signed=False, label="ENUM_PROCESS_MODULES_EX_FLAGS"),
    "ENUM_SERVICE_STATE": SimTypeInt(signed=False, label="ENUM_SERVICE_STATE"),
    "ENUM_SERVICE_STATUSA": SimStruct(OrderedDict((("lpServiceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServiceStatus", SimTypeRef("SERVICE_STATUS", SimStruct)),)), name="ENUM_SERVICE_STATUSA", pack=False, align=None),
    "ENUM_SERVICE_STATUSW": SimStruct(OrderedDict((("lpServiceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServiceStatus", SimTypeRef("SERVICE_STATUS", SimStruct)),)), name="ENUM_SERVICE_STATUSW", pack=False, align=None),
    "ENUM_SERVICE_STATUS_PROCESSA": SimStruct(OrderedDict((("lpServiceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServiceStatusProcess", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)),)), name="ENUM_SERVICE_STATUS_PROCESSA", pack=False, align=None),
    "ENUM_SERVICE_STATUS_PROCESSW": SimStruct(OrderedDict((("lpServiceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServiceStatusProcess", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)),)), name="ENUM_SERVICE_STATUS_PROCESSW", pack=False, align=None),
    "ENUM_SERVICE_TYPE": SimTypeInt(signed=False, label="ENUM_SERVICE_TYPE"),
    "ENUM_SYSTEM_CODE_PAGES_FLAGS": SimTypeInt(signed=False, label="ENUM_SYSTEM_CODE_PAGES_FLAGS"),
    "ENUM_SYSTEM_LANGUAGE_GROUPS_FLAGS": SimTypeInt(signed=False, label="ENUM_SYSTEM_LANGUAGE_GROUPS_FLAGS"),
    "EOC_ChangeType": SimTypeInt(signed=False, label="EOC_ChangeType"),
    "EOF_WAIT_BLOCK": SimStruct(OrderedDict((("EofWaitLinks", SimTypeRef("LIST_ENTRY", SimStruct)), ("Event", SimTypeRef("KEVENT", SimStruct)),)), name="EOF_WAIT_BLOCK", pack=False, align=None),
    "EOLE_AUTHENTICATION_CAPABILITIES": SimTypeInt(signed=False, label="EOLE_AUTHENTICATION_CAPABILITIES"),
    "EPcxConnectionType": SimTypeInt(signed=False, label="EPcxConnectionType"),
    "EPcxGenLocation": SimTypeInt(signed=False, label="EPcxGenLocation"),
    "EPcxGeoLocation": SimTypeInt(signed=False, label="EPcxGeoLocation"),
    "EPolicyAction": SimTypeInt(signed=False, label="EPolicyAction"),
    "EPrintPropertyType": SimTypeInt(signed=False, label="EPrintPropertyType"),
    "EPrintTicketScope": SimTypeInt(signed=False, label="EPrintTicketScope"),
    "EPrintXPSJobOperation": SimTypeInt(signed=False, label="EPrintXPSJobOperation"),
    "EPrintXPSJobProgress": SimTypeInt(signed=False, label="EPrintXPSJobProgress"),
    "EPxcPortConnection": SimTypeInt(signed=False, label="EPxcPortConnection"),
    "ERASE_TAPE_TYPE": SimTypeInt(signed=False, label="ERASE_TAPE_TYPE"),
    "ERESOURCE": SimStruct(OrderedDict((("SystemResourcesList", SimTypeRef("LIST_ENTRY", SimStruct)), ("OwnerTable", SimTypePointer(SimTypeRef("OWNER_ENTRY", SimStruct), offset=0)), ("ActiveCount", SimTypeShort(signed=True, label="Int16")), ("Anonymous1", SimUnion({"Flag": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("ReservedLowFlags", SimTypeChar(label="Byte")), ("WaiterPriority", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("SharedWaiters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ExclusiveWaiters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OwnerEntry", SimTypeRef("OWNER_ENTRY", SimStruct)), ("ActiveEntries", SimTypeInt(signed=False, label="UInt32")), ("ContentionCount", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSharedWaiters", SimTypeInt(signed=False, label="UInt32")), ("NumberOfExclusiveWaiters", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"Address": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "CreatorBackTraceIndex": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")), ("SpinLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="ERESOURCE", pack=False, align=None),
    "ERF": SimStruct(OrderedDict((("erfOper", SimTypeInt(signed=True, label="Int32")), ("erfType", SimTypeInt(signed=True, label="Int32")), ("fError", SimTypeInt(signed=True, label="Int32")),)), name="ERF", pack=False, align=None),
    "ERRLOG_OTHER_INFO": SimStruct(OrderedDict((("alrter_errcode", SimTypeInt(signed=False, label="UInt32")), ("alrter_offset", SimTypeInt(signed=False, label="UInt32")),)), name="ERRLOG_OTHER_INFO", pack=False, align=None),
    "ERRORINFO": SimStruct(OrderedDict((("hrError", SimTypeInt(signed=True, label="Int32")), ("dwMinor", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")), ("iid", SimTypeBottom(label="Guid")), ("dispid", SimTypeInt(signed=True, label="Int32")),)), name="ERRORINFO", pack=False, align=None),
    "ERRORRESUMEACTION": SimTypeInt(signed=False, label="ERRORRESUMEACTION"),
    "ERROR_ADVISE_MESSAGE_TYPE": SimTypeInt(signed=False, label="ERROR_ADVISE_MESSAGE_TYPE"),
    "ERROR_ADVISE_RESULT": SimTypeInt(signed=False, label="ERROR_ADVISE_RESULT"),
    "ERROR_LOG": SimStruct(OrderedDict((("el_len", SimTypeInt(signed=False, label="UInt32")), ("el_reserved", SimTypeInt(signed=False, label="UInt32")), ("el_time", SimTypeInt(signed=False, label="UInt32")), ("el_error", SimTypeInt(signed=False, label="UInt32")), ("el_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("el_text", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("el_data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("el_data_size", SimTypeInt(signed=False, label="UInt32")), ("el_nstrings", SimTypeInt(signed=False, label="UInt32")),)), name="ERROR_LOG", pack=False, align=None),
    "ERROR_NOTIFICATION": SimStruct(OrderedDict((("cbEntryID", SimTypeInt(signed=False, label="UInt32")), ("lpEntryID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("scode", SimTypeInt(signed=True, label="Int32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpMAPIError", SimTypePointer(SimTypeRef("MAPIERROR", SimStruct), offset=0)),)), name="ERROR_NOTIFICATION", pack=False, align=None),
    "ERROR_SPEC": SimStruct(OrderedDict((("errs_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("errs_u", SimUnion({"errs_ipv4": SimTypeRef("Error_Spec_IPv4", SimStruct)}, name="<anon>", label="None")),)), name="ERROR_SPEC", pack=False, align=None),
    "ERole": SimTypeInt(signed=False, label="ERole"),
    "ESCAPE_COMM_FUNCTION": SimTypeInt(signed=False, label="ESCAPE_COMM_FUNCTION"),
    "ESymbolReadingPolicy": SimTypeInt(signed=False, label="ESymbolReadingPolicy"),
    "ETHERNET_HEADER": SimStruct(OrderedDict((("Destination", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 6), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Source", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 6), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous", SimUnion({"Type": SimTypeShort(signed=False, label="UInt16"), "Length": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="ETHERNET_HEADER", pack=False, align=None),
    "ETH_FILTER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ETO_OPTIONS": SimTypeInt(signed=False, label="ETO_OPTIONS"),
    "ETWENABLECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("EVENT_FILTER_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["SourceId", "ControlCode", "Level", "MatchAnyKeyword", "MatchAllKeyword", "FilterData", "CallbackContext"]), offset=0),
    "ETW_BUFFER_CALLBACK_INFORMATION": SimStruct(OrderedDict((("TraceHandle", SimTypeLongLong(signed=False, label="UInt64")), ("LogfileHeader", SimTypePointer(SimTypeRef("TRACE_LOGFILE_HEADER", SimStruct), offset=0)), ("BuffersRead", SimTypeInt(signed=False, label="UInt32")),)), name="ETW_BUFFER_CALLBACK_INFORMATION", pack=False, align=None),
    "ETW_BUFFER_CONTEXT": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("ProcessorNumber", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("LoggerId", SimTypeShort(signed=False, label="UInt16")),)), name="ETW_BUFFER_CONTEXT", pack=False, align=None),
    "ETW_BUFFER_HEADER": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ClientContext", SimTypeRef("ETW_BUFFER_CONTEXT", SimStruct)), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("FilledBytes", SimTypeInt(signed=False, label="UInt32")), ("Reserved4", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)),)), name="ETW_BUFFER_HEADER", pack=False, align=None),
    "ETW_COMPRESSION_RESUMPTION_MODE": SimTypeInt(signed=False, label="ETW_COMPRESSION_RESUMPTION_MODE"),
    "ETW_OPEN_TRACE_OPTIONS": SimStruct(OrderedDict((("ProcessTraceModes", SimTypeInt(signed=False, label="ETW_PROCESS_TRACE_MODES")), ("EventCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_RECORD", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["EventRecord"]), offset=0)), ("EventCallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ETW_BUFFER_HEADER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("ETW_BUFFER_CALLBACK_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Buffer", "BufferSize", "ConsumerInfo", "CallbackContext"]), offset=0)), ("BufferCallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="ETW_OPEN_TRACE_OPTIONS", pack=False, align=None),
    "ETW_PMC_COUNTER_OWNER": SimStruct(OrderedDict((("OwnerType", SimTypeInt(signed=False, label="ETW_PMC_COUNTER_OWNER_TYPE")), ("ProfileSource", SimTypeInt(signed=False, label="UInt32")), ("OwnerTag", SimTypeInt(signed=False, label="UInt32")),)), name="ETW_PMC_COUNTER_OWNER", pack=False, align=None),
    "ETW_PMC_COUNTER_OWNERSHIP_STATUS": SimStruct(OrderedDict((("ProcessorNumber", SimTypeInt(signed=False, label="UInt32")), ("NumberOfCounters", SimTypeInt(signed=False, label="UInt32")), ("CounterOwners", SimTypePointer(SimTypeRef("ETW_PMC_COUNTER_OWNER", SimStruct), offset=0)),)), name="ETW_PMC_COUNTER_OWNERSHIP_STATUS", pack=False, align=None),
    "ETW_PMC_COUNTER_OWNER_TYPE": SimTypeInt(signed=False, label="ETW_PMC_COUNTER_OWNER_TYPE"),
    "ETW_PMC_SESSION_INFO": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("LoggerId", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("ProfileSourceCount", SimTypeInt(signed=False, label="UInt32")), ("HookIdCount", SimTypeInt(signed=False, label="UInt32")),)), name="ETW_PMC_SESSION_INFO", pack=False, align=None),
    "ETW_PROCESS_HANDLE_INFO_TYPE": SimTypeInt(signed=False, label="ETW_PROCESS_HANDLE_INFO_TYPE"),
    "ETW_PROCESS_TRACE_MODES": SimTypeInt(signed=False, label="ETW_PROCESS_TRACE_MODES"),
    "ETW_PROVIDER_TRAIT_TYPE": SimTypeInt(signed=False, label="ETW_PROVIDER_TRAIT_TYPE"),
    "ETW_TRACE_PARTITION_INFORMATION": SimStruct(OrderedDict((("PartitionId", SimTypeBottom(label="Guid")), ("ParentId", SimTypeBottom(label="Guid")), ("QpcOffsetFromRoot", SimTypeLongLong(signed=True, label="Int64")), ("PartitionType", SimTypeInt(signed=False, label="UInt32")),)), name="ETW_TRACE_PARTITION_INFORMATION", pack=False, align=None),
    "ETW_TRACE_PARTITION_INFORMATION_V2": SimStruct(OrderedDict((("QpcOffsetFromRoot", SimTypeLongLong(signed=True, label="Int64")), ("PartitionType", SimTypeInt(signed=False, label="UInt32")), ("PartitionId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ParentId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ETW_TRACE_PARTITION_INFORMATION_V2", pack=False, align=None),
    "ETW_TRACE_SESSION_SETTINGS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("MinimumBuffers", SimTypeInt(signed=False, label="UInt32")), ("MaximumBuffers", SimTypeInt(signed=False, label="UInt32")), ("LoggerMode", SimTypeInt(signed=False, label="UInt32")), ("FlushTimer", SimTypeInt(signed=False, label="UInt32")), ("FlushThreshold", SimTypeInt(signed=False, label="UInt32")), ("ClockType", SimTypeInt(signed=False, label="UInt32")),)), name="ETW_TRACE_SESSION_SETTINGS", pack=False, align=None),
    "ETaskType": SimTypeInt(signed=False, label="ETaskType"),
    "EVENTLOGRECORD": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("RecordNumber", SimTypeInt(signed=False, label="UInt32")), ("TimeGenerated", SimTypeInt(signed=False, label="UInt32")), ("TimeWritten", SimTypeInt(signed=False, label="UInt32")), ("EventID", SimTypeInt(signed=False, label="UInt32")), ("EventType", SimTypeInt(signed=False, label="REPORT_EVENT_TYPE")), ("NumStrings", SimTypeShort(signed=False, label="UInt16")), ("EventCategory", SimTypeShort(signed=False, label="UInt16")), ("ReservedFlags", SimTypeShort(signed=False, label="UInt16")), ("ClosingRecordNumber", SimTypeInt(signed=False, label="UInt32")), ("StringOffset", SimTypeInt(signed=False, label="UInt32")), ("UserSidLength", SimTypeInt(signed=False, label="UInt32")), ("UserSidOffset", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="EVENTLOGRECORD", pack=False, align=None),
    "EVENTLOG_FULL_INFORMATION": SimStruct(OrderedDict((("dwFull", SimTypeInt(signed=False, label="UInt32")),)), name="EVENTLOG_FULL_INFORMATION", pack=False, align=None),
    "EVENTMSG": SimStruct(OrderedDict((("message", SimTypeInt(signed=False, label="UInt32")), ("paramL", SimTypeInt(signed=False, label="UInt32")), ("paramH", SimTypeInt(signed=False, label="UInt32")), ("time", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="EVENTMSG", pack=False, align=None),
    "EVENTSECURITYOPERATION": SimTypeInt(signed=False, label="EVENTSECURITYOPERATION"),
    "EVENTSFORLOGFILE": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("szLogicalLogFile", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ulNumRecords", SimTypeInt(signed=False, label="UInt32")), ("pEventLogRecords", SimTypePointer(SimTypeRef("EVENTLOGRECORD", SimStruct), offset=0)),)), name="EVENTSFORLOGFILE", pack=False, align=None),
    "EVENT_DATA_DESCRIPTOR": SimStruct(OrderedDict((("Ptr", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="EVENT_DATA_DESCRIPTOR", pack=False, align=None),
    "EVENT_DATA_HEADER": SimStruct(OrderedDict((("cbEventDataHeader", SimTypeInt(signed=False, label="UInt32")), ("guidEventType", SimTypeBottom(label="Guid")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbEventDataSid", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_DATA_HEADER", pack=False, align=None),
    "EVENT_DESCRIPTOR": SimStruct(OrderedDict((("Id", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeChar(label="Byte")), ("Channel", SimTypeChar(label="Byte")), ("Level", SimTypeChar(label="Byte")), ("Opcode", SimTypeChar(label="Byte")), ("Task", SimTypeShort(signed=False, label="UInt16")), ("Keyword", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_DESCRIPTOR", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_EVENT_KEY": SimStruct(OrderedDict((("Key", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_EXTENDED_ITEM_EVENT_KEY", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_INSTANCE": SimStruct(OrderedDict((("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("ParentInstanceId", SimTypeInt(signed=False, label="UInt32")), ("ParentGuid", SimTypeBottom(label="Guid")),)), name="EVENT_EXTENDED_ITEM_INSTANCE", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_PEBS_INDEX": SimStruct(OrderedDict((("PebsIndex", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_EXTENDED_ITEM_PEBS_INDEX", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_PMC_COUNTERS": SimStruct(OrderedDict((("Counter", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="EVENT_EXTENDED_ITEM_PMC_COUNTERS", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_PROCESS_START_KEY": SimStruct(OrderedDict((("ProcessStartKey", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_EXTENDED_ITEM_PROCESS_START_KEY", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_RELATED_ACTIVITYID": SimStruct(OrderedDict((("RelatedActivityId", SimTypeBottom(label="Guid")),)), name="EVENT_EXTENDED_ITEM_RELATED_ACTIVITYID", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_STACK_KEY32": SimStruct(OrderedDict((("MatchId", SimTypeLongLong(signed=False, label="UInt64")), ("StackKey", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_EXTENDED_ITEM_STACK_KEY32", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_STACK_KEY64": SimStruct(OrderedDict((("MatchId", SimTypeLongLong(signed=False, label="UInt64")), ("StackKey", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_EXTENDED_ITEM_STACK_KEY64", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_STACK_TRACE32": SimStruct(OrderedDict((("MatchId", SimTypeLongLong(signed=False, label="UInt64")), ("Address", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="EVENT_EXTENDED_ITEM_STACK_TRACE32", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_STACK_TRACE64": SimStruct(OrderedDict((("MatchId", SimTypeLongLong(signed=False, label="UInt64")), ("Address", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="EVENT_EXTENDED_ITEM_STACK_TRACE64", pack=False, align=None),
    "EVENT_EXTENDED_ITEM_TS_ID": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_EXTENDED_ITEM_TS_ID", pack=False, align=None),
    "EVENT_FIELD_TYPE": SimTypeInt(signed=False, label="EVENT_FIELD_TYPE"),
    "EVENT_FILTER_DESCRIPTOR": SimStruct(OrderedDict((("Ptr", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_FILTER_DESCRIPTOR", pack=False, align=None),
    "EVENT_FILTER_EVENT_ID": SimStruct(OrderedDict((("FilterIn", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Count", SimTypeShort(signed=False, label="UInt16")), ("Events", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="EVENT_FILTER_EVENT_ID", pack=False, align=None),
    "EVENT_FILTER_EVENT_NAME": SimStruct(OrderedDict((("MatchAnyKeyword", SimTypeLongLong(signed=False, label="UInt64")), ("MatchAllKeyword", SimTypeLongLong(signed=False, label="UInt64")), ("Level", SimTypeChar(label="Byte")), ("FilterIn", SimTypeChar(label="Byte")), ("NameCount", SimTypeShort(signed=False, label="UInt16")), ("Names", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EVENT_FILTER_EVENT_NAME", pack=False, align=None),
    "EVENT_FILTER_HEADER": SimStruct(OrderedDict((("Id", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("InstanceId", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NextOffset", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_FILTER_HEADER", pack=False, align=None),
    "EVENT_FILTER_LEVEL_KW": SimStruct(OrderedDict((("MatchAnyKeyword", SimTypeLongLong(signed=False, label="UInt64")), ("MatchAllKeyword", SimTypeLongLong(signed=False, label="UInt64")), ("Level", SimTypeChar(label="Byte")), ("FilterIn", SimTypeChar(label="Byte")),)), name="EVENT_FILTER_LEVEL_KW", pack=False, align=None),
    "EVENT_HEADER": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("HeaderType", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("EventProperty", SimTypeShort(signed=False, label="UInt16")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("ProviderId", SimTypeBottom(label="Guid")), ("EventDescriptor", SimTypeRef("EVENT_DESCRIPTOR", SimStruct)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("KernelTime", SimTypeInt(signed=False, label="UInt32")), ("UserTime", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ActivityId", SimTypeBottom(label="Guid")),)), name="EVENT_HEADER", pack=False, align=None),
    "EVENT_HEADER_EXTENDED_DATA_ITEM": SimStruct(OrderedDict((("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("ExtType", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)), ("DataSize", SimTypeShort(signed=False, label="UInt16")), ("DataPtr", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_HEADER_EXTENDED_DATA_ITEM", pack=False, align=None),
    "EVENT_INFO_CLASS": SimTypeInt(signed=False, label="EVENT_INFO_CLASS"),
    "EVENT_INSTANCE_HEADER": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"FieldTypeFlags": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("HeaderType", SimTypeChar(label="Byte")), ("MarkerFlags", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Version": SimTypeInt(signed=False, label="UInt32"), "Class": SimTypeRef("_Class_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("RegHandle", SimTypeLongLong(signed=False, label="UInt64")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("ParentInstanceId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous3", SimUnion({"Anonymous1": SimStruct(OrderedDict((("KernelTime", SimTypeInt(signed=False, label="UInt32")), ("UserTime", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous2": SimStruct(OrderedDict((("EventId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("ParentRegHandle", SimTypeLongLong(signed=False, label="UInt64")),)), name="EVENT_INSTANCE_HEADER", pack=False, align=None),
    "EVENT_INSTANCE_INFO": SimStruct(OrderedDict((("RegHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InstanceId", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_INSTANCE_INFO", pack=False, align=None),
    "EVENT_MAP_ENTRY": SimStruct(OrderedDict((("OutputOffset", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Value": SimTypeInt(signed=False, label="UInt32"), "InputOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="EVENT_MAP_ENTRY", pack=False, align=None),
    "EVENT_MAP_INFO": SimStruct(OrderedDict((("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("Flag", SimTypeInt(signed=False, label="MAP_FLAGS")), ("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"MapEntryValueType": SimTypeInt(signed=False, label="MAP_VALUETYPE"), "FormatStringOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MapEntryArray", SimTypePointer(SimTypeRef("EVENT_MAP_ENTRY", SimStruct), offset=0)),)), name="EVENT_MAP_INFO", pack=False, align=None),
    "EVENT_PROPERTY_INFO": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="PROPERTY_FLAGS")), ("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"nonStructType": SimTypeRef("_nonStructType", SimStruct), "structType": SimTypeRef("_structType", SimStruct), "customSchemaType": SimTypeRef("_customSchemaType", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"count": SimTypeShort(signed=False, label="UInt16"), "countPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"length": SimTypeShort(signed=False, label="UInt16"), "lengthPropertyIndex": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="EVENT_PROPERTY_INFO", pack=False, align=None),
    "EVENT_RECORD": SimStruct(OrderedDict((("EventHeader", SimTypeRef("EVENT_HEADER", SimStruct)), ("BufferContext", SimTypeRef("ETW_BUFFER_CONTEXT", SimStruct)), ("ExtendedDataCount", SimTypeShort(signed=False, label="UInt16")), ("UserDataLength", SimTypeShort(signed=False, label="UInt16")), ("ExtendedData", SimTypePointer(SimTypeRef("EVENT_HEADER_EXTENDED_DATA_ITEM", SimStruct), offset=0)), ("UserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UserContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="EVENT_RECORD", pack=False, align=None),
    "EVENT_TRACE": SimStruct(OrderedDict((("Header", SimTypeRef("EVENT_TRACE_HEADER", SimStruct)), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("ParentInstanceId", SimTypeInt(signed=False, label="UInt32")), ("ParentGuid", SimTypeBottom(label="Guid")), ("MofData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MofLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ClientContext": SimTypeInt(signed=False, label="UInt32"), "BufferContext": SimTypeRef("ETW_BUFFER_CONTEXT", SimStruct)}, name="<anon>", label="None")),)), name="EVENT_TRACE", pack=False, align=None),
    "EVENT_TRACE_CONTROL": SimTypeInt(signed=False, label="EVENT_TRACE_CONTROL"),
    "EVENT_TRACE_FLAG": SimTypeInt(signed=False, label="EVENT_TRACE_FLAG"),
    "EVENT_TRACE_HEADER": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"FieldTypeFlags": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("HeaderType", SimTypeChar(label="Byte")), ("MarkerFlags", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Version": SimTypeInt(signed=False, label="UInt32"), "Class": SimTypeRef("_Class_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("Anonymous3", SimUnion({"Guid": SimTypeBottom(label="Guid"), "GuidPtr": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Anonymous4", SimUnion({"Anonymous1": SimStruct(OrderedDict((("KernelTime", SimTypeInt(signed=False, label="UInt32")), ("UserTime", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "ProcessorTime": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous2": SimStruct(OrderedDict((("ClientContext", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="EVENT_TRACE_HEADER", pack=False, align=None),
    "EVENT_TRACE_LOGFILEA": SimStruct(OrderedDict((("LogFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("LoggerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CurrentTime", SimTypeLongLong(signed=True, label="Int64")), ("BuffersRead", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"LogFileMode": SimTypeInt(signed=False, label="UInt32"), "ProcessTraceMode": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CurrentEvent", SimTypeRef("EVENT_TRACE", SimStruct)), ("LogfileHeader", SimTypeRef("TRACE_LOGFILE_HEADER", SimStruct)), ("BufferCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE_LOGFILEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Logfile"]), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Filled", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"EventCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pEvent"]), offset=0), "EventRecordCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_RECORD", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["EventRecord"]), offset=0)}, name="<anon>", label="None")), ("IsKernelTrace", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="EVENT_TRACE_LOGFILEA", pack=False, align=None),
    "EVENT_TRACE_LOGFILEW": SimStruct(OrderedDict((("LogFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LoggerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CurrentTime", SimTypeLongLong(signed=True, label="Int64")), ("BuffersRead", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"LogFileMode": SimTypeInt(signed=False, label="UInt32"), "ProcessTraceMode": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CurrentEvent", SimTypeRef("EVENT_TRACE", SimStruct)), ("LogfileHeader", SimTypeRef("TRACE_LOGFILE_HEADER", SimStruct)), ("BufferCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE_LOGFILEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Logfile"]), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Filled", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"EventCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pEvent"]), offset=0), "EventRecordCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_RECORD", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["EventRecord"]), offset=0)}, name="<anon>", label="None")), ("IsKernelTrace", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="EVENT_TRACE_LOGFILEW", pack=False, align=None),
    "EVENT_TRACE_PROPERTIES": SimStruct(OrderedDict((("Wnode", SimTypeRef("WNODE_HEADER", SimStruct)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("MinimumBuffers", SimTypeInt(signed=False, label="UInt32")), ("MaximumBuffers", SimTypeInt(signed=False, label="UInt32")), ("MaximumFileSize", SimTypeInt(signed=False, label="UInt32")), ("LogFileMode", SimTypeInt(signed=False, label="UInt32")), ("FlushTimer", SimTypeInt(signed=False, label="UInt32")), ("EnableFlags", SimTypeInt(signed=False, label="EVENT_TRACE_FLAG")), ("Anonymous", SimUnion({"AgeLimit": SimTypeInt(signed=True, label="Int32"), "FlushThreshold": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("NumberOfBuffers", SimTypeInt(signed=False, label="UInt32")), ("FreeBuffers", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("BuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("LogBuffersLost", SimTypeInt(signed=False, label="UInt32")), ("RealTimeBuffersLost", SimTypeInt(signed=False, label="UInt32")), ("LoggerThreadId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("LogFileNameOffset", SimTypeInt(signed=False, label="UInt32")), ("LoggerNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="EVENT_TRACE_PROPERTIES", pack=False, align=None),
    "EVENT_TRACE_PROPERTIES_V2": SimStruct(OrderedDict((("Wnode", SimTypeRef("WNODE_HEADER", SimStruct)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("MinimumBuffers", SimTypeInt(signed=False, label="UInt32")), ("MaximumBuffers", SimTypeInt(signed=False, label="UInt32")), ("MaximumFileSize", SimTypeInt(signed=False, label="UInt32")), ("LogFileMode", SimTypeInt(signed=False, label="UInt32")), ("FlushTimer", SimTypeInt(signed=False, label="UInt32")), ("EnableFlags", SimTypeInt(signed=False, label="EVENT_TRACE_FLAG")), ("Anonymous1", SimUnion({"AgeLimit": SimTypeInt(signed=True, label="Int32"), "FlushThreshold": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("NumberOfBuffers", SimTypeInt(signed=False, label="UInt32")), ("FreeBuffers", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("BuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("LogBuffersLost", SimTypeInt(signed=False, label="UInt32")), ("RealTimeBuffersLost", SimTypeInt(signed=False, label="UInt32")), ("LoggerThreadId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("LogFileNameOffset", SimTypeInt(signed=False, label="UInt32")), ("LoggerNameOffset", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "V2Control": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("FilterDescCount", SimTypeInt(signed=False, label="UInt32")), ("FilterDesc", SimTypePointer(SimTypeRef("EVENT_FILTER_DESCRIPTOR", SimStruct), offset=0)), ("Anonymous3", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "V2Options": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="EVENT_TRACE_PROPERTIES_V2", pack=False, align=None),
    "EVENT_TYPE": SimTypeInt(signed=False, label="EVENT_TYPE"),
    "EVRFilterConfigPrefs": SimTypeInt(signed=False, label="EVRFilterConfigPrefs"),
    "EVT_CHANNEL_CLOCK_TYPE": SimTypeInt(signed=False, label="EVT_CHANNEL_CLOCK_TYPE"),
    "EVT_CHANNEL_CONFIG_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_CHANNEL_CONFIG_PROPERTY_ID"),
    "EVT_CHANNEL_ISOLATION_TYPE": SimTypeInt(signed=False, label="EVT_CHANNEL_ISOLATION_TYPE"),
    "EVT_CHANNEL_REFERENCE_FLAGS": SimTypeInt(signed=False, label="EVT_CHANNEL_REFERENCE_FLAGS"),
    "EVT_CHANNEL_SID_TYPE": SimTypeInt(signed=False, label="EVT_CHANNEL_SID_TYPE"),
    "EVT_CHANNEL_TYPE": SimTypeInt(signed=False, label="EVT_CHANNEL_TYPE"),
    "EVT_EVENT_METADATA_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_EVENT_METADATA_PROPERTY_ID"),
    "EVT_EVENT_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_EVENT_PROPERTY_ID"),
    "EVT_EXPORTLOG_FLAGS": SimTypeInt(signed=False, label="EVT_EXPORTLOG_FLAGS"),
    "EVT_FORMAT_MESSAGE_FLAGS": SimTypeInt(signed=False, label="EVT_FORMAT_MESSAGE_FLAGS"),
    "EVT_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "EVT_LOGIN_CLASS": SimTypeInt(signed=False, label="EVT_LOGIN_CLASS"),
    "EVT_LOG_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_LOG_PROPERTY_ID"),
    "EVT_OPEN_LOG_FLAGS": SimTypeInt(signed=False, label="EVT_OPEN_LOG_FLAGS"),
    "EVT_PUBLISHER_METADATA_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_PUBLISHER_METADATA_PROPERTY_ID"),
    "EVT_QUERY_FLAGS": SimTypeInt(signed=False, label="EVT_QUERY_FLAGS"),
    "EVT_QUERY_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_QUERY_PROPERTY_ID"),
    "EVT_RENDER_CONTEXT_FLAGS": SimTypeInt(signed=False, label="EVT_RENDER_CONTEXT_FLAGS"),
    "EVT_RENDER_FLAGS": SimTypeInt(signed=False, label="EVT_RENDER_FLAGS"),
    "EVT_RPC_LOGIN": SimStruct(OrderedDict((("Server", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("User", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="EVT_RPC_LOGIN", pack=False, align=None),
    "EVT_RPC_LOGIN_FLAGS": SimTypeInt(signed=False, label="EVT_RPC_LOGIN_FLAGS"),
    "EVT_SEEK_FLAGS": SimTypeInt(signed=False, label="EVT_SEEK_FLAGS"),
    "EVT_SUBSCRIBE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="EVT_SUBSCRIBE_NOTIFY_ACTION"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Action", "UserContext", "Event"]), offset=0),
    "EVT_SUBSCRIBE_FLAGS": SimTypeInt(signed=False, label="EVT_SUBSCRIBE_FLAGS"),
    "EVT_SUBSCRIBE_NOTIFY_ACTION": SimTypeInt(signed=False, label="EVT_SUBSCRIBE_NOTIFY_ACTION"),
    "EVT_SYSTEM_PROPERTY_ID": SimTypeInt(signed=False, label="EVT_SYSTEM_PROPERTY_ID"),
    "EVT_VARIANT": SimStruct(OrderedDict((("Anonymous", SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "SByteVal": SimTypeChar(label="SByte"), "Int16Val": SimTypeShort(signed=True, label="Int16"), "Int32Val": SimTypeInt(signed=True, label="Int32"), "Int64Val": SimTypeLongLong(signed=True, label="Int64"), "ByteVal": SimTypeChar(label="Byte"), "UInt16Val": SimTypeShort(signed=False, label="UInt16"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "UInt64Val": SimTypeLongLong(signed=False, label="UInt64"), "SingleVal": SimTypeFloat(size=32), "DoubleVal": SimTypeFloat(size=64), "FileTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "SysTimeVal": SimTypePointer(SimTypeRef("SYSTEMTIME", SimStruct), offset=0), "GuidVal": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "AnsiStringVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "SidVal": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "SizeTVal": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "SByteArr": SimTypePointer(SimTypeChar(label="SByte"), offset=0), "Int16Arr": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int64Arr": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "ByteArr": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "UInt16Arr": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "UInt32Arr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "UInt64Arr": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "SingleArr": SimTypePointer(SimTypeFloat(size=32), offset=0), "DoubleArr": SimTypePointer(SimTypeFloat(size=64), offset=0), "FileTimeArr": SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0), "SysTimeArr": SimTypePointer(SimTypeRef("SYSTEMTIME", SimStruct), offset=0), "GuidArr": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "AnsiStringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), "SidArr": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), "SizeTArr": SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), "EvtHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "XmlVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "XmlValArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="<anon>", label="None")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="EVT_VARIANT", pack=False, align=None),
    "EVT_VARIANT_TYPE": SimTypeInt(signed=False, label="EVT_VARIANT_TYPE"),
    "EVT_VHF_ASYNC_OPERATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("HID_XFER_PACKET", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["VhfClientContext", "VhfOperationHandle", "VhfOperationContext", "HidTransferPacket"]), offset=0),
    "EVT_VHF_CLEANUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["VhfClientContext"]), offset=0),
    "EVT_VHF_READY_FOR_NEXT_READ_REPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["VhfClientContext"]), offset=0),
    "EV_EXTRA_CERT_CHAIN_POLICY_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwRootProgramQualifierFlags", SimTypeInt(signed=False, label="CERT_ROOT_PROGRAM_FLAGS")),)), name="EV_EXTRA_CERT_CHAIN_POLICY_PARA", pack=False, align=None),
    "EV_EXTRA_CERT_CHAIN_POLICY_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwQualifiers", SimTypeInt(signed=False, label="UInt32")), ("dwIssuanceUsageIndex", SimTypeInt(signed=False, label="UInt32")),)), name="EV_EXTRA_CERT_CHAIN_POLICY_STATUS", pack=False, align=None),
    "EXBMINFOHEADER": SimStruct(OrderedDict((("bmi", SimTypeRef("BITMAPINFOHEADER", SimStruct)), ("biExtDataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="EXBMINFOHEADER", pack=False, align=None),
    "EXCEPINFO": SimStruct(OrderedDict((("wCode", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("bstrSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrHelpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHelpContext", SimTypeInt(signed=False, label="UInt32")), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfnDeferredFillIn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pExcepInfo"]), offset=0)), ("scode", SimTypeInt(signed=True, label="Int32")),)), name="EXCEPINFO", pack=False, align=None),
    "EXCEPTION_DEBUG_INFO": SimStruct(OrderedDict((("ExceptionRecord", SimTypeRef("EXCEPTION_RECORD", SimStruct)), ("dwFirstChance", SimTypeInt(signed=False, label="UInt32")),)), name="EXCEPTION_DEBUG_INFO", pack=False, align=None),
    "EXCEPTION_DISPOSITION": SimTypeInt(signed=False, label="EXCEPTION_DISPOSITION"),
    "EXCEPTION_POINTERS": SimStruct(OrderedDict((("ExceptionRecord", SimTypePointer(SimTypeRef("EXCEPTION_RECORD", SimStruct), offset=0)), ("ContextRecord", SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0)),)), name="EXCEPTION_POINTERS", pack=False, align=None),
    "EXCEPTION_RECORD": SimStruct(OrderedDict((("ExceptionCode", SimTypeInt(signed=True, label="Int32")), ("ExceptionFlags", SimTypeInt(signed=False, label="UInt32")), ("ExceptionRecord", SimTypePointer(SimTypeRef("EXCEPTION_RECORD", SimStruct), offset=0)), ("ExceptionAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberParameters", SimTypeInt(signed=False, label="UInt32")), ("ExceptionInformation", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 15)),)), name="EXCEPTION_RECORD", pack=False, align=None),
    "EXCEPTION_RECORD32": SimStruct(OrderedDict((("ExceptionCode", SimTypeInt(signed=True, label="Int32")), ("ExceptionFlags", SimTypeInt(signed=False, label="UInt32")), ("ExceptionRecord", SimTypeInt(signed=False, label="UInt32")), ("ExceptionAddress", SimTypeInt(signed=False, label="UInt32")), ("NumberParameters", SimTypeInt(signed=False, label="UInt32")), ("ExceptionInformation", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 15)),)), name="EXCEPTION_RECORD32", pack=False, align=None),
    "EXCEPTION_RECORD64": SimStruct(OrderedDict((("ExceptionCode", SimTypeInt(signed=True, label="Int32")), ("ExceptionFlags", SimTypeInt(signed=False, label="UInt32")), ("ExceptionRecord", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionAddress", SimTypeLongLong(signed=False, label="UInt64")), ("NumberParameters", SimTypeInt(signed=False, label="UInt32")), ("__unusedAlignment", SimTypeInt(signed=False, label="UInt32")), ("ExceptionInformation", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 15)),)), name="EXCEPTION_RECORD64", pack=False, align=None),
    "EXCEPTION_REGISTRATION_RECORD": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("EXCEPTION_REGISTRATION_RECORD", SimStruct), offset=0)), ("Handler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_RECORD", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="EXCEPTION_DISPOSITION"), arg_names=["ExceptionRecord", "EstablisherFrame", "ContextRecord", "DispatcherContext"]), offset=0)),)), name="EXCEPTION_REGISTRATION_RECORD", pack=False, align=None),
    "EXCEPTION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_RECORD", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="EXCEPTION_DISPOSITION"), arg_names=["ExceptionRecord", "EstablisherFrame", "ContextRecord", "DispatcherContext"]), offset=0),
    "EXECUTION_STATE": SimTypeInt(signed=False, label="EXECUTION_STATE"),
    "EXFAT_STATISTICS": SimStruct(OrderedDict((("CreateHits", SimTypeInt(signed=False, label="UInt32")), ("SuccessfulCreates", SimTypeInt(signed=False, label="UInt32")), ("FailedCreates", SimTypeInt(signed=False, label="UInt32")), ("NonCachedReads", SimTypeInt(signed=False, label="UInt32")), ("NonCachedReadBytes", SimTypeInt(signed=False, label="UInt32")), ("NonCachedWrites", SimTypeInt(signed=False, label="UInt32")), ("NonCachedWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("NonCachedDiskReads", SimTypeInt(signed=False, label="UInt32")), ("NonCachedDiskWrites", SimTypeInt(signed=False, label="UInt32")),)), name="EXFAT_STATISTICS", pack=False, align=None),
    "EXIT_PROCESS_DEBUG_INFO": SimStruct(OrderedDict((("dwExitCode", SimTypeInt(signed=False, label="UInt32")),)), name="EXIT_PROCESS_DEBUG_INFO", pack=False, align=None),
    "EXIT_THREAD_DEBUG_INFO": SimStruct(OrderedDict((("dwExitCode", SimTypeInt(signed=False, label="UInt32")),)), name="EXIT_THREAD_DEBUG_INFO", pack=False, align=None),
    "EXIT_WINDOWS_FLAGS": SimTypeInt(signed=False, label="EXIT_WINDOWS_FLAGS"),
    "EXPANDBUTTONSTATES": SimTypeInt(signed=False, label="EXPANDBUTTONSTATES"),
    "EXPANDOBUTTONSTATES": SimTypeInt(signed=False, label="EXPANDOBUTTONSTATES"),
    "EXPAND_STACK_CALLOUT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Parameter"]), offset=0),
    "EXPAND_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="EXPAND_VIRTUAL_DISK_FLAG"),
    "EXPAND_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="EXPAND_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="EXPAND_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "EXPAND_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="EXPAND_VIRTUAL_DISK_VERSION"),
    "EXPLICIT_ACCESS_A": SimStruct(OrderedDict((("grfAccessPermissions", SimTypeInt(signed=False, label="UInt32")), ("grfAccessMode", SimTypeInt(signed=False, label="ACCESS_MODE")), ("grfInheritance", SimTypeInt(signed=False, label="ACE_FLAGS")), ("Trustee", SimTypeRef("TRUSTEE_A", SimStruct)),)), name="EXPLICIT_ACCESS_A", pack=False, align=None),
    "EXPLICIT_ACCESS_W": SimStruct(OrderedDict((("grfAccessPermissions", SimTypeInt(signed=False, label="UInt32")), ("grfAccessMode", SimTypeInt(signed=False, label="ACCESS_MODE")), ("grfInheritance", SimTypeInt(signed=False, label="ACE_FLAGS")), ("Trustee", SimTypeRef("TRUSTEE_W", SimStruct)),)), name="EXPLICIT_ACCESS_W", pack=False, align=None),
    "EXPLORERBARPARTS": SimTypeInt(signed=False, label="EXPLORERBARPARTS"),
    "EXPLORER_BROWSER_FILL_FLAGS": SimTypeInt(signed=False, label="EXPLORER_BROWSER_FILL_FLAGS"),
    "EXPLORER_BROWSER_OPTIONS": SimTypeInt(signed=False, label="EXPLORER_BROWSER_OPTIONS"),
    "EXPORT_SECURITY_CONTEXT_FLAGS": SimTypeInt(signed=False, label="EXPORT_SECURITY_CONTEXT_FLAGS"),
    "EXPORT_SECURITY_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "EXPR_EVAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "EXPR_TOKEN": SimTypeInt(signed=False, label="EXPR_TOKEN"),
    "EXP_DARWIN_LINK": SimStruct(OrderedDict((("dbh", SimTypeRef("DATABLOCK_HEADER", SimStruct)), ("szDarwinID", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szwDarwinID", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="EXP_DARWIN_LINK", pack=False, align=None),
    "EXP_PROPERTYSTORAGE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("abPropertyStorage", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EXP_PROPERTYSTORAGE", pack=False, align=None),
    "EXP_SPECIAL_FOLDER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("idSpecialFolder", SimTypeInt(signed=False, label="UInt32")), ("cbOffset", SimTypeInt(signed=False, label="UInt32")),)), name="EXP_SPECIAL_FOLDER", pack=False, align=None),
    "EXP_SZ_LINK": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("szTarget", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("swzTarget", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="EXP_SZ_LINK", pack=False, align=None),
    "EXTCHKBOX": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("pTitle", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pSeparator", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pCheckedName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("IconID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wReserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 4)), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)),)), name="EXTCHKBOX", pack=False, align=None),
    "EXTCONN": SimTypeInt(signed=False, label="EXTCONN"),
    "EXTDLL_ITERATERTLBALANCEDNODES": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EntryAddress", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["RootNode", "EntryOffset", "Callback", "CallbackContext"]), offset=0),
    "EXTDLL_QUERYDATABYTAG": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "dwDataTag", "pQueryInfo", "pData", "cbData"]), offset=0),
    "EXTDLL_QUERYDATABYTAGEX": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "dwDataTag", "pQueryInfo", "pData", "cbData", "pDataEx", "cbDataEx"]), offset=0),
    "EXTENDED_AGP_REGISTER": SimTypeInt(signed=False, label="EXTENDED_AGP_REGISTER"),
    "EXTENDED_CREATE_INFORMATION": SimStruct(OrderedDict((("ExtendedCreateFlags", SimTypeLongLong(signed=True, label="Int64")), ("EaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EaLength", SimTypeInt(signed=False, label="UInt32")),)), name="EXTENDED_CREATE_INFORMATION", pack=False, align=None),
    "EXTENDED_CREATE_INFORMATION_32": SimStruct(OrderedDict((("ExtendedCreateFlags", SimTypeLongLong(signed=True, label="Int64")), ("EaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EaLength", SimTypeInt(signed=False, label="UInt32")),)), name="EXTENDED_CREATE_INFORMATION_32", pack=False, align=None),
    "EXTENDED_ENCRYPTED_DATA_INFO": SimStruct(OrderedDict((("ExtendedCode", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="EXTENDED_ENCRYPTED_DATA_INFO", pack=False, align=None),
    "EXTENDED_NAME_FORMAT": SimTypeInt(signed=False, label="EXTENDED_NAME_FORMAT"),
    "EXTENDED_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("pbEventParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EXTENDED_NOTIFICATION", pack=False, align=None),
    "EXTENSION_CONTROL_BLOCK": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ConnID", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwHttpStatusCode", SimTypeInt(signed=False, label="UInt32")), ("lpszLogData", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("lpszMethod", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszQueryString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszPathInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszPathTranslated", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbTotalBytes", SimTypeInt(signed=False, label="UInt32")), ("cbAvailable", SimTypeInt(signed=False, label="UInt32")), ("lpbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszContentType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("GetServerVariable", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("WriteClient", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("ReadClient", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("ServerSupportFunction", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0)),)), name="EXTENSION_CONTROL_BLOCK", pack=False, align=None),
    "EXTENT_READ_CACHE_INFO_BUFFER": SimStruct(OrderedDict((("AllocatedCache", SimTypeLongLong(signed=True, label="Int64")), ("PopulatedCache", SimTypeLongLong(signed=True, label="Int64")), ("InErrorCache", SimTypeLongLong(signed=True, label="Int64")),)), name="EXTENT_READ_CACHE_INFO_BUFFER", pack=False, align=None),
    "EXTLOGFONTA": SimStruct(OrderedDict((("elfLogFont", SimTypeRef("LOGFONTA", SimStruct)), ("elfFullName", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("elfStyle", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("elfVersion", SimTypeInt(signed=False, label="UInt32")), ("elfStyleSize", SimTypeInt(signed=False, label="UInt32")), ("elfMatch", SimTypeInt(signed=False, label="UInt32")), ("elfReserved", SimTypeInt(signed=False, label="UInt32")), ("elfVendorId", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("elfCulture", SimTypeInt(signed=False, label="UInt32")), ("elfPanose", SimTypeRef("PANOSE", SimStruct)),)), name="EXTLOGFONTA", pack=False, align=None),
    "EXTLOGFONTW": SimStruct(OrderedDict((("elfLogFont", SimTypeRef("LOGFONTW", SimStruct)), ("elfFullName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("elfStyle", SimTypeArray(SimTypeChar(label="Char"), 32)), ("elfVersion", SimTypeInt(signed=False, label="UInt32")), ("elfStyleSize", SimTypeInt(signed=False, label="UInt32")), ("elfMatch", SimTypeInt(signed=False, label="UInt32")), ("elfReserved", SimTypeInt(signed=False, label="UInt32")), ("elfVendorId", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("elfCulture", SimTypeInt(signed=False, label="UInt32")), ("elfPanose", SimTypeRef("PANOSE", SimStruct)),)), name="EXTLOGFONTW", pack=False, align=None),
    "EXTLOGPEN": SimStruct(OrderedDict((("elpPenStyle", SimTypeInt(signed=False, label="UInt32")), ("elpWidth", SimTypeInt(signed=False, label="UInt32")), ("elpBrushStyle", SimTypeInt(signed=False, label="UInt32")), ("elpColor", SimTypeInt(signed=False, label="UInt32")), ("elpHatch", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("elpNumEntries", SimTypeInt(signed=False, label="UInt32")), ("elpStyleEntry", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="EXTLOGPEN", pack=False, align=None),
    "EXTLOGPEN32": SimStruct(OrderedDict((("elpPenStyle", SimTypeInt(signed=False, label="UInt32")), ("elpWidth", SimTypeInt(signed=False, label="UInt32")), ("elpBrushStyle", SimTypeInt(signed=False, label="UInt32")), ("elpColor", SimTypeInt(signed=False, label="UInt32")), ("elpHatch", SimTypeInt(signed=False, label="UInt32")), ("elpNumEntries", SimTypeInt(signed=False, label="UInt32")), ("elpStyleEntry", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="EXTLOGPEN32", pack=False, align=None),
    "EXTPUSH": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("pTitle", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Anonymous1", SimUnion({"DlgProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), "pfnCallBack": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0)}, name="<anon>", label="None")), ("IconID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous2", SimUnion({"DlgTemplateID": SimTypeShort(signed=False, label="UInt16"), "hDlgTemplate": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)),)), name="EXTPUSH", pack=False, align=None),
    "EXTRASEARCH": SimStruct(OrderedDict((("guidSearch", SimTypeBottom(label="Guid")), ("wszFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 80)), ("wszUrl", SimTypeArray(SimTypeChar(label="Char"), 2084)),)), name="EXTRASEARCH", pack=False, align=None),
    "EXTSTACKTRACE": SimStruct(OrderedDict((("FramePointer", SimTypeInt(signed=False, label="UInt32")), ("ProgramCounter", SimTypeInt(signed=False, label="UInt32")), ("ReturnAddress", SimTypeInt(signed=False, label="UInt32")), ("Args", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="EXTSTACKTRACE", pack=False, align=None),
    "EXTSTACKTRACE32": SimStruct(OrderedDict((("FramePointer", SimTypeInt(signed=False, label="UInt32")), ("ProgramCounter", SimTypeInt(signed=False, label="UInt32")), ("ReturnAddress", SimTypeInt(signed=False, label="UInt32")), ("Args", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="EXTSTACKTRACE32", pack=False, align=None),
    "EXTSTACKTRACE64": SimStruct(OrderedDict((("FramePointer", SimTypeLongLong(signed=False, label="UInt64")), ("ProgramCounter", SimTypeLongLong(signed=False, label="UInt64")), ("ReturnAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Args", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)),)), name="EXTSTACKTRACE64", pack=False, align=None),
    "EXTS_JOB_PROCESS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Job", "Process", "Context"]), offset=0),
    "EXTS_TABLE_ENTRY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Entry", "Context"]), offset=0),
    "EXTTEXTMETRIC": SimStruct(OrderedDict((("emSize", SimTypeShort(signed=True, label="Int16")), ("emPointSize", SimTypeShort(signed=True, label="Int16")), ("emOrientation", SimTypeShort(signed=True, label="Int16")), ("emMasterHeight", SimTypeShort(signed=True, label="Int16")), ("emMinScale", SimTypeShort(signed=True, label="Int16")), ("emMaxScale", SimTypeShort(signed=True, label="Int16")), ("emMasterUnits", SimTypeShort(signed=True, label="Int16")), ("emCapHeight", SimTypeShort(signed=True, label="Int16")), ("emXHeight", SimTypeShort(signed=True, label="Int16")), ("emLowerCaseAscent", SimTypeShort(signed=True, label="Int16")), ("emLowerCaseDescent", SimTypeShort(signed=True, label="Int16")), ("emSlant", SimTypeShort(signed=True, label="Int16")), ("emSuperScript", SimTypeShort(signed=True, label="Int16")), ("emSubScript", SimTypeShort(signed=True, label="Int16")), ("emSuperScriptSize", SimTypeShort(signed=True, label="Int16")), ("emSubScriptSize", SimTypeShort(signed=True, label="Int16")), ("emUnderlineOffset", SimTypeShort(signed=True, label="Int16")), ("emUnderlineWidth", SimTypeShort(signed=True, label="Int16")), ("emDoubleUpperUnderlineOffset", SimTypeShort(signed=True, label="Int16")), ("emDoubleLowerUnderlineOffset", SimTypeShort(signed=True, label="Int16")), ("emDoubleUpperUnderlineWidth", SimTypeShort(signed=True, label="Int16")), ("emDoubleLowerUnderlineWidth", SimTypeShort(signed=True, label="Int16")), ("emStrikeOutOffset", SimTypeShort(signed=True, label="Int16")), ("emStrikeOutWidth", SimTypeShort(signed=True, label="Int16")), ("emKernPairs", SimTypeShort(signed=False, label="UInt16")), ("emKernTracks", SimTypeShort(signed=False, label="UInt16")),)), name="EXTTEXTMETRIC", pack=False, align=None),
    "EXT_ANALYSIS_PLUGIN": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypeInt(signed=False, label="FA_EXTENSION_PLUGIN_PHASE"), SimTypeBottom(label="IDebugFailureAnalysis2")], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "CallPhase", "pAnalysis"]), offset=0),
    "EXT_ANALYZER": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeBottom(label="IDebugFailureAnalysis")], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "BucketSuffix", "cbBucketSuffix", "DebugText", "cbDebugText", "Flags", "pAnalysis"]), offset=0),
    "EXT_API_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="EXT_API_VERSION", pack=False, align=None),
    "EXT_CAB_XML_DATA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("XmlObjectTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NumSubTags", SimTypeInt(signed=False, label="UInt32")), ("SubTags", SimTypePointer(SimTypeRef("_SUBTAGS", SimStruct), offset=0)),)), name="EXT_CAB_XML_DATA", pack=False, align=None),
    "EXT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Timer", "Context"]), offset=0),
    "EXT_DECODE_ERROR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_DECODE_ERROR", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pDecodeError"]), offset=0),
    "EXT_DELETE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "EXT_DELETE_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DeleteCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DeleteContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="EXT_DELETE_PARAMETERS", pack=False, align=None),
    "EXT_FIND_FILE": SimStruct(OrderedDict((("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IndexedSize", SimTypeLongLong(signed=False, label="UInt64")), ("ImageTimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ImageCheckSum", SimTypeInt(signed=False, label="UInt32")), ("ExtraInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ExtraInfoSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileMapping", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileMappingSize", SimTypeLongLong(signed=False, label="UInt64")), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FoundFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FoundFileNameChars", SimTypeInt(signed=False, label="UInt32")),)), name="EXT_FIND_FILE", pack=False, align=None),
    "EXT_FLOOD_FILL_TYPE": SimTypeInt(signed=False, label="EXT_FLOOD_FILL_TYPE"),
    "EXT_GET_DEBUG_FAILURE_ANALYSIS": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="Guid"), SimTypePointer(SimTypeBottom(label="IDebugFailureAnalysis2"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Flags", "ClassId", "ppAnalysis"]), offset=0),
    "EXT_GET_ENVIRONMENT_VARIABLE": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Peb", "Variable", "Buffer", "BufferSize"]), offset=0),
    "EXT_GET_FAILURE_ANALYSIS": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="IDebugFailureAnalysis"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Flags", "ppAnalysis"]), offset=0),
    "EXT_GET_FA_ENTRIES_DATA": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FA_ENTRY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Count", "Entries"]), offset=0),
    "EXT_GET_HANDLE_TRACE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "TraceType", "StartIndex", "HandleValue", "StackFunctions", "StackTraceSize"]), offset=0),
    "EXT_MATCH_PATTERN_A": SimStruct(OrderedDict((("Str", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Pattern", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CaseSensitive", SimTypeInt(signed=False, label="UInt32")),)), name="EXT_MATCH_PATTERN_A", pack=False, align=None),
    "EXT_RELOAD_TRIAGER": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4")], SimTypeInt(signed=True, label="Int32"), arg_names=["Client"]), offset=0),
    "EXT_TARGET_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypePointer(SimTypeRef("TARGET_DEBUG_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pTargetInfo"]), offset=0),
    "EXT_TDOP": SimTypeInt(signed=False, label="EXT_TDOP"),
    "EXT_TRIAGE_FOLLOWUP": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("DEBUG_TRIAGE_FOLLOWUP_INFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Client", "SymbolName", "OwnerInfo"]), offset=0),
    "EXT_TYPED_DATA": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="EXT_TDOP")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("InData", SimTypeRef("DEBUG_TYPED_DATA", SimStruct)), ("OutData", SimTypeRef("DEBUG_TYPED_DATA", SimStruct)), ("InStrIndex", SimTypeInt(signed=False, label="UInt32")), ("In32", SimTypeInt(signed=False, label="UInt32")), ("Out32", SimTypeInt(signed=False, label="UInt32")), ("In64", SimTypeLongLong(signed=False, label="UInt64")), ("Out64", SimTypeLongLong(signed=False, label="UInt64")), ("StrBufferIndex", SimTypeInt(signed=False, label="UInt32")), ("StrBufferChars", SimTypeInt(signed=False, label="UInt32")), ("StrCharsNeeded", SimTypeInt(signed=False, label="UInt32")), ("DataBufferIndex", SimTypeInt(signed=False, label="UInt32")), ("DataBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("DataBytesNeeded", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="EXT_TYPED_DATA", pack=False, align=None),
    "EXT_XML_DATA": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient4"), SimTypePointer(SimTypeRef("EXT_CAB_XML_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pXmpData"]), offset=0),
    "EX_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackContext", "Argument1", "Argument2"]), offset=0),
    "EX_POOL_PRIORITY": SimTypeInt(signed=False, label="EX_POOL_PRIORITY"),
    "EX_PROP_INFO_FLAGS": SimTypeInt(signed=False, label="EX_PROP_INFO_FLAGS"),
    "EX_RUNDOWN_REF": SimStruct(OrderedDict((("Anonymous", SimUnion({"Count": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Ptr": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="EX_RUNDOWN_REF", pack=False, align=None),
    "EXpsCompressionOptions": SimTypeInt(signed=False, label="EXpsCompressionOptions"),
    "EXpsFontOptions": SimTypeInt(signed=False, label="EXpsFontOptions"),
    "EXpsFontRestriction": SimTypeInt(signed=False, label="EXpsFontRestriction"),
    "EXpsJobConsumption": SimTypeInt(signed=False, label="EXpsJobConsumption"),
    "EapCertificateCredential": SimStruct(OrderedDict((("certHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("password", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="EapCertificateCredential", pack=False, align=None),
    "EapCode": SimTypeInt(signed=False, label="EapCode"),
    "EapCredential": SimStruct(OrderedDict((("credType", SimTypeInt(signed=False, label="EapCredentialType")), ("credData", SimUnion({"username_password": SimTypeRef("EapUsernamePasswordCredential", SimStruct), "certificate": SimTypeRef("EapCertificateCredential", SimStruct), "sim": SimTypeRef("EapSimCredential", SimStruct)}, name="<anon>", label="None")),)), name="EapCredential", pack=False, align=None),
    "EapCredentialType": SimTypeInt(signed=False, label="EapCredentialType"),
    "EapCredentialTypeData": SimUnion({"username_password": SimTypeRef("EapUsernamePasswordCredential", SimStruct), "certificate": SimTypeRef("EapCertificateCredential", SimStruct), "sim": SimTypeRef("EapSimCredential", SimStruct)}, name="<anon>", label="None"),
    "EapHostPeerAuthParams": SimTypeInt(signed=False, label="EapHostPeerAuthParams"),
    "EapHostPeerMethodResult": SimStruct(OrderedDict((("fIsSuccess", SimTypeInt(signed=True, label="Int32")), ("dwFailureReasonCode", SimTypeInt(signed=False, label="UInt32")), ("fSaveConnectionData", SimTypeInt(signed=True, label="Int32")), ("dwSizeofConnectionData", SimTypeInt(signed=False, label="UInt32")), ("pConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fSaveUserData", SimTypeInt(signed=True, label="Int32")), ("dwSizeofUserData", SimTypeInt(signed=False, label="UInt32")), ("pUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pAttribArray", SimTypePointer(SimTypeRef("EAP_ATTRIBUTES", SimStruct), offset=0)), ("isolationState", SimTypeInt(signed=False, label="ISOLATION_STATE")), ("pEapMethodInfo", SimTypePointer(SimTypeRef("EAP_METHOD_INFO", SimStruct), offset=0)), ("pEapError", SimTypePointer(SimTypeRef("EAP_ERROR", SimStruct), offset=0)),)), name="EapHostPeerMethodResult", pack=False, align=None),
    "EapHostPeerMethodResultReason": SimTypeInt(signed=False, label="EapHostPeerMethodResultReason"),
    "EapHostPeerResponseAction": SimTypeInt(signed=False, label="EapHostPeerResponseAction"),
    "EapPacket": SimStruct(OrderedDict((("Code", SimTypeChar(label="Byte")), ("Id", SimTypeChar(label="Byte")), ("Length", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="EapPacket", pack=False, align=None),
    "EapPeerMethodOutput": SimStruct(OrderedDict((("action", SimTypeInt(signed=False, label="EapPeerMethodResponseAction")), ("fAllowNotifications", SimTypeInt(signed=True, label="Int32")),)), name="EapPeerMethodOutput", pack=False, align=None),
    "EapPeerMethodResponseAction": SimTypeInt(signed=False, label="EapPeerMethodResponseAction"),
    "EapPeerMethodResult": SimStruct(OrderedDict((("fIsSuccess", SimTypeInt(signed=True, label="Int32")), ("dwFailureReasonCode", SimTypeInt(signed=False, label="UInt32")), ("fSaveConnectionData", SimTypeInt(signed=True, label="Int32")), ("dwSizeofConnectionData", SimTypeInt(signed=False, label="UInt32")), ("pConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fSaveUserData", SimTypeInt(signed=True, label="Int32")), ("dwSizeofUserData", SimTypeInt(signed=False, label="UInt32")), ("pUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pAttribArray", SimTypePointer(SimTypeRef("EAP_ATTRIBUTES", SimStruct), offset=0)), ("pEapError", SimTypePointer(SimTypeRef("EAP_ERROR", SimStruct), offset=0)), ("pNgcKerbTicket", SimTypePointer(SimTypeRef("NgcTicketContext", SimStruct), offset=0)), ("fSaveToCredMan", SimTypeInt(signed=True, label="Int32")),)), name="EapPeerMethodResult", pack=False, align=None),
    "EapPeerMethodResultReason": SimTypeInt(signed=False, label="EapPeerMethodResultReason"),
    "EapSimCredential": SimStruct(OrderedDict((("iccID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="EapSimCredential", pack=False, align=None),
    "EapUsernamePasswordCredential": SimStruct(OrderedDict((("username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("password", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="EapUsernamePasswordCredential", pack=False, align=None),
    "Effect": SimStruct(OrderedDict((("lpVtbl", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("nativeEffect", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("auxDataSize", SimTypeInt(signed=True, label="Int32")), ("auxData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("useAuxData", SimTypeInt(signed=True, label="Int32")),)), name="Effect", pack=False, align=None),
    "EmfPlusRecordType": SimTypeInt(signed=False, label="EmfPlusRecordType"),
    "EmfToWmfBitsFlags": SimTypeInt(signed=False, label="EmfToWmfBitsFlags"),
    "EmfType": SimTypeInt(signed=False, label="EmfType"),
    "EmulationType": SimTypeInt(signed=False, label="EmulationType"),
    "EnTag_Mode": SimTypeInt(signed=False, label="EnTag_Mode"),
    "EnTvRat_CAE_TV": SimTypeInt(signed=False, label="EnTvRat_CAE_TV"),
    "EnTvRat_CAF_TV": SimTypeInt(signed=False, label="EnTvRat_CAF_TV"),
    "EnTvRat_GenericLevel": SimTypeInt(signed=False, label="EnTvRat_GenericLevel"),
    "EnTvRat_MPAA": SimTypeInt(signed=False, label="EnTvRat_MPAA"),
    "EnTvRat_System": SimTypeInt(signed=False, label="EnTvRat_System"),
    "EnTvRat_US_TV": SimTypeInt(signed=False, label="EnTvRat_US_TV"),
    "EncDecEvents": SimTypeInt(signed=False, label="EncDecEvents"),
    "EncoderParameter": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("NumberOfValues", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="EncoderParameter", pack=False, align=None),
    "EncoderParameterValueType": SimTypeInt(signed=False, label="EncoderParameterValueType"),
    "EncoderParameters": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Parameter", SimTypePointer(SimTypeRef("EncoderParameter", SimStruct), offset=0)),)), name="EncoderParameters", pack=False, align=None),
    "EncoderValue": SimTypeInt(signed=False, label="EncoderValue"),
    "EncodingType": SimTypeInt(signed=False, label="EncodingType"),
    "EndpointConnectorType": SimTypeInt(signed=False, label="EndpointConnectorType"),
    "EndpointFormFactor": SimTypeInt(signed=False, label="EndpointFormFactor"),
    "EnrollmentCAProperty": SimTypeInt(signed=False, label="EnrollmentCAProperty"),
    "EnrollmentDisplayStatus": SimTypeInt(signed=False, label="EnrollmentDisplayStatus"),
    "EnrollmentEnrollStatus": SimTypeInt(signed=False, label="EnrollmentEnrollStatus"),
    "EnrollmentPolicyFlags": SimTypeInt(signed=False, label="EnrollmentPolicyFlags"),
    "EnrollmentPolicyServerPropertyFlags": SimTypeInt(signed=False, label="EnrollmentPolicyServerPropertyFlags"),
    "EnrollmentSelectionStatus": SimTypeInt(signed=False, label="EnrollmentSelectionStatus"),
    "EnrollmentTemplateProperty": SimTypeInt(signed=False, label="EnrollmentTemplateProperty"),
    "EntitlementType": SimTypeInt(signed=False, label="EntitlementType"),
    "EnumType": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ValueInts", SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0)), ("ValueStrings", SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0)),)), name="EnumType", pack=False, align=None),
    "EnumerateMetafileProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="EmfPlusRecordType"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "ErrorClass": SimTypeInt(signed=False, label="ErrorClass"),
    "Error_Spec_IPv4": SimStruct(OrderedDict((("errs_errnode", SimTypeRef("IN_ADDR", SimStruct)), ("errs_flags", SimTypeChar(label="Byte")), ("errs_code", SimTypeChar(label="Byte")), ("errs_value", SimTypeShort(signed=False, label="UInt16")),)), name="Error_Spec_IPv4", pack=False, align=None),
    "EventArgsType": SimTypeInt(signed=False, label="EventArgsType"),
    "EventMask": SimTypeInt(signed=False, label="EventMask"),
    "EventPipeProviderCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("COR_PRF_FILTER_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["source_id", "is_enabled", "level", "match_any_keywords", "match_all_keywords", "filter_data", "callback_data"]), offset=0),
    "ExpandCollapseState": SimTypeInt(signed=False, label="ExpandCollapseState"),
    "ExtendedDebugPropertyInfo": SimStruct(OrderedDict((("dwValidFields", SimTypeInt(signed=False, label="UInt32")), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszFullName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAttrib", SimTypeInt(signed=False, label="UInt32")), ("pDebugProp", SimTypeBottom(label="IDebugProperty")), ("nDISPID", SimTypeInt(signed=False, label="UInt32")), ("nType", SimTypeInt(signed=False, label="UInt32")), ("varValue", SimTypeRef("VARIANT", SimStruct)), ("plbValue", SimTypeBottom(label="ILockBytes")), ("pDebugExtProp", SimTypeBottom(label="IDebugExtendedProperty")),)), name="ExtendedDebugPropertyInfo", pack=False, align=None),
    "ExtendedErrorParamTypes": SimTypeInt(signed=False, label="ExtendedErrorParamTypes"),
    "ExtendedHeapRestrictions": SimStruct(OrderedDict((("ddsCapsEx", SimTypeRef("DDSCAPSEX", SimStruct)), ("ddsCapsExAlt", SimTypeRef("DDSCAPSEX", SimStruct)),)), name="ExtendedHeapRestrictions", pack=False, align=None),
    "ExtendedIsolationState": SimTypeInt(signed=False, label="ExtendedIsolationState"),
    "ExtendedProperty": SimStruct(OrderedDict((("PropertyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PropertyValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ExtendedProperty", pack=False, align=None),
    "Extension": SimTypeBottom(label="Extension"),
    "ExtensionValidationContexts": SimTypeInt(signed=False, label="ExtensionValidationContexts"),
    "ExtensionValidationResults": SimTypeInt(signed=False, label="ExtensionValidationResults"),
    "Extensions": SimTypeBottom(label="Extensions"),
    "FACILITY_CODE": SimTypeInt(signed=False, label="FACILITY_CODE"),
    "FAILURE_TYPE": SimTypeInt(signed=False, label="FAILURE_TYPE"),
    "FALLBACK_INDEX": SimTypeInt(signed=False, label="FALLBACK_INDEX"),
    "FARPROC": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0),
    "FAST_IO_ACQUIRE_FILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["FileObject"]), offset=0),
    "FAST_IO_ACQUIRE_FOR_CCFLUSH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "DeviceObject"]), offset=0),
    "FAST_IO_ACQUIRE_FOR_MOD_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "EndingOffset", "ResourceToRelease", "DeviceObject"]), offset=0),
    "FAST_IO_CHECK_IF_POSSIBLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "Wait", "LockKey", "CheckForReadOperation", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_DETACH_DEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["SourceDevice", "TargetDevice"]), offset=0),
    "FAST_IO_DEVICE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "InputBuffer", "InputBufferLength", "OutputBuffer", "OutputBufferLength", "IoControlCode", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_DISPATCH": SimStruct(OrderedDict((("SizeOfFastIoDispatch", SimTypeInt(signed=False, label="UInt32")), ("FastIoCheckIfPossible", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "Wait", "LockKey", "CheckForReadOperation", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoRead", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "Wait", "LockKey", "Buffer", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoWrite", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "Wait", "LockKey", "Buffer", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoQueryBasicInfo", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("FILE_BASIC_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "Buffer", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoQueryStandardInfo", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("FILE_STANDARD_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "Buffer", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoLock", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "ProcessId", "Key", "FailImmediately", "ExclusiveLock", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoUnlockSingle", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "ProcessId", "Key", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoUnlockAll", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "ProcessId", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoUnlockAllByKey", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "ProcessId", "Key", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("FastIoDeviceControl", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "InputBuffer", "InputBufferLength", "OutputBuffer", "OutputBufferLength", "IoControlCode", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("AcquireFileForNtCreateSection", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["FileObject"]), offset=0), offset=0)), ("ReleaseFileForNtCreateSection", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["FileObject"]), offset=0), offset=0)), ("FastIoDetachDevice", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["SourceDevice", "TargetDevice"]), offset=0), offset=0)), ("FastIoQueryNetworkOpenInfo", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("FILE_NETWORK_OPEN_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "Buffer", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("AcquireForModWrite", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "EndingOffset", "ResourceToRelease", "DeviceObject"]), offset=0), offset=0)), ("MdlRead", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "MdlChain", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("MdlReadComplete", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "MdlChain", "DeviceObject"]), offset=0), offset=0)), ("PrepareMdlWrite", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "MdlChain", "IoStatus", "DeviceObject"]), offset=0), offset=0)), ("MdlWriteComplete", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "MdlChain", "DeviceObject"]), offset=0), offset=0)), ("FastIoReadCompressed", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("COMPRESSED_DATA_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "Buffer", "MdlChain", "IoStatus", "CompressedDataInfo", "CompressedDataInfoLength", "DeviceObject"]), offset=0), offset=0)), ("FastIoWriteCompressed", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("COMPRESSED_DATA_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "Buffer", "MdlChain", "IoStatus", "CompressedDataInfo", "CompressedDataInfoLength", "DeviceObject"]), offset=0), offset=0)), ("MdlReadCompleteCompressed", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "MdlChain", "DeviceObject"]), offset=0), offset=0)), ("MdlWriteCompleteCompressed", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "MdlChain", "DeviceObject"]), offset=0), offset=0)), ("FastIoQueryOpen", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILE_NETWORK_OPEN_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["Irp", "NetworkInformation", "DeviceObject"]), offset=0), offset=0)), ("ReleaseForModWrite", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "ResourceToRelease", "DeviceObject"]), offset=0), offset=0)), ("AcquireForCcFlush", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "DeviceObject"]), offset=0), offset=0)), ("ReleaseForCcFlush", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "DeviceObject"]), offset=0), offset=0)),)), name="FAST_IO_DISPATCH", pack=False, align=None),
    "FAST_IO_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "ProcessId", "Key", "FailImmediately", "ExclusiveLock", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_MDL_READ": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "MdlChain", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_MDL_READ_COMPLETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "MdlChain", "DeviceObject"]), offset=0),
    "FAST_IO_MDL_READ_COMPLETE_COMPRESSED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "MdlChain", "DeviceObject"]), offset=0),
    "FAST_IO_MDL_WRITE_COMPLETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "MdlChain", "DeviceObject"]), offset=0),
    "FAST_IO_MDL_WRITE_COMPLETE_COMPRESSED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "MdlChain", "DeviceObject"]), offset=0),
    "FAST_IO_POSSIBLE": SimTypeInt(signed=False, label="FAST_IO_POSSIBLE"),
    "FAST_IO_PREPARE_MDL_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "MdlChain", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_QUERY_BASIC_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("FILE_BASIC_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "Buffer", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_QUERY_NETWORK_OPEN_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("FILE_NETWORK_OPEN_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "Buffer", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_QUERY_OPEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILE_NETWORK_OPEN_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["Irp", "NetworkInformation", "DeviceObject"]), offset=0),
    "FAST_IO_QUERY_STANDARD_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("FILE_STANDARD_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "Wait", "Buffer", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_READ": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "Wait", "LockKey", "Buffer", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_READ_COMPRESSED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("COMPRESSED_DATA_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "Buffer", "MdlChain", "IoStatus", "CompressedDataInfo", "CompressedDataInfoLength", "DeviceObject"]), offset=0),
    "FAST_IO_RELEASE_FILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["FileObject"]), offset=0),
    "FAST_IO_RELEASE_FOR_CCFLUSH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "DeviceObject"]), offset=0),
    "FAST_IO_RELEASE_FOR_MOD_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileObject", "ResourceToRelease", "DeviceObject"]), offset=0),
    "FAST_IO_UNLOCK_ALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "ProcessId", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_UNLOCK_ALL_BY_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "ProcessId", "Key", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_UNLOCK_SINGLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "ProcessId", "Key", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "Wait", "LockKey", "Buffer", "IoStatus", "DeviceObject"]), offset=0),
    "FAST_IO_WRITE_COMPRESSED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeRef("COMPRESSED_DATA_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["FileObject", "FileOffset", "Length", "LockKey", "Buffer", "MdlChain", "IoStatus", "CompressedDataInfo", "CompressedDataInfoLength", "DeviceObject"]), offset=0),
    "FAST_MUTEX": SimStruct(OrderedDict((("Count", SimTypeInt(signed=True, label="Int32")), ("Owner", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Contention", SimTypeInt(signed=False, label="UInt32")), ("Event", SimTypeRef("KEVENT", SimStruct)), ("OldIrql", SimTypeInt(signed=False, label="UInt32")),)), name="FAST_MUTEX", pack=False, align=None),
    "FAT_STATISTICS": SimStruct(OrderedDict((("CreateHits", SimTypeInt(signed=False, label="UInt32")), ("SuccessfulCreates", SimTypeInt(signed=False, label="UInt32")), ("FailedCreates", SimTypeInt(signed=False, label="UInt32")), ("NonCachedReads", SimTypeInt(signed=False, label="UInt32")), ("NonCachedReadBytes", SimTypeInt(signed=False, label="UInt32")), ("NonCachedWrites", SimTypeInt(signed=False, label="UInt32")), ("NonCachedWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("NonCachedDiskReads", SimTypeInt(signed=False, label="UInt32")), ("NonCachedDiskWrites", SimTypeInt(signed=False, label="UInt32")),)), name="FAT_STATISTICS", pack=False, align=None),
    "FAULT_INFORMATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="FAULT_INFORMATION_ARCH")), ("IsStage1", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Arm64": SimTypeRef("FAULT_INFORMATION_ARM64", SimStruct), "X64": SimTypeRef("FAULT_INFORMATION_X64", SimStruct)}, name="<anon>", label="None")),)), name="FAULT_INFORMATION", pack=False, align=None),
    "FAULT_INFORMATION_ARCH": SimTypeInt(signed=False, label="FAULT_INFORMATION_ARCH"),
    "FAULT_INFORMATION_ARM64": SimStruct(OrderedDict((("DomainHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FaultAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PhysicalDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("InputMappingId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("FAULT_INFORMATION_ARM64_FLAGS", SimStruct)), ("Type", SimTypeInt(signed=False, label="FAULT_INFORMATION_ARM64_TYPE")), ("IommuBaseAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="FAULT_INFORMATION_ARM64", pack=False, align=None),
    "FAULT_INFORMATION_ARM64_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="FAULT_INFORMATION_ARM64_FLAGS", pack=False, align=None),
    "FAULT_INFORMATION_ARM64_TYPE": SimTypeInt(signed=False, label="FAULT_INFORMATION_ARM64_TYPE"),
    "FAULT_INFORMATION_X64": SimStruct(OrderedDict((("DomainHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FaultAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeRef("FAULT_INFORMATION_X64_FLAGS", SimStruct)), ("Type", SimTypeInt(signed=False, label="FAULT_INFORMATION_ARM64_TYPE")), ("IommuBaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PciSegment", SimTypeInt(signed=False, label="UInt32")),)), name="FAULT_INFORMATION_X64", pack=False, align=None),
    "FAULT_INFORMATION_X64_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="FAULT_INFORMATION_X64_FLAGS", pack=False, align=None),
    "FAXROUTE_ENABLE": SimTypeInt(signed=False, label="FAXROUTE_ENABLE"),
    "FAX_ACCESS_RIGHTS_ENUM": SimTypeInt(signed=False, label="FAX_ACCESS_RIGHTS_ENUM"),
    "FAX_ACCESS_RIGHTS_ENUM_2": SimTypeInt(signed=False, label="FAX_ACCESS_RIGHTS_ENUM_2"),
    "FAX_ACCOUNT_EVENTS_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_ACCOUNT_EVENTS_TYPE_ENUM"),
    "FAX_CONFIGURATIONA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Retries", SimTypeInt(signed=False, label="UInt32")), ("RetryDelay", SimTypeInt(signed=False, label="UInt32")), ("DirtyDays", SimTypeInt(signed=False, label="UInt32")), ("Branding", SimTypeInt(signed=True, label="Int32")), ("UseDeviceTsid", SimTypeInt(signed=True, label="Int32")), ("ServerCp", SimTypeInt(signed=True, label="Int32")), ("PauseServerQueue", SimTypeInt(signed=True, label="Int32")), ("StartCheapTime", SimTypeRef("FAX_TIME", SimStruct)), ("StopCheapTime", SimTypeRef("FAX_TIME", SimStruct)), ("ArchiveOutgoingFaxes", SimTypeInt(signed=True, label="Int32")), ("ArchiveDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Reserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_CONFIGURATIONA", pack=False, align=None),
    "FAX_CONFIGURATIONW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Retries", SimTypeInt(signed=False, label="UInt32")), ("RetryDelay", SimTypeInt(signed=False, label="UInt32")), ("DirtyDays", SimTypeInt(signed=False, label="UInt32")), ("Branding", SimTypeInt(signed=True, label="Int32")), ("UseDeviceTsid", SimTypeInt(signed=True, label="Int32")), ("ServerCp", SimTypeInt(signed=True, label="Int32")), ("PauseServerQueue", SimTypeInt(signed=True, label="Int32")), ("StartCheapTime", SimTypeRef("FAX_TIME", SimStruct)), ("StopCheapTime", SimTypeRef("FAX_TIME", SimStruct)), ("ArchiveOutgoingFaxes", SimTypeInt(signed=True, label="Int32")), ("ArchiveDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Reserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_CONFIGURATIONW", pack=False, align=None),
    "FAX_CONTEXT_INFOA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ServerName", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="FAX_CONTEXT_INFOA", pack=False, align=None),
    "FAX_CONTEXT_INFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ServerName", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="FAX_CONTEXT_INFOW", pack=False, align=None),
    "FAX_COVERPAGE_INFOA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("CoverPageName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UseServerCoverPage", SimTypeInt(signed=True, label="Int32")), ("RecName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecFaxNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecCompany", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecStreetAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecCity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecState", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecZip", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecCountry", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecDepartment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecOfficeLocation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecHomePhone", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecOfficePhone", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrFaxNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrCompany", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrDepartment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrOfficeLocation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrHomePhone", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SdrOfficePhone", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Note", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Subject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TimeSent", SimTypeRef("SYSTEMTIME", SimStruct)), ("PageCount", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_COVERPAGE_INFOA", pack=False, align=None),
    "FAX_COVERPAGE_INFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("CoverPageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UseServerCoverPage", SimTypeInt(signed=True, label="Int32")), ("RecName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecFaxNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecStreetAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecCity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecState", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecZip", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecCountry", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecDepartment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecOfficeLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecHomePhone", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecOfficePhone", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrFaxNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrDepartment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrOfficeLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrHomePhone", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SdrOfficePhone", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Note", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Subject", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TimeSent", SimTypeRef("SYSTEMTIME", SimStruct)), ("PageCount", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_COVERPAGE_INFOW", pack=False, align=None),
    "FAX_COVERPAGE_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_COVERPAGE_TYPE_ENUM"),
    "FAX_DEVICE_RECEIVE_MODE_ENUM": SimTypeInt(signed=False, label="FAX_DEVICE_RECEIVE_MODE_ENUM"),
    "FAX_DEVICE_STATUSA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("CallerId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Csid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CurrentPage", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DocumentName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("JobType", SimTypeInt(signed=False, label="UInt32")), ("PhoneNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RoutingString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeRef("FILETIME", SimStruct)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("StatusString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SubmittedTime", SimTypeRef("FILETIME", SimStruct)), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("Tsid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_DEVICE_STATUSA", pack=False, align=None),
    "FAX_DEVICE_STATUSW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("CallerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Csid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CurrentPage", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("JobType", SimTypeInt(signed=False, label="UInt32")), ("PhoneNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RoutingString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeRef("FILETIME", SimStruct)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("StatusString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SubmittedTime", SimTypeRef("FILETIME", SimStruct)), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("Tsid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_DEVICE_STATUSW", pack=False, align=None),
    "FAX_DEV_STATUS": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("StatusId", SimTypeInt(signed=False, label="UInt32")), ("StringId", SimTypeInt(signed=False, label="UInt32")), ("PageCount", SimTypeInt(signed=False, label="UInt32")), ("CSI", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CallerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RoutingInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="FAX_DEV_STATUS", pack=False, align=None),
    "FAX_ENUM_DELIVERY_REPORT_TYPES": SimTypeInt(signed=False, label="FAX_ENUM_DELIVERY_REPORT_TYPES"),
    "FAX_ENUM_DEVICE_ID_SOURCE": SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"),
    "FAX_ENUM_JOB_COMMANDS": SimTypeInt(signed=False, label="FAX_ENUM_JOB_COMMANDS"),
    "FAX_ENUM_JOB_SEND_ATTRIBUTES": SimTypeInt(signed=False, label="FAX_ENUM_JOB_SEND_ATTRIBUTES"),
    "FAX_ENUM_LOG_CATEGORIES": SimTypeInt(signed=False, label="FAX_ENUM_LOG_CATEGORIES"),
    "FAX_ENUM_LOG_LEVELS": SimTypeInt(signed=False, label="FAX_ENUM_LOG_LEVELS"),
    "FAX_ENUM_PORT_OPEN_TYPE": SimTypeInt(signed=False, label="FAX_ENUM_PORT_OPEN_TYPE"),
    "FAX_EVENTA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeRef("FILETIME", SimStruct)), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("EventId", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_EVENTA", pack=False, align=None),
    "FAX_EVENTW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeRef("FILETIME", SimStruct)), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("EventId", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_EVENTW", pack=False, align=None),
    "FAX_GLOBAL_ROUTING_INFOA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FriendlyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FunctionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ExtensionImageName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ExtensionFriendlyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_GLOBAL_ROUTING_INFOA", pack=False, align=None),
    "FAX_GLOBAL_ROUTING_INFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FunctionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ExtensionImageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ExtensionFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_GLOBAL_ROUTING_INFOW", pack=False, align=None),
    "FAX_GROUP_STATUS_ENUM": SimTypeInt(signed=False, label="FAX_GROUP_STATUS_ENUM"),
    "FAX_JOB_ENTRYA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")), ("UserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("JobType", SimTypeInt(signed=False, label="UInt32")), ("QueueStatus", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PageCount", SimTypeInt(signed=False, label="UInt32")), ("RecipientNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderCompany", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderDept", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BillingCode", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ScheduleAction", SimTypeInt(signed=False, label="UInt32")), ("ScheduleTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("DeliveryReportType", SimTypeInt(signed=False, label="UInt32")), ("DeliveryReportAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DocumentName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_JOB_ENTRYA", pack=False, align=None),
    "FAX_JOB_ENTRYW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")), ("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("JobType", SimTypeInt(signed=False, label="UInt32")), ("QueueStatus", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PageCount", SimTypeInt(signed=False, label="UInt32")), ("RecipientNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderDept", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BillingCode", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ScheduleAction", SimTypeInt(signed=False, label="UInt32")), ("ScheduleTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("DeliveryReportType", SimTypeInt(signed=False, label="UInt32")), ("DeliveryReportAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_JOB_ENTRYW", pack=False, align=None),
    "FAX_JOB_EXTENDED_STATUS_ENUM": SimTypeInt(signed=False, label="FAX_JOB_EXTENDED_STATUS_ENUM"),
    "FAX_JOB_OPERATIONS_ENUM": SimTypeInt(signed=False, label="FAX_JOB_OPERATIONS_ENUM"),
    "FAX_JOB_PARAMA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("RecipientNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderCompany", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderDept", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BillingCode", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ScheduleAction", SimTypeInt(signed=False, label="UInt32")), ("ScheduleTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("DeliveryReportType", SimTypeInt(signed=False, label="UInt32")), ("DeliveryReportAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DocumentName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CallHandle", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)),)), name="FAX_JOB_PARAMA", pack=False, align=None),
    "FAX_JOB_PARAMW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("RecipientNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderDept", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BillingCode", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ScheduleAction", SimTypeInt(signed=False, label="UInt32")), ("ScheduleTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("DeliveryReportType", SimTypeInt(signed=False, label="UInt32")), ("DeliveryReportAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CallHandle", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)),)), name="FAX_JOB_PARAMW", pack=False, align=None),
    "FAX_JOB_STATUS_ENUM": SimTypeInt(signed=False, label="FAX_JOB_STATUS_ENUM"),
    "FAX_JOB_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_JOB_TYPE_ENUM"),
    "FAX_LOG_CATEGORYA": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Category", SimTypeInt(signed=False, label="UInt32")), ("Level", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_LOG_CATEGORYA", pack=False, align=None),
    "FAX_LOG_CATEGORYW": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Category", SimTypeInt(signed=False, label="UInt32")), ("Level", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_LOG_CATEGORYW", pack=False, align=None),
    "FAX_LOG_LEVEL_ENUM": SimTypeInt(signed=False, label="FAX_LOG_LEVEL_ENUM"),
    "FAX_PORT_INFOA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Rings", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Csid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_PORT_INFOA", pack=False, align=None),
    "FAX_PORT_INFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Rings", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Csid", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_PORT_INFOW", pack=False, align=None),
    "FAX_PRINT_INFOA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DocName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RecipientNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderCompany", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderDept", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SenderBillingCode", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Reserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DrEmailAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("OutputFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_PRINT_INFOA", pack=False, align=None),
    "FAX_PRINT_INFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DocName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecipientName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RecipientNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderDept", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SenderBillingCode", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Reserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DrEmailAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("OutputFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_PRINT_INFOW", pack=False, align=None),
    "FAX_PRIORITY_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_PRIORITY_TYPE_ENUM"),
    "FAX_PROVIDER_STATUS_ENUM": SimTypeInt(signed=False, label="FAX_PROVIDER_STATUS_ENUM"),
    "FAX_RECEIPT_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_RECEIPT_TYPE_ENUM"),
    "FAX_RECEIVE": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReceiverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReceiverNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="FAX_RECEIVE", pack=False, align=None),
    "FAX_ROUTE": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("JobId", SimTypeInt(signed=False, label="UInt32")), ("ElapsedTime", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveTime", SimTypeLongLong(signed=False, label="UInt64")), ("PageCount", SimTypeInt(signed=False, label="UInt32")), ("Csid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Tsid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CallerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RoutingInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReceiverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReceiverNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("RoutingInfoData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RoutingInfoDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="FAX_ROUTE", pack=False, align=None),
    "FAX_ROUTE_CALLBACKROUTINES": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("FaxRouteAddFile", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "FileName", "Guid"]), offset=0)), ("FaxRouteDeleteFile", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "FileName"]), offset=0)), ("FaxRouteGetFile", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "Index", "FileNameBuffer", "RequiredSize"]), offset=0)), ("FaxRouteEnumFiles", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "GuidOwner", "GuidCaller", "FileName", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "Guid", "FileEnumerator", "Context"]), offset=0)), ("FaxRouteModifyRoutingData", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "RoutingGuid", "RoutingData", "RoutingDataSize"]), offset=0)),)), name="FAX_ROUTE_CALLBACKROUTINES", pack=False, align=None),
    "FAX_ROUTING_METHODA": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Guid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FriendlyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FunctionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ExtensionImageName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ExtensionFriendlyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FAX_ROUTING_METHODA", pack=False, align=None),
    "FAX_ROUTING_METHODW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Guid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FunctionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ExtensionImageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ExtensionFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FAX_ROUTING_METHODW", pack=False, align=None),
    "FAX_ROUTING_RULE_CODE_ENUM": SimTypeInt(signed=False, label="FAX_ROUTING_RULE_CODE_ENUM"),
    "FAX_RULE_STATUS_ENUM": SimTypeInt(signed=False, label="FAX_RULE_STATUS_ENUM"),
    "FAX_SCHEDULE_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_SCHEDULE_TYPE_ENUM"),
    "FAX_SEND": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CallerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CallerNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReceiverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReceiverNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Branding", SimTypeInt(signed=True, label="Int32")), ("CallHandle", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="FAX_SEND", pack=False, align=None),
    "FAX_SERVER_APIVERSION_ENUM": SimTypeInt(signed=False, label="FAX_SERVER_APIVERSION_ENUM"),
    "FAX_SERVER_EVENTS_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_SERVER_EVENTS_TYPE_ENUM"),
    "FAX_SMTP_AUTHENTICATION_TYPE_ENUM": SimTypeInt(signed=False, label="FAX_SMTP_AUTHENTICATION_TYPE_ENUM"),
    "FAX_TIME": SimStruct(OrderedDict((("Hour", SimTypeShort(signed=False, label="UInt16")), ("Minute", SimTypeShort(signed=False, label="UInt16")),)), name="FAX_TIME", pack=False, align=None),
    "FA_ENTRY": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="DEBUG_FLR_PARAM_TYPE")), ("FullSize", SimTypeShort(signed=False, label="UInt16")), ("DataSize", SimTypeShort(signed=False, label="UInt16")),)), name="FA_ENTRY", pack=False, align=None),
    "FA_ENTRY_TYPE": SimTypeInt(signed=False, label="FA_ENTRY_TYPE"),
    "FA_EXTENSION_PLUGIN_PHASE": SimTypeInt(signed=False, label="FA_EXTENSION_PLUGIN_PHASE"),
    "FCACHE_CREATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpstrName", "lpvData", "cbFileSize", "cbFileSizeHigh"]), offset=0),
    "FCACHE_RICHCREATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpstrName", "lpvData", "cbFileSize", "cbFileSizeHigh", "pfDidWeScanIt", "pfIsStuffed", "pfStoredWithDots", "pfStoredWithTerminatingDot"]), offset=0),
    "FCIERROR": SimTypeInt(signed=False, label="FCIERROR"),
    "FDAP": SimTypeInt(signed=False, label="FDAP"),
    "FDEX_PROP_FLAGS": SimTypeInt(signed=False, label="FDEX_PROP_FLAGS"),
    "FDE_OVERWRITE_RESPONSE": SimTypeInt(signed=False, label="FDE_OVERWRITE_RESPONSE"),
    "FDE_SHAREVIOLATION_RESPONSE": SimTypeInt(signed=False, label="FDE_SHAREVIOLATION_RESPONSE"),
    "FDICABINETINFO": SimStruct(OrderedDict((("cbCabinet", SimTypeInt(signed=True, label="Int32")), ("cFolders", SimTypeShort(signed=False, label="UInt16")), ("cFiles", SimTypeShort(signed=False, label="UInt16")), ("setID", SimTypeShort(signed=False, label="UInt16")), ("iCabinet", SimTypeShort(signed=False, label="UInt16")), ("fReserve", SimTypeInt(signed=True, label="Int32")), ("hasprev", SimTypeInt(signed=True, label="Int32")), ("hasnext", SimTypeInt(signed=True, label="Int32")),)), name="FDICABINETINFO", pack=False, align=None),
    "FDICREATE_CPU_TYPE": SimTypeInt(signed=False, label="FDICREATE_CPU_TYPE"),
    "FDIDECRYPT": SimStruct(OrderedDict((("fdidt", SimTypeInt(signed=False, label="FDIDECRYPTTYPE")), ("pvUser", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Anonymous", SimUnion({"cabinet": SimTypeRef("_cabinet_e__Struct", SimStruct), "folder": SimTypeRef("_folder_e__Struct", SimStruct), "decrypt": SimTypeRef("_decrypt_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FDIDECRYPT", pack=False, align=None),
    "FDIDECRYPTTYPE": SimTypeInt(signed=False, label="FDIDECRYPTTYPE"),
    "FDIERROR": SimTypeInt(signed=False, label="FDIERROR"),
    "FDINOTIFICATION": SimStruct(OrderedDict((("cb", SimTypeInt(signed=True, label="Int32")), ("psz1", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("psz2", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("psz3", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hf", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("date", SimTypeShort(signed=False, label="UInt16")), ("time", SimTypeShort(signed=False, label="UInt16")), ("attribs", SimTypeShort(signed=False, label="UInt16")), ("setID", SimTypeShort(signed=False, label="UInt16")), ("iCabinet", SimTypeShort(signed=False, label="UInt16")), ("iFolder", SimTypeShort(signed=False, label="UInt16")), ("fdie", SimTypeInt(signed=False, label="FDIERROR")),)), name="FDINOTIFICATION", pack=False, align=None),
    "FDINOTIFICATIONTYPE": SimTypeInt(signed=False, label="FDINOTIFICATIONTYPE"),
    "FDISPILLFILE": SimStruct(OrderedDict((("ach", SimTypeArray(SimTypeChar(label="SByte"), 2)), ("cbFile", SimTypeInt(signed=True, label="Int32")),)), name="FDISPILLFILE", pack=False, align=None),
    "FD_DEVICEMETRICS": SimStruct(OrderedDict((("flRealizedType", SimTypeInt(signed=False, label="UInt32")), ("pteBase", SimTypeRef("POINTE", SimStruct)), ("pteSide", SimTypeRef("POINTE", SimStruct)), ("lD", SimTypeInt(signed=True, label="Int32")), ("fxMaxAscender", SimTypeInt(signed=True, label="Int32")), ("fxMaxDescender", SimTypeInt(signed=True, label="Int32")), ("ptlUnderline1", SimTypeRef("POINTL", SimStruct)), ("ptlStrikeOut", SimTypeRef("POINTL", SimStruct)), ("ptlULThickness", SimTypeRef("POINTL", SimStruct)), ("ptlSOThickness", SimTypeRef("POINTL", SimStruct)), ("cxMax", SimTypeInt(signed=False, label="UInt32")), ("cyMax", SimTypeInt(signed=False, label="UInt32")), ("cjGlyphMax", SimTypeInt(signed=False, label="UInt32")), ("fdxQuantized", SimTypeRef("FD_XFORM", SimStruct)), ("lNonLinearExtLeading", SimTypeInt(signed=True, label="Int32")), ("lNonLinearIntLeading", SimTypeInt(signed=True, label="Int32")), ("lNonLinearMaxCharWidth", SimTypeInt(signed=True, label="Int32")), ("lNonLinearAvgCharWidth", SimTypeInt(signed=True, label="Int32")), ("lMinA", SimTypeInt(signed=True, label="Int32")), ("lMinC", SimTypeInt(signed=True, label="Int32")), ("lMinD", SimTypeInt(signed=True, label="Int32")), ("alReserved", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="FD_DEVICEMETRICS", pack=False, align=None),
    "FD_FLAGS": SimTypeInt(signed=False, label="FD_FLAGS"),
    "FD_GLYPHATTR": SimStruct(OrderedDict((("cjThis", SimTypeInt(signed=False, label="UInt32")), ("cGlyphs", SimTypeInt(signed=False, label="UInt32")), ("iMode", SimTypeInt(signed=False, label="UInt32")), ("aGlyphAttr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FD_GLYPHATTR", pack=False, align=None),
    "FD_GLYPHSET": SimStruct(OrderedDict((("cjThis", SimTypeInt(signed=False, label="UInt32")), ("flAccel", SimTypeInt(signed=False, label="UInt32")), ("cGlyphsSupported", SimTypeInt(signed=False, label="UInt32")), ("cRuns", SimTypeInt(signed=False, label="UInt32")), ("awcrun", SimTypePointer(SimTypeRef("WCRUN", SimStruct), offset=0)),)), name="FD_GLYPHSET", pack=False, align=None),
    "FD_KERNINGPAIR": SimStruct(OrderedDict((("wcFirst", SimTypeChar(label="Char")), ("wcSecond", SimTypeChar(label="Char")), ("fwdKern", SimTypeShort(signed=True, label="Int16")),)), name="FD_KERNINGPAIR", pack=False, align=None),
    "FD_LIGATURE": SimStruct(OrderedDict((("culThis", SimTypeInt(signed=False, label="UInt32")), ("ulType", SimTypeInt(signed=False, label="UInt32")), ("cLigatures", SimTypeInt(signed=False, label="UInt32")), ("alig", SimTypePointer(SimTypeRef("LIGATURE", SimStruct), offset=0)),)), name="FD_LIGATURE", pack=False, align=None),
    "FD_SET": SimStruct(OrderedDict((("fd_count", SimTypeInt(signed=False, label="UInt32")), ("fd_array", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 64)),)), name="FD_SET", pack=False, align=None),
    "FD_XFORM": SimStruct(OrderedDict((("eXX", SimTypeInt(signed=False, label="UInt32")), ("eXY", SimTypeInt(signed=False, label="UInt32")), ("eYX", SimTypeInt(signed=False, label="UInt32")), ("eYY", SimTypeInt(signed=False, label="UInt32")),)), name="FD_XFORM", pack=False, align=None),
    "FEATURE_CHANGE_TIME": SimTypeInt(signed=False, label="FEATURE_CHANGE_TIME"),
    "FEATURE_ENABLED_STATE": SimTypeInt(signed=False, label="FEATURE_ENABLED_STATE"),
    "FEATURE_ERROR": SimStruct(OrderedDict((("hr", SimTypeInt(signed=True, label="Int32")), ("lineNumber", SimTypeShort(signed=False, label="UInt16")), ("file", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("process", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("module", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("callerReturnAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("callerModule", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("message", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("originLineNumber", SimTypeShort(signed=False, label="UInt16")), ("originFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("originModule", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("originCallerReturnAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("originCallerModule", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("originName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FEATURE_ERROR", pack=False, align=None),
    "FEATURE_STATE_CHANGE_SUBSCRIPTION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "FECMethod": SimTypeInt(signed=False, label="FECMethod"),
    "FEEDBACK_TYPE": SimTypeInt(signed=False, label="FEEDBACK_TYPE"),
    "FEEDS_BACKGROUNDSYNC_ACTION": SimTypeInt(signed=False, label="FEEDS_BACKGROUNDSYNC_ACTION"),
    "FEEDS_BACKGROUNDSYNC_STATUS": SimTypeInt(signed=False, label="FEEDS_BACKGROUNDSYNC_STATUS"),
    "FEEDS_DOWNLOAD_ERROR": SimTypeInt(signed=False, label="FEEDS_DOWNLOAD_ERROR"),
    "FEEDS_DOWNLOAD_STATUS": SimTypeInt(signed=False, label="FEEDS_DOWNLOAD_STATUS"),
    "FEEDS_ERROR_CODE": SimTypeInt(signed=False, label="FEEDS_ERROR_CODE"),
    "FEEDS_EVENTS_ITEM_COUNT_FLAGS": SimTypeInt(signed=False, label="FEEDS_EVENTS_ITEM_COUNT_FLAGS"),
    "FEEDS_EVENTS_MASK": SimTypeInt(signed=False, label="FEEDS_EVENTS_MASK"),
    "FEEDS_EVENTS_SCOPE": SimTypeInt(signed=False, label="FEEDS_EVENTS_SCOPE"),
    "FEEDS_SYNC_SETTING": SimTypeInt(signed=False, label="FEEDS_SYNC_SETTING"),
    "FEEDS_XML_FILTER_FLAGS": SimTypeInt(signed=False, label="FEEDS_XML_FILTER_FLAGS"),
    "FEEDS_XML_INCLUDE_FLAGS": SimTypeInt(signed=False, label="FEEDS_XML_INCLUDE_FLAGS"),
    "FEEDS_XML_SORT_ORDER": SimTypeInt(signed=False, label="FEEDS_XML_SORT_ORDER"),
    "FEEDS_XML_SORT_PROPERTY": SimTypeInt(signed=False, label="FEEDS_XML_SORT_PROPERTY"),
    "FExecuteInAppDomainCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cookie"]), offset=0),
    "FFFP_MODE": SimTypeInt(signed=False, label="FFFP_MODE"),
    "FH_BACKUP_STATUS": SimTypeInt(signed=False, label="FH_BACKUP_STATUS"),
    "FH_DEVICE_VALIDATION_RESULT": SimTypeInt(signed=False, label="FH_DEVICE_VALIDATION_RESULT"),
    "FH_LOCAL_POLICY_TYPE": SimTypeInt(signed=False, label="FH_LOCAL_POLICY_TYPE"),
    "FH_OVERLAPPED": SimStruct(OrderedDict((("Internal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InternalHigh", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("OffsetHigh", SimTypeInt(signed=False, label="UInt32")), ("hEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCompletion", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FIO_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("FH_OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pContext", "lpo", "cb", "dwCompletionStatus"]), offset=0)), ("Reserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved4", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="FH_OVERLAPPED", pack=False, align=None),
    "FH_PROTECTED_ITEM_CATEGORY": SimTypeInt(signed=False, label="FH_PROTECTED_ITEM_CATEGORY"),
    "FH_RETENTION_TYPES": SimTypeInt(signed=False, label="FH_RETENTION_TYPES"),
    "FH_SERVICE_PIPE_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "FH_TARGET_DRIVE_TYPES": SimTypeInt(signed=False, label="FH_TARGET_DRIVE_TYPES"),
    "FH_TARGET_PROPERTY_TYPE": SimTypeInt(signed=False, label="FH_TARGET_PROPERTY_TYPE"),
    "FIELD_INFO": SimStruct(OrderedDict((("fName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("printName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")), ("fOptions", SimTypeInt(signed=False, label="UInt32")), ("address", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"fieldCallBack": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pBuffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("TypeId", SimTypeInt(signed=False, label="UInt32")), ("FieldOffset", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("BitField", SimTypeRef("_BitField", SimStruct)), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="FIELD_INFO", pack=False, align=None),
    "FILEDESCRIPTORA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")), ("sizel", SimTypeRef("SIZE", SimStruct)), ("pointl", SimTypeRef("POINTL", SimStruct)), ("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ftCreationTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastWriteTime", SimTypeRef("FILETIME", SimStruct)), ("nFileSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeLow", SimTypeInt(signed=False, label="UInt32")), ("cFileName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="FILEDESCRIPTORA", pack=False, align=None),
    "FILEDESCRIPTORW": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")), ("sizel", SimTypeRef("SIZE", SimStruct)), ("pointl", SimTypeRef("POINTL", SimStruct)), ("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ftCreationTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastWriteTime", SimTypeRef("FILETIME", SimStruct)), ("nFileSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeLow", SimTypeInt(signed=False, label="UInt32")), ("cFileName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="FILEDESCRIPTORW", pack=False, align=None),
    "FILEGROUPDESCRIPTORA": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("fgd", SimTypePointer(SimTypeRef("FILEDESCRIPTORA", SimStruct), offset=0)),)), name="FILEGROUPDESCRIPTORA", pack=False, align=None),
    "FILEGROUPDESCRIPTORW": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("fgd", SimTypePointer(SimTypeRef("FILEDESCRIPTORW", SimStruct), offset=0)),)), name="FILEGROUPDESCRIPTORW", pack=False, align=None),
    "FILEMUIINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFileType", SimTypeInt(signed=False, label="UInt32")), ("pChecksum", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("pServiceChecksum", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwLanguageNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTypeIDMainSize", SimTypeInt(signed=False, label="UInt32")), ("dwTypeIDMainOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTypeNameMainOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTypeIDMUISize", SimTypeInt(signed=False, label="UInt32")), ("dwTypeIDMUIOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTypeNameMUIOffset", SimTypeInt(signed=False, label="UInt32")), ("abBuffer", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="FILEMUIINFO", pack=False, align=None),
    "FILEOPENDIALOGOPTIONS": SimTypeInt(signed=False, label="FILEOPENDIALOGOPTIONS"),
    "FILEOPERATION_FLAGS": SimTypeInt(signed=False, label="FILEOPERATION_FLAGS"),
    "FILEPATHS_A": SimStruct(OrderedDict((("Target", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Source", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILEPATHS_A", pack=False, align=None),
    "FILEPATHS_SIGNERINFO_A": SimStruct(OrderedDict((("Target", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Source", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DigitalSigner", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Version", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CatalogFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FILEPATHS_SIGNERINFO_A", pack=False, align=None),
    "FILEPATHS_SIGNERINFO_W": SimStruct(OrderedDict((("Target", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Source", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DigitalSigner", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Version", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CatalogFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILEPATHS_SIGNERINFO_W", pack=False, align=None),
    "FILEPATHS_W": SimStruct(OrderedDict((("Target", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Source", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILEPATHS_W", pack=False, align=None),
    "FILESETINFO": SimStruct(OrderedDict((("bstrRemoteFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrLocalFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSizeHint", SimTypeInt(signed=False, label="UInt32")),)), name="FILESETINFO", pack=False, align=None),
    "FILESHARE_CHANGE": SimStruct(OrderedDict((("Change", SimTypeInt(signed=False, label="FILESHARE_CHANGE_ENUM")), ("ShareName", SimTypeArray(SimTypeChar(label="Char"), 84)),)), name="FILESHARE_CHANGE", pack=False, align=None),
    "FILESHARE_CHANGE_ENUM": SimTypeInt(signed=False, label="FILESHARE_CHANGE_ENUM"),
    "FILESHARE_CHANGE_LIST": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("ChangeEntry", SimTypePointer(SimTypeRef("FILESHARE_CHANGE", SimStruct), offset=0)),)), name="FILESHARE_CHANGE_LIST", pack=False, align=None),
    "FILESYSTEM_STATISTICS": SimStruct(OrderedDict((("FileSystemType", SimTypeInt(signed=False, label="FILESYSTEM_STATISTICS_TYPE")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("SizeOfCompleteStructure", SimTypeInt(signed=False, label="UInt32")), ("UserFileReads", SimTypeInt(signed=False, label="UInt32")), ("UserFileReadBytes", SimTypeInt(signed=False, label="UInt32")), ("UserDiskReads", SimTypeInt(signed=False, label="UInt32")), ("UserFileWrites", SimTypeInt(signed=False, label="UInt32")), ("UserFileWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("UserDiskWrites", SimTypeInt(signed=False, label="UInt32")), ("MetaDataReads", SimTypeInt(signed=False, label="UInt32")), ("MetaDataReadBytes", SimTypeInt(signed=False, label="UInt32")), ("MetaDataDiskReads", SimTypeInt(signed=False, label="UInt32")), ("MetaDataWrites", SimTypeInt(signed=False, label="UInt32")), ("MetaDataWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("MetaDataDiskWrites", SimTypeInt(signed=False, label="UInt32")),)), name="FILESYSTEM_STATISTICS", pack=False, align=None),
    "FILESYSTEM_STATISTICS_EX": SimStruct(OrderedDict((("FileSystemType", SimTypeInt(signed=False, label="FILESYSTEM_STATISTICS_TYPE")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("SizeOfCompleteStructure", SimTypeInt(signed=False, label="UInt32")), ("UserFileReads", SimTypeLongLong(signed=False, label="UInt64")), ("UserFileReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("UserDiskReads", SimTypeLongLong(signed=False, label="UInt64")), ("UserFileWrites", SimTypeLongLong(signed=False, label="UInt64")), ("UserFileWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("UserDiskWrites", SimTypeLongLong(signed=False, label="UInt64")), ("MetaDataReads", SimTypeLongLong(signed=False, label="UInt64")), ("MetaDataReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MetaDataDiskReads", SimTypeLongLong(signed=False, label="UInt64")), ("MetaDataWrites", SimTypeLongLong(signed=False, label="UInt64")), ("MetaDataWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MetaDataDiskWrites", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILESYSTEM_STATISTICS_EX", pack=False, align=None),
    "FILESYSTEM_STATISTICS_TYPE": SimTypeInt(signed=False, label="FILESYSTEM_STATISTICS_TYPE"),
    "FILETIME": SimStruct(OrderedDict((("dwLowDateTime", SimTypeInt(signed=False, label="UInt32")), ("dwHighDateTime", SimTypeInt(signed=False, label="UInt32")),)), name="FILETIME", pack=False, align=None),
    "FILETYPEATTRIBUTEFLAGS": SimTypeInt(signed=False, label="FILETYPEATTRIBUTEFLAGS"),
    "FILE_ACCESSMODE": SimTypeInt(signed=False, label="FILE_ACCESSMODE"),
    "FILE_ACCESS_INFORMATION": SimStruct(OrderedDict((("AccessFlags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_ACCESS_INFORMATION", pack=False, align=None),
    "FILE_ACCESS_RIGHTS": SimTypeInt(signed=False, label="FILE_ACCESS_RIGHTS"),
    "FILE_ACTION": SimTypeInt(signed=False, label="FILE_ACTION"),
    "FILE_ALIGNMENT_INFO": SimStruct(OrderedDict((("AlignmentRequirement", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_ALIGNMENT_INFO", pack=False, align=None),
    "FILE_ALIGNMENT_INFORMATION": SimStruct(OrderedDict((("AlignmentRequirement", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_ALIGNMENT_INFORMATION", pack=False, align=None),
    "FILE_ALLOCATED_RANGE_BUFFER": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_ALLOCATED_RANGE_BUFFER", pack=False, align=None),
    "FILE_ALLOCATION_INFO": SimStruct(OrderedDict((("AllocationSize", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_ALLOCATION_INFO", pack=False, align=None),
    "FILE_ALLOCATION_INFORMATION": SimStruct(OrderedDict((("AllocationSize", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_ALLOCATION_INFORMATION", pack=False, align=None),
    "FILE_ALL_INFORMATION": SimStruct(OrderedDict((("BasicInformation", SimTypeRef("FILE_BASIC_INFORMATION", SimStruct)), ("StandardInformation", SimTypeRef("FILE_STANDARD_INFORMATION", SimStruct)), ("InternalInformation", SimTypeRef("FILE_INTERNAL_INFORMATION", SimStruct)), ("EaInformation", SimTypeRef("FILE_EA_INFORMATION", SimStruct)), ("AccessInformation", SimTypeRef("FILE_ACCESS_INFORMATION", SimStruct)), ("PositionInformation", SimTypeRef("FILE_POSITION_INFORMATION", SimStruct)), ("ModeInformation", SimTypeRef("FILE_MODE_INFORMATION", SimStruct)), ("AlignmentInformation", SimTypeRef("FILE_ALIGNMENT_INFORMATION", SimStruct)), ("NameInformation", SimTypeRef("FILE_NAME_INFORMATION", SimStruct)),)), name="FILE_ALL_INFORMATION", pack=False, align=None),
    "FILE_ATTRIBUTES_ARRAY": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("dwSumFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwProductFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("rgdwFileAttributes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="FILE_ATTRIBUTES_ARRAY", pack=False, align=None),
    "FILE_ATTRIBUTE_TAG_INFO": SimStruct(OrderedDict((("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ReparseTag", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_ATTRIBUTE_TAG_INFO", pack=False, align=None),
    "FILE_ATTRIBUTE_TAG_INFORMATION": SimStruct(OrderedDict((("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ReparseTag", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_ATTRIBUTE_TAG_INFORMATION", pack=False, align=None),
    "FILE_BASIC_INFO": SimStruct(OrderedDict((("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_BASIC_INFO", pack=False, align=None),
    "FILE_BASIC_INFORMATION": SimStruct(OrderedDict((("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_BASIC_INFORMATION", pack=False, align=None),
    "FILE_BOTH_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("ShortNameLength", SimTypeChar(label="SByte")), ("ShortName", SimTypeArray(SimTypeChar(label="Char"), 12)), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_BOTH_DIR_INFORMATION", pack=False, align=None),
    "FILE_CASE_SENSITIVE_INFO": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_CASE_SENSITIVE_INFO", pack=False, align=None),
    "FILE_CASE_SENSITIVE_INFORMATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_CASE_SENSITIVE_INFORMATION", pack=False, align=None),
    "FILE_COMPLETION_INFORMATION": SimStruct(OrderedDict((("Port", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FILE_COMPLETION_INFORMATION", pack=False, align=None),
    "FILE_COMPRESSION_INFO": SimStruct(OrderedDict((("CompressedFileSize", SimTypeLongLong(signed=True, label="Int64")), ("CompressionFormat", SimTypeInt(signed=False, label="COMPRESSION_FORMAT")), ("CompressionUnitShift", SimTypeChar(label="Byte")), ("ChunkShift", SimTypeChar(label="Byte")), ("ClusterShift", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="FILE_COMPRESSION_INFO", pack=False, align=None),
    "FILE_COMPRESSION_INFORMATION": SimStruct(OrderedDict((("CompressedFileSize", SimTypeLongLong(signed=True, label="Int64")), ("CompressionFormat", SimTypeShort(signed=False, label="UInt16")), ("CompressionUnitShift", SimTypeChar(label="Byte")), ("ChunkShift", SimTypeChar(label="Byte")), ("ClusterShift", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="FILE_COMPRESSION_INFORMATION", pack=False, align=None),
    "FILE_CREATION_DISPOSITION": SimTypeInt(signed=False, label="FILE_CREATION_DISPOSITION"),
    "FILE_DESIRED_STORAGE_CLASS_INFORMATION": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="FILE_STORAGE_TIER_CLASS")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_DESIRED_STORAGE_CLASS_INFORMATION", pack=False, align=None),
    "FILE_DEVICE_TYPE": SimTypeInt(signed=False, label="FILE_DEVICE_TYPE"),
    "FILE_DIRECTORY_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_DIRECTORY_INFORMATION", pack=False, align=None),
    "FILE_DISPOSITION_INFO": SimStruct(OrderedDict((("DeleteFile", SimTypeChar(label="Byte")),)), name="FILE_DISPOSITION_INFO", pack=False, align=None),
    "FILE_DISPOSITION_INFORMATION": SimStruct(OrderedDict((("DeleteFile", SimTypeChar(label="Byte")),)), name="FILE_DISPOSITION_INFORMATION", pack=False, align=None),
    "FILE_DISPOSITION_INFORMATION_EX": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="FILE_DISPOSITION_INFORMATION_EX_FLAGS")),)), name="FILE_DISPOSITION_INFORMATION_EX", pack=False, align=None),
    "FILE_DISPOSITION_INFORMATION_EX_FLAGS": SimTypeInt(signed=False, label="FILE_DISPOSITION_INFORMATION_EX_FLAGS"),
    "FILE_DISPOSITION_INFO_EX": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="FILE_DISPOSITION_INFO_EX_FLAGS")),)), name="FILE_DISPOSITION_INFO_EX", pack=False, align=None),
    "FILE_DISPOSITION_INFO_EX_FLAGS": SimTypeInt(signed=False, label="FILE_DISPOSITION_INFO_EX_FLAGS"),
    "FILE_EA_INFORMATION": SimStruct(OrderedDict((("EaSize", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_EA_INFORMATION", pack=False, align=None),
    "FILE_END_OF_FILE_INFO": SimStruct(OrderedDict((("EndOfFile", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_END_OF_FILE_INFO", pack=False, align=None),
    "FILE_END_OF_FILE_INFORMATION": SimStruct(OrderedDict((("EndOfFile", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_END_OF_FILE_INFORMATION", pack=False, align=None),
    "FILE_END_OF_FILE_INFORMATION_EX": SimStruct(OrderedDict((("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("PagingFileSizeInMM", SimTypeLongLong(signed=True, label="Int64")), ("PagingFileMaxSize", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_END_OF_FILE_INFORMATION_EX", pack=False, align=None),
    "FILE_EXTENT": SimStruct(OrderedDict((("VolumeOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ExtentLength", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILE_EXTENT", pack=False, align=None),
    "FILE_FLAGS_AND_ATTRIBUTES": SimTypeInt(signed=False, label="FILE_FLAGS_AND_ATTRIBUTES"),
    "FILE_FLUSH_MODE": SimTypeInt(signed=False, label="FILE_FLUSH_MODE"),
    "FILE_FS_ATTRIBUTE_INFORMATION": SimStruct(OrderedDict((("FileSystemAttributes", SimTypeInt(signed=False, label="UInt32")), ("MaximumComponentNameLength", SimTypeInt(signed=True, label="Int32")), ("FileSystemNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileSystemName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_FS_ATTRIBUTE_INFORMATION", pack=False, align=None),
    "FILE_FS_CONTROL_INFORMATION": SimStruct(OrderedDict((("FreeSpaceStartFiltering", SimTypeLongLong(signed=True, label="Int64")), ("FreeSpaceThreshold", SimTypeLongLong(signed=True, label="Int64")), ("FreeSpaceStopFiltering", SimTypeLongLong(signed=True, label="Int64")), ("DefaultQuotaThreshold", SimTypeLongLong(signed=True, label="Int64")), ("DefaultQuotaLimit", SimTypeLongLong(signed=True, label="Int64")), ("FileSystemControlFlags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_CONTROL_INFORMATION", pack=False, align=None),
    "FILE_FS_DATA_COPY_INFORMATION": SimStruct(OrderedDict((("NumberOfCopies", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_DATA_COPY_INFORMATION", pack=False, align=None),
    "FILE_FS_DEVICE_INFORMATION": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("Characteristics", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_DEVICE_INFORMATION", pack=False, align=None),
    "FILE_FS_DRIVER_PATH_INFORMATION": SimStruct(OrderedDict((("DriverInPath", SimTypeChar(label="Byte")), ("DriverNameLength", SimTypeInt(signed=False, label="UInt32")), ("DriverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_FS_DRIVER_PATH_INFORMATION", pack=False, align=None),
    "FILE_FS_FULL_SIZE_INFORMATION": SimStruct(OrderedDict((("TotalAllocationUnits", SimTypeLongLong(signed=True, label="Int64")), ("CallerAvailableAllocationUnits", SimTypeLongLong(signed=True, label="Int64")), ("ActualAvailableAllocationUnits", SimTypeLongLong(signed=True, label="Int64")), ("SectorsPerAllocationUnit", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_FULL_SIZE_INFORMATION", pack=False, align=None),
    "FILE_FS_FULL_SIZE_INFORMATION_EX": SimStruct(OrderedDict((("ActualTotalAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("ActualAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("ActualPoolUnavailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("CallerTotalAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("CallerAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("CallerPoolUnavailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("UsedAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("TotalReservedAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeStorageReserveAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableCommittedAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("PoolAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("SectorsPerAllocationUnit", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_FULL_SIZE_INFORMATION_EX", pack=False, align=None),
    "FILE_FS_LABEL_INFORMATION": SimStruct(OrderedDict((("VolumeLabelLength", SimTypeInt(signed=False, label="UInt32")), ("VolumeLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_FS_LABEL_INFORMATION", pack=False, align=None),
    "FILE_FS_METADATA_SIZE_INFORMATION": SimStruct(OrderedDict((("TotalMetadataAllocationUnits", SimTypeLongLong(signed=True, label="Int64")), ("SectorsPerAllocationUnit", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_METADATA_SIZE_INFORMATION", pack=False, align=None),
    "FILE_FS_OBJECTID_INFORMATION": SimStruct(OrderedDict((("ObjectId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ExtendedInfo", SimTypeArray(SimTypeChar(label="Byte"), 48)),)), name="FILE_FS_OBJECTID_INFORMATION", pack=False, align=None),
    "FILE_FS_PERSISTENT_VOLUME_INFORMATION": SimStruct(OrderedDict((("VolumeFlags", SimTypeInt(signed=False, label="UInt32")), ("FlagMask", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_PERSISTENT_VOLUME_INFORMATION", pack=False, align=None),
    "FILE_FS_SECTOR_SIZE_INFORMATION": SimStruct(OrderedDict((("LogicalBytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("PhysicalBytesPerSectorForAtomicity", SimTypeInt(signed=False, label="UInt32")), ("PhysicalBytesPerSectorForPerformance", SimTypeInt(signed=False, label="UInt32")), ("FileSystemEffectivePhysicalBytesPerSectorForAtomicity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ByteOffsetForSectorAlignment", SimTypeInt(signed=False, label="UInt32")), ("ByteOffsetForPartitionAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_SECTOR_SIZE_INFORMATION", pack=False, align=None),
    "FILE_FS_SIZE_INFORMATION": SimStruct(OrderedDict((("TotalAllocationUnits", SimTypeLongLong(signed=True, label="Int64")), ("AvailableAllocationUnits", SimTypeLongLong(signed=True, label="Int64")), ("SectorsPerAllocationUnit", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_SIZE_INFORMATION", pack=False, align=None),
    "FILE_FS_VOLUME_FLAGS_INFORMATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_FS_VOLUME_FLAGS_INFORMATION", pack=False, align=None),
    "FILE_FS_VOLUME_INFORMATION": SimStruct(OrderedDict((("VolumeCreationTime", SimTypeLongLong(signed=True, label="Int64")), ("VolumeSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("VolumeLabelLength", SimTypeInt(signed=False, label="UInt32")), ("SupportsObjects", SimTypeChar(label="Byte")), ("VolumeLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_FS_VOLUME_INFORMATION", pack=False, align=None),
    "FILE_FULL_DIR_INFO": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_FULL_DIR_INFO", pack=False, align=None),
    "FILE_FULL_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_FULL_DIR_INFORMATION", pack=False, align=None),
    "FILE_FULL_EA_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeChar(label="Byte")), ("EaNameLength", SimTypeChar(label="Byte")), ("EaValueLength", SimTypeShort(signed=False, label="UInt16")), ("EaName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="FILE_FULL_EA_INFORMATION", pack=False, align=None),
    "FILE_GET_EA_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("EaNameLength", SimTypeChar(label="Byte")), ("EaName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="FILE_GET_EA_INFORMATION", pack=False, align=None),
    "FILE_GET_QUOTA_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("SidLength", SimTypeInt(signed=False, label="UInt32")), ("Sid", SimTypeRef("SID", SimStruct)),)), name="FILE_GET_QUOTA_INFORMATION", pack=False, align=None),
    "FILE_ID_128": SimStruct(OrderedDict((("Identifier", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="FILE_ID_128", pack=False, align=None),
    "FILE_ID_BOTH_DIR_INFO": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("ShortNameLength", SimTypeChar(label="SByte")), ("ShortName", SimTypeArray(SimTypeChar(label="Char"), 12)), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_BOTH_DIR_INFO", pack=False, align=None),
    "FILE_ID_BOTH_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("ShortNameLength", SimTypeChar(label="SByte")), ("ShortName", SimTypeArray(SimTypeChar(label="Char"), 12)), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_BOTH_DIR_INFORMATION", pack=False, align=None),
    "FILE_ID_DESCRIPTOR": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="FILE_ID_TYPE")), ("Anonymous", SimUnion({"FileId": SimTypeLongLong(signed=True, label="Int64"), "ObjectId": SimTypeBottom(label="Guid"), "ExtendedFileId": SimTypeRef("FILE_ID_128", SimStruct)}, name="<anon>", label="None")),)), name="FILE_ID_DESCRIPTOR", pack=False, align=None),
    "FILE_ID_EXTD_BOTH_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("ReparsePointTag", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeRef("FILE_ID_128", SimStruct)), ("ShortNameLength", SimTypeChar(label="SByte")), ("ShortName", SimTypeArray(SimTypeChar(label="Char"), 12)), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_EXTD_BOTH_DIR_INFORMATION", pack=False, align=None),
    "FILE_ID_EXTD_DIR_INFO": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("ReparsePointTag", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeRef("FILE_ID_128", SimStruct)), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_EXTD_DIR_INFO", pack=False, align=None),
    "FILE_ID_EXTD_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("ReparsePointTag", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeRef("FILE_ID_128", SimStruct)), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_EXTD_DIR_INFORMATION", pack=False, align=None),
    "FILE_ID_FULL_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_FULL_DIR_INFORMATION", pack=False, align=None),
    "FILE_ID_GLOBAL_TX_DIR_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("LockingTransactionId", SimTypeBottom(label="Guid")), ("TxInfoFlags", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_ID_GLOBAL_TX_DIR_INFORMATION", pack=False, align=None),
    "FILE_ID_INFO": SimStruct(OrderedDict((("VolumeSerialNumber", SimTypeLongLong(signed=False, label="UInt64")), ("FileId", SimTypeRef("FILE_ID_128", SimStruct)),)), name="FILE_ID_INFO", pack=False, align=None),
    "FILE_ID_INFORMATION": SimStruct(OrderedDict((("VolumeSerialNumber", SimTypeLongLong(signed=False, label="UInt64")), ("FileId", SimTypeRef("FILE_ID_128", SimStruct)),)), name="FILE_ID_INFORMATION", pack=False, align=None),
    "FILE_ID_TYPE": SimTypeInt(signed=False, label="FILE_ID_TYPE"),
    "FILE_INFORMATION_CLASS": SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS"),
    "FILE_INFORMATION_DEFINITION": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")), ("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileNameLengthOffset", SimTypeInt(signed=False, label="UInt32")), ("FileNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_INFORMATION_DEFINITION", pack=False, align=None),
    "FILE_INFO_2": SimStruct(OrderedDict((("fi2_id", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_INFO_2", pack=False, align=None),
    "FILE_INFO_3": SimStruct(OrderedDict((("fi3_id", SimTypeInt(signed=False, label="UInt32")), ("fi3_permissions", SimTypeInt(signed=False, label="FILE_INFO_FLAGS_PERMISSIONS")), ("fi3_num_locks", SimTypeInt(signed=False, label="UInt32")), ("fi3_pathname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fi3_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_INFO_3", pack=False, align=None),
    "FILE_INFO_BY_HANDLE_CLASS": SimTypeInt(signed=False, label="FILE_INFO_BY_HANDLE_CLASS"),
    "FILE_INFO_FLAGS_PERMISSIONS": SimTypeInt(signed=False, label="FILE_INFO_FLAGS_PERMISSIONS"),
    "FILE_INITIATE_REPAIR_OUTPUT_BUFFER": SimStruct(OrderedDict((("Hint1", SimTypeLongLong(signed=False, label="UInt64")), ("Hint2", SimTypeLongLong(signed=False, label="UInt64")), ("Clsn", SimTypeLongLong(signed=False, label="UInt64")), ("Status", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_INITIATE_REPAIR_OUTPUT_BUFFER", pack=False, align=None),
    "FILE_INTERNAL_INFORMATION": SimStruct(OrderedDict((("IndexNumber", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_INTERNAL_INFORMATION", pack=False, align=None),
    "FILE_IN_CABINET_INFO_A": SimStruct(OrderedDict((("NameInCabinet", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FileSize", SimTypeInt(signed=False, label="UInt32")), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("DosDate", SimTypeShort(signed=False, label="UInt16")), ("DosTime", SimTypeShort(signed=False, label="UInt16")), ("DosAttribs", SimTypeShort(signed=False, label="UInt16")), ("FullTargetName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="FILE_IN_CABINET_INFO_A", pack=False, align=None),
    "FILE_IN_CABINET_INFO_W": SimStruct(OrderedDict((("NameInCabinet", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FileSize", SimTypeInt(signed=False, label="UInt32")), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("DosDate", SimTypeShort(signed=False, label="UInt16")), ("DosTime", SimTypeShort(signed=False, label="UInt16")), ("DosAttribs", SimTypeShort(signed=False, label="UInt16")), ("FullTargetName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="FILE_IN_CABINET_INFO_W", pack=False, align=None),
    "FILE_IOSTATUSBLOCK_RANGE_INFORMATION": SimStruct(OrderedDict((("IoStatusBlockRange", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_IOSTATUSBLOCK_RANGE_INFORMATION", pack=False, align=None),
    "FILE_IO_COMPLETION_NOTIFICATION_INFORMATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_IO_COMPLETION_NOTIFICATION_INFORMATION", pack=False, align=None),
    "FILE_IO_PRIORITY_HINT_INFO": SimStruct(OrderedDict((("PriorityHint", SimTypeInt(signed=False, label="PRIORITY_HINT")),)), name="FILE_IO_PRIORITY_HINT_INFO", pack=False, align=None),
    "FILE_IO_PRIORITY_HINT_INFORMATION": SimStruct(OrderedDict((("PriorityHint", SimTypeInt(signed=False, label="IO_PRIORITY_HINT")),)), name="FILE_IO_PRIORITY_HINT_INFORMATION", pack=False, align=None),
    "FILE_IO_PRIORITY_HINT_INFORMATION_EX": SimStruct(OrderedDict((("PriorityHint", SimTypeInt(signed=False, label="IO_PRIORITY_HINT")), ("BoostOutstanding", SimTypeChar(label="Byte")),)), name="FILE_IO_PRIORITY_HINT_INFORMATION_EX", pack=False, align=None),
    "FILE_IS_REMOTE_DEVICE_INFORMATION": SimStruct(OrderedDict((("IsRemote", SimTypeChar(label="Byte")),)), name="FILE_IS_REMOTE_DEVICE_INFORMATION", pack=False, align=None),
    "FILE_KNOWN_FOLDER_INFORMATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="FILE_KNOWN_FOLDER_TYPE")),)), name="FILE_KNOWN_FOLDER_INFORMATION", pack=False, align=None),
    "FILE_KNOWN_FOLDER_TYPE": SimTypeInt(signed=False, label="FILE_KNOWN_FOLDER_TYPE"),
    "FILE_LAYOUT_ENTRY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("NextFileOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("FirstNameOffset", SimTypeInt(signed=False, label="UInt32")), ("FirstStreamOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtraInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtraInfoLength", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_LAYOUT_ENTRY", pack=False, align=None),
    "FILE_LAYOUT_INFO_ENTRY": SimStruct(OrderedDict((("BasicInformation", SimTypeRef("_BasicInformation_e__Struct", SimStruct)), ("OwnerId", SimTypeInt(signed=False, label="UInt32")), ("SecurityId", SimTypeInt(signed=False, label="UInt32")), ("Usn", SimTypeLongLong(signed=True, label="Int64")), ("StorageReserveId", SimTypeInt(signed=False, label="STORAGE_RESERVE_ID")),)), name="FILE_LAYOUT_INFO_ENTRY", pack=False, align=None),
    "FILE_LAYOUT_NAME_ENTRY": SimStruct(OrderedDict((("NextNameOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ParentFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_LAYOUT_NAME_ENTRY", pack=False, align=None),
    "FILE_LEVEL_TRIM": SimStruct(OrderedDict((("Key", SimTypeInt(signed=False, label="UInt32")), ("NumRanges", SimTypeInt(signed=False, label="UInt32")), ("Ranges", SimTypePointer(SimTypeRef("FILE_LEVEL_TRIM_RANGE", SimStruct), offset=0)),)), name="FILE_LEVEL_TRIM", pack=False, align=None),
    "FILE_LEVEL_TRIM_OUTPUT": SimStruct(OrderedDict((("NumRangesProcessed", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_LEVEL_TRIM_OUTPUT", pack=False, align=None),
    "FILE_LEVEL_TRIM_RANGE": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILE_LEVEL_TRIM_RANGE", pack=False, align=None),
    "FILE_LINKS_FULL_ID_INFORMATION": SimStruct(OrderedDict((("BytesNeeded", SimTypeInt(signed=False, label="UInt32")), ("EntriesReturned", SimTypeInt(signed=False, label="UInt32")), ("Entry", SimTypeRef("FILE_LINK_ENTRY_FULL_ID_INFORMATION", SimStruct)),)), name="FILE_LINKS_FULL_ID_INFORMATION", pack=False, align=None),
    "FILE_LINKS_INFORMATION": SimStruct(OrderedDict((("BytesNeeded", SimTypeInt(signed=False, label="UInt32")), ("EntriesReturned", SimTypeInt(signed=False, label="UInt32")), ("Entry", SimTypeRef("FILE_LINK_ENTRY_INFORMATION", SimStruct)),)), name="FILE_LINKS_INFORMATION", pack=False, align=None),
    "FILE_LINK_ENTRY_FULL_ID_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("ParentFileId", SimTypeRef("FILE_ID_128", SimStruct)), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_LINK_ENTRY_FULL_ID_INFORMATION", pack=False, align=None),
    "FILE_LINK_ENTRY_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("ParentFileId", SimTypeLongLong(signed=True, label="Int64")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_LINK_ENTRY_INFORMATION", pack=False, align=None),
    "FILE_LINK_INFORMATION": SimStruct(OrderedDict((("Anonymous", SimUnion({"ReplaceIfExists": SimTypeChar(label="Byte"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RootDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_LINK_INFORMATION", pack=False, align=None),
    "FILE_LOCK": SimStruct(OrderedDict((("CompleteLockIrpRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Irp"]), offset=0)), ("UnlockRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FILE_LOCK_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "FileLockInfo"]), offset=0)), ("FastIoIsQuestionable", SimTypeChar(label="Byte")), ("SpareC", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("LockInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LastReturnedLockInfo", SimTypeRef("FILE_LOCK_INFO", SimStruct)), ("LastReturnedLock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LockRequestsInProgress", SimTypeInt(signed=True, label="Int32")),)), name="FILE_LOCK", pack=False, align=None),
    "FILE_LOCK_INFO": SimStruct(OrderedDict((("StartingByte", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("ExclusiveLock", SimTypeChar(label="Byte")), ("Key", SimTypeInt(signed=False, label="UInt32")), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("ProcessId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EndingByte", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_LOCK_INFO", pack=False, align=None),
    "FILE_MAILSLOT_QUERY_INFORMATION": SimStruct(OrderedDict((("MaximumMessageSize", SimTypeInt(signed=False, label="UInt32")), ("MailslotQuota", SimTypeInt(signed=False, label="UInt32")), ("NextMessageSize", SimTypeInt(signed=False, label="UInt32")), ("MessagesAvailable", SimTypeInt(signed=False, label="UInt32")), ("ReadTimeout", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_MAILSLOT_QUERY_INFORMATION", pack=False, align=None),
    "FILE_MAILSLOT_SET_INFORMATION": SimStruct(OrderedDict((("ReadTimeout", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="FILE_MAILSLOT_SET_INFORMATION", pack=False, align=None),
    "FILE_MAKE_COMPATIBLE_BUFFER": SimStruct(OrderedDict((("CloseDisc", SimTypeChar(label="Byte")),)), name="FILE_MAKE_COMPATIBLE_BUFFER", pack=False, align=None),
    "FILE_MAP": SimTypeInt(signed=False, label="FILE_MAP"),
    "FILE_MEMORY_PARTITION_INFORMATION": SimStruct(OrderedDict((("OwnerPartitionHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("NoCrossPartitionAccess", SimTypeChar(label="Byte")), ("Spare", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="_Anonymous_e__Struct", pack=False, align=None), "AllFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="FILE_MEMORY_PARTITION_INFORMATION", pack=False, align=None),
    "FILE_MODE_INFORMATION": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_MODE_INFORMATION", pack=False, align=None),
    "FILE_MOVE_CLUSTER_INFORMATION": SimStruct(OrderedDict((("ClusterCount", SimTypeInt(signed=False, label="UInt32")), ("RootDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_MOVE_CLUSTER_INFORMATION", pack=False, align=None),
    "FILE_NAMES_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NAMES_INFORMATION", pack=False, align=None),
    "FILE_NAME_INFO": SimStruct(OrderedDict((("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NAME_INFO", pack=False, align=None),
    "FILE_NAME_INFORMATION": SimStruct(OrderedDict((("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NAME_INFORMATION", pack=False, align=None),
    "FILE_NETWORK_OPEN_INFORMATION": SimStruct(OrderedDict((("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_NETWORK_OPEN_INFORMATION", pack=False, align=None),
    "FILE_NETWORK_PHYSICAL_NAME_INFORMATION": SimStruct(OrderedDict((("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NETWORK_PHYSICAL_NAME_INFORMATION", pack=False, align=None),
    "FILE_NOTIFY_CHANGE": SimTypeInt(signed=False, label="FILE_NOTIFY_CHANGE"),
    "FILE_NOTIFY_EXTENDED_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="FILE_ACTION")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastModificationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("AllocatedLength", SimTypeLongLong(signed=True, label="Int64")), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ReparsePointTag": SimTypeInt(signed=False, label="UInt32"), "EaSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("ParentFileId", SimTypeLongLong(signed=True, label="Int64")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NOTIFY_EXTENDED_INFORMATION", pack=False, align=None),
    "FILE_NOTIFY_FULL_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastModificationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("AllocatedLength", SimTypeLongLong(signed=True, label="Int64")), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ReparsePointTag": SimTypeInt(signed=False, label="UInt32"), "EaSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("ParentFileId", SimTypeLongLong(signed=True, label="Int64")), ("FileNameLength", SimTypeShort(signed=False, label="UInt16")), ("FileNameFlags", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NOTIFY_FULL_INFORMATION", pack=False, align=None),
    "FILE_NOTIFY_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="FILE_ACTION")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_NOTIFY_INFORMATION", pack=False, align=None),
    "FILE_NUMA_NODE_INFORMATION": SimStruct(OrderedDict((("NodeNumber", SimTypeShort(signed=False, label="UInt16")),)), name="FILE_NUMA_NODE_INFORMATION", pack=False, align=None),
    "FILE_OBJECT": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=True, label="Int16")), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("Vpb", SimTypePointer(SimTypeRef("VPB", SimStruct), offset=0)), ("FsContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FsContext2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SectionObjectPointer", SimTypePointer(SimTypeRef("SECTION_OBJECT_POINTERS", SimStruct), offset=0)), ("PrivateCacheMap", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FinalStatus", SimTypeInt(signed=True, label="Int32")), ("RelatedFileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("LockOperation", SimTypeChar(label="Byte")), ("DeletePending", SimTypeChar(label="Byte")), ("ReadAccess", SimTypeChar(label="Byte")), ("WriteAccess", SimTypeChar(label="Byte")), ("DeleteAccess", SimTypeChar(label="Byte")), ("SharedRead", SimTypeChar(label="Byte")), ("SharedWrite", SimTypeChar(label="Byte")), ("SharedDelete", SimTypeChar(label="Byte")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypeRef("UNICODE_STRING", SimStruct)), ("CurrentByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("Waiters", SimTypeInt(signed=False, label="UInt32")), ("Busy", SimTypeInt(signed=False, label="UInt32")), ("LastLock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Lock", SimTypeRef("KEVENT", SimStruct)), ("Event", SimTypeRef("KEVENT", SimStruct)), ("CompletionContext", SimTypePointer(SimTypeRef("IO_COMPLETION_CONTEXT", SimStruct), offset=0)), ("IrpListLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("IrpList", SimTypeRef("LIST_ENTRY", SimStruct)), ("FileObjectExtension", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FILE_OBJECT", pack=False, align=None),
    "FILE_OBJECTID_BUFFER": SimStruct(OrderedDict((("ObjectId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("BirthVolumeId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("BirthObjectId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("DomainId", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="_Anonymous_e__Struct", pack=False, align=None), "ExtendedInfo": SimTypeArray(SimTypeChar(label="Byte"), 48)}, name="<anon>", label="None")),)), name="FILE_OBJECTID_BUFFER", pack=False, align=None),
    "FILE_OBJECTID_INFORMATION": SimStruct(OrderedDict((("FileReference", SimTypeLongLong(signed=True, label="Int64")), ("ObjectId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("BirthVolumeId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("BirthObjectId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("DomainId", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="_Anonymous_e__Struct", pack=False, align=None), "ExtendedInfo": SimTypeArray(SimTypeChar(label="Byte"), 48)}, name="<anon>", label="None")),)), name="FILE_OBJECTID_INFORMATION", pack=False, align=None),
    "FILE_OPENMODE": SimTypeInt(signed=False, label="FILE_OPENMODE"),
    "FILE_OPERATION_FLAGS2": SimTypeInt(signed=False, label="FILE_OPERATION_FLAGS2"),
    "FILE_PIPE_ASSIGN_EVENT_BUFFER": SimStruct(OrderedDict((("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("KeyValue", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PIPE_ASSIGN_EVENT_BUFFER", pack=False, align=None),
    "FILE_PIPE_CLIENT_PROCESS_BUFFER": SimStruct(OrderedDict((("ClientSession", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ClientProcess", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FILE_PIPE_CLIENT_PROCESS_BUFFER", pack=False, align=None),
    "FILE_PIPE_CLIENT_PROCESS_BUFFER_EX": SimStruct(OrderedDict((("ClientSession", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ClientProcess", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ClientComputerNameLength", SimTypeShort(signed=False, label="UInt16")), ("ClientComputerBuffer", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="FILE_PIPE_CLIENT_PROCESS_BUFFER_EX", pack=False, align=None),
    "FILE_PIPE_CLIENT_PROCESS_BUFFER_V2": SimStruct(OrderedDict((("ClientSession", SimTypeLongLong(signed=False, label="UInt64")), ("ClientProcess", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FILE_PIPE_CLIENT_PROCESS_BUFFER_V2", pack=False, align=None),
    "FILE_PIPE_CREATE_SYMLINK_INPUT": SimStruct(OrderedDict((("NameOffset", SimTypeShort(signed=False, label="UInt16")), ("NameLength", SimTypeShort(signed=False, label="UInt16")), ("SubstituteNameOffset", SimTypeShort(signed=False, label="UInt16")), ("SubstituteNameLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PIPE_CREATE_SYMLINK_INPUT", pack=False, align=None),
    "FILE_PIPE_DELETE_SYMLINK_INPUT": SimStruct(OrderedDict((("NameOffset", SimTypeShort(signed=False, label="UInt16")), ("NameLength", SimTypeShort(signed=False, label="UInt16")),)), name="FILE_PIPE_DELETE_SYMLINK_INPUT", pack=False, align=None),
    "FILE_PIPE_EVENT_BUFFER": SimStruct(OrderedDict((("NamedPipeState", SimTypeInt(signed=False, label="UInt32")), ("EntryType", SimTypeInt(signed=False, label="UInt32")), ("ByteCount", SimTypeInt(signed=False, label="UInt32")), ("KeyValue", SimTypeInt(signed=False, label="UInt32")), ("NumberRequests", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PIPE_EVENT_BUFFER", pack=False, align=None),
    "FILE_PIPE_INFORMATION": SimStruct(OrderedDict((("ReadMode", SimTypeInt(signed=False, label="UInt32")), ("CompletionMode", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PIPE_INFORMATION", pack=False, align=None),
    "FILE_PIPE_LOCAL_INFORMATION": SimStruct(OrderedDict((("NamedPipeType", SimTypeInt(signed=False, label="UInt32")), ("NamedPipeConfiguration", SimTypeInt(signed=False, label="UInt32")), ("MaximumInstances", SimTypeInt(signed=False, label="UInt32")), ("CurrentInstances", SimTypeInt(signed=False, label="UInt32")), ("InboundQuota", SimTypeInt(signed=False, label="UInt32")), ("ReadDataAvailable", SimTypeInt(signed=False, label="UInt32")), ("OutboundQuota", SimTypeInt(signed=False, label="UInt32")), ("WriteQuotaAvailable", SimTypeInt(signed=False, label="UInt32")), ("NamedPipeState", SimTypeInt(signed=False, label="UInt32")), ("NamedPipeEnd", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PIPE_LOCAL_INFORMATION", pack=False, align=None),
    "FILE_PIPE_PEEK_BUFFER": SimStruct(OrderedDict((("NamedPipeState", SimTypeInt(signed=False, label="UInt32")), ("ReadDataAvailable", SimTypeInt(signed=False, label="UInt32")), ("NumberOfMessages", SimTypeInt(signed=False, label="UInt32")), ("MessageLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="FILE_PIPE_PEEK_BUFFER", pack=False, align=None),
    "FILE_PIPE_REMOTE_INFORMATION": SimStruct(OrderedDict((("CollectDataTime", SimTypeLongLong(signed=True, label="Int64")), ("MaximumCollectionCount", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PIPE_REMOTE_INFORMATION", pack=False, align=None),
    "FILE_PIPE_SILO_ARRIVAL_INPUT": SimStruct(OrderedDict((("JobHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="FILE_PIPE_SILO_ARRIVAL_INPUT", pack=False, align=None),
    "FILE_PIPE_WAIT_FOR_BUFFER": SimStruct(OrderedDict((("Timeout", SimTypeLongLong(signed=True, label="Int64")), ("NameLength", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSpecified", SimTypeChar(label="Byte")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_PIPE_WAIT_FOR_BUFFER", pack=False, align=None),
    "FILE_POSITION_INFORMATION": SimStruct(OrderedDict((("CurrentByteOffset", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_POSITION_INFORMATION", pack=False, align=None),
    "FILE_PREFETCH": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Prefetch", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="FILE_PREFETCH", pack=False, align=None),
    "FILE_PREFETCH_EX": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Prefetch", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="FILE_PREFETCH_EX", pack=False, align=None),
    "FILE_PROCESS_IDS_USING_FILE_INFORMATION": SimStruct(OrderedDict((("NumberOfProcessIdsInList", SimTypeInt(signed=False, label="UInt32")), ("ProcessIdList", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)),)), name="FILE_PROCESS_IDS_USING_FILE_INFORMATION", pack=False, align=None),
    "FILE_PROVIDER_EXTERNAL_INFO_V0": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Algorithm", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PROVIDER_EXTERNAL_INFO_V0", pack=False, align=None),
    "FILE_PROVIDER_EXTERNAL_INFO_V1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Algorithm", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_PROVIDER_EXTERNAL_INFO_V1", pack=False, align=None),
    "FILE_QUERY_ON_DISK_VOL_INFO_BUFFER": SimStruct(OrderedDict((("DirectoryCount", SimTypeLongLong(signed=True, label="Int64")), ("FileCount", SimTypeLongLong(signed=True, label="Int64")), ("FsFormatMajVersion", SimTypeShort(signed=False, label="UInt16")), ("FsFormatMinVersion", SimTypeShort(signed=False, label="UInt16")), ("FsFormatName", SimTypeArray(SimTypeChar(label="Char"), 12)), ("FormatTime", SimTypeLongLong(signed=True, label="Int64")), ("LastUpdateTime", SimTypeLongLong(signed=True, label="Int64")), ("CopyrightInfo", SimTypeArray(SimTypeChar(label="Char"), 34)), ("AbstractInfo", SimTypeArray(SimTypeChar(label="Char"), 34)), ("FormattingImplementationInfo", SimTypeArray(SimTypeChar(label="Char"), 34)), ("LastModifyingImplementationInfo", SimTypeArray(SimTypeChar(label="Char"), 34)),)), name="FILE_QUERY_ON_DISK_VOL_INFO_BUFFER", pack=False, align=None),
    "FILE_QUERY_SPARING_BUFFER": SimStruct(OrderedDict((("SparingUnitBytes", SimTypeInt(signed=False, label="UInt32")), ("SoftwareSparing", SimTypeChar(label="Byte")), ("TotalSpareBlocks", SimTypeInt(signed=False, label="UInt32")), ("FreeSpareBlocks", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_QUERY_SPARING_BUFFER", pack=False, align=None),
    "FILE_QUOTA_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("SidLength", SimTypeInt(signed=False, label="UInt32")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("QuotaUsed", SimTypeLongLong(signed=True, label="Int64")), ("QuotaThreshold", SimTypeLongLong(signed=True, label="Int64")), ("QuotaLimit", SimTypeLongLong(signed=True, label="Int64")), ("Sid", SimTypeRef("SID", SimStruct)),)), name="FILE_QUOTA_INFORMATION", pack=False, align=None),
    "FILE_REFERENCE_RANGE": SimStruct(OrderedDict((("StartingFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("EndingFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILE_REFERENCE_RANGE", pack=False, align=None),
    "FILE_REGION_INFO": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("Usage", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_REGION_INFO", pack=False, align=None),
    "FILE_REGION_INPUT": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("DesiredUsage", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_REGION_INPUT", pack=False, align=None),
    "FILE_REGION_OUTPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("TotalRegionEntryCount", SimTypeInt(signed=False, label="UInt32")), ("RegionEntryCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Region", SimTypePointer(SimTypeRef("FILE_REGION_INFO", SimStruct), offset=0)),)), name="FILE_REGION_OUTPUT", pack=False, align=None),
    "FILE_REMOTE_PROTOCOL_INFO": SimStruct(OrderedDict((("StructureVersion", SimTypeShort(signed=False, label="UInt16")), ("StructureSize", SimTypeShort(signed=False, label="UInt16")), ("Protocol", SimTypeInt(signed=False, label="UInt32")), ("ProtocolMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("ProtocolMinorVersion", SimTypeShort(signed=False, label="UInt16")), ("ProtocolRevision", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("GenericReserved", SimTypeRef("_GenericReserved_e__Struct", SimStruct)), ("ProtocolSpecific", SimUnion({"Smb2": SimTypeRef("_Smb2_e__Struct", SimStruct), "Reserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)}, name="<anon>", label="None")),)), name="FILE_REMOTE_PROTOCOL_INFO", pack=False, align=None),
    "FILE_REMOTE_PROTOCOL_INFORMATION": SimStruct(OrderedDict((("StructureVersion", SimTypeShort(signed=False, label="UInt16")), ("StructureSize", SimTypeShort(signed=False, label="UInt16")), ("Protocol", SimTypeInt(signed=False, label="UInt32")), ("ProtocolMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("ProtocolMinorVersion", SimTypeShort(signed=False, label="UInt16")), ("ProtocolRevision", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("GenericReserved", SimTypeRef("_GenericReserved_e__Struct", SimStruct)), ("ProtocolSpecific", SimUnion({"Smb2": SimTypeRef("_Smb2_e__Struct", SimStruct), "Reserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)}, name="<anon>", label="None")),)), name="FILE_REMOTE_PROTOCOL_INFORMATION", pack=False, align=None),
    "FILE_RENAME_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"ReplaceIfExists": SimTypeChar(label="Byte"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RootDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_RENAME_INFO", pack=False, align=None),
    "FILE_RENAME_INFORMATION": SimStruct(OrderedDict((("Anonymous", SimUnion({"ReplaceIfExists": SimTypeChar(label="Byte"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RootDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_RENAME_INFORMATION", pack=False, align=None),
    "FILE_REPARSE_POINT_INFORMATION": SimStruct(OrderedDict((("FileReference", SimTypeLongLong(signed=True, label="Int64")), ("Tag", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_REPARSE_POINT_INFORMATION", pack=False, align=None),
    "FILE_SEGMENT_ELEMENT": SimUnion({"Buffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Alignment": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "FILE_SET_DEFECT_MGMT_BUFFER": SimStruct(OrderedDict((("Disable", SimTypeChar(label="Byte")),)), name="FILE_SET_DEFECT_MGMT_BUFFER", pack=False, align=None),
    "FILE_SET_SPARSE_BUFFER": SimStruct(OrderedDict((("SetSparse", SimTypeChar(label="Byte")),)), name="FILE_SET_SPARSE_BUFFER", pack=False, align=None),
    "FILE_SFIO_RESERVE_INFORMATION": SimStruct(OrderedDict((("RequestsPerPeriod", SimTypeInt(signed=False, label="UInt32")), ("Period", SimTypeInt(signed=False, label="UInt32")), ("RetryFailures", SimTypeChar(label="Byte")), ("Discardable", SimTypeChar(label="Byte")), ("RequestSize", SimTypeInt(signed=False, label="UInt32")), ("NumOutstandingRequests", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_SFIO_RESERVE_INFORMATION", pack=False, align=None),
    "FILE_SFIO_VOLUME_INFORMATION": SimStruct(OrderedDict((("MaximumRequestsPerPeriod", SimTypeInt(signed=False, label="UInt32")), ("MinimumPeriod", SimTypeInt(signed=False, label="UInt32")), ("MinimumTransferSize", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_SFIO_VOLUME_INFORMATION", pack=False, align=None),
    "FILE_SHARE_MODE": SimTypeInt(signed=False, label="FILE_SHARE_MODE"),
    "FILE_STANDARD_INFO": SimStruct(OrderedDict((("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("NumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("DeletePending", SimTypeChar(label="Byte")), ("Directory", SimTypeChar(label="Byte")),)), name="FILE_STANDARD_INFO", pack=False, align=None),
    "FILE_STANDARD_INFORMATION": SimStruct(OrderedDict((("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("NumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("DeletePending", SimTypeChar(label="Byte")), ("Directory", SimTypeChar(label="Byte")),)), name="FILE_STANDARD_INFORMATION", pack=False, align=None),
    "FILE_STANDARD_INFORMATION_EX": SimStruct(OrderedDict((("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("NumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("DeletePending", SimTypeChar(label="Byte")), ("Directory", SimTypeChar(label="Byte")), ("AlternateStream", SimTypeChar(label="Byte")), ("MetadataAttribute", SimTypeChar(label="Byte")),)), name="FILE_STANDARD_INFORMATION_EX", pack=False, align=None),
    "FILE_STANDARD_LINK_INFORMATION": SimStruct(OrderedDict((("NumberOfAccessibleLinks", SimTypeInt(signed=False, label="UInt32")), ("TotalNumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("DeletePending", SimTypeChar(label="Byte")), ("Directory", SimTypeChar(label="Byte")),)), name="FILE_STANDARD_LINK_INFORMATION", pack=False, align=None),
    "FILE_STAT_INFORMATION": SimStruct(OrderedDict((("FileId", SimTypeLongLong(signed=True, label="Int64")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ReparseTag", SimTypeInt(signed=False, label="UInt32")), ("NumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("EffectiveAccess", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_STAT_INFORMATION", pack=False, align=None),
    "FILE_STAT_LX_INFORMATION": SimStruct(OrderedDict((("FileId", SimTypeLongLong(signed=True, label="Int64")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ReparseTag", SimTypeInt(signed=False, label="UInt32")), ("NumberOfLinks", SimTypeInt(signed=False, label="UInt32")), ("EffectiveAccess", SimTypeInt(signed=False, label="UInt32")), ("LxFlags", SimTypeInt(signed=False, label="UInt32")), ("LxUid", SimTypeInt(signed=False, label="UInt32")), ("LxGid", SimTypeInt(signed=False, label="UInt32")), ("LxMode", SimTypeInt(signed=False, label="UInt32")), ("LxDeviceIdMajor", SimTypeInt(signed=False, label="UInt32")), ("LxDeviceIdMinor", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_STAT_LX_INFORMATION", pack=False, align=None),
    "FILE_STORAGE_INFO": SimStruct(OrderedDict((("LogicalBytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("PhysicalBytesPerSectorForAtomicity", SimTypeInt(signed=False, label="UInt32")), ("PhysicalBytesPerSectorForPerformance", SimTypeInt(signed=False, label="UInt32")), ("FileSystemEffectivePhysicalBytesPerSectorForAtomicity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ByteOffsetForSectorAlignment", SimTypeInt(signed=False, label="UInt32")), ("ByteOffsetForPartitionAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_STORAGE_INFO", pack=False, align=None),
    "FILE_STORAGE_RESERVE_ID_INFORMATION": SimStruct(OrderedDict((("StorageReserveId", SimTypeInt(signed=False, label="STORAGE_RESERVE_ID")),)), name="FILE_STORAGE_RESERVE_ID_INFORMATION", pack=False, align=None),
    "FILE_STORAGE_TIER": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("Name", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("ProvisionedCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("MediaType", SimTypeInt(signed=False, label="FILE_STORAGE_TIER_MEDIA_TYPE")), ("Class", SimTypeInt(signed=False, label="FILE_STORAGE_TIER_CLASS")),)), name="FILE_STORAGE_TIER", pack=False, align=None),
    "FILE_STORAGE_TIER_CLASS": SimTypeInt(signed=False, label="FILE_STORAGE_TIER_CLASS"),
    "FILE_STORAGE_TIER_FLAGS": SimTypeInt(signed=False, label="FILE_STORAGE_TIER_FLAGS"),
    "FILE_STORAGE_TIER_MEDIA_TYPE": SimTypeInt(signed=False, label="FILE_STORAGE_TIER_MEDIA_TYPE"),
    "FILE_STORAGE_TIER_REGION": SimStruct(OrderedDict((("TierId", SimTypeBottom(label="Guid")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILE_STORAGE_TIER_REGION", pack=False, align=None),
    "FILE_STREAM_INFO": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("StreamNameLength", SimTypeInt(signed=False, label="UInt32")), ("StreamSize", SimTypeLongLong(signed=True, label="Int64")), ("StreamAllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("StreamName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_STREAM_INFO", pack=False, align=None),
    "FILE_STREAM_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("StreamNameLength", SimTypeInt(signed=False, label="UInt32")), ("StreamSize", SimTypeLongLong(signed=True, label="Int64")), ("StreamAllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("StreamName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_STREAM_INFORMATION", pack=False, align=None),
    "FILE_SYSTEM_RECOGNITION_INFORMATION": SimStruct(OrderedDict((("FileSystem", SimTypeArray(SimTypeChar(label="SByte"), 9)),)), name="FILE_SYSTEM_RECOGNITION_INFORMATION", pack=False, align=None),
    "FILE_TIMESTAMPS": SimStruct(OrderedDict((("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_TIMESTAMPS", pack=False, align=None),
    "FILE_TRACKING_INFORMATION": SimStruct(OrderedDict((("DestinationFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ObjectInformationLength", SimTypeInt(signed=False, label="UInt32")), ("ObjectInformation", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="FILE_TRACKING_INFORMATION", pack=False, align=None),
    "FILE_TYPE": SimTypeInt(signed=False, label="FILE_TYPE"),
    "FILE_TYPE_NOTIFICATION_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumFileTypeIDs", SimTypeInt(signed=False, label="UInt32")), ("FileTypeID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="FILE_TYPE_NOTIFICATION_INPUT", pack=False, align=None),
    "FILE_UNPROTECT_OPTIONS": SimStruct(OrderedDict((("audit", SimTypeChar(label="Byte")),)), name="FILE_UNPROTECT_OPTIONS", pack=False, align=None),
    "FILE_USAGE_TYPE": SimTypeInt(signed=False, label="FILE_USAGE_TYPE"),
    "FILE_VALID_DATA_LENGTH_INFORMATION": SimStruct(OrderedDict((("ValidDataLength", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_VALID_DATA_LENGTH_INFORMATION", pack=False, align=None),
    "FILE_VOLUME_NAME_INFORMATION": SimStruct(OrderedDict((("DeviceNameLength", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILE_VOLUME_NAME_INFORMATION", pack=False, align=None),
    "FILE_WRITE_FLAGS": SimTypeInt(signed=False, label="FILE_WRITE_FLAGS"),
    "FILE_ZERO_DATA_INFORMATION": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("BeyondFinalZero", SimTypeLongLong(signed=True, label="Int64")),)), name="FILE_ZERO_DATA_INFORMATION", pack=False, align=None),
    "FILE_ZERO_DATA_INFORMATION_EX": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("BeyondFinalZero", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FILE_ZERO_DATA_INFORMATION_EX", pack=False, align=None),
    "FILLSTATES": SimTypeInt(signed=False, label="FILLSTATES"),
    "FILLTYPE": SimTypeInt(signed=False, label="FILLTYPE"),
    "FILLVERTSTATES": SimTypeInt(signed=False, label="FILLVERTSTATES"),
    "FILTERDBS": SimStruct(OrderedDict((("Anonymous", SimUnion({"EthDB": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), "NullDB": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")), ("TrDB", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("YYYDB", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("XXXDB", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FILTERDBS", pack=False, align=None),
    "FILTERED_DATA_SOURCES": SimStruct(OrderedDict((("pwcsExtension", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwcsMime", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pClsid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pwcsOverride", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILTERED_DATA_SOURCES", pack=False, align=None),
    "FILTERING_TYPE": SimTypeInt(signed=False, label="FILTERING_TYPE"),
    "FILTERKEYS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iWaitMSec", SimTypeInt(signed=False, label="UInt32")), ("iDelayMSec", SimTypeInt(signed=False, label="UInt32")), ("iRepeatMSec", SimTypeInt(signed=False, label="UInt32")), ("iBounceMSec", SimTypeInt(signed=False, label="UInt32")),)), name="FILTERKEYS", pack=False, align=None),
    "FILTERREGION": SimStruct(OrderedDict((("idChunk", SimTypeInt(signed=False, label="UInt32")), ("cwcStart", SimTypeInt(signed=False, label="UInt32")), ("cwcExtent", SimTypeInt(signed=False, label="UInt32")),)), name="FILTERREGION", pack=False, align=None),
    "FILTER_AGGREGATE_BASIC_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Type", SimUnion({"MiniFilter": SimTypeRef("_MiniFilter_e__Struct", SimStruct), "LegacyFilter": SimTypeRef("_LegacyFilter_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FILTER_AGGREGATE_BASIC_INFORMATION", pack=False, align=None),
    "FILTER_AGGREGATE_STANDARD_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Type", SimUnion({"MiniFilter": SimTypeRef("_MiniFilter_e__Struct", SimStruct), "LegacyFilter": SimTypeRef("_LegacyFilter_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FILTER_AGGREGATE_STANDARD_INFORMATION", pack=False, align=None),
    "FILTER_COMBINATION_TYPE": SimTypeInt(signed=False, label="FILTER_COMBINATION_TYPE"),
    "FILTER_FULL_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FrameID", SimTypeInt(signed=False, label="UInt32")), ("NumberOfInstances", SimTypeInt(signed=False, label="UInt32")), ("FilterNameLength", SimTypeShort(signed=False, label="UInt16")), ("FilterNameBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILTER_FULL_INFORMATION", pack=False, align=None),
    "FILTER_INFO": SimStruct(OrderedDict((("achName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("pGraph", SimTypeBottom(label="IFilterGraph")),)), name="FILTER_INFO", pack=False, align=None),
    "FILTER_INFORMATION_CLASS": SimTypeInt(signed=False, label="FILTER_INFORMATION_CLASS"),
    "FILTER_MESSAGE_HEADER": SimStruct(OrderedDict((("ReplyLength", SimTypeInt(signed=False, label="UInt32")), ("MessageId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILTER_MESSAGE_HEADER", pack=False, align=None),
    "FILTER_REPLY_HEADER": SimStruct(OrderedDict((("Status", SimTypeInt(signed=True, label="Int32")), ("MessageId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FILTER_REPLY_HEADER", pack=False, align=None),
    "FILTER_SPEC": SimStruct(OrderedDict((("filt_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("filt_u", SimUnion({"filt_ipv4": SimTypeRef("Filter_Spec_IPv4", SimStruct), "filt_ipv4gpi": SimTypeRef("Filter_Spec_IPv4GPI", SimStruct)}, name="<anon>", label="None")),)), name="FILTER_SPEC", pack=False, align=None),
    "FILTER_STATE": SimTypeInt(signed=False, label="FILTER_STATE"),
    "FILTER_VOLUME_BASIC_INFORMATION": SimStruct(OrderedDict((("FilterVolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("FilterVolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILTER_VOLUME_BASIC_INFORMATION", pack=False, align=None),
    "FILTER_VOLUME_INFORMATION_CLASS": SimTypeInt(signed=False, label="FILTER_VOLUME_INFORMATION_CLASS"),
    "FILTER_VOLUME_STANDARD_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FrameID", SimTypeInt(signed=False, label="UInt32")), ("FileSystemType", SimTypeInt(signed=False, label="FLT_FILESYSTEM_TYPE")), ("FilterVolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("FilterVolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FILTER_VOLUME_STANDARD_INFORMATION", pack=False, align=None),
    "FINDEX_INFO_LEVELS": SimTypeInt(signed=False, label="FINDEX_INFO_LEVELS"),
    "FINDEX_SEARCH_OPS": SimTypeInt(signed=False, label="FINDEX_SEARCH_OPS"),
    "FINDFRAME_FLAGS": SimTypeInt(signed=False, label="FINDFRAME_FLAGS"),
    "FINDREPLACEA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="FINDREPLACE_FLAGS")), ("lpstrFindWhat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrReplaceWith", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wFindWhatLen", SimTypeShort(signed=False, label="UInt16")), ("wReplaceWithLen", SimTypeShort(signed=False, label="UInt16")), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FINDREPLACEA", pack=False, align=None),
    "FINDREPLACEW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="FINDREPLACE_FLAGS")), ("lpstrFindWhat", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrReplaceWith", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wFindWhatLen", SimTypeShort(signed=False, label="UInt16")), ("wReplaceWithLen", SimTypeShort(signed=False, label="UInt16")), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FINDREPLACEW", pack=False, align=None),
    "FINDREPLACE_FLAGS": SimTypeInt(signed=False, label="FINDREPLACE_FLAGS"),
    "FINDTEXTA": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("lpstrText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FINDTEXTA", pack=False, align=None),
    "FINDTEXTEXA": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("lpstrText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("chrgText", SimTypeRef("CHARRANGE", SimStruct)),)), name="FINDTEXTEXA", pack=False, align=None),
    "FINDTEXTEXW": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("lpstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("chrgText", SimTypeRef("CHARRANGE", SimStruct)),)), name="FINDTEXTEXW", pack=False, align=None),
    "FINDTEXTW": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("lpstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FINDTEXTW", pack=False, align=None),
    "FINDTEXT_FLAGS": SimTypeInt(signed=False, label="FINDTEXT_FLAGS"),
    "FIND_BY_SID_DATA": SimStruct(OrderedDict((("Restart", SimTypeInt(signed=False, label="UInt32")), ("Sid", SimTypeRef("SID", SimStruct)),)), name="FIND_BY_SID_DATA", pack=False, align=None),
    "FIND_BY_SID_OUTPUT": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FIND_BY_SID_OUTPUT", pack=False, align=None),
    "FIND_FIRST_EX_FLAGS": SimTypeInt(signed=False, label="FIND_FIRST_EX_FLAGS"),
    "FIND_NAME_BUFFER": SimStruct(OrderedDict((("length", SimTypeChar(label="Byte")), ("access_control", SimTypeChar(label="Byte")), ("frame_control", SimTypeChar(label="Byte")), ("destination_addr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("source_addr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("routing_info", SimTypeArray(SimTypeChar(label="Byte"), 18)),)), name="FIND_NAME_BUFFER", pack=False, align=None),
    "FIND_NAME_HEADER": SimStruct(OrderedDict((("node_count", SimTypeShort(signed=False, label="UInt16")), ("reserved", SimTypeChar(label="Byte")), ("unique_group", SimTypeChar(label="Byte")),)), name="FIND_NAME_HEADER", pack=False, align=None),
    "FINISH_MODE": SimTypeInt(signed=False, label="FINISH_MODE"),
    "FIO_CONTEXT": SimStruct(OrderedDict((("m_dwTempHack", SimTypeInt(signed=False, label="UInt32")), ("m_dwSignature", SimTypeInt(signed=False, label="UInt32")), ("m_hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("m_dwLinesOffset", SimTypeInt(signed=False, label="UInt32")), ("m_dwHeaderLength", SimTypeInt(signed=False, label="UInt32")),)), name="FIO_CONTEXT", pack=False, align=None),
    "FIRMWARE_ACTIVATION_HISTORY_ENTRY": SimStruct(OrderedDict((("VersionNumber", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("Reserved0", SimTypeShort(signed=False, label="UInt16")), ("ActivationCount", SimTypeShort(signed=False, label="UInt16")), ("Timestamp", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("PowerCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("PreviousFirmware", SimTypeLongLong(signed=False, label="UInt64")), ("NewFirmware", SimTypeLongLong(signed=False, label="UInt64")), ("SlotNumber", SimTypeChar(label="Byte")), ("CommitActionType", SimTypeChar(label="Byte")), ("Result", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 14)),)), name="FIRMWARE_ACTIVATION_HISTORY_ENTRY", pack=False, align=None),
    "FIRMWARE_REQUEST_BLOCK": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DataBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("DataBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="FIRMWARE_REQUEST_BLOCK", pack=False, align=None),
    "FIRMWARE_TABLE_PROVIDER": SimTypeInt(signed=False, label="FIRMWARE_TABLE_PROVIDER"),
    "FIRMWARE_TYPE": SimTypeInt(signed=False, label="FIRMWARE_TYPE"),
    "FIXED": SimStruct(OrderedDict((("fract", SimTypeShort(signed=False, label="UInt16")), ("value", SimTypeShort(signed=True, label="Int16")),)), name="FIXED", pack=False, align=None),
    "FIXED_INFO_W2KSP1": SimStruct(OrderedDict((("HostName", SimTypeArray(SimTypeChar(label="SByte"), 132)), ("DomainName", SimTypeArray(SimTypeChar(label="SByte"), 132)), ("CurrentDnsServer", SimTypePointer(SimTypeRef("IP_ADDR_STRING", SimStruct), offset=0)), ("DnsServerList", SimTypeRef("IP_ADDR_STRING", SimStruct)), ("NodeType", SimTypeInt(signed=False, label="UInt32")), ("ScopeId", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("EnableRouting", SimTypeInt(signed=False, label="UInt32")), ("EnableProxy", SimTypeInt(signed=False, label="UInt32")), ("EnableDns", SimTypeInt(signed=False, label="UInt32")),)), name="FIXED_INFO_W2KSP1", pack=False, align=None),
    "FLAGGED_BYTE_BLOB": SimStruct(OrderedDict((("fFlags", SimTypeInt(signed=False, label="UInt32")), ("clSize", SimTypeInt(signed=False, label="UInt32")), ("abData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FLAGGED_BYTE_BLOB", pack=False, align=None),
    "FLAGGED_WORD_BLOB": SimStruct(OrderedDict((("fFlags", SimTypeInt(signed=False, label="UInt32")), ("clSize", SimTypeInt(signed=False, label="UInt32")), ("asData", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="FLAGGED_WORD_BLOB", pack=False, align=None),
    "FLAG_STGMEDIUM": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=True, label="Int32")), ("fPassOwnership", SimTypeInt(signed=True, label="Int32")), ("Stgmed", SimTypeRef("STGMEDIUM", SimStruct)),)), name="FLAG_STGMEDIUM", pack=False, align=None),
    "FLASHWINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="FLASHWINFO_FLAGS")), ("uCount", SimTypeInt(signed=False, label="UInt32")), ("dwTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="FLASHWINFO", pack=False, align=None),
    "FLASHWINFO_FLAGS": SimTypeInt(signed=False, label="FLASHWINFO_FLAGS"),
    "FLATENTRY": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("abEntry", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FLATENTRY", pack=False, align=None),
    "FLATENTRYLIST": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("cbEntries", SimTypeInt(signed=False, label="UInt32")), ("abEntries", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FLATENTRYLIST", pack=False, align=None),
    "FLATMTSIDLIST": SimStruct(OrderedDict((("cMTSIDs", SimTypeInt(signed=False, label="UInt32")), ("cbMTSIDs", SimTypeInt(signed=False, label="UInt32")), ("abMTSIDs", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FLATMTSIDLIST", pack=False, align=None),
    "FLAT_STRING": SimStruct(OrderedDict((("MaximumLength", SimTypeShort(signed=True, label="Int16")), ("Length", SimTypeShort(signed=True, label="Int16")), ("Buffer", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="FLAT_STRING", pack=False, align=None),
    "FLICKACTION_COMMANDCODE": SimTypeInt(signed=False, label="FLICKACTION_COMMANDCODE"),
    "FLICKDIRECTION": SimTypeInt(signed=False, label="FLICKDIRECTION"),
    "FLICKMODE": SimTypeInt(signed=False, label="FLICKMODE"),
    "FLICK_DATA": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="FLICK_DATA", pack=False, align=None),
    "FLICK_POINT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="FLICK_POINT", pack=False, align=None),
    "FLOAT128": SimStruct(OrderedDict((("LowPart", SimTypeLongLong(signed=True, label="Int64")), ("HighPart", SimTypeLongLong(signed=True, label="Int64")),)), name="FLOAT128", pack=False, align=None),
    "FLOATING_SAVE_AREA": SimStruct(OrderedDict((("ControlWord", SimTypeInt(signed=False, label="UInt32")), ("StatusWord", SimTypeInt(signed=False, label="UInt32")), ("TagWord", SimTypeInt(signed=False, label="UInt32")), ("ErrorOffset", SimTypeInt(signed=False, label="UInt32")), ("ErrorSelector", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSelector", SimTypeInt(signed=False, label="UInt32")), ("RegisterArea", SimTypeArray(SimTypeChar(label="Byte"), 80)), ("Spare0", SimTypeInt(signed=False, label="UInt32")),)), name="FLOATING_SAVE_AREA", pack=False, align=None),
    "FLOATOBJ": SimStruct(OrderedDict((("ul1", SimTypeInt(signed=False, label="UInt32")), ("ul2", SimTypeInt(signed=False, label="UInt32")),)), name="FLOATOBJ", pack=False, align=None),
    "FLOATOBJ_XFORM": SimStruct(OrderedDict((("eM11", SimTypeRef("FLOATOBJ", SimStruct)), ("eM12", SimTypeRef("FLOATOBJ", SimStruct)), ("eM21", SimTypeRef("FLOATOBJ", SimStruct)), ("eM22", SimTypeRef("FLOATOBJ", SimStruct)), ("eDx", SimTypeRef("FLOATOBJ", SimStruct)), ("eDy", SimTypeRef("FLOATOBJ", SimStruct)),)), name="FLOATOBJ_XFORM", pack=False, align=None),
    "FLOAT_LONG": SimUnion({"e": SimTypeInt(signed=False, label="UInt32"), "l": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"),
    "FLOWDESCRIPTOR": SimStruct(OrderedDict((("FlowSpec", SimTypeRef("FLOWSPEC", SimStruct)), ("NumFilters", SimTypeInt(signed=False, label="UInt32")), ("FilterList", SimTypePointer(SimTypeRef("RSVP_FILTERSPEC", SimStruct), offset=0)),)), name="FLOWDESCRIPTOR", pack=False, align=None),
    "FLOWSPEC": SimStruct(OrderedDict((("TokenRate", SimTypeInt(signed=False, label="UInt32")), ("TokenBucketSize", SimTypeInt(signed=False, label="UInt32")), ("PeakBandwidth", SimTypeInt(signed=False, label="UInt32")), ("Latency", SimTypeInt(signed=False, label="UInt32")), ("DelayVariation", SimTypeInt(signed=False, label="UInt32")), ("ServiceType", SimTypeInt(signed=False, label="UInt32")), ("MaxSduSize", SimTypeInt(signed=False, label="UInt32")), ("MinimumPolicedSize", SimTypeInt(signed=False, label="UInt32")),)), name="FLOWSPEC", pack=False, align=None),
    "FLOW_DESC": SimStruct(OrderedDict((("u1", SimUnion({"stspec": SimTypePointer(SimTypeRef("SENDER_TSPEC", SimStruct), offset=0), "isflow": SimTypePointer(SimTypeRef("IS_FLOWSPEC", SimStruct), offset=0)}, name="<anon>", label="None")), ("u2", SimUnion({"stemp": SimTypePointer(SimTypeRef("FILTER_SPEC", SimStruct), offset=0), "fspec": SimTypePointer(SimTypeRef("FILTER_SPEC", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FLOW_DESC", pack=False, align=None),
    "FLT_CALLBACK_DATA": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Thread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Iopb", SimTypePointer(SimTypeRef("FLT_IO_PARAMETER_BLOCK", SimStruct), offset=0)), ("IoStatus", SimTypeRef("IO_STATUS_BLOCK", SimStruct)), ("TagData", SimTypePointer(SimTypeRef("FLT_TAG_DATA_BUFFER", SimStruct), offset=0)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("QueueLinks", SimTypeRef("LIST_ENTRY", SimStruct)), ("QueueContext", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)),)), name="_Anonymous_e__Struct", pack=False, align=None), "FilterContext": SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 4)}, name="<anon>", label="None")), ("RequestorMode", SimTypeChar(label="SByte")),)), name="FLT_CALLBACK_DATA", pack=False, align=None),
    "FLT_CALLBACK_DATA_QUEUE": SimStruct(OrderedDict((("Csq", SimTypeRef("IO_CSQ", SimStruct)), ("Flags", SimTypeInt(signed=False, label="FLT_CALLBACK_DATA_QUEUE_FLAGS")), ("Instance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InsertIo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Cbdq", "Cbd", "InsertContext"]), offset=0)), ("RemoveIo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Cbd"]), offset=0)), ("PeekNextIo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), arg_names=["Cbdq", "Cbd", "PeekContext"]), offset=0)), ("Acquire", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Irql"]), offset=0)), ("Release", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Irql"]), offset=0)), ("CompleteCanceledIo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Cbd"]), offset=0)),)), name="FLT_CALLBACK_DATA_QUEUE", pack=False, align=None),
    "FLT_CALLBACK_DATA_QUEUE_FLAGS": SimTypeInt(signed=False, label="FLT_CALLBACK_DATA_QUEUE_FLAGS"),
    "FLT_CONTEXT_REGISTRATION": SimStruct(OrderedDict((("ContextType", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("ContextCleanupCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["Context", "ContextType"]), offset=0)), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PoolTag", SimTypeInt(signed=False, label="UInt32")), ("ContextAllocateCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="POOL_TYPE"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PoolType", "Size", "ContextType"]), offset=0)), ("ContextFreeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["Pool", "ContextType"]), offset=0)), ("Reserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FLT_CONTEXT_REGISTRATION", pack=False, align=None),
    "FLT_CREATEFILE_TARGET_ECP_CONTEXT": SimStruct(OrderedDict((("Instance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Volume", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FileNameInformation", SimTypePointer(SimTypeRef("FLT_FILE_NAME_INFORMATION", SimStruct), offset=0)), ("Flags", SimTypeShort(signed=False, label="UInt16")),)), name="FLT_CREATEFILE_TARGET_ECP_CONTEXT", pack=False, align=None),
    "FLT_FILESYSTEM_TYPE": SimTypeInt(signed=False, label="FLT_FILESYSTEM_TYPE"),
    "FLT_FILE_NAME_INFORMATION": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("NamesParsed", SimTypeShort(signed=False, label="UInt16")), ("Format", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypeRef("UNICODE_STRING", SimStruct)), ("Volume", SimTypeRef("UNICODE_STRING", SimStruct)), ("Share", SimTypeRef("UNICODE_STRING", SimStruct)), ("Extension", SimTypeRef("UNICODE_STRING", SimStruct)), ("Stream", SimTypeRef("UNICODE_STRING", SimStruct)), ("FinalComponent", SimTypeRef("UNICODE_STRING", SimStruct)), ("ParentDir", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="FLT_FILE_NAME_INFORMATION", pack=False, align=None),
    "FLT_IO_PARAMETER_BLOCK": SimStruct(OrderedDict((("IrpFlags", SimTypeInt(signed=False, label="UInt32")), ("MajorFunction", SimTypeChar(label="Byte")), ("MinorFunction", SimTypeChar(label="Byte")), ("OperationFlags", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TargetFileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("TargetInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Parameters", SimUnion({"Create": SimTypeRef("_Create_e__Struct", SimStruct), "CreatePipe": SimTypeRef("_CreatePipe_e__Struct", SimStruct), "CreateMailslot": SimTypeRef("_CreateMailslot_e__Struct", SimStruct), "Read": SimTypeRef("_Read_e__Struct", SimStruct), "Write": SimTypeRef("_Write_e__Struct", SimStruct), "QueryFileInformation": SimTypeRef("_QueryFileInformation_e__Struct", SimStruct), "SetFileInformation": SimTypeRef("_SetFileInformation_e__Struct", SimStruct), "QueryEa": SimTypeRef("_QueryEa_e__Struct", SimStruct), "SetEa": SimTypeRef("_SetEa_e__Struct", SimStruct), "QueryVolumeInformation": SimTypeRef("_QueryVolumeInformation_e__Struct", SimStruct), "SetVolumeInformation": SimTypeRef("_SetVolumeInformation_e__Struct", SimStruct), "DirectoryControl": SimUnion({"QueryDirectory": SimTypeRef("_QueryDirectory_e__Struct", SimStruct), "NotifyDirectory": SimTypeRef("_NotifyDirectory_e__Struct", SimStruct), "NotifyDirectoryEx": SimTypeRef("_NotifyDirectoryEx_e__Struct", SimStruct)}, name="<anon>", label="None"), "FileSystemControl": SimUnion({"VerifyVolume": SimTypeRef("_VerifyVolume_e__Struct", SimStruct), "Common": SimTypeRef("_Common_e__Struct", SimStruct), "Neither": SimTypeRef("_Neither_e__Struct", SimStruct), "Buffered": SimTypeRef("_Buffered_e__Struct", SimStruct), "Direct": SimTypeRef("_Direct_e__Struct", SimStruct)}, name="<anon>", label="None"), "DeviceIoControl": SimUnion({"Common": SimTypeRef("_Common_e__Struct", SimStruct), "Neither": SimTypeRef("_Neither_e__Struct", SimStruct), "Buffered": SimTypeRef("_Buffered_e__Struct", SimStruct), "Direct": SimTypeRef("_Direct_e__Struct", SimStruct), "FastIo": SimTypeRef("_FastIo_e__Struct", SimStruct)}, name="<anon>", label="None"), "LockControl": SimTypeRef("_LockControl_e__Struct", SimStruct), "QuerySecurity": SimTypeRef("_QuerySecurity_e__Struct", SimStruct), "SetSecurity": SimTypeRef("_SetSecurity_e__Struct", SimStruct), "WMI": SimTypeRef("_WMI_e__Struct", SimStruct), "QueryQuota": SimTypeRef("_QueryQuota_e__Struct", SimStruct), "SetQuota": SimTypeRef("_SetQuota_e__Struct", SimStruct), "Pnp": SimUnion({"StartDevice": SimTypeRef("_StartDevice_e__Struct", SimStruct), "QueryDeviceRelations": SimTypeRef("_QueryDeviceRelations_e__Struct", SimStruct), "QueryInterface": SimTypeRef("_QueryInterface_e__Struct", SimStruct), "DeviceCapabilities": SimTypeRef("_DeviceCapabilities_e__Struct", SimStruct), "FilterResourceRequirements": SimTypeRef("_FilterResourceRequirements_e__Struct", SimStruct), "ReadWriteConfig": SimTypeRef("_ReadWriteConfig_e__Struct", SimStruct), "SetLock": SimTypeRef("_SetLock_e__Struct", SimStruct), "QueryId": SimTypeRef("_QueryId_e__Struct", SimStruct), "QueryDeviceText": SimTypeRef("_QueryDeviceText_e__Struct", SimStruct), "UsageNotification": SimTypeRef("_UsageNotification_e__Struct", SimStruct)}, name="<anon>", label="None"), "AcquireForSectionSynchronization": SimTypeRef("_AcquireForSectionSynchronization_e__Struct", SimStruct), "AcquireForModifiedPageWriter": SimTypeRef("_AcquireForModifiedPageWriter_e__Struct", SimStruct), "ReleaseForModifiedPageWriter": SimTypeRef("_ReleaseForModifiedPageWriter_e__Struct", SimStruct), "QueryOpen": SimTypeRef("_QueryOpen_e__Struct", SimStruct), "FastIoCheckIfPossible": SimTypeRef("_FastIoCheckIfPossible_e__Struct", SimStruct), "NetworkQueryOpen": SimTypeRef("_NetworkQueryOpen_e__Struct", SimStruct), "MdlRead": SimTypeRef("_MdlRead_e__Struct", SimStruct), "MdlReadComplete": SimTypeRef("_MdlReadComplete_e__Struct", SimStruct), "PrepareMdlWrite": SimTypeRef("_PrepareMdlWrite_e__Struct", SimStruct), "MdlWriteComplete": SimTypeRef("_MdlWriteComplete_e__Struct", SimStruct), "MountVolume": SimTypeRef("_MountVolume_e__Struct", SimStruct), "Others": SimTypeRef("_Others_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FLT_IO_PARAMETER_BLOCK", pack=False, align=None),
    "FLT_NAME_CONTROL": SimStruct(OrderedDict((("Name", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="FLT_NAME_CONTROL", pack=False, align=None),
    "FLT_OPERATION_REGISTRATION": SimStruct(OrderedDict((("MajorFunction", SimTypeChar(label="Byte")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PreOperation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="FLT_PREOP_CALLBACK_STATUS"), arg_names=["Data", "FltObjects", "CompletionContext"]), offset=0)), ("PostOperation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="FLT_POSTOP_CALLBACK_STATUS"), arg_names=["Data", "FltObjects", "CompletionContext", "Flags"]), offset=0)), ("Reserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FLT_OPERATION_REGISTRATION", pack=False, align=None),
    "FLT_PARAMETERS": SimUnion({"Create": SimTypeRef("_Create_e__Struct", SimStruct), "CreatePipe": SimTypeRef("_CreatePipe_e__Struct", SimStruct), "CreateMailslot": SimTypeRef("_CreateMailslot_e__Struct", SimStruct), "Read": SimTypeRef("_Read_e__Struct", SimStruct), "Write": SimTypeRef("_Write_e__Struct", SimStruct), "QueryFileInformation": SimTypeRef("_QueryFileInformation_e__Struct", SimStruct), "SetFileInformation": SimTypeRef("_SetFileInformation_e__Struct", SimStruct), "QueryEa": SimTypeRef("_QueryEa_e__Struct", SimStruct), "SetEa": SimTypeRef("_SetEa_e__Struct", SimStruct), "QueryVolumeInformation": SimTypeRef("_QueryVolumeInformation_e__Struct", SimStruct), "SetVolumeInformation": SimTypeRef("_SetVolumeInformation_e__Struct", SimStruct), "DirectoryControl": SimUnion({"QueryDirectory": SimTypeRef("_QueryDirectory_e__Struct", SimStruct), "NotifyDirectory": SimTypeRef("_NotifyDirectory_e__Struct", SimStruct), "NotifyDirectoryEx": SimTypeRef("_NotifyDirectoryEx_e__Struct", SimStruct)}, name="<anon>", label="None"), "FileSystemControl": SimUnion({"VerifyVolume": SimTypeRef("_VerifyVolume_e__Struct", SimStruct), "Common": SimTypeRef("_Common_e__Struct", SimStruct), "Neither": SimTypeRef("_Neither_e__Struct", SimStruct), "Buffered": SimTypeRef("_Buffered_e__Struct", SimStruct), "Direct": SimTypeRef("_Direct_e__Struct", SimStruct)}, name="<anon>", label="None"), "DeviceIoControl": SimUnion({"Common": SimTypeRef("_Common_e__Struct", SimStruct), "Neither": SimTypeRef("_Neither_e__Struct", SimStruct), "Buffered": SimTypeRef("_Buffered_e__Struct", SimStruct), "Direct": SimTypeRef("_Direct_e__Struct", SimStruct), "FastIo": SimTypeRef("_FastIo_e__Struct", SimStruct)}, name="<anon>", label="None"), "LockControl": SimTypeRef("_LockControl_e__Struct", SimStruct), "QuerySecurity": SimTypeRef("_QuerySecurity_e__Struct", SimStruct), "SetSecurity": SimTypeRef("_SetSecurity_e__Struct", SimStruct), "WMI": SimTypeRef("_WMI_e__Struct", SimStruct), "QueryQuota": SimTypeRef("_QueryQuota_e__Struct", SimStruct), "SetQuota": SimTypeRef("_SetQuota_e__Struct", SimStruct), "Pnp": SimUnion({"StartDevice": SimTypeRef("_StartDevice_e__Struct", SimStruct), "QueryDeviceRelations": SimTypeRef("_QueryDeviceRelations_e__Struct", SimStruct), "QueryInterface": SimTypeRef("_QueryInterface_e__Struct", SimStruct), "DeviceCapabilities": SimTypeRef("_DeviceCapabilities_e__Struct", SimStruct), "FilterResourceRequirements": SimTypeRef("_FilterResourceRequirements_e__Struct", SimStruct), "ReadWriteConfig": SimTypeRef("_ReadWriteConfig_e__Struct", SimStruct), "SetLock": SimTypeRef("_SetLock_e__Struct", SimStruct), "QueryId": SimTypeRef("_QueryId_e__Struct", SimStruct), "QueryDeviceText": SimTypeRef("_QueryDeviceText_e__Struct", SimStruct), "UsageNotification": SimTypeRef("_UsageNotification_e__Struct", SimStruct)}, name="<anon>", label="None"), "AcquireForSectionSynchronization": SimTypeRef("_AcquireForSectionSynchronization_e__Struct", SimStruct), "AcquireForModifiedPageWriter": SimTypeRef("_AcquireForModifiedPageWriter_e__Struct", SimStruct), "ReleaseForModifiedPageWriter": SimTypeRef("_ReleaseForModifiedPageWriter_e__Struct", SimStruct), "QueryOpen": SimTypeRef("_QueryOpen_e__Struct", SimStruct), "FastIoCheckIfPossible": SimTypeRef("_FastIoCheckIfPossible_e__Struct", SimStruct), "NetworkQueryOpen": SimTypeRef("_NetworkQueryOpen_e__Struct", SimStruct), "MdlRead": SimTypeRef("_MdlRead_e__Struct", SimStruct), "MdlReadComplete": SimTypeRef("_MdlReadComplete_e__Struct", SimStruct), "PrepareMdlWrite": SimTypeRef("_PrepareMdlWrite_e__Struct", SimStruct), "MdlWriteComplete": SimTypeRef("_MdlWriteComplete_e__Struct", SimStruct), "MountVolume": SimTypeRef("_MountVolume_e__Struct", SimStruct), "Others": SimTypeRef("_Others_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "FLT_POSTOP_CALLBACK_STATUS": SimTypeInt(signed=False, label="FLT_POSTOP_CALLBACK_STATUS"),
    "FLT_PREOP_CALLBACK_STATUS": SimTypeInt(signed=False, label="FLT_PREOP_CALLBACK_STATUS"),
    "FLT_REGISTRATION": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ContextRegistration", SimTypePointer(SimTypeRef("FLT_CONTEXT_REGISTRATION", SimStruct), offset=0)), ("OperationRegistration", SimTypePointer(SimTypeRef("FLT_OPERATION_REGISTRATION", SimStruct), offset=0)), ("FilterUnloadCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Flags"]), offset=0)), ("InstanceSetupCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FLT_FILESYSTEM_TYPE")], SimTypeInt(signed=True, label="Int32"), arg_names=["FltObjects", "Flags", "VolumeDeviceType", "VolumeFilesystemType"]), offset=0)), ("InstanceQueryTeardownCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FltObjects", "Flags"]), offset=0)), ("InstanceTeardownStartCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["FltObjects", "Reason"]), offset=0)), ("InstanceTeardownCompleteCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["FltObjects", "Reason"]), offset=0)), ("GenerateFileNameCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("FLT_NAME_CONTROL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "FileObject", "CallbackData", "NameOptions", "CacheFileNameInformation", "FileName"]), offset=0)), ("NormalizeNameComponentCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILE_NAMES_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "ParentDirectory", "VolumeNameLength", "Component", "ExpandComponentName", "ExpandComponentNameLength", "Flags", "NormalizationContext"]), offset=0)), ("NormalizeContextCleanupCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["NormalizationContext"]), offset=0)), ("TransactionNotificationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FltObjects", "TransactionContext", "NotificationMask"]), offset=0)), ("NormalizeNameComponentExCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILE_NAMES_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "FileObject", "ParentDirectory", "VolumeNameLength", "Component", "ExpandComponentName", "ExpandComponentNameLength", "Flags", "NormalizationContext"]), offset=0)), ("SectionNotificationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "SectionContext", "Data"]), offset=0)),)), name="FLT_REGISTRATION", pack=False, align=None),
    "FLT_RELATED_CONTEXTS": SimStruct(OrderedDict((("VolumeContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InstanceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StreamContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StreamHandleContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransactionContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FLT_RELATED_CONTEXTS", pack=False, align=None),
    "FLT_RELATED_CONTEXTS_EX": SimStruct(OrderedDict((("VolumeContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InstanceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StreamContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StreamHandleContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransactionContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SectionContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FLT_RELATED_CONTEXTS_EX", pack=False, align=None),
    "FLT_RELATED_OBJECTS": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("TransactionContext", SimTypeShort(signed=False, label="UInt16")), ("Filter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Volume", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Instance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("Transaction", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="FLT_RELATED_OBJECTS", pack=False, align=None),
    "FLT_SET_CONTEXT_OPERATION": SimTypeInt(signed=False, label="FLT_SET_CONTEXT_OPERATION"),
    "FLT_TAG_DATA_BUFFER": SimStruct(OrderedDict((("FileTag", SimTypeInt(signed=False, label="UInt32")), ("TagDataLength", SimTypeShort(signed=False, label="UInt16")), ("UnparsedNameLength", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"SymbolicLinkReparseBuffer": SimTypeRef("_SymbolicLinkReparseBuffer_e__Struct", SimStruct), "MountPointReparseBuffer": SimTypeRef("_MountPointReparseBuffer_e__Struct", SimStruct), "GenericReparseBuffer": SimTypeRef("_GenericReparseBuffer_e__Struct", SimStruct), "GenericGUIDReparseBuffer": SimTypeRef("_GenericGUIDReparseBuffer_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FLT_TAG_DATA_BUFFER", pack=False, align=None),
    "FLT_VOLUME_PROPERTIES": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("DeviceCharacteristics", SimTypeInt(signed=False, label="UInt32")), ("DeviceObjectFlags", SimTypeInt(signed=False, label="UInt32")), ("AlignmentRequirement", SimTypeInt(signed=False, label="UInt32")), ("SectorSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("FileSystemDriverName", SimTypeRef("UNICODE_STRING", SimStruct)), ("FileSystemDeviceName", SimTypeRef("UNICODE_STRING", SimStruct)), ("RealDeviceName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="FLT_VOLUME_PROPERTIES", pack=False, align=None),
    "FLYOUTPARTS": SimTypeInt(signed=False, label="FLYOUTPARTS"),
    "FLYOUT_PLACEMENT": SimTypeInt(signed=False, label="FLYOUT_PLACEMENT"),
    "FLockClrVersionCallback": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "FMTOWNS_SND_WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="FMTOWNS_SND_WAVEFORMAT", pack=False, align=None),
    "FNAPONOTIFICATIONCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("APO_REG_PROPERTIES", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProperties", "pvRefData"]), offset=0),
    "FNCERTSRVBACKUPCLOSE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc"]), offset=0),
    "FNCERTSRVBACKUPEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc"]), offset=0),
    "FNCERTSRVBACKUPFREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0),
    "FNCERTSRVBACKUPGETBACKUPLOGSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "ppwszzBackupLogFiles", "pcbSize"]), offset=0),
    "FNCERTSRVBACKUPGETDATABASENAMESW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "ppwszzAttachmentInformation", "pcbSize"]), offset=0),
    "FNCERTSRVBACKUPGETDYNAMICFILELISTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "ppwszzFileList", "pcbSize"]), offset=0),
    "FNCERTSRVBACKUPOPENFILEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "pwszAttachmentName", "cbReadHintSize", "pliFileSize"]), offset=0),
    "FNCERTSRVBACKUPPREPAREW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszServerName", "grbitJet", "dwBackupFlags", "phbc"]), offset=0),
    "FNCERTSRVBACKUPREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "pvBuffer", "cbBuffer", "pcbRead"]), offset=0),
    "FNCERTSRVBACKUPTRUNCATELOGS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc"]), offset=0),
    "FNCERTSRVISSERVERONLINEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszServerName", "pfServerOnline"]), offset=0),
    "FNCERTSRVRESTOREEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc"]), offset=0),
    "FNCERTSRVRESTOREGETDATABASELOCATIONSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "ppwszzDatabaseLocationList", "pcbSize"]), offset=0),
    "FNCERTSRVRESTOREPREPAREW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszServerName", "dwRestoreFlags", "phbc"]), offset=0),
    "FNCERTSRVRESTOREREGISTERCOMPLETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "hrRestoreState"]), offset=0),
    "FNCERTSRVRESTOREREGISTERW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("CSEDB_RSTMAPW", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hbc", "pwszCheckPointFilePath", "pwszLogPath", "rgrstmap", "crstmap", "pwszBackupLogPath", "genLow", "genHigh"]), offset=0),
    "FNCERTSRVSERVERCONTROLW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszServerName", "dwControlFlags", "pcbOut", "ppbOut"]), offset=0),
    "FNIMPORTPFXTOPROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="ImportPFXFlags"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndParent", "pbPFX", "cbPFX", "ImportFlags", "pwszPassword", "pwszProviderName", "pwszReaderName", "pwszContainerNamePrefix", "pwszPin", "pwszFriendlyName", "pcCertOut", "prgpCertOut"]), offset=0),
    "FNIMPORTPFXTOPROVIDERFREEDATA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), label="LPArray", offset=0)], SimTypeBottom(label="Void"), arg_names=["cCert", "rgpCert"]), offset=0),
    "FN_OBJECT_MGR_FUNCTS": SimStruct(OrderedDict((("Placeholder", SimTypeInt(signed=False, label="UInt32")),)), name="FN_OBJECT_MGR_FUNCTS", pack=False, align=None),
    "FN_PROGRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="PROG_INVOKE_SETTING"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["pObjectName", "Status", "pInvokeSetting", "Args", "SecuritySet"]), offset=0),
    "FOCUS_EVENT_RECORD": SimStruct(OrderedDict((("bSetFocus", SimTypeInt(signed=True, label="Int32")),)), name="FOCUS_EVENT_RECORD", pack=False, align=None),
    "FOLDERFLAGS": SimTypeInt(signed=False, label="FOLDERFLAGS"),
    "FOLDERLOGICALVIEWMODE": SimTypeInt(signed=False, label="FOLDERLOGICALVIEWMODE"),
    "FOLDERSETDATA": SimStruct(OrderedDict((("_fs", SimTypeRef("FOLDERSETTINGS", SimStruct)), ("_vidRestore", SimTypeBottom(label="Guid")), ("_dwViewPriority", SimTypeInt(signed=False, label="UInt32")),)), name="FOLDERSETDATA", pack=False, align=None),
    "FOLDERSETTINGS": SimStruct(OrderedDict((("ViewMode", SimTypeInt(signed=False, label="UInt32")), ("fFlags", SimTypeInt(signed=False, label="UInt32")),)), name="FOLDERSETTINGS", pack=False, align=None),
    "FOLDERVIEWMODE": SimTypeInt(signed=False, label="FOLDERVIEWMODE"),
    "FOLDERVIEWOPTIONS": SimTypeInt(signed=False, label="FOLDERVIEWOPTIONS"),
    "FOLDER_ENUM_MODE": SimTypeInt(signed=False, label="FOLDER_ENUM_MODE"),
    "FOLD_STRING_MAP_FLAGS": SimTypeInt(signed=False, label="FOLD_STRING_MAP_FLAGS"),
    "FOLLOW_FLAGS": SimTypeInt(signed=False, label="FOLLOW_FLAGS"),
    "FONTDESC": SimStruct(OrderedDict((("cbSizeofstruct", SimTypeInt(signed=False, label="UInt32")), ("lpstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cySize", SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("sWeight", SimTypeShort(signed=True, label="Int16")), ("sCharset", SimTypeShort(signed=True, label="Int16")), ("fItalic", SimTypeInt(signed=True, label="Int32")), ("fUnderline", SimTypeInt(signed=True, label="Int32")), ("fStrikethrough", SimTypeInt(signed=True, label="Int32")),)), name="FONTDESC", pack=False, align=None),
    "FONTDIFF": SimStruct(OrderedDict((("jReserved1", SimTypeChar(label="Byte")), ("jReserved2", SimTypeChar(label="Byte")), ("jReserved3", SimTypeChar(label="Byte")), ("bWeight", SimTypeChar(label="Byte")), ("usWinWeight", SimTypeShort(signed=False, label="UInt16")), ("fsSelection", SimTypeShort(signed=False, label="UInt16")), ("fwdAveCharWidth", SimTypeShort(signed=True, label="Int16")), ("fwdMaxCharInc", SimTypeShort(signed=True, label="Int16")), ("ptlCaret", SimTypeRef("POINTL", SimStruct)),)), name="FONTDIFF", pack=False, align=None),
    "FONTENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LOGFONTA", SimStruct), offset=0), SimTypePointer(SimTypeRef("TEXTMETRICA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "FONTENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LOGFONTW", SimStruct), offset=0), SimTypePointer(SimTypeRef("TEXTMETRICW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "FONTINFO": SimStruct(OrderedDict((("cjThis", SimTypeInt(signed=False, label="UInt32")), ("flCaps", SimTypeInt(signed=False, label="UInt32")), ("cGlyphsSupported", SimTypeInt(signed=False, label="UInt32")), ("cjMaxGlyph1", SimTypeInt(signed=False, label="UInt32")), ("cjMaxGlyph4", SimTypeInt(signed=False, label="UInt32")), ("cjMaxGlyph8", SimTypeInt(signed=False, label="UInt32")), ("cjMaxGlyph32", SimTypeInt(signed=False, label="UInt32")),)), name="FONTINFO", pack=False, align=None),
    "FONTOBJ": SimStruct(OrderedDict((("iUniq", SimTypeInt(signed=False, label="UInt32")), ("iFace", SimTypeInt(signed=False, label="UInt32")), ("cxMax", SimTypeInt(signed=False, label="UInt32")), ("flFontType", SimTypeInt(signed=False, label="UInt32")), ("iTTUniq", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iFile", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("sizLogResPpi", SimTypeRef("SIZE", SimStruct)), ("ulStyleSize", SimTypeInt(signed=False, label="UInt32")), ("pvConsumer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pvProducer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FONTOBJ", pack=False, align=None),
    "FONTSIGNATURE": SimStruct(OrderedDict((("fsUsb", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("fsCsb", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="FONTSIGNATURE", pack=False, align=None),
    "FONTSIM": SimStruct(OrderedDict((("dpBold", SimTypeInt(signed=True, label="Int32")), ("dpItalic", SimTypeInt(signed=True, label="Int32")), ("dpBoldItalic", SimTypeInt(signed=True, label="Int32")),)), name="FONTSIM", pack=False, align=None),
    "FONT_CHARSET": SimTypeInt(signed=False, label="FONT_CHARSET"),
    "FONT_CLIP_PRECISION": SimTypeInt(signed=False, label="FONT_CLIP_PRECISION"),
    "FONT_FAMILY": SimTypeInt(signed=False, label="FONT_FAMILY"),
    "FONT_IMAGE_INFO": SimStruct(OrderedDict((("FontSize", SimTypeRef("COORD", SimStruct)), ("ImageBits", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FONT_IMAGE_INFO", pack=False, align=None),
    "FONT_LICENSE_PRIVS": SimTypeInt(signed=False, label="FONT_LICENSE_PRIVS"),
    "FONT_OUTPUT_PRECISION": SimTypeInt(signed=False, label="FONT_OUTPUT_PRECISION"),
    "FONT_PITCH": SimTypeInt(signed=False, label="FONT_PITCH"),
    "FONT_QUALITY": SimTypeInt(signed=False, label="FONT_QUALITY"),
    "FONT_RESOURCE_CHARACTERISTICS": SimTypeInt(signed=False, label="FONT_RESOURCE_CHARACTERISTICS"),
    "FONT_WEIGHT": SimTypeInt(signed=False, label="FONT_WEIGHT"),
    "FORCE_LEVEL_FLAGS": SimTypeInt(signed=False, label="FORCE_LEVEL_FLAGS"),
    "FOREGROUND_WINDOW_LOCK_CODE": SimTypeInt(signed=False, label="FOREGROUND_WINDOW_LOCK_CODE"),
    "FOREIGN_STATUS": SimTypeInt(signed=False, label="FOREIGN_STATUS"),
    "FORK_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="FORK_VIRTUAL_DISK_FLAG"),
    "FORK_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="FORK_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FORK_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "FORK_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="FORK_VIRTUAL_DISK_VERSION"),
    "FORMATETC": SimStruct(OrderedDict((("cfFormat", SimTypeShort(signed=False, label="UInt16")), ("ptd", SimTypePointer(SimTypeRef("DVTARGETDEVICE", SimStruct), offset=0)), ("dwAspect", SimTypeInt(signed=False, label="UInt32")), ("lindex", SimTypeInt(signed=True, label="Int32")), ("tymed", SimTypeInt(signed=False, label="UInt32")),)), name="FORMATETC", pack=False, align=None),
    "FORMATRANGE": SimStruct(OrderedDict((("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hdcTarget", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)), ("rcPage", SimTypeRef("RECT", SimStruct)), ("chrg", SimTypeRef("CHARRANGE", SimStruct)),)), name="FORMATRANGE", pack=False, align=None),
    "FORMAT_EX_PARAMETERS": SimStruct(OrderedDict((("MediaType", SimTypeInt(signed=False, label="MEDIA_TYPE")), ("StartCylinderNumber", SimTypeInt(signed=False, label="UInt32")), ("EndCylinderNumber", SimTypeInt(signed=False, label="UInt32")), ("StartHeadNumber", SimTypeInt(signed=False, label="UInt32")), ("EndHeadNumber", SimTypeInt(signed=False, label="UInt32")), ("FormatGapLength", SimTypeShort(signed=False, label="UInt16")), ("SectorsPerTrack", SimTypeShort(signed=False, label="UInt16")), ("SectorNumber", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="FORMAT_EX_PARAMETERS", pack=False, align=None),
    "FORMAT_MESSAGE_OPTIONS": SimTypeInt(signed=False, label="FORMAT_MESSAGE_OPTIONS"),
    "FORMAT_PARAMETERS": SimStruct(OrderedDict((("MediaType", SimTypeInt(signed=False, label="MEDIA_TYPE")), ("StartCylinderNumber", SimTypeInt(signed=False, label="UInt32")), ("EndCylinderNumber", SimTypeInt(signed=False, label="UInt32")), ("StartHeadNumber", SimTypeInt(signed=False, label="UInt32")), ("EndHeadNumber", SimTypeInt(signed=False, label="UInt32")),)), name="FORMAT_PARAMETERS", pack=False, align=None),
    "FORM_INFO_1A": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Size", SimTypeRef("SIZE", SimStruct)), ("ImageableArea", SimTypeRef("RECTL", SimStruct)),)), name="FORM_INFO_1A", pack=False, align=None),
    "FORM_INFO_1W": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Size", SimTypeRef("SIZE", SimStruct)), ("ImageableArea", SimTypeRef("RECTL", SimStruct)),)), name="FORM_INFO_1W", pack=False, align=None),
    "FORM_INFO_2A": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Size", SimTypeRef("SIZE", SimStruct)), ("ImageableArea", SimTypeRef("RECTL", SimStruct)), ("pKeyword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StringType", SimTypeInt(signed=False, label="UInt32")), ("pMuiDll", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwResourceId", SimTypeInt(signed=False, label="UInt32")), ("pDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wLangId", SimTypeShort(signed=False, label="UInt16")),)), name="FORM_INFO_2A", pack=False, align=None),
    "FORM_INFO_2W": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Size", SimTypeRef("SIZE", SimStruct)), ("ImageableArea", SimTypeRef("RECTL", SimStruct)), ("pKeyword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StringType", SimTypeInt(signed=False, label="UInt32")), ("pMuiDll", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwResourceId", SimTypeInt(signed=False, label="UInt32")), ("pDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wLangId", SimTypeShort(signed=False, label="UInt16")),)), name="FORM_INFO_2W", pack=False, align=None),
    "FORTCMD": SimTypeInt(signed=False, label="FORTCMD"),
    "FORTSTAT": SimTypeInt(signed=False, label="FORTSTAT"),
    "FOUND_IMAGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DOS_IMAGE_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ImageInfo"]), offset=0),
    "FPGA_BUS_SCAN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "FPGA_CONTROL_CONFIG_SPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Enable"]), offset=0),
    "FPGA_CONTROL_ERROR_REPORTING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "UncorrectableMask", "CorrectableMask", "DisableErrorReporting"]), offset=0),
    "FPGA_CONTROL_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("BusScan", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ControlLink", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("ControlConfigSpace", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("ControlErrorReporting", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="FPGA_CONTROL_INTERFACE", pack=False, align=None),
    "FPGA_CONTROL_LINK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Enable"]), offset=0),
    "FPO_DATA": SimStruct(OrderedDict((("ulOffStart", SimTypeInt(signed=False, label="UInt32")), ("cbProcSize", SimTypeInt(signed=False, label="UInt32")), ("cdwLocals", SimTypeInt(signed=False, label="UInt32")), ("cdwParams", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="FPO_DATA", pack=False, align=None),
    "FRAMEBOTTOMSTATES": SimTypeInt(signed=False, label="FRAMEBOTTOMSTATES"),
    "FRAMELEFTSTATES": SimTypeInt(signed=False, label="FRAMELEFTSTATES"),
    "FRAMEOPTIONS_FLAGS": SimTypeInt(signed=False, label="FRAMEOPTIONS_FLAGS"),
    "FRAMERIGHTSTATES": SimTypeInt(signed=False, label="FRAMERIGHTSTATES"),
    "FRAMESTATES": SimTypeInt(signed=False, label="FRAMESTATES"),
    "FRAMING_CACHE_OPS": SimTypeInt(signed=False, label="FRAMING_CACHE_OPS"),
    "FRAMING_PROP": SimTypeInt(signed=False, label="FRAMING_PROP"),
    "FREEOBJPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVEROBJ", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDriverObj"]), offset=0),
    "FREE_CONTEXT_BUFFER_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "FREE_CREDENTIALS_HANDLE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "FREE_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Buffer"]), offset=0),
    "FREE_VIRTUAL_MEMORY_EX_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackContext", "ProcessHandle", "BaseAddress", "RegionSize", "FreeType"]), offset=0),
    "FSCNTL_SCREEN_INFO": SimStruct(OrderedDict((("Position", SimTypeRef("COORD", SimStruct)), ("ScreenSize", SimTypeRef("COORD", SimStruct)), ("nNumberOfChars", SimTypeInt(signed=False, label="UInt32")),)), name="FSCNTL_SCREEN_INFO", pack=False, align=None),
    "FSCTL_GET_INTEGRITY_INFORMATION_BUFFER": SimStruct(OrderedDict((("ChecksumAlgorithm", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ChecksumChunkSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("ClusterSizeInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="FSCTL_GET_INTEGRITY_INFORMATION_BUFFER", pack=False, align=None),
    "FSCTL_GHOST_FILE_EXTENTS_INPUT_BUFFER": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")), ("RecallOwnerGuid", SimTypeBottom(label="Guid")), ("RecallMetadataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("RecallMetadataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FSCTL_GHOST_FILE_EXTENTS_INPUT_BUFFER", pack=False, align=None),
    "FSCTL_OFFLOAD_READ_INPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TokenTimeToLive", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("FileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("CopyLength", SimTypeLongLong(signed=False, label="UInt64")),)), name="FSCTL_OFFLOAD_READ_INPUT", pack=False, align=None),
    "FSCTL_OFFLOAD_READ_OUTPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TransferLength", SimTypeLongLong(signed=False, label="UInt64")), ("Token", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="FSCTL_OFFLOAD_READ_OUTPUT", pack=False, align=None),
    "FSCTL_OFFLOAD_WRITE_INPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("CopyLength", SimTypeLongLong(signed=False, label="UInt64")), ("TransferOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Token", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="FSCTL_OFFLOAD_WRITE_INPUT", pack=False, align=None),
    "FSCTL_OFFLOAD_WRITE_OUTPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("LengthWritten", SimTypeLongLong(signed=False, label="UInt64")),)), name="FSCTL_OFFLOAD_WRITE_OUTPUT", pack=False, align=None),
    "FSCTL_QUERY_FAT_BPB_BUFFER": SimStruct(OrderedDict((("First0x24BytesOfBootSector", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="FSCTL_QUERY_FAT_BPB_BUFFER", pack=False, align=None),
    "FSCTL_QUERY_GHOSTED_FILE_EXTENTS_INPUT_RANGE": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")),)), name="FSCTL_QUERY_GHOSTED_FILE_EXTENTS_INPUT_RANGE", pack=False, align=None),
    "FSCTL_QUERY_GHOSTED_FILE_EXTENTS_OUTPUT": SimStruct(OrderedDict((("ExtentCount", SimTypeInt(signed=False, label="UInt32")), ("TotalExtentCount", SimTypeInt(signed=False, label="UInt32")), ("Extents", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FSCTL_QUERY_GHOSTED_FILE_EXTENTS_OUTPUT", pack=False, align=None),
    "FSCTL_QUERY_REGION_INFO_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumberOfTierIds", SimTypeInt(signed=False, label="UInt32")), ("TierIds", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="FSCTL_QUERY_REGION_INFO_INPUT", pack=False, align=None),
    "FSCTL_QUERY_REGION_INFO_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Alignment", SimTypeLongLong(signed=False, label="UInt64")), ("TotalNumberOfRegions", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRegionsReturned", SimTypeInt(signed=False, label="UInt32")), ("Regions", SimTypePointer(SimTypeRef("FILE_STORAGE_TIER_REGION", SimStruct), offset=0)),)), name="FSCTL_QUERY_REGION_INFO_OUTPUT", pack=False, align=None),
    "FSCTL_QUERY_STORAGE_CLASSES_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="FILE_STORAGE_TIER_FLAGS")), ("TotalNumberOfTiers", SimTypeInt(signed=False, label="UInt32")), ("NumberOfTiersReturned", SimTypeInt(signed=False, label="UInt32")), ("Tiers", SimTypePointer(SimTypeRef("FILE_STORAGE_TIER", SimStruct), offset=0)),)), name="FSCTL_QUERY_STORAGE_CLASSES_OUTPUT", pack=False, align=None),
    "FSCTL_QUERY_VOLUME_NUMA_INFO_OUTPUT": SimStruct(OrderedDict((("NumaNode", SimTypeInt(signed=False, label="UInt32")),)), name="FSCTL_QUERY_VOLUME_NUMA_INFO_OUTPUT", pack=False, align=None),
    "FSCTL_SET_INTEGRITY_INFORMATION_BUFFER": SimStruct(OrderedDict((("ChecksumAlgorithm", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="FSCTL_SET_INTEGRITY_INFORMATION_BUFFER", pack=False, align=None),
    "FSCTL_SET_INTEGRITY_INFORMATION_BUFFER_EX": SimStruct(OrderedDict((("EnableIntegrity", SimTypeChar(label="Byte")), ("KeepIntegrityStateUnchanged", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 7)),)), name="FSCTL_SET_INTEGRITY_INFORMATION_BUFFER_EX", pack=False, align=None),
    "FSCTL_UNMAP_SPACE_INPUT_BUFFER": SimStruct(OrderedDict((("BytesToUnmap", SimTypeLongLong(signed=True, label="Int64")),)), name="FSCTL_UNMAP_SPACE_INPUT_BUFFER", pack=False, align=None),
    "FSCTL_UNMAP_SPACE_OUTPUT": SimStruct(OrderedDict((("BytesUnmapped", SimTypeLongLong(signed=True, label="Int64")),)), name="FSCTL_UNMAP_SPACE_OUTPUT", pack=False, align=None),
    "FSM_STATE": SimTypeInt(signed=False, label="FSM_STATE"),
    "FSRTL_ADVANCED_FCB_HEADER": SimStruct(OrderedDict((("Base", SimTypeRef("FSRTL_COMMON_FCB_HEADER", SimStruct)), ("FastMutex", SimTypePointer(SimTypeRef("FAST_MUTEX", SimStruct), offset=0)), ("FilterContexts", SimTypeRef("LIST_ENTRY", SimStruct)), ("PushLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("FileContextSupportPointer", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("Anonymous", SimUnion({"Oplock": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ReservedForRemote": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("AePushLock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BypassIoOpenCount", SimTypeInt(signed=False, label="UInt32")),)), name="FSRTL_ADVANCED_FCB_HEADER", pack=False, align=None),
    "FSRTL_AUXILIARY_BUFFER": SimStruct(OrderedDict((("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Mdl", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="FSRTL_AUXILIARY_BUFFER", pack=False, align=None),
    "FSRTL_CHANGE_BACKING_TYPE": SimTypeInt(signed=False, label="FSRTL_CHANGE_BACKING_TYPE"),
    "FSRTL_COMMON_FCB_HEADER": SimStruct(OrderedDict((("NodeTypeCode", SimTypeShort(signed=True, label="Int16")), ("NodeByteSize", SimTypeShort(signed=True, label="Int16")), ("Flags", SimTypeChar(label="Byte")), ("IsFastIoPossible", SimTypeChar(label="Byte")), ("Flags2", SimTypeChar(label="Byte")), ("_bitfield", SimTypeChar(label="Byte")), ("Resource", SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0)), ("PagingIoResource", SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0)), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("ValidDataLength", SimTypeLongLong(signed=True, label="Int64")),)), name="FSRTL_COMMON_FCB_HEADER", pack=False, align=None),
    "FSRTL_COMPARISON_RESULT": SimTypeInt(signed=False, label="FSRTL_COMPARISON_RESULT"),
    "FSRTL_MUP_PROVIDER_INFO_LEVEL_1": SimStruct(OrderedDict((("ProviderId", SimTypeInt(signed=False, label="UInt32")),)), name="FSRTL_MUP_PROVIDER_INFO_LEVEL_1", pack=False, align=None),
    "FSRTL_MUP_PROVIDER_INFO_LEVEL_2": SimStruct(OrderedDict((("ProviderId", SimTypeInt(signed=False, label="UInt32")), ("ProviderName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="FSRTL_MUP_PROVIDER_INFO_LEVEL_2", pack=False, align=None),
    "FSRTL_PER_FILEOBJECT_CONTEXT": SimStruct(OrderedDict((("Links", SimTypeRef("LIST_ENTRY", SimStruct)), ("OwnerId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InstanceId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FSRTL_PER_FILEOBJECT_CONTEXT", pack=False, align=None),
    "FSRTL_PER_FILE_CONTEXT": SimStruct(OrderedDict((("Links", SimTypeRef("LIST_ENTRY", SimStruct)), ("OwnerId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InstanceId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FreeCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)),)), name="FSRTL_PER_FILE_CONTEXT", pack=False, align=None),
    "FSRTL_PER_STREAM_CONTEXT": SimStruct(OrderedDict((("Links", SimTypeRef("LIST_ENTRY", SimStruct)), ("OwnerId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InstanceId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FreeCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)),)), name="FSRTL_PER_STREAM_CONTEXT", pack=False, align=None),
    "FSRTL_UNC_PROVIDER_REGISTRATION": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"ProviderFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"HardeningCapabilities": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="FSRTL_UNC_PROVIDER_REGISTRATION", pack=False, align=None),
    "FSVIDEO_COPY_FRAME_BUFFER": SimStruct(OrderedDict((("SrcScreen", SimTypeRef("FSCNTL_SCREEN_INFO", SimStruct)), ("DestScreen", SimTypeRef("FSCNTL_SCREEN_INFO", SimStruct)),)), name="FSVIDEO_COPY_FRAME_BUFFER", pack=False, align=None),
    "FSVIDEO_CURSOR_POSITION": SimStruct(OrderedDict((("Coord", SimTypeRef("VIDEO_CURSOR_POSITION", SimStruct)), ("dwType", SimTypeInt(signed=False, label="UInt32")),)), name="FSVIDEO_CURSOR_POSITION", pack=False, align=None),
    "FSVIDEO_MODE_INFORMATION": SimStruct(OrderedDict((("VideoMode", SimTypeRef("VIDEO_MODE_INFORMATION", SimStruct)), ("VideoMemory", SimTypeRef("VIDEO_MEMORY_INFORMATION", SimStruct)),)), name="FSVIDEO_MODE_INFORMATION", pack=False, align=None),
    "FSVIDEO_REVERSE_MOUSE_POINTER": SimStruct(OrderedDict((("Screen", SimTypeRef("FSCNTL_SCREEN_INFO", SimStruct)), ("dwType", SimTypeInt(signed=False, label="UInt32")),)), name="FSVIDEO_REVERSE_MOUSE_POINTER", pack=False, align=None),
    "FSVIDEO_SCREEN_INFORMATION": SimStruct(OrderedDict((("ScreenSize", SimTypeRef("COORD", SimStruct)), ("FontSize", SimTypeRef("COORD", SimStruct)),)), name="FSVIDEO_SCREEN_INFORMATION", pack=False, align=None),
    "FSVIDEO_WRITE_TO_FRAME_BUFFER": SimStruct(OrderedDict((("SrcBuffer", SimTypePointer(SimTypeRef("CHAR_IMAGE_INFO", SimStruct), offset=0)), ("DestScreen", SimTypeRef("FSCNTL_SCREEN_INFO", SimStruct)),)), name="FSVIDEO_WRITE_TO_FRAME_BUFFER", pack=False, align=None),
    "FS_BPIO_INFLAGS": SimTypeInt(signed=False, label="FS_BPIO_INFLAGS"),
    "FS_BPIO_INFO": SimStruct(OrderedDict((("ActiveBypassIoCount", SimTypeInt(signed=False, label="UInt32")), ("StorageDriverNameLen", SimTypeShort(signed=False, label="UInt16")), ("StorageDriverName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="FS_BPIO_INFO", pack=False, align=None),
    "FS_BPIO_INPUT": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="FS_BPIO_OPERATIONS")), ("InFlags", SimTypeInt(signed=False, label="FS_BPIO_INFLAGS")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="FS_BPIO_INPUT", pack=False, align=None),
    "FS_BPIO_OPERATIONS": SimTypeInt(signed=False, label="FS_BPIO_OPERATIONS"),
    "FS_BPIO_OUTFLAGS": SimTypeInt(signed=False, label="FS_BPIO_OUTFLAGS"),
    "FS_BPIO_OUTPUT": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="FS_BPIO_OPERATIONS")), ("OutFlags", SimTypeInt(signed=False, label="FS_BPIO_OUTFLAGS")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"Enable": SimTypeRef("FS_BPIO_RESULTS", SimStruct), "Query": SimTypeRef("FS_BPIO_RESULTS", SimStruct), "VolumeStackResume": SimTypeRef("FS_BPIO_RESULTS", SimStruct), "StreamResume": SimTypeRef("FS_BPIO_RESULTS", SimStruct), "GetInfo": SimTypeRef("FS_BPIO_INFO", SimStruct)}, name="<anon>", label="None")),)), name="FS_BPIO_OUTPUT", pack=False, align=None),
    "FS_BPIO_RESULTS": SimStruct(OrderedDict((("OpStatus", SimTypeInt(signed=False, label="UInt32")), ("FailingDriverNameLen", SimTypeShort(signed=False, label="UInt16")), ("FailingDriverName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("FailureReasonLen", SimTypeShort(signed=False, label="UInt16")), ("FailureReason", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="FS_BPIO_RESULTS", pack=False, align=None),
    "FS_FILTER_CALLBACKS": SimStruct(OrderedDict((("SizeOfFsFilterCallbacks", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("PreAcquireForSectionSynchronization", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostAcquireForSectionSynchronization", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)), ("PreReleaseForSectionSynchronization", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostReleaseForSectionSynchronization", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)), ("PreAcquireForCcFlush", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostAcquireForCcFlush", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)), ("PreReleaseForCcFlush", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostReleaseForCcFlush", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)), ("PreAcquireForModifiedPageWriter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostAcquireForModifiedPageWriter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)), ("PreReleaseForModifiedPageWriter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostReleaseForModifiedPageWriter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)), ("PreQueryOpen", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0)), ("PostQueryOpen", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0)),)), name="FS_FILTER_CALLBACKS", pack=False, align=None),
    "FS_FILTER_CALLBACK_DATA": SimStruct(OrderedDict((("SizeOfFsFilterCallbackData", SimTypeInt(signed=False, label="UInt32")), ("Operation", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("Parameters", SimUnion({"AcquireForModifiedPageWriter": SimTypeRef("_AcquireForModifiedPageWriter_e__Struct", SimStruct), "ReleaseForModifiedPageWriter": SimTypeRef("_ReleaseForModifiedPageWriter_e__Struct", SimStruct), "AcquireForSectionSynchronization": SimTypeRef("_AcquireForSectionSynchronization_e__Struct", SimStruct), "QueryOpen": SimTypeRef("_QueryOpen_e__Struct", SimStruct), "Others": SimTypeRef("_Others_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FS_FILTER_CALLBACK_DATA", pack=False, align=None),
    "FS_FILTER_PARAMETERS": SimUnion({"AcquireForModifiedPageWriter": SimTypeRef("_AcquireForModifiedPageWriter_e__Struct", SimStruct), "ReleaseForModifiedPageWriter": SimTypeRef("_ReleaseForModifiedPageWriter_e__Struct", SimStruct), "AcquireForSectionSynchronization": SimTypeRef("_AcquireForSectionSynchronization_e__Struct", SimStruct), "QueryOpen": SimTypeRef("_QueryOpen_e__Struct", SimStruct), "Others": SimTypeRef("_Others_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "FS_FILTER_SECTION_SYNC_OUTPUT": SimStruct(OrderedDict((("StructureSize", SimTypeInt(signed=False, label="UInt32")), ("SizeReturned", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DesiredReadAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="FS_FILTER_SECTION_SYNC_OUTPUT", pack=False, align=None),
    "FS_FILTER_SECTION_SYNC_TYPE": SimTypeInt(signed=False, label="FS_FILTER_SECTION_SYNC_TYPE"),
    "FS_FILTER_STREAM_FO_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="FS_FILTER_STREAM_FO_NOTIFICATION_TYPE"),
    "FS_INFORMATION_CLASS": SimTypeInt(signed=False, label="FS_INFORMATION_CLASS"),
    "FTP_ACCESS": SimTypeInt(signed=False, label="FTP_ACCESS"),
    "FTP_FLAGS": SimTypeInt(signed=False, label="FTP_FLAGS"),
    "FTP_PROCESS_STATUS": SimTypeInt(signed=False, label="FTP_PROCESS_STATUS"),
    "FT_STATE_EVENT_CAUSE": SimTypeInt(signed=False, label="FT_STATE_EVENT_CAUSE"),
    "FULLDUPLEX_SUPPORT": SimTypeInt(signed=False, label="FULLDUPLEX_SUPPORT"),
    "FULLPROPSPEC": SimStruct(OrderedDict((("guidPropSet", SimTypeBottom(label="Guid")), ("psProperty", SimTypeRef("PROPSPEC", SimStruct)),)), name="FULLPROPSPEC", pack=False, align=None),
    "FULL_PTR_XLAT_TABLES": SimStruct(OrderedDict((("RefIdToPointer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PointerToRefId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NextRefId", SimTypeInt(signed=False, label="UInt32")), ("XlatSide", SimTypeInt(signed=False, label="XLAT_SIDE")),)), name="FULL_PTR_XLAT_TABLES", pack=False, align=None),
    "FULL_RESPONSE_PROPERTY_ID": SimTypeInt(signed=False, label="FULL_RESPONSE_PROPERTY_ID"),
    "FUNCDESC": SimStruct(OrderedDict((("memid", SimTypeInt(signed=True, label="Int32")), ("lprgscode", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lprgelemdescParam", SimTypePointer(SimTypeRef("ELEMDESC", SimStruct), offset=0)), ("funckind", SimTypeInt(signed=False, label="FUNCKIND")), ("invkind", SimTypeInt(signed=False, label="INVOKEKIND")), ("callconv", SimTypeInt(signed=False, label="CALLCONV")), ("cParams", SimTypeShort(signed=True, label="Int16")), ("cParamsOpt", SimTypeShort(signed=True, label="Int16")), ("oVft", SimTypeShort(signed=True, label="Int16")), ("cScodes", SimTypeShort(signed=True, label="Int16")), ("elemdescFunc", SimTypeRef("ELEMDESC", SimStruct)), ("wFuncFlags", SimTypeInt(signed=False, label="FUNCFLAGS")),)), name="FUNCDESC", pack=False, align=None),
    "FUNCFLAGS": SimTypeInt(signed=False, label="FUNCFLAGS"),
    "FUNCKIND": SimTypeInt(signed=False, label="FUNCKIND"),
    "FUNCTION_LEVEL_DEVICE_RESET_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceResetCompletion", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("CompletionContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="FUNCTION_LEVEL_DEVICE_RESET_PARAMETERS", pack=False, align=None),
    "FUSION_INSTALL_REFERENCE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidScheme", SimTypeBottom(label="Guid")), ("szIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szNonCannonicalData", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FUSION_INSTALL_REFERENCE", pack=False, align=None),
    "FVTEXTTYPE": SimTypeInt(signed=False, label="FVTEXTTYPE"),
    "FWMI_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Wnode", "Context"]), offset=0),
    "FWPM_ACTION0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="FWP_ACTION_TYPE")), ("Anonymous", SimUnion({"filterType": SimTypeBottom(label="Guid"), "calloutKey": SimTypeBottom(label="Guid")}, name="<anon>", label="None")),)), name="FWPM_ACTION0", pack=False, align=None),
    "FWPM_APPC_NETWORK_CAPABILITY_TYPE": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"),
    "FWPM_CALLOUT0": SimStruct(OrderedDict((("calloutKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("applicableLayer", SimTypeBottom(label="Guid")), ("calloutId", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_CALLOUT0", pack=False, align=None),
    "FWPM_CALLOUT_CHANGE0": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE")), ("calloutKey", SimTypeBottom(label="Guid")), ("calloutId", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_CALLOUT_CHANGE0", pack=False, align=None),
    "FWPM_CALLOUT_CHANGE_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_CALLOUT_CHANGE0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0),
    "FWPM_CALLOUT_ENUM_TEMPLATE0": SimStruct(OrderedDict((("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("layerKey", SimTypeBottom(label="Guid")),)), name="FWPM_CALLOUT_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_CALLOUT_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_CALLOUT_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_CALLOUT_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_CHANGE_TYPE": SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE"),
    "FWPM_CLASSIFY_OPTION0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE")), ("value", SimTypeRef("FWP_VALUE0", SimStruct)),)), name="FWPM_CLASSIFY_OPTION0", pack=False, align=None),
    "FWPM_CLASSIFY_OPTIONS0": SimStruct(OrderedDict((("numOptions", SimTypeInt(signed=False, label="UInt32")), ("options", SimTypePointer(SimTypeRef("FWPM_CLASSIFY_OPTION0", SimStruct), offset=0)),)), name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None),
    "FWPM_CONNECTION0": SimStruct(OrderedDict((("connectionId", SimTypeLongLong(signed=False, label="UInt64")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ipsecTrafficModeType", SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")), ("keyModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("mmCrypto", SimTypeRef("IKEEXT_PROPOSAL0", SimStruct)), ("mmPeer", SimTypeRef("IKEEXT_CREDENTIAL2", SimStruct)), ("emPeer", SimTypeRef("IKEEXT_CREDENTIAL2", SimStruct)), ("bytesTransferredIn", SimTypeLongLong(signed=False, label="UInt64")), ("bytesTransferredOut", SimTypeLongLong(signed=False, label="UInt64")), ("bytesTransferredTotal", SimTypeLongLong(signed=False, label="UInt64")), ("startSysTime", SimTypeRef("FILETIME", SimStruct)),)), name="FWPM_CONNECTION0", pack=False, align=None),
    "FWPM_CONNECTION_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="FWPM_CONNECTION_EVENT_TYPE"), SimTypePointer(SimTypeRef("FWPM_CONNECTION0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "eventType", "connection"]), offset=0),
    "FWPM_CONNECTION_ENUM_TEMPLATE0": SimStruct(OrderedDict((("connectionId", SimTypeLongLong(signed=False, label="UInt64")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_CONNECTION_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_CONNECTION_EVENT_TYPE": SimTypeInt(signed=False, label="FWPM_CONNECTION_EVENT_TYPE"),
    "FWPM_CONNECTION_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_CONNECTION_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_CONNECTION_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_DISPLAY_DATA0": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("description", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FWPM_DISPLAY_DATA0", pack=False, align=None),
    "FWPM_DYNAMIC_KEYWORD_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["notification", "context"]), offset=0),
    "FWPM_ENGINE_OPTION": SimTypeInt(signed=False, label="FWPM_ENGINE_OPTION"),
    "FWPM_FIELD0": SimStruct(OrderedDict((("fieldKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("type", SimTypeInt(signed=False, label="FWPM_FIELD_TYPE")), ("dataType", SimTypeInt(signed=False, label="FWP_DATA_TYPE")),)), name="FWPM_FIELD0", pack=False, align=None),
    "FWPM_FIELD_TYPE": SimTypeInt(signed=False, label="FWPM_FIELD_TYPE"),
    "FWPM_FILTER0": SimStruct(OrderedDict((("filterKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="FWPM_FILTER_FLAGS")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("layerKey", SimTypeBottom(label="Guid")), ("subLayerKey", SimTypeBottom(label="Guid")), ("weight", SimTypeRef("FWP_VALUE0", SimStruct)), ("numFilterConditions", SimTypeInt(signed=False, label="UInt32")), ("filterCondition", SimTypePointer(SimTypeRef("FWPM_FILTER_CONDITION0", SimStruct), offset=0)), ("action", SimTypeRef("FWPM_ACTION0", SimStruct)), ("Anonymous", SimUnion({"rawContext": SimTypeLongLong(signed=False, label="UInt64"), "providerContextKey": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("reserved", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("effectiveWeight", SimTypeRef("FWP_VALUE0", SimStruct)),)), name="FWPM_FILTER0", pack=False, align=None),
    "FWPM_FILTER_CHANGE0": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE")), ("filterKey", SimTypeBottom(label="Guid")), ("filterId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_FILTER_CHANGE0", pack=False, align=None),
    "FWPM_FILTER_CHANGE_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_FILTER_CHANGE0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0),
    "FWPM_FILTER_CONDITION0": SimStruct(OrderedDict((("fieldKey", SimTypeBottom(label="Guid")), ("matchType", SimTypeInt(signed=False, label="FWP_MATCH_TYPE")), ("conditionValue", SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)),)), name="FWPM_FILTER_CONDITION0", pack=False, align=None),
    "FWPM_FILTER_ENUM_TEMPLATE0": SimStruct(OrderedDict((("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("layerKey", SimTypeBottom(label="Guid")), ("enumType", SimTypeInt(signed=False, label="FWP_FILTER_ENUM_TYPE")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerContextTemplate", SimTypePointer(SimTypeRef("FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", SimStruct), offset=0)), ("numFilterConditions", SimTypeInt(signed=False, label="UInt32")), ("filterCondition", SimTypePointer(SimTypeRef("FWPM_FILTER_CONDITION0", SimStruct), offset=0)), ("actionMask", SimTypeInt(signed=False, label="UInt32")), ("calloutKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="FWPM_FILTER_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_FILTER_FLAGS": SimTypeInt(signed=False, label="FWPM_FILTER_FLAGS"),
    "FWPM_FILTER_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_FILTER_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_FILTER_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_LAYER0": SimStruct(OrderedDict((("layerKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("numFields", SimTypeInt(signed=False, label="UInt32")), ("field", SimTypePointer(SimTypeRef("FWPM_FIELD0", SimStruct), offset=0)), ("defaultSubLayerKey", SimTypeBottom(label="Guid")), ("layerId", SimTypeShort(signed=False, label="UInt16")),)), name="FWPM_LAYER0", pack=False, align=None),
    "FWPM_LAYER_ENUM_TEMPLATE0": SimStruct(OrderedDict((("reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_LAYER_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_LAYER_STATISTICS0": SimStruct(OrderedDict((("layerId", SimTypeBottom(label="Guid")), ("classifyPermitCount", SimTypeInt(signed=False, label="UInt32")), ("classifyBlockCount", SimTypeInt(signed=False, label="UInt32")), ("classifyVetoCount", SimTypeInt(signed=False, label="UInt32")), ("numCacheEntries", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_LAYER_STATISTICS0", pack=False, align=None),
    "FWPM_NETWORK_CONNECTION_POLICY_SETTING0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="FWP_NETWORK_CONNECTION_POLICY_SETTING_TYPE")), ("value", SimTypeRef("FWP_VALUE0", SimStruct)),)), name="FWPM_NETWORK_CONNECTION_POLICY_SETTING0", pack=False, align=None),
    "FWPM_NETWORK_CONNECTION_POLICY_SETTINGS0": SimStruct(OrderedDict((("numSettings", SimTypeInt(signed=False, label="UInt32")), ("settings", SimTypePointer(SimTypeRef("FWPM_NETWORK_CONNECTION_POLICY_SETTING0", SimStruct), offset=0)),)), name="FWPM_NETWORK_CONNECTION_POLICY_SETTINGS0", pack=False, align=None),
    "FWPM_NET_EVENT0": SimStruct(OrderedDict((("header", SimTypeRef("FWPM_NET_EVENT_HEADER0", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE")), ("Anonymous", SimUnion({"ikeMmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_MM_FAILURE0", SimStruct), offset=0), "ikeQmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", SimStruct), offset=0), "ikeEmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_EM_FAILURE0", SimStruct), offset=0), "classifyDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP0", SimStruct), offset=0), "ipsecDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", SimStruct), offset=0), "idpDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_DOSP_DROP0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT0", pack=False, align=None),
    "FWPM_NET_EVENT1": SimStruct(OrderedDict((("header", SimTypeRef("FWPM_NET_EVENT_HEADER1", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE")), ("Anonymous", SimUnion({"ikeMmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", SimStruct), offset=0), "ikeQmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", SimStruct), offset=0), "ikeEmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", SimStruct), offset=0), "classifyDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP1", SimStruct), offset=0), "ipsecDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", SimStruct), offset=0), "idpDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_DOSP_DROP0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT1", pack=False, align=None),
    "FWPM_NET_EVENT2": SimStruct(OrderedDict((("header", SimTypeRef("FWPM_NET_EVENT_HEADER2", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE")), ("Anonymous", SimUnion({"ikeMmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", SimStruct), offset=0), "ikeQmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", SimStruct), offset=0), "ikeEmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", SimStruct), offset=0), "classifyDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP2", SimStruct), offset=0), "ipsecDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", SimStruct), offset=0), "idpDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_DOSP_DROP0", SimStruct), offset=0), "classifyAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_ALLOW0", SimStruct), offset=0), "capabilityDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_DROP0", SimStruct), offset=0), "capabilityAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_ALLOW0", SimStruct), offset=0), "classifyDropMac": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT2", pack=False, align=None),
    "FWPM_NET_EVENT3": SimStruct(OrderedDict((("header", SimTypeRef("FWPM_NET_EVENT_HEADER3", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE")), ("Anonymous", SimUnion({"ikeMmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", SimStruct), offset=0), "ikeQmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", SimStruct), offset=0), "ikeEmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", SimStruct), offset=0), "classifyDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP2", SimStruct), offset=0), "ipsecDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", SimStruct), offset=0), "idpDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_DOSP_DROP0", SimStruct), offset=0), "classifyAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_ALLOW0", SimStruct), offset=0), "capabilityDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_DROP0", SimStruct), offset=0), "capabilityAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_ALLOW0", SimStruct), offset=0), "classifyDropMac": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT3", pack=False, align=None),
    "FWPM_NET_EVENT4": SimStruct(OrderedDict((("header", SimTypeRef("FWPM_NET_EVENT_HEADER3", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE")), ("Anonymous", SimUnion({"ikeMmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_MM_FAILURE2", SimStruct), offset=0), "ikeQmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_QM_FAILURE1", SimStruct), offset=0), "ikeEmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", SimStruct), offset=0), "classifyDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP2", SimStruct), offset=0), "ipsecDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", SimStruct), offset=0), "idpDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_DOSP_DROP0", SimStruct), offset=0), "classifyAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_ALLOW0", SimStruct), offset=0), "capabilityDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_DROP0", SimStruct), offset=0), "capabilityAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_ALLOW0", SimStruct), offset=0), "classifyDropMac": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT4", pack=False, align=None),
    "FWPM_NET_EVENT5": SimStruct(OrderedDict((("header", SimTypeRef("FWPM_NET_EVENT_HEADER3", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE")), ("Anonymous", SimUnion({"ikeMmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_MM_FAILURE2", SimStruct), offset=0), "ikeQmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_QM_FAILURE1", SimStruct), offset=0), "ikeEmFailure": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", SimStruct), offset=0), "classifyDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP2", SimStruct), offset=0), "ipsecDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", SimStruct), offset=0), "idpDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_IPSEC_DOSP_DROP0", SimStruct), offset=0), "classifyAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_ALLOW0", SimStruct), offset=0), "capabilityDrop": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_DROP0", SimStruct), offset=0), "capabilityAllow": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CAPABILITY_ALLOW0", SimStruct), offset=0), "classifyDropMac": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", SimStruct), offset=0), "lpmPacketArrival": SimTypePointer(SimTypeRef("FWPM_NET_EVENT_LPM_PACKET_ARRIVAL0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT5", pack=False, align=None),
    "FWPM_NET_EVENT_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_NET_EVENT1", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0),
    "FWPM_NET_EVENT_CALLBACK1": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_NET_EVENT2", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0),
    "FWPM_NET_EVENT_CALLBACK2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_NET_EVENT3", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0),
    "FWPM_NET_EVENT_CALLBACK3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_NET_EVENT4", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0),
    "FWPM_NET_EVENT_CALLBACK4": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_NET_EVENT5", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0),
    "FWPM_NET_EVENT_CAPABILITY_ALLOW0": SimStruct(OrderedDict((("networkCapabilityId", SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE")), ("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("isLoopback", SimTypeInt(signed=True, label="Int32")),)), name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None),
    "FWPM_NET_EVENT_CAPABILITY_DROP0": SimStruct(OrderedDict((("networkCapabilityId", SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE")), ("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("isLoopback", SimTypeInt(signed=True, label="Int32")),)), name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None),
    "FWPM_NET_EVENT_CLASSIFY_ALLOW0": SimStruct(OrderedDict((("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("layerId", SimTypeShort(signed=False, label="UInt16")), ("reauthReason", SimTypeInt(signed=False, label="UInt32")), ("originalProfile", SimTypeInt(signed=False, label="UInt32")), ("currentProfile", SimTypeInt(signed=False, label="UInt32")), ("msFwpDirection", SimTypeInt(signed=False, label="UInt32")), ("isLoopback", SimTypeInt(signed=True, label="Int32")),)), name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None),
    "FWPM_NET_EVENT_CLASSIFY_DROP0": SimStruct(OrderedDict((("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("layerId", SimTypeShort(signed=False, label="UInt16")),)), name="FWPM_NET_EVENT_CLASSIFY_DROP0", pack=False, align=None),
    "FWPM_NET_EVENT_CLASSIFY_DROP1": SimStruct(OrderedDict((("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("layerId", SimTypeShort(signed=False, label="UInt16")), ("reauthReason", SimTypeInt(signed=False, label="UInt32")), ("originalProfile", SimTypeInt(signed=False, label="UInt32")), ("currentProfile", SimTypeInt(signed=False, label="UInt32")), ("msFwpDirection", SimTypeInt(signed=False, label="UInt32")), ("isLoopback", SimTypeInt(signed=True, label="Int32")),)), name="FWPM_NET_EVENT_CLASSIFY_DROP1", pack=False, align=None),
    "FWPM_NET_EVENT_CLASSIFY_DROP2": SimStruct(OrderedDict((("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("layerId", SimTypeShort(signed=False, label="UInt16")), ("reauthReason", SimTypeInt(signed=False, label="UInt32")), ("originalProfile", SimTypeInt(signed=False, label="UInt32")), ("currentProfile", SimTypeInt(signed=False, label="UInt32")), ("msFwpDirection", SimTypeInt(signed=False, label="UInt32")), ("isLoopback", SimTypeInt(signed=True, label="Int32")), ("vSwitchId", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("vSwitchSourcePort", SimTypeInt(signed=False, label="UInt32")), ("vSwitchDestinationPort", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None),
    "FWPM_NET_EVENT_CLASSIFY_DROP_MAC0": SimStruct(OrderedDict((("localMacAddr", SimTypeRef("FWP_BYTE_ARRAY6", SimStruct)), ("remoteMacAddr", SimTypeRef("FWP_BYTE_ARRAY6", SimStruct)), ("mediaType", SimTypeInt(signed=False, label="UInt32")), ("ifType", SimTypeInt(signed=False, label="UInt32")), ("etherType", SimTypeShort(signed=False, label="UInt16")), ("ndisPortNumber", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypeInt(signed=False, label="UInt32")), ("vlanTag", SimTypeShort(signed=False, label="UInt16")), ("ifLuid", SimTypeLongLong(signed=False, label="UInt64")), ("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("layerId", SimTypeShort(signed=False, label="UInt16")), ("reauthReason", SimTypeInt(signed=False, label="UInt32")), ("originalProfile", SimTypeInt(signed=False, label="UInt32")), ("currentProfile", SimTypeInt(signed=False, label="UInt32")), ("msFwpDirection", SimTypeInt(signed=False, label="UInt32")), ("isLoopback", SimTypeInt(signed=True, label="Int32")), ("vSwitchId", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("vSwitchSourcePort", SimTypeInt(signed=False, label="UInt32")), ("vSwitchDestinationPort", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None),
    "FWPM_NET_EVENT_ENUM_TEMPLATE0": SimStruct(OrderedDict((("startTime", SimTypeRef("FILETIME", SimStruct)), ("endTime", SimTypeRef("FILETIME", SimStruct)), ("numFilterConditions", SimTypeInt(signed=False, label="UInt32")), ("filterCondition", SimTypePointer(SimTypeRef("FWPM_FILTER_CONDITION0", SimStruct), offset=0)),)), name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_NET_EVENT_HEADER0": SimStruct(OrderedDict((("timeStamp", SimTypeRef("FILETIME", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("ipProtocol", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("localPort", SimTypeShort(signed=False, label="UInt16")), ("remotePort", SimTypeShort(signed=False, label="UInt16")), ("scopeId", SimTypeInt(signed=False, label="UInt32")), ("appId", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("userId", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)),)), name="FWPM_NET_EVENT_HEADER0", pack=False, align=None),
    "FWPM_NET_EVENT_HEADER1": SimStruct(OrderedDict((("timeStamp", SimTypeRef("FILETIME", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("ipProtocol", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("localPort", SimTypeShort(signed=False, label="UInt16")), ("remotePort", SimTypeShort(signed=False, label="UInt16")), ("scopeId", SimTypeInt(signed=False, label="UInt32")), ("appId", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("userId", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("Anonymous3", SimUnion({"Anonymous": SimStruct(OrderedDict((("reserved1", SimTypeInt(signed=False, label="FWP_AF")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("reserved2", SimTypeRef("FWP_BYTE_ARRAY6", SimStruct)), ("reserved3", SimTypeRef("FWP_BYTE_ARRAY6", SimStruct)), ("reserved4", SimTypeInt(signed=False, label="UInt32")), ("reserved5", SimTypeInt(signed=False, label="UInt32")), ("reserved6", SimTypeShort(signed=False, label="UInt16")), ("reserved7", SimTypeInt(signed=False, label="UInt32")), ("reserved8", SimTypeInt(signed=False, label="UInt32")), ("reserved9", SimTypeShort(signed=False, label="UInt16")), ("reserved10", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="FWPM_NET_EVENT_HEADER1", pack=False, align=None),
    "FWPM_NET_EVENT_HEADER2": SimStruct(OrderedDict((("timeStamp", SimTypeRef("FILETIME", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("ipProtocol", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("localPort", SimTypeShort(signed=False, label="UInt16")), ("remotePort", SimTypeShort(signed=False, label="UInt16")), ("scopeId", SimTypeInt(signed=False, label="UInt32")), ("appId", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("userId", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("addressFamily", SimTypeInt(signed=False, label="FWP_AF")), ("packageSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)),)), name="FWPM_NET_EVENT_HEADER2", pack=False, align=None),
    "FWPM_NET_EVENT_HEADER3": SimStruct(OrderedDict((("timeStamp", SimTypeRef("FILETIME", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("ipProtocol", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)}, name="<anon>", label="None")), ("localPort", SimTypeShort(signed=False, label="UInt16")), ("remotePort", SimTypeShort(signed=False, label="UInt16")), ("scopeId", SimTypeInt(signed=False, label="UInt32")), ("appId", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("userId", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("addressFamily", SimTypeInt(signed=False, label="FWP_AF")), ("packageSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("enterpriseId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("policyFlags", SimTypeLongLong(signed=False, label="UInt64")), ("effectiveName", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="FWPM_NET_EVENT_HEADER3", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_EM_FAILURE0": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("emState", SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("emAuthMethod", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("endCertHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("mmId", SimTypeLongLong(signed=False, label="UInt64")), ("qmFilterId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE0", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_EM_FAILURE1": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("emState", SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("emAuthMethod", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("endCertHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("mmId", SimTypeLongLong(signed=False, label="UInt64")), ("qmFilterId", SimTypeLongLong(signed=False, label="UInt64")), ("localPrincipalNameForAuth", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("remotePrincipalNameForAuth", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("numLocalPrincipalGroupSids", SimTypeInt(signed=False, label="UInt32")), ("localPrincipalGroupSids", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("numRemotePrincipalGroupSids", SimTypeInt(signed=False, label="UInt32")), ("remotePrincipalGroupSids", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("saTrafficType", SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")),)), name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_MM_FAILURE0": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("keyingModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("mmState", SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("mmAuthMethod", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("endCertHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("mmId", SimTypeLongLong(signed=False, label="UInt64")), ("mmFilterId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE0", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_MM_FAILURE1": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("keyingModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("mmState", SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("mmAuthMethod", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("endCertHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("mmId", SimTypeLongLong(signed=False, label="UInt64")), ("mmFilterId", SimTypeLongLong(signed=False, label="UInt64")), ("localPrincipalNameForAuth", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("remotePrincipalNameForAuth", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("numLocalPrincipalGroupSids", SimTypeInt(signed=False, label="UInt32")), ("localPrincipalGroupSids", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("numRemotePrincipalGroupSids", SimTypeInt(signed=False, label="UInt32")), ("remotePrincipalGroupSids", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_MM_FAILURE2": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("keyingModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("mmState", SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("mmAuthMethod", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("endCertHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("mmId", SimTypeLongLong(signed=False, label="UInt64")), ("mmFilterId", SimTypeLongLong(signed=False, label="UInt64")), ("localPrincipalNameForAuth", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("remotePrincipalNameForAuth", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("numLocalPrincipalGroupSids", SimTypeInt(signed=False, label="UInt32")), ("localPrincipalGroupSids", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("numRemotePrincipalGroupSids", SimTypeInt(signed=False, label="UInt32")), ("remotePrincipalGroupSids", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("providerContextKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE2", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_QM_FAILURE0": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("keyingModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("qmState", SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("saTrafficType", SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")), ("Anonymous1", SimUnion({"localSubNet": SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteSubNet": SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)}, name="<anon>", label="None")), ("qmFilterId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None),
    "FWPM_NET_EVENT_IKEEXT_QM_FAILURE1": SimStruct(OrderedDict((("failureErrorCode", SimTypeInt(signed=False, label="UInt32")), ("failurePoint", SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT")), ("keyingModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("qmState", SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE")), ("saRole", SimTypeInt(signed=False, label="IKEEXT_SA_ROLE")), ("saTrafficType", SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")), ("Anonymous1", SimUnion({"localSubNet": SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteSubNet": SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)}, name="<anon>", label="None")), ("qmFilterId", SimTypeLongLong(signed=False, label="UInt64")), ("mmSaLuid", SimTypeLongLong(signed=False, label="UInt64")), ("mmProviderContextKey", SimTypeBottom(label="Guid")),)), name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE1", pack=False, align=None),
    "FWPM_NET_EVENT_IPSEC_DOSP_DROP0": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("failureStatus", SimTypeInt(signed=True, label="Int32")), ("direction", SimTypeInt(signed=False, label="FWP_DIRECTION")),)), name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None),
    "FWPM_NET_EVENT_IPSEC_KERNEL_DROP0": SimStruct(OrderedDict((("failureStatus", SimTypeInt(signed=True, label="Int32")), ("direction", SimTypeInt(signed=False, label="FWP_DIRECTION")), ("spi", SimTypeInt(signed=False, label="UInt32")), ("filterId", SimTypeLongLong(signed=False, label="UInt64")), ("layerId", SimTypeShort(signed=False, label="UInt16")),)), name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None),
    "FWPM_NET_EVENT_LPM_PACKET_ARRIVAL0": SimStruct(OrderedDict((("spi", SimTypeInt(signed=False, label="UInt32")),)), name="FWPM_NET_EVENT_LPM_PACKET_ARRIVAL0", pack=False, align=None),
    "FWPM_NET_EVENT_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_NET_EVENT_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_NET_EVENT_TYPE": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"),
    "FWPM_PROVIDER0": SimStruct(OrderedDict((("providerKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("serviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FWPM_PROVIDER0", pack=False, align=None),
    "FWPM_PROVIDER_CHANGE0": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE")), ("providerKey", SimTypeBottom(label="Guid")),)), name="FWPM_PROVIDER_CHANGE0", pack=False, align=None),
    "FWPM_PROVIDER_CHANGE_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_PROVIDER_CHANGE0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0),
    "FWPM_PROVIDER_CONTEXT0": SimStruct(OrderedDict((("providerContextKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")), ("Anonymous", SimUnion({"keyingPolicy": SimTypePointer(SimTypeRef("IPSEC_KEYING_POLICY0", SimStruct), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY0", SimStruct), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY0", SimStruct), offset=0), "authipQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY0", SimStruct), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY0", SimStruct), offset=0), "ikeMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY0", SimStruct), offset=0), "authIpMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY0", SimStruct), offset=0), "dataBuffer": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "classifyOptions": SimTypePointer(SimTypeRef("FWPM_CLASSIFY_OPTIONS0", SimStruct), offset=0)}, name="<anon>", label="None")), ("providerContextId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT1": SimStruct(OrderedDict((("providerContextKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")), ("Anonymous", SimUnion({"keyingPolicy": SimTypePointer(SimTypeRef("IPSEC_KEYING_POLICY0", SimStruct), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY1", SimStruct), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY1", SimStruct), offset=0), "authipQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY1", SimStruct), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY1", SimStruct), offset=0), "ikeMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY1", SimStruct), offset=0), "authIpMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY1", SimStruct), offset=0), "dataBuffer": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "classifyOptions": SimTypePointer(SimTypeRef("FWPM_CLASSIFY_OPTIONS0", SimStruct), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY1", SimStruct), offset=0), "ikeV2MmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY1", SimStruct), offset=0), "idpOptions": SimTypePointer(SimTypeRef("IPSEC_DOSP_OPTIONS0", SimStruct), offset=0)}, name="<anon>", label="None")), ("providerContextId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT2": SimStruct(OrderedDict((("providerContextKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")), ("Anonymous", SimUnion({"keyingPolicy": SimTypePointer(SimTypeRef("IPSEC_KEYING_POLICY1", SimStruct), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY2", SimStruct), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY2", SimStruct), offset=0), "authipQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY2", SimStruct), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY2", SimStruct), offset=0), "ikeMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY2", SimStruct), offset=0), "authIpMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY2", SimStruct), offset=0), "dataBuffer": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "classifyOptions": SimTypePointer(SimTypeRef("FWPM_CLASSIFY_OPTIONS0", SimStruct), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY2", SimStruct), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY2", SimStruct), offset=0), "ikeV2MmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY2", SimStruct), offset=0), "idpOptions": SimTypePointer(SimTypeRef("IPSEC_DOSP_OPTIONS0", SimStruct), offset=0)}, name="<anon>", label="None")), ("providerContextId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT3": SimStruct(OrderedDict((("providerContextKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("type", SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")), ("Anonymous", SimUnion({"keyingPolicy": SimTypePointer(SimTypeRef("IPSEC_KEYING_POLICY1", SimStruct), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY2", SimStruct), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY3", SimStruct), offset=0), "authipQmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY2", SimStruct), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY3", SimStruct), offset=0), "ikeMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY2", SimStruct), offset=0), "authIpMmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY2", SimStruct), offset=0), "dataBuffer": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "classifyOptions": SimTypePointer(SimTypeRef("FWPM_CLASSIFY_OPTIONS0", SimStruct), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimTypeRef("IPSEC_TUNNEL_POLICY3", SimStruct), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimTypeRef("IPSEC_TRANSPORT_POLICY2", SimStruct), offset=0), "ikeV2MmPolicy": SimTypePointer(SimTypeRef("IKEEXT_POLICY2", SimStruct), offset=0), "idpOptions": SimTypePointer(SimTypeRef("IPSEC_DOSP_OPTIONS0", SimStruct), offset=0), "networkConnectionPolicy": SimTypePointer(SimTypeRef("FWPM_NETWORK_CONNECTION_POLICY_SETTINGS0", SimStruct), offset=0)}, name="<anon>", label="None")), ("providerContextId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_PROVIDER_CONTEXT3", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT_CHANGE0": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE")), ("providerContextKey", SimTypeBottom(label="Guid")), ("providerContextId", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_PROVIDER_CONTEXT_CHANGE0", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT_CHANGE_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_PROVIDER_CONTEXT_CHANGE0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0),
    "FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0": SimStruct(OrderedDict((("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerContextType", SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")),)), name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_PROVIDER_CONTEXT_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_PROVIDER_CONTEXT_TYPE": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"),
    "FWPM_PROVIDER_ENUM_TEMPLATE0": SimStruct(OrderedDict((("reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_PROVIDER_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_PROVIDER_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_PROVIDER_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_PROVIDER_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_SERVICE_STATE": SimTypeInt(signed=False, label="FWPM_SERVICE_STATE"),
    "FWPM_SERVICE_STATE_CHANGE_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="FWPM_SERVICE_STATE")], SimTypeBottom(label="Void"), arg_names=["context", "newState"]), offset=0),
    "FWPM_SESSION0": SimStruct(OrderedDict((("sessionKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("txnWaitTimeoutInMSec", SimTypeInt(signed=False, label="UInt32")), ("processId", SimTypeInt(signed=False, label="UInt32")), ("sid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("kernelMode", SimTypeInt(signed=True, label="Int32")),)), name="FWPM_SESSION0", pack=False, align=None),
    "FWPM_SESSION_ENUM_TEMPLATE0": SimStruct(OrderedDict((("reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_SESSION_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_STATISTICS0": SimStruct(OrderedDict((("numLayerStatistics", SimTypeInt(signed=False, label="UInt32")), ("layerStatistics", SimTypePointer(SimTypeRef("FWPM_LAYER_STATISTICS0", SimStruct), offset=0)), ("inboundAllowedConnectionsV4", SimTypeInt(signed=False, label="UInt32")), ("inboundBlockedConnectionsV4", SimTypeInt(signed=False, label="UInt32")), ("outboundAllowedConnectionsV4", SimTypeInt(signed=False, label="UInt32")), ("outboundBlockedConnectionsV4", SimTypeInt(signed=False, label="UInt32")), ("inboundAllowedConnectionsV6", SimTypeInt(signed=False, label="UInt32")), ("inboundBlockedConnectionsV6", SimTypeInt(signed=False, label="UInt32")), ("outboundAllowedConnectionsV6", SimTypeInt(signed=False, label="UInt32")), ("outboundBlockedConnectionsV6", SimTypeInt(signed=False, label="UInt32")), ("inboundActiveConnectionsV4", SimTypeInt(signed=False, label="UInt32")), ("outboundActiveConnectionsV4", SimTypeInt(signed=False, label="UInt32")), ("inboundActiveConnectionsV6", SimTypeInt(signed=False, label="UInt32")), ("outboundActiveConnectionsV6", SimTypeInt(signed=False, label="UInt32")), ("reauthDirInbound", SimTypeLongLong(signed=False, label="UInt64")), ("reauthDirOutbound", SimTypeLongLong(signed=False, label="UInt64")), ("reauthFamilyV4", SimTypeLongLong(signed=False, label="UInt64")), ("reauthFamilyV6", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoOther", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoIPv4", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoIPv6", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoICMP", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoICMP6", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoUDP", SimTypeLongLong(signed=False, label="UInt64")), ("reauthProtoTCP", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonPolicyChange", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonNewArrivalInterface", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonNewNextHopInterface", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonProfileCrossing", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonClassifyCompletion", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonIPSecPropertiesChanged", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonMidStreamInspection", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonSocketPropertyChanged", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonNewInboundMCastBCastPacket", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonEDPPolicyChanged", SimTypeLongLong(signed=False, label="UInt64")), ("reauthReasonProxyHandleChanged", SimTypeLongLong(signed=False, label="UInt64")),)), name="FWPM_STATISTICS0", pack=False, align=None),
    "FWPM_SUBLAYER0": SimStruct(OrderedDict((("subLayerKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("providerData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("weight", SimTypeShort(signed=False, label="UInt16")),)), name="FWPM_SUBLAYER0", pack=False, align=None),
    "FWPM_SUBLAYER_CHANGE0": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE")), ("subLayerKey", SimTypeBottom(label="Guid")),)), name="FWPM_SUBLAYER_CHANGE0", pack=False, align=None),
    "FWPM_SUBLAYER_CHANGE_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_SUBLAYER_CHANGE0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0),
    "FWPM_SUBLAYER_ENUM_TEMPLATE0": SimStruct(OrderedDict((("providerKey", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="FWPM_SUBLAYER_ENUM_TEMPLATE0", pack=False, align=None),
    "FWPM_SUBLAYER_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("FWPM_SUBLAYER_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_SUBLAYER_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_SUBSCRIPTION_FLAGS": SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS"),
    "FWPM_SYSTEM_PORTS0": SimStruct(OrderedDict((("numTypes", SimTypeInt(signed=False, label="UInt32")), ("types", SimTypePointer(SimTypeRef("FWPM_SYSTEM_PORTS_BY_TYPE0", SimStruct), offset=0)),)), name="FWPM_SYSTEM_PORTS0", pack=False, align=None),
    "FWPM_SYSTEM_PORTS_BY_TYPE0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="FWPM_SYSTEM_PORT_TYPE")), ("numPorts", SimTypeInt(signed=False, label="UInt32")), ("ports", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="FWPM_SYSTEM_PORTS_BY_TYPE0", pack=False, align=None),
    "FWPM_SYSTEM_PORTS_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_SYSTEM_PORTS0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sysPorts"]), offset=0),
    "FWPM_SYSTEM_PORT_TYPE": SimTypeInt(signed=False, label="FWPM_SYSTEM_PORT_TYPE"),
    "FWPM_VSWITCH_EVENT0": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="FWPM_VSWITCH_EVENT_TYPE")), ("vSwitchId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"positionInfo": SimTypeRef("_positionInfo_e__Struct", SimStruct), "reorderInfo": SimTypeRef("_reorderInfo_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="FWPM_VSWITCH_EVENT0", pack=False, align=None),
    "FWPM_VSWITCH_EVENT_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FWPM_VSWITCH_EVENT0", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "vSwitchEvent"]), offset=0),
    "FWPM_VSWITCH_EVENT_SUBSCRIPTION0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="FWPM_VSWITCH_EVENT_SUBSCRIPTION0", pack=False, align=None),
    "FWPM_VSWITCH_EVENT_TYPE": SimTypeInt(signed=False, label="FWPM_VSWITCH_EVENT_TYPE"),
    "FWP_ACTION_TYPE": SimTypeInt(signed=False, label="FWP_ACTION_TYPE"),
    "FWP_AF": SimTypeInt(signed=False, label="FWP_AF"),
    "FWP_BYTE_ARRAY16": SimStruct(OrderedDict((("byteArray16", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="FWP_BYTE_ARRAY16", pack=False, align=None),
    "FWP_BYTE_ARRAY6": SimStruct(OrderedDict((("byteArray6", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="FWP_BYTE_ARRAY6", pack=False, align=None),
    "FWP_BYTE_BLOB": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="FWP_BYTE_BLOB", pack=False, align=None),
    "FWP_CLASSIFY_OPTION_TYPE": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"),
    "FWP_CONDITION_VALUE0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="FWP_DATA_TYPE")), ("Anonymous", SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimTypeRef("FWP_BYTE_ARRAY16", SimStruct), offset=0), "byteBlob": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "sid": SimTypePointer(SimTypeRef("SID", SimStruct), offset=0), "sd": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "tokenInformation": SimTypePointer(SimTypeRef("FWP_TOKEN_INFORMATION", SimStruct), offset=0), "tokenAccessInformation": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimTypeRef("FWP_BYTE_ARRAY6", SimStruct), offset=0), "v4AddrMask": SimTypePointer(SimTypeRef("FWP_V4_ADDR_AND_MASK", SimStruct), offset=0), "v6AddrMask": SimTypePointer(SimTypeRef("FWP_V6_ADDR_AND_MASK", SimStruct), offset=0), "rangeValue": SimTypePointer(SimTypeRef("FWP_RANGE0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWP_CONDITION_VALUE0", pack=False, align=None),
    "FWP_DATA_TYPE": SimTypeInt(signed=False, label="FWP_DATA_TYPE"),
    "FWP_DIRECTION": SimTypeInt(signed=False, label="FWP_DIRECTION"),
    "FWP_ETHER_ENCAP_METHOD": SimTypeInt(signed=False, label="FWP_ETHER_ENCAP_METHOD"),
    "FWP_FILTER_ENUM_TYPE": SimTypeInt(signed=False, label="FWP_FILTER_ENUM_TYPE"),
    "FWP_IP_VERSION": SimTypeInt(signed=False, label="FWP_IP_VERSION"),
    "FWP_MATCH_TYPE": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"),
    "FWP_NETWORK_CONNECTION_POLICY_SETTING_TYPE": SimTypeInt(signed=False, label="FWP_NETWORK_CONNECTION_POLICY_SETTING_TYPE"),
    "FWP_RANGE0": SimStruct(OrderedDict((("valueLow", SimTypeRef("FWP_VALUE0", SimStruct)), ("valueHigh", SimTypeRef("FWP_VALUE0", SimStruct)),)), name="FWP_RANGE0", pack=False, align=None),
    "FWP_TOKEN_INFORMATION": SimStruct(OrderedDict((("sidCount", SimTypeInt(signed=False, label="UInt32")), ("sids", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)), ("restrictedSidCount", SimTypeInt(signed=False, label="UInt32")), ("restrictedSids", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)),)), name="FWP_TOKEN_INFORMATION", pack=False, align=None),
    "FWP_V4_ADDR_AND_MASK": SimStruct(OrderedDict((("addr", SimTypeInt(signed=False, label="UInt32")), ("mask", SimTypeInt(signed=False, label="UInt32")),)), name="FWP_V4_ADDR_AND_MASK", pack=False, align=None),
    "FWP_V6_ADDR_AND_MASK": SimStruct(OrderedDict((("addr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("prefixLength", SimTypeChar(label="Byte")),)), name="FWP_V6_ADDR_AND_MASK", pack=False, align=None),
    "FWP_VALUE0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="FWP_DATA_TYPE")), ("Anonymous", SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimTypeRef("FWP_BYTE_ARRAY16", SimStruct), offset=0), "byteBlob": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "sid": SimTypePointer(SimTypeRef("SID", SimStruct), offset=0), "sd": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "tokenInformation": SimTypePointer(SimTypeRef("FWP_TOKEN_INFORMATION", SimStruct), offset=0), "tokenAccessInformation": SimTypePointer(SimTypeRef("FWP_BYTE_BLOB", SimStruct), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimTypeRef("FWP_BYTE_ARRAY6", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="FWP_VALUE0", pack=False, align=None),
    "FWP_VSWITCH_NETWORK_TYPE": SimTypeInt(signed=False, label="FWP_VSWITCH_NETWORK_TYPE"),
    "FW_DYNAMIC_KEYWORD_ADDRESS0": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("keyword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("addresses", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="FW_DYNAMIC_KEYWORD_ADDRESS0", pack=False, align=None),
    "FW_DYNAMIC_KEYWORD_ADDRESS_DATA0": SimStruct(OrderedDict((("dynamicKeywordAddress", SimTypeRef("FW_DYNAMIC_KEYWORD_ADDRESS0", SimStruct)), ("next", SimTypePointer(SimTypeRef("FW_DYNAMIC_KEYWORD_ADDRESS_DATA0", SimStruct), offset=0)), ("schemaVersion", SimTypeShort(signed=False, label="UInt16")), ("originType", SimTypeInt(signed=False, label="FW_DYNAMIC_KEYWORD_ORIGIN_TYPE")),)), name="FW_DYNAMIC_KEYWORD_ADDRESS_DATA0", pack=False, align=None),
    "FW_DYNAMIC_KEYWORD_ADDRESS_ENUM_FLAGS": SimTypeInt(signed=False, label="FW_DYNAMIC_KEYWORD_ADDRESS_ENUM_FLAGS"),
    "FW_DYNAMIC_KEYWORD_ADDRESS_FLAGS": SimTypeInt(signed=False, label="FW_DYNAMIC_KEYWORD_ADDRESS_FLAGS"),
    "FW_DYNAMIC_KEYWORD_ORIGIN_TYPE": SimTypeInt(signed=False, label="FW_DYNAMIC_KEYWORD_ORIGIN_TYPE"),
    "FXECHO_INITDATA": SimStruct(OrderedDict((("MaxDelay", SimTypeFloat(size=32)),)), name="FXECHO_INITDATA", pack=False, align=None),
    "FXECHO_PARAMETERS": SimStruct(OrderedDict((("WetDryMix", SimTypeFloat(size=32)), ("Feedback", SimTypeFloat(size=32)), ("Delay", SimTypeFloat(size=32)),)), name="FXECHO_PARAMETERS", pack=False, align=None),
    "FXEQ_PARAMETERS": SimStruct(OrderedDict((("FrequencyCenter0", SimTypeFloat(size=32)), ("Gain0", SimTypeFloat(size=32)), ("Bandwidth0", SimTypeFloat(size=32)), ("FrequencyCenter1", SimTypeFloat(size=32)), ("Gain1", SimTypeFloat(size=32)), ("Bandwidth1", SimTypeFloat(size=32)), ("FrequencyCenter2", SimTypeFloat(size=32)), ("Gain2", SimTypeFloat(size=32)), ("Bandwidth2", SimTypeFloat(size=32)), ("FrequencyCenter3", SimTypeFloat(size=32)), ("Gain3", SimTypeFloat(size=32)), ("Bandwidth3", SimTypeFloat(size=32)),)), name="FXEQ_PARAMETERS", pack=False, align=None),
    "FXMASTERINGLIMITER_PARAMETERS": SimStruct(OrderedDict((("Release", SimTypeInt(signed=False, label="UInt32")), ("Loudness", SimTypeInt(signed=False, label="UInt32")),)), name="FXMASTERINGLIMITER_PARAMETERS", pack=False, align=None),
    "FXREVERB_PARAMETERS": SimStruct(OrderedDict((("Diffusion", SimTypeFloat(size=32)), ("RoomSize", SimTypeFloat(size=32)),)), name="FXREVERB_PARAMETERS", pack=False, align=None),
    "FaceCharacterization": SimStruct(OrderedDict((("BlinkScoreLeft", SimTypeInt(signed=False, label="UInt32")), ("BlinkScoreRight", SimTypeInt(signed=False, label="UInt32")), ("FacialExpression", SimTypeInt(signed=False, label="UInt32")), ("FacialExpressionScore", SimTypeInt(signed=False, label="UInt32")),)), name="FaceCharacterization", pack=False, align=None),
    "FaceCharacterizationBlobHeader": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="FaceCharacterizationBlobHeader", pack=False, align=None),
    "FaceRectInfo": SimStruct(OrderedDict((("Region", SimTypeRef("RECT", SimStruct)), ("confidenceLevel", SimTypeInt(signed=True, label="Int32")),)), name="FaceRectInfo", pack=False, align=None),
    "FaceRectInfoBlobHeader": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="FaceRectInfoBlobHeader", pack=False, align=None),
    "FailureCategory": SimTypeInt(signed=False, label="FailureCategory"),
    "FailureCategoryMapping": SimStruct(OrderedDict((("mappingCompliance", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 5)),)), name="FailureCategoryMapping", pack=False, align=None),
    "FhBackupStopReason": SimTypeInt(signed=False, label="FhBackupStopReason"),
    "FileFormat": SimTypeInt(signed=False, label="FileFormat"),
    "FillMode": SimTypeInt(signed=False, label="FillMode"),
    "FillType": SimTypeInt(signed=False, label="FillType"),
    "FilterType": SimTypeInt(signed=False, label="FilterType"),
    "Filter_Spec_IPv4": SimStruct(OrderedDict((("filt_ipaddr", SimTypeRef("IN_ADDR", SimStruct)), ("filt_unused", SimTypeShort(signed=False, label="UInt16")), ("filt_port", SimTypeShort(signed=False, label="UInt16")),)), name="Filter_Spec_IPv4", pack=False, align=None),
    "Filter_Spec_IPv4GPI": SimStruct(OrderedDict((("filt_ipaddr", SimTypeRef("IN_ADDR", SimStruct)), ("filt_gpi", SimTypeInt(signed=False, label="UInt32")),)), name="Filter_Spec_IPv4GPI", pack=False, align=None),
    "FindSimilarFileIndexResults": SimStruct(OrderedDict((("m_FileIndex", SimTypeInt(signed=False, label="UInt32")), ("m_MatchCount", SimTypeInt(signed=False, label="UInt32")),)), name="FindSimilarFileIndexResults", pack=False, align=None),
    "FixupInfo": SimStruct(OrderedDict((("state", SimTypeInt(signed=False, label="FixupState")), ("percentage", SimTypeChar(label="Byte")), ("resultCodes", SimTypeRef("ResultCodes", SimStruct)), ("fixupMsgId", SimTypeInt(signed=False, label="UInt32")),)), name="FixupInfo", pack=False, align=None),
    "FixupState": SimTypeInt(signed=False, label="FixupState"),
    "FlagList": SimStruct(OrderedDict((("cFlags", SimTypeInt(signed=False, label="UInt32")), ("ulFlag", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="FlagList", pack=False, align=None),
    "FlowDirections": SimTypeInt(signed=False, label="FlowDirections"),
    "FlushIntention": SimTypeInt(signed=False, label="FlushIntention"),
    "Folder": SimTypeBottom(label="Folder"),
    "Folder2": SimTypeBottom(label="Folder2"),
    "Folder3": SimTypeBottom(label="Folder3"),
    "FolderActionSteps": SimTypeInt(signed=False, label="FolderActionSteps"),
    "FolderItem": SimTypeBottom(label="FolderItem"),
    "FolderItem2": SimTypeBottom(label="FolderItem2"),
    "FolderItemVerb": SimTypeBottom(label="FolderItemVerb"),
    "FolderItemVerbs": SimTypeBottom(label="FolderItemVerbs"),
    "FolderItems": SimTypeBottom(label="FolderItems"),
    "FolderItems2": SimTypeBottom(label="FolderItems2"),
    "FolderItems3": SimTypeBottom(label="FolderItems3"),
    "Font": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "FontCollection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "FontFamily": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "FontStyle": SimTypeInt(signed=False, label="FontStyle"),
    "FormatNotSupportedEvents": SimTypeInt(signed=False, label="FormatNotSupportedEvents"),
    "Frame": SimTypeBottom(label="Frame"),
    "FsiFileSystems": SimTypeInt(signed=False, label="FsiFileSystems"),
    "FsiItemType": SimTypeInt(signed=False, label="FsiItemType"),
    "FsrmAccountType": SimTypeInt(signed=False, label="FsrmAccountType"),
    "FsrmActionType": SimTypeInt(signed=False, label="FsrmActionType"),
    "FsrmClassificationLoggingFlags": SimTypeInt(signed=False, label="FsrmClassificationLoggingFlags"),
    "FsrmCollectionState": SimTypeInt(signed=False, label="FsrmCollectionState"),
    "FsrmCommitOptions": SimTypeInt(signed=False, label="FsrmCommitOptions"),
    "FsrmEnumOptions": SimTypeInt(signed=False, label="FsrmEnumOptions"),
    "FsrmEventType": SimTypeInt(signed=False, label="FsrmEventType"),
    "FsrmExecutionOption": SimTypeInt(signed=False, label="FsrmExecutionOption"),
    "FsrmFileConditionType": SimTypeInt(signed=False, label="FsrmFileConditionType"),
    "FsrmFileManagementLoggingFlags": SimTypeInt(signed=False, label="FsrmFileManagementLoggingFlags"),
    "FsrmFileManagementType": SimTypeInt(signed=False, label="FsrmFileManagementType"),
    "FsrmFileScreenFlags": SimTypeInt(signed=False, label="FsrmFileScreenFlags"),
    "FsrmFileStreamingInterfaceType": SimTypeInt(signed=False, label="FsrmFileStreamingInterfaceType"),
    "FsrmFileStreamingMode": SimTypeInt(signed=False, label="FsrmFileStreamingMode"),
    "FsrmFileSystemPropertyId": SimTypeInt(signed=False, label="FsrmFileSystemPropertyId"),
    "FsrmGetFilePropertyOptions": SimTypeInt(signed=False, label="FsrmGetFilePropertyOptions"),
    "FsrmPipelineModuleType": SimTypeInt(signed=False, label="FsrmPipelineModuleType"),
    "FsrmPropertyBagField": SimTypeInt(signed=False, label="FsrmPropertyBagField"),
    "FsrmPropertyBagFlags": SimTypeInt(signed=False, label="FsrmPropertyBagFlags"),
    "FsrmPropertyConditionType": SimTypeInt(signed=False, label="FsrmPropertyConditionType"),
    "FsrmPropertyDefinitionAppliesTo": SimTypeInt(signed=False, label="FsrmPropertyDefinitionAppliesTo"),
    "FsrmPropertyDefinitionFlags": SimTypeInt(signed=False, label="FsrmPropertyDefinitionFlags"),
    "FsrmPropertyDefinitionType": SimTypeInt(signed=False, label="FsrmPropertyDefinitionType"),
    "FsrmPropertyFlags": SimTypeInt(signed=False, label="FsrmPropertyFlags"),
    "FsrmPropertyValueType": SimTypeInt(signed=False, label="FsrmPropertyValueType"),
    "FsrmQuotaFlags": SimTypeInt(signed=False, label="FsrmQuotaFlags"),
    "FsrmReportFilter": SimTypeInt(signed=False, label="FsrmReportFilter"),
    "FsrmReportFormat": SimTypeInt(signed=False, label="FsrmReportFormat"),
    "FsrmReportGenerationContext": SimTypeInt(signed=False, label="FsrmReportGenerationContext"),
    "FsrmReportLimit": SimTypeInt(signed=False, label="FsrmReportLimit"),
    "FsrmReportRunningStatus": SimTypeInt(signed=False, label="FsrmReportRunningStatus"),
    "FsrmReportType": SimTypeInt(signed=False, label="FsrmReportType"),
    "FsrmRuleFlags": SimTypeInt(signed=False, label="FsrmRuleFlags"),
    "FsrmRuleType": SimTypeInt(signed=False, label="FsrmRuleType"),
    "FsrmStorageModuleCaps": SimTypeInt(signed=False, label="FsrmStorageModuleCaps"),
    "FsrmStorageModuleType": SimTypeInt(signed=False, label="FsrmStorageModuleType"),
    "FsrmTemplateApplyOptions": SimTypeInt(signed=False, label="FsrmTemplateApplyOptions"),
    "FunctionEnter": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["funcID"]), offset=0),
    "FunctionEnter2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("COR_PRF_FUNCTION_ARGUMENT_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["funcId", "clientData", "func", "argumentInfo"]), offset=0),
    "FunctionEnter3": SimTypePointer(SimTypeFunction([SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")], SimTypeBottom(label="Void"), arg_names=["functionIDOrClientID"]), offset=0),
    "FunctionEnter3WithInfo": SimTypePointer(SimTypeFunction([SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["functionIDOrClientID", "eltInfo"]), offset=0),
    "FunctionIDMapper": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["funcId", "pbHookFunction"]), offset=0),
    "FunctionIDMapper2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["funcId", "clientData", "pbHookFunction"]), offset=0),
    "FunctionIDOrClientID": SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None"),
    "FunctionLeave": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["funcID"]), offset=0),
    "FunctionLeave2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("COR_PRF_FUNCTION_ARGUMENT_RANGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["funcId", "clientData", "func", "retvalRange"]), offset=0),
    "FunctionLeave3": SimTypePointer(SimTypeFunction([SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")], SimTypeBottom(label="Void"), arg_names=["functionIDOrClientID"]), offset=0),
    "FunctionLeave3WithInfo": SimTypePointer(SimTypeFunction([SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["functionIDOrClientID", "eltInfo"]), offset=0),
    "FunctionTailcall": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["funcID"]), offset=0),
    "FunctionTailcall2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["funcId", "clientData", "func"]), offset=0),
    "FunctionTailcall3": SimTypePointer(SimTypeFunction([SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")], SimTypeBottom(label="Void"), arg_names=["functionIDOrClientID"]), offset=0),
    "FunctionTailcall3WithInfo": SimTypePointer(SimTypeFunction([SimUnion({"functionID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "clientID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["functionIDOrClientID", "eltInfo"]), offset=0),
    "G721_ADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("nAuxBlockSize", SimTypeShort(signed=False, label="UInt16")),)), name="G721_ADPCMWAVEFORMAT", pack=False, align=None),
    "G723_ADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("cbExtraSize", SimTypeShort(signed=False, label="UInt16")), ("nAuxBlockSize", SimTypeShort(signed=False, label="UInt16")),)), name="G723_ADPCMWAVEFORMAT", pack=False, align=None),
    "GAMESTATS_OPEN_RESULT": SimTypeInt(signed=False, label="GAMESTATS_OPEN_RESULT"),
    "GAMESTATS_OPEN_TYPE": SimTypeInt(signed=False, label="GAMESTATS_OPEN_TYPE"),
    "GAME_INSTALL_SCOPE": SimTypeInt(signed=False, label="GAME_INSTALL_SCOPE"),
    "GAMING_DEVICE_DEVICE_ID": SimTypeInt(signed=False, label="GAMING_DEVICE_DEVICE_ID"),
    "GAMING_DEVICE_MODEL_INFORMATION": SimStruct(OrderedDict((("vendorId", SimTypeInt(signed=False, label="GAMING_DEVICE_VENDOR_ID")), ("deviceId", SimTypeInt(signed=False, label="GAMING_DEVICE_DEVICE_ID")),)), name="GAMING_DEVICE_MODEL_INFORMATION", pack=False, align=None),
    "GAMING_DEVICE_VENDOR_ID": SimTypeInt(signed=False, label="GAMING_DEVICE_VENDOR_ID"),
    "GAMMARAMP": SimStruct(OrderedDict((("Red", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Green", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Blue", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="GAMMARAMP", pack=False, align=None),
    "GAMMA_RAMP_DXGI_1": SimStruct(OrderedDict((("Scale", SimTypeRef("GAMMA_RAMP_RGB", SimStruct)), ("Offset", SimTypeRef("GAMMA_RAMP_RGB", SimStruct)), ("GammaCurve", SimTypeArray(SimTypeRef("GAMMA_RAMP_RGB", SimStruct), 1025)),)), name="GAMMA_RAMP_DXGI_1", pack=False, align=None),
    "GAMMA_RAMP_RGB": SimStruct(OrderedDict((("Red", SimTypeFloat(size=32)), ("Green", SimTypeFloat(size=32)), ("Blue", SimTypeFloat(size=32)),)), name="GAMMA_RAMP_RGB", pack=False, align=None),
    "GAMMA_RAMP_RGB256x3x16": SimStruct(OrderedDict((("Red", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Green", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Blue", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="GAMMA_RAMP_RGB256x3x16", pack=False, align=None),
    "GCP_RESULTSA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpOrder", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpDx", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lpCaretPos", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lpClass", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpGlyphs", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nGlyphs", SimTypeInt(signed=False, label="UInt32")), ("nMaxFit", SimTypeInt(signed=True, label="Int32")),)), name="GCP_RESULTSA", pack=False, align=None),
    "GCP_RESULTSW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpOrder", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpDx", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lpCaretPos", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("lpClass", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpGlyphs", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nGlyphs", SimTypeInt(signed=False, label="UInt32")), ("nMaxFit", SimTypeInt(signed=True, label="Int32")),)), name="GCP_RESULTSW", pack=False, align=None),
    "GDIINFO": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("ulTechnology", SimTypeInt(signed=False, label="UInt32")), ("ulHorzSize", SimTypeInt(signed=False, label="UInt32")), ("ulVertSize", SimTypeInt(signed=False, label="UInt32")), ("ulHorzRes", SimTypeInt(signed=False, label="UInt32")), ("ulVertRes", SimTypeInt(signed=False, label="UInt32")), ("cBitsPixel", SimTypeInt(signed=False, label="UInt32")), ("cPlanes", SimTypeInt(signed=False, label="UInt32")), ("ulNumColors", SimTypeInt(signed=False, label="UInt32")), ("flRaster", SimTypeInt(signed=False, label="UInt32")), ("ulLogPixelsX", SimTypeInt(signed=False, label="UInt32")), ("ulLogPixelsY", SimTypeInt(signed=False, label="UInt32")), ("flTextCaps", SimTypeInt(signed=False, label="UInt32")), ("ulDACRed", SimTypeInt(signed=False, label="UInt32")), ("ulDACGreen", SimTypeInt(signed=False, label="UInt32")), ("ulDACBlue", SimTypeInt(signed=False, label="UInt32")), ("ulAspectX", SimTypeInt(signed=False, label="UInt32")), ("ulAspectY", SimTypeInt(signed=False, label="UInt32")), ("ulAspectXY", SimTypeInt(signed=False, label="UInt32")), ("xStyleStep", SimTypeInt(signed=True, label="Int32")), ("yStyleStep", SimTypeInt(signed=True, label="Int32")), ("denStyleStep", SimTypeInt(signed=True, label="Int32")), ("ptlPhysOffset", SimTypeRef("POINTL", SimStruct)), ("szlPhysSize", SimTypeRef("SIZE", SimStruct)), ("ulNumPalReg", SimTypeInt(signed=False, label="UInt32")), ("ciDevice", SimTypeRef("COLORINFO", SimStruct)), ("ulDevicePelsDPI", SimTypeInt(signed=False, label="UInt32")), ("ulPrimaryOrder", SimTypeInt(signed=False, label="UInt32")), ("ulHTPatternSize", SimTypeInt(signed=False, label="UInt32")), ("ulHTOutputFormat", SimTypeInt(signed=False, label="UInt32")), ("flHTFlags", SimTypeInt(signed=False, label="UInt32")), ("ulVRefresh", SimTypeInt(signed=False, label="UInt32")), ("ulBltAlignment", SimTypeInt(signed=False, label="UInt32")), ("ulPanningHorzRes", SimTypeInt(signed=False, label="UInt32")), ("ulPanningVertRes", SimTypeInt(signed=False, label="UInt32")), ("xPanningAlignment", SimTypeInt(signed=False, label="UInt32")), ("yPanningAlignment", SimTypeInt(signed=False, label="UInt32")), ("cxHTPat", SimTypeInt(signed=False, label="UInt32")), ("cyHTPat", SimTypeInt(signed=False, label="UInt32")), ("pHTPatA", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHTPatB", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHTPatC", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("flShadeBlend", SimTypeInt(signed=False, label="UInt32")), ("ulPhysicalPixelCharacteristics", SimTypeInt(signed=False, label="UInt32")), ("ulPhysicalPixelGamma", SimTypeInt(signed=False, label="UInt32")),)), name="GDIINFO", pack=False, align=None),
    "GDI_IMAGE_TYPE": SimTypeInt(signed=False, label="GDI_IMAGE_TYPE"),
    "GDI_NONREMOTE": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("DWORD_BLOB", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="GDI_NONREMOTE", pack=False, align=None),
    "GDI_OBJECT": SimStruct(OrderedDict((("ObjectType", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"hBitmap": SimTypePointer(SimTypeRef("userHBITMAP", SimStruct), offset=0), "hPalette": SimTypePointer(SimTypeRef("userHPALETTE", SimStruct), offset=0), "hGeneric": SimTypePointer(SimTypeRef("userHGLOBAL", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="GDI_OBJECT", pack=False, align=None),
    "GDI_REGION_TYPE": SimTypeInt(signed=False, label="GDI_REGION_TYPE"),
    "GENERATE_NAME_CONTEXT": SimStruct(OrderedDict((("Checksum", SimTypeShort(signed=False, label="UInt16")), ("ChecksumInserted", SimTypeChar(label="Byte")), ("NameLength", SimTypeChar(label="Byte")), ("NameBuffer", SimTypeArray(SimTypeChar(label="Char"), 8)), ("ExtensionLength", SimTypeInt(signed=False, label="UInt32")), ("ExtensionBuffer", SimTypeArray(SimTypeChar(label="Char"), 4)), ("LastIndexValue", SimTypeInt(signed=False, label="UInt32")),)), name="GENERATE_NAME_CONTEXT", pack=False, align=None),
    "GENERIC3CHANNEL": SimStruct(OrderedDict((("ch1", SimTypeShort(signed=False, label="UInt16")), ("ch2", SimTypeShort(signed=False, label="UInt16")), ("ch3", SimTypeShort(signed=False, label="UInt16")),)), name="GENERIC3CHANNEL", pack=False, align=None),
    "GENERIC_ACCESS_RIGHTS": SimTypeInt(signed=False, label="GENERIC_ACCESS_RIGHTS"),
    "GENERIC_BINDING_INFO": SimStruct(OrderedDict((("pObj", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")), ("pfnBind", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0)), ("pfnUnbind", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)),)), name="GENERIC_BINDING_INFO", pack=False, align=None),
    "GENERIC_BINDING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0),
    "GENERIC_BINDING_ROUTINE_PAIR": SimStruct(OrderedDict((("pfnBind", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0)), ("pfnUnbind", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)),)), name="GENERIC_BINDING_ROUTINE_PAIR", pack=False, align=None),
    "GENERIC_MAPPING": SimStruct(OrderedDict((("GenericRead", SimTypeInt(signed=False, label="UInt32")), ("GenericWrite", SimTypeInt(signed=False, label="UInt32")), ("GenericExecute", SimTypeInt(signed=False, label="UInt32")), ("GenericAll", SimTypeInt(signed=False, label="UInt32")),)), name="GENERIC_MAPPING", pack=False, align=None),
    "GENERIC_UNBIND_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "GENERIC_XML_TOKEN": SimStruct(OrderedDict((("createDate", SimTypeRef("FILETIME", SimStruct)), ("expiryDate", SimTypeRef("FILETIME", SimStruct)), ("xmlToken", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("internalTokenReference", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("externalTokenReference", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GENERIC_XML_TOKEN", pack=False, align=None),
    "GEN_GET_NETCARD_TIME": SimStruct(OrderedDict((("ReadTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="GEN_GET_NETCARD_TIME", pack=False, align=None),
    "GEN_GET_TIME_CAPS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ClockPrecision", SimTypeInt(signed=False, label="UInt32")),)), name="GEN_GET_TIME_CAPS", pack=False, align=None),
    "GEO_ENUMNAMEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "GEO_ENUMPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "GESTURECONFIG": SimStruct(OrderedDict((("dwID", SimTypeInt(signed=False, label="GESTURECONFIG_ID")), ("dwWant", SimTypeInt(signed=False, label="UInt32")), ("dwBlock", SimTypeInt(signed=False, label="UInt32")),)), name="GESTURECONFIG", pack=False, align=None),
    "GESTURECONFIG_FLAGS": SimTypeInt(signed=False, label="GESTURECONFIG_FLAGS"),
    "GESTURECONFIG_ID": SimTypeInt(signed=False, label="GESTURECONFIG_ID"),
    "GESTUREINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwID", SimTypeInt(signed=False, label="UInt32")), ("hwndTarget", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ptsLocation", SimTypeRef("POINTS", SimStruct)), ("dwInstanceID", SimTypeInt(signed=False, label="UInt32")), ("dwSequenceID", SimTypeInt(signed=False, label="UInt32")), ("ullArguments", SimTypeLongLong(signed=False, label="UInt64")), ("cbExtraArgs", SimTypeInt(signed=False, label="UInt32")),)), name="GESTUREINFO", pack=False, align=None),
    "GESTURENOTIFYSTRUCT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndTarget", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ptsLocation", SimTypeRef("POINTS", SimStruct)), ("dwInstanceID", SimTypeInt(signed=False, label="UInt32")),)), name="GESTURENOTIFYSTRUCT", pack=False, align=None),
    "GESTURE_DATA": SimStruct(OrderedDict((("gestureId", SimTypeInt(signed=True, label="Int32")), ("recoConfidence", SimTypeInt(signed=True, label="Int32")), ("strokeCount", SimTypeInt(signed=True, label="Int32")),)), name="GESTURE_DATA", pack=False, align=None),
    "GESTURE_TYPE": SimTypeInt(signed=False, label="GESTURE_TYPE"),
    "GETCLIPBMETADATA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("IsDelayRendered", SimTypeInt(signed=True, label="Int32")), ("IsSynthetic", SimTypeInt(signed=True, label="Int32")),)), name="GETCLIPBMETADATA", pack=False, align=None),
    "GETCONTEXTMENUEX": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="GETCONTEXTMENUEX", pack=False, align=None),
    "GETFINALPATHNAMEBYHANDLE_FLAGS": SimTypeInt(signed=False, label="GETFINALPATHNAMEBYHANDLE_FLAGS"),
    "GETPROPERTYSTOREFLAGS": SimTypeInt(signed=False, label="GETPROPERTYSTOREFLAGS"),
    "GETTEXTEX": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="GETTEXTEX_FLAGS")), ("codepage", SimTypeInt(signed=False, label="UInt32")), ("lpDefaultChar", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpUsedDefChar", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="GETTEXTEX", pack=False, align=None),
    "GETTEXTEX_FLAGS": SimTypeInt(signed=False, label="GETTEXTEX_FLAGS"),
    "GETTEXTLENGTHEX": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="GETTEXTLENGTHEX_FLAGS")), ("codepage", SimTypeInt(signed=False, label="UInt32")),)), name="GETTEXTLENGTHEX", pack=False, align=None),
    "GETTEXTLENGTHEX_FLAGS": SimTypeInt(signed=False, label="GETTEXTLENGTHEX_FLAGS"),
    "GETVERSIONINPARAMS": SimStruct(OrderedDict((("bVersion", SimTypeChar(label="Byte")), ("bRevision", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("bIDEDeviceMap", SimTypeChar(label="Byte")), ("fCapabilities", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="GETVERSIONINPARAMS", pack=False, align=None),
    "GET_ADAPTERS_ADDRESSES_FLAGS": SimTypeInt(signed=False, label="GET_ADAPTERS_ADDRESSES_FLAGS"),
    "GET_ANCESTOR_FLAGS": SimTypeInt(signed=False, label="GET_ANCESTOR_FLAGS"),
    "GET_CHANGER_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("NumberTransportElements", SimTypeShort(signed=False, label="UInt16")), ("NumberStorageElements", SimTypeShort(signed=False, label="UInt16")), ("NumberCleanerSlots", SimTypeShort(signed=False, label="UInt16")), ("NumberIEElements", SimTypeShort(signed=False, label="UInt16")), ("NumberDataTransferElements", SimTypeShort(signed=False, label="UInt16")), ("NumberOfDoors", SimTypeShort(signed=False, label="UInt16")), ("FirstSlotNumber", SimTypeShort(signed=False, label="UInt16")), ("FirstDriveNumber", SimTypeShort(signed=False, label="UInt16")), ("FirstTransportNumber", SimTypeShort(signed=False, label="UInt16")), ("FirstIEPortNumber", SimTypeShort(signed=False, label="UInt16")), ("FirstCleanerSlotAddress", SimTypeShort(signed=False, label="UInt16")), ("MagazineSize", SimTypeShort(signed=False, label="UInt16")), ("DriveCleanTimeout", SimTypeInt(signed=False, label="UInt32")), ("Features0", SimTypeInt(signed=False, label="CHANGER_FEATURES")), ("Features1", SimTypeInt(signed=False, label="GET_CHANGER_PARAMETERS_FEATURES1")), ("MoveFromTransport", SimTypeChar(label="Byte")), ("MoveFromSlot", SimTypeChar(label="Byte")), ("MoveFromIePort", SimTypeChar(label="Byte")), ("MoveFromDrive", SimTypeChar(label="Byte")), ("ExchangeFromTransport", SimTypeChar(label="Byte")), ("ExchangeFromSlot", SimTypeChar(label="Byte")), ("ExchangeFromIePort", SimTypeChar(label="Byte")), ("ExchangeFromDrive", SimTypeChar(label="Byte")), ("LockUnlockCapabilities", SimTypeChar(label="Byte")), ("PositionCapabilities", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="GET_CHANGER_PARAMETERS", pack=False, align=None),
    "GET_CHANGER_PARAMETERS_FEATURES1": SimTypeInt(signed=False, label="GET_CHANGER_PARAMETERS_FEATURES1"),
    "GET_CHARACTER_PLACEMENT_FLAGS": SimTypeInt(signed=False, label="GET_CHARACTER_PLACEMENT_FLAGS"),
    "GET_CLASS_LONG_INDEX": SimTypeInt(signed=False, label="GET_CLASS_LONG_INDEX"),
    "GET_CONTEXT_EX": SimStruct(OrderedDict((("Status", SimTypeInt(signed=False, label="UInt32")), ("ContextSize", SimTypeInt(signed=False, label="UInt32")), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="GET_CONTEXT_EX", pack=False, align=None),
    "GET_CONVERSION_LIST_FLAG": SimTypeInt(signed=False, label="GET_CONVERSION_LIST_FLAG"),
    "GET_CURRENT_PROCESS_ADDRESS": SimStruct(OrderedDict((("Processor", SimTypeInt(signed=False, label="UInt32")), ("CurrentThread", SimTypeLongLong(signed=False, label="UInt64")), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="GET_CURRENT_PROCESS_ADDRESS", pack=False, align=None),
    "GET_CURRENT_THREAD_ADDRESS": SimStruct(OrderedDict((("Processor", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="GET_CURRENT_THREAD_ADDRESS", pack=False, align=None),
    "GET_D3COLD_CAPABILITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "D3ColdSupported"]), offset=0),
    "GET_D3COLD_LAST_TRANSITION_STATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3COLD_LAST_TRANSITION_STATUS"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "LastTransitionStatus"]), offset=0),
    "GET_DANDIDATE_FLAGS": SimTypeInt(signed=False, label="GET_DANDIDATE_FLAGS"),
    "GET_DCX_FLAGS": SimTypeInt(signed=False, label="GET_DCX_FLAGS"),
    "GET_DEVICE_CAPS_INDEX": SimTypeInt(signed=False, label="GET_DEVICE_CAPS_INDEX"),
    "GET_DEVICE_INTERNAL_STATUS_DATA_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("RequestDataType", SimTypeInt(signed=False, label="DEVICE_INTERNAL_STATUS_DATA_REQUEST_TYPE")), ("RequestDataSet", SimTypeInt(signed=False, label="DEVICE_INTERNAL_STATUS_DATA_SET")),)), name="GET_DEVICE_INTERNAL_STATUS_DATA_REQUEST", pack=False, align=None),
    "GET_DEVICE_RESET_STATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimUnion({"Pci": SimTypeRef("_Pci_e__Struct", SimStruct), "Acpi": SimTypeRef("_Acpi_e__Struct", SimStruct), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), offset=0), SimTypePointer(SimUnion({"u": SimTypeRef("_u_e__Struct", SimStruct), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InterfaceContext", "IsResetting", "ResetTypeSelected", "Flags"]), offset=0),
    "GET_DISK_ATTRIBUTES": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeLongLong(signed=False, label="UInt64")),)), name="GET_DISK_ATTRIBUTES", pack=False, align=None),
    "GET_DMA_ADAPTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEVICE_DESCRIPTION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), arg_names=["Context", "DeviceDescriptor", "NumberOfMapRegisters"]), offset=0),
    "GET_EXPRESSION_EX": SimStruct(OrderedDict((("Expression", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Remainder", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="GET_EXPRESSION_EX", pack=False, align=None),
    "GET_FILEEX_INFO_LEVELS": SimTypeInt(signed=False, label="GET_FILEEX_INFO_LEVELS"),
    "GET_FILE_VERSION_INFO_FLAGS": SimTypeInt(signed=False, label="GET_FILE_VERSION_INFO_FLAGS"),
    "GET_FILTER_FILE_IDENTIFIER_INPUT": SimStruct(OrderedDict((("AltitudeLength", SimTypeShort(signed=False, label="UInt16")), ("Altitude", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GET_FILTER_FILE_IDENTIFIER_INPUT", pack=False, align=None),
    "GET_FILTER_FILE_IDENTIFIER_OUTPUT": SimStruct(OrderedDict((("FilterFileIdentifierLength", SimTypeShort(signed=False, label="UInt16")), ("FilterFileIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GET_FILTER_FILE_IDENTIFIER_OUTPUT", pack=False, align=None),
    "GET_GLYPH_OUTLINE_FORMAT": SimTypeInt(signed=False, label="GET_GLYPH_OUTLINE_FORMAT"),
    "GET_GUIDE_LINE_TYPE": SimTypeInt(signed=False, label="GET_GUIDE_LINE_TYPE"),
    "GET_GUI_RESOURCES_FLAGS": SimTypeInt(signed=False, label="GET_GUI_RESOURCES_FLAGS"),
    "GET_IDLE_WAKE_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), SimTypePointer(SimTypeInt(signed=False, label="DEVICE_WAKE_DEPTH"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "SystemPowerState", "DeepestWakeableDstate"]), offset=0),
    "GET_INPUT_LINE": SimStruct(OrderedDict((("Prompt", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("InputSize", SimTypeInt(signed=False, label="UInt32")),)), name="GET_INPUT_LINE", pack=False, align=None),
    "GET_LENGTH_INFORMATION": SimStruct(OrderedDict((("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="GET_LENGTH_INFORMATION", pack=False, align=None),
    "GET_MEDIA_TYPES": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("MediaInfoCount", SimTypeInt(signed=False, label="UInt32")), ("MediaInfo", SimTypePointer(SimTypeRef("DEVICE_MEDIA_INFO", SimStruct), offset=0)),)), name="GET_MEDIA_TYPES", pack=False, align=None),
    "GET_MENU_DEFAULT_ITEM_FLAGS": SimTypeInt(signed=False, label="GET_MENU_DEFAULT_ITEM_FLAGS"),
    "GET_MOUSE_MOVE_POINTS_EX_RESOLUTION": SimTypeInt(signed=False, label="GET_MOUSE_MOVE_POINTS_EX_RESOLUTION"),
    "GET_OPERATION_CONTEXT_PARAMS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="RESDLL_CONTEXT_OPERATION_TYPE")), ("Priority", SimTypeInt(signed=False, label="UInt32")),)), name="GET_OPERATION_CONTEXT_PARAMS", pack=False, align=None),
    "GET_PEB_ADDRESS": SimStruct(OrderedDict((("CurrentThread", SimTypeLongLong(signed=False, label="UInt64")), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="GET_PEB_ADDRESS", pack=False, align=None),
    "GET_SDEV_IDENTIFIER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeLongLong(signed=False, label="UInt64"), arg_names=["InterfaceContext"]), offset=0),
    "GET_SET_DEVICE_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "DataType", "Buffer", "Offset", "Length"]), offset=0),
    "GET_SET_SYMPATH": SimStruct(OrderedDict((("Args", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Result", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Length", SimTypeInt(signed=True, label="Int32")),)), name="GET_SET_SYMPATH", pack=False, align=None),
    "GET_STOCK_OBJECT_FLAGS": SimTypeInt(signed=False, label="GET_STOCK_OBJECT_FLAGS"),
    "GET_STORAGE_DEPENDENCY_FLAG": SimTypeInt(signed=False, label="GET_STORAGE_DEPENDENCY_FLAG"),
    "GET_TAPE_DRIVE_PARAMETERS_OPERATION": SimTypeInt(signed=False, label="GET_TAPE_DRIVE_PARAMETERS_OPERATION"),
    "GET_TEB_ADDRESS": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="GET_TEB_ADDRESS", pack=False, align=None),
    "GET_TEXT_AND_PROPERTY_UPDATES_FLAGS": SimTypeInt(signed=False, label="GET_TEXT_AND_PROPERTY_UPDATES_FLAGS"),
    "GET_THEME_BITMAP_FLAGS": SimTypeInt(signed=False, label="GET_THEME_BITMAP_FLAGS"),
    "GET_UPDATED_BUS_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CM_RESOURCE_LIST", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CM_RESOURCE_LIST", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "UpdatedResourceList", "UpdatedTranslatedResourceList"]), offset=0),
    "GET_VIRTUAL_DEVICE_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "VirtualFunction", "Buffer", "Offset", "Length"]), offset=0),
    "GET_VIRTUAL_DEVICE_LOCATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "VirtualFunction", "SegmentNumber", "BusNumber", "FunctionNumber"]), offset=0),
    "GET_VIRTUAL_DEVICE_RESOURCES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "CapturedBusNumbers"]), offset=0),
    "GET_VIRTUAL_DISK_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="GET_VIRTUAL_DISK_INFO_VERSION")), ("Anonymous", SimUnion({"Size": SimTypeRef("_Size_e__Struct", SimStruct), "Identifier": SimTypeBottom(label="Guid"), "ParentLocation": SimTypeRef("_ParentLocation_e__Struct", SimStruct), "ParentIdentifier": SimTypeBottom(label="Guid"), "ParentTimestamp": SimTypeInt(signed=False, label="UInt32"), "VirtualStorageType": SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct), "ProviderSubtype": SimTypeInt(signed=False, label="UInt32"), "Is4kAligned": SimTypeInt(signed=True, label="Int32"), "IsLoaded": SimTypeInt(signed=True, label="Int32"), "PhysicalDisk": SimTypeRef("_PhysicalDisk_e__Struct", SimStruct), "VhdPhysicalSectorSize": SimTypeInt(signed=False, label="UInt32"), "SmallestSafeVirtualSize": SimTypeLongLong(signed=False, label="UInt64"), "FragmentationPercentage": SimTypeInt(signed=False, label="UInt32"), "VirtualDiskId": SimTypeBottom(label="Guid"), "ChangeTrackingState": SimTypeRef("_ChangeTrackingState_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="GET_VIRTUAL_DISK_INFO", pack=False, align=None),
    "GET_VIRTUAL_DISK_INFO_VERSION": SimTypeInt(signed=False, label="GET_VIRTUAL_DISK_INFO_VERSION"),
    "GET_VIRTUAL_FUNCTION_PROBED_BARS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "BaseRegisterValues"]), offset=0),
    "GET_WINDOW_CMD": SimTypeInt(signed=False, label="GET_WINDOW_CMD"),
    "GHOSTED_FILE_EXTENT": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteCount", SimTypeLongLong(signed=True, label="Int64")), ("RecallOwnerGuid", SimTypeBottom(label="Guid")), ("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("RecallMetadataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("RecallMetadataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GHOSTED_FILE_EXTENT", pack=False, align=None),
    "GLOBALENTRY": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwAddress", SimTypeInt(signed=False, label="UInt32")), ("dwBlockSize", SimTypeInt(signed=False, label="UInt32")), ("hBlock", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wcLock", SimTypeShort(signed=False, label="UInt16")), ("wcPageLock", SimTypeShort(signed=False, label="UInt16")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("wHeapPresent", SimTypeInt(signed=True, label="Int32")), ("hOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("wData", SimTypeShort(signed=False, label="UInt16")), ("dwNext", SimTypeInt(signed=False, label="UInt32")), ("dwNextAlt", SimTypeInt(signed=False, label="UInt32")),)), name="GLOBALENTRY", pack=False, align=None),
    "GLOBALOPT_EH_VALUES": SimTypeInt(signed=False, label="GLOBALOPT_EH_VALUES"),
    "GLOBALOPT_PROPERTIES": SimTypeInt(signed=False, label="GLOBALOPT_PROPERTIES"),
    "GLOBALOPT_RO_FLAGS": SimTypeInt(signed=False, label="GLOBALOPT_RO_FLAGS"),
    "GLOBALOPT_RPCTP_VALUES": SimTypeInt(signed=False, label="GLOBALOPT_RPCTP_VALUES"),
    "GLOBALOPT_UNMARSHALING_POLICY_VALUES": SimTypeInt(signed=False, label="GLOBALOPT_UNMARSHALING_POLICY_VALUES"),
    "GLOBAL_ALLOC_FLAGS": SimTypeInt(signed=False, label="GLOBAL_ALLOC_FLAGS"),
    "GLOBAL_FILTER": SimTypeInt(signed=False, label="GLOBAL_FILTER"),
    "GLOBAL_MACHINE_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("LidOpenWakeAc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("LidOpenWakeDc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("BroadcastCapacityResolution", SimTypeInt(signed=False, label="UInt32")),)), name="GLOBAL_MACHINE_POWER_POLICY", pack=False, align=None),
    "GLOBAL_POWER_POLICY": SimStruct(OrderedDict((("user", SimTypeRef("GLOBAL_USER_POWER_POLICY", SimStruct)), ("mach", SimTypeRef("GLOBAL_MACHINE_POWER_POLICY", SimStruct)),)), name="GLOBAL_POWER_POLICY", pack=False, align=None),
    "GLOBAL_USER_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("PowerButtonAc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("PowerButtonDc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("SleepButtonAc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("SleepButtonDc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("LidCloseAc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("LidCloseDc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("DischargePolicy", SimTypeArray(SimTypeRef("SYSTEM_POWER_LEVEL", SimStruct), 4)), ("GlobalFlags", SimTypeInt(signed=False, label="UInt32")),)), name="GLOBAL_USER_POWER_POLICY", pack=False, align=None),
    "GLUnurbs": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GLUnurbsErrorProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUquadric": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GLUquadricErrorProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUtessBeginDataProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "GLUtessBeginProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUtessCombineDataProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFloat(size=64), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeFloat(size=32), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "GLUtessCombineProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFloat(size=64), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeFloat(size=32), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "GLUtessEdgeFlagDataProc": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "GLUtessEdgeFlagProc": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUtessEndDataProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUtessEndProc": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "GLUtessErrorDataProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "GLUtessErrorProc": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUtessVertexDataProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "GLUtessVertexProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "GLUtesselator": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GLYPHBITS": SimStruct(OrderedDict((("ptlOrigin", SimTypeRef("POINTL", SimStruct)), ("sizlBitmap", SimTypeRef("SIZE", SimStruct)), ("aj", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GLYPHBITS", pack=False, align=None),
    "GLYPHDATA": SimStruct(OrderedDict((("gdf", SimUnion({"pgb": SimTypePointer(SimTypeRef("GLYPHBITS", SimStruct), offset=0), "ppo": SimTypePointer(SimTypeRef("PATHOBJ", SimStruct), offset=0)}, name="<anon>", label="None")), ("hg", SimTypeInt(signed=False, label="UInt32")), ("fxD", SimTypeInt(signed=True, label="Int32")), ("fxA", SimTypeInt(signed=True, label="Int32")), ("fxAB", SimTypeInt(signed=True, label="Int32")), ("fxInkTop", SimTypeInt(signed=True, label="Int32")), ("fxInkBottom", SimTypeInt(signed=True, label="Int32")), ("rclInk", SimTypeRef("RECTL", SimStruct)), ("ptqD", SimTypeRef("POINTQF", SimStruct)),)), name="GLYPHDATA", pack=False, align=None),
    "GLYPHDEF": SimUnion({"pgb": SimTypePointer(SimTypeRef("GLYPHBITS", SimStruct), offset=0), "ppo": SimTypePointer(SimTypeRef("PATHOBJ", SimStruct), offset=0)}, name="<anon>", label="None"),
    "GLYPHFONTSIZINGTYPE": SimTypeInt(signed=False, label="GLYPHFONTSIZINGTYPE"),
    "GLYPHMETRICS": SimStruct(OrderedDict((("gmBlackBoxX", SimTypeInt(signed=False, label="UInt32")), ("gmBlackBoxY", SimTypeInt(signed=False, label="UInt32")), ("gmptGlyphOrigin", SimTypeRef("POINT", SimStruct)), ("gmCellIncX", SimTypeShort(signed=True, label="Int16")), ("gmCellIncY", SimTypeShort(signed=True, label="Int16")),)), name="GLYPHMETRICS", pack=False, align=None),
    "GLYPHMETRICSFLOAT": SimStruct(OrderedDict((("gmfBlackBoxX", SimTypeFloat(size=32)), ("gmfBlackBoxY", SimTypeFloat(size=32)), ("gmfptGlyphOrigin", SimTypeRef("POINTFLOAT", SimStruct)), ("gmfCellIncX", SimTypeFloat(size=32)), ("gmfCellIncY", SimTypeFloat(size=32)),)), name="GLYPHMETRICSFLOAT", pack=False, align=None),
    "GLYPHPOS": SimStruct(OrderedDict((("hg", SimTypeInt(signed=False, label="UInt32")), ("pgdf", SimTypePointer(SimUnion({"pgb": SimTypePointer(SimTypeRef("GLYPHBITS", SimStruct), offset=0), "ppo": SimTypePointer(SimTypeRef("PATHOBJ", SimStruct), offset=0)}, name="<anon>", label="None"), offset=0)), ("ptl", SimTypeRef("POINTL", SimStruct)),)), name="GLYPHPOS", pack=False, align=None),
    "GLYPHRUN": SimStruct(OrderedDict((("wcLow", SimTypeChar(label="Char")), ("wGlyphCount", SimTypeShort(signed=False, label="UInt16")),)), name="GLYPHRUN", pack=False, align=None),
    "GLYPHSET": SimStruct(OrderedDict((("cbThis", SimTypeInt(signed=False, label="UInt32")), ("flAccel", SimTypeInt(signed=False, label="UInt32")), ("cGlyphsSupported", SimTypeInt(signed=False, label="UInt32")), ("cRanges", SimTypeInt(signed=False, label="UInt32")), ("ranges", SimTypePointer(SimTypeRef("WCRANGE", SimStruct), offset=0)),)), name="GLYPHSET", pack=False, align=None),
    "GLYPHSTATES": SimTypeInt(signed=False, label="GLYPHSTATES"),
    "GLYPHTYPE": SimTypeInt(signed=False, label="GLYPHTYPE"),
    "GNSS_AGNSS_INJECT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("InjectionType", SimTypeInt(signed=False, label="GNSS_AGNSS_REQUEST_TYPE")), ("InjectionStatus", SimTypeInt(signed=True, label="Int32")), ("InjectionDataSize", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("Anonymous", SimUnion({"Time": SimTypeRef("GNSS_AGNSS_INJECTTIME", SimStruct), "Position": SimTypeRef("GNSS_AGNSS_INJECTPOSITION", SimStruct), "BlobData": SimTypeRef("GNSS_AGNSS_INJECTBLOB", SimStruct)}, name="<anon>", label="None")),)), name="GNSS_AGNSS_INJECT", pack=False, align=None),
    "GNSS_AGNSS_INJECTBLOB": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("BlobOui", SimTypeInt(signed=False, label="UInt32")), ("BlobVersion", SimTypeInt(signed=False, label="UInt32")), ("AgnssFormat", SimTypeInt(signed=False, label="UInt32")), ("BlobSize", SimTypeInt(signed=False, label="UInt32")), ("BlobData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GNSS_AGNSS_INJECTBLOB", pack=False, align=None),
    "GNSS_AGNSS_INJECTPOSITION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Age", SimTypeInt(signed=False, label="UInt32")), ("BasicData", SimTypeRef("GNSS_FIXDATA_BASIC", SimStruct)), ("AccuracyData", SimTypeRef("GNSS_FIXDATA_ACCURACY", SimStruct)),)), name="GNSS_AGNSS_INJECTPOSITION", pack=False, align=None),
    "GNSS_AGNSS_INJECTTIME": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("UtcTime", SimTypeRef("FILETIME", SimStruct)), ("TimeUncertainty", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_AGNSS_INJECTTIME", pack=False, align=None),
    "GNSS_AGNSS_REQUEST_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestType", SimTypeInt(signed=False, label="GNSS_AGNSS_REQUEST_TYPE")), ("BlobFormat", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_AGNSS_REQUEST_PARAM", pack=False, align=None),
    "GNSS_AGNSS_REQUEST_TYPE": SimTypeInt(signed=False, label="GNSS_AGNSS_REQUEST_TYPE"),
    "GNSS_BREADCRUMBING_ALERT_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_BREADCRUMBING_ALERT_DATA", pack=False, align=None),
    "GNSS_BREADCRUMBING_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("MaximumHorizontalUncertainty", SimTypeInt(signed=False, label="UInt32")), ("MinDistanceBetweenFixes", SimTypeInt(signed=False, label="UInt32")), ("MaximumErrorTimeoutMs", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_BREADCRUMBING_PARAM", pack=False, align=None),
    "GNSS_BREADCRUMB_LIST": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("NumCrumbs", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"v1": SimTypeArray(SimTypeRef("GNSS_BREADCRUMB_V1", SimStruct), 50)}, name="<anon>", label="None")),)), name="GNSS_BREADCRUMB_LIST", pack=False, align=None),
    "GNSS_BREADCRUMB_V1": SimStruct(OrderedDict((("FixTimeStamp", SimTypeRef("FILETIME", SimStruct)), ("Latitude", SimTypeFloat(size=64)), ("Longitude", SimTypeFloat(size=64)), ("HorizontalAccuracy", SimTypeInt(signed=False, label="UInt32")), ("Speed", SimTypeShort(signed=False, label="UInt16")), ("SpeedAccuracy", SimTypeShort(signed=False, label="UInt16")), ("Altitude", SimTypeShort(signed=True, label="Int16")), ("AltitudeAccuracy", SimTypeShort(signed=False, label="UInt16")), ("Heading", SimTypeShort(signed=True, label="Int16")), ("HeadingAccuracy", SimTypeChar(label="Byte")), ("FixSuccess", SimTypeChar(label="Byte")),)), name="GNSS_BREADCRUMB_V1", pack=False, align=None),
    "GNSS_CHIPSETINFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerID", SimTypeArray(SimTypeChar(label="Char"), 25)), ("HardwareID", SimTypeArray(SimTypeChar(label="Char"), 25)), ("FirmwareVersion", SimTypeArray(SimTypeChar(label="Char"), 20)), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_CHIPSETINFO", pack=False, align=None),
    "GNSS_CONTINUOUSTRACKING_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("PreferredInterval", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_CONTINUOUSTRACKING_PARAM", pack=False, align=None),
    "GNSS_CP_NI_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestorId", SimTypeArray(SimTypeChar(label="Char"), 260)), ("NotificationText", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="GNSS_CP_NI_INFO", pack=False, align=None),
    "GNSS_CWTESTDATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("TestResultStatus", SimTypeInt(signed=True, label="Int32")), ("SignalToNoiseRatio", SimTypeFloat(size=64)), ("Frequency", SimTypeFloat(size=64)), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_CWTESTDATA", pack=False, align=None),
    "GNSS_DEVICE_CAPABILITY": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SupportMultipleFixSessions", SimTypeInt(signed=True, label="Int32")), ("SupportMultipleAppSessions", SimTypeInt(signed=True, label="Int32")), ("RequireAGnssInjection", SimTypeInt(signed=True, label="Int32")), ("AgnssFormatSupported", SimTypeInt(signed=False, label="UInt32")), ("AgnssFormatPreferred", SimTypeInt(signed=False, label="UInt32")), ("SupportDistanceTracking", SimTypeInt(signed=True, label="Int32")), ("SupportContinuousTracking", SimTypeInt(signed=True, label="Int32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=True, label="Int32")), ("Reserved3", SimTypeInt(signed=True, label="Int32")), ("Reserved4", SimTypeInt(signed=True, label="Int32")), ("Reserved5", SimTypeInt(signed=True, label="Int32")), ("GeofencingSupport", SimTypeInt(signed=False, label="UInt32")), ("Reserved6", SimTypeInt(signed=True, label="Int32")), ("Reserved7", SimTypeInt(signed=True, label="Int32")), ("SupportCpLocation", SimTypeInt(signed=True, label="Int32")), ("SupportUplV2", SimTypeInt(signed=True, label="Int32")), ("SupportSuplV1", SimTypeInt(signed=True, label="Int32")), ("SupportSuplV2", SimTypeInt(signed=True, label="Int32")), ("SupportedSuplVersion", SimTypeRef("GNSS_SUPL_VERSION", SimStruct)), ("MaxGeofencesSupported", SimTypeInt(signed=False, label="UInt32")), ("SupportMultipleSuplRootCert", SimTypeInt(signed=True, label="Int32")), ("GnssBreadCrumbPayloadVersion", SimTypeInt(signed=False, label="UInt32")), ("MaxGnssBreadCrumbFixes", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 496)),)), name="GNSS_DEVICE_CAPABILITY", pack=False, align=None),
    "GNSS_DISTANCETRACKING_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("MovementThreshold", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_DISTANCETRACKING_PARAM", pack=False, align=None),
    "GNSS_DRIVERCOMMAND_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("CommandType", SimTypeInt(signed=False, label="GNSS_DRIVERCOMMAND_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("CommandDataSize", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("CommandData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GNSS_DRIVERCOMMAND_PARAM", pack=False, align=None),
    "GNSS_DRIVERCOMMAND_TYPE": SimTypeInt(signed=False, label="GNSS_DRIVERCOMMAND_TYPE"),
    "GNSS_DRIVER_REQUEST": SimTypeInt(signed=False, label="GNSS_DRIVER_REQUEST"),
    "GNSS_DRIVER_REQUEST_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Request", SimTypeInt(signed=False, label="GNSS_DRIVER_REQUEST")), ("RequestFlag", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_DRIVER_REQUEST_DATA", pack=False, align=None),
    "GNSS_ERRORINFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("IsRecoverable", SimTypeInt(signed=True, label="Int32")), ("ErrorDescription", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_ERRORINFO", pack=False, align=None),
    "GNSS_EVENT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("EventType", SimTypeInt(signed=False, label="GNSS_EVENT_TYPE")), ("EventDataSize", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("Anonymous", SimUnion({"FixData": SimTypeRef("GNSS_FIXDATA", SimStruct), "AgnssRequest": SimTypeRef("GNSS_AGNSS_REQUEST_PARAM", SimStruct), "NiRequest": SimTypeRef("GNSS_NI_REQUEST_PARAM", SimStruct), "ErrorInformation": SimTypeRef("GNSS_ERRORINFO", SimStruct), "NmeaData": SimTypeRef("GNSS_NMEA_DATA", SimStruct), "GeofenceAlertData": SimTypeRef("GNSS_GEOFENCE_ALERT_DATA", SimStruct), "BreadcrumbAlertData": SimTypeRef("GNSS_BREADCRUMBING_ALERT_DATA", SimStruct), "GeofencesTrackingStatus": SimTypeRef("GNSS_GEOFENCES_TRACKINGSTATUS_DATA", SimStruct), "DriverRequestData": SimTypeRef("GNSS_DRIVER_REQUEST_DATA", SimStruct), "CustomData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="GNSS_EVENT", pack=False, align=None),
    "GNSS_EVENT_2": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("EventType", SimTypeInt(signed=False, label="GNSS_EVENT_TYPE")), ("EventDataSize", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("Anonymous", SimUnion({"FixData": SimTypeRef("GNSS_FIXDATA", SimStruct), "FixData2": SimTypeRef("GNSS_FIXDATA_2", SimStruct), "AgnssRequest": SimTypeRef("GNSS_AGNSS_REQUEST_PARAM", SimStruct), "NiRequest": SimTypeRef("GNSS_NI_REQUEST_PARAM", SimStruct), "ErrorInformation": SimTypeRef("GNSS_ERRORINFO", SimStruct), "NmeaData": SimTypeRef("GNSS_NMEA_DATA", SimStruct), "GeofenceAlertData": SimTypeRef("GNSS_GEOFENCE_ALERT_DATA", SimStruct), "BreadcrumbAlertData": SimTypeRef("GNSS_BREADCRUMBING_ALERT_DATA", SimStruct), "GeofencesTrackingStatus": SimTypeRef("GNSS_GEOFENCES_TRACKINGSTATUS_DATA", SimStruct), "DriverRequestData": SimTypeRef("GNSS_DRIVER_REQUEST_DATA", SimStruct), "CustomData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="GNSS_EVENT_2", pack=False, align=None),
    "GNSS_EVENT_TYPE": SimTypeInt(signed=False, label="GNSS_EVENT_TYPE"),
    "GNSS_FIXDATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("FixSessionID", SimTypeInt(signed=False, label="UInt32")), ("FixTimeStamp", SimTypeRef("FILETIME", SimStruct)), ("IsFinalFix", SimTypeInt(signed=True, label="Int32")), ("FixStatus", SimTypeInt(signed=True, label="Int32")), ("FixLevelOfDetails", SimTypeInt(signed=False, label="UInt32")), ("BasicData", SimTypeRef("GNSS_FIXDATA_BASIC", SimStruct)), ("AccuracyData", SimTypeRef("GNSS_FIXDATA_ACCURACY", SimStruct)), ("SatelliteData", SimTypeRef("GNSS_FIXDATA_SATELLITE", SimStruct)),)), name="GNSS_FIXDATA", pack=False, align=None),
    "GNSS_FIXDATA_2": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("FixSessionID", SimTypeInt(signed=False, label="UInt32")), ("FixTimeStamp", SimTypeRef("FILETIME", SimStruct)), ("IsFinalFix", SimTypeInt(signed=True, label="Int32")), ("FixStatus", SimTypeInt(signed=True, label="Int32")), ("FixLevelOfDetails", SimTypeInt(signed=False, label="UInt32")), ("BasicData", SimTypeRef("GNSS_FIXDATA_BASIC_2", SimStruct)), ("AccuracyData", SimTypeRef("GNSS_FIXDATA_ACCURACY_2", SimStruct)), ("SatelliteData", SimTypeRef("GNSS_FIXDATA_SATELLITE", SimStruct)),)), name="GNSS_FIXDATA_2", pack=False, align=None),
    "GNSS_FIXDATA_ACCURACY": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("HorizontalAccuracy", SimTypeInt(signed=False, label="UInt32")), ("HorizontalErrorMajorAxis", SimTypeInt(signed=False, label="UInt32")), ("HorizontalErrorMinorAxis", SimTypeInt(signed=False, label="UInt32")), ("HorizontalErrorAngle", SimTypeInt(signed=False, label="UInt32")), ("HeadingAccuracy", SimTypeInt(signed=False, label="UInt32")), ("AltitudeAccuracy", SimTypeInt(signed=False, label="UInt32")), ("SpeedAccuracy", SimTypeInt(signed=False, label="UInt32")), ("HorizontalConfidence", SimTypeInt(signed=False, label="UInt32")), ("HeadingConfidence", SimTypeInt(signed=False, label="UInt32")), ("AltitudeConfidence", SimTypeInt(signed=False, label="UInt32")), ("SpeedConfidence", SimTypeInt(signed=False, label="UInt32")), ("PositionDilutionOfPrecision", SimTypeFloat(size=32)), ("HorizontalDilutionOfPrecision", SimTypeFloat(size=32)), ("VerticalDilutionOfPrecision", SimTypeFloat(size=32)),)), name="GNSS_FIXDATA_ACCURACY", pack=False, align=None),
    "GNSS_FIXDATA_ACCURACY_2": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("HorizontalAccuracy", SimTypeFloat(size=64)), ("HorizontalErrorMajorAxis", SimTypeFloat(size=64)), ("HorizontalErrorMinorAxis", SimTypeFloat(size=64)), ("HorizontalErrorAngle", SimTypeFloat(size=64)), ("HeadingAccuracy", SimTypeFloat(size=64)), ("AltitudeAccuracy", SimTypeFloat(size=64)), ("SpeedAccuracy", SimTypeFloat(size=64)), ("HorizontalConfidence", SimTypeInt(signed=False, label="UInt32")), ("HeadingConfidence", SimTypeInt(signed=False, label="UInt32")), ("AltitudeConfidence", SimTypeInt(signed=False, label="UInt32")), ("SpeedConfidence", SimTypeInt(signed=False, label="UInt32")), ("PositionDilutionOfPrecision", SimTypeFloat(size=64)), ("HorizontalDilutionOfPrecision", SimTypeFloat(size=64)), ("VerticalDilutionOfPrecision", SimTypeFloat(size=64)), ("GeometricDilutionOfPrecision", SimTypeFloat(size=64)), ("TimeDilutionOfPrecision", SimTypeFloat(size=64)),)), name="GNSS_FIXDATA_ACCURACY_2", pack=False, align=None),
    "GNSS_FIXDATA_BASIC": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Latitude", SimTypeFloat(size=64)), ("Longitude", SimTypeFloat(size=64)), ("Altitude", SimTypeFloat(size=64)), ("Speed", SimTypeFloat(size=64)), ("Heading", SimTypeFloat(size=64)),)), name="GNSS_FIXDATA_BASIC", pack=False, align=None),
    "GNSS_FIXDATA_BASIC_2": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Latitude", SimTypeFloat(size=64)), ("Longitude", SimTypeFloat(size=64)), ("Altitude", SimTypeFloat(size=64)), ("Speed", SimTypeFloat(size=64)), ("Heading", SimTypeFloat(size=64)), ("AltitudeEllipsoid", SimTypeFloat(size=64)),)), name="GNSS_FIXDATA_BASIC_2", pack=False, align=None),
    "GNSS_FIXDATA_SATELLITE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SatelliteCount", SimTypeInt(signed=False, label="UInt32")), ("SatelliteArray", SimTypeArray(SimTypeRef("GNSS_SATELLITEINFO", SimStruct), 64)),)), name="GNSS_FIXDATA_SATELLITE", pack=False, align=None),
    "GNSS_FIXSESSIONTYPE": SimTypeInt(signed=False, label="GNSS_FIXSESSIONTYPE"),
    "GNSS_FIXSESSION_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("FixSessionID", SimTypeInt(signed=False, label="UInt32")), ("SessionType", SimTypeInt(signed=False, label="GNSS_FIXSESSIONTYPE")), ("HorizontalAccuracy", SimTypeInt(signed=False, label="UInt32")), ("HorizontalConfidence", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 9)), ("FixLevelOfDetails", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"SingleShotParam": SimTypeRef("GNSS_SINGLESHOT_PARAM", SimStruct), "DistanceParam": SimTypeRef("GNSS_DISTANCETRACKING_PARAM", SimStruct), "ContinuousParam": SimTypeRef("GNSS_CONTINUOUSTRACKING_PARAM", SimStruct), "LkgFixParam": SimTypeRef("GNSS_LKGFIX_PARAM", SimStruct), "UnusedParam": SimTypeArray(SimTypeChar(label="Byte"), 268)}, name="<anon>", label="None")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="GNSS_FIXSESSION_PARAM", pack=False, align=None),
    "GNSS_GEOFENCES_TRACKINGSTATUS_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")), ("StatusTimeStamp", SimTypeRef("FILETIME", SimStruct)), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_GEOFENCES_TRACKINGSTATUS_DATA", pack=False, align=None),
    "GNSS_GEOFENCE_ALERT_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("GeofenceID", SimTypeInt(signed=False, label="UInt32")), ("GeofenceState", SimTypeInt(signed=False, label="GNSS_GEOFENCE_STATE")), ("FixBasicData", SimTypeRef("GNSS_FIXDATA_BASIC", SimStruct)), ("FixAccuracyData", SimTypeRef("GNSS_FIXDATA_ACCURACY", SimStruct)), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_GEOFENCE_ALERT_DATA", pack=False, align=None),
    "GNSS_GEOFENCE_CREATE_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("AlertTypes", SimTypeInt(signed=False, label="UInt32")), ("InitialState", SimTypeInt(signed=False, label="GNSS_GEOFENCE_STATE")), ("Boundary", SimTypeRef("GNSS_GEOREGION", SimStruct)), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_GEOFENCE_CREATE_PARAM", pack=False, align=None),
    "GNSS_GEOFENCE_CREATE_RESPONSE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("CreationStatus", SimTypeInt(signed=True, label="Int32")), ("GeofenceID", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_GEOFENCE_CREATE_RESPONSE", pack=False, align=None),
    "GNSS_GEOFENCE_DELETE_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("GeofenceID", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_GEOFENCE_DELETE_PARAM", pack=False, align=None),
    "GNSS_GEOFENCE_STATE": SimTypeInt(signed=False, label="GNSS_GEOFENCE_STATE"),
    "GNSS_GEOREGION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("GeoRegionType", SimTypeInt(signed=False, label="GNSS_GEOREGIONTYPE")), ("Anonymous", SimUnion({"Circle": SimTypeRef("GNSS_GEOREGION_CIRCLE", SimStruct), "Unused": SimTypeArray(SimTypeChar(label="Byte"), 512)}, name="<anon>", label="None")),)), name="GNSS_GEOREGION", pack=False, align=None),
    "GNSS_GEOREGIONTYPE": SimTypeInt(signed=False, label="GNSS_GEOREGIONTYPE"),
    "GNSS_GEOREGION_CIRCLE": SimStruct(OrderedDict((("Latitude", SimTypeFloat(size=64)), ("Longitude", SimTypeFloat(size=64)), ("RadiusInMeters", SimTypeFloat(size=64)),)), name="GNSS_GEOREGION_CIRCLE", pack=False, align=None),
    "GNSS_LKGFIX_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_LKGFIX_PARAM", pack=False, align=None),
    "GNSS_NI_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="GNSS_NI_NOTIFICATION_TYPE"),
    "GNSS_NI_PLANE_TYPE": SimTypeInt(signed=False, label="GNSS_NI_PLANE_TYPE"),
    "GNSS_NI_REQUEST_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestId", SimTypeInt(signed=False, label="UInt32")), ("RequestType", SimTypeInt(signed=False, label="GNSS_NI_REQUEST_TYPE")), ("NotificationType", SimTypeInt(signed=False, label="GNSS_NI_NOTIFICATION_TYPE")), ("RequestPlaneType", SimTypeInt(signed=False, label="GNSS_NI_PLANE_TYPE")), ("Anonymous", SimUnion({"SuplNiInfo": SimTypeRef("GNSS_SUPL_NI_INFO", SimStruct), "CpNiInfo": SimTypeRef("GNSS_CP_NI_INFO", SimStruct), "V2UplNiInfo": SimTypeRef("GNSS_V2UPL_NI_INFO", SimStruct)}, name="<anon>", label="None")), ("ResponseTimeInSec", SimTypeInt(signed=False, label="UInt32")), ("EmergencyLocation", SimTypeInt(signed=True, label="Int32")),)), name="GNSS_NI_REQUEST_PARAM", pack=False, align=None),
    "GNSS_NI_REQUEST_TYPE": SimTypeInt(signed=False, label="GNSS_NI_REQUEST_TYPE"),
    "GNSS_NI_RESPONSE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestId", SimTypeInt(signed=False, label="UInt32")), ("UserResponse", SimTypeInt(signed=False, label="GNSS_NI_USER_RESPONSE")),)), name="GNSS_NI_RESPONSE", pack=False, align=None),
    "GNSS_NI_USER_RESPONSE": SimTypeInt(signed=False, label="GNSS_NI_USER_RESPONSE"),
    "GNSS_NMEA_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("NmeaSentences", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="GNSS_NMEA_DATA", pack=False, align=None),
    "GNSS_PLATFORM_CAPABILITY": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SupportAgnssInjection", SimTypeInt(signed=True, label="Int32")), ("AgnssFormatSupported", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 516)),)), name="GNSS_PLATFORM_CAPABILITY", pack=False, align=None),
    "GNSS_SATELLITEINFO": SimStruct(OrderedDict((("SatelliteId", SimTypeInt(signed=False, label="UInt32")), ("UsedInPositiong", SimTypeInt(signed=True, label="Int32")), ("Elevation", SimTypeFloat(size=64)), ("Azimuth", SimTypeFloat(size=64)), ("SignalToNoiseRatio", SimTypeFloat(size=64)),)), name="GNSS_SATELLITEINFO", pack=False, align=None),
    "GNSS_SELFTESTCONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("TestType", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("InBufLen", SimTypeInt(signed=False, label="UInt32")), ("InBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GNSS_SELFTESTCONFIG", pack=False, align=None),
    "GNSS_SELFTESTRESULT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("TestResultStatus", SimTypeInt(signed=True, label="Int32")), ("Result", SimTypeInt(signed=False, label="UInt32")), ("PinFailedBitMask", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("OutBufLen", SimTypeInt(signed=False, label="UInt32")), ("OutBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GNSS_SELFTESTRESULT", pack=False, align=None),
    "GNSS_SINGLESHOT_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("ResponseTime", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_SINGLESHOT_PARAM", pack=False, align=None),
    "GNSS_STOPFIXSESSION_PARAM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("FixSessionID", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_STOPFIXSESSION_PARAM", pack=False, align=None),
    "GNSS_SUPL_CERT_ACTION": SimTypeInt(signed=False, label="GNSS_SUPL_CERT_ACTION"),
    "GNSS_SUPL_CERT_CONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("CertAction", SimTypeInt(signed=False, label="GNSS_SUPL_CERT_ACTION")), ("SuplCertName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("CertSize", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)), ("CertData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GNSS_SUPL_CERT_CONFIG", pack=False, align=None),
    "GNSS_SUPL_HSLP_CONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SuplHslp", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("SuplHslpFromImsi", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_SUPL_HSLP_CONFIG", pack=False, align=None),
    "GNSS_SUPL_NI_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestorId", SimTypeArray(SimTypeChar(label="Char"), 260)), ("ClientName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("SuplNiUrl", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="GNSS_SUPL_NI_INFO", pack=False, align=None),
    "GNSS_SUPL_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_SUPL_VERSION", pack=False, align=None),
    "GNSS_SUPL_VERSION_2": SimStruct(OrderedDict((("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ServiceIndicator", SimTypeInt(signed=False, label="UInt32")),)), name="GNSS_SUPL_VERSION_2", pack=False, align=None),
    "GNSS_V2UPL_CONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("MPC", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("PDE", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("ApplicationTypeIndicator_MR", SimTypeChar(label="Byte")), ("Unused", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="GNSS_V2UPL_CONFIG", pack=False, align=None),
    "GNSS_V2UPL_NI_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestorId", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="GNSS_V2UPL_NI_INFO", pack=False, align=None),
    "GOBJENUMPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "GOFFSET": SimStruct(OrderedDict((("du", SimTypeInt(signed=True, label="Int32")), ("dv", SimTypeInt(signed=True, label="Int32")),)), name="GOFFSET", pack=False, align=None),
    "GOPHER_ABSTRACT_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("ShortAbstract", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("AbstractFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_ABSTRACT_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_ADMIN_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Comment", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("EmailAddress", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_ADMIN_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_ASK_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("QuestionType", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("QuestionText", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_ASK_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_ATTRIBUTE_ENUMERATOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("GOPHER_ATTRIBUTE_TYPE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpAttributeInfo", "dwError"]), offset=0),
    "GOPHER_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("CategoryId", SimTypeInt(signed=False, label="UInt32")), ("AttributeId", SimTypeInt(signed=False, label="UInt32")), ("AttributeType", SimUnion({"Admin": SimTypeRef("GOPHER_ADMIN_ATTRIBUTE_TYPE", SimStruct), "ModDate": SimTypeRef("GOPHER_MOD_DATE_ATTRIBUTE_TYPE", SimStruct), "Ttl": SimTypeRef("GOPHER_TTL_ATTRIBUTE_TYPE", SimStruct), "Score": SimTypeRef("GOPHER_SCORE_ATTRIBUTE_TYPE", SimStruct), "ScoreRange": SimTypeRef("GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE", SimStruct), "Site": SimTypeRef("GOPHER_SITE_ATTRIBUTE_TYPE", SimStruct), "Organization": SimTypeRef("GOPHER_ORGANIZATION_ATTRIBUTE_TYPE", SimStruct), "Location": SimTypeRef("GOPHER_LOCATION_ATTRIBUTE_TYPE", SimStruct), "GeographicalLocation": SimTypeRef("GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE", SimStruct), "TimeZone": SimTypeRef("GOPHER_TIMEZONE_ATTRIBUTE_TYPE", SimStruct), "Provider": SimTypeRef("GOPHER_PROVIDER_ATTRIBUTE_TYPE", SimStruct), "Version": SimTypeRef("GOPHER_VERSION_ATTRIBUTE_TYPE", SimStruct), "Abstract": SimTypeRef("GOPHER_ABSTRACT_ATTRIBUTE_TYPE", SimStruct), "View": SimTypeRef("GOPHER_VIEW_ATTRIBUTE_TYPE", SimStruct), "Veronica": SimTypeRef("GOPHER_VERONICA_ATTRIBUTE_TYPE", SimStruct), "Ask": SimTypeRef("GOPHER_ASK_ATTRIBUTE_TYPE", SimStruct), "Unknown": SimTypeRef("GOPHER_UNKNOWN_ATTRIBUTE_TYPE", SimStruct)}, name="<anon>", label="None")),)), name="GOPHER_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_FIND_DATAA": SimStruct(OrderedDict((("DisplayString", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("GopherType", SimTypeInt(signed=False, label="GOPHER_TYPE")), ("SizeLow", SimTypeInt(signed=False, label="UInt32")), ("SizeHigh", SimTypeInt(signed=False, label="UInt32")), ("LastModificationTime", SimTypeRef("FILETIME", SimStruct)), ("Locator", SimTypeArray(SimTypeChar(label="SByte"), 654)),)), name="GOPHER_FIND_DATAA", pack=False, align=None),
    "GOPHER_FIND_DATAW": SimStruct(OrderedDict((("DisplayString", SimTypeArray(SimTypeChar(label="Char"), 129)), ("GopherType", SimTypeInt(signed=False, label="GOPHER_TYPE")), ("SizeLow", SimTypeInt(signed=False, label="UInt32")), ("SizeHigh", SimTypeInt(signed=False, label="UInt32")), ("LastModificationTime", SimTypeRef("FILETIME", SimStruct)), ("Locator", SimTypeArray(SimTypeChar(label="Char"), 654)),)), name="GOPHER_FIND_DATAW", pack=False, align=None),
    "GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("DegreesNorth", SimTypeInt(signed=True, label="Int32")), ("MinutesNorth", SimTypeInt(signed=True, label="Int32")), ("SecondsNorth", SimTypeInt(signed=True, label="Int32")), ("DegreesEast", SimTypeInt(signed=True, label="Int32")), ("MinutesEast", SimTypeInt(signed=True, label="Int32")), ("SecondsEast", SimTypeInt(signed=True, label="Int32")),)), name="GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_LOCATION_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Location", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_LOCATION_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_MOD_DATE_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("DateAndTime", SimTypeRef("FILETIME", SimStruct)),)), name="GOPHER_MOD_DATE_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_ORGANIZATION_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Organization", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_ORGANIZATION_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_PROVIDER_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Provider", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_PROVIDER_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_SCORE_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Score", SimTypeInt(signed=True, label="Int32")),)), name="GOPHER_SCORE_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("LowerBound", SimTypeInt(signed=True, label="Int32")), ("UpperBound", SimTypeInt(signed=True, label="Int32")),)), name="GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_SITE_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Site", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_SITE_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_TIMEZONE_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Zone", SimTypeInt(signed=True, label="Int32")),)), name="GOPHER_TIMEZONE_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_TTL_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Ttl", SimTypeInt(signed=False, label="UInt32")),)), name="GOPHER_TTL_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_TYPE": SimTypeInt(signed=False, label="GOPHER_TYPE"),
    "GOPHER_UNKNOWN_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Text", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_UNKNOWN_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_VERONICA_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("TreeWalk", SimTypeInt(signed=True, label="Int32")),)), name="GOPHER_VERONICA_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_VERSION_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("Version", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="GOPHER_VERSION_ATTRIBUTE_TYPE", pack=False, align=None),
    "GOPHER_VIEW_ATTRIBUTE_TYPE": SimStruct(OrderedDict((("ContentType", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Language", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="GOPHER_VIEW_ATTRIBUTE_TYPE", pack=False, align=None),
    "GPA_MEMORY_CHUNK": SimStruct(OrderedDict((("GuestPhysicalStartPageIndex", SimTypeLongLong(signed=False, label="UInt64")), ("PageCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="GPA_MEMORY_CHUNK", pack=False, align=None),
    "GPFIDL_FLAGS": SimTypeInt(signed=False, label="GPFIDL_FLAGS"),
    "GPIOBUTTONS_BUTTON_TYPE": SimTypeInt(signed=False, label="GPIOBUTTONS_BUTTON_TYPE"),
    "GPMBackupType": SimTypeInt(signed=False, label="GPMBackupType"),
    "GPMDestinationOption": SimTypeInt(signed=False, label="GPMDestinationOption"),
    "GPMEntryType": SimTypeInt(signed=False, label="GPMEntryType"),
    "GPMPermissionType": SimTypeInt(signed=False, label="GPMPermissionType"),
    "GPMRSOPMode": SimTypeInt(signed=False, label="GPMRSOPMode"),
    "GPMReportType": SimTypeInt(signed=False, label="GPMReportType"),
    "GPMReportingOptions": SimTypeInt(signed=False, label="GPMReportingOptions"),
    "GPMSOMType": SimTypeInt(signed=False, label="GPMSOMType"),
    "GPMSearchOperation": SimTypeInt(signed=False, label="GPMSearchOperation"),
    "GPMSearchProperty": SimTypeInt(signed=False, label="GPMSearchProperty"),
    "GPMStarterGPOType": SimTypeInt(signed=False, label="GPMStarterGPOType"),
    "GPOBROWSEINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpInitialOU", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDSPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwDSPathSize", SimTypeInt(signed=False, label="UInt32")), ("lpName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNameSize", SimTypeInt(signed=False, label="UInt32")), ("gpoType", SimTypeInt(signed=False, label="GROUP_POLICY_OBJECT_TYPE")), ("gpoHint", SimTypeInt(signed=False, label="GROUP_POLICY_HINT_TYPE")),)), name="GPOBROWSEINFO", pack=False, align=None),
    "GPO_LINK": SimTypeInt(signed=False, label="GPO_LINK"),
    "GPO_OPEN_FLAGS": SimTypeInt(signed=False, label="GPO_OPEN_FLAGS"),
    "GPO_OPTIONS": SimTypeInt(signed=False, label="GPO_OPTIONS"),
    "GPO_SECTION": SimTypeInt(signed=False, label="GPO_SECTION"),
    "GPT_ATTRIBUTES": SimTypeInt(signed=False, label="GPT_ATTRIBUTES"),
    "GPUP_DRIVER_ESCAPE_INPUT": SimStruct(OrderedDict((("vfLUID", SimTypeRef("LUID", SimStruct)),)), name="GPUP_DRIVER_ESCAPE_INPUT", pack=False, align=None),
    "GP_LOG_PAGE_DESCRIPTOR": SimStruct(OrderedDict((("LogAddress", SimTypeShort(signed=False, label="UInt16")), ("LogSectors", SimTypeShort(signed=False, label="UInt16")),)), name="GP_LOG_PAGE_DESCRIPTOR", pack=False, align=None),
    "GRADIENT_FILL": SimTypeInt(signed=False, label="GRADIENT_FILL"),
    "GRADIENT_RECT": SimStruct(OrderedDict((("UpperLeft", SimTypeInt(signed=False, label="UInt32")), ("LowerRight", SimTypeInt(signed=False, label="UInt32")),)), name="GRADIENT_RECT", pack=False, align=None),
    "GRADIENT_TRIANGLE": SimStruct(OrderedDict((("Vertex1", SimTypeInt(signed=False, label="UInt32")), ("Vertex2", SimTypeInt(signed=False, label="UInt32")), ("Vertex3", SimTypeInt(signed=False, label="UInt32")),)), name="GRADIENT_TRIANGLE", pack=False, align=None),
    "GRAPHICS_MODE": SimTypeInt(signed=False, label="GRAPHICS_MODE"),
    "GRAYCOLOR": SimStruct(OrderedDict((("gray", SimTypeShort(signed=False, label="UInt16")),)), name="GRAYCOLOR", pack=False, align=None),
    "GRAYSTRINGPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "GRE_CONFIG_PARAMS0": SimStruct(OrderedDict((("dwNumPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")),)), name="GRE_CONFIG_PARAMS0", pack=False, align=None),
    "GRIDCELLBACKGROUNDSTATES": SimTypeInt(signed=False, label="GRIDCELLBACKGROUNDSTATES"),
    "GRIDCELLSTATES": SimTypeInt(signed=False, label="GRIDCELLSTATES"),
    "GRIDCELLUPPERSTATES": SimTypeInt(signed=False, label="GRIDCELLUPPERSTATES"),
    "GRIPPERSTATES": SimTypeInt(signed=False, label="GRIPPERSTATES"),
    "GROUPBOXSTATES": SimTypeInt(signed=False, label="GROUPBOXSTATES"),
    "GROUPHEADERLINESTATES": SimTypeInt(signed=False, label="GROUPHEADERLINESTATES"),
    "GROUPHEADERSTATES": SimTypeInt(signed=False, label="GROUPHEADERSTATES"),
    "GROUPPROP": SimTypeInt(signed=False, label="GROUPPROP"),
    "GROUPTYPINGCHANGE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("fGroupTyping", SimTypeInt(signed=True, label="Int32")),)), name="GROUPTYPINGCHANGE", pack=False, align=None),
    "GROUP_AFFINITY": SimStruct(OrderedDict((("Mask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Group", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)),)), name="GROUP_AFFINITY", pack=False, align=None),
    "GROUP_FAILURE_INFO": SimStruct(OrderedDict((("dwFailoverAttemptsRemaining", SimTypeInt(signed=False, label="UInt32")), ("dwFailoverPeriodRemaining", SimTypeInt(signed=False, label="UInt32")),)), name="GROUP_FAILURE_INFO", pack=False, align=None),
    "GROUP_FAILURE_INFO_BUFFER": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Info", SimTypeRef("GROUP_FAILURE_INFO", SimStruct)),)), name="GROUP_FAILURE_INFO_BUFFER", pack=False, align=None),
    "GROUP_FILTER": SimStruct(OrderedDict((("gf_interface", SimTypeInt(signed=False, label="UInt32")), ("gf_group", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("gf_fmode", SimTypeInt(signed=False, label="MULTICAST_MODE_TYPE")), ("gf_numsrc", SimTypeInt(signed=False, label="UInt32")), ("gf_slist", SimTypePointer(SimTypeRef("SOCKADDR_STORAGE", SimStruct), offset=0)),)), name="GROUP_FILTER", pack=False, align=None),
    "GROUP_INFO_0": SimStruct(OrderedDict((("grpi0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GROUP_INFO_0", pack=False, align=None),
    "GROUP_INFO_1": SimStruct(OrderedDict((("grpi1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi1_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GROUP_INFO_1", pack=False, align=None),
    "GROUP_INFO_1002": SimStruct(OrderedDict((("grpi1002_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GROUP_INFO_1002", pack=False, align=None),
    "GROUP_INFO_1005": SimStruct(OrderedDict((("grpi1005_attributes", SimTypeInt(signed=False, label="UInt32")),)), name="GROUP_INFO_1005", pack=False, align=None),
    "GROUP_INFO_2": SimStruct(OrderedDict((("grpi2_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi2_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi2_group_id", SimTypeInt(signed=False, label="UInt32")), ("grpi2_attributes", SimTypeInt(signed=False, label="UInt32")),)), name="GROUP_INFO_2", pack=False, align=None),
    "GROUP_INFO_3": SimStruct(OrderedDict((("grpi3_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi3_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi3_group_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("grpi3_attributes", SimTypeInt(signed=False, label="UInt32")),)), name="GROUP_INFO_3", pack=False, align=None),
    "GROUP_NAME_SYNTAX": SimTypeInt(signed=False, label="GROUP_NAME_SYNTAX"),
    "GROUP_POLICY_HINT_TYPE": SimTypeInt(signed=False, label="GROUP_POLICY_HINT_TYPE"),
    "GROUP_POLICY_OBJECTA": SimStruct(OrderedDict((("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpDSPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpFileSysPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szGPOName", SimTypeArray(SimTypeChar(label="SByte"), 50)), ("GPOLink", SimTypeInt(signed=False, label="GPO_LINK")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pNext", SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0)), ("pPrev", SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0)), ("lpExtensions", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lParam2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpLink", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="GROUP_POLICY_OBJECTA", pack=False, align=None),
    "GROUP_POLICY_OBJECTW": SimStruct(OrderedDict((("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpDSPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpFileSysPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szGPOName", SimTypeArray(SimTypeChar(label="Char"), 50)), ("GPOLink", SimTypeInt(signed=False, label="GPO_LINK")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pNext", SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTW", SimStruct), offset=0)), ("pPrev", SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTW", SimStruct), offset=0)), ("lpExtensions", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lParam2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpLink", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GROUP_POLICY_OBJECTW", pack=False, align=None),
    "GROUP_POLICY_OBJECT_TYPE": SimTypeInt(signed=False, label="GROUP_POLICY_OBJECT_TYPE"),
    "GROUP_RELATIONSHIP": SimStruct(OrderedDict((("MaximumGroupCount", SimTypeShort(signed=False, label="UInt16")), ("ActiveGroupCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("GroupInfo", SimTypePointer(SimTypeRef("PROCESSOR_GROUP_INFO", SimStruct), offset=0)),)), name="GROUP_RELATIONSHIP", pack=False, align=None),
    "GROUP_REQ": SimStruct(OrderedDict((("gr_interface", SimTypeInt(signed=False, label="UInt32")), ("gr_group", SimTypeRef("SOCKADDR_STORAGE", SimStruct)),)), name="GROUP_REQ", pack=False, align=None),
    "GROUP_SOURCE_REQ": SimStruct(OrderedDict((("gsr_interface", SimTypeInt(signed=False, label="UInt32")), ("gsr_group", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("gsr_source", SimTypeRef("SOCKADDR_STORAGE", SimStruct)),)), name="GROUP_SOURCE_REQ", pack=False, align=None),
    "GROUP_USERS_INFO_0": SimStruct(OrderedDict((("grui0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="GROUP_USERS_INFO_0", pack=False, align=None),
    "GROUP_USERS_INFO_1": SimStruct(OrderedDict((("grui1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grui1_attributes", SimTypeInt(signed=False, label="UInt32")),)), name="GROUP_USERS_INFO_1", pack=False, align=None),
    "GRP_PLACEMENT_OPTIONS": SimTypeInt(signed=False, label="GRP_PLACEMENT_OPTIONS"),
    "GSM610WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="GSM610WAVEFORMAT", pack=False, align=None),
    "GUEST_OS_INFO": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "ClosedSource": SimTypeRef("_ClosedSource_e__Struct", SimStruct), "OpenSource": SimTypeRef("_OpenSource_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "GUEST_OS_MICROSOFT_IDS": SimTypeInt(signed=False, label="GUEST_OS_MICROSOFT_IDS"),
    "GUEST_OS_OPENSOURCE_IDS": SimTypeInt(signed=False, label="GUEST_OS_OPENSOURCE_IDS"),
    "GUEST_OS_VENDOR": SimTypeInt(signed=False, label="GUEST_OS_VENDOR"),
    "GUEST_SYMBOLS_PROVIDER_DEBUG_INFO_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["InfoMessage"]), offset=0),
    "GUIDELINE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPrivateSize", SimTypeInt(signed=False, label="UInt32")), ("dwPrivateOffset", SimTypeInt(signed=False, label="UInt32")),)), name="GUIDELINE", pack=False, align=None),
    "GUIDKIND": SimTypeInt(signed=False, label="GUIDKIND"),
    "GUID_IO_DISK_CLONE_ARRIVAL_INFORMATION": SimStruct(OrderedDict((("DiskNumber", SimTypeInt(signed=False, label="UInt32")),)), name="GUID_IO_DISK_CLONE_ARRIVAL_INFORMATION", pack=False, align=None),
    "GUITHREADINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="GUITHREADINFO_FLAGS")), ("hwndActive", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndFocus", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndCapture", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndMenuOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndMoveSize", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndCaret", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rcCaret", SimTypeRef("RECT", SimStruct)),)), name="GUITHREADINFO", pack=False, align=None),
    "GUITHREADINFO_FLAGS": SimTypeInt(signed=False, label="GUITHREADINFO_FLAGS"),
    "Gads_parms_t": SimStruct(OrderedDict((("Gads_serv_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("Gads_Ctot_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Gads_Ctot", SimTypeInt(signed=False, label="UInt32")), ("Gads_Dtot_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Gads_Dtot", SimTypeInt(signed=False, label="UInt32")), ("Gads_Csum_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Gads_Csum", SimTypeInt(signed=False, label="UInt32")), ("Gads_Dsum_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Gads_Dsum", SimTypeInt(signed=False, label="UInt32")),)), name="Gads_parms_t", pack=False, align=None),
    "GameUICompletionRoutine": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["returnCode", "context"]), offset=0),
    "GamutBoundaryDescription": SimStruct(OrderedDict((("pPrimaries", SimTypePointer(SimTypeRef("PrimaryJabColors", SimStruct), offset=0)), ("cNeutralSamples", SimTypeInt(signed=False, label="UInt32")), ("pNeutralSamples", SimTypePointer(SimTypeRef("JabColorF", SimStruct), offset=0)), ("pReferenceShell", SimTypePointer(SimTypeRef("GamutShell", SimStruct), offset=0)), ("pPlausibleShell", SimTypePointer(SimTypeRef("GamutShell", SimStruct), offset=0)), ("pPossibleShell", SimTypePointer(SimTypeRef("GamutShell", SimStruct), offset=0)),)), name="GamutBoundaryDescription", pack=False, align=None),
    "GamutShell": SimStruct(OrderedDict((("JMin", SimTypeFloat(size=32)), ("JMax", SimTypeFloat(size=32)), ("cVertices", SimTypeInt(signed=False, label="UInt32")), ("cTriangles", SimTypeInt(signed=False, label="UInt32")), ("pVertices", SimTypePointer(SimTypeRef("JabColorF", SimStruct), offset=0)), ("pTriangles", SimTypePointer(SimTypeRef("GamutShellTriangle", SimStruct), offset=0)),)), name="GamutShell", pack=False, align=None),
    "GamutShellTriangle": SimStruct(OrderedDict((("aVertexIndex", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="GamutShellTriangle", pack=False, align=None),
    "GdiplusAbort": SimTypeBottom(label="GdiplusAbort"),
    "GdiplusStartupInput": SimStruct(OrderedDict((("GdiplusVersion", SimTypeInt(signed=False, label="UInt32")), ("DebugEventCallback", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SuppressBackgroundThread", SimTypeInt(signed=True, label="Int32")), ("SuppressExternalCodecs", SimTypeInt(signed=True, label="Int32")),)), name="GdiplusStartupInput", pack=False, align=None),
    "GdiplusStartupInputEx": SimStruct(OrderedDict((("Base", SimTypeRef("GdiplusStartupInput", SimStruct)), ("StartupParameters", SimTypeInt(signed=True, label="Int32")),)), name="GdiplusStartupInputEx", pack=False, align=None),
    "GdiplusStartupOutput": SimStruct(OrderedDict((("NotificationHook", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NotificationUnhook", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="GdiplusStartupOutput", pack=False, align=None),
    "GdiplusStartupParams": SimTypeInt(signed=False, label="GdiplusStartupParams"),
    "GenAdspecParams": SimStruct(OrderedDict((("gen_parm_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("gen_parm_hopcnt_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("gen_parm_hopcnt", SimTypeInt(signed=False, label="UInt32")), ("gen_parm_pathbw_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("gen_parm_path_bw", SimTypeFloat(size=32)), ("gen_parm_minlat_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("gen_parm_min_latency", SimTypeInt(signed=False, label="UInt32")), ("gen_parm_compmtu_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("gen_parm_composed_MTU", SimTypeInt(signed=False, label="UInt32")),)), name="GenAdspecParams", pack=False, align=None),
    "GenTspec": SimStruct(OrderedDict((("gen_Tspec_serv_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("gen_Tspec_parm_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("gen_Tspec_parms", SimTypeRef("GenTspecParms", SimStruct)),)), name="GenTspec", pack=False, align=None),
    "GenTspecParms": SimStruct(OrderedDict((("TB_Tspec_r", SimTypeFloat(size=32)), ("TB_Tspec_b", SimTypeFloat(size=32)), ("TB_Tspec_p", SimTypeFloat(size=32)), ("TB_Tspec_m", SimTypeInt(signed=False, label="UInt32")), ("TB_Tspec_M", SimTypeInt(signed=False, label="UInt32")),)), name="GenTspecParms", pack=False, align=None),
    "Gender": SimTypeInt(signed=False, label="Gender"),
    "GeneratorParametersType": SimTypeInt(signed=False, label="GeneratorParametersType"),
    "GenericFontFamily": SimTypeInt(signed=False, label="GenericFontFamily"),
    "GetAppTrackerDataFlags": SimTypeInt(signed=False, label="GetAppTrackerDataFlags"),
    "GetThumbnailImageAbort": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "GpAdjustableArrowCap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpBitmap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpBrush": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpCachedBitmap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpCustomLineCap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpFont": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpFontCollection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpFontFamily": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpGraphics": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpHatch": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpImage": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpImageAttributes": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpInstalledFontCollection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpLineGradient": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpMetafile": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpPath": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpPathGradient": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpPathIterator": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpPen": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpPrivateFontCollection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpRegion": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpSolidFill": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpStringFormat": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GpTestControlEnum": SimTypeInt(signed=False, label="GpTestControlEnum"),
    "GpTexture": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "GuarFlowSpec": SimStruct(OrderedDict((("Guar_serv_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("Guar_Tspec_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Guar_Tspec_parms", SimTypeRef("GenTspecParms", SimStruct)), ("Guar_Rspec_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Guar_Rspec", SimTypeRef("GuarRspec", SimStruct)),)), name="GuarFlowSpec", pack=False, align=None),
    "GuarRspec": SimStruct(OrderedDict((("Guar_R", SimTypeFloat(size=32)), ("Guar_S", SimTypeInt(signed=False, label="UInt32")),)), name="GuarRspec", pack=False, align=None),
    "GuardInterval": SimTypeInt(signed=False, label="GuardInterval"),
    "HACCEL": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HACCESSOR": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "HACMDRIVER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HACMDRIVERID": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HACMOBJ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HACMSTREAM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HALIGN": SimTypeInt(signed=False, label="HALIGN"),
    "HAL_AMLI_BAD_IO_ADDRESS_LIST": SimStruct(OrderedDict((("BadAddrBegin", SimTypeInt(signed=False, label="UInt32")), ("BadAddrSize", SimTypeInt(signed=False, label="UInt32")), ("OSVersionTrigger", SimTypeInt(signed=False, label="UInt32")), ("IOHandler", SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["fRead", "dwAddr", "dwSize", "pdwData"]), offset=0)),)), name="HAL_AMLI_BAD_IO_ADDRESS_LIST", pack=False, align=None),
    "HAL_APIC_DESTINATION_MODE": SimTypeInt(signed=False, label="HAL_APIC_DESTINATION_MODE"),
    "HAL_BUS_INFORMATION": SimStruct(OrderedDict((("BusType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("ConfigurationType", SimTypeInt(signed=False, label="BUS_DATA_TYPE")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="HAL_BUS_INFORMATION", pack=False, align=None),
    "HAL_CALLBACKS": SimStruct(OrderedDict((("SetSystemInformation", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("BusCheck", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HAL_CALLBACKS", pack=False, align=None),
    "HAL_DISPATCH": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("HalQuerySystemInformation", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="HAL_QUERY_INFORMATION_CLASS"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InformationClass", "BufferSize", "Buffer", "ReturnedLength"]), offset=0)), ("HalSetSystemInformation", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="HAL_SET_INFORMATION_CLASS"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InformationClass", "BufferSize", "Buffer"]), offset=0)), ("HalQueryBusSlots", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["BusHandler", "BufferSize", "SlotNumbers", "ReturnedLength"]), offset=0)), ("Spare1", SimTypeInt(signed=False, label="UInt32")), ("HalExamineMBR", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "SectorSize", "MBRTypeIdentifier", "Buffer"]), offset=0)), ("HalIoReadPartitionTable", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("DRIVE_LAYOUT_INFORMATION", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "SectorSize", "ReturnRecognizedPartitions", "PartitionBuffer"]), offset=0)), ("HalIoSetPartitionInformation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "SectorSize", "PartitionNumber", "PartitionType"]), offset=0)), ("HalIoWritePartitionTable", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DRIVE_LAYOUT_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "SectorSize", "SectorsPerTrack", "NumberOfHeads", "PartitionBuffer"]), offset=0)), ("HalReferenceHandlerForBus", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["InterfaceType", "BusNumber"]), offset=0)), ("HalReferenceBusHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler"]), offset=0)), ("HalDereferenceBusHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler"]), offset=0)), ("HalInitPnpDriver", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("HalInitPowerManagement", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PM_DISPATCH_TABLE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("PM_DISPATCH_TABLE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PmDriverDispatchTable", "PmHalDispatchTable"]), offset=0)), ("HalGetDmaAdapter", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEVICE_DESCRIPTION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), arg_names=["PhysicalDeviceObject", "DeviceDescriptor", "NumberOfMapRegisters"]), offset=0)), ("HalGetInterruptTranslator", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("TRANSLATOR_INTERFACE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ParentInterfaceType", "ParentBusNumber", "BridgeInterfaceType", "Size", "Version", "Translator", "BridgeBusNumber"]), offset=0)), ("HalStartMirroring", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("HalEndMirroring", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["PassNumber"]), offset=0)), ("HalMirrorPhysicalMemory", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PhysicalAddress", "NumberOfBytes"]), offset=0)), ("HalEndOfBoot", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("HalMirrorVerify", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PhysicalAddress", "NumberOfBytes"]), offset=0)), ("HalGetCachedAcpiTable", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Signature", "OemId", "OemTableId"]), offset=0)), ("HalSetPciErrorHandlerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)], SimTypeBottom(label="Void"), arg_names=["Callback"]), offset=0)), ("HalGetPrmCache", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["FirmwareList", "UpdateList"]), offset=0)),)), name="HAL_DISPATCH", pack=False, align=None),
    "HAL_DISPLAY_BIOS_INFORMATION": SimTypeInt(signed=False, label="HAL_DISPLAY_BIOS_INFORMATION"),
    "HAL_DMA_CRASH_DUMP_REGISTER_TYPE": SimTypeInt(signed=False, label="HAL_DMA_CRASH_DUMP_REGISTER_TYPE"),
    "HAL_ERROR_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("InitMaxSize", SimTypeInt(signed=False, label="UInt32")), ("McaMaxSize", SimTypeInt(signed=False, label="UInt32")), ("McaPreviousEventsCount", SimTypeInt(signed=False, label="UInt32")), ("McaCorrectedEventsCount", SimTypeInt(signed=False, label="UInt32")), ("McaKernelDeliveryFails", SimTypeInt(signed=False, label="UInt32")), ("McaDriverDpcQueueFails", SimTypeInt(signed=False, label="UInt32")), ("McaReserved", SimTypeInt(signed=False, label="UInt32")), ("CmcMaxSize", SimTypeInt(signed=False, label="UInt32")), ("CmcPollingInterval", SimTypeInt(signed=False, label="UInt32")), ("CmcInterruptsCount", SimTypeInt(signed=False, label="UInt32")), ("CmcKernelDeliveryFails", SimTypeInt(signed=False, label="UInt32")), ("CmcDriverDpcQueueFails", SimTypeInt(signed=False, label="UInt32")), ("CmcGetStateFails", SimTypeInt(signed=False, label="UInt32")), ("CmcClearStateFails", SimTypeInt(signed=False, label="UInt32")), ("CmcReserved", SimTypeInt(signed=False, label="UInt32")), ("CmcLogId", SimTypeLongLong(signed=False, label="UInt64")), ("CpeMaxSize", SimTypeInt(signed=False, label="UInt32")), ("CpePollingInterval", SimTypeInt(signed=False, label="UInt32")), ("CpeInterruptsCount", SimTypeInt(signed=False, label="UInt32")), ("CpeKernelDeliveryFails", SimTypeInt(signed=False, label="UInt32")), ("CpeDriverDpcQueueFails", SimTypeInt(signed=False, label="UInt32")), ("CpeGetStateFails", SimTypeInt(signed=False, label="UInt32")), ("CpeClearStateFails", SimTypeInt(signed=False, label="UInt32")), ("CpeInterruptSources", SimTypeInt(signed=False, label="UInt32")), ("CpeLogId", SimTypeLongLong(signed=False, label="UInt64")), ("KernelReserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)),)), name="HAL_ERROR_INFO", pack=False, align=None),
    "HAL_MCA_INTERFACE": SimStruct(OrderedDict((("Lock", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Unlock", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ReadRegister", SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["BankNumber", "Exception"]), offset=0)),)), name="HAL_MCA_INTERFACE", pack=False, align=None),
    "HAL_PLATFORM_INFORMATION": SimStruct(OrderedDict((("PlatformFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HAL_PLATFORM_INFORMATION", pack=False, align=None),
    "HAL_POWER_INFORMATION": SimStruct(OrderedDict((("TBD", SimTypeInt(signed=False, label="UInt32")),)), name="HAL_POWER_INFORMATION", pack=False, align=None),
    "HAL_PROCESSOR_FEATURE": SimStruct(OrderedDict((("UsableFeatureBits", SimTypeInt(signed=False, label="UInt32")),)), name="HAL_PROCESSOR_FEATURE", pack=False, align=None),
    "HAL_PROCESSOR_SPEED_INFORMATION": SimStruct(OrderedDict((("ProcessorSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="HAL_PROCESSOR_SPEED_INFORMATION", pack=False, align=None),
    "HAL_QUERY_INFORMATION_CLASS": SimTypeInt(signed=False, label="HAL_QUERY_INFORMATION_CLASS"),
    "HAL_SET_INFORMATION_CLASS": SimTypeInt(signed=False, label="HAL_SET_INFORMATION_CLASS"),
    "HAMSICONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HAMSISESSION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HANDEDNESS": SimTypeInt(signed=False, label="HANDEDNESS"),
    "HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HANDLER_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwControl"]), offset=0),
    "HANDLER_FUNCTION_EX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwControl", "dwEventType", "lpEventData", "lpContext"]), offset=0),
    "HANDLETABLE": SimStruct(OrderedDict((("objectHandle", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="HANDLETABLE", pack=False, align=None),
    "HANDLE_FLAGS": SimTypeInt(signed=False, label="HANDLE_FLAGS"),
    "HANDLE_PTR": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "HANDLE_SDP_TYPE": SimTypeLongLong(signed=False, label="UInt64"),
    "HANG_INFO": SimStruct(OrderedDict((("fAppHangMonitorEnabled", SimTypeInt(signed=True, label="Int32")), ("fTerminateOnHang", SimTypeInt(signed=True, label="Int32")), ("DumpType", SimTypeInt(signed=False, label="DUMPTYPE")), ("dwHangTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwDumpCount", SimTypeInt(signed=False, label="UInt32")), ("dwInfoMsgCount", SimTypeInt(signed=False, label="UInt32")),)), name="HANG_INFO", pack=False, align=None),
    "HARDWAREHOOKSTRUCT": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("message", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HARDWAREHOOKSTRUCT", pack=False, align=None),
    "HARDWAREINPUT": SimStruct(OrderedDict((("uMsg", SimTypeInt(signed=False, label="UInt32")), ("wParamL", SimTypeShort(signed=False, label="UInt16")), ("wParamH", SimTypeShort(signed=False, label="UInt16")),)), name="HARDWAREINPUT", pack=False, align=None),
    "HARDWARE_ADDRESS": SimStruct(OrderedDict((("Address", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="HARDWARE_ADDRESS", pack=False, align=None),
    "HARDWARE_COUNTER": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HARDWARE_COUNTER_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Index", SimTypeLongLong(signed=False, label="UInt64")),)), name="HARDWARE_COUNTER", pack=False, align=None),
    "HARDWARE_COUNTER_DATA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HARDWARE_COUNTER_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="HARDWARE_COUNTER_DATA", pack=False, align=None),
    "HARDWARE_COUNTER_TYPE": SimTypeInt(signed=False, label="HARDWARE_COUNTER_TYPE"),
    "HASHALGORITHM_ENUM": SimTypeInt(signed=False, label="HASHALGORITHM_ENUM"),
    "HATCH_BRUSH_STYLE": SimTypeInt(signed=False, label="HATCH_BRUSH_STYLE"),
    "HBITMAP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HBLUETOOTH_DEVICE_FIND": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HBLUETOOTH_RADIO_FIND": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HBM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HBRUSH": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCD_ISO_STAT_COUNTERS": SimStruct(OrderedDict((("LateUrbs", SimTypeShort(signed=False, label="UInt16")), ("DoubleBufferedPackets", SimTypeShort(signed=False, label="UInt16")), ("TransfersCF_5ms", SimTypeShort(signed=False, label="UInt16")), ("TransfersCF_2ms", SimTypeShort(signed=False, label="UInt16")), ("TransfersCF_1ms", SimTypeShort(signed=False, label="UInt16")), ("MaxInterruptLatency", SimTypeShort(signed=False, label="UInt16")), ("BadStartFrame", SimTypeShort(signed=False, label="UInt16")), ("StaleUrbs", SimTypeShort(signed=False, label="UInt16")), ("IsoPacketNotAccesed", SimTypeShort(signed=False, label="UInt16")), ("IsoPacketHWError", SimTypeShort(signed=False, label="UInt16")), ("SmallestUrbPacketCount", SimTypeShort(signed=False, label="UInt16")), ("LargestUrbPacketCount", SimTypeShort(signed=False, label="UInt16")), ("IsoCRC_Error", SimTypeShort(signed=False, label="UInt16")), ("IsoOVERRUN_Error", SimTypeShort(signed=False, label="UInt16")), ("IsoINTERNAL_Error", SimTypeShort(signed=False, label="UInt16")), ("IsoUNKNOWN_Error", SimTypeShort(signed=False, label="UInt16")), ("IsoBytesTransferred", SimTypeInt(signed=False, label="UInt32")), ("LateMissedCount", SimTypeShort(signed=False, label="UInt16")), ("HWIsoMissedCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved7", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="HCD_ISO_STAT_COUNTERS", pack=False, align=None),
    "HCD_STAT_COUNTERS": SimStruct(OrderedDict((("BytesTransferred", SimTypeInt(signed=False, label="UInt32")), ("IsoMissedCount", SimTypeShort(signed=False, label="UInt16")), ("DataOverrunErrorCount", SimTypeShort(signed=False, label="UInt16")), ("CrcErrorCount", SimTypeShort(signed=False, label="UInt16")), ("ScheduleOverrunCount", SimTypeShort(signed=False, label="UInt16")), ("TimeoutErrorCount", SimTypeShort(signed=False, label="UInt16")), ("InternalHcErrorCount", SimTypeShort(signed=False, label="UInt16")), ("BufferOverrunErrorCount", SimTypeShort(signed=False, label="UInt16")), ("SWErrorCount", SimTypeShort(signed=False, label="UInt16")), ("StallPidCount", SimTypeShort(signed=False, label="UInt16")), ("PortDisableCount", SimTypeShort(signed=False, label="UInt16")),)), name="HCD_STAT_COUNTERS", pack=False, align=None),
    "HCD_STAT_INFORMATION_1": SimStruct(OrderedDict((("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("ResetCounters", SimTypeInt(signed=False, label="UInt32")), ("TimeRead", SimTypeLongLong(signed=True, label="Int64")), ("Counters", SimTypeRef("HCD_STAT_COUNTERS", SimStruct)),)), name="HCD_STAT_INFORMATION_1", pack=False, align=None),
    "HCD_STAT_INFORMATION_2": SimStruct(OrderedDict((("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("ResetCounters", SimTypeInt(signed=False, label="UInt32")), ("TimeRead", SimTypeLongLong(signed=True, label="Int64")), ("LockedMemoryUsed", SimTypeInt(signed=True, label="Int32")), ("Counters", SimTypeRef("HCD_STAT_COUNTERS", SimStruct)), ("IsoCounters", SimTypeRef("HCD_ISO_STAT_COUNTERS", SimStruct)),)), name="HCD_STAT_INFORMATION_2", pack=False, align=None),
    "HCERTCHAINENGINE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCERTSTORE": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "HCERTSTOREPROV": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "HCHANGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCLUSCRYPTPROVIDER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCLUSENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCLUSENUMEX": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCLUSTER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCMNOTIFICATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCN_NOTIFICATIONS": SimTypeInt(signed=False, label="HCN_NOTIFICATIONS"),
    "HCN_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["NotificationType", "Context", "NotificationStatus", "NotificationData"]), offset=0),
    "HCN_PORT_ACCESS": SimTypeInt(signed=False, label="HCN_PORT_ACCESS"),
    "HCN_PORT_PROTOCOL": SimTypeInt(signed=False, label="HCN_PORT_PROTOCOL"),
    "HCN_PORT_RANGE_ENTRY": SimStruct(OrderedDict((("OwningPartitionId", SimTypeBottom(label="Guid")), ("TargetPartitionId", SimTypeBottom(label="Guid")), ("Protocol", SimTypeInt(signed=False, label="HCN_PORT_PROTOCOL")), ("Priority", SimTypeLongLong(signed=False, label="UInt64")), ("ReservationType", SimTypeInt(signed=False, label="UInt32")), ("SharingFlags", SimTypeInt(signed=False, label="UInt32")), ("DeliveryMode", SimTypeInt(signed=False, label="UInt32")), ("StartingPort", SimTypeShort(signed=False, label="UInt16")), ("EndingPort", SimTypeShort(signed=False, label="UInt16")),)), name="HCN_PORT_RANGE_ENTRY", pack=False, align=None),
    "HCN_PORT_RANGE_RESERVATION": SimStruct(OrderedDict((("startingPort", SimTypeShort(signed=False, label="UInt16")), ("endingPort", SimTypeShort(signed=False, label="UInt16")),)), name="HCN_PORT_RANGE_RESERVATION", pack=False, align=None),
    "HCOLORSPACE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCOMDB": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCONN": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "HCONV": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCONVLIST": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCRYPTASYNC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCRYPTPROV_LEGACY": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "HCRYPTPROV_OR_NCRYPT_KEY_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "HCS_CALLBACK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCS_CREATE_OPTIONS": SimTypeInt(signed=False, label="HCS_CREATE_OPTIONS"),
    "HCS_CREATE_OPTIONS_1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="HCS_CREATE_OPTIONS")), ("UserToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeRef("SECURITY_DESCRIPTOR", SimStruct), offset=0)), ("CallbackOptions", SimTypeInt(signed=False, label="HCS_EVENT_OPTIONS")), ("CallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("HCS_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["event", "context"]), offset=0)),)), name="HCS_CREATE_OPTIONS_1", pack=False, align=None),
    "HCS_EVENT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HCS_EVENT_TYPE")), ("EventData", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Operation", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HCS_EVENT", pack=False, align=None),
    "HCS_EVENT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("HCS_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["event", "context"]), offset=0),
    "HCS_EVENT_OPTIONS": SimTypeInt(signed=False, label="HCS_EVENT_OPTIONS"),
    "HCS_EVENT_TYPE": SimTypeInt(signed=False, label="HCS_EVENT_TYPE"),
    "HCS_NOTIFICATIONS": SimTypeInt(signed=False, label="HCS_NOTIFICATIONS"),
    "HCS_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["notificationType", "context", "notificationStatus", "notificationData"]), offset=0),
    "HCS_NOTIFICATION_FLAGS": SimTypeInt(signed=False, label="HCS_NOTIFICATION_FLAGS"),
    "HCS_OPERATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCS_OPERATION_COMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "context"]), offset=0),
    "HCS_OPERATION_OPTIONS": SimTypeInt(signed=False, label="HCS_OPERATION_OPTIONS"),
    "HCS_OPERATION_TYPE": SimTypeInt(signed=False, label="HCS_OPERATION_TYPE"),
    "HCS_PROCESS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCS_PROCESS_INFORMATION": SimStruct(OrderedDict((("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("StdInput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("StdOutput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("StdError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HCS_PROCESS_INFORMATION", pack=False, align=None),
    "HCS_RESOURCE_TYPE": SimTypeInt(signed=False, label="HCS_RESOURCE_TYPE"),
    "HCS_SYSTEM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HCURSOR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDC_MAP_MODE": SimTypeInt(signed=False, label="HDC_MAP_MODE"),
    "HDDEDATA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDESK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDEV": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDEVINFO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDEVNOTIFY": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "HDEVQUERY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDHITTESTINFO": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("flags", SimTypeInt(signed=False, label="HEADER_HITTEST_INFO_FLAGS")), ("iItem", SimTypeInt(signed=True, label="Int32")),)), name="HDHITTESTINFO", pack=False, align=None),
    "HDIAGNOSTIC_DATA_QUERY_SESSION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDIAGNOSTIC_EVENT_CATEGORY_DESCRIPTION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDIAGNOSTIC_EVENT_PRODUCER_DESCRIPTION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDIAGNOSTIC_EVENT_TAG_DESCRIPTION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDIAGNOSTIC_RECORD": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDIAGNOSTIC_REPORT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDITEMA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="HDI_MASK")), ("cxy", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hbm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("fmt", SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_FLAGS")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iOrder", SimTypeInt(signed=True, label="Int32")), ("type", SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_TYPE")), ("pvFilter", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("state", SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_STATE")),)), name="HDITEMA", pack=False, align=None),
    "HDITEMW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="HDI_MASK")), ("cxy", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hbm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("fmt", SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_FLAGS")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iOrder", SimTypeInt(signed=True, label="Int32")), ("type", SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_TYPE")), ("pvFilter", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("state", SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_STATE")),)), name="HDITEMW", pack=False, align=None),
    "HDI_MASK": SimTypeInt(signed=False, label="HDI_MASK"),
    "HDLAYOUT": SimStruct(OrderedDict((("prc", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("pwpos", SimTypePointer(SimTypeRef("WINDOWPOS", SimStruct), offset=0)),)), name="HDLAYOUT", pack=False, align=None),
    "HDPA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDROP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVCALL": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVDIALOGINSTANCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVLINE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVMSPLINE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVOBJ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVPHONE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDRVR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDSA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HDV_DEVICE_HOST_FLAGS": SimTypeInt(signed=False, label="HDV_DEVICE_HOST_FLAGS"),
    "HDV_DEVICE_TYPE": SimTypeInt(signed=False, label="HDV_DEVICE_TYPE"),
    "HDV_DOORBELL_FLAGS": SimTypeInt(signed=False, label="HDV_DOORBELL_FLAGS"),
    "HDV_MMIO_MAPPING_FLAGS": SimTypeInt(signed=False, label="HDV_MMIO_MAPPING_FLAGS"),
    "HDV_PCI_BAR_SELECTOR": SimTypeInt(signed=False, label="HDV_PCI_BAR_SELECTOR"),
    "HDV_PCI_DEVICE_GET_DETAILS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("HDV_PCI_PNP_ID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "pnpId", "probedBarsCount", "probedBars"]), offset=0),
    "HDV_PCI_DEVICE_INITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext"]), offset=0),
    "HDV_PCI_DEVICE_INTERFACE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="HDV_PCI_INTERFACE_VERSION")), ("Initialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext"]), offset=0)), ("Teardown", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["deviceContext"]), offset=0)), ("SetConfiguration", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "configurationValueCount", "configurationValues"]), offset=0)), ("GetDetails", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("HDV_PCI_PNP_ID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "pnpId", "probedBarsCount", "probedBars"]), offset=0)), ("Start", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext"]), offset=0)), ("Stop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["deviceContext"]), offset=0)), ("ReadConfigSpace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "offset", "value"]), offset=0)), ("WriteConfigSpace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "offset", "value"]), offset=0)), ("ReadInterceptedMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="HDV_PCI_BAR_SELECTOR"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "barIndex", "offset", "length", "value"]), offset=0)), ("WriteInterceptedMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="HDV_PCI_BAR_SELECTOR"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "barIndex", "offset", "length", "value"]), offset=0)),)), name="HDV_PCI_DEVICE_INTERFACE", pack=False, align=None),
    "HDV_PCI_DEVICE_SET_CONFIGURATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "configurationValueCount", "configurationValues"]), offset=0),
    "HDV_PCI_DEVICE_START": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext"]), offset=0),
    "HDV_PCI_DEVICE_STOP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["deviceContext"]), offset=0),
    "HDV_PCI_DEVICE_TEARDOWN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["deviceContext"]), offset=0),
    "HDV_PCI_INTERFACE_VERSION": SimTypeInt(signed=False, label="HDV_PCI_INTERFACE_VERSION"),
    "HDV_PCI_PNP_ID": SimStruct(OrderedDict((("VendorID", SimTypeShort(signed=False, label="UInt16")), ("DeviceID", SimTypeShort(signed=False, label="UInt16")), ("RevisionID", SimTypeChar(label="Byte")), ("ProgIf", SimTypeChar(label="Byte")), ("SubClass", SimTypeChar(label="Byte")), ("BaseClass", SimTypeChar(label="Byte")), ("SubVendorID", SimTypeShort(signed=False, label="UInt16")), ("SubSystemID", SimTypeShort(signed=False, label="UInt16")),)), name="HDV_PCI_PNP_ID", pack=False, align=None),
    "HDV_PCI_READ_CONFIG_SPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "offset", "value"]), offset=0),
    "HDV_PCI_READ_INTERCEPTED_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="HDV_PCI_BAR_SELECTOR"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "barIndex", "offset", "length", "value"]), offset=0),
    "HDV_PCI_WRITE_CONFIG_SPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "offset", "value"]), offset=0),
    "HDV_PCI_WRITE_INTERCEPTED_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="HDV_PCI_BAR_SELECTOR"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["deviceContext", "barIndex", "offset", "length", "value"]), offset=0),
    "HDWP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HD_TEXTFILTERA": SimStruct(OrderedDict((("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")),)), name="HD_TEXTFILTERA", pack=False, align=None),
    "HD_TEXTFILTERW": SimStruct(OrderedDict((("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")),)), name="HD_TEXTFILTERW", pack=False, align=None),
    "HEAACWAVEFORMAT": SimStruct(OrderedDict((("wfInfo", SimTypeRef("HEAACWAVEINFO", SimStruct)), ("pbAudioSpecificConfig", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HEAACWAVEFORMAT", pack=False, align=None),
    "HEAACWAVEINFO": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wPayloadType", SimTypeShort(signed=False, label="UInt16")), ("wAudioProfileLevelIndication", SimTypeShort(signed=False, label="UInt16")), ("wStructType", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="HEAACWAVEINFO", pack=False, align=None),
    "HEADERAREASTATES": SimTypeInt(signed=False, label="HEADERAREASTATES"),
    "HEADERCLOSESTATES": SimTypeInt(signed=False, label="HEADERCLOSESTATES"),
    "HEADERDROPDOWNFILTERSTATES": SimTypeInt(signed=False, label="HEADERDROPDOWNFILTERSTATES"),
    "HEADERDROPDOWNSTATES": SimTypeInt(signed=False, label="HEADERDROPDOWNSTATES"),
    "HEADERITEMLEFTSTATES": SimTypeInt(signed=False, label="HEADERITEMLEFTSTATES"),
    "HEADERITEMRIGHTSTATES": SimTypeInt(signed=False, label="HEADERITEMRIGHTSTATES"),
    "HEADERITEMSTATES": SimTypeInt(signed=False, label="HEADERITEMSTATES"),
    "HEADEROVERFLOWSTATES": SimTypeInt(signed=False, label="HEADEROVERFLOWSTATES"),
    "HEADERPARTS": SimTypeInt(signed=False, label="HEADERPARTS"),
    "HEADERPINSTATES": SimTypeInt(signed=False, label="HEADERPINSTATES"),
    "HEADERSORTARROWSTATES": SimTypeInt(signed=False, label="HEADERSORTARROWSTATES"),
    "HEADERSTYLESTATES": SimTypeInt(signed=False, label="HEADERSTYLESTATES"),
    "HEADER_CONTROL_FORMAT_FLAGS": SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_FLAGS"),
    "HEADER_CONTROL_FORMAT_STATE": SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_STATE"),
    "HEADER_CONTROL_FORMAT_TYPE": SimTypeInt(signed=False, label="HEADER_CONTROL_FORMAT_TYPE"),
    "HEADER_CONTROL_NOTIFICATION_BUTTON": SimTypeInt(signed=False, label="HEADER_CONTROL_NOTIFICATION_BUTTON"),
    "HEADER_HITTEST_INFO_FLAGS": SimTypeInt(signed=False, label="HEADER_HITTEST_INFO_FLAGS"),
    "HEAPALIAS": SimStruct(OrderedDict((("fpVidMem", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpAlias", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwAliasSize", SimTypeInt(signed=False, label="UInt32")),)), name="HEAPALIAS", pack=False, align=None),
    "HEAPALIASINFO": SimStruct(OrderedDict((("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumHeaps", SimTypeInt(signed=False, label="UInt32")), ("lpAliases", SimTypePointer(SimTypeRef("HEAPALIAS", SimStruct), offset=0)),)), name="HEAPALIASINFO", pack=False, align=None),
    "HEAPALIGNMENT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("ExecuteBuffer", SimTypeRef("SURFACEALIGNMENT", SimStruct)), ("Overlay", SimTypeRef("SURFACEALIGNMENT", SimStruct)), ("Texture", SimTypeRef("SURFACEALIGNMENT", SimStruct)), ("ZBuffer", SimTypeRef("SURFACEALIGNMENT", SimStruct)), ("AlphaBuffer", SimTypeRef("SURFACEALIGNMENT", SimStruct)), ("Offscreen", SimTypeRef("SURFACEALIGNMENT", SimStruct)), ("FlipTarget", SimTypeRef("SURFACEALIGNMENT", SimStruct)),)), name="HEAPALIGNMENT", pack=False, align=None),
    "HEAPENTRY32": SimStruct(OrderedDict((("dwSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwBlockSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="HEAPENTRY32_FLAGS")), ("dwLockCount", SimTypeInt(signed=False, label="UInt32")), ("dwResvd", SimTypeInt(signed=False, label="UInt32")), ("th32ProcessID", SimTypeInt(signed=False, label="UInt32")), ("th32HeapID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="HEAPENTRY32", pack=False, align=None),
    "HEAPENTRY32_FLAGS": SimTypeInt(signed=False, label="HEAPENTRY32_FLAGS"),
    "HEAPLIST32": SimStruct(OrderedDict((("dwSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("th32ProcessID", SimTypeInt(signed=False, label="UInt32")), ("th32HeapID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HEAPLIST32", pack=False, align=None),
    "HEAP_FLAGS": SimTypeInt(signed=False, label="HEAP_FLAGS"),
    "HEAP_INFORMATION_CLASS": SimTypeInt(signed=False, label="HEAP_INFORMATION_CLASS"),
    "HEAP_MEMORY_INFO_CLASS": SimTypeInt(signed=False, label="HEAP_MEMORY_INFO_CLASS"),
    "HEAP_OPTIMIZE_RESOURCES_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="HEAP_OPTIMIZE_RESOURCES_INFORMATION", pack=False, align=None),
    "HEAP_SUMMARY": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("cbAllocated", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cbCommitted", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cbReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cbMaxReserve", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="HEAP_SUMMARY", pack=False, align=None),
    "HELPBUTTONSTATES": SimTypeInt(signed=False, label="HELPBUTTONSTATES"),
    "HELPER_ATTRIBUTE": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("type", SimTypeInt(signed=False, label="ATTRIBUTE_TYPE")), ("Anonymous", SimUnion({"Boolean": SimTypeInt(signed=True, label="Int32"), "Char": SimTypeChar(label="Byte"), "Byte": SimTypeChar(label="Byte"), "Short": SimTypeShort(signed=True, label="Int16"), "Word": SimTypeShort(signed=False, label="UInt16"), "Int": SimTypeInt(signed=True, label="Int32"), "DWord": SimTypeInt(signed=False, label="UInt32"), "Int64": SimTypeLongLong(signed=True, label="Int64"), "UInt64": SimTypeLongLong(signed=False, label="UInt64"), "PWStr": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Guid": SimTypeBottom(label="Guid"), "LifeTime": SimTypeRef("LIFE_TIME", SimStruct), "Address": SimTypeRef("DIAG_SOCKADDR", SimStruct), "OctetString": SimTypeRef("OCTET_STRING", SimStruct)}, name="<anon>", label="None")),)), name="HELPER_ATTRIBUTE", pack=False, align=None),
    "HELPINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iContextType", SimTypeInt(signed=False, label="HELP_INFO_TYPE")), ("iCtrlId", SimTypeInt(signed=True, label="Int32")), ("hItemHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwContextId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MousePos", SimTypeRef("POINT", SimStruct)),)), name="HELPINFO", pack=False, align=None),
    "HELPLINKSTATES": SimTypeInt(signed=False, label="HELPLINKSTATES"),
    "HELPWININFOA": SimStruct(OrderedDict((("wStructSize", SimTypeInt(signed=True, label="Int32")), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("dx", SimTypeInt(signed=True, label="Int32")), ("dy", SimTypeInt(signed=True, label="Int32")), ("wMax", SimTypeInt(signed=True, label="Int32")), ("rgchMember", SimTypeArray(SimTypeChar(label="SByte"), 2)),)), name="HELPWININFOA", pack=False, align=None),
    "HELPWININFOW": SimStruct(OrderedDict((("wStructSize", SimTypeInt(signed=True, label="Int32")), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("dx", SimTypeInt(signed=True, label="Int32")), ("dy", SimTypeInt(signed=True, label="Int32")), ("wMax", SimTypeInt(signed=True, label="Int32")), ("rgchMember", SimTypeArray(SimTypeChar(label="Char"), 2)),)), name="HELPWININFOW", pack=False, align=None),
    "HELP_INFO_TYPE": SimTypeInt(signed=False, label="HELP_INFO_TYPE"),
    "HENHMETAFILE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HFASTMUTEX": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HFILTER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HFILTER_INSTANCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HFONT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGDIOBJ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGESTUREINFO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGLOBAL": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "HGLRC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGROUP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGROUPENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGROUPENUMEX": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGROUPSET": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HGROUPSETENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HHNTRACK": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pszCurUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("idAction", SimTypeInt(signed=True, label="Int32")), ("phhWinType", SimTypePointer(SimTypeRef("HH_WINTYPE", SimStruct), offset=0)),)), name="HHNTRACK", pack=False, align=None),
    "HHN_NOTIFY": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pszUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HHN_NOTIFY", pack=False, align=None),
    "HHOOK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HH_AKLINK": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("fReserved", SimTypeInt(signed=True, label="Int32")), ("pszKeywords", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszUrl", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszMsgText", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszMsgTitle", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszWindow", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("fIndexOnFail", SimTypeInt(signed=True, label="Int32")),)), name="HH_AKLINK", pack=False, align=None),
    "HH_ENUM_CAT": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("pszCatName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszCatDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HH_ENUM_CAT", pack=False, align=None),
    "HH_ENUM_IT": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("iType", SimTypeInt(signed=True, label="Int32")), ("pszCatName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszITName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszITDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HH_ENUM_IT", pack=False, align=None),
    "HH_FTS_QUERY": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("fUniCodeStrings", SimTypeInt(signed=True, label="Int32")), ("pszSearchQuery", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("iProximity", SimTypeInt(signed=True, label="Int32")), ("fStemmedSearch", SimTypeInt(signed=True, label="Int32")), ("fTitleOnly", SimTypeInt(signed=True, label="Int32")), ("fExecute", SimTypeInt(signed=True, label="Int32")), ("pszWindow", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="HH_FTS_QUERY", pack=False, align=None),
    "HH_GLOBAL_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="HH_GPROPID")), ("var", SimTypeRef("VARIANT", SimStruct)),)), name="HH_GLOBAL_PROPERTY", pack=False, align=None),
    "HH_GPROPID": SimTypeInt(signed=False, label="HH_GPROPID"),
    "HH_POPUP": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("hinst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("idString", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pt", SimTypeRef("POINT", SimStruct)), ("clrForeground", SimTypeInt(signed=False, label="UInt32")), ("clrBackground", SimTypeInt(signed=False, label="UInt32")), ("rcMargins", SimTypeRef("RECT", SimStruct)), ("pszFont", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="HH_POPUP", pack=False, align=None),
    "HH_SET_INFOTYPE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("pszCatName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszInfoTypeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HH_SET_INFOTYPE", pack=False, align=None),
    "HH_WINTYPE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=True, label="Int32")), ("fUniCodeStrings", SimTypeInt(signed=True, label="Int32")), ("pszType", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("fsValidMembers", SimTypeInt(signed=False, label="UInt32")), ("fsWinProperties", SimTypeInt(signed=False, label="UInt32")), ("pszCaption", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("dwStyles", SimTypeInt(signed=False, label="UInt32")), ("dwExStyles", SimTypeInt(signed=False, label="UInt32")), ("rcWindowPos", SimTypeRef("RECT", SimStruct)), ("nShowState", SimTypeInt(signed=True, label="Int32")), ("hwndHelp", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndCaller", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("paInfoTypes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("hwndToolBar", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndNavigation", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndHTML", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iNavWidth", SimTypeInt(signed=True, label="Int32")), ("rcHTML", SimTypeRef("RECT", SimStruct)), ("pszToc", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszIndex", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszHome", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("fsToolBarFlags", SimTypeInt(signed=False, label="UInt32")), ("fNotExpanded", SimTypeInt(signed=True, label="Int32")), ("curNavType", SimTypeInt(signed=True, label="Int32")), ("tabpos", SimTypeInt(signed=True, label="Int32")), ("idNotify", SimTypeInt(signed=True, label="Int32")), ("tabOrder", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("cHistory", SimTypeInt(signed=True, label="Int32")), ("pszJump1", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszJump2", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszUrlJump1", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pszUrlJump2", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("rcMinSize", SimTypeRef("RECT", SimStruct)), ("cbInfoTypes", SimTypeInt(signed=True, label="Int32")), ("pszCustomTabs", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="HH_WINTYPE", pack=False, align=None),
    "HIBERFILE_BUCKET": SimStruct(OrderedDict((("MaxPhysicalMemory", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalMemoryPercent", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="HIBERFILE_BUCKET", pack=False, align=None),
    "HIBERFILE_BUCKET_SIZE": SimTypeInt(signed=False, label="HIBERFILE_BUCKET_SIZE"),
    "HIC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HICON": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HIDD_ATTRIBUTES": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("VendorID", SimTypeShort(signed=False, label="UInt16")), ("ProductID", SimTypeShort(signed=False, label="UInt16")), ("VersionNumber", SimTypeShort(signed=False, label="UInt16")),)), name="HIDD_ATTRIBUTES", pack=False, align=None),
    "HIDD_CONFIGURATION": SimStruct(OrderedDict((("cookie", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")), ("RingBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="HIDD_CONFIGURATION", pack=False, align=None),
    "HIDP_BUTTON_ARRAY_DATA": SimStruct(OrderedDict((("ArrayIndex", SimTypeShort(signed=False, label="UInt16")), ("On", SimTypeChar(label="Byte")),)), name="HIDP_BUTTON_ARRAY_DATA", pack=False, align=None),
    "HIDP_BUTTON_CAPS": SimStruct(OrderedDict((("UsagePage", SimTypeShort(signed=False, label="UInt16")), ("ReportID", SimTypeChar(label="Byte")), ("IsAlias", SimTypeChar(label="Byte")), ("BitField", SimTypeShort(signed=False, label="UInt16")), ("LinkCollection", SimTypeShort(signed=False, label="UInt16")), ("LinkUsage", SimTypeShort(signed=False, label="UInt16")), ("LinkUsagePage", SimTypeShort(signed=False, label="UInt16")), ("IsRange", SimTypeChar(label="Byte")), ("IsStringRange", SimTypeChar(label="Byte")), ("IsDesignatorRange", SimTypeChar(label="Byte")), ("IsAbsolute", SimTypeChar(label="Byte")), ("ReportCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 9)), ("Anonymous", SimUnion({"Range": SimTypeRef("_Range_e__Struct", SimStruct), "NotRange": SimTypeRef("_NotRange_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="HIDP_BUTTON_CAPS", pack=False, align=None),
    "HIDP_CAPS": SimStruct(OrderedDict((("Usage", SimTypeShort(signed=False, label="UInt16")), ("UsagePage", SimTypeShort(signed=False, label="UInt16")), ("InputReportByteLength", SimTypeShort(signed=False, label="UInt16")), ("OutputReportByteLength", SimTypeShort(signed=False, label="UInt16")), ("FeatureReportByteLength", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 17)), ("NumberLinkCollectionNodes", SimTypeShort(signed=False, label="UInt16")), ("NumberInputButtonCaps", SimTypeShort(signed=False, label="UInt16")), ("NumberInputValueCaps", SimTypeShort(signed=False, label="UInt16")), ("NumberInputDataIndices", SimTypeShort(signed=False, label="UInt16")), ("NumberOutputButtonCaps", SimTypeShort(signed=False, label="UInt16")), ("NumberOutputValueCaps", SimTypeShort(signed=False, label="UInt16")), ("NumberOutputDataIndices", SimTypeShort(signed=False, label="UInt16")), ("NumberFeatureButtonCaps", SimTypeShort(signed=False, label="UInt16")), ("NumberFeatureValueCaps", SimTypeShort(signed=False, label="UInt16")), ("NumberFeatureDataIndices", SimTypeShort(signed=False, label="UInt16")),)), name="HIDP_CAPS", pack=False, align=None),
    "HIDP_DATA": SimStruct(OrderedDict((("DataIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"RawValue": SimTypeInt(signed=False, label="UInt32"), "On": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="HIDP_DATA", pack=False, align=None),
    "HIDP_EXTENDED_ATTRIBUTES": SimStruct(OrderedDict((("NumGlobalUnknowns", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("GlobalUnknowns", SimTypePointer(SimTypeRef("HIDP_UNKNOWN_TOKEN", SimStruct), offset=0)), ("Data", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="HIDP_EXTENDED_ATTRIBUTES", pack=False, align=None),
    "HIDP_KEYBOARD_DIRECTION": SimTypeInt(signed=False, label="HIDP_KEYBOARD_DIRECTION"),
    "HIDP_KEYBOARD_MODIFIER_STATE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ul": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="HIDP_KEYBOARD_MODIFIER_STATE", pack=False, align=None),
    "HIDP_LINK_COLLECTION_NODE": SimStruct(OrderedDict((("LinkUsage", SimTypeShort(signed=False, label="UInt16")), ("LinkUsagePage", SimTypeShort(signed=False, label="UInt16")), ("Parent", SimTypeShort(signed=False, label="UInt16")), ("NumberOfChildren", SimTypeShort(signed=False, label="UInt16")), ("NextSibling", SimTypeShort(signed=False, label="UInt16")), ("FirstChild", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("UserContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HIDP_LINK_COLLECTION_NODE", pack=False, align=None),
    "HIDP_REPORT_TYPE": SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"),
    "HIDP_UNKNOWN_TOKEN": SimStruct(OrderedDict((("Token", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("BitField", SimTypeInt(signed=False, label="UInt32")),)), name="HIDP_UNKNOWN_TOKEN", pack=False, align=None),
    "HIDP_VALUE_CAPS": SimStruct(OrderedDict((("UsagePage", SimTypeShort(signed=False, label="UInt16")), ("ReportID", SimTypeChar(label="Byte")), ("IsAlias", SimTypeChar(label="Byte")), ("BitField", SimTypeShort(signed=False, label="UInt16")), ("LinkCollection", SimTypeShort(signed=False, label="UInt16")), ("LinkUsage", SimTypeShort(signed=False, label="UInt16")), ("LinkUsagePage", SimTypeShort(signed=False, label="UInt16")), ("IsRange", SimTypeChar(label="Byte")), ("IsStringRange", SimTypeChar(label="Byte")), ("IsDesignatorRange", SimTypeChar(label="Byte")), ("IsAbsolute", SimTypeChar(label="Byte")), ("HasNull", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("BitSize", SimTypeShort(signed=False, label="UInt16")), ("ReportCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 5)), ("UnitsExp", SimTypeInt(signed=False, label="UInt32")), ("Units", SimTypeInt(signed=False, label="UInt32")), ("LogicalMin", SimTypeInt(signed=True, label="Int32")), ("LogicalMax", SimTypeInt(signed=True, label="Int32")), ("PhysicalMin", SimTypeInt(signed=True, label="Int32")), ("PhysicalMax", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"Range": SimTypeRef("_Range_e__Struct", SimStruct), "NotRange": SimTypeRef("_NotRange_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="HIDP_VALUE_CAPS", pack=False, align=None),
    "HID_COLLECTION_INFORMATION": SimStruct(OrderedDict((("DescriptorSize", SimTypeInt(signed=False, label="UInt32")), ("Polled", SimTypeChar(label="Byte")), ("Reserved1", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("VendorID", SimTypeShort(signed=False, label="UInt16")), ("ProductID", SimTypeShort(signed=False, label="UInt16")), ("VersionNumber", SimTypeShort(signed=False, label="UInt16")),)), name="HID_COLLECTION_INFORMATION", pack=False, align=None),
    "HID_DRIVER_CONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("RingBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="HID_DRIVER_CONFIG", pack=False, align=None),
    "HID_XFER_PACKET": SimStruct(OrderedDict((("reportBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("reportBufferLen", SimTypeInt(signed=False, label="UInt32")), ("reportId", SimTypeChar(label="Byte")),)), name="HID_XFER_PACKET", pack=False, align=None),
    "HIFTIMESTAMPCHANGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HIGHCONTRASTA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="HIGHCONTRASTW_FLAGS")), ("lpszDefaultScheme", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HIGHCONTRASTA", pack=False, align=None),
    "HIGHCONTRASTW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="HIGHCONTRASTW_FLAGS")), ("lpszDefaultScheme", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HIGHCONTRASTW", pack=False, align=None),
    "HIGHCONTRASTW_FLAGS": SimTypeInt(signed=False, label="HIGHCONTRASTW_FLAGS"),
    "HIMAGELIST": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HIMC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HIMCC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HINSTANCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HINTERACTIONCONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HIORING": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HISTOGRAM_BUCKET": SimStruct(OrderedDict((("Reads", SimTypeInt(signed=False, label="UInt32")), ("Writes", SimTypeInt(signed=False, label="UInt32")),)), name="HISTOGRAM_BUCKET", pack=False, align=None),
    "HITRANGE": SimStruct(OrderedDict((("iPosition", SimTypeInt(signed=False, label="UInt32")), ("cLength", SimTypeInt(signed=False, label="UInt32")),)), name="HITRANGE", pack=False, align=None),
    "HITRESULT": SimTypeInt(signed=False, label="HITRESULT"),
    "HIT_LOGGING_INFO": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszLoggedUrlName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StartTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("EndTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("lpszExtendedInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HIT_LOGGING_INFO", pack=False, align=None),
    "HIT_TEST_BACKGROUND_OPTIONS": SimTypeInt(signed=False, label="HIT_TEST_BACKGROUND_OPTIONS"),
    "HKEY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HKL": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HLBWIF_FLAGS": SimTypeInt(signed=False, label="HLBWIF_FLAGS"),
    "HLBWINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("grfHLBWIF", SimTypeInt(signed=False, label="UInt32")), ("rcFramePos", SimTypeRef("RECT", SimStruct)), ("rcDocPos", SimTypeRef("RECT", SimStruct)), ("hltbinfo", SimTypeRef("HLTBINFO", SimStruct)),)), name="HLBWINFO", pack=False, align=None),
    "HLFNAMEF": SimTypeInt(signed=False, label="HLFNAMEF"),
    "HLID_INFO": SimTypeInt(signed=False, label="HLID_INFO"),
    "HLINKGETREF": SimTypeInt(signed=False, label="HLINKGETREF"),
    "HLINKMISC": SimTypeInt(signed=False, label="HLINKMISC"),
    "HLINKSETF": SimTypeInt(signed=False, label="HLINKSETF"),
    "HLINKWHICHMK": SimTypeInt(signed=False, label="HLINKWHICHMK"),
    "HLITEM": SimStruct(OrderedDict((("uHLID", SimTypeInt(signed=False, label="UInt32")), ("pwzFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HLITEM", pack=False, align=None),
    "HLNF": SimTypeInt(signed=False, label="HLNF"),
    "HLOCAL": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "HLOG": SimStruct(OrderedDict((("time", SimTypeInt(signed=False, label="UInt32")), ("last_flags", SimTypeInt(signed=False, label="UInt32")), ("offset", SimTypeInt(signed=False, label="UInt32")), ("rec_offset", SimTypeInt(signed=False, label="UInt32")),)), name="HLOG", pack=False, align=None),
    "HLQF_INFO": SimTypeInt(signed=False, label="HLQF_INFO"),
    "HLSHORTCUTF": SimTypeInt(signed=False, label="HLSHORTCUTF"),
    "HLSR": SimTypeInt(signed=False, label="HLSR"),
    "HLSURF": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HLTBINFO": SimStruct(OrderedDict((("uDockType", SimTypeInt(signed=False, label="UInt32")), ("rcTbPos", SimTypeRef("RECT", SimStruct)),)), name="HLTBINFO", pack=False, align=None),
    "HLTB_INFO": SimTypeInt(signed=False, label="HLTB_INFO"),
    "HLTRANSLATEF": SimTypeInt(signed=False, label="HLTRANSLATEF"),
    "HMAC_INFO": SimStruct(OrderedDict((("HashAlgid", SimTypeInt(signed=False, label="ALG_ID")), ("pbInnerString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbInnerString", SimTypeInt(signed=False, label="UInt32")), ("pbOuterString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbOuterString", SimTypeInt(signed=False, label="UInt32")),)), name="HMAC_INFO", pack=False, align=None),
    "HMENU": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMETAFILE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMIDI": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMIDIIN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMIDIOUT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMIDISTRM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMIXER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMIXEROBJ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMMIO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMODULE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HMONITOR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNETINTERFACE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNETINTERFACEENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNETWORK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNETWORKENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNODE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNODEENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HNODEENUMEX": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HOLD_PARAMETER": SimTypeInt(signed=False, label="HOLD_PARAMETER"),
    "HOMEGROUPSHARINGCHOICES": SimTypeInt(signed=False, label="HOMEGROUPSHARINGCHOICES"),
    "HOOKPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["code", "wParam", "lParam"]), offset=0),
    "HORZSCROLLSTATES": SimTypeInt(signed=False, label="HORZSCROLLSTATES"),
    "HORZTHUMBSTATES": SimTypeInt(signed=False, label="HORZTHUMBSTATES"),
    "HOSTENT": SimStruct(OrderedDict((("h_name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("h_aliases", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("h_addrtype", SimTypeShort(signed=True, label="Int16")), ("h_length", SimTypeShort(signed=True, label="Int16")), ("h_addr_list", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)),)), name="HOSTENT", pack=False, align=None),
    "HOST_TYPE": SimTypeInt(signed=False, label="HOST_TYPE"),
    "HOTGLYPHSTATES": SimTypeInt(signed=False, label="HOTGLYPHSTATES"),
    "HOT_KEY_MODIFIERS": SimTypeInt(signed=False, label="HOT_KEY_MODIFIERS"),
    "HOVERBACKGROUNDSTATES": SimTypeInt(signed=False, label="HOVERBACKGROUNDSTATES"),
    "HPALETTE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPCON": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPEN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPOWERNOTIFY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPROPSHEETPAGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPROVIDER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPSS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPSSWALK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPSXA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HPTPROVIDER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRASCONN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRAWINPUT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRECOALT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRECOCONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRECOGNIZER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRECOLATTICE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRECOWORDLIST": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREGBATCH": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREGBATCHNOTIFICATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREGBATCHPORT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREGREADBATCH": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREGREADBATCHREPLY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREPORT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HREPORTSTORE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRESENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRESENUMEX": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRESOURCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRESTYPEENUM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRESULT": SimTypeInt(signed=True, label="Int32"),
    "HRGN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HRSRC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSAVEDUILANGUAGES": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSEMAPHORE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSE_CUSTOM_ERROR_INFO": SimStruct(OrderedDict((("pszStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uHttpSubError", SimTypeShort(signed=False, label="UInt16")), ("fAsync", SimTypeInt(signed=True, label="Int32")),)), name="HSE_CUSTOM_ERROR_INFO", pack=False, align=None),
    "HSE_EXEC_UNICODE_URL_INFO": SimStruct(OrderedDict((("pszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszMethod", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszChildHeaders", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserInfo", SimTypePointer(SimTypeRef("HSE_EXEC_UNICODE_URL_USER_INFO", SimStruct), offset=0)), ("pEntity", SimTypePointer(SimTypeRef("HSE_EXEC_URL_ENTITY_INFO", SimStruct), offset=0)), ("dwExecUrlFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_EXEC_UNICODE_URL_INFO", pack=False, align=None),
    "HSE_EXEC_UNICODE_URL_USER_INFO": SimStruct(OrderedDict((("hImpersonationToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszCustomUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCustomAuthType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HSE_EXEC_UNICODE_URL_USER_INFO", pack=False, align=None),
    "HSE_EXEC_URL_ENTITY_INFO": SimStruct(OrderedDict((("cbAvailable", SimTypeInt(signed=False, label="UInt32")), ("lpbData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HSE_EXEC_URL_ENTITY_INFO", pack=False, align=None),
    "HSE_EXEC_URL_INFO": SimStruct(OrderedDict((("pszUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszMethod", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszChildHeaders", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserInfo", SimTypePointer(SimTypeRef("HSE_EXEC_URL_USER_INFO", SimStruct), offset=0)), ("pEntity", SimTypePointer(SimTypeRef("HSE_EXEC_URL_ENTITY_INFO", SimStruct), offset=0)), ("dwExecUrlFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_EXEC_URL_INFO", pack=False, align=None),
    "HSE_EXEC_URL_STATUS": SimStruct(OrderedDict((("uHttpStatusCode", SimTypeShort(signed=False, label="UInt16")), ("uHttpSubStatus", SimTypeShort(signed=False, label="UInt16")), ("dwWin32Error", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_EXEC_URL_STATUS", pack=False, align=None),
    "HSE_EXEC_URL_USER_INFO": SimStruct(OrderedDict((("hImpersonationToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszCustomUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszCustomAuthType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HSE_EXEC_URL_USER_INFO", pack=False, align=None),
    "HSE_RESPONSE_VECTOR": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHeaders", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nElementCount", SimTypeInt(signed=False, label="UInt32")), ("lpElementArray", SimTypePointer(SimTypeRef("HSE_VECTOR_ELEMENT", SimStruct), offset=0)),)), name="HSE_RESPONSE_VECTOR", pack=False, align=None),
    "HSE_SEND_HEADER_EX_INFO": SimStruct(OrderedDict((("pszStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHeader", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchStatus", SimTypeInt(signed=False, label="UInt32")), ("cchHeader", SimTypeInt(signed=False, label="UInt32")), ("fKeepConn", SimTypeInt(signed=True, label="Int32")),)), name="HSE_SEND_HEADER_EX_INFO", pack=False, align=None),
    "HSE_TF_INFO": SimStruct(OrderedDict((("pfnHseIO", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXTENSION_CONTROL_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pECB", "pContext", "cbIO", "dwError"]), offset=0)), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszStatusCode", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BytesToWrite", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("pHead", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HeadLength", SimTypeInt(signed=False, label="UInt32")), ("pTail", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TailLength", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_TF_INFO", pack=False, align=None),
    "HSE_TRACE_INFO": SimStruct(OrderedDict((("fTraceRequest", SimTypeInt(signed=True, label="Int32")), ("TraceContextId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_TRACE_INFO", pack=False, align=None),
    "HSE_UNICODE_URL_MAPEX_INFO": SimStruct(OrderedDict((("lpszPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cchMatchingPath", SimTypeInt(signed=False, label="UInt32")), ("cchMatchingURL", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_UNICODE_URL_MAPEX_INFO", pack=False, align=None),
    "HSE_URL_MAPEX_INFO": SimStruct(OrderedDict((("lpszPath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cchMatchingPath", SimTypeInt(signed=False, label="UInt32")), ("cchMatchingURL", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="HSE_URL_MAPEX_INFO", pack=False, align=None),
    "HSE_VECTOR_ELEMENT": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="UInt32")), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbOffset", SimTypeLongLong(signed=False, label="UInt64")), ("cbSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="HSE_VECTOR_ELEMENT", pack=False, align=None),
    "HSE_VERSION_INFO": SimStruct(OrderedDict((("dwExtensionVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszExtensionDesc", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="HSE_VERSION_INFO", pack=False, align=None),
    "HSPRITE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSP_UPGRADE_IMAGEDATA": SimStruct(OrderedDict((("hashAlgID", SimTypeShort(signed=False, label="UInt16")), ("digestSize", SimTypeShort(signed=False, label="UInt16")), ("digest", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("fileName", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="HSP_UPGRADE_IMAGEDATA", pack=False, align=None),
    "HSTR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSURF": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSWDEVICE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSYNTHETICPOINTERDEVICE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSZ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HSZPAIR": SimStruct(OrderedDict((("hszSvc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszTopic", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HSZPAIR", pack=False, align=None),
    "HTAPICALL": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTAPILINE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTAPIPHONE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTASK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTHEME": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTHREAD_NETWORK_CONTEXT": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ThreadContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HTHREAD_NETWORK_CONTEXT", pack=False, align=None),
    "HTMLAnchorEvents": SimTypeBottom(label="HTMLAnchorEvents"),
    "HTMLAnchorEvents2": SimTypeBottom(label="HTMLAnchorEvents2"),
    "HTMLAppFlag": SimTypeInt(signed=False, label="HTMLAppFlag"),
    "HTMLAreaEvents": SimTypeBottom(label="HTMLAreaEvents"),
    "HTMLAreaEvents2": SimTypeBottom(label="HTMLAreaEvents2"),
    "HTMLBorder": SimTypeInt(signed=False, label="HTMLBorder"),
    "HTMLBorderStyle": SimTypeInt(signed=False, label="HTMLBorderStyle"),
    "HTMLButtonElementEvents": SimTypeBottom(label="HTMLButtonElementEvents"),
    "HTMLButtonElementEvents2": SimTypeBottom(label="HTMLButtonElementEvents2"),
    "HTMLCaptionFlag": SimTypeInt(signed=False, label="HTMLCaptionFlag"),
    "HTMLControlElementEvents": SimTypeBottom(label="HTMLControlElementEvents"),
    "HTMLControlElementEvents2": SimTypeBottom(label="HTMLControlElementEvents2"),
    "HTMLDlgBorder": SimTypeInt(signed=False, label="HTMLDlgBorder"),
    "HTMLDlgCenter": SimTypeInt(signed=False, label="HTMLDlgCenter"),
    "HTMLDlgEdge": SimTypeInt(signed=False, label="HTMLDlgEdge"),
    "HTMLDlgFlag": SimTypeInt(signed=False, label="HTMLDlgFlag"),
    "HTMLDocumentEvents": SimTypeBottom(label="HTMLDocumentEvents"),
    "HTMLDocumentEvents2": SimTypeBottom(label="HTMLDocumentEvents2"),
    "HTMLDocumentEvents3": SimTypeBottom(label="HTMLDocumentEvents3"),
    "HTMLDocumentEvents4": SimTypeBottom(label="HTMLDocumentEvents4"),
    "HTMLElementEvents": SimTypeBottom(label="HTMLElementEvents"),
    "HTMLElementEvents2": SimTypeBottom(label="HTMLElementEvents2"),
    "HTMLElementEvents3": SimTypeBottom(label="HTMLElementEvents3"),
    "HTMLElementEvents4": SimTypeBottom(label="HTMLElementEvents4"),
    "HTMLFormElementEvents": SimTypeBottom(label="HTMLFormElementEvents"),
    "HTMLFormElementEvents2": SimTypeBottom(label="HTMLFormElementEvents2"),
    "HTMLFrameSiteEvents": SimTypeBottom(label="HTMLFrameSiteEvents"),
    "HTMLFrameSiteEvents2": SimTypeBottom(label="HTMLFrameSiteEvents2"),
    "HTMLImgEvents": SimTypeBottom(label="HTMLImgEvents"),
    "HTMLImgEvents2": SimTypeBottom(label="HTMLImgEvents2"),
    "HTMLInputFileElementEvents": SimTypeBottom(label="HTMLInputFileElementEvents"),
    "HTMLInputFileElementEvents2": SimTypeBottom(label="HTMLInputFileElementEvents2"),
    "HTMLInputImageEvents": SimTypeBottom(label="HTMLInputImageEvents"),
    "HTMLInputImageEvents2": SimTypeBottom(label="HTMLInputImageEvents2"),
    "HTMLInputTextElementEvents": SimTypeBottom(label="HTMLInputTextElementEvents"),
    "HTMLInputTextElementEvents2": SimTypeBottom(label="HTMLInputTextElementEvents2"),
    "HTMLLabelEvents": SimTypeBottom(label="HTMLLabelEvents"),
    "HTMLLabelEvents2": SimTypeBottom(label="HTMLLabelEvents2"),
    "HTMLLinkElementEvents": SimTypeBottom(label="HTMLLinkElementEvents"),
    "HTMLLinkElementEvents2": SimTypeBottom(label="HTMLLinkElementEvents2"),
    "HTMLMapEvents": SimTypeBottom(label="HTMLMapEvents"),
    "HTMLMapEvents2": SimTypeBottom(label="HTMLMapEvents2"),
    "HTMLMarqueeElementEvents": SimTypeBottom(label="HTMLMarqueeElementEvents"),
    "HTMLMarqueeElementEvents2": SimTypeBottom(label="HTMLMarqueeElementEvents2"),
    "HTMLMaximizeFlag": SimTypeInt(signed=False, label="HTMLMaximizeFlag"),
    "HTMLMinimizeFlag": SimTypeInt(signed=False, label="HTMLMinimizeFlag"),
    "HTMLNamespaceEvents": SimTypeBottom(label="HTMLNamespaceEvents"),
    "HTMLObjectElementEvents": SimTypeBottom(label="HTMLObjectElementEvents"),
    "HTMLObjectElementEvents2": SimTypeBottom(label="HTMLObjectElementEvents2"),
    "HTMLOptionButtonElementEvents": SimTypeBottom(label="HTMLOptionButtonElementEvents"),
    "HTMLOptionButtonElementEvents2": SimTypeBottom(label="HTMLOptionButtonElementEvents2"),
    "HTMLScriptEvents": SimTypeBottom(label="HTMLScriptEvents"),
    "HTMLScriptEvents2": SimTypeBottom(label="HTMLScriptEvents2"),
    "HTMLSelectElementEvents": SimTypeBottom(label="HTMLSelectElementEvents"),
    "HTMLSelectElementEvents2": SimTypeBottom(label="HTMLSelectElementEvents2"),
    "HTMLStyleElementEvents": SimTypeBottom(label="HTMLStyleElementEvents"),
    "HTMLStyleElementEvents2": SimTypeBottom(label="HTMLStyleElementEvents2"),
    "HTMLSysMenuFlag": SimTypeInt(signed=False, label="HTMLSysMenuFlag"),
    "HTMLTableEvents": SimTypeBottom(label="HTMLTableEvents"),
    "HTMLTableEvents2": SimTypeBottom(label="HTMLTableEvents2"),
    "HTMLTextContainerEvents": SimTypeBottom(label="HTMLTextContainerEvents"),
    "HTMLTextContainerEvents2": SimTypeBottom(label="HTMLTextContainerEvents2"),
    "HTMLWindowEvents": SimTypeBottom(label="HTMLWindowEvents"),
    "HTMLWindowEvents2": SimTypeBottom(label="HTMLWindowEvents2"),
    "HTMLWindowEvents3": SimTypeBottom(label="HTMLWindowEvents3"),
    "HTMLWindowState": SimTypeInt(signed=False, label="HTMLWindowState"),
    "HTMLXMLHttpRequestEvents": SimTypeBottom(label="HTMLXMLHttpRequestEvents"),
    "HTML_HELP_COMMAND": SimTypeInt(signed=False, label="HTML_HELP_COMMAND"),
    "HTML_PAINTER": SimTypeInt(signed=False, label="HTML_PAINTER"),
    "HTML_PAINTER_INFO": SimStruct(OrderedDict((("lFlags", SimTypeInt(signed=True, label="Int32")), ("lZOrder", SimTypeInt(signed=True, label="Int32")), ("iidDrawObject", SimTypeBottom(label="Guid")), ("rcExpand", SimTypeRef("RECT", SimStruct)),)), name="HTML_PAINTER_INFO", pack=False, align=None),
    "HTML_PAINT_DRAW_FLAGS": SimTypeInt(signed=False, label="HTML_PAINT_DRAW_FLAGS"),
    "HTML_PAINT_DRAW_INFO": SimStruct(OrderedDict((("rcViewport", SimTypeRef("RECT", SimStruct)), ("hrgnUpdate", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xform", SimTypeRef("HTML_PAINT_XFORM", SimStruct)),)), name="HTML_PAINT_DRAW_INFO", pack=False, align=None),
    "HTML_PAINT_DRAW_INFO_FLAGS": SimTypeInt(signed=False, label="HTML_PAINT_DRAW_INFO_FLAGS"),
    "HTML_PAINT_EVENT_FLAGS": SimTypeInt(signed=False, label="HTML_PAINT_EVENT_FLAGS"),
    "HTML_PAINT_XFORM": SimStruct(OrderedDict((("eM11", SimTypeFloat(size=32)), ("eM12", SimTypeFloat(size=32)), ("eM21", SimTypeFloat(size=32)), ("eM22", SimTypeFloat(size=32)), ("eDx", SimTypeFloat(size=32)), ("eDy", SimTypeFloat(size=32)),)), name="HTML_PAINT_XFORM", pack=False, align=None),
    "HTML_PAINT_ZORDER": SimTypeInt(signed=False, label="HTML_PAINT_ZORDER"),
    "HTOUCHINPUT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTREEITEM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTTP2_SETTINGS_LIMITS_PARAM": SimStruct(OrderedDict((("Http2MaxSettingsPerFrame", SimTypeInt(signed=False, label="UInt32")), ("Http2MaxSettingsPerMinute", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP2_SETTINGS_LIMITS_PARAM", pack=False, align=None),
    "HTTP2_WINDOW_SIZE_PARAM": SimStruct(OrderedDict((("Http2ReceiveWindowSize", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP2_WINDOW_SIZE_PARAM", pack=False, align=None),
    "HTTPAPI_VERSION": SimStruct(OrderedDict((("HttpApiMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("HttpApiMinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="HTTPAPI_VERSION", pack=False, align=None),
    "HTTPSPOLICY_CALLBACK_DATA_AUTH_TYPE": SimTypeInt(signed=False, label="HTTPSPOLICY_CALLBACK_DATA_AUTH_TYPE"),
    "HTTPSPolicyCallbackData": SimStruct(OrderedDict((("Anonymous", SimUnion({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "cbSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dwAuthType", SimTypeInt(signed=False, label="HTTPSPOLICY_CALLBACK_DATA_AUTH_TYPE")), ("fdwChecks", SimTypeInt(signed=False, label="UInt32")), ("pwszServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTPSPolicyCallbackData", pack=False, align=None),
    "HTTP_503_RESPONSE_VERBOSITY": SimTypeInt(signed=False, label="HTTP_503_RESPONSE_VERBOSITY"),
    "HTTP_ADDREQ_FLAG": SimTypeInt(signed=False, label="HTTP_ADDREQ_FLAG"),
    "HTTP_AUTHENTICATION_HARDENING_LEVELS": SimTypeInt(signed=False, label="HTTP_AUTHENTICATION_HARDENING_LEVELS"),
    "HTTP_AUTH_STATUS": SimTypeInt(signed=False, label="HTTP_AUTH_STATUS"),
    "HTTP_BANDWIDTH_LIMIT_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("MaxBandwidth", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_BANDWIDTH_LIMIT_INFO", pack=False, align=None),
    "HTTP_BINDING_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("RequestQueueHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HTTP_BINDING_INFO", pack=False, align=None),
    "HTTP_BYTE_RANGE": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="HTTP_BYTE_RANGE", pack=False, align=None),
    "HTTP_CACHE_POLICY": SimStruct(OrderedDict((("Policy", SimTypeInt(signed=False, label="HTTP_CACHE_POLICY_TYPE")), ("SecondsToLive", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_CACHE_POLICY", pack=False, align=None),
    "HTTP_CACHE_POLICY_TYPE": SimTypeInt(signed=False, label="HTTP_CACHE_POLICY_TYPE"),
    "HTTP_CHANNEL_BIND_INFO": SimStruct(OrderedDict((("Hardening", SimTypeInt(signed=False, label="HTTP_AUTHENTICATION_HARDENING_LEVELS")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServiceNames", SimTypePointer(SimTypePointer(SimTypeRef("HTTP_SERVICE_BINDING_BASE", SimStruct), offset=0), offset=0)), ("NumberOfServiceNames", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_CHANNEL_BIND_INFO", pack=False, align=None),
    "HTTP_CONNECTION_LIMIT_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("MaxConnections", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_CONNECTION_LIMIT_INFO", pack=False, align=None),
    "HTTP_COOKED_URL": SimStruct(OrderedDict((("FullUrlLength", SimTypeShort(signed=False, label="UInt16")), ("HostLength", SimTypeShort(signed=False, label="UInt16")), ("AbsPathLength", SimTypeShort(signed=False, label="UInt16")), ("QueryStringLength", SimTypeShort(signed=False, label="UInt16")), ("pFullUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pHost", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAbsPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pQueryString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_COOKED_URL", pack=False, align=None),
    "HTTP_CREATE_REQUEST_QUEUE_PROPERTY_ID": SimTypeInt(signed=False, label="HTTP_CREATE_REQUEST_QUEUE_PROPERTY_ID"),
    "HTTP_CREATE_REQUEST_QUEUE_PROPERTY_INFO": SimStruct(OrderedDict((("PropertyId", SimTypeInt(signed=False, label="HTTP_CREATE_REQUEST_QUEUE_PROPERTY_ID")), ("PropertyInfoLength", SimTypeInt(signed=False, label="UInt32")), ("PropertyInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_CREATE_REQUEST_QUEUE_PROPERTY_INFO", pack=False, align=None),
    "HTTP_DATA_CHUNK": SimStruct(OrderedDict((("DataChunkType", SimTypeInt(signed=False, label="HTTP_DATA_CHUNK_TYPE")), ("Anonymous", SimUnion({"FromMemory": SimTypeRef("_FromMemory_e__Struct", SimStruct), "FromFileHandle": SimTypeRef("_FromFileHandle_e__Struct", SimStruct), "FromFragmentCache": SimTypeRef("_FromFragmentCache_e__Struct", SimStruct), "FromFragmentCacheEx": SimTypeRef("_FromFragmentCacheEx_e__Struct", SimStruct), "Trailers": SimTypeRef("_Trailers_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="HTTP_DATA_CHUNK", pack=False, align=None),
    "HTTP_DATA_CHUNK_TYPE": SimTypeInt(signed=False, label="HTTP_DATA_CHUNK_TYPE"),
    "HTTP_DELEGATE_REQUEST_PROPERTY_ID": SimTypeInt(signed=False, label="HTTP_DELEGATE_REQUEST_PROPERTY_ID"),
    "HTTP_DELEGATE_REQUEST_PROPERTY_INFO": SimStruct(OrderedDict((("PropertyId", SimTypeInt(signed=False, label="HTTP_DELEGATE_REQUEST_PROPERTY_ID")), ("PropertyInfoLength", SimTypeInt(signed=False, label="UInt32")), ("PropertyInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_DELEGATE_REQUEST_PROPERTY_INFO", pack=False, align=None),
    "HTTP_ENABLED_STATE": SimTypeInt(signed=False, label="HTTP_ENABLED_STATE"),
    "HTTP_ERROR_HEADERS_PARAM": SimStruct(OrderedDict((("StatusCode", SimTypeShort(signed=False, label="UInt16")), ("HeaderCount", SimTypeShort(signed=False, label="UInt16")), ("Headers", SimTypePointer(SimTypeRef("HTTP_UNKNOWN_HEADER", SimStruct), offset=0)),)), name="HTTP_ERROR_HEADERS_PARAM", pack=False, align=None),
    "HTTP_FEATURE_ID": SimTypeInt(signed=False, label="HTTP_FEATURE_ID"),
    "HTTP_FILTER_ACCESS_DENIED": SimStruct(OrderedDict((("pszURL", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPhysicalPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwReason", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_ACCESS_DENIED", pack=False, align=None),
    "HTTP_FILTER_AUTHENT": SimStruct(OrderedDict((("pszUser", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbUserBuff", SimTypeInt(signed=False, label="UInt32")), ("pszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPasswordBuff", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_AUTHENT", pack=False, align=None),
    "HTTP_FILTER_AUTH_COMPLETE_INFO": SimStruct(OrderedDict((("GetHeader", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetHeader", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddHeader", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetUserToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("HttpStatus", SimTypeInt(signed=False, label="UInt32")), ("fResetAuth", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_AUTH_COMPLETE_INFO", pack=False, align=None),
    "HTTP_FILTER_CONTEXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("ServerContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("fIsSecurePort", SimTypeInt(signed=True, label="Int32")), ("pFilterContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("GetServerVariable", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddResponseHeaders", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("WriteClient", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AllocMem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ServerSupportFunction", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HTTP_FILTER_CONTEXT", pack=False, align=None),
    "HTTP_FILTER_LOG": SimStruct(OrderedDict((("pszClientHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszClientUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszOperation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszTarget", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwHttpStatus", SimTypeInt(signed=False, label="UInt32")), ("dwWin32Status", SimTypeInt(signed=False, label="UInt32")), ("dwBytesSent", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRecvd", SimTypeInt(signed=False, label="UInt32")), ("msTimeForProcessing", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_LOG", pack=False, align=None),
    "HTTP_FILTER_PREPROC_HEADERS": SimStruct(OrderedDict((("GetHeader", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetHeader", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddHeader", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("HttpStatus", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_PREPROC_HEADERS", pack=False, align=None),
    "HTTP_FILTER_RAW_DATA": SimStruct(OrderedDict((("pvInData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbInData", SimTypeInt(signed=False, label="UInt32")), ("cbInBuffer", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_RAW_DATA", pack=False, align=None),
    "HTTP_FILTER_URL_MAP": SimStruct(OrderedDict((("pszURL", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPhysicalPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPathBuff", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_URL_MAP", pack=False, align=None),
    "HTTP_FILTER_URL_MAP_EX": SimStruct(OrderedDict((("pszURL", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPhysicalPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPathBuff", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cchMatchingPath", SimTypeInt(signed=False, label="UInt32")), ("cchMatchingURL", SimTypeInt(signed=False, label="UInt32")), ("pszScriptMapEntry", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HTTP_FILTER_URL_MAP_EX", pack=False, align=None),
    "HTTP_FILTER_VERSION": SimStruct(OrderedDict((("dwServerFilterVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFilterVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszFilterDesc", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FILTER_VERSION", pack=False, align=None),
    "HTTP_FLOWRATE_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("MaxBandwidth", SimTypeInt(signed=False, label="UInt32")), ("MaxPeakBandwidth", SimTypeInt(signed=False, label="UInt32")), ("BurstSize", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_FLOWRATE_INFO", pack=False, align=None),
    "HTTP_HEADER_ID": SimTypeInt(signed=False, label="HTTP_HEADER_ID"),
    "HTTP_INITIALIZE": SimTypeInt(signed=False, label="HTTP_INITIALIZE"),
    "HTTP_KNOWN_HEADER": SimStruct(OrderedDict((("RawValueLength", SimTypeShort(signed=False, label="UInt16")), ("pRawValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HTTP_KNOWN_HEADER", pack=False, align=None),
    "HTTP_LISTEN_ENDPOINT_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("EnableSharing", SimTypeChar(label="Byte")),)), name="HTTP_LISTEN_ENDPOINT_INFO", pack=False, align=None),
    "HTTP_LOGGING_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("LoggingFlags", SimTypeInt(signed=False, label="UInt32")), ("SoftwareName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SoftwareNameLength", SimTypeShort(signed=False, label="UInt16")), ("DirectoryNameLength", SimTypeShort(signed=False, label="UInt16")), ("DirectoryName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Format", SimTypeInt(signed=False, label="HTTP_LOGGING_TYPE")), ("Fields", SimTypeInt(signed=False, label="UInt32")), ("pExtFields", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumOfExtFields", SimTypeShort(signed=False, label="UInt16")), ("MaxRecordSize", SimTypeShort(signed=False, label="UInt16")), ("RolloverType", SimTypeInt(signed=False, label="HTTP_LOGGING_ROLLOVER_TYPE")), ("RolloverSize", SimTypeInt(signed=False, label="UInt32")), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_LOGGING_INFO", pack=False, align=None),
    "HTTP_LOGGING_ROLLOVER_TYPE": SimTypeInt(signed=False, label="HTTP_LOGGING_ROLLOVER_TYPE"),
    "HTTP_LOGGING_TYPE": SimTypeInt(signed=False, label="HTTP_LOGGING_TYPE"),
    "HTTP_LOG_DATA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HTTP_LOG_DATA_TYPE")),)), name="HTTP_LOG_DATA", pack=False, align=None),
    "HTTP_LOG_DATA_TYPE": SimTypeInt(signed=False, label="HTTP_LOG_DATA_TYPE"),
    "HTTP_LOG_FIELDS_DATA": SimStruct(OrderedDict((("Base", SimTypeRef("HTTP_LOG_DATA", SimStruct)), ("UserNameLength", SimTypeShort(signed=False, label="UInt16")), ("UriStemLength", SimTypeShort(signed=False, label="UInt16")), ("ClientIpLength", SimTypeShort(signed=False, label="UInt16")), ("ServerNameLength", SimTypeShort(signed=False, label="UInt16")), ("ServiceNameLength", SimTypeShort(signed=False, label="UInt16")), ("ServerIpLength", SimTypeShort(signed=False, label="UInt16")), ("MethodLength", SimTypeShort(signed=False, label="UInt16")), ("UriQueryLength", SimTypeShort(signed=False, label="UInt16")), ("HostLength", SimTypeShort(signed=False, label="UInt16")), ("UserAgentLength", SimTypeShort(signed=False, label="UInt16")), ("CookieLength", SimTypeShort(signed=False, label="UInt16")), ("ReferrerLength", SimTypeShort(signed=False, label="UInt16")), ("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UriStem", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ClientIp", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServiceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerIp", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Method", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UriQuery", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Host", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UserAgent", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Cookie", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Referrer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServerPort", SimTypeShort(signed=False, label="UInt16")), ("ProtocolStatus", SimTypeShort(signed=False, label="UInt16")), ("Win32Status", SimTypeInt(signed=False, label="UInt32")), ("MethodNum", SimTypeInt(signed=False, label="HTTP_VERB")), ("SubStatus", SimTypeShort(signed=False, label="UInt16")),)), name="HTTP_LOG_FIELDS_DATA", pack=False, align=None),
    "HTTP_MULTIPLE_KNOWN_HEADERS": SimStruct(OrderedDict((("HeaderId", SimTypeInt(signed=False, label="HTTP_HEADER_ID")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("KnownHeaderCount", SimTypeShort(signed=False, label="UInt16")), ("KnownHeaders", SimTypePointer(SimTypeRef("HTTP_KNOWN_HEADER", SimStruct), offset=0)),)), name="HTTP_MULTIPLE_KNOWN_HEADERS", pack=False, align=None),
    "HTTP_PERFORMANCE_PARAM": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HTTP_PERFORMANCE_PARAM_TYPE")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_PERFORMANCE_PARAM", pack=False, align=None),
    "HTTP_PERFORMANCE_PARAM_TYPE": SimTypeInt(signed=False, label="HTTP_PERFORMANCE_PARAM_TYPE"),
    "HTTP_POLICY_EXTENSION_INIT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="HTTP_POLICY_EXTENSION_VERSION"), SimTypeInt(signed=False, label="HTTP_POLICY_EXTENSION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Version", "Type", "pvData", "cbData"]), offset=0),
    "HTTP_POLICY_EXTENSION_SHUTDOWN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="HTTP_POLICY_EXTENSION_TYPE")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Type"]), offset=0),
    "HTTP_POLICY_EXTENSION_TYPE": SimTypeInt(signed=False, label="HTTP_POLICY_EXTENSION_TYPE"),
    "HTTP_POLICY_EXTENSION_VERSION": SimTypeInt(signed=False, label="HTTP_POLICY_EXTENSION_VERSION"),
    "HTTP_PROPERTY_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_PROPERTY_FLAGS", pack=False, align=None),
    "HTTP_PROTECTION_LEVEL_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("Level", SimTypeInt(signed=False, label="HTTP_PROTECTION_LEVEL_TYPE")),)), name="HTTP_PROTECTION_LEVEL_INFO", pack=False, align=None),
    "HTTP_PROTECTION_LEVEL_TYPE": SimTypeInt(signed=False, label="HTTP_PROTECTION_LEVEL_TYPE"),
    "HTTP_PUSH_NOTIFICATION_STATUS": SimStruct(OrderedDict((("ChannelStatusValid", SimTypeInt(signed=True, label="Int32")), ("ChannelStatus", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_PUSH_NOTIFICATION_STATUS", pack=False, align=None),
    "HTTP_PUSH_TRANSPORT_SETTING": SimStruct(OrderedDict((("TransportSettingId", SimTypeBottom(label="Guid")), ("BrokerEventId", SimTypeBottom(label="Guid")),)), name="HTTP_PUSH_TRANSPORT_SETTING", pack=False, align=None),
    "HTTP_PUSH_WAIT_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HTTP_PUSH_WAIT_TYPE": SimTypeInt(signed=False, label="HTTP_PUSH_WAIT_TYPE"),
    "HTTP_QOS_SETTING_INFO": SimStruct(OrderedDict((("QosType", SimTypeInt(signed=False, label="HTTP_QOS_SETTING_TYPE")), ("QosSetting", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_QOS_SETTING_INFO", pack=False, align=None),
    "HTTP_QOS_SETTING_TYPE": SimTypeInt(signed=False, label="HTTP_QOS_SETTING_TYPE"),
    "HTTP_QUERY_REQUEST_QUALIFIER_QUIC": SimStruct(OrderedDict((("Freshness", SimTypeLongLong(signed=False, label="UInt64")),)), name="HTTP_QUERY_REQUEST_QUALIFIER_QUIC", pack=False, align=None),
    "HTTP_QUERY_REQUEST_QUALIFIER_TCP": SimStruct(OrderedDict((("Freshness", SimTypeLongLong(signed=False, label="UInt64")),)), name="HTTP_QUERY_REQUEST_QUALIFIER_TCP", pack=False, align=None),
    "HTTP_QUIC_API_TIMINGS": SimStruct(OrderedDict((("ConnectionTimings", SimTypeRef("HTTP_QUIC_CONNECTION_API_TIMINGS", SimStruct)), ("StreamTimings", SimTypeRef("HTTP_QUIC_STREAM_API_TIMINGS", SimStruct)),)), name="HTTP_QUIC_API_TIMINGS", pack=False, align=None),
    "HTTP_QUIC_CONNECTION_API_TIMINGS": SimStruct(OrderedDict((("OpenTime", SimTypeLongLong(signed=False, label="UInt64")), ("CloseTime", SimTypeLongLong(signed=False, label="UInt64")), ("StartTime", SimTypeLongLong(signed=False, label="UInt64")), ("ShutdownTime", SimTypeLongLong(signed=False, label="UInt64")), ("SecConfigCreateTime", SimTypeLongLong(signed=False, label="UInt64")), ("SecConfigDeleteTime", SimTypeLongLong(signed=False, label="UInt64")), ("GetParamCount", SimTypeLongLong(signed=False, label="UInt64")), ("GetParamSum", SimTypeLongLong(signed=False, label="UInt64")), ("SetParamCount", SimTypeLongLong(signed=False, label="UInt64")), ("SetParamSum", SimTypeLongLong(signed=False, label="UInt64")), ("SetCallbackHandlerCount", SimTypeLongLong(signed=False, label="UInt64")), ("SetCallbackHandlerSum", SimTypeLongLong(signed=False, label="UInt64")), ("ControlStreamTimings", SimTypeRef("HTTP_QUIC_STREAM_API_TIMINGS", SimStruct)),)), name="HTTP_QUIC_CONNECTION_API_TIMINGS", pack=False, align=None),
    "HTTP_QUIC_STREAM_API_TIMINGS": SimStruct(OrderedDict((("OpenCount", SimTypeLongLong(signed=False, label="UInt64")), ("OpenSum", SimTypeLongLong(signed=False, label="UInt64")), ("CloseCount", SimTypeLongLong(signed=False, label="UInt64")), ("CloseSum", SimTypeLongLong(signed=False, label="UInt64")), ("StartCount", SimTypeLongLong(signed=False, label="UInt64")), ("StartSum", SimTypeLongLong(signed=False, label="UInt64")), ("ShutdownCount", SimTypeLongLong(signed=False, label="UInt64")), ("ShutdownSum", SimTypeLongLong(signed=False, label="UInt64")), ("SendCount", SimTypeLongLong(signed=False, label="UInt64")), ("SendSum", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveSetEnabledCount", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveSetEnabledSum", SimTypeLongLong(signed=False, label="UInt64")), ("GetParamCount", SimTypeLongLong(signed=False, label="UInt64")), ("GetParamSum", SimTypeLongLong(signed=False, label="UInt64")), ("SetParamCount", SimTypeLongLong(signed=False, label="UInt64")), ("SetParamSum", SimTypeLongLong(signed=False, label="UInt64")), ("SetCallbackHandlerCount", SimTypeLongLong(signed=False, label="UInt64")), ("SetCallbackHandlerSum", SimTypeLongLong(signed=False, label="UInt64")),)), name="HTTP_QUIC_STREAM_API_TIMINGS", pack=False, align=None),
    "HTTP_QUIC_STREAM_REQUEST_STATS": SimStruct(OrderedDict((("StreamWaitStart", SimTypeLongLong(signed=False, label="UInt64")), ("StreamWaitEnd", SimTypeLongLong(signed=False, label="UInt64")), ("RequestHeadersCompressionStart", SimTypeLongLong(signed=False, label="UInt64")), ("RequestHeadersCompressionEnd", SimTypeLongLong(signed=False, label="UInt64")), ("ResponseHeadersDecompressionStart", SimTypeLongLong(signed=False, label="UInt64")), ("ResponseHeadersDecompressionEnd", SimTypeLongLong(signed=False, label="UInt64")), ("RequestHeadersCompressedSize", SimTypeLongLong(signed=False, label="UInt64")), ("ResponseHeadersCompressedSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="HTTP_QUIC_STREAM_REQUEST_STATS", pack=False, align=None),
    "HTTP_RECEIVE_HTTP_REQUEST_FLAGS": SimTypeInt(signed=False, label="HTTP_RECEIVE_HTTP_REQUEST_FLAGS"),
    "HTTP_REQUEST_AUTH_INFO": SimStruct(OrderedDict((("AuthStatus", SimTypeInt(signed=False, label="HTTP_AUTH_STATUS")), ("SecStatus", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AuthType", SimTypeInt(signed=False, label="HTTP_REQUEST_AUTH_TYPE")), ("AccessToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ContextAttributes", SimTypeInt(signed=False, label="UInt32")), ("PackedContextLength", SimTypeInt(signed=False, label="UInt32")), ("PackedContextType", SimTypeInt(signed=False, label="UInt32")), ("PackedContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MutualAuthDataLength", SimTypeInt(signed=False, label="UInt32")), ("pMutualAuthData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PackageNameLength", SimTypeShort(signed=False, label="UInt16")), ("pPackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_REQUEST_AUTH_INFO", pack=False, align=None),
    "HTTP_REQUEST_AUTH_TYPE": SimTypeInt(signed=False, label="HTTP_REQUEST_AUTH_TYPE"),
    "HTTP_REQUEST_CHANNEL_BIND_STATUS": SimStruct(OrderedDict((("ServiceName", SimTypePointer(SimTypeRef("HTTP_SERVICE_BINDING_BASE", SimStruct), offset=0)), ("ChannelToken", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ChannelTokenSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_REQUEST_CHANNEL_BIND_STATUS", pack=False, align=None),
    "HTTP_REQUEST_HEADERS": SimStruct(OrderedDict((("UnknownHeaderCount", SimTypeShort(signed=False, label="UInt16")), ("pUnknownHeaders", SimTypePointer(SimTypeRef("HTTP_UNKNOWN_HEADER", SimStruct), offset=0)), ("TrailerCount", SimTypeShort(signed=False, label="UInt16")), ("pTrailers", SimTypePointer(SimTypeRef("HTTP_UNKNOWN_HEADER", SimStruct), offset=0)), ("KnownHeaders", SimTypeArray(SimTypeRef("HTTP_KNOWN_HEADER", SimStruct), 41)),)), name="HTTP_REQUEST_HEADERS", pack=False, align=None),
    "HTTP_REQUEST_INFO": SimStruct(OrderedDict((("InfoType", SimTypeInt(signed=False, label="HTTP_REQUEST_INFO_TYPE")), ("InfoLength", SimTypeInt(signed=False, label="UInt32")), ("pInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_REQUEST_INFO", pack=False, align=None),
    "HTTP_REQUEST_INFO_TYPE": SimTypeInt(signed=False, label="HTTP_REQUEST_INFO_TYPE"),
    "HTTP_REQUEST_PROPERTY": SimTypeInt(signed=False, label="HTTP_REQUEST_PROPERTY"),
    "HTTP_REQUEST_PROPERTY_SNI": SimStruct(OrderedDict((("Hostname", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_REQUEST_PROPERTY_SNI", pack=False, align=None),
    "HTTP_REQUEST_PROPERTY_STREAM_ERROR": SimStruct(OrderedDict((("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_REQUEST_PROPERTY_STREAM_ERROR", pack=False, align=None),
    "HTTP_REQUEST_SIZING_INFO": SimStruct(OrderedDict((("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("RequestIndex", SimTypeInt(signed=False, label="UInt32")), ("RequestSizingCount", SimTypeInt(signed=False, label="UInt32")), ("RequestSizing", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)),)), name="HTTP_REQUEST_SIZING_INFO", pack=False, align=None),
    "HTTP_REQUEST_SIZING_TYPE": SimTypeInt(signed=False, label="HTTP_REQUEST_SIZING_TYPE"),
    "HTTP_REQUEST_TIMES": SimStruct(OrderedDict((("cTimes", SimTypeInt(signed=False, label="UInt32")), ("rgTimes", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 32)),)), name="HTTP_REQUEST_TIMES", pack=False, align=None),
    "HTTP_REQUEST_TIMING_INFO": SimStruct(OrderedDict((("RequestTimingCount", SimTypeInt(signed=False, label="UInt32")), ("RequestTiming", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 30)),)), name="HTTP_REQUEST_TIMING_INFO", pack=False, align=None),
    "HTTP_REQUEST_TIMING_TYPE": SimTypeInt(signed=False, label="HTTP_REQUEST_TIMING_TYPE"),
    "HTTP_REQUEST_TOKEN_BINDING_INFO": SimStruct(OrderedDict((("TokenBinding", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TokenBindingSize", SimTypeInt(signed=False, label="UInt32")), ("EKM", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("EKMSize", SimTypeInt(signed=False, label="UInt32")), ("KeyType", SimTypeChar(label="Byte")),)), name="HTTP_REQUEST_TOKEN_BINDING_INFO", pack=False, align=None),
    "HTTP_REQUEST_V1": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ConnectionId", SimTypeLongLong(signed=False, label="UInt64")), ("RequestId", SimTypeLongLong(signed=False, label="UInt64")), ("UrlContext", SimTypeLongLong(signed=False, label="UInt64")), ("Version", SimTypeRef("HTTP_VERSION", SimStruct)), ("Verb", SimTypeInt(signed=False, label="HTTP_VERB")), ("UnknownVerbLength", SimTypeShort(signed=False, label="UInt16")), ("RawUrlLength", SimTypeShort(signed=False, label="UInt16")), ("pUnknownVerb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pRawUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CookedUrl", SimTypeRef("HTTP_COOKED_URL", SimStruct)), ("Address", SimTypeRef("HTTP_TRANSPORT_ADDRESS", SimStruct)), ("Headers", SimTypeRef("HTTP_REQUEST_HEADERS", SimStruct)), ("BytesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("EntityChunkCount", SimTypeShort(signed=False, label="UInt16")), ("pEntityChunks", SimTypePointer(SimTypeRef("HTTP_DATA_CHUNK", SimStruct), offset=0)), ("RawConnectionId", SimTypeLongLong(signed=False, label="UInt64")), ("pSslInfo", SimTypePointer(SimTypeRef("HTTP_SSL_INFO", SimStruct), offset=0)),)), name="HTTP_REQUEST_V1", pack=False, align=None),
    "HTTP_REQUEST_V2": SimStruct(OrderedDict((("Base", SimTypeRef("HTTP_REQUEST_V1", SimStruct)), ("RequestInfoCount", SimTypeShort(signed=False, label="UInt16")), ("pRequestInfo", SimTypePointer(SimTypeRef("HTTP_REQUEST_INFO", SimStruct), offset=0)),)), name="HTTP_REQUEST_V2", pack=False, align=None),
    "HTTP_RESPONSE_HEADERS": SimStruct(OrderedDict((("UnknownHeaderCount", SimTypeShort(signed=False, label="UInt16")), ("pUnknownHeaders", SimTypePointer(SimTypeRef("HTTP_UNKNOWN_HEADER", SimStruct), offset=0)), ("TrailerCount", SimTypeShort(signed=False, label="UInt16")), ("pTrailers", SimTypePointer(SimTypeRef("HTTP_UNKNOWN_HEADER", SimStruct), offset=0)), ("KnownHeaders", SimTypeArray(SimTypeRef("HTTP_KNOWN_HEADER", SimStruct), 30)),)), name="HTTP_RESPONSE_HEADERS", pack=False, align=None),
    "HTTP_RESPONSE_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HTTP_RESPONSE_INFO_TYPE")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("pInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_RESPONSE_INFO", pack=False, align=None),
    "HTTP_RESPONSE_INFO_TYPE": SimTypeInt(signed=False, label="HTTP_RESPONSE_INFO_TYPE"),
    "HTTP_RESPONSE_V1": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeRef("HTTP_VERSION", SimStruct)), ("StatusCode", SimTypeShort(signed=False, label="UInt16")), ("ReasonLength", SimTypeShort(signed=False, label="UInt16")), ("pReason", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Headers", SimTypeRef("HTTP_RESPONSE_HEADERS", SimStruct)), ("EntityChunkCount", SimTypeShort(signed=False, label="UInt16")), ("pEntityChunks", SimTypePointer(SimTypeRef("HTTP_DATA_CHUNK", SimStruct), offset=0)),)), name="HTTP_RESPONSE_V1", pack=False, align=None),
    "HTTP_RESPONSE_V2": SimStruct(OrderedDict((("Base", SimTypeRef("HTTP_RESPONSE_V1", SimStruct)), ("ResponseInfoCount", SimTypeShort(signed=False, label="UInt16")), ("pResponseInfo", SimTypePointer(SimTypeRef("HTTP_RESPONSE_INFO", SimStruct), offset=0)),)), name="HTTP_RESPONSE_V2", pack=False, align=None),
    "HTTP_SCHEME": SimTypeInt(signed=False, label="HTTP_SCHEME"),
    "HTTP_SERVER_AUTHENTICATION_BASIC_PARAMS": SimStruct(OrderedDict((("RealmLength", SimTypeShort(signed=False, label="UInt16")), ("Realm", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_SERVER_AUTHENTICATION_BASIC_PARAMS", pack=False, align=None),
    "HTTP_SERVER_AUTHENTICATION_DIGEST_PARAMS": SimStruct(OrderedDict((("DomainNameLength", SimTypeShort(signed=False, label="UInt16")), ("DomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RealmLength", SimTypeShort(signed=False, label="UInt16")), ("Realm", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_SERVER_AUTHENTICATION_DIGEST_PARAMS", pack=False, align=None),
    "HTTP_SERVER_AUTHENTICATION_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("AuthSchemes", SimTypeInt(signed=False, label="UInt32")), ("ReceiveMutualAuth", SimTypeChar(label="Byte")), ("ReceiveContextHandle", SimTypeChar(label="Byte")), ("DisableNTLMCredentialCaching", SimTypeChar(label="Byte")), ("ExFlags", SimTypeChar(label="Byte")), ("DigestParams", SimTypeRef("HTTP_SERVER_AUTHENTICATION_DIGEST_PARAMS", SimStruct)), ("BasicParams", SimTypeRef("HTTP_SERVER_AUTHENTICATION_BASIC_PARAMS", SimStruct)),)), name="HTTP_SERVER_AUTHENTICATION_INFO", pack=False, align=None),
    "HTTP_SERVER_PROPERTY": SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"),
    "HTTP_SERVICE_BINDING_A": SimStruct(OrderedDict((("Base", SimTypeRef("HTTP_SERVICE_BINDING_BASE", SimStruct)), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_BINDING_A", pack=False, align=None),
    "HTTP_SERVICE_BINDING_BASE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="HTTP_SERVICE_BINDING_TYPE")),)), name="HTTP_SERVICE_BINDING_BASE", pack=False, align=None),
    "HTTP_SERVICE_BINDING_TYPE": SimTypeInt(signed=False, label="HTTP_SERVICE_BINDING_TYPE"),
    "HTTP_SERVICE_BINDING_W": SimStruct(OrderedDict((("Base", SimTypeRef("HTTP_SERVICE_BINDING_BASE", SimStruct)), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_BINDING_W", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_CACHE_KEY": SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_CACHE_KEY"),
    "HTTP_SERVICE_CONFIG_CACHE_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_CACHE_KEY")), ("ParamDesc", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_CACHE_SET", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_ID": SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_ID"),
    "HTTP_SERVICE_CONFIG_IP_LISTEN_PARAM": SimStruct(OrderedDict((("AddrLength", SimTypeShort(signed=False, label="UInt16")), ("pAddress", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)),)), name="HTTP_SERVICE_CONFIG_IP_LISTEN_PARAM", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_IP_LISTEN_QUERY": SimStruct(OrderedDict((("AddrCount", SimTypeInt(signed=False, label="UInt32")), ("AddrList", SimTypePointer(SimTypeRef("SOCKADDR_STORAGE", SimStruct), offset=0)),)), name="HTTP_SERVICE_CONFIG_IP_LISTEN_QUERY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_QUERY_TYPE": SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE"),
    "HTTP_SERVICE_CONFIG_SETTING_KEY": SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_SETTING_KEY"),
    "HTTP_SERVICE_CONFIG_SETTING_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_SETTING_KEY")), ("ParamDesc", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_SETTING_SET", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_CCS_KEY": SimStruct(OrderedDict((("LocalAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_CCS_KEY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_CCS_QUERY": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_CCS_KEY", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_SSL_CCS_QUERY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_CCS_QUERY_EX": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_CCS_KEY", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")), ("ParamType", SimTypeInt(signed=False, label="HTTP_SSL_SERVICE_CONFIG_EX_PARAM_TYPE")),)), name="HTTP_SERVICE_CONFIG_SSL_CCS_QUERY_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_CCS_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_CCS_KEY", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_PARAM", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_CCS_SET", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_CCS_SET_EX": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_CCS_KEY", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_PARAM_EX", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_CCS_SET_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_KEY": SimStruct(OrderedDict((("pIpPort", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)),)), name="HTTP_SERVICE_CONFIG_SSL_KEY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_KEY_EX": SimStruct(OrderedDict((("IpPort", SimTypeRef("SOCKADDR_STORAGE", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_KEY_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_PARAM": SimStruct(OrderedDict((("SslHashLength", SimTypeInt(signed=False, label="UInt32")), ("pSslHash", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AppId", SimTypeBottom(label="Guid")), ("pSslCertStoreName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DefaultCertCheckMode", SimTypeInt(signed=False, label="UInt32")), ("DefaultRevocationFreshnessTime", SimTypeInt(signed=False, label="UInt32")), ("DefaultRevocationUrlRetrievalTimeout", SimTypeInt(signed=False, label="UInt32")), ("pDefaultSslCtlIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDefaultSslCtlStoreName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DefaultFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_SSL_PARAM", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_PARAM_EX": SimStruct(OrderedDict((("ParamType", SimTypeInt(signed=False, label="HTTP_SSL_SERVICE_CONFIG_EX_PARAM_TYPE")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"Http2WindowSizeParam": SimTypeRef("HTTP2_WINDOW_SIZE_PARAM", SimStruct), "Http2SettingsLimitsParam": SimTypeRef("HTTP2_SETTINGS_LIMITS_PARAM", SimStruct), "HttpPerformanceParam": SimTypeRef("HTTP_PERFORMANCE_PARAM", SimStruct), "HttpTlsRestrictionsParam": SimTypeRef("HTTP_TLS_RESTRICTIONS_PARAM", SimStruct), "HttpErrorHeadersParam": SimTypeRef("HTTP_ERROR_HEADERS_PARAM", SimStruct), "HttpTlsSessionTicketKeysParam": SimTypeRef("HTTP_TLS_SESSION_TICKET_KEYS_PARAM", SimStruct)}, name="<anon>", label="None")),)), name="HTTP_SERVICE_CONFIG_SSL_PARAM_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_QUERY": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_KEY", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_SSL_QUERY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_QUERY_EX": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_KEY_EX", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")), ("ParamType", SimTypeInt(signed=False, label="HTTP_SSL_SERVICE_CONFIG_EX_PARAM_TYPE")),)), name="HTTP_SERVICE_CONFIG_SSL_QUERY_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_KEY", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_PARAM", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_SET", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SET_EX": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_KEY_EX", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_PARAM_EX", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_SET_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SNI_KEY": SimStruct(OrderedDict((("IpPort", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("Host", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_SERVICE_CONFIG_SSL_SNI_KEY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SNI_QUERY": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_SNI_KEY", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_SSL_SNI_QUERY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SNI_QUERY_EX": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_SNI_KEY", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")), ("ParamType", SimTypeInt(signed=False, label="HTTP_SSL_SERVICE_CONFIG_EX_PARAM_TYPE")),)), name="HTTP_SERVICE_CONFIG_SSL_SNI_QUERY_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SNI_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_SNI_KEY", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_PARAM", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_SNI_SET", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_SSL_SNI_SET_EX": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_SNI_KEY", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_SSL_PARAM_EX", SimStruct)),)), name="HTTP_SERVICE_CONFIG_SSL_SNI_SET_EX", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_TIMEOUT_KEY": SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_TIMEOUT_KEY"),
    "HTTP_SERVICE_CONFIG_TIMEOUT_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_TIMEOUT_KEY")), ("ParamDesc", SimTypeShort(signed=False, label="UInt16")),)), name="HTTP_SERVICE_CONFIG_TIMEOUT_SET", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_URLACL_KEY": SimStruct(OrderedDict((("pUrlPrefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_SERVICE_CONFIG_URLACL_KEY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_URLACL_PARAM": SimStruct(OrderedDict((("pStringSecurityDescriptor", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_SERVICE_CONFIG_URLACL_PARAM", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_URLACL_QUERY": SimStruct(OrderedDict((("QueryDesc", SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_QUERY_TYPE")), ("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_URLACL_KEY", SimStruct)), ("dwToken", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SERVICE_CONFIG_URLACL_QUERY", pack=False, align=None),
    "HTTP_SERVICE_CONFIG_URLACL_SET": SimStruct(OrderedDict((("KeyDesc", SimTypeRef("HTTP_SERVICE_CONFIG_URLACL_KEY", SimStruct)), ("ParamDesc", SimTypeRef("HTTP_SERVICE_CONFIG_URLACL_PARAM", SimStruct)),)), name="HTTP_SERVICE_CONFIG_URLACL_SET", pack=False, align=None),
    "HTTP_SSL_CLIENT_CERT_INFO": SimStruct(OrderedDict((("CertFlags", SimTypeInt(signed=False, label="UInt32")), ("CertEncodedSize", SimTypeInt(signed=False, label="UInt32")), ("pCertEncoded", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Token", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CertDeniedByMapper", SimTypeChar(label="Byte")),)), name="HTTP_SSL_CLIENT_CERT_INFO", pack=False, align=None),
    "HTTP_SSL_INFO": SimStruct(OrderedDict((("ServerCertKeySize", SimTypeShort(signed=False, label="UInt16")), ("ConnectionKeySize", SimTypeShort(signed=False, label="UInt16")), ("ServerCertIssuerSize", SimTypeInt(signed=False, label="UInt32")), ("ServerCertSubjectSize", SimTypeInt(signed=False, label="UInt32")), ("pServerCertIssuer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pServerCertSubject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pClientCertInfo", SimTypePointer(SimTypeRef("HTTP_SSL_CLIENT_CERT_INFO", SimStruct), offset=0)), ("SslClientCertNegotiated", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SSL_INFO", pack=False, align=None),
    "HTTP_SSL_PROTOCOL_INFO": SimStruct(OrderedDict((("Protocol", SimTypeInt(signed=False, label="UInt32")), ("CipherType", SimTypeInt(signed=False, label="UInt32")), ("CipherStrength", SimTypeInt(signed=False, label="UInt32")), ("HashType", SimTypeInt(signed=False, label="UInt32")), ("HashStrength", SimTypeInt(signed=False, label="UInt32")), ("KeyExchangeType", SimTypeInt(signed=False, label="UInt32")), ("KeyExchangeStrength", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_SSL_PROTOCOL_INFO", pack=False, align=None),
    "HTTP_SSL_SERVICE_CONFIG_EX_PARAM_TYPE": SimTypeInt(signed=False, label="HTTP_SSL_SERVICE_CONFIG_EX_PARAM_TYPE"),
    "HTTP_STATE_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("State", SimTypeInt(signed=False, label="HTTP_ENABLED_STATE")),)), name="HTTP_STATE_INFO", pack=False, align=None),
    "HTTP_TIMEOUT_LIMIT_INFO": SimStruct(OrderedDict((("Flags", SimTypeRef("HTTP_PROPERTY_FLAGS", SimStruct)), ("EntityBody", SimTypeShort(signed=False, label="UInt16")), ("DrainEntityBody", SimTypeShort(signed=False, label="UInt16")), ("RequestQueue", SimTypeShort(signed=False, label="UInt16")), ("IdleConnection", SimTypeShort(signed=False, label="UInt16")), ("HeaderWait", SimTypeShort(signed=False, label="UInt16")), ("MinSendRate", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_TIMEOUT_LIMIT_INFO", pack=False, align=None),
    "HTTP_TLS_RESTRICTIONS_PARAM": SimStruct(OrderedDict((("RestrictionCount", SimTypeInt(signed=False, label="UInt32")), ("TlsRestrictions", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_TLS_RESTRICTIONS_PARAM", pack=False, align=None),
    "HTTP_TLS_SESSION_TICKET_KEYS_PARAM": SimStruct(OrderedDict((("SessionTicketKeyCount", SimTypeInt(signed=False, label="UInt32")), ("SessionTicketKeys", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="HTTP_TLS_SESSION_TICKET_KEYS_PARAM", pack=False, align=None),
    "HTTP_TRACE_CONFIGURATION": SimStruct(OrderedDict((("pProviderGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwAreas", SimTypeInt(signed=False, label="UInt32")), ("dwVerbosity", SimTypeInt(signed=False, label="UInt32")), ("fProviderEnabled", SimTypeInt(signed=True, label="Int32")),)), name="HTTP_TRACE_CONFIGURATION", pack=False, align=None),
    "HTTP_TRACE_EVENT": SimStruct(OrderedDict((("pProviderGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwArea", SimTypeInt(signed=False, label="UInt32")), ("pAreaGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwEvent", SimTypeInt(signed=False, label="UInt32")), ("pszEventName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwEventVersion", SimTypeInt(signed=False, label="UInt32")), ("dwVerbosity", SimTypeInt(signed=False, label="UInt32")), ("pActivityGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pRelatedActivityGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwTimeStamp", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cEventItems", SimTypeInt(signed=False, label="UInt32")), ("pEventItems", SimTypePointer(SimTypeRef("HTTP_TRACE_EVENT_ITEM", SimStruct), offset=0)),)), name="HTTP_TRACE_EVENT", pack=False, align=None),
    "HTTP_TRACE_EVENT_ITEM": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwDataType", SimTypeInt(signed=False, label="HTTP_TRACE_TYPE")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pszDataDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="HTTP_TRACE_EVENT_ITEM", pack=False, align=None),
    "HTTP_TRACE_TYPE": SimTypeInt(signed=False, label="HTTP_TRACE_TYPE"),
    "HTTP_TRANSPORT_ADDRESS": SimStruct(OrderedDict((("pRemoteAddress", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("pLocalAddress", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)),)), name="HTTP_TRANSPORT_ADDRESS", pack=False, align=None),
    "HTTP_UNKNOWN_HEADER": SimStruct(OrderedDict((("NameLength", SimTypeShort(signed=False, label="UInt16")), ("RawValueLength", SimTypeShort(signed=False, label="UInt16")), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pRawValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="HTTP_UNKNOWN_HEADER", pack=False, align=None),
    "HTTP_VERB": SimTypeInt(signed=False, label="HTTP_VERB"),
    "HTTP_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="HTTP_VERSION", pack=False, align=None),
    "HTTP_VERSION_INFO": SimStruct(OrderedDict((("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_VERSION_INFO", pack=False, align=None),
    "HTTP_WEB_SOCKET_ASYNC_RESULT": SimStruct(OrderedDict((("AsyncResult", SimTypeRef("INTERNET_ASYNC_RESULT", SimStruct)), ("Operation", SimTypeInt(signed=False, label="HTTP_WEB_SOCKET_OPERATION")), ("BufferType", SimTypeInt(signed=False, label="HTTP_WEB_SOCKET_BUFFER_TYPE")), ("dwBytesTransferred", SimTypeInt(signed=False, label="UInt32")),)), name="HTTP_WEB_SOCKET_ASYNC_RESULT", pack=False, align=None),
    "HTTP_WEB_SOCKET_BUFFER_TYPE": SimTypeInt(signed=False, label="HTTP_WEB_SOCKET_BUFFER_TYPE"),
    "HTTP_WEB_SOCKET_CLOSE_STATUS": SimTypeInt(signed=False, label="HTTP_WEB_SOCKET_CLOSE_STATUS"),
    "HTTP_WEB_SOCKET_OPERATION": SimTypeInt(signed=False, label="HTTP_WEB_SOCKET_OPERATION"),
    "HTTP_WSK_API_TIMINGS": SimStruct(OrderedDict((("ConnectCount", SimTypeLongLong(signed=False, label="UInt64")), ("ConnectSum", SimTypeLongLong(signed=False, label="UInt64")), ("DisconnectCount", SimTypeLongLong(signed=False, label="UInt64")), ("DisconnectSum", SimTypeLongLong(signed=False, label="UInt64")), ("SendCount", SimTypeLongLong(signed=False, label="UInt64")), ("SendSum", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveCount", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveSum", SimTypeLongLong(signed=False, label="UInt64")), ("ReleaseCount", SimTypeLongLong(signed=False, label="UInt64")), ("ReleaseSum", SimTypeLongLong(signed=False, label="UInt64")), ("ControlSocketCount", SimTypeLongLong(signed=False, label="UInt64")), ("ControlSocketSum", SimTypeLongLong(signed=False, label="UInt64")),)), name="HTTP_WSK_API_TIMINGS", pack=False, align=None),
    "HT_OPTIONS": SimTypeInt(signed=False, label="HT_OPTIONS"),
    "HT_RESULTS": SimTypeInt(signed=False, label="HT_RESULTS"),
    "HUB_DEVICE_CONFIG_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("HubFlags", SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("HardwareIds", SimTypeRef("USB_ID_STRING", SimStruct)), ("CompatibleIds", SimTypeRef("USB_ID_STRING", SimStruct)), ("DeviceDescription", SimTypeRef("USB_ID_STRING", SimStruct)), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 19)), ("UxdSettings", SimTypeRef("USB_HUB_DEVICE_UXD_SETTINGS", SimStruct)),)), name="HUB_DEVICE_CONFIG_INFO", pack=False, align=None),
    "HUIAEVENT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HUIANODE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HUIAPATTERNOBJECT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HUIATEXTRANGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HUMAN_PRESENCE_DETECTION_TYPE": SimTypeInt(signed=False, label="HUMAN_PRESENCE_DETECTION_TYPE"),
    "HUMAN_PRESENCE_DETECTION_TYPE_COUNT": SimTypeInt(signed=False, label="HUMAN_PRESENCE_DETECTION_TYPE_COUNT"),
    "HUMPD": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HVIDEO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HVL_WHEA_ERROR_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WHEA_RECOVERY_CONTEXT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["RecoveryContext", "PlatformDirected", "Poisoned"]), offset=0),
    "HVSOCKET_ADDRESS_INFO": SimStruct(OrderedDict((("SystemId", SimTypeBottom(label="Guid")), ("VirtualMachineId", SimTypeBottom(label="Guid")), ("SiloId", SimTypeBottom(label="Guid")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="HVSOCKET_ADDRESS_INFO", pack=False, align=None),
    "HWAVE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWAVEIN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWAVEOUT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWINEVENTHOOK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWINSTA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWINWATCH": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWND": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HWPROFILEINFO_A": SimStruct(OrderedDict((("HWPI_ulHWProfile", SimTypeInt(signed=False, label="UInt32")), ("HWPI_szFriendlyName", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("HWPI_dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HWPROFILEINFO_A", pack=False, align=None),
    "HWPROFILEINFO_W": SimStruct(OrderedDict((("HWPI_ulHWProfile", SimTypeInt(signed=False, label="UInt32")), ("HWPI_szFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 80)), ("HWPI_dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="HWPROFILEINFO_W", pack=False, align=None),
    "HWPROFILE_CHANGE_NOTIFICATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Event", SimTypeBottom(label="Guid")),)), name="HWPROFILE_CHANGE_NOTIFICATION", pack=False, align=None),
    "HW_PROFILE_INFOA": SimStruct(OrderedDict((("dwDockInfo", SimTypeInt(signed=False, label="UInt32")), ("szHwProfileGuid", SimTypeArray(SimTypeChar(label="SByte"), 39)), ("szHwProfileName", SimTypeArray(SimTypeChar(label="SByte"), 80)),)), name="HW_PROFILE_INFOA", pack=False, align=None),
    "HW_PROFILE_INFOW": SimStruct(OrderedDict((("dwDockInfo", SimTypeInt(signed=False, label="UInt32")), ("szHwProfileGuid", SimTypeArray(SimTypeChar(label="Char"), 39)), ("szHwProfileName", SimTypeArray(SimTypeChar(label="Char"), 80)),)), name="HW_PROFILE_INFOW", pack=False, align=None),
    "HYBRID_DEMOTE_BY_SIZE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SourcePriority", SimTypeChar(label="Byte")), ("TargetPriority", SimTypeChar(label="Byte")), ("Reserved0", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("LbaCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="HYBRID_DEMOTE_BY_SIZE", pack=False, align=None),
    "HYBRID_DIRTY_THRESHOLDS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DirtyLowThreshold", SimTypeInt(signed=False, label="UInt32")), ("DirtyHighThreshold", SimTypeInt(signed=False, label="UInt32")),)), name="HYBRID_DIRTY_THRESHOLDS", pack=False, align=None),
    "HYBRID_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("HybridSupported", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=False, label="NVCACHE_STATUS")), ("CacheTypeEffective", SimTypeInt(signed=False, label="NVCACHE_TYPE")), ("CacheTypeDefault", SimTypeInt(signed=False, label="NVCACHE_TYPE")), ("FractionBase", SimTypeInt(signed=False, label="UInt32")), ("CacheSize", SimTypeLongLong(signed=False, label="UInt64")), ("Attributes", SimTypeRef("_Attributes_e__Struct", SimStruct)), ("Priorities", SimTypeRef("_Priorities_e__Struct", SimStruct)),)), name="HYBRID_INFORMATION", pack=False, align=None),
    "HYBRID_REQUEST_BLOCK": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DataBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("DataBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="HYBRID_REQUEST_BLOCK", pack=False, align=None),
    "HYPERLINKSTATES": SimTypeInt(signed=False, label="HYPERLINKSTATES"),
    "HYPERLINKTEXTSTATES": SimTypeInt(signed=False, label="HYPERLINKTEXTSTATES"),
    "HYPER_SIZEDARR": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="HYPER_SIZEDARR", pack=False, align=None),
    "HYPHENATEINFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=True, label="Int16")), ("dxHyphenateZone", SimTypeShort(signed=True, label="Int16")), ("pfnHyphenate", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="HYPHENATEINFO", pack=False, align=None),
    "HYPHRESULT": SimStruct(OrderedDict((("khyph", SimTypeInt(signed=False, label="KHYPH")), ("ichHyph", SimTypeInt(signed=True, label="Int32")), ("chHyph", SimTypeChar(label="Char")),)), name="HYPHRESULT", pack=False, align=None),
    "HYPOTHESIS": SimStruct(OrderedDict((("pwszClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("celt", SimTypeInt(signed=False, label="UInt32")), ("rgAttributes", SimTypePointer(SimTypeRef("HELPER_ATTRIBUTE", SimStruct), offset=0)),)), name="HYPOTHESIS", pack=False, align=None),
    "HandleType": SimTypeInt(signed=False, label="HandleType"),
    "HandleUnion": SimUnion({"Handle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Index": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "HatchStyle": SimTypeInt(signed=False, label="HatchStyle"),
    "HelperAttributeInfo": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("type", SimTypeInt(signed=False, label="ATTRIBUTE_TYPE")),)), name="HelperAttributeInfo", pack=False, align=None),
    "HiFiCOLOR": SimStruct(OrderedDict((("channel", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="HiFiCOLOR", pack=False, align=None),
    "HierarchyAlpha": SimTypeInt(signed=False, label="HierarchyAlpha"),
    "HistogramBlobHeader": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Histograms", SimTypeInt(signed=False, label="UInt32")),)), name="HistogramBlobHeader", pack=False, align=None),
    "HistogramDataHeader": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ChannelMask", SimTypeInt(signed=False, label="UInt32")), ("Linear", SimTypeInt(signed=False, label="UInt32")),)), name="HistogramDataHeader", pack=False, align=None),
    "HistogramFormat": SimTypeInt(signed=False, label="HistogramFormat"),
    "HistogramGrid": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Region", SimTypeRef("RECT", SimStruct)),)), name="HistogramGrid", pack=False, align=None),
    "HistogramHeader": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Bins", SimTypeInt(signed=False, label="UInt32")), ("FourCC", SimTypeInt(signed=False, label="UInt32")), ("ChannelMasks", SimTypeInt(signed=False, label="UInt32")), ("Grid", SimTypeRef("HistogramGrid", SimStruct)),)), name="HistogramHeader", pack=False, align=None),
    "HorizontalTextAlignment": SimTypeInt(signed=False, label="HorizontalTextAlignment"),
    "HostDialogHelper": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "HotkeyPrefix": SimTypeInt(signed=False, label="HotkeyPrefix"),
    "HrtfApoInit": SimStruct(OrderedDict((("distanceDecay", SimTypePointer(SimTypeRef("HrtfDistanceDecay", SimStruct), offset=0)), ("directivity", SimTypePointer(SimTypeRef("HrtfDirectivity", SimStruct), offset=0)),)), name="HrtfApoInit", pack=False, align=None),
    "HrtfDirectivity": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="HrtfDirectivityType")), ("scaling", SimTypeFloat(size=32)),)), name="HrtfDirectivity", pack=False, align=None),
    "HrtfDirectivityCardioid": SimStruct(OrderedDict((("directivity", SimTypeRef("HrtfDirectivity", SimStruct)), ("order", SimTypeFloat(size=32)),)), name="HrtfDirectivityCardioid", pack=False, align=None),
    "HrtfDirectivityCone": SimStruct(OrderedDict((("directivity", SimTypeRef("HrtfDirectivity", SimStruct)), ("innerAngle", SimTypeFloat(size=32)), ("outerAngle", SimTypeFloat(size=32)),)), name="HrtfDirectivityCone", pack=False, align=None),
    "HrtfDirectivityType": SimTypeInt(signed=False, label="HrtfDirectivityType"),
    "HrtfDistanceDecay": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="HrtfDistanceDecayType")), ("maxGain", SimTypeFloat(size=32)), ("minGain", SimTypeFloat(size=32)), ("unityGainDistance", SimTypeFloat(size=32)), ("cutoffDistance", SimTypeFloat(size=32)),)), name="HrtfDistanceDecay", pack=False, align=None),
    "HrtfDistanceDecayType": SimTypeInt(signed=False, label="HrtfDistanceDecayType"),
    "HrtfEnvironment": SimTypeInt(signed=False, label="HrtfEnvironment"),
    "HrtfOrientation": SimStruct(OrderedDict((("element", SimTypeArray(SimTypeFloat(size=32), 9)),)), name="HrtfOrientation", pack=False, align=None),
    "HrtfPosition": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("z", SimTypeFloat(size=32)),)), name="HrtfPosition", pack=False, align=None),
    "HueSaturationLightness": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="HueSaturationLightness", pack=False, align=None),
    "HueSaturationLightnessParams": SimStruct(OrderedDict((("hueLevel", SimTypeInt(signed=True, label="Int32")), ("saturationLevel", SimTypeInt(signed=True, label="Int32")), ("lightnessLevel", SimTypeInt(signed=True, label="Int32")),)), name="HueSaturationLightnessParams", pack=False, align=None),
    "HypothesisResult": SimStruct(OrderedDict((("hypothesis", SimTypeRef("HYPOTHESIS", SimStruct)), ("pathStatus", SimTypeInt(signed=False, label="DIAGNOSIS_STATUS")),)), name="HypothesisResult", pack=False, align=None),
    "IABContainer": SimTypeBottom(label="IABContainer"),
    "IACList": SimTypeBottom(label="IACList"),
    "IACList2": SimTypeBottom(label="IACList2"),
    "IADMEXT": SimTypeBottom(label="IADMEXT"),
    "IADesktopP2": SimTypeBottom(label="IADesktopP2"),
    "IADs": SimTypeBottom(label="IADs"),
    "IADsADSystemInfo": SimTypeBottom(label="IADsADSystemInfo"),
    "IADsAccessControlEntry": SimTypeBottom(label="IADsAccessControlEntry"),
    "IADsAccessControlList": SimTypeBottom(label="IADsAccessControlList"),
    "IADsAcl": SimTypeBottom(label="IADsAcl"),
    "IADsAggregatee": SimTypeBottom(label="IADsAggregatee"),
    "IADsAggregator": SimTypeBottom(label="IADsAggregator"),
    "IADsBackLink": SimTypeBottom(label="IADsBackLink"),
    "IADsCaseIgnoreList": SimTypeBottom(label="IADsCaseIgnoreList"),
    "IADsClass": SimTypeBottom(label="IADsClass"),
    "IADsCollection": SimTypeBottom(label="IADsCollection"),
    "IADsComputer": SimTypeBottom(label="IADsComputer"),
    "IADsComputerOperations": SimTypeBottom(label="IADsComputerOperations"),
    "IADsContainer": SimTypeBottom(label="IADsContainer"),
    "IADsDNWithBinary": SimTypeBottom(label="IADsDNWithBinary"),
    "IADsDNWithString": SimTypeBottom(label="IADsDNWithString"),
    "IADsDeleteOps": SimTypeBottom(label="IADsDeleteOps"),
    "IADsDomain": SimTypeBottom(label="IADsDomain"),
    "IADsEmail": SimTypeBottom(label="IADsEmail"),
    "IADsExtension": SimTypeBottom(label="IADsExtension"),
    "IADsFaxNumber": SimTypeBottom(label="IADsFaxNumber"),
    "IADsFileService": SimTypeBottom(label="IADsFileService"),
    "IADsFileServiceOperations": SimTypeBottom(label="IADsFileServiceOperations"),
    "IADsFileShare": SimTypeBottom(label="IADsFileShare"),
    "IADsGroup": SimTypeBottom(label="IADsGroup"),
    "IADsHold": SimTypeBottom(label="IADsHold"),
    "IADsLargeInteger": SimTypeBottom(label="IADsLargeInteger"),
    "IADsLocality": SimTypeBottom(label="IADsLocality"),
    "IADsMembers": SimTypeBottom(label="IADsMembers"),
    "IADsNameTranslate": SimTypeBottom(label="IADsNameTranslate"),
    "IADsNamespaces": SimTypeBottom(label="IADsNamespaces"),
    "IADsNetAddress": SimTypeBottom(label="IADsNetAddress"),
    "IADsO": SimTypeBottom(label="IADsO"),
    "IADsOU": SimTypeBottom(label="IADsOU"),
    "IADsObjectOptions": SimTypeBottom(label="IADsObjectOptions"),
    "IADsOctetList": SimTypeBottom(label="IADsOctetList"),
    "IADsOpenDSObject": SimTypeBottom(label="IADsOpenDSObject"),
    "IADsPath": SimTypeBottom(label="IADsPath"),
    "IADsPathname": SimTypeBottom(label="IADsPathname"),
    "IADsPostalAddress": SimTypeBottom(label="IADsPostalAddress"),
    "IADsPrintJob": SimTypeBottom(label="IADsPrintJob"),
    "IADsPrintJobOperations": SimTypeBottom(label="IADsPrintJobOperations"),
    "IADsPrintQueue": SimTypeBottom(label="IADsPrintQueue"),
    "IADsPrintQueueOperations": SimTypeBottom(label="IADsPrintQueueOperations"),
    "IADsProperty": SimTypeBottom(label="IADsProperty"),
    "IADsPropertyEntry": SimTypeBottom(label="IADsPropertyEntry"),
    "IADsPropertyList": SimTypeBottom(label="IADsPropertyList"),
    "IADsPropertyValue": SimTypeBottom(label="IADsPropertyValue"),
    "IADsPropertyValue2": SimTypeBottom(label="IADsPropertyValue2"),
    "IADsReplicaPointer": SimTypeBottom(label="IADsReplicaPointer"),
    "IADsResource": SimTypeBottom(label="IADsResource"),
    "IADsSecurityDescriptor": SimTypeBottom(label="IADsSecurityDescriptor"),
    "IADsSecurityUtility": SimTypeBottom(label="IADsSecurityUtility"),
    "IADsService": SimTypeBottom(label="IADsService"),
    "IADsServiceOperations": SimTypeBottom(label="IADsServiceOperations"),
    "IADsSession": SimTypeBottom(label="IADsSession"),
    "IADsSyntax": SimTypeBottom(label="IADsSyntax"),
    "IADsTSUserEx": SimTypeBottom(label="IADsTSUserEx"),
    "IADsTimestamp": SimTypeBottom(label="IADsTimestamp"),
    "IADsTypedName": SimTypeBottom(label="IADsTypedName"),
    "IADsUser": SimTypeBottom(label="IADsUser"),
    "IADsWinNTSystemInfo": SimTypeBottom(label="IADsWinNTSystemInfo"),
    "IAMAnalogVideoDecoder": SimTypeBottom(label="IAMAnalogVideoDecoder"),
    "IAMAnalogVideoEncoder": SimTypeBottom(label="IAMAnalogVideoEncoder"),
    "IAMAsyncReaderTimestampScaling": SimTypeBottom(label="IAMAsyncReaderTimestampScaling"),
    "IAMAudioInputMixer": SimTypeBottom(label="IAMAudioInputMixer"),
    "IAMAudioRendererStats": SimTypeBottom(label="IAMAudioRendererStats"),
    "IAMBufferNegotiation": SimTypeBottom(label="IAMBufferNegotiation"),
    "IAMCameraControl": SimTypeBottom(label="IAMCameraControl"),
    "IAMCertifiedOutputProtection": SimTypeBottom(label="IAMCertifiedOutputProtection"),
    "IAMChannelInfo": SimTypeBottom(label="IAMChannelInfo"),
    "IAMClockAdjust": SimTypeBottom(label="IAMClockAdjust"),
    "IAMClockSlave": SimTypeBottom(label="IAMClockSlave"),
    "IAMCollection": SimTypeBottom(label="IAMCollection"),
    "IAMCopyCaptureFileProgress": SimTypeBottom(label="IAMCopyCaptureFileProgress"),
    "IAMCrossbar": SimTypeBottom(label="IAMCrossbar"),
    "IAMDecoderCaps": SimTypeBottom(label="IAMDecoderCaps"),
    "IAMDevMemoryAllocator": SimTypeBottom(label="IAMDevMemoryAllocator"),
    "IAMDevMemoryControl": SimTypeBottom(label="IAMDevMemoryControl"),
    "IAMDeviceRemoval": SimTypeBottom(label="IAMDeviceRemoval"),
    "IAMDirectSound": SimTypeBottom(label="IAMDirectSound"),
    "IAMDroppedFrames": SimTypeBottom(label="IAMDroppedFrames"),
    "IAMExtDevice": SimTypeBottom(label="IAMExtDevice"),
    "IAMExtTransport": SimTypeBottom(label="IAMExtTransport"),
    "IAMExtendedErrorInfo": SimTypeBottom(label="IAMExtendedErrorInfo"),
    "IAMExtendedSeeking": SimTypeBottom(label="IAMExtendedSeeking"),
    "IAMFilterGraphCallback": SimTypeBottom(label="IAMFilterGraphCallback"),
    "IAMFilterMiscFlags": SimTypeBottom(label="IAMFilterMiscFlags"),
    "IAMGraphBuilderCallback": SimTypeBottom(label="IAMGraphBuilderCallback"),
    "IAMGraphStreams": SimTypeBottom(label="IAMGraphStreams"),
    "IAMLatency": SimTypeBottom(label="IAMLatency"),
    "IAMLine21Decoder": SimTypeBottom(label="IAMLine21Decoder"),
    "IAMMediaContent": SimTypeBottom(label="IAMMediaContent"),
    "IAMMediaContent2": SimTypeBottom(label="IAMMediaContent2"),
    "IAMMediaStream": SimTypeBottom(label="IAMMediaStream"),
    "IAMMediaTypeSample": SimTypeBottom(label="IAMMediaTypeSample"),
    "IAMMediaTypeStream": SimTypeBottom(label="IAMMediaTypeStream"),
    "IAMMultiMediaStream": SimTypeBottom(label="IAMMultiMediaStream"),
    "IAMNetShowConfig": SimTypeBottom(label="IAMNetShowConfig"),
    "IAMNetShowExProps": SimTypeBottom(label="IAMNetShowExProps"),
    "IAMNetShowPreroll": SimTypeBottom(label="IAMNetShowPreroll"),
    "IAMNetworkStatus": SimTypeBottom(label="IAMNetworkStatus"),
    "IAMOpenProgress": SimTypeBottom(label="IAMOpenProgress"),
    "IAMOverlayFX": SimTypeBottom(label="IAMOverlayFX"),
    "IAMParse": SimTypeBottom(label="IAMParse"),
    "IAMPhysicalPinInfo": SimTypeBottom(label="IAMPhysicalPinInfo"),
    "IAMPlayList": SimTypeBottom(label="IAMPlayList"),
    "IAMPlayListItem": SimTypeBottom(label="IAMPlayListItem"),
    "IAMPluginControl": SimTypeBottom(label="IAMPluginControl"),
    "IAMPushSource": SimTypeBottom(label="IAMPushSource"),
    "IAMRebuild": SimTypeBottom(label="IAMRebuild"),
    "IAMResourceControl": SimTypeBottom(label="IAMResourceControl"),
    "IAMStats": SimTypeBottom(label="IAMStats"),
    "IAMStreamConfig": SimTypeBottom(label="IAMStreamConfig"),
    "IAMStreamControl": SimTypeBottom(label="IAMStreamControl"),
    "IAMStreamSelect": SimTypeBottom(label="IAMStreamSelect"),
    "IAMTVAudio": SimTypeBottom(label="IAMTVAudio"),
    "IAMTVAudioNotification": SimTypeBottom(label="IAMTVAudioNotification"),
    "IAMTVTuner": SimTypeBottom(label="IAMTVTuner"),
    "IAMTimecodeDisplay": SimTypeBottom(label="IAMTimecodeDisplay"),
    "IAMTimecodeGenerator": SimTypeBottom(label="IAMTimecodeGenerator"),
    "IAMTimecodeReader": SimTypeBottom(label="IAMTimecodeReader"),
    "IAMTuner": SimTypeBottom(label="IAMTuner"),
    "IAMTunerNotification": SimTypeBottom(label="IAMTunerNotification"),
    "IAMVfwCaptureDialogs": SimTypeBottom(label="IAMVfwCaptureDialogs"),
    "IAMVfwCompressDialogs": SimTypeBottom(label="IAMVfwCompressDialogs"),
    "IAMVideoAccelerator": SimTypeBottom(label="IAMVideoAccelerator"),
    "IAMVideoAcceleratorNotify": SimTypeBottom(label="IAMVideoAcceleratorNotify"),
    "IAMVideoCompression": SimTypeBottom(label="IAMVideoCompression"),
    "IAMVideoControl": SimTypeBottom(label="IAMVideoControl"),
    "IAMVideoDecimationProperties": SimTypeBottom(label="IAMVideoDecimationProperties"),
    "IAMVideoProcAmp": SimTypeBottom(label="IAMVideoProcAmp"),
    "IAMWMBufferPass": SimTypeBottom(label="IAMWMBufferPass"),
    "IAMWMBufferPassCallback": SimTypeBottom(label="IAMWMBufferPassCallback"),
    "IAMWstDecoder": SimTypeBottom(label="IAMWstDecoder"),
    "IAMovieSetup": SimTypeBottom(label="IAMovieSetup"),
    "IASCOMMONPROPERTIES": SimTypeInt(signed=False, label="IASCOMMONPROPERTIES"),
    "IASCOMPONENTPROPERTIES": SimTypeInt(signed=False, label="IASCOMPONENTPROPERTIES"),
    "IASDATASTORE": SimTypeInt(signed=False, label="IASDATASTORE"),
    "IASDOMAINTYPE": SimTypeInt(signed=False, label="IASDOMAINTYPE"),
    "IASOSTYPE": SimTypeInt(signed=False, label="IASOSTYPE"),
    "IASPROPERTIES": SimTypeInt(signed=False, label="IASPROPERTIES"),
    "IASSEMBLYCACHE_UNINSTALL_DISPOSITION": SimTypeInt(signed=False, label="IASSEMBLYCACHE_UNINSTALL_DISPOSITION"),
    "IATSCChannelTuneRequest": SimTypeBottom(label="IATSCChannelTuneRequest"),
    "IATSCComponentType": SimTypeBottom(label="IATSCComponentType"),
    "IATSCLocator": SimTypeBottom(label="IATSCLocator"),
    "IATSCLocator2": SimTypeBottom(label="IATSCLocator2"),
    "IATSCTuningSpace": SimTypeBottom(label="IATSCTuningSpace"),
    "IATSC_EIT": SimTypeBottom(label="IATSC_EIT"),
    "IATSC_ETT": SimTypeBottom(label="IATSC_ETT"),
    "IATSC_MGT": SimTypeBottom(label="IATSC_MGT"),
    "IATSC_STT": SimTypeBottom(label="IATSC_STT"),
    "IATSC_VCT": SimTypeBottom(label="IATSC_VCT"),
    "IAVIEditStream": SimTypeBottom(label="IAVIEditStream"),
    "IAVIFile": SimTypeBottom(label="IAVIFile"),
    "IAVIPersistFile": SimTypeBottom(label="IAVIPersistFile"),
    "IAVIStream": SimTypeBottom(label="IAVIStream"),
    "IAVIStreaming": SimTypeBottom(label="IAVIStreaming"),
    "IAccClientDocMgr": SimTypeBottom(label="IAccClientDocMgr"),
    "IAccDictionary": SimTypeBottom(label="IAccDictionary"),
    "IAccIdentity": SimTypeBottom(label="IAccIdentity"),
    "IAccPropServer": SimTypeBottom(label="IAccPropServer"),
    "IAccPropServices": SimTypeBottom(label="IAccPropServices"),
    "IAccServerDocMgr": SimTypeBottom(label="IAccServerDocMgr"),
    "IAccStore": SimTypeBottom(label="IAccStore"),
    "IAccessibilityDockingService": SimTypeBottom(label="IAccessibilityDockingService"),
    "IAccessibilityDockingServiceCallback": SimTypeBottom(label="IAccessibilityDockingServiceCallback"),
    "IAccessible": SimTypeBottom(label="IAccessible"),
    "IAccessibleEx": SimTypeBottom(label="IAccessibleEx"),
    "IAccessibleHandler": SimTypeBottom(label="IAccessibleHandler"),
    "IAccessibleHostingElementProviders": SimTypeBottom(label="IAccessibleHostingElementProviders"),
    "IAccessibleObject": SimTypeBottom(label="IAccessibleObject"),
    "IAccessibleWinSAT": SimTypeBottom(label="IAccessibleWinSAT"),
    "IAccessibleWindowlessSite": SimTypeBottom(label="IAccessibleWindowlessSite"),
    "IAccessor": SimTypeBottom(label="IAccessor"),
    "IAccountingProviderConfig": SimTypeBottom(label="IAccountingProviderConfig"),
    "IAcousticEchoCancellationControl": SimTypeBottom(label="IAcousticEchoCancellationControl"),
    "IAction": SimTypeBottom(label="IAction"),
    "IActionCollection": SimTypeBottom(label="IActionCollection"),
    "IActionOnCLREvent": SimTypeBottom(label="IActionOnCLREvent"),
    "IActionProgress": SimTypeBottom(label="IActionProgress"),
    "IActionProgressDialog": SimTypeBottom(label="IActionProgressDialog"),
    "IActivateAudioInterfaceAsyncOperation": SimTypeBottom(label="IActivateAudioInterfaceAsyncOperation"),
    "IActivateAudioInterfaceCompletionHandler": SimTypeBottom(label="IActivateAudioInterfaceCompletionHandler"),
    "IActivationFilter": SimTypeBottom(label="IActivationFilter"),
    "IActiveDesktopP": SimTypeBottom(label="IActiveDesktopP"),
    "IActiveIME": SimTypeBottom(label="IActiveIME"),
    "IActiveIME2": SimTypeBottom(label="IActiveIME2"),
    "IActiveIMMApp": SimTypeBottom(label="IActiveIMMApp"),
    "IActiveIMMIME": SimTypeBottom(label="IActiveIMMIME"),
    "IActiveIMMMessagePumpOwner": SimTypeBottom(label="IActiveIMMMessagePumpOwner"),
    "IActiveIMMRegistrar": SimTypeBottom(label="IActiveIMMRegistrar"),
    "IActiveScript": SimTypeBottom(label="IActiveScript"),
    "IActiveScriptAuthor": SimTypeBottom(label="IActiveScriptAuthor"),
    "IActiveScriptAuthorProcedure": SimTypeBottom(label="IActiveScriptAuthorProcedure"),
    "IActiveScriptDebug32": SimTypeBottom(label="IActiveScriptDebug32"),
    "IActiveScriptDebug64": SimTypeBottom(label="IActiveScriptDebug64"),
    "IActiveScriptEncode": SimTypeBottom(label="IActiveScriptEncode"),
    "IActiveScriptError": SimTypeBottom(label="IActiveScriptError"),
    "IActiveScriptError64": SimTypeBottom(label="IActiveScriptError64"),
    "IActiveScriptErrorDebug": SimTypeBottom(label="IActiveScriptErrorDebug"),
    "IActiveScriptErrorDebug110": SimTypeBottom(label="IActiveScriptErrorDebug110"),
    "IActiveScriptGarbageCollector": SimTypeBottom(label="IActiveScriptGarbageCollector"),
    "IActiveScriptHostEncode": SimTypeBottom(label="IActiveScriptHostEncode"),
    "IActiveScriptParse32": SimTypeBottom(label="IActiveScriptParse32"),
    "IActiveScriptParse64": SimTypeBottom(label="IActiveScriptParse64"),
    "IActiveScriptParseProcedure2_32": SimTypeBottom(label="IActiveScriptParseProcedure2_32"),
    "IActiveScriptParseProcedure2_64": SimTypeBottom(label="IActiveScriptParseProcedure2_64"),
    "IActiveScriptParseProcedure32": SimTypeBottom(label="IActiveScriptParseProcedure32"),
    "IActiveScriptParseProcedure64": SimTypeBottom(label="IActiveScriptParseProcedure64"),
    "IActiveScriptParseProcedureOld32": SimTypeBottom(label="IActiveScriptParseProcedureOld32"),
    "IActiveScriptParseProcedureOld64": SimTypeBottom(label="IActiveScriptParseProcedureOld64"),
    "IActiveScriptProfilerCallback": SimTypeBottom(label="IActiveScriptProfilerCallback"),
    "IActiveScriptProfilerCallback2": SimTypeBottom(label="IActiveScriptProfilerCallback2"),
    "IActiveScriptProfilerCallback3": SimTypeBottom(label="IActiveScriptProfilerCallback3"),
    "IActiveScriptProfilerControl": SimTypeBottom(label="IActiveScriptProfilerControl"),
    "IActiveScriptProfilerControl2": SimTypeBottom(label="IActiveScriptProfilerControl2"),
    "IActiveScriptProfilerControl3": SimTypeBottom(label="IActiveScriptProfilerControl3"),
    "IActiveScriptProfilerControl4": SimTypeBottom(label="IActiveScriptProfilerControl4"),
    "IActiveScriptProfilerControl5": SimTypeBottom(label="IActiveScriptProfilerControl5"),
    "IActiveScriptProfilerHeapEnum": SimTypeBottom(label="IActiveScriptProfilerHeapEnum"),
    "IActiveScriptProperty": SimTypeBottom(label="IActiveScriptProperty"),
    "IActiveScriptSIPInfo": SimTypeBottom(label="IActiveScriptSIPInfo"),
    "IActiveScriptSite": SimTypeBottom(label="IActiveScriptSite"),
    "IActiveScriptSiteDebug32": SimTypeBottom(label="IActiveScriptSiteDebug32"),
    "IActiveScriptSiteDebug64": SimTypeBottom(label="IActiveScriptSiteDebug64"),
    "IActiveScriptSiteDebugEx": SimTypeBottom(label="IActiveScriptSiteDebugEx"),
    "IActiveScriptSiteInterruptPoll": SimTypeBottom(label="IActiveScriptSiteInterruptPoll"),
    "IActiveScriptSiteTraceInfo": SimTypeBottom(label="IActiveScriptSiteTraceInfo"),
    "IActiveScriptSiteUIControl": SimTypeBottom(label="IActiveScriptSiteUIControl"),
    "IActiveScriptSiteWindow": SimTypeBottom(label="IActiveScriptSiteWindow"),
    "IActiveScriptStats": SimTypeBottom(label="IActiveScriptStats"),
    "IActiveScriptStringCompare": SimTypeBottom(label="IActiveScriptStringCompare"),
    "IActiveScriptTraceInfo": SimTypeBottom(label="IActiveScriptTraceInfo"),
    "IActiveScriptWinRTErrorDebug": SimTypeBottom(label="IActiveScriptWinRTErrorDebug"),
    "IActiveXUIHandlerSite": SimTypeBottom(label="IActiveXUIHandlerSite"),
    "IActiveXUIHandlerSite2": SimTypeBottom(label="IActiveXUIHandlerSite2"),
    "IActiveXUIHandlerSite3": SimTypeBottom(label="IActiveXUIHandlerSite3"),
    "IAddrBook": SimTypeBottom(label="IAddrBook"),
    "IAddrExclusionControl": SimTypeBottom(label="IAddrExclusionControl"),
    "IAddrTrackingControl": SimTypeBottom(label="IAddrTrackingControl"),
    "IAdvancedMediaCapture": SimTypeBottom(label="IAdvancedMediaCapture"),
    "IAdvancedMediaCaptureInitializationSettings": SimTypeBottom(label="IAdvancedMediaCaptureInitializationSettings"),
    "IAdvancedMediaCaptureSettings": SimTypeBottom(label="IAdvancedMediaCaptureSettings"),
    "IAdviseSink": SimTypeBottom(label="IAdviseSink"),
    "IAdviseSink2": SimTypeBottom(label="IAdviseSink2"),
    "IAdviseSinkEx": SimTypeBottom(label="IAdviseSinkEx"),
    "IAgileObject": SimTypeBottom(label="IAgileObject"),
    "IAlertDataCollector": SimTypeBottom(label="IAlertDataCollector"),
    "IAlterIndex": SimTypeBottom(label="IAlterIndex"),
    "IAlterTable": SimTypeBottom(label="IAlterTable"),
    "IAlternativeName": SimTypeBottom(label="IAlternativeName"),
    "IAlternativeNames": SimTypeBottom(label="IAlternativeNames"),
    "IAmsiStream": SimTypeBottom(label="IAmsiStream"),
    "IAnalogAudioComponentType": SimTypeBottom(label="IAnalogAudioComponentType"),
    "IAnalogLocator": SimTypeBottom(label="IAnalogLocator"),
    "IAnalogRadioTuningSpace": SimTypeBottom(label="IAnalogRadioTuningSpace"),
    "IAnalogRadioTuningSpace2": SimTypeBottom(label="IAnalogRadioTuningSpace2"),
    "IAnalogTVTuningSpace": SimTypeBottom(label="IAnalogTVTuningSpace"),
    "IAnchor": SimTypeBottom(label="IAnchor"),
    "IAnchorClick": SimTypeBottom(label="IAnchorClick"),
    "IAnnotationProvider": SimTypeBottom(label="IAnnotationProvider"),
    "IAntimalware": SimTypeBottom(label="IAntimalware"),
    "IAntimalware2": SimTypeBottom(label="IAntimalware2"),
    "IAntimalwareProvider": SimTypeBottom(label="IAntimalwareProvider"),
    "IAntimalwareProvider2": SimTypeBottom(label="IAntimalwareProvider2"),
    "IAntimalwareUacProvider": SimTypeBottom(label="IAntimalwareUacProvider"),
    "IApartmentCallback": SimTypeBottom(label="IApartmentCallback"),
    "IApiTracingDataCollector": SimTypeBottom(label="IApiTracingDataCollector"),
    "IApoAcousticEchoCancellation": SimTypeBottom(label="IApoAcousticEchoCancellation"),
    "IApoAuxiliaryInputConfiguration": SimTypeBottom(label="IApoAuxiliaryInputConfiguration"),
    "IApoAuxiliaryInputRT": SimTypeBottom(label="IApoAuxiliaryInputRT"),
    "IAppActivationUIInfo": SimTypeBottom(label="IAppActivationUIInfo"),
    "IAppDomainBinding": SimTypeBottom(label="IAppDomainBinding"),
    "IAppDomainHelper": SimTypeBottom(label="IAppDomainHelper"),
    "IAppPublisher": SimTypeBottom(label="IAppPublisher"),
    "IAppVisibility": SimTypeBottom(label="IAppVisibility"),
    "IAppVisibilityEvents": SimTypeBottom(label="IAppVisibilityEvents"),
    "IApplicationActivationManager": SimTypeBottom(label="IApplicationActivationManager"),
    "IApplicationAssociationRegistration": SimTypeBottom(label="IApplicationAssociationRegistration"),
    "IApplicationAssociationRegistrationUI": SimTypeBottom(label="IApplicationAssociationRegistrationUI"),
    "IApplicationDebugger": SimTypeBottom(label="IApplicationDebugger"),
    "IApplicationDebuggerUI": SimTypeBottom(label="IApplicationDebuggerUI"),
    "IApplicationDesignModeSettings": SimTypeBottom(label="IApplicationDesignModeSettings"),
    "IApplicationDesignModeSettings2": SimTypeBottom(label="IApplicationDesignModeSettings2"),
    "IApplicationDestinations": SimTypeBottom(label="IApplicationDestinations"),
    "IApplicationDocumentLists": SimTypeBottom(label="IApplicationDocumentLists"),
    "IAppxAppInstallerReader": SimTypeBottom(label="IAppxAppInstallerReader"),
    "IAppxBlockMapBlock": SimTypeBottom(label="IAppxBlockMapBlock"),
    "IAppxBlockMapBlocksEnumerator": SimTypeBottom(label="IAppxBlockMapBlocksEnumerator"),
    "IAppxBlockMapFile": SimTypeBottom(label="IAppxBlockMapFile"),
    "IAppxBlockMapFilesEnumerator": SimTypeBottom(label="IAppxBlockMapFilesEnumerator"),
    "IAppxBlockMapReader": SimTypeBottom(label="IAppxBlockMapReader"),
    "IAppxBundleFactory": SimTypeBottom(label="IAppxBundleFactory"),
    "IAppxBundleFactory2": SimTypeBottom(label="IAppxBundleFactory2"),
    "IAppxBundleManifestOptionalBundleInfo": SimTypeBottom(label="IAppxBundleManifestOptionalBundleInfo"),
    "IAppxBundleManifestOptionalBundleInfoEnumerator": SimTypeBottom(label="IAppxBundleManifestOptionalBundleInfoEnumerator"),
    "IAppxBundleManifestPackageInfo": SimTypeBottom(label="IAppxBundleManifestPackageInfo"),
    "IAppxBundleManifestPackageInfo2": SimTypeBottom(label="IAppxBundleManifestPackageInfo2"),
    "IAppxBundleManifestPackageInfo3": SimTypeBottom(label="IAppxBundleManifestPackageInfo3"),
    "IAppxBundleManifestPackageInfo4": SimTypeBottom(label="IAppxBundleManifestPackageInfo4"),
    "IAppxBundleManifestPackageInfoEnumerator": SimTypeBottom(label="IAppxBundleManifestPackageInfoEnumerator"),
    "IAppxBundleManifestReader": SimTypeBottom(label="IAppxBundleManifestReader"),
    "IAppxBundleManifestReader2": SimTypeBottom(label="IAppxBundleManifestReader2"),
    "IAppxBundleReader": SimTypeBottom(label="IAppxBundleReader"),
    "IAppxBundleWriter": SimTypeBottom(label="IAppxBundleWriter"),
    "IAppxBundleWriter2": SimTypeBottom(label="IAppxBundleWriter2"),
    "IAppxBundleWriter3": SimTypeBottom(label="IAppxBundleWriter3"),
    "IAppxBundleWriter4": SimTypeBottom(label="IAppxBundleWriter4"),
    "IAppxContentGroup": SimTypeBottom(label="IAppxContentGroup"),
    "IAppxContentGroupFilesEnumerator": SimTypeBottom(label="IAppxContentGroupFilesEnumerator"),
    "IAppxContentGroupMapReader": SimTypeBottom(label="IAppxContentGroupMapReader"),
    "IAppxContentGroupMapWriter": SimTypeBottom(label="IAppxContentGroupMapWriter"),
    "IAppxContentGroupsEnumerator": SimTypeBottom(label="IAppxContentGroupsEnumerator"),
    "IAppxDigestProvider": SimTypeBottom(label="IAppxDigestProvider"),
    "IAppxEncryptedBundleWriter": SimTypeBottom(label="IAppxEncryptedBundleWriter"),
    "IAppxEncryptedBundleWriter2": SimTypeBottom(label="IAppxEncryptedBundleWriter2"),
    "IAppxEncryptedBundleWriter3": SimTypeBottom(label="IAppxEncryptedBundleWriter3"),
    "IAppxEncryptedPackageWriter": SimTypeBottom(label="IAppxEncryptedPackageWriter"),
    "IAppxEncryptedPackageWriter2": SimTypeBottom(label="IAppxEncryptedPackageWriter2"),
    "IAppxEncryptionFactory": SimTypeBottom(label="IAppxEncryptionFactory"),
    "IAppxEncryptionFactory2": SimTypeBottom(label="IAppxEncryptionFactory2"),
    "IAppxEncryptionFactory3": SimTypeBottom(label="IAppxEncryptionFactory3"),
    "IAppxEncryptionFactory4": SimTypeBottom(label="IAppxEncryptionFactory4"),
    "IAppxEncryptionFactory5": SimTypeBottom(label="IAppxEncryptionFactory5"),
    "IAppxFactory": SimTypeBottom(label="IAppxFactory"),
    "IAppxFactory2": SimTypeBottom(label="IAppxFactory2"),
    "IAppxFactory3": SimTypeBottom(label="IAppxFactory3"),
    "IAppxFile": SimTypeBottom(label="IAppxFile"),
    "IAppxFilesEnumerator": SimTypeBottom(label="IAppxFilesEnumerator"),
    "IAppxManifestApplication": SimTypeBottom(label="IAppxManifestApplication"),
    "IAppxManifestApplicationsEnumerator": SimTypeBottom(label="IAppxManifestApplicationsEnumerator"),
    "IAppxManifestCapabilitiesEnumerator": SimTypeBottom(label="IAppxManifestCapabilitiesEnumerator"),
    "IAppxManifestDeviceCapabilitiesEnumerator": SimTypeBottom(label="IAppxManifestDeviceCapabilitiesEnumerator"),
    "IAppxManifestDriverConstraint": SimTypeBottom(label="IAppxManifestDriverConstraint"),
    "IAppxManifestDriverConstraintsEnumerator": SimTypeBottom(label="IAppxManifestDriverConstraintsEnumerator"),
    "IAppxManifestDriverDependenciesEnumerator": SimTypeBottom(label="IAppxManifestDriverDependenciesEnumerator"),
    "IAppxManifestDriverDependency": SimTypeBottom(label="IAppxManifestDriverDependency"),
    "IAppxManifestHostRuntimeDependenciesEnumerator": SimTypeBottom(label="IAppxManifestHostRuntimeDependenciesEnumerator"),
    "IAppxManifestHostRuntimeDependency": SimTypeBottom(label="IAppxManifestHostRuntimeDependency"),
    "IAppxManifestHostRuntimeDependency2": SimTypeBottom(label="IAppxManifestHostRuntimeDependency2"),
    "IAppxManifestMainPackageDependenciesEnumerator": SimTypeBottom(label="IAppxManifestMainPackageDependenciesEnumerator"),
    "IAppxManifestMainPackageDependency": SimTypeBottom(label="IAppxManifestMainPackageDependency"),
    "IAppxManifestOSPackageDependenciesEnumerator": SimTypeBottom(label="IAppxManifestOSPackageDependenciesEnumerator"),
    "IAppxManifestOSPackageDependency": SimTypeBottom(label="IAppxManifestOSPackageDependency"),
    "IAppxManifestOptionalPackageInfo": SimTypeBottom(label="IAppxManifestOptionalPackageInfo"),
    "IAppxManifestPackageDependenciesEnumerator": SimTypeBottom(label="IAppxManifestPackageDependenciesEnumerator"),
    "IAppxManifestPackageDependency": SimTypeBottom(label="IAppxManifestPackageDependency"),
    "IAppxManifestPackageDependency2": SimTypeBottom(label="IAppxManifestPackageDependency2"),
    "IAppxManifestPackageDependency3": SimTypeBottom(label="IAppxManifestPackageDependency3"),
    "IAppxManifestPackageId": SimTypeBottom(label="IAppxManifestPackageId"),
    "IAppxManifestPackageId2": SimTypeBottom(label="IAppxManifestPackageId2"),
    "IAppxManifestProperties": SimTypeBottom(label="IAppxManifestProperties"),
    "IAppxManifestQualifiedResource": SimTypeBottom(label="IAppxManifestQualifiedResource"),
    "IAppxManifestQualifiedResourcesEnumerator": SimTypeBottom(label="IAppxManifestQualifiedResourcesEnumerator"),
    "IAppxManifestReader": SimTypeBottom(label="IAppxManifestReader"),
    "IAppxManifestReader2": SimTypeBottom(label="IAppxManifestReader2"),
    "IAppxManifestReader3": SimTypeBottom(label="IAppxManifestReader3"),
    "IAppxManifestReader4": SimTypeBottom(label="IAppxManifestReader4"),
    "IAppxManifestReader5": SimTypeBottom(label="IAppxManifestReader5"),
    "IAppxManifestReader6": SimTypeBottom(label="IAppxManifestReader6"),
    "IAppxManifestReader7": SimTypeBottom(label="IAppxManifestReader7"),
    "IAppxManifestResourcesEnumerator": SimTypeBottom(label="IAppxManifestResourcesEnumerator"),
    "IAppxManifestTargetDeviceFamiliesEnumerator": SimTypeBottom(label="IAppxManifestTargetDeviceFamiliesEnumerator"),
    "IAppxManifestTargetDeviceFamily": SimTypeBottom(label="IAppxManifestTargetDeviceFamily"),
    "IAppxPackageEditor": SimTypeBottom(label="IAppxPackageEditor"),
    "IAppxPackageReader": SimTypeBottom(label="IAppxPackageReader"),
    "IAppxPackageWriter": SimTypeBottom(label="IAppxPackageWriter"),
    "IAppxPackageWriter2": SimTypeBottom(label="IAppxPackageWriter2"),
    "IAppxPackageWriter3": SimTypeBottom(label="IAppxPackageWriter3"),
    "IAppxPackagingDiagnosticEventSink": SimTypeBottom(label="IAppxPackagingDiagnosticEventSink"),
    "IAppxPackagingDiagnosticEventSinkManager": SimTypeBottom(label="IAppxPackagingDiagnosticEventSinkManager"),
    "IAppxSourceContentGroupMapReader": SimTypeBottom(label="IAppxSourceContentGroupMapReader"),
    "IAssemblyCache": SimTypeBottom(label="IAssemblyCache"),
    "IAssemblyCacheItem": SimTypeBottom(label="IAssemblyCacheItem"),
    "IAssemblyLocator": SimTypeBottom(label="IAssemblyLocator"),
    "IAssemblyName": SimTypeBottom(label="IAssemblyName"),
    "IAssocHandler": SimTypeBottom(label="IAssocHandler"),
    "IAssocHandlerInvoker": SimTypeBottom(label="IAssocHandlerInvoker"),
    "IAssociatedIdentityProvider": SimTypeBottom(label="IAssociatedIdentityProvider"),
    "IAsyncErrorNotify": SimTypeBottom(label="IAsyncErrorNotify"),
    "IAsyncGetSendNotificationCookie": SimTypeBottom(label="IAsyncGetSendNotificationCookie"),
    "IAsyncGetSrvReferralCookie": SimTypeBottom(label="IAsyncGetSrvReferralCookie"),
    "IAsyncManager": SimTypeBottom(label="IAsyncManager"),
    "IAsyncReader": SimTypeBottom(label="IAsyncReader"),
    "IAsyncRpcChannelBuffer": SimTypeBottom(label="IAsyncRpcChannelBuffer"),
    "IAsynchronousDataRetriever": SimTypeBottom(label="IAsynchronousDataRetriever"),
    "IAtscContentAdvisoryDescriptor": SimTypeBottom(label="IAtscContentAdvisoryDescriptor"),
    "IAtscPsipParser": SimTypeBottom(label="IAtscPsipParser"),
    "IAttach": SimTypeBottom(label="IAttach"),
    "IAttachmentExecute": SimTypeBottom(label="IAttachmentExecute"),
    "IAttributeGet": SimTypeBottom(label="IAttributeGet"),
    "IAttributeSet": SimTypeBottom(label="IAttributeSet"),
    "IAudioAmbisonicsControl": SimTypeBottom(label="IAudioAmbisonicsControl"),
    "IAudioAutoGainControl": SimTypeBottom(label="IAudioAutoGainControl"),
    "IAudioBass": SimTypeBottom(label="IAudioBass"),
    "IAudioCaptureClient": SimTypeBottom(label="IAudioCaptureClient"),
    "IAudioChannelConfig": SimTypeBottom(label="IAudioChannelConfig"),
    "IAudioClient": SimTypeBottom(label="IAudioClient"),
    "IAudioClient2": SimTypeBottom(label="IAudioClient2"),
    "IAudioClient3": SimTypeBottom(label="IAudioClient3"),
    "IAudioClientDuckingControl": SimTypeBottom(label="IAudioClientDuckingControl"),
    "IAudioClock": SimTypeBottom(label="IAudioClock"),
    "IAudioClock2": SimTypeBottom(label="IAudioClock2"),
    "IAudioClockAdjustment": SimTypeBottom(label="IAudioClockAdjustment"),
    "IAudioData": SimTypeBottom(label="IAudioData"),
    "IAudioDeviceEndpoint": SimTypeBottom(label="IAudioDeviceEndpoint"),
    "IAudioDeviceModulesClient": SimTypeBottom(label="IAudioDeviceModulesClient"),
    "IAudioEffectsChangedNotificationClient": SimTypeBottom(label="IAudioEffectsChangedNotificationClient"),
    "IAudioEffectsManager": SimTypeBottom(label="IAudioEffectsManager"),
    "IAudioEndpoint": SimTypeBottom(label="IAudioEndpoint"),
    "IAudioEndpointControl": SimTypeBottom(label="IAudioEndpointControl"),
    "IAudioEndpointFormatControl": SimTypeBottom(label="IAudioEndpointFormatControl"),
    "IAudioEndpointLastBufferControl": SimTypeBottom(label="IAudioEndpointLastBufferControl"),
    "IAudioEndpointOffloadStreamMeter": SimTypeBottom(label="IAudioEndpointOffloadStreamMeter"),
    "IAudioEndpointOffloadStreamMute": SimTypeBottom(label="IAudioEndpointOffloadStreamMute"),
    "IAudioEndpointOffloadStreamVolume": SimTypeBottom(label="IAudioEndpointOffloadStreamVolume"),
    "IAudioEndpointRT": SimTypeBottom(label="IAudioEndpointRT"),
    "IAudioEndpointVolume": SimTypeBottom(label="IAudioEndpointVolume"),
    "IAudioEndpointVolumeCallback": SimTypeBottom(label="IAudioEndpointVolumeCallback"),
    "IAudioEndpointVolumeEx": SimTypeBottom(label="IAudioEndpointVolumeEx"),
    "IAudioFormatEnumerator": SimTypeBottom(label="IAudioFormatEnumerator"),
    "IAudioInputEndpointRT": SimTypeBottom(label="IAudioInputEndpointRT"),
    "IAudioInputSelector": SimTypeBottom(label="IAudioInputSelector"),
    "IAudioLfxControl": SimTypeBottom(label="IAudioLfxControl"),
    "IAudioLoudness": SimTypeBottom(label="IAudioLoudness"),
    "IAudioMediaStream": SimTypeBottom(label="IAudioMediaStream"),
    "IAudioMediaType": SimTypeBottom(label="IAudioMediaType"),
    "IAudioMeterInformation": SimTypeBottom(label="IAudioMeterInformation"),
    "IAudioMidrange": SimTypeBottom(label="IAudioMidrange"),
    "IAudioMute": SimTypeBottom(label="IAudioMute"),
    "IAudioOutputEndpointRT": SimTypeBottom(label="IAudioOutputEndpointRT"),
    "IAudioOutputSelector": SimTypeBottom(label="IAudioOutputSelector"),
    "IAudioPeakMeter": SimTypeBottom(label="IAudioPeakMeter"),
    "IAudioProcessingObject": SimTypeBottom(label="IAudioProcessingObject"),
    "IAudioProcessingObjectConfiguration": SimTypeBottom(label="IAudioProcessingObjectConfiguration"),
    "IAudioProcessingObjectLoggingService": SimTypeBottom(label="IAudioProcessingObjectLoggingService"),
    "IAudioProcessingObjectNotifications": SimTypeBottom(label="IAudioProcessingObjectNotifications"),
    "IAudioProcessingObjectNotifications2": SimTypeBottom(label="IAudioProcessingObjectNotifications2"),
    "IAudioProcessingObjectRT": SimTypeBottom(label="IAudioProcessingObjectRT"),
    "IAudioProcessingObjectRTQueueService": SimTypeBottom(label="IAudioProcessingObjectRTQueueService"),
    "IAudioProcessingObjectVBR": SimTypeBottom(label="IAudioProcessingObjectVBR"),
    "IAudioRenderClient": SimTypeBottom(label="IAudioRenderClient"),
    "IAudioSessionControl": SimTypeBottom(label="IAudioSessionControl"),
    "IAudioSessionControl2": SimTypeBottom(label="IAudioSessionControl2"),
    "IAudioSessionEnumerator": SimTypeBottom(label="IAudioSessionEnumerator"),
    "IAudioSessionEvents": SimTypeBottom(label="IAudioSessionEvents"),
    "IAudioSessionManager": SimTypeBottom(label="IAudioSessionManager"),
    "IAudioSessionManager2": SimTypeBottom(label="IAudioSessionManager2"),
    "IAudioSessionNotification": SimTypeBottom(label="IAudioSessionNotification"),
    "IAudioSessionSite": SimTypeBottom(label="IAudioSessionSite"),
    "IAudioSourceProvider": SimTypeBottom(label="IAudioSourceProvider"),
    "IAudioStateMonitor": SimTypeBottom(label="IAudioStateMonitor"),
    "IAudioStreamSample": SimTypeBottom(label="IAudioStreamSample"),
    "IAudioStreamVolume": SimTypeBottom(label="IAudioStreamVolume"),
    "IAudioSystemEffects": SimTypeBottom(label="IAudioSystemEffects"),
    "IAudioSystemEffects2": SimTypeBottom(label="IAudioSystemEffects2"),
    "IAudioSystemEffects3": SimTypeBottom(label="IAudioSystemEffects3"),
    "IAudioSystemEffectsCustomFormats": SimTypeBottom(label="IAudioSystemEffectsCustomFormats"),
    "IAudioSystemEffectsPropertyChangeNotificationClient": SimTypeBottom(label="IAudioSystemEffectsPropertyChangeNotificationClient"),
    "IAudioSystemEffectsPropertyStore": SimTypeBottom(label="IAudioSystemEffectsPropertyStore"),
    "IAudioTreble": SimTypeBottom(label="IAudioTreble"),
    "IAudioViewManagerService": SimTypeBottom(label="IAudioViewManagerService"),
    "IAudioVolumeDuckNotification": SimTypeBottom(label="IAudioVolumeDuckNotification"),
    "IAudioVolumeLevel": SimTypeBottom(label="IAudioVolumeLevel"),
    "IAuthenticate": SimTypeBottom(label="IAuthenticate"),
    "IAuthenticateEx": SimTypeBottom(label="IAuthenticateEx"),
    "IAuthenticationProviderConfig": SimTypeBottom(label="IAuthenticationProviderConfig"),
    "IAutoComplete": SimTypeBottom(label="IAutoComplete"),
    "IAutoComplete2": SimTypeBottom(label="IAutoComplete2"),
    "IAutoCompleteDropDown": SimTypeBottom(label="IAutoCompleteDropDown"),
    "IAutomaticUpdates": SimTypeBottom(label="IAutomaticUpdates"),
    "IAutomaticUpdates2": SimTypeBottom(label="IAutomaticUpdates2"),
    "IAutomaticUpdatesResults": SimTypeBottom(label="IAutomaticUpdatesResults"),
    "IAutomaticUpdatesSettings": SimTypeBottom(label="IAutomaticUpdatesSettings"),
    "IAutomaticUpdatesSettings2": SimTypeBottom(label="IAutomaticUpdatesSettings2"),
    "IAutomaticUpdatesSettings3": SimTypeBottom(label="IAutomaticUpdatesSettings3"),
    "IAuxInTuningSpace": SimTypeBottom(label="IAuxInTuningSpace"),
    "IAuxInTuningSpace2": SimTypeBottom(label="IAuxInTuningSpace2"),
    "IAzApplication": SimTypeBottom(label="IAzApplication"),
    "IAzApplication2": SimTypeBottom(label="IAzApplication2"),
    "IAzApplication3": SimTypeBottom(label="IAzApplication3"),
    "IAzApplicationGroup": SimTypeBottom(label="IAzApplicationGroup"),
    "IAzApplicationGroup2": SimTypeBottom(label="IAzApplicationGroup2"),
    "IAzApplicationGroups": SimTypeBottom(label="IAzApplicationGroups"),
    "IAzApplications": SimTypeBottom(label="IAzApplications"),
    "IAzAuthorizationStore": SimTypeBottom(label="IAzAuthorizationStore"),
    "IAzAuthorizationStore2": SimTypeBottom(label="IAzAuthorizationStore2"),
    "IAzAuthorizationStore3": SimTypeBottom(label="IAzAuthorizationStore3"),
    "IAzBizRuleContext": SimTypeBottom(label="IAzBizRuleContext"),
    "IAzBizRuleInterfaces": SimTypeBottom(label="IAzBizRuleInterfaces"),
    "IAzBizRuleParameters": SimTypeBottom(label="IAzBizRuleParameters"),
    "IAzClientContext": SimTypeBottom(label="IAzClientContext"),
    "IAzClientContext2": SimTypeBottom(label="IAzClientContext2"),
    "IAzClientContext3": SimTypeBottom(label="IAzClientContext3"),
    "IAzNameResolver": SimTypeBottom(label="IAzNameResolver"),
    "IAzObjectPicker": SimTypeBottom(label="IAzObjectPicker"),
    "IAzOperation": SimTypeBottom(label="IAzOperation"),
    "IAzOperation2": SimTypeBottom(label="IAzOperation2"),
    "IAzOperations": SimTypeBottom(label="IAzOperations"),
    "IAzPrincipalLocator": SimTypeBottom(label="IAzPrincipalLocator"),
    "IAzRole": SimTypeBottom(label="IAzRole"),
    "IAzRoleAssignment": SimTypeBottom(label="IAzRoleAssignment"),
    "IAzRoleAssignments": SimTypeBottom(label="IAzRoleAssignments"),
    "IAzRoleDefinition": SimTypeBottom(label="IAzRoleDefinition"),
    "IAzRoleDefinitions": SimTypeBottom(label="IAzRoleDefinitions"),
    "IAzRoles": SimTypeBottom(label="IAzRoles"),
    "IAzScope": SimTypeBottom(label="IAzScope"),
    "IAzScope2": SimTypeBottom(label="IAzScope2"),
    "IAzScopes": SimTypeBottom(label="IAzScopes"),
    "IAzTask": SimTypeBottom(label="IAzTask"),
    "IAzTask2": SimTypeBottom(label="IAzTask2"),
    "IAzTasks": SimTypeBottom(label="IAzTasks"),
    "IBDAComparable": SimTypeBottom(label="IBDAComparable"),
    "IBDACreateTuneRequestEx": SimTypeBottom(label="IBDACreateTuneRequestEx"),
    "IBDA_AUX": SimTypeBottom(label="IBDA_AUX"),
    "IBDA_AutoDemodulate": SimTypeBottom(label="IBDA_AutoDemodulate"),
    "IBDA_AutoDemodulateEx": SimTypeBottom(label="IBDA_AutoDemodulateEx"),
    "IBDA_ConditionalAccess": SimTypeBottom(label="IBDA_ConditionalAccess"),
    "IBDA_ConditionalAccessEx": SimTypeBottom(label="IBDA_ConditionalAccessEx"),
    "IBDA_DRIDRMService": SimTypeBottom(label="IBDA_DRIDRMService"),
    "IBDA_DRIWMDRMSession": SimTypeBottom(label="IBDA_DRIWMDRMSession"),
    "IBDA_DRM": SimTypeBottom(label="IBDA_DRM"),
    "IBDA_DRMService": SimTypeBottom(label="IBDA_DRMService"),
    "IBDA_DeviceControl": SimTypeBottom(label="IBDA_DeviceControl"),
    "IBDA_DiagnosticProperties": SimTypeBottom(label="IBDA_DiagnosticProperties"),
    "IBDA_DigitalDemodulator": SimTypeBottom(label="IBDA_DigitalDemodulator"),
    "IBDA_DigitalDemodulator2": SimTypeBottom(label="IBDA_DigitalDemodulator2"),
    "IBDA_DigitalDemodulator3": SimTypeBottom(label="IBDA_DigitalDemodulator3"),
    "IBDA_DiseqCommand": SimTypeBottom(label="IBDA_DiseqCommand"),
    "IBDA_EasMessage": SimTypeBottom(label="IBDA_EasMessage"),
    "IBDA_Encoder": SimTypeBottom(label="IBDA_Encoder"),
    "IBDA_EthernetFilter": SimTypeBottom(label="IBDA_EthernetFilter"),
    "IBDA_EventingService": SimTypeBottom(label="IBDA_EventingService"),
    "IBDA_FDC": SimTypeBottom(label="IBDA_FDC"),
    "IBDA_FrequencyFilter": SimTypeBottom(label="IBDA_FrequencyFilter"),
    "IBDA_GuideDataDeliveryService": SimTypeBottom(label="IBDA_GuideDataDeliveryService"),
    "IBDA_IPSinkControl": SimTypeBottom(label="IBDA_IPSinkControl"),
    "IBDA_IPSinkInfo": SimTypeBottom(label="IBDA_IPSinkInfo"),
    "IBDA_IPV4Filter": SimTypeBottom(label="IBDA_IPV4Filter"),
    "IBDA_IPV6Filter": SimTypeBottom(label="IBDA_IPV6Filter"),
    "IBDA_ISDBConditionalAccess": SimTypeBottom(label="IBDA_ISDBConditionalAccess"),
    "IBDA_LNBInfo": SimTypeBottom(label="IBDA_LNBInfo"),
    "IBDA_MUX": SimTypeBottom(label="IBDA_MUX"),
    "IBDA_NameValueService": SimTypeBottom(label="IBDA_NameValueService"),
    "IBDA_NetworkProvider": SimTypeBottom(label="IBDA_NetworkProvider"),
    "IBDA_NullTransform": SimTypeBottom(label="IBDA_NullTransform"),
    "IBDA_PinControl": SimTypeBottom(label="IBDA_PinControl"),
    "IBDA_SignalProperties": SimTypeBottom(label="IBDA_SignalProperties"),
    "IBDA_SignalStatistics": SimTypeBottom(label="IBDA_SignalStatistics"),
    "IBDA_TIF_REGISTRATION": SimTypeBottom(label="IBDA_TIF_REGISTRATION"),
    "IBDA_Topology": SimTypeBottom(label="IBDA_Topology"),
    "IBDA_TransportStreamInfo": SimTypeBottom(label="IBDA_TransportStreamInfo"),
    "IBDA_TransportStreamSelector": SimTypeBottom(label="IBDA_TransportStreamSelector"),
    "IBDA_UserActivityService": SimTypeBottom(label="IBDA_UserActivityService"),
    "IBDA_VoidTransform": SimTypeBottom(label="IBDA_VoidTransform"),
    "IBDA_WMDRMSession": SimTypeBottom(label="IBDA_WMDRMSession"),
    "IBDA_WMDRMTuner": SimTypeBottom(label="IBDA_WMDRMTuner"),
    "IBFCacheable": SimTypeBottom(label="IBFCacheable"),
    "IBITSExtensionSetup": SimTypeBottom(label="IBITSExtensionSetup"),
    "IBITSExtensionSetupFactory": SimTypeBottom(label="IBITSExtensionSetupFactory"),
    "IBPCSatelliteTuner": SimTypeBottom(label="IBPCSatelliteTuner"),
    "IBackgroundCopyCallback": SimTypeBottom(label="IBackgroundCopyCallback"),
    "IBackgroundCopyCallback1": SimTypeBottom(label="IBackgroundCopyCallback1"),
    "IBackgroundCopyCallback2": SimTypeBottom(label="IBackgroundCopyCallback2"),
    "IBackgroundCopyCallback3": SimTypeBottom(label="IBackgroundCopyCallback3"),
    "IBackgroundCopyError": SimTypeBottom(label="IBackgroundCopyError"),
    "IBackgroundCopyFile": SimTypeBottom(label="IBackgroundCopyFile"),
    "IBackgroundCopyFile2": SimTypeBottom(label="IBackgroundCopyFile2"),
    "IBackgroundCopyFile3": SimTypeBottom(label="IBackgroundCopyFile3"),
    "IBackgroundCopyFile4": SimTypeBottom(label="IBackgroundCopyFile4"),
    "IBackgroundCopyFile5": SimTypeBottom(label="IBackgroundCopyFile5"),
    "IBackgroundCopyFile6": SimTypeBottom(label="IBackgroundCopyFile6"),
    "IBackgroundCopyGroup": SimTypeBottom(label="IBackgroundCopyGroup"),
    "IBackgroundCopyJob": SimTypeBottom(label="IBackgroundCopyJob"),
    "IBackgroundCopyJob1": SimTypeBottom(label="IBackgroundCopyJob1"),
    "IBackgroundCopyJob2": SimTypeBottom(label="IBackgroundCopyJob2"),
    "IBackgroundCopyJob3": SimTypeBottom(label="IBackgroundCopyJob3"),
    "IBackgroundCopyJob4": SimTypeBottom(label="IBackgroundCopyJob4"),
    "IBackgroundCopyJob5": SimTypeBottom(label="IBackgroundCopyJob5"),
    "IBackgroundCopyJobHttpOptions": SimTypeBottom(label="IBackgroundCopyJobHttpOptions"),
    "IBackgroundCopyJobHttpOptions2": SimTypeBottom(label="IBackgroundCopyJobHttpOptions2"),
    "IBackgroundCopyJobHttpOptions3": SimTypeBottom(label="IBackgroundCopyJobHttpOptions3"),
    "IBackgroundCopyManager": SimTypeBottom(label="IBackgroundCopyManager"),
    "IBackgroundCopyQMgr": SimTypeBottom(label="IBackgroundCopyQMgr"),
    "IBackgroundCopyServerCertificateValidationCallback": SimTypeBottom(label="IBackgroundCopyServerCertificateValidationCallback"),
    "IBandHost": SimTypeBottom(label="IBandHost"),
    "IBandSite": SimTypeBottom(label="IBandSite"),
    "IBannerNotificationHandler": SimTypeBottom(label="IBannerNotificationHandler"),
    "IBanneredBar": SimTypeBottom(label="IBanneredBar"),
    "IBaseFilter": SimTypeBottom(label="IBaseFilter"),
    "IBaseVideoMixer": SimTypeBottom(label="IBaseVideoMixer"),
    "IBasicAudio": SimTypeBottom(label="IBasicAudio"),
    "IBasicVideo": SimTypeBottom(label="IBasicVideo"),
    "IBasicVideo2": SimTypeBottom(label="IBasicVideo2"),
    "IBidiAsyncNotifyChannel": SimTypeBottom(label="IBidiAsyncNotifyChannel"),
    "IBidiRequest": SimTypeBottom(label="IBidiRequest"),
    "IBidiRequestContainer": SimTypeBottom(label="IBidiRequestContainer"),
    "IBidiSpl": SimTypeBottom(label="IBidiSpl"),
    "IBidiSpl2": SimTypeBottom(label="IBidiSpl2"),
    "IBinaryConverter": SimTypeBottom(label="IBinaryConverter"),
    "IBinaryConverter2": SimTypeBottom(label="IBinaryConverter2"),
    "IBindCallbackRedirect": SimTypeBottom(label="IBindCallbackRedirect"),
    "IBindCtx": SimTypeBottom(label="IBindCtx"),
    "IBindEventHandler": SimTypeBottom(label="IBindEventHandler"),
    "IBindHost": SimTypeBottom(label="IBindHost"),
    "IBindHttpSecurity": SimTypeBottom(label="IBindHttpSecurity"),
    "IBindProtocol": SimTypeBottom(label="IBindProtocol"),
    "IBindResource": SimTypeBottom(label="IBindResource"),
    "IBindStatusCallback": SimTypeBottom(label="IBindStatusCallback"),
    "IBindStatusCallbackEx": SimTypeBottom(label="IBindStatusCallbackEx"),
    "IBinding": SimTypeBottom(label="IBinding"),
    "IBitmapData": SimTypeBottom(label="IBitmapData"),
    "IBitsPeer": SimTypeBottom(label="IBitsPeer"),
    "IBitsPeerCacheAdministration": SimTypeBottom(label="IBitsPeerCacheAdministration"),
    "IBitsPeerCacheRecord": SimTypeBottom(label="IBitsPeerCacheRecord"),
    "IBitsTokenOptions": SimTypeBottom(label="IBitsTokenOptions"),
    "IBlockFormats": SimTypeBottom(label="IBlockFormats"),
    "IBlockRange": SimTypeBottom(label="IBlockRange"),
    "IBlockRangeList": SimTypeBottom(label="IBlockRangeList"),
    "IBlockingLock": SimTypeBottom(label="IBlockingLock"),
    "IBootOptions": SimTypeBottom(label="IBootOptions"),
    "IBootTrigger": SimTypeBottom(label="IBootTrigger"),
    "IBriefcaseInitiator": SimTypeBottom(label="IBriefcaseInitiator"),
    "IBroadcastEvent": SimTypeBottom(label="IBroadcastEvent"),
    "IBroadcastEventEx": SimTypeBottom(label="IBroadcastEventEx"),
    "IBrowserFrameOptions": SimTypeBottom(label="IBrowserFrameOptions"),
    "IBrowserService": SimTypeBottom(label="IBrowserService"),
    "IBrowserService2": SimTypeBottom(label="IBrowserService2"),
    "IBrowserService3": SimTypeBottom(label="IBrowserService3"),
    "IBrowserService4": SimTypeBottom(label="IBrowserService4"),
    "IBufferingTime": SimTypeBottom(label="IBufferingTime"),
    "IBurnVerification": SimTypeBottom(label="IBurnVerification"),
    "ICAT": SimTypeBottom(label="ICAT"),
    "ICCOMPRESS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpbiOutput", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpOutput", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpbiInput", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpInput", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpckid", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpdwFlags", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lFrameNum", SimTypeInt(signed=True, label="Int32")), ("dwFrameSize", SimTypeInt(signed=False, label="UInt32")), ("dwQuality", SimTypeInt(signed=False, label="UInt32")), ("lpbiPrev", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpPrev", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="ICCOMPRESS", pack=False, align=None),
    "ICCOMPRESSFRAMES": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpbiOutput", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lOutput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpbiInput", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lInput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lStartFrame", SimTypeInt(signed=True, label="Int32")), ("lFrameCount", SimTypeInt(signed=True, label="Int32")), ("lQuality", SimTypeInt(signed=True, label="Int32")), ("lDataRate", SimTypeInt(signed=True, label="Int32")), ("lKeyRate", SimTypeInt(signed=True, label="Int32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwScale", SimTypeInt(signed=False, label="UInt32")), ("dwOverheadPerFrame", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("GetData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PutData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ICCOMPRESSFRAMES", pack=False, align=None),
    "ICCSubStreamFiltering": SimTypeBottom(label="ICCSubStreamFiltering"),
    "ICDBurn": SimTypeBottom(label="ICDBurn"),
    "ICDBurnExt": SimTypeBottom(label="ICDBurnExt"),
    "ICDECOMPRESS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpbiInput", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpInput", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpbiOutput", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpOutput", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ckid", SimTypeInt(signed=False, label="UInt32")),)), name="ICDECOMPRESS", pack=False, align=None),
    "ICDECOMPRESSEX": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpbiSrc", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpSrc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpbiDst", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpDst", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("xDst", SimTypeInt(signed=True, label="Int32")), ("yDst", SimTypeInt(signed=True, label="Int32")), ("dxDst", SimTypeInt(signed=True, label="Int32")), ("dyDst", SimTypeInt(signed=True, label="Int32")), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("dxSrc", SimTypeInt(signed=True, label="Int32")), ("dySrc", SimTypeInt(signed=True, label="Int32")),)), name="ICDECOMPRESSEX", pack=False, align=None),
    "ICDRAW": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpFormat", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("lTime", SimTypeInt(signed=True, label="Int32")),)), name="ICDRAW", pack=False, align=None),
    "ICDRAWBEGIN": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hpal", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xDst", SimTypeInt(signed=True, label="Int32")), ("yDst", SimTypeInt(signed=True, label="Int32")), ("dxDst", SimTypeInt(signed=True, label="Int32")), ("dyDst", SimTypeInt(signed=True, label="Int32")), ("lpbi", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("xSrc", SimTypeInt(signed=True, label="Int32")), ("ySrc", SimTypeInt(signed=True, label="Int32")), ("dxSrc", SimTypeInt(signed=True, label="Int32")), ("dySrc", SimTypeInt(signed=True, label="Int32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwScale", SimTypeInt(signed=False, label="UInt32")),)), name="ICDRAWBEGIN", pack=False, align=None),
    "ICDRAWSUGGEST": SimStruct(OrderedDict((("lpbiIn", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpbiSuggest", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("dxSrc", SimTypeInt(signed=True, label="Int32")), ("dySrc", SimTypeInt(signed=True, label="Int32")), ("dxDst", SimTypeInt(signed=True, label="Int32")), ("dyDst", SimTypeInt(signed=True, label="Int32")), ("hicDecompressor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ICDRAWSUGGEST", pack=False, align=None),
    "ICEnroll": SimTypeBottom(label="ICEnroll"),
    "ICEnroll2": SimTypeBottom(label="ICEnroll2"),
    "ICEnroll3": SimTypeBottom(label="ICEnroll3"),
    "ICEnroll4": SimTypeBottom(label="ICEnroll4"),
    "ICINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwVersionICM", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 16)), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 128)), ("szDriver", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="ICINFO", pack=False, align=None),
    "ICLRAppDomainResourceMonitor": SimTypeBottom(label="ICLRAppDomainResourceMonitor"),
    "ICLRAssemblyIdentityManager": SimTypeBottom(label="ICLRAssemblyIdentityManager"),
    "ICLRAssemblyReferenceList": SimTypeBottom(label="ICLRAssemblyReferenceList"),
    "ICLRControl": SimTypeBottom(label="ICLRControl"),
    "ICLRDebugManager": SimTypeBottom(label="ICLRDebugManager"),
    "ICLRDebugging": SimTypeBottom(label="ICLRDebugging"),
    "ICLRDebuggingLibraryProvider": SimTypeBottom(label="ICLRDebuggingLibraryProvider"),
    "ICLRDomainManager": SimTypeBottom(label="ICLRDomainManager"),
    "ICLRErrorReportingManager": SimTypeBottom(label="ICLRErrorReportingManager"),
    "ICLRGCManager": SimTypeBottom(label="ICLRGCManager"),
    "ICLRGCManager2": SimTypeBottom(label="ICLRGCManager2"),
    "ICLRHostBindingPolicyManager": SimTypeBottom(label="ICLRHostBindingPolicyManager"),
    "ICLRHostProtectionManager": SimTypeBottom(label="ICLRHostProtectionManager"),
    "ICLRIoCompletionManager": SimTypeBottom(label="ICLRIoCompletionManager"),
    "ICLRMemoryNotificationCallback": SimTypeBottom(label="ICLRMemoryNotificationCallback"),
    "ICLRMetaHost": SimTypeBottom(label="ICLRMetaHost"),
    "ICLRMetaHostPolicy": SimTypeBottom(label="ICLRMetaHostPolicy"),
    "ICLROnEventManager": SimTypeBottom(label="ICLROnEventManager"),
    "ICLRPolicyManager": SimTypeBottom(label="ICLRPolicyManager"),
    "ICLRProbingAssemblyEnum": SimTypeBottom(label="ICLRProbingAssemblyEnum"),
    "ICLRProfiling": SimTypeBottom(label="ICLRProfiling"),
    "ICLRReferenceAssemblyEnum": SimTypeBottom(label="ICLRReferenceAssemblyEnum"),
    "ICLRRuntimeHost": SimTypeBottom(label="ICLRRuntimeHost"),
    "ICLRRuntimeInfo": SimTypeBottom(label="ICLRRuntimeInfo"),
    "ICLRStrongName": SimTypeBottom(label="ICLRStrongName"),
    "ICLRStrongName2": SimTypeBottom(label="ICLRStrongName2"),
    "ICLRStrongName3": SimTypeBottom(label="ICLRStrongName3"),
    "ICLRSyncManager": SimTypeBottom(label="ICLRSyncManager"),
    "ICLRTask": SimTypeBottom(label="ICLRTask"),
    "ICLRTask2": SimTypeBottom(label="ICLRTask2"),
    "ICLRTaskManager": SimTypeBottom(label="ICLRTaskManager"),
    "ICMENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "ICMENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "ICMP4_TIME_EXCEED_CODE": SimTypeInt(signed=False, label="ICMP4_TIME_EXCEED_CODE"),
    "ICMP4_TYPE": SimTypeInt(signed=False, label="ICMP4_TYPE"),
    "ICMP4_UNREACH_CODE": SimTypeInt(signed=False, label="ICMP4_UNREACH_CODE"),
    "ICMP6_TYPE": SimTypeInt(signed=False, label="ICMP6_TYPE"),
    "ICMPV4_ADDRESS_MASK_MESSAGE": SimStruct(OrderedDict((("Header", SimTypeRef("ICMP_MESSAGE", SimStruct)), ("AddressMask", SimTypeInt(signed=False, label="UInt32")),)), name="ICMPV4_ADDRESS_MASK_MESSAGE", pack=False, align=None),
    "ICMPV4_ROUTER_ADVERT_ENTRY": SimStruct(OrderedDict((("RouterAdvertAddr", SimTypeRef("IN_ADDR", SimStruct)), ("PreferenceLevel", SimTypeInt(signed=True, label="Int32")),)), name="ICMPV4_ROUTER_ADVERT_ENTRY", pack=False, align=None),
    "ICMPV4_ROUTER_ADVERT_HEADER": SimStruct(OrderedDict((("RaHeader", SimTypeRef("ICMP_MESSAGE", SimStruct)),)), name="ICMPV4_ROUTER_ADVERT_HEADER", pack=False, align=None),
    "ICMPV4_ROUTER_SOLICIT": SimStruct(OrderedDict((("RsHeader", SimTypeRef("ICMP_MESSAGE", SimStruct)),)), name="ICMPV4_ROUTER_SOLICIT", pack=False, align=None),
    "ICMPV4_TIMESTAMP_MESSAGE": SimStruct(OrderedDict((("Header", SimTypeRef("ICMP_MESSAGE", SimStruct)), ("OriginateTimestamp", SimTypeInt(signed=False, label="UInt32")), ("ReceiveTimestamp", SimTypeInt(signed=False, label="UInt32")), ("TransmitTimestamp", SimTypeInt(signed=False, label="UInt32")),)), name="ICMPV4_TIMESTAMP_MESSAGE", pack=False, align=None),
    "ICMPV6_ECHO_REPLY_LH": SimStruct(OrderedDict((("Address", SimTypeRef("IPV6_ADDRESS_EX", SimStruct)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("RoundTripTime", SimTypeInt(signed=False, label="UInt32")),)), name="ICMPV6_ECHO_REPLY_LH", pack=False, align=None),
    "ICMP_ECHO_REPLY": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("RoundTripTime", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Options", SimTypeRef("IP_OPTION_INFORMATION", SimStruct)),)), name="ICMP_ECHO_REPLY", pack=False, align=None),
    "ICMP_ECHO_REPLY32": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("RoundTripTime", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Options", SimTypeRef("IP_OPTION_INFORMATION32", SimStruct)),)), name="ICMP_ECHO_REPLY32", pack=False, align=None),
    "ICMP_ERROR_INFO": SimStruct(OrderedDict((("srcaddress", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("protocol", SimTypeInt(signed=False, label="IPPROTO")), ("type", SimTypeChar(label="Byte")), ("code", SimTypeChar(label="Byte")),)), name="ICMP_ERROR_INFO", pack=False, align=None),
    "ICMP_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Code", SimTypeChar(label="Byte")), ("Checksum", SimTypeShort(signed=False, label="UInt16")),)), name="ICMP_HEADER", pack=False, align=None),
    "ICMP_MESSAGE": SimStruct(OrderedDict((("Header", SimTypeRef("ICMP_HEADER", SimStruct)), ("Data", SimUnion({"Data32": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Data16": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2), "Data8": SimTypeArray(SimTypeChar(label="Byte"), 4)}, name="<anon>", label="None")),)), name="ICMP_MESSAGE", pack=False, align=None),
    "ICM_COMMAND": SimTypeInt(signed=False, label="ICM_COMMAND"),
    "ICM_MODE": SimTypeInt(signed=False, label="ICM_MODE"),
    "ICOMAdminCatalog": SimTypeBottom(label="ICOMAdminCatalog"),
    "ICOMAdminCatalog2": SimTypeBottom(label="ICOMAdminCatalog2"),
    "ICOMLBArguments": SimTypeBottom(label="ICOMLBArguments"),
    "ICONEFFECT": SimTypeInt(signed=False, label="ICONEFFECT"),
    "ICONINFO": SimStruct(OrderedDict((("fIcon", SimTypeInt(signed=True, label="Int32")), ("xHotspot", SimTypeInt(signed=False, label="UInt32")), ("yHotspot", SimTypeInt(signed=False, label="UInt32")), ("hbmMask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmColor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ICONINFO", pack=False, align=None),
    "ICONINFOEXA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fIcon", SimTypeInt(signed=True, label="Int32")), ("xHotspot", SimTypeInt(signed=False, label="UInt32")), ("yHotspot", SimTypeInt(signed=False, label="UInt32")), ("hbmMask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmColor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wResID", SimTypeShort(signed=False, label="UInt16")), ("szModName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szResName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="ICONINFOEXA", pack=False, align=None),
    "ICONINFOEXW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fIcon", SimTypeInt(signed=True, label="Int32")), ("xHotspot", SimTypeInt(signed=False, label="UInt32")), ("yHotspot", SimTypeInt(signed=False, label="UInt32")), ("hbmMask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmColor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wResID", SimTypeShort(signed=False, label="UInt16")), ("szModName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szResName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="ICONINFOEXW", pack=False, align=None),
    "ICONMETRICSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iHorzSpacing", SimTypeInt(signed=True, label="Int32")), ("iVertSpacing", SimTypeInt(signed=True, label="Int32")), ("iTitleWrap", SimTypeInt(signed=True, label="Int32")), ("lfFont", SimTypeRef("LOGFONTA", SimStruct)),)), name="ICONMETRICSA", pack=False, align=None),
    "ICONMETRICSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iHorzSpacing", SimTypeInt(signed=True, label="Int32")), ("iVertSpacing", SimTypeInt(signed=True, label="Int32")), ("iTitleWrap", SimTypeInt(signed=True, label="Int32")), ("lfFont", SimTypeRef("LOGFONTW", SimStruct)),)), name="ICONMETRICSW", pack=False, align=None),
    "ICOPEN": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("fccHandler", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pV1Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pV2Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dnDevNode", SimTypeInt(signed=False, label="UInt32")),)), name="ICOPEN", pack=False, align=None),
    "ICPALETTE": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iStart", SimTypeInt(signed=True, label="Int32")), ("iLen", SimTypeInt(signed=True, label="Int32")), ("lppe", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)),)), name="ICPALETTE", pack=False, align=None),
    "ICSETSTATUSPROC": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Status", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="ICSETSTATUSPROC", pack=False, align=None),
    "ICSSFilter": SimTypeBottom(label="ICSSFilter"),
    "ICSSFilterSite": SimTypeBottom(label="ICSSFilterSite"),
    "ICS_TARGETTYPE": SimTypeInt(signed=False, label="ICS_TARGETTYPE"),
    "ICallFactory": SimTypeBottom(label="ICallFactory"),
    "ICallFrame": SimTypeBottom(label="ICallFrame"),
    "ICallFrameEvents": SimTypeBottom(label="ICallFrameEvents"),
    "ICallFrameWalker": SimTypeBottom(label="ICallFrameWalker"),
    "ICallIndirect": SimTypeBottom(label="ICallIndirect"),
    "ICallInterceptor": SimTypeBottom(label="ICallInterceptor"),
    "ICallUnmarshal": SimTypeBottom(label="ICallUnmarshal"),
    "ICameraControl": SimTypeBottom(label="ICameraControl"),
    "ICameraUIControl": SimTypeBottom(label="ICameraUIControl"),
    "ICameraUIControlEventCallback": SimTypeBottom(label="ICameraUIControlEventCallback"),
    "ICanHandleException": SimTypeBottom(label="ICanHandleException"),
    "ICancelMethodCalls": SimTypeBottom(label="ICancelMethodCalls"),
    "ICanvasGradient": SimTypeBottom(label="ICanvasGradient"),
    "ICanvasImageData": SimTypeBottom(label="ICanvasImageData"),
    "ICanvasPattern": SimTypeBottom(label="ICanvasPattern"),
    "ICanvasPixelArray": SimTypeBottom(label="ICanvasPixelArray"),
    "ICanvasPixelArrayData": SimTypeBottom(label="ICanvasPixelArrayData"),
    "ICanvasRenderingContext2D": SimTypeBottom(label="ICanvasRenderingContext2D"),
    "ICanvasTextMetrics": SimTypeBottom(label="ICanvasTextMetrics"),
    "ICaptionServiceDescriptor": SimTypeBottom(label="ICaptionServiceDescriptor"),
    "ICaptureGraphBuilder": SimTypeBottom(label="ICaptureGraphBuilder"),
    "ICaptureGraphBuilder2": SimTypeBottom(label="ICaptureGraphBuilder2"),
    "ICaretPositionProvider": SimTypeBottom(label="ICaretPositionProvider"),
    "ICatInformation": SimTypeBottom(label="ICatInformation"),
    "ICatRegister": SimTypeBottom(label="ICatRegister"),
    "ICatalog": SimTypeBottom(label="ICatalog"),
    "ICatalogCollection": SimTypeBottom(label="ICatalogCollection"),
    "ICatalogFileInfo": SimTypeBottom(label="ICatalogFileInfo"),
    "ICatalogObject": SimTypeBottom(label="ICatalogObject"),
    "ICatalogServices": SimTypeBottom(label="ICatalogServices"),
    "ICategorizer": SimTypeBottom(label="ICategorizer"),
    "ICategory": SimTypeBottom(label="ICategory"),
    "ICategoryCollection": SimTypeBottom(label="ICategoryCollection"),
    "ICategoryProvider": SimTypeBottom(label="ICategoryProvider"),
    "ICcgDomainAuthCredentials": SimTypeBottom(label="ICcgDomainAuthCredentials"),
    "ICertAdmin": SimTypeBottom(label="ICertAdmin"),
    "ICertAdmin2": SimTypeBottom(label="ICertAdmin2"),
    "ICertConfig": SimTypeBottom(label="ICertConfig"),
    "ICertConfig2": SimTypeBottom(label="ICertConfig2"),
    "ICertEncodeAltName": SimTypeBottom(label="ICertEncodeAltName"),
    "ICertEncodeAltName2": SimTypeBottom(label="ICertEncodeAltName2"),
    "ICertEncodeBitString": SimTypeBottom(label="ICertEncodeBitString"),
    "ICertEncodeBitString2": SimTypeBottom(label="ICertEncodeBitString2"),
    "ICertEncodeCRLDistInfo": SimTypeBottom(label="ICertEncodeCRLDistInfo"),
    "ICertEncodeCRLDistInfo2": SimTypeBottom(label="ICertEncodeCRLDistInfo2"),
    "ICertEncodeDateArray": SimTypeBottom(label="ICertEncodeDateArray"),
    "ICertEncodeDateArray2": SimTypeBottom(label="ICertEncodeDateArray2"),
    "ICertEncodeLongArray": SimTypeBottom(label="ICertEncodeLongArray"),
    "ICertEncodeLongArray2": SimTypeBottom(label="ICertEncodeLongArray2"),
    "ICertEncodeStringArray": SimTypeBottom(label="ICertEncodeStringArray"),
    "ICertEncodeStringArray2": SimTypeBottom(label="ICertEncodeStringArray2"),
    "ICertExit": SimTypeBottom(label="ICertExit"),
    "ICertExit2": SimTypeBottom(label="ICertExit2"),
    "ICertGetConfig": SimTypeBottom(label="ICertGetConfig"),
    "ICertManageModule": SimTypeBottom(label="ICertManageModule"),
    "ICertPolicy": SimTypeBottom(label="ICertPolicy"),
    "ICertPolicy2": SimTypeBottom(label="ICertPolicy2"),
    "ICertProperties": SimTypeBottom(label="ICertProperties"),
    "ICertProperty": SimTypeBottom(label="ICertProperty"),
    "ICertPropertyArchived": SimTypeBottom(label="ICertPropertyArchived"),
    "ICertPropertyArchivedKeyHash": SimTypeBottom(label="ICertPropertyArchivedKeyHash"),
    "ICertPropertyAutoEnroll": SimTypeBottom(label="ICertPropertyAutoEnroll"),
    "ICertPropertyBackedUp": SimTypeBottom(label="ICertPropertyBackedUp"),
    "ICertPropertyDescription": SimTypeBottom(label="ICertPropertyDescription"),
    "ICertPropertyEnrollment": SimTypeBottom(label="ICertPropertyEnrollment"),
    "ICertPropertyEnrollmentPolicyServer": SimTypeBottom(label="ICertPropertyEnrollmentPolicyServer"),
    "ICertPropertyFriendlyName": SimTypeBottom(label="ICertPropertyFriendlyName"),
    "ICertPropertyKeyProvInfo": SimTypeBottom(label="ICertPropertyKeyProvInfo"),
    "ICertPropertyRenewal": SimTypeBottom(label="ICertPropertyRenewal"),
    "ICertPropertyRequestOriginator": SimTypeBottom(label="ICertPropertyRequestOriginator"),
    "ICertPropertySHA1Hash": SimTypeBottom(label="ICertPropertySHA1Hash"),
    "ICertRequest": SimTypeBottom(label="ICertRequest"),
    "ICertRequest2": SimTypeBottom(label="ICertRequest2"),
    "ICertRequest3": SimTypeBottom(label="ICertRequest3"),
    "ICertRequestD": SimTypeBottom(label="ICertRequestD"),
    "ICertRequestD2": SimTypeBottom(label="ICertRequestD2"),
    "ICertServerExit": SimTypeBottom(label="ICertServerExit"),
    "ICertServerPolicy": SimTypeBottom(label="ICertServerPolicy"),
    "ICertSrvSetup": SimTypeBottom(label="ICertSrvSetup"),
    "ICertSrvSetupKeyInformation": SimTypeBottom(label="ICertSrvSetupKeyInformation"),
    "ICertSrvSetupKeyInformationCollection": SimTypeBottom(label="ICertSrvSetupKeyInformationCollection"),
    "ICertView": SimTypeBottom(label="ICertView"),
    "ICertView2": SimTypeBottom(label="ICertView2"),
    "ICertificateAttestationChallenge": SimTypeBottom(label="ICertificateAttestationChallenge"),
    "ICertificateAttestationChallenge2": SimTypeBottom(label="ICertificateAttestationChallenge2"),
    "ICertificateEnrollmentPolicyServerSetup": SimTypeBottom(label="ICertificateEnrollmentPolicyServerSetup"),
    "ICertificateEnrollmentServerSetup": SimTypeBottom(label="ICertificateEnrollmentServerSetup"),
    "ICertificatePolicies": SimTypeBottom(label="ICertificatePolicies"),
    "ICertificatePolicy": SimTypeBottom(label="ICertificatePolicy"),
    "ICertificationAuthorities": SimTypeBottom(label="ICertificationAuthorities"),
    "ICertificationAuthority": SimTypeBottom(label="ICertificationAuthority"),
    "IChangeConflict": SimTypeBottom(label="IChangeConflict"),
    "IChangeUnitException": SimTypeBottom(label="IChangeUnitException"),
    "IChangeUnitListFilterInfo": SimTypeBottom(label="IChangeUnitListFilterInfo"),
    "IChannelAudioVolume": SimTypeBottom(label="IChannelAudioVolume"),
    "IChannelCredentials": SimTypeBottom(label="IChannelCredentials"),
    "IChannelHook": SimTypeBottom(label="IChannelHook"),
    "IChannelIDTuneRequest": SimTypeBottom(label="IChannelIDTuneRequest"),
    "IChannelTuneRequest": SimTypeBottom(label="IChannelTuneRequest"),
    "IChapteredRowset": SimTypeBottom(label="IChapteredRowset"),
    "ICheckSxsConfig": SimTypeBottom(label="ICheckSxsConfig"),
    "ICivicAddressReport": SimTypeBottom(label="ICivicAddressReport"),
    "ICivicAddressReportFactory": SimTypeBottom(label="ICivicAddressReportFactory"),
    "IClassActivator": SimTypeBottom(label="IClassActivator"),
    "IClassFactory": SimTypeBottom(label="IClassFactory"),
    "IClassFactory2": SimTypeBottom(label="IClassFactory2"),
    "IClassFactoryEx": SimTypeBottom(label="IClassFactoryEx"),
    "IClientCaps": SimTypeBottom(label="IClientCaps"),
    "IClientSecurity": SimTypeBottom(label="IClientSecurity"),
    "IClipServiceNotificationHelper": SimTypeBottom(label="IClipServiceNotificationHelper"),
    "IClockVector": SimTypeBottom(label="IClockVector"),
    "IClockVectorElement": SimTypeBottom(label="IClockVectorElement"),
    "IClonableWrapper": SimTypeBottom(label="IClonableWrapper"),
    "ICloneViewHelper": SimTypeBottom(label="ICloneViewHelper"),
    "IClusterDetector": SimTypeBottom(label="IClusterDetector"),
    "ICoCreateLocally": SimTypeBottom(label="ICoCreateLocally"),
    "ICoCreatedLocally": SimTypeBottom(label="ICoCreatedLocally"),
    "ICodeAddressConcept": SimTypeBottom(label="ICodeAddressConcept"),
    "ICodeInstall": SimTypeBottom(label="ICodeInstall"),
    "ICodecAPI": SimTypeBottom(label="ICodecAPI"),
    "IColumnData": SimTypeBottom(label="IColumnData"),
    "IColumnManager": SimTypeBottom(label="IColumnManager"),
    "IColumnMapper": SimTypeBottom(label="IColumnMapper"),
    "IColumnMapperCreator": SimTypeBottom(label="IColumnMapperCreator"),
    "IColumnProvider": SimTypeBottom(label="IColumnProvider"),
    "IColumnsInfo": SimTypeBottom(label="IColumnsInfo"),
    "IColumnsInfo2": SimTypeBottom(label="IColumnsInfo2"),
    "IColumnsRowset": SimTypeBottom(label="IColumnsRowset"),
    "IComActivityEvents": SimTypeBottom(label="IComActivityEvents"),
    "IComApp2Events": SimTypeBottom(label="IComApp2Events"),
    "IComAppEvents": SimTypeBottom(label="IComAppEvents"),
    "IComCRMEvents": SimTypeBottom(label="IComCRMEvents"),
    "IComExceptionEvents": SimTypeBottom(label="IComExceptionEvents"),
    "IComHandlerAction": SimTypeBottom(label="IComHandlerAction"),
    "IComIdentityEvents": SimTypeBottom(label="IComIdentityEvents"),
    "IComInstance2Events": SimTypeBottom(label="IComInstance2Events"),
    "IComInstanceEvents": SimTypeBottom(label="IComInstanceEvents"),
    "IComLTxEvents": SimTypeBottom(label="IComLTxEvents"),
    "IComMethod2Events": SimTypeBottom(label="IComMethod2Events"),
    "IComMethodEvents": SimTypeBottom(label="IComMethodEvents"),
    "IComMtaThreadPoolKnobs": SimTypeBottom(label="IComMtaThreadPoolKnobs"),
    "IComObjectConstruction2Events": SimTypeBottom(label="IComObjectConstruction2Events"),
    "IComObjectConstructionEvents": SimTypeBottom(label="IComObjectConstructionEvents"),
    "IComObjectEvents": SimTypeBottom(label="IComObjectEvents"),
    "IComObjectPool2Events": SimTypeBottom(label="IComObjectPool2Events"),
    "IComObjectPoolEvents": SimTypeBottom(label="IComObjectPoolEvents"),
    "IComObjectPoolEvents2": SimTypeBottom(label="IComObjectPoolEvents2"),
    "IComQCEvents": SimTypeBottom(label="IComQCEvents"),
    "IComResourceEvents": SimTypeBottom(label="IComResourceEvents"),
    "IComSecurityEvents": SimTypeBottom(label="IComSecurityEvents"),
    "IComStaThreadPoolKnobs": SimTypeBottom(label="IComStaThreadPoolKnobs"),
    "IComStaThreadPoolKnobs2": SimTypeBottom(label="IComStaThreadPoolKnobs2"),
    "IComThreadEvents": SimTypeBottom(label="IComThreadEvents"),
    "IComThreadingInfo": SimTypeBottom(label="IComThreadingInfo"),
    "IComTrackingInfoCollection": SimTypeBottom(label="IComTrackingInfoCollection"),
    "IComTrackingInfoEvents": SimTypeBottom(label="IComTrackingInfoEvents"),
    "IComTrackingInfoObject": SimTypeBottom(label="IComTrackingInfoObject"),
    "IComTrackingInfoProperties": SimTypeBottom(label="IComTrackingInfoProperties"),
    "IComTransaction2Events": SimTypeBottom(label="IComTransaction2Events"),
    "IComTransactionEvents": SimTypeBottom(label="IComTransactionEvents"),
    "IComUserEvent": SimTypeBottom(label="IComUserEvent"),
    "ICombinedFilterInfo": SimTypeBottom(label="ICombinedFilterInfo"),
    "ICommDlgBrowser": SimTypeBottom(label="ICommDlgBrowser"),
    "ICommDlgBrowser2": SimTypeBottom(label="ICommDlgBrowser2"),
    "ICommDlgBrowser3": SimTypeBottom(label="ICommDlgBrowser3"),
    "ICommand": SimTypeBottom(label="ICommand"),
    "ICommandCost": SimTypeBottom(label="ICommandCost"),
    "ICommandPersist": SimTypeBottom(label="ICommandPersist"),
    "ICommandPrepare": SimTypeBottom(label="ICommandPrepare"),
    "ICommandProperties": SimTypeBottom(label="ICommandProperties"),
    "ICommandStream": SimTypeBottom(label="ICommandStream"),
    "ICommandText": SimTypeBottom(label="ICommandText"),
    "ICommandValidate": SimTypeBottom(label="ICommandValidate"),
    "ICommandWithParameters": SimTypeBottom(label="ICommandWithParameters"),
    "ICommonQuery": SimTypeBottom(label="ICommonQuery"),
    "IComparableConcept": SimTypeBottom(label="IComparableConcept"),
    "IComponent": SimTypeBottom(label="IComponent"),
    "IComponent2": SimTypeBottom(label="IComponent2"),
    "IComponentAuthenticate": SimTypeBottom(label="IComponentAuthenticate"),
    "IComponentData": SimTypeBottom(label="IComponentData"),
    "IComponentData2": SimTypeBottom(label="IComponentData2"),
    "IComponentType": SimTypeBottom(label="IComponentType"),
    "IComponentTypes": SimTypeBottom(label="IComponentTypes"),
    "IComponentUtil": SimTypeBottom(label="IComponentUtil"),
    "IComponents": SimTypeBottom(label="IComponents"),
    "IComponentsOld": SimTypeBottom(label="IComponentsOld"),
    "ICompositionFramePresentStatistics": SimTypeBottom(label="ICompositionFramePresentStatistics"),
    "IComprehensiveSpellCheckProvider": SimTypeBottom(label="IComprehensiveSpellCheckProvider"),
    "IComputerInfoChangeNotify": SimTypeBottom(label="IComputerInfoChangeNotify"),
    "ICondition": SimTypeBottom(label="ICondition"),
    "ICondition2": SimTypeBottom(label="ICondition2"),
    "IConditionFactory": SimTypeBottom(label="IConditionFactory"),
    "IConditionFactory2": SimTypeBottom(label="IConditionFactory2"),
    "IConditionGenerator": SimTypeBottom(label="IConditionGenerator"),
    "IConfigAsfWriter": SimTypeBottom(label="IConfigAsfWriter"),
    "IConfigAsfWriter2": SimTypeBottom(label="IConfigAsfWriter2"),
    "IConfigAviMux": SimTypeBottom(label="IConfigAviMux"),
    "IConfigInterleaving": SimTypeBottom(label="IConfigInterleaving"),
    "IConfigurationDataCollector": SimTypeBottom(label="IConfigurationDataCollector"),
    "IConnectableCredentialProviderCredential": SimTypeBottom(label="IConnectableCredentialProviderCredential"),
    "IConnectedIdentityProvider": SimTypeBottom(label="IConnectedIdentityProvider"),
    "IConnectionPoint": SimTypeBottom(label="IConnectionPoint"),
    "IConnectionPointContainer": SimTypeBottom(label="IConnectionPointContainer"),
    "IConnectionRequestCallback": SimTypeBottom(label="IConnectionRequestCallback"),
    "IConnector": SimTypeBottom(label="IConnector"),
    "IConsole": SimTypeBottom(label="IConsole"),
    "IConsole2": SimTypeBottom(label="IConsole2"),
    "IConsole3": SimTypeBottom(label="IConsole3"),
    "IConsoleNameSpace": SimTypeBottom(label="IConsoleNameSpace"),
    "IConsoleNameSpace2": SimTypeBottom(label="IConsoleNameSpace2"),
    "IConsolePower": SimTypeBottom(label="IConsolePower"),
    "IConsolePowerSink": SimTypeBottom(label="IConsolePowerSink"),
    "IConsoleVerb": SimTypeBottom(label="IConsoleVerb"),
    "IConstraintConflict": SimTypeBottom(label="IConstraintConflict"),
    "IConstructReplicaKeyMap": SimTypeBottom(label="IConstructReplicaKeyMap"),
    "IContact": SimTypeBottom(label="IContact"),
    "IContactAggregationAggregate": SimTypeBottom(label="IContactAggregationAggregate"),
    "IContactAggregationAggregateCollection": SimTypeBottom(label="IContactAggregationAggregateCollection"),
    "IContactAggregationContact": SimTypeBottom(label="IContactAggregationContact"),
    "IContactAggregationContactCollection": SimTypeBottom(label="IContactAggregationContactCollection"),
    "IContactAggregationGroup": SimTypeBottom(label="IContactAggregationGroup"),
    "IContactAggregationGroupCollection": SimTypeBottom(label="IContactAggregationGroupCollection"),
    "IContactAggregationLink": SimTypeBottom(label="IContactAggregationLink"),
    "IContactAggregationLinkCollection": SimTypeBottom(label="IContactAggregationLinkCollection"),
    "IContactAggregationManager": SimTypeBottom(label="IContactAggregationManager"),
    "IContactAggregationServerPerson": SimTypeBottom(label="IContactAggregationServerPerson"),
    "IContactAggregationServerPersonCollection": SimTypeBottom(label="IContactAggregationServerPersonCollection"),
    "IContactCollection": SimTypeBottom(label="IContactCollection"),
    "IContactManager": SimTypeBottom(label="IContactManager"),
    "IContactManagerInterop": SimTypeBottom(label="IContactManagerInterop"),
    "IContactProperties": SimTypeBottom(label="IContactProperties"),
    "IContactPropertyCollection": SimTypeBottom(label="IContactPropertyCollection"),
    "IContainerActivationHelper": SimTypeBottom(label="IContainerActivationHelper"),
    "IContentPrefetcherTaskTrigger": SimTypeBottom(label="IContentPrefetcherTaskTrigger"),
    "IContext": SimTypeBottom(label="IContext"),
    "IContextCallback": SimTypeBottom(label="IContextCallback"),
    "IContextMenu": SimTypeBottom(label="IContextMenu"),
    "IContextMenu2": SimTypeBottom(label="IContextMenu2"),
    "IContextMenu3": SimTypeBottom(label="IContextMenu3"),
    "IContextMenuCB": SimTypeBottom(label="IContextMenuCB"),
    "IContextMenuCallback": SimTypeBottom(label="IContextMenuCallback"),
    "IContextMenuCallback2": SimTypeBottom(label="IContextMenuCallback2"),
    "IContextMenuProvider": SimTypeBottom(label="IContextMenuProvider"),
    "IContextMenuSite": SimTypeBottom(label="IContextMenuSite"),
    "IContextProperties": SimTypeBottom(label="IContextProperties"),
    "IContextSecurityPerimeter": SimTypeBottom(label="IContextSecurityPerimeter"),
    "IContextState": SimTypeBottom(label="IContextState"),
    "IContinue": SimTypeBottom(label="IContinue"),
    "IContinueCallback": SimTypeBottom(label="IContinueCallback"),
    "IControlChangeNotify": SimTypeBottom(label="IControlChangeNotify"),
    "IControlInterface": SimTypeBottom(label="IControlInterface"),
    "IControlbar": SimTypeBottom(label="IControlbar"),
    "IConvertType": SimTypeBottom(label="IConvertType"),
    "ICopyHookA": SimTypeBottom(label="ICopyHookA"),
    "ICopyHookW": SimTypeBottom(label="ICopyHookW"),
    "ICorConfiguration": SimTypeBottom(label="ICorConfiguration"),
    "ICorProfilerAssemblyReferenceProvider": SimTypeBottom(label="ICorProfilerAssemblyReferenceProvider"),
    "ICorProfilerCallback": SimTypeBottom(label="ICorProfilerCallback"),
    "ICorProfilerCallback10": SimTypeBottom(label="ICorProfilerCallback10"),
    "ICorProfilerCallback11": SimTypeBottom(label="ICorProfilerCallback11"),
    "ICorProfilerCallback2": SimTypeBottom(label="ICorProfilerCallback2"),
    "ICorProfilerCallback3": SimTypeBottom(label="ICorProfilerCallback3"),
    "ICorProfilerCallback4": SimTypeBottom(label="ICorProfilerCallback4"),
    "ICorProfilerCallback5": SimTypeBottom(label="ICorProfilerCallback5"),
    "ICorProfilerCallback6": SimTypeBottom(label="ICorProfilerCallback6"),
    "ICorProfilerCallback7": SimTypeBottom(label="ICorProfilerCallback7"),
    "ICorProfilerCallback8": SimTypeBottom(label="ICorProfilerCallback8"),
    "ICorProfilerCallback9": SimTypeBottom(label="ICorProfilerCallback9"),
    "ICorProfilerFunctionControl": SimTypeBottom(label="ICorProfilerFunctionControl"),
    "ICorProfilerFunctionEnum": SimTypeBottom(label="ICorProfilerFunctionEnum"),
    "ICorProfilerInfo": SimTypeBottom(label="ICorProfilerInfo"),
    "ICorProfilerInfo10": SimTypeBottom(label="ICorProfilerInfo10"),
    "ICorProfilerInfo11": SimTypeBottom(label="ICorProfilerInfo11"),
    "ICorProfilerInfo12": SimTypeBottom(label="ICorProfilerInfo12"),
    "ICorProfilerInfo13": SimTypeBottom(label="ICorProfilerInfo13"),
    "ICorProfilerInfo14": SimTypeBottom(label="ICorProfilerInfo14"),
    "ICorProfilerInfo2": SimTypeBottom(label="ICorProfilerInfo2"),
    "ICorProfilerInfo3": SimTypeBottom(label="ICorProfilerInfo3"),
    "ICorProfilerInfo4": SimTypeBottom(label="ICorProfilerInfo4"),
    "ICorProfilerInfo5": SimTypeBottom(label="ICorProfilerInfo5"),
    "ICorProfilerInfo6": SimTypeBottom(label="ICorProfilerInfo6"),
    "ICorProfilerInfo7": SimTypeBottom(label="ICorProfilerInfo7"),
    "ICorProfilerInfo8": SimTypeBottom(label="ICorProfilerInfo8"),
    "ICorProfilerInfo9": SimTypeBottom(label="ICorProfilerInfo9"),
    "ICorProfilerMethodEnum": SimTypeBottom(label="ICorProfilerMethodEnum"),
    "ICorProfilerModuleEnum": SimTypeBottom(label="ICorProfilerModuleEnum"),
    "ICorProfilerObjectEnum": SimTypeBottom(label="ICorProfilerObjectEnum"),
    "ICorProfilerThreadEnum": SimTypeBottom(label="ICorProfilerThreadEnum"),
    "ICorRuntimeHost": SimTypeBottom(label="ICorRuntimeHost"),
    "ICorThreadpool": SimTypeBottom(label="ICorThreadpool"),
    "ICoreFragment": SimTypeBottom(label="ICoreFragment"),
    "ICoreFragmentInspector": SimTypeBottom(label="ICoreFragmentInspector"),
    "ICounterItem": SimTypeBottom(label="ICounterItem"),
    "ICounterItem2": SimTypeBottom(label="ICounterItem2"),
    "ICounters": SimTypeBottom(label="ICounters"),
    "ICreateDevEnum": SimTypeBottom(label="ICreateDevEnum"),
    "ICreateDeviceAccessAsync": SimTypeBottom(label="ICreateDeviceAccessAsync"),
    "ICreateErrorInfo": SimTypeBottom(label="ICreateErrorInfo"),
    "ICreateObject": SimTypeBottom(label="ICreateObject"),
    "ICreateProcessInputs": SimTypeBottom(label="ICreateProcessInputs"),
    "ICreatePropBagOnRegKey": SimTypeBottom(label="ICreatePropBagOnRegKey"),
    "ICreateRow": SimTypeBottom(label="ICreateRow"),
    "ICreateTypeInfo": SimTypeBottom(label="ICreateTypeInfo"),
    "ICreateTypeInfo2": SimTypeBottom(label="ICreateTypeInfo2"),
    "ICreateTypeLib": SimTypeBottom(label="ICreateTypeLib"),
    "ICreateTypeLib2": SimTypeBottom(label="ICreateTypeLib2"),
    "ICreateWithLocalTransaction": SimTypeBottom(label="ICreateWithLocalTransaction"),
    "ICreateWithTipTransactionEx": SimTypeBottom(label="ICreateWithTipTransactionEx"),
    "ICreateWithTransactionEx": SimTypeBottom(label="ICreateWithTransactionEx"),
    "ICreatingProcess": SimTypeBottom(label="ICreatingProcess"),
    "ICredentialProvider": SimTypeBottom(label="ICredentialProvider"),
    "ICredentialProviderCredential": SimTypeBottom(label="ICredentialProviderCredential"),
    "ICredentialProviderCredential2": SimTypeBottom(label="ICredentialProviderCredential2"),
    "ICredentialProviderCredentialEvents": SimTypeBottom(label="ICredentialProviderCredentialEvents"),
    "ICredentialProviderCredentialEvents2": SimTypeBottom(label="ICredentialProviderCredentialEvents2"),
    "ICredentialProviderCredentialWithFieldOptions": SimTypeBottom(label="ICredentialProviderCredentialWithFieldOptions"),
    "ICredentialProviderEvents": SimTypeBottom(label="ICredentialProviderEvents"),
    "ICredentialProviderFilter": SimTypeBottom(label="ICredentialProviderFilter"),
    "ICredentialProviderSetUserArray": SimTypeBottom(label="ICredentialProviderSetUserArray"),
    "ICredentialProviderUser": SimTypeBottom(label="ICredentialProviderUser"),
    "ICredentialProviderUserArray": SimTypeBottom(label="ICredentialProviderUserArray"),
    "ICrmCompensator": SimTypeBottom(label="ICrmCompensator"),
    "ICrmCompensatorVariants": SimTypeBottom(label="ICrmCompensatorVariants"),
    "ICrmFormatLogRecords": SimTypeBottom(label="ICrmFormatLogRecords"),
    "ICrmLogControl": SimTypeBottom(label="ICrmLogControl"),
    "ICrmMonitor": SimTypeBottom(label="ICrmMonitor"),
    "ICrmMonitorClerks": SimTypeBottom(label="ICrmMonitorClerks"),
    "ICrmMonitorLogRecords": SimTypeBottom(label="ICrmMonitorLogRecords"),
    "ICryptAttribute": SimTypeBottom(label="ICryptAttribute"),
    "ICryptAttributes": SimTypeBottom(label="ICryptAttributes"),
    "ICspAlgorithm": SimTypeBottom(label="ICspAlgorithm"),
    "ICspAlgorithms": SimTypeBottom(label="ICspAlgorithms"),
    "ICspInformation": SimTypeBottom(label="ICspInformation"),
    "ICspInformations": SimTypeBottom(label="ICspInformations"),
    "ICspStatus": SimTypeBottom(label="ICspStatus"),
    "ICspStatuses": SimTypeBottom(label="ICspStatuses"),
    "ICurrentItem": SimTypeBottom(label="ICurrentItem"),
    "ICurrentWorkingDirectory": SimTypeBottom(label="ICurrentWorkingDirectory"),
    "ICustomDestinationList": SimTypeBottom(label="ICustomDestinationList"),
    "ICustomDoc": SimTypeBottom(label="ICustomDoc"),
    "ICustomFilterInfo": SimTypeBottom(label="ICustomFilterInfo"),
    "ICustomNavigationProvider": SimTypeBottom(label="ICustomNavigationProvider"),
    "ID2D1AnalysisTransform": SimTypeBottom(label="ID2D1AnalysisTransform"),
    "ID2D1Bitmap": SimTypeBottom(label="ID2D1Bitmap"),
    "ID2D1Bitmap1": SimTypeBottom(label="ID2D1Bitmap1"),
    "ID2D1BitmapBrush": SimTypeBottom(label="ID2D1BitmapBrush"),
    "ID2D1BitmapBrush1": SimTypeBottom(label="ID2D1BitmapBrush1"),
    "ID2D1BitmapRenderTarget": SimTypeBottom(label="ID2D1BitmapRenderTarget"),
    "ID2D1BlendTransform": SimTypeBottom(label="ID2D1BlendTransform"),
    "ID2D1BorderTransform": SimTypeBottom(label="ID2D1BorderTransform"),
    "ID2D1BoundsAdjustmentTransform": SimTypeBottom(label="ID2D1BoundsAdjustmentTransform"),
    "ID2D1Brush": SimTypeBottom(label="ID2D1Brush"),
    "ID2D1ColorContext": SimTypeBottom(label="ID2D1ColorContext"),
    "ID2D1ColorContext1": SimTypeBottom(label="ID2D1ColorContext1"),
    "ID2D1CommandList": SimTypeBottom(label="ID2D1CommandList"),
    "ID2D1CommandSink": SimTypeBottom(label="ID2D1CommandSink"),
    "ID2D1CommandSink1": SimTypeBottom(label="ID2D1CommandSink1"),
    "ID2D1CommandSink2": SimTypeBottom(label="ID2D1CommandSink2"),
    "ID2D1CommandSink3": SimTypeBottom(label="ID2D1CommandSink3"),
    "ID2D1CommandSink4": SimTypeBottom(label="ID2D1CommandSink4"),
    "ID2D1CommandSink5": SimTypeBottom(label="ID2D1CommandSink5"),
    "ID2D1ComputeInfo": SimTypeBottom(label="ID2D1ComputeInfo"),
    "ID2D1ComputeTransform": SimTypeBottom(label="ID2D1ComputeTransform"),
    "ID2D1ConcreteTransform": SimTypeBottom(label="ID2D1ConcreteTransform"),
    "ID2D1DCRenderTarget": SimTypeBottom(label="ID2D1DCRenderTarget"),
    "ID2D1Device": SimTypeBottom(label="ID2D1Device"),
    "ID2D1Device1": SimTypeBottom(label="ID2D1Device1"),
    "ID2D1Device2": SimTypeBottom(label="ID2D1Device2"),
    "ID2D1Device3": SimTypeBottom(label="ID2D1Device3"),
    "ID2D1Device4": SimTypeBottom(label="ID2D1Device4"),
    "ID2D1Device5": SimTypeBottom(label="ID2D1Device5"),
    "ID2D1Device6": SimTypeBottom(label="ID2D1Device6"),
    "ID2D1Device7": SimTypeBottom(label="ID2D1Device7"),
    "ID2D1DeviceContext": SimTypeBottom(label="ID2D1DeviceContext"),
    "ID2D1DeviceContext1": SimTypeBottom(label="ID2D1DeviceContext1"),
    "ID2D1DeviceContext2": SimTypeBottom(label="ID2D1DeviceContext2"),
    "ID2D1DeviceContext3": SimTypeBottom(label="ID2D1DeviceContext3"),
    "ID2D1DeviceContext4": SimTypeBottom(label="ID2D1DeviceContext4"),
    "ID2D1DeviceContext5": SimTypeBottom(label="ID2D1DeviceContext5"),
    "ID2D1DeviceContext6": SimTypeBottom(label="ID2D1DeviceContext6"),
    "ID2D1DeviceContext7": SimTypeBottom(label="ID2D1DeviceContext7"),
    "ID2D1DrawInfo": SimTypeBottom(label="ID2D1DrawInfo"),
    "ID2D1DrawTransform": SimTypeBottom(label="ID2D1DrawTransform"),
    "ID2D1DrawingStateBlock": SimTypeBottom(label="ID2D1DrawingStateBlock"),
    "ID2D1DrawingStateBlock1": SimTypeBottom(label="ID2D1DrawingStateBlock1"),
    "ID2D1Effect": SimTypeBottom(label="ID2D1Effect"),
    "ID2D1EffectContext": SimTypeBottom(label="ID2D1EffectContext"),
    "ID2D1EffectContext1": SimTypeBottom(label="ID2D1EffectContext1"),
    "ID2D1EffectContext2": SimTypeBottom(label="ID2D1EffectContext2"),
    "ID2D1EffectImpl": SimTypeBottom(label="ID2D1EffectImpl"),
    "ID2D1EllipseGeometry": SimTypeBottom(label="ID2D1EllipseGeometry"),
    "ID2D1Factory": SimTypeBottom(label="ID2D1Factory"),
    "ID2D1Factory1": SimTypeBottom(label="ID2D1Factory1"),
    "ID2D1Factory2": SimTypeBottom(label="ID2D1Factory2"),
    "ID2D1Factory3": SimTypeBottom(label="ID2D1Factory3"),
    "ID2D1Factory4": SimTypeBottom(label="ID2D1Factory4"),
    "ID2D1Factory5": SimTypeBottom(label="ID2D1Factory5"),
    "ID2D1Factory6": SimTypeBottom(label="ID2D1Factory6"),
    "ID2D1Factory7": SimTypeBottom(label="ID2D1Factory7"),
    "ID2D1Factory8": SimTypeBottom(label="ID2D1Factory8"),
    "ID2D1GdiInteropRenderTarget": SimTypeBottom(label="ID2D1GdiInteropRenderTarget"),
    "ID2D1GdiMetafile": SimTypeBottom(label="ID2D1GdiMetafile"),
    "ID2D1GdiMetafile1": SimTypeBottom(label="ID2D1GdiMetafile1"),
    "ID2D1GdiMetafileSink": SimTypeBottom(label="ID2D1GdiMetafileSink"),
    "ID2D1GdiMetafileSink1": SimTypeBottom(label="ID2D1GdiMetafileSink1"),
    "ID2D1Geometry": SimTypeBottom(label="ID2D1Geometry"),
    "ID2D1GeometryGroup": SimTypeBottom(label="ID2D1GeometryGroup"),
    "ID2D1GeometryRealization": SimTypeBottom(label="ID2D1GeometryRealization"),
    "ID2D1GeometrySink": SimTypeBottom(label="ID2D1GeometrySink"),
    "ID2D1GradientMesh": SimTypeBottom(label="ID2D1GradientMesh"),
    "ID2D1GradientStopCollection": SimTypeBottom(label="ID2D1GradientStopCollection"),
    "ID2D1GradientStopCollection1": SimTypeBottom(label="ID2D1GradientStopCollection1"),
    "ID2D1HwndRenderTarget": SimTypeBottom(label="ID2D1HwndRenderTarget"),
    "ID2D1Image": SimTypeBottom(label="ID2D1Image"),
    "ID2D1ImageBrush": SimTypeBottom(label="ID2D1ImageBrush"),
    "ID2D1ImageSource": SimTypeBottom(label="ID2D1ImageSource"),
    "ID2D1ImageSourceFromWic": SimTypeBottom(label="ID2D1ImageSourceFromWic"),
    "ID2D1Ink": SimTypeBottom(label="ID2D1Ink"),
    "ID2D1InkStyle": SimTypeBottom(label="ID2D1InkStyle"),
    "ID2D1Layer": SimTypeBottom(label="ID2D1Layer"),
    "ID2D1LinearGradientBrush": SimTypeBottom(label="ID2D1LinearGradientBrush"),
    "ID2D1LookupTable3D": SimTypeBottom(label="ID2D1LookupTable3D"),
    "ID2D1Mesh": SimTypeBottom(label="ID2D1Mesh"),
    "ID2D1Multithread": SimTypeBottom(label="ID2D1Multithread"),
    "ID2D1OffsetTransform": SimTypeBottom(label="ID2D1OffsetTransform"),
    "ID2D1PathGeometry": SimTypeBottom(label="ID2D1PathGeometry"),
    "ID2D1PathGeometry1": SimTypeBottom(label="ID2D1PathGeometry1"),
    "ID2D1PrintControl": SimTypeBottom(label="ID2D1PrintControl"),
    "ID2D1Properties": SimTypeBottom(label="ID2D1Properties"),
    "ID2D1RadialGradientBrush": SimTypeBottom(label="ID2D1RadialGradientBrush"),
    "ID2D1RectangleGeometry": SimTypeBottom(label="ID2D1RectangleGeometry"),
    "ID2D1RenderInfo": SimTypeBottom(label="ID2D1RenderInfo"),
    "ID2D1RenderTarget": SimTypeBottom(label="ID2D1RenderTarget"),
    "ID2D1Resource": SimTypeBottom(label="ID2D1Resource"),
    "ID2D1ResourceTexture": SimTypeBottom(label="ID2D1ResourceTexture"),
    "ID2D1RoundedRectangleGeometry": SimTypeBottom(label="ID2D1RoundedRectangleGeometry"),
    "ID2D1SimplifiedGeometrySink": SimTypeBottom(label="ID2D1SimplifiedGeometrySink"),
    "ID2D1SolidColorBrush": SimTypeBottom(label="ID2D1SolidColorBrush"),
    "ID2D1SourceTransform": SimTypeBottom(label="ID2D1SourceTransform"),
    "ID2D1SpriteBatch": SimTypeBottom(label="ID2D1SpriteBatch"),
    "ID2D1StrokeStyle": SimTypeBottom(label="ID2D1StrokeStyle"),
    "ID2D1StrokeStyle1": SimTypeBottom(label="ID2D1StrokeStyle1"),
    "ID2D1SvgAttribute": SimTypeBottom(label="ID2D1SvgAttribute"),
    "ID2D1SvgDocument": SimTypeBottom(label="ID2D1SvgDocument"),
    "ID2D1SvgElement": SimTypeBottom(label="ID2D1SvgElement"),
    "ID2D1SvgGlyphStyle": SimTypeBottom(label="ID2D1SvgGlyphStyle"),
    "ID2D1SvgPaint": SimTypeBottom(label="ID2D1SvgPaint"),
    "ID2D1SvgPathData": SimTypeBottom(label="ID2D1SvgPathData"),
    "ID2D1SvgPointCollection": SimTypeBottom(label="ID2D1SvgPointCollection"),
    "ID2D1SvgStrokeDashArray": SimTypeBottom(label="ID2D1SvgStrokeDashArray"),
    "ID2D1TessellationSink": SimTypeBottom(label="ID2D1TessellationSink"),
    "ID2D1Transform": SimTypeBottom(label="ID2D1Transform"),
    "ID2D1TransformGraph": SimTypeBottom(label="ID2D1TransformGraph"),
    "ID2D1TransformNode": SimTypeBottom(label="ID2D1TransformNode"),
    "ID2D1TransformedGeometry": SimTypeBottom(label="ID2D1TransformedGeometry"),
    "ID2D1TransformedImageSource": SimTypeBottom(label="ID2D1TransformedImageSource"),
    "ID2D1VertexBuffer": SimTypeBottom(label="ID2D1VertexBuffer"),
    "ID3D10Asynchronous": SimTypeBottom(label="ID3D10Asynchronous"),
    "ID3D10BlendState": SimTypeBottom(label="ID3D10BlendState"),
    "ID3D10BlendState1": SimTypeBottom(label="ID3D10BlendState1"),
    "ID3D10Buffer": SimTypeBottom(label="ID3D10Buffer"),
    "ID3D10Counter": SimTypeBottom(label="ID3D10Counter"),
    "ID3D10Debug": SimTypeBottom(label="ID3D10Debug"),
    "ID3D10DepthStencilState": SimTypeBottom(label="ID3D10DepthStencilState"),
    "ID3D10DepthStencilView": SimTypeBottom(label="ID3D10DepthStencilView"),
    "ID3D10Device": SimTypeBottom(label="ID3D10Device"),
    "ID3D10Device1": SimTypeBottom(label="ID3D10Device1"),
    "ID3D10DeviceChild": SimTypeBottom(label="ID3D10DeviceChild"),
    "ID3D10Effect": SimTypeBottom(label="ID3D10Effect"),
    "ID3D10EffectBlendVariable": SimTypeBottom(label="ID3D10EffectBlendVariable"),
    "ID3D10EffectConstantBuffer": SimTypeBottom(label="ID3D10EffectConstantBuffer"),
    "ID3D10EffectDepthStencilVariable": SimTypeBottom(label="ID3D10EffectDepthStencilVariable"),
    "ID3D10EffectDepthStencilViewVariable": SimTypeBottom(label="ID3D10EffectDepthStencilViewVariable"),
    "ID3D10EffectMatrixVariable": SimTypeBottom(label="ID3D10EffectMatrixVariable"),
    "ID3D10EffectPass": SimTypeBottom(label="ID3D10EffectPass"),
    "ID3D10EffectPool": SimTypeBottom(label="ID3D10EffectPool"),
    "ID3D10EffectRasterizerVariable": SimTypeBottom(label="ID3D10EffectRasterizerVariable"),
    "ID3D10EffectRenderTargetViewVariable": SimTypeBottom(label="ID3D10EffectRenderTargetViewVariable"),
    "ID3D10EffectSamplerVariable": SimTypeBottom(label="ID3D10EffectSamplerVariable"),
    "ID3D10EffectScalarVariable": SimTypeBottom(label="ID3D10EffectScalarVariable"),
    "ID3D10EffectShaderResourceVariable": SimTypeBottom(label="ID3D10EffectShaderResourceVariable"),
    "ID3D10EffectShaderVariable": SimTypeBottom(label="ID3D10EffectShaderVariable"),
    "ID3D10EffectStringVariable": SimTypeBottom(label="ID3D10EffectStringVariable"),
    "ID3D10EffectTechnique": SimTypeBottom(label="ID3D10EffectTechnique"),
    "ID3D10EffectType": SimTypeBottom(label="ID3D10EffectType"),
    "ID3D10EffectVariable": SimTypeBottom(label="ID3D10EffectVariable"),
    "ID3D10EffectVectorVariable": SimTypeBottom(label="ID3D10EffectVectorVariable"),
    "ID3D10GeometryShader": SimTypeBottom(label="ID3D10GeometryShader"),
    "ID3D10InfoQueue": SimTypeBottom(label="ID3D10InfoQueue"),
    "ID3D10InputLayout": SimTypeBottom(label="ID3D10InputLayout"),
    "ID3D10Multithread": SimTypeBottom(label="ID3D10Multithread"),
    "ID3D10PixelShader": SimTypeBottom(label="ID3D10PixelShader"),
    "ID3D10Predicate": SimTypeBottom(label="ID3D10Predicate"),
    "ID3D10Query": SimTypeBottom(label="ID3D10Query"),
    "ID3D10RasterizerState": SimTypeBottom(label="ID3D10RasterizerState"),
    "ID3D10RenderTargetView": SimTypeBottom(label="ID3D10RenderTargetView"),
    "ID3D10Resource": SimTypeBottom(label="ID3D10Resource"),
    "ID3D10SamplerState": SimTypeBottom(label="ID3D10SamplerState"),
    "ID3D10ShaderReflection": SimTypeBottom(label="ID3D10ShaderReflection"),
    "ID3D10ShaderReflection1": SimTypeBottom(label="ID3D10ShaderReflection1"),
    "ID3D10ShaderReflectionConstantBuffer": SimTypeBottom(label="ID3D10ShaderReflectionConstantBuffer"),
    "ID3D10ShaderReflectionType": SimTypeBottom(label="ID3D10ShaderReflectionType"),
    "ID3D10ShaderReflectionVariable": SimTypeBottom(label="ID3D10ShaderReflectionVariable"),
    "ID3D10ShaderResourceView": SimTypeBottom(label="ID3D10ShaderResourceView"),
    "ID3D10ShaderResourceView1": SimTypeBottom(label="ID3D10ShaderResourceView1"),
    "ID3D10StateBlock": SimTypeBottom(label="ID3D10StateBlock"),
    "ID3D10SwitchToRef": SimTypeBottom(label="ID3D10SwitchToRef"),
    "ID3D10Texture1D": SimTypeBottom(label="ID3D10Texture1D"),
    "ID3D10Texture2D": SimTypeBottom(label="ID3D10Texture2D"),
    "ID3D10Texture3D": SimTypeBottom(label="ID3D10Texture3D"),
    "ID3D10VertexShader": SimTypeBottom(label="ID3D10VertexShader"),
    "ID3D10View": SimTypeBottom(label="ID3D10View"),
    "ID3D11Asynchronous": SimTypeBottom(label="ID3D11Asynchronous"),
    "ID3D11AuthenticatedChannel": SimTypeBottom(label="ID3D11AuthenticatedChannel"),
    "ID3D11BlendState": SimTypeBottom(label="ID3D11BlendState"),
    "ID3D11BlendState1": SimTypeBottom(label="ID3D11BlendState1"),
    "ID3D11Buffer": SimTypeBottom(label="ID3D11Buffer"),
    "ID3D11ClassInstance": SimTypeBottom(label="ID3D11ClassInstance"),
    "ID3D11ClassLinkage": SimTypeBottom(label="ID3D11ClassLinkage"),
    "ID3D11CommandList": SimTypeBottom(label="ID3D11CommandList"),
    "ID3D11ComputeShader": SimTypeBottom(label="ID3D11ComputeShader"),
    "ID3D11Counter": SimTypeBottom(label="ID3D11Counter"),
    "ID3D11CryptoSession": SimTypeBottom(label="ID3D11CryptoSession"),
    "ID3D11Debug": SimTypeBottom(label="ID3D11Debug"),
    "ID3D11DepthStencilState": SimTypeBottom(label="ID3D11DepthStencilState"),
    "ID3D11DepthStencilView": SimTypeBottom(label="ID3D11DepthStencilView"),
    "ID3D11Device": SimTypeBottom(label="ID3D11Device"),
    "ID3D11Device1": SimTypeBottom(label="ID3D11Device1"),
    "ID3D11Device2": SimTypeBottom(label="ID3D11Device2"),
    "ID3D11Device3": SimTypeBottom(label="ID3D11Device3"),
    "ID3D11Device4": SimTypeBottom(label="ID3D11Device4"),
    "ID3D11Device5": SimTypeBottom(label="ID3D11Device5"),
    "ID3D11DeviceChild": SimTypeBottom(label="ID3D11DeviceChild"),
    "ID3D11DeviceContext": SimTypeBottom(label="ID3D11DeviceContext"),
    "ID3D11DeviceContext1": SimTypeBottom(label="ID3D11DeviceContext1"),
    "ID3D11DeviceContext2": SimTypeBottom(label="ID3D11DeviceContext2"),
    "ID3D11DeviceContext3": SimTypeBottom(label="ID3D11DeviceContext3"),
    "ID3D11DeviceContext4": SimTypeBottom(label="ID3D11DeviceContext4"),
    "ID3D11DomainShader": SimTypeBottom(label="ID3D11DomainShader"),
    "ID3D11Fence": SimTypeBottom(label="ID3D11Fence"),
    "ID3D11FunctionLinkingGraph": SimTypeBottom(label="ID3D11FunctionLinkingGraph"),
    "ID3D11FunctionParameterReflection": SimTypeBottom(label="ID3D11FunctionParameterReflection"),
    "ID3D11FunctionReflection": SimTypeBottom(label="ID3D11FunctionReflection"),
    "ID3D11GeometryShader": SimTypeBottom(label="ID3D11GeometryShader"),
    "ID3D11HullShader": SimTypeBottom(label="ID3D11HullShader"),
    "ID3D11InfoQueue": SimTypeBottom(label="ID3D11InfoQueue"),
    "ID3D11InputLayout": SimTypeBottom(label="ID3D11InputLayout"),
    "ID3D11LibraryReflection": SimTypeBottom(label="ID3D11LibraryReflection"),
    "ID3D11Linker": SimTypeBottom(label="ID3D11Linker"),
    "ID3D11LinkingNode": SimTypeBottom(label="ID3D11LinkingNode"),
    "ID3D11Module": SimTypeBottom(label="ID3D11Module"),
    "ID3D11ModuleInstance": SimTypeBottom(label="ID3D11ModuleInstance"),
    "ID3D11Multithread": SimTypeBottom(label="ID3D11Multithread"),
    "ID3D11On12Device": SimTypeBottom(label="ID3D11On12Device"),
    "ID3D11On12Device1": SimTypeBottom(label="ID3D11On12Device1"),
    "ID3D11On12Device2": SimTypeBottom(label="ID3D11On12Device2"),
    "ID3D11PixelShader": SimTypeBottom(label="ID3D11PixelShader"),
    "ID3D11Predicate": SimTypeBottom(label="ID3D11Predicate"),
    "ID3D11Query": SimTypeBottom(label="ID3D11Query"),
    "ID3D11Query1": SimTypeBottom(label="ID3D11Query1"),
    "ID3D11RasterizerState": SimTypeBottom(label="ID3D11RasterizerState"),
    "ID3D11RasterizerState1": SimTypeBottom(label="ID3D11RasterizerState1"),
    "ID3D11RasterizerState2": SimTypeBottom(label="ID3D11RasterizerState2"),
    "ID3D11RefDefaultTrackingOptions": SimTypeBottom(label="ID3D11RefDefaultTrackingOptions"),
    "ID3D11RefTrackingOptions": SimTypeBottom(label="ID3D11RefTrackingOptions"),
    "ID3D11RenderTargetView": SimTypeBottom(label="ID3D11RenderTargetView"),
    "ID3D11RenderTargetView1": SimTypeBottom(label="ID3D11RenderTargetView1"),
    "ID3D11Resource": SimTypeBottom(label="ID3D11Resource"),
    "ID3D11SamplerState": SimTypeBottom(label="ID3D11SamplerState"),
    "ID3D11ShaderReflection": SimTypeBottom(label="ID3D11ShaderReflection"),
    "ID3D11ShaderReflectionConstantBuffer": SimTypeBottom(label="ID3D11ShaderReflectionConstantBuffer"),
    "ID3D11ShaderReflectionType": SimTypeBottom(label="ID3D11ShaderReflectionType"),
    "ID3D11ShaderReflectionVariable": SimTypeBottom(label="ID3D11ShaderReflectionVariable"),
    "ID3D11ShaderResourceView": SimTypeBottom(label="ID3D11ShaderResourceView"),
    "ID3D11ShaderResourceView1": SimTypeBottom(label="ID3D11ShaderResourceView1"),
    "ID3D11ShaderTrace": SimTypeBottom(label="ID3D11ShaderTrace"),
    "ID3D11ShaderTraceFactory": SimTypeBottom(label="ID3D11ShaderTraceFactory"),
    "ID3D11SwitchToRef": SimTypeBottom(label="ID3D11SwitchToRef"),
    "ID3D11Texture1D": SimTypeBottom(label="ID3D11Texture1D"),
    "ID3D11Texture2D": SimTypeBottom(label="ID3D11Texture2D"),
    "ID3D11Texture2D1": SimTypeBottom(label="ID3D11Texture2D1"),
    "ID3D11Texture3D": SimTypeBottom(label="ID3D11Texture3D"),
    "ID3D11Texture3D1": SimTypeBottom(label="ID3D11Texture3D1"),
    "ID3D11TracingDevice": SimTypeBottom(label="ID3D11TracingDevice"),
    "ID3D11UnorderedAccessView": SimTypeBottom(label="ID3D11UnorderedAccessView"),
    "ID3D11UnorderedAccessView1": SimTypeBottom(label="ID3D11UnorderedAccessView1"),
    "ID3D11VertexShader": SimTypeBottom(label="ID3D11VertexShader"),
    "ID3D11VideoContext": SimTypeBottom(label="ID3D11VideoContext"),
    "ID3D11VideoContext1": SimTypeBottom(label="ID3D11VideoContext1"),
    "ID3D11VideoContext2": SimTypeBottom(label="ID3D11VideoContext2"),
    "ID3D11VideoContext3": SimTypeBottom(label="ID3D11VideoContext3"),
    "ID3D11VideoDecoder": SimTypeBottom(label="ID3D11VideoDecoder"),
    "ID3D11VideoDecoderOutputView": SimTypeBottom(label="ID3D11VideoDecoderOutputView"),
    "ID3D11VideoDevice": SimTypeBottom(label="ID3D11VideoDevice"),
    "ID3D11VideoDevice1": SimTypeBottom(label="ID3D11VideoDevice1"),
    "ID3D11VideoDevice2": SimTypeBottom(label="ID3D11VideoDevice2"),
    "ID3D11VideoProcessor": SimTypeBottom(label="ID3D11VideoProcessor"),
    "ID3D11VideoProcessorEnumerator": SimTypeBottom(label="ID3D11VideoProcessorEnumerator"),
    "ID3D11VideoProcessorEnumerator1": SimTypeBottom(label="ID3D11VideoProcessorEnumerator1"),
    "ID3D11VideoProcessorInputView": SimTypeBottom(label="ID3D11VideoProcessorInputView"),
    "ID3D11VideoProcessorOutputView": SimTypeBottom(label="ID3D11VideoProcessorOutputView"),
    "ID3D11View": SimTypeBottom(label="ID3D11View"),
    "ID3D12CommandAllocator": SimTypeBottom(label="ID3D12CommandAllocator"),
    "ID3D12CommandList": SimTypeBottom(label="ID3D12CommandList"),
    "ID3D12CommandQueue": SimTypeBottom(label="ID3D12CommandQueue"),
    "ID3D12CommandSignature": SimTypeBottom(label="ID3D12CommandSignature"),
    "ID3D12Debug": SimTypeBottom(label="ID3D12Debug"),
    "ID3D12Debug1": SimTypeBottom(label="ID3D12Debug1"),
    "ID3D12Debug2": SimTypeBottom(label="ID3D12Debug2"),
    "ID3D12Debug3": SimTypeBottom(label="ID3D12Debug3"),
    "ID3D12Debug4": SimTypeBottom(label="ID3D12Debug4"),
    "ID3D12Debug5": SimTypeBottom(label="ID3D12Debug5"),
    "ID3D12Debug6": SimTypeBottom(label="ID3D12Debug6"),
    "ID3D12DebugCommandList": SimTypeBottom(label="ID3D12DebugCommandList"),
    "ID3D12DebugCommandList1": SimTypeBottom(label="ID3D12DebugCommandList1"),
    "ID3D12DebugCommandList2": SimTypeBottom(label="ID3D12DebugCommandList2"),
    "ID3D12DebugCommandList3": SimTypeBottom(label="ID3D12DebugCommandList3"),
    "ID3D12DebugCommandQueue": SimTypeBottom(label="ID3D12DebugCommandQueue"),
    "ID3D12DebugCommandQueue1": SimTypeBottom(label="ID3D12DebugCommandQueue1"),
    "ID3D12DebugDevice": SimTypeBottom(label="ID3D12DebugDevice"),
    "ID3D12DebugDevice1": SimTypeBottom(label="ID3D12DebugDevice1"),
    "ID3D12DebugDevice2": SimTypeBottom(label="ID3D12DebugDevice2"),
    "ID3D12DescriptorHeap": SimTypeBottom(label="ID3D12DescriptorHeap"),
    "ID3D12Device": SimTypeBottom(label="ID3D12Device"),
    "ID3D12Device1": SimTypeBottom(label="ID3D12Device1"),
    "ID3D12Device10": SimTypeBottom(label="ID3D12Device10"),
    "ID3D12Device11": SimTypeBottom(label="ID3D12Device11"),
    "ID3D12Device12": SimTypeBottom(label="ID3D12Device12"),
    "ID3D12Device13": SimTypeBottom(label="ID3D12Device13"),
    "ID3D12Device2": SimTypeBottom(label="ID3D12Device2"),
    "ID3D12Device3": SimTypeBottom(label="ID3D12Device3"),
    "ID3D12Device4": SimTypeBottom(label="ID3D12Device4"),
    "ID3D12Device5": SimTypeBottom(label="ID3D12Device5"),
    "ID3D12Device6": SimTypeBottom(label="ID3D12Device6"),
    "ID3D12Device7": SimTypeBottom(label="ID3D12Device7"),
    "ID3D12Device8": SimTypeBottom(label="ID3D12Device8"),
    "ID3D12Device9": SimTypeBottom(label="ID3D12Device9"),
    "ID3D12DeviceChild": SimTypeBottom(label="ID3D12DeviceChild"),
    "ID3D12DeviceConfiguration": SimTypeBottom(label="ID3D12DeviceConfiguration"),
    "ID3D12DeviceFactory": SimTypeBottom(label="ID3D12DeviceFactory"),
    "ID3D12DeviceRemovedExtendedData": SimTypeBottom(label="ID3D12DeviceRemovedExtendedData"),
    "ID3D12DeviceRemovedExtendedData1": SimTypeBottom(label="ID3D12DeviceRemovedExtendedData1"),
    "ID3D12DeviceRemovedExtendedData2": SimTypeBottom(label="ID3D12DeviceRemovedExtendedData2"),
    "ID3D12DeviceRemovedExtendedDataSettings": SimTypeBottom(label="ID3D12DeviceRemovedExtendedDataSettings"),
    "ID3D12DeviceRemovedExtendedDataSettings1": SimTypeBottom(label="ID3D12DeviceRemovedExtendedDataSettings1"),
    "ID3D12DeviceRemovedExtendedDataSettings2": SimTypeBottom(label="ID3D12DeviceRemovedExtendedDataSettings2"),
    "ID3D12Fence": SimTypeBottom(label="ID3D12Fence"),
    "ID3D12Fence1": SimTypeBottom(label="ID3D12Fence1"),
    "ID3D12FunctionParameterReflection": SimTypeBottom(label="ID3D12FunctionParameterReflection"),
    "ID3D12FunctionReflection": SimTypeBottom(label="ID3D12FunctionReflection"),
    "ID3D12GraphicsCommandList": SimTypeBottom(label="ID3D12GraphicsCommandList"),
    "ID3D12GraphicsCommandList1": SimTypeBottom(label="ID3D12GraphicsCommandList1"),
    "ID3D12GraphicsCommandList2": SimTypeBottom(label="ID3D12GraphicsCommandList2"),
    "ID3D12GraphicsCommandList3": SimTypeBottom(label="ID3D12GraphicsCommandList3"),
    "ID3D12GraphicsCommandList4": SimTypeBottom(label="ID3D12GraphicsCommandList4"),
    "ID3D12GraphicsCommandList5": SimTypeBottom(label="ID3D12GraphicsCommandList5"),
    "ID3D12GraphicsCommandList6": SimTypeBottom(label="ID3D12GraphicsCommandList6"),
    "ID3D12GraphicsCommandList7": SimTypeBottom(label="ID3D12GraphicsCommandList7"),
    "ID3D12GraphicsCommandList8": SimTypeBottom(label="ID3D12GraphicsCommandList8"),
    "ID3D12GraphicsCommandList9": SimTypeBottom(label="ID3D12GraphicsCommandList9"),
    "ID3D12Heap": SimTypeBottom(label="ID3D12Heap"),
    "ID3D12Heap1": SimTypeBottom(label="ID3D12Heap1"),
    "ID3D12InfoQueue": SimTypeBottom(label="ID3D12InfoQueue"),
    "ID3D12InfoQueue1": SimTypeBottom(label="ID3D12InfoQueue1"),
    "ID3D12LibraryReflection": SimTypeBottom(label="ID3D12LibraryReflection"),
    "ID3D12LifetimeOwner": SimTypeBottom(label="ID3D12LifetimeOwner"),
    "ID3D12LifetimeTracker": SimTypeBottom(label="ID3D12LifetimeTracker"),
    "ID3D12ManualWriteTrackingResource": SimTypeBottom(label="ID3D12ManualWriteTrackingResource"),
    "ID3D12MetaCommand": SimTypeBottom(label="ID3D12MetaCommand"),
    "ID3D12Object": SimTypeBottom(label="ID3D12Object"),
    "ID3D12Pageable": SimTypeBottom(label="ID3D12Pageable"),
    "ID3D12PipelineLibrary": SimTypeBottom(label="ID3D12PipelineLibrary"),
    "ID3D12PipelineLibrary1": SimTypeBottom(label="ID3D12PipelineLibrary1"),
    "ID3D12PipelineState": SimTypeBottom(label="ID3D12PipelineState"),
    "ID3D12ProtectedResourceSession": SimTypeBottom(label="ID3D12ProtectedResourceSession"),
    "ID3D12ProtectedResourceSession1": SimTypeBottom(label="ID3D12ProtectedResourceSession1"),
    "ID3D12ProtectedSession": SimTypeBottom(label="ID3D12ProtectedSession"),
    "ID3D12QueryHeap": SimTypeBottom(label="ID3D12QueryHeap"),
    "ID3D12Resource": SimTypeBottom(label="ID3D12Resource"),
    "ID3D12Resource1": SimTypeBottom(label="ID3D12Resource1"),
    "ID3D12Resource2": SimTypeBottom(label="ID3D12Resource2"),
    "ID3D12RootSignature": SimTypeBottom(label="ID3D12RootSignature"),
    "ID3D12RootSignatureDeserializer": SimTypeBottom(label="ID3D12RootSignatureDeserializer"),
    "ID3D12SDKConfiguration": SimTypeBottom(label="ID3D12SDKConfiguration"),
    "ID3D12SDKConfiguration1": SimTypeBottom(label="ID3D12SDKConfiguration1"),
    "ID3D12ShaderCacheSession": SimTypeBottom(label="ID3D12ShaderCacheSession"),
    "ID3D12ShaderReflection": SimTypeBottom(label="ID3D12ShaderReflection"),
    "ID3D12ShaderReflectionConstantBuffer": SimTypeBottom(label="ID3D12ShaderReflectionConstantBuffer"),
    "ID3D12ShaderReflectionType": SimTypeBottom(label="ID3D12ShaderReflectionType"),
    "ID3D12ShaderReflectionVariable": SimTypeBottom(label="ID3D12ShaderReflectionVariable"),
    "ID3D12SharingContract": SimTypeBottom(label="ID3D12SharingContract"),
    "ID3D12StateObject": SimTypeBottom(label="ID3D12StateObject"),
    "ID3D12StateObjectProperties": SimTypeBottom(label="ID3D12StateObjectProperties"),
    "ID3D12SwapChainAssistant": SimTypeBottom(label="ID3D12SwapChainAssistant"),
    "ID3D12Tools": SimTypeBottom(label="ID3D12Tools"),
    "ID3D12VersionedRootSignatureDeserializer": SimTypeBottom(label="ID3D12VersionedRootSignatureDeserializer"),
    "ID3D12VideoDecodeCommandList": SimTypeBottom(label="ID3D12VideoDecodeCommandList"),
    "ID3D12VideoDecodeCommandList1": SimTypeBottom(label="ID3D12VideoDecodeCommandList1"),
    "ID3D12VideoDecodeCommandList2": SimTypeBottom(label="ID3D12VideoDecodeCommandList2"),
    "ID3D12VideoDecodeCommandList3": SimTypeBottom(label="ID3D12VideoDecodeCommandList3"),
    "ID3D12VideoDecoder": SimTypeBottom(label="ID3D12VideoDecoder"),
    "ID3D12VideoDecoder1": SimTypeBottom(label="ID3D12VideoDecoder1"),
    "ID3D12VideoDecoderHeap": SimTypeBottom(label="ID3D12VideoDecoderHeap"),
    "ID3D12VideoDecoderHeap1": SimTypeBottom(label="ID3D12VideoDecoderHeap1"),
    "ID3D12VideoDevice": SimTypeBottom(label="ID3D12VideoDevice"),
    "ID3D12VideoDevice1": SimTypeBottom(label="ID3D12VideoDevice1"),
    "ID3D12VideoDevice2": SimTypeBottom(label="ID3D12VideoDevice2"),
    "ID3D12VideoDevice3": SimTypeBottom(label="ID3D12VideoDevice3"),
    "ID3D12VideoEncodeCommandList": SimTypeBottom(label="ID3D12VideoEncodeCommandList"),
    "ID3D12VideoEncodeCommandList1": SimTypeBottom(label="ID3D12VideoEncodeCommandList1"),
    "ID3D12VideoEncodeCommandList2": SimTypeBottom(label="ID3D12VideoEncodeCommandList2"),
    "ID3D12VideoEncodeCommandList3": SimTypeBottom(label="ID3D12VideoEncodeCommandList3"),
    "ID3D12VideoEncoder": SimTypeBottom(label="ID3D12VideoEncoder"),
    "ID3D12VideoEncoderHeap": SimTypeBottom(label="ID3D12VideoEncoderHeap"),
    "ID3D12VideoExtensionCommand": SimTypeBottom(label="ID3D12VideoExtensionCommand"),
    "ID3D12VideoMotionEstimator": SimTypeBottom(label="ID3D12VideoMotionEstimator"),
    "ID3D12VideoMotionVectorHeap": SimTypeBottom(label="ID3D12VideoMotionVectorHeap"),
    "ID3D12VideoProcessCommandList": SimTypeBottom(label="ID3D12VideoProcessCommandList"),
    "ID3D12VideoProcessCommandList1": SimTypeBottom(label="ID3D12VideoProcessCommandList1"),
    "ID3D12VideoProcessCommandList2": SimTypeBottom(label="ID3D12VideoProcessCommandList2"),
    "ID3D12VideoProcessCommandList3": SimTypeBottom(label="ID3D12VideoProcessCommandList3"),
    "ID3D12VideoProcessor": SimTypeBottom(label="ID3D12VideoProcessor"),
    "ID3D12VideoProcessor1": SimTypeBottom(label="ID3D12VideoProcessor1"),
    "ID3D12VirtualizationGuestDevice": SimTypeBottom(label="ID3D12VirtualizationGuestDevice"),
    "ID3DBlob": SimTypeBottom(label="ID3DBlob"),
    "ID3DDestructionNotifier": SimTypeBottom(label="ID3DDestructionNotifier"),
    "ID3DDeviceContextState": SimTypeBottom(label="ID3DDeviceContextState"),
    "ID3DInclude": SimTypeBottom(label="ID3DInclude"),
    "ID3DUserDefinedAnnotation": SimTypeBottom(label="ID3DUserDefinedAnnotation"),
    "ID3DX11FFT": SimTypeBottom(label="ID3DX11FFT"),
    "ID3DX11Scan": SimTypeBottom(label="ID3DX11Scan"),
    "ID3DX11SegmentedScan": SimTypeBottom(label="ID3DX11SegmentedScan"),
    "IDBAsynchNotify": SimTypeBottom(label="IDBAsynchNotify"),
    "IDBAsynchStatus": SimTypeBottom(label="IDBAsynchStatus"),
    "IDBBinderProperties": SimTypeBottom(label="IDBBinderProperties"),
    "IDBCreateCommand": SimTypeBottom(label="IDBCreateCommand"),
    "IDBCreateSession": SimTypeBottom(label="IDBCreateSession"),
    "IDBDataSourceAdmin": SimTypeBottom(label="IDBDataSourceAdmin"),
    "IDBInfo": SimTypeBottom(label="IDBInfo"),
    "IDBInitialize": SimTypeBottom(label="IDBInitialize"),
    "IDBPromptInitialize": SimTypeBottom(label="IDBPromptInitialize"),
    "IDBProperties": SimTypeBottom(label="IDBProperties"),
    "IDBSchemaCommand": SimTypeBottom(label="IDBSchemaCommand"),
    "IDBSchemaRowset": SimTypeBottom(label="IDBSchemaRowset"),
    "IDCInfo": SimTypeBottom(label="IDCInfo"),
    "IDCompositionAffineTransform2DEffect": SimTypeBottom(label="IDCompositionAffineTransform2DEffect"),
    "IDCompositionAnimation": SimTypeBottom(label="IDCompositionAnimation"),
    "IDCompositionArithmeticCompositeEffect": SimTypeBottom(label="IDCompositionArithmeticCompositeEffect"),
    "IDCompositionBlendEffect": SimTypeBottom(label="IDCompositionBlendEffect"),
    "IDCompositionBrightnessEffect": SimTypeBottom(label="IDCompositionBrightnessEffect"),
    "IDCompositionClip": SimTypeBottom(label="IDCompositionClip"),
    "IDCompositionColorMatrixEffect": SimTypeBottom(label="IDCompositionColorMatrixEffect"),
    "IDCompositionCompositeEffect": SimTypeBottom(label="IDCompositionCompositeEffect"),
    "IDCompositionDelegatedInkTrail": SimTypeBottom(label="IDCompositionDelegatedInkTrail"),
    "IDCompositionDesktopDevice": SimTypeBottom(label="IDCompositionDesktopDevice"),
    "IDCompositionDevice": SimTypeBottom(label="IDCompositionDevice"),
    "IDCompositionDevice2": SimTypeBottom(label="IDCompositionDevice2"),
    "IDCompositionDevice3": SimTypeBottom(label="IDCompositionDevice3"),
    "IDCompositionDevice4": SimTypeBottom(label="IDCompositionDevice4"),
    "IDCompositionDeviceDebug": SimTypeBottom(label="IDCompositionDeviceDebug"),
    "IDCompositionEffect": SimTypeBottom(label="IDCompositionEffect"),
    "IDCompositionEffectGroup": SimTypeBottom(label="IDCompositionEffectGroup"),
    "IDCompositionFilterEffect": SimTypeBottom(label="IDCompositionFilterEffect"),
    "IDCompositionGaussianBlurEffect": SimTypeBottom(label="IDCompositionGaussianBlurEffect"),
    "IDCompositionHueRotationEffect": SimTypeBottom(label="IDCompositionHueRotationEffect"),
    "IDCompositionInkTrailDevice": SimTypeBottom(label="IDCompositionInkTrailDevice"),
    "IDCompositionLinearTransferEffect": SimTypeBottom(label="IDCompositionLinearTransferEffect"),
    "IDCompositionMatrixTransform": SimTypeBottom(label="IDCompositionMatrixTransform"),
    "IDCompositionMatrixTransform3D": SimTypeBottom(label="IDCompositionMatrixTransform3D"),
    "IDCompositionRectangleClip": SimTypeBottom(label="IDCompositionRectangleClip"),
    "IDCompositionRotateTransform": SimTypeBottom(label="IDCompositionRotateTransform"),
    "IDCompositionRotateTransform3D": SimTypeBottom(label="IDCompositionRotateTransform3D"),
    "IDCompositionSaturationEffect": SimTypeBottom(label="IDCompositionSaturationEffect"),
    "IDCompositionScaleTransform": SimTypeBottom(label="IDCompositionScaleTransform"),
    "IDCompositionScaleTransform3D": SimTypeBottom(label="IDCompositionScaleTransform3D"),
    "IDCompositionShadowEffect": SimTypeBottom(label="IDCompositionShadowEffect"),
    "IDCompositionSkewTransform": SimTypeBottom(label="IDCompositionSkewTransform"),
    "IDCompositionSurface": SimTypeBottom(label="IDCompositionSurface"),
    "IDCompositionSurfaceFactory": SimTypeBottom(label="IDCompositionSurfaceFactory"),
    "IDCompositionTableTransferEffect": SimTypeBottom(label="IDCompositionTableTransferEffect"),
    "IDCompositionTarget": SimTypeBottom(label="IDCompositionTarget"),
    "IDCompositionTexture": SimTypeBottom(label="IDCompositionTexture"),
    "IDCompositionTransform": SimTypeBottom(label="IDCompositionTransform"),
    "IDCompositionTransform3D": SimTypeBottom(label="IDCompositionTransform3D"),
    "IDCompositionTranslateTransform": SimTypeBottom(label="IDCompositionTranslateTransform"),
    "IDCompositionTranslateTransform3D": SimTypeBottom(label="IDCompositionTranslateTransform3D"),
    "IDCompositionTurbulenceEffect": SimTypeBottom(label="IDCompositionTurbulenceEffect"),
    "IDCompositionVirtualSurface": SimTypeBottom(label="IDCompositionVirtualSurface"),
    "IDCompositionVisual": SimTypeBottom(label="IDCompositionVisual"),
    "IDCompositionVisual2": SimTypeBottom(label="IDCompositionVisual2"),
    "IDCompositionVisual3": SimTypeBottom(label="IDCompositionVisual3"),
    "IDCompositionVisualDebug": SimTypeBottom(label="IDCompositionVisualDebug"),
    "IDDVideoPortContainer": SimTypeBottom(label="IDDVideoPortContainer"),
    "IDDrawExclModeVideo": SimTypeBottom(label="IDDrawExclModeVideo"),
    "IDDrawExclModeVideoCallback": SimTypeBottom(label="IDDrawExclModeVideoCallback"),
    "IDENTITY_TYPE": SimTypeInt(signed=False, label="IDENTITY_TYPE"),
    "IDENTITY_URL": SimTypeInt(signed=False, label="IDENTITY_URL"),
    "IDEREGS": SimStruct(OrderedDict((("bFeaturesReg", SimTypeChar(label="Byte")), ("bSectorCountReg", SimTypeChar(label="Byte")), ("bSectorNumberReg", SimTypeChar(label="Byte")), ("bCylLowReg", SimTypeChar(label="Byte")), ("bCylHighReg", SimTypeChar(label="Byte")), ("bDriveHeadReg", SimTypeChar(label="Byte")), ("bCommandReg", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")),)), name="IDEREGS", pack=False, align=None),
    "IDE_IO_CONTROL": SimStruct(OrderedDict((("HeaderLength", SimTypeInt(signed=False, label="UInt32")), ("Signature", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("ControlCode", SimTypeInt(signed=False, label="UInt32")), ("ReturnStatus", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")),)), name="IDE_IO_CONTROL", pack=False, align=None),
    "IDLDESC": SimStruct(OrderedDict((("dwReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wIDLFlags", SimTypeInt(signed=False, label="IDLFLAGS")),)), name="IDLDESC", pack=False, align=None),
    "IDLFLAGS": SimTypeInt(signed=False, label="IDLFLAGS"),
    "IDL_CS_CONVERT": SimTypeInt(signed=False, label="IDL_CS_CONVERT"),
    "IDMLBindingTable": SimTypeBottom(label="IDMLBindingTable"),
    "IDMLCommandRecorder": SimTypeBottom(label="IDMLCommandRecorder"),
    "IDMLCompiledOperator": SimTypeBottom(label="IDMLCompiledOperator"),
    "IDMLDebugDevice": SimTypeBottom(label="IDMLDebugDevice"),
    "IDMLDevice": SimTypeBottom(label="IDMLDevice"),
    "IDMLDevice1": SimTypeBottom(label="IDMLDevice1"),
    "IDMLDeviceChild": SimTypeBottom(label="IDMLDeviceChild"),
    "IDMLDispatchable": SimTypeBottom(label="IDMLDispatchable"),
    "IDMLObject": SimTypeBottom(label="IDMLObject"),
    "IDMLOperator": SimTypeBottom(label="IDMLOperator"),
    "IDMLOperatorInitializer": SimTypeBottom(label="IDMLOperatorInitializer"),
    "IDMLPageable": SimTypeBottom(label="IDMLPageable"),
    "IDMOQualityControl": SimTypeBottom(label="IDMOQualityControl"),
    "IDMOVideoOutputOptimizations": SimTypeBottom(label="IDMOVideoOutputOptimizations"),
    "IDMOWrapperFilter": SimTypeBottom(label="IDMOWrapperFilter"),
    "IDOMBeforeUnloadEvent": SimTypeBottom(label="IDOMBeforeUnloadEvent"),
    "IDOMCloseEvent": SimTypeBottom(label="IDOMCloseEvent"),
    "IDOMCompositionEvent": SimTypeBottom(label="IDOMCompositionEvent"),
    "IDOMCustomEvent": SimTypeBottom(label="IDOMCustomEvent"),
    "IDOMDocumentType": SimTypeBottom(label="IDOMDocumentType"),
    "IDOMDragEvent": SimTypeBottom(label="IDOMDragEvent"),
    "IDOMEvent": SimTypeBottom(label="IDOMEvent"),
    "IDOMEventRegistrationCallback": SimTypeBottom(label="IDOMEventRegistrationCallback"),
    "IDOMException": SimTypeBottom(label="IDOMException"),
    "IDOMFocusEvent": SimTypeBottom(label="IDOMFocusEvent"),
    "IDOMKeyboardEvent": SimTypeBottom(label="IDOMKeyboardEvent"),
    "IDOMMSAnimationEvent": SimTypeBottom(label="IDOMMSAnimationEvent"),
    "IDOMMSManipulationEvent": SimTypeBottom(label="IDOMMSManipulationEvent"),
    "IDOMMSTransitionEvent": SimTypeBottom(label="IDOMMSTransitionEvent"),
    "IDOMMessageEvent": SimTypeBottom(label="IDOMMessageEvent"),
    "IDOMMouseEvent": SimTypeBottom(label="IDOMMouseEvent"),
    "IDOMMouseWheelEvent": SimTypeBottom(label="IDOMMouseWheelEvent"),
    "IDOMMutationEvent": SimTypeBottom(label="IDOMMutationEvent"),
    "IDOMNodeIterator": SimTypeBottom(label="IDOMNodeIterator"),
    "IDOMParser": SimTypeBottom(label="IDOMParser"),
    "IDOMParserFactory": SimTypeBottom(label="IDOMParserFactory"),
    "IDOMProcessingInstruction": SimTypeBottom(label="IDOMProcessingInstruction"),
    "IDOMProgressEvent": SimTypeBottom(label="IDOMProgressEvent"),
    "IDOMSiteModeEvent": SimTypeBottom(label="IDOMSiteModeEvent"),
    "IDOMStorageEvent": SimTypeBottom(label="IDOMStorageEvent"),
    "IDOMTextEvent": SimTypeBottom(label="IDOMTextEvent"),
    "IDOMTreeWalker": SimTypeBottom(label="IDOMTreeWalker"),
    "IDOMUIEvent": SimTypeBottom(label="IDOMUIEvent"),
    "IDOMWheelEvent": SimTypeBottom(label="IDOMWheelEvent"),
    "IDOMXmlSerializer": SimTypeBottom(label="IDOMXmlSerializer"),
    "IDOMXmlSerializerFactory": SimTypeBottom(label="IDOMXmlSerializerFactory"),
    "IDPE_ATTR": SimStruct(OrderedDict((("PeAttribLength", SimTypeShort(signed=False, label="UInt16")), ("PeAttribType", SimTypeChar(label="Byte")), ("PeAttribSubType", SimTypeChar(label="Byte")), ("PeAttribValue", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="IDPE_ATTR", pack=False, align=None),
    "IDShowPlugin": SimTypeBottom(label="IDShowPlugin"),
    "IDTFilter": SimTypeBottom(label="IDTFilter"),
    "IDTFilter2": SimTypeBottom(label="IDTFilter2"),
    "IDTFilter3": SimTypeBottom(label="IDTFilter3"),
    "IDTFilterConfig": SimTypeBottom(label="IDTFilterConfig"),
    "IDTFilterEvents": SimTypeBottom(label="IDTFilterEvents"),
    "IDTFilterLicenseRenewal": SimTypeBottom(label="IDTFilterLicenseRenewal"),
    "IDVBCLocator": SimTypeBottom(label="IDVBCLocator"),
    "IDVBSLocator": SimTypeBottom(label="IDVBSLocator"),
    "IDVBSLocator2": SimTypeBottom(label="IDVBSLocator2"),
    "IDVBSTuningSpace": SimTypeBottom(label="IDVBSTuningSpace"),
    "IDVBTLocator": SimTypeBottom(label="IDVBTLocator"),
    "IDVBTLocator2": SimTypeBottom(label="IDVBTLocator2"),
    "IDVBTuneRequest": SimTypeBottom(label="IDVBTuneRequest"),
    "IDVBTuningSpace": SimTypeBottom(label="IDVBTuningSpace"),
    "IDVBTuningSpace2": SimTypeBottom(label="IDVBTuningSpace2"),
    "IDVB_BAT": SimTypeBottom(label="IDVB_BAT"),
    "IDVB_DIT": SimTypeBottom(label="IDVB_DIT"),
    "IDVB_EIT": SimTypeBottom(label="IDVB_EIT"),
    "IDVB_EIT2": SimTypeBottom(label="IDVB_EIT2"),
    "IDVB_NIT": SimTypeBottom(label="IDVB_NIT"),
    "IDVB_RST": SimTypeBottom(label="IDVB_RST"),
    "IDVB_SDT": SimTypeBottom(label="IDVB_SDT"),
    "IDVB_SIT": SimTypeBottom(label="IDVB_SIT"),
    "IDVB_ST": SimTypeBottom(label="IDVB_ST"),
    "IDVB_TDT": SimTypeBottom(label="IDVB_TDT"),
    "IDVB_TOT": SimTypeBottom(label="IDVB_TOT"),
    "IDVEnc": SimTypeBottom(label="IDVEnc"),
    "IDVRGB219": SimTypeBottom(label="IDVRGB219"),
    "IDVSplitter": SimTypeBottom(label="IDVSplitter"),
    "IDWriteAsyncResult": SimTypeBottom(label="IDWriteAsyncResult"),
    "IDWriteBitmapRenderTarget": SimTypeBottom(label="IDWriteBitmapRenderTarget"),
    "IDWriteBitmapRenderTarget1": SimTypeBottom(label="IDWriteBitmapRenderTarget1"),
    "IDWriteBitmapRenderTarget2": SimTypeBottom(label="IDWriteBitmapRenderTarget2"),
    "IDWriteBitmapRenderTarget3": SimTypeBottom(label="IDWriteBitmapRenderTarget3"),
    "IDWriteColorGlyphRunEnumerator": SimTypeBottom(label="IDWriteColorGlyphRunEnumerator"),
    "IDWriteColorGlyphRunEnumerator1": SimTypeBottom(label="IDWriteColorGlyphRunEnumerator1"),
    "IDWriteFactory": SimTypeBottom(label="IDWriteFactory"),
    "IDWriteFactory1": SimTypeBottom(label="IDWriteFactory1"),
    "IDWriteFactory2": SimTypeBottom(label="IDWriteFactory2"),
    "IDWriteFactory3": SimTypeBottom(label="IDWriteFactory3"),
    "IDWriteFactory4": SimTypeBottom(label="IDWriteFactory4"),
    "IDWriteFactory5": SimTypeBottom(label="IDWriteFactory5"),
    "IDWriteFactory6": SimTypeBottom(label="IDWriteFactory6"),
    "IDWriteFactory7": SimTypeBottom(label="IDWriteFactory7"),
    "IDWriteFactory8": SimTypeBottom(label="IDWriteFactory8"),
    "IDWriteFont": SimTypeBottom(label="IDWriteFont"),
    "IDWriteFont1": SimTypeBottom(label="IDWriteFont1"),
    "IDWriteFont2": SimTypeBottom(label="IDWriteFont2"),
    "IDWriteFont3": SimTypeBottom(label="IDWriteFont3"),
    "IDWriteFontCollection": SimTypeBottom(label="IDWriteFontCollection"),
    "IDWriteFontCollection1": SimTypeBottom(label="IDWriteFontCollection1"),
    "IDWriteFontCollection2": SimTypeBottom(label="IDWriteFontCollection2"),
    "IDWriteFontCollection3": SimTypeBottom(label="IDWriteFontCollection3"),
    "IDWriteFontCollectionLoader": SimTypeBottom(label="IDWriteFontCollectionLoader"),
    "IDWriteFontDownloadListener": SimTypeBottom(label="IDWriteFontDownloadListener"),
    "IDWriteFontDownloadQueue": SimTypeBottom(label="IDWriteFontDownloadQueue"),
    "IDWriteFontFace": SimTypeBottom(label="IDWriteFontFace"),
    "IDWriteFontFace1": SimTypeBottom(label="IDWriteFontFace1"),
    "IDWriteFontFace2": SimTypeBottom(label="IDWriteFontFace2"),
    "IDWriteFontFace3": SimTypeBottom(label="IDWriteFontFace3"),
    "IDWriteFontFace4": SimTypeBottom(label="IDWriteFontFace4"),
    "IDWriteFontFace5": SimTypeBottom(label="IDWriteFontFace5"),
    "IDWriteFontFace6": SimTypeBottom(label="IDWriteFontFace6"),
    "IDWriteFontFace7": SimTypeBottom(label="IDWriteFontFace7"),
    "IDWriteFontFaceReference": SimTypeBottom(label="IDWriteFontFaceReference"),
    "IDWriteFontFaceReference1": SimTypeBottom(label="IDWriteFontFaceReference1"),
    "IDWriteFontFallback": SimTypeBottom(label="IDWriteFontFallback"),
    "IDWriteFontFallback1": SimTypeBottom(label="IDWriteFontFallback1"),
    "IDWriteFontFallbackBuilder": SimTypeBottom(label="IDWriteFontFallbackBuilder"),
    "IDWriteFontFamily": SimTypeBottom(label="IDWriteFontFamily"),
    "IDWriteFontFamily1": SimTypeBottom(label="IDWriteFontFamily1"),
    "IDWriteFontFamily2": SimTypeBottom(label="IDWriteFontFamily2"),
    "IDWriteFontFile": SimTypeBottom(label="IDWriteFontFile"),
    "IDWriteFontFileEnumerator": SimTypeBottom(label="IDWriteFontFileEnumerator"),
    "IDWriteFontFileLoader": SimTypeBottom(label="IDWriteFontFileLoader"),
    "IDWriteFontFileStream": SimTypeBottom(label="IDWriteFontFileStream"),
    "IDWriteFontList": SimTypeBottom(label="IDWriteFontList"),
    "IDWriteFontList1": SimTypeBottom(label="IDWriteFontList1"),
    "IDWriteFontList2": SimTypeBottom(label="IDWriteFontList2"),
    "IDWriteFontResource": SimTypeBottom(label="IDWriteFontResource"),
    "IDWriteFontSet": SimTypeBottom(label="IDWriteFontSet"),
    "IDWriteFontSet1": SimTypeBottom(label="IDWriteFontSet1"),
    "IDWriteFontSet2": SimTypeBottom(label="IDWriteFontSet2"),
    "IDWriteFontSet3": SimTypeBottom(label="IDWriteFontSet3"),
    "IDWriteFontSet4": SimTypeBottom(label="IDWriteFontSet4"),
    "IDWriteFontSetBuilder": SimTypeBottom(label="IDWriteFontSetBuilder"),
    "IDWriteFontSetBuilder1": SimTypeBottom(label="IDWriteFontSetBuilder1"),
    "IDWriteFontSetBuilder2": SimTypeBottom(label="IDWriteFontSetBuilder2"),
    "IDWriteGdiInterop": SimTypeBottom(label="IDWriteGdiInterop"),
    "IDWriteGdiInterop1": SimTypeBottom(label="IDWriteGdiInterop1"),
    "IDWriteGlyphRunAnalysis": SimTypeBottom(label="IDWriteGlyphRunAnalysis"),
    "IDWriteInMemoryFontFileLoader": SimTypeBottom(label="IDWriteInMemoryFontFileLoader"),
    "IDWriteInlineObject": SimTypeBottom(label="IDWriteInlineObject"),
    "IDWriteLocalFontFileLoader": SimTypeBottom(label="IDWriteLocalFontFileLoader"),
    "IDWriteLocalizedStrings": SimTypeBottom(label="IDWriteLocalizedStrings"),
    "IDWriteNumberSubstitution": SimTypeBottom(label="IDWriteNumberSubstitution"),
    "IDWritePaintReader": SimTypeBottom(label="IDWritePaintReader"),
    "IDWritePixelSnapping": SimTypeBottom(label="IDWritePixelSnapping"),
    "IDWriteRemoteFontFileLoader": SimTypeBottom(label="IDWriteRemoteFontFileLoader"),
    "IDWriteRemoteFontFileStream": SimTypeBottom(label="IDWriteRemoteFontFileStream"),
    "IDWriteRenderingParams": SimTypeBottom(label="IDWriteRenderingParams"),
    "IDWriteRenderingParams1": SimTypeBottom(label="IDWriteRenderingParams1"),
    "IDWriteRenderingParams2": SimTypeBottom(label="IDWriteRenderingParams2"),
    "IDWriteRenderingParams3": SimTypeBottom(label="IDWriteRenderingParams3"),
    "IDWriteStringList": SimTypeBottom(label="IDWriteStringList"),
    "IDWriteTextAnalysisSink": SimTypeBottom(label="IDWriteTextAnalysisSink"),
    "IDWriteTextAnalysisSink1": SimTypeBottom(label="IDWriteTextAnalysisSink1"),
    "IDWriteTextAnalysisSource": SimTypeBottom(label="IDWriteTextAnalysisSource"),
    "IDWriteTextAnalysisSource1": SimTypeBottom(label="IDWriteTextAnalysisSource1"),
    "IDWriteTextAnalyzer": SimTypeBottom(label="IDWriteTextAnalyzer"),
    "IDWriteTextAnalyzer1": SimTypeBottom(label="IDWriteTextAnalyzer1"),
    "IDWriteTextAnalyzer2": SimTypeBottom(label="IDWriteTextAnalyzer2"),
    "IDWriteTextFormat": SimTypeBottom(label="IDWriteTextFormat"),
    "IDWriteTextFormat1": SimTypeBottom(label="IDWriteTextFormat1"),
    "IDWriteTextFormat2": SimTypeBottom(label="IDWriteTextFormat2"),
    "IDWriteTextFormat3": SimTypeBottom(label="IDWriteTextFormat3"),
    "IDWriteTextLayout": SimTypeBottom(label="IDWriteTextLayout"),
    "IDWriteTextLayout1": SimTypeBottom(label="IDWriteTextLayout1"),
    "IDWriteTextLayout2": SimTypeBottom(label="IDWriteTextLayout2"),
    "IDWriteTextLayout3": SimTypeBottom(label="IDWriteTextLayout3"),
    "IDWriteTextLayout4": SimTypeBottom(label="IDWriteTextLayout4"),
    "IDWriteTextRenderer": SimTypeBottom(label="IDWriteTextRenderer"),
    "IDWriteTextRenderer1": SimTypeBottom(label="IDWriteTextRenderer1"),
    "IDWriteTypography": SimTypeBottom(label="IDWriteTypography"),
    "IDXCoreAdapter": SimTypeBottom(label="IDXCoreAdapter"),
    "IDXCoreAdapterFactory": SimTypeBottom(label="IDXCoreAdapterFactory"),
    "IDXCoreAdapterList": SimTypeBottom(label="IDXCoreAdapterList"),
    "IDXGIAdapter": SimTypeBottom(label="IDXGIAdapter"),
    "IDXGIAdapter1": SimTypeBottom(label="IDXGIAdapter1"),
    "IDXGIAdapter2": SimTypeBottom(label="IDXGIAdapter2"),
    "IDXGIAdapter3": SimTypeBottom(label="IDXGIAdapter3"),
    "IDXGIAdapter4": SimTypeBottom(label="IDXGIAdapter4"),
    "IDXGIDebug": SimTypeBottom(label="IDXGIDebug"),
    "IDXGIDebug1": SimTypeBottom(label="IDXGIDebug1"),
    "IDXGIDecodeSwapChain": SimTypeBottom(label="IDXGIDecodeSwapChain"),
    "IDXGIDevice": SimTypeBottom(label="IDXGIDevice"),
    "IDXGIDevice1": SimTypeBottom(label="IDXGIDevice1"),
    "IDXGIDevice2": SimTypeBottom(label="IDXGIDevice2"),
    "IDXGIDevice3": SimTypeBottom(label="IDXGIDevice3"),
    "IDXGIDevice4": SimTypeBottom(label="IDXGIDevice4"),
    "IDXGIDeviceSubObject": SimTypeBottom(label="IDXGIDeviceSubObject"),
    "IDXGIDisplayControl": SimTypeBottom(label="IDXGIDisplayControl"),
    "IDXGIFactory": SimTypeBottom(label="IDXGIFactory"),
    "IDXGIFactory1": SimTypeBottom(label="IDXGIFactory1"),
    "IDXGIFactory2": SimTypeBottom(label="IDXGIFactory2"),
    "IDXGIFactory3": SimTypeBottom(label="IDXGIFactory3"),
    "IDXGIFactory4": SimTypeBottom(label="IDXGIFactory4"),
    "IDXGIFactory5": SimTypeBottom(label="IDXGIFactory5"),
    "IDXGIFactory6": SimTypeBottom(label="IDXGIFactory6"),
    "IDXGIFactory7": SimTypeBottom(label="IDXGIFactory7"),
    "IDXGIFactoryMedia": SimTypeBottom(label="IDXGIFactoryMedia"),
    "IDXGIInfoQueue": SimTypeBottom(label="IDXGIInfoQueue"),
    "IDXGIKeyedMutex": SimTypeBottom(label="IDXGIKeyedMutex"),
    "IDXGIObject": SimTypeBottom(label="IDXGIObject"),
    "IDXGIOutput": SimTypeBottom(label="IDXGIOutput"),
    "IDXGIOutput1": SimTypeBottom(label="IDXGIOutput1"),
    "IDXGIOutput2": SimTypeBottom(label="IDXGIOutput2"),
    "IDXGIOutput3": SimTypeBottom(label="IDXGIOutput3"),
    "IDXGIOutput4": SimTypeBottom(label="IDXGIOutput4"),
    "IDXGIOutput5": SimTypeBottom(label="IDXGIOutput5"),
    "IDXGIOutput6": SimTypeBottom(label="IDXGIOutput6"),
    "IDXGIOutputDuplication": SimTypeBottom(label="IDXGIOutputDuplication"),
    "IDXGIResource": SimTypeBottom(label="IDXGIResource"),
    "IDXGIResource1": SimTypeBottom(label="IDXGIResource1"),
    "IDXGISurface": SimTypeBottom(label="IDXGISurface"),
    "IDXGISurface1": SimTypeBottom(label="IDXGISurface1"),
    "IDXGISurface2": SimTypeBottom(label="IDXGISurface2"),
    "IDXGISwapChain": SimTypeBottom(label="IDXGISwapChain"),
    "IDXGISwapChain1": SimTypeBottom(label="IDXGISwapChain1"),
    "IDXGISwapChain2": SimTypeBottom(label="IDXGISwapChain2"),
    "IDXGISwapChain3": SimTypeBottom(label="IDXGISwapChain3"),
    "IDXGISwapChain4": SimTypeBottom(label="IDXGISwapChain4"),
    "IDXGISwapChainMedia": SimTypeBottom(label="IDXGISwapChainMedia"),
    "IDXGraphicsAnalysis": SimTypeBottom(label="IDXGraphicsAnalysis"),
    "IDXVAHD_Device": SimTypeBottom(label="IDXVAHD_Device"),
    "IDXVAHD_VideoProcessor": SimTypeBottom(label="IDXVAHD_VideoProcessor"),
    "ID_ERROR_OBJECT": SimStruct(OrderedDict((("usIdErrLength", SimTypeShort(signed=False, label="UInt16")), ("ucAType", SimTypeChar(label="Byte")), ("ucSubType", SimTypeChar(label="Byte")), ("usReserved", SimTypeShort(signed=False, label="UInt16")), ("usIdErrorValue", SimTypeShort(signed=False, label="UInt16")), ("ucIdErrData", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="ID_ERROR_OBJECT", pack=False, align=None),
    "ID_PARAMETERS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("replicaId", SimTypeRef("ID_PARAMETER_PAIR", SimStruct)), ("itemId", SimTypeRef("ID_PARAMETER_PAIR", SimStruct)), ("changeUnitId", SimTypeRef("ID_PARAMETER_PAIR", SimStruct)),)), name="ID_PARAMETERS", pack=False, align=None),
    "ID_PARAMETER_PAIR": SimStruct(OrderedDict((("fIsVariable", SimTypeInt(signed=True, label="Int32")), ("cbIdSize", SimTypeShort(signed=False, label="UInt16")),)), name="ID_PARAMETER_PAIR", pack=False, align=None),
    "IDailyTrigger": SimTypeBottom(label="IDailyTrigger"),
    "IDataAdviseHolder": SimTypeBottom(label="IDataAdviseHolder"),
    "IDataCollector": SimTypeBottom(label="IDataCollector"),
    "IDataCollectorCollection": SimTypeBottom(label="IDataCollectorCollection"),
    "IDataCollectorSet": SimTypeBottom(label="IDataCollectorSet"),
    "IDataCollectorSetCollection": SimTypeBottom(label="IDataCollectorSetCollection"),
    "IDataConvert": SimTypeBottom(label="IDataConvert"),
    "IDataFilter": SimTypeBottom(label="IDataFilter"),
    "IDataInitialize": SimTypeBottom(label="IDataInitialize"),
    "IDataManager": SimTypeBottom(label="IDataManager"),
    "IDataModelConcept": SimTypeBottom(label="IDataModelConcept"),
    "IDataModelManager": SimTypeBottom(label="IDataModelManager"),
    "IDataModelManager2": SimTypeBottom(label="IDataModelManager2"),
    "IDataModelNameBinder": SimTypeBottom(label="IDataModelNameBinder"),
    "IDataModelScript": SimTypeBottom(label="IDataModelScript"),
    "IDataModelScriptClient": SimTypeBottom(label="IDataModelScriptClient"),
    "IDataModelScriptDebug": SimTypeBottom(label="IDataModelScriptDebug"),
    "IDataModelScriptDebug2": SimTypeBottom(label="IDataModelScriptDebug2"),
    "IDataModelScriptDebugBreakpoint": SimTypeBottom(label="IDataModelScriptDebugBreakpoint"),
    "IDataModelScriptDebugBreakpointEnumerator": SimTypeBottom(label="IDataModelScriptDebugBreakpointEnumerator"),
    "IDataModelScriptDebugClient": SimTypeBottom(label="IDataModelScriptDebugClient"),
    "IDataModelScriptDebugStack": SimTypeBottom(label="IDataModelScriptDebugStack"),
    "IDataModelScriptDebugStackFrame": SimTypeBottom(label="IDataModelScriptDebugStackFrame"),
    "IDataModelScriptDebugVariableSetEnumerator": SimTypeBottom(label="IDataModelScriptDebugVariableSetEnumerator"),
    "IDataModelScriptHostContext": SimTypeBottom(label="IDataModelScriptHostContext"),
    "IDataModelScriptManager": SimTypeBottom(label="IDataModelScriptManager"),
    "IDataModelScriptProvider": SimTypeBottom(label="IDataModelScriptProvider"),
    "IDataModelScriptProviderEnumerator": SimTypeBottom(label="IDataModelScriptProviderEnumerator"),
    "IDataModelScriptTemplate": SimTypeBottom(label="IDataModelScriptTemplate"),
    "IDataModelScriptTemplateEnumerator": SimTypeBottom(label="IDataModelScriptTemplateEnumerator"),
    "IDataObject": SimTypeBottom(label="IDataObject"),
    "IDataObjectAsyncCapability": SimTypeBottom(label="IDataObjectAsyncCapability"),
    "IDataObjectProvider": SimTypeBottom(label="IDataObjectProvider"),
    "IDataRetrieverCallback": SimTypeBottom(label="IDataRetrieverCallback"),
    "IDataSourceLocator": SimTypeBottom(label="IDataSourceLocator"),
    "IDataTransferManagerInterop": SimTypeBottom(label="IDataTransferManagerInterop"),
    "IDebugAdvanced": SimTypeBottom(label="IDebugAdvanced"),
    "IDebugAdvanced2": SimTypeBottom(label="IDebugAdvanced2"),
    "IDebugAdvanced3": SimTypeBottom(label="IDebugAdvanced3"),
    "IDebugAdvanced4": SimTypeBottom(label="IDebugAdvanced4"),
    "IDebugApplication11032": SimTypeBottom(label="IDebugApplication11032"),
    "IDebugApplication11064": SimTypeBottom(label="IDebugApplication11064"),
    "IDebugApplication32": SimTypeBottom(label="IDebugApplication32"),
    "IDebugApplication64": SimTypeBottom(label="IDebugApplication64"),
    "IDebugApplicationNode": SimTypeBottom(label="IDebugApplicationNode"),
    "IDebugApplicationNode100": SimTypeBottom(label="IDebugApplicationNode100"),
    "IDebugApplicationNodeEvents": SimTypeBottom(label="IDebugApplicationNodeEvents"),
    "IDebugApplicationThread": SimTypeBottom(label="IDebugApplicationThread"),
    "IDebugApplicationThread11032": SimTypeBottom(label="IDebugApplicationThread11032"),
    "IDebugApplicationThread11064": SimTypeBottom(label="IDebugApplicationThread11064"),
    "IDebugApplicationThread64": SimTypeBottom(label="IDebugApplicationThread64"),
    "IDebugApplicationThreadEvents110": SimTypeBottom(label="IDebugApplicationThreadEvents110"),
    "IDebugAsyncOperation": SimTypeBottom(label="IDebugAsyncOperation"),
    "IDebugAsyncOperationCallBack": SimTypeBottom(label="IDebugAsyncOperationCallBack"),
    "IDebugBreakpoint": SimTypeBottom(label="IDebugBreakpoint"),
    "IDebugBreakpoint2": SimTypeBottom(label="IDebugBreakpoint2"),
    "IDebugBreakpoint3": SimTypeBottom(label="IDebugBreakpoint3"),
    "IDebugCallbackNotificationHandler": SimTypeBottom(label="IDebugCallbackNotificationHandler"),
    "IDebugClient": SimTypeBottom(label="IDebugClient"),
    "IDebugClient2": SimTypeBottom(label="IDebugClient2"),
    "IDebugClient3": SimTypeBottom(label="IDebugClient3"),
    "IDebugClient4": SimTypeBottom(label="IDebugClient4"),
    "IDebugClient5": SimTypeBottom(label="IDebugClient5"),
    "IDebugClient6": SimTypeBottom(label="IDebugClient6"),
    "IDebugClient7": SimTypeBottom(label="IDebugClient7"),
    "IDebugClient8": SimTypeBottom(label="IDebugClient8"),
    "IDebugCodeContext": SimTypeBottom(label="IDebugCodeContext"),
    "IDebugControl": SimTypeBottom(label="IDebugControl"),
    "IDebugControl2": SimTypeBottom(label="IDebugControl2"),
    "IDebugControl3": SimTypeBottom(label="IDebugControl3"),
    "IDebugControl4": SimTypeBottom(label="IDebugControl4"),
    "IDebugControl5": SimTypeBottom(label="IDebugControl5"),
    "IDebugControl6": SimTypeBottom(label="IDebugControl6"),
    "IDebugControl7": SimTypeBottom(label="IDebugControl7"),
    "IDebugCookie": SimTypeBottom(label="IDebugCookie"),
    "IDebugDataSpaces": SimTypeBottom(label="IDebugDataSpaces"),
    "IDebugDataSpaces2": SimTypeBottom(label="IDebugDataSpaces2"),
    "IDebugDataSpaces3": SimTypeBottom(label="IDebugDataSpaces3"),
    "IDebugDataSpaces4": SimTypeBottom(label="IDebugDataSpaces4"),
    "IDebugDocument": SimTypeBottom(label="IDebugDocument"),
    "IDebugDocumentContext": SimTypeBottom(label="IDebugDocumentContext"),
    "IDebugDocumentHelper32": SimTypeBottom(label="IDebugDocumentHelper32"),
    "IDebugDocumentHelper64": SimTypeBottom(label="IDebugDocumentHelper64"),
    "IDebugDocumentHost": SimTypeBottom(label="IDebugDocumentHost"),
    "IDebugDocumentInfo": SimTypeBottom(label="IDebugDocumentInfo"),
    "IDebugDocumentProvider": SimTypeBottom(label="IDebugDocumentProvider"),
    "IDebugDocumentText": SimTypeBottom(label="IDebugDocumentText"),
    "IDebugDocumentTextAuthor": SimTypeBottom(label="IDebugDocumentTextAuthor"),
    "IDebugDocumentTextEvents": SimTypeBottom(label="IDebugDocumentTextEvents"),
    "IDebugDocumentTextExternalAuthor": SimTypeBottom(label="IDebugDocumentTextExternalAuthor"),
    "IDebugEventCallbacks": SimTypeBottom(label="IDebugEventCallbacks"),
    "IDebugEventCallbacksWide": SimTypeBottom(label="IDebugEventCallbacksWide"),
    "IDebugEventContextCallbacks": SimTypeBottom(label="IDebugEventContextCallbacks"),
    "IDebugExpression": SimTypeBottom(label="IDebugExpression"),
    "IDebugExpressionCallBack": SimTypeBottom(label="IDebugExpressionCallBack"),
    "IDebugExpressionContext": SimTypeBottom(label="IDebugExpressionContext"),
    "IDebugExtendedProperty": SimTypeBottom(label="IDebugExtendedProperty"),
    "IDebugFAEntryTags": SimTypeBottom(label="IDebugFAEntryTags"),
    "IDebugFailureAnalysis": SimTypeBottom(label="IDebugFailureAnalysis"),
    "IDebugFailureAnalysis2": SimTypeBottom(label="IDebugFailureAnalysis2"),
    "IDebugFailureAnalysis3": SimTypeBottom(label="IDebugFailureAnalysis3"),
    "IDebugFormatter": SimTypeBottom(label="IDebugFormatter"),
    "IDebugHelper": SimTypeBottom(label="IDebugHelper"),
    "IDebugHost": SimTypeBottom(label="IDebugHost"),
    "IDebugHostBaseClass": SimTypeBottom(label="IDebugHostBaseClass"),
    "IDebugHostConstant": SimTypeBottom(label="IDebugHostConstant"),
    "IDebugHostContext": SimTypeBottom(label="IDebugHostContext"),
    "IDebugHostData": SimTypeBottom(label="IDebugHostData"),
    "IDebugHostErrorSink": SimTypeBottom(label="IDebugHostErrorSink"),
    "IDebugHostEvaluator": SimTypeBottom(label="IDebugHostEvaluator"),
    "IDebugHostEvaluator2": SimTypeBottom(label="IDebugHostEvaluator2"),
    "IDebugHostExtensibility": SimTypeBottom(label="IDebugHostExtensibility"),
    "IDebugHostField": SimTypeBottom(label="IDebugHostField"),
    "IDebugHostMemory": SimTypeBottom(label="IDebugHostMemory"),
    "IDebugHostMemory2": SimTypeBottom(label="IDebugHostMemory2"),
    "IDebugHostModule": SimTypeBottom(label="IDebugHostModule"),
    "IDebugHostModule2": SimTypeBottom(label="IDebugHostModule2"),
    "IDebugHostModuleSignature": SimTypeBottom(label="IDebugHostModuleSignature"),
    "IDebugHostPublic": SimTypeBottom(label="IDebugHostPublic"),
    "IDebugHostScriptHost": SimTypeBottom(label="IDebugHostScriptHost"),
    "IDebugHostStatus": SimTypeBottom(label="IDebugHostStatus"),
    "IDebugHostSymbol": SimTypeBottom(label="IDebugHostSymbol"),
    "IDebugHostSymbol2": SimTypeBottom(label="IDebugHostSymbol2"),
    "IDebugHostSymbolEnumerator": SimTypeBottom(label="IDebugHostSymbolEnumerator"),
    "IDebugHostSymbols": SimTypeBottom(label="IDebugHostSymbols"),
    "IDebugHostType": SimTypeBottom(label="IDebugHostType"),
    "IDebugHostType2": SimTypeBottom(label="IDebugHostType2"),
    "IDebugHostTypeSignature": SimTypeBottom(label="IDebugHostTypeSignature"),
    "IDebugInputCallbacks": SimTypeBottom(label="IDebugInputCallbacks"),
    "IDebugOutputCallbacks": SimTypeBottom(label="IDebugOutputCallbacks"),
    "IDebugOutputCallbacks2": SimTypeBottom(label="IDebugOutputCallbacks2"),
    "IDebugOutputCallbacksWide": SimTypeBottom(label="IDebugOutputCallbacksWide"),
    "IDebugOutputStream": SimTypeBottom(label="IDebugOutputStream"),
    "IDebugPlmClient": SimTypeBottom(label="IDebugPlmClient"),
    "IDebugPlmClient2": SimTypeBottom(label="IDebugPlmClient2"),
    "IDebugPlmClient3": SimTypeBottom(label="IDebugPlmClient3"),
    "IDebugProperty": SimTypeBottom(label="IDebugProperty"),
    "IDebugPropertyEnumType_All": SimTypeBottom(label="IDebugPropertyEnumType_All"),
    "IDebugPropertyEnumType_Arguments": SimTypeBottom(label="IDebugPropertyEnumType_Arguments"),
    "IDebugPropertyEnumType_Locals": SimTypeBottom(label="IDebugPropertyEnumType_Locals"),
    "IDebugPropertyEnumType_LocalsPlusArgs": SimTypeBottom(label="IDebugPropertyEnumType_LocalsPlusArgs"),
    "IDebugPropertyEnumType_Registers": SimTypeBottom(label="IDebugPropertyEnumType_Registers"),
    "IDebugRegisters": SimTypeBottom(label="IDebugRegisters"),
    "IDebugRegisters2": SimTypeBottom(label="IDebugRegisters2"),
    "IDebugSessionProvider": SimTypeBottom(label="IDebugSessionProvider"),
    "IDebugStackFrame": SimTypeBottom(label="IDebugStackFrame"),
    "IDebugStackFrame110": SimTypeBottom(label="IDebugStackFrame110"),
    "IDebugStackFrameSniffer": SimTypeBottom(label="IDebugStackFrameSniffer"),
    "IDebugStackFrameSnifferEx32": SimTypeBottom(label="IDebugStackFrameSnifferEx32"),
    "IDebugStackFrameSnifferEx64": SimTypeBottom(label="IDebugStackFrameSnifferEx64"),
    "IDebugSymbolGroup": SimTypeBottom(label="IDebugSymbolGroup"),
    "IDebugSymbolGroup2": SimTypeBottom(label="IDebugSymbolGroup2"),
    "IDebugSymbols": SimTypeBottom(label="IDebugSymbols"),
    "IDebugSymbols2": SimTypeBottom(label="IDebugSymbols2"),
    "IDebugSymbols3": SimTypeBottom(label="IDebugSymbols3"),
    "IDebugSymbols4": SimTypeBottom(label="IDebugSymbols4"),
    "IDebugSymbols5": SimTypeBottom(label="IDebugSymbols5"),
    "IDebugSyncOperation": SimTypeBottom(label="IDebugSyncOperation"),
    "IDebugSystemObjects": SimTypeBottom(label="IDebugSystemObjects"),
    "IDebugSystemObjects2": SimTypeBottom(label="IDebugSystemObjects2"),
    "IDebugSystemObjects3": SimTypeBottom(label="IDebugSystemObjects3"),
    "IDebugSystemObjects4": SimTypeBottom(label="IDebugSystemObjects4"),
    "IDebugThreadCall32": SimTypeBottom(label="IDebugThreadCall32"),
    "IDebugThreadCall64": SimTypeBottom(label="IDebugThreadCall64"),
    "IDebuggerInfo": SimTypeBottom(label="IDebuggerInfo"),
    "IDebuggerThreadControl": SimTypeBottom(label="IDebuggerThreadControl"),
    "IDecimateVideoImage": SimTypeBottom(label="IDecimateVideoImage"),
    "IDedupBackupSupport": SimTypeBottom(label="IDedupBackupSupport"),
    "IDedupChunkLibrary": SimTypeBottom(label="IDedupChunkLibrary"),
    "IDedupDataPort": SimTypeBottom(label="IDedupDataPort"),
    "IDedupDataPortManager": SimTypeBottom(label="IDedupDataPortManager"),
    "IDedupIterateChunksHash32": SimTypeBottom(label="IDedupIterateChunksHash32"),
    "IDedupReadFileCallback": SimTypeBottom(label="IDedupReadFileCallback"),
    "IDefaultBrowserSyncSettings": SimTypeBottom(label="IDefaultBrowserSyncSettings"),
    "IDefaultExtractIconInit": SimTypeBottom(label="IDefaultExtractIconInit"),
    "IDefaultFolderMenuInitialize": SimTypeBottom(label="IDefaultFolderMenuInitialize"),
    "IDefaultLocation": SimTypeBottom(label="IDefaultLocation"),
    "IDeferredCommand": SimTypeBottom(label="IDeferredCommand"),
    "IDelayedPropertyStoreFactory": SimTypeBottom(label="IDelayedPropertyStoreFactory"),
    "IDelegateFolder": SimTypeBottom(label="IDelegateFolder"),
    "IDelegateItem": SimTypeBottom(label="IDelegateItem"),
    "IDeleteBrowsingHistory": SimTypeBottom(label="IDeleteBrowsingHistory"),
    "IDeskBand": SimTypeBottom(label="IDeskBand"),
    "IDeskBand2": SimTypeBottom(label="IDeskBand2"),
    "IDeskBandInfo": SimTypeBottom(label="IDeskBandInfo"),
    "IDeskBar": SimTypeBottom(label="IDeskBar"),
    "IDeskBarClient": SimTypeBottom(label="IDeskBarClient"),
    "IDesktopGadget": SimTypeBottom(label="IDesktopGadget"),
    "IDesktopWallpaper": SimTypeBottom(label="IDesktopWallpaper"),
    "IDestinationStreamFactory": SimTypeBottom(label="IDestinationStreamFactory"),
    "IDeveloperConsoleMessageReceiver": SimTypeBottom(label="IDeveloperConsoleMessageReceiver"),
    "IDeviceIoControl": SimTypeBottom(label="IDeviceIoControl"),
    "IDeviceModelPlugIn": SimTypeBottom(label="IDeviceModelPlugIn"),
    "IDeviceRect": SimTypeBottom(label="IDeviceRect"),
    "IDeviceRequestCompletionCallback": SimTypeBottom(label="IDeviceRequestCompletionCallback"),
    "IDeviceSpecificProperty": SimTypeBottom(label="IDeviceSpecificProperty"),
    "IDeviceTopology": SimTypeBottom(label="IDeviceTopology"),
    "IDiagnosticsScriptEngine": SimTypeBottom(label="IDiagnosticsScriptEngine"),
    "IDiagnosticsScriptEngineProvider": SimTypeBottom(label="IDiagnosticsScriptEngineProvider"),
    "IDiagnosticsScriptEngineSite": SimTypeBottom(label="IDiagnosticsScriptEngineSite"),
    "IDialBranding": SimTypeBottom(label="IDialBranding"),
    "IDialEngine": SimTypeBottom(label="IDialEngine"),
    "IDialEventSink": SimTypeBottom(label="IDialEventSink"),
    "IDigitalCableLocator": SimTypeBottom(label="IDigitalCableLocator"),
    "IDigitalCableTuneRequest": SimTypeBottom(label="IDigitalCableTuneRequest"),
    "IDigitalCableTuningSpace": SimTypeBottom(label="IDigitalCableTuningSpace"),
    "IDigitalLocator": SimTypeBottom(label="IDigitalLocator"),
    "IDirect3D9": SimTypeBottom(label="IDirect3D9"),
    "IDirect3D9Ex": SimTypeBottom(label="IDirect3D9Ex"),
    "IDirect3D9ExOverlayExtension": SimTypeBottom(label="IDirect3D9ExOverlayExtension"),
    "IDirect3DAuthenticatedChannel9": SimTypeBottom(label="IDirect3DAuthenticatedChannel9"),
    "IDirect3DBaseTexture9": SimTypeBottom(label="IDirect3DBaseTexture9"),
    "IDirect3DCryptoSession9": SimTypeBottom(label="IDirect3DCryptoSession9"),
    "IDirect3DCubeTexture9": SimTypeBottom(label="IDirect3DCubeTexture9"),
    "IDirect3DDevice9": SimTypeBottom(label="IDirect3DDevice9"),
    "IDirect3DDevice9Ex": SimTypeBottom(label="IDirect3DDevice9Ex"),
    "IDirect3DDevice9On12": SimTypeBottom(label="IDirect3DDevice9On12"),
    "IDirect3DDevice9Video": SimTypeBottom(label="IDirect3DDevice9Video"),
    "IDirect3DDeviceManager9": SimTypeBottom(label="IDirect3DDeviceManager9"),
    "IDirect3DIndexBuffer9": SimTypeBottom(label="IDirect3DIndexBuffer9"),
    "IDirect3DPixelShader9": SimTypeBottom(label="IDirect3DPixelShader9"),
    "IDirect3DQuery9": SimTypeBottom(label="IDirect3DQuery9"),
    "IDirect3DResource9": SimTypeBottom(label="IDirect3DResource9"),
    "IDirect3DStateBlock9": SimTypeBottom(label="IDirect3DStateBlock9"),
    "IDirect3DSurface9": SimTypeBottom(label="IDirect3DSurface9"),
    "IDirect3DSwapChain9": SimTypeBottom(label="IDirect3DSwapChain9"),
    "IDirect3DSwapChain9Ex": SimTypeBottom(label="IDirect3DSwapChain9Ex"),
    "IDirect3DTexture9": SimTypeBottom(label="IDirect3DTexture9"),
    "IDirect3DVertexBuffer9": SimTypeBottom(label="IDirect3DVertexBuffer9"),
    "IDirect3DVertexDeclaration9": SimTypeBottom(label="IDirect3DVertexDeclaration9"),
    "IDirect3DVertexShader9": SimTypeBottom(label="IDirect3DVertexShader9"),
    "IDirect3DVolume9": SimTypeBottom(label="IDirect3DVolume9"),
    "IDirect3DVolumeTexture9": SimTypeBottom(label="IDirect3DVolumeTexture9"),
    "IDirectDraw": SimTypeBottom(label="IDirectDraw"),
    "IDirectDraw2": SimTypeBottom(label="IDirectDraw2"),
    "IDirectDraw4": SimTypeBottom(label="IDirectDraw4"),
    "IDirectDraw7": SimTypeBottom(label="IDirectDraw7"),
    "IDirectDrawClipper": SimTypeBottom(label="IDirectDrawClipper"),
    "IDirectDrawColorControl": SimTypeBottom(label="IDirectDrawColorControl"),
    "IDirectDrawGammaControl": SimTypeBottom(label="IDirectDrawGammaControl"),
    "IDirectDrawKernel": SimTypeBottom(label="IDirectDrawKernel"),
    "IDirectDrawMediaSample": SimTypeBottom(label="IDirectDrawMediaSample"),
    "IDirectDrawMediaSampleAllocator": SimTypeBottom(label="IDirectDrawMediaSampleAllocator"),
    "IDirectDrawMediaStream": SimTypeBottom(label="IDirectDrawMediaStream"),
    "IDirectDrawPalette": SimTypeBottom(label="IDirectDrawPalette"),
    "IDirectDrawStreamSample": SimTypeBottom(label="IDirectDrawStreamSample"),
    "IDirectDrawSurface": SimTypeBottom(label="IDirectDrawSurface"),
    "IDirectDrawSurface2": SimTypeBottom(label="IDirectDrawSurface2"),
    "IDirectDrawSurface3": SimTypeBottom(label="IDirectDrawSurface3"),
    "IDirectDrawSurface4": SimTypeBottom(label="IDirectDrawSurface4"),
    "IDirectDrawSurface7": SimTypeBottom(label="IDirectDrawSurface7"),
    "IDirectDrawSurfaceKernel": SimTypeBottom(label="IDirectDrawSurfaceKernel"),
    "IDirectDrawVideo": SimTypeBottom(label="IDirectDrawVideo"),
    "IDirectDrawVideoPort": SimTypeBottom(label="IDirectDrawVideoPort"),
    "IDirectDrawVideoPortNotify": SimTypeBottom(label="IDirectDrawVideoPortNotify"),
    "IDirectInput2A": SimTypeBottom(label="IDirectInput2A"),
    "IDirectInput2W": SimTypeBottom(label="IDirectInput2W"),
    "IDirectInput7A": SimTypeBottom(label="IDirectInput7A"),
    "IDirectInput7W": SimTypeBottom(label="IDirectInput7W"),
    "IDirectInput8A": SimTypeBottom(label="IDirectInput8A"),
    "IDirectInput8W": SimTypeBottom(label="IDirectInput8W"),
    "IDirectInputA": SimTypeBottom(label="IDirectInputA"),
    "IDirectInputDevice2A": SimTypeBottom(label="IDirectInputDevice2A"),
    "IDirectInputDevice2W": SimTypeBottom(label="IDirectInputDevice2W"),
    "IDirectInputDevice7A": SimTypeBottom(label="IDirectInputDevice7A"),
    "IDirectInputDevice7W": SimTypeBottom(label="IDirectInputDevice7W"),
    "IDirectInputDevice8A": SimTypeBottom(label="IDirectInputDevice8A"),
    "IDirectInputDevice8W": SimTypeBottom(label="IDirectInputDevice8W"),
    "IDirectInputDeviceA": SimTypeBottom(label="IDirectInputDeviceA"),
    "IDirectInputDeviceW": SimTypeBottom(label="IDirectInputDeviceW"),
    "IDirectInputEffect": SimTypeBottom(label="IDirectInputEffect"),
    "IDirectInputEffectDriver": SimTypeBottom(label="IDirectInputEffectDriver"),
    "IDirectInputJoyConfig": SimTypeBottom(label="IDirectInputJoyConfig"),
    "IDirectInputJoyConfig8": SimTypeBottom(label="IDirectInputJoyConfig8"),
    "IDirectInputW": SimTypeBottom(label="IDirectInputW"),
    "IDirectManipulationAutoScrollBehavior": SimTypeBottom(label="IDirectManipulationAutoScrollBehavior"),
    "IDirectManipulationCompositor": SimTypeBottom(label="IDirectManipulationCompositor"),
    "IDirectManipulationCompositor2": SimTypeBottom(label="IDirectManipulationCompositor2"),
    "IDirectManipulationContent": SimTypeBottom(label="IDirectManipulationContent"),
    "IDirectManipulationDeferContactService": SimTypeBottom(label="IDirectManipulationDeferContactService"),
    "IDirectManipulationDragDropBehavior": SimTypeBottom(label="IDirectManipulationDragDropBehavior"),
    "IDirectManipulationDragDropEventHandler": SimTypeBottom(label="IDirectManipulationDragDropEventHandler"),
    "IDirectManipulationFrameInfoProvider": SimTypeBottom(label="IDirectManipulationFrameInfoProvider"),
    "IDirectManipulationInteractionEventHandler": SimTypeBottom(label="IDirectManipulationInteractionEventHandler"),
    "IDirectManipulationManager": SimTypeBottom(label="IDirectManipulationManager"),
    "IDirectManipulationManager2": SimTypeBottom(label="IDirectManipulationManager2"),
    "IDirectManipulationManager3": SimTypeBottom(label="IDirectManipulationManager3"),
    "IDirectManipulationPrimaryContent": SimTypeBottom(label="IDirectManipulationPrimaryContent"),
    "IDirectManipulationUpdateHandler": SimTypeBottom(label="IDirectManipulationUpdateHandler"),
    "IDirectManipulationUpdateManager": SimTypeBottom(label="IDirectManipulationUpdateManager"),
    "IDirectManipulationViewport": SimTypeBottom(label="IDirectManipulationViewport"),
    "IDirectManipulationViewport2": SimTypeBottom(label="IDirectManipulationViewport2"),
    "IDirectManipulationViewportEventHandler": SimTypeBottom(label="IDirectManipulationViewportEventHandler"),
    "IDirectMusic": SimTypeBottom(label="IDirectMusic"),
    "IDirectMusic8": SimTypeBottom(label="IDirectMusic8"),
    "IDirectMusicBuffer": SimTypeBottom(label="IDirectMusicBuffer"),
    "IDirectMusicCollection": SimTypeBottom(label="IDirectMusicCollection"),
    "IDirectMusicDownload": SimTypeBottom(label="IDirectMusicDownload"),
    "IDirectMusicDownloadedInstrument": SimTypeBottom(label="IDirectMusicDownloadedInstrument"),
    "IDirectMusicInstrument": SimTypeBottom(label="IDirectMusicInstrument"),
    "IDirectMusicPort": SimTypeBottom(label="IDirectMusicPort"),
    "IDirectMusicPortDownload": SimTypeBottom(label="IDirectMusicPortDownload"),
    "IDirectMusicSynth": SimTypeBottom(label="IDirectMusicSynth"),
    "IDirectMusicSynth8": SimTypeBottom(label="IDirectMusicSynth8"),
    "IDirectMusicSynthSink": SimTypeBottom(label="IDirectMusicSynthSink"),
    "IDirectMusicThru": SimTypeBottom(label="IDirectMusicThru"),
    "IDirectSound": SimTypeBottom(label="IDirectSound"),
    "IDirectSound3DBuffer": SimTypeBottom(label="IDirectSound3DBuffer"),
    "IDirectSound3DListener": SimTypeBottom(label="IDirectSound3DListener"),
    "IDirectSound8": SimTypeBottom(label="IDirectSound8"),
    "IDirectSoundBuffer": SimTypeBottom(label="IDirectSoundBuffer"),
    "IDirectSoundBuffer8": SimTypeBottom(label="IDirectSoundBuffer8"),
    "IDirectSoundCapture": SimTypeBottom(label="IDirectSoundCapture"),
    "IDirectSoundCaptureBuffer": SimTypeBottom(label="IDirectSoundCaptureBuffer"),
    "IDirectSoundCaptureBuffer8": SimTypeBottom(label="IDirectSoundCaptureBuffer8"),
    "IDirectSoundCaptureFXAec": SimTypeBottom(label="IDirectSoundCaptureFXAec"),
    "IDirectSoundCaptureFXNoiseSuppress": SimTypeBottom(label="IDirectSoundCaptureFXNoiseSuppress"),
    "IDirectSoundFXChorus": SimTypeBottom(label="IDirectSoundFXChorus"),
    "IDirectSoundFXCompressor": SimTypeBottom(label="IDirectSoundFXCompressor"),
    "IDirectSoundFXDistortion": SimTypeBottom(label="IDirectSoundFXDistortion"),
    "IDirectSoundFXEcho": SimTypeBottom(label="IDirectSoundFXEcho"),
    "IDirectSoundFXFlanger": SimTypeBottom(label="IDirectSoundFXFlanger"),
    "IDirectSoundFXGargle": SimTypeBottom(label="IDirectSoundFXGargle"),
    "IDirectSoundFXI3DL2Reverb": SimTypeBottom(label="IDirectSoundFXI3DL2Reverb"),
    "IDirectSoundFXParamEq": SimTypeBottom(label="IDirectSoundFXParamEq"),
    "IDirectSoundFXWavesReverb": SimTypeBottom(label="IDirectSoundFXWavesReverb"),
    "IDirectSoundFullDuplex": SimTypeBottom(label="IDirectSoundFullDuplex"),
    "IDirectSoundNotify": SimTypeBottom(label="IDirectSoundNotify"),
    "IDirectWriterLock": SimTypeBottom(label="IDirectWriterLock"),
    "IDirectXVideoAccelerationService": SimTypeBottom(label="IDirectXVideoAccelerationService"),
    "IDirectXVideoDecoder": SimTypeBottom(label="IDirectXVideoDecoder"),
    "IDirectXVideoDecoderService": SimTypeBottom(label="IDirectXVideoDecoderService"),
    "IDirectXVideoMemoryConfiguration": SimTypeBottom(label="IDirectXVideoMemoryConfiguration"),
    "IDirectXVideoProcessor": SimTypeBottom(label="IDirectXVideoProcessor"),
    "IDirectXVideoProcessorService": SimTypeBottom(label="IDirectXVideoProcessorService"),
    "IDirectoryObject": SimTypeBottom(label="IDirectoryObject"),
    "IDirectorySchemaMgmt": SimTypeBottom(label="IDirectorySchemaMgmt"),
    "IDirectorySearch": SimTypeBottom(label="IDirectorySearch"),
    "IDiscFormat2": SimTypeBottom(label="IDiscFormat2"),
    "IDiscFormat2Data": SimTypeBottom(label="IDiscFormat2Data"),
    "IDiscFormat2DataEventArgs": SimTypeBottom(label="IDiscFormat2DataEventArgs"),
    "IDiscFormat2Erase": SimTypeBottom(label="IDiscFormat2Erase"),
    "IDiscFormat2RawCD": SimTypeBottom(label="IDiscFormat2RawCD"),
    "IDiscFormat2RawCDEventArgs": SimTypeBottom(label="IDiscFormat2RawCDEventArgs"),
    "IDiscFormat2TrackAtOnce": SimTypeBottom(label="IDiscFormat2TrackAtOnce"),
    "IDiscFormat2TrackAtOnceEventArgs": SimTypeBottom(label="IDiscFormat2TrackAtOnceEventArgs"),
    "IDiscMaster": SimTypeBottom(label="IDiscMaster"),
    "IDiscMaster2": SimTypeBottom(label="IDiscMaster2"),
    "IDiscMasterProgressEvents": SimTypeBottom(label="IDiscMasterProgressEvents"),
    "IDiscRecorder": SimTypeBottom(label="IDiscRecorder"),
    "IDiscRecorder2": SimTypeBottom(label="IDiscRecorder2"),
    "IDiscRecorder2Ex": SimTypeBottom(label="IDiscRecorder2Ex"),
    "IDiskQuotaControl": SimTypeBottom(label="IDiskQuotaControl"),
    "IDiskQuotaEvents": SimTypeBottom(label="IDiskQuotaEvents"),
    "IDiskQuotaUser": SimTypeBottom(label="IDiskQuotaUser"),
    "IDiskQuotaUserBatch": SimTypeBottom(label="IDiskQuotaUserBatch"),
    "IDispCivicAddressReport": SimTypeBottom(label="IDispCivicAddressReport"),
    "IDispError": SimTypeBottom(label="IDispError"),
    "IDispLatLongReport": SimTypeBottom(label="IDispLatLongReport"),
    "IDispatch": SimTypeBottom(label="IDispatch"),
    "IDispatchEx": SimTypeBottom(label="IDispatchEx"),
    "IDispenserDriver": SimTypeBottom(label="IDispenserDriver"),
    "IDispenserManager": SimTypeBottom(label="IDispenserManager"),
    "IDisplayHelp": SimTypeBottom(label="IDisplayHelp"),
    "IDisplayItem": SimTypeBottom(label="IDisplayItem"),
    "IDisplayPointer": SimTypeBottom(label="IDisplayPointer"),
    "IDisplayServices": SimTypeBottom(label="IDisplayServices"),
    "IDistList": SimTypeBottom(label="IDistList"),
    "IDistributorNotify": SimTypeBottom(label="IDistributorNotify"),
    "IDithererImpl": SimTypeBottom(label="IDithererImpl"),
    "IDocHostShowUI": SimTypeBottom(label="IDocHostShowUI"),
    "IDocHostUIHandler": SimTypeBottom(label="IDocHostUIHandler"),
    "IDocHostUIHandler2": SimTypeBottom(label="IDocHostUIHandler2"),
    "IDocObjectService": SimTypeBottom(label="IDocObjectService"),
    "IDocViewSite": SimTypeBottom(label="IDocViewSite"),
    "IDocWrap": SimTypeBottom(label="IDocWrap"),
    "IDockProvider": SimTypeBottom(label="IDockProvider"),
    "IDockingWindow": SimTypeBottom(label="IDockingWindow"),
    "IDockingWindowFrame": SimTypeBottom(label="IDockingWindowFrame"),
    "IDockingWindowSite": SimTypeBottom(label="IDockingWindowSite"),
    "IDocumentEvent": SimTypeBottom(label="IDocumentEvent"),
    "IDocumentRange": SimTypeBottom(label="IDocumentRange"),
    "IDocumentSelector": SimTypeBottom(label="IDocumentSelector"),
    "IDocumentTraversal": SimTypeBottom(label="IDocumentTraversal"),
    "IDontSupportEventSubscription": SimTypeBottom(label="IDontSupportEventSubscription"),
    "IDot11AdHocInterface": SimTypeBottom(label="IDot11AdHocInterface"),
    "IDot11AdHocInterfaceNotificationSink": SimTypeBottom(label="IDot11AdHocInterfaceNotificationSink"),
    "IDot11AdHocManager": SimTypeBottom(label="IDot11AdHocManager"),
    "IDot11AdHocManagerNotificationSink": SimTypeBottom(label="IDot11AdHocManagerNotificationSink"),
    "IDot11AdHocNetwork": SimTypeBottom(label="IDot11AdHocNetwork"),
    "IDot11AdHocNetworkNotificationSink": SimTypeBottom(label="IDot11AdHocNetworkNotificationSink"),
    "IDot11AdHocSecuritySettings": SimTypeBottom(label="IDot11AdHocSecuritySettings"),
    "IDownloadBehavior": SimTypeBottom(label="IDownloadBehavior"),
    "IDownloadCompletedCallback": SimTypeBottom(label="IDownloadCompletedCallback"),
    "IDownloadCompletedCallbackArgs": SimTypeBottom(label="IDownloadCompletedCallbackArgs"),
    "IDownloadJob": SimTypeBottom(label="IDownloadJob"),
    "IDownloadManager": SimTypeBottom(label="IDownloadManager"),
    "IDownloadProgress": SimTypeBottom(label="IDownloadProgress"),
    "IDownloadProgressChangedCallback": SimTypeBottom(label="IDownloadProgressChangedCallback"),
    "IDownloadProgressChangedCallbackArgs": SimTypeBottom(label="IDownloadProgressChangedCallbackArgs"),
    "IDownloadResult": SimTypeBottom(label="IDownloadResult"),
    "IDragProvider": SimTypeBottom(label="IDragProvider"),
    "IDragSourceHelper": SimTypeBottom(label="IDragSourceHelper"),
    "IDragSourceHelper2": SimTypeBottom(label="IDragSourceHelper2"),
    "IDrawVideoImage": SimTypeBottom(label="IDrawVideoImage"),
    "IDropSource": SimTypeBottom(label="IDropSource"),
    "IDropSourceNotify": SimTypeBottom(label="IDropSourceNotify"),
    "IDropTarget": SimTypeBottom(label="IDropTarget"),
    "IDropTargetHelper": SimTypeBottom(label="IDropTargetHelper"),
    "IDropTargetProvider": SimTypeBottom(label="IDropTargetProvider"),
    "IDsAdminCreateObj": SimTypeBottom(label="IDsAdminCreateObj"),
    "IDsAdminNewObj": SimTypeBottom(label="IDsAdminNewObj"),
    "IDsAdminNewObjExt": SimTypeBottom(label="IDsAdminNewObjExt"),
    "IDsAdminNewObjPrimarySite": SimTypeBottom(label="IDsAdminNewObjPrimarySite"),
    "IDsAdminNotifyHandler": SimTypeBottom(label="IDsAdminNotifyHandler"),
    "IDsBrowseDomainTree": SimTypeBottom(label="IDsBrowseDomainTree"),
    "IDsDisplaySpecifier": SimTypeBottom(label="IDsDisplaySpecifier"),
    "IDsObjectPicker": SimTypeBottom(label="IDsObjectPicker"),
    "IDsObjectPickerCredentials": SimTypeBottom(label="IDsObjectPickerCredentials"),
    "IDtcLuConfigure": SimTypeBottom(label="IDtcLuConfigure"),
    "IDtcLuRecovery": SimTypeBottom(label="IDtcLuRecovery"),
    "IDtcLuRecoveryFactory": SimTypeBottom(label="IDtcLuRecoveryFactory"),
    "IDtcLuRecoveryInitiatedByDtc": SimTypeBottom(label="IDtcLuRecoveryInitiatedByDtc"),
    "IDtcLuRecoveryInitiatedByDtcStatusWork": SimTypeBottom(label="IDtcLuRecoveryInitiatedByDtcStatusWork"),
    "IDtcLuRecoveryInitiatedByDtcTransWork": SimTypeBottom(label="IDtcLuRecoveryInitiatedByDtcTransWork"),
    "IDtcLuRecoveryInitiatedByLu": SimTypeBottom(label="IDtcLuRecoveryInitiatedByLu"),
    "IDtcLuRecoveryInitiatedByLuWork": SimTypeBottom(label="IDtcLuRecoveryInitiatedByLuWork"),
    "IDtcLuRmEnlistment": SimTypeBottom(label="IDtcLuRmEnlistment"),
    "IDtcLuRmEnlistmentFactory": SimTypeBottom(label="IDtcLuRmEnlistmentFactory"),
    "IDtcLuRmEnlistmentSink": SimTypeBottom(label="IDtcLuRmEnlistmentSink"),
    "IDtcLuSubordinateDtc": SimTypeBottom(label="IDtcLuSubordinateDtc"),
    "IDtcLuSubordinateDtcFactory": SimTypeBottom(label="IDtcLuSubordinateDtcFactory"),
    "IDtcLuSubordinateDtcSink": SimTypeBottom(label="IDtcLuSubordinateDtcSink"),
    "IDtcNetworkAccessConfig": SimTypeBottom(label="IDtcNetworkAccessConfig"),
    "IDtcNetworkAccessConfig2": SimTypeBottom(label="IDtcNetworkAccessConfig2"),
    "IDtcNetworkAccessConfig3": SimTypeBottom(label="IDtcNetworkAccessConfig3"),
    "IDtcToXaHelper": SimTypeBottom(label="IDtcToXaHelper"),
    "IDtcToXaHelperFactory": SimTypeBottom(label="IDtcToXaHelperFactory"),
    "IDtcToXaHelperSinglePipe": SimTypeBottom(label="IDtcToXaHelperSinglePipe"),
    "IDtcToXaMapper": SimTypeBottom(label="IDtcToXaMapper"),
    "IDummyHICONIncluder": SimTypeBottom(label="IDummyHICONIncluder"),
    "IDummyMBNUCMExt": SimTypeBottom(label="IDummyMBNUCMExt"),
    "IDvbCableDeliverySystemDescriptor": SimTypeBottom(label="IDvbCableDeliverySystemDescriptor"),
    "IDvbComponentDescriptor": SimTypeBottom(label="IDvbComponentDescriptor"),
    "IDvbContentDescriptor": SimTypeBottom(label="IDvbContentDescriptor"),
    "IDvbContentIdentifierDescriptor": SimTypeBottom(label="IDvbContentIdentifierDescriptor"),
    "IDvbDataBroadcastDescriptor": SimTypeBottom(label="IDvbDataBroadcastDescriptor"),
    "IDvbDataBroadcastIDDescriptor": SimTypeBottom(label="IDvbDataBroadcastIDDescriptor"),
    "IDvbDefaultAuthorityDescriptor": SimTypeBottom(label="IDvbDefaultAuthorityDescriptor"),
    "IDvbExtendedEventDescriptor": SimTypeBottom(label="IDvbExtendedEventDescriptor"),
    "IDvbFrequencyListDescriptor": SimTypeBottom(label="IDvbFrequencyListDescriptor"),
    "IDvbHDSimulcastLogicalChannelDescriptor": SimTypeBottom(label="IDvbHDSimulcastLogicalChannelDescriptor"),
    "IDvbLinkageDescriptor": SimTypeBottom(label="IDvbLinkageDescriptor"),
    "IDvbLogicalChannel2Descriptor": SimTypeBottom(label="IDvbLogicalChannel2Descriptor"),
    "IDvbLogicalChannelDescriptor": SimTypeBottom(label="IDvbLogicalChannelDescriptor"),
    "IDvbLogicalChannelDescriptor2": SimTypeBottom(label="IDvbLogicalChannelDescriptor2"),
    "IDvbMultilingualServiceNameDescriptor": SimTypeBottom(label="IDvbMultilingualServiceNameDescriptor"),
    "IDvbNetworkNameDescriptor": SimTypeBottom(label="IDvbNetworkNameDescriptor"),
    "IDvbParentalRatingDescriptor": SimTypeBottom(label="IDvbParentalRatingDescriptor"),
    "IDvbPrivateDataSpecifierDescriptor": SimTypeBottom(label="IDvbPrivateDataSpecifierDescriptor"),
    "IDvbSatelliteDeliverySystemDescriptor": SimTypeBottom(label="IDvbSatelliteDeliverySystemDescriptor"),
    "IDvbServiceAttributeDescriptor": SimTypeBottom(label="IDvbServiceAttributeDescriptor"),
    "IDvbServiceDescriptor": SimTypeBottom(label="IDvbServiceDescriptor"),
    "IDvbServiceDescriptor2": SimTypeBottom(label="IDvbServiceDescriptor2"),
    "IDvbServiceListDescriptor": SimTypeBottom(label="IDvbServiceListDescriptor"),
    "IDvbShortEventDescriptor": SimTypeBottom(label="IDvbShortEventDescriptor"),
    "IDvbSiParser": SimTypeBottom(label="IDvbSiParser"),
    "IDvbSiParser2": SimTypeBottom(label="IDvbSiParser2"),
    "IDvbSubtitlingDescriptor": SimTypeBottom(label="IDvbSubtitlingDescriptor"),
    "IDvbTeletextDescriptor": SimTypeBottom(label="IDvbTeletextDescriptor"),
    "IDvbTerrestrial2DeliverySystemDescriptor": SimTypeBottom(label="IDvbTerrestrial2DeliverySystemDescriptor"),
    "IDvbTerrestrialDeliverySystemDescriptor": SimTypeBottom(label="IDvbTerrestrialDeliverySystemDescriptor"),
    "IDvdCmd": SimTypeBottom(label="IDvdCmd"),
    "IDvdControl": SimTypeBottom(label="IDvdControl"),
    "IDvdControl2": SimTypeBottom(label="IDvdControl2"),
    "IDvdGraphBuilder": SimTypeBottom(label="IDvdGraphBuilder"),
    "IDvdInfo": SimTypeBottom(label="IDvdInfo"),
    "IDvdInfo2": SimTypeBottom(label="IDvdInfo2"),
    "IDvdState": SimTypeBottom(label="IDvdState"),
    "IDxcAssembler": SimTypeBottom(label="IDxcAssembler"),
    "IDxcBlob": SimTypeBottom(label="IDxcBlob"),
    "IDxcBlobEncoding": SimTypeBottom(label="IDxcBlobEncoding"),
    "IDxcBlobUtf16": SimTypeBottom(label="IDxcBlobUtf16"),
    "IDxcBlobUtf8": SimTypeBottom(label="IDxcBlobUtf8"),
    "IDxcCompiler": SimTypeBottom(label="IDxcCompiler"),
    "IDxcCompiler2": SimTypeBottom(label="IDxcCompiler2"),
    "IDxcCompiler3": SimTypeBottom(label="IDxcCompiler3"),
    "IDxcCompilerArgs": SimTypeBottom(label="IDxcCompilerArgs"),
    "IDxcContainerBuilder": SimTypeBottom(label="IDxcContainerBuilder"),
    "IDxcContainerReflection": SimTypeBottom(label="IDxcContainerReflection"),
    "IDxcExtraOutputs": SimTypeBottom(label="IDxcExtraOutputs"),
    "IDxcIncludeHandler": SimTypeBottom(label="IDxcIncludeHandler"),
    "IDxcLibrary": SimTypeBottom(label="IDxcLibrary"),
    "IDxcLinker": SimTypeBottom(label="IDxcLinker"),
    "IDxcOperationResult": SimTypeBottom(label="IDxcOperationResult"),
    "IDxcOptimizer": SimTypeBottom(label="IDxcOptimizer"),
    "IDxcOptimizerPass": SimTypeBottom(label="IDxcOptimizerPass"),
    "IDxcPdbUtils": SimTypeBottom(label="IDxcPdbUtils"),
    "IDxcResult": SimTypeBottom(label="IDxcResult"),
    "IDxcUtils": SimTypeBottom(label="IDxcUtils"),
    "IDxcValidator": SimTypeBottom(label="IDxcValidator"),
    "IDxcValidator2": SimTypeBottom(label="IDxcValidator2"),
    "IDxcVersionInfo": SimTypeBottom(label="IDxcVersionInfo"),
    "IDxcVersionInfo2": SimTypeBottom(label="IDxcVersionInfo2"),
    "IDxcVersionInfo3": SimTypeBottom(label="IDxcVersionInfo3"),
    "IDynamicConceptProviderConcept": SimTypeBottom(label="IDynamicConceptProviderConcept"),
    "IDynamicHWHandler": SimTypeBottom(label="IDynamicHWHandler"),
    "IDynamicKeyProviderConcept": SimTypeBottom(label="IDynamicKeyProviderConcept"),
    "IDynamicPortMapping": SimTypeBottom(label="IDynamicPortMapping"),
    "IDynamicPortMappingCollection": SimTypeBottom(label="IDynamicPortMappingCollection"),
    "IDynamicRenderer": SimTypeBottom(label="IDynamicRenderer"),
    "IEAPProviderConfig": SimTypeBottom(label="IEAPProviderConfig"),
    "IEAPProviderConfig2": SimTypeBottom(label="IEAPProviderConfig2"),
    "IEAPProviderConfig3": SimTypeBottom(label="IEAPProviderConfig3"),
    "IEBARMENUSTATES": SimTypeInt(signed=False, label="IEBARMENUSTATES"),
    "IEC_GESTUREINFO": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("Cursor", SimTypeBottom(label="IInkCursor")), ("Strokes", SimTypeBottom(label="IInkStrokes")), ("Gestures", SimTypeRef("VARIANT", SimStruct)),)), name="IEC_GESTUREINFO", pack=False, align=None),
    "IEC_RECOGNITIONRESULTINFO": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("RecognitionResult", SimTypeBottom(label="IInkRecognitionResult")),)), name="IEC_RECOGNITIONRESULTINFO", pack=False, align=None),
    "IEC_STROKEINFO": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("Cursor", SimTypeBottom(label="IInkCursor")), ("Stroke", SimTypeBottom(label="IInkStrokeDisp")),)), name="IEC_STROKEINFO", pack=False, align=None),
    "IEISXMLNSREGISTEREDFN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszURI", "pCLSID"]), offset=0),
    "IELAUNCHOPTION_FLAGS": SimTypeInt(signed=False, label="IELAUNCHOPTION_FLAGS"),
    "IELAUNCHURLINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwCreationFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLaunchOptionFlags", SimTypeInt(signed=False, label="UInt32")),)), name="IELAUNCHURLINFO", pack=False, align=None),
    "IEObjectType": SimTypeInt(signed=False, label="IEObjectType"),
    "IEPDNFLAGS": SimTypeInt(signed=False, label="IEPDNFLAGS"),
    "IEREGISTERXMLNSFN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="Guid"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszURI", "clsid", "fMachine"]), offset=0),
    "IESCloseMmiEvent": SimTypeBottom(label="IESCloseMmiEvent"),
    "IESEvent": SimTypeBottom(label="IESEvent"),
    "IESEventFactory": SimTypeBottom(label="IESEventFactory"),
    "IESEventService": SimTypeBottom(label="IESEventService"),
    "IESEventServiceConfiguration": SimTypeBottom(label="IESEventServiceConfiguration"),
    "IESEvents": SimTypeBottom(label="IESEvents"),
    "IESFileExpiryDateEvent": SimTypeBottom(label="IESFileExpiryDateEvent"),
    "IESHORTCUTFLAGS": SimTypeInt(signed=False, label="IESHORTCUTFLAGS"),
    "IESIsdbCasResponseEvent": SimTypeBottom(label="IESIsdbCasResponseEvent"),
    "IESLicenseRenewalResultEvent": SimTypeBottom(label="IESLicenseRenewalResultEvent"),
    "IESOpenMmiEvent": SimTypeBottom(label="IESOpenMmiEvent"),
    "IESRequestTunerEvent": SimTypeBottom(label="IESRequestTunerEvent"),
    "IESValueUpdatedEvent": SimTypeBottom(label="IESValueUpdatedEvent"),
    "IETFilter": SimTypeBottom(label="IETFilter"),
    "IETFilterConfig": SimTypeBottom(label="IETFilterConfig"),
    "IETFilterEvents": SimTypeBottom(label="IETFilterEvents"),
    "IEVRFilterConfig": SimTypeBottom(label="IEVRFilterConfig"),
    "IEVRFilterConfigEx": SimTypeBottom(label="IEVRFilterConfigEx"),
    "IEVRTrustedVideoPlugin": SimTypeBottom(label="IEVRTrustedVideoPlugin"),
    "IEVRVideoStreamControl": SimTypeBottom(label="IEVRVideoStreamControl"),
    "IEditionUpgradeBroker": SimTypeBottom(label="IEditionUpgradeBroker"),
    "IEditionUpgradeHelper": SimTypeBottom(label="IEditionUpgradeHelper"),
    "IEffectivePermission": SimTypeBottom(label="IEffectivePermission"),
    "IEffectivePermission2": SimTypeBottom(label="IEffectivePermission2"),
    "IElementBehavior": SimTypeBottom(label="IElementBehavior"),
    "IElementBehaviorCategory": SimTypeBottom(label="IElementBehaviorCategory"),
    "IElementBehaviorFactory": SimTypeBottom(label="IElementBehaviorFactory"),
    "IElementBehaviorFocus": SimTypeBottom(label="IElementBehaviorFocus"),
    "IElementBehaviorLayout": SimTypeBottom(label="IElementBehaviorLayout"),
    "IElementBehaviorLayout2": SimTypeBottom(label="IElementBehaviorLayout2"),
    "IElementBehaviorRender": SimTypeBottom(label="IElementBehaviorRender"),
    "IElementBehaviorSite": SimTypeBottom(label="IElementBehaviorSite"),
    "IElementBehaviorSiteCategory": SimTypeBottom(label="IElementBehaviorSiteCategory"),
    "IElementBehaviorSiteLayout": SimTypeBottom(label="IElementBehaviorSiteLayout"),
    "IElementBehaviorSiteLayout2": SimTypeBottom(label="IElementBehaviorSiteLayout2"),
    "IElementBehaviorSiteOM": SimTypeBottom(label="IElementBehaviorSiteOM"),
    "IElementBehaviorSiteOM2": SimTypeBottom(label="IElementBehaviorSiteOM2"),
    "IElementBehaviorSiteRender": SimTypeBottom(label="IElementBehaviorSiteRender"),
    "IElementBehaviorSubmit": SimTypeBottom(label="IElementBehaviorSubmit"),
    "IElementNamespace": SimTypeBottom(label="IElementNamespace"),
    "IElementNamespaceFactory": SimTypeBottom(label="IElementNamespaceFactory"),
    "IElementNamespaceFactory2": SimTypeBottom(label="IElementNamespaceFactory2"),
    "IElementNamespaceFactoryCallback": SimTypeBottom(label="IElementNamespaceFactoryCallback"),
    "IElementNamespaceTable": SimTypeBottom(label="IElementNamespaceTable"),
    "IElementSegment": SimTypeBottom(label="IElementSegment"),
    "IElementSelector": SimTypeBottom(label="IElementSelector"),
    "IElementTraversal": SimTypeBottom(label="IElementTraversal"),
    "IEmailAction": SimTypeBottom(label="IEmailAction"),
    "IEmptyVolumeCache": SimTypeBottom(label="IEmptyVolumeCache"),
    "IEmptyVolumeCache2": SimTypeBottom(label="IEmptyVolumeCache2"),
    "IEmptyVolumeCacheCallBack": SimTypeBottom(label="IEmptyVolumeCacheCallBack"),
    "IEncoderAPI": SimTypeBottom(label="IEncoderAPI"),
    "IEncodingFilterFactory": SimTypeBottom(label="IEncodingFilterFactory"),
    "IEnhancedStorageACT": SimTypeBottom(label="IEnhancedStorageACT"),
    "IEnhancedStorageACT2": SimTypeBottom(label="IEnhancedStorageACT2"),
    "IEnhancedStorageACT3": SimTypeBottom(label="IEnhancedStorageACT3"),
    "IEnhancedStorageSilo": SimTypeBottom(label="IEnhancedStorageSilo"),
    "IEnhancedStorageSiloAction": SimTypeBottom(label="IEnhancedStorageSiloAction"),
    "IEnroll": SimTypeBottom(label="IEnroll"),
    "IEnroll2": SimTypeBottom(label="IEnroll2"),
    "IEnroll4": SimTypeBottom(label="IEnroll4"),
    "IEnterpriseDropTarget": SimTypeBottom(label="IEnterpriseDropTarget"),
    "IEntity": SimTypeBottom(label="IEntity"),
    "IEnumACDGroup": SimTypeBottom(label="IEnumACDGroup"),
    "IEnumACString": SimTypeBottom(label="IEnumACString"),
    "IEnumAddress": SimTypeBottom(label="IEnumAddress"),
    "IEnumAgent": SimTypeBottom(label="IEnumAgent"),
    "IEnumAgentHandler": SimTypeBottom(label="IEnumAgentHandler"),
    "IEnumAgentSession": SimTypeBottom(label="IEnumAgentSession"),
    "IEnumAssocHandlers": SimTypeBottom(label="IEnumAssocHandlers"),
    "IEnumBackgroundCopyFiles": SimTypeBottom(label="IEnumBackgroundCopyFiles"),
    "IEnumBackgroundCopyGroups": SimTypeBottom(label="IEnumBackgroundCopyGroups"),
    "IEnumBackgroundCopyJobs": SimTypeBottom(label="IEnumBackgroundCopyJobs"),
    "IEnumBackgroundCopyJobs1": SimTypeBottom(label="IEnumBackgroundCopyJobs1"),
    "IEnumBitsPeerCacheRecords": SimTypeBottom(label="IEnumBitsPeerCacheRecords"),
    "IEnumBitsPeers": SimTypeBottom(label="IEnumBitsPeers"),
    "IEnumBstr": SimTypeBottom(label="IEnumBstr"),
    "IEnumCATEGORYINFO": SimTypeBottom(label="IEnumCATEGORYINFO"),
    "IEnumCERTVIEWATTRIBUTE": SimTypeBottom(label="IEnumCERTVIEWATTRIBUTE"),
    "IEnumCERTVIEWCOLUMN": SimTypeBottom(label="IEnumCERTVIEWCOLUMN"),
    "IEnumCERTVIEWEXTENSION": SimTypeBottom(label="IEnumCERTVIEWEXTENSION"),
    "IEnumCERTVIEWROW": SimTypeBottom(label="IEnumCERTVIEWROW"),
    "IEnumCall": SimTypeBottom(label="IEnumCall"),
    "IEnumCallHub": SimTypeBottom(label="IEnumCallHub"),
    "IEnumCallingCard": SimTypeBottom(label="IEnumCallingCard"),
    "IEnumChangeUnitExceptions": SimTypeBottom(label="IEnumChangeUnitExceptions"),
    "IEnumClockVector": SimTypeBottom(label="IEnumClockVector"),
    "IEnumCodePage": SimTypeBottom(label="IEnumCodePage"),
    "IEnumComponentTypes": SimTypeBottom(label="IEnumComponentTypes"),
    "IEnumComponents": SimTypeBottom(label="IEnumComponents"),
    "IEnumConnectionPoints": SimTypeBottom(label="IEnumConnectionPoints"),
    "IEnumConnections": SimTypeBottom(label="IEnumConnections"),
    "IEnumContextProps": SimTypeBottom(label="IEnumContextProps"),
    "IEnumDMO": SimTypeBottom(label="IEnumDMO"),
    "IEnumDebugApplicationNodes": SimTypeBottom(label="IEnumDebugApplicationNodes"),
    "IEnumDebugCodeContexts": SimTypeBottom(label="IEnumDebugCodeContexts"),
    "IEnumDebugExpressionContexts": SimTypeBottom(label="IEnumDebugExpressionContexts"),
    "IEnumDebugExtendedPropertyInfo": SimTypeBottom(label="IEnumDebugExtendedPropertyInfo"),
    "IEnumDebugPropertyInfo": SimTypeBottom(label="IEnumDebugPropertyInfo"),
    "IEnumDebugStackFrames": SimTypeBottom(label="IEnumDebugStackFrames"),
    "IEnumDebugStackFrames64": SimTypeBottom(label="IEnumDebugStackFrames64"),
    "IEnumDialableAddrs": SimTypeBottom(label="IEnumDialableAddrs"),
    "IEnumDirectory": SimTypeBottom(label="IEnumDirectory"),
    "IEnumDirectoryObject": SimTypeBottom(label="IEnumDirectoryObject"),
    "IEnumDiscMasterFormats": SimTypeBottom(label="IEnumDiscMasterFormats"),
    "IEnumDiscRecorders": SimTypeBottom(label="IEnumDiscRecorders"),
    "IEnumDiskQuotaUsers": SimTypeBottom(label="IEnumDiskQuotaUsers"),
    "IEnumDot11AdHocInterfaces": SimTypeBottom(label="IEnumDot11AdHocInterfaces"),
    "IEnumDot11AdHocNetworks": SimTypeBottom(label="IEnumDot11AdHocNetworks"),
    "IEnumDot11AdHocSecuritySettings": SimTypeBottom(label="IEnumDot11AdHocSecuritySettings"),
    "IEnumEnhancedStorageACT": SimTypeBottom(label="IEnumEnhancedStorageACT"),
    "IEnumEventObject": SimTypeBottom(label="IEnumEventObject"),
    "IEnumExplorerCommand": SimTypeBottom(label="IEnumExplorerCommand"),
    "IEnumExtraSearch": SimTypeBottom(label="IEnumExtraSearch"),
    "IEnumFORMATETC": SimTypeBottom(label="IEnumFORMATETC"),
    "IEnumFeedClockVector": SimTypeBottom(label="IEnumFeedClockVector"),
    "IEnumFilters": SimTypeBottom(label="IEnumFilters"),
    "IEnumFsiItems": SimTypeBottom(label="IEnumFsiItems"),
    "IEnumFullIDList": SimTypeBottom(label="IEnumFullIDList"),
    "IEnumGUID": SimTypeBottom(label="IEnumGUID"),
    "IEnumGuideDataProperties": SimTypeBottom(label="IEnumGuideDataProperties"),
    "IEnumHLITEM": SimTypeBottom(label="IEnumHLITEM"),
    "IEnumIDList": SimTypeBottom(label="IEnumIDList"),
    "IEnumITfCompositionView": SimTypeBottom(label="IEnumITfCompositionView"),
    "IEnumInputContext": SimTypeBottom(label="IEnumInputContext"),
    "IEnumItemIds": SimTypeBottom(label="IEnumItemIds"),
    "IEnumItemProperties": SimTypeBottom(label="IEnumItemProperties"),
    "IEnumJsStackFrames": SimTypeBottom(label="IEnumJsStackFrames"),
    "IEnumLocation": SimTypeBottom(label="IEnumLocation"),
    "IEnumMSVidGraphSegment": SimTypeBottom(label="IEnumMSVidGraphSegment"),
    "IEnumManagerFrames": SimTypeBottom(label="IEnumManagerFrames"),
    "IEnumMcastScope": SimTypeBottom(label="IEnumMcastScope"),
    "IEnumMediaTypes": SimTypeBottom(label="IEnumMediaTypes"),
    "IEnumMoniker": SimTypeBottom(label="IEnumMoniker"),
    "IEnumMsmDependency": SimTypeBottom(label="IEnumMsmDependency"),
    "IEnumMsmError": SimTypeBottom(label="IEnumMsmError"),
    "IEnumMsmString": SimTypeBottom(label="IEnumMsmString"),
    "IEnumNames": SimTypeBottom(label="IEnumNames"),
    "IEnumNetCfgBindingInterface": SimTypeBottom(label="IEnumNetCfgBindingInterface"),
    "IEnumNetCfgBindingPath": SimTypeBottom(label="IEnumNetCfgBindingPath"),
    "IEnumNetCfgComponent": SimTypeBottom(label="IEnumNetCfgComponent"),
    "IEnumNetConnection": SimTypeBottom(label="IEnumNetConnection"),
    "IEnumNetSharingEveryConnection": SimTypeBottom(label="IEnumNetSharingEveryConnection"),
    "IEnumNetSharingPortMapping": SimTypeBottom(label="IEnumNetSharingPortMapping"),
    "IEnumNetSharingPrivateConnection": SimTypeBottom(label="IEnumNetSharingPrivateConnection"),
    "IEnumNetSharingPublicConnection": SimTypeBottom(label="IEnumNetSharingPublicConnection"),
    "IEnumNetworkConnections": SimTypeBottom(label="IEnumNetworkConnections"),
    "IEnumNetworks": SimTypeBottom(label="IEnumNetworks"),
    "IEnumOLEVERB": SimTypeBottom(label="IEnumOLEVERB"),
    "IEnumObjects": SimTypeBottom(label="IEnumObjects"),
    "IEnumOfflineFilesItems": SimTypeBottom(label="IEnumOfflineFilesItems"),
    "IEnumOfflineFilesSettings": SimTypeBottom(label="IEnumOfflineFilesSettings"),
    "IEnumOleDocumentViews": SimTypeBottom(label="IEnumOleDocumentViews"),
    "IEnumOleUndoUnits": SimTypeBottom(label="IEnumOleUndoUnits"),
    "IEnumOpenServiceActivity": SimTypeBottom(label="IEnumOpenServiceActivity"),
    "IEnumOpenServiceActivityCategory": SimTypeBottom(label="IEnumOpenServiceActivityCategory"),
    "IEnumPIDMap": SimTypeBottom(label="IEnumPIDMap"),
    "IEnumPhone": SimTypeBottom(label="IEnumPhone"),
    "IEnumPins": SimTypeBottom(label="IEnumPins"),
    "IEnumPluggableSuperclassInfo": SimTypeBottom(label="IEnumPluggableSuperclassInfo"),
    "IEnumPluggableTerminalClassInfo": SimTypeBottom(label="IEnumPluggableTerminalClassInfo"),
    "IEnumPortableDeviceConnectors": SimTypeBottom(label="IEnumPortableDeviceConnectors"),
    "IEnumPortableDeviceObjectIDs": SimTypeBottom(label="IEnumPortableDeviceObjectIDs"),
    "IEnumPrivacyRecords": SimTypeBottom(label="IEnumPrivacyRecords"),
    "IEnumProgressItems": SimTypeBottom(label="IEnumProgressItems"),
    "IEnumPublishedApps": SimTypeBottom(label="IEnumPublishedApps"),
    "IEnumQueue": SimTypeBottom(label="IEnumQueue"),
    "IEnumRangeExceptions": SimTypeBottom(label="IEnumRangeExceptions"),
    "IEnumReadyCallback": SimTypeBottom(label="IEnumReadyCallback"),
    "IEnumRegFilters": SimTypeBottom(label="IEnumRegFilters"),
    "IEnumRegisterWordA": SimTypeBottom(label="IEnumRegisterWordA"),
    "IEnumRegisterWordW": SimTypeBottom(label="IEnumRegisterWordW"),
    "IEnumRemoteDebugApplicationThreads": SimTypeBottom(label="IEnumRemoteDebugApplicationThreads"),
    "IEnumRemoteDebugApplications": SimTypeBottom(label="IEnumRemoteDebugApplications"),
    "IEnumResources": SimTypeBottom(label="IEnumResources"),
    "IEnumRfc1766": SimTypeBottom(label="IEnumRfc1766"),
    "IEnumSTATDATA": SimTypeBottom(label="IEnumSTATDATA"),
    "IEnumSTATPROPSETSTG": SimTypeBottom(label="IEnumSTATPROPSETSTG"),
    "IEnumSTATPROPSTG": SimTypeBottom(label="IEnumSTATPROPSTG"),
    "IEnumSTATSTG": SimTypeBottom(label="IEnumSTATSTG"),
    "IEnumSTATURL": SimTypeBottom(label="IEnumSTATURL"),
    "IEnumScript": SimTypeBottom(label="IEnumScript"),
    "IEnumSearchRoots": SimTypeBottom(label="IEnumSearchRoots"),
    "IEnumSearchScopeRules": SimTypeBottom(label="IEnumSearchScopeRules"),
    "IEnumShellItems": SimTypeBottom(label="IEnumShellItems"),
    "IEnumSingleItemExceptions": SimTypeBottom(label="IEnumSingleItemExceptions"),
    "IEnumSpObjectTokens": SimTypeBottom(label="IEnumSpObjectTokens"),
    "IEnumSpeechCommands": SimTypeBottom(label="IEnumSpeechCommands"),
    "IEnumSpellingError": SimTypeBottom(label="IEnumSpellingError"),
    "IEnumStream": SimTypeBottom(label="IEnumStream"),
    "IEnumStreamBufferRecordingAttrib": SimTypeBottom(label="IEnumStreamBufferRecordingAttrib"),
    "IEnumStreamIdMap": SimTypeBottom(label="IEnumStreamIdMap"),
    "IEnumString": SimTypeBottom(label="IEnumString"),
    "IEnumSubStream": SimTypeBottom(label="IEnumSubStream"),
    "IEnumSubscription": SimTypeBottom(label="IEnumSubscription"),
    "IEnumSyncChangeUnits": SimTypeBottom(label="IEnumSyncChangeUnits"),
    "IEnumSyncChanges": SimTypeBottom(label="IEnumSyncChanges"),
    "IEnumSyncMgrConflict": SimTypeBottom(label="IEnumSyncMgrConflict"),
    "IEnumSyncMgrEvents": SimTypeBottom(label="IEnumSyncMgrEvents"),
    "IEnumSyncMgrSyncItems": SimTypeBottom(label="IEnumSyncMgrSyncItems"),
    "IEnumSyncProviderConfigUIInfos": SimTypeBottom(label="IEnumSyncProviderConfigUIInfos"),
    "IEnumSyncProviderInfos": SimTypeBottom(label="IEnumSyncProviderInfos"),
    "IEnumTASK": SimTypeBottom(label="IEnumTASK"),
    "IEnumTerminal": SimTypeBottom(label="IEnumTerminal"),
    "IEnumTerminalClass": SimTypeBottom(label="IEnumTerminalClass"),
    "IEnumTfCandidates": SimTypeBottom(label="IEnumTfCandidates"),
    "IEnumTfContextViews": SimTypeBottom(label="IEnumTfContextViews"),
    "IEnumTfContexts": SimTypeBottom(label="IEnumTfContexts"),
    "IEnumTfDisplayAttributeInfo": SimTypeBottom(label="IEnumTfDisplayAttributeInfo"),
    "IEnumTfDocumentMgrs": SimTypeBottom(label="IEnumTfDocumentMgrs"),
    "IEnumTfFunctionProviders": SimTypeBottom(label="IEnumTfFunctionProviders"),
    "IEnumTfInputProcessorProfiles": SimTypeBottom(label="IEnumTfInputProcessorProfiles"),
    "IEnumTfLangBarItems": SimTypeBottom(label="IEnumTfLangBarItems"),
    "IEnumTfLanguageProfiles": SimTypeBottom(label="IEnumTfLanguageProfiles"),
    "IEnumTfLatticeElements": SimTypeBottom(label="IEnumTfLatticeElements"),
    "IEnumTfProperties": SimTypeBottom(label="IEnumTfProperties"),
    "IEnumTfPropertyValue": SimTypeBottom(label="IEnumTfPropertyValue"),
    "IEnumTfRanges": SimTypeBottom(label="IEnumTfRanges"),
    "IEnumTfUIElements": SimTypeBottom(label="IEnumTfUIElements"),
    "IEnumTravelLogEntry": SimTypeBottom(label="IEnumTravelLogEntry"),
    "IEnumTuneRequests": SimTypeBottom(label="IEnumTuneRequests"),
    "IEnumTuningSpaces": SimTypeBottom(label="IEnumTuningSpaces"),
    "IEnumUnknown": SimTypeBottom(label="IEnumUnknown"),
    "IEnumVARIANT": SimTypeBottom(label="IEnumVARIANT"),
    "IEnumVdsObject": SimTypeBottom(label="IEnumVdsObject"),
    "IEnumWIA_DEV_CAPS": SimTypeBottom(label="IEnumWIA_DEV_CAPS"),
    "IEnumWIA_DEV_INFO": SimTypeBottom(label="IEnumWIA_DEV_INFO"),
    "IEnumWIA_FORMAT_INFO": SimTypeBottom(label="IEnumWIA_FORMAT_INFO"),
    "IEnumWbemClassObject": SimTypeBottom(label="IEnumWbemClassObject"),
    "IEnumWiaItem": SimTypeBottom(label="IEnumWiaItem"),
    "IEnumWiaItem2": SimTypeBottom(label="IEnumWiaItem2"),
    "IEnumWorkItems": SimTypeBottom(label="IEnumWorkItems"),
    "IEnumerableView": SimTypeBottom(label="IEnumerableView"),
    "IEquatableConcept": SimTypeBottom(label="IEquatableConcept"),
    "IErrorInfo": SimTypeBottom(label="IErrorInfo"),
    "IErrorLog": SimTypeBottom(label="IErrorLog"),
    "IErrorLookup": SimTypeBottom(label="IErrorLookup"),
    "IErrorRecords": SimTypeBottom(label="IErrorRecords"),
    "IEvalRat": SimTypeBottom(label="IEvalRat"),
    "IEventClass": SimTypeBottom(label="IEventClass"),
    "IEventClass2": SimTypeBottom(label="IEventClass2"),
    "IEventControl": SimTypeBottom(label="IEventControl"),
    "IEventException": SimTypeBottom(label="IEventException"),
    "IEventObjectChange": SimTypeBottom(label="IEventObjectChange"),
    "IEventObjectChange2": SimTypeBottom(label="IEventObjectChange2"),
    "IEventObjectCollection": SimTypeBottom(label="IEventObjectCollection"),
    "IEventProperty": SimTypeBottom(label="IEventProperty"),
    "IEventPublisher": SimTypeBottom(label="IEventPublisher"),
    "IEventServerTrace": SimTypeBottom(label="IEventServerTrace"),
    "IEventSubscription": SimTypeBottom(label="IEventSubscription"),
    "IEventSystem": SimTypeBottom(label="IEventSystem"),
    "IEventTarget": SimTypeBottom(label="IEventTarget"),
    "IEventTarget2": SimTypeBottom(label="IEventTarget2"),
    "IEventTrigger": SimTypeBottom(label="IEventTrigger"),
    "IExecAction": SimTypeBottom(label="IExecAction"),
    "IExecAction2": SimTypeBottom(label="IExecAction2"),
    "IExecuteCommand": SimTypeBottom(label="IExecuteCommand"),
    "IExecuteCommandApplicationHostEnvironment": SimTypeBottom(label="IExecuteCommandApplicationHostEnvironment"),
    "IExecuteCommandHost": SimTypeBottom(label="IExecuteCommandHost"),
    "IExpDispSupport": SimTypeBottom(label="IExpDispSupport"),
    "IExpDispSupportXP": SimTypeBottom(label="IExpDispSupportXP"),
    "IExpandCollapseProvider": SimTypeBottom(label="IExpandCollapseProvider"),
    "IExplorerBrowser": SimTypeBottom(label="IExplorerBrowser"),
    "IExplorerBrowserEvents": SimTypeBottom(label="IExplorerBrowserEvents"),
    "IExplorerCommand": SimTypeBottom(label="IExplorerCommand"),
    "IExplorerCommandProvider": SimTypeBottom(label="IExplorerCommandProvider"),
    "IExplorerCommandState": SimTypeBottom(label="IExplorerCommandState"),
    "IExplorerPaneVisibility": SimTypeBottom(label="IExplorerPaneVisibility"),
    "IExtendContextMenu": SimTypeBottom(label="IExtendContextMenu"),
    "IExtendControlbar": SimTypeBottom(label="IExtendControlbar"),
    "IExtendPropertySheet": SimTypeBottom(label="IExtendPropertySheet"),
    "IExtendPropertySheet2": SimTypeBottom(label="IExtendPropertySheet2"),
    "IExtendTaskPad": SimTypeBottom(label="IExtendTaskPad"),
    "IExtendView": SimTypeBottom(label="IExtendView"),
    "IExtensionServices": SimTypeBottom(label="IExtensionServices"),
    "IExtensionValidation": SimTypeBottom(label="IExtensionValidation"),
    "IExternalConnection": SimTypeBottom(label="IExternalConnection"),
    "IExtractIconA": SimTypeBottom(label="IExtractIconA"),
    "IExtractIconW": SimTypeBottom(label="IExtractIconW"),
    "IExtractImage": SimTypeBottom(label="IExtractImage"),
    "IExtractImage2": SimTypeBottom(label="IExtractImage2"),
    "IFClipNotificationHelper": SimTypeBottom(label="IFClipNotificationHelper"),
    "IFEClassFactory": SimTypeBottom(label="IFEClassFactory"),
    "IFECommon": SimTypeBottom(label="IFECommon"),
    "IFEDictionary": SimTypeBottom(label="IFEDictionary"),
    "IFELanguage": SimTypeBottom(label="IFELanguage"),
    "IFIEXTRA": SimStruct(OrderedDict((("ulIdentifier", SimTypeInt(signed=False, label="UInt32")), ("dpFontSig", SimTypeInt(signed=True, label="Int32")), ("cig", SimTypeInt(signed=False, label="UInt32")), ("dpDesignVector", SimTypeInt(signed=True, label="Int32")), ("dpAxesInfoW", SimTypeInt(signed=True, label="Int32")), ("aulReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IFIEXTRA", pack=False, align=None),
    "IFILTERMAPPER_MERIT": SimTypeInt(signed=False, label="IFILTERMAPPER_MERIT"),
    "IFILTER_FLAGS": SimTypeInt(signed=False, label="IFILTER_FLAGS"),
    "IFILTER_INIT": SimTypeInt(signed=False, label="IFILTER_INIT"),
    "IFIMETRICS": SimStruct(OrderedDict((("cjThis", SimTypeInt(signed=False, label="UInt32")), ("cjIfiExtra", SimTypeInt(signed=False, label="UInt32")), ("dpwszFamilyName", SimTypeInt(signed=True, label="Int32")), ("dpwszStyleName", SimTypeInt(signed=True, label="Int32")), ("dpwszFaceName", SimTypeInt(signed=True, label="Int32")), ("dpwszUniqueName", SimTypeInt(signed=True, label="Int32")), ("dpFontSim", SimTypeInt(signed=True, label="Int32")), ("lEmbedId", SimTypeInt(signed=True, label="Int32")), ("lItalicAngle", SimTypeInt(signed=True, label="Int32")), ("lCharBias", SimTypeInt(signed=True, label="Int32")), ("dpCharSets", SimTypeInt(signed=True, label="Int32")), ("jWinCharSet", SimTypeChar(label="Byte")), ("jWinPitchAndFamily", SimTypeChar(label="Byte")), ("usWinWeight", SimTypeShort(signed=False, label="UInt16")), ("flInfo", SimTypeInt(signed=False, label="UInt32")), ("fsSelection", SimTypeShort(signed=False, label="UInt16")), ("fsType", SimTypeShort(signed=False, label="UInt16")), ("fwdUnitsPerEm", SimTypeShort(signed=True, label="Int16")), ("fwdLowestPPEm", SimTypeShort(signed=True, label="Int16")), ("fwdWinAscender", SimTypeShort(signed=True, label="Int16")), ("fwdWinDescender", SimTypeShort(signed=True, label="Int16")), ("fwdMacAscender", SimTypeShort(signed=True, label="Int16")), ("fwdMacDescender", SimTypeShort(signed=True, label="Int16")), ("fwdMacLineGap", SimTypeShort(signed=True, label="Int16")), ("fwdTypoAscender", SimTypeShort(signed=True, label="Int16")), ("fwdTypoDescender", SimTypeShort(signed=True, label="Int16")), ("fwdTypoLineGap", SimTypeShort(signed=True, label="Int16")), ("fwdAveCharWidth", SimTypeShort(signed=True, label="Int16")), ("fwdMaxCharInc", SimTypeShort(signed=True, label="Int16")), ("fwdCapHeight", SimTypeShort(signed=True, label="Int16")), ("fwdXHeight", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptXSize", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptYSize", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptXOffset", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptYOffset", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptXSize", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptYSize", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptXOffset", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptYOffset", SimTypeShort(signed=True, label="Int16")), ("fwdUnderscoreSize", SimTypeShort(signed=True, label="Int16")), ("fwdUnderscorePosition", SimTypeShort(signed=True, label="Int16")), ("fwdStrikeoutSize", SimTypeShort(signed=True, label="Int16")), ("fwdStrikeoutPosition", SimTypeShort(signed=True, label="Int16")), ("chFirstChar", SimTypeChar(label="Byte")), ("chLastChar", SimTypeChar(label="Byte")), ("chDefaultChar", SimTypeChar(label="Byte")), ("chBreakChar", SimTypeChar(label="Byte")), ("wcFirstChar", SimTypeChar(label="Char")), ("wcLastChar", SimTypeChar(label="Char")), ("wcDefaultChar", SimTypeChar(label="Char")), ("wcBreakChar", SimTypeChar(label="Char")), ("ptlBaseline", SimTypeRef("POINTL", SimStruct)), ("ptlAspect", SimTypeRef("POINTL", SimStruct)), ("ptlCaret", SimTypeRef("POINTL", SimStruct)), ("rclFontBox", SimTypeRef("RECTL", SimStruct)), ("achVendId", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("cKerningPairs", SimTypeInt(signed=False, label="UInt32")), ("ulPanoseCulture", SimTypeInt(signed=False, label="UInt32")), ("panose", SimTypeRef("PANOSE", SimStruct)),)), name="IFIMETRICS", pack=False, align=None),
    "IF_ACCESS_TYPE": SimTypeInt(signed=False, label="IF_ACCESS_TYPE"),
    "IF_ADMINISTRATIVE_STATE": SimTypeInt(signed=False, label="IF_ADMINISTRATIVE_STATE"),
    "IF_COUNTED_STRING_LH": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("String", SimTypeArray(SimTypeChar(label="Char"), 257)),)), name="IF_COUNTED_STRING_LH", pack=False, align=None),
    "IF_OPER_STATUS": SimTypeInt(signed=False, label="IF_OPER_STATUS"),
    "IF_PHYSICAL_ADDRESS_LH": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="IF_PHYSICAL_ADDRESS_LH", pack=False, align=None),
    "IFastRundown": SimTypeBottom(label="IFastRundown"),
    "IFaxAccount": SimTypeBottom(label="IFaxAccount"),
    "IFaxAccountFolders": SimTypeBottom(label="IFaxAccountFolders"),
    "IFaxAccountIncomingArchive": SimTypeBottom(label="IFaxAccountIncomingArchive"),
    "IFaxAccountIncomingQueue": SimTypeBottom(label="IFaxAccountIncomingQueue"),
    "IFaxAccountNotify": SimTypeBottom(label="IFaxAccountNotify"),
    "IFaxAccountOutgoingArchive": SimTypeBottom(label="IFaxAccountOutgoingArchive"),
    "IFaxAccountOutgoingQueue": SimTypeBottom(label="IFaxAccountOutgoingQueue"),
    "IFaxAccountSet": SimTypeBottom(label="IFaxAccountSet"),
    "IFaxAccounts": SimTypeBottom(label="IFaxAccounts"),
    "IFaxActivity": SimTypeBottom(label="IFaxActivity"),
    "IFaxActivityLogging": SimTypeBottom(label="IFaxActivityLogging"),
    "IFaxConfiguration": SimTypeBottom(label="IFaxConfiguration"),
    "IFaxDevice": SimTypeBottom(label="IFaxDevice"),
    "IFaxDeviceIds": SimTypeBottom(label="IFaxDeviceIds"),
    "IFaxDeviceProvider": SimTypeBottom(label="IFaxDeviceProvider"),
    "IFaxDeviceProviders": SimTypeBottom(label="IFaxDeviceProviders"),
    "IFaxDevices": SimTypeBottom(label="IFaxDevices"),
    "IFaxDocument": SimTypeBottom(label="IFaxDocument"),
    "IFaxDocument2": SimTypeBottom(label="IFaxDocument2"),
    "IFaxEventLogging": SimTypeBottom(label="IFaxEventLogging"),
    "IFaxFolders": SimTypeBottom(label="IFaxFolders"),
    "IFaxInboundRouting": SimTypeBottom(label="IFaxInboundRouting"),
    "IFaxInboundRoutingExtension": SimTypeBottom(label="IFaxInboundRoutingExtension"),
    "IFaxInboundRoutingExtensions": SimTypeBottom(label="IFaxInboundRoutingExtensions"),
    "IFaxInboundRoutingMethod": SimTypeBottom(label="IFaxInboundRoutingMethod"),
    "IFaxInboundRoutingMethods": SimTypeBottom(label="IFaxInboundRoutingMethods"),
    "IFaxIncomingArchive": SimTypeBottom(label="IFaxIncomingArchive"),
    "IFaxIncomingJob": SimTypeBottom(label="IFaxIncomingJob"),
    "IFaxIncomingJobs": SimTypeBottom(label="IFaxIncomingJobs"),
    "IFaxIncomingMessage": SimTypeBottom(label="IFaxIncomingMessage"),
    "IFaxIncomingMessage2": SimTypeBottom(label="IFaxIncomingMessage2"),
    "IFaxIncomingMessageIterator": SimTypeBottom(label="IFaxIncomingMessageIterator"),
    "IFaxIncomingQueue": SimTypeBottom(label="IFaxIncomingQueue"),
    "IFaxJobStatus": SimTypeBottom(label="IFaxJobStatus"),
    "IFaxLoggingOptions": SimTypeBottom(label="IFaxLoggingOptions"),
    "IFaxOutboundRouting": SimTypeBottom(label="IFaxOutboundRouting"),
    "IFaxOutboundRoutingGroup": SimTypeBottom(label="IFaxOutboundRoutingGroup"),
    "IFaxOutboundRoutingGroups": SimTypeBottom(label="IFaxOutboundRoutingGroups"),
    "IFaxOutboundRoutingRule": SimTypeBottom(label="IFaxOutboundRoutingRule"),
    "IFaxOutboundRoutingRules": SimTypeBottom(label="IFaxOutboundRoutingRules"),
    "IFaxOutgoingArchive": SimTypeBottom(label="IFaxOutgoingArchive"),
    "IFaxOutgoingJob": SimTypeBottom(label="IFaxOutgoingJob"),
    "IFaxOutgoingJob2": SimTypeBottom(label="IFaxOutgoingJob2"),
    "IFaxOutgoingJobs": SimTypeBottom(label="IFaxOutgoingJobs"),
    "IFaxOutgoingMessage": SimTypeBottom(label="IFaxOutgoingMessage"),
    "IFaxOutgoingMessage2": SimTypeBottom(label="IFaxOutgoingMessage2"),
    "IFaxOutgoingMessageIterator": SimTypeBottom(label="IFaxOutgoingMessageIterator"),
    "IFaxOutgoingQueue": SimTypeBottom(label="IFaxOutgoingQueue"),
    "IFaxReceiptOptions": SimTypeBottom(label="IFaxReceiptOptions"),
    "IFaxRecipient": SimTypeBottom(label="IFaxRecipient"),
    "IFaxRecipients": SimTypeBottom(label="IFaxRecipients"),
    "IFaxSecurity": SimTypeBottom(label="IFaxSecurity"),
    "IFaxSecurity2": SimTypeBottom(label="IFaxSecurity2"),
    "IFaxSender": SimTypeBottom(label="IFaxSender"),
    "IFaxServer": SimTypeBottom(label="IFaxServer"),
    "IFaxServer2": SimTypeBottom(label="IFaxServer2"),
    "IFaxServerNotify": SimTypeBottom(label="IFaxServerNotify"),
    "IFaxServerNotify2": SimTypeBottom(label="IFaxServerNotify2"),
    "IFeed": SimTypeBottom(label="IFeed"),
    "IFeed2": SimTypeBottom(label="IFeed2"),
    "IFeedClockVector": SimTypeBottom(label="IFeedClockVector"),
    "IFeedClockVectorElement": SimTypeBottom(label="IFeedClockVectorElement"),
    "IFeedEnclosure": SimTypeBottom(label="IFeedEnclosure"),
    "IFeedEvents": SimTypeBottom(label="IFeedEvents"),
    "IFeedFolder": SimTypeBottom(label="IFeedFolder"),
    "IFeedFolderEvents": SimTypeBottom(label="IFeedFolderEvents"),
    "IFeedItem": SimTypeBottom(label="IFeedItem"),
    "IFeedItem2": SimTypeBottom(label="IFeedItem2"),
    "IFeedsEnum": SimTypeBottom(label="IFeedsEnum"),
    "IFeedsManager": SimTypeBottom(label="IFeedsManager"),
    "IFhConfigMgr": SimTypeBottom(label="IFhConfigMgr"),
    "IFhReassociation": SimTypeBottom(label="IFhReassociation"),
    "IFhScopeIterator": SimTypeBottom(label="IFhScopeIterator"),
    "IFhTarget": SimTypeBottom(label="IFhTarget"),
    "IFileClient": SimTypeBottom(label="IFileClient"),
    "IFileDialog": SimTypeBottom(label="IFileDialog"),
    "IFileDialog2": SimTypeBottom(label="IFileDialog2"),
    "IFileDialogControlEvents": SimTypeBottom(label="IFileDialogControlEvents"),
    "IFileDialogCustomize": SimTypeBottom(label="IFileDialogCustomize"),
    "IFileDialogEvents": SimTypeBottom(label="IFileDialogEvents"),
    "IFileIo": SimTypeBottom(label="IFileIo"),
    "IFileIsInUse": SimTypeBottom(label="IFileIsInUse"),
    "IFileOpenDialog": SimTypeBottom(label="IFileOpenDialog"),
    "IFileOperation": SimTypeBottom(label="IFileOperation"),
    "IFileOperation2": SimTypeBottom(label="IFileOperation2"),
    "IFileOperationProgressSink": SimTypeBottom(label="IFileOperationProgressSink"),
    "IFileSaveDialog": SimTypeBottom(label="IFileSaveDialog"),
    "IFileSearchBand": SimTypeBottom(label="IFileSearchBand"),
    "IFileSinkFilter": SimTypeBottom(label="IFileSinkFilter"),
    "IFileSinkFilter2": SimTypeBottom(label="IFileSinkFilter2"),
    "IFileSourceFilter": SimTypeBottom(label="IFileSourceFilter"),
    "IFileSyncMergeHandler": SimTypeBottom(label="IFileSyncMergeHandler"),
    "IFileSystemBindData": SimTypeBottom(label="IFileSystemBindData"),
    "IFileSystemBindData2": SimTypeBottom(label="IFileSystemBindData2"),
    "IFileSystemImage": SimTypeBottom(label="IFileSystemImage"),
    "IFileSystemImage2": SimTypeBottom(label="IFileSystemImage2"),
    "IFileSystemImage3": SimTypeBottom(label="IFileSystemImage3"),
    "IFileSystemImageResult": SimTypeBottom(label="IFileSystemImageResult"),
    "IFileSystemImageResult2": SimTypeBottom(label="IFileSystemImageResult2"),
    "IFillLockBytes": SimTypeBottom(label="IFillLockBytes"),
    "IFilter": SimTypeBottom(label="IFilter"),
    "IFilterChain": SimTypeBottom(label="IFilterChain"),
    "IFilterGraph": SimTypeBottom(label="IFilterGraph"),
    "IFilterGraph2": SimTypeBottom(label="IFilterGraph2"),
    "IFilterGraph3": SimTypeBottom(label="IFilterGraph3"),
    "IFilterInfo": SimTypeBottom(label="IFilterInfo"),
    "IFilterKeyMap": SimTypeBottom(label="IFilterKeyMap"),
    "IFilterMapper": SimTypeBottom(label="IFilterMapper"),
    "IFilterMapper2": SimTypeBottom(label="IFilterMapper2"),
    "IFilterMapper3": SimTypeBottom(label="IFilterMapper3"),
    "IFilterRequestCallback": SimTypeBottom(label="IFilterRequestCallback"),
    "IFilterTrackingProvider": SimTypeBottom(label="IFilterTrackingProvider"),
    "IFilterTrackingRequestCallback": SimTypeBottom(label="IFilterTrackingRequestCallback"),
    "IFilterTrackingSyncChangeBuilder": SimTypeBottom(label="IFilterTrackingSyncChangeBuilder"),
    "IFindSimilarResults": SimTypeBottom(label="IFindSimilarResults"),
    "IFiringControl": SimTypeBottom(label="IFiringControl"),
    "IFixedDocument": SimTypeBottom(label="IFixedDocument"),
    "IFixedDocumentSequence": SimTypeBottom(label="IFixedDocumentSequence"),
    "IFixedPage": SimTypeBottom(label="IFixedPage"),
    "IFolderAction": SimTypeBottom(label="IFolderAction"),
    "IFolderActionCollection": SimTypeBottom(label="IFolderActionCollection"),
    "IFolderBandPriv": SimTypeBottom(label="IFolderBandPriv"),
    "IFolderFilter": SimTypeBottom(label="IFolderFilter"),
    "IFolderFilterSite": SimTypeBottom(label="IFolderFilterSite"),
    "IFolderView": SimTypeBottom(label="IFolderView"),
    "IFolderView2": SimTypeBottom(label="IFolderView2"),
    "IFolderViewHost": SimTypeBottom(label="IFolderViewHost"),
    "IFolderViewOC": SimTypeBottom(label="IFolderViewOC"),
    "IFolderViewOptions": SimTypeBottom(label="IFolderViewOptions"),
    "IFolderViewSettings": SimTypeBottom(label="IFolderViewSettings"),
    "IFont": SimTypeBottom(label="IFont"),
    "IFontDisp": SimTypeBottom(label="IFontDisp"),
    "IFontEventsDisp": SimTypeBottom(label="IFontEventsDisp"),
    "IFontNames": SimTypeBottom(label="IFontNames"),
    "IForegroundTransfer": SimTypeBottom(label="IForegroundTransfer"),
    "IForgottenKnowledge": SimTypeBottom(label="IForgottenKnowledge"),
    "IFrameworkInputPane": SimTypeBottom(label="IFrameworkInputPane"),
    "IFrameworkInputPaneHandler": SimTypeBottom(label="IFrameworkInputPaneHandler"),
    "IFrequencyMap": SimTypeBottom(label="IFrequencyMap"),
    "IFsiDirectoryItem": SimTypeBottom(label="IFsiDirectoryItem"),
    "IFsiDirectoryItem2": SimTypeBottom(label="IFsiDirectoryItem2"),
    "IFsiFileItem": SimTypeBottom(label="IFsiFileItem"),
    "IFsiFileItem2": SimTypeBottom(label="IFsiFileItem2"),
    "IFsiItem": SimTypeBottom(label="IFsiItem"),
    "IFsiNamedStreams": SimTypeBottom(label="IFsiNamedStreams"),
    "IFsrmAccessDeniedRemediationClient": SimTypeBottom(label="IFsrmAccessDeniedRemediationClient"),
    "IFsrmAction": SimTypeBottom(label="IFsrmAction"),
    "IFsrmActionCommand": SimTypeBottom(label="IFsrmActionCommand"),
    "IFsrmActionEmail": SimTypeBottom(label="IFsrmActionEmail"),
    "IFsrmActionEmail2": SimTypeBottom(label="IFsrmActionEmail2"),
    "IFsrmActionEventLog": SimTypeBottom(label="IFsrmActionEventLog"),
    "IFsrmActionReport": SimTypeBottom(label="IFsrmActionReport"),
    "IFsrmAutoApplyQuota": SimTypeBottom(label="IFsrmAutoApplyQuota"),
    "IFsrmClassificationManager": SimTypeBottom(label="IFsrmClassificationManager"),
    "IFsrmClassificationManager2": SimTypeBottom(label="IFsrmClassificationManager2"),
    "IFsrmClassificationRule": SimTypeBottom(label="IFsrmClassificationRule"),
    "IFsrmClassifierModuleDefinition": SimTypeBottom(label="IFsrmClassifierModuleDefinition"),
    "IFsrmClassifierModuleImplementation": SimTypeBottom(label="IFsrmClassifierModuleImplementation"),
    "IFsrmCollection": SimTypeBottom(label="IFsrmCollection"),
    "IFsrmCommittableCollection": SimTypeBottom(label="IFsrmCommittableCollection"),
    "IFsrmDerivedObjectsResult": SimTypeBottom(label="IFsrmDerivedObjectsResult"),
    "IFsrmExportImport": SimTypeBottom(label="IFsrmExportImport"),
    "IFsrmFileCondition": SimTypeBottom(label="IFsrmFileCondition"),
    "IFsrmFileConditionProperty": SimTypeBottom(label="IFsrmFileConditionProperty"),
    "IFsrmFileGroup": SimTypeBottom(label="IFsrmFileGroup"),
    "IFsrmFileGroupImported": SimTypeBottom(label="IFsrmFileGroupImported"),
    "IFsrmFileGroupManager": SimTypeBottom(label="IFsrmFileGroupManager"),
    "IFsrmFileManagementJob": SimTypeBottom(label="IFsrmFileManagementJob"),
    "IFsrmFileManagementJobManager": SimTypeBottom(label="IFsrmFileManagementJobManager"),
    "IFsrmFileScreen": SimTypeBottom(label="IFsrmFileScreen"),
    "IFsrmFileScreenBase": SimTypeBottom(label="IFsrmFileScreenBase"),
    "IFsrmFileScreenException": SimTypeBottom(label="IFsrmFileScreenException"),
    "IFsrmFileScreenManager": SimTypeBottom(label="IFsrmFileScreenManager"),
    "IFsrmFileScreenTemplate": SimTypeBottom(label="IFsrmFileScreenTemplate"),
    "IFsrmFileScreenTemplateImported": SimTypeBottom(label="IFsrmFileScreenTemplateImported"),
    "IFsrmFileScreenTemplateManager": SimTypeBottom(label="IFsrmFileScreenTemplateManager"),
    "IFsrmMutableCollection": SimTypeBottom(label="IFsrmMutableCollection"),
    "IFsrmObject": SimTypeBottom(label="IFsrmObject"),
    "IFsrmPathMapper": SimTypeBottom(label="IFsrmPathMapper"),
    "IFsrmPipelineModuleConnector": SimTypeBottom(label="IFsrmPipelineModuleConnector"),
    "IFsrmPipelineModuleDefinition": SimTypeBottom(label="IFsrmPipelineModuleDefinition"),
    "IFsrmPipelineModuleImplementation": SimTypeBottom(label="IFsrmPipelineModuleImplementation"),
    "IFsrmProperty": SimTypeBottom(label="IFsrmProperty"),
    "IFsrmPropertyBag": SimTypeBottom(label="IFsrmPropertyBag"),
    "IFsrmPropertyBag2": SimTypeBottom(label="IFsrmPropertyBag2"),
    "IFsrmPropertyCondition": SimTypeBottom(label="IFsrmPropertyCondition"),
    "IFsrmPropertyDefinition": SimTypeBottom(label="IFsrmPropertyDefinition"),
    "IFsrmPropertyDefinition2": SimTypeBottom(label="IFsrmPropertyDefinition2"),
    "IFsrmPropertyDefinitionValue": SimTypeBottom(label="IFsrmPropertyDefinitionValue"),
    "IFsrmQuota": SimTypeBottom(label="IFsrmQuota"),
    "IFsrmQuotaBase": SimTypeBottom(label="IFsrmQuotaBase"),
    "IFsrmQuotaManager": SimTypeBottom(label="IFsrmQuotaManager"),
    "IFsrmQuotaManagerEx": SimTypeBottom(label="IFsrmQuotaManagerEx"),
    "IFsrmQuotaObject": SimTypeBottom(label="IFsrmQuotaObject"),
    "IFsrmQuotaTemplate": SimTypeBottom(label="IFsrmQuotaTemplate"),
    "IFsrmQuotaTemplateImported": SimTypeBottom(label="IFsrmQuotaTemplateImported"),
    "IFsrmQuotaTemplateManager": SimTypeBottom(label="IFsrmQuotaTemplateManager"),
    "IFsrmReport": SimTypeBottom(label="IFsrmReport"),
    "IFsrmReportJob": SimTypeBottom(label="IFsrmReportJob"),
    "IFsrmReportManager": SimTypeBottom(label="IFsrmReportManager"),
    "IFsrmReportScheduler": SimTypeBottom(label="IFsrmReportScheduler"),
    "IFsrmRule": SimTypeBottom(label="IFsrmRule"),
    "IFsrmSetting": SimTypeBottom(label="IFsrmSetting"),
    "IFsrmStorageModuleDefinition": SimTypeBottom(label="IFsrmStorageModuleDefinition"),
    "IFsrmStorageModuleImplementation": SimTypeBottom(label="IFsrmStorageModuleImplementation"),
    "IFtpAuthenticationProvider": SimTypeBottom(label="IFtpAuthenticationProvider"),
    "IFtpAuthorizationProvider": SimTypeBottom(label="IFtpAuthorizationProvider"),
    "IFtpHomeDirectoryProvider": SimTypeBottom(label="IFtpHomeDirectoryProvider"),
    "IFtpLogProvider": SimTypeBottom(label="IFtpLogProvider"),
    "IFtpPostprocessProvider": SimTypeBottom(label="IFtpPostprocessProvider"),
    "IFtpPreprocessProvider": SimTypeBottom(label="IFtpPreprocessProvider"),
    "IFtpProviderConstruct": SimTypeBottom(label="IFtpProviderConstruct"),
    "IFtpRoleProvider": SimTypeBottom(label="IFtpRoleProvider"),
    "IFullScreenVideo": SimTypeBottom(label="IFullScreenVideo"),
    "IFullScreenVideoEx": SimTypeBottom(label="IFullScreenVideoEx"),
    "IFunctionDiscovery": SimTypeBottom(label="IFunctionDiscovery"),
    "IFunctionDiscoveryNotification": SimTypeBottom(label="IFunctionDiscoveryNotification"),
    "IFunctionDiscoveryProvider": SimTypeBottom(label="IFunctionDiscoveryProvider"),
    "IFunctionDiscoveryProviderFactory": SimTypeBottom(label="IFunctionDiscoveryProviderFactory"),
    "IFunctionDiscoveryProviderQuery": SimTypeBottom(label="IFunctionDiscoveryProviderQuery"),
    "IFunctionDiscoveryServiceProvider": SimTypeBottom(label="IFunctionDiscoveryServiceProvider"),
    "IFunctionInstance": SimTypeBottom(label="IFunctionInstance"),
    "IFunctionInstanceCollection": SimTypeBottom(label="IFunctionInstanceCollection"),
    "IFunctionInstanceCollectionQuery": SimTypeBottom(label="IFunctionInstanceCollectionQuery"),
    "IFunctionInstanceQuery": SimTypeBottom(label="IFunctionInstanceQuery"),
    "IGCHost": SimTypeBottom(label="IGCHost"),
    "IGCHost2": SimTypeBottom(label="IGCHost2"),
    "IGCHostControl": SimTypeBottom(label="IGCHostControl"),
    "IGCThreadControl": SimTypeBottom(label="IGCThreadControl"),
    "IGMPV3_QUERY_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"MaxRespCode": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Checksum", SimTypeShort(signed=False, label="UInt16")), ("MulticastAddress", SimTypeRef("IN_ADDR", SimStruct)), ("_bitfield", SimTypeChar(label="Byte")), ("Anonymous2", SimUnion({"QueriersQueryInterfaceCode": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("SourceCount", SimTypeShort(signed=False, label="UInt16")),)), name="IGMPV3_QUERY_HEADER", pack=False, align=None),
    "IGMPV3_REPORT_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Checksum", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("RecordCount", SimTypeShort(signed=False, label="UInt16")),)), name="IGMPV3_REPORT_HEADER", pack=False, align=None),
    "IGMPV3_REPORT_RECORD_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("AuxillaryDataLength", SimTypeChar(label="Byte")), ("SourceCount", SimTypeShort(signed=False, label="UInt16")), ("MulticastAddress", SimTypeRef("IN_ADDR", SimStruct)),)), name="IGMPV3_REPORT_RECORD_HEADER", pack=False, align=None),
    "IGMP_HEADER": SimStruct(OrderedDict((("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "VersionType": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Reserved": SimTypeChar(label="Byte"), "MaxRespTime": SimTypeChar(label="Byte"), "Code": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Checksum", SimTypeShort(signed=False, label="UInt16")), ("MulticastAddress", SimTypeRef("IN_ADDR", SimStruct)),)), name="IGMP_HEADER", pack=False, align=None),
    "IGMP_MAX_RESP_CODE_TYPE": SimTypeInt(signed=False, label="IGMP_MAX_RESP_CODE_TYPE"),
    "IGNOREMIME": SimTypeInt(signed=False, label="IGNOREMIME"),
    "IGPEInformation": SimTypeBottom(label="IGPEInformation"),
    "IGPM": SimTypeBottom(label="IGPM"),
    "IGPM2": SimTypeBottom(label="IGPM2"),
    "IGPMAsyncCancel": SimTypeBottom(label="IGPMAsyncCancel"),
    "IGPMAsyncProgress": SimTypeBottom(label="IGPMAsyncProgress"),
    "IGPMBackup": SimTypeBottom(label="IGPMBackup"),
    "IGPMBackupCollection": SimTypeBottom(label="IGPMBackupCollection"),
    "IGPMBackupDir": SimTypeBottom(label="IGPMBackupDir"),
    "IGPMBackupDirEx": SimTypeBottom(label="IGPMBackupDirEx"),
    "IGPMCSECollection": SimTypeBottom(label="IGPMCSECollection"),
    "IGPMClientSideExtension": SimTypeBottom(label="IGPMClientSideExtension"),
    "IGPMConstants": SimTypeBottom(label="IGPMConstants"),
    "IGPMConstants2": SimTypeBottom(label="IGPMConstants2"),
    "IGPMDomain": SimTypeBottom(label="IGPMDomain"),
    "IGPMDomain2": SimTypeBottom(label="IGPMDomain2"),
    "IGPMDomain3": SimTypeBottom(label="IGPMDomain3"),
    "IGPMGPO": SimTypeBottom(label="IGPMGPO"),
    "IGPMGPO2": SimTypeBottom(label="IGPMGPO2"),
    "IGPMGPO3": SimTypeBottom(label="IGPMGPO3"),
    "IGPMGPOCollection": SimTypeBottom(label="IGPMGPOCollection"),
    "IGPMGPOLink": SimTypeBottom(label="IGPMGPOLink"),
    "IGPMGPOLinksCollection": SimTypeBottom(label="IGPMGPOLinksCollection"),
    "IGPMMapEntry": SimTypeBottom(label="IGPMMapEntry"),
    "IGPMMapEntryCollection": SimTypeBottom(label="IGPMMapEntryCollection"),
    "IGPMMigrationTable": SimTypeBottom(label="IGPMMigrationTable"),
    "IGPMPermission": SimTypeBottom(label="IGPMPermission"),
    "IGPMRSOP": SimTypeBottom(label="IGPMRSOP"),
    "IGPMResult": SimTypeBottom(label="IGPMResult"),
    "IGPMSOM": SimTypeBottom(label="IGPMSOM"),
    "IGPMSOMCollection": SimTypeBottom(label="IGPMSOMCollection"),
    "IGPMSearchCriteria": SimTypeBottom(label="IGPMSearchCriteria"),
    "IGPMSecurityInfo": SimTypeBottom(label="IGPMSecurityInfo"),
    "IGPMSitesContainer": SimTypeBottom(label="IGPMSitesContainer"),
    "IGPMStarterGPO": SimTypeBottom(label="IGPMStarterGPO"),
    "IGPMStarterGPOBackup": SimTypeBottom(label="IGPMStarterGPOBackup"),
    "IGPMStarterGPOBackupCollection": SimTypeBottom(label="IGPMStarterGPOBackupCollection"),
    "IGPMStarterGPOCollection": SimTypeBottom(label="IGPMStarterGPOCollection"),
    "IGPMStatusMessage": SimTypeBottom(label="IGPMStatusMessage"),
    "IGPMStatusMsgCollection": SimTypeBottom(label="IGPMStatusMsgCollection"),
    "IGPMTrustee": SimTypeBottom(label="IGPMTrustee"),
    "IGPMWMIFilter": SimTypeBottom(label="IGPMWMIFilter"),
    "IGPMWMIFilterCollection": SimTypeBottom(label="IGPMWMIFilterCollection"),
    "IGP_ID": SimTypeInt(signed=False, label="IGP_ID"),
    "IGameExplorer": SimTypeBottom(label="IGameExplorer"),
    "IGameExplorer2": SimTypeBottom(label="IGameExplorer2"),
    "IGameStatistics": SimTypeBottom(label="IGameStatistics"),
    "IGameStatisticsMgr": SimTypeBottom(label="IGameStatisticsMgr"),
    "IGamutMapModelPlugIn": SimTypeBottom(label="IGamutMapModelPlugIn"),
    "IGenericDescriptor": SimTypeBottom(label="IGenericDescriptor"),
    "IGenericDescriptor2": SimTypeBottom(label="IGenericDescriptor2"),
    "IGestureRecognizer": SimTypeBottom(label="IGestureRecognizer"),
    "IGetAppTrackerData": SimTypeBottom(label="IGetAppTrackerData"),
    "IGetBindHandle": SimTypeBottom(label="IGetBindHandle"),
    "IGetCapabilitiesKey": SimTypeBottom(label="IGetCapabilitiesKey"),
    "IGetClusterDataInfo": SimTypeBottom(label="IGetClusterDataInfo"),
    "IGetClusterGroupInfo": SimTypeBottom(label="IGetClusterGroupInfo"),
    "IGetClusterNetInterfaceInfo": SimTypeBottom(label="IGetClusterNetInterfaceInfo"),
    "IGetClusterNetworkInfo": SimTypeBottom(label="IGetClusterNetworkInfo"),
    "IGetClusterNodeInfo": SimTypeBottom(label="IGetClusterNodeInfo"),
    "IGetClusterObjectInfo": SimTypeBottom(label="IGetClusterObjectInfo"),
    "IGetClusterResourceInfo": SimTypeBottom(label="IGetClusterResourceInfo"),
    "IGetClusterUIInfo": SimTypeBottom(label="IGetClusterUIInfo"),
    "IGetContextProperties": SimTypeBottom(label="IGetContextProperties"),
    "IGetDataSource": SimTypeBottom(label="IGetDataSource"),
    "IGetDispenser": SimTypeBottom(label="IGetDispenser"),
    "IGetFrame": SimTypeBottom(label="IGetFrame"),
    "IGetOleObject": SimTypeBottom(label="IGetOleObject"),
    "IGetRow": SimTypeBottom(label="IGetRow"),
    "IGetSVGDocument": SimTypeBottom(label="IGetSVGDocument"),
    "IGetSecurityCallContext": SimTypeBottom(label="IGetSecurityCallContext"),
    "IGetServiceIds": SimTypeBottom(label="IGetServiceIds"),
    "IGetSession": SimTypeBottom(label="IGetSession"),
    "IGetSourceRow": SimTypeBottom(label="IGetSourceRow"),
    "IGetVBAObject": SimTypeBottom(label="IGetVBAObject"),
    "IGlobalInterfaceTable": SimTypeBottom(label="IGlobalInterfaceTable"),
    "IGlobalOptions": SimTypeBottom(label="IGlobalOptions"),
    "IGpnvsCommonBase": SimTypeBottom(label="IGpnvsCommonBase"),
    "IGraphBuilder": SimTypeBottom(label="IGraphBuilder"),
    "IGraphConfig": SimTypeBottom(label="IGraphConfig"),
    "IGraphConfigCallback": SimTypeBottom(label="IGraphConfigCallback"),
    "IGraphVersion": SimTypeBottom(label="IGraphVersion"),
    "IGridItemProvider": SimTypeBottom(label="IGridItemProvider"),
    "IGridProvider": SimTypeBottom(label="IGridProvider"),
    "IGroupPolicyObject": SimTypeBottom(label="IGroupPolicyObject"),
    "IGuideData": SimTypeBottom(label="IGuideData"),
    "IGuideDataEvent": SimTypeBottom(label="IGuideDataEvent"),
    "IGuideDataLoader": SimTypeBottom(label="IGuideDataLoader"),
    "IGuideDataProperty": SimTypeBottom(label="IGuideDataProperty"),
    "IHTCAttachBehavior": SimTypeBottom(label="IHTCAttachBehavior"),
    "IHTCAttachBehavior2": SimTypeBottom(label="IHTCAttachBehavior2"),
    "IHTCDefaultDispatch": SimTypeBottom(label="IHTCDefaultDispatch"),
    "IHTCDescBehavior": SimTypeBottom(label="IHTCDescBehavior"),
    "IHTCEventBehavior": SimTypeBottom(label="IHTCEventBehavior"),
    "IHTCMethodBehavior": SimTypeBottom(label="IHTCMethodBehavior"),
    "IHTCPropertyBehavior": SimTypeBottom(label="IHTCPropertyBehavior"),
    "IHTMLAnchorElement": SimTypeBottom(label="IHTMLAnchorElement"),
    "IHTMLAnchorElement2": SimTypeBottom(label="IHTMLAnchorElement2"),
    "IHTMLAnchorElement3": SimTypeBottom(label="IHTMLAnchorElement3"),
    "IHTMLAppBehavior": SimTypeBottom(label="IHTMLAppBehavior"),
    "IHTMLAppBehavior2": SimTypeBottom(label="IHTMLAppBehavior2"),
    "IHTMLAppBehavior3": SimTypeBottom(label="IHTMLAppBehavior3"),
    "IHTMLApplicationCache": SimTypeBottom(label="IHTMLApplicationCache"),
    "IHTMLAreaElement": SimTypeBottom(label="IHTMLAreaElement"),
    "IHTMLAreaElement2": SimTypeBottom(label="IHTMLAreaElement2"),
    "IHTMLAreasCollection": SimTypeBottom(label="IHTMLAreasCollection"),
    "IHTMLAreasCollection2": SimTypeBottom(label="IHTMLAreasCollection2"),
    "IHTMLAreasCollection3": SimTypeBottom(label="IHTMLAreasCollection3"),
    "IHTMLAreasCollection4": SimTypeBottom(label="IHTMLAreasCollection4"),
    "IHTMLAttributeCollection": SimTypeBottom(label="IHTMLAttributeCollection"),
    "IHTMLAttributeCollection2": SimTypeBottom(label="IHTMLAttributeCollection2"),
    "IHTMLAttributeCollection3": SimTypeBottom(label="IHTMLAttributeCollection3"),
    "IHTMLAttributeCollection4": SimTypeBottom(label="IHTMLAttributeCollection4"),
    "IHTMLAudioElement": SimTypeBottom(label="IHTMLAudioElement"),
    "IHTMLAudioElementFactory": SimTypeBottom(label="IHTMLAudioElementFactory"),
    "IHTMLBGsound": SimTypeBottom(label="IHTMLBGsound"),
    "IHTMLBRElement": SimTypeBottom(label="IHTMLBRElement"),
    "IHTMLBaseElement": SimTypeBottom(label="IHTMLBaseElement"),
    "IHTMLBaseElement2": SimTypeBottom(label="IHTMLBaseElement2"),
    "IHTMLBaseFontElement": SimTypeBottom(label="IHTMLBaseFontElement"),
    "IHTMLBlockElement": SimTypeBottom(label="IHTMLBlockElement"),
    "IHTMLBlockElement2": SimTypeBottom(label="IHTMLBlockElement2"),
    "IHTMLBlockElement3": SimTypeBottom(label="IHTMLBlockElement3"),
    "IHTMLBodyElement": SimTypeBottom(label="IHTMLBodyElement"),
    "IHTMLBodyElement2": SimTypeBottom(label="IHTMLBodyElement2"),
    "IHTMLBodyElement3": SimTypeBottom(label="IHTMLBodyElement3"),
    "IHTMLBodyElement4": SimTypeBottom(label="IHTMLBodyElement4"),
    "IHTMLBodyElement5": SimTypeBottom(label="IHTMLBodyElement5"),
    "IHTMLBookmarkCollection": SimTypeBottom(label="IHTMLBookmarkCollection"),
    "IHTMLButtonElement": SimTypeBottom(label="IHTMLButtonElement"),
    "IHTMLButtonElement2": SimTypeBottom(label="IHTMLButtonElement2"),
    "IHTMLCSSImportRule": SimTypeBottom(label="IHTMLCSSImportRule"),
    "IHTMLCSSMediaList": SimTypeBottom(label="IHTMLCSSMediaList"),
    "IHTMLCSSMediaRule": SimTypeBottom(label="IHTMLCSSMediaRule"),
    "IHTMLCSSNamespaceRule": SimTypeBottom(label="IHTMLCSSNamespaceRule"),
    "IHTMLCSSRule": SimTypeBottom(label="IHTMLCSSRule"),
    "IHTMLCSSStyleDeclaration": SimTypeBottom(label="IHTMLCSSStyleDeclaration"),
    "IHTMLCSSStyleDeclaration2": SimTypeBottom(label="IHTMLCSSStyleDeclaration2"),
    "IHTMLCSSStyleDeclaration3": SimTypeBottom(label="IHTMLCSSStyleDeclaration3"),
    "IHTMLCSSStyleDeclaration4": SimTypeBottom(label="IHTMLCSSStyleDeclaration4"),
    "IHTMLCanvasElement": SimTypeBottom(label="IHTMLCanvasElement"),
    "IHTMLCaret": SimTypeBottom(label="IHTMLCaret"),
    "IHTMLChangeLog": SimTypeBottom(label="IHTMLChangeLog"),
    "IHTMLChangePlayback": SimTypeBottom(label="IHTMLChangePlayback"),
    "IHTMLChangeSink": SimTypeBottom(label="IHTMLChangeSink"),
    "IHTMLCommentElement": SimTypeBottom(label="IHTMLCommentElement"),
    "IHTMLCommentElement2": SimTypeBottom(label="IHTMLCommentElement2"),
    "IHTMLCommentElement3": SimTypeBottom(label="IHTMLCommentElement3"),
    "IHTMLComputedStyle": SimTypeBottom(label="IHTMLComputedStyle"),
    "IHTMLControlElement": SimTypeBottom(label="IHTMLControlElement"),
    "IHTMLControlRange": SimTypeBottom(label="IHTMLControlRange"),
    "IHTMLControlRange2": SimTypeBottom(label="IHTMLControlRange2"),
    "IHTMLCurrentStyle": SimTypeBottom(label="IHTMLCurrentStyle"),
    "IHTMLCurrentStyle2": SimTypeBottom(label="IHTMLCurrentStyle2"),
    "IHTMLCurrentStyle3": SimTypeBottom(label="IHTMLCurrentStyle3"),
    "IHTMLCurrentStyle4": SimTypeBottom(label="IHTMLCurrentStyle4"),
    "IHTMLCurrentStyle5": SimTypeBottom(label="IHTMLCurrentStyle5"),
    "IHTMLDDElement": SimTypeBottom(label="IHTMLDDElement"),
    "IHTMLDListElement": SimTypeBottom(label="IHTMLDListElement"),
    "IHTMLDOMAttribute": SimTypeBottom(label="IHTMLDOMAttribute"),
    "IHTMLDOMAttribute2": SimTypeBottom(label="IHTMLDOMAttribute2"),
    "IHTMLDOMAttribute3": SimTypeBottom(label="IHTMLDOMAttribute3"),
    "IHTMLDOMAttribute4": SimTypeBottom(label="IHTMLDOMAttribute4"),
    "IHTMLDOMChildrenCollection": SimTypeBottom(label="IHTMLDOMChildrenCollection"),
    "IHTMLDOMChildrenCollection2": SimTypeBottom(label="IHTMLDOMChildrenCollection2"),
    "IHTMLDOMConstructor": SimTypeBottom(label="IHTMLDOMConstructor"),
    "IHTMLDOMConstructorCollection": SimTypeBottom(label="IHTMLDOMConstructorCollection"),
    "IHTMLDOMImplementation": SimTypeBottom(label="IHTMLDOMImplementation"),
    "IHTMLDOMImplementation2": SimTypeBottom(label="IHTMLDOMImplementation2"),
    "IHTMLDOMNode": SimTypeBottom(label="IHTMLDOMNode"),
    "IHTMLDOMNode2": SimTypeBottom(label="IHTMLDOMNode2"),
    "IHTMLDOMNode3": SimTypeBottom(label="IHTMLDOMNode3"),
    "IHTMLDOMRange": SimTypeBottom(label="IHTMLDOMRange"),
    "IHTMLDOMTextNode": SimTypeBottom(label="IHTMLDOMTextNode"),
    "IHTMLDOMTextNode2": SimTypeBottom(label="IHTMLDOMTextNode2"),
    "IHTMLDOMTextNode3": SimTypeBottom(label="IHTMLDOMTextNode3"),
    "IHTMLDTElement": SimTypeBottom(label="IHTMLDTElement"),
    "IHTMLDataTransfer": SimTypeBottom(label="IHTMLDataTransfer"),
    "IHTMLDatabinding": SimTypeBottom(label="IHTMLDatabinding"),
    "IHTMLDialog": SimTypeBottom(label="IHTMLDialog"),
    "IHTMLDialog2": SimTypeBottom(label="IHTMLDialog2"),
    "IHTMLDialog3": SimTypeBottom(label="IHTMLDialog3"),
    "IHTMLDivElement": SimTypeBottom(label="IHTMLDivElement"),
    "IHTMLDivPosition": SimTypeBottom(label="IHTMLDivPosition"),
    "IHTMLDocument": SimTypeBottom(label="IHTMLDocument"),
    "IHTMLDocument2": SimTypeBottom(label="IHTMLDocument2"),
    "IHTMLDocument3": SimTypeBottom(label="IHTMLDocument3"),
    "IHTMLDocument4": SimTypeBottom(label="IHTMLDocument4"),
    "IHTMLDocument5": SimTypeBottom(label="IHTMLDocument5"),
    "IHTMLDocument6": SimTypeBottom(label="IHTMLDocument6"),
    "IHTMLDocument7": SimTypeBottom(label="IHTMLDocument7"),
    "IHTMLDocument8": SimTypeBottom(label="IHTMLDocument8"),
    "IHTMLDocumentCompatibleInfo": SimTypeBottom(label="IHTMLDocumentCompatibleInfo"),
    "IHTMLDocumentCompatibleInfoCollection": SimTypeBottom(label="IHTMLDocumentCompatibleInfoCollection"),
    "IHTMLEditDesigner": SimTypeBottom(label="IHTMLEditDesigner"),
    "IHTMLEditHost": SimTypeBottom(label="IHTMLEditHost"),
    "IHTMLEditHost2": SimTypeBottom(label="IHTMLEditHost2"),
    "IHTMLEditServices": SimTypeBottom(label="IHTMLEditServices"),
    "IHTMLEditServices2": SimTypeBottom(label="IHTMLEditServices2"),
    "IHTMLElement": SimTypeBottom(label="IHTMLElement"),
    "IHTMLElement2": SimTypeBottom(label="IHTMLElement2"),
    "IHTMLElement3": SimTypeBottom(label="IHTMLElement3"),
    "IHTMLElement4": SimTypeBottom(label="IHTMLElement4"),
    "IHTMLElement5": SimTypeBottom(label="IHTMLElement5"),
    "IHTMLElement6": SimTypeBottom(label="IHTMLElement6"),
    "IHTMLElement7": SimTypeBottom(label="IHTMLElement7"),
    "IHTMLElementAppliedStyles": SimTypeBottom(label="IHTMLElementAppliedStyles"),
    "IHTMLElementCollection": SimTypeBottom(label="IHTMLElementCollection"),
    "IHTMLElementCollection2": SimTypeBottom(label="IHTMLElementCollection2"),
    "IHTMLElementCollection3": SimTypeBottom(label="IHTMLElementCollection3"),
    "IHTMLElementCollection4": SimTypeBottom(label="IHTMLElementCollection4"),
    "IHTMLElementDefaults": SimTypeBottom(label="IHTMLElementDefaults"),
    "IHTMLElementRender": SimTypeBottom(label="IHTMLElementRender"),
    "IHTMLEmbedElement": SimTypeBottom(label="IHTMLEmbedElement"),
    "IHTMLEmbedElement2": SimTypeBottom(label="IHTMLEmbedElement2"),
    "IHTMLEventObj": SimTypeBottom(label="IHTMLEventObj"),
    "IHTMLEventObj2": SimTypeBottom(label="IHTMLEventObj2"),
    "IHTMLEventObj3": SimTypeBottom(label="IHTMLEventObj3"),
    "IHTMLEventObj4": SimTypeBottom(label="IHTMLEventObj4"),
    "IHTMLEventObj5": SimTypeBottom(label="IHTMLEventObj5"),
    "IHTMLEventObj6": SimTypeBottom(label="IHTMLEventObj6"),
    "IHTMLFieldSetElement": SimTypeBottom(label="IHTMLFieldSetElement"),
    "IHTMLFieldSetElement2": SimTypeBottom(label="IHTMLFieldSetElement2"),
    "IHTMLFiltersCollection": SimTypeBottom(label="IHTMLFiltersCollection"),
    "IHTMLFontElement": SimTypeBottom(label="IHTMLFontElement"),
    "IHTMLFontNamesCollection": SimTypeBottom(label="IHTMLFontNamesCollection"),
    "IHTMLFontSizesCollection": SimTypeBottom(label="IHTMLFontSizesCollection"),
    "IHTMLFormElement": SimTypeBottom(label="IHTMLFormElement"),
    "IHTMLFormElement2": SimTypeBottom(label="IHTMLFormElement2"),
    "IHTMLFormElement3": SimTypeBottom(label="IHTMLFormElement3"),
    "IHTMLFormElement4": SimTypeBottom(label="IHTMLFormElement4"),
    "IHTMLFrameBase": SimTypeBottom(label="IHTMLFrameBase"),
    "IHTMLFrameBase2": SimTypeBottom(label="IHTMLFrameBase2"),
    "IHTMLFrameBase3": SimTypeBottom(label="IHTMLFrameBase3"),
    "IHTMLFrameElement": SimTypeBottom(label="IHTMLFrameElement"),
    "IHTMLFrameElement2": SimTypeBottom(label="IHTMLFrameElement2"),
    "IHTMLFrameElement3": SimTypeBottom(label="IHTMLFrameElement3"),
    "IHTMLFrameSetElement": SimTypeBottom(label="IHTMLFrameSetElement"),
    "IHTMLFrameSetElement2": SimTypeBottom(label="IHTMLFrameSetElement2"),
    "IHTMLFrameSetElement3": SimTypeBottom(label="IHTMLFrameSetElement3"),
    "IHTMLFramesCollection2": SimTypeBottom(label="IHTMLFramesCollection2"),
    "IHTMLGenericElement": SimTypeBottom(label="IHTMLGenericElement"),
    "IHTMLHRElement": SimTypeBottom(label="IHTMLHRElement"),
    "IHTMLHeadElement": SimTypeBottom(label="IHTMLHeadElement"),
    "IHTMLHeadElement2": SimTypeBottom(label="IHTMLHeadElement2"),
    "IHTMLHeaderElement": SimTypeBottom(label="IHTMLHeaderElement"),
    "IHTMLHtmlElement": SimTypeBottom(label="IHTMLHtmlElement"),
    "IHTMLIFrameElement": SimTypeBottom(label="IHTMLIFrameElement"),
    "IHTMLIFrameElement2": SimTypeBottom(label="IHTMLIFrameElement2"),
    "IHTMLIFrameElement3": SimTypeBottom(label="IHTMLIFrameElement3"),
    "IHTMLIPrintCollection": SimTypeBottom(label="IHTMLIPrintCollection"),
    "IHTMLImageElementFactory": SimTypeBottom(label="IHTMLImageElementFactory"),
    "IHTMLImgElement": SimTypeBottom(label="IHTMLImgElement"),
    "IHTMLImgElement2": SimTypeBottom(label="IHTMLImgElement2"),
    "IHTMLImgElement3": SimTypeBottom(label="IHTMLImgElement3"),
    "IHTMLImgElement4": SimTypeBottom(label="IHTMLImgElement4"),
    "IHTMLInputButtonElement": SimTypeBottom(label="IHTMLInputButtonElement"),
    "IHTMLInputElement": SimTypeBottom(label="IHTMLInputElement"),
    "IHTMLInputElement2": SimTypeBottom(label="IHTMLInputElement2"),
    "IHTMLInputElement3": SimTypeBottom(label="IHTMLInputElement3"),
    "IHTMLInputFileElement": SimTypeBottom(label="IHTMLInputFileElement"),
    "IHTMLInputHiddenElement": SimTypeBottom(label="IHTMLInputHiddenElement"),
    "IHTMLInputImage": SimTypeBottom(label="IHTMLInputImage"),
    "IHTMLInputRangeElement": SimTypeBottom(label="IHTMLInputRangeElement"),
    "IHTMLInputTextElement": SimTypeBottom(label="IHTMLInputTextElement"),
    "IHTMLInputTextElement2": SimTypeBottom(label="IHTMLInputTextElement2"),
    "IHTMLIsIndexElement": SimTypeBottom(label="IHTMLIsIndexElement"),
    "IHTMLIsIndexElement2": SimTypeBottom(label="IHTMLIsIndexElement2"),
    "IHTMLLIElement": SimTypeBottom(label="IHTMLLIElement"),
    "IHTMLLabelElement": SimTypeBottom(label="IHTMLLabelElement"),
    "IHTMLLabelElement2": SimTypeBottom(label="IHTMLLabelElement2"),
    "IHTMLLegendElement": SimTypeBottom(label="IHTMLLegendElement"),
    "IHTMLLegendElement2": SimTypeBottom(label="IHTMLLegendElement2"),
    "IHTMLLinkElement": SimTypeBottom(label="IHTMLLinkElement"),
    "IHTMLLinkElement2": SimTypeBottom(label="IHTMLLinkElement2"),
    "IHTMLLinkElement3": SimTypeBottom(label="IHTMLLinkElement3"),
    "IHTMLLinkElement4": SimTypeBottom(label="IHTMLLinkElement4"),
    "IHTMLLinkElement5": SimTypeBottom(label="IHTMLLinkElement5"),
    "IHTMLListElement": SimTypeBottom(label="IHTMLListElement"),
    "IHTMLListElement2": SimTypeBottom(label="IHTMLListElement2"),
    "IHTMLLocation": SimTypeBottom(label="IHTMLLocation"),
    "IHTMLMSCSSKeyframeRule": SimTypeBottom(label="IHTMLMSCSSKeyframeRule"),
    "IHTMLMSCSSKeyframesRule": SimTypeBottom(label="IHTMLMSCSSKeyframesRule"),
    "IHTMLMSImgElement": SimTypeBottom(label="IHTMLMSImgElement"),
    "IHTMLMSMediaElement": SimTypeBottom(label="IHTMLMSMediaElement"),
    "IHTMLMapElement": SimTypeBottom(label="IHTMLMapElement"),
    "IHTMLMarqueeElement": SimTypeBottom(label="IHTMLMarqueeElement"),
    "IHTMLMediaElement": SimTypeBottom(label="IHTMLMediaElement"),
    "IHTMLMediaElement2": SimTypeBottom(label="IHTMLMediaElement2"),
    "IHTMLMediaError": SimTypeBottom(label="IHTMLMediaError"),
    "IHTMLMetaElement": SimTypeBottom(label="IHTMLMetaElement"),
    "IHTMLMetaElement2": SimTypeBottom(label="IHTMLMetaElement2"),
    "IHTMLMetaElement3": SimTypeBottom(label="IHTMLMetaElement3"),
    "IHTMLMimeTypesCollection": SimTypeBottom(label="IHTMLMimeTypesCollection"),
    "IHTMLModelessInit": SimTypeBottom(label="IHTMLModelessInit"),
    "IHTMLNamespace": SimTypeBottom(label="IHTMLNamespace"),
    "IHTMLNamespaceCollection": SimTypeBottom(label="IHTMLNamespaceCollection"),
    "IHTMLNextIdElement": SimTypeBottom(label="IHTMLNextIdElement"),
    "IHTMLNoShowElement": SimTypeBottom(label="IHTMLNoShowElement"),
    "IHTMLOListElement": SimTypeBottom(label="IHTMLOListElement"),
    "IHTMLOMWindowServices": SimTypeBottom(label="IHTMLOMWindowServices"),
    "IHTMLObjectElement": SimTypeBottom(label="IHTMLObjectElement"),
    "IHTMLObjectElement2": SimTypeBottom(label="IHTMLObjectElement2"),
    "IHTMLObjectElement3": SimTypeBottom(label="IHTMLObjectElement3"),
    "IHTMLObjectElement4": SimTypeBottom(label="IHTMLObjectElement4"),
    "IHTMLObjectElement5": SimTypeBottom(label="IHTMLObjectElement5"),
    "IHTMLOpsProfile": SimTypeBottom(label="IHTMLOpsProfile"),
    "IHTMLOptionButtonElement": SimTypeBottom(label="IHTMLOptionButtonElement"),
    "IHTMLOptionElement": SimTypeBottom(label="IHTMLOptionElement"),
    "IHTMLOptionElement3": SimTypeBottom(label="IHTMLOptionElement3"),
    "IHTMLOptionElement4": SimTypeBottom(label="IHTMLOptionElement4"),
    "IHTMLOptionElementFactory": SimTypeBottom(label="IHTMLOptionElementFactory"),
    "IHTMLOptionsHolder": SimTypeBottom(label="IHTMLOptionsHolder"),
    "IHTMLPaintSite": SimTypeBottom(label="IHTMLPaintSite"),
    "IHTMLPainter": SimTypeBottom(label="IHTMLPainter"),
    "IHTMLPainterEventInfo": SimTypeBottom(label="IHTMLPainterEventInfo"),
    "IHTMLPainterOverlay": SimTypeBottom(label="IHTMLPainterOverlay"),
    "IHTMLParaElement": SimTypeBottom(label="IHTMLParaElement"),
    "IHTMLParamElement": SimTypeBottom(label="IHTMLParamElement"),
    "IHTMLParamElement2": SimTypeBottom(label="IHTMLParamElement2"),
    "IHTMLPerformance": SimTypeBottom(label="IHTMLPerformance"),
    "IHTMLPerformanceNavigation": SimTypeBottom(label="IHTMLPerformanceNavigation"),
    "IHTMLPerformanceTiming": SimTypeBottom(label="IHTMLPerformanceTiming"),
    "IHTMLPersistData": SimTypeBottom(label="IHTMLPersistData"),
    "IHTMLPersistDataOM": SimTypeBottom(label="IHTMLPersistDataOM"),
    "IHTMLPhraseElement": SimTypeBottom(label="IHTMLPhraseElement"),
    "IHTMLPhraseElement2": SimTypeBottom(label="IHTMLPhraseElement2"),
    "IHTMLPhraseElement3": SimTypeBottom(label="IHTMLPhraseElement3"),
    "IHTMLPluginsCollection": SimTypeBottom(label="IHTMLPluginsCollection"),
    "IHTMLPopup": SimTypeBottom(label="IHTMLPopup"),
    "IHTMLProgressElement": SimTypeBottom(label="IHTMLProgressElement"),
    "IHTMLRect": SimTypeBottom(label="IHTMLRect"),
    "IHTMLRect2": SimTypeBottom(label="IHTMLRect2"),
    "IHTMLRectCollection": SimTypeBottom(label="IHTMLRectCollection"),
    "IHTMLRenderStyle": SimTypeBottom(label="IHTMLRenderStyle"),
    "IHTMLRuleStyle": SimTypeBottom(label="IHTMLRuleStyle"),
    "IHTMLRuleStyle2": SimTypeBottom(label="IHTMLRuleStyle2"),
    "IHTMLRuleStyle3": SimTypeBottom(label="IHTMLRuleStyle3"),
    "IHTMLRuleStyle4": SimTypeBottom(label="IHTMLRuleStyle4"),
    "IHTMLRuleStyle5": SimTypeBottom(label="IHTMLRuleStyle5"),
    "IHTMLRuleStyle6": SimTypeBottom(label="IHTMLRuleStyle6"),
    "IHTMLScreen": SimTypeBottom(label="IHTMLScreen"),
    "IHTMLScreen2": SimTypeBottom(label="IHTMLScreen2"),
    "IHTMLScreen3": SimTypeBottom(label="IHTMLScreen3"),
    "IHTMLScreen4": SimTypeBottom(label="IHTMLScreen4"),
    "IHTMLScriptElement": SimTypeBottom(label="IHTMLScriptElement"),
    "IHTMLScriptElement2": SimTypeBottom(label="IHTMLScriptElement2"),
    "IHTMLScriptElement3": SimTypeBottom(label="IHTMLScriptElement3"),
    "IHTMLScriptElement4": SimTypeBottom(label="IHTMLScriptElement4"),
    "IHTMLSelectElement": SimTypeBottom(label="IHTMLSelectElement"),
    "IHTMLSelectElement2": SimTypeBottom(label="IHTMLSelectElement2"),
    "IHTMLSelectElement4": SimTypeBottom(label="IHTMLSelectElement4"),
    "IHTMLSelectElement5": SimTypeBottom(label="IHTMLSelectElement5"),
    "IHTMLSelectElement6": SimTypeBottom(label="IHTMLSelectElement6"),
    "IHTMLSelectElementEx": SimTypeBottom(label="IHTMLSelectElementEx"),
    "IHTMLSelection": SimTypeBottom(label="IHTMLSelection"),
    "IHTMLSelectionObject": SimTypeBottom(label="IHTMLSelectionObject"),
    "IHTMLSelectionObject2": SimTypeBottom(label="IHTMLSelectionObject2"),
    "IHTMLSourceElement": SimTypeBottom(label="IHTMLSourceElement"),
    "IHTMLSpanElement": SimTypeBottom(label="IHTMLSpanElement"),
    "IHTMLSpanFlow": SimTypeBottom(label="IHTMLSpanFlow"),
    "IHTMLStorage": SimTypeBottom(label="IHTMLStorage"),
    "IHTMLStorage2": SimTypeBottom(label="IHTMLStorage2"),
    "IHTMLStyle": SimTypeBottom(label="IHTMLStyle"),
    "IHTMLStyle2": SimTypeBottom(label="IHTMLStyle2"),
    "IHTMLStyle3": SimTypeBottom(label="IHTMLStyle3"),
    "IHTMLStyle4": SimTypeBottom(label="IHTMLStyle4"),
    "IHTMLStyle5": SimTypeBottom(label="IHTMLStyle5"),
    "IHTMLStyle6": SimTypeBottom(label="IHTMLStyle6"),
    "IHTMLStyleElement": SimTypeBottom(label="IHTMLStyleElement"),
    "IHTMLStyleElement2": SimTypeBottom(label="IHTMLStyleElement2"),
    "IHTMLStyleEnabled": SimTypeBottom(label="IHTMLStyleEnabled"),
    "IHTMLStyleFontFace": SimTypeBottom(label="IHTMLStyleFontFace"),
    "IHTMLStyleFontFace2": SimTypeBottom(label="IHTMLStyleFontFace2"),
    "IHTMLStyleMedia": SimTypeBottom(label="IHTMLStyleMedia"),
    "IHTMLStyleSheet": SimTypeBottom(label="IHTMLStyleSheet"),
    "IHTMLStyleSheet2": SimTypeBottom(label="IHTMLStyleSheet2"),
    "IHTMLStyleSheet3": SimTypeBottom(label="IHTMLStyleSheet3"),
    "IHTMLStyleSheet4": SimTypeBottom(label="IHTMLStyleSheet4"),
    "IHTMLStyleSheetPage": SimTypeBottom(label="IHTMLStyleSheetPage"),
    "IHTMLStyleSheetPage2": SimTypeBottom(label="IHTMLStyleSheetPage2"),
    "IHTMLStyleSheetPagesCollection": SimTypeBottom(label="IHTMLStyleSheetPagesCollection"),
    "IHTMLStyleSheetRule": SimTypeBottom(label="IHTMLStyleSheetRule"),
    "IHTMLStyleSheetRule2": SimTypeBottom(label="IHTMLStyleSheetRule2"),
    "IHTMLStyleSheetRuleApplied": SimTypeBottom(label="IHTMLStyleSheetRuleApplied"),
    "IHTMLStyleSheetRulesAppliedCollection": SimTypeBottom(label="IHTMLStyleSheetRulesAppliedCollection"),
    "IHTMLStyleSheetRulesCollection": SimTypeBottom(label="IHTMLStyleSheetRulesCollection"),
    "IHTMLStyleSheetRulesCollection2": SimTypeBottom(label="IHTMLStyleSheetRulesCollection2"),
    "IHTMLStyleSheetsCollection": SimTypeBottom(label="IHTMLStyleSheetsCollection"),
    "IHTMLStyleSheetsCollection2": SimTypeBottom(label="IHTMLStyleSheetsCollection2"),
    "IHTMLSubmitData": SimTypeBottom(label="IHTMLSubmitData"),
    "IHTMLTable": SimTypeBottom(label="IHTMLTable"),
    "IHTMLTable2": SimTypeBottom(label="IHTMLTable2"),
    "IHTMLTable3": SimTypeBottom(label="IHTMLTable3"),
    "IHTMLTable4": SimTypeBottom(label="IHTMLTable4"),
    "IHTMLTableCaption": SimTypeBottom(label="IHTMLTableCaption"),
    "IHTMLTableCell": SimTypeBottom(label="IHTMLTableCell"),
    "IHTMLTableCell2": SimTypeBottom(label="IHTMLTableCell2"),
    "IHTMLTableCell3": SimTypeBottom(label="IHTMLTableCell3"),
    "IHTMLTableCol": SimTypeBottom(label="IHTMLTableCol"),
    "IHTMLTableCol2": SimTypeBottom(label="IHTMLTableCol2"),
    "IHTMLTableCol3": SimTypeBottom(label="IHTMLTableCol3"),
    "IHTMLTableRow": SimTypeBottom(label="IHTMLTableRow"),
    "IHTMLTableRow2": SimTypeBottom(label="IHTMLTableRow2"),
    "IHTMLTableRow3": SimTypeBottom(label="IHTMLTableRow3"),
    "IHTMLTableRow4": SimTypeBottom(label="IHTMLTableRow4"),
    "IHTMLTableRowMetrics": SimTypeBottom(label="IHTMLTableRowMetrics"),
    "IHTMLTableSection": SimTypeBottom(label="IHTMLTableSection"),
    "IHTMLTableSection2": SimTypeBottom(label="IHTMLTableSection2"),
    "IHTMLTableSection3": SimTypeBottom(label="IHTMLTableSection3"),
    "IHTMLTableSection4": SimTypeBottom(label="IHTMLTableSection4"),
    "IHTMLTextAreaElement": SimTypeBottom(label="IHTMLTextAreaElement"),
    "IHTMLTextAreaElement2": SimTypeBottom(label="IHTMLTextAreaElement2"),
    "IHTMLTextContainer": SimTypeBottom(label="IHTMLTextContainer"),
    "IHTMLTextElement": SimTypeBottom(label="IHTMLTextElement"),
    "IHTMLTextRangeMetrics": SimTypeBottom(label="IHTMLTextRangeMetrics"),
    "IHTMLTextRangeMetrics2": SimTypeBottom(label="IHTMLTextRangeMetrics2"),
    "IHTMLTimeRanges": SimTypeBottom(label="IHTMLTimeRanges"),
    "IHTMLTimeRanges2": SimTypeBottom(label="IHTMLTimeRanges2"),
    "IHTMLTitleElement": SimTypeBottom(label="IHTMLTitleElement"),
    "IHTMLTxtRange": SimTypeBottom(label="IHTMLTxtRange"),
    "IHTMLTxtRangeCollection": SimTypeBottom(label="IHTMLTxtRangeCollection"),
    "IHTMLUListElement": SimTypeBottom(label="IHTMLUListElement"),
    "IHTMLUniqueName": SimTypeBottom(label="IHTMLUniqueName"),
    "IHTMLUnknownElement": SimTypeBottom(label="IHTMLUnknownElement"),
    "IHTMLUrnCollection": SimTypeBottom(label="IHTMLUrnCollection"),
    "IHTMLUserDataOM": SimTypeBottom(label="IHTMLUserDataOM"),
    "IHTMLVideoElement": SimTypeBottom(label="IHTMLVideoElement"),
    "IHTMLWindow2": SimTypeBottom(label="IHTMLWindow2"),
    "IHTMLWindow3": SimTypeBottom(label="IHTMLWindow3"),
    "IHTMLWindow4": SimTypeBottom(label="IHTMLWindow4"),
    "IHTMLWindow5": SimTypeBottom(label="IHTMLWindow5"),
    "IHTMLWindow6": SimTypeBottom(label="IHTMLWindow6"),
    "IHTMLWindow7": SimTypeBottom(label="IHTMLWindow7"),
    "IHTMLWindow8": SimTypeBottom(label="IHTMLWindow8"),
    "IHTMLXDomainRequest": SimTypeBottom(label="IHTMLXDomainRequest"),
    "IHTMLXDomainRequestFactory": SimTypeBottom(label="IHTMLXDomainRequestFactory"),
    "IHTMLXMLHttpRequest": SimTypeBottom(label="IHTMLXMLHttpRequest"),
    "IHTMLXMLHttpRequest2": SimTypeBottom(label="IHTMLXMLHttpRequest2"),
    "IHTMLXMLHttpRequestFactory": SimTypeBottom(label="IHTMLXMLHttpRequestFactory"),
    "IHWEventHandler": SimTypeBottom(label="IHWEventHandler"),
    "IHWEventHandler2": SimTypeBottom(label="IHWEventHandler2"),
    "IHandlerActivationHost": SimTypeBottom(label="IHandlerActivationHost"),
    "IHandlerInfo": SimTypeBottom(label="IHandlerInfo"),
    "IHandlerInfo2": SimTypeBottom(label="IHandlerInfo2"),
    "IHandwrittenTextInsertion": SimTypeBottom(label="IHandwrittenTextInsertion"),
    "IHardwareAudioEngineBase": SimTypeBottom(label="IHardwareAudioEngineBase"),
    "IHeaderCtrl": SimTypeBottom(label="IHeaderCtrl"),
    "IHeaderCtrl2": SimTypeBottom(label="IHeaderCtrl2"),
    "IHeaderFooter": SimTypeBottom(label="IHeaderFooter"),
    "IHeaderFooter2": SimTypeBottom(label="IHeaderFooter2"),
    "IHighlightRenderingServices": SimTypeBottom(label="IHighlightRenderingServices"),
    "IHighlightSegment": SimTypeBottom(label="IHighlightSegment"),
    "IHlink": SimTypeBottom(label="IHlink"),
    "IHlinkBrowseContext": SimTypeBottom(label="IHlinkBrowseContext"),
    "IHlinkFrame": SimTypeBottom(label="IHlinkFrame"),
    "IHlinkSite": SimTypeBottom(label="IHlinkSite"),
    "IHlinkTarget": SimTypeBottom(label="IHlinkTarget"),
    "IHolder": SimTypeBottom(label="IHolder"),
    "IHomeGroup": SimTypeBottom(label="IHomeGroup"),
    "IHomePage": SimTypeBottom(label="IHomePage"),
    "IHomePageSetting": SimTypeBottom(label="IHomePageSetting"),
    "IHostAssemblyManager": SimTypeBottom(label="IHostAssemblyManager"),
    "IHostAssemblyStore": SimTypeBottom(label="IHostAssemblyStore"),
    "IHostAutoEvent": SimTypeBottom(label="IHostAutoEvent"),
    "IHostBehaviorInit": SimTypeBottom(label="IHostBehaviorInit"),
    "IHostControl": SimTypeBottom(label="IHostControl"),
    "IHostCrst": SimTypeBottom(label="IHostCrst"),
    "IHostDataModelAccess": SimTypeBottom(label="IHostDataModelAccess"),
    "IHostDialogHelper": SimTypeBottom(label="IHostDialogHelper"),
    "IHostGCManager": SimTypeBottom(label="IHostGCManager"),
    "IHostIoCompletionManager": SimTypeBottom(label="IHostIoCompletionManager"),
    "IHostMalloc": SimTypeBottom(label="IHostMalloc"),
    "IHostManualEvent": SimTypeBottom(label="IHostManualEvent"),
    "IHostMemoryManager": SimTypeBottom(label="IHostMemoryManager"),
    "IHostPolicyManager": SimTypeBottom(label="IHostPolicyManager"),
    "IHostSecurityContext": SimTypeBottom(label="IHostSecurityContext"),
    "IHostSecurityManager": SimTypeBottom(label="IHostSecurityManager"),
    "IHostSemaphore": SimTypeBottom(label="IHostSemaphore"),
    "IHostSyncManager": SimTypeBottom(label="IHostSyncManager"),
    "IHostTask": SimTypeBottom(label="IHostTask"),
    "IHostTaskManager": SimTypeBottom(label="IHostTaskManager"),
    "IHostThreadpoolManager": SimTypeBottom(label="IHostThreadpoolManager"),
    "IHtmlDlgSafeHelper": SimTypeBottom(label="IHtmlDlgSafeHelper"),
    "IHttpNegotiate": SimTypeBottom(label="IHttpNegotiate"),
    "IHttpNegotiate2": SimTypeBottom(label="IHttpNegotiate2"),
    "IHttpNegotiate3": SimTypeBottom(label="IHttpNegotiate3"),
    "IHttpSecurity": SimTypeBottom(label="IHttpSecurity"),
    "IICCSVGColor": SimTypeBottom(label="IICCSVGColor"),
    "IIE70DispatchEx": SimTypeBottom(label="IIE70DispatchEx"),
    "IIE80DispatchEx": SimTypeBottom(label="IIE80DispatchEx"),
    "IIEWebDriverManager": SimTypeBottom(label="IIEWebDriverManager"),
    "IIEWebDriverSite": SimTypeBottom(label="IIEWebDriverSite"),
    "IIMEServices": SimTypeBottom(label="IIMEServices"),
    "IIOCancelInformation": SimTypeBottom(label="IIOCancelInformation"),
    "IIPDVDec": SimTypeBottom(label="IIPDVDec"),
    "IISDBSLocator": SimTypeBottom(label="IISDBSLocator"),
    "IISDB_BIT": SimTypeBottom(label="IISDB_BIT"),
    "IISDB_CDT": SimTypeBottom(label="IISDB_CDT"),
    "IISDB_EMM": SimTypeBottom(label="IISDB_EMM"),
    "IISDB_LDT": SimTypeBottom(label="IISDB_LDT"),
    "IISDB_NBIT": SimTypeBottom(label="IISDB_NBIT"),
    "IISDB_SDT": SimTypeBottom(label="IISDB_SDT"),
    "IISDB_SDTT": SimTypeBottom(label="IISDB_SDTT"),
    "IITDatabase": SimTypeBottom(label="IITDatabase"),
    "IITPropList": SimTypeBottom(label="IITPropList"),
    "IITResultSet": SimTypeBottom(label="IITResultSet"),
    "IIdentityAdvise": SimTypeBottom(label="IIdentityAdvise"),
    "IIdentityAuthentication": SimTypeBottom(label="IIdentityAuthentication"),
    "IIdentityName": SimTypeBottom(label="IIdentityName"),
    "IIdentityProvider": SimTypeBottom(label="IIdentityProvider"),
    "IIdentityStore": SimTypeBottom(label="IIdentityStore"),
    "IIdentityStoreEx": SimTypeBottom(label="IIdentityStoreEx"),
    "IIdleSettings": SimTypeBottom(label="IIdleSettings"),
    "IIdleTrigger": SimTypeBottom(label="IIdleTrigger"),
    "IImageBytes": SimTypeBottom(label="IImageBytes"),
    "IImageDecodeEventSink": SimTypeBottom(label="IImageDecodeEventSink"),
    "IImageDecodeEventSink2": SimTypeBottom(label="IImageDecodeEventSink2"),
    "IImageDecodeFilter": SimTypeBottom(label="IImageDecodeFilter"),
    "IImageInformation": SimTypeBottom(label="IImageInformation"),
    "IImageList": SimTypeBottom(label="IImageList"),
    "IImageList2": SimTypeBottom(label="IImageList2"),
    "IImageRecompress": SimTypeBottom(label="IImageRecompress"),
    "IImePad": SimTypeBottom(label="IImePad"),
    "IImePadApplet": SimTypeBottom(label="IImePadApplet"),
    "IImePlugInDictDictionaryList": SimTypeBottom(label="IImePlugInDictDictionaryList"),
    "IImeSpecifyApplets": SimTypeBottom(label="IImeSpecifyApplets"),
    "IImgCreateErrorInfo": SimTypeBottom(label="IImgCreateErrorInfo"),
    "IImgErrorInfo": SimTypeBottom(label="IImgErrorInfo"),
    "IIndependentFlipFramePresentStatistics": SimTypeBottom(label="IIndependentFlipFramePresentStatistics"),
    "IIndexDefinition": SimTypeBottom(label="IIndexDefinition"),
    "IIndexableConcept": SimTypeBottom(label="IIndexableConcept"),
    "IInertiaProcessor": SimTypeBottom(label="IInertiaProcessor"),
    "IInitializeCommand": SimTypeBottom(label="IInitializeCommand"),
    "IInitializeNetworkFolder": SimTypeBottom(label="IInitializeNetworkFolder"),
    "IInitializeObject": SimTypeBottom(label="IInitializeObject"),
    "IInitializeSpy": SimTypeBottom(label="IInitializeSpy"),
    "IInitializeWithBindCtx": SimTypeBottom(label="IInitializeWithBindCtx"),
    "IInitializeWithFile": SimTypeBottom(label="IInitializeWithFile"),
    "IInitializeWithItem": SimTypeBottom(label="IInitializeWithItem"),
    "IInitializeWithPropertyStore": SimTypeBottom(label="IInitializeWithPropertyStore"),
    "IInitializeWithStream": SimTypeBottom(label="IInitializeWithStream"),
    "IInitializeWithWindow": SimTypeBottom(label="IInitializeWithWindow"),
    "IInitiateWinSATAssessment": SimTypeBottom(label="IInitiateWinSATAssessment"),
    "IInk": SimTypeBottom(label="IInk"),
    "IInkCollector": SimTypeBottom(label="IInkCollector"),
    "IInkCommitRequestHandler": SimTypeBottom(label="IInkCommitRequestHandler"),
    "IInkCursor": SimTypeBottom(label="IInkCursor"),
    "IInkCursorButton": SimTypeBottom(label="IInkCursorButton"),
    "IInkCursorButtons": SimTypeBottom(label="IInkCursorButtons"),
    "IInkCursors": SimTypeBottom(label="IInkCursors"),
    "IInkCustomStrokes": SimTypeBottom(label="IInkCustomStrokes"),
    "IInkD2DRenderer": SimTypeBottom(label="IInkD2DRenderer"),
    "IInkD2DRenderer2": SimTypeBottom(label="IInkD2DRenderer2"),
    "IInkDesktopHost": SimTypeBottom(label="IInkDesktopHost"),
    "IInkDisp": SimTypeBottom(label="IInkDisp"),
    "IInkDivider": SimTypeBottom(label="IInkDivider"),
    "IInkDivisionResult": SimTypeBottom(label="IInkDivisionResult"),
    "IInkDivisionUnit": SimTypeBottom(label="IInkDivisionUnit"),
    "IInkDivisionUnits": SimTypeBottom(label="IInkDivisionUnits"),
    "IInkDrawingAttributes": SimTypeBottom(label="IInkDrawingAttributes"),
    "IInkEdit": SimTypeBottom(label="IInkEdit"),
    "IInkExtendedProperties": SimTypeBottom(label="IInkExtendedProperties"),
    "IInkExtendedProperty": SimTypeBottom(label="IInkExtendedProperty"),
    "IInkGesture": SimTypeBottom(label="IInkGesture"),
    "IInkHostWorkItem": SimTypeBottom(label="IInkHostWorkItem"),
    "IInkLineInfo": SimTypeBottom(label="IInkLineInfo"),
    "IInkOverlay": SimTypeBottom(label="IInkOverlay"),
    "IInkPicture": SimTypeBottom(label="IInkPicture"),
    "IInkPresenterDesktop": SimTypeBottom(label="IInkPresenterDesktop"),
    "IInkRecognitionAlternate": SimTypeBottom(label="IInkRecognitionAlternate"),
    "IInkRecognitionAlternates": SimTypeBottom(label="IInkRecognitionAlternates"),
    "IInkRecognitionResult": SimTypeBottom(label="IInkRecognitionResult"),
    "IInkRecognizer": SimTypeBottom(label="IInkRecognizer"),
    "IInkRecognizer2": SimTypeBottom(label="IInkRecognizer2"),
    "IInkRecognizerContext": SimTypeBottom(label="IInkRecognizerContext"),
    "IInkRecognizerContext2": SimTypeBottom(label="IInkRecognizerContext2"),
    "IInkRecognizerGuide": SimTypeBottom(label="IInkRecognizerGuide"),
    "IInkRecognizers": SimTypeBottom(label="IInkRecognizers"),
    "IInkRectangle": SimTypeBottom(label="IInkRectangle"),
    "IInkRenderer": SimTypeBottom(label="IInkRenderer"),
    "IInkStrokeDisp": SimTypeBottom(label="IInkStrokeDisp"),
    "IInkStrokes": SimTypeBottom(label="IInkStrokes"),
    "IInkTablet": SimTypeBottom(label="IInkTablet"),
    "IInkTablet2": SimTypeBottom(label="IInkTablet2"),
    "IInkTablet3": SimTypeBottom(label="IInkTablet3"),
    "IInkTablets": SimTypeBottom(label="IInkTablets"),
    "IInkTransform": SimTypeBottom(label="IInkTransform"),
    "IInkWordList": SimTypeBottom(label="IInkWordList"),
    "IInkWordList2": SimTypeBottom(label="IInkWordList2"),
    "IInputObject": SimTypeBottom(label="IInputObject"),
    "IInputObject2": SimTypeBottom(label="IInputObject2"),
    "IInputObjectSite": SimTypeBottom(label="IInputObjectSite"),
    "IInputPaneAnimationCoordinator": SimTypeBottom(label="IInputPaneAnimationCoordinator"),
    "IInputPanelConfiguration": SimTypeBottom(label="IInputPanelConfiguration"),
    "IInputPanelInvocationConfiguration": SimTypeBottom(label="IInputPanelInvocationConfiguration"),
    "IInputPanelWindowHandle": SimTypeBottom(label="IInputPanelWindowHandle"),
    "IInsertItem": SimTypeBottom(label="IInsertItem"),
    "IInstallationAgent": SimTypeBottom(label="IInstallationAgent"),
    "IInstallationBehavior": SimTypeBottom(label="IInstallationBehavior"),
    "IInstallationCompletedCallback": SimTypeBottom(label="IInstallationCompletedCallback"),
    "IInstallationCompletedCallbackArgs": SimTypeBottom(label="IInstallationCompletedCallbackArgs"),
    "IInstallationJob": SimTypeBottom(label="IInstallationJob"),
    "IInstallationProgress": SimTypeBottom(label="IInstallationProgress"),
    "IInstallationProgressChangedCallback": SimTypeBottom(label="IInstallationProgressChangedCallback"),
    "IInstallationProgressChangedCallbackArgs": SimTypeBottom(label="IInstallationProgressChangedCallbackArgs"),
    "IInstallationResult": SimTypeBottom(label="IInstallationResult"),
    "IIntelliForms": SimTypeBottom(label="IIntelliForms"),
    "IInterFilterCommunicator": SimTypeBottom(label="IInterFilterCommunicator"),
    "IInterfaceRelated": SimTypeBottom(label="IInterfaceRelated"),
    "IInternalDocWrap": SimTypeBottom(label="IInternalDocWrap"),
    "IInternalUnknown": SimTypeBottom(label="IInternalUnknown"),
    "IInternet": SimTypeBottom(label="IInternet"),
    "IInternetBindInfo": SimTypeBottom(label="IInternetBindInfo"),
    "IInternetBindInfoEx": SimTypeBottom(label="IInternetBindInfoEx"),
    "IInternetExplorerManager": SimTypeBottom(label="IInternetExplorerManager"),
    "IInternetExplorerManager2": SimTypeBottom(label="IInternetExplorerManager2"),
    "IInternetHostSecurityManager": SimTypeBottom(label="IInternetHostSecurityManager"),
    "IInternetPriority": SimTypeBottom(label="IInternetPriority"),
    "IInternetProtocol": SimTypeBottom(label="IInternetProtocol"),
    "IInternetProtocolEx": SimTypeBottom(label="IInternetProtocolEx"),
    "IInternetProtocolInfo": SimTypeBottom(label="IInternetProtocolInfo"),
    "IInternetProtocolRoot": SimTypeBottom(label="IInternetProtocolRoot"),
    "IInternetProtocolSink": SimTypeBottom(label="IInternetProtocolSink"),
    "IInternetProtocolSinkStackable": SimTypeBottom(label="IInternetProtocolSinkStackable"),
    "IInternetSecurityManager": SimTypeBottom(label="IInternetSecurityManager"),
    "IInternetSecurityManagerEx": SimTypeBottom(label="IInternetSecurityManagerEx"),
    "IInternetSecurityManagerEx2": SimTypeBottom(label="IInternetSecurityManagerEx2"),
    "IInternetSecurityMgrSite": SimTypeBottom(label="IInternetSecurityMgrSite"),
    "IInternetSession": SimTypeBottom(label="IInternetSession"),
    "IInternetThreadSwitch": SimTypeBottom(label="IInternetThreadSwitch"),
    "IInternetZoneManager": SimTypeBottom(label="IInternetZoneManager"),
    "IInternetZoneManagerEx": SimTypeBottom(label="IInternetZoneManagerEx"),
    "IInternetZoneManagerEx2": SimTypeBottom(label="IInternetZoneManagerEx2"),
    "IInterval": SimTypeBottom(label="IInterval"),
    "IInvalidProductLicenseException": SimTypeBottom(label="IInvalidProductLicenseException"),
    "IInvokeProvider": SimTypeBottom(label="IInvokeProvider"),
    "IIsdbAudioComponentDescriptor": SimTypeBottom(label="IIsdbAudioComponentDescriptor"),
    "IIsdbCAContractInformationDescriptor": SimTypeBottom(label="IIsdbCAContractInformationDescriptor"),
    "IIsdbCADescriptor": SimTypeBottom(label="IIsdbCADescriptor"),
    "IIsdbCAServiceDescriptor": SimTypeBottom(label="IIsdbCAServiceDescriptor"),
    "IIsdbComponentGroupDescriptor": SimTypeBottom(label="IIsdbComponentGroupDescriptor"),
    "IIsdbDataContentDescriptor": SimTypeBottom(label="IIsdbDataContentDescriptor"),
    "IIsdbDigitalCopyControlDescriptor": SimTypeBottom(label="IIsdbDigitalCopyControlDescriptor"),
    "IIsdbDownloadContentDescriptor": SimTypeBottom(label="IIsdbDownloadContentDescriptor"),
    "IIsdbEmergencyInformationDescriptor": SimTypeBottom(label="IIsdbEmergencyInformationDescriptor"),
    "IIsdbEventGroupDescriptor": SimTypeBottom(label="IIsdbEventGroupDescriptor"),
    "IIsdbHierarchicalTransmissionDescriptor": SimTypeBottom(label="IIsdbHierarchicalTransmissionDescriptor"),
    "IIsdbLogoTransmissionDescriptor": SimTypeBottom(label="IIsdbLogoTransmissionDescriptor"),
    "IIsdbSIParameterDescriptor": SimTypeBottom(label="IIsdbSIParameterDescriptor"),
    "IIsdbSeriesDescriptor": SimTypeBottom(label="IIsdbSeriesDescriptor"),
    "IIsdbSiParser2": SimTypeBottom(label="IIsdbSiParser2"),
    "IIsdbTSInformationDescriptor": SimTypeBottom(label="IIsdbTSInformationDescriptor"),
    "IIsdbTerrestrialDeliverySystemDescriptor": SimTypeBottom(label="IIsdbTerrestrialDeliverySystemDescriptor"),
    "IIsoImageManager": SimTypeBottom(label="IIsoImageManager"),
    "IIsolatedAppLauncher": SimTypeBottom(label="IIsolatedAppLauncher"),
    "IIsolatedProcessLauncher": SimTypeBottom(label="IIsolatedProcessLauncher"),
    "IIsolatedProcessLauncher2": SimTypeBottom(label="IIsolatedProcessLauncher2"),
    "IItemContainerProvider": SimTypeBottom(label="IItemContainerProvider"),
    "IItemEnumerator": SimTypeBottom(label="IItemEnumerator"),
    "IItemNameLimits": SimTypeBottom(label="IItemNameLimits"),
    "IIterableConcept": SimTypeBottom(label="IIterableConcept"),
    "IJolietDiscMaster": SimTypeBottom(label="IJolietDiscMaster"),
    "IJsDebug": SimTypeBottom(label="IJsDebug"),
    "IJsDebugBreakPoint": SimTypeBottom(label="IJsDebugBreakPoint"),
    "IJsDebugDataTarget": SimTypeBottom(label="IJsDebugDataTarget"),
    "IJsDebugFrame": SimTypeBottom(label="IJsDebugFrame"),
    "IJsDebugProcess": SimTypeBottom(label="IJsDebugProcess"),
    "IJsDebugProperty": SimTypeBottom(label="IJsDebugProperty"),
    "IJsDebugStackWalker": SimTypeBottom(label="IJsDebugStackWalker"),
    "IJsEnumDebugProperty": SimTypeBottom(label="IJsEnumDebugProperty"),
    "IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"),
    "IKEEXT_AUTHENTICATION_METHOD0": SimStruct(OrderedDict((("authenticationMethodType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("Anonymous", SimUnion({"presharedKeyAuthentication": SimTypeRef("IKEEXT_PRESHARED_KEY_AUTHENTICATION0", SimStruct), "certificateAuthentication": SimTypeRef("IKEEXT_CERTIFICATE_AUTHENTICATION0", SimStruct), "kerberosAuthentication": SimTypeRef("IKEEXT_KERBEROS_AUTHENTICATION0", SimStruct), "ntlmV2Authentication": SimTypeRef("IKEEXT_NTLM_V2_AUTHENTICATION0", SimStruct), "sslAuthentication": SimTypeRef("IKEEXT_CERTIFICATE_AUTHENTICATION0", SimStruct), "cgaAuthentication": SimTypeRef("IKEEXT_IPV6_CGA_AUTHENTICATION0", SimStruct)}, name="<anon>", label="None")),)), name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None),
    "IKEEXT_AUTHENTICATION_METHOD1": SimStruct(OrderedDict((("authenticationMethodType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("Anonymous", SimUnion({"presharedKeyAuthentication": SimTypeRef("IKEEXT_PRESHARED_KEY_AUTHENTICATION1", SimStruct), "certificateAuthentication": SimTypeRef("IKEEXT_CERTIFICATE_AUTHENTICATION1", SimStruct), "kerberosAuthentication": SimTypeRef("IKEEXT_KERBEROS_AUTHENTICATION0", SimStruct), "ntlmV2Authentication": SimTypeRef("IKEEXT_NTLM_V2_AUTHENTICATION0", SimStruct), "sslAuthentication": SimTypeRef("IKEEXT_CERTIFICATE_AUTHENTICATION1", SimStruct), "cgaAuthentication": SimTypeRef("IKEEXT_IPV6_CGA_AUTHENTICATION0", SimStruct), "eapAuthentication": SimTypeRef("IKEEXT_EAP_AUTHENTICATION0", SimStruct)}, name="<anon>", label="None")),)), name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None),
    "IKEEXT_AUTHENTICATION_METHOD2": SimStruct(OrderedDict((("authenticationMethodType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("Anonymous", SimUnion({"presharedKeyAuthentication": SimTypeRef("IKEEXT_PRESHARED_KEY_AUTHENTICATION1", SimStruct), "certificateAuthentication": SimTypeRef("IKEEXT_CERTIFICATE_AUTHENTICATION2", SimStruct), "kerberosAuthentication": SimTypeRef("IKEEXT_KERBEROS_AUTHENTICATION1", SimStruct), "reservedAuthentication": SimTypeRef("IKEEXT_RESERVED_AUTHENTICATION0", SimStruct), "ntlmV2Authentication": SimTypeRef("IKEEXT_NTLM_V2_AUTHENTICATION0", SimStruct), "sslAuthentication": SimTypeRef("IKEEXT_CERTIFICATE_AUTHENTICATION2", SimStruct), "cgaAuthentication": SimTypeRef("IKEEXT_IPV6_CGA_AUTHENTICATION0", SimStruct), "eapAuthentication": SimTypeRef("IKEEXT_EAP_AUTHENTICATION0", SimStruct)}, name="<anon>", label="None")),)), name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None),
    "IKEEXT_AUTHENTICATION_METHOD_TYPE": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"),
    "IKEEXT_CERTIFICATE_AUTHENTICATION0": SimStruct(OrderedDict((("inboundConfigType", SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE")), ("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("inboundRootArraySize", SimTypeInt(signed=False, label="UInt32")), ("inboundRootArray", SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)}, name="<anon>", label="None")), ("outboundConfigType", SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE")), ("Anonymous2", SimUnion({"Anonymous": SimStruct(OrderedDict((("outboundRootArraySize", SimTypeInt(signed=False, label="UInt32")), ("outboundRootArray", SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)}, name="<anon>", label="None")), ("flags", SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")),)), name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_CERTIFICATE_AUTHENTICATION1": SimStruct(OrderedDict((("inboundConfigType", SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE")), ("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("inboundRootArraySize", SimTypeInt(signed=False, label="UInt32")), ("inboundRootArray", SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)}, name="<anon>", label="None")), ("outboundConfigType", SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE")), ("Anonymous2", SimUnion({"Anonymous": SimStruct(OrderedDict((("outboundRootArraySize", SimTypeInt(signed=False, label="UInt32")), ("outboundRootArray", SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimTypeRef("IKEEXT_CERT_ROOT_CONFIG0", SimStruct), offset=0)}, name="<anon>", label="None")), ("flags", SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")), ("localCertLocationUrl", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None),
    "IKEEXT_CERTIFICATE_AUTHENTICATION2": SimStruct(OrderedDict((("inboundConfigType", SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE")), ("Anonymous1", SimUnion({"Anonymous1": SimStruct(OrderedDict((("inboundRootArraySize", SimTypeInt(signed=False, label="UInt32")), ("inboundRootCriteria", SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CRITERIA0", SimStruct), offset=0)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("inboundEnterpriseStoreArraySize", SimTypeInt(signed=False, label="UInt32")), ("inboundEnterpriseStoreCriteria", SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CRITERIA0", SimStruct), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct(OrderedDict((("inboundRootStoreArraySize", SimTypeInt(signed=False, label="UInt32")), ("inboundTrustedRootStoreCriteria", SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CRITERIA0", SimStruct), offset=0)),)), name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("outboundConfigType", SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE")), ("Anonymous2", SimUnion({"Anonymous1": SimStruct(OrderedDict((("outboundRootArraySize", SimTypeInt(signed=False, label="UInt32")), ("outboundRootCriteria", SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CRITERIA0", SimStruct), offset=0)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("outboundEnterpriseStoreArraySize", SimTypeInt(signed=False, label="UInt32")), ("outboundEnterpriseStoreCriteria", SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CRITERIA0", SimStruct), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct(OrderedDict((("outboundRootStoreArraySize", SimTypeInt(signed=False, label="UInt32")), ("outboundTrustedRootStoreCriteria", SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CRITERIA0", SimStruct), offset=0)),)), name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("flags", SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")), ("localCertLocationUrl", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None),
    "IKEEXT_CERTIFICATE_CREDENTIAL0": SimStruct(OrderedDict((("subjectName", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("certHash", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_CERTIFICATE_CREDENTIAL0", pack=False, align=None),
    "IKEEXT_CERTIFICATE_CREDENTIAL1": SimStruct(OrderedDict((("subjectName", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("certHash", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("certificate", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None),
    "IKEEXT_CERTIFICATE_CRITERIA0": SimStruct(OrderedDict((("certData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("certHash", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("eku", SimTypePointer(SimTypeRef("IKEEXT_CERT_EKUS0", SimStruct), offset=0)), ("name", SimTypePointer(SimTypeRef("IKEEXT_CERT_NAME0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None),
    "IKEEXT_CERT_AUTH": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"),
    "IKEEXT_CERT_CONFIG_TYPE": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"),
    "IKEEXT_CERT_CRITERIA_NAME_TYPE": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"),
    "IKEEXT_CERT_EKUS0": SimStruct(OrderedDict((("numEku", SimTypeInt(signed=False, label="UInt32")), ("eku", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="IKEEXT_CERT_EKUS0", pack=False, align=None),
    "IKEEXT_CERT_FLAGS": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS"),
    "IKEEXT_CERT_NAME0": SimStruct(OrderedDict((("nameType", SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE")), ("certName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IKEEXT_CERT_NAME0", pack=False, align=None),
    "IKEEXT_CERT_ROOT_CONFIG0": SimStruct(OrderedDict((("certData", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("flags", SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")),)), name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None),
    "IKEEXT_CIPHER_ALGORITHM0": SimStruct(OrderedDict((("algoIdentifier", SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE")), ("keyLen", SimTypeInt(signed=False, label="UInt32")), ("rounds", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None),
    "IKEEXT_CIPHER_TYPE": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"),
    "IKEEXT_COMMON_STATISTICS0": SimStruct(OrderedDict((("v4Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS0", SimStruct)), ("v6Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS0", SimStruct)), ("totalPacketsReceived", SimTypeInt(signed=False, label="UInt32")), ("totalInvalidPacketsReceived", SimTypeInt(signed=False, label="UInt32")), ("currentQueuedWorkitems", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_COMMON_STATISTICS0", pack=False, align=None),
    "IKEEXT_COMMON_STATISTICS1": SimStruct(OrderedDict((("v4Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS1", SimStruct)), ("v6Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS1", SimStruct)), ("totalPacketsReceived", SimTypeInt(signed=False, label="UInt32")), ("totalInvalidPacketsReceived", SimTypeInt(signed=False, label="UInt32")), ("currentQueuedWorkitems", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_COMMON_STATISTICS1", pack=False, align=None),
    "IKEEXT_COOKIE_PAIR0": SimStruct(OrderedDict((("initiator", SimTypeLongLong(signed=False, label="UInt64")), ("responder", SimTypeLongLong(signed=False, label="UInt64")),)), name="IKEEXT_COOKIE_PAIR0", pack=False, align=None),
    "IKEEXT_CREDENTIAL0": SimStruct(OrderedDict((("authenticationMethodType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("impersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")), ("Anonymous", SimUnion({"presharedKey": SimTypePointer(SimTypeRef("IKEEXT_PRESHARED_KEY_AUTHENTICATION0", SimStruct), offset=0), "certificate": SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CREDENTIAL0", SimStruct), offset=0), "name": SimTypePointer(SimTypeRef("IKEEXT_NAME_CREDENTIAL0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IKEEXT_CREDENTIAL0", pack=False, align=None),
    "IKEEXT_CREDENTIAL1": SimStruct(OrderedDict((("authenticationMethodType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("impersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")), ("Anonymous", SimUnion({"presharedKey": SimTypePointer(SimTypeRef("IKEEXT_PRESHARED_KEY_AUTHENTICATION1", SimStruct), offset=0), "certificate": SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CREDENTIAL1", SimStruct), offset=0), "name": SimTypePointer(SimTypeRef("IKEEXT_NAME_CREDENTIAL0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IKEEXT_CREDENTIAL1", pack=False, align=None),
    "IKEEXT_CREDENTIAL2": SimStruct(OrderedDict((("authenticationMethodType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE")), ("impersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")), ("Anonymous", SimUnion({"presharedKey": SimTypePointer(SimTypeRef("IKEEXT_PRESHARED_KEY_AUTHENTICATION1", SimStruct), offset=0), "certificate": SimTypePointer(SimTypeRef("IKEEXT_CERTIFICATE_CREDENTIAL1", SimStruct), offset=0), "name": SimTypePointer(SimTypeRef("IKEEXT_NAME_CREDENTIAL0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IKEEXT_CREDENTIAL2", pack=False, align=None),
    "IKEEXT_CREDENTIALS0": SimStruct(OrderedDict((("numCredentials", SimTypeInt(signed=False, label="UInt32")), ("credentials", SimTypePointer(SimTypeRef("IKEEXT_CREDENTIAL_PAIR0", SimStruct), offset=0)),)), name="IKEEXT_CREDENTIALS0", pack=False, align=None),
    "IKEEXT_CREDENTIALS1": SimStruct(OrderedDict((("numCredentials", SimTypeInt(signed=False, label="UInt32")), ("credentials", SimTypePointer(SimTypeRef("IKEEXT_CREDENTIAL_PAIR1", SimStruct), offset=0)),)), name="IKEEXT_CREDENTIALS1", pack=False, align=None),
    "IKEEXT_CREDENTIALS2": SimStruct(OrderedDict((("numCredentials", SimTypeInt(signed=False, label="UInt32")), ("credentials", SimTypePointer(SimTypeRef("IKEEXT_CREDENTIAL_PAIR2", SimStruct), offset=0)),)), name="IKEEXT_CREDENTIALS2", pack=False, align=None),
    "IKEEXT_CREDENTIAL_PAIR0": SimStruct(OrderedDict((("localCredentials", SimTypeRef("IKEEXT_CREDENTIAL0", SimStruct)), ("peerCredentials", SimTypeRef("IKEEXT_CREDENTIAL0", SimStruct)),)), name="IKEEXT_CREDENTIAL_PAIR0", pack=False, align=None),
    "IKEEXT_CREDENTIAL_PAIR1": SimStruct(OrderedDict((("localCredentials", SimTypeRef("IKEEXT_CREDENTIAL1", SimStruct)), ("peerCredentials", SimTypeRef("IKEEXT_CREDENTIAL1", SimStruct)),)), name="IKEEXT_CREDENTIAL_PAIR1", pack=False, align=None),
    "IKEEXT_CREDENTIAL_PAIR2": SimStruct(OrderedDict((("localCredentials", SimTypeRef("IKEEXT_CREDENTIAL2", SimStruct)), ("peerCredentials", SimTypeRef("IKEEXT_CREDENTIAL2", SimStruct)),)), name="IKEEXT_CREDENTIAL_PAIR2", pack=False, align=None),
    "IKEEXT_DH_GROUP": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"),
    "IKEEXT_EAP_AUTHENTICATION0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")),)), name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_EAP_AUTHENTICATION_FLAGS": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS"),
    "IKEEXT_EM_POLICY0": SimStruct(OrderedDict((("numAuthenticationMethods", SimTypeInt(signed=False, label="UInt32")), ("authenticationMethods", SimTypePointer(SimTypeRef("IKEEXT_AUTHENTICATION_METHOD0", SimStruct), offset=0)), ("initiatorImpersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")),)), name="IKEEXT_EM_POLICY0", pack=False, align=None),
    "IKEEXT_EM_POLICY1": SimStruct(OrderedDict((("numAuthenticationMethods", SimTypeInt(signed=False, label="UInt32")), ("authenticationMethods", SimTypePointer(SimTypeRef("IKEEXT_AUTHENTICATION_METHOD1", SimStruct), offset=0)), ("initiatorImpersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")),)), name="IKEEXT_EM_POLICY1", pack=False, align=None),
    "IKEEXT_EM_POLICY2": SimStruct(OrderedDict((("numAuthenticationMethods", SimTypeInt(signed=False, label="UInt32")), ("authenticationMethods", SimTypePointer(SimTypeRef("IKEEXT_AUTHENTICATION_METHOD2", SimStruct), offset=0)), ("initiatorImpersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")),)), name="IKEEXT_EM_POLICY2", pack=False, align=None),
    "IKEEXT_EM_SA_STATE": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"),
    "IKEEXT_INTEGRITY_ALGORITHM0": SimStruct(OrderedDict((("algoIdentifier", SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")),)), name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None),
    "IKEEXT_INTEGRITY_TYPE": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE"),
    "IKEEXT_IPV6_CGA_AUTHENTICATION0": SimStruct(OrderedDict((("keyContainerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cspName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cspType", SimTypeInt(signed=False, label="UInt32")), ("cgaModifier", SimTypeRef("FWP_BYTE_ARRAY16", SimStruct)), ("cgaCollisionCount", SimTypeChar(label="Byte")),)), name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS0": SimStruct(OrderedDict((("totalSocketReceiveFailures", SimTypeInt(signed=False, label="UInt32")), ("totalSocketSendFailures", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS0", pack=False, align=None),
    "IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS1": SimStruct(OrderedDict((("totalSocketReceiveFailures", SimTypeInt(signed=False, label="UInt32")), ("totalSocketSendFailures", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS1", pack=False, align=None),
    "IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0": SimStruct(OrderedDict((("currentActiveMainModes", SimTypeInt(signed=False, label="UInt32")), ("totalMainModesStarted", SimTypeInt(signed=False, label="UInt32")), ("totalSuccessfulMainModes", SimTypeInt(signed=False, label="UInt32")), ("totalFailedMainModes", SimTypeInt(signed=False, label="UInt32")), ("totalResponderMainModes", SimTypeInt(signed=False, label="UInt32")), ("currentNewResponderMainModes", SimTypeInt(signed=False, label="UInt32")), ("currentActiveQuickModes", SimTypeInt(signed=False, label="UInt32")), ("totalQuickModesStarted", SimTypeInt(signed=False, label="UInt32")), ("totalSuccessfulQuickModes", SimTypeInt(signed=False, label="UInt32")), ("totalFailedQuickModes", SimTypeInt(signed=False, label="UInt32")), ("totalAcquires", SimTypeInt(signed=False, label="UInt32")), ("totalReinitAcquires", SimTypeInt(signed=False, label="UInt32")), ("currentActiveExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalExtendedModesStarted", SimTypeInt(signed=False, label="UInt32")), ("totalSuccessfulExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalFailedExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalImpersonationExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalImpersonationMainModes", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", pack=False, align=None),
    "IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1": SimStruct(OrderedDict((("currentActiveMainModes", SimTypeInt(signed=False, label="UInt32")), ("totalMainModesStarted", SimTypeInt(signed=False, label="UInt32")), ("totalSuccessfulMainModes", SimTypeInt(signed=False, label="UInt32")), ("totalFailedMainModes", SimTypeInt(signed=False, label="UInt32")), ("totalResponderMainModes", SimTypeInt(signed=False, label="UInt32")), ("currentNewResponderMainModes", SimTypeInt(signed=False, label="UInt32")), ("currentActiveQuickModes", SimTypeInt(signed=False, label="UInt32")), ("totalQuickModesStarted", SimTypeInt(signed=False, label="UInt32")), ("totalSuccessfulQuickModes", SimTypeInt(signed=False, label="UInt32")), ("totalFailedQuickModes", SimTypeInt(signed=False, label="UInt32")), ("totalAcquires", SimTypeInt(signed=False, label="UInt32")), ("totalReinitAcquires", SimTypeInt(signed=False, label="UInt32")), ("currentActiveExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalExtendedModesStarted", SimTypeInt(signed=False, label="UInt32")), ("totalSuccessfulExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalFailedExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalImpersonationExtendedModes", SimTypeInt(signed=False, label="UInt32")), ("totalImpersonationMainModes", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None),
    "IKEEXT_KERBEROS_AUTHENTICATION0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")),)), name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_KERBEROS_AUTHENTICATION1": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")), ("proxyServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None),
    "IKEEXT_KERBEROS_AUTHENTICATION_FLAGS": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"),
    "IKEEXT_KEYMODULE_STATISTICS0": SimStruct(OrderedDict((("v4Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", SimStruct)), ("v6Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", SimStruct)), ("errorFrequencyTable", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 97)), ("mainModeNegotiationTime", SimTypeInt(signed=False, label="UInt32")), ("quickModeNegotiationTime", SimTypeInt(signed=False, label="UInt32")), ("extendedModeNegotiationTime", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_KEYMODULE_STATISTICS0", pack=False, align=None),
    "IKEEXT_KEYMODULE_STATISTICS1": SimStruct(OrderedDict((("v4Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", SimStruct)), ("v6Statistics", SimTypeRef("IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", SimStruct)), ("errorFrequencyTable", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 97)), ("mainModeNegotiationTime", SimTypeInt(signed=False, label="UInt32")), ("quickModeNegotiationTime", SimTypeInt(signed=False, label="UInt32")), ("extendedModeNegotiationTime", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_KEYMODULE_STATISTICS1", pack=False, align=None),
    "IKEEXT_KEY_MODULE_TYPE": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"),
    "IKEEXT_MM_SA_STATE": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"),
    "IKEEXT_NAME_CREDENTIAL0": SimStruct(OrderedDict((("principalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None),
    "IKEEXT_NTLM_V2_AUTHENTICATION0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_POLICY0": SimStruct(OrderedDict((("softExpirationTime", SimTypeInt(signed=False, label="UInt32")), ("numAuthenticationMethods", SimTypeInt(signed=False, label="UInt32")), ("authenticationMethods", SimTypePointer(SimTypeRef("IKEEXT_AUTHENTICATION_METHOD0", SimStruct), offset=0)), ("initiatorImpersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")), ("numIkeProposals", SimTypeInt(signed=False, label="UInt32")), ("ikeProposals", SimTypePointer(SimTypeRef("IKEEXT_PROPOSAL0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG")), ("maxDynamicFilters", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_POLICY0", pack=False, align=None),
    "IKEEXT_POLICY1": SimStruct(OrderedDict((("softExpirationTime", SimTypeInt(signed=False, label="UInt32")), ("numAuthenticationMethods", SimTypeInt(signed=False, label="UInt32")), ("authenticationMethods", SimTypePointer(SimTypeRef("IKEEXT_AUTHENTICATION_METHOD1", SimStruct), offset=0)), ("initiatorImpersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")), ("numIkeProposals", SimTypeInt(signed=False, label="UInt32")), ("ikeProposals", SimTypePointer(SimTypeRef("IKEEXT_PROPOSAL0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG")), ("maxDynamicFilters", SimTypeInt(signed=False, label="UInt32")), ("retransmitDurationSecs", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_POLICY1", pack=False, align=None),
    "IKEEXT_POLICY2": SimStruct(OrderedDict((("softExpirationTime", SimTypeInt(signed=False, label="UInt32")), ("numAuthenticationMethods", SimTypeInt(signed=False, label="UInt32")), ("authenticationMethods", SimTypePointer(SimTypeRef("IKEEXT_AUTHENTICATION_METHOD2", SimStruct), offset=0)), ("initiatorImpersonationType", SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")), ("numIkeProposals", SimTypeInt(signed=False, label="UInt32")), ("ikeProposals", SimTypePointer(SimTypeRef("IKEEXT_PROPOSAL0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG")), ("maxDynamicFilters", SimTypeInt(signed=False, label="UInt32")), ("retransmitDurationSecs", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_POLICY2", pack=False, align=None),
    "IKEEXT_POLICY_FLAG": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"),
    "IKEEXT_PRESHARED_KEY_AUTHENTICATION0": SimStruct(OrderedDict((("presharedKey", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_PRESHARED_KEY_AUTHENTICATION1": SimStruct(OrderedDict((("presharedKey", SimTypeRef("FWP_BYTE_BLOB", SimStruct)), ("flags", SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")),)), name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None),
    "IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS"),
    "IKEEXT_PROPOSAL0": SimStruct(OrderedDict((("cipherAlgorithm", SimTypeRef("IKEEXT_CIPHER_ALGORITHM0", SimStruct)), ("integrityAlgorithm", SimTypeRef("IKEEXT_INTEGRITY_ALGORITHM0", SimStruct)), ("maxLifetimeSeconds", SimTypeInt(signed=False, label="UInt32")), ("dhGroup", SimTypeInt(signed=False, label="IKEEXT_DH_GROUP")), ("quickModeLimit", SimTypeInt(signed=False, label="UInt32")),)), name="IKEEXT_PROPOSAL0", pack=False, align=None),
    "IKEEXT_QM_SA_STATE": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"),
    "IKEEXT_RESERVED_AUTHENTICATION0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")),)), name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None),
    "IKEEXT_RESERVED_AUTHENTICATION_FLAGS": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS"),
    "IKEEXT_SA_DETAILS0": SimStruct(OrderedDict((("saId", SimTypeLongLong(signed=False, label="UInt64")), ("keyModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("ikeTraffic", SimTypeRef("IKEEXT_TRAFFIC0", SimStruct)), ("ikeProposal", SimTypeRef("IKEEXT_PROPOSAL0", SimStruct)), ("cookiePair", SimTypeRef("IKEEXT_COOKIE_PAIR0", SimStruct)), ("ikeCredentials", SimTypeRef("IKEEXT_CREDENTIALS0", SimStruct)), ("ikePolicyKey", SimTypeBottom(label="Guid")), ("virtualIfTunnelId", SimTypeLongLong(signed=False, label="UInt64")),)), name="IKEEXT_SA_DETAILS0", pack=False, align=None),
    "IKEEXT_SA_DETAILS1": SimStruct(OrderedDict((("saId", SimTypeLongLong(signed=False, label="UInt64")), ("keyModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("ikeTraffic", SimTypeRef("IKEEXT_TRAFFIC0", SimStruct)), ("ikeProposal", SimTypeRef("IKEEXT_PROPOSAL0", SimStruct)), ("cookiePair", SimTypeRef("IKEEXT_COOKIE_PAIR0", SimStruct)), ("ikeCredentials", SimTypeRef("IKEEXT_CREDENTIALS1", SimStruct)), ("ikePolicyKey", SimTypeBottom(label="Guid")), ("virtualIfTunnelId", SimTypeLongLong(signed=False, label="UInt64")), ("correlationKey", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_SA_DETAILS1", pack=False, align=None),
    "IKEEXT_SA_DETAILS2": SimStruct(OrderedDict((("saId", SimTypeLongLong(signed=False, label="UInt64")), ("keyModuleType", SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("ikeTraffic", SimTypeRef("IKEEXT_TRAFFIC0", SimStruct)), ("ikeProposal", SimTypeRef("IKEEXT_PROPOSAL0", SimStruct)), ("cookiePair", SimTypeRef("IKEEXT_COOKIE_PAIR0", SimStruct)), ("ikeCredentials", SimTypeRef("IKEEXT_CREDENTIALS2", SimStruct)), ("ikePolicyKey", SimTypeBottom(label="Guid")), ("virtualIfTunnelId", SimTypeLongLong(signed=False, label="UInt64")), ("correlationKey", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_SA_DETAILS2", pack=False, align=None),
    "IKEEXT_SA_ENUM_TEMPLATE0": SimStruct(OrderedDict((("localSubNet", SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)), ("remoteSubNet", SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)), ("localMainModeCertHash", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IKEEXT_SA_ENUM_TEMPLATE0", pack=False, align=None),
    "IKEEXT_SA_ROLE": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"),
    "IKEEXT_STATISTICS0": SimStruct(OrderedDict((("ikeStatistics", SimTypeRef("IKEEXT_KEYMODULE_STATISTICS0", SimStruct)), ("authipStatistics", SimTypeRef("IKEEXT_KEYMODULE_STATISTICS0", SimStruct)), ("commonStatistics", SimTypeRef("IKEEXT_COMMON_STATISTICS0", SimStruct)),)), name="IKEEXT_STATISTICS0", pack=False, align=None),
    "IKEEXT_STATISTICS1": SimStruct(OrderedDict((("ikeStatistics", SimTypeRef("IKEEXT_KEYMODULE_STATISTICS1", SimStruct)), ("authipStatistics", SimTypeRef("IKEEXT_KEYMODULE_STATISTICS1", SimStruct)), ("ikeV2Statistics", SimTypeRef("IKEEXT_KEYMODULE_STATISTICS1", SimStruct)), ("commonStatistics", SimTypeRef("IKEEXT_COMMON_STATISTICS1", SimStruct)),)), name="IKEEXT_STATISTICS1", pack=False, align=None),
    "IKEEXT_TRAFFIC0": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("authIpFilterId", SimTypeLongLong(signed=False, label="UInt64")),)), name="IKEEXT_TRAFFIC0", pack=False, align=None),
    "IKEV2_CONFIG_PARAMS": SimStruct(OrderedDict((("dwNumPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTunnelConfigParamFlags", SimTypeInt(signed=False, label="UInt32")), ("TunnelConfigParams", SimTypeRef("IKEV2_TUNNEL_CONFIG_PARAMS4", SimStruct)),)), name="IKEV2_CONFIG_PARAMS", pack=False, align=None),
    "IKEV2_ID_PAYLOAD_TYPE": SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE"),
    "IKEV2_PROJECTION_INFO": SimStruct(OrderedDict((("dwIPv4NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("IPv4SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwIPv6NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("bInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bRemoteInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bPrefix", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")), ("IPv6SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionMethod", SimTypeInt(signed=False, label="UInt32")),)), name="IKEV2_PROJECTION_INFO", pack=False, align=None),
    "IKEV2_PROJECTION_INFO2": SimStruct(OrderedDict((("dwIPv4NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("IPv4SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwIPv6NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("bInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bRemoteInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bPrefix", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")), ("IPv6SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwEmbeddedEAPTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionMethod", SimTypeInt(signed=False, label="UInt32")),)), name="IKEV2_PROJECTION_INFO2", pack=False, align=None),
    "IKEV2_TUNNEL_CONFIG_PARAMS2": SimStruct(OrderedDict((("dwIdleTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwNetworkBlackoutTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSizeForRenegotiation", SimTypeInt(signed=False, label="UInt32")), ("dwConfigOptions", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCertificates", SimTypeInt(signed=False, label="UInt32")), ("certificateNames", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("machineCertificateName", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)),)), name="IKEV2_TUNNEL_CONFIG_PARAMS2", pack=False, align=None),
    "IKEV2_TUNNEL_CONFIG_PARAMS3": SimStruct(OrderedDict((("dwIdleTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwNetworkBlackoutTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSizeForRenegotiation", SimTypeInt(signed=False, label="UInt32")), ("dwConfigOptions", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCertificates", SimTypeInt(signed=False, label="UInt32")), ("certificateNames", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("machineCertificateName", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)), ("dwTotalEkus", SimTypeInt(signed=False, label="UInt32")), ("certificateEKUs", SimTypePointer(SimTypeRef("MPR_CERT_EKU", SimStruct), offset=0)), ("machineCertificateHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="IKEV2_TUNNEL_CONFIG_PARAMS3", pack=False, align=None),
    "IKEV2_TUNNEL_CONFIG_PARAMS4": SimStruct(OrderedDict((("dwIdleTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwNetworkBlackoutTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSizeForRenegotiation", SimTypeInt(signed=False, label="UInt32")), ("dwConfigOptions", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCertificates", SimTypeInt(signed=False, label="UInt32")), ("certificateNames", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("machineCertificateName", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)), ("dwTotalEkus", SimTypeInt(signed=False, label="UInt32")), ("certificateEKUs", SimTypePointer(SimTypeRef("MPR_CERT_EKU", SimStruct), offset=0)), ("machineCertificateHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwMmSaLifeTime", SimTypeInt(signed=False, label="UInt32")),)), name="IKEV2_TUNNEL_CONFIG_PARAMS4", pack=False, align=None),
    "IKE_AUTHENTICATION_INFORMATION": SimStruct(OrderedDict((("AuthMethod", SimTypeInt(signed=False, label="IKE_AUTHENTICATION_METHOD")), ("Anonymous", SimUnion({"PsKey": SimTypeRef("IKE_AUTHENTICATION_PRESHARED_KEY", SimStruct)}, name="<anon>", label="None")),)), name="IKE_AUTHENTICATION_INFORMATION", pack=False, align=None),
    "IKE_AUTHENTICATION_METHOD": SimTypeInt(signed=False, label="IKE_AUTHENTICATION_METHOD"),
    "IKE_AUTHENTICATION_PRESHARED_KEY": SimStruct(OrderedDict((("SecurityFlags", SimTypeLongLong(signed=False, label="UInt64")), ("IdType", SimTypeChar(label="Byte")), ("IdLengthInBytes", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("KeyLengthInBytes", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="IKE_AUTHENTICATION_PRESHARED_KEY", pack=False, align=None),
    "IKernelTransaction": SimTypeBottom(label="IKernelTransaction"),
    "IKeyEnumerator": SimTypeBottom(label="IKeyEnumerator"),
    "IKeyStore": SimTypeBottom(label="IKeyStore"),
    "IKnowledgeSyncProvider": SimTypeBottom(label="IKnowledgeSyncProvider"),
    "IKnownFolder": SimTypeBottom(label="IKnownFolder"),
    "IKnownFolderManager": SimTypeBottom(label="IKnownFolderManager"),
    "IKsAggregateControl": SimTypeBottom(label="IKsAggregateControl"),
    "IKsAllocator": SimTypeBottom(label="IKsAllocator"),
    "IKsAllocatorEx": SimTypeBottom(label="IKsAllocatorEx"),
    "IKsClockPropertySet": SimTypeBottom(label="IKsClockPropertySet"),
    "IKsControl": SimTypeBottom(label="IKsControl"),
    "IKsDataTypeCompletion": SimTypeBottom(label="IKsDataTypeCompletion"),
    "IKsDataTypeHandler": SimTypeBottom(label="IKsDataTypeHandler"),
    "IKsFormatSupport": SimTypeBottom(label="IKsFormatSupport"),
    "IKsInterfaceHandler": SimTypeBottom(label="IKsInterfaceHandler"),
    "IKsJackContainerId": SimTypeBottom(label="IKsJackContainerId"),
    "IKsJackDescription": SimTypeBottom(label="IKsJackDescription"),
    "IKsJackDescription2": SimTypeBottom(label="IKsJackDescription2"),
    "IKsJackDescription3": SimTypeBottom(label="IKsJackDescription3"),
    "IKsJackSinkInformation": SimTypeBottom(label="IKsJackSinkInformation"),
    "IKsNodeControl": SimTypeBottom(label="IKsNodeControl"),
    "IKsNotifyEvent": SimTypeBottom(label="IKsNotifyEvent"),
    "IKsObject": SimTypeBottom(label="IKsObject"),
    "IKsPin": SimTypeBottom(label="IKsPin"),
    "IKsPinEx": SimTypeBottom(label="IKsPinEx"),
    "IKsPinFactory": SimTypeBottom(label="IKsPinFactory"),
    "IKsPinPipe": SimTypeBottom(label="IKsPinPipe"),
    "IKsPropertySet": SimTypeBottom(label="IKsPropertySet"),
    "IKsQualityForwarder": SimTypeBottom(label="IKsQualityForwarder"),
    "IKsTopology": SimTypeBottom(label="IKsTopology"),
    "IKsTopologyInfo": SimTypeBottom(label="IKsTopologyInfo"),
    "ILBEvents": SimTypeBottom(label="ILBEvents"),
    "ILanguageComponentType": SimTypeBottom(label="ILanguageComponentType"),
    "ILastResourceManager": SimTypeBottom(label="ILastResourceManager"),
    "ILatLongReport": SimTypeBottom(label="ILatLongReport"),
    "ILatLongReportFactory": SimTypeBottom(label="ILatLongReportFactory"),
    "ILaunchSourceAppUserModelId": SimTypeBottom(label="ILaunchSourceAppUserModelId"),
    "ILaunchSourceViewSizePreference": SimTypeBottom(label="ILaunchSourceViewSizePreference"),
    "ILaunchTargetMonitor": SimTypeBottom(label="ILaunchTargetMonitor"),
    "ILaunchTargetViewSizePreference": SimTypeBottom(label="ILaunchTargetViewSizePreference"),
    "ILaunchUIContext": SimTypeBottom(label="ILaunchUIContext"),
    "ILaunchUIContextProvider": SimTypeBottom(label="ILaunchUIContextProvider"),
    "ILayoutRect": SimTypeBottom(label="ILayoutRect"),
    "ILayoutStorage": SimTypeBottom(label="ILayoutStorage"),
    "ILegacyIAccessibleProvider": SimTypeBottom(label="ILegacyIAccessibleProvider"),
    "ILineInfo": SimTypeBottom(label="ILineInfo"),
    "ILoadChangeContext": SimTypeBottom(label="ILoadChangeContext"),
    "ILoadFilter": SimTypeBottom(label="ILoadFilter"),
    "ILoadFilterWithPrivateComActivation": SimTypeBottom(label="ILoadFilterWithPrivateComActivation"),
    "ILocation": SimTypeBottom(label="ILocation"),
    "ILocationEvents": SimTypeBottom(label="ILocationEvents"),
    "ILocationPermissions": SimTypeBottom(label="ILocationPermissions"),
    "ILocationPower": SimTypeBottom(label="ILocationPower"),
    "ILocationReport": SimTypeBottom(label="ILocationReport"),
    "ILocationReportFactory": SimTypeBottom(label="ILocationReportFactory"),
    "ILocator": SimTypeBottom(label="ILocator"),
    "ILockBytes": SimTypeBottom(label="ILockBytes"),
    "ILogFileItem": SimTypeBottom(label="ILogFileItem"),
    "ILogFiles": SimTypeBottom(label="ILogFiles"),
    "ILogonTrigger": SimTypeBottom(label="ILogonTrigger"),
    "IMAADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="IMAADPCMWAVEFORMAT", pack=False, align=None),
    "IMAGEHLP_CBA_EVENT": SimStruct(OrderedDict((("severity", SimTypeInt(signed=False, label="IMAGEHLP_CBA_EVENT_SEVERITY")), ("code", SimTypeInt(signed=False, label="UInt32")), ("desc", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IMAGEHLP_CBA_EVENT", pack=False, align=None),
    "IMAGEHLP_CBA_EVENTW": SimStruct(OrderedDict((("severity", SimTypeInt(signed=False, label="IMAGEHLP_CBA_EVENT_SEVERITY")), ("code", SimTypeInt(signed=False, label="UInt32")), ("desc", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IMAGEHLP_CBA_EVENTW", pack=False, align=None),
    "IMAGEHLP_CBA_EVENT_SEVERITY": SimTypeInt(signed=False, label="IMAGEHLP_CBA_EVENT_SEVERITY"),
    "IMAGEHLP_CBA_READ_MEMORY": SimStruct(OrderedDict((("addr", SimTypeLongLong(signed=False, label="UInt64")), ("buf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("bytes", SimTypeInt(signed=False, label="UInt32")), ("bytesread", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IMAGEHLP_CBA_READ_MEMORY", pack=False, align=None),
    "IMAGEHLP_DEFERRED_SYMBOL_LOAD": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Reparse", SimTypeChar(label="Byte")), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMAGEHLP_DEFERRED_SYMBOL_LOAD", pack=False, align=None),
    "IMAGEHLP_DEFERRED_SYMBOL_LOAD64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Reparse", SimTypeChar(label="Byte")), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_DEFERRED_SYMBOL_LOAD64", pack=False, align=None),
    "IMAGEHLP_DEFERRED_SYMBOL_LOADW64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypeArray(SimTypeChar(label="Char"), 261)), ("Reparse", SimTypeChar(label="Byte")), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_DEFERRED_SYMBOL_LOADW64", pack=False, align=None),
    "IMAGEHLP_DUPLICATE_SYMBOL": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("NumberOfDups", SimTypeInt(signed=False, label="UInt32")), ("Symbol", SimTypePointer(SimTypeRef("IMAGEHLP_SYMBOL", SimStruct), offset=0)), ("SelectedSymbol", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_DUPLICATE_SYMBOL", pack=False, align=None),
    "IMAGEHLP_DUPLICATE_SYMBOL64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("NumberOfDups", SimTypeInt(signed=False, label="UInt32")), ("Symbol", SimTypePointer(SimTypeRef("IMAGEHLP_SYMBOL64", SimStruct), offset=0)), ("SelectedSymbol", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_DUPLICATE_SYMBOL64", pack=False, align=None),
    "IMAGEHLP_EXTENDED_OPTIONS": SimTypeInt(signed=False, label="IMAGEHLP_EXTENDED_OPTIONS"),
    "IMAGEHLP_GET_TYPE_INFO_FLAGS": SimTypeInt(signed=False, label="IMAGEHLP_GET_TYPE_INFO_FLAGS"),
    "IMAGEHLP_GET_TYPE_INFO_PARAMS": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="IMAGEHLP_GET_TYPE_INFO_FLAGS")), ("NumIds", SimTypeInt(signed=False, label="UInt32")), ("TypeIds", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("TagFilter", SimTypeLongLong(signed=False, label="UInt64")), ("NumReqs", SimTypeInt(signed=False, label="UInt32")), ("ReqKinds", SimTypePointer(SimTypeInt(signed=False, label="IMAGEHLP_SYMBOL_TYPE_INFO"), offset=0)), ("ReqOffsets", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)), ("ReqSizes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ReqStride", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EntriesMatched", SimTypeInt(signed=False, label="UInt32")), ("EntriesFilled", SimTypeInt(signed=False, label="UInt32")), ("TagsFound", SimTypeLongLong(signed=False, label="UInt64")), ("AllReqsValid", SimTypeLongLong(signed=False, label="UInt64")), ("NumReqsValid", SimTypeInt(signed=False, label="UInt32")), ("ReqsValid", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="IMAGEHLP_GET_TYPE_INFO_PARAMS", pack=False, align=None),
    "IMAGEHLP_HD_TYPE": SimTypeInt(signed=False, label="IMAGEHLP_HD_TYPE"),
    "IMAGEHLP_JIT_SYMBOLMAP": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")),)), name="IMAGEHLP_JIT_SYMBOLMAP", pack=False, align=None),
    "IMAGEHLP_LINE": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Address", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_LINE", pack=False, align=None),
    "IMAGEHLP_LINE64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="IMAGEHLP_LINE64", pack=False, align=None),
    "IMAGEHLP_LINEW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="IMAGEHLP_LINEW", pack=False, align=None),
    "IMAGEHLP_LINEW64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="IMAGEHLP_LINEW64", pack=False, align=None),
    "IMAGEHLP_MODULE": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeInt(signed=False, label="UInt32")), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("NumSyms", SimTypeInt(signed=False, label="UInt32")), ("SymType", SimTypeInt(signed=False, label="SYM_TYPE")), ("ModuleName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("ImageName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("LoadedImageName", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="IMAGEHLP_MODULE", pack=False, align=None),
    "IMAGEHLP_MODULE64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("NumSyms", SimTypeInt(signed=False, label="UInt32")), ("SymType", SimTypeInt(signed=False, label="SYM_TYPE")), ("ModuleName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("ImageName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("LoadedImageName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("LoadedPdbName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("CVSig", SimTypeInt(signed=False, label="UInt32")), ("CVData", SimTypeArray(SimTypeChar(label="SByte"), 780)), ("PdbSig", SimTypeInt(signed=False, label="UInt32")), ("PdbSig70", SimTypeBottom(label="Guid")), ("PdbAge", SimTypeInt(signed=False, label="UInt32")), ("PdbUnmatched", SimTypeInt(signed=True, label="Int32")), ("DbgUnmatched", SimTypeInt(signed=True, label="Int32")), ("LineNumbers", SimTypeInt(signed=True, label="Int32")), ("GlobalSymbols", SimTypeInt(signed=True, label="Int32")), ("TypeInfo", SimTypeInt(signed=True, label="Int32")), ("SourceIndexed", SimTypeInt(signed=True, label="Int32")), ("Publics", SimTypeInt(signed=True, label="Int32")), ("MachineType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_MODULE64", pack=False, align=None),
    "IMAGEHLP_MODULE64_EX": SimStruct(OrderedDict((("Module", SimTypeRef("IMAGEHLP_MODULE64", SimStruct)), ("RegionFlags", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_MODULE64_EX", pack=False, align=None),
    "IMAGEHLP_MODULEW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeInt(signed=False, label="UInt32")), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("NumSyms", SimTypeInt(signed=False, label="UInt32")), ("SymType", SimTypeInt(signed=False, label="SYM_TYPE")), ("ModuleName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("ImageName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("LoadedImageName", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="IMAGEHLP_MODULEW", pack=False, align=None),
    "IMAGEHLP_MODULEW64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("NumSyms", SimTypeInt(signed=False, label="UInt32")), ("SymType", SimTypeInt(signed=False, label="SYM_TYPE")), ("ModuleName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("ImageName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("LoadedImageName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("LoadedPdbName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("CVSig", SimTypeInt(signed=False, label="UInt32")), ("CVData", SimTypeArray(SimTypeChar(label="Char"), 780)), ("PdbSig", SimTypeInt(signed=False, label="UInt32")), ("PdbSig70", SimTypeBottom(label="Guid")), ("PdbAge", SimTypeInt(signed=False, label="UInt32")), ("PdbUnmatched", SimTypeInt(signed=True, label="Int32")), ("DbgUnmatched", SimTypeInt(signed=True, label="Int32")), ("LineNumbers", SimTypeInt(signed=True, label="Int32")), ("GlobalSymbols", SimTypeInt(signed=True, label="Int32")), ("TypeInfo", SimTypeInt(signed=True, label="Int32")), ("SourceIndexed", SimTypeInt(signed=True, label="Int32")), ("Publics", SimTypeInt(signed=True, label="Int32")), ("MachineType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_MODULEW64", pack=False, align=None),
    "IMAGEHLP_MODULEW64_EX": SimStruct(OrderedDict((("Module", SimTypeRef("IMAGEHLP_MODULEW64", SimStruct)), ("RegionFlags", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_MODULEW64_EX", pack=False, align=None),
    "IMAGEHLP_SF_TYPE": SimTypeInt(signed=False, label="IMAGEHLP_SF_TYPE"),
    "IMAGEHLP_STACK_FRAME": SimStruct(OrderedDict((("InstructionOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ReturnOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FrameOffset", SimTypeLongLong(signed=False, label="UInt64")), ("StackOffset", SimTypeLongLong(signed=False, label="UInt64")), ("BackingStoreOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FuncTableEntry", SimTypeLongLong(signed=False, label="UInt64")), ("Params", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 5)), ("Virtual", SimTypeInt(signed=True, label="Int32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGEHLP_STACK_FRAME", pack=False, align=None),
    "IMAGEHLP_STATUS_REASON": SimTypeInt(signed=False, label="IMAGEHLP_STATUS_REASON"),
    "IMAGEHLP_SYMBOL": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="IMAGEHLP_SYMBOL", pack=False, align=None),
    "IMAGEHLP_SYMBOL64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="IMAGEHLP_SYMBOL64", pack=False, align=None),
    "IMAGEHLP_SYMBOL64_PACKAGE": SimStruct(OrderedDict((("sym", SimTypeRef("IMAGEHLP_SYMBOL64", SimStruct)), ("name", SimTypeArray(SimTypeChar(label="SByte"), 2001)),)), name="IMAGEHLP_SYMBOL64_PACKAGE", pack=False, align=None),
    "IMAGEHLP_SYMBOLW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IMAGEHLP_SYMBOLW", pack=False, align=None),
    "IMAGEHLP_SYMBOLW64": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IMAGEHLP_SYMBOLW64", pack=False, align=None),
    "IMAGEHLP_SYMBOLW64_PACKAGE": SimStruct(OrderedDict((("sym", SimTypeRef("IMAGEHLP_SYMBOLW64", SimStruct)), ("name", SimTypeArray(SimTypeChar(label="Char"), 2001)),)), name="IMAGEHLP_SYMBOLW64_PACKAGE", pack=False, align=None),
    "IMAGEHLP_SYMBOLW_PACKAGE": SimStruct(OrderedDict((("sym", SimTypeRef("IMAGEHLP_SYMBOLW", SimStruct)), ("name", SimTypeArray(SimTypeChar(label="Char"), 2001)),)), name="IMAGEHLP_SYMBOLW_PACKAGE", pack=False, align=None),
    "IMAGEHLP_SYMBOL_PACKAGE": SimStruct(OrderedDict((("sym", SimTypeRef("IMAGEHLP_SYMBOL", SimStruct)), ("name", SimTypeArray(SimTypeChar(label="SByte"), 2001)),)), name="IMAGEHLP_SYMBOL_PACKAGE", pack=False, align=None),
    "IMAGEHLP_SYMBOL_SRC": SimStruct(OrderedDict((("sizeofstruct", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="UInt32")), ("file", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="IMAGEHLP_SYMBOL_SRC", pack=False, align=None),
    "IMAGEHLP_SYMBOL_TYPE_INFO": SimTypeInt(signed=False, label="IMAGEHLP_SYMBOL_TYPE_INFO"),
    "IMAGEINFO": SimStruct(OrderedDict((("hbmImage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmMask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Unused1", SimTypeInt(signed=True, label="Int32")), ("Unused2", SimTypeInt(signed=True, label="Int32")), ("rcImage", SimTypeRef("RECT", SimStruct)),)), name="IMAGEINFO", pack=False, align=None),
    "IMAGELAYOUT": SimTypeInt(signed=False, label="IMAGELAYOUT"),
    "IMAGELISTDRAWPARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("himl", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("i", SimTypeInt(signed=True, label="Int32")), ("hdcDst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")), ("xBitmap", SimTypeInt(signed=True, label="Int32")), ("yBitmap", SimTypeInt(signed=True, label="Int32")), ("rgbBk", SimTypeInt(signed=False, label="UInt32")), ("rgbFg", SimTypeInt(signed=False, label="UInt32")), ("fStyle", SimTypeInt(signed=False, label="UInt32")), ("dwRop", SimTypeInt(signed=False, label="UInt32")), ("fState", SimTypeInt(signed=False, label="UInt32")), ("Frame", SimTypeInt(signed=False, label="UInt32")), ("crEffect", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGELISTDRAWPARAMS", pack=False, align=None),
    "IMAGELISTSTATS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cAlloc", SimTypeInt(signed=True, label="Int32")), ("cUsed", SimTypeInt(signed=True, label="Int32")), ("cStandby", SimTypeInt(signed=True, label="Int32")),)), name="IMAGELISTSTATS", pack=False, align=None),
    "IMAGELIST_CREATION_FLAGS": SimTypeInt(signed=False, label="IMAGELIST_CREATION_FLAGS"),
    "IMAGESELECTTYPE": SimTypeInt(signed=False, label="IMAGESELECTTYPE"),
    "IMAGE_ALPHA64_RUNTIME_FUNCTION_ENTRY": SimStruct(OrderedDict((("BeginAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EndAddress", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionHandler", SimTypeLongLong(signed=False, label="UInt64")), ("HandlerData", SimTypeLongLong(signed=False, label="UInt64")), ("PrologEndAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="IMAGE_ALPHA64_RUNTIME_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_ALPHA_RUNTIME_FUNCTION_ENTRY": SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")), ("ExceptionHandler", SimTypeInt(signed=False, label="UInt32")), ("HandlerData", SimTypeInt(signed=False, label="UInt32")), ("PrologEndAddress", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_ALPHA_RUNTIME_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_ARCHITECTURE_ENTRY": SimStruct(OrderedDict((("FixupInstRVA", SimTypeInt(signed=False, label="UInt32")), ("NewInst", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_ARCHITECTURE_ENTRY", pack=False, align=None),
    "IMAGE_ARCHITECTURE_HEADER": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("FirstEntryRVA", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_ARCHITECTURE_HEADER", pack=False, align=None),
    "IMAGE_ARCHIVE_MEMBER_HEADER": SimStruct(OrderedDict((("Name", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Date", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("UserID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("GroupID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Mode", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Size", SimTypeArray(SimTypeChar(label="Byte"), 10)), ("EndHeader", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="IMAGE_ARCHIVE_MEMBER_HEADER", pack=False, align=None),
    "IMAGE_ARM64_RUNTIME_FUNCTION_ENTRY": SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"UnwindData": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="IMAGE_ARM64_RUNTIME_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_ARM64_RUNTIME_FUNCTION_ENTRY_XDATA": SimUnion({"HeaderData": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "IMAGE_ARM_RUNTIME_FUNCTION_ENTRY": SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"UnwindData": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="IMAGE_ARM_RUNTIME_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_AUX_SYMBOL": SimUnion({"Sym": SimTypeRef("_Sym_e__Struct", SimStruct), "File": SimTypeRef("_File_e__Struct", SimStruct), "Section": SimTypeRef("_Section_e__Struct", SimStruct), "TokenDef": SimTypeRef("IMAGE_AUX_SYMBOL_TOKEN_DEF", SimStruct), "CRC": SimTypeRef("_CRC_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "IMAGE_AUX_SYMBOL_EX": SimUnion({"Sym": SimTypeRef("_Sym_e__Struct", SimStruct), "File": SimTypeRef("_File_e__Struct", SimStruct), "Section": SimTypeRef("_Section_e__Struct", SimStruct), "Anonymous": SimStruct(OrderedDict((("TokenDef", SimTypeRef("IMAGE_AUX_SYMBOL_TOKEN_DEF", SimStruct)), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="_Anonymous_e__Struct", pack=False, align=None), "CRC": SimTypeRef("_CRC_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "IMAGE_AUX_SYMBOL_TOKEN_DEF": SimStruct(OrderedDict((("bAuxType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("SymbolTableIndex", SimTypeInt(signed=False, label="UInt32")), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 12)),)), name="IMAGE_AUX_SYMBOL_TOKEN_DEF", pack=False, align=None),
    "IMAGE_AUX_SYMBOL_TYPE": SimTypeInt(signed=False, label="IMAGE_AUX_SYMBOL_TYPE"),
    "IMAGE_BASE_RELOCATION": SimStruct(OrderedDict((("VirtualAddress", SimTypeInt(signed=False, label="UInt32")), ("SizeOfBlock", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_BASE_RELOCATION", pack=False, align=None),
    "IMAGE_BDD_DYNAMIC_RELOCATION": SimStruct(OrderedDict((("Left", SimTypeShort(signed=False, label="UInt16")), ("Right", SimTypeShort(signed=False, label="UInt16")), ("Value", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_BDD_DYNAMIC_RELOCATION", pack=False, align=None),
    "IMAGE_BDD_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("BDDSize", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_BDD_INFO", pack=False, align=None),
    "IMAGE_BOUND_FORWARDER_REF": SimStruct(OrderedDict((("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("OffsetModuleName", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_BOUND_FORWARDER_REF", pack=False, align=None),
    "IMAGE_BOUND_IMPORT_DESCRIPTOR": SimStruct(OrderedDict((("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("OffsetModuleName", SimTypeShort(signed=False, label="UInt16")), ("NumberOfModuleForwarderRefs", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_BOUND_IMPORT_DESCRIPTOR", pack=False, align=None),
    "IMAGE_CE_RUNTIME_FUNCTION_ENTRY": SimStruct(OrderedDict((("FuncStart", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_CE_RUNTIME_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_COFF_SYMBOLS_HEADER": SimStruct(OrderedDict((("NumberOfSymbols", SimTypeInt(signed=False, label="UInt32")), ("LvaToFirstSymbol", SimTypeInt(signed=False, label="UInt32")), ("NumberOfLinenumbers", SimTypeInt(signed=False, label="UInt32")), ("LvaToFirstLinenumber", SimTypeInt(signed=False, label="UInt32")), ("RvaToFirstByteOfCode", SimTypeInt(signed=False, label="UInt32")), ("RvaToLastByteOfCode", SimTypeInt(signed=False, label="UInt32")), ("RvaToFirstByteOfData", SimTypeInt(signed=False, label="UInt32")), ("RvaToLastByteOfData", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_COFF_SYMBOLS_HEADER", pack=False, align=None),
    "IMAGE_COR20_HEADER": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("MajorRuntimeVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorRuntimeVersion", SimTypeShort(signed=False, label="UInt16")), ("MetaData", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"EntryPointToken": SimTypeInt(signed=False, label="UInt32"), "EntryPointRVA": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Resources", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)), ("StrongNameSignature", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)), ("CodeManagerTable", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)), ("VTableFixups", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)), ("ExportAddressTableJumps", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)), ("ManagedNativeHeader", SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct)),)), name="IMAGE_COR20_HEADER", pack=False, align=None),
    "IMAGE_DATA_DIRECTORY": SimStruct(OrderedDict((("VirtualAddress", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DATA_DIRECTORY", pack=False, align=None),
    "IMAGE_DEBUG_DIRECTORY": SimStruct(OrderedDict((("Characteristics", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeInt(signed=False, label="IMAGE_DEBUG_TYPE")), ("SizeOfData", SimTypeInt(signed=False, label="UInt32")), ("AddressOfRawData", SimTypeInt(signed=False, label="UInt32")), ("PointerToRawData", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DEBUG_DIRECTORY", pack=False, align=None),
    "IMAGE_DEBUG_INFORMATION": SimStruct(OrderedDict((("List", SimTypeRef("LIST_ENTRY", SimStruct)), ("ReservedSize", SimTypeInt(signed=False, label="UInt32")), ("ReservedMappedBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ReservedMachine", SimTypeShort(signed=False, label="UInt16")), ("ReservedCharacteristics", SimTypeShort(signed=False, label="UInt16")), ("ReservedCheckSum", SimTypeInt(signed=False, label="UInt32")), ("ImageBase", SimTypeInt(signed=False, label="UInt32")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("ReservedNumberOfSections", SimTypeInt(signed=False, label="UInt32")), ("ReservedSections", SimTypePointer(SimTypeRef("IMAGE_SECTION_HEADER", SimStruct), offset=0)), ("ReservedExportedNamesSize", SimTypeInt(signed=False, label="UInt32")), ("ReservedExportedNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ReservedNumberOfFunctionTableEntries", SimTypeInt(signed=False, label="UInt32")), ("ReservedFunctionTableEntries", SimTypePointer(SimTypeRef("IMAGE_FUNCTION_ENTRY", SimStruct), offset=0)), ("ReservedLowestFunctionStartingAddress", SimTypeInt(signed=False, label="UInt32")), ("ReservedHighestFunctionEndingAddress", SimTypeInt(signed=False, label="UInt32")), ("ReservedNumberOfFpoTableEntries", SimTypeInt(signed=False, label="UInt32")), ("ReservedFpoTableEntries", SimTypePointer(SimTypeRef("FPO_DATA", SimStruct), offset=0)), ("SizeOfCoffSymbols", SimTypeInt(signed=False, label="UInt32")), ("CoffSymbols", SimTypePointer(SimTypeRef("IMAGE_COFF_SYMBOLS_HEADER", SimStruct), offset=0)), ("ReservedSizeOfCodeViewSymbols", SimTypeInt(signed=False, label="UInt32")), ("ReservedCodeViewSymbols", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ImageFilePath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ImageFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ReservedDebugFilePath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ReservedTimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ReservedRomImage", SimTypeInt(signed=True, label="Int32")), ("ReservedDebugDirectory", SimTypePointer(SimTypeRef("IMAGE_DEBUG_DIRECTORY", SimStruct), offset=0)), ("ReservedNumberOfDebugDirectories", SimTypeInt(signed=False, label="UInt32")), ("ReservedOriginalFunctionTableBaseAddress", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="IMAGE_DEBUG_INFORMATION", pack=False, align=None),
    "IMAGE_DEBUG_MISC": SimStruct(OrderedDict((("DataType", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Unicode", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="IMAGE_DEBUG_MISC", pack=False, align=None),
    "IMAGE_DEBUG_TYPE": SimTypeInt(signed=False, label="IMAGE_DEBUG_TYPE"),
    "IMAGE_DELAYLOAD_DESCRIPTOR": SimStruct(OrderedDict((("Attributes", SimUnion({"AllAttributes": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("DllNameRVA", SimTypeInt(signed=False, label="UInt32")), ("ModuleHandleRVA", SimTypeInt(signed=False, label="UInt32")), ("ImportAddressTableRVA", SimTypeInt(signed=False, label="UInt32")), ("ImportNameTableRVA", SimTypeInt(signed=False, label="UInt32")), ("BoundImportAddressTableRVA", SimTypeInt(signed=False, label="UInt32")), ("UnloadInformationTableRVA", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DELAYLOAD_DESCRIPTOR", pack=False, align=None),
    "IMAGE_DIRECTORY_ENTRY": SimTypeInt(signed=False, label="IMAGE_DIRECTORY_ENTRY"),
    "IMAGE_DLL_CHARACTERISTICS": SimTypeInt(signed=False, label="IMAGE_DLL_CHARACTERISTICS"),
    "IMAGE_DOS_HEADER": SimStruct(OrderedDict((("e_magic", SimTypeShort(signed=False, label="UInt16")), ("e_cblp", SimTypeShort(signed=False, label="UInt16")), ("e_cp", SimTypeShort(signed=False, label="UInt16")), ("e_crlc", SimTypeShort(signed=False, label="UInt16")), ("e_cparhdr", SimTypeShort(signed=False, label="UInt16")), ("e_minalloc", SimTypeShort(signed=False, label="UInt16")), ("e_maxalloc", SimTypeShort(signed=False, label="UInt16")), ("e_ss", SimTypeShort(signed=False, label="UInt16")), ("e_sp", SimTypeShort(signed=False, label="UInt16")), ("e_csum", SimTypeShort(signed=False, label="UInt16")), ("e_ip", SimTypeShort(signed=False, label="UInt16")), ("e_cs", SimTypeShort(signed=False, label="UInt16")), ("e_lfarlc", SimTypeShort(signed=False, label="UInt16")), ("e_ovno", SimTypeShort(signed=False, label="UInt16")), ("e_res", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 4)), ("e_oemid", SimTypeShort(signed=False, label="UInt16")), ("e_oeminfo", SimTypeShort(signed=False, label="UInt16")), ("e_res2", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 10)), ("e_lfanew", SimTypeInt(signed=True, label="Int32")),)), name="IMAGE_DOS_HEADER", pack=False, align=None),
    "IMAGE_DYNAMIC_RELOCATION32": SimStruct(OrderedDict((("Symbol", SimTypeInt(signed=False, label="UInt32")), ("BaseRelocSize", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DYNAMIC_RELOCATION32", pack=False, align=None),
    "IMAGE_DYNAMIC_RELOCATION32_V2": SimStruct(OrderedDict((("HeaderSize", SimTypeInt(signed=False, label="UInt32")), ("FixupInfoSize", SimTypeInt(signed=False, label="UInt32")), ("Symbol", SimTypeInt(signed=False, label="UInt32")), ("SymbolGroup", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DYNAMIC_RELOCATION32_V2", pack=False, align=None),
    "IMAGE_DYNAMIC_RELOCATION64": SimStruct(OrderedDict((("Symbol", SimTypeLongLong(signed=False, label="UInt64")), ("BaseRelocSize", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DYNAMIC_RELOCATION64", pack=False, align=None),
    "IMAGE_DYNAMIC_RELOCATION64_V2": SimStruct(OrderedDict((("HeaderSize", SimTypeInt(signed=False, label="UInt32")), ("FixupInfoSize", SimTypeInt(signed=False, label="UInt32")), ("Symbol", SimTypeLongLong(signed=False, label="UInt64")), ("SymbolGroup", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DYNAMIC_RELOCATION64_V2", pack=False, align=None),
    "IMAGE_DYNAMIC_RELOCATION_TABLE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_DYNAMIC_RELOCATION_TABLE", pack=False, align=None),
    "IMAGE_EPILOGUE_DYNAMIC_RELOCATION_HEADER": SimStruct(OrderedDict((("EpilogueCount", SimTypeInt(signed=False, label="UInt32")), ("EpilogueByteCount", SimTypeChar(label="Byte")), ("BranchDescriptorElementSize", SimTypeChar(label="Byte")), ("BranchDescriptorCount", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_EPILOGUE_DYNAMIC_RELOCATION_HEADER", pack=False, align=None),
    "IMAGE_EXPORT_DIRECTORY": SimStruct(OrderedDict((("Characteristics", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("Name", SimTypeInt(signed=False, label="UInt32")), ("Base", SimTypeInt(signed=False, label="UInt32")), ("NumberOfFunctions", SimTypeInt(signed=False, label="UInt32")), ("NumberOfNames", SimTypeInt(signed=False, label="UInt32")), ("AddressOfFunctions", SimTypeInt(signed=False, label="UInt32")), ("AddressOfNames", SimTypeInt(signed=False, label="UInt32")), ("AddressOfNameOrdinals", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_EXPORT_DIRECTORY", pack=False, align=None),
    "IMAGE_FILE_CHARACTERISTICS": SimTypeInt(signed=False, label="IMAGE_FILE_CHARACTERISTICS"),
    "IMAGE_FILE_CHARACTERISTICS2": SimTypeInt(signed=False, label="IMAGE_FILE_CHARACTERISTICS2"),
    "IMAGE_FILE_HEADER": SimStruct(OrderedDict((("Machine", SimTypeInt(signed=False, label="IMAGE_FILE_MACHINE")), ("NumberOfSections", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("PointerToSymbolTable", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSymbols", SimTypeInt(signed=False, label="UInt32")), ("SizeOfOptionalHeader", SimTypeShort(signed=False, label="UInt16")), ("Characteristics", SimTypeInt(signed=False, label="IMAGE_FILE_CHARACTERISTICS")),)), name="IMAGE_FILE_HEADER", pack=False, align=None),
    "IMAGE_FILE_MACHINE": SimTypeInt(signed=False, label="IMAGE_FILE_MACHINE"),
    "IMAGE_FLAGS": SimTypeInt(signed=False, label="IMAGE_FLAGS"),
    "IMAGE_FUNCTION_ENTRY": SimStruct(OrderedDict((("StartingAddress", SimTypeInt(signed=False, label="UInt32")), ("EndingAddress", SimTypeInt(signed=False, label="UInt32")), ("EndOfPrologue", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_FUNCTION_ENTRY64": SimStruct(OrderedDict((("StartingAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EndingAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"EndOfPrologue": SimTypeLongLong(signed=False, label="UInt64"), "UnwindInfoAddress": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="IMAGE_FUNCTION_ENTRY64", pack=False, align=None),
    "IMAGE_FUNCTION_OVERRIDE_DYNAMIC_RELOCATION": SimStruct(OrderedDict((("OriginalRva", SimTypeInt(signed=False, label="UInt32")), ("BDDOffset", SimTypeInt(signed=False, label="UInt32")), ("RvaSize", SimTypeInt(signed=False, label="UInt32")), ("BaseRelocSize", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_FUNCTION_OVERRIDE_DYNAMIC_RELOCATION", pack=False, align=None),
    "IMAGE_FUNCTION_OVERRIDE_HEADER": SimStruct(OrderedDict((("FuncOverrideSize", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_FUNCTION_OVERRIDE_HEADER", pack=False, align=None),
    "IMAGE_HOT_PATCH_BASE": SimStruct(OrderedDict((("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("OriginalTimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("OriginalCheckSum", SimTypeInt(signed=False, label="UInt32")), ("CodeIntegrityInfo", SimTypeInt(signed=False, label="UInt32")), ("CodeIntegritySize", SimTypeInt(signed=False, label="UInt32")), ("PatchTable", SimTypeInt(signed=False, label="UInt32")), ("BufferOffset", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_HOT_PATCH_BASE", pack=False, align=None),
    "IMAGE_HOT_PATCH_HASHES": SimStruct(OrderedDict((("SHA256", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("SHA1", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="IMAGE_HOT_PATCH_HASHES", pack=False, align=None),
    "IMAGE_HOT_PATCH_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("BaseImageList", SimTypeInt(signed=False, label="UInt32")), ("BaseImageCount", SimTypeInt(signed=False, label="UInt32")), ("BufferOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtraPatchSize", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_HOT_PATCH_INFO", pack=False, align=None),
    "IMAGE_IMPORT_BY_NAME": SimStruct(OrderedDict((("Hint", SimTypeShort(signed=False, label="UInt16")), ("Name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="IMAGE_IMPORT_BY_NAME", pack=False, align=None),
    "IMAGE_IMPORT_CONTROL_TRANSFER_DYNAMIC_RELOCATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_IMPORT_CONTROL_TRANSFER_DYNAMIC_RELOCATION", pack=False, align=None),
    "IMAGE_IMPORT_DESCRIPTOR": SimStruct(OrderedDict((("Anonymous", SimUnion({"Characteristics": SimTypeInt(signed=False, label="UInt32"), "OriginalFirstThunk": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ForwarderChain", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypeInt(signed=False, label="UInt32")), ("FirstThunk", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_IMPORT_DESCRIPTOR", pack=False, align=None),
    "IMAGE_INDIR_CONTROL_TRANSFER_DYNAMIC_RELOCATION": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_INDIR_CONTROL_TRANSFER_DYNAMIC_RELOCATION", pack=False, align=None),
    "IMAGE_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"Properties": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("ImageBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ImageSelector", SimTypeInt(signed=False, label="UInt32")), ("ImageSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ImageSectionNumber", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_INFO", pack=False, align=None),
    "IMAGE_INFO_EX": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ImageInfo", SimTypeRef("IMAGE_INFO", SimStruct)), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)),)), name="IMAGE_INFO_EX", pack=False, align=None),
    "IMAGE_LINENUMBER": SimStruct(OrderedDict((("Type", SimUnion({"SymbolTableIndex": SimTypeInt(signed=False, label="UInt32"), "VirtualAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Linenumber", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_LINENUMBER", pack=False, align=None),
    "IMAGE_LIST_COPY_FLAGS": SimTypeInt(signed=False, label="IMAGE_LIST_COPY_FLAGS"),
    "IMAGE_LIST_DRAW_STYLE": SimTypeInt(signed=False, label="IMAGE_LIST_DRAW_STYLE"),
    "IMAGE_LIST_ITEM_FLAGS": SimTypeInt(signed=False, label="IMAGE_LIST_ITEM_FLAGS"),
    "IMAGE_LIST_WRITE_STREAM_FLAGS": SimTypeInt(signed=False, label="IMAGE_LIST_WRITE_STREAM_FLAGS"),
    "IMAGE_LOAD_CONFIG_CODE_INTEGRITY": SimStruct(OrderedDict((("Flags", SimTypeShort(signed=False, label="UInt16")), ("Catalog", SimTypeShort(signed=False, label="UInt16")), ("CatalogOffset", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_LOAD_CONFIG_CODE_INTEGRITY", pack=False, align=None),
    "IMAGE_LOAD_CONFIG_DIRECTORY32": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("GlobalFlagsClear", SimTypeInt(signed=False, label="UInt32")), ("GlobalFlagsSet", SimTypeInt(signed=False, label="UInt32")), ("CriticalSectionDefaultTimeout", SimTypeInt(signed=False, label="UInt32")), ("DeCommitFreeBlockThreshold", SimTypeInt(signed=False, label="UInt32")), ("DeCommitTotalFreeThreshold", SimTypeInt(signed=False, label="UInt32")), ("LockPrefixTable", SimTypeInt(signed=False, label="UInt32")), ("MaximumAllocationSize", SimTypeInt(signed=False, label="UInt32")), ("VirtualMemoryThreshold", SimTypeInt(signed=False, label="UInt32")), ("ProcessHeapFlags", SimTypeInt(signed=False, label="UInt32")), ("ProcessAffinityMask", SimTypeInt(signed=False, label="UInt32")), ("CSDVersion", SimTypeShort(signed=False, label="UInt16")), ("DependentLoadFlags", SimTypeShort(signed=False, label="UInt16")), ("EditList", SimTypeInt(signed=False, label="UInt32")), ("SecurityCookie", SimTypeInt(signed=False, label="UInt32")), ("SEHandlerTable", SimTypeInt(signed=False, label="UInt32")), ("SEHandlerCount", SimTypeInt(signed=False, label="UInt32")), ("GuardCFCheckFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("GuardCFDispatchFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("GuardCFFunctionTable", SimTypeInt(signed=False, label="UInt32")), ("GuardCFFunctionCount", SimTypeInt(signed=False, label="UInt32")), ("GuardFlags", SimTypeInt(signed=False, label="UInt32")), ("CodeIntegrity", SimTypeRef("IMAGE_LOAD_CONFIG_CODE_INTEGRITY", SimStruct)), ("GuardAddressTakenIatEntryTable", SimTypeInt(signed=False, label="UInt32")), ("GuardAddressTakenIatEntryCount", SimTypeInt(signed=False, label="UInt32")), ("GuardLongJumpTargetTable", SimTypeInt(signed=False, label="UInt32")), ("GuardLongJumpTargetCount", SimTypeInt(signed=False, label="UInt32")), ("DynamicValueRelocTable", SimTypeInt(signed=False, label="UInt32")), ("CHPEMetadataPointer", SimTypeInt(signed=False, label="UInt32")), ("GuardRFFailureRoutine", SimTypeInt(signed=False, label="UInt32")), ("GuardRFFailureRoutineFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("DynamicValueRelocTableOffset", SimTypeInt(signed=False, label="UInt32")), ("DynamicValueRelocTableSection", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("GuardRFVerifyStackPointerFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("HotPatchTableOffset", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("EnclaveConfigurationPointer", SimTypeInt(signed=False, label="UInt32")), ("VolatileMetadataPointer", SimTypeInt(signed=False, label="UInt32")), ("GuardEHContinuationTable", SimTypeInt(signed=False, label="UInt32")), ("GuardEHContinuationCount", SimTypeInt(signed=False, label="UInt32")), ("GuardXFGCheckFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("GuardXFGDispatchFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("GuardXFGTableDispatchFunctionPointer", SimTypeInt(signed=False, label="UInt32")), ("CastGuardOsDeterminedFailureMode", SimTypeInt(signed=False, label="UInt32")), ("GuardMemcpyFunctionPointer", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_LOAD_CONFIG_DIRECTORY32", pack=False, align=None),
    "IMAGE_LOAD_CONFIG_DIRECTORY64": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("GlobalFlagsClear", SimTypeInt(signed=False, label="UInt32")), ("GlobalFlagsSet", SimTypeInt(signed=False, label="UInt32")), ("CriticalSectionDefaultTimeout", SimTypeInt(signed=False, label="UInt32")), ("DeCommitFreeBlockThreshold", SimTypeLongLong(signed=False, label="UInt64")), ("DeCommitTotalFreeThreshold", SimTypeLongLong(signed=False, label="UInt64")), ("LockPrefixTable", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumAllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualMemoryThreshold", SimTypeLongLong(signed=False, label="UInt64")), ("ProcessAffinityMask", SimTypeLongLong(signed=False, label="UInt64")), ("ProcessHeapFlags", SimTypeInt(signed=False, label="UInt32")), ("CSDVersion", SimTypeShort(signed=False, label="UInt16")), ("DependentLoadFlags", SimTypeShort(signed=False, label="UInt16")), ("EditList", SimTypeLongLong(signed=False, label="UInt64")), ("SecurityCookie", SimTypeLongLong(signed=False, label="UInt64")), ("SEHandlerTable", SimTypeLongLong(signed=False, label="UInt64")), ("SEHandlerCount", SimTypeLongLong(signed=False, label="UInt64")), ("GuardCFCheckFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("GuardCFDispatchFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("GuardCFFunctionTable", SimTypeLongLong(signed=False, label="UInt64")), ("GuardCFFunctionCount", SimTypeLongLong(signed=False, label="UInt64")), ("GuardFlags", SimTypeInt(signed=False, label="UInt32")), ("CodeIntegrity", SimTypeRef("IMAGE_LOAD_CONFIG_CODE_INTEGRITY", SimStruct)), ("GuardAddressTakenIatEntryTable", SimTypeLongLong(signed=False, label="UInt64")), ("GuardAddressTakenIatEntryCount", SimTypeLongLong(signed=False, label="UInt64")), ("GuardLongJumpTargetTable", SimTypeLongLong(signed=False, label="UInt64")), ("GuardLongJumpTargetCount", SimTypeLongLong(signed=False, label="UInt64")), ("DynamicValueRelocTable", SimTypeLongLong(signed=False, label="UInt64")), ("CHPEMetadataPointer", SimTypeLongLong(signed=False, label="UInt64")), ("GuardRFFailureRoutine", SimTypeLongLong(signed=False, label="UInt64")), ("GuardRFFailureRoutineFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("DynamicValueRelocTableOffset", SimTypeInt(signed=False, label="UInt32")), ("DynamicValueRelocTableSection", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("GuardRFVerifyStackPointerFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("HotPatchTableOffset", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("EnclaveConfigurationPointer", SimTypeLongLong(signed=False, label="UInt64")), ("VolatileMetadataPointer", SimTypeLongLong(signed=False, label="UInt64")), ("GuardEHContinuationTable", SimTypeLongLong(signed=False, label="UInt64")), ("GuardEHContinuationCount", SimTypeLongLong(signed=False, label="UInt64")), ("GuardXFGCheckFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("GuardXFGDispatchFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("GuardXFGTableDispatchFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("CastGuardOsDeterminedFailureMode", SimTypeLongLong(signed=False, label="UInt64")), ("GuardMemcpyFunctionPointer", SimTypeLongLong(signed=False, label="UInt64")),)), name="IMAGE_LOAD_CONFIG_DIRECTORY64", pack=False, align=None),
    "IMAGE_NOTE": SimStruct(OrderedDict((("Module", SimTypeArray(SimTypeChar(label="SByte"), 10)), ("FileName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("hModule", SimTypeShort(signed=False, label="UInt16")), ("hTask", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_NOTE", pack=False, align=None),
    "IMAGE_NT_HEADERS32": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("FileHeader", SimTypeRef("IMAGE_FILE_HEADER", SimStruct)), ("OptionalHeader", SimTypeRef("IMAGE_OPTIONAL_HEADER32", SimStruct)),)), name="IMAGE_NT_HEADERS32", pack=False, align=None),
    "IMAGE_NT_HEADERS64": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("FileHeader", SimTypeRef("IMAGE_FILE_HEADER", SimStruct)), ("OptionalHeader", SimTypeRef("IMAGE_OPTIONAL_HEADER64", SimStruct)),)), name="IMAGE_NT_HEADERS64", pack=False, align=None),
    "IMAGE_OPTIONAL_HEADER32": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="IMAGE_OPTIONAL_HEADER_MAGIC")), ("MajorLinkerVersion", SimTypeChar(label="Byte")), ("MinorLinkerVersion", SimTypeChar(label="Byte")), ("SizeOfCode", SimTypeInt(signed=False, label="UInt32")), ("SizeOfInitializedData", SimTypeInt(signed=False, label="UInt32")), ("SizeOfUninitializedData", SimTypeInt(signed=False, label="UInt32")), ("AddressOfEntryPoint", SimTypeInt(signed=False, label="UInt32")), ("BaseOfCode", SimTypeInt(signed=False, label="UInt32")), ("BaseOfData", SimTypeInt(signed=False, label="UInt32")), ("ImageBase", SimTypeInt(signed=False, label="UInt32")), ("SectionAlignment", SimTypeInt(signed=False, label="UInt32")), ("FileAlignment", SimTypeInt(signed=False, label="UInt32")), ("MajorOperatingSystemVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorOperatingSystemVersion", SimTypeShort(signed=False, label="UInt16")), ("MajorImageVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorImageVersion", SimTypeShort(signed=False, label="UInt16")), ("MajorSubsystemVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorSubsystemVersion", SimTypeShort(signed=False, label="UInt16")), ("Win32VersionValue", SimTypeInt(signed=False, label="UInt32")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("SizeOfHeaders", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("Subsystem", SimTypeInt(signed=False, label="IMAGE_SUBSYSTEM")), ("DllCharacteristics", SimTypeInt(signed=False, label="IMAGE_DLL_CHARACTERISTICS")), ("SizeOfStackReserve", SimTypeInt(signed=False, label="UInt32")), ("SizeOfStackCommit", SimTypeInt(signed=False, label="UInt32")), ("SizeOfHeapReserve", SimTypeInt(signed=False, label="UInt32")), ("SizeOfHeapCommit", SimTypeInt(signed=False, label="UInt32")), ("LoaderFlags", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRvaAndSizes", SimTypeInt(signed=False, label="UInt32")), ("DataDirectory", SimTypeArray(SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct), 16)),)), name="IMAGE_OPTIONAL_HEADER32", pack=False, align=None),
    "IMAGE_OPTIONAL_HEADER64": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="IMAGE_OPTIONAL_HEADER_MAGIC")), ("MajorLinkerVersion", SimTypeChar(label="Byte")), ("MinorLinkerVersion", SimTypeChar(label="Byte")), ("SizeOfCode", SimTypeInt(signed=False, label="UInt32")), ("SizeOfInitializedData", SimTypeInt(signed=False, label="UInt32")), ("SizeOfUninitializedData", SimTypeInt(signed=False, label="UInt32")), ("AddressOfEntryPoint", SimTypeInt(signed=False, label="UInt32")), ("BaseOfCode", SimTypeInt(signed=False, label="UInt32")), ("ImageBase", SimTypeLongLong(signed=False, label="UInt64")), ("SectionAlignment", SimTypeInt(signed=False, label="UInt32")), ("FileAlignment", SimTypeInt(signed=False, label="UInt32")), ("MajorOperatingSystemVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorOperatingSystemVersion", SimTypeShort(signed=False, label="UInt16")), ("MajorImageVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorImageVersion", SimTypeShort(signed=False, label="UInt16")), ("MajorSubsystemVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorSubsystemVersion", SimTypeShort(signed=False, label="UInt16")), ("Win32VersionValue", SimTypeInt(signed=False, label="UInt32")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("SizeOfHeaders", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("Subsystem", SimTypeInt(signed=False, label="IMAGE_SUBSYSTEM")), ("DllCharacteristics", SimTypeInt(signed=False, label="IMAGE_DLL_CHARACTERISTICS")), ("SizeOfStackReserve", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfStackCommit", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfHeapReserve", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfHeapCommit", SimTypeLongLong(signed=False, label="UInt64")), ("LoaderFlags", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRvaAndSizes", SimTypeInt(signed=False, label="UInt32")), ("DataDirectory", SimTypeArray(SimTypeRef("IMAGE_DATA_DIRECTORY", SimStruct), 16)),)), name="IMAGE_OPTIONAL_HEADER64", pack=False, align=None),
    "IMAGE_OPTIONAL_HEADER_MAGIC": SimTypeInt(signed=False, label="IMAGE_OPTIONAL_HEADER_MAGIC"),
    "IMAGE_OS2_HEADER": SimStruct(OrderedDict((("ne_magic", SimTypeShort(signed=False, label="UInt16")), ("ne_ver", SimTypeChar(label="SByte")), ("ne_rev", SimTypeChar(label="SByte")), ("ne_enttab", SimTypeShort(signed=False, label="UInt16")), ("ne_cbenttab", SimTypeShort(signed=False, label="UInt16")), ("ne_crc", SimTypeInt(signed=True, label="Int32")), ("ne_flags", SimTypeShort(signed=False, label="UInt16")), ("ne_autodata", SimTypeShort(signed=False, label="UInt16")), ("ne_heap", SimTypeShort(signed=False, label="UInt16")), ("ne_stack", SimTypeShort(signed=False, label="UInt16")), ("ne_csip", SimTypeInt(signed=True, label="Int32")), ("ne_sssp", SimTypeInt(signed=True, label="Int32")), ("ne_cseg", SimTypeShort(signed=False, label="UInt16")), ("ne_cmod", SimTypeShort(signed=False, label="UInt16")), ("ne_cbnrestab", SimTypeShort(signed=False, label="UInt16")), ("ne_segtab", SimTypeShort(signed=False, label="UInt16")), ("ne_rsrctab", SimTypeShort(signed=False, label="UInt16")), ("ne_restab", SimTypeShort(signed=False, label="UInt16")), ("ne_modtab", SimTypeShort(signed=False, label="UInt16")), ("ne_imptab", SimTypeShort(signed=False, label="UInt16")), ("ne_nrestab", SimTypeInt(signed=True, label="Int32")), ("ne_cmovent", SimTypeShort(signed=False, label="UInt16")), ("ne_align", SimTypeShort(signed=False, label="UInt16")), ("ne_cres", SimTypeShort(signed=False, label="UInt16")), ("ne_exetyp", SimTypeChar(label="Byte")), ("ne_flagsothers", SimTypeChar(label="Byte")), ("ne_pretthunks", SimTypeShort(signed=False, label="UInt16")), ("ne_psegrefbytes", SimTypeShort(signed=False, label="UInt16")), ("ne_swaparea", SimTypeShort(signed=False, label="UInt16")), ("ne_expver", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_OS2_HEADER", pack=False, align=None),
    "IMAGE_POLICY_ENTRY": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="IMAGE_POLICY_ENTRY_TYPE")), ("PolicyId", SimTypeInt(signed=False, label="IMAGE_POLICY_ID")), ("u", SimUnion({"None": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "BoolValue": SimTypeChar(label="Byte"), "Int8Value": SimTypeChar(label="SByte"), "UInt8Value": SimTypeChar(label="Byte"), "Int16Value": SimTypeShort(signed=True, label="Int16"), "UInt16Value": SimTypeShort(signed=False, label="UInt16"), "Int32Value": SimTypeInt(signed=True, label="Int32"), "UInt32Value": SimTypeInt(signed=False, label="UInt32"), "Int64Value": SimTypeLongLong(signed=True, label="Int64"), "UInt64Value": SimTypeLongLong(signed=False, label="UInt64"), "AnsiStringValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "UnicodeStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="IMAGE_POLICY_ENTRY", pack=False, align=None),
    "IMAGE_POLICY_ENTRY_TYPE": SimTypeInt(signed=False, label="IMAGE_POLICY_ENTRY_TYPE"),
    "IMAGE_POLICY_ID": SimTypeInt(signed=False, label="IMAGE_POLICY_ID"),
    "IMAGE_POLICY_METADATA": SimStruct(OrderedDict((("Version", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("ApplicationId", SimTypeLongLong(signed=False, label="UInt64")), ("Policies", SimTypePointer(SimTypeRef("IMAGE_POLICY_ENTRY", SimStruct), offset=0)),)), name="IMAGE_POLICY_METADATA", pack=False, align=None),
    "IMAGE_PROLOGUE_DYNAMIC_RELOCATION_HEADER": SimStruct(OrderedDict((("PrologueByteCount", SimTypeChar(label="Byte")),)), name="IMAGE_PROLOGUE_DYNAMIC_RELOCATION_HEADER", pack=False, align=None),
    "IMAGE_RELOCATION": SimStruct(OrderedDict((("Anonymous", SimUnion({"VirtualAddress": SimTypeInt(signed=False, label="UInt32"), "RelocCount": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SymbolTableIndex", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_RELOCATION", pack=False, align=None),
    "IMAGE_RESOURCE_DATA_ENTRY": SimStruct(OrderedDict((("OffsetToData", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("CodePage", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_RESOURCE_DATA_ENTRY", pack=False, align=None),
    "IMAGE_RESOURCE_DIRECTORY": SimStruct(OrderedDict((("Characteristics", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("NumberOfNamedEntries", SimTypeShort(signed=False, label="UInt16")), ("NumberOfIdEntries", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_RESOURCE_DIRECTORY", pack=False, align=None),
    "IMAGE_RESOURCE_DIRECTORY_ENTRY": SimStruct(OrderedDict((("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Name": SimTypeInt(signed=False, label="UInt32"), "Id": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"OffsetToData": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="IMAGE_RESOURCE_DIRECTORY_ENTRY", pack=False, align=None),
    "IMAGE_RESOURCE_DIRECTORY_STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("NameString", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="IMAGE_RESOURCE_DIRECTORY_STRING", pack=False, align=None),
    "IMAGE_RESOURCE_DIR_STRING_U": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("NameString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IMAGE_RESOURCE_DIR_STRING_U", pack=False, align=None),
    "IMAGE_ROM_HEADERS": SimStruct(OrderedDict((("FileHeader", SimTypeRef("IMAGE_FILE_HEADER", SimStruct)), ("OptionalHeader", SimTypeRef("IMAGE_ROM_OPTIONAL_HEADER", SimStruct)),)), name="IMAGE_ROM_HEADERS", pack=False, align=None),
    "IMAGE_ROM_OPTIONAL_HEADER": SimStruct(OrderedDict((("Magic", SimTypeShort(signed=False, label="UInt16")), ("MajorLinkerVersion", SimTypeChar(label="Byte")), ("MinorLinkerVersion", SimTypeChar(label="Byte")), ("SizeOfCode", SimTypeInt(signed=False, label="UInt32")), ("SizeOfInitializedData", SimTypeInt(signed=False, label="UInt32")), ("SizeOfUninitializedData", SimTypeInt(signed=False, label="UInt32")), ("AddressOfEntryPoint", SimTypeInt(signed=False, label="UInt32")), ("BaseOfCode", SimTypeInt(signed=False, label="UInt32")), ("BaseOfData", SimTypeInt(signed=False, label="UInt32")), ("BaseOfBss", SimTypeInt(signed=False, label="UInt32")), ("GprMask", SimTypeInt(signed=False, label="UInt32")), ("CprMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("GpValue", SimTypeInt(signed=False, label="UInt32")),)), name="IMAGE_ROM_OPTIONAL_HEADER", pack=False, align=None),
    "IMAGE_RUNTIME_FUNCTION_ENTRY": SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"UnwindInfoAddress": SimTypeInt(signed=False, label="UInt32"), "UnwindData": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="IMAGE_RUNTIME_FUNCTION_ENTRY", pack=False, align=None),
    "IMAGE_SECTION_CHARACTERISTICS": SimTypeInt(signed=False, label="IMAGE_SECTION_CHARACTERISTICS"),
    "IMAGE_SECTION_HEADER": SimStruct(OrderedDict((("Name", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Misc", SimUnion({"PhysicalAddress": SimTypeInt(signed=False, label="UInt32"), "VirtualSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("VirtualAddress", SimTypeInt(signed=False, label="UInt32")), ("SizeOfRawData", SimTypeInt(signed=False, label="UInt32")), ("PointerToRawData", SimTypeInt(signed=False, label="UInt32")), ("PointerToRelocations", SimTypeInt(signed=False, label="UInt32")), ("PointerToLinenumbers", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRelocations", SimTypeShort(signed=False, label="UInt16")), ("NumberOfLinenumbers", SimTypeShort(signed=False, label="UInt16")), ("Characteristics", SimTypeInt(signed=False, label="IMAGE_SECTION_CHARACTERISTICS")),)), name="IMAGE_SECTION_HEADER", pack=False, align=None),
    "IMAGE_SEPARATE_DEBUG_HEADER": SimStruct(OrderedDict((("Signature", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Machine", SimTypeShort(signed=False, label="UInt16")), ("Characteristics", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("ImageBase", SimTypeInt(signed=False, label="UInt32")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSections", SimTypeInt(signed=False, label="UInt32")), ("ExportedNamesSize", SimTypeInt(signed=False, label="UInt32")), ("DebugDirectorySize", SimTypeInt(signed=False, label="UInt32")), ("SectionAlignment", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="IMAGE_SEPARATE_DEBUG_HEADER", pack=False, align=None),
    "IMAGE_SUBSYSTEM": SimTypeInt(signed=False, label="IMAGE_SUBSYSTEM"),
    "IMAGE_SWITCHTABLE_BRANCH_DYNAMIC_RELOCATION": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_SWITCHTABLE_BRANCH_DYNAMIC_RELOCATION", pack=False, align=None),
    "IMAGE_SYMBOL": SimStruct(OrderedDict((("N", SimUnion({"ShortName": SimTypeArray(SimTypeChar(label="Byte"), 8), "Name": SimTypeRef("_Name_e__Struct", SimStruct), "LongName": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)}, name="<anon>", label="None")), ("Value", SimTypeInt(signed=False, label="UInt32")), ("SectionNumber", SimTypeShort(signed=True, label="Int16")), ("Type", SimTypeShort(signed=False, label="UInt16")), ("StorageClass", SimTypeChar(label="Byte")), ("NumberOfAuxSymbols", SimTypeChar(label="Byte")),)), name="IMAGE_SYMBOL", pack=False, align=None),
    "IMAGE_SYMBOL_EX": SimStruct(OrderedDict((("N", SimUnion({"ShortName": SimTypeArray(SimTypeChar(label="Byte"), 8), "Name": SimTypeRef("_Name_e__Struct", SimStruct), "LongName": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)}, name="<anon>", label="None")), ("Value", SimTypeInt(signed=False, label="UInt32")), ("SectionNumber", SimTypeInt(signed=True, label="Int32")), ("Type", SimTypeShort(signed=False, label="UInt16")), ("StorageClass", SimTypeChar(label="Byte")), ("NumberOfAuxSymbols", SimTypeChar(label="Byte")),)), name="IMAGE_SYMBOL_EX", pack=False, align=None),
    "IMAGE_THUNK_DATA32": SimStruct(OrderedDict((("u1", SimUnion({"ForwarderString": SimTypeInt(signed=False, label="UInt32"), "Function": SimTypeInt(signed=False, label="UInt32"), "Ordinal": SimTypeInt(signed=False, label="UInt32"), "AddressOfData": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="IMAGE_THUNK_DATA32", pack=False, align=None),
    "IMAGE_THUNK_DATA64": SimStruct(OrderedDict((("u1", SimUnion({"ForwarderString": SimTypeLongLong(signed=False, label="UInt64"), "Function": SimTypeLongLong(signed=False, label="UInt64"), "Ordinal": SimTypeLongLong(signed=False, label="UInt64"), "AddressOfData": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="IMAGE_THUNK_DATA64", pack=False, align=None),
    "IMAGE_TLS_DIRECTORY32": SimStruct(OrderedDict((("StartAddressOfRawData", SimTypeInt(signed=False, label="UInt32")), ("EndAddressOfRawData", SimTypeInt(signed=False, label="UInt32")), ("AddressOfIndex", SimTypeInt(signed=False, label="UInt32")), ("AddressOfCallBacks", SimTypeInt(signed=False, label="UInt32")), ("SizeOfZeroFill", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Characteristics": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="IMAGE_TLS_DIRECTORY32", pack=False, align=None),
    "IMAGE_TLS_DIRECTORY64": SimStruct(OrderedDict((("StartAddressOfRawData", SimTypeLongLong(signed=False, label="UInt64")), ("EndAddressOfRawData", SimTypeLongLong(signed=False, label="UInt64")), ("AddressOfIndex", SimTypeLongLong(signed=False, label="UInt64")), ("AddressOfCallBacks", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfZeroFill", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Characteristics": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="IMAGE_TLS_DIRECTORY64", pack=False, align=None),
    "IMAGE_VXD_HEADER": SimStruct(OrderedDict((("e32_magic", SimTypeShort(signed=False, label="UInt16")), ("e32_border", SimTypeChar(label="Byte")), ("e32_worder", SimTypeChar(label="Byte")), ("e32_level", SimTypeInt(signed=False, label="UInt32")), ("e32_cpu", SimTypeShort(signed=False, label="UInt16")), ("e32_os", SimTypeShort(signed=False, label="UInt16")), ("e32_ver", SimTypeInt(signed=False, label="UInt32")), ("e32_mflags", SimTypeInt(signed=False, label="UInt32")), ("e32_mpages", SimTypeInt(signed=False, label="UInt32")), ("e32_startobj", SimTypeInt(signed=False, label="UInt32")), ("e32_eip", SimTypeInt(signed=False, label="UInt32")), ("e32_stackobj", SimTypeInt(signed=False, label="UInt32")), ("e32_esp", SimTypeInt(signed=False, label="UInt32")), ("e32_pagesize", SimTypeInt(signed=False, label="UInt32")), ("e32_lastpagesize", SimTypeInt(signed=False, label="UInt32")), ("e32_fixupsize", SimTypeInt(signed=False, label="UInt32")), ("e32_fixupsum", SimTypeInt(signed=False, label="UInt32")), ("e32_ldrsize", SimTypeInt(signed=False, label="UInt32")), ("e32_ldrsum", SimTypeInt(signed=False, label="UInt32")), ("e32_objtab", SimTypeInt(signed=False, label="UInt32")), ("e32_objcnt", SimTypeInt(signed=False, label="UInt32")), ("e32_objmap", SimTypeInt(signed=False, label="UInt32")), ("e32_itermap", SimTypeInt(signed=False, label="UInt32")), ("e32_rsrctab", SimTypeInt(signed=False, label="UInt32")), ("e32_rsrccnt", SimTypeInt(signed=False, label="UInt32")), ("e32_restab", SimTypeInt(signed=False, label="UInt32")), ("e32_enttab", SimTypeInt(signed=False, label="UInt32")), ("e32_dirtab", SimTypeInt(signed=False, label="UInt32")), ("e32_dircnt", SimTypeInt(signed=False, label="UInt32")), ("e32_fpagetab", SimTypeInt(signed=False, label="UInt32")), ("e32_frectab", SimTypeInt(signed=False, label="UInt32")), ("e32_impmod", SimTypeInt(signed=False, label="UInt32")), ("e32_impmodcnt", SimTypeInt(signed=False, label="UInt32")), ("e32_impproc", SimTypeInt(signed=False, label="UInt32")), ("e32_pagesum", SimTypeInt(signed=False, label="UInt32")), ("e32_datapage", SimTypeInt(signed=False, label="UInt32")), ("e32_preload", SimTypeInt(signed=False, label="UInt32")), ("e32_nrestab", SimTypeInt(signed=False, label="UInt32")), ("e32_cbnrestab", SimTypeInt(signed=False, label="UInt32")), ("e32_nressum", SimTypeInt(signed=False, label="UInt32")), ("e32_autodata", SimTypeInt(signed=False, label="UInt32")), ("e32_debuginfo", SimTypeInt(signed=False, label="UInt32")), ("e32_debuglen", SimTypeInt(signed=False, label="UInt32")), ("e32_instpreload", SimTypeInt(signed=False, label="UInt32")), ("e32_instdemand", SimTypeInt(signed=False, label="UInt32")), ("e32_heapsize", SimTypeInt(signed=False, label="UInt32")), ("e32_res3", SimTypeArray(SimTypeChar(label="Byte"), 12)), ("e32_winresoff", SimTypeInt(signed=False, label="UInt32")), ("e32_winreslen", SimTypeInt(signed=False, label="UInt32")), ("e32_devid", SimTypeShort(signed=False, label="UInt16")), ("e32_ddkver", SimTypeShort(signed=False, label="UInt16")),)), name="IMAGE_VXD_HEADER", pack=False, align=None),
    "IMAPIAdviseSink": SimTypeBottom(label="IMAPIAdviseSink"),
    "IMAPIContainer": SimTypeBottom(label="IMAPIContainer"),
    "IMAPIControl": SimTypeBottom(label="IMAPIControl"),
    "IMAPIFolder": SimTypeBottom(label="IMAPIFolder"),
    "IMAPIProgress": SimTypeBottom(label="IMAPIProgress"),
    "IMAPIProp": SimTypeBottom(label="IMAPIProp"),
    "IMAPIStatus": SimTypeBottom(label="IMAPIStatus"),
    "IMAPITable": SimTypeBottom(label="IMAPITable"),
    "IMAPI_BURN_VERIFICATION_LEVEL": SimTypeInt(signed=False, label="IMAPI_BURN_VERIFICATION_LEVEL"),
    "IMAPI_CD_SECTOR_TYPE": SimTypeInt(signed=False, label="IMAPI_CD_SECTOR_TYPE"),
    "IMAPI_CD_TRACK_DIGITAL_COPY_SETTING": SimTypeInt(signed=False, label="IMAPI_CD_TRACK_DIGITAL_COPY_SETTING"),
    "IMAPI_FEATURE_PAGE_TYPE": SimTypeInt(signed=False, label="IMAPI_FEATURE_PAGE_TYPE"),
    "IMAPI_FORMAT2_DATA_MEDIA_STATE": SimTypeInt(signed=False, label="IMAPI_FORMAT2_DATA_MEDIA_STATE"),
    "IMAPI_FORMAT2_DATA_WRITE_ACTION": SimTypeInt(signed=False, label="IMAPI_FORMAT2_DATA_WRITE_ACTION"),
    "IMAPI_FORMAT2_RAW_CD_DATA_SECTOR_TYPE": SimTypeInt(signed=False, label="IMAPI_FORMAT2_RAW_CD_DATA_SECTOR_TYPE"),
    "IMAPI_FORMAT2_RAW_CD_WRITE_ACTION": SimTypeInt(signed=False, label="IMAPI_FORMAT2_RAW_CD_WRITE_ACTION"),
    "IMAPI_FORMAT2_TAO_WRITE_ACTION": SimTypeInt(signed=False, label="IMAPI_FORMAT2_TAO_WRITE_ACTION"),
    "IMAPI_MEDIA_PHYSICAL_TYPE": SimTypeInt(signed=False, label="IMAPI_MEDIA_PHYSICAL_TYPE"),
    "IMAPI_MEDIA_WRITE_PROTECT_STATE": SimTypeInt(signed=False, label="IMAPI_MEDIA_WRITE_PROTECT_STATE"),
    "IMAPI_MODE_PAGE_REQUEST_TYPE": SimTypeInt(signed=False, label="IMAPI_MODE_PAGE_REQUEST_TYPE"),
    "IMAPI_MODE_PAGE_TYPE": SimTypeInt(signed=False, label="IMAPI_MODE_PAGE_TYPE"),
    "IMAPI_PROFILE_TYPE": SimTypeInt(signed=False, label="IMAPI_PROFILE_TYPE"),
    "IMAPI_READ_TRACK_ADDRESS_TYPE": SimTypeInt(signed=False, label="IMAPI_READ_TRACK_ADDRESS_TYPE"),
    "IMCENUMPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "IMDDataset": SimTypeBottom(label="IMDDataset"),
    "IMDFind": SimTypeBottom(label="IMDFind"),
    "IMDRangeRowset": SimTypeBottom(label="IMDRangeRowset"),
    "IMDSPDevice": SimTypeBottom(label="IMDSPDevice"),
    "IMDSPDevice2": SimTypeBottom(label="IMDSPDevice2"),
    "IMDSPDevice3": SimTypeBottom(label="IMDSPDevice3"),
    "IMDSPDeviceControl": SimTypeBottom(label="IMDSPDeviceControl"),
    "IMDSPDirectTransfer": SimTypeBottom(label="IMDSPDirectTransfer"),
    "IMDSPEnumDevice": SimTypeBottom(label="IMDSPEnumDevice"),
    "IMDSPEnumStorage": SimTypeBottom(label="IMDSPEnumStorage"),
    "IMDSPObject": SimTypeBottom(label="IMDSPObject"),
    "IMDSPObject2": SimTypeBottom(label="IMDSPObject2"),
    "IMDSPObjectInfo": SimTypeBottom(label="IMDSPObjectInfo"),
    "IMDSPRevoked": SimTypeBottom(label="IMDSPRevoked"),
    "IMDSPStorage": SimTypeBottom(label="IMDSPStorage"),
    "IMDSPStorage2": SimTypeBottom(label="IMDSPStorage2"),
    "IMDSPStorage3": SimTypeBottom(label="IMDSPStorage3"),
    "IMDSPStorage4": SimTypeBottom(label="IMDSPStorage4"),
    "IMDSPStorageGlobals": SimTypeBottom(label="IMDSPStorageGlobals"),
    "IMDServiceProvider": SimTypeBottom(label="IMDServiceProvider"),
    "IMDServiceProvider2": SimTypeBottom(label="IMDServiceProvider2"),
    "IMDServiceProvider3": SimTypeBottom(label="IMDServiceProvider3"),
    "IMEAPPLETCFG": SimStruct(OrderedDict((("dwConfig", SimTypeInt(signed=False, label="UInt32")), ("wchTitle", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wchTitleFontFace", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwCharSet", SimTypeInt(signed=False, label="UInt32")), ("iCategory", SimTypeInt(signed=True, label="Int32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("langID", SimTypeShort(signed=False, label="UInt16")), ("dummy", SimTypeShort(signed=False, label="UInt16")), ("lReserved1", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMEAPPLETCFG", pack=False, align=None),
    "IMEAPPLETUI": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("width", SimTypeInt(signed=True, label="Int32")), ("height", SimTypeInt(signed=True, label="Int32")), ("minWidth", SimTypeInt(signed=True, label="Int32")), ("minHeight", SimTypeInt(signed=True, label="Int32")), ("maxWidth", SimTypeInt(signed=True, label="Int32")), ("maxHeight", SimTypeInt(signed=True, label="Int32")), ("lReserved1", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lReserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMEAPPLETUI", pack=False, align=None),
    "IMECHARINFO": SimStruct(OrderedDict((("wch", SimTypeChar(label="Char")), ("dwCharInfo", SimTypeInt(signed=False, label="UInt32")),)), name="IMECHARINFO", pack=False, align=None),
    "IMECHARPOSITION": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCharPos", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)), ("cLineHeight", SimTypeInt(signed=False, label="UInt32")), ("rcDocument", SimTypeRef("RECT", SimStruct)),)), name="IMECHARPOSITION", pack=False, align=None),
    "IMECOMPOSITIONSTRINGINFO": SimStruct(OrderedDict((("iCompStrLen", SimTypeInt(signed=True, label="Int32")), ("iCaretPos", SimTypeInt(signed=True, label="Int32")), ("iEditStart", SimTypeInt(signed=True, label="Int32")), ("iEditLen", SimTypeInt(signed=True, label="Int32")), ("iTargetStart", SimTypeInt(signed=True, label="Int32")), ("iTargetLen", SimTypeInt(signed=True, label="Int32")),)), name="IMECOMPOSITIONSTRINGINFO", pack=False, align=None),
    "IMECOMPTEXT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=True, label="Int32")), ("flags", SimTypeInt(signed=False, label="IMECOMPTEXT_FLAGS")),)), name="IMECOMPTEXT", pack=False, align=None),
    "IMECOMPTEXT_FLAGS": SimTypeInt(signed=False, label="IMECOMPTEXT_FLAGS"),
    "IMEDLG": SimStruct(OrderedDict((("cbIMEDLG", SimTypeInt(signed=True, label="Int32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpwstrWord", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nTabId", SimTypeInt(signed=True, label="Int32")),)), name="IMEDLG", pack=False, align=None),
    "IMEDP": SimStruct(OrderedDict((("wrdModifier", SimTypeRef("IMEWRD", SimStruct)), ("wrdModifiee", SimTypeRef("IMEWRD", SimStruct)), ("relID", SimTypeInt(signed=False, label="IMEREL")),)), name="IMEDP", pack=False, align=None),
    "IMEFAREASTINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IMEFAREASTINFO", pack=False, align=None),
    "IMEFMT": SimTypeInt(signed=False, label="IMEFMT"),
    "IMEINFO": SimStruct(OrderedDict((("dwPrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("fdwProperty", SimTypeInt(signed=False, label="UInt32")), ("fdwConversionCaps", SimTypeInt(signed=False, label="UInt32")), ("fdwSentenceCaps", SimTypeInt(signed=False, label="UInt32")), ("fdwUICaps", SimTypeInt(signed=False, label="UInt32")), ("fdwSCSCaps", SimTypeInt(signed=False, label="UInt32")), ("fdwSelectCaps", SimTypeInt(signed=False, label="UInt32")),)), name="IMEINFO", pack=False, align=None),
    "IMEITEM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("iType", SimTypeInt(signed=True, label="Int32")), ("lpItemData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IMEITEM", pack=False, align=None),
    "IMEITEMCANDIDATE": SimStruct(OrderedDict((("uCount", SimTypeInt(signed=False, label="UInt32")), ("imeItem", SimTypePointer(SimTypeRef("IMEITEM", SimStruct), offset=0)),)), name="IMEITEMCANDIDATE", pack=False, align=None),
    "IMEKMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("hIMC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cKeyList", SimTypeInt(signed=False, label="UInt32")), ("pKeyList", SimTypePointer(SimTypeRef("IMEKMSKEY", SimStruct), offset=0)),)), name="IMEKMS", pack=False, align=None),
    "IMEKMSFUNCDESC": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("idLang", SimTypeShort(signed=False, label="UInt16")), ("dwControl", SimTypeInt(signed=False, label="UInt32")), ("pwszDescription", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="IMEKMSFUNCDESC", pack=False, align=None),
    "IMEKMSINIT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMEKMSINIT", pack=False, align=None),
    "IMEKMSINVK": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("hIMC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwControl", SimTypeInt(signed=False, label="UInt32")),)), name="IMEKMSINVK", pack=False, align=None),
    "IMEKMSKEY": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("dwCompStatus", SimTypeInt(signed=False, label="UInt32")), ("dwVKEY", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"dwControl": SimTypeInt(signed=False, label="UInt32"), "dwNotUsed": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"pwszDscr": SimTypeArray(SimTypeChar(label="Char"), 31), "pwszNoUse": SimTypeArray(SimTypeChar(label="Char"), 31)}, name="<anon>", label="None")),)), name="IMEKMSKEY", pack=False, align=None),
    "IMEKMSKMP": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("hIMC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("idLang", SimTypeShort(signed=False, label="UInt16")), ("wVKStart", SimTypeShort(signed=False, label="UInt16")), ("wVKEnd", SimTypeShort(signed=False, label="UInt16")), ("cKeyList", SimTypeInt(signed=True, label="Int32")), ("pKeyList", SimTypePointer(SimTypeRef("IMEKMSKEY", SimStruct), offset=0)),)), name="IMEKMSKMP", pack=False, align=None),
    "IMEKMSNTFY": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("hIMC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fSelect", SimTypeInt(signed=True, label="Int32")),)), name="IMEKMSNTFY", pack=False, align=None),
    "IMEMENUITEMINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fType", SimTypeInt(signed=False, label="UInt32")), ("fState", SimTypeInt(signed=False, label="UInt32")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("hbmpChecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmpUnchecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwItemData", SimTypeInt(signed=False, label="UInt32")), ("szString", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("hbmpItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMEMENUITEMINFOA", pack=False, align=None),
    "IMEMENUITEMINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fType", SimTypeInt(signed=False, label="UInt32")), ("fState", SimTypeInt(signed=False, label="UInt32")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("hbmpChecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmpUnchecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwItemData", SimTypeInt(signed=False, label="UInt32")), ("szString", SimTypeArray(SimTypeChar(label="Char"), 80)), ("hbmpItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMEMENUITEMINFOW", pack=False, align=None),
    "IMEPROA": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InstDate", SimTypeRef("DATETIME", SimStruct)), ("wVersion", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Byte"), 50)), ("szName", SimTypeArray(SimTypeChar(label="Byte"), 80)), ("szOptions", SimTypeArray(SimTypeChar(label="Byte"), 30)),)), name="IMEPROA", pack=False, align=None),
    "IMEPROW": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InstDate", SimTypeRef("DATETIME", SimStruct)), ("wVersion", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 50)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 80)), ("szOptions", SimTypeArray(SimTypeChar(label="Char"), 30)),)), name="IMEPROW", pack=False, align=None),
    "IMEREG": SimTypeInt(signed=False, label="IMEREG"),
    "IMEREL": SimTypeInt(signed=False, label="IMEREL"),
    "IMESHF": SimStruct(OrderedDict((("cbShf", SimTypeShort(signed=False, label="UInt16")), ("verDic", SimTypeShort(signed=False, label="UInt16")), ("szTitle", SimTypeArray(SimTypeChar(label="SByte"), 48)), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("szCopyright", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="IMESHF", pack=False, align=None),
    "IMESTRINGCANDIDATE": SimStruct(OrderedDict((("uCount", SimTypeInt(signed=False, label="UInt32")), ("lpwstr", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="IMESTRINGCANDIDATE", pack=False, align=None),
    "IMESTRINGCANDIDATEINFO": SimStruct(OrderedDict((("dwFarEastId", SimTypeInt(signed=False, label="UInt32")), ("lpFarEastInfo", SimTypePointer(SimTypeRef("IMEFAREASTINFO", SimStruct), offset=0)), ("fInfoMask", SimTypeInt(signed=False, label="UInt32")), ("iSelIndex", SimTypeInt(signed=True, label="Int32")), ("uCount", SimTypeInt(signed=False, label="UInt32")), ("lpwstr", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="IMESTRINGCANDIDATEINFO", pack=False, align=None),
    "IMESTRINGINFO": SimStruct(OrderedDict((("dwFarEastId", SimTypeInt(signed=False, label="UInt32")), ("lpwstr", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IMESTRINGINFO", pack=False, align=None),
    "IMESTRUCT": SimStruct(OrderedDict((("fnc", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wCount", SimTypeInt(signed=False, label="UInt32")), ("dchSource", SimTypeInt(signed=False, label="UInt32")), ("dchDest", SimTypeInt(signed=False, label="UInt32")), ("lParam1", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lParam2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lParam3", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IMESTRUCT", pack=False, align=None),
    "IMEUCT": SimTypeInt(signed=False, label="IMEUCT"),
    "IMEWRD": SimStruct(OrderedDict((("pwchReading", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwchDisplay", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"ulPos": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("nPos1", SimTypeShort(signed=False, label="UInt16")), ("nPos2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("rgulAttrs", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("cbComment", SimTypeInt(signed=True, label="Int32")), ("uct", SimTypeInt(signed=False, label="IMEUCT")), ("pvComment", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IMEWRD", pack=False, align=None),
    "IME_COMPOSITION_STRING": SimTypeInt(signed=False, label="IME_COMPOSITION_STRING"),
    "IME_CONVERSION_MODE": SimTypeInt(signed=False, label="IME_CONVERSION_MODE"),
    "IME_ESCAPE": SimTypeInt(signed=False, label="IME_ESCAPE"),
    "IME_HOTKEY_IDENTIFIER": SimTypeInt(signed=False, label="IME_HOTKEY_IDENTIFIER"),
    "IME_PAD_REQUEST_FLAGS": SimTypeInt(signed=False, label="IME_PAD_REQUEST_FLAGS"),
    "IME_SENTENCE_MODE": SimTypeInt(signed=False, label="IME_SENTENCE_MODE"),
    "IMF2DBuffer": SimTypeBottom(label="IMF2DBuffer"),
    "IMF2DBuffer2": SimTypeBottom(label="IMF2DBuffer2"),
    "IMFASFContentInfo": SimTypeBottom(label="IMFASFContentInfo"),
    "IMFASFIndexer": SimTypeBottom(label="IMFASFIndexer"),
    "IMFASFMultiplexer": SimTypeBottom(label="IMFASFMultiplexer"),
    "IMFASFMutualExclusion": SimTypeBottom(label="IMFASFMutualExclusion"),
    "IMFASFProfile": SimTypeBottom(label="IMFASFProfile"),
    "IMFASFSplitter": SimTypeBottom(label="IMFASFSplitter"),
    "IMFASFStreamConfig": SimTypeBottom(label="IMFASFStreamConfig"),
    "IMFASFStreamPrioritization": SimTypeBottom(label="IMFASFStreamPrioritization"),
    "IMFASFStreamSelector": SimTypeBottom(label="IMFASFStreamSelector"),
    "IMFActivate": SimTypeBottom(label="IMFActivate"),
    "IMFAsyncCallback": SimTypeBottom(label="IMFAsyncCallback"),
    "IMFAsyncCallbackLogging": SimTypeBottom(label="IMFAsyncCallbackLogging"),
    "IMFAsyncResult": SimTypeBottom(label="IMFAsyncResult"),
    "IMFAttributes": SimTypeBottom(label="IMFAttributes"),
    "IMFAudioMediaType": SimTypeBottom(label="IMFAudioMediaType"),
    "IMFAudioPolicy": SimTypeBottom(label="IMFAudioPolicy"),
    "IMFAudioStreamVolume": SimTypeBottom(label="IMFAudioStreamVolume"),
    "IMFBufferListNotify": SimTypeBottom(label="IMFBufferListNotify"),
    "IMFByteStream": SimTypeBottom(label="IMFByteStream"),
    "IMFByteStreamBuffering": SimTypeBottom(label="IMFByteStreamBuffering"),
    "IMFByteStreamCacheControl": SimTypeBottom(label="IMFByteStreamCacheControl"),
    "IMFByteStreamCacheControl2": SimTypeBottom(label="IMFByteStreamCacheControl2"),
    "IMFByteStreamHandler": SimTypeBottom(label="IMFByteStreamHandler"),
    "IMFByteStreamProxyClassFactory": SimTypeBottom(label="IMFByteStreamProxyClassFactory"),
    "IMFByteStreamTimeSeek": SimTypeBottom(label="IMFByteStreamTimeSeek"),
    "IMFCameraConfigurationManager": SimTypeBottom(label="IMFCameraConfigurationManager"),
    "IMFCameraControlDefaults": SimTypeBottom(label="IMFCameraControlDefaults"),
    "IMFCameraControlDefaultsCollection": SimTypeBottom(label="IMFCameraControlDefaultsCollection"),
    "IMFCameraControlMonitor": SimTypeBottom(label="IMFCameraControlMonitor"),
    "IMFCameraControlNotify": SimTypeBottom(label="IMFCameraControlNotify"),
    "IMFCameraOcclusionStateMonitor": SimTypeBottom(label="IMFCameraOcclusionStateMonitor"),
    "IMFCameraOcclusionStateReport": SimTypeBottom(label="IMFCameraOcclusionStateReport"),
    "IMFCameraOcclusionStateReportCallback": SimTypeBottom(label="IMFCameraOcclusionStateReportCallback"),
    "IMFCameraSyncObject": SimTypeBottom(label="IMFCameraSyncObject"),
    "IMFCaptureEngine": SimTypeBottom(label="IMFCaptureEngine"),
    "IMFCaptureEngineClassFactory": SimTypeBottom(label="IMFCaptureEngineClassFactory"),
    "IMFCaptureEngineOnEventCallback": SimTypeBottom(label="IMFCaptureEngineOnEventCallback"),
    "IMFCaptureEngineOnSampleCallback": SimTypeBottom(label="IMFCaptureEngineOnSampleCallback"),
    "IMFCaptureEngineOnSampleCallback2": SimTypeBottom(label="IMFCaptureEngineOnSampleCallback2"),
    "IMFCapturePhotoConfirmation": SimTypeBottom(label="IMFCapturePhotoConfirmation"),
    "IMFCapturePhotoSink": SimTypeBottom(label="IMFCapturePhotoSink"),
    "IMFCapturePreviewSink": SimTypeBottom(label="IMFCapturePreviewSink"),
    "IMFCaptureRecordSink": SimTypeBottom(label="IMFCaptureRecordSink"),
    "IMFCaptureSink": SimTypeBottom(label="IMFCaptureSink"),
    "IMFCaptureSink2": SimTypeBottom(label="IMFCaptureSink2"),
    "IMFCaptureSource": SimTypeBottom(label="IMFCaptureSource"),
    "IMFCdmSuspendNotify": SimTypeBottom(label="IMFCdmSuspendNotify"),
    "IMFClock": SimTypeBottom(label="IMFClock"),
    "IMFClockConsumer": SimTypeBottom(label="IMFClockConsumer"),
    "IMFClockStateSink": SimTypeBottom(label="IMFClockStateSink"),
    "IMFCollection": SimTypeBottom(label="IMFCollection"),
    "IMFContentDecryptionModule": SimTypeBottom(label="IMFContentDecryptionModule"),
    "IMFContentDecryptionModuleAccess": SimTypeBottom(label="IMFContentDecryptionModuleAccess"),
    "IMFContentDecryptionModuleFactory": SimTypeBottom(label="IMFContentDecryptionModuleFactory"),
    "IMFContentDecryptionModuleSession": SimTypeBottom(label="IMFContentDecryptionModuleSession"),
    "IMFContentDecryptionModuleSessionCallbacks": SimTypeBottom(label="IMFContentDecryptionModuleSessionCallbacks"),
    "IMFContentDecryptorContext": SimTypeBottom(label="IMFContentDecryptorContext"),
    "IMFContentEnabler": SimTypeBottom(label="IMFContentEnabler"),
    "IMFContentProtectionDevice": SimTypeBottom(label="IMFContentProtectionDevice"),
    "IMFContentProtectionManager": SimTypeBottom(label="IMFContentProtectionManager"),
    "IMFD3D12SynchronizationObject": SimTypeBottom(label="IMFD3D12SynchronizationObject"),
    "IMFD3D12SynchronizationObjectCommands": SimTypeBottom(label="IMFD3D12SynchronizationObjectCommands"),
    "IMFDLNASinkInit": SimTypeBottom(label="IMFDLNASinkInit"),
    "IMFDRMNetHelper": SimTypeBottom(label="IMFDRMNetHelper"),
    "IMFDXGIBuffer": SimTypeBottom(label="IMFDXGIBuffer"),
    "IMFDXGIDeviceManager": SimTypeBottom(label="IMFDXGIDeviceManager"),
    "IMFDXGIDeviceManagerSource": SimTypeBottom(label="IMFDXGIDeviceManagerSource"),
    "IMFDesiredSample": SimTypeBottom(label="IMFDesiredSample"),
    "IMFDeviceTransform": SimTypeBottom(label="IMFDeviceTransform"),
    "IMFDeviceTransformCallback": SimTypeBottom(label="IMFDeviceTransformCallback"),
    "IMFExtendedCameraControl": SimTypeBottom(label="IMFExtendedCameraControl"),
    "IMFExtendedCameraController": SimTypeBottom(label="IMFExtendedCameraController"),
    "IMFExtendedCameraIntrinsicModel": SimTypeBottom(label="IMFExtendedCameraIntrinsicModel"),
    "IMFExtendedCameraIntrinsics": SimTypeBottom(label="IMFExtendedCameraIntrinsics"),
    "IMFExtendedCameraIntrinsicsDistortionModel6KT": SimTypeBottom(label="IMFExtendedCameraIntrinsicsDistortionModel6KT"),
    "IMFExtendedCameraIntrinsicsDistortionModelArcTan": SimTypeBottom(label="IMFExtendedCameraIntrinsicsDistortionModelArcTan"),
    "IMFExtendedDRMTypeSupport": SimTypeBottom(label="IMFExtendedDRMTypeSupport"),
    "IMFFieldOfUseMFTUnlock": SimTypeBottom(label="IMFFieldOfUseMFTUnlock"),
    "IMFFinalizableMediaSink": SimTypeBottom(label="IMFFinalizableMediaSink"),
    "IMFGetService": SimTypeBottom(label="IMFGetService"),
    "IMFHDCPStatus": SimTypeBottom(label="IMFHDCPStatus"),
    "IMFHttpDownloadRequest": SimTypeBottom(label="IMFHttpDownloadRequest"),
    "IMFHttpDownloadSession": SimTypeBottom(label="IMFHttpDownloadSession"),
    "IMFHttpDownloadSessionProvider": SimTypeBottom(label="IMFHttpDownloadSessionProvider"),
    "IMFImageSharingEngine": SimTypeBottom(label="IMFImageSharingEngine"),
    "IMFImageSharingEngineClassFactory": SimTypeBottom(label="IMFImageSharingEngineClassFactory"),
    "IMFInputTrustAuthority": SimTypeBottom(label="IMFInputTrustAuthority"),
    "IMFLocalMFTRegistration": SimTypeBottom(label="IMFLocalMFTRegistration"),
    "IMFMediaBuffer": SimTypeBottom(label="IMFMediaBuffer"),
    "IMFMediaEngine": SimTypeBottom(label="IMFMediaEngine"),
    "IMFMediaEngineAudioEndpointId": SimTypeBottom(label="IMFMediaEngineAudioEndpointId"),
    "IMFMediaEngineClassFactory": SimTypeBottom(label="IMFMediaEngineClassFactory"),
    "IMFMediaEngineClassFactory2": SimTypeBottom(label="IMFMediaEngineClassFactory2"),
    "IMFMediaEngineClassFactory3": SimTypeBottom(label="IMFMediaEngineClassFactory3"),
    "IMFMediaEngineClassFactory4": SimTypeBottom(label="IMFMediaEngineClassFactory4"),
    "IMFMediaEngineClassFactoryEx": SimTypeBottom(label="IMFMediaEngineClassFactoryEx"),
    "IMFMediaEngineEME": SimTypeBottom(label="IMFMediaEngineEME"),
    "IMFMediaEngineEMENotify": SimTypeBottom(label="IMFMediaEngineEMENotify"),
    "IMFMediaEngineEx": SimTypeBottom(label="IMFMediaEngineEx"),
    "IMFMediaEngineExtension": SimTypeBottom(label="IMFMediaEngineExtension"),
    "IMFMediaEngineNeedKeyNotify": SimTypeBottom(label="IMFMediaEngineNeedKeyNotify"),
    "IMFMediaEngineNotify": SimTypeBottom(label="IMFMediaEngineNotify"),
    "IMFMediaEngineOPMInfo": SimTypeBottom(label="IMFMediaEngineOPMInfo"),
    "IMFMediaEngineProtectedContent": SimTypeBottom(label="IMFMediaEngineProtectedContent"),
    "IMFMediaEngineSrcElements": SimTypeBottom(label="IMFMediaEngineSrcElements"),
    "IMFMediaEngineSrcElementsEx": SimTypeBottom(label="IMFMediaEngineSrcElementsEx"),
    "IMFMediaEngineSupportsSourceTransfer": SimTypeBottom(label="IMFMediaEngineSupportsSourceTransfer"),
    "IMFMediaEngineTransferSource": SimTypeBottom(label="IMFMediaEngineTransferSource"),
    "IMFMediaEngineWebSupport": SimTypeBottom(label="IMFMediaEngineWebSupport"),
    "IMFMediaError": SimTypeBottom(label="IMFMediaError"),
    "IMFMediaEvent": SimTypeBottom(label="IMFMediaEvent"),
    "IMFMediaEventGenerator": SimTypeBottom(label="IMFMediaEventGenerator"),
    "IMFMediaEventQueue": SimTypeBottom(label="IMFMediaEventQueue"),
    "IMFMediaKeySession": SimTypeBottom(label="IMFMediaKeySession"),
    "IMFMediaKeySession2": SimTypeBottom(label="IMFMediaKeySession2"),
    "IMFMediaKeySessionNotify": SimTypeBottom(label="IMFMediaKeySessionNotify"),
    "IMFMediaKeySessionNotify2": SimTypeBottom(label="IMFMediaKeySessionNotify2"),
    "IMFMediaKeySystemAccess": SimTypeBottom(label="IMFMediaKeySystemAccess"),
    "IMFMediaKeys": SimTypeBottom(label="IMFMediaKeys"),
    "IMFMediaKeys2": SimTypeBottom(label="IMFMediaKeys2"),
    "IMFMediaSession": SimTypeBottom(label="IMFMediaSession"),
    "IMFMediaSharingEngine": SimTypeBottom(label="IMFMediaSharingEngine"),
    "IMFMediaSharingEngineClassFactory": SimTypeBottom(label="IMFMediaSharingEngineClassFactory"),
    "IMFMediaSink": SimTypeBottom(label="IMFMediaSink"),
    "IMFMediaSinkPreroll": SimTypeBottom(label="IMFMediaSinkPreroll"),
    "IMFMediaSource": SimTypeBottom(label="IMFMediaSource"),
    "IMFMediaSource2": SimTypeBottom(label="IMFMediaSource2"),
    "IMFMediaSourceEx": SimTypeBottom(label="IMFMediaSourceEx"),
    "IMFMediaSourceExtension": SimTypeBottom(label="IMFMediaSourceExtension"),
    "IMFMediaSourceExtensionLiveSeekableRange": SimTypeBottom(label="IMFMediaSourceExtensionLiveSeekableRange"),
    "IMFMediaSourceExtensionNotify": SimTypeBottom(label="IMFMediaSourceExtensionNotify"),
    "IMFMediaSourcePresentationProvider": SimTypeBottom(label="IMFMediaSourcePresentationProvider"),
    "IMFMediaSourceTopologyProvider": SimTypeBottom(label="IMFMediaSourceTopologyProvider"),
    "IMFMediaStream": SimTypeBottom(label="IMFMediaStream"),
    "IMFMediaStream2": SimTypeBottom(label="IMFMediaStream2"),
    "IMFMediaStreamSourceSampleRequest": SimTypeBottom(label="IMFMediaStreamSourceSampleRequest"),
    "IMFMediaTimeRange": SimTypeBottom(label="IMFMediaTimeRange"),
    "IMFMediaType": SimTypeBottom(label="IMFMediaType"),
    "IMFMediaTypeHandler": SimTypeBottom(label="IMFMediaTypeHandler"),
    "IMFMetadata": SimTypeBottom(label="IMFMetadata"),
    "IMFMetadataProvider": SimTypeBottom(label="IMFMetadataProvider"),
    "IMFMuxStreamAttributesManager": SimTypeBottom(label="IMFMuxStreamAttributesManager"),
    "IMFMuxStreamMediaTypeManager": SimTypeBottom(label="IMFMuxStreamMediaTypeManager"),
    "IMFMuxStreamSampleManager": SimTypeBottom(label="IMFMuxStreamSampleManager"),
    "IMFNetCredential": SimTypeBottom(label="IMFNetCredential"),
    "IMFNetCredentialCache": SimTypeBottom(label="IMFNetCredentialCache"),
    "IMFNetCredentialManager": SimTypeBottom(label="IMFNetCredentialManager"),
    "IMFNetCrossOriginSupport": SimTypeBottom(label="IMFNetCrossOriginSupport"),
    "IMFNetProxyLocator": SimTypeBottom(label="IMFNetProxyLocator"),
    "IMFNetProxyLocatorFactory": SimTypeBottom(label="IMFNetProxyLocatorFactory"),
    "IMFNetResourceFilter": SimTypeBottom(label="IMFNetResourceFilter"),
    "IMFNetSchemeHandlerConfig": SimTypeBottom(label="IMFNetSchemeHandlerConfig"),
    "IMFObjectReferenceStream": SimTypeBottom(label="IMFObjectReferenceStream"),
    "IMFOutputPolicy": SimTypeBottom(label="IMFOutputPolicy"),
    "IMFOutputSchema": SimTypeBottom(label="IMFOutputSchema"),
    "IMFOutputTrustAuthority": SimTypeBottom(label="IMFOutputTrustAuthority"),
    "IMFPMPClient": SimTypeBottom(label="IMFPMPClient"),
    "IMFPMPClientApp": SimTypeBottom(label="IMFPMPClientApp"),
    "IMFPMPHost": SimTypeBottom(label="IMFPMPHost"),
    "IMFPMPHostApp": SimTypeBottom(label="IMFPMPHostApp"),
    "IMFPMPServer": SimTypeBottom(label="IMFPMPServer"),
    "IMFPMediaItem": SimTypeBottom(label="IMFPMediaItem"),
    "IMFPMediaPlayer": SimTypeBottom(label="IMFPMediaPlayer"),
    "IMFPMediaPlayerCallback": SimTypeBottom(label="IMFPMediaPlayerCallback"),
    "IMFPluginControl": SimTypeBottom(label="IMFPluginControl"),
    "IMFPluginControl2": SimTypeBottom(label="IMFPluginControl2"),
    "IMFPresentationClock": SimTypeBottom(label="IMFPresentationClock"),
    "IMFPresentationDescriptor": SimTypeBottom(label="IMFPresentationDescriptor"),
    "IMFPresentationTimeSource": SimTypeBottom(label="IMFPresentationTimeSource"),
    "IMFProtectedEnvironmentAccess": SimTypeBottom(label="IMFProtectedEnvironmentAccess"),
    "IMFQualityAdvise": SimTypeBottom(label="IMFQualityAdvise"),
    "IMFQualityAdvise2": SimTypeBottom(label="IMFQualityAdvise2"),
    "IMFQualityAdviseLimits": SimTypeBottom(label="IMFQualityAdviseLimits"),
    "IMFQualityManager": SimTypeBottom(label="IMFQualityManager"),
    "IMFRateControl": SimTypeBottom(label="IMFRateControl"),
    "IMFRateSupport": SimTypeBottom(label="IMFRateSupport"),
    "IMFReadWriteClassFactory": SimTypeBottom(label="IMFReadWriteClassFactory"),
    "IMFRealTimeClient": SimTypeBottom(label="IMFRealTimeClient"),
    "IMFRealTimeClientEx": SimTypeBottom(label="IMFRealTimeClientEx"),
    "IMFRelativePanelReport": SimTypeBottom(label="IMFRelativePanelReport"),
    "IMFRelativePanelWatcher": SimTypeBottom(label="IMFRelativePanelWatcher"),
    "IMFRemoteAsyncCallback": SimTypeBottom(label="IMFRemoteAsyncCallback"),
    "IMFRemoteDesktopPlugin": SimTypeBottom(label="IMFRemoteDesktopPlugin"),
    "IMFRemoteProxy": SimTypeBottom(label="IMFRemoteProxy"),
    "IMFSAMIStyle": SimTypeBottom(label="IMFSAMIStyle"),
    "IMFSSLCertificateManager": SimTypeBottom(label="IMFSSLCertificateManager"),
    "IMFSample": SimTypeBottom(label="IMFSample"),
    "IMFSampleAllocatorControl": SimTypeBottom(label="IMFSampleAllocatorControl"),
    "IMFSampleGrabberSinkCallback": SimTypeBottom(label="IMFSampleGrabberSinkCallback"),
    "IMFSampleGrabberSinkCallback2": SimTypeBottom(label="IMFSampleGrabberSinkCallback2"),
    "IMFSampleOutputStream": SimTypeBottom(label="IMFSampleOutputStream"),
    "IMFSampleProtection": SimTypeBottom(label="IMFSampleProtection"),
    "IMFSaveJob": SimTypeBottom(label="IMFSaveJob"),
    "IMFSchemeHandler": SimTypeBottom(label="IMFSchemeHandler"),
    "IMFSecureBuffer": SimTypeBottom(label="IMFSecureBuffer"),
    "IMFSecureChannel": SimTypeBottom(label="IMFSecureChannel"),
    "IMFSeekInfo": SimTypeBottom(label="IMFSeekInfo"),
    "IMFSensorActivitiesReport": SimTypeBottom(label="IMFSensorActivitiesReport"),
    "IMFSensorActivitiesReportCallback": SimTypeBottom(label="IMFSensorActivitiesReportCallback"),
    "IMFSensorActivityMonitor": SimTypeBottom(label="IMFSensorActivityMonitor"),
    "IMFSensorActivityReport": SimTypeBottom(label="IMFSensorActivityReport"),
    "IMFSensorDevice": SimTypeBottom(label="IMFSensorDevice"),
    "IMFSensorGroup": SimTypeBottom(label="IMFSensorGroup"),
    "IMFSensorProcessActivity": SimTypeBottom(label="IMFSensorProcessActivity"),
    "IMFSensorProfile": SimTypeBottom(label="IMFSensorProfile"),
    "IMFSensorProfileCollection": SimTypeBottom(label="IMFSensorProfileCollection"),
    "IMFSensorStream": SimTypeBottom(label="IMFSensorStream"),
    "IMFSensorTransformFactory": SimTypeBottom(label="IMFSensorTransformFactory"),
    "IMFSequencerSource": SimTypeBottom(label="IMFSequencerSource"),
    "IMFSharingEngineClassFactory": SimTypeBottom(label="IMFSharingEngineClassFactory"),
    "IMFShutdown": SimTypeBottom(label="IMFShutdown"),
    "IMFSignedLibrary": SimTypeBottom(label="IMFSignedLibrary"),
    "IMFSimpleAudioVolume": SimTypeBottom(label="IMFSimpleAudioVolume"),
    "IMFSinkWriter": SimTypeBottom(label="IMFSinkWriter"),
    "IMFSinkWriterCallback": SimTypeBottom(label="IMFSinkWriterCallback"),
    "IMFSinkWriterCallback2": SimTypeBottom(label="IMFSinkWriterCallback2"),
    "IMFSinkWriterEncoderConfig": SimTypeBottom(label="IMFSinkWriterEncoderConfig"),
    "IMFSinkWriterEx": SimTypeBottom(label="IMFSinkWriterEx"),
    "IMFSourceBuffer": SimTypeBottom(label="IMFSourceBuffer"),
    "IMFSourceBufferAppendMode": SimTypeBottom(label="IMFSourceBufferAppendMode"),
    "IMFSourceBufferList": SimTypeBottom(label="IMFSourceBufferList"),
    "IMFSourceBufferNotify": SimTypeBottom(label="IMFSourceBufferNotify"),
    "IMFSourceOpenMonitor": SimTypeBottom(label="IMFSourceOpenMonitor"),
    "IMFSourceReader": SimTypeBottom(label="IMFSourceReader"),
    "IMFSourceReaderCallback": SimTypeBottom(label="IMFSourceReaderCallback"),
    "IMFSourceReaderCallback2": SimTypeBottom(label="IMFSourceReaderCallback2"),
    "IMFSourceReaderEx": SimTypeBottom(label="IMFSourceReaderEx"),
    "IMFSourceResolver": SimTypeBottom(label="IMFSourceResolver"),
    "IMFSpatialAudioObjectBuffer": SimTypeBottom(label="IMFSpatialAudioObjectBuffer"),
    "IMFSpatialAudioSample": SimTypeBottom(label="IMFSpatialAudioSample"),
    "IMFStreamDescriptor": SimTypeBottom(label="IMFStreamDescriptor"),
    "IMFStreamSink": SimTypeBottom(label="IMFStreamSink"),
    "IMFStreamingSinkConfig": SimTypeBottom(label="IMFStreamingSinkConfig"),
    "IMFSystemId": SimTypeBottom(label="IMFSystemId"),
    "IMFTimecodeTranslate": SimTypeBottom(label="IMFTimecodeTranslate"),
    "IMFTimedText": SimTypeBottom(label="IMFTimedText"),
    "IMFTimedTextBinary": SimTypeBottom(label="IMFTimedTextBinary"),
    "IMFTimedTextBouten": SimTypeBottom(label="IMFTimedTextBouten"),
    "IMFTimedTextCue": SimTypeBottom(label="IMFTimedTextCue"),
    "IMFTimedTextCueList": SimTypeBottom(label="IMFTimedTextCueList"),
    "IMFTimedTextFormattedText": SimTypeBottom(label="IMFTimedTextFormattedText"),
    "IMFTimedTextNotify": SimTypeBottom(label="IMFTimedTextNotify"),
    "IMFTimedTextRegion": SimTypeBottom(label="IMFTimedTextRegion"),
    "IMFTimedTextRuby": SimTypeBottom(label="IMFTimedTextRuby"),
    "IMFTimedTextStyle": SimTypeBottom(label="IMFTimedTextStyle"),
    "IMFTimedTextStyle2": SimTypeBottom(label="IMFTimedTextStyle2"),
    "IMFTimedTextTrack": SimTypeBottom(label="IMFTimedTextTrack"),
    "IMFTimedTextTrackList": SimTypeBottom(label="IMFTimedTextTrackList"),
    "IMFTimer": SimTypeBottom(label="IMFTimer"),
    "IMFTopoLoader": SimTypeBottom(label="IMFTopoLoader"),
    "IMFTopology": SimTypeBottom(label="IMFTopology"),
    "IMFTopologyNode": SimTypeBottom(label="IMFTopologyNode"),
    "IMFTopologyNodeAttributeEditor": SimTypeBottom(label="IMFTopologyNodeAttributeEditor"),
    "IMFTopologyServiceLookup": SimTypeBottom(label="IMFTopologyServiceLookup"),
    "IMFTopologyServiceLookupClient": SimTypeBottom(label="IMFTopologyServiceLookupClient"),
    "IMFTrackedSample": SimTypeBottom(label="IMFTrackedSample"),
    "IMFTranscodeProfile": SimTypeBottom(label="IMFTranscodeProfile"),
    "IMFTranscodeSinkInfoProvider": SimTypeBottom(label="IMFTranscodeSinkInfoProvider"),
    "IMFTransform": SimTypeBottom(label="IMFTransform"),
    "IMFTrustedInput": SimTypeBottom(label="IMFTrustedInput"),
    "IMFTrustedOutput": SimTypeBottom(label="IMFTrustedOutput"),
    "IMFVideoCaptureSampleAllocator": SimTypeBottom(label="IMFVideoCaptureSampleAllocator"),
    "IMFVideoDeviceID": SimTypeBottom(label="IMFVideoDeviceID"),
    "IMFVideoDisplayControl": SimTypeBottom(label="IMFVideoDisplayControl"),
    "IMFVideoMediaType": SimTypeBottom(label="IMFVideoMediaType"),
    "IMFVideoMixerBitmap": SimTypeBottom(label="IMFVideoMixerBitmap"),
    "IMFVideoMixerControl": SimTypeBottom(label="IMFVideoMixerControl"),
    "IMFVideoMixerControl2": SimTypeBottom(label="IMFVideoMixerControl2"),
    "IMFVideoPositionMapper": SimTypeBottom(label="IMFVideoPositionMapper"),
    "IMFVideoPresenter": SimTypeBottom(label="IMFVideoPresenter"),
    "IMFVideoProcessor": SimTypeBottom(label="IMFVideoProcessor"),
    "IMFVideoProcessorControl": SimTypeBottom(label="IMFVideoProcessorControl"),
    "IMFVideoProcessorControl2": SimTypeBottom(label="IMFVideoProcessorControl2"),
    "IMFVideoProcessorControl3": SimTypeBottom(label="IMFVideoProcessorControl3"),
    "IMFVideoRenderer": SimTypeBottom(label="IMFVideoRenderer"),
    "IMFVideoRendererEffectControl": SimTypeBottom(label="IMFVideoRendererEffectControl"),
    "IMFVideoSampleAllocator": SimTypeBottom(label="IMFVideoSampleAllocator"),
    "IMFVideoSampleAllocatorCallback": SimTypeBottom(label="IMFVideoSampleAllocatorCallback"),
    "IMFVideoSampleAllocatorEx": SimTypeBottom(label="IMFVideoSampleAllocatorEx"),
    "IMFVideoSampleAllocatorNotify": SimTypeBottom(label="IMFVideoSampleAllocatorNotify"),
    "IMFVideoSampleAllocatorNotifyEx": SimTypeBottom(label="IMFVideoSampleAllocatorNotifyEx"),
    "IMFVirtualCamera": SimTypeBottom(label="IMFVirtualCamera"),
    "IMFWorkQueueServices": SimTypeBottom(label="IMFWorkQueueServices"),
    "IMFWorkQueueServicesEx": SimTypeBottom(label="IMFWorkQueueServicesEx"),
    "IMILBitmapEffect": SimTypeBottom(label="IMILBitmapEffect"),
    "IMILBitmapEffectConnections": SimTypeBottom(label="IMILBitmapEffectConnections"),
    "IMILBitmapEffectConnectionsInfo": SimTypeBottom(label="IMILBitmapEffectConnectionsInfo"),
    "IMILBitmapEffectConnector": SimTypeBottom(label="IMILBitmapEffectConnector"),
    "IMILBitmapEffectConnectorInfo": SimTypeBottom(label="IMILBitmapEffectConnectorInfo"),
    "IMILBitmapEffectEvents": SimTypeBottom(label="IMILBitmapEffectEvents"),
    "IMILBitmapEffectFactory": SimTypeBottom(label="IMILBitmapEffectFactory"),
    "IMILBitmapEffectGroup": SimTypeBottom(label="IMILBitmapEffectGroup"),
    "IMILBitmapEffectGroupImpl": SimTypeBottom(label="IMILBitmapEffectGroupImpl"),
    "IMILBitmapEffectImpl": SimTypeBottom(label="IMILBitmapEffectImpl"),
    "IMILBitmapEffectInputConnector": SimTypeBottom(label="IMILBitmapEffectInputConnector"),
    "IMILBitmapEffectInteriorInputConnector": SimTypeBottom(label="IMILBitmapEffectInteriorInputConnector"),
    "IMILBitmapEffectInteriorOutputConnector": SimTypeBottom(label="IMILBitmapEffectInteriorOutputConnector"),
    "IMILBitmapEffectOutputConnector": SimTypeBottom(label="IMILBitmapEffectOutputConnector"),
    "IMILBitmapEffectOutputConnectorImpl": SimTypeBottom(label="IMILBitmapEffectOutputConnectorImpl"),
    "IMILBitmapEffectPrimitive": SimTypeBottom(label="IMILBitmapEffectPrimitive"),
    "IMILBitmapEffectPrimitiveImpl": SimTypeBottom(label="IMILBitmapEffectPrimitiveImpl"),
    "IMILBitmapEffectRenderContext": SimTypeBottom(label="IMILBitmapEffectRenderContext"),
    "IMILBitmapEffectRenderContextImpl": SimTypeBottom(label="IMILBitmapEffectRenderContextImpl"),
    "IMILBitmapEffects": SimTypeBottom(label="IMILBitmapEffects"),
    "IMLOperatorAttributes": SimTypeBottom(label="IMLOperatorAttributes"),
    "IMLOperatorKernel": SimTypeBottom(label="IMLOperatorKernel"),
    "IMLOperatorKernelContext": SimTypeBottom(label="IMLOperatorKernelContext"),
    "IMLOperatorKernelCreationContext": SimTypeBottom(label="IMLOperatorKernelCreationContext"),
    "IMLOperatorKernelFactory": SimTypeBottom(label="IMLOperatorKernelFactory"),
    "IMLOperatorRegistry": SimTypeBottom(label="IMLOperatorRegistry"),
    "IMLOperatorShapeInferenceContext": SimTypeBottom(label="IMLOperatorShapeInferenceContext"),
    "IMLOperatorShapeInferrer": SimTypeBottom(label="IMLOperatorShapeInferrer"),
    "IMLOperatorTensor": SimTypeBottom(label="IMLOperatorTensor"),
    "IMLOperatorTensorShapeDescription": SimTypeBottom(label="IMLOperatorTensorShapeDescription"),
    "IMLOperatorTypeInferenceContext": SimTypeBottom(label="IMLOperatorTypeInferenceContext"),
    "IMLOperatorTypeInferrer": SimTypeBottom(label="IMLOperatorTypeInferrer"),
    "IMLangCodePages": SimTypeBottom(label="IMLangCodePages"),
    "IMLangConvertCharset": SimTypeBottom(label="IMLangConvertCharset"),
    "IMLangFontLink": SimTypeBottom(label="IMLangFontLink"),
    "IMLangFontLink2": SimTypeBottom(label="IMLangFontLink2"),
    "IMLangLineBreakConsole": SimTypeBottom(label="IMLangLineBreakConsole"),
    "IMLangString": SimTypeBottom(label="IMLangString"),
    "IMLangStringAStr": SimTypeBottom(label="IMLangStringAStr"),
    "IMLangStringBufA": SimTypeBottom(label="IMLangStringBufA"),
    "IMLangStringBufW": SimTypeBottom(label="IMLangStringBufW"),
    "IMLangStringWStr": SimTypeBottom(label="IMLangStringWStr"),
    "IMMCVersionInfo": SimTypeBottom(label="IMMCVersionInfo"),
    "IMMDevice": SimTypeBottom(label="IMMDevice"),
    "IMMDeviceActivator": SimTypeBottom(label="IMMDeviceActivator"),
    "IMMDeviceCollection": SimTypeBottom(label="IMMDeviceCollection"),
    "IMMDeviceEnumerator": SimTypeBottom(label="IMMDeviceEnumerator"),
    "IMMEndpoint": SimTypeBottom(label="IMMEndpoint"),
    "IMMNotificationClient": SimTypeBottom(label="IMMNotificationClient"),
    "IMMPID_CPV_ENUM": SimTypeInt(signed=False, label="IMMPID_CPV_ENUM"),
    "IMMPID_MPV_ENUM": SimTypeInt(signed=False, label="IMMPID_MPV_ENUM"),
    "IMMPID_MP_ENUM": SimTypeInt(signed=False, label="IMMPID_MP_ENUM"),
    "IMMPID_NMP_ENUM": SimTypeInt(signed=False, label="IMMPID_NMP_ENUM"),
    "IMMPID_RPV_ENUM": SimTypeInt(signed=False, label="IMMPID_RPV_ENUM"),
    "IMMPID_RP_ENUM": SimTypeInt(signed=False, label="IMMPID_RP_ENUM"),
    "IMMP_MPV_STORE_DRIVER_HANDLE": SimStruct(OrderedDict((("guidSignature", SimTypeBottom(label="Guid")),)), name="IMMP_MPV_STORE_DRIVER_HANDLE", pack=False, align=None),
    "IMPEG2Component": SimTypeBottom(label="IMPEG2Component"),
    "IMPEG2ComponentType": SimTypeBottom(label="IMPEG2ComponentType"),
    "IMPEG2PIDMap": SimTypeBottom(label="IMPEG2PIDMap"),
    "IMPEG2StreamIdMap": SimTypeBottom(label="IMPEG2StreamIdMap"),
    "IMPEG2TuneRequest": SimTypeBottom(label="IMPEG2TuneRequest"),
    "IMPEG2TuneRequestFactory": SimTypeBottom(label="IMPEG2TuneRequestFactory"),
    "IMPEG2TuneRequestSupport": SimTypeBottom(label="IMPEG2TuneRequestSupport"),
    "IMPEG2_TIF_CONTROL": SimTypeBottom(label="IMPEG2_TIF_CONTROL"),
    "IMPERSONATE_SECURITY_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "IMPLTYPEFLAGS": SimTypeInt(signed=False, label="IMPLTYPEFLAGS"),
    "IMPORT_OBJECT_HEADER": SimStruct(OrderedDict((("Sig1", SimTypeShort(signed=False, label="UInt16")), ("Sig2", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Machine", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("SizeOfData", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Ordinal": SimTypeShort(signed=False, label="UInt16"), "Hint": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="IMPORT_OBJECT_HEADER", pack=False, align=None),
    "IMPORT_OBJECT_NAME_TYPE": SimTypeInt(signed=False, label="IMPORT_OBJECT_NAME_TYPE"),
    "IMPORT_OBJECT_TYPE": SimTypeInt(signed=False, label="IMPORT_OBJECT_TYPE"),
    "IMPORT_SECURITY_CONTEXT_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "IMPORT_SECURITY_CONTEXT_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "IMSAdminBase2W": SimTypeBottom(label="IMSAdminBase2W"),
    "IMSAdminBase3W": SimTypeBottom(label="IMSAdminBase3W"),
    "IMSAdminBaseSinkW": SimTypeBottom(label="IMSAdminBaseSinkW"),
    "IMSAdminBaseW": SimTypeBottom(label="IMSAdminBaseW"),
    "IMSCEPSetup": SimTypeBottom(label="IMSCEPSetup"),
    "IMSEventBinder": SimTypeBottom(label="IMSEventBinder"),
    "IMSImpExpHelpW": SimTypeBottom(label="IMSImpExpHelpW"),
    "IMSMQApplication": SimTypeBottom(label="IMSMQApplication"),
    "IMSMQApplication2": SimTypeBottom(label="IMSMQApplication2"),
    "IMSMQApplication3": SimTypeBottom(label="IMSMQApplication3"),
    "IMSMQCollection": SimTypeBottom(label="IMSMQCollection"),
    "IMSMQCoordinatedTransactionDispenser": SimTypeBottom(label="IMSMQCoordinatedTransactionDispenser"),
    "IMSMQCoordinatedTransactionDispenser2": SimTypeBottom(label="IMSMQCoordinatedTransactionDispenser2"),
    "IMSMQCoordinatedTransactionDispenser3": SimTypeBottom(label="IMSMQCoordinatedTransactionDispenser3"),
    "IMSMQDestination": SimTypeBottom(label="IMSMQDestination"),
    "IMSMQEvent": SimTypeBottom(label="IMSMQEvent"),
    "IMSMQEvent2": SimTypeBottom(label="IMSMQEvent2"),
    "IMSMQEvent3": SimTypeBottom(label="IMSMQEvent3"),
    "IMSMQManagement": SimTypeBottom(label="IMSMQManagement"),
    "IMSMQMessage": SimTypeBottom(label="IMSMQMessage"),
    "IMSMQMessage2": SimTypeBottom(label="IMSMQMessage2"),
    "IMSMQMessage3": SimTypeBottom(label="IMSMQMessage3"),
    "IMSMQMessage4": SimTypeBottom(label="IMSMQMessage4"),
    "IMSMQOutgoingQueueManagement": SimTypeBottom(label="IMSMQOutgoingQueueManagement"),
    "IMSMQPrivateDestination": SimTypeBottom(label="IMSMQPrivateDestination"),
    "IMSMQPrivateEvent": SimTypeBottom(label="IMSMQPrivateEvent"),
    "IMSMQQuery": SimTypeBottom(label="IMSMQQuery"),
    "IMSMQQuery2": SimTypeBottom(label="IMSMQQuery2"),
    "IMSMQQuery3": SimTypeBottom(label="IMSMQQuery3"),
    "IMSMQQuery4": SimTypeBottom(label="IMSMQQuery4"),
    "IMSMQQueue": SimTypeBottom(label="IMSMQQueue"),
    "IMSMQQueue2": SimTypeBottom(label="IMSMQQueue2"),
    "IMSMQQueue3": SimTypeBottom(label="IMSMQQueue3"),
    "IMSMQQueue4": SimTypeBottom(label="IMSMQQueue4"),
    "IMSMQQueueInfo": SimTypeBottom(label="IMSMQQueueInfo"),
    "IMSMQQueueInfo2": SimTypeBottom(label="IMSMQQueueInfo2"),
    "IMSMQQueueInfo3": SimTypeBottom(label="IMSMQQueueInfo3"),
    "IMSMQQueueInfo4": SimTypeBottom(label="IMSMQQueueInfo4"),
    "IMSMQQueueInfos": SimTypeBottom(label="IMSMQQueueInfos"),
    "IMSMQQueueInfos2": SimTypeBottom(label="IMSMQQueueInfos2"),
    "IMSMQQueueInfos3": SimTypeBottom(label="IMSMQQueueInfos3"),
    "IMSMQQueueInfos4": SimTypeBottom(label="IMSMQQueueInfos4"),
    "IMSMQQueueManagement": SimTypeBottom(label="IMSMQQueueManagement"),
    "IMSMQTransaction": SimTypeBottom(label="IMSMQTransaction"),
    "IMSMQTransaction2": SimTypeBottom(label="IMSMQTransaction2"),
    "IMSMQTransaction3": SimTypeBottom(label="IMSMQTransaction3"),
    "IMSMQTransactionDispenser": SimTypeBottom(label="IMSMQTransactionDispenser"),
    "IMSMQTransactionDispenser2": SimTypeBottom(label="IMSMQTransactionDispenser2"),
    "IMSMQTransactionDispenser3": SimTypeBottom(label="IMSMQTransactionDispenser3"),
    "IMSVidAnalogTuner": SimTypeBottom(label="IMSVidAnalogTuner"),
    "IMSVidAnalogTuner2": SimTypeBottom(label="IMSVidAnalogTuner2"),
    "IMSVidAnalogTunerEvent": SimTypeBottom(label="IMSVidAnalogTunerEvent"),
    "IMSVidAudioRenderer": SimTypeBottom(label="IMSVidAudioRenderer"),
    "IMSVidAudioRendererDevices": SimTypeBottom(label="IMSVidAudioRendererDevices"),
    "IMSVidAudioRendererEvent": SimTypeBottom(label="IMSVidAudioRendererEvent"),
    "IMSVidAudioRendererEvent2": SimTypeBottom(label="IMSVidAudioRendererEvent2"),
    "IMSVidClosedCaptioning": SimTypeBottom(label="IMSVidClosedCaptioning"),
    "IMSVidClosedCaptioning2": SimTypeBottom(label="IMSVidClosedCaptioning2"),
    "IMSVidClosedCaptioning3": SimTypeBottom(label="IMSVidClosedCaptioning3"),
    "IMSVidCompositionSegment": SimTypeBottom(label="IMSVidCompositionSegment"),
    "IMSVidCtl": SimTypeBottom(label="IMSVidCtl"),
    "IMSVidDataServices": SimTypeBottom(label="IMSVidDataServices"),
    "IMSVidDataServicesEvent": SimTypeBottom(label="IMSVidDataServicesEvent"),
    "IMSVidDevice": SimTypeBottom(label="IMSVidDevice"),
    "IMSVidDevice2": SimTypeBottom(label="IMSVidDevice2"),
    "IMSVidDeviceEvent": SimTypeBottom(label="IMSVidDeviceEvent"),
    "IMSVidEVR": SimTypeBottom(label="IMSVidEVR"),
    "IMSVidEVREvent": SimTypeBottom(label="IMSVidEVREvent"),
    "IMSVidEncoder": SimTypeBottom(label="IMSVidEncoder"),
    "IMSVidFeature": SimTypeBottom(label="IMSVidFeature"),
    "IMSVidFeatureEvent": SimTypeBottom(label="IMSVidFeatureEvent"),
    "IMSVidFeatures": SimTypeBottom(label="IMSVidFeatures"),
    "IMSVidFilePlayback": SimTypeBottom(label="IMSVidFilePlayback"),
    "IMSVidFilePlayback2": SimTypeBottom(label="IMSVidFilePlayback2"),
    "IMSVidFilePlaybackEvent": SimTypeBottom(label="IMSVidFilePlaybackEvent"),
    "IMSVidGenericSink": SimTypeBottom(label="IMSVidGenericSink"),
    "IMSVidGenericSink2": SimTypeBottom(label="IMSVidGenericSink2"),
    "IMSVidGraphSegment": SimTypeBottom(label="IMSVidGraphSegment"),
    "IMSVidGraphSegmentContainer": SimTypeBottom(label="IMSVidGraphSegmentContainer"),
    "IMSVidGraphSegmentUserInput": SimTypeBottom(label="IMSVidGraphSegmentUserInput"),
    "IMSVidInputDevice": SimTypeBottom(label="IMSVidInputDevice"),
    "IMSVidInputDeviceEvent": SimTypeBottom(label="IMSVidInputDeviceEvent"),
    "IMSVidInputDevices": SimTypeBottom(label="IMSVidInputDevices"),
    "IMSVidOutputDevice": SimTypeBottom(label="IMSVidOutputDevice"),
    "IMSVidOutputDeviceEvent": SimTypeBottom(label="IMSVidOutputDeviceEvent"),
    "IMSVidOutputDevices": SimTypeBottom(label="IMSVidOutputDevices"),
    "IMSVidPlayback": SimTypeBottom(label="IMSVidPlayback"),
    "IMSVidPlaybackEvent": SimTypeBottom(label="IMSVidPlaybackEvent"),
    "IMSVidRect": SimTypeBottom(label="IMSVidRect"),
    "IMSVidStreamBufferRecordingControl": SimTypeBottom(label="IMSVidStreamBufferRecordingControl"),
    "IMSVidStreamBufferSink": SimTypeBottom(label="IMSVidStreamBufferSink"),
    "IMSVidStreamBufferSink2": SimTypeBottom(label="IMSVidStreamBufferSink2"),
    "IMSVidStreamBufferSink3": SimTypeBottom(label="IMSVidStreamBufferSink3"),
    "IMSVidStreamBufferSinkEvent": SimTypeBottom(label="IMSVidStreamBufferSinkEvent"),
    "IMSVidStreamBufferSinkEvent2": SimTypeBottom(label="IMSVidStreamBufferSinkEvent2"),
    "IMSVidStreamBufferSinkEvent3": SimTypeBottom(label="IMSVidStreamBufferSinkEvent3"),
    "IMSVidStreamBufferSinkEvent4": SimTypeBottom(label="IMSVidStreamBufferSinkEvent4"),
    "IMSVidStreamBufferSource": SimTypeBottom(label="IMSVidStreamBufferSource"),
    "IMSVidStreamBufferSource2": SimTypeBottom(label="IMSVidStreamBufferSource2"),
    "IMSVidStreamBufferSourceEvent": SimTypeBottom(label="IMSVidStreamBufferSourceEvent"),
    "IMSVidStreamBufferSourceEvent2": SimTypeBottom(label="IMSVidStreamBufferSourceEvent2"),
    "IMSVidStreamBufferSourceEvent3": SimTypeBottom(label="IMSVidStreamBufferSourceEvent3"),
    "IMSVidStreamBufferV2SourceEvent": SimTypeBottom(label="IMSVidStreamBufferV2SourceEvent"),
    "IMSVidTuner": SimTypeBottom(label="IMSVidTuner"),
    "IMSVidTunerEvent": SimTypeBottom(label="IMSVidTunerEvent"),
    "IMSVidVMR9": SimTypeBottom(label="IMSVidVMR9"),
    "IMSVidVRGraphSegment": SimTypeBottom(label="IMSVidVRGraphSegment"),
    "IMSVidVideoInputDevice": SimTypeBottom(label="IMSVidVideoInputDevice"),
    "IMSVidVideoRenderer": SimTypeBottom(label="IMSVidVideoRenderer"),
    "IMSVidVideoRenderer2": SimTypeBottom(label="IMSVidVideoRenderer2"),
    "IMSVidVideoRendererDevices": SimTypeBottom(label="IMSVidVideoRendererDevices"),
    "IMSVidVideoRendererEvent": SimTypeBottom(label="IMSVidVideoRendererEvent"),
    "IMSVidVideoRendererEvent2": SimTypeBottom(label="IMSVidVideoRendererEvent2"),
    "IMSVidWebDVD": SimTypeBottom(label="IMSVidWebDVD"),
    "IMSVidWebDVD2": SimTypeBottom(label="IMSVidWebDVD2"),
    "IMSVidWebDVDAdm": SimTypeBottom(label="IMSVidWebDVDAdm"),
    "IMSVidWebDVDEvent": SimTypeBottom(label="IMSVidWebDVDEvent"),
    "IMSVidXDS": SimTypeBottom(label="IMSVidXDS"),
    "IMSVidXDSEvent": SimTypeBottom(label="IMSVidXDSEvent"),
    "IMTSActivity": SimTypeBottom(label="IMTSActivity"),
    "IMTSCall": SimTypeBottom(label="IMTSCall"),
    "IMTSLocator": SimTypeBottom(label="IMTSLocator"),
    "IMXAttributes": SimTypeBottom(label="IMXAttributes"),
    "IMXNamespaceManager": SimTypeBottom(label="IMXNamespaceManager"),
    "IMXNamespacePrefixes": SimTypeBottom(label="IMXNamespacePrefixes"),
    "IMXReaderControl": SimTypeBottom(label="IMXReaderControl"),
    "IMXSchemaDeclHandler": SimTypeBottom(label="IMXSchemaDeclHandler"),
    "IMXWriter": SimTypeBottom(label="IMXWriter"),
    "IMXXMLFilter": SimTypeBottom(label="IMXXMLFilter"),
    "IMachineDebugManager": SimTypeBottom(label="IMachineDebugManager"),
    "IMachineDebugManagerCookie": SimTypeBottom(label="IMachineDebugManagerCookie"),
    "IMachineDebugManagerEvents": SimTypeBottom(label="IMachineDebugManagerEvents"),
    "IMachineGlobalObjectTable": SimTypeBottom(label="IMachineGlobalObjectTable"),
    "IMailUser": SimTypeBottom(label="IMailUser"),
    "IMaintenanceSettings": SimTypeBottom(label="IMaintenanceSettings"),
    "IMalloc": SimTypeBottom(label="IMalloc"),
    "IMallocSpy": SimTypeBottom(label="IMallocSpy"),
    "IManagedActivationEvents": SimTypeBottom(label="IManagedActivationEvents"),
    "IManagedObject": SimTypeBottom(label="IManagedObject"),
    "IManagedObjectInfo": SimTypeBottom(label="IManagedObjectInfo"),
    "IManagedPoolAction": SimTypeBottom(label="IManagedPoolAction"),
    "IManagedPooledObj": SimTypeBottom(label="IManagedPooledObj"),
    "IManipulationProcessor": SimTypeBottom(label="IManipulationProcessor"),
    "IMapMIMEToCLSID": SimTypeBottom(label="IMapMIMEToCLSID"),
    "IMarkupContainer": SimTypeBottom(label="IMarkupContainer"),
    "IMarkupContainer2": SimTypeBottom(label="IMarkupContainer2"),
    "IMarkupPointer": SimTypeBottom(label="IMarkupPointer"),
    "IMarkupPointer2": SimTypeBottom(label="IMarkupPointer2"),
    "IMarkupServices": SimTypeBottom(label="IMarkupServices"),
    "IMarkupServices2": SimTypeBottom(label="IMarkupServices2"),
    "IMarkupTextFrags": SimTypeBottom(label="IMarkupTextFrags"),
    "IMarshal": SimTypeBottom(label="IMarshal"),
    "IMarshal2": SimTypeBottom(label="IMarshal2"),
    "IMarshalingStream": SimTypeBottom(label="IMarshalingStream"),
    "IMathInputControl": SimTypeBottom(label="IMathInputControl"),
    "IMbnConnection": SimTypeBottom(label="IMbnConnection"),
    "IMbnConnectionContext": SimTypeBottom(label="IMbnConnectionContext"),
    "IMbnConnectionContextEvents": SimTypeBottom(label="IMbnConnectionContextEvents"),
    "IMbnConnectionEvents": SimTypeBottom(label="IMbnConnectionEvents"),
    "IMbnConnectionManager": SimTypeBottom(label="IMbnConnectionManager"),
    "IMbnConnectionManagerEvents": SimTypeBottom(label="IMbnConnectionManagerEvents"),
    "IMbnConnectionProfile": SimTypeBottom(label="IMbnConnectionProfile"),
    "IMbnConnectionProfileEvents": SimTypeBottom(label="IMbnConnectionProfileEvents"),
    "IMbnConnectionProfileManager": SimTypeBottom(label="IMbnConnectionProfileManager"),
    "IMbnConnectionProfileManagerEvents": SimTypeBottom(label="IMbnConnectionProfileManagerEvents"),
    "IMbnDeviceService": SimTypeBottom(label="IMbnDeviceService"),
    "IMbnDeviceServiceStateEvents": SimTypeBottom(label="IMbnDeviceServiceStateEvents"),
    "IMbnDeviceServicesContext": SimTypeBottom(label="IMbnDeviceServicesContext"),
    "IMbnDeviceServicesEvents": SimTypeBottom(label="IMbnDeviceServicesEvents"),
    "IMbnDeviceServicesManager": SimTypeBottom(label="IMbnDeviceServicesManager"),
    "IMbnInterface": SimTypeBottom(label="IMbnInterface"),
    "IMbnInterfaceEvents": SimTypeBottom(label="IMbnInterfaceEvents"),
    "IMbnInterfaceManager": SimTypeBottom(label="IMbnInterfaceManager"),
    "IMbnInterfaceManagerEvents": SimTypeBottom(label="IMbnInterfaceManagerEvents"),
    "IMbnMultiCarrier": SimTypeBottom(label="IMbnMultiCarrier"),
    "IMbnMultiCarrierEvents": SimTypeBottom(label="IMbnMultiCarrierEvents"),
    "IMbnPin": SimTypeBottom(label="IMbnPin"),
    "IMbnPinEvents": SimTypeBottom(label="IMbnPinEvents"),
    "IMbnPinManager": SimTypeBottom(label="IMbnPinManager"),
    "IMbnPinManagerEvents": SimTypeBottom(label="IMbnPinManagerEvents"),
    "IMbnRadio": SimTypeBottom(label="IMbnRadio"),
    "IMbnRadioEvents": SimTypeBottom(label="IMbnRadioEvents"),
    "IMbnRegistration": SimTypeBottom(label="IMbnRegistration"),
    "IMbnRegistrationEvents": SimTypeBottom(label="IMbnRegistrationEvents"),
    "IMbnServiceActivation": SimTypeBottom(label="IMbnServiceActivation"),
    "IMbnServiceActivationEvents": SimTypeBottom(label="IMbnServiceActivationEvents"),
    "IMbnSignal": SimTypeBottom(label="IMbnSignal"),
    "IMbnSignalEvents": SimTypeBottom(label="IMbnSignalEvents"),
    "IMbnSms": SimTypeBottom(label="IMbnSms"),
    "IMbnSmsConfiguration": SimTypeBottom(label="IMbnSmsConfiguration"),
    "IMbnSmsEvents": SimTypeBottom(label="IMbnSmsEvents"),
    "IMbnSmsReadMsgPdu": SimTypeBottom(label="IMbnSmsReadMsgPdu"),
    "IMbnSmsReadMsgTextCdma": SimTypeBottom(label="IMbnSmsReadMsgTextCdma"),
    "IMbnSubscriberInformation": SimTypeBottom(label="IMbnSubscriberInformation"),
    "IMbnVendorSpecificEvents": SimTypeBottom(label="IMbnVendorSpecificEvents"),
    "IMbnVendorSpecificOperation": SimTypeBottom(label="IMbnVendorSpecificOperation"),
    "IMcastAddressAllocation": SimTypeBottom(label="IMcastAddressAllocation"),
    "IMcastLeaseInfo": SimTypeBottom(label="IMcastLeaseInfo"),
    "IMcastScope": SimTypeBottom(label="IMcastScope"),
    "IMceBurnerControl": SimTypeBottom(label="IMceBurnerControl"),
    "IMediaActivityNotifySite": SimTypeBottom(label="IMediaActivityNotifySite"),
    "IMediaBuffer": SimTypeBottom(label="IMediaBuffer"),
    "IMediaControl": SimTypeBottom(label="IMediaControl"),
    "IMediaEvent": SimTypeBottom(label="IMediaEvent"),
    "IMediaEventEx": SimTypeBottom(label="IMediaEventEx"),
    "IMediaEventSink": SimTypeBottom(label="IMediaEventSink"),
    "IMediaFilter": SimTypeBottom(label="IMediaFilter"),
    "IMediaObject": SimTypeBottom(label="IMediaObject"),
    "IMediaObjectInPlace": SimTypeBottom(label="IMediaObjectInPlace"),
    "IMediaParamInfo": SimTypeBottom(label="IMediaParamInfo"),
    "IMediaParams": SimTypeBottom(label="IMediaParams"),
    "IMediaPosition": SimTypeBottom(label="IMediaPosition"),
    "IMediaPropertyBag": SimTypeBottom(label="IMediaPropertyBag"),
    "IMediaRadioManager": SimTypeBottom(label="IMediaRadioManager"),
    "IMediaRadioManagerNotifySink": SimTypeBottom(label="IMediaRadioManagerNotifySink"),
    "IMediaSample": SimTypeBottom(label="IMediaSample"),
    "IMediaSample2": SimTypeBottom(label="IMediaSample2"),
    "IMediaSample2Config": SimTypeBottom(label="IMediaSample2Config"),
    "IMediaSeeking": SimTypeBottom(label="IMediaSeeking"),
    "IMediaStream": SimTypeBottom(label="IMediaStream"),
    "IMediaStreamFilter": SimTypeBottom(label="IMediaStreamFilter"),
    "IMediaTypeInfo": SimTypeBottom(label="IMediaTypeInfo"),
    "IMemAllocator": SimTypeBottom(label="IMemAllocator"),
    "IMemAllocatorCallbackTemp": SimTypeBottom(label="IMemAllocatorCallbackTemp"),
    "IMemAllocatorNotifyCallbackTemp": SimTypeBottom(label="IMemAllocatorNotifyCallbackTemp"),
    "IMemInputPin": SimTypeBottom(label="IMemInputPin"),
    "IMemoryAllocator": SimTypeBottom(label="IMemoryAllocator"),
    "IMemoryData": SimTypeBottom(label="IMemoryData"),
    "IMenuBand": SimTypeBottom(label="IMenuBand"),
    "IMenuButton": SimTypeBottom(label="IMenuButton"),
    "IMenuPopup": SimTypeBottom(label="IMenuPopup"),
    "IMessage": SimTypeBottom(label="IMessage"),
    "IMessageFilter": SimTypeBottom(label="IMessageFilter"),
    "IMessageMover": SimTypeBottom(label="IMessageMover"),
    "IMessageView": SimTypeBottom(label="IMessageView"),
    "IMetaData": SimTypeBottom(label="IMetaData"),
    "IMethodMalloc": SimTypeBottom(label="IMethodMalloc"),
    "IMixerOCX": SimTypeBottom(label="IMixerOCX"),
    "IMixerOCXNotify": SimTypeBottom(label="IMixerOCXNotify"),
    "IMixerPinConfig": SimTypeBottom(label="IMixerPinConfig"),
    "IMixerPinConfig2": SimTypeBottom(label="IMixerPinConfig2"),
    "IModalWindow": SimTypeBottom(label="IModalWindow"),
    "IModelIterator": SimTypeBottom(label="IModelIterator"),
    "IModelKeyReference": SimTypeBottom(label="IModelKeyReference"),
    "IModelKeyReference2": SimTypeBottom(label="IModelKeyReference2"),
    "IModelMethod": SimTypeBottom(label="IModelMethod"),
    "IModelObject": SimTypeBottom(label="IModelObject"),
    "IModelPropertyAccessor": SimTypeBottom(label="IModelPropertyAccessor"),
    "IMofCompiler": SimTypeBottom(label="IMofCompiler"),
    "IMoniker": SimTypeBottom(label="IMoniker"),
    "IMonikerProp": SimTypeBottom(label="IMonikerProp"),
    "IMonthlyDOWTrigger": SimTypeBottom(label="IMonthlyDOWTrigger"),
    "IMonthlyTrigger": SimTypeBottom(label="IMonthlyTrigger"),
    "IMpeg2Data": SimTypeBottom(label="IMpeg2Data"),
    "IMpeg2Demultiplexer": SimTypeBottom(label="IMpeg2Demultiplexer"),
    "IMpeg2Stream": SimTypeBottom(label="IMpeg2Stream"),
    "IMpeg2TableFilter": SimTypeBottom(label="IMpeg2TableFilter"),
    "IMpegAudioDecoder": SimTypeBottom(label="IMpegAudioDecoder"),
    "IMsgStore": SimTypeBottom(label="IMsgStore"),
    "IMsmDependencies": SimTypeBottom(label="IMsmDependencies"),
    "IMsmDependency": SimTypeBottom(label="IMsmDependency"),
    "IMsmError": SimTypeBottom(label="IMsmError"),
    "IMsmErrors": SimTypeBottom(label="IMsmErrors"),
    "IMsmGetFiles": SimTypeBottom(label="IMsmGetFiles"),
    "IMsmMerge": SimTypeBottom(label="IMsmMerge"),
    "IMsmStrings": SimTypeBottom(label="IMsmStrings"),
    "IMtsEventInfo": SimTypeBottom(label="IMtsEventInfo"),
    "IMtsEvents": SimTypeBottom(label="IMtsEvents"),
    "IMtsGrp": SimTypeBottom(label="IMtsGrp"),
    "IMultiInterfaceEventControl": SimTypeBottom(label="IMultiInterfaceEventControl"),
    "IMultiInterfacePublisherFilter": SimTypeBottom(label="IMultiInterfacePublisherFilter"),
    "IMultiLanguage": SimTypeBottom(label="IMultiLanguage"),
    "IMultiLanguage2": SimTypeBottom(label="IMultiLanguage2"),
    "IMultiLanguage3": SimTypeBottom(label="IMultiLanguage3"),
    "IMultiMediaStream": SimTypeBottom(label="IMultiMediaStream"),
    "IMultiQI": SimTypeBottom(label="IMultiQI"),
    "IMultipleResults": SimTypeBottom(label="IMultipleResults"),
    "IMultipleViewProvider": SimTypeBottom(label="IMultipleViewProvider"),
    "IMultisession": SimTypeBottom(label="IMultisession"),
    "IMultisessionRandomWrite": SimTypeBottom(label="IMultisessionRandomWrite"),
    "IMultisessionSequential": SimTypeBottom(label="IMultisessionSequential"),
    "IMultisessionSequential2": SimTypeBottom(label="IMultisessionSequential2"),
    "IN6_ADDR": SimStruct(OrderedDict((("u", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")),)), name="IN6_ADDR", pack=False, align=None),
    "IN6_PKTINFO": SimStruct(OrderedDict((("ipi6_addr", SimTypeRef("IN6_ADDR", SimStruct)), ("ipi6_ifindex", SimTypeInt(signed=False, label="UInt32")),)), name="IN6_PKTINFO", pack=False, align=None),
    "IN6_PKTINFO_EX": SimStruct(OrderedDict((("pkt_info", SimTypeRef("IN6_PKTINFO", SimStruct)), ("scope_id", SimTypeRef("SCOPE_ID", SimStruct)),)), name="IN6_PKTINFO_EX", pack=False, align=None),
    "INATEventManager": SimTypeBottom(label="INATEventManager"),
    "INATExternalIPAddressCallback": SimTypeBottom(label="INATExternalIPAddressCallback"),
    "INATNumberOfEntriesCallback": SimTypeBottom(label="INATNumberOfEntriesCallback"),
    "INCREMENTAL_ACCESS_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ftLastModifiedTime", SimTypeRef("FILETIME", SimStruct)),)), name="INCREMENTAL_ACCESS_INFO", pack=False, align=None),
    "INDESPolicy": SimTypeBottom(label="INDESPolicy"),
    "INDICATOR_LIST": SimStruct(OrderedDict((("MakeCode", SimTypeShort(signed=False, label="UInt16")), ("IndicatorFlags", SimTypeShort(signed=False, label="UInt16")),)), name="INDICATOR_LIST", pack=False, align=None),
    "INDIRECT_DISPLAY_INFO": SimStruct(OrderedDict((("DisplayAdapterLuid", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumMonitors", SimTypeInt(signed=False, label="UInt32")), ("DisplayAdapterTargetBase", SimTypeInt(signed=False, label="UInt32")),)), name="INDIRECT_DISPLAY_INFO", pack=False, align=None),
    "INERTIA_PARAMETER": SimTypeInt(signed=False, label="INERTIA_PARAMETER"),
    "INET_FIREWALL_AC_BINARIES": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("binaries", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="INET_FIREWALL_AC_BINARIES", pack=False, align=None),
    "INET_FIREWALL_AC_CAPABILITIES": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("capabilities", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)),)), name="INET_FIREWALL_AC_CAPABILITIES", pack=False, align=None),
    "INET_FIREWALL_AC_CHANGE": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="INET_FIREWALL_AC_CHANGE_TYPE")), ("createType", SimTypeInt(signed=False, label="INET_FIREWALL_AC_CREATION_TYPE")), ("appContainerSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("userSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("displayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"capabilities": SimTypeRef("INET_FIREWALL_AC_CAPABILITIES", SimStruct), "binaries": SimTypeRef("INET_FIREWALL_AC_BINARIES", SimStruct)}, name="<anon>", label="None")),)), name="INET_FIREWALL_AC_CHANGE", pack=False, align=None),
    "INET_FIREWALL_AC_CHANGE_TYPE": SimTypeInt(signed=False, label="INET_FIREWALL_AC_CHANGE_TYPE"),
    "INET_FIREWALL_AC_CREATION_TYPE": SimTypeInt(signed=False, label="INET_FIREWALL_AC_CREATION_TYPE"),
    "INET_FIREWALL_APP_CONTAINER": SimStruct(OrderedDict((("appContainerSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("userSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)), ("appContainerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("displayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("capabilities", SimTypeRef("INET_FIREWALL_AC_CAPABILITIES", SimStruct)), ("binaries", SimTypeRef("INET_FIREWALL_AC_BINARIES", SimStruct)), ("workingDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("packageFullName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="INET_FIREWALL_APP_CONTAINER", pack=False, align=None),
    "INET_PORT_RANGE": SimStruct(OrderedDict((("StartPort", SimTypeShort(signed=False, label="UInt16")), ("NumberOfPorts", SimTypeShort(signed=False, label="UInt16")),)), name="INET_PORT_RANGE", pack=False, align=None),
    "INET_PORT_RESERVATION_INFORMATION": SimStruct(OrderedDict((("OwningPid", SimTypeInt(signed=False, label="UInt32")),)), name="INET_PORT_RESERVATION_INFORMATION", pack=False, align=None),
    "INET_PORT_RESERVATION_INSTANCE": SimStruct(OrderedDict((("Reservation", SimTypeRef("INET_PORT_RANGE", SimStruct)), ("Token", SimTypeRef("INET_PORT_RESERVATION_TOKEN", SimStruct)),)), name="INET_PORT_RESERVATION_INSTANCE", pack=False, align=None),
    "INET_PORT_RESERVATION_TOKEN": SimStruct(OrderedDict((("Token", SimTypeLongLong(signed=False, label="UInt64")),)), name="INET_PORT_RESERVATION_TOKEN", pack=False, align=None),
    "INET_ZONE_MANAGER_CONSTANTS": SimTypeInt(signed=False, label="INET_ZONE_MANAGER_CONSTANTS"),
    "INFCONTEXT": SimStruct(OrderedDict((("Inf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CurrentInf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Section", SimTypeInt(signed=False, label="UInt32")), ("Line", SimTypeInt(signed=False, label="UInt32")),)), name="INFCONTEXT", pack=False, align=None),
    "INFORMATIONCARD_ASYMMETRIC_CRYPTO_PARAMETERS": SimStruct(OrderedDict((("keySize", SimTypeInt(signed=True, label="Int32")), ("keyExchangeAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("signatureAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="INFORMATIONCARD_ASYMMETRIC_CRYPTO_PARAMETERS", pack=False, align=None),
    "INFORMATIONCARD_CRYPTO_HANDLE": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="HandleType")), ("expiration", SimTypeLongLong(signed=True, label="Int64")), ("cryptoParameters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="INFORMATIONCARD_CRYPTO_HANDLE", pack=False, align=None),
    "INFORMATIONCARD_HASH_CRYPTO_PARAMETERS": SimStruct(OrderedDict((("hashSize", SimTypeInt(signed=True, label="Int32")), ("transform", SimTypeRef("INFORMATIONCARD_TRANSFORM_CRYPTO_PARAMETERS", SimStruct)),)), name="INFORMATIONCARD_HASH_CRYPTO_PARAMETERS", pack=False, align=None),
    "INFORMATIONCARD_SYMMETRIC_CRYPTO_PARAMETERS": SimStruct(OrderedDict((("keySize", SimTypeInt(signed=True, label="Int32")), ("blockSize", SimTypeInt(signed=True, label="Int32")), ("feedbackSize", SimTypeInt(signed=True, label="Int32")),)), name="INFORMATIONCARD_SYMMETRIC_CRYPTO_PARAMETERS", pack=False, align=None),
    "INFORMATIONCARD_TRANSFORM_CRYPTO_PARAMETERS": SimStruct(OrderedDict((("inputBlockSize", SimTypeInt(signed=True, label="Int32")), ("outputBlockSize", SimTypeInt(signed=True, label="Int32")), ("canTransformMultipleBlocks", SimTypeInt(signed=True, label="Int32")), ("canReuseTransform", SimTypeInt(signed=True, label="Int32")),)), name="INFORMATIONCARD_TRANSFORM_CRYPTO_PARAMETERS", pack=False, align=None),
    "INF_STYLE": SimTypeInt(signed=False, label="INF_STYLE"),
    "INHERITED_FROMA": SimStruct(OrderedDict((("GenerationGap", SimTypeInt(signed=True, label="Int32")), ("AncestorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="INHERITED_FROMA", pack=False, align=None),
    "INHERITED_FROMW": SimStruct(OrderedDict((("GenerationGap", SimTypeInt(signed=True, label="Int32")), ("AncestorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="INHERITED_FROMW", pack=False, align=None),
    "INITCOMMONCONTROLSEX": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwICC", SimTypeInt(signed=False, label="INITCOMMONCONTROLSEX_ICC")),)), name="INITCOMMONCONTROLSEX", pack=False, align=None),
    "INITCOMMONCONTROLSEX_ICC": SimTypeInt(signed=False, label="INITCOMMONCONTROLSEX_ICC"),
    "INITIALIZE_SECURITY_CONTEXT_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10", "param11"]), offset=0),
    "INITIALIZE_SECURITY_CONTEXT_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10", "param11"]), offset=0),
    "INITIAL_PRIVILEGE_SET": SimStruct(OrderedDict((("PrivilegeCount", SimTypeInt(signed=False, label="UInt32")), ("Control", SimTypeInt(signed=False, label="UInt32")), ("Privilege", SimTypeArray(SimTypeRef("LUID_AND_ATTRIBUTES", SimStruct), 3)),)), name="INITIAL_PRIVILEGE_SET", pack=False, align=None),
    "INIT_ONCE": SimUnion({"Ptr": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"),
    "INIT_SECURITY_INTERFACE_A": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("SecurityFunctionTableA", SimStruct), offset=0)), offset=0),
    "INIT_SECURITY_INTERFACE_W": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("SecurityFunctionTableW", SimStruct), offset=0)), offset=0),
    "INKMETRIC": SimStruct(OrderedDict((("iHeight", SimTypeInt(signed=True, label="Int32")), ("iFontAscent", SimTypeInt(signed=True, label="Int32")), ("iFontDescent", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("color", SimTypeInt(signed=False, label="UInt32")),)), name="INKMETRIC", pack=False, align=None),
    "INK_HIGH_CONTRAST_ADJUSTMENT": SimTypeInt(signed=False, label="INK_HIGH_CONTRAST_ADJUSTMENT"),
    "INK_METRIC_FLAGS": SimTypeInt(signed=False, label="INK_METRIC_FLAGS"),
    "INLINE_FRAME_CONTEXT": SimUnion({"ContextValue": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("FrameId", SimTypeChar(label="Byte")), ("FrameType", SimTypeChar(label="Byte")), ("FrameSignature", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "INPUT": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="INPUT_TYPE")), ("Anonymous", SimUnion({"mi": SimTypeRef("MOUSEINPUT", SimStruct), "ki": SimTypeRef("KEYBDINPUT", SimStruct), "hi": SimTypeRef("HARDWAREINPUT", SimStruct)}, name="<anon>", label="None")),)), name="INPUT", pack=False, align=None),
    "INPUTCONTEXT": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fOpen", SimTypeInt(signed=True, label="Int32")), ("ptStatusWndPos", SimTypeRef("POINT", SimStruct)), ("ptSoftKbdPos", SimTypeRef("POINT", SimStruct)), ("fdwConversion", SimTypeInt(signed=False, label="UInt32")), ("fdwSentence", SimTypeInt(signed=False, label="UInt32")), ("lfFont", SimUnion({"A": SimTypeRef("LOGFONTA", SimStruct), "W": SimTypeRef("LOGFONTW", SimStruct)}, name="<anon>", label="None")), ("cfCompForm", SimTypeRef("COMPOSITIONFORM", SimStruct)), ("cfCandForm", SimTypeArray(SimTypeRef("CANDIDATEFORM", SimStruct), 4)), ("hCompStr", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCandInfo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hGuideLine", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hPrivate", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwNumMsgBuf", SimTypeInt(signed=False, label="UInt32")), ("hMsgBuf", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fdwInit", SimTypeInt(signed=False, label="UInt32")), ("dwReserve", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="INPUTCONTEXT", pack=False, align=None),
    "INPUT_BUTTON_ENABLE_INFO": SimStruct(OrderedDict((("ButtonType", SimTypeInt(signed=False, label="GPIOBUTTONS_BUTTON_TYPE")), ("Enabled", SimTypeChar(label="Byte")),)), name="INPUT_BUTTON_ENABLE_INFO", pack=False, align=None),
    "INPUT_INJECTION_VALUE": SimStruct(OrderedDict((("page", SimTypeShort(signed=False, label="UInt16")), ("usage", SimTypeShort(signed=False, label="UInt16")), ("value", SimTypeInt(signed=True, label="Int32")), ("index", SimTypeShort(signed=False, label="UInt16")),)), name="INPUT_INJECTION_VALUE", pack=False, align=None),
    "INPUT_MAPPING_ELEMENT": SimStruct(OrderedDict((("InputMappingId", SimTypeInt(signed=False, label="UInt32")),)), name="INPUT_MAPPING_ELEMENT", pack=False, align=None),
    "INPUT_MESSAGE_DEVICE_TYPE": SimTypeInt(signed=False, label="INPUT_MESSAGE_DEVICE_TYPE"),
    "INPUT_MESSAGE_ORIGIN_ID": SimTypeInt(signed=False, label="INPUT_MESSAGE_ORIGIN_ID"),
    "INPUT_MESSAGE_SOURCE": SimStruct(OrderedDict((("deviceType", SimTypeInt(signed=False, label="INPUT_MESSAGE_DEVICE_TYPE")), ("originId", SimTypeInt(signed=False, label="INPUT_MESSAGE_ORIGIN_ID")),)), name="INPUT_MESSAGE_SOURCE", pack=False, align=None),
    "INPUT_RECORD": SimStruct(OrderedDict((("EventType", SimTypeShort(signed=False, label="UInt16")), ("Event", SimUnion({"KeyEvent": SimTypeRef("KEY_EVENT_RECORD", SimStruct), "MouseEvent": SimTypeRef("MOUSE_EVENT_RECORD", SimStruct), "WindowBufferSizeEvent": SimTypeRef("WINDOW_BUFFER_SIZE_RECORD", SimStruct), "MenuEvent": SimTypeRef("MENU_EVENT_RECORD", SimStruct), "FocusEvent": SimTypeRef("FOCUS_EVENT_RECORD", SimStruct)}, name="<anon>", label="None")),)), name="INPUT_RECORD", pack=False, align=None),
    "INPUT_TRANSFORM": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_13", SimTypeFloat(size=32)), ("_14", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_23", SimTypeFloat(size=32)), ("_24", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)), ("_33", SimTypeFloat(size=32)), ("_34", SimTypeFloat(size=32)), ("_41", SimTypeFloat(size=32)), ("_42", SimTypeFloat(size=32)), ("_43", SimTypeFloat(size=32)), ("_44", SimTypeFloat(size=32)),)), name="_Anonymous_e__Struct", pack=False, align=None), "m": SimTypeArray(SimTypeFloat(size=32), 16)}, name="<anon>", label="None")),)), name="INPUT_TRANSFORM", pack=False, align=None),
    "INPUT_TYPE": SimTypeInt(signed=False, label="INPUT_TYPE"),
    "INSERTPSUIPAGE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeChar(label="Byte")), ("Mode", SimTypeChar(label="Byte")), ("dwData1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwData2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwData3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="INSERTPSUIPAGE_INFO", pack=False, align=None),
    "INSERT_OBJECT_FLAGS": SimTypeInt(signed=False, label="INSERT_OBJECT_FLAGS"),
    "INSERT_TEXT_AT_SELECTION_FLAGS": SimTypeInt(signed=False, label="INSERT_TEXT_AT_SELECTION_FLAGS"),
    "INSNetSourceCreator": SimTypeBottom(label="INSNetSourceCreator"),
    "INSSBuffer": SimTypeBottom(label="INSSBuffer"),
    "INSSBuffer2": SimTypeBottom(label="INSSBuffer2"),
    "INSSBuffer3": SimTypeBottom(label="INSSBuffer3"),
    "INSSBuffer4": SimTypeBottom(label="INSSBuffer4"),
    "INSTALLDATA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="INSTALLSPECTYPE")), ("Spec", SimUnion({"AppName": SimTypeRef("_AppName_e__Struct", SimStruct), "FileExt": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ProgId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "COMClass": SimTypeRef("_COMClass_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="INSTALLDATA", pack=False, align=None),
    "INSTALLFEATUREATTRIBUTE": SimTypeInt(signed=False, label="INSTALLFEATUREATTRIBUTE"),
    "INSTALLLEVEL": SimTypeInt(signed=False, label="INSTALLLEVEL"),
    "INSTALLLOGATTRIBUTES": SimTypeInt(signed=False, label="INSTALLLOGATTRIBUTES"),
    "INSTALLLOGMODE": SimTypeInt(signed=False, label="INSTALLLOGMODE"),
    "INSTALLMESSAGE": SimTypeInt(signed=False, label="INSTALLMESSAGE"),
    "INSTALLMODE": SimTypeInt(signed=False, label="INSTALLMODE"),
    "INSTALLSPEC": SimUnion({"AppName": SimTypeRef("_AppName_e__Struct", SimStruct), "FileExt": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ProgId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "COMClass": SimTypeRef("_COMClass_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "INSTALLSPECTYPE": SimTypeInt(signed=False, label="INSTALLSPECTYPE"),
    "INSTALLSTATE": SimTypeInt(signed=False, label="INSTALLSTATE"),
    "INSTALLTYPE": SimTypeInt(signed=False, label="INSTALLTYPE"),
    "INSTALLUILEVEL": SimTypeInt(signed=False, label="INSTALLUILEVEL"),
    "INSTALLUI_HANDLERA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvContext", "iMessageType", "szMessage"]), offset=0),
    "INSTALLUI_HANDLERW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvContext", "iMessageType", "szMessage"]), offset=0),
    "INSTANCE_AGGREGATE_STANDARD_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Type", SimUnion({"MiniFilter": SimTypeRef("_MiniFilter_e__Struct", SimStruct), "LegacyFilter": SimTypeRef("_LegacyFilter_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="INSTANCE_AGGREGATE_STANDARD_INFORMATION", pack=False, align=None),
    "INSTANCE_BASIC_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("InstanceNameLength", SimTypeShort(signed=False, label="UInt16")), ("InstanceNameBufferOffset", SimTypeShort(signed=False, label="UInt16")),)), name="INSTANCE_BASIC_INFORMATION", pack=False, align=None),
    "INSTANCE_FULL_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("InstanceNameLength", SimTypeShort(signed=False, label="UInt16")), ("InstanceNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("AltitudeLength", SimTypeShort(signed=False, label="UInt16")), ("AltitudeBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("VolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("VolumeNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("FilterNameLength", SimTypeShort(signed=False, label="UInt16")), ("FilterNameBufferOffset", SimTypeShort(signed=False, label="UInt16")),)), name="INSTANCE_FULL_INFORMATION", pack=False, align=None),
    "INSTANCE_INFORMATION_CLASS": SimTypeInt(signed=False, label="INSTANCE_INFORMATION_CLASS"),
    "INSTANCE_PARTIAL_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("InstanceNameLength", SimTypeShort(signed=False, label="UInt16")), ("InstanceNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("AltitudeLength", SimTypeShort(signed=False, label="UInt16")), ("AltitudeBufferOffset", SimTypeShort(signed=False, label="UInt16")),)), name="INSTANCE_PARTIAL_INFORMATION", pack=False, align=None),
    "INSTHEADER": SimStruct(OrderedDict((("cRegions", SimTypeInt(signed=False, label="UInt32")), ("Locale", SimTypeRef("MIDILOCALE", SimStruct)),)), name="INSTHEADER", pack=False, align=None),
    "INTEL_CACHE_INFO_EAX": SimUnion({"Ulong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "INTEL_CACHE_INFO_EBX": SimUnion({"Ulong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "INTEL_CACHE_TYPE": SimTypeInt(signed=False, label="INTEL_CACHE_TYPE"),
    "INTENT_TO_SEAL_ATTRIBUTE": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("seal", SimTypeChar(label="Byte")),)), name="INTENT_TO_SEAL_ATTRIBUTE", pack=False, align=None),
    "INTERACTION_ARGUMENTS_CROSS_SLIDE": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="CROSS_SLIDE_FLAGS")),)), name="INTERACTION_ARGUMENTS_CROSS_SLIDE", pack=False, align=None),
    "INTERACTION_ARGUMENTS_MANIPULATION": SimStruct(OrderedDict((("delta", SimTypeRef("MANIPULATION_TRANSFORM", SimStruct)), ("cumulative", SimTypeRef("MANIPULATION_TRANSFORM", SimStruct)), ("velocity", SimTypeRef("MANIPULATION_VELOCITY", SimStruct)), ("railsState", SimTypeInt(signed=False, label="MANIPULATION_RAILS_STATE")),)), name="INTERACTION_ARGUMENTS_MANIPULATION", pack=False, align=None),
    "INTERACTION_ARGUMENTS_TAP": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")),)), name="INTERACTION_ARGUMENTS_TAP", pack=False, align=None),
    "INTERACTION_CONFIGURATION_FLAGS": SimTypeInt(signed=False, label="INTERACTION_CONFIGURATION_FLAGS"),
    "INTERACTION_CONTEXT_CONFIGURATION": SimStruct(OrderedDict((("interactionId", SimTypeInt(signed=False, label="INTERACTION_ID")), ("enable", SimTypeInt(signed=False, label="INTERACTION_CONFIGURATION_FLAGS")),)), name="INTERACTION_CONTEXT_CONFIGURATION", pack=False, align=None),
    "INTERACTION_CONTEXT_OUTPUT": SimStruct(OrderedDict((("interactionId", SimTypeInt(signed=False, label="INTERACTION_ID")), ("interactionFlags", SimTypeInt(signed=False, label="INTERACTION_FLAGS")), ("inputType", SimTypeInt(signed=False, label="POINTER_INPUT_TYPE")), ("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("arguments", SimUnion({"manipulation": SimTypeRef("INTERACTION_ARGUMENTS_MANIPULATION", SimStruct), "tap": SimTypeRef("INTERACTION_ARGUMENTS_TAP", SimStruct), "crossSlide": SimTypeRef("INTERACTION_ARGUMENTS_CROSS_SLIDE", SimStruct)}, name="<anon>", label="None")),)), name="INTERACTION_CONTEXT_OUTPUT", pack=False, align=None),
    "INTERACTION_CONTEXT_OUTPUT2": SimStruct(OrderedDict((("interactionId", SimTypeInt(signed=False, label="INTERACTION_ID")), ("interactionFlags", SimTypeInt(signed=False, label="INTERACTION_FLAGS")), ("inputType", SimTypeInt(signed=False, label="POINTER_INPUT_TYPE")), ("contactCount", SimTypeInt(signed=False, label="UInt32")), ("currentContactCount", SimTypeInt(signed=False, label="UInt32")), ("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("arguments", SimUnion({"manipulation": SimTypeRef("INTERACTION_ARGUMENTS_MANIPULATION", SimStruct), "tap": SimTypeRef("INTERACTION_ARGUMENTS_TAP", SimStruct), "crossSlide": SimTypeRef("INTERACTION_ARGUMENTS_CROSS_SLIDE", SimStruct)}, name="<anon>", label="None")),)), name="INTERACTION_CONTEXT_OUTPUT2", pack=False, align=None),
    "INTERACTION_CONTEXT_OUTPUT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("INTERACTION_CONTEXT_OUTPUT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["clientData", "output"]), offset=0),
    "INTERACTION_CONTEXT_OUTPUT_CALLBACK2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("INTERACTION_CONTEXT_OUTPUT2", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["clientData", "output"]), offset=0),
    "INTERACTION_CONTEXT_PROPERTY": SimTypeInt(signed=False, label="INTERACTION_CONTEXT_PROPERTY"),
    "INTERACTION_FLAGS": SimTypeInt(signed=False, label="INTERACTION_FLAGS"),
    "INTERACTION_ID": SimTypeInt(signed=False, label="INTERACTION_ID"),
    "INTERACTION_STATE": SimTypeInt(signed=False, label="INTERACTION_STATE"),
    "INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)),)), name="INTERFACE", pack=False, align=None),
    "INTERFACEDATA": SimStruct(OrderedDict((("pmethdata", SimTypePointer(SimTypeRef("METHODDATA", SimStruct), offset=0)), ("cMembers", SimTypeInt(signed=False, label="UInt32")),)), name="INTERFACEDATA", pack=False, align=None),
    "INTERFACEINFO": SimStruct(OrderedDict((("pUnk", SimTypeBottom(label="IUnknown")), ("iid", SimTypeBottom(label="Guid")), ("wMethod", SimTypeShort(signed=False, label="UInt16")),)), name="INTERFACEINFO", pack=False, align=None),
    "INTERFACE_HARDWARE_CROSSTIMESTAMP": SimStruct(OrderedDict((("SystemTimestamp1", SimTypeLongLong(signed=False, label="UInt64")), ("HardwareClockTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("SystemTimestamp2", SimTypeLongLong(signed=False, label="UInt64")),)), name="INTERFACE_HARDWARE_CROSSTIMESTAMP", pack=False, align=None),
    "INTERFACE_HARDWARE_TIMESTAMP_CAPABILITIES": SimStruct(OrderedDict((("PtpV2OverUdpIPv4EventMessageReceive", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv4AllMessageReceive", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv4EventMessageTransmit", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv4AllMessageTransmit", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6EventMessageReceive", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6AllMessageReceive", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6EventMessageTransmit", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6AllMessageTransmit", SimTypeChar(label="Byte")), ("AllReceive", SimTypeChar(label="Byte")), ("AllTransmit", SimTypeChar(label="Byte")), ("TaggedTransmit", SimTypeChar(label="Byte")),)), name="INTERFACE_HARDWARE_TIMESTAMP_CAPABILITIES", pack=False, align=None),
    "INTERFACE_INFO": SimStruct(OrderedDict((("iiFlags", SimTypeInt(signed=False, label="UInt32")), ("iiAddress", SimUnion({"Address": SimTypeRef("SOCKADDR", SimStruct), "AddressIn": SimTypeRef("SOCKADDR_IN", SimStruct), "AddressIn6": SimTypeRef("sockaddr_in6_old", SimStruct)}, name="<anon>", label="None")), ("iiBroadcastAddress", SimUnion({"Address": SimTypeRef("SOCKADDR", SimStruct), "AddressIn": SimTypeRef("SOCKADDR_IN", SimStruct), "AddressIn6": SimTypeRef("sockaddr_in6_old", SimStruct)}, name="<anon>", label="None")), ("iiNetmask", SimUnion({"Address": SimTypeRef("SOCKADDR", SimStruct), "AddressIn": SimTypeRef("SOCKADDR_IN", SimStruct), "AddressIn6": SimTypeRef("sockaddr_in6_old", SimStruct)}, name="<anon>", label="None")),)), name="INTERFACE_INFO", pack=False, align=None),
    "INTERFACE_INFO_EX": SimStruct(OrderedDict((("iiFlags", SimTypeInt(signed=False, label="UInt32")), ("iiAddress", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("iiBroadcastAddress", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("iiNetmask", SimTypeRef("SOCKET_ADDRESS", SimStruct)),)), name="INTERFACE_INFO_EX", pack=False, align=None),
    "INTERFACE_SOFTWARE_TIMESTAMP_CAPABILITIES": SimStruct(OrderedDict((("AllReceive", SimTypeChar(label="Byte")), ("AllTransmit", SimTypeChar(label="Byte")), ("TaggedTransmit", SimTypeChar(label="Byte")),)), name="INTERFACE_SOFTWARE_TIMESTAMP_CAPABILITIES", pack=False, align=None),
    "INTERFACE_TIMESTAMP_CAPABILITIES": SimStruct(OrderedDict((("HardwareClockFrequencyHz", SimTypeLongLong(signed=False, label="UInt64")), ("SupportsCrossTimestamp", SimTypeChar(label="Byte")), ("HardwareCapabilities", SimTypeRef("INTERFACE_HARDWARE_TIMESTAMP_CAPABILITIES", SimStruct)), ("SoftwareCapabilities", SimTypeRef("INTERFACE_SOFTWARE_TIMESTAMP_CAPABILITIES", SimStruct)),)), name="INTERFACE_TIMESTAMP_CAPABILITIES", pack=False, align=None),
    "INTERFACE_TYPE": SimTypeInt(signed=False, label="INTERFACE_TYPE"),
    "INTERLEAVED_AUDIO_FORMAT_INFORMATION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("PrimaryChannelCount", SimTypeInt(signed=False, label="UInt32")), ("PrimaryChannelStartPosition", SimTypeInt(signed=False, label="UInt32")), ("PrimaryChannelMask", SimTypeInt(signed=False, label="UInt32")), ("InterleavedChannelCount", SimTypeInt(signed=False, label="UInt32")), ("InterleavedChannelStartPosition", SimTypeInt(signed=False, label="UInt32")), ("InterleavedChannelMask", SimTypeInt(signed=False, label="UInt32")),)), name="INTERLEAVED_AUDIO_FORMAT_INFORMATION", pack=False, align=None),
    "INTERLOCKED_RESULT": SimTypeInt(signed=False, label="INTERLOCKED_RESULT"),
    "INTERNAL_IF_OPER_STATUS": SimTypeInt(signed=False, label="INTERNAL_IF_OPER_STATUS"),
    "INTERNETEXPLORERCONFIGURATION": SimTypeInt(signed=False, label="INTERNETEXPLORERCONFIGURATION"),
    "INTERNETFEATURELIST": SimTypeInt(signed=False, label="INTERNETFEATURELIST"),
    "INTERNET_ACCESS_TYPE": SimTypeInt(signed=False, label="INTERNET_ACCESS_TYPE"),
    "INTERNET_ASYNC_RESULT": SimStruct(OrderedDict((("dwResult", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_ASYNC_RESULT", pack=False, align=None),
    "INTERNET_AUTH_NOTIFY_DATA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("pfnNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("dwContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="INTERNET_AUTH_NOTIFY_DATA", pack=False, align=None),
    "INTERNET_AUTODIAL": SimTypeInt(signed=False, label="INTERNET_AUTODIAL"),
    "INTERNET_BUFFERSA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("Next", SimTypePointer(SimTypeRef("INTERNET_BUFFERSA", SimStruct), offset=0)), ("lpcszHeader", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwHeadersLength", SimTypeInt(signed=False, label="UInt32")), ("dwHeadersTotal", SimTypeInt(signed=False, label="UInt32")), ("lpvBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwBufferLength", SimTypeInt(signed=False, label="UInt32")), ("dwBufferTotal", SimTypeInt(signed=False, label="UInt32")), ("dwOffsetLow", SimTypeInt(signed=False, label="UInt32")), ("dwOffsetHigh", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_BUFFERSA", pack=False, align=None),
    "INTERNET_BUFFERSW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("Next", SimTypePointer(SimTypeRef("INTERNET_BUFFERSW", SimStruct), offset=0)), ("lpcszHeader", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHeadersLength", SimTypeInt(signed=False, label="UInt32")), ("dwHeadersTotal", SimTypeInt(signed=False, label="UInt32")), ("lpvBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwBufferLength", SimTypeInt(signed=False, label="UInt32")), ("dwBufferTotal", SimTypeInt(signed=False, label="UInt32")), ("dwOffsetLow", SimTypeInt(signed=False, label="UInt32")), ("dwOffsetHigh", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_BUFFERSW", pack=False, align=None),
    "INTERNET_CACHE_CONFIG_INFOA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("dwContainer", SimTypeInt(signed=False, label="UInt32")), ("dwQuota", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")), ("fPerUser", SimTypeInt(signed=True, label="Int32")), ("dwSyncMode", SimTypeInt(signed=False, label="UInt32")), ("dwNumCachePaths", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("CachePath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwCacheSize", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "CachePaths": SimTypePointer(SimTypeRef("INTERNET_CACHE_CONFIG_PATH_ENTRYA", SimStruct), offset=0)}, name="<anon>", label="None")), ("dwNormalUsage", SimTypeInt(signed=False, label="UInt32")), ("dwExemptUsage", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CACHE_CONFIG_INFOA", pack=False, align=None),
    "INTERNET_CACHE_CONFIG_INFOW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("dwContainer", SimTypeInt(signed=False, label="UInt32")), ("dwQuota", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")), ("fPerUser", SimTypeInt(signed=True, label="Int32")), ("dwSyncMode", SimTypeInt(signed=False, label="UInt32")), ("dwNumCachePaths", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("CachePath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwCacheSize", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "CachePaths": SimTypePointer(SimTypeRef("INTERNET_CACHE_CONFIG_PATH_ENTRYW", SimStruct), offset=0)}, name="<anon>", label="None")), ("dwNormalUsage", SimTypeInt(signed=False, label="UInt32")), ("dwExemptUsage", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CACHE_CONFIG_INFOW", pack=False, align=None),
    "INTERNET_CACHE_CONFIG_PATH_ENTRYA": SimStruct(OrderedDict((("CachePath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwCacheSize", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CACHE_CONFIG_PATH_ENTRYA", pack=False, align=None),
    "INTERNET_CACHE_CONFIG_PATH_ENTRYW": SimStruct(OrderedDict((("CachePath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwCacheSize", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CACHE_CONFIG_PATH_ENTRYW", pack=False, align=None),
    "INTERNET_CACHE_CONTAINER_INFOA": SimStruct(OrderedDict((("dwCacheVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszCachePrefix", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszVolumeLabel", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszVolumeTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="INTERNET_CACHE_CONTAINER_INFOA", pack=False, align=None),
    "INTERNET_CACHE_CONTAINER_INFOW": SimStruct(OrderedDict((("dwCacheVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszCachePrefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszVolumeLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszVolumeTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="INTERNET_CACHE_CONTAINER_INFOW", pack=False, align=None),
    "INTERNET_CACHE_ENTRY_INFOA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszSourceUrlName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszLocalFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CacheEntryType", SimTypeInt(signed=False, label="UInt32")), ("dwUseCount", SimTypeInt(signed=False, label="UInt32")), ("dwHitRate", SimTypeInt(signed=False, label="UInt32")), ("dwSizeLow", SimTypeInt(signed=False, label="UInt32")), ("dwSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("LastModifiedTime", SimTypeRef("FILETIME", SimStruct)), ("ExpireTime", SimTypeRef("FILETIME", SimStruct)), ("LastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("LastSyncTime", SimTypeRef("FILETIME", SimStruct)), ("lpHeaderInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwHeaderInfoSize", SimTypeInt(signed=False, label="UInt32")), ("lpszFileExtension", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Anonymous", SimUnion({"dwReserved": SimTypeInt(signed=False, label="UInt32"), "dwExemptDelta": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="INTERNET_CACHE_ENTRY_INFOA", pack=False, align=None),
    "INTERNET_CACHE_ENTRY_INFOW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszSourceUrlName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszLocalFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CacheEntryType", SimTypeInt(signed=False, label="UInt32")), ("dwUseCount", SimTypeInt(signed=False, label="UInt32")), ("dwHitRate", SimTypeInt(signed=False, label="UInt32")), ("dwSizeLow", SimTypeInt(signed=False, label="UInt32")), ("dwSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("LastModifiedTime", SimTypeRef("FILETIME", SimStruct)), ("ExpireTime", SimTypeRef("FILETIME", SimStruct)), ("LastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("LastSyncTime", SimTypeRef("FILETIME", SimStruct)), ("lpHeaderInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHeaderInfoSize", SimTypeInt(signed=False, label="UInt32")), ("lpszFileExtension", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"dwReserved": SimTypeInt(signed=False, label="UInt32"), "dwExemptDelta": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="INTERNET_CACHE_ENTRY_INFOW", pack=False, align=None),
    "INTERNET_CACHE_GROUP_INFOA": SimStruct(OrderedDict((("dwGroupSize", SimTypeInt(signed=False, label="UInt32")), ("dwGroupFlags", SimTypeInt(signed=False, label="UInt32")), ("dwGroupType", SimTypeInt(signed=False, label="UInt32")), ("dwDiskUsage", SimTypeInt(signed=False, label="UInt32")), ("dwDiskQuota", SimTypeInt(signed=False, label="UInt32")), ("dwOwnerStorage", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("szGroupName", SimTypeArray(SimTypeChar(label="SByte"), 120)),)), name="INTERNET_CACHE_GROUP_INFOA", pack=False, align=None),
    "INTERNET_CACHE_GROUP_INFOW": SimStruct(OrderedDict((("dwGroupSize", SimTypeInt(signed=False, label="UInt32")), ("dwGroupFlags", SimTypeInt(signed=False, label="UInt32")), ("dwGroupType", SimTypeInt(signed=False, label="UInt32")), ("dwDiskUsage", SimTypeInt(signed=False, label="UInt32")), ("dwDiskQuota", SimTypeInt(signed=False, label="UInt32")), ("dwOwnerStorage", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("szGroupName", SimTypeArray(SimTypeChar(label="Char"), 120)),)), name="INTERNET_CACHE_GROUP_INFOW", pack=False, align=None),
    "INTERNET_CACHE_TIMESTAMPS": SimStruct(OrderedDict((("ftExpires", SimTypeRef("FILETIME", SimStruct)), ("ftLastModified", SimTypeRef("FILETIME", SimStruct)),)), name="INTERNET_CACHE_TIMESTAMPS", pack=False, align=None),
    "INTERNET_CALLBACK_COOKIE": SimStruct(OrderedDict((("pcwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftExpires", SimTypeRef("FILETIME", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CALLBACK_COOKIE", pack=False, align=None),
    "INTERNET_CERTIFICATE_INFO": SimStruct(OrderedDict((("ftExpiry", SimTypeRef("FILETIME", SimStruct)), ("ftStart", SimTypeRef("FILETIME", SimStruct)), ("lpszSubjectInfo", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszIssuerInfo", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszProtocolName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszSignatureAlgName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszEncryptionAlgName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("dwKeySize", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CERTIFICATE_INFO", pack=False, align=None),
    "INTERNET_CONNECTED_INFO": SimStruct(OrderedDict((("dwConnectedState", SimTypeInt(signed=False, label="INTERNET_STATE")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_CONNECTED_INFO", pack=False, align=None),
    "INTERNET_CONNECTION": SimTypeInt(signed=False, label="INTERNET_CONNECTION"),
    "INTERNET_COOKIE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszDomain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pftExpires", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszP3PPolicy", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="INTERNET_COOKIE", pack=False, align=None),
    "INTERNET_COOKIE2": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ftExpires", SimTypeRef("FILETIME", SimStruct)), ("fExpiresSet", SimTypeInt(signed=True, label="Int32")),)), name="INTERNET_COOKIE2", pack=False, align=None),
    "INTERNET_COOKIE_FLAGS": SimTypeInt(signed=False, label="INTERNET_COOKIE_FLAGS"),
    "INTERNET_CREDENTIALS": SimStruct(OrderedDict((("lpcwszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPort", SimTypeInt(signed=False, label="UInt32")), ("dwScheme", SimTypeInt(signed=False, label="UInt32")), ("lpcwszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpcwszRealm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fAuthIdentity", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("lpcwszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpcwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "pAuthIdentityOpaque": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="INTERNET_CREDENTIALS", pack=False, align=None),
    "INTERNET_DIAGNOSTIC_SOCKET_INFO": SimStruct(OrderedDict((("Socket", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SourcePort", SimTypeInt(signed=False, label="UInt32")), ("DestPort", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_DIAGNOSTIC_SOCKET_INFO", pack=False, align=None),
    "INTERNET_DOWNLOAD_MODE_HANDLE": SimStruct(OrderedDict((("pcwszFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("phFile", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="INTERNET_DOWNLOAD_MODE_HANDLE", pack=False, align=None),
    "INTERNET_END_BROWSER_SESSION_DATA": SimStruct(OrderedDict((("lpBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_END_BROWSER_SESSION_DATA", pack=False, align=None),
    "INTERNET_PER_CONN": SimTypeInt(signed=False, label="INTERNET_PER_CONN"),
    "INTERNET_PER_CONN_OPTIONA": SimStruct(OrderedDict((("dwOption", SimTypeInt(signed=False, label="INTERNET_PER_CONN")), ("Value", SimUnion({"dwValue": SimTypeInt(signed=False, label="UInt32"), "pszValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "ftValue": SimTypeRef("FILETIME", SimStruct)}, name="<anon>", label="None")),)), name="INTERNET_PER_CONN_OPTIONA", pack=False, align=None),
    "INTERNET_PER_CONN_OPTIONW": SimStruct(OrderedDict((("dwOption", SimTypeInt(signed=False, label="INTERNET_PER_CONN")), ("Value", SimUnion({"dwValue": SimTypeInt(signed=False, label="UInt32"), "pszValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ftValue": SimTypeRef("FILETIME", SimStruct)}, name="<anon>", label="None")),)), name="INTERNET_PER_CONN_OPTIONW", pack=False, align=None),
    "INTERNET_PER_CONN_OPTION_LISTA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pszConnection", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwOptionCount", SimTypeInt(signed=False, label="UInt32")), ("dwOptionError", SimTypeInt(signed=False, label="UInt32")), ("pOptions", SimTypePointer(SimTypeRef("INTERNET_PER_CONN_OPTIONA", SimStruct), offset=0)),)), name="INTERNET_PER_CONN_OPTION_LISTA", pack=False, align=None),
    "INTERNET_PER_CONN_OPTION_LISTW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pszConnection", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwOptionCount", SimTypeInt(signed=False, label="UInt32")), ("dwOptionError", SimTypeInt(signed=False, label="UInt32")), ("pOptions", SimTypePointer(SimTypeRef("INTERNET_PER_CONN_OPTIONW", SimStruct), offset=0)),)), name="INTERNET_PER_CONN_OPTION_LISTW", pack=False, align=None),
    "INTERNET_PREFETCH_STATUS": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_PREFETCH_STATUS", pack=False, align=None),
    "INTERNET_PROXY_INFO": SimStruct(OrderedDict((("dwAccessType", SimTypeInt(signed=False, label="INTERNET_ACCESS_TYPE")), ("lpszProxy", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszProxyBypass", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="INTERNET_PROXY_INFO", pack=False, align=None),
    "INTERNET_SCHEME": SimTypeInt(signed=False, label="INTERNET_SCHEME"),
    "INTERNET_SECURITY_CONNECTION_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fSecure", SimTypeInt(signed=True, label="Int32")), ("connectionInfo", SimTypeRef("SecPkgContext_ConnectionInfo", SimStruct)), ("cipherInfo", SimTypeRef("SecPkgContext_CipherInfo", SimStruct)),)), name="INTERNET_SECURITY_CONNECTION_INFO", pack=False, align=None),
    "INTERNET_SECURITY_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pCertificate", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("pcCertChain", SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0)), ("connectionInfo", SimTypeRef("SecPkgContext_ConnectionInfo", SimStruct)), ("cipherInfo", SimTypeRef("SecPkgContext_CipherInfo", SimStruct)), ("pcUnverifiedCertChain", SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0)), ("channelBindingToken", SimTypeRef("SecPkgContext_Bindings", SimStruct)),)), name="INTERNET_SECURITY_INFO", pack=False, align=None),
    "INTERNET_SERVER_CONNECTION_STATE": SimStruct(OrderedDict((("lpcwszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fProxy", SimTypeInt(signed=True, label="Int32")), ("dwCounter", SimTypeInt(signed=False, label="UInt32")), ("dwConnectionLimit", SimTypeInt(signed=False, label="UInt32")), ("dwAvailableCreates", SimTypeInt(signed=False, label="UInt32")), ("dwAvailableKeepAlives", SimTypeInt(signed=False, label="UInt32")), ("dwActiveConnections", SimTypeInt(signed=False, label="UInt32")), ("dwWaiters", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_SERVER_CONNECTION_STATE", pack=False, align=None),
    "INTERNET_STATE": SimTypeInt(signed=False, label="INTERNET_STATE"),
    "INTERNET_VERSION_INFO": SimStruct(OrderedDict((("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="INTERNET_VERSION_INFO", pack=False, align=None),
    "INTERVAL_LIMIT_KIND": SimTypeInt(signed=False, label="INTERVAL_LIMIT_KIND"),
    "INTLIST": SimStruct(OrderedDict((("iValueCount", SimTypeInt(signed=True, label="Int32")), ("iValues", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 402)),)), name="INTLIST", pack=False, align=None),
    "INVOC": SimStruct(OrderedDict((("dwCount", SimTypeInt(signed=False, label="UInt32")), ("loOffset", SimTypeInt(signed=False, label="UInt32")),)), name="INVOC", pack=False, align=None),
    "INVOKEKIND": SimTypeInt(signed=False, label="INVOKEKIND"),
    "IN_ADDR": SimStruct(OrderedDict((("S_un", SimUnion({"S_un_b": SimTypeRef("_S_un_b_e__Struct", SimStruct), "S_un_w": SimTypeRef("_S_un_w_e__Struct", SimStruct), "S_addr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="IN_ADDR", pack=False, align=None),
    "IN_ADDR_IPV4": SimUnion({"Addr": SimTypeInt(signed=False, label="UInt32"), "AddrBytes": SimTypeArray(SimTypeChar(label="Byte"), 4)}, name="<anon>", label="None"),
    "IN_ADDR_IPV6": SimStruct(OrderedDict((("Addr", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="IN_ADDR_IPV6", pack=False, align=None),
    "IN_PKTINFO": SimStruct(OrderedDict((("ipi_addr", SimTypeRef("IN_ADDR", SimStruct)), ("ipi_ifindex", SimTypeInt(signed=False, label="UInt32")),)), name="IN_PKTINFO", pack=False, align=None),
    "IN_PKTINFO_EX": SimStruct(OrderedDict((("pkt_info", SimTypeRef("IN_PKTINFO", SimStruct)), ("scope_id", SimTypeRef("SCOPE_ID", SimStruct)),)), name="IN_PKTINFO_EX", pack=False, align=None),
    "IN_RECVERR": SimStruct(OrderedDict((("protocol", SimTypeInt(signed=False, label="IPPROTO")), ("info", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeChar(label="Byte")), ("code", SimTypeChar(label="Byte")),)), name="IN_RECVERR", pack=False, align=None),
    "INameSpaceTreeAccessible": SimTypeBottom(label="INameSpaceTreeAccessible"),
    "INameSpaceTreeControl": SimTypeBottom(label="INameSpaceTreeControl"),
    "INameSpaceTreeControl2": SimTypeBottom(label="INameSpaceTreeControl2"),
    "INameSpaceTreeControlCustomDraw": SimTypeBottom(label="INameSpaceTreeControlCustomDraw"),
    "INameSpaceTreeControlDropHandler": SimTypeBottom(label="INameSpaceTreeControlDropHandler"),
    "INameSpaceTreeControlEvents": SimTypeBottom(label="INameSpaceTreeControlEvents"),
    "INameSpaceTreeControlFolderCapabilities": SimTypeBottom(label="INameSpaceTreeControlFolderCapabilities"),
    "INamedEntity": SimTypeBottom(label="INamedEntity"),
    "INamedEntityCollector": SimTypeBottom(label="INamedEntityCollector"),
    "INamedPropertyBag": SimTypeBottom(label="INamedPropertyBag"),
    "INamedPropertyStore": SimTypeBottom(label="INamedPropertyStore"),
    "INamespaceWalk": SimTypeBottom(label="INamespaceWalk"),
    "INamespaceWalkCB": SimTypeBottom(label="INamespaceWalkCB"),
    "INamespaceWalkCB2": SimTypeBottom(label="INamespaceWalkCB2"),
    "INavigatorDoNotTrack": SimTypeBottom(label="INavigatorDoNotTrack"),
    "INavigatorGeolocation": SimTypeBottom(label="INavigatorGeolocation"),
    "INetCfg": SimTypeBottom(label="INetCfg"),
    "INetCfgBindingInterface": SimTypeBottom(label="INetCfgBindingInterface"),
    "INetCfgBindingPath": SimTypeBottom(label="INetCfgBindingPath"),
    "INetCfgClass": SimTypeBottom(label="INetCfgClass"),
    "INetCfgClassSetup": SimTypeBottom(label="INetCfgClassSetup"),
    "INetCfgClassSetup2": SimTypeBottom(label="INetCfgClassSetup2"),
    "INetCfgComponent": SimTypeBottom(label="INetCfgComponent"),
    "INetCfgComponentBindings": SimTypeBottom(label="INetCfgComponentBindings"),
    "INetCfgComponentControl": SimTypeBottom(label="INetCfgComponentControl"),
    "INetCfgComponentNotifyBinding": SimTypeBottom(label="INetCfgComponentNotifyBinding"),
    "INetCfgComponentNotifyGlobal": SimTypeBottom(label="INetCfgComponentNotifyGlobal"),
    "INetCfgComponentPropertyUi": SimTypeBottom(label="INetCfgComponentPropertyUi"),
    "INetCfgComponentSetup": SimTypeBottom(label="INetCfgComponentSetup"),
    "INetCfgComponentSysPrep": SimTypeBottom(label="INetCfgComponentSysPrep"),
    "INetCfgComponentUpperEdge": SimTypeBottom(label="INetCfgComponentUpperEdge"),
    "INetCfgLock": SimTypeBottom(label="INetCfgLock"),
    "INetCfgPnpReconfigCallback": SimTypeBottom(label="INetCfgPnpReconfigCallback"),
    "INetCfgSysPrep": SimTypeBottom(label="INetCfgSysPrep"),
    "INetConnection": SimTypeBottom(label="INetConnection"),
    "INetConnectionConnectUi": SimTypeBottom(label="INetConnectionConnectUi"),
    "INetConnectionManager": SimTypeBottom(label="INetConnectionManager"),
    "INetConnectionProps": SimTypeBottom(label="INetConnectionProps"),
    "INetDiagExtensibleHelper": SimTypeBottom(label="INetDiagExtensibleHelper"),
    "INetDiagHelper": SimTypeBottom(label="INetDiagHelper"),
    "INetDiagHelperEx": SimTypeBottom(label="INetDiagHelperEx"),
    "INetDiagHelperInfo": SimTypeBottom(label="INetDiagHelperInfo"),
    "INetDiagHelperUtilFactory": SimTypeBottom(label="INetDiagHelperUtilFactory"),
    "INetFwAuthorizedApplication": SimTypeBottom(label="INetFwAuthorizedApplication"),
    "INetFwAuthorizedApplications": SimTypeBottom(label="INetFwAuthorizedApplications"),
    "INetFwIcmpSettings": SimTypeBottom(label="INetFwIcmpSettings"),
    "INetFwMgr": SimTypeBottom(label="INetFwMgr"),
    "INetFwOpenPort": SimTypeBottom(label="INetFwOpenPort"),
    "INetFwOpenPorts": SimTypeBottom(label="INetFwOpenPorts"),
    "INetFwPolicy": SimTypeBottom(label="INetFwPolicy"),
    "INetFwPolicy2": SimTypeBottom(label="INetFwPolicy2"),
    "INetFwProduct": SimTypeBottom(label="INetFwProduct"),
    "INetFwProducts": SimTypeBottom(label="INetFwProducts"),
    "INetFwProfile": SimTypeBottom(label="INetFwProfile"),
    "INetFwRemoteAdminSettings": SimTypeBottom(label="INetFwRemoteAdminSettings"),
    "INetFwRule": SimTypeBottom(label="INetFwRule"),
    "INetFwRule2": SimTypeBottom(label="INetFwRule2"),
    "INetFwRule3": SimTypeBottom(label="INetFwRule3"),
    "INetFwRules": SimTypeBottom(label="INetFwRules"),
    "INetFwService": SimTypeBottom(label="INetFwService"),
    "INetFwServiceRestriction": SimTypeBottom(label="INetFwServiceRestriction"),
    "INetFwServices": SimTypeBottom(label="INetFwServices"),
    "INetLanConnectionUiInfo": SimTypeBottom(label="INetLanConnectionUiInfo"),
    "INetRasConnectionIpUiInfo": SimTypeBottom(label="INetRasConnectionIpUiInfo"),
    "INetSharingConfiguration": SimTypeBottom(label="INetSharingConfiguration"),
    "INetSharingEveryConnectionCollection": SimTypeBottom(label="INetSharingEveryConnectionCollection"),
    "INetSharingManager": SimTypeBottom(label="INetSharingManager"),
    "INetSharingPortMapping": SimTypeBottom(label="INetSharingPortMapping"),
    "INetSharingPortMappingCollection": SimTypeBottom(label="INetSharingPortMappingCollection"),
    "INetSharingPortMappingProps": SimTypeBottom(label="INetSharingPortMappingProps"),
    "INetSharingPrivateConnectionCollection": SimTypeBottom(label="INetSharingPrivateConnectionCollection"),
    "INetSharingPublicConnectionCollection": SimTypeBottom(label="INetSharingPublicConnectionCollection"),
    "INetwork": SimTypeBottom(label="INetwork"),
    "INetwork2": SimTypeBottom(label="INetwork2"),
    "INetworkConnection": SimTypeBottom(label="INetworkConnection"),
    "INetworkConnection2": SimTypeBottom(label="INetworkConnection2"),
    "INetworkConnectionCost": SimTypeBottom(label="INetworkConnectionCost"),
    "INetworkConnectionCostEvents": SimTypeBottom(label="INetworkConnectionCostEvents"),
    "INetworkConnectionEvents": SimTypeBottom(label="INetworkConnectionEvents"),
    "INetworkCostManager": SimTypeBottom(label="INetworkCostManager"),
    "INetworkCostManagerEvents": SimTypeBottom(label="INetworkCostManagerEvents"),
    "INetworkEvents": SimTypeBottom(label="INetworkEvents"),
    "INetworkFolderInternal": SimTypeBottom(label="INetworkFolderInternal"),
    "INetworkListManager": SimTypeBottom(label="INetworkListManager"),
    "INetworkListManagerEvents": SimTypeBottom(label="INetworkListManagerEvents"),
    "INetworkSettings": SimTypeBottom(label="INetworkSettings"),
    "INetworkTransportSettings": SimTypeBottom(label="INetworkTransportSettings"),
    "INewMenuClient": SimTypeBottom(label="INewMenuClient"),
    "INewShortcutHookA": SimTypeBottom(label="INewShortcutHookA"),
    "INewShortcutHookW": SimTypeBottom(label="INewShortcutHookW"),
    "INewWDEvents": SimTypeBottom(label="INewWDEvents"),
    "INewWindowManager": SimTypeBottom(label="INewWindowManager"),
    "INoMarshal": SimTypeBottom(label="INoMarshal"),
    "INodeProperties": SimTypeBottom(label="INodeProperties"),
    "INotificationActivationCallback": SimTypeBottom(label="INotificationActivationCallback"),
    "INotificationTransportSync": SimTypeBottom(label="INotificationTransportSync"),
    "INotifyReplica": SimTypeBottom(label="INotifyReplica"),
    "IOCSPAdmin": SimTypeBottom(label="IOCSPAdmin"),
    "IOCSPCAConfiguration": SimTypeBottom(label="IOCSPCAConfiguration"),
    "IOCSPCAConfigurationCollection": SimTypeBottom(label="IOCSPCAConfigurationCollection"),
    "IOCSPProperty": SimTypeBottom(label="IOCSPProperty"),
    "IOCSPPropertyCollection": SimTypeBottom(label="IOCSPPropertyCollection"),
    "IOD_DESFLAGS": SimTypeInt(signed=False, label="IOD_DESFLAGS"),
    "IOMMU_DEVICE_CREATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IOMMU_DEVICE_CREATION_CONFIGURATION", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "DeviceConfig", "DmaDeviceOut"]), offset=0),
    "IOMMU_DEVICE_CREATION_CONFIGURATION": SimStruct(OrderedDict((("NextConfiguration", SimTypeRef("LIST_ENTRY", SimStruct)), ("ConfigType", SimTypeInt(signed=False, label="IOMMU_DEVICE_CREATION_CONFIGURATION_TYPE")), ("Anonymous", SimUnion({"Acpi": SimTypeRef("IOMMU_DEVICE_CREATION_CONFIGURATION_ACPI", SimStruct), "DeviceId": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="IOMMU_DEVICE_CREATION_CONFIGURATION", pack=False, align=None),
    "IOMMU_DEVICE_CREATION_CONFIGURATION_ACPI": SimStruct(OrderedDict((("InputMappingBase", SimTypeInt(signed=False, label="UInt32")), ("MappingsCount", SimTypeInt(signed=False, label="UInt32")),)), name="IOMMU_DEVICE_CREATION_CONFIGURATION_ACPI", pack=False, align=None),
    "IOMMU_DEVICE_CREATION_CONFIGURATION_TYPE": SimTypeInt(signed=False, label="IOMMU_DEVICE_CREATION_CONFIGURATION_TYPE"),
    "IOMMU_DEVICE_DELETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaDevice"]), offset=0),
    "IOMMU_DEVICE_FAULT_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FAULT_INFORMATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "FaultInformation"]), offset=0),
    "IOMMU_DEVICE_QUERY_DOMAIN_TYPES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaDevice", "AvailableDomains"]), offset=0),
    "IOMMU_DMA_DEVICE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "IOMMU_DMA_DOMAIN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "IOMMU_DMA_DOMAIN_CREATION_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "IOMMU_DMA_DOMAIN_TYPE": SimTypeInt(signed=False, label="IOMMU_DMA_DOMAIN_TYPE"),
    "IOMMU_DMA_LOGICAL_ADDRESS_TOKEN": SimStruct(OrderedDict((("LogicalAddressBase", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="IOMMU_DMA_LOGICAL_ADDRESS_TOKEN", pack=False, align=None),
    "IOMMU_DMA_LOGICAL_ADDRESS_TOKEN_MAPPED_SEGMENT": SimStruct(OrderedDict((("OwningToken", SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ADDRESS_TOKEN", SimStruct), offset=0)), ("Offset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="IOMMU_DMA_LOGICAL_ADDRESS_TOKEN_MAPPED_SEGMENT", pack=False, align=None),
    "IOMMU_DMA_LOGICAL_ALLOCATOR_CONFIG": SimStruct(OrderedDict((("LogicalAllocatorType", SimTypeInt(signed=False, label="IOMMU_DMA_LOGICAL_ALLOCATOR_TYPE")), ("Anonymous", SimUnion({"BuddyAllocatorConfig": SimTypeRef("_BuddyAllocatorConfig_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="IOMMU_DMA_LOGICAL_ALLOCATOR_CONFIG", pack=False, align=None),
    "IOMMU_DMA_LOGICAL_ALLOCATOR_TYPE": SimTypeInt(signed=False, label="IOMMU_DMA_LOGICAL_ALLOCATOR_TYPE"),
    "IOMMU_DMA_RESERVED_REGION": SimStruct(OrderedDict((("RegionNext", SimTypePointer(SimTypeRef("IOMMU_DMA_RESERVED_REGION", SimStruct), offset=0)), ("Base", SimTypeLongLong(signed=False, label="UInt64")), ("NumberOfPages", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ShouldMap", SimTypeChar(label="Byte")),)), name="IOMMU_DMA_RESERVED_REGION", pack=False, align=None),
    "IOMMU_DOMAIN_ATTACH_DEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "PhysicalDeviceObject", "InputMappingIdBase", "MappingCount"]), offset=0),
    "IOMMU_DOMAIN_ATTACH_DEVICE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "DmaDevice"]), offset=0),
    "IOMMU_DOMAIN_CONFIGURE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("DOMAIN_CONFIGURATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Configuration"]), offset=0),
    "IOMMU_DOMAIN_CREATE": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["OsManagedPageTable", "DomainOut"]), offset=0),
    "IOMMU_DOMAIN_CREATE_EX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="IOMMU_DMA_DOMAIN_TYPE"), SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ALLOCATOR_CONFIG", SimStruct), offset=0), SimTypePointer(SimTypeRef("IOMMU_DMA_RESERVED_REGION", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DomainType", "Flags", "LogicalAllocatorConfig", "ReservedRegions", "DomainOut"]), offset=0),
    "IOMMU_DOMAIN_DELETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain"]), offset=0),
    "IOMMU_DOMAIN_DETACH_DEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "PhysicalDeviceObject", "InputMappingId"]), offset=0),
    "IOMMU_DOMAIN_DETACH_DEVICE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaDevice"]), offset=0),
    "IOMMU_FLUSH_DOMAIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain"]), offset=0),
    "IOMMU_FLUSH_DOMAIN_VA_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "LastLevel", "Number", "VaList"]), offset=0),
    "IOMMU_FREE_RESERVED_LOGICAL_ADDRESS_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ADDRESS_TOKEN", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogicalAddressToken"]), offset=0),
    "IOMMU_INTERFACE_STATE_CHANGE": SimStruct(OrderedDict((("PresentFields", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("AvailableDomainTypes", SimTypeInt(signed=False, label="UInt32")),)), name="IOMMU_INTERFACE_STATE_CHANGE", pack=False, align=None),
    "IOMMU_INTERFACE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IOMMU_INTERFACE_STATE_CHANGE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["StateChange", "Context"]), offset=0),
    "IOMMU_INTERFACE_STATE_CHANGE_FIELDS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "IOMMU_MAP_IDENTITY_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Permissions", "Mdl"]), offset=0),
    "IOMMU_MAP_IDENTITY_RANGE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IOMMU_MAP_PHYSICAL_ADDRESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Permissions", "PhysicalAddressToMap"]), offset=0),
    "IOMMU_MAP_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Permissions", "Mdl", "LogicalAddress"]), offset=0),
    "IOMMU_MAP_LOGICAL_RANGE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IOMMU_MAP_PHYSICAL_ADDRESS", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Permissions", "PhysicalAddressToMap", "ExplicitLogicalAddress", "MinLogicalAddress", "MaxLogicalAddress", "LogicalAddressOut"]), offset=0),
    "IOMMU_MAP_PHYSICAL_ADDRESS": SimStruct(OrderedDict((("MapType", SimTypeInt(signed=False, label="IOMMU_MAP_PHYSICAL_ADDRESS_TYPE")), ("Anonymous", SimUnion({"Mdl": SimTypeRef("_Mdl_e__Struct", SimStruct), "ContiguousRange": SimTypeRef("_ContiguousRange_e__Struct", SimStruct), "PfnArray": SimTypeRef("_PfnArray_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="IOMMU_MAP_PHYSICAL_ADDRESS", pack=False, align=None),
    "IOMMU_MAP_PHYSICAL_ADDRESS_TYPE": SimTypeInt(signed=False, label="IOMMU_MAP_PHYSICAL_ADDRESS_TYPE"),
    "IOMMU_MAP_RESERVED_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ADDRESS_TOKEN", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IOMMU_MAP_PHYSICAL_ADDRESS", SimStruct), offset=0), SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ADDRESS_TOKEN_MAPPED_SEGMENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogicalAddressToken", "Offset", "Permissions", "PhysicalAddressToMap", "MappedSegment"]), offset=0),
    "IOMMU_QUERY_INPUT_MAPPINGS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("INPUT_MAPPING_ELEMENT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PhysicalDeviceObject", "Buffer", "BufferLength", "ReturnLength"]), offset=0),
    "IOMMU_REGISTER_INTERFACE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["StateChangeCallback", "Context", "DmaDevice", "StateFields"]), offset=0),
    "IOMMU_RESERVE_LOGICAL_ADDRESS_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ADDRESS_TOKEN", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Size", "ExplicitLogicalAddress", "MinLogicalAddress", "MaxLogicalAddress", "LogicalAddressToken"]), offset=0),
    "IOMMU_SET_DEVICE_FAULT_REPORTING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("DEVICE_FAULT_CONFIGURATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PhysicalDeviceObject", "InputMappingIdBase", "Enable", "FaultConfig"]), offset=0),
    "IOMMU_SET_DEVICE_FAULT_REPORTING_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("DEVICE_FAULT_CONFIGURATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaDevice", "InputMappingIdBase", "Enable", "FaultConfig"]), offset=0),
    "IOMMU_UNMAP_IDENTITY_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "Mdl"]), offset=0),
    "IOMMU_UNMAP_IDENTITY_RANGE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("IOMMU_MAP_PHYSICAL_ADDRESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "MappedPhysicalAddress"]), offset=0),
    "IOMMU_UNMAP_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["Domain", "LogicalAddress", "NumberOfPages"]), offset=0),
    "IOMMU_UNMAP_RESERVED_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IOMMU_DMA_LOGICAL_ADDRESS_TOKEN_MAPPED_SEGMENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MappedSegment"]), offset=0),
    "IOMMU_UNREGISTER_INTERFACE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["StateChangeCallback", "DmaDevice"]), offset=0),
    "IOPMVideoOutput": SimTypeBottom(label="IOPMVideoOutput"),
    "IORING_BUFFER_INFO": SimStruct(OrderedDict((("Address", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="IORING_BUFFER_INFO", pack=False, align=None),
    "IORING_BUFFER_REF": SimStruct(OrderedDict((("Kind", SimTypeInt(signed=False, label="IORING_REF_KIND")), ("Buffer", SimUnion({"Address": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "IndexAndOffset": SimTypeRef("IORING_REGISTERED_BUFFER", SimStruct)}, name="<anon>", label="None")),)), name="IORING_BUFFER_REF", pack=False, align=None),
    "IORING_CAPABILITIES": SimStruct(OrderedDict((("MaxVersion", SimTypeInt(signed=False, label="IORING_VERSION")), ("MaxSubmissionQueueSize", SimTypeInt(signed=False, label="UInt32")), ("MaxCompletionQueueSize", SimTypeInt(signed=False, label="UInt32")), ("FeatureFlags", SimTypeInt(signed=False, label="IORING_FEATURE_FLAGS")),)), name="IORING_CAPABILITIES", pack=False, align=None),
    "IORING_CQE": SimStruct(OrderedDict((("UserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ResultCode", SimTypeInt(signed=True, label="Int32")), ("Information", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="IORING_CQE", pack=False, align=None),
    "IORING_CREATE_ADVISORY_FLAGS": SimTypeInt(signed=False, label="IORING_CREATE_ADVISORY_FLAGS"),
    "IORING_CREATE_FLAGS": SimStruct(OrderedDict((("Required", SimTypeInt(signed=False, label="IORING_CREATE_REQUIRED_FLAGS")), ("Advisory", SimTypeInt(signed=False, label="IORING_CREATE_ADVISORY_FLAGS")),)), name="IORING_CREATE_FLAGS", pack=False, align=None),
    "IORING_CREATE_REQUIRED_FLAGS": SimTypeInt(signed=False, label="IORING_CREATE_REQUIRED_FLAGS"),
    "IORING_FEATURE_FLAGS": SimTypeInt(signed=False, label="IORING_FEATURE_FLAGS"),
    "IORING_HANDLE_REF": SimStruct(OrderedDict((("Kind", SimTypeInt(signed=False, label="IORING_REF_KIND")), ("Handle", SimUnion({"Handle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Index": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="IORING_HANDLE_REF", pack=False, align=None),
    "IORING_INFO": SimStruct(OrderedDict((("IoRingVersion", SimTypeInt(signed=False, label="IORING_VERSION")), ("Flags", SimTypeRef("IORING_CREATE_FLAGS", SimStruct)), ("SubmissionQueueSize", SimTypeInt(signed=False, label="UInt32")), ("CompletionQueueSize", SimTypeInt(signed=False, label="UInt32")),)), name="IORING_INFO", pack=False, align=None),
    "IORING_OP_CODE": SimTypeInt(signed=False, label="IORING_OP_CODE"),
    "IORING_REF_KIND": SimTypeInt(signed=False, label="IORING_REF_KIND"),
    "IORING_REGISTERED_BUFFER": SimStruct(OrderedDict((("BufferIndex", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="IORING_REGISTERED_BUFFER", pack=False, align=None),
    "IORING_SQE_FLAGS": SimTypeInt(signed=False, label="IORING_SQE_FLAGS"),
    "IORING_VERSION": SimTypeInt(signed=False, label="IORING_VERSION"),
    "IOSPACE": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeInt(signed=False, label="UInt32")),)), name="IOSPACE", pack=False, align=None),
    "IOSPACE32": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeInt(signed=False, label="UInt32")),)), name="IOSPACE32", pack=False, align=None),
    "IOSPACE64": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeInt(signed=False, label="UInt32")),)), name="IOSPACE64", pack=False, align=None),
    "IOSPACE_EX": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeInt(signed=False, label="UInt32")), ("InterfaceType", SimTypeInt(signed=False, label="UInt32")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("AddressSpace", SimTypeInt(signed=False, label="UInt32")),)), name="IOSPACE_EX", pack=False, align=None),
    "IOSPACE_EX32": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeInt(signed=False, label="UInt32")), ("InterfaceType", SimTypeInt(signed=False, label="UInt32")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("AddressSpace", SimTypeInt(signed=False, label="UInt32")),)), name="IOSPACE_EX32", pack=False, align=None),
    "IOSPACE_EX64": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeInt(signed=False, label="UInt32")), ("InterfaceType", SimTypeInt(signed=False, label="UInt32")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("AddressSpace", SimTypeInt(signed=False, label="UInt32")),)), name="IOSPACE_EX64", pack=False, align=None),
    "IO_ACCESS_MODE": SimTypeInt(signed=False, label="IO_ACCESS_MODE"),
    "IO_ACCESS_TYPE": SimTypeInt(signed=False, label="IO_ACCESS_TYPE"),
    "IO_ALLOCATION_ACTION": SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"),
    "IO_ATTRIBUTION_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AllFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ServiceStartTime", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="IO_ATTRIBUTION_INFORMATION", pack=False, align=None),
    "IO_BLOCK": SimStruct(OrderedDict((("uOffset", SimTypeInt(signed=False, label="UInt32")), ("uLength", SimTypeInt(signed=False, label="UInt32")), ("pbyData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uIndex", SimTypeInt(signed=False, label="UInt32")),)), name="IO_BLOCK", pack=False, align=None),
    "IO_BLOCK_EX": SimStruct(OrderedDict((("uOffset", SimTypeInt(signed=False, label="UInt32")), ("uLength", SimTypeInt(signed=False, label="UInt32")), ("pbyData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uIndex", SimTypeInt(signed=False, label="UInt32")), ("bRequest", SimTypeChar(label="Byte")), ("bmRequestType", SimTypeChar(label="Byte")), ("fTransferDirectionIn", SimTypeChar(label="Byte")),)), name="IO_BLOCK_EX", pack=False, align=None),
    "IO_CAPABILITY": SimTypeInt(signed=False, label="IO_CAPABILITY"),
    "IO_COMPLETION_CONTEXT": SimStruct(OrderedDict((("Port", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UsageCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IO_COMPLETION_CONTEXT", pack=False, align=None),
    "IO_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "Irp", "Context"]), offset=0),
    "IO_COMPLETION_ROUTINE_RESULT": SimTypeInt(signed=False, label="IO_COMPLETION_ROUTINE_RESULT"),
    "IO_CONNECT_INTERRUPT_FULLY_SPECIFIED_PARAMETERS": SimStruct(OrderedDict((("PhysicalDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("InterruptObject", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("ServiceRoutine", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)), ("ServiceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SpinLock", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)), ("SynchronizeIrql", SimTypeChar(label="Byte")), ("FloatingSave", SimTypeChar(label="Byte")), ("ShareVector", SimTypeChar(label="Byte")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("Irql", SimTypeChar(label="Byte")), ("InterruptMode", SimTypeInt(signed=False, label="KINTERRUPT_MODE")), ("ProcessorEnableMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Group", SimTypeShort(signed=False, label="UInt16")),)), name="IO_CONNECT_INTERRUPT_FULLY_SPECIFIED_PARAMETERS", pack=False, align=None),
    "IO_CONNECT_INTERRUPT_LINE_BASED_PARAMETERS": SimStruct(OrderedDict((("PhysicalDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("InterruptObject", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("ServiceRoutine", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)), ("ServiceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SpinLock", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)), ("SynchronizeIrql", SimTypeChar(label="Byte")), ("FloatingSave", SimTypeChar(label="Byte")),)), name="IO_CONNECT_INTERRUPT_LINE_BASED_PARAMETERS", pack=False, align=None),
    "IO_CONNECT_INTERRUPT_MESSAGE_BASED_PARAMETERS": SimStruct(OrderedDict((("PhysicalDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("ConnectionContext", SimUnion({"Generic": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), "InterruptMessageTable": SimTypePointer(SimTypePointer(SimTypeRef("IO_INTERRUPT_MESSAGE_INFO", SimStruct), offset=0), offset=0), "InterruptObject": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")), ("MessageServiceRoutine", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)), ("ServiceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SpinLock", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)), ("SynchronizeIrql", SimTypeChar(label="Byte")), ("FloatingSave", SimTypeChar(label="Byte")), ("FallBackServiceRoutine", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)),)), name="IO_CONNECT_INTERRUPT_MESSAGE_BASED_PARAMETERS", pack=False, align=None),
    "IO_CONNECT_INTERRUPT_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"FullySpecified": SimTypeRef("IO_CONNECT_INTERRUPT_FULLY_SPECIFIED_PARAMETERS", SimStruct), "LineBased": SimTypeRef("IO_CONNECT_INTERRUPT_LINE_BASED_PARAMETERS", SimStruct), "MessageBased": SimTypeRef("IO_CONNECT_INTERRUPT_MESSAGE_BASED_PARAMETERS", SimStruct)}, name="<anon>", label="None")),)), name="IO_CONNECT_INTERRUPT_PARAMETERS", pack=False, align=None),
    "IO_CONTAINER_INFORMATION_CLASS": SimTypeInt(signed=False, label="IO_CONTAINER_INFORMATION_CLASS"),
    "IO_CONTAINER_NOTIFICATION_CLASS": SimTypeInt(signed=False, label="IO_CONTAINER_NOTIFICATION_CLASS"),
    "IO_COUNTERS": SimStruct(OrderedDict((("ReadOperationCount", SimTypeLongLong(signed=False, label="UInt64")), ("WriteOperationCount", SimTypeLongLong(signed=False, label="UInt64")), ("OtherOperationCount", SimTypeLongLong(signed=False, label="UInt64")), ("ReadTransferCount", SimTypeLongLong(signed=False, label="UInt64")), ("WriteTransferCount", SimTypeLongLong(signed=False, label="UInt64")), ("OtherTransferCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="IO_COUNTERS", pack=False, align=None),
    "IO_CREATE_STREAM_FILE_OPTIONS": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("TargetDeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)),)), name="IO_CREATE_STREAM_FILE_OPTIONS", pack=False, align=None),
    "IO_CSQ": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("CsqInsertIrp", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("CsqRemoveIrp", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("CsqPeekNextIrp", SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), offset=0)), ("CsqAcquireLock", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("CsqReleaseLock", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("CsqCompleteCanceledIrp", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ReservePointer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IO_CSQ", pack=False, align=None),
    "IO_CSQ_ACQUIRE_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Csq", "Irql"]), offset=0),
    "IO_CSQ_COMPLETE_CANCELED_IRP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Csq", "Irp"]), offset=0),
    "IO_CSQ_INSERT_IRP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Csq", "Irp"]), offset=0),
    "IO_CSQ_INSERT_IRP_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Csq", "Irp", "InsertContext"]), offset=0),
    "IO_CSQ_IRP_CONTEXT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Irp", SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), ("Csq", SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0)),)), name="IO_CSQ_IRP_CONTEXT", pack=False, align=None),
    "IO_CSQ_PEEK_NEXT_IRP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), arg_names=["Csq", "Irp", "PeekContext"]), offset=0),
    "IO_CSQ_RELEASE_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Csq", "Irql"]), offset=0),
    "IO_CSQ_REMOVE_IRP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_CSQ", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Csq", "Irp"]), offset=0),
    "IO_DES": SimStruct(OrderedDict((("IOD_Count", SimTypeInt(signed=False, label="UInt32")), ("IOD_Type", SimTypeInt(signed=False, label="UInt32")), ("IOD_Alloc_Base", SimTypeLongLong(signed=False, label="UInt64")), ("IOD_Alloc_End", SimTypeLongLong(signed=False, label="UInt64")), ("IOD_DesFlags", SimTypeInt(signed=False, label="IOD_DESFLAGS")),)), name="IO_DES", pack=False, align=None),
    "IO_DEVICE_HINT_ECP_CONTEXT": SimStruct(OrderedDict((("TargetDevice", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("RemainingName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="IO_DEVICE_HINT_ECP_CONTEXT", pack=False, align=None),
    "IO_DISCONNECT_INTERRUPT_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ConnectionContext", SimUnion({"Generic": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "InterruptObject": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "InterruptMessageTable": SimTypePointer(SimTypeRef("IO_INTERRUPT_MESSAGE_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IO_DISCONNECT_INTERRUPT_PARAMETERS", pack=False, align=None),
    "IO_DPC_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KDPC", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Dpc", "DeviceObject", "Irp", "Context"]), offset=0),
    "IO_DRIVER_CREATE_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=True, label="Int16")), ("ExtraCreateParameter", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("DeviceObjectHint", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TxnParameters", SimTypePointer(SimTypeRef("TXN_PARAMETER_BLOCK", SimStruct), offset=0)), ("SiloContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IO_DRIVER_CREATE_CONTEXT", pack=False, align=None),
    "IO_ERROR_LOG_MESSAGE": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("DriverNameLength", SimTypeShort(signed=False, label="UInt16")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("DriverNameOffset", SimTypeInt(signed=False, label="UInt32")), ("EntryData", SimTypeRef("IO_ERROR_LOG_PACKET", SimStruct)),)), name="IO_ERROR_LOG_MESSAGE", pack=False, align=None),
    "IO_ERROR_LOG_PACKET": SimStruct(OrderedDict((("MajorFunctionCode", SimTypeChar(label="Byte")), ("RetryCount", SimTypeChar(label="Byte")), ("DumpDataSize", SimTypeShort(signed=False, label="UInt16")), ("NumberOfStrings", SimTypeShort(signed=False, label="UInt16")), ("StringOffset", SimTypeShort(signed=False, label="UInt16")), ("EventCategory", SimTypeShort(signed=False, label="UInt16")), ("ErrorCode", SimTypeInt(signed=True, label="Int32")), ("UniqueErrorValue", SimTypeInt(signed=False, label="UInt32")), ("FinalStatus", SimTypeInt(signed=True, label="Int32")), ("SequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("IoControlCode", SimTypeInt(signed=False, label="UInt32")), ("DeviceOffset", SimTypeLongLong(signed=True, label="Int64")), ("DumpData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IO_ERROR_LOG_PACKET", pack=False, align=None),
    "IO_FOEXT_SHADOW_FILE": SimStruct(OrderedDict((("BackingFileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("BackingFltInstance", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IO_FOEXT_SHADOW_FILE", pack=False, align=None),
    "IO_FOEXT_SILO_PARAMETERS": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SiloContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IO_FOEXT_SILO_PARAMETERS", pack=False, align=None),
    "IO_INTERRUPT_MESSAGE_INFO": SimStruct(OrderedDict((("UnifiedIrql", SimTypeChar(label="Byte")), ("MessageCount", SimTypeInt(signed=False, label="UInt32")), ("MessageInfo", SimTypePointer(SimTypeRef("IO_INTERRUPT_MESSAGE_INFO_ENTRY", SimStruct), offset=0)),)), name="IO_INTERRUPT_MESSAGE_INFO", pack=False, align=None),
    "IO_INTERRUPT_MESSAGE_INFO_ENTRY": SimStruct(OrderedDict((("MessageAddress", SimTypeLongLong(signed=True, label="Int64")), ("TargetProcessorSet", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InterruptObject", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("MessageData", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("Irql", SimTypeChar(label="Byte")), ("Mode", SimTypeInt(signed=False, label="KINTERRUPT_MODE")), ("Polarity", SimTypeInt(signed=False, label="KINTERRUPT_POLARITY")),)), name="IO_INTERRUPT_MESSAGE_INFO_ENTRY", pack=False, align=None),
    "IO_IRP_EXT_TRACK_OFFSET_HEADER": SimStruct(OrderedDict((("Validation", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("TrackedOffsetCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_IRP_EXT_TRACK_OFFSET_HEADER", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_IRP_EXT_TRACK_OFFSET_HEADER", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64")], SimTypeBottom(label="Void"), arg_names=["SourceContext", "TargetContext", "RelativeOffset"]), offset=0)),)), name="IO_IRP_EXT_TRACK_OFFSET_HEADER", pack=False, align=None),
    "IO_NOTIFICATION_EVENT_CATEGORY": SimTypeInt(signed=False, label="IO_NOTIFICATION_EVENT_CATEGORY"),
    "IO_PAGING_PRIORITY": SimTypeInt(signed=False, label="IO_PAGING_PRIORITY"),
    "IO_PERSISTED_MEMORY_ENUMERATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DriverObject", "PhysicalDeviceObject", "PhysicalDeviceId", "DataTag", "DataVersion", "Context"]), offset=0),
    "IO_PRIORITY_HINT": SimTypeInt(signed=False, label="IO_PRIORITY_HINT"),
    "IO_PRIORITY_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ThreadPriority", SimTypeInt(signed=False, label="UInt32")), ("PagePriority", SimTypeInt(signed=False, label="UInt32")), ("IoPriority", SimTypeInt(signed=False, label="IO_PRIORITY_HINT")),)), name="IO_PRIORITY_INFO", pack=False, align=None),
    "IO_QUERY_DEVICE_DATA_FORMAT": SimTypeInt(signed=False, label="IO_QUERY_DEVICE_DATA_FORMAT"),
    "IO_RANGE": SimStruct(OrderedDict((("IOR_Align", SimTypeLongLong(signed=False, label="UInt64")), ("IOR_nPorts", SimTypeInt(signed=False, label="UInt32")), ("IOR_Min", SimTypeLongLong(signed=False, label="UInt64")), ("IOR_Max", SimTypeLongLong(signed=False, label="UInt64")), ("IOR_RangeFlags", SimTypeInt(signed=False, label="IOD_DESFLAGS")), ("IOR_Alias", SimTypeLongLong(signed=False, label="UInt64")),)), name="IO_RANGE", pack=False, align=None),
    "IO_REMOVE_LOCK": SimStruct(OrderedDict((("Common", SimTypeRef("IO_REMOVE_LOCK_COMMON_BLOCK", SimStruct)),)), name="IO_REMOVE_LOCK", pack=False, align=None),
    "IO_REMOVE_LOCK_COMMON_BLOCK": SimStruct(OrderedDict((("Removed", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("IoCount", SimTypeInt(signed=True, label="Int32")), ("RemoveEvent", SimTypeRef("KEVENT", SimStruct)),)), name="IO_REMOVE_LOCK_COMMON_BLOCK", pack=False, align=None),
    "IO_REMOVE_LOCK_DBG_BLOCK": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=True, label="Int32")), ("HighWatermark", SimTypeInt(signed=False, label="UInt32")), ("MaxLockedTicks", SimTypeLongLong(signed=True, label="Int64")), ("AllocateTag", SimTypeInt(signed=True, label="Int32")), ("LockList", SimTypeRef("LIST_ENTRY", SimStruct)), ("Spin", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("LowMemoryCount", SimTypeInt(signed=True, label="Int32")), ("Reserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("Reserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Blocks", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="IO_REMOVE_LOCK_DBG_BLOCK", pack=False, align=None),
    "IO_REPORT_INTERRUPT_ACTIVE_STATE_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ConnectionContext", SimUnion({"Generic": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "InterruptObject": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "InterruptMessageTable": SimTypePointer(SimTypeRef("IO_INTERRUPT_MESSAGE_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IO_REPORT_INTERRUPT_ACTIVE_STATE_PARAMETERS", pack=False, align=None),
    "IO_RESOURCE": SimStruct(OrderedDict((("IO_Header", SimTypeRef("IO_DES", SimStruct)), ("IO_Data", SimTypePointer(SimTypeRef("IO_RANGE", SimStruct), offset=0)),)), name="IO_RESOURCE", pack=False, align=None),
    "IO_RESOURCE_DESCRIPTOR": SimStruct(OrderedDict((("Option", SimTypeChar(label="Byte")), ("Type", SimTypeChar(label="Byte")), ("ShareDisposition", SimTypeChar(label="Byte")), ("Spare1", SimTypeChar(label="Byte")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Spare2", SimTypeShort(signed=False, label="UInt16")), ("u", SimUnion({"Port": SimTypeRef("_Port_e__Struct", SimStruct), "Memory": SimTypeRef("_Memory_e__Struct", SimStruct), "Interrupt": SimTypeRef("_Interrupt_e__Struct", SimStruct), "Dma": SimTypeRef("_Dma_e__Struct", SimStruct), "DmaV3": SimTypeRef("_DmaV3_e__Struct", SimStruct), "Generic": SimTypeRef("_Generic_e__Struct", SimStruct), "DevicePrivate": SimTypeRef("_DevicePrivate_e__Struct", SimStruct), "BusNumber": SimTypeRef("_BusNumber_e__Struct", SimStruct), "ConfigData": SimTypeRef("_ConfigData_e__Struct", SimStruct), "Memory40": SimTypeRef("_Memory40_e__Struct", SimStruct), "Memory48": SimTypeRef("_Memory48_e__Struct", SimStruct), "Memory64": SimTypeRef("_Memory64_e__Struct", SimStruct), "Connection": SimTypeRef("_Connection_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="IO_RESOURCE_DESCRIPTOR", pack=False, align=None),
    "IO_RESOURCE_LIST": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Revision", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Descriptors", SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0)),)), name="IO_RESOURCE_LIST", pack=False, align=None),
    "IO_RESOURCE_REQUIREMENTS_LIST": SimStruct(OrderedDict((("ListSize", SimTypeInt(signed=False, label="UInt32")), ("InterfaceType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("SlotNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("AlternativeLists", SimTypeInt(signed=False, label="UInt32")), ("List", SimTypePointer(SimTypeRef("IO_RESOURCE_LIST", SimStruct), offset=0)),)), name="IO_RESOURCE_REQUIREMENTS_LIST", pack=False, align=None),
    "IO_SCSI_CAPABILITIES": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("MaximumTransferLength", SimTypeInt(signed=False, label="UInt32")), ("MaximumPhysicalPages", SimTypeInt(signed=False, label="UInt32")), ("SupportedAsynchronousEvents", SimTypeInt(signed=False, label="UInt32")), ("AlignmentMask", SimTypeInt(signed=False, label="UInt32")), ("TaggedQueuing", SimTypeChar(label="Byte")), ("AdapterScansDown", SimTypeChar(label="Byte")), ("AdapterUsesPio", SimTypeChar(label="Byte")),)), name="IO_SCSI_CAPABILITIES", pack=False, align=None),
    "IO_SECURITY_CONTEXT": SimStruct(OrderedDict((("SecurityQos", SimTypePointer(SimTypeRef("SECURITY_QUALITY_OF_SERVICE", SimStruct), offset=0)), ("AccessState", SimTypePointer(SimTypeRef("ACCESS_STATE", SimStruct), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("FullCreateOptions", SimTypeInt(signed=False, label="UInt32")),)), name="IO_SECURITY_CONTEXT", pack=False, align=None),
    "IO_SESSION_CONNECT_INFO": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("LocalSession", SimTypeChar(label="Byte")),)), name="IO_SESSION_CONNECT_INFO", pack=False, align=None),
    "IO_SESSION_EVENT": SimTypeInt(signed=False, label="IO_SESSION_EVENT"),
    "IO_SESSION_NOTIFICATION_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["SessionObject", "IoObject", "Event", "Context", "NotificationPayload", "PayloadLength"]), offset=0),
    "IO_SESSION_STATE": SimTypeInt(signed=False, label="IO_SESSION_STATE"),
    "IO_SESSION_STATE_INFORMATION": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("SessionState", SimTypeInt(signed=False, label="IO_SESSION_STATE")), ("LocalSession", SimTypeChar(label="Byte")),)), name="IO_SESSION_STATE_INFORMATION", pack=False, align=None),
    "IO_SESSION_STATE_NOTIFICATION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("IoObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EventMask", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IO_SESSION_STATE_NOTIFICATION", pack=False, align=None),
    "IO_STACK_LOCATION": SimStruct(OrderedDict((("MajorFunction", SimTypeChar(label="Byte")), ("MinorFunction", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Control", SimTypeChar(label="Byte")), ("Parameters", SimUnion({"Create": SimTypeRef("_Create_e__Struct", SimStruct), "CreatePipe": SimTypeRef("_CreatePipe_e__Struct", SimStruct), "CreateMailslot": SimTypeRef("_CreateMailslot_e__Struct", SimStruct), "Read": SimTypeRef("_Read_e__Struct", SimStruct), "Write": SimTypeRef("_Write_e__Struct", SimStruct), "QueryDirectory": SimTypeRef("_QueryDirectory_e__Struct", SimStruct), "NotifyDirectory": SimTypeRef("_NotifyDirectory_e__Struct", SimStruct), "NotifyDirectoryEx": SimTypeRef("_NotifyDirectoryEx_e__Struct", SimStruct), "QueryFile": SimTypeRef("_QueryFile_e__Struct", SimStruct), "SetFile": SimTypeRef("_SetFile_e__Struct", SimStruct), "QueryEa": SimTypeRef("_QueryEa_e__Struct", SimStruct), "SetEa": SimTypeRef("_SetEa_e__Struct", SimStruct), "QueryVolume": SimTypeRef("_QueryVolume_e__Struct", SimStruct), "SetVolume": SimTypeRef("_SetVolume_e__Struct", SimStruct), "FileSystemControl": SimTypeRef("_FileSystemControl_e__Struct", SimStruct), "LockControl": SimTypeRef("_LockControl_e__Struct", SimStruct), "DeviceIoControl": SimTypeRef("_DeviceIoControl_e__Struct", SimStruct), "QuerySecurity": SimTypeRef("_QuerySecurity_e__Struct", SimStruct), "SetSecurity": SimTypeRef("_SetSecurity_e__Struct", SimStruct), "MountVolume": SimTypeRef("_MountVolume_e__Struct", SimStruct), "VerifyVolume": SimTypeRef("_VerifyVolume_e__Struct", SimStruct), "Scsi": SimTypeRef("_Scsi_e__Struct", SimStruct), "QueryQuota": SimTypeRef("_QueryQuota_e__Struct", SimStruct), "SetQuota": SimTypeRef("_SetQuota_e__Struct", SimStruct), "QueryDeviceRelations": SimTypeRef("_QueryDeviceRelations_e__Struct", SimStruct), "QueryInterface": SimTypeRef("_QueryInterface_e__Struct", SimStruct), "DeviceCapabilities": SimTypeRef("_DeviceCapabilities_e__Struct", SimStruct), "FilterResourceRequirements": SimTypeRef("_FilterResourceRequirements_e__Struct", SimStruct), "ReadWriteConfig": SimTypeRef("_ReadWriteConfig_e__Struct", SimStruct), "SetLock": SimTypeRef("_SetLock_e__Struct", SimStruct), "QueryId": SimTypeRef("_QueryId_e__Struct", SimStruct), "QueryDeviceText": SimTypeRef("_QueryDeviceText_e__Struct", SimStruct), "UsageNotification": SimTypeRef("_UsageNotification_e__Struct", SimStruct), "WaitWake": SimTypeRef("_WaitWake_e__Struct", SimStruct), "PowerSequence": SimTypeRef("_PowerSequence_e__Struct", SimStruct), "Power": SimTypeRef("_Power_e__Struct", SimStruct), "StartDevice": SimTypeRef("_StartDevice_e__Struct", SimStruct), "WMI": SimTypeRef("_WMI_e__Struct", SimStruct), "Others": SimTypeRef("_Others_e__Struct", SimStruct)}, name="<anon>", label="None")), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("CompletionRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="IO_STACK_LOCATION", pack=False, align=None),
    "IO_STATUS_BLOCK": SimStruct(OrderedDict((("Anonymous", SimUnion({"Status": SimTypeInt(signed=True, label="Int32"), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("Information", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="IO_STATUS_BLOCK", pack=False, align=None),
    "IO_STATUS_BLOCK32": SimStruct(OrderedDict((("Status", SimTypeInt(signed=True, label="Int32")), ("Information", SimTypeInt(signed=False, label="UInt32")),)), name="IO_STATUS_BLOCK32", pack=False, align=None),
    "IO_STATUS_BLOCK64": SimStruct(OrderedDict((("Anonymous", SimUnion({"Status": SimTypeInt(signed=True, label="Int32"), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("Information", SimTypeLongLong(signed=False, label="UInt64")),)), name="IO_STATUS_BLOCK64", pack=False, align=None),
    "IO_STOP_ON_SYMLINK_FILTER_ECP_v0": SimStruct(OrderedDict((("Out", SimTypeRef("_Out_e__Struct", SimStruct)),)), name="IO_STOP_ON_SYMLINK_FILTER_ECP_v0", pack=False, align=None),
    "IO_TIMER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Context"]), offset=0),
    "IO_WORKITEM_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Context"]), offset=0),
    "IO_WORKITEM_ROUTINE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["IoObject", "Context", "IoWorkItem"]), offset=0),
    "IObjMgr": SimTypeBottom(label="IObjMgr"),
    "IObjPool": SimTypeBottom(label="IObjPool"),
    "IObjectAccessControl": SimTypeBottom(label="IObjectAccessControl"),
    "IObjectArray": SimTypeBottom(label="IObjectArray"),
    "IObjectCollection": SimTypeBottom(label="IObjectCollection"),
    "IObjectConstruct": SimTypeBottom(label="IObjectConstruct"),
    "IObjectConstructString": SimTypeBottom(label="IObjectConstructString"),
    "IObjectContext": SimTypeBottom(label="IObjectContext"),
    "IObjectContextActivity": SimTypeBottom(label="IObjectContextActivity"),
    "IObjectContextInfo": SimTypeBottom(label="IObjectContextInfo"),
    "IObjectContextInfo2": SimTypeBottom(label="IObjectContextInfo2"),
    "IObjectContextTip": SimTypeBottom(label="IObjectContextTip"),
    "IObjectControl": SimTypeBottom(label="IObjectControl"),
    "IObjectHandle": SimTypeBottom(label="IObjectHandle"),
    "IObjectId": SimTypeBottom(label="IObjectId"),
    "IObjectIdentity": SimTypeBottom(label="IObjectIdentity"),
    "IObjectIds": SimTypeBottom(label="IObjectIds"),
    "IObjectModelProvider": SimTypeBottom(label="IObjectModelProvider"),
    "IObjectProvider": SimTypeBottom(label="IObjectProvider"),
    "IObjectSafety": SimTypeBottom(label="IObjectSafety"),
    "IObjectWithAppUserModelID": SimTypeBottom(label="IObjectWithAppUserModelID"),
    "IObjectWithBackReferences": SimTypeBottom(label="IObjectWithBackReferences"),
    "IObjectWithCancelEvent": SimTypeBottom(label="IObjectWithCancelEvent"),
    "IObjectWithFolderEnumMode": SimTypeBottom(label="IObjectWithFolderEnumMode"),
    "IObjectWithProgID": SimTypeBottom(label="IObjectWithProgID"),
    "IObjectWithPropertyKey": SimTypeBottom(label="IObjectWithPropertyKey"),
    "IObjectWithSelection": SimTypeBottom(label="IObjectWithSelection"),
    "IObjectWithSite": SimTypeBottom(label="IObjectWithSite"),
    "IOfflineFilesCache": SimTypeBottom(label="IOfflineFilesCache"),
    "IOfflineFilesCache2": SimTypeBottom(label="IOfflineFilesCache2"),
    "IOfflineFilesChangeInfo": SimTypeBottom(label="IOfflineFilesChangeInfo"),
    "IOfflineFilesConnectionInfo": SimTypeBottom(label="IOfflineFilesConnectionInfo"),
    "IOfflineFilesDirectoryItem": SimTypeBottom(label="IOfflineFilesDirectoryItem"),
    "IOfflineFilesDirtyInfo": SimTypeBottom(label="IOfflineFilesDirtyInfo"),
    "IOfflineFilesErrorInfo": SimTypeBottom(label="IOfflineFilesErrorInfo"),
    "IOfflineFilesEvents": SimTypeBottom(label="IOfflineFilesEvents"),
    "IOfflineFilesEvents2": SimTypeBottom(label="IOfflineFilesEvents2"),
    "IOfflineFilesEvents3": SimTypeBottom(label="IOfflineFilesEvents3"),
    "IOfflineFilesEvents4": SimTypeBottom(label="IOfflineFilesEvents4"),
    "IOfflineFilesEventsFilter": SimTypeBottom(label="IOfflineFilesEventsFilter"),
    "IOfflineFilesFileItem": SimTypeBottom(label="IOfflineFilesFileItem"),
    "IOfflineFilesFileSysInfo": SimTypeBottom(label="IOfflineFilesFileSysInfo"),
    "IOfflineFilesGhostInfo": SimTypeBottom(label="IOfflineFilesGhostInfo"),
    "IOfflineFilesItem": SimTypeBottom(label="IOfflineFilesItem"),
    "IOfflineFilesItemContainer": SimTypeBottom(label="IOfflineFilesItemContainer"),
    "IOfflineFilesItemFilter": SimTypeBottom(label="IOfflineFilesItemFilter"),
    "IOfflineFilesPinInfo": SimTypeBottom(label="IOfflineFilesPinInfo"),
    "IOfflineFilesPinInfo2": SimTypeBottom(label="IOfflineFilesPinInfo2"),
    "IOfflineFilesProgress": SimTypeBottom(label="IOfflineFilesProgress"),
    "IOfflineFilesServerItem": SimTypeBottom(label="IOfflineFilesServerItem"),
    "IOfflineFilesSetting": SimTypeBottom(label="IOfflineFilesSetting"),
    "IOfflineFilesShareInfo": SimTypeBottom(label="IOfflineFilesShareInfo"),
    "IOfflineFilesShareItem": SimTypeBottom(label="IOfflineFilesShareItem"),
    "IOfflineFilesSimpleProgress": SimTypeBottom(label="IOfflineFilesSimpleProgress"),
    "IOfflineFilesSuspend": SimTypeBottom(label="IOfflineFilesSuspend"),
    "IOfflineFilesSuspendInfo": SimTypeBottom(label="IOfflineFilesSuspendInfo"),
    "IOfflineFilesSyncConflictHandler": SimTypeBottom(label="IOfflineFilesSyncConflictHandler"),
    "IOfflineFilesSyncErrorInfo": SimTypeBottom(label="IOfflineFilesSyncErrorInfo"),
    "IOfflineFilesSyncErrorItemInfo": SimTypeBottom(label="IOfflineFilesSyncErrorItemInfo"),
    "IOfflineFilesSyncProgress": SimTypeBottom(label="IOfflineFilesSyncProgress"),
    "IOfflineFilesTransparentCacheInfo": SimTypeBottom(label="IOfflineFilesTransparentCacheInfo"),
    "IOleAdviseHolder": SimTypeBottom(label="IOleAdviseHolder"),
    "IOleCache": SimTypeBottom(label="IOleCache"),
    "IOleCache2": SimTypeBottom(label="IOleCache2"),
    "IOleCacheControl": SimTypeBottom(label="IOleCacheControl"),
    "IOleClientSite": SimTypeBottom(label="IOleClientSite"),
    "IOleCommandTarget": SimTypeBottom(label="IOleCommandTarget"),
    "IOleContainer": SimTypeBottom(label="IOleContainer"),
    "IOleControl": SimTypeBottom(label="IOleControl"),
    "IOleControlSite": SimTypeBottom(label="IOleControlSite"),
    "IOleDocument": SimTypeBottom(label="IOleDocument"),
    "IOleDocumentSite": SimTypeBottom(label="IOleDocumentSite"),
    "IOleDocumentView": SimTypeBottom(label="IOleDocumentView"),
    "IOleInPlaceActiveObject": SimTypeBottom(label="IOleInPlaceActiveObject"),
    "IOleInPlaceFrame": SimTypeBottom(label="IOleInPlaceFrame"),
    "IOleInPlaceObject": SimTypeBottom(label="IOleInPlaceObject"),
    "IOleInPlaceObjectWindowless": SimTypeBottom(label="IOleInPlaceObjectWindowless"),
    "IOleInPlaceSite": SimTypeBottom(label="IOleInPlaceSite"),
    "IOleInPlaceSiteEx": SimTypeBottom(label="IOleInPlaceSiteEx"),
    "IOleInPlaceSiteWindowless": SimTypeBottom(label="IOleInPlaceSiteWindowless"),
    "IOleInPlaceUIWindow": SimTypeBottom(label="IOleInPlaceUIWindow"),
    "IOleItemContainer": SimTypeBottom(label="IOleItemContainer"),
    "IOleLink": SimTypeBottom(label="IOleLink"),
    "IOleObject": SimTypeBottom(label="IOleObject"),
    "IOleParentUndoUnit": SimTypeBottom(label="IOleParentUndoUnit"),
    "IOleUILinkContainerA": SimTypeBottom(label="IOleUILinkContainerA"),
    "IOleUILinkContainerW": SimTypeBottom(label="IOleUILinkContainerW"),
    "IOleUILinkInfoA": SimTypeBottom(label="IOleUILinkInfoA"),
    "IOleUILinkInfoW": SimTypeBottom(label="IOleUILinkInfoW"),
    "IOleUIObjInfoA": SimTypeBottom(label="IOleUIObjInfoA"),
    "IOleUIObjInfoW": SimTypeBottom(label="IOleUIObjInfoW"),
    "IOleUndoManager": SimTypeBottom(label="IOleUndoManager"),
    "IOleUndoUnit": SimTypeBottom(label="IOleUndoUnit"),
    "IOleWindow": SimTypeBottom(label="IOleWindow"),
    "IOmHistory": SimTypeBottom(label="IOmHistory"),
    "IOmNavigator": SimTypeBottom(label="IOmNavigator"),
    "IOpLockStatus": SimTypeBottom(label="IOpLockStatus"),
    "IOpcCertificateEnumerator": SimTypeBottom(label="IOpcCertificateEnumerator"),
    "IOpcCertificateSet": SimTypeBottom(label="IOpcCertificateSet"),
    "IOpcDigitalSignature": SimTypeBottom(label="IOpcDigitalSignature"),
    "IOpcDigitalSignatureEnumerator": SimTypeBottom(label="IOpcDigitalSignatureEnumerator"),
    "IOpcDigitalSignatureManager": SimTypeBottom(label="IOpcDigitalSignatureManager"),
    "IOpcFactory": SimTypeBottom(label="IOpcFactory"),
    "IOpcPackage": SimTypeBottom(label="IOpcPackage"),
    "IOpcPart": SimTypeBottom(label="IOpcPart"),
    "IOpcPartEnumerator": SimTypeBottom(label="IOpcPartEnumerator"),
    "IOpcPartSet": SimTypeBottom(label="IOpcPartSet"),
    "IOpcPartUri": SimTypeBottom(label="IOpcPartUri"),
    "IOpcRelationship": SimTypeBottom(label="IOpcRelationship"),
    "IOpcRelationshipEnumerator": SimTypeBottom(label="IOpcRelationshipEnumerator"),
    "IOpcRelationshipSelector": SimTypeBottom(label="IOpcRelationshipSelector"),
    "IOpcRelationshipSelectorEnumerator": SimTypeBottom(label="IOpcRelationshipSelectorEnumerator"),
    "IOpcRelationshipSelectorSet": SimTypeBottom(label="IOpcRelationshipSelectorSet"),
    "IOpcRelationshipSet": SimTypeBottom(label="IOpcRelationshipSet"),
    "IOpcSignatureCustomObject": SimTypeBottom(label="IOpcSignatureCustomObject"),
    "IOpcSignatureCustomObjectEnumerator": SimTypeBottom(label="IOpcSignatureCustomObjectEnumerator"),
    "IOpcSignatureCustomObjectSet": SimTypeBottom(label="IOpcSignatureCustomObjectSet"),
    "IOpcSignaturePartReference": SimTypeBottom(label="IOpcSignaturePartReference"),
    "IOpcSignaturePartReferenceEnumerator": SimTypeBottom(label="IOpcSignaturePartReferenceEnumerator"),
    "IOpcSignaturePartReferenceSet": SimTypeBottom(label="IOpcSignaturePartReferenceSet"),
    "IOpcSignatureReference": SimTypeBottom(label="IOpcSignatureReference"),
    "IOpcSignatureReferenceEnumerator": SimTypeBottom(label="IOpcSignatureReferenceEnumerator"),
    "IOpcSignatureReferenceSet": SimTypeBottom(label="IOpcSignatureReferenceSet"),
    "IOpcSignatureRelationshipReference": SimTypeBottom(label="IOpcSignatureRelationshipReference"),
    "IOpcSignatureRelationshipReferenceEnumerator": SimTypeBottom(label="IOpcSignatureRelationshipReferenceEnumerator"),
    "IOpcSignatureRelationshipReferenceSet": SimTypeBottom(label="IOpcSignatureRelationshipReferenceSet"),
    "IOpcSigningOptions": SimTypeBottom(label="IOpcSigningOptions"),
    "IOpcUri": SimTypeBottom(label="IOpcUri"),
    "IOpenControlPanel": SimTypeBottom(label="IOpenControlPanel"),
    "IOpenRowset": SimTypeBottom(label="IOpenRowset"),
    "IOpenSearchSource": SimTypeBottom(label="IOpenSearchSource"),
    "IOpenService": SimTypeBottom(label="IOpenService"),
    "IOpenServiceActivity": SimTypeBottom(label="IOpenServiceActivity"),
    "IOpenServiceActivityCategory": SimTypeBottom(label="IOpenServiceActivityCategory"),
    "IOpenServiceActivityInput": SimTypeBottom(label="IOpenServiceActivityInput"),
    "IOpenServiceActivityManager": SimTypeBottom(label="IOpenServiceActivityManager"),
    "IOpenServiceActivityOutputContext": SimTypeBottom(label="IOpenServiceActivityOutputContext"),
    "IOpenServiceManager": SimTypeBottom(label="IOpenServiceManager"),
    "IOperationsProgressDialog": SimTypeBottom(label="IOperationsProgressDialog"),
    "IOplockStorage": SimTypeBottom(label="IOplockStorage"),
    "IOptionDescription": SimTypeBottom(label="IOptionDescription"),
    "IOverlay": SimTypeBottom(label="IOverlay"),
    "IOverlayNotify": SimTypeBottom(label="IOverlayNotify"),
    "IOverlayNotify2": SimTypeBottom(label="IOverlayNotify2"),
    "IP4_ARRAY": SimStruct(OrderedDict((("AddrCount", SimTypeInt(signed=False, label="UInt32")), ("AddrArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IP4_ARRAY", pack=False, align=None),
    "IP6_ADDRESS": SimUnion({"IP6Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4), "IP6Word": SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8), "IP6Byte": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"),
    "IPAT": SimTypeBottom(label="IPAT"),
    "IPBDAAttributesDescriptor": SimTypeBottom(label="IPBDAAttributesDescriptor"),
    "IPBDAEntitlementDescriptor": SimTypeBottom(label="IPBDAEntitlementDescriptor"),
    "IPBDASiParser": SimTypeBottom(label="IPBDASiParser"),
    "IPBDA_EIT": SimTypeBottom(label="IPBDA_EIT"),
    "IPBDA_Services": SimTypeBottom(label="IPBDA_Services"),
    "IPFILTERPROPERTIES": SimTypeInt(signed=False, label="IPFILTERPROPERTIES"),
    "IPMApplicationInfo": SimTypeBottom(label="IPMApplicationInfo"),
    "IPMApplicationInfoEnumerator": SimTypeBottom(label="IPMApplicationInfoEnumerator"),
    "IPMBackgroundServiceAgentInfo": SimTypeBottom(label="IPMBackgroundServiceAgentInfo"),
    "IPMBackgroundServiceAgentInfoEnumerator": SimTypeBottom(label="IPMBackgroundServiceAgentInfoEnumerator"),
    "IPMBackgroundWorkerInfo": SimTypeBottom(label="IPMBackgroundWorkerInfo"),
    "IPMBackgroundWorkerInfoEnumerator": SimTypeBottom(label="IPMBackgroundWorkerInfoEnumerator"),
    "IPMDeploymentManager": SimTypeBottom(label="IPMDeploymentManager"),
    "IPMEnumerationManager": SimTypeBottom(label="IPMEnumerationManager"),
    "IPMExtensionCachedFileUpdaterInfo": SimTypeBottom(label="IPMExtensionCachedFileUpdaterInfo"),
    "IPMExtensionContractInfo": SimTypeBottom(label="IPMExtensionContractInfo"),
    "IPMExtensionFileExtensionInfo": SimTypeBottom(label="IPMExtensionFileExtensionInfo"),
    "IPMExtensionFileOpenPickerInfo": SimTypeBottom(label="IPMExtensionFileOpenPickerInfo"),
    "IPMExtensionFileSavePickerInfo": SimTypeBottom(label="IPMExtensionFileSavePickerInfo"),
    "IPMExtensionInfo": SimTypeBottom(label="IPMExtensionInfo"),
    "IPMExtensionInfoEnumerator": SimTypeBottom(label="IPMExtensionInfoEnumerator"),
    "IPMExtensionProtocolInfo": SimTypeBottom(label="IPMExtensionProtocolInfo"),
    "IPMExtensionShareTargetInfo": SimTypeBottom(label="IPMExtensionShareTargetInfo"),
    "IPMI_OS_SEL_RECORD": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("RecordType", SimTypeInt(signed=False, label="IPMI_OS_SEL_RECORD_TYPE")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="IPMI_OS_SEL_RECORD", pack=False, align=None),
    "IPMI_OS_SEL_RECORD_TYPE": SimTypeInt(signed=False, label="IPMI_OS_SEL_RECORD_TYPE"),
    "IPMLiveTileJobInfo": SimTypeBottom(label="IPMLiveTileJobInfo"),
    "IPMLiveTileJobInfoEnumerator": SimTypeBottom(label="IPMLiveTileJobInfoEnumerator"),
    "IPMT": SimTypeBottom(label="IPMT"),
    "IPMTaskInfo": SimTypeBottom(label="IPMTaskInfo"),
    "IPMTaskInfoEnumerator": SimTypeBottom(label="IPMTaskInfoEnumerator"),
    "IPMTileInfo": SimTypeBottom(label="IPMTileInfo"),
    "IPMTileInfoEnumerator": SimTypeBottom(label="IPMTileInfoEnumerator"),
    "IPMTilePropertyEnumerator": SimTypeBottom(label="IPMTilePropertyEnumerator"),
    "IPMTilePropertyInfo": SimTypeBottom(label="IPMTilePropertyInfo"),
    "IPNG_ADDRESS": SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"),
    "IPNPXAssociation": SimTypeBottom(label="IPNPXAssociation"),
    "IPNPXDeviceAssociation": SimTypeBottom(label="IPNPXDeviceAssociation"),
    "IPPROTO": SimTypeInt(signed=False, label="IPPROTO"),
    "IPSEC_ADDRESS_INFO0": SimStruct(OrderedDict((("numV4Addresses", SimTypeInt(signed=False, label="UInt32")), ("v4Addresses", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("numV6Addresses", SimTypeInt(signed=False, label="UInt32")), ("v6Addresses", SimTypePointer(SimTypeRef("FWP_BYTE_ARRAY16", SimStruct), offset=0)),)), name="IPSEC_ADDRESS_INFO0", pack=False, align=None),
    "IPSEC_AGGREGATE_DROP_PACKET_STATISTICS0": SimStruct(OrderedDict((("invalidSpisOnInbound", SimTypeInt(signed=False, label="UInt32")), ("decryptionFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("authenticationFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("udpEspValidationFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("replayCheckFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("invalidClearTextInbound", SimTypeInt(signed=False, label="UInt32")), ("saNotInitializedOnInbound", SimTypeInt(signed=False, label="UInt32")), ("receiveOverIncorrectSaInbound", SimTypeInt(signed=False, label="UInt32")), ("secureReceivesNotMatchingFilters", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_AGGREGATE_DROP_PACKET_STATISTICS0", pack=False, align=None),
    "IPSEC_AGGREGATE_DROP_PACKET_STATISTICS1": SimStruct(OrderedDict((("invalidSpisOnInbound", SimTypeInt(signed=False, label="UInt32")), ("decryptionFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("authenticationFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("udpEspValidationFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("replayCheckFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("invalidClearTextInbound", SimTypeInt(signed=False, label="UInt32")), ("saNotInitializedOnInbound", SimTypeInt(signed=False, label="UInt32")), ("receiveOverIncorrectSaInbound", SimTypeInt(signed=False, label="UInt32")), ("secureReceivesNotMatchingFilters", SimTypeInt(signed=False, label="UInt32")), ("totalDropPacketsInbound", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_AGGREGATE_DROP_PACKET_STATISTICS1", pack=False, align=None),
    "IPSEC_AGGREGATE_SA_STATISTICS0": SimStruct(OrderedDict((("activeSas", SimTypeInt(signed=False, label="UInt32")), ("pendingSaNegotiations", SimTypeInt(signed=False, label="UInt32")), ("totalSasAdded", SimTypeInt(signed=False, label="UInt32")), ("totalSasDeleted", SimTypeInt(signed=False, label="UInt32")), ("successfulRekeys", SimTypeInt(signed=False, label="UInt32")), ("activeTunnels", SimTypeInt(signed=False, label="UInt32")), ("offloadedSas", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_AGGREGATE_SA_STATISTICS0", pack=False, align=None),
    "IPSEC_AH_DROP_PACKET_STATISTICS0": SimStruct(OrderedDict((("invalidSpisOnInbound", SimTypeInt(signed=False, label="UInt32")), ("authenticationFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("replayCheckFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("saNotInitializedOnInbound", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_AH_DROP_PACKET_STATISTICS0", pack=False, align=None),
    "IPSEC_AUTH_AND_CIPHER_TRANSFORM0": SimStruct(OrderedDict((("authTransform", SimTypeRef("IPSEC_AUTH_TRANSFORM0", SimStruct)), ("cipherTransform", SimTypeRef("IPSEC_CIPHER_TRANSFORM0", SimStruct)),)), name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None),
    "IPSEC_AUTH_TRANSFORM0": SimStruct(OrderedDict((("authTransformId", SimTypeRef("IPSEC_AUTH_TRANSFORM_ID0", SimStruct)), ("cryptoModuleId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None),
    "IPSEC_AUTH_TRANSFORM_ID0": SimStruct(OrderedDict((("authType", SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE")), ("authConfig", SimTypeChar(label="Byte")),)), name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None),
    "IPSEC_AUTH_TYPE": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"),
    "IPSEC_CIPHER_TRANSFORM0": SimStruct(OrderedDict((("cipherTransformId", SimTypeRef("IPSEC_CIPHER_TRANSFORM_ID0", SimStruct)), ("cryptoModuleId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None),
    "IPSEC_CIPHER_TRANSFORM_ID0": SimStruct(OrderedDict((("cipherType", SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE")), ("cipherConfig", SimTypeChar(label="Byte")),)), name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None),
    "IPSEC_CIPHER_TYPE": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"),
    "IPSEC_DOSP_FLAGS": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"),
    "IPSEC_DOSP_OPTIONS0": SimStruct(OrderedDict((("stateIdleTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("perIPRateLimitQueueIdleTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("ipV6IPsecUnauthDscp", SimTypeChar(label="Byte")), ("ipV6IPsecUnauthRateLimitBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("ipV6IPsecUnauthPerIPRateLimitBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("ipV6IPsecAuthDscp", SimTypeChar(label="Byte")), ("ipV6IPsecAuthRateLimitBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("icmpV6Dscp", SimTypeChar(label="Byte")), ("icmpV6RateLimitBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("ipV6FilterExemptDscp", SimTypeChar(label="Byte")), ("ipV6FilterExemptRateLimitBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("defBlockExemptDscp", SimTypeChar(label="Byte")), ("defBlockExemptRateLimitBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("maxStateEntries", SimTypeInt(signed=False, label="UInt32")), ("maxPerIPRateLimitQueues", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS")), ("numPublicIFLuids", SimTypeInt(signed=False, label="UInt32")), ("publicIFLuids", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("numInternalIFLuids", SimTypeInt(signed=False, label="UInt32")), ("internalIFLuids", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("publicV6AddrMask", SimTypeRef("FWP_V6_ADDR_AND_MASK", SimStruct)), ("internalV6AddrMask", SimTypeRef("FWP_V6_ADDR_AND_MASK", SimStruct)),)), name="IPSEC_DOSP_OPTIONS0", pack=False, align=None),
    "IPSEC_DOSP_STATE0": SimStruct(OrderedDict((("publicHostV6Addr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("internalHostV6Addr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("totalInboundIPv6IPsecAuthPackets", SimTypeLongLong(signed=False, label="UInt64")), ("totalOutboundIPv6IPsecAuthPackets", SimTypeLongLong(signed=False, label="UInt64")), ("durationSecs", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_DOSP_STATE0", pack=False, align=None),
    "IPSEC_DOSP_STATE_ENUM_TEMPLATE0": SimStruct(OrderedDict((("publicV6AddrMask", SimTypeRef("FWP_V6_ADDR_AND_MASK", SimStruct)), ("internalV6AddrMask", SimTypeRef("FWP_V6_ADDR_AND_MASK", SimStruct)),)), name="IPSEC_DOSP_STATE_ENUM_TEMPLATE0", pack=False, align=None),
    "IPSEC_DOSP_STATISTICS0": SimStruct(OrderedDict((("totalStateEntriesCreated", SimTypeLongLong(signed=False, label="UInt64")), ("currentStateEntries", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundAllowedIPv6IPsecUnauthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundRatelimitDiscardedIPv6IPsecUnauthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundPerIPRatelimitDiscardedIPv6IPsecUnauthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundOtherDiscardedIPv6IPsecUnauthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundAllowedIPv6IPsecAuthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundRatelimitDiscardedIPv6IPsecAuthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundOtherDiscardedIPv6IPsecAuthPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundAllowedICMPv6Pkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundRatelimitDiscardedICMPv6Pkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundAllowedIPv6FilterExemptPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundRatelimitDiscardedIPv6FilterExemptPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundDiscardedIPv6FilterBlockPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundAllowedDefBlockExemptPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundRatelimitDiscardedDefBlockExemptPkts", SimTypeLongLong(signed=False, label="UInt64")), ("totalInboundDiscardedDefBlockPkts", SimTypeLongLong(signed=False, label="UInt64")), ("currentInboundIPv6IPsecUnauthPerIPRateLimitQueues", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_DOSP_STATISTICS0", pack=False, align=None),
    "IPSEC_ESP_DROP_PACKET_STATISTICS0": SimStruct(OrderedDict((("invalidSpisOnInbound", SimTypeInt(signed=False, label="UInt32")), ("decryptionFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("authenticationFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("replayCheckFailuresOnInbound", SimTypeInt(signed=False, label="UInt32")), ("saNotInitializedOnInbound", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_ESP_DROP_PACKET_STATISTICS0", pack=False, align=None),
    "IPSEC_FAILURE_POINT": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"),
    "IPSEC_GETSPI0": SimStruct(OrderedDict((("inboundIpsecTraffic", SimTypeRef("IPSEC_TRAFFIC0", SimStruct)), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"inboundUdpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("rngCryptoModuleID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="IPSEC_GETSPI0", pack=False, align=None),
    "IPSEC_GETSPI1": SimStruct(OrderedDict((("inboundIpsecTraffic", SimTypeRef("IPSEC_TRAFFIC1", SimStruct)), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"inboundUdpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("rngCryptoModuleID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="IPSEC_GETSPI1", pack=False, align=None),
    "IPSEC_ID0": SimStruct(OrderedDict((("mmTargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("emTargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("numTokens", SimTypeInt(signed=False, label="UInt32")), ("tokens", SimTypePointer(SimTypeRef("IPSEC_TOKEN0", SimStruct), offset=0)), ("explicitCredentials", SimTypeLongLong(signed=False, label="UInt64")), ("logonId", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_ID0", pack=False, align=None),
    "IPSEC_KEYING_POLICY0": SimStruct(OrderedDict((("numKeyMods", SimTypeInt(signed=False, label="UInt32")), ("keyModKeys", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="IPSEC_KEYING_POLICY0", pack=False, align=None),
    "IPSEC_KEYING_POLICY1": SimStruct(OrderedDict((("numKeyMods", SimTypeInt(signed=False, label="UInt32")), ("keyModKeys", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_KEYING_POLICY1", pack=False, align=None),
    "IPSEC_KEYMODULE_STATE0": SimStruct(OrderedDict((("keyModuleKey", SimTypeBottom(label="Guid")), ("stateBlob", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IPSEC_KEYMODULE_STATE0", pack=False, align=None),
    "IPSEC_KEY_MANAGER0": SimStruct(OrderedDict((("keyManagerKey", SimTypeBottom(label="Guid")), ("displayData", SimTypeRef("FWPM_DISPLAY_DATA0", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("keyDictationTimeoutHint", SimTypeChar(label="Byte")),)), name="IPSEC_KEY_MANAGER0", pack=False, align=None),
    "IPSEC_KEY_MANAGER_CALLBACKS0": SimStruct(OrderedDict((("reserved", SimTypeBottom(label="Guid")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("keyDictationCheck", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IKEEXT_TRAFFIC0", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ikeTraffic", "willDictateKey", "weight"]), offset=0)), ("keyDictation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0), SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["inboundSaDetails", "outboundSaDetails", "keyingModuleGenKey"]), offset=0)), ("keyNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0), SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["inboundSa", "outboundSa"]), offset=0)),)), name="IPSEC_KEY_MANAGER_CALLBACKS0", pack=False, align=None),
    "IPSEC_KEY_MANAGER_DICTATE_KEY0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0), SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["inboundSaDetails", "outboundSaDetails", "keyingModuleGenKey"]), offset=0),
    "IPSEC_KEY_MANAGER_KEY_DICTATION_CHECK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IKEEXT_TRAFFIC0", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ikeTraffic", "willDictateKey", "weight"]), offset=0),
    "IPSEC_KEY_MANAGER_NOTIFY_KEY0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0), SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["inboundSa", "outboundSa"]), offset=0),
    "IPSEC_PFS_GROUP": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"),
    "IPSEC_POLICY_FLAG": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"),
    "IPSEC_PROPOSAL0": SimStruct(OrderedDict((("lifetime", SimTypeRef("IPSEC_SA_LIFETIME0", SimStruct)), ("numSaTransforms", SimTypeInt(signed=False, label="UInt32")), ("saTransforms", SimTypePointer(SimTypeRef("IPSEC_SA_TRANSFORM0", SimStruct), offset=0)), ("pfsGroup", SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")),)), name="IPSEC_PROPOSAL0", pack=False, align=None),
    "IPSEC_SA0": SimStruct(OrderedDict((("spi", SimTypeInt(signed=False, label="UInt32")), ("saTransformType", SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE")), ("Anonymous", SimUnion({"ahInformation": SimTypePointer(SimTypeRef("IPSEC_SA_AUTH_INFORMATION0", SimStruct), offset=0), "espAuthInformation": SimTypePointer(SimTypeRef("IPSEC_SA_AUTH_INFORMATION0", SimStruct), offset=0), "espCipherInformation": SimTypePointer(SimTypeRef("IPSEC_SA_CIPHER_INFORMATION0", SimStruct), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimTypeRef("IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", SimStruct), offset=0), "espAuthFwInformation": SimTypePointer(SimTypeRef("IPSEC_SA_AUTH_INFORMATION0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IPSEC_SA0", pack=False, align=None),
    "IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0": SimStruct(OrderedDict((("saCipherInformation", SimTypeRef("IPSEC_SA_CIPHER_INFORMATION0", SimStruct)), ("saAuthInformation", SimTypeRef("IPSEC_SA_AUTH_INFORMATION0", SimStruct)),)), name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None),
    "IPSEC_SA_AUTH_INFORMATION0": SimStruct(OrderedDict((("authTransform", SimTypeRef("IPSEC_AUTH_TRANSFORM0", SimStruct)), ("authKey", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None),
    "IPSEC_SA_BUNDLE0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS")), ("lifetime", SimTypeRef("IPSEC_SA_LIFETIME0", SimStruct)), ("idleTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("ndAllowClearTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("ipsecId", SimTypePointer(SimTypeRef("IPSEC_ID0", SimStruct), offset=0)), ("napContext", SimTypeInt(signed=False, label="UInt32")), ("qmSaId", SimTypeInt(signed=False, label="UInt32")), ("numSAs", SimTypeInt(signed=False, label="UInt32")), ("saList", SimTypePointer(SimTypeRef("IPSEC_SA0", SimStruct), offset=0)), ("keyModuleState", SimTypePointer(SimTypeRef("IPSEC_KEYMODULE_STATE0", SimStruct), offset=0)), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("mmSaId", SimTypeLongLong(signed=False, label="UInt64")), ("pfsGroup", SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")),)), name="IPSEC_SA_BUNDLE0", pack=False, align=None),
    "IPSEC_SA_BUNDLE1": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS")), ("lifetime", SimTypeRef("IPSEC_SA_LIFETIME0", SimStruct)), ("idleTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("ndAllowClearTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("ipsecId", SimTypePointer(SimTypeRef("IPSEC_ID0", SimStruct), offset=0)), ("napContext", SimTypeInt(signed=False, label="UInt32")), ("qmSaId", SimTypeInt(signed=False, label="UInt32")), ("numSAs", SimTypeInt(signed=False, label="UInt32")), ("saList", SimTypePointer(SimTypeRef("IPSEC_SA0", SimStruct), offset=0)), ("keyModuleState", SimTypePointer(SimTypeRef("IPSEC_KEYMODULE_STATE0", SimStruct), offset=0)), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("mmSaId", SimTypeLongLong(signed=False, label="UInt64")), ("pfsGroup", SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")), ("saLookupContext", SimTypeBottom(label="Guid")), ("qmFilterId", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_SA_BUNDLE1", pack=False, align=None),
    "IPSEC_SA_BUNDLE_FLAGS": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"),
    "IPSEC_SA_CIPHER_INFORMATION0": SimStruct(OrderedDict((("cipherTransform", SimTypeRef("IPSEC_CIPHER_TRANSFORM0", SimStruct)), ("cipherKey", SimTypeRef("FWP_BYTE_BLOB", SimStruct)),)), name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None),
    "IPSEC_SA_CONTEXT0": SimStruct(OrderedDict((("saContextId", SimTypeLongLong(signed=False, label="UInt64")), ("inboundSa", SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS0", SimStruct), offset=0)), ("outboundSa", SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS0", SimStruct), offset=0)),)), name="IPSEC_SA_CONTEXT0", pack=False, align=None),
    "IPSEC_SA_CONTEXT1": SimStruct(OrderedDict((("saContextId", SimTypeLongLong(signed=False, label="UInt64")), ("inboundSa", SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0)), ("outboundSa", SimTypePointer(SimTypeRef("IPSEC_SA_DETAILS1", SimStruct), offset=0)),)), name="IPSEC_SA_CONTEXT1", pack=False, align=None),
    "IPSEC_SA_CONTEXT_CALLBACK0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IPSEC_SA_CONTEXT_CHANGE0", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0),
    "IPSEC_SA_CONTEXT_CHANGE0": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="IPSEC_SA_CONTEXT_EVENT_TYPE0")), ("saContextId", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_SA_CONTEXT_CHANGE0", pack=False, align=None),
    "IPSEC_SA_CONTEXT_ENUM_TEMPLATE0": SimStruct(OrderedDict((("localSubNet", SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)), ("remoteSubNet", SimTypeRef("FWP_CONDITION_VALUE0", SimStruct)),)), name="IPSEC_SA_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None),
    "IPSEC_SA_CONTEXT_EVENT_TYPE0": SimTypeInt(signed=False, label="IPSEC_SA_CONTEXT_EVENT_TYPE0"),
    "IPSEC_SA_CONTEXT_SUBSCRIPTION0": SimStruct(OrderedDict((("enumTemplate", SimTypePointer(SimTypeRef("IPSEC_SA_CONTEXT_ENUM_TEMPLATE0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("sessionKey", SimTypeBottom(label="Guid")),)), name="IPSEC_SA_CONTEXT_SUBSCRIPTION0", pack=False, align=None),
    "IPSEC_SA_DETAILS0": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("saDirection", SimTypeInt(signed=False, label="FWP_DIRECTION")), ("traffic", SimTypeRef("IPSEC_TRAFFIC0", SimStruct)), ("saBundle", SimTypeRef("IPSEC_SA_BUNDLE0", SimStruct)), ("Anonymous", SimUnion({"udpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("transportFilter", SimTypePointer(SimTypeRef("FWPM_FILTER0", SimStruct), offset=0)),)), name="IPSEC_SA_DETAILS0", pack=False, align=None),
    "IPSEC_SA_DETAILS1": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("saDirection", SimTypeInt(signed=False, label="FWP_DIRECTION")), ("traffic", SimTypeRef("IPSEC_TRAFFIC1", SimStruct)), ("saBundle", SimTypeRef("IPSEC_SA_BUNDLE1", SimStruct)), ("Anonymous", SimUnion({"udpEncapsulation": SimTypePointer(SimTypeRef("IPSEC_V4_UDP_ENCAPSULATION0", SimStruct), offset=0)}, name="<anon>", label="None")), ("transportFilter", SimTypePointer(SimTypeRef("FWPM_FILTER0", SimStruct), offset=0)), ("virtualIfTunnelInfo", SimTypeRef("IPSEC_VIRTUAL_IF_TUNNEL_INFO0", SimStruct)),)), name="IPSEC_SA_DETAILS1", pack=False, align=None),
    "IPSEC_SA_ENUM_TEMPLATE0": SimStruct(OrderedDict((("saDirection", SimTypeInt(signed=False, label="FWP_DIRECTION")),)), name="IPSEC_SA_ENUM_TEMPLATE0", pack=False, align=None),
    "IPSEC_SA_IDLE_TIMEOUT0": SimStruct(OrderedDict((("idleTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("idleTimeoutSecondsFailOver", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None),
    "IPSEC_SA_LIFETIME0": SimStruct(OrderedDict((("lifetimeSeconds", SimTypeInt(signed=False, label="UInt32")), ("lifetimeKilobytes", SimTypeInt(signed=False, label="UInt32")), ("lifetimePackets", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_SA_LIFETIME0", pack=False, align=None),
    "IPSEC_SA_TRANSFORM0": SimStruct(OrderedDict((("ipsecTransformType", SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE")), ("Anonymous", SimUnion({"ahTransform": SimTypePointer(SimTypeRef("IPSEC_AUTH_TRANSFORM0", SimStruct), offset=0), "espAuthTransform": SimTypePointer(SimTypeRef("IPSEC_AUTH_TRANSFORM0", SimStruct), offset=0), "espCipherTransform": SimTypePointer(SimTypeRef("IPSEC_CIPHER_TRANSFORM0", SimStruct), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimTypeRef("IPSEC_AUTH_AND_CIPHER_TRANSFORM0", SimStruct), offset=0), "espAuthFwTransform": SimTypePointer(SimTypeRef("IPSEC_AUTH_TRANSFORM0", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="IPSEC_SA_TRANSFORM0", pack=False, align=None),
    "IPSEC_STATISTICS0": SimStruct(OrderedDict((("aggregateSaStatistics", SimTypeRef("IPSEC_AGGREGATE_SA_STATISTICS0", SimStruct)), ("espDropPacketStatistics", SimTypeRef("IPSEC_ESP_DROP_PACKET_STATISTICS0", SimStruct)), ("ahDropPacketStatistics", SimTypeRef("IPSEC_AH_DROP_PACKET_STATISTICS0", SimStruct)), ("aggregateDropPacketStatistics", SimTypeRef("IPSEC_AGGREGATE_DROP_PACKET_STATISTICS0", SimStruct)), ("inboundTrafficStatistics", SimTypeRef("IPSEC_TRAFFIC_STATISTICS0", SimStruct)), ("outboundTrafficStatistics", SimTypeRef("IPSEC_TRAFFIC_STATISTICS0", SimStruct)),)), name="IPSEC_STATISTICS0", pack=False, align=None),
    "IPSEC_STATISTICS1": SimStruct(OrderedDict((("aggregateSaStatistics", SimTypeRef("IPSEC_AGGREGATE_SA_STATISTICS0", SimStruct)), ("espDropPacketStatistics", SimTypeRef("IPSEC_ESP_DROP_PACKET_STATISTICS0", SimStruct)), ("ahDropPacketStatistics", SimTypeRef("IPSEC_AH_DROP_PACKET_STATISTICS0", SimStruct)), ("aggregateDropPacketStatistics", SimTypeRef("IPSEC_AGGREGATE_DROP_PACKET_STATISTICS1", SimStruct)), ("inboundTrafficStatistics", SimTypeRef("IPSEC_TRAFFIC_STATISTICS1", SimStruct)), ("outboundTrafficStatistics", SimTypeRef("IPSEC_TRAFFIC_STATISTICS1", SimStruct)),)), name="IPSEC_STATISTICS1", pack=False, align=None),
    "IPSEC_TOKEN0": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE")), ("principal", SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL")), ("mode", SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE")), ("token", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_TOKEN0", pack=False, align=None),
    "IPSEC_TOKEN_MODE": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"),
    "IPSEC_TOKEN_PRINCIPAL": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"),
    "IPSEC_TOKEN_TYPE": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"),
    "IPSEC_TRAFFIC0": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("trafficType", SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")), ("Anonymous3", SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("remotePort", SimTypeShort(signed=False, label="UInt16")),)), name="IPSEC_TRAFFIC0", pack=False, align=None),
    "IPSEC_TRAFFIC1": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("trafficType", SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")), ("Anonymous3", SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("remotePort", SimTypeShort(signed=False, label="UInt16")), ("localPort", SimTypeShort(signed=False, label="UInt16")), ("ipProtocol", SimTypeChar(label="Byte")), ("localIfLuid", SimTypeLongLong(signed=False, label="UInt64")), ("realIfProfileId", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_TRAFFIC1", pack=False, align=None),
    "IPSEC_TRAFFIC_SELECTOR0": SimStruct(OrderedDict((("protocolId", SimTypeChar(label="Byte")), ("portStart", SimTypeShort(signed=False, label="UInt16")), ("portEnd", SimTypeShort(signed=False, label="UInt16")), ("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")),)), name="IPSEC_TRAFFIC_SELECTOR0", pack=False, align=None),
    "IPSEC_TRAFFIC_SELECTOR_POLICY0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("numLocalTrafficSelectors", SimTypeInt(signed=False, label="UInt32")), ("localTrafficSelectors", SimTypePointer(SimTypeRef("IPSEC_TRAFFIC_SELECTOR0", SimStruct), offset=0)), ("numRemoteTrafficSelectors", SimTypeInt(signed=False, label="UInt32")), ("remoteTrafficSelectors", SimTypePointer(SimTypeRef("IPSEC_TRAFFIC_SELECTOR0", SimStruct), offset=0)),)), name="IPSEC_TRAFFIC_SELECTOR_POLICY0", pack=False, align=None),
    "IPSEC_TRAFFIC_STATISTICS0": SimStruct(OrderedDict((("encryptedByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("authenticatedAHByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("authenticatedESPByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("transportByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("tunnelByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("offloadByteCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_TRAFFIC_STATISTICS0", pack=False, align=None),
    "IPSEC_TRAFFIC_STATISTICS1": SimStruct(OrderedDict((("encryptedByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("authenticatedAHByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("authenticatedESPByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("transportByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("tunnelByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("offloadByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("totalSuccessfulPackets", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_TRAFFIC_STATISTICS1", pack=False, align=None),
    "IPSEC_TRAFFIC_TYPE": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"),
    "IPSEC_TRANSFORM_TYPE": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"),
    "IPSEC_TRANSPORT_POLICY0": SimStruct(OrderedDict((("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG")), ("ndAllowClearTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY0", SimStruct), offset=0)),)), name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None),
    "IPSEC_TRANSPORT_POLICY1": SimStruct(OrderedDict((("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG")), ("ndAllowClearTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY1", SimStruct), offset=0)),)), name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None),
    "IPSEC_TRANSPORT_POLICY2": SimStruct(OrderedDict((("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("flags", SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG")), ("ndAllowClearTimeoutSeconds", SimTypeInt(signed=False, label="UInt32")), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY2", SimStruct), offset=0)),)), name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None),
    "IPSEC_TUNNEL_ENDPOINT0": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous", SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")),)), name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None),
    "IPSEC_TUNNEL_ENDPOINTS0": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")),)), name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None),
    "IPSEC_TUNNEL_ENDPOINTS1": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("localIfLuid", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None),
    "IPSEC_TUNNEL_ENDPOINTS2": SimStruct(OrderedDict((("ipVersion", SimTypeInt(signed=False, label="FWP_IP_VERSION")), ("Anonymous1", SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("localIfLuid", SimTypeLongLong(signed=False, label="UInt64")), ("remoteFqdn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("numAddresses", SimTypeInt(signed=False, label="UInt32")), ("remoteAddresses", SimTypePointer(SimTypeRef("IPSEC_TUNNEL_ENDPOINT0", SimStruct), offset=0)),)), name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None),
    "IPSEC_TUNNEL_POLICY0": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG")), ("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("tunnelEndpoints", SimTypeRef("IPSEC_TUNNEL_ENDPOINTS0", SimStruct)), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY0", SimStruct), offset=0)),)), name="IPSEC_TUNNEL_POLICY0", pack=False, align=None),
    "IPSEC_TUNNEL_POLICY1": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG")), ("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("tunnelEndpoints", SimTypeRef("IPSEC_TUNNEL_ENDPOINTS1", SimStruct)), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY1", SimStruct), offset=0)),)), name="IPSEC_TUNNEL_POLICY1", pack=False, align=None),
    "IPSEC_TUNNEL_POLICY2": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG")), ("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("tunnelEndpoints", SimTypeRef("IPSEC_TUNNEL_ENDPOINTS2", SimStruct)), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY2", SimStruct), offset=0)), ("fwdPathSaLifetime", SimTypeInt(signed=False, label="UInt32")),)), name="IPSEC_TUNNEL_POLICY2", pack=False, align=None),
    "IPSEC_TUNNEL_POLICY3": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("numIpsecProposals", SimTypeInt(signed=False, label="UInt32")), ("ipsecProposals", SimTypePointer(SimTypeRef("IPSEC_PROPOSAL0", SimStruct), offset=0)), ("tunnelEndpoints", SimTypeRef("IPSEC_TUNNEL_ENDPOINTS2", SimStruct)), ("saIdleTimeout", SimTypeRef("IPSEC_SA_IDLE_TIMEOUT0", SimStruct)), ("emPolicy", SimTypePointer(SimTypeRef("IKEEXT_EM_POLICY2", SimStruct), offset=0)), ("fwdPathSaLifetime", SimTypeInt(signed=False, label="UInt32")), ("compartmentId", SimTypeInt(signed=False, label="UInt32")), ("numTrafficSelectorPolicy", SimTypeInt(signed=False, label="UInt32")), ("trafficSelectorPolicies", SimTypePointer(SimTypeRef("IPSEC_TRAFFIC_SELECTOR_POLICY0", SimStruct), offset=0)),)), name="IPSEC_TUNNEL_POLICY3", pack=False, align=None),
    "IPSEC_V4_UDP_ENCAPSULATION0": SimStruct(OrderedDict((("localUdpEncapPort", SimTypeShort(signed=False, label="UInt16")), ("remoteUdpEncapPort", SimTypeShort(signed=False, label="UInt16")),)), name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None),
    "IPSEC_VIRTUAL_IF_TUNNEL_INFO0": SimStruct(OrderedDict((("virtualIfTunnelId", SimTypeLongLong(signed=False, label="UInt64")), ("trafficSelectorId", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None),
    "IPSFactoryBuffer": SimTypeBottom(label="IPSFactoryBuffer"),
    "IPSITables": SimTypeBottom(label="IPSITables"),
    "IPTFilterLicenseRenewal": SimTypeBottom(label="IPTFilterLicenseRenewal"),
    "IPTLS_METADATA": SimStruct(OrderedDict((("SequenceNumber", SimTypeLongLong(signed=False, label="UInt64")),)), name="IPTLS_METADATA", pack=False, align=None),
    "IPV4_HEADER": SimStruct(OrderedDict((("Anonymous1", SimUnion({"VersionAndHeaderLength": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"TypeOfServiceAndEcnField": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("TotalLength", SimTypeShort(signed=False, label="UInt16")), ("Identification", SimTypeShort(signed=False, label="UInt16")), ("Anonymous3", SimUnion({"FlagsAndOffset": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("TimeToLive", SimTypeChar(label="Byte")), ("Protocol", SimTypeChar(label="Byte")), ("HeaderChecksum", SimTypeShort(signed=False, label="UInt16")), ("SourceAddress", SimTypeRef("IN_ADDR", SimStruct)), ("DestinationAddress", SimTypeRef("IN_ADDR", SimStruct)),)), name="IPV4_HEADER", pack=False, align=None),
    "IPV4_OPTION_HEADER": SimStruct(OrderedDict((("Anonymous", SimUnion({"OptionType": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("OptionLength", SimTypeChar(label="Byte")),)), name="IPV4_OPTION_HEADER", pack=False, align=None),
    "IPV4_OPTION_TYPE": SimTypeInt(signed=False, label="IPV4_OPTION_TYPE"),
    "IPV4_ROUTING_HEADER": SimStruct(OrderedDict((("OptionHeader", SimTypeRef("IPV4_OPTION_HEADER", SimStruct)), ("Pointer", SimTypeChar(label="Byte")),)), name="IPV4_ROUTING_HEADER", pack=False, align=None),
    "IPV4_TIMESTAMP_OPTION": SimStruct(OrderedDict((("OptionHeader", SimTypeRef("IPV4_OPTION_HEADER", SimStruct)), ("Pointer", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"FlagsOverflow": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="IPV4_TIMESTAMP_OPTION", pack=False, align=None),
    "IPV6_ADDRESS_EX": SimStruct(OrderedDict((("sin6_port", SimTypeShort(signed=False, label="UInt16")), ("sin6_flowinfo", SimTypeInt(signed=False, label="UInt32")), ("sin6_addr", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)), ("sin6_scope_id", SimTypeInt(signed=False, label="UInt32")),)), name="IPV6_ADDRESS_EX", pack=False, align=None),
    "IPV6_EXTENSION_HEADER": SimStruct(OrderedDict((("NextHeader", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")),)), name="IPV6_EXTENSION_HEADER", pack=False, align=None),
    "IPV6_FRAGMENT_HEADER": SimStruct(OrderedDict((("NextHeader", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "OffsetAndFlags": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Id", SimTypeInt(signed=False, label="UInt32")),)), name="IPV6_FRAGMENT_HEADER", pack=False, align=None),
    "IPV6_HEADER": SimStruct(OrderedDict((("Anonymous", SimUnion({"VersionClassFlow": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("PayloadLength", SimTypeShort(signed=False, label="UInt16")), ("NextHeader", SimTypeChar(label="Byte")), ("HopLimit", SimTypeChar(label="Byte")), ("SourceAddress", SimTypeRef("IN6_ADDR", SimStruct)), ("DestinationAddress", SimTypeRef("IN6_ADDR", SimStruct)),)), name="IPV6_HEADER", pack=False, align=None),
    "IPV6_MREQ": SimStruct(OrderedDict((("ipv6mr_multiaddr", SimTypeRef("IN6_ADDR", SimStruct)), ("ipv6mr_interface", SimTypeInt(signed=False, label="UInt32")),)), name="IPV6_MREQ", pack=False, align=None),
    "IPV6_NEIGHBOR_ADVERTISEMENT_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "IPV6_OPTION_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("DataLength", SimTypeChar(label="Byte")),)), name="IPV6_OPTION_HEADER", pack=False, align=None),
    "IPV6_OPTION_JUMBOGRAM": SimStruct(OrderedDict((("Header", SimTypeRef("IPV6_OPTION_HEADER", SimStruct)), ("JumbogramLength", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="IPV6_OPTION_JUMBOGRAM", pack=False, align=None),
    "IPV6_OPTION_ROUTER_ALERT": SimStruct(OrderedDict((("Header", SimTypeRef("IPV6_OPTION_HEADER", SimStruct)), ("Value", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="IPV6_OPTION_ROUTER_ALERT", pack=False, align=None),
    "IPV6_OPTION_TYPE": SimTypeInt(signed=False, label="IPV6_OPTION_TYPE"),
    "IPV6_ROUTER_ADVERTISEMENT_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "IPV6_ROUTING_HEADER": SimStruct(OrderedDict((("NextHeader", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("RoutingType", SimTypeChar(label="Byte")), ("SegmentsLeft", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="IPV6_ROUTING_HEADER", pack=False, align=None),
    "IPX_ADDRESS_DATA": SimStruct(OrderedDict((("adapternum", SimTypeInt(signed=True, label="Int32")), ("netnum", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("nodenum", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("wan", SimTypeChar(label="Byte")), ("status", SimTypeChar(label="Byte")), ("maxpkt", SimTypeInt(signed=True, label="Int32")), ("linkspeed", SimTypeInt(signed=False, label="UInt32")),)), name="IPX_ADDRESS_DATA", pack=False, align=None),
    "IPX_NETNUM_DATA": SimStruct(OrderedDict((("netnum", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("hopcount", SimTypeShort(signed=False, label="UInt16")), ("netdelay", SimTypeShort(signed=False, label="UInt16")), ("cardnum", SimTypeInt(signed=True, label="Int32")), ("router", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="IPX_NETNUM_DATA", pack=False, align=None),
    "IPX_PATTERN": SimStruct(OrderedDict((("Src", SimTypeRef("_Src_e__Struct", SimStruct)), ("Dest", SimTypeRef("_Src_e__Struct", SimStruct)),)), name="IPX_PATTERN", pack=False, align=None),
    "IPX_SPXCONNSTATUS_DATA": SimStruct(OrderedDict((("ConnectionState", SimTypeChar(label="Byte")), ("WatchDogActive", SimTypeChar(label="Byte")), ("LocalConnectionId", SimTypeShort(signed=False, label="UInt16")), ("RemoteConnectionId", SimTypeShort(signed=False, label="UInt16")), ("LocalSequenceNumber", SimTypeShort(signed=False, label="UInt16")), ("LocalAckNumber", SimTypeShort(signed=False, label="UInt16")), ("LocalAllocNumber", SimTypeShort(signed=False, label="UInt16")), ("RemoteAckNumber", SimTypeShort(signed=False, label="UInt16")), ("RemoteAllocNumber", SimTypeShort(signed=False, label="UInt16")), ("LocalSocket", SimTypeShort(signed=False, label="UInt16")), ("ImmediateAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("RemoteNetwork", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("RemoteNode", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("RemoteSocket", SimTypeShort(signed=False, label="UInt16")), ("RetransmissionCount", SimTypeShort(signed=False, label="UInt16")), ("EstimatedRoundTripDelay", SimTypeShort(signed=False, label="UInt16")), ("RetransmittedPackets", SimTypeShort(signed=False, label="UInt16")), ("SuppressedPacket", SimTypeShort(signed=False, label="UInt16")),)), name="IPX_SPXCONNSTATUS_DATA", pack=False, align=None),
    "IP_ADAPTER_ADDRESSES_LH": SimStruct(OrderedDict((("Anonymous1", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("IfIndex", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_ADDRESSES_LH", SimStruct), offset=0)), ("AdapterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FirstUnicastAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_UNICAST_ADDRESS_LH", SimStruct), offset=0)), ("FirstAnycastAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_ANYCAST_ADDRESS_XP", SimStruct), offset=0)), ("FirstMulticastAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_MULTICAST_ADDRESS_XP", SimStruct), offset=0)), ("FirstDnsServerAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_DNS_SERVER_ADDRESS_XP", SimStruct), offset=0)), ("DnsSuffix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PhysicalAddress", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("PhysicalAddressLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Mtu", SimTypeInt(signed=False, label="UInt32")), ("IfType", SimTypeInt(signed=False, label="UInt32")), ("OperStatus", SimTypeInt(signed=False, label="IF_OPER_STATUS")), ("Ipv6IfIndex", SimTypeInt(signed=False, label="UInt32")), ("ZoneIndices", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)), ("FirstPrefix", SimTypePointer(SimTypeRef("IP_ADAPTER_PREFIX_XP", SimStruct), offset=0)), ("TransmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("FirstWinsServerAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_WINS_SERVER_ADDRESS_LH", SimStruct), offset=0)), ("FirstGatewayAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_GATEWAY_ADDRESS_LH", SimStruct), offset=0)), ("Ipv4Metric", SimTypeInt(signed=False, label="UInt32")), ("Ipv6Metric", SimTypeInt(signed=False, label="UInt32")), ("Luid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("Dhcpv4Server", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("CompartmentId", SimTypeInt(signed=False, label="UInt32")), ("NetworkGuid", SimTypeBottom(label="Guid")), ("ConnectionType", SimTypeInt(signed=False, label="NET_IF_CONNECTION_TYPE")), ("TunnelType", SimTypeInt(signed=False, label="TUNNEL_TYPE")), ("Dhcpv6Server", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("Dhcpv6ClientDuid", SimTypeArray(SimTypeChar(label="Byte"), 130)), ("Dhcpv6ClientDuidLength", SimTypeInt(signed=False, label="UInt32")), ("Dhcpv6Iaid", SimTypeInt(signed=False, label="UInt32")), ("FirstDnsSuffix", SimTypePointer(SimTypeRef("IP_ADAPTER_DNS_SUFFIX", SimStruct), offset=0)),)), name="IP_ADAPTER_ADDRESSES_LH", pack=False, align=None),
    "IP_ADAPTER_ADDRESSES_XP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("IfIndex", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_ADDRESSES_XP", SimStruct), offset=0)), ("AdapterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FirstUnicastAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_UNICAST_ADDRESS_XP", SimStruct), offset=0)), ("FirstAnycastAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_ANYCAST_ADDRESS_XP", SimStruct), offset=0)), ("FirstMulticastAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_MULTICAST_ADDRESS_XP", SimStruct), offset=0)), ("FirstDnsServerAddress", SimTypePointer(SimTypeRef("IP_ADAPTER_DNS_SERVER_ADDRESS_XP", SimStruct), offset=0)), ("DnsSuffix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PhysicalAddress", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("PhysicalAddressLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Mtu", SimTypeInt(signed=False, label="UInt32")), ("IfType", SimTypeInt(signed=False, label="UInt32")), ("OperStatus", SimTypeInt(signed=False, label="IF_OPER_STATUS")), ("Ipv6IfIndex", SimTypeInt(signed=False, label="UInt32")), ("ZoneIndices", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)), ("FirstPrefix", SimTypePointer(SimTypeRef("IP_ADAPTER_PREFIX_XP", SimStruct), offset=0)),)), name="IP_ADAPTER_ADDRESSES_XP", pack=False, align=None),
    "IP_ADAPTER_ANYCAST_ADDRESS_XP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_ANYCAST_ADDRESS_XP", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)),)), name="IP_ADAPTER_ANYCAST_ADDRESS_XP", pack=False, align=None),
    "IP_ADAPTER_DNS_SERVER_ADDRESS_XP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_DNS_SERVER_ADDRESS_XP", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)),)), name="IP_ADAPTER_DNS_SERVER_ADDRESS_XP", pack=False, align=None),
    "IP_ADAPTER_DNS_SUFFIX": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_DNS_SUFFIX", SimStruct), offset=0)), ("String", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="IP_ADAPTER_DNS_SUFFIX", pack=False, align=None),
    "IP_ADAPTER_GATEWAY_ADDRESS_LH": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_GATEWAY_ADDRESS_LH", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)),)), name="IP_ADAPTER_GATEWAY_ADDRESS_LH", pack=False, align=None),
    "IP_ADAPTER_INDEX_MAP": SimStruct(OrderedDict((("Index", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="IP_ADAPTER_INDEX_MAP", pack=False, align=None),
    "IP_ADAPTER_INFO": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_INFO", SimStruct), offset=0)), ("ComboIndex", SimTypeInt(signed=False, label="UInt32")), ("AdapterName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Description", SimTypeArray(SimTypeChar(label="SByte"), 132)), ("AddressLength", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("DhcpEnabled", SimTypeInt(signed=False, label="UInt32")), ("CurrentIpAddress", SimTypePointer(SimTypeRef("IP_ADDR_STRING", SimStruct), offset=0)), ("IpAddressList", SimTypeRef("IP_ADDR_STRING", SimStruct)), ("GatewayList", SimTypeRef("IP_ADDR_STRING", SimStruct)), ("DhcpServer", SimTypeRef("IP_ADDR_STRING", SimStruct)), ("HaveWins", SimTypeInt(signed=True, label="Int32")), ("PrimaryWinsServer", SimTypeRef("IP_ADDR_STRING", SimStruct)), ("SecondaryWinsServer", SimTypeRef("IP_ADDR_STRING", SimStruct)), ("LeaseObtained", SimTypeLongLong(signed=True, label="Int64")), ("LeaseExpires", SimTypeLongLong(signed=True, label="Int64")),)), name="IP_ADAPTER_INFO", pack=False, align=None),
    "IP_ADAPTER_MULTICAST_ADDRESS_XP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_MULTICAST_ADDRESS_XP", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)),)), name="IP_ADAPTER_MULTICAST_ADDRESS_XP", pack=False, align=None),
    "IP_ADAPTER_ORDER_MAP": SimStruct(OrderedDict((("NumAdapters", SimTypeInt(signed=False, label="UInt32")), ("AdapterOrder", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IP_ADAPTER_ORDER_MAP", pack=False, align=None),
    "IP_ADAPTER_PREFIX_XP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_PREFIX_XP", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("PrefixLength", SimTypeInt(signed=False, label="UInt32")),)), name="IP_ADAPTER_PREFIX_XP", pack=False, align=None),
    "IP_ADAPTER_UNICAST_ADDRESS_LH": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_UNICAST_ADDRESS_LH", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("PrefixOrigin", SimTypeInt(signed=False, label="NL_PREFIX_ORIGIN")), ("SuffixOrigin", SimTypeInt(signed=False, label="NL_SUFFIX_ORIGIN")), ("DadState", SimTypeInt(signed=False, label="NL_DAD_STATE")), ("ValidLifetime", SimTypeInt(signed=False, label="UInt32")), ("PreferredLifetime", SimTypeInt(signed=False, label="UInt32")), ("LeaseLifetime", SimTypeInt(signed=False, label="UInt32")), ("OnLinkPrefixLength", SimTypeChar(label="Byte")),)), name="IP_ADAPTER_UNICAST_ADDRESS_LH", pack=False, align=None),
    "IP_ADAPTER_UNICAST_ADDRESS_XP": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_UNICAST_ADDRESS_XP", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("PrefixOrigin", SimTypeInt(signed=False, label="NL_PREFIX_ORIGIN")), ("SuffixOrigin", SimTypeInt(signed=False, label="NL_SUFFIX_ORIGIN")), ("DadState", SimTypeInt(signed=False, label="NL_DAD_STATE")), ("ValidLifetime", SimTypeInt(signed=False, label="UInt32")), ("PreferredLifetime", SimTypeInt(signed=False, label="UInt32")), ("LeaseLifetime", SimTypeInt(signed=False, label="UInt32")),)), name="IP_ADAPTER_UNICAST_ADDRESS_XP", pack=False, align=None),
    "IP_ADAPTER_WINS_SERVER_ADDRESS_LH": SimStruct(OrderedDict((("Anonymous", SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Next", SimTypePointer(SimTypeRef("IP_ADAPTER_WINS_SERVER_ADDRESS_LH", SimStruct), offset=0)), ("Address", SimTypeRef("SOCKET_ADDRESS", SimStruct)),)), name="IP_ADAPTER_WINS_SERVER_ADDRESS_LH", pack=False, align=None),
    "IP_ADDRESS_PREFIX": SimStruct(OrderedDict((("Prefix", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("PrefixLength", SimTypeChar(label="Byte")),)), name="IP_ADDRESS_PREFIX", pack=False, align=None),
    "IP_ADDRESS_STRING": SimStruct(OrderedDict((("String", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="IP_ADDRESS_STRING", pack=False, align=None),
    "IP_ADDR_STRING": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("IP_ADDR_STRING", SimStruct), offset=0)), ("IpAddress", SimTypeRef("IP_ADDRESS_STRING", SimStruct)), ("IpMask", SimTypeRef("IP_ADDRESS_STRING", SimStruct)), ("Context", SimTypeInt(signed=False, label="UInt32")),)), name="IP_ADDR_STRING", pack=False, align=None),
    "IP_INTERFACE_INFO": SimStruct(OrderedDict((("NumAdapters", SimTypeInt(signed=True, label="Int32")), ("Adapter", SimTypePointer(SimTypeRef("IP_ADAPTER_INDEX_MAP", SimStruct), offset=0)),)), name="IP_INTERFACE_INFO", pack=False, align=None),
    "IP_INTERFACE_NAME_INFO_W2KSP1": SimStruct(OrderedDict((("Index", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeInt(signed=False, label="UInt32")), ("ConnectionType", SimTypeChar(label="Byte")), ("AccessType", SimTypeChar(label="Byte")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("InterfaceGuid", SimTypeBottom(label="Guid")),)), name="IP_INTERFACE_NAME_INFO_W2KSP1", pack=False, align=None),
    "IP_MCAST_COUNTER_INFO": SimStruct(OrderedDict((("InMcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("OutMcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("InMcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("OutMcastPkts", SimTypeLongLong(signed=False, label="UInt64")),)), name="IP_MCAST_COUNTER_INFO", pack=False, align=None),
    "IP_MREQ": SimStruct(OrderedDict((("imr_multiaddr", SimTypeRef("IN_ADDR", SimStruct)), ("imr_interface", SimTypeRef("IN_ADDR", SimStruct)),)), name="IP_MREQ", pack=False, align=None),
    "IP_MREQ_SOURCE": SimStruct(OrderedDict((("imr_multiaddr", SimTypeRef("IN_ADDR", SimStruct)), ("imr_sourceaddr", SimTypeRef("IN_ADDR", SimStruct)), ("imr_interface", SimTypeRef("IN_ADDR", SimStruct)),)), name="IP_MREQ_SOURCE", pack=False, align=None),
    "IP_MSFILTER": SimStruct(OrderedDict((("imsf_multiaddr", SimTypeRef("IN_ADDR", SimStruct)), ("imsf_interface", SimTypeRef("IN_ADDR", SimStruct)), ("imsf_fmode", SimTypeInt(signed=False, label="MULTICAST_MODE_TYPE")), ("imsf_numsrc", SimTypeInt(signed=False, label="UInt32")), ("imsf_slist", SimTypePointer(SimTypeRef("IN_ADDR", SimStruct), offset=0)),)), name="IP_MSFILTER", pack=False, align=None),
    "IP_OPTION_INFORMATION": SimStruct(OrderedDict((("Ttl", SimTypeChar(label="Byte")), ("Tos", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("OptionsSize", SimTypeChar(label="Byte")), ("OptionsData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="IP_OPTION_INFORMATION", pack=False, align=None),
    "IP_OPTION_INFORMATION32": SimStruct(OrderedDict((("Ttl", SimTypeChar(label="Byte")), ("Tos", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("OptionsSize", SimTypeChar(label="Byte")), ("OptionsData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="IP_OPTION_INFORMATION32", pack=False, align=None),
    "IP_OPTION_TIMESTAMP_FLAGS": SimTypeInt(signed=False, label="IP_OPTION_TIMESTAMP_FLAGS"),
    "IP_PATTERN": SimStruct(OrderedDict((("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("SrcAddr", SimTypeInt(signed=False, label="UInt32")), ("DstAddr", SimTypeInt(signed=False, label="UInt32")), ("S_un", SimUnion({"S_un_ports": SimTypeRef("_S_un_ports_e__Struct", SimStruct), "S_un_icmp": SimTypeRef("_S_un_icmp_e__Struct", SimStruct), "S_Spi": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ProtocolId", SimTypeChar(label="Byte")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="IP_PATTERN", pack=False, align=None),
    "IP_PER_ADAPTER_INFO_W2KSP1": SimStruct(OrderedDict((("AutoconfigEnabled", SimTypeInt(signed=False, label="UInt32")), ("AutoconfigActive", SimTypeInt(signed=False, label="UInt32")), ("CurrentDnsServer", SimTypePointer(SimTypeRef("IP_ADDR_STRING", SimStruct), offset=0)), ("DnsServerList", SimTypeRef("IP_ADDR_STRING", SimStruct)),)), name="IP_PER_ADAPTER_INFO_W2KSP1", pack=False, align=None),
    "IP_UNIDIRECTIONAL_ADAPTER_ADDRESS": SimStruct(OrderedDict((("NumAdapters", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="IP_UNIDIRECTIONAL_ADAPTER_ADDRESS", pack=False, align=None),
    "IPackageDebugSettings": SimTypeBottom(label="IPackageDebugSettings"),
    "IPackageDebugSettings2": SimTypeBottom(label="IPackageDebugSettings2"),
    "IPackageExecutionStateChangeNotification": SimTypeBottom(label="IPackageExecutionStateChangeNotification"),
    "IPackageUtil": SimTypeBottom(label="IPackageUtil"),
    "IParentAndItem": SimTypeBottom(label="IParentAndItem"),
    "IParentRowset": SimTypeBottom(label="IParentRowset"),
    "IParseAndCreateItem": SimTypeBottom(label="IParseAndCreateItem"),
    "IParseDisplayName": SimTypeBottom(label="IParseDisplayName"),
    "IPart": SimTypeBottom(label="IPart"),
    "IPartBase": SimTypeBottom(label="IPartBase"),
    "IPartColorProfile": SimTypeBottom(label="IPartColorProfile"),
    "IPartDiscardControl": SimTypeBottom(label="IPartDiscardControl"),
    "IPartFont": SimTypeBottom(label="IPartFont"),
    "IPartFont2": SimTypeBottom(label="IPartFont2"),
    "IPartImage": SimTypeBottom(label="IPartImage"),
    "IPartPrintTicket": SimTypeBottom(label="IPartPrintTicket"),
    "IPartResourceDictionary": SimTypeBottom(label="IPartResourceDictionary"),
    "IPartThumbnail": SimTypeBottom(label="IPartThumbnail"),
    "IPartsList": SimTypeBottom(label="IPartsList"),
    "IPeerFactory": SimTypeBottom(label="IPeerFactory"),
    "IPenInputPanel": SimTypeBottom(label="IPenInputPanel"),
    "IPerChannelDbLevel": SimTypeBottom(label="IPerChannelDbLevel"),
    "IPerPropertyBrowsing": SimTypeBottom(label="IPerPropertyBrowsing"),
    "IPerPropertyBrowsing2": SimTypeBottom(label="IPerPropertyBrowsing2"),
    "IPerformanceCounterDataCollector": SimTypeBottom(label="IPerformanceCounterDataCollector"),
    "IPersist": SimTypeBottom(label="IPersist"),
    "IPersistFile": SimTypeBottom(label="IPersistFile"),
    "IPersistFolder": SimTypeBottom(label="IPersistFolder"),
    "IPersistFolder2": SimTypeBottom(label="IPersistFolder2"),
    "IPersistFolder3": SimTypeBottom(label="IPersistFolder3"),
    "IPersistHistory": SimTypeBottom(label="IPersistHistory"),
    "IPersistIDList": SimTypeBottom(label="IPersistIDList"),
    "IPersistMediaPropertyBag": SimTypeBottom(label="IPersistMediaPropertyBag"),
    "IPersistMemory": SimTypeBottom(label="IPersistMemory"),
    "IPersistMoniker": SimTypeBottom(label="IPersistMoniker"),
    "IPersistPropertyBag": SimTypeBottom(label="IPersistPropertyBag"),
    "IPersistPropertyBag2": SimTypeBottom(label="IPersistPropertyBag2"),
    "IPersistQuery": SimTypeBottom(label="IPersistQuery"),
    "IPersistSerializedPropStorage": SimTypeBottom(label="IPersistSerializedPropStorage"),
    "IPersistSerializedPropStorage2": SimTypeBottom(label="IPersistSerializedPropStorage2"),
    "IPersistStorage": SimTypeBottom(label="IPersistStorage"),
    "IPersistStream": SimTypeBottom(label="IPersistStream"),
    "IPersistStreamInit": SimTypeBottom(label="IPersistStreamInit"),
    "IPersistTuneXml": SimTypeBottom(label="IPersistTuneXml"),
    "IPersistTuneXmlUtility": SimTypeBottom(label="IPersistTuneXmlUtility"),
    "IPersistTuneXmlUtility2": SimTypeBottom(label="IPersistTuneXmlUtility2"),
    "IPhotoAcquire": SimTypeBottom(label="IPhotoAcquire"),
    "IPhotoAcquireDeviceSelectionDialog": SimTypeBottom(label="IPhotoAcquireDeviceSelectionDialog"),
    "IPhotoAcquireItem": SimTypeBottom(label="IPhotoAcquireItem"),
    "IPhotoAcquireOptionsDialog": SimTypeBottom(label="IPhotoAcquireOptionsDialog"),
    "IPhotoAcquirePlugin": SimTypeBottom(label="IPhotoAcquirePlugin"),
    "IPhotoAcquireProgressCB": SimTypeBottom(label="IPhotoAcquireProgressCB"),
    "IPhotoAcquireSettings": SimTypeBottom(label="IPhotoAcquireSettings"),
    "IPhotoAcquireSource": SimTypeBottom(label="IPhotoAcquireSource"),
    "IPhotoProgressActionCB": SimTypeBottom(label="IPhotoProgressActionCB"),
    "IPhotoProgressDialog": SimTypeBottom(label="IPhotoProgressDialog"),
    "IPhraseSink": SimTypeBottom(label="IPhraseSink"),
    "IPicture": SimTypeBottom(label="IPicture"),
    "IPicture2": SimTypeBottom(label="IPicture2"),
    "IPictureDisp": SimTypeBottom(label="IPictureDisp"),
    "IPin": SimTypeBottom(label="IPin"),
    "IPinConnection": SimTypeBottom(label="IPinConnection"),
    "IPinFlowControl": SimTypeBottom(label="IPinFlowControl"),
    "IPinInfo": SimTypeBottom(label="IPinInfo"),
    "IPipeByte": SimTypeBottom(label="IPipeByte"),
    "IPipeDouble": SimTypeBottom(label="IPipeDouble"),
    "IPipeLong": SimTypeBottom(label="IPipeLong"),
    "IPlayToControl": SimTypeBottom(label="IPlayToControl"),
    "IPlayToControlWithCapabilities": SimTypeBottom(label="IPlayToControlWithCapabilities"),
    "IPlayToSourceClassFactory": SimTypeBottom(label="IPlayToSourceClassFactory"),
    "IPlaybackControl": SimTypeBottom(label="IPlaybackControl"),
    "IPointerInactive": SimTypeBottom(label="IPointerInactive"),
    "IPolicyQualifier": SimTypeBottom(label="IPolicyQualifier"),
    "IPolicyQualifiers": SimTypeBottom(label="IPolicyQualifiers"),
    "IPoolManager": SimTypeBottom(label="IPoolManager"),
    "IPortableDevice": SimTypeBottom(label="IPortableDevice"),
    "IPortableDeviceCapabilities": SimTypeBottom(label="IPortableDeviceCapabilities"),
    "IPortableDeviceConnector": SimTypeBottom(label="IPortableDeviceConnector"),
    "IPortableDeviceContent": SimTypeBottom(label="IPortableDeviceContent"),
    "IPortableDeviceContent2": SimTypeBottom(label="IPortableDeviceContent2"),
    "IPortableDeviceDataStream": SimTypeBottom(label="IPortableDeviceDataStream"),
    "IPortableDeviceDispatchFactory": SimTypeBottom(label="IPortableDeviceDispatchFactory"),
    "IPortableDeviceEventCallback": SimTypeBottom(label="IPortableDeviceEventCallback"),
    "IPortableDeviceKeyCollection": SimTypeBottom(label="IPortableDeviceKeyCollection"),
    "IPortableDeviceManager": SimTypeBottom(label="IPortableDeviceManager"),
    "IPortableDevicePropVariantCollection": SimTypeBottom(label="IPortableDevicePropVariantCollection"),
    "IPortableDeviceProperties": SimTypeBottom(label="IPortableDeviceProperties"),
    "IPortableDevicePropertiesBulk": SimTypeBottom(label="IPortableDevicePropertiesBulk"),
    "IPortableDevicePropertiesBulkCallback": SimTypeBottom(label="IPortableDevicePropertiesBulkCallback"),
    "IPortableDeviceResources": SimTypeBottom(label="IPortableDeviceResources"),
    "IPortableDeviceService": SimTypeBottom(label="IPortableDeviceService"),
    "IPortableDeviceServiceActivation": SimTypeBottom(label="IPortableDeviceServiceActivation"),
    "IPortableDeviceServiceCapabilities": SimTypeBottom(label="IPortableDeviceServiceCapabilities"),
    "IPortableDeviceServiceManager": SimTypeBottom(label="IPortableDeviceServiceManager"),
    "IPortableDeviceServiceMethodCallback": SimTypeBottom(label="IPortableDeviceServiceMethodCallback"),
    "IPortableDeviceServiceMethods": SimTypeBottom(label="IPortableDeviceServiceMethods"),
    "IPortableDeviceServiceOpenCallback": SimTypeBottom(label="IPortableDeviceServiceOpenCallback"),
    "IPortableDeviceUnitsStream": SimTypeBottom(label="IPortableDeviceUnitsStream"),
    "IPortableDeviceValues": SimTypeBottom(label="IPortableDeviceValues"),
    "IPortableDeviceValuesCollection": SimTypeBottom(label="IPortableDeviceValuesCollection"),
    "IPortableDeviceWebControl": SimTypeBottom(label="IPortableDeviceWebControl"),
    "IPreferredRuntimeTypeConcept": SimTypeBottom(label="IPreferredRuntimeTypeConcept"),
    "IPrepareInfo": SimTypeBottom(label="IPrepareInfo"),
    "IPrepareInfo2": SimTypeBottom(label="IPrepareInfo2"),
    "IPresentStatistics": SimTypeBottom(label="IPresentStatistics"),
    "IPresentStatusPresentStatistics": SimTypeBottom(label="IPresentStatusPresentStatistics"),
    "IPresentationBuffer": SimTypeBottom(label="IPresentationBuffer"),
    "IPresentationContent": SimTypeBottom(label="IPresentationContent"),
    "IPresentationFactory": SimTypeBottom(label="IPresentationFactory"),
    "IPresentationManager": SimTypeBottom(label="IPresentationManager"),
    "IPresentationSurface": SimTypeBottom(label="IPresentationSurface"),
    "IPreviewHandler": SimTypeBottom(label="IPreviewHandler"),
    "IPreviewHandlerFrame": SimTypeBottom(label="IPreviewHandlerFrame"),
    "IPreviewHandlerVisuals": SimTypeBottom(label="IPreviewHandlerVisuals"),
    "IPreviewItem": SimTypeBottom(label="IPreviewItem"),
    "IPreviousVersionsInfo": SimTypeBottom(label="IPreviousVersionsInfo"),
    "IPrincipal": SimTypeBottom(label="IPrincipal"),
    "IPrincipal2": SimTypeBottom(label="IPrincipal2"),
    "IPrint": SimTypeBottom(label="IPrint"),
    "IPrintAsyncCookie": SimTypeBottom(label="IPrintAsyncCookie"),
    "IPrintAsyncNewChannelCookie": SimTypeBottom(label="IPrintAsyncNewChannelCookie"),
    "IPrintAsyncNotify": SimTypeBottom(label="IPrintAsyncNotify"),
    "IPrintAsyncNotifyCallback": SimTypeBottom(label="IPrintAsyncNotifyCallback"),
    "IPrintAsyncNotifyChannel": SimTypeBottom(label="IPrintAsyncNotifyChannel"),
    "IPrintAsyncNotifyDataObject": SimTypeBottom(label="IPrintAsyncNotifyDataObject"),
    "IPrintAsyncNotifyRegistration": SimTypeBottom(label="IPrintAsyncNotifyRegistration"),
    "IPrintAsyncNotifyServerReferral": SimTypeBottom(label="IPrintAsyncNotifyServerReferral"),
    "IPrintBidiAsyncNotifyRegistration": SimTypeBottom(label="IPrintBidiAsyncNotifyRegistration"),
    "IPrintClassObjectFactory": SimTypeBottom(label="IPrintClassObjectFactory"),
    "IPrintCoreHelper": SimTypeBottom(label="IPrintCoreHelper"),
    "IPrintCoreHelperPS": SimTypeBottom(label="IPrintCoreHelperPS"),
    "IPrintCoreHelperUni": SimTypeBottom(label="IPrintCoreHelperUni"),
    "IPrintCoreHelperUni2": SimTypeBottom(label="IPrintCoreHelperUni2"),
    "IPrintCoreUI2": SimTypeBottom(label="IPrintCoreUI2"),
    "IPrintDialogCallback": SimTypeBottom(label="IPrintDialogCallback"),
    "IPrintDialogServices": SimTypeBottom(label="IPrintDialogServices"),
    "IPrintDocumentPackageStatusEvent": SimTypeBottom(label="IPrintDocumentPackageStatusEvent"),
    "IPrintDocumentPackageTarget": SimTypeBottom(label="IPrintDocumentPackageTarget"),
    "IPrintDocumentPackageTarget2": SimTypeBottom(label="IPrintDocumentPackageTarget2"),
    "IPrintDocumentPackageTargetFactory": SimTypeBottom(label="IPrintDocumentPackageTargetFactory"),
    "IPrintJob": SimTypeBottom(label="IPrintJob"),
    "IPrintJobCollection": SimTypeBottom(label="IPrintJobCollection"),
    "IPrintManagerTemplatePrinter": SimTypeBottom(label="IPrintManagerTemplatePrinter"),
    "IPrintManagerTemplatePrinter2": SimTypeBottom(label="IPrintManagerTemplatePrinter2"),
    "IPrintOemCommon": SimTypeBottom(label="IPrintOemCommon"),
    "IPrintOemDriverUI": SimTypeBottom(label="IPrintOemDriverUI"),
    "IPrintOemUI": SimTypeBottom(label="IPrintOemUI"),
    "IPrintOemUI2": SimTypeBottom(label="IPrintOemUI2"),
    "IPrintOemUIMXDC": SimTypeBottom(label="IPrintOemUIMXDC"),
    "IPrintPipelineFilter": SimTypeBottom(label="IPrintPipelineFilter"),
    "IPrintPipelineManagerControl": SimTypeBottom(label="IPrintPipelineManagerControl"),
    "IPrintPipelineProgressReport": SimTypeBottom(label="IPrintPipelineProgressReport"),
    "IPrintPipelinePropertyBag": SimTypeBottom(label="IPrintPipelinePropertyBag"),
    "IPrintPreviewDxgiPackageTarget": SimTypeBottom(label="IPrintPreviewDxgiPackageTarget"),
    "IPrintReadStream": SimTypeBottom(label="IPrintReadStream"),
    "IPrintReadStreamFactory": SimTypeBottom(label="IPrintReadStreamFactory"),
    "IPrintSchemaAsyncOperation": SimTypeBottom(label="IPrintSchemaAsyncOperation"),
    "IPrintSchemaAsyncOperationEvent": SimTypeBottom(label="IPrintSchemaAsyncOperationEvent"),
    "IPrintSchemaCapabilities": SimTypeBottom(label="IPrintSchemaCapabilities"),
    "IPrintSchemaCapabilities2": SimTypeBottom(label="IPrintSchemaCapabilities2"),
    "IPrintSchemaDisplayableElement": SimTypeBottom(label="IPrintSchemaDisplayableElement"),
    "IPrintSchemaElement": SimTypeBottom(label="IPrintSchemaElement"),
    "IPrintSchemaFeature": SimTypeBottom(label="IPrintSchemaFeature"),
    "IPrintSchemaNUpOption": SimTypeBottom(label="IPrintSchemaNUpOption"),
    "IPrintSchemaOption": SimTypeBottom(label="IPrintSchemaOption"),
    "IPrintSchemaOptionCollection": SimTypeBottom(label="IPrintSchemaOptionCollection"),
    "IPrintSchemaPageImageableSize": SimTypeBottom(label="IPrintSchemaPageImageableSize"),
    "IPrintSchemaPageMediaSizeOption": SimTypeBottom(label="IPrintSchemaPageMediaSizeOption"),
    "IPrintSchemaParameterDefinition": SimTypeBottom(label="IPrintSchemaParameterDefinition"),
    "IPrintSchemaParameterInitializer": SimTypeBottom(label="IPrintSchemaParameterInitializer"),
    "IPrintSchemaTicket": SimTypeBottom(label="IPrintSchemaTicket"),
    "IPrintSchemaTicket2": SimTypeBottom(label="IPrintSchemaTicket2"),
    "IPrintTaskRequestFactory": SimTypeBottom(label="IPrintTaskRequestFactory"),
    "IPrintTaskRequestHandler": SimTypeBottom(label="IPrintTaskRequestHandler"),
    "IPrintTicketProvider": SimTypeBottom(label="IPrintTicketProvider"),
    "IPrintTicketProvider2": SimTypeBottom(label="IPrintTicketProvider2"),
    "IPrintUnidiAsyncNotifyRegistration": SimTypeBottom(label="IPrintUnidiAsyncNotifyRegistration"),
    "IPrintWriteStream": SimTypeBottom(label="IPrintWriteStream"),
    "IPrintWriteStreamFlush": SimTypeBottom(label="IPrintWriteStreamFlush"),
    "IPrinterBidiSetRequestCallback": SimTypeBottom(label="IPrinterBidiSetRequestCallback"),
    "IPrinterExtensionAsyncOperation": SimTypeBottom(label="IPrinterExtensionAsyncOperation"),
    "IPrinterExtensionContext": SimTypeBottom(label="IPrinterExtensionContext"),
    "IPrinterExtensionContextCollection": SimTypeBottom(label="IPrinterExtensionContextCollection"),
    "IPrinterExtensionEvent": SimTypeBottom(label="IPrinterExtensionEvent"),
    "IPrinterExtensionEventArgs": SimTypeBottom(label="IPrinterExtensionEventArgs"),
    "IPrinterExtensionManager": SimTypeBottom(label="IPrinterExtensionManager"),
    "IPrinterExtensionRequest": SimTypeBottom(label="IPrinterExtensionRequest"),
    "IPrinterPropertyBag": SimTypeBottom(label="IPrinterPropertyBag"),
    "IPrinterQueue": SimTypeBottom(label="IPrinterQueue"),
    "IPrinterQueue2": SimTypeBottom(label="IPrinterQueue2"),
    "IPrinterQueueEvent": SimTypeBottom(label="IPrinterQueueEvent"),
    "IPrinterQueueView": SimTypeBottom(label="IPrinterQueueView"),
    "IPrinterQueueViewEvent": SimTypeBottom(label="IPrinterQueueViewEvent"),
    "IPrinterScriptContext": SimTypeBottom(label="IPrinterScriptContext"),
    "IPrinterScriptablePropertyBag": SimTypeBottom(label="IPrinterScriptablePropertyBag"),
    "IPrinterScriptablePropertyBag2": SimTypeBottom(label="IPrinterScriptablePropertyBag2"),
    "IPrinterScriptableSequentialStream": SimTypeBottom(label="IPrinterScriptableSequentialStream"),
    "IPrinterScriptableStream": SimTypeBottom(label="IPrinterScriptableStream"),
    "IPrivateDispatch": SimTypeBottom(label="IPrivateDispatch"),
    "IPrivateUnknown": SimTypeBottom(label="IPrivateUnknown"),
    "IProcessDebugManager32": SimTypeBottom(label="IProcessDebugManager32"),
    "IProcessDebugManager64": SimTypeBottom(label="IProcessDebugManager64"),
    "IProcessInitControl": SimTypeBottom(label="IProcessInitControl"),
    "IProcessInitializer": SimTypeBottom(label="IProcessInitializer"),
    "IProcessLock": SimTypeBottom(label="IProcessLock"),
    "IProfSect": SimTypeBottom(label="IProfSect"),
    "IProfferService": SimTypeBottom(label="IProfferService"),
    "IProgressDialog": SimTypeBottom(label="IProgressDialog"),
    "IProgressItem": SimTypeBottom(label="IProgressItem"),
    "IProgressItems": SimTypeBottom(label="IProgressItems"),
    "IProgressNotify": SimTypeBottom(label="IProgressNotify"),
    "IProofOfPossessionCookieInfoManager": SimTypeBottom(label="IProofOfPossessionCookieInfoManager"),
    "IProofOfPossessionCookieInfoManager2": SimTypeBottom(label="IProofOfPossessionCookieInfoManager2"),
    "IPropData": SimTypeBottom(label="IPropData"),
    "IPropertyBag": SimTypeBottom(label="IPropertyBag"),
    "IPropertyBag2": SimTypeBottom(label="IPropertyBag2"),
    "IPropertyChange": SimTypeBottom(label="IPropertyChange"),
    "IPropertyChangeArray": SimTypeBottom(label="IPropertyChangeArray"),
    "IPropertyDescription": SimTypeBottom(label="IPropertyDescription"),
    "IPropertyDescription2": SimTypeBottom(label="IPropertyDescription2"),
    "IPropertyDescriptionAliasInfo": SimTypeBottom(label="IPropertyDescriptionAliasInfo"),
    "IPropertyDescriptionList": SimTypeBottom(label="IPropertyDescriptionList"),
    "IPropertyDescriptionRelatedPropertyInfo": SimTypeBottom(label="IPropertyDescriptionRelatedPropertyInfo"),
    "IPropertyDescriptionSearchInfo": SimTypeBottom(label="IPropertyDescriptionSearchInfo"),
    "IPropertyEnumType": SimTypeBottom(label="IPropertyEnumType"),
    "IPropertyEnumType2": SimTypeBottom(label="IPropertyEnumType2"),
    "IPropertyEnumTypeList": SimTypeBottom(label="IPropertyEnumTypeList"),
    "IPropertyKeyStore": SimTypeBottom(label="IPropertyKeyStore"),
    "IPropertyNotifySink": SimTypeBottom(label="IPropertyNotifySink"),
    "IPropertyPage": SimTypeBottom(label="IPropertyPage"),
    "IPropertyPage2": SimTypeBottom(label="IPropertyPage2"),
    "IPropertyPageSite": SimTypeBottom(label="IPropertyPageSite"),
    "IPropertySetStorage": SimTypeBottom(label="IPropertySetStorage"),
    "IPropertySheetCallback": SimTypeBottom(label="IPropertySheetCallback"),
    "IPropertySheetProvider": SimTypeBottom(label="IPropertySheetProvider"),
    "IPropertyStorage": SimTypeBottom(label="IPropertyStorage"),
    "IPropertyStore": SimTypeBottom(label="IPropertyStore"),
    "IPropertyStoreCache": SimTypeBottom(label="IPropertyStoreCache"),
    "IPropertyStoreCapabilities": SimTypeBottom(label="IPropertyStoreCapabilities"),
    "IPropertyStoreCollection": SimTypeBottom(label="IPropertyStoreCollection"),
    "IPropertyStoreFactory": SimTypeBottom(label="IPropertyStoreFactory"),
    "IPropertySystem": SimTypeBottom(label="IPropertySystem"),
    "IPropertySystemChangeNotify": SimTypeBottom(label="IPropertySystemChangeNotify"),
    "IPropertyUI": SimTypeBottom(label="IPropertyUI"),
    "IProtectFocus": SimTypeBottom(label="IProtectFocus"),
    "IProtectedModeMenuServices": SimTypeBottom(label="IProtectedModeMenuServices"),
    "IProtectionPolicyManagerInterop": SimTypeBottom(label="IProtectionPolicyManagerInterop"),
    "IProtectionPolicyManagerInterop2": SimTypeBottom(label="IProtectionPolicyManagerInterop2"),
    "IProtectionPolicyManagerInterop3": SimTypeBottom(label="IProtectionPolicyManagerInterop3"),
    "IProtocolHandlerSite": SimTypeBottom(label="IProtocolHandlerSite"),
    "IProvideClassInfo": SimTypeBottom(label="IProvideClassInfo"),
    "IProvideClassInfo2": SimTypeBottom(label="IProvideClassInfo2"),
    "IProvideExpressionContexts": SimTypeBottom(label="IProvideExpressionContexts"),
    "IProvideMoniker": SimTypeBottom(label="IProvideMoniker"),
    "IProvideMultipleClassInfo": SimTypeBottom(label="IProvideMultipleClassInfo"),
    "IProvideRuntimeContext": SimTypeBottom(label="IProvideRuntimeContext"),
    "IProvideTaskPage": SimTypeBottom(label="IProvideTaskPage"),
    "IProvideWinSATAssessmentInfo": SimTypeBottom(label="IProvideWinSATAssessmentInfo"),
    "IProvideWinSATResultsInfo": SimTypeBottom(label="IProvideWinSATResultsInfo"),
    "IProvideWinSATVisuals": SimTypeBottom(label="IProvideWinSATVisuals"),
    "IProviderAdmin": SimTypeBottom(label="IProviderAdmin"),
    "IProviderConverter": SimTypeBottom(label="IProviderConverter"),
    "IProviderProperties": SimTypeBottom(label="IProviderProperties"),
    "IProviderPropertyConstraintCollection": SimTypeBottom(label="IProviderPropertyConstraintCollection"),
    "IProviderPublishing": SimTypeBottom(label="IProviderPublishing"),
    "IProviderQueryConstraintCollection": SimTypeBottom(label="IProviderQueryConstraintCollection"),
    "IProvisioningDomain": SimTypeBottom(label="IProvisioningDomain"),
    "IProvisioningProfileWireless": SimTypeBottom(label="IProvisioningProfileWireless"),
    "IProxyProviderWinEventHandler": SimTypeBottom(label="IProxyProviderWinEventHandler"),
    "IProxyProviderWinEventSink": SimTypeBottom(label="IProxyProviderWinEventSink"),
    "IPublishedApp": SimTypeBottom(label="IPublishedApp"),
    "IPublishedApp2": SimTypeBottom(label="IPublishedApp2"),
    "IPublisherFilter": SimTypeBottom(label="IPublisherFilter"),
    "IPublishingWizard": SimTypeBottom(label="IPublishingWizard"),
    "IQualProp": SimTypeBottom(label="IQualProp"),
    "IQualityControl": SimTypeBottom(label="IQualityControl"),
    "IQueryAllWinSATAssessments": SimTypeBottom(label="IQueryAllWinSATAssessments"),
    "IQueryAssociations": SimTypeBottom(label="IQueryAssociations"),
    "IQueryCancelAutoPlay": SimTypeBottom(label="IQueryCancelAutoPlay"),
    "IQueryCodePage": SimTypeBottom(label="IQueryCodePage"),
    "IQueryContinue": SimTypeBottom(label="IQueryContinue"),
    "IQueryContinueWithStatus": SimTypeBottom(label="IQueryContinueWithStatus"),
    "IQueryForm": SimTypeBottom(label="IQueryForm"),
    "IQueryInfo": SimTypeBottom(label="IQueryInfo"),
    "IQueryOEMWinSATCustomization": SimTypeBottom(label="IQueryOEMWinSATCustomization"),
    "IQueryParser": SimTypeBottom(label="IQueryParser"),
    "IQueryParserManager": SimTypeBottom(label="IQueryParserManager"),
    "IQueryRecentWinSATAssessment": SimTypeBottom(label="IQueryRecentWinSATAssessment"),
    "IQuerySolution": SimTypeBottom(label="IQuerySolution"),
    "IQueueCommand": SimTypeBottom(label="IQueueCommand"),
    "IQuickActivate": SimTypeBottom(label="IQuickActivate"),
    "IRDPSRAPIApplication": SimTypeBottom(label="IRDPSRAPIApplication"),
    "IRDPSRAPIApplicationFilter": SimTypeBottom(label="IRDPSRAPIApplicationFilter"),
    "IRDPSRAPIApplicationList": SimTypeBottom(label="IRDPSRAPIApplicationList"),
    "IRDPSRAPIAttendee": SimTypeBottom(label="IRDPSRAPIAttendee"),
    "IRDPSRAPIAttendeeDisconnectInfo": SimTypeBottom(label="IRDPSRAPIAttendeeDisconnectInfo"),
    "IRDPSRAPIAttendeeManager": SimTypeBottom(label="IRDPSRAPIAttendeeManager"),
    "IRDPSRAPIAudioStream": SimTypeBottom(label="IRDPSRAPIAudioStream"),
    "IRDPSRAPIClipboardUseEvents": SimTypeBottom(label="IRDPSRAPIClipboardUseEvents"),
    "IRDPSRAPIDebug": SimTypeBottom(label="IRDPSRAPIDebug"),
    "IRDPSRAPIFrameBuffer": SimTypeBottom(label="IRDPSRAPIFrameBuffer"),
    "IRDPSRAPIInvitation": SimTypeBottom(label="IRDPSRAPIInvitation"),
    "IRDPSRAPIInvitationManager": SimTypeBottom(label="IRDPSRAPIInvitationManager"),
    "IRDPSRAPIPerfCounterLogger": SimTypeBottom(label="IRDPSRAPIPerfCounterLogger"),
    "IRDPSRAPIPerfCounterLoggingManager": SimTypeBottom(label="IRDPSRAPIPerfCounterLoggingManager"),
    "IRDPSRAPISessionProperties": SimTypeBottom(label="IRDPSRAPISessionProperties"),
    "IRDPSRAPISharingSession": SimTypeBottom(label="IRDPSRAPISharingSession"),
    "IRDPSRAPISharingSession2": SimTypeBottom(label="IRDPSRAPISharingSession2"),
    "IRDPSRAPITcpConnectionInfo": SimTypeBottom(label="IRDPSRAPITcpConnectionInfo"),
    "IRDPSRAPITransportStream": SimTypeBottom(label="IRDPSRAPITransportStream"),
    "IRDPSRAPITransportStreamBuffer": SimTypeBottom(label="IRDPSRAPITransportStreamBuffer"),
    "IRDPSRAPITransportStreamEvents": SimTypeBottom(label="IRDPSRAPITransportStreamEvents"),
    "IRDPSRAPIViewer": SimTypeBottom(label="IRDPSRAPIViewer"),
    "IRDPSRAPIVirtualChannel": SimTypeBottom(label="IRDPSRAPIVirtualChannel"),
    "IRDPSRAPIVirtualChannelManager": SimTypeBottom(label="IRDPSRAPIVirtualChannelManager"),
    "IRDPSRAPIWindow": SimTypeBottom(label="IRDPSRAPIWindow"),
    "IRDPSRAPIWindowList": SimTypeBottom(label="IRDPSRAPIWindowList"),
    "IRDPViewerInputSink": SimTypeBottom(label="IRDPViewerInputSink"),
    "IRMHelper": SimTypeBottom(label="IRMHelper"),
    "IROTData": SimTypeBottom(label="IROTData"),
    "IRP": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AssociatedIrp", SimUnion({"MasterIrp": SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), "IrpCount": SimTypeInt(signed=True, label="Int32"), "SystemBuffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("ThreadListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("IoStatus", SimTypeRef("IO_STATUS_BLOCK", SimStruct)), ("RequestorMode", SimTypeChar(label="SByte")), ("PendingReturned", SimTypeChar(label="Byte")), ("StackCount", SimTypeChar(label="SByte")), ("CurrentLocation", SimTypeChar(label="SByte")), ("Cancel", SimTypeChar(label="Byte")), ("CancelIrql", SimTypeChar(label="Byte")), ("ApcEnvironment", SimTypeChar(label="SByte")), ("AllocationFlags", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"UserIosb": SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), "IoRingContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("UserEvent", SimTypePointer(SimTypeRef("KEVENT", SimStruct), offset=0)), ("Overlay", SimUnion({"AsynchronousParameters": SimTypeRef("_AsynchronousParameters_e__Struct", SimStruct), "AllocationSize": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("CancelRoutine", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp"]), offset=0), offset=0)), ("UserBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Tail", SimUnion({"Overlay": SimTypeRef("_Overlay_e__Struct", SimStruct), "Apc": SimTypeRef("KAPC", SimStruct), "CompletionKey": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="IRP", pack=False, align=None),
    "IRQD_FLAGS": SimTypeInt(signed=False, label="IRQD_FLAGS"),
    "IRQ_DES_32": SimStruct(OrderedDict((("IRQD_Count", SimTypeInt(signed=False, label="UInt32")), ("IRQD_Type", SimTypeInt(signed=False, label="UInt32")), ("IRQD_Flags", SimTypeInt(signed=False, label="IRQD_FLAGS")), ("IRQD_Alloc_Num", SimTypeInt(signed=False, label="UInt32")), ("IRQD_Affinity", SimTypeInt(signed=False, label="UInt32")),)), name="IRQ_DES_32", pack=False, align=None),
    "IRQ_DES_64": SimStruct(OrderedDict((("IRQD_Count", SimTypeInt(signed=False, label="UInt32")), ("IRQD_Type", SimTypeInt(signed=False, label="UInt32")), ("IRQD_Flags", SimTypeInt(signed=False, label="IRQD_FLAGS")), ("IRQD_Alloc_Num", SimTypeInt(signed=False, label="UInt32")), ("IRQD_Affinity", SimTypeLongLong(signed=False, label="UInt64")),)), name="IRQ_DES_64", pack=False, align=None),
    "IRQ_DEVICE_POLICY": SimTypeInt(signed=False, label="IRQ_DEVICE_POLICY"),
    "IRQ_GROUP_POLICY": SimTypeInt(signed=False, label="IRQ_GROUP_POLICY"),
    "IRQ_PRIORITY": SimTypeInt(signed=False, label="IRQ_PRIORITY"),
    "IRQ_RANGE": SimStruct(OrderedDict((("IRQR_Min", SimTypeInt(signed=False, label="UInt32")), ("IRQR_Max", SimTypeInt(signed=False, label="UInt32")), ("IRQR_Flags", SimTypeInt(signed=False, label="IRQD_FLAGS")),)), name="IRQ_RANGE", pack=False, align=None),
    "IRQ_RESOURCE_32": SimStruct(OrderedDict((("IRQ_Header", SimTypeRef("IRQ_DES_32", SimStruct)), ("IRQ_Data", SimTypePointer(SimTypeRef("IRQ_RANGE", SimStruct), offset=0)),)), name="IRQ_RESOURCE_32", pack=False, align=None),
    "IRQ_RESOURCE_64": SimStruct(OrderedDict((("IRQ_Header", SimTypeRef("IRQ_DES_64", SimStruct)), ("IRQ_Data", SimTypePointer(SimTypeRef("IRQ_RANGE", SimStruct), offset=0)),)), name="IRQ_RESOURCE_64", pack=False, align=None),
    "IRSOPInformation": SimTypeBottom(label="IRSOPInformation"),
    "IRTCBuddy": SimTypeBottom(label="IRTCBuddy"),
    "IRTCBuddy2": SimTypeBottom(label="IRTCBuddy2"),
    "IRTCBuddyEvent": SimTypeBottom(label="IRTCBuddyEvent"),
    "IRTCBuddyEvent2": SimTypeBottom(label="IRTCBuddyEvent2"),
    "IRTCBuddyGroup": SimTypeBottom(label="IRTCBuddyGroup"),
    "IRTCBuddyGroupEvent": SimTypeBottom(label="IRTCBuddyGroupEvent"),
    "IRTCClient": SimTypeBottom(label="IRTCClient"),
    "IRTCClient2": SimTypeBottom(label="IRTCClient2"),
    "IRTCClientEvent": SimTypeBottom(label="IRTCClientEvent"),
    "IRTCClientPortManagement": SimTypeBottom(label="IRTCClientPortManagement"),
    "IRTCClientPresence": SimTypeBottom(label="IRTCClientPresence"),
    "IRTCClientPresence2": SimTypeBottom(label="IRTCClientPresence2"),
    "IRTCClientProvisioning": SimTypeBottom(label="IRTCClientProvisioning"),
    "IRTCClientProvisioning2": SimTypeBottom(label="IRTCClientProvisioning2"),
    "IRTCCollection": SimTypeBottom(label="IRTCCollection"),
    "IRTCDispatchEventNotification": SimTypeBottom(label="IRTCDispatchEventNotification"),
    "IRTCEnumBuddies": SimTypeBottom(label="IRTCEnumBuddies"),
    "IRTCEnumGroups": SimTypeBottom(label="IRTCEnumGroups"),
    "IRTCEnumParticipants": SimTypeBottom(label="IRTCEnumParticipants"),
    "IRTCEnumPresenceDevices": SimTypeBottom(label="IRTCEnumPresenceDevices"),
    "IRTCEnumProfiles": SimTypeBottom(label="IRTCEnumProfiles"),
    "IRTCEnumUserSearchResults": SimTypeBottom(label="IRTCEnumUserSearchResults"),
    "IRTCEnumWatchers": SimTypeBottom(label="IRTCEnumWatchers"),
    "IRTCEventNotification": SimTypeBottom(label="IRTCEventNotification"),
    "IRTCInfoEvent": SimTypeBottom(label="IRTCInfoEvent"),
    "IRTCIntensityEvent": SimTypeBottom(label="IRTCIntensityEvent"),
    "IRTCMediaEvent": SimTypeBottom(label="IRTCMediaEvent"),
    "IRTCMediaRequestEvent": SimTypeBottom(label="IRTCMediaRequestEvent"),
    "IRTCMessagingEvent": SimTypeBottom(label="IRTCMessagingEvent"),
    "IRTCParticipant": SimTypeBottom(label="IRTCParticipant"),
    "IRTCParticipantStateChangeEvent": SimTypeBottom(label="IRTCParticipantStateChangeEvent"),
    "IRTCPortManager": SimTypeBottom(label="IRTCPortManager"),
    "IRTCPresenceContact": SimTypeBottom(label="IRTCPresenceContact"),
    "IRTCPresenceDataEvent": SimTypeBottom(label="IRTCPresenceDataEvent"),
    "IRTCPresenceDevice": SimTypeBottom(label="IRTCPresenceDevice"),
    "IRTCPresencePropertyEvent": SimTypeBottom(label="IRTCPresencePropertyEvent"),
    "IRTCPresenceStatusEvent": SimTypeBottom(label="IRTCPresenceStatusEvent"),
    "IRTCProfile": SimTypeBottom(label="IRTCProfile"),
    "IRTCProfile2": SimTypeBottom(label="IRTCProfile2"),
    "IRTCProfileEvent": SimTypeBottom(label="IRTCProfileEvent"),
    "IRTCProfileEvent2": SimTypeBottom(label="IRTCProfileEvent2"),
    "IRTCReInviteEvent": SimTypeBottom(label="IRTCReInviteEvent"),
    "IRTCRegistrationStateChangeEvent": SimTypeBottom(label="IRTCRegistrationStateChangeEvent"),
    "IRTCRoamingEvent": SimTypeBottom(label="IRTCRoamingEvent"),
    "IRTCSession": SimTypeBottom(label="IRTCSession"),
    "IRTCSession2": SimTypeBottom(label="IRTCSession2"),
    "IRTCSessionCallControl": SimTypeBottom(label="IRTCSessionCallControl"),
    "IRTCSessionDescriptionManager": SimTypeBottom(label="IRTCSessionDescriptionManager"),
    "IRTCSessionOperationCompleteEvent": SimTypeBottom(label="IRTCSessionOperationCompleteEvent"),
    "IRTCSessionOperationCompleteEvent2": SimTypeBottom(label="IRTCSessionOperationCompleteEvent2"),
    "IRTCSessionPortManagement": SimTypeBottom(label="IRTCSessionPortManagement"),
    "IRTCSessionReferStatusEvent": SimTypeBottom(label="IRTCSessionReferStatusEvent"),
    "IRTCSessionReferredEvent": SimTypeBottom(label="IRTCSessionReferredEvent"),
    "IRTCSessionStateChangeEvent": SimTypeBottom(label="IRTCSessionStateChangeEvent"),
    "IRTCSessionStateChangeEvent2": SimTypeBottom(label="IRTCSessionStateChangeEvent2"),
    "IRTCUserSearch": SimTypeBottom(label="IRTCUserSearch"),
    "IRTCUserSearchQuery": SimTypeBottom(label="IRTCUserSearchQuery"),
    "IRTCUserSearchResult": SimTypeBottom(label="IRTCUserSearchResult"),
    "IRTCUserSearchResultsEvent": SimTypeBottom(label="IRTCUserSearchResultsEvent"),
    "IRTCWatcher": SimTypeBottom(label="IRTCWatcher"),
    "IRTCWatcher2": SimTypeBottom(label="IRTCWatcher2"),
    "IRTCWatcherEvent": SimTypeBottom(label="IRTCWatcherEvent"),
    "IRTCWatcherEvent2": SimTypeBottom(label="IRTCWatcherEvent2"),
    "IRadialControllerConfigurationInterop": SimTypeBottom(label="IRadialControllerConfigurationInterop"),
    "IRadialControllerIndependentInputSourceInterop": SimTypeBottom(label="IRadialControllerIndependentInputSourceInterop"),
    "IRadialControllerInterop": SimTypeBottom(label="IRadialControllerInterop"),
    "IRadioInstance": SimTypeBottom(label="IRadioInstance"),
    "IRadioInstanceCollection": SimTypeBottom(label="IRadioInstanceCollection"),
    "IRangeException": SimTypeBottom(label="IRangeException"),
    "IRangeValueProvider": SimTypeBottom(label="IRangeValueProvider"),
    "IRawCDImageCreator": SimTypeBottom(label="IRawCDImageCreator"),
    "IRawCDImageTrackInfo": SimTypeBottom(label="IRawCDImageTrackInfo"),
    "IRawElementProviderAdviseEvents": SimTypeBottom(label="IRawElementProviderAdviseEvents"),
    "IRawElementProviderFragment": SimTypeBottom(label="IRawElementProviderFragment"),
    "IRawElementProviderFragmentRoot": SimTypeBottom(label="IRawElementProviderFragmentRoot"),
    "IRawElementProviderHostingAccessibles": SimTypeBottom(label="IRawElementProviderHostingAccessibles"),
    "IRawElementProviderHwndOverride": SimTypeBottom(label="IRawElementProviderHwndOverride"),
    "IRawElementProviderSimple": SimTypeBottom(label="IRawElementProviderSimple"),
    "IRawElementProviderSimple2": SimTypeBottom(label="IRawElementProviderSimple2"),
    "IRawElementProviderSimple3": SimTypeBottom(label="IRawElementProviderSimple3"),
    "IRawElementProviderWindowlessSite": SimTypeBottom(label="IRawElementProviderWindowlessSite"),
    "IRawEnumerator": SimTypeBottom(label="IRawEnumerator"),
    "IRdcComparator": SimTypeBottom(label="IRdcComparator"),
    "IRdcFileReader": SimTypeBottom(label="IRdcFileReader"),
    "IRdcFileWriter": SimTypeBottom(label="IRdcFileWriter"),
    "IRdcGenerator": SimTypeBottom(label="IRdcGenerator"),
    "IRdcGeneratorFilterMaxParameters": SimTypeBottom(label="IRdcGeneratorFilterMaxParameters"),
    "IRdcGeneratorParameters": SimTypeBottom(label="IRdcGeneratorParameters"),
    "IRdcLibrary": SimTypeBottom(label="IRdcLibrary"),
    "IRdcSignatureReader": SimTypeBottom(label="IRdcSignatureReader"),
    "IRdcSimilarityGenerator": SimTypeBottom(label="IRdcSimilarityGenerator"),
    "IReadData": SimTypeBottom(label="IReadData"),
    "IRealTimeStylus": SimTypeBottom(label="IRealTimeStylus"),
    "IRealTimeStylus2": SimTypeBottom(label="IRealTimeStylus2"),
    "IRealTimeStylus3": SimTypeBottom(label="IRealTimeStylus3"),
    "IRealTimeStylusSynchronization": SimTypeBottom(label="IRealTimeStylusSynchronization"),
    "IReconcilableObject": SimTypeBottom(label="IReconcilableObject"),
    "IReconcileInitiator": SimTypeBottom(label="IReconcileInitiator"),
    "IRecordInfo": SimTypeBottom(label="IRecordInfo"),
    "IRecoverableError": SimTypeBottom(label="IRecoverableError"),
    "IRecoverableErrorData": SimTypeBottom(label="IRecoverableErrorData"),
    "IRedbookDiscMaster": SimTypeBottom(label="IRedbookDiscMaster"),
    "IReferenceClock": SimTypeBottom(label="IReferenceClock"),
    "IReferenceClock2": SimTypeBottom(label="IReferenceClock2"),
    "IReferenceClockTimerControl": SimTypeBottom(label="IReferenceClockTimerControl"),
    "IRegFilterInfo": SimTypeBottom(label="IRegFilterInfo"),
    "IRegTreeItem": SimTypeBottom(label="IRegTreeItem"),
    "IRegisterProvider": SimTypeBottom(label="IRegisterProvider"),
    "IRegisterServiceProvider": SimTypeBottom(label="IRegisterServiceProvider"),
    "IRegisterTuner": SimTypeBottom(label="IRegisterTuner"),
    "IRegisteredSyncProvider": SimTypeBottom(label="IRegisteredSyncProvider"),
    "IRegisteredTask": SimTypeBottom(label="IRegisteredTask"),
    "IRegisteredTaskCollection": SimTypeBottom(label="IRegisteredTaskCollection"),
    "IRegistrationInfo": SimTypeBottom(label="IRegistrationInfo"),
    "IRegistrationTrigger": SimTypeBottom(label="IRegistrationTrigger"),
    "IRelatedItem": SimTypeBottom(label="IRelatedItem"),
    "IRelationship": SimTypeBottom(label="IRelationship"),
    "IReleaseMarshalBuffers": SimTypeBottom(label="IReleaseMarshalBuffers"),
    "IRemoteComponentUtil": SimTypeBottom(label="IRemoteComponentUtil"),
    "IRemoteComputer": SimTypeBottom(label="IRemoteComputer"),
    "IRemoteDebugApplication": SimTypeBottom(label="IRemoteDebugApplication"),
    "IRemoteDebugApplication110": SimTypeBottom(label="IRemoteDebugApplication110"),
    "IRemoteDebugApplicationEvents": SimTypeBottom(label="IRemoteDebugApplicationEvents"),
    "IRemoteDebugApplicationThread": SimTypeBottom(label="IRemoteDebugApplicationThread"),
    "IRemoteDebugCriticalErrorEvent110": SimTypeBottom(label="IRemoteDebugCriticalErrorEvent110"),
    "IRemoteDebugInfoEvent110": SimTypeBottom(label="IRemoteDebugInfoEvent110"),
    "IRemoteDesktopClient": SimTypeBottom(label="IRemoteDesktopClient"),
    "IRemoteDesktopClientActions": SimTypeBottom(label="IRemoteDesktopClientActions"),
    "IRemoteDesktopClientSettings": SimTypeBottom(label="IRemoteDesktopClientSettings"),
    "IRemoteDesktopClientTouchPointer": SimTypeBottom(label="IRemoteDesktopClientTouchPointer"),
    "IRemoteSystemAdditionalInfoProvider": SimTypeBottom(label="IRemoteSystemAdditionalInfoProvider"),
    "IRendezvousApplication": SimTypeBottom(label="IRendezvousApplication"),
    "IRendezvousSession": SimTypeBottom(label="IRendezvousSession"),
    "IRepetitionPattern": SimTypeBottom(label="IRepetitionPattern"),
    "IReplicaKeyMap": SimTypeBottom(label="IReplicaKeyMap"),
    "IRequestFilteredSync": SimTypeBottom(label="IRequestFilteredSync"),
    "IRequiredExtensions": SimTypeBottom(label="IRequiredExtensions"),
    "IResolveShellLink": SimTypeBottom(label="IResolveShellLink"),
    "IResourceConsumer": SimTypeBottom(label="IResourceConsumer"),
    "IResourceManager": SimTypeBottom(label="IResourceManager"),
    "IResourceManager2": SimTypeBottom(label="IResourceManager2"),
    "IResourceManagerFactory": SimTypeBottom(label="IResourceManagerFactory"),
    "IResourceManagerFactory2": SimTypeBottom(label="IResourceManagerFactory2"),
    "IResourceManagerRejoinable": SimTypeBottom(label="IResourceManagerRejoinable"),
    "IResourceManagerSink": SimTypeBottom(label="IResourceManagerSink"),
    "IResultData": SimTypeBottom(label="IResultData"),
    "IResultData2": SimTypeBottom(label="IResultData2"),
    "IResultDataCompare": SimTypeBottom(label="IResultDataCompare"),
    "IResultDataCompareEx": SimTypeBottom(label="IResultDataCompareEx"),
    "IResultOwnerData": SimTypeBottom(label="IResultOwnerData"),
    "IResultsFolder": SimTypeBottom(label="IResultsFolder"),
    "IRichChunk": SimTypeBottom(label="IRichChunk"),
    "IRichEditOle": SimTypeBottom(label="IRichEditOle"),
    "IRichEditOleCallback": SimTypeBottom(label="IRichEditOleCallback"),
    "IRichEditUiaInformation": SimTypeBottom(label="IRichEditUiaInformation"),
    "IRicheditUiaOverrides": SimTypeBottom(label="IRicheditUiaOverrides"),
    "IRicheditWindowlessAccessibility": SimTypeBottom(label="IRicheditWindowlessAccessibility"),
    "IRoleAssociationUtil": SimTypeBottom(label="IRoleAssociationUtil"),
    "IRootStorage": SimTypeBottom(label="IRootStorage"),
    "IRouterProtocolConfig": SimTypeBottom(label="IRouterProtocolConfig"),
    "IRow": SimTypeBottom(label="IRow"),
    "IRowChange": SimTypeBottom(label="IRowChange"),
    "IRowPosition": SimTypeBottom(label="IRowPosition"),
    "IRowPositionChange": SimTypeBottom(label="IRowPositionChange"),
    "IRowSchemaChange": SimTypeBottom(label="IRowSchemaChange"),
    "IRowset": SimTypeBottom(label="IRowset"),
    "IRowsetAsynch": SimTypeBottom(label="IRowsetAsynch"),
    "IRowsetBookmark": SimTypeBottom(label="IRowsetBookmark"),
    "IRowsetChange": SimTypeBottom(label="IRowsetChange"),
    "IRowsetChangeExtInfo": SimTypeBottom(label="IRowsetChangeExtInfo"),
    "IRowsetChapterMember": SimTypeBottom(label="IRowsetChapterMember"),
    "IRowsetCopyRows": SimTypeBottom(label="IRowsetCopyRows"),
    "IRowsetCurrentIndex": SimTypeBottom(label="IRowsetCurrentIndex"),
    "IRowsetEvents": SimTypeBottom(label="IRowsetEvents"),
    "IRowsetExactScroll": SimTypeBottom(label="IRowsetExactScroll"),
    "IRowsetFastLoad": SimTypeBottom(label="IRowsetFastLoad"),
    "IRowsetFind": SimTypeBottom(label="IRowsetFind"),
    "IRowsetIdentity": SimTypeBottom(label="IRowsetIdentity"),
    "IRowsetIndex": SimTypeBottom(label="IRowsetIndex"),
    "IRowsetInfo": SimTypeBottom(label="IRowsetInfo"),
    "IRowsetKeys": SimTypeBottom(label="IRowsetKeys"),
    "IRowsetLocate": SimTypeBottom(label="IRowsetLocate"),
    "IRowsetNewRowAfter": SimTypeBottom(label="IRowsetNewRowAfter"),
    "IRowsetNextRowset": SimTypeBottom(label="IRowsetNextRowset"),
    "IRowsetNotify": SimTypeBottom(label="IRowsetNotify"),
    "IRowsetPrioritization": SimTypeBottom(label="IRowsetPrioritization"),
    "IRowsetQueryStatus": SimTypeBottom(label="IRowsetQueryStatus"),
    "IRowsetRefresh": SimTypeBottom(label="IRowsetRefresh"),
    "IRowsetResynch": SimTypeBottom(label="IRowsetResynch"),
    "IRowsetScroll": SimTypeBottom(label="IRowsetScroll"),
    "IRowsetUpdate": SimTypeBottom(label="IRowsetUpdate"),
    "IRowsetView": SimTypeBottom(label="IRowsetView"),
    "IRowsetWatchAll": SimTypeBottom(label="IRowsetWatchAll"),
    "IRowsetWatchNotify": SimTypeBottom(label="IRowsetWatchNotify"),
    "IRowsetWatchRegion": SimTypeBottom(label="IRowsetWatchRegion"),
    "IRowsetWithParameters": SimTypeBottom(label="IRowsetWithParameters"),
    "IRpcChannelBuffer": SimTypeBottom(label="IRpcChannelBuffer"),
    "IRpcChannelBuffer2": SimTypeBottom(label="IRpcChannelBuffer2"),
    "IRpcChannelBuffer3": SimTypeBottom(label="IRpcChannelBuffer3"),
    "IRpcHelper": SimTypeBottom(label="IRpcHelper"),
    "IRpcOptions": SimTypeBottom(label="IRpcOptions"),
    "IRpcProxyBuffer": SimTypeBottom(label="IRpcProxyBuffer"),
    "IRpcStubBuffer": SimTypeBottom(label="IRpcStubBuffer"),
    "IRpcSyntaxNegotiate": SimTypeBottom(label="IRpcSyntaxNegotiate"),
    "IRtwqAsyncCallback": SimTypeBottom(label="IRtwqAsyncCallback"),
    "IRtwqAsyncResult": SimTypeBottom(label="IRtwqAsyncResult"),
    "IRtwqPlatformEvents": SimTypeBottom(label="IRtwqPlatformEvents"),
    "IRulesApplied": SimTypeBottom(label="IRulesApplied"),
    "IRulesAppliedCollection": SimTypeBottom(label="IRulesAppliedCollection"),
    "IRunnableObject": SimTypeBottom(label="IRunnableObject"),
    "IRunnableTask": SimTypeBottom(label="IRunnableTask"),
    "IRunningObjectTable": SimTypeBottom(label="IRunningObjectTable"),
    "IRunningTask": SimTypeBottom(label="IRunningTask"),
    "IRunningTaskCollection": SimTypeBottom(label="IRunningTaskCollection"),
    "ISAXAttributes": SimTypeBottom(label="ISAXAttributes"),
    "ISAXContentHandler": SimTypeBottom(label="ISAXContentHandler"),
    "ISAXDTDHandler": SimTypeBottom(label="ISAXDTDHandler"),
    "ISAXDeclHandler": SimTypeBottom(label="ISAXDeclHandler"),
    "ISAXEntityResolver": SimTypeBottom(label="ISAXEntityResolver"),
    "ISAXErrorHandler": SimTypeBottom(label="ISAXErrorHandler"),
    "ISAXLexicalHandler": SimTypeBottom(label="ISAXLexicalHandler"),
    "ISAXLocator": SimTypeBottom(label="ISAXLocator"),
    "ISAXXMLFilter": SimTypeBottom(label="ISAXXMLFilter"),
    "ISAXXMLReader": SimTypeBottom(label="ISAXXMLReader"),
    "ISBE2Crossbar": SimTypeBottom(label="ISBE2Crossbar"),
    "ISBE2EnumStream": SimTypeBottom(label="ISBE2EnumStream"),
    "ISBE2FileScan": SimTypeBottom(label="ISBE2FileScan"),
    "ISBE2GlobalEvent": SimTypeBottom(label="ISBE2GlobalEvent"),
    "ISBE2GlobalEvent2": SimTypeBottom(label="ISBE2GlobalEvent2"),
    "ISBE2MediaTypeProfile": SimTypeBottom(label="ISBE2MediaTypeProfile"),
    "ISBE2SpanningEvent": SimTypeBottom(label="ISBE2SpanningEvent"),
    "ISBE2StreamMap": SimTypeBottom(label="ISBE2StreamMap"),
    "ISCPSecureAuthenticate": SimTypeBottom(label="ISCPSecureAuthenticate"),
    "ISCPSecureAuthenticate2": SimTypeBottom(label="ISCPSecureAuthenticate2"),
    "ISCPSecureExchange": SimTypeBottom(label="ISCPSecureExchange"),
    "ISCPSecureExchange2": SimTypeBottom(label="ISCPSecureExchange2"),
    "ISCPSecureExchange3": SimTypeBottom(label="ISCPSecureExchange3"),
    "ISCPSecureQuery": SimTypeBottom(label="ISCPSecureQuery"),
    "ISCPSecureQuery2": SimTypeBottom(label="ISCPSecureQuery2"),
    "ISCPSecureQuery3": SimTypeBottom(label="ISCPSecureQuery3"),
    "ISCPSession": SimTypeBottom(label="ISCPSession"),
    "ISCSI_AUTH_TYPES": SimTypeInt(signed=False, label="ISCSI_AUTH_TYPES"),
    "ISCSI_CONNECTION_INFOA": SimStruct(OrderedDict((("ConnectionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("InitiatorAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TargetAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("InitiatorSocket", SimTypeShort(signed=False, label="UInt16")), ("TargetSocket", SimTypeShort(signed=False, label="UInt16")), ("CID", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="ISCSI_CONNECTION_INFOA", pack=False, align=None),
    "ISCSI_CONNECTION_INFOW": SimStruct(OrderedDict((("ConnectionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("InitiatorAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TargetAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InitiatorSocket", SimTypeShort(signed=False, label="UInt16")), ("TargetSocket", SimTypeShort(signed=False, label="UInt16")), ("CID", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="ISCSI_CONNECTION_INFOW", pack=False, align=None),
    "ISCSI_CONNECTION_INFO_EX": SimStruct(OrderedDict((("ConnectionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("State", SimTypeChar(label="Byte")), ("Protocol", SimTypeChar(label="Byte")), ("HeaderDigest", SimTypeChar(label="Byte")), ("DataDigest", SimTypeChar(label="Byte")), ("MaxRecvDataSegmentLength", SimTypeInt(signed=False, label="UInt32")), ("AuthType", SimTypeInt(signed=False, label="ISCSI_AUTH_TYPES")), ("EstimatedThroughput", SimTypeLongLong(signed=False, label="UInt64")), ("MaxDatagramSize", SimTypeInt(signed=False, label="UInt32")),)), name="ISCSI_CONNECTION_INFO_EX", pack=False, align=None),
    "ISCSI_DEVICE_ON_SESSIONA": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("TargetName", SimTypeArray(SimTypeChar(label="SByte"), 224)), ("ScsiAddress", SimTypeRef("SCSI_ADDRESS", SimStruct)), ("DeviceInterfaceType", SimTypeBottom(label="Guid")), ("DeviceInterfaceName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("LegacyName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("StorageDeviceNumber", SimTypeRef("STORAGE_DEVICE_NUMBER", SimStruct)), ("DeviceInstance", SimTypeInt(signed=False, label="UInt32")),)), name="ISCSI_DEVICE_ON_SESSIONA", pack=False, align=None),
    "ISCSI_DEVICE_ON_SESSIONW": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("TargetName", SimTypeArray(SimTypeChar(label="Char"), 224)), ("ScsiAddress", SimTypeRef("SCSI_ADDRESS", SimStruct)), ("DeviceInterfaceType", SimTypeBottom(label="Guid")), ("DeviceInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("LegacyName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("StorageDeviceNumber", SimTypeRef("STORAGE_DEVICE_NUMBER", SimStruct)), ("DeviceInstance", SimTypeInt(signed=False, label="UInt32")),)), name="ISCSI_DEVICE_ON_SESSIONW", pack=False, align=None),
    "ISCSI_DIGEST_TYPES": SimTypeInt(signed=False, label="ISCSI_DIGEST_TYPES"),
    "ISCSI_LOGIN_OPTIONS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("InformationSpecified", SimTypeInt(signed=False, label="UInt32")), ("LoginFlags", SimTypeInt(signed=False, label="UInt32")), ("AuthType", SimTypeInt(signed=False, label="ISCSI_AUTH_TYPES")), ("HeaderDigest", SimTypeInt(signed=False, label="ISCSI_DIGEST_TYPES")), ("DataDigest", SimTypeInt(signed=False, label="ISCSI_DIGEST_TYPES")), ("MaximumConnections", SimTypeInt(signed=False, label="UInt32")), ("DefaultTime2Wait", SimTypeInt(signed=False, label="UInt32")), ("DefaultTime2Retain", SimTypeInt(signed=False, label="UInt32")), ("UsernameLength", SimTypeInt(signed=False, label="UInt32")), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Username", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Password", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ISCSI_LOGIN_OPTIONS", pack=False, align=None),
    "ISCSI_SESSION_INFOA": SimStruct(OrderedDict((("SessionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("InitiatorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TargetNodeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TargetName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ISID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("TSID", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("ConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("Connections", SimTypePointer(SimTypeRef("ISCSI_CONNECTION_INFOA", SimStruct), offset=0)),)), name="ISCSI_SESSION_INFOA", pack=False, align=None),
    "ISCSI_SESSION_INFOW": SimStruct(OrderedDict((("SessionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("InitiatorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TargetNodeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ISID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("TSID", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("ConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("Connections", SimTypePointer(SimTypeRef("ISCSI_CONNECTION_INFOW", SimStruct), offset=0)),)), name="ISCSI_SESSION_INFOW", pack=False, align=None),
    "ISCSI_SESSION_INFO_EX": SimStruct(OrderedDict((("SessionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("InitialR2t", SimTypeChar(label="Byte")), ("ImmediateData", SimTypeChar(label="Byte")), ("Type", SimTypeChar(label="Byte")), ("DataSequenceInOrder", SimTypeChar(label="Byte")), ("DataPduInOrder", SimTypeChar(label="Byte")), ("ErrorRecoveryLevel", SimTypeChar(label="Byte")), ("MaxOutstandingR2t", SimTypeInt(signed=False, label="UInt32")), ("FirstBurstLength", SimTypeInt(signed=False, label="UInt32")), ("MaxBurstLength", SimTypeInt(signed=False, label="UInt32")), ("MaximumConnections", SimTypeInt(signed=False, label="UInt32")), ("ConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("Connections", SimTypePointer(SimTypeRef("ISCSI_CONNECTION_INFO_EX", SimStruct), offset=0)),)), name="ISCSI_SESSION_INFO_EX", pack=False, align=None),
    "ISCSI_TARGET_MAPPINGA": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("TargetName", SimTypeArray(SimTypeChar(label="SByte"), 224)), ("OSDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("SessionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("OSBusNumber", SimTypeInt(signed=False, label="UInt32")), ("OSTargetNumber", SimTypeInt(signed=False, label="UInt32")), ("LUNCount", SimTypeInt(signed=False, label="UInt32")), ("LUNList", SimTypePointer(SimTypeRef("SCSI_LUN_LIST", SimStruct), offset=0)),)), name="ISCSI_TARGET_MAPPINGA", pack=False, align=None),
    "ISCSI_TARGET_MAPPINGW": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("TargetName", SimTypeArray(SimTypeChar(label="Char"), 224)), ("OSDeviceName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("SessionId", SimTypeRef("ISCSI_UNIQUE_SESSION_ID", SimStruct)), ("OSBusNumber", SimTypeInt(signed=False, label="UInt32")), ("OSTargetNumber", SimTypeInt(signed=False, label="UInt32")), ("LUNCount", SimTypeInt(signed=False, label="UInt32")), ("LUNList", SimTypePointer(SimTypeRef("SCSI_LUN_LIST", SimStruct), offset=0)),)), name="ISCSI_TARGET_MAPPINGW", pack=False, align=None),
    "ISCSI_TARGET_PORTALA": SimStruct(OrderedDict((("SymbolicName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Address", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Socket", SimTypeShort(signed=False, label="UInt16")),)), name="ISCSI_TARGET_PORTALA", pack=False, align=None),
    "ISCSI_TARGET_PORTALW": SimStruct(OrderedDict((("SymbolicName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Address", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Socket", SimTypeShort(signed=False, label="UInt16")),)), name="ISCSI_TARGET_PORTALW", pack=False, align=None),
    "ISCSI_TARGET_PORTAL_GROUPA": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Portals", SimTypePointer(SimTypeRef("ISCSI_TARGET_PORTALA", SimStruct), offset=0)),)), name="ISCSI_TARGET_PORTAL_GROUPA", pack=False, align=None),
    "ISCSI_TARGET_PORTAL_GROUPW": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Portals", SimTypePointer(SimTypeRef("ISCSI_TARGET_PORTALW", SimStruct), offset=0)),)), name="ISCSI_TARGET_PORTAL_GROUPW", pack=False, align=None),
    "ISCSI_TARGET_PORTAL_INFOA": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("InitiatorPortNumber", SimTypeInt(signed=False, label="UInt32")), ("SymbolicName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Address", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Socket", SimTypeShort(signed=False, label="UInt16")),)), name="ISCSI_TARGET_PORTAL_INFOA", pack=False, align=None),
    "ISCSI_TARGET_PORTAL_INFOW": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("InitiatorPortNumber", SimTypeInt(signed=False, label="UInt32")), ("SymbolicName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Address", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Socket", SimTypeShort(signed=False, label="UInt16")),)), name="ISCSI_TARGET_PORTAL_INFOW", pack=False, align=None),
    "ISCSI_TARGET_PORTAL_INFO_EXA": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("InitiatorPortNumber", SimTypeInt(signed=False, label="UInt32")), ("SymbolicName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Address", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Socket", SimTypeShort(signed=False, label="UInt16")), ("SecurityFlags", SimTypeLongLong(signed=False, label="UInt64")), ("LoginOptions", SimTypeRef("ISCSI_LOGIN_OPTIONS", SimStruct)),)), name="ISCSI_TARGET_PORTAL_INFO_EXA", pack=False, align=None),
    "ISCSI_TARGET_PORTAL_INFO_EXW": SimStruct(OrderedDict((("InitiatorName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("InitiatorPortNumber", SimTypeInt(signed=False, label="UInt32")), ("SymbolicName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Address", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Socket", SimTypeShort(signed=False, label="UInt16")), ("SecurityFlags", SimTypeLongLong(signed=False, label="UInt64")), ("LoginOptions", SimTypeRef("ISCSI_LOGIN_OPTIONS", SimStruct)),)), name="ISCSI_TARGET_PORTAL_INFO_EXW", pack=False, align=None),
    "ISCSI_UNIQUE_SESSION_ID": SimStruct(OrderedDict((("AdapterUnique", SimTypeLongLong(signed=False, label="UInt64")), ("AdapterSpecific", SimTypeLongLong(signed=False, label="UInt64")),)), name="ISCSI_UNIQUE_SESSION_ID", pack=False, align=None),
    "ISCSI_VERSION_INFO": SimStruct(OrderedDict((("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("BuildNumber", SimTypeInt(signed=False, label="UInt32")),)), name="ISCSI_VERSION_INFO", pack=False, align=None),
    "ISCTE_EAS": SimTypeBottom(label="ISCTE_EAS"),
    "ISC_REQ_FLAGS": SimTypeInt(signed=False, label="ISC_REQ_FLAGS"),
    "ISC_REQ_HIGH_FLAGS": SimTypeInt(signed=False, label="ISC_REQ_HIGH_FLAGS"),
    "ISClusApplication": SimTypeBottom(label="ISClusApplication"),
    "ISClusCryptoKeys": SimTypeBottom(label="ISClusCryptoKeys"),
    "ISClusDisk": SimTypeBottom(label="ISClusDisk"),
    "ISClusDisks": SimTypeBottom(label="ISClusDisks"),
    "ISClusNetInterface": SimTypeBottom(label="ISClusNetInterface"),
    "ISClusNetInterfaces": SimTypeBottom(label="ISClusNetInterfaces"),
    "ISClusNetwork": SimTypeBottom(label="ISClusNetwork"),
    "ISClusNetworkNetInterfaces": SimTypeBottom(label="ISClusNetworkNetInterfaces"),
    "ISClusNetworks": SimTypeBottom(label="ISClusNetworks"),
    "ISClusNode": SimTypeBottom(label="ISClusNode"),
    "ISClusNodeNetInterfaces": SimTypeBottom(label="ISClusNodeNetInterfaces"),
    "ISClusNodes": SimTypeBottom(label="ISClusNodes"),
    "ISClusPartition": SimTypeBottom(label="ISClusPartition"),
    "ISClusPartitionEx": SimTypeBottom(label="ISClusPartitionEx"),
    "ISClusPartitions": SimTypeBottom(label="ISClusPartitions"),
    "ISClusProperties": SimTypeBottom(label="ISClusProperties"),
    "ISClusProperty": SimTypeBottom(label="ISClusProperty"),
    "ISClusPropertyValue": SimTypeBottom(label="ISClusPropertyValue"),
    "ISClusPropertyValueData": SimTypeBottom(label="ISClusPropertyValueData"),
    "ISClusPropertyValues": SimTypeBottom(label="ISClusPropertyValues"),
    "ISClusRefObject": SimTypeBottom(label="ISClusRefObject"),
    "ISClusRegistryKeys": SimTypeBottom(label="ISClusRegistryKeys"),
    "ISClusResDependencies": SimTypeBottom(label="ISClusResDependencies"),
    "ISClusResDependents": SimTypeBottom(label="ISClusResDependents"),
    "ISClusResGroup": SimTypeBottom(label="ISClusResGroup"),
    "ISClusResGroupPreferredOwnerNodes": SimTypeBottom(label="ISClusResGroupPreferredOwnerNodes"),
    "ISClusResGroupResources": SimTypeBottom(label="ISClusResGroupResources"),
    "ISClusResGroups": SimTypeBottom(label="ISClusResGroups"),
    "ISClusResPossibleOwnerNodes": SimTypeBottom(label="ISClusResPossibleOwnerNodes"),
    "ISClusResType": SimTypeBottom(label="ISClusResType"),
    "ISClusResTypePossibleOwnerNodes": SimTypeBottom(label="ISClusResTypePossibleOwnerNodes"),
    "ISClusResTypeResources": SimTypeBottom(label="ISClusResTypeResources"),
    "ISClusResTypes": SimTypeBottom(label="ISClusResTypes"),
    "ISClusResource": SimTypeBottom(label="ISClusResource"),
    "ISClusResources": SimTypeBottom(label="ISClusResources"),
    "ISClusScsiAddress": SimTypeBottom(label="ISClusScsiAddress"),
    "ISClusVersion": SimTypeBottom(label="ISClusVersion"),
    "ISCluster": SimTypeBottom(label="ISCluster"),
    "ISClusterNames": SimTypeBottom(label="ISClusterNames"),
    "ISDBCAS_REQUEST_ID": SimTypeInt(signed=False, label="ISDBCAS_REQUEST_ID"),
    "ISDomainNames": SimTypeBottom(label="ISDomainNames"),
    "ISIInbandEPG": SimTypeBottom(label="ISIInbandEPG"),
    "ISIInbandEPGEvent": SimTypeBottom(label="ISIInbandEPGEvent"),
    "ISOFLAG": SimTypeInt(signed=False, label="ISOFLAG"),
    "ISOLATIONLEVEL": SimTypeInt(signed=False, label="ISOLATIONLEVEL"),
    "ISOLATION_STATE": SimTypeInt(signed=False, label="ISOLATION_STATE"),
    "ISQLErrorInfo": SimTypeBottom(label="ISQLErrorInfo"),
    "ISQLGetDiagField": SimTypeBottom(label="ISQLGetDiagField"),
    "ISQLRequestDiagFields": SimTypeBottom(label="ISQLRequestDiagFields"),
    "ISQLServerErrorInfo": SimTypeBottom(label="ISQLServerErrorInfo"),
    "ISVGAElement": SimTypeBottom(label="ISVGAElement"),
    "ISVGAngle": SimTypeBottom(label="ISVGAngle"),
    "ISVGAnimatedAngle": SimTypeBottom(label="ISVGAnimatedAngle"),
    "ISVGAnimatedBoolean": SimTypeBottom(label="ISVGAnimatedBoolean"),
    "ISVGAnimatedEnumeration": SimTypeBottom(label="ISVGAnimatedEnumeration"),
    "ISVGAnimatedInteger": SimTypeBottom(label="ISVGAnimatedInteger"),
    "ISVGAnimatedLength": SimTypeBottom(label="ISVGAnimatedLength"),
    "ISVGAnimatedLengthList": SimTypeBottom(label="ISVGAnimatedLengthList"),
    "ISVGAnimatedNumber": SimTypeBottom(label="ISVGAnimatedNumber"),
    "ISVGAnimatedNumberList": SimTypeBottom(label="ISVGAnimatedNumberList"),
    "ISVGAnimatedPathData": SimTypeBottom(label="ISVGAnimatedPathData"),
    "ISVGAnimatedPoints": SimTypeBottom(label="ISVGAnimatedPoints"),
    "ISVGAnimatedPreserveAspectRatio": SimTypeBottom(label="ISVGAnimatedPreserveAspectRatio"),
    "ISVGAnimatedRect": SimTypeBottom(label="ISVGAnimatedRect"),
    "ISVGAnimatedString": SimTypeBottom(label="ISVGAnimatedString"),
    "ISVGAnimatedTransformList": SimTypeBottom(label="ISVGAnimatedTransformList"),
    "ISVGCircleElement": SimTypeBottom(label="ISVGCircleElement"),
    "ISVGClipPathElement": SimTypeBottom(label="ISVGClipPathElement"),
    "ISVGDefsElement": SimTypeBottom(label="ISVGDefsElement"),
    "ISVGDescElement": SimTypeBottom(label="ISVGDescElement"),
    "ISVGDocument": SimTypeBottom(label="ISVGDocument"),
    "ISVGElement": SimTypeBottom(label="ISVGElement"),
    "ISVGElementInstance": SimTypeBottom(label="ISVGElementInstance"),
    "ISVGElementInstanceList": SimTypeBottom(label="ISVGElementInstanceList"),
    "ISVGEllipseElement": SimTypeBottom(label="ISVGEllipseElement"),
    "ISVGException": SimTypeBottom(label="ISVGException"),
    "ISVGExternalResourcesRequired": SimTypeBottom(label="ISVGExternalResourcesRequired"),
    "ISVGFitToViewBox": SimTypeBottom(label="ISVGFitToViewBox"),
    "ISVGGElement": SimTypeBottom(label="ISVGGElement"),
    "ISVGGradientElement": SimTypeBottom(label="ISVGGradientElement"),
    "ISVGImageElement": SimTypeBottom(label="ISVGImageElement"),
    "ISVGLangSpace": SimTypeBottom(label="ISVGLangSpace"),
    "ISVGLength": SimTypeBottom(label="ISVGLength"),
    "ISVGLengthList": SimTypeBottom(label="ISVGLengthList"),
    "ISVGLineElement": SimTypeBottom(label="ISVGLineElement"),
    "ISVGLinearGradientElement": SimTypeBottom(label="ISVGLinearGradientElement"),
    "ISVGLocatable": SimTypeBottom(label="ISVGLocatable"),
    "ISVGMarkerElement": SimTypeBottom(label="ISVGMarkerElement"),
    "ISVGMaskElement": SimTypeBottom(label="ISVGMaskElement"),
    "ISVGMatrix": SimTypeBottom(label="ISVGMatrix"),
    "ISVGMetadataElement": SimTypeBottom(label="ISVGMetadataElement"),
    "ISVGNumber": SimTypeBottom(label="ISVGNumber"),
    "ISVGNumberList": SimTypeBottom(label="ISVGNumberList"),
    "ISVGPaint": SimTypeBottom(label="ISVGPaint"),
    "ISVGPathElement": SimTypeBottom(label="ISVGPathElement"),
    "ISVGPathSeg": SimTypeBottom(label="ISVGPathSeg"),
    "ISVGPathSegArcAbs": SimTypeBottom(label="ISVGPathSegArcAbs"),
    "ISVGPathSegArcRel": SimTypeBottom(label="ISVGPathSegArcRel"),
    "ISVGPathSegClosePath": SimTypeBottom(label="ISVGPathSegClosePath"),
    "ISVGPathSegCurvetoCubicAbs": SimTypeBottom(label="ISVGPathSegCurvetoCubicAbs"),
    "ISVGPathSegCurvetoCubicRel": SimTypeBottom(label="ISVGPathSegCurvetoCubicRel"),
    "ISVGPathSegCurvetoCubicSmoothAbs": SimTypeBottom(label="ISVGPathSegCurvetoCubicSmoothAbs"),
    "ISVGPathSegCurvetoCubicSmoothRel": SimTypeBottom(label="ISVGPathSegCurvetoCubicSmoothRel"),
    "ISVGPathSegCurvetoQuadraticAbs": SimTypeBottom(label="ISVGPathSegCurvetoQuadraticAbs"),
    "ISVGPathSegCurvetoQuadraticRel": SimTypeBottom(label="ISVGPathSegCurvetoQuadraticRel"),
    "ISVGPathSegCurvetoQuadraticSmoothAbs": SimTypeBottom(label="ISVGPathSegCurvetoQuadraticSmoothAbs"),
    "ISVGPathSegCurvetoQuadraticSmoothRel": SimTypeBottom(label="ISVGPathSegCurvetoQuadraticSmoothRel"),
    "ISVGPathSegLinetoAbs": SimTypeBottom(label="ISVGPathSegLinetoAbs"),
    "ISVGPathSegLinetoHorizontalAbs": SimTypeBottom(label="ISVGPathSegLinetoHorizontalAbs"),
    "ISVGPathSegLinetoHorizontalRel": SimTypeBottom(label="ISVGPathSegLinetoHorizontalRel"),
    "ISVGPathSegLinetoRel": SimTypeBottom(label="ISVGPathSegLinetoRel"),
    "ISVGPathSegLinetoVerticalAbs": SimTypeBottom(label="ISVGPathSegLinetoVerticalAbs"),
    "ISVGPathSegLinetoVerticalRel": SimTypeBottom(label="ISVGPathSegLinetoVerticalRel"),
    "ISVGPathSegList": SimTypeBottom(label="ISVGPathSegList"),
    "ISVGPathSegMovetoAbs": SimTypeBottom(label="ISVGPathSegMovetoAbs"),
    "ISVGPathSegMovetoRel": SimTypeBottom(label="ISVGPathSegMovetoRel"),
    "ISVGPatternElement": SimTypeBottom(label="ISVGPatternElement"),
    "ISVGPoint": SimTypeBottom(label="ISVGPoint"),
    "ISVGPointList": SimTypeBottom(label="ISVGPointList"),
    "ISVGPolygonElement": SimTypeBottom(label="ISVGPolygonElement"),
    "ISVGPolylineElement": SimTypeBottom(label="ISVGPolylineElement"),
    "ISVGPreserveAspectRatio": SimTypeBottom(label="ISVGPreserveAspectRatio"),
    "ISVGRadialGradientElement": SimTypeBottom(label="ISVGRadialGradientElement"),
    "ISVGRect": SimTypeBottom(label="ISVGRect"),
    "ISVGRectElement": SimTypeBottom(label="ISVGRectElement"),
    "ISVGSVGElement": SimTypeBottom(label="ISVGSVGElement"),
    "ISVGScriptElement": SimTypeBottom(label="ISVGScriptElement"),
    "ISVGStopElement": SimTypeBottom(label="ISVGStopElement"),
    "ISVGStringList": SimTypeBottom(label="ISVGStringList"),
    "ISVGStylable": SimTypeBottom(label="ISVGStylable"),
    "ISVGStyleElement": SimTypeBottom(label="ISVGStyleElement"),
    "ISVGSwitchElement": SimTypeBottom(label="ISVGSwitchElement"),
    "ISVGSymbolElement": SimTypeBottom(label="ISVGSymbolElement"),
    "ISVGTSpanElement": SimTypeBottom(label="ISVGTSpanElement"),
    "ISVGTests": SimTypeBottom(label="ISVGTests"),
    "ISVGTextContentElement": SimTypeBottom(label="ISVGTextContentElement"),
    "ISVGTextElement": SimTypeBottom(label="ISVGTextElement"),
    "ISVGTextPathElement": SimTypeBottom(label="ISVGTextPathElement"),
    "ISVGTextPositioningElement": SimTypeBottom(label="ISVGTextPositioningElement"),
    "ISVGTitleElement": SimTypeBottom(label="ISVGTitleElement"),
    "ISVGTransform": SimTypeBottom(label="ISVGTransform"),
    "ISVGTransformList": SimTypeBottom(label="ISVGTransformList"),
    "ISVGTransformable": SimTypeBottom(label="ISVGTransformable"),
    "ISVGURIReference": SimTypeBottom(label="ISVGURIReference"),
    "ISVGUseElement": SimTypeBottom(label="ISVGUseElement"),
    "ISVGViewElement": SimTypeBottom(label="ISVGViewElement"),
    "ISVGViewSpec": SimTypeBottom(label="ISVGViewSpec"),
    "ISVGZoomAndPan": SimTypeBottom(label="ISVGZoomAndPan"),
    "ISVGZoomEvent": SimTypeBottom(label="ISVGZoomEvent"),
    "ISWbemDateTime": SimTypeBottom(label="ISWbemDateTime"),
    "ISWbemEventSource": SimTypeBottom(label="ISWbemEventSource"),
    "ISWbemLastError": SimTypeBottom(label="ISWbemLastError"),
    "ISWbemLocator": SimTypeBottom(label="ISWbemLocator"),
    "ISWbemMethod": SimTypeBottom(label="ISWbemMethod"),
    "ISWbemMethodSet": SimTypeBottom(label="ISWbemMethodSet"),
    "ISWbemNamedValue": SimTypeBottom(label="ISWbemNamedValue"),
    "ISWbemNamedValueSet": SimTypeBottom(label="ISWbemNamedValueSet"),
    "ISWbemObject": SimTypeBottom(label="ISWbemObject"),
    "ISWbemObjectEx": SimTypeBottom(label="ISWbemObjectEx"),
    "ISWbemObjectPath": SimTypeBottom(label="ISWbemObjectPath"),
    "ISWbemObjectSet": SimTypeBottom(label="ISWbemObjectSet"),
    "ISWbemPrivilege": SimTypeBottom(label="ISWbemPrivilege"),
    "ISWbemPrivilegeSet": SimTypeBottom(label="ISWbemPrivilegeSet"),
    "ISWbemProperty": SimTypeBottom(label="ISWbemProperty"),
    "ISWbemPropertySet": SimTypeBottom(label="ISWbemPropertySet"),
    "ISWbemQualifier": SimTypeBottom(label="ISWbemQualifier"),
    "ISWbemQualifierSet": SimTypeBottom(label="ISWbemQualifierSet"),
    "ISWbemRefreshableItem": SimTypeBottom(label="ISWbemRefreshableItem"),
    "ISWbemRefresher": SimTypeBottom(label="ISWbemRefresher"),
    "ISWbemSecurity": SimTypeBottom(label="ISWbemSecurity"),
    "ISWbemServices": SimTypeBottom(label="ISWbemServices"),
    "ISWbemServicesEx": SimTypeBottom(label="ISWbemServicesEx"),
    "ISWbemSink": SimTypeBottom(label="ISWbemSink"),
    "ISWbemSinkEvents": SimTypeBottom(label="ISWbemSinkEvents"),
    "IS_ADSPEC_BODY": SimStruct(OrderedDict((("adspec_mh", SimTypeRef("IntServMainHdr", SimStruct)), ("adspec_genparms", SimTypeRef("GenAdspecParams", SimStruct)),)), name="IS_ADSPEC_BODY", pack=False, align=None),
    "IS_FLOWSPEC": SimStruct(OrderedDict((("flow_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("flow_body", SimTypeRef("IntServFlowSpec", SimStruct)),)), name="IS_FLOWSPEC", pack=False, align=None),
    "IS_TEXT_UNICODE_RESULT": SimTypeInt(signed=False, label="IS_TEXT_UNICODE_RESULT"),
    "IS_VALID_LOCALE_FLAGS": SimTypeInt(signed=False, label="IS_VALID_LOCALE_FLAGS"),
    "IScanningTuner": SimTypeBottom(label="IScanningTuner"),
    "IScanningTunerEx": SimTypeBottom(label="IScanningTunerEx"),
    "ISceSvcAttachmentData": SimTypeBottom(label="ISceSvcAttachmentData"),
    "ISceSvcAttachmentPersistInfo": SimTypeBottom(label="ISceSvcAttachmentPersistInfo"),
    "ISchedule": SimTypeBottom(label="ISchedule"),
    "IScheduleCollection": SimTypeBottom(label="IScheduleCollection"),
    "IScheduledWorkItem": SimTypeBottom(label="IScheduledWorkItem"),
    "ISchema": SimTypeBottom(label="ISchema"),
    "ISchemaAny": SimTypeBottom(label="ISchemaAny"),
    "ISchemaAttribute": SimTypeBottom(label="ISchemaAttribute"),
    "ISchemaAttributeGroup": SimTypeBottom(label="ISchemaAttributeGroup"),
    "ISchemaComplexType": SimTypeBottom(label="ISchemaComplexType"),
    "ISchemaElement": SimTypeBottom(label="ISchemaElement"),
    "ISchemaIdentityConstraint": SimTypeBottom(label="ISchemaIdentityConstraint"),
    "ISchemaItem": SimTypeBottom(label="ISchemaItem"),
    "ISchemaItemCollection": SimTypeBottom(label="ISchemaItemCollection"),
    "ISchemaLocalizerSupport": SimTypeBottom(label="ISchemaLocalizerSupport"),
    "ISchemaLock": SimTypeBottom(label="ISchemaLock"),
    "ISchemaModelGroup": SimTypeBottom(label="ISchemaModelGroup"),
    "ISchemaNotation": SimTypeBottom(label="ISchemaNotation"),
    "ISchemaParticle": SimTypeBottom(label="ISchemaParticle"),
    "ISchemaProvider": SimTypeBottom(label="ISchemaProvider"),
    "ISchemaStringCollection": SimTypeBottom(label="ISchemaStringCollection"),
    "ISchemaType": SimTypeBottom(label="ISchemaType"),
    "IScopedOperations": SimTypeBottom(label="IScopedOperations"),
    "IScriptEntry": SimTypeBottom(label="IScriptEntry"),
    "IScriptErrorList": SimTypeBottom(label="IScriptErrorList"),
    "IScriptEventHandler": SimTypeBottom(label="IScriptEventHandler"),
    "IScriptEventHandlerSourceInfo": SimTypeBottom(label="IScriptEventHandlerSourceInfo"),
    "IScriptInvocationContext": SimTypeBottom(label="IScriptInvocationContext"),
    "IScriptNode": SimTypeBottom(label="IScriptNode"),
    "IScriptScriptlet": SimTypeBottom(label="IScriptScriptlet"),
    "IScrollItemProvider": SimTypeBottom(label="IScrollItemProvider"),
    "IScrollProvider": SimTypeBottom(label="IScrollProvider"),
    "IScrollableContextMenu": SimTypeBottom(label="IScrollableContextMenu"),
    "IScrollableContextMenu2": SimTypeBottom(label="IScrollableContextMenu2"),
    "ISdo": SimTypeBottom(label="ISdo"),
    "ISdoCollection": SimTypeBottom(label="ISdoCollection"),
    "ISdoDictionaryOld": SimTypeBottom(label="ISdoDictionaryOld"),
    "ISdoMachine": SimTypeBottom(label="ISdoMachine"),
    "ISdoMachine2": SimTypeBottom(label="ISdoMachine2"),
    "ISdoServiceControl": SimTypeBottom(label="ISdoServiceControl"),
    "ISearchBoxInfo": SimTypeBottom(label="ISearchBoxInfo"),
    "ISearchCatalogManager": SimTypeBottom(label="ISearchCatalogManager"),
    "ISearchCatalogManager2": SimTypeBottom(label="ISearchCatalogManager2"),
    "ISearchCompletedCallback": SimTypeBottom(label="ISearchCompletedCallback"),
    "ISearchCompletedCallbackArgs": SimTypeBottom(label="ISearchCompletedCallbackArgs"),
    "ISearchContext": SimTypeBottom(label="ISearchContext"),
    "ISearchCrawlScopeManager": SimTypeBottom(label="ISearchCrawlScopeManager"),
    "ISearchCrawlScopeManager2": SimTypeBottom(label="ISearchCrawlScopeManager2"),
    "ISearchFolderItemFactory": SimTypeBottom(label="ISearchFolderItemFactory"),
    "ISearchItemsChangedSink": SimTypeBottom(label="ISearchItemsChangedSink"),
    "ISearchJob": SimTypeBottom(label="ISearchJob"),
    "ISearchLanguageSupport": SimTypeBottom(label="ISearchLanguageSupport"),
    "ISearchManager": SimTypeBottom(label="ISearchManager"),
    "ISearchManager2": SimTypeBottom(label="ISearchManager2"),
    "ISearchNotifyInlineSite": SimTypeBottom(label="ISearchNotifyInlineSite"),
    "ISearchPersistentItemsChangedSink": SimTypeBottom(label="ISearchPersistentItemsChangedSink"),
    "ISearchProtocol": SimTypeBottom(label="ISearchProtocol"),
    "ISearchProtocol2": SimTypeBottom(label="ISearchProtocol2"),
    "ISearchProtocolThreadContext": SimTypeBottom(label="ISearchProtocolThreadContext"),
    "ISearchQueryHelper": SimTypeBottom(label="ISearchQueryHelper"),
    "ISearchQueryHits": SimTypeBottom(label="ISearchQueryHits"),
    "ISearchResult": SimTypeBottom(label="ISearchResult"),
    "ISearchRoot": SimTypeBottom(label="ISearchRoot"),
    "ISearchScopeRule": SimTypeBottom(label="ISearchScopeRule"),
    "ISearchViewChangedSink": SimTypeBottom(label="ISearchViewChangedSink"),
    "ISectionList": SimTypeBottom(label="ISectionList"),
    "ISecureUrlHost": SimTypeBottom(label="ISecureUrlHost"),
    "ISecurityCallContext": SimTypeBottom(label="ISecurityCallContext"),
    "ISecurityCallersColl": SimTypeBottom(label="ISecurityCallersColl"),
    "ISecurityIdentityColl": SimTypeBottom(label="ISecurityIdentityColl"),
    "ISecurityInfo": SimTypeBottom(label="ISecurityInfo"),
    "ISecurityInformation": SimTypeBottom(label="ISecurityInformation"),
    "ISecurityInformation2": SimTypeBottom(label="ISecurityInformation2"),
    "ISecurityInformation3": SimTypeBottom(label="ISecurityInformation3"),
    "ISecurityInformation4": SimTypeBottom(label="ISecurityInformation4"),
    "ISecurityObjectTypeInfo": SimTypeBottom(label="ISecurityObjectTypeInfo"),
    "ISecurityProperty": SimTypeBottom(label="ISecurityProperty"),
    "ISeekingPassThru": SimTypeBottom(label="ISeekingPassThru"),
    "ISegment": SimTypeBottom(label="ISegment"),
    "ISegmentList": SimTypeBottom(label="ISegmentList"),
    "ISegmentListIterator": SimTypeBottom(label="ISegmentListIterator"),
    "ISelectCOMLBServer": SimTypeBottom(label="ISelectCOMLBServer"),
    "ISelectionItemProvider": SimTypeBottom(label="ISelectionItemProvider"),
    "ISelectionProvider": SimTypeBottom(label="ISelectionProvider"),
    "ISelectionProvider2": SimTypeBottom(label="ISelectionProvider2"),
    "ISelectionServices": SimTypeBottom(label="ISelectionServices"),
    "ISelectionServicesListener": SimTypeBottom(label="ISelectionServicesListener"),
    "ISelector": SimTypeBottom(label="ISelector"),
    "ISendMethodEvents": SimTypeBottom(label="ISendMethodEvents"),
    "ISensLogon": SimTypeBottom(label="ISensLogon"),
    "ISensLogon2": SimTypeBottom(label="ISensLogon2"),
    "ISensNetwork": SimTypeBottom(label="ISensNetwork"),
    "ISensOnNow": SimTypeBottom(label="ISensOnNow"),
    "ISensor": SimTypeBottom(label="ISensor"),
    "ISensorCollection": SimTypeBottom(label="ISensorCollection"),
    "ISensorDataReport": SimTypeBottom(label="ISensorDataReport"),
    "ISensorEvents": SimTypeBottom(label="ISensorEvents"),
    "ISensorManager": SimTypeBottom(label="ISensorManager"),
    "ISensorManagerEvents": SimTypeBottom(label="ISensorManagerEvents"),
    "ISequenceNumber": SimTypeBottom(label="ISequenceNumber"),
    "ISequentialStream": SimTypeBottom(label="ISequentialStream"),
    "IServerSecurity": SimTypeBottom(label="IServerSecurity"),
    "IServerXMLHTTPRequest": SimTypeBottom(label="IServerXMLHTTPRequest"),
    "IServerXMLHTTPRequest2": SimTypeBottom(label="IServerXMLHTTPRequest2"),
    "IService": SimTypeBottom(label="IService"),
    "IServiceActivity": SimTypeBottom(label="IServiceActivity"),
    "IServiceCall": SimTypeBottom(label="IServiceCall"),
    "IServiceComTIIntrinsicsConfig": SimTypeBottom(label="IServiceComTIIntrinsicsConfig"),
    "IServiceIISIntrinsicsConfig": SimTypeBottom(label="IServiceIISIntrinsicsConfig"),
    "IServiceInheritanceConfig": SimTypeBottom(label="IServiceInheritanceConfig"),
    "IServiceLocationDescriptor": SimTypeBottom(label="IServiceLocationDescriptor"),
    "IServicePartitionConfig": SimTypeBottom(label="IServicePartitionConfig"),
    "IServicePool": SimTypeBottom(label="IServicePool"),
    "IServicePoolConfig": SimTypeBottom(label="IServicePoolConfig"),
    "IServiceProvider": SimTypeBottom(label="IServiceProvider"),
    "IServiceSxsConfig": SimTypeBottom(label="IServiceSxsConfig"),
    "IServiceSynchronizationConfig": SimTypeBottom(label="IServiceSynchronizationConfig"),
    "IServiceSysTxnConfig": SimTypeBottom(label="IServiceSysTxnConfig"),
    "IServiceThreadPoolConfig": SimTypeBottom(label="IServiceThreadPoolConfig"),
    "IServiceTrackerConfig": SimTypeBottom(label="IServiceTrackerConfig"),
    "IServiceTransactionConfig": SimTypeBottom(label="IServiceTransactionConfig"),
    "IServiceTransactionConfigBase": SimTypeBottom(label="IServiceTransactionConfigBase"),
    "ISessionProperties": SimTypeBottom(label="ISessionProperties"),
    "ISessionStateChangeTrigger": SimTypeBottom(label="ISessionStateChangeTrigger"),
    "ISettingsContext": SimTypeBottom(label="ISettingsContext"),
    "ISettingsEngine": SimTypeBottom(label="ISettingsEngine"),
    "ISettingsIdentity": SimTypeBottom(label="ISettingsIdentity"),
    "ISettingsItem": SimTypeBottom(label="ISettingsItem"),
    "ISettingsNamespace": SimTypeBottom(label="ISettingsNamespace"),
    "ISettingsResult": SimTypeBottom(label="ISettingsResult"),
    "ISharedBitmap": SimTypeBottom(label="ISharedBitmap"),
    "ISharedProperty": SimTypeBottom(label="ISharedProperty"),
    "ISharedPropertyGroup": SimTypeBottom(label="ISharedPropertyGroup"),
    "ISharedPropertyGroupManager": SimTypeBottom(label="ISharedPropertyGroupManager"),
    "ISharingConfigurationManager": SimTypeBottom(label="ISharingConfigurationManager"),
    "IShellApp": SimTypeBottom(label="IShellApp"),
    "IShellBrowser": SimTypeBottom(label="IShellBrowser"),
    "IShellChangeNotify": SimTypeBottom(label="IShellChangeNotify"),
    "IShellDetails": SimTypeBottom(label="IShellDetails"),
    "IShellDispatch": SimTypeBottom(label="IShellDispatch"),
    "IShellDispatch2": SimTypeBottom(label="IShellDispatch2"),
    "IShellDispatch3": SimTypeBottom(label="IShellDispatch3"),
    "IShellDispatch4": SimTypeBottom(label="IShellDispatch4"),
    "IShellDispatch5": SimTypeBottom(label="IShellDispatch5"),
    "IShellDispatch6": SimTypeBottom(label="IShellDispatch6"),
    "IShellExtInit": SimTypeBottom(label="IShellExtInit"),
    "IShellFavoritesNameSpace": SimTypeBottom(label="IShellFavoritesNameSpace"),
    "IShellFolder": SimTypeBottom(label="IShellFolder"),
    "IShellFolder2": SimTypeBottom(label="IShellFolder2"),
    "IShellFolderBand": SimTypeBottom(label="IShellFolderBand"),
    "IShellFolderView": SimTypeBottom(label="IShellFolderView"),
    "IShellFolderViewCB": SimTypeBottom(label="IShellFolderViewCB"),
    "IShellFolderViewDual": SimTypeBottom(label="IShellFolderViewDual"),
    "IShellFolderViewDual2": SimTypeBottom(label="IShellFolderViewDual2"),
    "IShellFolderViewDual3": SimTypeBottom(label="IShellFolderViewDual3"),
    "IShellIcon": SimTypeBottom(label="IShellIcon"),
    "IShellIconOverlay": SimTypeBottom(label="IShellIconOverlay"),
    "IShellIconOverlayIdentifier": SimTypeBottom(label="IShellIconOverlayIdentifier"),
    "IShellIconOverlayManager": SimTypeBottom(label="IShellIconOverlayManager"),
    "IShellImageData": SimTypeBottom(label="IShellImageData"),
    "IShellImageDataAbort": SimTypeBottom(label="IShellImageDataAbort"),
    "IShellImageDataFactory": SimTypeBottom(label="IShellImageDataFactory"),
    "IShellItem": SimTypeBottom(label="IShellItem"),
    "IShellItem2": SimTypeBottom(label="IShellItem2"),
    "IShellItemArray": SimTypeBottom(label="IShellItemArray"),
    "IShellItemFilter": SimTypeBottom(label="IShellItemFilter"),
    "IShellItemImageFactory": SimTypeBottom(label="IShellItemImageFactory"),
    "IShellItemResources": SimTypeBottom(label="IShellItemResources"),
    "IShellLibrary": SimTypeBottom(label="IShellLibrary"),
    "IShellLinkA": SimTypeBottom(label="IShellLinkA"),
    "IShellLinkDataList": SimTypeBottom(label="IShellLinkDataList"),
    "IShellLinkDual": SimTypeBottom(label="IShellLinkDual"),
    "IShellLinkDual2": SimTypeBottom(label="IShellLinkDual2"),
    "IShellLinkW": SimTypeBottom(label="IShellLinkW"),
    "IShellMenu": SimTypeBottom(label="IShellMenu"),
    "IShellMenuCallback": SimTypeBottom(label="IShellMenuCallback"),
    "IShellNameSpace": SimTypeBottom(label="IShellNameSpace"),
    "IShellPropSheetExt": SimTypeBottom(label="IShellPropSheetExt"),
    "IShellRunDll": SimTypeBottom(label="IShellRunDll"),
    "IShellService": SimTypeBottom(label="IShellService"),
    "IShellTaskScheduler": SimTypeBottom(label="IShellTaskScheduler"),
    "IShellUIHelper": SimTypeBottom(label="IShellUIHelper"),
    "IShellUIHelper2": SimTypeBottom(label="IShellUIHelper2"),
    "IShellUIHelper3": SimTypeBottom(label="IShellUIHelper3"),
    "IShellUIHelper4": SimTypeBottom(label="IShellUIHelper4"),
    "IShellUIHelper5": SimTypeBottom(label="IShellUIHelper5"),
    "IShellUIHelper6": SimTypeBottom(label="IShellUIHelper6"),
    "IShellUIHelper7": SimTypeBottom(label="IShellUIHelper7"),
    "IShellUIHelper8": SimTypeBottom(label="IShellUIHelper8"),
    "IShellUIHelper9": SimTypeBottom(label="IShellUIHelper9"),
    "IShellView": SimTypeBottom(label="IShellView"),
    "IShellView2": SimTypeBottom(label="IShellView2"),
    "IShellView3": SimTypeBottom(label="IShellView3"),
    "IShellWindows": SimTypeBottom(label="IShellWindows"),
    "IShowMessageAction": SimTypeBottom(label="IShowMessageAction"),
    "ISideShowBulkCapabilities": SimTypeBottom(label="ISideShowBulkCapabilities"),
    "ISideShowCapabilities": SimTypeBottom(label="ISideShowCapabilities"),
    "ISideShowCapabilitiesCollection": SimTypeBottom(label="ISideShowCapabilitiesCollection"),
    "ISideShowContent": SimTypeBottom(label="ISideShowContent"),
    "ISideShowContentManager": SimTypeBottom(label="ISideShowContentManager"),
    "ISideShowEvents": SimTypeBottom(label="ISideShowEvents"),
    "ISideShowKeyCollection": SimTypeBottom(label="ISideShowKeyCollection"),
    "ISideShowNotification": SimTypeBottom(label="ISideShowNotification"),
    "ISideShowNotificationManager": SimTypeBottom(label="ISideShowNotificationManager"),
    "ISideShowPropVariantCollection": SimTypeBottom(label="ISideShowPropVariantCollection"),
    "ISideShowSession": SimTypeBottom(label="ISideShowSession"),
    "ISignerCertificate": SimTypeBottom(label="ISignerCertificate"),
    "ISignerCertificates": SimTypeBottom(label="ISignerCertificates"),
    "ISimilarity": SimTypeBottom(label="ISimilarity"),
    "ISimilarityFileIdTable": SimTypeBottom(label="ISimilarityFileIdTable"),
    "ISimilarityReportProgress": SimTypeBottom(label="ISimilarityReportProgress"),
    "ISimilarityTableDumpState": SimTypeBottom(label="ISimilarityTableDumpState"),
    "ISimilarityTraitsMappedView": SimTypeBottom(label="ISimilarityTraitsMappedView"),
    "ISimilarityTraitsMapping": SimTypeBottom(label="ISimilarityTraitsMapping"),
    "ISimilarityTraitsTable": SimTypeBottom(label="ISimilarityTraitsTable"),
    "ISimpleAudioVolume": SimTypeBottom(label="ISimpleAudioVolume"),
    "ISimpleCommandCreator": SimTypeBottom(label="ISimpleCommandCreator"),
    "ISimpleConnectionPoint": SimTypeBottom(label="ISimpleConnectionPoint"),
    "ISimpleFrameSite": SimTypeBottom(label="ISimpleFrameSite"),
    "ISingleItemException": SimTypeBottom(label="ISingleItemException"),
    "ISketchInk": SimTypeBottom(label="ISketchInk"),
    "ISmimeCapabilities": SimTypeBottom(label="ISmimeCapabilities"),
    "ISmimeCapability": SimTypeBottom(label="ISmimeCapability"),
    "ISnapinAbout": SimTypeBottom(label="ISnapinAbout"),
    "ISnapinHelp": SimTypeBottom(label="ISnapinHelp"),
    "ISnapinHelp2": SimTypeBottom(label="ISnapinHelp2"),
    "ISnapinProperties": SimTypeBottom(label="ISnapinProperties"),
    "ISnapinPropertiesCallback": SimTypeBottom(label="ISnapinPropertiesCallback"),
    "ISniffStream": SimTypeBottom(label="ISniffStream"),
    "ISoftDistExt": SimTypeBottom(label="ISoftDistExt"),
    "ISortColumnArray": SimTypeBottom(label="ISortColumnArray"),
    "ISourcesRowset": SimTypeBottom(label="ISourcesRowset"),
    "ISpAudio": SimTypeBottom(label="ISpAudio"),
    "ISpCFGInterpreter": SimTypeBottom(label="ISpCFGInterpreter"),
    "ISpCFGInterpreterSite": SimTypeBottom(label="ISpCFGInterpreterSite"),
    "ISpContainerLexicon": SimTypeBottom(label="ISpContainerLexicon"),
    "ISpDataKey": SimTypeBottom(label="ISpDataKey"),
    "ISpDisplayAlternates": SimTypeBottom(label="ISpDisplayAlternates"),
    "ISpEnginePronunciation": SimTypeBottom(label="ISpEnginePronunciation"),
    "ISpErrorLog": SimTypeBottom(label="ISpErrorLog"),
    "ISpEventSink": SimTypeBottom(label="ISpEventSink"),
    "ISpEventSource": SimTypeBottom(label="ISpEventSource"),
    "ISpEventSource2": SimTypeBottom(label="ISpEventSource2"),
    "ISpGramCompBackend": SimTypeBottom(label="ISpGramCompBackend"),
    "ISpGrammarBuilder": SimTypeBottom(label="ISpGrammarBuilder"),
    "ISpGrammarBuilder2": SimTypeBottom(label="ISpGrammarBuilder2"),
    "ISpGrammarCompiler": SimTypeBottom(label="ISpGrammarCompiler"),
    "ISpITNProcessor": SimTypeBottom(label="ISpITNProcessor"),
    "ISpLexicon": SimTypeBottom(label="ISpLexicon"),
    "ISpMMSysAudio": SimTypeBottom(label="ISpMMSysAudio"),
    "ISpNotifyCallback": SimTypeBottom(label="ISpNotifyCallback"),
    "ISpNotifySink": SimTypeBottom(label="ISpNotifySink"),
    "ISpNotifySource": SimTypeBottom(label="ISpNotifySource"),
    "ISpNotifyTranslator": SimTypeBottom(label="ISpNotifyTranslator"),
    "ISpObjectToken": SimTypeBottom(label="ISpObjectToken"),
    "ISpObjectTokenCategory": SimTypeBottom(label="ISpObjectTokenCategory"),
    "ISpObjectTokenEnumBuilder": SimTypeBottom(label="ISpObjectTokenEnumBuilder"),
    "ISpObjectTokenInit": SimTypeBottom(label="ISpObjectTokenInit"),
    "ISpObjectWithToken": SimTypeBottom(label="ISpObjectWithToken"),
    "ISpPhoneConverter": SimTypeBottom(label="ISpPhoneConverter"),
    "ISpPhoneticAlphabetConverter": SimTypeBottom(label="ISpPhoneticAlphabetConverter"),
    "ISpPhoneticAlphabetSelection": SimTypeBottom(label="ISpPhoneticAlphabetSelection"),
    "ISpPhrase": SimTypeBottom(label="ISpPhrase"),
    "ISpPhrase2": SimTypeBottom(label="ISpPhrase2"),
    "ISpPhraseAlt": SimTypeBottom(label="ISpPhraseAlt"),
    "ISpPhraseBuilder": SimTypeBottom(label="ISpPhraseBuilder"),
    "ISpPrivateEngineCallEx": SimTypeBottom(label="ISpPrivateEngineCallEx"),
    "ISpProperties": SimTypeBottom(label="ISpProperties"),
    "ISpRecoContext": SimTypeBottom(label="ISpRecoContext"),
    "ISpRecoContext2": SimTypeBottom(label="ISpRecoContext2"),
    "ISpRecoGrammar": SimTypeBottom(label="ISpRecoGrammar"),
    "ISpRecoGrammar2": SimTypeBottom(label="ISpRecoGrammar2"),
    "ISpRecoResult": SimTypeBottom(label="ISpRecoResult"),
    "ISpRecoResult2": SimTypeBottom(label="ISpRecoResult2"),
    "ISpRecognizer": SimTypeBottom(label="ISpRecognizer"),
    "ISpRecognizer2": SimTypeBottom(label="ISpRecognizer2"),
    "ISpRegDataKey": SimTypeBottom(label="ISpRegDataKey"),
    "ISpResourceManager": SimTypeBottom(label="ISpResourceManager"),
    "ISpSRAlternates": SimTypeBottom(label="ISpSRAlternates"),
    "ISpSRAlternates2": SimTypeBottom(label="ISpSRAlternates2"),
    "ISpSREngine": SimTypeBottom(label="ISpSREngine"),
    "ISpSREngine2": SimTypeBottom(label="ISpSREngine2"),
    "ISpSREngineSite": SimTypeBottom(label="ISpSREngineSite"),
    "ISpSREngineSite2": SimTypeBottom(label="ISpSREngineSite2"),
    "ISpSerializeState": SimTypeBottom(label="ISpSerializeState"),
    "ISpShortcut": SimTypeBottom(label="ISpShortcut"),
    "ISpStream": SimTypeBottom(label="ISpStream"),
    "ISpStreamFormat": SimTypeBottom(label="ISpStreamFormat"),
    "ISpStreamFormatConverter": SimTypeBottom(label="ISpStreamFormatConverter"),
    "ISpTTSEngine": SimTypeBottom(label="ISpTTSEngine"),
    "ISpTTSEngineSite": SimTypeBottom(label="ISpTTSEngineSite"),
    "ISpTask": SimTypeBottom(label="ISpTask"),
    "ISpTaskManager": SimTypeBottom(label="ISpTaskManager"),
    "ISpThreadControl": SimTypeBottom(label="ISpThreadControl"),
    "ISpThreadTask": SimTypeBottom(label="ISpThreadTask"),
    "ISpTokenUI": SimTypeBottom(label="ISpTokenUI"),
    "ISpTranscript": SimTypeBottom(label="ISpTranscript"),
    "ISpVoice": SimTypeBottom(label="ISpVoice"),
    "ISpXMLRecoResult": SimTypeBottom(label="ISpXMLRecoResult"),
    "ISpatialAudioClient": SimTypeBottom(label="ISpatialAudioClient"),
    "ISpatialAudioClient2": SimTypeBottom(label="ISpatialAudioClient2"),
    "ISpatialAudioMetadataClient": SimTypeBottom(label="ISpatialAudioMetadataClient"),
    "ISpatialAudioMetadataCopier": SimTypeBottom(label="ISpatialAudioMetadataCopier"),
    "ISpatialAudioMetadataItems": SimTypeBottom(label="ISpatialAudioMetadataItems"),
    "ISpatialAudioMetadataItemsBuffer": SimTypeBottom(label="ISpatialAudioMetadataItemsBuffer"),
    "ISpatialAudioMetadataReader": SimTypeBottom(label="ISpatialAudioMetadataReader"),
    "ISpatialAudioMetadataWriter": SimTypeBottom(label="ISpatialAudioMetadataWriter"),
    "ISpatialAudioObject": SimTypeBottom(label="ISpatialAudioObject"),
    "ISpatialAudioObjectBase": SimTypeBottom(label="ISpatialAudioObjectBase"),
    "ISpatialAudioObjectForHrtf": SimTypeBottom(label="ISpatialAudioObjectForHrtf"),
    "ISpatialAudioObjectForMetadataCommands": SimTypeBottom(label="ISpatialAudioObjectForMetadataCommands"),
    "ISpatialAudioObjectForMetadataItems": SimTypeBottom(label="ISpatialAudioObjectForMetadataItems"),
    "ISpatialAudioObjectRenderStream": SimTypeBottom(label="ISpatialAudioObjectRenderStream"),
    "ISpatialAudioObjectRenderStreamBase": SimTypeBottom(label="ISpatialAudioObjectRenderStreamBase"),
    "ISpatialAudioObjectRenderStreamForHrtf": SimTypeBottom(label="ISpatialAudioObjectRenderStreamForHrtf"),
    "ISpatialAudioObjectRenderStreamForMetadata": SimTypeBottom(label="ISpatialAudioObjectRenderStreamForMetadata"),
    "ISpatialAudioObjectRenderStreamNotify": SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify"),
    "ISpecifyParticularPages": SimTypeBottom(label="ISpecifyParticularPages"),
    "ISpecifyPropertyPages": SimTypeBottom(label="ISpecifyPropertyPages"),
    "ISpeechAudio": SimTypeBottom(label="ISpeechAudio"),
    "ISpeechAudioBufferInfo": SimTypeBottom(label="ISpeechAudioBufferInfo"),
    "ISpeechAudioFormat": SimTypeBottom(label="ISpeechAudioFormat"),
    "ISpeechAudioStatus": SimTypeBottom(label="ISpeechAudioStatus"),
    "ISpeechBaseStream": SimTypeBottom(label="ISpeechBaseStream"),
    "ISpeechCommandProvider": SimTypeBottom(label="ISpeechCommandProvider"),
    "ISpeechCustomStream": SimTypeBottom(label="ISpeechCustomStream"),
    "ISpeechDataKey": SimTypeBottom(label="ISpeechDataKey"),
    "ISpeechFileStream": SimTypeBottom(label="ISpeechFileStream"),
    "ISpeechGrammarRule": SimTypeBottom(label="ISpeechGrammarRule"),
    "ISpeechGrammarRuleState": SimTypeBottom(label="ISpeechGrammarRuleState"),
    "ISpeechGrammarRuleStateTransition": SimTypeBottom(label="ISpeechGrammarRuleStateTransition"),
    "ISpeechGrammarRuleStateTransitions": SimTypeBottom(label="ISpeechGrammarRuleStateTransitions"),
    "ISpeechGrammarRules": SimTypeBottom(label="ISpeechGrammarRules"),
    "ISpeechLexicon": SimTypeBottom(label="ISpeechLexicon"),
    "ISpeechLexiconPronunciation": SimTypeBottom(label="ISpeechLexiconPronunciation"),
    "ISpeechLexiconPronunciations": SimTypeBottom(label="ISpeechLexiconPronunciations"),
    "ISpeechLexiconWord": SimTypeBottom(label="ISpeechLexiconWord"),
    "ISpeechLexiconWords": SimTypeBottom(label="ISpeechLexiconWords"),
    "ISpeechMMSysAudio": SimTypeBottom(label="ISpeechMMSysAudio"),
    "ISpeechMemoryStream": SimTypeBottom(label="ISpeechMemoryStream"),
    "ISpeechObjectToken": SimTypeBottom(label="ISpeechObjectToken"),
    "ISpeechObjectTokenCategory": SimTypeBottom(label="ISpeechObjectTokenCategory"),
    "ISpeechObjectTokens": SimTypeBottom(label="ISpeechObjectTokens"),
    "ISpeechPhoneConverter": SimTypeBottom(label="ISpeechPhoneConverter"),
    "ISpeechPhraseAlternate": SimTypeBottom(label="ISpeechPhraseAlternate"),
    "ISpeechPhraseAlternates": SimTypeBottom(label="ISpeechPhraseAlternates"),
    "ISpeechPhraseElement": SimTypeBottom(label="ISpeechPhraseElement"),
    "ISpeechPhraseElements": SimTypeBottom(label="ISpeechPhraseElements"),
    "ISpeechPhraseInfo": SimTypeBottom(label="ISpeechPhraseInfo"),
    "ISpeechPhraseInfoBuilder": SimTypeBottom(label="ISpeechPhraseInfoBuilder"),
    "ISpeechPhraseProperties": SimTypeBottom(label="ISpeechPhraseProperties"),
    "ISpeechPhraseProperty": SimTypeBottom(label="ISpeechPhraseProperty"),
    "ISpeechPhraseReplacement": SimTypeBottom(label="ISpeechPhraseReplacement"),
    "ISpeechPhraseReplacements": SimTypeBottom(label="ISpeechPhraseReplacements"),
    "ISpeechPhraseRule": SimTypeBottom(label="ISpeechPhraseRule"),
    "ISpeechPhraseRules": SimTypeBottom(label="ISpeechPhraseRules"),
    "ISpeechRecoContext": SimTypeBottom(label="ISpeechRecoContext"),
    "ISpeechRecoGrammar": SimTypeBottom(label="ISpeechRecoGrammar"),
    "ISpeechRecoResult": SimTypeBottom(label="ISpeechRecoResult"),
    "ISpeechRecoResult2": SimTypeBottom(label="ISpeechRecoResult2"),
    "ISpeechRecoResultDispatch": SimTypeBottom(label="ISpeechRecoResultDispatch"),
    "ISpeechRecoResultTimes": SimTypeBottom(label="ISpeechRecoResultTimes"),
    "ISpeechRecognizer": SimTypeBottom(label="ISpeechRecognizer"),
    "ISpeechRecognizerStatus": SimTypeBottom(label="ISpeechRecognizerStatus"),
    "ISpeechResourceLoader": SimTypeBottom(label="ISpeechResourceLoader"),
    "ISpeechTextSelectionInformation": SimTypeBottom(label="ISpeechTextSelectionInformation"),
    "ISpeechVoice": SimTypeBottom(label="ISpeechVoice"),
    "ISpeechVoiceStatus": SimTypeBottom(label="ISpeechVoiceStatus"),
    "ISpeechWaveFormatEx": SimTypeBottom(label="ISpeechWaveFormatEx"),
    "ISpeechXMLRecoResult": SimTypeBottom(label="ISpeechXMLRecoResult"),
    "ISpellCheckProvider": SimTypeBottom(label="ISpellCheckProvider"),
    "ISpellCheckProviderFactory": SimTypeBottom(label="ISpellCheckProviderFactory"),
    "ISpellChecker": SimTypeBottom(label="ISpellChecker"),
    "ISpellChecker2": SimTypeBottom(label="ISpellChecker2"),
    "ISpellCheckerChangedEventHandler": SimTypeBottom(label="ISpellCheckerChangedEventHandler"),
    "ISpellCheckerFactory": SimTypeBottom(label="ISpellCheckerFactory"),
    "ISpellingError": SimTypeBottom(label="ISpellingError"),
    "ISpreadsheetItemProvider": SimTypeBottom(label="ISpreadsheetItemProvider"),
    "ISpreadsheetProvider": SimTypeBottom(label="ISpreadsheetProvider"),
    "IStartMenuPinnedList": SimTypeBottom(label="IStartMenuPinnedList"),
    "IStaticPortMapping": SimTypeBottom(label="IStaticPortMapping"),
    "IStaticPortMappingCollection": SimTypeBottom(label="IStaticPortMappingCollection"),
    "IStdMarshalInfo": SimTypeBottom(label="IStdMarshalInfo"),
    "IStemSink": SimTypeBottom(label="IStemSink"),
    "IStemmer": SimTypeBottom(label="IStemmer"),
    "IStemmerConfig": SimTypeBottom(label="IStemmerConfig"),
    "IStiDevice": SimTypeBottom(label="IStiDevice"),
    "IStiDeviceControl": SimTypeBottom(label="IStiDeviceControl"),
    "IStiUSD": SimTypeBottom(label="IStiUSD"),
    "IStillImageW": SimTypeBottom(label="IStillImageW"),
    "IStorage": SimTypeBottom(label="IStorage"),
    "IStorageProviderBanners": SimTypeBottom(label="IStorageProviderBanners"),
    "IStorageProviderCopyHook": SimTypeBottom(label="IStorageProviderCopyHook"),
    "IStorageProviderHandler": SimTypeBottom(label="IStorageProviderHandler"),
    "IStorageProviderPropertyHandler": SimTypeBottom(label="IStorageProviderPropertyHandler"),
    "IStream": SimTypeBottom(label="IStream"),
    "IStreamAsync": SimTypeBottom(label="IStreamAsync"),
    "IStreamBufferConfigure": SimTypeBottom(label="IStreamBufferConfigure"),
    "IStreamBufferConfigure2": SimTypeBottom(label="IStreamBufferConfigure2"),
    "IStreamBufferConfigure3": SimTypeBottom(label="IStreamBufferConfigure3"),
    "IStreamBufferDataCounters": SimTypeBottom(label="IStreamBufferDataCounters"),
    "IStreamBufferInitialize": SimTypeBottom(label="IStreamBufferInitialize"),
    "IStreamBufferMediaSeeking": SimTypeBottom(label="IStreamBufferMediaSeeking"),
    "IStreamBufferMediaSeeking2": SimTypeBottom(label="IStreamBufferMediaSeeking2"),
    "IStreamBufferRecComp": SimTypeBottom(label="IStreamBufferRecComp"),
    "IStreamBufferRecordControl": SimTypeBottom(label="IStreamBufferRecordControl"),
    "IStreamBufferRecordingAttribute": SimTypeBottom(label="IStreamBufferRecordingAttribute"),
    "IStreamBufferSink": SimTypeBottom(label="IStreamBufferSink"),
    "IStreamBufferSink2": SimTypeBottom(label="IStreamBufferSink2"),
    "IStreamBufferSink3": SimTypeBottom(label="IStreamBufferSink3"),
    "IStreamBufferSource": SimTypeBottom(label="IStreamBufferSource"),
    "IStreamBuilder": SimTypeBottom(label="IStreamBuilder"),
    "IStreamConcatenate": SimTypeBottom(label="IStreamConcatenate"),
    "IStreamInterleave": SimTypeBottom(label="IStreamInterleave"),
    "IStreamPseudoRandomBased": SimTypeBottom(label="IStreamPseudoRandomBased"),
    "IStreamSample": SimTypeBottom(label="IStreamSample"),
    "IStreamUnbufferedInfo": SimTypeBottom(label="IStreamUnbufferedInfo"),
    "IStringCollection": SimTypeBottom(label="IStringCollection"),
    "IStringDisplayableConcept": SimTypeBottom(label="IStringDisplayableConcept"),
    "IStringTable": SimTypeBottom(label="IStringTable"),
    "IStrokeBuilder": SimTypeBottom(label="IStrokeBuilder"),
    "IStylesProvider": SimTypeBottom(label="IStylesProvider"),
    "IStylusAsyncPlugin": SimTypeBottom(label="IStylusAsyncPlugin"),
    "IStylusPlugin": SimTypeBottom(label="IStylusPlugin"),
    "IStylusSyncPlugin": SimTypeBottom(label="IStylusSyncPlugin"),
    "ISubscriptionItem": SimTypeBottom(label="ISubscriptionItem"),
    "ISubscriptionMgr": SimTypeBottom(label="ISubscriptionMgr"),
    "ISubscriptionMgr2": SimTypeBottom(label="ISubscriptionMgr2"),
    "ISubunit": SimTypeBottom(label="ISubunit"),
    "ISupportAllowLowerTrustActivation": SimTypeBottom(label="ISupportAllowLowerTrustActivation"),
    "ISupportErrorInfo": SimTypeBottom(label="ISupportErrorInfo"),
    "ISupportFilteredSync": SimTypeBottom(label="ISupportFilteredSync"),
    "ISupportLastWriteTime": SimTypeBottom(label="ISupportLastWriteTime"),
    "ISurfacePresenter": SimTypeBottom(label="ISurfacePresenter"),
    "ISurfacePresenterFlip": SimTypeBottom(label="ISurfacePresenterFlip"),
    "ISurfacePresenterFlip2": SimTypeBottom(label="ISurfacePresenterFlip2"),
    "ISurfacePresenterFlipBuffer": SimTypeBottom(label="ISurfacePresenterFlipBuffer"),
    "ISurrogate": SimTypeBottom(label="ISurrogate"),
    "ISurrogateService": SimTypeBottom(label="ISurrogateService"),
    "ISuspensionDependencyManager": SimTypeBottom(label="ISuspensionDependencyManager"),
    "ISyncCallback": SimTypeBottom(label="ISyncCallback"),
    "ISyncCallback2": SimTypeBottom(label="ISyncCallback2"),
    "ISyncChange": SimTypeBottom(label="ISyncChange"),
    "ISyncChangeBatch": SimTypeBottom(label="ISyncChangeBatch"),
    "ISyncChangeBatch2": SimTypeBottom(label="ISyncChangeBatch2"),
    "ISyncChangeBatchAdvanced": SimTypeBottom(label="ISyncChangeBatchAdvanced"),
    "ISyncChangeBatchBase": SimTypeBottom(label="ISyncChangeBatchBase"),
    "ISyncChangeBatchBase2": SimTypeBottom(label="ISyncChangeBatchBase2"),
    "ISyncChangeBatchWithFilterKeyMap": SimTypeBottom(label="ISyncChangeBatchWithFilterKeyMap"),
    "ISyncChangeBatchWithPrerequisite": SimTypeBottom(label="ISyncChangeBatchWithPrerequisite"),
    "ISyncChangeBuilder": SimTypeBottom(label="ISyncChangeBuilder"),
    "ISyncChangeUnit": SimTypeBottom(label="ISyncChangeUnit"),
    "ISyncChangeWithFilterKeyMap": SimTypeBottom(label="ISyncChangeWithFilterKeyMap"),
    "ISyncChangeWithPrerequisite": SimTypeBottom(label="ISyncChangeWithPrerequisite"),
    "ISyncConstraintCallback": SimTypeBottom(label="ISyncConstraintCallback"),
    "ISyncDataConverter": SimTypeBottom(label="ISyncDataConverter"),
    "ISyncFilter": SimTypeBottom(label="ISyncFilter"),
    "ISyncFilterDeserializer": SimTypeBottom(label="ISyncFilterDeserializer"),
    "ISyncFilterInfo": SimTypeBottom(label="ISyncFilterInfo"),
    "ISyncFilterInfo2": SimTypeBottom(label="ISyncFilterInfo2"),
    "ISyncFullEnumerationChange": SimTypeBottom(label="ISyncFullEnumerationChange"),
    "ISyncFullEnumerationChangeBatch": SimTypeBottom(label="ISyncFullEnumerationChangeBatch"),
    "ISyncFullEnumerationChangeBatch2": SimTypeBottom(label="ISyncFullEnumerationChangeBatch2"),
    "ISyncKnowledge": SimTypeBottom(label="ISyncKnowledge"),
    "ISyncKnowledge2": SimTypeBottom(label="ISyncKnowledge2"),
    "ISyncMergeTombstoneChange": SimTypeBottom(label="ISyncMergeTombstoneChange"),
    "ISyncMgrConflict": SimTypeBottom(label="ISyncMgrConflict"),
    "ISyncMgrConflictFolder": SimTypeBottom(label="ISyncMgrConflictFolder"),
    "ISyncMgrConflictItems": SimTypeBottom(label="ISyncMgrConflictItems"),
    "ISyncMgrConflictPresenter": SimTypeBottom(label="ISyncMgrConflictPresenter"),
    "ISyncMgrConflictResolutionItems": SimTypeBottom(label="ISyncMgrConflictResolutionItems"),
    "ISyncMgrConflictResolveInfo": SimTypeBottom(label="ISyncMgrConflictResolveInfo"),
    "ISyncMgrConflictStore": SimTypeBottom(label="ISyncMgrConflictStore"),
    "ISyncMgrControl": SimTypeBottom(label="ISyncMgrControl"),
    "ISyncMgrEnumItems": SimTypeBottom(label="ISyncMgrEnumItems"),
    "ISyncMgrEvent": SimTypeBottom(label="ISyncMgrEvent"),
    "ISyncMgrEventLinkUIOperation": SimTypeBottom(label="ISyncMgrEventLinkUIOperation"),
    "ISyncMgrEventStore": SimTypeBottom(label="ISyncMgrEventStore"),
    "ISyncMgrHandler": SimTypeBottom(label="ISyncMgrHandler"),
    "ISyncMgrHandlerCollection": SimTypeBottom(label="ISyncMgrHandlerCollection"),
    "ISyncMgrHandlerInfo": SimTypeBottom(label="ISyncMgrHandlerInfo"),
    "ISyncMgrRegister": SimTypeBottom(label="ISyncMgrRegister"),
    "ISyncMgrResolutionHandler": SimTypeBottom(label="ISyncMgrResolutionHandler"),
    "ISyncMgrScheduleWizardUIOperation": SimTypeBottom(label="ISyncMgrScheduleWizardUIOperation"),
    "ISyncMgrSessionCreator": SimTypeBottom(label="ISyncMgrSessionCreator"),
    "ISyncMgrSyncCallback": SimTypeBottom(label="ISyncMgrSyncCallback"),
    "ISyncMgrSyncItem": SimTypeBottom(label="ISyncMgrSyncItem"),
    "ISyncMgrSyncItemContainer": SimTypeBottom(label="ISyncMgrSyncItemContainer"),
    "ISyncMgrSyncItemInfo": SimTypeBottom(label="ISyncMgrSyncItemInfo"),
    "ISyncMgrSyncResult": SimTypeBottom(label="ISyncMgrSyncResult"),
    "ISyncMgrSynchronize": SimTypeBottom(label="ISyncMgrSynchronize"),
    "ISyncMgrSynchronizeCallback": SimTypeBottom(label="ISyncMgrSynchronizeCallback"),
    "ISyncMgrSynchronizeInvoke": SimTypeBottom(label="ISyncMgrSynchronizeInvoke"),
    "ISyncMgrUIOperation": SimTypeBottom(label="ISyncMgrUIOperation"),
    "ISyncProvider": SimTypeBottom(label="ISyncProvider"),
    "ISyncProviderConfigUI": SimTypeBottom(label="ISyncProviderConfigUI"),
    "ISyncProviderConfigUIInfo": SimTypeBottom(label="ISyncProviderConfigUIInfo"),
    "ISyncProviderInfo": SimTypeBottom(label="ISyncProviderInfo"),
    "ISyncProviderRegistration": SimTypeBottom(label="ISyncProviderRegistration"),
    "ISyncRegistrationChange": SimTypeBottom(label="ISyncRegistrationChange"),
    "ISyncSessionExtendedErrorInfo": SimTypeBottom(label="ISyncSessionExtendedErrorInfo"),
    "ISyncSessionState": SimTypeBottom(label="ISyncSessionState"),
    "ISyncSessionState2": SimTypeBottom(label="ISyncSessionState2"),
    "ISynchronize": SimTypeBottom(label="ISynchronize"),
    "ISynchronizeContainer": SimTypeBottom(label="ISynchronizeContainer"),
    "ISynchronizeEvent": SimTypeBottom(label="ISynchronizeEvent"),
    "ISynchronizeHandle": SimTypeBottom(label="ISynchronizeHandle"),
    "ISynchronizeMutex": SimTypeBottom(label="ISynchronizeMutex"),
    "ISynchronizedInputProvider": SimTypeBottom(label="ISynchronizedInputProvider"),
    "ISynchronousDataRetriever": SimTypeBottom(label="ISynchronousDataRetriever"),
    "ISystemAppEventData": SimTypeBottom(label="ISystemAppEventData"),
    "ISystemInformation": SimTypeBottom(label="ISystemInformation"),
    "ISystemMonitor": SimTypeBottom(label="ISystemMonitor"),
    "ISystemMonitor2": SimTypeBottom(label="ISystemMonitor2"),
    "ISystemMonitorEvents": SimTypeBottom(label="ISystemMonitorEvents"),
    "ITACDGroup": SimTypeBottom(label="ITACDGroup"),
    "ITACDGroupEvent": SimTypeBottom(label="ITACDGroupEvent"),
    "ITAMMediaFormat": SimTypeBottom(label="ITAMMediaFormat"),
    "ITASRTerminalEvent": SimTypeBottom(label="ITASRTerminalEvent"),
    "ITAddress": SimTypeBottom(label="ITAddress"),
    "ITAddress2": SimTypeBottom(label="ITAddress2"),
    "ITAddressCapabilities": SimTypeBottom(label="ITAddressCapabilities"),
    "ITAddressDeviceSpecificEvent": SimTypeBottom(label="ITAddressDeviceSpecificEvent"),
    "ITAddressEvent": SimTypeBottom(label="ITAddressEvent"),
    "ITAddressTranslation": SimTypeBottom(label="ITAddressTranslation"),
    "ITAddressTranslationInfo": SimTypeBottom(label="ITAddressTranslationInfo"),
    "ITAgent": SimTypeBottom(label="ITAgent"),
    "ITAgentEvent": SimTypeBottom(label="ITAgentEvent"),
    "ITAgentHandler": SimTypeBottom(label="ITAgentHandler"),
    "ITAgentHandlerEvent": SimTypeBottom(label="ITAgentHandlerEvent"),
    "ITAgentSession": SimTypeBottom(label="ITAgentSession"),
    "ITAgentSessionEvent": SimTypeBottom(label="ITAgentSessionEvent"),
    "ITAllocatorProperties": SimTypeBottom(label="ITAllocatorProperties"),
    "ITAutomatedPhoneControl": SimTypeBottom(label="ITAutomatedPhoneControl"),
    "ITBasicAudioTerminal": SimTypeBottom(label="ITBasicAudioTerminal"),
    "ITBasicCallControl": SimTypeBottom(label="ITBasicCallControl"),
    "ITBasicCallControl2": SimTypeBottom(label="ITBasicCallControl2"),
    "ITCallHub": SimTypeBottom(label="ITCallHub"),
    "ITCallHubEvent": SimTypeBottom(label="ITCallHubEvent"),
    "ITCallInfo": SimTypeBottom(label="ITCallInfo"),
    "ITCallInfo2": SimTypeBottom(label="ITCallInfo2"),
    "ITCallInfoChangeEvent": SimTypeBottom(label="ITCallInfoChangeEvent"),
    "ITCallMediaEvent": SimTypeBottom(label="ITCallMediaEvent"),
    "ITCallNotificationEvent": SimTypeBottom(label="ITCallNotificationEvent"),
    "ITCallStateEvent": SimTypeBottom(label="ITCallStateEvent"),
    "ITCallingCard": SimTypeBottom(label="ITCallingCard"),
    "ITCollection": SimTypeBottom(label="ITCollection"),
    "ITCollection2": SimTypeBottom(label="ITCollection2"),
    "ITCustomTone": SimTypeBottom(label="ITCustomTone"),
    "ITDetectTone": SimTypeBottom(label="ITDetectTone"),
    "ITDigitDetectionEvent": SimTypeBottom(label="ITDigitDetectionEvent"),
    "ITDigitGenerationEvent": SimTypeBottom(label="ITDigitGenerationEvent"),
    "ITDigitsGatheredEvent": SimTypeBottom(label="ITDigitsGatheredEvent"),
    "ITDirectory": SimTypeBottom(label="ITDirectory"),
    "ITDirectoryObject": SimTypeBottom(label="ITDirectoryObject"),
    "ITDirectoryObjectConference": SimTypeBottom(label="ITDirectoryObjectConference"),
    "ITDirectoryObjectUser": SimTypeBottom(label="ITDirectoryObjectUser"),
    "ITDispatchMapper": SimTypeBottom(label="ITDispatchMapper"),
    "ITEMIDLIST": SimStruct(OrderedDict((("mkid", SimTypeRef("SHITEMID", SimStruct)),)), name="ITEMIDLIST", pack=False, align=None),
    "ITEMPROP": SimStruct(OrderedDict((("variantValue", SimTypeRef("VARIANT", SimStruct)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ITEMPROP", pack=False, align=None),
    "ITEMSPACING": SimStruct(OrderedDict((("cxSmall", SimTypeInt(signed=True, label="Int32")), ("cySmall", SimTypeInt(signed=True, label="Int32")), ("cxLarge", SimTypeInt(signed=True, label="Int32")), ("cyLarge", SimTypeInt(signed=True, label="Int32")),)), name="ITEMSPACING", pack=False, align=None),
    "ITEMSTATES": SimTypeInt(signed=False, label="ITEMSTATES"),
    "ITEM_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pcwszFromEMail", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszCatalogName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszContentClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ITEM_INFO", pack=False, align=None),
    "ITFileTerminalEvent": SimTypeBottom(label="ITFileTerminalEvent"),
    "ITFileTrack": SimTypeBottom(label="ITFileTrack"),
    "ITForwardInformation": SimTypeBottom(label="ITForwardInformation"),
    "ITForwardInformation2": SimTypeBottom(label="ITForwardInformation2"),
    "ITILSConfig": SimTypeBottom(label="ITILSConfig"),
    "ITLegacyAddressMediaControl": SimTypeBottom(label="ITLegacyAddressMediaControl"),
    "ITLegacyAddressMediaControl2": SimTypeBottom(label="ITLegacyAddressMediaControl2"),
    "ITLegacyCallMediaControl": SimTypeBottom(label="ITLegacyCallMediaControl"),
    "ITLegacyCallMediaControl2": SimTypeBottom(label="ITLegacyCallMediaControl2"),
    "ITLegacyWaveSupport": SimTypeBottom(label="ITLegacyWaveSupport"),
    "ITLocationInfo": SimTypeBottom(label="ITLocationInfo"),
    "ITMSPAddress": SimTypeBottom(label="ITMSPAddress"),
    "ITMediaControl": SimTypeBottom(label="ITMediaControl"),
    "ITMediaPlayback": SimTypeBottom(label="ITMediaPlayback"),
    "ITMediaRecord": SimTypeBottom(label="ITMediaRecord"),
    "ITMediaSupport": SimTypeBottom(label="ITMediaSupport"),
    "ITMultiTrackTerminal": SimTypeBottom(label="ITMultiTrackTerminal"),
    "ITPhone": SimTypeBottom(label="ITPhone"),
    "ITPhoneDeviceSpecificEvent": SimTypeBottom(label="ITPhoneDeviceSpecificEvent"),
    "ITPhoneEvent": SimTypeBottom(label="ITPhoneEvent"),
    "ITPluggableTerminalClassInfo": SimTypeBottom(label="ITPluggableTerminalClassInfo"),
    "ITPluggableTerminalEventSink": SimTypeBottom(label="ITPluggableTerminalEventSink"),
    "ITPluggableTerminalEventSinkRegistration": SimTypeBottom(label="ITPluggableTerminalEventSinkRegistration"),
    "ITPluggableTerminalSuperclassInfo": SimTypeBottom(label="ITPluggableTerminalSuperclassInfo"),
    "ITPrivateEvent": SimTypeBottom(label="ITPrivateEvent"),
    "ITQOSEvent": SimTypeBottom(label="ITQOSEvent"),
    "ITQueue": SimTypeBottom(label="ITQueue"),
    "ITQueueEvent": SimTypeBottom(label="ITQueueEvent"),
    "ITRendezvous": SimTypeBottom(label="ITRendezvous"),
    "ITRequest": SimTypeBottom(label="ITRequest"),
    "ITRequestEvent": SimTypeBottom(label="ITRequestEvent"),
    "ITSDT": SimTypeBottom(label="ITSDT"),
    "ITSGAccountingEngine": SimTypeBottom(label="ITSGAccountingEngine"),
    "ITSGAuthenticateUserSink": SimTypeBottom(label="ITSGAuthenticateUserSink"),
    "ITSGAuthenticationEngine": SimTypeBottom(label="ITSGAuthenticationEngine"),
    "ITSGAuthorizeConnectionSink": SimTypeBottom(label="ITSGAuthorizeConnectionSink"),
    "ITSGAuthorizeResourceSink": SimTypeBottom(label="ITSGAuthorizeResourceSink"),
    "ITSGPolicyEngine": SimTypeBottom(label="ITSGPolicyEngine"),
    "ITScriptableAudioFormat": SimTypeBottom(label="ITScriptableAudioFormat"),
    "ITStaticAudioTerminal": SimTypeBottom(label="ITStaticAudioTerminal"),
    "ITStream": SimTypeBottom(label="ITStream"),
    "ITStreamControl": SimTypeBottom(label="ITStreamControl"),
    "ITSubStream": SimTypeBottom(label="ITSubStream"),
    "ITSubStreamControl": SimTypeBottom(label="ITSubStreamControl"),
    "ITTAPI": SimTypeBottom(label="ITTAPI"),
    "ITTAPI2": SimTypeBottom(label="ITTAPI2"),
    "ITTAPICallCenter": SimTypeBottom(label="ITTAPICallCenter"),
    "ITTAPIDispatchEventNotification": SimTypeBottom(label="ITTAPIDispatchEventNotification"),
    "ITTAPIEventNotification": SimTypeBottom(label="ITTAPIEventNotification"),
    "ITTAPIObjectEvent": SimTypeBottom(label="ITTAPIObjectEvent"),
    "ITTAPIObjectEvent2": SimTypeBottom(label="ITTAPIObjectEvent2"),
    "ITTTSTerminalEvent": SimTypeBottom(label="ITTTSTerminalEvent"),
    "ITTerminal": SimTypeBottom(label="ITTerminal"),
    "ITTerminalSupport": SimTypeBottom(label="ITTerminalSupport"),
    "ITTerminalSupport2": SimTypeBottom(label="ITTerminalSupport2"),
    "ITToneDetectionEvent": SimTypeBottom(label="ITToneDetectionEvent"),
    "ITToneTerminalEvent": SimTypeBottom(label="ITToneTerminalEvent"),
    "ITableCreation": SimTypeBottom(label="ITableCreation"),
    "ITableData": SimTypeBottom(label="ITableData"),
    "ITableDefinition": SimTypeBottom(label="ITableDefinition"),
    "ITableDefinitionWithConstraints": SimTypeBottom(label="ITableDefinitionWithConstraints"),
    "ITableItemProvider": SimTypeBottom(label="ITableItemProvider"),
    "ITableProvider": SimTypeBottom(label="ITableProvider"),
    "ITableRename": SimTypeBottom(label="ITableRename"),
    "ITargetContainer": SimTypeBottom(label="ITargetContainer"),
    "ITargetEmbedding": SimTypeBottom(label="ITargetEmbedding"),
    "ITargetFrame": SimTypeBottom(label="ITargetFrame"),
    "ITargetFrame2": SimTypeBottom(label="ITargetFrame2"),
    "ITargetFramePriv": SimTypeBottom(label="ITargetFramePriv"),
    "ITargetFramePriv2": SimTypeBottom(label="ITargetFramePriv2"),
    "ITargetInfo": SimTypeBottom(label="ITargetInfo"),
    "ITargetNotify": SimTypeBottom(label="ITargetNotify"),
    "ITargetNotify2": SimTypeBottom(label="ITargetNotify2"),
    "ITask": SimTypeBottom(label="ITask"),
    "ITaskDefinition": SimTypeBottom(label="ITaskDefinition"),
    "ITaskFolder": SimTypeBottom(label="ITaskFolder"),
    "ITaskFolderCollection": SimTypeBottom(label="ITaskFolderCollection"),
    "ITaskHandler": SimTypeBottom(label="ITaskHandler"),
    "ITaskHandlerStatus": SimTypeBottom(label="ITaskHandlerStatus"),
    "ITaskNamedValueCollection": SimTypeBottom(label="ITaskNamedValueCollection"),
    "ITaskNamedValuePair": SimTypeBottom(label="ITaskNamedValuePair"),
    "ITaskScheduler": SimTypeBottom(label="ITaskScheduler"),
    "ITaskService": SimTypeBottom(label="ITaskService"),
    "ITaskSettings": SimTypeBottom(label="ITaskSettings"),
    "ITaskSettings2": SimTypeBottom(label="ITaskSettings2"),
    "ITaskSettings3": SimTypeBottom(label="ITaskSettings3"),
    "ITaskTrigger": SimTypeBottom(label="ITaskTrigger"),
    "ITaskVariables": SimTypeBottom(label="ITaskVariables"),
    "ITaskbarList": SimTypeBottom(label="ITaskbarList"),
    "ITaskbarList2": SimTypeBottom(label="ITaskbarList2"),
    "ITaskbarList3": SimTypeBottom(label="ITaskbarList3"),
    "ITaskbarList4": SimTypeBottom(label="ITaskbarList4"),
    "ITemplatePrinter": SimTypeBottom(label="ITemplatePrinter"),
    "ITemplatePrinter2": SimTypeBottom(label="ITemplatePrinter2"),
    "ITemplatePrinter3": SimTypeBottom(label="ITemplatePrinter3"),
    "ITemplateSdo": SimTypeBottom(label="ITemplateSdo"),
    "ITextChildProvider": SimTypeBottom(label="ITextChildProvider"),
    "ITextDisplays": SimTypeBottom(label="ITextDisplays"),
    "ITextDocument": SimTypeBottom(label="ITextDocument"),
    "ITextDocument2": SimTypeBottom(label="ITextDocument2"),
    "ITextDocument2Old": SimTypeBottom(label="ITextDocument2Old"),
    "ITextEditProvider": SimTypeBottom(label="ITextEditProvider"),
    "ITextFont": SimTypeBottom(label="ITextFont"),
    "ITextFont2": SimTypeBottom(label="ITextFont2"),
    "ITextHost": SimTypeBottom(label="ITextHost"),
    "ITextHost2": SimTypeBottom(label="ITextHost2"),
    "ITextInputPanel": SimTypeBottom(label="ITextInputPanel"),
    "ITextInputPanelEventSink": SimTypeBottom(label="ITextInputPanelEventSink"),
    "ITextInputPanelRunInfo": SimTypeBottom(label="ITextInputPanelRunInfo"),
    "ITextPara": SimTypeBottom(label="ITextPara"),
    "ITextPara2": SimTypeBottom(label="ITextPara2"),
    "ITextProvider": SimTypeBottom(label="ITextProvider"),
    "ITextProvider2": SimTypeBottom(label="ITextProvider2"),
    "ITextRange": SimTypeBottom(label="ITextRange"),
    "ITextRange2": SimTypeBottom(label="ITextRange2"),
    "ITextRangeProvider": SimTypeBottom(label="ITextRangeProvider"),
    "ITextRangeProvider2": SimTypeBottom(label="ITextRangeProvider2"),
    "ITextRow": SimTypeBottom(label="ITextRow"),
    "ITextSelection": SimTypeBottom(label="ITextSelection"),
    "ITextSelection2": SimTypeBottom(label="ITextSelection2"),
    "ITextServices": SimTypeBottom(label="ITextServices"),
    "ITextServices2": SimTypeBottom(label="ITextServices2"),
    "ITextStoreACP": SimTypeBottom(label="ITextStoreACP"),
    "ITextStoreACP2": SimTypeBottom(label="ITextStoreACP2"),
    "ITextStoreACPEx": SimTypeBottom(label="ITextStoreACPEx"),
    "ITextStoreACPServices": SimTypeBottom(label="ITextStoreACPServices"),
    "ITextStoreACPSink": SimTypeBottom(label="ITextStoreACPSink"),
    "ITextStoreACPSinkEx": SimTypeBottom(label="ITextStoreACPSinkEx"),
    "ITextStoreAnchor": SimTypeBottom(label="ITextStoreAnchor"),
    "ITextStoreAnchorEx": SimTypeBottom(label="ITextStoreAnchorEx"),
    "ITextStoreAnchorSink": SimTypeBottom(label="ITextStoreAnchorSink"),
    "ITextStoreSinkAnchorEx": SimTypeBottom(label="ITextStoreSinkAnchorEx"),
    "ITextStory": SimTypeBottom(label="ITextStory"),
    "ITextStoryRanges": SimTypeBottom(label="ITextStoryRanges"),
    "ITextStoryRanges2": SimTypeBottom(label="ITextStoryRanges2"),
    "ITextStrings": SimTypeBottom(label="ITextStrings"),
    "ITfActiveLanguageProfileNotifySink": SimTypeBottom(label="ITfActiveLanguageProfileNotifySink"),
    "ITfCandidateList": SimTypeBottom(label="ITfCandidateList"),
    "ITfCandidateListUIElement": SimTypeBottom(label="ITfCandidateListUIElement"),
    "ITfCandidateListUIElementBehavior": SimTypeBottom(label="ITfCandidateListUIElementBehavior"),
    "ITfCandidateString": SimTypeBottom(label="ITfCandidateString"),
    "ITfCategoryMgr": SimTypeBottom(label="ITfCategoryMgr"),
    "ITfCleanupContextDurationSink": SimTypeBottom(label="ITfCleanupContextDurationSink"),
    "ITfCleanupContextSink": SimTypeBottom(label="ITfCleanupContextSink"),
    "ITfClientId": SimTypeBottom(label="ITfClientId"),
    "ITfCompartment": SimTypeBottom(label="ITfCompartment"),
    "ITfCompartmentEventSink": SimTypeBottom(label="ITfCompartmentEventSink"),
    "ITfCompartmentMgr": SimTypeBottom(label="ITfCompartmentMgr"),
    "ITfComposition": SimTypeBottom(label="ITfComposition"),
    "ITfCompositionSink": SimTypeBottom(label="ITfCompositionSink"),
    "ITfCompositionView": SimTypeBottom(label="ITfCompositionView"),
    "ITfConfigureSystemKeystrokeFeed": SimTypeBottom(label="ITfConfigureSystemKeystrokeFeed"),
    "ITfContext": SimTypeBottom(label="ITfContext"),
    "ITfContextComposition": SimTypeBottom(label="ITfContextComposition"),
    "ITfContextKeyEventSink": SimTypeBottom(label="ITfContextKeyEventSink"),
    "ITfContextOwner": SimTypeBottom(label="ITfContextOwner"),
    "ITfContextOwnerCompositionServices": SimTypeBottom(label="ITfContextOwnerCompositionServices"),
    "ITfContextOwnerCompositionSink": SimTypeBottom(label="ITfContextOwnerCompositionSink"),
    "ITfContextOwnerServices": SimTypeBottom(label="ITfContextOwnerServices"),
    "ITfContextView": SimTypeBottom(label="ITfContextView"),
    "ITfCreatePropertyStore": SimTypeBottom(label="ITfCreatePropertyStore"),
    "ITfDisplayAttributeInfo": SimTypeBottom(label="ITfDisplayAttributeInfo"),
    "ITfDisplayAttributeMgr": SimTypeBottom(label="ITfDisplayAttributeMgr"),
    "ITfDisplayAttributeNotifySink": SimTypeBottom(label="ITfDisplayAttributeNotifySink"),
    "ITfDisplayAttributeProvider": SimTypeBottom(label="ITfDisplayAttributeProvider"),
    "ITfDocumentMgr": SimTypeBottom(label="ITfDocumentMgr"),
    "ITfEditRecord": SimTypeBottom(label="ITfEditRecord"),
    "ITfEditSession": SimTypeBottom(label="ITfEditSession"),
    "ITfEditTransactionSink": SimTypeBottom(label="ITfEditTransactionSink"),
    "ITfFnAdviseText": SimTypeBottom(label="ITfFnAdviseText"),
    "ITfFnBalloon": SimTypeBottom(label="ITfFnBalloon"),
    "ITfFnConfigure": SimTypeBottom(label="ITfFnConfigure"),
    "ITfFnConfigureRegisterEudc": SimTypeBottom(label="ITfFnConfigureRegisterEudc"),
    "ITfFnConfigureRegisterWord": SimTypeBottom(label="ITfFnConfigureRegisterWord"),
    "ITfFnCustomSpeechCommand": SimTypeBottom(label="ITfFnCustomSpeechCommand"),
    "ITfFnGetLinguisticAlternates": SimTypeBottom(label="ITfFnGetLinguisticAlternates"),
    "ITfFnGetPreferredTouchKeyboardLayout": SimTypeBottom(label="ITfFnGetPreferredTouchKeyboardLayout"),
    "ITfFnGetSAPIObject": SimTypeBottom(label="ITfFnGetSAPIObject"),
    "ITfFnLMInternal": SimTypeBottom(label="ITfFnLMInternal"),
    "ITfFnLMProcessor": SimTypeBottom(label="ITfFnLMProcessor"),
    "ITfFnLangProfileUtil": SimTypeBottom(label="ITfFnLangProfileUtil"),
    "ITfFnPlayBack": SimTypeBottom(label="ITfFnPlayBack"),
    "ITfFnPropertyUIStatus": SimTypeBottom(label="ITfFnPropertyUIStatus"),
    "ITfFnReconversion": SimTypeBottom(label="ITfFnReconversion"),
    "ITfFnSearchCandidateProvider": SimTypeBottom(label="ITfFnSearchCandidateProvider"),
    "ITfFnShowHelp": SimTypeBottom(label="ITfFnShowHelp"),
    "ITfFunction": SimTypeBottom(label="ITfFunction"),
    "ITfFunctionProvider": SimTypeBottom(label="ITfFunctionProvider"),
    "ITfInputProcessorProfileActivationSink": SimTypeBottom(label="ITfInputProcessorProfileActivationSink"),
    "ITfInputProcessorProfileMgr": SimTypeBottom(label="ITfInputProcessorProfileMgr"),
    "ITfInputProcessorProfileSubstituteLayout": SimTypeBottom(label="ITfInputProcessorProfileSubstituteLayout"),
    "ITfInputProcessorProfiles": SimTypeBottom(label="ITfInputProcessorProfiles"),
    "ITfInputProcessorProfilesEx": SimTypeBottom(label="ITfInputProcessorProfilesEx"),
    "ITfInputScope": SimTypeBottom(label="ITfInputScope"),
    "ITfInputScope2": SimTypeBottom(label="ITfInputScope2"),
    "ITfInsertAtSelection": SimTypeBottom(label="ITfInsertAtSelection"),
    "ITfIntegratableCandidateListUIElement": SimTypeBottom(label="ITfIntegratableCandidateListUIElement"),
    "ITfKeyEventSink": SimTypeBottom(label="ITfKeyEventSink"),
    "ITfKeyTraceEventSink": SimTypeBottom(label="ITfKeyTraceEventSink"),
    "ITfKeystrokeMgr": SimTypeBottom(label="ITfKeystrokeMgr"),
    "ITfLMLattice": SimTypeBottom(label="ITfLMLattice"),
    "ITfLangBarEventSink": SimTypeBottom(label="ITfLangBarEventSink"),
    "ITfLangBarItem": SimTypeBottom(label="ITfLangBarItem"),
    "ITfLangBarItemBalloon": SimTypeBottom(label="ITfLangBarItemBalloon"),
    "ITfLangBarItemBitmap": SimTypeBottom(label="ITfLangBarItemBitmap"),
    "ITfLangBarItemBitmapButton": SimTypeBottom(label="ITfLangBarItemBitmapButton"),
    "ITfLangBarItemButton": SimTypeBottom(label="ITfLangBarItemButton"),
    "ITfLangBarItemMgr": SimTypeBottom(label="ITfLangBarItemMgr"),
    "ITfLangBarItemSink": SimTypeBottom(label="ITfLangBarItemSink"),
    "ITfLangBarMgr": SimTypeBottom(label="ITfLangBarMgr"),
    "ITfLanguageProfileNotifySink": SimTypeBottom(label="ITfLanguageProfileNotifySink"),
    "ITfMSAAControl": SimTypeBottom(label="ITfMSAAControl"),
    "ITfMenu": SimTypeBottom(label="ITfMenu"),
    "ITfMessagePump": SimTypeBottom(label="ITfMessagePump"),
    "ITfMouseSink": SimTypeBottom(label="ITfMouseSink"),
    "ITfMouseTracker": SimTypeBottom(label="ITfMouseTracker"),
    "ITfMouseTrackerACP": SimTypeBottom(label="ITfMouseTrackerACP"),
    "ITfPersistentPropertyLoaderACP": SimTypeBottom(label="ITfPersistentPropertyLoaderACP"),
    "ITfPreservedKeyNotifySink": SimTypeBottom(label="ITfPreservedKeyNotifySink"),
    "ITfProperty": SimTypeBottom(label="ITfProperty"),
    "ITfPropertyStore": SimTypeBottom(label="ITfPropertyStore"),
    "ITfQueryEmbedded": SimTypeBottom(label="ITfQueryEmbedded"),
    "ITfRange": SimTypeBottom(label="ITfRange"),
    "ITfRangeACP": SimTypeBottom(label="ITfRangeACP"),
    "ITfRangeBackup": SimTypeBottom(label="ITfRangeBackup"),
    "ITfReadOnlyProperty": SimTypeBottom(label="ITfReadOnlyProperty"),
    "ITfReadingInformationUIElement": SimTypeBottom(label="ITfReadingInformationUIElement"),
    "ITfReverseConversion": SimTypeBottom(label="ITfReverseConversion"),
    "ITfReverseConversionList": SimTypeBottom(label="ITfReverseConversionList"),
    "ITfReverseConversionMgr": SimTypeBottom(label="ITfReverseConversionMgr"),
    "ITfSource": SimTypeBottom(label="ITfSource"),
    "ITfSourceSingle": SimTypeBottom(label="ITfSourceSingle"),
    "ITfSpeechUIServer": SimTypeBottom(label="ITfSpeechUIServer"),
    "ITfStatusSink": SimTypeBottom(label="ITfStatusSink"),
    "ITfSystemDeviceTypeLangBarItem": SimTypeBottom(label="ITfSystemDeviceTypeLangBarItem"),
    "ITfSystemLangBarItem": SimTypeBottom(label="ITfSystemLangBarItem"),
    "ITfSystemLangBarItemSink": SimTypeBottom(label="ITfSystemLangBarItemSink"),
    "ITfSystemLangBarItemText": SimTypeBottom(label="ITfSystemLangBarItemText"),
    "ITfTextEditSink": SimTypeBottom(label="ITfTextEditSink"),
    "ITfTextInputProcessor": SimTypeBottom(label="ITfTextInputProcessor"),
    "ITfTextInputProcessorEx": SimTypeBottom(label="ITfTextInputProcessorEx"),
    "ITfTextLayoutSink": SimTypeBottom(label="ITfTextLayoutSink"),
    "ITfThreadFocusSink": SimTypeBottom(label="ITfThreadFocusSink"),
    "ITfThreadMgr": SimTypeBottom(label="ITfThreadMgr"),
    "ITfThreadMgr2": SimTypeBottom(label="ITfThreadMgr2"),
    "ITfThreadMgrEventSink": SimTypeBottom(label="ITfThreadMgrEventSink"),
    "ITfThreadMgrEx": SimTypeBottom(label="ITfThreadMgrEx"),
    "ITfToolTipUIElement": SimTypeBottom(label="ITfToolTipUIElement"),
    "ITfTransitoryExtensionSink": SimTypeBottom(label="ITfTransitoryExtensionSink"),
    "ITfTransitoryExtensionUIElement": SimTypeBottom(label="ITfTransitoryExtensionUIElement"),
    "ITfUIElement": SimTypeBottom(label="ITfUIElement"),
    "ITfUIElementMgr": SimTypeBottom(label="ITfUIElementMgr"),
    "ITfUIElementSink": SimTypeBottom(label="ITfUIElementSink"),
    "IThreadPoolKnobs": SimTypeBottom(label="IThreadPoolKnobs"),
    "IThumbnailCache": SimTypeBottom(label="IThumbnailCache"),
    "IThumbnailCachePrimer": SimTypeBottom(label="IThumbnailCachePrimer"),
    "IThumbnailCapture": SimTypeBottom(label="IThumbnailCapture"),
    "IThumbnailExtractor": SimTypeBottom(label="IThumbnailExtractor"),
    "IThumbnailHandlerFactory": SimTypeBottom(label="IThumbnailHandlerFactory"),
    "IThumbnailProvider": SimTypeBottom(label="IThumbnailProvider"),
    "IThumbnailSettings": SimTypeBottom(label="IThumbnailSettings"),
    "IThumbnailStreamCache": SimTypeBottom(label="IThumbnailStreamCache"),
    "ITimeAndNoticeControl": SimTypeBottom(label="ITimeAndNoticeControl"),
    "ITimeTrigger": SimTypeBottom(label="ITimeTrigger"),
    "ITimer": SimTypeBottom(label="ITimer"),
    "ITimerEx": SimTypeBottom(label="ITimerEx"),
    "ITimerService": SimTypeBottom(label="ITimerService"),
    "ITimerSink": SimTypeBottom(label="ITimerSink"),
    "ITipAutoCompleteClient": SimTypeBottom(label="ITipAutoCompleteClient"),
    "ITipAutoCompleteProvider": SimTypeBottom(label="ITipAutoCompleteProvider"),
    "ITipHelper": SimTypeBottom(label="ITipHelper"),
    "ITipPullSink": SimTypeBottom(label="ITipPullSink"),
    "ITipTransaction": SimTypeBottom(label="ITipTransaction"),
    "ITmNodeName": SimTypeBottom(label="ITmNodeName"),
    "ITnef": SimTypeBottom(label="ITnef"),
    "IToc": SimTypeBottom(label="IToc"),
    "ITocCollection": SimTypeBottom(label="ITocCollection"),
    "ITocEntry": SimTypeBottom(label="ITocEntry"),
    "ITocEntryList": SimTypeBottom(label="ITocEntryList"),
    "ITocParser": SimTypeBottom(label="ITocParser"),
    "IToggleProvider": SimTypeBottom(label="IToggleProvider"),
    "ITokenCollection": SimTypeBottom(label="ITokenCollection"),
    "IToolbar": SimTypeBottom(label="IToolbar"),
    "ITpmVirtualSmartCardManager": SimTypeBottom(label="ITpmVirtualSmartCardManager"),
    "ITpmVirtualSmartCardManager2": SimTypeBottom(label="ITpmVirtualSmartCardManager2"),
    "ITpmVirtualSmartCardManager3": SimTypeBottom(label="ITpmVirtualSmartCardManager3"),
    "ITpmVirtualSmartCardManagerStatusCallback": SimTypeBottom(label="ITpmVirtualSmartCardManagerStatusCallback"),
    "ITraceDataCollector": SimTypeBottom(label="ITraceDataCollector"),
    "ITraceDataProvider": SimTypeBottom(label="ITraceDataProvider"),
    "ITraceDataProviderCollection": SimTypeBottom(label="ITraceDataProviderCollection"),
    "ITraceEvent": SimTypeBottom(label="ITraceEvent"),
    "ITraceEventCallback": SimTypeBottom(label="ITraceEventCallback"),
    "ITraceRelogger": SimTypeBottom(label="ITraceRelogger"),
    "ITrackShellMenu": SimTypeBottom(label="ITrackShellMenu"),
    "ITrackingProtection": SimTypeBottom(label="ITrackingProtection"),
    "ITransaction": SimTypeBottom(label="ITransaction"),
    "ITransaction2": SimTypeBottom(label="ITransaction2"),
    "ITransactionCloner": SimTypeBottom(label="ITransactionCloner"),
    "ITransactionContext": SimTypeBottom(label="ITransactionContext"),
    "ITransactionContextEx": SimTypeBottom(label="ITransactionContextEx"),
    "ITransactionDispenser": SimTypeBottom(label="ITransactionDispenser"),
    "ITransactionEnlistmentAsync": SimTypeBottom(label="ITransactionEnlistmentAsync"),
    "ITransactionExport": SimTypeBottom(label="ITransactionExport"),
    "ITransactionExportFactory": SimTypeBottom(label="ITransactionExportFactory"),
    "ITransactionImport": SimTypeBottom(label="ITransactionImport"),
    "ITransactionImportWhereabouts": SimTypeBottom(label="ITransactionImportWhereabouts"),
    "ITransactionJoin": SimTypeBottom(label="ITransactionJoin"),
    "ITransactionLastEnlistmentAsync": SimTypeBottom(label="ITransactionLastEnlistmentAsync"),
    "ITransactionLastResourceAsync": SimTypeBottom(label="ITransactionLastResourceAsync"),
    "ITransactionLocal": SimTypeBottom(label="ITransactionLocal"),
    "ITransactionObject": SimTypeBottom(label="ITransactionObject"),
    "ITransactionOptions": SimTypeBottom(label="ITransactionOptions"),
    "ITransactionOutcomeEvents": SimTypeBottom(label="ITransactionOutcomeEvents"),
    "ITransactionPhase0EnlistmentAsync": SimTypeBottom(label="ITransactionPhase0EnlistmentAsync"),
    "ITransactionPhase0Factory": SimTypeBottom(label="ITransactionPhase0Factory"),
    "ITransactionPhase0NotifyAsync": SimTypeBottom(label="ITransactionPhase0NotifyAsync"),
    "ITransactionProperty": SimTypeBottom(label="ITransactionProperty"),
    "ITransactionProxy": SimTypeBottom(label="ITransactionProxy"),
    "ITransactionReceiver": SimTypeBottom(label="ITransactionReceiver"),
    "ITransactionReceiverFactory": SimTypeBottom(label="ITransactionReceiverFactory"),
    "ITransactionResource": SimTypeBottom(label="ITransactionResource"),
    "ITransactionResourceAsync": SimTypeBottom(label="ITransactionResourceAsync"),
    "ITransactionResourcePool": SimTypeBottom(label="ITransactionResourcePool"),
    "ITransactionStatus": SimTypeBottom(label="ITransactionStatus"),
    "ITransactionTransmitter": SimTypeBottom(label="ITransactionTransmitter"),
    "ITransactionTransmitterFactory": SimTypeBottom(label="ITransactionTransmitterFactory"),
    "ITransactionVoterBallotAsync2": SimTypeBottom(label="ITransactionVoterBallotAsync2"),
    "ITransactionVoterFactory2": SimTypeBottom(label="ITransactionVoterFactory2"),
    "ITransactionVoterNotifyAsync2": SimTypeBottom(label="ITransactionVoterNotifyAsync2"),
    "ITranscodeImage": SimTypeBottom(label="ITranscodeImage"),
    "ITransferAdviseSink": SimTypeBottom(label="ITransferAdviseSink"),
    "ITransferDestination": SimTypeBottom(label="ITransferDestination"),
    "ITransferMediumItem": SimTypeBottom(label="ITransferMediumItem"),
    "ITransferSource": SimTypeBottom(label="ITransferSource"),
    "ITransformProvider": SimTypeBottom(label="ITransformProvider"),
    "ITransformProvider2": SimTypeBottom(label="ITransformProvider2"),
    "ITransportSettingsInternal": SimTypeBottom(label="ITransportSettingsInternal"),
    "ITravelEntry": SimTypeBottom(label="ITravelEntry"),
    "ITravelLog": SimTypeBottom(label="ITravelLog"),
    "ITravelLogClient": SimTypeBottom(label="ITravelLogClient"),
    "ITravelLogEntry": SimTypeBottom(label="ITravelLogEntry"),
    "ITravelLogStg": SimTypeBottom(label="ITravelLogStg"),
    "ITrayDeskBand": SimTypeBottom(label="ITrayDeskBand"),
    "ITridentEventSink": SimTypeBottom(label="ITridentEventSink"),
    "ITridentTouchInput": SimTypeBottom(label="ITridentTouchInput"),
    "ITridentTouchInputSite": SimTypeBottom(label="ITridentTouchInputSite"),
    "ITrigger": SimTypeBottom(label="ITrigger"),
    "ITriggerCollection": SimTypeBottom(label="ITriggerCollection"),
    "ITrusteeAdmin": SimTypeBottom(label="ITrusteeAdmin"),
    "ITrusteeGroupAdmin": SimTypeBottom(label="ITrusteeGroupAdmin"),
    "ITsSbBaseNotifySink": SimTypeBottom(label="ITsSbBaseNotifySink"),
    "ITsSbClientConnection": SimTypeBottom(label="ITsSbClientConnection"),
    "ITsSbClientConnectionPropertySet": SimTypeBottom(label="ITsSbClientConnectionPropertySet"),
    "ITsSbEnvironment": SimTypeBottom(label="ITsSbEnvironment"),
    "ITsSbEnvironmentPropertySet": SimTypeBottom(label="ITsSbEnvironmentPropertySet"),
    "ITsSbFilterPluginStore": SimTypeBottom(label="ITsSbFilterPluginStore"),
    "ITsSbGenericNotifySink": SimTypeBottom(label="ITsSbGenericNotifySink"),
    "ITsSbGlobalStore": SimTypeBottom(label="ITsSbGlobalStore"),
    "ITsSbLoadBalanceResult": SimTypeBottom(label="ITsSbLoadBalanceResult"),
    "ITsSbLoadBalancing": SimTypeBottom(label="ITsSbLoadBalancing"),
    "ITsSbLoadBalancingNotifySink": SimTypeBottom(label="ITsSbLoadBalancingNotifySink"),
    "ITsSbOrchestration": SimTypeBottom(label="ITsSbOrchestration"),
    "ITsSbOrchestrationNotifySink": SimTypeBottom(label="ITsSbOrchestrationNotifySink"),
    "ITsSbPlacement": SimTypeBottom(label="ITsSbPlacement"),
    "ITsSbPlacementNotifySink": SimTypeBottom(label="ITsSbPlacementNotifySink"),
    "ITsSbPlugin": SimTypeBottom(label="ITsSbPlugin"),
    "ITsSbPluginNotifySink": SimTypeBottom(label="ITsSbPluginNotifySink"),
    "ITsSbPluginPropertySet": SimTypeBottom(label="ITsSbPluginPropertySet"),
    "ITsSbPropertySet": SimTypeBottom(label="ITsSbPropertySet"),
    "ITsSbProvider": SimTypeBottom(label="ITsSbProvider"),
    "ITsSbProvisioning": SimTypeBottom(label="ITsSbProvisioning"),
    "ITsSbProvisioningPluginNotifySink": SimTypeBottom(label="ITsSbProvisioningPluginNotifySink"),
    "ITsSbResourceNotification": SimTypeBottom(label="ITsSbResourceNotification"),
    "ITsSbResourceNotificationEx": SimTypeBottom(label="ITsSbResourceNotificationEx"),
    "ITsSbResourcePlugin": SimTypeBottom(label="ITsSbResourcePlugin"),
    "ITsSbResourcePluginStore": SimTypeBottom(label="ITsSbResourcePluginStore"),
    "ITsSbServiceNotification": SimTypeBottom(label="ITsSbServiceNotification"),
    "ITsSbSession": SimTypeBottom(label="ITsSbSession"),
    "ITsSbTarget": SimTypeBottom(label="ITsSbTarget"),
    "ITsSbTargetPropertySet": SimTypeBottom(label="ITsSbTargetPropertySet"),
    "ITsSbTaskInfo": SimTypeBottom(label="ITsSbTaskInfo"),
    "ITsSbTaskPlugin": SimTypeBottom(label="ITsSbTaskPlugin"),
    "ITsSbTaskPluginNotifySink": SimTypeBottom(label="ITsSbTaskPluginNotifySink"),
    "ITuneRequest": SimTypeBottom(label="ITuneRequest"),
    "ITuneRequestInfo": SimTypeBottom(label="ITuneRequestInfo"),
    "ITuneRequestInfoEx": SimTypeBottom(label="ITuneRequestInfoEx"),
    "ITuner": SimTypeBottom(label="ITuner"),
    "ITunerCap": SimTypeBottom(label="ITunerCap"),
    "ITunerCapEx": SimTypeBottom(label="ITunerCapEx"),
    "ITuningSpace": SimTypeBottom(label="ITuningSpace"),
    "ITuningSpaceContainer": SimTypeBottom(label="ITuningSpaceContainer"),
    "ITuningSpaces": SimTypeBottom(label="ITuningSpaces"),
    "ITxProxyHolder": SimTypeBottom(label="ITxProxyHolder"),
    "ITypeChangeEvents": SimTypeBottom(label="ITypeChangeEvents"),
    "ITypeComp": SimTypeBottom(label="ITypeComp"),
    "ITypeFactory": SimTypeBottom(label="ITypeFactory"),
    "ITypeInfo": SimTypeBottom(label="ITypeInfo"),
    "ITypeInfo2": SimTypeBottom(label="ITypeInfo2"),
    "ITypeLib": SimTypeBottom(label="ITypeLib"),
    "ITypeLib2": SimTypeBottom(label="ITypeLib2"),
    "ITypeLibRegistration": SimTypeBottom(label="ITypeLibRegistration"),
    "ITypeLibRegistrationReader": SimTypeBottom(label="ITypeLibRegistrationReader"),
    "ITypeMarshal": SimTypeBottom(label="ITypeMarshal"),
    "ITypeName": SimTypeBottom(label="ITypeName"),
    "ITypeNameBuilder": SimTypeBottom(label="ITypeNameBuilder"),
    "ITypeNameFactory": SimTypeBottom(label="ITypeNameFactory"),
    "IUIAnimationInterpolator": SimTypeBottom(label="IUIAnimationInterpolator"),
    "IUIAnimationInterpolator2": SimTypeBottom(label="IUIAnimationInterpolator2"),
    "IUIAnimationLoopIterationChangeHandler2": SimTypeBottom(label="IUIAnimationLoopIterationChangeHandler2"),
    "IUIAnimationManager": SimTypeBottom(label="IUIAnimationManager"),
    "IUIAnimationManager2": SimTypeBottom(label="IUIAnimationManager2"),
    "IUIAnimationManagerEventHandler": SimTypeBottom(label="IUIAnimationManagerEventHandler"),
    "IUIAnimationManagerEventHandler2": SimTypeBottom(label="IUIAnimationManagerEventHandler2"),
    "IUIAnimationPrimitiveInterpolation": SimTypeBottom(label="IUIAnimationPrimitiveInterpolation"),
    "IUIAnimationPriorityComparison": SimTypeBottom(label="IUIAnimationPriorityComparison"),
    "IUIAnimationPriorityComparison2": SimTypeBottom(label="IUIAnimationPriorityComparison2"),
    "IUIAnimationStoryboard": SimTypeBottom(label="IUIAnimationStoryboard"),
    "IUIAnimationStoryboard2": SimTypeBottom(label="IUIAnimationStoryboard2"),
    "IUIAnimationStoryboardEventHandler": SimTypeBottom(label="IUIAnimationStoryboardEventHandler"),
    "IUIAnimationStoryboardEventHandler2": SimTypeBottom(label="IUIAnimationStoryboardEventHandler2"),
    "IUIAnimationTimer": SimTypeBottom(label="IUIAnimationTimer"),
    "IUIAnimationTimerClientEventHandler": SimTypeBottom(label="IUIAnimationTimerClientEventHandler"),
    "IUIAnimationTimerEventHandler": SimTypeBottom(label="IUIAnimationTimerEventHandler"),
    "IUIAnimationTimerUpdateHandler": SimTypeBottom(label="IUIAnimationTimerUpdateHandler"),
    "IUIAnimationTransition": SimTypeBottom(label="IUIAnimationTransition"),
    "IUIAnimationTransition2": SimTypeBottom(label="IUIAnimationTransition2"),
    "IUIAnimationTransitionFactory": SimTypeBottom(label="IUIAnimationTransitionFactory"),
    "IUIAnimationTransitionFactory2": SimTypeBottom(label="IUIAnimationTransitionFactory2"),
    "IUIAnimationTransitionLibrary": SimTypeBottom(label="IUIAnimationTransitionLibrary"),
    "IUIAnimationTransitionLibrary2": SimTypeBottom(label="IUIAnimationTransitionLibrary2"),
    "IUIAnimationVariable": SimTypeBottom(label="IUIAnimationVariable"),
    "IUIAnimationVariable2": SimTypeBottom(label="IUIAnimationVariable2"),
    "IUIAnimationVariableChangeHandler": SimTypeBottom(label="IUIAnimationVariableChangeHandler"),
    "IUIAnimationVariableChangeHandler2": SimTypeBottom(label="IUIAnimationVariableChangeHandler2"),
    "IUIAnimationVariableCurveChangeHandler2": SimTypeBottom(label="IUIAnimationVariableCurveChangeHandler2"),
    "IUIAnimationVariableIntegerChangeHandler": SimTypeBottom(label="IUIAnimationVariableIntegerChangeHandler"),
    "IUIAnimationVariableIntegerChangeHandler2": SimTypeBottom(label="IUIAnimationVariableIntegerChangeHandler2"),
    "IUIApplication": SimTypeBottom(label="IUIApplication"),
    "IUIAutomation": SimTypeBottom(label="IUIAutomation"),
    "IUIAutomation2": SimTypeBottom(label="IUIAutomation2"),
    "IUIAutomation3": SimTypeBottom(label="IUIAutomation3"),
    "IUIAutomation4": SimTypeBottom(label="IUIAutomation4"),
    "IUIAutomation5": SimTypeBottom(label="IUIAutomation5"),
    "IUIAutomation6": SimTypeBottom(label="IUIAutomation6"),
    "IUIAutomationActiveTextPositionChangedEventHandler": SimTypeBottom(label="IUIAutomationActiveTextPositionChangedEventHandler"),
    "IUIAutomationAndCondition": SimTypeBottom(label="IUIAutomationAndCondition"),
    "IUIAutomationAnnotationPattern": SimTypeBottom(label="IUIAutomationAnnotationPattern"),
    "IUIAutomationBoolCondition": SimTypeBottom(label="IUIAutomationBoolCondition"),
    "IUIAutomationCacheRequest": SimTypeBottom(label="IUIAutomationCacheRequest"),
    "IUIAutomationChangesEventHandler": SimTypeBottom(label="IUIAutomationChangesEventHandler"),
    "IUIAutomationCondition": SimTypeBottom(label="IUIAutomationCondition"),
    "IUIAutomationCustomNavigationPattern": SimTypeBottom(label="IUIAutomationCustomNavigationPattern"),
    "IUIAutomationDockPattern": SimTypeBottom(label="IUIAutomationDockPattern"),
    "IUIAutomationDragPattern": SimTypeBottom(label="IUIAutomationDragPattern"),
    "IUIAutomationDropTargetPattern": SimTypeBottom(label="IUIAutomationDropTargetPattern"),
    "IUIAutomationElement": SimTypeBottom(label="IUIAutomationElement"),
    "IUIAutomationElement2": SimTypeBottom(label="IUIAutomationElement2"),
    "IUIAutomationElement3": SimTypeBottom(label="IUIAutomationElement3"),
    "IUIAutomationElement4": SimTypeBottom(label="IUIAutomationElement4"),
    "IUIAutomationElement5": SimTypeBottom(label="IUIAutomationElement5"),
    "IUIAutomationElement6": SimTypeBottom(label="IUIAutomationElement6"),
    "IUIAutomationElement7": SimTypeBottom(label="IUIAutomationElement7"),
    "IUIAutomationElement8": SimTypeBottom(label="IUIAutomationElement8"),
    "IUIAutomationElement9": SimTypeBottom(label="IUIAutomationElement9"),
    "IUIAutomationElementArray": SimTypeBottom(label="IUIAutomationElementArray"),
    "IUIAutomationEventHandler": SimTypeBottom(label="IUIAutomationEventHandler"),
    "IUIAutomationEventHandlerGroup": SimTypeBottom(label="IUIAutomationEventHandlerGroup"),
    "IUIAutomationExpandCollapsePattern": SimTypeBottom(label="IUIAutomationExpandCollapsePattern"),
    "IUIAutomationFocusChangedEventHandler": SimTypeBottom(label="IUIAutomationFocusChangedEventHandler"),
    "IUIAutomationGridItemPattern": SimTypeBottom(label="IUIAutomationGridItemPattern"),
    "IUIAutomationGridPattern": SimTypeBottom(label="IUIAutomationGridPattern"),
    "IUIAutomationInvokePattern": SimTypeBottom(label="IUIAutomationInvokePattern"),
    "IUIAutomationItemContainerPattern": SimTypeBottom(label="IUIAutomationItemContainerPattern"),
    "IUIAutomationLegacyIAccessiblePattern": SimTypeBottom(label="IUIAutomationLegacyIAccessiblePattern"),
    "IUIAutomationMultipleViewPattern": SimTypeBottom(label="IUIAutomationMultipleViewPattern"),
    "IUIAutomationNotCondition": SimTypeBottom(label="IUIAutomationNotCondition"),
    "IUIAutomationNotificationEventHandler": SimTypeBottom(label="IUIAutomationNotificationEventHandler"),
    "IUIAutomationObjectModelPattern": SimTypeBottom(label="IUIAutomationObjectModelPattern"),
    "IUIAutomationOrCondition": SimTypeBottom(label="IUIAutomationOrCondition"),
    "IUIAutomationPatternHandler": SimTypeBottom(label="IUIAutomationPatternHandler"),
    "IUIAutomationPatternInstance": SimTypeBottom(label="IUIAutomationPatternInstance"),
    "IUIAutomationPropertyChangedEventHandler": SimTypeBottom(label="IUIAutomationPropertyChangedEventHandler"),
    "IUIAutomationPropertyCondition": SimTypeBottom(label="IUIAutomationPropertyCondition"),
    "IUIAutomationProxyFactory": SimTypeBottom(label="IUIAutomationProxyFactory"),
    "IUIAutomationProxyFactoryEntry": SimTypeBottom(label="IUIAutomationProxyFactoryEntry"),
    "IUIAutomationProxyFactoryMapping": SimTypeBottom(label="IUIAutomationProxyFactoryMapping"),
    "IUIAutomationRangeValuePattern": SimTypeBottom(label="IUIAutomationRangeValuePattern"),
    "IUIAutomationRegistrar": SimTypeBottom(label="IUIAutomationRegistrar"),
    "IUIAutomationScrollItemPattern": SimTypeBottom(label="IUIAutomationScrollItemPattern"),
    "IUIAutomationScrollPattern": SimTypeBottom(label="IUIAutomationScrollPattern"),
    "IUIAutomationSelectionItemPattern": SimTypeBottom(label="IUIAutomationSelectionItemPattern"),
    "IUIAutomationSelectionPattern": SimTypeBottom(label="IUIAutomationSelectionPattern"),
    "IUIAutomationSelectionPattern2": SimTypeBottom(label="IUIAutomationSelectionPattern2"),
    "IUIAutomationSpreadsheetItemPattern": SimTypeBottom(label="IUIAutomationSpreadsheetItemPattern"),
    "IUIAutomationSpreadsheetPattern": SimTypeBottom(label="IUIAutomationSpreadsheetPattern"),
    "IUIAutomationStructureChangedEventHandler": SimTypeBottom(label="IUIAutomationStructureChangedEventHandler"),
    "IUIAutomationStylesPattern": SimTypeBottom(label="IUIAutomationStylesPattern"),
    "IUIAutomationSynchronizedInputPattern": SimTypeBottom(label="IUIAutomationSynchronizedInputPattern"),
    "IUIAutomationTableItemPattern": SimTypeBottom(label="IUIAutomationTableItemPattern"),
    "IUIAutomationTablePattern": SimTypeBottom(label="IUIAutomationTablePattern"),
    "IUIAutomationTextChildPattern": SimTypeBottom(label="IUIAutomationTextChildPattern"),
    "IUIAutomationTextEditPattern": SimTypeBottom(label="IUIAutomationTextEditPattern"),
    "IUIAutomationTextEditTextChangedEventHandler": SimTypeBottom(label="IUIAutomationTextEditTextChangedEventHandler"),
    "IUIAutomationTextPattern": SimTypeBottom(label="IUIAutomationTextPattern"),
    "IUIAutomationTextPattern2": SimTypeBottom(label="IUIAutomationTextPattern2"),
    "IUIAutomationTextRange": SimTypeBottom(label="IUIAutomationTextRange"),
    "IUIAutomationTextRange2": SimTypeBottom(label="IUIAutomationTextRange2"),
    "IUIAutomationTextRange3": SimTypeBottom(label="IUIAutomationTextRange3"),
    "IUIAutomationTextRangeArray": SimTypeBottom(label="IUIAutomationTextRangeArray"),
    "IUIAutomationTogglePattern": SimTypeBottom(label="IUIAutomationTogglePattern"),
    "IUIAutomationTransformPattern": SimTypeBottom(label="IUIAutomationTransformPattern"),
    "IUIAutomationTransformPattern2": SimTypeBottom(label="IUIAutomationTransformPattern2"),
    "IUIAutomationTreeWalker": SimTypeBottom(label="IUIAutomationTreeWalker"),
    "IUIAutomationValuePattern": SimTypeBottom(label="IUIAutomationValuePattern"),
    "IUIAutomationVirtualizedItemPattern": SimTypeBottom(label="IUIAutomationVirtualizedItemPattern"),
    "IUIAutomationWindowPattern": SimTypeBottom(label="IUIAutomationWindowPattern"),
    "IUICollection": SimTypeBottom(label="IUICollection"),
    "IUICollectionChangedEvent": SimTypeBottom(label="IUICollectionChangedEvent"),
    "IUICommandHandler": SimTypeBottom(label="IUICommandHandler"),
    "IUIContextualUI": SimTypeBottom(label="IUIContextualUI"),
    "IUIEventLogger": SimTypeBottom(label="IUIEventLogger"),
    "IUIEventingManager": SimTypeBottom(label="IUIEventingManager"),
    "IUIFramework": SimTypeBottom(label="IUIFramework"),
    "IUIImage": SimTypeBottom(label="IUIImage"),
    "IUIImageFromBitmap": SimTypeBottom(label="IUIImageFromBitmap"),
    "IUIManagerEventSink": SimTypeBottom(label="IUIManagerEventSink"),
    "IUIRibbon": SimTypeBottom(label="IUIRibbon"),
    "IUISimplePropertySet": SimTypeBottom(label="IUISimplePropertySet"),
    "IUMS": SimTypeBottom(label="IUMS"),
    "IUMSInitialize": SimTypeBottom(label="IUMSInitialize"),
    "IUNKNOWN_LIST": SimStruct(OrderedDict((("lpLink", SimTypePointer(SimTypeRef("IUNKNOWN_LIST", SimStruct), offset=0)), ("lpGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpIUnknown", SimTypeBottom(label="IUnknown")),)), name="IUNKNOWN_LIST", pack=False, align=None),
    "IUPnPAddressFamilyControl": SimTypeBottom(label="IUPnPAddressFamilyControl"),
    "IUPnPAsyncResult": SimTypeBottom(label="IUPnPAsyncResult"),
    "IUPnPDescriptionDocument": SimTypeBottom(label="IUPnPDescriptionDocument"),
    "IUPnPDescriptionDocumentCallback": SimTypeBottom(label="IUPnPDescriptionDocumentCallback"),
    "IUPnPDevice": SimTypeBottom(label="IUPnPDevice"),
    "IUPnPDeviceControl": SimTypeBottom(label="IUPnPDeviceControl"),
    "IUPnPDeviceControlHttpHeaders": SimTypeBottom(label="IUPnPDeviceControlHttpHeaders"),
    "IUPnPDeviceDocumentAccess": SimTypeBottom(label="IUPnPDeviceDocumentAccess"),
    "IUPnPDeviceDocumentAccessEx": SimTypeBottom(label="IUPnPDeviceDocumentAccessEx"),
    "IUPnPDeviceFinder": SimTypeBottom(label="IUPnPDeviceFinder"),
    "IUPnPDeviceFinderAddCallbackWithInterface": SimTypeBottom(label="IUPnPDeviceFinderAddCallbackWithInterface"),
    "IUPnPDeviceFinderCallback": SimTypeBottom(label="IUPnPDeviceFinderCallback"),
    "IUPnPDeviceProvider": SimTypeBottom(label="IUPnPDeviceProvider"),
    "IUPnPDevices": SimTypeBottom(label="IUPnPDevices"),
    "IUPnPEventSink": SimTypeBottom(label="IUPnPEventSink"),
    "IUPnPEventSource": SimTypeBottom(label="IUPnPEventSource"),
    "IUPnPHttpHeaderControl": SimTypeBottom(label="IUPnPHttpHeaderControl"),
    "IUPnPNAT": SimTypeBottom(label="IUPnPNAT"),
    "IUPnPRegistrar": SimTypeBottom(label="IUPnPRegistrar"),
    "IUPnPRemoteEndpointInfo": SimTypeBottom(label="IUPnPRemoteEndpointInfo"),
    "IUPnPReregistrar": SimTypeBottom(label="IUPnPReregistrar"),
    "IUPnPService": SimTypeBottom(label="IUPnPService"),
    "IUPnPServiceAsync": SimTypeBottom(label="IUPnPServiceAsync"),
    "IUPnPServiceCallback": SimTypeBottom(label="IUPnPServiceCallback"),
    "IUPnPServiceDocumentAccess": SimTypeBottom(label="IUPnPServiceDocumentAccess"),
    "IUPnPServiceEnumProperty": SimTypeBottom(label="IUPnPServiceEnumProperty"),
    "IUPnPServices": SimTypeBottom(label="IUPnPServices"),
    "IURLSearchHook": SimTypeBottom(label="IURLSearchHook"),
    "IURLSearchHook2": SimTypeBottom(label="IURLSearchHook2"),
    "IURL_INVOKECOMMAND_FLAGS": SimTypeInt(signed=False, label="IURL_INVOKECOMMAND_FLAGS"),
    "IURL_SETURL_FLAGS": SimTypeInt(signed=False, label="IURL_SETURL_FLAGS"),
    "IUniformResourceLocatorA": SimTypeBottom(label="IUniformResourceLocatorA"),
    "IUniformResourceLocatorW": SimTypeBottom(label="IUniformResourceLocatorW"),
    "IUnknown": SimTypeBottom(label="IUnknown"),
    "IUnsecuredApartment": SimTypeBottom(label="IUnsecuredApartment"),
    "IUpdate": SimTypeBottom(label="IUpdate"),
    "IUpdate2": SimTypeBottom(label="IUpdate2"),
    "IUpdate3": SimTypeBottom(label="IUpdate3"),
    "IUpdate4": SimTypeBottom(label="IUpdate4"),
    "IUpdate5": SimTypeBottom(label="IUpdate5"),
    "IUpdateCollection": SimTypeBottom(label="IUpdateCollection"),
    "IUpdateDownloadContent": SimTypeBottom(label="IUpdateDownloadContent"),
    "IUpdateDownloadContent2": SimTypeBottom(label="IUpdateDownloadContent2"),
    "IUpdateDownloadContentCollection": SimTypeBottom(label="IUpdateDownloadContentCollection"),
    "IUpdateDownloadResult": SimTypeBottom(label="IUpdateDownloadResult"),
    "IUpdateDownloader": SimTypeBottom(label="IUpdateDownloader"),
    "IUpdateException": SimTypeBottom(label="IUpdateException"),
    "IUpdateExceptionCollection": SimTypeBottom(label="IUpdateExceptionCollection"),
    "IUpdateHistoryEntry": SimTypeBottom(label="IUpdateHistoryEntry"),
    "IUpdateHistoryEntry2": SimTypeBottom(label="IUpdateHistoryEntry2"),
    "IUpdateHistoryEntryCollection": SimTypeBottom(label="IUpdateHistoryEntryCollection"),
    "IUpdateIDList": SimTypeBottom(label="IUpdateIDList"),
    "IUpdateIdentity": SimTypeBottom(label="IUpdateIdentity"),
    "IUpdateInstallationResult": SimTypeBottom(label="IUpdateInstallationResult"),
    "IUpdateInstaller": SimTypeBottom(label="IUpdateInstaller"),
    "IUpdateInstaller2": SimTypeBottom(label="IUpdateInstaller2"),
    "IUpdateInstaller3": SimTypeBottom(label="IUpdateInstaller3"),
    "IUpdateInstaller4": SimTypeBottom(label="IUpdateInstaller4"),
    "IUpdateLockdown": SimTypeBottom(label="IUpdateLockdown"),
    "IUpdateSearcher": SimTypeBottom(label="IUpdateSearcher"),
    "IUpdateSearcher2": SimTypeBottom(label="IUpdateSearcher2"),
    "IUpdateSearcher3": SimTypeBottom(label="IUpdateSearcher3"),
    "IUpdateService": SimTypeBottom(label="IUpdateService"),
    "IUpdateService2": SimTypeBottom(label="IUpdateService2"),
    "IUpdateServiceCollection": SimTypeBottom(label="IUpdateServiceCollection"),
    "IUpdateServiceManager": SimTypeBottom(label="IUpdateServiceManager"),
    "IUpdateServiceManager2": SimTypeBottom(label="IUpdateServiceManager2"),
    "IUpdateServiceRegistration": SimTypeBottom(label="IUpdateServiceRegistration"),
    "IUpdateSession": SimTypeBottom(label="IUpdateSession"),
    "IUpdateSession2": SimTypeBottom(label="IUpdateSession2"),
    "IUpdateSession3": SimTypeBottom(label="IUpdateSession3"),
    "IUri": SimTypeBottom(label="IUri"),
    "IUriBuilder": SimTypeBottom(label="IUriBuilder"),
    "IUriBuilderFactory": SimTypeBottom(label="IUriBuilderFactory"),
    "IUriContainer": SimTypeBottom(label="IUriContainer"),
    "IUrlAccessor": SimTypeBottom(label="IUrlAccessor"),
    "IUrlAccessor2": SimTypeBottom(label="IUrlAccessor2"),
    "IUrlAccessor3": SimTypeBottom(label="IUrlAccessor3"),
    "IUrlAccessor4": SimTypeBottom(label="IUrlAccessor4"),
    "IUrlHistoryNotify": SimTypeBottom(label="IUrlHistoryNotify"),
    "IUrlHistoryStg": SimTypeBottom(label="IUrlHistoryStg"),
    "IUrlHistoryStg2": SimTypeBottom(label="IUrlHistoryStg2"),
    "IUrlMon": SimTypeBottom(label="IUrlMon"),
    "IUseToBrowseItem": SimTypeBottom(label="IUseToBrowseItem"),
    "IUserAccountChangeCallback": SimTypeBottom(label="IUserAccountChangeCallback"),
    "IUserDictionariesRegistrar": SimTypeBottom(label="IUserDictionariesRegistrar"),
    "IUserInputString": SimTypeBottom(label="IUserInputString"),
    "IUserNotification": SimTypeBottom(label="IUserNotification"),
    "IUserNotification2": SimTypeBottom(label="IUserNotification2"),
    "IUserNotificationCallback": SimTypeBottom(label="IUserNotificationCallback"),
    "IVBFormat": SimTypeBottom(label="IVBFormat"),
    "IVBGetControl": SimTypeBottom(label="IVBGetControl"),
    "IVBMXNamespaceManager": SimTypeBottom(label="IVBMXNamespaceManager"),
    "IVBSAXAttributes": SimTypeBottom(label="IVBSAXAttributes"),
    "IVBSAXContentHandler": SimTypeBottom(label="IVBSAXContentHandler"),
    "IVBSAXDTDHandler": SimTypeBottom(label="IVBSAXDTDHandler"),
    "IVBSAXDeclHandler": SimTypeBottom(label="IVBSAXDeclHandler"),
    "IVBSAXEntityResolver": SimTypeBottom(label="IVBSAXEntityResolver"),
    "IVBSAXErrorHandler": SimTypeBottom(label="IVBSAXErrorHandler"),
    "IVBSAXLexicalHandler": SimTypeBottom(label="IVBSAXLexicalHandler"),
    "IVBSAXLocator": SimTypeBottom(label="IVBSAXLocator"),
    "IVBSAXXMLFilter": SimTypeBottom(label="IVBSAXXMLFilter"),
    "IVBSAXXMLReader": SimTypeBottom(label="IVBSAXXMLReader"),
    "IVMRAspectRatioControl": SimTypeBottom(label="IVMRAspectRatioControl"),
    "IVMRAspectRatioControl9": SimTypeBottom(label="IVMRAspectRatioControl9"),
    "IVMRDeinterlaceControl": SimTypeBottom(label="IVMRDeinterlaceControl"),
    "IVMRDeinterlaceControl9": SimTypeBottom(label="IVMRDeinterlaceControl9"),
    "IVMRFilterConfig": SimTypeBottom(label="IVMRFilterConfig"),
    "IVMRFilterConfig9": SimTypeBottom(label="IVMRFilterConfig9"),
    "IVMRImageCompositor": SimTypeBottom(label="IVMRImageCompositor"),
    "IVMRImageCompositor9": SimTypeBottom(label="IVMRImageCompositor9"),
    "IVMRImagePresenter": SimTypeBottom(label="IVMRImagePresenter"),
    "IVMRImagePresenter9": SimTypeBottom(label="IVMRImagePresenter9"),
    "IVMRImagePresenterConfig": SimTypeBottom(label="IVMRImagePresenterConfig"),
    "IVMRImagePresenterConfig9": SimTypeBottom(label="IVMRImagePresenterConfig9"),
    "IVMRImagePresenterExclModeConfig": SimTypeBottom(label="IVMRImagePresenterExclModeConfig"),
    "IVMRMixerBitmap": SimTypeBottom(label="IVMRMixerBitmap"),
    "IVMRMixerBitmap9": SimTypeBottom(label="IVMRMixerBitmap9"),
    "IVMRMixerControl": SimTypeBottom(label="IVMRMixerControl"),
    "IVMRMixerControl9": SimTypeBottom(label="IVMRMixerControl9"),
    "IVMRMonitorConfig": SimTypeBottom(label="IVMRMonitorConfig"),
    "IVMRMonitorConfig9": SimTypeBottom(label="IVMRMonitorConfig9"),
    "IVMRSurface": SimTypeBottom(label="IVMRSurface"),
    "IVMRSurface9": SimTypeBottom(label="IVMRSurface9"),
    "IVMRSurfaceAllocator": SimTypeBottom(label="IVMRSurfaceAllocator"),
    "IVMRSurfaceAllocator9": SimTypeBottom(label="IVMRSurfaceAllocator9"),
    "IVMRSurfaceAllocatorEx9": SimTypeBottom(label="IVMRSurfaceAllocatorEx9"),
    "IVMRSurfaceAllocatorNotify": SimTypeBottom(label="IVMRSurfaceAllocatorNotify"),
    "IVMRSurfaceAllocatorNotify9": SimTypeBottom(label="IVMRSurfaceAllocatorNotify9"),
    "IVMRVideoStreamControl": SimTypeBottom(label="IVMRVideoStreamControl"),
    "IVMRVideoStreamControl9": SimTypeBottom(label="IVMRVideoStreamControl9"),
    "IVMRWindowlessControl": SimTypeBottom(label="IVMRWindowlessControl"),
    "IVMRWindowlessControl9": SimTypeBottom(label="IVMRWindowlessControl9"),
    "IVPBaseConfig": SimTypeBottom(label="IVPBaseConfig"),
    "IVPBaseNotify": SimTypeBottom(label="IVPBaseNotify"),
    "IVPConfig": SimTypeBottom(label="IVPConfig"),
    "IVPManager": SimTypeBottom(label="IVPManager"),
    "IVPNotify": SimTypeBottom(label="IVPNotify"),
    "IVPNotify2": SimTypeBottom(label="IVPNotify2"),
    "IVPVBIConfig": SimTypeBottom(label="IVPVBIConfig"),
    "IVPVBINotify": SimTypeBottom(label="IVPVBINotify"),
    "IValidate": SimTypeBottom(label="IValidate"),
    "IValidateBinding": SimTypeBottom(label="IValidateBinding"),
    "IValueMap": SimTypeBottom(label="IValueMap"),
    "IValueMapItem": SimTypeBottom(label="IValueMapItem"),
    "IValueProvider": SimTypeBottom(label="IValueProvider"),
    "IVariantChangeType": SimTypeBottom(label="IVariantChangeType"),
    "IVdsAdmin": SimTypeBottom(label="IVdsAdmin"),
    "IVdsAdvancedDisk": SimTypeBottom(label="IVdsAdvancedDisk"),
    "IVdsAdvancedDisk2": SimTypeBottom(label="IVdsAdvancedDisk2"),
    "IVdsAdvancedDisk3": SimTypeBottom(label="IVdsAdvancedDisk3"),
    "IVdsAdviseSink": SimTypeBottom(label="IVdsAdviseSink"),
    "IVdsAsync": SimTypeBottom(label="IVdsAsync"),
    "IVdsController": SimTypeBottom(label="IVdsController"),
    "IVdsControllerControllerPort": SimTypeBottom(label="IVdsControllerControllerPort"),
    "IVdsControllerPort": SimTypeBottom(label="IVdsControllerPort"),
    "IVdsCreatePartitionEx": SimTypeBottom(label="IVdsCreatePartitionEx"),
    "IVdsDisk": SimTypeBottom(label="IVdsDisk"),
    "IVdsDisk2": SimTypeBottom(label="IVdsDisk2"),
    "IVdsDisk3": SimTypeBottom(label="IVdsDisk3"),
    "IVdsDiskOnline": SimTypeBottom(label="IVdsDiskOnline"),
    "IVdsDiskPartitionMF": SimTypeBottom(label="IVdsDiskPartitionMF"),
    "IVdsDiskPartitionMF2": SimTypeBottom(label="IVdsDiskPartitionMF2"),
    "IVdsDrive": SimTypeBottom(label="IVdsDrive"),
    "IVdsDrive2": SimTypeBottom(label="IVdsDrive2"),
    "IVdsHbaPort": SimTypeBottom(label="IVdsHbaPort"),
    "IVdsHwProvider": SimTypeBottom(label="IVdsHwProvider"),
    "IVdsHwProviderPrivate": SimTypeBottom(label="IVdsHwProviderPrivate"),
    "IVdsHwProviderPrivateMpio": SimTypeBottom(label="IVdsHwProviderPrivateMpio"),
    "IVdsHwProviderStoragePools": SimTypeBottom(label="IVdsHwProviderStoragePools"),
    "IVdsHwProviderType": SimTypeBottom(label="IVdsHwProviderType"),
    "IVdsHwProviderType2": SimTypeBottom(label="IVdsHwProviderType2"),
    "IVdsIscsiInitiatorAdapter": SimTypeBottom(label="IVdsIscsiInitiatorAdapter"),
    "IVdsIscsiInitiatorPortal": SimTypeBottom(label="IVdsIscsiInitiatorPortal"),
    "IVdsIscsiPortal": SimTypeBottom(label="IVdsIscsiPortal"),
    "IVdsIscsiPortalGroup": SimTypeBottom(label="IVdsIscsiPortalGroup"),
    "IVdsIscsiPortalLocal": SimTypeBottom(label="IVdsIscsiPortalLocal"),
    "IVdsIscsiTarget": SimTypeBottom(label="IVdsIscsiTarget"),
    "IVdsLun": SimTypeBottom(label="IVdsLun"),
    "IVdsLun2": SimTypeBottom(label="IVdsLun2"),
    "IVdsLunControllerPorts": SimTypeBottom(label="IVdsLunControllerPorts"),
    "IVdsLunIscsi": SimTypeBottom(label="IVdsLunIscsi"),
    "IVdsLunMpio": SimTypeBottom(label="IVdsLunMpio"),
    "IVdsLunNaming": SimTypeBottom(label="IVdsLunNaming"),
    "IVdsLunNumber": SimTypeBottom(label="IVdsLunNumber"),
    "IVdsLunPlex": SimTypeBottom(label="IVdsLunPlex"),
    "IVdsMaintenance": SimTypeBottom(label="IVdsMaintenance"),
    "IVdsOpenVDisk": SimTypeBottom(label="IVdsOpenVDisk"),
    "IVdsPack": SimTypeBottom(label="IVdsPack"),
    "IVdsPack2": SimTypeBottom(label="IVdsPack2"),
    "IVdsProvider": SimTypeBottom(label="IVdsProvider"),
    "IVdsProviderPrivate": SimTypeBottom(label="IVdsProviderPrivate"),
    "IVdsProviderSupport": SimTypeBottom(label="IVdsProviderSupport"),
    "IVdsRemovable": SimTypeBottom(label="IVdsRemovable"),
    "IVdsService": SimTypeBottom(label="IVdsService"),
    "IVdsServiceHba": SimTypeBottom(label="IVdsServiceHba"),
    "IVdsServiceInitialization": SimTypeBottom(label="IVdsServiceInitialization"),
    "IVdsServiceIscsi": SimTypeBottom(label="IVdsServiceIscsi"),
    "IVdsServiceLoader": SimTypeBottom(label="IVdsServiceLoader"),
    "IVdsServiceSAN": SimTypeBottom(label="IVdsServiceSAN"),
    "IVdsServiceSw": SimTypeBottom(label="IVdsServiceSw"),
    "IVdsServiceUninstallDisk": SimTypeBottom(label="IVdsServiceUninstallDisk"),
    "IVdsStoragePool": SimTypeBottom(label="IVdsStoragePool"),
    "IVdsSubSystem": SimTypeBottom(label="IVdsSubSystem"),
    "IVdsSubSystem2": SimTypeBottom(label="IVdsSubSystem2"),
    "IVdsSubSystemImportTarget": SimTypeBottom(label="IVdsSubSystemImportTarget"),
    "IVdsSubSystemInterconnect": SimTypeBottom(label="IVdsSubSystemInterconnect"),
    "IVdsSubSystemIscsi": SimTypeBottom(label="IVdsSubSystemIscsi"),
    "IVdsSubSystemNaming": SimTypeBottom(label="IVdsSubSystemNaming"),
    "IVdsSwProvider": SimTypeBottom(label="IVdsSwProvider"),
    "IVdsVDisk": SimTypeBottom(label="IVdsVDisk"),
    "IVdsVdProvider": SimTypeBottom(label="IVdsVdProvider"),
    "IVdsVolume": SimTypeBottom(label="IVdsVolume"),
    "IVdsVolume2": SimTypeBottom(label="IVdsVolume2"),
    "IVdsVolumeMF": SimTypeBottom(label="IVdsVolumeMF"),
    "IVdsVolumeMF2": SimTypeBottom(label="IVdsVolumeMF2"),
    "IVdsVolumeMF3": SimTypeBottom(label="IVdsVolumeMF3"),
    "IVdsVolumeOnline": SimTypeBottom(label="IVdsVolumeOnline"),
    "IVdsVolumePlex": SimTypeBottom(label="IVdsVolumePlex"),
    "IVdsVolumeShrink": SimTypeBottom(label="IVdsVolumeShrink"),
    "IVersionInfo": SimTypeBottom(label="IVersionInfo"),
    "IVideoEncoder": SimTypeBottom(label="IVideoEncoder"),
    "IVideoFrameStep": SimTypeBottom(label="IVideoFrameStep"),
    "IVideoProcAmp": SimTypeBottom(label="IVideoProcAmp"),
    "IVideoWindow": SimTypeBottom(label="IVideoWindow"),
    "IViewChapter": SimTypeBottom(label="IViewChapter"),
    "IViewExtensionCallback": SimTypeBottom(label="IViewExtensionCallback"),
    "IViewFilter": SimTypeBottom(label="IViewFilter"),
    "IViewHelper": SimTypeBottom(label="IViewHelper"),
    "IViewObject": SimTypeBottom(label="IViewObject"),
    "IViewObject2": SimTypeBottom(label="IViewObject2"),
    "IViewObjectEx": SimTypeBottom(label="IViewObjectEx"),
    "IViewObjectPresentFlip": SimTypeBottom(label="IViewObjectPresentFlip"),
    "IViewObjectPresentFlip2": SimTypeBottom(label="IViewObjectPresentFlip2"),
    "IViewObjectPresentFlipSite": SimTypeBottom(label="IViewObjectPresentFlipSite"),
    "IViewObjectPresentFlipSite2": SimTypeBottom(label="IViewObjectPresentFlipSite2"),
    "IViewObjectPresentNotify": SimTypeBottom(label="IViewObjectPresentNotify"),
    "IViewObjectPresentNotifySite": SimTypeBottom(label="IViewObjectPresentNotifySite"),
    "IViewObjectPresentSite": SimTypeBottom(label="IViewObjectPresentSite"),
    "IViewObjectPrint": SimTypeBottom(label="IViewObjectPrint"),
    "IViewRowset": SimTypeBottom(label="IViewRowset"),
    "IViewSort": SimTypeBottom(label="IViewSort"),
    "IViewStateIdentityItem": SimTypeBottom(label="IViewStateIdentityItem"),
    "IVirtualDesktopManager": SimTypeBottom(label="IVirtualDesktopManager"),
    "IVirtualizedItemProvider": SimTypeBottom(label="IVirtualizedItemProvider"),
    "IVisualProperties": SimTypeBottom(label="IVisualProperties"),
    "IVisualTreeService": SimTypeBottom(label="IVisualTreeService"),
    "IVisualTreeService2": SimTypeBottom(label="IVisualTreeService2"),
    "IVisualTreeService3": SimTypeBottom(label="IVisualTreeService3"),
    "IVisualTreeServiceCallback": SimTypeBottom(label="IVisualTreeServiceCallback"),
    "IVisualTreeServiceCallback2": SimTypeBottom(label="IVisualTreeServiceCallback2"),
    "IVssAdmin": SimTypeBottom(label="IVssAdmin"),
    "IVssAdminEx": SimTypeBottom(label="IVssAdminEx"),
    "IVssAsync": SimTypeBottom(label="IVssAsync"),
    "IVssComponent": SimTypeBottom(label="IVssComponent"),
    "IVssComponentEx": SimTypeBottom(label="IVssComponentEx"),
    "IVssComponentEx2": SimTypeBottom(label="IVssComponentEx2"),
    "IVssCreateExpressWriterMetadata": SimTypeBottom(label="IVssCreateExpressWriterMetadata"),
    "IVssCreateWriterMetadata": SimTypeBottom(label="IVssCreateWriterMetadata"),
    "IVssDifferentialSoftwareSnapshotMgmt": SimTypeBottom(label="IVssDifferentialSoftwareSnapshotMgmt"),
    "IVssDifferentialSoftwareSnapshotMgmt2": SimTypeBottom(label="IVssDifferentialSoftwareSnapshotMgmt2"),
    "IVssDifferentialSoftwareSnapshotMgmt3": SimTypeBottom(label="IVssDifferentialSoftwareSnapshotMgmt3"),
    "IVssEnumMgmtObject": SimTypeBottom(label="IVssEnumMgmtObject"),
    "IVssEnumObject": SimTypeBottom(label="IVssEnumObject"),
    "IVssExpressWriter": SimTypeBottom(label="IVssExpressWriter"),
    "IVssFileShareSnapshotProvider": SimTypeBottom(label="IVssFileShareSnapshotProvider"),
    "IVssHardwareSnapshotProvider": SimTypeBottom(label="IVssHardwareSnapshotProvider"),
    "IVssHardwareSnapshotProviderEx": SimTypeBottom(label="IVssHardwareSnapshotProviderEx"),
    "IVssProviderCreateSnapshotSet": SimTypeBottom(label="IVssProviderCreateSnapshotSet"),
    "IVssProviderNotifications": SimTypeBottom(label="IVssProviderNotifications"),
    "IVssSnapshotMgmt": SimTypeBottom(label="IVssSnapshotMgmt"),
    "IVssSnapshotMgmt2": SimTypeBottom(label="IVssSnapshotMgmt2"),
    "IVssSoftwareSnapshotProvider": SimTypeBottom(label="IVssSoftwareSnapshotProvider"),
    "IVssWMDependency": SimTypeBottom(label="IVssWMDependency"),
    "IVssWMFiledesc": SimTypeBottom(label="IVssWMFiledesc"),
    "IVssWriterComponents": SimTypeBottom(label="IVssWriterComponents"),
    "IWABExtInit": SimTypeBottom(label="IWABExtInit"),
    "IWABObject": SimTypeBottom(label="IWABObject"),
    "IWBScriptControl": SimTypeBottom(label="IWBScriptControl"),
    "IWCContextMenuCallback": SimTypeBottom(label="IWCContextMenuCallback"),
    "IWCNConnectNotify": SimTypeBottom(label="IWCNConnectNotify"),
    "IWCNDevice": SimTypeBottom(label="IWCNDevice"),
    "IWCPropertySheetCallback": SimTypeBottom(label="IWCPropertySheetCallback"),
    "IWCWizard97Callback": SimTypeBottom(label="IWCWizard97Callback"),
    "IWCWizardCallback": SimTypeBottom(label="IWCWizardCallback"),
    "IWEExtendContextMenu": SimTypeBottom(label="IWEExtendContextMenu"),
    "IWEExtendPropertySheet": SimTypeBottom(label="IWEExtendPropertySheet"),
    "IWEExtendWizard": SimTypeBottom(label="IWEExtendWizard"),
    "IWEExtendWizard97": SimTypeBottom(label="IWEExtendWizard97"),
    "IWEInvokeCommand": SimTypeBottom(label="IWEInvokeCommand"),
    "IWICBitmap": SimTypeBottom(label="IWICBitmap"),
    "IWICBitmapClipper": SimTypeBottom(label="IWICBitmapClipper"),
    "IWICBitmapCodecInfo": SimTypeBottom(label="IWICBitmapCodecInfo"),
    "IWICBitmapCodecProgressNotification": SimTypeBottom(label="IWICBitmapCodecProgressNotification"),
    "IWICBitmapDecoder": SimTypeBottom(label="IWICBitmapDecoder"),
    "IWICBitmapDecoderInfo": SimTypeBottom(label="IWICBitmapDecoderInfo"),
    "IWICBitmapEncoder": SimTypeBottom(label="IWICBitmapEncoder"),
    "IWICBitmapEncoderInfo": SimTypeBottom(label="IWICBitmapEncoderInfo"),
    "IWICBitmapFlipRotator": SimTypeBottom(label="IWICBitmapFlipRotator"),
    "IWICBitmapFrameDecode": SimTypeBottom(label="IWICBitmapFrameDecode"),
    "IWICBitmapFrameEncode": SimTypeBottom(label="IWICBitmapFrameEncode"),
    "IWICBitmapLock": SimTypeBottom(label="IWICBitmapLock"),
    "IWICBitmapScaler": SimTypeBottom(label="IWICBitmapScaler"),
    "IWICBitmapSource": SimTypeBottom(label="IWICBitmapSource"),
    "IWICBitmapSourceTransform": SimTypeBottom(label="IWICBitmapSourceTransform"),
    "IWICColorContext": SimTypeBottom(label="IWICColorContext"),
    "IWICColorTransform": SimTypeBottom(label="IWICColorTransform"),
    "IWICComponentFactory": SimTypeBottom(label="IWICComponentFactory"),
    "IWICComponentInfo": SimTypeBottom(label="IWICComponentInfo"),
    "IWICDdsDecoder": SimTypeBottom(label="IWICDdsDecoder"),
    "IWICDdsEncoder": SimTypeBottom(label="IWICDdsEncoder"),
    "IWICDdsFrameDecode": SimTypeBottom(label="IWICDdsFrameDecode"),
    "IWICDevelopRaw": SimTypeBottom(label="IWICDevelopRaw"),
    "IWICDevelopRawNotificationCallback": SimTypeBottom(label="IWICDevelopRawNotificationCallback"),
    "IWICEnumMetadataItem": SimTypeBottom(label="IWICEnumMetadataItem"),
    "IWICFastMetadataEncoder": SimTypeBottom(label="IWICFastMetadataEncoder"),
    "IWICFormatConverter": SimTypeBottom(label="IWICFormatConverter"),
    "IWICFormatConverterInfo": SimTypeBottom(label="IWICFormatConverterInfo"),
    "IWICImageEncoder": SimTypeBottom(label="IWICImageEncoder"),
    "IWICImagingFactory": SimTypeBottom(label="IWICImagingFactory"),
    "IWICImagingFactory2": SimTypeBottom(label="IWICImagingFactory2"),
    "IWICJpegFrameDecode": SimTypeBottom(label="IWICJpegFrameDecode"),
    "IWICJpegFrameEncode": SimTypeBottom(label="IWICJpegFrameEncode"),
    "IWICMetadataBlockReader": SimTypeBottom(label="IWICMetadataBlockReader"),
    "IWICMetadataBlockWriter": SimTypeBottom(label="IWICMetadataBlockWriter"),
    "IWICMetadataHandlerInfo": SimTypeBottom(label="IWICMetadataHandlerInfo"),
    "IWICMetadataQueryReader": SimTypeBottom(label="IWICMetadataQueryReader"),
    "IWICMetadataQueryWriter": SimTypeBottom(label="IWICMetadataQueryWriter"),
    "IWICMetadataReader": SimTypeBottom(label="IWICMetadataReader"),
    "IWICMetadataReaderInfo": SimTypeBottom(label="IWICMetadataReaderInfo"),
    "IWICMetadataWriter": SimTypeBottom(label="IWICMetadataWriter"),
    "IWICMetadataWriterInfo": SimTypeBottom(label="IWICMetadataWriterInfo"),
    "IWICPalette": SimTypeBottom(label="IWICPalette"),
    "IWICPersistStream": SimTypeBottom(label="IWICPersistStream"),
    "IWICPixelFormatInfo": SimTypeBottom(label="IWICPixelFormatInfo"),
    "IWICPixelFormatInfo2": SimTypeBottom(label="IWICPixelFormatInfo2"),
    "IWICPlanarBitmapFrameEncode": SimTypeBottom(label="IWICPlanarBitmapFrameEncode"),
    "IWICPlanarBitmapSourceTransform": SimTypeBottom(label="IWICPlanarBitmapSourceTransform"),
    "IWICPlanarFormatConverter": SimTypeBottom(label="IWICPlanarFormatConverter"),
    "IWICProgressCallback": SimTypeBottom(label="IWICProgressCallback"),
    "IWICProgressiveLevelControl": SimTypeBottom(label="IWICProgressiveLevelControl"),
    "IWICStream": SimTypeBottom(label="IWICStream"),
    "IWICStreamProvider": SimTypeBottom(label="IWICStreamProvider"),
    "IWMAddressAccess": SimTypeBottom(label="IWMAddressAccess"),
    "IWMAddressAccess2": SimTypeBottom(label="IWMAddressAccess2"),
    "IWMAuthorizer": SimTypeBottom(label="IWMAuthorizer"),
    "IWMBackupRestoreProps": SimTypeBottom(label="IWMBackupRestoreProps"),
    "IWMBandwidthSharing": SimTypeBottom(label="IWMBandwidthSharing"),
    "IWMClientConnections": SimTypeBottom(label="IWMClientConnections"),
    "IWMClientConnections2": SimTypeBottom(label="IWMClientConnections2"),
    "IWMCodecAMVideoAccelerator": SimTypeBottom(label="IWMCodecAMVideoAccelerator"),
    "IWMCodecInfo": SimTypeBottom(label="IWMCodecInfo"),
    "IWMCodecInfo2": SimTypeBottom(label="IWMCodecInfo2"),
    "IWMCodecInfo3": SimTypeBottom(label="IWMCodecInfo3"),
    "IWMCodecLeakyBucket": SimTypeBottom(label="IWMCodecLeakyBucket"),
    "IWMCodecOutputTimestamp": SimTypeBottom(label="IWMCodecOutputTimestamp"),
    "IWMCodecPrivateData": SimTypeBottom(label="IWMCodecPrivateData"),
    "IWMCodecProps": SimTypeBottom(label="IWMCodecProps"),
    "IWMCodecStrings": SimTypeBottom(label="IWMCodecStrings"),
    "IWMCodecVideoAccelerator": SimTypeBottom(label="IWMCodecVideoAccelerator"),
    "IWMColorConvProps": SimTypeBottom(label="IWMColorConvProps"),
    "IWMColorLegalizerProps": SimTypeBottom(label="IWMColorLegalizerProps"),
    "IWMCredentialCallback": SimTypeBottom(label="IWMCredentialCallback"),
    "IWMDMDevice": SimTypeBottom(label="IWMDMDevice"),
    "IWMDMDevice2": SimTypeBottom(label="IWMDMDevice2"),
    "IWMDMDevice3": SimTypeBottom(label="IWMDMDevice3"),
    "IWMDMDeviceControl": SimTypeBottom(label="IWMDMDeviceControl"),
    "IWMDMDeviceSession": SimTypeBottom(label="IWMDMDeviceSession"),
    "IWMDMEnumDevice": SimTypeBottom(label="IWMDMEnumDevice"),
    "IWMDMEnumStorage": SimTypeBottom(label="IWMDMEnumStorage"),
    "IWMDMLogger": SimTypeBottom(label="IWMDMLogger"),
    "IWMDMMetaData": SimTypeBottom(label="IWMDMMetaData"),
    "IWMDMNotification": SimTypeBottom(label="IWMDMNotification"),
    "IWMDMObjectInfo": SimTypeBottom(label="IWMDMObjectInfo"),
    "IWMDMOperation": SimTypeBottom(label="IWMDMOperation"),
    "IWMDMOperation2": SimTypeBottom(label="IWMDMOperation2"),
    "IWMDMOperation3": SimTypeBottom(label="IWMDMOperation3"),
    "IWMDMProgress": SimTypeBottom(label="IWMDMProgress"),
    "IWMDMProgress2": SimTypeBottom(label="IWMDMProgress2"),
    "IWMDMProgress3": SimTypeBottom(label="IWMDMProgress3"),
    "IWMDMRevoked": SimTypeBottom(label="IWMDMRevoked"),
    "IWMDMStorage": SimTypeBottom(label="IWMDMStorage"),
    "IWMDMStorage2": SimTypeBottom(label="IWMDMStorage2"),
    "IWMDMStorage3": SimTypeBottom(label="IWMDMStorage3"),
    "IWMDMStorage4": SimTypeBottom(label="IWMDMStorage4"),
    "IWMDMStorageControl": SimTypeBottom(label="IWMDMStorageControl"),
    "IWMDMStorageControl2": SimTypeBottom(label="IWMDMStorageControl2"),
    "IWMDMStorageControl3": SimTypeBottom(label="IWMDMStorageControl3"),
    "IWMDMStorageGlobals": SimTypeBottom(label="IWMDMStorageGlobals"),
    "IWMDRMEditor": SimTypeBottom(label="IWMDRMEditor"),
    "IWMDRMMessageParser": SimTypeBottom(label="IWMDRMMessageParser"),
    "IWMDRMReader": SimTypeBottom(label="IWMDRMReader"),
    "IWMDRMReader2": SimTypeBottom(label="IWMDRMReader2"),
    "IWMDRMReader3": SimTypeBottom(label="IWMDRMReader3"),
    "IWMDRMTranscryptionManager": SimTypeBottom(label="IWMDRMTranscryptionManager"),
    "IWMDRMTranscryptor": SimTypeBottom(label="IWMDRMTranscryptor"),
    "IWMDRMTranscryptor2": SimTypeBottom(label="IWMDRMTranscryptor2"),
    "IWMDRMWriter": SimTypeBottom(label="IWMDRMWriter"),
    "IWMDRMWriter2": SimTypeBottom(label="IWMDRMWriter2"),
    "IWMDRMWriter3": SimTypeBottom(label="IWMDRMWriter3"),
    "IWMDeviceManager": SimTypeBottom(label="IWMDeviceManager"),
    "IWMDeviceManager2": SimTypeBottom(label="IWMDeviceManager2"),
    "IWMDeviceManager3": SimTypeBottom(label="IWMDeviceManager3"),
    "IWMDeviceRegistration": SimTypeBottom(label="IWMDeviceRegistration"),
    "IWMFrameInterpProps": SimTypeBottom(label="IWMFrameInterpProps"),
    "IWMGetSecureChannel": SimTypeBottom(label="IWMGetSecureChannel"),
    "IWMHeaderInfo": SimTypeBottom(label="IWMHeaderInfo"),
    "IWMHeaderInfo2": SimTypeBottom(label="IWMHeaderInfo2"),
    "IWMHeaderInfo3": SimTypeBottom(label="IWMHeaderInfo3"),
    "IWMIExtension": SimTypeBottom(label="IWMIExtension"),
    "IWMIStreamProps": SimTypeBottom(label="IWMIStreamProps"),
    "IWMImageInfo": SimTypeBottom(label="IWMImageInfo"),
    "IWMIndexer": SimTypeBottom(label="IWMIndexer"),
    "IWMIndexer2": SimTypeBottom(label="IWMIndexer2"),
    "IWMInputMediaProps": SimTypeBottom(label="IWMInputMediaProps"),
    "IWMInterlaceProps": SimTypeBottom(label="IWMInterlaceProps"),
    "IWMLanguageList": SimTypeBottom(label="IWMLanguageList"),
    "IWMLicenseBackup": SimTypeBottom(label="IWMLicenseBackup"),
    "IWMLicenseRestore": SimTypeBottom(label="IWMLicenseRestore"),
    "IWMLicenseRevocationAgent": SimTypeBottom(label="IWMLicenseRevocationAgent"),
    "IWMMediaProps": SimTypeBottom(label="IWMMediaProps"),
    "IWMMetadataEditor": SimTypeBottom(label="IWMMetadataEditor"),
    "IWMMetadataEditor2": SimTypeBottom(label="IWMMetadataEditor2"),
    "IWMMutualExclusion": SimTypeBottom(label="IWMMutualExclusion"),
    "IWMMutualExclusion2": SimTypeBottom(label="IWMMutualExclusion2"),
    "IWMOutputMediaProps": SimTypeBottom(label="IWMOutputMediaProps"),
    "IWMPAudioRenderConfig": SimTypeBottom(label="IWMPAudioRenderConfig"),
    "IWMPCdrom": SimTypeBottom(label="IWMPCdrom"),
    "IWMPCdromBurn": SimTypeBottom(label="IWMPCdromBurn"),
    "IWMPCdromCollection": SimTypeBottom(label="IWMPCdromCollection"),
    "IWMPCdromRip": SimTypeBottom(label="IWMPCdromRip"),
    "IWMPClosedCaption": SimTypeBottom(label="IWMPClosedCaption"),
    "IWMPClosedCaption2": SimTypeBottom(label="IWMPClosedCaption2"),
    "IWMPContentContainer": SimTypeBottom(label="IWMPContentContainer"),
    "IWMPContentContainerList": SimTypeBottom(label="IWMPContentContainerList"),
    "IWMPContentPartner": SimTypeBottom(label="IWMPContentPartner"),
    "IWMPContentPartnerCallback": SimTypeBottom(label="IWMPContentPartnerCallback"),
    "IWMPControls": SimTypeBottom(label="IWMPControls"),
    "IWMPControls2": SimTypeBottom(label="IWMPControls2"),
    "IWMPControls3": SimTypeBottom(label="IWMPControls3"),
    "IWMPConvert": SimTypeBottom(label="IWMPConvert"),
    "IWMPCore": SimTypeBottom(label="IWMPCore"),
    "IWMPCore2": SimTypeBottom(label="IWMPCore2"),
    "IWMPCore3": SimTypeBottom(label="IWMPCore3"),
    "IWMPDVD": SimTypeBottom(label="IWMPDVD"),
    "IWMPDownloadCollection": SimTypeBottom(label="IWMPDownloadCollection"),
    "IWMPDownloadItem": SimTypeBottom(label="IWMPDownloadItem"),
    "IWMPDownloadItem2": SimTypeBottom(label="IWMPDownloadItem2"),
    "IWMPDownloadManager": SimTypeBottom(label="IWMPDownloadManager"),
    "IWMPEffects": SimTypeBottom(label="IWMPEffects"),
    "IWMPEffects2": SimTypeBottom(label="IWMPEffects2"),
    "IWMPError": SimTypeBottom(label="IWMPError"),
    "IWMPErrorItem": SimTypeBottom(label="IWMPErrorItem"),
    "IWMPErrorItem2": SimTypeBottom(label="IWMPErrorItem2"),
    "IWMPEvents": SimTypeBottom(label="IWMPEvents"),
    "IWMPEvents2": SimTypeBottom(label="IWMPEvents2"),
    "IWMPEvents3": SimTypeBottom(label="IWMPEvents3"),
    "IWMPEvents4": SimTypeBottom(label="IWMPEvents4"),
    "IWMPFolderMonitorServices": SimTypeBottom(label="IWMPFolderMonitorServices"),
    "IWMPGraphCreation": SimTypeBottom(label="IWMPGraphCreation"),
    "IWMPLibrary": SimTypeBottom(label="IWMPLibrary"),
    "IWMPLibrary2": SimTypeBottom(label="IWMPLibrary2"),
    "IWMPLibraryServices": SimTypeBottom(label="IWMPLibraryServices"),
    "IWMPLibrarySharingServices": SimTypeBottom(label="IWMPLibrarySharingServices"),
    "IWMPMedia": SimTypeBottom(label="IWMPMedia"),
    "IWMPMedia2": SimTypeBottom(label="IWMPMedia2"),
    "IWMPMedia3": SimTypeBottom(label="IWMPMedia3"),
    "IWMPMediaCollection": SimTypeBottom(label="IWMPMediaCollection"),
    "IWMPMediaCollection2": SimTypeBottom(label="IWMPMediaCollection2"),
    "IWMPMediaPluginRegistrar": SimTypeBottom(label="IWMPMediaPluginRegistrar"),
    "IWMPMetadataPicture": SimTypeBottom(label="IWMPMetadataPicture"),
    "IWMPMetadataText": SimTypeBottom(label="IWMPMetadataText"),
    "IWMPNetwork": SimTypeBottom(label="IWMPNetwork"),
    "IWMPNodeRealEstate": SimTypeBottom(label="IWMPNodeRealEstate"),
    "IWMPNodeRealEstateHost": SimTypeBottom(label="IWMPNodeRealEstateHost"),
    "IWMPNodeWindowed": SimTypeBottom(label="IWMPNodeWindowed"),
    "IWMPNodeWindowedHost": SimTypeBottom(label="IWMPNodeWindowedHost"),
    "IWMPNodeWindowless": SimTypeBottom(label="IWMPNodeWindowless"),
    "IWMPNodeWindowlessHost": SimTypeBottom(label="IWMPNodeWindowlessHost"),
    "IWMPPlayer": SimTypeBottom(label="IWMPPlayer"),
    "IWMPPlayer2": SimTypeBottom(label="IWMPPlayer2"),
    "IWMPPlayer3": SimTypeBottom(label="IWMPPlayer3"),
    "IWMPPlayer4": SimTypeBottom(label="IWMPPlayer4"),
    "IWMPPlayerApplication": SimTypeBottom(label="IWMPPlayerApplication"),
    "IWMPPlayerServices": SimTypeBottom(label="IWMPPlayerServices"),
    "IWMPPlayerServices2": SimTypeBottom(label="IWMPPlayerServices2"),
    "IWMPPlaylist": SimTypeBottom(label="IWMPPlaylist"),
    "IWMPPlaylistArray": SimTypeBottom(label="IWMPPlaylistArray"),
    "IWMPPlaylistCollection": SimTypeBottom(label="IWMPPlaylistCollection"),
    "IWMPPlugin": SimTypeBottom(label="IWMPPlugin"),
    "IWMPPluginEnable": SimTypeBottom(label="IWMPPluginEnable"),
    "IWMPPluginUI": SimTypeBottom(label="IWMPPluginUI"),
    "IWMPQuery": SimTypeBottom(label="IWMPQuery"),
    "IWMPRemoteMediaServices": SimTypeBottom(label="IWMPRemoteMediaServices"),
    "IWMPRenderConfig": SimTypeBottom(label="IWMPRenderConfig"),
    "IWMPServices": SimTypeBottom(label="IWMPServices"),
    "IWMPSettings": SimTypeBottom(label="IWMPSettings"),
    "IWMPSettings2": SimTypeBottom(label="IWMPSettings2"),
    "IWMPSkinManager": SimTypeBottom(label="IWMPSkinManager"),
    "IWMPStringCollection": SimTypeBottom(label="IWMPStringCollection"),
    "IWMPStringCollection2": SimTypeBottom(label="IWMPStringCollection2"),
    "IWMPSubscriptionService": SimTypeBottom(label="IWMPSubscriptionService"),
    "IWMPSubscriptionService2": SimTypeBottom(label="IWMPSubscriptionService2"),
    "IWMPSubscriptionServiceCallback": SimTypeBottom(label="IWMPSubscriptionServiceCallback"),
    "IWMPSyncDevice": SimTypeBottom(label="IWMPSyncDevice"),
    "IWMPSyncDevice2": SimTypeBottom(label="IWMPSyncDevice2"),
    "IWMPSyncDevice3": SimTypeBottom(label="IWMPSyncDevice3"),
    "IWMPSyncServices": SimTypeBottom(label="IWMPSyncServices"),
    "IWMPTranscodePolicy": SimTypeBottom(label="IWMPTranscodePolicy"),
    "IWMPUserEventSink": SimTypeBottom(label="IWMPUserEventSink"),
    "IWMPVideoRenderConfig": SimTypeBottom(label="IWMPVideoRenderConfig"),
    "IWMPWindowMessageSink": SimTypeBottom(label="IWMPWindowMessageSink"),
    "IWMPacketSize": SimTypeBottom(label="IWMPacketSize"),
    "IWMPacketSize2": SimTypeBottom(label="IWMPacketSize2"),
    "IWMPlayerHook": SimTypeBottom(label="IWMPlayerHook"),
    "IWMPlayerTimestampHook": SimTypeBottom(label="IWMPlayerTimestampHook"),
    "IWMProfile": SimTypeBottom(label="IWMProfile"),
    "IWMProfile2": SimTypeBottom(label="IWMProfile2"),
    "IWMProfile3": SimTypeBottom(label="IWMProfile3"),
    "IWMProfileManager": SimTypeBottom(label="IWMProfileManager"),
    "IWMProfileManager2": SimTypeBottom(label="IWMProfileManager2"),
    "IWMProfileManagerLanguage": SimTypeBottom(label="IWMProfileManagerLanguage"),
    "IWMPropertyVault": SimTypeBottom(label="IWMPropertyVault"),
    "IWMProximityDetection": SimTypeBottom(label="IWMProximityDetection"),
    "IWMReader": SimTypeBottom(label="IWMReader"),
    "IWMReaderAccelerator": SimTypeBottom(label="IWMReaderAccelerator"),
    "IWMReaderAdvanced": SimTypeBottom(label="IWMReaderAdvanced"),
    "IWMReaderAdvanced2": SimTypeBottom(label="IWMReaderAdvanced2"),
    "IWMReaderAdvanced3": SimTypeBottom(label="IWMReaderAdvanced3"),
    "IWMReaderAdvanced4": SimTypeBottom(label="IWMReaderAdvanced4"),
    "IWMReaderAdvanced5": SimTypeBottom(label="IWMReaderAdvanced5"),
    "IWMReaderAdvanced6": SimTypeBottom(label="IWMReaderAdvanced6"),
    "IWMReaderAllocatorEx": SimTypeBottom(label="IWMReaderAllocatorEx"),
    "IWMReaderCallback": SimTypeBottom(label="IWMReaderCallback"),
    "IWMReaderCallbackAdvanced": SimTypeBottom(label="IWMReaderCallbackAdvanced"),
    "IWMReaderNetworkConfig": SimTypeBottom(label="IWMReaderNetworkConfig"),
    "IWMReaderNetworkConfig2": SimTypeBottom(label="IWMReaderNetworkConfig2"),
    "IWMReaderPlaylistBurn": SimTypeBottom(label="IWMReaderPlaylistBurn"),
    "IWMReaderStreamClock": SimTypeBottom(label="IWMReaderStreamClock"),
    "IWMReaderTimecode": SimTypeBottom(label="IWMReaderTimecode"),
    "IWMReaderTypeNegotiation": SimTypeBottom(label="IWMReaderTypeNegotiation"),
    "IWMRegisterCallback": SimTypeBottom(label="IWMRegisterCallback"),
    "IWMRegisteredDevice": SimTypeBottom(label="IWMRegisteredDevice"),
    "IWMResamplerProps": SimTypeBottom(label="IWMResamplerProps"),
    "IWMResizerProps": SimTypeBottom(label="IWMResizerProps"),
    "IWMSBufferAllocator": SimTypeBottom(label="IWMSBufferAllocator"),
    "IWMSInternalAdminNetSource": SimTypeBottom(label="IWMSInternalAdminNetSource"),
    "IWMSInternalAdminNetSource2": SimTypeBottom(label="IWMSInternalAdminNetSource2"),
    "IWMSInternalAdminNetSource3": SimTypeBottom(label="IWMSInternalAdminNetSource3"),
    "IWMSampleExtensionSupport": SimTypeBottom(label="IWMSampleExtensionSupport"),
    "IWMSecureChannel": SimTypeBottom(label="IWMSecureChannel"),
    "IWMStatusCallback": SimTypeBottom(label="IWMStatusCallback"),
    "IWMStreamConfig": SimTypeBottom(label="IWMStreamConfig"),
    "IWMStreamConfig2": SimTypeBottom(label="IWMStreamConfig2"),
    "IWMStreamConfig3": SimTypeBottom(label="IWMStreamConfig3"),
    "IWMStreamList": SimTypeBottom(label="IWMStreamList"),
    "IWMStreamPrioritization": SimTypeBottom(label="IWMStreamPrioritization"),
    "IWMSyncReader": SimTypeBottom(label="IWMSyncReader"),
    "IWMSyncReader2": SimTypeBottom(label="IWMSyncReader2"),
    "IWMValidate": SimTypeBottom(label="IWMValidate"),
    "IWMVideoDecoderHurryup": SimTypeBottom(label="IWMVideoDecoderHurryup"),
    "IWMVideoDecoderReconBuffer": SimTypeBottom(label="IWMVideoDecoderReconBuffer"),
    "IWMVideoForceKeyFrame": SimTypeBottom(label="IWMVideoForceKeyFrame"),
    "IWMVideoMediaProps": SimTypeBottom(label="IWMVideoMediaProps"),
    "IWMWatermarkInfo": SimTypeBottom(label="IWMWatermarkInfo"),
    "IWMWriter": SimTypeBottom(label="IWMWriter"),
    "IWMWriterAdvanced": SimTypeBottom(label="IWMWriterAdvanced"),
    "IWMWriterAdvanced2": SimTypeBottom(label="IWMWriterAdvanced2"),
    "IWMWriterAdvanced3": SimTypeBottom(label="IWMWriterAdvanced3"),
    "IWMWriterFileSink": SimTypeBottom(label="IWMWriterFileSink"),
    "IWMWriterFileSink2": SimTypeBottom(label="IWMWriterFileSink2"),
    "IWMWriterFileSink3": SimTypeBottom(label="IWMWriterFileSink3"),
    "IWMWriterNetworkSink": SimTypeBottom(label="IWMWriterNetworkSink"),
    "IWMWriterPostView": SimTypeBottom(label="IWMWriterPostView"),
    "IWMWriterPostViewCallback": SimTypeBottom(label="IWMWriterPostViewCallback"),
    "IWMWriterPreprocess": SimTypeBottom(label="IWMWriterPreprocess"),
    "IWMWriterPushSink": SimTypeBottom(label="IWMWriterPushSink"),
    "IWMWriterSink": SimTypeBottom(label="IWMWriterSink"),
    "IWPCBlockedUrls": SimTypeBottom(label="IWPCBlockedUrls"),
    "IWPCGamesSettings": SimTypeBottom(label="IWPCGamesSettings"),
    "IWPCProviderConfig": SimTypeBottom(label="IWPCProviderConfig"),
    "IWPCProviderState": SimTypeBottom(label="IWPCProviderState"),
    "IWPCProviderSupport": SimTypeBottom(label="IWPCProviderSupport"),
    "IWPCSettings": SimTypeBottom(label="IWPCSettings"),
    "IWPCWebSettings": SimTypeBottom(label="IWPCWebSettings"),
    "IWRdsEnhancedFastReconnectArbitrator": SimTypeBottom(label="IWRdsEnhancedFastReconnectArbitrator"),
    "IWRdsGraphicsChannel": SimTypeBottom(label="IWRdsGraphicsChannel"),
    "IWRdsGraphicsChannelEvents": SimTypeBottom(label="IWRdsGraphicsChannelEvents"),
    "IWRdsGraphicsChannelManager": SimTypeBottom(label="IWRdsGraphicsChannelManager"),
    "IWRdsProtocolConnection": SimTypeBottom(label="IWRdsProtocolConnection"),
    "IWRdsProtocolConnectionCallback": SimTypeBottom(label="IWRdsProtocolConnectionCallback"),
    "IWRdsProtocolConnectionSettings": SimTypeBottom(label="IWRdsProtocolConnectionSettings"),
    "IWRdsProtocolLicenseConnection": SimTypeBottom(label="IWRdsProtocolLicenseConnection"),
    "IWRdsProtocolListener": SimTypeBottom(label="IWRdsProtocolListener"),
    "IWRdsProtocolListenerCallback": SimTypeBottom(label="IWRdsProtocolListenerCallback"),
    "IWRdsProtocolLogonErrorRedirector": SimTypeBottom(label="IWRdsProtocolLogonErrorRedirector"),
    "IWRdsProtocolManager": SimTypeBottom(label="IWRdsProtocolManager"),
    "IWRdsProtocolSettings": SimTypeBottom(label="IWRdsProtocolSettings"),
    "IWRdsProtocolShadowCallback": SimTypeBottom(label="IWRdsProtocolShadowCallback"),
    "IWRdsProtocolShadowConnection": SimTypeBottom(label="IWRdsProtocolShadowConnection"),
    "IWRdsWddmIddProps": SimTypeBottom(label="IWRdsWddmIddProps"),
    "IWRdsWddmIddProps1": SimTypeBottom(label="IWRdsWddmIddProps1"),
    "IWSCDefaultProduct": SimTypeBottom(label="IWSCDefaultProduct"),
    "IWSCProductList": SimTypeBottom(label="IWSCProductList"),
    "IWSDAddress": SimTypeBottom(label="IWSDAddress"),
    "IWSDAsyncCallback": SimTypeBottom(label="IWSDAsyncCallback"),
    "IWSDAsyncResult": SimTypeBottom(label="IWSDAsyncResult"),
    "IWSDAttachment": SimTypeBottom(label="IWSDAttachment"),
    "IWSDDeviceHost": SimTypeBottom(label="IWSDDeviceHost"),
    "IWSDDeviceHostNotify": SimTypeBottom(label="IWSDDeviceHostNotify"),
    "IWSDDeviceProxy": SimTypeBottom(label="IWSDDeviceProxy"),
    "IWSDEndpointProxy": SimTypeBottom(label="IWSDEndpointProxy"),
    "IWSDEventingStatus": SimTypeBottom(label="IWSDEventingStatus"),
    "IWSDHttpAddress": SimTypeBottom(label="IWSDHttpAddress"),
    "IWSDHttpAuthParameters": SimTypeBottom(label="IWSDHttpAuthParameters"),
    "IWSDHttpMessageParameters": SimTypeBottom(label="IWSDHttpMessageParameters"),
    "IWSDInboundAttachment": SimTypeBottom(label="IWSDInboundAttachment"),
    "IWSDMessageParameters": SimTypeBottom(label="IWSDMessageParameters"),
    "IWSDMetadataExchange": SimTypeBottom(label="IWSDMetadataExchange"),
    "IWSDOutboundAttachment": SimTypeBottom(label="IWSDOutboundAttachment"),
    "IWSDSSLClientCertificate": SimTypeBottom(label="IWSDSSLClientCertificate"),
    "IWSDScopeMatchingRule": SimTypeBottom(label="IWSDScopeMatchingRule"),
    "IWSDServiceMessaging": SimTypeBottom(label="IWSDServiceMessaging"),
    "IWSDServiceProxy": SimTypeBottom(label="IWSDServiceProxy"),
    "IWSDServiceProxyEventing": SimTypeBottom(label="IWSDServiceProxyEventing"),
    "IWSDSignatureProperty": SimTypeBottom(label="IWSDSignatureProperty"),
    "IWSDTransportAddress": SimTypeBottom(label="IWSDTransportAddress"),
    "IWSDUdpAddress": SimTypeBottom(label="IWSDUdpAddress"),
    "IWSDUdpMessageParameters": SimTypeBottom(label="IWSDUdpMessageParameters"),
    "IWSDXMLContext": SimTypeBottom(label="IWSDXMLContext"),
    "IWSDiscoveredService": SimTypeBottom(label="IWSDiscoveredService"),
    "IWSDiscoveryProvider": SimTypeBottom(label="IWSDiscoveryProvider"),
    "IWSDiscoveryProviderNotify": SimTypeBottom(label="IWSDiscoveryProviderNotify"),
    "IWSDiscoveryPublisher": SimTypeBottom(label="IWSDiscoveryPublisher"),
    "IWSDiscoveryPublisherNotify": SimTypeBottom(label="IWSDiscoveryPublisherNotify"),
    "IWSMan": SimTypeBottom(label="IWSMan"),
    "IWSManConnectionOptions": SimTypeBottom(label="IWSManConnectionOptions"),
    "IWSManConnectionOptionsEx": SimTypeBottom(label="IWSManConnectionOptionsEx"),
    "IWSManConnectionOptionsEx2": SimTypeBottom(label="IWSManConnectionOptionsEx2"),
    "IWSManEnumerator": SimTypeBottom(label="IWSManEnumerator"),
    "IWSManEx": SimTypeBottom(label="IWSManEx"),
    "IWSManEx2": SimTypeBottom(label="IWSManEx2"),
    "IWSManEx3": SimTypeBottom(label="IWSManEx3"),
    "IWSManInternal": SimTypeBottom(label="IWSManInternal"),
    "IWSManResourceLocator": SimTypeBottom(label="IWSManResourceLocator"),
    "IWSManResourceLocatorInternal": SimTypeBottom(label="IWSManResourceLocatorInternal"),
    "IWSManSession": SimTypeBottom(label="IWSManSession"),
    "IWTSBitmapRenderService": SimTypeBottom(label="IWTSBitmapRenderService"),
    "IWTSBitmapRenderer": SimTypeBottom(label="IWTSBitmapRenderer"),
    "IWTSBitmapRendererCallback": SimTypeBottom(label="IWTSBitmapRendererCallback"),
    "IWTSListener": SimTypeBottom(label="IWTSListener"),
    "IWTSListenerCallback": SimTypeBottom(label="IWTSListenerCallback"),
    "IWTSPlugin": SimTypeBottom(label="IWTSPlugin"),
    "IWTSPluginServiceProvider": SimTypeBottom(label="IWTSPluginServiceProvider"),
    "IWTSProtocolConnection": SimTypeBottom(label="IWTSProtocolConnection"),
    "IWTSProtocolConnectionCallback": SimTypeBottom(label="IWTSProtocolConnectionCallback"),
    "IWTSProtocolLicenseConnection": SimTypeBottom(label="IWTSProtocolLicenseConnection"),
    "IWTSProtocolListener": SimTypeBottom(label="IWTSProtocolListener"),
    "IWTSProtocolListenerCallback": SimTypeBottom(label="IWTSProtocolListenerCallback"),
    "IWTSProtocolLogonErrorRedirector": SimTypeBottom(label="IWTSProtocolLogonErrorRedirector"),
    "IWTSProtocolManager": SimTypeBottom(label="IWTSProtocolManager"),
    "IWTSProtocolShadowCallback": SimTypeBottom(label="IWTSProtocolShadowCallback"),
    "IWTSProtocolShadowConnection": SimTypeBottom(label="IWTSProtocolShadowConnection"),
    "IWTSSBPlugin": SimTypeBottom(label="IWTSSBPlugin"),
    "IWTSVirtualChannel": SimTypeBottom(label="IWTSVirtualChannel"),
    "IWTSVirtualChannelCallback": SimTypeBottom(label="IWTSVirtualChannelCallback"),
    "IWTSVirtualChannelManager": SimTypeBottom(label="IWTSVirtualChannelManager"),
    "IWaaSAssessor": SimTypeBottom(label="IWaaSAssessor"),
    "IWaitMultiple": SimTypeBottom(label="IWaitMultiple"),
    "IWbemAddressResolution": SimTypeBottom(label="IWbemAddressResolution"),
    "IWbemBackupRestore": SimTypeBottom(label="IWbemBackupRestore"),
    "IWbemBackupRestoreEx": SimTypeBottom(label="IWbemBackupRestoreEx"),
    "IWbemCallResult": SimTypeBottom(label="IWbemCallResult"),
    "IWbemClassObject": SimTypeBottom(label="IWbemClassObject"),
    "IWbemClientConnectionTransport": SimTypeBottom(label="IWbemClientConnectionTransport"),
    "IWbemClientTransport": SimTypeBottom(label="IWbemClientTransport"),
    "IWbemConfigureRefresher": SimTypeBottom(label="IWbemConfigureRefresher"),
    "IWbemConnectorLogin": SimTypeBottom(label="IWbemConnectorLogin"),
    "IWbemConstructClassObject": SimTypeBottom(label="IWbemConstructClassObject"),
    "IWbemContext": SimTypeBottom(label="IWbemContext"),
    "IWbemDecoupledBasicEventProvider": SimTypeBottom(label="IWbemDecoupledBasicEventProvider"),
    "IWbemDecoupledRegistrar": SimTypeBottom(label="IWbemDecoupledRegistrar"),
    "IWbemEventConsumerProvider": SimTypeBottom(label="IWbemEventConsumerProvider"),
    "IWbemEventProvider": SimTypeBottom(label="IWbemEventProvider"),
    "IWbemEventProviderQuerySink": SimTypeBottom(label="IWbemEventProviderQuerySink"),
    "IWbemEventProviderSecurity": SimTypeBottom(label="IWbemEventProviderSecurity"),
    "IWbemEventSink": SimTypeBottom(label="IWbemEventSink"),
    "IWbemHiPerfEnum": SimTypeBottom(label="IWbemHiPerfEnum"),
    "IWbemHiPerfProvider": SimTypeBottom(label="IWbemHiPerfProvider"),
    "IWbemLevel1Login": SimTypeBottom(label="IWbemLevel1Login"),
    "IWbemLocator": SimTypeBottom(label="IWbemLocator"),
    "IWbemObjectAccess": SimTypeBottom(label="IWbemObjectAccess"),
    "IWbemObjectSink": SimTypeBottom(label="IWbemObjectSink"),
    "IWbemObjectSinkEx": SimTypeBottom(label="IWbemObjectSinkEx"),
    "IWbemObjectTextSrc": SimTypeBottom(label="IWbemObjectTextSrc"),
    "IWbemPath": SimTypeBottom(label="IWbemPath"),
    "IWbemPathKeyList": SimTypeBottom(label="IWbemPathKeyList"),
    "IWbemPropertyProvider": SimTypeBottom(label="IWbemPropertyProvider"),
    "IWbemProviderIdentity": SimTypeBottom(label="IWbemProviderIdentity"),
    "IWbemProviderInit": SimTypeBottom(label="IWbemProviderInit"),
    "IWbemProviderInitSink": SimTypeBottom(label="IWbemProviderInitSink"),
    "IWbemQualifierSet": SimTypeBottom(label="IWbemQualifierSet"),
    "IWbemQuery": SimTypeBottom(label="IWbemQuery"),
    "IWbemRefresher": SimTypeBottom(label="IWbemRefresher"),
    "IWbemServices": SimTypeBottom(label="IWbemServices"),
    "IWbemShutdown": SimTypeBottom(label="IWbemShutdown"),
    "IWbemStatusCodeText": SimTypeBottom(label="IWbemStatusCodeText"),
    "IWbemTransport": SimTypeBottom(label="IWbemTransport"),
    "IWbemUnboundObjectSink": SimTypeBottom(label="IWbemUnboundObjectSink"),
    "IWbemUnsecuredApartment": SimTypeBottom(label="IWbemUnsecuredApartment"),
    "IWdsTransportCacheable": SimTypeBottom(label="IWdsTransportCacheable"),
    "IWdsTransportClient": SimTypeBottom(label="IWdsTransportClient"),
    "IWdsTransportCollection": SimTypeBottom(label="IWdsTransportCollection"),
    "IWdsTransportConfigurationManager": SimTypeBottom(label="IWdsTransportConfigurationManager"),
    "IWdsTransportConfigurationManager2": SimTypeBottom(label="IWdsTransportConfigurationManager2"),
    "IWdsTransportContent": SimTypeBottom(label="IWdsTransportContent"),
    "IWdsTransportContentProvider": SimTypeBottom(label="IWdsTransportContentProvider"),
    "IWdsTransportDiagnosticsPolicy": SimTypeBottom(label="IWdsTransportDiagnosticsPolicy"),
    "IWdsTransportManager": SimTypeBottom(label="IWdsTransportManager"),
    "IWdsTransportMulticastSessionPolicy": SimTypeBottom(label="IWdsTransportMulticastSessionPolicy"),
    "IWdsTransportNamespace": SimTypeBottom(label="IWdsTransportNamespace"),
    "IWdsTransportNamespaceAutoCast": SimTypeBottom(label="IWdsTransportNamespaceAutoCast"),
    "IWdsTransportNamespaceManager": SimTypeBottom(label="IWdsTransportNamespaceManager"),
    "IWdsTransportNamespaceScheduledCast": SimTypeBottom(label="IWdsTransportNamespaceScheduledCast"),
    "IWdsTransportNamespaceScheduledCastAutoStart": SimTypeBottom(label="IWdsTransportNamespaceScheduledCastAutoStart"),
    "IWdsTransportNamespaceScheduledCastManualStart": SimTypeBottom(label="IWdsTransportNamespaceScheduledCastManualStart"),
    "IWdsTransportServer": SimTypeBottom(label="IWdsTransportServer"),
    "IWdsTransportServer2": SimTypeBottom(label="IWdsTransportServer2"),
    "IWdsTransportServicePolicy": SimTypeBottom(label="IWdsTransportServicePolicy"),
    "IWdsTransportServicePolicy2": SimTypeBottom(label="IWdsTransportServicePolicy2"),
    "IWdsTransportSession": SimTypeBottom(label="IWdsTransportSession"),
    "IWdsTransportSetupManager": SimTypeBottom(label="IWdsTransportSetupManager"),
    "IWdsTransportSetupManager2": SimTypeBottom(label="IWdsTransportSetupManager2"),
    "IWdsTransportTftpClient": SimTypeBottom(label="IWdsTransportTftpClient"),
    "IWdsTransportTftpManager": SimTypeBottom(label="IWdsTransportTftpManager"),
    "IWebAppDiagnosticsObjectInitialization": SimTypeBottom(label="IWebAppDiagnosticsObjectInitialization"),
    "IWebAppDiagnosticsSetup": SimTypeBottom(label="IWebAppDiagnosticsSetup"),
    "IWebApplicationActivation": SimTypeBottom(label="IWebApplicationActivation"),
    "IWebApplicationAuthoringMode": SimTypeBottom(label="IWebApplicationAuthoringMode"),
    "IWebApplicationHost": SimTypeBottom(label="IWebApplicationHost"),
    "IWebApplicationNavigationEvents": SimTypeBottom(label="IWebApplicationNavigationEvents"),
    "IWebApplicationScriptEvents": SimTypeBottom(label="IWebApplicationScriptEvents"),
    "IWebApplicationUIEvents": SimTypeBottom(label="IWebApplicationUIEvents"),
    "IWebApplicationUpdateEvents": SimTypeBottom(label="IWebApplicationUpdateEvents"),
    "IWebBridge": SimTypeBottom(label="IWebBridge"),
    "IWebBrowser": SimTypeBottom(label="IWebBrowser"),
    "IWebBrowser2": SimTypeBottom(label="IWebBrowser2"),
    "IWebBrowserApp": SimTypeBottom(label="IWebBrowserApp"),
    "IWebBrowserEventsService": SimTypeBottom(label="IWebBrowserEventsService"),
    "IWebBrowserEventsUrlService": SimTypeBottom(label="IWebBrowserEventsUrlService"),
    "IWebGeocoordinates": SimTypeBottom(label="IWebGeocoordinates"),
    "IWebGeolocation": SimTypeBottom(label="IWebGeolocation"),
    "IWebGeoposition": SimTypeBottom(label="IWebGeoposition"),
    "IWebGeopositionError": SimTypeBottom(label="IWebGeopositionError"),
    "IWebProxy": SimTypeBottom(label="IWebProxy"),
    "IWebWizardExtension": SimTypeBottom(label="IWebWizardExtension"),
    "IWebWizardHost": SimTypeBottom(label="IWebWizardHost"),
    "IWebWizardHost2": SimTypeBottom(label="IWebWizardHost2"),
    "IWeeklyTrigger": SimTypeBottom(label="IWeeklyTrigger"),
    "IWiaAppErrorHandler": SimTypeBottom(label="IWiaAppErrorHandler"),
    "IWiaDataCallback": SimTypeBottom(label="IWiaDataCallback"),
    "IWiaDataTransfer": SimTypeBottom(label="IWiaDataTransfer"),
    "IWiaDevMgr": SimTypeBottom(label="IWiaDevMgr"),
    "IWiaDevMgr2": SimTypeBottom(label="IWiaDevMgr2"),
    "IWiaDrvItem": SimTypeBottom(label="IWiaDrvItem"),
    "IWiaErrorHandler": SimTypeBottom(label="IWiaErrorHandler"),
    "IWiaEventCallback": SimTypeBottom(label="IWiaEventCallback"),
    "IWiaImageFilter": SimTypeBottom(label="IWiaImageFilter"),
    "IWiaItem": SimTypeBottom(label="IWiaItem"),
    "IWiaItem2": SimTypeBottom(label="IWiaItem2"),
    "IWiaItemExtras": SimTypeBottom(label="IWiaItemExtras"),
    "IWiaLog": SimTypeBottom(label="IWiaLog"),
    "IWiaLogEx": SimTypeBottom(label="IWiaLogEx"),
    "IWiaMiniDrv": SimTypeBottom(label="IWiaMiniDrv"),
    "IWiaMiniDrvCallBack": SimTypeBottom(label="IWiaMiniDrvCallBack"),
    "IWiaMiniDrvTransferCallback": SimTypeBottom(label="IWiaMiniDrvTransferCallback"),
    "IWiaNotifyDevMgr": SimTypeBottom(label="IWiaNotifyDevMgr"),
    "IWiaPreview": SimTypeBottom(label="IWiaPreview"),
    "IWiaPropertyStorage": SimTypeBottom(label="IWiaPropertyStorage"),
    "IWiaSegmentationFilter": SimTypeBottom(label="IWiaSegmentationFilter"),
    "IWiaTransfer": SimTypeBottom(label="IWiaTransfer"),
    "IWiaTransferCallback": SimTypeBottom(label="IWiaTransferCallback"),
    "IWiaUIExtension": SimTypeBottom(label="IWiaUIExtension"),
    "IWiaUIExtension2": SimTypeBottom(label="IWiaUIExtension2"),
    "IWiaVideo": SimTypeBottom(label="IWiaVideo"),
    "IWinHttpRequest": SimTypeBottom(label="IWinHttpRequest"),
    "IWinHttpRequestEvents": SimTypeBottom(label="IWinHttpRequestEvents"),
    "IWinInetCacheHints": SimTypeBottom(label="IWinInetCacheHints"),
    "IWinInetCacheHints2": SimTypeBottom(label="IWinInetCacheHints2"),
    "IWinInetFileStream": SimTypeBottom(label="IWinInetFileStream"),
    "IWinInetHttpInfo": SimTypeBottom(label="IWinInetHttpInfo"),
    "IWinInetHttpTimeouts": SimTypeBottom(label="IWinInetHttpTimeouts"),
    "IWinInetInfo": SimTypeBottom(label="IWinInetInfo"),
    "IWinMLEvaluationContext": SimTypeBottom(label="IWinMLEvaluationContext"),
    "IWinMLModel": SimTypeBottom(label="IWinMLModel"),
    "IWinMLRuntime": SimTypeBottom(label="IWinMLRuntime"),
    "IWinMLRuntimeFactory": SimTypeBottom(label="IWinMLRuntimeFactory"),
    "IWinSATInitiateEvents": SimTypeBottom(label="IWinSATInitiateEvents"),
    "IWindowForBindingUI": SimTypeBottom(label="IWindowForBindingUI"),
    "IWindowProvider": SimTypeBottom(label="IWindowProvider"),
    "IWindowsDriverUpdate": SimTypeBottom(label="IWindowsDriverUpdate"),
    "IWindowsDriverUpdate2": SimTypeBottom(label="IWindowsDriverUpdate2"),
    "IWindowsDriverUpdate3": SimTypeBottom(label="IWindowsDriverUpdate3"),
    "IWindowsDriverUpdate4": SimTypeBottom(label="IWindowsDriverUpdate4"),
    "IWindowsDriverUpdate5": SimTypeBottom(label="IWindowsDriverUpdate5"),
    "IWindowsDriverUpdateEntry": SimTypeBottom(label="IWindowsDriverUpdateEntry"),
    "IWindowsDriverUpdateEntryCollection": SimTypeBottom(label="IWindowsDriverUpdateEntryCollection"),
    "IWindowsLockModeHelper": SimTypeBottom(label="IWindowsLockModeHelper"),
    "IWindowsMediaLibrarySharingDevice": SimTypeBottom(label="IWindowsMediaLibrarySharingDevice"),
    "IWindowsMediaLibrarySharingDeviceProperties": SimTypeBottom(label="IWindowsMediaLibrarySharingDeviceProperties"),
    "IWindowsMediaLibrarySharingDeviceProperty": SimTypeBottom(label="IWindowsMediaLibrarySharingDeviceProperty"),
    "IWindowsMediaLibrarySharingDevices": SimTypeBottom(label="IWindowsMediaLibrarySharingDevices"),
    "IWindowsMediaLibrarySharingServices": SimTypeBottom(label="IWindowsMediaLibrarySharingServices"),
    "IWindowsParentalControls": SimTypeBottom(label="IWindowsParentalControls"),
    "IWindowsParentalControlsCore": SimTypeBottom(label="IWindowsParentalControlsCore"),
    "IWindowsUpdateAgentInfo": SimTypeBottom(label="IWindowsUpdateAgentInfo"),
    "IWizardExtension": SimTypeBottom(label="IWizardExtension"),
    "IWizardSite": SimTypeBottom(label="IWizardSite"),
    "IWordBreaker": SimTypeBottom(label="IWordBreaker"),
    "IWordBreakerConfig": SimTypeBottom(label="IWordBreakerConfig"),
    "IWordFormSink": SimTypeBottom(label="IWordFormSink"),
    "IWordSink": SimTypeBottom(label="IWordSink"),
    "IWorkspace": SimTypeBottom(label="IWorkspace"),
    "IWorkspace2": SimTypeBottom(label="IWorkspace2"),
    "IWorkspace3": SimTypeBottom(label="IWorkspace3"),
    "IWorkspaceClientExt": SimTypeBottom(label="IWorkspaceClientExt"),
    "IWorkspaceRegistration": SimTypeBottom(label="IWorkspaceRegistration"),
    "IWorkspaceRegistration2": SimTypeBottom(label="IWorkspaceRegistration2"),
    "IWorkspaceReportMessage": SimTypeBottom(label="IWorkspaceReportMessage"),
    "IWorkspaceResTypeRegistry": SimTypeBottom(label="IWorkspaceResTypeRegistry"),
    "IWorkspaceScriptable": SimTypeBottom(label="IWorkspaceScriptable"),
    "IWorkspaceScriptable2": SimTypeBottom(label="IWorkspaceScriptable2"),
    "IWorkspaceScriptable3": SimTypeBottom(label="IWorkspaceScriptable3"),
    "IWpdSerializer": SimTypeBottom(label="IWpdSerializer"),
    "IWrappedProtocol": SimTypeBottom(label="IWrappedProtocol"),
    "IWriteEngine2": SimTypeBottom(label="IWriteEngine2"),
    "IWriteEngine2EventArgs": SimTypeBottom(label="IWriteEngine2EventArgs"),
    "IWriteSpeedDescriptor": SimTypeBottom(label="IWriteSpeedDescriptor"),
    "IWsbApplicationAsync": SimTypeBottom(label="IWsbApplicationAsync"),
    "IWsbApplicationBackupSupport": SimTypeBottom(label="IWsbApplicationBackupSupport"),
    "IWsbApplicationRestoreSupport": SimTypeBottom(label="IWsbApplicationRestoreSupport"),
    "IWscProduct": SimTypeBottom(label="IWscProduct"),
    "IWscProduct2": SimTypeBottom(label="IWscProduct2"),
    "IWscProduct3": SimTypeBottom(label="IWscProduct3"),
    "IX500DistinguishedName": SimTypeBottom(label="IX500DistinguishedName"),
    "IX509Attribute": SimTypeBottom(label="IX509Attribute"),
    "IX509AttributeArchiveKey": SimTypeBottom(label="IX509AttributeArchiveKey"),
    "IX509AttributeArchiveKeyHash": SimTypeBottom(label="IX509AttributeArchiveKeyHash"),
    "IX509AttributeClientId": SimTypeBottom(label="IX509AttributeClientId"),
    "IX509AttributeCspProvider": SimTypeBottom(label="IX509AttributeCspProvider"),
    "IX509AttributeExtensions": SimTypeBottom(label="IX509AttributeExtensions"),
    "IX509AttributeOSVersion": SimTypeBottom(label="IX509AttributeOSVersion"),
    "IX509AttributeRenewalCertificate": SimTypeBottom(label="IX509AttributeRenewalCertificate"),
    "IX509Attributes": SimTypeBottom(label="IX509Attributes"),
    "IX509CertificateRequest": SimTypeBottom(label="IX509CertificateRequest"),
    "IX509CertificateRequestCertificate": SimTypeBottom(label="IX509CertificateRequestCertificate"),
    "IX509CertificateRequestCertificate2": SimTypeBottom(label="IX509CertificateRequestCertificate2"),
    "IX509CertificateRequestCmc": SimTypeBottom(label="IX509CertificateRequestCmc"),
    "IX509CertificateRequestCmc2": SimTypeBottom(label="IX509CertificateRequestCmc2"),
    "IX509CertificateRequestPkcs10": SimTypeBottom(label="IX509CertificateRequestPkcs10"),
    "IX509CertificateRequestPkcs10V2": SimTypeBottom(label="IX509CertificateRequestPkcs10V2"),
    "IX509CertificateRequestPkcs10V3": SimTypeBottom(label="IX509CertificateRequestPkcs10V3"),
    "IX509CertificateRequestPkcs10V4": SimTypeBottom(label="IX509CertificateRequestPkcs10V4"),
    "IX509CertificateRequestPkcs7": SimTypeBottom(label="IX509CertificateRequestPkcs7"),
    "IX509CertificateRequestPkcs7V2": SimTypeBottom(label="IX509CertificateRequestPkcs7V2"),
    "IX509CertificateRevocationList": SimTypeBottom(label="IX509CertificateRevocationList"),
    "IX509CertificateRevocationListEntries": SimTypeBottom(label="IX509CertificateRevocationListEntries"),
    "IX509CertificateRevocationListEntry": SimTypeBottom(label="IX509CertificateRevocationListEntry"),
    "IX509CertificateTemplate": SimTypeBottom(label="IX509CertificateTemplate"),
    "IX509CertificateTemplateWritable": SimTypeBottom(label="IX509CertificateTemplateWritable"),
    "IX509CertificateTemplates": SimTypeBottom(label="IX509CertificateTemplates"),
    "IX509EndorsementKey": SimTypeBottom(label="IX509EndorsementKey"),
    "IX509Enrollment": SimTypeBottom(label="IX509Enrollment"),
    "IX509Enrollment2": SimTypeBottom(label="IX509Enrollment2"),
    "IX509EnrollmentHelper": SimTypeBottom(label="IX509EnrollmentHelper"),
    "IX509EnrollmentPolicyServer": SimTypeBottom(label="IX509EnrollmentPolicyServer"),
    "IX509EnrollmentStatus": SimTypeBottom(label="IX509EnrollmentStatus"),
    "IX509EnrollmentWebClassFactory": SimTypeBottom(label="IX509EnrollmentWebClassFactory"),
    "IX509Extension": SimTypeBottom(label="IX509Extension"),
    "IX509ExtensionAlternativeNames": SimTypeBottom(label="IX509ExtensionAlternativeNames"),
    "IX509ExtensionAuthorityKeyIdentifier": SimTypeBottom(label="IX509ExtensionAuthorityKeyIdentifier"),
    "IX509ExtensionBasicConstraints": SimTypeBottom(label="IX509ExtensionBasicConstraints"),
    "IX509ExtensionCertificatePolicies": SimTypeBottom(label="IX509ExtensionCertificatePolicies"),
    "IX509ExtensionEnhancedKeyUsage": SimTypeBottom(label="IX509ExtensionEnhancedKeyUsage"),
    "IX509ExtensionKeyUsage": SimTypeBottom(label="IX509ExtensionKeyUsage"),
    "IX509ExtensionMSApplicationPolicies": SimTypeBottom(label="IX509ExtensionMSApplicationPolicies"),
    "IX509ExtensionSmimeCapabilities": SimTypeBottom(label="IX509ExtensionSmimeCapabilities"),
    "IX509ExtensionSubjectKeyIdentifier": SimTypeBottom(label="IX509ExtensionSubjectKeyIdentifier"),
    "IX509ExtensionTemplate": SimTypeBottom(label="IX509ExtensionTemplate"),
    "IX509ExtensionTemplateName": SimTypeBottom(label="IX509ExtensionTemplateName"),
    "IX509Extensions": SimTypeBottom(label="IX509Extensions"),
    "IX509MachineEnrollmentFactory": SimTypeBottom(label="IX509MachineEnrollmentFactory"),
    "IX509NameValuePair": SimTypeBottom(label="IX509NameValuePair"),
    "IX509NameValuePairs": SimTypeBottom(label="IX509NameValuePairs"),
    "IX509PolicyServerListManager": SimTypeBottom(label="IX509PolicyServerListManager"),
    "IX509PolicyServerUrl": SimTypeBottom(label="IX509PolicyServerUrl"),
    "IX509PrivateKey": SimTypeBottom(label="IX509PrivateKey"),
    "IX509PrivateKey2": SimTypeBottom(label="IX509PrivateKey2"),
    "IX509PublicKey": SimTypeBottom(label="IX509PublicKey"),
    "IX509SCEPEnrollment": SimTypeBottom(label="IX509SCEPEnrollment"),
    "IX509SCEPEnrollment2": SimTypeBottom(label="IX509SCEPEnrollment2"),
    "IX509SCEPEnrollmentHelper": SimTypeBottom(label="IX509SCEPEnrollmentHelper"),
    "IX509SignatureInformation": SimTypeBottom(label="IX509SignatureInformation"),
    "IXAConfig": SimTypeBottom(label="IXAConfig"),
    "IXAObtainRMInfo": SimTypeBottom(label="IXAObtainRMInfo"),
    "IXAPO": SimTypeBottom(label="IXAPO"),
    "IXAPOHrtfParameters": SimTypeBottom(label="IXAPOHrtfParameters"),
    "IXAPOParameters": SimTypeBottom(label="IXAPOParameters"),
    "IXATransLookup": SimTypeBottom(label="IXATransLookup"),
    "IXATransLookup2": SimTypeBottom(label="IXATransLookup2"),
    "IXAudio2": SimTypeBottom(label="IXAudio2"),
    "IXAudio2EngineCallback": SimTypeBottom(label="IXAudio2EngineCallback"),
    "IXAudio2Extension": SimTypeBottom(label="IXAudio2Extension"),
    "IXAudio2MasteringVoice": SimTypeBottom(label="IXAudio2MasteringVoice"),
    "IXAudio2SourceVoice": SimTypeBottom(label="IXAudio2SourceVoice"),
    "IXAudio2SubmixVoice": SimTypeBottom(label="IXAudio2SubmixVoice"),
    "IXAudio2Voice": SimTypeBottom(label="IXAudio2Voice"),
    "IXAudio2VoiceCallback": SimTypeBottom(label="IXAudio2VoiceCallback"),
    "IXDSCodec": SimTypeBottom(label="IXDSCodec"),
    "IXDSCodecConfig": SimTypeBottom(label="IXDSCodecConfig"),
    "IXDSCodecEvents": SimTypeBottom(label="IXDSCodecEvents"),
    "IXDSToRat": SimTypeBottom(label="IXDSToRat"),
    "IXFeed": SimTypeBottom(label="IXFeed"),
    "IXFeed2": SimTypeBottom(label="IXFeed2"),
    "IXFeedEnclosure": SimTypeBottom(label="IXFeedEnclosure"),
    "IXFeedEvents": SimTypeBottom(label="IXFeedEvents"),
    "IXFeedFolder": SimTypeBottom(label="IXFeedFolder"),
    "IXFeedFolderEvents": SimTypeBottom(label="IXFeedFolderEvents"),
    "IXFeedItem": SimTypeBottom(label="IXFeedItem"),
    "IXFeedItem2": SimTypeBottom(label="IXFeedItem2"),
    "IXFeedsEnum": SimTypeBottom(label="IXFeedsEnum"),
    "IXFeedsManager": SimTypeBottom(label="IXFeedsManager"),
    "IXMLAttribute": SimTypeBottom(label="IXMLAttribute"),
    "IXMLDOMAttribute": SimTypeBottom(label="IXMLDOMAttribute"),
    "IXMLDOMCDATASection": SimTypeBottom(label="IXMLDOMCDATASection"),
    "IXMLDOMCharacterData": SimTypeBottom(label="IXMLDOMCharacterData"),
    "IXMLDOMComment": SimTypeBottom(label="IXMLDOMComment"),
    "IXMLDOMDocument": SimTypeBottom(label="IXMLDOMDocument"),
    "IXMLDOMDocument2": SimTypeBottom(label="IXMLDOMDocument2"),
    "IXMLDOMDocument3": SimTypeBottom(label="IXMLDOMDocument3"),
    "IXMLDOMDocumentFragment": SimTypeBottom(label="IXMLDOMDocumentFragment"),
    "IXMLDOMDocumentType": SimTypeBottom(label="IXMLDOMDocumentType"),
    "IXMLDOMElement": SimTypeBottom(label="IXMLDOMElement"),
    "IXMLDOMEntity": SimTypeBottom(label="IXMLDOMEntity"),
    "IXMLDOMEntityReference": SimTypeBottom(label="IXMLDOMEntityReference"),
    "IXMLDOMImplementation": SimTypeBottom(label="IXMLDOMImplementation"),
    "IXMLDOMNamedNodeMap": SimTypeBottom(label="IXMLDOMNamedNodeMap"),
    "IXMLDOMNode": SimTypeBottom(label="IXMLDOMNode"),
    "IXMLDOMNodeList": SimTypeBottom(label="IXMLDOMNodeList"),
    "IXMLDOMNotation": SimTypeBottom(label="IXMLDOMNotation"),
    "IXMLDOMParseError": SimTypeBottom(label="IXMLDOMParseError"),
    "IXMLDOMParseError2": SimTypeBottom(label="IXMLDOMParseError2"),
    "IXMLDOMParseErrorCollection": SimTypeBottom(label="IXMLDOMParseErrorCollection"),
    "IXMLDOMProcessingInstruction": SimTypeBottom(label="IXMLDOMProcessingInstruction"),
    "IXMLDOMSchemaCollection": SimTypeBottom(label="IXMLDOMSchemaCollection"),
    "IXMLDOMSchemaCollection2": SimTypeBottom(label="IXMLDOMSchemaCollection2"),
    "IXMLDOMSelection": SimTypeBottom(label="IXMLDOMSelection"),
    "IXMLDOMText": SimTypeBottom(label="IXMLDOMText"),
    "IXMLDSOControl": SimTypeBottom(label="IXMLDSOControl"),
    "IXMLDocument": SimTypeBottom(label="IXMLDocument"),
    "IXMLDocument2": SimTypeBottom(label="IXMLDocument2"),
    "IXMLElement": SimTypeBottom(label="IXMLElement"),
    "IXMLElement2": SimTypeBottom(label="IXMLElement2"),
    "IXMLElementCollection": SimTypeBottom(label="IXMLElementCollection"),
    "IXMLError": SimTypeBottom(label="IXMLError"),
    "IXMLGenericParse": SimTypeBottom(label="IXMLGenericParse"),
    "IXMLGraphBuilder": SimTypeBottom(label="IXMLGraphBuilder"),
    "IXMLHTTPRequest": SimTypeBottom(label="IXMLHTTPRequest"),
    "IXMLHTTPRequest2": SimTypeBottom(label="IXMLHTTPRequest2"),
    "IXMLHTTPRequest2Callback": SimTypeBottom(label="IXMLHTTPRequest2Callback"),
    "IXMLHTTPRequest3": SimTypeBottom(label="IXMLHTTPRequest3"),
    "IXMLHTTPRequest3Callback": SimTypeBottom(label="IXMLHTTPRequest3Callback"),
    "IXMLHttpRequestEventTarget": SimTypeBottom(label="IXMLHttpRequestEventTarget"),
    "IXSLProcessor": SimTypeBottom(label="IXSLProcessor"),
    "IXSLTemplate": SimTypeBottom(label="IXSLTemplate"),
    "IXTLRuntime": SimTypeBottom(label="IXTLRuntime"),
    "IXamlDiagnostics": SimTypeBottom(label="IXamlDiagnostics"),
    "IXblIdpAuthManager": SimTypeBottom(label="IXblIdpAuthManager"),
    "IXblIdpAuthManager2": SimTypeBottom(label="IXblIdpAuthManager2"),
    "IXblIdpAuthTokenResult": SimTypeBottom(label="IXblIdpAuthTokenResult"),
    "IXblIdpAuthTokenResult2": SimTypeBottom(label="IXblIdpAuthTokenResult2"),
    "IXmlReader": SimTypeBottom(label="IXmlReader"),
    "IXmlResolver": SimTypeBottom(label="IXmlResolver"),
    "IXmlWriter": SimTypeBottom(label="IXmlWriter"),
    "IXmlWriterLite": SimTypeBottom(label="IXmlWriterLite"),
    "IXpsDocument": SimTypeBottom(label="IXpsDocument"),
    "IXpsDocumentConsumer": SimTypeBottom(label="IXpsDocumentConsumer"),
    "IXpsDocumentPackageTarget": SimTypeBottom(label="IXpsDocumentPackageTarget"),
    "IXpsDocumentPackageTarget3D": SimTypeBottom(label="IXpsDocumentPackageTarget3D"),
    "IXpsDocumentProvider": SimTypeBottom(label="IXpsDocumentProvider"),
    "IXpsOMBrush": SimTypeBottom(label="IXpsOMBrush"),
    "IXpsOMCanvas": SimTypeBottom(label="IXpsOMCanvas"),
    "IXpsOMColorProfileResource": SimTypeBottom(label="IXpsOMColorProfileResource"),
    "IXpsOMColorProfileResourceCollection": SimTypeBottom(label="IXpsOMColorProfileResourceCollection"),
    "IXpsOMCoreProperties": SimTypeBottom(label="IXpsOMCoreProperties"),
    "IXpsOMDashCollection": SimTypeBottom(label="IXpsOMDashCollection"),
    "IXpsOMDictionary": SimTypeBottom(label="IXpsOMDictionary"),
    "IXpsOMDocument": SimTypeBottom(label="IXpsOMDocument"),
    "IXpsOMDocumentCollection": SimTypeBottom(label="IXpsOMDocumentCollection"),
    "IXpsOMDocumentSequence": SimTypeBottom(label="IXpsOMDocumentSequence"),
    "IXpsOMDocumentStructureResource": SimTypeBottom(label="IXpsOMDocumentStructureResource"),
    "IXpsOMFontResource": SimTypeBottom(label="IXpsOMFontResource"),
    "IXpsOMFontResourceCollection": SimTypeBottom(label="IXpsOMFontResourceCollection"),
    "IXpsOMGeometry": SimTypeBottom(label="IXpsOMGeometry"),
    "IXpsOMGeometryFigure": SimTypeBottom(label="IXpsOMGeometryFigure"),
    "IXpsOMGeometryFigureCollection": SimTypeBottom(label="IXpsOMGeometryFigureCollection"),
    "IXpsOMGlyphs": SimTypeBottom(label="IXpsOMGlyphs"),
    "IXpsOMGlyphsEditor": SimTypeBottom(label="IXpsOMGlyphsEditor"),
    "IXpsOMGradientBrush": SimTypeBottom(label="IXpsOMGradientBrush"),
    "IXpsOMGradientStop": SimTypeBottom(label="IXpsOMGradientStop"),
    "IXpsOMGradientStopCollection": SimTypeBottom(label="IXpsOMGradientStopCollection"),
    "IXpsOMImageBrush": SimTypeBottom(label="IXpsOMImageBrush"),
    "IXpsOMImageResource": SimTypeBottom(label="IXpsOMImageResource"),
    "IXpsOMImageResourceCollection": SimTypeBottom(label="IXpsOMImageResourceCollection"),
    "IXpsOMLinearGradientBrush": SimTypeBottom(label="IXpsOMLinearGradientBrush"),
    "IXpsOMMatrixTransform": SimTypeBottom(label="IXpsOMMatrixTransform"),
    "IXpsOMNameCollection": SimTypeBottom(label="IXpsOMNameCollection"),
    "IXpsOMObjectFactory": SimTypeBottom(label="IXpsOMObjectFactory"),
    "IXpsOMObjectFactory1": SimTypeBottom(label="IXpsOMObjectFactory1"),
    "IXpsOMPackage": SimTypeBottom(label="IXpsOMPackage"),
    "IXpsOMPackage1": SimTypeBottom(label="IXpsOMPackage1"),
    "IXpsOMPackageTarget": SimTypeBottom(label="IXpsOMPackageTarget"),
    "IXpsOMPackageWriter": SimTypeBottom(label="IXpsOMPackageWriter"),
    "IXpsOMPackageWriter3D": SimTypeBottom(label="IXpsOMPackageWriter3D"),
    "IXpsOMPage": SimTypeBottom(label="IXpsOMPage"),
    "IXpsOMPage1": SimTypeBottom(label="IXpsOMPage1"),
    "IXpsOMPageReference": SimTypeBottom(label="IXpsOMPageReference"),
    "IXpsOMPageReferenceCollection": SimTypeBottom(label="IXpsOMPageReferenceCollection"),
    "IXpsOMPart": SimTypeBottom(label="IXpsOMPart"),
    "IXpsOMPartResources": SimTypeBottom(label="IXpsOMPartResources"),
    "IXpsOMPartUriCollection": SimTypeBottom(label="IXpsOMPartUriCollection"),
    "IXpsOMPath": SimTypeBottom(label="IXpsOMPath"),
    "IXpsOMPrintTicketResource": SimTypeBottom(label="IXpsOMPrintTicketResource"),
    "IXpsOMRadialGradientBrush": SimTypeBottom(label="IXpsOMRadialGradientBrush"),
    "IXpsOMRemoteDictionaryResource": SimTypeBottom(label="IXpsOMRemoteDictionaryResource"),
    "IXpsOMRemoteDictionaryResource1": SimTypeBottom(label="IXpsOMRemoteDictionaryResource1"),
    "IXpsOMRemoteDictionaryResourceCollection": SimTypeBottom(label="IXpsOMRemoteDictionaryResourceCollection"),
    "IXpsOMResource": SimTypeBottom(label="IXpsOMResource"),
    "IXpsOMShareable": SimTypeBottom(label="IXpsOMShareable"),
    "IXpsOMSignatureBlockResource": SimTypeBottom(label="IXpsOMSignatureBlockResource"),
    "IXpsOMSignatureBlockResourceCollection": SimTypeBottom(label="IXpsOMSignatureBlockResourceCollection"),
    "IXpsOMSolidColorBrush": SimTypeBottom(label="IXpsOMSolidColorBrush"),
    "IXpsOMStoryFragmentsResource": SimTypeBottom(label="IXpsOMStoryFragmentsResource"),
    "IXpsOMThumbnailGenerator": SimTypeBottom(label="IXpsOMThumbnailGenerator"),
    "IXpsOMTileBrush": SimTypeBottom(label="IXpsOMTileBrush"),
    "IXpsOMVisual": SimTypeBottom(label="IXpsOMVisual"),
    "IXpsOMVisualBrush": SimTypeBottom(label="IXpsOMVisualBrush"),
    "IXpsOMVisualCollection": SimTypeBottom(label="IXpsOMVisualCollection"),
    "IXpsPartIterator": SimTypeBottom(label="IXpsPartIterator"),
    "IXpsPrintJob": SimTypeBottom(label="IXpsPrintJob"),
    "IXpsPrintJobStream": SimTypeBottom(label="IXpsPrintJobStream"),
    "IXpsRasterizationFactory": SimTypeBottom(label="IXpsRasterizationFactory"),
    "IXpsRasterizationFactory1": SimTypeBottom(label="IXpsRasterizationFactory1"),
    "IXpsRasterizationFactory2": SimTypeBottom(label="IXpsRasterizationFactory2"),
    "IXpsRasterizer": SimTypeBottom(label="IXpsRasterizer"),
    "IXpsRasterizerNotificationCallback": SimTypeBottom(label="IXpsRasterizerNotificationCallback"),
    "IXpsSignature": SimTypeBottom(label="IXpsSignature"),
    "IXpsSignatureBlock": SimTypeBottom(label="IXpsSignatureBlock"),
    "IXpsSignatureBlockCollection": SimTypeBottom(label="IXpsSignatureBlockCollection"),
    "IXpsSignatureCollection": SimTypeBottom(label="IXpsSignatureCollection"),
    "IXpsSignatureManager": SimTypeBottom(label="IXpsSignatureManager"),
    "IXpsSignatureRequest": SimTypeBottom(label="IXpsSignatureRequest"),
    "IXpsSignatureRequestCollection": SimTypeBottom(label="IXpsSignatureRequestCollection"),
    "IXpsSigningOptions": SimTypeBottom(label="IXpsSigningOptions"),
    "IZoneIdentifier": SimTypeBottom(label="IZoneIdentifier"),
    "IZoneIdentifier2": SimTypeBottom(label="IZoneIdentifier2"),
    "IZoomEvents": SimTypeBottom(label="IZoomEvents"),
    "I_RpcFreeCalloutStateFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RDR_CALLOUT_STATE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallOutState"]), offset=0),
    "I_RpcPerformCalloutFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("RDR_CALLOUT_STATE", SimStruct), offset=0), SimTypeInt(signed=False, label="RPC_HTTP_REDIRECTOR_STAGE")], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "CallOutState", "Stage"]), offset=0),
    "I_RpcProxyCallbackInterface": SimStruct(OrderedDict((("IsValidMachineFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Machine", "DotMachine", "PortNumber"]), offset=0)), ("GetClientAddressFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "Buffer", "BufferLength"]), offset=0)), ("GetConnectionTimeoutFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ConnectionTimeout"]), offset=0)), ("PerformCalloutFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("RDR_CALLOUT_STATE", SimStruct), offset=0), SimTypeInt(signed=False, label="RPC_HTTP_REDIRECTOR_STAGE")], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "CallOutState", "Stage"]), offset=0)), ("FreeCalloutStateFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RDR_CALLOUT_STATE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallOutState"]), offset=0)), ("GetClientSessionAndResourceUUIDFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "SessionIdPresent", "SessionId", "ResourceIdPresent", "ResourceId"]), offset=0)), ("ProxyFilterIfFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "IfUuid", "IfMajorVersion", "fAllow"]), offset=0)), ("RpcProxyUpdatePerfCounterFn", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="RpcPerfCounters"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Counter", "ModifyTrend", "Size"]), offset=0)), ("RpcProxyUpdatePerfCounterBackendServerFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["MachineName", "IsConnectEvent"]), offset=0)),)), name="I_RpcProxyCallbackInterface", pack=False, align=None),
    "I_RpcProxyFilterIfFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "IfUuid", "IfMajorVersion", "fAllow"]), offset=0),
    "I_RpcProxyGetClientAddressFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "Buffer", "BufferLength"]), offset=0),
    "I_RpcProxyGetClientSessionAndResourceUUID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Context", "SessionIdPresent", "SessionId", "ResourceIdPresent", "ResourceId"]), offset=0),
    "I_RpcProxyGetConnectionTimeoutFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ConnectionTimeout"]), offset=0),
    "I_RpcProxyIsValidMachineFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Machine", "DotMachine", "PortNumber"]), offset=0),
    "I_RpcProxyUpdatePerfCounterBackendServerFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["MachineName", "IsConnectEvent"]), offset=0),
    "I_RpcProxyUpdatePerfCounterFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="RpcPerfCounters"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Counter", "ModifyTrend", "Size"]), offset=0),
    "IconIdentifier": SimTypeInt(signed=False, label="IconIdentifier"),
    "IdentityUpdateEvent": SimTypeInt(signed=False, label="IdentityUpdateEvent"),
    "Image": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ImageAbort": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "ImageCodecFlags": SimTypeInt(signed=False, label="ImageCodecFlags"),
    "ImageCodecInfo": SimStruct(OrderedDict((("Clsid", SimTypeBottom(label="Guid")), ("FormatID", SimTypeBottom(label="Guid")), ("CodecName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DllName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FormatDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FilenameExtension", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MimeType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SigCount", SimTypeInt(signed=False, label="UInt32")), ("SigSize", SimTypeInt(signed=False, label="UInt32")), ("SigPattern", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SigMask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="ImageCodecInfo", pack=False, align=None),
    "ImageFlags": SimTypeInt(signed=False, label="ImageFlags"),
    "ImageItemData": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("Desc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DescSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Cookie", SimTypeInt(signed=False, label="UInt32")),)), name="ImageItemData", pack=False, align=None),
    "ImageLockMode": SimTypeInt(signed=False, label="ImageLockMode"),
    "ImageType": SimTypeInt(signed=False, label="ImageType"),
    "ImgErrorInfo": SimStruct(OrderedDict((("description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("guid", SimTypeBottom(label="Guid")), ("helpContext", SimTypeInt(signed=False, label="UInt32")), ("helpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("source", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("devDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("errorID", SimTypeBottom(label="Guid")), ("cUserParameters", SimTypeInt(signed=False, label="UInt32")), ("aUserParameters", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("userFallback", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("exceptionID", SimTypeInt(signed=False, label="UInt32")),)), name="ImgErrorInfo", pack=False, align=None),
    "ImportPFXFlags": SimTypeInt(signed=False, label="ImportPFXFlags"),
    "InPlaceDirection": SimTypeInt(signed=False, label="InPlaceDirection"),
    "InPlaceState": SimTypeInt(signed=False, label="InPlaceState"),
    "IncomingCookieState": SimStruct(OrderedDict((("cSession", SimTypeInt(signed=True, label="Int32")), ("cPersistent", SimTypeInt(signed=True, label="Int32")), ("cAccepted", SimTypeInt(signed=True, label="Int32")), ("cLeashed", SimTypeInt(signed=True, label="Int32")), ("cDowngraded", SimTypeInt(signed=True, label="Int32")), ("cBlocked", SimTypeInt(signed=True, label="Int32")), ("pszLocation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="IncomingCookieState", pack=False, align=None),
    "IndexedResourceQualifier": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="IndexedResourceQualifier", pack=False, align=None),
    "InkApplicationGesture": SimTypeInt(signed=False, label="InkApplicationGesture"),
    "InkBoundingBoxMode": SimTypeInt(signed=False, label="InkBoundingBoxMode"),
    "InkClipboardFormats": SimTypeInt(signed=False, label="InkClipboardFormats"),
    "InkClipboardModes": SimTypeInt(signed=False, label="InkClipboardModes"),
    "InkCollectionMode": SimTypeInt(signed=False, label="InkCollectionMode"),
    "InkCollectorEventInterest": SimTypeInt(signed=False, label="InkCollectorEventInterest"),
    "InkCursorButtonState": SimTypeInt(signed=False, label="InkCursorButtonState"),
    "InkDisplayMode": SimTypeInt(signed=False, label="InkDisplayMode"),
    "InkDivisionType": SimTypeInt(signed=False, label="InkDivisionType"),
    "InkEditStatus": SimTypeInt(signed=False, label="InkEditStatus"),
    "InkExtractFlags": SimTypeInt(signed=False, label="InkExtractFlags"),
    "InkInsertMode": SimTypeInt(signed=False, label="InkInsertMode"),
    "InkMode": SimTypeInt(signed=False, label="InkMode"),
    "InkMouseButton": SimTypeInt(signed=False, label="InkMouseButton"),
    "InkMousePointer": SimTypeInt(signed=False, label="InkMousePointer"),
    "InkOverlayAttachMode": SimTypeInt(signed=False, label="InkOverlayAttachMode"),
    "InkOverlayEditingMode": SimTypeInt(signed=False, label="InkOverlayEditingMode"),
    "InkOverlayEraserMode": SimTypeInt(signed=False, label="InkOverlayEraserMode"),
    "InkPenTip": SimTypeInt(signed=False, label="InkPenTip"),
    "InkPersistenceCompressionMode": SimTypeInt(signed=False, label="InkPersistenceCompressionMode"),
    "InkPersistenceFormat": SimTypeInt(signed=False, label="InkPersistenceFormat"),
    "InkPictureSizeMode": SimTypeInt(signed=False, label="InkPictureSizeMode"),
    "InkRasterOperation": SimTypeInt(signed=False, label="InkRasterOperation"),
    "InkRecoGuide": SimStruct(OrderedDict((("rectWritingBox", SimTypeRef("RECT", SimStruct)), ("rectDrawnBox", SimTypeRef("RECT", SimStruct)), ("cRows", SimTypeInt(signed=True, label="Int32")), ("cColumns", SimTypeInt(signed=True, label="Int32")), ("midline", SimTypeInt(signed=True, label="Int32")),)), name="InkRecoGuide", pack=False, align=None),
    "InkRecognitionAlternatesSelection": SimTypeInt(signed=False, label="InkRecognitionAlternatesSelection"),
    "InkRecognitionConfidence": SimTypeInt(signed=False, label="InkRecognitionConfidence"),
    "InkRecognitionModes": SimTypeInt(signed=False, label="InkRecognitionModes"),
    "InkRecognitionStatus": SimTypeInt(signed=False, label="InkRecognitionStatus"),
    "InkRecognizerCapabilities": SimTypeInt(signed=False, label="InkRecognizerCapabilities"),
    "InkRecognizerCharacterAutoCompletionMode": SimTypeInt(signed=False, label="InkRecognizerCharacterAutoCompletionMode"),
    "InkSelectionConstants": SimTypeInt(signed=False, label="InkSelectionConstants"),
    "InkShiftKeyModifierFlags": SimTypeInt(signed=False, label="InkShiftKeyModifierFlags"),
    "InkSystemGesture": SimTypeInt(signed=False, label="InkSystemGesture"),
    "InnerRequestLevel": SimTypeInt(signed=False, label="InnerRequestLevel"),
    "InputScope": SimTypeInt(signed=False, label="InputScope"),
    "InstallResponseRestrictionFlags": SimTypeInt(signed=False, label="InstallResponseRestrictionFlags"),
    "InstallationImpact": SimTypeInt(signed=False, label="InstallationImpact"),
    "InstallationRebootBehavior": SimTypeInt(signed=False, label="InstallationRebootBehavior"),
    "InstalledFontCollection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "IntServFlowSpec": SimStruct(OrderedDict((("spec_mh", SimTypeRef("IntServMainHdr", SimStruct)), ("spec_u", SimUnion({"CL_spec": SimTypeRef("CtrlLoadFlowspec", SimStruct), "G_spec": SimTypeRef("GuarFlowSpec", SimStruct), "Q_spec": SimTypeRef("QualAppFlowSpec", SimStruct)}, name="<anon>", label="None")),)), name="IntServFlowSpec", pack=False, align=None),
    "IntServMainHdr": SimStruct(OrderedDict((("ismh_version", SimTypeChar(label="Byte")), ("ismh_unused", SimTypeChar(label="Byte")), ("ismh_len32b", SimTypeShort(signed=False, label="UInt16")),)), name="IntServMainHdr", pack=False, align=None),
    "IntServParmHdr": SimStruct(OrderedDict((("isph_parm_num", SimTypeChar(label="Byte")), ("isph_flags", SimTypeChar(label="Byte")), ("isph_len32b", SimTypeShort(signed=False, label="UInt16")),)), name="IntServParmHdr", pack=False, align=None),
    "IntServServiceHdr": SimStruct(OrderedDict((("issh_service", SimTypeChar(label="Byte")), ("issh_flags", SimTypeChar(label="Byte")), ("issh_len32b", SimTypeShort(signed=False, label="UInt16")),)), name="IntServServiceHdr", pack=False, align=None),
    "IntServTspecBody": SimStruct(OrderedDict((("st_mh", SimTypeRef("IntServMainHdr", SimStruct)), ("tspec_u", SimUnion({"gen_stspec": SimTypeRef("GenTspec", SimStruct), "qual_stspec": SimTypeRef("QualTspec", SimStruct)}, name="<anon>", label="None")),)), name="IntServTspecBody", pack=False, align=None),
    "InteractionMode": SimTypeInt(signed=False, label="InteractionMode"),
    "InterleavingMode": SimTypeInt(signed=False, label="InterleavingMode"),
    "InternetCookieHistory": SimStruct(OrderedDict((("fAccepted", SimTypeInt(signed=True, label="Int32")), ("fLeashed", SimTypeInt(signed=True, label="Int32")), ("fDowngraded", SimTypeInt(signed=True, label="Int32")), ("fRejected", SimTypeInt(signed=True, label="Int32")),)), name="InternetCookieHistory", pack=False, align=None),
    "InternetCookieState": SimTypeInt(signed=False, label="InternetCookieState"),
    "InterpolationMode": SimTypeInt(signed=False, label="InterpolationMode"),
    "IntrinsicKind": SimTypeInt(signed=False, label="IntrinsicKind"),
    "Ipv4Address": SimStruct(OrderedDict((("addr", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="Ipv4Address", pack=False, align=None),
    "Ipv6Address": SimStruct(OrderedDict((("addr", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="Ipv6Address", pack=False, align=None),
    "IsolatedAppLauncherTelemetryParameters": SimStruct(OrderedDict((("EnableForLaunch", SimTypeInt(signed=True, label="Int32")), ("CorrelationGUID", SimTypeBottom(label="Guid")),)), name="IsolatedAppLauncherTelemetryParameters", pack=False, align=None),
    "IsolationInfo": SimStruct(OrderedDict((("isolationState", SimTypeInt(signed=False, label="IsolationState")), ("probEndTime", SimTypeRef("FILETIME", SimStruct)), ("failureUrl", SimTypeRef("CountedString", SimStruct)),)), name="IsolationInfo", pack=False, align=None),
    "IsolationInfoEx": SimStruct(OrderedDict((("isolationState", SimTypeInt(signed=False, label="IsolationState")), ("extendedIsolationState", SimTypeInt(signed=False, label="ExtendedIsolationState")), ("probEndTime", SimTypeRef("FILETIME", SimStruct)), ("failureUrl", SimTypeRef("CountedString", SimStruct)),)), name="IsolationInfoEx", pack=False, align=None),
    "IsolationState": SimTypeInt(signed=False, label="IsolationState"),
    "ItemDataPosition": SimTypeInt(signed=False, label="ItemDataPosition"),
    "ItsPubPlugin": SimTypeBottom(label="ItsPubPlugin"),
    "ItsPubPlugin2": SimTypeBottom(label="ItsPubPlugin2"),
    "Iwfolders": SimTypeBottom(label="Iwfolders"),
    "JAVA_TRUST": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("flag", SimTypeInt(signed=False, label="UInt32")), ("fAllActiveXPermissions", SimTypeInt(signed=True, label="Int32")), ("fAllPermissions", SimTypeInt(signed=True, label="Int32")), ("dwEncodingType", SimTypeInt(signed=False, label="UInt32")), ("pbJavaPermissions", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbJavaPermissions", SimTypeInt(signed=False, label="UInt32")), ("pbSigner", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbSigner", SimTypeInt(signed=False, label="UInt32")), ("pwszZone", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("guidZone", SimTypeBottom(label="Guid")), ("hVerify", SimTypeInt(signed=True, label="Int32")),)), name="JAVA_TRUST", pack=False, align=None),
    "JChColorF": SimStruct(OrderedDict((("J", SimTypeFloat(size=32)), ("C", SimTypeFloat(size=32)), ("h", SimTypeFloat(size=32)),)), name="JChColorF", pack=False, align=None),
    "JET_API_PTR": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_BKINFO": SimStruct(OrderedDict((("lgposMark", SimTypeRef("JET_LGPOS", SimStruct)), ("Anonymous", SimUnion({"logtimeMark": SimTypeRef("JET_LOGTIME", SimStruct), "bklogtimeMark": SimTypeRef("JET_BKLOGTIME", SimStruct)}, name="<anon>", label="None")), ("genLow", SimTypeInt(signed=False, label="UInt32")), ("genHigh", SimTypeInt(signed=False, label="UInt32")),)), name="JET_BKINFO", pack=False, align=None),
    "JET_BKLOGTIME": SimStruct(OrderedDict((("bSeconds", SimTypeChar(label="SByte")), ("bMinutes", SimTypeChar(label="SByte")), ("bHours", SimTypeChar(label="SByte")), ("bDay", SimTypeChar(label="SByte")), ("bMonth", SimTypeChar(label="SByte")), ("bYear", SimTypeChar(label="SByte")), ("Anonymous1", SimUnion({"bFiller1": SimTypeChar(label="SByte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"bFiller2": SimTypeChar(label="SByte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JET_BKLOGTIME", pack=False, align=None),
    "JET_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["sesid", "dbid", "tableid", "cbtyp", "pvArg1", "pvArg2", "pvContext", "ulUnused"]), offset=0),
    "JET_COLUMNBASE_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("columnid", SimTypeInt(signed=False, label="UInt32")), ("coltyp", SimTypeInt(signed=False, label="UInt32")), ("wCountry", SimTypeShort(signed=False, label="UInt16")), ("langid", SimTypeShort(signed=False, label="UInt16")), ("cp", SimTypeShort(signed=False, label="UInt16")), ("wFiller", SimTypeShort(signed=False, label="UInt16")), ("cbMax", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("szBaseTableName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("szBaseColumnName", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="JET_COLUMNBASE_A", pack=False, align=None),
    "JET_COLUMNBASE_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("columnid", SimTypeInt(signed=False, label="UInt32")), ("coltyp", SimTypeInt(signed=False, label="UInt32")), ("wCountry", SimTypeShort(signed=False, label="UInt16")), ("langid", SimTypeShort(signed=False, label="UInt16")), ("cp", SimTypeShort(signed=False, label="UInt16")), ("wFiller", SimTypeShort(signed=False, label="UInt16")), ("cbMax", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("szBaseTableName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("szBaseColumnName", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="JET_COLUMNBASE_W", pack=False, align=None),
    "JET_COLUMNCREATE_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szColumnName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("coltyp", SimTypeInt(signed=False, label="UInt32")), ("cbMax", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("pvDefault", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbDefault", SimTypeInt(signed=False, label="UInt32")), ("cp", SimTypeInt(signed=False, label="UInt32")), ("columnid", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")),)), name="JET_COLUMNCREATE_A", pack=False, align=None),
    "JET_COLUMNCREATE_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szColumnName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("coltyp", SimTypeInt(signed=False, label="UInt32")), ("cbMax", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("pvDefault", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbDefault", SimTypeInt(signed=False, label="UInt32")), ("cp", SimTypeInt(signed=False, label="UInt32")), ("columnid", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")),)), name="JET_COLUMNCREATE_W", pack=False, align=None),
    "JET_COLUMNDEF": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("columnid", SimTypeInt(signed=False, label="UInt32")), ("coltyp", SimTypeInt(signed=False, label="UInt32")), ("wCountry", SimTypeShort(signed=False, label="UInt16")), ("langid", SimTypeShort(signed=False, label="UInt16")), ("cp", SimTypeShort(signed=False, label="UInt16")), ("wCollate", SimTypeShort(signed=False, label="UInt16")), ("cbMax", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")),)), name="JET_COLUMNDEF", pack=False, align=None),
    "JET_COLUMNLIST": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cRecord", SimTypeInt(signed=False, label="UInt32")), ("columnidPresentationOrder", SimTypeInt(signed=False, label="UInt32")), ("columnidcolumnname", SimTypeInt(signed=False, label="UInt32")), ("columnidcolumnid", SimTypeInt(signed=False, label="UInt32")), ("columnidcoltyp", SimTypeInt(signed=False, label="UInt32")), ("columnidCountry", SimTypeInt(signed=False, label="UInt32")), ("columnidLangid", SimTypeInt(signed=False, label="UInt32")), ("columnidCp", SimTypeInt(signed=False, label="UInt32")), ("columnidCollate", SimTypeInt(signed=False, label="UInt32")), ("columnidcbMax", SimTypeInt(signed=False, label="UInt32")), ("columnidgrbit", SimTypeInt(signed=False, label="UInt32")), ("columnidDefault", SimTypeInt(signed=False, label="UInt32")), ("columnidBaseTableName", SimTypeInt(signed=False, label="UInt32")), ("columnidBaseColumnName", SimTypeInt(signed=False, label="UInt32")), ("columnidDefinitionName", SimTypeInt(signed=False, label="UInt32")),)), name="JET_COLUMNLIST", pack=False, align=None),
    "JET_COMMIT_ID": SimStruct(OrderedDict((("signLog", SimTypeRef("JET_SIGNATURE", SimStruct)), ("reserved", SimTypeInt(signed=True, label="Int32")), ("commitId", SimTypeLongLong(signed=True, label="Int64")),)), name="JET_COMMIT_ID", pack=False, align=None),
    "JET_CONDITIONALCOLUMN_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szColumnName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("grbit", SimTypeInt(signed=False, label="UInt32")),)), name="JET_CONDITIONALCOLUMN_A", pack=False, align=None),
    "JET_CONDITIONALCOLUMN_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szColumnName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grbit", SimTypeInt(signed=False, label="UInt32")),)), name="JET_CONDITIONALCOLUMN_W", pack=False, align=None),
    "JET_CONVERT_A": SimStruct(OrderedDict((("szOldDll", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Anonymous", SimUnion({"fFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JET_CONVERT_A", pack=False, align=None),
    "JET_CONVERT_W": SimStruct(OrderedDict((("szOldDll", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"fFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JET_CONVERT_W", pack=False, align=None),
    "JET_DBINFOMISC": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("ulUpdate", SimTypeInt(signed=False, label="UInt32")), ("signDb", SimTypeRef("JET_SIGNATURE", SimStruct)), ("dbstate", SimTypeInt(signed=False, label="UInt32")), ("lgposConsistent", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeConsistent", SimTypeRef("JET_LOGTIME", SimStruct)), ("logtimeAttach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposAttach", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeDetach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposDetach", SimTypeRef("JET_LGPOS", SimStruct)), ("signLog", SimTypeRef("JET_SIGNATURE", SimStruct)), ("bkinfoFullPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoIncPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoFullCur", SimTypeRef("JET_BKINFO", SimStruct)), ("fShadowingDisabled", SimTypeInt(signed=False, label="UInt32")), ("fUpgradeDb", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("lSPNumber", SimTypeInt(signed=True, label="Int32")), ("cbPageSize", SimTypeInt(signed=False, label="UInt32")),)), name="JET_DBINFOMISC", pack=False, align=None),
    "JET_DBINFOMISC2": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("ulUpdate", SimTypeInt(signed=False, label="UInt32")), ("signDb", SimTypeRef("JET_SIGNATURE", SimStruct)), ("dbstate", SimTypeInt(signed=False, label="UInt32")), ("lgposConsistent", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeConsistent", SimTypeRef("JET_LOGTIME", SimStruct)), ("logtimeAttach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposAttach", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeDetach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposDetach", SimTypeRef("JET_LGPOS", SimStruct)), ("signLog", SimTypeRef("JET_SIGNATURE", SimStruct)), ("bkinfoFullPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoIncPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoFullCur", SimTypeRef("JET_BKINFO", SimStruct)), ("fShadowingDisabled", SimTypeInt(signed=False, label="UInt32")), ("fUpgradeDb", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("lSPNumber", SimTypeInt(signed=True, label="Int32")), ("cbPageSize", SimTypeInt(signed=False, label="UInt32")), ("genMinRequired", SimTypeInt(signed=False, label="UInt32")), ("genMaxRequired", SimTypeInt(signed=False, label="UInt32")), ("logtimeGenMaxCreate", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulRepairCount", SimTypeInt(signed=False, label="UInt32")), ("logtimeRepair", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulRepairCountOld", SimTypeInt(signed=False, label="UInt32")), ("ulECCFixSuccess", SimTypeInt(signed=False, label="UInt32")), ("logtimeECCFixSuccess", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulECCFixSuccessOld", SimTypeInt(signed=False, label="UInt32")), ("ulECCFixFail", SimTypeInt(signed=False, label="UInt32")), ("logtimeECCFixFail", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulECCFixFailOld", SimTypeInt(signed=False, label="UInt32")), ("ulBadChecksum", SimTypeInt(signed=False, label="UInt32")), ("logtimeBadChecksum", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulBadChecksumOld", SimTypeInt(signed=False, label="UInt32")),)), name="JET_DBINFOMISC2", pack=False, align=None),
    "JET_DBINFOMISC3": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("ulUpdate", SimTypeInt(signed=False, label="UInt32")), ("signDb", SimTypeRef("JET_SIGNATURE", SimStruct)), ("dbstate", SimTypeInt(signed=False, label="UInt32")), ("lgposConsistent", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeConsistent", SimTypeRef("JET_LOGTIME", SimStruct)), ("logtimeAttach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposAttach", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeDetach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposDetach", SimTypeRef("JET_LGPOS", SimStruct)), ("signLog", SimTypeRef("JET_SIGNATURE", SimStruct)), ("bkinfoFullPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoIncPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoFullCur", SimTypeRef("JET_BKINFO", SimStruct)), ("fShadowingDisabled", SimTypeInt(signed=False, label="UInt32")), ("fUpgradeDb", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("lSPNumber", SimTypeInt(signed=True, label="Int32")), ("cbPageSize", SimTypeInt(signed=False, label="UInt32")), ("genMinRequired", SimTypeInt(signed=False, label="UInt32")), ("genMaxRequired", SimTypeInt(signed=False, label="UInt32")), ("logtimeGenMaxCreate", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulRepairCount", SimTypeInt(signed=False, label="UInt32")), ("logtimeRepair", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulRepairCountOld", SimTypeInt(signed=False, label="UInt32")), ("ulECCFixSuccess", SimTypeInt(signed=False, label="UInt32")), ("logtimeECCFixSuccess", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulECCFixSuccessOld", SimTypeInt(signed=False, label="UInt32")), ("ulECCFixFail", SimTypeInt(signed=False, label="UInt32")), ("logtimeECCFixFail", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulECCFixFailOld", SimTypeInt(signed=False, label="UInt32")), ("ulBadChecksum", SimTypeInt(signed=False, label="UInt32")), ("logtimeBadChecksum", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulBadChecksumOld", SimTypeInt(signed=False, label="UInt32")), ("genCommitted", SimTypeInt(signed=False, label="UInt32")),)), name="JET_DBINFOMISC3", pack=False, align=None),
    "JET_DBINFOMISC4": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("ulUpdate", SimTypeInt(signed=False, label="UInt32")), ("signDb", SimTypeRef("JET_SIGNATURE", SimStruct)), ("dbstate", SimTypeInt(signed=False, label="UInt32")), ("lgposConsistent", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeConsistent", SimTypeRef("JET_LOGTIME", SimStruct)), ("logtimeAttach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposAttach", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeDetach", SimTypeRef("JET_LOGTIME", SimStruct)), ("lgposDetach", SimTypeRef("JET_LGPOS", SimStruct)), ("signLog", SimTypeRef("JET_SIGNATURE", SimStruct)), ("bkinfoFullPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoIncPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoFullCur", SimTypeRef("JET_BKINFO", SimStruct)), ("fShadowingDisabled", SimTypeInt(signed=False, label="UInt32")), ("fUpgradeDb", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("lSPNumber", SimTypeInt(signed=True, label="Int32")), ("cbPageSize", SimTypeInt(signed=False, label="UInt32")), ("genMinRequired", SimTypeInt(signed=False, label="UInt32")), ("genMaxRequired", SimTypeInt(signed=False, label="UInt32")), ("logtimeGenMaxCreate", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulRepairCount", SimTypeInt(signed=False, label="UInt32")), ("logtimeRepair", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulRepairCountOld", SimTypeInt(signed=False, label="UInt32")), ("ulECCFixSuccess", SimTypeInt(signed=False, label="UInt32")), ("logtimeECCFixSuccess", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulECCFixSuccessOld", SimTypeInt(signed=False, label="UInt32")), ("ulECCFixFail", SimTypeInt(signed=False, label="UInt32")), ("logtimeECCFixFail", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulECCFixFailOld", SimTypeInt(signed=False, label="UInt32")), ("ulBadChecksum", SimTypeInt(signed=False, label="UInt32")), ("logtimeBadChecksum", SimTypeRef("JET_LOGTIME", SimStruct)), ("ulBadChecksumOld", SimTypeInt(signed=False, label="UInt32")), ("genCommitted", SimTypeInt(signed=False, label="UInt32")), ("bkinfoCopyPrev", SimTypeRef("JET_BKINFO", SimStruct)), ("bkinfoDiffPrev", SimTypeRef("JET_BKINFO", SimStruct)),)), name="JET_DBINFOMISC4", pack=False, align=None),
    "JET_DBINFOUPGRADE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cbFilesizeLow", SimTypeInt(signed=False, label="UInt32")), ("cbFilesizeHigh", SimTypeInt(signed=False, label="UInt32")), ("cbFreeSpaceRequiredLow", SimTypeInt(signed=False, label="UInt32")), ("cbFreeSpaceRequiredHigh", SimTypeInt(signed=False, label="UInt32")), ("csecToUpgrade", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ulFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JET_DBINFOUPGRADE", pack=False, align=None),
    "JET_ENUMCOLUMN": SimStruct(OrderedDict((("columnid", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("cEnumColumnValue", SimTypeInt(signed=False, label="UInt32")), ("rgEnumColumnValue", SimTypePointer(SimTypeRef("JET_ENUMCOLUMNVALUE", SimStruct), offset=0)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JET_ENUMCOLUMN", pack=False, align=None),
    "JET_ENUMCOLUMNID": SimStruct(OrderedDict((("columnid", SimTypeInt(signed=False, label="UInt32")), ("ctagSequence", SimTypeInt(signed=False, label="UInt32")), ("rgtagSequence", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="JET_ENUMCOLUMNID", pack=False, align=None),
    "JET_ENUMCOLUMNVALUE": SimStruct(OrderedDict((("itagSequence", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="JET_ENUMCOLUMNVALUE", pack=False, align=None),
    "JET_ERRCAT": SimTypeInt(signed=False, label="JET_ERRCAT"),
    "JET_ERRINFOBASIC_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("errValue", SimTypeInt(signed=True, label="Int32")), ("errcatMostSpecific", SimTypeInt(signed=False, label="JET_ERRCAT")), ("rgCategoricalHierarchy", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("lSourceLine", SimTypeInt(signed=False, label="UInt32")), ("rgszSourceFile", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="JET_ERRINFOBASIC_W", pack=False, align=None),
    "JET_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_INDEXCHECKING": SimTypeInt(signed=False, label="JET_INDEXCHECKING"),
    "JET_INDEXCREATE2_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szIndexName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lcid": SimTypeInt(signed=False, label="UInt32"), "pidxunicode": SimTypePointer(SimTypeRef("JET_UNICODEINDEX", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"cbVarSegMac": SimTypeInt(signed=False, label="UInt32"), "ptuplelimits": SimTypePointer(SimTypeRef("JET_TUPLELIMITS", SimStruct), offset=0)}, name="<anon>", label="None")), ("rgconditionalcolumn", SimTypePointer(SimTypeRef("JET_CONDITIONALCOLUMN_A", SimStruct), offset=0)), ("cConditionalColumn", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")), ("pSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)),)), name="JET_INDEXCREATE2_A", pack=False, align=None),
    "JET_INDEXCREATE2_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szIndexName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lcid": SimTypeInt(signed=False, label="UInt32"), "pidxunicode": SimTypePointer(SimTypeRef("JET_UNICODEINDEX", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"cbVarSegMac": SimTypeInt(signed=False, label="UInt32"), "ptuplelimits": SimTypePointer(SimTypeRef("JET_TUPLELIMITS", SimStruct), offset=0)}, name="<anon>", label="None")), ("rgconditionalcolumn", SimTypePointer(SimTypeRef("JET_CONDITIONALCOLUMN_W", SimStruct), offset=0)), ("cConditionalColumn", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")), ("pSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)),)), name="JET_INDEXCREATE2_W", pack=False, align=None),
    "JET_INDEXCREATE3_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szIndexName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("pidxunicode", SimTypePointer(SimTypeRef("JET_UNICODEINDEX2", SimStruct), offset=0)), ("Anonymous", SimUnion({"cbVarSegMac": SimTypeInt(signed=False, label="UInt32"), "ptuplelimits": SimTypePointer(SimTypeRef("JET_TUPLELIMITS", SimStruct), offset=0)}, name="<anon>", label="None")), ("rgconditionalcolumn", SimTypePointer(SimTypeRef("JET_CONDITIONALCOLUMN_A", SimStruct), offset=0)), ("cConditionalColumn", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")), ("pSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)),)), name="JET_INDEXCREATE3_A", pack=False, align=None),
    "JET_INDEXCREATE3_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szIndexName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("pidxunicode", SimTypePointer(SimTypeRef("JET_UNICODEINDEX2", SimStruct), offset=0)), ("Anonymous", SimUnion({"cbVarSegMac": SimTypeInt(signed=False, label="UInt32"), "ptuplelimits": SimTypePointer(SimTypeRef("JET_TUPLELIMITS", SimStruct), offset=0)}, name="<anon>", label="None")), ("rgconditionalcolumn", SimTypePointer(SimTypeRef("JET_CONDITIONALCOLUMN_W", SimStruct), offset=0)), ("cConditionalColumn", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")), ("pSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)),)), name="JET_INDEXCREATE3_W", pack=False, align=None),
    "JET_INDEXCREATE_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szIndexName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lcid": SimTypeInt(signed=False, label="UInt32"), "pidxunicode": SimTypePointer(SimTypeRef("JET_UNICODEINDEX", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"cbVarSegMac": SimTypeInt(signed=False, label="UInt32"), "ptuplelimits": SimTypePointer(SimTypeRef("JET_TUPLELIMITS", SimStruct), offset=0)}, name="<anon>", label="None")), ("rgconditionalcolumn", SimTypePointer(SimTypeRef("JET_CONDITIONALCOLUMN_A", SimStruct), offset=0)), ("cConditionalColumn", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")),)), name="JET_INDEXCREATE_A", pack=False, align=None),
    "JET_INDEXCREATE_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szIndexName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"lcid": SimTypeInt(signed=False, label="UInt32"), "pidxunicode": SimTypePointer(SimTypeRef("JET_UNICODEINDEX", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"cbVarSegMac": SimTypeInt(signed=False, label="UInt32"), "ptuplelimits": SimTypePointer(SimTypeRef("JET_TUPLELIMITS", SimStruct), offset=0)}, name="<anon>", label="None")), ("rgconditionalcolumn", SimTypePointer(SimTypeRef("JET_CONDITIONALCOLUMN_W", SimStruct), offset=0)), ("cConditionalColumn", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")),)), name="JET_INDEXCREATE_W", pack=False, align=None),
    "JET_INDEXID": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("rgbIndexId", SimTypeArray(SimTypeChar(label="Byte"), 12)),)), name="JET_INDEXID", pack=False, align=None),
    "JET_INDEXLIST": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cRecord", SimTypeInt(signed=False, label="UInt32")), ("columnidindexname", SimTypeInt(signed=False, label="UInt32")), ("columnidgrbitIndex", SimTypeInt(signed=False, label="UInt32")), ("columnidcKey", SimTypeInt(signed=False, label="UInt32")), ("columnidcEntry", SimTypeInt(signed=False, label="UInt32")), ("columnidcPage", SimTypeInt(signed=False, label="UInt32")), ("columnidcColumn", SimTypeInt(signed=False, label="UInt32")), ("columnidiColumn", SimTypeInt(signed=False, label="UInt32")), ("columnidcolumnid", SimTypeInt(signed=False, label="UInt32")), ("columnidcoltyp", SimTypeInt(signed=False, label="UInt32")), ("columnidCountry", SimTypeInt(signed=False, label="UInt32")), ("columnidLangid", SimTypeInt(signed=False, label="UInt32")), ("columnidCp", SimTypeInt(signed=False, label="UInt32")), ("columnidCollate", SimTypeInt(signed=False, label="UInt32")), ("columnidgrbitColumn", SimTypeInt(signed=False, label="UInt32")), ("columnidcolumnname", SimTypeInt(signed=False, label="UInt32")), ("columnidLCMapFlags", SimTypeInt(signed=False, label="UInt32")),)), name="JET_INDEXLIST", pack=False, align=None),
    "JET_INDEXRANGE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("grbit", SimTypeInt(signed=False, label="UInt32")),)), name="JET_INDEXRANGE", pack=False, align=None),
    "JET_INDEX_COLUMN": SimStruct(OrderedDict((("columnid", SimTypeInt(signed=False, label="UInt32")), ("relop", SimTypeInt(signed=False, label="JET_RELOP")), ("pv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cb", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")),)), name="JET_INDEX_COLUMN", pack=False, align=None),
    "JET_INDEX_RANGE": SimStruct(OrderedDict((("rgStartColumns", SimTypePointer(SimTypeRef("JET_INDEX_COLUMN", SimStruct), offset=0)), ("cStartColumns", SimTypeInt(signed=False, label="UInt32")), ("rgEndColumns", SimTypePointer(SimTypeRef("JET_INDEX_COLUMN", SimStruct), offset=0)), ("cEndColumns", SimTypeInt(signed=False, label="UInt32")),)), name="JET_INDEX_RANGE", pack=False, align=None),
    "JET_INSTANCE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_INSTANCE_INFO_A": SimStruct(OrderedDict((("hInstanceId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szInstanceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cDatabases", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szDatabaseFileName", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("szDatabaseDisplayName", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("szDatabaseSLVFileName_Obsolete", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)),)), name="JET_INSTANCE_INFO_A", pack=False, align=None),
    "JET_INSTANCE_INFO_W": SimStruct(OrderedDict((("hInstanceId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cDatabases", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szDatabaseFileName", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)), ("szDatabaseDisplayName", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)), ("szDatabaseSLVFileName_Obsolete", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)),)), name="JET_INSTANCE_INFO_W", pack=False, align=None),
    "JET_LGPOS": SimStruct(OrderedDict((("ib", SimTypeShort(signed=False, label="UInt16")), ("isec", SimTypeShort(signed=False, label="UInt16")), ("lGeneration", SimTypeInt(signed=True, label="Int32")),)), name="JET_LGPOS", pack=False, align=None),
    "JET_LOGINFO_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulGenLow", SimTypeInt(signed=False, label="UInt32")), ("ulGenHigh", SimTypeInt(signed=False, label="UInt32")), ("szBaseName", SimTypeArray(SimTypeChar(label="SByte"), 4)),)), name="JET_LOGINFO_A", pack=False, align=None),
    "JET_LOGINFO_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulGenLow", SimTypeInt(signed=False, label="UInt32")), ("ulGenHigh", SimTypeInt(signed=False, label="UInt32")), ("szBaseName", SimTypeArray(SimTypeChar(label="Char"), 4)),)), name="JET_LOGINFO_W", pack=False, align=None),
    "JET_LOGTIME": SimStruct(OrderedDict((("bSeconds", SimTypeChar(label="SByte")), ("bMinutes", SimTypeChar(label="SByte")), ("bHours", SimTypeChar(label="SByte")), ("bDay", SimTypeChar(label="SByte")), ("bMonth", SimTypeChar(label="SByte")), ("bYear", SimTypeChar(label="SByte")), ("Anonymous1", SimUnion({"bFiller1": SimTypeChar(label="SByte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"bFiller2": SimTypeChar(label="SByte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JET_LOGTIME", pack=False, align=None),
    "JET_LS": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_OBJECTINFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("objtyp", SimTypeInt(signed=False, label="UInt32")), ("dtCreate", SimTypeFloat(size=64)), ("dtUpdate", SimTypeFloat(size=64)), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("cRecord", SimTypeInt(signed=False, label="UInt32")), ("cPage", SimTypeInt(signed=False, label="UInt32")),)), name="JET_OBJECTINFO", pack=False, align=None),
    "JET_OBJECTLIST": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cRecord", SimTypeInt(signed=False, label="UInt32")), ("columnidcontainername", SimTypeInt(signed=False, label="UInt32")), ("columnidobjectname", SimTypeInt(signed=False, label="UInt32")), ("columnidobjtyp", SimTypeInt(signed=False, label="UInt32")), ("columniddtCreate", SimTypeInt(signed=False, label="UInt32")), ("columniddtUpdate", SimTypeInt(signed=False, label="UInt32")), ("columnidgrbit", SimTypeInt(signed=False, label="UInt32")), ("columnidflags", SimTypeInt(signed=False, label="UInt32")), ("columnidcRecord", SimTypeInt(signed=False, label="UInt32")), ("columnidcPage", SimTypeInt(signed=False, label="UInt32")),)), name="JET_OBJECTLIST", pack=False, align=None),
    "JET_OPENTEMPORARYTABLE": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("prgcolumndef", SimTypePointer(SimTypeRef("JET_COLUMNDEF", SimStruct), offset=0)), ("ccolumn", SimTypeInt(signed=False, label="UInt32")), ("pidxunicode", SimTypePointer(SimTypeRef("JET_UNICODEINDEX", SimStruct), offset=0)), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("prgcolumnid", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")), ("cbVarSegMac", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="JET_OPENTEMPORARYTABLE", pack=False, align=None),
    "JET_OPENTEMPORARYTABLE2": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("prgcolumndef", SimTypePointer(SimTypeRef("JET_COLUMNDEF", SimStruct), offset=0)), ("ccolumn", SimTypeInt(signed=False, label="UInt32")), ("pidxunicode", SimTypePointer(SimTypeRef("JET_UNICODEINDEX2", SimStruct), offset=0)), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("prgcolumnid", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cbKeyMost", SimTypeInt(signed=False, label="UInt32")), ("cbVarSegMac", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="JET_OPENTEMPORARYTABLE2", pack=False, align=None),
    "JET_OPERATIONCONTEXT": SimStruct(OrderedDict((("ulUserID", SimTypeInt(signed=False, label="UInt32")), ("nOperationID", SimTypeChar(label="Byte")), ("nOperationType", SimTypeChar(label="Byte")), ("nClientType", SimTypeChar(label="Byte")), ("fFlags", SimTypeChar(label="Byte")),)), name="JET_OPERATIONCONTEXT", pack=False, align=None),
    "JET_OSSNAPID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_PFNDURABLECOMMITCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("JET_COMMIT_ID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["instance", "pCommitIdSeen", "grbit"]), offset=0),
    "JET_PFNREALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["pvContext", "pv", "cb"]), offset=0),
    "JET_PFNSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["sesid", "snp", "snt", "pv"]), offset=0),
    "JET_RECORDLIST": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cRecord", SimTypeInt(signed=False, label="UInt32")), ("columnidBookmark", SimTypeInt(signed=False, label="UInt32")),)), name="JET_RECORDLIST", pack=False, align=None),
    "JET_RECPOS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("centriesLT", SimTypeInt(signed=False, label="UInt32")), ("centriesInRange", SimTypeInt(signed=False, label="UInt32")), ("centriesTotal", SimTypeInt(signed=False, label="UInt32")),)), name="JET_RECPOS", pack=False, align=None),
    "JET_RECPOS2": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("centriesLTDeprecated", SimTypeInt(signed=False, label="UInt32")), ("centriesInRangeDeprecated", SimTypeInt(signed=False, label="UInt32")), ("centriesTotalDeprecated", SimTypeInt(signed=False, label="UInt32")), ("centriesLT", SimTypeLongLong(signed=False, label="UInt64")), ("centriesTotal", SimTypeLongLong(signed=False, label="UInt64")),)), name="JET_RECPOS2", pack=False, align=None),
    "JET_RECSIZE": SimStruct(OrderedDict((("cbData", SimTypeLongLong(signed=False, label="UInt64")), ("cbLongValueData", SimTypeLongLong(signed=False, label="UInt64")), ("cbOverhead", SimTypeLongLong(signed=False, label="UInt64")), ("cbLongValueOverhead", SimTypeLongLong(signed=False, label="UInt64")), ("cNonTaggedColumns", SimTypeLongLong(signed=False, label="UInt64")), ("cTaggedColumns", SimTypeLongLong(signed=False, label="UInt64")), ("cLongValues", SimTypeLongLong(signed=False, label="UInt64")), ("cMultiValues", SimTypeLongLong(signed=False, label="UInt64")),)), name="JET_RECSIZE", pack=False, align=None),
    "JET_RECSIZE2": SimStruct(OrderedDict((("cbData", SimTypeLongLong(signed=False, label="UInt64")), ("cbLongValueData", SimTypeLongLong(signed=False, label="UInt64")), ("cbOverhead", SimTypeLongLong(signed=False, label="UInt64")), ("cbLongValueOverhead", SimTypeLongLong(signed=False, label="UInt64")), ("cNonTaggedColumns", SimTypeLongLong(signed=False, label="UInt64")), ("cTaggedColumns", SimTypeLongLong(signed=False, label="UInt64")), ("cLongValues", SimTypeLongLong(signed=False, label="UInt64")), ("cMultiValues", SimTypeLongLong(signed=False, label="UInt64")), ("cCompressedColumns", SimTypeLongLong(signed=False, label="UInt64")), ("cbDataCompressed", SimTypeLongLong(signed=False, label="UInt64")), ("cbLongValueDataCompressed", SimTypeLongLong(signed=False, label="UInt64")),)), name="JET_RECSIZE2", pack=False, align=None),
    "JET_RELOP": SimTypeInt(signed=False, label="JET_RELOP"),
    "JET_RETINFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("ibLongValue", SimTypeInt(signed=False, label="UInt32")), ("itagSequence", SimTypeInt(signed=False, label="UInt32")), ("columnidNextTagged", SimTypeInt(signed=False, label="UInt32")),)), name="JET_RETINFO", pack=False, align=None),
    "JET_RETRIEVECOLUMN": SimStruct(OrderedDict((("columnid", SimTypeInt(signed=False, label="UInt32")), ("pvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("cbActual", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ibLongValue", SimTypeInt(signed=False, label="UInt32")), ("itagSequence", SimTypeInt(signed=False, label="UInt32")), ("columnidNextTagged", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")),)), name="JET_RETRIEVECOLUMN", pack=False, align=None),
    "JET_RSTINFO_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("rgrstmap", SimTypePointer(SimTypeRef("JET_RSTMAP_A", SimStruct), offset=0)), ("crstmap", SimTypeInt(signed=True, label="Int32")), ("lgposStop", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeStop", SimTypeRef("JET_LOGTIME", SimStruct)), ("pfnStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["sesid", "snp", "snt", "pv"]), offset=0)),)), name="JET_RSTINFO_A", pack=False, align=None),
    "JET_RSTINFO_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("rgrstmap", SimTypePointer(SimTypeRef("JET_RSTMAP_W", SimStruct), offset=0)), ("crstmap", SimTypeInt(signed=True, label="Int32")), ("lgposStop", SimTypeRef("JET_LGPOS", SimStruct)), ("logtimeStop", SimTypeRef("JET_LOGTIME", SimStruct)), ("pfnStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["sesid", "snp", "snt", "pv"]), offset=0)),)), name="JET_RSTINFO_W", pack=False, align=None),
    "JET_RSTMAP_A": SimStruct(OrderedDict((("szDatabaseName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szNewDatabaseName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="JET_RSTMAP_A", pack=False, align=None),
    "JET_RSTMAP_W": SimStruct(OrderedDict((("szDatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szNewDatabaseName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="JET_RSTMAP_W", pack=False, align=None),
    "JET_SESID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_SETCOLUMN": SimStruct(OrderedDict((("columnid", SimTypeInt(signed=False, label="UInt32")), ("pvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ibLongValue", SimTypeInt(signed=False, label="UInt32")), ("itagSequence", SimTypeInt(signed=False, label="UInt32")), ("err", SimTypeInt(signed=True, label="Int32")),)), name="JET_SETCOLUMN", pack=False, align=None),
    "JET_SETINFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("ibLongValue", SimTypeInt(signed=False, label="UInt32")), ("itagSequence", SimTypeInt(signed=False, label="UInt32")),)), name="JET_SETINFO", pack=False, align=None),
    "JET_SETSYSPARAM_A": SimStruct(OrderedDict((("paramid", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("sz", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("err", SimTypeInt(signed=True, label="Int32")),)), name="JET_SETSYSPARAM_A", pack=False, align=None),
    "JET_SETSYSPARAM_W": SimStruct(OrderedDict((("paramid", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("sz", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("err", SimTypeInt(signed=True, label="Int32")),)), name="JET_SETSYSPARAM_W", pack=False, align=None),
    "JET_SIGNATURE": SimStruct(OrderedDict((("ulRandom", SimTypeInt(signed=False, label="UInt32")), ("logtimeCreate", SimTypeRef("JET_LOGTIME", SimStruct)), ("szComputerName", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="JET_SIGNATURE", pack=False, align=None),
    "JET_SNPROG": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cunitDone", SimTypeInt(signed=False, label="UInt32")), ("cunitTotal", SimTypeInt(signed=False, label="UInt32")),)), name="JET_SNPROG", pack=False, align=None),
    "JET_SPACEHINTS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("ulInitialDensity", SimTypeInt(signed=False, label="UInt32")), ("cbInitial", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("ulMaintDensity", SimTypeInt(signed=False, label="UInt32")), ("ulGrowth", SimTypeInt(signed=False, label="UInt32")), ("cbMinExtent", SimTypeInt(signed=False, label="UInt32")), ("cbMaxExtent", SimTypeInt(signed=False, label="UInt32")),)), name="JET_SPACEHINTS", pack=False, align=None),
    "JET_TABLECREATE2_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_A", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE_A", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("szCallback", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbtyp", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE2_A", pack=False, align=None),
    "JET_TABLECREATE2_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_W", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE_W", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("szCallback", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbtyp", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE2_W", pack=False, align=None),
    "JET_TABLECREATE3_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_A", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE2_A", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("szCallback", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbtyp", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("pSeqSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("pLVSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("cbSeparateLV", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE3_A", pack=False, align=None),
    "JET_TABLECREATE3_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_W", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE2_W", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("szCallback", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbtyp", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("pSeqSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("pLVSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("cbSeparateLV", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE3_W", pack=False, align=None),
    "JET_TABLECREATE4_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_A", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE3_A", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("szCallback", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbtyp", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("pSeqSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("pLVSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("cbSeparateLV", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE4_A", pack=False, align=None),
    "JET_TABLECREATE4_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_W", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE3_W", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("szCallback", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbtyp", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("pSeqSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("pLVSpacehints", SimTypePointer(SimTypeRef("JET_SPACEHINTS", SimStruct), offset=0)), ("cbSeparateLV", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE4_W", pack=False, align=None),
    "JET_TABLECREATE_A": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_A", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE_A", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE_A", pack=False, align=None),
    "JET_TABLECREATE_W": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("szTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szTemplateTableName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulPages", SimTypeInt(signed=False, label="UInt32")), ("ulDensity", SimTypeInt(signed=False, label="UInt32")), ("rgcolumncreate", SimTypePointer(SimTypeRef("JET_COLUMNCREATE_W", SimStruct), offset=0)), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("rgindexcreate", SimTypePointer(SimTypeRef("JET_INDEXCREATE_W", SimStruct), offset=0)), ("cIndexes", SimTypeInt(signed=False, label="UInt32")), ("grbit", SimTypeInt(signed=False, label="UInt32")), ("tableid", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCreated", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TABLECREATE_W", pack=False, align=None),
    "JET_TABLEID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "JET_THREADSTATS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cPageReferenced", SimTypeInt(signed=False, label="UInt32")), ("cPageRead", SimTypeInt(signed=False, label="UInt32")), ("cPagePreread", SimTypeInt(signed=False, label="UInt32")), ("cPageDirtied", SimTypeInt(signed=False, label="UInt32")), ("cPageRedirtied", SimTypeInt(signed=False, label="UInt32")), ("cLogRecord", SimTypeInt(signed=False, label="UInt32")), ("cbLogRecord", SimTypeInt(signed=False, label="UInt32")),)), name="JET_THREADSTATS", pack=False, align=None),
    "JET_THREADSTATS2": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cPageReferenced", SimTypeInt(signed=False, label="UInt32")), ("cPageRead", SimTypeInt(signed=False, label="UInt32")), ("cPagePreread", SimTypeInt(signed=False, label="UInt32")), ("cPageDirtied", SimTypeInt(signed=False, label="UInt32")), ("cPageRedirtied", SimTypeInt(signed=False, label="UInt32")), ("cLogRecord", SimTypeInt(signed=False, label="UInt32")), ("cbLogRecord", SimTypeInt(signed=False, label="UInt32")), ("cusecPageCacheMiss", SimTypeLongLong(signed=False, label="UInt64")), ("cPageCacheMiss", SimTypeInt(signed=False, label="UInt32")),)), name="JET_THREADSTATS2", pack=False, align=None),
    "JET_TUPLELIMITS": SimStruct(OrderedDict((("chLengthMin", SimTypeInt(signed=False, label="UInt32")), ("chLengthMax", SimTypeInt(signed=False, label="UInt32")), ("chToIndexMax", SimTypeInt(signed=False, label="UInt32")), ("cchIncrement", SimTypeInt(signed=False, label="UInt32")), ("ichStart", SimTypeInt(signed=False, label="UInt32")),)), name="JET_TUPLELIMITS", pack=False, align=None),
    "JET_UNICODEINDEX": SimStruct(OrderedDict((("lcid", SimTypeInt(signed=False, label="UInt32")), ("dwMapFlags", SimTypeInt(signed=False, label="UInt32")),)), name="JET_UNICODEINDEX", pack=False, align=None),
    "JET_UNICODEINDEX2": SimStruct(OrderedDict((("szLocaleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMapFlags", SimTypeInt(signed=False, label="UInt32")),)), name="JET_UNICODEINDEX2", pack=False, align=None),
    "JET_USERDEFINEDDEFAULT_A": SimStruct(OrderedDict((("szCallback", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pbUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbUserData", SimTypeInt(signed=False, label="UInt32")), ("szDependantColumns", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="JET_USERDEFINEDDEFAULT_A", pack=False, align=None),
    "JET_USERDEFINEDDEFAULT_W": SimStruct(OrderedDict((("szCallback", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pbUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbUserData", SimTypeInt(signed=False, label="UInt32")), ("szDependantColumns", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="JET_USERDEFINEDDEFAULT_W", pack=False, align=None),
    "JIT_DEBUG_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwProcessorArchitecture", SimTypeInt(signed=False, label="UInt32")), ("dwThreadID", SimTypeInt(signed=False, label="UInt32")), ("dwReserved0", SimTypeInt(signed=False, label="UInt32")), ("lpExceptionAddress", SimTypeLongLong(signed=False, label="UInt64")), ("lpExceptionRecord", SimTypeLongLong(signed=False, label="UInt64")), ("lpContextRecord", SimTypeLongLong(signed=False, label="UInt64")),)), name="JIT_DEBUG_INFO", pack=False, align=None),
    "JOBOBJECTINFOCLASS": SimTypeInt(signed=False, label="JOBOBJECTINFOCLASS"),
    "JOBOBJECT_ASSOCIATE_COMPLETION_PORT": SimStruct(OrderedDict((("CompletionKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CompletionPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="JOBOBJECT_ASSOCIATE_COMPLETION_PORT", pack=False, align=None),
    "JOBOBJECT_BASIC_ACCOUNTING_INFORMATION": SimStruct(OrderedDict((("TotalUserTime", SimTypeLongLong(signed=True, label="Int64")), ("TotalKernelTime", SimTypeLongLong(signed=True, label="Int64")), ("ThisPeriodTotalUserTime", SimTypeLongLong(signed=True, label="Int64")), ("ThisPeriodTotalKernelTime", SimTypeLongLong(signed=True, label="Int64")), ("TotalPageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("TotalProcesses", SimTypeInt(signed=False, label="UInt32")), ("ActiveProcesses", SimTypeInt(signed=False, label="UInt32")), ("TotalTerminatedProcesses", SimTypeInt(signed=False, label="UInt32")),)), name="JOBOBJECT_BASIC_ACCOUNTING_INFORMATION", pack=False, align=None),
    "JOBOBJECT_BASIC_AND_IO_ACCOUNTING_INFORMATION": SimStruct(OrderedDict((("BasicInfo", SimTypeRef("JOBOBJECT_BASIC_ACCOUNTING_INFORMATION", SimStruct)), ("IoInfo", SimTypeRef("IO_COUNTERS", SimStruct)),)), name="JOBOBJECT_BASIC_AND_IO_ACCOUNTING_INFORMATION", pack=False, align=None),
    "JOBOBJECT_BASIC_LIMIT_INFORMATION": SimStruct(OrderedDict((("PerProcessUserTimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("PerJobUserTimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("LimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")), ("MinimumWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaximumWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ActiveProcessLimit", SimTypeInt(signed=False, label="UInt32")), ("Affinity", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("SchedulingClass", SimTypeInt(signed=False, label="UInt32")),)), name="JOBOBJECT_BASIC_LIMIT_INFORMATION", pack=False, align=None),
    "JOBOBJECT_BASIC_PROCESS_ID_LIST": SimStruct(OrderedDict((("NumberOfAssignedProcesses", SimTypeInt(signed=False, label="UInt32")), ("NumberOfProcessIdsInList", SimTypeInt(signed=False, label="UInt32")), ("ProcessIdList", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)),)), name="JOBOBJECT_BASIC_PROCESS_ID_LIST", pack=False, align=None),
    "JOBOBJECT_BASIC_UI_RESTRICTIONS": SimStruct(OrderedDict((("UIRestrictionsClass", SimTypeInt(signed=False, label="JOB_OBJECT_UILIMIT")),)), name="JOBOBJECT_BASIC_UI_RESTRICTIONS", pack=False, align=None),
    "JOBOBJECT_CPU_RATE_CONTROL_INFORMATION": SimStruct(OrderedDict((("ControlFlags", SimTypeInt(signed=False, label="JOB_OBJECT_CPU_RATE_CONTROL")), ("Anonymous", SimUnion({"CpuRate": SimTypeInt(signed=False, label="UInt32"), "Weight": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("MinRate", SimTypeShort(signed=False, label="UInt16")), ("MaxRate", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="JOBOBJECT_CPU_RATE_CONTROL_INFORMATION", pack=False, align=None),
    "JOBOBJECT_END_OF_JOB_TIME_INFORMATION": SimStruct(OrderedDict((("EndOfJobTimeAction", SimTypeInt(signed=False, label="JOB_OBJECT_TERMINATE_AT_END_ACTION")),)), name="JOBOBJECT_END_OF_JOB_TIME_INFORMATION", pack=False, align=None),
    "JOBOBJECT_EXTENDED_LIMIT_INFORMATION": SimStruct(OrderedDict((("BasicLimitInformation", SimTypeRef("JOBOBJECT_BASIC_LIMIT_INFORMATION", SimStruct)), ("IoInfo", SimTypeRef("IO_COUNTERS", SimStruct)), ("ProcessMemoryLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("JobMemoryLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakProcessMemoryUsed", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakJobMemoryUsed", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="JOBOBJECT_EXTENDED_LIMIT_INFORMATION", pack=False, align=None),
    "JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS": SimTypeInt(signed=False, label="JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS"),
    "JOBOBJECT_IO_ATTRIBUTION_INFORMATION": SimStruct(OrderedDict((("ControlFlags", SimTypeInt(signed=False, label="UInt32")), ("ReadStats", SimTypeRef("JOBOBJECT_IO_ATTRIBUTION_STATS", SimStruct)), ("WriteStats", SimTypeRef("JOBOBJECT_IO_ATTRIBUTION_STATS", SimStruct)),)), name="JOBOBJECT_IO_ATTRIBUTION_INFORMATION", pack=False, align=None),
    "JOBOBJECT_IO_ATTRIBUTION_STATS": SimStruct(OrderedDict((("IoCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TotalNonOverlappedQueueTime", SimTypeLongLong(signed=False, label="UInt64")), ("TotalNonOverlappedServiceTime", SimTypeLongLong(signed=False, label="UInt64")), ("TotalSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="JOBOBJECT_IO_ATTRIBUTION_STATS", pack=False, align=None),
    "JOBOBJECT_IO_RATE_CONTROL_INFORMATION": SimStruct(OrderedDict((("MaxIops", SimTypeLongLong(signed=True, label="Int64")), ("MaxBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("ReservationIops", SimTypeLongLong(signed=True, label="Int64")), ("VolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BaseIoSize", SimTypeInt(signed=False, label="UInt32")), ("ControlFlags", SimTypeInt(signed=False, label="UInt32")),)), name="JOBOBJECT_IO_RATE_CONTROL_INFORMATION", pack=False, align=None),
    "JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V1": SimStruct(OrderedDict((("MaxIops", SimTypeLongLong(signed=True, label="Int64")), ("MaxBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("ReservationIops", SimTypeLongLong(signed=True, label="Int64")), ("VolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BaseIoSize", SimTypeInt(signed=False, label="UInt32")), ("ControlFlags", SimTypeInt(signed=False, label="JOB_OBJECT_IO_RATE_CONTROL_FLAGS")), ("VolumeNameLength", SimTypeShort(signed=False, label="UInt16")),)), name="JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V1", pack=False, align=None),
    "JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V2": SimStruct(OrderedDict((("MaxIops", SimTypeLongLong(signed=True, label="Int64")), ("MaxBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("ReservationIops", SimTypeLongLong(signed=True, label="Int64")), ("VolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BaseIoSize", SimTypeInt(signed=False, label="UInt32")), ("ControlFlags", SimTypeInt(signed=False, label="JOB_OBJECT_IO_RATE_CONTROL_FLAGS")), ("VolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("CriticalReservationIops", SimTypeLongLong(signed=True, label="Int64")), ("ReservationBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("CriticalReservationBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("MaxTimePercent", SimTypeLongLong(signed=True, label="Int64")), ("ReservationTimePercent", SimTypeLongLong(signed=True, label="Int64")), ("CriticalReservationTimePercent", SimTypeLongLong(signed=True, label="Int64")),)), name="JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V2", pack=False, align=None),
    "JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V3": SimStruct(OrderedDict((("MaxIops", SimTypeLongLong(signed=True, label="Int64")), ("MaxBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("ReservationIops", SimTypeLongLong(signed=True, label="Int64")), ("VolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BaseIoSize", SimTypeInt(signed=False, label="UInt32")), ("ControlFlags", SimTypeInt(signed=False, label="JOB_OBJECT_IO_RATE_CONTROL_FLAGS")), ("VolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("CriticalReservationIops", SimTypeLongLong(signed=True, label="Int64")), ("ReservationBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("CriticalReservationBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("MaxTimePercent", SimTypeLongLong(signed=True, label="Int64")), ("ReservationTimePercent", SimTypeLongLong(signed=True, label="Int64")), ("CriticalReservationTimePercent", SimTypeLongLong(signed=True, label="Int64")), ("SoftMaxIops", SimTypeLongLong(signed=True, label="Int64")), ("SoftMaxBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("SoftMaxTimePercent", SimTypeLongLong(signed=True, label="Int64")), ("LimitExcessNotifyIops", SimTypeLongLong(signed=True, label="Int64")), ("LimitExcessNotifyBandwidth", SimTypeLongLong(signed=True, label="Int64")), ("LimitExcessNotifyTimePercent", SimTypeLongLong(signed=True, label="Int64")),)), name="JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V3", pack=False, align=None),
    "JOBOBJECT_JOBSET_INFORMATION": SimStruct(OrderedDict((("MemberLevel", SimTypeInt(signed=False, label="UInt32")),)), name="JOBOBJECT_JOBSET_INFORMATION", pack=False, align=None),
    "JOBOBJECT_LIMIT_VIOLATION_INFORMATION": SimStruct(OrderedDict((("LimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")), ("ViolationLimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")), ("IoReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("IoReadBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("PerJobUserTime", SimTypeLongLong(signed=True, label="Int64")), ("PerJobUserTimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("JobMemory", SimTypeLongLong(signed=False, label="UInt64")), ("JobMemoryLimit", SimTypeLongLong(signed=False, label="UInt64")), ("RateControlTolerance", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("RateControlToleranceLimit", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")),)), name="JOBOBJECT_LIMIT_VIOLATION_INFORMATION", pack=False, align=None),
    "JOBOBJECT_LIMIT_VIOLATION_INFORMATION_2": SimStruct(OrderedDict((("LimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")), ("ViolationLimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")), ("IoReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("IoReadBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("PerJobUserTime", SimTypeLongLong(signed=True, label="Int64")), ("PerJobUserTimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("JobMemory", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous1", SimUnion({"JobHighMemoryLimit": SimTypeLongLong(signed=False, label="UInt64"), "JobMemoryLimit": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"RateControlTolerance": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE"), "CpuRateControlTolerance": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"RateControlToleranceLimit": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE"), "CpuRateControlToleranceLimit": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")}, name="<anon>", label="None")), ("JobLowMemoryLimit", SimTypeLongLong(signed=False, label="UInt64")), ("IoRateControlTolerance", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("IoRateControlToleranceLimit", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("NetRateControlTolerance", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("NetRateControlToleranceLimit", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")),)), name="JOBOBJECT_LIMIT_VIOLATION_INFORMATION_2", pack=False, align=None),
    "JOBOBJECT_NET_RATE_CONTROL_INFORMATION": SimStruct(OrderedDict((("MaxBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("ControlFlags", SimTypeInt(signed=False, label="JOB_OBJECT_NET_RATE_CONTROL_FLAGS")), ("DscpTag", SimTypeChar(label="Byte")),)), name="JOBOBJECT_NET_RATE_CONTROL_INFORMATION", pack=False, align=None),
    "JOBOBJECT_NOTIFICATION_LIMIT_INFORMATION": SimStruct(OrderedDict((("IoReadBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("PerJobUserTimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("JobMemoryLimit", SimTypeLongLong(signed=False, label="UInt64")), ("RateControlTolerance", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("RateControlToleranceInterval", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL")), ("LimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")),)), name="JOBOBJECT_NOTIFICATION_LIMIT_INFORMATION", pack=False, align=None),
    "JOBOBJECT_NOTIFICATION_LIMIT_INFORMATION_2": SimStruct(OrderedDict((("IoReadBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteBytesLimit", SimTypeLongLong(signed=False, label="UInt64")), ("PerJobUserTimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("Anonymous1", SimUnion({"JobHighMemoryLimit": SimTypeLongLong(signed=False, label="UInt64"), "JobMemoryLimit": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"RateControlTolerance": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE"), "CpuRateControlTolerance": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"RateControlToleranceInterval": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL"), "CpuRateControlToleranceInterval": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL")}, name="<anon>", label="None")), ("LimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT")), ("IoRateControlTolerance", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("JobLowMemoryLimit", SimTypeLongLong(signed=False, label="UInt64")), ("IoRateControlToleranceInterval", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL")), ("NetRateControlTolerance", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE")), ("NetRateControlToleranceInterval", SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL")),)), name="JOBOBJECT_NOTIFICATION_LIMIT_INFORMATION_2", pack=False, align=None),
    "JOBOBJECT_RATE_CONTROL_TOLERANCE": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE"),
    "JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL": SimTypeInt(signed=False, label="JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL"),
    "JOBOBJECT_SECURITY_LIMIT_INFORMATION": SimStruct(OrderedDict((("SecurityLimitFlags", SimTypeInt(signed=False, label="JOB_OBJECT_SECURITY")), ("JobToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SidsToDisable", SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0)), ("PrivilegesToDelete", SimTypePointer(SimTypeRef("TOKEN_PRIVILEGES", SimStruct), offset=0)), ("RestrictedSids", SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0)),)), name="JOBOBJECT_SECURITY_LIMIT_INFORMATION", pack=False, align=None),
    "JOB_INFO_1A": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDocument", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("PagesPrinted", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="JOB_INFO_1A", pack=False, align=None),
    "JOB_INFO_1W": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDocument", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pStatus", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("PagesPrinted", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="JOB_INFO_1W", pack=False, align=None),
    "JOB_INFO_2A": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDocument", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pNotifyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPrintProcessor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("UntilTime", SimTypeInt(signed=False, label="UInt32")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("Time", SimTypeInt(signed=False, label="UInt32")), ("PagesPrinted", SimTypeInt(signed=False, label="UInt32")),)), name="JOB_INFO_2A", pack=False, align=None),
    "JOB_INFO_2W": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDocument", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pNotifyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrintProcessor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)), ("pStatus", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("UntilTime", SimTypeInt(signed=False, label="UInt32")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("Time", SimTypeInt(signed=False, label="UInt32")), ("PagesPrinted", SimTypeInt(signed=False, label="UInt32")),)), name="JOB_INFO_2W", pack=False, align=None),
    "JOB_INFO_3": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("NextJobId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="JOB_INFO_3", pack=False, align=None),
    "JOB_INFO_4A": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDocument", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pNotifyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPrintProcessor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("UntilTime", SimTypeInt(signed=False, label="UInt32")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("Time", SimTypeInt(signed=False, label="UInt32")), ("PagesPrinted", SimTypeInt(signed=False, label="UInt32")), ("SizeHigh", SimTypeInt(signed=True, label="Int32")),)), name="JOB_INFO_4A", pack=False, align=None),
    "JOB_INFO_4W": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDocument", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pNotifyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrintProcessor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)), ("pStatus", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("UntilTime", SimTypeInt(signed=False, label="UInt32")), ("TotalPages", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("Time", SimTypeInt(signed=False, label="UInt32")), ("PagesPrinted", SimTypeInt(signed=False, label="UInt32")), ("SizeHigh", SimTypeInt(signed=True, label="Int32")),)), name="JOB_INFO_4W", pack=False, align=None),
    "JOB_OBJECT_CPU_RATE_CONTROL": SimTypeInt(signed=False, label="JOB_OBJECT_CPU_RATE_CONTROL"),
    "JOB_OBJECT_IO_RATE_CONTROL_FLAGS": SimTypeInt(signed=False, label="JOB_OBJECT_IO_RATE_CONTROL_FLAGS"),
    "JOB_OBJECT_LIMIT": SimTypeInt(signed=False, label="JOB_OBJECT_LIMIT"),
    "JOB_OBJECT_NET_RATE_CONTROL_FLAGS": SimTypeInt(signed=False, label="JOB_OBJECT_NET_RATE_CONTROL_FLAGS"),
    "JOB_OBJECT_SECURITY": SimTypeInt(signed=False, label="JOB_OBJECT_SECURITY"),
    "JOB_OBJECT_TERMINATE_AT_END_ACTION": SimTypeInt(signed=False, label="JOB_OBJECT_TERMINATE_AT_END_ACTION"),
    "JOB_OBJECT_UILIMIT": SimTypeInt(signed=False, label="JOB_OBJECT_UILIMIT"),
    "JOB_SET_ARRAY": SimStruct(OrderedDict((("JobHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("MemberLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="JOB_SET_ARRAY", pack=False, align=None),
    "JOYCALIBRATE": SimStruct(OrderedDict((("wXbase", SimTypeInt(signed=False, label="UInt32")), ("wXdelta", SimTypeInt(signed=False, label="UInt32")), ("wYbase", SimTypeInt(signed=False, label="UInt32")), ("wYdelta", SimTypeInt(signed=False, label="UInt32")), ("wZbase", SimTypeInt(signed=False, label="UInt32")), ("wZdelta", SimTypeInt(signed=False, label="UInt32")),)), name="JOYCALIBRATE", pack=False, align=None),
    "JOYCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("wXmin", SimTypeInt(signed=False, label="UInt32")), ("wXmax", SimTypeInt(signed=False, label="UInt32")), ("wYmin", SimTypeInt(signed=False, label="UInt32")), ("wYmax", SimTypeInt(signed=False, label="UInt32")), ("wZmin", SimTypeInt(signed=False, label="UInt32")), ("wZmax", SimTypeInt(signed=False, label="UInt32")), ("wNumButtons", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMin", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMax", SimTypeInt(signed=False, label="UInt32")), ("wRmin", SimTypeInt(signed=False, label="UInt32")), ("wRmax", SimTypeInt(signed=False, label="UInt32")), ("wUmin", SimTypeInt(signed=False, label="UInt32")), ("wUmax", SimTypeInt(signed=False, label="UInt32")), ("wVmin", SimTypeInt(signed=False, label="UInt32")), ("wVmax", SimTypeInt(signed=False, label="UInt32")), ("wCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxAxes", SimTypeInt(signed=False, label="UInt32")), ("wNumAxes", SimTypeInt(signed=False, label="UInt32")), ("wMaxButtons", SimTypeInt(signed=False, label="UInt32")), ("szRegKey", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szOEMVxD", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="JOYCAPS2A", pack=False, align=None),
    "JOYCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wXmin", SimTypeInt(signed=False, label="UInt32")), ("wXmax", SimTypeInt(signed=False, label="UInt32")), ("wYmin", SimTypeInt(signed=False, label="UInt32")), ("wYmax", SimTypeInt(signed=False, label="UInt32")), ("wZmin", SimTypeInt(signed=False, label="UInt32")), ("wZmax", SimTypeInt(signed=False, label="UInt32")), ("wNumButtons", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMin", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMax", SimTypeInt(signed=False, label="UInt32")), ("wRmin", SimTypeInt(signed=False, label="UInt32")), ("wRmax", SimTypeInt(signed=False, label="UInt32")), ("wUmin", SimTypeInt(signed=False, label="UInt32")), ("wUmax", SimTypeInt(signed=False, label="UInt32")), ("wVmin", SimTypeInt(signed=False, label="UInt32")), ("wVmax", SimTypeInt(signed=False, label="UInt32")), ("wCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxAxes", SimTypeInt(signed=False, label="UInt32")), ("wNumAxes", SimTypeInt(signed=False, label="UInt32")), ("wMaxButtons", SimTypeInt(signed=False, label="UInt32")), ("szRegKey", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szOEMVxD", SimTypeArray(SimTypeChar(label="Char"), 260)), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="JOYCAPS2W", pack=False, align=None),
    "JOYCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("wXmin", SimTypeInt(signed=False, label="UInt32")), ("wXmax", SimTypeInt(signed=False, label="UInt32")), ("wYmin", SimTypeInt(signed=False, label="UInt32")), ("wYmax", SimTypeInt(signed=False, label="UInt32")), ("wZmin", SimTypeInt(signed=False, label="UInt32")), ("wZmax", SimTypeInt(signed=False, label="UInt32")), ("wNumButtons", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMin", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMax", SimTypeInt(signed=False, label="UInt32")), ("wRmin", SimTypeInt(signed=False, label="UInt32")), ("wRmax", SimTypeInt(signed=False, label="UInt32")), ("wUmin", SimTypeInt(signed=False, label="UInt32")), ("wUmax", SimTypeInt(signed=False, label="UInt32")), ("wVmin", SimTypeInt(signed=False, label="UInt32")), ("wVmax", SimTypeInt(signed=False, label="UInt32")), ("wCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxAxes", SimTypeInt(signed=False, label="UInt32")), ("wNumAxes", SimTypeInt(signed=False, label="UInt32")), ("wMaxButtons", SimTypeInt(signed=False, label="UInt32")), ("szRegKey", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szOEMVxD", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="JOYCAPSA", pack=False, align=None),
    "JOYCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wXmin", SimTypeInt(signed=False, label="UInt32")), ("wXmax", SimTypeInt(signed=False, label="UInt32")), ("wYmin", SimTypeInt(signed=False, label="UInt32")), ("wYmax", SimTypeInt(signed=False, label="UInt32")), ("wZmin", SimTypeInt(signed=False, label="UInt32")), ("wZmax", SimTypeInt(signed=False, label="UInt32")), ("wNumButtons", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMin", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMax", SimTypeInt(signed=False, label="UInt32")), ("wRmin", SimTypeInt(signed=False, label="UInt32")), ("wRmax", SimTypeInt(signed=False, label="UInt32")), ("wUmin", SimTypeInt(signed=False, label="UInt32")), ("wUmax", SimTypeInt(signed=False, label="UInt32")), ("wVmin", SimTypeInt(signed=False, label="UInt32")), ("wVmax", SimTypeInt(signed=False, label="UInt32")), ("wCaps", SimTypeInt(signed=False, label="UInt32")), ("wMaxAxes", SimTypeInt(signed=False, label="UInt32")), ("wNumAxes", SimTypeInt(signed=False, label="UInt32")), ("wMaxButtons", SimTypeInt(signed=False, label="UInt32")), ("szRegKey", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szOEMVxD", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="JOYCAPSW", pack=False, align=None),
    "JOYINFO": SimStruct(OrderedDict((("wXpos", SimTypeInt(signed=False, label="UInt32")), ("wYpos", SimTypeInt(signed=False, label="UInt32")), ("wZpos", SimTypeInt(signed=False, label="UInt32")), ("wButtons", SimTypeInt(signed=False, label="UInt32")),)), name="JOYINFO", pack=False, align=None),
    "JOYINFOEX": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwXpos", SimTypeInt(signed=False, label="UInt32")), ("dwYpos", SimTypeInt(signed=False, label="UInt32")), ("dwZpos", SimTypeInt(signed=False, label="UInt32")), ("dwRpos", SimTypeInt(signed=False, label="UInt32")), ("dwUpos", SimTypeInt(signed=False, label="UInt32")), ("dwVpos", SimTypeInt(signed=False, label="UInt32")), ("dwButtons", SimTypeInt(signed=False, label="UInt32")), ("dwButtonNumber", SimTypeInt(signed=False, label="UInt32")), ("dwPOV", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="JOYINFOEX", pack=False, align=None),
    "JOYPOS": SimStruct(OrderedDict((("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwZ", SimTypeInt(signed=False, label="UInt32")), ("dwR", SimTypeInt(signed=False, label="UInt32")), ("dwU", SimTypeInt(signed=False, label="UInt32")), ("dwV", SimTypeInt(signed=False, label="UInt32")),)), name="JOYPOS", pack=False, align=None),
    "JOYRANGE": SimStruct(OrderedDict((("jpMin", SimTypeRef("JOYPOS", SimStruct)), ("jpMax", SimTypeRef("JOYPOS", SimStruct)), ("jpCenter", SimTypeRef("JOYPOS", SimStruct)),)), name="JOYRANGE", pack=False, align=None),
    "JOYREGHWCONFIG": SimStruct(OrderedDict((("hws", SimTypeRef("JOYREGHWSETTINGS", SimStruct)), ("dwUsageSettings", SimTypeInt(signed=False, label="UInt32")), ("hwv", SimTypeRef("JOYREGHWVALUES", SimStruct)), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="JOYREGHWCONFIG", pack=False, align=None),
    "JOYREGHWSETTINGS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumButtons", SimTypeInt(signed=False, label="UInt32")),)), name="JOYREGHWSETTINGS", pack=False, align=None),
    "JOYREGHWVALUES": SimStruct(OrderedDict((("jrvHardware", SimTypeRef("JOYRANGE", SimStruct)), ("dwPOVValues", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("dwCalFlags", SimTypeInt(signed=False, label="UInt32")),)), name="JOYREGHWVALUES", pack=False, align=None),
    "JOYREGUSERVALUES": SimStruct(OrderedDict((("dwTimeOut", SimTypeInt(signed=False, label="UInt32")), ("jrvRanges", SimTypeRef("JOYRANGE", SimStruct)), ("jpDeadZone", SimTypeRef("JOYPOS", SimStruct)),)), name="JOYREGUSERVALUES", pack=False, align=None),
    "JPEGINFOHEADER": SimStruct(OrderedDict((("JPEGSize", SimTypeInt(signed=False, label="UInt32")), ("JPEGProcess", SimTypeInt(signed=False, label="UInt32")), ("JPEGColorSpaceID", SimTypeInt(signed=False, label="UInt32")), ("JPEGBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("JPEGHSubSampling", SimTypeInt(signed=False, label="UInt32")), ("JPEGVSubSampling", SimTypeInt(signed=False, label="UInt32")),)), name="JPEGINFOHEADER", pack=False, align=None),
    "JS_NATIVE_FRAME": SimStruct(OrderedDict((("InstructionOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ReturnOffset", SimTypeLongLong(signed=False, label="UInt64")), ("FrameOffset", SimTypeLongLong(signed=False, label="UInt64")), ("StackOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="JS_NATIVE_FRAME", pack=False, align=None),
    "JS_PROPERTY_ATTRIBUTES": SimTypeInt(signed=False, label="JS_PROPERTY_ATTRIBUTES"),
    "JS_PROPERTY_MEMBERS": SimTypeInt(signed=False, label="JS_PROPERTY_MEMBERS"),
    "JabColorF": SimStruct(OrderedDict((("J", SimTypeFloat(size=32)), ("a", SimTypeFloat(size=32)), ("b", SimTypeFloat(size=32)),)), name="JabColorF", pack=False, align=None),
    "JsBackgroundWorkItemCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["callbackState"]), offset=0),
    "JsBeforeCollectCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["callbackState"]), offset=0),
    "JsDebugPropertyInfo": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("type", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("value", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fullName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("attr", SimTypeInt(signed=False, label="JS_PROPERTY_ATTRIBUTES")),)), name="JsDebugPropertyInfo", pack=False, align=None),
    "JsDebugReadMemoryFlags": SimTypeInt(signed=False, label="JsDebugReadMemoryFlags"),
    "JsErrorCode": SimTypeInt(signed=False, label="JsErrorCode"),
    "JsFinalizeCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["data"]), offset=0),
    "JsMemoryAllocationCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="JsMemoryEventType"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBool(), arg_names=["callbackState", "allocationEvent", "allocationSize"]), offset=0),
    "JsMemoryEventType": SimTypeInt(signed=False, label="JsMemoryEventType"),
    "JsNativeFunction": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeBool(), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["callee", "isConstructCall", "arguments", "argumentCount", "callbackState"]), offset=0),
    "JsRuntimeAttributes": SimTypeInt(signed=False, label="JsRuntimeAttributes"),
    "JsRuntimeVersion": SimTypeInt(signed=False, label="JsRuntimeVersion"),
    "JsThreadServiceCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["callbackState"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBool(), arg_names=["callback", "callbackState"]), offset=0),
    "JsValueType": SimTypeInt(signed=False, label="JsValueType"),
    "KADDRESS_RANGE": SimStruct(OrderedDict((("Address", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KADDRESS_RANGE", pack=False, align=None),
    "KADDRESS_RANGE_DESCRIPTOR": SimStruct(OrderedDict((("AddressRanges", SimTypePointer(SimTypeRef("KADDRESS_RANGE", SimStruct), offset=0)), ("AddressRangeCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KADDRESS_RANGE_DESCRIPTOR", pack=False, align=None),
    "KAGGETDIAG": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("vDiagInfo", SimTypeRef("VARIANT", SimStruct)), ("sDiagField", SimTypeShort(signed=True, label="Int16")),)), name="KAGGETDIAG", pack=False, align=None),
    "KAGREQDIAG": SimStruct(OrderedDict((("ulDiagFlags", SimTypeInt(signed=False, label="UInt32")), ("vt", SimTypeInt(signed=False, label="VARENUM")), ("sDiagField", SimTypeShort(signed=True, label="Int16")),)), name="KAGREQDIAG", pack=False, align=None),
    "KAGREQDIAGFLAGSENUM": SimTypeInt(signed=False, label="KAGREQDIAGFLAGSENUM"),
    "KAPC": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("AllFlags", SimTypeChar(label="Byte")), ("Size", SimTypeChar(label="Byte")), ("SpareByte1", SimTypeChar(label="Byte")), ("SpareLong0", SimTypeInt(signed=False, label="UInt32")), ("Thread", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("ApcListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("Reserved", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 3)), ("NormalContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SystemArgument1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SystemArgument2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ApcStateIndex", SimTypeChar(label="SByte")), ("ApcMode", SimTypeChar(label="SByte")), ("Inserted", SimTypeChar(label="Byte")),)), name="KAPC", pack=False, align=None),
    "KAPC_STATE": SimStruct(OrderedDict((("ApcListHead", SimTypeArray(SimTypeRef("LIST_ENTRY", SimStruct), 2)), ("Process", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("Anonymous1", SimUnion({"InProgressFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("KernelApcPending", SimTypeChar(label="Byte")), ("Anonymous2", SimUnion({"UserApcPendingAll": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="KAPC_STATE", pack=False, align=None),
    "KBDLLHOOKSTRUCT": SimStruct(OrderedDict((("vkCode", SimTypeInt(signed=False, label="UInt32")), ("scanCode", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="KBDLLHOOKSTRUCT_FLAGS")), ("time", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KBDLLHOOKSTRUCT", pack=False, align=None),
    "KBDLLHOOKSTRUCT_FLAGS": SimTypeInt(signed=False, label="KBDLLHOOKSTRUCT_FLAGS"),
    "KBDNLSTABLES": SimStruct(OrderedDict((("OEMIdentifier", SimTypeShort(signed=False, label="UInt16")), ("LayoutInformation", SimTypeShort(signed=False, label="UInt16")), ("NumOfVkToF", SimTypeInt(signed=False, label="UInt32")), ("pVkToF", SimTypePointer(SimTypeRef("VK_F", SimStruct), offset=0)), ("NumOfMouseVKey", SimTypeInt(signed=True, label="Int32")), ("pusMouseVKey", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="KBDNLSTABLES", pack=False, align=None),
    "KBDTABLES": SimStruct(OrderedDict((("pCharModifiers", SimTypePointer(SimTypeRef("MODIFIERS", SimStruct), offset=0)), ("pVkToWcharTable", SimTypePointer(SimTypeRef("VK_TO_WCHAR_TABLE", SimStruct), offset=0)), ("pDeadKey", SimTypePointer(SimTypeRef("DEADKEY", SimStruct), offset=0)), ("pKeyNames", SimTypePointer(SimTypeRef("VSC_LPWSTR", SimStruct), offset=0)), ("pKeyNamesExt", SimTypePointer(SimTypeRef("VSC_LPWSTR", SimStruct), offset=0)), ("pKeyNamesDead", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)), ("pusVSCtoVK", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("bMaxVSCtoVK", SimTypeChar(label="Byte")), ("pVSCtoVK_E0", SimTypePointer(SimTypeRef("VSC_VK", SimStruct), offset=0)), ("pVSCtoVK_E1", SimTypePointer(SimTypeRef("VSC_VK", SimStruct), offset=0)), ("fLocaleFlags", SimTypeInt(signed=False, label="UInt32")), ("nLgMax", SimTypeChar(label="Byte")), ("cbLgEntry", SimTypeChar(label="Byte")), ("pLigature", SimTypePointer(SimTypeRef("LIGATURE1", SimStruct), offset=0)), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwSubType", SimTypeInt(signed=False, label="UInt32")),)), name="KBDTABLES", pack=False, align=None),
    "KBDTABLE_DESC": SimStruct(OrderedDict((("wszDllName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwSubType", SimTypeInt(signed=False, label="UInt32")),)), name="KBDTABLE_DESC", pack=False, align=None),
    "KBDTABLE_MULTI": SimStruct(OrderedDict((("nTables", SimTypeInt(signed=False, label="UInt32")), ("aKbdTables", SimTypeArray(SimTypeRef("KBDTABLE_DESC", SimStruct), 8)),)), name="KBDTABLE_MULTI", pack=False, align=None),
    "KBD_TYPE_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwSubType", SimTypeInt(signed=False, label="UInt32")),)), name="KBD_TYPE_INFO", pack=False, align=None),
    "KBUGCHECK_ADD_PAGES": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Count", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KBUGCHECK_ADD_PAGES", pack=False, align=None),
    "KBUGCHECK_BUFFER_DUMP_STATE": SimTypeInt(signed=False, label="KBUGCHECK_BUFFER_DUMP_STATE"),
    "KBUGCHECK_CALLBACK_REASON": SimTypeInt(signed=False, label="KBUGCHECK_CALLBACK_REASON"),
    "KBUGCHECK_CALLBACK_RECORD": SimStruct(OrderedDict((("Entry", SimTypeRef("LIST_ENTRY", SimStruct)), ("CallbackRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Component", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Checksum", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("State", SimTypeChar(label="Byte")),)), name="KBUGCHECK_CALLBACK_RECORD", pack=False, align=None),
    "KBUGCHECK_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "Length"]), offset=0),
    "KBUGCHECK_DUMP_IO": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferLength", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="KBUGCHECK_DUMP_IO_TYPE")),)), name="KBUGCHECK_DUMP_IO", pack=False, align=None),
    "KBUGCHECK_DUMP_IO_TYPE": SimTypeInt(signed=False, label="KBUGCHECK_DUMP_IO_TYPE"),
    "KBUGCHECK_REASON_CALLBACK_RECORD": SimStruct(OrderedDict((("Entry", SimTypeRef("LIST_ENTRY", SimStruct)), ("CallbackRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Component", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Checksum", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reason", SimTypeInt(signed=False, label="KBUGCHECK_CALLBACK_REASON")), ("State", SimTypeChar(label="Byte")),)), name="KBUGCHECK_REASON_CALLBACK_RECORD", pack=False, align=None),
    "KBUGCHECK_REASON_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="KBUGCHECK_CALLBACK_REASON"), SimTypePointer(SimTypeRef("KBUGCHECK_REASON_CALLBACK_RECORD", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Reason", "Record", "ReasonSpecificData", "ReasonSpecificDataLength"]), offset=0),
    "KBUGCHECK_REMOVE_PAGES": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Count", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KBUGCHECK_REMOVE_PAGES", pack=False, align=None),
    "KBUGCHECK_SECONDARY_DUMP_DATA": SimStruct(OrderedDict((("InBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InBufferLength", SimTypeInt(signed=False, label="UInt32")), ("MaximumAllowed", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("OutBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="KBUGCHECK_SECONDARY_DUMP_DATA", pack=False, align=None),
    "KBUGCHECK_SECONDARY_DUMP_DATA_EX": SimStruct(OrderedDict((("InBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InBufferLength", SimTypeInt(signed=False, label="UInt32")), ("MaximumAllowed", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("OutBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutBufferLength", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DumpType", SimTypeInt(signed=False, label="UInt32")), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BugCheckParameter2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BugCheckParameter3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BugCheckParameter4", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KBUGCHECK_SECONDARY_DUMP_DATA_EX", pack=False, align=None),
    "KBUGCHECK_TRIAGE_DUMP_DATA": SimStruct(OrderedDict((("DataArray", SimTypePointer(SimTypeRef("KTRIAGE_DUMP_DATA_ARRAY", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxVirtMemSize", SimTypeInt(signed=False, label="UInt32")), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("BugCheckParameter1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BugCheckParameter2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BugCheckParameter3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BugCheckParameter4", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KBUGCHECK_TRIAGE_DUMP_DATA", pack=False, align=None),
    "KCRM_MARSHAL_HEADER": SimStruct(OrderedDict((("VersionMajor", SimTypeInt(signed=False, label="UInt32")), ("VersionMinor", SimTypeInt(signed=False, label="UInt32")), ("NumProtocols", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeInt(signed=False, label="UInt32")),)), name="KCRM_MARSHAL_HEADER", pack=False, align=None),
    "KCRM_PROTOCOL_BLOB": SimStruct(OrderedDict((("ProtocolId", SimTypeBottom(label="Guid")), ("StaticInfoLength", SimTypeInt(signed=False, label="UInt32")), ("TransactionIdInfoLength", SimTypeInt(signed=False, label="UInt32")), ("Unused1", SimTypeInt(signed=False, label="UInt32")), ("Unused2", SimTypeInt(signed=False, label="UInt32")),)), name="KCRM_PROTOCOL_BLOB", pack=False, align=None),
    "KCRM_TRANSACTION_BLOB": SimStruct(OrderedDict((("UOW", SimTypeBottom(label="Guid")), ("TmIdentity", SimTypeBottom(label="Guid")), ("IsolationLevel", SimTypeInt(signed=False, label="UInt32")), ("IsolationFlags", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Description", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="KCRM_TRANSACTION_BLOB", pack=False, align=None),
    "KDC_PROXY_CACHE_ENTRY_DATA": SimStruct(OrderedDict((("SinceLastUsed", SimTypeLongLong(signed=False, label="UInt64")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProxyServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProxyServerVdir", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProxyServerPort", SimTypeShort(signed=False, label="UInt16")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("CredUserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("CredDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("GlobalCache", SimTypeChar(label="Byte")),)), name="KDC_PROXY_CACHE_ENTRY_DATA", pack=False, align=None),
    "KDDEBUGGER_DATA32": SimStruct(OrderedDict((("Header", SimTypeRef("DBGKD_DEBUG_DATA_HEADER32", SimStruct)), ("KernBase", SimTypeInt(signed=False, label="UInt32")), ("BreakpointWithStatus", SimTypeInt(signed=False, label="UInt32")), ("SavedContext", SimTypeInt(signed=False, label="UInt32")), ("ThCallbackStack", SimTypeShort(signed=False, label="UInt16")), ("NextCallback", SimTypeShort(signed=False, label="UInt16")), ("FramePointer", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("KiCallUserMode", SimTypeInt(signed=False, label="UInt32")), ("KeUserCallbackDispatcher", SimTypeInt(signed=False, label="UInt32")), ("PsLoadedModuleList", SimTypeInt(signed=False, label="UInt32")), ("PsActiveProcessHead", SimTypeInt(signed=False, label="UInt32")), ("PspCidTable", SimTypeInt(signed=False, label="UInt32")), ("ExpSystemResourcesList", SimTypeInt(signed=False, label="UInt32")), ("ExpPagedPoolDescriptor", SimTypeInt(signed=False, label="UInt32")), ("ExpNumberOfPagedPools", SimTypeInt(signed=False, label="UInt32")), ("KeTimeIncrement", SimTypeInt(signed=False, label="UInt32")), ("KeBugCheckCallbackListHead", SimTypeInt(signed=False, label="UInt32")), ("KiBugcheckData", SimTypeInt(signed=False, label="UInt32")), ("IopErrorLogListHead", SimTypeInt(signed=False, label="UInt32")), ("ObpRootDirectoryObject", SimTypeInt(signed=False, label="UInt32")), ("ObpTypeObjectType", SimTypeInt(signed=False, label="UInt32")), ("MmSystemCacheStart", SimTypeInt(signed=False, label="UInt32")), ("MmSystemCacheEnd", SimTypeInt(signed=False, label="UInt32")), ("MmSystemCacheWs", SimTypeInt(signed=False, label="UInt32")), ("MmPfnDatabase", SimTypeInt(signed=False, label="UInt32")), ("MmSystemPtesStart", SimTypeInt(signed=False, label="UInt32")), ("MmSystemPtesEnd", SimTypeInt(signed=False, label="UInt32")), ("MmSubsectionBase", SimTypeInt(signed=False, label="UInt32")), ("MmNumberOfPagingFiles", SimTypeInt(signed=False, label="UInt32")), ("MmLowestPhysicalPage", SimTypeInt(signed=False, label="UInt32")), ("MmHighestPhysicalPage", SimTypeInt(signed=False, label="UInt32")), ("MmNumberOfPhysicalPages", SimTypeInt(signed=False, label="UInt32")), ("MmMaximumNonPagedPoolInBytes", SimTypeInt(signed=False, label="UInt32")), ("MmNonPagedSystemStart", SimTypeInt(signed=False, label="UInt32")), ("MmNonPagedPoolStart", SimTypeInt(signed=False, label="UInt32")), ("MmNonPagedPoolEnd", SimTypeInt(signed=False, label="UInt32")), ("MmPagedPoolStart", SimTypeInt(signed=False, label="UInt32")), ("MmPagedPoolEnd", SimTypeInt(signed=False, label="UInt32")), ("MmPagedPoolInformation", SimTypeInt(signed=False, label="UInt32")), ("MmPageSize", SimTypeInt(signed=False, label="UInt32")), ("MmSizeOfPagedPoolInBytes", SimTypeInt(signed=False, label="UInt32")), ("MmTotalCommitLimit", SimTypeInt(signed=False, label="UInt32")), ("MmTotalCommittedPages", SimTypeInt(signed=False, label="UInt32")), ("MmSharedCommit", SimTypeInt(signed=False, label="UInt32")), ("MmDriverCommit", SimTypeInt(signed=False, label="UInt32")), ("MmProcessCommit", SimTypeInt(signed=False, label="UInt32")), ("MmPagedPoolCommit", SimTypeInt(signed=False, label="UInt32")), ("MmExtendedCommit", SimTypeInt(signed=False, label="UInt32")), ("MmZeroedPageListHead", SimTypeInt(signed=False, label="UInt32")), ("MmFreePageListHead", SimTypeInt(signed=False, label="UInt32")), ("MmStandbyPageListHead", SimTypeInt(signed=False, label="UInt32")), ("MmModifiedPageListHead", SimTypeInt(signed=False, label="UInt32")), ("MmModifiedNoWritePageListHead", SimTypeInt(signed=False, label="UInt32")), ("MmAvailablePages", SimTypeInt(signed=False, label="UInt32")), ("MmResidentAvailablePages", SimTypeInt(signed=False, label="UInt32")), ("PoolTrackTable", SimTypeInt(signed=False, label="UInt32")), ("NonPagedPoolDescriptor", SimTypeInt(signed=False, label="UInt32")), ("MmHighestUserAddress", SimTypeInt(signed=False, label="UInt32")), ("MmSystemRangeStart", SimTypeInt(signed=False, label="UInt32")), ("MmUserProbeAddress", SimTypeInt(signed=False, label="UInt32")), ("KdPrintCircularBuffer", SimTypeInt(signed=False, label="UInt32")), ("KdPrintCircularBufferEnd", SimTypeInt(signed=False, label="UInt32")), ("KdPrintWritePointer", SimTypeInt(signed=False, label="UInt32")), ("KdPrintRolloverCount", SimTypeInt(signed=False, label="UInt32")), ("MmLoadedUserImageList", SimTypeInt(signed=False, label="UInt32")),)), name="KDDEBUGGER_DATA32", pack=False, align=None),
    "KDDEBUGGER_DATA64": SimStruct(OrderedDict((("Header", SimTypeRef("DBGKD_DEBUG_DATA_HEADER64", SimStruct)), ("KernBase", SimTypeLongLong(signed=False, label="UInt64")), ("BreakpointWithStatus", SimTypeLongLong(signed=False, label="UInt64")), ("SavedContext", SimTypeLongLong(signed=False, label="UInt64")), ("ThCallbackStack", SimTypeShort(signed=False, label="UInt16")), ("NextCallback", SimTypeShort(signed=False, label="UInt16")), ("FramePointer", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("KiCallUserMode", SimTypeLongLong(signed=False, label="UInt64")), ("KeUserCallbackDispatcher", SimTypeLongLong(signed=False, label="UInt64")), ("PsLoadedModuleList", SimTypeLongLong(signed=False, label="UInt64")), ("PsActiveProcessHead", SimTypeLongLong(signed=False, label="UInt64")), ("PspCidTable", SimTypeLongLong(signed=False, label="UInt64")), ("ExpSystemResourcesList", SimTypeLongLong(signed=False, label="UInt64")), ("ExpPagedPoolDescriptor", SimTypeLongLong(signed=False, label="UInt64")), ("ExpNumberOfPagedPools", SimTypeLongLong(signed=False, label="UInt64")), ("KeTimeIncrement", SimTypeLongLong(signed=False, label="UInt64")), ("KeBugCheckCallbackListHead", SimTypeLongLong(signed=False, label="UInt64")), ("KiBugcheckData", SimTypeLongLong(signed=False, label="UInt64")), ("IopErrorLogListHead", SimTypeLongLong(signed=False, label="UInt64")), ("ObpRootDirectoryObject", SimTypeLongLong(signed=False, label="UInt64")), ("ObpTypeObjectType", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemCacheStart", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemCacheEnd", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemCacheWs", SimTypeLongLong(signed=False, label="UInt64")), ("MmPfnDatabase", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemPtesStart", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemPtesEnd", SimTypeLongLong(signed=False, label="UInt64")), ("MmSubsectionBase", SimTypeLongLong(signed=False, label="UInt64")), ("MmNumberOfPagingFiles", SimTypeLongLong(signed=False, label="UInt64")), ("MmLowestPhysicalPage", SimTypeLongLong(signed=False, label="UInt64")), ("MmHighestPhysicalPage", SimTypeLongLong(signed=False, label="UInt64")), ("MmNumberOfPhysicalPages", SimTypeLongLong(signed=False, label="UInt64")), ("MmMaximumNonPagedPoolInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MmNonPagedSystemStart", SimTypeLongLong(signed=False, label="UInt64")), ("MmNonPagedPoolStart", SimTypeLongLong(signed=False, label="UInt64")), ("MmNonPagedPoolEnd", SimTypeLongLong(signed=False, label="UInt64")), ("MmPagedPoolStart", SimTypeLongLong(signed=False, label="UInt64")), ("MmPagedPoolEnd", SimTypeLongLong(signed=False, label="UInt64")), ("MmPagedPoolInformation", SimTypeLongLong(signed=False, label="UInt64")), ("MmPageSize", SimTypeLongLong(signed=False, label="UInt64")), ("MmSizeOfPagedPoolInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MmTotalCommitLimit", SimTypeLongLong(signed=False, label="UInt64")), ("MmTotalCommittedPages", SimTypeLongLong(signed=False, label="UInt64")), ("MmSharedCommit", SimTypeLongLong(signed=False, label="UInt64")), ("MmDriverCommit", SimTypeLongLong(signed=False, label="UInt64")), ("MmProcessCommit", SimTypeLongLong(signed=False, label="UInt64")), ("MmPagedPoolCommit", SimTypeLongLong(signed=False, label="UInt64")), ("MmExtendedCommit", SimTypeLongLong(signed=False, label="UInt64")), ("MmZeroedPageListHead", SimTypeLongLong(signed=False, label="UInt64")), ("MmFreePageListHead", SimTypeLongLong(signed=False, label="UInt64")), ("MmStandbyPageListHead", SimTypeLongLong(signed=False, label="UInt64")), ("MmModifiedPageListHead", SimTypeLongLong(signed=False, label="UInt64")), ("MmModifiedNoWritePageListHead", SimTypeLongLong(signed=False, label="UInt64")), ("MmAvailablePages", SimTypeLongLong(signed=False, label="UInt64")), ("MmResidentAvailablePages", SimTypeLongLong(signed=False, label="UInt64")), ("PoolTrackTable", SimTypeLongLong(signed=False, label="UInt64")), ("NonPagedPoolDescriptor", SimTypeLongLong(signed=False, label="UInt64")), ("MmHighestUserAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemRangeStart", SimTypeLongLong(signed=False, label="UInt64")), ("MmUserProbeAddress", SimTypeLongLong(signed=False, label="UInt64")), ("KdPrintCircularBuffer", SimTypeLongLong(signed=False, label="UInt64")), ("KdPrintCircularBufferEnd", SimTypeLongLong(signed=False, label="UInt64")), ("KdPrintWritePointer", SimTypeLongLong(signed=False, label="UInt64")), ("KdPrintRolloverCount", SimTypeLongLong(signed=False, label="UInt64")), ("MmLoadedUserImageList", SimTypeLongLong(signed=False, label="UInt64")), ("NtBuildLab", SimTypeLongLong(signed=False, label="UInt64")), ("KiNormalSystemCall", SimTypeLongLong(signed=False, label="UInt64")), ("KiProcessorBlock", SimTypeLongLong(signed=False, label="UInt64")), ("MmUnloadedDrivers", SimTypeLongLong(signed=False, label="UInt64")), ("MmLastUnloadedDriver", SimTypeLongLong(signed=False, label="UInt64")), ("MmTriageActionTaken", SimTypeLongLong(signed=False, label="UInt64")), ("MmSpecialPoolTag", SimTypeLongLong(signed=False, label="UInt64")), ("KernelVerifier", SimTypeLongLong(signed=False, label="UInt64")), ("MmVerifierData", SimTypeLongLong(signed=False, label="UInt64")), ("MmAllocatedNonPagedPool", SimTypeLongLong(signed=False, label="UInt64")), ("MmPeakCommitment", SimTypeLongLong(signed=False, label="UInt64")), ("MmTotalCommitLimitMaximum", SimTypeLongLong(signed=False, label="UInt64")), ("CmNtCSDVersion", SimTypeLongLong(signed=False, label="UInt64")), ("MmPhysicalMemoryBlock", SimTypeLongLong(signed=False, label="UInt64")), ("MmSessionBase", SimTypeLongLong(signed=False, label="UInt64")), ("MmSessionSize", SimTypeLongLong(signed=False, label="UInt64")), ("MmSystemParentTablePage", SimTypeLongLong(signed=False, label="UInt64")), ("MmVirtualTranslationBase", SimTypeLongLong(signed=False, label="UInt64")), ("OffsetKThreadNextProcessor", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadTeb", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadKernelStack", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadInitialStack", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadApcProcess", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadState", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadBStore", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadBStoreLimit", SimTypeShort(signed=False, label="UInt16")), ("SizeEProcess", SimTypeShort(signed=False, label="UInt16")), ("OffsetEprocessPeb", SimTypeShort(signed=False, label="UInt16")), ("OffsetEprocessParentCID", SimTypeShort(signed=False, label="UInt16")), ("OffsetEprocessDirectoryTableBase", SimTypeShort(signed=False, label="UInt16")), ("SizePrcb", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbDpcRoutine", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbCurrentThread", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbMhz", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbCpuType", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbVendorString", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbProcStateContext", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbNumber", SimTypeShort(signed=False, label="UInt16")), ("SizeEThread", SimTypeShort(signed=False, label="UInt16")), ("L1tfHighPhysicalBitIndex", SimTypeChar(label="Byte")), ("L1tfSwizzleBitIndex", SimTypeChar(label="Byte")), ("Padding0", SimTypeInt(signed=False, label="UInt32")), ("KdPrintCircularBufferPtr", SimTypeLongLong(signed=False, label="UInt64")), ("KdPrintBufferSize", SimTypeLongLong(signed=False, label="UInt64")), ("KeLoaderBlock", SimTypeLongLong(signed=False, label="UInt64")), ("SizePcr", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrSelfPcr", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrCurrentPrcb", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrContainedPrcb", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrInitialBStore", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrBStoreLimit", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrInitialStack", SimTypeShort(signed=False, label="UInt16")), ("OffsetPcrStackLimit", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbPcrPage", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbProcStateSpecialReg", SimTypeShort(signed=False, label="UInt16")), ("GdtR0Code", SimTypeShort(signed=False, label="UInt16")), ("GdtR0Data", SimTypeShort(signed=False, label="UInt16")), ("GdtR0Pcr", SimTypeShort(signed=False, label="UInt16")), ("GdtR3Code", SimTypeShort(signed=False, label="UInt16")), ("GdtR3Data", SimTypeShort(signed=False, label="UInt16")), ("GdtR3Teb", SimTypeShort(signed=False, label="UInt16")), ("GdtLdt", SimTypeShort(signed=False, label="UInt16")), ("GdtTss", SimTypeShort(signed=False, label="UInt16")), ("Gdt64R3CmCode", SimTypeShort(signed=False, label="UInt16")), ("Gdt64R3CmTeb", SimTypeShort(signed=False, label="UInt16")), ("IopNumTriageDumpDataBlocks", SimTypeLongLong(signed=False, label="UInt64")), ("IopTriageDumpDataBlocks", SimTypeLongLong(signed=False, label="UInt64")), ("VfCrashDataBlock", SimTypeLongLong(signed=False, label="UInt64")), ("MmBadPagesDetected", SimTypeLongLong(signed=False, label="UInt64")), ("MmZeroedPageSingleBitErrorsDetected", SimTypeLongLong(signed=False, label="UInt64")), ("EtwpDebuggerData", SimTypeLongLong(signed=False, label="UInt64")), ("OffsetPrcbContext", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbMaxBreakpoints", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbMaxWatchpoints", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadStackLimit", SimTypeInt(signed=False, label="UInt32")), ("OffsetKThreadStackBase", SimTypeInt(signed=False, label="UInt32")), ("OffsetKThreadQueueListEntry", SimTypeInt(signed=False, label="UInt32")), ("OffsetEThreadIrpList", SimTypeInt(signed=False, label="UInt32")), ("OffsetPrcbIdleThread", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbNormalDpcState", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbDpcStack", SimTypeShort(signed=False, label="UInt16")), ("OffsetPrcbIsrStack", SimTypeShort(signed=False, label="UInt16")), ("SizeKDPC_STACK_FRAME", SimTypeShort(signed=False, label="UInt16")), ("OffsetKPriQueueThreadListHead", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadWaitReason", SimTypeShort(signed=False, label="UInt16")), ("Padding1", SimTypeShort(signed=False, label="UInt16")), ("PteBase", SimTypeLongLong(signed=False, label="UInt64")), ("RetpolineStubFunctionTable", SimTypeLongLong(signed=False, label="UInt64")), ("RetpolineStubFunctionTableSize", SimTypeInt(signed=False, label="UInt32")), ("RetpolineStubOffset", SimTypeInt(signed=False, label="UInt32")), ("RetpolineStubSize", SimTypeInt(signed=False, label="UInt32")), ("OffsetEProcessMmHotPatchContext", SimTypeShort(signed=False, label="UInt16")), ("OffsetKThreadShadowStackLimit", SimTypeInt(signed=False, label="UInt32")), ("OffsetKThreadShadowStackBase", SimTypeInt(signed=False, label="UInt32")), ("ShadowStackEnabled", SimTypeLongLong(signed=False, label="UInt64")), ("PointerAuthMask", SimTypeLongLong(signed=False, label="UInt64")), ("OffsetPrcbExceptionStack", SimTypeShort(signed=False, label="UInt16")),)), name="KDDEBUGGER_DATA64", pack=False, align=None),
    "KDEFERRED_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KDPC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Dpc", "DeferredContext", "SystemArgument1", "SystemArgument2"]), offset=0),
    "KDEVICE_QUEUE": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=True, label="Int16")), ("DeviceListHead", SimTypeRef("LIST_ENTRY", SimStruct)), ("Lock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Busy", SimTypeChar(label="Byte")),)), name="KDEVICE_QUEUE", pack=False, align=None),
    "KDEVICE_QUEUE_ENTRY": SimStruct(OrderedDict((("DeviceListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("SortKey", SimTypeInt(signed=False, label="UInt32")), ("Inserted", SimTypeChar(label="Byte")),)), name="KDEVICE_QUEUE_ENTRY", pack=False, align=None),
    "KDEXTS_LOCK_CALLBACKROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KDEXTS_LOCK_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pLock", "Context"]), offset=0),
    "KDEXTS_LOCK_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("OwningThread", SimTypeLongLong(signed=False, label="UInt64")), ("ExclusiveOwned", SimTypeInt(signed=True, label="Int32")), ("NumOwners", SimTypeInt(signed=False, label="UInt32")), ("ContentionCount", SimTypeInt(signed=False, label="UInt32")), ("NumExclusiveWaiters", SimTypeInt(signed=False, label="UInt32")), ("NumSharedWaiters", SimTypeInt(signed=False, label="UInt32")), ("pOwnerThreads", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("pWaiterThreads", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="KDEXTS_LOCK_INFO", pack=False, align=None),
    "KDEXTS_PTE_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("VirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PpeAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PdeAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PteAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Pfn", SimTypeLongLong(signed=False, label="UInt64")), ("Levels", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")),)), name="KDEXTS_PTE_INFO", pack=False, align=None),
    "KDEXT_DUMP_HANDLE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KDEXT_HANDLE_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["HandleInfo", "Flags", "Context"]), offset=0),
    "KDEXT_FILELOCK_OWNER": SimStruct(OrderedDict((("Sizeofstruct", SimTypeInt(signed=False, label="UInt32")), ("FileObject", SimTypeLongLong(signed=False, label="UInt64")), ("OwnerThread", SimTypeLongLong(signed=False, label="UInt64")), ("WaitIrp", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceObject", SimTypeLongLong(signed=False, label="UInt64")), ("BlockingDirver", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="KDEXT_FILELOCK_OWNER", pack=False, align=None),
    "KDEXT_HANDLE_INFORMATION": SimStruct(OrderedDict((("HandleTableEntry", SimTypeLongLong(signed=False, label="UInt64")), ("Handle", SimTypeLongLong(signed=False, label="UInt64")), ("Object", SimTypeLongLong(signed=False, label="UInt64")), ("ObjectBody", SimTypeLongLong(signed=False, label="UInt64")), ("GrantedAccess", SimTypeLongLong(signed=False, label="UInt64")), ("HandleAttributes", SimTypeInt(signed=False, label="UInt32")), ("PagedOut", SimTypeChar(label="Byte")),)), name="KDEXT_HANDLE_INFORMATION", pack=False, align=None),
    "KDEXT_PROCESS_FIND_PARAMS": SimStruct(OrderedDict((("SizeofStruct", SimTypeInt(signed=False, label="UInt32")), ("Pid", SimTypeInt(signed=False, label="UInt32")), ("Session", SimTypeInt(signed=False, label="UInt32")), ("ImageName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KDEXT_PROCESS_FIND_PARAMS", pack=False, align=None),
    "KDEXT_THREAD_FIND_PARAMS": SimStruct(OrderedDict((("SizeofStruct", SimTypeInt(signed=False, label="UInt32")), ("StackPointer", SimTypeLongLong(signed=False, label="UInt64")), ("Cid", SimTypeInt(signed=False, label="UInt32")), ("Thread", SimTypeLongLong(signed=False, label="UInt64")),)), name="KDEXT_THREAD_FIND_PARAMS", pack=False, align=None),
    "KDHELP": SimStruct(OrderedDict((("Thread", SimTypeInt(signed=False, label="UInt32")), ("ThCallbackStack", SimTypeInt(signed=False, label="UInt32")), ("NextCallback", SimTypeInt(signed=False, label="UInt32")), ("FramePointer", SimTypeInt(signed=False, label="UInt32")), ("KiCallUserMode", SimTypeInt(signed=False, label="UInt32")), ("KeUserCallbackDispatcher", SimTypeInt(signed=False, label="UInt32")), ("SystemRangeStart", SimTypeInt(signed=False, label="UInt32")), ("ThCallbackBStore", SimTypeInt(signed=False, label="UInt32")), ("KiUserExceptionDispatcher", SimTypeInt(signed=False, label="UInt32")), ("StackBase", SimTypeInt(signed=False, label="UInt32")), ("StackLimit", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)),)), name="KDHELP", pack=False, align=None),
    "KDHELP64": SimStruct(OrderedDict((("Thread", SimTypeLongLong(signed=False, label="UInt64")), ("ThCallbackStack", SimTypeInt(signed=False, label="UInt32")), ("ThCallbackBStore", SimTypeInt(signed=False, label="UInt32")), ("NextCallback", SimTypeInt(signed=False, label="UInt32")), ("FramePointer", SimTypeInt(signed=False, label="UInt32")), ("KiCallUserMode", SimTypeLongLong(signed=False, label="UInt64")), ("KeUserCallbackDispatcher", SimTypeLongLong(signed=False, label="UInt64")), ("SystemRangeStart", SimTypeLongLong(signed=False, label="UInt64")), ("KiUserExceptionDispatcher", SimTypeLongLong(signed=False, label="UInt64")), ("StackBase", SimTypeLongLong(signed=False, label="UInt64")), ("StackLimit", SimTypeLongLong(signed=False, label="UInt64")), ("BuildVersion", SimTypeInt(signed=False, label="UInt32")), ("RetpolineStubFunctionTableSize", SimTypeInt(signed=False, label="UInt32")), ("RetpolineStubFunctionTable", SimTypeLongLong(signed=False, label="UInt64")), ("RetpolineStubOffset", SimTypeInt(signed=False, label="UInt32")), ("RetpolineStubSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="KDHELP64", pack=False, align=None),
    "KDPC": SimStruct(OrderedDict((("Anonymous", SimUnion({"TargetInfoAsUlong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Importance", SimTypeChar(label="Byte")), ("Number", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("DpcListEntry", SimTypeRef("SINGLE_LIST_ENTRY", SimStruct)), ("ProcessorHistory", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DeferredRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DeferredContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SystemArgument1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SystemArgument2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DpcData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KDPC", pack=False, align=None),
    "KDPC_IMPORTANCE": SimTypeInt(signed=False, label="KDPC_IMPORTANCE"),
    "KDPC_WATCHDOG_INFORMATION": SimStruct(OrderedDict((("DpcTimeLimit", SimTypeInt(signed=False, label="UInt32")), ("DpcTimeCount", SimTypeInt(signed=False, label="UInt32")), ("DpcWatchdogLimit", SimTypeInt(signed=False, label="UInt32")), ("DpcWatchdogCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KDPC_WATCHDOG_INFORMATION", pack=False, align=None),
    "KD_CALLBACK_ACTION": SimTypeInt(signed=False, label="KD_CALLBACK_ACTION"),
    "KD_NAMESPACE_ENUM": SimTypeInt(signed=False, label="KD_NAMESPACE_ENUM"),
    "KD_OPTION": SimTypeInt(signed=False, label="KD_OPTION"),
    "KENLISTMENT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KERB_ADDRESS_TYPE": SimTypeInt(signed=False, label="KERB_ADDRESS_TYPE"),
    "KERB_ADD_BINDING_CACHE_ENTRY_EX_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("RealmName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("KdcAddress", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AddressType", SimTypeInt(signed=False, label="KERB_ADDRESS_TYPE")), ("DcFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_ADD_BINDING_CACHE_ENTRY_EX_REQUEST", pack=False, align=None),
    "KERB_ADD_BINDING_CACHE_ENTRY_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("RealmName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("KdcAddress", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AddressType", SimTypeInt(signed=False, label="KERB_ADDRESS_TYPE")),)), name="KERB_ADD_BINDING_CACHE_ENTRY_REQUEST", pack=False, align=None),
    "KERB_ADD_CREDENTIALS_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="KERB_REQUEST_FLAGS")),)), name="KERB_ADD_CREDENTIALS_REQUEST", pack=False, align=None),
    "KERB_ADD_CREDENTIALS_REQUEST_EX": SimStruct(OrderedDict((("Credentials", SimTypeRef("KERB_ADD_CREDENTIALS_REQUEST", SimStruct)), ("PrincipalNameCount", SimTypeInt(signed=False, label="UInt32")), ("PrincipalNames", SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)),)), name="KERB_ADD_CREDENTIALS_REQUEST_EX", pack=False, align=None),
    "KERB_AUTH_DATA": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_AUTH_DATA", pack=False, align=None),
    "KERB_BINDING_CACHE_ENTRY_DATA": SimStruct(OrderedDict((("DiscoveryTime", SimTypeLongLong(signed=False, label="UInt64")), ("RealmName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("KdcAddress", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AddressType", SimTypeInt(signed=False, label="KERB_ADDRESS_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DcFlags", SimTypeInt(signed=False, label="UInt32")), ("CacheFlags", SimTypeInt(signed=False, label="UInt32")), ("KdcName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_BINDING_CACHE_ENTRY_DATA", pack=False, align=None),
    "KERB_CERTIFICATE_HASHINFO": SimStruct(OrderedDict((("StoreNameLength", SimTypeShort(signed=False, label="UInt16")), ("HashLength", SimTypeShort(signed=False, label="UInt16")),)), name="KERB_CERTIFICATE_HASHINFO", pack=False, align=None),
    "KERB_CERTIFICATE_INFO": SimStruct(OrderedDict((("CertInfoSize", SimTypeInt(signed=False, label="UInt32")), ("InfoType", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_CERTIFICATE_INFO", pack=False, align=None),
    "KERB_CERTIFICATE_INFO_TYPE": SimTypeInt(signed=False, label="KERB_CERTIFICATE_INFO_TYPE"),
    "KERB_CERTIFICATE_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Pin", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CspDataLength", SimTypeInt(signed=False, label="UInt32")), ("CspData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_CERTIFICATE_LOGON", pack=False, align=None),
    "KERB_CERTIFICATE_S4U_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("UserPrincipalName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("CertificateLength", SimTypeInt(signed=False, label="UInt32")), ("Certificate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_CERTIFICATE_S4U_LOGON", pack=False, align=None),
    "KERB_CERTIFICATE_UNLOCK_LOGON": SimStruct(OrderedDict((("Logon", SimTypeRef("KERB_CERTIFICATE_LOGON", SimStruct)), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_CERTIFICATE_UNLOCK_LOGON", pack=False, align=None),
    "KERB_CHANGEPASSWORD_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AccountName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("OldPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("NewPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Impersonating", SimTypeChar(label="Byte")),)), name="KERB_CHANGEPASSWORD_REQUEST", pack=False, align=None),
    "KERB_CLEANUP_MACHINE_PKINIT_CREDS_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_CLEANUP_MACHINE_PKINIT_CREDS_REQUEST", pack=False, align=None),
    "KERB_CLOUD_KERBEROS_DEBUG_DATA": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_CLOUD_KERBEROS_DEBUG_DATA", pack=False, align=None),
    "KERB_CLOUD_KERBEROS_DEBUG_DATA_V0": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_CLOUD_KERBEROS_DEBUG_DATA_V0", pack=False, align=None),
    "KERB_CLOUD_KERBEROS_DEBUG_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_CLOUD_KERBEROS_DEBUG_REQUEST", pack=False, align=None),
    "KERB_CLOUD_KERBEROS_DEBUG_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="KERB_CLOUD_KERBEROS_DEBUG_RESPONSE", pack=False, align=None),
    "KERB_CRYPTO_KEY": SimStruct(OrderedDict((("KeyType", SimTypeInt(signed=False, label="KERB_CRYPTO_KEY_TYPE")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_CRYPTO_KEY", pack=False, align=None),
    "KERB_CRYPTO_KEY32": SimStruct(OrderedDict((("KeyType", SimTypeInt(signed=True, label="Int32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_CRYPTO_KEY32", pack=False, align=None),
    "KERB_CRYPTO_KEY_TYPE": SimTypeInt(signed=False, label="KERB_CRYPTO_KEY_TYPE"),
    "KERB_DECRYPT_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CryptoType", SimTypeInt(signed=True, label="Int32")), ("KeyUsage", SimTypeInt(signed=True, label="Int32")), ("Key", SimTypeRef("KERB_CRYPTO_KEY", SimStruct)), ("EncryptedDataSize", SimTypeInt(signed=False, label="UInt32")), ("InitialVectorSize", SimTypeInt(signed=False, label="UInt32")), ("InitialVector", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("EncryptedData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_DECRYPT_REQUEST", pack=False, align=None),
    "KERB_DECRYPT_RESPONSE": SimStruct(OrderedDict((("DecryptedData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_DECRYPT_RESPONSE", pack=False, align=None),
    "KERB_EXTERNAL_NAME": SimStruct(OrderedDict((("NameType", SimTypeShort(signed=True, label="Int16")), ("NameCount", SimTypeShort(signed=False, label="UInt16")), ("Names", SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)),)), name="KERB_EXTERNAL_NAME", pack=False, align=None),
    "KERB_EXTERNAL_TICKET": SimStruct(OrderedDict((("ServiceName", SimTypePointer(SimTypeRef("KERB_EXTERNAL_NAME", SimStruct), offset=0)), ("TargetName", SimTypePointer(SimTypeRef("KERB_EXTERNAL_NAME", SimStruct), offset=0)), ("ClientName", SimTypePointer(SimTypeRef("KERB_EXTERNAL_NAME", SimStruct), offset=0)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("TargetDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AltTargetDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("SessionKey", SimTypeRef("KERB_CRYPTO_KEY", SimStruct)), ("TicketFlags", SimTypeInt(signed=False, label="KERB_TICKET_FLAGS")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("KeyExpirationTime", SimTypeLongLong(signed=True, label="Int64")), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("RenewUntil", SimTypeLongLong(signed=True, label="Int64")), ("TimeSkew", SimTypeLongLong(signed=True, label="Int64")), ("EncodedTicketSize", SimTypeInt(signed=False, label="UInt32")), ("EncodedTicket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_EXTERNAL_TICKET", pack=False, align=None),
    "KERB_INTERACTIVE_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE")), ("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_INTERACTIVE_LOGON", pack=False, align=None),
    "KERB_INTERACTIVE_PROFILE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROFILE_BUFFER_TYPE")), ("LogonCount", SimTypeShort(signed=False, label="UInt16")), ("BadPasswordCount", SimTypeShort(signed=False, label="UInt16")), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("LogoffTime", SimTypeLongLong(signed=True, label="Int64")), ("KickOffTime", SimTypeLongLong(signed=True, label="Int64")), ("PasswordLastSet", SimTypeLongLong(signed=True, label="Int64")), ("PasswordCanChange", SimTypeLongLong(signed=True, label="Int64")), ("PasswordMustChange", SimTypeLongLong(signed=True, label="Int64")), ("LogonScript", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectory", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("FullName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProfilePath", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectoryDrive", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_INTERACTIVE_PROFILE", pack=False, align=None),
    "KERB_INTERACTIVE_UNLOCK_LOGON": SimStruct(OrderedDict((("Logon", SimTypeRef("KERB_INTERACTIVE_LOGON", SimStruct)), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_INTERACTIVE_UNLOCK_LOGON", pack=False, align=None),
    "KERB_LOGON_SUBMIT_TYPE": SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE"),
    "KERB_NET_ADDRESS": SimStruct(OrderedDict((("Family", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_NET_ADDRESS", pack=False, align=None),
    "KERB_NET_ADDRESSES": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("Addresses", SimTypePointer(SimTypeRef("KERB_NET_ADDRESS", SimStruct), offset=0)),)), name="KERB_NET_ADDRESSES", pack=False, align=None),
    "KERB_PROFILE_BUFFER_TYPE": SimTypeInt(signed=False, label="KERB_PROFILE_BUFFER_TYPE"),
    "KERB_PROTOCOL_MESSAGE_TYPE": SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE"),
    "KERB_PURGE_BINDING_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")),)), name="KERB_PURGE_BINDING_CACHE_REQUEST", pack=False, align=None),
    "KERB_PURGE_KDC_PROXY_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_PURGE_KDC_PROXY_CACHE_REQUEST", pack=False, align=None),
    "KERB_PURGE_KDC_PROXY_CACHE_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfPurged", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_PURGE_KDC_PROXY_CACHE_RESPONSE", pack=False, align=None),
    "KERB_PURGE_TKT_CACHE_EX_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TicketTemplate", SimTypeRef("KERB_TICKET_CACHE_INFO_EX", SimStruct)),)), name="KERB_PURGE_TKT_CACHE_EX_REQUEST", pack=False, align=None),
    "KERB_PURGE_TKT_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("ServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("RealmName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_PURGE_TKT_CACHE_REQUEST", pack=False, align=None),
    "KERB_QUERY_BINDING_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")),)), name="KERB_QUERY_BINDING_CACHE_REQUEST", pack=False, align=None),
    "KERB_QUERY_BINDING_CACHE_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfEntries", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeRef("KERB_BINDING_CACHE_ENTRY_DATA", SimStruct), offset=0)),)), name="KERB_QUERY_BINDING_CACHE_RESPONSE", pack=False, align=None),
    "KERB_QUERY_DOMAIN_EXTENDED_POLICIES_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_QUERY_DOMAIN_EXTENDED_POLICIES_REQUEST", pack=False, align=None),
    "KERB_QUERY_DOMAIN_EXTENDED_POLICIES_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ExtendedPolicies", SimTypeInt(signed=False, label="UInt32")), ("DsFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_QUERY_DOMAIN_EXTENDED_POLICIES_RESPONSE", pack=False, align=None),
    "KERB_QUERY_KDC_PROXY_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_QUERY_KDC_PROXY_CACHE_REQUEST", pack=False, align=None),
    "KERB_QUERY_KDC_PROXY_CACHE_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfEntries", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeRef("KDC_PROXY_CACHE_ENTRY_DATA", SimStruct), offset=0)),)), name="KERB_QUERY_KDC_PROXY_CACHE_RESPONSE", pack=False, align=None),
    "KERB_QUERY_S4U2PROXY_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_QUERY_S4U2PROXY_CACHE_REQUEST", pack=False, align=None),
    "KERB_QUERY_S4U2PROXY_CACHE_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfCreds", SimTypeInt(signed=False, label="UInt32")), ("Creds", SimTypePointer(SimTypeRef("KERB_S4U2PROXY_CRED", SimStruct), offset=0)),)), name="KERB_QUERY_S4U2PROXY_CACHE_RESPONSE", pack=False, align=None),
    "KERB_QUERY_TKT_CACHE_EX2_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfTickets", SimTypeInt(signed=False, label="UInt32")), ("Tickets", SimTypePointer(SimTypeRef("KERB_TICKET_CACHE_INFO_EX2", SimStruct), offset=0)),)), name="KERB_QUERY_TKT_CACHE_EX2_RESPONSE", pack=False, align=None),
    "KERB_QUERY_TKT_CACHE_EX3_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfTickets", SimTypeInt(signed=False, label="UInt32")), ("Tickets", SimTypePointer(SimTypeRef("KERB_TICKET_CACHE_INFO_EX3", SimStruct), offset=0)),)), name="KERB_QUERY_TKT_CACHE_EX3_RESPONSE", pack=False, align=None),
    "KERB_QUERY_TKT_CACHE_EX_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfTickets", SimTypeInt(signed=False, label="UInt32")), ("Tickets", SimTypePointer(SimTypeRef("KERB_TICKET_CACHE_INFO_EX", SimStruct), offset=0)),)), name="KERB_QUERY_TKT_CACHE_EX_RESPONSE", pack=False, align=None),
    "KERB_QUERY_TKT_CACHE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_QUERY_TKT_CACHE_REQUEST", pack=False, align=None),
    "KERB_QUERY_TKT_CACHE_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CountOfTickets", SimTypeInt(signed=False, label="UInt32")), ("Tickets", SimTypePointer(SimTypeRef("KERB_TICKET_CACHE_INFO", SimStruct), offset=0)),)), name="KERB_QUERY_TKT_CACHE_RESPONSE", pack=False, align=None),
    "KERB_REFRESH_POLICY_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_REFRESH_POLICY_REQUEST", pack=False, align=None),
    "KERB_REFRESH_POLICY_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_REFRESH_POLICY_RESPONSE", pack=False, align=None),
    "KERB_REFRESH_SCCRED_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("CredentialBlob", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_REFRESH_SCCRED_REQUEST", pack=False, align=None),
    "KERB_REQUEST_FLAGS": SimTypeInt(signed=False, label="KERB_REQUEST_FLAGS"),
    "KERB_RETRIEVE_KEY_TAB_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_RETRIEVE_KEY_TAB_REQUEST", pack=False, align=None),
    "KERB_RETRIEVE_KEY_TAB_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("KeyTabLength", SimTypeInt(signed=False, label="UInt32")), ("KeyTab", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_RETRIEVE_KEY_TAB_RESPONSE", pack=False, align=None),
    "KERB_RETRIEVE_TKT_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("TargetName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("TicketFlags", SimTypeInt(signed=False, label="UInt32")), ("CacheOptions", SimTypeInt(signed=False, label="UInt32")), ("EncryptionType", SimTypeInt(signed=False, label="KERB_CRYPTO_KEY_TYPE")), ("CredentialsHandle", SimTypeRef("SecHandle", SimStruct)),)), name="KERB_RETRIEVE_TKT_REQUEST", pack=False, align=None),
    "KERB_RETRIEVE_TKT_RESPONSE": SimStruct(OrderedDict((("Ticket", SimTypeRef("KERB_EXTERNAL_TICKET", SimStruct)),)), name="KERB_RETRIEVE_TKT_RESPONSE", pack=False, align=None),
    "KERB_S4U2PROXY_CACHE_ENTRY_INFO": SimStruct(OrderedDict((("ServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("LastStatus", SimTypeInt(signed=True, label="Int32")), ("Expiry", SimTypeLongLong(signed=True, label="Int64")),)), name="KERB_S4U2PROXY_CACHE_ENTRY_INFO", pack=False, align=None),
    "KERB_S4U2PROXY_CRED": SimStruct(OrderedDict((("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("LastStatus", SimTypeInt(signed=True, label="Int32")), ("Expiry", SimTypeLongLong(signed=True, label="Int64")), ("CountOfEntries", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeRef("KERB_S4U2PROXY_CACHE_ENTRY_INFO", SimStruct), offset=0)),)), name="KERB_S4U2PROXY_CRED", pack=False, align=None),
    "KERB_S4U_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ClientUpn", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ClientRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_S4U_LOGON", pack=False, align=None),
    "KERB_SETPASSWORD_EX_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("CredentialsHandle", SimTypeRef("SecHandle", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AccountRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AccountName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ClientRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ClientName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Impersonating", SimTypeChar(label="Byte")), ("KdcAddress", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("KdcAddressType", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_SETPASSWORD_EX_REQUEST", pack=False, align=None),
    "KERB_SETPASSWORD_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("CredentialsHandle", SimTypeRef("SecHandle", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AccountName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_SETPASSWORD_REQUEST", pack=False, align=None),
    "KERB_SMART_CARD_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE")), ("Pin", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("CspDataLength", SimTypeInt(signed=False, label="UInt32")), ("CspData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_SMART_CARD_LOGON", pack=False, align=None),
    "KERB_SMART_CARD_PROFILE": SimStruct(OrderedDict((("Profile", SimTypeRef("KERB_INTERACTIVE_PROFILE", SimStruct)), ("CertificateSize", SimTypeInt(signed=False, label="UInt32")), ("CertificateData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_SMART_CARD_PROFILE", pack=False, align=None),
    "KERB_SMART_CARD_UNLOCK_LOGON": SimStruct(OrderedDict((("Logon", SimTypeRef("KERB_SMART_CARD_LOGON", SimStruct)), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_SMART_CARD_UNLOCK_LOGON", pack=False, align=None),
    "KERB_SUBMIT_TKT_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeRef("KERB_CRYPTO_KEY32", SimStruct)), ("KerbCredSize", SimTypeInt(signed=False, label="UInt32")), ("KerbCredOffset", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_SUBMIT_TKT_REQUEST", pack=False, align=None),
    "KERB_TICKET_CACHE_INFO": SimStruct(OrderedDict((("ServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("RealmName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("RenewTime", SimTypeLongLong(signed=True, label="Int64")), ("EncryptionType", SimTypeInt(signed=True, label="Int32")), ("TicketFlags", SimTypeInt(signed=False, label="KERB_TICKET_FLAGS")),)), name="KERB_TICKET_CACHE_INFO", pack=False, align=None),
    "KERB_TICKET_CACHE_INFO_EX": SimStruct(OrderedDict((("ClientName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ClientRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ServerRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("RenewTime", SimTypeLongLong(signed=True, label="Int64")), ("EncryptionType", SimTypeInt(signed=True, label="Int32")), ("TicketFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_TICKET_CACHE_INFO_EX", pack=False, align=None),
    "KERB_TICKET_CACHE_INFO_EX2": SimStruct(OrderedDict((("ClientName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ClientRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ServerRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("RenewTime", SimTypeLongLong(signed=True, label="Int64")), ("EncryptionType", SimTypeInt(signed=True, label="Int32")), ("TicketFlags", SimTypeInt(signed=False, label="UInt32")), ("SessionKeyType", SimTypeInt(signed=False, label="UInt32")), ("BranchId", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_TICKET_CACHE_INFO_EX2", pack=False, align=None),
    "KERB_TICKET_CACHE_INFO_EX3": SimStruct(OrderedDict((("ClientName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ClientRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ServerName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ServerRealm", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("RenewTime", SimTypeLongLong(signed=True, label="Int64")), ("EncryptionType", SimTypeInt(signed=True, label="Int32")), ("TicketFlags", SimTypeInt(signed=False, label="UInt32")), ("SessionKeyType", SimTypeInt(signed=False, label="UInt32")), ("BranchId", SimTypeInt(signed=False, label="UInt32")), ("CacheFlags", SimTypeInt(signed=False, label="UInt32")), ("KdcCalled", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="KERB_TICKET_CACHE_INFO_EX3", pack=False, align=None),
    "KERB_TICKET_FLAGS": SimTypeInt(signed=False, label="KERB_TICKET_FLAGS"),
    "KERB_TICKET_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_LOGON_SUBMIT_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServiceTicketLength", SimTypeInt(signed=False, label="UInt32")), ("TicketGrantingTicketLength", SimTypeInt(signed=False, label="UInt32")), ("ServiceTicket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TicketGrantingTicket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KERB_TICKET_LOGON", pack=False, align=None),
    "KERB_TICKET_PROFILE": SimStruct(OrderedDict((("Profile", SimTypeRef("KERB_INTERACTIVE_PROFILE", SimStruct)), ("SessionKey", SimTypeRef("KERB_CRYPTO_KEY", SimStruct)),)), name="KERB_TICKET_PROFILE", pack=False, align=None),
    "KERB_TICKET_UNLOCK_LOGON": SimStruct(OrderedDict((("Logon", SimTypeRef("KERB_TICKET_LOGON", SimStruct)), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="KERB_TICKET_UNLOCK_LOGON", pack=False, align=None),
    "KERB_TRANSFER_CRED_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="KERB_PROTOCOL_MESSAGE_TYPE")), ("OriginLogonId", SimTypeRef("LUID", SimStruct)), ("DestinationLogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KERB_TRANSFER_CRED_REQUEST", pack=False, align=None),
    "KERNDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwKernPairNum", SimTypeInt(signed=False, label="UInt32")), ("KernPair", SimTypePointer(SimTypeRef("FD_KERNINGPAIR", SimStruct), offset=0)),)), name="KERNDATA", pack=False, align=None),
    "KERNEL_CET_CONTEXT": SimStruct(OrderedDict((("Ssp", SimTypeLongLong(signed=False, label="UInt64")), ("Rip", SimTypeLongLong(signed=False, label="UInt64")), ("SegCs", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"AllFlags": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Fill", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)),)), name="KERNEL_CET_CONTEXT", pack=False, align=None),
    "KERNEL_SOFT_RESTART_NOTIFICATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Event", SimTypeBottom(label="Guid")),)), name="KERNEL_SOFT_RESTART_NOTIFICATION", pack=False, align=None),
    "KERNEL_USER_TIMES": SimStruct(OrderedDict((("CreateTime", SimTypeLongLong(signed=True, label="Int64")), ("ExitTime", SimTypeLongLong(signed=True, label="Int64")), ("KernelTime", SimTypeLongLong(signed=True, label="Int64")), ("UserTime", SimTypeLongLong(signed=True, label="Int64")),)), name="KERNEL_USER_TIMES", pack=False, align=None),
    "KERNINGPAIR": SimStruct(OrderedDict((("wFirst", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")), ("iKernAmount", SimTypeInt(signed=True, label="Int32")),)), name="KERNINGPAIR", pack=False, align=None),
    "KEVENT": SimStruct(OrderedDict((("Header", SimTypeRef("DISPATCHER_HEADER", SimStruct)),)), name="KEVENT", pack=False, align=None),
    "KEYBDINPUT": SimStruct(OrderedDict((("wVk", SimTypeInt(signed=False, label="VIRTUAL_KEY")), ("wScan", SimTypeShort(signed=False, label="UInt16")), ("dwFlags", SimTypeInt(signed=False, label="KEYBD_EVENT_FLAGS")), ("time", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="KEYBDINPUT", pack=False, align=None),
    "KEYBD_EVENT_FLAGS": SimTypeInt(signed=False, label="KEYBD_EVENT_FLAGS"),
    "KEYBOARD_ATTRIBUTES": SimStruct(OrderedDict((("KeyboardIdentifier", SimTypeRef("KEYBOARD_ID", SimStruct)), ("KeyboardMode", SimTypeShort(signed=False, label="UInt16")), ("NumberOfFunctionKeys", SimTypeShort(signed=False, label="UInt16")), ("NumberOfIndicators", SimTypeShort(signed=False, label="UInt16")), ("NumberOfKeysTotal", SimTypeShort(signed=False, label="UInt16")), ("InputDataQueueLength", SimTypeInt(signed=False, label="UInt32")), ("KeyRepeatMinimum", SimTypeRef("KEYBOARD_TYPEMATIC_PARAMETERS", SimStruct)), ("KeyRepeatMaximum", SimTypeRef("KEYBOARD_TYPEMATIC_PARAMETERS", SimStruct)),)), name="KEYBOARD_ATTRIBUTES", pack=False, align=None),
    "KEYBOARD_EXTENDED_ATTRIBUTES": SimStruct(OrderedDict((("Version", SimTypeChar(label="Byte")), ("FormFactor", SimTypeChar(label="Byte")), ("KeyType", SimTypeChar(label="Byte")), ("PhysicalLayout", SimTypeChar(label="Byte")), ("VendorSpecificPhysicalLayout", SimTypeChar(label="Byte")), ("IETFLanguageTagIndex", SimTypeChar(label="Byte")), ("ImplementedInputAssistControls", SimTypeChar(label="Byte")),)), name="KEYBOARD_EXTENDED_ATTRIBUTES", pack=False, align=None),
    "KEYBOARD_ID": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Subtype", SimTypeChar(label="Byte")),)), name="KEYBOARD_ID", pack=False, align=None),
    "KEYBOARD_IME_STATUS": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")), ("ImeOpen", SimTypeInt(signed=False, label="UInt32")), ("ImeConvMode", SimTypeInt(signed=False, label="UInt32")),)), name="KEYBOARD_IME_STATUS", pack=False, align=None),
    "KEYBOARD_INDICATOR_PARAMETERS": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")), ("LedFlags", SimTypeShort(signed=False, label="UInt16")),)), name="KEYBOARD_INDICATOR_PARAMETERS", pack=False, align=None),
    "KEYBOARD_INDICATOR_TRANSLATION": SimStruct(OrderedDict((("NumberOfIndicatorKeys", SimTypeShort(signed=False, label="UInt16")), ("IndicatorList", SimTypePointer(SimTypeRef("INDICATOR_LIST", SimStruct), offset=0)),)), name="KEYBOARD_INDICATOR_TRANSLATION", pack=False, align=None),
    "KEYBOARD_INPUT_DATA": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")), ("MakeCode", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("ExtraInformation", SimTypeInt(signed=False, label="UInt32")),)), name="KEYBOARD_INPUT_DATA", pack=False, align=None),
    "KEYBOARD_TYPEMATIC_PARAMETERS": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")), ("Rate", SimTypeShort(signed=False, label="UInt16")), ("Delay", SimTypeShort(signed=False, label="UInt16")),)), name="KEYBOARD_TYPEMATIC_PARAMETERS", pack=False, align=None),
    "KEYBOARD_UNIT_ID_PARAMETER": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")),)), name="KEYBOARD_UNIT_ID_PARAMETER", pack=False, align=None),
    "KEYMODIFIER": SimTypeInt(signed=False, label="KEYMODIFIER"),
    "KEYMODIFIERS": SimTypeInt(signed=False, label="KEYMODIFIERS"),
    "KEY_BASIC_INFORMATION": SimStruct(OrderedDict((("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KEY_BASIC_INFORMATION", pack=False, align=None),
    "KEY_CACHED_INFORMATION": SimStruct(OrderedDict((("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("SubKeys", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLen", SimTypeInt(signed=False, label="UInt32")), ("Values", SimTypeInt(signed=False, label="UInt32")), ("MaxValueNameLen", SimTypeInt(signed=False, label="UInt32")), ("MaxValueDataLen", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_CACHED_INFORMATION", pack=False, align=None),
    "KEY_CONTROL_FLAGS_INFORMATION": SimStruct(OrderedDict((("ControlFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_CONTROL_FLAGS_INFORMATION", pack=False, align=None),
    "KEY_EVENT_RECORD": SimStruct(OrderedDict((("bKeyDown", SimTypeInt(signed=True, label="Int32")), ("wRepeatCount", SimTypeShort(signed=False, label="UInt16")), ("wVirtualKeyCode", SimTypeShort(signed=False, label="UInt16")), ("wVirtualScanCode", SimTypeShort(signed=False, label="UInt16")), ("uChar", SimUnion({"UnicodeChar": SimTypeChar(label="Char"), "AsciiChar": SimTypeChar(label="SByte")}, name="<anon>", label="None")), ("dwControlKeyState", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_EVENT_RECORD", pack=False, align=None),
    "KEY_FULL_INFORMATION": SimStruct(OrderedDict((("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("ClassOffset", SimTypeInt(signed=False, label="UInt32")), ("ClassLength", SimTypeInt(signed=False, label="UInt32")), ("SubKeys", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLen", SimTypeInt(signed=False, label="UInt32")), ("MaxClassLen", SimTypeInt(signed=False, label="UInt32")), ("Values", SimTypeInt(signed=False, label="UInt32")), ("MaxValueNameLen", SimTypeInt(signed=False, label="UInt32")), ("MaxValueDataLen", SimTypeInt(signed=False, label="UInt32")), ("Class", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KEY_FULL_INFORMATION", pack=False, align=None),
    "KEY_INFORMATION_CLASS": SimTypeInt(signed=False, label="KEY_INFORMATION_CLASS"),
    "KEY_LAYER_INFORMATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_LAYER_INFORMATION", pack=False, align=None),
    "KEY_NAME_INFORMATION": SimStruct(OrderedDict((("NameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KEY_NAME_INFORMATION", pack=False, align=None),
    "KEY_NODE_INFORMATION": SimStruct(OrderedDict((("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("ClassOffset", SimTypeInt(signed=False, label="UInt32")), ("ClassLength", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KEY_NODE_INFORMATION", pack=False, align=None),
    "KEY_SET_INFORMATION_CLASS": SimTypeInt(signed=False, label="KEY_SET_INFORMATION_CLASS"),
    "KEY_SET_VIRTUALIZATION_INFORMATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_SET_VIRTUALIZATION_INFORMATION", pack=False, align=None),
    "KEY_TRUST_INFORMATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_TRUST_INFORMATION", pack=False, align=None),
    "KEY_TYPE_SUBTYPE": SimStruct(OrderedDict((("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeBottom(label="Guid")), ("Subtype", SimTypeBottom(label="Guid")),)), name="KEY_TYPE_SUBTYPE", pack=False, align=None),
    "KEY_VALUE_BASIC_INFORMATION": SimStruct(OrderedDict((("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KEY_VALUE_BASIC_INFORMATION", pack=False, align=None),
    "KEY_VALUE_ENTRY": SimStruct(OrderedDict((("ValueName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_VALUE_ENTRY", pack=False, align=None),
    "KEY_VALUE_FULL_INFORMATION": SimStruct(OrderedDict((("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KEY_VALUE_FULL_INFORMATION", pack=False, align=None),
    "KEY_VALUE_INFORMATION_CLASS": SimTypeInt(signed=False, label="KEY_VALUE_INFORMATION_CLASS"),
    "KEY_VALUE_LAYER_INFORMATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_VALUE_LAYER_INFORMATION", pack=False, align=None),
    "KEY_VALUE_PARTIAL_INFORMATION": SimStruct(OrderedDict((("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KEY_VALUE_PARTIAL_INFORMATION", pack=False, align=None),
    "KEY_VALUE_PARTIAL_INFORMATION_ALIGN64": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KEY_VALUE_PARTIAL_INFORMATION_ALIGN64", pack=False, align=None),
    "KEY_VIRTUALIZATION_INFORMATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_VIRTUALIZATION_INFORMATION", pack=False, align=None),
    "KEY_WOW64_FLAGS_INFORMATION": SimStruct(OrderedDict((("UserFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KEY_WOW64_FLAGS_INFORMATION", pack=False, align=None),
    "KEY_WRITE_TIME_INFORMATION": SimStruct(OrderedDict((("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")),)), name="KEY_WRITE_TIME_INFORMATION", pack=False, align=None),
    "KE_PROCESSOR_CHANGE_NOTIFY_CONTEXT": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="KE_PROCESSOR_CHANGE_NOTIFY_STATE")), ("NtNumber", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")), ("ProcNumber", SimTypeRef("PROCESSOR_NUMBER", SimStruct)),)), name="KE_PROCESSOR_CHANGE_NOTIFY_CONTEXT", pack=False, align=None),
    "KE_PROCESSOR_CHANGE_NOTIFY_STATE": SimTypeInt(signed=False, label="KE_PROCESSOR_CHANGE_NOTIFY_STATE"),
    "KFLOATING_SAVE": SimStruct(OrderedDict((("ControlWord", SimTypeInt(signed=False, label="UInt32")), ("StatusWord", SimTypeInt(signed=False, label="UInt32")), ("ErrorOffset", SimTypeInt(signed=False, label="UInt32")), ("ErrorSelector", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSelector", SimTypeInt(signed=False, label="UInt32")), ("Spare0", SimTypeInt(signed=False, label="UInt32")), ("Spare1", SimTypeInt(signed=False, label="UInt32")),)), name="KFLOATING_SAVE", pack=False, align=None),
    "KF_CATEGORY": SimTypeInt(signed=False, label="KF_CATEGORY"),
    "KGATE": SimStruct(OrderedDict((("Header", SimTypeRef("DISPATCHER_HEADER", SimStruct)),)), name="KGATE", pack=False, align=None),
    "KGDT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KHYPH": SimTypeInt(signed=False, label="KHYPH"),
    "KIDT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KINTERRUPT_MODE": SimTypeInt(signed=False, label="KINTERRUPT_MODE"),
    "KINTERRUPT_POLARITY": SimTypeInt(signed=False, label="KINTERRUPT_POLARITY"),
    "KIPI_BROADCAST_WORKER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["Argument"]), offset=0),
    "KLOCK_QUEUE_HANDLE": SimStruct(OrderedDict((("LockQueue", SimTypeRef("KSPIN_LOCK_QUEUE", SimStruct)), ("OldIrql", SimTypeChar(label="Byte")),)), name="KLOCK_QUEUE_HANDLE", pack=False, align=None),
    "KMESSAGE_SERVICE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["Interrupt", "ServiceContext", "MessageID"]), offset=0),
    "KMTQUERYADAPTERINFOTYPE": SimTypeInt(signed=False, label="KMTQUERYADAPTERINFOTYPE"),
    "KMTUMDVERSION": SimTypeInt(signed=False, label="KMTUMDVERSION"),
    "KMT_DISPLAY_UMD_VERSION": SimTypeInt(signed=False, label="KMT_DISPLAY_UMD_VERSION"),
    "KMUTANT": SimStruct(OrderedDict((("Header", SimTypeRef("DISPATCHER_HEADER", SimStruct)), ("MutantListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("OwnerThread", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("Anonymous", SimUnion({"MutantFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("ApcDisable", SimTypeChar(label="Byte")),)), name="KMUTANT", pack=False, align=None),
    "KNONVOLATILE_CONTEXT_POINTERS": SimStruct(OrderedDict((("Dummy", SimTypeInt(signed=False, label="UInt32")),)), name="KNONVOLATILE_CONTEXT_POINTERS", pack=False, align=None),
    "KNONVOLATILE_CONTEXT_POINTERS_ARM64": SimStruct(OrderedDict((("X19", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X20", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X21", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X22", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X23", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X24", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X25", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X26", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X27", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("X28", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("Fp", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("Lr", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D8", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D9", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D10", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D11", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D12", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D13", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D14", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("D15", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="KNONVOLATILE_CONTEXT_POINTERS_ARM64", pack=False, align=None),
    "KNOWLEDGE_COOKIE_COMPARISON_RESULT": SimTypeInt(signed=False, label="KNOWLEDGE_COOKIE_COMPARISON_RESULT"),
    "KNOWNDESTCATEGORY": SimTypeInt(signed=False, label="KNOWNDESTCATEGORY"),
    "KNOWNFOLDER_DEFINITION": SimStruct(OrderedDict((("category", SimTypeInt(signed=False, label="KF_CATEGORY")), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fidParent", SimTypeBottom(label="Guid")), ("pszRelativePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszParsingName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszTooltip", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLocalizedName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszIcon", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSecurity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAttributes", SimTypeInt(signed=False, label="UInt32")), ("kfdFlags", SimTypeInt(signed=False, label="UInt32")), ("ftidType", SimTypeBottom(label="Guid")),)), name="KNOWNFOLDER_DEFINITION", pack=False, align=None),
    "KNOWN_FOLDER_FLAG": SimTypeInt(signed=False, label="KNOWN_FOLDER_FLAG"),
    "KPCR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KPRCB": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KPROFILE_SOURCE": SimTypeInt(signed=False, label="KPROFILE_SOURCE"),
    "KQUEUE": SimStruct(OrderedDict((("Header", SimTypeRef("DISPATCHER_HEADER", SimStruct)), ("EntryListHead", SimTypeRef("LIST_ENTRY", SimStruct)), ("CurrentCount", SimTypeInt(signed=False, label="UInt32")), ("MaximumCount", SimTypeInt(signed=False, label="UInt32")), ("ThreadListHead", SimTypeRef("LIST_ENTRY", SimStruct)),)), name="KQUEUE", pack=False, align=None),
    "KRESOURCEMANAGER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KSAC3_ALTERNATE_AUDIO": SimStruct(OrderedDict((("fStereo", SimTypeInt(signed=True, label="Int32")), ("DualMode", SimTypeInt(signed=False, label="UInt32")),)), name="KSAC3_ALTERNATE_AUDIO", pack=False, align=None),
    "KSAC3_BIT_STREAM_MODE": SimStruct(OrderedDict((("BitStreamMode", SimTypeInt(signed=True, label="Int32")),)), name="KSAC3_BIT_STREAM_MODE", pack=False, align=None),
    "KSAC3_DIALOGUE_LEVEL": SimStruct(OrderedDict((("DialogueLevel", SimTypeInt(signed=False, label="UInt32")),)), name="KSAC3_DIALOGUE_LEVEL", pack=False, align=None),
    "KSAC3_DOWNMIX": SimStruct(OrderedDict((("fDownMix", SimTypeInt(signed=True, label="Int32")), ("fDolbySurround", SimTypeInt(signed=True, label="Int32")),)), name="KSAC3_DOWNMIX", pack=False, align=None),
    "KSAC3_ERROR_CONCEALMENT": SimStruct(OrderedDict((("fRepeatPreviousBlock", SimTypeInt(signed=True, label="Int32")), ("fErrorInCurrentBlock", SimTypeInt(signed=True, label="Int32")),)), name="KSAC3_ERROR_CONCEALMENT", pack=False, align=None),
    "KSAC3_ROOM_TYPE": SimStruct(OrderedDict((("fLargeRoom", SimTypeInt(signed=True, label="Int32")),)), name="KSAC3_ROOM_TYPE", pack=False, align=None),
    "KSALLOCATORMODE": SimTypeInt(signed=False, label="KSALLOCATORMODE"),
    "KSALLOCATOR_FRAMING": SimStruct(OrderedDict((("Anonymous1", SimUnion({"OptionsFlags": SimTypeInt(signed=False, label="UInt32"), "RequirementsFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("PoolType", SimTypeInt(signed=False, label="UInt32")), ("Frames", SimTypeInt(signed=False, label="UInt32")), ("FrameSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"FileAlignment": SimTypeInt(signed=False, label="UInt32"), "FramePitch": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSALLOCATOR_FRAMING", pack=False, align=None),
    "KSALLOCATOR_FRAMING_EX": SimStruct(OrderedDict((("CountItems", SimTypeInt(signed=False, label="UInt32")), ("PinFlags", SimTypeInt(signed=False, label="UInt32")), ("OutputCompression", SimTypeRef("KS_COMPRESSION", SimStruct)), ("PinWeight", SimTypeInt(signed=False, label="UInt32")), ("FramingItem", SimTypePointer(SimTypeRef("KS_FRAMING_ITEM", SimStruct), offset=0)),)), name="KSALLOCATOR_FRAMING_EX", pack=False, align=None),
    "KSATTRIBUTE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Attribute", SimTypeBottom(label="Guid")),)), name="KSATTRIBUTE", pack=False, align=None),
    "KSATTRIBUTE_AUDIOSIGNALPROCESSING_MODE": SimStruct(OrderedDict((("AttributeHeader", SimTypeRef("KSATTRIBUTE", SimStruct)), ("SignalProcessingMode", SimTypeBottom(label="Guid")),)), name="KSATTRIBUTE_AUDIOSIGNALPROCESSING_MODE", pack=False, align=None),
    "KSAUDIOENGINE_BUFFER_SIZE_RANGE": SimStruct(OrderedDict((("MinBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("MaxBufferBytes", SimTypeInt(signed=False, label="UInt32")),)), name="KSAUDIOENGINE_BUFFER_SIZE_RANGE", pack=False, align=None),
    "KSAUDIOENGINE_DESCRIPTOR": SimStruct(OrderedDict((("nHostPinId", SimTypeInt(signed=False, label="UInt32")), ("nOffloadPinId", SimTypeInt(signed=False, label="UInt32")), ("nLoopbackPinId", SimTypeInt(signed=False, label="UInt32")),)), name="KSAUDIOENGINE_DESCRIPTOR", pack=False, align=None),
    "KSAUDIOENGINE_DEVICECONTROLS": SimStruct(OrderedDict((("Volume", SimTypeInt(signed=False, label="EDeviceControlUseType")), ("Mute", SimTypeInt(signed=False, label="EDeviceControlUseType")), ("PeakMeter", SimTypeInt(signed=False, label="EDeviceControlUseType")),)), name="KSAUDIOENGINE_DEVICECONTROLS", pack=False, align=None),
    "KSAUDIOENGINE_VOLUMELEVEL": SimStruct(OrderedDict((("TargetVolume", SimTypeInt(signed=True, label="Int32")), ("CurveType", SimTypeInt(signed=False, label="AUDIO_CURVE_TYPE")), ("CurveDuration", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSAUDIOENGINE_VOLUMELEVEL", pack=False, align=None),
    "KSAUDIOMODULE_DESCRIPTOR": SimStruct(OrderedDict((("ClassId", SimTypeBottom(label="Guid")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("VersionMajor", SimTypeInt(signed=False, label="UInt32")), ("VersionMinor", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="KSAUDIOMODULE_DESCRIPTOR", pack=False, align=None),
    "KSAUDIOMODULE_NOTIFICATION": SimStruct(OrderedDict((("Anonymous", SimUnion({"ProviderId": SimTypeRef("_ProviderId_e__Struct", SimStruct), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="KSAUDIOMODULE_NOTIFICATION", pack=False, align=None),
    "KSAUDIOMODULE_PROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ClassId", SimTypeBottom(label="Guid")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")),)), name="KSAUDIOMODULE_PROPERTY", pack=False, align=None),
    "KSAUDIO_CHANNEL_CONFIG": SimStruct(OrderedDict((("ActiveSpeakerPositions", SimTypeInt(signed=True, label="Int32")),)), name="KSAUDIO_CHANNEL_CONFIG", pack=False, align=None),
    "KSAUDIO_COPY_PROTECTION": SimStruct(OrderedDict((("fCopyrighted", SimTypeInt(signed=True, label="Int32")), ("fOriginal", SimTypeInt(signed=True, label="Int32")),)), name="KSAUDIO_COPY_PROTECTION", pack=False, align=None),
    "KSAUDIO_DYNAMIC_RANGE": SimStruct(OrderedDict((("QuietCompression", SimTypeInt(signed=False, label="UInt32")), ("LoudCompression", SimTypeInt(signed=False, label="UInt32")),)), name="KSAUDIO_DYNAMIC_RANGE", pack=False, align=None),
    "KSAUDIO_MICROPHONE_COORDINATES": SimStruct(OrderedDict((("usType", SimTypeShort(signed=False, label="UInt16")), ("wXCoord", SimTypeShort(signed=True, label="Int16")), ("wYCoord", SimTypeShort(signed=True, label="Int16")), ("wZCoord", SimTypeShort(signed=True, label="Int16")), ("wVerticalAngle", SimTypeShort(signed=True, label="Int16")), ("wHorizontalAngle", SimTypeShort(signed=True, label="Int16")),)), name="KSAUDIO_MICROPHONE_COORDINATES", pack=False, align=None),
    "KSAUDIO_MIC_ARRAY_GEOMETRY": SimStruct(OrderedDict((("usVersion", SimTypeShort(signed=False, label="UInt16")), ("usMicArrayType", SimTypeShort(signed=False, label="UInt16")), ("wVerticalAngleBegin", SimTypeShort(signed=True, label="Int16")), ("wVerticalAngleEnd", SimTypeShort(signed=True, label="Int16")), ("wHorizontalAngleBegin", SimTypeShort(signed=True, label="Int16")), ("wHorizontalAngleEnd", SimTypeShort(signed=True, label="Int16")), ("usFrequencyBandLo", SimTypeShort(signed=False, label="UInt16")), ("usFrequencyBandHi", SimTypeShort(signed=False, label="UInt16")), ("usNumberOfMicrophones", SimTypeShort(signed=False, label="UInt16")), ("KsMicCoord", SimTypePointer(SimTypeRef("KSAUDIO_MICROPHONE_COORDINATES", SimStruct), offset=0)),)), name="KSAUDIO_MIC_ARRAY_GEOMETRY", pack=False, align=None),
    "KSAUDIO_MIXCAP_TABLE": SimStruct(OrderedDict((("InputChannels", SimTypeInt(signed=False, label="UInt32")), ("OutputChannels", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypePointer(SimTypeRef("KSAUDIO_MIX_CAPS", SimStruct), offset=0)),)), name="KSAUDIO_MIXCAP_TABLE", pack=False, align=None),
    "KSAUDIO_MIXLEVEL": SimStruct(OrderedDict((("Mute", SimTypeInt(signed=True, label="Int32")), ("Level", SimTypeInt(signed=True, label="Int32")),)), name="KSAUDIO_MIXLEVEL", pack=False, align=None),
    "KSAUDIO_MIX_CAPS": SimStruct(OrderedDict((("Mute", SimTypeInt(signed=True, label="Int32")), ("Minimum", SimTypeInt(signed=True, label="Int32")), ("Maximum", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"Reset": SimTypeInt(signed=True, label="Int32"), "Resolution": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="KSAUDIO_MIX_CAPS", pack=False, align=None),
    "KSAUDIO_PACKETSIZE_CONSTRAINTS": SimStruct(OrderedDict((("MinPacketPeriodInHns", SimTypeInt(signed=False, label="UInt32")), ("PacketSizeFileAlignment", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("NumProcessingModeConstraints", SimTypeInt(signed=False, label="UInt32")), ("ProcessingModeConstraints", SimTypePointer(SimTypeRef("KSAUDIO_PACKETSIZE_PROCESSINGMODE_CONSTRAINT", SimStruct), offset=0)),)), name="KSAUDIO_PACKETSIZE_CONSTRAINTS", pack=False, align=None),
    "KSAUDIO_PACKETSIZE_CONSTRAINTS2": SimStruct(OrderedDict((("MinPacketPeriodInHns", SimTypeInt(signed=False, label="UInt32")), ("PacketSizeFileAlignment", SimTypeInt(signed=False, label="UInt32")), ("MaxPacketSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("NumProcessingModeConstraints", SimTypeInt(signed=False, label="UInt32")), ("ProcessingModeConstraints", SimTypePointer(SimTypeRef("KSAUDIO_PACKETSIZE_PROCESSINGMODE_CONSTRAINT", SimStruct), offset=0)),)), name="KSAUDIO_PACKETSIZE_CONSTRAINTS2", pack=False, align=None),
    "KSAUDIO_PACKETSIZE_PROCESSINGMODE_CONSTRAINT": SimStruct(OrderedDict((("ProcessingMode", SimTypeBottom(label="Guid")), ("SamplesPerProcessingPacket", SimTypeInt(signed=False, label="UInt32")), ("ProcessingPacketDurationInHns", SimTypeInt(signed=False, label="UInt32")),)), name="KSAUDIO_PACKETSIZE_PROCESSINGMODE_CONSTRAINT", pack=False, align=None),
    "KSAUDIO_POSITION": SimStruct(OrderedDict((("PlayOffset", SimTypeLongLong(signed=False, label="UInt64")), ("WriteOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSAUDIO_POSITION", pack=False, align=None),
    "KSAUDIO_POSITIONEX": SimStruct(OrderedDict((("TimerFrequency", SimTypeLongLong(signed=True, label="Int64")), ("TimeStamp1", SimTypeLongLong(signed=True, label="Int64")), ("Position", SimTypeRef("KSAUDIO_POSITION", SimStruct)), ("TimeStamp2", SimTypeLongLong(signed=True, label="Int64")),)), name="KSAUDIO_POSITIONEX", pack=False, align=None),
    "KSAUDIO_PRESENTATION_POSITION": SimStruct(OrderedDict((("u64PositionInBlocks", SimTypeLongLong(signed=False, label="UInt64")), ("u64QPCPosition", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSAUDIO_PRESENTATION_POSITION", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_BACKGROUNDSEGMENTATION_CONFIGCAPS": SimStruct(OrderedDict((("Resolution", SimTypeRef("SIZE", SimStruct)), ("MaxFrameRate", SimTypeRef("_MaxFrameRate_e__Struct", SimStruct)), ("MaskResolution", SimTypeRef("SIZE", SimStruct)), ("SubType", SimTypeBottom(label="Guid")),)), name="KSCAMERA_EXTENDEDPROP_BACKGROUNDSEGMENTATION_CONFIGCAPS", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_CAMERAOFFSET": SimStruct(OrderedDict((("PitchAngle", SimTypeInt(signed=True, label="Int32")), ("YawAngle", SimTypeInt(signed=True, label="Int32")), ("Flag", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_CAMERAOFFSET", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_CONFIGCAPS": SimStruct(OrderedDict((("ResolutionX", SimTypeInt(signed=True, label="Int32")), ("ResolutionY", SimTypeInt(signed=True, label="Int32")), ("PorchTop", SimTypeInt(signed=True, label="Int32")), ("PorchLeft", SimTypeInt(signed=True, label="Int32")), ("PorchBottom", SimTypeInt(signed=True, label="Int32")), ("PorchRight", SimTypeInt(signed=True, label="Int32")), ("NonUpscalingWindowSize", SimTypeInt(signed=True, label="Int32")), ("MinWindowSize", SimTypeInt(signed=True, label="Int32")), ("MaxWindowSize", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeInt(signed=True, label="Int32")),)), name="KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_CONFIGCAPS", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_CONFIGCAPSHEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_CONFIGCAPSHEADER", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_SETTING": SimStruct(OrderedDict((("OriginX", SimTypeInt(signed=True, label="Int32")), ("OriginY", SimTypeInt(signed=True, label="Int32")), ("WindowSize", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_SETTING", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_EVCOMPENSATION": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="UInt32")), ("Min", SimTypeInt(signed=True, label="Int32")), ("Max", SimTypeInt(signed=True, label="Int32")), ("Value", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_EVCOMPENSATION", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_FIELDOFVIEW": SimStruct(OrderedDict((("NormalizedFocalLengthX", SimTypeInt(signed=False, label="UInt32")), ("NormalizedFocalLengthY", SimTypeInt(signed=False, label="UInt32")), ("Flag", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_FIELDOFVIEW", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_FOCUSSTATE": SimTypeInt(signed=False, label="KSCAMERA_EXTENDEDPROP_FOCUSSTATE"),
    "KSCAMERA_EXTENDEDPROP_HEADER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("PinId", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Result", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Capability", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_HEADER", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_METADATAINFO": SimStruct(OrderedDict((("BufferAlignment", SimTypeInt(signed=True, label="Int32")), ("MaxMetadataBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_METADATAINFO", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_MetadataAlignment": SimTypeInt(signed=False, label="KSCAMERA_EXTENDEDPROP_MetadataAlignment"),
    "KSCAMERA_EXTENDEDPROP_PHOTOMODE": SimStruct(OrderedDict((("RequestedHistoryFrames", SimTypeInt(signed=False, label="UInt32")), ("MaxHistoryFrames", SimTypeInt(signed=False, label="UInt32")), ("SubMode", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_PHOTOMODE", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_PROFILE": SimStruct(OrderedDict((("ProfileId", SimTypeBottom(label="Guid")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_PROFILE", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROITYPE": SimTypeInt(signed=False, label="KSCAMERA_EXTENDEDPROP_ROITYPE"),
    "KSCAMERA_EXTENDEDPROP_ROI_CONFIGCAPS": SimStruct(OrderedDict((("ControlId", SimTypeInt(signed=False, label="UInt32")), ("MaxNumberOfROIs", SimTypeInt(signed=False, label="UInt32")), ("Capability", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_ROI_CONFIGCAPS", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_CONFIGCAPSHEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ConfigCapCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_ROI_CONFIGCAPSHEADER", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_EXPOSURE": SimStruct(OrderedDict((("ROIInfo", SimTypeRef("KSCAMERA_EXTENDEDPROP_ROI_INFO", SimStruct)), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_ROI_EXPOSURE", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_FOCUS": SimStruct(OrderedDict((("ROIInfo", SimTypeRef("KSCAMERA_EXTENDEDPROP_ROI_INFO", SimStruct)), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_ROI_FOCUS", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_INFO": SimStruct(OrderedDict((("Region", SimTypeRef("RECT", SimStruct)), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Weight", SimTypeInt(signed=True, label="Int32")), ("RegionOfInterestType", SimTypeInt(signed=True, label="Int32")),)), name="KSCAMERA_EXTENDEDPROP_ROI_INFO", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_ISPCONTROL": SimStruct(OrderedDict((("ControlId", SimTypeInt(signed=False, label="UInt32")), ("ROICount", SimTypeInt(signed=False, label="UInt32")), ("Result", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_EXTENDEDPROP_ROI_ISPCONTROL", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_ISPCONTROLHEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ControlCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_ROI_ISPCONTROLHEADER", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_ROI_WHITEBALANCE": SimStruct(OrderedDict((("ROIInfo", SimTypeRef("KSCAMERA_EXTENDEDPROP_ROI_INFO", SimStruct)), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_ROI_WHITEBALANCE", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_VALUE": SimStruct(OrderedDict((("Value", SimUnion({"dbl": SimTypeFloat(size=64), "ull": SimTypeLongLong(signed=False, label="UInt64"), "ul": SimTypeInt(signed=False, label="UInt32"), "ratio": SimTypeLongLong(signed=False, label="UInt64"), "l": SimTypeInt(signed=True, label="Int32"), "ll": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="KSCAMERA_EXTENDEDPROP_VALUE", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_VIDEOPROCSETTING": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="UInt32")), ("Min", SimTypeInt(signed=True, label="Int32")), ("Max", SimTypeInt(signed=True, label="Int32")), ("Step", SimTypeInt(signed=True, label="Int32")), ("VideoProc", SimTypeRef("KSCAMERA_EXTENDEDPROP_VALUE", SimStruct)), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_EXTENDEDPROP_VIDEOPROCSETTING", pack=False, align=None),
    "KSCAMERA_EXTENDEDPROP_WBPRESET": SimTypeInt(signed=False, label="KSCAMERA_EXTENDEDPROP_WBPRESET"),
    "KSCAMERA_EXTENDEDPROP_WHITEBALANCE_MODE": SimTypeInt(signed=False, label="KSCAMERA_EXTENDEDPROP_WHITEBALANCE_MODE"),
    "KSCAMERA_MAXVIDEOFPS_FORPHOTORES": SimStruct(OrderedDict((("PhotoResWidth", SimTypeInt(signed=False, label="UInt32")), ("PhotoResHeight", SimTypeInt(signed=False, label="UInt32")), ("PreviewFPSNum", SimTypeInt(signed=False, label="UInt32")), ("PreviewFPSDenom", SimTypeInt(signed=False, label="UInt32")), ("CaptureFPSNum", SimTypeInt(signed=False, label="UInt32")), ("CaptureFPSDenom", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_MAXVIDEOFPS_FORPHOTORES", pack=False, align=None),
    "KSCAMERA_METADATA_BACKGROUNDSEGMENTATIONMASK": SimStruct(OrderedDict((("Header", SimTypeRef("KSCAMERA_METADATA_ITEMHEADER", SimStruct)), ("MaskCoverageBoundingBox", SimTypeRef("RECT", SimStruct)), ("MaskResolution", SimTypeRef("SIZE", SimStruct)), ("ForegroundBoundingBox", SimTypeRef("RECT", SimStruct)), ("MaskData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSCAMERA_METADATA_BACKGROUNDSEGMENTATIONMASK", pack=False, align=None),
    "KSCAMERA_METADATA_CAPTURESTATS": SimStruct(OrderedDict((("Header", SimTypeRef("KSCAMERA_METADATA_ITEMHEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ExposureTime", SimTypeLongLong(signed=False, label="UInt64")), ("ExposureCompensationFlags", SimTypeLongLong(signed=False, label="UInt64")), ("ExposureCompensationValue", SimTypeInt(signed=True, label="Int32")), ("IsoSpeed", SimTypeInt(signed=False, label="UInt32")), ("FocusState", SimTypeInt(signed=False, label="UInt32")), ("LensPosition", SimTypeInt(signed=False, label="UInt32")), ("WhiteBalance", SimTypeInt(signed=False, label="UInt32")), ("Flash", SimTypeInt(signed=False, label="UInt32")), ("FlashPower", SimTypeInt(signed=False, label="UInt32")), ("ZoomFactor", SimTypeInt(signed=False, label="UInt32")), ("SceneMode", SimTypeLongLong(signed=False, label="UInt64")), ("SensorFramerate", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_METADATA_CAPTURESTATS", pack=False, align=None),
    "KSCAMERA_METADATA_DIGITALWINDOW": SimStruct(OrderedDict((("Header", SimTypeRef("KSCAMERA_METADATA_ITEMHEADER", SimStruct)), ("Window", SimTypeRef("KSCAMERA_EXTENDEDPROP_DIGITALWINDOW_SETTING", SimStruct)),)), name="KSCAMERA_METADATA_DIGITALWINDOW", pack=False, align=None),
    "KSCAMERA_METADATA_FRAMEILLUMINATION": SimStruct(OrderedDict((("Header", SimTypeRef("KSCAMERA_METADATA_ITEMHEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_METADATA_FRAMEILLUMINATION", pack=False, align=None),
    "KSCAMERA_METADATA_ITEMHEADER": SimStruct(OrderedDict((("MetadataId", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_METADATA_ITEMHEADER", pack=False, align=None),
    "KSCAMERA_METADATA_PHOTOCONFIRMATION": SimStruct(OrderedDict((("Header", SimTypeRef("KSCAMERA_METADATA_ITEMHEADER", SimStruct)), ("PhotoConfirmationIndex", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_METADATA_PHOTOCONFIRMATION", pack=False, align=None),
    "KSCAMERA_MetadataId": SimTypeInt(signed=False, label="KSCAMERA_MetadataId"),
    "KSCAMERA_PERFRAMESETTING_CAP_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ItemCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_PERFRAMESETTING_CAP_HEADER", pack=False, align=None),
    "KSCAMERA_PERFRAMESETTING_CAP_ITEM_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_PERFRAMESETTING_CAP_ITEM_HEADER", pack=False, align=None),
    "KSCAMERA_PERFRAMESETTING_CUSTOM_ITEM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeBottom(label="Guid")),)), name="KSCAMERA_PERFRAMESETTING_CUSTOM_ITEM", pack=False, align=None),
    "KSCAMERA_PERFRAMESETTING_FRAME_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeInt(signed=False, label="UInt32")), ("ItemCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_PERFRAMESETTING_FRAME_HEADER", pack=False, align=None),
    "KSCAMERA_PERFRAMESETTING_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("FrameCount", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeBottom(label="Guid")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("LoopCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_PERFRAMESETTING_HEADER", pack=False, align=None),
    "KSCAMERA_PERFRAMESETTING_ITEM_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSCAMERA_PERFRAMESETTING_ITEM_HEADER", pack=False, align=None),
    "KSCAMERA_PERFRAMESETTING_ITEM_TYPE": SimTypeInt(signed=False, label="KSCAMERA_PERFRAMESETTING_ITEM_TYPE"),
    "KSCAMERA_PROFILE_CONCURRENCYINFO": SimStruct(OrderedDict((("ReferenceGuid", SimTypeBottom(label="Guid")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ProfileCount", SimTypeInt(signed=False, label="UInt32")), ("Profiles", SimTypePointer(SimTypeRef("KSCAMERA_PROFILE_INFO", SimStruct), offset=0)),)), name="KSCAMERA_PROFILE_CONCURRENCYINFO", pack=False, align=None),
    "KSCAMERA_PROFILE_INFO": SimStruct(OrderedDict((("ProfileId", SimTypeBottom(label="Guid")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("PinCount", SimTypeInt(signed=False, label="UInt32")), ("Pins", SimTypePointer(SimTypeRef("KSCAMERA_PROFILE_PININFO", SimStruct), offset=0)),)), name="KSCAMERA_PROFILE_INFO", pack=False, align=None),
    "KSCAMERA_PROFILE_MEDIAINFO": SimStruct(OrderedDict((("Resolution", SimTypeRef("_Resolution_e__Struct", SimStruct)), ("MaxFrameRate", SimTypeRef("_MaxFrameRate_e__Struct", SimStruct)), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Data0", SimTypeInt(signed=False, label="UInt32")), ("Data1", SimTypeInt(signed=False, label="UInt32")), ("Data2", SimTypeInt(signed=False, label="UInt32")), ("Data3", SimTypeInt(signed=False, label="UInt32")),)), name="KSCAMERA_PROFILE_MEDIAINFO", pack=False, align=None),
    "KSCAMERA_PROFILE_PININFO": SimStruct(OrderedDict((("PinCategory", SimTypeBottom(label="Guid")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("PinIndex", SimTypeShort(signed=False, label="UInt16")), ("ProfileSensorType", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Reserved": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MediaInfoCount", SimTypeInt(signed=False, label="UInt32")), ("MediaInfos", SimTypePointer(SimTypeRef("KSCAMERA_PROFILE_MEDIAINFO", SimStruct), offset=0)),)), name="KSCAMERA_PROFILE_PININFO", pack=False, align=None),
    "KSCLOCK_CREATE": SimStruct(OrderedDict((("CreateFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KSCLOCK_CREATE", pack=False, align=None),
    "KSCOMPONENTID": SimStruct(OrderedDict((("Manufacturer", SimTypeBottom(label="Guid")), ("Product", SimTypeBottom(label="Guid")), ("Component", SimTypeBottom(label="Guid")), ("Name", SimTypeBottom(label="Guid")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")),)), name="KSCOMPONENTID", pack=False, align=None),
    "KSCORRELATED_TIME": SimStruct(OrderedDict((("Time", SimTypeLongLong(signed=True, label="Int64")), ("SystemTime", SimTypeLongLong(signed=True, label="Int64")),)), name="KSCORRELATED_TIME", pack=False, align=None),
    "KSDATAFORMAT": SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"),
    "KSDATARANGE_AUDIO": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("MaximumChannels", SimTypeInt(signed=False, label="UInt32")), ("MinimumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MaximumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MinimumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaximumSampleFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="KSDATARANGE_AUDIO", pack=False, align=None),
    "KSDATARANGE_MUSIC": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("Technology", SimTypeBottom(label="Guid")), ("Channels", SimTypeInt(signed=False, label="UInt32")), ("Notes", SimTypeInt(signed=False, label="UInt32")), ("ChannelMask", SimTypeInt(signed=False, label="UInt32")),)), name="KSDATARANGE_MUSIC", pack=False, align=None),
    "KSDEGRADE_STANDARD": SimTypeInt(signed=False, label="KSDEGRADE_STANDARD"),
    "KSDEVICE_PROFILE_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Camera": SimTypeRef("_Camera_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="KSDEVICE_PROFILE_INFO", pack=False, align=None),
    "KSDEVICE_THERMAL_STATE": SimTypeInt(signed=False, label="KSDEVICE_THERMAL_STATE"),
    "KSDISPLAYCHANGE": SimStruct(OrderedDict((("PelsWidth", SimTypeInt(signed=False, label="UInt32")), ("PelsHeight", SimTypeInt(signed=False, label="UInt32")), ("BitsPerPel", SimTypeInt(signed=False, label="UInt32")), ("DeviceID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KSDISPLAYCHANGE", pack=False, align=None),
    "KSDS3D_BUFFER_ALL": SimStruct(OrderedDict((("Position", SimTypeRef("DS3DVECTOR", SimStruct)), ("Velocity", SimTypeRef("DS3DVECTOR", SimStruct)), ("InsideConeAngle", SimTypeInt(signed=False, label="UInt32")), ("OutsideConeAngle", SimTypeInt(signed=False, label="UInt32")), ("ConeOrientation", SimTypeRef("DS3DVECTOR", SimStruct)), ("ConeOutsideVolume", SimTypeInt(signed=True, label="Int32")), ("MinDistance", SimTypeFloat(size=32)), ("MaxDistance", SimTypeFloat(size=32)), ("Mode", SimTypeInt(signed=False, label="UInt32")),)), name="KSDS3D_BUFFER_ALL", pack=False, align=None),
    "KSDS3D_BUFFER_CONE_ANGLES": SimStruct(OrderedDict((("InsideConeAngle", SimTypeInt(signed=False, label="UInt32")), ("OutsideConeAngle", SimTypeInt(signed=False, label="UInt32")),)), name="KSDS3D_BUFFER_CONE_ANGLES", pack=False, align=None),
    "KSDS3D_HRTF_COEFF_FORMAT": SimTypeInt(signed=False, label="KSDS3D_HRTF_COEFF_FORMAT"),
    "KSDS3D_HRTF_FILTER_FORMAT_MSG": SimStruct(OrderedDict((("FilterMethod", SimTypeInt(signed=False, label="KSDS3D_HRTF_FILTER_METHOD")), ("CoeffFormat", SimTypeInt(signed=False, label="KSDS3D_HRTF_COEFF_FORMAT")), ("Version", SimTypeInt(signed=False, label="KSDS3D_HRTF_FILTER_VERSION")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSDS3D_HRTF_FILTER_FORMAT_MSG", pack=False, align=None),
    "KSDS3D_HRTF_FILTER_METHOD": SimTypeInt(signed=False, label="KSDS3D_HRTF_FILTER_METHOD"),
    "KSDS3D_HRTF_FILTER_QUALITY": SimTypeInt(signed=False, label="KSDS3D_HRTF_FILTER_QUALITY"),
    "KSDS3D_HRTF_FILTER_VERSION": SimTypeInt(signed=False, label="KSDS3D_HRTF_FILTER_VERSION"),
    "KSDS3D_HRTF_INIT_MSG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Quality", SimTypeInt(signed=False, label="KSDS3D_HRTF_FILTER_QUALITY")), ("SampleRate", SimTypeFloat(size=32)), ("MaxFilterSize", SimTypeInt(signed=False, label="UInt32")), ("FilterTransientMuteLength", SimTypeInt(signed=False, label="UInt32")), ("FilterOverlapBufferLength", SimTypeInt(signed=False, label="UInt32")), ("OutputOverlapBufferLength", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSDS3D_HRTF_INIT_MSG", pack=False, align=None),
    "KSDS3D_HRTF_PARAMS_MSG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Enabled", SimTypeInt(signed=False, label="UInt32")), ("SwapChannels", SimTypeInt(signed=True, label="Int32")), ("ZeroAzimuth", SimTypeInt(signed=True, label="Int32")), ("CrossFadeOutput", SimTypeInt(signed=True, label="Int32")), ("FilterSize", SimTypeInt(signed=False, label="UInt32")),)), name="KSDS3D_HRTF_PARAMS_MSG", pack=False, align=None),
    "KSDS3D_ITD_PARAMS": SimStruct(OrderedDict((("Channel", SimTypeInt(signed=True, label="Int32")), ("VolSmoothScale", SimTypeFloat(size=32)), ("TotalDryAttenuation", SimTypeFloat(size=32)), ("TotalWetAttenuation", SimTypeFloat(size=32)), ("SmoothFrequency", SimTypeInt(signed=True, label="Int32")), ("Delay", SimTypeInt(signed=True, label="Int32")),)), name="KSDS3D_ITD_PARAMS", pack=False, align=None),
    "KSDS3D_ITD_PARAMS_MSG": SimStruct(OrderedDict((("Enabled", SimTypeInt(signed=False, label="UInt32")), ("LeftParams", SimTypeRef("KSDS3D_ITD_PARAMS", SimStruct)), ("RightParams", SimTypeRef("KSDS3D_ITD_PARAMS", SimStruct)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSDS3D_ITD_PARAMS_MSG", pack=False, align=None),
    "KSDS3D_LISTENER_ALL": SimStruct(OrderedDict((("Position", SimTypeRef("DS3DVECTOR", SimStruct)), ("Velocity", SimTypeRef("DS3DVECTOR", SimStruct)), ("OrientFront", SimTypeRef("DS3DVECTOR", SimStruct)), ("OrientTop", SimTypeRef("DS3DVECTOR", SimStruct)), ("DistanceFactor", SimTypeFloat(size=32)), ("RolloffFactor", SimTypeFloat(size=32)), ("DopplerFactor", SimTypeFloat(size=32)),)), name="KSDS3D_LISTENER_ALL", pack=False, align=None),
    "KSDS3D_LISTENER_ORIENTATION": SimStruct(OrderedDict((("Front", SimTypeRef("DS3DVECTOR", SimStruct)), ("Top", SimTypeRef("DS3DVECTOR", SimStruct)),)), name="KSDS3D_LISTENER_ORIENTATION", pack=False, align=None),
    "KSEC_CONTEXT_TYPE": SimTypeInt(signed=False, label="KSEC_CONTEXT_TYPE"),
    "KSEC_LIST_ENTRY": SimStruct(OrderedDict((("List", SimTypeRef("LIST_ENTRY", SimStruct)), ("RefCount", SimTypeInt(signed=True, label="Int32")), ("Signature", SimTypeInt(signed=False, label="UInt32")), ("OwningList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSEC_LIST_ENTRY", pack=False, align=None),
    "KSEMAPHORE": SimStruct(OrderedDict((("Header", SimTypeRef("DISPATCHER_HEADER", SimStruct)), ("Limit", SimTypeInt(signed=True, label="Int32")),)), name="KSEMAPHORE", pack=False, align=None),
    "KSERROR": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=False, label="UInt32")),)), name="KSERROR", pack=False, align=None),
    "KSERVICE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Interrupt", "ServiceContext"]), offset=0),
    "KSEVENTDATA": SimStruct(OrderedDict((("NotificationType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"EventHandle": SimTypeRef("_EventHandle_e__Struct", SimStruct), "SemaphoreHandle": SimTypeRef("_SemaphoreHandle_e__Struct", SimStruct), "Alignment": SimTypeRef("_Alignment_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="KSEVENTDATA", pack=False, align=None),
    "KSEVENTDATA_BDA_RF_TUNER_SCAN_S": SimStruct(OrderedDict((("EventData", SimTypeRef("KSEVENTDATA", SimStruct)), ("StartFrequency", SimTypeInt(signed=False, label="UInt32")), ("EndFrequency", SimTypeInt(signed=False, label="UInt32")), ("LockRequested", SimTypeInt(signed=False, label="BDA_LockType")),)), name="KSEVENTDATA_BDA_RF_TUNER_SCAN_S", pack=False, align=None),
    "KSEVENT_AUDIO_CONTROL_CHANGE": SimTypeInt(signed=False, label="KSEVENT_AUDIO_CONTROL_CHANGE"),
    "KSEVENT_BDA_EVENT_TYPE": SimTypeInt(signed=False, label="KSEVENT_BDA_EVENT_TYPE"),
    "KSEVENT_BDA_TUNER": SimTypeInt(signed=False, label="KSEVENT_BDA_TUNER"),
    "KSEVENT_CAMERACONTROL": SimTypeInt(signed=False, label="KSEVENT_CAMERACONTROL"),
    "KSEVENT_CAMERAEVENT": SimTypeInt(signed=False, label="KSEVENT_CAMERAEVENT"),
    "KSEVENT_CLOCK_POSITION": SimTypeInt(signed=False, label="KSEVENT_CLOCK_POSITION"),
    "KSEVENT_CONNECTION": SimTypeInt(signed=False, label="KSEVENT_CONNECTION"),
    "KSEVENT_CROSSBAR": SimTypeInt(signed=False, label="KSEVENT_CROSSBAR"),
    "KSEVENT_DEVCMD": SimTypeInt(signed=False, label="KSEVENT_DEVCMD"),
    "KSEVENT_DEVICE": SimTypeInt(signed=False, label="KSEVENT_DEVICE"),
    "KSEVENT_DYNAMICFORMATCHANGE": SimTypeInt(signed=False, label="KSEVENT_DYNAMICFORMATCHANGE"),
    "KSEVENT_LOOPEDSTREAMING": SimTypeInt(signed=False, label="KSEVENT_LOOPEDSTREAMING"),
    "KSEVENT_PINCAPS_CHANGENOTIFICATIONS": SimTypeInt(signed=False, label="KSEVENT_PINCAPS_CHANGENOTIFICATIONS"),
    "KSEVENT_SOUNDDETECTOR": SimTypeInt(signed=False, label="KSEVENT_SOUNDDETECTOR"),
    "KSEVENT_STREAMALLOCATOR": SimTypeInt(signed=False, label="KSEVENT_STREAMALLOCATOR"),
    "KSEVENT_TELEPHONY": SimTypeInt(signed=False, label="KSEVENT_TELEPHONY"),
    "KSEVENT_TIME_INTERVAL": SimStruct(OrderedDict((("EventData", SimTypeRef("KSEVENTDATA", SimStruct)), ("TimeBase", SimTypeLongLong(signed=True, label="Int64")), ("Interval", SimTypeLongLong(signed=True, label="Int64")),)), name="KSEVENT_TIME_INTERVAL", pack=False, align=None),
    "KSEVENT_TIME_MARK": SimStruct(OrderedDict((("EventData", SimTypeRef("KSEVENTDATA", SimStruct)), ("MarkTime", SimTypeLongLong(signed=True, label="Int64")),)), name="KSEVENT_TIME_MARK", pack=False, align=None),
    "KSEVENT_TUNER": SimTypeInt(signed=False, label="KSEVENT_TUNER"),
    "KSEVENT_TUNER_INITIATE_SCAN_S": SimStruct(OrderedDict((("EventData", SimTypeRef("KSEVENTDATA", SimStruct)), ("StartFrequency", SimTypeInt(signed=False, label="UInt32")), ("EndFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="KSEVENT_TUNER_INITIATE_SCAN_S", pack=False, align=None),
    "KSEVENT_TVAUDIO": SimTypeInt(signed=False, label="KSEVENT_TVAUDIO"),
    "KSEVENT_VIDCAPTOSTI": SimTypeInt(signed=False, label="KSEVENT_VIDCAPTOSTI"),
    "KSEVENT_VIDEODECODER": SimTypeInt(signed=False, label="KSEVENT_VIDEODECODER"),
    "KSEVENT_VOLUMELIMIT": SimTypeInt(signed=False, label="KSEVENT_VOLUMELIMIT"),
    "KSEVENT_VPNOTIFY": SimTypeInt(signed=False, label="KSEVENT_VPNOTIFY"),
    "KSEVENT_VPVBINOTIFY": SimTypeInt(signed=False, label="KSEVENT_VPVBINOTIFY"),
    "KSE_NODE": SimStruct(OrderedDict((("Event", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NodeId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSE_NODE", pack=False, align=None),
    "KSE_PIN": SimStruct(OrderedDict((("Event", SimTypeRef("KSIDENTIFIER", SimStruct)), ("PinId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSE_PIN", pack=False, align=None),
    "KSFRAMETIME": SimStruct(OrderedDict((("Duration", SimTypeLongLong(signed=True, label="Int64")), ("FrameFlags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSFRAMETIME", pack=False, align=None),
    "KSGOP_USERDATA": SimStruct(OrderedDict((("sc", SimTypeInt(signed=False, label="UInt32")), ("reserved1", SimTypeInt(signed=False, label="UInt32")), ("cFields", SimTypeChar(label="Byte")), ("l21Data", SimTypeArray(SimTypeChar(label="SByte"), 3)),)), name="KSGOP_USERDATA", pack=False, align=None),
    "KSIDENTIFIER": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Set", SimTypeBottom(label="Guid")), ("Id", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="KSIDENTIFIER", pack=False, align=None),
    "KSINTERFACE_FILEIO": SimTypeInt(signed=False, label="KSINTERFACE_FILEIO"),
    "KSINTERFACE_MEDIA": SimTypeInt(signed=False, label="KSINTERFACE_MEDIA"),
    "KSINTERFACE_STANDARD": SimTypeInt(signed=False, label="KSINTERFACE_STANDARD"),
    "KSINTERVAL": SimStruct(OrderedDict((("TimeBase", SimTypeLongLong(signed=True, label="Int64")), ("Interval", SimTypeLongLong(signed=True, label="Int64")),)), name="KSINTERVAL", pack=False, align=None),
    "KSIOOPERATION": SimTypeInt(signed=False, label="KSIOOPERATION"),
    "KSJACK_DESCRIPTION": SimStruct(OrderedDict((("ChannelMapping", SimTypeInt(signed=False, label="UInt32")), ("Color", SimTypeInt(signed=False, label="UInt32")), ("ConnectionType", SimTypeInt(signed=False, label="EPcxConnectionType")), ("GeoLocation", SimTypeInt(signed=False, label="EPcxGeoLocation")), ("GenLocation", SimTypeInt(signed=False, label="EPcxGenLocation")), ("PortConnection", SimTypeInt(signed=False, label="EPxcPortConnection")), ("IsConnected", SimTypeInt(signed=True, label="Int32")),)), name="KSJACK_DESCRIPTION", pack=False, align=None),
    "KSJACK_DESCRIPTION2": SimStruct(OrderedDict((("DeviceStateInfo", SimTypeInt(signed=False, label="UInt32")), ("JackCapabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSJACK_DESCRIPTION2", pack=False, align=None),
    "KSJACK_DESCRIPTION3": SimStruct(OrderedDict((("ConfigId", SimTypeInt(signed=False, label="UInt32")),)), name="KSJACK_DESCRIPTION3", pack=False, align=None),
    "KSJACK_SINK_CONNECTIONTYPE": SimTypeInt(signed=False, label="KSJACK_SINK_CONNECTIONTYPE"),
    "KSJACK_SINK_INFORMATION": SimStruct(OrderedDict((("ConnType", SimTypeInt(signed=False, label="KSJACK_SINK_CONNECTIONTYPE")), ("ManufacturerId", SimTypeShort(signed=False, label="UInt16")), ("ProductId", SimTypeShort(signed=False, label="UInt16")), ("AudioLatency", SimTypeShort(signed=False, label="UInt16")), ("HDCPCapable", SimTypeInt(signed=True, label="Int32")), ("AICapable", SimTypeInt(signed=True, label="Int32")), ("SinkDescriptionLength", SimTypeChar(label="Byte")), ("SinkDescription", SimTypeArray(SimTypeChar(label="Char"), 32)), ("PortId", SimTypeRef("LUID", SimStruct)),)), name="KSJACK_SINK_INFORMATION", pack=False, align=None),
    "KSMETHOD_BDA_CAS_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_CAS_SERVICE"),
    "KSMETHOD_BDA_CHANGE_SYNC": SimTypeInt(signed=False, label="KSMETHOD_BDA_CHANGE_SYNC"),
    "KSMETHOD_BDA_DEBUG_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_DEBUG_SERVICE"),
    "KSMETHOD_BDA_DEVICE_CONFIGURATION": SimTypeInt(signed=False, label="KSMETHOD_BDA_DEVICE_CONFIGURATION"),
    "KSMETHOD_BDA_DRM": SimTypeInt(signed=False, label="KSMETHOD_BDA_DRM"),
    "KSMETHOD_BDA_EVENTING_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_EVENTING_SERVICE"),
    "KSMETHOD_BDA_GDDS_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_GDDS_SERVICE"),
    "KSMETHOD_BDA_GPNV_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_GPNV_SERVICE"),
    "KSMETHOD_BDA_ISDB_CAS": SimTypeInt(signed=False, label="KSMETHOD_BDA_ISDB_CAS"),
    "KSMETHOD_BDA_MUX_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_MUX_SERVICE"),
    "KSMETHOD_BDA_SCAN_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_SCAN_SERVICE"),
    "KSMETHOD_BDA_TS_SELECTOR": SimTypeInt(signed=False, label="KSMETHOD_BDA_TS_SELECTOR"),
    "KSMETHOD_BDA_TUNER_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_TUNER_SERVICE"),
    "KSMETHOD_BDA_USERACTIVITY_SERVICE": SimTypeInt(signed=False, label="KSMETHOD_BDA_USERACTIVITY_SERVICE"),
    "KSMETHOD_BDA_WMDRM": SimTypeInt(signed=False, label="KSMETHOD_BDA_WMDRM"),
    "KSMETHOD_BDA_WMDRM_TUNER": SimTypeInt(signed=False, label="KSMETHOD_BDA_WMDRM_TUNER"),
    "KSMETHOD_OPMVIDEOOUTPUT": SimTypeInt(signed=False, label="KSMETHOD_OPMVIDEOOUTPUT"),
    "KSMETHOD_STREAMALLOCATOR": SimTypeInt(signed=False, label="KSMETHOD_STREAMALLOCATOR"),
    "KSMETHOD_STREAMIO": SimTypeInt(signed=False, label="KSMETHOD_STREAMIO"),
    "KSMETHOD_WAVETABLE": SimTypeInt(signed=False, label="KSMETHOD_WAVETABLE"),
    "KSMICARRAY_MICARRAYTYPE": SimTypeInt(signed=False, label="KSMICARRAY_MICARRAYTYPE"),
    "KSMICARRAY_MICTYPE": SimTypeInt(signed=False, label="KSMICARRAY_MICTYPE"),
    "KSMPEGVID_RECT": SimStruct(OrderedDict((("StartX", SimTypeInt(signed=False, label="UInt32")), ("StartY", SimTypeInt(signed=False, label="UInt32")), ("EndX", SimTypeInt(signed=False, label="UInt32")), ("EndY", SimTypeInt(signed=False, label="UInt32")),)), name="KSMPEGVID_RECT", pack=False, align=None),
    "KSMULTIPLE_DATA_PROP": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("MultipleItem", SimTypeRef("KSMULTIPLE_ITEM", SimStruct)),)), name="KSMULTIPLE_DATA_PROP", pack=False, align=None),
    "KSMULTIPLE_ITEM": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="KSMULTIPLE_ITEM", pack=False, align=None),
    "KSMUSICFORMAT": SimStruct(OrderedDict((("TimeDeltaMs", SimTypeInt(signed=False, label="UInt32")), ("ByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="KSMUSICFORMAT", pack=False, align=None),
    "KSM_BDA_BUFFER": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulBufferSize", SimTypeInt(signed=False, label="UInt32")), ("argbBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_BUFFER", pack=False, align=None),
    "KSM_BDA_CAS_CAPTURETOKEN": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulTokenLength", SimTypeInt(signed=False, label="UInt32")), ("argbToken", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_CAS_CAPTURETOKEN", pack=False, align=None),
    "KSM_BDA_CAS_CLOSEMMIDIALOG": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulDialogRequest", SimTypeInt(signed=False, label="UInt32")), ("cLanguage", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("ulDialogNumber", SimTypeInt(signed=False, label="UInt32")), ("ulReason", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_CAS_CLOSEMMIDIALOG", pack=False, align=None),
    "KSM_BDA_CAS_ENTITLEMENTTOKEN": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulDialogRequest", SimTypeInt(signed=False, label="UInt32")), ("cLanguage", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("ulRequestType", SimTypeInt(signed=False, label="UInt32")), ("ulEntitlementTokenLen", SimTypeInt(signed=False, label="UInt32")), ("argbEntitlementToken", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_CAS_ENTITLEMENTTOKEN", pack=False, align=None),
    "KSM_BDA_CAS_OPENBROADCASTMMI": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulDialogRequest", SimTypeInt(signed=False, label="UInt32")), ("cLanguage", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("ulEventId", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_CAS_OPENBROADCASTMMI", pack=False, align=None),
    "KSM_BDA_DEBUG_LEVEL": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ucDebugLevel", SimTypeChar(label="Byte")), ("ulDebugStringSize", SimTypeInt(signed=False, label="UInt32")), ("argbDebugString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_DEBUG_LEVEL", pack=False, align=None),
    "KSM_BDA_DRM_SETDRM": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("NewDRMuuid", SimTypeBottom(label="Guid")),)), name="KSM_BDA_DRM_SETDRM", pack=False, align=None),
    "KSM_BDA_EVENT_COMPLETE": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulEventID", SimTypeInt(signed=False, label="UInt32")), ("ulEventResult", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_EVENT_COMPLETE", pack=False, align=None),
    "KSM_BDA_GDDS_SERVICEFROMTUNEXML": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulTuneXmlLength", SimTypeInt(signed=False, label="UInt32")), ("argbTuneXml", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_GDDS_SERVICEFROMTUNEXML", pack=False, align=None),
    "KSM_BDA_GDDS_TUNEXMLFROMIDX": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulIdx", SimTypeLongLong(signed=False, label="UInt64")),)), name="KSM_BDA_GDDS_TUNEXMLFROMIDX", pack=False, align=None),
    "KSM_BDA_GPNV_GETVALUE": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulNameLength", SimTypeInt(signed=False, label="UInt32")), ("cLanguage", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("argbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_GPNV_GETVALUE", pack=False, align=None),
    "KSM_BDA_GPNV_NAMEINDEX": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulValueNameIndex", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_GPNV_NAMEINDEX", pack=False, align=None),
    "KSM_BDA_GPNV_SETVALUE": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulDialogRequest", SimTypeInt(signed=False, label="UInt32")), ("cLanguage", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("ulNameLength", SimTypeInt(signed=False, label="UInt32")), ("ulValueLength", SimTypeInt(signed=False, label="UInt32")), ("argbName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_GPNV_SETVALUE", pack=False, align=None),
    "KSM_BDA_ISDBCAS_REQUEST": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulRequestID", SimTypeInt(signed=False, label="UInt32")), ("ulIsdbCommandSize", SimTypeInt(signed=False, label="UInt32")), ("argbIsdbCommandData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_ISDBCAS_REQUEST", pack=False, align=None),
    "KSM_BDA_PIN": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Anonymous", SimUnion({"PinId": SimTypeInt(signed=False, label="UInt32"), "PinType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_PIN", pack=False, align=None),
    "KSM_BDA_PIN_PAIR": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Anonymous1", SimUnion({"InputPinId": SimTypeInt(signed=False, label="UInt32"), "InputPinType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"OutputPinId": SimTypeInt(signed=False, label="UInt32"), "OutputPinType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="KSM_BDA_PIN_PAIR", pack=False, align=None),
    "KSM_BDA_SCAN_CAPABILTIES": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("uuidBroadcastStandard", SimTypeBottom(label="Guid")),)), name="KSM_BDA_SCAN_CAPABILTIES", pack=False, align=None),
    "KSM_BDA_SCAN_FILTER": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulScanModulationTypeSize", SimTypeInt(signed=False, label="UInt32")), ("AnalogVideoStandards", SimTypeLongLong(signed=False, label="UInt64")), ("argbScanModulationTypes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_SCAN_FILTER", pack=False, align=None),
    "KSM_BDA_SCAN_START": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("LowerFrequency", SimTypeInt(signed=False, label="UInt32")), ("HigherFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_SCAN_START", pack=False, align=None),
    "KSM_BDA_TS_SELECTOR_SETTSID": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("usTSID", SimTypeShort(signed=False, label="UInt16")),)), name="KSM_BDA_TS_SELECTOR_SETTSID", pack=False, align=None),
    "KSM_BDA_TUNER_TUNEREQUEST": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulTuneLength", SimTypeInt(signed=False, label="UInt32")), ("argbTuneData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_TUNER_TUNEREQUEST", pack=False, align=None),
    "KSM_BDA_USERACTIVITY_USEREASON": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulUseReason", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_USERACTIVITY_USEREASON", pack=False, align=None),
    "KSM_BDA_WMDRMTUNER_GETPIDPROTECTION": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulPID", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_WMDRMTUNER_GETPIDPROTECTION", pack=False, align=None),
    "KSM_BDA_WMDRMTUNER_PURCHASEENTITLEMENT": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulDialogRequest", SimTypeInt(signed=False, label="UInt32")), ("cLanguage", SimTypeArray(SimTypeChar(label="SByte"), 12)), ("ulPurchaseTokenLength", SimTypeInt(signed=False, label="UInt32")), ("argbDataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_WMDRMTUNER_PURCHASEENTITLEMENT", pack=False, align=None),
    "KSM_BDA_WMDRMTUNER_SETPIDPROTECTION": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulPID", SimTypeInt(signed=False, label="UInt32")), ("uuidKeyID", SimTypeBottom(label="Guid")),)), name="KSM_BDA_WMDRMTUNER_SETPIDPROTECTION", pack=False, align=None),
    "KSM_BDA_WMDRMTUNER_SYNCVALUE": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulSyncValue", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_BDA_WMDRMTUNER_SYNCVALUE", pack=False, align=None),
    "KSM_BDA_WMDRM_LICENSE": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("uuidKeyID", SimTypeBottom(label="Guid")),)), name="KSM_BDA_WMDRM_LICENSE", pack=False, align=None),
    "KSM_BDA_WMDRM_RENEWLICENSE": SimStruct(OrderedDict((("NodeMethod", SimTypeRef("KSM_NODE", SimStruct)), ("ulXMRLicenseLength", SimTypeInt(signed=False, label="UInt32")), ("ulEntitlementTokenLength", SimTypeInt(signed=False, label="UInt32")), ("argbDataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KSM_BDA_WMDRM_RENEWLICENSE", pack=False, align=None),
    "KSM_NODE": SimStruct(OrderedDict((("Method", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NodeId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSM_NODE", pack=False, align=None),
    "KSNODEPROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NodeId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSNODEPROPERTY", pack=False, align=None),
    "KSNODEPROPERTY_AUDIO_3D_LISTENER": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSNODEPROPERTY", SimStruct)), ("ListenerId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSNODEPROPERTY_AUDIO_3D_LISTENER", pack=False, align=None),
    "KSNODEPROPERTY_AUDIO_CHANNEL": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSNODEPROPERTY", SimStruct)), ("Channel", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSNODEPROPERTY_AUDIO_CHANNEL", pack=False, align=None),
    "KSNODEPROPERTY_AUDIO_DEV_SPECIFIC": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSNODEPROPERTY", SimStruct)), ("DevSpecificId", SimTypeInt(signed=False, label="UInt32")), ("DeviceInfo", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="KSNODEPROPERTY_AUDIO_DEV_SPECIFIC", pack=False, align=None),
    "KSNODEPROPERTY_AUDIO_PROPERTY": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSNODEPROPERTY", SimStruct)), ("AppContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSNODEPROPERTY_AUDIO_PROPERTY", pack=False, align=None),
    "KSNODE_CREATE": SimStruct(OrderedDict((("CreateFlags", SimTypeInt(signed=False, label="UInt32")), ("Node", SimTypeInt(signed=False, label="UInt32")),)), name="KSNODE_CREATE", pack=False, align=None),
    "KSPEEKOPERATION": SimTypeInt(signed=False, label="KSPEEKOPERATION"),
    "KSPIN_CINSTANCES": SimStruct(OrderedDict((("PossibleCount", SimTypeInt(signed=False, label="UInt32")), ("CurrentCount", SimTypeInt(signed=False, label="UInt32")),)), name="KSPIN_CINSTANCES", pack=False, align=None),
    "KSPIN_COMMUNICATION": SimTypeInt(signed=False, label="KSPIN_COMMUNICATION"),
    "KSPIN_CONNECT": SimStruct(OrderedDict((("Interface", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Medium", SimTypeRef("KSIDENTIFIER", SimStruct)), ("PinId", SimTypeInt(signed=False, label="UInt32")), ("PinToHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Priority", SimTypeRef("KSPRIORITY", SimStruct)),)), name="KSPIN_CONNECT", pack=False, align=None),
    "KSPIN_DATAFLOW": SimTypeInt(signed=False, label="KSPIN_DATAFLOW"),
    "KSPIN_LOCK_QUEUE": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("KSPIN_LOCK_QUEUE", SimStruct), offset=0)), ("Lock", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)),)), name="KSPIN_LOCK_QUEUE", pack=False, align=None),
    "KSPIN_LOCK_QUEUE_NUMBER": SimTypeInt(signed=False, label="KSPIN_LOCK_QUEUE_NUMBER"),
    "KSPIN_MDL_CACHING_EVENT": SimTypeInt(signed=False, label="KSPIN_MDL_CACHING_EVENT"),
    "KSPIN_MDL_CACHING_NOTIFICATION": SimStruct(OrderedDict((("Event", SimTypeInt(signed=False, label="KSPIN_MDL_CACHING_EVENT")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSPIN_MDL_CACHING_NOTIFICATION", pack=False, align=None),
    "KSPIN_MDL_CACHING_NOTIFICATION32": SimStruct(OrderedDict((("Event", SimTypeInt(signed=False, label="KSPIN_MDL_CACHING_EVENT")), ("Buffer", SimTypeInt(signed=False, label="UInt32")),)), name="KSPIN_MDL_CACHING_NOTIFICATION32", pack=False, align=None),
    "KSPIN_PHYSICALCONNECTION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Pin", SimTypeInt(signed=False, label="UInt32")), ("SymbolicLinkName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KSPIN_PHYSICALCONNECTION", pack=False, align=None),
    "KSPPROPERTY_ALLOCATOR_MDLCACHING": SimTypeInt(signed=False, label="KSPPROPERTY_ALLOCATOR_MDLCACHING"),
    "KSPRIORITY": SimStruct(OrderedDict((("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("PrioritySubClass", SimTypeInt(signed=False, label="UInt32")),)), name="KSPRIORITY", pack=False, align=None),
    "KSPROPERTY_AC3": SimTypeInt(signed=False, label="KSPROPERTY_AC3"),
    "KSPROPERTY_ALLOCATOR_CONTROL": SimTypeInt(signed=False, label="KSPROPERTY_ALLOCATOR_CONTROL"),
    "KSPROPERTY_ALLOCATOR_CONTROL_CAPTURE_CAPS_S": SimStruct(OrderedDict((("InterleavedCapSupported", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_ALLOCATOR_CONTROL_CAPTURE_CAPS_S", pack=False, align=None),
    "KSPROPERTY_ALLOCATOR_CONTROL_CAPTURE_INTERLEAVE_S": SimStruct(OrderedDict((("InterleavedCapPossible", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_ALLOCATOR_CONTROL_CAPTURE_INTERLEAVE_S", pack=False, align=None),
    "KSPROPERTY_ALLOCATOR_CONTROL_SURFACE_SIZE_S": SimStruct(OrderedDict((("CX", SimTypeInt(signed=False, label="UInt32")), ("CY", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_ALLOCATOR_CONTROL_SURFACE_SIZE_S", pack=False, align=None),
    "KSPROPERTY_AUDDECOUT": SimTypeInt(signed=False, label="KSPROPERTY_AUDDECOUT"),
    "KSPROPERTY_AUDIO": SimTypeInt(signed=False, label="KSPROPERTY_AUDIO"),
    "KSPROPERTY_AUDIOENGINE": SimTypeInt(signed=False, label="KSPROPERTY_AUDIOENGINE"),
    "KSPROPERTY_AUDIOMODULE": SimTypeInt(signed=False, label="KSPROPERTY_AUDIOMODULE"),
    "KSPROPERTY_AUDIOPOSTURE": SimTypeInt(signed=False, label="KSPROPERTY_AUDIOPOSTURE"),
    "KSPROPERTY_AUDIORESOURCEMANAGEMENT": SimTypeInt(signed=False, label="KSPROPERTY_AUDIORESOURCEMANAGEMENT"),
    "KSPROPERTY_AUDIOSIGNALPROCESSING": SimTypeInt(signed=False, label="KSPROPERTY_AUDIOSIGNALPROCESSING"),
    "KSPROPERTY_BDA_AUTODEMODULATE": SimTypeInt(signed=False, label="KSPROPERTY_BDA_AUTODEMODULATE"),
    "KSPROPERTY_BDA_CA": SimTypeInt(signed=False, label="KSPROPERTY_BDA_CA"),
    "KSPROPERTY_BDA_CA_EVENT": SimTypeInt(signed=False, label="KSPROPERTY_BDA_CA_EVENT"),
    "KSPROPERTY_BDA_DIGITAL_DEMODULATOR": SimTypeInt(signed=False, label="KSPROPERTY_BDA_DIGITAL_DEMODULATOR"),
    "KSPROPERTY_BDA_DISEQC_COMMAND": SimTypeInt(signed=False, label="KSPROPERTY_BDA_DISEQC_COMMAND"),
    "KSPROPERTY_BDA_DISEQC_EVENT": SimTypeInt(signed=False, label="KSPROPERTY_BDA_DISEQC_EVENT"),
    "KSPROPERTY_BDA_ETHERNET_FILTER": SimTypeInt(signed=False, label="KSPROPERTY_BDA_ETHERNET_FILTER"),
    "KSPROPERTY_BDA_FREQUENCY_FILTER": SimTypeInt(signed=False, label="KSPROPERTY_BDA_FREQUENCY_FILTER"),
    "KSPROPERTY_BDA_IPv4_FILTER": SimTypeInt(signed=False, label="KSPROPERTY_BDA_IPv4_FILTER"),
    "KSPROPERTY_BDA_IPv6_FILTER": SimTypeInt(signed=False, label="KSPROPERTY_BDA_IPv6_FILTER"),
    "KSPROPERTY_BDA_LNB_INFO": SimTypeInt(signed=False, label="KSPROPERTY_BDA_LNB_INFO"),
    "KSPROPERTY_BDA_NULL_TRANSFORM": SimTypeInt(signed=False, label="KSPROPERTY_BDA_NULL_TRANSFORM"),
    "KSPROPERTY_BDA_PIDFILTER": SimTypeInt(signed=False, label="KSPROPERTY_BDA_PIDFILTER"),
    "KSPROPERTY_BDA_PIN_CONTROL": SimTypeInt(signed=False, label="KSPROPERTY_BDA_PIN_CONTROL"),
    "KSPROPERTY_BDA_PIN_EVENT": SimTypeInt(signed=False, label="KSPROPERTY_BDA_PIN_EVENT"),
    "KSPROPERTY_BDA_RF_TUNER_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSP_NODE", SimStruct)), ("Mode", SimTypeInt(signed=False, label="UInt32")), ("AnalogStandardsSupported", SimTypeInt(signed=False, label="UInt32")), ("DigitalStandardsSupported", SimTypeInt(signed=False, label="UInt32")), ("MinFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaxFrequency", SimTypeInt(signed=False, label="UInt32")), ("SettlingTime", SimTypeInt(signed=False, label="UInt32")), ("AnalogSensingRange", SimTypeInt(signed=False, label="UInt32")), ("DigitalSensingRange", SimTypeInt(signed=False, label="UInt32")), ("MilliSecondsPerMHz", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_BDA_RF_TUNER_CAPS_S", pack=False, align=None),
    "KSPROPERTY_BDA_RF_TUNER_SCAN_STATUS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSP_NODE", SimStruct)), ("CurrentFrequency", SimTypeInt(signed=False, label="UInt32")), ("FrequencyRangeMin", SimTypeInt(signed=False, label="UInt32")), ("FrequencyRangeMax", SimTypeInt(signed=False, label="UInt32")), ("MilliSecondsLeft", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_BDA_RF_TUNER_SCAN_STATUS_S", pack=False, align=None),
    "KSPROPERTY_BDA_RF_TUNER_STANDARD_MODE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSP_NODE", SimStruct)), ("AutoDetect", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_BDA_RF_TUNER_STANDARD_MODE_S", pack=False, align=None),
    "KSPROPERTY_BDA_RF_TUNER_STANDARD_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSP_NODE", SimStruct)), ("SignalType", SimTypeInt(signed=False, label="BDA_SignalType")), ("SignalStandard", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_BDA_RF_TUNER_STANDARD_S", pack=False, align=None),
    "KSPROPERTY_BDA_SIGNAL_STATS": SimTypeInt(signed=False, label="KSPROPERTY_BDA_SIGNAL_STATS"),
    "KSPROPERTY_BDA_TOPOLOGY": SimTypeInt(signed=False, label="KSPROPERTY_BDA_TOPOLOGY"),
    "KSPROPERTY_BDA_VOID_TRANSFORM": SimTypeInt(signed=False, label="KSPROPERTY_BDA_VOID_TRANSFORM"),
    "KSPROPERTY_BIBLIOGRAPHIC": SimTypeInt(signed=False, label="KSPROPERTY_BIBLIOGRAPHIC"),
    "KSPROPERTY_BOUNDS_LONG": SimUnion({"Anonymous1": SimStruct(OrderedDict((("SignedMinimum", SimTypeInt(signed=True, label="Int32")), ("SignedMaximum", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("UnsignedMinimum", SimTypeInt(signed=False, label="UInt32")), ("UnsignedMaximum", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "KSPROPERTY_BOUNDS_LONGLONG": SimUnion({"Anonymous1": SimStruct(OrderedDict((("SignedMinimum", SimTypeLongLong(signed=True, label="Int64")), ("SignedMaximum", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("UnsignedMinimum", SimTypeLongLong(signed=False, label="UInt64")), ("UnsignedMaximum", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "KSPROPERTY_BTAUDIO": SimTypeInt(signed=False, label="KSPROPERTY_BTAUDIO"),
    "KSPROPERTY_CAMERACONTROL_EXTENDED_PROPERTY": SimTypeInt(signed=False, label="KSPROPERTY_CAMERACONTROL_EXTENDED_PROPERTY"),
    "KSPROPERTY_CAMERACONTROL_FLASH": SimTypeInt(signed=False, label="KSPROPERTY_CAMERACONTROL_FLASH"),
    "KSPROPERTY_CAMERACONTROL_FLASH_S": SimStruct(OrderedDict((("Flash", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CAMERACONTROL_FLASH_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_FOCAL_LENGTH_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("lOcularFocalLength", SimTypeInt(signed=True, label="Int32")), ("lObjectiveFocalLengthMin", SimTypeInt(signed=True, label="Int32")), ("lObjectiveFocalLengthMax", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_CAMERACONTROL_FOCAL_LENGTH_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_IMAGE_PIN_CAPABILITY": SimTypeInt(signed=False, label="KSPROPERTY_CAMERACONTROL_IMAGE_PIN_CAPABILITY"),
    "KSPROPERTY_CAMERACONTROL_IMAGE_PIN_CAPABILITY_S": SimStruct(OrderedDict((("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CAMERACONTROL_IMAGE_PIN_CAPABILITY_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_NODE_FOCAL_LENGTH_S": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSNODEPROPERTY", SimStruct)), ("lOcularFocalLength", SimTypeInt(signed=True, label="Int32")), ("lObjectiveFocalLengthMin", SimTypeInt(signed=True, label="Int32")), ("lObjectiveFocalLengthMax", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_CAMERACONTROL_NODE_FOCAL_LENGTH_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_NODE_S": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CAMERACONTROL_NODE_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_NODE_S2": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("Value1", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("Value2", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_CAMERACONTROL_NODE_S2", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_PERFRAMESETTING_PROPERTY": SimTypeInt(signed=False, label="KSPROPERTY_CAMERACONTROL_PERFRAMESETTING_PROPERTY"),
    "KSPROPERTY_CAMERACONTROL_REGION_OF_INTEREST": SimTypeInt(signed=False, label="KSPROPERTY_CAMERACONTROL_REGION_OF_INTEREST"),
    "KSPROPERTY_CAMERACONTROL_REGION_OF_INTEREST_S": SimStruct(OrderedDict((("FocusRect", SimTypeRef("RECT", SimStruct)), ("AutoFocusLock", SimTypeInt(signed=True, label="Int32")), ("AutoExposureLock", SimTypeInt(signed=True, label="Int32")), ("AutoWhitebalanceLock", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"Capabilities": SimTypeInt(signed=False, label="UInt32"), "Configuration": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="KSPROPERTY_CAMERACONTROL_REGION_OF_INTEREST_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CAMERACONTROL_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_S2": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value1", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("Value2", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_CAMERACONTROL_S2", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_S_EX": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("FocusRect", SimTypeRef("RECT", SimStruct)),)), name="KSPROPERTY_CAMERACONTROL_S_EX", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_VIDEOSTABILIZATION_MODE_S": SimStruct(OrderedDict((("VideoStabilizationMode", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CAMERACONTROL_VIDEOSTABILIZATION_MODE_S", pack=False, align=None),
    "KSPROPERTY_CAMERACONTROL_VIDEO_STABILIZATION_MODE": SimTypeInt(signed=False, label="KSPROPERTY_CAMERACONTROL_VIDEO_STABILIZATION_MODE"),
    "KSPROPERTY_CAMERA_PHOTOTRIGGERTIME_FLAGS": SimTypeInt(signed=False, label="KSPROPERTY_CAMERA_PHOTOTRIGGERTIME_FLAGS"),
    "KSPROPERTY_CLOCK": SimTypeInt(signed=False, label="KSPROPERTY_CLOCK"),
    "KSPROPERTY_CONNECTION": SimTypeInt(signed=False, label="KSPROPERTY_CONNECTION"),
    "KSPROPERTY_COPYPROT": SimTypeInt(signed=False, label="KSPROPERTY_COPYPROT"),
    "KSPROPERTY_CROSSBAR_ACTIVE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("IndexInputPin", SimTypeInt(signed=False, label="UInt32")), ("Active", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CROSSBAR_ACTIVE_S", pack=False, align=None),
    "KSPROPERTY_CROSSBAR_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NumberOfInputs", SimTypeInt(signed=False, label="UInt32")), ("NumberOfOutputs", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CROSSBAR_CAPS_S", pack=False, align=None),
    "KSPROPERTY_CROSSBAR_PININFO_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Direction", SimTypeInt(signed=False, label="KSPIN_DATAFLOW")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("PinType", SimTypeInt(signed=False, label="UInt32")), ("RelatedPinIndex", SimTypeInt(signed=False, label="UInt32")), ("Medium", SimTypeRef("KSIDENTIFIER", SimStruct)),)), name="KSPROPERTY_CROSSBAR_PININFO_S", pack=False, align=None),
    "KSPROPERTY_CROSSBAR_ROUTE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("IndexInputPin", SimTypeInt(signed=False, label="UInt32")), ("IndexOutputPin", SimTypeInt(signed=False, label="UInt32")), ("CanRoute", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_CROSSBAR_ROUTE_S", pack=False, align=None),
    "KSPROPERTY_CYCLIC": SimTypeInt(signed=False, label="KSPROPERTY_CYCLIC"),
    "KSPROPERTY_DESCRIPTION": SimStruct(OrderedDict((("AccessFlags", SimTypeInt(signed=False, label="UInt32")), ("DescriptionSize", SimTypeInt(signed=False, label="UInt32")), ("PropTypeSet", SimTypeRef("KSIDENTIFIER", SimStruct)), ("MembersListCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_DESCRIPTION", pack=False, align=None),
    "KSPROPERTY_DIRECTSOUND3DBUFFER": SimTypeInt(signed=False, label="KSPROPERTY_DIRECTSOUND3DBUFFER"),
    "KSPROPERTY_DIRECTSOUND3DLISTENER": SimTypeInt(signed=False, label="KSPROPERTY_DIRECTSOUND3DLISTENER"),
    "KSPROPERTY_DRMAUDIOSTREAM": SimTypeInt(signed=False, label="KSPROPERTY_DRMAUDIOSTREAM"),
    "KSPROPERTY_DROPPEDFRAMES_CURRENT_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("PictureNumber", SimTypeLongLong(signed=True, label="Int64")), ("DropCount", SimTypeLongLong(signed=True, label="Int64")), ("AverageFrameSize", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_DROPPEDFRAMES_CURRENT_S", pack=False, align=None),
    "KSPROPERTY_DVDSUBPIC": SimTypeInt(signed=False, label="KSPROPERTY_DVDSUBPIC"),
    "KSPROPERTY_EXTDEVICE": SimTypeInt(signed=False, label="KSPROPERTY_EXTDEVICE"),
    "KSPROPERTY_EXTDEVICE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("u", SimUnion({"Capabilities": SimTypeRef("DEVCAPS", SimStruct), "DevPort": SimTypeInt(signed=False, label="UInt32"), "PowerState": SimTypeInt(signed=False, label="UInt32"), "pawchString": SimTypeArray(SimTypeChar(label="Char"), 260), "NodeUniqueID": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)}, name="<anon>", label="None")),)), name="KSPROPERTY_EXTDEVICE_S", pack=False, align=None),
    "KSPROPERTY_EXTENSION_UNIT": SimTypeInt(signed=False, label="KSPROPERTY_EXTENSION_UNIT"),
    "KSPROPERTY_EXTXPORT": SimTypeInt(signed=False, label="KSPROPERTY_EXTXPORT"),
    "KSPROPERTY_EXTXPORT_NODE_S": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("u", SimUnion({"Capabilities": SimTypeInt(signed=False, label="UInt32"), "SignalMode": SimTypeInt(signed=False, label="UInt32"), "LoadMedium": SimTypeInt(signed=False, label="UInt32"), "MediumInfo": SimTypeRef("MEDIUM_INFO", SimStruct), "XPrtState": SimTypeRef("TRANSPORT_STATE", SimStruct), "Timecode": SimTypeRef("_Timecode_e__Struct", SimStruct), "dwTimecode": SimTypeInt(signed=False, label="UInt32"), "dwAbsTrackNumber": SimTypeInt(signed=False, label="UInt32"), "RawAVC": SimTypeRef("_RawAVC_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="KSPROPERTY_EXTXPORT_NODE_S", pack=False, align=None),
    "KSPROPERTY_EXTXPORT_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("u", SimUnion({"Capabilities": SimTypeInt(signed=False, label="UInt32"), "SignalMode": SimTypeInt(signed=False, label="UInt32"), "LoadMedium": SimTypeInt(signed=False, label="UInt32"), "MediumInfo": SimTypeRef("MEDIUM_INFO", SimStruct), "XPrtState": SimTypeRef("TRANSPORT_STATE", SimStruct), "Timecode": SimTypeRef("_Timecode_e__Struct", SimStruct), "dwTimecode": SimTypeInt(signed=False, label="UInt32"), "dwAbsTrackNumber": SimTypeInt(signed=False, label="UInt32"), "RawAVC": SimTypeRef("_RawAVC_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="KSPROPERTY_EXTXPORT_S", pack=False, align=None),
    "KSPROPERTY_FMRX_CONTROL": SimTypeInt(signed=False, label="KSPROPERTY_FMRX_CONTROL"),
    "KSPROPERTY_FMRX_TOPOLOGY": SimTypeInt(signed=False, label="KSPROPERTY_FMRX_TOPOLOGY"),
    "KSPROPERTY_GENERAL": SimTypeInt(signed=False, label="KSPROPERTY_GENERAL"),
    "KSPROPERTY_HRTF3D": SimTypeInt(signed=False, label="KSPROPERTY_HRTF3D"),
    "KSPROPERTY_IDS_BDA_TABLE": SimTypeInt(signed=False, label="KSPROPERTY_IDS_BDA_TABLE"),
    "KSPROPERTY_INTERLEAVEDAUDIO": SimTypeInt(signed=False, label="KSPROPERTY_INTERLEAVEDAUDIO"),
    "KSPROPERTY_IPSINK": SimTypeInt(signed=False, label="KSPROPERTY_IPSINK"),
    "KSPROPERTY_ITD3D": SimTypeInt(signed=False, label="KSPROPERTY_ITD3D"),
    "KSPROPERTY_JACK": SimTypeInt(signed=False, label="KSPROPERTY_JACK"),
    "KSPROPERTY_MEDIAAVAILABLE": SimStruct(OrderedDict((("Earliest", SimTypeLongLong(signed=True, label="Int64")), ("Latest", SimTypeLongLong(signed=True, label="Int64")),)), name="KSPROPERTY_MEDIAAVAILABLE", pack=False, align=None),
    "KSPROPERTY_MEDIASEEKING": SimTypeInt(signed=False, label="KSPROPERTY_MEDIASEEKING"),
    "KSPROPERTY_MEMBERSHEADER": SimStruct(OrderedDict((("MembersFlags", SimTypeInt(signed=False, label="UInt32")), ("MembersSize", SimTypeInt(signed=False, label="UInt32")), ("MembersCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_MEMBERSHEADER", pack=False, align=None),
    "KSPROPERTY_MPEG2VID": SimTypeInt(signed=False, label="KSPROPERTY_MPEG2VID"),
    "KSPROPERTY_MPEG4_MEDIATYPE_ATTRIBUTES": SimTypeInt(signed=False, label="KSPROPERTY_MPEG4_MEDIATYPE_ATTRIBUTES"),
    "KSPROPERTY_NETWORKCAMERACONTROL_EVENT_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("KSCAMERA_METADATA_ITEMHEADER", SimStruct)), ("EventFilter", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="KSPROPERTY_NETWORKCAMERACONTROL_EVENT_INFO", pack=False, align=None),
    "KSPROPERTY_NETWORKCAMERACONTROL_METADATA_INFO": SimStruct(OrderedDict((("MetadataItems", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PTZStatus", SimTypeInt(signed=True, label="Int32")), ("Events", SimTypeInt(signed=True, label="Int32")), ("Analytics", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_NETWORKCAMERACONTROL_METADATA_INFO", pack=False, align=None),
    "KSPROPERTY_NETWORKCAMERACONTROL_METADATA_TYPE": SimTypeInt(signed=False, label="KSPROPERTY_NETWORKCAMERACONTROL_METADATA_TYPE"),
    "KSPROPERTY_NETWORKCAMERACONTROL_NTPINFO_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="KSPROPERTY_NETWORKCAMERACONTROL_NTPINFO_TYPE")),)), name="KSPROPERTY_NETWORKCAMERACONTROL_NTPINFO_HEADER", pack=False, align=None),
    "KSPROPERTY_NETWORKCAMERACONTROL_NTPINFO_TYPE": SimTypeInt(signed=False, label="KSPROPERTY_NETWORKCAMERACONTROL_NTPINFO_TYPE"),
    "KSPROPERTY_NETWORKCAMERACONTROL_PROPERTY": SimTypeInt(signed=False, label="KSPROPERTY_NETWORKCAMERACONTROL_PROPERTY"),
    "KSPROPERTY_OVERLAYUPDATE": SimTypeInt(signed=False, label="KSPROPERTY_OVERLAYUPDATE"),
    "KSPROPERTY_PIN": SimTypeInt(signed=False, label="KSPROPERTY_PIN"),
    "KSPROPERTY_POSITIONS": SimStruct(OrderedDict((("Current", SimTypeLongLong(signed=True, label="Int64")), ("Stop", SimTypeLongLong(signed=True, label="Int64")), ("CurrentFlags", SimTypeInt(signed=False, label="KS_SEEKING_FLAGS")), ("StopFlags", SimTypeInt(signed=False, label="KS_SEEKING_FLAGS")),)), name="KSPROPERTY_POSITIONS", pack=False, align=None),
    "KSPROPERTY_QUALITY": SimTypeInt(signed=False, label="KSPROPERTY_QUALITY"),
    "KSPROPERTY_RTAUDIO": SimTypeInt(signed=False, label="KSPROPERTY_RTAUDIO"),
    "KSPROPERTY_SELECTOR_NODE_S": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_SELECTOR_NODE_S", pack=False, align=None),
    "KSPROPERTY_SELECTOR_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_SELECTOR_S", pack=False, align=None),
    "KSPROPERTY_SERIAL": SimStruct(OrderedDict((("PropTypeSet", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Id", SimTypeInt(signed=False, label="UInt32")), ("PropertyLength", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_SERIAL", pack=False, align=None),
    "KSPROPERTY_SERIALHDR": SimStruct(OrderedDict((("PropertySet", SimTypeBottom(label="Guid")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_SERIALHDR", pack=False, align=None),
    "KSPROPERTY_SOUNDDETECTOR": SimTypeInt(signed=False, label="KSPROPERTY_SOUNDDETECTOR"),
    "KSPROPERTY_SPHLI": SimStruct(OrderedDict((("HLISS", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("StartPTM", SimTypeInt(signed=False, label="UInt32")), ("EndPTM", SimTypeInt(signed=False, label="UInt32")), ("StartX", SimTypeShort(signed=False, label="UInt16")), ("StartY", SimTypeShort(signed=False, label="UInt16")), ("StopX", SimTypeShort(signed=False, label="UInt16")), ("StopY", SimTypeShort(signed=False, label="UInt16")), ("ColCon", SimTypeRef("KS_COLCON", SimStruct)),)), name="KSPROPERTY_SPHLI", pack=False, align=None),
    "KSPROPERTY_SPPAL": SimStruct(OrderedDict((("sppal", SimTypeArray(SimTypeRef("KS_DVD_YUV", SimStruct), 16)),)), name="KSPROPERTY_SPPAL", pack=False, align=None),
    "KSPROPERTY_STEPPING_LONG": SimStruct(OrderedDict((("SteppingDelta", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Bounds", SimUnion({"Anonymous1": SimStruct(OrderedDict((("SignedMinimum", SimTypeInt(signed=True, label="Int32")), ("SignedMaximum", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("UnsignedMinimum", SimTypeInt(signed=False, label="UInt32")), ("UnsignedMaximum", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="KSPROPERTY_STEPPING_LONG", pack=False, align=None),
    "KSPROPERTY_STEPPING_LONGLONG": SimStruct(OrderedDict((("SteppingDelta", SimTypeLongLong(signed=False, label="UInt64")), ("Bounds", SimUnion({"Anonymous1": SimStruct(OrderedDict((("SignedMinimum", SimTypeLongLong(signed=True, label="Int64")), ("SignedMaximum", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("UnsignedMinimum", SimTypeLongLong(signed=False, label="UInt64")), ("UnsignedMaximum", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="KSPROPERTY_STEPPING_LONGLONG", pack=False, align=None),
    "KSPROPERTY_STREAM": SimTypeInt(signed=False, label="KSPROPERTY_STREAM"),
    "KSPROPERTY_STREAMINTERFACE": SimTypeInt(signed=False, label="KSPROPERTY_STREAMINTERFACE"),
    "KSPROPERTY_TELEPHONY_CONTROL": SimTypeInt(signed=False, label="KSPROPERTY_TELEPHONY_CONTROL"),
    "KSPROPERTY_TELEPHONY_TOPOLOGY": SimTypeInt(signed=False, label="KSPROPERTY_TELEPHONY_TOPOLOGY"),
    "KSPROPERTY_TIMECODE": SimTypeInt(signed=False, label="KSPROPERTY_TIMECODE"),
    "KSPROPERTY_TIMECODE_NODE_S": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("TimecodeSamp", SimTypeRef("TIMECODE_SAMPLE", SimStruct)),)), name="KSPROPERTY_TIMECODE_NODE_S", pack=False, align=None),
    "KSPROPERTY_TIMECODE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("TimecodeSamp", SimTypeRef("TIMECODE_SAMPLE", SimStruct)),)), name="KSPROPERTY_TIMECODE_S", pack=False, align=None),
    "KSPROPERTY_TOPOLOGY": SimTypeInt(signed=False, label="KSPROPERTY_TOPOLOGY"),
    "KSPROPERTY_TOPOLOGYNODE": SimTypeInt(signed=False, label="KSPROPERTY_TOPOLOGYNODE"),
    "KSPROPERTY_TUNER": SimTypeInt(signed=False, label="KSPROPERTY_TUNER"),
    "KSPROPERTY_TUNER_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ModesSupported", SimTypeInt(signed=False, label="UInt32")), ("VideoMedium", SimTypeRef("KSIDENTIFIER", SimStruct)), ("TVAudioMedium", SimTypeRef("KSIDENTIFIER", SimStruct)), ("RadioAudioMedium", SimTypeRef("KSIDENTIFIER", SimStruct)),)), name="KSPROPERTY_TUNER_CAPS_S", pack=False, align=None),
    "KSPROPERTY_TUNER_FREQUENCY_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Frequency", SimTypeInt(signed=False, label="UInt32")), ("LastFrequency", SimTypeInt(signed=False, label="UInt32")), ("TuningFlags", SimTypeInt(signed=False, label="UInt32")), ("VideoSubChannel", SimTypeInt(signed=False, label="UInt32")), ("AudioSubChannel", SimTypeInt(signed=False, label="UInt32")), ("Channel", SimTypeInt(signed=False, label="UInt32")), ("Country", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_FREQUENCY_S", pack=False, align=None),
    "KSPROPERTY_TUNER_IF_MEDIUM_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("IFMedium", SimTypeRef("KSIDENTIFIER", SimStruct)),)), name="KSPROPERTY_TUNER_IF_MEDIUM_S", pack=False, align=None),
    "KSPROPERTY_TUNER_INPUT_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("InputIndex", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_INPUT_S", pack=False, align=None),
    "KSPROPERTY_TUNER_MODES": SimTypeInt(signed=False, label="KSPROPERTY_TUNER_MODES"),
    "KSPROPERTY_TUNER_MODE_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Mode", SimTypeInt(signed=False, label="UInt32")), ("StandardsSupported", SimTypeInt(signed=False, label="UInt32")), ("MinFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaxFrequency", SimTypeInt(signed=False, label="UInt32")), ("TuningGranularity", SimTypeInt(signed=False, label="UInt32")), ("NumberOfInputs", SimTypeInt(signed=False, label="UInt32")), ("SettlingTime", SimTypeInt(signed=False, label="UInt32")), ("Strategy", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_MODE_CAPS_S", pack=False, align=None),
    "KSPROPERTY_TUNER_MODE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Mode", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_MODE_S", pack=False, align=None),
    "KSPROPERTY_TUNER_NETWORKTYPE_SCAN_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NetworkType", SimTypeBottom(label="Guid")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("NetworkTunerCapabilities", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSPROPERTY_TUNER_NETWORKTYPE_SCAN_CAPS_S", pack=False, align=None),
    "KSPROPERTY_TUNER_SCAN_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("fSupportsHardwareAssistedScanning", SimTypeInt(signed=True, label="Int32")), ("SupportedBroadcastStandards", SimTypeInt(signed=False, label="UInt32")), ("GUIDBucket", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lengthofBucket", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_SCAN_CAPS_S", pack=False, align=None),
    "KSPROPERTY_TUNER_SCAN_STATUS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("LockStatus", SimTypeInt(signed=False, label="TunerLockType")), ("CurrentFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_SCAN_STATUS_S", pack=False, align=None),
    "KSPROPERTY_TUNER_STANDARD_MODE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("AutoDetect", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_TUNER_STANDARD_MODE_S", pack=False, align=None),
    "KSPROPERTY_TUNER_STANDARD_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Standard", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_STANDARD_S", pack=False, align=None),
    "KSPROPERTY_TUNER_STATUS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("CurrentFrequency", SimTypeInt(signed=False, label="UInt32")), ("PLLOffset", SimTypeInt(signed=False, label="UInt32")), ("SignalStrength", SimTypeInt(signed=False, label="UInt32")), ("Busy", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TUNER_STATUS_S", pack=False, align=None),
    "KSPROPERTY_TVAUDIO_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("InputMedium", SimTypeRef("KSIDENTIFIER", SimStruct)), ("OutputMedium", SimTypeRef("KSIDENTIFIER", SimStruct)),)), name="KSPROPERTY_TVAUDIO_CAPS_S", pack=False, align=None),
    "KSPROPERTY_TVAUDIO_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Mode", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_TVAUDIO_S", pack=False, align=None),
    "KSPROPERTY_VBICAP": SimTypeInt(signed=False, label="KSPROPERTY_VBICAP"),
    "KSPROPERTY_VBICODECFILTERING": SimTypeInt(signed=False, label="KSPROPERTY_VBICODECFILTERING"),
    "KSPROPERTY_VBICODECFILTERING_CC_SUBSTREAMS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Substreams", SimTypeRef("VBICODECFILTERING_CC_SUBSTREAMS", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_CC_SUBSTREAMS_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_NABTS_SUBSTREAMS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Substreams", SimTypeRef("VBICODECFILTERING_NABTS_SUBSTREAMS", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_NABTS_SUBSTREAMS_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_SCANLINES_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Scanlines", SimTypeRef("VBICODECFILTERING_SCANLINES", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_SCANLINES_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_STATISTICS_CC_PIN_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Statistics", SimTypeRef("VBICODECFILTERING_STATISTICS_CC_PIN", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_STATISTICS_CC_PIN_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_STATISTICS_CC_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Statistics", SimTypeRef("VBICODECFILTERING_STATISTICS_CC", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_STATISTICS_CC_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_STATISTICS_COMMON_PIN_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Statistics", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON_PIN", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_STATISTICS_COMMON_PIN_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_STATISTICS_COMMON_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Statistics", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_STATISTICS_COMMON_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_STATISTICS_NABTS_PIN_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Statistics", SimTypeRef("VBICODECFILTERING_STATISTICS_NABTS_PIN", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_STATISTICS_NABTS_PIN_S", pack=False, align=None),
    "KSPROPERTY_VBICODECFILTERING_STATISTICS_NABTS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Statistics", SimTypeRef("VBICODECFILTERING_STATISTICS_NABTS", SimStruct)),)), name="KSPROPERTY_VBICODECFILTERING_STATISTICS_NABTS_S", pack=False, align=None),
    "KSPROPERTY_VIDCAP_CAMERACONTROL": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_CAMERACONTROL"),
    "KSPROPERTY_VIDCAP_CROSSBAR": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_CROSSBAR"),
    "KSPROPERTY_VIDCAP_DROPPEDFRAMES": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_DROPPEDFRAMES"),
    "KSPROPERTY_VIDCAP_SELECTOR": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_SELECTOR"),
    "KSPROPERTY_VIDCAP_TVAUDIO": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_TVAUDIO"),
    "KSPROPERTY_VIDCAP_VIDEOCOMPRESSION": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_VIDEOCOMPRESSION"),
    "KSPROPERTY_VIDCAP_VIDEOCONTROL": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_VIDEOCONTROL"),
    "KSPROPERTY_VIDCAP_VIDEODECODER": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_VIDEODECODER"),
    "KSPROPERTY_VIDCAP_VIDEOENCODER": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_VIDEOENCODER"),
    "KSPROPERTY_VIDCAP_VIDEOPROCAMP": SimTypeInt(signed=False, label="KSPROPERTY_VIDCAP_VIDEOPROCAMP"),
    "KSPROPERTY_VIDEOCOMPRESSION_GETINFO_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("DefaultKeyFrameRate", SimTypeInt(signed=True, label="Int32")), ("DefaultPFrameRate", SimTypeInt(signed=True, label="Int32")), ("DefaultQuality", SimTypeInt(signed=True, label="Int32")), ("NumberOfQualitySettings", SimTypeInt(signed=True, label="Int32")), ("Capabilities", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_VIDEOCOMPRESSION_GETINFO_S", pack=False, align=None),
    "KSPROPERTY_VIDEOCOMPRESSION_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_VIDEOCOMPRESSION_S", pack=False, align=None),
    "KSPROPERTY_VIDEOCOMPRESSION_S1": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEOCOMPRESSION_S1", pack=False, align=None),
    "KSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("RangeIndex", SimTypeInt(signed=False, label="UInt32")), ("Dimensions", SimTypeRef("SIZE", SimStruct)), ("CurrentActualFrameRate", SimTypeLongLong(signed=True, label="Int64")), ("CurrentMaxAvailableFrameRate", SimTypeLongLong(signed=True, label="Int64")),)), name="KSPROPERTY_VIDEOCONTROL_ACTUAL_FRAME_RATE_S", pack=False, align=None),
    "KSPROPERTY_VIDEOCONTROL_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("VideoControlCaps", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEOCONTROL_CAPS_S", pack=False, align=None),
    "KSPROPERTY_VIDEOCONTROL_FRAME_RATES_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("RangeIndex", SimTypeInt(signed=False, label="UInt32")), ("Dimensions", SimTypeRef("SIZE", SimStruct)),)), name="KSPROPERTY_VIDEOCONTROL_FRAME_RATES_S", pack=False, align=None),
    "KSPROPERTY_VIDEOCONTROL_MODE_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("Mode", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_VIDEOCONTROL_MODE_S", pack=False, align=None),
    "KSPROPERTY_VIDEODECODER_CAPS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StandardsSupported", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("SettlingTime", SimTypeInt(signed=False, label="UInt32")), ("HSyncPerVSync", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEODECODER_CAPS_S", pack=False, align=None),
    "KSPROPERTY_VIDEODECODER_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEODECODER_S", pack=False, align=None),
    "KSPROPERTY_VIDEODECODER_STATUS2_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NumberOfLines", SimTypeInt(signed=False, label="UInt32")), ("SignalLocked", SimTypeInt(signed=False, label="UInt32")), ("ChromaLock", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEODECODER_STATUS2_S", pack=False, align=None),
    "KSPROPERTY_VIDEODECODER_STATUS_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NumberOfLines", SimTypeInt(signed=False, label="UInt32")), ("SignalLocked", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEODECODER_STATUS_S", pack=False, align=None),
    "KSPROPERTY_VIDEOENCODER_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEOENCODER_S", pack=False, align=None),
    "KSPROPERTY_VIDEOPROCAMP_NODE_S": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEOPROCAMP_NODE_S", pack=False, align=None),
    "KSPROPERTY_VIDEOPROCAMP_NODE_S2": SimStruct(OrderedDict((("NodeProperty", SimTypeRef("KSP_NODE", SimStruct)), ("Value1", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("Value2", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_VIDEOPROCAMP_NODE_S2", pack=False, align=None),
    "KSPROPERTY_VIDEOPROCAMP_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="KSPROPERTY_VIDEOPROCAMP_S", pack=False, align=None),
    "KSPROPERTY_VIDEOPROCAMP_S2": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Value1", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("Value2", SimTypeInt(signed=True, label="Int32")),)), name="KSPROPERTY_VIDEOPROCAMP_S2", pack=False, align=None),
    "KSPROPERTY_VIDMEM_TRANSPORT": SimTypeInt(signed=False, label="KSPROPERTY_VIDMEM_TRANSPORT"),
    "KSPROPERTY_VPCONFIG": SimTypeInt(signed=False, label="KSPROPERTY_VPCONFIG"),
    "KSPROPERTY_WAVE": SimTypeInt(signed=False, label="KSPROPERTY_WAVE"),
    "KSP_BDA_NODE_PIN": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("ulNodeType", SimTypeInt(signed=False, label="UInt32")), ("ulInputPinId", SimTypeInt(signed=False, label="UInt32")), ("ulOutputPinId", SimTypeInt(signed=False, label="UInt32")),)), name="KSP_BDA_NODE_PIN", pack=False, align=None),
    "KSP_NODE": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NodeId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSP_NODE", pack=False, align=None),
    "KSP_NODE_ESPID": SimStruct(OrderedDict((("Property", SimTypeRef("KSP_NODE", SimStruct)), ("EsPid", SimTypeInt(signed=False, label="UInt32")),)), name="KSP_NODE_ESPID", pack=False, align=None),
    "KSP_PIN": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("PinId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="KSP_PIN", pack=False, align=None),
    "KSP_TIMEFORMAT": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("SourceFormat", SimTypeBottom(label="Guid")), ("TargetFormat", SimTypeBottom(label="Guid")), ("Time", SimTypeLongLong(signed=True, label="Int64")),)), name="KSP_TIMEFORMAT", pack=False, align=None),
    "KSQUALITY": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Proportion", SimTypeInt(signed=False, label="UInt32")), ("DeltaTime", SimTypeLongLong(signed=True, label="Int64")),)), name="KSQUALITY", pack=False, align=None),
    "KSQUALITY_MANAGER": SimStruct(OrderedDict((("QualityManager", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSQUALITY_MANAGER", pack=False, align=None),
    "KSQUERYBUFFER": SimStruct(OrderedDict((("Event", SimTypeRef("KSIDENTIFIER", SimStruct)), ("EventData", SimTypePointer(SimTypeRef("KSEVENTDATA", SimStruct), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSQUERYBUFFER", pack=False, align=None),
    "KSRATE": SimStruct(OrderedDict((("PresentationStart", SimTypeLongLong(signed=True, label="Int64")), ("Duration", SimTypeLongLong(signed=True, label="Int64")), ("Interface", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Rate", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="KSRATE", pack=False, align=None),
    "KSRATE_CAPABILITY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Rate", SimTypeRef("KSRATE", SimStruct)),)), name="KSRATE_CAPABILITY", pack=False, align=None),
    "KSRELATIVEEVENT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ObjectHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "ObjectPointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Event", SimTypeRef("KSIDENTIFIER", SimStruct)), ("EventData", SimTypeRef("KSEVENTDATA", SimStruct)),)), name="KSRELATIVEEVENT", pack=False, align=None),
    "KSRESET": SimTypeInt(signed=False, label="KSRESET"),
    "KSRESOLUTION": SimStruct(OrderedDict((("Granularity", SimTypeLongLong(signed=True, label="Int64")), ("Error", SimTypeLongLong(signed=True, label="Int64")),)), name="KSRESOLUTION", pack=False, align=None),
    "KSRTAUDIO_BUFFER": SimStruct(OrderedDict((("BufferAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ActualBufferSize", SimTypeInt(signed=False, label="UInt32")), ("CallMemoryBarrier", SimTypeInt(signed=True, label="Int32")),)), name="KSRTAUDIO_BUFFER", pack=False, align=None),
    "KSRTAUDIO_BUFFER32": SimStruct(OrderedDict((("BufferAddress", SimTypeInt(signed=False, label="UInt32")), ("ActualBufferSize", SimTypeInt(signed=False, label="UInt32")), ("CallMemoryBarrier", SimTypeInt(signed=True, label="Int32")),)), name="KSRTAUDIO_BUFFER32", pack=False, align=None),
    "KSRTAUDIO_BUFFER_PROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RequestedBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_BUFFER_PROPERTY", pack=False, align=None),
    "KSRTAUDIO_BUFFER_PROPERTY32": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypeInt(signed=False, label="UInt32")), ("RequestedBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_BUFFER_PROPERTY32", pack=False, align=None),
    "KSRTAUDIO_BUFFER_PROPERTY_WITH_NOTIFICATION": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RequestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("NotificationCount", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_BUFFER_PROPERTY_WITH_NOTIFICATION", pack=False, align=None),
    "KSRTAUDIO_BUFFER_PROPERTY_WITH_NOTIFICATION32": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypeInt(signed=False, label="UInt32")), ("RequestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("NotificationCount", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_BUFFER_PROPERTY_WITH_NOTIFICATION32", pack=False, align=None),
    "KSRTAUDIO_GETREADPACKET_INFO": SimStruct(OrderedDict((("PacketNumber", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PerformanceCounterValue", SimTypeLongLong(signed=False, label="UInt64")), ("MoreData", SimTypeInt(signed=True, label="Int32")),)), name="KSRTAUDIO_GETREADPACKET_INFO", pack=False, align=None),
    "KSRTAUDIO_HWLATENCY": SimStruct(OrderedDict((("FifoSize", SimTypeInt(signed=False, label="UInt32")), ("ChipsetDelay", SimTypeInt(signed=False, label="UInt32")), ("CodecDelay", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_HWLATENCY", pack=False, align=None),
    "KSRTAUDIO_HWREGISTER": SimStruct(OrderedDict((("Register", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Numerator", SimTypeLongLong(signed=False, label="UInt64")), ("Denominator", SimTypeLongLong(signed=False, label="UInt64")), ("Accuracy", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_HWREGISTER", pack=False, align=None),
    "KSRTAUDIO_HWREGISTER32": SimStruct(OrderedDict((("Register", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Numerator", SimTypeLongLong(signed=False, label="UInt64")), ("Denominator", SimTypeLongLong(signed=False, label="UInt64")), ("Accuracy", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_HWREGISTER32", pack=False, align=None),
    "KSRTAUDIO_HWREGISTER_PROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSRTAUDIO_HWREGISTER_PROPERTY", pack=False, align=None),
    "KSRTAUDIO_HWREGISTER_PROPERTY32": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_HWREGISTER_PROPERTY32", pack=False, align=None),
    "KSRTAUDIO_NOTIFICATION_EVENT_PROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NotificationEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="KSRTAUDIO_NOTIFICATION_EVENT_PROPERTY", pack=False, align=None),
    "KSRTAUDIO_NOTIFICATION_EVENT_PROPERTY32": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("NotificationEvent", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_NOTIFICATION_EVENT_PROPERTY32", pack=False, align=None),
    "KSRTAUDIO_PACKETVREGISTER": SimStruct(OrderedDict((("CompletedPacketCount", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("CompletedPacketQPC", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("CompletedPacketHash", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)),)), name="KSRTAUDIO_PACKETVREGISTER", pack=False, align=None),
    "KSRTAUDIO_PACKETVREGISTER_PROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSRTAUDIO_PACKETVREGISTER_PROPERTY", pack=False, align=None),
    "KSRTAUDIO_SETWRITEPACKET_INFO": SimStruct(OrderedDict((("PacketNumber", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("EosPacketLength", SimTypeInt(signed=False, label="UInt32")),)), name="KSRTAUDIO_SETWRITEPACKET_INFO", pack=False, align=None),
    "KSSOUNDDETECTORPROPERTY": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("EventId", SimTypeBottom(label="Guid")),)), name="KSSOUNDDETECTORPROPERTY", pack=False, align=None),
    "KSSTATE": SimTypeInt(signed=False, label="KSSTATE"),
    "KSSTREAMALLOCATOR_STATUS": SimStruct(OrderedDict((("Framing", SimTypeRef("KSALLOCATOR_FRAMING", SimStruct)), ("AllocatedFrames", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSSTREAMALLOCATOR_STATUS", pack=False, align=None),
    "KSSTREAMALLOCATOR_STATUS_EX": SimStruct(OrderedDict((("Framing", SimTypeRef("KSALLOCATOR_FRAMING_EX", SimStruct)), ("AllocatedFrames", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSSTREAMALLOCATOR_STATUS_EX", pack=False, align=None),
    "KSSTREAM_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("TypeSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("PresentationTime", SimTypeRef("KSTIME", SimStruct)), ("Duration", SimTypeLongLong(signed=True, label="Int64")), ("FrameExtent", SimTypeInt(signed=False, label="UInt32")), ("DataUsed", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OptionsFlags", SimTypeInt(signed=False, label="UInt32")),)), name="KSSTREAM_HEADER", pack=False, align=None),
    "KSSTREAM_METADATA_INFO": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("UsedSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SystemVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSSTREAM_METADATA_INFO", pack=False, align=None),
    "KSSTREAM_SEGMENT": SimStruct(OrderedDict((("KsInterfaceHandler", SimTypeBottom(label="IKsInterfaceHandler")), ("KsDataTypeHandler", SimTypeBottom(label="IKsDataTypeHandler")), ("IoOperation", SimTypeInt(signed=False, label="KSIOOPERATION")), ("CompletionEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="KSSTREAM_SEGMENT", pack=False, align=None),
    "KSSTREAM_UVC_METADATA": SimStruct(OrderedDict((("StartOfFrameTimestamp", SimTypeRef("KSSTREAM_UVC_METADATATYPE_TIMESTAMP", SimStruct)), ("EndOfFrameTimestamp", SimTypeRef("KSSTREAM_UVC_METADATATYPE_TIMESTAMP", SimStruct)),)), name="KSSTREAM_UVC_METADATA", pack=False, align=None),
    "KSSTREAM_UVC_METADATATYPE_TIMESTAMP": SimStruct(OrderedDict((("PresentationTimeStamp", SimTypeInt(signed=False, label="UInt32")), ("SourceClockReference", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "SCRToken": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Reserved0", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="KSSTREAM_UVC_METADATATYPE_TIMESTAMP", pack=False, align=None),
    "KSTART_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["StartContext"]), offset=0),
    "KSTELEPHONY_CALLCONTROL": SimStruct(OrderedDict((("CallType", SimTypeInt(signed=False, label="TELEPHONY_CALLTYPE")), ("CallControlOp", SimTypeInt(signed=False, label="TELEPHONY_CALLCONTROLOP")),)), name="KSTELEPHONY_CALLCONTROL", pack=False, align=None),
    "KSTELEPHONY_CALLINFO": SimStruct(OrderedDict((("CallType", SimTypeInt(signed=False, label="TELEPHONY_CALLTYPE")), ("CallState", SimTypeInt(signed=False, label="TELEPHONY_CALLSTATE")),)), name="KSTELEPHONY_CALLINFO", pack=False, align=None),
    "KSTELEPHONY_PROVIDERCHANGE": SimStruct(OrderedDict((("CallType", SimTypeInt(signed=False, label="TELEPHONY_CALLTYPE")), ("ProviderChangeOp", SimTypeInt(signed=False, label="TELEPHONY_PROVIDERCHANGEOP")),)), name="KSTELEPHONY_PROVIDERCHANGE", pack=False, align=None),
    "KSTIME": SimStruct(OrderedDict((("Time", SimTypeLongLong(signed=True, label="Int64")), ("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="KSTIME", pack=False, align=None),
    "KSTOPOLOGY": SimStruct(OrderedDict((("CategoriesCount", SimTypeInt(signed=False, label="UInt32")), ("Categories", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("TopologyNodesCount", SimTypeInt(signed=False, label="UInt32")), ("TopologyNodes", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("TopologyConnectionsCount", SimTypeInt(signed=False, label="UInt32")), ("TopologyConnections", SimTypePointer(SimTypeRef("KSTOPOLOGY_CONNECTION", SimStruct), offset=0)), ("TopologyNodesNames", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSTOPOLOGY", pack=False, align=None),
    "KSTOPOLOGY_CONNECTION": SimStruct(OrderedDict((("FromNode", SimTypeInt(signed=False, label="UInt32")), ("FromNodePin", SimTypeInt(signed=False, label="UInt32")), ("ToNode", SimTypeInt(signed=False, label="UInt32")), ("ToNodePin", SimTypeInt(signed=False, label="UInt32")),)), name="KSTOPOLOGY_CONNECTION", pack=False, align=None),
    "KSTOPOLOGY_ENDPOINTID": SimStruct(OrderedDict((("TopologyName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("PinId", SimTypeInt(signed=False, label="UInt32")),)), name="KSTOPOLOGY_ENDPOINTID", pack=False, align=None),
    "KSTOPOLOGY_ENDPOINTIDPAIR": SimStruct(OrderedDict((("RenderEndpoint", SimTypeRef("KSTOPOLOGY_ENDPOINTID", SimStruct)), ("CaptureEndpoint", SimTypeRef("KSTOPOLOGY_ENDPOINTID", SimStruct)),)), name="KSTOPOLOGY_ENDPOINTIDPAIR", pack=False, align=None),
    "KSVPMAXPIXELRATE": SimStruct(OrderedDict((("Size", SimTypeRef("KS_AMVPSIZE", SimStruct)), ("MaxPixelsPerSecond", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="KSVPMAXPIXELRATE", pack=False, align=None),
    "KSVPSIZE_PROP": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Size", SimTypeRef("KS_AMVPSIZE", SimStruct)),)), name="KSVPSIZE_PROP", pack=False, align=None),
    "KSVPSURFACEPARAMS": SimStruct(OrderedDict((("dwPitch", SimTypeInt(signed=False, label="UInt32")), ("dwXOrigin", SimTypeInt(signed=False, label="UInt32")), ("dwYOrigin", SimTypeInt(signed=False, label="UInt32")),)), name="KSVPSURFACEPARAMS", pack=False, align=None),
    "KSWAVETABLE_WAVE_DESC": SimStruct(OrderedDict((("Identifier", SimTypeRef("KSIDENTIFIER", SimStruct)), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Looped", SimTypeInt(signed=True, label="Int32")), ("LoopPoint", SimTypeInt(signed=False, label="UInt32")), ("InROM", SimTypeInt(signed=True, label="Int32")), ("Format", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="KSWAVETABLE_WAVE_DESC", pack=False, align=None),
    "KSWAVE_BUFFER": SimStruct(OrderedDict((("Attributes", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("BufferAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KSWAVE_BUFFER", pack=False, align=None),
    "KSWAVE_COMPATCAPS": SimStruct(OrderedDict((("ulDeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="KSWAVE_COMPATCAPS", pack=False, align=None),
    "KSWAVE_INPUT_CAPABILITIES": SimStruct(OrderedDict((("MaximumChannelsPerConnection", SimTypeInt(signed=False, label="UInt32")), ("MinimumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MaximumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MinimumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaximumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("TotalConnections", SimTypeInt(signed=False, label="UInt32")), ("ActiveConnections", SimTypeInt(signed=False, label="UInt32")),)), name="KSWAVE_INPUT_CAPABILITIES", pack=False, align=None),
    "KSWAVE_OUTPUT_CAPABILITIES": SimStruct(OrderedDict((("MaximumChannelsPerConnection", SimTypeInt(signed=False, label="UInt32")), ("MinimumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MaximumBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("MinimumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaximumSampleFrequency", SimTypeInt(signed=False, label="UInt32")), ("TotalConnections", SimTypeInt(signed=False, label="UInt32")), ("StaticConnections", SimTypeInt(signed=False, label="UInt32")), ("StreamingConnections", SimTypeInt(signed=False, label="UInt32")), ("ActiveConnections", SimTypeInt(signed=False, label="UInt32")), ("ActiveStaticConnections", SimTypeInt(signed=False, label="UInt32")), ("ActiveStreamingConnections", SimTypeInt(signed=False, label="UInt32")), ("Total3DConnections", SimTypeInt(signed=False, label="UInt32")), ("Static3DConnections", SimTypeInt(signed=False, label="UInt32")), ("Streaming3DConnections", SimTypeInt(signed=False, label="UInt32")), ("Active3DConnections", SimTypeInt(signed=False, label="UInt32")), ("ActiveStatic3DConnections", SimTypeInt(signed=False, label="UInt32")), ("ActiveStreaming3DConnections", SimTypeInt(signed=False, label="UInt32")), ("TotalSampleMemory", SimTypeInt(signed=False, label="UInt32")), ("FreeSampleMemory", SimTypeInt(signed=False, label="UInt32")), ("LargestFreeContiguousSampleMemory", SimTypeInt(signed=False, label="UInt32")),)), name="KSWAVE_OUTPUT_CAPABILITIES", pack=False, align=None),
    "KSWAVE_VOLUME": SimStruct(OrderedDict((("LeftAttenuation", SimTypeInt(signed=True, label="Int32")), ("RightAttenuation", SimTypeInt(signed=True, label="Int32")),)), name="KSWAVE_VOLUME", pack=False, align=None),
    "KSYNCHRONIZE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["SynchronizeContext"]), offset=0),
    "KSYSTEM_TIME": SimStruct(OrderedDict((("LowPart", SimTypeInt(signed=False, label="UInt32")), ("High1Time", SimTypeInt(signed=True, label="Int32")), ("High2Time", SimTypeInt(signed=True, label="Int32")),)), name="KSYSTEM_TIME", pack=False, align=None),
    "KS_AMPixAspectRatio": SimTypeInt(signed=False, label="KS_AMPixAspectRatio"),
    "KS_AMVPDATAINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMicrosecondsPerField", SimTypeInt(signed=False, label="UInt32")), ("amvpDimInfo", SimTypeRef("KS_AMVPDIMINFO", SimStruct)), ("dwPictAspectRatioX", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioY", SimTypeInt(signed=False, label="UInt32")), ("bEnableDoubleClock", SimTypeInt(signed=True, label="Int32")), ("bEnableVACT", SimTypeInt(signed=True, label="Int32")), ("bDataIsInterlaced", SimTypeInt(signed=True, label="Int32")), ("lHalfLinesOdd", SimTypeInt(signed=True, label="Int32")), ("bFieldPolarityInverted", SimTypeInt(signed=True, label="Int32")), ("dwNumLinesInVREF", SimTypeInt(signed=False, label="UInt32")), ("lHalfLinesEven", SimTypeInt(signed=True, label="Int32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")),)), name="KS_AMVPDATAINFO", pack=False, align=None),
    "KS_AMVPDIMINFO": SimStruct(OrderedDict((("dwFieldWidth", SimTypeInt(signed=False, label="UInt32")), ("dwFieldHeight", SimTypeInt(signed=False, label="UInt32")), ("dwVBIWidth", SimTypeInt(signed=False, label="UInt32")), ("dwVBIHeight", SimTypeInt(signed=False, label="UInt32")), ("rcValidRegion", SimTypeRef("RECT", SimStruct)),)), name="KS_AMVPDIMINFO", pack=False, align=None),
    "KS_AMVPSIZE": SimStruct(OrderedDict((("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")),)), name="KS_AMVPSIZE", pack=False, align=None),
    "KS_AMVP_MODE": SimTypeInt(signed=False, label="KS_AMVP_MODE"),
    "KS_AMVP_SELECTFORMATBY": SimTypeInt(signed=False, label="KS_AMVP_SELECTFORMATBY"),
    "KS_AM_ExactRateChange": SimStruct(OrderedDict((("OutputZeroTime", SimTypeLongLong(signed=True, label="Int64")), ("Rate", SimTypeInt(signed=True, label="Int32")),)), name="KS_AM_ExactRateChange", pack=False, align=None),
    "KS_AM_PROPERTY_TS_RATE_CHANGE": SimTypeInt(signed=False, label="KS_AM_PROPERTY_TS_RATE_CHANGE"),
    "KS_AM_SimpleRateChange": SimStruct(OrderedDict((("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("Rate", SimTypeInt(signed=True, label="Int32")),)), name="KS_AM_SimpleRateChange", pack=False, align=None),
    "KS_ANALOGVIDEOINFO": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwActiveWidth", SimTypeInt(signed=False, label="UInt32")), ("dwActiveHeight", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")),)), name="KS_ANALOGVIDEOINFO", pack=False, align=None),
    "KS_AnalogVideoStandard": SimTypeInt(signed=False, label="KS_AnalogVideoStandard"),
    "KS_BDA_FRAME_INFO": SimStruct(OrderedDict((("ExtendedHeaderSize", SimTypeInt(signed=False, label="UInt32")), ("dwFrameFlags", SimTypeInt(signed=False, label="UInt32")), ("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("ulChannelNumber", SimTypeInt(signed=False, label="UInt32")), ("ulSubchannelNumber", SimTypeInt(signed=False, label="UInt32")), ("ulReason", SimTypeInt(signed=False, label="UInt32")),)), name="KS_BDA_FRAME_INFO", pack=False, align=None),
    "KS_BITMAPINFOHEADER": SimStruct(OrderedDict((("biSize", SimTypeInt(signed=False, label="UInt32")), ("biWidth", SimTypeInt(signed=True, label="Int32")), ("biHeight", SimTypeInt(signed=True, label="Int32")), ("biPlanes", SimTypeShort(signed=False, label="UInt16")), ("biBitCount", SimTypeShort(signed=False, label="UInt16")), ("biCompression", SimTypeInt(signed=False, label="UInt32")), ("biSizeImage", SimTypeInt(signed=False, label="UInt32")), ("biXPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("biYPelsPerMeter", SimTypeInt(signed=True, label="Int32")), ("biClrUsed", SimTypeInt(signed=False, label="UInt32")), ("biClrImportant", SimTypeInt(signed=False, label="UInt32")),)), name="KS_BITMAPINFOHEADER", pack=False, align=None),
    "KS_COLCON": SimStruct(OrderedDict((("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("_bitfield3", SimTypeChar(label="Byte")), ("_bitfield4", SimTypeChar(label="Byte")),)), name="KS_COLCON", pack=False, align=None),
    "KS_COMPRESSION": SimStruct(OrderedDict((("RatioNumerator", SimTypeInt(signed=False, label="UInt32")), ("RatioDenominator", SimTypeInt(signed=False, label="UInt32")), ("RatioConstantMargin", SimTypeInt(signed=False, label="UInt32")),)), name="KS_COMPRESSION", pack=False, align=None),
    "KS_COPY_MACROVISION": SimStruct(OrderedDict((("MACROVISIONLevel", SimTypeInt(signed=False, label="UInt32")),)), name="KS_COPY_MACROVISION", pack=False, align=None),
    "KS_COPY_MACROVISION_LEVEL": SimTypeInt(signed=False, label="KS_COPY_MACROVISION_LEVEL"),
    "KS_CameraControlAsyncOperation": SimTypeInt(signed=False, label="KS_CameraControlAsyncOperation"),
    "KS_CompressionCaps": SimTypeInt(signed=False, label="KS_CompressionCaps"),
    "KS_DATAFORMAT_H264VIDEOINFO": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("H264VideoInfoHeader", SimTypeRef("KS_H264VIDEOINFO", SimStruct)),)), name="KS_DATAFORMAT_H264VIDEOINFO", pack=False, align=None),
    "KS_DATAFORMAT_IMAGEINFO": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("ImageInfoHeader", SimTypeRef("KS_BITMAPINFOHEADER", SimStruct)),)), name="KS_DATAFORMAT_IMAGEINFO", pack=False, align=None),
    "KS_DATAFORMAT_MPEGVIDEOINFO2": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("MpegVideoInfoHeader2", SimTypeRef("KS_MPEGVIDEOINFO2", SimStruct)),)), name="KS_DATAFORMAT_MPEGVIDEOINFO2", pack=False, align=None),
    "KS_DATAFORMAT_VBIINFOHEADER": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("VBIInfoHeader", SimTypeRef("KS_VBIINFOHEADER", SimStruct)),)), name="KS_DATAFORMAT_VBIINFOHEADER", pack=False, align=None),
    "KS_DATAFORMAT_VIDEOINFOHEADER": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("VideoInfoHeader", SimTypeRef("KS_VIDEOINFOHEADER", SimStruct)),)), name="KS_DATAFORMAT_VIDEOINFOHEADER", pack=False, align=None),
    "KS_DATAFORMAT_VIDEOINFOHEADER2": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("VideoInfoHeader2", SimTypeRef("KS_VIDEOINFOHEADER2", SimStruct)),)), name="KS_DATAFORMAT_VIDEOINFOHEADER2", pack=False, align=None),
    "KS_DATAFORMAT_VIDEOINFO_PALETTE": SimStruct(OrderedDict((("DataFormat", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("VideoInfo", SimTypeRef("KS_VIDEOINFO", SimStruct)),)), name="KS_DATAFORMAT_VIDEOINFO_PALETTE", pack=False, align=None),
    "KS_DATARANGE_ANALOGVIDEO": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("AnalogVideoInfo", SimTypeRef("KS_ANALOGVIDEOINFO", SimStruct)),)), name="KS_DATARANGE_ANALOGVIDEO", pack=False, align=None),
    "KS_DATARANGE_BDA_ANTENNA": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="KS_DATARANGE_BDA_ANTENNA", pack=False, align=None),
    "KS_DATARANGE_BDA_TRANSPORT": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("BdaTransportInfo", SimTypeRef("BDA_TRANSPORT_INFO", SimStruct)),)), name="KS_DATARANGE_BDA_TRANSPORT", pack=False, align=None),
    "KS_DATARANGE_H264_VIDEO": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VideoInfoHeader", SimTypeRef("KS_H264VIDEOINFO", SimStruct)),)), name="KS_DATARANGE_H264_VIDEO", pack=False, align=None),
    "KS_DATARANGE_IMAGE": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("ImageInfoHeader", SimTypeRef("KS_BITMAPINFOHEADER", SimStruct)),)), name="KS_DATARANGE_IMAGE", pack=False, align=None),
    "KS_DATARANGE_MPEG1_VIDEO": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VideoInfoHeader", SimTypeRef("KS_MPEG1VIDEOINFO", SimStruct)),)), name="KS_DATARANGE_MPEG1_VIDEO", pack=False, align=None),
    "KS_DATARANGE_MPEG2_VIDEO": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VideoInfoHeader", SimTypeRef("KS_MPEGVIDEOINFO2", SimStruct)),)), name="KS_DATARANGE_MPEG2_VIDEO", pack=False, align=None),
    "KS_DATARANGE_VIDEO": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VideoInfoHeader", SimTypeRef("KS_VIDEOINFOHEADER", SimStruct)),)), name="KS_DATARANGE_VIDEO", pack=False, align=None),
    "KS_DATARANGE_VIDEO2": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VideoInfoHeader", SimTypeRef("KS_VIDEOINFOHEADER2", SimStruct)),)), name="KS_DATARANGE_VIDEO2", pack=False, align=None),
    "KS_DATARANGE_VIDEO_PALETTE": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VideoInfo", SimTypeRef("KS_VIDEOINFO", SimStruct)),)), name="KS_DATARANGE_VIDEO_PALETTE", pack=False, align=None),
    "KS_DATARANGE_VIDEO_VBI": SimStruct(OrderedDict((("DataRange", SimUnion({"Anonymous": SimStruct(OrderedDict((("FormatSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SampleSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MajorFormat", SimTypeBottom(label="Guid")), ("SubFormat", SimTypeBottom(label="Guid")), ("Specifier", SimTypeBottom(label="Guid")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("StreamDescriptionFlags", SimTypeInt(signed=False, label="UInt32")), ("MemoryAllocationFlags", SimTypeInt(signed=False, label="UInt32")), ("ConfigCaps", SimTypeRef("KS_VIDEO_STREAM_CONFIG_CAPS", SimStruct)), ("VBIInfoHeader", SimTypeRef("KS_VBIINFOHEADER", SimStruct)),)), name="KS_DATARANGE_VIDEO_VBI", pack=False, align=None),
    "KS_DVDCOPYSTATE": SimTypeInt(signed=False, label="KS_DVDCOPYSTATE"),
    "KS_DVDCOPY_BUSKEY": SimStruct(OrderedDict((("BusKey", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("Reserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KS_DVDCOPY_BUSKEY", pack=False, align=None),
    "KS_DVDCOPY_CHLGKEY": SimStruct(OrderedDict((("ChlgKey", SimTypeArray(SimTypeChar(label="Byte"), 10)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="KS_DVDCOPY_CHLGKEY", pack=False, align=None),
    "KS_DVDCOPY_DISCKEY": SimStruct(OrderedDict((("DiscKey", SimTypeArray(SimTypeChar(label="Byte"), 2048)),)), name="KS_DVDCOPY_DISCKEY", pack=False, align=None),
    "KS_DVDCOPY_REGION": SimStruct(OrderedDict((("Reserved", SimTypeChar(label="Byte")), ("RegionData", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="KS_DVDCOPY_REGION", pack=False, align=None),
    "KS_DVDCOPY_SET_COPY_STATE": SimStruct(OrderedDict((("DVDCopyState", SimTypeInt(signed=False, label="UInt32")),)), name="KS_DVDCOPY_SET_COPY_STATE", pack=False, align=None),
    "KS_DVDCOPY_TITLEKEY": SimStruct(OrderedDict((("KeyFlags", SimTypeInt(signed=False, label="UInt32")), ("ReservedNT", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("TitleKey", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="KS_DVDCOPY_TITLEKEY", pack=False, align=None),
    "KS_DVD_YCrCb": SimStruct(OrderedDict((("Reserved", SimTypeChar(label="Byte")), ("Y", SimTypeChar(label="Byte")), ("Cr", SimTypeChar(label="Byte")), ("Cb", SimTypeChar(label="Byte")),)), name="KS_DVD_YCrCb", pack=False, align=None),
    "KS_DVD_YUV": SimStruct(OrderedDict((("Reserved", SimTypeChar(label="Byte")), ("Y", SimTypeChar(label="Byte")), ("V", SimTypeChar(label="Byte")), ("U", SimTypeChar(label="Byte")),)), name="KS_DVD_YUV", pack=False, align=None),
    "KS_FRAME_INFO": SimStruct(OrderedDict((("ExtendedHeaderSize", SimTypeInt(signed=False, label="UInt32")), ("dwFrameFlags", SimTypeInt(signed=False, label="UInt32")), ("PictureNumber", SimTypeLongLong(signed=True, label="Int64")), ("DropCount", SimTypeLongLong(signed=True, label="Int64")), ("hDirectDraw", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hSurfaceHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DirectDrawRect", SimTypeRef("RECT", SimStruct)), ("Anonymous1", SimUnion({"lSurfacePitch": SimTypeInt(signed=True, label="Int32"), "Reserved1": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"Anonymous": SimStruct(OrderedDict((("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("Reserved4", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "FrameCompletionNumber": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="KS_FRAME_INFO", pack=False, align=None),
    "KS_FRAMING_ITEM": SimStruct(OrderedDict((("MemoryType", SimTypeBottom(label="Guid")), ("BusType", SimTypeBottom(label="Guid")), ("MemoryFlags", SimTypeInt(signed=False, label="UInt32")), ("BusFlags", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Frames", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"FileAlignment": SimTypeInt(signed=False, label="UInt32"), "FramePitch": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("MemoryTypeWeight", SimTypeInt(signed=False, label="UInt32")), ("PhysicalRange", SimTypeRef("KS_FRAMING_RANGE", SimStruct)), ("FramingRange", SimTypeRef("KS_FRAMING_RANGE_WEIGHTED", SimStruct)),)), name="KS_FRAMING_ITEM", pack=False, align=None),
    "KS_FRAMING_RANGE": SimStruct(OrderedDict((("MinFrameSize", SimTypeInt(signed=False, label="UInt32")), ("MaxFrameSize", SimTypeInt(signed=False, label="UInt32")), ("Stepping", SimTypeInt(signed=False, label="UInt32")),)), name="KS_FRAMING_RANGE", pack=False, align=None),
    "KS_FRAMING_RANGE_WEIGHTED": SimStruct(OrderedDict((("Range", SimTypeRef("KS_FRAMING_RANGE", SimStruct)), ("InPlaceWeight", SimTypeInt(signed=False, label="UInt32")), ("NotInPlaceWeight", SimTypeInt(signed=False, label="UInt32")),)), name="KS_FRAMING_RANGE_WEIGHTED", pack=False, align=None),
    "KS_H264VIDEOINFO": SimStruct(OrderedDict((("wWidth", SimTypeShort(signed=False, label="UInt16")), ("wHeight", SimTypeShort(signed=False, label="UInt16")), ("wSARwidth", SimTypeShort(signed=False, label="UInt16")), ("wSARheight", SimTypeShort(signed=False, label="UInt16")), ("wProfile", SimTypeShort(signed=False, label="UInt16")), ("bLevelIDC", SimTypeChar(label="Byte")), ("wConstrainedToolset", SimTypeShort(signed=False, label="UInt16")), ("bmSupportedUsages", SimTypeInt(signed=False, label="UInt32")), ("bmCapabilities", SimTypeShort(signed=False, label="UInt16")), ("bmSVCCapabilities", SimTypeInt(signed=False, label="UInt32")), ("bmMVCCapabilities", SimTypeInt(signed=False, label="UInt32")), ("dwFrameInterval", SimTypeInt(signed=False, label="UInt32")), ("bMaxCodecConfigDelay", SimTypeChar(label="Byte")), ("bmSupportedSliceModes", SimTypeChar(label="Byte")), ("bmSupportedSyncFrameTypes", SimTypeChar(label="Byte")), ("bResolutionScaling", SimTypeChar(label="Byte")), ("bSimulcastSupport", SimTypeChar(label="Byte")), ("bmSupportedRateControlModes", SimTypeChar(label="Byte")), ("wMaxMBperSecOneResolutionNoScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecTwoResolutionsNoScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecThreeResolutionsNoScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecFourResolutionsNoScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecOneResolutionTemporalScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecTwoResolutionsTemporalScalablility", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecThreeResolutionsTemporalScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecFourResolutionsTemporalScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecOneResolutionTemporalQualityScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecTwoResolutionsTemporalQualityScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecThreeResolutionsTemporalQualityScalablity", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecFourResolutionsTemporalQualityScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecOneResolutionTemporalSpatialScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecTwoResolutionsTemporalSpatialScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecThreeResolutionsTemporalSpatialScalablity", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecFourResolutionsTemporalSpatialScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecOneResolutionFullScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecTwoResolutionsFullScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecThreeResolutionsFullScalability", SimTypeShort(signed=False, label="UInt16")), ("wMaxMBperSecFourResolutionsFullScalability", SimTypeShort(signed=False, label="UInt16")),)), name="KS_H264VIDEOINFO", pack=False, align=None),
    "KS_LogicalMemoryType": SimTypeInt(signed=False, label="KS_LogicalMemoryType"),
    "KS_MPEG1VIDEOINFO": SimStruct(OrderedDict((("hdr", SimTypeRef("KS_VIDEOINFOHEADER", SimStruct)), ("dwStartTimeCode", SimTypeInt(signed=False, label="UInt32")), ("cbSequenceHeader", SimTypeInt(signed=False, label="UInt32")), ("bSequenceHeader", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="KS_MPEG1VIDEOINFO", pack=False, align=None),
    "KS_MPEG2Level": SimTypeInt(signed=False, label="KS_MPEG2Level"),
    "KS_MPEG2Profile": SimTypeInt(signed=False, label="KS_MPEG2Profile"),
    "KS_MPEGAUDIOINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="KS_MPEGAUDIOINFO", pack=False, align=None),
    "KS_MPEGVIDEOINFO2": SimStruct(OrderedDict((("hdr", SimTypeRef("KS_VIDEOINFOHEADER2", SimStruct)), ("dwStartTimeCode", SimTypeInt(signed=False, label="UInt32")), ("cbSequenceHeader", SimTypeInt(signed=False, label="UInt32")), ("dwProfile", SimTypeInt(signed=False, label="UInt32")), ("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bSequenceHeader", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="KS_MPEGVIDEOINFO2", pack=False, align=None),
    "KS_PhysicalConnectorType": SimTypeInt(signed=False, label="KS_PhysicalConnectorType"),
    "KS_RGBQUAD": SimStruct(OrderedDict((("rgbBlue", SimTypeChar(label="Byte")), ("rgbGreen", SimTypeChar(label="Byte")), ("rgbRed", SimTypeChar(label="Byte")), ("rgbReserved", SimTypeChar(label="Byte")),)), name="KS_RGBQUAD", pack=False, align=None),
    "KS_SEEKING_CAPABILITIES": SimTypeInt(signed=False, label="KS_SEEKING_CAPABILITIES"),
    "KS_SEEKING_FLAGS": SimTypeInt(signed=False, label="KS_SEEKING_FLAGS"),
    "KS_TRUECOLORINFO": SimStruct(OrderedDict((("dwBitMasks", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("bmiColors", SimTypeArray(SimTypeRef("KS_RGBQUAD", SimStruct), 256)),)), name="KS_TRUECOLORINFO", pack=False, align=None),
    "KS_TUNER_STRATEGY": SimTypeInt(signed=False, label="KS_TUNER_STRATEGY"),
    "KS_TUNER_TUNING_FLAGS": SimTypeInt(signed=False, label="KS_TUNER_TUNING_FLAGS"),
    "KS_TVTUNER_CHANGE_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("dwAnalogVideoStandard", SimTypeInt(signed=False, label="UInt32")), ("dwChannel", SimTypeInt(signed=False, label="UInt32")),)), name="KS_TVTUNER_CHANGE_INFO", pack=False, align=None),
    "KS_VBIINFOHEADER": SimStruct(OrderedDict((("StartLine", SimTypeInt(signed=False, label="UInt32")), ("EndLine", SimTypeInt(signed=False, label="UInt32")), ("SamplingFrequency", SimTypeInt(signed=False, label="UInt32")), ("MinLineStartTime", SimTypeInt(signed=False, label="UInt32")), ("MaxLineStartTime", SimTypeInt(signed=False, label="UInt32")), ("ActualLineStartTime", SimTypeInt(signed=False, label="UInt32")), ("ActualLineEndTime", SimTypeInt(signed=False, label="UInt32")), ("VideoStandard", SimTypeInt(signed=False, label="UInt32")), ("SamplesPerLine", SimTypeInt(signed=False, label="UInt32")), ("StrideInBytes", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="KS_VBIINFOHEADER", pack=False, align=None),
    "KS_VBI_FRAME_INFO": SimStruct(OrderedDict((("ExtendedHeaderSize", SimTypeInt(signed=False, label="UInt32")), ("dwFrameFlags", SimTypeInt(signed=False, label="UInt32")), ("PictureNumber", SimTypeLongLong(signed=True, label="Int64")), ("DropCount", SimTypeLongLong(signed=True, label="Int64")), ("dwSamplingFrequency", SimTypeInt(signed=False, label="UInt32")), ("TvTunerChangeInfo", SimTypeRef("KS_TVTUNER_CHANGE_INFO", SimStruct)), ("VBIInfoHeader", SimTypeRef("KS_VBIINFOHEADER", SimStruct)),)), name="KS_VBI_FRAME_INFO", pack=False, align=None),
    "KS_VIDEODECODER_FLAGS": SimTypeInt(signed=False, label="KS_VIDEODECODER_FLAGS"),
    "KS_VIDEOINFO": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("bmiHeader", SimTypeRef("KS_BITMAPINFOHEADER", SimStruct)), ("Anonymous", SimUnion({"bmiColors": SimTypeArray(SimTypeRef("KS_RGBQUAD", SimStruct), 256), "dwBitMasks": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3), "TrueColorInfo": SimTypeRef("KS_TRUECOLORINFO", SimStruct)}, name="<anon>", label="None")),)), name="KS_VIDEOINFO", pack=False, align=None),
    "KS_VIDEOINFOHEADER": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("bmiHeader", SimTypeRef("KS_BITMAPINFOHEADER", SimStruct)),)), name="KS_VIDEOINFOHEADER", pack=False, align=None),
    "KS_VIDEOINFOHEADER2": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("dwInterlaceFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCopyProtectFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioX", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioY", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwControlFlags": SimTypeInt(signed=False, label="UInt32"), "dwReserved1": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("bmiHeader", SimTypeRef("KS_BITMAPINFOHEADER", SimStruct)),)), name="KS_VIDEOINFOHEADER2", pack=False, align=None),
    "KS_VIDEO_STREAM_CONFIG_CAPS": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("VideoStandard", SimTypeInt(signed=False, label="UInt32")), ("InputSize", SimTypeRef("SIZE", SimStruct)), ("MinCroppingSize", SimTypeRef("SIZE", SimStruct)), ("MaxCroppingSize", SimTypeRef("SIZE", SimStruct)), ("CropGranularityX", SimTypeInt(signed=True, label="Int32")), ("CropGranularityY", SimTypeInt(signed=True, label="Int32")), ("CropAlignX", SimTypeInt(signed=True, label="Int32")), ("CropAlignY", SimTypeInt(signed=True, label="Int32")), ("MinOutputSize", SimTypeRef("SIZE", SimStruct)), ("MaxOutputSize", SimTypeRef("SIZE", SimStruct)), ("OutputGranularityX", SimTypeInt(signed=True, label="Int32")), ("OutputGranularityY", SimTypeInt(signed=True, label="Int32")), ("StretchTapsX", SimTypeInt(signed=True, label="Int32")), ("StretchTapsY", SimTypeInt(signed=True, label="Int32")), ("ShrinkTapsX", SimTypeInt(signed=True, label="Int32")), ("ShrinkTapsY", SimTypeInt(signed=True, label="Int32")), ("MinFrameInterval", SimTypeLongLong(signed=True, label="Int64")), ("MaxFrameInterval", SimTypeLongLong(signed=True, label="Int64")), ("MinBitsPerSecond", SimTypeInt(signed=True, label="Int32")), ("MaxBitsPerSecond", SimTypeInt(signed=True, label="Int32")),)), name="KS_VIDEO_STREAM_CONFIG_CAPS", pack=False, align=None),
    "KS_VideoControlFlags": SimTypeInt(signed=False, label="KS_VideoControlFlags"),
    "KS_VideoStreamingHints": SimTypeInt(signed=False, label="KS_VideoStreamingHints"),
    "KTIMER": SimStruct(OrderedDict((("Header", SimTypeRef("DISPATCHER_HEADER", SimStruct)), ("DueTime", SimTypeLongLong(signed=False, label="UInt64")), ("TimerListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("Dpc", SimTypePointer(SimTypeRef("KDPC", SimStruct), offset=0)), ("Period", SimTypeInt(signed=False, label="UInt32")),)), name="KTIMER", pack=False, align=None),
    "KTM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KTMOBJECT_CURSOR": SimStruct(OrderedDict((("LastQuery", SimTypeBottom(label="Guid")), ("ObjectIdCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectIds", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="KTMOBJECT_CURSOR", pack=False, align=None),
    "KTMOBJECT_TYPE": SimTypeInt(signed=False, label="KTMOBJECT_TYPE"),
    "KTRANSACTION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KTRIAGE_DUMP_DATA_ARRAY": SimStruct(OrderedDict((("List", SimTypeRef("LIST_ENTRY", SimStruct)), ("NumBlocksUsed", SimTypeInt(signed=False, label="UInt32")), ("NumBlocksTotal", SimTypeInt(signed=False, label="UInt32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("MaxDataSize", SimTypeInt(signed=False, label="UInt32")), ("ComponentNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ComponentName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Blocks", SimTypePointer(SimTypeRef("KADDRESS_RANGE", SimStruct), offset=0)),)), name="KTRIAGE_DUMP_DATA_ARRAY", pack=False, align=None),
    "KTSS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "KUSER_SHARED_DATA": SimStruct(OrderedDict((("TickCountLowDeprecated", SimTypeInt(signed=False, label="UInt32")), ("TickCountMultiplier", SimTypeInt(signed=False, label="UInt32")), ("InterruptTime", SimTypeRef("KSYSTEM_TIME", SimStruct)), ("SystemTime", SimTypeRef("KSYSTEM_TIME", SimStruct)), ("TimeZoneBias", SimTypeRef("KSYSTEM_TIME", SimStruct)), ("ImageNumberLow", SimTypeShort(signed=False, label="UInt16")), ("ImageNumberHigh", SimTypeShort(signed=False, label="UInt16")), ("NtSystemRoot", SimTypeArray(SimTypeChar(label="Char"), 260)), ("MaxStackTraceDepth", SimTypeInt(signed=False, label="UInt32")), ("CryptoExponent", SimTypeInt(signed=False, label="UInt32")), ("TimeZoneId", SimTypeInt(signed=False, label="UInt32")), ("LargePageMinimum", SimTypeInt(signed=False, label="UInt32")), ("AitSamplingValue", SimTypeInt(signed=False, label="UInt32")), ("AppCompatFlag", SimTypeInt(signed=False, label="UInt32")), ("RNGSeedVersion", SimTypeLongLong(signed=False, label="UInt64")), ("GlobalValidationRunlevel", SimTypeInt(signed=False, label="UInt32")), ("TimeZoneBiasStamp", SimTypeInt(signed=True, label="Int32")), ("NtBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("NtProductType", SimTypeInt(signed=False, label="NT_PRODUCT_TYPE")), ("ProductTypeIsValid", SimTypeChar(label="Byte")), ("Reserved0", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NativeProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("NtMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("NtMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ProcessorFeatures", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("TimeSlip", SimTypeInt(signed=False, label="UInt32")), ("AlternativeArchitecture", SimTypeInt(signed=False, label="ALTERNATIVE_ARCHITECTURE_TYPE")), ("BootId", SimTypeInt(signed=False, label="UInt32")), ("SystemExpirationDate", SimTypeLongLong(signed=True, label="Int64")), ("SuiteMask", SimTypeInt(signed=False, label="UInt32")), ("KdDebuggerEnabled", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"MitigationPolicies": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("CyclesPerYield", SimTypeShort(signed=False, label="UInt16")), ("ActiveConsoleId", SimTypeInt(signed=False, label="UInt32")), ("DismountCount", SimTypeInt(signed=False, label="UInt32")), ("ComPlusPackage", SimTypeInt(signed=False, label="UInt32")), ("LastSystemRITEventTickCount", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPhysicalPages", SimTypeInt(signed=False, label="UInt32")), ("SafeBootMode", SimTypeChar(label="Byte")), ("Anonymous2", SimUnion({"VirtualizationFlags": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Reserved12", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Anonymous3", SimUnion({"SharedDataFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("DataFlagsPad", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("TestRetInstruction", SimTypeLongLong(signed=False, label="UInt64")), ("QpcFrequency", SimTypeLongLong(signed=True, label="Int64")), ("SystemCall", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("SystemCallPad", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)), ("Anonymous4", SimUnion({"TickCount": SimTypeRef("KSYSTEM_TIME", SimStruct), "TickCountQuad": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("ReservedTickCountOverlay", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("TickCountPad", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Cookie", SimTypeInt(signed=False, label="UInt32")), ("CookiePad", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ConsoleSessionForegroundProcessId", SimTypeLongLong(signed=True, label="Int64")), ("TimeUpdateLock", SimTypeLongLong(signed=False, label="UInt64")), ("BaselineSystemTimeQpc", SimTypeLongLong(signed=False, label="UInt64")), ("BaselineInterruptTimeQpc", SimTypeLongLong(signed=False, label="UInt64")), ("QpcSystemTimeIncrement", SimTypeLongLong(signed=False, label="UInt64")), ("QpcInterruptTimeIncrement", SimTypeLongLong(signed=False, label="UInt64")), ("QpcSystemTimeIncrementShift", SimTypeChar(label="Byte")), ("QpcInterruptTimeIncrementShift", SimTypeChar(label="Byte")), ("UnparkedProcessorCount", SimTypeShort(signed=False, label="UInt16")), ("EnclaveFeatureMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("TelemetryCoverageRound", SimTypeInt(signed=False, label="UInt32")), ("UserModeGlobalLogger", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)), ("ImageFileExecutionOptions", SimTypeInt(signed=False, label="UInt32")), ("LangGenerationCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved4", SimTypeLongLong(signed=False, label="UInt64")), ("InterruptTimeBias", SimTypeLongLong(signed=False, label="UInt64")), ("QpcBias", SimTypeLongLong(signed=False, label="UInt64")), ("ActiveProcessorCount", SimTypeInt(signed=False, label="UInt32")), ("ActiveGroupCount", SimTypeChar(label="Byte")), ("Reserved9", SimTypeChar(label="Byte")), ("Anonymous5", SimUnion({"QpcData": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("QpcBypassEnabled", SimTypeChar(label="Byte")), ("QpcShift", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("TimeZoneBiasEffectiveStart", SimTypeLongLong(signed=True, label="Int64")), ("TimeZoneBiasEffectiveEnd", SimTypeLongLong(signed=True, label="Int64")), ("XState", SimTypeRef("XSTATE_CONFIGURATION", SimStruct)), ("FeatureConfigurationChangeStamp", SimTypeRef("KSYSTEM_TIME", SimStruct)), ("Spare", SimTypeInt(signed=False, label="UInt32")), ("UserPointerAuthMask", SimTypeLongLong(signed=False, label="UInt64")),)), name="KUSER_SHARED_DATA", pack=False, align=None),
    "KWAIT_BLOCK": SimStruct(OrderedDict((("WaitListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("WaitType", SimTypeChar(label="Byte")), ("BlockState", SimTypeChar(label="Byte")), ("WaitKey", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"Thread": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), "NotificationQueue": SimTypePointer(SimTypeRef("KQUEUE", SimStruct), offset=0), "Dpc": SimTypePointer(SimTypeRef("KDPC", SimStruct), offset=0)}, name="<anon>", label="None")), ("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SparePtr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KWAIT_BLOCK", pack=False, align=None),
    "KWAIT_CHAIN": SimStruct(OrderedDict((("Head", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="KWAIT_CHAIN", pack=False, align=None),
    "KWAIT_REASON": SimTypeInt(signed=False, label="KWAIT_REASON"),
    "KeyAttestationClaimType": SimTypeInt(signed=False, label="KeyAttestationClaimType"),
    "KeyCombinationType": SimTypeInt(signed=False, label="KeyCombinationType"),
    "KeyCredentialManagerInfo": SimStruct(OrderedDict((("containerId", SimTypeBottom(label="Guid")),)), name="KeyCredentialManagerInfo", pack=False, align=None),
    "KeyCredentialManagerOperationErrorStates": SimTypeInt(signed=False, label="KeyCredentialManagerOperationErrorStates"),
    "KeyCredentialManagerOperationType": SimTypeInt(signed=False, label="KeyCredentialManagerOperationType"),
    "KeyIdentifierHashAlgorithm": SimTypeInt(signed=False, label="KeyIdentifierHashAlgorithm"),
    "KnownGamingPrivileges": SimTypeInt(signed=False, label="KnownGamingPrivileges"),
    "KspCompleteTokenFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Token"]), offset=0),
    "KspDeleteContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "LsaContextId"]), offset=0),
    "KspGetTokenFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "ImpersonationToken", "RawToken"]), offset=0),
    "KspInitContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "ContextData", "NewContextId"]), offset=0),
    "KspInitPackageFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_KERNEL_FUNCTIONS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FunctionTable"]), offset=0),
    "KspMakeSignatureFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "fQOP", "Message", "MessageSeqNo"]), offset=0),
    "KspMapHandleFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "LsaContextId"]), offset=0),
    "KspQueryAttributesFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Attribute", "Buffer"]), offset=0),
    "KspSealMessageFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "fQOP", "Message", "MessageSeqNo"]), offset=0),
    "KspSerializeAuthDataFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvAuthData", "Size", "SerializedData"]), offset=0),
    "KspSetPagingModeFn": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["PagingMode"]), offset=0),
    "KspUnsealMessageFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Message", "MessageSeqNo", "pfQOP"]), offset=0),
    "KspVerifySignatureFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Message", "MessageSeqNo", "pfQOP"]), offset=0),
    "L2TP_CONFIG_PARAMS0": SimStruct(OrderedDict((("dwNumPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")),)), name="L2TP_CONFIG_PARAMS0", pack=False, align=None),
    "L2TP_CONFIG_PARAMS1": SimStruct(OrderedDict((("dwNumPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTunnelConfigParamFlags", SimTypeInt(signed=False, label="UInt32")), ("TunnelConfigParams", SimTypeRef("L2TP_TUNNEL_CONFIG_PARAMS2", SimStruct)),)), name="L2TP_CONFIG_PARAMS1", pack=False, align=None),
    "L2TP_TUNNEL_CONFIG_PARAMS1": SimStruct(OrderedDict((("dwIdleTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSizeForRenegotiation", SimTypeInt(signed=False, label="UInt32")), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)),)), name="L2TP_TUNNEL_CONFIG_PARAMS1", pack=False, align=None),
    "L2TP_TUNNEL_CONFIG_PARAMS2": SimStruct(OrderedDict((("dwIdleTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSizeForRenegotiation", SimTypeInt(signed=False, label="UInt32")), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)), ("dwMmSaLifeTime", SimTypeInt(signed=False, label="UInt32")),)), name="L2TP_TUNNEL_CONFIG_PARAMS2", pack=False, align=None),
    "L2_NOTIFICATION_DATA": SimStruct(OrderedDict((("NotificationSource", SimTypeInt(signed=False, label="WLAN_NOTIFICATION_SOURCES")), ("NotificationCode", SimTypeInt(signed=False, label="UInt32")), ("InterfaceGuid", SimTypeBottom(label="Guid")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="L2_NOTIFICATION_DATA", pack=False, align=None),
    "LABELSTATES": SimTypeInt(signed=False, label="LABELSTATES"),
    "LANA_ENUM": SimStruct(OrderedDict((("length", SimTypeChar(label="Byte")), ("lana", SimTypeArray(SimTypeChar(label="Byte"), 255)),)), name="LANA_ENUM", pack=False, align=None),
    "LANGGROUPLOCALE_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LANGGROUPLOCALE_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LANGUAGEGROUP_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LANGUAGEGROUP_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LANG_BAR_ITEM_ICON_MODE_FLAGS": SimTypeInt(signed=False, label="LANG_BAR_ITEM_ICON_MODE_FLAGS"),
    "LARGE_MCB": SimStruct(OrderedDict((("GuardedMutex", SimTypePointer(SimTypeRef("FAST_MUTEX", SimStruct), offset=0)), ("BaseMcb", SimTypeRef("BASE_MCB", SimStruct)),)), name="LARGE_MCB", pack=False, align=None),
    "LASTINPUTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")),)), name="LASTINPUTINFO", pack=False, align=None),
    "LATENCY_MONITOR_FEATURE_STATUS": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="LATENCY_MONITOR_FEATURE_STATUS", pack=False, align=None),
    "LATENCY_STAMP": SimStruct(OrderedDict((("Trim3", SimTypeLongLong(signed=False, label="UInt64")), ("Write3", SimTypeLongLong(signed=False, label="UInt64")), ("Read3", SimTypeLongLong(signed=False, label="UInt64")), ("Trim2", SimTypeLongLong(signed=False, label="UInt64")), ("Write2", SimTypeLongLong(signed=False, label="UInt64")), ("Read2", SimTypeLongLong(signed=False, label="UInt64")), ("Trim1", SimTypeLongLong(signed=False, label="UInt64")), ("Write1", SimTypeLongLong(signed=False, label="UInt64")), ("Read1", SimTypeLongLong(signed=False, label="UInt64")), ("Trim0", SimTypeLongLong(signed=False, label="UInt64")), ("Write0", SimTypeLongLong(signed=False, label="UInt64")), ("Read0", SimTypeLongLong(signed=False, label="UInt64")),)), name="LATENCY_STAMP", pack=False, align=None),
    "LATENCY_STAMP_UNITS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="LATENCY_STAMP_UNITS", pack=False, align=None),
    "LATENCY_TIME": SimTypeInt(signed=False, label="LATENCY_TIME"),
    "LATTICE_METRICS": SimStruct(OrderedDict((("lsBaseline", SimTypeRef("LINE_SEGMENT", SimStruct)), ("iMidlineOffset", SimTypeShort(signed=True, label="Int16")),)), name="LATTICE_METRICS", pack=False, align=None),
    "LAYERED_WINDOW_ATTRIBUTES_FLAGS": SimTypeInt(signed=False, label="LAYERED_WINDOW_ATTRIBUTES_FLAGS"),
    "LAYERPLANEDESCRIPTOR": SimStruct(OrderedDict((("nSize", SimTypeShort(signed=False, label="UInt16")), ("nVersion", SimTypeShort(signed=False, label="UInt16")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iPixelType", SimTypeChar(label="Byte")), ("cColorBits", SimTypeChar(label="Byte")), ("cRedBits", SimTypeChar(label="Byte")), ("cRedShift", SimTypeChar(label="Byte")), ("cGreenBits", SimTypeChar(label="Byte")), ("cGreenShift", SimTypeChar(label="Byte")), ("cBlueBits", SimTypeChar(label="Byte")), ("cBlueShift", SimTypeChar(label="Byte")), ("cAlphaBits", SimTypeChar(label="Byte")), ("cAlphaShift", SimTypeChar(label="Byte")), ("cAccumBits", SimTypeChar(label="Byte")), ("cAccumRedBits", SimTypeChar(label="Byte")), ("cAccumGreenBits", SimTypeChar(label="Byte")), ("cAccumBlueBits", SimTypeChar(label="Byte")), ("cAccumAlphaBits", SimTypeChar(label="Byte")), ("cDepthBits", SimTypeChar(label="Byte")), ("cStencilBits", SimTypeChar(label="Byte")), ("cAuxBuffers", SimTypeChar(label="Byte")), ("iLayerPlane", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("crTransparent", SimTypeInt(signed=False, label="UInt32")),)), name="LAYERPLANEDESCRIPTOR", pack=False, align=None),
    "LCN_WEAK_REFERENCE_BUFFER": SimStruct(OrderedDict((("Lcn", SimTypeLongLong(signed=True, label="Int64")), ("LengthInClusters", SimTypeLongLong(signed=True, label="Int64")), ("ReferenceCount", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeShort(signed=False, label="UInt16")),)), name="LCN_WEAK_REFERENCE_BUFFER", pack=False, align=None),
    "LCN_WEAK_REFERENCE_CREATE_INPUT_BUFFER": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="LCN_WEAK_REFERENCE_CREATE_INPUT_BUFFER", pack=False, align=None),
    "LCSCSTYPE": SimTypeInt(signed=False, label="LCSCSTYPE"),
    "LDAP": SimStruct(OrderedDict((("ld_sb", SimTypeRef("_ld_sb_e__Struct", SimStruct)), ("ld_host", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ld_version", SimTypeInt(signed=False, label="UInt32")), ("ld_lberoptions", SimTypeChar(label="Byte")), ("ld_deref", SimTypeInt(signed=False, label="UInt32")), ("ld_timelimit", SimTypeInt(signed=False, label="UInt32")), ("ld_sizelimit", SimTypeInt(signed=False, label="UInt32")), ("ld_errno", SimTypeInt(signed=False, label="UInt32")), ("ld_matched", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ld_error", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ld_msgid", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 25)), ("ld_cldaptries", SimTypeInt(signed=False, label="UInt32")), ("ld_cldaptimeout", SimTypeInt(signed=False, label="UInt32")), ("ld_refhoplimit", SimTypeInt(signed=False, label="UInt32")), ("ld_options", SimTypeInt(signed=False, label="UInt32")),)), name="LDAP", pack=False, align=None),
    "LDAPAPIFeatureInfoA": SimStruct(OrderedDict((("ldapaif_info_version", SimTypeInt(signed=True, label="Int32")), ("ldapaif_name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ldapaif_version", SimTypeInt(signed=True, label="Int32")),)), name="LDAPAPIFeatureInfoA", pack=False, align=None),
    "LDAPAPIFeatureInfoW": SimStruct(OrderedDict((("ldapaif_info_version", SimTypeInt(signed=True, label="Int32")), ("ldapaif_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ldapaif_version", SimTypeInt(signed=True, label="Int32")),)), name="LDAPAPIFeatureInfoW", pack=False, align=None),
    "LDAPAPIInfoA": SimStruct(OrderedDict((("ldapai_info_version", SimTypeInt(signed=True, label="Int32")), ("ldapai_api_version", SimTypeInt(signed=True, label="Int32")), ("ldapai_protocol_version", SimTypeInt(signed=True, label="Int32")), ("ldapai_extensions", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("ldapai_vendor_name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ldapai_vendor_version", SimTypeInt(signed=True, label="Int32")),)), name="LDAPAPIInfoA", pack=False, align=None),
    "LDAPAPIInfoW": SimStruct(OrderedDict((("ldapai_info_version", SimTypeInt(signed=True, label="Int32")), ("ldapai_api_version", SimTypeInt(signed=True, label="Int32")), ("ldapai_protocol_version", SimTypeInt(signed=True, label="Int32")), ("ldapai_extensions", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("ldapai_vendor_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ldapai_vendor_version", SimTypeInt(signed=True, label="Int32")),)), name="LDAPAPIInfoW", pack=False, align=None),
    "LDAPControlA": SimStruct(OrderedDict((("ldctl_oid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ldctl_value", SimTypeRef("LDAP_BERVAL", SimStruct)), ("ldctl_iscritical", SimTypeChar(label="Byte")),)), name="LDAPControlA", pack=False, align=None),
    "LDAPControlW": SimStruct(OrderedDict((("ldctl_oid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ldctl_value", SimTypeRef("LDAP_BERVAL", SimStruct)), ("ldctl_iscritical", SimTypeChar(label="Byte")),)), name="LDAPControlW", pack=False, align=None),
    "LDAPMessage": SimStruct(OrderedDict((("lm_msgid", SimTypeInt(signed=False, label="UInt32")), ("lm_msgtype", SimTypeInt(signed=False, label="UInt32")), ("lm_ber", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lm_chain", SimTypePointer(SimTypeRef("LDAPMessage", SimStruct), offset=0)), ("lm_next", SimTypePointer(SimTypeRef("LDAPMessage", SimStruct), offset=0)), ("lm_time", SimTypeInt(signed=False, label="UInt32")), ("Connection", SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0)), ("Request", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lm_returncode", SimTypeInt(signed=False, label="UInt32")), ("lm_referral", SimTypeShort(signed=False, label="UInt16")), ("lm_chased", SimTypeChar(label="Byte")), ("lm_eom", SimTypeChar(label="Byte")), ("ConnectionReferenced", SimTypeChar(label="Byte")),)), name="LDAPMessage", pack=False, align=None),
    "LDAPModA": SimStruct(OrderedDict((("mod_op", SimTypeInt(signed=False, label="UInt32")), ("mod_type", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("mod_vals", SimUnion({"modv_strvals": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), "modv_bvals": SimTypePointer(SimTypePointer(SimTypeRef("LDAP_BERVAL", SimStruct), offset=0), offset=0)}, name="<anon>", label="None")),)), name="LDAPModA", pack=False, align=None),
    "LDAPModW": SimStruct(OrderedDict((("mod_op", SimTypeInt(signed=False, label="UInt32")), ("mod_type", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("mod_vals", SimUnion({"modv_strvals": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "modv_bvals": SimTypePointer(SimTypePointer(SimTypeRef("LDAP_BERVAL", SimStruct), offset=0), offset=0)}, name="<anon>", label="None")),)), name="LDAPModW", pack=False, align=None),
    "LDAPSortKeyA": SimStruct(OrderedDict((("sk_attrtype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("sk_matchruleoid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("sk_reverseorder", SimTypeChar(label="Byte")),)), name="LDAPSortKeyA", pack=False, align=None),
    "LDAPSortKeyW": SimStruct(OrderedDict((("sk_attrtype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sk_matchruleoid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sk_reverseorder", SimTypeChar(label="Byte")),)), name="LDAPSortKeyW", pack=False, align=None),
    "LDAPVLVInfo": SimStruct(OrderedDict((("ldvlv_version", SimTypeInt(signed=True, label="Int32")), ("ldvlv_before_count", SimTypeInt(signed=False, label="UInt32")), ("ldvlv_after_count", SimTypeInt(signed=False, label="UInt32")), ("ldvlv_offset", SimTypeInt(signed=False, label="UInt32")), ("ldvlv_count", SimTypeInt(signed=False, label="UInt32")), ("ldvlv_attrvalue", SimTypePointer(SimTypeRef("LDAP_BERVAL", SimStruct), offset=0)), ("ldvlv_context", SimTypePointer(SimTypeRef("LDAP_BERVAL", SimStruct), offset=0)), ("ldvlv_extradata", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="LDAPVLVInfo", pack=False, align=None),
    "LDAP_BERVAL": SimStruct(OrderedDict((("bv_len", SimTypeInt(signed=False, label="UInt32")), ("bv_val", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="LDAP_BERVAL", pack=False, align=None),
    "LDAP_REFERRAL_CALLBACK": SimStruct(OrderedDict((("SizeOfCallbacks", SimTypeInt(signed=False, label="UInt32")), ("QueryForConnection", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PrimaryConnection", "ReferralFromConnection", "NewDN", "HostName", "PortNumber", "SecAuthIdentity", "CurrentUserToken", "ConnectionToUse"]), offset=0)), ("NotifyRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["PrimaryConnection", "ReferralFromConnection", "NewDN", "HostName", "NewConnection", "PortNumber", "SecAuthIdentity", "CurrentUser", "ErrorCodeFromBind"]), offset=0)), ("DereferenceRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PrimaryConnection", "ConnectionToDereference"]), offset=0)),)), name="LDAP_REFERRAL_CALLBACK", pack=False, align=None),
    "LDAP_RETCODE": SimTypeInt(signed=False, label="LDAP_RETCODE"),
    "LDAP_TIMEVAL": SimStruct(OrderedDict((("tv_sec", SimTypeInt(signed=True, label="Int32")), ("tv_usec", SimTypeInt(signed=True, label="Int32")),)), name="LDAP_TIMEVAL", pack=False, align=None),
    "LDAP_VERSION_INFO": SimStruct(OrderedDict((("lv_size", SimTypeInt(signed=False, label="UInt32")), ("lv_major", SimTypeInt(signed=False, label="UInt32")), ("lv_minor", SimTypeInt(signed=False, label="UInt32")),)), name="LDAP_VERSION_INFO", pack=False, align=None),
    "LDR_DATA_TABLE_ENTRY": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("InMemoryOrderLinks", SimTypeRef("LIST_ENTRY", SimStruct)), ("Reserved2", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("DllBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved3", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("FullDllName", SimTypeRef("UNICODE_STRING", SimStruct)), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Reserved5", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 3)), ("Anonymous", SimUnion({"CheckSum": SimTypeInt(signed=False, label="UInt32"), "Reserved6": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")),)), name="LDR_DATA_TABLE_ENTRY", pack=False, align=None),
    "LDT_ENTRY": SimStruct(OrderedDict((("LimitLow", SimTypeShort(signed=False, label="UInt16")), ("BaseLow", SimTypeShort(signed=False, label="UInt16")), ("HighWord", SimUnion({"Bytes": SimTypeRef("_Bytes_e__Struct", SimStruct), "Bits": SimTypeRef("_Bits_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="LDT_ENTRY", pack=False, align=None),
    "LEGACY_BUS_INFORMATION": SimStruct(OrderedDict((("BusTypeGuid", SimTypeBottom(label="Guid")), ("LegacyBusType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")),)), name="LEGACY_BUS_INFORMATION", pack=False, align=None),
    "LEGACY_IDENTITY_UI_PARAMS": SimStruct(OrderedDict((("eapType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSizeofConnectionData", SimTypeInt(signed=False, label="UInt32")), ("pConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeofUserData", SimTypeInt(signed=False, label="UInt32")), ("pUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeofUserDataOut", SimTypeInt(signed=False, label="UInt32")), ("pUserDataOut", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")),)), name="LEGACY_IDENTITY_UI_PARAMS", pack=False, align=None),
    "LEGACY_INTERACTIVE_UI_PARAMS": SimStruct(OrderedDict((("eapType", SimTypeInt(signed=False, label="UInt32")), ("dwSizeofContextData", SimTypeInt(signed=False, label="UInt32")), ("pContextData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeofInteractiveUIData", SimTypeInt(signed=False, label="UInt32")), ("pInteractiveUIData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")),)), name="LEGACY_INTERACTIVE_UI_PARAMS", pack=False, align=None),
    "LHITTESTINFO": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("item", SimTypeRef("LITEM", SimStruct)),)), name="LHITTESTINFO", pack=False, align=None),
    "LIBFLAGS": SimTypeInt(signed=False, label="LIBFLAGS"),
    "LIBRARYFOLDERFILTER": SimTypeInt(signed=False, label="LIBRARYFOLDERFILTER"),
    "LIBRARYMANAGEDIALOGOPTIONS": SimTypeInt(signed=False, label="LIBRARYMANAGEDIALOGOPTIONS"),
    "LIBRARYOPTIONFLAGS": SimTypeInt(signed=False, label="LIBRARYOPTIONFLAGS"),
    "LIBRARYSAVEFLAGS": SimTypeInt(signed=False, label="LIBRARYSAVEFLAGS"),
    "LICINFO": SimStruct(OrderedDict((("cbLicInfo", SimTypeInt(signed=True, label="Int32")), ("fRuntimeKeyAvail", SimTypeInt(signed=True, label="Int32")), ("fLicVerified", SimTypeInt(signed=True, label="Int32")),)), name="LICINFO", pack=False, align=None),
    "LIFE_TIME": SimStruct(OrderedDict((("startTime", SimTypeRef("FILETIME", SimStruct)), ("endTime", SimTypeRef("FILETIME", SimStruct)),)), name="LIFE_TIME", pack=False, align=None),
    "LIGATURE": SimStruct(OrderedDict((("culSize", SimTypeInt(signed=False, label="UInt32")), ("pwsz", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("chglyph", SimTypeInt(signed=False, label="UInt32")), ("ahglyph", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="LIGATURE", pack=False, align=None),
    "LIGATURE1": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("ModificationNumber", SimTypeShort(signed=False, label="UInt16")), ("wch", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LIGATURE1", pack=False, align=None),
    "LIGATURE2": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("ModificationNumber", SimTypeShort(signed=False, label="UInt16")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 2)),)), name="LIGATURE2", pack=False, align=None),
    "LIGATURE3": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("ModificationNumber", SimTypeShort(signed=False, label="UInt16")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 3)),)), name="LIGATURE3", pack=False, align=None),
    "LIGATURE4": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("ModificationNumber", SimTypeShort(signed=False, label="UInt16")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 4)),)), name="LIGATURE4", pack=False, align=None),
    "LIGATURE5": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("ModificationNumber", SimTypeShort(signed=False, label="UInt16")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 5)),)), name="LIGATURE5", pack=False, align=None),
    "LINEADDRESSCAPS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwLineDeviceID", SimTypeInt(signed=False, label="UInt32")), ("dwAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAddressSharing", SimTypeInt(signed=False, label="UInt32")), ("dwAddressStates", SimTypeInt(signed=False, label="UInt32")), ("dwCallInfoStates", SimTypeInt(signed=False, label="UInt32")), ("dwCallerIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCalledIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectionIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectingIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCallStates", SimTypeInt(signed=False, label="UInt32")), ("dwDialToneModes", SimTypeInt(signed=False, label="UInt32")), ("dwBusyModes", SimTypeInt(signed=False, label="UInt32")), ("dwSpecialInfo", SimTypeInt(signed=False, label="UInt32")), ("dwDisconnectModes", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumActiveCalls", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumOnHoldCalls", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumOnHoldPendingCalls", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumConference", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumTransConf", SimTypeInt(signed=False, label="UInt32")), ("dwAddrCapFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCallFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwRemoveFromConfCaps", SimTypeInt(signed=False, label="UInt32")), ("dwRemoveFromConfState", SimTypeInt(signed=False, label="UInt32")), ("dwTransferModes", SimTypeInt(signed=False, label="UInt32")), ("dwParkModes", SimTypeInt(signed=False, label="UInt32")), ("dwForwardModes", SimTypeInt(signed=False, label="UInt32")), ("dwMaxForwardEntries", SimTypeInt(signed=False, label="UInt32")), ("dwMaxSpecificEntries", SimTypeInt(signed=False, label="UInt32")), ("dwMinFwdNumRings", SimTypeInt(signed=False, label="UInt32")), ("dwMaxFwdNumRings", SimTypeInt(signed=False, label="UInt32")), ("dwMaxCallCompletions", SimTypeInt(signed=False, label="UInt32")), ("dwCallCompletionConds", SimTypeInt(signed=False, label="UInt32")), ("dwCallCompletionModes", SimTypeInt(signed=False, label="UInt32")), ("dwNumCompletionMessages", SimTypeInt(signed=False, label="UInt32")), ("dwCompletionMsgTextEntrySize", SimTypeInt(signed=False, label="UInt32")), ("dwCompletionMsgTextSize", SimTypeInt(signed=False, label="UInt32")), ("dwCompletionMsgTextOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAddressFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwPredictiveAutoTransferStates", SimTypeInt(signed=False, label="UInt32")), ("dwNumCallTreatments", SimTypeInt(signed=False, label="UInt32")), ("dwCallTreatmentListSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallTreatmentListOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassesSize", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwMaxCallDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallFeatures2", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNoAnswerTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedModes", SimTypeInt(signed=False, label="UInt32")), ("dwOfferingModes", SimTypeInt(signed=False, label="UInt32")), ("dwAvailableMediaModes", SimTypeInt(signed=False, label="UInt32")),)), name="LINEADDRESSCAPS", pack=False, align=None),
    "LINEADDRESSSTATUS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumInUse", SimTypeInt(signed=False, label="UInt32")), ("dwNumActiveCalls", SimTypeInt(signed=False, label="UInt32")), ("dwNumOnHoldCalls", SimTypeInt(signed=False, label="UInt32")), ("dwNumOnHoldPendCalls", SimTypeInt(signed=False, label="UInt32")), ("dwAddressFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwNumRingsNoAnswer", SimTypeInt(signed=False, label="UInt32")), ("dwForwardNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwForwardSize", SimTypeInt(signed=False, label="UInt32")), ("dwForwardOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalModesSize", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalModesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEADDRESSSTATUS", pack=False, align=None),
    "LINEAGENTACTIVITYENTRY": SimStruct(OrderedDict((("dwID", SimTypeInt(signed=False, label="UInt32")), ("dwNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTACTIVITYENTRY", pack=False, align=None),
    "LINEAGENTACTIVITYLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwListSize", SimTypeInt(signed=False, label="UInt32")), ("dwListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTACTIVITYLIST", pack=False, align=None),
    "LINEAGENTCAPS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentHandlerInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentHandlerInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCapsVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwStates", SimTypeInt(signed=False, label="UInt32")), ("dwNextStates", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumGroupEntries", SimTypeInt(signed=False, label="UInt32")), ("dwAgentStatusMessages", SimTypeInt(signed=False, label="UInt32")), ("dwNumAgentExtensionIDs", SimTypeInt(signed=False, label="UInt32")), ("dwAgentExtensionIDListSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentExtensionIDListOffset", SimTypeInt(signed=False, label="UInt32")), ("ProxyGUID", SimTypeBottom(label="Guid")),)), name="LINEAGENTCAPS", pack=False, align=None),
    "LINEAGENTENTRY": SimStruct(OrderedDict((("hAgent", SimTypeInt(signed=False, label="UInt32")), ("dwNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPINSize", SimTypeInt(signed=False, label="UInt32")), ("dwPINOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTENTRY", pack=False, align=None),
    "LINEAGENTGROUPENTRY": SimStruct(OrderedDict((("GroupID", SimTypeRef("_GroupID_e__Struct", SimStruct)), ("dwNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTGROUPENTRY", pack=False, align=None),
    "LINEAGENTGROUPLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwListSize", SimTypeInt(signed=False, label="UInt32")), ("dwListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTGROUPLIST", pack=False, align=None),
    "LINEAGENTINFO": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentState", SimTypeInt(signed=False, label="UInt32")), ("dwNextAgentState", SimTypeInt(signed=False, label="UInt32")), ("dwMeasurementPeriod", SimTypeInt(signed=False, label="UInt32")), ("cyOverallCallRate", SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("dwNumberOfACDCalls", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfIncomingCalls", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfOutgoingCalls", SimTypeInt(signed=False, label="UInt32")), ("dwTotalACDTalkTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalACDCallTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalACDWrapUpTime", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTINFO", pack=False, align=None),
    "LINEAGENTLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwListSize", SimTypeInt(signed=False, label="UInt32")), ("dwListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTLIST", pack=False, align=None),
    "LINEAGENTSESSIONENTRY": SimStruct(OrderedDict((("hAgentSession", SimTypeInt(signed=False, label="UInt32")), ("hAgent", SimTypeInt(signed=False, label="UInt32")), ("GroupID", SimTypeBottom(label="Guid")), ("dwWorkingAddressID", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTSESSIONENTRY", pack=False, align=None),
    "LINEAGENTSESSIONINFO": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentSessionState", SimTypeInt(signed=False, label="UInt32")), ("dwNextAgentSessionState", SimTypeInt(signed=False, label="UInt32")), ("dateSessionStartTime", SimTypeFloat(size=64)), ("dwSessionDuration", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfCalls", SimTypeInt(signed=False, label="UInt32")), ("dwTotalTalkTime", SimTypeInt(signed=False, label="UInt32")), ("dwAverageTalkTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCallTime", SimTypeInt(signed=False, label="UInt32")), ("dwAverageCallTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalWrapUpTime", SimTypeInt(signed=False, label="UInt32")), ("dwAverageWrapUpTime", SimTypeInt(signed=False, label="UInt32")), ("cyACDCallRate", SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("dwLongestTimeToAnswer", SimTypeInt(signed=False, label="UInt32")), ("dwAverageTimeToAnswer", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTSESSIONINFO", pack=False, align=None),
    "LINEAGENTSESSIONLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwListSize", SimTypeInt(signed=False, label="UInt32")), ("dwListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTSESSIONLIST", pack=False, align=None),
    "LINEAGENTSTATUS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwGroupListSize", SimTypeInt(signed=False, label="UInt32")), ("dwGroupListOffset", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwNextState", SimTypeInt(signed=False, label="UInt32")), ("dwActivityID", SimTypeInt(signed=False, label="UInt32")), ("dwActivitySize", SimTypeInt(signed=False, label="UInt32")), ("dwActivityOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAgentFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwValidStates", SimTypeInt(signed=False, label="UInt32")), ("dwValidNextStates", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAGENTSTATUS", pack=False, align=None),
    "LINEAPPINFO": SimStruct(OrderedDict((("dwMachineNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwMachineNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwUserNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwUserNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwModuleFilenameSize", SimTypeInt(signed=False, label="UInt32")), ("dwModuleFilenameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwFriendlyNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwFriendlyNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwMediaModes", SimTypeInt(signed=False, label="UInt32")), ("dwAddressID", SimTypeInt(signed=False, label="UInt32")),)), name="LINEAPPINFO", pack=False, align=None),
    "LINEATTRS": SimStruct(OrderedDict((("fl", SimTypeInt(signed=False, label="UInt32")), ("iJoin", SimTypeInt(signed=False, label="UInt32")), ("iEndCap", SimTypeInt(signed=False, label="UInt32")), ("elWidth", SimUnion({"e": SimTypeInt(signed=False, label="UInt32"), "l": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("eMiterLimit", SimTypeInt(signed=False, label="UInt32")), ("cstyle", SimTypeInt(signed=False, label="UInt32")), ("pstyle", SimTypePointer(SimUnion({"e": SimTypeInt(signed=False, label="UInt32"), "l": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"), offset=0)), ("elStyleState", SimUnion({"e": SimTypeInt(signed=False, label="UInt32"), "l": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="LINEATTRS", pack=False, align=None),
    "LINECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hDevice", "dwMessage", "dwInstance", "dwParam1", "dwParam2", "dwParam3"]), offset=0),
    "LINECALLINFO": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("hLine", SimTypeInt(signed=False, label="UInt32")), ("dwLineDeviceID", SimTypeInt(signed=False, label="UInt32")), ("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("dwBearerMode", SimTypeInt(signed=False, label="UInt32")), ("dwRate", SimTypeInt(signed=False, label="UInt32")), ("dwMediaMode", SimTypeInt(signed=False, label="UInt32")), ("dwAppSpecific", SimTypeInt(signed=False, label="UInt32")), ("dwCallID", SimTypeInt(signed=False, label="UInt32")), ("dwRelatedCallID", SimTypeInt(signed=False, label="UInt32")), ("dwCallParamFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCallStates", SimTypeInt(signed=False, label="UInt32")), ("dwMonitorDigitModes", SimTypeInt(signed=False, label="UInt32")), ("dwMonitorMediaModes", SimTypeInt(signed=False, label="UInt32")), ("DialParams", SimTypeRef("LINEDIALPARAMS", SimStruct)), ("dwOrigin", SimTypeInt(signed=False, label="UInt32")), ("dwReason", SimTypeInt(signed=False, label="UInt32")), ("dwCompletionID", SimTypeInt(signed=False, label="UInt32")), ("dwNumOwners", SimTypeInt(signed=False, label="UInt32")), ("dwNumMonitors", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("dwTrunk", SimTypeInt(signed=False, label="UInt32")), ("dwCallerIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCallerIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallerIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCallerIDNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallerIDNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCalledIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCalledIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwCalledIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCalledIDNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwCalledIDNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedIDNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwConnectedIDNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectionIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectionIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectionIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectionIDNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectionIDNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectingIDFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectingIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectingIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectingIDNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwRedirectingIDNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAppNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwAppNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayableAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayableAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCalledPartySize", SimTypeInt(signed=False, label="UInt32")), ("dwCalledPartyOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCommentSize", SimTypeInt(signed=False, label="UInt32")), ("dwCommentOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDisplaySize", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayOffset", SimTypeInt(signed=False, label="UInt32")), ("dwUserUserInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwUserUserInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwHighLevelCompSize", SimTypeInt(signed=False, label="UInt32")), ("dwHighLevelCompOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLowLevelCompSize", SimTypeInt(signed=False, label="UInt32")), ("dwLowLevelCompOffset", SimTypeInt(signed=False, label="UInt32")), ("dwChargingInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwChargingInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalModesSize", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalModesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCallTreatment", SimTypeInt(signed=False, label="UInt32")), ("dwCallDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSendingFlowspecSize", SimTypeInt(signed=False, label="UInt32")), ("dwSendingFlowspecOffset", SimTypeInt(signed=False, label="UInt32")), ("dwReceivingFlowspecSize", SimTypeInt(signed=False, label="UInt32")), ("dwReceivingFlowspecOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINECALLINFO", pack=False, align=None),
    "LINECALLLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallsNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwCallsSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallsOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINECALLLIST", pack=False, align=None),
    "LINECALLPARAMS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwBearerMode", SimTypeInt(signed=False, label="UInt32")), ("dwMinRate", SimTypeInt(signed=False, label="UInt32")), ("dwMaxRate", SimTypeInt(signed=False, label="UInt32")), ("dwMediaMode", SimTypeInt(signed=False, label="UInt32")), ("dwCallParamFlags", SimTypeInt(signed=False, label="UInt32")), ("dwAddressMode", SimTypeInt(signed=False, label="UInt32")), ("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("DialParams", SimTypeRef("LINEDIALPARAMS", SimStruct)), ("dwOrigAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwOrigAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayableAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayableAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCalledPartySize", SimTypeInt(signed=False, label="UInt32")), ("dwCalledPartyOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCommentSize", SimTypeInt(signed=False, label="UInt32")), ("dwCommentOffset", SimTypeInt(signed=False, label="UInt32")), ("dwUserUserInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwUserUserInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwHighLevelCompSize", SimTypeInt(signed=False, label="UInt32")), ("dwHighLevelCompOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLowLevelCompSize", SimTypeInt(signed=False, label="UInt32")), ("dwLowLevelCompOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPredictiveAutoTransferStates", SimTypeInt(signed=False, label="UInt32")), ("dwTargetAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwTargetAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSendingFlowspecSize", SimTypeInt(signed=False, label="UInt32")), ("dwSendingFlowspecOffset", SimTypeInt(signed=False, label="UInt32")), ("dwReceivingFlowspecSize", SimTypeInt(signed=False, label="UInt32")), ("dwReceivingFlowspecOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassSize", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceConfigSize", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceConfigOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCallDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwNoAnswerTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwCallingPartyIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallingPartyIDOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINECALLPARAMS", pack=False, align=None),
    "LINECALLSTATUS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallState", SimTypeInt(signed=False, label="UInt32")), ("dwCallStateMode", SimTypeInt(signed=False, label="UInt32")), ("dwCallPrivilege", SimTypeInt(signed=False, label="UInt32")), ("dwCallFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCallFeatures2", SimTypeInt(signed=False, label="UInt32")), ("tStateEntryTime", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="LINECALLSTATUS", pack=False, align=None),
    "LINECALLTREATMENTENTRY": SimStruct(OrderedDict((("dwCallTreatmentID", SimTypeInt(signed=False, label="UInt32")), ("dwCallTreatmentNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallTreatmentNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINECALLTREATMENTENTRY", pack=False, align=None),
    "LINECARDENTRY": SimStruct(OrderedDict((("dwPermanentCardID", SimTypeInt(signed=False, label="UInt32")), ("dwCardNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwCardNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCardNumberDigits", SimTypeInt(signed=False, label="UInt32")), ("dwSameAreaRuleSize", SimTypeInt(signed=False, label="UInt32")), ("dwSameAreaRuleOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLongDistanceRuleSize", SimTypeInt(signed=False, label="UInt32")), ("dwLongDistanceRuleOffset", SimTypeInt(signed=False, label="UInt32")), ("dwInternationalRuleSize", SimTypeInt(signed=False, label="UInt32")), ("dwInternationalRuleOffset", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")),)), name="LINECARDENTRY", pack=False, align=None),
    "LINECOUNTRYENTRY": SimStruct(OrderedDict((("dwCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("dwNextCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwCountryNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwCountryNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSameAreaRuleSize", SimTypeInt(signed=False, label="UInt32")), ("dwSameAreaRuleOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLongDistanceRuleSize", SimTypeInt(signed=False, label="UInt32")), ("dwLongDistanceRuleOffset", SimTypeInt(signed=False, label="UInt32")), ("dwInternationalRuleSize", SimTypeInt(signed=False, label="UInt32")), ("dwInternationalRuleOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINECOUNTRYENTRY", pack=False, align=None),
    "LINECOUNTRYLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumCountries", SimTypeInt(signed=False, label="UInt32")), ("dwCountryListSize", SimTypeInt(signed=False, label="UInt32")), ("dwCountryListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINECOUNTRYLIST", pack=False, align=None),
    "LINEDDAPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LINEDEVCAPS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSwitchInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwSwitchInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPermanentLineID", SimTypeInt(signed=False, label="UInt32")), ("dwLineNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwLineNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwStringFormat", SimTypeInt(signed=False, label="UInt32")), ("dwAddressModes", SimTypeInt(signed=False, label="UInt32")), ("dwNumAddresses", SimTypeInt(signed=False, label="UInt32")), ("dwBearerModes", SimTypeInt(signed=False, label="UInt32")), ("dwMaxRate", SimTypeInt(signed=False, label="UInt32")), ("dwMediaModes", SimTypeInt(signed=False, label="UInt32")), ("dwGenerateToneModes", SimTypeInt(signed=False, label="UInt32")), ("dwGenerateToneMaxNumFreq", SimTypeInt(signed=False, label="UInt32")), ("dwGenerateDigitModes", SimTypeInt(signed=False, label="UInt32")), ("dwMonitorToneMaxNumFreq", SimTypeInt(signed=False, label="UInt32")), ("dwMonitorToneMaxNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwMonitorDigitModes", SimTypeInt(signed=False, label="UInt32")), ("dwGatherDigitsMinTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwGatherDigitsMaxTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwMedCtlDigitMaxListSize", SimTypeInt(signed=False, label="UInt32")), ("dwMedCtlMediaMaxListSize", SimTypeInt(signed=False, label="UInt32")), ("dwMedCtlToneMaxListSize", SimTypeInt(signed=False, label="UInt32")), ("dwMedCtlCallStateMaxListSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevCapFlags", SimTypeInt(signed=False, label="UInt32")), ("dwMaxNumActiveCalls", SimTypeInt(signed=False, label="UInt32")), ("dwAnswerMode", SimTypeInt(signed=False, label="UInt32")), ("dwRingModes", SimTypeInt(signed=False, label="UInt32")), ("dwLineStates", SimTypeInt(signed=False, label="UInt32")), ("dwUUIAcceptSize", SimTypeInt(signed=False, label="UInt32")), ("dwUUIAnswerSize", SimTypeInt(signed=False, label="UInt32")), ("dwUUIMakeCallSize", SimTypeInt(signed=False, label="UInt32")), ("dwUUIDropSize", SimTypeInt(signed=False, label="UInt32")), ("dwUUISendUserUserInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwUUICallInfoSize", SimTypeInt(signed=False, label="UInt32")), ("MinDialParams", SimTypeRef("LINEDIALPARAMS", SimStruct)), ("MaxDialParams", SimTypeRef("LINEDIALPARAMS", SimStruct)), ("DefaultDialParams", SimTypeRef("LINEDIALPARAMS", SimStruct)), ("dwNumTerminals", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalCapsSize", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalCapsOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalTextEntrySize", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalTextSize", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalTextOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLineFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwSettableDevStatus", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassesSize", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassesOffset", SimTypeInt(signed=False, label="UInt32")), ("PermanentLineGuid", SimTypeBottom(label="Guid")),)), name="LINEDEVCAPS", pack=False, align=None),
    "LINEDEVSTATUS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumOpens", SimTypeInt(signed=False, label="UInt32")), ("dwOpenMediaModes", SimTypeInt(signed=False, label="UInt32")), ("dwNumActiveCalls", SimTypeInt(signed=False, label="UInt32")), ("dwNumOnHoldCalls", SimTypeInt(signed=False, label="UInt32")), ("dwNumOnHoldPendCalls", SimTypeInt(signed=False, label="UInt32")), ("dwLineFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwNumCallCompletions", SimTypeInt(signed=False, label="UInt32")), ("dwRingMode", SimTypeInt(signed=False, label="UInt32")), ("dwSignalLevel", SimTypeInt(signed=False, label="UInt32")), ("dwBatteryLevel", SimTypeInt(signed=False, label="UInt32")), ("dwRoamMode", SimTypeInt(signed=False, label="UInt32")), ("dwDevStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalModesSize", SimTypeInt(signed=False, label="UInt32")), ("dwTerminalModesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAvailableMediaModes", SimTypeInt(signed=False, label="UInt32")), ("dwAppInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwAppInfoOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEDEVSTATUS", pack=False, align=None),
    "LINEDIALPARAMS": SimStruct(OrderedDict((("dwDialPause", SimTypeInt(signed=False, label="UInt32")), ("dwDialSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwDigitDuration", SimTypeInt(signed=False, label="UInt32")), ("dwWaitForDialtone", SimTypeInt(signed=False, label="UInt32")),)), name="LINEDIALPARAMS", pack=False, align=None),
    "LINEEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["htLine", "htCall", "dwMsg", "dwParam1", "dwParam2", "dwParam3"]), offset=0),
    "LINEEXTENSIONID": SimStruct(OrderedDict((("dwExtensionID0", SimTypeInt(signed=False, label="UInt32")), ("dwExtensionID1", SimTypeInt(signed=False, label="UInt32")), ("dwExtensionID2", SimTypeInt(signed=False, label="UInt32")), ("dwExtensionID3", SimTypeInt(signed=False, label="UInt32")),)), name="LINEEXTENSIONID", pack=False, align=None),
    "LINEFORWARD": SimStruct(OrderedDict((("dwForwardMode", SimTypeInt(signed=False, label="UInt32")), ("dwCallerAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallerAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDestCountryCode", SimTypeInt(signed=False, label="UInt32")), ("dwDestAddressSize", SimTypeInt(signed=False, label="UInt32")), ("dwDestAddressOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEFORWARD", pack=False, align=None),
    "LINEFORWARDLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("ForwardList", SimTypePointer(SimTypeRef("LINEFORWARD", SimStruct), offset=0)),)), name="LINEFORWARDLIST", pack=False, align=None),
    "LINEGENERATETONE": SimStruct(OrderedDict((("dwFrequency", SimTypeInt(signed=False, label="UInt32")), ("dwCadenceOn", SimTypeInt(signed=False, label="UInt32")), ("dwCadenceOff", SimTypeInt(signed=False, label="UInt32")), ("dwVolume", SimTypeInt(signed=False, label="UInt32")),)), name="LINEGENERATETONE", pack=False, align=None),
    "LINEINITIALIZEEXPARAMS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("Handles", SimUnion({"hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hCompletionPort": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("dwCompletionKey", SimTypeInt(signed=False, label="UInt32")),)), name="LINEINITIALIZEEXPARAMS", pack=False, align=None),
    "LINELOCATIONENTRY": SimStruct(OrderedDict((("dwPermanentLocationID", SimTypeInt(signed=False, label="UInt32")), ("dwLocationNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwLocationNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("dwCityCodeSize", SimTypeInt(signed=False, label="UInt32")), ("dwCityCodeOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredCardID", SimTypeInt(signed=False, label="UInt32")), ("dwLocalAccessCodeSize", SimTypeInt(signed=False, label="UInt32")), ("dwLocalAccessCodeOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLongDistanceAccessCodeSize", SimTypeInt(signed=False, label="UInt32")), ("dwLongDistanceAccessCodeOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTollPrefixListSize", SimTypeInt(signed=False, label="UInt32")), ("dwTollPrefixListOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwCancelCallWaitingSize", SimTypeInt(signed=False, label="UInt32")), ("dwCancelCallWaitingOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINELOCATIONENTRY", pack=False, align=None),
    "LINEMEDIACONTROLCALLSTATE": SimStruct(OrderedDict((("dwCallStates", SimTypeInt(signed=False, label="UInt32")), ("dwMediaControl", SimTypeInt(signed=False, label="UInt32")),)), name="LINEMEDIACONTROLCALLSTATE", pack=False, align=None),
    "LINEMEDIACONTROLDIGIT": SimStruct(OrderedDict((("dwDigit", SimTypeInt(signed=False, label="UInt32")), ("dwDigitModes", SimTypeInt(signed=False, label="UInt32")), ("dwMediaControl", SimTypeInt(signed=False, label="UInt32")),)), name="LINEMEDIACONTROLDIGIT", pack=False, align=None),
    "LINEMEDIACONTROLMEDIA": SimStruct(OrderedDict((("dwMediaModes", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")), ("dwMediaControl", SimTypeInt(signed=False, label="UInt32")),)), name="LINEMEDIACONTROLMEDIA", pack=False, align=None),
    "LINEMEDIACONTROLTONE": SimStruct(OrderedDict((("dwAppSpecific", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency1", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency2", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency3", SimTypeInt(signed=False, label="UInt32")), ("dwMediaControl", SimTypeInt(signed=False, label="UInt32")),)), name="LINEMEDIACONTROLTONE", pack=False, align=None),
    "LINEMESSAGE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("dwMessageID", SimTypeInt(signed=False, label="UInt32")), ("dwCallbackInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwParam1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwParam2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwParam3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="LINEMESSAGE", pack=False, align=None),
    "LINEMONITORTONE": SimStruct(OrderedDict((("dwAppSpecific", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency1", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency2", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency3", SimTypeInt(signed=False, label="UInt32")),)), name="LINEMONITORTONE", pack=False, align=None),
    "LINEPROVIDERENTRY": SimStruct(OrderedDict((("dwPermanentProviderID", SimTypeInt(signed=False, label="UInt32")), ("dwProviderFilenameSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderFilenameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEPROVIDERENTRY", pack=False, align=None),
    "LINEPROVIDERLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumProviders", SimTypeInt(signed=False, label="UInt32")), ("dwProviderListSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEPROVIDERLIST", pack=False, align=None),
    "LINEPROXYREQUEST": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwClientMachineNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwClientMachineNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwClientUserNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwClientUserNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwClientAppAPIVersion", SimTypeInt(signed=False, label="UInt32")), ("dwRequestType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"SetAgentGroup": SimTypeRef("_SetAgentGroup_e__Struct", SimStruct), "SetAgentState": SimTypeRef("_SetAgentState_e__Struct", SimStruct), "SetAgentActivity": SimTypeRef("_SetAgentActivity_e__Struct", SimStruct), "GetAgentCaps": SimTypeRef("_GetAgentCaps_e__Struct", SimStruct), "GetAgentStatus": SimTypeRef("_GetAgentStatus_e__Struct", SimStruct), "AgentSpecific": SimTypeRef("_AgentSpecific_e__Struct", SimStruct), "GetAgentActivityList": SimTypeRef("_GetAgentActivityList_e__Struct", SimStruct), "GetAgentGroupList": SimTypeRef("_GetAgentGroupList_e__Struct", SimStruct), "CreateAgent": SimTypeRef("_CreateAgent_e__Struct", SimStruct), "SetAgentStateEx": SimTypeRef("_SetAgentStateEx_e__Struct", SimStruct), "SetAgentMeasurementPeriod": SimTypeRef("_SetAgentMeasurementPeriod_e__Struct", SimStruct), "GetAgentInfo": SimTypeRef("_GetAgentInfo_e__Struct", SimStruct), "CreateAgentSession": SimTypeRef("_CreateAgentSession_e__Struct", SimStruct), "GetAgentSessionList": SimTypeRef("_GetAgentSessionList_e__Struct", SimStruct), "GetAgentSessionInfo": SimTypeRef("_GetAgentSessionInfo_e__Struct", SimStruct), "SetAgentSessionState": SimTypeRef("_SetAgentSessionState_e__Struct", SimStruct), "GetQueueList": SimTypeRef("_GetQueueList_e__Struct", SimStruct), "SetQueueMeasurementPeriod": SimTypeRef("_SetQueueMeasurementPeriod_e__Struct", SimStruct), "GetQueueInfo": SimTypeRef("_GetQueueInfo_e__Struct", SimStruct), "GetGroupList": SimTypeRef("_GetGroupList_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="LINEPROXYREQUEST", pack=False, align=None),
    "LINEPROXYREQUESTLIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwListSize", SimTypeInt(signed=False, label="UInt32")), ("dwListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEPROXYREQUESTLIST", pack=False, align=None),
    "LINEQUEUEENTRY": SimStruct(OrderedDict((("dwQueueID", SimTypeInt(signed=False, label="UInt32")), ("dwNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEQUEUEENTRY", pack=False, align=None),
    "LINEQUEUEINFO": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwMeasurementPeriod", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCallsQueued", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentCallsQueued", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCallsAbandoned", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCallsFlowedIn", SimTypeInt(signed=False, label="UInt32")), ("dwTotalCallsFlowedOut", SimTypeInt(signed=False, label="UInt32")), ("dwLongestEverWaitTime", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentLongestWaitTime", SimTypeInt(signed=False, label="UInt32")), ("dwAverageWaitTime", SimTypeInt(signed=False, label="UInt32")), ("dwFinalDisposition", SimTypeInt(signed=False, label="UInt32")),)), name="LINEQUEUEINFO", pack=False, align=None),
    "LINEQUEUELIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("dwListSize", SimTypeInt(signed=False, label="UInt32")), ("dwListOffset", SimTypeInt(signed=False, label="UInt32")),)), name="LINEQUEUELIST", pack=False, align=None),
    "LINEREQMAKECALL": SimStruct(OrderedDict((("szDestAddress", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("szAppName", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("szCalledParty", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("szComment", SimTypeArray(SimTypeChar(label="SByte"), 80)),)), name="LINEREQMAKECALL", pack=False, align=None),
    "LINEREQMAKECALLW": SimStruct(OrderedDict((("szDestAddress", SimTypeArray(SimTypeChar(label="Char"), 80)), ("szAppName", SimTypeArray(SimTypeChar(label="Char"), 40)), ("szCalledParty", SimTypeArray(SimTypeChar(label="Char"), 40)), ("szComment", SimTypeArray(SimTypeChar(label="Char"), 80)),)), name="LINEREQMAKECALLW", pack=False, align=None),
    "LINEREQMEDIACALL": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wRequestID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szDeviceClass", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("ucDeviceID", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSecure", SimTypeInt(signed=False, label="UInt32")), ("szDestAddress", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("szAppName", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("szCalledParty", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("szComment", SimTypeArray(SimTypeChar(label="SByte"), 80)),)), name="LINEREQMEDIACALL", pack=False, align=None),
    "LINEREQMEDIACALLW": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wRequestID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szDeviceClass", SimTypeArray(SimTypeChar(label="Char"), 40)), ("ucDeviceID", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSecure", SimTypeInt(signed=False, label="UInt32")), ("szDestAddress", SimTypeArray(SimTypeChar(label="Char"), 80)), ("szAppName", SimTypeArray(SimTypeChar(label="Char"), 40)), ("szCalledParty", SimTypeArray(SimTypeChar(label="Char"), 40)), ("szComment", SimTypeArray(SimTypeChar(label="Char"), 80)),)), name="LINEREQMEDIACALLW", pack=False, align=None),
    "LINETERMCAPS": SimStruct(OrderedDict((("dwTermDev", SimTypeInt(signed=False, label="UInt32")), ("dwTermModes", SimTypeInt(signed=False, label="UInt32")), ("dwTermSharing", SimTypeInt(signed=False, label="UInt32")),)), name="LINETERMCAPS", pack=False, align=None),
    "LINETRANSLATECAPS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumLocations", SimTypeInt(signed=False, label="UInt32")), ("dwLocationListSize", SimTypeInt(signed=False, label="UInt32")), ("dwLocationListOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentLocationID", SimTypeInt(signed=False, label="UInt32")), ("dwNumCards", SimTypeInt(signed=False, label="UInt32")), ("dwCardListSize", SimTypeInt(signed=False, label="UInt32")), ("dwCardListOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentPreferredCardID", SimTypeInt(signed=False, label="UInt32")),)), name="LINETRANSLATECAPS", pack=False, align=None),
    "LINETRANSLATEOUTPUT": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwDialableStringSize", SimTypeInt(signed=False, label="UInt32")), ("dwDialableStringOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayableStringSize", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayableStringOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentCountry", SimTypeInt(signed=False, label="UInt32")), ("dwDestCountry", SimTypeInt(signed=False, label="UInt32")), ("dwTranslateResults", SimTypeInt(signed=False, label="UInt32")),)), name="LINETRANSLATEOUTPUT", pack=False, align=None),
    "LINE_DIRECTION": SimTypeInt(signed=False, label="LINE_DIRECTION"),
    "LINE_METRICS": SimTypeInt(signed=False, label="LINE_METRICS"),
    "LINE_SEGMENT": SimStruct(OrderedDict((("PtA", SimTypeRef("POINT", SimStruct)), ("PtB", SimTypeRef("POINT", SimStruct)),)), name="LINE_SEGMENT", pack=False, align=None),
    "LINGER": SimStruct(OrderedDict((("l_onoff", SimTypeShort(signed=False, label="UInt16")), ("l_linger", SimTypeShort(signed=False, label="UInt16")),)), name="LINGER", pack=False, align=None),
    "LINKHEADERSTATES": SimTypeInt(signed=False, label="LINKHEADERSTATES"),
    "LINKPARTS": SimTypeInt(signed=False, label="LINKPARTS"),
    "LINKSTATES": SimTypeInt(signed=False, label="LINKSTATES"),
    "LINK_SHARE_ACCESS": SimStruct(OrderedDict((("OpenCount", SimTypeInt(signed=False, label="UInt32")), ("Deleters", SimTypeInt(signed=False, label="UInt32")), ("SharedDelete", SimTypeInt(signed=False, label="UInt32")),)), name="LINK_SHARE_ACCESS", pack=False, align=None),
    "LINK_TRACKING_INFORMATION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="LINK_TRACKING_INFORMATION_TYPE")), ("VolumeId", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="LINK_TRACKING_INFORMATION", pack=False, align=None),
    "LINK_TRACKING_INFORMATION_TYPE": SimTypeInt(signed=False, label="LINK_TRACKING_INFORMATION_TYPE"),
    "LISTBOXPARTS": SimTypeInt(signed=False, label="LISTBOXPARTS"),
    "LISTITEMSTATES": SimTypeInt(signed=False, label="LISTITEMSTATES"),
    "LISTVIEWPARTS": SimTypeInt(signed=False, label="LISTVIEWPARTS"),
    "LIST_ENTRY": SimStruct(OrderedDict((("Flink", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("Blink", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)),)), name="LIST_ENTRY", pack=False, align=None),
    "LIST_ENTRY32": SimStruct(OrderedDict((("Flink", SimTypeInt(signed=False, label="UInt32")), ("Blink", SimTypeInt(signed=False, label="UInt32")),)), name="LIST_ENTRY32", pack=False, align=None),
    "LIST_ENTRY64": SimStruct(OrderedDict((("Flink", SimTypeLongLong(signed=False, label="UInt64")), ("Blink", SimTypeLongLong(signed=False, label="UInt64")),)), name="LIST_ENTRY64", pack=False, align=None),
    "LIST_ITEM_FLAGS": SimTypeInt(signed=False, label="LIST_ITEM_FLAGS"),
    "LIST_ITEM_STATE_FLAGS": SimTypeInt(signed=False, label="LIST_ITEM_STATE_FLAGS"),
    "LIST_VIEW_BACKGROUND_IMAGE_FLAGS": SimTypeInt(signed=False, label="LIST_VIEW_BACKGROUND_IMAGE_FLAGS"),
    "LIST_VIEW_GROUP_ALIGN_FLAGS": SimTypeInt(signed=False, label="LIST_VIEW_GROUP_ALIGN_FLAGS"),
    "LIST_VIEW_GROUP_STATE_FLAGS": SimTypeInt(signed=False, label="LIST_VIEW_GROUP_STATE_FLAGS"),
    "LIST_VIEW_ITEM_COLUMN_FORMAT_FLAGS": SimTypeInt(signed=False, label="LIST_VIEW_ITEM_COLUMN_FORMAT_FLAGS"),
    "LIST_VIEW_ITEM_FLAGS": SimTypeInt(signed=False, label="LIST_VIEW_ITEM_FLAGS"),
    "LIST_VIEW_ITEM_STATE_FLAGS": SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS"),
    "LITEM": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="LIST_ITEM_FLAGS")), ("iLink", SimTypeInt(signed=True, label="Int32")), ("state", SimTypeInt(signed=False, label="LIST_ITEM_STATE_FLAGS")), ("stateMask", SimTypeInt(signed=False, label="LIST_ITEM_STATE_FLAGS")), ("szID", SimTypeArray(SimTypeChar(label="Char"), 48)), ("szUrl", SimTypeArray(SimTypeChar(label="Char"), 2084)),)), name="LITEM", pack=False, align=None),
    "LLFILETIME": SimStruct(OrderedDict((("Anonymous", SimUnion({"ll": SimTypeLongLong(signed=True, label="Int64"), "ft": SimTypeRef("FILETIME", SimStruct)}, name="<anon>", label="None")),)), name="LLFILETIME", pack=False, align=None),
    "LMR_QUERY_INFO_CLASS": SimTypeInt(signed=False, label="LMR_QUERY_INFO_CLASS"),
    "LMR_QUERY_INFO_PARAM": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="LMR_QUERY_INFO_CLASS")),)), name="LMR_QUERY_INFO_PARAM", pack=False, align=None),
    "LMR_QUERY_SESSION_INFO": SimStruct(OrderedDict((("SessionId", SimTypeLongLong(signed=False, label="UInt64")),)), name="LMR_QUERY_SESSION_INFO", pack=False, align=None),
    "LM_IRPARMS": SimStruct(OrderedDict((("nTXDataBytes", SimTypeInt(signed=False, label="UInt32")), ("nRXDataBytes", SimTypeInt(signed=False, label="UInt32")), ("nBaudRate", SimTypeInt(signed=False, label="UInt32")), ("thresholdTime", SimTypeInt(signed=False, label="UInt32")), ("discTime", SimTypeInt(signed=False, label="UInt32")), ("nMSLinkTurn", SimTypeShort(signed=False, label="UInt16")), ("nTXPackets", SimTypeChar(label="Byte")), ("nRXPackets", SimTypeChar(label="Byte")),)), name="LM_IRPARMS", pack=False, align=None),
    "LM_OWF_PASSWORD": SimStruct(OrderedDict((("data", SimTypeArray(SimTypeRef("CYPHER_BLOCK", SimStruct), 2)),)), name="LM_OWF_PASSWORD", pack=False, align=None),
    "LNB_Source": SimTypeInt(signed=False, label="LNB_Source"),
    "LOADED_IMAGE": SimStruct(OrderedDict((("ModuleName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("MappedAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FileHeader", SimTypePointer(SimTypeRef("IMAGE_NT_HEADERS32", SimStruct), offset=0)), ("LastRvaSection", SimTypePointer(SimTypeRef("IMAGE_SECTION_HEADER", SimStruct), offset=0)), ("NumberOfSections", SimTypeInt(signed=False, label="UInt32")), ("Sections", SimTypePointer(SimTypeRef("IMAGE_SECTION_HEADER", SimStruct), offset=0)), ("Characteristics", SimTypeInt(signed=False, label="IMAGE_FILE_CHARACTERISTICS2")), ("fSystemImage", SimTypeChar(label="Byte")), ("fDOSImage", SimTypeChar(label="Byte")), ("fReadOnly", SimTypeChar(label="Byte")), ("Version", SimTypeChar(label="Byte")), ("Links", SimTypeRef("LIST_ENTRY", SimStruct)), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")),)), name="LOADED_IMAGE", pack=False, align=None),
    "LOADER_PARAMETER_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LOADER_PARTITION_INFORMATION_EX": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="UInt32")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Signature": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="LOADER_PARTITION_INFORMATION_EX", pack=False, align=None),
    "LOAD_DLL_DEBUG_INFO": SimStruct(OrderedDict((("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpBaseOfDll", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDebugInfoFileOffset", SimTypeInt(signed=False, label="UInt32")), ("nDebugInfoSize", SimTypeInt(signed=False, label="UInt32")), ("lpImageName", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("fUnicode", SimTypeShort(signed=False, label="UInt16")),)), name="LOAD_DLL_DEBUG_INFO", pack=False, align=None),
    "LOAD_LIBRARY_FLAGS": SimTypeInt(signed=False, label="LOAD_LIBRARY_FLAGS"),
    "LOAD_PICTURE_FLAGS": SimTypeInt(signed=False, label="LOAD_PICTURE_FLAGS"),
    "LOCALESIGNATURE": SimStruct(OrderedDict((("lsUsb", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("lsCsbDefault", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lsCsbSupported", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="LOCALESIGNATURE", pack=False, align=None),
    "LOCALE_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "LOCALE_ENUMPROCEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LOCALE_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "LOCALGROUP_INFO_0": SimStruct(OrderedDict((("lgrpi0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_INFO_0", pack=False, align=None),
    "LOCALGROUP_INFO_1": SimStruct(OrderedDict((("lgrpi1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lgrpi1_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_INFO_1", pack=False, align=None),
    "LOCALGROUP_INFO_1002": SimStruct(OrderedDict((("lgrpi1002_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_INFO_1002", pack=False, align=None),
    "LOCALGROUP_MEMBERS_INFO_0": SimStruct(OrderedDict((("lgrmi0_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="LOCALGROUP_MEMBERS_INFO_0", pack=False, align=None),
    "LOCALGROUP_MEMBERS_INFO_1": SimStruct(OrderedDict((("lgrmi1_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lgrmi1_sidusage", SimTypeInt(signed=False, label="SID_NAME_USE")), ("lgrmi1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_MEMBERS_INFO_1", pack=False, align=None),
    "LOCALGROUP_MEMBERS_INFO_2": SimStruct(OrderedDict((("lgrmi2_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lgrmi2_sidusage", SimTypeInt(signed=False, label="SID_NAME_USE")), ("lgrmi2_domainandname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_MEMBERS_INFO_2", pack=False, align=None),
    "LOCALGROUP_MEMBERS_INFO_3": SimStruct(OrderedDict((("lgrmi3_domainandname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_MEMBERS_INFO_3", pack=False, align=None),
    "LOCALGROUP_USERS_INFO_0": SimStruct(OrderedDict((("lgrui0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOCALGROUP_USERS_INFO_0", pack=False, align=None),
    "LOCALMANAGEDAPPLICATION": SimStruct(OrderedDict((("pszDeploymentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszProductId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwState", SimTypeInt(signed=False, label="UInt32")),)), name="LOCALMANAGEDAPPLICATION", pack=False, align=None),
    "LOCAL_ALLOC_FLAGS": SimTypeInt(signed=False, label="LOCAL_ALLOC_FLAGS"),
    "LOCATION_DESIRED_ACCURACY": SimTypeInt(signed=False, label="LOCATION_DESIRED_ACCURACY"),
    "LOCATION_POSITION_SOURCE": SimTypeInt(signed=False, label="LOCATION_POSITION_SOURCE"),
    "LOCATION_REPORT_STATUS": SimTypeInt(signed=False, label="LOCATION_REPORT_STATUS"),
    "LOCKMODEENUM": SimTypeInt(signed=False, label="LOCKMODEENUM"),
    "LOCKTYPE": SimTypeInt(signed=False, label="LOCKTYPE"),
    "LOCK_FILE_FLAGS": SimTypeInt(signed=False, label="LOCK_FILE_FLAGS"),
    "LOCK_OPERATION": SimTypeInt(signed=False, label="LOCK_OPERATION"),
    "LOCK_STATE": SimStruct(OrderedDict((("LockState", SimTypeShort(signed=False, label="UInt16")), ("OldIrql", SimTypeChar(label="Byte")),)), name="LOCK_STATE", pack=False, align=None),
    "LOGBRUSH": SimStruct(OrderedDict((("lbStyle", SimTypeInt(signed=False, label="BRUSH_STYLE")), ("lbColor", SimTypeInt(signed=False, label="UInt32")), ("lbHatch", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="LOGBRUSH", pack=False, align=None),
    "LOGBRUSH32": SimStruct(OrderedDict((("lbStyle", SimTypeInt(signed=False, label="BRUSH_STYLE")), ("lbColor", SimTypeInt(signed=False, label="UInt32")), ("lbHatch", SimTypeInt(signed=False, label="UInt32")),)), name="LOGBRUSH32", pack=False, align=None),
    "LOGCOLORSPACEA": SimStruct(OrderedDict((("lcsSignature", SimTypeInt(signed=False, label="UInt32")), ("lcsVersion", SimTypeInt(signed=False, label="UInt32")), ("lcsSize", SimTypeInt(signed=False, label="UInt32")), ("lcsCSType", SimTypeInt(signed=False, label="LCSCSTYPE")), ("lcsIntent", SimTypeInt(signed=True, label="Int32")), ("lcsEndpoints", SimTypeRef("CIEXYZTRIPLE", SimStruct)), ("lcsGammaRed", SimTypeInt(signed=False, label="UInt32")), ("lcsGammaGreen", SimTypeInt(signed=False, label="UInt32")), ("lcsGammaBlue", SimTypeInt(signed=False, label="UInt32")), ("lcsFilename", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="LOGCOLORSPACEA", pack=False, align=None),
    "LOGCOLORSPACEW": SimStruct(OrderedDict((("lcsSignature", SimTypeInt(signed=False, label="UInt32")), ("lcsVersion", SimTypeInt(signed=False, label="UInt32")), ("lcsSize", SimTypeInt(signed=False, label="UInt32")), ("lcsCSType", SimTypeInt(signed=False, label="LCSCSTYPE")), ("lcsIntent", SimTypeInt(signed=True, label="Int32")), ("lcsEndpoints", SimTypeRef("CIEXYZTRIPLE", SimStruct)), ("lcsGammaRed", SimTypeInt(signed=False, label="UInt32")), ("lcsGammaGreen", SimTypeInt(signed=False, label="UInt32")), ("lcsGammaBlue", SimTypeInt(signed=False, label="UInt32")), ("lcsFilename", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="LOGCOLORSPACEW", pack=False, align=None),
    "LOGFONTA": SimStruct(OrderedDict((("lfHeight", SimTypeInt(signed=True, label="Int32")), ("lfWidth", SimTypeInt(signed=True, label="Int32")), ("lfEscapement", SimTypeInt(signed=True, label="Int32")), ("lfOrientation", SimTypeInt(signed=True, label="Int32")), ("lfWeight", SimTypeInt(signed=True, label="Int32")), ("lfItalic", SimTypeChar(label="Byte")), ("lfUnderline", SimTypeChar(label="Byte")), ("lfStrikeOut", SimTypeChar(label="Byte")), ("lfCharSet", SimTypeInt(signed=False, label="FONT_CHARSET")), ("lfOutPrecision", SimTypeInt(signed=False, label="FONT_OUTPUT_PRECISION")), ("lfClipPrecision", SimTypeInt(signed=False, label="FONT_CLIP_PRECISION")), ("lfQuality", SimTypeInt(signed=False, label="FONT_QUALITY")), ("lfPitchAndFamily", SimTypeChar(label="Byte")), ("lfFaceName", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="LOGFONTA", pack=False, align=None),
    "LOGFONTW": SimStruct(OrderedDict((("lfHeight", SimTypeInt(signed=True, label="Int32")), ("lfWidth", SimTypeInt(signed=True, label="Int32")), ("lfEscapement", SimTypeInt(signed=True, label="Int32")), ("lfOrientation", SimTypeInt(signed=True, label="Int32")), ("lfWeight", SimTypeInt(signed=True, label="Int32")), ("lfItalic", SimTypeChar(label="Byte")), ("lfUnderline", SimTypeChar(label="Byte")), ("lfStrikeOut", SimTypeChar(label="Byte")), ("lfCharSet", SimTypeInt(signed=False, label="FONT_CHARSET")), ("lfOutPrecision", SimTypeInt(signed=False, label="FONT_OUTPUT_PRECISION")), ("lfClipPrecision", SimTypeInt(signed=False, label="FONT_CLIP_PRECISION")), ("lfQuality", SimTypeInt(signed=False, label="FONT_QUALITY")), ("lfPitchAndFamily", SimTypeChar(label="Byte")), ("lfFaceName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="LOGFONTW", pack=False, align=None),
    "LOGGING_PARAMETERS": SimStruct(OrderedDict((("pszSessionId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRemoteIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRemoteIpPort", SimTypeInt(signed=False, label="UInt32")), ("pszLocalIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLocalIpPort", SimTypeInt(signed=False, label="UInt32")), ("BytesSent", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("pszCommand", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCommandParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszFullPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwElapsedMilliseconds", SimTypeInt(signed=False, label="UInt32")), ("FtpStatus", SimTypeInt(signed=False, label="UInt32")), ("FtpSubStatus", SimTypeInt(signed=False, label="UInt32")), ("hrStatus", SimTypeInt(signed=True, label="Int32")), ("pszInformation", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOGGING_PARAMETERS", pack=False, align=None),
    "LOGICAL_PROCESSOR_RELATIONSHIP": SimTypeInt(signed=False, label="LOGICAL_PROCESSOR_RELATIONSHIP"),
    "LOGOFFBUTTONSSTATES": SimTypeInt(signed=False, label="LOGOFFBUTTONSSTATES"),
    "LOGON32_LOGON": SimTypeInt(signed=False, label="LOGON32_LOGON"),
    "LOGON32_PROVIDER": SimTypeInt(signed=False, label="LOGON32_PROVIDER"),
    "LOGON_HOURS": SimStruct(OrderedDict((("UnitsPerWeek", SimTypeShort(signed=False, label="UInt16")), ("LogonHours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="LOGON_HOURS", pack=False, align=None),
    "LOGPALETTE": SimStruct(OrderedDict((("palVersion", SimTypeShort(signed=False, label="UInt16")), ("palNumEntries", SimTypeShort(signed=False, label="UInt16")), ("palPalEntry", SimTypePointer(SimTypeRef("PALETTEENTRY", SimStruct), offset=0)),)), name="LOGPALETTE", pack=False, align=None),
    "LOGPEN": SimStruct(OrderedDict((("lopnStyle", SimTypeInt(signed=False, label="UInt32")), ("lopnWidth", SimTypeRef("POINT", SimStruct)), ("lopnColor", SimTypeInt(signed=False, label="UInt32")),)), name="LOGPEN", pack=False, align=None),
    "LOG_LEVEL": SimTypeInt(signed=False, label="LOG_LEVEL"),
    "LOG_MANAGEMENT_CALLBACKS": SimStruct(OrderedDict((("CallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AdvanceTailCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeRef("CLS_LSN", SimStruct), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hLogFile", "lsnTarget", "pvClientContext"]), offset=0)), ("LogFullHandlerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hLogFile", "dwError", "fLogIsPinned", "pvClientContext"]), offset=0)), ("LogUnpinnedCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hLogFile", "pvClientContext"]), offset=0)),)), name="LOG_MANAGEMENT_CALLBACKS", pack=False, align=None),
    "LONG_SECTION": SimStruct(OrderedDict((("TableId", SimTypeChar(label="Byte")), ("Header", SimUnion({"S": SimTypeRef("MPEG_HEADER_BITS_MIDL", SimStruct), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("TableIdExtension", SimTypeShort(signed=False, label="UInt16")), ("Version", SimUnion({"S": SimTypeRef("MPEG_HEADER_VERSION_BITS_MIDL", SimStruct), "B": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("SectionNumber", SimTypeChar(label="Byte")), ("LastSectionNumber", SimTypeChar(label="Byte")), ("RemainingData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="LONG_SECTION", pack=False, align=None),
    "LOOKUP_STREAM_FROM_CLUSTER_ENTRY": SimStruct(OrderedDict((("OffsetToNext", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=True, label="Int64")), ("Cluster", SimTypeLongLong(signed=True, label="Int64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LOOKUP_STREAM_FROM_CLUSTER_ENTRY", pack=False, align=None),
    "LOOKUP_STREAM_FROM_CLUSTER_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumberOfClusters", SimTypeInt(signed=False, label="UInt32")), ("Cluster", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="LOOKUP_STREAM_FROM_CLUSTER_INPUT", pack=False, align=None),
    "LOOKUP_STREAM_FROM_CLUSTER_OUTPUT": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("NumberOfMatches", SimTypeInt(signed=False, label="UInt32")), ("BufferSizeRequired", SimTypeInt(signed=False, label="UInt32")),)), name="LOOKUP_STREAM_FROM_CLUSTER_OUTPUT", pack=False, align=None),
    "LOOPEDSTREAMING_POSITION_EVENT_DATA": SimStruct(OrderedDict((("KsEventData", SimTypeRef("KSEVENTDATA", SimStruct)), ("Position", SimTypeLongLong(signed=False, label="UInt64")),)), name="LOOPEDSTREAMING_POSITION_EVENT_DATA", pack=False, align=None),
    "LPACMDRIVERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LPALLOCATEBUFFER": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbSize", "lppBuffer"]), offset=0),
    "LPALLOCATEMORE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbSize", "lpObject", "lppBuffer"]), offset=0),
    "LPARAM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LPBLOCKINGCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwContext"]), offset=0),
    "LPBMCALLBACKFN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPCALL_BACK_USER_INTERRUPT_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "LPCCHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPCFHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPCLIPPERCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirectDrawClipper"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpDDClipper", "hWnd", "code", "lpContext"]), offset=0),
    "LPCONDITIONPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpCallerId", "lpCallerData", "lpSQOS", "lpGQOS", "lpCalleeId", "lpCalleeData", "g", "dwCallbackData"]), offset=0),
    "LPCQADDFORMSPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CQFORM", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lParam", "pForm"]), offset=0),
    "LPCQADDPAGESPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("CQPAGE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lParam", "clsidForm", "pPage"]), offset=0),
    "LPCQPAGEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CQPAGE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPage", "hwnd", "uMsg", "wParam", "lParam"]), offset=0),
    "LPCREATECONVERSATIONINDEX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbParent", "lpbParent", "lpcbConvIndex", "lppbConvIndex"]), offset=0),
    "LPD3DENUMDEVICESCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("D3DDEVICEDESC", SimStruct), offset=0), SimTypePointer(SimTypeRef("D3DDEVICEDESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpGuid", "lpDeviceDescription", "lpDeviceName", "param3", "param4", "param5"]), offset=0),
    "LPD3DENUMDEVICESCALLBACK7": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("D3DDEVICEDESC7", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpDeviceDescription", "lpDeviceName", "param2", "param3"]), offset=0),
    "LPD3DENUMPIXELFORMATSCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpDDPixFmt", "lpContext"]), offset=0),
    "LPD3DENUMTEXTUREFORMATSCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpDdsd", "lpContext"]), offset=0),
    "LPD3DHAL_CLEAR2CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CLEAR2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_CLEARCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CLEARDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_CONTEXTCREATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CONTEXTCREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_CONTEXTDESTROYALLCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CONTEXTDESTROYALLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_CONTEXTDESTROYCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_CONTEXTDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_DRAWONEINDEXEDPRIMITIVECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWONEINDEXEDPRIMITIVEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_DRAWONEPRIMITIVECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWONEPRIMITIVEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_DRAWPRIMITIVES2CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWPRIMITIVES2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_DRAWPRIMITIVESCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_DRAWPRIMITIVESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_GETSTATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_GETSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_RENDERPRIMITIVECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_RENDERPRIMITIVEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_RENDERSTATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_RENDERSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_SCENECAPTURECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_SCENECAPTUREDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_SETRENDERTARGETCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_SETRENDERTARGETDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_TEXTURECREATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTURECREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_TEXTUREDESTROYCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTUREDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_TEXTUREGETSURFCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTUREGETSURFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_TEXTURESWAPCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_TEXTURESWAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DHAL_VALIDATETEXTURESTAGESTATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DHAL_VALIDATETEXTURESTAGESTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_CLEAR2CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CLEAR2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_CONTEXTCREATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CONTEXTCREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_CONTEXTDESTROYALLCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CONTEXTDESTROYALLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_CONTEXTDESTROYCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_CONTEXTDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_DRAWPRIMITIVES2CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_DRAWPRIMITIVES2DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_SCENECAPTURECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_SCENECAPTUREDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_SETRENDERTARGETCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_SETRENDERTARGETDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_TEXTURECREATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTURECREATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_TEXTUREDESTROYCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTUREDESTROYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_TEXTUREGETSURFCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTUREGETSURFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_TEXTURESWAPCB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_TEXTURESWAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DNTHAL_VALIDATETEXTURESTAGESTATECB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DNTHAL_VALIDATETEXTURESTAGESTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPD3DVALIDATECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpUserArg", "dwOffset"]), offset=0),
    "LPDD32BITDRIVERINIT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwContext"]), offset=0),
    "LPDDENUMCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPDDENUMCALLBACKEXA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LPDDENUMCALLBACKEXW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LPDDENUMCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPDDENUMMODESCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDDENUMMODESCALLBACK2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDSURFACEDESC2", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDDENUMSURFACESCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirectDrawSurface"), SimTypePointer(SimTypeRef("DDSURFACEDESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPDDENUMSURFACESCALLBACK2": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirectDrawSurface4"), SimTypePointer(SimTypeRef("DDSURFACEDESC2", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPDDENUMSURFACESCALLBACK7": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirectDrawSurface7"), SimTypePointer(SimTypeRef("DDSURFACEDESC2", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPDDENUMVIDEOCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDVIDEOPORTCAPS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDDFXROP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LPDDGAMMACALIBRATORPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDGAMMARAMP", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDDHALCOLORCB_COLORCONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_COLORCONTROLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALEXEBUFCB_CANCREATEEXEBUF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALEXEBUFCB_CREATEEXEBUF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALEXEBUFCB_DESTROYEXEBUF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALEXEBUFCB_LOCKEXEBUF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALEXEBUFCB_UNLOCKEXEBUF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALKERNELCB_SYNCSURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SYNCSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALKERNELCB_SYNCVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SYNCVIDEOPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_BEGINFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_BEGINMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_CREATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_DESTROY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_ENDFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_ENDMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_GETCOMPBUFFINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPCOMPBUFFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_GETFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPFORMATSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_GETGUIDS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETMOCOMPGUIDSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_GETINTERNALINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETINTERNALMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_QUERYSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_QUERYMOCOMPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALMOCOMPCB_RENDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_RENDERMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALPALCB_DESTROYPALETTE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALPALCB_SETENTRIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETENTRIESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_ADDATTACHEDSURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_ADDATTACHEDSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_BLT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_DESTROYSURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_FLIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_GETBLTSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETBLTSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_GETFLIPSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_SETCLIPLIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETCLIPLISTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_SETCOLORKEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_SETOVERLAYPOSITION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETOVERLAYPOSITIONDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_SETPALETTE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_UNLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALSURFCB_UPDATEOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATEOVERLAYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_CANCREATEVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_COLORCONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_VPORTCOLORDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_CREATEVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_DESTROYVPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_FLIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETBANDWIDTH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTBANDWIDTHDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETFIELD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTFIELDDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETFLIPSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETINPUTFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTINPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETLINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETOUTPUTFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTOUTPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETSIGNALSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTSIGNALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_GETVPORTCONNECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETVPORTCONNECTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_UPDATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHALVPORTCB_WAITFORSYNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_WAITFORVPORTSYNCDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_CANCREATESURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_CREATEPALETTE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATEPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_CREATESURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_CREATESURFACEEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_CREATESURFACEEXDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_DESTROYDDLOCAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYDDLOCALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_DESTROYDRIVER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DESTROYDRIVERDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_FLIPTOGDISURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_FLIPTOGDISURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_GETAVAILDRIVERMEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETAVAILDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_GETDRIVERINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETDRIVERINFODATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_GETDRIVERSTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETDRIVERSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_GETHEAPALIGNMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETHEAPALIGNMENTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_GETSCANLINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_GETSCANLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_SETCOLORKEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_DRVSETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_SETEXCLUSIVEMODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETEXCLUSIVEMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_SETINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHALINFO", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpDDHalInfo", "reset"]), offset=0),
    "LPDDHAL_SETMODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_SETMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_UPDATENONLOCALHEAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_UPDATENONLOCALHEAPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHAL_VIDMEMALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["lpDD", "heap", "dwWidth", "dwHeight"]), offset=0),
    "LPDDHAL_VIDMEMFREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["lpDD", "heap", "fpMem"]), offset=0),
    "LPDDHAL_WAITFORVERTICALBLANK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDHAL_WAITFORVERTICALBLANKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "LPDDHEL_INIT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DDRAWI_DIRECTDRAW_GBL", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDHCP_CONTROL": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwControlCode", "lpReserved"]), offset=0),
    "LPDHCP_DELETE_CLIENT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["IpAddress", "HwAddress", "HwAddressLength", "Reserved", "ClientType"]), offset=0),
    "LPDHCP_DROP_SEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "Reserved", "PktContext"]), offset=0),
    "LPDHCP_ENTRY_POINT_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DHCP_CALLOUT_TABLE", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ChainDlls", "CalloutVersion", "CalloutTbl"]), offset=0),
    "LPDHCP_GIVE_ADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "AltAddress", "AddrType", "LeaseTime", "Reserved", "PktContext"]), offset=0),
    "LPDHCP_HANDLE_OPTIONS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DHCP_SERVER_OPTIONS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "Reserved", "PktContext", "ServerOptions"]), offset=0),
    "LPDHCP_NEWPKT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "IpAddress", "Reserved", "PktContext", "ProcessIt"]), offset=0),
    "LPDHCP_PROB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Packet", "PacketSize", "ControlCode", "IpAddress", "AltAddress", "Reserved", "PktContext"]), offset=0),
    "LPDICONFIGUREDEVICESCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMCREATEDEFFECTOBJECTSCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirectInputEffect"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMDEVICEOBJECTSCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIDEVICEOBJECTINSTANCEA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMDEVICEOBJECTSCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIDEVICEOBJECTINSTANCEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMDEVICESBYSEMANTICSCBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIDEVICEINSTANCEA", SimStruct), offset=0), SimTypeBottom(label="IDirectInputDevice8A"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LPDIENUMDEVICESBYSEMANTICSCBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIDEVICEINSTANCEW", SimStruct), offset=0), SimTypeBottom(label="IDirectInputDevice8W"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LPDIENUMDEVICESCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIDEVICEINSTANCEA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMDEVICESCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIDEVICEINSTANCEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMEFFECTSCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIEFFECTINFOA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMEFFECTSCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIEFFECTINFOW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIENUMEFFECTSINFILECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DIFILEEFFECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIJOYTYPECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPDIRECTDRAWENUMERATEEXA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpCallback", "lpContext", "dwFlags"]), offset=0),
    "LPDIRECTDRAWENUMERATEEXW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpCallback", "lpContext", "dwFlags"]), offset=0),
    "LPDISPATCHNOTIFICATIONS": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ulFlags"]), offset=0),
    "LPDISPLAYVAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="RESULTTYPES"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "uiType", "szwVal", "szwDescription", "szwLocation"]), offset=0),
    "LPDRVCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hdrvr", "uMsg", "dwUser", "dw1", "dw2"]), offset=0),
    "LPDSENUMATTRIBUTES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lParam", "pszAttributeName", "pszDisplayName", "dwFlags"]), offset=0),
    "LPDSENUMCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPDSENUMCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPEVALCOMCALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="STATUSTYPES"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iStatus", "szData", "pContext"]), offset=0),
    "LPEXCEPFINO_DEFERRED_FILLIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pExcepInfo"]), offset=0),
    "LPFIBER_START_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFiberParameter"]), offset=0),
    "LPFNABSDI": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ulUIParam", "lpvmsg"]), offset=0),
    "LPFNACCESSIBLECHILDREN": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IAccessible"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["paccContainer", "iChildStart", "cChildren", "rgvarChildren", "pcObtained"]), offset=0),
    "LPFNACCESSIBLEOBJECTFROMPOINT": SimTypePointer(SimTypeFunction([SimTypeRef("POINT", SimStruct), SimTypePointer(SimTypeBottom(label="IAccessible"), offset=0), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ptScreen", "ppacc", "pvarChild"]), offset=0),
    "LPFNACCESSIBLEOBJECTFROMWINDOW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "dwId", "riid", "ppvObject"]), offset=0),
    "LPFNADDPROPSHEETPAGES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPFNBUTTON": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ulUIParam", "lpvContext", "cbEntryID", "lpSelection", "ulFlags"]), offset=0),
    "LPFNCANUNLOADNOW": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "LPFNCCINFOA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CCINFOA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["acci"]), offset=0),
    "LPFNCCINFOW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CCINFOW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["acci"]), offset=0),
    "LPFNCCSIZETOTEXTA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["flStyle", "flExtStyle", "hfont", "pszText"]), offset=0),
    "LPFNCCSIZETOTEXTW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["flStyle", "flExtStyle", "hfont", "pszText"]), offset=0),
    "LPFNCCSTYLEA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CCSTYLEA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pccs"]), offset=0),
    "LPFNCCSTYLEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CCSTYLEW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pccs"]), offset=0),
    "LPFNCREATESTDACCESSIBLEOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "idObject", "riid", "ppvObject"]), offset=0),
    "LPFNDEVCAPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "LPFNDEVMODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "LPFNDFMCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IShellFolder"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IDataObject"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["psf", "hwnd", "pdtobj", "uMsg", "wParam", "lParam"]), offset=0),
    "LPFNDIRECTSOUNDDEVICEENUMERATECALLBACK1": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_1_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPFNDIRECTSOUNDDEVICEENUMERATECALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_A_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPFNDIRECTSOUNDDEVICEENUMERATECALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_W_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPFNDISMISS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ulUIParam", "lpvContext"]), offset=0),
    "LPFNEXTDEVIO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lParam", "dwFlags", "dwIoControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpOverlapped"]), offset=0),
    "LPFNGETCLASSOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPFNLRESULTFROMOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeBottom(label="IUnknown")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["riid", "wParam", "punk"]), offset=0),
    "LPFNOBJECTFROMLRESULT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lResult", "riid", "wParam", "ppvObject"]), offset=0),
    "LPFNOLEUIHOOK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPFNPSPCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0),
    "LPFNPSPCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0),
    "LPFNSHOWJOYCPL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWnd"]), offset=0),
    "LPFNSVADDPROPSHEETPAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "LPFNVIEWCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IShellView"), SimTypeBottom(label="IShellFolder"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["psvOuter", "psf", "hwndMain", "uMsg", "wParam", "lParam"]), offset=0),
    "LPFN_ACCEPTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["sListenSocket", "sAcceptSocket", "lpOutputBuffer", "dwReceiveDataLength", "dwLocalAddressLength", "dwRemoteAddressLength", "lpdwBytesReceived", "lpOverlapped"]), offset=0),
    "LPFN_CONNECTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpSendBuffer", "dwSendDataLength", "lpdwBytesSent", "lpOverlapped"]), offset=0),
    "LPFN_DISCONNECTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpOverlapped", "dwFlags", "dwReserved"]), offset=0),
    "LPFN_GETACCEPTEXSOCKADDRS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpOutputBuffer", "dwReceiveDataLength", "dwLocalAddressLength", "dwRemoteAddressLength", "LocalSockaddr", "LocalSockaddrLength", "RemoteSockaddr", "RemoteSockaddrLength"]), offset=0),
    "LPFN_NSPAPI": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "LPFN_RIOCLOSECOMPLETIONQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["CQ"]), offset=0),
    "LPFN_RIOCREATECOMPLETIONQUEUE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RIO_NOTIFICATION_COMPLETION", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["QueueSize", "NotificationCompletion"]), offset=0),
    "LPFN_RIOCREATEREQUESTQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["Socket", "MaxOutstandingReceive", "MaxReceiveDataBuffers", "MaxOutstandingSend", "MaxSendDataBuffers", "ReceiveCQ", "SendCQ", "SocketContext"]), offset=0),
    "LPFN_RIODEQUEUECOMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIORESULT", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["CQ", "Array", "ArraySize"]), offset=0),
    "LPFN_RIODEREGISTERBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["BufferId"]), offset=0),
    "LPFN_RIONOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CQ"]), offset=0),
    "LPFN_RIORECEIVE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "Flags", "RequestContext"]), offset=0),
    "LPFN_RIORECEIVEEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "pLocalAddress", "pRemoteAddress", "pControlContext", "pFlags", "Flags", "RequestContext"]), offset=0),
    "LPFN_RIOREGISTERBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["DataBuffer", "DataLength"]), offset=0),
    "LPFN_RIORESIZECOMPLETIONQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CQ", "QueueSize"]), offset=0),
    "LPFN_RIORESIZEREQUESTQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["RQ", "MaxOutstandingReceive", "MaxOutstandingSend"]), offset=0),
    "LPFN_RIOSEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "Flags", "RequestContext"]), offset=0),
    "LPFN_RIOSENDEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "pLocalAddress", "pRemoteAddress", "pControlContext", "pFlags", "Flags", "RequestContext"]), offset=0),
    "LPFN_TRANSMITFILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeRef("TRANSMIT_FILE_BUFFERS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hSocket", "hFile", "nNumberOfBytesToWrite", "nNumberOfBytesPerSend", "lpOverlapped", "lpTransmitBuffers", "dwReserved"]), offset=0),
    "LPFN_TRANSMITPACKETS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("TRANSMIT_PACKETS_ELEMENT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hSocket", "lpPacketArray", "nElementCount", "nSendSize", "lpOverlapped", "dwFlags"]), offset=0),
    "LPFN_WSAPOLL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSAPOLLFD", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["fdarray", "nfds", "timeout"]), offset=0),
    "LPFN_WSARECVMSG": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSAMSG", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpMsg", "lpdwNumberOfBytesRecvd", "lpOverlapped", "lpCompletionRoutine"]), offset=0),
    "LPFN_WSASENDMSG": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSAMSG", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpMsg", "dwFlags", "lpNumberOfBytesSent", "lpOverlapped", "lpCompletionRoutine"]), offset=0),
    "LPFREEBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpBuffer"]), offset=0),
    "LPFRHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPGETTNEFSTREAMCODEPAGE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IStream"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpStream", "lpulCodepage", "lpulSubCodepage"]), offset=0),
    "LPGROUP_CALLBACK_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPHANDLER_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwControl"]), offset=0),
    "LPHANDLER_FUNCTION_EX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwControl", "dwEventType", "lpEventData", "lpContext"]), offset=0),
    "LPINTERNET_STATUS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hInternet", "dwContext", "dwInternetStatus", "lpvStatusInformation", "dwStatusInformationLength"]), offset=0),
    "LPLOOKUPSERVICE_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwError", "dwBytes", "lpOverlapped"]), offset=0),
    "LPMAPIADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MapiRecipDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MapiRecipDesc", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpszCaption", "nEditFields", "lpszLabels", "nRecips", "lpRecips", "flFlags", "ulReserved", "lpnNewRecips", "lppNewRecips"]), offset=0),
    "LPMAPIDELETEMAIL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpszMessageID", "flFlags", "ulReserved"]), offset=0),
    "LPMAPIDETAILS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("MapiRecipDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpRecip", "flFlags", "ulReserved"]), offset=0),
    "LPMAPIFINDNEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpszMessageType", "lpszSeedMessageID", "flFlags", "ulReserved", "lpszMessageID"]), offset=0),
    "LPMAPIFREEBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pv"]), offset=0),
    "LPMAPILOGOFF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "flFlags", "ulReserved"]), offset=0),
    "LPMAPILOGON": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ulUIParam", "lpszProfileName", "lpszPassword", "flFlags", "ulReserved", "lplhSession"]), offset=0),
    "LPMAPIREADMAIL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("MapiMessage", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpszMessageID", "flFlags", "ulReserved", "lppMessage"]), offset=0),
    "LPMAPIRESOLVENAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("MapiRecipDesc", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpszName", "flFlags", "ulReserved", "lppRecip"]), offset=0),
    "LPMAPISAVEMAIL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("MapiMessage", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpMessage", "flFlags", "ulReserved", "lpszMessageID"]), offset=0),
    "LPMAPISENDDOCUMENTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ulUIParam", "lpszDelimChar", "lpszFilePaths", "lpszFileNames", "ulReserved"]), offset=0),
    "LPMAPISENDMAIL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("MapiMessage", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpMessage", "flFlags", "ulReserved"]), offset=0),
    "LPMAPISENDMAILW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("MapiMessageW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lhSession", "ulUIParam", "lpMessage", "flFlags", "ulReserved"]), offset=0),
    "LPMIDICALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hdrvr", "uMsg", "dwUser", "dw1", "dw2"]), offset=0),
    "LPMIPTABLE": SimStruct(OrderedDict((("ulIfIndex", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeInt(signed=False, label="UInt32")), ("IfIpAddr", SimTypeRef("IN_ADDR", SimStruct)), ("IfNetMask", SimTypeRef("IN_ADDR", SimStruct)),)), name="LPMIPTABLE", pack=False, align=None),
    "LPMMIOPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpmmioinfo", "uMsg", "lParam1", "lParam2"]), offset=0),
    "LPMSGSESS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LPM_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LPM_INIT_INFO": SimStruct(OrderedDict((("PcmVersionNumber", SimTypeInt(signed=False, label="UInt32")), ("ResultTimeLimit", SimTypeInt(signed=False, label="UInt32")), ("ConfiguredLpmCount", SimTypeInt(signed=True, label="Int32")), ("AllocMemory", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Size"]), offset=0)), ("FreeMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)), ("PcmAdmitResultCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("POLICY_DECISION", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), arg_names=["LpmHandle", "RequestHandle", "ulPcmActionFlags", "LpmError", "PolicyDecisionsCount", "pPolicyDecisions"]), offset=0)), ("GetRsvpObjectsCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeRef("RsvpObjHdr", SimStruct), offset=0), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), arg_names=["LpmHandle", "RequestHandle", "LpmError", "RsvpObjectsCount", "ppRsvpObjects"]), offset=0)),)), name="LPM_INIT_INFO", pack=False, align=None),
    "LPNODE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CLUSTER_NODE_STATE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPNOTIFCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("NOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpvContext", "cNotification", "lpNotifications"]), offset=0),
    "LPNSPCLEANUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId"]), offset=0),
    "LPNSPGETSERVICECLASSINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpdwBufSize", "lpServiceClassInfo"]), offset=0),
    "LPNSPINSTALLSERVICECLASS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpServiceClassInfo"]), offset=0),
    "LPNSPIOCTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSACOMPLETION", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup", "dwControlCode", "lpvInBuffer", "cbInBuffer", "lpvOutBuffer", "cbOutBuffer", "lpcbBytesReturned", "lpCompletion", "lpThreadId"]), offset=0),
    "LPNSPLOOKUPSERVICEBEGIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSETW", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpqsRestrictions", "lpServiceClassInfo", "dwControlFlags", "lphLookup"]), offset=0),
    "LPNSPLOOKUPSERVICEEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup"]), offset=0),
    "LPNSPLOOKUPSERVICENEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSETW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup", "dwControlFlags", "lpdwBufferLength", "lpqsResults"]), offset=0),
    "LPNSPREMOVESERVICECLASS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpServiceClassId"]), offset=0),
    "LPNSPSETSERVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSETW", SimStruct), offset=0), SimTypeInt(signed=False, label="WSAESETSERVICEOP"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpServiceClassInfo", "lpqsRegInfo", "essOperation", "dwControlFlags"]), offset=0),
    "LPNSPSTARTUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("NSP_ROUTINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpnspRoutines"]), offset=0),
    "LPNSPV2CLEANUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "pvClientSessionArg"]), offset=0),
    "LPNSPV2CLIENTSESSIONRUNDOWN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpProviderId", "pvClientSessionArg"]), offset=0),
    "LPNSPV2LOOKUPSERVICEBEGIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSET2W", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpqsRestrictions", "dwControlFlags", "lpvClientSessionArg", "lphLookup"]), offset=0),
    "LPNSPV2LOOKUPSERVICEEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup"]), offset=0),
    "LPNSPV2LOOKUPSERVICENEXTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSET2W", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["hAsyncCall", "hLookup", "dwControlFlags", "lpdwBufferLength", "lpqsResults"]), offset=0),
    "LPNSPV2SETSERVICEEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSET2W", SimStruct), offset=0), SimTypeInt(signed=False, label="WSAESETSERVICEOP"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hAsyncCall", "lpProviderId", "lpqsRegInfo", "essOperation", "dwControlFlags", "lpvClientSessionArg"]), offset=0),
    "LPNSPV2STARTUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "ppvClientSessionArg"]), offset=0),
    "LPOCNCHKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPOCNCONNPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPOCNCONNPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPOCNDSCPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPOFNHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPOPENSTREAMONFILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbSize", "lppBuffer"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpBuffer"]), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeBottom(label="IStream"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpAllocateBuffer", "lpFreeBuffer", "ulFlags", "lpszFileName", "lpszPrefix", "lppStream"]), offset=0),
    "LPOPENTNEFSTREAM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeBottom(label="IStream"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMessage"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="ITnef"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpvSupport", "lpStream", "lpszStreamName", "ulFlags", "lpMessage", "wKeyVal", "lppTNEF"]), offset=0),
    "LPOPENTNEFSTREAMEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeBottom(label="IStream"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMessage"), SimTypeShort(signed=False, label="UInt16"), SimTypeBottom(label="IAddrBook"), SimTypePointer(SimTypeBottom(label="ITnef"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpvSupport", "lpStream", "lpszStreamName", "ulFlags", "lpMessage", "wKeyVal", "lpAdressBook", "lppTNEF"]), offset=0),
    "LPOVERLAPPED_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwErrorCode", "dwNumberOfBytesTransfered", "lpOverlapped"]), offset=0),
    "LPPAGEPAINTHOOK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPPAGESETUPHOOK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPPRINTHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPPROC_THREAD_ATTRIBUTE_LIST": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "LPPROGRESS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="LPPROGRESS_ROUTINE_CALLBACK_REASON"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["TotalFileSize", "TotalBytesTransferred", "StreamSize", "StreamBytesTransferred", "dwStreamNumber", "dwCallbackReason", "hSourceFile", "hDestinationFile", "lpData"]), offset=0),
    "LPPROGRESS_ROUTINE_CALLBACK_REASON": SimTypeInt(signed=False, label="LPPROGRESS_ROUTINE_CALLBACK_REASON"),
    "LPRESOURCE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "LPRESOURCE_CALLBACK_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPSERVICE_CALLBACK_PROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["lParam", "hAsyncTaskHandle"]), offset=0),
    "LPSERVICE_MAIN_FUNCTIONA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNumServicesArgs", "lpServiceArgVectors"]), offset=0),
    "LPSERVICE_MAIN_FUNCTIONW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNumServicesArgs", "lpServiceArgVectors"]), offset=0),
    "LPSETUPHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "LPTASKCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["dwInst"]), offset=0),
    "LPTHREAD_START_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0),
    "LPTIMECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["uTimerID", "uMsg", "dwUser", "dw1", "dw2"]), offset=0),
    "LPTOP_LEVEL_EXCEPTION_FILTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ExceptionInfo"]), offset=0),
    "LPWABACTIONITEM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LPWABALLOCATEBUFFER": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IWABObject"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpWABObject", "cbSize", "lppBuffer"]), offset=0),
    "LPWABALLOCATEMORE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IWABObject"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpWABObject", "cbSize", "lpObject", "lppBuffer"]), offset=0),
    "LPWABFREEBUFFER": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IWABObject"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpWABObject", "lpBuffer"]), offset=0),
    "LPWABOPEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="IAddrBook"), offset=0), SimTypePointer(SimTypeBottom(label="IWABObject"), offset=0), SimTypePointer(SimTypeRef("WAB_PARAM", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lppAdrBook", "lppWABObject", "lpWP", "Reserved2"]), offset=0),
    "LPWABOPENEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="IAddrBook"), offset=0), SimTypePointer(SimTypeBottom(label="IWABObject"), offset=0), SimTypePointer(SimTypeRef("WAB_PARAM", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbSize", "lppBuffer"]), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbSize", "lpObject", "lppBuffer"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpBuffer"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lppAdrBook", "lppWABObject", "lpWP", "Reserved", "fnAllocateBuffer", "fnAllocateMore", "fnFreeBuffer"]), offset=0),
    "LPWAVECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hdrvr", "uMsg", "dwUser", "dw1", "dw2"]), offset=0),
    "LPWPUCLOSEEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "lpErrno"]), offset=0),
    "LPWPUCLOSESOCKETHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpErrno"]), offset=0),
    "LPWPUCLOSETHREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpThreadId", "lpErrno"]), offset=0),
    "LPWPUCOMPLETEOVERLAPPEDREQUEST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpOverlapped", "dwError", "cbTransferred", "lpErrno"]), offset=0),
    "LPWPUCREATEEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpErrno"]), offset=0),
    "LPWPUCREATESOCKETHANDLE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["dwCatalogEntryId", "dwContext", "lpErrno"]), offset=0),
    "LPWPUFDISSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "fdset"]), offset=0),
    "LPWPUGETPROVIDERPATH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpszProviderDllPath", "lpProviderDllPathLen", "lpErrno"]), offset=0),
    "LPWPUMODIFYIFSHANDLE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["dwCatalogEntryId", "ProposedHandle", "lpErrno"]), offset=0),
    "LPWPUOPENCURRENTTHREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpThreadId", "lpErrno"]), offset=0),
    "LPWPUPOSTMESSAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "Msg", "wParam", "lParam"]), offset=0),
    "LPWPUQUERYBLOCKINGCALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwContext"]), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCatalogEntryId", "lplpfnCallback", "lpdwContext", "lpErrno"]), offset=0),
    "LPWPUQUERYSOCKETHANDLECONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpContext", "lpErrno"]), offset=0),
    "LPWPUQUEUEAPC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["dwContext"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpThreadId", "lpfnUserApc", "dwContext", "lpErrno"]), offset=0),
    "LPWPURESETEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "lpErrno"]), offset=0),
    "LPWPUSETEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "lpErrno"]), offset=0),
    "LPWSAOVERLAPPED_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0),
    "LPWSAUSERAPC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["dwContext"]), offset=0),
    "LPWSCDEINSTALLPROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpErrno"]), offset=0),
    "LPWSCENABLENSPROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "fEnable"]), offset=0),
    "LPWSCENUMPROTOCOLS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpiProtocols", "lpProtocolBuffer", "lpdwBufferLength", "lpErrno"]), offset=0),
    "LPWSCGETPROVIDERPATH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpszProviderDllPath", "lpProviderDllPathLen", "lpErrno"]), offset=0),
    "LPWSCINSTALLNAMESPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszIdentifier", "lpszPathName", "dwNameSpace", "dwVersion", "lpProviderId"]), offset=0),
    "LPWSCINSTALLPROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpszProviderDllPath", "lpProtocolInfoList", "dwNumberOfEntries", "lpErrno"]), offset=0),
    "LPWSCUNINSTALLNAMESPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId"]), offset=0),
    "LPWSCUPDATEPROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpszProviderDllPath", "lpProtocolInfoList", "dwNumberOfEntries", "lpErrno"]), offset=0),
    "LPWSCWRITENAMESPACEORDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "dwNumberOfEntries"]), offset=0),
    "LPWSCWRITEPROVIDERORDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpwdCatalogEntryId", "dwNumberOfEntries"]), offset=0),
    "LPWSPACCEPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpCallerId", "lpCallerData", "lpSQOS", "lpGQOS", "lpCalleeId", "lpCalleeData", "g", "dwCallbackData"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["s", "addr", "addrlen", "lpfnCondition", "dwCallbackData", "lpErrno"]), offset=0),
    "LPWSPADDRESSTOSTRING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpsaAddress", "dwAddressLength", "lpProtocolInfo", "lpszAddressString", "lpdwAddressStringLength", "lpErrno"]), offset=0),
    "LPWSPASYNCSELECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "hWnd", "wMsg", "lEvent", "lpErrno"]), offset=0),
    "LPWSPBIND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpErrno"]), offset=0),
    "LPWSPCANCELBLOCKINGCALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpErrno"]), offset=0),
    "LPWSPCLEANUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpErrno"]), offset=0),
    "LPWSPCLOSESOCKET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpErrno"]), offset=0),
    "LPWSPCONNECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpCallerData", "lpCalleeData", "lpSQOS", "lpGQOS", "lpErrno"]), offset=0),
    "LPWSPDUPLICATESOCKET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "dwProcessId", "lpProtocolInfo", "lpErrno"]), offset=0),
    "LPWSPENUMNETWORKEVENTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WSANETWORKEVENTS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "hEventObject", "lpNetworkEvents", "lpErrno"]), offset=0),
    "LPWSPEVENTSELECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "hEventObject", "lNetworkEvents", "lpErrno"]), offset=0),
    "LPWSPGETOVERLAPPEDRESULT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpOverlapped", "lpcbTransfer", "fWait", "lpdwFlags", "lpErrno"]), offset=0),
    "LPWSPGETPEERNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpErrno"]), offset=0),
    "LPWSPGETQOSBYNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpQOSName", "lpQOS", "lpErrno"]), offset=0),
    "LPWSPGETSOCKNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpErrno"]), offset=0),
    "LPWSPGETSOCKOPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "level", "optname", "optval", "optlen", "lpErrno"]), offset=0),
    "LPWSPIOCTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "dwIoControlCode", "lpvInBuffer", "cbInBuffer", "lpvOutBuffer", "cbOutBuffer", "lpcbBytesReturned", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0),
    "LPWSPJOINLEAF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["s", "name", "namelen", "lpCallerData", "lpCalleeData", "lpSQOS", "lpGQOS", "dwFlags", "lpErrno"]), offset=0),
    "LPWSPLISTEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "backlog", "lpErrno"]), offset=0),
    "LPWSPRECV": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesRecvd", "lpFlags", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0),
    "LPWSPRECVDISCONNECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpInboundDisconnectData", "lpErrno"]), offset=0),
    "LPWSPRECVFROM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesRecvd", "lpFlags", "lpFrom", "lpFromlen", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0),
    "LPWSPSELECT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("TIMEVAL", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["nfds", "readfds", "writefds", "exceptfds", "timeout", "lpErrno"]), offset=0),
    "LPWSPSEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesSent", "dwFlags", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0),
    "LPWSPSENDDISCONNECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpOutboundDisconnectData", "lpErrno"]), offset=0),
    "LPWSPSENDTO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesSent", "dwFlags", "lpTo", "iTolen", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0),
    "LPWSPSETSOCKOPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "level", "optname", "optval", "optlen", "lpErrno"]), offset=0),
    "LPWSPSHUTDOWN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "how", "lpErrno"]), offset=0),
    "LPWSPSOCKET": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["af", "type", "protocol", "lpProtocolInfo", "g", "dwFlags", "lpErrno"]), offset=0),
    "LPWSPSTARTUP": SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("WSPDATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypeRef("WSPUPCALLTABLE", SimStruct), SimTypePointer(SimTypeRef("WSPPROC_TABLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["wVersionRequested", "lpWSPData", "lpProtocolInfo", "UpcallTable", "lpProcTable"]), offset=0),
    "LPWSPSTRINGTOADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AddressString", "AddressFamily", "lpProtocolInfo", "lpAddress", "lpAddressLength", "lpErrno"]), offset=0),
    "LRESULT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LRPC_SYSTEM_HANDLE_MARSHAL_DIRECTION": SimTypeInt(signed=False, label="LRPC_SYSTEM_HANDLE_MARSHAL_DIRECTION"),
    "LSA_AP_POST_LOGON_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_POST_LOGON_USER_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PostLogonUserInfo"]), offset=0),
    "LSA_AUTH_INFORMATION": SimStruct(OrderedDict((("LastUpdateTime", SimTypeLongLong(signed=True, label="Int64")), ("AuthType", SimTypeInt(signed=False, label="LSA_AUTH_INFORMATION_AUTH_TYPE")), ("AuthInfoLength", SimTypeInt(signed=False, label="UInt32")), ("AuthInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="LSA_AUTH_INFORMATION", pack=False, align=None),
    "LSA_AUTH_INFORMATION_AUTH_TYPE": SimTypeInt(signed=False, label="LSA_AUTH_INFORMATION_AUTH_TYPE"),
    "LSA_DISPATCH_TABLE": SimStruct(OrderedDict((("CreateLogonSession", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0)), ("DeleteLogonSession", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0)), ("AddCredential", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "PrimaryKeyValue", "Credentials"]), offset=0)), ("GetCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "QueryContext", "RetrieveAllCredentials", "PrimaryKeyValue", "PrimaryKeyLength", "Credentials"]), offset=0)), ("DeleteCredential", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "PrimaryKeyValue"]), offset=0)), ("AllocateLsaHeap", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0)), ("FreeLsaHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0)), ("AllocateClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LengthRequired", "ClientBaseAddress"]), offset=0)), ("FreeClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ClientBaseAddress"]), offset=0)), ("CopyToClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "Length", "ClientBaseAddress", "BufferToCopy"]), offset=0)), ("CopyFromClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "Length", "BufferToCopy", "ClientBaseAddress"]), offset=0)),)), name="LSA_DISPATCH_TABLE", pack=False, align=None),
    "LSA_ENUMERATION_INFORMATION": SimStruct(OrderedDict((("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="LSA_ENUMERATION_INFORMATION", pack=False, align=None),
    "LSA_FOREST_TRUST_BINARY_DATA": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="LSA_FOREST_TRUST_BINARY_DATA", pack=False, align=None),
    "LSA_FOREST_TRUST_COLLISION_INFORMATION": SimStruct(OrderedDict((("RecordCount", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypePointer(SimTypeRef("LSA_FOREST_TRUST_COLLISION_RECORD", SimStruct), offset=0), offset=0)),)), name="LSA_FOREST_TRUST_COLLISION_INFORMATION", pack=False, align=None),
    "LSA_FOREST_TRUST_COLLISION_RECORD": SimStruct(OrderedDict((("Index", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="LSA_FOREST_TRUST_COLLISION_RECORD_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="LSA_FOREST_TRUST_COLLISION_RECORD", pack=False, align=None),
    "LSA_FOREST_TRUST_COLLISION_RECORD_TYPE": SimTypeInt(signed=False, label="LSA_FOREST_TRUST_COLLISION_RECORD_TYPE"),
    "LSA_FOREST_TRUST_DOMAIN_INFO": SimStruct(OrderedDict((("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DnsName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("NetbiosName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="LSA_FOREST_TRUST_DOMAIN_INFO", pack=False, align=None),
    "LSA_FOREST_TRUST_INFORMATION": SimStruct(OrderedDict((("RecordCount", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypePointer(SimTypeRef("LSA_FOREST_TRUST_RECORD", SimStruct), offset=0), offset=0)),)), name="LSA_FOREST_TRUST_INFORMATION", pack=False, align=None),
    "LSA_FOREST_TRUST_INFORMATION2": SimStruct(OrderedDict((("RecordCount", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypePointer(SimTypeRef("LSA_FOREST_TRUST_RECORD2", SimStruct), offset=0), offset=0)),)), name="LSA_FOREST_TRUST_INFORMATION2", pack=False, align=None),
    "LSA_FOREST_TRUST_RECORD": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ForestTrustType", SimTypeInt(signed=False, label="LSA_FOREST_TRUST_RECORD_TYPE")), ("Time", SimTypeLongLong(signed=True, label="Int64")), ("ForestTrustData", SimUnion({"TopLevelName": SimTypeRef("LSA_UNICODE_STRING", SimStruct), "DomainInfo": SimTypeRef("LSA_FOREST_TRUST_DOMAIN_INFO", SimStruct), "Data": SimTypeRef("LSA_FOREST_TRUST_BINARY_DATA", SimStruct)}, name="<anon>", label="None")),)), name="LSA_FOREST_TRUST_RECORD", pack=False, align=None),
    "LSA_FOREST_TRUST_RECORD2": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ForestTrustType", SimTypeInt(signed=False, label="LSA_FOREST_TRUST_RECORD_TYPE")), ("Time", SimTypeLongLong(signed=True, label="Int64")), ("ForestTrustData", SimUnion({"TopLevelName": SimTypeRef("LSA_UNICODE_STRING", SimStruct), "DomainInfo": SimTypeRef("LSA_FOREST_TRUST_DOMAIN_INFO", SimStruct), "BinaryData": SimTypeRef("LSA_FOREST_TRUST_BINARY_DATA", SimStruct), "ScannerInfo": SimTypeRef("LSA_FOREST_TRUST_SCANNER_INFO", SimStruct)}, name="<anon>", label="None")),)), name="LSA_FOREST_TRUST_RECORD2", pack=False, align=None),
    "LSA_FOREST_TRUST_RECORD_TYPE": SimTypeInt(signed=False, label="LSA_FOREST_TRUST_RECORD_TYPE"),
    "LSA_FOREST_TRUST_SCANNER_INFO": SimStruct(OrderedDict((("DomainSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DnsName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("NetbiosName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="LSA_FOREST_TRUST_SCANNER_INFO", pack=False, align=None),
    "LSA_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "LSA_LAST_INTER_LOGON_INFO": SimStruct(OrderedDict((("LastSuccessfulLogon", SimTypeLongLong(signed=True, label="Int64")), ("LastFailedLogon", SimTypeLongLong(signed=True, label="Int64")), ("FailedAttemptCountSinceLastSuccessfulLogon", SimTypeInt(signed=False, label="UInt32")),)), name="LSA_LAST_INTER_LOGON_INFO", pack=False, align=None),
    "LSA_LOOKUP_DOMAIN_INFO_CLASS": SimTypeInt(signed=False, label="LSA_LOOKUP_DOMAIN_INFO_CLASS"),
    "LSA_OBJECT_ATTRIBUTES": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("RootDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ObjectName", SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityQualityOfService", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="LSA_OBJECT_ATTRIBUTES", pack=False, align=None),
    "LSA_REFERENCED_DOMAIN_LIST": SimStruct(OrderedDict((("Entries", SimTypeInt(signed=False, label="UInt32")), ("Domains", SimTypePointer(SimTypeRef("LSA_TRUST_INFORMATION", SimStruct), offset=0)),)), name="LSA_REFERENCED_DOMAIN_LIST", pack=False, align=None),
    "LSA_SECPKG_FUNCTION_TABLE": SimStruct(OrderedDict((("CreateLogonSession", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0)), ("DeleteLogonSession", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0)), ("AddCredential", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "PrimaryKeyValue", "Credentials"]), offset=0)), ("GetCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "QueryContext", "RetrieveAllCredentials", "PrimaryKeyValue", "PrimaryKeyLength", "Credentials"]), offset=0)), ("DeleteCredential", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "PrimaryKeyValue"]), offset=0)), ("AllocateLsaHeap", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0)), ("FreeLsaHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0)), ("AllocateClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LengthRequired", "ClientBaseAddress"]), offset=0)), ("FreeClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ClientBaseAddress"]), offset=0)), ("CopyToClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "Length", "ClientBaseAddress", "BufferToCopy"]), offset=0)), ("CopyFromClientBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "Length", "BufferToCopy", "ClientBaseAddress"]), offset=0)), ("ImpersonateClient", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnloadPackage", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DuplicateHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SourceHandle", "DestionationHandle"]), offset=0)), ("SaveSupplementalCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "SupplementalCredSize", "SupplementalCreds", "Synchronous"]), offset=0)), ("CreateThread", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["SecurityAttributes", "StackSize", "StartFunction", "ThreadParameter", "CreationFlags", "ThreadId"]), offset=0)), ("GetClientInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_CLIENT_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientInfo"]), offset=0)), ("RegisterNotification", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["StartFunction", "Parameter", "NotificationType", "NotificationClass", "NotificationFlags", "IntervalMinutes", "WaitEvent"]), offset=0)), ("CancelNotification", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["NotifyHandle"]), offset=0)), ("MapBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InputBuffer", "OutputBuffer"]), offset=0)), ("CreateToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "TokenSource", "LogonType", "ImpersonationLevel", "TokenInformationType", "TokenInformation", "TokenGroups", "AccountName", "AuthorityName", "Workstation", "ProfilePath", "Token", "SubStatus"]), offset=0)), ("AuditLogon", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "SubStatus", "AccountName", "AuthenticatingAuthority", "WorkstationName", "UserSid", "LogonType", "TokenSource", "LogonId"]), offset=0)), ("CallPackage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackage", "ProtocolSubmitBuffer", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0)), ("FreeReturnBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0)), ("GetCallInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_CALL_INFO", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["Info"]), offset=0)), ("CallPackageEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackage", "ClientBufferBase", "ProtocolSubmitBuffer", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0)), ("CreateSharedMemory", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["MaxSize", "InitialSize"]), offset=0)), ("AllocateSharedMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["SharedMem", "Size"]), offset=0)), ("FreeSharedMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["SharedMem", "Memory"]), offset=0)), ("DeleteSharedMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["SharedMem"]), offset=0)), ("OpenSamUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="SECPKG_NAME_TYPE"), SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Name", "NameType", "Prefix", "AllowGuest", "Reserved", "UserHandle"]), offset=0)), ("GetUserCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserHandle", "PrimaryCreds", "PrimaryCredsSize", "SupplementalCreds", "SupplementalCredsSize"]), offset=0)), ("GetUserAuthData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserHandle", "UserAuthData", "UserAuthDataSize"]), offset=0)), ("CloseSamUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserHandle"]), offset=0)), ("ConvertAuthDataToToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserAuthData", "UserAuthDataSize", "ImpersonationLevel", "TokenSource", "LogonType", "AuthorityName", "Token", "LogonId", "AccountName", "SubStatus"]), offset=0)), ("ClientCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Callback", "Argument1", "Argument2", "Input", "Output"]), offset=0)), ("UpdateCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrimaryCredentials", "Credentials"]), offset=0)), ("GetAuthDataForUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="SECPKG_NAME_TYPE"), SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Name", "NameType", "Prefix", "UserAuthData", "UserAuthDataSize", "UserFlatName"]), offset=0)), ("CrackSingleName", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FormatOffered", "PerformAtGC", "NameInput", "Prefix", "RequestedFormat", "CrackedName", "DnsDomainName", "SubStatus"]), offset=0)), ("AuditAccountLogon", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["AuditId", "Success", "Source", "ClientName", "MappedName", "Status"]), offset=0)), ("CallPackagePassthrough", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackage", "ClientBufferBase", "ProtocolSubmitBuffer", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0)), ("CrediRead", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "CredFlags", "TargetName", "Type", "Flags", "Credential"]), offset=0)), ("CrediReadDomainCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CREDENTIAL_TARGET_INFORMATIONW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "CredFlags", "TargetInfo", "Flags", "Count", "Credential"]), offset=0)), ("CrediFreeCredentials", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), label="LPArray", offset=0)], SimTypeBottom(label="Void"), arg_names=["Count", "Credentials"]), offset=0)), ("LsaProtectMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferSize"]), offset=0)), ("LsaUnprotectMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferSize"]), offset=0)), ("OpenTokenByLogonId", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "RetTokenHandle"]), offset=0)), ("ExpandAuthDataForDomain", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserAuthData", "UserAuthDataSize", "Reserved", "ExpandedAuthData", "ExpandedAuthDataSize"]), offset=0)), ("AllocatePrivateHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0)), ("FreePrivateHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0)), ("CreateTokenEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SECPKG_SESSIONINFO_TYPE"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "TokenSource", "LogonType", "ImpersonationLevel", "TokenInformationType", "TokenInformation", "TokenGroups", "Workstation", "ProfilePath", "SessionInformation", "SessionInformationType", "Token", "SubStatus"]), offset=0)), ("CrediWrite", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("ENCRYPTED_CREDENTIALW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "CredFlags", "Credential", "Flags"]), offset=0)), ("CrediUnmarshalandDecodeString", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MarshaledString", "Blob", "BlobSize", "IsFailureFatal"]), offset=0)), ("DummyFunction6", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferSize"]), offset=0)), ("GetExtendedCallFlags", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Flags"]), offset=0)), ("DuplicateTokenHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SourceHandle", "DestionationHandle"]), offset=0)), ("GetServiceAccountPassword", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="CRED_FETCH"), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AccountName", "DomainName", "CredFetch", "FileTimeExpiry", "CurrentPassword", "PreviousPassword", "FileTimeCurrPwdValidForOutbound"]), offset=0)), ("DummyFunction7", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferSize"]), offset=0)), ("AuditLogonEx", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "SubStatus", "AccountName", "AuthenticatingAuthority", "WorkstationName", "UserSid", "LogonType", "ImpersonationLevel", "TokenSource", "LogonId"]), offset=0)), ("CheckProtectedUserByToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserToken", "ProtectedUser"]), offset=0)), ("QueryClientRequest", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "QueryType", "ReplyBuffer"]), offset=0)), ("GetAppModeInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserFunction", "Argument1", "Argument2", "UserData", "ReturnToLsa"]), offset=0)), ("SetAppModeInfo", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["UserFunction", "Argument1", "Argument2", "UserData", "ReturnToLsa"]), offset=0)), ("GetClientInfoEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_CLIENT_INFO_EX", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientInfo", "StructSize"]), offset=0)),)), name="LSA_SECPKG_FUNCTION_TABLE", pack=False, align=None),
    "LSA_STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="LSA_STRING", pack=False, align=None),
    "LSA_TOKEN_INFORMATION_NULL": SimStruct(OrderedDict((("ExpirationTime", SimTypeLongLong(signed=True, label="Int64")), ("Groups", SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0)),)), name="LSA_TOKEN_INFORMATION_NULL", pack=False, align=None),
    "LSA_TOKEN_INFORMATION_TYPE": SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"),
    "LSA_TOKEN_INFORMATION_V1": SimStruct(OrderedDict((("ExpirationTime", SimTypeLongLong(signed=True, label="Int64")), ("User", SimTypeRef("TOKEN_USER", SimStruct)), ("Groups", SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0)), ("PrimaryGroup", SimTypeRef("TOKEN_PRIMARY_GROUP", SimStruct)), ("Privileges", SimTypePointer(SimTypeRef("TOKEN_PRIVILEGES", SimStruct), offset=0)), ("Owner", SimTypeRef("TOKEN_OWNER", SimStruct)), ("DefaultDacl", SimTypeRef("TOKEN_DEFAULT_DACL", SimStruct)),)), name="LSA_TOKEN_INFORMATION_V1", pack=False, align=None),
    "LSA_TOKEN_INFORMATION_V3": SimStruct(OrderedDict((("ExpirationTime", SimTypeLongLong(signed=True, label="Int64")), ("User", SimTypeRef("TOKEN_USER", SimStruct)), ("Groups", SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0)), ("PrimaryGroup", SimTypeRef("TOKEN_PRIMARY_GROUP", SimStruct)), ("Privileges", SimTypePointer(SimTypeRef("TOKEN_PRIVILEGES", SimStruct), offset=0)), ("Owner", SimTypeRef("TOKEN_OWNER", SimStruct)), ("DefaultDacl", SimTypeRef("TOKEN_DEFAULT_DACL", SimStruct)), ("UserClaims", SimTypeRef("TOKEN_USER_CLAIMS", SimStruct)), ("DeviceClaims", SimTypeRef("TOKEN_DEVICE_CLAIMS", SimStruct)), ("DeviceGroups", SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0)),)), name="LSA_TOKEN_INFORMATION_V3", pack=False, align=None),
    "LSA_TRANSLATED_NAME": SimStruct(OrderedDict((("Use", SimTypeInt(signed=False, label="SID_NAME_USE")), ("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainIndex", SimTypeInt(signed=True, label="Int32")),)), name="LSA_TRANSLATED_NAME", pack=False, align=None),
    "LSA_TRANSLATED_SID": SimStruct(OrderedDict((("Use", SimTypeInt(signed=False, label="SID_NAME_USE")), ("RelativeId", SimTypeInt(signed=False, label="UInt32")), ("DomainIndex", SimTypeInt(signed=True, label="Int32")),)), name="LSA_TRANSLATED_SID", pack=False, align=None),
    "LSA_TRANSLATED_SID2": SimStruct(OrderedDict((("Use", SimTypeInt(signed=False, label="SID_NAME_USE")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DomainIndex", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="LSA_TRANSLATED_SID2", pack=False, align=None),
    "LSA_TRUST_INFORMATION": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="LSA_TRUST_INFORMATION", pack=False, align=None),
    "LSA_UNICODE_STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="LSA_UNICODE_STRING", pack=False, align=None),
    "LUID": SimStruct(OrderedDict((("LowPart", SimTypeInt(signed=False, label="UInt32")), ("HighPart", SimTypeInt(signed=True, label="Int32")),)), name="LUID", pack=False, align=None),
    "LUID_AND_ATTRIBUTES": SimStruct(OrderedDict((("Luid", SimTypeRef("LUID", SimStruct)), ("Attributes", SimTypeInt(signed=False, label="TOKEN_PRIVILEGES_ATTRIBUTES")),)), name="LUID_AND_ATTRIBUTES", pack=False, align=None),
    "LVBKIMAGEA": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="LIST_VIEW_BACKGROUND_IMAGE_FLAGS")), ("hbm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszImage", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchImageMax", SimTypeInt(signed=False, label="UInt32")), ("xOffsetPercent", SimTypeInt(signed=True, label="Int32")), ("yOffsetPercent", SimTypeInt(signed=True, label="Int32")),)), name="LVBKIMAGEA", pack=False, align=None),
    "LVBKIMAGEW": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="LIST_VIEW_BACKGROUND_IMAGE_FLAGS")), ("hbm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszImage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchImageMax", SimTypeInt(signed=False, label="UInt32")), ("xOffsetPercent", SimTypeInt(signed=True, label="Int32")), ("yOffsetPercent", SimTypeInt(signed=True, label="Int32")),)), name="LVBKIMAGEW", pack=False, align=None),
    "LVCOLUMNA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="LVCOLUMNW_MASK")), ("fmt", SimTypeInt(signed=False, label="LVCOLUMNW_FORMAT")), ("cx", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iOrder", SimTypeInt(signed=True, label="Int32")), ("cxMin", SimTypeInt(signed=True, label="Int32")), ("cxDefault", SimTypeInt(signed=True, label="Int32")), ("cxIdeal", SimTypeInt(signed=True, label="Int32")),)), name="LVCOLUMNA", pack=False, align=None),
    "LVCOLUMNW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="LVCOLUMNW_MASK")), ("fmt", SimTypeInt(signed=False, label="LVCOLUMNW_FORMAT")), ("cx", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iOrder", SimTypeInt(signed=True, label="Int32")), ("cxMin", SimTypeInt(signed=True, label="Int32")), ("cxDefault", SimTypeInt(signed=True, label="Int32")), ("cxIdeal", SimTypeInt(signed=True, label="Int32")),)), name="LVCOLUMNW", pack=False, align=None),
    "LVCOLUMNW_FORMAT": SimTypeInt(signed=False, label="LVCOLUMNW_FORMAT"),
    "LVCOLUMNW_MASK": SimTypeInt(signed=False, label="LVCOLUMNW_MASK"),
    "LVFINDINFOA": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="LVFINDINFOW_FLAGS")), ("psz", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pt", SimTypeRef("POINT", SimStruct)), ("vkDirection", SimTypeInt(signed=False, label="UInt32")),)), name="LVFINDINFOA", pack=False, align=None),
    "LVFINDINFOW": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="LVFINDINFOW_FLAGS")), ("psz", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pt", SimTypeRef("POINT", SimStruct)), ("vkDirection", SimTypeInt(signed=False, label="UInt32")),)), name="LVFINDINFOW", pack=False, align=None),
    "LVFINDINFOW_FLAGS": SimTypeInt(signed=False, label="LVFINDINFOW_FLAGS"),
    "LVFOOTERINFO": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("cItems", SimTypeInt(signed=False, label="UInt32")),)), name="LVFOOTERINFO", pack=False, align=None),
    "LVFOOTERITEM": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="LVFOOTERITEM_MASK")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("state", SimTypeInt(signed=False, label="UInt32")), ("stateMask", SimTypeInt(signed=False, label="UInt32")),)), name="LVFOOTERITEM", pack=False, align=None),
    "LVFOOTERITEM_MASK": SimTypeInt(signed=False, label="LVFOOTERITEM_MASK"),
    "LVGROUP": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("mask", SimTypeInt(signed=False, label="LVGROUP_MASK")), ("pszHeader", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchHeader", SimTypeInt(signed=True, label="Int32")), ("pszFooter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchFooter", SimTypeInt(signed=True, label="Int32")), ("iGroupId", SimTypeInt(signed=True, label="Int32")), ("stateMask", SimTypeInt(signed=False, label="LIST_VIEW_GROUP_STATE_FLAGS")), ("state", SimTypeInt(signed=False, label="LIST_VIEW_GROUP_STATE_FLAGS")), ("uAlign", SimTypeInt(signed=False, label="LIST_VIEW_GROUP_ALIGN_FLAGS")), ("pszSubtitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchSubtitle", SimTypeInt(signed=False, label="UInt32")), ("pszTask", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTask", SimTypeInt(signed=False, label="UInt32")), ("pszDescriptionTop", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchDescriptionTop", SimTypeInt(signed=False, label="UInt32")), ("pszDescriptionBottom", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchDescriptionBottom", SimTypeInt(signed=False, label="UInt32")), ("iTitleImage", SimTypeInt(signed=True, label="Int32")), ("iExtendedImage", SimTypeInt(signed=True, label="Int32")), ("iFirstItem", SimTypeInt(signed=True, label="Int32")), ("cItems", SimTypeInt(signed=False, label="UInt32")), ("pszSubsetTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchSubsetTitle", SimTypeInt(signed=False, label="UInt32")),)), name="LVGROUP", pack=False, align=None),
    "LVGROUPMETRICS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("mask", SimTypeInt(signed=False, label="UInt32")), ("Left", SimTypeInt(signed=False, label="UInt32")), ("Top", SimTypeInt(signed=False, label="UInt32")), ("Right", SimTypeInt(signed=False, label="UInt32")), ("Bottom", SimTypeInt(signed=False, label="UInt32")), ("crLeft", SimTypeInt(signed=False, label="UInt32")), ("crTop", SimTypeInt(signed=False, label="UInt32")), ("crRight", SimTypeInt(signed=False, label="UInt32")), ("crBottom", SimTypeInt(signed=False, label="UInt32")), ("crHeader", SimTypeInt(signed=False, label="UInt32")), ("crFooter", SimTypeInt(signed=False, label="UInt32")),)), name="LVGROUPMETRICS", pack=False, align=None),
    "LVGROUP_MASK": SimTypeInt(signed=False, label="LVGROUP_MASK"),
    "LVHITTESTINFO": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("flags", SimTypeInt(signed=False, label="LVHITTESTINFO_FLAGS")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("iGroup", SimTypeInt(signed=True, label="Int32")),)), name="LVHITTESTINFO", pack=False, align=None),
    "LVHITTESTINFO_FLAGS": SimTypeInt(signed=False, label="LVHITTESTINFO_FLAGS"),
    "LVINSERTGROUPSORTED": SimStruct(OrderedDict((("pfnGroupCompare", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lvGroup", SimTypeRef("LVGROUP", SimStruct)),)), name="LVINSERTGROUPSORTED", pack=False, align=None),
    "LVINSERTMARK": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="LVINSERTMARK", pack=False, align=None),
    "LVITEMA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_FLAGS")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("state", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS")), ("stateMask", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIndent", SimTypeInt(signed=True, label="Int32")), ("iGroupId", SimTypeInt(signed=True, label="Int32")), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("puColumns", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("piColFmt", SimTypePointer(SimTypeInt(signed=False, label="LIST_VIEW_ITEM_COLUMN_FORMAT_FLAGS"), offset=0)), ("iGroup", SimTypeInt(signed=True, label="Int32")),)), name="LVITEMA", pack=False, align=None),
    "LVITEMA_GROUP_ID": SimTypeInt(signed=False, label="LVITEMA_GROUP_ID"),
    "LVITEMINDEX": SimStruct(OrderedDict((("iItem", SimTypeInt(signed=True, label="Int32")), ("iGroup", SimTypeInt(signed=True, label="Int32")),)), name="LVITEMINDEX", pack=False, align=None),
    "LVITEMW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_FLAGS")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("state", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS")), ("stateMask", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIndent", SimTypeInt(signed=True, label="Int32")), ("iGroupId", SimTypeInt(signed=True, label="Int32")), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("puColumns", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("piColFmt", SimTypePointer(SimTypeInt(signed=False, label="LIST_VIEW_ITEM_COLUMN_FORMAT_FLAGS"), offset=0)), ("iGroup", SimTypeInt(signed=True, label="Int32")),)), name="LVITEMW", pack=False, align=None),
    "LVSETINFOTIP": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")),)), name="LVSETINFOTIP", pack=False, align=None),
    "LVTILEINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("cColumns", SimTypeInt(signed=False, label="UInt32")), ("puColumns", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("piColFmt", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="LVTILEINFO", pack=False, align=None),
    "LVTILEVIEWINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="LVTILEVIEWINFO_MASK")), ("dwFlags", SimTypeInt(signed=False, label="LVTILEVIEWINFO_FLAGS")), ("sizeTile", SimTypeRef("SIZE", SimStruct)), ("cLines", SimTypeInt(signed=True, label="Int32")), ("rcLabelMargin", SimTypeRef("RECT", SimStruct)),)), name="LVTILEVIEWINFO", pack=False, align=None),
    "LVTILEVIEWINFO_FLAGS": SimTypeInt(signed=False, label="LVTILEVIEWINFO_FLAGS"),
    "LVTILEVIEWINFO_MASK": SimTypeInt(signed=False, label="LVTILEVIEWINFO_MASK"),
    "LZOPENFILE_STYLE": SimTypeInt(signed=False, label="LZOPENFILE_STYLE"),
    "LabCOLOR": SimStruct(OrderedDict((("L", SimTypeShort(signed=False, label="UInt16")), ("a", SimTypeShort(signed=False, label="UInt16")), ("b", SimTypeShort(signed=False, label="UInt16")),)), name="LabCOLOR", pack=False, align=None),
    "LanguageInfo": SimStruct(OrderedDict((("LangID", SimTypeShort(signed=False, label="UInt16")), ("lISOLangCode", SimTypeInt(signed=True, label="Int32")),)), name="LanguageInfo", pack=False, align=None),
    "LanguageKind": SimTypeInt(signed=False, label="LanguageKind"),
    "Levels": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="Levels", pack=False, align=None),
    "LevelsParams": SimStruct(OrderedDict((("highlight", SimTypeInt(signed=True, label="Int32")), ("midtone", SimTypeInt(signed=True, label="Int32")), ("shadow", SimTypeInt(signed=True, label="Int32")),)), name="LevelsParams", pack=False, align=None),
    "LicenseEventBlockReason": SimTypeInt(signed=False, label="LicenseEventBlockReason"),
    "LicenseProtectionStatus": SimTypeInt(signed=False, label="LicenseProtectionStatus"),
    "LineCap": SimTypeInt(signed=False, label="LineCap"),
    "LineJoin": SimTypeInt(signed=False, label="LineJoin"),
    "LinearGradientMode": SimTypeInt(signed=False, label="LinearGradientMode"),
    "LiveSetting": SimTypeInt(signed=False, label="LiveSetting"),
    "Location": SimStruct(OrderedDict((("HostDefined", SimTypeLongLong(signed=False, label="UInt64")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")),)), name="Location", pack=False, align=None),
    "LocationCodeSchemeType": SimTypeInt(signed=False, label="LocationCodeSchemeType"),
    "LocationKind": SimTypeInt(signed=False, label="LocationKind"),
    "LockModes": SimTypeInt(signed=False, label="LockModes"),
    "M128A": SimStruct(OrderedDict((("Low", SimTypeLongLong(signed=False, label="UInt64")), ("High", SimTypeLongLong(signed=True, label="Int64")),)), name="M128A", pack=False, align=None),
    "MACHINE_ATTRIBUTES": SimTypeInt(signed=False, label="MACHINE_ATTRIBUTES"),
    "MACHINE_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("MinSleepAc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("MinSleepDc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("ReducedLatencySleepAc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("ReducedLatencySleepDc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("DozeTimeoutAc", SimTypeInt(signed=False, label="UInt32")), ("DozeTimeoutDc", SimTypeInt(signed=False, label="UInt32")), ("DozeS4TimeoutAc", SimTypeInt(signed=False, label="UInt32")), ("DozeS4TimeoutDc", SimTypeInt(signed=False, label="UInt32")), ("MinThrottleAc", SimTypeChar(label="Byte")), ("MinThrottleDc", SimTypeChar(label="Byte")), ("pad1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("OverThrottledAc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("OverThrottledDc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)),)), name="MACHINE_POWER_POLICY", pack=False, align=None),
    "MACHINE_PROCESSOR_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("ProcessorPolicyAc", SimTypeRef("PROCESSOR_POWER_POLICY", SimStruct)), ("ProcessorPolicyDc", SimTypeRef("PROCESSOR_POWER_POLICY", SimStruct)),)), name="MACHINE_PROCESSOR_POWER_POLICY", pack=False, align=None),
    "MACINFO": SimStruct(OrderedDict((("fUsed", SimTypeInt(signed=True, label="Int32")), ("abMacState", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="MACINFO", pack=False, align=None),
    "MACROBLOCK_DATA": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("motionVectorX", SimTypeShort(signed=True, label="Int16")), ("motionVectorY", SimTypeShort(signed=True, label="Int16")), ("QPDelta", SimTypeInt(signed=True, label="Int32")),)), name="MACROBLOCK_DATA", pack=False, align=None),
    "MAGCOLOREFFECT": SimStruct(OrderedDict((("transform", SimTypeArray(SimTypeFloat(size=32), 25)),)), name="MAGCOLOREFFECT", pack=False, align=None),
    "MAGIMAGEHEADER": SimStruct(OrderedDict((("width", SimTypeInt(signed=False, label="UInt32")), ("height", SimTypeInt(signed=False, label="UInt32")), ("format", SimTypeBottom(label="Guid")), ("stride", SimTypeInt(signed=False, label="UInt32")), ("offset", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MAGIMAGEHEADER", pack=False, align=None),
    "MAGNETOMETER_ACCURACY": SimTypeInt(signed=False, label="MAGNETOMETER_ACCURACY"),
    "MAGTRANSFORM": SimStruct(OrderedDict((("v", SimTypeArray(SimTypeFloat(size=32), 9)),)), name="MAGTRANSFORM", pack=False, align=None),
    "MAILSLOT_CREATE_PARAMETERS": SimStruct(OrderedDict((("MailslotQuota", SimTypeInt(signed=False, label="UInt32")), ("MaximumMessageSize", SimTypeInt(signed=False, label="UInt32")), ("ReadTimeout", SimTypeLongLong(signed=True, label="Int64")), ("TimeoutSpecified", SimTypeChar(label="Byte")),)), name="MAILSLOT_CREATE_PARAMETERS", pack=False, align=None),
    "MAINTENANCE_MODE_TYPE_ENUM": SimTypeInt(signed=False, label="MAINTENANCE_MODE_TYPE_ENUM"),
    "MAKE_SIGNATURE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "MALLOC_FREE_STRUCT": SimStruct(OrderedDict((("pfnAllocate", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnFree", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MALLOC_FREE_STRUCT", pack=False, align=None),
    "MALLOC_TYPE": SimTypeInt(signed=False, label="MALLOC_TYPE"),
    "MANAGEDAPPLICATION": SimStruct(OrderedDict((("pszPackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPublisher", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwVersionHi", SimTypeInt(signed=False, label="UInt32")), ("dwVersionLo", SimTypeInt(signed=False, label="UInt32")), ("dwRevision", SimTypeInt(signed=False, label="UInt32")), ("GpoId", SimTypeBottom(label="Guid")), ("pszPolicyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProductId", SimTypeBottom(label="Guid")), ("Language", SimTypeShort(signed=False, label="UInt16")), ("pszOwner", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCompany", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszComments", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszContact", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSupportUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPathType", SimTypeInt(signed=False, label="UInt32")), ("bInstalled", SimTypeInt(signed=True, label="Int32")),)), name="MANAGEDAPPLICATION", pack=False, align=None),
    "MANAGEMENT_REGISTRATION_INFO": SimStruct(OrderedDict((("fDeviceRegisteredWithManagement", SimTypeInt(signed=True, label="Int32")), ("dwDeviceRegistionKind", SimTypeInt(signed=False, label="UInt32")), ("pszUPN", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszMDMServiceUri", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MANAGEMENT_REGISTRATION_INFO", pack=False, align=None),
    "MANAGEMENT_SERVICE_INFO": SimStruct(OrderedDict((("pszMDMServiceUri", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszAuthenticationUri", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MANAGEMENT_SERVICE_INFO", pack=False, align=None),
    "MANCODE": SimTypeInt(signed=False, label="MANCODE"),
    "MANDATORY_LEVEL": SimTypeInt(signed=False, label="MANDATORY_LEVEL"),
    "MANIPULATION_PROCESSOR_MANIPULATIONS": SimTypeInt(signed=False, label="MANIPULATION_PROCESSOR_MANIPULATIONS"),
    "MANIPULATION_RAILS_STATE": SimTypeInt(signed=False, label="MANIPULATION_RAILS_STATE"),
    "MANIPULATION_TRANSFORM": SimStruct(OrderedDict((("translationX", SimTypeFloat(size=32)), ("translationY", SimTypeFloat(size=32)), ("scale", SimTypeFloat(size=32)), ("expansion", SimTypeFloat(size=32)), ("rotation", SimTypeFloat(size=32)),)), name="MANIPULATION_TRANSFORM", pack=False, align=None),
    "MANIPULATION_VELOCITY": SimStruct(OrderedDict((("velocityX", SimTypeFloat(size=32)), ("velocityY", SimTypeFloat(size=32)), ("velocityExpansion", SimTypeFloat(size=32)), ("velocityAngular", SimTypeFloat(size=32)),)), name="MANIPULATION_VELOCITY", pack=False, align=None),
    "MAPIERROR": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszError", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lpszComponent", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("ulLowLevelError", SimTypeInt(signed=False, label="UInt32")), ("ulContext", SimTypeInt(signed=False, label="UInt32")),)), name="MAPIERROR", pack=False, align=None),
    "MAPINAMEID": SimStruct(OrderedDict((("lpguid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ulKind", SimTypeInt(signed=False, label="UInt32")), ("Kind", SimUnion({"lID": SimTypeInt(signed=True, label="Int32"), "lpwstrName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="MAPINAMEID", pack=False, align=None),
    "MAPIUID": SimStruct(OrderedDict((("ab", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="MAPIUID", pack=False, align=None),
    "MAPPING_DATA_RANGE": SimStruct(OrderedDict((("dwStartIndex", SimTypeInt(signed=False, label="UInt32")), ("dwEndIndex", SimTypeInt(signed=False, label="UInt32")), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwDescriptionLength", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("pszContentType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("prgActionIds", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwActionsCount", SimTypeInt(signed=False, label="UInt32")), ("prgActionDisplayNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="MAPPING_DATA_RANGE", pack=False, align=None),
    "MAPPING_ENUM_OPTIONS": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pszCategory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInputLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOutputLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInputScript", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOutputScript", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInputContentType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOutputContentType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="MAPPING_ENUM_OPTIONS", pack=False, align=None),
    "MAPPING_OPTIONS": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pszInputLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOutputLanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInputScript", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOutputScript", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInputContentType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOutputContentType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUILanguage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnRecognizeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MAPPING_PROPERTY_BAG", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["pBag", "data", "dwDataSize", "Result"]), offset=0)), ("pRecognizeCallerData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwRecognizeCallerDataSize", SimTypeInt(signed=False, label="UInt32")), ("pfnActionCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MAPPING_PROPERTY_BAG", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["pBag", "data", "dwDataSize", "Result"]), offset=0)), ("pActionCallerData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwActionCallerDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlag", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="MAPPING_OPTIONS", pack=False, align=None),
    "MAPPING_PROPERTY_BAG": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("prgResultRanges", SimTypePointer(SimTypeRef("MAPPING_DATA_RANGE", SimStruct), offset=0)), ("dwRangesCount", SimTypeInt(signed=False, label="UInt32")), ("pServiceData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwServiceDataSize", SimTypeInt(signed=False, label="UInt32")), ("pCallerData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCallerDataSize", SimTypeInt(signed=False, label="UInt32")), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MAPPING_PROPERTY_BAG", pack=False, align=None),
    "MAPPING_SERVICE_INFO": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pszCopyright", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("wMinorVersion", SimTypeShort(signed=False, label="UInt16")), ("wBuildVersion", SimTypeShort(signed=False, label="UInt16")), ("wStepVersion", SimTypeShort(signed=False, label="UInt16")), ("dwInputContentTypesCount", SimTypeInt(signed=False, label="UInt32")), ("prgInputContentTypes", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwOutputContentTypesCount", SimTypeInt(signed=False, label="UInt32")), ("prgOutputContentTypes", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwInputLanguagesCount", SimTypeInt(signed=False, label="UInt32")), ("prgInputLanguages", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwOutputLanguagesCount", SimTypeInt(signed=False, label="UInt32")), ("prgOutputLanguages", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwInputScriptsCount", SimTypeInt(signed=False, label="UInt32")), ("prgInputScripts", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwOutputScriptsCount", SimTypeInt(signed=False, label="UInt32")), ("prgOutputScripts", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("guid", SimTypeBottom(label="Guid")), ("pszCategory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("pPrivateData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="MAPPING_SERVICE_INFO", pack=False, align=None),
    "MAPTABLE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwGlyphNum", SimTypeInt(signed=False, label="UInt32")), ("Trans", SimTypePointer(SimTypeRef("TRANSDATA", SimStruct), offset=0)),)), name="MAPTABLE", pack=False, align=None),
    "MAP_FLAGS": SimTypeInt(signed=False, label="MAP_FLAGS"),
    "MAP_REGISTER_ENTRY": SimStruct(OrderedDict((("MapRegister", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("WriteToDevice", SimTypeChar(label="Byte")),)), name="MAP_REGISTER_ENTRY", pack=False, align=None),
    "MAP_VALUETYPE": SimTypeInt(signed=False, label="MAP_VALUETYPE"),
    "MAP_VIRTUAL_KEY_TYPE": SimTypeInt(signed=False, label="MAP_VIRTUAL_KEY_TYPE"),
    "MARGINS": SimStruct(OrderedDict((("cxLeftWidth", SimTypeInt(signed=True, label="Int32")), ("cxRightWidth", SimTypeInt(signed=True, label="Int32")), ("cyTopHeight", SimTypeInt(signed=True, label="Int32")), ("cyBottomHeight", SimTypeInt(signed=True, label="Int32")),)), name="MARGINS", pack=False, align=None),
    "MARKUPTEXTSTATES": SimTypeInt(signed=False, label="MARKUPTEXTSTATES"),
    "MARKUP_CONTEXT_TYPE": SimTypeInt(signed=False, label="MARKUP_CONTEXT_TYPE"),
    "MARK_HANDLE_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"UsnSourceInfo": SimTypeInt(signed=False, label="UInt32"), "CopyNumber": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("VolumeHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("HandleInfo", SimTypeInt(signed=False, label="UInt32")),)), name="MARK_HANDLE_INFO", pack=False, align=None),
    "MARK_HANDLE_INFO32": SimStruct(OrderedDict((("Anonymous", SimUnion({"UsnSourceInfo": SimTypeInt(signed=False, label="UInt32"), "CopyNumber": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("VolumeHandle", SimTypeInt(signed=False, label="UInt32")), ("HandleInfo", SimTypeInt(signed=False, label="UInt32")),)), name="MARK_HANDLE_INFO32", pack=False, align=None),
    "MAT2": SimStruct(OrderedDict((("eM11", SimTypeRef("FIXED", SimStruct)), ("eM12", SimTypeRef("FIXED", SimStruct)), ("eM21", SimTypeRef("FIXED", SimStruct)), ("eM22", SimTypeRef("FIXED", SimStruct)),)), name="MAT2", pack=False, align=None),
    "MATRIX3X3": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("A11", SimTypeFloat(size=32)), ("A12", SimTypeFloat(size=32)), ("A13", SimTypeFloat(size=32)), ("A21", SimTypeFloat(size=32)), ("A22", SimTypeFloat(size=32)), ("A23", SimTypeFloat(size=32)), ("A31", SimTypeFloat(size=32)), ("A32", SimTypeFloat(size=32)), ("A33", SimTypeFloat(size=32)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("V1", SimTypeRef("VEC3D", SimStruct)), ("V2", SimTypeRef("VEC3D", SimStruct)), ("V3", SimTypeRef("VEC3D", SimStruct)),)), name="_Anonymous2_e__Struct", pack=False, align=None), "M": SimTypeArray(SimTypeFloat(size=32), 9)}, name="<anon>", label="None")),)), name="MATRIX3X3", pack=False, align=None),
    "MAXBUTTONSTATES": SimTypeInt(signed=False, label="MAXBUTTONSTATES"),
    "MAXCAPTIONSTATES": SimTypeInt(signed=False, label="MAXCAPTIONSTATES"),
    "MAXMEDIALABEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pMaxSize"]), offset=0),
    "MAXVERSIONTESTED_INFO": SimStruct(OrderedDict((("MaxVersionTested", SimTypeLongLong(signed=False, label="UInt64")),)), name="MAXVERSIONTESTED_INFO", pack=False, align=None),
    "MBN_ACTIVATION_STATE": SimTypeInt(signed=False, label="MBN_ACTIVATION_STATE"),
    "MBN_AUTH_PROTOCOL": SimTypeInt(signed=False, label="MBN_AUTH_PROTOCOL"),
    "MBN_BAND_CLASS": SimTypeInt(signed=False, label="MBN_BAND_CLASS"),
    "MBN_CELLULAR_CLASS": SimTypeInt(signed=False, label="MBN_CELLULAR_CLASS"),
    "MBN_COMPRESSION": SimTypeInt(signed=False, label="MBN_COMPRESSION"),
    "MBN_CONNECTION_MODE": SimTypeInt(signed=False, label="MBN_CONNECTION_MODE"),
    "MBN_CONTEXT": SimStruct(OrderedDict((("contextID", SimTypeInt(signed=False, label="UInt32")), ("contextType", SimTypeInt(signed=False, label="MBN_CONTEXT_TYPE")), ("accessString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("userName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("compression", SimTypeInt(signed=False, label="MBN_COMPRESSION")), ("authType", SimTypeInt(signed=False, label="MBN_AUTH_PROTOCOL")),)), name="MBN_CONTEXT", pack=False, align=None),
    "MBN_CONTEXT_CONSTANTS": SimTypeInt(signed=False, label="MBN_CONTEXT_CONSTANTS"),
    "MBN_CONTEXT_TYPE": SimTypeInt(signed=False, label="MBN_CONTEXT_TYPE"),
    "MBN_CTRL_CAPS": SimTypeInt(signed=False, label="MBN_CTRL_CAPS"),
    "MBN_DATA_CLASS": SimTypeInt(signed=False, label="MBN_DATA_CLASS"),
    "MBN_DEVICE_SERVICE": SimStruct(OrderedDict((("deviceServiceID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dataWriteSupported", SimTypeShort(signed=True, label="Int16")), ("dataReadSupported", SimTypeShort(signed=True, label="Int16")),)), name="MBN_DEVICE_SERVICE", pack=False, align=None),
    "MBN_DEVICE_SERVICES_INTERFACE_STATE": SimTypeInt(signed=False, label="MBN_DEVICE_SERVICES_INTERFACE_STATE"),
    "MBN_DEVICE_SERVICE_SESSIONS_STATE": SimTypeInt(signed=False, label="MBN_DEVICE_SERVICE_SESSIONS_STATE"),
    "MBN_INTERFACE_CAPS": SimStruct(OrderedDict((("cellularClass", SimTypeInt(signed=False, label="MBN_CELLULAR_CLASS")), ("voiceClass", SimTypeInt(signed=False, label="MBN_VOICE_CLASS")), ("dataClass", SimTypeInt(signed=False, label="UInt32")), ("customDataClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("gsmBandClass", SimTypeInt(signed=False, label="UInt32")), ("cdmaBandClass", SimTypeInt(signed=False, label="UInt32")), ("customBandClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("smsCaps", SimTypeInt(signed=False, label="UInt32")), ("controlCaps", SimTypeInt(signed=False, label="UInt32")), ("deviceID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("manufacturer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("model", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("firmwareInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MBN_INTERFACE_CAPS", pack=False, align=None),
    "MBN_INTERFACE_CAPS_CONSTANTS": SimTypeInt(signed=False, label="MBN_INTERFACE_CAPS_CONSTANTS"),
    "MBN_MSG_STATUS": SimTypeInt(signed=False, label="MBN_MSG_STATUS"),
    "MBN_PIN_CONSTANTS": SimTypeInt(signed=False, label="MBN_PIN_CONSTANTS"),
    "MBN_PIN_FORMAT": SimTypeInt(signed=False, label="MBN_PIN_FORMAT"),
    "MBN_PIN_INFO": SimStruct(OrderedDict((("pinState", SimTypeInt(signed=False, label="MBN_PIN_STATE")), ("pinType", SimTypeInt(signed=False, label="MBN_PIN_TYPE")), ("attemptsRemaining", SimTypeInt(signed=False, label="UInt32")),)), name="MBN_PIN_INFO", pack=False, align=None),
    "MBN_PIN_MODE": SimTypeInt(signed=False, label="MBN_PIN_MODE"),
    "MBN_PIN_STATE": SimTypeInt(signed=False, label="MBN_PIN_STATE"),
    "MBN_PIN_TYPE": SimTypeInt(signed=False, label="MBN_PIN_TYPE"),
    "MBN_PROVIDER": SimStruct(OrderedDict((("providerID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("providerState", SimTypeInt(signed=False, label="UInt32")), ("providerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dataClass", SimTypeInt(signed=False, label="UInt32")),)), name="MBN_PROVIDER", pack=False, align=None),
    "MBN_PROVIDER2": SimStruct(OrderedDict((("provider", SimTypeRef("MBN_PROVIDER", SimStruct)), ("cellularClass", SimTypeInt(signed=False, label="MBN_CELLULAR_CLASS")), ("signalStrength", SimTypeInt(signed=False, label="UInt32")), ("signalError", SimTypeInt(signed=False, label="UInt32")),)), name="MBN_PROVIDER2", pack=False, align=None),
    "MBN_PROVIDER_CONSTANTS": SimTypeInt(signed=False, label="MBN_PROVIDER_CONSTANTS"),
    "MBN_PROVIDER_STATE": SimTypeInt(signed=False, label="MBN_PROVIDER_STATE"),
    "MBN_RADIO": SimTypeInt(signed=False, label="MBN_RADIO"),
    "MBN_READY_STATE": SimTypeInt(signed=False, label="MBN_READY_STATE"),
    "MBN_REGISTER_MODE": SimTypeInt(signed=False, label="MBN_REGISTER_MODE"),
    "MBN_REGISTER_STATE": SimTypeInt(signed=False, label="MBN_REGISTER_STATE"),
    "MBN_REGISTRATION_CONSTANTS": SimTypeInt(signed=False, label="MBN_REGISTRATION_CONSTANTS"),
    "MBN_SIGNAL_CONSTANTS": SimTypeInt(signed=False, label="MBN_SIGNAL_CONSTANTS"),
    "MBN_SMS_CAPS": SimTypeInt(signed=False, label="MBN_SMS_CAPS"),
    "MBN_SMS_CDMA_ENCODING": SimTypeInt(signed=False, label="MBN_SMS_CDMA_ENCODING"),
    "MBN_SMS_CDMA_LANG": SimTypeInt(signed=False, label="MBN_SMS_CDMA_LANG"),
    "MBN_SMS_FILTER": SimStruct(OrderedDict((("flag", SimTypeInt(signed=False, label="MBN_SMS_FLAG")), ("messageIndex", SimTypeInt(signed=False, label="UInt32")),)), name="MBN_SMS_FILTER", pack=False, align=None),
    "MBN_SMS_FLAG": SimTypeInt(signed=False, label="MBN_SMS_FLAG"),
    "MBN_SMS_FORMAT": SimTypeInt(signed=False, label="MBN_SMS_FORMAT"),
    "MBN_SMS_STATUS_FLAG": SimTypeInt(signed=False, label="MBN_SMS_STATUS_FLAG"),
    "MBN_SMS_STATUS_INFO": SimStruct(OrderedDict((("flag", SimTypeInt(signed=False, label="UInt32")), ("messageIndex", SimTypeInt(signed=False, label="UInt32")),)), name="MBN_SMS_STATUS_INFO", pack=False, align=None),
    "MBN_VOICE_CALL_STATE": SimTypeInt(signed=False, label="MBN_VOICE_CALL_STATE"),
    "MBN_VOICE_CLASS": SimTypeInt(signed=False, label="MBN_VOICE_CLASS"),
    "MCAST_CLIENT_UID": SimStruct(OrderedDict((("ClientUID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClientUIDLength", SimTypeInt(signed=False, label="UInt32")),)), name="MCAST_CLIENT_UID", pack=False, align=None),
    "MCAST_LEASE_REQUEST": SimStruct(OrderedDict((("LeaseStartTime", SimTypeInt(signed=True, label="Int32")), ("MaxLeaseStartTime", SimTypeInt(signed=True, label="Int32")), ("LeaseDuration", SimTypeInt(signed=False, label="UInt32")), ("MinLeaseDuration", SimTypeInt(signed=False, label="UInt32")), ("ServerAddress", SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("MinAddrCount", SimTypeShort(signed=False, label="UInt16")), ("AddrCount", SimTypeShort(signed=False, label="UInt16")), ("pAddrBuf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCAST_LEASE_REQUEST", pack=False, align=None),
    "MCAST_LEASE_RESPONSE": SimStruct(OrderedDict((("LeaseStartTime", SimTypeInt(signed=True, label="Int32")), ("LeaseEndTime", SimTypeInt(signed=True, label="Int32")), ("ServerAddress", SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("AddrCount", SimTypeShort(signed=False, label="UInt16")), ("pAddrBuf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCAST_LEASE_RESPONSE", pack=False, align=None),
    "MCAST_SCOPE_CTX": SimStruct(OrderedDict((("ScopeID", SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Interface", SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("ServerID", SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")),)), name="MCAST_SCOPE_CTX", pack=False, align=None),
    "MCAST_SCOPE_ENTRY": SimStruct(OrderedDict((("ScopeCtx", SimTypeRef("MCAST_SCOPE_CTX", SimStruct)), ("LastAddr", SimUnion({"IpAddrV4": SimTypeInt(signed=False, label="UInt32"), "IpAddrV6": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("TTL", SimTypeInt(signed=False, label="UInt32")), ("ScopeDesc", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="MCAST_SCOPE_ENTRY", pack=False, align=None),
    "MCA_DRIVER_INFO": SimStruct(OrderedDict((("ExceptionCallback", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DpcCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MCA_DRIVER_INFO", pack=False, align=None),
    "MCA_EXCEPTION": SimStruct(OrderedDict((("VersionNumber", SimTypeInt(signed=False, label="UInt32")), ("ExceptionType", SimTypeInt(signed=False, label="MCA_EXCEPTION_TYPE")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("ProcessorNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"Mca": SimTypeRef("_Mca_e__Struct", SimStruct), "Mce": SimTypeRef("_Mce_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ExtCnt", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("ExtReg", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 24)),)), name="MCA_EXCEPTION", pack=False, align=None),
    "MCA_EXCEPTION_TYPE": SimTypeInt(signed=False, label="MCA_EXCEPTION_TYPE"),
    "MCB": SimStruct(OrderedDict((("DummyFieldThatSizesThisStructureCorrectly", SimTypeRef("LARGE_MCB", SimStruct)),)), name="MCB", pack=False, align=None),
    "MCGRIDINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwPart", SimTypeInt(signed=False, label="MCGRIDINFO_PART")), ("dwFlags", SimTypeInt(signed=False, label="MCGRIDINFO_FLAGS")), ("iCalendar", SimTypeInt(signed=True, label="Int32")), ("iRow", SimTypeInt(signed=True, label="Int32")), ("iCol", SimTypeInt(signed=True, label="Int32")), ("bSelected", SimTypeInt(signed=True, label="Int32")), ("stStart", SimTypeRef("SYSTEMTIME", SimStruct)), ("stEnd", SimTypeRef("SYSTEMTIME", SimStruct)), ("rc", SimTypeRef("RECT", SimStruct)), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchName", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MCGRIDINFO", pack=False, align=None),
    "MCGRIDINFO_FLAGS": SimTypeInt(signed=False, label="MCGRIDINFO_FLAGS"),
    "MCGRIDINFO_PART": SimTypeInt(signed=False, label="MCGRIDINFO_PART"),
    "MCG_CAP": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "MCG_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "MCHITTESTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)), ("uHit", SimTypeInt(signed=False, label="MCHITTESTINFO_HIT_FLAGS")), ("st", SimTypeRef("SYSTEMTIME", SimStruct)), ("rc", SimTypeRef("RECT", SimStruct)), ("iOffset", SimTypeInt(signed=True, label="Int32")), ("iRow", SimTypeInt(signed=True, label="Int32")), ("iCol", SimTypeInt(signed=True, label="Int32")),)), name="MCHITTESTINFO", pack=False, align=None),
    "MCHITTESTINFO_HIT_FLAGS": SimTypeInt(signed=False, label="MCHITTESTINFO_HIT_FLAGS"),
    "MCI_ADDR": SimUnion({"Anonymous": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "MCI_ANIM_OPEN_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_ANIM_OPEN_PARMSA", pack=False, align=None),
    "MCI_ANIM_OPEN_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_ANIM_OPEN_PARMSW", pack=False, align=None),
    "MCI_ANIM_PLAY_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("dwSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_ANIM_PLAY_PARMS", pack=False, align=None),
    "MCI_ANIM_RECT_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_ANIM_RECT_PARMS", pack=False, align=None),
    "MCI_ANIM_STEP_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_ANIM_STEP_PARMS", pack=False, align=None),
    "MCI_ANIM_UPDATE_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_ANIM_UPDATE_PARMS", pack=False, align=None),
    "MCI_ANIM_WINDOW_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nCmdShow", SimTypeInt(signed=False, label="UInt32")), ("lpstrText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_ANIM_WINDOW_PARMSA", pack=False, align=None),
    "MCI_ANIM_WINDOW_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nCmdShow", SimTypeInt(signed=False, label="UInt32")), ("lpstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_ANIM_WINDOW_PARMSW", pack=False, align=None),
    "MCI_BREAK_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("nVirtKey", SimTypeInt(signed=True, label="Int32")), ("hwndBreak", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_BREAK_PARMS", pack=False, align=None),
    "MCI_DGV_CAPTURE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_CAPTURE_PARMSA", pack=False, align=None),
    "MCI_DGV_CAPTURE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_CAPTURE_PARMSW", pack=False, align=None),
    "MCI_DGV_COPY_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("rc", SimTypeRef("RECT", SimStruct)), ("dwAudioStream", SimTypeInt(signed=False, label="UInt32")), ("dwVideoStream", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_COPY_PARMS", pack=False, align=None),
    "MCI_DGV_CUE_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTo", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_CUE_PARMS", pack=False, align=None),
    "MCI_DGV_CUT_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("rc", SimTypeRef("RECT", SimStruct)), ("dwAudioStream", SimTypeInt(signed=False, label="UInt32")), ("dwVideoStream", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_CUT_PARMS", pack=False, align=None),
    "MCI_DGV_DELETE_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("rc", SimTypeRef("RECT", SimStruct)), ("dwAudioStream", SimTypeInt(signed=False, label="UInt32")), ("dwVideoStream", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_DELETE_PARMS", pack=False, align=None),
    "MCI_DGV_INFO_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwRetSize", SimTypeInt(signed=False, label="UInt32")), ("dwItem", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_INFO_PARMSA", pack=False, align=None),
    "MCI_DGV_INFO_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRetSize", SimTypeInt(signed=False, label="UInt32")), ("dwItem", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_INFO_PARMSW", pack=False, align=None),
    "MCI_DGV_LIST_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwNumber", SimTypeInt(signed=False, label="UInt32")), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("lpstrAlgorithm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_DGV_LIST_PARMSA", pack=False, align=None),
    "MCI_DGV_LIST_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwNumber", SimTypeInt(signed=False, label="UInt32")), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("lpstrAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_DGV_LIST_PARMSW", pack=False, align=None),
    "MCI_DGV_MONITOR_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwSource", SimTypeInt(signed=False, label="UInt32")), ("dwMethod", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_MONITOR_PARMS", pack=False, align=None),
    "MCI_DGV_OPEN_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_DGV_OPEN_PARMSA", pack=False, align=None),
    "MCI_DGV_OPEN_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_DGV_OPEN_PARMSW", pack=False, align=None),
    "MCI_DGV_PASTE_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("rc", SimTypeRef("RECT", SimStruct)), ("dwAudioStream", SimTypeInt(signed=False, label="UInt32")), ("dwVideoStream", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_PASTE_PARMS", pack=False, align=None),
    "MCI_DGV_QUALITY_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("lpstrName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_QUALITY_PARMSA", pack=False, align=None),
    "MCI_DGV_QUALITY_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("lpstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwHandle", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_QUALITY_PARMSW", pack=False, align=None),
    "MCI_DGV_RECORD_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("rc", SimTypeRef("RECT", SimStruct)), ("dwAudioStream", SimTypeInt(signed=False, label="UInt32")), ("dwVideoStream", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_RECORD_PARMS", pack=False, align=None),
    "MCI_DGV_RECT_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_RECT_PARMS", pack=False, align=None),
    "MCI_DGV_RESERVE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_RESERVE_PARMSA", pack=False, align=None),
    "MCI_DGV_RESERVE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_RESERVE_PARMSW", pack=False, align=None),
    "MCI_DGV_RESTORE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_RESTORE_PARMSA", pack=False, align=None),
    "MCI_DGV_RESTORE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_RESTORE_PARMSW", pack=False, align=None),
    "MCI_DGV_SAVE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_SAVE_PARMSA", pack=False, align=None),
    "MCI_DGV_SAVE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_DGV_SAVE_PARMSW", pack=False, align=None),
    "MCI_DGV_SETAUDIO_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwValue", SimTypeInt(signed=False, label="UInt32")), ("dwOver", SimTypeInt(signed=False, label="UInt32")), ("lpstrAlgorithm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrQuality", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_DGV_SETAUDIO_PARMSA", pack=False, align=None),
    "MCI_DGV_SETAUDIO_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwValue", SimTypeInt(signed=False, label="UInt32")), ("dwOver", SimTypeInt(signed=False, label="UInt32")), ("lpstrAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrQuality", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_DGV_SETAUDIO_PARMSW", pack=False, align=None),
    "MCI_DGV_SETVIDEO_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwValue", SimTypeInt(signed=False, label="UInt32")), ("dwOver", SimTypeInt(signed=False, label="UInt32")), ("lpstrAlgorithm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrQuality", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSourceNumber", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_SETVIDEO_PARMSA", pack=False, align=None),
    "MCI_DGV_SETVIDEO_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwValue", SimTypeInt(signed=False, label="UInt32")), ("dwOver", SimTypeInt(signed=False, label="UInt32")), ("lpstrAlgorithm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrQuality", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSourceNumber", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_SETVIDEO_PARMSW", pack=False, align=None),
    "MCI_DGV_SET_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTimeFormat", SimTypeInt(signed=False, label="UInt32")), ("dwAudio", SimTypeInt(signed=False, label="UInt32")), ("dwFileFormat", SimTypeInt(signed=False, label="UInt32")), ("dwSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_SET_PARMS", pack=False, align=None),
    "MCI_DGV_SIGNAL_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwPosition", SimTypeInt(signed=False, label="UInt32")), ("dwPeriod", SimTypeInt(signed=False, label="UInt32")), ("dwUserParm", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_SIGNAL_PARMS", pack=False, align=None),
    "MCI_DGV_STATUS_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReturn", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwTrack", SimTypeInt(signed=False, label="UInt32")), ("lpstrDrive", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwReference", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_STATUS_PARMSA", pack=False, align=None),
    "MCI_DGV_STATUS_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReturn", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwTrack", SimTypeInt(signed=False, label="UInt32")), ("lpstrDrive", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwReference", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_STATUS_PARMSW", pack=False, align=None),
    "MCI_DGV_STEP_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_DGV_STEP_PARMS", pack=False, align=None),
    "MCI_DGV_UPDATE_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_DGV_UPDATE_PARMS", pack=False, align=None),
    "MCI_DGV_WINDOW_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nCmdShow", SimTypeInt(signed=False, label="UInt32")), ("lpstrText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_DGV_WINDOW_PARMSA", pack=False, align=None),
    "MCI_DGV_WINDOW_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nCmdShow", SimTypeInt(signed=False, label="UInt32")), ("lpstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_DGV_WINDOW_PARMSW", pack=False, align=None),
    "MCI_GENERIC_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MCI_GENERIC_PARMS", pack=False, align=None),
    "MCI_GETDEVCAPS_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReturn", SimTypeInt(signed=False, label="UInt32")), ("dwItem", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_GETDEVCAPS_PARMS", pack=False, align=None),
    "MCI_INFO_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwRetSize", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_INFO_PARMSA", pack=False, align=None),
    "MCI_INFO_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRetSize", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_INFO_PARMSW", pack=False, align=None),
    "MCI_LOAD_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_LOAD_PARMSA", pack=False, align=None),
    "MCI_LOAD_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_LOAD_PARMSW", pack=False, align=None),
    "MCI_OPEN_DRIVER_PARMS": SimStruct(OrderedDict((("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrParams", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wCustomCommandTable", SimTypeInt(signed=False, label="UInt32")), ("wType", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_OPEN_DRIVER_PARMS", pack=False, align=None),
    "MCI_OPEN_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_OPEN_PARMSA", pack=False, align=None),
    "MCI_OPEN_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_OPEN_PARMSW", pack=False, align=None),
    "MCI_OVLY_LOAD_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_OVLY_LOAD_PARMSA", pack=False, align=None),
    "MCI_OVLY_LOAD_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_OVLY_LOAD_PARMSW", pack=False, align=None),
    "MCI_OVLY_OPEN_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_OVLY_OPEN_PARMSA", pack=False, align=None),
    "MCI_OVLY_OPEN_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MCI_OVLY_OPEN_PARMSW", pack=False, align=None),
    "MCI_OVLY_RECT_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_OVLY_RECT_PARMS", pack=False, align=None),
    "MCI_OVLY_SAVE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_OVLY_SAVE_PARMSA", pack=False, align=None),
    "MCI_OVLY_SAVE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="MCI_OVLY_SAVE_PARMSW", pack=False, align=None),
    "MCI_OVLY_WINDOW_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nCmdShow", SimTypeInt(signed=False, label="UInt32")), ("lpstrText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_OVLY_WINDOW_PARMSA", pack=False, align=None),
    "MCI_OVLY_WINDOW_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nCmdShow", SimTypeInt(signed=False, label="UInt32")), ("lpstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_OVLY_WINDOW_PARMSW", pack=False, align=None),
    "MCI_PLAY_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_PLAY_PARMS", pack=False, align=None),
    "MCI_RECORD_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_RECORD_PARMS", pack=False, align=None),
    "MCI_SAVE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_SAVE_PARMSA", pack=False, align=None),
    "MCI_SAVE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_SAVE_PARMSW", pack=False, align=None),
    "MCI_SEEK_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTo", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_SEEK_PARMS", pack=False, align=None),
    "MCI_SEQ_SET_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTimeFormat", SimTypeInt(signed=False, label="UInt32")), ("dwAudio", SimTypeInt(signed=False, label="UInt32")), ("dwTempo", SimTypeInt(signed=False, label="UInt32")), ("dwPort", SimTypeInt(signed=False, label="UInt32")), ("dwSlave", SimTypeInt(signed=False, label="UInt32")), ("dwMaster", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_SEQ_SET_PARMS", pack=False, align=None),
    "MCI_SET_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTimeFormat", SimTypeInt(signed=False, label="UInt32")), ("dwAudio", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_SET_PARMS", pack=False, align=None),
    "MCI_STATS": SimUnion({"MciStats": SimTypeRef("_MciStats_e__Struct", SimStruct), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "MCI_STATUS": SimUnion({"CommonBits": SimTypeRef("MCI_STATUS_BITS_COMMON", SimStruct), "AmdBits": SimTypeRef("MCI_STATUS_AMD_BITS", SimStruct), "IntelBits": SimTypeRef("MCI_STATUS_INTEL_BITS", SimStruct), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "MCI_STATUS_AMD_BITS": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="MCI_STATUS_AMD_BITS", pack=False, align=None),
    "MCI_STATUS_BITS_COMMON": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="MCI_STATUS_BITS_COMMON", pack=False, align=None),
    "MCI_STATUS_INTEL_BITS": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="MCI_STATUS_INTEL_BITS", pack=False, align=None),
    "MCI_STATUS_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwReturn", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItem", SimTypeInt(signed=False, label="UInt32")), ("dwTrack", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_STATUS_PARMS", pack=False, align=None),
    "MCI_SYSINFO_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwRetSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumber", SimTypeInt(signed=False, label="UInt32")), ("wDeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_SYSINFO_PARMSA", pack=False, align=None),
    "MCI_SYSINFO_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrReturn", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRetSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumber", SimTypeInt(signed=False, label="UInt32")), ("wDeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_SYSINFO_PARMSW", pack=False, align=None),
    "MCI_VD_ESCAPE_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrCommand", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MCI_VD_ESCAPE_PARMSA", pack=False, align=None),
    "MCI_VD_ESCAPE_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrCommand", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MCI_VD_ESCAPE_PARMSW", pack=False, align=None),
    "MCI_VD_PLAY_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")), ("dwSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_VD_PLAY_PARMS", pack=False, align=None),
    "MCI_VD_STEP_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_VD_STEP_PARMS", pack=False, align=None),
    "MCI_WAVE_DELETE_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFrom", SimTypeInt(signed=False, label="UInt32")), ("dwTo", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_WAVE_DELETE_PARMS", pack=False, align=None),
    "MCI_WAVE_OPEN_PARMSA": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwBufferSeconds", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_WAVE_OPEN_PARMSA", pack=False, align=None),
    "MCI_WAVE_OPEN_PARMSW": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wDeviceID", SimTypeInt(signed=False, label="UInt32")), ("lpstrDeviceType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrElementName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrAlias", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwBufferSeconds", SimTypeInt(signed=False, label="UInt32")),)), name="MCI_WAVE_OPEN_PARMSW", pack=False, align=None),
    "MCI_WAVE_SET_PARMS": SimStruct(OrderedDict((("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTimeFormat", SimTypeInt(signed=False, label="UInt32")), ("dwAudio", SimTypeInt(signed=False, label="UInt32")), ("wInput", SimTypeInt(signed=False, label="UInt32")), ("wOutput", SimTypeInt(signed=False, label="UInt32")), ("wFormatTag", SimTypeShort(signed=False, label="UInt16")), ("wReserved2", SimTypeShort(signed=False, label="UInt16")), ("nChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved3", SimTypeShort(signed=False, label="UInt16")), ("nSamplesPerSec", SimTypeInt(signed=False, label="UInt32")), ("nAvgBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("nBlockAlign", SimTypeShort(signed=False, label="UInt16")), ("wReserved4", SimTypeShort(signed=False, label="UInt16")), ("wBitsPerSample", SimTypeShort(signed=False, label="UInt16")), ("wReserved5", SimTypeShort(signed=False, label="UInt16")),)), name="MCI_WAVE_SET_PARMS", pack=False, align=None),
    "MC_COLOR_TEMPERATURE": SimTypeInt(signed=False, label="MC_COLOR_TEMPERATURE"),
    "MC_DISPLAY_TECHNOLOGY_TYPE": SimTypeInt(signed=False, label="MC_DISPLAY_TECHNOLOGY_TYPE"),
    "MC_DRIVE_TYPE": SimTypeInt(signed=False, label="MC_DRIVE_TYPE"),
    "MC_GAIN_TYPE": SimTypeInt(signed=False, label="MC_GAIN_TYPE"),
    "MC_POSITION_TYPE": SimTypeInt(signed=False, label="MC_POSITION_TYPE"),
    "MC_SIZE_TYPE": SimTypeInt(signed=False, label="MC_SIZE_TYPE"),
    "MC_TIMING_REPORT": SimStruct(OrderedDict((("dwHorizontalFrequencyInHZ", SimTypeInt(signed=False, label="UInt32")), ("dwVerticalFrequencyInHZ", SimTypeInt(signed=False, label="UInt32")), ("bTimingStatusByte", SimTypeChar(label="Byte")),)), name="MC_TIMING_REPORT", pack=False, align=None),
    "MC_VCP_CODE_TYPE": SimTypeInt(signed=False, label="MC_VCP_CODE_TYPE"),
    "MDAInfo": SimStruct(OrderedDict((("lpMDACaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpMDAMessage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpStackTrace", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MDAInfo", pack=False, align=None),
    "MDAXISINFO": SimStruct(OrderedDict((("cbSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iAxis", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cDimensions", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cCoordinates", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rgcColumns", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)), ("rgpwszDimensionNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="MDAXISINFO", pack=False, align=None),
    "MDEVICECAPSEX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pCaps", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MDEVICECAPSEX", pack=False, align=None),
    "MDICLOSEBUTTONSTATES": SimTypeInt(signed=False, label="MDICLOSEBUTTONSTATES"),
    "MDICREATESTRUCTA": SimStruct(OrderedDict((("szClass", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")), ("style", SimTypeInt(signed=False, label="WINDOW_STYLE")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MDICREATESTRUCTA", pack=False, align=None),
    "MDICREATESTRUCTW": SimStruct(OrderedDict((("szClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")), ("style", SimTypeInt(signed=False, label="WINDOW_STYLE")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MDICREATESTRUCTW", pack=False, align=None),
    "MDIMINBUTTONSTATES": SimTypeInt(signed=False, label="MDIMINBUTTONSTATES"),
    "MDINEXTMENU": SimStruct(OrderedDict((("hmenuIn", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hmenuNext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndNext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MDINEXTMENU", pack=False, align=None),
    "MDIRESTOREBUTTONSTATES": SimTypeInt(signed=False, label="MDIRESTOREBUTTONSTATES"),
    "MDL": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)), ("Size", SimTypeShort(signed=True, label="Int16")), ("MdlFlags", SimTypeShort(signed=True, label="Int16")), ("Process", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("MappedSystemVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StartVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ByteCount", SimTypeInt(signed=False, label="UInt32")), ("ByteOffset", SimTypeInt(signed=False, label="UInt32")),)), name="MDL", pack=False, align=None),
    "MDNS_QUERY_HANDLE": SimStruct(OrderedDict((("nameBuf", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("pSubscription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pWnfCallbackParams", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("stateNameData", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="MDNS_QUERY_HANDLE", pack=False, align=None),
    "MDNS_QUERY_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ulRefCount", SimTypeInt(signed=False, label="UInt32")), ("Query", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("QueryType", SimTypeShort(signed=False, label="UInt16")), ("QueryOptions", SimTypeLongLong(signed=False, label="UInt64")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pQueryCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MDNS_QUERY_HANDLE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pQueryContext", "pQueryHandle", "pQueryResults"]), offset=0)), ("pQueryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("fAnswerReceived", SimTypeInt(signed=True, label="Int32")), ("ulResendCount", SimTypeInt(signed=False, label="UInt32")),)), name="MDNS_QUERY_REQUEST", pack=False, align=None),
    "MD_CHANGE_OBJECT_W": SimStruct(OrderedDict((("pszMDPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMDChangeType", SimTypeInt(signed=False, label="UInt32")), ("dwMDNumDataIDs", SimTypeInt(signed=False, label="UInt32")), ("pdwMDDataIDs", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="MD_CHANGE_OBJECT_W", pack=False, align=None),
    "MD_FLAGS": SimTypeInt(signed=False, label="MD_FLAGS"),
    "MEASURED_LATENCY": SimStruct(OrderedDict((("Trim3", SimTypeShort(signed=False, label="UInt16")), ("Write3", SimTypeShort(signed=False, label="UInt16")), ("Read3", SimTypeShort(signed=False, label="UInt16")), ("Trim2", SimTypeShort(signed=False, label="UInt16")), ("Write2", SimTypeShort(signed=False, label="UInt16")), ("Read2", SimTypeShort(signed=False, label="UInt16")), ("Trim1", SimTypeShort(signed=False, label="UInt16")), ("Write1", SimTypeShort(signed=False, label="UInt16")), ("Read1", SimTypeShort(signed=False, label="UInt16")), ("Trim0", SimTypeShort(signed=False, label="UInt16")), ("Write0", SimTypeShort(signed=False, label="UInt16")), ("Read0", SimTypeShort(signed=False, label="UInt16")),)), name="MEASURED_LATENCY", pack=False, align=None),
    "MEASUREITEMSTRUCT": SimStruct(OrderedDict((("CtlType", SimTypeInt(signed=False, label="DRAWITEMSTRUCT_CTL_TYPE")), ("CtlID", SimTypeInt(signed=False, label="UInt32")), ("itemID", SimTypeInt(signed=False, label="UInt32")), ("itemWidth", SimTypeInt(signed=False, label="UInt32")), ("itemHeight", SimTypeInt(signed=False, label="UInt32")), ("itemData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MEASUREITEMSTRUCT", pack=False, align=None),
    "MEDIAPLAYBACK_STATE": SimTypeInt(signed=False, label="MEDIAPLAYBACK_STATE"),
    "MEDIASPACEADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="MEDIASPACEADPCMWAVEFORMAT", pack=False, align=None),
    "MEDIA_ACTIVITY_NOTIFY_TYPE": SimTypeInt(signed=False, label="MEDIA_ACTIVITY_NOTIFY_TYPE"),
    "MEDIA_EVENT_GENERATOR_GET_EVENT_FLAGS": SimTypeInt(signed=False, label="MEDIA_EVENT_GENERATOR_GET_EVENT_FLAGS"),
    "MEDIA_FLAGS": SimTypeInt(signed=False, label="MEDIA_FLAGS"),
    "MEDIA_SAMPLE_CONTENT": SimTypeInt(signed=False, label="MEDIA_SAMPLE_CONTENT"),
    "MEDIA_SPECIFIC_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("ClassId", SimTypeInt(signed=False, label="NDIS_CLASS_ID")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ClassInformation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MEDIA_SPECIFIC_INFORMATION", pack=False, align=None),
    "MEDIA_TYPE": SimTypeInt(signed=False, label="MEDIA_TYPE"),
    "MEDIA_TYPES": SimTypeInt(signed=False, label="MEDIA_TYPES"),
    "MEDIUM_INFO": SimStruct(OrderedDict((("MediaPresent", SimTypeInt(signed=True, label="Int32")), ("MediaType", SimTypeInt(signed=False, label="UInt32")), ("RecordInhibit", SimTypeInt(signed=True, label="Int32")),)), name="MEDIUM_INFO", pack=False, align=None),
    "MEMCTX": SimTypeInt(signed=False, label="MEMCTX"),
    "MEMORYSTATUS": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwMemoryLoad", SimTypeInt(signed=False, label="UInt32")), ("dwTotalPhys", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwAvailPhys", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTotalPageFile", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwAvailPageFile", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTotalVirtual", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwAvailVirtual", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MEMORYSTATUS", pack=False, align=None),
    "MEMORYSTATUSEX": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwMemoryLoad", SimTypeInt(signed=False, label="UInt32")), ("ullTotalPhys", SimTypeLongLong(signed=False, label="UInt64")), ("ullAvailPhys", SimTypeLongLong(signed=False, label="UInt64")), ("ullTotalPageFile", SimTypeLongLong(signed=False, label="UInt64")), ("ullAvailPageFile", SimTypeLongLong(signed=False, label="UInt64")), ("ullTotalVirtual", SimTypeLongLong(signed=False, label="UInt64")), ("ullAvailVirtual", SimTypeLongLong(signed=False, label="UInt64")), ("ullAvailExtendedVirtual", SimTypeLongLong(signed=False, label="UInt64")),)), name="MEMORYSTATUSEX", pack=False, align=None),
    "MEMORY_BASIC_INFORMATION": SimStruct(OrderedDict((("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationProtect", SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS")), ("RegionSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("State", SimTypeInt(signed=False, label="VIRTUAL_ALLOCATION_TYPE")), ("Protect", SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS")), ("Type", SimTypeInt(signed=False, label="PAGE_TYPE")),)), name="MEMORY_BASIC_INFORMATION", pack=False, align=None),
    "MEMORY_BASIC_INFORMATION32": SimStruct(OrderedDict((("BaseAddress", SimTypeInt(signed=False, label="UInt32")), ("AllocationBase", SimTypeInt(signed=False, label="UInt32")), ("AllocationProtect", SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS")), ("RegionSize", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="VIRTUAL_ALLOCATION_TYPE")), ("Protect", SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS")), ("Type", SimTypeInt(signed=False, label="PAGE_TYPE")),)), name="MEMORY_BASIC_INFORMATION32", pack=False, align=None),
    "MEMORY_BASIC_INFORMATION64": SimStruct(OrderedDict((("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationBase", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationProtect", SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS")), ("__alignment1", SimTypeInt(signed=False, label="UInt32")), ("RegionSize", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypeInt(signed=False, label="VIRTUAL_ALLOCATION_TYPE")), ("Protect", SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS")), ("Type", SimTypeInt(signed=False, label="PAGE_TYPE")), ("__alignment2", SimTypeInt(signed=False, label="UInt32")),)), name="MEMORY_BASIC_INFORMATION64", pack=False, align=None),
    "MEMORY_CACHING_TYPE": SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"),
    "MEMORY_CACHING_TYPE_ORIG": SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE_ORIG"),
    "MEMORY_INFORMATION_CLASS": SimTypeInt(signed=False, label="MEMORY_INFORMATION_CLASS"),
    "MEMORY_MAPPED_VIEW_ADDRESS": SimStruct(OrderedDict((("Value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MEMORY_MAPPED_VIEW_ADDRESS", pack=False, align=None),
    "MEMORY_PARTITION_DEDICATED_MEMORY_ATTRIBUTE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="MEM_DEDICATED_ATTRIBUTE_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="MEMORY_PARTITION_DEDICATED_MEMORY_ATTRIBUTE", pack=False, align=None),
    "MEMORY_PARTITION_DEDICATED_MEMORY_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("SizeOfInformation", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AttributesOffset", SimTypeInt(signed=False, label="UInt32")), ("AttributeCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("TypeId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MEMORY_PARTITION_DEDICATED_MEMORY_INFORMATION", pack=False, align=None),
    "MEMORY_PARTITION_DEDICATED_MEMORY_OPEN_INFORMATION": SimStruct(OrderedDict((("DedicatedMemoryTypeId", SimTypeLongLong(signed=False, label="UInt64")), ("HandleAttributes", SimTypeInt(signed=False, label="UInt32")), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("DedicatedMemoryPartitionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MEMORY_PARTITION_DEDICATED_MEMORY_OPEN_INFORMATION", pack=False, align=None),
    "MEMORY_PRIORITY": SimTypeInt(signed=False, label="MEMORY_PRIORITY"),
    "MEMORY_PRIORITY_INFORMATION": SimStruct(OrderedDict((("MemoryPriority", SimTypeInt(signed=False, label="MEMORY_PRIORITY")),)), name="MEMORY_PRIORITY_INFORMATION", pack=False, align=None),
    "MEMORY_RANGE_ENTRY": SimStruct(OrderedDict((("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MEMORY_RANGE_ENTRY", pack=False, align=None),
    "MEMORY_RESOURCE_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="MEMORY_RESOURCE_NOTIFICATION_TYPE"),
    "MEM_ADDRESS_REQUIREMENTS": SimStruct(OrderedDict((("LowestStartingAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HighestEndingAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Alignment", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MEM_ADDRESS_REQUIREMENTS", pack=False, align=None),
    "MEM_DEDICATED_ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="MEM_DEDICATED_ATTRIBUTE_TYPE"),
    "MEM_DES": SimStruct(OrderedDict((("MD_Count", SimTypeInt(signed=False, label="UInt32")), ("MD_Type", SimTypeInt(signed=False, label="UInt32")), ("MD_Alloc_Base", SimTypeLongLong(signed=False, label="UInt64")), ("MD_Alloc_End", SimTypeLongLong(signed=False, label="UInt64")), ("MD_Flags", SimTypeInt(signed=False, label="MD_FLAGS")), ("MD_Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="MEM_DES", pack=False, align=None),
    "MEM_EXTENDED_PARAMETER": SimStruct(OrderedDict((("Anonymous1", SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous1_e__Struct", pack=False, align=None)), ("Anonymous2", SimUnion({"ULong64": SimTypeLongLong(signed=False, label="UInt64"), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Size": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Handle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "ULong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="MEM_EXTENDED_PARAMETER", pack=False, align=None),
    "MEM_EXTENDED_PARAMETER_TYPE": SimTypeInt(signed=False, label="MEM_EXTENDED_PARAMETER_TYPE"),
    "MEM_LARGE_DES": SimStruct(OrderedDict((("MLD_Count", SimTypeInt(signed=False, label="UInt32")), ("MLD_Type", SimTypeInt(signed=False, label="UInt32")), ("MLD_Alloc_Base", SimTypeLongLong(signed=False, label="UInt64")), ("MLD_Alloc_End", SimTypeLongLong(signed=False, label="UInt64")), ("MLD_Flags", SimTypeInt(signed=False, label="UInt32")), ("MLD_Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="MEM_LARGE_DES", pack=False, align=None),
    "MEM_LARGE_RANGE": SimStruct(OrderedDict((("MLR_Align", SimTypeLongLong(signed=False, label="UInt64")), ("MLR_nBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MLR_Min", SimTypeLongLong(signed=False, label="UInt64")), ("MLR_Max", SimTypeLongLong(signed=False, label="UInt64")), ("MLR_Flags", SimTypeInt(signed=False, label="UInt32")), ("MLR_Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="MEM_LARGE_RANGE", pack=False, align=None),
    "MEM_LARGE_RESOURCE": SimStruct(OrderedDict((("MEM_LARGE_Header", SimTypeRef("MEM_LARGE_DES", SimStruct)), ("MEM_LARGE_Data", SimTypePointer(SimTypeRef("MEM_LARGE_RANGE", SimStruct), offset=0)),)), name="MEM_LARGE_RESOURCE", pack=False, align=None),
    "MEM_RANGE": SimStruct(OrderedDict((("MR_Align", SimTypeLongLong(signed=False, label="UInt64")), ("MR_nBytes", SimTypeInt(signed=False, label="UInt32")), ("MR_Min", SimTypeLongLong(signed=False, label="UInt64")), ("MR_Max", SimTypeLongLong(signed=False, label="UInt64")), ("MR_Flags", SimTypeInt(signed=False, label="MD_FLAGS")), ("MR_Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="MEM_RANGE", pack=False, align=None),
    "MEM_RESOURCE": SimStruct(OrderedDict((("MEM_Header", SimTypeRef("MEM_DES", SimStruct)), ("MEM_Data", SimTypePointer(SimTypeRef("MEM_RANGE", SimStruct), offset=0)),)), name="MEM_RESOURCE", pack=False, align=None),
    "MEM_SECTION_EXTENDED_PARAMETER_TYPE": SimTypeInt(signed=False, label="MEM_SECTION_EXTENDED_PARAMETER_TYPE"),
    "MENUBANDHANDLERCID": SimTypeInt(signed=False, label="MENUBANDHANDLERCID"),
    "MENUBANDPARTS": SimTypeInt(signed=False, label="MENUBANDPARTS"),
    "MENUBANDSTATES": SimTypeInt(signed=False, label="MENUBANDSTATES"),
    "MENUBARINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcBar", SimTypeRef("RECT", SimStruct)), ("hMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="MENUBARINFO", pack=False, align=None),
    "MENUBUTTONDATA": SimStruct(OrderedDict((("idCommand", SimTypeInt(signed=True, label="Int32")), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")),)), name="MENUBUTTONDATA", pack=False, align=None),
    "MENUEX_TEMPLATE_HEADER": SimStruct(OrderedDict((("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wOffset", SimTypeShort(signed=False, label="UInt16")), ("dwHelpId", SimTypeInt(signed=False, label="UInt32")),)), name="MENUEX_TEMPLATE_HEADER", pack=False, align=None),
    "MENUEX_TEMPLATE_ITEM": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("uId", SimTypeInt(signed=False, label="UInt32")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("szText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MENUEX_TEMPLATE_ITEM", pack=False, align=None),
    "MENUGETOBJECTINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="MENUGETOBJECTINFO_FLAGS")), ("uPos", SimTypeInt(signed=False, label="UInt32")), ("hmenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("riid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pvObj", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MENUGETOBJECTINFO", pack=False, align=None),
    "MENUGETOBJECTINFO_FLAGS": SimTypeInt(signed=False, label="MENUGETOBJECTINFO_FLAGS"),
    "MENUINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="MENUINFO_MASK")), ("dwStyle", SimTypeInt(signed=False, label="MENUINFO_STYLE")), ("cyMax", SimTypeInt(signed=False, label="UInt32")), ("hbrBack", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwContextHelpID", SimTypeInt(signed=False, label="UInt32")), ("dwMenuData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MENUINFO", pack=False, align=None),
    "MENUINFO_MASK": SimTypeInt(signed=False, label="MENUINFO_MASK"),
    "MENUINFO_STYLE": SimTypeInt(signed=False, label="MENUINFO_STYLE"),
    "MENUITEMINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="MENU_ITEM_MASK")), ("fType", SimTypeInt(signed=False, label="MENU_ITEM_TYPE")), ("fState", SimTypeInt(signed=False, label="MENU_ITEM_STATE")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("hSubMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmpChecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmpUnchecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwItemData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTypeData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cch", SimTypeInt(signed=False, label="UInt32")), ("hbmpItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MENUITEMINFOA", pack=False, align=None),
    "MENUITEMINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="MENU_ITEM_MASK")), ("fType", SimTypeInt(signed=False, label="MENU_ITEM_TYPE")), ("fState", SimTypeInt(signed=False, label="MENU_ITEM_STATE")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("hSubMenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmpChecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbmpUnchecked", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwItemData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwTypeData", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cch", SimTypeInt(signed=False, label="UInt32")), ("hbmpItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MENUITEMINFOW", pack=False, align=None),
    "MENUITEMTEMPLATE": SimStruct(OrderedDict((("mtOption", SimTypeShort(signed=False, label="UInt16")), ("mtID", SimTypeShort(signed=False, label="UInt16")), ("mtString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MENUITEMTEMPLATE", pack=False, align=None),
    "MENUITEMTEMPLATEHEADER": SimStruct(OrderedDict((("versionNumber", SimTypeShort(signed=False, label="UInt16")), ("offset", SimTypeShort(signed=False, label="UInt16")),)), name="MENUITEMTEMPLATEHEADER", pack=False, align=None),
    "MENUPARTS": SimTypeInt(signed=False, label="MENUPARTS"),
    "MENUPOPUPPOPUPFLAGS": SimTypeInt(signed=False, label="MENUPOPUPPOPUPFLAGS"),
    "MENUPOPUPSELECT": SimTypeInt(signed=False, label="MENUPOPUPSELECT"),
    "MENUTEMPLATEEX": SimStruct(OrderedDict((("Anonymous", SimUnion({"Menu": SimTypeRef("_Menu_e__Struct", SimStruct), "MenuEx": SimTypeRef("_MenuEx_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="MENUTEMPLATEEX", pack=False, align=None),
    "MENU_EVENT_RECORD": SimStruct(OrderedDict((("dwCommandId", SimTypeInt(signed=False, label="UInt32")),)), name="MENU_EVENT_RECORD", pack=False, align=None),
    "MENU_ITEM_FLAGS": SimTypeInt(signed=False, label="MENU_ITEM_FLAGS"),
    "MENU_ITEM_MASK": SimTypeInt(signed=False, label="MENU_ITEM_MASK"),
    "MENU_ITEM_STATE": SimTypeInt(signed=False, label="MENU_ITEM_STATE"),
    "MENU_ITEM_TYPE": SimTypeInt(signed=False, label="MENU_ITEM_TYPE"),
    "MERGE_UPDATE_STATUS": SimTypeInt(signed=False, label="MERGE_UPDATE_STATUS"),
    "MERGE_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="MERGE_VIRTUAL_DISK_FLAG"),
    "MERGE_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="MERGE_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct), "Version2": SimTypeRef("_Version2_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="MERGE_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "MERGE_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="MERGE_VIRTUAL_DISK_VERSION"),
    "MESSAGEBOX_PARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMessage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Style", SimTypeInt(signed=False, label="UInt32")), ("dwTimeout", SimTypeInt(signed=False, label="UInt32")), ("bWait", SimTypeInt(signed=True, label="Int32")),)), name="MESSAGEBOX_PARAMS", pack=False, align=None),
    "MESSAGEBOX_RESULT": SimTypeInt(signed=False, label="MESSAGEBOX_RESULT"),
    "MESSAGEBOX_STYLE": SimTypeInt(signed=False, label="MESSAGEBOX_STYLE"),
    "MESSAGE_RESOURCE_BLOCK": SimStruct(OrderedDict((("LowId", SimTypeInt(signed=False, label="UInt32")), ("HighId", SimTypeInt(signed=False, label="UInt32")), ("OffsetToEntries", SimTypeInt(signed=False, label="UInt32")),)), name="MESSAGE_RESOURCE_BLOCK", pack=False, align=None),
    "MESSAGE_RESOURCE_DATA": SimStruct(OrderedDict((("NumberOfBlocks", SimTypeInt(signed=False, label="UInt32")), ("Blocks", SimTypePointer(SimTypeRef("MESSAGE_RESOURCE_BLOCK", SimStruct), offset=0)),)), name="MESSAGE_RESOURCE_DATA", pack=False, align=None),
    "MESSAGE_RESOURCE_ENTRY": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Text", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MESSAGE_RESOURCE_ENTRY", pack=False, align=None),
    "METADATATYPES": SimTypeInt(signed=False, label="METADATATYPES"),
    "METADATA_GETALL_INTERNAL_RECORD": SimStruct(OrderedDict((("dwMDIdentifier", SimTypeInt(signed=False, label="UInt32")), ("dwMDAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwMDUserType", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataType", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataLen", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwMDDataOffset": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbMDData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("dwMDDataTag", SimTypeInt(signed=False, label="UInt32")),)), name="METADATA_GETALL_INTERNAL_RECORD", pack=False, align=None),
    "METADATA_GETALL_RECORD": SimStruct(OrderedDict((("dwMDIdentifier", SimTypeInt(signed=False, label="UInt32")), ("dwMDAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwMDUserType", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataType", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataLen", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataTag", SimTypeInt(signed=False, label="UInt32")),)), name="METADATA_GETALL_RECORD", pack=False, align=None),
    "METADATA_HANDLE_INFO": SimStruct(OrderedDict((("dwMDPermissions", SimTypeInt(signed=False, label="UInt32")), ("dwMDSystemChangeNumber", SimTypeInt(signed=False, label="UInt32")),)), name="METADATA_HANDLE_INFO", pack=False, align=None),
    "METADATA_RECORD": SimStruct(OrderedDict((("dwMDIdentifier", SimTypeInt(signed=False, label="UInt32")), ("dwMDAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwMDUserType", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataType", SimTypeInt(signed=False, label="UInt32")), ("dwMDDataLen", SimTypeInt(signed=False, label="UInt32")), ("pbMDData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwMDDataTag", SimTypeInt(signed=False, label="UInt32")),)), name="METADATA_RECORD", pack=False, align=None),
    "METAFILEPICT": SimStruct(OrderedDict((("mm", SimTypeInt(signed=True, label="Int32")), ("xExt", SimTypeInt(signed=True, label="Int32")), ("yExt", SimTypeInt(signed=True, label="Int32")), ("hMF", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="METAFILEPICT", pack=False, align=None),
    "METAHEADER": SimStruct(OrderedDict((("mtType", SimTypeShort(signed=False, label="UInt16")), ("mtHeaderSize", SimTypeShort(signed=False, label="UInt16")), ("mtVersion", SimTypeShort(signed=False, label="UInt16")), ("mtSize", SimTypeInt(signed=False, label="UInt32")), ("mtNoObjects", SimTypeShort(signed=False, label="UInt16")), ("mtMaxRecord", SimTypeInt(signed=False, label="UInt32")), ("mtNoParameters", SimTypeShort(signed=False, label="UInt16")),)), name="METAHEADER", pack=False, align=None),
    "METAHOST_CONFIG_FLAGS": SimTypeInt(signed=False, label="METAHOST_CONFIG_FLAGS"),
    "METAHOST_POLICY_FLAGS": SimTypeInt(signed=False, label="METAHOST_POLICY_FLAGS"),
    "METARECORD": SimStruct(OrderedDict((("rdSize", SimTypeInt(signed=False, label="UInt32")), ("rdFunction", SimTypeShort(signed=False, label="UInt16")), ("rdParm", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="METARECORD", pack=False, align=None),
    "METHODDATA": SimStruct(OrderedDict((("szName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ppdata", SimTypePointer(SimTypeRef("PARAMDATA", SimStruct), offset=0)), ("dispid", SimTypeInt(signed=True, label="Int32")), ("iMeth", SimTypeInt(signed=False, label="UInt32")), ("cc", SimTypeInt(signed=False, label="CALLCONV")), ("cArgs", SimTypeInt(signed=False, label="UInt32")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("vtReturn", SimTypeInt(signed=False, label="VARENUM")),)), name="METHODDATA", pack=False, align=None),
    "MF2DBuffer_LockFlags": SimTypeInt(signed=False, label="MF2DBuffer_LockFlags"),
    "MF3DVideoOutputType": SimTypeInt(signed=False, label="MF3DVideoOutputType"),
    "MFARGB": SimStruct(OrderedDict((("rgbBlue", SimTypeChar(label="Byte")), ("rgbGreen", SimTypeChar(label="Byte")), ("rgbRed", SimTypeChar(label="Byte")), ("rgbAlpha", SimTypeChar(label="Byte")),)), name="MFARGB", pack=False, align=None),
    "MFASF_INDEXER_FLAGS": SimTypeInt(signed=False, label="MFASF_INDEXER_FLAGS"),
    "MFASF_MULTIPLEXERFLAGS": SimTypeInt(signed=False, label="MFASF_MULTIPLEXERFLAGS"),
    "MFASF_SPLITTERFLAGS": SimTypeInt(signed=False, label="MFASF_SPLITTERFLAGS"),
    "MFASF_STREAMSELECTOR_FLAGS": SimTypeInt(signed=False, label="MFASF_STREAMSELECTOR_FLAGS"),
    "MFASYNCRESULT": SimTypeBottom(label="MFASYNCRESULT"),
    "MFASYNC_WORKQUEUE_TYPE": SimTypeInt(signed=False, label="MFASYNC_WORKQUEUE_TYPE"),
    "MFAYUVSample": SimStruct(OrderedDict((("bCrValue", SimTypeChar(label="Byte")), ("bCbValue", SimTypeChar(label="Byte")), ("bYValue", SimTypeChar(label="Byte")), ("bSampleAlpha8", SimTypeChar(label="Byte")),)), name="MFAYUVSample", pack=False, align=None),
    "MFAudioConstriction": SimTypeInt(signed=False, label="MFAudioConstriction"),
    "MFAudioDecoderDegradationInfo": SimStruct(OrderedDict((("eDegradationReason", SimTypeInt(signed=False, label="MFT_AUDIO_DECODER_DEGRADATION_REASON")), ("eType", SimTypeInt(signed=False, label="MFT_AUDIO_DECODER_DEGRADATION_TYPE")),)), name="MFAudioDecoderDegradationInfo", pack=False, align=None),
    "MFBYTESTREAM_BUFFERING_PARAMS": SimStruct(OrderedDict((("cbTotalFileSize", SimTypeLongLong(signed=False, label="UInt64")), ("cbPlayableDataSize", SimTypeLongLong(signed=False, label="UInt64")), ("prgBuckets", SimTypePointer(SimTypeRef("MF_LEAKY_BUCKET_PAIR", SimStruct), offset=0)), ("cBuckets", SimTypeInt(signed=False, label="UInt32")), ("qwNetBufferingTime", SimTypeLongLong(signed=False, label="UInt64")), ("qwExtraBufferingTimeDuringSeek", SimTypeLongLong(signed=False, label="UInt64")), ("qwPlayDuration", SimTypeLongLong(signed=False, label="UInt64")), ("dRate", SimTypeFloat(size=32)),)), name="MFBYTESTREAM_BUFFERING_PARAMS", pack=False, align=None),
    "MFBYTESTREAM_SEEK_ORIGIN": SimTypeInt(signed=False, label="MFBYTESTREAM_SEEK_ORIGIN"),
    "MFCARD_DES": SimStruct(OrderedDict((("PMF_Count", SimTypeInt(signed=False, label="UInt32")), ("PMF_Type", SimTypeInt(signed=False, label="UInt32")), ("PMF_Flags", SimTypeInt(signed=False, label="PMF_FLAGS")), ("PMF_ConfigOptions", SimTypeChar(label="Byte")), ("PMF_IoResourceIndex", SimTypeChar(label="Byte")), ("PMF_Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("PMF_ConfigRegisterBase", SimTypeInt(signed=False, label="UInt32")),)), name="MFCARD_DES", pack=False, align=None),
    "MFCARD_RESOURCE": SimStruct(OrderedDict((("MfCard_Header", SimTypeRef("MFCARD_DES", SimStruct)),)), name="MFCARD_RESOURCE", pack=False, align=None),
    "MFCLOCK_CHARACTERISTICS_FLAGS": SimTypeInt(signed=False, label="MFCLOCK_CHARACTERISTICS_FLAGS"),
    "MFCLOCK_PROPERTIES": SimStruct(OrderedDict((("qwCorrelationRate", SimTypeLongLong(signed=False, label="UInt64")), ("guidClockId", SimTypeBottom(label="Guid")), ("dwClockFlags", SimTypeInt(signed=False, label="UInt32")), ("qwClockFrequency", SimTypeLongLong(signed=False, label="UInt64")), ("dwClockTolerance", SimTypeInt(signed=False, label="UInt32")), ("dwClockJitter", SimTypeInt(signed=False, label="UInt32")),)), name="MFCLOCK_PROPERTIES", pack=False, align=None),
    "MFCLOCK_RELATIONAL_FLAGS": SimTypeInt(signed=False, label="MFCLOCK_RELATIONAL_FLAGS"),
    "MFCLOCK_STATE": SimTypeInt(signed=False, label="MFCLOCK_STATE"),
    "MFCONTENTPROTECTIONDEVICE_INPUT_DATA": SimStruct(OrderedDict((("HWProtectionFunctionID", SimTypeInt(signed=False, label="UInt32")), ("PrivateDataByteCount", SimTypeInt(signed=False, label="UInt32")), ("HWProtectionDataByteCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("InputData", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="MFCONTENTPROTECTIONDEVICE_INPUT_DATA", pack=False, align=None),
    "MFCONTENTPROTECTIONDEVICE_OUTPUT_DATA": SimStruct(OrderedDict((("PrivateDataByteCount", SimTypeInt(signed=False, label="UInt32")), ("MaxHWProtectionDataByteCount", SimTypeInt(signed=False, label="UInt32")), ("HWProtectionDataByteCount", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")), ("TransportTimeInHundredsOfNanoseconds", SimTypeLongLong(signed=True, label="Int64")), ("ExecutionTimeInHundredsOfNanoseconds", SimTypeLongLong(signed=True, label="Int64")), ("OutputData", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="MFCONTENTPROTECTIONDEVICE_OUTPUT_DATA", pack=False, align=None),
    "MFCONTENTPROTECTIONDEVICE_REALTIMECLIENT_DATA": SimStruct(OrderedDict((("TaskIndex", SimTypeInt(signed=False, label="UInt32")), ("ClassName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("BasePriority", SimTypeInt(signed=True, label="Int32")),)), name="MFCONTENTPROTECTIONDEVICE_REALTIMECLIENT_DATA", pack=False, align=None),
    "MFCameraExtrinsic_CalibratedTransform": SimStruct(OrderedDict((("CalibrationId", SimTypeBottom(label="Guid")), ("Position", SimTypeRef("MF_FLOAT3", SimStruct)), ("Orientation", SimTypeRef("MF_QUATERNION", SimStruct)),)), name="MFCameraExtrinsic_CalibratedTransform", pack=False, align=None),
    "MFCameraExtrinsics": SimStruct(OrderedDict((("TransformCount", SimTypeInt(signed=False, label="UInt32")), ("CalibratedTransforms", SimTypePointer(SimTypeRef("MFCameraExtrinsic_CalibratedTransform", SimStruct), offset=0)),)), name="MFCameraExtrinsics", pack=False, align=None),
    "MFCameraIntrinsic_CameraModel": SimStruct(OrderedDict((("FocalLength_x", SimTypeFloat(size=32)), ("FocalLength_y", SimTypeFloat(size=32)), ("PrincipalPoint_x", SimTypeFloat(size=32)), ("PrincipalPoint_y", SimTypeFloat(size=32)),)), name="MFCameraIntrinsic_CameraModel", pack=False, align=None),
    "MFCameraIntrinsic_DistortionModel": SimStruct(OrderedDict((("Radial_k1", SimTypeFloat(size=32)), ("Radial_k2", SimTypeFloat(size=32)), ("Radial_k3", SimTypeFloat(size=32)), ("Tangential_p1", SimTypeFloat(size=32)), ("Tangential_p2", SimTypeFloat(size=32)),)), name="MFCameraIntrinsic_DistortionModel", pack=False, align=None),
    "MFCameraIntrinsic_DistortionModel6KT": SimStruct(OrderedDict((("Radial_k1", SimTypeFloat(size=32)), ("Radial_k2", SimTypeFloat(size=32)), ("Radial_k3", SimTypeFloat(size=32)), ("Radial_k4", SimTypeFloat(size=32)), ("Radial_k5", SimTypeFloat(size=32)), ("Radial_k6", SimTypeFloat(size=32)), ("Tangential_p1", SimTypeFloat(size=32)), ("Tangential_p2", SimTypeFloat(size=32)),)), name="MFCameraIntrinsic_DistortionModel6KT", pack=False, align=None),
    "MFCameraIntrinsic_DistortionModelArcTan": SimStruct(OrderedDict((("Radial_k0", SimTypeFloat(size=32)), ("DistortionCenter_x", SimTypeFloat(size=32)), ("DistortionCenter_y", SimTypeFloat(size=32)), ("Tangential_x", SimTypeFloat(size=32)), ("Tangential_y", SimTypeFloat(size=32)),)), name="MFCameraIntrinsic_DistortionModelArcTan", pack=False, align=None),
    "MFCameraIntrinsic_DistortionModelType": SimTypeInt(signed=False, label="MFCameraIntrinsic_DistortionModelType"),
    "MFCameraIntrinsic_PinholeCameraModel": SimStruct(OrderedDict((("FocalLength", SimTypeRef("MF_FLOAT2", SimStruct)), ("PrincipalPoint", SimTypeRef("MF_FLOAT2", SimStruct)),)), name="MFCameraIntrinsic_PinholeCameraModel", pack=False, align=None),
    "MFCameraOcclusionState": SimTypeInt(signed=False, label="MFCameraOcclusionState"),
    "MFDepthMeasurement": SimTypeInt(signed=False, label="MFDepthMeasurement"),
    "MFENUMPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("HANDLETABLE", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeRef("METARECORD", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hdc", "lpht", "lpMR", "nObj", "param4"]), offset=0),
    "MFExtendedCameraIntrinsic_IntrinsicModel": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("SplitFrameId", SimTypeInt(signed=False, label="UInt32")), ("CameraModel", SimTypeRef("MFCameraIntrinsic_CameraModel", SimStruct)),)), name="MFExtendedCameraIntrinsic_IntrinsicModel", pack=False, align=None),
    "MFFOLDDOWN_MATRIX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cSrcChannels", SimTypeInt(signed=False, label="UInt32")), ("cDstChannels", SimTypeInt(signed=False, label="UInt32")), ("dwChannelMask", SimTypeInt(signed=False, label="UInt32")), ("Coeff", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 64)),)), name="MFFOLDDOWN_MATRIX", pack=False, align=None),
    "MFFrameSourceTypes": SimTypeInt(signed=False, label="MFFrameSourceTypes"),
    "MFINPUTTRUSTAUTHORITY_ACCESS_ACTION": SimStruct(OrderedDict((("Action", SimTypeInt(signed=False, label="MFPOLICYMANAGER_ACTION")), ("pbTicket", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbTicket", SimTypeInt(signed=False, label="UInt32")),)), name="MFINPUTTRUSTAUTHORITY_ACCESS_ACTION", pack=False, align=None),
    "MFINPUTTRUSTAUTHORITY_ACCESS_PARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVer", SimTypeInt(signed=False, label="UInt32")), ("cbSignatureOffset", SimTypeInt(signed=False, label="UInt32")), ("cbSignatureSize", SimTypeInt(signed=False, label="UInt32")), ("cbExtensionOffset", SimTypeInt(signed=False, label="UInt32")), ("cbExtensionSize", SimTypeInt(signed=False, label="UInt32")), ("cActions", SimTypeInt(signed=False, label="UInt32")), ("rgOutputActions", SimTypePointer(SimTypeRef("MFINPUTTRUSTAUTHORITY_ACCESS_ACTION", SimStruct), offset=0)),)), name="MFINPUTTRUSTAUTHORITY_ACCESS_PARAMS", pack=False, align=None),
    "MFMEDIASOURCE_CHARACTERISTICS": SimTypeInt(signed=False, label="MFMEDIASOURCE_CHARACTERISTICS"),
    "MFMPEG2DLNASINKSTATS": SimStruct(OrderedDict((("cBytesWritten", SimTypeLongLong(signed=False, label="UInt64")), ("fPAL", SimTypeInt(signed=True, label="Int32")), ("fccVideo", SimTypeInt(signed=False, label="UInt32")), ("dwVideoWidth", SimTypeInt(signed=False, label="UInt32")), ("dwVideoHeight", SimTypeInt(signed=False, label="UInt32")), ("cVideoFramesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("cVideoFramesEncoded", SimTypeLongLong(signed=False, label="UInt64")), ("cVideoFramesSkipped", SimTypeLongLong(signed=False, label="UInt64")), ("cBlackVideoFramesEncoded", SimTypeLongLong(signed=False, label="UInt64")), ("cVideoFramesDuplicated", SimTypeLongLong(signed=False, label="UInt64")), ("cAudioSamplesPerSec", SimTypeInt(signed=False, label="UInt32")), ("cAudioChannels", SimTypeInt(signed=False, label="UInt32")), ("cAudioBytesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("cAudioFramesEncoded", SimTypeLongLong(signed=False, label="UInt64")),)), name="MFMPEG2DLNASINKSTATS", pack=False, align=None),
    "MFMediaKeyStatus": SimStruct(OrderedDict((("pbKeyId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKeyId", SimTypeInt(signed=False, label="UInt32")), ("eMediaKeyStatus", SimTypeInt(signed=False, label="MF_MEDIAKEY_STATUS")),)), name="MFMediaKeyStatus", pack=False, align=None),
    "MFNETSOURCE_CACHE_STATE": SimTypeInt(signed=False, label="MFNETSOURCE_CACHE_STATE"),
    "MFNETSOURCE_PROTOCOL_TYPE": SimTypeInt(signed=False, label="MFNETSOURCE_PROTOCOL_TYPE"),
    "MFNETSOURCE_STATISTICS_IDS": SimTypeInt(signed=False, label="MFNETSOURCE_STATISTICS_IDS"),
    "MFNETSOURCE_TRANSPORT_TYPE": SimTypeInt(signed=False, label="MFNETSOURCE_TRANSPORT_TYPE"),
    "MFNET_PROXYSETTINGS": SimTypeInt(signed=False, label="MFNET_PROXYSETTINGS"),
    "MFNetAuthenticationFlags": SimTypeInt(signed=False, label="MFNetAuthenticationFlags"),
    "MFNetCredentialManagerGetParam": SimStruct(OrderedDict((("hrOp", SimTypeInt(signed=True, label="Int32")), ("fAllowLoggedOnUser", SimTypeInt(signed=True, label="Int32")), ("fClearTextPackage", SimTypeInt(signed=True, label="Int32")), ("pszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSite", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRealm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPackage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nRetries", SimTypeInt(signed=True, label="Int32")),)), name="MFNetCredentialManagerGetParam", pack=False, align=None),
    "MFNetCredentialOptions": SimTypeInt(signed=False, label="MFNetCredentialOptions"),
    "MFNetCredentialRequirements": SimTypeInt(signed=False, label="MFNetCredentialRequirements"),
    "MFNominalRange": SimTypeInt(signed=False, label="MFNominalRange"),
    "MFOffset": SimStruct(OrderedDict((("fract", SimTypeShort(signed=False, label="UInt16")), ("value", SimTypeShort(signed=True, label="Int16")),)), name="MFOffset", pack=False, align=None),
    "MFPERIODICCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown")], SimTypeBottom(label="Void"), arg_names=["pContext"]), offset=0),
    "MFPMPSESSION_CREATION_FLAGS": SimTypeInt(signed=False, label="MFPMPSESSION_CREATION_FLAGS"),
    "MFPOLICYMANAGER_ACTION": SimTypeInt(signed=False, label="MFPOLICYMANAGER_ACTION"),
    "MFP_ACQUIRE_USER_CREDENTIAL_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("dwUserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fProceedWithAuthentication", SimTypeInt(signed=True, label="Int32")), ("hrAuthenticationStatus", SimTypeInt(signed=True, label="Int32")), ("pwszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszSite", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszRealm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPackage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nRetries", SimTypeInt(signed=True, label="Int32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("pCredential", SimTypeBottom(label="IMFNetCredential")),)), name="MFP_ACQUIRE_USER_CREDENTIAL_EVENT", pack=False, align=None),
    "MFP_CREATION_OPTIONS": SimTypeInt(signed=False, label="MFP_CREATION_OPTIONS"),
    "MFP_ERROR_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)),)), name="MFP_ERROR_EVENT", pack=False, align=None),
    "MFP_EVENT_HEADER": SimStruct(OrderedDict((("eEventType", SimTypeInt(signed=False, label="MFP_EVENT_TYPE")), ("hrEvent", SimTypeInt(signed=True, label="Int32")), ("pMediaPlayer", SimTypeBottom(label="IMFPMediaPlayer")), ("eState", SimTypeInt(signed=False, label="MFP_MEDIAPLAYER_STATE")), ("pPropertyStore", SimTypeBottom(label="IPropertyStore")),)), name="MFP_EVENT_HEADER", pack=False, align=None),
    "MFP_EVENT_TYPE": SimTypeInt(signed=False, label="MFP_EVENT_TYPE"),
    "MFP_FRAME_STEP_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_FRAME_STEP_EVENT", pack=False, align=None),
    "MFP_MEDIAITEM_CLEARED_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_MEDIAITEM_CLEARED_EVENT", pack=False, align=None),
    "MFP_MEDIAITEM_CREATED_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")), ("dwUserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MFP_MEDIAITEM_CREATED_EVENT", pack=False, align=None),
    "MFP_MEDIAITEM_SET_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_MEDIAITEM_SET_EVENT", pack=False, align=None),
    "MFP_MEDIAPLAYER_STATE": SimTypeInt(signed=False, label="MFP_MEDIAPLAYER_STATE"),
    "MFP_MF_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("MFEventType", SimTypeInt(signed=False, label="UInt32")), ("pMFMediaEvent", SimTypeBottom(label="IMFMediaEvent")), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_MF_EVENT", pack=False, align=None),
    "MFP_PAUSE_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_PAUSE_EVENT", pack=False, align=None),
    "MFP_PLAYBACK_ENDED_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_PLAYBACK_ENDED_EVENT", pack=False, align=None),
    "MFP_PLAY_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_PLAY_EVENT", pack=False, align=None),
    "MFP_POSITION_SET_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_POSITION_SET_EVENT", pack=False, align=None),
    "MFP_RATE_SET_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")), ("flRate", SimTypeFloat(size=32)),)), name="MFP_RATE_SET_EVENT", pack=False, align=None),
    "MFP_STOP_EVENT": SimStruct(OrderedDict((("header", SimTypeRef("MFP_EVENT_HEADER", SimStruct)), ("pMediaItem", SimTypeBottom(label="IMFPMediaItem")),)), name="MFP_STOP_EVENT", pack=False, align=None),
    "MFPaletteEntry": SimUnion({"ARGB": SimTypeRef("MFARGB", SimStruct), "AYCbCr": SimTypeRef("MFAYUVSample", SimStruct)}, name="<anon>", label="None"),
    "MFPinholeCameraIntrinsic_IntrinsicModel": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("CameraModel", SimTypeRef("MFCameraIntrinsic_PinholeCameraModel", SimStruct)), ("DistortionModel", SimTypeRef("MFCameraIntrinsic_DistortionModel", SimStruct)),)), name="MFPinholeCameraIntrinsic_IntrinsicModel", pack=False, align=None),
    "MFPinholeCameraIntrinsics": SimStruct(OrderedDict((("IntrinsicModelCount", SimTypeInt(signed=False, label="UInt32")), ("IntrinsicModels", SimTypePointer(SimTypeRef("MFPinholeCameraIntrinsic_IntrinsicModel", SimStruct), offset=0)),)), name="MFPinholeCameraIntrinsics", pack=False, align=None),
    "MFRATE_DIRECTION": SimTypeInt(signed=False, label="MFRATE_DIRECTION"),
    "MFRR_COMPONENTS": SimStruct(OrderedDict((("dwRRInfoVersion", SimTypeInt(signed=False, label="UInt32")), ("dwRRComponents", SimTypeInt(signed=False, label="UInt32")), ("pRRComponents", SimTypePointer(SimTypeRef("MFRR_COMPONENT_HASH_INFO", SimStruct), offset=0)),)), name="MFRR_COMPONENTS", pack=False, align=None),
    "MFRR_COMPONENT_HASH_INFO": SimStruct(OrderedDict((("ulReason", SimTypeInt(signed=False, label="UInt32")), ("rgHeaderHash", SimTypeArray(SimTypeChar(label="Char"), 43)), ("rgPublicKeyHash", SimTypeArray(SimTypeChar(label="Char"), 43)), ("wszName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="MFRR_COMPONENT_HASH_INFO", pack=False, align=None),
    "MFRatio": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="MFRatio", pack=False, align=None),
    "MFSESSION_GETFULLTOPOLOGY_FLAGS": SimTypeInt(signed=False, label="MFSESSION_GETFULLTOPOLOGY_FLAGS"),
    "MFSESSION_SETTOPOLOGY_FLAGS": SimTypeInt(signed=False, label="MFSESSION_SETTOPOLOGY_FLAGS"),
    "MFSHUTDOWN_STATUS": SimTypeInt(signed=False, label="MFSHUTDOWN_STATUS"),
    "MFSINK_WMDRMACTION": SimTypeInt(signed=False, label="MFSINK_WMDRMACTION"),
    "MFSTREAMSINK_MARKER_TYPE": SimTypeInt(signed=False, label="MFSTREAMSINK_MARKER_TYPE"),
    "MFSampleAllocatorUsage": SimTypeInt(signed=False, label="MFSampleAllocatorUsage"),
    "MFSampleEncryptionProtectionScheme": SimTypeInt(signed=False, label="MFSampleEncryptionProtectionScheme"),
    "MFSensorDeviceMode": SimTypeInt(signed=False, label="MFSensorDeviceMode"),
    "MFSensorDeviceType": SimTypeInt(signed=False, label="MFSensorDeviceType"),
    "MFSensorStreamType": SimTypeInt(signed=False, label="MFSensorStreamType"),
    "MFSequencerTopologyFlags": SimTypeInt(signed=False, label="MFSequencerTopologyFlags"),
    "MFStandardVideoFormat": SimTypeInt(signed=False, label="MFStandardVideoFormat"),
    "MFTIMER_FLAGS": SimTypeInt(signed=False, label="MFTIMER_FLAGS"),
    "MFTOPOLOGY_DXVA_MODE": SimTypeInt(signed=False, label="MFTOPOLOGY_DXVA_MODE"),
    "MFTOPOLOGY_HARDWARE_MODE": SimTypeInt(signed=False, label="MFTOPOLOGY_HARDWARE_MODE"),
    "MFTOPONODE_ATTRIBUTE_UPDATE": SimStruct(OrderedDict((("NodeId", SimTypeLongLong(signed=False, label="UInt64")), ("guidAttributeKey", SimTypeBottom(label="Guid")), ("attrType", SimTypeInt(signed=False, label="MF_ATTRIBUTE_TYPE")), ("Anonymous", SimUnion({"u32": SimTypeInt(signed=False, label="UInt32"), "u64": SimTypeLongLong(signed=False, label="UInt64"), "d": SimTypeFloat(size=64)}, name="<anon>", label="None")),)), name="MFTOPONODE_ATTRIBUTE_UPDATE", pack=False, align=None),
    "MFT_AUDIO_DECODER_DEGRADATION_REASON": SimTypeInt(signed=False, label="MFT_AUDIO_DECODER_DEGRADATION_REASON"),
    "MFT_AUDIO_DECODER_DEGRADATION_TYPE": SimTypeInt(signed=False, label="MFT_AUDIO_DECODER_DEGRADATION_TYPE"),
    "MFT_DRAIN_TYPE": SimTypeInt(signed=False, label="MFT_DRAIN_TYPE"),
    "MFT_ENUM_DATA": SimStruct(OrderedDict((("StartFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("LowUsn", SimTypeLongLong(signed=True, label="Int64")), ("HighUsn", SimTypeLongLong(signed=True, label="Int64")), ("MinMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="MFT_ENUM_DATA", pack=False, align=None),
    "MFT_ENUM_DATA_V0": SimStruct(OrderedDict((("StartFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("LowUsn", SimTypeLongLong(signed=True, label="Int64")), ("HighUsn", SimTypeLongLong(signed=True, label="Int64")),)), name="MFT_ENUM_DATA_V0", pack=False, align=None),
    "MFT_ENUM_DATA_V1": SimStruct(OrderedDict((("StartFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("LowUsn", SimTypeLongLong(signed=True, label="Int64")), ("HighUsn", SimTypeLongLong(signed=True, label="Int64")), ("MinMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="MFT_ENUM_DATA_V1", pack=False, align=None),
    "MFT_ENUM_FLAG": SimTypeInt(signed=False, label="MFT_ENUM_FLAG"),
    "MFT_INPUT_STREAM_INFO": SimStruct(OrderedDict((("hnsMaxLatency", SimTypeLongLong(signed=True, label="Int64")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cbMaxLookahead", SimTypeInt(signed=False, label="UInt32")), ("cbAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="MFT_INPUT_STREAM_INFO", pack=False, align=None),
    "MFT_MESSAGE_TYPE": SimTypeInt(signed=False, label="MFT_MESSAGE_TYPE"),
    "MFT_OUTPUT_DATA_BUFFER": SimStruct(OrderedDict((("dwStreamID", SimTypeInt(signed=False, label="UInt32")), ("pSample", SimTypeBottom(label="IMFSample")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("pEvents", SimTypeBottom(label="IMFCollection")),)), name="MFT_OUTPUT_DATA_BUFFER", pack=False, align=None),
    "MFT_OUTPUT_STREAM_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cbAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="MFT_OUTPUT_STREAM_INFO", pack=False, align=None),
    "MFT_REGISTER_TYPE_INFO": SimStruct(OrderedDict((("guidMajorType", SimTypeBottom(label="Guid")), ("guidSubtype", SimTypeBottom(label="Guid")),)), name="MFT_REGISTER_TYPE_INFO", pack=False, align=None),
    "MFT_REGISTRATION_INFO": SimStruct(OrderedDict((("clsid", SimTypeBottom(label="Guid")), ("guidCategory", SimTypeBottom(label="Guid")), ("uiFlags", SimTypeInt(signed=False, label="UInt32")), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cInTypes", SimTypeInt(signed=False, label="UInt32")), ("pInTypes", SimTypePointer(SimTypeRef("MFT_REGISTER_TYPE_INFO", SimStruct), offset=0)), ("cOutTypes", SimTypeInt(signed=False, label="UInt32")), ("pOutTypes", SimTypePointer(SimTypeRef("MFT_REGISTER_TYPE_INFO", SimStruct), offset=0)),)), name="MFT_REGISTRATION_INFO", pack=False, align=None),
    "MFT_STREAM_STATE_PARAM": SimStruct(OrderedDict((("StreamId", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="MF_STREAM_STATE")),)), name="MFT_STREAM_STATE_PARAM", pack=False, align=None),
    "MFVIDEOFORMAT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("videoInfo", SimTypeRef("MFVideoInfo", SimStruct)), ("guidFormat", SimTypeBottom(label="Guid")), ("compressedInfo", SimTypeRef("MFVideoCompressedInfo", SimStruct)), ("surfaceInfo", SimTypeRef("MFVideoSurfaceInfo", SimStruct)),)), name="MFVIDEOFORMAT", pack=False, align=None),
    "MFVP_MESSAGE_TYPE": SimTypeInt(signed=False, label="MFVP_MESSAGE_TYPE"),
    "MFVideo3DFormat": SimTypeInt(signed=False, label="MFVideo3DFormat"),
    "MFVideo3DSampleFormat": SimTypeInt(signed=False, label="MFVideo3DSampleFormat"),
    "MFVideoAlphaBitmap": SimStruct(OrderedDict((("GetBitmapFromDC", SimTypeInt(signed=True, label="Int32")), ("bitmap", SimUnion({"hdc": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pDDS": SimTypeBottom(label="IDirect3DSurface9")}, name="<anon>", label="None")), ("params", SimTypeRef("MFVideoAlphaBitmapParams", SimStruct)),)), name="MFVideoAlphaBitmap", pack=False, align=None),
    "MFVideoAlphaBitmapFlags": SimTypeInt(signed=False, label="MFVideoAlphaBitmapFlags"),
    "MFVideoAlphaBitmapParams": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("clrSrcKey", SimTypeInt(signed=False, label="UInt32")), ("rcSrc", SimTypeRef("RECT", SimStruct)), ("nrcDest", SimTypeRef("MFVideoNormalizedRect", SimStruct)), ("fAlpha", SimTypeFloat(size=32)), ("dwFilterMode", SimTypeInt(signed=False, label="UInt32")),)), name="MFVideoAlphaBitmapParams", pack=False, align=None),
    "MFVideoArea": SimStruct(OrderedDict((("OffsetX", SimTypeRef("MFOffset", SimStruct)), ("OffsetY", SimTypeRef("MFOffset", SimStruct)), ("Area", SimTypeRef("SIZE", SimStruct)),)), name="MFVideoArea", pack=False, align=None),
    "MFVideoAspectRatioMode": SimTypeInt(signed=False, label="MFVideoAspectRatioMode"),
    "MFVideoChromaSubsampling": SimTypeInt(signed=False, label="MFVideoChromaSubsampling"),
    "MFVideoCompressedInfo": SimStruct(OrderedDict((("AvgBitrate", SimTypeLongLong(signed=True, label="Int64")), ("AvgBitErrorRate", SimTypeLongLong(signed=True, label="Int64")), ("MaxKeyFrameSpacing", SimTypeInt(signed=False, label="UInt32")),)), name="MFVideoCompressedInfo", pack=False, align=None),
    "MFVideoDRMFlags": SimTypeInt(signed=False, label="MFVideoDRMFlags"),
    "MFVideoDSPMode": SimTypeInt(signed=False, label="MFVideoDSPMode"),
    "MFVideoFlags": SimTypeInt(signed=False, label="MFVideoFlags"),
    "MFVideoInfo": SimStruct(OrderedDict((("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("PixelAspectRatio", SimTypeRef("MFRatio", SimStruct)), ("SourceChromaSubsampling", SimTypeInt(signed=False, label="MFVideoChromaSubsampling")), ("InterlaceMode", SimTypeInt(signed=False, label="MFVideoInterlaceMode")), ("TransferFunction", SimTypeInt(signed=False, label="MFVideoTransferFunction")), ("ColorPrimaries", SimTypeInt(signed=False, label="MFVideoPrimaries")), ("TransferMatrix", SimTypeInt(signed=False, label="MFVideoTransferMatrix")), ("SourceLighting", SimTypeInt(signed=False, label="MFVideoLighting")), ("FramesPerSecond", SimTypeRef("MFRatio", SimStruct)), ("NominalRange", SimTypeInt(signed=False, label="MFNominalRange")), ("GeometricAperture", SimTypeRef("MFVideoArea", SimStruct)), ("MinimumDisplayAperture", SimTypeRef("MFVideoArea", SimStruct)), ("PanScanAperture", SimTypeRef("MFVideoArea", SimStruct)), ("VideoFlags", SimTypeLongLong(signed=False, label="UInt64")),)), name="MFVideoInfo", pack=False, align=None),
    "MFVideoInterlaceMode": SimTypeInt(signed=False, label="MFVideoInterlaceMode"),
    "MFVideoLighting": SimTypeInt(signed=False, label="MFVideoLighting"),
    "MFVideoMixPrefs": SimTypeInt(signed=False, label="MFVideoMixPrefs"),
    "MFVideoNormalizedRect": SimStruct(OrderedDict((("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("right", SimTypeFloat(size=32)), ("bottom", SimTypeFloat(size=32)),)), name="MFVideoNormalizedRect", pack=False, align=None),
    "MFVideoPadFlags": SimTypeInt(signed=False, label="MFVideoPadFlags"),
    "MFVideoPrimaries": SimTypeInt(signed=False, label="MFVideoPrimaries"),
    "MFVideoRenderPrefs": SimTypeInt(signed=False, label="MFVideoRenderPrefs"),
    "MFVideoRotationFormat": SimTypeInt(signed=False, label="MFVideoRotationFormat"),
    "MFVideoSphericalFormat": SimTypeInt(signed=False, label="MFVideoSphericalFormat"),
    "MFVideoSphericalProjectionMode": SimTypeInt(signed=False, label="MFVideoSphericalProjectionMode"),
    "MFVideoSrcContentHintFlags": SimTypeInt(signed=False, label="MFVideoSrcContentHintFlags"),
    "MFVideoSurfaceInfo": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="UInt32")), ("PaletteEntries", SimTypeInt(signed=False, label="UInt32")), ("Palette", SimTypePointer(SimUnion({"ARGB": SimTypeRef("MFARGB", SimStruct), "AYCbCr": SimTypeRef("MFAYUVSample", SimStruct)}, name="<anon>", label="None"), offset=0)),)), name="MFVideoSurfaceInfo", pack=False, align=None),
    "MFVideoTransferFunction": SimTypeInt(signed=False, label="MFVideoTransferFunction"),
    "MFVideoTransferMatrix": SimTypeInt(signed=False, label="MFVideoTransferMatrix"),
    "MFVirtualCameraAccess": SimTypeInt(signed=False, label="MFVirtualCameraAccess"),
    "MFVirtualCameraLifetime": SimTypeInt(signed=False, label="MFVirtualCameraLifetime"),
    "MFVirtualCameraType": SimTypeInt(signed=False, label="MFVirtualCameraType"),
    "MFWaveFormatExConvertFlags": SimTypeInt(signed=False, label="MFWaveFormatExConvertFlags"),
    "MF_ACTIVATE_CUSTOM_MIXER": SimTypeInt(signed=False, label="MF_ACTIVATE_CUSTOM_MIXER"),
    "MF_ACTIVATE_CUSTOM_PRESENTER": SimTypeInt(signed=False, label="MF_ACTIVATE_CUSTOM_PRESENTER"),
    "MF_ATTRIBUTES_MATCH_TYPE": SimTypeInt(signed=False, label="MF_ATTRIBUTES_MATCH_TYPE"),
    "MF_ATTRIBUTE_SERIALIZE_OPTIONS": SimTypeInt(signed=False, label="MF_ATTRIBUTE_SERIALIZE_OPTIONS"),
    "MF_ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="MF_ATTRIBUTE_TYPE"),
    "MF_AUVRHP_ROOMMODEL": SimTypeInt(signed=False, label="MF_AUVRHP_ROOMMODEL"),
    "MF_BYTE_STREAM_CACHE_RANGE": SimStruct(OrderedDict((("qwStartOffset", SimTypeLongLong(signed=False, label="UInt64")), ("qwEndOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="MF_BYTE_STREAM_CACHE_RANGE", pack=False, align=None),
    "MF_CAMERA_CONTROL_CONFIGURATION_TYPE": SimTypeInt(signed=False, label="MF_CAMERA_CONTROL_CONFIGURATION_TYPE"),
    "MF_CAMERA_CONTROL_RANGE_INFO": SimStruct(OrderedDict((("minValue", SimTypeInt(signed=True, label="Int32")), ("maxValue", SimTypeInt(signed=True, label="Int32")), ("stepValue", SimTypeInt(signed=True, label="Int32")), ("defaultValue", SimTypeInt(signed=True, label="Int32")),)), name="MF_CAMERA_CONTROL_RANGE_INFO", pack=False, align=None),
    "MF_CAPTURE_ENGINE_AUDIO_PROCESSING_MODE": SimTypeInt(signed=False, label="MF_CAPTURE_ENGINE_AUDIO_PROCESSING_MODE"),
    "MF_CAPTURE_ENGINE_DEVICE_TYPE": SimTypeInt(signed=False, label="MF_CAPTURE_ENGINE_DEVICE_TYPE"),
    "MF_CAPTURE_ENGINE_MEDIA_CATEGORY_TYPE": SimTypeInt(signed=False, label="MF_CAPTURE_ENGINE_MEDIA_CATEGORY_TYPE"),
    "MF_CAPTURE_ENGINE_SINK_TYPE": SimTypeInt(signed=False, label="MF_CAPTURE_ENGINE_SINK_TYPE"),
    "MF_CAPTURE_ENGINE_SOURCE": SimTypeInt(signed=False, label="MF_CAPTURE_ENGINE_SOURCE"),
    "MF_CAPTURE_ENGINE_STREAM_CATEGORY": SimTypeInt(signed=False, label="MF_CAPTURE_ENGINE_STREAM_CATEGORY"),
    "MF_CONNECT_METHOD": SimTypeInt(signed=False, label="MF_CONNECT_METHOD"),
    "MF_CROSS_ORIGIN_POLICY": SimTypeInt(signed=False, label="MF_CROSS_ORIGIN_POLICY"),
    "MF_CUSTOM_DECODE_UNIT_TYPE": SimTypeInt(signed=False, label="MF_CUSTOM_DECODE_UNIT_TYPE"),
    "MF_EVENT_TYPE": SimTypeInt(signed=False, label="MF_EVENT_TYPE"),
    "MF_FILE_ACCESSMODE": SimTypeInt(signed=False, label="MF_FILE_ACCESSMODE"),
    "MF_FILE_FLAGS": SimTypeInt(signed=False, label="MF_FILE_FLAGS"),
    "MF_FILE_OPENMODE": SimTypeInt(signed=False, label="MF_FILE_OPENMODE"),
    "MF_FLOAT2": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="MF_FLOAT2", pack=False, align=None),
    "MF_FLOAT3": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("z", SimTypeFloat(size=32)),)), name="MF_FLOAT3", pack=False, align=None),
    "MF_HDCP_STATUS": SimTypeInt(signed=False, label="MF_HDCP_STATUS"),
    "MF_LEAKY_BUCKET_PAIR": SimStruct(OrderedDict((("dwBitrate", SimTypeInt(signed=False, label="UInt32")), ("msBufferWindow", SimTypeInt(signed=False, label="UInt32")),)), name="MF_LEAKY_BUCKET_PAIR", pack=False, align=None),
    "MF_MDL_SHARED_PAYLOAD_KEY": SimUnion({"combined": SimTypeRef("_combined_e__Struct", SimStruct), "GMDLHandle": SimTypeBottom(label="Guid")}, name="<anon>", label="None"),
    "MF_MEDIAKEYSESSION_MESSAGETYPE": SimTypeInt(signed=False, label="MF_MEDIAKEYSESSION_MESSAGETYPE"),
    "MF_MEDIAKEYSESSION_TYPE": SimTypeInt(signed=False, label="MF_MEDIAKEYSESSION_TYPE"),
    "MF_MEDIAKEYS_REQUIREMENT": SimTypeInt(signed=False, label="MF_MEDIAKEYS_REQUIREMENT"),
    "MF_MEDIAKEY_STATUS": SimTypeInt(signed=False, label="MF_MEDIAKEY_STATUS"),
    "MF_MEDIASOURCE_STATUS_INFO": SimTypeInt(signed=False, label="MF_MEDIASOURCE_STATUS_INFO"),
    "MF_MEDIA_ENGINE_CANPLAY": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_CANPLAY"),
    "MF_MEDIA_ENGINE_CREATEFLAGS": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_CREATEFLAGS"),
    "MF_MEDIA_ENGINE_ERR": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_ERR"),
    "MF_MEDIA_ENGINE_EVENT": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_EVENT"),
    "MF_MEDIA_ENGINE_EXTENSION_TYPE": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_EXTENSION_TYPE"),
    "MF_MEDIA_ENGINE_FRAME_PROTECTION_FLAGS": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_FRAME_PROTECTION_FLAGS"),
    "MF_MEDIA_ENGINE_KEYERR": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_KEYERR"),
    "MF_MEDIA_ENGINE_NETWORK": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_NETWORK"),
    "MF_MEDIA_ENGINE_OPM_STATUS": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_OPM_STATUS"),
    "MF_MEDIA_ENGINE_PRELOAD": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_PRELOAD"),
    "MF_MEDIA_ENGINE_PROTECTION_FLAGS": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_PROTECTION_FLAGS"),
    "MF_MEDIA_ENGINE_READY": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_READY"),
    "MF_MEDIA_ENGINE_S3D_PACKING_MODE": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_S3D_PACKING_MODE"),
    "MF_MEDIA_ENGINE_SEEK_MODE": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_SEEK_MODE"),
    "MF_MEDIA_ENGINE_STATISTIC": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_STATISTIC"),
    "MF_MEDIA_ENGINE_STREAMTYPE_FAILED": SimTypeInt(signed=False, label="MF_MEDIA_ENGINE_STREAMTYPE_FAILED"),
    "MF_MEDIA_SHARING_ENGINE_EVENT": SimTypeInt(signed=False, label="MF_MEDIA_SHARING_ENGINE_EVENT"),
    "MF_MSE_APPEND_MODE": SimTypeInt(signed=False, label="MF_MSE_APPEND_MODE"),
    "MF_MSE_ERROR": SimTypeInt(signed=False, label="MF_MSE_ERROR"),
    "MF_MSE_OPUS_SUPPORT_TYPE": SimTypeInt(signed=False, label="MF_MSE_OPUS_SUPPORT_TYPE"),
    "MF_MSE_READY": SimTypeInt(signed=False, label="MF_MSE_READY"),
    "MF_MSE_VP9_SUPPORT_TYPE": SimTypeInt(signed=False, label="MF_MSE_VP9_SUPPORT_TYPE"),
    "MF_MT_D3D_RESOURCE_VERSION_ENUM": SimTypeInt(signed=False, label="MF_MT_D3D_RESOURCE_VERSION_ENUM"),
    "MF_OBJECT_TYPE": SimTypeInt(signed=False, label="MF_OBJECT_TYPE"),
    "MF_OPM_ACP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="MF_OPM_ACP_PROTECTION_LEVEL"),
    "MF_OPM_CGMSA_PROTECTION_LEVEL": SimTypeInt(signed=False, label="MF_OPM_CGMSA_PROTECTION_LEVEL"),
    "MF_PLUGIN_CONTROL_POLICY": SimTypeInt(signed=False, label="MF_PLUGIN_CONTROL_POLICY"),
    "MF_Plugin_Type": SimTypeInt(signed=False, label="MF_Plugin_Type"),
    "MF_QUALITY_ADVISE_FLAGS": SimTypeInt(signed=False, label="MF_QUALITY_ADVISE_FLAGS"),
    "MF_QUALITY_DROP_MODE": SimTypeInt(signed=False, label="MF_QUALITY_DROP_MODE"),
    "MF_QUALITY_LEVEL": SimTypeInt(signed=False, label="MF_QUALITY_LEVEL"),
    "MF_QUATERNION": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("z", SimTypeFloat(size=32)), ("w", SimTypeFloat(size=32)),)), name="MF_QUATERNION", pack=False, align=None),
    "MF_RESOLUTION_FLAGS": SimTypeInt(signed=False, label="MF_RESOLUTION_FLAGS"),
    "MF_SERVICE_LOOKUP_TYPE": SimTypeInt(signed=False, label="MF_SERVICE_LOOKUP_TYPE"),
    "MF_SHARING_ENGINE_EVENT": SimTypeInt(signed=False, label="MF_SHARING_ENGINE_EVENT"),
    "MF_SINK_WRITER_CONSTANTS": SimTypeInt(signed=False, label="MF_SINK_WRITER_CONSTANTS"),
    "MF_SINK_WRITER_STATISTICS": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("llLastTimestampReceived", SimTypeLongLong(signed=True, label="Int64")), ("llLastTimestampEncoded", SimTypeLongLong(signed=True, label="Int64")), ("llLastTimestampProcessed", SimTypeLongLong(signed=True, label="Int64")), ("llLastStreamTickReceived", SimTypeLongLong(signed=True, label="Int64")), ("llLastSinkSampleRequest", SimTypeLongLong(signed=True, label="Int64")), ("qwNumSamplesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("qwNumSamplesEncoded", SimTypeLongLong(signed=False, label="UInt64")), ("qwNumSamplesProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("qwNumStreamTicksReceived", SimTypeLongLong(signed=False, label="UInt64")), ("dwByteCountQueued", SimTypeInt(signed=False, label="UInt32")), ("qwByteCountProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("dwNumOutstandingSinkSampleRequests", SimTypeInt(signed=False, label="UInt32")), ("dwAverageSampleRateReceived", SimTypeInt(signed=False, label="UInt32")), ("dwAverageSampleRateEncoded", SimTypeInt(signed=False, label="UInt32")), ("dwAverageSampleRateProcessed", SimTypeInt(signed=False, label="UInt32")),)), name="MF_SINK_WRITER_STATISTICS", pack=False, align=None),
    "MF_SOURCE_READER_CONSTANTS": SimTypeInt(signed=False, label="MF_SOURCE_READER_CONSTANTS"),
    "MF_SOURCE_READER_CONTROL_FLAG": SimTypeInt(signed=False, label="MF_SOURCE_READER_CONTROL_FLAG"),
    "MF_SOURCE_READER_CURRENT_TYPE_CONSTANTS": SimTypeInt(signed=False, label="MF_SOURCE_READER_CURRENT_TYPE_CONSTANTS"),
    "MF_SOURCE_READER_FLAG": SimTypeInt(signed=False, label="MF_SOURCE_READER_FLAG"),
    "MF_STREAM_STATE": SimTypeInt(signed=False, label="MF_STREAM_STATE"),
    "MF_TIMED_TEXT_ALIGNMENT": SimTypeInt(signed=False, label="MF_TIMED_TEXT_ALIGNMENT"),
    "MF_TIMED_TEXT_BOUTEN_POSITION": SimTypeInt(signed=False, label="MF_TIMED_TEXT_BOUTEN_POSITION"),
    "MF_TIMED_TEXT_BOUTEN_TYPE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_BOUTEN_TYPE"),
    "MF_TIMED_TEXT_CUE_EVENT": SimTypeInt(signed=False, label="MF_TIMED_TEXT_CUE_EVENT"),
    "MF_TIMED_TEXT_DECORATION": SimTypeInt(signed=False, label="MF_TIMED_TEXT_DECORATION"),
    "MF_TIMED_TEXT_DISPLAY_ALIGNMENT": SimTypeInt(signed=False, label="MF_TIMED_TEXT_DISPLAY_ALIGNMENT"),
    "MF_TIMED_TEXT_ERROR_CODE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_ERROR_CODE"),
    "MF_TIMED_TEXT_FONT_STYLE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_FONT_STYLE"),
    "MF_TIMED_TEXT_RUBY_ALIGN": SimTypeInt(signed=False, label="MF_TIMED_TEXT_RUBY_ALIGN"),
    "MF_TIMED_TEXT_RUBY_POSITION": SimTypeInt(signed=False, label="MF_TIMED_TEXT_RUBY_POSITION"),
    "MF_TIMED_TEXT_RUBY_RESERVE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_RUBY_RESERVE"),
    "MF_TIMED_TEXT_SCROLL_MODE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_SCROLL_MODE"),
    "MF_TIMED_TEXT_TRACK_KIND": SimTypeInt(signed=False, label="MF_TIMED_TEXT_TRACK_KIND"),
    "MF_TIMED_TEXT_TRACK_READY_STATE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_TRACK_READY_STATE"),
    "MF_TIMED_TEXT_UNIT_TYPE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_UNIT_TYPE"),
    "MF_TIMED_TEXT_WRITING_MODE": SimTypeInt(signed=False, label="MF_TIMED_TEXT_WRITING_MODE"),
    "MF_TOPOLOGY_RESOLUTION_STATUS_FLAGS": SimTypeInt(signed=False, label="MF_TOPOLOGY_RESOLUTION_STATUS_FLAGS"),
    "MF_TOPOLOGY_TYPE": SimTypeInt(signed=False, label="MF_TOPOLOGY_TYPE"),
    "MF_TOPONODE_DRAIN_MODE": SimTypeInt(signed=False, label="MF_TOPONODE_DRAIN_MODE"),
    "MF_TOPONODE_FLUSH_MODE": SimTypeInt(signed=False, label="MF_TOPONODE_FLUSH_MODE"),
    "MF_TOPOSTATUS": SimTypeInt(signed=False, label="MF_TOPOSTATUS"),
    "MF_TRANSCODE_ADJUST_PROFILE_FLAGS": SimTypeInt(signed=False, label="MF_TRANSCODE_ADJUST_PROFILE_FLAGS"),
    "MF_TRANSCODE_SINK_INFO": SimStruct(OrderedDict((("dwVideoStreamID", SimTypeInt(signed=False, label="UInt32")), ("pVideoMediaType", SimTypeBottom(label="IMFMediaType")), ("dwAudioStreamID", SimTypeInt(signed=False, label="UInt32")), ("pAudioMediaType", SimTypeBottom(label="IMFMediaType")),)), name="MF_TRANSCODE_SINK_INFO", pack=False, align=None),
    "MF_TRANSCODE_TOPOLOGYMODE_FLAGS": SimTypeInt(signed=False, label="MF_TRANSCODE_TOPOLOGYMODE_FLAGS"),
    "MF_TRANSFER_VIDEO_FRAME_FLAGS": SimTypeInt(signed=False, label="MF_TRANSFER_VIDEO_FRAME_FLAGS"),
    "MF_URL_TRUST_STATUS": SimTypeInt(signed=False, label="MF_URL_TRUST_STATUS"),
    "MF_VIDEO_PROCESSOR_ALGORITHM_TYPE": SimTypeInt(signed=False, label="MF_VIDEO_PROCESSOR_ALGORITHM_TYPE"),
    "MF_VIDEO_PROCESSOR_MIRROR": SimTypeInt(signed=False, label="MF_VIDEO_PROCESSOR_MIRROR"),
    "MF_VIDEO_PROCESSOR_ROTATION": SimTypeInt(signed=False, label="MF_VIDEO_PROCESSOR_ROTATION"),
    "MF_VIDEO_SPHERICAL_VIEWDIRECTION": SimStruct(OrderedDict((("iHeading", SimTypeInt(signed=True, label="Int32")), ("iPitch", SimTypeInt(signed=True, label="Int32")), ("iRoll", SimTypeInt(signed=True, label="Int32")),)), name="MF_VIDEO_SPHERICAL_VIEWDIRECTION", pack=False, align=None),
    "MGM_ENUM_TYPES": SimTypeInt(signed=False, label="MGM_ENUM_TYPES"),
    "MGM_IF_ENTRY": SimStruct(OrderedDict((("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwIfNextHopAddr", SimTypeInt(signed=False, label="UInt32")), ("bIGMP", SimTypeInt(signed=True, label="Int32")), ("bIsEnabled", SimTypeInt(signed=True, label="Int32")),)), name="MGM_IF_ENTRY", pack=False, align=None),
    "MIBICMPINFO": SimStruct(OrderedDict((("icmpInStats", SimTypeRef("MIBICMPSTATS", SimStruct)), ("icmpOutStats", SimTypeRef("MIBICMPSTATS", SimStruct)),)), name="MIBICMPINFO", pack=False, align=None),
    "MIBICMPSTATS": SimStruct(OrderedDict((("dwMsgs", SimTypeInt(signed=False, label="UInt32")), ("dwErrors", SimTypeInt(signed=False, label="UInt32")), ("dwDestUnreachs", SimTypeInt(signed=False, label="UInt32")), ("dwTimeExcds", SimTypeInt(signed=False, label="UInt32")), ("dwParmProbs", SimTypeInt(signed=False, label="UInt32")), ("dwSrcQuenchs", SimTypeInt(signed=False, label="UInt32")), ("dwRedirects", SimTypeInt(signed=False, label="UInt32")), ("dwEchos", SimTypeInt(signed=False, label="UInt32")), ("dwEchoReps", SimTypeInt(signed=False, label="UInt32")), ("dwTimestamps", SimTypeInt(signed=False, label="UInt32")), ("dwTimestampReps", SimTypeInt(signed=False, label="UInt32")), ("dwAddrMasks", SimTypeInt(signed=False, label="UInt32")), ("dwAddrMaskReps", SimTypeInt(signed=False, label="UInt32")),)), name="MIBICMPSTATS", pack=False, align=None),
    "MIBICMPSTATS_EX_XPSP1": SimStruct(OrderedDict((("dwMsgs", SimTypeInt(signed=False, label="UInt32")), ("dwErrors", SimTypeInt(signed=False, label="UInt32")), ("rgdwTypeCount", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 256)),)), name="MIBICMPSTATS_EX_XPSP1", pack=False, align=None),
    "MIB_ANYCASTIPADDRESS_ROW": SimStruct(OrderedDict((("Address", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("ScopeId", SimTypeRef("SCOPE_ID", SimStruct)),)), name="MIB_ANYCASTIPADDRESS_ROW", pack=False, align=None),
    "MIB_ANYCASTIPADDRESS_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_ANYCASTIPADDRESS_ROW", SimStruct), offset=0)),)), name="MIB_ANYCASTIPADDRESS_TABLE", pack=False, align=None),
    "MIB_BEST_IF": SimStruct(OrderedDict((("dwDestAddr", SimTypeInt(signed=False, label="UInt32")), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_BEST_IF", pack=False, align=None),
    "MIB_BOUNDARYROW": SimStruct(OrderedDict((("dwGroupAddress", SimTypeInt(signed=False, label="UInt32")), ("dwGroupMask", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_BOUNDARYROW", pack=False, align=None),
    "MIB_ICMP": SimStruct(OrderedDict((("stats", SimTypeRef("MIBICMPINFO", SimStruct)),)), name="MIB_ICMP", pack=False, align=None),
    "MIB_ICMP_EX_XPSP1": SimStruct(OrderedDict((("icmpInStats", SimTypeRef("MIBICMPSTATS_EX_XPSP1", SimStruct)), ("icmpOutStats", SimTypeRef("MIBICMPSTATS_EX_XPSP1", SimStruct)),)), name="MIB_ICMP_EX_XPSP1", pack=False, align=None),
    "MIB_IFNUMBER": SimStruct(OrderedDict((("dwValue", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IFNUMBER", pack=False, align=None),
    "MIB_IFROW": SimStruct(OrderedDict((("wszName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwMtu", SimTypeInt(signed=False, label="UInt32")), ("dwSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwPhysAddrLen", SimTypeInt(signed=False, label="UInt32")), ("bPhysAddr", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwAdminStatus", SimTypeInt(signed=False, label="UInt32")), ("dwOperStatus", SimTypeInt(signed=False, label="INTERNAL_IF_OPER_STATUS")), ("dwLastChange", SimTypeInt(signed=False, label="UInt32")), ("dwInOctets", SimTypeInt(signed=False, label="UInt32")), ("dwInUcastPkts", SimTypeInt(signed=False, label="UInt32")), ("dwInNUcastPkts", SimTypeInt(signed=False, label="UInt32")), ("dwInDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwInErrors", SimTypeInt(signed=False, label="UInt32")), ("dwInUnknownProtos", SimTypeInt(signed=False, label="UInt32")), ("dwOutOctets", SimTypeInt(signed=False, label="UInt32")), ("dwOutUcastPkts", SimTypeInt(signed=False, label="UInt32")), ("dwOutNUcastPkts", SimTypeInt(signed=False, label="UInt32")), ("dwOutDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwOutErrors", SimTypeInt(signed=False, label="UInt32")), ("dwOutQLen", SimTypeInt(signed=False, label="UInt32")), ("dwDescrLen", SimTypeInt(signed=False, label="UInt32")), ("bDescr", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="MIB_IFROW", pack=False, align=None),
    "MIB_IFSTACK_ROW": SimStruct(OrderedDict((("HigherLayerInterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("LowerLayerInterfaceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IFSTACK_ROW", pack=False, align=None),
    "MIB_IFSTACK_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_IFSTACK_ROW", SimStruct), offset=0)),)), name="MIB_IFSTACK_TABLE", pack=False, align=None),
    "MIB_IFSTATUS": SimStruct(OrderedDict((("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwAdminStatus", SimTypeInt(signed=False, label="UInt32")), ("dwOperationalStatus", SimTypeInt(signed=False, label="UInt32")), ("bMHbeatActive", SimTypeInt(signed=True, label="Int32")), ("bMHbeatAlive", SimTypeInt(signed=True, label="Int32")),)), name="MIB_IFSTATUS", pack=False, align=None),
    "MIB_IFTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IFROW", SimStruct), offset=0)),)), name="MIB_IFTABLE", pack=False, align=None),
    "MIB_IF_ENTRY_LEVEL": SimTypeInt(signed=False, label="MIB_IF_ENTRY_LEVEL"),
    "MIB_IF_ROW2": SimStruct(OrderedDict((("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("InterfaceGuid", SimTypeBottom(label="Guid")), ("Alias", SimTypeArray(SimTypeChar(label="Char"), 257)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 257)), ("PhysicalAddressLength", SimTypeInt(signed=False, label="UInt32")), ("PhysicalAddress", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("PermanentPhysicalAddress", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("Mtu", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("TunnelType", SimTypeInt(signed=False, label="TUNNEL_TYPE")), ("MediaType", SimTypeInt(signed=False, label="NDIS_MEDIUM")), ("PhysicalMediumType", SimTypeInt(signed=False, label="NDIS_PHYSICAL_MEDIUM")), ("AccessType", SimTypeInt(signed=False, label="NET_IF_ACCESS_TYPE")), ("DirectionType", SimTypeInt(signed=False, label="NET_IF_DIRECTION_TYPE")), ("InterfaceAndOperStatusFlags", SimTypeRef("_InterfaceAndOperStatusFlags_e__Struct", SimStruct)), ("OperStatus", SimTypeInt(signed=False, label="IF_OPER_STATUS")), ("AdminStatus", SimTypeInt(signed=False, label="NET_IF_ADMIN_STATUS")), ("MediaConnectState", SimTypeInt(signed=False, label="NET_IF_MEDIA_CONNECT_STATE")), ("NetworkGuid", SimTypeBottom(label="Guid")), ("ConnectionType", SimTypeInt(signed=False, label="NET_IF_CONNECTION_TYPE")), ("TransmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("ReceiveLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("InOctets", SimTypeLongLong(signed=False, label="UInt64")), ("InUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("InNUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("InDiscards", SimTypeLongLong(signed=False, label="UInt64")), ("InErrors", SimTypeLongLong(signed=False, label="UInt64")), ("InUnknownProtos", SimTypeLongLong(signed=False, label="UInt64")), ("InUcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("InMulticastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("InBroadcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("OutOctets", SimTypeLongLong(signed=False, label="UInt64")), ("OutUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("OutNUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("OutDiscards", SimTypeLongLong(signed=False, label="UInt64")), ("OutErrors", SimTypeLongLong(signed=False, label="UInt64")), ("OutUcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("OutMulticastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("OutBroadcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("OutQLen", SimTypeLongLong(signed=False, label="UInt64")),)), name="MIB_IF_ROW2", pack=False, align=None),
    "MIB_IF_TABLE2": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_IF_ROW2", SimStruct), offset=0)),)), name="MIB_IF_TABLE2", pack=False, align=None),
    "MIB_IF_TABLE_LEVEL": SimTypeInt(signed=False, label="MIB_IF_TABLE_LEVEL"),
    "MIB_INVERTEDIFSTACK_ROW": SimStruct(OrderedDict((("LowerLayerInterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("HigherLayerInterfaceIndex", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_INVERTEDIFSTACK_ROW", pack=False, align=None),
    "MIB_INVERTEDIFSTACK_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_INVERTEDIFSTACK_ROW", SimStruct), offset=0)),)), name="MIB_INVERTEDIFSTACK_TABLE", pack=False, align=None),
    "MIB_IPADDRROW_W2K": SimStruct(OrderedDict((("dwAddr", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwBCastAddr", SimTypeInt(signed=False, label="UInt32")), ("dwReasmSize", SimTypeInt(signed=False, label="UInt32")), ("unused1", SimTypeShort(signed=False, label="UInt16")), ("unused2", SimTypeShort(signed=False, label="UInt16")),)), name="MIB_IPADDRROW_W2K", pack=False, align=None),
    "MIB_IPADDRROW_XP": SimStruct(OrderedDict((("dwAddr", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwBCastAddr", SimTypeInt(signed=False, label="UInt32")), ("dwReasmSize", SimTypeInt(signed=False, label="UInt32")), ("unused1", SimTypeShort(signed=False, label="UInt16")), ("wType", SimTypeShort(signed=False, label="UInt16")),)), name="MIB_IPADDRROW_XP", pack=False, align=None),
    "MIB_IPADDRTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPADDRROW_XP", SimStruct), offset=0)),)), name="MIB_IPADDRTABLE", pack=False, align=None),
    "MIB_IPDESTROW": SimStruct(OrderedDict((("ForwardRow", SimTypeRef("MIB_IPFORWARDROW", SimStruct)), ("dwForwardPreference", SimTypeInt(signed=False, label="UInt32")), ("dwForwardViewSet", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPDESTROW", pack=False, align=None),
    "MIB_IPDESTTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPDESTROW", SimStruct), offset=0)),)), name="MIB_IPDESTTABLE", pack=False, align=None),
    "MIB_IPFORWARDNUMBER": SimStruct(OrderedDict((("dwValue", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPFORWARDNUMBER", pack=False, align=None),
    "MIB_IPFORWARDROW": SimStruct(OrderedDict((("dwForwardDest", SimTypeInt(signed=False, label="UInt32")), ("dwForwardMask", SimTypeInt(signed=False, label="UInt32")), ("dwForwardPolicy", SimTypeInt(signed=False, label="UInt32")), ("dwForwardNextHop", SimTypeInt(signed=False, label="UInt32")), ("dwForwardIfIndex", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"dwForwardType": SimTypeInt(signed=False, label="UInt32"), "ForwardType": SimTypeInt(signed=False, label="MIB_IPFORWARD_TYPE")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"dwForwardProto": SimTypeInt(signed=False, label="UInt32"), "ForwardProto": SimTypeInt(signed=False, label="NL_ROUTE_PROTOCOL")}, name="<anon>", label="None")), ("dwForwardAge", SimTypeInt(signed=False, label="UInt32")), ("dwForwardNextHopAS", SimTypeInt(signed=False, label="UInt32")), ("dwForwardMetric1", SimTypeInt(signed=False, label="UInt32")), ("dwForwardMetric2", SimTypeInt(signed=False, label="UInt32")), ("dwForwardMetric3", SimTypeInt(signed=False, label="UInt32")), ("dwForwardMetric4", SimTypeInt(signed=False, label="UInt32")), ("dwForwardMetric5", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPFORWARDROW", pack=False, align=None),
    "MIB_IPFORWARDTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPFORWARDROW", SimStruct), offset=0)),)), name="MIB_IPFORWARDTABLE", pack=False, align=None),
    "MIB_IPFORWARD_ROW2": SimStruct(OrderedDict((("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("DestinationPrefix", SimTypeRef("IP_ADDRESS_PREFIX", SimStruct)), ("NextHop", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("SitePrefixLength", SimTypeChar(label="Byte")), ("ValidLifetime", SimTypeInt(signed=False, label="UInt32")), ("PreferredLifetime", SimTypeInt(signed=False, label="UInt32")), ("Metric", SimTypeInt(signed=False, label="UInt32")), ("Protocol", SimTypeInt(signed=False, label="NL_ROUTE_PROTOCOL")), ("Loopback", SimTypeChar(label="Byte")), ("AutoconfigureAddress", SimTypeChar(label="Byte")), ("Publish", SimTypeChar(label="Byte")), ("Immortal", SimTypeChar(label="Byte")), ("Age", SimTypeInt(signed=False, label="UInt32")), ("Origin", SimTypeInt(signed=False, label="NL_ROUTE_ORIGIN")),)), name="MIB_IPFORWARD_ROW2", pack=False, align=None),
    "MIB_IPFORWARD_TABLE2": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_IPFORWARD_ROW2", SimStruct), offset=0)),)), name="MIB_IPFORWARD_TABLE2", pack=False, align=None),
    "MIB_IPFORWARD_TYPE": SimTypeInt(signed=False, label="MIB_IPFORWARD_TYPE"),
    "MIB_IPINTERFACE_ROW": SimStruct(OrderedDict((("Family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("MaxReassemblySize", SimTypeInt(signed=False, label="UInt32")), ("InterfaceIdentifier", SimTypeLongLong(signed=False, label="UInt64")), ("MinRouterAdvertisementInterval", SimTypeInt(signed=False, label="UInt32")), ("MaxRouterAdvertisementInterval", SimTypeInt(signed=False, label="UInt32")), ("AdvertisingEnabled", SimTypeChar(label="Byte")), ("ForwardingEnabled", SimTypeChar(label="Byte")), ("WeakHostSend", SimTypeChar(label="Byte")), ("WeakHostReceive", SimTypeChar(label="Byte")), ("UseAutomaticMetric", SimTypeChar(label="Byte")), ("UseNeighborUnreachabilityDetection", SimTypeChar(label="Byte")), ("ManagedAddressConfigurationSupported", SimTypeChar(label="Byte")), ("OtherStatefulConfigurationSupported", SimTypeChar(label="Byte")), ("AdvertiseDefaultRoute", SimTypeChar(label="Byte")), ("RouterDiscoveryBehavior", SimTypeInt(signed=False, label="NL_ROUTER_DISCOVERY_BEHAVIOR")), ("DadTransmits", SimTypeInt(signed=False, label="UInt32")), ("BaseReachableTime", SimTypeInt(signed=False, label="UInt32")), ("RetransmitTime", SimTypeInt(signed=False, label="UInt32")), ("PathMtuDiscoveryTimeout", SimTypeInt(signed=False, label="UInt32")), ("LinkLocalAddressBehavior", SimTypeInt(signed=False, label="NL_LINK_LOCAL_ADDRESS_BEHAVIOR")), ("LinkLocalAddressTimeout", SimTypeInt(signed=False, label="UInt32")), ("ZoneIndices", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)), ("SitePrefixLength", SimTypeInt(signed=False, label="UInt32")), ("Metric", SimTypeInt(signed=False, label="UInt32")), ("NlMtu", SimTypeInt(signed=False, label="UInt32")), ("Connected", SimTypeChar(label="Byte")), ("SupportsWakeUpPatterns", SimTypeChar(label="Byte")), ("SupportsNeighborDiscovery", SimTypeChar(label="Byte")), ("SupportsRouterDiscovery", SimTypeChar(label="Byte")), ("ReachableTime", SimTypeInt(signed=False, label="UInt32")), ("TransmitOffload", SimTypeRef("NL_INTERFACE_OFFLOAD_ROD", SimStruct)), ("ReceiveOffload", SimTypeRef("NL_INTERFACE_OFFLOAD_ROD", SimStruct)), ("DisableDefaultRoutes", SimTypeChar(label="Byte")),)), name="MIB_IPINTERFACE_ROW", pack=False, align=None),
    "MIB_IPINTERFACE_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_IPINTERFACE_ROW", SimStruct), offset=0)),)), name="MIB_IPINTERFACE_TABLE", pack=False, align=None),
    "MIB_IPMCAST_BOUNDARY": SimStruct(OrderedDict((("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwGroupAddress", SimTypeInt(signed=False, label="UInt32")), ("dwGroupMask", SimTypeInt(signed=False, label="UInt32")), ("dwStatus", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_BOUNDARY", pack=False, align=None),
    "MIB_IPMCAST_BOUNDARY_TABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPMCAST_BOUNDARY", SimStruct), offset=0)),)), name="MIB_IPMCAST_BOUNDARY_TABLE", pack=False, align=None),
    "MIB_IPMCAST_GLOBAL": SimStruct(OrderedDict((("dwEnable", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_GLOBAL", pack=False, align=None),
    "MIB_IPMCAST_IF_ENTRY": SimStruct(OrderedDict((("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwTtl", SimTypeInt(signed=False, label="UInt32")), ("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRateLimit", SimTypeInt(signed=False, label="UInt32")), ("ulInMcastOctets", SimTypeInt(signed=False, label="UInt32")), ("ulOutMcastOctets", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_IF_ENTRY", pack=False, align=None),
    "MIB_IPMCAST_IF_TABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPMCAST_IF_ENTRY", SimStruct), offset=0)),)), name="MIB_IPMCAST_IF_TABLE", pack=False, align=None),
    "MIB_IPMCAST_MFE": SimStruct(OrderedDict((("dwGroup", SimTypeInt(signed=False, label="UInt32")), ("dwSource", SimTypeInt(signed=False, label="UInt32")), ("dwSrcMask", SimTypeInt(signed=False, label="UInt32")), ("dwUpStrmNgbr", SimTypeInt(signed=False, label="UInt32")), ("dwInIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwInIfProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRouteProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRouteNetwork", SimTypeInt(signed=False, label="UInt32")), ("dwRouteMask", SimTypeInt(signed=False, label="UInt32")), ("ulUpTime", SimTypeInt(signed=False, label="UInt32")), ("ulExpiryTime", SimTypeInt(signed=False, label="UInt32")), ("ulTimeOut", SimTypeInt(signed=False, label="UInt32")), ("ulNumOutIf", SimTypeInt(signed=False, label="UInt32")), ("fFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("rgmioOutInfo", SimTypePointer(SimTypeRef("MIB_IPMCAST_OIF_XP", SimStruct), offset=0)),)), name="MIB_IPMCAST_MFE", pack=False, align=None),
    "MIB_IPMCAST_MFE_STATS": SimStruct(OrderedDict((("dwGroup", SimTypeInt(signed=False, label="UInt32")), ("dwSource", SimTypeInt(signed=False, label="UInt32")), ("dwSrcMask", SimTypeInt(signed=False, label="UInt32")), ("dwUpStrmNgbr", SimTypeInt(signed=False, label="UInt32")), ("dwInIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwInIfProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRouteProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRouteNetwork", SimTypeInt(signed=False, label="UInt32")), ("dwRouteMask", SimTypeInt(signed=False, label="UInt32")), ("ulUpTime", SimTypeInt(signed=False, label="UInt32")), ("ulExpiryTime", SimTypeInt(signed=False, label="UInt32")), ("ulNumOutIf", SimTypeInt(signed=False, label="UInt32")), ("ulInPkts", SimTypeInt(signed=False, label="UInt32")), ("ulInOctets", SimTypeInt(signed=False, label="UInt32")), ("ulPktsDifferentIf", SimTypeInt(signed=False, label="UInt32")), ("ulQueueOverflow", SimTypeInt(signed=False, label="UInt32")), ("rgmiosOutStats", SimTypePointer(SimTypeRef("MIB_IPMCAST_OIF_STATS_LH", SimStruct), offset=0)),)), name="MIB_IPMCAST_MFE_STATS", pack=False, align=None),
    "MIB_IPMCAST_MFE_STATS_EX_XP": SimStruct(OrderedDict((("dwGroup", SimTypeInt(signed=False, label="UInt32")), ("dwSource", SimTypeInt(signed=False, label="UInt32")), ("dwSrcMask", SimTypeInt(signed=False, label="UInt32")), ("dwUpStrmNgbr", SimTypeInt(signed=False, label="UInt32")), ("dwInIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwInIfProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRouteProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRouteNetwork", SimTypeInt(signed=False, label="UInt32")), ("dwRouteMask", SimTypeInt(signed=False, label="UInt32")), ("ulUpTime", SimTypeInt(signed=False, label="UInt32")), ("ulExpiryTime", SimTypeInt(signed=False, label="UInt32")), ("ulNumOutIf", SimTypeInt(signed=False, label="UInt32")), ("ulInPkts", SimTypeInt(signed=False, label="UInt32")), ("ulInOctets", SimTypeInt(signed=False, label="UInt32")), ("ulPktsDifferentIf", SimTypeInt(signed=False, label="UInt32")), ("ulQueueOverflow", SimTypeInt(signed=False, label="UInt32")), ("ulUninitMfe", SimTypeInt(signed=False, label="UInt32")), ("ulNegativeMfe", SimTypeInt(signed=False, label="UInt32")), ("ulInDiscards", SimTypeInt(signed=False, label="UInt32")), ("ulInHdrErrors", SimTypeInt(signed=False, label="UInt32")), ("ulTotalOutPackets", SimTypeInt(signed=False, label="UInt32")), ("rgmiosOutStats", SimTypePointer(SimTypeRef("MIB_IPMCAST_OIF_STATS_LH", SimStruct), offset=0)),)), name="MIB_IPMCAST_MFE_STATS_EX_XP", pack=False, align=None),
    "MIB_IPMCAST_OIF_STATS_LH": SimStruct(OrderedDict((("dwOutIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwNextHopAddr", SimTypeInt(signed=False, label="UInt32")), ("dwDialContext", SimTypeInt(signed=False, label="UInt32")), ("ulTtlTooLow", SimTypeInt(signed=False, label="UInt32")), ("ulFragNeeded", SimTypeInt(signed=False, label="UInt32")), ("ulOutPackets", SimTypeInt(signed=False, label="UInt32")), ("ulOutDiscards", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_OIF_STATS_LH", pack=False, align=None),
    "MIB_IPMCAST_OIF_STATS_W2K": SimStruct(OrderedDict((("dwOutIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwNextHopAddr", SimTypeInt(signed=False, label="UInt32")), ("pvDialContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulTtlTooLow", SimTypeInt(signed=False, label="UInt32")), ("ulFragNeeded", SimTypeInt(signed=False, label="UInt32")), ("ulOutPackets", SimTypeInt(signed=False, label="UInt32")), ("ulOutDiscards", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_OIF_STATS_W2K", pack=False, align=None),
    "MIB_IPMCAST_OIF_W2K": SimStruct(OrderedDict((("dwOutIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwNextHopAddr", SimTypeInt(signed=False, label="UInt32")), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_OIF_W2K", pack=False, align=None),
    "MIB_IPMCAST_OIF_XP": SimStruct(OrderedDict((("dwOutIfIndex", SimTypeInt(signed=False, label="UInt32")), ("dwNextHopAddr", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_OIF_XP", pack=False, align=None),
    "MIB_IPMCAST_SCOPE": SimStruct(OrderedDict((("dwGroupAddress", SimTypeInt(signed=False, label="UInt32")), ("dwGroupMask", SimTypeInt(signed=False, label="UInt32")), ("snNameBuffer", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("dwStatus", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPMCAST_SCOPE", pack=False, align=None),
    "MIB_IPNETROW_LH": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwPhysAddrLen", SimTypeInt(signed=False, label="UInt32")), ("bPhysAddr", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwAddr", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwType": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="MIB_IPNET_TYPE")}, name="<anon>", label="None")),)), name="MIB_IPNETROW_LH", pack=False, align=None),
    "MIB_IPNETROW_W2K": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwPhysAddrLen", SimTypeInt(signed=False, label="UInt32")), ("bPhysAddr", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwAddr", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPNETROW_W2K", pack=False, align=None),
    "MIB_IPNETTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPNETROW_LH", SimStruct), offset=0)),)), name="MIB_IPNETTABLE", pack=False, align=None),
    "MIB_IPNET_ROW2": SimStruct(OrderedDict((("Address", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("PhysicalAddress", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("PhysicalAddressLength", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="NL_NEIGHBOR_STATE")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Flags": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("ReachabilityTime", SimUnion({"LastReachable": SimTypeInt(signed=False, label="UInt32"), "LastUnreachable": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="MIB_IPNET_ROW2", pack=False, align=None),
    "MIB_IPNET_TABLE2": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_IPNET_ROW2", SimStruct), offset=0)),)), name="MIB_IPNET_TABLE2", pack=False, align=None),
    "MIB_IPNET_TYPE": SimTypeInt(signed=False, label="MIB_IPNET_TYPE"),
    "MIB_IPPATH_ROW": SimStruct(OrderedDict((("Source", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("Destination", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("CurrentNextHop", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("PathMtu", SimTypeInt(signed=False, label="UInt32")), ("RttMean", SimTypeInt(signed=False, label="UInt32")), ("RttDeviation", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"LastReachable": SimTypeInt(signed=False, label="UInt32"), "LastUnreachable": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("IsReachable", SimTypeChar(label="Byte")), ("LinkTransmitSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("LinkReceiveSpeed", SimTypeLongLong(signed=False, label="UInt64")),)), name="MIB_IPPATH_ROW", pack=False, align=None),
    "MIB_IPPATH_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_IPPATH_ROW", SimStruct), offset=0)),)), name="MIB_IPPATH_TABLE", pack=False, align=None),
    "MIB_IPSTATS_FORWARDING": SimTypeInt(signed=False, label="MIB_IPSTATS_FORWARDING"),
    "MIB_IPSTATS_LH": SimStruct(OrderedDict((("Anonymous", SimUnion({"dwForwarding": SimTypeInt(signed=False, label="UInt32"), "Forwarding": SimTypeInt(signed=False, label="MIB_IPSTATS_FORWARDING")}, name="<anon>", label="None")), ("dwDefaultTTL", SimTypeInt(signed=False, label="UInt32")), ("dwInReceives", SimTypeInt(signed=False, label="UInt32")), ("dwInHdrErrors", SimTypeInt(signed=False, label="UInt32")), ("dwInAddrErrors", SimTypeInt(signed=False, label="UInt32")), ("dwForwDatagrams", SimTypeInt(signed=False, label="UInt32")), ("dwInUnknownProtos", SimTypeInt(signed=False, label="UInt32")), ("dwInDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwInDelivers", SimTypeInt(signed=False, label="UInt32")), ("dwOutRequests", SimTypeInt(signed=False, label="UInt32")), ("dwRoutingDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwOutDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwOutNoRoutes", SimTypeInt(signed=False, label="UInt32")), ("dwReasmTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwReasmReqds", SimTypeInt(signed=False, label="UInt32")), ("dwReasmOks", SimTypeInt(signed=False, label="UInt32")), ("dwReasmFails", SimTypeInt(signed=False, label="UInt32")), ("dwFragOks", SimTypeInt(signed=False, label="UInt32")), ("dwFragFails", SimTypeInt(signed=False, label="UInt32")), ("dwFragCreates", SimTypeInt(signed=False, label="UInt32")), ("dwNumIf", SimTypeInt(signed=False, label="UInt32")), ("dwNumAddr", SimTypeInt(signed=False, label="UInt32")), ("dwNumRoutes", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPSTATS_LH", pack=False, align=None),
    "MIB_IPSTATS_W2K": SimStruct(OrderedDict((("dwForwarding", SimTypeInt(signed=False, label="UInt32")), ("dwDefaultTTL", SimTypeInt(signed=False, label="UInt32")), ("dwInReceives", SimTypeInt(signed=False, label="UInt32")), ("dwInHdrErrors", SimTypeInt(signed=False, label="UInt32")), ("dwInAddrErrors", SimTypeInt(signed=False, label="UInt32")), ("dwForwDatagrams", SimTypeInt(signed=False, label="UInt32")), ("dwInUnknownProtos", SimTypeInt(signed=False, label="UInt32")), ("dwInDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwInDelivers", SimTypeInt(signed=False, label="UInt32")), ("dwOutRequests", SimTypeInt(signed=False, label="UInt32")), ("dwRoutingDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwOutDiscards", SimTypeInt(signed=False, label="UInt32")), ("dwOutNoRoutes", SimTypeInt(signed=False, label="UInt32")), ("dwReasmTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwReasmReqds", SimTypeInt(signed=False, label="UInt32")), ("dwReasmOks", SimTypeInt(signed=False, label="UInt32")), ("dwReasmFails", SimTypeInt(signed=False, label="UInt32")), ("dwFragOks", SimTypeInt(signed=False, label="UInt32")), ("dwFragFails", SimTypeInt(signed=False, label="UInt32")), ("dwFragCreates", SimTypeInt(signed=False, label="UInt32")), ("dwNumIf", SimTypeInt(signed=False, label="UInt32")), ("dwNumAddr", SimTypeInt(signed=False, label="UInt32")), ("dwNumRoutes", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_IPSTATS_W2K", pack=False, align=None),
    "MIB_IP_NETWORK_CONNECTION_BANDWIDTH_ESTIMATES": SimStruct(OrderedDict((("InboundBandwidthInformation", SimTypeRef("NL_BANDWIDTH_INFORMATION", SimStruct)), ("OutboundBandwidthInformation", SimTypeRef("NL_BANDWIDTH_INFORMATION", SimStruct)),)), name="MIB_IP_NETWORK_CONNECTION_BANDWIDTH_ESTIMATES", pack=False, align=None),
    "MIB_MCAST_LIMIT_ROW": SimStruct(OrderedDict((("dwTtl", SimTypeInt(signed=False, label="UInt32")), ("dwRateLimit", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_MCAST_LIMIT_ROW", pack=False, align=None),
    "MIB_MFE_STATS_TABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPMCAST_MFE_STATS", SimStruct), offset=0)),)), name="MIB_MFE_STATS_TABLE", pack=False, align=None),
    "MIB_MFE_STATS_TABLE_EX_XP": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypePointer(SimTypeRef("MIB_IPMCAST_MFE_STATS_EX_XP", SimStruct), offset=0), offset=0)),)), name="MIB_MFE_STATS_TABLE_EX_XP", pack=False, align=None),
    "MIB_MFE_TABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_IPMCAST_MFE", SimStruct), offset=0)),)), name="MIB_MFE_TABLE", pack=False, align=None),
    "MIB_MULTICASTIPADDRESS_ROW": SimStruct(OrderedDict((("Address", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ScopeId", SimTypeRef("SCOPE_ID", SimStruct)),)), name="MIB_MULTICASTIPADDRESS_ROW", pack=False, align=None),
    "MIB_MULTICASTIPADDRESS_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_MULTICASTIPADDRESS_ROW", SimStruct), offset=0)),)), name="MIB_MULTICASTIPADDRESS_TABLE", pack=False, align=None),
    "MIB_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="MIB_NOTIFICATION_TYPE"),
    "MIB_OPAQUE_INFO": SimStruct(OrderedDict((("dwId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ullAlign": SimTypeLongLong(signed=False, label="UInt64"), "rgbyData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="MIB_OPAQUE_INFO", pack=False, align=None),
    "MIB_OPAQUE_QUERY": SimStruct(OrderedDict((("dwVarId", SimTypeInt(signed=False, label="UInt32")), ("rgdwVarIndex", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="MIB_OPAQUE_QUERY", pack=False, align=None),
    "MIB_PROXYARP": SimStruct(OrderedDict((("dwAddress", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_PROXYARP", pack=False, align=None),
    "MIB_ROUTESTATE": SimStruct(OrderedDict((("bRoutesSetToStack", SimTypeInt(signed=True, label="Int32")),)), name="MIB_ROUTESTATE", pack=False, align=None),
    "MIB_TCP6ROW": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="MIB_TCP_STATE")), ("LocalAddr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("RemoteAddr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwRemoteScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCP6ROW", pack=False, align=None),
    "MIB_TCP6ROW2": SimStruct(OrderedDict((("LocalAddr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("RemoteAddr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwRemoteScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="MIB_TCP_STATE")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("dwOffloadState", SimTypeInt(signed=False, label="TCP_CONNECTION_OFFLOAD_STATE")),)), name="MIB_TCP6ROW2", pack=False, align=None),
    "MIB_TCP6ROW_OWNER_MODULE": SimStruct(OrderedDict((("ucLocalAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("ucRemoteAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwRemoteScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("liCreateTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("OwningModuleInfo", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)),)), name="MIB_TCP6ROW_OWNER_MODULE", pack=False, align=None),
    "MIB_TCP6ROW_OWNER_PID": SimStruct(OrderedDict((("ucLocalAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("ucRemoteAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwRemoteScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCP6ROW_OWNER_PID", pack=False, align=None),
    "MIB_TCP6TABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCP6ROW", SimStruct), offset=0)),)), name="MIB_TCP6TABLE", pack=False, align=None),
    "MIB_TCP6TABLE2": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCP6ROW2", SimStruct), offset=0)),)), name="MIB_TCP6TABLE2", pack=False, align=None),
    "MIB_TCP6TABLE_OWNER_MODULE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCP6ROW_OWNER_MODULE", SimStruct), offset=0)),)), name="MIB_TCP6TABLE_OWNER_MODULE", pack=False, align=None),
    "MIB_TCP6TABLE_OWNER_PID": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCP6ROW_OWNER_PID", SimStruct), offset=0)),)), name="MIB_TCP6TABLE_OWNER_PID", pack=False, align=None),
    "MIB_TCPROW2": SimStruct(OrderedDict((("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteAddr", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("dwOffloadState", SimTypeInt(signed=False, label="TCP_CONNECTION_OFFLOAD_STATE")),)), name="MIB_TCPROW2", pack=False, align=None),
    "MIB_TCPROW_LH": SimStruct(OrderedDict((("Anonymous", SimUnion({"dwState": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="MIB_TCP_STATE")}, name="<anon>", label="None")), ("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteAddr", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCPROW_LH", pack=False, align=None),
    "MIB_TCPROW_OWNER_MODULE": SimStruct(OrderedDict((("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteAddr", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("liCreateTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("OwningModuleInfo", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)),)), name="MIB_TCPROW_OWNER_MODULE", pack=False, align=None),
    "MIB_TCPROW_OWNER_PID": SimStruct(OrderedDict((("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteAddr", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCPROW_OWNER_PID", pack=False, align=None),
    "MIB_TCPROW_W2K": SimStruct(OrderedDict((("dwState", SimTypeInt(signed=False, label="UInt32")), ("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteAddr", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCPROW_W2K", pack=False, align=None),
    "MIB_TCPSTATS2": SimStruct(OrderedDict((("RtoAlgorithm", SimTypeInt(signed=False, label="TCP_RTO_ALGORITHM")), ("dwRtoMin", SimTypeInt(signed=False, label="UInt32")), ("dwRtoMax", SimTypeInt(signed=False, label="UInt32")), ("dwMaxConn", SimTypeInt(signed=False, label="UInt32")), ("dwActiveOpens", SimTypeInt(signed=False, label="UInt32")), ("dwPassiveOpens", SimTypeInt(signed=False, label="UInt32")), ("dwAttemptFails", SimTypeInt(signed=False, label="UInt32")), ("dwEstabResets", SimTypeInt(signed=False, label="UInt32")), ("dwCurrEstab", SimTypeInt(signed=False, label="UInt32")), ("dw64InSegs", SimTypeLongLong(signed=False, label="UInt64")), ("dw64OutSegs", SimTypeLongLong(signed=False, label="UInt64")), ("dwRetransSegs", SimTypeInt(signed=False, label="UInt32")), ("dwInErrs", SimTypeInt(signed=False, label="UInt32")), ("dwOutRsts", SimTypeInt(signed=False, label="UInt32")), ("dwNumConns", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCPSTATS2", pack=False, align=None),
    "MIB_TCPSTATS_LH": SimStruct(OrderedDict((("Anonymous", SimUnion({"dwRtoAlgorithm": SimTypeInt(signed=False, label="UInt32"), "RtoAlgorithm": SimTypeInt(signed=False, label="TCP_RTO_ALGORITHM")}, name="<anon>", label="None")), ("dwRtoMin", SimTypeInt(signed=False, label="UInt32")), ("dwRtoMax", SimTypeInt(signed=False, label="UInt32")), ("dwMaxConn", SimTypeInt(signed=False, label="UInt32")), ("dwActiveOpens", SimTypeInt(signed=False, label="UInt32")), ("dwPassiveOpens", SimTypeInt(signed=False, label="UInt32")), ("dwAttemptFails", SimTypeInt(signed=False, label="UInt32")), ("dwEstabResets", SimTypeInt(signed=False, label="UInt32")), ("dwCurrEstab", SimTypeInt(signed=False, label="UInt32")), ("dwInSegs", SimTypeInt(signed=False, label="UInt32")), ("dwOutSegs", SimTypeInt(signed=False, label="UInt32")), ("dwRetransSegs", SimTypeInt(signed=False, label="UInt32")), ("dwInErrs", SimTypeInt(signed=False, label="UInt32")), ("dwOutRsts", SimTypeInt(signed=False, label="UInt32")), ("dwNumConns", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCPSTATS_LH", pack=False, align=None),
    "MIB_TCPSTATS_W2K": SimStruct(OrderedDict((("dwRtoAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwRtoMin", SimTypeInt(signed=False, label="UInt32")), ("dwRtoMax", SimTypeInt(signed=False, label="UInt32")), ("dwMaxConn", SimTypeInt(signed=False, label="UInt32")), ("dwActiveOpens", SimTypeInt(signed=False, label="UInt32")), ("dwPassiveOpens", SimTypeInt(signed=False, label="UInt32")), ("dwAttemptFails", SimTypeInt(signed=False, label="UInt32")), ("dwEstabResets", SimTypeInt(signed=False, label="UInt32")), ("dwCurrEstab", SimTypeInt(signed=False, label="UInt32")), ("dwInSegs", SimTypeInt(signed=False, label="UInt32")), ("dwOutSegs", SimTypeInt(signed=False, label="UInt32")), ("dwRetransSegs", SimTypeInt(signed=False, label="UInt32")), ("dwInErrs", SimTypeInt(signed=False, label="UInt32")), ("dwOutRsts", SimTypeInt(signed=False, label="UInt32")), ("dwNumConns", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_TCPSTATS_W2K", pack=False, align=None),
    "MIB_TCPTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCPROW_LH", SimStruct), offset=0)),)), name="MIB_TCPTABLE", pack=False, align=None),
    "MIB_TCPTABLE2": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCPROW2", SimStruct), offset=0)),)), name="MIB_TCPTABLE2", pack=False, align=None),
    "MIB_TCPTABLE_OWNER_MODULE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCPROW_OWNER_MODULE", SimStruct), offset=0)),)), name="MIB_TCPTABLE_OWNER_MODULE", pack=False, align=None),
    "MIB_TCPTABLE_OWNER_PID": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_TCPROW_OWNER_PID", SimStruct), offset=0)),)), name="MIB_TCPTABLE_OWNER_PID", pack=False, align=None),
    "MIB_TCP_STATE": SimTypeInt(signed=False, label="MIB_TCP_STATE"),
    "MIB_UDP6ROW": SimStruct(OrderedDict((("dwLocalAddr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDP6ROW", pack=False, align=None),
    "MIB_UDP6ROW2": SimStruct(OrderedDict((("ucLocalAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("liCreateTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "dwFlags": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("OwningModuleInfo", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)), ("ucRemoteAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwRemoteScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDP6ROW2", pack=False, align=None),
    "MIB_UDP6ROW_OWNER_MODULE": SimStruct(OrderedDict((("ucLocalAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("liCreateTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "dwFlags": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("OwningModuleInfo", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)),)), name="MIB_UDP6ROW_OWNER_MODULE", pack=False, align=None),
    "MIB_UDP6ROW_OWNER_PID": SimStruct(OrderedDict((("ucLocalAddr", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("dwLocalScopeId", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDP6ROW_OWNER_PID", pack=False, align=None),
    "MIB_UDP6TABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDP6ROW", SimStruct), offset=0)),)), name="MIB_UDP6TABLE", pack=False, align=None),
    "MIB_UDP6TABLE2": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDP6ROW2", SimStruct), offset=0)),)), name="MIB_UDP6TABLE2", pack=False, align=None),
    "MIB_UDP6TABLE_OWNER_MODULE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDP6ROW_OWNER_MODULE", SimStruct), offset=0)),)), name="MIB_UDP6TABLE_OWNER_MODULE", pack=False, align=None),
    "MIB_UDP6TABLE_OWNER_PID": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDP6ROW_OWNER_PID", SimStruct), offset=0)),)), name="MIB_UDP6TABLE_OWNER_PID", pack=False, align=None),
    "MIB_UDPROW": SimStruct(OrderedDict((("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDPROW", pack=False, align=None),
    "MIB_UDPROW2": SimStruct(OrderedDict((("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("liCreateTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "dwFlags": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("OwningModuleInfo", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)), ("dwRemoteAddr", SimTypeInt(signed=False, label="UInt32")), ("dwRemotePort", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDPROW2", pack=False, align=None),
    "MIB_UDPROW_OWNER_MODULE": SimStruct(OrderedDict((("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")), ("liCreateTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "dwFlags": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("OwningModuleInfo", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)),)), name="MIB_UDPROW_OWNER_MODULE", pack=False, align=None),
    "MIB_UDPROW_OWNER_PID": SimStruct(OrderedDict((("dwLocalAddr", SimTypeInt(signed=False, label="UInt32")), ("dwLocalPort", SimTypeInt(signed=False, label="UInt32")), ("dwOwningPid", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDPROW_OWNER_PID", pack=False, align=None),
    "MIB_UDPSTATS": SimStruct(OrderedDict((("dwInDatagrams", SimTypeInt(signed=False, label="UInt32")), ("dwNoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwInErrors", SimTypeInt(signed=False, label="UInt32")), ("dwOutDatagrams", SimTypeInt(signed=False, label="UInt32")), ("dwNumAddrs", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDPSTATS", pack=False, align=None),
    "MIB_UDPSTATS2": SimStruct(OrderedDict((("dw64InDatagrams", SimTypeLongLong(signed=False, label="UInt64")), ("dwNoPorts", SimTypeInt(signed=False, label="UInt32")), ("dwInErrors", SimTypeInt(signed=False, label="UInt32")), ("dw64OutDatagrams", SimTypeLongLong(signed=False, label="UInt64")), ("dwNumAddrs", SimTypeInt(signed=False, label="UInt32")),)), name="MIB_UDPSTATS2", pack=False, align=None),
    "MIB_UDPTABLE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDPROW", SimStruct), offset=0)),)), name="MIB_UDPTABLE", pack=False, align=None),
    "MIB_UDPTABLE2": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDPROW2", SimStruct), offset=0)),)), name="MIB_UDPTABLE2", pack=False, align=None),
    "MIB_UDPTABLE_OWNER_MODULE": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDPROW_OWNER_MODULE", SimStruct), offset=0)),)), name="MIB_UDPTABLE_OWNER_MODULE", pack=False, align=None),
    "MIB_UDPTABLE_OWNER_PID": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("table", SimTypePointer(SimTypeRef("MIB_UDPROW_OWNER_PID", SimStruct), offset=0)),)), name="MIB_UDPTABLE_OWNER_PID", pack=False, align=None),
    "MIB_UNICASTIPADDRESS_ROW": SimStruct(OrderedDict((("Address", SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None")), ("InterfaceLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("PrefixOrigin", SimTypeInt(signed=False, label="NL_PREFIX_ORIGIN")), ("SuffixOrigin", SimTypeInt(signed=False, label="NL_SUFFIX_ORIGIN")), ("ValidLifetime", SimTypeInt(signed=False, label="UInt32")), ("PreferredLifetime", SimTypeInt(signed=False, label="UInt32")), ("OnLinkPrefixLength", SimTypeChar(label="Byte")), ("SkipAsSource", SimTypeChar(label="Byte")), ("DadState", SimTypeInt(signed=False, label="NL_DAD_STATE")), ("ScopeId", SimTypeRef("SCOPE_ID", SimStruct)), ("CreationTimeStamp", SimTypeLongLong(signed=True, label="Int64")),)), name="MIB_UNICASTIPADDRESS_ROW", pack=False, align=None),
    "MIB_UNICASTIPADDRESS_TABLE": SimStruct(OrderedDict((("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("Table", SimTypePointer(SimTypeRef("MIB_UNICASTIPADDRESS_ROW", SimStruct), offset=0)),)), name="MIB_UNICASTIPADDRESS_TABLE", pack=False, align=None),
    "MICUIELEMENT": SimTypeInt(signed=False, label="MICUIELEMENT"),
    "MICUIELEMENTSTATE": SimTypeInt(signed=False, label="MICUIELEMENTSTATE"),
    "MIC_ARRAY_MODE": SimTypeInt(signed=False, label="MIC_ARRAY_MODE"),
    "MIDIEVENT": SimStruct(OrderedDict((("dwDeltaTime", SimTypeInt(signed=False, label="UInt32")), ("dwStreamID", SimTypeInt(signed=False, label="UInt32")), ("dwEvent", SimTypeInt(signed=False, label="UInt32")), ("dwParms", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="MIDIEVENT", pack=False, align=None),
    "MIDIHDR": SimStruct(OrderedDict((("lpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwBufferLength", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRecorded", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpNext", SimTypePointer(SimTypeRef("MIDIHDR", SimStruct), offset=0)), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 8)),)), name="MIDIHDR", pack=False, align=None),
    "MIDIINCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="MIDIINCAPS2A", pack=False, align=None),
    "MIDIINCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="MIDIINCAPS2W", pack=False, align=None),
    "MIDIINCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIINCAPSA", pack=False, align=None),
    "MIDIINCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIINCAPSW", pack=False, align=None),
    "MIDILOCALE": SimStruct(OrderedDict((("ulBank", SimTypeInt(signed=False, label="UInt32")), ("ulInstrument", SimTypeInt(signed=False, label="UInt32")),)), name="MIDILOCALE", pack=False, align=None),
    "MIDIOPENDESC": SimStruct(OrderedDict((("hMidi", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dnDevNode", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cIds", SimTypeInt(signed=False, label="UInt32")), ("rgIds", SimTypePointer(SimTypeRef("MIDIOPENSTRMID", SimStruct), offset=0)),)), name="MIDIOPENDESC", pack=False, align=None),
    "MIDIOPENSTRMID": SimStruct(OrderedDict((("dwStreamID", SimTypeInt(signed=False, label="UInt32")), ("uDeviceID", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIOPENSTRMID", pack=False, align=None),
    "MIDIOUTCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wVoices", SimTypeShort(signed=False, label="UInt16")), ("wNotes", SimTypeShort(signed=False, label="UInt16")), ("wChannelMask", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="MIDIOUTCAPS2A", pack=False, align=None),
    "MIDIOUTCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wVoices", SimTypeShort(signed=False, label="UInt16")), ("wNotes", SimTypeShort(signed=False, label="UInt16")), ("wChannelMask", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="MIDIOUTCAPS2W", pack=False, align=None),
    "MIDIOUTCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wVoices", SimTypeShort(signed=False, label="UInt16")), ("wNotes", SimTypeShort(signed=False, label="UInt16")), ("wChannelMask", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIOUTCAPSA", pack=False, align=None),
    "MIDIOUTCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wTechnology", SimTypeShort(signed=False, label="UInt16")), ("wVoices", SimTypeShort(signed=False, label="UInt16")), ("wNotes", SimTypeShort(signed=False, label="UInt16")), ("wChannelMask", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIOUTCAPSW", pack=False, align=None),
    "MIDIPROPTEMPO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwTempo", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIPROPTEMPO", pack=False, align=None),
    "MIDIPROPTIMEDIV": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwTimeDiv", SimTypeInt(signed=False, label="UInt32")),)), name="MIDIPROPTIMEDIV", pack=False, align=None),
    "MIDISTRMBUFFVER": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMid", SimTypeInt(signed=False, label="UInt32")), ("dwOEMVersion", SimTypeInt(signed=False, label="UInt32")),)), name="MIDISTRMBUFFVER", pack=False, align=None),
    "MIDI_WAVE_OPEN_TYPE": SimTypeInt(signed=False, label="MIDI_WAVE_OPEN_TYPE"),
    "MIDL_ES_ALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["state", "pbuffer", "psize"]), offset=0),
    "MIDL_ES_CODE": SimTypeInt(signed=False, label="MIDL_ES_CODE"),
    "MIDL_ES_HANDLE_STYLE": SimTypeInt(signed=False, label="MIDL_ES_HANDLE_STYLE"),
    "MIDL_ES_READ": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["state", "pbuffer", "psize"]), offset=0),
    "MIDL_ES_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["state", "buffer", "size"]), offset=0),
    "MIDL_FORMAT_STRING": SimStruct(OrderedDict((("Pad", SimTypeShort(signed=True, label="Int16")), ("Format", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MIDL_FORMAT_STRING", pack=False, align=None),
    "MIDL_INTERCEPTION_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ProcString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ProcFormatOffsetTable", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ProcCount", SimTypeInt(signed=False, label="UInt32")), ("TypeString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MIDL_INTERCEPTION_INFO", pack=False, align=None),
    "MIDL_INTERFACE_METHOD_PROPERTIES": SimStruct(OrderedDict((("MethodCount", SimTypeShort(signed=False, label="UInt16")), ("MethodProperties", SimTypePointer(SimTypePointer(SimTypeRef("MIDL_METHOD_PROPERTY_MAP", SimStruct), offset=0), offset=0)),)), name="MIDL_INTERFACE_METHOD_PROPERTIES", pack=False, align=None),
    "MIDL_METHOD_PROPERTY": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MIDL_METHOD_PROPERTY", pack=False, align=None),
    "MIDL_METHOD_PROPERTY_MAP": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Properties", SimTypePointer(SimTypeRef("MIDL_METHOD_PROPERTY", SimStruct), offset=0)),)), name="MIDL_METHOD_PROPERTY_MAP", pack=False, align=None),
    "MIDL_SERVER_INFO": SimStruct(OrderedDict((("pStubDesc", SimTypePointer(SimTypeRef("MIDL_STUB_DESC", SimStruct), offset=0)), ("DispatchTable", SimTypePointer(SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0), offset=0)), ("ProcString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FmtStringOffset", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ThunkTable", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0), offset=0)), ("pTransferSyntax", SimTypePointer(SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct), offset=0)), ("nCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pSyntaxInfo", SimTypePointer(SimTypeRef("MIDL_SYNTAX_INFO", SimStruct), offset=0)),)), name="MIDL_SERVER_INFO", pack=False, align=None),
    "MIDL_STUBLESS_PROXY_INFO": SimStruct(OrderedDict((("pStubDesc", SimTypePointer(SimTypeRef("MIDL_STUB_DESC", SimStruct), offset=0)), ("ProcFormatString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FormatStringOffset", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("pTransferSyntax", SimTypePointer(SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct), offset=0)), ("nCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pSyntaxInfo", SimTypePointer(SimTypeRef("MIDL_SYNTAX_INFO", SimStruct), offset=0)),)), name="MIDL_STUBLESS_PROXY_INFO", pack=False, align=None),
    "MIDL_STUB_DESC": SimStruct(OrderedDict((("RpcInterfaceInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfnAllocate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("IMPLICIT_HANDLE_INFO", SimUnion({"pAutoHandle": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), "pPrimitiveHandle": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), "pGenericBindingInfo": SimTypePointer(SimTypeRef("GENERIC_BINDING_INFO", SimStruct), offset=0)}, name="<anon>", label="None")), ("apfnNdrRundownRoutines", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0), offset=0)), ("aGenericBindingRoutinePairs", SimTypePointer(SimTypeRef("GENERIC_BINDING_ROUTINE_PAIR", SimStruct), offset=0)), ("apfnExprEval", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0), offset=0)), ("aXmitQuintuple", SimTypePointer(SimTypeRef("XMIT_ROUTINE_QUINTUPLE", SimStruct), offset=0)), ("pFormatTypes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fCheckBounds", SimTypeInt(signed=True, label="Int32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("pMallocFreeStruct", SimTypePointer(SimTypeRef("MALLOC_FREE_STRUCT", SimStruct), offset=0)), ("MIDLVersion", SimTypeInt(signed=True, label="Int32")), ("CommFaultOffsets", SimTypePointer(SimTypeRef("COMM_FAULT_OFFSETS", SimStruct), offset=0)), ("aUserMarshalQuadruple", SimTypePointer(SimTypeRef("USER_MARSHAL_ROUTINE_QUADRUPLE", SimStruct), offset=0)), ("NotifyRoutineTable", SimTypePointer(SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), offset=0)), ("mFlags", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CsRoutineTables", SimTypePointer(SimTypeRef("NDR_CS_ROUTINES", SimStruct), offset=0)), ("ProxyServerInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pExprInfo", SimTypePointer(SimTypeRef("NDR_EXPR_DESC", SimStruct), offset=0)),)), name="MIDL_STUB_DESC", pack=False, align=None),
    "MIDL_STUB_MESSAGE": SimStruct(OrderedDict((("RpcMsg", SimTypePointer(SimTypeRef("RPC_MESSAGE", SimStruct), offset=0)), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BufferStart", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BufferEnd", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BufferMark", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("BufferLength", SimTypeInt(signed=False, label="UInt32")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("Memory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("IsClient", SimTypeChar(label="Byte")), ("Pad", SimTypeChar(label="Byte")), ("uFlags2", SimTypeShort(signed=False, label="UInt16")), ("ReuseBuffer", SimTypeInt(signed=True, label="Int32")), ("pAllocAllNodesContext", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("pPointerQueueState", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("IgnoreEmbeddedPointers", SimTypeInt(signed=True, label="Int32")), ("PointerBufferMark", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CorrDespIncrement", SimTypeChar(label="Byte")), ("uFlags", SimTypeChar(label="Byte")), ("UniquePtrCount", SimTypeShort(signed=False, label="UInt16")), ("MaxCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("ActualCount", SimTypeInt(signed=False, label="UInt32")), ("pfnAllocate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("StackTop", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPresentedType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pTransmitType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SavedHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StubDesc", SimTypePointer(SimTypeRef("MIDL_STUB_DESC", SimStruct), offset=0)), ("FullPtrXlatTables", SimTypePointer(SimTypeRef("FULL_PTR_XLAT_TABLES", SimStruct), offset=0)), ("FullPtrRefId", SimTypeInt(signed=False, label="UInt32")), ("PointerLength", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=True, label="Int32")), ("dwDestContext", SimTypeInt(signed=False, label="UInt32")), ("pvDestContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SavedContextHandles", SimTypePointer(SimTypePointer(SimTypeRef("NDR_SCONTEXT", SimStruct), offset=0), offset=0)), ("ParamNumber", SimTypeInt(signed=True, label="Int32")), ("pRpcChannelBuffer", SimTypeBottom(label="IRpcChannelBuffer")), ("pArrayInfo", SimTypePointer(SimTypeRef("ARRAY_INFO", SimStruct), offset=0)), ("SizePtrCountArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("SizePtrOffsetArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("SizePtrLengthArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pArgQueue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwStubPhase", SimTypeInt(signed=False, label="UInt32")), ("LowStackMark", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pAsyncMsg", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pCorrInfo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pCorrMemory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMemoryList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pCSInfo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ConformanceMark", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("VarianceMark", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Unused", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pContext", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("ContextHandleHash", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pUserMarshalList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved51_3", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved51_4", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved51_5", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MIDL_STUB_MESSAGE", pack=False, align=None),
    "MIDL_SYNTAX_INFO": SimStruct(OrderedDict((("TransferSyntax", SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct)), ("DispatchTable", SimTypePointer(SimTypeRef("RPC_DISPATCH_TABLE", SimStruct), offset=0)), ("ProcString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FmtStringOffset", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("TypeString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("aUserMarshalQuadruple", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pMethodProperties", SimTypePointer(SimTypeRef("MIDL_INTERFACE_METHOD_PROPERTIES", SimStruct), offset=0)), ("pReserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MIDL_SYNTAX_INFO", pack=False, align=None),
    "MIDL_TYPE_PICKLING_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)),)), name="MIDL_TYPE_PICKLING_INFO", pack=False, align=None),
    "MIDL_WINRT_TYPE_SERIALIZATION_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("TypeFormatString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FormatStringSize", SimTypeShort(signed=False, label="UInt16")), ("TypeOffset", SimTypeShort(signed=False, label="UInt16")), ("StubDesc", SimTypePointer(SimTypeRef("MIDL_STUB_DESC", SimStruct), offset=0)),)), name="MIDL_WINRT_TYPE_SERIALIZATION_INFO", pack=False, align=None),
    "MILMatrixF": SimStruct(OrderedDict((("_11", SimTypeFloat(size=64)), ("_12", SimTypeFloat(size=64)), ("_13", SimTypeFloat(size=64)), ("_14", SimTypeFloat(size=64)), ("_21", SimTypeFloat(size=64)), ("_22", SimTypeFloat(size=64)), ("_23", SimTypeFloat(size=64)), ("_24", SimTypeFloat(size=64)), ("_31", SimTypeFloat(size=64)), ("_32", SimTypeFloat(size=64)), ("_33", SimTypeFloat(size=64)), ("_34", SimTypeFloat(size=64)), ("_41", SimTypeFloat(size=64)), ("_42", SimTypeFloat(size=64)), ("_43", SimTypeFloat(size=64)), ("_44", SimTypeFloat(size=64)),)), name="MILMatrixF", pack=False, align=None),
    "MIMEASSOCIATIONDIALOG_IN_FLAGS": SimTypeInt(signed=False, label="MIMEASSOCIATIONDIALOG_IN_FLAGS"),
    "MIMECONTF": SimTypeInt(signed=False, label="MIMECONTF"),
    "MIMECPINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("uiCodePage", SimTypeInt(signed=False, label="UInt32")), ("uiFamilyCodePage", SimTypeInt(signed=False, label="UInt32")), ("wszDescription", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wszWebCharset", SimTypeArray(SimTypeChar(label="Char"), 50)), ("wszHeaderCharset", SimTypeArray(SimTypeChar(label="Char"), 50)), ("wszBodyCharset", SimTypeArray(SimTypeChar(label="Char"), 50)), ("wszFixedWidthFont", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wszProportionalFont", SimTypeArray(SimTypeChar(label="Char"), 32)), ("bGDICharset", SimTypeChar(label="Byte")),)), name="MIMECPINFO", pack=False, align=None),
    "MIMECSETINFO": SimStruct(OrderedDict((("uiCodePage", SimTypeInt(signed=False, label="UInt32")), ("uiInternetEncoding", SimTypeInt(signed=False, label="UInt32")), ("wszCharset", SimTypeArray(SimTypeChar(label="Char"), 50)),)), name="MIMECSETINFO", pack=False, align=None),
    "MINBUTTONSTATES": SimTypeInt(signed=False, label="MINBUTTONSTATES"),
    "MINCAPTIONSTATES": SimTypeInt(signed=False, label="MINCAPTIONSTATES"),
    "MINIDRV_TRANSFER_CONTEXT": SimStruct(OrderedDict((("lSize", SimTypeInt(signed=True, label="Int32")), ("lWidthInPixels", SimTypeInt(signed=True, label="Int32")), ("lLines", SimTypeInt(signed=True, label="Int32")), ("lDepth", SimTypeInt(signed=True, label="Int32")), ("lXRes", SimTypeInt(signed=True, label="Int32")), ("lYRes", SimTypeInt(signed=True, label="Int32")), ("lCompression", SimTypeInt(signed=True, label="Int32")), ("guidFormatID", SimTypeBottom(label="Guid")), ("tymed", SimTypeInt(signed=True, label="Int32")), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cbOffset", SimTypeInt(signed=True, label="Int32")), ("lBufferSize", SimTypeInt(signed=True, label="Int32")), ("lActiveBuffer", SimTypeInt(signed=True, label="Int32")), ("lNumBuffers", SimTypeInt(signed=True, label="Int32")), ("pBaseBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pTransferBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("bTransferDataCB", SimTypeInt(signed=True, label="Int32")), ("bClassDrvAllocBuf", SimTypeInt(signed=True, label="Int32")), ("lClientAddress", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pIWiaMiniDrvCallBack", SimTypeBottom(label="IWiaMiniDrvCallBack")), ("lImageSize", SimTypeInt(signed=True, label="Int32")), ("lHeaderSize", SimTypeInt(signed=True, label="Int32")), ("lItemSize", SimTypeInt(signed=True, label="Int32")), ("cbWidthInBytes", SimTypeInt(signed=True, label="Int32")), ("lPage", SimTypeInt(signed=True, label="Int32")), ("lCurIfdOffset", SimTypeInt(signed=True, label="Int32")), ("lPrevIfdOffset", SimTypeInt(signed=True, label="Int32")),)), name="MINIDRV_TRANSFER_CONTEXT", pack=False, align=None),
    "MINIDUMP_CALLBACK_INFORMATION": SimStruct(OrderedDict((("CallbackRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MINIDUMP_CALLBACK_INPUT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MINIDUMP_CALLBACK_OUTPUT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackParam", "CallbackInput", "CallbackOutput"]), offset=0)), ("CallbackParam", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MINIDUMP_CALLBACK_INFORMATION", pack=False, align=None),
    "MINIDUMP_CALLBACK_INPUT": SimStruct(OrderedDict((("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallbackType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Status": SimTypeInt(signed=True, label="Int32"), "Thread": SimTypeRef("MINIDUMP_THREAD_CALLBACK", SimStruct), "ThreadEx": SimTypeRef("MINIDUMP_THREAD_EX_CALLBACK", SimStruct), "Module": SimTypeRef("MINIDUMP_MODULE_CALLBACK", SimStruct), "IncludeThread": SimTypeRef("MINIDUMP_INCLUDE_THREAD_CALLBACK", SimStruct), "IncludeModule": SimTypeRef("MINIDUMP_INCLUDE_MODULE_CALLBACK", SimStruct), "Io": SimTypeRef("MINIDUMP_IO_CALLBACK", SimStruct), "ReadMemoryFailure": SimTypeRef("MINIDUMP_READ_MEMORY_FAILURE_CALLBACK", SimStruct), "SecondaryFlags": SimTypeInt(signed=False, label="UInt32"), "VmQuery": SimTypeRef("MINIDUMP_VM_QUERY_CALLBACK", SimStruct), "VmPreRead": SimTypeRef("MINIDUMP_VM_PRE_READ_CALLBACK", SimStruct), "VmPostRead": SimTypeRef("MINIDUMP_VM_POST_READ_CALLBACK", SimStruct)}, name="<anon>", label="None")),)), name="MINIDUMP_CALLBACK_INPUT", pack=False, align=None),
    "MINIDUMP_CALLBACK_OUTPUT": SimStruct(OrderedDict((("Anonymous", SimUnion({"ModuleWriteFlags": SimTypeInt(signed=False, label="UInt32"), "ThreadWriteFlags": SimTypeInt(signed=False, label="UInt32"), "SecondaryFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimStruct(OrderedDict((("MemoryBase", SimTypeLongLong(signed=False, label="UInt64")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("CheckCancel", SimTypeInt(signed=True, label="Int32")), ("Cancel", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "Handle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Anonymous3": SimStruct(OrderedDict((("VmRegion", SimTypeRef("MINIDUMP_MEMORY_INFO", SimStruct)), ("Continue", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous3_e__Struct", pack=False, align=None), "Anonymous4": SimStruct(OrderedDict((("VmQueryStatus", SimTypeInt(signed=True, label="Int32")), ("VmQueryResult", SimTypeRef("MINIDUMP_MEMORY_INFO", SimStruct)),)), name="_Anonymous4_e__Struct", pack=False, align=None), "Anonymous5": SimStruct(OrderedDict((("VmReadStatus", SimTypeInt(signed=True, label="Int32")), ("VmReadBytesCompleted", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous5_e__Struct", pack=False, align=None), "Status": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="MINIDUMP_CALLBACK_OUTPUT", pack=False, align=None),
    "MINIDUMP_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MINIDUMP_CALLBACK_INPUT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MINIDUMP_CALLBACK_OUTPUT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackParam", "CallbackInput", "CallbackOutput"]), offset=0),
    "MINIDUMP_CALLBACK_TYPE": SimTypeInt(signed=False, label="MINIDUMP_CALLBACK_TYPE"),
    "MINIDUMP_DIRECTORY": SimStruct(OrderedDict((("StreamType", SimTypeInt(signed=False, label="UInt32")), ("Location", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)),)), name="MINIDUMP_DIRECTORY", pack=False, align=None),
    "MINIDUMP_EXCEPTION": SimStruct(OrderedDict((("ExceptionCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionFlags", SimTypeInt(signed=False, label="UInt32")), ("ExceptionRecord", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionAddress", SimTypeLongLong(signed=False, label="UInt64")), ("NumberParameters", SimTypeInt(signed=False, label="UInt32")), ("__unusedAlignment", SimTypeInt(signed=False, label="UInt32")), ("ExceptionInformation", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 15)),)), name="MINIDUMP_EXCEPTION", pack=False, align=None),
    "MINIDUMP_EXCEPTION_INFORMATION": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ExceptionPointers", SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0)), ("ClientPointers", SimTypeInt(signed=True, label="Int32")),)), name="MINIDUMP_EXCEPTION_INFORMATION", pack=False, align=None),
    "MINIDUMP_EXCEPTION_INFORMATION64": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ExceptionRecord", SimTypeLongLong(signed=False, label="UInt64")), ("ContextRecord", SimTypeLongLong(signed=False, label="UInt64")), ("ClientPointers", SimTypeInt(signed=True, label="Int32")),)), name="MINIDUMP_EXCEPTION_INFORMATION64", pack=False, align=None),
    "MINIDUMP_EXCEPTION_STREAM": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("__alignment", SimTypeInt(signed=False, label="UInt32")), ("ExceptionRecord", SimTypeRef("MINIDUMP_EXCEPTION", SimStruct)), ("ThreadContext", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)),)), name="MINIDUMP_EXCEPTION_STREAM", pack=False, align=None),
    "MINIDUMP_FUNCTION_TABLE_DESCRIPTOR": SimStruct(OrderedDict((("MinimumAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumAddress", SimTypeLongLong(signed=False, label="UInt64")), ("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("SizeOfAlignPad", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_FUNCTION_TABLE_DESCRIPTOR", pack=False, align=None),
    "MINIDUMP_FUNCTION_TABLE_STREAM": SimStruct(OrderedDict((("SizeOfHeader", SimTypeInt(signed=False, label="UInt32")), ("SizeOfDescriptor", SimTypeInt(signed=False, label="UInt32")), ("SizeOfNativeDescriptor", SimTypeInt(signed=False, label="UInt32")), ("SizeOfFunctionEntry", SimTypeInt(signed=False, label="UInt32")), ("NumberOfDescriptors", SimTypeInt(signed=False, label="UInt32")), ("SizeOfAlignPad", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_FUNCTION_TABLE_STREAM", pack=False, align=None),
    "MINIDUMP_HANDLE_DATA_STREAM": SimStruct(OrderedDict((("SizeOfHeader", SimTypeInt(signed=False, label="UInt32")), ("SizeOfDescriptor", SimTypeInt(signed=False, label="UInt32")), ("NumberOfDescriptors", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_HANDLE_DATA_STREAM", pack=False, align=None),
    "MINIDUMP_HANDLE_DESCRIPTOR": SimStruct(OrderedDict((("Handle", SimTypeLongLong(signed=False, label="UInt64")), ("TypeNameRva", SimTypeInt(signed=False, label="UInt32")), ("ObjectNameRva", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PointerCount", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_HANDLE_DESCRIPTOR", pack=False, align=None),
    "MINIDUMP_HANDLE_DESCRIPTOR_2": SimStruct(OrderedDict((("Handle", SimTypeLongLong(signed=False, label="UInt64")), ("TypeNameRva", SimTypeInt(signed=False, label="UInt32")), ("ObjectNameRva", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PointerCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectInfoRva", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_HANDLE_DESCRIPTOR_2", pack=False, align=None),
    "MINIDUMP_HANDLE_OBJECT_INFORMATION": SimStruct(OrderedDict((("NextInfoRva", SimTypeInt(signed=False, label="UInt32")), ("InfoType", SimTypeInt(signed=False, label="UInt32")), ("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_HANDLE_OBJECT_INFORMATION", pack=False, align=None),
    "MINIDUMP_HANDLE_OBJECT_INFORMATION_TYPE": SimTypeInt(signed=False, label="MINIDUMP_HANDLE_OBJECT_INFORMATION_TYPE"),
    "MINIDUMP_HANDLE_OPERATION_LIST": SimStruct(OrderedDict((("SizeOfHeader", SimTypeInt(signed=False, label="UInt32")), ("SizeOfEntry", SimTypeInt(signed=False, label="UInt32")), ("NumberOfEntries", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_HANDLE_OPERATION_LIST", pack=False, align=None),
    "MINIDUMP_HEADER": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("NumberOfStreams", SimTypeInt(signed=False, label="UInt32")), ("StreamDirectoryRva", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "TimeDateStamp": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_HEADER", pack=False, align=None),
    "MINIDUMP_INCLUDE_MODULE_CALLBACK": SimStruct(OrderedDict((("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_INCLUDE_MODULE_CALLBACK", pack=False, align=None),
    "MINIDUMP_INCLUDE_THREAD_CALLBACK": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_INCLUDE_THREAD_CALLBACK", pack=False, align=None),
    "MINIDUMP_IO_CALLBACK": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferBytes", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_IO_CALLBACK", pack=False, align=None),
    "MINIDUMP_LOCATION_DESCRIPTOR": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Rva", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_LOCATION_DESCRIPTOR", pack=False, align=None),
    "MINIDUMP_LOCATION_DESCRIPTOR64": SimStruct(OrderedDict((("DataSize", SimTypeLongLong(signed=False, label="UInt64")), ("Rva", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_LOCATION_DESCRIPTOR64", pack=False, align=None),
    "MINIDUMP_MEMORY64_LIST": SimStruct(OrderedDict((("NumberOfMemoryRanges", SimTypeLongLong(signed=False, label="UInt64")), ("BaseRva", SimTypeLongLong(signed=False, label="UInt64")), ("MemoryRanges", SimTypePointer(SimTypeRef("MINIDUMP_MEMORY_DESCRIPTOR64", SimStruct), offset=0)),)), name="MINIDUMP_MEMORY64_LIST", pack=False, align=None),
    "MINIDUMP_MEMORY_DESCRIPTOR": SimStruct(OrderedDict((("StartOfMemoryRange", SimTypeLongLong(signed=False, label="UInt64")), ("Memory", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)),)), name="MINIDUMP_MEMORY_DESCRIPTOR", pack=False, align=None),
    "MINIDUMP_MEMORY_DESCRIPTOR64": SimStruct(OrderedDict((("StartOfMemoryRange", SimTypeLongLong(signed=False, label="UInt64")), ("DataSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_MEMORY_DESCRIPTOR64", pack=False, align=None),
    "MINIDUMP_MEMORY_INFO": SimStruct(OrderedDict((("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationBase", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationProtect", SimTypeInt(signed=False, label="UInt32")), ("__alignment1", SimTypeInt(signed=False, label="UInt32")), ("RegionSize", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypeInt(signed=False, label="VIRTUAL_ALLOCATION_TYPE")), ("Protect", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("__alignment2", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_MEMORY_INFO", pack=False, align=None),
    "MINIDUMP_MEMORY_INFO_LIST": SimStruct(OrderedDict((("SizeOfHeader", SimTypeInt(signed=False, label="UInt32")), ("SizeOfEntry", SimTypeInt(signed=False, label="UInt32")), ("NumberOfEntries", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_MEMORY_INFO_LIST", pack=False, align=None),
    "MINIDUMP_MEMORY_LIST": SimStruct(OrderedDict((("NumberOfMemoryRanges", SimTypeInt(signed=False, label="UInt32")), ("MemoryRanges", SimTypePointer(SimTypeRef("MINIDUMP_MEMORY_DESCRIPTOR", SimStruct), offset=0)),)), name="MINIDUMP_MEMORY_LIST", pack=False, align=None),
    "MINIDUMP_MISC_INFO": SimStruct(OrderedDict((("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("Flags1", SimTypeInt(signed=False, label="MINIDUMP_MISC_INFO_FLAGS")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessCreateTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessUserTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessKernelTime", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_MISC_INFO", pack=False, align=None),
    "MINIDUMP_MISC_INFO_2": SimStruct(OrderedDict((("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("Flags1", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessCreateTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessUserTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessKernelTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMhzLimit", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentIdleState", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_MISC_INFO_2", pack=False, align=None),
    "MINIDUMP_MISC_INFO_3": SimStruct(OrderedDict((("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("Flags1", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessCreateTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessUserTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessKernelTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMhzLimit", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessIntegrityLevel", SimTypeInt(signed=False, label="UInt32")), ("ProcessExecuteFlags", SimTypeInt(signed=False, label="UInt32")), ("ProtectedProcess", SimTypeInt(signed=False, label="UInt32")), ("TimeZoneId", SimTypeInt(signed=False, label="UInt32")), ("TimeZone", SimTypeRef("TIME_ZONE_INFORMATION", SimStruct)),)), name="MINIDUMP_MISC_INFO_3", pack=False, align=None),
    "MINIDUMP_MISC_INFO_4": SimStruct(OrderedDict((("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("Flags1", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessCreateTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessUserTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessKernelTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMhzLimit", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessIntegrityLevel", SimTypeInt(signed=False, label="UInt32")), ("ProcessExecuteFlags", SimTypeInt(signed=False, label="UInt32")), ("ProtectedProcess", SimTypeInt(signed=False, label="UInt32")), ("TimeZoneId", SimTypeInt(signed=False, label="UInt32")), ("TimeZone", SimTypeRef("TIME_ZONE_INFORMATION", SimStruct)), ("BuildString", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DbgBldStr", SimTypeArray(SimTypeChar(label="Char"), 40)),)), name="MINIDUMP_MISC_INFO_4", pack=False, align=None),
    "MINIDUMP_MISC_INFO_5": SimStruct(OrderedDict((("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("Flags1", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessCreateTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessUserTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessKernelTime", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentMhz", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMhzLimit", SimTypeInt(signed=False, label="UInt32")), ("ProcessorMaxIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessorCurrentIdleState", SimTypeInt(signed=False, label="UInt32")), ("ProcessIntegrityLevel", SimTypeInt(signed=False, label="UInt32")), ("ProcessExecuteFlags", SimTypeInt(signed=False, label="UInt32")), ("ProtectedProcess", SimTypeInt(signed=False, label="UInt32")), ("TimeZoneId", SimTypeInt(signed=False, label="UInt32")), ("TimeZone", SimTypeRef("TIME_ZONE_INFORMATION", SimStruct)), ("BuildString", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DbgBldStr", SimTypeArray(SimTypeChar(label="Char"), 40)), ("XStateData", SimTypeRef("XSTATE_CONFIG_FEATURE_MSC_INFO", SimStruct)), ("ProcessCookie", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_MISC_INFO_5", pack=False, align=None),
    "MINIDUMP_MISC_INFO_FLAGS": SimTypeInt(signed=False, label="MINIDUMP_MISC_INFO_FLAGS"),
    "MINIDUMP_MODULE": SimStruct(OrderedDict((("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ModuleNameRva", SimTypeInt(signed=False, label="UInt32")), ("VersionInfo", SimTypeRef("VS_FIXEDFILEINFO", SimStruct)), ("CvRecord", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)), ("MiscRecord", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)), ("Reserved0", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_MODULE", pack=False, align=None),
    "MINIDUMP_MODULE_CALLBACK": SimStruct(OrderedDict((("FullPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("VersionInfo", SimTypeRef("VS_FIXEDFILEINFO", SimStruct)), ("CvRecord", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SizeOfCvRecord", SimTypeInt(signed=False, label="UInt32")), ("MiscRecord", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SizeOfMiscRecord", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_MODULE_CALLBACK", pack=False, align=None),
    "MINIDUMP_MODULE_LIST": SimStruct(OrderedDict((("NumberOfModules", SimTypeInt(signed=False, label="UInt32")), ("Modules", SimTypePointer(SimTypeRef("MINIDUMP_MODULE", SimStruct), offset=0)),)), name="MINIDUMP_MODULE_LIST", pack=False, align=None),
    "MINIDUMP_PROCESS_VM_COUNTERS_1": SimStruct(OrderedDict((("Revision", SimTypeShort(signed=False, label="UInt16")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypeLongLong(signed=False, label="UInt64")), ("WorkingSetSize", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaPeakPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaPeakNonPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaNonPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PagefileUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PeakPagefileUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PrivateUsage", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_PROCESS_VM_COUNTERS_1", pack=False, align=None),
    "MINIDUMP_PROCESS_VM_COUNTERS_2": SimStruct(OrderedDict((("Revision", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypeLongLong(signed=False, label="UInt64")), ("WorkingSetSize", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaPeakPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaPeakNonPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("QuotaNonPagedPoolUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PagefileUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PeakPagefileUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PeakVirtualSize", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualSize", SimTypeLongLong(signed=False, label="UInt64")), ("PrivateUsage", SimTypeLongLong(signed=False, label="UInt64")), ("PrivateWorkingSetSize", SimTypeLongLong(signed=False, label="UInt64")), ("SharedCommitUsage", SimTypeLongLong(signed=False, label="UInt64")), ("JobSharedCommitUsage", SimTypeLongLong(signed=False, label="UInt64")), ("JobPrivateCommitUsage", SimTypeLongLong(signed=False, label="UInt64")), ("JobPeakPrivateCommitUsage", SimTypeLongLong(signed=False, label="UInt64")), ("JobPrivateCommitLimit", SimTypeLongLong(signed=False, label="UInt64")), ("JobTotalCommitLimit", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_PROCESS_VM_COUNTERS_2", pack=False, align=None),
    "MINIDUMP_READ_MEMORY_FAILURE_CALLBACK": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Bytes", SimTypeInt(signed=False, label="UInt32")), ("FailureStatus", SimTypeInt(signed=True, label="Int32")),)), name="MINIDUMP_READ_MEMORY_FAILURE_CALLBACK", pack=False, align=None),
    "MINIDUMP_SECONDARY_FLAGS": SimTypeInt(signed=False, label="MINIDUMP_SECONDARY_FLAGS"),
    "MINIDUMP_STREAM_TYPE": SimTypeInt(signed=False, label="MINIDUMP_STREAM_TYPE"),
    "MINIDUMP_STRING": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MINIDUMP_STRING", pack=False, align=None),
    "MINIDUMP_SYSTEM_BASIC_INFORMATION": SimStruct(OrderedDict((("TimerResolution", SimTypeInt(signed=False, label="UInt32")), ("PageSize", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPhysicalPages", SimTypeInt(signed=False, label="UInt32")), ("LowestPhysicalPageNumber", SimTypeInt(signed=False, label="UInt32")), ("HighestPhysicalPageNumber", SimTypeInt(signed=False, label="UInt32")), ("AllocationGranularity", SimTypeInt(signed=False, label="UInt32")), ("MinimumUserModeAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumUserModeAddress", SimTypeLongLong(signed=False, label="UInt64")), ("ActiveProcessorsAffinityMask", SimTypeLongLong(signed=False, label="UInt64")), ("NumberOfProcessors", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_SYSTEM_BASIC_INFORMATION", pack=False, align=None),
    "MINIDUMP_SYSTEM_BASIC_PERFORMANCE_INFORMATION": SimStruct(OrderedDict((("AvailablePages", SimTypeLongLong(signed=False, label="UInt64")), ("CommittedPages", SimTypeLongLong(signed=False, label="UInt64")), ("CommitLimit", SimTypeLongLong(signed=False, label="UInt64")), ("PeakCommitment", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_SYSTEM_BASIC_PERFORMANCE_INFORMATION", pack=False, align=None),
    "MINIDUMP_SYSTEM_FILECACHE_INFORMATION": SimStruct(OrderedDict((("CurrentSize", SimTypeLongLong(signed=False, label="UInt64")), ("PeakSize", SimTypeLongLong(signed=False, label="UInt64")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("MinimumWorkingSet", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumWorkingSet", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentSizeIncludingTransitionInPages", SimTypeLongLong(signed=False, label="UInt64")), ("PeakSizeIncludingTransitionInPages", SimTypeLongLong(signed=False, label="UInt64")), ("TransitionRePurposeCount", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_SYSTEM_FILECACHE_INFORMATION", pack=False, align=None),
    "MINIDUMP_SYSTEM_INFO": SimStruct(OrderedDict((("ProcessorArchitecture", SimTypeInt(signed=False, label="PROCESSOR_ARCHITECTURE")), ("ProcessorLevel", SimTypeShort(signed=False, label="UInt16")), ("ProcessorRevision", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"Reserved0": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("NumberOfProcessors", SimTypeChar(label="Byte")), ("ProductType", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("BuildNumber", SimTypeInt(signed=False, label="UInt32")), ("PlatformId", SimTypeInt(signed=False, label="VER_PLATFORM")), ("CSDVersionRva", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"Reserved1": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("SuiteMask", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Cpu", SimUnion({"X86CpuInfo": SimTypeRef("_X86CpuInfo_e__Struct", SimStruct), "OtherCpuInfo": SimTypeRef("_OtherCpuInfo_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="MINIDUMP_SYSTEM_INFO", pack=False, align=None),
    "MINIDUMP_SYSTEM_MEMORY_INFO_1": SimStruct(OrderedDict((("Revision", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("BasicInfo", SimTypeRef("MINIDUMP_SYSTEM_BASIC_INFORMATION", SimStruct)), ("FileCacheInfo", SimTypeRef("MINIDUMP_SYSTEM_FILECACHE_INFORMATION", SimStruct)), ("BasicPerfInfo", SimTypeRef("MINIDUMP_SYSTEM_BASIC_PERFORMANCE_INFORMATION", SimStruct)), ("PerfInfo", SimTypeRef("MINIDUMP_SYSTEM_PERFORMANCE_INFORMATION", SimStruct)),)), name="MINIDUMP_SYSTEM_MEMORY_INFO_1", pack=False, align=None),
    "MINIDUMP_SYSTEM_PERFORMANCE_INFORMATION": SimStruct(OrderedDict((("IdleProcessTime", SimTypeLongLong(signed=False, label="UInt64")), ("IoReadTransferCount", SimTypeLongLong(signed=False, label="UInt64")), ("IoWriteTransferCount", SimTypeLongLong(signed=False, label="UInt64")), ("IoOtherTransferCount", SimTypeLongLong(signed=False, label="UInt64")), ("IoReadOperationCount", SimTypeInt(signed=False, label="UInt32")), ("IoWriteOperationCount", SimTypeInt(signed=False, label="UInt32")), ("IoOtherOperationCount", SimTypeInt(signed=False, label="UInt32")), ("AvailablePages", SimTypeInt(signed=False, label="UInt32")), ("CommittedPages", SimTypeInt(signed=False, label="UInt32")), ("CommitLimit", SimTypeInt(signed=False, label="UInt32")), ("PeakCommitment", SimTypeInt(signed=False, label="UInt32")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("CopyOnWriteCount", SimTypeInt(signed=False, label="UInt32")), ("TransitionCount", SimTypeInt(signed=False, label="UInt32")), ("CacheTransitionCount", SimTypeInt(signed=False, label="UInt32")), ("DemandZeroCount", SimTypeInt(signed=False, label="UInt32")), ("PageReadCount", SimTypeInt(signed=False, label="UInt32")), ("PageReadIoCount", SimTypeInt(signed=False, label="UInt32")), ("CacheReadCount", SimTypeInt(signed=False, label="UInt32")), ("CacheIoCount", SimTypeInt(signed=False, label="UInt32")), ("DirtyPagesWriteCount", SimTypeInt(signed=False, label="UInt32")), ("DirtyWriteIoCount", SimTypeInt(signed=False, label="UInt32")), ("MappedPagesWriteCount", SimTypeInt(signed=False, label="UInt32")), ("MappedWriteIoCount", SimTypeInt(signed=False, label="UInt32")), ("PagedPoolPages", SimTypeInt(signed=False, label="UInt32")), ("NonPagedPoolPages", SimTypeInt(signed=False, label="UInt32")), ("PagedPoolAllocs", SimTypeInt(signed=False, label="UInt32")), ("PagedPoolFrees", SimTypeInt(signed=False, label="UInt32")), ("NonPagedPoolAllocs", SimTypeInt(signed=False, label="UInt32")), ("NonPagedPoolFrees", SimTypeInt(signed=False, label="UInt32")), ("FreeSystemPtes", SimTypeInt(signed=False, label="UInt32")), ("ResidentSystemCodePage", SimTypeInt(signed=False, label="UInt32")), ("TotalSystemDriverPages", SimTypeInt(signed=False, label="UInt32")), ("TotalSystemCodePages", SimTypeInt(signed=False, label="UInt32")), ("NonPagedPoolLookasideHits", SimTypeInt(signed=False, label="UInt32")), ("PagedPoolLookasideHits", SimTypeInt(signed=False, label="UInt32")), ("AvailablePagedPoolPages", SimTypeInt(signed=False, label="UInt32")), ("ResidentSystemCachePage", SimTypeInt(signed=False, label="UInt32")), ("ResidentPagedPoolPage", SimTypeInt(signed=False, label="UInt32")), ("ResidentSystemDriverPage", SimTypeInt(signed=False, label="UInt32")), ("CcFastReadNoWait", SimTypeInt(signed=False, label="UInt32")), ("CcFastReadWait", SimTypeInt(signed=False, label="UInt32")), ("CcFastReadResourceMiss", SimTypeInt(signed=False, label="UInt32")), ("CcFastReadNotPossible", SimTypeInt(signed=False, label="UInt32")), ("CcFastMdlReadNoWait", SimTypeInt(signed=False, label="UInt32")), ("CcFastMdlReadWait", SimTypeInt(signed=False, label="UInt32")), ("CcFastMdlReadResourceMiss", SimTypeInt(signed=False, label="UInt32")), ("CcFastMdlReadNotPossible", SimTypeInt(signed=False, label="UInt32")), ("CcMapDataNoWait", SimTypeInt(signed=False, label="UInt32")), ("CcMapDataWait", SimTypeInt(signed=False, label="UInt32")), ("CcMapDataNoWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcMapDataWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcPinMappedDataCount", SimTypeInt(signed=False, label="UInt32")), ("CcPinReadNoWait", SimTypeInt(signed=False, label="UInt32")), ("CcPinReadWait", SimTypeInt(signed=False, label="UInt32")), ("CcPinReadNoWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcPinReadWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcCopyReadNoWait", SimTypeInt(signed=False, label="UInt32")), ("CcCopyReadWait", SimTypeInt(signed=False, label="UInt32")), ("CcCopyReadNoWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcCopyReadWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcMdlReadNoWait", SimTypeInt(signed=False, label="UInt32")), ("CcMdlReadWait", SimTypeInt(signed=False, label="UInt32")), ("CcMdlReadNoWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcMdlReadWaitMiss", SimTypeInt(signed=False, label="UInt32")), ("CcReadAheadIos", SimTypeInt(signed=False, label="UInt32")), ("CcLazyWriteIos", SimTypeInt(signed=False, label="UInt32")), ("CcLazyWritePages", SimTypeInt(signed=False, label="UInt32")), ("CcDataFlushes", SimTypeInt(signed=False, label="UInt32")), ("CcDataPages", SimTypeInt(signed=False, label="UInt32")), ("ContextSwitches", SimTypeInt(signed=False, label="UInt32")), ("FirstLevelTbFills", SimTypeInt(signed=False, label="UInt32")), ("SecondLevelTbFills", SimTypeInt(signed=False, label="UInt32")), ("SystemCalls", SimTypeInt(signed=False, label="UInt32")), ("CcTotalDirtyPages", SimTypeLongLong(signed=False, label="UInt64")), ("CcDirtyPageThreshold", SimTypeLongLong(signed=False, label="UInt64")), ("ResidentAvailablePages", SimTypeLongLong(signed=True, label="Int64")), ("SharedCommittedPages", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_SYSTEM_PERFORMANCE_INFORMATION", pack=False, align=None),
    "MINIDUMP_THREAD": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("SuspendCount", SimTypeInt(signed=False, label="UInt32")), ("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Teb", SimTypeLongLong(signed=False, label="UInt64")), ("Stack", SimTypeRef("MINIDUMP_MEMORY_DESCRIPTOR", SimStruct)), ("ThreadContext", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)),)), name="MINIDUMP_THREAD", pack=False, align=None),
    "MINIDUMP_THREAD_CALLBACK": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ThreadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Context", SimTypeRef("CONTEXT", SimStruct)), ("SizeOfContext", SimTypeInt(signed=False, label="UInt32")), ("StackBase", SimTypeLongLong(signed=False, label="UInt64")), ("StackEnd", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_THREAD_CALLBACK", pack=False, align=None),
    "MINIDUMP_THREAD_EX": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("SuspendCount", SimTypeInt(signed=False, label="UInt32")), ("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("Teb", SimTypeLongLong(signed=False, label="UInt64")), ("Stack", SimTypeRef("MINIDUMP_MEMORY_DESCRIPTOR", SimStruct)), ("ThreadContext", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)), ("BackingStore", SimTypeRef("MINIDUMP_MEMORY_DESCRIPTOR", SimStruct)),)), name="MINIDUMP_THREAD_EX", pack=False, align=None),
    "MINIDUMP_THREAD_EX_CALLBACK": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ThreadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Context", SimTypeRef("CONTEXT", SimStruct)), ("SizeOfContext", SimTypeInt(signed=False, label="UInt32")), ("StackBase", SimTypeLongLong(signed=False, label="UInt64")), ("StackEnd", SimTypeLongLong(signed=False, label="UInt64")), ("BackingStoreBase", SimTypeLongLong(signed=False, label="UInt64")), ("BackingStoreEnd", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_THREAD_EX_CALLBACK", pack=False, align=None),
    "MINIDUMP_THREAD_EX_LIST": SimStruct(OrderedDict((("NumberOfThreads", SimTypeInt(signed=False, label="UInt32")), ("Threads", SimTypePointer(SimTypeRef("MINIDUMP_THREAD_EX", SimStruct), offset=0)),)), name="MINIDUMP_THREAD_EX_LIST", pack=False, align=None),
    "MINIDUMP_THREAD_INFO": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("DumpFlags", SimTypeInt(signed=False, label="MINIDUMP_THREAD_INFO_DUMP_FLAGS")), ("DumpError", SimTypeInt(signed=False, label="UInt32")), ("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("CreateTime", SimTypeLongLong(signed=False, label="UInt64")), ("ExitTime", SimTypeLongLong(signed=False, label="UInt64")), ("KernelTime", SimTypeLongLong(signed=False, label="UInt64")), ("UserTime", SimTypeLongLong(signed=False, label="UInt64")), ("StartAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Affinity", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_THREAD_INFO", pack=False, align=None),
    "MINIDUMP_THREAD_INFO_DUMP_FLAGS": SimTypeInt(signed=False, label="MINIDUMP_THREAD_INFO_DUMP_FLAGS"),
    "MINIDUMP_THREAD_INFO_LIST": SimStruct(OrderedDict((("SizeOfHeader", SimTypeInt(signed=False, label="UInt32")), ("SizeOfEntry", SimTypeInt(signed=False, label="UInt32")), ("NumberOfEntries", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_THREAD_INFO_LIST", pack=False, align=None),
    "MINIDUMP_THREAD_LIST": SimStruct(OrderedDict((("NumberOfThreads", SimTypeInt(signed=False, label="UInt32")), ("Threads", SimTypePointer(SimTypeRef("MINIDUMP_THREAD", SimStruct), offset=0)),)), name="MINIDUMP_THREAD_LIST", pack=False, align=None),
    "MINIDUMP_THREAD_NAME": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("RvaOfThreadName", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_THREAD_NAME", pack=False, align=None),
    "MINIDUMP_THREAD_NAME_LIST": SimStruct(OrderedDict((("NumberOfThreadNames", SimTypeInt(signed=False, label="UInt32")), ("ThreadNames", SimTypePointer(SimTypeRef("MINIDUMP_THREAD_NAME", SimStruct), offset=0)),)), name="MINIDUMP_THREAD_NAME_LIST", pack=False, align=None),
    "MINIDUMP_TOKEN_INFO_HEADER": SimStruct(OrderedDict((("TokenSize", SimTypeInt(signed=False, label="UInt32")), ("TokenId", SimTypeInt(signed=False, label="UInt32")), ("TokenHandle", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_TOKEN_INFO_HEADER", pack=False, align=None),
    "MINIDUMP_TOKEN_INFO_LIST": SimStruct(OrderedDict((("TokenListSize", SimTypeInt(signed=False, label="UInt32")), ("TokenListEntries", SimTypeInt(signed=False, label="UInt32")), ("ListHeaderSize", SimTypeInt(signed=False, label="UInt32")), ("ElementHeaderSize", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_TOKEN_INFO_LIST", pack=False, align=None),
    "MINIDUMP_TYPE": SimTypeInt(signed=False, label="MINIDUMP_TYPE"),
    "MINIDUMP_UNLOADED_MODULE": SimStruct(OrderedDict((("BaseOfImage", SimTypeLongLong(signed=False, label="UInt64")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("ModuleNameRva", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_UNLOADED_MODULE", pack=False, align=None),
    "MINIDUMP_UNLOADED_MODULE_LIST": SimStruct(OrderedDict((("SizeOfHeader", SimTypeInt(signed=False, label="UInt32")), ("SizeOfEntry", SimTypeInt(signed=False, label="UInt32")), ("NumberOfEntries", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_UNLOADED_MODULE_LIST", pack=False, align=None),
    "MINIDUMP_USER_RECORD": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Memory", SimTypeRef("MINIDUMP_LOCATION_DESCRIPTOR", SimStruct)),)), name="MINIDUMP_USER_RECORD", pack=False, align=None),
    "MINIDUMP_USER_STREAM": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MINIDUMP_USER_STREAM", pack=False, align=None),
    "MINIDUMP_USER_STREAM_INFORMATION": SimStruct(OrderedDict((("UserStreamCount", SimTypeInt(signed=False, label="UInt32")), ("UserStreamArray", SimTypePointer(SimTypeRef("MINIDUMP_USER_STREAM", SimStruct), offset=0)),)), name="MINIDUMP_USER_STREAM_INFORMATION", pack=False, align=None),
    "MINIDUMP_VM_POST_READ_CALLBACK": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Completed", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="MINIDUMP_VM_POST_READ_CALLBACK", pack=False, align=None),
    "MINIDUMP_VM_PRE_READ_CALLBACK": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="MINIDUMP_VM_PRE_READ_CALLBACK", pack=False, align=None),
    "MINIDUMP_VM_QUERY_CALLBACK": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")),)), name="MINIDUMP_VM_QUERY_CALLBACK", pack=False, align=None),
    "MINIMIZEDMETRICS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iWidth", SimTypeInt(signed=True, label="Int32")), ("iHorzGap", SimTypeInt(signed=True, label="Int32")), ("iVertGap", SimTypeInt(signed=True, label="Int32")), ("iArrange", SimTypeInt(signed=False, label="MINIMIZEDMETRICS_ARRANGE")),)), name="MINIMIZEDMETRICS", pack=False, align=None),
    "MINIMIZEDMETRICS_ARRANGE": SimTypeInt(signed=False, label="MINIMIZEDMETRICS_ARRANGE"),
    "MINIPORT_CO_ACTIVATE_VC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MiniportVcContext", "CallParameters"]), offset=0),
    "MINIPORT_CO_CREATE_VC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MiniportAdapterContext", "NdisVcHandle", "MiniportVcContext"]), offset=0),
    "MINIPORT_CO_DEACTIVATE_VC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MiniportVcContext"]), offset=0),
    "MINIPORT_CO_DELETE_VC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MiniportVcContext"]), offset=0),
    "MINMAXINFO": SimStruct(OrderedDict((("ptReserved", SimTypeRef("POINT", SimStruct)), ("ptMaxSize", SimTypeRef("POINT", SimStruct)), ("ptMaxPosition", SimTypeRef("POINT", SimStruct)), ("ptMinTrackSize", SimTypeRef("POINT", SimStruct)), ("ptMaxTrackSize", SimTypeRef("POINT", SimStruct)),)), name="MINMAXINFO", pack=False, align=None),
    "MIPI_DSI_CAPS": SimStruct(OrderedDict((("DSITypeMajor", SimTypeChar(label="Byte")), ("DSITypeMinor", SimTypeChar(label="Byte")), ("SpecVersionMajor", SimTypeChar(label="Byte")), ("SpecVersionMinor", SimTypeChar(label="Byte")), ("SpecVersionPatch", SimTypeChar(label="Byte")), ("TargetMaximumReturnPacketSize", SimTypeShort(signed=False, label="UInt16")), ("ResultCodeFlags", SimTypeChar(label="Byte")), ("ResultCodeStatus", SimTypeChar(label="Byte")), ("Revision", SimTypeChar(label="Byte")), ("Level", SimTypeChar(label="Byte")), ("DeviceClassHi", SimTypeChar(label="Byte")), ("DeviceClassLo", SimTypeChar(label="Byte")), ("ManufacturerHi", SimTypeChar(label="Byte")), ("ManufacturerLo", SimTypeChar(label="Byte")), ("ProductHi", SimTypeChar(label="Byte")), ("ProductLo", SimTypeChar(label="Byte")), ("LengthHi", SimTypeChar(label="Byte")), ("LengthLo", SimTypeChar(label="Byte")),)), name="MIPI_DSI_CAPS", pack=False, align=None),
    "MIPI_DSI_PACKET": SimStruct(OrderedDict((("Anonymous1", SimUnion({"DataId": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Anonymous": SimStruct(OrderedDict((("Data0", SimTypeChar(label="Byte")), ("Data1", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "LongWriteWordCount": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("EccFiller", SimTypeChar(label="Byte")), ("Payload", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="MIPI_DSI_PACKET", pack=False, align=None),
    "MIPI_DSI_RESET": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Results": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="MIPI_DSI_RESET", pack=False, align=None),
    "MIPI_DSI_TRANSMISSION": SimStruct(OrderedDict((("TotalBufferSize", SimTypeInt(signed=False, label="UInt32")), ("PacketCount", SimTypeChar(label="Byte")), ("FailedPacket", SimTypeChar(label="Byte")), ("Anonymous", SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)), ("ReadWordCount", SimTypeShort(signed=False, label="UInt16")), ("FinalCommandExtraPayload", SimTypeShort(signed=False, label="UInt16")), ("MipiErrors", SimTypeShort(signed=False, label="UInt16")), ("HostErrors", SimTypeShort(signed=False, label="UInt16")), ("Packets", SimTypePointer(SimTypeRef("MIPI_DSI_PACKET", SimStruct), offset=0)),)), name="MIPI_DSI_TRANSMISSION", pack=False, align=None),
    "MIRROR_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="MIRROR_VIRTUAL_DISK_FLAG"),
    "MIRROR_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="MIRROR_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="MIRROR_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "MIRROR_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="MIRROR_VIRTUAL_DISK_VERSION"),
    "MIXERCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cDestinations", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="MIXERCAPS2A", pack=False, align=None),
    "MIXERCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cDestinations", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="MIXERCAPS2W", pack=False, align=None),
    "MIXERCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cDestinations", SimTypeInt(signed=False, label="UInt32")),)), name="MIXERCAPSA", pack=False, align=None),
    "MIXERCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("cDestinations", SimTypeInt(signed=False, label="UInt32")),)), name="MIXERCAPSW", pack=False, align=None),
    "MIXERCONTROLA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwControlID", SimTypeInt(signed=False, label="UInt32")), ("dwControlType", SimTypeInt(signed=False, label="UInt32")), ("fdwControl", SimTypeInt(signed=False, label="UInt32")), ("cMultipleItems", SimTypeInt(signed=False, label="UInt32")), ("szShortName", SimTypeArray(SimTypeChar(label="SByte"), 16)), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Bounds", SimUnion({"Anonymous1": SimStruct(OrderedDict((("lMinimum", SimTypeInt(signed=True, label="Int32")), ("lMaximum", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("dwMinimum", SimTypeInt(signed=False, label="UInt32")), ("dwMaximum", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "dwReserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)}, name="<anon>", label="None")), ("Metrics", SimUnion({"cSteps": SimTypeInt(signed=False, label="UInt32"), "cbCustomData": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)}, name="<anon>", label="None")),)), name="MIXERCONTROLA", pack=False, align=None),
    "MIXERCONTROLDETAILS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwControlID", SimTypeInt(signed=False, label="UInt32")), ("cChannels", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hwndOwner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "cMultipleItems": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("cbDetails", SimTypeInt(signed=False, label="UInt32")), ("paDetails", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MIXERCONTROLDETAILS", pack=False, align=None),
    "MIXERCONTROLDETAILS_BOOLEAN": SimStruct(OrderedDict((("fValue", SimTypeInt(signed=True, label="Int32")),)), name="MIXERCONTROLDETAILS_BOOLEAN", pack=False, align=None),
    "MIXERCONTROLDETAILS_LISTTEXTA": SimStruct(OrderedDict((("dwParam1", SimTypeInt(signed=False, label="UInt32")), ("dwParam2", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="MIXERCONTROLDETAILS_LISTTEXTA", pack=False, align=None),
    "MIXERCONTROLDETAILS_LISTTEXTW": SimStruct(OrderedDict((("dwParam1", SimTypeInt(signed=False, label="UInt32")), ("dwParam2", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="MIXERCONTROLDETAILS_LISTTEXTW", pack=False, align=None),
    "MIXERCONTROLDETAILS_SIGNED": SimStruct(OrderedDict((("lValue", SimTypeInt(signed=True, label="Int32")),)), name="MIXERCONTROLDETAILS_SIGNED", pack=False, align=None),
    "MIXERCONTROLDETAILS_UNSIGNED": SimStruct(OrderedDict((("dwValue", SimTypeInt(signed=False, label="UInt32")),)), name="MIXERCONTROLDETAILS_UNSIGNED", pack=False, align=None),
    "MIXERCONTROLW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwControlID", SimTypeInt(signed=False, label="UInt32")), ("dwControlType", SimTypeInt(signed=False, label="UInt32")), ("fdwControl", SimTypeInt(signed=False, label="UInt32")), ("cMultipleItems", SimTypeInt(signed=False, label="UInt32")), ("szShortName", SimTypeArray(SimTypeChar(label="Char"), 16)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("Bounds", SimUnion({"Anonymous1": SimStruct(OrderedDict((("lMinimum", SimTypeInt(signed=True, label="Int32")), ("lMaximum", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("dwMinimum", SimTypeInt(signed=False, label="UInt32")), ("dwMaximum", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "dwReserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)}, name="<anon>", label="None")), ("Metrics", SimUnion({"cSteps": SimTypeInt(signed=False, label="UInt32"), "cbCustomData": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)}, name="<anon>", label="None")),)), name="MIXERCONTROLW", pack=False, align=None),
    "MIXERLINEA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwDestination", SimTypeInt(signed=False, label="UInt32")), ("dwSource", SimTypeInt(signed=False, label="UInt32")), ("dwLineID", SimTypeInt(signed=False, label="UInt32")), ("fdwLine", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwComponentType", SimTypeInt(signed=False, label="MIXERLINE_COMPONENTTYPE")), ("cChannels", SimTypeInt(signed=False, label="UInt32")), ("cConnections", SimTypeInt(signed=False, label="UInt32")), ("cControls", SimTypeInt(signed=False, label="UInt32")), ("szShortName", SimTypeArray(SimTypeChar(label="SByte"), 16)), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Target", SimTypeRef("_Target_e__Struct", SimStruct)),)), name="MIXERLINEA", pack=False, align=None),
    "MIXERLINECONTROLSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwLineID", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwControlID": SimTypeInt(signed=False, label="UInt32"), "dwControlType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("cControls", SimTypeInt(signed=False, label="UInt32")), ("cbmxctrl", SimTypeInt(signed=False, label="UInt32")), ("pamxctrl", SimTypePointer(SimTypeRef("MIXERCONTROLA", SimStruct), offset=0)),)), name="MIXERLINECONTROLSA", pack=False, align=None),
    "MIXERLINECONTROLSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwLineID", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwControlID": SimTypeInt(signed=False, label="UInt32"), "dwControlType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("cControls", SimTypeInt(signed=False, label="UInt32")), ("cbmxctrl", SimTypeInt(signed=False, label="UInt32")), ("pamxctrl", SimTypePointer(SimTypeRef("MIXERCONTROLW", SimStruct), offset=0)),)), name="MIXERLINECONTROLSW", pack=False, align=None),
    "MIXERLINEW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwDestination", SimTypeInt(signed=False, label="UInt32")), ("dwSource", SimTypeInt(signed=False, label="UInt32")), ("dwLineID", SimTypeInt(signed=False, label="UInt32")), ("fdwLine", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwComponentType", SimTypeInt(signed=False, label="MIXERLINE_COMPONENTTYPE")), ("cChannels", SimTypeInt(signed=False, label="UInt32")), ("cConnections", SimTypeInt(signed=False, label="UInt32")), ("cControls", SimTypeInt(signed=False, label="UInt32")), ("szShortName", SimTypeArray(SimTypeChar(label="Char"), 16)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("Target", SimTypeRef("_Target_e__Struct", SimStruct)),)), name="MIXERLINEW", pack=False, align=None),
    "MIXERLINE_COMPONENTTYPE": SimTypeInt(signed=False, label="MIXERLINE_COMPONENTTYPE"),
    "MIXEROPENDESC": SimStruct(OrderedDict((("hmx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pReserved0", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dnDevNode", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MIXEROPENDESC", pack=False, align=None),
    "MI_Application": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_ApplicationFT", SimStruct), offset=0)),)), name="MI_Application", pack=False, align=None),
    "MI_ApplicationFT": SimStruct(OrderedDict((("Close", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewHostedProvider", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewDestinationOptions", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewOperationOptions", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewSubscriptionDeliveryOptions", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewSerializer", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewDeserializer", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewInstanceFromClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_ApplicationFT", pack=False, align=None),
    "MI_Array": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Array", pack=False, align=None),
    "MI_ArrayField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Array", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ArrayField", pack=False, align=None),
    "MI_BooleanA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_BooleanA", pack=False, align=None),
    "MI_BooleanAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_BooleanA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_BooleanAField", pack=False, align=None),
    "MI_BooleanField": SimStruct(OrderedDict((("value", SimTypeChar(label="Byte")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_BooleanField", pack=False, align=None),
    "MI_CallbackMode": SimTypeInt(signed=False, label="MI_CallbackMode"),
    "MI_CancelCallback": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="MI_CancellationReason"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["reason", "callbackData"]), offset=0),
    "MI_CancellationReason": SimTypeInt(signed=False, label="MI_CancellationReason"),
    "MI_Char16A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Char16A", pack=False, align=None),
    "MI_Char16AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Char16A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Char16AField", pack=False, align=None),
    "MI_Char16Field": SimStruct(OrderedDict((("value", SimTypeShort(signed=False, label="UInt16")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Char16Field", pack=False, align=None),
    "MI_Class": SimStruct(OrderedDict((("ft", SimTypePointer(SimTypeRef("MI_ClassFT", SimStruct), offset=0)), ("classDecl", SimTypePointer(SimTypeRef("MI_ClassDecl", SimStruct), offset=0)), ("namespaceName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("serverName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 4)),)), name="MI_Class", pack=False, align=None),
    "MI_ClassDecl": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("code", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("qualifiers", SimTypePointer(SimTypePointer(SimTypeRef("MI_Qualifier", SimStruct), offset=0), offset=0)), ("numQualifiers", SimTypeInt(signed=False, label="UInt32")), ("properties", SimTypePointer(SimTypePointer(SimTypeRef("MI_PropertyDecl", SimStruct), offset=0), offset=0)), ("numProperties", SimTypeInt(signed=False, label="UInt32")), ("size", SimTypeInt(signed=False, label="UInt32")), ("superClass", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("superClassDecl", SimTypePointer(SimTypeRef("MI_ClassDecl", SimStruct), offset=0)), ("methods", SimTypePointer(SimTypePointer(SimTypeRef("MI_MethodDecl", SimStruct), offset=0), offset=0)), ("numMethods", SimTypeInt(signed=False, label="UInt32")), ("schema", SimTypePointer(SimTypeRef("MI_SchemaDecl", SimStruct), offset=0)), ("providerFT", SimTypePointer(SimTypeRef("MI_ProviderFT", SimStruct), offset=0)), ("owningClass", SimTypePointer(SimTypeRef("MI_Class", SimStruct), offset=0)),)), name="MI_ClassDecl", pack=False, align=None),
    "MI_ClassFT": SimStruct(OrderedDict((("GetClassNameA", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetNameSpace", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetServerName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElementCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElementAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetClassQualifierSet", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetMethodCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetMethodAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetMethod", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetParentClassName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetParentClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Delete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Clone", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_ClassFT", pack=False, align=None),
    "MI_ClientFT_V1": SimStruct(OrderedDict((("applicationFT", SimTypePointer(SimTypeRef("MI_ApplicationFT", SimStruct), offset=0)), ("sessionFT", SimTypePointer(SimTypeRef("MI_SessionFT", SimStruct), offset=0)), ("operationFT", SimTypePointer(SimTypeRef("MI_OperationFT", SimStruct), offset=0)), ("hostedProviderFT", SimTypePointer(SimTypeRef("MI_HostedProviderFT", SimStruct), offset=0)), ("serializerFT", SimTypePointer(SimTypeRef("MI_SerializerFT", SimStruct), offset=0)), ("deserializerFT", SimTypePointer(SimTypeRef("MI_DeserializerFT", SimStruct), offset=0)), ("subscribeDeliveryOptionsFT", SimTypePointer(SimTypeRef("MI_SubscriptionDeliveryOptionsFT", SimStruct), offset=0)), ("destinationOptionsFT", SimTypePointer(SimTypeRef("MI_DestinationOptionsFT", SimStruct), offset=0)), ("operationOptionsFT", SimTypePointer(SimTypeRef("MI_OperationOptionsFT", SimStruct), offset=0)), ("utilitiesFT", SimTypePointer(SimTypeRef("MI_UtilitiesFT", SimStruct), offset=0)),)), name="MI_ClientFT_V1", pack=False, align=None),
    "MI_ConstBooleanA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstBooleanA", pack=False, align=None),
    "MI_ConstBooleanAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstBooleanA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstBooleanAField", pack=False, align=None),
    "MI_ConstBooleanField": SimStruct(OrderedDict((("value", SimTypeChar(label="Byte")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstBooleanField", pack=False, align=None),
    "MI_ConstChar16A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstChar16A", pack=False, align=None),
    "MI_ConstChar16AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstChar16A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstChar16AField", pack=False, align=None),
    "MI_ConstChar16Field": SimStruct(OrderedDict((("value", SimTypeShort(signed=False, label="UInt16")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstChar16Field", pack=False, align=None),
    "MI_ConstDatetimeA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeRef("MI_Datetime", SimStruct), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstDatetimeA", pack=False, align=None),
    "MI_ConstDatetimeAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstDatetimeA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstDatetimeAField", pack=False, align=None),
    "MI_ConstDatetimeField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Datetime", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstDatetimeField", pack=False, align=None),
    "MI_ConstInstanceA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstInstanceA", pack=False, align=None),
    "MI_ConstInstanceAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstInstanceA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstInstanceAField", pack=False, align=None),
    "MI_ConstInstanceField": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstInstanceField", pack=False, align=None),
    "MI_ConstReal32A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstReal32A", pack=False, align=None),
    "MI_ConstReal32AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstReal32A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstReal32AField", pack=False, align=None),
    "MI_ConstReal32Field": SimStruct(OrderedDict((("value", SimTypeFloat(size=32)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstReal32Field", pack=False, align=None),
    "MI_ConstReal64A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeFloat(size=64), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstReal64A", pack=False, align=None),
    "MI_ConstReal64AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstReal64A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstReal64AField", pack=False, align=None),
    "MI_ConstReal64Field": SimStruct(OrderedDict((("value", SimTypeFloat(size=64)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstReal64Field", pack=False, align=None),
    "MI_ConstReferenceA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstReferenceA", pack=False, align=None),
    "MI_ConstReferenceAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstReferenceA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstReferenceAField", pack=False, align=None),
    "MI_ConstReferenceField": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstReferenceField", pack=False, align=None),
    "MI_ConstSint16A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstSint16A", pack=False, align=None),
    "MI_ConstSint16AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstSint16A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint16AField", pack=False, align=None),
    "MI_ConstSint16Field": SimStruct(OrderedDict((("value", SimTypeShort(signed=True, label="Int16")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint16Field", pack=False, align=None),
    "MI_ConstSint32A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstSint32A", pack=False, align=None),
    "MI_ConstSint32AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstSint32A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint32AField", pack=False, align=None),
    "MI_ConstSint32Field": SimStruct(OrderedDict((("value", SimTypeInt(signed=True, label="Int32")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint32Field", pack=False, align=None),
    "MI_ConstSint64A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstSint64A", pack=False, align=None),
    "MI_ConstSint64AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstSint64A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint64AField", pack=False, align=None),
    "MI_ConstSint64Field": SimStruct(OrderedDict((("value", SimTypeLongLong(signed=True, label="Int64")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint64Field", pack=False, align=None),
    "MI_ConstSint8A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstSint8A", pack=False, align=None),
    "MI_ConstSint8AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstSint8A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint8AField", pack=False, align=None),
    "MI_ConstSint8Field": SimStruct(OrderedDict((("value", SimTypeChar(label="SByte")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstSint8Field", pack=False, align=None),
    "MI_ConstStringA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstStringA", pack=False, align=None),
    "MI_ConstStringAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstStringA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstStringAField", pack=False, align=None),
    "MI_ConstStringField": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstStringField", pack=False, align=None),
    "MI_ConstUint16A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstUint16A", pack=False, align=None),
    "MI_ConstUint16AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstUint16A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint16AField", pack=False, align=None),
    "MI_ConstUint16Field": SimStruct(OrderedDict((("value", SimTypeShort(signed=False, label="UInt16")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint16Field", pack=False, align=None),
    "MI_ConstUint32A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstUint32A", pack=False, align=None),
    "MI_ConstUint32AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstUint32A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint32AField", pack=False, align=None),
    "MI_ConstUint32Field": SimStruct(OrderedDict((("value", SimTypeInt(signed=False, label="UInt32")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint32Field", pack=False, align=None),
    "MI_ConstUint64A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstUint64A", pack=False, align=None),
    "MI_ConstUint64AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstUint64A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint64AField", pack=False, align=None),
    "MI_ConstUint64Field": SimStruct(OrderedDict((("value", SimTypeLongLong(signed=False, label="UInt64")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint64Field", pack=False, align=None),
    "MI_ConstUint8A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ConstUint8A", pack=False, align=None),
    "MI_ConstUint8AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ConstUint8A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint8AField", pack=False, align=None),
    "MI_ConstUint8Field": SimStruct(OrderedDict((("value", SimTypeChar(label="Byte")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ConstUint8Field", pack=False, align=None),
    "MI_Context": SimStruct(OrderedDict((("ft", SimTypePointer(SimTypeRef("MI_ContextFT", SimStruct), offset=0)), ("reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 3)),)), name="MI_Context", pack=False, align=None),
    "MI_ContextFT": SimStruct(OrderedDict((("PostResult", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PostInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PostIndication", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ConstructInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ConstructParameters", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewDynamicInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NewParameters", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Canceled", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetLocale", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RegisterCancel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RequestUnload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RefuseUnload", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetLocalSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetStringOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetStringOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetNumberOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCustomOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCustomOptionCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCustomOptionAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("WriteMessage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("WriteProgress", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("WriteStreamParameter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("WriteCimError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PromptUser", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ShouldProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ShouldContinue", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PostError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PostCimError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("WriteError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_ContextFT", pack=False, align=None),
    "MI_Datetime": SimStruct(OrderedDict((("isTimestamp", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"timestamp": SimTypeRef("MI_Timestamp", SimStruct), "interval": SimTypeRef("MI_Interval", SimStruct)}, name="<anon>", label="None")),)), name="MI_Datetime", pack=False, align=None),
    "MI_DatetimeA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeRef("MI_Datetime", SimStruct), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_DatetimeA", pack=False, align=None),
    "MI_DatetimeAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_DatetimeA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_DatetimeAField", pack=False, align=None),
    "MI_DatetimeField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Datetime", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_DatetimeField", pack=False, align=None),
    "MI_Deserializer": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_Deserializer", pack=False, align=None),
    "MI_DeserializerFT": SimStruct(OrderedDict((("Close", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeserializeClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Class_GetClassName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Class_GetParentClassName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeserializeInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Instance_GetClassName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_DeserializerFT", pack=False, align=None),
    "MI_Deserializer_ClassObjectNeeded": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MI_Class", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="MI_Result"), arg_names=["context", "serverName", "namespaceName", "className", "requestedClassObject"]), offset=0),
    "MI_DestinationOptions": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_DestinationOptionsFT", SimStruct), offset=0)),)), name="MI_DestinationOptions", pack=False, align=None),
    "MI_DestinationOptionsFT": SimStruct(OrderedDict((("Delete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetNumber", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddCredentials", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetNumber", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOptionCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOptionAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCredentialsCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCredentialsAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCredentialsPasswordAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Clone", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetInterval", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetInterval", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_DestinationOptionsFT", pack=False, align=None),
    "MI_DestinationOptions_ImpersonationType": SimTypeInt(signed=False, label="MI_DestinationOptions_ImpersonationType"),
    "MI_ErrorCategory": SimTypeInt(signed=False, label="MI_ErrorCategory"),
    "MI_FeatureDecl": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("code", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("qualifiers", SimTypePointer(SimTypePointer(SimTypeRef("MI_Qualifier", SimStruct), offset=0), offset=0)), ("numQualifiers", SimTypeInt(signed=False, label="UInt32")),)), name="MI_FeatureDecl", pack=False, align=None),
    "MI_Filter": SimStruct(OrderedDict((("ft", SimTypePointer(SimTypeRef("MI_FilterFT", SimStruct), offset=0)), ("reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 3)),)), name="MI_Filter", pack=False, align=None),
    "MI_FilterFT": SimStruct(OrderedDict((("Evaluate", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetExpression", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_FilterFT", pack=False, align=None),
    "MI_HostedProvider": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_HostedProviderFT", SimStruct), offset=0)),)), name="MI_HostedProvider", pack=False, align=None),
    "MI_HostedProviderFT": SimStruct(OrderedDict((("Close", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetApplication", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_HostedProviderFT", pack=False, align=None),
    "MI_Instance": SimStruct(OrderedDict((("ft", SimTypePointer(SimTypeRef("MI_InstanceFT", SimStruct), offset=0)), ("classDecl", SimTypePointer(SimTypeRef("MI_ClassDecl", SimStruct), offset=0)), ("serverName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("nameSpace", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 4)),)), name="MI_Instance", pack=False, align=None),
    "MI_InstanceA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_InstanceA", pack=False, align=None),
    "MI_InstanceAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_InstanceA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_InstanceAField", pack=False, align=None),
    "MI_InstanceExFT": SimStruct(OrderedDict((("parent", SimTypeRef("MI_InstanceFT", SimStruct)), ("Normalize", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_InstanceExFT", pack=False, align=None),
    "MI_InstanceFT": SimStruct(OrderedDict((("Clone", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Destruct", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Delete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IsA", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetClassNameA", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetNameSpace", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetNameSpace", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElementCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetElementAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElementAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ClearElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ClearElementAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetServerName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetServerName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_InstanceFT", pack=False, align=None),
    "MI_InstanceField": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_InstanceField", pack=False, align=None),
    "MI_Interval": SimStruct(OrderedDict((("days", SimTypeInt(signed=False, label="UInt32")), ("hours", SimTypeInt(signed=False, label="UInt32")), ("minutes", SimTypeInt(signed=False, label="UInt32")), ("seconds", SimTypeInt(signed=False, label="UInt32")), ("microseconds", SimTypeInt(signed=False, label="UInt32")), ("__padding1", SimTypeInt(signed=False, label="UInt32")), ("__padding2", SimTypeInt(signed=False, label="UInt32")), ("__padding3", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Interval", pack=False, align=None),
    "MI_LocaleType": SimTypeInt(signed=False, label="MI_LocaleType"),
    "MI_MainFunction": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Server", SimStruct), offset=0)], SimTypePointer(SimTypeRef("MI_Module", SimStruct), offset=0), arg_names=["server"]), offset=0),
    "MI_MethodDecl": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("code", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("qualifiers", SimTypePointer(SimTypePointer(SimTypeRef("MI_Qualifier", SimStruct), offset=0), offset=0)), ("numQualifiers", SimTypeInt(signed=False, label="UInt32")), ("parameters", SimTypePointer(SimTypePointer(SimTypeRef("MI_ParameterDecl", SimStruct), offset=0), offset=0)), ("numParameters", SimTypeInt(signed=False, label="UInt32")), ("size", SimTypeInt(signed=False, label="UInt32")), ("returnType", SimTypeInt(signed=False, label="UInt32")), ("origin", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("propagator", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("schema", SimTypePointer(SimTypeRef("MI_SchemaDecl", SimStruct), offset=0)), ("function", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "methodName", "instanceName", "parameters"]), offset=0)),)), name="MI_MethodDecl", pack=False, align=None),
    "MI_MethodDecl_Invoke": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "methodName", "instanceName", "parameters"]), offset=0),
    "MI_Module": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("generatorVersion", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("charSize", SimTypeInt(signed=False, label="UInt32")), ("schemaDecl", SimTypePointer(SimTypeRef("MI_SchemaDecl", SimStruct), offset=0)), ("Load", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context"]), offset=0)), ("Unload", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context"]), offset=0)), ("dynamicProviderFT", SimTypePointer(SimTypeRef("MI_ProviderFT", SimStruct), offset=0)),)), name="MI_Module", pack=False, align=None),
    "MI_Module_Load": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context"]), offset=0),
    "MI_Module_Self": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "MI_Module_Unload": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context"]), offset=0),
    "MI_ObjectDecl": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("code", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("qualifiers", SimTypePointer(SimTypePointer(SimTypeRef("MI_Qualifier", SimStruct), offset=0), offset=0)), ("numQualifiers", SimTypeInt(signed=False, label="UInt32")), ("properties", SimTypePointer(SimTypePointer(SimTypeRef("MI_PropertyDecl", SimStruct), offset=0), offset=0)), ("numProperties", SimTypeInt(signed=False, label="UInt32")), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ObjectDecl", pack=False, align=None),
    "MI_Operation": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_OperationFT", SimStruct), offset=0)),)), name="MI_Operation", pack=False, align=None),
    "MI_OperationCallback_Class": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Class", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="MI_Result"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "classResult", "moreResults", "resultCode", "errorString", "errorDetails", "resultAcknowledgement"]), offset=0),
    "MI_OperationCallback_Indication": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="MI_Result"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "instance", "bookmark", "machineID", "moreResults", "resultCode", "errorString", "errorDetails", "resultAcknowledgement"]), offset=0),
    "MI_OperationCallback_Instance": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="MI_Result"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "instance", "moreResults", "resultCode", "errorString", "errorDetails", "resultAcknowledgement"]), offset=0),
    "MI_OperationCallback_PromptUser": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="MI_PromptType"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "message", "promptType", "promptUserResult"]), offset=0),
    "MI_OperationCallback_ResponseType": SimTypeInt(signed=False, label="MI_OperationCallback_ResponseType"),
    "MI_OperationCallback_StreamedParameter": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="MI_Type"), SimTypePointer(SimUnion({"boolean": SimTypeChar(label="Byte"), "uint8": SimTypeChar(label="Byte"), "sint8": SimTypeChar(label="SByte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "sint16": SimTypeShort(signed=True, label="Int16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "sint32": SimTypeInt(signed=True, label="Int32"), "uint64": SimTypeLongLong(signed=False, label="UInt64"), "sint64": SimTypeLongLong(signed=True, label="Int64"), "real32": SimTypeFloat(size=32), "real64": SimTypeFloat(size=64), "char16": SimTypeShort(signed=False, label="UInt16"), "datetime": SimTypeRef("MI_Datetime", SimStruct), "string": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "instance": SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), "reference": SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), "booleana": SimTypeRef("MI_BooleanA", SimStruct), "uint8a": SimTypeRef("MI_Uint8A", SimStruct), "sint8a": SimTypeRef("MI_Sint8A", SimStruct), "uint16a": SimTypeRef("MI_Uint16A", SimStruct), "sint16a": SimTypeRef("MI_Sint16A", SimStruct), "uint32a": SimTypeRef("MI_Uint32A", SimStruct), "sint32a": SimTypeRef("MI_Sint32A", SimStruct), "uint64a": SimTypeRef("MI_Uint64A", SimStruct), "sint64a": SimTypeRef("MI_Sint64A", SimStruct), "real32a": SimTypeRef("MI_Real32A", SimStruct), "real64a": SimTypeRef("MI_Real64A", SimStruct), "char16a": SimTypeRef("MI_Char16A", SimStruct), "datetimea": SimTypeRef("MI_DatetimeA", SimStruct), "stringa": SimTypeRef("MI_StringA", SimStruct), "referencea": SimTypeRef("MI_ReferenceA", SimStruct), "instancea": SimTypeRef("MI_InstanceA", SimStruct), "array": SimTypeRef("MI_Array", SimStruct)}, name="<anon>", label="None"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "parameterName", "resultType", "result", "resultAcknowledgement"]), offset=0),
    "MI_OperationCallback_WriteError": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "instance", "writeErrorResult"]), offset=0),
    "MI_OperationCallback_WriteMessage": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "channel", "message"]), offset=0),
    "MI_OperationCallback_WriteProgress": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "activity", "currentOperation", "statusDescription", "percentageComplete", "secondsRemaining"]), offset=0),
    "MI_OperationCallbacks": SimStruct(OrderedDict((("callbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("promptUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="MI_PromptType"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "message", "promptType", "promptUserResult"]), offset=0)), ("writeError", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "instance", "writeErrorResult"]), offset=0)), ("writeMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "channel", "message"]), offset=0)), ("writeProgress", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "activity", "currentOperation", "statusDescription", "percentageComplete", "secondsRemaining"]), offset=0)), ("instanceResult", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="MI_Result"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "instance", "moreResults", "resultCode", "errorString", "errorDetails", "resultAcknowledgement"]), offset=0)), ("indicationResult", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="MI_Result"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "instance", "bookmark", "machineID", "moreResults", "resultCode", "errorString", "errorDetails", "resultAcknowledgement"]), offset=0)), ("classResult", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Class", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="MI_Result"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "classResult", "moreResults", "resultCode", "errorString", "errorDetails", "resultAcknowledgement"]), offset=0)), ("streamedParameterResult", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MI_Operation", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="MI_Type"), SimTypePointer(SimUnion({"boolean": SimTypeChar(label="Byte"), "uint8": SimTypeChar(label="Byte"), "sint8": SimTypeChar(label="SByte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "sint16": SimTypeShort(signed=True, label="Int16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "sint32": SimTypeInt(signed=True, label="Int32"), "uint64": SimTypeLongLong(signed=False, label="UInt64"), "sint64": SimTypeLongLong(signed=True, label="Int64"), "real32": SimTypeFloat(size=32), "real64": SimTypeFloat(size=64), "char16": SimTypeShort(signed=False, label="UInt16"), "datetime": SimTypeRef("MI_Datetime", SimStruct), "string": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "instance": SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), "reference": SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), "booleana": SimTypeRef("MI_BooleanA", SimStruct), "uint8a": SimTypeRef("MI_Uint8A", SimStruct), "sint8a": SimTypeRef("MI_Sint8A", SimStruct), "uint16a": SimTypeRef("MI_Uint16A", SimStruct), "sint16a": SimTypeRef("MI_Sint16A", SimStruct), "uint32a": SimTypeRef("MI_Uint32A", SimStruct), "sint32a": SimTypeRef("MI_Sint32A", SimStruct), "uint64a": SimTypeRef("MI_Uint64A", SimStruct), "sint64a": SimTypeRef("MI_Sint64A", SimStruct), "real32a": SimTypeRef("MI_Real32A", SimStruct), "real64a": SimTypeRef("MI_Real64A", SimStruct), "char16a": SimTypeRef("MI_Char16A", SimStruct), "datetimea": SimTypeRef("MI_DatetimeA", SimStruct), "stringa": SimTypeRef("MI_StringA", SimStruct), "referencea": SimTypeRef("MI_ReferenceA", SimStruct), "instancea": SimTypeRef("MI_InstanceA", SimStruct), "array": SimTypeRef("MI_Array", SimStruct)}, name="<anon>", label="None"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["operation", "callbackContext", "parameterName", "resultType", "result", "resultAcknowledgement"]), offset=0)),)), name="MI_OperationCallbacks", pack=False, align=None),
    "MI_OperationFT": SimStruct(OrderedDict((("Close", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Cancel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetSession", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetIndication", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_OperationFT", pack=False, align=None),
    "MI_OperationOptions": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_OperationOptionsFT", SimStruct), offset=0)),)), name="MI_OperationOptions", pack=False, align=None),
    "MI_OperationOptionsFT": SimStruct(OrderedDict((("Delete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetNumber", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetCustomOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetNumber", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOptionCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOptionAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetEnabledChannels", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Clone", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetInterval", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetInterval", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_OperationOptionsFT", pack=False, align=None),
    "MI_ParameterDecl": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("code", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("qualifiers", SimTypePointer(SimTypePointer(SimTypeRef("MI_Qualifier", SimStruct), offset=0), offset=0)), ("numQualifiers", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="UInt32")), ("className", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("subscript", SimTypeInt(signed=False, label="UInt32")), ("offset", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ParameterDecl", pack=False, align=None),
    "MI_ParameterSet": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_ParameterSetFT", SimStruct), offset=0)),)), name="MI_ParameterSet", pack=False, align=None),
    "MI_ParameterSetFT": SimStruct(OrderedDict((("GetMethodReturnType", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetParameterCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetParameterAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetParameter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_ParameterSetFT", pack=False, align=None),
    "MI_PromptType": SimTypeInt(signed=False, label="MI_PromptType"),
    "MI_PropertyDecl": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("code", SimTypeInt(signed=False, label="UInt32")), ("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("qualifiers", SimTypePointer(SimTypePointer(SimTypeRef("MI_Qualifier", SimStruct), offset=0), offset=0)), ("numQualifiers", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="UInt32")), ("className", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("subscript", SimTypeInt(signed=False, label="UInt32")), ("offset", SimTypeInt(signed=False, label="UInt32")), ("origin", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("propagator", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MI_PropertyDecl", pack=False, align=None),
    "MI_PropertySet": SimStruct(OrderedDict((("ft", SimTypePointer(SimTypeRef("MI_PropertySetFT", SimStruct), offset=0)), ("reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 3)),)), name="MI_PropertySet", pack=False, align=None),
    "MI_PropertySetFT": SimStruct(OrderedDict((("GetElementCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ContainsElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddElement", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetElementAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Clear", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Destruct", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Delete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Clone", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_PropertySetFT", pack=False, align=None),
    "MI_ProviderArchitecture": SimTypeInt(signed=False, label="MI_ProviderArchitecture"),
    "MI_ProviderFT": SimStruct(OrderedDict((("Load", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "selfModule", "context"]), offset=0)), ("Unload", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context"]), offset=0)), ("GetInstance", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName", "propertySet"]), offset=0)), ("EnumerateInstances", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "propertySet", "keysOnly", "filter"]), offset=0)), ("CreateInstance", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "newInstance"]), offset=0)), ("ModifyInstance", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "modifiedInstance", "propertySet"]), offset=0)), ("DeleteInstance", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName"]), offset=0)), ("AssociatorInstances", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName", "resultClass", "role", "resultRole", "propertySet", "keysOnly", "filter"]), offset=0)), ("ReferenceInstances", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName", "role", "propertySet", "keysOnly", "filter"]), offset=0)), ("EnableIndications", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "indicationsContext", "nameSpace", "className"]), offset=0)), ("DisableIndications", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "indicationsContext", "nameSpace", "className"]), offset=0)), ("Subscribe", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "filter", "bookmark", "subscriptionID", "subscriptionSelf"]), offset=0)), ("Unsubscribe", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "subscriptionID", "subscriptionSelf"]), offset=0)), ("Invoke", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "methodName", "instanceName", "inputParameters"]), offset=0)),)), name="MI_ProviderFT", pack=False, align=None),
    "MI_ProviderFT_AssociatorInstances": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName", "resultClass", "role", "resultRole", "propertySet", "keysOnly", "filter"]), offset=0),
    "MI_ProviderFT_CreateInstance": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "newInstance"]), offset=0),
    "MI_ProviderFT_DeleteInstance": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName"]), offset=0),
    "MI_ProviderFT_DisableIndications": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "indicationsContext", "nameSpace", "className"]), offset=0),
    "MI_ProviderFT_EnableIndications": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "indicationsContext", "nameSpace", "className"]), offset=0),
    "MI_ProviderFT_EnumerateInstances": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "propertySet", "keysOnly", "filter"]), offset=0),
    "MI_ProviderFT_GetInstance": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName", "propertySet"]), offset=0),
    "MI_ProviderFT_Invoke": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "methodName", "instanceName", "inputParameters"]), offset=0),
    "MI_ProviderFT_Load": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "selfModule", "context"]), offset=0),
    "MI_ProviderFT_ModifyInstance": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "modifiedInstance", "propertySet"]), offset=0),
    "MI_ProviderFT_ReferenceInstances": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_PropertySet", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "instanceName", "role", "propertySet", "keysOnly", "filter"]), offset=0),
    "MI_ProviderFT_Subscribe": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("MI_Filter", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "filter", "bookmark", "subscriptionID", "subscriptionSelf"]), offset=0),
    "MI_ProviderFT_Unload": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context"]), offset=0),
    "MI_ProviderFT_Unsubscribe": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MI_Context", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["self", "context", "nameSpace", "className", "subscriptionID", "subscriptionSelf"]), offset=0),
    "MI_Qualifier": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("type", SimTypeInt(signed=False, label="UInt32")), ("flavor", SimTypeInt(signed=False, label="UInt32")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MI_Qualifier", pack=False, align=None),
    "MI_QualifierDecl": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("type", SimTypeInt(signed=False, label="UInt32")), ("scope", SimTypeInt(signed=False, label="UInt32")), ("flavor", SimTypeInt(signed=False, label="UInt32")), ("subscript", SimTypeInt(signed=False, label="UInt32")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MI_QualifierDecl", pack=False, align=None),
    "MI_QualifierSet": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_QualifierSetFT", SimStruct), offset=0)),)), name="MI_QualifierSet", pack=False, align=None),
    "MI_QualifierSetFT": SimStruct(OrderedDict((("GetQualifierCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetQualifierAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetQualifier", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_QualifierSetFT", pack=False, align=None),
    "MI_Real32A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Real32A", pack=False, align=None),
    "MI_Real32AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Real32A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Real32AField", pack=False, align=None),
    "MI_Real32Field": SimStruct(OrderedDict((("value", SimTypeFloat(size=32)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Real32Field", pack=False, align=None),
    "MI_Real64A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeFloat(size=64), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Real64A", pack=False, align=None),
    "MI_Real64AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Real64A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Real64AField", pack=False, align=None),
    "MI_Real64Field": SimStruct(OrderedDict((("value", SimTypeFloat(size=64)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Real64Field", pack=False, align=None),
    "MI_ReferenceA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_ReferenceA", pack=False, align=None),
    "MI_ReferenceAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_ReferenceA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ReferenceAField", pack=False, align=None),
    "MI_ReferenceField": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_ReferenceField", pack=False, align=None),
    "MI_Result": SimTypeInt(signed=False, label="MI_Result"),
    "MI_SchemaDecl": SimStruct(OrderedDict((("qualifierDecls", SimTypePointer(SimTypePointer(SimTypeRef("MI_QualifierDecl", SimStruct), offset=0), offset=0)), ("numQualifierDecls", SimTypeInt(signed=False, label="UInt32")), ("classDecls", SimTypePointer(SimTypePointer(SimTypeRef("MI_ClassDecl", SimStruct), offset=0), offset=0)), ("numClassDecls", SimTypeInt(signed=False, label="UInt32")),)), name="MI_SchemaDecl", pack=False, align=None),
    "MI_Serializer": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_Serializer", pack=False, align=None),
    "MI_SerializerFT": SimStruct(OrderedDict((("Close", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SerializeClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SerializeInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_SerializerFT", pack=False, align=None),
    "MI_Server": SimStruct(OrderedDict((("serverFT", SimTypePointer(SimTypeRef("MI_ServerFT", SimStruct), offset=0)), ("contextFT", SimTypePointer(SimTypeRef("MI_ContextFT", SimStruct), offset=0)), ("instanceFT", SimTypePointer(SimTypeRef("MI_InstanceFT", SimStruct), offset=0)), ("propertySetFT", SimTypePointer(SimTypeRef("MI_PropertySetFT", SimStruct), offset=0)), ("filterFT", SimTypePointer(SimTypeRef("MI_FilterFT", SimStruct), offset=0)),)), name="MI_Server", pack=False, align=None),
    "MI_ServerFT": SimStruct(OrderedDict((("GetVersion", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetSystemName", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_ServerFT", pack=False, align=None),
    "MI_Session": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_SessionFT", SimStruct), offset=0)),)), name="MI_Session", pack=False, align=None),
    "MI_SessionCallbacks": SimStruct(OrderedDict((("callbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("writeMessage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("writeError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_SessionCallbacks", pack=False, align=None),
    "MI_SessionFT": SimStruct(OrderedDict((("Close", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetApplication", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ModifyInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CreateInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeleteInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Invoke", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EnumerateInstances", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("QueryInstances", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AssociatorInstances", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ReferenceInstances", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Subscribe", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EnumerateClasses", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("TestConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_SessionFT", pack=False, align=None),
    "MI_Sint16A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Sint16A", pack=False, align=None),
    "MI_Sint16AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Sint16A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint16AField", pack=False, align=None),
    "MI_Sint16Field": SimStruct(OrderedDict((("value", SimTypeShort(signed=True, label="Int16")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint16Field", pack=False, align=None),
    "MI_Sint32A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Sint32A", pack=False, align=None),
    "MI_Sint32AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Sint32A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint32AField", pack=False, align=None),
    "MI_Sint32Field": SimStruct(OrderedDict((("value", SimTypeInt(signed=True, label="Int32")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint32Field", pack=False, align=None),
    "MI_Sint64A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Sint64A", pack=False, align=None),
    "MI_Sint64AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Sint64A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint64AField", pack=False, align=None),
    "MI_Sint64Field": SimStruct(OrderedDict((("value", SimTypeLongLong(signed=True, label="Int64")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint64Field", pack=False, align=None),
    "MI_Sint8A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Sint8A", pack=False, align=None),
    "MI_Sint8AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Sint8A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint8AField", pack=False, align=None),
    "MI_Sint8Field": SimStruct(OrderedDict((("value", SimTypeChar(label="SByte")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Sint8Field", pack=False, align=None),
    "MI_StringA": SimStruct(OrderedDict((("data", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_StringA", pack=False, align=None),
    "MI_StringAField": SimStruct(OrderedDict((("value", SimTypeRef("MI_StringA", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_StringAField", pack=False, align=None),
    "MI_StringField": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_StringField", pack=False, align=None),
    "MI_SubscriptionDeliveryOptions": SimStruct(OrderedDict((("reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("reserved2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ft", SimTypePointer(SimTypeRef("MI_SubscriptionDeliveryOptionsFT", SimStruct), offset=0)),)), name="MI_SubscriptionDeliveryOptions", pack=False, align=None),
    "MI_SubscriptionDeliveryOptionsFT": SimStruct(OrderedDict((("SetString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetNumber", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetDateTime", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetInterval", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AddCredentials", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Delete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetNumber", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetDateTime", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetInterval", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOptionCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOptionAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetOption", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCredentialsCount", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCredentialsAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetCredentialsPasswordAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Clone", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_SubscriptionDeliveryOptionsFT", pack=False, align=None),
    "MI_SubscriptionDeliveryType": SimTypeInt(signed=False, label="MI_SubscriptionDeliveryType"),
    "MI_Timestamp": SimStruct(OrderedDict((("year", SimTypeInt(signed=False, label="UInt32")), ("month", SimTypeInt(signed=False, label="UInt32")), ("day", SimTypeInt(signed=False, label="UInt32")), ("hour", SimTypeInt(signed=False, label="UInt32")), ("minute", SimTypeInt(signed=False, label="UInt32")), ("second", SimTypeInt(signed=False, label="UInt32")), ("microseconds", SimTypeInt(signed=False, label="UInt32")), ("utc", SimTypeInt(signed=True, label="Int32")),)), name="MI_Timestamp", pack=False, align=None),
    "MI_Type": SimTypeInt(signed=False, label="MI_Type"),
    "MI_Uint16A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Uint16A", pack=False, align=None),
    "MI_Uint16AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Uint16A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint16AField", pack=False, align=None),
    "MI_Uint16Field": SimStruct(OrderedDict((("value", SimTypeShort(signed=False, label="UInt16")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint16Field", pack=False, align=None),
    "MI_Uint32A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Uint32A", pack=False, align=None),
    "MI_Uint32AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Uint32A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint32AField", pack=False, align=None),
    "MI_Uint32Field": SimStruct(OrderedDict((("value", SimTypeInt(signed=False, label="UInt32")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint32Field", pack=False, align=None),
    "MI_Uint64A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Uint64A", pack=False, align=None),
    "MI_Uint64AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Uint64A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint64AField", pack=False, align=None),
    "MI_Uint64Field": SimStruct(OrderedDict((("value", SimTypeLongLong(signed=False, label="UInt64")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint64Field", pack=False, align=None),
    "MI_Uint8A": SimStruct(OrderedDict((("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")),)), name="MI_Uint8A", pack=False, align=None),
    "MI_Uint8AField": SimStruct(OrderedDict((("value", SimTypeRef("MI_Uint8A", SimStruct)), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint8AField", pack=False, align=None),
    "MI_Uint8Field": SimStruct(OrderedDict((("value", SimTypeChar(label="Byte")), ("exists", SimTypeChar(label="Byte")), ("flags", SimTypeChar(label="Byte")),)), name="MI_Uint8Field", pack=False, align=None),
    "MI_UserCredentials": SimStruct(OrderedDict((("authenticationType", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("credentials", SimUnion({"usernamePassword": SimTypeRef("MI_UsernamePasswordCreds", SimStruct), "certificateThumbprint": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="<anon>", label="None")),)), name="MI_UserCredentials", pack=False, align=None),
    "MI_UsernamePasswordCreds": SimStruct(OrderedDict((("domain", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("username", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("password", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="MI_UsernamePasswordCreds", pack=False, align=None),
    "MI_UtilitiesFT": SimStruct(OrderedDict((("MapErrorToMiErrorCategory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CimErrorFromErrorCode", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MI_UtilitiesFT", pack=False, align=None),
    "MI_Value": SimUnion({"boolean": SimTypeChar(label="Byte"), "uint8": SimTypeChar(label="Byte"), "sint8": SimTypeChar(label="SByte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "sint16": SimTypeShort(signed=True, label="Int16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "sint32": SimTypeInt(signed=True, label="Int32"), "uint64": SimTypeLongLong(signed=False, label="UInt64"), "sint64": SimTypeLongLong(signed=True, label="Int64"), "real32": SimTypeFloat(size=32), "real64": SimTypeFloat(size=64), "char16": SimTypeShort(signed=False, label="UInt16"), "datetime": SimTypeRef("MI_Datetime", SimStruct), "string": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "instance": SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), "reference": SimTypePointer(SimTypeRef("MI_Instance", SimStruct), offset=0), "booleana": SimTypeRef("MI_BooleanA", SimStruct), "uint8a": SimTypeRef("MI_Uint8A", SimStruct), "sint8a": SimTypeRef("MI_Sint8A", SimStruct), "uint16a": SimTypeRef("MI_Uint16A", SimStruct), "sint16a": SimTypeRef("MI_Sint16A", SimStruct), "uint32a": SimTypeRef("MI_Uint32A", SimStruct), "sint32a": SimTypeRef("MI_Sint32A", SimStruct), "uint64a": SimTypeRef("MI_Uint64A", SimStruct), "sint64a": SimTypeRef("MI_Sint64A", SimStruct), "real32a": SimTypeRef("MI_Real32A", SimStruct), "real64a": SimTypeRef("MI_Real64A", SimStruct), "char16a": SimTypeRef("MI_Char16A", SimStruct), "datetimea": SimTypeRef("MI_DatetimeA", SimStruct), "stringa": SimTypeRef("MI_StringA", SimStruct), "referencea": SimTypeRef("MI_ReferenceA", SimStruct), "instancea": SimTypeRef("MI_InstanceA", SimStruct), "array": SimTypeRef("MI_Array", SimStruct)}, name="<anon>", label="None"),
    "MKRREDUCE": SimTypeInt(signed=False, label="MKRREDUCE"),
    "MKSYS": SimTypeInt(signed=False, label="MKSYS"),
    "MLCONVCHAR": SimTypeInt(signed=False, label="MLCONVCHAR"),
    "MLCP": SimTypeInt(signed=False, label="MLCP"),
    "MLDETECTCP": SimTypeInt(signed=False, label="MLDETECTCP"),
    "MLDV2_QUERY_HEADER": SimStruct(OrderedDict((("IcmpHeader", SimTypeRef("ICMP_HEADER", SimStruct)), ("Anonymous1", SimUnion({"MaxRespCode": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("MulticastAddress", SimTypeRef("IN6_ADDR", SimStruct)), ("_bitfield", SimTypeChar(label="Byte")), ("Anonymous2", SimUnion({"QueriersQueryInterfaceCode": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("SourceCount", SimTypeShort(signed=False, label="UInt16")),)), name="MLDV2_QUERY_HEADER", pack=False, align=None),
    "MLDV2_REPORT_HEADER": SimStruct(OrderedDict((("IcmpHeader", SimTypeRef("ICMP_HEADER", SimStruct)), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("RecordCount", SimTypeShort(signed=False, label="UInt16")),)), name="MLDV2_REPORT_HEADER", pack=False, align=None),
    "MLDV2_REPORT_RECORD_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("AuxillaryDataLength", SimTypeChar(label="Byte")), ("SourceCount", SimTypeShort(signed=False, label="UInt16")), ("MulticastAddress", SimTypeRef("IN6_ADDR", SimStruct)),)), name="MLDV2_REPORT_RECORD_HEADER", pack=False, align=None),
    "MLD_HEADER": SimStruct(OrderedDict((("IcmpHeader", SimTypeRef("ICMP_HEADER", SimStruct)), ("MaxRespTime", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("MulticastAddress", SimTypeRef("IN6_ADDR", SimStruct)),)), name="MLD_HEADER", pack=False, align=None),
    "MLD_MAX_RESP_CODE_TYPE": SimTypeInt(signed=False, label="MLD_MAX_RESP_CODE_TYPE"),
    "MLOperatorAttribute": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("type", SimTypeInt(signed=False, label="MLOperatorAttributeType")), ("required", SimTypeChar(label="Byte")),)), name="MLOperatorAttribute", pack=False, align=None),
    "MLOperatorAttributeNameValue": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("type", SimTypeInt(signed=False, label="MLOperatorAttributeType")), ("valueCount", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"reserved": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ints": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "strings": SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0), "floats": SimTypePointer(SimTypeFloat(size=32), offset=0)}, name="<anon>", label="None")),)), name="MLOperatorAttributeNameValue", pack=False, align=None),
    "MLOperatorAttributeType": SimTypeInt(signed=False, label="MLOperatorAttributeType"),
    "MLOperatorEdgeDescription": SimStruct(OrderedDict((("edgeType", SimTypeInt(signed=False, label="MLOperatorEdgeType")), ("Anonymous", SimUnion({"reserved": SimTypeLongLong(signed=False, label="UInt64"), "tensorDataType": SimTypeInt(signed=False, label="MLOperatorTensorDataType")}, name="<anon>", label="None")),)), name="MLOperatorEdgeDescription", pack=False, align=None),
    "MLOperatorEdgeType": SimTypeInt(signed=False, label="MLOperatorEdgeType"),
    "MLOperatorEdgeTypeConstraint": SimStruct(OrderedDict((("typeLabel", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("allowedTypes", SimTypePointer(SimTypeRef("MLOperatorEdgeDescription", SimStruct), offset=0)), ("allowedTypeCount", SimTypeInt(signed=False, label="UInt32")),)), name="MLOperatorEdgeTypeConstraint", pack=False, align=None),
    "MLOperatorExecutionType": SimTypeInt(signed=False, label="MLOperatorExecutionType"),
    "MLOperatorKernelDescription": SimStruct(OrderedDict((("domain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("minimumOperatorSetVersion", SimTypeInt(signed=True, label="Int32")), ("executionType", SimTypeInt(signed=False, label="MLOperatorExecutionType")), ("typeConstraints", SimTypePointer(SimTypeRef("MLOperatorEdgeTypeConstraint", SimStruct), offset=0)), ("typeConstraintCount", SimTypeInt(signed=False, label="UInt32")), ("defaultAttributes", SimTypePointer(SimTypeRef("MLOperatorAttributeNameValue", SimStruct), offset=0)), ("defaultAttributeCount", SimTypeInt(signed=False, label="UInt32")), ("options", SimTypeInt(signed=False, label="MLOperatorKernelOptions")), ("executionOptions", SimTypeInt(signed=False, label="UInt32")),)), name="MLOperatorKernelDescription", pack=False, align=None),
    "MLOperatorKernelOptions": SimTypeInt(signed=False, label="MLOperatorKernelOptions"),
    "MLOperatorParameterOptions": SimTypeInt(signed=False, label="MLOperatorParameterOptions"),
    "MLOperatorSchemaDescription": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("operatorSetVersionAtLastChange", SimTypeInt(signed=True, label="Int32")), ("inputs", SimTypePointer(SimTypeRef("MLOperatorSchemaEdgeDescription", SimStruct), offset=0)), ("inputCount", SimTypeInt(signed=False, label="UInt32")), ("outputs", SimTypePointer(SimTypeRef("MLOperatorSchemaEdgeDescription", SimStruct), offset=0)), ("outputCount", SimTypeInt(signed=False, label="UInt32")), ("typeConstraints", SimTypePointer(SimTypeRef("MLOperatorEdgeTypeConstraint", SimStruct), offset=0)), ("typeConstraintCount", SimTypeInt(signed=False, label="UInt32")), ("attributes", SimTypePointer(SimTypeRef("MLOperatorAttribute", SimStruct), offset=0)), ("attributeCount", SimTypeInt(signed=False, label="UInt32")), ("defaultAttributes", SimTypePointer(SimTypeRef("MLOperatorAttributeNameValue", SimStruct), offset=0)), ("defaultAttributeCount", SimTypeInt(signed=False, label="UInt32")),)), name="MLOperatorSchemaDescription", pack=False, align=None),
    "MLOperatorSchemaEdgeDescription": SimStruct(OrderedDict((("options", SimTypeInt(signed=False, label="MLOperatorParameterOptions")), ("typeFormat", SimTypeInt(signed=False, label="MLOperatorSchemaEdgeTypeFormat")), ("Anonymous", SimUnion({"reserved": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "typeLabel": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "edgeDescription": SimTypeRef("MLOperatorEdgeDescription", SimStruct)}, name="<anon>", label="None")),)), name="MLOperatorSchemaEdgeDescription", pack=False, align=None),
    "MLOperatorSchemaEdgeTypeFormat": SimTypeInt(signed=False, label="MLOperatorSchemaEdgeTypeFormat"),
    "MLOperatorSetId": SimStruct(OrderedDict((("domain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("version", SimTypeInt(signed=True, label="Int32")),)), name="MLOperatorSetId", pack=False, align=None),
    "MLOperatorTensorDataType": SimTypeInt(signed=False, label="MLOperatorTensorDataType"),
    "MLSTR_FLAGS": SimTypeInt(signed=False, label="MLSTR_FLAGS"),
    "MMCBUTTON": SimStruct(OrderedDict((("nBitmap", SimTypeInt(signed=True, label="Int32")), ("idCommand", SimTypeInt(signed=True, label="Int32")), ("fsState", SimTypeChar(label="Byte")), ("fsType", SimTypeChar(label="Byte")), ("lpButtonText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpTooltipText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MMCBUTTON", pack=False, align=None),
    "MMCKINFO": SimStruct(OrderedDict((("ckid", SimTypeInt(signed=False, label="UInt32")), ("cksize", SimTypeInt(signed=False, label="UInt32")), ("fccType", SimTypeInt(signed=False, label="UInt32")), ("dwDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="MMCKINFO", pack=False, align=None),
    "MMC_ACTION_TYPE": SimTypeInt(signed=False, label="MMC_ACTION_TYPE"),
    "MMC_BUTTON_STATE": SimTypeInt(signed=False, label="MMC_BUTTON_STATE"),
    "MMC_COLUMN_DATA": SimStruct(OrderedDict((("nColIndex", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("nWidth", SimTypeInt(signed=True, label="Int32")), ("ulReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MMC_COLUMN_DATA", pack=False, align=None),
    "MMC_COLUMN_SET_DATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("nNumCols", SimTypeInt(signed=True, label="Int32")), ("pColData", SimTypePointer(SimTypeRef("MMC_COLUMN_DATA", SimStruct), offset=0)),)), name="MMC_COLUMN_SET_DATA", pack=False, align=None),
    "MMC_CONSOLE_VERB": SimTypeInt(signed=False, label="MMC_CONSOLE_VERB"),
    "MMC_CONTROL_TYPE": SimTypeInt(signed=False, label="MMC_CONTROL_TYPE"),
    "MMC_EXPANDSYNC_STRUCT": SimStruct(OrderedDict((("bHandled", SimTypeInt(signed=True, label="Int32")), ("bExpanding", SimTypeInt(signed=True, label="Int32")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MMC_EXPANDSYNC_STRUCT", pack=False, align=None),
    "MMC_EXT_VIEW_DATA": SimStruct(OrderedDict((("viewID", SimTypeBottom(label="Guid")), ("pszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszViewTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszTooltipText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bReplacesDefaultView", SimTypeInt(signed=True, label="Int32")),)), name="MMC_EXT_VIEW_DATA", pack=False, align=None),
    "MMC_FILTERDATA": SimStruct(OrderedDict((("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("lValue", SimTypeInt(signed=True, label="Int32")),)), name="MMC_FILTERDATA", pack=False, align=None),
    "MMC_FILTER_CHANGE_CODE": SimTypeInt(signed=False, label="MMC_FILTER_CHANGE_CODE"),
    "MMC_FILTER_TYPE": SimTypeInt(signed=False, label="MMC_FILTER_TYPE"),
    "MMC_LISTPAD_INFO": SimStruct(OrderedDict((("szTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szButtonText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nCommandID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MMC_LISTPAD_INFO", pack=False, align=None),
    "MMC_MENU_COMMAND_IDS": SimTypeInt(signed=False, label="MMC_MENU_COMMAND_IDS"),
    "MMC_NOTIFY_TYPE": SimTypeInt(signed=False, label="MMC_NOTIFY_TYPE"),
    "MMC_PROPERTY_ACTION": SimTypeInt(signed=False, label="MMC_PROPERTY_ACTION"),
    "MMC_RESTORE_VIEW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cookie", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pViewType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lViewOptions", SimTypeInt(signed=True, label="Int32")),)), name="MMC_RESTORE_VIEW", pack=False, align=None),
    "MMC_RESULT_VIEW_STYLE": SimTypeInt(signed=False, label="MMC_RESULT_VIEW_STYLE"),
    "MMC_SCOPE_ITEM_STATE": SimTypeInt(signed=False, label="MMC_SCOPE_ITEM_STATE"),
    "MMC_SNAPIN_PROPERTY": SimStruct(OrderedDict((("pszPropName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("varValue", SimTypeRef("VARIANT", SimStruct)), ("eAction", SimTypeInt(signed=False, label="MMC_PROPERTY_ACTION")),)), name="MMC_SNAPIN_PROPERTY", pack=False, align=None),
    "MMC_SORT_DATA": SimStruct(OrderedDict((("nColIndex", SimTypeInt(signed=True, label="Int32")), ("dwSortOptions", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MMC_SORT_DATA", pack=False, align=None),
    "MMC_SORT_SET_DATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=True, label="Int32")), ("nNumItems", SimTypeInt(signed=True, label="Int32")), ("pSortData", SimTypePointer(SimTypeRef("MMC_SORT_DATA", SimStruct), offset=0)),)), name="MMC_SORT_SET_DATA", pack=False, align=None),
    "MMC_TASK": SimStruct(OrderedDict((("sDisplayObject", SimTypeRef("MMC_TASK_DISPLAY_OBJECT", SimStruct)), ("szText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szHelpString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("eActionType", SimTypeInt(signed=False, label="MMC_ACTION_TYPE")), ("Anonymous", SimUnion({"nCommandID": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "szActionURL": SimTypePointer(SimTypeChar(label="Char"), offset=0), "szScript": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="MMC_TASK", pack=False, align=None),
    "MMC_TASK_DISPLAY_BITMAP": SimStruct(OrderedDict((("szMouseOverBitmap", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szMouseOffBitmap", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MMC_TASK_DISPLAY_BITMAP", pack=False, align=None),
    "MMC_TASK_DISPLAY_OBJECT": SimStruct(OrderedDict((("eDisplayType", SimTypeInt(signed=False, label="MMC_TASK_DISPLAY_TYPE")), ("Anonymous", SimUnion({"uBitmap": SimTypeRef("MMC_TASK_DISPLAY_BITMAP", SimStruct), "uSymbol": SimTypeRef("MMC_TASK_DISPLAY_SYMBOL", SimStruct)}, name="<anon>", label="None")),)), name="MMC_TASK_DISPLAY_OBJECT", pack=False, align=None),
    "MMC_TASK_DISPLAY_SYMBOL": SimStruct(OrderedDict((("szFontFamilyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szURLtoEOT", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szSymbolString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MMC_TASK_DISPLAY_SYMBOL", pack=False, align=None),
    "MMC_TASK_DISPLAY_TYPE": SimTypeInt(signed=False, label="MMC_TASK_DISPLAY_TYPE"),
    "MMC_VIEW_TYPE": SimTypeInt(signed=False, label="MMC_VIEW_TYPE"),
    "MMC_VISIBLE_COLUMNS": SimStruct(OrderedDict((("nVisibleColumns", SimTypeInt(signed=True, label="Int32")), ("rgVisibleCols", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="MMC_VISIBLE_COLUMNS", pack=False, align=None),
    "MMFLUSH_TYPE": SimTypeInt(signed=False, label="MMFLUSH_TYPE"),
    "MMIOINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fccIOProc", SimTypeInt(signed=False, label="UInt32")), ("pIOProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpmmioinfo", "uMsg", "lParam1", "lParam2"]), offset=0)), ("wErrorRet", SimTypeInt(signed=False, label="UInt32")), ("htask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cchBuffer", SimTypeInt(signed=True, label="Int32")), ("pchBuffer", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pchNext", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pchEndRead", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pchEndWrite", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("lBufOffset", SimTypeInt(signed=True, label="Int32")), ("lDiskOffset", SimTypeInt(signed=True, label="Int32")), ("adwInfo", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("hmmio", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MMIOINFO", pack=False, align=None),
    "MMSSF_GET_INFORMATION_FLAGS": SimTypeInt(signed=False, label="MMSSF_GET_INFORMATION_FLAGS"),
    "MMTIME": SimStruct(OrderedDict((("wType", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"ms": SimTypeInt(signed=False, label="UInt32"), "sample": SimTypeInt(signed=False, label="UInt32"), "cb": SimTypeInt(signed=False, label="UInt32"), "ticks": SimTypeInt(signed=False, label="UInt32"), "smpte": SimTypeRef("_smpte_e__Struct", SimStruct), "midi": SimTypeRef("_midi_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="MMTIME", pack=False, align=None),
    "MM_COPY_ADDRESS": SimStruct(OrderedDict((("Anonymous", SimUnion({"VirtualAddress": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "PhysicalAddress": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="MM_COPY_ADDRESS", pack=False, align=None),
    "MM_FLAGS": SimTypeInt(signed=False, label="MM_FLAGS"),
    "MM_MDL_PAGE_CONTENTS_STATE": SimTypeInt(signed=False, label="MM_MDL_PAGE_CONTENTS_STATE"),
    "MM_MDL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DriverContext", "MappedVa"]), offset=0),
    "MM_PAGE_PRIORITY": SimTypeInt(signed=False, label="MM_PAGE_PRIORITY"),
    "MM_PHYSICAL_ADDRESS_LIST": SimStruct(OrderedDict((("PhysicalAddress", SimTypeLongLong(signed=True, label="Int64")), ("NumberOfBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MM_PHYSICAL_ADDRESS_LIST", pack=False, align=None),
    "MM_PREFETCH_FLAGS": SimUnion({"Flags": SimTypeRef("_Flags_e__Struct", SimStruct), "AllFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "MM_ROTATE_DIRECTION": SimTypeInt(signed=False, label="MM_ROTATE_DIRECTION"),
    "MM_SYSTEMSIZE": SimTypeInt(signed=False, label="MM_SYSTEMSIZE"),
    "MODE": SimTypeInt(signed=False, label="MODE"),
    "MODEMDEVCAPS": SimStruct(OrderedDict((("dwActualSize", SimTypeInt(signed=False, label="UInt32")), ("dwRequiredSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwModemProviderVersion", SimTypeInt(signed=False, label="UInt32")), ("dwModemManufacturerOffset", SimTypeInt(signed=False, label="UInt32")), ("dwModemManufacturerSize", SimTypeInt(signed=False, label="UInt32")), ("dwModemModelOffset", SimTypeInt(signed=False, label="UInt32")), ("dwModemModelSize", SimTypeInt(signed=False, label="UInt32")), ("dwModemVersionOffset", SimTypeInt(signed=False, label="UInt32")), ("dwModemVersionSize", SimTypeInt(signed=False, label="UInt32")), ("dwDialOptions", SimTypeInt(signed=False, label="MODEMDEVCAPS_DIAL_OPTIONS")), ("dwCallSetupFailTimer", SimTypeInt(signed=False, label="UInt32")), ("dwInactivityTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerVolume", SimTypeInt(signed=False, label="MODEMDEVCAPS_SPEAKER_VOLUME")), ("dwSpeakerMode", SimTypeInt(signed=False, label="MODEMDEVCAPS_SPEAKER_MODE")), ("dwModemOptions", SimTypeInt(signed=False, label="UInt32")), ("dwMaxDTERate", SimTypeInt(signed=False, label="UInt32")), ("dwMaxDCERate", SimTypeInt(signed=False, label="UInt32")), ("abVariablePortion", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MODEMDEVCAPS", pack=False, align=None),
    "MODEMDEVCAPS_DIAL_OPTIONS": SimTypeInt(signed=False, label="MODEMDEVCAPS_DIAL_OPTIONS"),
    "MODEMDEVCAPS_SPEAKER_MODE": SimTypeInt(signed=False, label="MODEMDEVCAPS_SPEAKER_MODE"),
    "MODEMDEVCAPS_SPEAKER_VOLUME": SimTypeInt(signed=False, label="MODEMDEVCAPS_SPEAKER_VOLUME"),
    "MODEMSETTINGS": SimStruct(OrderedDict((("dwActualSize", SimTypeInt(signed=False, label="UInt32")), ("dwRequiredSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwCallSetupFailTimer", SimTypeInt(signed=False, label="UInt32")), ("dwInactivityTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerVolume", SimTypeInt(signed=False, label="MODEM_SPEAKER_VOLUME")), ("dwSpeakerMode", SimTypeInt(signed=False, label="MODEMSETTINGS_SPEAKER_MODE")), ("dwPreferredModemOptions", SimTypeInt(signed=False, label="UInt32")), ("dwNegotiatedModemOptions", SimTypeInt(signed=False, label="UInt32")), ("dwNegotiatedDCERate", SimTypeInt(signed=False, label="UInt32")), ("abVariablePortion", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MODEMSETTINGS", pack=False, align=None),
    "MODEMSETTINGS_SPEAKER_MODE": SimTypeInt(signed=False, label="MODEMSETTINGS_SPEAKER_MODE"),
    "MODEM_SPEAKER_VOLUME": SimTypeInt(signed=False, label="MODEM_SPEAKER_VOLUME"),
    "MODEM_STATUS_FLAGS": SimTypeInt(signed=False, label="MODEM_STATUS_FLAGS"),
    "MODIFIERKEYS_FLAGS": SimTypeInt(signed=False, label="MODIFIERKEYS_FLAGS"),
    "MODIFIERS": SimStruct(OrderedDict((("pVkToBit", SimTypePointer(SimTypeRef("VK_TO_BIT", SimStruct), offset=0)), ("wMaxModBits", SimTypeShort(signed=False, label="UInt16")), ("ModNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MODIFIERS", pack=False, align=None),
    "MODIFY_VHDSET_FLAG": SimTypeInt(signed=False, label="MODIFY_VHDSET_FLAG"),
    "MODIFY_VHDSET_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="MODIFY_VHDSET_VERSION")), ("Anonymous", SimUnion({"SnapshotPath": SimTypeRef("_SnapshotPath_e__Struct", SimStruct), "SnapshotId": SimTypeBottom(label="Guid"), "DefaultFilePath": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="MODIFY_VHDSET_PARAMETERS", pack=False, align=None),
    "MODIFY_VHDSET_VERSION": SimTypeInt(signed=False, label="MODIFY_VHDSET_VERSION"),
    "MODIFY_WORLD_TRANSFORM_MODE": SimTypeInt(signed=False, label="MODIFY_WORLD_TRANSFORM_MODE"),
    "MODLOAD_CVMISC": SimStruct(OrderedDict((("oCV", SimTypeInt(signed=False, label="UInt32")), ("cCV", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("oMisc", SimTypeInt(signed=False, label="UInt32")), ("cMisc", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dtImage", SimTypeInt(signed=False, label="UInt32")), ("cImage", SimTypeInt(signed=False, label="UInt32")),)), name="MODLOAD_CVMISC", pack=False, align=None),
    "MODLOAD_DATA": SimStruct(OrderedDict((("ssize", SimTypeInt(signed=False, label="UInt32")), ("ssig", SimTypeInt(signed=False, label="MODLOAD_DATA_TYPE")), ("data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="MODLOAD_DATA", pack=False, align=None),
    "MODLOAD_DATA_TYPE": SimTypeInt(signed=False, label="MODLOAD_DATA_TYPE"),
    "MODLOAD_PDBGUID_PDBAGE": SimStruct(OrderedDict((("PdbGuid", SimTypeBottom(label="Guid")), ("PdbAge", SimTypeInt(signed=False, label="UInt32")),)), name="MODLOAD_PDBGUID_PDBAGE", pack=False, align=None),
    "MODULEENTRY": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szModule", SimTypeArray(SimTypeChar(label="SByte"), 10)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wcUsage", SimTypeShort(signed=False, label="UInt16")), ("szExePath", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("wNext", SimTypeShort(signed=False, label="UInt16")),)), name="MODULEENTRY", pack=False, align=None),
    "MODULEENTRY32": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("th32ModuleID", SimTypeInt(signed=False, label="UInt32")), ("th32ProcessID", SimTypeInt(signed=False, label="UInt32")), ("GlblcntUsage", SimTypeInt(signed=False, label="UInt32")), ("ProccntUsage", SimTypeInt(signed=False, label="UInt32")), ("modBaseAddr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("modBaseSize", SimTypeInt(signed=False, label="UInt32")), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szModule", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("szExePath", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="MODULEENTRY32", pack=False, align=None),
    "MODULEENTRY32W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("th32ModuleID", SimTypeInt(signed=False, label="UInt32")), ("th32ProcessID", SimTypeInt(signed=False, label="UInt32")), ("GlblcntUsage", SimTypeInt(signed=False, label="UInt32")), ("ProccntUsage", SimTypeInt(signed=False, label="UInt32")), ("modBaseAddr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("modBaseSize", SimTypeInt(signed=False, label="UInt32")), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szModule", SimTypeArray(SimTypeChar(label="Char"), 256)), ("szExePath", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="MODULEENTRY32W", pack=False, align=None),
    "MODULEINFO": SimStruct(OrderedDict((("lpBaseOfDll", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("EntryPoint", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MODULEINFO", pack=False, align=None),
    "MODULE_INFO": SimStruct(OrderedDict((("ProcessImageName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Image", SimTypeRef("DOS_IMAGE_INFO", SimStruct)),)), name="MODULE_INFO", pack=False, align=None),
    "MODULE_TYPE_INFO": SimStruct(OrderedDict((("dataLength", SimTypeShort(signed=False, label="UInt16")), ("leaf", SimTypeShort(signed=False, label="UInt16")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MODULE_TYPE_INFO", pack=False, align=None),
    "MODULE_WRITE_FLAGS": SimTypeInt(signed=False, label="MODULE_WRITE_FLAGS"),
    "MOF_FIELD": SimStruct(OrderedDict((("DataPtr", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("DataType", SimTypeInt(signed=False, label="UInt32")),)), name="MOF_FIELD", pack=False, align=None),
    "MONCBSTRUCT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwRet", SimTypeInt(signed=False, label="UInt32")), ("wType", SimTypeInt(signed=False, label="UInt32")), ("wFmt", SimTypeInt(signed=False, label="UInt32")), ("hConv", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hsz1", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hsz2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwData1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwData2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cc", SimTypeRef("CONVCONTEXT", SimStruct)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="MONCBSTRUCT", pack=False, align=None),
    "MONCONVSTRUCT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("fConnect", SimTypeInt(signed=True, label="Int32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszSvc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszTopic", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hConvClient", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hConvServer", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MONCONVSTRUCT", pack=False, align=None),
    "MONERRSTRUCT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("wLastError", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MONERRSTRUCT", pack=False, align=None),
    "MONHSZSTRUCTA": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("fsAction", SimTypeInt(signed=True, label="Int32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hsz", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("str", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="MONHSZSTRUCTA", pack=False, align=None),
    "MONHSZSTRUCTW": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("fsAction", SimTypeInt(signed=True, label="Int32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hsz", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("str", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MONHSZSTRUCTW", pack=False, align=None),
    "MONIKERPROPERTY": SimTypeInt(signed=False, label="MONIKERPROPERTY"),
    "MONITOR": SimStruct(OrderedDict((("pfnEnumPorts", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0)), ("pfnOpenPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("pfnOpenPortEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MONITOR2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0)), ("pfnStartDocPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0)), ("pfnWritePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnReadPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnEndDocPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("pfnClosePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("pfnAddPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnAddPortEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnConfigurePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnDeletePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnGetPrinterDataFromPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0)), ("pfnSetPortTimeOuts", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("COMMTIMEOUTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnXcvOpenPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnXcvDataPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0)), ("pfnXcvClosePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)),)), name="MONITOR", pack=False, align=None),
    "MONITOR2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnEnumPorts", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0)), ("pfnOpenPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnOpenPortEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MONITOR2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0)), ("pfnStartDocPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0)), ("pfnWritePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnReadPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnEndDocPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("pfnClosePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("pfnAddPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnAddPortEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0)), ("pfnConfigurePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnDeletePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnGetPrinterDataFromPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0)), ("pfnSetPortTimeOuts", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("COMMTIMEOUTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnXcvOpenPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pfnXcvDataPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0)), ("pfnXcvClosePort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("pfnShutdown", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("pfnSendRecvBidiDataFromPort", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("BIDI_REQUEST_CONTAINER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("BIDI_RESPONSE_CONTAINER", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0)), ("pfnNotifyUsedPorts", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnNotifyUnusedPorts", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnPowerEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("POWERBROADCAST_SETTING", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)),)), name="MONITOR2", pack=False, align=None),
    "MONITORENUMPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "MONITOREX": SimStruct(OrderedDict((("dwMonitorSize", SimTypeInt(signed=False, label="UInt32")), ("Monitor", SimTypeRef("MONITOR", SimStruct)),)), name="MONITOREX", pack=False, align=None),
    "MONITORINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcMonitor", SimTypeRef("RECT", SimStruct)), ("rcWork", SimTypeRef("RECT", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="MONITORINFO", pack=False, align=None),
    "MONITORINFOEXA": SimStruct(OrderedDict((("monitorInfo", SimTypeRef("MONITORINFO", SimStruct)), ("szDevice", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="MONITORINFOEXA", pack=False, align=None),
    "MONITORINFOEXW": SimStruct(OrderedDict((("monitorInfo", SimTypeRef("MONITORINFO", SimStruct)), ("szDevice", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="MONITORINFOEXW", pack=False, align=None),
    "MONITORINIT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hSpooler", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hckRegistryRoot", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pMonitorReg", SimTypePointer(SimTypeRef("MONITORREG", SimStruct), offset=0)), ("bLocal", SimTypeInt(signed=True, label="Int32")), ("pszServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MONITORINIT", pack=False, align=None),
    "MONITORREG": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fpCreateKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpOpenKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpCloseKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeleteKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpQueryInfoKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetValue", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeleteValue", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumValue", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpQueryValue", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MONITORREG", pack=False, align=None),
    "MONITORUI": SimStruct(OrderedDict((("dwMonitorUISize", SimTypeInt(signed=False, label="UInt32")), ("pfnAddPortUI", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnConfigurePortUI", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnDeletePortUI", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MONITORUI", pack=False, align=None),
    "MONITOR_APP_VISIBILITY": SimTypeInt(signed=False, label="MONITOR_APP_VISIBILITY"),
    "MONITOR_DISPLAY_STATE": SimTypeInt(signed=False, label="MONITOR_DISPLAY_STATE"),
    "MONITOR_DPI_TYPE": SimTypeInt(signed=False, label="MONITOR_DPI_TYPE"),
    "MONITOR_FROM_FLAGS": SimTypeInt(signed=False, label="MONITOR_FROM_FLAGS"),
    "MONITOR_INFO_1A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MONITOR_INFO_1A", pack=False, align=None),
    "MONITOR_INFO_1W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MONITOR_INFO_1W", pack=False, align=None),
    "MONITOR_INFO_2A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDLLName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MONITOR_INFO_2A", pack=False, align=None),
    "MONITOR_INFO_2W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDLLName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MONITOR_INFO_2W", pack=False, align=None),
    "MONITOR_STATE": SimStruct(OrderedDict((("LastUpdate", SimTypeLongLong(signed=True, label="Int64")), ("State", SimTypeInt(signed=False, label="RESOURCE_MONITOR_STATE")), ("ActiveResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ResmonStop", SimTypeInt(signed=True, label="Int32")),)), name="MONITOR_STATE", pack=False, align=None),
    "MONLINKSTRUCT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fEstablished", SimTypeInt(signed=True, label="Int32")), ("fNoData", SimTypeInt(signed=True, label="Int32")), ("hszSvc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszTopic", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hszItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wFmt", SimTypeInt(signed=False, label="UInt32")), ("fServer", SimTypeInt(signed=True, label="Int32")), ("hConvServer", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hConvClient", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MONLINKSTRUCT", pack=False, align=None),
    "MONMSGSTRUCT": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("hwndTo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wMsg", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dmhd", SimTypeRef("DDEML_MSG_HOOK_DATA", SimStruct)),)), name="MONMSGSTRUCT", pack=False, align=None),
    "MONTHCALPARTS": SimTypeInt(signed=False, label="MONTHCALPARTS"),
    "MONTHLYDATE": SimStruct(OrderedDict((("rgfDays", SimTypeInt(signed=False, label="UInt32")), ("rgfMonths", SimTypeShort(signed=False, label="UInt16")),)), name="MONTHLYDATE", pack=False, align=None),
    "MONTHLYDOW": SimStruct(OrderedDict((("wWhichWeek", SimTypeShort(signed=False, label="UInt16")), ("rgfDaysOfTheWeek", SimTypeShort(signed=False, label="UInt16")), ("rgfMonths", SimTypeShort(signed=False, label="UInt16")),)), name="MONTHLYDOW", pack=False, align=None),
    "MONTH_CALDENDAR_MESSAGES_VIEW": SimTypeInt(signed=False, label="MONTH_CALDENDAR_MESSAGES_VIEW"),
    "MOREPROGRAMSARROWBACKSTATES": SimTypeInt(signed=False, label="MOREPROGRAMSARROWBACKSTATES"),
    "MOREPROGRAMSARROWSTATES": SimTypeInt(signed=False, label="MOREPROGRAMSARROWSTATES"),
    "MOREPROGRAMSTABSTATES": SimTypeInt(signed=False, label="MOREPROGRAMSTABSTATES"),
    "MORRSLT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwchOutput", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchOutput", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"pwchRead": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwchComp": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"cchRead": SimTypeShort(signed=False, label="UInt16"), "cchComp": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("pchInputPos", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("pchOutputIdxWDD", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("Anonymous3", SimUnion({"pchReadIdxWDD": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pchCompIdxWDD": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="<anon>", label="None")), ("paMonoRubyPos", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("pWDD", SimTypePointer(SimTypeRef("WDD", SimStruct), offset=0)), ("cWDD", SimTypeInt(signed=True, label="Int32")), ("pPrivate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BLKBuff", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MORRSLT", pack=False, align=None),
    "MOUSEHOOKSTRUCT": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wHitTestCode", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MOUSEHOOKSTRUCT", pack=False, align=None),
    "MOUSEHOOKSTRUCTEX": SimStruct(OrderedDict((("Base", SimTypeRef("MOUSEHOOKSTRUCT", SimStruct)), ("mouseData", SimTypeInt(signed=False, label="UInt32")),)), name="MOUSEHOOKSTRUCTEX", pack=False, align=None),
    "MOUSEINPUT": SimStruct(OrderedDict((("dx", SimTypeInt(signed=True, label="Int32")), ("dy", SimTypeInt(signed=True, label="Int32")), ("mouseData", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="MOUSE_EVENT_FLAGS")), ("time", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MOUSEINPUT", pack=False, align=None),
    "MOUSEKEYS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iMaxSpeed", SimTypeInt(signed=False, label="UInt32")), ("iTimeToMaxSpeed", SimTypeInt(signed=False, label="UInt32")), ("iCtrlSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="MOUSEKEYS", pack=False, align=None),
    "MOUSEMOVEPOINT": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("time", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MOUSEMOVEPOINT", pack=False, align=None),
    "MOUSE_ATTRIBUTES": SimStruct(OrderedDict((("MouseIdentifier", SimTypeShort(signed=False, label="UInt16")), ("NumberOfButtons", SimTypeShort(signed=False, label="UInt16")), ("SampleRate", SimTypeShort(signed=False, label="UInt16")), ("InputDataQueueLength", SimTypeInt(signed=False, label="UInt32")),)), name="MOUSE_ATTRIBUTES", pack=False, align=None),
    "MOUSE_EVENT_FLAGS": SimTypeInt(signed=False, label="MOUSE_EVENT_FLAGS"),
    "MOUSE_EVENT_RECORD": SimStruct(OrderedDict((("dwMousePosition", SimTypeRef("COORD", SimStruct)), ("dwButtonState", SimTypeInt(signed=False, label="UInt32")), ("dwControlKeyState", SimTypeInt(signed=False, label="UInt32")), ("dwEventFlags", SimTypeInt(signed=False, label="UInt32")),)), name="MOUSE_EVENT_RECORD", pack=False, align=None),
    "MOUSE_INPUT_DATA": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"Buttons": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("ButtonFlags", SimTypeShort(signed=False, label="UInt16")), ("ButtonData", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("RawButtons", SimTypeInt(signed=False, label="UInt32")), ("LastX", SimTypeInt(signed=True, label="Int32")), ("LastY", SimTypeInt(signed=True, label="Int32")), ("ExtraInformation", SimTypeInt(signed=False, label="UInt32")),)), name="MOUSE_INPUT_DATA", pack=False, align=None),
    "MOUSE_STATE": SimTypeInt(signed=False, label="MOUSE_STATE"),
    "MOUSE_UNIT_ID_PARAMETER": SimStruct(OrderedDict((("UnitId", SimTypeShort(signed=False, label="UInt16")),)), name="MOUSE_UNIT_ID_PARAMETER", pack=False, align=None),
    "MOUSE_WHEEL_PARAMETER": SimTypeInt(signed=False, label="MOUSE_WHEEL_PARAMETER"),
    "MOVEREGION_INFO": SimStruct(OrderedDict((("FrameNumber", SimTypeInt(signed=False, label="UInt32")), ("NumMoveRegions", SimTypeInt(signed=False, label="UInt32")), ("MoveRegions", SimTypePointer(SimTypeRef("MOVE_RECT", SimStruct), offset=0)),)), name="MOVEREGION_INFO", pack=False, align=None),
    "MOVESTATES": SimTypeInt(signed=False, label="MOVESTATES"),
    "MOVEUNIT_ACTION": SimTypeInt(signed=False, label="MOVEUNIT_ACTION"),
    "MOVE_FILE_DATA": SimStruct(OrderedDict((("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("StartingVcn", SimTypeLongLong(signed=True, label="Int64")), ("StartingLcn", SimTypeLongLong(signed=True, label="Int64")), ("ClusterCount", SimTypeInt(signed=False, label="UInt32")),)), name="MOVE_FILE_DATA", pack=False, align=None),
    "MOVE_FILE_DATA32": SimStruct(OrderedDict((("FileHandle", SimTypeInt(signed=False, label="UInt32")), ("StartingVcn", SimTypeLongLong(signed=True, label="Int64")), ("StartingLcn", SimTypeLongLong(signed=True, label="Int64")), ("ClusterCount", SimTypeInt(signed=False, label="UInt32")),)), name="MOVE_FILE_DATA32", pack=False, align=None),
    "MOVE_FILE_FLAGS": SimTypeInt(signed=False, label="MOVE_FILE_FLAGS"),
    "MOVE_FILE_RECORD_DATA": SimStruct(OrderedDict((("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SourceFileRecord", SimTypeLongLong(signed=True, label="Int64")), ("TargetFileRecord", SimTypeLongLong(signed=True, label="Int64")),)), name="MOVE_FILE_RECORD_DATA", pack=False, align=None),
    "MOVE_RECT": SimStruct(OrderedDict((("SourcePoint", SimTypeRef("POINT", SimStruct)), ("DestRect", SimTypeRef("RECT", SimStruct)),)), name="MOVE_RECT", pack=False, align=None),
    "MPEG1VIDEOINFO": SimStruct(OrderedDict((("hdr", SimTypeRef("VIDEOINFOHEADER", SimStruct)), ("dwStartTimeCode", SimTypeInt(signed=False, label="UInt32")), ("cbSequenceHeader", SimTypeInt(signed=False, label="UInt32")), ("bSequenceHeader", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MPEG1VIDEOINFO", pack=False, align=None),
    "MPEG1WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("fwHeadLayer", SimTypeShort(signed=False, label="UInt16")), ("dwHeadBitrate", SimTypeInt(signed=False, label="UInt32")), ("fwHeadMode", SimTypeShort(signed=False, label="UInt16")), ("fwHeadModeExt", SimTypeShort(signed=False, label="UInt16")), ("wHeadEmphasis", SimTypeShort(signed=False, label="UInt16")), ("fwHeadFlags", SimTypeShort(signed=False, label="UInt16")), ("dwPTSLow", SimTypeInt(signed=False, label="UInt32")), ("dwPTSHigh", SimTypeInt(signed=False, label="UInt32")),)), name="MPEG1WAVEFORMAT", pack=False, align=None),
    "MPEG2StreamType": SimTypeInt(signed=False, label="MPEG2StreamType"),
    "MPEG2VIDEOINFO": SimStruct(OrderedDict((("hdr", SimTypeRef("VIDEOINFOHEADER2", SimStruct)), ("dwStartTimeCode", SimTypeInt(signed=False, label="UInt32")), ("cbSequenceHeader", SimTypeInt(signed=False, label="UInt32")), ("dwProfile", SimTypeInt(signed=False, label="UInt32")), ("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="MPEG2VIDEOINFO_FLAGS")), ("dwSequenceHeader", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="MPEG2VIDEOINFO", pack=False, align=None),
    "MPEG2VIDEOINFO_FLAGS": SimTypeInt(signed=False, label="MPEG2VIDEOINFO_FLAGS"),
    "MPEG2_FILTER": SimStruct(OrderedDict((("bVersionNumber", SimTypeChar(label="Byte")), ("wFilterSize", SimTypeShort(signed=False, label="UInt16")), ("fUseRawFilteringBits", SimTypeInt(signed=True, label="Int32")), ("Filter", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Mask", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("fSpecifyTableIdExtension", SimTypeInt(signed=True, label="Int32")), ("TableIdExtension", SimTypeShort(signed=False, label="UInt16")), ("fSpecifyVersion", SimTypeInt(signed=True, label="Int32")), ("Version", SimTypeChar(label="Byte")), ("fSpecifySectionNumber", SimTypeInt(signed=True, label="Int32")), ("SectionNumber", SimTypeChar(label="Byte")), ("fSpecifyCurrentNext", SimTypeInt(signed=True, label="Int32")), ("fNext", SimTypeInt(signed=True, label="Int32")), ("fSpecifyDsmccOptions", SimTypeInt(signed=True, label="Int32")), ("Dsmcc", SimTypeRef("DSMCC_FILTER_OPTIONS", SimStruct)), ("fSpecifyAtscOptions", SimTypeInt(signed=True, label="Int32")), ("Atsc", SimTypeRef("ATSC_FILTER_OPTIONS", SimStruct)),)), name="MPEG2_FILTER", pack=False, align=None),
    "MPEG2_FILTER2": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("bVersionNumber", SimTypeChar(label="Byte")), ("wFilterSize", SimTypeShort(signed=False, label="UInt16")), ("fUseRawFilteringBits", SimTypeInt(signed=True, label="Int32")), ("Filter", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Mask", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("fSpecifyTableIdExtension", SimTypeInt(signed=True, label="Int32")), ("TableIdExtension", SimTypeShort(signed=False, label="UInt16")), ("fSpecifyVersion", SimTypeInt(signed=True, label="Int32")), ("Version", SimTypeChar(label="Byte")), ("fSpecifySectionNumber", SimTypeInt(signed=True, label="Int32")), ("SectionNumber", SimTypeChar(label="Byte")), ("fSpecifyCurrentNext", SimTypeInt(signed=True, label="Int32")), ("fNext", SimTypeInt(signed=True, label="Int32")), ("fSpecifyDsmccOptions", SimTypeInt(signed=True, label="Int32")), ("Dsmcc", SimTypeRef("DSMCC_FILTER_OPTIONS", SimStruct)), ("fSpecifyAtscOptions", SimTypeInt(signed=True, label="Int32")), ("Atsc", SimTypeRef("ATSC_FILTER_OPTIONS", SimStruct)),)), name="_Anonymous_e__Struct", pack=False, align=None), "bVersion1Bytes": SimTypeArray(SimTypeChar(label="Byte"), 124)}, name="<anon>", label="None")), ("fSpecifyDvbEitOptions", SimTypeInt(signed=True, label="Int32")), ("DvbEit", SimTypeRef("DVB_EIT_FILTER_OPTIONS", SimStruct)),)), name="MPEG2_FILTER2", pack=False, align=None),
    "MPEG2_TRANSPORT_STRIDE": SimStruct(OrderedDict((("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPacketLength", SimTypeInt(signed=False, label="UInt32")), ("dwStride", SimTypeInt(signed=False, label="UInt32")),)), name="MPEG2_TRANSPORT_STRIDE", pack=False, align=None),
    "MPEGLAYER3WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wID", SimTypeShort(signed=False, label="UInt16")), ("fdwFlags", SimTypeInt(signed=False, label="MPEGLAYER3WAVEFORMAT_FLAGS")), ("nBlockSize", SimTypeShort(signed=False, label="UInt16")), ("nFramesPerBlock", SimTypeShort(signed=False, label="UInt16")), ("nCodecDelay", SimTypeShort(signed=False, label="UInt16")),)), name="MPEGLAYER3WAVEFORMAT", pack=False, align=None),
    "MPEGLAYER3WAVEFORMAT_FLAGS": SimTypeInt(signed=False, label="MPEGLAYER3WAVEFORMAT_FLAGS"),
    "MPEG_BCS_DEMUX": SimStruct(OrderedDict((("AVMGraphId", SimTypeInt(signed=False, label="UInt32")),)), name="MPEG_BCS_DEMUX", pack=False, align=None),
    "MPEG_CONTEXT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="MPEG_CONTEXT_TYPE")), ("U", SimUnion({"Demux": SimTypeRef("MPEG_BCS_DEMUX", SimStruct), "Winsock": SimTypeRef("MPEG_WINSOCK", SimStruct)}, name="<anon>", label="None")),)), name="MPEG_CONTEXT", pack=False, align=None),
    "MPEG_CONTEXT_TYPE": SimTypeInt(signed=False, label="MPEG_CONTEXT_TYPE"),
    "MPEG_CURRENT_NEXT_BIT": SimTypeInt(signed=False, label="MPEG_CURRENT_NEXT_BIT"),
    "MPEG_DATE": SimStruct(OrderedDict((("Date", SimTypeChar(label="Byte")), ("Month", SimTypeChar(label="Byte")), ("Year", SimTypeShort(signed=False, label="UInt16")),)), name="MPEG_DATE", pack=False, align=None),
    "MPEG_DATE_AND_TIME": SimStruct(OrderedDict((("D", SimTypeRef("MPEG_DATE", SimStruct)), ("T", SimTypeRef("MPEG_TIME", SimStruct)),)), name="MPEG_DATE_AND_TIME", pack=False, align=None),
    "MPEG_HEADER_BITS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="MPEG_HEADER_BITS", pack=False, align=None),
    "MPEG_HEADER_BITS_MIDL": SimStruct(OrderedDict((("Bits", SimTypeShort(signed=False, label="UInt16")),)), name="MPEG_HEADER_BITS_MIDL", pack=False, align=None),
    "MPEG_HEADER_VERSION_BITS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="MPEG_HEADER_VERSION_BITS", pack=False, align=None),
    "MPEG_HEADER_VERSION_BITS_MIDL": SimStruct(OrderedDict((("Bits", SimTypeChar(label="Byte")),)), name="MPEG_HEADER_VERSION_BITS_MIDL", pack=False, align=None),
    "MPEG_PACKET_LIST": SimStruct(OrderedDict((("wPacketCount", SimTypeShort(signed=False, label="UInt16")), ("PacketList", SimTypePointer(SimTypePointer(SimTypeRef("MPEG_RQST_PACKET", SimStruct), offset=0), offset=0)),)), name="MPEG_PACKET_LIST", pack=False, align=None),
    "MPEG_REQUEST_TYPE": SimTypeInt(signed=False, label="MPEG_REQUEST_TYPE"),
    "MPEG_RQST_PACKET": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("pSection", SimTypePointer(SimTypeRef("SECTION", SimStruct), offset=0)),)), name="MPEG_RQST_PACKET", pack=False, align=None),
    "MPEG_SERVICE_REQUEST": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="MPEG_REQUEST_TYPE")), ("Context", SimTypeRef("MPEG_CONTEXT", SimStruct)), ("Pid", SimTypeShort(signed=False, label="UInt16")), ("TableId", SimTypeChar(label="Byte")), ("Filter", SimTypeRef("MPEG2_FILTER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="MPEG_SERVICE_REQUEST", pack=False, align=None),
    "MPEG_SERVICE_RESPONSE": SimStruct(OrderedDict((("IPAddress", SimTypeInt(signed=False, label="UInt32")), ("Port", SimTypeShort(signed=False, label="UInt16")),)), name="MPEG_SERVICE_RESPONSE", pack=False, align=None),
    "MPEG_STREAM_BUFFER": SimStruct(OrderedDict((("hr", SimTypeInt(signed=True, label="Int32")), ("dwDataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwSizeOfDataRead", SimTypeInt(signed=False, label="UInt32")), ("pDataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MPEG_STREAM_BUFFER", pack=False, align=None),
    "MPEG_STREAM_FILTER": SimStruct(OrderedDict((("wPidValue", SimTypeShort(signed=False, label="UInt16")), ("dwFilterSize", SimTypeInt(signed=False, label="UInt32")), ("fCrcEnabled", SimTypeInt(signed=True, label="Int32")), ("rgchFilter", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("rgchMask", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="MPEG_STREAM_FILTER", pack=False, align=None),
    "MPEG_TIME": SimStruct(OrderedDict((("Hours", SimTypeChar(label="Byte")), ("Minutes", SimTypeChar(label="Byte")), ("Seconds", SimTypeChar(label="Byte")),)), name="MPEG_TIME", pack=False, align=None),
    "MPEG_WINSOCK": SimStruct(OrderedDict((("AVMGraphId", SimTypeInt(signed=False, label="UInt32")),)), name="MPEG_WINSOCK", pack=False, align=None),
    "MPE_ELEMENT": SimStruct(OrderedDict((("pid", SimTypeShort(signed=False, label="UInt16")), ("bComponentTag", SimTypeChar(label="Byte")), ("pNext", SimTypePointer(SimTypeRef("MPE_ELEMENT", SimStruct), offset=0)),)), name="MPE_ELEMENT", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH": SimStruct(OrderedDict((("PassThrough", SimTypeRef("SCSI_PASS_THROUGH", SimStruct)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH32": SimStruct(OrderedDict((("PassThrough", SimTypeRef("SCSI_PASS_THROUGH32", SimStruct)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH32", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH32_EX": SimStruct(OrderedDict((("PassThroughOffset", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH32_EX", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH_DIRECT": SimStruct(OrderedDict((("PassThrough", SimTypeRef("SCSI_PASS_THROUGH_DIRECT", SimStruct)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH_DIRECT", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH_DIRECT32": SimStruct(OrderedDict((("PassThrough", SimTypeRef("SCSI_PASS_THROUGH_DIRECT32", SimStruct)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH_DIRECT32", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH_DIRECT32_EX": SimStruct(OrderedDict((("PassThroughOffset", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH_DIRECT32_EX", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH_DIRECT_EX": SimStruct(OrderedDict((("PassThroughOffset", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH_DIRECT_EX", pack=False, align=None),
    "MPIO_PASS_THROUGH_PATH_EX": SimStruct(OrderedDict((("PassThroughOffset", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("PortNumber", SimTypeChar(label="Byte")), ("MpioPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="MPIO_PASS_THROUGH_PATH_EX", pack=False, align=None),
    "MPRAPI_ADMIN_DLL_CALLBACKS": SimStruct(OrderedDict((("revision", SimTypeChar(label="Byte")), ("lpfnMprAdminGetIpAddressForUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnMprAdminReleaseIpAddress", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0)), ("lpfnMprAdminGetIpv6AddressForUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("IN6_ADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnMprAdminReleaseIpV6AddressForUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("IN6_ADDR", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0)), ("lpfnRasAdminAcceptNewLink", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_PORT_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_PORT_1", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("lpfnRasAdminLinkHangupNotification", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_PORT_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_PORT_1", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)), ("lpfnRasAdminTerminateDll", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("lpfnRasAdminAcceptNewConnectionEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("lpfnRasAdminAcceptEndpointChangeEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("lpfnRasAdminAcceptReauthenticationEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("lpfnRasAdminConnectionHangupNotificationEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("lpfnRASValidatePreAuthenticatedConnectionEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("AUTH_VALIDATION_EX", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0)),)), name="MPRAPI_ADMIN_DLL_CALLBACKS", pack=False, align=None),
    "MPRAPI_OBJECT_HEADER": SimStruct(OrderedDict((("revision", SimTypeChar(label="Byte")), ("type", SimTypeChar(label="Byte")), ("size", SimTypeShort(signed=False, label="UInt16")),)), name="MPRAPI_OBJECT_HEADER", pack=False, align=None),
    "MPRAPI_OBJECT_TYPE": SimTypeInt(signed=False, label="MPRAPI_OBJECT_TYPE"),
    "MPRAPI_TUNNEL_CONFIG_PARAMS0": SimStruct(OrderedDict((("IkeConfigParams", SimTypeRef("IKEV2_CONFIG_PARAMS", SimStruct)), ("PptpConfigParams", SimTypeRef("PPTP_CONFIG_PARAMS", SimStruct)), ("L2tpConfigParams", SimTypeRef("L2TP_CONFIG_PARAMS1", SimStruct)), ("SstpConfigParams", SimTypeRef("SSTP_CONFIG_PARAMS", SimStruct)),)), name="MPRAPI_TUNNEL_CONFIG_PARAMS0", pack=False, align=None),
    "MPRAPI_TUNNEL_CONFIG_PARAMS1": SimStruct(OrderedDict((("IkeConfigParams", SimTypeRef("IKEV2_CONFIG_PARAMS", SimStruct)), ("PptpConfigParams", SimTypeRef("PPTP_CONFIG_PARAMS", SimStruct)), ("L2tpConfigParams", SimTypeRef("L2TP_CONFIG_PARAMS1", SimStruct)), ("SstpConfigParams", SimTypeRef("SSTP_CONFIG_PARAMS", SimStruct)), ("GREConfigParams", SimTypeRef("GRE_CONFIG_PARAMS0", SimStruct)),)), name="MPRAPI_TUNNEL_CONFIG_PARAMS1", pack=False, align=None),
    "MPR_CERT_EKU": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("IsEKUOID", SimTypeInt(signed=True, label="Int32")), ("pwszEKU", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MPR_CERT_EKU", pack=False, align=None),
    "MPR_CREDENTIALSEX_0": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpbCredentialsInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MPR_CREDENTIALSEX_0", pack=False, align=None),
    "MPR_CREDENTIALSEX_1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpbCredentialsInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MPR_CREDENTIALSEX_1", pack=False, align=None),
    "MPR_DEVICE_0": SimStruct(OrderedDict((("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)),)), name="MPR_DEVICE_0", pack=False, align=None),
    "MPR_DEVICE_1": SimStruct(OrderedDict((("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szAlternates", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MPR_DEVICE_1", pack=False, align=None),
    "MPR_ET": SimTypeInt(signed=False, label="MPR_ET"),
    "MPR_FILTER_0": SimStruct(OrderedDict((("fEnable", SimTypeInt(signed=True, label="Int32")),)), name="MPR_FILTER_0", pack=False, align=None),
    "MPR_IFTRANSPORT_0": SimStruct(OrderedDict((("dwTransportId", SimTypeInt(signed=False, label="UInt32")), ("hIfTransport", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wszIfTransportName", SimTypeArray(SimTypeChar(label="Char"), 41)),)), name="MPR_IFTRANSPORT_0", pack=False, align=None),
    "MPR_IF_CUSTOMINFOEX0": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("customIkev2Config", SimTypeRef("ROUTER_IKEv2_IF_CUSTOM_CONFIG0", SimStruct)),)), name="MPR_IF_CUSTOMINFOEX0", pack=False, align=None),
    "MPR_IF_CUSTOMINFOEX1": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("customIkev2Config", SimTypeRef("ROUTER_IKEv2_IF_CUSTOM_CONFIG1", SimStruct)),)), name="MPR_IF_CUSTOMINFOEX1", pack=False, align=None),
    "MPR_IF_CUSTOMINFOEX2": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("customIkev2Config", SimTypeRef("ROUTER_IKEv2_IF_CUSTOM_CONFIG2", SimStruct)),)), name="MPR_IF_CUSTOMINFOEX2", pack=False, align=None),
    "MPR_INTERFACE_0": SimStruct(OrderedDict((("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fEnabled", SimTypeInt(signed=True, label="Int32")), ("dwIfType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionState", SimTypeInt(signed=False, label="ROUTER_CONNECTION_STATE")), ("fUnReachabilityReasons", SimTypeInt(signed=False, label="UInt32")), ("dwLastError", SimTypeInt(signed=False, label="UInt32")),)), name="MPR_INTERFACE_0", pack=False, align=None),
    "MPR_INTERFACE_1": SimStruct(OrderedDict((("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fEnabled", SimTypeInt(signed=True, label="Int32")), ("dwIfType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionState", SimTypeInt(signed=False, label="ROUTER_CONNECTION_STATE")), ("fUnReachabilityReasons", SimTypeInt(signed=False, label="UInt32")), ("dwLastError", SimTypeInt(signed=False, label="UInt32")), ("lpwsDialoutHoursRestriction", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MPR_INTERFACE_1", pack=False, align=None),
    "MPR_INTERFACE_2": SimStruct(OrderedDict((("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fEnabled", SimTypeInt(signed=True, label="Int32")), ("dwIfType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionState", SimTypeInt(signed=False, label="ROUTER_CONNECTION_STATE")), ("fUnReachabilityReasons", SimTypeInt(signed=False, label="UInt32")), ("dwLastError", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions", SimTypeInt(signed=False, label="UInt32")), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szAlternates", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ipaddr", SimTypeInt(signed=False, label="UInt32")), ("ipaddrDns", SimTypeInt(signed=False, label="UInt32")), ("ipaddrDnsAlt", SimTypeInt(signed=False, label="UInt32")), ("ipaddrWins", SimTypeInt(signed=False, label="UInt32")), ("ipaddrWinsAlt", SimTypeInt(signed=False, label="UInt32")), ("dwfNetProtocols", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szX25PadType", SimTypeArray(SimTypeChar(label="Char"), 33)), ("szX25Address", SimTypeArray(SimTypeChar(label="Char"), 201)), ("szX25Facilities", SimTypeArray(SimTypeChar(label="Char"), 201)), ("szX25UserData", SimTypeArray(SimTypeChar(label="Char"), 201)), ("dwChannels", SimTypeInt(signed=False, label="UInt32")), ("dwSubEntries", SimTypeInt(signed=False, label="UInt32")), ("dwDialMode", SimTypeInt(signed=False, label="MPR_INTERFACE_DIAL_MODE")), ("dwDialExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwDialExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwIdleDisconnectSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionType", SimTypeInt(signed=False, label="MPR_ET")), ("dwCustomAuthKey", SimTypeInt(signed=False, label="UInt32")), ("dwCustomAuthDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpbCustomAuthData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("guidId", SimTypeBottom(label="Guid")), ("dwVpnStrategy", SimTypeInt(signed=False, label="MPR_VS")),)), name="MPR_INTERFACE_2", pack=False, align=None),
    "MPR_INTERFACE_3": SimStruct(OrderedDict((("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fEnabled", SimTypeInt(signed=True, label="Int32")), ("dwIfType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionState", SimTypeInt(signed=False, label="ROUTER_CONNECTION_STATE")), ("fUnReachabilityReasons", SimTypeInt(signed=False, label="UInt32")), ("dwLastError", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions", SimTypeInt(signed=False, label="UInt32")), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szAlternates", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ipaddr", SimTypeInt(signed=False, label="UInt32")), ("ipaddrDns", SimTypeInt(signed=False, label="UInt32")), ("ipaddrDnsAlt", SimTypeInt(signed=False, label="UInt32")), ("ipaddrWins", SimTypeInt(signed=False, label="UInt32")), ("ipaddrWinsAlt", SimTypeInt(signed=False, label="UInt32")), ("dwfNetProtocols", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szX25PadType", SimTypeArray(SimTypeChar(label="Char"), 33)), ("szX25Address", SimTypeArray(SimTypeChar(label="Char"), 201)), ("szX25Facilities", SimTypeArray(SimTypeChar(label="Char"), 201)), ("szX25UserData", SimTypeArray(SimTypeChar(label="Char"), 201)), ("dwChannels", SimTypeInt(signed=False, label="UInt32")), ("dwSubEntries", SimTypeInt(signed=False, label="UInt32")), ("dwDialMode", SimTypeInt(signed=False, label="MPR_INTERFACE_DIAL_MODE")), ("dwDialExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwDialExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwIdleDisconnectSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionType", SimTypeInt(signed=False, label="MPR_ET")), ("dwCustomAuthKey", SimTypeInt(signed=False, label="UInt32")), ("dwCustomAuthDataSize", SimTypeInt(signed=False, label="UInt32")), ("lpbCustomAuthData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("guidId", SimTypeBottom(label="Guid")), ("dwVpnStrategy", SimTypeInt(signed=False, label="MPR_VS")), ("AddressCount", SimTypeInt(signed=False, label="UInt32")), ("ipv6addrDns", SimTypeRef("IN6_ADDR", SimStruct)), ("ipv6addrDnsAlt", SimTypeRef("IN6_ADDR", SimStruct)), ("ipv6addr", SimTypePointer(SimTypeRef("IN6_ADDR", SimStruct), offset=0)),)), name="MPR_INTERFACE_3", pack=False, align=None),
    "MPR_INTERFACE_DIAL_MODE": SimTypeInt(signed=False, label="MPR_INTERFACE_DIAL_MODE"),
    "MPR_IPINIP_INTERFACE_0": SimStruct(OrderedDict((("wszFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("Guid", SimTypeBottom(label="Guid")),)), name="MPR_IPINIP_INTERFACE_0", pack=False, align=None),
    "MPR_PROTOCOL_0": SimStruct(OrderedDict((("dwProtocolId", SimTypeInt(signed=False, label="UInt32")), ("wszProtocol", SimTypeArray(SimTypeChar(label="Char"), 41)), ("wszDLLName", SimTypeArray(SimTypeChar(label="Char"), 49)),)), name="MPR_PROTOCOL_0", pack=False, align=None),
    "MPR_SERVER_0": SimStruct(OrderedDict((("fLanOnlyMode", SimTypeInt(signed=True, label="Int32")), ("dwUpTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortsInUse", SimTypeInt(signed=False, label="UInt32")),)), name="MPR_SERVER_0", pack=False, align=None),
    "MPR_SERVER_1": SimStruct(OrderedDict((("dwNumPptpPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPptpPortFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumL2tpPorts", SimTypeInt(signed=False, label="UInt32")), ("dwL2tpPortFlags", SimTypeInt(signed=False, label="UInt32")),)), name="MPR_SERVER_1", pack=False, align=None),
    "MPR_SERVER_2": SimStruct(OrderedDict((("dwNumPptpPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPptpPortFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumL2tpPorts", SimTypeInt(signed=False, label="UInt32")), ("dwL2tpPortFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumSstpPorts", SimTypeInt(signed=False, label="UInt32")), ("dwSstpPortFlags", SimTypeInt(signed=False, label="UInt32")),)), name="MPR_SERVER_2", pack=False, align=None),
    "MPR_SERVER_EX0": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("fLanOnlyMode", SimTypeInt(signed=False, label="UInt32")), ("dwUpTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortsInUse", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ConfigParams", SimTypeRef("MPRAPI_TUNNEL_CONFIG_PARAMS0", SimStruct)),)), name="MPR_SERVER_EX0", pack=False, align=None),
    "MPR_SERVER_EX1": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("fLanOnlyMode", SimTypeInt(signed=False, label="UInt32")), ("dwUpTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortsInUse", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ConfigParams", SimTypeRef("MPRAPI_TUNNEL_CONFIG_PARAMS1", SimStruct)),)), name="MPR_SERVER_EX1", pack=False, align=None),
    "MPR_SERVER_SET_CONFIG_EX0": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("setConfigForProtocols", SimTypeInt(signed=False, label="UInt32")), ("ConfigParams", SimTypeRef("MPRAPI_TUNNEL_CONFIG_PARAMS0", SimStruct)),)), name="MPR_SERVER_SET_CONFIG_EX0", pack=False, align=None),
    "MPR_SERVER_SET_CONFIG_EX1": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("setConfigForProtocols", SimTypeInt(signed=False, label="UInt32")), ("ConfigParams", SimTypeRef("MPRAPI_TUNNEL_CONFIG_PARAMS1", SimStruct)),)), name="MPR_SERVER_SET_CONFIG_EX1", pack=False, align=None),
    "MPR_TRANSPORT_0": SimStruct(OrderedDict((("dwTransportId", SimTypeInt(signed=False, label="UInt32")), ("hTransport", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wszTransportName", SimTypeArray(SimTypeChar(label="Char"), 41)),)), name="MPR_TRANSPORT_0", pack=False, align=None),
    "MPR_VPN_TRAFFIC_SELECTOR": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="MPR_VPN_TS_TYPE")), ("protocolId", SimTypeChar(label="Byte")), ("portStart", SimTypeShort(signed=False, label="UInt16")), ("portEnd", SimTypeShort(signed=False, label="UInt16")), ("tsPayloadId", SimTypeShort(signed=False, label="UInt16")), ("addrStart", SimTypeRef("VPN_TS_IP_ADDRESS", SimStruct)), ("addrEnd", SimTypeRef("VPN_TS_IP_ADDRESS", SimStruct)),)), name="MPR_VPN_TRAFFIC_SELECTOR", pack=False, align=None),
    "MPR_VPN_TRAFFIC_SELECTORS": SimStruct(OrderedDict((("numTsi", SimTypeInt(signed=False, label="UInt32")), ("numTsr", SimTypeInt(signed=False, label="UInt32")), ("tsI", SimTypePointer(SimTypeRef("MPR_VPN_TRAFFIC_SELECTOR", SimStruct), offset=0)), ("tsR", SimTypePointer(SimTypeRef("MPR_VPN_TRAFFIC_SELECTOR", SimStruct), offset=0)),)), name="MPR_VPN_TRAFFIC_SELECTORS", pack=False, align=None),
    "MPR_VPN_TS_TYPE": SimTypeInt(signed=False, label="MPR_VPN_TS_TYPE"),
    "MPR_VS": SimTypeInt(signed=False, label="MPR_VS"),
    "MP_CURVE_TYPE": SimTypeInt(signed=False, label="MP_CURVE_TYPE"),
    "MP_DEVICE_DATA_SET_RANGE": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("LengthInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="MP_DEVICE_DATA_SET_RANGE", pack=False, align=None),
    "MP_ENVELOPE_SEGMENT": SimStruct(OrderedDict((("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("valStart", SimTypeFloat(size=32)), ("valEnd", SimTypeFloat(size=32)), ("iCurve", SimTypeInt(signed=False, label="MP_CURVE_TYPE")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="MP_ENVELOPE_SEGMENT", pack=False, align=None),
    "MP_PARAMINFO": SimStruct(OrderedDict((("mpType", SimTypeInt(signed=False, label="MP_TYPE")), ("mopCaps", SimTypeInt(signed=False, label="UInt32")), ("mpdMinValue", SimTypeFloat(size=32)), ("mpdMaxValue", SimTypeFloat(size=32)), ("mpdNeutralValue", SimTypeFloat(size=32)), ("szUnitText", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szLabel", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="MP_PARAMINFO", pack=False, align=None),
    "MP_STORAGE_DIAGNOSTIC_LEVEL": SimTypeInt(signed=False, label="MP_STORAGE_DIAGNOSTIC_LEVEL"),
    "MP_STORAGE_DIAGNOSTIC_TARGET_TYPE": SimTypeInt(signed=False, label="MP_STORAGE_DIAGNOSTIC_TARGET_TYPE"),
    "MP_TYPE": SimTypeInt(signed=False, label="MP_TYPE"),
    "MQACCESS": SimTypeInt(signed=False, label="MQACCESS"),
    "MQAUTHENTICATE": SimTypeInt(signed=False, label="MQAUTHENTICATE"),
    "MQCALG": SimTypeInt(signed=False, label="MQCALG"),
    "MQCERT_REGISTER": SimTypeInt(signed=False, label="MQCERT_REGISTER"),
    "MQCOLUMNSET": SimStruct(OrderedDict((("cCol", SimTypeInt(signed=False, label="UInt32")), ("aCol", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="MQCOLUMNSET", pack=False, align=None),
    "MQConnectionState": SimTypeInt(signed=False, label="MQConnectionState"),
    "MQDEFAULT": SimTypeInt(signed=False, label="MQDEFAULT"),
    "MQERROR": SimTypeInt(signed=False, label="MQERROR"),
    "MQJOURNAL": SimTypeInt(signed=False, label="MQJOURNAL"),
    "MQMAX": SimTypeInt(signed=False, label="MQMAX"),
    "MQMGMTPROPS": SimStruct(OrderedDict((("cProp", SimTypeInt(signed=False, label="UInt32")), ("aPropID", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("aPropVar", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("aStatus", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="MQMGMTPROPS", pack=False, align=None),
    "MQMSGACKNOWLEDGEMENT": SimTypeInt(signed=False, label="MQMSGACKNOWLEDGEMENT"),
    "MQMSGAUTHENTICATION": SimTypeInt(signed=False, label="MQMSGAUTHENTICATION"),
    "MQMSGAUTHLEVEL": SimTypeInt(signed=False, label="MQMSGAUTHLEVEL"),
    "MQMSGCLASS": SimTypeInt(signed=False, label="MQMSGCLASS"),
    "MQMSGCURSOR": SimTypeInt(signed=False, label="MQMSGCURSOR"),
    "MQMSGDELIVERY": SimTypeInt(signed=False, label="MQMSGDELIVERY"),
    "MQMSGIDSIZE": SimTypeInt(signed=False, label="MQMSGIDSIZE"),
    "MQMSGJOURNAL": SimTypeInt(signed=False, label="MQMSGJOURNAL"),
    "MQMSGMAX": SimTypeInt(signed=False, label="MQMSGMAX"),
    "MQMSGPRIVLEVEL": SimTypeInt(signed=False, label="MQMSGPRIVLEVEL"),
    "MQMSGPROPS": SimStruct(OrderedDict((("cProp", SimTypeInt(signed=False, label="UInt32")), ("aPropID", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("aPropVar", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("aStatus", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="MQMSGPROPS", pack=False, align=None),
    "MQMSGSENDERIDTYPE": SimTypeInt(signed=False, label="MQMSGSENDERIDTYPE"),
    "MQMSGTRACE": SimTypeInt(signed=False, label="MQMSGTRACE"),
    "MQPRIORITY": SimTypeInt(signed=False, label="MQPRIORITY"),
    "MQPRIVATEPROPS": SimStruct(OrderedDict((("cProp", SimTypeInt(signed=False, label="UInt32")), ("aPropID", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("aPropVar", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("aStatus", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="MQPRIVATEPROPS", pack=False, align=None),
    "MQPRIVLEVEL": SimTypeInt(signed=False, label="MQPRIVLEVEL"),
    "MQPROPERTYRESTRICTION": SimStruct(OrderedDict((("rel", SimTypeInt(signed=False, label="UInt32")), ("prop", SimTypeInt(signed=False, label="UInt32")), ("prval", SimTypeRef("PROPVARIANT", SimStruct)),)), name="MQPROPERTYRESTRICTION", pack=False, align=None),
    "MQQMPROPS": SimStruct(OrderedDict((("cProp", SimTypeInt(signed=False, label="UInt32")), ("aPropID", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("aPropVar", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("aStatus", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="MQQMPROPS", pack=False, align=None),
    "MQQUEUEACCESSMASK": SimTypeInt(signed=False, label="MQQUEUEACCESSMASK"),
    "MQQUEUEPROPS": SimStruct(OrderedDict((("cProp", SimTypeInt(signed=False, label="UInt32")), ("aPropID", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("aPropVar", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("aStatus", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="MQQUEUEPROPS", pack=False, align=None),
    "MQRESTRICTION": SimStruct(OrderedDict((("cRes", SimTypeInt(signed=False, label="UInt32")), ("paPropRes", SimTypePointer(SimTypeRef("MQPROPERTYRESTRICTION", SimStruct), offset=0)),)), name="MQRESTRICTION", pack=False, align=None),
    "MQSHARE": SimTypeInt(signed=False, label="MQSHARE"),
    "MQSORTKEY": SimStruct(OrderedDict((("propColumn", SimTypeInt(signed=False, label="UInt32")), ("dwOrder", SimTypeInt(signed=False, label="UInt32")),)), name="MQSORTKEY", pack=False, align=None),
    "MQSORTSET": SimStruct(OrderedDict((("cCol", SimTypeInt(signed=False, label="UInt32")), ("aCol", SimTypePointer(SimTypeRef("MQSORTKEY", SimStruct), offset=0)),)), name="MQSORTSET", pack=False, align=None),
    "MQTRANSACTION": SimTypeInt(signed=False, label="MQTRANSACTION"),
    "MQTRANSACTIONAL": SimTypeInt(signed=False, label="MQTRANSACTIONAL"),
    "MQWARNING": SimTypeInt(signed=False, label="MQWARNING"),
    "MSAAMENUINFO": SimStruct(OrderedDict((("dwMSAASignature", SimTypeInt(signed=False, label="UInt32")), ("cchWText", SimTypeInt(signed=False, label="UInt32")), ("pszWText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MSAAMENUINFO", pack=False, align=None),
    "MSAUDIO1WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")), ("wEncodeOptions", SimTypeShort(signed=False, label="UInt16")),)), name="MSAUDIO1WAVEFORMAT", pack=False, align=None),
    "MSA_INFO_0": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="MSA_INFO_STATE")),)), name="MSA_INFO_0", pack=False, align=None),
    "MSA_INFO_LEVEL": SimTypeInt(signed=False, label="MSA_INFO_LEVEL"),
    "MSA_INFO_STATE": SimTypeInt(signed=False, label="MSA_INFO_STATE"),
    "MSCEPSetupProperty": SimTypeInt(signed=False, label="MSCEPSetupProperty"),
    "MSDSDBINITPROPENUM": SimTypeInt(signed=False, label="MSDSDBINITPROPENUM"),
    "MSDSSESSIONPROPENUM": SimTypeInt(signed=False, label="MSDSSESSIONPROPENUM"),
    "MSG": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("message", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("time", SimTypeInt(signed=False, label="UInt32")), ("pt", SimTypeRef("POINT", SimStruct)),)), name="MSG", pack=False, align=None),
    "MSGBOXCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("HELPINFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpHelpInfo"]), offset=0),
    "MSGBOXPARAMSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="MESSAGEBOX_STYLE")), ("lpszIcon", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwContextHelpId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfnMsgBoxCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("HELPINFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpHelpInfo"]), offset=0)), ("dwLanguageId", SimTypeInt(signed=False, label="UInt32")),)), name="MSGBOXPARAMSA", pack=False, align=None),
    "MSGBOXPARAMSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwStyle", SimTypeInt(signed=False, label="MESSAGEBOX_STYLE")), ("lpszIcon", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwContextHelpId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpfnMsgBoxCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("HELPINFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpHelpInfo"]), offset=0)), ("dwLanguageId", SimTypeInt(signed=False, label="UInt32")),)), name="MSGBOXPARAMSW", pack=False, align=None),
    "MSGCALLRELEASE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMessage")], SimTypeBottom(label="Void"), arg_names=["ulCallerData", "lpMessage"]), offset=0),
    "MSGFILTER": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("msg", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="MSGFILTER", pack=False, align=None),
    "MSGFLTINFO_STATUS": SimTypeInt(signed=False, label="MSGFLTINFO_STATUS"),
    "MSG_INFO_0": SimStruct(OrderedDict((("msgi0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MSG_INFO_0", pack=False, align=None),
    "MSG_INFO_1": SimStruct(OrderedDict((("msgi1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("msgi1_forward_flag", SimTypeInt(signed=False, label="UInt32")), ("msgi1_forward", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MSG_INFO_1", pack=False, align=None),
    "MSG_WAIT_FOR_MULTIPLE_OBJECTS_EX_FLAGS": SimTypeInt(signed=False, label="MSG_WAIT_FOR_MULTIPLE_OBJECTS_EX_FLAGS"),
    "MSHCTX": SimTypeInt(signed=False, label="MSHCTX"),
    "MSHLFLAGS": SimTypeInt(signed=False, label="MSHLFLAGS"),
    "MSIADVERTISEOPTIONFLAGS": SimTypeInt(signed=False, label="MSIADVERTISEOPTIONFLAGS"),
    "MSIARCHITECTUREFLAGS": SimTypeInt(signed=False, label="MSIARCHITECTUREFLAGS"),
    "MSIASSEMBLYINFO": SimTypeInt(signed=False, label="MSIASSEMBLYINFO"),
    "MSICODE": SimTypeInt(signed=False, label="MSICODE"),
    "MSICOLINFO": SimTypeInt(signed=False, label="MSICOLINFO"),
    "MSICONDITION": SimTypeInt(signed=False, label="MSICONDITION"),
    "MSICOSTTREE": SimTypeInt(signed=False, label="MSICOSTTREE"),
    "MSIDBERROR": SimTypeInt(signed=False, label="MSIDBERROR"),
    "MSIDBSTATE": SimTypeInt(signed=False, label="MSIDBSTATE"),
    "MSIFILEHASHINFO": SimStruct(OrderedDict((("dwFileHashInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwData", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="MSIFILEHASHINFO", pack=False, align=None),
    "MSIHANDLE": SimTypeInt(signed=False, label="UInt32"),
    "MSIINSTALLCONTEXT": SimTypeInt(signed=False, label="MSIINSTALLCONTEXT"),
    "MSIMODIFY": SimTypeInt(signed=False, label="MSIMODIFY"),
    "MSIOPENPACKAGEFLAGS": SimTypeInt(signed=False, label="MSIOPENPACKAGEFLAGS"),
    "MSIPATCHDATATYPE": SimTypeInt(signed=False, label="MSIPATCHDATATYPE"),
    "MSIPATCHSEQUENCEINFOA": SimStruct(OrderedDict((("szPatchData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ePatchDataType", SimTypeInt(signed=False, label="MSIPATCHDATATYPE")), ("dwOrder", SimTypeInt(signed=False, label="UInt32")), ("uStatus", SimTypeInt(signed=False, label="UInt32")),)), name="MSIPATCHSEQUENCEINFOA", pack=False, align=None),
    "MSIPATCHSEQUENCEINFOW": SimStruct(OrderedDict((("szPatchData", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ePatchDataType", SimTypeInt(signed=False, label="MSIPATCHDATATYPE")), ("dwOrder", SimTypeInt(signed=False, label="UInt32")), ("uStatus", SimTypeInt(signed=False, label="UInt32")),)), name="MSIPATCHSEQUENCEINFOW", pack=False, align=None),
    "MSIPATCHSTATE": SimTypeInt(signed=False, label="MSIPATCHSTATE"),
    "MSIRUNMODE": SimTypeInt(signed=False, label="MSIRUNMODE"),
    "MSISOURCETYPE": SimTypeInt(signed=False, label="MSISOURCETYPE"),
    "MSITRANSACTION": SimTypeInt(signed=False, label="MSITRANSACTION"),
    "MSITRANSACTIONSTATE": SimTypeInt(signed=False, label="MSITRANSACTIONSTATE"),
    "MSITRANSFORM_ERROR": SimTypeInt(signed=False, label="MSITRANSFORM_ERROR"),
    "MSITRANSFORM_VALIDATE": SimTypeInt(signed=False, label="MSITRANSFORM_VALIDATE"),
    "MSLLHOOKSTRUCT": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("mouseData", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("time", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="MSLLHOOKSTRUCT", pack=False, align=None),
    "MSP_ADDRESS_EVENT": SimTypeInt(signed=False, label="MSP_ADDRESS_EVENT"),
    "MSP_CALL_EVENT": SimTypeInt(signed=False, label="MSP_CALL_EVENT"),
    "MSP_CALL_EVENT_CAUSE": SimTypeInt(signed=False, label="MSP_CALL_EVENT_CAUSE"),
    "MSP_EVENT": SimTypeInt(signed=False, label="MSP_EVENT"),
    "MSP_EVENT_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Event", SimTypeInt(signed=False, label="MSP_EVENT")), ("hCall", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("Anonymous", SimUnion({"MSP_ADDRESS_EVENT_INFO": SimTypeRef("_MSP_ADDRESS_EVENT_INFO_e__Struct", SimStruct), "MSP_CALL_EVENT_INFO": SimTypeRef("_MSP_CALL_EVENT_INFO_e__Struct", SimStruct), "MSP_TSP_DATA": SimTypeRef("_MSP_TSP_DATA_e__Struct", SimStruct), "MSP_PRIVATE_EVENT_INFO": SimTypeRef("_MSP_PRIVATE_EVENT_INFO_e__Struct", SimStruct), "MSP_FILE_TERMINAL_EVENT_INFO": SimTypeRef("_MSP_FILE_TERMINAL_EVENT_INFO_e__Struct", SimStruct), "MSP_ASR_TERMINAL_EVENT_INFO": SimTypeRef("_MSP_ASR_TERMINAL_EVENT_INFO_e__Struct", SimStruct), "MSP_TTS_TERMINAL_EVENT_INFO": SimTypeRef("_MSP_TTS_TERMINAL_EVENT_INFO_e__Struct", SimStruct), "MSP_TONE_TERMINAL_EVENT_INFO": SimTypeRef("_MSP_TONE_TERMINAL_EVENT_INFO_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="MSP_EVENT_INFO", pack=False, align=None),
    "MSV1_0": SimTypeInt(signed=False, label="MSV1_0"),
    "MSV1_0_AVID": SimTypeInt(signed=False, label="MSV1_0_AVID"),
    "MSV1_0_AV_PAIR": SimStruct(OrderedDict((("AvId", SimTypeShort(signed=False, label="UInt16")), ("AvLen", SimTypeShort(signed=False, label="UInt16")),)), name="MSV1_0_AV_PAIR", pack=False, align=None),
    "MSV1_0_CHANGEPASSWORD_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AccountName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("OldPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("NewPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Impersonating", SimTypeChar(label="Byte")),)), name="MSV1_0_CHANGEPASSWORD_REQUEST", pack=False, align=None),
    "MSV1_0_CHANGEPASSWORD_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("PasswordInfoValid", SimTypeChar(label="Byte")), ("DomainPasswordInfo", SimTypeRef("DOMAIN_PASSWORD_INFORMATION", SimStruct)),)), name="MSV1_0_CHANGEPASSWORD_RESPONSE", pack=False, align=None),
    "MSV1_0_CREDENTIAL_KEY": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="MSV1_0_CREDENTIAL_KEY", pack=False, align=None),
    "MSV1_0_CREDENTIAL_KEY_TYPE": SimTypeInt(signed=False, label="MSV1_0_CREDENTIAL_KEY_TYPE"),
    "MSV1_0_ENUMUSERS_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")),)), name="MSV1_0_ENUMUSERS_REQUEST", pack=False, align=None),
    "MSV1_0_ENUMUSERS_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("NumberOfLoggedOnUsers", SimTypeInt(signed=False, label="UInt32")), ("LogonIds", SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)), ("EnumHandles", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="MSV1_0_ENUMUSERS_RESPONSE", pack=False, align=None),
    "MSV1_0_GETCHALLENRESP_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("ParameterControl", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Password", SimTypeRef("UNICODE_STRING", SimStruct)), ("ChallengeToClient", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("UserName", SimTypeRef("UNICODE_STRING", SimStruct)), ("LogonDomainName", SimTypeRef("UNICODE_STRING", SimStruct)), ("ServerName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="MSV1_0_GETCHALLENRESP_REQUEST", pack=False, align=None),
    "MSV1_0_GETCHALLENRESP_REQUEST_V1": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("ParameterControl", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Password", SimTypeRef("UNICODE_STRING", SimStruct)), ("ChallengeToClient", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="MSV1_0_GETCHALLENRESP_REQUEST_V1", pack=False, align=None),
    "MSV1_0_GETCHALLENRESP_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("CaseSensitiveChallengeResponse", SimTypeRef("STRING", SimStruct)), ("CaseInsensitiveChallengeResponse", SimTypeRef("STRING", SimStruct)), ("UserName", SimTypeRef("UNICODE_STRING", SimStruct)), ("LogonDomainName", SimTypeRef("UNICODE_STRING", SimStruct)), ("UserSessionKey", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("LanmanSessionKey", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="MSV1_0_GETCHALLENRESP_RESPONSE", pack=False, align=None),
    "MSV1_0_GETUSERINFO_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="MSV1_0_GETUSERINFO_REQUEST", pack=False, align=None),
    "MSV1_0_GETUSERINFO_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("UserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UserName", SimTypeRef("UNICODE_STRING", SimStruct)), ("LogonDomainName", SimTypeRef("UNICODE_STRING", SimStruct)), ("LogonServer", SimTypeRef("UNICODE_STRING", SimStruct)), ("LogonType", SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE")),)), name="MSV1_0_GETUSERINFO_RESPONSE", pack=False, align=None),
    "MSV1_0_INTERACTIVE_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_LOGON_SUBMIT_TYPE")), ("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="MSV1_0_INTERACTIVE_LOGON", pack=False, align=None),
    "MSV1_0_INTERACTIVE_PROFILE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROFILE_BUFFER_TYPE")), ("LogonCount", SimTypeShort(signed=False, label="UInt16")), ("BadPasswordCount", SimTypeShort(signed=False, label="UInt16")), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("LogoffTime", SimTypeLongLong(signed=True, label="Int64")), ("KickOffTime", SimTypeLongLong(signed=True, label="Int64")), ("PasswordLastSet", SimTypeLongLong(signed=True, label="Int64")), ("PasswordCanChange", SimTypeLongLong(signed=True, label="Int64")), ("PasswordMustChange", SimTypeLongLong(signed=True, label="Int64")), ("LogonScript", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectory", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("FullName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProfilePath", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectoryDrive", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserFlags", SimTypeInt(signed=False, label="UInt32")),)), name="MSV1_0_INTERACTIVE_PROFILE", pack=False, align=None),
    "MSV1_0_IUM_SUPPLEMENTAL_CREDENTIAL": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("EncryptedCredsSize", SimTypeInt(signed=False, label="UInt32")), ("EncryptedCreds", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MSV1_0_IUM_SUPPLEMENTAL_CREDENTIAL", pack=False, align=None),
    "MSV1_0_LM20_CHALLENGE_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")),)), name="MSV1_0_LM20_CHALLENGE_REQUEST", pack=False, align=None),
    "MSV1_0_LM20_CHALLENGE_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("ChallengeToClient", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="MSV1_0_LM20_CHALLENGE_RESPONSE", pack=False, align=None),
    "MSV1_0_LM20_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_LOGON_SUBMIT_TYPE")), ("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Workstation", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ChallengeToClient", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("CaseSensitiveChallengeResponse", SimTypeRef("LSA_STRING", SimStruct)), ("CaseInsensitiveChallengeResponse", SimTypeRef("LSA_STRING", SimStruct)), ("ParameterControl", SimTypeInt(signed=False, label="UInt32")),)), name="MSV1_0_LM20_LOGON", pack=False, align=None),
    "MSV1_0_LM20_LOGON_PROFILE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROFILE_BUFFER_TYPE")), ("KickOffTime", SimTypeLongLong(signed=True, label="Int64")), ("LogoffTime", SimTypeLongLong(signed=True, label="Int64")), ("UserFlags", SimTypeInt(signed=False, label="MSV_SUB_AUTHENTICATION_FILTER")), ("UserSessionKey", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LanmanSessionKey", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserParameters", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="MSV1_0_LM20_LOGON_PROFILE", pack=False, align=None),
    "MSV1_0_LOGON_SUBMIT_TYPE": SimTypeInt(signed=False, label="MSV1_0_LOGON_SUBMIT_TYPE"),
    "MSV1_0_NTLM3_RESPONSE": SimStruct(OrderedDict((("Response", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("RespType", SimTypeChar(label="Byte")), ("HiRespType", SimTypeChar(label="Byte")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("MsgWord", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeLongLong(signed=False, label="UInt64")), ("ChallengeFromClient", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("AvPairsOff", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MSV1_0_NTLM3_RESPONSE", pack=False, align=None),
    "MSV1_0_PASSTHROUGH_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("PackageName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("LogonData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Pad", SimTypeInt(signed=False, label="UInt32")),)), name="MSV1_0_PASSTHROUGH_REQUEST", pack=False, align=None),
    "MSV1_0_PASSTHROUGH_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("Pad", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("ValidationData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MSV1_0_PASSTHROUGH_RESPONSE", pack=False, align=None),
    "MSV1_0_PROFILE_BUFFER_TYPE": SimTypeInt(signed=False, label="MSV1_0_PROFILE_BUFFER_TYPE"),
    "MSV1_0_PROTOCOL_MESSAGE_TYPE": SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE"),
    "MSV1_0_REMOTE_SUPPLEMENTAL_CREDENTIAL": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CredentialKey", SimTypeRef("MSV1_0_CREDENTIAL_KEY", SimStruct)), ("CredentialKeyType", SimTypeInt(signed=False, label="MSV1_0_CREDENTIAL_KEY_TYPE")), ("EncryptedCredsSize", SimTypeInt(signed=False, label="UInt32")), ("EncryptedCreds", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MSV1_0_REMOTE_SUPPLEMENTAL_CREDENTIAL", pack=False, align=None),
    "MSV1_0_S4U_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_LOGON_SUBMIT_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("UserPrincipalName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="MSV1_0_S4U_LOGON", pack=False, align=None),
    "MSV1_0_SUBAUTH_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_LOGON_SUBMIT_TYPE")), ("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Workstation", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ChallengeToClient", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("AuthenticationInfo1", SimTypeRef("LSA_STRING", SimStruct)), ("AuthenticationInfo2", SimTypeRef("LSA_STRING", SimStruct)), ("ParameterControl", SimTypeInt(signed=False, label="MSV_SUBAUTH_LOGON_PARAMETER_CONTROL")), ("SubAuthPackageId", SimTypeInt(signed=False, label="UInt32")),)), name="MSV1_0_SUBAUTH_LOGON", pack=False, align=None),
    "MSV1_0_SUBAUTH_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("SubAuthPackageId", SimTypeInt(signed=False, label="UInt32")), ("SubAuthInfoLength", SimTypeInt(signed=False, label="UInt32")), ("SubAuthSubmitBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MSV1_0_SUBAUTH_REQUEST", pack=False, align=None),
    "MSV1_0_SUBAUTH_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="MSV1_0_PROTOCOL_MESSAGE_TYPE")), ("SubAuthInfoLength", SimTypeInt(signed=False, label="UInt32")), ("SubAuthReturnBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MSV1_0_SUBAUTH_RESPONSE", pack=False, align=None),
    "MSV1_0_SUPPLEMENTAL_CREDENTIAL": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="MSV_SUPPLEMENTAL_CREDENTIAL_FLAGS")), ("LmPassword", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("NtPassword", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="MSV1_0_SUPPLEMENTAL_CREDENTIAL", pack=False, align=None),
    "MSV1_0_SUPPLEMENTAL_CREDENTIAL_V2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NtPassword", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("CredentialKey", SimTypeRef("MSV1_0_CREDENTIAL_KEY", SimStruct)),)), name="MSV1_0_SUPPLEMENTAL_CREDENTIAL_V2", pack=False, align=None),
    "MSV1_0_SUPPLEMENTAL_CREDENTIAL_V3": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CredentialKeyType", SimTypeInt(signed=False, label="MSV1_0_CREDENTIAL_KEY_TYPE")), ("NtPassword", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("CredentialKey", SimTypeRef("MSV1_0_CREDENTIAL_KEY", SimStruct)), ("ShaPassword", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="MSV1_0_SUPPLEMENTAL_CREDENTIAL_V3", pack=False, align=None),
    "MSV1_0_VALIDATION_INFO": SimStruct(OrderedDict((("LogoffTime", SimTypeLongLong(signed=True, label="Int64")), ("KickoffTime", SimTypeLongLong(signed=True, label="Int64")), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("SessionKey", SimTypeRef("USER_SESSION_KEY", SimStruct)), ("Authoritative", SimTypeChar(label="Byte")), ("UserFlags", SimTypeInt(signed=False, label="UInt32")), ("WhichFields", SimTypeInt(signed=False, label="UInt32")), ("UserId", SimTypeInt(signed=False, label="UInt32")),)), name="MSV1_0_VALIDATION_INFO", pack=False, align=None),
    "MSV_SUBAUTH_LOGON_PARAMETER_CONTROL": SimTypeInt(signed=False, label="MSV_SUBAUTH_LOGON_PARAMETER_CONTROL"),
    "MSV_SUB_AUTHENTICATION_FILTER": SimTypeInt(signed=False, label="MSV_SUB_AUTHENTICATION_FILTER"),
    "MSV_SUPPLEMENTAL_CREDENTIAL_FLAGS": SimTypeInt(signed=False, label="MSV_SUPPLEMENTAL_CREDENTIAL_FLAGS"),
    "MSVidCCService": SimTypeInt(signed=False, label="MSVidCCService"),
    "MSVidCtlButtonstate": SimTypeInt(signed=False, label="MSVidCtlButtonstate"),
    "MSVidCtlStateList": SimTypeInt(signed=False, label="MSVidCtlStateList"),
    "MSVidSegmentType": SimTypeInt(signed=False, label="MSVidSegmentType"),
    "MSVidSinkStreams": SimTypeInt(signed=False, label="MSVidSinkStreams"),
    "MSViddispidList": SimTypeInt(signed=False, label="MSViddispidList"),
    "MS_ADDINFO_BLOB": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cbMemObject", SimTypeInt(signed=False, label="UInt32")), ("pbMemObject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbMemSignedMsg", SimTypeInt(signed=False, label="UInt32")), ("pbMemSignedMsg", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MS_ADDINFO_BLOB", pack=False, align=None),
    "MS_ADDINFO_CATALOGMEMBER": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pStore", SimTypePointer(SimTypeRef("CRYPTCATSTORE", SimStruct), offset=0)), ("pMember", SimTypePointer(SimTypeRef("CRYPTCATMEMBER", SimStruct), offset=0)),)), name="MS_ADDINFO_CATALOGMEMBER", pack=False, align=None),
    "MS_ADDINFO_FLAT": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pIndirectData", SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)),)), name="MS_ADDINFO_FLAT", pack=False, align=None),
    "MTP_COMMAND_DATA_IN": SimStruct(OrderedDict((("OpCode", SimTypeShort(signed=False, label="UInt16")), ("NumParams", SimTypeInt(signed=False, label="UInt32")), ("Params", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)), ("NextPhase", SimTypeInt(signed=False, label="UInt32")), ("CommandWriteDataSize", SimTypeInt(signed=False, label="UInt32")), ("CommandWriteData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MTP_COMMAND_DATA_IN", pack=False, align=None),
    "MTP_COMMAND_DATA_OUT": SimStruct(OrderedDict((("ResponseCode", SimTypeShort(signed=False, label="UInt16")), ("NumParams", SimTypeInt(signed=False, label="UInt32")), ("Params", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)), ("CommandReadDataSize", SimTypeInt(signed=False, label="UInt32")), ("CommandReadData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MTP_COMMAND_DATA_OUT", pack=False, align=None),
    "MTSAdminErrorCodes": SimTypeInt(signed=False, label="MTSAdminErrorCodes"),
    "MTSID": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("ab", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MTSID", pack=False, align=None),
    "MTSPackageExportOptions": SimTypeInt(signed=False, label="MTSPackageExportOptions"),
    "MTSPackageInstallOptions": SimTypeInt(signed=False, label="MTSPackageInstallOptions"),
    "MT_ARBITRARY_HEADER": SimStruct(OrderedDict((("majortype", SimTypeBottom(label="Guid")), ("subtype", SimTypeBottom(label="Guid")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("lSampleSize", SimTypeInt(signed=False, label="UInt32")), ("formattype", SimTypeBottom(label="Guid")),)), name="MT_ARBITRARY_HEADER", pack=False, align=None),
    "MT_CUSTOM_VIDEO_PRIMARIES": SimStruct(OrderedDict((("fRx", SimTypeFloat(size=32)), ("fRy", SimTypeFloat(size=32)), ("fGx", SimTypeFloat(size=32)), ("fGy", SimTypeFloat(size=32)), ("fBx", SimTypeFloat(size=32)), ("fBy", SimTypeFloat(size=32)), ("fWx", SimTypeFloat(size=32)), ("fWy", SimTypeFloat(size=32)),)), name="MT_CUSTOM_VIDEO_PRIMARIES", pack=False, align=None),
    "MULTICAST_MODE_TYPE": SimTypeInt(signed=False, label="MULTICAST_MODE_TYPE"),
    "MULTICLASSINFO_FLAGS": SimTypeInt(signed=False, label="MULTICLASSINFO_FLAGS"),
    "MULTIKEYHELPA": SimStruct(OrderedDict((("mkSize", SimTypeInt(signed=False, label="UInt32")), ("mkKeylist", SimTypeChar(label="SByte")), ("szKeyphrase", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="MULTIKEYHELPA", pack=False, align=None),
    "MULTIKEYHELPW": SimStruct(OrderedDict((("mkSize", SimTypeInt(signed=False, label="UInt32")), ("mkKeylist", SimTypeChar(label="Char")), ("szKeyphrase", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MULTIKEYHELPW", pack=False, align=None),
    "MULTIPLE_TRUSTEE_OPERATION": SimTypeInt(signed=False, label="MULTIPLE_TRUSTEE_OPERATION"),
    "MULTI_BYTE_TO_WIDE_CHAR_FLAGS": SimTypeInt(signed=False, label="MULTI_BYTE_TO_WIDE_CHAR_FLAGS"),
    "MULTI_QI": SimStruct(OrderedDict((("pIID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pItf", SimTypeBottom(label="IUnknown")), ("hr", SimTypeInt(signed=True, label="Int32")),)), name="MULTI_QI", pack=False, align=None),
    "MUX_PID_TYPE": SimTypeInt(signed=False, label="MUX_PID_TYPE"),
    "MU_TELEMETRY_SECTION": SimStruct(OrderedDict((("ComponentID", SimTypeBottom(label="Guid")), ("SubComponentID", SimTypeBottom(label="Guid")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ErrorStatusValue", SimTypeInt(signed=False, label="UInt32")), ("AdditionalInfo1", SimTypeLongLong(signed=False, label="UInt64")), ("AdditionalInfo2", SimTypeLongLong(signed=False, label="UInt64")),)), name="MU_TELEMETRY_SECTION", pack=False, align=None),
    "MW_FILTERMODE": SimTypeInt(signed=False, label="MW_FILTERMODE"),
    "MXDC_ESCAPE_HEADER_T": SimStruct(OrderedDict((("cbInput", SimTypeInt(signed=False, label="UInt32")), ("cbOutput", SimTypeInt(signed=False, label="UInt32")), ("opCode", SimTypeInt(signed=False, label="UInt32")),)), name="MXDC_ESCAPE_HEADER_T", pack=False, align=None),
    "MXDC_GET_FILENAME_DATA_T": SimStruct(OrderedDict((("cbOutput", SimTypeInt(signed=False, label="UInt32")), ("wszData", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MXDC_GET_FILENAME_DATA_T", pack=False, align=None),
    "MXDC_IMAGE_TYPE_ENUMS": SimTypeInt(signed=False, label="MXDC_IMAGE_TYPE_ENUMS"),
    "MXDC_LANDSCAPE_ROTATION_ENUMS": SimTypeInt(signed=False, label="MXDC_LANDSCAPE_ROTATION_ENUMS"),
    "MXDC_PRINTTICKET_DATA_T": SimStruct(OrderedDict((("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MXDC_PRINTTICKET_DATA_T", pack=False, align=None),
    "MXDC_PRINTTICKET_ESCAPE_T": SimStruct(OrderedDict((("mxdcEscape", SimTypeRef("MXDC_ESCAPE_HEADER_T", SimStruct)), ("printTicketData", SimTypeRef("MXDC_PRINTTICKET_DATA_T", SimStruct)),)), name="MXDC_PRINTTICKET_ESCAPE_T", pack=False, align=None),
    "MXDC_S0PAGE_DATA_T": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MXDC_S0PAGE_DATA_T", pack=False, align=None),
    "MXDC_S0PAGE_PASSTHROUGH_ESCAPE_T": SimStruct(OrderedDict((("mxdcEscape", SimTypeRef("MXDC_ESCAPE_HEADER_T", SimStruct)), ("xpsS0PageData", SimTypeRef("MXDC_S0PAGE_DATA_T", SimStruct)),)), name="MXDC_S0PAGE_PASSTHROUGH_ESCAPE_T", pack=False, align=None),
    "MXDC_S0PAGE_RESOURCE_ESCAPE_T": SimStruct(OrderedDict((("mxdcEscape", SimTypeRef("MXDC_ESCAPE_HEADER_T", SimStruct)), ("xpsS0PageResourcePassthrough", SimTypeRef("MXDC_XPS_S0PAGE_RESOURCE_T", SimStruct)),)), name="MXDC_S0PAGE_RESOURCE_ESCAPE_T", pack=False, align=None),
    "MXDC_S0_PAGE_ENUMS": SimTypeInt(signed=False, label="MXDC_S0_PAGE_ENUMS"),
    "MXDC_XPS_S0PAGE_RESOURCE_T": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwResourceType", SimTypeInt(signed=False, label="UInt32")), ("szUri", SimTypeArray(SimTypeChar(label="Byte"), 260)), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("bData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MXDC_XPS_S0PAGE_RESOURCE_T", pack=False, align=None),
    "MachineGlobalObjectTableRegistrationToken": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "MagImageScalingCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeRef("MAGIMAGEHEADER", SimStruct), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeRef("MAGIMAGEHEADER", SimStruct), SimTypeRef("RECT", SimStruct), SimTypeRef("RECT", SimStruct), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "srcdata", "srcheader", "destdata", "destheader", "unclipped", "clipped", "dirty"]), offset=0),
    "MagnetometerAccuracy": SimTypeInt(signed=False, label="MagnetometerAccuracy"),
    "MainAVIHeader": SimStruct(OrderedDict((("dwMicroSecPerFrame", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("dwPaddingGranularity", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTotalFrames", SimTypeInt(signed=False, label="UInt32")), ("dwInitialFrames", SimTypeInt(signed=False, label="UInt32")), ("dwStreams", SimTypeInt(signed=False, label="UInt32")), ("dwSuggestedBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="MainAVIHeader", pack=False, align=None),
    "MapiFileDesc": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("flFlags", SimTypeInt(signed=False, label="UInt32")), ("nPosition", SimTypeInt(signed=False, label="UInt32")), ("lpszPathName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpFileType", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MapiFileDesc", pack=False, align=None),
    "MapiFileDescW": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("flFlags", SimTypeInt(signed=False, label="UInt32")), ("nPosition", SimTypeInt(signed=False, label="UInt32")), ("lpszPathName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpFileType", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MapiFileDescW", pack=False, align=None),
    "MapiFileTagExt": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("cbTag", SimTypeInt(signed=False, label="UInt32")), ("lpTag", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbEncoding", SimTypeInt(signed=False, label="UInt32")), ("lpEncoding", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="MapiFileTagExt", pack=False, align=None),
    "MapiMessage": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("lpszSubject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszNoteText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszMessageType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszDateReceived", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszConversationID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("flFlags", SimTypeInt(signed=False, label="UInt32")), ("lpOriginator", SimTypePointer(SimTypeRef("MapiRecipDesc", SimStruct), offset=0)), ("nRecipCount", SimTypeInt(signed=False, label="UInt32")), ("lpRecips", SimTypePointer(SimTypeRef("MapiRecipDesc", SimStruct), offset=0)), ("nFileCount", SimTypeInt(signed=False, label="UInt32")), ("lpFiles", SimTypePointer(SimTypeRef("MapiFileDesc", SimStruct), offset=0)),)), name="MapiMessage", pack=False, align=None),
    "MapiMessageW": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("lpszSubject", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszNoteText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszMessageType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszDateReceived", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszConversationID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("flFlags", SimTypeInt(signed=False, label="UInt32")), ("lpOriginator", SimTypePointer(SimTypeRef("MapiRecipDescW", SimStruct), offset=0)), ("nRecipCount", SimTypeInt(signed=False, label="UInt32")), ("lpRecips", SimTypePointer(SimTypeRef("MapiRecipDescW", SimStruct), offset=0)), ("nFileCount", SimTypeInt(signed=False, label="UInt32")), ("lpFiles", SimTypePointer(SimTypeRef("MapiFileDescW", SimStruct), offset=0)),)), name="MapiMessageW", pack=False, align=None),
    "MapiRecipDesc": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("ulRecipClass", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulEIDSize", SimTypeInt(signed=False, label="UInt32")), ("lpEntryID", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MapiRecipDesc", pack=False, align=None),
    "MapiRecipDescW": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("ulRecipClass", SimTypeInt(signed=False, label="UInt32")), ("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulEIDSize", SimTypeInt(signed=False, label="UInt32")), ("lpEntryID", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MapiRecipDescW", pack=False, align=None),
    "Matrix": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "MatrixOrder": SimTypeInt(signed=False, label="MatrixOrder"),
    "MediaLabelInfo": SimStruct(OrderedDict((("LabelType", SimTypeArray(SimTypeChar(label="Char"), 64)), ("LabelIDSize", SimTypeInt(signed=False, label="UInt32")), ("LabelID", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("LabelAppDescr", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="MediaLabelInfo", pack=False, align=None),
    "MenuItem": SimTypeBottom(label="MenuItem"),
    "MetadataBit": SimTypeInt(signed=False, label="MetadataBit"),
    "MetadataTimeStamps": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypeLongLong(signed=True, label="Int64")), ("Presentation", SimTypeLongLong(signed=True, label="Int64")),)), name="MetadataTimeStamps", pack=False, align=None),
    "Metafile": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "MetafileFrameUnit": SimTypeInt(signed=False, label="MetafileFrameUnit"),
    "MetafileHeader": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="MetafileType")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("EmfPlusFlags", SimTypeInt(signed=False, label="UInt32")), ("DpiX", SimTypeFloat(size=32)), ("DpiY", SimTypeFloat(size=32)), ("X", SimTypeInt(signed=True, label="Int32")), ("Y", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=True, label="Int32")), ("Height", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"WmfHeader": SimTypeRef("METAHEADER", SimStruct), "EmfHeader": SimTypeRef("ENHMETAHEADER3", SimStruct)}, name="<anon>", label="None")), ("EmfPlusHeaderSize", SimTypeInt(signed=True, label="Int32")), ("LogicalDpiX", SimTypeInt(signed=True, label="Int32")), ("LogicalDpiY", SimTypeInt(signed=True, label="Int32")),)), name="MetafileHeader", pack=False, align=None),
    "MetafileType": SimTypeInt(signed=False, label="MetafileType"),
    "MilMatrix3x2D": SimStruct(OrderedDict((("S_11", SimTypeFloat(size=64)), ("S_12", SimTypeFloat(size=64)), ("S_21", SimTypeFloat(size=64)), ("S_22", SimTypeFloat(size=64)), ("DX", SimTypeFloat(size=64)), ("DY", SimTypeFloat(size=64)),)), name="MilMatrix3x2D", pack=False, align=None),
    "MilPoint2D": SimStruct(OrderedDict((("X", SimTypeFloat(size=64)), ("Y", SimTypeFloat(size=64)),)), name="MilPoint2D", pack=False, align=None),
    "MilRectD": SimStruct(OrderedDict((("left", SimTypeFloat(size=64)), ("top", SimTypeFloat(size=64)), ("right", SimTypeFloat(size=64)), ("bottom", SimTypeFloat(size=64)),)), name="MilRectD", pack=False, align=None),
    "ModelObjectKind": SimTypeInt(signed=False, label="ModelObjectKind"),
    "ModulationType": SimTypeInt(signed=False, label="ModulationType"),
    "ModuleBindInfo": SimStruct(OrderedDict((("dwAppDomainId", SimTypeInt(signed=False, label="UInt32")), ("lpAssemblyIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpModuleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ModuleBindInfo", pack=False, align=None),
    "MouseButton": SimTypeInt(signed=False, label="MouseButton"),
    "Mpeg2TableSampleHdr": SimStruct(OrderedDict((("SectionCount", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("SectionOffsets", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="Mpeg2TableSampleHdr", pack=False, align=None),
    "MrmDumpType": SimTypeInt(signed=False, label="MrmDumpType"),
    "MrmIndexerFlags": SimTypeInt(signed=False, label="MrmIndexerFlags"),
    "MrmPackagingMode": SimTypeInt(signed=False, label="MrmPackagingMode"),
    "MrmPackagingOptions": SimTypeInt(signed=False, label="MrmPackagingOptions"),
    "MrmPlatformVersion": SimTypeInt(signed=False, label="MrmPlatformVersion"),
    "MrmResourceIndexerHandle": SimStruct(OrderedDict((("handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="MrmResourceIndexerHandle", pack=False, align=None),
    "MrmResourceIndexerMessage": SimStruct(OrderedDict((("severity", SimTypeInt(signed=False, label="MrmResourceIndexerMessageSeverity")), ("id", SimTypeInt(signed=False, label="UInt32")), ("text", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="MrmResourceIndexerMessage", pack=False, align=None),
    "MrmResourceIndexerMessageSeverity": SimTypeInt(signed=False, label="MrmResourceIndexerMessageSeverity"),
    "NABTSFEC_BUFFER": SimStruct(OrderedDict((("dataSize", SimTypeInt(signed=False, label="UInt32")), ("groupID", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("data", SimTypeArray(SimTypeChar(label="Byte"), 448)),)), name="NABTSFEC_BUFFER", pack=False, align=None),
    "NABTS_BUFFER": SimStruct(OrderedDict((("ScanlinesRequested", SimTypeRef("VBICODECFILTERING_SCANLINES", SimStruct)), ("PictureNumber", SimTypeLongLong(signed=True, label="Int64")), ("NabtsLines", SimTypeArray(SimTypeRef("NABTS_BUFFER_LINE", SimStruct), 11)),)), name="NABTS_BUFFER", pack=False, align=None),
    "NABTS_BUFFER_LINE": SimStruct(OrderedDict((("Confidence", SimTypeChar(label="Byte")), ("Bytes", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="NABTS_BUFFER_LINE", pack=False, align=None),
    "NAMEDCOLOR": SimStruct(OrderedDict((("dwIndex", SimTypeInt(signed=False, label="UInt32")),)), name="NAMEDCOLOR", pack=False, align=None),
    "NAMED_ENTITY_CERTAINTY": SimTypeInt(signed=False, label="NAMED_ENTITY_CERTAINTY"),
    "NAMED_PIPE_CREATE_PARAMETERS": SimStruct(OrderedDict((("NamedPipeType", SimTypeInt(signed=False, label="UInt32")), ("ReadMode", SimTypeInt(signed=False, label="UInt32")), ("CompletionMode", SimTypeInt(signed=False, label="UInt32")), ("MaximumInstances", SimTypeInt(signed=False, label="UInt32")), ("InboundQuota", SimTypeInt(signed=False, label="UInt32")), ("OutboundQuota", SimTypeInt(signed=False, label="UInt32")), ("DefaultTimeout", SimTypeLongLong(signed=True, label="Int64")), ("TimeoutSpecified", SimTypeChar(label="Byte")),)), name="NAMED_PIPE_CREATE_PARAMETERS", pack=False, align=None),
    "NAMED_PIPE_MODE": SimTypeInt(signed=False, label="NAMED_PIPE_MODE"),
    "NAMED_PROFILE_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("dwCountDevCoordinates", SimTypeInt(signed=False, label="UInt32")), ("szPrefix", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szSuffix", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="NAMED_PROFILE_INFO", pack=False, align=None),
    "NAMEENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "NAMEENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "NAMESPACEWALKFLAG": SimTypeInt(signed=False, label="NAMESPACEWALKFLAG"),
    "NAMESPROPERTIES": SimTypeInt(signed=False, label="NAMESPROPERTIES"),
    "NAME_BUFFER": SimStruct(OrderedDict((("name", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("name_num", SimTypeChar(label="Byte")), ("name_flags", SimTypeChar(label="Byte")),)), name="NAME_BUFFER", pack=False, align=None),
    "NAME_CACHE_CONTEXT": SimStruct(OrderedDict((("m_dwSignature", SimTypeInt(signed=False, label="UInt32")),)), name="NAME_CACHE_CONTEXT", pack=False, align=None),
    "NAPI_DOMAIN_DESCRIPTION_BLOB": SimStruct(OrderedDict((("AuthLevel", SimTypeInt(signed=False, label="UInt32")), ("cchDomainName", SimTypeInt(signed=False, label="UInt32")), ("OffsetNextDomainDescription", SimTypeInt(signed=False, label="UInt32")), ("OffsetThisDomainName", SimTypeInt(signed=False, label="UInt32")),)), name="NAPI_DOMAIN_DESCRIPTION_BLOB", pack=False, align=None),
    "NAPI_PROVIDER_INSTALLATION_BLOB": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwProviderType", SimTypeInt(signed=False, label="UInt32")), ("fSupportsWildCard", SimTypeInt(signed=False, label="UInt32")), ("cDomains", SimTypeInt(signed=False, label="UInt32")), ("OffsetFirstDomain", SimTypeInt(signed=False, label="UInt32")),)), name="NAPI_PROVIDER_INSTALLATION_BLOB", pack=False, align=None),
    "NAPI_PROVIDER_LEVEL": SimTypeInt(signed=False, label="NAPI_PROVIDER_LEVEL"),
    "NAPI_PROVIDER_TYPE": SimTypeInt(signed=False, label="NAPI_PROVIDER_TYPE"),
    "NAPPROPERTIES": SimTypeInt(signed=False, label="NAPPROPERTIES"),
    "NATIVE_DISPLAY_ORIENTATION": SimTypeInt(signed=False, label="NATIVE_DISPLAY_ORIENTATION"),
    "NATLANGUAGERESTRICTION": SimStruct(OrderedDict((("prop", SimTypeRef("FULLPROPSPEC", SimStruct)), ("pwcsPhrase", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lcid", SimTypeInt(signed=False, label="UInt32")),)), name="NATLANGUAGERESTRICTION", pack=False, align=None),
    "NAVIGATEDATA": SimStruct(OrderedDict((("ulTarget", SimTypeInt(signed=False, label="UInt32")), ("ulURL", SimTypeInt(signed=False, label="UInt32")), ("ulRefURL", SimTypeInt(signed=False, label="UInt32")), ("ulPostData", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NAVIGATEDATA", pack=False, align=None),
    "NAVIGATEFRAME_FLAGS": SimTypeInt(signed=False, label="NAVIGATEFRAME_FLAGS"),
    "NAVIGATIONPARTS": SimTypeInt(signed=False, label="NAVIGATIONPARTS"),
    "NAVNEXTSTATES": SimTypeInt(signed=False, label="NAVNEXTSTATES"),
    "NAVPREVSTATES": SimTypeInt(signed=False, label="NAVPREVSTATES"),
    "NAV_BACKBUTTONSTATES": SimTypeInt(signed=False, label="NAV_BACKBUTTONSTATES"),
    "NAV_FORWARDBUTTONSTATES": SimTypeInt(signed=False, label="NAV_FORWARDBUTTONSTATES"),
    "NAV_MENUBUTTONSTATES": SimTypeInt(signed=False, label="NAV_MENUBUTTONSTATES"),
    "NCB": SimStruct(OrderedDict((("ncb_command", SimTypeChar(label="Byte")), ("ncb_retcode", SimTypeChar(label="Byte")), ("ncb_lsn", SimTypeChar(label="Byte")), ("ncb_num", SimTypeChar(label="Byte")), ("ncb_buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ncb_length", SimTypeShort(signed=False, label="UInt16")), ("ncb_callname", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ncb_name", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ncb_rto", SimTypeChar(label="Byte")), ("ncb_sto", SimTypeChar(label="Byte")), ("ncb_post", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ncb_lana_num", SimTypeChar(label="Byte")), ("ncb_cmd_cplt", SimTypeChar(label="Byte")), ("ncb_reserve", SimTypeArray(SimTypeChar(label="Byte"), 10)), ("ncb_event", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NCB", pack=False, align=None),
    "NCCALCSIZE_PARAMS": SimStruct(OrderedDict((("rgrc", SimTypeArray(SimTypeRef("RECT", SimStruct), 3)), ("lppos", SimTypePointer(SimTypeRef("WINDOWPOS", SimStruct), offset=0)),)), name="NCCALCSIZE_PARAMS", pack=False, align=None),
    "NCPNP_RECONFIG_LAYER": SimTypeInt(signed=False, label="NCPNP_RECONFIG_LAYER"),
    "NCRP_FLAGS": SimTypeInt(signed=False, label="NCRP_FLAGS"),
    "NCRYPT_ALGORITHM_NAME_CLASS": SimTypeInt(signed=False, label="NCRYPT_ALGORITHM_NAME_CLASS"),
    "NCRYPT_ALLOC_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnAlloc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0)),)), name="NCRYPT_ALLOC_PARA", pack=False, align=None),
    "NCRYPT_CIPHER_PADDING_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pbIV", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbIV", SimTypeInt(signed=False, label="UInt32")), ("pbOtherInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbOtherInfo", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_CIPHER_PADDING_INFO", pack=False, align=None),
    "NCRYPT_DESCRIPTOR_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NCRYPT_EXPORTED_ISOLATED_KEY_ENVELOPE": SimStruct(OrderedDict((("Header", SimTypeRef("NCRYPT_EXPORTED_ISOLATED_KEY_HEADER", SimStruct)),)), name="NCRYPT_EXPORTED_ISOLATED_KEY_ENVELOPE", pack=False, align=None),
    "NCRYPT_EXPORTED_ISOLATED_KEY_HEADER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("KeyUsage", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("cbAlgName", SimTypeInt(signed=False, label="UInt32")), ("cbNonce", SimTypeInt(signed=False, label="UInt32")), ("cbAuthTag", SimTypeInt(signed=False, label="UInt32")), ("cbWrappingKey", SimTypeInt(signed=False, label="UInt32")), ("cbIsolatedKey", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_EXPORTED_ISOLATED_KEY_HEADER", pack=False, align=None),
    "NCRYPT_FLAGS": SimTypeInt(signed=False, label="NCRYPT_FLAGS"),
    "NCRYPT_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "NCRYPT_HASH_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "NCRYPT_ISOLATED_KEY_ATTESTED_ATTRIBUTES": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("cbPublicKeyBlob", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_ISOLATED_KEY_ATTESTED_ATTRIBUTES", pack=False, align=None),
    "NCRYPT_KEY_ACCESS_POLICY_BLOB": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwPolicyFlags", SimTypeInt(signed=False, label="UInt32")), ("cbUserSid", SimTypeInt(signed=False, label="UInt32")), ("cbApplicationSid", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_KEY_ACCESS_POLICY_BLOB", pack=False, align=None),
    "NCRYPT_KEY_ATTEST_PADDING_INFO": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("pbKeyBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKeyBlob", SimTypeInt(signed=False, label="UInt32")), ("pbKeyAuth", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKeyAuth", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_KEY_ATTEST_PADDING_INFO", pack=False, align=None),
    "NCRYPT_KEY_BLOB_HEADER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("cbAlgName", SimTypeInt(signed=False, label="UInt32")), ("cbKeyData", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_KEY_BLOB_HEADER", pack=False, align=None),
    "NCRYPT_KEY_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "NCRYPT_OPERATION": SimTypeInt(signed=False, label="NCRYPT_OPERATION"),
    "NCRYPT_PCP_HMAC_AUTH_SIGNATURE_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("iExpiration", SimTypeInt(signed=True, label="Int32")), ("pabNonce", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("pabPolicyRef", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("pabHMAC", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NCRYPT_PCP_HMAC_AUTH_SIGNATURE_INFO", pack=False, align=None),
    "NCRYPT_PCP_RAW_POLICYDIGEST_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbDigest", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_PCP_RAW_POLICYDIGEST_INFO", pack=False, align=None),
    "NCRYPT_PCP_TPM_FW_VERSION_INFO": SimStruct(OrderedDict((("major1", SimTypeShort(signed=False, label="UInt16")), ("major2", SimTypeShort(signed=False, label="UInt16")), ("minor1", SimTypeShort(signed=False, label="UInt16")), ("minor2", SimTypeShort(signed=False, label="UInt16")),)), name="NCRYPT_PCP_TPM_FW_VERSION_INFO", pack=False, align=None),
    "NCRYPT_PCP_TPM_WEB_AUTHN_ATTESTATION_STATEMENT": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("HeaderSize", SimTypeInt(signed=False, label="UInt32")), ("cbCertifyInfo", SimTypeInt(signed=False, label="UInt32")), ("cbSignature", SimTypeInt(signed=False, label="UInt32")), ("cbTpmPublic", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_PCP_TPM_WEB_AUTHN_ATTESTATION_STATEMENT", pack=False, align=None),
    "NCRYPT_PLATFORM_ATTEST_PADDING_INFO": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("pcrMask", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_PLATFORM_ATTEST_PADDING_INFO", pack=False, align=None),
    "NCRYPT_PROTECT_STREAM_INFO": SimStruct(OrderedDict((("pfnStreamOutput", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackCtxt", "pbData", "cbData", "fFinal"]), offset=0)), ("pvCallbackCtxt", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NCRYPT_PROTECT_STREAM_INFO", pack=False, align=None),
    "NCRYPT_PROTECT_STREAM_INFO_EX": SimStruct(OrderedDict((("pfnStreamOutput", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackCtxt", "pbData", "cbData", "hDescriptor", "fFinal"]), offset=0)), ("pvCallbackCtxt", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NCRYPT_PROTECT_STREAM_INFO_EX", pack=False, align=None),
    "NCRYPT_PROV_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "NCRYPT_SECRET_HANDLE": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "NCRYPT_STREAM_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NCRYPT_SUPPORTED_LENGTHS": SimStruct(OrderedDict((("dwMinLength", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLength", SimTypeInt(signed=False, label="UInt32")), ("dwIncrement", SimTypeInt(signed=False, label="UInt32")), ("dwDefaultLength", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_SUPPORTED_LENGTHS", pack=False, align=None),
    "NCRYPT_TPM_LOADABLE_KEY_BLOB_HEADER": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("cbHeader", SimTypeInt(signed=False, label="UInt32")), ("cbPublic", SimTypeInt(signed=False, label="UInt32")), ("cbPrivate", SimTypeInt(signed=False, label="UInt32")), ("cbName", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_TPM_LOADABLE_KEY_BLOB_HEADER", pack=False, align=None),
    "NCRYPT_TPM_PLATFORM_ATTESTATION_STATEMENT": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("pcrAlg", SimTypeInt(signed=False, label="UInt32")), ("cbSignature", SimTypeInt(signed=False, label="UInt32")), ("cbQuote", SimTypeInt(signed=False, label="UInt32")), ("cbPcrs", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_TPM_PLATFORM_ATTESTATION_STATEMENT", pack=False, align=None),
    "NCRYPT_UI_POLICY": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszCreationTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NCRYPT_UI_POLICY", pack=False, align=None),
    "NCRYPT_VSM_KEY_ATTESTATION_CLAIM_RESTRICTIONS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("TrustletId", SimTypeLongLong(signed=False, label="UInt64")), ("MinSvn", SimTypeInt(signed=False, label="UInt32")), ("FlagsMask", SimTypeInt(signed=False, label="UInt32")), ("FlagsExpected", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_VSM_KEY_ATTESTATION_CLAIM_RESTRICTIONS", pack=False, align=None),
    "NCRYPT_VSM_KEY_ATTESTATION_STATEMENT": SimStruct(OrderedDict((("Magic", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("cbSignature", SimTypeInt(signed=False, label="UInt32")), ("cbReport", SimTypeInt(signed=False, label="UInt32")), ("cbAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="NCRYPT_VSM_KEY_ATTESTATION_STATEMENT", pack=False, align=None),
    "NC_ADDRESS": SimStruct(OrderedDict((("pAddrInfo", SimTypePointer(SimTypeRef("NET_ADDRESS_INFO", SimStruct), offset=0)), ("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("PrefixLength", SimTypeChar(label="Byte")),)), name="NC_ADDRESS", pack=False, align=None),
    "NCryptAlgorithmName": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwClass", SimTypeInt(signed=False, label="NCRYPT_ALGORITHM_NAME_CLASS")), ("dwAlgOperations", SimTypeInt(signed=False, label="NCRYPT_OPERATION")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NCryptAlgorithmName", pack=False, align=None),
    "NCryptKeyName": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszAlgid", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLegacyKeySpec", SimTypeInt(signed=False, label="CERT_KEY_SPEC")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NCryptKeyName", pack=False, align=None),
    "NCryptProviderName": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NCryptProviderName", pack=False, align=None),
    "NDIS_802_11_AI_REQFI": SimStruct(OrderedDict((("Capabilities", SimTypeShort(signed=False, label="UInt16")), ("ListenInterval", SimTypeShort(signed=False, label="UInt16")), ("CurrentAPAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="NDIS_802_11_AI_REQFI", pack=False, align=None),
    "NDIS_802_11_AI_RESFI": SimStruct(OrderedDict((("Capabilities", SimTypeShort(signed=False, label="UInt16")), ("StatusCode", SimTypeShort(signed=False, label="UInt16")), ("AssociationId", SimTypeShort(signed=False, label="UInt16")),)), name="NDIS_802_11_AI_RESFI", pack=False, align=None),
    "NDIS_802_11_ASSOCIATION_INFORMATION": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("AvailableRequestFixedIEs", SimTypeShort(signed=False, label="UInt16")), ("RequestFixedIEs", SimTypeRef("NDIS_802_11_AI_REQFI", SimStruct)), ("RequestIELength", SimTypeInt(signed=False, label="UInt32")), ("OffsetRequestIEs", SimTypeInt(signed=False, label="UInt32")), ("AvailableResponseFixedIEs", SimTypeShort(signed=False, label="UInt16")), ("ResponseFixedIEs", SimTypeRef("NDIS_802_11_AI_RESFI", SimStruct)), ("ResponseIELength", SimTypeInt(signed=False, label="UInt32")), ("OffsetResponseIEs", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_802_11_ASSOCIATION_INFORMATION", pack=False, align=None),
    "NDIS_802_11_AUTHENTICATION_ENCRYPTION": SimStruct(OrderedDict((("AuthModeSupported", SimTypeInt(signed=False, label="NDIS_802_11_AUTHENTICATION_MODE")), ("EncryptStatusSupported", SimTypeInt(signed=False, label="NDIS_802_11_WEP_STATUS")),)), name="NDIS_802_11_AUTHENTICATION_ENCRYPTION", pack=False, align=None),
    "NDIS_802_11_AUTHENTICATION_EVENT": SimStruct(OrderedDict((("Status", SimTypeRef("NDIS_802_11_STATUS_INDICATION", SimStruct)), ("Request", SimTypePointer(SimTypeRef("NDIS_802_11_AUTHENTICATION_REQUEST", SimStruct), offset=0)),)), name="NDIS_802_11_AUTHENTICATION_EVENT", pack=False, align=None),
    "NDIS_802_11_AUTHENTICATION_MODE": SimTypeInt(signed=False, label="NDIS_802_11_AUTHENTICATION_MODE"),
    "NDIS_802_11_AUTHENTICATION_REQUEST": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Bssid", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_802_11_AUTHENTICATION_REQUEST", pack=False, align=None),
    "NDIS_802_11_BSSID_LIST": SimStruct(OrderedDict((("NumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("Bssid", SimTypePointer(SimTypeRef("NDIS_WLAN_BSSID", SimStruct), offset=0)),)), name="NDIS_802_11_BSSID_LIST", pack=False, align=None),
    "NDIS_802_11_BSSID_LIST_EX": SimStruct(OrderedDict((("NumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("Bssid", SimTypePointer(SimTypeRef("NDIS_WLAN_BSSID_EX", SimStruct), offset=0)),)), name="NDIS_802_11_BSSID_LIST_EX", pack=False, align=None),
    "NDIS_802_11_CAPABILITY": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("NoOfPMKIDs", SimTypeInt(signed=False, label="UInt32")), ("NoOfAuthEncryptPairsSupported", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationEncryptionSupported", SimTypePointer(SimTypeRef("NDIS_802_11_AUTHENTICATION_ENCRYPTION", SimStruct), offset=0)),)), name="NDIS_802_11_CAPABILITY", pack=False, align=None),
    "NDIS_802_11_CONFIGURATION": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("BeaconPeriod", SimTypeInt(signed=False, label="UInt32")), ("ATIMWindow", SimTypeInt(signed=False, label="UInt32")), ("DSConfig", SimTypeInt(signed=False, label="UInt32")), ("FHConfig", SimTypeRef("NDIS_802_11_CONFIGURATION_FH", SimStruct)),)), name="NDIS_802_11_CONFIGURATION", pack=False, align=None),
    "NDIS_802_11_CONFIGURATION_FH": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("HopPattern", SimTypeInt(signed=False, label="UInt32")), ("HopSet", SimTypeInt(signed=False, label="UInt32")), ("DwellTime", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_802_11_CONFIGURATION_FH", pack=False, align=None),
    "NDIS_802_11_FIXED_IEs": SimStruct(OrderedDict((("Timestamp", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("BeaconInterval", SimTypeShort(signed=False, label="UInt16")), ("Capabilities", SimTypeShort(signed=False, label="UInt16")),)), name="NDIS_802_11_FIXED_IEs", pack=False, align=None),
    "NDIS_802_11_KEY": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("KeyIndex", SimTypeInt(signed=False, label="UInt32")), ("KeyLength", SimTypeInt(signed=False, label="UInt32")), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("KeyRSC", SimTypeLongLong(signed=False, label="UInt64")), ("KeyMaterial", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDIS_802_11_KEY", pack=False, align=None),
    "NDIS_802_11_MEDIA_STREAM_MODE": SimTypeInt(signed=False, label="NDIS_802_11_MEDIA_STREAM_MODE"),
    "NDIS_802_11_NETWORK_INFRASTRUCTURE": SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_INFRASTRUCTURE"),
    "NDIS_802_11_NETWORK_TYPE": SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_TYPE"),
    "NDIS_802_11_NETWORK_TYPE_LIST": SimStruct(OrderedDict((("NumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("NetworkType", SimTypePointer(SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_TYPE"), offset=0)),)), name="NDIS_802_11_NETWORK_TYPE_LIST", pack=False, align=None),
    "NDIS_802_11_NON_BCAST_SSID_LIST": SimStruct(OrderedDict((("NumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("Non_Bcast_Ssid", SimTypePointer(SimTypeRef("NDIS_802_11_SSID", SimStruct), offset=0)),)), name="NDIS_802_11_NON_BCAST_SSID_LIST", pack=False, align=None),
    "NDIS_802_11_PMKID": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("BSSIDInfoCount", SimTypeInt(signed=False, label="UInt32")), ("BSSIDInfo", SimTypePointer(SimTypeRef("BSSID_INFO", SimStruct), offset=0)),)), name="NDIS_802_11_PMKID", pack=False, align=None),
    "NDIS_802_11_PMKID_CANDIDATE_LIST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("NumCandidates", SimTypeInt(signed=False, label="UInt32")), ("CandidateList", SimTypePointer(SimTypeRef("PMKID_CANDIDATE", SimStruct), offset=0)),)), name="NDIS_802_11_PMKID_CANDIDATE_LIST", pack=False, align=None),
    "NDIS_802_11_POWER_MODE": SimTypeInt(signed=False, label="NDIS_802_11_POWER_MODE"),
    "NDIS_802_11_PRIVACY_FILTER": SimTypeInt(signed=False, label="NDIS_802_11_PRIVACY_FILTER"),
    "NDIS_802_11_RADIO_STATUS": SimTypeInt(signed=False, label="NDIS_802_11_RADIO_STATUS"),
    "NDIS_802_11_RELOAD_DEFAULTS": SimTypeInt(signed=False, label="NDIS_802_11_RELOAD_DEFAULTS"),
    "NDIS_802_11_REMOVE_KEY": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("KeyIndex", SimTypeInt(signed=False, label="UInt32")), ("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="NDIS_802_11_REMOVE_KEY", pack=False, align=None),
    "NDIS_802_11_SSID": SimStruct(OrderedDict((("SsidLength", SimTypeInt(signed=False, label="UInt32")), ("Ssid", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NDIS_802_11_SSID", pack=False, align=None),
    "NDIS_802_11_STATISTICS": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("TransmittedFragmentCount", SimTypeLongLong(signed=True, label="Int64")), ("MulticastTransmittedFrameCount", SimTypeLongLong(signed=True, label="Int64")), ("FailedCount", SimTypeLongLong(signed=True, label="Int64")), ("RetryCount", SimTypeLongLong(signed=True, label="Int64")), ("MultipleRetryCount", SimTypeLongLong(signed=True, label="Int64")), ("RTSSuccessCount", SimTypeLongLong(signed=True, label="Int64")), ("RTSFailureCount", SimTypeLongLong(signed=True, label="Int64")), ("ACKFailureCount", SimTypeLongLong(signed=True, label="Int64")), ("FrameDuplicateCount", SimTypeLongLong(signed=True, label="Int64")), ("ReceivedFragmentCount", SimTypeLongLong(signed=True, label="Int64")), ("MulticastReceivedFrameCount", SimTypeLongLong(signed=True, label="Int64")), ("FCSErrorCount", SimTypeLongLong(signed=True, label="Int64")), ("TKIPLocalMICFailures", SimTypeLongLong(signed=True, label="Int64")), ("TKIPICVErrorCount", SimTypeLongLong(signed=True, label="Int64")), ("TKIPCounterMeasuresInvoked", SimTypeLongLong(signed=True, label="Int64")), ("TKIPReplays", SimTypeLongLong(signed=True, label="Int64")), ("CCMPFormatErrors", SimTypeLongLong(signed=True, label="Int64")), ("CCMPReplays", SimTypeLongLong(signed=True, label="Int64")), ("CCMPDecryptErrors", SimTypeLongLong(signed=True, label="Int64")), ("FourWayHandshakeFailures", SimTypeLongLong(signed=True, label="Int64")), ("WEPUndecryptableCount", SimTypeLongLong(signed=True, label="Int64")), ("WEPICVErrorCount", SimTypeLongLong(signed=True, label="Int64")), ("DecryptSuccessCount", SimTypeLongLong(signed=True, label="Int64")), ("DecryptFailureCount", SimTypeLongLong(signed=True, label="Int64")),)), name="NDIS_802_11_STATISTICS", pack=False, align=None),
    "NDIS_802_11_STATUS_INDICATION": SimStruct(OrderedDict((("StatusType", SimTypeInt(signed=False, label="NDIS_802_11_STATUS_TYPE")),)), name="NDIS_802_11_STATUS_INDICATION", pack=False, align=None),
    "NDIS_802_11_STATUS_TYPE": SimTypeInt(signed=False, label="NDIS_802_11_STATUS_TYPE"),
    "NDIS_802_11_TEST": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"AuthenticationEvent": SimTypeRef("NDIS_802_11_AUTHENTICATION_EVENT", SimStruct), "RssiTrigger": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="NDIS_802_11_TEST", pack=False, align=None),
    "NDIS_802_11_VARIABLE_IEs": SimStruct(OrderedDict((("ElementID", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDIS_802_11_VARIABLE_IEs", pack=False, align=None),
    "NDIS_802_11_WEP": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("KeyIndex", SimTypeInt(signed=False, label="UInt32")), ("KeyLength", SimTypeInt(signed=False, label="UInt32")), ("KeyMaterial", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDIS_802_11_WEP", pack=False, align=None),
    "NDIS_802_11_WEP_STATUS": SimTypeInt(signed=False, label="NDIS_802_11_WEP_STATUS"),
    "NDIS_802_5_RING_STATE": SimTypeInt(signed=False, label="NDIS_802_5_RING_STATE"),
    "NDIS_AF_LIST": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_CALL_MANAGER_CHARACTERISTICS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_CLASS_ID": SimTypeInt(signed=False, label="NDIS_CLASS_ID"),
    "NDIS_CONFIGURATION_PARAMETER": SimStruct(OrderedDict((("ParameterType", SimTypeInt(signed=False, label="NDIS_PARAMETER_TYPE")), ("ParameterData", SimUnion({"IntegerData": SimTypeInt(signed=False, label="UInt32"), "StringData": SimTypeRef("UNICODE_STRING", SimStruct), "BinaryData": SimTypeRef("BINARY_DATA", SimStruct)}, name="<anon>", label="None")),)), name="NDIS_CONFIGURATION_PARAMETER", pack=False, align=None),
    "NDIS_CO_DEVICE_PROFILE": SimStruct(OrderedDict((("DeviceDescription", SimTypeRef("NDIS_VAR_DATA_DESC", SimStruct)), ("DevSpecificInfo", SimTypeRef("NDIS_VAR_DATA_DESC", SimStruct)), ("ulTAPISupplementaryPassThru", SimTypeInt(signed=False, label="UInt32")), ("ulAddressModes", SimTypeInt(signed=False, label="UInt32")), ("ulNumAddresses", SimTypeInt(signed=False, label="UInt32")), ("ulBearerModes", SimTypeInt(signed=False, label="UInt32")), ("ulMaxTxRate", SimTypeInt(signed=False, label="UInt32")), ("ulMinTxRate", SimTypeInt(signed=False, label="UInt32")), ("ulMaxRxRate", SimTypeInt(signed=False, label="UInt32")), ("ulMinRxRate", SimTypeInt(signed=False, label="UInt32")), ("ulMediaModes", SimTypeInt(signed=False, label="UInt32")), ("ulGenerateToneModes", SimTypeInt(signed=False, label="UInt32")), ("ulGenerateToneMaxNumFreq", SimTypeInt(signed=False, label="UInt32")), ("ulGenerateDigitModes", SimTypeInt(signed=False, label="UInt32")), ("ulMonitorToneMaxNumFreq", SimTypeInt(signed=False, label="UInt32")), ("ulMonitorToneMaxNumEntries", SimTypeInt(signed=False, label="UInt32")), ("ulMonitorDigitModes", SimTypeInt(signed=False, label="UInt32")), ("ulGatherDigitsMinTimeout", SimTypeInt(signed=False, label="UInt32")), ("ulGatherDigitsMaxTimeout", SimTypeInt(signed=False, label="UInt32")), ("ulDevCapFlags", SimTypeInt(signed=False, label="UInt32")), ("ulMaxNumActiveCalls", SimTypeInt(signed=False, label="UInt32")), ("ulAnswerMode", SimTypeInt(signed=False, label="UInt32")), ("ulUUIAcceptSize", SimTypeInt(signed=False, label="UInt32")), ("ulUUIAnswerSize", SimTypeInt(signed=False, label="UInt32")), ("ulUUIMakeCallSize", SimTypeInt(signed=False, label="UInt32")), ("ulUUIDropSize", SimTypeInt(signed=False, label="UInt32")), ("ulUUISendUserUserInfoSize", SimTypeInt(signed=False, label="UInt32")), ("ulUUICallInfoSize", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_CO_DEVICE_PROFILE", pack=False, align=None),
    "NDIS_CO_LINK_SPEED": SimStruct(OrderedDict((("Outbound", SimTypeInt(signed=False, label="UInt32")), ("Inbound", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_CO_LINK_SPEED", pack=False, align=None),
    "NDIS_DEVICE_PNP_EVENT": SimTypeInt(signed=False, label="NDIS_DEVICE_PNP_EVENT"),
    "NDIS_DEVICE_POWER_STATE": SimTypeInt(signed=False, label="NDIS_DEVICE_POWER_STATE"),
    "NDIS_DMA_BLOCK": SimStruct(OrderedDict((("MapRegisterBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationEvent", SimTypeRef("KEVENT", SimStruct)), ("SystemAdapterObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Miniport", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InProgress", SimTypeChar(label="Byte")),)), name="NDIS_DMA_BLOCK", pack=False, align=None),
    "NDIS_DMA_DESCRIPTION": SimStruct(OrderedDict((("DemandMode", SimTypeChar(label="Byte")), ("AutoInitialize", SimTypeChar(label="Byte")), ("DmaChannelSpecified", SimTypeChar(label="Byte")), ("DmaWidth", SimTypeInt(signed=False, label="DMA_WIDTH")), ("DmaSpeed", SimTypeInt(signed=False, label="DMA_SPEED")), ("DmaPort", SimTypeInt(signed=False, label="UInt32")), ("DmaChannel", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_DMA_DESCRIPTION", pack=False, align=None),
    "NDIS_ENVIRONMENT_TYPE": SimTypeInt(signed=False, label="NDIS_ENVIRONMENT_TYPE"),
    "NDIS_EVENT": SimStruct(OrderedDict((("Event", SimTypeRef("KEVENT", SimStruct)),)), name="NDIS_EVENT", pack=False, align=None),
    "NDIS_FDDI_ATTACHMENT_TYPE": SimTypeInt(signed=False, label="NDIS_FDDI_ATTACHMENT_TYPE"),
    "NDIS_FDDI_LCONNECTION_STATE": SimTypeInt(signed=False, label="NDIS_FDDI_LCONNECTION_STATE"),
    "NDIS_FDDI_RING_MGT_STATE": SimTypeInt(signed=False, label="NDIS_FDDI_RING_MGT_STATE"),
    "NDIS_GUID": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("Anonymous", SimUnion({"Oid": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_GUID", pack=False, align=None),
    "NDIS_HARDWARE_CROSSTIMESTAMP": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SystemTimestamp1", SimTypeLongLong(signed=False, label="UInt64")), ("HardwareClockTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("SystemTimestamp2", SimTypeLongLong(signed=False, label="UInt64")),)), name="NDIS_HARDWARE_CROSSTIMESTAMP", pack=False, align=None),
    "NDIS_HARDWARE_STATUS": SimTypeInt(signed=False, label="NDIS_HARDWARE_STATUS"),
    "NDIS_INTERFACE_INFORMATION": SimStruct(OrderedDict((("ifOperStatus", SimTypeInt(signed=False, label="NET_IF_OPER_STATUS")), ("ifOperStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("MediaConnectState", SimTypeInt(signed=False, label="NET_IF_MEDIA_CONNECT_STATE")), ("MediaDuplexState", SimTypeInt(signed=False, label="NET_IF_MEDIA_DUPLEX_STATE")), ("ifMtu", SimTypeInt(signed=False, label="UInt32")), ("ifPromiscuousMode", SimTypeChar(label="Byte")), ("ifDeviceWakeUpEnable", SimTypeChar(label="Byte")), ("XmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("RcvLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("ifLastChange", SimTypeLongLong(signed=False, label="UInt64")), ("ifCounterDiscontinuityTime", SimTypeLongLong(signed=False, label="UInt64")), ("ifInUnknownProtos", SimTypeLongLong(signed=False, label="UInt64")), ("ifInDiscards", SimTypeLongLong(signed=False, label="UInt64")), ("ifInErrors", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInMulticastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInBroadcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutMulticastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutBroadcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifOutErrors", SimTypeLongLong(signed=False, label="UInt64")), ("ifOutDiscards", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInUcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInMulticastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInBroadcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutUcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutMulticastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutBroadcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("CompartmentId", SimTypeInt(signed=False, label="UInt32")), ("SupportedStatistics", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_INTERFACE_INFORMATION", pack=False, align=None),
    "NDIS_INTERFACE_TYPE": SimTypeInt(signed=False, label="NDIS_INTERFACE_TYPE"),
    "NDIS_INTERRUPT_MODERATION": SimTypeInt(signed=False, label="NDIS_INTERRUPT_MODERATION"),
    "NDIS_INTERRUPT_MODERATION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("InterruptModeration", SimTypeInt(signed=False, label="NDIS_INTERRUPT_MODERATION")),)), name="NDIS_INTERRUPT_MODERATION_PARAMETERS", pack=False, align=None),
    "NDIS_IPSEC_OFFLOAD_V1": SimStruct(OrderedDict((("Supported", SimTypeRef("_Supported_e__Struct", SimStruct)), ("IPv4AH", SimTypeRef("_IPv4AH_e__Struct", SimStruct)), ("IPv4ESP", SimTypeRef("_IPv4ESP_e__Struct", SimStruct)),)), name="NDIS_IPSEC_OFFLOAD_V1", pack=False, align=None),
    "NDIS_IP_OPER_STATE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("IpOperationalStatus", SimTypeRef("NDIS_IP_OPER_STATUS", SimStruct)),)), name="NDIS_IP_OPER_STATE", pack=False, align=None),
    "NDIS_IP_OPER_STATUS": SimStruct(OrderedDict((("AddressFamily", SimTypeInt(signed=False, label="UInt32")), ("OperationalStatus", SimTypeInt(signed=False, label="NET_IF_OPER_STATUS")), ("OperationalStatusFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_IP_OPER_STATUS", pack=False, align=None),
    "NDIS_IP_OPER_STATUS_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumberofAddressFamiliesReturned", SimTypeInt(signed=False, label="UInt32")), ("IpOperationalStatus", SimTypeArray(SimTypeRef("NDIS_IP_OPER_STATUS", SimStruct), 32)),)), name="NDIS_IP_OPER_STATUS_INFO", pack=False, align=None),
    "NDIS_IRDA_PACKET_INFO": SimStruct(OrderedDict((("ExtraBOFs", SimTypeInt(signed=False, label="UInt32")), ("MinTurnAroundTime", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_IRDA_PACKET_INFO", pack=False, align=None),
    "NDIS_LINK_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("MediaDuplexState", SimTypeInt(signed=False, label="NET_IF_MEDIA_DUPLEX_STATE")), ("XmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("RcvLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("PauseFunctions", SimTypeInt(signed=False, label="NDIS_SUPPORTED_PAUSE_FUNCTIONS")), ("AutoNegotiationFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_LINK_PARAMETERS", pack=False, align=None),
    "NDIS_LINK_SPEED": SimStruct(OrderedDict((("XmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("RcvLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")),)), name="NDIS_LINK_SPEED", pack=False, align=None),
    "NDIS_LINK_STATE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("MediaConnectState", SimTypeInt(signed=False, label="NET_IF_MEDIA_CONNECT_STATE")), ("MediaDuplexState", SimTypeInt(signed=False, label="NET_IF_MEDIA_DUPLEX_STATE")), ("XmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("RcvLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("PauseFunctions", SimTypeInt(signed=False, label="NDIS_SUPPORTED_PAUSE_FUNCTIONS")), ("AutoNegotiationFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_LINK_STATE", pack=False, align=None),
    "NDIS_MEDIA_STATE": SimTypeInt(signed=False, label="NDIS_MEDIA_STATE"),
    "NDIS_MEDIUM": SimTypeInt(signed=False, label="NDIS_MEDIUM"),
    "NDIS_MINIPORT_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_MINIPORT_TIMER": SimStruct(OrderedDict((("Timer", SimTypeRef("KTIMER", SimStruct)), ("Dpc", SimTypeRef("KDPC", SimStruct)), ("MiniportTimerFunction", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("MiniportTimerContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Miniport", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("NextTimer", SimTypePointer(SimTypeRef("NDIS_MINIPORT_TIMER", SimStruct), offset=0)),)), name="NDIS_MINIPORT_TIMER", pack=False, align=None),
    "NDIS_M_DRIVER_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_NETWORK_CHANGE_TYPE": SimTypeInt(signed=False, label="NDIS_NETWORK_CHANGE_TYPE"),
    "NDIS_OBJECT_HEADER": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Revision", SimTypeChar(label="Byte")), ("Size", SimTypeShort(signed=False, label="UInt16")),)), name="NDIS_OBJECT_HEADER", pack=False, align=None),
    "NDIS_OFFLOAD": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Checksum", SimTypeRef("NDIS_TCP_IP_CHECKSUM_OFFLOAD", SimStruct)), ("LsoV1", SimTypeRef("NDIS_TCP_LARGE_SEND_OFFLOAD_V1", SimStruct)), ("IPsecV1", SimTypeRef("NDIS_IPSEC_OFFLOAD_V1", SimStruct)), ("LsoV2", SimTypeRef("NDIS_TCP_LARGE_SEND_OFFLOAD_V2", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_OFFLOAD", pack=False, align=None),
    "NDIS_OFFLOAD_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("IPv4Checksum", SimTypeChar(label="Byte")), ("TCPIPv4Checksum", SimTypeChar(label="Byte")), ("UDPIPv4Checksum", SimTypeChar(label="Byte")), ("TCPIPv6Checksum", SimTypeChar(label="Byte")), ("UDPIPv6Checksum", SimTypeChar(label="Byte")), ("LsoV1", SimTypeChar(label="Byte")), ("IPsecV1", SimTypeChar(label="Byte")), ("LsoV2IPv4", SimTypeChar(label="Byte")), ("LsoV2IPv6", SimTypeChar(label="Byte")), ("TcpConnectionIPv4", SimTypeChar(label="Byte")), ("TcpConnectionIPv6", SimTypeChar(label="Byte")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_OFFLOAD_PARAMETERS", pack=False, align=None),
    "NDIS_OPEN_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_OPER_STATE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("OperationalStatus", SimTypeInt(signed=False, label="NET_IF_OPER_STATUS")), ("OperationalStatusFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_OPER_STATE", pack=False, align=None),
    "NDIS_PACKET_8021Q_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"TagHeader": SimTypeRef("_TagHeader_e__Struct", SimStruct), "Value": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="NDIS_PACKET_8021Q_INFO", pack=False, align=None),
    "NDIS_PARAMETER_TYPE": SimTypeInt(signed=False, label="NDIS_PARAMETER_TYPE"),
    "NDIS_PCI_DEVICE_CUSTOM_PROPERTIES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("CurrentSpeedAndMode", SimTypeInt(signed=False, label="UInt32")), ("CurrentPayloadSize", SimTypeInt(signed=False, label="UInt32")), ("MaxPayloadSize", SimTypeInt(signed=False, label="UInt32")), ("MaxReadRequestSize", SimTypeInt(signed=False, label="UInt32")), ("CurrentLinkSpeed", SimTypeInt(signed=False, label="UInt32")), ("CurrentLinkWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxLinkSpeed", SimTypeInt(signed=False, label="UInt32")), ("MaxLinkWidth", SimTypeInt(signed=False, label="UInt32")), ("PciExpressVersion", SimTypeInt(signed=False, label="UInt32")), ("InterruptType", SimTypeInt(signed=False, label="UInt32")), ("MaxInterruptMessages", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_PCI_DEVICE_CUSTOM_PROPERTIES", pack=False, align=None),
    "NDIS_PD_COUNTER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_PD_FILTER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_PD_PROVIDER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_PER_PACKET_INFO": SimTypeInt(signed=False, label="NDIS_PER_PACKET_INFO"),
    "NDIS_PHYSICAL_ADDRESS_UNIT": SimStruct(OrderedDict((("PhysicalAddress", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_PHYSICAL_ADDRESS_UNIT", pack=False, align=None),
    "NDIS_PHYSICAL_MEDIUM": SimTypeInt(signed=False, label="NDIS_PHYSICAL_MEDIUM"),
    "NDIS_PM_PACKET_PATTERN": SimStruct(OrderedDict((("Priority", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("MaskSize", SimTypeInt(signed=False, label="UInt32")), ("PatternOffset", SimTypeInt(signed=False, label="UInt32")), ("PatternSize", SimTypeInt(signed=False, label="UInt32")), ("PatternFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_PM_PACKET_PATTERN", pack=False, align=None),
    "NDIS_PM_WAKE_UP_CAPABILITIES": SimStruct(OrderedDict((("MinMagicPacketWakeUp", SimTypeInt(signed=False, label="NDIS_DEVICE_POWER_STATE")), ("MinPatternWakeUp", SimTypeInt(signed=False, label="NDIS_DEVICE_POWER_STATE")), ("MinLinkChangeWakeUp", SimTypeInt(signed=False, label="NDIS_DEVICE_POWER_STATE")),)), name="NDIS_PM_WAKE_UP_CAPABILITIES", pack=False, align=None),
    "NDIS_PNP_CAPABILITIES": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("WakeUpCapabilities", SimTypeRef("NDIS_PM_WAKE_UP_CAPABILITIES", SimStruct)),)), name="NDIS_PNP_CAPABILITIES", pack=False, align=None),
    "NDIS_POLL_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_PORT": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("NDIS_PORT", SimStruct), offset=0)), ("NdisReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MiniportReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ProtocolReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PortCharacteristics", SimTypeRef("NDIS_PORT_CHARACTERISTICS", SimStruct)),)), name="NDIS_PORT", pack=False, align=None),
    "NDIS_PORT_ARRAY": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("NumberOfPorts", SimTypeInt(signed=False, label="UInt32")), ("OffsetFirstPort", SimTypeInt(signed=False, label="UInt32")), ("ElementSize", SimTypeInt(signed=False, label="UInt32")), ("Ports", SimTypePointer(SimTypeRef("NDIS_PORT_CHARACTERISTICS", SimStruct), offset=0)),)), name="NDIS_PORT_ARRAY", pack=False, align=None),
    "NDIS_PORT_AUTHENTICATION_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("SendControlState", SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE")), ("RcvControlState", SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE")), ("SendAuthorizationState", SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE")), ("RcvAuthorizationState", SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE")),)), name="NDIS_PORT_AUTHENTICATION_PARAMETERS", pack=False, align=None),
    "NDIS_PORT_AUTHORIZATION_STATE": SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE"),
    "NDIS_PORT_CHARACTERISTICS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PortNumber", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="NDIS_PORT_TYPE")), ("MediaConnectState", SimTypeInt(signed=False, label="NET_IF_MEDIA_CONNECT_STATE")), ("XmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("RcvLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("Direction", SimTypeInt(signed=False, label="NET_IF_DIRECTION_TYPE")), ("SendControlState", SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE")), ("RcvControlState", SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE")), ("SendAuthorizationState", SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE")), ("RcvAuthorizationState", SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE")),)), name="NDIS_PORT_CHARACTERISTICS", pack=False, align=None),
    "NDIS_PORT_CONTROL_STATE": SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE"),
    "NDIS_PORT_STATE": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("MediaConnectState", SimTypeInt(signed=False, label="NET_IF_MEDIA_CONNECT_STATE")), ("XmitLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("RcvLinkSpeed", SimTypeLongLong(signed=False, label="UInt64")), ("Direction", SimTypeInt(signed=False, label="NET_IF_DIRECTION_TYPE")), ("SendControlState", SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE")), ("RcvControlState", SimTypeInt(signed=False, label="NDIS_PORT_CONTROL_STATE")), ("SendAuthorizationState", SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE")), ("RcvAuthorizationState", SimTypeInt(signed=False, label="NDIS_PORT_AUTHORIZATION_STATE")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_PORT_STATE", pack=False, align=None),
    "NDIS_PORT_TYPE": SimTypeInt(signed=False, label="NDIS_PORT_TYPE"),
    "NDIS_POWER_PROFILE": SimTypeInt(signed=False, label="NDIS_POWER_PROFILE"),
    "NDIS_PROC": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "NDIS_PROCESSOR_TYPE": SimTypeInt(signed=False, label="NDIS_PROCESSOR_TYPE"),
    "NDIS_PROCESSOR_VENDOR": SimTypeInt(signed=False, label="NDIS_PROCESSOR_VENDOR"),
    "NDIS_PROC_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("NDIS_WORK_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["WorkItem", "Context"]), offset=0),
    "NDIS_PROTOCOL_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDIS_RECEIVE_HASH_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("HashInformation", SimTypeInt(signed=False, label="UInt32")), ("HashSecretKeySize", SimTypeShort(signed=False, label="UInt16")), ("HashSecretKeyOffset", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_RECEIVE_HASH_PARAMETERS", pack=False, align=None),
    "NDIS_RECEIVE_SCALE_CAPABILITIES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("CapabilitiesFlags", SimTypeInt(signed=False, label="UInt32")), ("NumberOfInterruptMessages", SimTypeInt(signed=False, label="UInt32")), ("NumberOfReceiveQueues", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_RECEIVE_SCALE_CAPABILITIES", pack=False, align=None),
    "NDIS_RECEIVE_SCALE_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("BaseCpuNumber", SimTypeShort(signed=False, label="UInt16")), ("HashInformation", SimTypeInt(signed=False, label="UInt32")), ("IndirectionTableSize", SimTypeShort(signed=False, label="UInt16")), ("IndirectionTableOffset", SimTypeInt(signed=False, label="UInt32")), ("HashSecretKeySize", SimTypeShort(signed=False, label="UInt16")), ("HashSecretKeyOffset", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_RECEIVE_SCALE_PARAMETERS", pack=False, align=None),
    "NDIS_REQUEST_TYPE": SimTypeInt(signed=False, label="NDIS_REQUEST_TYPE"),
    "NDIS_RW_LOCK": SimStruct(OrderedDict((("Anonymous1", SimUnion({"Anonymous": SimStruct(OrderedDict((("SpinLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "Reserved": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"RefCount": SimTypeArray(SimUnion({"RefCount": SimTypeInt(signed=False, label="UInt32"), "cacheLine": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), 32), "RefCountEx": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 128), "Anonymous": SimStruct(OrderedDict((("RefCountLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedRefCount", SimTypeInt(signed=False, label="UInt32")), ("WriterWaiting", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="NDIS_RW_LOCK", pack=False, align=None),
    "NDIS_RW_LOCK_REFCOUNT": SimUnion({"RefCount": SimTypeInt(signed=False, label="UInt32"), "cacheLine": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"),
    "NDIS_SPIN_LOCK": SimStruct(OrderedDict((("SpinLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("OldIrql", SimTypeChar(label="Byte")),)), name="NDIS_SPIN_LOCK", pack=False, align=None),
    "NDIS_STATISTICS_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("SupportedStatistics", SimTypeInt(signed=False, label="UInt32")), ("ifInDiscards", SimTypeLongLong(signed=False, label="UInt64")), ("ifInErrors", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInMulticastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInBroadcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutUcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutMulticastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutBroadcastPkts", SimTypeLongLong(signed=False, label="UInt64")), ("ifOutErrors", SimTypeLongLong(signed=False, label="UInt64")), ("ifOutDiscards", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInUcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInMulticastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCInBroadcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutUcastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutMulticastOctets", SimTypeLongLong(signed=False, label="UInt64")), ("ifHCOutBroadcastOctets", SimTypeLongLong(signed=False, label="UInt64")),)), name="NDIS_STATISTICS_INFO", pack=False, align=None),
    "NDIS_STATISTICS_VALUE": SimStruct(OrderedDict((("Oid", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDIS_STATISTICS_VALUE", pack=False, align=None),
    "NDIS_STATISTICS_VALUE_EX": SimStruct(OrderedDict((("Oid", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDIS_STATISTICS_VALUE_EX", pack=False, align=None),
    "NDIS_SUPPORTED_PAUSE_FUNCTIONS": SimTypeInt(signed=False, label="NDIS_SUPPORTED_PAUSE_FUNCTIONS"),
    "NDIS_TCP_CONNECTION_OFFLOAD": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("TcpConnectionOffloadCapacity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_TCP_CONNECTION_OFFLOAD", pack=False, align=None),
    "NDIS_TCP_IP_CHECKSUM_OFFLOAD": SimStruct(OrderedDict((("IPv4Transmit", SimTypeRef("_IPv4Transmit_e__Struct", SimStruct)), ("IPv4Receive", SimTypeRef("_IPv4Receive_e__Struct", SimStruct)), ("IPv6Transmit", SimTypeRef("_IPv6Transmit_e__Struct", SimStruct)), ("IPv6Receive", SimTypeRef("_IPv6Receive_e__Struct", SimStruct)),)), name="NDIS_TCP_IP_CHECKSUM_OFFLOAD", pack=False, align=None),
    "NDIS_TCP_IP_CHECKSUM_PACKET_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"Transmit": SimTypeRef("_Transmit_e__Struct", SimStruct), "Receive": SimTypeRef("_Receive_e__Struct", SimStruct), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="NDIS_TCP_IP_CHECKSUM_PACKET_INFO", pack=False, align=None),
    "NDIS_TCP_LARGE_SEND_OFFLOAD_V1": SimStruct(OrderedDict((("IPv4", SimTypeRef("_IPv4_e__Struct", SimStruct)),)), name="NDIS_TCP_LARGE_SEND_OFFLOAD_V1", pack=False, align=None),
    "NDIS_TCP_LARGE_SEND_OFFLOAD_V2": SimStruct(OrderedDict((("IPv4", SimTypeRef("_IPv4_e__Struct", SimStruct)), ("IPv6", SimTypeRef("_IPv6_e__Struct", SimStruct)),)), name="NDIS_TCP_LARGE_SEND_OFFLOAD_V2", pack=False, align=None),
    "NDIS_TIMEOUT_DPC_REQUEST_CAPABILITIES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TimeoutArrayLength", SimTypeInt(signed=False, label="UInt32")), ("TimeoutArray", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="NDIS_TIMEOUT_DPC_REQUEST_CAPABILITIES", pack=False, align=None),
    "NDIS_TIMER": SimStruct(OrderedDict((("Timer", SimTypeRef("KTIMER", SimStruct)), ("Dpc", SimTypeRef("KDPC", SimStruct)),)), name="NDIS_TIMER", pack=False, align=None),
    "NDIS_TIMER_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["SystemSpecific1", "FunctionContext", "SystemSpecific2", "SystemSpecific3"]), offset=0),
    "NDIS_TIMESTAMP_CAPABILITIES": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("HardwareClockFrequencyHz", SimTypeLongLong(signed=False, label="UInt64")), ("CrossTimestamp", SimTypeChar(label="Byte")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("TimestampFlags", SimTypeRef("NDIS_TIMESTAMP_CAPABILITY_FLAGS", SimStruct)),)), name="NDIS_TIMESTAMP_CAPABILITIES", pack=False, align=None),
    "NDIS_TIMESTAMP_CAPABILITY_FLAGS": SimStruct(OrderedDict((("PtpV2OverUdpIPv4EventMsgReceiveHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv4AllMsgReceiveHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv4EventMsgTransmitHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv4AllMsgTransmitHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6EventMsgReceiveHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6AllMsgReceiveHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6EventMsgTransmitHw", SimTypeChar(label="Byte")), ("PtpV2OverUdpIPv6AllMsgTransmitHw", SimTypeChar(label="Byte")), ("AllReceiveHw", SimTypeChar(label="Byte")), ("AllTransmitHw", SimTypeChar(label="Byte")), ("TaggedTransmitHw", SimTypeChar(label="Byte")), ("AllReceiveSw", SimTypeChar(label="Byte")), ("AllTransmitSw", SimTypeChar(label="Byte")), ("TaggedTransmitSw", SimTypeChar(label="Byte")),)), name="NDIS_TIMESTAMP_CAPABILITY_FLAGS", pack=False, align=None),
    "NDIS_VAR_DATA_DESC": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Offset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="NDIS_VAR_DATA_DESC", pack=False, align=None),
    "NDIS_WAN_FRAGMENT": SimStruct(OrderedDict((("RemoteAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("LocalAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="NDIS_WAN_FRAGMENT", pack=False, align=None),
    "NDIS_WAN_GET_STATS": SimStruct(OrderedDict((("LocalAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("BytesSent", SimTypeInt(signed=False, label="UInt32")), ("BytesRcvd", SimTypeInt(signed=False, label="UInt32")), ("FramesSent", SimTypeInt(signed=False, label="UInt32")), ("FramesRcvd", SimTypeInt(signed=False, label="UInt32")), ("CRCErrors", SimTypeInt(signed=False, label="UInt32")), ("TimeoutErrors", SimTypeInt(signed=False, label="UInt32")), ("AlignmentErrors", SimTypeInt(signed=False, label="UInt32")), ("SerialOverrunErrors", SimTypeInt(signed=False, label="UInt32")), ("FramingErrors", SimTypeInt(signed=False, label="UInt32")), ("BufferOverrunErrors", SimTypeInt(signed=False, label="UInt32")), ("BytesTransmittedUncompressed", SimTypeInt(signed=False, label="UInt32")), ("BytesReceivedUncompressed", SimTypeInt(signed=False, label="UInt32")), ("BytesTransmittedCompressed", SimTypeInt(signed=False, label="UInt32")), ("BytesReceivedCompressed", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_WAN_GET_STATS", pack=False, align=None),
    "NDIS_WAN_HEADER_FORMAT": SimTypeInt(signed=False, label="NDIS_WAN_HEADER_FORMAT"),
    "NDIS_WAN_LINE_DOWN": SimStruct(OrderedDict((("RemoteAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("LocalAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="NDIS_WAN_LINE_DOWN", pack=False, align=None),
    "NDIS_WAN_LINE_UP": SimStruct(OrderedDict((("LinkSpeed", SimTypeInt(signed=False, label="UInt32")), ("MaximumTotalSize", SimTypeInt(signed=False, label="UInt32")), ("Quality", SimTypeInt(signed=False, label="NDIS_WAN_QUALITY")), ("SendWindow", SimTypeShort(signed=False, label="UInt16")), ("RemoteAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("LocalAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ProtocolBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ProtocolBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ProtocolType", SimTypeShort(signed=False, label="UInt16")), ("DeviceName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="NDIS_WAN_LINE_UP", pack=False, align=None),
    "NDIS_WAN_MEDIUM_SUBTYPE": SimTypeInt(signed=False, label="NDIS_WAN_MEDIUM_SUBTYPE"),
    "NDIS_WAN_PROTOCOL_CAPS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_WAN_PROTOCOL_CAPS", pack=False, align=None),
    "NDIS_WAN_QUALITY": SimTypeInt(signed=False, label="NDIS_WAN_QUALITY"),
    "NDIS_WLAN_BSSID": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("MacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Ssid", SimTypeRef("NDIS_802_11_SSID", SimStruct)), ("Privacy", SimTypeInt(signed=False, label="UInt32")), ("Rssi", SimTypeInt(signed=True, label="Int32")), ("NetworkTypeInUse", SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_TYPE")), ("Configuration", SimTypeRef("NDIS_802_11_CONFIGURATION", SimStruct)), ("InfrastructureMode", SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_INFRASTRUCTURE")), ("SupportedRates", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="NDIS_WLAN_BSSID", pack=False, align=None),
    "NDIS_WLAN_BSSID_EX": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("MacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Ssid", SimTypeRef("NDIS_802_11_SSID", SimStruct)), ("Privacy", SimTypeInt(signed=False, label="UInt32")), ("Rssi", SimTypeInt(signed=True, label="Int32")), ("NetworkTypeInUse", SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_TYPE")), ("Configuration", SimTypeRef("NDIS_802_11_CONFIGURATION", SimStruct)), ("InfrastructureMode", SimTypeInt(signed=False, label="NDIS_802_11_NETWORK_INFRASTRUCTURE")), ("SupportedRates", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("IELength", SimTypeInt(signed=False, label="UInt32")), ("IEs", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDIS_WLAN_BSSID_EX", pack=False, align=None),
    "NDIS_WMI_ENUM_ADAPTER": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("IfIndex", SimTypeInt(signed=False, label="UInt32")), ("NetLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("DeviceNameLength", SimTypeShort(signed=False, label="UInt16")), ("DeviceName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="NDIS_WMI_ENUM_ADAPTER", pack=False, align=None),
    "NDIS_WMI_EVENT_HEADER": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("IfIndex", SimTypeInt(signed=False, label="UInt32")), ("NetLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("RequestId", SimTypeLongLong(signed=False, label="UInt64")), ("PortNumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceNameLength", SimTypeInt(signed=False, label="UInt32")), ("DeviceNameOffset", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="NDIS_WMI_EVENT_HEADER", pack=False, align=None),
    "NDIS_WMI_IPSEC_OFFLOAD_V1": SimStruct(OrderedDict((("Supported", SimTypeRef("_Supported_e__Struct", SimStruct)), ("IPv4AH", SimTypeRef("_IPv4AH_e__Struct", SimStruct)), ("IPv4ESP", SimTypeRef("_IPv4ESP_e__Struct", SimStruct)),)), name="NDIS_WMI_IPSEC_OFFLOAD_V1", pack=False, align=None),
    "NDIS_WMI_METHOD_HEADER": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PortNumber", SimTypeInt(signed=False, label="UInt32")), ("NetLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("RequestId", SimTypeLongLong(signed=False, label="UInt64")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="NDIS_WMI_METHOD_HEADER", pack=False, align=None),
    "NDIS_WMI_OFFLOAD": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Checksum", SimTypeRef("NDIS_WMI_TCP_IP_CHECKSUM_OFFLOAD", SimStruct)), ("LsoV1", SimTypeRef("NDIS_WMI_TCP_LARGE_SEND_OFFLOAD_V1", SimStruct)), ("IPsecV1", SimTypeRef("NDIS_WMI_IPSEC_OFFLOAD_V1", SimStruct)), ("LsoV2", SimTypeRef("NDIS_WMI_TCP_LARGE_SEND_OFFLOAD_V2", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_WMI_OFFLOAD", pack=False, align=None),
    "NDIS_WMI_OUTPUT_INFO": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SupportedRevision", SimTypeChar(label="Byte")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_WMI_OUTPUT_INFO", pack=False, align=None),
    "NDIS_WMI_SET_HEADER": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("PortNumber", SimTypeInt(signed=False, label="UInt32")), ("NetLuid", SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None")), ("RequestId", SimTypeLongLong(signed=False, label="UInt64")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="NDIS_WMI_SET_HEADER", pack=False, align=None),
    "NDIS_WMI_TCP_CONNECTION_OFFLOAD": SimStruct(OrderedDict((("Header", SimTypeRef("NDIS_OBJECT_HEADER", SimStruct)), ("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("SupportIPv4", SimTypeInt(signed=False, label="UInt32")), ("SupportIPv6", SimTypeInt(signed=False, label="UInt32")), ("SupportIPv6ExtensionHeaders", SimTypeInt(signed=False, label="UInt32")), ("SupportSack", SimTypeInt(signed=False, label="UInt32")), ("TcpConnectionOffloadCapacity", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="NDIS_WMI_TCP_CONNECTION_OFFLOAD", pack=False, align=None),
    "NDIS_WMI_TCP_IP_CHECKSUM_OFFLOAD": SimStruct(OrderedDict((("IPv4Transmit", SimTypeRef("_IPv4Transmit_e__Struct", SimStruct)), ("IPv4Receive", SimTypeRef("_IPv4Receive_e__Struct", SimStruct)), ("IPv6Transmit", SimTypeRef("_IPv6Transmit_e__Struct", SimStruct)), ("IPv6Receive", SimTypeRef("_IPv6Receive_e__Struct", SimStruct)),)), name="NDIS_WMI_TCP_IP_CHECKSUM_OFFLOAD", pack=False, align=None),
    "NDIS_WMI_TCP_LARGE_SEND_OFFLOAD_V1": SimStruct(OrderedDict((("IPv4", SimTypeRef("_IPv4_e__Struct", SimStruct)),)), name="NDIS_WMI_TCP_LARGE_SEND_OFFLOAD_V1", pack=False, align=None),
    "NDIS_WMI_TCP_LARGE_SEND_OFFLOAD_V2": SimStruct(OrderedDict((("IPv4", SimTypeRef("_IPv4_e__Struct", SimStruct)), ("IPv6", SimTypeRef("_IPv6_e__Struct", SimStruct)),)), name="NDIS_WMI_TCP_LARGE_SEND_OFFLOAD_V2", pack=False, align=None),
    "NDIS_WORK_ITEM": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Routine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("WrapperReserved", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NDIS_WORK_ITEM", pack=False, align=None),
    "NDIS_WRAPPER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDK_ADAPTER_INFO": SimStruct(OrderedDict((("Version", SimTypeRef("NDK_VERSION", SimStruct)), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("MaxRegistrationSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaxWindowSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("FRMRPageCount", SimTypeInt(signed=False, label="UInt32")), ("MaxInitiatorRequestSge", SimTypeInt(signed=False, label="UInt32")), ("MaxReceiveRequestSge", SimTypeInt(signed=False, label="UInt32")), ("MaxReadRequestSge", SimTypeInt(signed=False, label="UInt32")), ("MaxTransferLength", SimTypeInt(signed=False, label="UInt32")), ("MaxInlineDataSize", SimTypeInt(signed=False, label="UInt32")), ("MaxInboundReadLimit", SimTypeInt(signed=False, label="UInt32")), ("MaxOutboundReadLimit", SimTypeInt(signed=False, label="UInt32")), ("MaxReceiveQueueDepth", SimTypeInt(signed=False, label="UInt32")), ("MaxInitiatorQueueDepth", SimTypeInt(signed=False, label="UInt32")), ("MaxSrqDepth", SimTypeInt(signed=False, label="UInt32")), ("MaxCqDepth", SimTypeInt(signed=False, label="UInt32")), ("LargeRequestThreshold", SimTypeInt(signed=False, label="UInt32")), ("MaxCallerData", SimTypeInt(signed=False, label="UInt32")), ("MaxCalleeData", SimTypeInt(signed=False, label="UInt32")), ("AdapterFlags", SimTypeInt(signed=False, label="UInt32")), ("RdmaTechnology", SimTypeInt(signed=False, label="NDK_RDMA_TECHNOLOGY")),)), name="NDK_ADAPTER_INFO", pack=False, align=None),
    "NDK_RDMA_TECHNOLOGY": SimTypeInt(signed=False, label="NDK_RDMA_TECHNOLOGY"),
    "NDK_VERSION": SimStruct(OrderedDict((("Major", SimTypeShort(signed=False, label="UInt16")), ("Minor", SimTypeShort(signed=False, label="UInt16")),)), name="NDK_VERSION", pack=False, align=None),
    "NDR64_ARRAY_ELEMENT_INFO": SimStruct(OrderedDict((("ElementMemSize", SimTypeInt(signed=False, label="UInt32")), ("Element", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_ARRAY_ELEMENT_INFO", pack=False, align=None),
    "NDR64_ARRAY_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_ARRAY_FLAGS", pack=False, align=None),
    "NDR64_BINDINGS": SimUnion({"Primitive": SimTypeRef("NDR64_BIND_PRIMITIVE", SimStruct), "Generic": SimTypeRef("NDR64_BIND_GENERIC", SimStruct), "Context": SimTypeRef("NDR64_BIND_CONTEXT", SimStruct)}, name="<anon>", label="None"),
    "NDR64_BIND_AND_NOTIFY_EXTENSION": SimStruct(OrderedDict((("Binding", SimTypeRef("NDR64_BIND_CONTEXT", SimStruct)), ("NotifyIndex", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_BIND_AND_NOTIFY_EXTENSION", pack=False, align=None),
    "NDR64_BIND_CONTEXT": SimStruct(OrderedDict((("HandleType", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("StackOffset", SimTypeShort(signed=False, label="UInt16")), ("RoutineIndex", SimTypeChar(label="Byte")), ("Ordinal", SimTypeChar(label="Byte")),)), name="NDR64_BIND_CONTEXT", pack=False, align=None),
    "NDR64_BIND_GENERIC": SimStruct(OrderedDict((("HandleType", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("StackOffset", SimTypeShort(signed=False, label="UInt16")), ("RoutineIndex", SimTypeChar(label="Byte")), ("Size", SimTypeChar(label="Byte")),)), name="NDR64_BIND_GENERIC", pack=False, align=None),
    "NDR64_BIND_PRIMITIVE": SimStruct(OrderedDict((("HandleType", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("StackOffset", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_BIND_PRIMITIVE", pack=False, align=None),
    "NDR64_BOGUS_ARRAY_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_ARRAY_FLAGS", SimStruct)), ("NumberDims", SimTypeChar(label="Byte")), ("NumberElements", SimTypeInt(signed=False, label="UInt32")), ("Element", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_BOGUS_ARRAY_HEADER_FORMAT", pack=False, align=None),
    "NDR64_BOGUS_STRUCTURE_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_STRUCTURE_FLAGS", SimStruct)), ("Reserve", SimTypeChar(label="Byte")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("OriginalMemberLayout", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OriginalPointerLayout", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PointerLayout", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_BOGUS_STRUCTURE_HEADER_FORMAT", pack=False, align=None),
    "NDR64_BUFFER_ALIGN_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_BUFFER_ALIGN_FORMAT", pack=False, align=None),
    "NDR64_CONFORMANT_STRING_FORMAT": SimStruct(OrderedDict((("Header", SimTypeRef("NDR64_STRING_HEADER_FORMAT", SimStruct)),)), name="NDR64_CONFORMANT_STRING_FORMAT", pack=False, align=None),
    "NDR64_CONF_ARRAY_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_ARRAY_FLAGS", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("ElementSize", SimTypeInt(signed=False, label="UInt32")), ("ConfDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_CONF_ARRAY_HEADER_FORMAT", pack=False, align=None),
    "NDR64_CONF_BOGUS_STRUCTURE_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_STRUCTURE_FLAGS", SimStruct)), ("Dimensions", SimTypeChar(label="Byte")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("OriginalMemberLayout", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OriginalPointerLayout", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PointerLayout", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ConfArrayDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_CONF_BOGUS_STRUCTURE_HEADER_FORMAT", pack=False, align=None),
    "NDR64_CONF_STRUCTURE_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_STRUCTURE_FLAGS", SimStruct)), ("Reserve", SimTypeChar(label="Byte")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("ArrayDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_CONF_STRUCTURE_HEADER_FORMAT", pack=False, align=None),
    "NDR64_CONF_VAR_ARRAY_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_ARRAY_FLAGS", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("ElementSize", SimTypeInt(signed=False, label="UInt32")), ("ConfDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("VarDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_CONF_VAR_ARRAY_HEADER_FORMAT", pack=False, align=None),
    "NDR64_CONF_VAR_BOGUS_ARRAY_HEADER_FORMAT": SimStruct(OrderedDict((("FixedArrayFormat", SimTypeRef("NDR64_BOGUS_ARRAY_HEADER_FORMAT", SimStruct)), ("ConfDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("VarDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OffsetDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_CONF_VAR_BOGUS_ARRAY_HEADER_FORMAT", pack=False, align=None),
    "NDR64_CONSTANT_IID_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Guid", SimTypeBottom(label="Guid")),)), name="NDR64_CONSTANT_IID_FORMAT", pack=False, align=None),
    "NDR64_CONTEXT_HANDLE_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_CONTEXT_HANDLE_FLAGS", pack=False, align=None),
    "NDR64_CONTEXT_HANDLE_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("ContextFlags", SimTypeChar(label="Byte")), ("RundownRoutineIndex", SimTypeChar(label="Byte")), ("Ordinal", SimTypeChar(label="Byte")),)), name="NDR64_CONTEXT_HANDLE_FORMAT", pack=False, align=None),
    "NDR64_EMBEDDED_COMPLEX_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Reserve1", SimTypeChar(label="Byte")), ("Reserve2", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_EMBEDDED_COMPLEX_FORMAT", pack=False, align=None),
    "NDR64_ENCAPSULATED_UNION": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("SwitchType", SimTypeChar(label="Byte")), ("MemoryOffset", SimTypeInt(signed=False, label="UInt32")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_ENCAPSULATED_UNION", pack=False, align=None),
    "NDR64_EXPR_CONST32": SimStruct(OrderedDict((("ExprType", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("ConstValue", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_EXPR_CONST32", pack=False, align=None),
    "NDR64_EXPR_CONST64": SimStruct(OrderedDict((("ExprType", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("ConstValue", SimTypeLongLong(signed=True, label="Int64")),)), name="NDR64_EXPR_CONST64", pack=False, align=None),
    "NDR64_EXPR_NOOP": SimStruct(OrderedDict((("ExprType", SimTypeChar(label="Byte")), ("Size", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_EXPR_NOOP", pack=False, align=None),
    "NDR64_EXPR_OPERATOR": SimStruct(OrderedDict((("ExprType", SimTypeChar(label="Byte")), ("Operator", SimTypeChar(label="Byte")), ("CastType", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="NDR64_EXPR_OPERATOR", pack=False, align=None),
    "NDR64_EXPR_VAR": SimStruct(OrderedDict((("ExprType", SimTypeChar(label="Byte")), ("VarType", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_EXPR_VAR", pack=False, align=None),
    "NDR64_FIXED_REPEAT_FORMAT": SimStruct(OrderedDict((("RepeatFormat", SimTypeRef("NDR64_REPEAT_FORMAT", SimStruct)), ("Iterations", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_FIXED_REPEAT_FORMAT", pack=False, align=None),
    "NDR64_FIX_ARRAY_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_ARRAY_FLAGS", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("TotalSize", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_FIX_ARRAY_HEADER_FORMAT", pack=False, align=None),
    "NDR64_IID_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_IID_FLAGS", pack=False, align=None),
    "NDR64_IID_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("IIDDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_IID_FORMAT", pack=False, align=None),
    "NDR64_MEMPAD_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Reserve1", SimTypeChar(label="Byte")), ("MemPad", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_MEMPAD_FORMAT", pack=False, align=None),
    "NDR64_NON_CONFORMANT_STRING_FORMAT": SimStruct(OrderedDict((("Header", SimTypeRef("NDR64_STRING_HEADER_FORMAT", SimStruct)), ("TotalSize", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_NON_CONFORMANT_STRING_FORMAT", pack=False, align=None),
    "NDR64_NON_ENCAPSULATED_UNION": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("SwitchType", SimTypeChar(label="Byte")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("Switch", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_NON_ENCAPSULATED_UNION", pack=False, align=None),
    "NDR64_NO_REPEAT_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_NO_REPEAT_FORMAT", pack=False, align=None),
    "NDR64_PARAM_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_PARAM_FLAGS", pack=False, align=None),
    "NDR64_PARAM_FORMAT": SimStruct(OrderedDict((("Type", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attributes", SimTypeRef("NDR64_PARAM_FLAGS", SimStruct)), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("StackOffset", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_PARAM_FORMAT", pack=False, align=None),
    "NDR64_PIPE_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_PIPE_FLAGS", pack=False, align=None),
    "NDR64_PIPE_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Type", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_PIPE_FORMAT", pack=False, align=None),
    "NDR64_POINTER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Pointee", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_POINTER_FORMAT", pack=False, align=None),
    "NDR64_POINTER_INSTANCE_HEADER_FORMAT": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_POINTER_INSTANCE_HEADER_FORMAT", pack=False, align=None),
    "NDR64_POINTER_REPEAT_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_POINTER_REPEAT_FLAGS", pack=False, align=None),
    "NDR64_PROC_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_PROC_FLAGS", pack=False, align=None),
    "NDR64_PROC_FORMAT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("StackSize", SimTypeInt(signed=False, label="UInt32")), ("ConstantClientBufferSize", SimTypeInt(signed=False, label="UInt32")), ("ConstantServerBufferSize", SimTypeInt(signed=False, label="UInt32")), ("RpcFlags", SimTypeShort(signed=False, label="UInt16")), ("FloatDoubleMask", SimTypeShort(signed=False, label="UInt16")), ("NumberOfParams", SimTypeShort(signed=False, label="UInt16")), ("ExtensionSize", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_PROC_FORMAT", pack=False, align=None),
    "NDR64_RANGED_STRING_FORMAT": SimStruct(OrderedDict((("Header", SimTypeRef("NDR64_STRING_HEADER_FORMAT", SimStruct)), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Min", SimTypeLongLong(signed=False, label="UInt64")), ("Max", SimTypeLongLong(signed=False, label="UInt64")),)), name="NDR64_RANGED_STRING_FORMAT", pack=False, align=None),
    "NDR64_RANGE_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("RangeType", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("MinValue", SimTypeLongLong(signed=True, label="Int64")), ("MaxValue", SimTypeLongLong(signed=True, label="Int64")),)), name="NDR64_RANGE_FORMAT", pack=False, align=None),
    "NDR64_RANGE_PIPE_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Type", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MemorySize", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("MinValue", SimTypeInt(signed=False, label="UInt32")), ("MaxValue", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_RANGE_PIPE_FORMAT", pack=False, align=None),
    "NDR64_REPEAT_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_POINTER_REPEAT_FLAGS", SimStruct)), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Increment", SimTypeInt(signed=False, label="UInt32")), ("OffsetToArray", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPointers", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_REPEAT_FORMAT", pack=False, align=None),
    "NDR64_RPC_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_RPC_FLAGS", pack=False, align=None),
    "NDR64_SIMPLE_MEMBER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_SIMPLE_MEMBER_FORMAT", pack=False, align=None),
    "NDR64_SIMPLE_REGION_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("RegionSize", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_SIMPLE_REGION_FORMAT", pack=False, align=None),
    "NDR64_SIZED_CONFORMANT_STRING_FORMAT": SimStruct(OrderedDict((("Header", SimTypeRef("NDR64_STRING_HEADER_FORMAT", SimStruct)), ("SizeDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_SIZED_CONFORMANT_STRING_FORMAT", pack=False, align=None),
    "NDR64_STRING_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_STRING_FLAGS", pack=False, align=None),
    "NDR64_STRING_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_STRING_FLAGS", SimStruct)), ("ElementSize", SimTypeShort(signed=False, label="UInt16")),)), name="NDR64_STRING_HEADER_FORMAT", pack=False, align=None),
    "NDR64_STRUCTURE_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_STRUCTURE_FLAGS", pack=False, align=None),
    "NDR64_STRUCTURE_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_STRUCTURE_FLAGS", SimStruct)), ("Reserve", SimTypeChar(label="Byte")), ("MemorySize", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_STRUCTURE_HEADER_FORMAT", pack=False, align=None),
    "NDR64_SYSTEM_HANDLE_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("HandleType", SimTypeChar(label="Byte")), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_SYSTEM_HANDLE_FORMAT", pack=False, align=None),
    "NDR64_TRANSMIT_AS_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_TRANSMIT_AS_FLAGS", pack=False, align=None),
    "NDR64_TRANSMIT_AS_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("RoutineIndex", SimTypeShort(signed=False, label="UInt16")), ("TransmittedTypeWireAlignment", SimTypeShort(signed=False, label="UInt16")), ("MemoryAlignment", SimTypeShort(signed=False, label="UInt16")), ("PresentedTypeMemorySize", SimTypeInt(signed=False, label="UInt32")), ("TransmittedTypeBufferSize", SimTypeInt(signed=False, label="UInt32")), ("TransmittedType", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_TRANSMIT_AS_FORMAT", pack=False, align=None),
    "NDR64_TYPE_STRICT_CONTEXT_HANDLE": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("RealFormatCode", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CtxtFlags", SimTypeInt(signed=False, label="UInt32")), ("CtxtID", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_TYPE_STRICT_CONTEXT_HANDLE", pack=False, align=None),
    "NDR64_UNION_ARM": SimStruct(OrderedDict((("CaseValue", SimTypeLongLong(signed=True, label="Int64")), ("Type", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_UNION_ARM", pack=False, align=None),
    "NDR64_UNION_ARM_SELECTOR": SimStruct(OrderedDict((("Reserved1", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("Arms", SimTypeInt(signed=False, label="UInt32")),)), name="NDR64_UNION_ARM_SELECTOR", pack=False, align=None),
    "NDR64_USER_MARSHAL_FLAGS": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NDR64_USER_MARSHAL_FLAGS", pack=False, align=None),
    "NDR64_USER_MARSHAL_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("RoutineIndex", SimTypeShort(signed=False, label="UInt16")), ("TransmittedTypeWireAlignment", SimTypeShort(signed=False, label="UInt16")), ("MemoryAlignment", SimTypeShort(signed=False, label="UInt16")), ("UserTypeMemorySize", SimTypeInt(signed=False, label="UInt32")), ("TransmittedTypeBufferSize", SimTypeInt(signed=False, label="UInt32")), ("TransmittedType", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_USER_MARSHAL_FORMAT", pack=False, align=None),
    "NDR64_VAR_ARRAY_HEADER_FORMAT": SimStruct(OrderedDict((("FormatCode", SimTypeChar(label="Byte")), ("Alignment", SimTypeChar(label="Byte")), ("Flags", SimTypeRef("NDR64_ARRAY_FLAGS", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("TotalSize", SimTypeInt(signed=False, label="UInt32")), ("ElementSize", SimTypeInt(signed=False, label="UInt32")), ("VarDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR64_VAR_ARRAY_HEADER_FORMAT", pack=False, align=None),
    "NDR_ALLOC_ALL_NODES_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDR_CS_ROUTINES": SimStruct(OrderedDict((("pSizeConvertRoutines", SimTypePointer(SimTypeRef("NDR_CS_SIZE_CONVERT_ROUTINES", SimStruct), offset=0)), ("pTagGettingRoutines", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "fServerSide", "pulSendingTag", "pulDesiredReceivingTag", "pulReceivingTag", "pStatus"]), offset=0), offset=0)),)), name="NDR_CS_ROUTINES", pack=False, align=None),
    "NDR_CS_SIZE_CONVERT_ROUTINES": SimStruct(OrderedDict((("pfnNetSize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="IDL_CS_CONVERT"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "ulLocalBufferSize", "conversionType", "pulNetworkBufferSize", "pStatus"]), offset=0)), ("pfnToNetCs", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "pLocalData", "ulLocalDataLength", "pNetworkData", "pulNetworkDataLength", "pStatus"]), offset=0)), ("pfnLocalSize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="IDL_CS_CONVERT"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "ulNetworkBufferSize", "conversionType", "pulLocalBufferSize", "pStatus"]), offset=0)), ("pfnFromNetCs", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hBinding", "ulNetworkCodeSet", "pNetworkData", "ulNetworkDataLength", "ulLocalBufferSize", "pLocalData", "pulLocalDataLength", "pStatus"]), offset=0)),)), name="NDR_CS_SIZE_CONVERT_ROUTINES", pack=False, align=None),
    "NDR_EXPR_DESC": SimStruct(OrderedDict((("pOffset", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("pFormatExpr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NDR_EXPR_DESC", pack=False, align=None),
    "NDR_NOTIFY2_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["flag"]), offset=0),
    "NDR_NOTIFY_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "NDR_POINTER_QUEUE_STATE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NDR_RUNDOWN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "NDR_SCONTEXT": SimStruct(OrderedDict((("pad", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("userContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NDR_SCONTEXT", pack=False, align=None),
    "NDR_USER_MARSHAL_INFO": SimStruct(OrderedDict((("InformationLevel", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Level1": SimTypeRef("NDR_USER_MARSHAL_INFO_LEVEL1", SimStruct)}, name="<anon>", label="None")),)), name="NDR_USER_MARSHAL_INFO", pack=False, align=None),
    "NDR_USER_MARSHAL_INFO_LEVEL1": SimStruct(OrderedDict((("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("pfnAllocate", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnFree", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pRpcChannelBuffer", SimTypeBottom(label="IRpcChannelBuffer")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 5)),)), name="NDR_USER_MARSHAL_INFO_LEVEL1", pack=False, align=None),
    "ND_NEIGHBOR_ADVERT_HEADER": SimStruct(OrderedDict((("nd_na_hdr", SimTypeRef("ICMP_MESSAGE", SimStruct)), ("nd_na_target", SimTypeRef("IN6_ADDR", SimStruct)),)), name="ND_NEIGHBOR_ADVERT_HEADER", pack=False, align=None),
    "ND_NEIGHBOR_SOLICIT_HEADER": SimStruct(OrderedDict((("nd_ns_hdr", SimTypeRef("ICMP_MESSAGE", SimStruct)), ("nd_ns_target", SimTypeRef("IN6_ADDR", SimStruct)),)), name="ND_NEIGHBOR_SOLICIT_HEADER", pack=False, align=None),
    "ND_OPTION_DNSSL": SimStruct(OrderedDict((("nd_opt_dnssl_type", SimTypeChar(label="Byte")), ("nd_opt_dnssl_len", SimTypeChar(label="Byte")), ("nd_opt_dnssl_reserved", SimTypeShort(signed=False, label="UInt16")), ("nd_opt_dnssl_lifetime", SimTypeInt(signed=False, label="UInt32")),)), name="ND_OPTION_DNSSL", pack=False, align=None),
    "ND_OPTION_HDR": SimStruct(OrderedDict((("nd_opt_type", SimTypeChar(label="Byte")), ("nd_opt_len", SimTypeChar(label="Byte")),)), name="ND_OPTION_HDR", pack=False, align=None),
    "ND_OPTION_MTU": SimStruct(OrderedDict((("nd_opt_mtu_type", SimTypeChar(label="Byte")), ("nd_opt_mtu_len", SimTypeChar(label="Byte")), ("nd_opt_mtu_reserved", SimTypeShort(signed=False, label="UInt16")), ("nd_opt_mtu_mtu", SimTypeInt(signed=False, label="UInt32")),)), name="ND_OPTION_MTU", pack=False, align=None),
    "ND_OPTION_PREFIX_INFO": SimStruct(OrderedDict((("nd_opt_pi_type", SimTypeChar(label="Byte")), ("nd_opt_pi_len", SimTypeChar(label="Byte")), ("nd_opt_pi_prefix_len", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"nd_opt_pi_flags_reserved": SimTypeChar(label="Byte"), "Flags": SimTypeRef("_Flags_e__Struct", SimStruct)}, name="<anon>", label="None")), ("nd_opt_pi_valid_time", SimTypeInt(signed=False, label="UInt32")), ("nd_opt_pi_preferred_time", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"nd_opt_pi_reserved2": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("nd_opt_pi_reserved3", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("nd_opt_pi_site_prefix_len", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("nd_opt_pi_prefix", SimTypeRef("IN6_ADDR", SimStruct)),)), name="ND_OPTION_PREFIX_INFO", pack=False, align=None),
    "ND_OPTION_RDNSS": SimStruct(OrderedDict((("nd_opt_rdnss_type", SimTypeChar(label="Byte")), ("nd_opt_rdnss_len", SimTypeChar(label="Byte")), ("nd_opt_rdnss_reserved", SimTypeShort(signed=False, label="UInt16")), ("nd_opt_rdnss_lifetime", SimTypeInt(signed=False, label="UInt32")),)), name="ND_OPTION_RDNSS", pack=False, align=None),
    "ND_OPTION_RD_HDR": SimStruct(OrderedDict((("nd_opt_rh_type", SimTypeChar(label="Byte")), ("nd_opt_rh_len", SimTypeChar(label="Byte")), ("nd_opt_rh_reserved1", SimTypeShort(signed=False, label="UInt16")), ("nd_opt_rh_reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="ND_OPTION_RD_HDR", pack=False, align=None),
    "ND_OPTION_ROUTE_INFO": SimStruct(OrderedDict((("nd_opt_ri_type", SimTypeChar(label="Byte")), ("nd_opt_ri_len", SimTypeChar(label="Byte")), ("nd_opt_ri_prefix_len", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"nd_opt_ri_flags_reserved": SimTypeChar(label="Byte"), "Flags": SimTypeRef("_Flags_e__Struct", SimStruct)}, name="<anon>", label="None")), ("nd_opt_ri_route_lifetime", SimTypeInt(signed=False, label="UInt32")), ("nd_opt_ri_prefix", SimTypeRef("IN6_ADDR", SimStruct)),)), name="ND_OPTION_ROUTE_INFO", pack=False, align=None),
    "ND_OPTION_TYPE": SimTypeInt(signed=False, label="ND_OPTION_TYPE"),
    "ND_REDIRECT_HEADER": SimStruct(OrderedDict((("nd_rd_hdr", SimTypeRef("ICMP_MESSAGE", SimStruct)), ("nd_rd_target", SimTypeRef("IN6_ADDR", SimStruct)), ("nd_rd_dst", SimTypeRef("IN6_ADDR", SimStruct)),)), name="ND_REDIRECT_HEADER", pack=False, align=None),
    "ND_ROUTER_ADVERT_HEADER": SimStruct(OrderedDict((("nd_ra_hdr", SimTypeRef("ICMP_MESSAGE", SimStruct)), ("nd_ra_reachable", SimTypeInt(signed=False, label="UInt32")), ("nd_ra_retransmit", SimTypeInt(signed=False, label="UInt32")),)), name="ND_ROUTER_ADVERT_HEADER", pack=False, align=None),
    "ND_ROUTER_SOLICIT_HEADER": SimStruct(OrderedDict((("nd_rs_hdr", SimTypeRef("ICMP_MESSAGE", SimStruct)),)), name="ND_ROUTER_SOLICIT_HEADER", pack=False, align=None),
    "NEARPROC": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0),
    "NEGOTIATE_CALLER_NAME_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)),)), name="NEGOTIATE_CALLER_NAME_REQUEST", pack=False, align=None),
    "NEGOTIATE_CALLER_NAME_RESPONSE": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="UInt32")), ("CallerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NEGOTIATE_CALLER_NAME_RESPONSE", pack=False, align=None),
    "NEGOTIATE_MESSAGES": SimTypeInt(signed=False, label="NEGOTIATE_MESSAGES"),
    "NEGOTIATE_PACKAGE_PREFIX": SimStruct(OrderedDict((("PackageId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PackageDataA", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PackageDataW", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PrefixLen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Prefix", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NEGOTIATE_PACKAGE_PREFIX", pack=False, align=None),
    "NEGOTIATE_PACKAGE_PREFIXES": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="UInt32")), ("PrefixCount", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Pad", SimTypeInt(signed=False, label="UInt32")),)), name="NEGOTIATE_PACKAGE_PREFIXES", pack=False, align=None),
    "NETCONMGR_ENUM_FLAGS": SimTypeInt(signed=False, label="NETCONMGR_ENUM_FLAGS"),
    "NETCONNECTINFOSTRUCT": SimStruct(OrderedDict((("cbStructure", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwDelay", SimTypeInt(signed=False, label="UInt32")), ("dwOptDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="NETCONNECTINFOSTRUCT", pack=False, align=None),
    "NETCONUI_CONNECT_FLAGS": SimTypeInt(signed=False, label="NETCONUI_CONNECT_FLAGS"),
    "NETCON_CHARACTERISTIC_FLAGS": SimTypeInt(signed=False, label="NETCON_CHARACTERISTIC_FLAGS"),
    "NETCON_MEDIATYPE": SimTypeInt(signed=False, label="NETCON_MEDIATYPE"),
    "NETCON_PROPERTIES": SimStruct(OrderedDict((("guidId", SimTypeBottom(label="Guid")), ("pszwName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszwDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Status", SimTypeInt(signed=False, label="NETCON_STATUS")), ("MediaType", SimTypeInt(signed=False, label="NETCON_MEDIATYPE")), ("dwCharacter", SimTypeInt(signed=False, label="UInt32")), ("clsidThisObject", SimTypeBottom(label="Guid")), ("clsidUiObject", SimTypeBottom(label="Guid")),)), name="NETCON_PROPERTIES", pack=False, align=None),
    "NETCON_STATUS": SimTypeInt(signed=False, label="NETCON_STATUS"),
    "NETCON_TYPE": SimTypeInt(signed=False, label="NETCON_TYPE"),
    "NETINFOSTRUCT": SimStruct(OrderedDict((("cbStructure", SimTypeInt(signed=False, label="UInt32")), ("dwProviderVersion", SimTypeInt(signed=False, label="UInt32")), ("dwStatus", SimTypeInt(signed=False, label="WIN32_ERROR")), ("dwCharacteristics", SimTypeInt(signed=False, label="NETINFOSTRUCT_CHARACTERISTICS")), ("dwHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("wNetType", SimTypeShort(signed=False, label="UInt16")), ("dwPrinters", SimTypeInt(signed=False, label="UInt32")), ("dwDrives", SimTypeInt(signed=False, label="UInt32")),)), name="NETINFOSTRUCT", pack=False, align=None),
    "NETINFOSTRUCT_CHARACTERISTICS": SimTypeInt(signed=False, label="NETINFOSTRUCT_CHARACTERISTICS"),
    "NETISO_ERROR_TYPE": SimTypeInt(signed=False, label="NETISO_ERROR_TYPE"),
    "NETISO_FLAG": SimTypeInt(signed=False, label="NETISO_FLAG"),
    "NETLOGON_GENERIC_INFO": SimStruct(OrderedDict((("Identity", SimTypeRef("NETLOGON_LOGON_IDENTITY_INFO", SimStruct)), ("PackageName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("LogonData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NETLOGON_GENERIC_INFO", pack=False, align=None),
    "NETLOGON_INFO_1": SimStruct(OrderedDict((("netlog1_flags", SimTypeInt(signed=False, label="UInt32")), ("netlog1_pdc_connection_status", SimTypeInt(signed=False, label="UInt32")),)), name="NETLOGON_INFO_1", pack=False, align=None),
    "NETLOGON_INFO_2": SimStruct(OrderedDict((("netlog2_flags", SimTypeInt(signed=False, label="UInt32")), ("netlog2_pdc_connection_status", SimTypeInt(signed=False, label="UInt32")), ("netlog2_trusted_dc_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("netlog2_tc_connection_status", SimTypeInt(signed=False, label="UInt32")),)), name="NETLOGON_INFO_2", pack=False, align=None),
    "NETLOGON_INFO_3": SimStruct(OrderedDict((("netlog3_flags", SimTypeInt(signed=False, label="UInt32")), ("netlog3_logon_attempts", SimTypeInt(signed=False, label="UInt32")), ("netlog3_reserved1", SimTypeInt(signed=False, label="UInt32")), ("netlog3_reserved2", SimTypeInt(signed=False, label="UInt32")), ("netlog3_reserved3", SimTypeInt(signed=False, label="UInt32")), ("netlog3_reserved4", SimTypeInt(signed=False, label="UInt32")), ("netlog3_reserved5", SimTypeInt(signed=False, label="UInt32")),)), name="NETLOGON_INFO_3", pack=False, align=None),
    "NETLOGON_INFO_4": SimStruct(OrderedDict((("netlog4_trusted_dc_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("netlog4_trusted_domain_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NETLOGON_INFO_4", pack=False, align=None),
    "NETLOGON_INTERACTIVE_INFO": SimStruct(OrderedDict((("Identity", SimTypeRef("NETLOGON_LOGON_IDENTITY_INFO", SimStruct)), ("LmOwfPassword", SimTypeRef("LM_OWF_PASSWORD", SimStruct)), ("NtOwfPassword", SimTypeRef("LM_OWF_PASSWORD", SimStruct)),)), name="NETLOGON_INTERACTIVE_INFO", pack=False, align=None),
    "NETLOGON_LOGON_IDENTITY_INFO": SimStruct(OrderedDict((("LogonDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ParameterControl", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeLongLong(signed=True, label="Int64")), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Workstation", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="NETLOGON_LOGON_IDENTITY_INFO", pack=False, align=None),
    "NETLOGON_LOGON_INFO_CLASS": SimTypeInt(signed=False, label="NETLOGON_LOGON_INFO_CLASS"),
    "NETLOGON_NETWORK_INFO": SimStruct(OrderedDict((("Identity", SimTypeRef("NETLOGON_LOGON_IDENTITY_INFO", SimStruct)), ("LmChallenge", SimTypeRef("CLEAR_BLOCK", SimStruct)), ("NtChallengeResponse", SimTypeRef("LSA_STRING", SimStruct)), ("LmChallengeResponse", SimTypeRef("LSA_STRING", SimStruct)),)), name="NETLOGON_NETWORK_INFO", pack=False, align=None),
    "NETLOGON_SERVICE_INFO": SimStruct(OrderedDict((("Identity", SimTypeRef("NETLOGON_LOGON_IDENTITY_INFO", SimStruct)), ("LmOwfPassword", SimTypeRef("LM_OWF_PASSWORD", SimStruct)), ("NtOwfPassword", SimTypeRef("LM_OWF_PASSWORD", SimStruct)),)), name="NETLOGON_SERVICE_INFO", pack=False, align=None),
    "NETRESOURCE2A": SimStruct(OrderedDict((("dwScope", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwUsage", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayType", SimTypeInt(signed=False, label="UInt32")), ("lpLocalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpRemoteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ns_info", SimTypeRef("NS_INFOA", SimStruct)), ("ServiceType", SimTypeBottom(label="Guid")), ("dwProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpiProtocols", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="NETRESOURCE2A", pack=False, align=None),
    "NETRESOURCE2W": SimStruct(OrderedDict((("dwScope", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwUsage", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayType", SimTypeInt(signed=False, label="UInt32")), ("lpLocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpRemoteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ns_info", SimTypeRef("NS_INFOA", SimStruct)), ("ServiceType", SimTypeBottom(label="Guid")), ("dwProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpiProtocols", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="NETRESOURCE2W", pack=False, align=None),
    "NETRESOURCEA": SimStruct(OrderedDict((("dwScope", SimTypeInt(signed=False, label="NET_RESOURCE_SCOPE")), ("dwType", SimTypeInt(signed=False, label="NET_RESOURCE_TYPE")), ("dwDisplayType", SimTypeInt(signed=False, label="UInt32")), ("dwUsage", SimTypeInt(signed=False, label="UInt32")), ("lpLocalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpRemoteName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpProvider", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NETRESOURCEA", pack=False, align=None),
    "NETRESOURCEW": SimStruct(OrderedDict((("dwScope", SimTypeInt(signed=False, label="NET_RESOURCE_SCOPE")), ("dwType", SimTypeInt(signed=False, label="NET_RESOURCE_TYPE")), ("dwDisplayType", SimTypeInt(signed=False, label="UInt32")), ("dwUsage", SimTypeInt(signed=False, label="UInt32")), ("lpLocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpRemoteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NETRESOURCEW", pack=False, align=None),
    "NETSETUP_JOIN_STATUS": SimTypeInt(signed=False, label="NETSETUP_JOIN_STATUS"),
    "NETSETUP_NAME_TYPE": SimTypeInt(signed=False, label="NETSETUP_NAME_TYPE"),
    "NETSETUP_PROVISION": SimTypeInt(signed=False, label="NETSETUP_PROVISION"),
    "NETSETUP_PROVISIONING_PARAMS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpMachineAccountOU", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDcName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwProvisionOptions", SimTypeInt(signed=False, label="NETSETUP_PROVISION")), ("aCertTemplateNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("cCertTemplateNames", SimTypeInt(signed=False, label="UInt32")), ("aMachinePolicyNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("cMachinePolicyNames", SimTypeInt(signed=False, label="UInt32")), ("aMachinePolicyPaths", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("cMachinePolicyPaths", SimTypeInt(signed=False, label="UInt32")), ("lpNetbiosName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpSiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpPrimaryDNSDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NETSETUP_PROVISIONING_PARAMS", pack=False, align=None),
    "NETSOURCE_URLCREDPOLICY_SETTINGS": SimTypeInt(signed=False, label="NETSOURCE_URLCREDPOLICY_SETTINGS"),
    "NETWORK_ADDRESS": SimStruct(OrderedDict((("AddressLength", SimTypeShort(signed=False, label="UInt16")), ("AddressType", SimTypeShort(signed=False, label="UInt16")), ("Address", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NETWORK_ADDRESS", pack=False, align=None),
    "NETWORK_ADDRESS_IP": SimStruct(OrderedDict((("sin_port", SimTypeShort(signed=False, label="UInt16")), ("IN_ADDR", SimTypeInt(signed=False, label="UInt32")), ("sin_zero", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="NETWORK_ADDRESS_IP", pack=False, align=None),
    "NETWORK_ADDRESS_IP6": SimStruct(OrderedDict((("sin6_port", SimTypeShort(signed=False, label="UInt16")), ("sin6_flowinfo", SimTypeInt(signed=False, label="UInt32")), ("sin6_addr", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)), ("sin6_scope_id", SimTypeInt(signed=False, label="UInt32")),)), name="NETWORK_ADDRESS_IP6", pack=False, align=None),
    "NETWORK_ADDRESS_IPX": SimStruct(OrderedDict((("NetworkAddress", SimTypeInt(signed=False, label="UInt32")), ("NodeAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Socket", SimTypeShort(signed=False, label="UInt16")),)), name="NETWORK_ADDRESS_IPX", pack=False, align=None),
    "NETWORK_ADDRESS_LIST": SimStruct(OrderedDict((("AddressCount", SimTypeInt(signed=True, label="Int32")), ("AddressType", SimTypeShort(signed=False, label="UInt16")), ("Address", SimTypePointer(SimTypeRef("NETWORK_ADDRESS", SimStruct), offset=0)),)), name="NETWORK_ADDRESS_LIST", pack=False, align=None),
    "NETWORK_APP_INSTANCE_EA": SimStruct(OrderedDict((("AppInstanceID", SimTypeBottom(label="Guid")), ("CsvFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NETWORK_APP_INSTANCE_EA", pack=False, align=None),
    "NETWORK_APP_INSTANCE_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("AppInstanceID", SimTypeBottom(label="Guid")),)), name="NETWORK_APP_INSTANCE_ECP_CONTEXT", pack=False, align=None),
    "NETWORK_APP_INSTANCE_VERSION_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("VersionHigh", SimTypeLongLong(signed=False, label="UInt64")), ("VersionLow", SimTypeLongLong(signed=False, label="UInt64")),)), name="NETWORK_APP_INSTANCE_VERSION_ECP_CONTEXT", pack=False, align=None),
    "NETWORK_INSTALL_TIME": SimTypeInt(signed=False, label="NETWORK_INSTALL_TIME"),
    "NETWORK_NAME": SimStruct(OrderedDict((("Name", SimTypeRef("FLAT_STRING", SimStruct)),)), name="NETWORK_NAME", pack=False, align=None),
    "NETWORK_NAME_FORMAT_FLAGS": SimTypeInt(signed=False, label="NETWORK_NAME_FORMAT_FLAGS"),
    "NETWORK_OPEN_ECP_CONTEXT": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimStruct(OrderedDict((("in", SimTypeRef("_in_e__Struct", SimStruct)), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="_Anonymous_e__Struct", pack=False, align=None)),)), name="NETWORK_OPEN_ECP_CONTEXT", pack=False, align=None),
    "NETWORK_OPEN_ECP_CONTEXT_V0": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimStruct(OrderedDict((("in", SimTypeRef("_in_e__Struct", SimStruct)), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="_Anonymous_e__Struct", pack=False, align=None)),)), name="NETWORK_OPEN_ECP_CONTEXT_V0", pack=False, align=None),
    "NETWORK_OPEN_INTEGRITY_QUALIFIER": SimTypeInt(signed=False, label="NETWORK_OPEN_INTEGRITY_QUALIFIER"),
    "NETWORK_OPEN_LOCATION_QUALIFIER": SimTypeInt(signed=False, label="NETWORK_OPEN_LOCATION_QUALIFIER"),
    "NETWORK_UPGRADE_TYPE": SimTypeInt(signed=False, label="NETWORK_UPGRADE_TYPE"),
    "NET_ADDRESS_FORMAT": SimTypeInt(signed=False, label="NET_ADDRESS_FORMAT"),
    "NET_ADDRESS_INFO": SimStruct(OrderedDict((("Format", SimTypeInt(signed=False, label="NET_ADDRESS_FORMAT")), ("Anonymous", SimUnion({"NamedAddress": SimTypeRef("_NamedAddress_e__Struct", SimStruct), "Ipv4Address": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6Address": SimTypeRef("SOCKADDR_IN6", SimStruct), "IpAddress": SimTypeRef("SOCKADDR", SimStruct)}, name="<anon>", label="None")),)), name="NET_ADDRESS_INFO", pack=False, align=None),
    "NET_COMPUTER_NAME_TYPE": SimTypeInt(signed=False, label="NET_COMPUTER_NAME_TYPE"),
    "NET_DISPLAY_GROUP": SimStruct(OrderedDict((("grpi3_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi3_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grpi3_group_id", SimTypeInt(signed=False, label="UInt32")), ("grpi3_attributes", SimTypeInt(signed=False, label="UInt32")), ("grpi3_next_index", SimTypeInt(signed=False, label="UInt32")),)), name="NET_DISPLAY_GROUP", pack=False, align=None),
    "NET_DISPLAY_MACHINE": SimStruct(OrderedDict((("usri2_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri2_user_id", SimTypeInt(signed=False, label="UInt32")), ("usri2_next_index", SimTypeInt(signed=False, label="UInt32")),)), name="NET_DISPLAY_MACHINE", pack=False, align=None),
    "NET_DISPLAY_USER": SimStruct(OrderedDict((("usri1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri1_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_user_id", SimTypeInt(signed=False, label="UInt32")), ("usri1_next_index", SimTypeInt(signed=False, label="UInt32")),)), name="NET_DISPLAY_USER", pack=False, align=None),
    "NET_FW_ACTION": SimTypeInt(signed=False, label="NET_FW_ACTION"),
    "NET_FW_AUTHENTICATE_TYPE": SimTypeInt(signed=False, label="NET_FW_AUTHENTICATE_TYPE"),
    "NET_FW_EDGE_TRAVERSAL_TYPE": SimTypeInt(signed=False, label="NET_FW_EDGE_TRAVERSAL_TYPE"),
    "NET_FW_IP_PROTOCOL": SimTypeInt(signed=False, label="NET_FW_IP_PROTOCOL"),
    "NET_FW_IP_VERSION": SimTypeInt(signed=False, label="NET_FW_IP_VERSION"),
    "NET_FW_MODIFY_STATE": SimTypeInt(signed=False, label="NET_FW_MODIFY_STATE"),
    "NET_FW_POLICY_TYPE": SimTypeInt(signed=False, label="NET_FW_POLICY_TYPE"),
    "NET_FW_PROFILE_TYPE": SimTypeInt(signed=False, label="NET_FW_PROFILE_TYPE"),
    "NET_FW_PROFILE_TYPE2": SimTypeInt(signed=False, label="NET_FW_PROFILE_TYPE2"),
    "NET_FW_RULE_CATEGORY": SimTypeInt(signed=False, label="NET_FW_RULE_CATEGORY"),
    "NET_FW_RULE_DIRECTION": SimTypeInt(signed=False, label="NET_FW_RULE_DIRECTION"),
    "NET_FW_SCOPE": SimTypeInt(signed=False, label="NET_FW_SCOPE"),
    "NET_FW_SERVICE_TYPE": SimTypeInt(signed=False, label="NET_FW_SERVICE_TYPE"),
    "NET_IF_ACCESS_TYPE": SimTypeInt(signed=False, label="NET_IF_ACCESS_TYPE"),
    "NET_IF_ADMIN_STATUS": SimTypeInt(signed=False, label="NET_IF_ADMIN_STATUS"),
    "NET_IF_ALIAS_LH": SimStruct(OrderedDict((("ifAliasLength", SimTypeShort(signed=False, label="UInt16")), ("ifAliasOffset", SimTypeShort(signed=False, label="UInt16")),)), name="NET_IF_ALIAS_LH", pack=False, align=None),
    "NET_IF_CONNECTION_TYPE": SimTypeInt(signed=False, label="NET_IF_CONNECTION_TYPE"),
    "NET_IF_DIRECTION_TYPE": SimTypeInt(signed=False, label="NET_IF_DIRECTION_TYPE"),
    "NET_IF_MEDIA_CONNECT_STATE": SimTypeInt(signed=False, label="NET_IF_MEDIA_CONNECT_STATE"),
    "NET_IF_MEDIA_DUPLEX_STATE": SimTypeInt(signed=False, label="NET_IF_MEDIA_DUPLEX_STATE"),
    "NET_IF_OPER_STATUS": SimTypeInt(signed=False, label="NET_IF_OPER_STATUS"),
    "NET_IF_RCV_ADDRESS_LH": SimStruct(OrderedDict((("ifRcvAddressType", SimTypeInt(signed=False, label="NET_IF_RCV_ADDRESS_TYPE")), ("ifRcvAddressLength", SimTypeShort(signed=False, label="UInt16")), ("ifRcvAddressOffset", SimTypeShort(signed=False, label="UInt16")),)), name="NET_IF_RCV_ADDRESS_LH", pack=False, align=None),
    "NET_IF_RCV_ADDRESS_TYPE": SimTypeInt(signed=False, label="NET_IF_RCV_ADDRESS_TYPE"),
    "NET_INTERFACE_CONTEXT": SimStruct(OrderedDict((("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("ConfigurationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NET_INTERFACE_CONTEXT", pack=False, align=None),
    "NET_INTERFACE_CONTEXT_TABLE": SimStruct(OrderedDict((("InterfaceContextHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NumberOfEntries", SimTypeInt(signed=False, label="UInt32")), ("InterfaceContextArray", SimTypePointer(SimTypeRef("NET_INTERFACE_CONTEXT", SimStruct), offset=0)),)), name="NET_INTERFACE_CONTEXT_TABLE", pack=False, align=None),
    "NET_JOIN_DOMAIN_JOIN_OPTIONS": SimTypeInt(signed=False, label="NET_JOIN_DOMAIN_JOIN_OPTIONS"),
    "NET_LUID_LH": SimUnion({"Value": SimTypeLongLong(signed=False, label="UInt64"), "Info": SimTypeRef("_Info_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "NET_PHYSICAL_LOCATION_LH": SimStruct(OrderedDict((("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("SlotNumber", SimTypeInt(signed=False, label="UInt32")), ("FunctionNumber", SimTypeInt(signed=False, label="UInt32")),)), name="NET_PHYSICAL_LOCATION_LH", pack=False, align=None),
    "NET_REMOTE_COMPUTER_SUPPORTS_OPTIONS": SimTypeInt(signed=False, label="NET_REMOTE_COMPUTER_SUPPORTS_OPTIONS"),
    "NET_REQUEST_PROVISION_OPTIONS": SimTypeInt(signed=False, label="NET_REQUEST_PROVISION_OPTIONS"),
    "NET_RESOURCE_SCOPE": SimTypeInt(signed=False, label="NET_RESOURCE_SCOPE"),
    "NET_RESOURCE_TYPE": SimTypeInt(signed=False, label="NET_RESOURCE_TYPE"),
    "NET_SERVER_TYPE": SimTypeInt(signed=False, label="NET_SERVER_TYPE"),
    "NET_USER_ENUM_FILTER_FLAGS": SimTypeInt(signed=False, label="NET_USER_ENUM_FILTER_FLAGS"),
    "NET_USE_CONNECT_FLAGS": SimTypeInt(signed=False, label="NET_USE_CONNECT_FLAGS"),
    "NET_VALIDATE_AUTHENTICATION_INPUT_ARG": SimStruct(OrderedDict((("InputPersistedFields", SimTypeRef("NET_VALIDATE_PERSISTED_FIELDS", SimStruct)), ("PasswordMatched", SimTypeChar(label="Byte")),)), name="NET_VALIDATE_AUTHENTICATION_INPUT_ARG", pack=False, align=None),
    "NET_VALIDATE_OUTPUT_ARG": SimStruct(OrderedDict((("ChangedPersistedFields", SimTypeRef("NET_VALIDATE_PERSISTED_FIELDS", SimStruct)), ("ValidationStatus", SimTypeInt(signed=False, label="UInt32")),)), name="NET_VALIDATE_OUTPUT_ARG", pack=False, align=None),
    "NET_VALIDATE_PASSWORD_CHANGE_INPUT_ARG": SimStruct(OrderedDict((("InputPersistedFields", SimTypeRef("NET_VALIDATE_PERSISTED_FIELDS", SimStruct)), ("ClearPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UserAccountName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HashedPassword", SimTypeRef("NET_VALIDATE_PASSWORD_HASH", SimStruct)), ("PasswordMatch", SimTypeChar(label="Byte")),)), name="NET_VALIDATE_PASSWORD_CHANGE_INPUT_ARG", pack=False, align=None),
    "NET_VALIDATE_PASSWORD_HASH": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Hash", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NET_VALIDATE_PASSWORD_HASH", pack=False, align=None),
    "NET_VALIDATE_PASSWORD_RESET_INPUT_ARG": SimStruct(OrderedDict((("InputPersistedFields", SimTypeRef("NET_VALIDATE_PERSISTED_FIELDS", SimStruct)), ("ClearPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UserAccountName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HashedPassword", SimTypeRef("NET_VALIDATE_PASSWORD_HASH", SimStruct)), ("PasswordMustChangeAtNextLogon", SimTypeChar(label="Byte")), ("ClearLockout", SimTypeChar(label="Byte")),)), name="NET_VALIDATE_PASSWORD_RESET_INPUT_ARG", pack=False, align=None),
    "NET_VALIDATE_PASSWORD_TYPE": SimTypeInt(signed=False, label="NET_VALIDATE_PASSWORD_TYPE"),
    "NET_VALIDATE_PERSISTED_FIELDS": SimStruct(OrderedDict((("PresentFields", SimTypeInt(signed=False, label="UInt32")), ("PasswordLastSet", SimTypeRef("FILETIME", SimStruct)), ("BadPasswordTime", SimTypeRef("FILETIME", SimStruct)), ("LockoutTime", SimTypeRef("FILETIME", SimStruct)), ("BadPasswordCount", SimTypeInt(signed=False, label="UInt32")), ("PasswordHistoryLength", SimTypeInt(signed=False, label="UInt32")), ("PasswordHistory", SimTypePointer(SimTypeRef("NET_VALIDATE_PASSWORD_HASH", SimStruct), offset=0)),)), name="NET_VALIDATE_PERSISTED_FIELDS", pack=False, align=None),
    "NEWCPLINFOA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwHelpContext", SimTypeInt(signed=False, label="UInt32")), ("lData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szInfo", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szHelpFile", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="NEWCPLINFOA", pack=False, align=None),
    "NEWCPLINFOW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwHelpContext", SimTypeInt(signed=False, label="UInt32")), ("lData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szInfo", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szHelpFile", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="NEWCPLINFOW", pack=False, align=None),
    "NEWMAIL_NOTIFICATION": SimStruct(OrderedDict((("cbEntryID", SimTypeInt(signed=False, label="UInt32")), ("lpEntryID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("cbParentID", SimTypeInt(signed=False, label="UInt32")), ("lpParentID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpszMessageClass", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("ulMessageFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NEWMAIL_NOTIFICATION", pack=False, align=None),
    "NEWTEXTMETRICA": SimStruct(OrderedDict((("tmHeight", SimTypeInt(signed=True, label="Int32")), ("tmAscent", SimTypeInt(signed=True, label="Int32")), ("tmDescent", SimTypeInt(signed=True, label="Int32")), ("tmInternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmExternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmAveCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmMaxCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmWeight", SimTypeInt(signed=True, label="Int32")), ("tmOverhang", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectX", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectY", SimTypeInt(signed=True, label="Int32")), ("tmFirstChar", SimTypeChar(label="Byte")), ("tmLastChar", SimTypeChar(label="Byte")), ("tmDefaultChar", SimTypeChar(label="Byte")), ("tmBreakChar", SimTypeChar(label="Byte")), ("tmItalic", SimTypeChar(label="Byte")), ("tmUnderlined", SimTypeChar(label="Byte")), ("tmStruckOut", SimTypeChar(label="Byte")), ("tmPitchAndFamily", SimTypeInt(signed=False, label="TMPF_FLAGS")), ("tmCharSet", SimTypeChar(label="Byte")), ("ntmFlags", SimTypeInt(signed=False, label="UInt32")), ("ntmSizeEM", SimTypeInt(signed=False, label="UInt32")), ("ntmCellHeight", SimTypeInt(signed=False, label="UInt32")), ("ntmAvgWidth", SimTypeInt(signed=False, label="UInt32")),)), name="NEWTEXTMETRICA", pack=False, align=None),
    "NEWTEXTMETRICEXA": SimStruct(OrderedDict((("ntmTm", SimTypeRef("NEWTEXTMETRICA", SimStruct)), ("ntmFontSig", SimTypeRef("FONTSIGNATURE", SimStruct)),)), name="NEWTEXTMETRICEXA", pack=False, align=None),
    "NEWTEXTMETRICEXW": SimStruct(OrderedDict((("ntmTm", SimTypeRef("NEWTEXTMETRICW", SimStruct)), ("ntmFontSig", SimTypeRef("FONTSIGNATURE", SimStruct)),)), name="NEWTEXTMETRICEXW", pack=False, align=None),
    "NEWTEXTMETRICW": SimStruct(OrderedDict((("tmHeight", SimTypeInt(signed=True, label="Int32")), ("tmAscent", SimTypeInt(signed=True, label="Int32")), ("tmDescent", SimTypeInt(signed=True, label="Int32")), ("tmInternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmExternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmAveCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmMaxCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmWeight", SimTypeInt(signed=True, label="Int32")), ("tmOverhang", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectX", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectY", SimTypeInt(signed=True, label="Int32")), ("tmFirstChar", SimTypeChar(label="Char")), ("tmLastChar", SimTypeChar(label="Char")), ("tmDefaultChar", SimTypeChar(label="Char")), ("tmBreakChar", SimTypeChar(label="Char")), ("tmItalic", SimTypeChar(label="Byte")), ("tmUnderlined", SimTypeChar(label="Byte")), ("tmStruckOut", SimTypeChar(label="Byte")), ("tmPitchAndFamily", SimTypeInt(signed=False, label="TMPF_FLAGS")), ("tmCharSet", SimTypeChar(label="Byte")), ("ntmFlags", SimTypeInt(signed=False, label="UInt32")), ("ntmSizeEM", SimTypeInt(signed=False, label="UInt32")), ("ntmCellHeight", SimTypeInt(signed=False, label="UInt32")), ("ntmAvgWidth", SimTypeInt(signed=False, label="UInt32")),)), name="NEWTEXTMETRICW", pack=False, align=None),
    "NEW_EVENT_DATA_AVAILABLE": SimStruct(OrderedDict((("cbNewEventDataAvailable", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")),)), name="NEW_EVENT_DATA_AVAILABLE", pack=False, align=None),
    "NEW_LOG_FILE_FREQUENCY": SimTypeInt(signed=False, label="NEW_LOG_FILE_FREQUENCY"),
    "NFS_OPEN_ECP_CONTEXT": SimStruct(OrderedDict((("ExportAlias", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("ClientSocketAddress", SimTypePointer(SimTypeRef("SOCKADDR_STORAGE", SimStruct), offset=0)),)), name="NFS_OPEN_ECP_CONTEXT", pack=False, align=None),
    "NLA_BLOB": SimStruct(OrderedDict((("header", SimTypeRef("_header_e__Struct", SimStruct)), ("data", SimUnion({"rawData": SimTypePointer(SimTypeChar(label="SByte"), offset=0), "interfaceData": SimTypeRef("_interfaceData_e__Struct", SimStruct), "locationData": SimTypeRef("_locationData_e__Struct", SimStruct), "connectivity": SimTypeRef("_connectivity_e__Struct", SimStruct), "ICS": SimTypeRef("_ICS_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="NLA_BLOB", pack=False, align=None),
    "NLA_BLOB_DATA_TYPE": SimTypeInt(signed=False, label="NLA_BLOB_DATA_TYPE"),
    "NLA_CONNECTIVITY_TYPE": SimTypeInt(signed=False, label="NLA_CONNECTIVITY_TYPE"),
    "NLA_INTERNET": SimTypeInt(signed=False, label="NLA_INTERNET"),
    "NLM_CONNECTION_COST": SimTypeInt(signed=False, label="NLM_CONNECTION_COST"),
    "NLM_CONNECTION_PROPERTY_CHANGE": SimTypeInt(signed=False, label="NLM_CONNECTION_PROPERTY_CHANGE"),
    "NLM_CONNECTIVITY": SimTypeInt(signed=False, label="NLM_CONNECTIVITY"),
    "NLM_DATAPLAN_STATUS": SimStruct(OrderedDict((("InterfaceGuid", SimTypeBottom(label="Guid")), ("UsageData", SimTypeRef("NLM_USAGE_DATA", SimStruct)), ("DataLimitInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("InboundBandwidthInKbps", SimTypeInt(signed=False, label="UInt32")), ("OutboundBandwidthInKbps", SimTypeInt(signed=False, label="UInt32")), ("NextBillingCycle", SimTypeRef("FILETIME", SimStruct)), ("MaxTransferSizeInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NLM_DATAPLAN_STATUS", pack=False, align=None),
    "NLM_DOMAIN_AUTHENTICATION_KIND": SimTypeInt(signed=False, label="NLM_DOMAIN_AUTHENTICATION_KIND"),
    "NLM_DOMAIN_TYPE": SimTypeInt(signed=False, label="NLM_DOMAIN_TYPE"),
    "NLM_ENUM_NETWORK": SimTypeInt(signed=False, label="NLM_ENUM_NETWORK"),
    "NLM_INTERNET_CONNECTIVITY": SimTypeInt(signed=False, label="NLM_INTERNET_CONNECTIVITY"),
    "NLM_NETWORK_CATEGORY": SimTypeInt(signed=False, label="NLM_NETWORK_CATEGORY"),
    "NLM_NETWORK_CLASS": SimTypeInt(signed=False, label="NLM_NETWORK_CLASS"),
    "NLM_NETWORK_PROPERTY_CHANGE": SimTypeInt(signed=False, label="NLM_NETWORK_PROPERTY_CHANGE"),
    "NLM_SIMULATED_PROFILE_INFO": SimStruct(OrderedDict((("ProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("cost", SimTypeInt(signed=False, label="NLM_CONNECTION_COST")), ("UsageInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("DataLimitInMegabytes", SimTypeInt(signed=False, label="UInt32")),)), name="NLM_SIMULATED_PROFILE_INFO", pack=False, align=None),
    "NLM_SOCKADDR": SimStruct(OrderedDict((("data", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="NLM_SOCKADDR", pack=False, align=None),
    "NLM_USAGE_DATA": SimStruct(OrderedDict((("UsageInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("LastSyncTime", SimTypeRef("FILETIME", SimStruct)),)), name="NLM_USAGE_DATA", pack=False, align=None),
    "NLSTABLEINFO": SimStruct(OrderedDict((("OemTableInfo", SimTypeRef("CPTABLEINFO", SimStruct)), ("AnsiTableInfo", SimTypeRef("CPTABLEINFO", SimStruct)), ("UpperCaseTable", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("LowerCaseTable", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="NLSTABLEINFO", pack=False, align=None),
    "NLSVERSIONINFO": SimStruct(OrderedDict((("dwNLSVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwNLSVersion", SimTypeInt(signed=False, label="UInt32")), ("dwDefinedVersion", SimTypeInt(signed=False, label="UInt32")), ("dwEffectiveId", SimTypeInt(signed=False, label="UInt32")), ("guidCustomVersion", SimTypeBottom(label="Guid")),)), name="NLSVERSIONINFO", pack=False, align=None),
    "NLSVERSIONINFOEX": SimStruct(OrderedDict((("dwNLSVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwNLSVersion", SimTypeInt(signed=False, label="UInt32")), ("dwDefinedVersion", SimTypeInt(signed=False, label="UInt32")), ("dwEffectiveId", SimTypeInt(signed=False, label="UInt32")), ("guidCustomVersion", SimTypeBottom(label="Guid")),)), name="NLSVERSIONINFOEX", pack=False, align=None),
    "NL_ADDRESS_TYPE": SimTypeInt(signed=False, label="NL_ADDRESS_TYPE"),
    "NL_BANDWIDTH_FLAG": SimTypeInt(signed=False, label="NL_BANDWIDTH_FLAG"),
    "NL_BANDWIDTH_INFORMATION": SimStruct(OrderedDict((("Bandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("Instability", SimTypeLongLong(signed=False, label="UInt64")), ("BandwidthPeaked", SimTypeChar(label="Byte")),)), name="NL_BANDWIDTH_INFORMATION", pack=False, align=None),
    "NL_DAD_STATE": SimTypeInt(signed=False, label="NL_DAD_STATE"),
    "NL_INTERFACE_NETWORK_CATEGORY_STATE": SimTypeInt(signed=False, label="NL_INTERFACE_NETWORK_CATEGORY_STATE"),
    "NL_INTERFACE_OFFLOAD_ROD": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="NL_INTERFACE_OFFLOAD_ROD", pack=False, align=None),
    "NL_LINK_LOCAL_ADDRESS_BEHAVIOR": SimTypeInt(signed=False, label="NL_LINK_LOCAL_ADDRESS_BEHAVIOR"),
    "NL_NEIGHBOR_STATE": SimTypeInt(signed=False, label="NL_NEIGHBOR_STATE"),
    "NL_NETWORK_CATEGORY": SimTypeInt(signed=False, label="NL_NETWORK_CATEGORY"),
    "NL_NETWORK_CONNECTIVITY_COST_HINT": SimTypeInt(signed=False, label="NL_NETWORK_CONNECTIVITY_COST_HINT"),
    "NL_NETWORK_CONNECTIVITY_HINT": SimStruct(OrderedDict((("ConnectivityLevel", SimTypeInt(signed=False, label="NL_NETWORK_CONNECTIVITY_LEVEL_HINT")), ("ConnectivityCost", SimTypeInt(signed=False, label="NL_NETWORK_CONNECTIVITY_COST_HINT")), ("ApproachingDataLimit", SimTypeChar(label="Byte")), ("OverDataLimit", SimTypeChar(label="Byte")), ("Roaming", SimTypeChar(label="Byte")),)), name="NL_NETWORK_CONNECTIVITY_HINT", pack=False, align=None),
    "NL_NETWORK_CONNECTIVITY_LEVEL_HINT": SimTypeInt(signed=False, label="NL_NETWORK_CONNECTIVITY_LEVEL_HINT"),
    "NL_PATH_BANDWIDTH_ROD": SimStruct(OrderedDict((("Bandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("Instability", SimTypeLongLong(signed=False, label="UInt64")), ("BandwidthPeaked", SimTypeChar(label="Byte")),)), name="NL_PATH_BANDWIDTH_ROD", pack=False, align=None),
    "NL_PREFIX_ORIGIN": SimTypeInt(signed=False, label="NL_PREFIX_ORIGIN"),
    "NL_ROUTER_DISCOVERY_BEHAVIOR": SimTypeInt(signed=False, label="NL_ROUTER_DISCOVERY_BEHAVIOR"),
    "NL_ROUTE_ORIGIN": SimTypeInt(signed=False, label="NL_ROUTE_ORIGIN"),
    "NL_ROUTE_PROTOCOL": SimTypeInt(signed=False, label="NL_ROUTE_PROTOCOL"),
    "NL_SUFFIX_ORIGIN": SimTypeInt(signed=False, label="NL_SUFFIX_ORIGIN"),
    "NMBCDROPDOWN": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("rcButton", SimTypeRef("RECT", SimStruct)),)), name="NMBCDROPDOWN", pack=False, align=None),
    "NMBCHOTITEM": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="NMTBHOTITEM_FLAGS")),)), name="NMBCHOTITEM", pack=False, align=None),
    "NMCBEDRAGBEGINA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItemid", SimTypeInt(signed=True, label="Int32")), ("szText", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="NMCBEDRAGBEGINA", pack=False, align=None),
    "NMCBEDRAGBEGINW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItemid", SimTypeInt(signed=True, label="Int32")), ("szText", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="NMCBEDRAGBEGINW", pack=False, align=None),
    "NMCBEENDEDITA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("fChanged", SimTypeInt(signed=True, label="Int32")), ("iNewSelection", SimTypeInt(signed=True, label="Int32")), ("szText", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("iWhy", SimTypeInt(signed=True, label="Int32")),)), name="NMCBEENDEDITA", pack=False, align=None),
    "NMCBEENDEDITW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("fChanged", SimTypeInt(signed=True, label="Int32")), ("iNewSelection", SimTypeInt(signed=True, label="Int32")), ("szText", SimTypeArray(SimTypeChar(label="Char"), 260)), ("iWhy", SimTypeInt(signed=True, label="Int32")),)), name="NMCBEENDEDITW", pack=False, align=None),
    "NMCHAR": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("ch", SimTypeInt(signed=False, label="UInt32")), ("dwItemPrev", SimTypeInt(signed=False, label="UInt32")), ("dwItemNext", SimTypeInt(signed=False, label="UInt32")),)), name="NMCHAR", pack=False, align=None),
    "NMCOMBOBOXEXA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("ceItem", SimTypeRef("COMBOBOXEXITEMA", SimStruct)),)), name="NMCOMBOBOXEXA", pack=False, align=None),
    "NMCOMBOBOXEXW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("ceItem", SimTypeRef("COMBOBOXEXITEMW", SimStruct)),)), name="NMCOMBOBOXEXW", pack=False, align=None),
    "NMCUSTOMDRAW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwDrawStage", SimTypeInt(signed=False, label="NMCUSTOMDRAW_DRAW_STAGE")), ("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)), ("dwItemSpec", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("uItemState", SimTypeInt(signed=False, label="NMCUSTOMDRAW_DRAW_STATE_FLAGS")), ("lItemlParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMCUSTOMDRAW", pack=False, align=None),
    "NMCUSTOMDRAW_DRAW_STAGE": SimTypeInt(signed=False, label="NMCUSTOMDRAW_DRAW_STAGE"),
    "NMCUSTOMDRAW_DRAW_STATE_FLAGS": SimTypeInt(signed=False, label="NMCUSTOMDRAW_DRAW_STATE_FLAGS"),
    "NMCUSTOMSPLITRECTINFO": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("rcClient", SimTypeRef("RECT", SimStruct)), ("rcButton", SimTypeRef("RECT", SimStruct)), ("rcSplit", SimTypeRef("RECT", SimStruct)),)), name="NMCUSTOMSPLITRECTINFO", pack=False, align=None),
    "NMCUSTOMTEXT": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nCount", SimTypeInt(signed=True, label="Int32")), ("lpRect", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("uFormat", SimTypeInt(signed=False, label="UInt32")), ("fLink", SimTypeInt(signed=True, label="Int32")),)), name="NMCUSTOMTEXT", pack=False, align=None),
    "NMDATETIMECHANGE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="NMDATETIMECHANGE_FLAGS")), ("st", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="NMDATETIMECHANGE", pack=False, align=None),
    "NMDATETIMECHANGE_FLAGS": SimTypeInt(signed=False, label="NMDATETIMECHANGE_FLAGS"),
    "NMDATETIMEFORMATA": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("pszFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("st", SimTypeRef("SYSTEMTIME", SimStruct)), ("pszDisplay", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szDisplay", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="NMDATETIMEFORMATA", pack=False, align=None),
    "NMDATETIMEFORMATQUERYA": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("pszFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szMax", SimTypeRef("SIZE", SimStruct)),)), name="NMDATETIMEFORMATQUERYA", pack=False, align=None),
    "NMDATETIMEFORMATQUERYW": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("pszFormat", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szMax", SimTypeRef("SIZE", SimStruct)),)), name="NMDATETIMEFORMATQUERYW", pack=False, align=None),
    "NMDATETIMEFORMATW": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("pszFormat", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("st", SimTypeRef("SYSTEMTIME", SimStruct)), ("pszDisplay", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szDisplay", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="NMDATETIMEFORMATW", pack=False, align=None),
    "NMDATETIMESTRINGA": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("pszUserString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("st", SimTypeRef("SYSTEMTIME", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMDATETIMESTRINGA", pack=False, align=None),
    "NMDATETIMESTRINGW": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("pszUserString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("st", SimTypeRef("SYSTEMTIME", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMDATETIMESTRINGW", pack=False, align=None),
    "NMDATETIMEWMKEYDOWNA": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("nVirtKey", SimTypeInt(signed=True, label="Int32")), ("pszFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("st", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="NMDATETIMEWMKEYDOWNA", pack=False, align=None),
    "NMDATETIMEWMKEYDOWNW": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("nVirtKey", SimTypeInt(signed=True, label="Int32")), ("pszFormat", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("st", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="NMDATETIMEWMKEYDOWNW", pack=False, align=None),
    "NMDAYSTATE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("stStart", SimTypeRef("SYSTEMTIME", SimStruct)), ("cDayState", SimTypeInt(signed=True, label="Int32")), ("prgDayState", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="NMDAYSTATE", pack=False, align=None),
    "NMHDDISPINFOA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("mask", SimTypeInt(signed=False, label="HDI_MASK")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMHDDISPINFOA", pack=False, align=None),
    "NMHDDISPINFOW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("mask", SimTypeInt(signed=False, label="HDI_MASK")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMHDDISPINFOW", pack=False, align=None),
    "NMHDFILTERBTNCLICK": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("rc", SimTypeRef("RECT", SimStruct)),)), name="NMHDFILTERBTNCLICK", pack=False, align=None),
    "NMHDR": SimStruct(OrderedDict((("hwndFrom", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("idFrom", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("code", SimTypeInt(signed=False, label="UInt32")),)), name="NMHDR", pack=False, align=None),
    "NMHEADERA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iButton", SimTypeInt(signed=False, label="HEADER_CONTROL_NOTIFICATION_BUTTON")), ("pitem", SimTypePointer(SimTypeRef("HDITEMA", SimStruct), offset=0)),)), name="NMHEADERA", pack=False, align=None),
    "NMHEADERW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iButton", SimTypeInt(signed=False, label="HEADER_CONTROL_NOTIFICATION_BUTTON")), ("pitem", SimTypePointer(SimTypeRef("HDITEMW", SimStruct), offset=0)),)), name="NMHEADERW", pack=False, align=None),
    "NMIPADDRESS": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iField", SimTypeInt(signed=True, label="Int32")), ("iValue", SimTypeInt(signed=True, label="Int32")),)), name="NMIPADDRESS", pack=False, align=None),
    "NMITEMACTIVATE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("uNewState", SimTypeInt(signed=False, label="UInt32")), ("uOldState", SimTypeInt(signed=False, label="UInt32")), ("uChanged", SimTypeInt(signed=False, label="UInt32")), ("ptAction", SimTypeRef("POINT", SimStruct)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uKeyFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMITEMACTIVATE", pack=False, align=None),
    "NMI_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["Context", "Handled"]), offset=0),
    "NMKEY": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("nVKey", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMKEY", pack=False, align=None),
    "NMLINK": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("LITEM", SimStruct)),)), name="NMLINK", pack=False, align=None),
    "NMLISTVIEW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("uNewState", SimTypeInt(signed=False, label="UInt32")), ("uOldState", SimTypeInt(signed=False, label="UInt32")), ("uChanged", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_FLAGS")), ("ptAction", SimTypeRef("POINT", SimStruct)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMLISTVIEW", pack=False, align=None),
    "NMLVCACHEHINT": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iFrom", SimTypeInt(signed=True, label="Int32")), ("iTo", SimTypeInt(signed=True, label="Int32")),)), name="NMLVCACHEHINT", pack=False, align=None),
    "NMLVCUSTOMDRAW": SimStruct(OrderedDict((("nmcd", SimTypeRef("NMCUSTOMDRAW", SimStruct)), ("clrText", SimTypeInt(signed=False, label="UInt32")), ("clrTextBk", SimTypeInt(signed=False, label="UInt32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("dwItemType", SimTypeInt(signed=False, label="NMLVCUSTOMDRAW_ITEM_TYPE")), ("clrFace", SimTypeInt(signed=False, label="UInt32")), ("iIconEffect", SimTypeInt(signed=True, label="Int32")), ("iIconPhase", SimTypeInt(signed=True, label="Int32")), ("iPartId", SimTypeInt(signed=True, label="Int32")), ("iStateId", SimTypeInt(signed=True, label="Int32")), ("rcText", SimTypeRef("RECT", SimStruct)), ("uAlign", SimTypeInt(signed=False, label="LIST_VIEW_GROUP_ALIGN_FLAGS")),)), name="NMLVCUSTOMDRAW", pack=False, align=None),
    "NMLVCUSTOMDRAW_ITEM_TYPE": SimTypeInt(signed=False, label="NMLVCUSTOMDRAW_ITEM_TYPE"),
    "NMLVDISPINFOA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("LVITEMA", SimStruct)),)), name="NMLVDISPINFOA", pack=False, align=None),
    "NMLVDISPINFOW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("LVITEMW", SimStruct)),)), name="NMLVDISPINFOW", pack=False, align=None),
    "NMLVEMPTYMARKUP": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="NMLVEMPTYMARKUP_FLAGS")), ("szMarkup", SimTypeArray(SimTypeChar(label="Char"), 2084)),)), name="NMLVEMPTYMARKUP", pack=False, align=None),
    "NMLVEMPTYMARKUP_FLAGS": SimTypeInt(signed=False, label="NMLVEMPTYMARKUP_FLAGS"),
    "NMLVFINDITEMA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iStart", SimTypeInt(signed=True, label="Int32")), ("lvfi", SimTypeRef("LVFINDINFOA", SimStruct)),)), name="NMLVFINDITEMA", pack=False, align=None),
    "NMLVFINDITEMW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iStart", SimTypeInt(signed=True, label="Int32")), ("lvfi", SimTypeRef("LVFINDINFOW", SimStruct)),)), name="NMLVFINDITEMW", pack=False, align=None),
    "NMLVGETINFOTIPA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="NMLVGETINFOTIP_FLAGS")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMLVGETINFOTIPA", pack=False, align=None),
    "NMLVGETINFOTIPW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="NMLVGETINFOTIP_FLAGS")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMLVGETINFOTIPW", pack=False, align=None),
    "NMLVGETINFOTIP_FLAGS": SimTypeInt(signed=False, label="NMLVGETINFOTIP_FLAGS"),
    "NMLVKEYDOWN": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("wVKey", SimTypeShort(signed=False, label="UInt16")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="NMLVKEYDOWN", pack=False, align=None),
    "NMLVLINK": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("link", SimTypeRef("LITEM", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("iSubItem", SimTypeInt(signed=True, label="Int32")),)), name="NMLVLINK", pack=False, align=None),
    "NMLVODSTATECHANGE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iFrom", SimTypeInt(signed=True, label="Int32")), ("iTo", SimTypeInt(signed=True, label="Int32")), ("uNewState", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS")), ("uOldState", SimTypeInt(signed=False, label="LIST_VIEW_ITEM_STATE_FLAGS")),)), name="NMLVODSTATECHANGE", pack=False, align=None),
    "NMLVSCROLL": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dx", SimTypeInt(signed=True, label="Int32")), ("dy", SimTypeInt(signed=True, label="Int32")),)), name="NMLVSCROLL", pack=False, align=None),
    "NMMOUSE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwItemSpec", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwItemData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pt", SimTypeRef("POINT", SimStruct)), ("dwHitInfo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMMOUSE", pack=False, align=None),
    "NMOBJECTNOTIFY": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("piid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hResult", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMOBJECTNOTIFY", pack=False, align=None),
    "NMPGCALCSIZE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwFlag", SimTypeInt(signed=False, label="NMPGCALCSIZE_FLAGS")), ("iWidth", SimTypeInt(signed=True, label="Int32")), ("iHeight", SimTypeInt(signed=True, label="Int32")),)), name="NMPGCALCSIZE", pack=False, align=None),
    "NMPGCALCSIZE_FLAGS": SimTypeInt(signed=False, label="NMPGCALCSIZE_FLAGS"),
    "NMPGHOTITEM": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("idOld", SimTypeInt(signed=True, label="Int32")), ("idNew", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMPGHOTITEM", pack=False, align=None),
    "NMPGSCROLL": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("fwKeys", SimTypeInt(signed=False, label="NMPGSCROLL_KEYS")), ("rcParent", SimTypeRef("RECT", SimStruct)), ("iDir", SimTypeInt(signed=False, label="NMPGSCROLL_DIR")), ("iXpos", SimTypeInt(signed=True, label="Int32")), ("iYpos", SimTypeInt(signed=True, label="Int32")), ("iScroll", SimTypeInt(signed=True, label="Int32")),)), name="NMPGSCROLL", pack=False, align=None),
    "NMPGSCROLL_DIR": SimTypeInt(signed=False, label="NMPGSCROLL_DIR"),
    "NMPGSCROLL_KEYS": SimTypeInt(signed=False, label="NMPGSCROLL_KEYS"),
    "NMRBAUTOSIZE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("fChanged", SimTypeInt(signed=True, label="Int32")), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("rcActual", SimTypeRef("RECT", SimStruct)),)), name="NMRBAUTOSIZE", pack=False, align=None),
    "NMREBAR": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwMask", SimTypeInt(signed=False, label="NMREBAR_MASK_FLAGS")), ("uBand", SimTypeInt(signed=False, label="UInt32")), ("fStyle", SimTypeInt(signed=False, label="UInt32")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMREBAR", pack=False, align=None),
    "NMREBARAUTOBREAK": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("uBand", SimTypeInt(signed=False, label="UInt32")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uMsg", SimTypeInt(signed=False, label="UInt32")), ("fStyleCurrent", SimTypeInt(signed=False, label="UInt32")), ("fAutoBreak", SimTypeInt(signed=True, label="Int32")),)), name="NMREBARAUTOBREAK", pack=False, align=None),
    "NMREBARCHEVRON": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("uBand", SimTypeInt(signed=False, label="UInt32")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)), ("lParamNM", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMREBARCHEVRON", pack=False, align=None),
    "NMREBARCHILDSIZE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("uBand", SimTypeInt(signed=False, label="UInt32")), ("wID", SimTypeInt(signed=False, label="UInt32")), ("rcChild", SimTypeRef("RECT", SimStruct)), ("rcBand", SimTypeRef("RECT", SimStruct)),)), name="NMREBARCHILDSIZE", pack=False, align=None),
    "NMREBARSPLITTER": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("rcSizing", SimTypeRef("RECT", SimStruct)),)), name="NMREBARSPLITTER", pack=False, align=None),
    "NMREBAR_MASK_FLAGS": SimTypeInt(signed=False, label="NMREBAR_MASK_FLAGS"),
    "NMSEARCHWEB": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("entrypoint", SimTypeInt(signed=False, label="EC_SEARCHWEB_ENTRYPOINT")), ("hasQueryText", SimTypeInt(signed=True, label="Int32")), ("invokeSucceeded", SimTypeInt(signed=True, label="Int32")),)), name="NMSEARCHWEB", pack=False, align=None),
    "NMSELCHANGE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("stSelStart", SimTypeRef("SYSTEMTIME", SimStruct)), ("stSelEnd", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="NMSELCHANGE", pack=False, align=None),
    "NMS_VBXADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")),)), name="NMS_VBXADPCMWAVEFORMAT", pack=False, align=None),
    "NMTBCUSTOMDRAW": SimStruct(OrderedDict((("nmcd", SimTypeRef("NMCUSTOMDRAW", SimStruct)), ("hbrMonoDither", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbrLines", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hpenLines", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("clrText", SimTypeInt(signed=False, label="UInt32")), ("clrMark", SimTypeInt(signed=False, label="UInt32")), ("clrTextHighlight", SimTypeInt(signed=False, label="UInt32")), ("clrBtnFace", SimTypeInt(signed=False, label="UInt32")), ("clrBtnHighlight", SimTypeInt(signed=False, label="UInt32")), ("clrHighlightHotTrack", SimTypeInt(signed=False, label="UInt32")), ("rcText", SimTypeRef("RECT", SimStruct)), ("nStringBkMode", SimTypeInt(signed=True, label="Int32")), ("nHLStringBkMode", SimTypeInt(signed=True, label="Int32")), ("iListGap", SimTypeInt(signed=True, label="Int32")),)), name="NMTBCUSTOMDRAW", pack=False, align=None),
    "NMTBDISPINFOA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwMask", SimTypeInt(signed=False, label="NMTBDISPINFOW_MASK")), ("idCommand", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchText", SimTypeInt(signed=True, label="Int32")),)), name="NMTBDISPINFOA", pack=False, align=None),
    "NMTBDISPINFOW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwMask", SimTypeInt(signed=False, label="NMTBDISPINFOW_MASK")), ("idCommand", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchText", SimTypeInt(signed=True, label="Int32")),)), name="NMTBDISPINFOW", pack=False, align=None),
    "NMTBDISPINFOW_MASK": SimTypeInt(signed=False, label="NMTBDISPINFOW_MASK"),
    "NMTBGETINFOTIPA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTBGETINFOTIPA", pack=False, align=None),
    "NMTBGETINFOTIPW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTBGETINFOTIPW", pack=False, align=None),
    "NMTBHOTITEM": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("idOld", SimTypeInt(signed=True, label="Int32")), ("idNew", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="NMTBHOTITEM_FLAGS")),)), name="NMTBHOTITEM", pack=False, align=None),
    "NMTBHOTITEM_FLAGS": SimTypeInt(signed=False, label="NMTBHOTITEM_FLAGS"),
    "NMTBRESTORE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pCurrent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("cButtons", SimTypeInt(signed=True, label="Int32")), ("cbBytesPerRecord", SimTypeInt(signed=True, label="Int32")), ("tbButton", SimTypeRef("TBBUTTON", SimStruct)),)), name="NMTBRESTORE", pack=False, align=None),
    "NMTBSAVE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pData", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pCurrent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("iItem", SimTypeInt(signed=True, label="Int32")), ("cButtons", SimTypeInt(signed=True, label="Int32")), ("tbButton", SimTypeRef("TBBUTTON", SimStruct)),)), name="NMTBSAVE", pack=False, align=None),
    "NMTCKEYDOWN": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("wVKey", SimTypeShort(signed=False, label="UInt16")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="NMTCKEYDOWN", pack=False, align=None),
    "NMTOOLBARA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("tbButton", SimTypeRef("TBBUTTON", SimStruct)), ("cchText", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rcButton", SimTypeRef("RECT", SimStruct)),)), name="NMTOOLBARA", pack=False, align=None),
    "NMTOOLBARW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iItem", SimTypeInt(signed=True, label="Int32")), ("tbButton", SimTypeRef("TBBUTTON", SimStruct)), ("cchText", SimTypeInt(signed=True, label="Int32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rcButton", SimTypeRef("RECT", SimStruct)),)), name="NMTOOLBARW", pack=False, align=None),
    "NMTOOLTIPSCREATED": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("hwndToolTips", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTOOLTIPSCREATED", pack=False, align=None),
    "NMTRBTHUMBPOSCHANGING": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("dwPos", SimTypeInt(signed=False, label="UInt32")), ("nReason", SimTypeInt(signed=True, label="Int32")),)), name="NMTRBTHUMBPOSCHANGING", pack=False, align=None),
    "NMTREEVIEWA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("action", SimTypeInt(signed=False, label="NM_TREEVIEW_ACTION")), ("itemOld", SimTypeRef("TVITEMA", SimStruct)), ("itemNew", SimTypeRef("TVITEMA", SimStruct)), ("ptDrag", SimTypeRef("POINT", SimStruct)),)), name="NMTREEVIEWA", pack=False, align=None),
    "NMTREEVIEWW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("action", SimTypeInt(signed=False, label="NM_TREEVIEW_ACTION")), ("itemOld", SimTypeRef("TVITEMW", SimStruct)), ("itemNew", SimTypeRef("TVITEMW", SimStruct)), ("ptDrag", SimTypeRef("POINT", SimStruct)),)), name="NMTREEVIEWW", pack=False, align=None),
    "NMTTCUSTOMDRAW": SimStruct(OrderedDict((("nmcd", SimTypeRef("NMCUSTOMDRAW", SimStruct)), ("uDrawFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NMTTCUSTOMDRAW", pack=False, align=None),
    "NMTTDISPINFOA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lpszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szText", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("hinst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uFlags", SimTypeInt(signed=False, label="TOOLTIP_FLAGS")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTTDISPINFOA", pack=False, align=None),
    "NMTTDISPINFOW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lpszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szText", SimTypeArray(SimTypeChar(label="Char"), 80)), ("hinst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uFlags", SimTypeInt(signed=False, label="TOOLTIP_FLAGS")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTTDISPINFOW", pack=False, align=None),
    "NMTVASYNCDRAW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pimldp", SimTypePointer(SimTypeRef("IMAGELISTDRAWPARAMS", SimStruct), offset=0)), ("hr", SimTypeInt(signed=True, label="Int32")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwRetFlags", SimTypeInt(signed=False, label="UInt32")), ("iRetImageIndex", SimTypeInt(signed=True, label="Int32")),)), name="NMTVASYNCDRAW", pack=False, align=None),
    "NMTVCUSTOMDRAW": SimStruct(OrderedDict((("nmcd", SimTypeRef("NMCUSTOMDRAW", SimStruct)), ("clrText", SimTypeInt(signed=False, label="UInt32")), ("clrTextBk", SimTypeInt(signed=False, label="UInt32")), ("iLevel", SimTypeInt(signed=True, label="Int32")),)), name="NMTVCUSTOMDRAW", pack=False, align=None),
    "NMTVDISPINFOA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("TVITEMA", SimStruct)),)), name="NMTVDISPINFOA", pack=False, align=None),
    "NMTVDISPINFOEXA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("TVITEMEXA", SimStruct)),)), name="NMTVDISPINFOEXA", pack=False, align=None),
    "NMTVDISPINFOEXW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("TVITEMEXW", SimStruct)),)), name="NMTVDISPINFOEXW", pack=False, align=None),
    "NMTVDISPINFOW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("item", SimTypeRef("TVITEMW", SimStruct)),)), name="NMTVDISPINFOW", pack=False, align=None),
    "NMTVGETINFOTIPA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTVGETINFOTIPA", pack=False, align=None),
    "NMTVGETINFOTIPW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTVGETINFOTIPW", pack=False, align=None),
    "NMTVITEMCHANGE": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("uChanged", SimTypeInt(signed=False, label="UInt32")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uStateNew", SimTypeInt(signed=False, label="UInt32")), ("uStateOld", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NMTVITEMCHANGE", pack=False, align=None),
    "NMTVKEYDOWN": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("wVKey", SimTypeShort(signed=False, label="UInt16")), ("flags", SimTypeInt(signed=False, label="UInt32")),)), name="NMTVKEYDOWN", pack=False, align=None),
    "NMTVSTATEIMAGECHANGING": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("hti", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iOldStateImageIndex", SimTypeInt(signed=True, label="Int32")), ("iNewStateImageIndex", SimTypeInt(signed=True, label="Int32")),)), name="NMTVSTATEIMAGECHANGING", pack=False, align=None),
    "NMUPDOWN": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("iPos", SimTypeInt(signed=True, label="Int32")), ("iDelta", SimTypeInt(signed=True, label="Int32")),)), name="NMUPDOWN", pack=False, align=None),
    "NMVIEWCHANGE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("dwOldView", SimTypeInt(signed=False, label="MONTH_CALDENDAR_MESSAGES_VIEW")), ("dwNewView", SimTypeInt(signed=False, label="MONTH_CALDENDAR_MESSAGES_VIEW")),)), name="NMVIEWCHANGE", pack=False, align=None),
    "NM_TREEVIEW_ACTION": SimTypeInt(signed=False, label="NM_TREEVIEW_ACTION"),
    "NODERESTRICTION": SimStruct(OrderedDict((("cRes", SimTypeInt(signed=False, label="UInt32")), ("paRes", SimTypePointer(SimTypePointer(SimTypeRef("RESTRICTION", SimStruct), offset=0), offset=0)), ("reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NODERESTRICTION", pack=False, align=None),
    "NODE_CLUSTER_STATE": SimTypeInt(signed=False, label="NODE_CLUSTER_STATE"),
    "NONCLIENTMETRICSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iBorderWidth", SimTypeInt(signed=True, label="Int32")), ("iScrollWidth", SimTypeInt(signed=True, label="Int32")), ("iScrollHeight", SimTypeInt(signed=True, label="Int32")), ("iCaptionWidth", SimTypeInt(signed=True, label="Int32")), ("iCaptionHeight", SimTypeInt(signed=True, label="Int32")), ("lfCaptionFont", SimTypeRef("LOGFONTA", SimStruct)), ("iSmCaptionWidth", SimTypeInt(signed=True, label="Int32")), ("iSmCaptionHeight", SimTypeInt(signed=True, label="Int32")), ("lfSmCaptionFont", SimTypeRef("LOGFONTA", SimStruct)), ("iMenuWidth", SimTypeInt(signed=True, label="Int32")), ("iMenuHeight", SimTypeInt(signed=True, label="Int32")), ("lfMenuFont", SimTypeRef("LOGFONTA", SimStruct)), ("lfStatusFont", SimTypeRef("LOGFONTA", SimStruct)), ("lfMessageFont", SimTypeRef("LOGFONTA", SimStruct)), ("iPaddedBorderWidth", SimTypeInt(signed=True, label="Int32")),)), name="NONCLIENTMETRICSA", pack=False, align=None),
    "NONCLIENTMETRICSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("iBorderWidth", SimTypeInt(signed=True, label="Int32")), ("iScrollWidth", SimTypeInt(signed=True, label="Int32")), ("iScrollHeight", SimTypeInt(signed=True, label="Int32")), ("iCaptionWidth", SimTypeInt(signed=True, label="Int32")), ("iCaptionHeight", SimTypeInt(signed=True, label="Int32")), ("lfCaptionFont", SimTypeRef("LOGFONTW", SimStruct)), ("iSmCaptionWidth", SimTypeInt(signed=True, label="Int32")), ("iSmCaptionHeight", SimTypeInt(signed=True, label="Int32")), ("lfSmCaptionFont", SimTypeRef("LOGFONTW", SimStruct)), ("iMenuWidth", SimTypeInt(signed=True, label="Int32")), ("iMenuHeight", SimTypeInt(signed=True, label="Int32")), ("lfMenuFont", SimTypeRef("LOGFONTW", SimStruct)), ("lfStatusFont", SimTypeRef("LOGFONTW", SimStruct)), ("lfMessageFont", SimTypeRef("LOGFONTW", SimStruct)), ("iPaddedBorderWidth", SimTypeInt(signed=True, label="Int32")),)), name="NONCLIENTMETRICSW", pack=False, align=None),
    "NONESTATES": SimTypeInt(signed=False, label="NONESTATES"),
    "NON_PAGED_DEBUG_INFO": SimStruct(OrderedDict((("Signature", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Machine", SimTypeShort(signed=False, label="UInt16")), ("Characteristics", SimTypeShort(signed=False, label="UInt16")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("ImageBase", SimTypeLongLong(signed=False, label="UInt64")),)), name="NON_PAGED_DEBUG_INFO", pack=False, align=None),
    "NORMALGROUPCOLLAPSESTATES": SimTypeInt(signed=False, label="NORMALGROUPCOLLAPSESTATES"),
    "NORMALGROUPEXPANDSTATES": SimTypeInt(signed=False, label="NORMALGROUPEXPANDSTATES"),
    "NORMALIZEDRECT": SimStruct(OrderedDict((("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("right", SimTypeFloat(size=32)), ("bottom", SimTypeFloat(size=32)),)), name="NORMALIZEDRECT", pack=False, align=None),
    "NORM_FORM": SimTypeInt(signed=False, label="NORM_FORM"),
    "NOTIFICATION": SimStruct(OrderedDict((("ulEventType", SimTypeInt(signed=False, label="UInt32")), ("ulAlignPad", SimTypeInt(signed=False, label="UInt32")), ("info", SimUnion({"err": SimTypeRef("ERROR_NOTIFICATION", SimStruct), "newmail": SimTypeRef("NEWMAIL_NOTIFICATION", SimStruct), "obj": SimTypeRef("OBJECT_NOTIFICATION", SimStruct), "tab": SimTypeRef("TABLE_NOTIFICATION", SimStruct), "ext": SimTypeRef("EXTENDED_NOTIFICATION", SimStruct), "statobj": SimTypeRef("STATUS_OBJECT_NOTIFICATION", SimStruct)}, name="<anon>", label="None")),)), name="NOTIFICATION", pack=False, align=None),
    "NOTIFICATION_CALLBACK_COMMANDS": SimTypeInt(signed=False, label="NOTIFICATION_CALLBACK_COMMANDS"),
    "NOTIFICATION_CONFIG_1": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fdwFlags", SimTypeInt(signed=False, label="UInt32")), ("pfnNotifyCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("PRINTER_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCommand", "pContext", "dwColor", "pNofityInfo", "fdwFlags", "pdwResult"]), offset=0)), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NOTIFICATION_CONFIG_1", pack=False, align=None),
    "NOTIFICATION_CONFIG_FLAGS": SimTypeInt(signed=False, label="NOTIFICATION_CONFIG_FLAGS"),
    "NOTIFICATION_USER_INPUT_DATA": SimStruct(OrderedDict((("Key", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NOTIFICATION_USER_INPUT_DATA", pack=False, align=None),
    "NOTIFKEY": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("ab", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NOTIFKEY", pack=False, align=None),
    "NOTIFYADD": SimStruct(OrderedDict((("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NetResource", SimTypeRef("NETRESOURCEA", SimStruct)), ("dwAddFlags", SimTypeInt(signed=False, label="NET_USE_CONNECT_FLAGS")),)), name="NOTIFYADD", pack=False, align=None),
    "NOTIFYCANCEL": SimStruct(OrderedDict((("lpName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fForce", SimTypeInt(signed=True, label="Int32")),)), name="NOTIFYCANCEL", pack=False, align=None),
    "NOTIFYICONDATAA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uID", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="NOTIFY_ICON_DATA_FLAGS")), ("uCallbackMessage", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szTip", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("dwState", SimTypeInt(signed=False, label="NOTIFY_ICON_STATE")), ("dwStateMask", SimTypeInt(signed=False, label="NOTIFY_ICON_STATE")), ("szInfo", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Anonymous", SimUnion({"uTimeout": SimTypeInt(signed=False, label="UInt32"), "uVersion": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("szInfoTitle", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("dwInfoFlags", SimTypeInt(signed=False, label="NOTIFY_ICON_INFOTIP_FLAGS")), ("guidItem", SimTypeBottom(label="Guid")), ("hBalloonIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NOTIFYICONDATAA", pack=False, align=None),
    "NOTIFYICONDATAW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uID", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="NOTIFY_ICON_DATA_FLAGS")), ("uCallbackMessage", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szTip", SimTypeArray(SimTypeChar(label="Char"), 128)), ("dwState", SimTypeInt(signed=False, label="NOTIFY_ICON_STATE")), ("dwStateMask", SimTypeInt(signed=False, label="NOTIFY_ICON_STATE")), ("szInfo", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Anonymous", SimUnion({"uTimeout": SimTypeInt(signed=False, label="UInt32"), "uVersion": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("szInfoTitle", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwInfoFlags", SimTypeInt(signed=False, label="NOTIFY_ICON_INFOTIP_FLAGS")), ("guidItem", SimTypeBottom(label="Guid")), ("hBalloonIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NOTIFYICONDATAW", pack=False, align=None),
    "NOTIFYICONIDENTIFIER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uID", SimTypeInt(signed=False, label="UInt32")), ("guidItem", SimTypeBottom(label="Guid")),)), name="NOTIFYICONIDENTIFIER", pack=False, align=None),
    "NOTIFYINFO": SimStruct(OrderedDict((("dwNotifyStatus", SimTypeInt(signed=False, label="UInt32")), ("dwOperationStatus", SimTypeInt(signed=False, label="UInt32")), ("lpContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NOTIFYINFO", pack=False, align=None),
    "NOTIFYOFNEWCONNECTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["PrimaryConnection", "ReferralFromConnection", "NewDN", "HostName", "NewConnection", "PortNumber", "SecAuthIdentity", "CurrentUser", "ErrorCodeFromBind"]), offset=0),
    "NOTIFY_FILTER_AND_TYPE": SimStruct(OrderedDict((("dwObjectType", SimTypeInt(signed=False, label="UInt32")), ("FilterFlags", SimTypeLongLong(signed=True, label="Int64")),)), name="NOTIFY_FILTER_AND_TYPE", pack=False, align=None),
    "NOTIFY_ICON_DATA_FLAGS": SimTypeInt(signed=False, label="NOTIFY_ICON_DATA_FLAGS"),
    "NOTIFY_ICON_INFOTIP_FLAGS": SimTypeInt(signed=False, label="NOTIFY_ICON_INFOTIP_FLAGS"),
    "NOTIFY_ICON_MESSAGE": SimTypeInt(signed=False, label="NOTIFY_ICON_MESSAGE"),
    "NOTIFY_ICON_STATE": SimTypeInt(signed=False, label="NOTIFY_ICON_STATE"),
    "NOTIFY_IME_ACTION": SimTypeInt(signed=False, label="NOTIFY_IME_ACTION"),
    "NOTIFY_IME_INDEX": SimTypeInt(signed=False, label="NOTIFY_IME_INDEX"),
    "NOTIFY_USER_POWER_SETTING": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")),)), name="NOTIFY_USER_POWER_SETTING", pack=False, align=None),
    "NOTRESTRICTION": SimStruct(OrderedDict((("pRes", SimTypePointer(SimTypeRef("RESTRICTION", SimStruct), offset=0)),)), name="NOTRESTRICTION", pack=False, align=None),
    "NPDIRECTORY_NOTIFY_OPERATION": SimTypeInt(signed=False, label="NPDIRECTORY_NOTIFY_OPERATION"),
    "NPEM_CAPABILITY_STANDARD": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NPEM_CONTROL_ENABLE_DISABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "EnableNpem"]), offset=0),
    "NPEM_CONTROL_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetNpemSupportState", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryStandardCapabilities", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("SetStandardControl", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryNpemControl", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)),)), name="NPEM_CONTROL_INTERFACE", pack=False, align=None),
    "NPEM_CONTROL_QUERY_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context"]), offset=0),
    "NPEM_CONTROL_QUERY_STANDARD_CAPABILITIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "StandardCapabilities"]), offset=0),
    "NPEM_CONTROL_SET_STANDARD_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="NPEM_CONTROL_STANDARD_CONTROL_BIT"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "StandardControl", "Set"]), offset=0),
    "NPEM_CONTROL_STANDARD_CONTROL_BIT": SimTypeInt(signed=False, label="NPEM_CONTROL_STANDARD_CONTROL_BIT"),
    "NPI_MODULEID": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeInt(signed=False, label="NPI_MODULEID_TYPE")), ("Anonymous", SimUnion({"Guid": SimTypeBottom(label="Guid"), "IfLuid": SimTypeRef("LUID", SimStruct)}, name="<anon>", label="None")),)), name="NPI_MODULEID", pack=False, align=None),
    "NPI_MODULEID_TYPE": SimTypeInt(signed=False, label="NPI_MODULEID_TYPE"),
    "NP_PROPERTY_DIALOG_SELECTION": SimTypeInt(signed=False, label="NP_PROPERTY_DIALOG_SELECTION"),
    "NRESARRAY": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("nr", SimTypePointer(SimTypeRef("NETRESOURCEA", SimStruct), offset=0)),)), name="NRESARRAY", pack=False, align=None),
    "NSID": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("uchType", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("xtype", SimTypeInt(signed=False, label="UInt32")), ("lTime", SimTypeInt(signed=True, label="Int32")), ("address", SimUnion({"alias": SimTypeRef("ADDRALIAS", SimStruct), "rgchInterNet": SimTypePointer(SimTypeChar(label="SByte"), offset=0)}, name="<anon>", label="None")),)), name="NSID", pack=False, align=None),
    "NSPV2_ROUTINE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("NSPv2Startup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "ppvClientSessionArg"]), offset=0)), ("NSPv2Cleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "pvClientSessionArg"]), offset=0)), ("NSPv2LookupServiceBegin", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSET2W", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpqsRestrictions", "dwControlFlags", "lpvClientSessionArg", "lphLookup"]), offset=0)), ("NSPv2LookupServiceNextEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSET2W", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["hAsyncCall", "hLookup", "dwControlFlags", "lpdwBufferLength", "lpqsResults"]), offset=0)), ("NSPv2LookupServiceEnd", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup"]), offset=0)), ("NSPv2SetServiceEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSET2W", SimStruct), offset=0), SimTypeInt(signed=False, label="WSAESETSERVICEOP"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hAsyncCall", "lpProviderId", "lpqsRegInfo", "essOperation", "dwControlFlags", "lpvClientSessionArg"]), offset=0)), ("NSPv2ClientSessionRundown", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpProviderId", "pvClientSessionArg"]), offset=0)),)), name="NSPV2_ROUTINE", pack=False, align=None),
    "NSP_ROUTINE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("NSPCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId"]), offset=0)), ("NSPLookupServiceBegin", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSETW", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpqsRestrictions", "lpServiceClassInfo", "dwControlFlags", "lphLookup"]), offset=0)), ("NSPLookupServiceNext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSETW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup", "dwControlFlags", "lpdwBufferLength", "lpqsResults"]), offset=0)), ("NSPLookupServiceEnd", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup"]), offset=0)), ("NSPSetService", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSAQUERYSETW", SimStruct), offset=0), SimTypeInt(signed=False, label="WSAESETSERVICEOP"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpServiceClassInfo", "lpqsRegInfo", "essOperation", "dwControlFlags"]), offset=0)), ("NSPInstallServiceClass", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpServiceClassInfo"]), offset=0)), ("NSPRemoveServiceClass", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpServiceClassId"]), offset=0)), ("NSPGetServiceClassInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSASERVICECLASSINFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpdwBufSize", "lpServiceClassInfo"]), offset=0)), ("NSPIoctl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WSACOMPLETION", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hLookup", "dwControlCode", "lpvInBuffer", "cbInBuffer", "lpvOutBuffer", "cbOutBuffer", "lpcbBytesReturned", "lpCompletion", "lpThreadId"]), offset=0)),)), name="NSP_ROUTINE", pack=False, align=None),
    "NSTCCUSTOMDRAW": SimStruct(OrderedDict((("psi", SimTypeBottom(label="IShellItem")), ("uItemState", SimTypeInt(signed=False, label="UInt32")), ("nstcis", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("iImage", SimTypeInt(signed=True, label="Int32")), ("himl", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iLevel", SimTypeInt(signed=True, label="Int32")), ("iIndent", SimTypeInt(signed=True, label="Int32")),)), name="NSTCCUSTOMDRAW", pack=False, align=None),
    "NSTCFOLDERCAPABILITIES": SimTypeInt(signed=False, label="NSTCFOLDERCAPABILITIES"),
    "NSTCGNI": SimTypeInt(signed=False, label="NSTCGNI"),
    "NSTCSTYLE2": SimTypeInt(signed=False, label="NSTCSTYLE2"),
    "NS_CMD_FLAGS": SimTypeInt(signed=False, label="NS_CMD_FLAGS"),
    "NS_CONTEXT_ATTRIBUTES": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "_ullAlign": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("pwszContext", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("guidHelper", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ulPriority", SimTypeInt(signed=False, label="UInt32")), ("ulNumTopCmds", SimTypeInt(signed=False, label="UInt32")), ("pTopCmds", SimTypePointer(SimTypeRef("CMD_ENTRY", SimStruct), offset=0)), ("ulNumGroups", SimTypeInt(signed=False, label="UInt32")), ("pCmdGroups", SimTypePointer(SimTypeRef("CMD_GROUP_ENTRY", SimStruct), offset=0)), ("pfnCommitFn", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwAction"]), offset=0)), ("pfnDumpFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pwszRouter", "ppwcArguments", "dwArgCount", "pvData"]), offset=0)), ("pfnConnectFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pwszMachine"]), offset=0)), ("pReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfnOsVersionCheck", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CIMOSType", "CIMOSProductSuite", "CIMOSVersion", "CIMOSBuildNumber", "CIMServicePackMajorVersion", "CIMServicePackMinorVersion", "uiReserved", "dwReserved"]), offset=0)),)), name="NS_CONTEXT_ATTRIBUTES", pack=False, align=None),
    "NS_EVENTS": SimTypeInt(signed=False, label="NS_EVENTS"),
    "NS_HELPER_ATTRIBUTES": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "_ullAlign": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("guidHelper", SimTypeBottom(label="Guid")), ("pfnStart", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pguidParent", "dwVersion"]), offset=0)), ("pfnStop", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwReserved"]), offset=0)),)), name="NS_HELPER_ATTRIBUTES", pack=False, align=None),
    "NS_INFOA": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwNameSpaceFlags", SimTypeInt(signed=False, label="UInt32")), ("lpNameSpace", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NS_INFOA", pack=False, align=None),
    "NS_INFOW": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwNameSpaceFlags", SimTypeInt(signed=False, label="UInt32")), ("lpNameSpace", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NS_INFOW", pack=False, align=None),
    "NS_MODE_CHANGE": SimTypeInt(signed=False, label="NS_MODE_CHANGE"),
    "NS_REQS": SimTypeInt(signed=False, label="NS_REQS"),
    "NS_SERVICE_INFOA": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("ServiceInfo", SimTypeRef("SERVICE_INFOA", SimStruct)),)), name="NS_SERVICE_INFOA", pack=False, align=None),
    "NS_SERVICE_INFOW": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("ServiceInfo", SimTypeRef("SERVICE_INFOW", SimStruct)),)), name="NS_SERVICE_INFOW", pack=False, align=None),
    "NTCREATEFILE_CREATE_DISPOSITION": SimTypeInt(signed=False, label="NTCREATEFILE_CREATE_DISPOSITION"),
    "NTCREATEFILE_CREATE_OPTIONS": SimTypeInt(signed=False, label="NTCREATEFILE_CREATE_OPTIONS"),
    "NTEVENTLOGPROPERTIES": SimTypeInt(signed=False, label="NTEVENTLOGPROPERTIES"),
    "NTExtendedHeapRestrictions": SimStruct(OrderedDict((("ddsCapsEx", SimTypeRef("DDSCAPSEX", SimStruct)), ("ddsCapsExAlt", SimTypeRef("DDSCAPSEX", SimStruct)),)), name="NTExtendedHeapRestrictions", pack=False, align=None),
    "NTFS_DEREF_EXPORTED_SECURITY_DESCRIPTOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Vcb", "SecurityDescriptor"]), offset=0),
    "NTFS_EXTENDED_VOLUME_DATA": SimStruct(OrderedDict((("ByteCount", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("BytesPerPhysicalSector", SimTypeInt(signed=False, label="UInt32")), ("LfsMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("LfsMinorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxDeviceTrimExtentCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDeviceTrimByteCount", SimTypeInt(signed=False, label="UInt32")), ("MaxVolumeTrimExtentCount", SimTypeInt(signed=False, label="UInt32")), ("MaxVolumeTrimByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="NTFS_EXTENDED_VOLUME_DATA", pack=False, align=None),
    "NTFS_FILE_RECORD_INPUT_BUFFER": SimStruct(OrderedDict((("FileReferenceNumber", SimTypeLongLong(signed=True, label="Int64")),)), name="NTFS_FILE_RECORD_INPUT_BUFFER", pack=False, align=None),
    "NTFS_FILE_RECORD_OUTPUT_BUFFER": SimStruct(OrderedDict((("FileReferenceNumber", SimTypeLongLong(signed=True, label="Int64")), ("FileRecordLength", SimTypeInt(signed=False, label="UInt32")), ("FileRecordBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NTFS_FILE_RECORD_OUTPUT_BUFFER", pack=False, align=None),
    "NTFS_STATISTICS": SimStruct(OrderedDict((("LogFileFullExceptions", SimTypeInt(signed=False, label="UInt32")), ("OtherExceptions", SimTypeInt(signed=False, label="UInt32")), ("MftReads", SimTypeInt(signed=False, label="UInt32")), ("MftReadBytes", SimTypeInt(signed=False, label="UInt32")), ("MftWrites", SimTypeInt(signed=False, label="UInt32")), ("MftWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("MftWritesUserLevel", SimTypeRef("_MftWritesUserLevel_e__Struct", SimStruct)), ("MftWritesFlushForLogFileFull", SimTypeShort(signed=False, label="UInt16")), ("MftWritesLazyWriter", SimTypeShort(signed=False, label="UInt16")), ("MftWritesUserRequest", SimTypeShort(signed=False, label="UInt16")), ("Mft2Writes", SimTypeInt(signed=False, label="UInt32")), ("Mft2WriteBytes", SimTypeInt(signed=False, label="UInt32")), ("Mft2WritesUserLevel", SimTypeRef("_Mft2WritesUserLevel_e__Struct", SimStruct)), ("Mft2WritesFlushForLogFileFull", SimTypeShort(signed=False, label="UInt16")), ("Mft2WritesLazyWriter", SimTypeShort(signed=False, label="UInt16")), ("Mft2WritesUserRequest", SimTypeShort(signed=False, label="UInt16")), ("RootIndexReads", SimTypeInt(signed=False, label="UInt32")), ("RootIndexReadBytes", SimTypeInt(signed=False, label="UInt32")), ("RootIndexWrites", SimTypeInt(signed=False, label="UInt32")), ("RootIndexWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("BitmapReads", SimTypeInt(signed=False, label="UInt32")), ("BitmapReadBytes", SimTypeInt(signed=False, label="UInt32")), ("BitmapWrites", SimTypeInt(signed=False, label="UInt32")), ("BitmapWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("BitmapWritesFlushForLogFileFull", SimTypeShort(signed=False, label="UInt16")), ("BitmapWritesLazyWriter", SimTypeShort(signed=False, label="UInt16")), ("BitmapWritesUserRequest", SimTypeShort(signed=False, label="UInt16")), ("BitmapWritesUserLevel", SimTypeRef("_BitmapWritesUserLevel_e__Struct", SimStruct)), ("MftBitmapReads", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapReadBytes", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapWrites", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapWritesFlushForLogFileFull", SimTypeShort(signed=False, label="UInt16")), ("MftBitmapWritesLazyWriter", SimTypeShort(signed=False, label="UInt16")), ("MftBitmapWritesUserRequest", SimTypeShort(signed=False, label="UInt16")), ("MftBitmapWritesUserLevel", SimTypeRef("_MftBitmapWritesUserLevel_e__Struct", SimStruct)), ("UserIndexReads", SimTypeInt(signed=False, label="UInt32")), ("UserIndexReadBytes", SimTypeInt(signed=False, label="UInt32")), ("UserIndexWrites", SimTypeInt(signed=False, label="UInt32")), ("UserIndexWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("LogFileReads", SimTypeInt(signed=False, label="UInt32")), ("LogFileReadBytes", SimTypeInt(signed=False, label="UInt32")), ("LogFileWrites", SimTypeInt(signed=False, label="UInt32")), ("LogFileWriteBytes", SimTypeInt(signed=False, label="UInt32")), ("Allocate", SimTypeRef("_Allocate_e__Struct", SimStruct)), ("DiskResourcesExhausted", SimTypeInt(signed=False, label="UInt32")),)), name="NTFS_STATISTICS", pack=False, align=None),
    "NTFS_STATISTICS_EX": SimStruct(OrderedDict((("LogFileFullExceptions", SimTypeInt(signed=False, label="UInt32")), ("OtherExceptions", SimTypeInt(signed=False, label="UInt32")), ("MftReads", SimTypeLongLong(signed=False, label="UInt64")), ("MftReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MftWrites", SimTypeLongLong(signed=False, label="UInt64")), ("MftWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MftWritesUserLevel", SimTypeRef("_MftWritesUserLevel_e__Struct", SimStruct)), ("MftWritesFlushForLogFileFull", SimTypeInt(signed=False, label="UInt32")), ("MftWritesLazyWriter", SimTypeInt(signed=False, label="UInt32")), ("MftWritesUserRequest", SimTypeInt(signed=False, label="UInt32")), ("Mft2Writes", SimTypeLongLong(signed=False, label="UInt64")), ("Mft2WriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Mft2WritesUserLevel", SimTypeRef("_Mft2WritesUserLevel_e__Struct", SimStruct)), ("Mft2WritesFlushForLogFileFull", SimTypeInt(signed=False, label="UInt32")), ("Mft2WritesLazyWriter", SimTypeInt(signed=False, label="UInt32")), ("Mft2WritesUserRequest", SimTypeInt(signed=False, label="UInt32")), ("RootIndexReads", SimTypeLongLong(signed=False, label="UInt64")), ("RootIndexReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("RootIndexWrites", SimTypeLongLong(signed=False, label="UInt64")), ("RootIndexWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("BitmapReads", SimTypeLongLong(signed=False, label="UInt64")), ("BitmapReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("BitmapWrites", SimTypeLongLong(signed=False, label="UInt64")), ("BitmapWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("BitmapWritesFlushForLogFileFull", SimTypeInt(signed=False, label="UInt32")), ("BitmapWritesLazyWriter", SimTypeInt(signed=False, label="UInt32")), ("BitmapWritesUserRequest", SimTypeInt(signed=False, label="UInt32")), ("BitmapWritesUserLevel", SimTypeRef("_BitmapWritesUserLevel_e__Struct", SimStruct)), ("MftBitmapReads", SimTypeLongLong(signed=False, label="UInt64")), ("MftBitmapReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MftBitmapWrites", SimTypeLongLong(signed=False, label="UInt64")), ("MftBitmapWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("MftBitmapWritesFlushForLogFileFull", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapWritesLazyWriter", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapWritesUserRequest", SimTypeInt(signed=False, label="UInt32")), ("MftBitmapWritesUserLevel", SimTypeRef("_MftBitmapWritesUserLevel_e__Struct", SimStruct)), ("UserIndexReads", SimTypeLongLong(signed=False, label="UInt64")), ("UserIndexReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("UserIndexWrites", SimTypeLongLong(signed=False, label="UInt64")), ("UserIndexWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("LogFileReads", SimTypeLongLong(signed=False, label="UInt64")), ("LogFileReadBytes", SimTypeLongLong(signed=False, label="UInt64")), ("LogFileWrites", SimTypeLongLong(signed=False, label="UInt64")), ("LogFileWriteBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Allocate", SimTypeRef("_Allocate_e__Struct", SimStruct)), ("DiskResourcesExhausted", SimTypeInt(signed=False, label="UInt32")), ("VolumeTrimCount", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeTrimTime", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeTrimByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("FileLevelTrimCount", SimTypeLongLong(signed=False, label="UInt64")), ("FileLevelTrimTime", SimTypeLongLong(signed=False, label="UInt64")), ("FileLevelTrimByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeTrimSkippedCount", SimTypeLongLong(signed=False, label="UInt64")), ("VolumeTrimSkippedByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("NtfsFillStatInfoFromMftRecordCalledCount", SimTypeLongLong(signed=False, label="UInt64")), ("NtfsFillStatInfoFromMftRecordBailedBecauseOfAttributeListCount", SimTypeLongLong(signed=False, label="UInt64")), ("NtfsFillStatInfoFromMftRecordBailedBecauseOfNonResReparsePointCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="NTFS_STATISTICS_EX", pack=False, align=None),
    "NTFS_VOLUME_DATA_BUFFER": SimStruct(OrderedDict((("VolumeSerialNumber", SimTypeLongLong(signed=True, label="Int64")), ("NumberSectors", SimTypeLongLong(signed=True, label="Int64")), ("TotalClusters", SimTypeLongLong(signed=True, label="Int64")), ("FreeClusters", SimTypeLongLong(signed=True, label="Int64")), ("TotalReserved", SimTypeLongLong(signed=True, label="Int64")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("BytesPerCluster", SimTypeInt(signed=False, label="UInt32")), ("BytesPerFileRecordSegment", SimTypeInt(signed=False, label="UInt32")), ("ClustersPerFileRecordSegment", SimTypeInt(signed=False, label="UInt32")), ("MftValidDataLength", SimTypeLongLong(signed=True, label="Int64")), ("MftStartLcn", SimTypeLongLong(signed=True, label="Int64")), ("Mft2StartLcn", SimTypeLongLong(signed=True, label="Int64")), ("MftZoneStart", SimTypeLongLong(signed=True, label="Int64")), ("MftZoneEnd", SimTypeLongLong(signed=True, label="Int64")),)), name="NTFS_VOLUME_DATA_BUFFER", pack=False, align=None),
    "NTMS_ALLOCATION_INFORMATION": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocatedFrom", SimTypeBottom(label="Guid")),)), name="NTMS_ALLOCATION_INFORMATION", pack=False, align=None),
    "NTMS_ASYNC_IO": SimStruct(OrderedDict((("OperationId", SimTypeBottom(label="Guid")), ("EventId", SimTypeBottom(label="Guid")), ("dwOperationType", SimTypeInt(signed=False, label="UInt32")), ("dwResult", SimTypeInt(signed=False, label="UInt32")), ("dwAsyncState", SimTypeInt(signed=False, label="UInt32")), ("hEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("bOnStateChange", SimTypeInt(signed=True, label="Int32")),)), name="NTMS_ASYNC_IO", pack=False, align=None),
    "NTMS_CHANGERINFORMATIONA": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("ChangerType", SimTypeBottom(label="Guid")), ("szSerialNumber", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szRevision", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("ScsiPort", SimTypeShort(signed=False, label="UInt16")), ("ScsiBus", SimTypeShort(signed=False, label="UInt16")), ("ScsiTarget", SimTypeShort(signed=False, label="UInt16")), ("ScsiLun", SimTypeShort(signed=False, label="UInt16")), ("Library", SimTypeBottom(label="Guid")),)), name="NTMS_CHANGERINFORMATIONA", pack=False, align=None),
    "NTMS_CHANGERINFORMATIONW": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("ChangerType", SimTypeBottom(label="Guid")), ("szSerialNumber", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szRevision", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("ScsiPort", SimTypeShort(signed=False, label="UInt16")), ("ScsiBus", SimTypeShort(signed=False, label="UInt16")), ("ScsiTarget", SimTypeShort(signed=False, label="UInt16")), ("ScsiLun", SimTypeShort(signed=False, label="UInt16")), ("Library", SimTypeBottom(label="Guid")),)), name="NTMS_CHANGERINFORMATIONW", pack=False, align=None),
    "NTMS_CHANGERTYPEINFORMATIONA": SimStruct(OrderedDict((("szVendor", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("szProduct", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("DeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_CHANGERTYPEINFORMATIONA", pack=False, align=None),
    "NTMS_CHANGERTYPEINFORMATIONW": SimStruct(OrderedDict((("szVendor", SimTypeArray(SimTypeChar(label="Char"), 128)), ("szProduct", SimTypeArray(SimTypeChar(label="Char"), 128)), ("DeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_CHANGERTYPEINFORMATIONW", pack=False, align=None),
    "NTMS_COMPUTERINFORMATION": SimStruct(OrderedDict((("dwLibRequestPurgeTime", SimTypeInt(signed=False, label="UInt32")), ("dwOpRequestPurgeTime", SimTypeInt(signed=False, label="UInt32")), ("dwLibRequestFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOpRequestFlags", SimTypeInt(signed=False, label="UInt32")), ("dwMediaPoolPolicy", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_COMPUTERINFORMATION", pack=False, align=None),
    "NTMS_DRIVEINFORMATIONA": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("DriveType", SimTypeBottom(label="Guid")), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szSerialNumber", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szRevision", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("ScsiPort", SimTypeShort(signed=False, label="UInt16")), ("ScsiBus", SimTypeShort(signed=False, label="UInt16")), ("ScsiTarget", SimTypeShort(signed=False, label="UInt16")), ("ScsiLun", SimTypeShort(signed=False, label="UInt16")), ("dwMountCount", SimTypeInt(signed=False, label="UInt32")), ("LastCleanedTs", SimTypeRef("SYSTEMTIME", SimStruct)), ("SavedPartitionId", SimTypeBottom(label="Guid")), ("Library", SimTypeBottom(label="Guid")), ("Reserved", SimTypeBottom(label="Guid")), ("dwDeferDismountDelay", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_DRIVEINFORMATIONA", pack=False, align=None),
    "NTMS_DRIVEINFORMATIONW": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("DriveType", SimTypeBottom(label="Guid")), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szSerialNumber", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szRevision", SimTypeArray(SimTypeChar(label="Char"), 32)), ("ScsiPort", SimTypeShort(signed=False, label="UInt16")), ("ScsiBus", SimTypeShort(signed=False, label="UInt16")), ("ScsiTarget", SimTypeShort(signed=False, label="UInt16")), ("ScsiLun", SimTypeShort(signed=False, label="UInt16")), ("dwMountCount", SimTypeInt(signed=False, label="UInt32")), ("LastCleanedTs", SimTypeRef("SYSTEMTIME", SimStruct)), ("SavedPartitionId", SimTypeBottom(label="Guid")), ("Library", SimTypeBottom(label="Guid")), ("Reserved", SimTypeBottom(label="Guid")), ("dwDeferDismountDelay", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_DRIVEINFORMATIONW", pack=False, align=None),
    "NTMS_DRIVETYPEINFORMATIONA": SimStruct(OrderedDict((("szVendor", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("szProduct", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("NumberOfHeads", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="FILE_DEVICE_TYPE")),)), name="NTMS_DRIVETYPEINFORMATIONA", pack=False, align=None),
    "NTMS_DRIVETYPEINFORMATIONW": SimStruct(OrderedDict((("szVendor", SimTypeArray(SimTypeChar(label="Char"), 128)), ("szProduct", SimTypeArray(SimTypeChar(label="Char"), 128)), ("NumberOfHeads", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="FILE_DEVICE_TYPE")),)), name="NTMS_DRIVETYPEINFORMATIONW", pack=False, align=None),
    "NTMS_FILESYSTEM_INFO": SimStruct(OrderedDict((("FileSystemType", SimTypeArray(SimTypeChar(label="Char"), 64)), ("VolumeName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("SerialNumber", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_FILESYSTEM_INFO", pack=False, align=None),
    "NTMS_I1_LIBRARYINFORMATION": SimStruct(OrderedDict((("LibraryType", SimTypeInt(signed=False, label="UInt32")), ("CleanerSlot", SimTypeBottom(label="Guid")), ("CleanerSlotDefault", SimTypeBottom(label="Guid")), ("LibrarySupportsDriveCleaning", SimTypeInt(signed=True, label="Int32")), ("BarCodeReaderInstalled", SimTypeInt(signed=True, label="Int32")), ("InventoryMethod", SimTypeInt(signed=False, label="UInt32")), ("dwCleanerUsesRemaining", SimTypeInt(signed=False, label="UInt32")), ("FirstDriveNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfDrives", SimTypeInt(signed=False, label="UInt32")), ("FirstSlotNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfSlots", SimTypeInt(signed=False, label="UInt32")), ("FirstDoorNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfDoors", SimTypeInt(signed=False, label="UInt32")), ("FirstPortNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPorts", SimTypeInt(signed=False, label="UInt32")), ("FirstChangerNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfChangers", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfMedia", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfMediaTypes", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfLibRequests", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeBottom(label="Guid")),)), name="NTMS_I1_LIBRARYINFORMATION", pack=False, align=None),
    "NTMS_I1_LIBREQUESTINFORMATIONA": SimStruct(OrderedDict((("OperationCode", SimTypeInt(signed=False, label="UInt32")), ("OperationOption", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("PartitionId", SimTypeBottom(label="Guid")), ("DriveId", SimTypeBottom(label="Guid")), ("PhysMediaId", SimTypeBottom(label="Guid")), ("Library", SimTypeBottom(label="Guid")), ("SlotId", SimTypeBottom(label="Guid")), ("TimeQueued", SimTypeRef("SYSTEMTIME", SimStruct)), ("TimeCompleted", SimTypeRef("SYSTEMTIME", SimStruct)), ("szApplication", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="NTMS_I1_LIBREQUESTINFORMATIONA", pack=False, align=None),
    "NTMS_I1_LIBREQUESTINFORMATIONW": SimStruct(OrderedDict((("OperationCode", SimTypeInt(signed=False, label="UInt32")), ("OperationOption", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("PartitionId", SimTypeBottom(label="Guid")), ("DriveId", SimTypeBottom(label="Guid")), ("PhysMediaId", SimTypeBottom(label="Guid")), ("Library", SimTypeBottom(label="Guid")), ("SlotId", SimTypeBottom(label="Guid")), ("TimeQueued", SimTypeRef("SYSTEMTIME", SimStruct)), ("TimeCompleted", SimTypeRef("SYSTEMTIME", SimStruct)), ("szApplication", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="NTMS_I1_LIBREQUESTINFORMATIONW", pack=False, align=None),
    "NTMS_I1_OBJECTINFORMATIONA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("Created", SimTypeRef("SYSTEMTIME", SimStruct)), ("Modified", SimTypeRef("SYSTEMTIME", SimStruct)), ("ObjectGuid", SimTypeBottom(label="Guid")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("dwOperationalState", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 127)), ("Info", SimUnion({"Drive": SimTypeRef("NTMS_DRIVEINFORMATIONA", SimStruct), "DriveType": SimTypeRef("NTMS_DRIVETYPEINFORMATIONA", SimStruct), "Library": SimTypeRef("NTMS_I1_LIBRARYINFORMATION", SimStruct), "Changer": SimTypeRef("NTMS_CHANGERINFORMATIONA", SimStruct), "ChangerType": SimTypeRef("NTMS_CHANGERTYPEINFORMATIONA", SimStruct), "StorageSlot": SimTypeRef("NTMS_STORAGESLOTINFORMATION", SimStruct), "IEDoor": SimTypeRef("NTMS_IEDOORINFORMATION", SimStruct), "IEPort": SimTypeRef("NTMS_IEPORTINFORMATION", SimStruct), "PhysicalMedia": SimTypeRef("NTMS_I1_PMIDINFORMATIONA", SimStruct), "LogicalMedia": SimTypeRef("NTMS_LMIDINFORMATION", SimStruct), "Partition": SimTypeRef("NTMS_I1_PARTITIONINFORMATIONA", SimStruct), "MediaPool": SimTypeRef("NTMS_MEDIAPOOLINFORMATION", SimStruct), "MediaType": SimTypeRef("NTMS_MEDIATYPEINFORMATION", SimStruct), "LibRequest": SimTypeRef("NTMS_I1_LIBREQUESTINFORMATIONA", SimStruct), "OpRequest": SimTypeRef("NTMS_I1_OPREQUESTINFORMATIONA", SimStruct)}, name="<anon>", label="None")),)), name="NTMS_I1_OBJECTINFORMATIONA", pack=False, align=None),
    "NTMS_I1_OBJECTINFORMATIONW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("Created", SimTypeRef("SYSTEMTIME", SimStruct)), ("Modified", SimTypeRef("SYSTEMTIME", SimStruct)), ("ObjectGuid", SimTypeBottom(label="Guid")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("dwOperationalState", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 127)), ("Info", SimUnion({"Drive": SimTypeRef("NTMS_DRIVEINFORMATIONW", SimStruct), "DriveType": SimTypeRef("NTMS_DRIVETYPEINFORMATIONW", SimStruct), "Library": SimTypeRef("NTMS_I1_LIBRARYINFORMATION", SimStruct), "Changer": SimTypeRef("NTMS_CHANGERINFORMATIONW", SimStruct), "ChangerType": SimTypeRef("NTMS_CHANGERTYPEINFORMATIONW", SimStruct), "StorageSlot": SimTypeRef("NTMS_STORAGESLOTINFORMATION", SimStruct), "IEDoor": SimTypeRef("NTMS_IEDOORINFORMATION", SimStruct), "IEPort": SimTypeRef("NTMS_IEPORTINFORMATION", SimStruct), "PhysicalMedia": SimTypeRef("NTMS_I1_PMIDINFORMATIONW", SimStruct), "LogicalMedia": SimTypeRef("NTMS_LMIDINFORMATION", SimStruct), "Partition": SimTypeRef("NTMS_I1_PARTITIONINFORMATIONW", SimStruct), "MediaPool": SimTypeRef("NTMS_MEDIAPOOLINFORMATION", SimStruct), "MediaType": SimTypeRef("NTMS_MEDIATYPEINFORMATION", SimStruct), "LibRequest": SimTypeRef("NTMS_I1_LIBREQUESTINFORMATIONW", SimStruct), "OpRequest": SimTypeRef("NTMS_I1_OPREQUESTINFORMATIONW", SimStruct)}, name="<anon>", label="None")),)), name="NTMS_I1_OBJECTINFORMATIONW", pack=False, align=None),
    "NTMS_I1_OPREQUESTINFORMATIONA": SimStruct(OrderedDict((("Request", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("State", SimTypeInt(signed=False, label="UInt32")), ("szMessage", SimTypeArray(SimTypeChar(label="SByte"), 127)), ("Arg1Type", SimTypeInt(signed=False, label="UInt32")), ("Arg1", SimTypeBottom(label="Guid")), ("Arg2Type", SimTypeInt(signed=False, label="UInt32")), ("Arg2", SimTypeBottom(label="Guid")), ("szApplication", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="NTMS_I1_OPREQUESTINFORMATIONA", pack=False, align=None),
    "NTMS_I1_OPREQUESTINFORMATIONW": SimStruct(OrderedDict((("Request", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("State", SimTypeInt(signed=False, label="UInt32")), ("szMessage", SimTypeArray(SimTypeChar(label="Char"), 127)), ("Arg1Type", SimTypeInt(signed=False, label="UInt32")), ("Arg1", SimTypeBottom(label="Guid")), ("Arg2Type", SimTypeInt(signed=False, label="UInt32")), ("Arg2", SimTypeBottom(label="Guid")), ("szApplication", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="NTMS_I1_OPREQUESTINFORMATIONW", pack=False, align=None),
    "NTMS_I1_PARTITIONINFORMATIONA": SimStruct(OrderedDict((("PhysicalMedia", SimTypeBottom(label="Guid")), ("LogicalMedia", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Side", SimTypeShort(signed=False, label="UInt16")), ("dwOmidLabelIdLength", SimTypeInt(signed=False, label="UInt32")), ("OmidLabelId", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("szOmidLabelType", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szOmidLabelInfo", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("dwMountCount", SimTypeInt(signed=False, label="UInt32")), ("dwAllocateCount", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_I1_PARTITIONINFORMATIONA", pack=False, align=None),
    "NTMS_I1_PARTITIONINFORMATIONW": SimStruct(OrderedDict((("PhysicalMedia", SimTypeBottom(label="Guid")), ("LogicalMedia", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Side", SimTypeShort(signed=False, label="UInt16")), ("dwOmidLabelIdLength", SimTypeInt(signed=False, label="UInt32")), ("OmidLabelId", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("szOmidLabelType", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szOmidLabelInfo", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwMountCount", SimTypeInt(signed=False, label="UInt32")), ("dwAllocateCount", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_I1_PARTITIONINFORMATIONW", pack=False, align=None),
    "NTMS_I1_PMIDINFORMATIONA": SimStruct(OrderedDict((("CurrentLibrary", SimTypeBottom(label="Guid")), ("MediaPool", SimTypeBottom(label="Guid")), ("Location", SimTypeBottom(label="Guid")), ("LocationType", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeBottom(label="Guid")), ("HomeSlot", SimTypeBottom(label="Guid")), ("szBarCode", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BarCodeState", SimTypeInt(signed=False, label="UInt32")), ("szSequenceNumber", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("MediaState", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPartitions", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_I1_PMIDINFORMATIONA", pack=False, align=None),
    "NTMS_I1_PMIDINFORMATIONW": SimStruct(OrderedDict((("CurrentLibrary", SimTypeBottom(label="Guid")), ("MediaPool", SimTypeBottom(label="Guid")), ("Location", SimTypeBottom(label="Guid")), ("LocationType", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeBottom(label="Guid")), ("HomeSlot", SimTypeBottom(label="Guid")), ("szBarCode", SimTypeArray(SimTypeChar(label="Char"), 64)), ("BarCodeState", SimTypeInt(signed=False, label="UInt32")), ("szSequenceNumber", SimTypeArray(SimTypeChar(label="Char"), 32)), ("MediaState", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPartitions", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_I1_PMIDINFORMATIONW", pack=False, align=None),
    "NTMS_IEDOORINFORMATION": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("MaxOpenSecs", SimTypeShort(signed=False, label="UInt16")), ("Library", SimTypeBottom(label="Guid")),)), name="NTMS_IEDOORINFORMATION", pack=False, align=None),
    "NTMS_IEPORTINFORMATION": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("Content", SimTypeInt(signed=False, label="UInt32")), ("Position", SimTypeInt(signed=False, label="UInt32")), ("MaxExtendSecs", SimTypeShort(signed=False, label="UInt16")), ("Library", SimTypeBottom(label="Guid")),)), name="NTMS_IEPORTINFORMATION", pack=False, align=None),
    "NTMS_LIBRARYINFORMATION": SimStruct(OrderedDict((("LibraryType", SimTypeInt(signed=False, label="UInt32")), ("CleanerSlot", SimTypeBottom(label="Guid")), ("CleanerSlotDefault", SimTypeBottom(label="Guid")), ("LibrarySupportsDriveCleaning", SimTypeInt(signed=True, label="Int32")), ("BarCodeReaderInstalled", SimTypeInt(signed=True, label="Int32")), ("InventoryMethod", SimTypeInt(signed=False, label="UInt32")), ("dwCleanerUsesRemaining", SimTypeInt(signed=False, label="UInt32")), ("FirstDriveNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfDrives", SimTypeInt(signed=False, label="UInt32")), ("FirstSlotNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfSlots", SimTypeInt(signed=False, label="UInt32")), ("FirstDoorNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfDoors", SimTypeInt(signed=False, label="UInt32")), ("FirstPortNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPorts", SimTypeInt(signed=False, label="UInt32")), ("FirstChangerNumber", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfChangers", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfMedia", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfMediaTypes", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfLibRequests", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeBottom(label="Guid")), ("AutoRecovery", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_LIBRARYINFORMATION", pack=False, align=None),
    "NTMS_LIBREQUESTINFORMATIONA": SimStruct(OrderedDict((("OperationCode", SimTypeInt(signed=False, label="UInt32")), ("OperationOption", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("PartitionId", SimTypeBottom(label="Guid")), ("DriveId", SimTypeBottom(label="Guid")), ("PhysMediaId", SimTypeBottom(label="Guid")), ("Library", SimTypeBottom(label="Guid")), ("SlotId", SimTypeBottom(label="Guid")), ("TimeQueued", SimTypeRef("SYSTEMTIME", SimStruct)), ("TimeCompleted", SimTypeRef("SYSTEMTIME", SimStruct)), ("szApplication", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("dwErrorCode", SimTypeInt(signed=False, label="UInt32")), ("WorkItemId", SimTypeBottom(label="Guid")), ("dwPriority", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_LIBREQUESTINFORMATIONA", pack=False, align=None),
    "NTMS_LIBREQUESTINFORMATIONW": SimStruct(OrderedDict((("OperationCode", SimTypeInt(signed=False, label="UInt32")), ("OperationOption", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("PartitionId", SimTypeBottom(label="Guid")), ("DriveId", SimTypeBottom(label="Guid")), ("PhysMediaId", SimTypeBottom(label="Guid")), ("Library", SimTypeBottom(label="Guid")), ("SlotId", SimTypeBottom(label="Guid")), ("TimeQueued", SimTypeRef("SYSTEMTIME", SimStruct)), ("TimeCompleted", SimTypeRef("SYSTEMTIME", SimStruct)), ("szApplication", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwErrorCode", SimTypeInt(signed=False, label="UInt32")), ("WorkItemId", SimTypeBottom(label="Guid")), ("dwPriority", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_LIBREQUESTINFORMATIONW", pack=False, align=None),
    "NTMS_LMIDINFORMATION": SimStruct(OrderedDict((("MediaPool", SimTypeBottom(label="Guid")), ("dwNumberOfPartitions", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_LMIDINFORMATION", pack=False, align=None),
    "NTMS_MEDIAPOOLINFORMATION": SimStruct(OrderedDict((("PoolType", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeBottom(label="Guid")), ("Parent", SimTypeBottom(label="Guid")), ("AllocationPolicy", SimTypeInt(signed=False, label="UInt32")), ("DeallocationPolicy", SimTypeInt(signed=False, label="UInt32")), ("dwMaxAllocates", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPhysicalMedia", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfLogicalMedia", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfMediaPools", SimTypeInt(signed=False, label="UInt32")),)), name="NTMS_MEDIAPOOLINFORMATION", pack=False, align=None),
    "NTMS_MEDIATYPEINFORMATION": SimStruct(OrderedDict((("MediaType", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSides", SimTypeInt(signed=False, label="UInt32")), ("ReadWriteCharacteristics", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="FILE_DEVICE_TYPE")),)), name="NTMS_MEDIATYPEINFORMATION", pack=False, align=None),
    "NTMS_MOUNT_INFORMATION": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="NTMS_MOUNT_INFORMATION", pack=False, align=None),
    "NTMS_NOTIFICATIONINFORMATION": SimStruct(OrderedDict((("dwOperation", SimTypeInt(signed=False, label="UInt32")), ("ObjectId", SimTypeBottom(label="Guid")),)), name="NTMS_NOTIFICATIONINFORMATION", pack=False, align=None),
    "NTMS_OBJECTINFORMATIONA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("Created", SimTypeRef("SYSTEMTIME", SimStruct)), ("Modified", SimTypeRef("SYSTEMTIME", SimStruct)), ("ObjectGuid", SimTypeBottom(label="Guid")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("dwOperationalState", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 127)), ("Info", SimUnion({"Drive": SimTypeRef("NTMS_DRIVEINFORMATIONA", SimStruct), "DriveType": SimTypeRef("NTMS_DRIVETYPEINFORMATIONA", SimStruct), "Library": SimTypeRef("NTMS_LIBRARYINFORMATION", SimStruct), "Changer": SimTypeRef("NTMS_CHANGERINFORMATIONA", SimStruct), "ChangerType": SimTypeRef("NTMS_CHANGERTYPEINFORMATIONA", SimStruct), "StorageSlot": SimTypeRef("NTMS_STORAGESLOTINFORMATION", SimStruct), "IEDoor": SimTypeRef("NTMS_IEDOORINFORMATION", SimStruct), "IEPort": SimTypeRef("NTMS_IEPORTINFORMATION", SimStruct), "PhysicalMedia": SimTypeRef("NTMS_PMIDINFORMATIONA", SimStruct), "LogicalMedia": SimTypeRef("NTMS_LMIDINFORMATION", SimStruct), "Partition": SimTypeRef("NTMS_PARTITIONINFORMATIONA", SimStruct), "MediaPool": SimTypeRef("NTMS_MEDIAPOOLINFORMATION", SimStruct), "MediaType": SimTypeRef("NTMS_MEDIATYPEINFORMATION", SimStruct), "LibRequest": SimTypeRef("NTMS_LIBREQUESTINFORMATIONA", SimStruct), "OpRequest": SimTypeRef("NTMS_OPREQUESTINFORMATIONA", SimStruct), "Computer": SimTypeRef("NTMS_COMPUTERINFORMATION", SimStruct)}, name="<anon>", label="None")),)), name="NTMS_OBJECTINFORMATIONA", pack=False, align=None),
    "NTMS_OBJECTINFORMATIONW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("Created", SimTypeRef("SYSTEMTIME", SimStruct)), ("Modified", SimTypeRef("SYSTEMTIME", SimStruct)), ("ObjectGuid", SimTypeBottom(label="Guid")), ("Enabled", SimTypeInt(signed=True, label="Int32")), ("dwOperationalState", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 127)), ("Info", SimUnion({"Drive": SimTypeRef("NTMS_DRIVEINFORMATIONW", SimStruct), "DriveType": SimTypeRef("NTMS_DRIVETYPEINFORMATIONW", SimStruct), "Library": SimTypeRef("NTMS_LIBRARYINFORMATION", SimStruct), "Changer": SimTypeRef("NTMS_CHANGERINFORMATIONW", SimStruct), "ChangerType": SimTypeRef("NTMS_CHANGERTYPEINFORMATIONW", SimStruct), "StorageSlot": SimTypeRef("NTMS_STORAGESLOTINFORMATION", SimStruct), "IEDoor": SimTypeRef("NTMS_IEDOORINFORMATION", SimStruct), "IEPort": SimTypeRef("NTMS_IEPORTINFORMATION", SimStruct), "PhysicalMedia": SimTypeRef("NTMS_PMIDINFORMATIONW", SimStruct), "LogicalMedia": SimTypeRef("NTMS_LMIDINFORMATION", SimStruct), "Partition": SimTypeRef("NTMS_PARTITIONINFORMATIONW", SimStruct), "MediaPool": SimTypeRef("NTMS_MEDIAPOOLINFORMATION", SimStruct), "MediaType": SimTypeRef("NTMS_MEDIATYPEINFORMATION", SimStruct), "LibRequest": SimTypeRef("NTMS_LIBREQUESTINFORMATIONW", SimStruct), "OpRequest": SimTypeRef("NTMS_OPREQUESTINFORMATIONW", SimStruct), "Computer": SimTypeRef("NTMS_COMPUTERINFORMATION", SimStruct)}, name="<anon>", label="None")),)), name="NTMS_OBJECTINFORMATIONW", pack=False, align=None),
    "NTMS_OMID_TYPE": SimTypeInt(signed=False, label="NTMS_OMID_TYPE"),
    "NTMS_OPREQUESTINFORMATIONA": SimStruct(OrderedDict((("Request", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("State", SimTypeInt(signed=False, label="UInt32")), ("szMessage", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Arg1Type", SimTypeInt(signed=False, label="UInt32")), ("Arg1", SimTypeBottom(label="Guid")), ("Arg2Type", SimTypeInt(signed=False, label="UInt32")), ("Arg2", SimTypeBottom(label="Guid")), ("szApplication", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="NTMS_OPREQUESTINFORMATIONA", pack=False, align=None),
    "NTMS_OPREQUESTINFORMATIONW": SimStruct(OrderedDict((("Request", SimTypeInt(signed=False, label="UInt32")), ("Submitted", SimTypeRef("SYSTEMTIME", SimStruct)), ("State", SimTypeInt(signed=False, label="UInt32")), ("szMessage", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Arg1Type", SimTypeInt(signed=False, label="UInt32")), ("Arg1", SimTypeBottom(label="Guid")), ("Arg2Type", SimTypeInt(signed=False, label="UInt32")), ("Arg2", SimTypeBottom(label="Guid")), ("szApplication", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szUser", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szComputer", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="NTMS_OPREQUESTINFORMATIONW", pack=False, align=None),
    "NTMS_PARTITIONINFORMATIONA": SimStruct(OrderedDict((("PhysicalMedia", SimTypeBottom(label="Guid")), ("LogicalMedia", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Side", SimTypeShort(signed=False, label="UInt16")), ("dwOmidLabelIdLength", SimTypeInt(signed=False, label="UInt32")), ("OmidLabelId", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("szOmidLabelType", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("szOmidLabelInfo", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("dwMountCount", SimTypeInt(signed=False, label="UInt32")), ("dwAllocateCount", SimTypeInt(signed=False, label="UInt32")), ("Capacity", SimTypeLongLong(signed=True, label="Int64")),)), name="NTMS_PARTITIONINFORMATIONA", pack=False, align=None),
    "NTMS_PARTITIONINFORMATIONW": SimStruct(OrderedDict((("PhysicalMedia", SimTypeBottom(label="Guid")), ("LogicalMedia", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Side", SimTypeShort(signed=False, label="UInt16")), ("dwOmidLabelIdLength", SimTypeInt(signed=False, label="UInt32")), ("OmidLabelId", SimTypeArray(SimTypeChar(label="Byte"), 255)), ("szOmidLabelType", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szOmidLabelInfo", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwMountCount", SimTypeInt(signed=False, label="UInt32")), ("dwAllocateCount", SimTypeInt(signed=False, label="UInt32")), ("Capacity", SimTypeLongLong(signed=True, label="Int64")),)), name="NTMS_PARTITIONINFORMATIONW", pack=False, align=None),
    "NTMS_PMIDINFORMATIONA": SimStruct(OrderedDict((("CurrentLibrary", SimTypeBottom(label="Guid")), ("MediaPool", SimTypeBottom(label="Guid")), ("Location", SimTypeBottom(label="Guid")), ("LocationType", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeBottom(label="Guid")), ("HomeSlot", SimTypeBottom(label="Guid")), ("szBarCode", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("BarCodeState", SimTypeInt(signed=False, label="UInt32")), ("szSequenceNumber", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("MediaState", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPartitions", SimTypeInt(signed=False, label="UInt32")), ("dwMediaTypeCode", SimTypeInt(signed=False, label="UInt32")), ("dwDensityCode", SimTypeInt(signed=False, label="UInt32")), ("MountedPartition", SimTypeBottom(label="Guid")),)), name="NTMS_PMIDINFORMATIONA", pack=False, align=None),
    "NTMS_PMIDINFORMATIONW": SimStruct(OrderedDict((("CurrentLibrary", SimTypeBottom(label="Guid")), ("MediaPool", SimTypeBottom(label="Guid")), ("Location", SimTypeBottom(label="Guid")), ("LocationType", SimTypeInt(signed=False, label="UInt32")), ("MediaType", SimTypeBottom(label="Guid")), ("HomeSlot", SimTypeBottom(label="Guid")), ("szBarCode", SimTypeArray(SimTypeChar(label="Char"), 64)), ("BarCodeState", SimTypeInt(signed=False, label="UInt32")), ("szSequenceNumber", SimTypeArray(SimTypeChar(label="Char"), 32)), ("MediaState", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPartitions", SimTypeInt(signed=False, label="UInt32")), ("dwMediaTypeCode", SimTypeInt(signed=False, label="UInt32")), ("dwDensityCode", SimTypeInt(signed=False, label="UInt32")), ("MountedPartition", SimTypeBottom(label="Guid")),)), name="NTMS_PMIDINFORMATIONW", pack=False, align=None),
    "NTMS_STORAGESLOTINFORMATION": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Library", SimTypeBottom(label="Guid")),)), name="NTMS_STORAGESLOTINFORMATION", pack=False, align=None),
    "NTSAMPROPERTIES": SimTypeInt(signed=False, label="NTSAMPROPERTIES"),
    "NTSCSI_UNICODE_STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="NTSCSI_UNICODE_STRING", pack=False, align=None),
    "NTSTATUS": SimTypeInt(signed=True, label="Int32"),
    "NTSTATUS_FACILITY_CODE": SimTypeInt(signed=False, label="NTSTATUS_FACILITY_CODE"),
    "NTSTATUS_SEVERITY_CODE": SimTypeInt(signed=False, label="NTSTATUS_SEVERITY_CODE"),
    "NT_CONSOLE_PROPS": SimStruct(OrderedDict((("dbh", SimTypeRef("DATABLOCK_HEADER", SimStruct)), ("wFillAttribute", SimTypeShort(signed=False, label="UInt16")), ("wPopupFillAttribute", SimTypeShort(signed=False, label="UInt16")), ("dwScreenBufferSize", SimTypeRef("COORD", SimStruct)), ("dwWindowSize", SimTypeRef("COORD", SimStruct)), ("dwWindowOrigin", SimTypeRef("COORD", SimStruct)), ("nFont", SimTypeInt(signed=False, label="UInt32")), ("nInputBufferSize", SimTypeInt(signed=False, label="UInt32")), ("dwFontSize", SimTypeRef("COORD", SimStruct)), ("uFontFamily", SimTypeInt(signed=False, label="UInt32")), ("uFontWeight", SimTypeInt(signed=False, label="UInt32")), ("FaceName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("uCursorSize", SimTypeInt(signed=False, label="UInt32")), ("bFullScreen", SimTypeInt(signed=True, label="Int32")), ("bQuickEdit", SimTypeInt(signed=True, label="Int32")), ("bInsertMode", SimTypeInt(signed=True, label="Int32")), ("bAutoPosition", SimTypeInt(signed=True, label="Int32")), ("uHistoryBufferSize", SimTypeInt(signed=False, label="UInt32")), ("uNumberOfHistoryBuffers", SimTypeInt(signed=False, label="UInt32")), ("bHistoryNoDup", SimTypeInt(signed=True, label="Int32")), ("ColorTable", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)),)), name="NT_CONSOLE_PROPS", pack=False, align=None),
    "NT_FE_CONSOLE_PROPS": SimStruct(OrderedDict((("dbh", SimTypeRef("DATABLOCK_HEADER", SimStruct)), ("uCodePage", SimTypeInt(signed=False, label="UInt32")),)), name="NT_FE_CONSOLE_PROPS", pack=False, align=None),
    "NT_PRODUCT_TYPE": SimTypeInt(signed=False, label="NT_PRODUCT_TYPE"),
    "NT_TIB": SimStruct(OrderedDict((("ExceptionList", SimTypePointer(SimTypeRef("EXCEPTION_REGISTRATION_RECORD", SimStruct), offset=0)), ("StackBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StackLimit", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SubSystemTib", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Anonymous", SimUnion({"FiberData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Version": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ArbitraryUserPointer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Self", SimTypePointer(SimTypeRef("NT_TIB", SimStruct), offset=0)),)), name="NT_TIB", pack=False, align=None),
    "NT_TIB32": SimStruct(OrderedDict((("ExceptionList", SimTypeInt(signed=False, label="UInt32")), ("StackBase", SimTypeInt(signed=False, label="UInt32")), ("StackLimit", SimTypeInt(signed=False, label="UInt32")), ("SubSystemTib", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"FiberData": SimTypeInt(signed=False, label="UInt32"), "Version": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ArbitraryUserPointer", SimTypeInt(signed=False, label="UInt32")), ("Self", SimTypeInt(signed=False, label="UInt32")),)), name="NT_TIB32", pack=False, align=None),
    "NT_TIB64": SimStruct(OrderedDict((("ExceptionList", SimTypeLongLong(signed=False, label="UInt64")), ("StackBase", SimTypeLongLong(signed=False, label="UInt64")), ("StackLimit", SimTypeLongLong(signed=False, label="UInt64")), ("SubSystemTib", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"FiberData": SimTypeLongLong(signed=False, label="UInt64"), "Version": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ArbitraryUserPointer", SimTypeLongLong(signed=False, label="UInt64")), ("Self", SimTypeLongLong(signed=False, label="UInt64")),)), name="NT_TIB64", pack=False, align=None),
    "NULL_FILTER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "NUMA_NODE_RELATIONSHIP": SimStruct(OrderedDict((("NodeNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 18)), ("GroupCount", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"GroupMask": SimTypeRef("GROUP_AFFINITY", SimStruct), "GroupMasks": SimTypePointer(SimTypeRef("GROUP_AFFINITY", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="NUMA_NODE_RELATIONSHIP", pack=False, align=None),
    "NUMBERFMTA": SimStruct(OrderedDict((("NumDigits", SimTypeInt(signed=False, label="UInt32")), ("LeadingZero", SimTypeInt(signed=False, label="UInt32")), ("Grouping", SimTypeInt(signed=False, label="UInt32")), ("lpDecimalSep", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpThousandSep", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("NegativeOrder", SimTypeInt(signed=False, label="UInt32")),)), name="NUMBERFMTA", pack=False, align=None),
    "NUMBERFMTW": SimStruct(OrderedDict((("NumDigits", SimTypeInt(signed=False, label="UInt32")), ("LeadingZero", SimTypeInt(signed=False, label="UInt32")), ("Grouping", SimTypeInt(signed=False, label="UInt32")), ("lpDecimalSep", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpThousandSep", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NegativeOrder", SimTypeInt(signed=False, label="UInt32")),)), name="NUMBERFMTW", pack=False, align=None),
    "NUMPARSE": SimStruct(OrderedDict((("cDig", SimTypeInt(signed=True, label="Int32")), ("dwInFlags", SimTypeInt(signed=False, label="NUMPARSE_FLAGS")), ("dwOutFlags", SimTypeInt(signed=False, label="NUMPARSE_FLAGS")), ("cchUsed", SimTypeInt(signed=True, label="Int32")), ("nBaseShift", SimTypeInt(signed=True, label="Int32")), ("nPwr10", SimTypeInt(signed=True, label="Int32")),)), name="NUMPARSE", pack=False, align=None),
    "NUMPARSE_FLAGS": SimTypeInt(signed=False, label="NUMPARSE_FLAGS"),
    "NVCACHE_HINT_PAYLOAD": SimStruct(OrderedDict((("Command", SimTypeChar(label="Byte")), ("Feature7_0", SimTypeChar(label="Byte")), ("Feature15_8", SimTypeChar(label="Byte")), ("Count15_8", SimTypeChar(label="Byte")), ("LBA7_0", SimTypeChar(label="Byte")), ("LBA15_8", SimTypeChar(label="Byte")), ("LBA23_16", SimTypeChar(label="Byte")), ("LBA31_24", SimTypeChar(label="Byte")), ("LBA39_32", SimTypeChar(label="Byte")), ("LBA47_40", SimTypeChar(label="Byte")), ("Auxiliary7_0", SimTypeChar(label="Byte")), ("Auxiliary23_16", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="NVCACHE_HINT_PAYLOAD", pack=False, align=None),
    "NVCACHE_PRIORITY_LEVEL_DESCRIPTOR": SimStruct(OrderedDict((("PriorityLevel", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ConsumedNVMSizeFraction", SimTypeInt(signed=False, label="UInt32")), ("ConsumedMappingResourcesFraction", SimTypeInt(signed=False, label="UInt32")), ("ConsumedNVMSizeForDirtyDataFraction", SimTypeInt(signed=False, label="UInt32")), ("ConsumedMappingResourcesForDirtyDataFraction", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="NVCACHE_PRIORITY_LEVEL_DESCRIPTOR", pack=False, align=None),
    "NVCACHE_REQUEST_BLOCK": SimStruct(OrderedDict((("NRBSize", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeShort(signed=False, label="UInt16")), ("NRBFlags", SimTypeInt(signed=False, label="UInt32")), ("NRBStatus", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("LBA", SimTypeLongLong(signed=False, label="UInt64")), ("DataBufSize", SimTypeInt(signed=False, label="UInt32")), ("NVCacheStatus", SimTypeInt(signed=False, label="UInt32")), ("NVCacheSubStatus", SimTypeInt(signed=False, label="UInt32")),)), name="NVCACHE_REQUEST_BLOCK", pack=False, align=None),
    "NVCACHE_STATUS": SimTypeInt(signed=False, label="NVCACHE_STATUS"),
    "NVCACHE_TYPE": SimTypeInt(signed=False, label="NVCACHE_TYPE"),
    "NVME_ACCESS_FREQUENCIES": SimTypeInt(signed=False, label="NVME_ACCESS_FREQUENCIES"),
    "NVME_ACCESS_LATENCIES": SimTypeInt(signed=False, label="NVME_ACCESS_LATENCIES"),
    "NVME_ACTIVE_NAMESPACE_ID_LIST": SimStruct(OrderedDict((("NSID", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 1024)),)), name="NVME_ACTIVE_NAMESPACE_ID_LIST", pack=False, align=None),
    "NVME_ADMIN_COMMANDS": SimTypeInt(signed=False, label="NVME_ADMIN_COMMANDS"),
    "NVME_ADMIN_COMPLETION_QUEUE_BASE_ADDRESS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "NVME_ADMIN_QUEUE_ATTRIBUTES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_ADMIN_SUBMISSION_QUEUE_BASE_ADDRESS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "NVME_AMS_OPTION": SimTypeInt(signed=False, label="NVME_AMS_OPTION"),
    "NVME_ASYNC_EVENT_ERROR_STATUS_CODES": SimTypeInt(signed=False, label="NVME_ASYNC_EVENT_ERROR_STATUS_CODES"),
    "NVME_ASYNC_EVENT_HEALTH_STATUS_CODES": SimTypeInt(signed=False, label="NVME_ASYNC_EVENT_HEALTH_STATUS_CODES"),
    "NVME_ASYNC_EVENT_IO_COMMAND_SET_STATUS_CODES": SimTypeInt(signed=False, label="NVME_ASYNC_EVENT_IO_COMMAND_SET_STATUS_CODES"),
    "NVME_ASYNC_EVENT_NOTICE_CODES": SimTypeInt(signed=False, label="NVME_ASYNC_EVENT_NOTICE_CODES"),
    "NVME_ASYNC_EVENT_TYPES": SimTypeInt(signed=False, label="NVME_ASYNC_EVENT_TYPES"),
    "NVME_ASYNC_EVENT_TYPE_VENDOR_SPECIFIC_CODES": SimTypeInt(signed=False, label="NVME_ASYNC_EVENT_TYPE_VENDOR_SPECIFIC_CODES"),
    "NVME_AUTO_POWER_STATE_TRANSITION_ENTRY": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_AUTO_POWER_STATE_TRANSITION_ENTRY", pack=False, align=None),
    "NVME_CC_SHN_SHUTDOWN_NOTIFICATIONS": SimTypeInt(signed=False, label="NVME_CC_SHN_SHUTDOWN_NOTIFICATIONS"),
    "NVME_CDW0_FEATURE_ENABLE_IEEE1667_SILO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW0_FEATURE_ERROR_INJECTION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW0_FEATURE_READONLY_WRITETHROUGH_MODE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW0_RESERVATION_PERSISTENCE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW0_RESERVATION_PERSISTENCE", pack=False, align=None),
    "NVME_CDW10_ABORT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_CREATE_IO_QUEUE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_DATASET_MANAGEMENT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_DIRECTIVE_RECEIVE": SimStruct(OrderedDict((("NUMD", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW10_DIRECTIVE_RECEIVE", pack=False, align=None),
    "NVME_CDW10_DIRECTIVE_SEND": SimStruct(OrderedDict((("NUMD", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW10_DIRECTIVE_SEND", pack=False, align=None),
    "NVME_CDW10_FIRMWARE_ACTIVATE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_FIRMWARE_DOWNLOAD": SimStruct(OrderedDict((("NUMD", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW10_FIRMWARE_DOWNLOAD", pack=False, align=None),
    "NVME_CDW10_FORMAT_NVM": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_GET_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_GET_LOG_PAGE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_GET_LOG_PAGE_V13": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_IDENTIFY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_RESERVATION_ACQUIRE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_RESERVATION_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_RESERVATION_RELEASE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_RESERVATION_REPORT": SimUnion({"Anonymous": SimStruct(OrderedDict((("NUMD", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_SANITIZE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_SECURITY_SEND_RECEIVE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_SET_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW10_ZONE_APPEND": SimStruct(OrderedDict((("SLBA", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_CDW10_ZONE_APPEND", pack=False, align=None),
    "NVME_CDW10_ZONE_MANAGEMENT_RECEIVE": SimStruct(OrderedDict((("SLBA", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_CDW10_ZONE_MANAGEMENT_RECEIVE", pack=False, align=None),
    "NVME_CDW10_ZONE_MANAGEMENT_SEND": SimStruct(OrderedDict((("SLBA", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_CDW10_ZONE_MANAGEMENT_SEND", pack=False, align=None),
    "NVME_CDW11_CREATE_IO_CQ": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_CREATE_IO_SQ": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_DATASET_MANAGEMENT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_DIRECTIVE_RECEIVE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_DIRECTIVE_SEND": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURES": SimUnion({"NumberOfQueues": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "InterruptCoalescing": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "InterruptVectorConfig": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "LbaRangeType": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Arbitration": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VolatileWriteCache": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsyncEventConfig": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PowerManagement": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AutoPowerStateTransition": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "TemperatureThreshold": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "ErrorRecovery": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "WriteAtomicityNormal": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "NonOperationalPowerState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "IoCommandSetProfile": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "ErrorInjection": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "HostIdentifier": SimTypeRef("NVME_CDW11_FEATURE_HOST_IDENTIFIER", SimStruct), "ReservationPersistence": SimTypeRef("NVME_CDW11_FEATURE_RESERVATION_PERSISTENCE", SimStruct), "ReservationNotificationMask": SimTypeRef("NVME_CDW11_FEATURE_RESERVATION_NOTIFICATION_MASK", SimStruct), "GetHostMetadata": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "SetHostMetadata": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_ARBITRATION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_ASYNC_EVENT_CONFIG": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_AUTO_POWER_STATE_TRANSITION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_CLEAR_FW_UPDATE_HISTORY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_CLEAR_PCIE_CORRECTABLE_ERROR_COUNTERS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_ENABLE_IEEE1667_SILO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_ERROR_RECOVERY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_GET_HOST_METADATA": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_HOST_IDENTIFIER": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW11_FEATURE_HOST_IDENTIFIER", pack=False, align=None),
    "NVME_CDW11_FEATURE_HOST_MEMORY_BUFFER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_INTERRUPT_COALESCING": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_INTERRUPT_VECTOR_CONFIG": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_IO_COMMAND_SET_PROFILE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_LBA_RANGE_TYPE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_NON_OPERATIONAL_POWER_STATE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_NUMBER_OF_QUEUES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_POWER_MANAGEMENT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_READONLY_WRITETHROUGH_MODE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_RESERVATION_NOTIFICATION_MASK": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW11_FEATURE_RESERVATION_NOTIFICATION_MASK", pack=False, align=None),
    "NVME_CDW11_FEATURE_RESERVATION_PERSISTENCE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW11_FEATURE_RESERVATION_PERSISTENCE", pack=False, align=None),
    "NVME_CDW11_FEATURE_SET_HOST_METADATA": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_SUPPORTED_CAPABILITY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_TEMPERATURE_THRESHOLD": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_VOLATILE_WRITE_CACHE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FEATURE_WRITE_ATOMICITY_NORMAL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_FIRMWARE_DOWNLOAD": SimStruct(OrderedDict((("OFST", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW11_FIRMWARE_DOWNLOAD", pack=False, align=None),
    "NVME_CDW11_GET_LOG_PAGE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_IDENTIFY": SimUnion({"Anonymous1": SimStruct(OrderedDict((("NVMSETID", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_RESERVATION_REPORT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_SANITIZE": SimUnion({"Anonymous": SimStruct(OrderedDict((("OVRPAT", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW11_SECURITY_RECEIVE": SimStruct(OrderedDict((("AL", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW11_SECURITY_RECEIVE", pack=False, align=None),
    "NVME_CDW11_SECURITY_SEND": SimStruct(OrderedDict((("TL", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW11_SECURITY_SEND", pack=False, align=None),
    "NVME_CDW12_DIRECTIVE_RECEIVE": SimUnion({"AllocateResources": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_DIRECTIVE_RECEIVE_STREAMS_ALLOCATE_RESOURCES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_DIRECTIVE_SEND": SimUnion({"EnableDirective": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_DIRECTIVE_SEND_IDENTIFY_ENABLE_DIRECTIVE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_FEATURES": SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("HSIZE", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_FEATURE_HOST_MEMORY_BUFFER": SimUnion({"Anonymous": SimStruct(OrderedDict((("HSIZE", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_GET_LOG_PAGE": SimStruct(OrderedDict((("LPOL", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW12_GET_LOG_PAGE", pack=False, align=None),
    "NVME_CDW12_READ_WRITE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW12_ZONE_APPEND": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW13_FEATURES": SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW13_FEATURE_HOST_MEMORY_BUFFER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW13_GET_LOG_PAGE": SimStruct(OrderedDict((("LPOU", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_CDW13_GET_LOG_PAGE", pack=False, align=None),
    "NVME_CDW13_READ_WRITE": SimUnion({"Anonymous": SimStruct(OrderedDict((("DSM", SimTypeRef("_DSM_e__Struct", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("DSPEC", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW13_ZONE_MANAGEMENT_RECEIVE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW13_ZONE_MANAGEMENT_SEND": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW14_FEATURES": SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("HMDLUA", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW14_FEATURE_HOST_MEMORY_BUFFER": SimUnion({"Anonymous": SimStruct(OrderedDict((("HMDLUA", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW14_GET_LOG_PAGE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW15_FEATURES": SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("HMDLEC", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW15_FEATURE_HOST_MEMORY_BUFFER": SimUnion({"Anonymous": SimStruct(OrderedDict((("HMDLEC", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW15_READ_WRITE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CDW15_ZONE_APPEND": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CHANGED_NAMESPACE_LIST_LOG": SimStruct(OrderedDict((("NSID", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 1024)),)), name="NVME_CHANGED_NAMESPACE_LIST_LOG", pack=False, align=None),
    "NVME_CHANGED_ZONE_LIST_LOG": SimStruct(OrderedDict((("ZoneIdentifiersCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("ZoneIdentifier", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 511)),)), name="NVME_CHANGED_ZONE_LIST_LOG", pack=False, align=None),
    "NVME_CMBSZ_SIZE_UNITS": SimTypeInt(signed=False, label="NVME_CMBSZ_SIZE_UNITS"),
    "NVME_COMMAND": SimStruct(OrderedDict((("CDW0", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("NSID", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("MPTR", SimTypeLongLong(signed=False, label="UInt64")), ("PRP1", SimTypeLongLong(signed=False, label="UInt64")), ("PRP2", SimTypeLongLong(signed=False, label="UInt64")), ("u", SimUnion({"GENERAL": SimTypeRef("_GENERAL_e__Struct", SimStruct), "IDENTIFY": SimTypeRef("_IDENTIFY_e__Struct", SimStruct), "ABORT": SimTypeRef("_ABORT_e__Struct", SimStruct), "GETFEATURES": SimTypeRef("_GETFEATURES_e__Struct", SimStruct), "SETFEATURES": SimTypeRef("_SETFEATURES_e__Struct", SimStruct), "GETLOGPAGE": SimTypeRef("_GETLOGPAGE_e__Struct", SimStruct), "CREATEIOCQ": SimTypeRef("_CREATEIOCQ_e__Struct", SimStruct), "CREATEIOSQ": SimTypeRef("_CREATEIOSQ_e__Struct", SimStruct), "DATASETMANAGEMENT": SimTypeRef("_DATASETMANAGEMENT_e__Struct", SimStruct), "SECURITYSEND": SimTypeRef("_SECURITYSEND_e__Struct", SimStruct), "SECURITYRECEIVE": SimTypeRef("_SECURITYRECEIVE_e__Struct", SimStruct), "FIRMWAREDOWNLOAD": SimTypeRef("_FIRMWAREDOWNLOAD_e__Struct", SimStruct), "FIRMWAREACTIVATE": SimTypeRef("_FIRMWAREACTIVATE_e__Struct", SimStruct), "FORMATNVM": SimTypeRef("_FORMATNVM_e__Struct", SimStruct), "DIRECTIVERECEIVE": SimTypeRef("_DIRECTIVERECEIVE_e__Struct", SimStruct), "DIRECTIVESEND": SimTypeRef("_DIRECTIVESEND_e__Struct", SimStruct), "SANITIZE": SimTypeRef("_SANITIZE_e__Struct", SimStruct), "READWRITE": SimTypeRef("_READWRITE_e__Struct", SimStruct), "RESERVATIONACQUIRE": SimTypeRef("_RESERVATIONACQUIRE_e__Struct", SimStruct), "RESERVATIONREGISTER": SimTypeRef("_RESERVATIONREGISTER_e__Struct", SimStruct), "RESERVATIONRELEASE": SimTypeRef("_RESERVATIONRELEASE_e__Struct", SimStruct), "RESERVATIONREPORT": SimTypeRef("_RESERVATIONREPORT_e__Struct", SimStruct), "ZONEMANAGEMENTSEND": SimTypeRef("_ZONEMANAGEMENTSEND_e__Struct", SimStruct), "ZONEMANAGEMENTRECEIVE": SimTypeRef("_ZONEMANAGEMENTRECEIVE_e__Struct", SimStruct), "ZONEAPPEND": SimTypeRef("_ZONEAPPEND_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="NVME_COMMAND", pack=False, align=None),
    "NVME_COMMAND_DWORD0": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_COMMAND_EFFECTS_DATA": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_COMMAND_EFFECTS_LOG": SimStruct(OrderedDict((("ACS", SimTypeArray(SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), 256)), ("IOCS", SimTypeArray(SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), 256)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2048)),)), name="NVME_COMMAND_EFFECTS_LOG", pack=False, align=None),
    "NVME_COMMAND_EFFECT_SBUMISSION_EXECUTION_LIMITS": SimTypeInt(signed=False, label="NVME_COMMAND_EFFECT_SBUMISSION_EXECUTION_LIMITS"),
    "NVME_COMMAND_SET_IDENTIFIERS": SimTypeInt(signed=False, label="NVME_COMMAND_SET_IDENTIFIERS"),
    "NVME_COMMAND_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "NVME_COMPLETION_DW0_ASYNC_EVENT_REQUEST": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_COMPLETION_DW0_ASYNC_EVENT_REQUEST", pack=False, align=None),
    "NVME_COMPLETION_DW0_DIRECTIVE_RECEIVE_STREAMS_ALLOCATE_RESOURCES": SimStruct(OrderedDict((("Anonymous", SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)), ("AsUlong", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_COMPLETION_DW0_DIRECTIVE_RECEIVE_STREAMS_ALLOCATE_RESOURCES", pack=False, align=None),
    "NVME_COMPLETION_ENTRY": SimStruct(OrderedDict((("DW0", SimTypeInt(signed=False, label="UInt32")), ("DW1", SimTypeInt(signed=False, label="UInt32")), ("DW2", SimUnion({"Anonymous": SimStruct(OrderedDict((("SQHD", SimTypeShort(signed=False, label="UInt16")), ("SQID", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DW3", SimUnion({"Anonymous": SimStruct(OrderedDict((("CID", SimTypeShort(signed=False, label="UInt16")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="NVME_COMPLETION_ENTRY", pack=False, align=None),
    "NVME_COMPLETION_QUEUE_HEAD_DOORBELL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CONTEXT_ATTRIBUTES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CONTROLLER_CAPABILITIES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "NVME_CONTROLLER_CONFIGURATION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CONTROLLER_LIST": SimStruct(OrderedDict((("NumberOfIdentifiers", SimTypeShort(signed=False, label="UInt16")), ("ControllerID", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2047)),)), name="NVME_CONTROLLER_LIST", pack=False, align=None),
    "NVME_CONTROLLER_MEMORY_BUFFER_LOCATION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CONTROLLER_MEMORY_BUFFER_SIZE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CONTROLLER_METADATA_ELEMENT_TYPES": SimTypeInt(signed=False, label="NVME_CONTROLLER_METADATA_ELEMENT_TYPES"),
    "NVME_CONTROLLER_REGISTERS": SimStruct(OrderedDict((("CAP", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("VS", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("INTMS", SimTypeInt(signed=False, label="UInt32")), ("INTMC", SimTypeInt(signed=False, label="UInt32")), ("CC", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("CSTS", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("NSSR", SimTypeRef("NVME_NVM_SUBSYSTEM_RESET", SimStruct)), ("AQA", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ASQ", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ACQ", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("CMBLOC", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CMBSZ", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 944)), ("Reserved3", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("Doorbells", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="NVME_CONTROLLER_REGISTERS", pack=False, align=None),
    "NVME_CONTROLLER_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_CSS_COMMAND_SETS": SimTypeInt(signed=False, label="NVME_CSS_COMMAND_SETS"),
    "NVME_CSTS_SHST_SHUTDOWN_STATUS": SimTypeInt(signed=False, label="NVME_CSTS_SHST_SHUTDOWN_STATUS"),
    "NVME_DEVICE_SELF_TEST_LOG": SimStruct(OrderedDict((("CurrentOperation", SimTypeRef("_CurrentOperation_e__Struct", SimStruct)), ("CurrentCompletion", SimTypeRef("_CurrentCompletion_e__Struct", SimStruct)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("ResultData", SimTypeArray(SimTypeRef("NVME_DEVICE_SELF_TEST_RESULT_DATA", SimStruct), 20)),)), name="NVME_DEVICE_SELF_TEST_LOG", pack=False, align=None),
    "NVME_DEVICE_SELF_TEST_RESULT_DATA": SimStruct(OrderedDict((("Status", SimTypeRef("_Status_e__Struct", SimStruct)), ("SegmentNumber", SimTypeChar(label="Byte")), ("ValidDiagnostics", SimTypeRef("_ValidDiagnostics_e__Struct", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("POH", SimTypeLongLong(signed=False, label="UInt64")), ("NSID", SimTypeInt(signed=False, label="UInt32")), ("FailingLBA", SimTypeLongLong(signed=False, label="UInt64")), ("StatusCodeType", SimTypeRef("_StatusCodeType_e__Struct", SimStruct)), ("StatusCode", SimTypeChar(label="Byte")), ("VendorSpecific", SimTypeShort(signed=False, label="UInt16")),)), name="NVME_DEVICE_SELF_TEST_RESULT_DATA", pack=False, align=None),
    "NVME_DIRECTIVE_IDENTIFY_RETURN_PARAMETERS": SimStruct(OrderedDict((("DirectivesSupported", SimTypeRef("NVME_DIRECTIVE_IDENTIFY_RETURN_PARAMETERS_DESCRIPTOR", SimStruct)), ("DirectivesEnabled", SimTypeRef("NVME_DIRECTIVE_IDENTIFY_RETURN_PARAMETERS_DESCRIPTOR", SimStruct)),)), name="NVME_DIRECTIVE_IDENTIFY_RETURN_PARAMETERS", pack=False, align=None),
    "NVME_DIRECTIVE_IDENTIFY_RETURN_PARAMETERS_DESCRIPTOR": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 31)),)), name="NVME_DIRECTIVE_IDENTIFY_RETURN_PARAMETERS_DESCRIPTOR", pack=False, align=None),
    "NVME_DIRECTIVE_RECEIVE_IDENTIFY_OPERATIONS": SimTypeInt(signed=False, label="NVME_DIRECTIVE_RECEIVE_IDENTIFY_OPERATIONS"),
    "NVME_DIRECTIVE_RECEIVE_STREAMS_OPERATIONS": SimTypeInt(signed=False, label="NVME_DIRECTIVE_RECEIVE_STREAMS_OPERATIONS"),
    "NVME_DIRECTIVE_SEND_IDENTIFY_OPERATIONS": SimTypeInt(signed=False, label="NVME_DIRECTIVE_SEND_IDENTIFY_OPERATIONS"),
    "NVME_DIRECTIVE_SEND_STREAMS_OPERATIONS": SimTypeInt(signed=False, label="NVME_DIRECTIVE_SEND_STREAMS_OPERATIONS"),
    "NVME_DIRECTIVE_STREAMS_GET_STATUS_DATA": SimStruct(OrderedDict((("OpenStreamCount", SimTypeShort(signed=False, label="UInt16")), ("StreamIdentifiers", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 65535)),)), name="NVME_DIRECTIVE_STREAMS_GET_STATUS_DATA", pack=False, align=None),
    "NVME_DIRECTIVE_STREAMS_RETURN_PARAMETERS": SimStruct(OrderedDict((("MSL", SimTypeShort(signed=False, label="UInt16")), ("NSSA", SimTypeShort(signed=False, label="UInt16")), ("NSSO", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 10)), ("SWS", SimTypeInt(signed=False, label="UInt32")), ("SGS", SimTypeShort(signed=False, label="UInt16")), ("NSA", SimTypeShort(signed=False, label="UInt16")), ("NSO", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="NVME_DIRECTIVE_STREAMS_RETURN_PARAMETERS", pack=False, align=None),
    "NVME_DIRECTIVE_TYPES": SimTypeInt(signed=False, label="NVME_DIRECTIVE_TYPES"),
    "NVME_ENDURANCE_GROUP_LOG": SimStruct(OrderedDict((("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("AvailableSpareThreshold", SimTypeChar(label="Byte")), ("PercentageUsed", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 26)), ("EnduranceEstimate", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("DataUnitsRead", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("DataUnitsWritten", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("MediaUnitsWritten", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 416)),)), name="NVME_ENDURANCE_GROUP_LOG", pack=False, align=None),
    "NVME_ERROR_INFO_LOG": SimStruct(OrderedDict((("ErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("SQID", SimTypeShort(signed=False, label="UInt16")), ("CMDID", SimTypeShort(signed=False, label="UInt16")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("ParameterErrorLocation", SimTypeRef("_ParameterErrorLocation_e__Struct", SimStruct)), ("Lba", SimTypeLongLong(signed=False, label="UInt64")), ("NameSpace", SimTypeInt(signed=False, label="UInt32")), ("VendorInfoAvailable", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("CommandSpecificInfo", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 24)),)), name="NVME_ERROR_INFO_LOG", pack=False, align=None),
    "NVME_ERROR_INJECTION_ENTRY": SimStruct(OrderedDict((("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Reserved1", SimTypeChar(label="Byte")), ("ErrorInjectionType", SimTypeShort(signed=False, label="UInt16")), ("ErrorInjectionTypeSpecific", SimTypeArray(SimTypeChar(label="Byte"), 28)),)), name="NVME_ERROR_INJECTION_ENTRY", pack=False, align=None),
    "NVME_ERROR_INJECTION_TYPES": SimTypeInt(signed=False, label="NVME_ERROR_INJECTION_TYPES"),
    "NVME_EXTENDED_REPORT_ZONE_INFO": SimStruct(OrderedDict((("ZoneCount", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 7)), ("Desc", SimTypePointer(SimTypeRef("NVME_ZONE_EXTENDED_REPORT_ZONE_DESC", SimStruct), offset=0)),)), name="NVME_EXTENDED_REPORT_ZONE_INFO", pack=False, align=None),
    "NVME_FEATURES": SimTypeInt(signed=False, label="NVME_FEATURES"),
    "NVME_FEATURE_HOST_IDENTIFIER_DATA": SimStruct(OrderedDict((("HOSTID", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="NVME_FEATURE_HOST_IDENTIFIER_DATA", pack=False, align=None),
    "NVME_FEATURE_HOST_METADATA_DATA": SimStruct(OrderedDict((("NumberOfMetadataElementDescriptors", SimTypeChar(label="Byte")), ("Reserved0", SimTypeChar(label="Byte")), ("MetadataElementDescriptors", SimTypeArray(SimTypeChar(label="Byte"), 4094)),)), name="NVME_FEATURE_HOST_METADATA_DATA", pack=False, align=None),
    "NVME_FEATURE_VALUE_CODES": SimTypeInt(signed=False, label="NVME_FEATURE_VALUE_CODES"),
    "NVME_FIRMWARE_ACTIVATE_ACTIONS": SimTypeInt(signed=False, label="NVME_FIRMWARE_ACTIVATE_ACTIONS"),
    "NVME_FIRMWARE_SLOT_INFO_LOG": SimStruct(OrderedDict((("AFI", SimTypeRef("_AFI_e__Struct", SimStruct)), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("FRS", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 7)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 448)),)), name="NVME_FIRMWARE_SLOT_INFO_LOG", pack=False, align=None),
    "NVME_FUSED_OPERATION_CODES": SimTypeInt(signed=False, label="NVME_FUSED_OPERATION_CODES"),
    "NVME_HEALTH_INFO_LOG": SimStruct(OrderedDict((("CriticalWarning", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Temperature", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("AvailableSpare", SimTypeChar(label="Byte")), ("AvailableSpareThreshold", SimTypeChar(label="Byte")), ("PercentageUsed", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 26)), ("DataUnitRead", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("DataUnitWritten", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("HostReadCommands", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("HostWrittenCommands", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ControllerBusyTime", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PowerCycle", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PowerOnHours", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("UnsafeShutdowns", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("MediaErrors", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ErrorInfoLogEntryCount", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("WarningCompositeTemperatureTime", SimTypeInt(signed=False, label="UInt32")), ("CriticalCompositeTemperatureTime", SimTypeInt(signed=False, label="UInt32")), ("TemperatureSensor1", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor2", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor3", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor4", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor5", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor6", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor7", SimTypeShort(signed=False, label="UInt16")), ("TemperatureSensor8", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 296)),)), name="NVME_HEALTH_INFO_LOG", pack=False, align=None),
    "NVME_HOST_MEMORY_BUFFER_DESCRIPTOR_ENTRY": SimStruct(OrderedDict((("BADD", SimTypeLongLong(signed=False, label="UInt64")), ("BSIZE", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_HOST_MEMORY_BUFFER_DESCRIPTOR_ENTRY", pack=False, align=None),
    "NVME_HOST_METADATA_ELEMENT_ACTIONS": SimTypeInt(signed=False, label="NVME_HOST_METADATA_ELEMENT_ACTIONS"),
    "NVME_HOST_METADATA_ELEMENT_DESCRIPTOR": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("EVAL", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NVME_HOST_METADATA_ELEMENT_DESCRIPTOR", pack=False, align=None),
    "NVME_IDENTIFIER_TYPE": SimTypeInt(signed=False, label="NVME_IDENTIFIER_TYPE"),
    "NVME_IDENTIFIER_TYPE_LENGTH": SimTypeInt(signed=False, label="NVME_IDENTIFIER_TYPE_LENGTH"),
    "NVME_IDENTIFY_CNS_CODES": SimTypeInt(signed=False, label="NVME_IDENTIFY_CNS_CODES"),
    "NVME_IDENTIFY_CONTROLLER_DATA": SimStruct(OrderedDict((("VID", SimTypeShort(signed=False, label="UInt16")), ("SSVID", SimTypeShort(signed=False, label="UInt16")), ("SN", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("MN", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("FR", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("RAB", SimTypeChar(label="Byte")), ("IEEE", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("CMIC", SimTypeRef("_CMIC_e__Struct", SimStruct)), ("MDTS", SimTypeChar(label="Byte")), ("CNTLID", SimTypeShort(signed=False, label="UInt16")), ("VER", SimTypeInt(signed=False, label="UInt32")), ("RTD3R", SimTypeInt(signed=False, label="UInt32")), ("RTD3E", SimTypeInt(signed=False, label="UInt32")), ("OAES", SimTypeRef("_OAES_e__Struct", SimStruct)), ("CTRATT", SimTypeRef("_CTRATT_e__Struct", SimStruct)), ("RRLS", SimTypeRef("_RRLS_e__Struct", SimStruct)), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 9)), ("CNTRLTYPE", SimTypeChar(label="Byte")), ("FGUID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("CRDT1", SimTypeShort(signed=False, label="UInt16")), ("CRDT2", SimTypeShort(signed=False, label="UInt16")), ("CRDT3", SimTypeShort(signed=False, label="UInt16")), ("Reserved0_1", SimTypeArray(SimTypeChar(label="Byte"), 106)), ("ReservedForManagement", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("OACS", SimTypeRef("_OACS_e__Struct", SimStruct)), ("ACL", SimTypeChar(label="Byte")), ("AERL", SimTypeChar(label="Byte")), ("FRMW", SimTypeRef("_FRMW_e__Struct", SimStruct)), ("LPA", SimTypeRef("_LPA_e__Struct", SimStruct)), ("ELPE", SimTypeChar(label="Byte")), ("NPSS", SimTypeChar(label="Byte")), ("AVSCC", SimTypeRef("_AVSCC_e__Struct", SimStruct)), ("APSTA", SimTypeRef("_APSTA_e__Struct", SimStruct)), ("WCTEMP", SimTypeShort(signed=False, label="UInt16")), ("CCTEMP", SimTypeShort(signed=False, label="UInt16")), ("MTFA", SimTypeShort(signed=False, label="UInt16")), ("HMPRE", SimTypeInt(signed=False, label="UInt32")), ("HMMIN", SimTypeInt(signed=False, label="UInt32")), ("TNVMCAP", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("UNVMCAP", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("RPMBS", SimTypeRef("_RPMBS_e__Struct", SimStruct)), ("EDSTT", SimTypeShort(signed=False, label="UInt16")), ("DSTO", SimTypeChar(label="Byte")), ("FWUG", SimTypeChar(label="Byte")), ("KAS", SimTypeShort(signed=False, label="UInt16")), ("HCTMA", SimTypeRef("_HCTMA_e__Struct", SimStruct)), ("MNTMT", SimTypeShort(signed=False, label="UInt16")), ("MXTMT", SimTypeShort(signed=False, label="UInt16")), ("SANICAP", SimTypeRef("_SANICAP_e__Struct", SimStruct)), ("HMMINDS", SimTypeInt(signed=False, label="UInt32")), ("HMMAXD", SimTypeShort(signed=False, label="UInt16")), ("NSETIDMAX", SimTypeShort(signed=False, label="UInt16")), ("ENDGIDMAX", SimTypeShort(signed=False, label="UInt16")), ("ANATT", SimTypeChar(label="Byte")), ("ANACAP", SimTypeRef("_ANACAP_e__Struct", SimStruct)), ("ANAGRPMAX", SimTypeInt(signed=False, label="UInt32")), ("NANAGRPID", SimTypeInt(signed=False, label="UInt32")), ("PELS", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 156)), ("SQES", SimTypeRef("_SQES_e__Struct", SimStruct)), ("CQES", SimTypeRef("_CQES_e__Struct", SimStruct)), ("MAXCMD", SimTypeShort(signed=False, label="UInt16")), ("NN", SimTypeInt(signed=False, label="UInt32")), ("ONCS", SimTypeRef("_ONCS_e__Struct", SimStruct)), ("FUSES", SimTypeRef("_FUSES_e__Struct", SimStruct)), ("FNA", SimTypeRef("_FNA_e__Struct", SimStruct)), ("VWC", SimTypeRef("_VWC_e__Struct", SimStruct)), ("AWUN", SimTypeShort(signed=False, label="UInt16")), ("AWUPF", SimTypeShort(signed=False, label="UInt16")), ("NVSCC", SimTypeRef("_NVSCC_e__Struct", SimStruct)), ("NWPC", SimTypeRef("_NWPC_e__Struct", SimStruct)), ("ACWU", SimTypeShort(signed=False, label="UInt16")), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("SGLS", SimTypeRef("_SGLS_e__Struct", SimStruct)), ("MNAN", SimTypeInt(signed=False, label="UInt32")), ("Reserved6", SimTypeArray(SimTypeChar(label="Byte"), 224)), ("SUBNQN", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("Reserved7", SimTypeArray(SimTypeChar(label="Byte"), 768)), ("Reserved8", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("PDS", SimTypeArray(SimTypeRef("NVME_POWER_STATE_DESC", SimStruct), 32)), ("VS", SimTypeArray(SimTypeChar(label="Byte"), 1024)),)), name="NVME_IDENTIFY_CONTROLLER_DATA", pack=False, align=None),
    "NVME_IDENTIFY_IO_COMMAND_SET": SimStruct(OrderedDict((("IOCommandSetVector", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 512)),)), name="NVME_IDENTIFY_IO_COMMAND_SET", pack=False, align=None),
    "NVME_IDENTIFY_NAMESPACE_DATA": SimStruct(OrderedDict((("NSZE", SimTypeLongLong(signed=False, label="UInt64")), ("NCAP", SimTypeLongLong(signed=False, label="UInt64")), ("NUSE", SimTypeLongLong(signed=False, label="UInt64")), ("NSFEAT", SimTypeRef("_NSFEAT_e__Struct", SimStruct)), ("NLBAF", SimTypeChar(label="Byte")), ("FLBAS", SimTypeRef("_FLBAS_e__Struct", SimStruct)), ("MC", SimTypeRef("_MC_e__Struct", SimStruct)), ("DPC", SimTypeRef("_DPC_e__Struct", SimStruct)), ("DPS", SimTypeRef("_DPS_e__Struct", SimStruct)), ("NMIC", SimTypeRef("_NMIC_e__Struct", SimStruct)), ("RESCAP", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("FPI", SimTypeRef("_FPI_e__Struct", SimStruct)), ("DLFEAT", SimTypeRef("_DLFEAT_e__Struct", SimStruct)), ("NAWUN", SimTypeShort(signed=False, label="UInt16")), ("NAWUPF", SimTypeShort(signed=False, label="UInt16")), ("NACWU", SimTypeShort(signed=False, label="UInt16")), ("NABSN", SimTypeShort(signed=False, label="UInt16")), ("NABO", SimTypeShort(signed=False, label="UInt16")), ("NABSPF", SimTypeShort(signed=False, label="UInt16")), ("NOIOB", SimTypeShort(signed=False, label="UInt16")), ("NVMCAP", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("NPWG", SimTypeShort(signed=False, label="UInt16")), ("NPWA", SimTypeShort(signed=False, label="UInt16")), ("NPDG", SimTypeShort(signed=False, label="UInt16")), ("NPDA", SimTypeShort(signed=False, label="UInt16")), ("NOWS", SimTypeShort(signed=False, label="UInt16")), ("MSSRL", SimTypeShort(signed=False, label="UInt16")), ("MCL", SimTypeInt(signed=False, label="UInt32")), ("MSRC", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 11)), ("ANAGRPID", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("NSATTR", SimTypeRef("_NSATTR_e__Struct", SimStruct)), ("NVMSETID", SimTypeShort(signed=False, label="UInt16")), ("ENDGID", SimTypeShort(signed=False, label="UInt16")), ("NGUID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("EUI64", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("LBAF", SimTypeArray(SimUnion({"Anonymous": SimStruct(OrderedDict((("MS", SimTypeShort(signed=False, label="UInt16")), ("LBADS", SimTypeChar(label="Byte")), ("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), 16)), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 192)), ("VS", SimTypeArray(SimTypeChar(label="Byte"), 3712)),)), name="NVME_IDENTIFY_NAMESPACE_DATA", pack=False, align=None),
    "NVME_IDENTIFY_NAMESPACE_DESCRIPTOR": SimStruct(OrderedDict((("NIDT", SimTypeChar(label="Byte")), ("NIDL", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("NID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NVME_IDENTIFY_NAMESPACE_DESCRIPTOR", pack=False, align=None),
    "NVME_IDENTIFY_NVM_SPECIFIC_CONTROLLER_IO_COMMAND_SET": SimStruct(OrderedDict((("VSL", SimTypeChar(label="Byte")), ("WZSL", SimTypeChar(label="Byte")), ("WUSL", SimTypeChar(label="Byte")), ("DMRL", SimTypeChar(label="Byte")), ("DMRSL", SimTypeInt(signed=False, label="UInt32")), ("DMSL", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 4080)),)), name="NVME_IDENTIFY_NVM_SPECIFIC_CONTROLLER_IO_COMMAND_SET", pack=False, align=None),
    "NVME_IDENTIFY_SPECIFIC_NAMESPACE_IO_COMMAND_SET": SimStruct(OrderedDict((("ZOC", SimTypeRef("_ZOC_e__Struct", SimStruct)), ("OZCS", SimTypeRef("_OZCS_e__Struct", SimStruct)), ("MAR", SimTypeInt(signed=False, label="UInt32")), ("MOR", SimTypeInt(signed=False, label="UInt32")), ("RRL", SimTypeInt(signed=False, label="UInt32")), ("FRL", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 2796)), ("LBAEF", SimTypeArray(SimTypeRef("NVME_LBA_ZONE_FORMAT", SimStruct), 16)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 768)), ("VS", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="NVME_IDENTIFY_SPECIFIC_NAMESPACE_IO_COMMAND_SET", pack=False, align=None),
    "NVME_IDENTIFY_ZNS_SPECIFIC_CONTROLLER_IO_COMMAND_SET": SimStruct(OrderedDict((("ZASL", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 4095)),)), name="NVME_IDENTIFY_ZNS_SPECIFIC_CONTROLLER_IO_COMMAND_SET", pack=False, align=None),
    "NVME_LBA_FORMAT": SimUnion({"Anonymous": SimStruct(OrderedDict((("MS", SimTypeShort(signed=False, label="UInt16")), ("LBADS", SimTypeChar(label="Byte")), ("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_LBA_RANGE": SimStruct(OrderedDict((("Attributes", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("LogicalBlockCount", SimTypeInt(signed=False, label="UInt32")), ("StartingLBA", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_LBA_RANGE", pack=False, align=None),
    "NVME_LBA_RANGET_TYPE_ENTRY": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Attributes", SimTypeRef("_Attributes_e__Struct", SimStruct)), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 14)), ("SLBA", SimTypeLongLong(signed=False, label="UInt64")), ("NLB", SimTypeLongLong(signed=False, label="UInt64")), ("GUID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="NVME_LBA_RANGET_TYPE_ENTRY", pack=False, align=None),
    "NVME_LBA_RANGE_TYPES": SimTypeInt(signed=False, label="NVME_LBA_RANGE_TYPES"),
    "NVME_LBA_ZONE_FORMAT": SimStruct(OrderedDict((("ZoneSize", SimTypeLongLong(signed=False, label="UInt64")), ("ZDES", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 7)),)), name="NVME_LBA_ZONE_FORMAT", pack=False, align=None),
    "NVME_LOG_PAGES": SimTypeInt(signed=False, label="NVME_LOG_PAGES"),
    "NVME_NAMESPACE_METADATA_ELEMENT_TYPES": SimTypeInt(signed=False, label="NVME_NAMESPACE_METADATA_ELEMENT_TYPES"),
    "NVME_NO_DEALLOCATE_MODIFIES_MEDIA_AFTER_SANITIZE": SimTypeInt(signed=False, label="NVME_NO_DEALLOCATE_MODIFIES_MEDIA_AFTER_SANITIZE"),
    "NVME_NVM_COMMANDS": SimTypeInt(signed=False, label="NVME_NVM_COMMANDS"),
    "NVME_NVM_QUEUE_PRIORITIES": SimTypeInt(signed=False, label="NVME_NVM_QUEUE_PRIORITIES"),
    "NVME_NVM_SUBSYSTEM_RESET": SimStruct(OrderedDict((("NSSRC", SimTypeInt(signed=False, label="UInt32")),)), name="NVME_NVM_SUBSYSTEM_RESET", pack=False, align=None),
    "NVME_OCP_DEVICE_CAPABILITIES_LOG": SimStruct(OrderedDict((("PciePorts", SimTypeShort(signed=False, label="UInt16")), ("OobMgmtSupport", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("WriteZeroesCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("SanitizeCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DatasetMgmtCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("WriteUncorrectableCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("FusedCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("MinimumValidDSSDPowerState", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeChar(label="Byte")), ("DssdDescriptors", SimTypeArray(SimTypeRef("DSSD_POWER_STATE_DESCRIPTOR", SimStruct), 127)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 3934)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_CAPABILITIES_LOG", pack=False, align=None),
    "NVME_OCP_DEVICE_ERROR_RECOVERY_LOG_V2": SimStruct(OrderedDict((("PanicResetWaitTime", SimTypeShort(signed=False, label="UInt16")), ("PanicResetAction", SimTypeRef("NVME_WCS_DEVICE_RESET_ACTION", SimStruct)), ("DeviceRecoveryAction1", SimTypeChar(label="Byte")), ("PanicId", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceCapabilitiesA", SimTypeRef("NVME_WCS_DEVICE_CAPABILITIES", SimStruct)), ("VendorSpecificRecoveryCode", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("VendorSpecificCommandCDW12", SimTypeInt(signed=False, label="UInt32")), ("VendorSpecificCommandCDW13", SimTypeInt(signed=False, label="UInt32")), ("VendorSpecificCommandTimeout", SimTypeChar(label="Byte")), ("DeviceRecoveryAction2", SimTypeChar(label="Byte")), ("DeviceRecoveryAction2Timeout", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 463)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_ERROR_RECOVERY_LOG_V2", pack=False, align=None),
    "NVME_OCP_DEVICE_FIRMWARE_ACTIVATION_HISTORY_LOG": SimStruct(OrderedDict((("LID", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ValidNumberOfEntries", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypeArray(SimTypeRef("FIRMWARE_ACTIVATION_HISTORY_ENTRY", SimStruct), 20)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2790)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_FIRMWARE_ACTIVATION_HISTORY_LOG", pack=False, align=None),
    "NVME_OCP_DEVICE_LATENCY_MONITOR_LOG": SimStruct(OrderedDict((("FeatureStatus", SimTypeRef("LATENCY_MONITOR_FEATURE_STATUS", SimStruct)), ("Reserved0", SimTypeChar(label="Byte")), ("ActiveBucketTimer", SimTypeShort(signed=False, label="UInt16")), ("ActiveBucketTimerThreshold", SimTypeShort(signed=False, label="UInt16")), ("ActiveThresholdA", SimTypeChar(label="Byte")), ("ActiveThresholdB", SimTypeChar(label="Byte")), ("ActiveThresholdC", SimTypeChar(label="Byte")), ("ActiveThresholdD", SimTypeChar(label="Byte")), ("ActiveLatencyConfig", SimTypeRef("ACTIVE_LATENCY_CONFIGURATION", SimStruct)), ("ActiveLatencyMinimumWindow", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 19)), ("ActiveBucketCounter0", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("ActiveBucketCounter1", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("ActiveBucketCounter2", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("ActiveBucketCounter3", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("ActiveLatencyStamp", SimTypeRef("LATENCY_STAMP", SimStruct)), ("ActiveMeasuredLatency", SimTypeRef("MEASURED_LATENCY", SimStruct)), ("ActiveLatencyStampUnits", SimTypeRef("LATENCY_STAMP_UNITS", SimStruct)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 22)), ("StaticBucketCounter0", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("StaticBucketCounter1", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("StaticBucketCounter2", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("StaticBucketCounter3", SimTypeRef("BUCKET_COUNTER", SimStruct)), ("StaticLatencyStamp", SimTypeRef("LATENCY_STAMP", SimStruct)), ("StaticMeasuredLatency", SimTypeRef("MEASURED_LATENCY", SimStruct)), ("StaticLatencyStampUnits", SimTypeRef("LATENCY_STAMP_UNITS", SimStruct)), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 22)), ("DebugLogTriggerEnable", SimTypeRef("DEBUG_BIT_FIELD", SimStruct)), ("DebugLogMeasuredLatency", SimTypeShort(signed=False, label="UInt16")), ("DebugLogLatencyStamp", SimTypeLongLong(signed=False, label="UInt64")), ("DebugLogPointer", SimTypeShort(signed=False, label="UInt16")), ("DebugCounterTriggerSource", SimTypeRef("DEBUG_BIT_FIELD", SimStruct)), ("DebugLogStampUnits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 29)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_LATENCY_MONITOR_LOG", pack=False, align=None),
    "NVME_OCP_DEVICE_SMART_INFORMATION_LOG_V3": SimStruct(OrderedDict((("MediaUnitsWritten", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("MediaUnitsRead", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("BadUserNANDBlockCount", SimTypeRef("_BadUserNANDBlockCount_e__Struct", SimStruct)), ("BadSystemNANDBlockCount", SimTypeRef("_BadSystemNANDBlockCount_e__Struct", SimStruct)), ("XORRecoveryCount", SimTypeLongLong(signed=False, label="UInt64")), ("UnrecoverableReadErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("SoftECCErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("EndToEndCorrectionCounts", SimTypeRef("_EndToEndCorrectionCounts_e__Struct", SimStruct)), ("PercentageSystemDataUsed", SimTypeChar(label="Byte")), ("RefreshCount", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("UserDataEraseCounts", SimTypeRef("_UserDataEraseCounts_e__Struct", SimStruct)), ("ThermalThrottling", SimTypeRef("_ThermalThrottling_e__Struct", SimStruct)), ("DSSDSpecVersion", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("PCIeCorrectableErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("IncompleteShutdownCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("PercentageFreeBlocks", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("CapacitorHealth", SimTypeShort(signed=False, label="UInt16")), ("NvmeErrata", SimTypeChar(label="Byte")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("UnalignedIOCount", SimTypeLongLong(signed=False, label="UInt64")), ("SecurityVersionNumber", SimTypeLongLong(signed=False, label="UInt64")), ("NUSE", SimTypeLongLong(signed=False, label="UInt64")), ("PLPStartCount", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("EnduranceEstimate", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PCIeLinkRetrainingCount", SimTypeLongLong(signed=False, label="UInt64")), ("PowerStateChangeCount", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 286)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_SMART_INFORMATION_LOG_V3", pack=False, align=None),
    "NVME_OCP_DEVICE_TCG_CONFIGURATION_LOG": SimStruct(OrderedDict((("State", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("LSPActivationCount", SimTypeChar(label="Byte")), ("TPRevertCount", SimTypeChar(label="Byte")), ("LSPRevertCount", SimTypeChar(label="Byte")), ("LOCount", SimTypeChar(label="Byte")), ("SUMLOCount", SimTypeChar(label="Byte")), ("RPLOCount", SimTypeChar(label="Byte")), ("NPLOCount", SimTypeChar(label="Byte")), ("RLLOCount", SimTypeChar(label="Byte")), ("WLLOCount", SimTypeChar(label="Byte")), ("RULOCount", SimTypeChar(label="Byte")), ("WULOCount", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("SIDAuthTryCount", SimTypeInt(signed=False, label="UInt32")), ("SIDAuthTryLimit", SimTypeInt(signed=False, label="UInt32")), ("ResetCount", SimTypeInt(signed=False, label="UInt32")), ("ResetLockCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 462)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_TCG_CONFIGURATION_LOG", pack=False, align=None),
    "NVME_OCP_DEVICE_TCG_HISTORY_LOG": SimStruct(OrderedDict((("LID", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("HistoryEntryCount", SimTypeInt(signed=False, label="UInt32")), ("HistoryEntries", SimTypeArray(SimTypeRef("TCG_HISTORY_ENTRY", SimStruct), 84)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 38)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_TCG_HISTORY_LOG", pack=False, align=None),
    "NVME_OCP_DEVICE_UNSUPPORTED_REQUIREMENTS_LOG": SimStruct(OrderedDict((("UnsupportedCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 14)), ("UnsupportedReqList", SimTypeArray(SimTypeRef("UNSUPPORTED_REQUIREMENT", SimStruct), 253)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 14)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_OCP_DEVICE_UNSUPPORTED_REQUIREMENTS_LOG", pack=False, align=None),
    "NVME_PERSISTENT_EVENT_LOG_EVENT_HEADER": SimStruct(OrderedDict((("EventType", SimTypeChar(label="Byte")), ("EventTypeRevision", SimTypeChar(label="Byte")), ("EventHeaderLength", SimTypeChar(label="Byte")), ("Reserved0", SimTypeChar(label="Byte")), ("ControllerIdentifier", SimTypeShort(signed=False, label="UInt16")), ("EventTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("VendorSpecificInformationLength", SimTypeShort(signed=False, label="UInt16")), ("EventLength", SimTypeShort(signed=False, label="UInt16")),)), name="NVME_PERSISTENT_EVENT_LOG_EVENT_HEADER", pack=False, align=None),
    "NVME_PERSISTENT_EVENT_LOG_EVENT_TYPES": SimTypeInt(signed=False, label="NVME_PERSISTENT_EVENT_LOG_EVENT_TYPES"),
    "NVME_PERSISTENT_EVENT_LOG_HEADER": SimStruct(OrderedDict((("LogIdentifier", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("TotalNumberOfEvents", SimTypeInt(signed=False, label="UInt32")), ("TotalLogLength", SimTypeLongLong(signed=False, label="UInt64")), ("LogRevision", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("LogHeaderLength", SimTypeShort(signed=False, label="UInt16")), ("Timestamp", SimTypeLongLong(signed=False, label="UInt64")), ("PowerOnHours", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PowerCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("PciVendorId", SimTypeShort(signed=False, label="UInt16")), ("PciSubsystemVendorId", SimTypeShort(signed=False, label="UInt16")), ("SerialNumber", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("ModelNumber", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("NVMSubsystemNVMeQualifiedName", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 108)), ("SupportedEventsBitmap", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NVME_PERSISTENT_EVENT_LOG_HEADER", pack=False, align=None),
    "NVME_POWER_STATE_DESC": SimStruct(OrderedDict((("MP", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeChar(label="Byte")), ("_bitfield1", SimTypeChar(label="Byte")), ("ENLAT", SimTypeInt(signed=False, label="UInt32")), ("EXLAT", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeChar(label="Byte")), ("_bitfield3", SimTypeChar(label="Byte")), ("_bitfield4", SimTypeChar(label="Byte")), ("_bitfield5", SimTypeChar(label="Byte")), ("IDLP", SimTypeShort(signed=False, label="UInt16")), ("_bitfield6", SimTypeChar(label="Byte")), ("Reserved7", SimTypeChar(label="Byte")), ("ACTP", SimTypeShort(signed=False, label="UInt16")), ("_bitfield7", SimTypeChar(label="Byte")), ("Reserved9", SimTypeArray(SimTypeChar(label="Byte"), 9)),)), name="NVME_POWER_STATE_DESC", pack=False, align=None),
    "NVME_PROTECTION_INFORMATION_TYPES": SimTypeInt(signed=False, label="NVME_PROTECTION_INFORMATION_TYPES"),
    "NVME_PRP_ENTRY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "NVME_REGISTERED_CONTROLLER_DATA": SimStruct(OrderedDict((("CNTLID", SimTypeShort(signed=False, label="UInt16")), ("RCSTS", SimTypeRef("_RCSTS_e__Struct", SimStruct)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("HOSTID", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("RKEY", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_REGISTERED_CONTROLLER_DATA", pack=False, align=None),
    "NVME_REGISTERED_CONTROLLER_EXTENDED_DATA": SimStruct(OrderedDict((("CNTLID", SimTypeShort(signed=False, label="UInt16")), ("RCSTS", SimTypeRef("_RCSTS_e__Struct", SimStruct)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("RKEY", SimTypeLongLong(signed=False, label="UInt64")), ("HOSTID", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NVME_REGISTERED_CONTROLLER_EXTENDED_DATA", pack=False, align=None),
    "NVME_REPORT_ZONE_INFO": SimStruct(OrderedDict((("ZoneCount", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 7)), ("ZoneDescriptor", SimTypePointer(SimTypeRef("NVME_ZONE_DESCRIPTOR", SimStruct), offset=0)),)), name="NVME_REPORT_ZONE_INFO", pack=False, align=None),
    "NVME_RESERVATION_ACQUIRE_ACTIONS": SimTypeInt(signed=False, label="NVME_RESERVATION_ACQUIRE_ACTIONS"),
    "NVME_RESERVATION_ACQUIRE_DATA_STRUCTURE": SimStruct(OrderedDict((("CRKEY", SimTypeLongLong(signed=False, label="UInt64")), ("PRKEY", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_RESERVATION_ACQUIRE_DATA_STRUCTURE", pack=False, align=None),
    "NVME_RESERVATION_NOTIFICATION_LOG": SimStruct(OrderedDict((("LogPageCount", SimTypeLongLong(signed=False, label="UInt64")), ("LogPageType", SimTypeChar(label="Byte")), ("AvailableLogPageCount", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("NameSpaceId", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 48)),)), name="NVME_RESERVATION_NOTIFICATION_LOG", pack=False, align=None),
    "NVME_RESERVATION_NOTIFICATION_TYPES": SimTypeInt(signed=False, label="NVME_RESERVATION_NOTIFICATION_TYPES"),
    "NVME_RESERVATION_REGISTER_ACTIONS": SimTypeInt(signed=False, label="NVME_RESERVATION_REGISTER_ACTIONS"),
    "NVME_RESERVATION_REGISTER_DATA_STRUCTURE": SimStruct(OrderedDict((("CRKEY", SimTypeLongLong(signed=False, label="UInt64")), ("NRKEY", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_RESERVATION_REGISTER_DATA_STRUCTURE", pack=False, align=None),
    "NVME_RESERVATION_REGISTER_PTPL_STATE_CHANGES": SimTypeInt(signed=False, label="NVME_RESERVATION_REGISTER_PTPL_STATE_CHANGES"),
    "NVME_RESERVATION_RELEASE_ACTIONS": SimTypeInt(signed=False, label="NVME_RESERVATION_RELEASE_ACTIONS"),
    "NVME_RESERVATION_RELEASE_DATA_STRUCTURE": SimStruct(OrderedDict((("CRKEY", SimTypeLongLong(signed=False, label="UInt64")),)), name="NVME_RESERVATION_RELEASE_DATA_STRUCTURE", pack=False, align=None),
    "NVME_RESERVATION_REPORT_STATUS_DATA_STRUCTURE": SimStruct(OrderedDict((("Header", SimTypeRef("NVME_RESERVATION_REPORT_STATUS_HEADER", SimStruct)), ("RegisteredControllersData", SimTypePointer(SimTypeRef("NVME_REGISTERED_CONTROLLER_DATA", SimStruct), offset=0)),)), name="NVME_RESERVATION_REPORT_STATUS_DATA_STRUCTURE", pack=False, align=None),
    "NVME_RESERVATION_REPORT_STATUS_EXTENDED_DATA_STRUCTURE": SimStruct(OrderedDict((("Header", SimTypeRef("NVME_RESERVATION_REPORT_STATUS_HEADER", SimStruct)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("RegisteredControllersExtendedData", SimTypePointer(SimTypeRef("NVME_REGISTERED_CONTROLLER_EXTENDED_DATA", SimStruct), offset=0)),)), name="NVME_RESERVATION_REPORT_STATUS_EXTENDED_DATA_STRUCTURE", pack=False, align=None),
    "NVME_RESERVATION_REPORT_STATUS_HEADER": SimStruct(OrderedDict((("GEN", SimTypeInt(signed=False, label="UInt32")), ("RTYPE", SimTypeChar(label="Byte")), ("REGCTL", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("PTPLS", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 14)),)), name="NVME_RESERVATION_REPORT_STATUS_HEADER", pack=False, align=None),
    "NVME_RESERVATION_TYPES": SimTypeInt(signed=False, label="NVME_RESERVATION_TYPES"),
    "NVME_SANITIZE_ACTION": SimTypeInt(signed=False, label="NVME_SANITIZE_ACTION"),
    "NVME_SANITIZE_OPERATION_STATUS": SimTypeInt(signed=False, label="NVME_SANITIZE_OPERATION_STATUS"),
    "NVME_SANITIZE_STATUS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="NVME_SANITIZE_STATUS", pack=False, align=None),
    "NVME_SANITIZE_STATUS_LOG": SimStruct(OrderedDict((("SPROG", SimTypeShort(signed=False, label="UInt16")), ("SSTAT", SimTypeRef("NVME_SANITIZE_STATUS", SimStruct)), ("SCDW10", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTimeForOverwrite", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTimeForBlockErase", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTimeForCryptoErase", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTimeForOverwriteWithNoDeallocateMediaModification", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTimeForBlockEraseWithNoDeallocateMediaModification", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTimeForCryptoEraseWithNoDeallocateMediaModification", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 480)),)), name="NVME_SANITIZE_STATUS_LOG", pack=False, align=None),
    "NVME_SCSI_NAME_STRING": SimStruct(OrderedDict((("PCIVendorID", SimTypeArray(SimTypeChar(label="SByte"), 4)), ("ModelNumber", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("NamespaceID", SimTypeArray(SimTypeChar(label="SByte"), 4)), ("SerialNumber", SimTypeArray(SimTypeChar(label="SByte"), 20)),)), name="NVME_SCSI_NAME_STRING", pack=False, align=None),
    "NVME_SECURE_ERASE_SETTINGS": SimTypeInt(signed=False, label="NVME_SECURE_ERASE_SETTINGS"),
    "NVME_SET_ATTRIBUTES_ENTRY": SimStruct(OrderedDict((("Identifier", SimTypeShort(signed=False, label="UInt16")), ("ENDGID", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Random4KBReadTypical", SimTypeInt(signed=False, label="UInt32")), ("OptimalWriteSize", SimTypeInt(signed=False, label="UInt32")), ("TotalCapacity", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("UnallocatedCapacity", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 80)),)), name="NVME_SET_ATTRIBUTES_ENTRY", pack=False, align=None),
    "NVME_STATUS_COMMAND_SPECIFIC_CODES": SimTypeInt(signed=False, label="NVME_STATUS_COMMAND_SPECIFIC_CODES"),
    "NVME_STATUS_GENERIC_COMMAND_CODES": SimTypeInt(signed=False, label="NVME_STATUS_GENERIC_COMMAND_CODES"),
    "NVME_STATUS_MEDIA_ERROR_CODES": SimTypeInt(signed=False, label="NVME_STATUS_MEDIA_ERROR_CODES"),
    "NVME_STATUS_TYPES": SimTypeInt(signed=False, label="NVME_STATUS_TYPES"),
    "NVME_SUBMISSION_QUEUE_TAIL_DOORBELL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_TELEMETRY_CONTROLLER_INITIATED_LOG": SimStruct(OrderedDict((("LogIdentifier", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("OrganizationID", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Area1LastBlock", SimTypeShort(signed=False, label="UInt16")), ("Area2LastBlock", SimTypeShort(signed=False, label="UInt16")), ("Area3LastBlock", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Area4LastBlock", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 362)), ("ControllerInitiatedDataAvailable", SimTypeChar(label="Byte")), ("ControllerInitiatedDataGenerationNumber", SimTypeChar(label="Byte")), ("ReasonIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="NVME_TELEMETRY_CONTROLLER_INITIATED_LOG", pack=False, align=None),
    "NVME_TELEMETRY_HOST_INITIATED_LOG": SimStruct(OrderedDict((("LogIdentifier", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("OrganizationID", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Area1LastBlock", SimTypeShort(signed=False, label="UInt16")), ("Area2LastBlock", SimTypeShort(signed=False, label="UInt16")), ("Area3LastBlock", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Area4LastBlock", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 361)), ("HostInitiatedDataGenerationNumber", SimTypeChar(label="Byte")), ("ControllerInitiatedDataAvailable", SimTypeChar(label="Byte")), ("ControllerInitiatedDataGenerationNumber", SimTypeChar(label="Byte")), ("ReasonIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="NVME_TELEMETRY_HOST_INITIATED_LOG", pack=False, align=None),
    "NVME_TEMPERATURE_THRESHOLD_TYPES": SimTypeInt(signed=False, label="NVME_TEMPERATURE_THRESHOLD_TYPES"),
    "NVME_VENDOR_LOG_PAGES": SimTypeInt(signed=False, label="NVME_VENDOR_LOG_PAGES"),
    "NVME_VERSION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "NVME_WCS_DEVICE_CAPABILITIES": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="NVME_WCS_DEVICE_CAPABILITIES", pack=False, align=None),
    "NVME_WCS_DEVICE_ERROR_RECOVERY_LOG": SimStruct(OrderedDict((("PanicResetWaitTime", SimTypeShort(signed=False, label="UInt16")), ("PanicResetAction", SimTypeRef("NVME_WCS_DEVICE_RESET_ACTION", SimStruct)), ("DriveRecoveryAction", SimTypeChar(label="Byte")), ("PanicId", SimTypeLongLong(signed=False, label="UInt64")), ("DeviceCapabilitiesA", SimTypeRef("NVME_WCS_DEVICE_CAPABILITIES", SimStruct)), ("VendorSpecificRecoveryCode", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("VendorSpecificCommandCDW12", SimTypeInt(signed=False, label="UInt32")), ("VendorSpecificCommandCDW13", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 466)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_WCS_DEVICE_ERROR_RECOVERY_LOG", pack=False, align=None),
    "NVME_WCS_DEVICE_RECOVERY_ACTION1": SimTypeInt(signed=False, label="NVME_WCS_DEVICE_RECOVERY_ACTION1"),
    "NVME_WCS_DEVICE_RECOVERY_ACTION2": SimTypeInt(signed=False, label="NVME_WCS_DEVICE_RECOVERY_ACTION2"),
    "NVME_WCS_DEVICE_RESET_ACTION": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="NVME_WCS_DEVICE_RESET_ACTION", pack=False, align=None),
    "NVME_WCS_DEVICE_SMART_ATTRIBUTES_LOG": SimStruct(OrderedDict((("VersionSpecificData", SimTypeArray(SimTypeChar(label="Byte"), 494)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_WCS_DEVICE_SMART_ATTRIBUTES_LOG", pack=False, align=None),
    "NVME_WCS_DEVICE_SMART_ATTRIBUTES_LOG_V2": SimStruct(OrderedDict((("MediaUnitsWritten", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("MediaUnitsRead", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("BadUserNANDBlockCount", SimTypeRef("_BadUserNANDBlockCount_e__Struct", SimStruct)), ("BadSystemNANDBlockCount", SimTypeRef("_BadSystemNANDBlockCount_e__Struct", SimStruct)), ("XORRecoveryCount", SimTypeLongLong(signed=False, label="UInt64")), ("UnrecoverableReadErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("SoftECCErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("EndToEndCorrectionCounts", SimTypeRef("_EndToEndCorrectionCounts_e__Struct", SimStruct)), ("PercentageSystemDataUsed", SimTypeChar(label="Byte")), ("RefreshCount", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("UserDataEraseCounts", SimTypeRef("_UserDataEraseCounts_e__Struct", SimStruct)), ("ThermalThrottling", SimTypeRef("_ThermalThrottling_e__Struct", SimStruct)), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("PCIeCorrectableErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("IncompleteShutdownCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("PercentageFreeBlocks", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("CapacitorHealth", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("UnalignedIOCount", SimTypeLongLong(signed=False, label="UInt64")), ("SecurityVersionNumber", SimTypeLongLong(signed=False, label="UInt64")), ("NUSE", SimTypeLongLong(signed=False, label="UInt64")), ("PLPStartCount", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("EnduranceEstimate", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 302)), ("LogPageVersionNumber", SimTypeShort(signed=False, label="UInt16")), ("LogPageGUID", SimTypeBottom(label="Guid")),)), name="NVME_WCS_DEVICE_SMART_ATTRIBUTES_LOG_V2", pack=False, align=None),
    "NVME_ZONE_DESCRIPTOR": SimStruct(OrderedDict((("Anonymous1", SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous1_e__Struct", pack=False, align=None)), ("Anonymous2", SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous2_e__Struct", pack=False, align=None)), ("ZA", SimTypeRef("_ZA_e__Struct", SimStruct)), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("ZCAP", SimTypeLongLong(signed=False, label="UInt64")), ("ZSLBA", SimTypeLongLong(signed=False, label="UInt64")), ("WritePointer", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="NVME_ZONE_DESCRIPTOR", pack=False, align=None),
    "NVME_ZONE_DESCRIPTOR_EXTENSION": SimStruct(OrderedDict((("ZoneDescriptorExtensionInfo", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="NVME_ZONE_DESCRIPTOR_EXTENSION", pack=False, align=None),
    "NVME_ZONE_EXTENDED_REPORT_ZONE_DESC": SimStruct(OrderedDict((("ZoneDescriptor", SimTypeRef("NVME_ZONE_DESCRIPTOR", SimStruct)), ("ZoneDescriptorExtension", SimTypePointer(SimTypeRef("NVME_ZONE_DESCRIPTOR_EXTENSION", SimStruct), offset=0)),)), name="NVME_ZONE_EXTENDED_REPORT_ZONE_DESC", pack=False, align=None),
    "NVME_ZONE_RECEIVE_ACTION": SimTypeInt(signed=False, label="NVME_ZONE_RECEIVE_ACTION"),
    "NVME_ZONE_RECEIVE_ACTION_SPECIFIC": SimTypeInt(signed=False, label="NVME_ZONE_RECEIVE_ACTION_SPECIFIC"),
    "NVME_ZONE_SEND_ACTION": SimTypeInt(signed=False, label="NVME_ZONE_SEND_ACTION"),
    "NVM_RESERVATION_CAPABILITIES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "NVM_SET_LIST": SimStruct(OrderedDict((("IdentifierCount", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 127)), ("Entry", SimTypePointer(SimTypeRef("NVME_SET_ATTRIBUTES_ENTRY", SimStruct), offset=0)),)), name="NVM_SET_LIST", pack=False, align=None),
    "NV_FEATURE_PARAMETER": SimStruct(OrderedDict((("NVPowerModeEnabled", SimTypeShort(signed=False, label="UInt16")), ("NVParameterReserv1", SimTypeShort(signed=False, label="UInt16")), ("NVCmdEnabled", SimTypeShort(signed=False, label="UInt16")), ("NVParameterReserv2", SimTypeShort(signed=False, label="UInt16")), ("NVPowerModeVer", SimTypeShort(signed=False, label="UInt16")), ("NVCmdVer", SimTypeShort(signed=False, label="UInt16")), ("NVSize", SimTypeInt(signed=False, label="UInt32")), ("NVReadSpeed", SimTypeShort(signed=False, label="UInt16")), ("NVWrtSpeed", SimTypeShort(signed=False, label="UInt16")), ("DeviceSpinUpTime", SimTypeInt(signed=False, label="UInt32")),)), name="NV_FEATURE_PARAMETER", pack=False, align=None),
    "NV_MEMORY_RANGE": SimStruct(OrderedDict((("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="NV_MEMORY_RANGE", pack=False, align=None),
    "NV_SEP_CACHE_PARAMETER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"CacheFlags": SimTypeRef("_CacheFlags_e__Struct", SimStruct), "CacheFlagsSet": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("WriteCacheType", SimTypeChar(label="Byte")), ("WriteCacheTypeEffective", SimTypeChar(label="Byte")), ("ParameterReserve1", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="NV_SEP_CACHE_PARAMETER", pack=False, align=None),
    "NV_SEP_WRITE_CACHE_TYPE": SimTypeInt(signed=False, label="NV_SEP_WRITE_CACHE_TYPE"),
    "NWMF": SimTypeInt(signed=False, label="NWMF"),
    "NapComponentRegistrationInfo": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="UInt32")), ("friendlyName", SimTypeRef("CountedString", SimStruct)), ("description", SimTypeRef("CountedString", SimStruct)), ("version", SimTypeRef("CountedString", SimStruct)), ("vendorName", SimTypeRef("CountedString", SimStruct)), ("infoClsid", SimTypeBottom(label="Guid")), ("configClsid", SimTypeBottom(label="Guid")), ("registrationDate", SimTypeRef("FILETIME", SimStruct)), ("componentType", SimTypeInt(signed=False, label="UInt32")),)), name="NapComponentRegistrationInfo", pack=False, align=None),
    "NapNotifyType": SimTypeInt(signed=False, label="NapNotifyType"),
    "NapTracingLevel": SimTypeInt(signed=False, label="NapTracingLevel"),
    "NavigateDirection": SimTypeInt(signed=False, label="NavigateDirection"),
    "NetworkSoH": SimStruct(OrderedDict((("size", SimTypeShort(signed=False, label="UInt16")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="NetworkSoH", pack=False, align=None),
    "NewProcessCauseConstants": SimTypeInt(signed=False, label="NewProcessCauseConstants"),
    "NgcTicketContext": SimStruct(OrderedDict((("wszTicket", SimTypeArray(SimTypeChar(label="Char"), 45)), ("hKey", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hImpersonateToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="NgcTicketContext", pack=False, align=None),
    "Node": SimTypeBottom(label="Node"),
    "NodeContainerType": SimTypeInt(signed=False, label="NodeContainerType"),
    "NodeUtilizationInfoElement": SimStruct(OrderedDict((("Id", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableMemory", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableMemoryAfterReclamation", SimTypeLongLong(signed=False, label="UInt64")),)), name="NodeUtilizationInfoElement", pack=False, align=None),
    "Nodes": SimTypeBottom(label="Nodes"),
    "NormalizeState": SimTypeInt(signed=False, label="NormalizeState"),
    "NotificationHandler": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["connectionId", "pContextData"]), offset=0),
    "NotificationHookProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="Status"), arg_names=["token"]), offset=0),
    "NotificationKind": SimTypeInt(signed=False, label="NotificationKind"),
    "NotificationProcessing": SimTypeInt(signed=False, label="NotificationProcessing"),
    "NotificationUnhookProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["token"]), offset=0),
    "NtmsAccessMask": SimTypeInt(signed=False, label="NtmsAccessMask"),
    "NtmsAllocateOptions": SimTypeInt(signed=False, label="NtmsAllocateOptions"),
    "NtmsAllocationPolicy": SimTypeInt(signed=False, label="NtmsAllocationPolicy"),
    "NtmsAsyncOperations": SimTypeInt(signed=False, label="NtmsAsyncOperations"),
    "NtmsAsyncStatus": SimTypeInt(signed=False, label="NtmsAsyncStatus"),
    "NtmsBarCodeState": SimTypeInt(signed=False, label="NtmsBarCodeState"),
    "NtmsCreateNtmsMediaOptions": SimTypeInt(signed=False, label="NtmsCreateNtmsMediaOptions"),
    "NtmsCreateOptions": SimTypeInt(signed=False, label="NtmsCreateOptions"),
    "NtmsDeallocationPolicy": SimTypeInt(signed=False, label="NtmsDeallocationPolicy"),
    "NtmsDismountOptions": SimTypeInt(signed=False, label="NtmsDismountOptions"),
    "NtmsDoorState": SimTypeInt(signed=False, label="NtmsDoorState"),
    "NtmsDriveState": SimTypeInt(signed=False, label="NtmsDriveState"),
    "NtmsDriveType": SimTypeInt(signed=False, label="NtmsDriveType"),
    "NtmsEjectOperation": SimTypeInt(signed=False, label="NtmsEjectOperation"),
    "NtmsEnumerateOption": SimTypeInt(signed=False, label="NtmsEnumerateOption"),
    "NtmsInjectOperation": SimTypeInt(signed=False, label="NtmsInjectOperation"),
    "NtmsInventoryMethod": SimTypeInt(signed=False, label="NtmsInventoryMethod"),
    "NtmsLibRequestFlags": SimTypeInt(signed=False, label="NtmsLibRequestFlags"),
    "NtmsLibraryFlags": SimTypeInt(signed=False, label="NtmsLibraryFlags"),
    "NtmsLibraryType": SimTypeInt(signed=False, label="NtmsLibraryType"),
    "NtmsLmOperation": SimTypeInt(signed=False, label="NtmsLmOperation"),
    "NtmsLmState": SimTypeInt(signed=False, label="NtmsLmState"),
    "NtmsMediaPoolPolicy": SimTypeInt(signed=False, label="NtmsMediaPoolPolicy"),
    "NtmsMediaState": SimTypeInt(signed=False, label="NtmsMediaState"),
    "NtmsMountOptions": SimTypeInt(signed=False, label="NtmsMountOptions"),
    "NtmsMountPriority": SimTypeInt(signed=False, label="NtmsMountPriority"),
    "NtmsNotificationOperations": SimTypeInt(signed=False, label="NtmsNotificationOperations"),
    "NtmsObjectsTypes": SimTypeInt(signed=False, label="NtmsObjectsTypes"),
    "NtmsOpRequestFlags": SimTypeInt(signed=False, label="NtmsOpRequestFlags"),
    "NtmsOperationalState": SimTypeInt(signed=False, label="NtmsOperationalState"),
    "NtmsOpreqCommand": SimTypeInt(signed=False, label="NtmsOpreqCommand"),
    "NtmsOpreqState": SimTypeInt(signed=False, label="NtmsOpreqState"),
    "NtmsPartitionState": SimTypeInt(signed=False, label="NtmsPartitionState"),
    "NtmsPoolType": SimTypeInt(signed=False, label="NtmsPoolType"),
    "NtmsPortContent": SimTypeInt(signed=False, label="NtmsPortContent"),
    "NtmsPortPosition": SimTypeInt(signed=False, label="NtmsPortPosition"),
    "NtmsReadWriteCharacteristics": SimTypeInt(signed=False, label="NtmsReadWriteCharacteristics"),
    "NtmsSessionOptions": SimTypeInt(signed=False, label="NtmsSessionOptions"),
    "NtmsSlotState": SimTypeInt(signed=False, label="NtmsSlotState"),
    "NtmsUIOperations": SimTypeInt(signed=False, label="NtmsUIOperations"),
    "NtmsUITypes": SimTypeInt(signed=False, label="NtmsUITypes"),
    "OA_BOOL": SimTypeInt(signed=False, label="OA_BOOL"),
    "OBJECTDESCRIPTOR": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")), ("dwDrawAspect", SimTypeInt(signed=False, label="UInt32")), ("sizel", SimTypeRef("SIZE", SimStruct)), ("pointl", SimTypeRef("POINTL", SimStruct)), ("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("dwFullUserTypeName", SimTypeInt(signed=False, label="UInt32")), ("dwSrcOfCopy", SimTypeInt(signed=False, label="UInt32")),)), name="OBJECTDESCRIPTOR", pack=False, align=None),
    "OBJECTID": SimStruct(OrderedDict((("Lineage", SimTypeBottom(label="Guid")), ("Uniquifier", SimTypeInt(signed=False, label="UInt32")),)), name="OBJECTID", pack=False, align=None),
    "OBJECTPOSITIONS": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("cObjectCount", SimTypeInt(signed=True, label="Int32")), ("pcpPositions", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="OBJECTPOSITIONS", pack=False, align=None),
    "OBJECTS_AND_NAME_A": SimStruct(OrderedDict((("ObjectsPresent", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeInt(signed=False, label="SE_OBJECT_TYPE")), ("ObjectTypeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("InheritedObjectTypeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ptstrName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OBJECTS_AND_NAME_A", pack=False, align=None),
    "OBJECTS_AND_NAME_W": SimStruct(OrderedDict((("ObjectsPresent", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeInt(signed=False, label="SE_OBJECT_TYPE")), ("ObjectTypeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InheritedObjectTypeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ptstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="OBJECTS_AND_NAME_W", pack=False, align=None),
    "OBJECTS_AND_SID": SimStruct(OrderedDict((("ObjectsPresent", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectTypeGuid", SimTypeBottom(label="Guid")), ("InheritedObjectTypeGuid", SimTypeBottom(label="Guid")), ("pSid", SimTypePointer(SimTypeRef("SID", SimStruct), offset=0)),)), name="OBJECTS_AND_SID", pack=False, align=None),
    "OBJECTTYPE": SimTypeInt(signed=False, label="OBJECTTYPE"),
    "OBJECT_ATTRIBUTES": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("RootDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ObjectName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityQualityOfService", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OBJECT_ATTRIBUTES", pack=False, align=None),
    "OBJECT_ATTRIBUTES32": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("RootDirectory", SimTypeInt(signed=False, label="UInt32")), ("ObjectName", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypeInt(signed=False, label="UInt32")), ("SecurityQualityOfService", SimTypeInt(signed=False, label="UInt32")),)), name="OBJECT_ATTRIBUTES32", pack=False, align=None),
    "OBJECT_ATTRIBUTES64": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("RootDirectory", SimTypeLongLong(signed=False, label="UInt64")), ("ObjectName", SimTypeLongLong(signed=False, label="UInt64")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypeLongLong(signed=False, label="UInt64")), ("SecurityQualityOfService", SimTypeLongLong(signed=False, label="UInt64")),)), name="OBJECT_ATTRIBUTES64", pack=False, align=None),
    "OBJECT_ATTRIB_FLAGS": SimTypeInt(signed=False, label="OBJECT_ATTRIB_FLAGS"),
    "OBJECT_HANDLE_INFORMATION": SimStruct(OrderedDict((("HandleAttributes", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")),)), name="OBJECT_HANDLE_INFORMATION", pack=False, align=None),
    "OBJECT_IDENTIFIER": SimTypeInt(signed=False, label="OBJECT_IDENTIFIER"),
    "OBJECT_INFORMATION_CLASS": SimTypeInt(signed=False, label="OBJECT_INFORMATION_CLASS"),
    "OBJECT_NAME_INFORMATION": SimStruct(OrderedDict((("Name", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="OBJECT_NAME_INFORMATION", pack=False, align=None),
    "OBJECT_NOTIFICATION": SimStruct(OrderedDict((("cbEntryID", SimTypeInt(signed=False, label="UInt32")), ("lpEntryID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("ulObjType", SimTypeInt(signed=False, label="UInt32")), ("cbParentID", SimTypeInt(signed=False, label="UInt32")), ("lpParentID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("cbOldID", SimTypeInt(signed=False, label="UInt32")), ("lpOldID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("cbOldParentID", SimTypeInt(signed=False, label="UInt32")), ("lpOldParentID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("lpPropTagArray", SimTypePointer(SimTypeRef("SPropTagArray", SimStruct), offset=0)),)), name="OBJECT_NOTIFICATION", pack=False, align=None),
    "OBJECT_PROPERTIES_FLAGS": SimTypeInt(signed=False, label="OBJECT_PROPERTIES_FLAGS"),
    "OBJECT_SECURITY_INFORMATION": SimTypeInt(signed=False, label="OBJECT_SECURITY_INFORMATION"),
    "OBJECT_TYPE_LIST": SimStruct(OrderedDict((("Level", SimTypeShort(signed=False, label="UInt16")), ("Sbz", SimTypeShort(signed=False, label="UInt16")), ("ObjectType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="OBJECT_TYPE_LIST", pack=False, align=None),
    "OBJ_TYPE": SimTypeInt(signed=False, label="OBJ_TYPE"),
    "OBO_TOKEN": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="OBO_TOKEN_TYPE")), ("pncc", SimTypeBottom(label="INetCfgComponent")), ("pszwManufacturer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszwProduct", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszwDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fRegistered", SimTypeInt(signed=True, label="Int32")),)), name="OBO_TOKEN", pack=False, align=None),
    "OBO_TOKEN_TYPE": SimTypeInt(signed=False, label="OBO_TOKEN_TYPE"),
    "OB_CALLBACK_REGISTRATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("OperationRegistrationCount", SimTypeShort(signed=False, label="UInt16")), ("Altitude", SimTypeRef("UNICODE_STRING", SimStruct)), ("RegistrationContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OperationRegistration", SimTypePointer(SimTypeRef("OB_OPERATION_REGISTRATION", SimStruct), offset=0)),)), name="OB_CALLBACK_REGISTRATION", pack=False, align=None),
    "OB_OPERATION_REGISTRATION": SimStruct(OrderedDict((("ObjectType", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("Operations", SimTypeInt(signed=False, label="UInt32")), ("PreOperation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("OB_PRE_OPERATION_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=False, label="OB_PREOP_CALLBACK_STATUS"), arg_names=["RegistrationContext", "OperationInformation"]), offset=0)), ("PostOperation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("OB_POST_OPERATION_INFORMATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["RegistrationContext", "OperationInformation"]), offset=0)),)), name="OB_OPERATION_REGISTRATION", pack=False, align=None),
    "OB_POST_CREATE_HANDLE_INFORMATION": SimStruct(OrderedDict((("GrantedAccess", SimTypeInt(signed=False, label="UInt32")),)), name="OB_POST_CREATE_HANDLE_INFORMATION", pack=False, align=None),
    "OB_POST_DUPLICATE_HANDLE_INFORMATION": SimStruct(OrderedDict((("GrantedAccess", SimTypeInt(signed=False, label="UInt32")),)), name="OB_POST_DUPLICATE_HANDLE_INFORMATION", pack=False, align=None),
    "OB_POST_OPERATION_INFORMATION": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectType", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ReturnStatus", SimTypeInt(signed=True, label="Int32")), ("Parameters", SimTypePointer(SimUnion({"CreateHandleInformation": SimTypeRef("OB_POST_CREATE_HANDLE_INFORMATION", SimStruct), "DuplicateHandleInformation": SimTypeRef("OB_POST_DUPLICATE_HANDLE_INFORMATION", SimStruct)}, name="<anon>", label="None"), offset=0)),)), name="OB_POST_OPERATION_INFORMATION", pack=False, align=None),
    "OB_POST_OPERATION_PARAMETERS": SimUnion({"CreateHandleInformation": SimTypeRef("OB_POST_CREATE_HANDLE_INFORMATION", SimStruct), "DuplicateHandleInformation": SimTypeRef("OB_POST_DUPLICATE_HANDLE_INFORMATION", SimStruct)}, name="<anon>", label="None"),
    "OB_PREOP_CALLBACK_STATUS": SimTypeInt(signed=False, label="OB_PREOP_CALLBACK_STATUS"),
    "OB_PRE_CREATE_HANDLE_INFORMATION": SimStruct(OrderedDict((("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("OriginalDesiredAccess", SimTypeInt(signed=False, label="UInt32")),)), name="OB_PRE_CREATE_HANDLE_INFORMATION", pack=False, align=None),
    "OB_PRE_DUPLICATE_HANDLE_INFORMATION": SimStruct(OrderedDict((("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("OriginalDesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("SourceProcess", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TargetProcess", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OB_PRE_DUPLICATE_HANDLE_INFORMATION", pack=False, align=None),
    "OB_PRE_OPERATION_INFORMATION": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectType", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Parameters", SimTypePointer(SimUnion({"CreateHandleInformation": SimTypeRef("OB_PRE_CREATE_HANDLE_INFORMATION", SimStruct), "DuplicateHandleInformation": SimTypeRef("OB_PRE_DUPLICATE_HANDLE_INFORMATION", SimStruct)}, name="<anon>", label="None"), offset=0)),)), name="OB_PRE_OPERATION_INFORMATION", pack=False, align=None),
    "OB_PRE_OPERATION_PARAMETERS": SimUnion({"CreateHandleInformation": SimTypeRef("OB_PRE_CREATE_HANDLE_INFORMATION", SimStruct), "DuplicateHandleInformation": SimTypeRef("OB_PRE_DUPLICATE_HANDLE_INFORMATION", SimStruct)}, name="<anon>", label="None"),
    "OCPFIPARAMS": SimStruct(OrderedDict((("cbStructSize", SimTypeInt(signed=False, label="UInt32")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cObjects", SimTypeInt(signed=False, label="UInt32")), ("lplpUnk", SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)), ("cPages", SimTypeInt(signed=False, label="UInt32")), ("lpPages", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("dispidInitialProperty", SimTypeInt(signed=True, label="Int32")),)), name="OCPFIPARAMS", pack=False, align=None),
    "OCSPRequestFlag": SimTypeInt(signed=False, label="OCSPRequestFlag"),
    "OCSPSigningFlag": SimTypeInt(signed=False, label="OCSPSigningFlag"),
    "OCSP_BASIC_RESPONSE_ENTRY": SimStruct(OrderedDict((("CertId", SimTypeRef("OCSP_CERT_ID", SimStruct)), ("dwCertStatus", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pRevokedInfo": SimTypePointer(SimTypeRef("OCSP_BASIC_REVOKED_INFO", SimStruct), offset=0)}, name="<anon>", label="None")), ("ThisUpdate", SimTypeRef("FILETIME", SimStruct)), ("NextUpdate", SimTypeRef("FILETIME", SimStruct)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="OCSP_BASIC_RESPONSE_ENTRY", pack=False, align=None),
    "OCSP_BASIC_RESPONSE_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwResponderIdChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ByNameResponderId": SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), "ByKeyResponderId": SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)}, name="<anon>", label="None")), ("ProducedAt", SimTypeRef("FILETIME", SimStruct)), ("cResponseEntry", SimTypeInt(signed=False, label="UInt32")), ("rgResponseEntry", SimTypePointer(SimTypeRef("OCSP_BASIC_RESPONSE_ENTRY", SimStruct), offset=0)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="OCSP_BASIC_RESPONSE_INFO", pack=False, align=None),
    "OCSP_BASIC_REVOKED_INFO": SimStruct(OrderedDict((("RevocationDate", SimTypeRef("FILETIME", SimStruct)), ("dwCrlReasonCode", SimTypeInt(signed=False, label="CERT_REVOCATION_STATUS_REASON")),)), name="OCSP_BASIC_REVOKED_INFO", pack=False, align=None),
    "OCSP_BASIC_SIGNED_RESPONSE_INFO": SimStruct(OrderedDict((("ToBeSigned", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SignatureInfo", SimTypeRef("OCSP_SIGNATURE_INFO", SimStruct)),)), name="OCSP_BASIC_SIGNED_RESPONSE_INFO", pack=False, align=None),
    "OCSP_CERT_ID": SimStruct(OrderedDict((("HashAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("IssuerNameHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("IssuerKeyHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("SerialNumber", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="OCSP_CERT_ID", pack=False, align=None),
    "OCSP_REQUEST_ENTRY": SimStruct(OrderedDict((("CertId", SimTypeRef("OCSP_CERT_ID", SimStruct)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="OCSP_REQUEST_ENTRY", pack=False, align=None),
    "OCSP_REQUEST_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pRequestorName", SimTypePointer(SimTypeRef("CERT_ALT_NAME_ENTRY", SimStruct), offset=0)), ("cRequestEntry", SimTypeInt(signed=False, label="UInt32")), ("rgRequestEntry", SimTypePointer(SimTypeRef("OCSP_REQUEST_ENTRY", SimStruct), offset=0)), ("cExtension", SimTypeInt(signed=False, label="UInt32")), ("rgExtension", SimTypePointer(SimTypeRef("CERT_EXTENSION", SimStruct), offset=0)),)), name="OCSP_REQUEST_INFO", pack=False, align=None),
    "OCSP_RESPONSE_INFO": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("pszObjId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Value", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="OCSP_RESPONSE_INFO", pack=False, align=None),
    "OCSP_SIGNATURE_INFO": SimStruct(OrderedDict((("SignatureAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Signature", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("cCertEncoded", SimTypeInt(signed=False, label="UInt32")), ("rgCertEncoded", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="OCSP_SIGNATURE_INFO", pack=False, align=None),
    "OCSP_SIGNED_REQUEST_INFO": SimStruct(OrderedDict((("ToBeSigned", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("pOptionalSignatureInfo", SimTypePointer(SimTypeRef("OCSP_SIGNATURE_INFO", SimStruct), offset=0)),)), name="OCSP_SIGNED_REQUEST_INFO", pack=False, align=None),
    "OCTET_STRING": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("lpValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OCTET_STRING", pack=False, align=None),
    "ODA_FLAGS": SimTypeInt(signed=False, label="ODA_FLAGS"),
    "ODBC_VS_ARGS": SimStruct(OrderedDict((("pguidEvent", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"wszArg": SimTypePointer(SimTypeChar(label="Char"), offset=0), "szArg": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"wszCorrelation": SimTypePointer(SimTypeChar(label="Char"), offset=0), "szCorrelation": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("RetCode", SimTypeShort(signed=True, label="Int16")),)), name="ODBC_VS_ARGS", pack=False, align=None),
    "ODS_FLAGS": SimTypeInt(signed=False, label="ODS_FLAGS"),
    "OEMCUIPCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CPSUICBPARAM", SimStruct), offset=0), SimTypePointer(SimTypeRef("OEMCUIPPARAM", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "OEMCUIPPARAM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("poemuiobj", SimTypePointer(SimTypeRef("OEMUIOBJ", SimStruct), offset=0)), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hOEMHeap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPublicDM", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pOEMDM", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pDrvOptItems", SimTypePointer(SimTypeRef("OPTITEM", SimStruct), offset=0)), ("cDrvOptItems", SimTypeInt(signed=False, label="UInt32")), ("pOEMOptItems", SimTypePointer(SimTypeRef("OPTITEM", SimStruct), offset=0)), ("cOEMOptItems", SimTypeInt(signed=False, label="UInt32")), ("pOEMUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OEMCUIPCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CPSUICBPARAM", SimStruct), offset=0), SimTypePointer(SimTypeRef("OEMCUIPPARAM", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)),)), name="OEMCUIPPARAM", pack=False, align=None),
    "OEMDMPARAM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pdriverobj", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPublicDMIn", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pPublicDMOut", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pOEMDMIn", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pOEMDMOut", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbBufSize", SimTypeInt(signed=False, label="UInt32")),)), name="OEMDMPARAM", pack=False, align=None),
    "OEMFONTINSTPARAM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hHeap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pFontInstallerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="OEMFONTINSTPARAM", pack=False, align=None),
    "OEMUIOBJ": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pOemUIProcs", SimTypePointer(SimTypeRef("OEMUIPROCS", SimStruct), offset=0)),)), name="OEMUIOBJ", pack=False, align=None),
    "OEMUIPROCS": SimStruct(OrderedDict((("DrvGetDriverSetting", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pdriverobj", "Feature", "pOutput", "cbSize", "pcbNeeded", "pdwOptionsReturned"]), offset=0)), ("DrvUpdateUISetting", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pdriverobj", "pOptItem", "dwPreviousSelection", "dwMode"]), offset=0)),)), name="OEMUIPROCS", pack=False, align=None),
    "OEMUIPSPARAM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("poemuiobj", SimTypePointer(SimTypeRef("OEMUIOBJ", SimStruct), offset=0)), ("hPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hOEMHeap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pPublicDM", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pOEMDM", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pOEMUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pOemEntry", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OEMUIPSPARAM", pack=False, align=None),
    "OEM_DMEXTRAHEADER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")),)), name="OEM_DMEXTRAHEADER", pack=False, align=None),
    "OEM_SOURCE_MEDIA_TYPE": SimTypeInt(signed=False, label="OEM_SOURCE_MEDIA_TYPE"),
    "OFFER_PRIORITY": SimTypeInt(signed=False, label="OFFER_PRIORITY"),
    "OFFLINEFILES_CACHING_MODE": SimTypeInt(signed=False, label="OFFLINEFILES_CACHING_MODE"),
    "OFFLINEFILES_COMPARE": SimTypeInt(signed=False, label="OFFLINEFILES_COMPARE"),
    "OFFLINEFILES_CONNECT_STATE": SimTypeInt(signed=False, label="OFFLINEFILES_CONNECT_STATE"),
    "OFFLINEFILES_EVENTS": SimTypeInt(signed=False, label="OFFLINEFILES_EVENTS"),
    "OFFLINEFILES_ITEM_COPY": SimTypeInt(signed=False, label="OFFLINEFILES_ITEM_COPY"),
    "OFFLINEFILES_ITEM_TIME": SimTypeInt(signed=False, label="OFFLINEFILES_ITEM_TIME"),
    "OFFLINEFILES_ITEM_TYPE": SimTypeInt(signed=False, label="OFFLINEFILES_ITEM_TYPE"),
    "OFFLINEFILES_OFFLINE_REASON": SimTypeInt(signed=False, label="OFFLINEFILES_OFFLINE_REASON"),
    "OFFLINEFILES_OP_RESPONSE": SimTypeInt(signed=False, label="OFFLINEFILES_OP_RESPONSE"),
    "OFFLINEFILES_PATHFILTER_MATCH": SimTypeInt(signed=False, label="OFFLINEFILES_PATHFILTER_MATCH"),
    "OFFLINEFILES_SETTING_VALUE_TYPE": SimTypeInt(signed=False, label="OFFLINEFILES_SETTING_VALUE_TYPE"),
    "OFFLINEFILES_SYNC_CONFLICT_RESOLVE": SimTypeInt(signed=False, label="OFFLINEFILES_SYNC_CONFLICT_RESOLVE"),
    "OFFLINEFILES_SYNC_OPERATION": SimTypeInt(signed=False, label="OFFLINEFILES_SYNC_OPERATION"),
    "OFFLINEFILES_SYNC_STATE": SimTypeInt(signed=False, label="OFFLINEFILES_SYNC_STATE"),
    "OFFLOAD_ALGO_INFO": SimStruct(OrderedDict((("algoIdentifier", SimTypeInt(signed=False, label="UInt32")), ("algoKeylen", SimTypeInt(signed=False, label="UInt32")), ("algoRounds", SimTypeInt(signed=False, label="UInt32")),)), name="OFFLOAD_ALGO_INFO", pack=False, align=None),
    "OFFLOAD_CONF_ALGO": SimTypeInt(signed=False, label="OFFLOAD_CONF_ALGO"),
    "OFFLOAD_INTEGRITY_ALGO": SimTypeInt(signed=False, label="OFFLOAD_INTEGRITY_ALGO"),
    "OFFLOAD_IPSEC_ADD_SA": SimStruct(OrderedDict((("SrcAddr", SimTypeInt(signed=False, label="UInt32")), ("SrcMask", SimTypeInt(signed=False, label="UInt32")), ("DestAddr", SimTypeInt(signed=False, label="UInt32")), ("DestMask", SimTypeInt(signed=False, label="UInt32")), ("Protocol", SimTypeInt(signed=False, label="UInt32")), ("SrcPort", SimTypeShort(signed=False, label="UInt16")), ("DestPort", SimTypeShort(signed=False, label="UInt16")), ("SrcTunnelAddr", SimTypeInt(signed=False, label="UInt32")), ("DestTunnelAddr", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("NumSAs", SimTypeShort(signed=True, label="Int16")), ("SecAssoc", SimTypeArray(SimTypeRef("OFFLOAD_SECURITY_ASSOCIATION", SimStruct), 3)), ("OffloadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("KeyLen", SimTypeInt(signed=False, label="UInt32")), ("KeyMat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OFFLOAD_IPSEC_ADD_SA", pack=False, align=None),
    "OFFLOAD_IPSEC_ADD_UDPESP_SA": SimStruct(OrderedDict((("SrcAddr", SimTypeInt(signed=False, label="UInt32")), ("SrcMask", SimTypeInt(signed=False, label="UInt32")), ("DstAddr", SimTypeInt(signed=False, label="UInt32")), ("DstMask", SimTypeInt(signed=False, label="UInt32")), ("Protocol", SimTypeInt(signed=False, label="UInt32")), ("SrcPort", SimTypeShort(signed=False, label="UInt16")), ("DstPort", SimTypeShort(signed=False, label="UInt16")), ("SrcTunnelAddr", SimTypeInt(signed=False, label="UInt32")), ("DstTunnelAddr", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("NumSAs", SimTypeShort(signed=True, label="Int16")), ("SecAssoc", SimTypeArray(SimTypeRef("OFFLOAD_SECURITY_ASSOCIATION", SimStruct), 3)), ("OffloadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EncapTypeEntry", SimTypeRef("OFFLOAD_IPSEC_UDPESP_ENCAPTYPE_ENTRY", SimStruct)), ("EncapTypeEntryOffldHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("KeyLen", SimTypeInt(signed=False, label="UInt32")), ("KeyMat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OFFLOAD_IPSEC_ADD_UDPESP_SA", pack=False, align=None),
    "OFFLOAD_IPSEC_DELETE_SA": SimStruct(OrderedDict((("OffloadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="OFFLOAD_IPSEC_DELETE_SA", pack=False, align=None),
    "OFFLOAD_IPSEC_DELETE_UDPESP_SA": SimStruct(OrderedDict((("OffloadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EncapTypeEntryOffldHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="OFFLOAD_IPSEC_DELETE_UDPESP_SA", pack=False, align=None),
    "OFFLOAD_IPSEC_UDPESP_ENCAPTYPE_ENTRY": SimStruct(OrderedDict((("UdpEncapType", SimTypeInt(signed=False, label="UDP_ENCAP_TYPE")), ("DstEncapPort", SimTypeShort(signed=False, label="UInt16")),)), name="OFFLOAD_IPSEC_UDPESP_ENCAPTYPE_ENTRY", pack=False, align=None),
    "OFFLOAD_OPERATION_E": SimTypeInt(signed=False, label="OFFLOAD_OPERATION_E"),
    "OFFLOAD_SECURITY_ASSOCIATION": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="OFFLOAD_OPERATION_E")), ("SPI", SimTypeInt(signed=False, label="UInt32")), ("IntegrityAlgo", SimTypeRef("OFFLOAD_ALGO_INFO", SimStruct)), ("ConfAlgo", SimTypeRef("OFFLOAD_ALGO_INFO", SimStruct)), ("Reserved", SimTypeRef("OFFLOAD_ALGO_INFO", SimStruct)),)), name="OFFLOAD_SECURITY_ASSOCIATION", pack=False, align=None),
    "OFFSETINSTANCEDATAANDLENGTH": SimStruct(OrderedDict((("OffsetInstanceData", SimTypeInt(signed=False, label="UInt32")), ("LengthInstanceData", SimTypeInt(signed=False, label="UInt32")),)), name="OFFSETINSTANCEDATAANDLENGTH", pack=False, align=None),
    "OFFSETTYPE": SimTypeInt(signed=False, label="OFFSETTYPE"),
    "OFNOTIFYA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lpOFN", SimTypePointer(SimTypeRef("OPENFILENAMEA", SimStruct), offset=0)), ("pszFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OFNOTIFYA", pack=False, align=None),
    "OFNOTIFYEXA": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lpOFN", SimTypePointer(SimTypeRef("OPENFILENAMEA", SimStruct), offset=0)), ("psf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pidl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OFNOTIFYEXA", pack=False, align=None),
    "OFNOTIFYEXW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lpOFN", SimTypePointer(SimTypeRef("OPENFILENAMEW", SimStruct), offset=0)), ("psf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pidl", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OFNOTIFYEXW", pack=False, align=None),
    "OFNOTIFYW": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lpOFN", SimTypePointer(SimTypeRef("OPENFILENAMEW", SimStruct), offset=0)), ("pszFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="OFNOTIFYW", pack=False, align=None),
    "OFSTRUCT": SimStruct(OrderedDict((("cBytes", SimTypeChar(label="Byte")), ("fFixedDisk", SimTypeChar(label="Byte")), ("nErrCode", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("szPathName", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="OFSTRUCT", pack=False, align=None),
    "OIBDG_FLAGS": SimTypeInt(signed=False, label="OIBDG_FLAGS"),
    "OIEXT": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("hInstCaller", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pHelpFile", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 4)),)), name="OIEXT", pack=False, align=None),
    "OLECLOSE": SimTypeInt(signed=False, label="OLECLOSE"),
    "OLECMD": SimStruct(OrderedDict((("cmdID", SimTypeInt(signed=False, label="UInt32")), ("cmdf", SimTypeInt(signed=False, label="UInt32")),)), name="OLECMD", pack=False, align=None),
    "OLECMDEXECOPT": SimTypeInt(signed=False, label="OLECMDEXECOPT"),
    "OLECMDF": SimTypeInt(signed=False, label="OLECMDF"),
    "OLECMDID": SimTypeInt(signed=False, label="OLECMDID"),
    "OLECMDID_BROWSERSTATEFLAG": SimTypeInt(signed=False, label="OLECMDID_BROWSERSTATEFLAG"),
    "OLECMDID_OPTICAL_ZOOMFLAG": SimTypeInt(signed=False, label="OLECMDID_OPTICAL_ZOOMFLAG"),
    "OLECMDID_PAGEACTIONFLAG": SimTypeInt(signed=False, label="OLECMDID_PAGEACTIONFLAG"),
    "OLECMDID_REFRESHFLAG": SimTypeInt(signed=False, label="OLECMDID_REFRESHFLAG"),
    "OLECMDID_VIEWPORT_MODE_FLAG": SimTypeInt(signed=False, label="OLECMDID_VIEWPORT_MODE_FLAG"),
    "OLECMDID_WINDOWSTATE_FLAG": SimTypeInt(signed=False, label="OLECMDID_WINDOWSTATE_FLAG"),
    "OLECMDTEXT": SimStruct(OrderedDict((("cmdtextf", SimTypeInt(signed=False, label="UInt32")), ("cwActual", SimTypeInt(signed=False, label="UInt32")), ("cwBuf", SimTypeInt(signed=False, label="UInt32")), ("rgwz", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="OLECMDTEXT", pack=False, align=None),
    "OLECMDTEXTF": SimTypeInt(signed=False, label="OLECMDTEXTF"),
    "OLECONTF": SimTypeInt(signed=False, label="OLECONTF"),
    "OLECREATE": SimTypeInt(signed=False, label="OLECREATE"),
    "OLEDBSimpleProvider": SimTypeBottom(label="OLEDBSimpleProvider"),
    "OLEDBSimpleProviderListener": SimTypeBottom(label="OLEDBSimpleProviderListener"),
    "OLEDCFLAGS": SimTypeInt(signed=False, label="OLEDCFLAGS"),
    "OLEGETMONIKER": SimTypeInt(signed=False, label="OLEGETMONIKER"),
    "OLEINPLACEFRAMEINFO": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("fMDIApp", SimTypeInt(signed=True, label="Int32")), ("hwndFrame", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("haccel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cAccelEntries", SimTypeInt(signed=False, label="UInt32")),)), name="OLEINPLACEFRAMEINFO", pack=False, align=None),
    "OLEIVERB": SimTypeInt(signed=False, label="OLEIVERB"),
    "OLELINKBIND": SimTypeInt(signed=False, label="OLELINKBIND"),
    "OLEMENUGROUPWIDTHS": SimStruct(OrderedDict((("width", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 6)),)), name="OLEMENUGROUPWIDTHS", pack=False, align=None),
    "OLEMISC": SimTypeInt(signed=False, label="OLEMISC"),
    "OLERENDER": SimTypeInt(signed=False, label="OLERENDER"),
    "OLESTREAM": SimStruct(OrderedDict((("lpstbl", SimTypePointer(SimTypeRef("OLESTREAMVTBL", SimStruct), offset=0)),)), name="OLESTREAM", pack=False, align=None),
    "OLESTREAMQUERYCONVERTOLELINKCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pClsid", "szClass", "szTopicName", "szItemName", "szUNCName", "linkUpdatingOption", "pvContext"]), offset=0),
    "OLESTREAMVTBL": SimStruct(OrderedDict((("Get", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Put", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="OLESTREAMVTBL", pack=False, align=None),
    "OLEUIBUSYA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="BUSY_DIALOG_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lphWndDialog", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="OLEUIBUSYA", pack=False, align=None),
    "OLEUIBUSYW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="BUSY_DIALOG_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hTask", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lphWndDialog", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="OLEUIBUSYW", pack=False, align=None),
    "OLEUICHANGEICONA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CHANGE_ICON_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("clsid", SimTypeBottom(label="Guid")), ("szIconExe", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("cchIconExe", SimTypeInt(signed=True, label="Int32")),)), name="OLEUICHANGEICONA", pack=False, align=None),
    "OLEUICHANGEICONW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CHANGE_ICON_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("clsid", SimTypeBottom(label="Guid")), ("szIconExe", SimTypeArray(SimTypeChar(label="Char"), 260)), ("cchIconExe", SimTypeInt(signed=True, label="Int32")),)), name="OLEUICHANGEICONW", pack=False, align=None),
    "OLEUICHANGESOURCEA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CHANGE_SOURCE_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpOFN", SimTypePointer(SimTypeRef("OPENFILENAMEA", SimStruct), offset=0)), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("lpOleUILinkContainer", SimTypeBottom(label="IOleUILinkContainerA")), ("dwLink", SimTypeInt(signed=False, label="UInt32")), ("lpszDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nFileLength", SimTypeInt(signed=False, label="UInt32")), ("lpszFrom", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszTo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OLEUICHANGESOURCEA", pack=False, align=None),
    "OLEUICHANGESOURCEW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="CHANGE_SOURCE_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpOFN", SimTypePointer(SimTypeRef("OPENFILENAMEW", SimStruct), offset=0)), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("lpOleUILinkContainer", SimTypeBottom(label="IOleUILinkContainerW")), ("dwLink", SimTypeInt(signed=False, label="UInt32")), ("lpszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nFileLength", SimTypeInt(signed=False, label="UInt32")), ("lpszFrom", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszTo", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="OLEUICHANGESOURCEW", pack=False, align=None),
    "OLEUICONVERTA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UI_CONVERT_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("clsid", SimTypeBottom(label="Guid")), ("clsidConvertDefault", SimTypeBottom(label="Guid")), ("clsidActivateDefault", SimTypeBottom(label="Guid")), ("clsidNew", SimTypeBottom(label="Guid")), ("dvAspect", SimTypeInt(signed=False, label="UInt32")), ("wFormat", SimTypeShort(signed=False, label="UInt16")), ("fIsLinkedObject", SimTypeInt(signed=True, label="Int32")), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpszUserType", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fObjectsIconChanged", SimTypeInt(signed=True, label="Int32")), ("lpszDefLabel", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cClsidExclude", SimTypeInt(signed=False, label="UInt32")), ("lpClsidExclude", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="OLEUICONVERTA", pack=False, align=None),
    "OLEUICONVERTW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UI_CONVERT_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("clsid", SimTypeBottom(label="Guid")), ("clsidConvertDefault", SimTypeBottom(label="Guid")), ("clsidActivateDefault", SimTypeBottom(label="Guid")), ("clsidNew", SimTypeBottom(label="Guid")), ("dvAspect", SimTypeInt(signed=False, label="UInt32")), ("wFormat", SimTypeShort(signed=False, label="UInt16")), ("fIsLinkedObject", SimTypeInt(signed=True, label="Int32")), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpszUserType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fObjectsIconChanged", SimTypeInt(signed=True, label="Int32")), ("lpszDefLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cClsidExclude", SimTypeInt(signed=False, label="UInt32")), ("lpClsidExclude", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="OLEUICONVERTW", pack=False, align=None),
    "OLEUIEDITLINKSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="EDIT_LINKS_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpOleUILinkContainer", SimTypeBottom(label="IOleUILinkContainerA")),)), name="OLEUIEDITLINKSA", pack=False, align=None),
    "OLEUIEDITLINKSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="EDIT_LINKS_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpOleUILinkContainer", SimTypeBottom(label="IOleUILinkContainerW")),)), name="OLEUIEDITLINKSW", pack=False, align=None),
    "OLEUIGNRLPROPSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("lpOP", SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSA", SimStruct), offset=0)),)), name="OLEUIGNRLPROPSA", pack=False, align=None),
    "OLEUIGNRLPROPSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("lpOP", SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSW", SimStruct), offset=0)),)), name="OLEUIGNRLPROPSW", pack=False, align=None),
    "OLEUIINSERTOBJECTA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="INSERT_OBJECT_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("clsid", SimTypeBottom(label="Guid")), ("lpszFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchFile", SimTypeInt(signed=False, label="UInt32")), ("cClsidExclude", SimTypeInt(signed=False, label="UInt32")), ("lpClsidExclude", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("iid", SimTypeBottom(label="Guid")), ("oleRender", SimTypeInt(signed=False, label="UInt32")), ("lpFormatEtc", SimTypePointer(SimTypeRef("FORMATETC", SimStruct), offset=0)), ("lpIOleClientSite", SimTypeBottom(label="IOleClientSite")), ("lpIStorage", SimTypeBottom(label="IStorage")), ("ppvObj", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("sc", SimTypeInt(signed=True, label="Int32")), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OLEUIINSERTOBJECTA", pack=False, align=None),
    "OLEUIINSERTOBJECTW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="INSERT_OBJECT_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("clsid", SimTypeBottom(label="Guid")), ("lpszFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchFile", SimTypeInt(signed=False, label="UInt32")), ("cClsidExclude", SimTypeInt(signed=False, label="UInt32")), ("lpClsidExclude", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("iid", SimTypeBottom(label="Guid")), ("oleRender", SimTypeInt(signed=False, label="UInt32")), ("lpFormatEtc", SimTypePointer(SimTypeRef("FORMATETC", SimStruct), offset=0)), ("lpIOleClientSite", SimTypeBottom(label="IOleClientSite")), ("lpIStorage", SimTypeBottom(label="IStorage")), ("ppvObj", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("sc", SimTypeInt(signed=True, label="Int32")), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="OLEUIINSERTOBJECTW", pack=False, align=None),
    "OLEUILINKPROPSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("lpOP", SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSA", SimStruct), offset=0)),)), name="OLEUILINKPROPSA", pack=False, align=None),
    "OLEUILINKPROPSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("lpOP", SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSW", SimStruct), offset=0)),)), name="OLEUILINKPROPSW", pack=False, align=None),
    "OLEUIOBJECTPROPSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="OBJECT_PROPERTIES_FLAGS")), ("lpPS", SimTypePointer(SimTypeRef("PROPSHEETHEADERA_V2", SimStruct), offset=0)), ("dwObject", SimTypeInt(signed=False, label="UInt32")), ("lpObjInfo", SimTypeBottom(label="IOleUIObjInfoA")), ("dwLink", SimTypeInt(signed=False, label="UInt32")), ("lpLinkInfo", SimTypeBottom(label="IOleUILinkInfoA")), ("lpGP", SimTypePointer(SimTypeRef("OLEUIGNRLPROPSA", SimStruct), offset=0)), ("lpVP", SimTypePointer(SimTypeRef("OLEUIVIEWPROPSA", SimStruct), offset=0)), ("lpLP", SimTypePointer(SimTypeRef("OLEUILINKPROPSA", SimStruct), offset=0)),)), name="OLEUIOBJECTPROPSA", pack=False, align=None),
    "OLEUIOBJECTPROPSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="OBJECT_PROPERTIES_FLAGS")), ("lpPS", SimTypePointer(SimTypeRef("PROPSHEETHEADERW_V2", SimStruct), offset=0)), ("dwObject", SimTypeInt(signed=False, label="UInt32")), ("lpObjInfo", SimTypeBottom(label="IOleUIObjInfoW")), ("dwLink", SimTypeInt(signed=False, label="UInt32")), ("lpLinkInfo", SimTypeBottom(label="IOleUILinkInfoW")), ("lpGP", SimTypePointer(SimTypeRef("OLEUIGNRLPROPSW", SimStruct), offset=0)), ("lpVP", SimTypePointer(SimTypeRef("OLEUIVIEWPROPSW", SimStruct), offset=0)), ("lpLP", SimTypePointer(SimTypeRef("OLEUILINKPROPSW", SimStruct), offset=0)),)), name="OLEUIOBJECTPROPSW", pack=False, align=None),
    "OLEUIPASTEENTRYA": SimStruct(OrderedDict((("fmtetc", SimTypeRef("FORMATETC", SimStruct)), ("lpstrFormatName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrResultText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwScratchSpace", SimTypeInt(signed=False, label="UInt32")),)), name="OLEUIPASTEENTRYA", pack=False, align=None),
    "OLEUIPASTEENTRYW": SimStruct(OrderedDict((("fmtetc", SimTypeRef("FORMATETC", SimStruct)), ("lpstrFormatName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrResultText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwScratchSpace", SimTypeInt(signed=False, label="UInt32")),)), name="OLEUIPASTEENTRYW", pack=False, align=None),
    "OLEUIPASTEFLAG": SimTypeInt(signed=False, label="OLEUIPASTEFLAG"),
    "OLEUIPASTESPECIALA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="PASTE_SPECIAL_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpSrcDataObj", SimTypeBottom(label="IDataObject")), ("arrPasteEntries", SimTypePointer(SimTypeRef("OLEUIPASTEENTRYA", SimStruct), offset=0)), ("cPasteEntries", SimTypeInt(signed=True, label="Int32")), ("arrLinkTypes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cLinkTypes", SimTypeInt(signed=True, label="Int32")), ("cClsidExclude", SimTypeInt(signed=False, label="UInt32")), ("lpClsidExclude", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("nSelectedIndex", SimTypeInt(signed=True, label="Int32")), ("fLink", SimTypeInt(signed=True, label="Int32")), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("sizel", SimTypeRef("SIZE", SimStruct)),)), name="OLEUIPASTESPECIALA", pack=False, align=None),
    "OLEUIPASTESPECIALW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="PASTE_SPECIAL_FLAGS")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hResource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpSrcDataObj", SimTypeBottom(label="IDataObject")), ("arrPasteEntries", SimTypePointer(SimTypeRef("OLEUIPASTEENTRYW", SimStruct), offset=0)), ("cPasteEntries", SimTypeInt(signed=True, label="Int32")), ("arrLinkTypes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cLinkTypes", SimTypeInt(signed=True, label="Int32")), ("cClsidExclude", SimTypeInt(signed=False, label="UInt32")), ("lpClsidExclude", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("nSelectedIndex", SimTypeInt(signed=True, label="Int32")), ("fLink", SimTypeInt(signed=True, label="Int32")), ("hMetaPict", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("sizel", SimTypeRef("SIZE", SimStruct)),)), name="OLEUIPASTESPECIALW", pack=False, align=None),
    "OLEUIVIEWPROPSA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="VIEW_OBJECT_PROPERTIES_FLAGS")), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("lpOP", SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSA", SimStruct), offset=0)), ("nScaleMin", SimTypeInt(signed=True, label="Int32")), ("nScaleMax", SimTypeInt(signed=True, label="Int32")),)), name="OLEUIVIEWPROPSA", pack=False, align=None),
    "OLEUIVIEWPROPSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="VIEW_OBJECT_PROPERTIES_FLAGS")), ("dwReserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("lpOP", SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSW", SimStruct), offset=0)), ("nScaleMin", SimTypeInt(signed=True, label="Int32")), ("nScaleMax", SimTypeInt(signed=True, label="Int32")),)), name="OLEUIVIEWPROPSW", pack=False, align=None),
    "OLEUPDATE": SimTypeInt(signed=False, label="OLEUPDATE"),
    "OLEVERB": SimStruct(OrderedDict((("lVerb", SimTypeInt(signed=False, label="OLEIVERB")), ("lpszVerbName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fuFlags", SimTypeInt(signed=False, label="MENU_ITEM_FLAGS")), ("grfAttribs", SimTypeInt(signed=False, label="UInt32")),)), name="OLEVERB", pack=False, align=None),
    "OLEVERBATTRIB": SimTypeInt(signed=False, label="OLEVERBATTRIB"),
    "OLEWHICHMK": SimTypeInt(signed=False, label="OLEWHICHMK"),
    "OLE_HANDLE": SimTypeInt(signed=False, label="UInt32"),
    "OLE_TM_CONFIG_PARAMS_V1": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwcConcurrencyHint", SimTypeInt(signed=False, label="UInt32")),)), name="OLE_TM_CONFIG_PARAMS_V1", pack=False, align=None),
    "OLE_TM_CONFIG_PARAMS_V2": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwcConcurrencyHint", SimTypeInt(signed=False, label="UInt32")), ("applicationType", SimTypeInt(signed=False, label="APPLICATIONTYPE")), ("clusterResourceId", SimTypeBottom(label="Guid")),)), name="OLE_TM_CONFIG_PARAMS_V2", pack=False, align=None),
    "OLE_TRISTATE": SimTypeInt(signed=False, label="OLE_TRISTATE"),
    "OLIADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="OLIADPCMWAVEFORMAT", pack=False, align=None),
    "OLICELPWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="OLICELPWAVEFORMAT", pack=False, align=None),
    "OLIGSMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="OLIGSMWAVEFORMAT", pack=False, align=None),
    "OLIOPRWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="OLIOPRWAVEFORMAT", pack=False, align=None),
    "OLISBCWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="OLISBCWAVEFORMAT", pack=False, align=None),
    "OMAP": SimStruct(OrderedDict((("rva", SimTypeInt(signed=False, label="UInt32")), ("rvaTo", SimTypeInt(signed=False, label="UInt32")),)), name="OMAP", pack=False, align=None),
    "ONDEMAND_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "ONEX_AUTH_IDENTITY": SimTypeInt(signed=False, label="ONEX_AUTH_IDENTITY"),
    "ONEX_AUTH_PARAMS": SimStruct(OrderedDict((("fUpdatePending", SimTypeInt(signed=True, label="Int32")), ("oneXConnProfile", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("authIdentity", SimTypeInt(signed=False, label="ONEX_AUTH_IDENTITY")), ("dwQuarantineState", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("dwSessionId", SimTypeInt(signed=False, label="UInt32")), ("hUserToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("OneXUserProfile", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("Identity", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("UserName", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("Domain", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)),)), name="ONEX_AUTH_PARAMS", pack=False, align=None),
    "ONEX_AUTH_RESTART_REASON": SimTypeInt(signed=False, label="ONEX_AUTH_RESTART_REASON"),
    "ONEX_AUTH_STATUS": SimTypeInt(signed=False, label="ONEX_AUTH_STATUS"),
    "ONEX_EAP_ERROR": SimStruct(OrderedDict((("dwWinError", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeRef("EAP_METHOD_TYPE", SimStruct)), ("dwReasonCode", SimTypeInt(signed=False, label="UInt32")), ("rootCauseGuid", SimTypeBottom(label="Guid")), ("repairGuid", SimTypeBottom(label="Guid")), ("helpLinkGuid", SimTypeBottom(label="Guid")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("RootCauseString", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("RepairString", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)),)), name="ONEX_EAP_ERROR", pack=False, align=None),
    "ONEX_EAP_METHOD_BACKEND_SUPPORT": SimTypeInt(signed=False, label="ONEX_EAP_METHOD_BACKEND_SUPPORT"),
    "ONEX_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="ONEX_NOTIFICATION_TYPE"),
    "ONEX_REASON_CODE": SimTypeInt(signed=False, label="ONEX_REASON_CODE"),
    "ONEX_RESULT_UPDATE_DATA": SimStruct(OrderedDict((("oneXStatus", SimTypeRef("ONEX_STATUS", SimStruct)), ("BackendSupport", SimTypeInt(signed=False, label="ONEX_EAP_METHOD_BACKEND_SUPPORT")), ("fBackendEngaged", SimTypeInt(signed=True, label="Int32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("authParams", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("eapError", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)),)), name="ONEX_RESULT_UPDATE_DATA", pack=False, align=None),
    "ONEX_STATUS": SimStruct(OrderedDict((("authStatus", SimTypeInt(signed=False, label="ONEX_AUTH_STATUS")), ("dwReason", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")),)), name="ONEX_STATUS", pack=False, align=None),
    "ONEX_USER_INFO": SimStruct(OrderedDict((("authIdentity", SimTypeInt(signed=False, label="ONEX_AUTH_IDENTITY")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("UserName", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)), ("DomainName", SimTypeRef("ONEX_VARIABLE_BLOB", SimStruct)),)), name="ONEX_USER_INFO", pack=False, align=None),
    "ONEX_VARIABLE_BLOB": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="ONEX_VARIABLE_BLOB", pack=False, align=None),
    "OOBE_COMPLETED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackContext"]), offset=0),
    "OPAQUECOMMAND": SimStruct(OrderedDict((("guidCommand", SimTypeBottom(label="Guid")), ("dwDataLen", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("abMAC", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="OPAQUECOMMAND", pack=False, align=None),
    "OPC_CANONICALIZATION_METHOD": SimTypeInt(signed=False, label="OPC_CANONICALIZATION_METHOD"),
    "OPC_CERTIFICATE_EMBEDDING_OPTION": SimTypeInt(signed=False, label="OPC_CERTIFICATE_EMBEDDING_OPTION"),
    "OPC_COMPRESSION_OPTIONS": SimTypeInt(signed=False, label="OPC_COMPRESSION_OPTIONS"),
    "OPC_READ_FLAGS": SimTypeInt(signed=False, label="OPC_READ_FLAGS"),
    "OPC_RELATIONSHIPS_SIGNING_OPTION": SimTypeInt(signed=False, label="OPC_RELATIONSHIPS_SIGNING_OPTION"),
    "OPC_RELATIONSHIP_SELECTOR": SimTypeInt(signed=False, label="OPC_RELATIONSHIP_SELECTOR"),
    "OPC_SIGNATURE_TIME_FORMAT": SimTypeInt(signed=False, label="OPC_SIGNATURE_TIME_FORMAT"),
    "OPC_SIGNATURE_VALIDATION_RESULT": SimTypeInt(signed=False, label="OPC_SIGNATURE_VALIDATION_RESULT"),
    "OPC_STREAM_IO_MODE": SimTypeInt(signed=False, label="OPC_STREAM_IO_MODE"),
    "OPC_URI_TARGET_MODE": SimTypeInt(signed=False, label="OPC_URI_TARGET_MODE"),
    "OPC_WRITE_FLAGS": SimTypeInt(signed=False, label="OPC_WRITE_FLAGS"),
    "OPENASINFO": SimStruct(OrderedDict((("pcszFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcszClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("oaifInFlags", SimTypeInt(signed=False, label="OPEN_AS_INFO_FLAGS")),)), name="OPENASINFO", pack=False, align=None),
    "OPENBOXSTATES": SimTypeInt(signed=False, label="OPENBOXSTATES"),
    "OPENCARDNAMEA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hSCardContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrGroupNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxGroupNames", SimTypeInt(signed=False, label="UInt32")), ("lpstrCardNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxCardNames", SimTypeInt(signed=False, label="UInt32")), ("rgguidInterfaces", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cguidInterfaces", SimTypeInt(signed=False, label="UInt32")), ("lpstrRdr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxRdr", SimTypeInt(signed=False, label="UInt32")), ("lpstrCard", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxCard", SimTypeInt(signed=False, label="UInt32")), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")), ("dwActiveProtocol", SimTypeInt(signed=False, label="UInt32")), ("lpfnConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnCheck", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("lpfnDisconnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0)), ("hCardHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="OPENCARDNAMEA", pack=False, align=None),
    "OPENCARDNAMEW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hSCardContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpstrGroupNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxGroupNames", SimTypeInt(signed=False, label="UInt32")), ("lpstrCardNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxCardNames", SimTypeInt(signed=False, label="UInt32")), ("rgguidInterfaces", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cguidInterfaces", SimTypeInt(signed=False, label="UInt32")), ("lpstrRdr", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxRdr", SimTypeInt(signed=False, label="UInt32")), ("lpstrCard", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxCard", SimTypeInt(signed=False, label="UInt32")), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")), ("dwActiveProtocol", SimTypeInt(signed=False, label="UInt32")), ("lpfnConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnCheck", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("lpfnDisconnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0)), ("hCardHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="OPENCARDNAMEW", pack=False, align=None),
    "OPENCARDNAME_EXA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("hSCardContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrSearchDesc", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pOpenCardSearchCriteria", SimTypePointer(SimTypeRef("OPENCARD_SEARCH_CRITERIAA", SimStruct), offset=0)), ("lpfnConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpstrRdr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxRdr", SimTypeInt(signed=False, label="UInt32")), ("lpstrCard", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxCard", SimTypeInt(signed=False, label="UInt32")), ("dwActiveProtocol", SimTypeInt(signed=False, label="UInt32")), ("hCardHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="OPENCARDNAME_EXA", pack=False, align=None),
    "OPENCARDNAME_EXW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("hSCardContext", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrSearchDesc", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pOpenCardSearchCriteria", SimTypePointer(SimTypeRef("OPENCARD_SEARCH_CRITERIAW", SimStruct), offset=0)), ("lpfnConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpstrRdr", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxRdr", SimTypeInt(signed=False, label="UInt32")), ("lpstrCard", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxCard", SimTypeInt(signed=False, label="UInt32")), ("dwActiveProtocol", SimTypeInt(signed=False, label="UInt32")), ("hCardHandle", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="OPENCARDNAME_EXW", pack=False, align=None),
    "OPENCARD_SEARCH_CRITERIAA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpstrGroupNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxGroupNames", SimTypeInt(signed=False, label="UInt32")), ("rgguidInterfaces", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cguidInterfaces", SimTypeInt(signed=False, label="UInt32")), ("lpstrCardNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxCardNames", SimTypeInt(signed=False, label="UInt32")), ("lpfnCheck", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("lpfnConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnDisconnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")),)), name="OPENCARD_SEARCH_CRITERIAA", pack=False, align=None),
    "OPENCARD_SEARCH_CRITERIAW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpstrGroupNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxGroupNames", SimTypeInt(signed=False, label="UInt32")), ("rgguidInterfaces", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cguidInterfaces", SimTypeInt(signed=False, label="UInt32")), ("lpstrCardNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxCardNames", SimTypeInt(signed=False, label="UInt32")), ("lpfnCheck", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("lpfnConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnDisconnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")),)), name="OPENCARD_SEARCH_CRITERIAW", pack=False, align=None),
    "OPENFILENAMEA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpstrFilter", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrCustomFilter", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxCustFilter", SimTypeInt(signed=False, label="UInt32")), ("nFilterIndex", SimTypeInt(signed=False, label="UInt32")), ("lpstrFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxFile", SimTypeInt(signed=False, label="UInt32")), ("lpstrFileTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxFileTitle", SimTypeInt(signed=False, label="UInt32")), ("lpstrInitialDir", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="OPEN_FILENAME_FLAGS")), ("nFileOffset", SimTypeShort(signed=False, label="UInt16")), ("nFileExtension", SimTypeShort(signed=False, label="UInt16")), ("lpstrDefExt", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("FlagsEx", SimTypeInt(signed=False, label="OPEN_FILENAME_FLAGS_EX")),)), name="OPENFILENAMEA", pack=False, align=None),
    "OPENFILENAMEW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpstrFilter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrCustomFilter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxCustFilter", SimTypeInt(signed=False, label="UInt32")), ("nFilterIndex", SimTypeInt(signed=False, label="UInt32")), ("lpstrFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxFile", SimTypeInt(signed=False, label="UInt32")), ("lpstrFileTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxFileTitle", SimTypeInt(signed=False, label="UInt32")), ("lpstrInitialDir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="OPEN_FILENAME_FLAGS")), ("nFileOffset", SimTypeShort(signed=False, label="UInt16")), ("nFileExtension", SimTypeShort(signed=False, label="UInt16")), ("lpstrDefExt", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("FlagsEx", SimTypeInt(signed=False, label="OPEN_FILENAME_FLAGS_EX")),)), name="OPENFILENAMEW", pack=False, align=None),
    "OPENFILENAME_NT4A": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpstrFilter", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrCustomFilter", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxCustFilter", SimTypeInt(signed=False, label="UInt32")), ("nFilterIndex", SimTypeInt(signed=False, label="UInt32")), ("lpstrFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxFile", SimTypeInt(signed=False, label="UInt32")), ("lpstrFileTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nMaxFileTitle", SimTypeInt(signed=False, label="UInt32")), ("lpstrInitialDir", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("nFileOffset", SimTypeShort(signed=False, label="UInt16")), ("nFileExtension", SimTypeShort(signed=False, label="UInt16")), ("lpstrDefExt", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OPENFILENAME_NT4A", pack=False, align=None),
    "OPENFILENAME_NT4W": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpstrFilter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrCustomFilter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxCustFilter", SimTypeInt(signed=False, label="UInt32")), ("nFilterIndex", SimTypeInt(signed=False, label="UInt32")), ("lpstrFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxFile", SimTypeInt(signed=False, label="UInt32")), ("lpstrFileTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxFileTitle", SimTypeInt(signed=False, label="UInt32")), ("lpstrInitialDir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpstrTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("nFileOffset", SimTypeShort(signed=False, label="UInt16")), ("nFileExtension", SimTypeShort(signed=False, label="UInt16")), ("lpstrDefExt", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="OPENFILENAME_NT4W", pack=False, align=None),
    "OPENQUERYWINDOW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("clsidHandler", SimTypeBottom(label="Guid")), ("pHandlerParameters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("clsidDefaultForm", SimTypeBottom(label="Guid")), ("pPersistQuery", SimTypeBottom(label="IPersistQuery")), ("Anonymous", SimUnion({"pFormParameters": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ppbFormParameters": SimTypeBottom(label="IPropertyBag")}, name="<anon>", label="None")),)), name="OPENQUERYWINDOW", pack=False, align=None),
    "OPENTYPE_FEATURE_RECORD": SimStruct(OrderedDict((("tagFeature", SimTypeInt(signed=False, label="UInt32")), ("lParameter", SimTypeInt(signed=True, label="Int32")),)), name="OPENTYPE_FEATURE_RECORD", pack=False, align=None),
    "OPEN_AS_INFO_FLAGS": SimTypeInt(signed=False, label="OPEN_AS_INFO_FLAGS"),
    "OPEN_FILENAME_FLAGS": SimTypeInt(signed=False, label="OPEN_FILENAME_FLAGS"),
    "OPEN_FILENAME_FLAGS_EX": SimTypeInt(signed=False, label="OPEN_FILENAME_FLAGS_EX"),
    "OPEN_PRINTER_PROPS_INFOA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pszSheetName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uSheetIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bModal", SimTypeInt(signed=True, label="Int32")),)), name="OPEN_PRINTER_PROPS_INFOA", pack=False, align=None),
    "OPEN_PRINTER_PROPS_INFOW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pszSheetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("uSheetIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("bModal", SimTypeInt(signed=True, label="Int32")),)), name="OPEN_PRINTER_PROPS_INFOW", pack=False, align=None),
    "OPEN_REPARSE_LIST": SimStruct(OrderedDict((("OpenReparseList", SimTypeRef("LIST_ENTRY", SimStruct)),)), name="OPEN_REPARSE_LIST", pack=False, align=None),
    "OPEN_REPARSE_LIST_ENTRY": SimStruct(OrderedDict((("OpenReparseListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("ReparseTag", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ReparseGuid", SimTypeBottom(label="Guid")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("RemainingLength", SimTypeShort(signed=False, label="UInt16")),)), name="OPEN_REPARSE_LIST_ENTRY", pack=False, align=None),
    "OPEN_THEME_DATA_FLAGS": SimTypeInt(signed=False, label="OPEN_THEME_DATA_FLAGS"),
    "OPEN_THREAD_WAIT_CHAIN_SESSION_FLAGS": SimTypeInt(signed=False, label="OPEN_THREAD_WAIT_CHAIN_SESSION_FLAGS"),
    "OPEN_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="OPEN_VIRTUAL_DISK_FLAG"),
    "OPEN_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="OPEN_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct), "Version2": SimTypeRef("_Version2_e__Struct", SimStruct), "Version3": SimTypeRef("_Version3_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="OPEN_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "OPEN_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="OPEN_VIRTUAL_DISK_VERSION"),
    "OPERATION_END_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("OperationId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="OPERATION_END_PARAMETERS_FLAGS")),)), name="OPERATION_END_PARAMETERS", pack=False, align=None),
    "OPERATION_END_PARAMETERS_FLAGS": SimTypeInt(signed=False, label="OPERATION_END_PARAMETERS_FLAGS"),
    "OPERATION_START_FLAGS": SimTypeInt(signed=False, label="OPERATION_START_FLAGS"),
    "OPERATION_START_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("OperationId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="OPERATION_START_FLAGS")),)), name="OPERATION_START_PARAMETERS", pack=False, align=None),
    "OPLOCK_KEY_CONTEXT": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("ParentOplockKey", SimTypeBottom(label="Guid")), ("TargetOplockKey", SimTypeBottom(label="Guid")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="OPLOCK_KEY_CONTEXT", pack=False, align=None),
    "OPLOCK_KEY_ECP_CONTEXT": SimStruct(OrderedDict((("OplockKey", SimTypeBottom(label="Guid")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="OPLOCK_KEY_ECP_CONTEXT", pack=False, align=None),
    "OPLOCK_NOTIFY_PARAMS": SimStruct(OrderedDict((("NotifyReason", SimTypeInt(signed=False, label="OPLOCK_NOTIFY_REASON")), ("NotifyContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Irp", SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="OPLOCK_NOTIFY_PARAMS", pack=False, align=None),
    "OPLOCK_NOTIFY_REASON": SimTypeInt(signed=False, label="OPLOCK_NOTIFY_REASON"),
    "OPM_ACP_AND_CGMSA_SIGNALING": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulAvailableTVProtectionStandards", SimTypeInt(signed=False, label="UInt32")), ("ulActiveTVProtectionStandard", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioValidMask1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioValidMask2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioValidMask3", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData3", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ulReserved3", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="OPM_ACP_AND_CGMSA_SIGNALING", pack=False, align=None),
    "OPM_ACP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="OPM_ACP_PROTECTION_LEVEL"),
    "OPM_ACTUAL_OUTPUT_FORMAT": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulDisplayWidth", SimTypeInt(signed=False, label="UInt32")), ("ulDisplayHeight", SimTypeInt(signed=False, label="UInt32")), ("dsfSampleInterleaveFormat", SimTypeInt(signed=False, label="DXVA2_SampleFormat")), ("d3dFormat", SimTypeInt(signed=False, label="D3DFORMAT")), ("ulFrequencyNumerator", SimTypeInt(signed=False, label="UInt32")), ("ulFrequencyDenominator", SimTypeInt(signed=False, label="UInt32")),)), name="OPM_ACTUAL_OUTPUT_FORMAT", pack=False, align=None),
    "OPM_BUS_TYPE": SimTypeInt(signed=False, label="OPM_BUS_TYPE"),
    "OPM_CGMSA": SimTypeInt(signed=False, label="OPM_CGMSA"),
    "OPM_CONFIGURE_PARAMETERS": SimStruct(OrderedDict((("omac", SimTypeRef("OPM_OMAC", SimStruct)), ("guidSetting", SimTypeBottom(label="Guid")), ("ulSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("cbParametersSize", SimTypeInt(signed=False, label="UInt32")), ("abParameters", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="OPM_CONFIGURE_PARAMETERS", pack=False, align=None),
    "OPM_CONNECTED_HDCP_DEVICE_INFORMATION": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulHDCPFlags", SimTypeInt(signed=False, label="UInt32")), ("ksvB", SimTypeRef("OPM_HDCP_KEY_SELECTION_VECTOR", SimStruct)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 11)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="OPM_CONNECTED_HDCP_DEVICE_INFORMATION", pack=False, align=None),
    "OPM_CONNECTOR_TYPE": SimTypeInt(signed=False, label="OPM_CONNECTOR_TYPE"),
    "OPM_COPP_COMPATIBLE_GET_INFO_PARAMETERS": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("guidInformation", SimTypeBottom(label="Guid")), ("ulSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("cbParametersSize", SimTypeInt(signed=False, label="UInt32")), ("abParameters", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="OPM_COPP_COMPATIBLE_GET_INFO_PARAMETERS", pack=False, align=None),
    "OPM_DPCP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="OPM_DPCP_PROTECTION_LEVEL"),
    "OPM_DVI_CHARACTERISTIC": SimTypeInt(signed=False, label="OPM_DVI_CHARACTERISTIC"),
    "OPM_ENCRYPTED_INITIALIZATION_PARAMETERS": SimStruct(OrderedDict((("abEncryptedInitializationParameters", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="OPM_ENCRYPTED_INITIALIZATION_PARAMETERS", pack=False, align=None),
    "OPM_GET_CODEC_INFO_INFORMATION": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("Merit", SimTypeInt(signed=False, label="UInt32")),)), name="OPM_GET_CODEC_INFO_INFORMATION", pack=False, align=None),
    "OPM_GET_CODEC_INFO_PARAMETERS": SimStruct(OrderedDict((("cbVerifier", SimTypeInt(signed=False, label="UInt32")), ("Verifier", SimTypeArray(SimTypeChar(label="Byte"), 4052)),)), name="OPM_GET_CODEC_INFO_PARAMETERS", pack=False, align=None),
    "OPM_GET_INFO_PARAMETERS": SimStruct(OrderedDict((("omac", SimTypeRef("OPM_OMAC", SimStruct)), ("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("guidInformation", SimTypeBottom(label="Guid")), ("ulSequenceNumber", SimTypeInt(signed=False, label="UInt32")), ("cbParametersSize", SimTypeInt(signed=False, label="UInt32")), ("abParameters", SimTypeArray(SimTypeChar(label="Byte"), 4056)),)), name="OPM_GET_INFO_PARAMETERS", pack=False, align=None),
    "OPM_HDCP_FLAGS": SimTypeInt(signed=False, label="OPM_HDCP_FLAGS"),
    "OPM_HDCP_KEY_SELECTION_VECTOR": SimStruct(OrderedDict((("abKeySelectionVector", SimTypeArray(SimTypeChar(label="Byte"), 5)),)), name="OPM_HDCP_KEY_SELECTION_VECTOR", pack=False, align=None),
    "OPM_HDCP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="OPM_HDCP_PROTECTION_LEVEL"),
    "OPM_HDCP_STATUS": SimTypeInt(signed=False, label="OPM_HDCP_STATUS"),
    "OPM_HDCP_TYPE": SimTypeInt(signed=False, label="OPM_HDCP_TYPE"),
    "OPM_IMAGE_ASPECT_RATIO_EN300294": SimTypeInt(signed=False, label="OPM_IMAGE_ASPECT_RATIO_EN300294"),
    "OPM_OMAC": SimStruct(OrderedDict((("abOMAC", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="OPM_OMAC", pack=False, align=None),
    "OPM_OUTPUT_HARDWARE_PROTECTION": SimTypeInt(signed=False, label="OPM_OUTPUT_HARDWARE_PROTECTION"),
    "OPM_OUTPUT_ID_DATA": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("OutputId", SimTypeLongLong(signed=False, label="UInt64")),)), name="OPM_OUTPUT_ID_DATA", pack=False, align=None),
    "OPM_PROTECTION_STANDARD_TYPE": SimTypeInt(signed=False, label="OPM_PROTECTION_STANDARD_TYPE"),
    "OPM_PROTECTION_TYPE": SimTypeInt(signed=False, label="OPM_PROTECTION_TYPE"),
    "OPM_RANDOM_NUMBER": SimStruct(OrderedDict((("abRandomNumber", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="OPM_RANDOM_NUMBER", pack=False, align=None),
    "OPM_REQUESTED_INFORMATION": SimStruct(OrderedDict((("omac", SimTypeRef("OPM_OMAC", SimStruct)), ("cbRequestedInformationSize", SimTypeInt(signed=False, label="UInt32")), ("abRequestedInformation", SimTypeArray(SimTypeChar(label="Byte"), 4076)),)), name="OPM_REQUESTED_INFORMATION", pack=False, align=None),
    "OPM_SET_ACP_AND_CGMSA_SIGNALING_PARAMETERS": SimStruct(OrderedDict((("ulNewTVProtectionStandard", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioChangeMask1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData1", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioChangeMask2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData2", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioChangeMask3", SimTypeInt(signed=False, label="UInt32")), ("ulAspectRatioData3", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ulReserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("ulReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="OPM_SET_ACP_AND_CGMSA_SIGNALING_PARAMETERS", pack=False, align=None),
    "OPM_SET_HDCP_SRM_PARAMETERS": SimStruct(OrderedDict((("ulSRMVersion", SimTypeInt(signed=False, label="UInt32")),)), name="OPM_SET_HDCP_SRM_PARAMETERS", pack=False, align=None),
    "OPM_SET_PROTECTION_LEVEL_PARAMETERS": SimStruct(OrderedDict((("ulProtectionType", SimTypeInt(signed=False, label="UInt32")), ("ulProtectionLevel", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="OPM_SET_PROTECTION_LEVEL_PARAMETERS", pack=False, align=None),
    "OPM_STANDARD_INFORMATION": SimStruct(OrderedDict((("rnRandomNumber", SimTypeRef("OPM_RANDOM_NUMBER", SimStruct)), ("ulStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("ulInformation", SimTypeInt(signed=False, label="UInt32")), ("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="OPM_STANDARD_INFORMATION", pack=False, align=None),
    "OPM_STATUS": SimTypeInt(signed=False, label="OPM_STATUS"),
    "OPM_TYPE": SimTypeInt(signed=False, label="OPM_TYPE"),
    "OPM_TYPE_ENFORCEMENT_HDCP_PROTECTION_LEVEL": SimTypeInt(signed=False, label="OPM_TYPE_ENFORCEMENT_HDCP_PROTECTION_LEVEL"),
    "OPM_VIDEO_OUTPUT_SEMANTICS": SimTypeInt(signed=False, label="OPM_VIDEO_OUTPUT_SEMANTICS"),
    "OPTCOMBO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("cListItem", SimTypeShort(signed=False, label="UInt16")), ("pListItem", SimTypePointer(SimTypeRef("OPTPARAM", SimStruct), offset=0)), ("Sel", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="OPTCOMBO", pack=False, align=None),
    "OPTIMAL_WEIGHT_TOTALS": SimStruct(OrderedDict((("MinTotalNominator", SimTypeLongLong(signed=True, label="Int64")), ("MaxTotalNominator", SimTypeLongLong(signed=True, label="Int64")), ("TotalDenominator", SimTypeLongLong(signed=True, label="Int64")),)), name="OPTIMAL_WEIGHT_TOTALS", pack=False, align=None),
    "OPTITEM": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Level", SimTypeChar(label="Byte")), ("DlgPageIdx", SimTypeChar(label="Byte")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("UserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Anonymous1", SimUnion({"Sel": SimTypeInt(signed=True, label="Int32"), "pSel": SimTypePointer(SimTypeChar(label="SByte"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"pExtChkBox": SimTypePointer(SimTypeRef("EXTCHKBOX", SimStruct), offset=0), "pExtPush": SimTypePointer(SimTypeRef("EXTPUSH", SimStruct), offset=0)}, name="<anon>", label="None")), ("pOptType", SimTypePointer(SimTypeRef("OPTTYPE", SimStruct), offset=0)), ("HelpIndex", SimTypeInt(signed=False, label="UInt32")), ("DMPubID", SimTypeChar(label="Byte")), ("UserItemID", SimTypeChar(label="Byte")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("pOIExt", SimTypePointer(SimTypeRef("OIEXT", SimStruct), offset=0)), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)),)), name="OPTITEM", pack=False, align=None),
    "OPTPARAM": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("Style", SimTypeChar(label="Byte")), ("pData", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("IconID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)),)), name="OPTPARAM", pack=False, align=None),
    "OPTTYPE": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Count", SimTypeShort(signed=False, label="UInt16")), ("BegCtrlID", SimTypeShort(signed=False, label="UInt16")), ("pOptParam", SimTypePointer(SimTypeRef("OPTPARAM", SimStruct), offset=0)), ("Style", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)),)), name="OPTTYPE", pack=False, align=None),
    "ORASADFUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "ORHKEY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ORIENTATION_PREFERENCE": SimTypeInt(signed=False, label="ORIENTATION_PREFERENCE"),
    "OS": SimTypeInt(signed=False, label="OS"),
    "OSPCOMP": SimTypeInt(signed=False, label="OSPCOMP"),
    "OSPFIND": SimTypeInt(signed=False, label="OSPFIND"),
    "OSPFORMAT": SimTypeInt(signed=False, label="OSPFORMAT"),
    "OSPRW": SimTypeInt(signed=False, label="OSPRW"),
    "OSPXFER": SimTypeInt(signed=False, label="OSPXFER"),
    "OSUpdateAssessment": SimStruct(OrderedDict((("isEndOfSupport", SimTypeInt(signed=True, label="Int32")), ("assessmentForCurrent", SimTypeRef("UpdateAssessment", SimStruct)), ("assessmentForUpToDate", SimTypeRef("UpdateAssessment", SimStruct)), ("securityStatus", SimTypeInt(signed=False, label="UpdateAssessmentStatus")), ("assessmentTime", SimTypeRef("FILETIME", SimStruct)), ("releaseInfoTime", SimTypeRef("FILETIME", SimStruct)), ("currentOSBuild", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("currentOSReleaseTime", SimTypeRef("FILETIME", SimStruct)), ("upToDateOSBuild", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("upToDateOSReleaseTime", SimTypeRef("FILETIME", SimStruct)),)), name="OSUpdateAssessment", pack=False, align=None),
    "OSVERSIONINFOA": SimStruct(OrderedDict((("dwOSVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("dwPlatformId", SimTypeInt(signed=False, label="UInt32")), ("szCSDVersion", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="OSVERSIONINFOA", pack=False, align=None),
    "OSVERSIONINFOEXA": SimStruct(OrderedDict((("dwOSVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("dwPlatformId", SimTypeInt(signed=False, label="UInt32")), ("szCSDVersion", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("wServicePackMajor", SimTypeShort(signed=False, label="UInt16")), ("wServicePackMinor", SimTypeShort(signed=False, label="UInt16")), ("wSuiteMask", SimTypeShort(signed=False, label="UInt16")), ("wProductType", SimTypeChar(label="Byte")), ("wReserved", SimTypeChar(label="Byte")),)), name="OSVERSIONINFOEXA", pack=False, align=None),
    "OSVERSIONINFOEXW": SimStruct(OrderedDict((("dwOSVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("dwPlatformId", SimTypeInt(signed=False, label="UInt32")), ("szCSDVersion", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wServicePackMajor", SimTypeShort(signed=False, label="UInt16")), ("wServicePackMinor", SimTypeShort(signed=False, label="UInt16")), ("wSuiteMask", SimTypeShort(signed=False, label="UInt16")), ("wProductType", SimTypeChar(label="Byte")), ("wReserved", SimTypeChar(label="Byte")),)), name="OSVERSIONINFOEXW", pack=False, align=None),
    "OSVERSIONINFOW": SimStruct(OrderedDict((("dwOSVersionInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("dwPlatformId", SimTypeInt(signed=False, label="UInt32")), ("szCSDVersion", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="OSVERSIONINFOW", pack=False, align=None),
    "OS_DEPLOYEMENT_STATE_VALUES": SimTypeInt(signed=False, label="OS_DEPLOYEMENT_STATE_VALUES"),
    "OS_INFO": SimStruct(OrderedDict((("MajorVer", SimTypeInt(signed=False, label="UInt32")), ("MinorVer", SimTypeInt(signed=False, label="UInt32")), ("Build", SimTypeInt(signed=False, label="UInt32")), ("BuildQfe", SimTypeInt(signed=False, label="UInt32")), ("ProductType", SimTypeInt(signed=False, label="UInt32")), ("Suite", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("s", SimTypeRef("_s_e__Struct", SimStruct)), ("SrvPackNumber", SimTypeInt(signed=False, label="UInt32")), ("ServicePackBuild", SimTypeInt(signed=False, label="UInt32")), ("Architecture", SimTypeInt(signed=False, label="UInt32")), ("Lcid", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("FullName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Language", SimTypeArray(SimTypeChar(label="SByte"), 30)), ("BuildVersion", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("ServicePackString", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="OS_INFO", pack=False, align=None),
    "OS_INFO_v1": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="OS_TYPE")), ("Anonymous", SimUnion({"Version": SimTypeRef("_Version_e__Struct", SimStruct), "Ver64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ProductType", SimTypeInt(signed=False, label="UInt32")), ("Suite", SimTypeInt(signed=False, label="UInt32")), ("s", SimTypeRef("_s_e__Struct", SimStruct)), ("SrvPackNumber", SimTypeInt(signed=False, label="UInt32")), ("Language", SimTypeArray(SimTypeChar(label="SByte"), 30)), ("OsString", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("ServicePackString", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="OS_INFO_v1", pack=False, align=None),
    "OS_PRODUCT_TYPE": SimTypeInt(signed=False, label="OS_PRODUCT_TYPE"),
    "OS_STRING": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("MicrosoftString", SimTypeArray(SimTypeChar(label="Char"), 7)), ("bVendorCode", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"bPad": SimTypeChar(label="Byte"), "bFlags": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="OS_STRING", pack=False, align=None),
    "OS_TYPE": SimTypeInt(signed=False, label="OS_TYPE"),
    "OUTLINETEXTMETRICA": SimStruct(OrderedDict((("otmSize", SimTypeInt(signed=False, label="UInt32")), ("otmTextMetrics", SimTypeRef("TEXTMETRICA", SimStruct)), ("otmFiller", SimTypeChar(label="Byte")), ("otmPanoseNumber", SimTypeRef("PANOSE", SimStruct)), ("otmfsSelection", SimTypeInt(signed=False, label="UInt32")), ("otmfsType", SimTypeInt(signed=False, label="UInt32")), ("otmsCharSlopeRise", SimTypeInt(signed=True, label="Int32")), ("otmsCharSlopeRun", SimTypeInt(signed=True, label="Int32")), ("otmItalicAngle", SimTypeInt(signed=True, label="Int32")), ("otmEMSquare", SimTypeInt(signed=False, label="UInt32")), ("otmAscent", SimTypeInt(signed=True, label="Int32")), ("otmDescent", SimTypeInt(signed=True, label="Int32")), ("otmLineGap", SimTypeInt(signed=False, label="UInt32")), ("otmsCapEmHeight", SimTypeInt(signed=False, label="UInt32")), ("otmsXHeight", SimTypeInt(signed=False, label="UInt32")), ("otmrcFontBox", SimTypeRef("RECT", SimStruct)), ("otmMacAscent", SimTypeInt(signed=True, label="Int32")), ("otmMacDescent", SimTypeInt(signed=True, label="Int32")), ("otmMacLineGap", SimTypeInt(signed=False, label="UInt32")), ("otmusMinimumPPEM", SimTypeInt(signed=False, label="UInt32")), ("otmptSubscriptSize", SimTypeRef("POINT", SimStruct)), ("otmptSubscriptOffset", SimTypeRef("POINT", SimStruct)), ("otmptSuperscriptSize", SimTypeRef("POINT", SimStruct)), ("otmptSuperscriptOffset", SimTypeRef("POINT", SimStruct)), ("otmsStrikeoutSize", SimTypeInt(signed=False, label="UInt32")), ("otmsStrikeoutPosition", SimTypeInt(signed=True, label="Int32")), ("otmsUnderscoreSize", SimTypeInt(signed=True, label="Int32")), ("otmsUnderscorePosition", SimTypeInt(signed=True, label="Int32")), ("otmpFamilyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("otmpFaceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("otmpStyleName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("otmpFullName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OUTLINETEXTMETRICA", pack=False, align=None),
    "OUTLINETEXTMETRICW": SimStruct(OrderedDict((("otmSize", SimTypeInt(signed=False, label="UInt32")), ("otmTextMetrics", SimTypeRef("TEXTMETRICW", SimStruct)), ("otmFiller", SimTypeChar(label="Byte")), ("otmPanoseNumber", SimTypeRef("PANOSE", SimStruct)), ("otmfsSelection", SimTypeInt(signed=False, label="UInt32")), ("otmfsType", SimTypeInt(signed=False, label="UInt32")), ("otmsCharSlopeRise", SimTypeInt(signed=True, label="Int32")), ("otmsCharSlopeRun", SimTypeInt(signed=True, label="Int32")), ("otmItalicAngle", SimTypeInt(signed=True, label="Int32")), ("otmEMSquare", SimTypeInt(signed=False, label="UInt32")), ("otmAscent", SimTypeInt(signed=True, label="Int32")), ("otmDescent", SimTypeInt(signed=True, label="Int32")), ("otmLineGap", SimTypeInt(signed=False, label="UInt32")), ("otmsCapEmHeight", SimTypeInt(signed=False, label="UInt32")), ("otmsXHeight", SimTypeInt(signed=False, label="UInt32")), ("otmrcFontBox", SimTypeRef("RECT", SimStruct)), ("otmMacAscent", SimTypeInt(signed=True, label="Int32")), ("otmMacDescent", SimTypeInt(signed=True, label="Int32")), ("otmMacLineGap", SimTypeInt(signed=False, label="UInt32")), ("otmusMinimumPPEM", SimTypeInt(signed=False, label="UInt32")), ("otmptSubscriptSize", SimTypeRef("POINT", SimStruct)), ("otmptSubscriptOffset", SimTypeRef("POINT", SimStruct)), ("otmptSuperscriptSize", SimTypeRef("POINT", SimStruct)), ("otmptSuperscriptOffset", SimTypeRef("POINT", SimStruct)), ("otmsStrikeoutSize", SimTypeInt(signed=False, label="UInt32")), ("otmsStrikeoutPosition", SimTypeInt(signed=True, label="Int32")), ("otmsUnderscoreSize", SimTypeInt(signed=True, label="Int32")), ("otmsUnderscorePosition", SimTypeInt(signed=True, label="Int32")), ("otmpFamilyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("otmpFaceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("otmpStyleName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("otmpFullName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OUTLINETEXTMETRICW", pack=False, align=None),
    "OUTPUTDUPL_CONTEXT_DEBUG_INFO": SimStruct(OrderedDict((("Status", SimTypeInt(signed=False, label="OUTPUTDUPL_CONTEXT_DEBUG_STATUS")), ("ProcessID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AccumulatedPresents", SimTypeInt(signed=False, label="UInt32")), ("LastPresentTime", SimTypeLongLong(signed=True, label="Int64")), ("LastMouseTime", SimTypeLongLong(signed=True, label="Int64")), ("ProcessName", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="OUTPUTDUPL_CONTEXT_DEBUG_INFO", pack=False, align=None),
    "OUTPUTDUPL_CONTEXT_DEBUG_STATUS": SimTypeInt(signed=False, label="OUTPUTDUPL_CONTEXT_DEBUG_STATUS"),
    "OUTPUT_COLOR_ENCODING": SimTypeInt(signed=False, label="OUTPUT_COLOR_ENCODING"),
    "OUTPUT_DEBUG_STRING_INFO": SimStruct(OrderedDict((("lpDebugStringData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fUnicode", SimTypeShort(signed=False, label="UInt16")), ("nDebugStringLength", SimTypeShort(signed=False, label="UInt16")),)), name="OUTPUT_DEBUG_STRING_INFO", pack=False, align=None),
    "OUTPUT_STATE": SimTypeInt(signed=False, label="OUTPUT_STATE"),
    "OUTPUT_WIRE_COLOR_SPACE_TYPE": SimTypeInt(signed=False, label="OUTPUT_WIRE_COLOR_SPACE_TYPE"),
    "OUTPUT_WIRE_FORMAT": SimStruct(OrderedDict((("ColorEncoding", SimTypeInt(signed=False, label="OUTPUT_COLOR_ENCODING")), ("BitsPerPixel", SimTypeInt(signed=False, label="UInt32")),)), name="OUTPUT_WIRE_FORMAT", pack=False, align=None),
    "OVERLAPPED": SimStruct(OrderedDict((("Internal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InternalHigh", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("OffsetHigh", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("hEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="OVERLAPPED", pack=False, align=None),
    "OVERLAPPED_ENTRY": SimStruct(OrderedDict((("lpCompletionKey", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpOverlapped", SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)), ("Internal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwNumberOfBytesTransferred", SimTypeInt(signed=False, label="UInt32")),)), name="OVERLAPPED_ENTRY", pack=False, align=None),
    "OVERRIDE_PREFETCH_PARAMETER": SimStruct(OrderedDict((("Value", SimTypeInt(signed=False, label="UInt32")),)), name="OVERRIDE_PREFETCH_PARAMETER", pack=False, align=None),
    "OWNER_ENTRY": SimStruct(OrderedDict((("OwnerThread", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "TableSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="OWNER_ENTRY", pack=False, align=None),
    "ObjectContext": SimTypeBottom(label="ObjectContext"),
    "ObjectControl": SimTypeBottom(label="ObjectControl"),
    "ObjectIdGroupId": SimTypeInt(signed=False, label="ObjectIdGroupId"),
    "ObjectIdPublicKeyFlags": SimTypeInt(signed=False, label="ObjectIdPublicKeyFlags"),
    "ObjectReferenceCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["root", "reference", "clientData"]), offset=0),
    "ObjectType": SimTypeInt(signed=False, label="ObjectType"),
    "OfflineFolderStatus": SimTypeInt(signed=False, label="OfflineFolderStatus"),
    "OpenServiceActivityContentType": SimTypeInt(signed=False, label="OpenServiceActivityContentType"),
    "OpenServiceErrors": SimTypeInt(signed=False, label="OpenServiceErrors"),
    "OperationResultCode": SimTypeInt(signed=False, label="OperationResultCode"),
    "OrientationType": SimTypeInt(signed=False, label="OrientationType"),
    "OutgoingCookieState": SimStruct(OrderedDict((("cSent", SimTypeInt(signed=True, label="Int32")), ("cSuppressed", SimTypeInt(signed=True, label="Int32")), ("pszLocation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="OutgoingCookieState", pack=False, align=None),
    "OutlineStyles": SimTypeInt(signed=False, label="OutlineStyles"),
    "PACKAGEDEPENDENCY_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PACKAGE_EXECUTION_STATE": SimTypeInt(signed=False, label="PACKAGE_EXECUTION_STATE"),
    "PACKAGE_ID": SimStruct(OrderedDict((("reserved", SimTypeInt(signed=False, label="UInt32")), ("processorArchitecture", SimTypeInt(signed=False, label="UInt32")), ("version", SimTypeRef("PACKAGE_VERSION", SimStruct)), ("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("publisher", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("resourceId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("publisherId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PACKAGE_ID", pack=False, align=None),
    "PACKAGE_INFO": SimStruct(OrderedDict((("reserved", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("packageFullName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("packageFamilyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("packageId", SimTypeRef("PACKAGE_ID", SimStruct)),)), name="PACKAGE_INFO", pack=False, align=None),
    "PACKAGE_VERSION": SimStruct(OrderedDict((("Anonymous", SimUnion({"Version": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Revision", SimTypeShort(signed=False, label="UInt16")), ("Build", SimTypeShort(signed=False, label="UInt16")), ("Minor", SimTypeShort(signed=False, label="UInt16")), ("Major", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PACKAGE_VERSION", pack=False, align=None),
    "PACKAGE_VIRTUALIZATION_CONTEXT_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PACKEDEVENTINFO": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("ulNumEventsForLogFile", SimTypeInt(signed=False, label="UInt32")), ("ulOffsets", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PACKEDEVENTINFO", pack=False, align=None),
    "PACKET_DESCRIPTION": SimStruct(OrderedDict((("cbPacketSize", SimTypeInt(signed=False, label="UInt32")), ("cPacketProperties", SimTypeInt(signed=False, label="UInt32")), ("pPacketProperties", SimTypePointer(SimTypeRef("PACKET_PROPERTY", SimStruct), offset=0)), ("cButtons", SimTypeInt(signed=False, label="UInt32")), ("pguidButtons", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="PACKET_DESCRIPTION", pack=False, align=None),
    "PACKET_PARAMETERS": SimStruct(OrderedDict((("DeviceAddress", SimTypeChar(label="Byte")), ("EndpointAddress", SimTypeChar(label="Byte")), ("MaximumPacketSize", SimTypeShort(signed=False, label="UInt16")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("HubDeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("PortTTNumber", SimTypeShort(signed=False, label="UInt16")), ("ErrorCount", SimTypeChar(label="Byte")), ("Pad", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("UsbdStatusCode", SimTypeInt(signed=True, label="Int32")), ("Data", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="PACKET_PARAMETERS", pack=False, align=None),
    "PACKET_PROPERTY": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("PropertyMetrics", SimTypeRef("PROPERTY_METRICS", SimStruct)),)), name="PACKET_PROPERTY", pack=False, align=None),
    "PACKMAN_RUNTIME": SimTypeInt(signed=False, label="PACKMAN_RUNTIME"),
    "PACQUIRE_FOR_LAZY_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["Context", "Wait"]), offset=0),
    "PACQUIRE_FOR_LAZY_WRITE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Context", "InFlags", "OutFlags"]), offset=0),
    "PACQUIRE_FOR_READ_AHEAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["Context", "Wait"]), offset=0),
    "PAC_CHANGES_CALLBACK_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("INET_FIREWALL_AC_CHANGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "pChange"]), offset=0),
    "PAFFINITY_TOKEN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PAGEACTION_UI": SimTypeInt(signed=False, label="PAGEACTION_UI"),
    "PAGEPARTS": SimTypeInt(signed=False, label="PAGEPARTS"),
    "PAGERANGE": SimStruct(OrderedDict((("nFromPage", SimTypeInt(signed=True, label="Int32")), ("nToPage", SimTypeInt(signed=True, label="Int32")),)), name="PAGERANGE", pack=False, align=None),
    "PAGESET": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("fOddPages", SimTypeInt(signed=True, label="Int32")), ("fEvenPages", SimTypeInt(signed=True, label="Int32")), ("cPageRange", SimTypeInt(signed=False, label="UInt32")), ("rgPages", SimTypePointer(SimTypeRef("PAGERANGE", SimStruct), offset=0)),)), name="PAGESET", pack=False, align=None),
    "PAGESETUPDLGA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDevNames", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="PAGESETUPDLG_FLAGS")), ("ptPaperSize", SimTypeRef("POINT", SimStruct)), ("rtMinMargin", SimTypeRef("RECT", SimStruct)), ("rtMargin", SimTypeRef("RECT", SimStruct)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnPageSetupHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnPagePaintHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpPageSetupTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hPageSetupTemplate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PAGESETUPDLGA", pack=False, align=None),
    "PAGESETUPDLGW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDevNames", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="PAGESETUPDLG_FLAGS")), ("ptPaperSize", SimTypeRef("POINT", SimStruct)), ("rtMinMargin", SimTypeRef("RECT", SimStruct)), ("rtMargin", SimTypeRef("RECT", SimStruct)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnPageSetupHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnPagePaintHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpPageSetupTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hPageSetupTemplate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PAGESETUPDLGW", pack=False, align=None),
    "PAGESETUPDLG_FLAGS": SimTypeInt(signed=False, label="PAGESETUPDLG_FLAGS"),
    "PAGE_PRIORITY_INFORMATION": SimStruct(OrderedDict((("PagePriority", SimTypeInt(signed=False, label="UInt32")),)), name="PAGE_PRIORITY_INFORMATION", pack=False, align=None),
    "PAGE_PROTECTION_FLAGS": SimTypeInt(signed=False, label="PAGE_PROTECTION_FLAGS"),
    "PAGE_TYPE": SimTypeInt(signed=False, label="PAGE_TYPE"),
    "PAGING_MODE": SimTypeInt(signed=False, label="PAGING_MODE"),
    "PAINTSTRUCT": SimStruct(OrderedDict((("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fErase", SimTypeInt(signed=True, label="Int32")), ("rcPaint", SimTypeRef("RECT", SimStruct)), ("fRestore", SimTypeInt(signed=True, label="Int32")), ("fIncUpdate", SimTypeInt(signed=True, label="Int32")), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="PAINTSTRUCT", pack=False, align=None),
    "PAINT_COLOR_GLYPH": SimStruct(OrderedDict((("glyphIndex", SimTypeInt(signed=False, label="UInt32")), ("clipBox", SimTypeRef("D2D_RECT_F", SimStruct)),)), name="PAINT_COLOR_GLYPH", pack=False, align=None),
    "PAINT_COMPOSITE": SimStruct(OrderedDict((("mode", SimTypeInt(signed=False, label="DWRITE_COLOR_COMPOSITE_MODE")),)), name="PAINT_COMPOSITE", pack=False, align=None),
    "PAINT_GLYPH": SimStruct(OrderedDict((("glyphIndex", SimTypeInt(signed=False, label="UInt32")),)), name="PAINT_GLYPH", pack=False, align=None),
    "PAINT_LAYERS": SimStruct(OrderedDict((("childCount", SimTypeInt(signed=False, label="UInt32")),)), name="PAINT_LAYERS", pack=False, align=None),
    "PAINT_LINEAR_GRADIENT": SimStruct(OrderedDict((("extendMode", SimTypeInt(signed=False, label="UInt32")), ("gradientStopCount", SimTypeInt(signed=False, label="UInt32")), ("x0", SimTypeFloat(size=32)), ("y0", SimTypeFloat(size=32)), ("x1", SimTypeFloat(size=32)), ("y1", SimTypeFloat(size=32)), ("x2", SimTypeFloat(size=32)), ("y2", SimTypeFloat(size=32)),)), name="PAINT_LINEAR_GRADIENT", pack=False, align=None),
    "PAINT_RADIAL_GRADIENT": SimStruct(OrderedDict((("extendMode", SimTypeInt(signed=False, label="UInt32")), ("gradientStopCount", SimTypeInt(signed=False, label="UInt32")), ("x0", SimTypeFloat(size=32)), ("y0", SimTypeFloat(size=32)), ("radius0", SimTypeFloat(size=32)), ("x1", SimTypeFloat(size=32)), ("y1", SimTypeFloat(size=32)), ("radius1", SimTypeFloat(size=32)),)), name="PAINT_RADIAL_GRADIENT", pack=False, align=None),
    "PAINT_SOLID_GLYPH": SimStruct(OrderedDict((("glyphIndex", SimTypeInt(signed=False, label="UInt32")), ("color", SimTypeRef("DWRITE_PAINT_COLOR", SimStruct)),)), name="PAINT_SOLID_GLYPH", pack=False, align=None),
    "PAINT_SWEEP_GRADIENT": SimStruct(OrderedDict((("extendMode", SimTypeInt(signed=False, label="UInt32")), ("gradientStopCount", SimTypeInt(signed=False, label="UInt32")), ("centerX", SimTypeFloat(size=32)), ("centerY", SimTypeFloat(size=32)), ("startAngle", SimTypeFloat(size=32)), ("endAngle", SimTypeFloat(size=32)),)), name="PAINT_SWEEP_GRADIENT", pack=False, align=None),
    "PAINT_UNION": SimUnion({"layers": SimTypeRef("PAINT_LAYERS", SimStruct), "solidGlyph": SimTypeRef("PAINT_SOLID_GLYPH", SimStruct), "solid": SimTypeRef("DWRITE_PAINT_COLOR", SimStruct), "linearGradient": SimTypeRef("PAINT_LINEAR_GRADIENT", SimStruct), "radialGradient": SimTypeRef("PAINT_RADIAL_GRADIENT", SimStruct), "sweepGradient": SimTypeRef("PAINT_SWEEP_GRADIENT", SimStruct), "glyph": SimTypeRef("PAINT_GLYPH", SimStruct), "colorGlyph": SimTypeRef("PAINT_COLOR_GLYPH", SimStruct), "transform": SimTypeRef("DWRITE_MATRIX", SimStruct), "composite": SimTypeRef("PAINT_COMPOSITE", SimStruct)}, name="<anon>", label="None"),
    "PALETTEENTRY": SimStruct(OrderedDict((("peRed", SimTypeChar(label="Byte")), ("peGreen", SimTypeChar(label="Byte")), ("peBlue", SimTypeChar(label="Byte")), ("peFlags", SimTypeChar(label="Byte")),)), name="PALETTEENTRY", pack=False, align=None),
    "PALLOCATE_ADAPTER_CHANNEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"), arg_names=["DeviceObject", "Irp", "MapRegisterBase", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "NumberOfMapRegisters", "ExecutionRoutine", "Context"]), offset=0),
    "PALLOCATE_ADAPTER_CHANNEL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"), arg_names=["DeviceObject", "Irp", "MapRegisterBase", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext", "NumberOfMapRegisters", "Flags", "ExecutionRoutine", "ExecutionContext", "MapRegisterBase"]), offset=0),
    "PALLOCATE_COMMON_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeChar(label="Byte")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["DmaAdapter", "Length", "LogicalAddress", "CacheEnabled"]), offset=0),
    "PALLOCATE_COMMON_BUFFER_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["DmaAdapter", "MaximumAddress", "Length", "LogicalAddress", "CacheEnabled", "PreferredNode"]), offset=0),
    "PALLOCATE_COMMON_BUFFER_VECTOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "LowAddress", "HighAddress", "CacheType", "IdealNode", "Flags", "NumberOfElements", "SizeOfElements", "VectorOut"]), offset=0),
    "PALLOCATE_COMMON_BUFFER_WITH_BOUNDS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["DmaAdapter", "MinimumAddress", "MaximumAddress", "Length", "Flags", "CacheType", "PreferredNode", "LogicalAddress"]), offset=0),
    "PALLOCATE_DOMAIN_COMMON_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="MEMORY_CACHING_TYPE"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DomainHandle", "MaximumAddress", "Length", "Flags", "CacheType", "PreferredNode", "LogicalAddress", "VirtualAddress"]), offset=0),
    "PALLOCATE_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0),
    "PALLOCATE_FUNCTION_EX": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0),
    "PALLOCATE_VIRTUAL_MEMORY_EX_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PALLOCMEM": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Size"]), offset=0),
    "PALOBJ": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")),)), name="PALOBJ", pack=False, align=None),
    "PANEL_BRIGHTNESS_SENSOR_DATA": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("AlsReading", SimTypeFloat(size=32)), ("ChromaticityCoordinate", SimTypeRef("CHROMATICITY_COORDINATE", SimStruct)), ("ColorTemperature", SimTypeFloat(size=32)),)), name="PANEL_BRIGHTNESS_SENSOR_DATA", pack=False, align=None),
    "PANEL_GET_BACKLIGHT_REDUCTION": SimStruct(OrderedDict((("BacklightUsersetting", SimTypeShort(signed=False, label="UInt16")), ("BacklightEffective", SimTypeShort(signed=False, label="UInt16")), ("GammaRamp", SimTypeRef("BACKLIGHT_REDUCTION_GAMMA_RAMP", SimStruct)),)), name="PANEL_GET_BACKLIGHT_REDUCTION", pack=False, align=None),
    "PANEL_GET_BRIGHTNESS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="BRIGHTNESS_INTERFACE_VERSION")), ("Anonymous", SimUnion({"Level": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("CurrentInMillinits", SimTypeInt(signed=False, label="UInt32")), ("TargetInMillinits", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PANEL_GET_BRIGHTNESS", pack=False, align=None),
    "PANEL_QUERY_BRIGHTNESS_CAPS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="BRIGHTNESS_INTERFACE_VERSION")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PANEL_QUERY_BRIGHTNESS_CAPS", pack=False, align=None),
    "PANEL_QUERY_BRIGHTNESS_RANGES": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="BRIGHTNESS_INTERFACE_VERSION")), ("Anonymous", SimUnion({"BrightnessLevel": SimTypeRef("BRIGHTNESS_LEVEL", SimStruct), "NitRanges": SimTypeRef("BRIGHTNESS_NIT_RANGES", SimStruct)}, name="<anon>", label="None")),)), name="PANEL_QUERY_BRIGHTNESS_RANGES", pack=False, align=None),
    "PANEL_SET_BACKLIGHT_OPTIMIZATION": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="BACKLIGHT_OPTIMIZATION_LEVEL")),)), name="PANEL_SET_BACKLIGHT_OPTIMIZATION", pack=False, align=None),
    "PANEL_SET_BRIGHTNESS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="BRIGHTNESS_INTERFACE_VERSION")), ("Anonymous", SimUnion({"Level": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("Millinits", SimTypeInt(signed=False, label="UInt32")), ("TransitionTimeInMs", SimTypeInt(signed=False, label="UInt32")), ("SensorData", SimTypeRef("PANEL_BRIGHTNESS_SENSOR_DATA", SimStruct)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PANEL_SET_BRIGHTNESS", pack=False, align=None),
    "PANEL_SET_BRIGHTNESS_STATE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PANEL_SET_BRIGHTNESS_STATE", pack=False, align=None),
    "PANOSE": SimStruct(OrderedDict((("bFamilyType", SimTypeInt(signed=False, label="PAN_FAMILY_TYPE")), ("bSerifStyle", SimTypeInt(signed=False, label="PAN_SERIF_STYLE")), ("bWeight", SimTypeInt(signed=False, label="PAN_WEIGHT")), ("bProportion", SimTypeInt(signed=False, label="PAN_PROPORTION")), ("bContrast", SimTypeInt(signed=False, label="PAN_CONTRAST")), ("bStrokeVariation", SimTypeInt(signed=False, label="PAN_STROKE_VARIATION")), ("bArmStyle", SimTypeInt(signed=False, label="PAN_ARM_STYLE")), ("bLetterform", SimTypeInt(signed=False, label="PAN_LETT_FORM")), ("bMidline", SimTypeInt(signed=False, label="PAN_MIDLINE")), ("bXHeight", SimTypeInt(signed=False, label="PAN_XHEIGHT")),)), name="PANOSE", pack=False, align=None),
    "PAN_ARM_STYLE": SimTypeInt(signed=False, label="PAN_ARM_STYLE"),
    "PAN_CONTRAST": SimTypeInt(signed=False, label="PAN_CONTRAST"),
    "PAN_FAMILY_TYPE": SimTypeInt(signed=False, label="PAN_FAMILY_TYPE"),
    "PAN_LETT_FORM": SimTypeInt(signed=False, label="PAN_LETT_FORM"),
    "PAN_MIDLINE": SimTypeInt(signed=False, label="PAN_MIDLINE"),
    "PAN_PROPORTION": SimTypeInt(signed=False, label="PAN_PROPORTION"),
    "PAN_SERIF_STYLE": SimTypeInt(signed=False, label="PAN_SERIF_STYLE"),
    "PAN_STROKE_VARIATION": SimTypeInt(signed=False, label="PAN_STROKE_VARIATION"),
    "PAN_WEIGHT": SimTypeInt(signed=False, label="PAN_WEIGHT"),
    "PAN_XHEIGHT": SimTypeInt(signed=False, label="PAN_XHEIGHT"),
    "PAPCFUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Parameter"]), offset=0),
    "PAPPCONSTRAIN_CHANGE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Constrained", "Context"]), offset=0),
    "PAPPCONSTRAIN_REGISTRATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PAPPSTATE_CHANGE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Quiesced", "Context"]), offset=0),
    "PAPPSTATE_REGISTRATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PARAFORMAT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="PARAFORMAT_MASK")), ("wNumbering", SimTypeInt(signed=False, label="PARAFORMAT_NUMBERING")), ("Anonymous", SimUnion({"wReserved": SimTypeShort(signed=False, label="UInt16"), "wEffects": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("dxStartIndent", SimTypeInt(signed=True, label="Int32")), ("dxRightIndent", SimTypeInt(signed=True, label="Int32")), ("dxOffset", SimTypeInt(signed=True, label="Int32")), ("wAlignment", SimTypeInt(signed=False, label="PARAFORMAT_ALIGNMENT")), ("cTabCount", SimTypeShort(signed=True, label="Int16")), ("rgxTabs", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)),)), name="PARAFORMAT", pack=False, align=None),
    "PARAFORMAT2": SimStruct(OrderedDict((("Base", SimTypeRef("PARAFORMAT", SimStruct)), ("dySpaceBefore", SimTypeInt(signed=True, label="Int32")), ("dySpaceAfter", SimTypeInt(signed=True, label="Int32")), ("dyLineSpacing", SimTypeInt(signed=True, label="Int32")), ("sStyle", SimTypeShort(signed=True, label="Int16")), ("bLineSpacingRule", SimTypeChar(label="Byte")), ("bOutlineLevel", SimTypeChar(label="Byte")), ("wShadingWeight", SimTypeShort(signed=False, label="UInt16")), ("wShadingStyle", SimTypeInt(signed=False, label="PARAFORMAT_SHADING_STYLE")), ("wNumberingStart", SimTypeShort(signed=False, label="UInt16")), ("wNumberingStyle", SimTypeInt(signed=False, label="PARAFORMAT_NUMBERING_STYLE")), ("wNumberingTab", SimTypeShort(signed=False, label="UInt16")), ("wBorderSpace", SimTypeShort(signed=False, label="UInt16")), ("wBorderWidth", SimTypeShort(signed=False, label="UInt16")), ("wBorders", SimTypeInt(signed=False, label="PARAFORMAT_BORDERS")),)), name="PARAFORMAT2", pack=False, align=None),
    "PARAFORMAT_ALIGNMENT": SimTypeInt(signed=False, label="PARAFORMAT_ALIGNMENT"),
    "PARAFORMAT_BORDERS": SimTypeInt(signed=False, label="PARAFORMAT_BORDERS"),
    "PARAFORMAT_MASK": SimTypeInt(signed=False, label="PARAFORMAT_MASK"),
    "PARAFORMAT_NUMBERING": SimTypeInt(signed=False, label="PARAFORMAT_NUMBERING"),
    "PARAFORMAT_NUMBERING_STYLE": SimTypeInt(signed=False, label="PARAFORMAT_NUMBERING_STYLE"),
    "PARAFORMAT_SHADING_STYLE": SimTypeInt(signed=False, label="PARAFORMAT_SHADING_STYLE"),
    "PARAMDATA": SimStruct(OrderedDict((("szName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("vt", SimTypeInt(signed=False, label="VARENUM")),)), name="PARAMDATA", pack=False, align=None),
    "PARAMDESC": SimStruct(OrderedDict((("pparamdescex", SimTypePointer(SimTypeRef("PARAMDESCEX", SimStruct), offset=0)), ("wParamFlags", SimTypeInt(signed=False, label="PARAMFLAGS")),)), name="PARAMDESC", pack=False, align=None),
    "PARAMDESCEX": SimStruct(OrderedDict((("cBytes", SimTypeInt(signed=False, label="UInt32")), ("varDefaultValue", SimTypeRef("VARIANT", SimStruct)),)), name="PARAMDESCEX", pack=False, align=None),
    "PARAMFLAGS": SimTypeInt(signed=False, label="PARAMFLAGS"),
    "PARAM_BUFFER": SimStruct(OrderedDict((("ParameterId", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PARAM_BUFFER", pack=False, align=None),
    "PARBITER_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="ARBITER_ACTION"), SimTypePointer(SimTypeRef("ARBITER_PARAMETERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Action", "Parameters"]), offset=0),
    "PARBITRATE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "LostQuorumResource"]), offset=0),
    "PARM_WPR_WATCHDOG_FOR_CURRENT_RESOURCE_CALL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "TimeoutInMs"]), offset=0),
    "PARSEACTION": SimTypeInt(signed=False, label="PARSEACTION"),
    "PARSEDURLA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszProtocol", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchProtocol", SimTypeInt(signed=False, label="UInt32")), ("pszSuffix", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchSuffix", SimTypeInt(signed=False, label="UInt32")), ("nScheme", SimTypeInt(signed=False, label="UInt32")),)), name="PARSEDURLA", pack=False, align=None),
    "PARSEDURLW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszProtocol", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchProtocol", SimTypeInt(signed=False, label="UInt32")), ("pszSuffix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchSuffix", SimTypeInt(signed=False, label="UInt32")), ("nScheme", SimTypeInt(signed=False, label="UInt32")),)), name="PARSEDURLW", pack=False, align=None),
    "PARSE_FLAGS": SimTypeInt(signed=False, label="PARSE_FLAGS"),
    "PARTITION_INFORMATION": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("PartitionLength", SimTypeLongLong(signed=True, label="Int64")), ("HiddenSectors", SimTypeInt(signed=False, label="UInt32")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("PartitionType", SimTypeChar(label="Byte")), ("BootIndicator", SimTypeChar(label="Byte")), ("RecognizedPartition", SimTypeChar(label="Byte")), ("RewritePartition", SimTypeChar(label="Byte")),)), name="PARTITION_INFORMATION", pack=False, align=None),
    "PARTITION_INFORMATION_CLASS": SimTypeInt(signed=False, label="PARTITION_INFORMATION_CLASS"),
    "PARTITION_INFORMATION_EX": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="PARTITION_STYLE")), ("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("PartitionLength", SimTypeLongLong(signed=True, label="Int64")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("RewritePartition", SimTypeChar(label="Byte")), ("IsServicePartition", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("PARTITION_INFORMATION_MBR", SimStruct), "Gpt": SimTypeRef("PARTITION_INFORMATION_GPT", SimStruct)}, name="<anon>", label="None")),)), name="PARTITION_INFORMATION_EX", pack=False, align=None),
    "PARTITION_INFORMATION_GPT": SimStruct(OrderedDict((("PartitionType", SimTypeBottom(label="Guid")), ("PartitionId", SimTypeBottom(label="Guid")), ("Attributes", SimTypeInt(signed=False, label="GPT_ATTRIBUTES")), ("Name", SimTypeArray(SimTypeChar(label="Char"), 36)),)), name="PARTITION_INFORMATION_GPT", pack=False, align=None),
    "PARTITION_INFORMATION_MBR": SimStruct(OrderedDict((("PartitionType", SimTypeChar(label="Byte")), ("BootIndicator", SimTypeChar(label="Byte")), ("RecognizedPartition", SimTypeChar(label="Byte")), ("HiddenSectors", SimTypeInt(signed=False, label="UInt32")), ("PartitionId", SimTypeBottom(label="Guid")),)), name="PARTITION_INFORMATION_MBR", pack=False, align=None),
    "PARTITION_STYLE": SimTypeInt(signed=False, label="PARTITION_STYLE"),
    "PASTE_SPECIAL_FLAGS": SimTypeInt(signed=False, label="PASTE_SPECIAL_FLAGS"),
    "PASYNC_READ_COMPLETION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Context"]), offset=0),
    "PATCH_IGNORE_RANGE": SimStruct(OrderedDict((("OffsetInOldFile", SimTypeInt(signed=False, label="UInt32")), ("LengthInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="PATCH_IGNORE_RANGE", pack=False, align=None),
    "PATCH_INTERLEAVE_MAP": SimStruct(OrderedDict((("CountRanges", SimTypeInt(signed=False, label="UInt32")), ("Range", SimTypePointer(SimStruct(OrderedDict((("OldOffset", SimTypeInt(signed=False, label="UInt32")), ("OldLength", SimTypeInt(signed=False, label="UInt32")), ("NewLength", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="PATCH_INTERLEAVE_MAP", pack=False, align=None),
    "PATCH_OLD_FILE_INFO": SimStruct(OrderedDict((("SizeOfThisStruct", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"OldFileNameA": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "OldFileNameW": SimTypePointer(SimTypeChar(label="Char"), offset=0), "OldFileHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("IgnoreRangeCount", SimTypeInt(signed=False, label="UInt32")), ("IgnoreRangeArray", SimTypePointer(SimTypeRef("PATCH_IGNORE_RANGE", SimStruct), offset=0)), ("RetainRangeCount", SimTypeInt(signed=False, label="UInt32")), ("RetainRangeArray", SimTypePointer(SimTypeRef("PATCH_RETAIN_RANGE", SimStruct), offset=0)),)), name="PATCH_OLD_FILE_INFO", pack=False, align=None),
    "PATCH_OLD_FILE_INFO_A": SimStruct(OrderedDict((("SizeOfThisStruct", SimTypeInt(signed=False, label="UInt32")), ("OldFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("IgnoreRangeCount", SimTypeInt(signed=False, label="UInt32")), ("IgnoreRangeArray", SimTypePointer(SimTypeRef("PATCH_IGNORE_RANGE", SimStruct), offset=0)), ("RetainRangeCount", SimTypeInt(signed=False, label="UInt32")), ("RetainRangeArray", SimTypePointer(SimTypeRef("PATCH_RETAIN_RANGE", SimStruct), offset=0)),)), name="PATCH_OLD_FILE_INFO_A", pack=False, align=None),
    "PATCH_OLD_FILE_INFO_H": SimStruct(OrderedDict((("SizeOfThisStruct", SimTypeInt(signed=False, label="UInt32")), ("OldFileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IgnoreRangeCount", SimTypeInt(signed=False, label="UInt32")), ("IgnoreRangeArray", SimTypePointer(SimTypeRef("PATCH_IGNORE_RANGE", SimStruct), offset=0)), ("RetainRangeCount", SimTypeInt(signed=False, label="UInt32")), ("RetainRangeArray", SimTypePointer(SimTypeRef("PATCH_RETAIN_RANGE", SimStruct), offset=0)),)), name="PATCH_OLD_FILE_INFO_H", pack=False, align=None),
    "PATCH_OLD_FILE_INFO_W": SimStruct(OrderedDict((("SizeOfThisStruct", SimTypeInt(signed=False, label="UInt32")), ("OldFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IgnoreRangeCount", SimTypeInt(signed=False, label="UInt32")), ("IgnoreRangeArray", SimTypePointer(SimTypeRef("PATCH_IGNORE_RANGE", SimStruct), offset=0)), ("RetainRangeCount", SimTypeInt(signed=False, label="UInt32")), ("RetainRangeArray", SimTypePointer(SimTypeRef("PATCH_RETAIN_RANGE", SimStruct), offset=0)),)), name="PATCH_OLD_FILE_INFO_W", pack=False, align=None),
    "PATCH_OPTION_DATA": SimStruct(OrderedDict((("SizeOfThisStruct", SimTypeInt(signed=False, label="UInt32")), ("SymbolOptionFlags", SimTypeInt(signed=False, label="UInt32")), ("NewFileSymbolPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("OldFileSymbolPathArray", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)), ("ExtendedOptionFlags", SimTypeInt(signed=False, label="UInt32")), ("SymLoadCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["WhichFile", "SymbolFileName", "SymType", "SymbolFileCheckSum", "SymbolFileTimeDate", "ImageFileCheckSum", "ImageFileTimeDate", "CallbackContext"]), offset=0)), ("SymLoadContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterleaveMapArray", SimTypePointer(SimTypePointer(SimTypeRef("PATCH_INTERLEAVE_MAP", SimStruct), offset=0), offset=0)), ("MaxLzxWindowSize", SimTypeInt(signed=False, label="UInt32")),)), name="PATCH_OPTION_DATA", pack=False, align=None),
    "PATCH_RETAIN_RANGE": SimStruct(OrderedDict((("OffsetInOldFile", SimTypeInt(signed=False, label="UInt32")), ("LengthInBytes", SimTypeInt(signed=False, label="UInt32")), ("OffsetInNewFile", SimTypeInt(signed=False, label="UInt32")),)), name="PATCH_RETAIN_RANGE", pack=False, align=None),
    "PATHCCH_OPTIONS": SimTypeInt(signed=False, label="PATHCCH_OPTIONS"),
    "PATHDATA": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("count", SimTypeInt(signed=False, label="UInt32")), ("pptfx", SimTypePointer(SimTypeRef("POINTFIX", SimStruct), offset=0)),)), name="PATHDATA", pack=False, align=None),
    "PATHNAME_BUFFER": SimStruct(OrderedDict((("PathNameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PATHNAME_BUFFER", pack=False, align=None),
    "PATHOBJ": SimStruct(OrderedDict((("fl", SimTypeInt(signed=False, label="UInt32")), ("cCurves", SimTypeInt(signed=False, label="UInt32")),)), name="PATHOBJ", pack=False, align=None),
    "PAYLOAD_FILTER_PREDICATE": SimStruct(OrderedDict((("FieldName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CompareOp", SimTypeShort(signed=False, label="UInt16")), ("Value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PAYLOAD_FILTER_PREDICATE", pack=False, align=None),
    "PAYLOAD_OPERATOR": SimTypeInt(signed=False, label="PAYLOAD_OPERATOR"),
    "PAudioStateMonitorCallback": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IAudioStateMonitor"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["audioStateMonitor", "context"]), offset=0),
    "PBAD_MEMORY_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PBDAParentalControl": SimStruct(OrderedDict((("rating_system_count", SimTypeInt(signed=False, label="UInt32")), ("rating_systems", SimTypePointer(SimTypeRef("RATING_SYSTEM", SimStruct), offset=0)),)), name="PBDAParentalControl", pack=False, align=None),
    "PBDA_TAG_ATTRIBUTE": SimStruct(OrderedDict((("TableUUId", SimTypeBottom(label="Guid")), ("TableId", SimTypeChar(label="Byte")), ("VersionNo", SimTypeShort(signed=False, label="UInt16")), ("TableDataSize", SimTypeInt(signed=False, label="UInt32")), ("TableData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PBDA_TAG_ATTRIBUTE", pack=False, align=None),
    "PBEGIN_RESCALL_AS_USER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "TokenHandle", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0),
    "PBEGIN_RESCALL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0),
    "PBEGIN_RESTYPECALL_AS_USER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "TokenHandle", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0),
    "PBEGIN_RESTYPECALL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned", "context", "ReturnedAsynchronously"]), offset=0),
    "PBOOT_DRIVER_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PBOUND_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="BOUND_CALLBACK_STATUS")), offset=0),
    "PBRANGE": SimStruct(OrderedDict((("iLow", SimTypeInt(signed=True, label="Int32")), ("iHigh", SimTypeInt(signed=True, label="Int32")),)), name="PBRANGE", pack=False, align=None),
    "PBUILD_MDL_FROM_SCATTER_GATHER_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "ScatterGather", "OriginalMdl", "TargetMdl"]), offset=0),
    "PBUILD_SCATTER_GATHER_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "Mdl", "CurrentVa", "Length", "ExecutionRoutine", "Context", "WriteToDevice", "ScatterGatherBuffer", "ScatterGatherLength"]), offset=0),
    "PBUILD_SCATTER_GATHER_LIST_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext", "Mdl", "Offset", "Length", "Flags", "ExecutionRoutine", "Context", "WriteToDevice", "ScatterGatherBuffer", "ScatterGatherLength", "DmaCompletionRoutine", "CompletionContext", "ScatterGatherList"]), offset=0),
    "PBUS_HANDLER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PCALCULATE_SCATTER_GATHER_LIST_SIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "CurrentVa", "Length", "ScatterGatherListSize", "pNumberOfMapRegisters"]), offset=0),
    "PCALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PCALLBACK_OBJECT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PCANCEL_ADAPTER_CHANNEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext"]), offset=0),
    "PCANCEL_MAPPED_TRANSFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DmaTransferContext"]), offset=0),
    "PCANCEL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "CancelFlags_RESERVED"]), offset=0),
    "PCCARD_DES": SimStruct(OrderedDict((("PCD_Count", SimTypeInt(signed=False, label="UInt32")), ("PCD_Type", SimTypeInt(signed=False, label="UInt32")), ("PCD_Flags", SimTypeInt(signed=False, label="PCD_FLAGS")), ("PCD_ConfigIndex", SimTypeChar(label="Byte")), ("PCD_Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("PCD_MemoryCardBase1", SimTypeInt(signed=False, label="UInt32")), ("PCD_MemoryCardBase2", SimTypeInt(signed=False, label="UInt32")), ("PCD_MemoryCardBase", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("PCD_MemoryFlags", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 2)), ("PCD_IoFlags", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="PCCARD_DES", pack=False, align=None),
    "PCCARD_RESOURCE": SimStruct(OrderedDict((("PcCard_Header", SimTypeRef("PCCARD_DES", SimStruct)),)), name="PCCARD_RESOURCE", pack=False, align=None),
    "PCC_POST_DEFERRED_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context1", "Context2"]), offset=0),
    "PCD_FLAGS": SimTypeInt(signed=False, label="PCD_FLAGS"),
    "PCHANGE_RESOURCE_PROCESS_FOR_DUMPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["resource", "processName", "processId", "isAdd"]), offset=0),
    "PCHANGE_RES_TYPE_PROCESS_FOR_DUMPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["resourceTypeName", "processName", "processId", "isAdd"]), offset=0),
    "PCHANNEL_INIT_EVENT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pInitHandle", "event", "pData", "dataLength"]), offset=0),
    "PCHANNEL_OPEN_EVENT_FN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["openHandle", "event", "pData", "dataLength", "totalLength", "dataFlags"]), offset=0),
    "PCHECK_FOR_TRAVERSE_ACCESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SECURITY_SUBJECT_CONTEXT", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["NotifyContext", "TargetContext", "SubjectContext"]), offset=0),
    "PCIBUSDATA": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("ReadConfig", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["BusHandler", "Slot", "Buffer", "Offset", "Length"]), offset=0)), ("WriteConfig", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["BusHandler", "Slot", "Buffer", "Offset", "Length"]), offset=0)), ("Pin2Line", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler", "RootHandler", "SlotNumber", "PciData"]), offset=0)), ("Line2Pin", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler", "RootHandler", "SlotNumber", "PciNewData", "PciOldData"]), offset=0)), ("ParentSlot", SimTypeRef("PCI_SLOT_NUMBER", SimStruct)), ("Reserved", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 4)),)), name="PCIBUSDATA", pack=False, align=None),
    "PCIX_BRIDGE_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("SecondaryStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("BridgeStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UpstreamSplitTransactionCapacity", SimTypeShort(signed=False, label="UInt16")), ("UpstreamSplitTransactionLimit", SimTypeShort(signed=False, label="UInt16")), ("DownstreamSplitTransactionCapacity", SimTypeShort(signed=False, label="UInt16")), ("DownstreamSplitTransactionLimit", SimTypeShort(signed=False, label="UInt16")), ("EccControlStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("EccFirstAddress", SimTypeInt(signed=False, label="UInt32")), ("EccSecondAddress", SimTypeInt(signed=False, label="UInt32")), ("EccAttribute", SimTypeInt(signed=False, label="UInt32")),)), name="PCIX_BRIDGE_CAPABILITY", pack=False, align=None),
    "PCI_ACS_BIT": SimTypeInt(signed=False, label="PCI_ACS_BIT"),
    "PCI_ADVANCED_FEATURES_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("Length", SimTypeChar(label="Byte")), ("Capabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="PCI_ADVANCED_FEATURES_CAPABILITY", pack=False, align=None),
    "PCI_AGP_APERTURE_PAGE_SIZE": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_AGP_APERTURE_PAGE_SIZE", pack=False, align=None),
    "PCI_AGP_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("AGPStatus", SimTypeRef("_PCI_AGP_STATUS", SimStruct)), ("AGPCommand", SimTypeRef("_PCI_AGP_COMMAND", SimStruct)),)), name="PCI_AGP_CAPABILITY", pack=False, align=None),
    "PCI_AGP_CONTROL": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_AGP_CONTROL", pack=False, align=None),
    "PCI_AGP_EXTENDED_CAPABILITY": SimStruct(OrderedDict((("IsochStatus", SimTypeRef("PCI_AGP_ISOCH_STATUS", SimStruct)), ("AgpControl", SimTypeRef("PCI_AGP_CONTROL", SimStruct)), ("ApertureSize", SimTypeShort(signed=False, label="UInt16")), ("AperturePageSize", SimTypeRef("PCI_AGP_APERTURE_PAGE_SIZE", SimStruct)), ("GartLow", SimTypeInt(signed=False, label="UInt32")), ("GartHigh", SimTypeInt(signed=False, label="UInt32")), ("IsochCommand", SimTypeRef("PCI_AGP_ISOCH_COMMAND", SimStruct)),)), name="PCI_AGP_EXTENDED_CAPABILITY", pack=False, align=None),
    "PCI_AGP_ISOCH_COMMAND": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_AGP_ISOCH_COMMAND", pack=False, align=None),
    "PCI_AGP_ISOCH_STATUS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_AGP_ISOCH_STATUS", pack=False, align=None),
    "PCI_ATS_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetAddressTranslationServices", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("InvalidateQueueDepth", SimTypeChar(label="Byte")),)), name="PCI_ATS_INTERFACE", pack=False, align=None),
    "PCI_BUS_INTERFACE_STANDARD": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("ReadConfig", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("WriteConfig", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("PinToLine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("LineToPin", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("RootBusCapability", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ExpressWakeControl", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("PrepareMultistageResume", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)),)), name="PCI_BUS_INTERFACE_STANDARD", pack=False, align=None),
    "PCI_BUS_WIDTH": SimTypeInt(signed=False, label="PCI_BUS_WIDTH"),
    "PCI_CAPABILITIES_HEADER": SimStruct(OrderedDict((("CapabilityID", SimTypeChar(label="Byte")), ("Next", SimTypeChar(label="Byte")),)), name="PCI_CAPABILITIES_HEADER", pack=False, align=None),
    "PCI_COMMON_CONFIG": SimStruct(OrderedDict((("Base", SimTypeRef("PCI_COMMON_HEADER", SimStruct)), ("DeviceSpecific", SimTypeArray(SimTypeChar(label="Byte"), 192)),)), name="PCI_COMMON_CONFIG", pack=False, align=None),
    "PCI_COMMON_HEADER": SimStruct(OrderedDict((("VendorID", SimTypeShort(signed=False, label="UInt16")), ("DeviceID", SimTypeShort(signed=False, label="UInt16")), ("Command", SimTypeShort(signed=False, label="UInt16")), ("Status", SimTypeShort(signed=False, label="UInt16")), ("RevisionID", SimTypeChar(label="Byte")), ("ProgIf", SimTypeChar(label="Byte")), ("SubClass", SimTypeChar(label="Byte")), ("BaseClass", SimTypeChar(label="Byte")), ("CacheLineSize", SimTypeChar(label="Byte")), ("LatencyTimer", SimTypeChar(label="Byte")), ("HeaderType", SimTypeChar(label="Byte")), ("BIST", SimTypeChar(label="Byte")), ("u", SimUnion({"type0": SimTypeRef("_PCI_HEADER_TYPE_0", SimStruct), "type1": SimTypeRef("_PCI_HEADER_TYPE_1", SimStruct), "type2": SimTypeRef("_PCI_HEADER_TYPE_2", SimStruct)}, name="<anon>", label="None")),)), name="PCI_COMMON_HEADER", pack=False, align=None),
    "PCI_DEBUGGING_DEVICE_IN_USE": SimStruct(OrderedDict((("Segment", SimTypeShort(signed=False, label="UInt16")), ("Bus", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_DEBUGGING_DEVICE_IN_USE", pack=False, align=None),
    "PCI_DEVICE_D3COLD_STATE_REASON": SimTypeInt(signed=False, label="PCI_DEVICE_D3COLD_STATE_REASON"),
    "PCI_DEVICE_PRESENCE_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("VendorID", SimTypeShort(signed=False, label="UInt16")), ("DeviceID", SimTypeShort(signed=False, label="UInt16")), ("RevisionID", SimTypeChar(label="Byte")), ("SubVendorID", SimTypeShort(signed=False, label="UInt16")), ("SubSystemID", SimTypeShort(signed=False, label="UInt16")), ("BaseClass", SimTypeChar(label="Byte")), ("SubClass", SimTypeChar(label="Byte")), ("ProgIf", SimTypeChar(label="Byte")),)), name="PCI_DEVICE_PRESENCE_PARAMETERS", pack=False, align=None),
    "PCI_DEVICE_PRESENT_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("IsDevicePresent", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)), ("IsDevicePresentEx", SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0)),)), name="PCI_DEVICE_PRESENT_INTERFACE", pack=False, align=None),
    "PCI_ERROR_HANDLER_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PCI_EXPRESS_ACS_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Capability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("EgressControl", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PCI_EXPRESS_ACS_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_ACS_CAPABILITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_ACS_CONTROL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_AER_CAPABILITIES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_AER_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("UncorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UncorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UncorrectableErrorSeverity", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CapabilitiesAndControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("HeaderLog", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("SecUncorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecUncorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecUncorrectableErrorSeverity", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecCapabilitiesAndControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecHeaderLog", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="PCI_EXPRESS_AER_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_ARI_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Capability", SimTypeRef("PCI_EXPRESS_ARI_CAPABILITY_REGISTER", SimStruct)), ("Control", SimTypeRef("PCI_EXPRESS_ARI_CONTROL_REGISTER", SimStruct)),)), name="PCI_EXPRESS_ARI_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_ARI_CAPABILITY_REGISTER": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_EXPRESS_ARI_CAPABILITY_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_ARI_CONTROL_REGISTER": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_EXPRESS_ARI_CONTROL_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_ASPM_CONTROL": SimTypeInt(signed=False, label="PCI_EXPRESS_ASPM_CONTROL"),
    "PCI_EXPRESS_ASPM_SUPPORT": SimTypeInt(signed=False, label="PCI_EXPRESS_ASPM_SUPPORT"),
    "PCI_EXPRESS_ATS_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Capability", SimTypeRef("PCI_EXPRESS_ATS_CAPABILITY_REGISTER", SimStruct)), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_ATS_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_ATS_CAPABILITY_REGISTER": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_EXPRESS_ATS_CAPABILITY_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_ATS_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_BRIDGE_AER_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("UncorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UncorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UncorrectableErrorSeverity", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CapabilitiesAndControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("HeaderLog", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("SecUncorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecUncorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecUncorrectableErrorSeverity", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecCapabilitiesAndControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SecHeaderLog", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="PCI_EXPRESS_BRIDGE_AER_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("ExpressCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DeviceCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DeviceControl", SimUnion({"Anonymous1": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DeviceStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("LinkCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("LinkControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("LinkStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("SlotCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SlotControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("SlotStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("RootControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("RootCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("RootStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DeviceCapabilities2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DeviceControl2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DeviceStatus2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("LinkCapabilities2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("LinkControl2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("LinkStatus2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_CARD_PRESENCE": SimTypeInt(signed=False, label="PCI_EXPRESS_CARD_PRESENCE"),
    "PCI_EXPRESS_CORRECTABLE_ERROR_MASK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CORRECTABLE_ERROR_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CXL_DVSEC_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("DvsecHeader1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DvsecHeader2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 46)),)), name="PCI_EXPRESS_CXL_DVSEC_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_CXL_DVSEC_CAPABILITY_REGISTER_V11": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CXL_DVSEC_CAPABILITY_V11": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("DvsecHeader1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DvsecHeader2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Capability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Control2", SimTypeShort(signed=False, label="UInt16")), ("Status2", SimTypeShort(signed=False, label="UInt16")), ("Lock", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Range1SizeHigh", SimTypeRef("PCI_EXPRESS_CXL_DVSEC_RANGE_SIZE_HIGH_REGISTER", SimStruct)), ("Range1SizeLow", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Range1BaseHigh", SimTypeRef("PCI_EXPRESS_CXL_DVSEC_RANGE_BASE_HIGH_REGISTER", SimStruct)), ("Range1BaseLow", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Range2SizeHigh", SimTypeRef("PCI_EXPRESS_CXL_DVSEC_RANGE_SIZE_HIGH_REGISTER", SimStruct)), ("Range2SizeLow", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Range2BaseHigh", SimTypeRef("PCI_EXPRESS_CXL_DVSEC_RANGE_BASE_HIGH_REGISTER", SimStruct)), ("Range2BaseLow", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_CXL_DVSEC_CAPABILITY_V11", pack=False, align=None),
    "PCI_EXPRESS_CXL_DVSEC_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CXL_DVSEC_LOCK_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CXL_DVSEC_RANGE_BASE_HIGH_REGISTER": SimStruct(OrderedDict((("MemBaseHigh", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_EXPRESS_CXL_DVSEC_RANGE_BASE_HIGH_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_CXL_DVSEC_RANGE_BASE_LOW_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CXL_DVSEC_RANGE_SIZE_HIGH_REGISTER": SimStruct(OrderedDict((("MemSizeHigh", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_EXPRESS_CXL_DVSEC_RANGE_SIZE_HIGH_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_CXL_DVSEC_RANGE_SIZE_LOW_REGISTER_V11": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_CXL_DVSEC_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DESIGNATED_VENDOR_SPECIFIC_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("DvsecHeader1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("DvsecHeader2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DvsecRegisters", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="PCI_EXPRESS_DESIGNATED_VENDOR_SPECIFIC_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_DESIGNATED_VENDOR_SPECIFIC_HEADER_1": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DESIGNATED_VENDOR_SPECIFIC_HEADER_2": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_CAPABILITIES_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_CONTROL_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_CONTROL_REGISTER": SimUnion({"Anonymous1": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_STATUS_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DEVICE_TYPE": SimTypeInt(signed=False, label="PCI_EXPRESS_DEVICE_TYPE"),
    "PCI_EXPRESS_DPC_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("DpcCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DpcControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DpcStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("DpcErrSrcId", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("RpPioStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RpPioMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RpPioSeverity", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RpPioSysError", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RpPioException", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RpPioHeaderLog", SimTypeRef("PCI_EXPRESS_DPC_RP_PIO_HEADERLOG_REGISTER", SimStruct)), ("RpPioImpSpecLog", SimUnion({"PioImpSpecLog": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RpPioPrefixLog", SimTypeRef("PCI_EXPRESS_DPC_RP_PIO_TLPPREFIXLOG_REGISTER", SimStruct)),)), name="PCI_EXPRESS_DPC_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_DPC_CAPS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_ERROR_SOURCE_ID": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_EXCEPTION_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_HEADERLOG_REGISTER": SimStruct(OrderedDict((("PioHeaderLogRegister", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="PCI_EXPRESS_DPC_RP_PIO_HEADERLOG_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_DPC_RP_PIO_IMPSPECLOG_REGISTER": SimUnion({"PioImpSpecLog": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_MASK_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_SEVERITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_SYSERR_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_DPC_RP_PIO_TLPPREFIXLOG_REGISTER": SimStruct(OrderedDict((("PioTlpPrefixLogRegister", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="PCI_EXPRESS_DPC_RP_PIO_TLPPREFIXLOG_REGISTER", pack=False, align=None),
    "PCI_EXPRESS_DPC_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER": SimStruct(OrderedDict((("CapabilityID", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", pack=False, align=None),
    "PCI_EXPRESS_ENTER_LINK_QUIESCENT_MODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PCI_EXPRESS_ERROR_SOURCE_ID": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield1", SimTypeShort(signed=False, label="UInt16")), ("_bitfield2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_EVENT_COLLECTOR_ENDPOINT_ASSOCIATION_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("AssociationBitmap", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_EXPRESS_EVENT_COLLECTOR_ENDPOINT_ASSOCIATION_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_EXIT_LINK_QUIESCENT_MODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PCI_EXPRESS_INDICATOR_STATE": SimTypeInt(signed=False, label="PCI_EXPRESS_INDICATOR_STATE"),
    "PCI_EXPRESS_L0s_EXIT_LATENCY": SimTypeInt(signed=False, label="PCI_EXPRESS_L0s_EXIT_LATENCY"),
    "PCI_EXPRESS_L1_EXIT_LATENCY": SimTypeInt(signed=False, label="PCI_EXPRESS_L1_EXIT_LATENCY"),
    "PCI_EXPRESS_L1_PM_SS_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_L1_PM_SS_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("L1PmSsCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("L1PmSsControl1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("L1PmSsControl2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_L1_PM_SS_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_L1_PM_SS_CONTROL_1_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_L1_PM_SS_CONTROL_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LANE_ERROR_STATUS": SimStruct(OrderedDict((("LaneBitmap", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_EXPRESS_LANE_ERROR_STATUS", pack=False, align=None),
    "PCI_EXPRESS_LINK_CAPABILITIES_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LINK_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LINK_CONTROL3": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_LINK_CONTROL3", pack=False, align=None),
    "PCI_EXPRESS_LINK_CONTROL_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LINK_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LINK_QUIESCENT_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("PciExpressEnterLinkQuiescentMode", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("PciExpressExitLinkQuiescentMode", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="PCI_EXPRESS_LINK_QUIESCENT_INTERFACE", pack=False, align=None),
    "PCI_EXPRESS_LINK_STATUS_2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LINK_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_LINK_SUBSTATE": SimTypeInt(signed=False, label="PCI_EXPRESS_LINK_SUBSTATE"),
    "PCI_EXPRESS_LTR_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Latency", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_LTR_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_LTR_MAX_LATENCY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_MAX_PAYLOAD_SIZE": SimTypeInt(signed=False, label="PCI_EXPRESS_MAX_PAYLOAD_SIZE"),
    "PCI_EXPRESS_MRL_STATE": SimTypeInt(signed=False, label="PCI_EXPRESS_MRL_STATE"),
    "PCI_EXPRESS_NPEM_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Capability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_NPEM_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_NPEM_CAPABILITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_NPEM_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_NPEM_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_PASID_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Capability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_PASID_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_PASID_CAPABILITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_PASID_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_PME_REQUESTOR_ID": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_POWER_STATE": SimTypeInt(signed=False, label="PCI_EXPRESS_POWER_STATE"),
    "PCI_EXPRESS_PRI_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("PRCapacity", SimTypeInt(signed=False, label="UInt32")), ("PRAllocation", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_EXPRESS_PRI_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_PRI_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_PRI_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_PTM_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("PtmCapability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("PtmControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_PTM_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_PTM_CAPABILITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_PTM_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_RCB": SimTypeInt(signed=False, label="PCI_EXPRESS_RCB"),
    "PCI_EXPRESS_RESIZABLE_BAR_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Entry", SimTypeArray(SimTypeRef("PCI_EXPRESS_RESIZABLE_BAR_ENTRY", SimStruct), 6)),)), name="PCI_EXPRESS_RESIZABLE_BAR_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_RESIZABLE_BAR_CAPABILITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_RESIZABLE_BAR_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_RESIZABLE_BAR_ENTRY": SimStruct(OrderedDict((("Capability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_RESIZABLE_BAR_ENTRY", pack=False, align=None),
    "PCI_EXPRESS_ROOTPORT_AER_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("UncorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UncorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("UncorrectableErrorSeverity", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CorrectableErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CorrectableErrorMask", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CapabilitiesAndControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("HeaderLog", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("RootErrorCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RootErrorStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ErrorSourceId", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield1", SimTypeShort(signed=False, label="UInt16")), ("_bitfield2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_ROOTPORT_AER_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_ROOT_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_ROOT_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_ROOT_ERROR_COMMAND": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_ROOT_ERROR_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_ROOT_PORT_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("ReadConfigSpace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Buffer", "Offset", "Length"]), offset=0)), ("WriteConfigSpace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Buffer", "Offset", "Length"]), offset=0)),)), name="PCI_EXPRESS_ROOT_PORT_INTERFACE", pack=False, align=None),
    "PCI_EXPRESS_ROOT_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SECONDARY_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("LinkControl3", SimTypeRef("PCI_EXPRESS_LINK_CONTROL3", SimStruct)), ("LaneErrorStatus", SimTypeRef("PCI_EXPRESS_LANE_ERROR_STATUS", SimStruct)),)), name="PCI_EXPRESS_SECONDARY_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_SEC_AER_CAPABILITIES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SEC_UNCORRECTABLE_ERROR_MASK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SEC_UNCORRECTABLE_ERROR_SEVERITY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SEC_UNCORRECTABLE_ERROR_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SERIAL_NUMBER_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("LowSerialNumber", SimTypeInt(signed=False, label="UInt32")), ("HighSerialNumber", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_EXPRESS_SERIAL_NUMBER_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_SLOT_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SLOT_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SLOT_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SRIOV_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("SRIOVCapabilities", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SRIOVControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("SRIOVStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("InitialVFs", SimTypeShort(signed=False, label="UInt16")), ("TotalVFs", SimTypeShort(signed=False, label="UInt16")), ("NumVFs", SimTypeShort(signed=False, label="UInt16")), ("FunctionDependencyLink", SimTypeChar(label="Byte")), ("RsvdP1", SimTypeChar(label="Byte")), ("FirstVFOffset", SimTypeShort(signed=False, label="UInt16")), ("VFStride", SimTypeShort(signed=False, label="UInt16")), ("RsvdP2", SimTypeShort(signed=False, label="UInt16")), ("VFDeviceId", SimTypeShort(signed=False, label="UInt16")), ("SupportedPageSizes", SimTypeInt(signed=False, label="UInt32")), ("SystemPageSize", SimTypeInt(signed=False, label="UInt32")), ("BaseAddresses", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)), ("VFMigrationStateArrayOffset", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_SRIOV_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_SRIOV_CAPS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SRIOV_CONTROL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SRIOV_MIGRATION_STATE_ARRAY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_SRIOV_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_TPH_REQUESTER_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("RequesterCapability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RequesterControl", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_EXPRESS_TPH_REQUESTER_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_TPH_REQUESTER_CAPABILITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_TPH_REQUESTER_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_TPH_ST_TABLE_ENTRY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "PCI_EXPRESS_UNCORRECTABLE_ERROR_MASK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_UNCORRECTABLE_ERROR_SEVERITY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_UNCORRECTABLE_ERROR_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_EXPRESS_VENDOR_SPECIFIC_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("VsecId", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_EXPRESS_VENDOR_SPECIFIC_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_VIRTUAL_CHANNEL_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_EXPRESS_ENHANCED_CAPABILITY_HEADER", SimStruct)), ("Capabilities1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Capabilities2", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Resource", SimTypeArray(SimTypeRef("VIRTUAL_RESOURCE", SimStruct), 8)),)), name="PCI_EXPRESS_VIRTUAL_CHANNEL_CAPABILITY", pack=False, align=None),
    "PCI_EXPRESS_WAKE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Context", "EnableWake"]), offset=0),
    "PCI_FIRMWARE_BUS_CAPS": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)), ("CurrentSpeedAndMode", SimTypeChar(label="Byte")), ("SupportedSpeedsAndModesLowByte", SimTypeChar(label="Byte")), ("SupportedSpeedsAndModesHighByte", SimTypeChar(label="Byte")), ("Voltage", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 7)),)), name="PCI_FIRMWARE_BUS_CAPS", pack=False, align=None),
    "PCI_FIRMWARE_BUS_CAPS_RETURN_BUFFER": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Status", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Caps", SimTypeRef("PCI_FIRMWARE_BUS_CAPS", SimStruct)),)), name="PCI_FIRMWARE_BUS_CAPS_RETURN_BUFFER", pack=False, align=None),
    "PCI_FPB_CAPABILITIES_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_FPB_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_FPB_CAPABILITY_HEADER", SimStruct)), ("CapabilitiesRegister", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RidVectorControl1Register", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RidVectorControl2Register", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MemLowVectorControlRegister", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MemHighVectorControl1Register", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MemHighVectorControl2Register", SimTypeRef("PCI_FPB_MEM_HIGH_VECTOR_CONTROL2_REGISTER", SimStruct)), ("VectorAccessControlRegister", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("VectorAccessDataRegister", SimTypeRef("PCI_FPB_VECTOR_ACCESS_DATA_REGISTER", SimStruct)),)), name="PCI_FPB_CAPABILITY", pack=False, align=None),
    "PCI_FPB_CAPABILITY_HEADER": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_FPB_CAPABILITY_HEADER", pack=False, align=None),
    "PCI_FPB_MEM_HIGH_VECTOR_CONTROL1_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_FPB_MEM_HIGH_VECTOR_CONTROL2_REGISTER": SimStruct(OrderedDict((("MemHighVectorStartUpper", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_FPB_MEM_HIGH_VECTOR_CONTROL2_REGISTER", pack=False, align=None),
    "PCI_FPB_MEM_LOW_VECTOR_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_FPB_RID_VECTOR_CONTROL1_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_FPB_RID_VECTOR_CONTROL2_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_FPB_VECTOR_ACCESS_CONTROL_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "PCI_FPB_VECTOR_ACCESS_DATA_REGISTER": SimStruct(OrderedDict((("VectorAccessData", SimTypeInt(signed=False, label="UInt32")),)), name="PCI_FPB_VECTOR_ACCESS_DATA_REGISTER", pack=False, align=None),
    "PCI_HARDWARE_INTERFACE": SimTypeInt(signed=False, label="PCI_HARDWARE_INTERFACE"),
    "PCI_IS_DEVICE_PRESENT": SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeChar(label="Byte"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["VendorID", "DeviceID", "RevisionID", "SubVendorID", "SubSystemID", "Flags"]), offset=0),
    "PCI_IS_DEVICE_PRESENT_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PCI_DEVICE_PRESENCE_PARAMETERS", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["Context", "Parameters"]), offset=0),
    "PCI_LINE_TO_PIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "PciNewData", "PciOldData"]), offset=0),
    "PCI_MSIX_GET_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "TableEntry", "MessageNumber", "Masked"]), offset=0),
    "PCI_MSIX_GET_TABLE_SIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "TableSize"]), offset=0),
    "PCI_MSIX_MASKUNMASK_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "TableEntry"]), offset=0),
    "PCI_MSIX_SET_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "TableEntry", "MessageNumber"]), offset=0),
    "PCI_MSIX_TABLE_CONFIG_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetTableEntry", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("MaskTableEntry", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("UnmaskTableEntry", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetTableEntry", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetTableSize", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="PCI_MSIX_TABLE_CONFIG_INTERFACE", pack=False, align=None),
    "PCI_PIN_TO_LINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "PciData"]), offset=0),
    "PCI_PMC": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")), ("Support", SimTypeRef("_PM_SUPPORT", SimStruct)),)), name="PCI_PMC", pack=False, align=None),
    "PCI_PMCSR": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_PMCSR", pack=False, align=None),
    "PCI_PMCSR_BSE": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="PCI_PMCSR_BSE", pack=False, align=None),
    "PCI_PM_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("PMC", SimUnion({"Capabilities": SimTypeRef("PCI_PMC", SimStruct), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("PMCSR", SimUnion({"ControlStatus": SimTypeRef("PCI_PMCSR", SimStruct), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("PMCSR_BSE", SimUnion({"BridgeSupport": SimTypeRef("PCI_PMCSR_BSE", SimStruct), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Data", SimTypeChar(label="Byte")),)), name="PCI_PM_CAPABILITY", pack=False, align=None),
    "PCI_PREPARE_MULTISTAGE_RESUME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PCI_READ_WRITE_CONFIG": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "BusOffset", "Slot", "Buffer", "Offset", "Length"]), offset=0),
    "PCI_ROOT_BUS_CAPABILITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PCI_ROOT_BUS_HARDWARE_CAPABILITY", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "HardwareCapability"]), offset=0),
    "PCI_ROOT_BUS_HARDWARE_CAPABILITY": SimStruct(OrderedDict((("SecondaryInterface", SimTypeInt(signed=False, label="PCI_HARDWARE_INTERFACE")), ("Anonymous", SimStruct(OrderedDict((("BusCapabilitiesFound", SimTypeChar(label="Byte")), ("CurrentSpeedAndMode", SimTypeInt(signed=False, label="UInt32")), ("SupportedSpeedsAndModes", SimTypeInt(signed=False, label="UInt32")), ("DeviceIDMessagingCapable", SimTypeChar(label="Byte")), ("SecondaryBusWidth", SimTypeInt(signed=False, label="PCI_BUS_WIDTH")),)), name="_Anonymous_e__Struct", pack=False, align=None)), ("OscFeatureSupport", SimTypeRef("PCI_ROOT_BUS_OSC_SUPPORT_FIELD", SimStruct)), ("OscControlRequest", SimTypeRef("PCI_ROOT_BUS_OSC_CONTROL_FIELD", SimStruct)), ("OscControlGranted", SimTypeRef("PCI_ROOT_BUS_OSC_CONTROL_FIELD", SimStruct)),)), name="PCI_ROOT_BUS_HARDWARE_CAPABILITY", pack=False, align=None),
    "PCI_ROOT_BUS_OSC_CONTROL_FIELD": SimStruct(OrderedDict((("u", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_ROOT_BUS_OSC_CONTROL_FIELD", pack=False, align=None),
    "PCI_ROOT_BUS_OSC_SUPPORT_FIELD": SimStruct(OrderedDict((("u", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_ROOT_BUS_OSC_SUPPORT_FIELD", pack=False, align=None),
    "PCI_SECURITY_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetAccessControlServices", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="PCI_SECURITY_INTERFACE", pack=False, align=None),
    "PCI_SECURITY_INTERFACE2": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SupportedScenarios", SimTypeInt(signed=False, label="UInt32")), ("SetAccessControlServices", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="PCI_SECURITY_INTERFACE2", pack=False, align=None),
    "PCI_SEGMENT_BUS_NUMBER": SimStruct(OrderedDict((("u", SimUnion({"bits": SimTypeRef("_bits_e__Struct", SimStruct), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_SEGMENT_BUS_NUMBER", pack=False, align=None),
    "PCI_SET_ACS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="PCI_ACS_BIT"), SimTypeInt(signed=False, label="PCI_ACS_BIT"), SimTypeInt(signed=False, label="PCI_ACS_BIT"), SimTypeInt(signed=False, label="PCI_ACS_BIT"), SimTypeInt(signed=False, label="PCI_ACS_BIT"), SimTypeInt(signed=False, label="PCI_ACS_BIT"), SimTypeInt(signed=False, label="PCI_ACS_BIT")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "EnableSourceValidation", "EnableTranslationBlocking", "EnableP2PRequestRedirect", "EnableCompletionRedirect", "EnableUpstreamForwarding", "EnableEgressControl", "EnableDirectTranslatedP2P"]), offset=0),
    "PCI_SET_ACS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ScenariosToModify", "ScenarioState"]), offset=0),
    "PCI_SET_ATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "EnableAts"]), offset=0),
    "PCI_SLOT_NUMBER": SimStruct(OrderedDict((("u", SimUnion({"bits": SimTypeRef("_bits_e__Struct", SimStruct), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_SLOT_NUMBER", pack=False, align=None),
    "PCI_SUBSYSTEM_IDS_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("SubVendorID", SimTypeShort(signed=False, label="UInt16")), ("SubSystemID", SimTypeShort(signed=False, label="UInt16")),)), name="PCI_SUBSYSTEM_IDS_CAPABILITY", pack=False, align=None),
    "PCI_VENDOR_SPECIFIC_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("VscLength", SimTypeChar(label="Byte")), ("VendorSpecific", SimTypeChar(label="Byte")),)), name="PCI_VENDOR_SPECIFIC_CAPABILITY", pack=False, align=None),
    "PCI_VIRTUALIZATION_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SetVirtualFunctionData", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("GetVirtualFunctionData", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("GetLocation", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetResources", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("EnableVirtualization", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetVirtualFunctionProbedBars", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="PCI_VIRTUALIZATION_INTERFACE", pack=False, align=None),
    "PCI_X_CAPABILITY": SimStruct(OrderedDict((("Header", SimTypeRef("PCI_CAPABILITIES_HEADER", SimStruct)), ("Command", SimUnion({"bits": SimTypeRef("_bits_e__Struct", SimStruct), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Status", SimUnion({"bits": SimTypeRef("_bits_e__Struct", SimStruct), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PCI_X_CAPABILITY", pack=False, align=None),
    "PCLFS_CLIENT_ADVANCE_TAIL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLS_LSN", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogFile", "TargetLsn", "ClientData"]), offset=0),
    "PCLFS_CLIENT_LFF_HANDLER_COMPLETE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogFile", "OperationStatus", "LogIsPinned", "ClientData"]), offset=0),
    "PCLFS_CLIENT_LOG_UNPINNED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogFile", "ClientData"]), offset=0),
    "PCLFS_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pvOverlapped", "ulReserved"]), offset=0),
    "PCLFS_SET_LOG_SIZE_COMPLETE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogFile", "OperationStatus", "ClientData"]), offset=0),
    "PCLOSE_CLUSTER_CRYPT_PROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusCryptProvider"]), offset=0),
    "PCLOSE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0),
    "PCLUSAPIClusWorkerCheckTerminate": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUS_WORKER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpWorker"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_GROUP_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroup", "hProviderGroup"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_GROUP_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroup", "hProviderGroup", "lpszReason"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_GROUP_GROUPSET_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroupSet", "hProviderGroupSet"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_GROUP_GROUPSET_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroupSet", "hProviderGroupSet", "lpszReason"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroup", "hProviderGroupSet"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroup", "hProviderGroupSet", "lpszReason"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNodeName", "pfnProgressCallback", "pvCallbackArg"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNodeName", "dwFlags", "pfnProgressCallback", "pvCallbackArg"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_RESOURCE_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hDependsOn"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_RESOURCE_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hDependsOn", "lpszReason"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_RESOURCE_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hNode"]), offset=0),
    "PCLUSAPI_ADD_CLUSTER_RESOURCE_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hNode", "lpszReason"]), offset=0),
    "PCLUSAPI_ADD_CROSS_CLUSTER_GROUPSET_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroupSet", "lpRemoteClusterName", "lpRemoteGroupSetName"]), offset=0),
    "PCLUSAPI_ADD_RESOURCE_TO_CLUSTER_SHARED_VOLUMES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_BACKUP_CLUSTER_DATABASE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszPathName"]), offset=0),
    "PCLUSAPI_CAN_RESOURCE_BE_DEPENDENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hResource", "hResourceDependent"]), offset=0),
    "PCLUSAPI_CHANGE_CLUSTER_RESOURCE_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hGroup"]), offset=0),
    "PCLUSAPI_CHANGE_CLUSTER_RESOURCE_GROUP_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hGroup", "Flags"]), offset=0),
    "PCLUSAPI_CHANGE_CLUSTER_RESOURCE_GROUP_EX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hGroup", "Flags", "lpszReason"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hCluster"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_GROUP_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroupSet"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_NETWORK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNetwork"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_NET_INTERFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNetInterface"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNode"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_NOTIFY_PORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hChange"]), offset=0),
    "PCLUSAPI_CLOSE_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_CLUSTER_ADD_GROUP_TO_AFFINITY_RULE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "ruleName", "hGroup"]), offset=0),
    "PCLUSAPI_CLUSTER_ADD_GROUP_TO_GROUPSET_WITH_DOMAINS_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "hGroup", "faultDomain", "updateDomain", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_ADD_GROUP_TO_GROUP_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "hGroup"]), offset=0),
    "PCLUSAPI_CLUSTER_AFFINITY_RULE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "affinityRuleName", "hHostNode", "dwControlCode", "lpInBuffer", "cbInBufferSize", "lpOutBuffer", "cbOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_CLOSE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_CLOSE_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusterEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_CREATE_AFFINITY_RULE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="CLUS_AFFINITY_RULE_TYPE")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "ruleName", "ruleType"]), offset=0),
    "PCLUSAPI_CLUSTER_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEnum", "dwIndex", "lpdwType", "lpszName", "lpcchName"]), offset=0),
    "PCLUSAPI_CLUSTER_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLUSTER_ENUM_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusterEnum", "dwIndex", "pItem", "cbItem"]), offset=0),
    "PCLUSAPI_CLUSTER_GET_ENUM_COUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_GET_ENUM_COUNT_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusterEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_CLOSE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_CLOSE_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupEnumEx"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupEnum", "dwIndex", "lpdwType", "lpszResourceName", "lpcchName"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLUSTER_GROUP_ENUM_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupEnumEx", "dwIndex", "pItem", "cbItem"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_GET_ENUM_COUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_GET_ENUM_COUNT_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupEnumEx"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_GROUPSET_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "hHostNode", "dwControlCode", "lpInBuffer", "cbInBufferSize", "lpOutBuffer", "cbOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_GROUPSET_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "hHostNode", "dwControlCode", "lpInBuffer", "cbInBufferSize", "lpOutBuffer", "cbOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_OPEN_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hGroup", "dwType"]), offset=0),
    "PCLUSAPI_CLUSTER_GROUP_OPEN_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszProperties", "cbProperties", "lpszRoProperties", "cbRoProperties", "dwFlags"]), offset=0),
    "PCLUSAPI_CLUSTER_NETWORK_CLOSE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetworkEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_NETWORK_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetwork", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_NETWORK_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetwork", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_NETWORK_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetworkEnum", "dwIndex", "lpdwType", "lpszName", "lpcchName"]), offset=0),
    "PCLUSAPI_CLUSTER_NETWORK_GET_ENUM_COUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetworkEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_NETWORK_OPEN_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNetwork", "dwType"]), offset=0),
    "PCLUSAPI_CLUSTER_NET_INTERFACE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetInterface", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_NET_INTERFACE_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetInterface", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_CLOSE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNodeEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_CLOSE_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNodeEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNodeEnum", "dwIndex", "lpdwType", "lpszName", "lpcchName"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLUSTER_ENUM_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNodeEnum", "dwIndex", "pItem", "cbItem"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_GET_ENUM_COUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNodeEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_GET_ENUM_COUNT_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNodeEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_OPEN_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNode", "dwType"]), offset=0),
    "PCLUSAPI_CLUSTER_NODE_OPEN_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNode", "dwType", "pOptions"]), offset=0),
    "PCLUSAPI_CLUSTER_OPEN_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "dwType"]), offset=0),
    "PCLUSAPI_CLUSTER_OPEN_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "dwType", "pOptions"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_CLOSE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_CREATE_BATCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "pHREGBATCH"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_CREATE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpszSubKey", "dwOptions", "samDesired", "lpSecurityAttributes", "phkResult", "lpdwDisposition"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_CREATE_KEY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpszSubKey", "dwOptions", "samDesired", "lpSecurityAttributes", "phkResult", "lpdwDisposition", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_DELETE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpszSubKey"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_DELETE_KEY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpSubKey", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_DELETE_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hKey", "lpszValueName"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_DELETE_VALUE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hKey", "lpszValueName", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_ENUM_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "dwIndex", "lpszName", "lpcchName", "lpftLastWriteTime"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_ENUM_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hKey", "dwIndex", "lpszValueName", "lpcchValueName", "lpdwType", "lpData", "lpcbData"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_GET_KEY_SECURITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "RequestedInformation", "pSecurityDescriptor", "lpcbSecurityDescriptor"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_OPEN_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpszSubKey", "samDesired", "phkResult"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_QUERY_INFO_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpcSubKeys", "lpcbMaxSubKeyLen", "lpcValues", "lpcbMaxValueNameLen", "lpcbMaxValueLen", "lpcbSecurityDescriptor", "lpftLastWriteTime"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_QUERY_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "lpszValueName", "lpdwValueType", "lpData", "lpcbData"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_SET_KEY_SECURITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "SecurityInformation", "pSecurityDescriptor"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_SET_KEY_SECURITY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "SecurityInformation", "pSecurityDescriptor", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_SET_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hKey", "lpszValueName", "dwType", "lpData", "cbData"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_SET_VALUE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hKey", "lpszValueName", "dwType", "lpData", "cbData", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_REG_SYNC_DATABASE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hCluster", "flags"]), offset=0),
    "PCLUSAPI_CLUSTER_REMOVE_AFFINITY_RULE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "ruleName"]), offset=0),
    "PCLUSAPI_CLUSTER_REMOVE_GROUP_FROM_AFFINITY_RULE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "ruleName", "hGroup"]), offset=0),
    "PCLUSAPI_CLUSTER_REMOVE_GROUP_FROM_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet"]), offset=0),
    "PCLUSAPI_CLUSTER_REMOVE_GROUP_FROM_GROUPSET_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_CLOSE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_CLOSE_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResourceEnumEx"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hHostNode", "dwControlCode", "lpInBuffer", "cbInBufferSize", "lpOutBuffer", "cbOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_CONTROL_AS_USER_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hHostNode", "dwControlCode", "lpInBuffer", "cbInBufferSize", "lpOutBuffer", "cbOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hHostNode", "dwControlCode", "lpInBuffer", "cbInBufferSize", "lpOutBuffer", "cbOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResEnum", "dwIndex", "lpdwType", "lpszName", "lpcchName"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLUSTER_RESOURCE_ENUM_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResourceEnumEx", "dwIndex", "pItem", "cbItem"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_GET_ENUM_COUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_GET_ENUM_COUNT_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResourceEnumEx"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_OPEN_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hResource", "dwType"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_OPEN_ENUM_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszProperties", "cbProperties", "lpszRoProperties", "cbRoProperties", "dwFlags"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_CLOSE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResTypeEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceTypeName", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_CONTROL_AS_USER_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceTypeName", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_CONTROL_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceTypeName", "hHostNode", "dwControlCode", "lpInBuffer", "nInBufferSize", "lpOutBuffer", "nOutBufferSize", "lpBytesReturned", "lpszReason"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResTypeEnum", "dwIndex", "lpdwType", "lpszName", "lpcchName"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_GET_ENUM_COUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResTypeEnum"]), offset=0),
    "PCLUSAPI_CLUSTER_RESOURCE_TYPE_OPEN_ENUM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszResourceTypeName", "dwType"]), offset=0),
    "PCLUSAPI_CLUSTER_UPGRADE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_UPGRADE_PHASE")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eUpgradePhase"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "perform", "pfnProgressCallback", "pvCallbackArg"]), offset=0),
    "PCLUSAPI_CLUS_WORKER_CREATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUS_WORKER", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUS_WORKER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pWorker", "lpThreadParameter"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpWorker", "lpStartAddress", "lpParameter"]), offset=0),
    "PCLUSAPI_CLUS_WORKER_TERMINATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUS_WORKER", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpWorker"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CREATE_CLUSTER_CONFIG", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pConfig", "pfnProgressCallback", "pvCallbackArg"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_AVAILABILITY_SET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("CLUSTER_AVAILABILITY_SET_CONFIG", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpAvailabilitySetName", "pAvailabilitySetConfig"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_CNOLESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CREATE_CLUSTER_CONFIG", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pConfig", "pfnProgressCallback", "pvCallbackArg"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszGroupName"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_GROUPEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("CLUSTER_CREATE_GROUP_INFO", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszGroupName", "pGroupInfo"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_GROUP_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszGroupSetName"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_NAME_ACCOUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CREATE_CLUSTER_NAME_ACCOUNT", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "pConfig", "pfnProgressCallback", "pvCallbackArg"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_NOTIFY_PORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hChange", "hCluster", "dwFilter", "dwNotifyKey"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_NOTIFY_PORT_V2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("NOTIFY_FILTER_AND_TYPE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hChange", "hCluster", "Filters", "dwFilterCount", "dwNotifyKey"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hGroup", "lpszResourceName", "lpszResourceType", "dwFlags"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_RESOURCE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hGroup", "lpszResourceName", "lpszResourceType", "dwFlags", "lpszReason"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_RESOURCE_TYPE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceTypeName", "lpszDisplayName", "lpszResourceTypeDll", "dwLooksAlivePollInterval", "dwIsAlivePollInterval"]), offset=0),
    "PCLUSAPI_CREATE_CLUSTER_RESOURCE_TYPE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceTypeName", "lpszDisplayName", "lpszResourceTypeDll", "dwLooksAlivePollInterval", "dwIsAlivePollInterval", "lpszReason"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_GROUP_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "lpszReason"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_GROUP_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_GROUP_GROUPSET_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "lpszReason"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_RESOURCE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszReason"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_RESOURCE_TYPE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceTypeName"]), offset=0),
    "PCLUSAPI_DELETE_CLUSTER_RESOURCE_TYPE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszTypeName", "lpszReason"]), offset=0),
    "PCLUSAPI_DESTROY_CLUSTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "pfnProgressCallback", "pvCallbackArg", "fdeleteVirtualComputerObjects"]), offset=0),
    "PCLUSAPI_DESTROY_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup"]), offset=0),
    "PCLUSAPI_DESTROY_CLUSTER_GROUP_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "lpszReason"]), offset=0),
    "PCLUSAPI_EVICT_CLUSTER_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode"]), offset=0),
    "PCLUSAPI_EVICT_CLUSTER_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "dwTimeOut", "phrCleanupStatus"]), offset=0),
    "PCLUSAPI_EVICT_CLUSTER_NODE_EX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "dwTimeout", "phrCleanupStatus", "lpszReason"]), offset=0),
    "PCLUSAPI_FAIL_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_FAIL_CLUSTER_RESOURCE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszReason"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_FROM_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hGroup"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_FROM_GROUP_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hGroupSet"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_FROM_NETWORK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNetwork"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_FROM_NET_INTERFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNetInterface"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_FROM_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNode"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_FROM_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_GROUP_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hGroup", "samDesired"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_GROUP_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="CLUSTER_GROUP_STATE"), arg_names=["hGroup", "lpszNodeName", "lpcchNodeName"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_INFORMATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("CLUSTERVERSIONINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszClusterName", "lpcchClusterName", "lpClusterInfo"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "samDesired"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NETWORK_ID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetwork", "lpszNetworkId", "lpcchName"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NETWORK_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNetwork", "samDesired"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NETWORK_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="CLUSTER_NETWORK_STATE"), arg_names=["hNetwork"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NET_INTERFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszNodeName", "lpszNetworkName", "lpszInterfaceName", "lpcchInterfaceName"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NET_INTERFACE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNetInterface", "samDesired"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NET_INTERFACE_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="CLUSTER_NETINTERFACE_STATE"), arg_names=["hNetInterface"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NODE_ID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "lpszNodeId", "lpcchName"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NODE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hNode", "samDesired"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NODE_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="CLUSTER_NODE_STATE"), arg_names=["hNode"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hChange", "lpdwNotifyKey", "lpdwFilterType", "lpszName", "lpcchName", "dwMilliseconds"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_NOTIFY_V2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("NOTIFY_FILTER_AND_TYPE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hChange", "lpdwNotifyKey", "pFilterAndType", "buffer", "lpcchBufferSize", "lpszObjectId", "lpcchObjectId", "lpszParentId", "lpcchParentId", "lpszName", "lpcchName", "lpszType", "lpcchType", "dwMilliseconds"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_QUORUM_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszResourceName", "lpcchResourceName", "lpszDeviceName", "lpcchDeviceName", "lpdwMaxQuorumLogSize"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_RESOURCE_DEPENDENCY_EXPRESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszDependencyExpression", "lpcchDependencyExpression"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_RESOURCE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hResource", "samDesired"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_RESOURCE_NETWORK_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hResource", "lpBuffer", "nSize"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_RESOURCE_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE"), arg_names=["hResource", "lpszNodeName", "lpcchNodeName", "lpszGroupName", "lpcchGroupName"]), offset=0),
    "PCLUSAPI_GET_CLUSTER_RESOURCE_TYPE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszTypeName", "samDesired"]), offset=0),
    "PCLUSAPI_GET_NODE_CLUSTER_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszNodeName", "pdwClusterState"]), offset=0),
    "PCLUSAPI_GET_NOTIFY_EVENT_HANDLE_V2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hChange", "lphTargetEvent"]), offset=0),
    "PCLUSAPI_IS_FILE_ON_CLUSTER_SHARED_VOLUME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszPathName", "pbFileIsOnSharedVolume"]), offset=0),
    "PCLUSAPI_MOVE_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hDestinationNode"]), offset=0),
    "PCLUSAPI_OFFLINE_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup"]), offset=0),
    "PCLUSAPI_OFFLINE_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_ONLINE_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hDestinationNode"]), offset=0),
    "PCLUSAPI_ONLINE_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpszClusterName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpszClusterName", "dwDesiredAccess", "lpdwGrantedAccess"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_GROUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszGroupName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_GROUP_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszGroupName", "dwDesiredAccess", "lpdwGrantedAccess"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_GROUP_GROUPSET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszGroupSetName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_NETINTERFACE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNetInterfaceName", "dwDesiredAccess", "lpdwGrantedAccess"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_NETWORK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNetworkName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_NETWORK_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNetworkName", "dwDesiredAccess", "lpdwGrantedAccess"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_NET_INTERFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszInterfaceName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNodeName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszNodeName", "dwDesiredAccess", "lpdwGrantedAccess"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszResourceName"]), offset=0),
    "PCLUSAPI_OPEN_CLUSTER_RESOURCE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "lpszResourceName", "dwDesiredAccess", "lpdwGrantedAccess"]), offset=0),
    "PCLUSAPI_OPEN_NODE_BY_ID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "nodeId"]), offset=0),
    "PCLUSAPI_PAUSE_CLUSTER_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode"]), offset=0),
    "PCLUSAPI_PAUSE_CLUSTER_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "bDrainNode", "dwPauseFlags", "hNodeDrainTarget"]), offset=0),
    "PCLUSAPI_PAUSE_CLUSTER_NODE_EX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "bDrainNode", "dwPauseFlags", "hNodeDrainTarget", "lpszReason"]), offset=0),
    "PCLUSAPI_PFN_REASON_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpParameter", "hCluster", "szReason", "lpSize"]), offset=0),
    "PCLUSAPI_REGISTER_CLUSTER_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hChange", "dwFilterType", "hObject", "dwNotifyKey"]), offset=0),
    "PCLUSAPI_REGISTER_CLUSTER_NOTIFY_V2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeRef("NOTIFY_FILTER_AND_TYPE", SimStruct), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hChange", "Filter", "hObject", "dwNotifyKey"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_GROUP_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hDependsOn"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_GROUP_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hDependsOn", "lpszReason"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_GROUP_GROUPSET_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "hDependsOn"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_GROUP_GROUPSET_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "hDependsOn", "lpszReason"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hDependsOn"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_GROUP_TO_GROUP_GROUPSET_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "hDependsOn", "lpszReason"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_NAME_ACCOUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_RESOURCE_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hDependsOn"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_RESOURCE_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hDependsOn", "lpszReason"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_RESOURCE_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hNode"]), offset=0),
    "PCLUSAPI_REMOVE_CLUSTER_RESOURCE_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "hNode", "lpszReason"]), offset=0),
    "PCLUSAPI_REMOVE_CROSS_CLUSTER_GROUPSET_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hDependentGroupSet", "lpRemoteClusterName", "lpRemoteGroupSetName"]), offset=0),
    "PCLUSAPI_REMOVE_RESOURCE_FROM_CLUSTER_SHARED_VOLUMES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource"]), offset=0),
    "PCLUSAPI_RESTART_CLUSTER_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "dwFlags"]), offset=0),
    "PCLUSAPI_RESTART_CLUSTER_RESOURCE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "dwFlags"]), offset=0),
    "PCLUSAPI_RESTORE_CLUSTER_DATABASE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszPathName", "bForce", "lpszQuorumDriveLetter"]), offset=0),
    "PCLUSAPI_RESUME_CLUSTER_NODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode"]), offset=0),
    "PCLUSAPI_RESUME_CLUSTER_NODE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CLUSTER_NODE_RESUME_FAILBACK_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "eResumeFailbackType", "dwResumeFlagsReserved"]), offset=0),
    "PCLUSAPI_RESUME_CLUSTER_NODE_EX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CLUSTER_NODE_RESUME_FAILBACK_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNode", "eResumeFailbackType", "dwResumeFlagsReserved", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_GROUP_GROUPSET_DEPENDENCY_EXPRESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "lpszDependencyExpression"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_GROUP_GROUPSET_DEPENDENCY_EXPRESSION_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "lpszDependencyExpression", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_GROUP_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "lpszGroupName"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_GROUP_NAME_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "lpszGroupName", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_GROUP_NODE_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "NodeCount", "NodeList"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_GROUP_NODE_LIST_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), label="LPArray", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "NodeCount", "NodeList", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_NAME_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszNewClusterName", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_NETWORK_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetwork", "lpszName"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_NETWORK_NAME_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNetwork", "lpszName", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_NETWORK_PRIORITY_ORDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "NetworkCount", "NetworkList"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_QUORUM_RESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszDeviceName", "dwMaxQuoLogSize"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_QUORUM_RESOURCE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszDeviceName", "dwMaxQuorumLogSize", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_RESOURCE_DEPENDENCY_EXPRESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszDependencyExpression"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_RESOURCE_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszResourceName"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_RESOURCE_NAME_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "lpszResourceName", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_CLUSTER_SERVICE_ACCOUNT_PASSWORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLUSTER_SET_PASSWORD_STATUS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszClusterName", "lpszNewPassword", "dwFlags", "lpReturnStatusBuffer", "lpcbReturnStatusBufferSize"]), offset=0),
    "PCLUSAPI_SET_GROUP_DEPENDENCY_EXPRESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroupSet", "lpszDependencyExpression"]), offset=0),
    "PCLUSAPI_SET_GROUP_DEPENDENCY_EXPRESSION_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hGroup", "lpszDependencyExpression", "lpszReason"]), offset=0),
    "PCLUSAPI_SET_REASON_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUSAPI_REASON_HANDLER", SimStruct), offset=0)], SimTypePointer(SimTypeRef("CLUSAPI_REASON_HANDLER", SimStruct), offset=0), arg_names=["lpHandler"]), offset=0),
    "PCLUSAPI_SHARED_VOLUME_SET_SNAPSHOT_STATE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SHARED_VOLUME_SNAPSHOT_STATE")], SimTypeInt(signed=False, label="UInt32"), arg_names=["guidSnapshotSet", "lpszVolumeName", "state"]), offset=0),
    "PCLUSAPI_SetClusterName": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "lpszNewClusterName"]), offset=0),
    "PCLUSTER_CLEAR_BACKUP_STATE_FOR_SHARED_VOLUME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszVolumePathName"]), offset=0),
    "PCLUSTER_DECRYPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusCryptProvider", "pCryptInput", "cbCryptInput", "ppCryptOutput", "pcbCryptOutput"]), offset=0),
    "PCLUSTER_ENCRYPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusCryptProvider", "pData", "cbData", "ppData", "pcbData"]), offset=0),
    "PCLUSTER_GET_VOLUME_NAME_FOR_VOLUME_MOUNT_POINT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszVolumeMountPoint", "lpszVolumeName", "cchBufferLength"]), offset=0),
    "PCLUSTER_GET_VOLUME_PATH_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszFileName", "lpszVolumePathName", "cchBufferLength"]), offset=0),
    "PCLUSTER_IS_PATH_ON_SHARED_VOLUME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszPathName"]), offset=0),
    "PCLUSTER_PREPARE_SHARED_VOLUME_FOR_BACKUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszFileName", "lpszVolumePathName", "lpcchVolumePathName", "lpszVolumeName", "lpcchVolumeName"]), offset=0),
    "PCLUSTER_REG_BATCH_ADD_COMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CLUSTER_REG_COMMAND"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegBatch", "dwCommand", "wzName", "dwOptions", "lpData", "cbData"]), offset=0),
    "PCLUSTER_REG_BATCH_CLOSE_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hBatchNotification"]), offset=0),
    "PCLUSTER_REG_BATCH_READ_COMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CLUSTER_BATCH_COMMAND", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hBatchNotification", "pBatchCommand"]), offset=0),
    "PCLUSTER_REG_CLOSE_BATCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegBatch", "bCommit", "failedCommandNumber"]), offset=0),
    "PCLUSTER_REG_CLOSE_BATCH_NOTIFY_PORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hBatchNotifyPort"]), offset=0),
    "PCLUSTER_REG_CLOSE_READ_BATCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegReadBatch", "phRegReadBatchReply"]), offset=0),
    "PCLUSTER_REG_CLOSE_READ_BATCH_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegReadBatch", "flags", "phRegReadBatchReply"]), offset=0),
    "PCLUSTER_REG_CLOSE_READ_BATCH_REPLY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegReadBatchReply"]), offset=0),
    "PCLUSTER_REG_CREATE_BATCH_NOTIFY_PORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "phBatchNotifyPort"]), offset=0),
    "PCLUSTER_REG_CREATE_READ_BATCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "phRegReadBatch"]), offset=0),
    "PCLUSTER_REG_GET_BATCH_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hBatchNotify", "phBatchNotification"]), offset=0),
    "PCLUSTER_REG_READ_BATCH_ADD_COMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegReadBatch", "wzSubkeyName", "wzValueName"]), offset=0),
    "PCLUSTER_REG_READ_BATCH_REPLY_NEXT_COMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CLUSTER_READ_BATCH_COMMAND", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegReadBatchReply", "pBatchCommand"]), offset=0),
    "PCLUSTER_SETUP_PROGRESS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_TYPE"), SimTypeInt(signed=False, label="CLUSTER_SETUP_PHASE_SEVERITY"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eSetupPhase", "ePhaseType", "ePhaseSeverity", "dwPercentComplete", "lpszObjectName", "dwStatus"]), offset=0),
    "PCLUSTER_SET_ACCOUNT_ACCESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "szAccountSID", "dwAccess", "dwControlType"]), offset=0),
    "PCLUSTER_UPGRADE_PROGRESS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CLUSTER_UPGRADE_PHASE")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackArg", "eUpgradePhase"]), offset=0),
    "PCMSCALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COLORMATCHSETUPA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PCMSCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COLORMATCHSETUPW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PCMWAVEFORMAT": SimStruct(OrderedDict((("wf", SimTypeRef("WAVEFORMAT", SimStruct)), ("wBitsPerSample", SimTypeShort(signed=False, label="UInt16")),)), name="PCMWAVEFORMAT", pack=False, align=None),
    "PCM_NOTIFY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CM_NOTIFY_ACTION"), SimTypePointer(SimTypeRef("CM_NOTIFY_EVENT_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hNotify", "Context", "Action", "EventData", "EventDataSize"]), offset=0),
    "PCOGETACTIVATIONSTATE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PCOGETCALLSTATE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PCOMPLETE_LOCK_IRP_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Irp"]), offset=0),
    "PCONFIGURE_ADAPTER_CHANNEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "FunctionNumber", "Context"]), offset=0),
    "PCOPYFILE2_PROGRESS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("COPYFILE2_MESSAGE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="COPYFILE2_MESSAGE_ACTION"), arg_names=["pMessage", "pvCallbackContext"]), offset=0),
    "PCRASHDUMP_POWER_ON": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PCREATE_COMMON_BUFFER_FROM_MDL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DMA_COMMON_BUFFER_EXTENDED_CONFIGURATION", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "ExtendedConfigs", "ExtendedConfigsCount", "LogicalAddress"]), offset=0),
    "PCREATE_PROCESS_NOTIFY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["ParentId", "ProcessId", "Create"]), offset=0),
    "PCREATE_PROCESS_NOTIFY_ROUTINE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("PS_CREATE_NOTIFY_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Process", "ProcessId", "CreateInfo"]), offset=0),
    "PCREATE_THREAD_NOTIFY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["ProcessId", "ThreadId", "Create"]), offset=0),
    "PCRYPT_DECRYPT_PRIVATE_KEY_FUNC": SimTypePointer(SimTypeFunction([SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Algorithm", "EncryptedPrivateKey", "pbClearTextKey", "pcbClearTextKey", "pVoidDecryptFunc"]), offset=0),
    "PCRYPT_ENCRYPT_PRIVATE_KEY_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAlgorithm", "pClearTextPrivateKey", "pbEncryptedKey", "pcbEncryptedKey", "pVoidEncryptFunc"]), offset=0),
    "PCRYPT_RESOLVE_HCRYPTPROV_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PRIVATE_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPrivateKeyInfo", "phCryptProv", "pVoidResolveFunc"]), offset=0),
    "PCS_RET": SimTypeInt(signed=False, label="PCS_RET"),
    "PCUSERIALIZEDPROPSTORAGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PCW_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="PCW_CALLBACK_TYPE"), SimTypePointer(SimUnion({"AddCounter": SimTypeRef("PCW_COUNTER_INFORMATION", SimStruct), "RemoveCounter": SimTypeRef("PCW_COUNTER_INFORMATION", SimStruct), "EnumerateInstances": SimTypeRef("PCW_MASK_INFORMATION", SimStruct), "CollectData": SimTypeRef("PCW_MASK_INFORMATION", SimStruct)}, name="<anon>", label="None"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Type", "Info", "Context"]), offset=0),
    "PCW_CALLBACK_INFORMATION": SimUnion({"AddCounter": SimTypeRef("PCW_COUNTER_INFORMATION", SimStruct), "RemoveCounter": SimTypeRef("PCW_COUNTER_INFORMATION", SimStruct), "EnumerateInstances": SimTypeRef("PCW_MASK_INFORMATION", SimStruct), "CollectData": SimTypeRef("PCW_MASK_INFORMATION", SimStruct)}, name="<anon>", label="None"),
    "PCW_CALLBACK_TYPE": SimTypeInt(signed=False, label="PCW_CALLBACK_TYPE"),
    "PCW_COUNTER_DESCRIPTOR": SimStruct(OrderedDict((("Id", SimTypeShort(signed=False, label="UInt16")), ("StructIndex", SimTypeShort(signed=False, label="UInt16")), ("Offset", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")),)), name="PCW_COUNTER_DESCRIPTOR", pack=False, align=None),
    "PCW_COUNTER_INFORMATION": SimStruct(OrderedDict((("CounterMask", SimTypeLongLong(signed=False, label="UInt64")), ("InstanceMask", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)),)), name="PCW_COUNTER_INFORMATION", pack=False, align=None),
    "PCW_DATA": SimStruct(OrderedDict((("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="PCW_DATA", pack=False, align=None),
    "PCW_MASK_INFORMATION": SimStruct(OrderedDict((("CounterMask", SimTypeLongLong(signed=False, label="UInt64")), ("InstanceMask", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("CollectMultiple", SimTypeChar(label="Byte")), ("Buffer", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CancelEvent", SimTypePointer(SimTypeRef("KEVENT", SimStruct), offset=0)),)), name="PCW_MASK_INFORMATION", pack=False, align=None),
    "PCW_REGISTRATION_FLAGS": SimTypeInt(signed=False, label="PCW_REGISTRATION_FLAGS"),
    "PCW_REGISTRATION_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("CounterCount", SimTypeInt(signed=False, label="UInt32")), ("Counters", SimTypePointer(SimTypeRef("PCW_COUNTER_DESCRIPTOR", SimStruct), offset=0)), ("Callback", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("CallbackContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="PCW_REGISTRATION_FLAGS")),)), name="PCW_REGISTRATION_INFORMATION", pack=False, align=None),
    "PCreateTextServices": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="ITextHost"), SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["punkOuter", "pITextHost", "ppUnk"]), offset=0),
    "PD2D1_EFFECT_FACTORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["effectImpl"]), offset=0),
    "PD2D1_PROPERTY_GET_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["effect", "data", "dataSize", "actualSize"]), offset=0),
    "PD2D1_PROPERTY_SET_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["effect", "data", "dataSize"]), offset=0),
    "PD3COLD_REQUEST_AUX_POWER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PD3COLD_REQUEST_CORE_POWER_RAIL": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PD3COLD_REQUEST_PERST_DELAY": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PDBGHELP_CREATE_USER_DUMP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DataType", "Data", "DataLength", "UserData"]), offset=0),
    "PDD_ALPHABLT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_CANCREATESURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CANCREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_COLORCB_COLORCONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_COLORCONTROLDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_CREATEPALETTE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATEPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_CREATESURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATESURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_CREATESURFACEEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATESURFACEEXDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_DESTROYDDLOCAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYDDLOCALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_DESTROYDRIVER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_DESTROYDRIVERDATA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PDD_FLIPTOGDISURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FLIPTOGDISURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_FREEDRIVERMEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FREEDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_GETAVAILDRIVERMEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETAVAILDRIVERMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_GETDRIVERINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETDRIVERINFODATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_GETDRIVERSTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETDRIVERSTATEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_GETSCANLINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETSCANLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_GETVPORTAUTOFLIPSURFACEDATA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PDD_KERNELCB_SYNCSURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SYNCSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_KERNELCB_SYNCVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SYNCVIDEOPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MAPMEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_MAPMEMORYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_BEGINFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_BEGINMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_CREATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATEMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_DESTROY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_ENDFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_ENDMOCOMPFRAMEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_GETCOMPBUFFINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETMOCOMPCOMPBUFFDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_GETFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETMOCOMPFORMATSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_GETGUIDS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETMOCOMPGUIDSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_GETINTERNALINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETINTERNALMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_QUERYSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_QUERYMOCOMPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_MOCOMPCB_RENDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_RENDERMOCOMPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_PALCB_DESTROYPALETTE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_PALCB_SETENTRIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETENTRIESDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SETCOLORKEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DRVSETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SETEXCLUSIVEMODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETEXCLUSIVEMODEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SETMODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SETMODEDATA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PDD_SURFCB_ADDATTACHEDSURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_ADDATTACHEDSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_BLT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_BLTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_DESTROYSURFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYSURFACEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_FLIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FLIPDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_GETBLTSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETBLTSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_GETFLIPSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_LOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_SETCLIPLIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETCLIPLISTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_SETCOLORKEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETCOLORKEYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_SETOVERLAYPOSITION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETOVERLAYPOSITIONDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_SETPALETTE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_SETPALETTEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_UNLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UNLOCKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_SURFCB_UPDATEOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UPDATEOVERLAYDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_CANCREATEVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CANCREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_COLORCONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_VPORTCOLORDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_CREATEVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_CREATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_DESTROYVPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DESTROYVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_FLIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_FLIPVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETAUTOFLIPSURF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETBANDWIDTH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTBANDWIDTHDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETFIELD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTFIELDDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETFLIPSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTFLIPSTATUSDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETINPUTFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTINPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETLINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTLINEDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETOUTPUTFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTOUTPUTFORMATDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETSIGNALSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTSIGNALDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_GETVPORTCONNECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_GETVPORTCONNECTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_UPDATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_UPDATEVPORTDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_VPORTCB_WAITFORSYNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_WAITFORVPORTSYNCDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDD_WAITFORVERTICALBLANK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_WAITFORVERTICALBLANKDATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PDEBUG_DEVICE_FOUND_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=False, label="KD_CALLBACK_ACTION"), arg_names=["Device"]), offset=0),
    "PDEBUG_EXTENSION_CALL": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Args"]), offset=0),
    "PDEBUG_EXTENSION_CANUNLOAD": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PDEBUG_EXTENSION_INITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Version", "Flags"]), offset=0),
    "PDEBUG_EXTENSION_KNOWN_STRUCT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Flags", "Offset", "TypeName", "Buffer", "BufferChars"]), offset=0),
    "PDEBUG_EXTENSION_KNOWN_STRUCT_EX": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Flags", "Offset", "TypeName", "Buffer", "BufferChars"]), offset=0),
    "PDEBUG_EXTENSION_NOTIFY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeBottom(label="Void"), arg_names=["Notify", "Argument"]), offset=0),
    "PDEBUG_EXTENSION_PROVIDE_VALUE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Flags", "Name", "Value", "TypeModBase", "TypeId", "TypeFlags"]), offset=0),
    "PDEBUG_EXTENSION_QUERY_VALUE_NAMES": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Flags", "Buffer", "BufferChars", "BufferNeeded"]), offset=0),
    "PDEBUG_EXTENSION_UNINITIALIZE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PDEBUG_EXTENSION_UNLOAD": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PDEBUG_PRINT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Output", "ComponentId", "Level"]), offset=0),
    "PDEBUG_STACK_PROVIDER_BEGINTHREADSTACKRECONSTRUCTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["StreamType", "MiniDumpStreamBuffer", "BufferSize"]), offset=0),
    "PDEBUG_STACK_PROVIDER_ENDTHREADSTACKRECONSTRUCTION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PDEBUG_STACK_PROVIDER_FREESTACKSYMFRAMES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("STACK_SYM_FRAME_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["StackSymFrames"]), offset=0),
    "PDEBUG_STACK_PROVIDER_RECONSTRUCTSTACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEBUG_STACK_FRAME_EX", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("STACK_SYM_FRAME_INFO", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SystemThreadId", "NativeFrames", "CountNativeFrames", "StackSymFrames", "StackSymFramesFilled"]), offset=0),
    "PDELAYLOAD_FAILURE_DLL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DELAYLOAD_INFO", SimStruct), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["NotificationReason", "DelayloadInfo"]), offset=0),
    "PDETECT_PROGRESS_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ProgressNotifyParam", "DetectComplete"]), offset=0),
    "PDEVICE_BUS_SPECIFIC_RESET_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PDEVICE_CHANGE_COMPLETE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PDEVICE_HANDLER_OBJECT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PDEVICE_NOTIFY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PDEVICE_NOTIFY_CALLBACK2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["NotificationContext", "NotifyCode"]), offset=0),
    "PDEVICE_NOTIFY_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Type", "Setting"]), offset=0),
    "PDEVICE_QUERY_BUS_SPECIFIC_RESET_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PDEVICE_RESET_COMPLETION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PDEVICE_RESET_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PDEV_QUERY_RESULT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEV_QUERY_RESULT_ACTION_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["hDevQuery", "pContext", "pActionData"]), offset=0),
    "PDH_BROWSE_DLG_CONFIG_A": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szDataSource", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szReturnPathBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchReturnPathLength", SimTypeInt(signed=False, label="UInt32")), ("pCallBack", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("dwCallBackArg", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CallBackStatus", SimTypeInt(signed=True, label="Int32")), ("dwDefaultDetailLevel", SimTypeInt(signed=False, label="PERF_DETAIL")), ("szDialogBoxCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PDH_BROWSE_DLG_CONFIG_A", pack=False, align=None),
    "PDH_BROWSE_DLG_CONFIG_HA": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDataSource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szReturnPathBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchReturnPathLength", SimTypeInt(signed=False, label="UInt32")), ("pCallBack", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("dwCallBackArg", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CallBackStatus", SimTypeInt(signed=True, label="Int32")), ("dwDefaultDetailLevel", SimTypeInt(signed=False, label="PERF_DETAIL")), ("szDialogBoxCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PDH_BROWSE_DLG_CONFIG_HA", pack=False, align=None),
    "PDH_BROWSE_DLG_CONFIG_HW": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDataSource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szReturnPathBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchReturnPathLength", SimTypeInt(signed=False, label="UInt32")), ("pCallBack", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("dwCallBackArg", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CallBackStatus", SimTypeInt(signed=True, label="Int32")), ("dwDefaultDetailLevel", SimTypeInt(signed=False, label="PERF_DETAIL")), ("szDialogBoxCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PDH_BROWSE_DLG_CONFIG_HW", pack=False, align=None),
    "PDH_BROWSE_DLG_CONFIG_W": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("hWndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szDataSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szReturnPathBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchReturnPathLength", SimTypeInt(signed=False, label="UInt32")), ("pCallBack", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("dwCallBackArg", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CallBackStatus", SimTypeInt(signed=True, label="Int32")), ("dwDefaultDetailLevel", SimTypeInt(signed=False, label="PERF_DETAIL")), ("szDialogBoxCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PDH_BROWSE_DLG_CONFIG_W", pack=False, align=None),
    "PDH_COUNTER_INFO_A": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("CVersion", SimTypeInt(signed=False, label="UInt32")), ("CStatus", SimTypeInt(signed=False, label="UInt32")), ("lScale", SimTypeInt(signed=True, label="Int32")), ("lDefaultScale", SimTypeInt(signed=True, label="Int32")), ("dwUserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwQueryUserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szFullPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Anonymous", SimUnion({"DataItemPath": SimTypeRef("PDH_DATA_ITEM_PATH_ELEMENTS_A", SimStruct), "CounterPath": SimTypeRef("PDH_COUNTER_PATH_ELEMENTS_A", SimStruct), "Anonymous": SimStruct(OrderedDict((("szMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szInstanceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szParentInstance", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwInstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("szCounterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("szExplainText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DataBuffer", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PDH_COUNTER_INFO_A", pack=False, align=None),
    "PDH_COUNTER_INFO_W": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("CVersion", SimTypeInt(signed=False, label="UInt32")), ("CStatus", SimTypeInt(signed=False, label="UInt32")), ("lScale", SimTypeInt(signed=True, label="Int32")), ("lDefaultScale", SimTypeInt(signed=True, label="Int32")), ("dwUserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwQueryUserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("szFullPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"DataItemPath": SimTypeRef("PDH_DATA_ITEM_PATH_ELEMENTS_W", SimStruct), "CounterPath": SimTypeRef("PDH_COUNTER_PATH_ELEMENTS_W", SimStruct), "Anonymous": SimStruct(OrderedDict((("szMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szParentInstance", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwInstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("szCounterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("szExplainText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DataBuffer", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PDH_COUNTER_INFO_W", pack=False, align=None),
    "PDH_COUNTER_PATH_ELEMENTS_A": SimStruct(OrderedDict((("szMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szObjectName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szInstanceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szParentInstance", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwInstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("szCounterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PDH_COUNTER_PATH_ELEMENTS_A", pack=False, align=None),
    "PDH_COUNTER_PATH_ELEMENTS_W": SimStruct(OrderedDict((("szMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szParentInstance", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwInstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("szCounterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PDH_COUNTER_PATH_ELEMENTS_W", pack=False, align=None),
    "PDH_DATA_ITEM_PATH_ELEMENTS_A": SimStruct(OrderedDict((("szMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ObjectGUID", SimTypeBottom(label="Guid")), ("dwItemId", SimTypeInt(signed=False, label="UInt32")), ("szInstanceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PDH_DATA_ITEM_PATH_ELEMENTS_A", pack=False, align=None),
    "PDH_DATA_ITEM_PATH_ELEMENTS_W": SimStruct(OrderedDict((("szMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ObjectGUID", SimTypeBottom(label="Guid")), ("dwItemId", SimTypeInt(signed=False, label="UInt32")), ("szInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PDH_DATA_ITEM_PATH_ELEMENTS_W", pack=False, align=None),
    "PDH_DLL_VERSION": SimTypeInt(signed=False, label="PDH_DLL_VERSION"),
    "PDH_FMT": SimTypeInt(signed=False, label="PDH_FMT"),
    "PDH_FMT_COUNTERVALUE": SimStruct(OrderedDict((("CStatus", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"longValue": SimTypeInt(signed=True, label="Int32"), "doubleValue": SimTypeFloat(size=64), "largeValue": SimTypeLongLong(signed=True, label="Int64"), "AnsiStringValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "WideStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="PDH_FMT_COUNTERVALUE", pack=False, align=None),
    "PDH_FMT_COUNTERVALUE_ITEM_A": SimStruct(OrderedDict((("szName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FmtValue", SimTypeRef("PDH_FMT_COUNTERVALUE", SimStruct)),)), name="PDH_FMT_COUNTERVALUE_ITEM_A", pack=False, align=None),
    "PDH_FMT_COUNTERVALUE_ITEM_W": SimStruct(OrderedDict((("szName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FmtValue", SimTypeRef("PDH_FMT_COUNTERVALUE", SimStruct)),)), name="PDH_FMT_COUNTERVALUE_ITEM_W", pack=False, align=None),
    "PDH_LOG": SimTypeInt(signed=False, label="PDH_LOG"),
    "PDH_LOG_SERVICE_QUERY_INFO_A": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLogQuota", SimTypeInt(signed=False, label="UInt32")), ("szLogFileCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szDefaultDir", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("szBaseFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwFileType", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("PdlAutoNameInterval", SimTypeInt(signed=False, label="UInt32")), ("PdlAutoNameUnits", SimTypeInt(signed=False, label="UInt32")), ("PdlCommandFilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PdlCounterList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PdlAutoNameFormat", SimTypeInt(signed=False, label="UInt32")), ("PdlSampleInterval", SimTypeInt(signed=False, label="UInt32")), ("PdlLogStartTime", SimTypeRef("FILETIME", SimStruct)), ("PdlLogEndTime", SimTypeRef("FILETIME", SimStruct)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("TlNumberOfBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlMinimumBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlMaximumBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlFreeBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlBufferSize", SimTypeInt(signed=False, label="UInt32")), ("TlEventsLost", SimTypeInt(signed=False, label="UInt32")), ("TlLoggerThreadId", SimTypeInt(signed=False, label="UInt32")), ("TlBuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("TlLogHandle", SimTypeInt(signed=False, label="UInt32")), ("TlLogFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PDH_LOG_SERVICE_QUERY_INFO_A", pack=False, align=None),
    "PDH_LOG_SERVICE_QUERY_INFO_W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLogQuota", SimTypeInt(signed=False, label="UInt32")), ("szLogFileCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szDefaultDir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szBaseFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFileType", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("PdlAutoNameInterval", SimTypeInt(signed=False, label="UInt32")), ("PdlAutoNameUnits", SimTypeInt(signed=False, label="UInt32")), ("PdlCommandFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PdlCounterList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PdlAutoNameFormat", SimTypeInt(signed=False, label="UInt32")), ("PdlSampleInterval", SimTypeInt(signed=False, label="UInt32")), ("PdlLogStartTime", SimTypeRef("FILETIME", SimStruct)), ("PdlLogEndTime", SimTypeRef("FILETIME", SimStruct)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("TlNumberOfBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlMinimumBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlMaximumBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlFreeBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlBufferSize", SimTypeInt(signed=False, label="UInt32")), ("TlEventsLost", SimTypeInt(signed=False, label="UInt32")), ("TlLoggerThreadId", SimTypeInt(signed=False, label="UInt32")), ("TlBuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("TlLogHandle", SimTypeInt(signed=False, label="UInt32")), ("TlLogFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PDH_LOG_SERVICE_QUERY_INFO_W", pack=False, align=None),
    "PDH_LOG_TYPE": SimTypeInt(signed=False, label="PDH_LOG_TYPE"),
    "PDH_PATH_FLAGS": SimTypeInt(signed=False, label="PDH_PATH_FLAGS"),
    "PDH_RAW_COUNTER": SimStruct(OrderedDict((("CStatus", SimTypeInt(signed=False, label="UInt32")), ("TimeStamp", SimTypeRef("FILETIME", SimStruct)), ("FirstValue", SimTypeLongLong(signed=True, label="Int64")), ("SecondValue", SimTypeLongLong(signed=True, label="Int64")), ("MultiCount", SimTypeInt(signed=False, label="UInt32")),)), name="PDH_RAW_COUNTER", pack=False, align=None),
    "PDH_RAW_COUNTER_ITEM_A": SimStruct(OrderedDict((("szName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RawValue", SimTypeRef("PDH_RAW_COUNTER", SimStruct)),)), name="PDH_RAW_COUNTER_ITEM_A", pack=False, align=None),
    "PDH_RAW_COUNTER_ITEM_W": SimStruct(OrderedDict((("szName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RawValue", SimTypeRef("PDH_RAW_COUNTER", SimStruct)),)), name="PDH_RAW_COUNTER_ITEM_W", pack=False, align=None),
    "PDH_RAW_LOG_RECORD": SimStruct(OrderedDict((("dwStructureSize", SimTypeInt(signed=False, label="UInt32")), ("dwRecordType", SimTypeInt(signed=False, label="PDH_LOG_TYPE")), ("dwItems", SimTypeInt(signed=False, label="UInt32")), ("RawBytes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PDH_RAW_LOG_RECORD", pack=False, align=None),
    "PDH_SELECT_DATA_SOURCE_FLAGS": SimTypeInt(signed=False, label="PDH_SELECT_DATA_SOURCE_FLAGS"),
    "PDH_STATISTICS": SimStruct(OrderedDict((("dwFormat", SimTypeInt(signed=False, label="UInt32")), ("count", SimTypeInt(signed=False, label="UInt32")), ("min", SimTypeRef("PDH_FMT_COUNTERVALUE", SimStruct)), ("max", SimTypeRef("PDH_FMT_COUNTERVALUE", SimStruct)), ("mean", SimTypeRef("PDH_FMT_COUNTERVALUE", SimStruct)),)), name="PDH_STATISTICS", pack=False, align=None),
    "PDH_TIME_INFO": SimStruct(OrderedDict((("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("SampleCount", SimTypeInt(signed=False, label="UInt32")),)), name="PDH_TIME_INFO", pack=False, align=None),
    "PDIRTY_PAGE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["FileObject", "FileOffset", "Length", "OldestLsn", "NewestLsn", "Context1", "Context2"]), offset=0),
    "PDMA_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PDNS_QUERY_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pQueryContext", "pQueryResults"]), offset=0),
    "PDNS_SERVICE_BROWSE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_RECORDW", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "pQueryContext", "pDnsRecord"]), offset=0),
    "PDNS_SERVICE_REGISTER_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_SERVICE_INSTANCE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "pQueryContext", "pInstance"]), offset=0),
    "PDNS_SERVICE_RESOLVE_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DNS_SERVICE_INSTANCE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "pQueryContext", "pInstance"]), offset=0),
    "PDOPSTATUS": SimTypeInt(signed=False, label="PDOPSTATUS"),
    "PDRIVER_CMC_EXCEPTION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MCA_EXCEPTION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "CmcLog"]), offset=0),
    "PDRIVER_CPE_EXCEPTION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MCA_EXCEPTION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "CmcLog"]), offset=0),
    "PDRIVER_EXCPTN_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MCA_EXCEPTION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "BankLog"]), offset=0),
    "PDRIVER_VERIFIER_THUNK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["Context"]), offset=0),
    "PDUMP_DEVICE_POWERON_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PDXGK_FSTATE_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "ComponentIndex", "NewFState", "PreNotification", "PrivateHandle"]), offset=0),
    "PDXGK_GRAPHICSPOWER_UNREGISTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceHandle", "PrivateHandle"]), offset=0),
    "PDXGK_INITIAL_COMPONENT_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="Guid"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "PrivateHandle", "ComponentIndex", "IsBlockingType", "InitialFState", "ComponentGuid", "PowerComponentMappingFlag"]), offset=0),
    "PDXGK_POWER_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="DEVICE_POWER_STATE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "NewGrfxPowerState", "PreNotification", "PrivateHandle"]), offset=0),
    "PDXGK_REMOVAL_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["GraphicsDeviceHandle", "PrivateHandle"]), offset=0),
    "PDXGK_SET_SHARED_POWER_COMPONENT_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceHandle", "PrivateHandle", "ComponentIndex", "Active"]), offset=0),
    "PDXVA2SW_CREATEVIDEOPROCESSDEVICE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirect3DDevice9"), SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pD3DD9", "pVideoDesc", "RenderTargetFormat", "MaxSubStreams", "phDevice"]), offset=0),
    "PDXVA2SW_DESTROYVIDEOPROCESSDEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice"]), offset=0),
    "PDXVA2SW_GETFILTERPROPERTYRANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVA2_ValueRange", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "FilterSetting", "pRange"]), offset=0),
    "PDXVA2SW_GETPROCAMPRANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVA2_ValueRange", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "ProcAmpCap", "pRange"]), offset=0),
    "PDXVA2SW_GETVIDEOPROCESSORCAPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypePointer(SimTypeRef("DXVA2_VideoProcessorCaps", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "pCaps"]), offset=0),
    "PDXVA2SW_GETVIDEOPROCESSORRENDERTARGETCOUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "pCount"]), offset=0),
    "PDXVA2SW_GETVIDEOPROCESSORRENDERTARGETS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "Count", "pFormats"]), offset=0),
    "PDXVA2SW_GETVIDEOPROCESSORSUBSTREAMFORMATCOUNT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "pCount"]), offset=0),
    "PDXVA2SW_GETVIDEOPROCESSORSUBSTREAMFORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DXVA2_VideoDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="D3DFORMAT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVideoDesc", "RenderTargetFormat", "Count", "pFormats"]), offset=0),
    "PDXVA2SW_VIDEOPROCESSBEGINFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice"]), offset=0),
    "PDXVA2SW_VIDEOPROCESSBLT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVA2_VIDEOPROCESSBLT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pBlt"]), offset=0),
    "PDXVA2SW_VIDEOPROCESSENDFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pHandleComplete"]), offset=0),
    "PDXVA2SW_VIDEOPROCESSSETRENDERTARGET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IDirect3DSurface9")], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pRenderTarget"]), offset=0),
    "PDXVAHDSW_CreateDevice": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirect3DDevice9Ex"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pD3DDevice", "phDevice"]), offset=0),
    "PDXVAHDSW_CreateVideoProcessor": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pVPGuid", "phVideoProcessor"]), offset=0),
    "PDXVAHDSW_DestroyDevice": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice"]), offset=0),
    "PDXVAHDSW_DestroyVideoProcessor": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor"]), offset=0),
    "PDXVAHDSW_GetVideoProcessBltStatePrivate": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_BLT_STATE_PRIVATE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "pData"]), offset=0),
    "PDXVAHDSW_GetVideoProcessStreamStatePrivate": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_STREAM_STATE_PRIVATE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "StreamNumber", "pData"]), offset=0),
    "PDXVAHDSW_GetVideoProcessorCaps": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_VPCAPS", SimStruct), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "Count", "pCaps"]), offset=0),
    "PDXVAHDSW_GetVideoProcessorCustomRates": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_CUSTOM_RATE_DATA", SimStruct), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pVPGuid", "Count", "pRates"]), offset=0),
    "PDXVAHDSW_GetVideoProcessorDeviceCaps": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypePointer(SimTypeRef("DXVAHD_VPDEVCAPS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "pCaps"]), offset=0),
    "PDXVAHDSW_GetVideoProcessorFilterRange": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DXVAHD_FILTER"), SimTypePointer(SimTypeRef("DXVAHD_FILTER_RANGE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "Filter", "pRange"]), offset=0),
    "PDXVAHDSW_GetVideoProcessorInputFormats": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "Count", "pFormats"]), offset=0),
    "PDXVAHDSW_GetVideoProcessorOutputFormats": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pContentDesc", "Usage", "Count", "pFormats"]), offset=0),
    "PDXVAHDSW_Plugin": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Size", "pCallbacks"]), offset=0),
    "PDXVAHDSW_ProposeVideoPrivateFormat": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3DFORMAT"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDevice", "pFormat"]), offset=0),
    "PDXVAHDSW_SetVideoProcessBltState": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="DXVAHD_BLT_STATE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "State", "DataSize", "pData"]), offset=0),
    "PDXVAHDSW_SetVideoProcessStreamState": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="DXVAHD_STREAM_STATE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "StreamNumber", "State", "DataSize", "pData"]), offset=0),
    "PDXVAHDSW_VideoProcessBltHD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IDirect3DSurface9"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXVAHD_STREAM_DATA", SimStruct), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hVideoProcessor", "pOutputSurface", "OutputFrame", "StreamCount", "pStreams"]), offset=0),
    "PDXVAHD_CreateDevice": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDirect3DDevice9Ex"), SimTypePointer(SimTypeRef("DXVAHD_CONTENT_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="DXVAHD_DEVICE_USAGE"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Size", "pCallbacks"]), offset=0), SimTypePointer(SimTypeBottom(label="IDXVAHD_Device"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pD3DDevice", "pContentDesc", "Usage", "pPlugin", "ppDevice"]), offset=0),
    "PDX_BOBNEXTFIELD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDBOBNEXTFIELDINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_ENABLEIRQ": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDENABLEIRQINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_FLIPOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDFLIPOVERLAYINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_FLIPVIDEOPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDFLIPVIDEOPORTINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_GETCURRENTAUTOFLIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETCURRENTAUTOFLIPININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDGETCURRENTAUTOFLIPOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_GETIRQINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETIRQINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_GETPOLARITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETPOLARITYININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDGETPOLARITYOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_GETPREVIOUSAUTOFLIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETPREVIOUSAUTOFLIPININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDGETPREVIOUSAUTOFLIPOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_GETTRANSFERSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDGETTRANSFERSTATUSOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_IRQCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DX_IRQDATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pIrqData"]), offset=0),
    "PDX_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDLOCKININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDLOCKOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_SETSTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDSETSTATEININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDSETSTATEOUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_SKIPNEXTFIELD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDSKIPNEXTFIELDINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PDX_TRANSFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DDTRANSFERININFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("DDTRANSFEROUTINFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PEB": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("BeingDebugged", SimTypeChar(label="Byte")), ("Reserved2", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Reserved3", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("Ldr", SimTypePointer(SimTypeRef("PEB_LDR_DATA", SimStruct), offset=0)), ("ProcessParameters", SimTypePointer(SimTypeRef("RTL_USER_PROCESS_PARAMETERS", SimStruct), offset=0)), ("Reserved4", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 3)), ("AtlThunkSListPtr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved5", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved6", SimTypeInt(signed=False, label="UInt32")), ("Reserved7", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved8", SimTypeInt(signed=False, label="UInt32")), ("AtlThunkSListPtr32", SimTypeInt(signed=False, label="UInt32")), ("Reserved9", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 45)), ("Reserved10", SimTypeArray(SimTypeChar(label="Byte"), 96)), ("PostProcessInitRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Reserved11", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("Reserved12", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("SessionId", SimTypeInt(signed=False, label="UInt32")),)), name="PEB", pack=False, align=None),
    "PEB_LDR_DATA": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Reserved2", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 3)), ("InMemoryOrderModuleList", SimTypeRef("LIST_ENTRY", SimStruct)),)), name="PEB_LDR_DATA", pack=False, align=None),
    "PEDOMETER_STEP_TYPE": SimTypeInt(signed=False, label="PEDOMETER_STEP_TYPE"),
    "PEDOMETER_STEP_TYPE_COUNT": SimTypeInt(signed=False, label="PEDOMETER_STEP_TYPE_COUNT"),
    "PEEK_MESSAGE_REMOVE_TYPE": SimTypeInt(signed=False, label="PEEK_MESSAGE_REMOVE_TYPE"),
    "PEERDIST_CLIENT_BASIC_INFO": SimStruct(OrderedDict((("fFlashCrowd", SimTypeInt(signed=True, label="Int32")),)), name="PEERDIST_CLIENT_BASIC_INFO", pack=False, align=None),
    "PEERDIST_CLIENT_INFO_BY_HANDLE_CLASS": SimTypeInt(signed=False, label="PEERDIST_CLIENT_INFO_BY_HANDLE_CLASS"),
    "PEERDIST_CONTENT_TAG": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="PEERDIST_CONTENT_TAG", pack=False, align=None),
    "PEERDIST_PUBLICATION_OPTIONS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="PEERDIST_PUBLICATION_OPTIONS", pack=False, align=None),
    "PEERDIST_RETRIEVAL_OPTIONS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwContentInfoMinVersion", SimTypeInt(signed=False, label="UInt32")), ("dwContentInfoMaxVersion", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="PEERDIST_RETRIEVAL_OPTIONS", pack=False, align=None),
    "PEERDIST_RETRIEVAL_OPTIONS_CONTENTINFO_VERSION_VALUE": SimTypeInt(signed=False, label="PEERDIST_RETRIEVAL_OPTIONS_CONTENTINFO_VERSION_VALUE"),
    "PEERDIST_STATUS": SimTypeInt(signed=False, label="PEERDIST_STATUS"),
    "PEERDIST_STATUS_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="PEERDIST_STATUS")), ("dwMinVer", SimTypeInt(signed=False, label="PEERDIST_RETRIEVAL_OPTIONS_CONTENTINFO_VERSION_VALUE")), ("dwMaxVer", SimTypeInt(signed=False, label="PEERDIST_RETRIEVAL_OPTIONS_CONTENTINFO_VERSION_VALUE")),)), name="PEERDIST_STATUS_INFO", pack=False, align=None),
    "PEER_ADDRESS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("sin6", SimTypeRef("SOCKADDR_IN6", SimStruct)),)), name="PEER_ADDRESS", pack=False, align=None),
    "PEER_APPLICATION": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("data", SimTypeRef("PEER_DATA", SimStruct)), ("pwzDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_APPLICATION", pack=False, align=None),
    "PEER_APPLICATION_REGISTRATION_INFO": SimStruct(OrderedDict((("application", SimTypeRef("PEER_APPLICATION", SimStruct)), ("pwzApplicationToLaunch", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzApplicationArguments", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPublicationScope", SimTypeInt(signed=False, label="UInt32")),)), name="PEER_APPLICATION_REGISTRATION_INFO", pack=False, align=None),
    "PEER_APPLICATION_REGISTRATION_TYPE": SimTypeInt(signed=False, label="PEER_APPLICATION_REGISTRATION_TYPE"),
    "PEER_APP_LAUNCH_INFO": SimStruct(OrderedDict((("pContact", SimTypePointer(SimTypeRef("PEER_CONTACT", SimStruct), offset=0)), ("pEndpoint", SimTypePointer(SimTypeRef("PEER_ENDPOINT", SimStruct), offset=0)), ("pInvitation", SimTypePointer(SimTypeRef("PEER_INVITATION", SimStruct), offset=0)),)), name="PEER_APP_LAUNCH_INFO", pack=False, align=None),
    "PEER_CHANGE_TYPE": SimTypeInt(signed=False, label="PEER_CHANGE_TYPE"),
    "PEER_COLLAB_EVENT_DATA": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="PEER_COLLAB_EVENT_TYPE")), ("Anonymous", SimUnion({"watchListChangedData": SimTypeRef("PEER_EVENT_WATCHLIST_CHANGED_DATA", SimStruct), "presenceChangedData": SimTypeRef("PEER_EVENT_PRESENCE_CHANGED_DATA", SimStruct), "applicationChangedData": SimTypeRef("PEER_EVENT_APPLICATION_CHANGED_DATA", SimStruct), "objectChangedData": SimTypeRef("PEER_EVENT_OBJECT_CHANGED_DATA", SimStruct), "endpointChangedData": SimTypeRef("PEER_EVENT_ENDPOINT_CHANGED_DATA", SimStruct), "peopleNearMeChangedData": SimTypeRef("PEER_EVENT_PEOPLE_NEAR_ME_CHANGED_DATA", SimStruct), "requestStatusChangedData": SimTypeRef("PEER_EVENT_REQUEST_STATUS_CHANGED_DATA", SimStruct)}, name="<anon>", label="None")),)), name="PEER_COLLAB_EVENT_DATA", pack=False, align=None),
    "PEER_COLLAB_EVENT_REGISTRATION": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="PEER_COLLAB_EVENT_TYPE")), ("pInstance", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="PEER_COLLAB_EVENT_REGISTRATION", pack=False, align=None),
    "PEER_COLLAB_EVENT_TYPE": SimTypeInt(signed=False, label="PEER_COLLAB_EVENT_TYPE"),
    "PEER_CONNECTION_FLAGS": SimTypeInt(signed=False, label="PEER_CONNECTION_FLAGS"),
    "PEER_CONNECTION_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ullConnectionId", SimTypeLongLong(signed=False, label="UInt64")), ("ullNodeId", SimTypeLongLong(signed=False, label="UInt64")), ("pwzPeerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("address", SimTypeRef("PEER_ADDRESS", SimStruct)),)), name="PEER_CONNECTION_INFO", pack=False, align=None),
    "PEER_CONNECTION_STATUS": SimTypeInt(signed=False, label="PEER_CONNECTION_STATUS"),
    "PEER_CONTACT": SimStruct(OrderedDict((("pwzPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzNickName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzEmailAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fWatch", SimTypeInt(signed=True, label="Int32")), ("WatcherPermissions", SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION")), ("credentials", SimTypeRef("PEER_DATA", SimStruct)),)), name="PEER_CONTACT", pack=False, align=None),
    "PEER_CREDENTIAL_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwzFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPublicKey", SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0)), ("pwzIssuerPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzIssuerFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftValidityStart", SimTypeRef("FILETIME", SimStruct)), ("ftValidityEnd", SimTypeRef("FILETIME", SimStruct)), ("cRoles", SimTypeInt(signed=False, label="UInt32")), ("pRoles", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="PEER_CREDENTIAL_INFO", pack=False, align=None),
    "PEER_DATA": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PEER_DATA", pack=False, align=None),
    "PEER_ENDPOINT": SimStruct(OrderedDict((("address", SimTypeRef("PEER_ADDRESS", SimStruct)), ("pwzEndpointName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_ENDPOINT", pack=False, align=None),
    "PEER_EVENT_APPLICATION_CHANGED_DATA": SimStruct(OrderedDict((("pContact", SimTypePointer(SimTypeRef("PEER_CONTACT", SimStruct), offset=0)), ("pEndpoint", SimTypePointer(SimTypeRef("PEER_ENDPOINT", SimStruct), offset=0)), ("changeType", SimTypeInt(signed=False, label="PEER_CHANGE_TYPE")), ("pApplication", SimTypePointer(SimTypeRef("PEER_APPLICATION", SimStruct), offset=0)),)), name="PEER_EVENT_APPLICATION_CHANGED_DATA", pack=False, align=None),
    "PEER_EVENT_CONNECTION_CHANGE_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="PEER_CONNECTION_STATUS")), ("ullConnectionId", SimTypeLongLong(signed=False, label="UInt64")), ("ullNodeId", SimTypeLongLong(signed=False, label="UInt64")), ("ullNextConnectionId", SimTypeLongLong(signed=False, label="UInt64")), ("hrConnectionFailedReason", SimTypeInt(signed=True, label="Int32")),)), name="PEER_EVENT_CONNECTION_CHANGE_DATA", pack=False, align=None),
    "PEER_EVENT_ENDPOINT_CHANGED_DATA": SimStruct(OrderedDict((("pContact", SimTypePointer(SimTypeRef("PEER_CONTACT", SimStruct), offset=0)), ("pEndpoint", SimTypePointer(SimTypeRef("PEER_ENDPOINT", SimStruct), offset=0)),)), name="PEER_EVENT_ENDPOINT_CHANGED_DATA", pack=False, align=None),
    "PEER_EVENT_INCOMING_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ullConnectionId", SimTypeLongLong(signed=False, label="UInt64")), ("type", SimTypeBottom(label="Guid")), ("data", SimTypeRef("PEER_DATA", SimStruct)),)), name="PEER_EVENT_INCOMING_DATA", pack=False, align=None),
    "PEER_EVENT_MEMBER_CHANGE_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("changeType", SimTypeInt(signed=False, label="PEER_MEMBER_CHANGE_TYPE")), ("pwzIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_EVENT_MEMBER_CHANGE_DATA", pack=False, align=None),
    "PEER_EVENT_NODE_CHANGE_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("changeType", SimTypeInt(signed=False, label="PEER_NODE_CHANGE_TYPE")), ("ullNodeId", SimTypeLongLong(signed=False, label="UInt64")), ("pwzPeerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_EVENT_NODE_CHANGE_DATA", pack=False, align=None),
    "PEER_EVENT_OBJECT_CHANGED_DATA": SimStruct(OrderedDict((("pContact", SimTypePointer(SimTypeRef("PEER_CONTACT", SimStruct), offset=0)), ("pEndpoint", SimTypePointer(SimTypeRef("PEER_ENDPOINT", SimStruct), offset=0)), ("changeType", SimTypeInt(signed=False, label="PEER_CHANGE_TYPE")), ("pObject", SimTypePointer(SimTypeRef("PEER_OBJECT", SimStruct), offset=0)),)), name="PEER_EVENT_OBJECT_CHANGED_DATA", pack=False, align=None),
    "PEER_EVENT_PEOPLE_NEAR_ME_CHANGED_DATA": SimStruct(OrderedDict((("changeType", SimTypeInt(signed=False, label="PEER_CHANGE_TYPE")), ("pPeopleNearMe", SimTypePointer(SimTypeRef("PEER_PEOPLE_NEAR_ME", SimStruct), offset=0)),)), name="PEER_EVENT_PEOPLE_NEAR_ME_CHANGED_DATA", pack=False, align=None),
    "PEER_EVENT_PRESENCE_CHANGED_DATA": SimStruct(OrderedDict((("pContact", SimTypePointer(SimTypeRef("PEER_CONTACT", SimStruct), offset=0)), ("pEndpoint", SimTypePointer(SimTypeRef("PEER_ENDPOINT", SimStruct), offset=0)), ("changeType", SimTypeInt(signed=False, label="PEER_CHANGE_TYPE")), ("pPresenceInfo", SimTypePointer(SimTypeRef("PEER_PRESENCE_INFO", SimStruct), offset=0)),)), name="PEER_EVENT_PRESENCE_CHANGED_DATA", pack=False, align=None),
    "PEER_EVENT_RECORD_CHANGE_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("changeType", SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE")), ("recordId", SimTypeBottom(label="Guid")), ("recordType", SimTypeBottom(label="Guid")),)), name="PEER_EVENT_RECORD_CHANGE_DATA", pack=False, align=None),
    "PEER_EVENT_REQUEST_STATUS_CHANGED_DATA": SimStruct(OrderedDict((("pEndpoint", SimTypePointer(SimTypeRef("PEER_ENDPOINT", SimStruct), offset=0)), ("hrChange", SimTypeInt(signed=True, label="Int32")),)), name="PEER_EVENT_REQUEST_STATUS_CHANGED_DATA", pack=False, align=None),
    "PEER_EVENT_SYNCHRONIZED_DATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("recordType", SimTypeBottom(label="Guid")),)), name="PEER_EVENT_SYNCHRONIZED_DATA", pack=False, align=None),
    "PEER_EVENT_WATCHLIST_CHANGED_DATA": SimStruct(OrderedDict((("pContact", SimTypePointer(SimTypeRef("PEER_CONTACT", SimStruct), offset=0)), ("changeType", SimTypeInt(signed=False, label="PEER_CHANGE_TYPE")),)), name="PEER_EVENT_WATCHLIST_CHANGED_DATA", pack=False, align=None),
    "PEER_GRAPH_EVENT_DATA": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="PEER_GRAPH_EVENT_TYPE")), ("Anonymous", SimUnion({"dwStatus": SimTypeInt(signed=False, label="PEER_GRAPH_STATUS_FLAGS"), "incomingData": SimTypeRef("PEER_EVENT_INCOMING_DATA", SimStruct), "recordChangeData": SimTypeRef("PEER_EVENT_RECORD_CHANGE_DATA", SimStruct), "connectionChangeData": SimTypeRef("PEER_EVENT_CONNECTION_CHANGE_DATA", SimStruct), "nodeChangeData": SimTypeRef("PEER_EVENT_NODE_CHANGE_DATA", SimStruct), "synchronizedData": SimTypeRef("PEER_EVENT_SYNCHRONIZED_DATA", SimStruct)}, name="<anon>", label="None")),)), name="PEER_GRAPH_EVENT_DATA", pack=False, align=None),
    "PEER_GRAPH_EVENT_REGISTRATION": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="PEER_GRAPH_EVENT_TYPE")), ("pType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="PEER_GRAPH_EVENT_REGISTRATION", pack=False, align=None),
    "PEER_GRAPH_EVENT_TYPE": SimTypeInt(signed=False, label="PEER_GRAPH_EVENT_TYPE"),
    "PEER_GRAPH_PROPERTIES": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwScope", SimTypeInt(signed=False, label="UInt32")), ("dwMaxRecordSize", SimTypeInt(signed=False, label="UInt32")), ("pwzGraphId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzCreatorId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulPresenceLifetime", SimTypeInt(signed=False, label="UInt32")), ("cPresenceMax", SimTypeInt(signed=False, label="UInt32")),)), name="PEER_GRAPH_PROPERTIES", pack=False, align=None),
    "PEER_GRAPH_PROPERTY_FLAGS": SimTypeInt(signed=False, label="PEER_GRAPH_PROPERTY_FLAGS"),
    "PEER_GRAPH_SCOPE": SimTypeInt(signed=False, label="PEER_GRAPH_SCOPE"),
    "PEER_GRAPH_STATUS_FLAGS": SimTypeInt(signed=False, label="PEER_GRAPH_STATUS_FLAGS"),
    "PEER_GROUP_AUTHENTICATION_SCHEME": SimTypeInt(signed=False, label="PEER_GROUP_AUTHENTICATION_SCHEME"),
    "PEER_GROUP_EVENT_DATA": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="PEER_GROUP_EVENT_TYPE")), ("Anonymous", SimUnion({"dwStatus": SimTypeInt(signed=False, label="PEER_GROUP_STATUS"), "incomingData": SimTypeRef("PEER_EVENT_INCOMING_DATA", SimStruct), "recordChangeData": SimTypeRef("PEER_EVENT_RECORD_CHANGE_DATA", SimStruct), "connectionChangeData": SimTypeRef("PEER_EVENT_CONNECTION_CHANGE_DATA", SimStruct), "memberChangeData": SimTypeRef("PEER_EVENT_MEMBER_CHANGE_DATA", SimStruct), "hrConnectionFailedReason": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="PEER_GROUP_EVENT_DATA", pack=False, align=None),
    "PEER_GROUP_EVENT_REGISTRATION": SimStruct(OrderedDict((("eventType", SimTypeInt(signed=False, label="PEER_GROUP_EVENT_TYPE")), ("pType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="PEER_GROUP_EVENT_REGISTRATION", pack=False, align=None),
    "PEER_GROUP_EVENT_TYPE": SimTypeInt(signed=False, label="PEER_GROUP_EVENT_TYPE"),
    "PEER_GROUP_ISSUE_CREDENTIAL_FLAGS": SimTypeInt(signed=False, label="PEER_GROUP_ISSUE_CREDENTIAL_FLAGS"),
    "PEER_GROUP_PROPERTIES": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwzCloud", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzClassifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzGroupPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzCreatorPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulMemberDataLifetime", SimTypeInt(signed=False, label="UInt32")), ("ulPresenceLifetime", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationSchemes", SimTypeInt(signed=False, label="UInt32")), ("pwzGroupPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("groupPasswordRole", SimTypeBottom(label="Guid")),)), name="PEER_GROUP_PROPERTIES", pack=False, align=None),
    "PEER_GROUP_PROPERTY_FLAGS": SimTypeInt(signed=False, label="PEER_GROUP_PROPERTY_FLAGS"),
    "PEER_GROUP_STATUS": SimTypeInt(signed=False, label="PEER_GROUP_STATUS"),
    "PEER_INVITATION": SimStruct(OrderedDict((("applicationId", SimTypeBottom(label="Guid")), ("applicationData", SimTypeRef("PEER_DATA", SimStruct)), ("pwzMessage", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_INVITATION", pack=False, align=None),
    "PEER_INVITATION_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwzCloudName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwScope", SimTypeInt(signed=False, label="UInt32")), ("dwCloudFlags", SimTypeInt(signed=False, label="UInt32")), ("pwzGroupPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzIssuerPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzSubjectPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzGroupFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzIssuerFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzSubjectFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftValidityStart", SimTypeRef("FILETIME", SimStruct)), ("ftValidityEnd", SimTypeRef("FILETIME", SimStruct)), ("cRoles", SimTypeInt(signed=False, label="UInt32")), ("pRoles", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cClassifiers", SimTypeInt(signed=False, label="UInt32")), ("ppwzClassifiers", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("pSubjectPublicKey", SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0)), ("authScheme", SimTypeInt(signed=False, label="PEER_GROUP_AUTHENTICATION_SCHEME")),)), name="PEER_INVITATION_INFO", pack=False, align=None),
    "PEER_INVITATION_RESPONSE": SimStruct(OrderedDict((("action", SimTypeInt(signed=False, label="PEER_INVITATION_RESPONSE_TYPE")), ("pwzMessage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hrExtendedInfo", SimTypeInt(signed=True, label="Int32")),)), name="PEER_INVITATION_RESPONSE", pack=False, align=None),
    "PEER_INVITATION_RESPONSE_TYPE": SimTypeInt(signed=False, label="PEER_INVITATION_RESPONSE_TYPE"),
    "PEER_MEMBER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwzIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzAttributes", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ullNodeId", SimTypeLongLong(signed=False, label="UInt64")), ("cAddresses", SimTypeInt(signed=False, label="UInt32")), ("pAddresses", SimTypePointer(SimTypeRef("PEER_ADDRESS", SimStruct), offset=0)), ("pCredentialInfo", SimTypePointer(SimTypeRef("PEER_CREDENTIAL_INFO", SimStruct), offset=0)),)), name="PEER_MEMBER", pack=False, align=None),
    "PEER_MEMBER_CHANGE_TYPE": SimTypeInt(signed=False, label="PEER_MEMBER_CHANGE_TYPE"),
    "PEER_MEMBER_FLAGS": SimTypeInt(signed=False, label="PEER_MEMBER_FLAGS"),
    "PEER_NAME_PAIR": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwzPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_NAME_PAIR", pack=False, align=None),
    "PEER_NODE_CHANGE_TYPE": SimTypeInt(signed=False, label="PEER_NODE_CHANGE_TYPE"),
    "PEER_NODE_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("ullNodeId", SimTypeLongLong(signed=False, label="UInt64")), ("pwzPeerId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cAddresses", SimTypeInt(signed=False, label="UInt32")), ("pAddresses", SimTypePointer(SimTypeRef("PEER_ADDRESS", SimStruct), offset=0)), ("pwzAttributes", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_NODE_INFO", pack=False, align=None),
    "PEER_OBJECT": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("data", SimTypeRef("PEER_DATA", SimStruct)), ("dwPublicationScope", SimTypeInt(signed=False, label="UInt32")),)), name="PEER_OBJECT", pack=False, align=None),
    "PEER_PEOPLE_NEAR_ME": SimStruct(OrderedDict((("pwzNickName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("endpoint", SimTypeRef("PEER_ENDPOINT", SimStruct)), ("id", SimTypeBottom(label="Guid")),)), name="PEER_PEOPLE_NEAR_ME", pack=False, align=None),
    "PEER_PNRP_CLOUD_INFO": SimStruct(OrderedDict((("pwzCloudName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwScope", SimTypeInt(signed=False, label="PNRP_SCOPE")), ("dwScopeId", SimTypeInt(signed=False, label="UInt32")),)), name="PEER_PNRP_CLOUD_INFO", pack=False, align=None),
    "PEER_PNRP_ENDPOINT_INFO": SimStruct(OrderedDict((("pwzPeerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cAddresses", SimTypeInt(signed=False, label="UInt32")), ("ppAddresses", SimTypePointer(SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), offset=0)), ("pwzComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("payload", SimTypeRef("PEER_DATA", SimStruct)),)), name="PEER_PNRP_ENDPOINT_INFO", pack=False, align=None),
    "PEER_PNRP_REGISTRATION_INFO": SimStruct(OrderedDict((("pwzCloudName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzPublishingIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cAddresses", SimTypeInt(signed=False, label="UInt32")), ("ppAddresses", SimTypePointer(SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), offset=0)), ("wPort", SimTypeShort(signed=False, label="UInt16")), ("pwzComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("payload", SimTypeRef("PEER_DATA", SimStruct)),)), name="PEER_PNRP_REGISTRATION_INFO", pack=False, align=None),
    "PEER_PRESENCE_INFO": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="PEER_PRESENCE_STATUS")), ("pwzDescriptiveText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PEER_PRESENCE_INFO", pack=False, align=None),
    "PEER_PRESENCE_STATUS": SimTypeInt(signed=False, label="PEER_PRESENCE_STATUS"),
    "PEER_PUBLICATION_SCOPE": SimTypeInt(signed=False, label="PEER_PUBLICATION_SCOPE"),
    "PEER_RECORD": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeBottom(label="Guid")), ("id", SimTypeBottom(label="Guid")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwzCreatorId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzModifiedById", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzAttributes", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftCreation", SimTypeRef("FILETIME", SimStruct)), ("ftExpiration", SimTypeRef("FILETIME", SimStruct)), ("ftLastModified", SimTypeRef("FILETIME", SimStruct)), ("securityData", SimTypeRef("PEER_DATA", SimStruct)), ("data", SimTypeRef("PEER_DATA", SimStruct)),)), name="PEER_RECORD", pack=False, align=None),
    "PEER_RECORD_CHANGE_TYPE": SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE"),
    "PEER_RECORD_FLAGS": SimTypeInt(signed=False, label="PEER_RECORD_FLAGS"),
    "PEER_SECURITY_INTERFACE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pwzSspFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzPackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbSecurityInfo", SimTypeInt(signed=False, label="UInt32")), ("pbSecurityInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfnValidateRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PEER_RECORD", SimStruct), offset=0), SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE")], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext", "pRecord", "changeType"]), offset=0)), ("pfnSecureRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PEER_RECORD", SimStruct), offset=0), SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE"), SimTypePointer(SimTypePointer(SimTypeRef("PEER_DATA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext", "pRecord", "changeType", "ppSecurityData"]), offset=0)), ("pfnFreeSecurityData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PEER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext", "pSecurityData"]), offset=0)), ("pfnAuthFailed", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext"]), offset=0)),)), name="PEER_SECURITY_INTERFACE", pack=False, align=None),
    "PEER_SIGNIN_FLAGS": SimTypeInt(signed=False, label="PEER_SIGNIN_FLAGS"),
    "PEER_VERSION_DATA": SimStruct(OrderedDict((("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wHighestVersion", SimTypeShort(signed=False, label="UInt16")),)), name="PEER_VERSION_DATA", pack=False, align=None),
    "PEER_WATCH_PERMISSION": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"),
    "PEJOB": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PELARRAY": SimStruct(OrderedDict((("paXCount", SimTypeInt(signed=True, label="Int32")), ("paYCount", SimTypeInt(signed=True, label="Int32")), ("paXExt", SimTypeInt(signed=True, label="Int32")), ("paYExt", SimTypeInt(signed=True, label="Int32")), ("paRGBs", SimTypeChar(label="Byte")),)), name="PELARRAY", pack=False, align=None),
    "PENABLECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="ENABLECALLBACK_ENABLED_STATE"), SimTypeChar(label="Byte"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("EVENT_FILTER_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["SourceId", "IsEnabled", "Level", "MatchAnyKeyword", "MatchAllKeyword", "FilterData", "CallbackContext"]), offset=0),
    "PENABLE_VIRTUALIZATION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PENDINGMSG": SimTypeInt(signed=False, label="PENDINGMSG"),
    "PENDINGTYPE": SimTypeInt(signed=False, label="PENDINGTYPE"),
    "PENDING_REQUEST_DESIRED_PROPERTY": SimTypeInt(signed=False, label="PENDING_REQUEST_DESIRED_PROPERTY"),
    "PEND_CONTROL_CALL": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "status"]), offset=0),
    "PEND_TYPE_CONTROL_CALL": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "status"]), offset=0),
    "PENUMDIRTREE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FilePath", "CallerData"]), offset=0),
    "PENUMDIRTREE_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FilePath", "CallerData"]), offset=0),
    "PENUMERATE_HANDLES": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KDEXT_HANDLE_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["HandleInfo", "Flags", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Process", "HandleToDump", "Flags", "Callback", "Context"]), offset=0),
    "PENUMERATE_HASH_TABLE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Entry", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "HashTable", "Callback", "Context"]), offset=0),
    "PENUMERATE_JOB_PROCESSES": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Job", "Process", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Job", "Callback", "Context"]), offset=0),
    "PENUMERATE_SYSTEM_LOCKS": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KDEXTS_LOCK_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pLock", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Flags", "Callback", "Context"]), offset=0),
    "PENUMLOADED_MODULES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ModuleName", "ModuleBase", "ModuleSize", "UserContext"]), offset=0),
    "PENUMLOADED_MODULES_CALLBACK64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ModuleName", "ModuleBase", "ModuleSize", "UserContext"]), offset=0),
    "PENUMLOADED_MODULES_CALLBACKW64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ModuleName", "ModuleBase", "ModuleSize", "UserContext"]), offset=0),
    "PENUMSOURCEFILETOKENSCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["token", "size"]), offset=0),
    "PENUM_PAGE_FILE_CALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("ENUM_PAGE_FILE_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "pPageFileInfo", "lpFilename"]), offset=0),
    "PENUM_PAGE_FILE_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("ENUM_PAGE_FILE_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "pPageFileInfo", "lpFilename"]), offset=0),
    "PEN_STYLE": SimTypeInt(signed=False, label="PEN_STYLE"),
    "PEPROCESS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PERBANDINFO": SimStruct(OrderedDict((("bRepeatThisBand", SimTypeInt(signed=True, label="Int32")), ("szlBand", SimTypeRef("SIZE", SimStruct)), ("ulHorzRes", SimTypeInt(signed=False, label="UInt32")), ("ulVertRes", SimTypeInt(signed=False, label="UInt32")),)), name="PERBANDINFO", pack=False, align=None),
    "PERCEIVED": SimTypeInt(signed=False, label="PERCEIVED"),
    "PERCEPTION_PAYLOAD_FIELD": SimStruct(OrderedDict((("FieldId", SimTypeBottom(label="Guid")), ("OffsetInBytes", SimTypeInt(signed=False, label="UInt32")), ("SizeInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="PERCEPTION_PAYLOAD_FIELD", pack=False, align=None),
    "PERCEPTION_STATE_STREAM_TIMESTAMPS": SimStruct(OrderedDict((("InputTimestampInQpcCounts", SimTypeLongLong(signed=True, label="Int64")), ("AvailableTimestampInQpcCounts", SimTypeLongLong(signed=True, label="Int64")),)), name="PERCEPTION_STATE_STREAM_TIMESTAMPS", pack=False, align=None),
    "PERFLIBREQUEST": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestCode", "Buffer", "BufferSize"]), offset=0),
    "PERFORMANCE_DATA": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeChar(label="Byte")), ("HwCountersCount", SimTypeChar(label="Byte")), ("ContextSwitchCount", SimTypeInt(signed=False, label="UInt32")), ("WaitReasonBitMap", SimTypeLongLong(signed=False, label="UInt64")), ("CycleTime", SimTypeLongLong(signed=False, label="UInt64")), ("RetryCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("HwCounters", SimTypeArray(SimTypeRef("HARDWARE_COUNTER_DATA", SimStruct), 16)),)), name="PERFORMANCE_DATA", pack=False, align=None),
    "PERFORMANCE_INFORMATION": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("CommitTotal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CommitLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CommitPeak", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PhysicalTotal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PhysicalAvailable", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SystemCache", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("KernelTotal", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("KernelPaged", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("KernelNonpaged", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PageSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("ProcessCount", SimTypeInt(signed=False, label="UInt32")), ("ThreadCount", SimTypeInt(signed=False, label="UInt32")),)), name="PERFORMANCE_INFORMATION", pack=False, align=None),
    "PERF_BIN": SimStruct(OrderedDict((("NumberOfBins", SimTypeInt(signed=False, label="UInt32")), ("TypeOfBin", SimTypeInt(signed=False, label="UInt32")), ("BinsRanges", SimTypePointer(SimTypeRef("BIN_RANGE", SimStruct), offset=0)),)), name="PERF_BIN", pack=False, align=None),
    "PERF_COUNTERSET_INFO": SimStruct(OrderedDict((("CounterSetGuid", SimTypeBottom(label="Guid")), ("ProviderGuid", SimTypeBottom(label="Guid")), ("NumCounters", SimTypeInt(signed=False, label="UInt32")), ("InstanceType", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTERSET_INFO", pack=False, align=None),
    "PERF_COUNTERSET_INSTANCE": SimStruct(OrderedDict((("CounterSetGuid", SimTypeBottom(label="Guid")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("InstanceNameOffset", SimTypeInt(signed=False, label="UInt32")), ("InstanceNameSize", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTERSET_INSTANCE", pack=False, align=None),
    "PERF_COUNTERSET_REG_INFO": SimStruct(OrderedDict((("CounterSetGuid", SimTypeBottom(label="Guid")), ("CounterSetType", SimTypeInt(signed=False, label="UInt32")), ("DetailLevel", SimTypeInt(signed=False, label="UInt32")), ("NumCounters", SimTypeInt(signed=False, label="UInt32")), ("InstanceType", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTERSET_REG_INFO", pack=False, align=None),
    "PERF_COUNTER_AGGREGATE_FUNC": SimTypeInt(signed=False, label="PERF_COUNTER_AGGREGATE_FUNC"),
    "PERF_COUNTER_BLOCK": SimStruct(OrderedDict((("ByteLength", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_BLOCK", pack=False, align=None),
    "PERF_COUNTER_DATA": SimStruct(OrderedDict((("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_DATA", pack=False, align=None),
    "PERF_COUNTER_DEFINITION": SimStruct(OrderedDict((("ByteLength", SimTypeInt(signed=False, label="UInt32")), ("CounterNameTitleIndex", SimTypeInt(signed=False, label="UInt32")), ("CounterNameTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CounterHelpTitleIndex", SimTypeInt(signed=False, label="UInt32")), ("CounterHelpTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DefaultScale", SimTypeInt(signed=True, label="Int32")), ("DetailLevel", SimTypeInt(signed=False, label="UInt32")), ("CounterType", SimTypeInt(signed=False, label="UInt32")), ("CounterSize", SimTypeInt(signed=False, label="UInt32")), ("CounterOffset", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_DEFINITION", pack=False, align=None),
    "PERF_COUNTER_HEADER": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="PerfCounterDataType")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_HEADER", pack=False, align=None),
    "PERF_COUNTER_IDENTIFIER": SimStruct(OrderedDict((("CounterSetGuid", SimTypeBottom(label="Guid")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("CounterId", SimTypeInt(signed=False, label="UInt32")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_IDENTIFIER", pack=False, align=None),
    "PERF_COUNTER_IDENTITY": SimStruct(OrderedDict((("CounterSetGuid", SimTypeBottom(label="Guid")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("CounterId", SimTypeInt(signed=False, label="UInt32")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("MachineOffset", SimTypeInt(signed=False, label="UInt32")), ("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_IDENTITY", pack=False, align=None),
    "PERF_COUNTER_INFO": SimStruct(OrderedDict((("CounterId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Attrib", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DetailLevel", SimTypeInt(signed=False, label="UInt32")), ("Scale", SimTypeInt(signed=True, label="Int32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_INFO", pack=False, align=None),
    "PERF_COUNTER_REG_INFO": SimStruct(OrderedDict((("CounterId", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Attrib", SimTypeLongLong(signed=False, label="UInt64")), ("DetailLevel", SimTypeInt(signed=False, label="UInt32")), ("DefaultScale", SimTypeInt(signed=True, label="Int32")), ("BaseCounterId", SimTypeInt(signed=False, label="UInt32")), ("PerfTimeId", SimTypeInt(signed=False, label="UInt32")), ("PerfFreqId", SimTypeInt(signed=False, label="UInt32")), ("MultiId", SimTypeInt(signed=False, label="UInt32")), ("AggregateFunc", SimTypeInt(signed=False, label="PERF_COUNTER_AGGREGATE_FUNC")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_COUNTER_REG_INFO", pack=False, align=None),
    "PERF_DATA_BLOCK": SimStruct(OrderedDict((("Signature", SimTypeArray(SimTypeChar(label="Char"), 4)), ("LittleEndian", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeInt(signed=False, label="UInt32")), ("TotalByteLength", SimTypeInt(signed=False, label="UInt32")), ("HeaderLength", SimTypeInt(signed=False, label="UInt32")), ("NumObjectTypes", SimTypeInt(signed=False, label="UInt32")), ("DefaultObject", SimTypeInt(signed=True, label="Int32")), ("SystemTime", SimTypeRef("SYSTEMTIME", SimStruct)), ("PerfTime", SimTypeLongLong(signed=True, label="Int64")), ("PerfFreq", SimTypeLongLong(signed=True, label="Int64")), ("PerfTime100nSec", SimTypeLongLong(signed=True, label="Int64")), ("SystemNameLength", SimTypeInt(signed=False, label="UInt32")), ("SystemNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_DATA_BLOCK", pack=False, align=None),
    "PERF_DATA_HEADER": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumCounters", SimTypeInt(signed=False, label="UInt32")), ("PerfTimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("PerfTime100NSec", SimTypeLongLong(signed=True, label="Int64")), ("PerfFreq", SimTypeLongLong(signed=True, label="Int64")), ("SystemTime", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="PERF_DATA_HEADER", pack=False, align=None),
    "PERF_DETAIL": SimTypeInt(signed=False, label="PERF_DETAIL"),
    "PERF_INSTANCE_DEFINITION": SimStruct(OrderedDict((("ByteLength", SimTypeInt(signed=False, label="UInt32")), ("ParentObjectTitleIndex", SimTypeInt(signed=False, label="UInt32")), ("ParentObjectInstance", SimTypeInt(signed=False, label="UInt32")), ("UniqueID", SimTypeInt(signed=True, label="Int32")), ("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_INSTANCE_DEFINITION", pack=False, align=None),
    "PERF_INSTANCE_HEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_INSTANCE_HEADER", pack=False, align=None),
    "PERF_MEM_ALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["AllocSize", "pContext"]), offset=0),
    "PERF_MEM_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pBuffer", "pContext"]), offset=0),
    "PERF_MULTI_COUNTERS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCounters", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_MULTI_COUNTERS", pack=False, align=None),
    "PERF_MULTI_INSTANCES": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwInstances", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_MULTI_INSTANCES", pack=False, align=None),
    "PERF_OBJECT_TYPE": SimStruct(OrderedDict((("TotalByteLength", SimTypeInt(signed=False, label="UInt32")), ("DefinitionLength", SimTypeInt(signed=False, label="UInt32")), ("HeaderLength", SimTypeInt(signed=False, label="UInt32")), ("ObjectNameTitleIndex", SimTypeInt(signed=False, label="UInt32")), ("ObjectNameTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ObjectHelpTitleIndex", SimTypeInt(signed=False, label="UInt32")), ("ObjectHelpTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DetailLevel", SimTypeInt(signed=False, label="UInt32")), ("NumCounters", SimTypeInt(signed=False, label="UInt32")), ("DefaultCounter", SimTypeInt(signed=True, label="Int32")), ("NumInstances", SimTypeInt(signed=True, label="Int32")), ("CodePage", SimTypeInt(signed=False, label="UInt32")), ("PerfTime", SimTypeLongLong(signed=True, label="Int64")), ("PerfFreq", SimTypeLongLong(signed=True, label="Int64")),)), name="PERF_OBJECT_TYPE", pack=False, align=None),
    "PERF_PROVIDER_CONTEXT": SimStruct(OrderedDict((("ContextSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ControlCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestCode", "Buffer", "BufferSize"]), offset=0)), ("MemAllocRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["AllocSize", "pContext"]), offset=0)), ("MemFreeRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pBuffer", "pContext"]), offset=0)), ("pMemContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PERF_PROVIDER_CONTEXT", pack=False, align=None),
    "PERF_STRING_BUFFER_HEADER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCounters", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_STRING_BUFFER_HEADER", pack=False, align=None),
    "PERF_STRING_COUNTER_HEADER": SimStruct(OrderedDict((("dwCounterId", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="PERF_STRING_COUNTER_HEADER", pack=False, align=None),
    "PERSISTENT_ISCSI_LOGIN_INFOA": SimStruct(OrderedDict((("TargetName", SimTypeArray(SimTypeChar(label="SByte"), 224)), ("IsInformationalSession", SimTypeChar(label="Byte")), ("InitiatorInstance", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("InitiatorPortNumber", SimTypeInt(signed=False, label="UInt32")), ("TargetPortal", SimTypeRef("ISCSI_TARGET_PORTALA", SimStruct)), ("SecurityFlags", SimTypeLongLong(signed=False, label="UInt64")), ("Mappings", SimTypePointer(SimTypeRef("ISCSI_TARGET_MAPPINGA", SimStruct), offset=0)), ("LoginOptions", SimTypeRef("ISCSI_LOGIN_OPTIONS", SimStruct)),)), name="PERSISTENT_ISCSI_LOGIN_INFOA", pack=False, align=None),
    "PERSISTENT_ISCSI_LOGIN_INFOW": SimStruct(OrderedDict((("TargetName", SimTypeArray(SimTypeChar(label="Char"), 224)), ("IsInformationalSession", SimTypeChar(label="Byte")), ("InitiatorInstance", SimTypeArray(SimTypeChar(label="Char"), 256)), ("InitiatorPortNumber", SimTypeInt(signed=False, label="UInt32")), ("TargetPortal", SimTypeRef("ISCSI_TARGET_PORTALW", SimStruct)), ("SecurityFlags", SimTypeLongLong(signed=False, label="UInt64")), ("Mappings", SimTypePointer(SimTypeRef("ISCSI_TARGET_MAPPINGW", SimStruct), offset=0)), ("LoginOptions", SimTypeRef("ISCSI_LOGIN_OPTIONS", SimStruct)),)), name="PERSISTENT_ISCSI_LOGIN_INFOW", pack=False, align=None),
    "PERSISTENT_RESERVE_COMMAND": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"PR_IN": SimTypeRef("_PR_IN_e__Struct", SimStruct), "PR_OUT": SimTypeRef("_PR_OUT_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PERSISTENT_RESERVE_COMMAND", pack=False, align=None),
    "PERSIST_FOLDER_TARGET_INFO": SimStruct(OrderedDict((("pidlTargetFolder", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("szTargetParsingName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szNetworkProvider", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwAttributes", SimTypeInt(signed=False, label="UInt32")), ("csidl", SimTypeInt(signed=True, label="Int32")),)), name="PERSIST_FOLDER_TARGET_INFO", pack=False, align=None),
    "PERUSERSECTIONA": SimStruct(OrderedDict((("szGUID", SimTypeArray(SimTypeChar(label="SByte"), 59)), ("szDispName", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("szLocale", SimTypeArray(SimTypeChar(label="SByte"), 10)), ("szStub", SimTypeArray(SimTypeChar(label="SByte"), 1040)), ("szVersion", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("szCompID", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("dwIsInstalled", SimTypeInt(signed=False, label="UInt32")), ("bRollback", SimTypeInt(signed=True, label="Int32")),)), name="PERUSERSECTIONA", pack=False, align=None),
    "PERUSERSECTIONW": SimStruct(OrderedDict((("szGUID", SimTypeArray(SimTypeChar(label="Char"), 59)), ("szDispName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("szLocale", SimTypeArray(SimTypeChar(label="Char"), 10)), ("szStub", SimTypeArray(SimTypeChar(label="Char"), 1040)), ("szVersion", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szCompID", SimTypeArray(SimTypeChar(label="Char"), 128)), ("dwIsInstalled", SimTypeInt(signed=False, label="UInt32")), ("bRollback", SimTypeInt(signed=True, label="Int32")),)), name="PERUSERSECTIONW", pack=False, align=None),
    "PESILO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PETHREAD": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PETWENABLECALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PETW_BUFFER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ETW_BUFFER_HEADER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("ETW_BUFFER_CALLBACK_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Buffer", "BufferSize", "ConsumerInfo", "CallbackContext"]), offset=0),
    "PETW_BUFFER_COMPLETION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ETW_BUFFER_HEADER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Buffer", "CallbackContext"]), offset=0),
    "PEVENT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pEvent"]), offset=0),
    "PEVENT_RECORD_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_RECORD", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["EventRecord"]), offset=0),
    "PEVENT_TRACE_BUFFER_CALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE_LOGFILEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Logfile"]), offset=0),
    "PEVENT_TRACE_BUFFER_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EVENT_TRACE_LOGFILEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Logfile"]), offset=0),
    "PEVT_VHF_ASYNC_OPERATION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PEVT_VHF_CLEANUP": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PEVT_VHF_READY_FOR_NEXT_READ_REPORT": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PEXCEPTION_FILTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ExceptionPointers", "EstablisherFrame"]), offset=0),
    "PEXPAND_STACK_CALLOUT": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PEXTEND_RES_CONTROL_CALL": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "newTimeoutInMs"]), offset=0),
    "PEXTEND_RES_TYPE_CONTROL_CALL": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "newTimeoutInMs"]), offset=0),
    "PEXT_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PEXT_DELETE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PEX_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PEX_RUNDOWN_REF_CACHE_AWARE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PEX_TIMER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFADDRESSTYPE": SimTypeInt(signed=False, label="PFADDRESSTYPE"),
    "PFAXABORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId"]), offset=0),
    "PFAXACCESSCHECK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "AccessMask"]), offset=0),
    "PFAXCLOSE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle"]), offset=0),
    "PFAXCOMPLETEJOBPARAMSA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("FAX_JOB_PARAMA", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobParams", "CoverpageInfo"]), offset=0),
    "PFAXCOMPLETEJOBPARAMSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("FAX_JOB_PARAMW", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobParams", "CoverpageInfo"]), offset=0),
    "PFAXCONNECTFAXSERVERA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MachineName", "FaxHandle"]), offset=0),
    "PFAXCONNECTFAXSERVERW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MachineName", "FaxHandle"]), offset=0),
    "PFAXDEVABORTOPERATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFAXDEVCONFIGURE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFAXDEVENDJOB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFAXDEVINITIALIZE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["FaxHandle", "hDevice", "dwMessage", "dwInstance", "dwParam1", "dwParam2", "dwParam3"]), offset=0), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "DeviceId", "Param1", "Param2", "Param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFAXDEVRECEIVE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("FAX_RECEIVE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFAXDEVREPORTSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_DEV_STATUS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFAXDEVSEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_SEND", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "CallHandle", "Reserved1", "Reserved2"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFAXDEVSHUTDOWN": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PFAXDEVSTARTJOB": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFAXDEVVIRTUALDEVICECREATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceCount", "DeviceNamePrefix", "DeviceIdPrefix", "CompletionPort", "CompletionKey"]), offset=0),
    "PFAXENABLEROUTINGMETHODA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "Enabled"]), offset=0),
    "PFAXENABLEROUTINGMETHODW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "Enabled"]), offset=0),
    "PFAXENUMGLOBALROUTINGINFOA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_GLOBAL_ROUTING_INFOA", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RoutingInfo", "MethodsReturned"]), offset=0),
    "PFAXENUMGLOBALROUTINGINFOW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_GLOBAL_ROUTING_INFOW", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RoutingInfo", "MethodsReturned"]), offset=0),
    "PFAXENUMJOBSA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_JOB_ENTRYA", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobEntry", "JobsReturned"]), offset=0),
    "PFAXENUMJOBSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_JOB_ENTRYW", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobEntry", "JobsReturned"]), offset=0),
    "PFAXENUMPORTSA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_PORT_INFOA", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "PortInfo", "PortsReturned"]), offset=0),
    "PFAXENUMPORTSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_PORT_INFOW", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "PortInfo", "PortsReturned"]), offset=0),
    "PFAXENUMROUTINGMETHODSA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_ROUTING_METHODA", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingMethod", "MethodsReturned"]), offset=0),
    "PFAXENUMROUTINGMETHODSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_ROUTING_METHODW", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingMethod", "MethodsReturned"]), offset=0),
    "PFAXFREEBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Buffer"]), offset=0),
    "PFAXGETCONFIGURATIONA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_CONFIGURATIONA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]), offset=0),
    "PFAXGETCONFIGURATIONW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_CONFIGURATIONW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]), offset=0),
    "PFAXGETDEVICESTATUSA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_DEVICE_STATUSA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "DeviceStatus"]), offset=0),
    "PFAXGETDEVICESTATUSW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_DEVICE_STATUSW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "DeviceStatus"]), offset=0),
    "PFAXGETJOBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("FAX_JOB_ENTRYA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "JobEntry"]), offset=0),
    "PFAXGETJOBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("FAX_JOB_ENTRYW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "JobEntry"]), offset=0),
    "PFAXGETLOGGINGCATEGORIESA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_LOG_CATEGORYA", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]), offset=0),
    "PFAXGETLOGGINGCATEGORIESW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_LOG_CATEGORYW", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]), offset=0),
    "PFAXGETPAGEDATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "Buffer", "BufferSize", "ImageWidth", "ImageHeight"]), offset=0),
    "PFAXGETPORTA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_PORT_INFOA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]), offset=0),
    "PFAXGETPORTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("FAX_PORT_INFOW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]), offset=0),
    "PFAXGETROUTINGINFOA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]), offset=0),
    "PFAXGETROUTINGINFOW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]), offset=0),
    "PFAXINITIALIZEEVENTQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "CompletionPort", "CompletionKey", "hWnd", "MessageStart"]), offset=0),
    "PFAXOPENPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "DeviceId", "Flags", "FaxPortHandle"]), offset=0),
    "PFAXPRINTCOVERPAGEA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FAX_CONTEXT_INFOA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxContextInfo", "CoverPageInfo"]), offset=0),
    "PFAXPRINTCOVERPAGEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FAX_CONTEXT_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxContextInfo", "CoverPageInfo"]), offset=0),
    "PFAXREGISTERROUTINGEXTENSIONW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Context", "MethodName", "FriendlyName", "FunctionName", "Guid"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "ExtensionName", "FriendlyName", "ImageName", "CallBack", "Context"]), offset=0),
    "PFAXREGISTERSERVICEPROVIDERW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceProvider", "FriendlyName", "ImageName", "TspName"]), offset=0),
    "PFAXROUTEADDFILE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "FileName", "Guid"]), offset=0),
    "PFAXROUTEDELETEFILE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "FileName"]), offset=0),
    "PFAXROUTEDEVICECHANGENOTIFICATION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFAXROUTEDEVICEENABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFAXROUTEENUMFILE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "GuidOwner", "GuidCaller", "FileName", "Context"]), offset=0),
    "PFAXROUTEENUMFILES": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "GuidOwner", "GuidCaller", "FileName", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "Guid", "FileEnumerator", "Context"]), offset=0),
    "PFAXROUTEGETFILE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "Index", "FileNameBuffer", "RequiredSize"]), offset=0),
    "PFAXROUTEGETROUTINGINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFAXROUTEINITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_ROUTE_CALLBACKROUTINES", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFAXROUTEMETHOD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FAX_ROUTE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFAXROUTEMODIFYROUTINGDATA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["JobId", "RoutingGuid", "RoutingData", "RoutingDataSize"]), offset=0),
    "PFAXROUTESETROUTINGINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFAXSENDDOCUMENTA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("FAX_JOB_PARAMA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "JobParams", "CoverpageInfo", "FaxJobId"]), offset=0),
    "PFAXSENDDOCUMENTFORBROADCASTA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FAX_JOB_PARAMA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RecipientNumber", "Context", "JobParams", "CoverpageInfo"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "FaxJobId", "FaxRecipientCallback", "Context"]), offset=0),
    "PFAXSENDDOCUMENTFORBROADCASTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FAX_JOB_PARAMW", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RecipientNumber", "Context", "JobParams", "CoverpageInfo"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "FaxJobId", "FaxRecipientCallback", "Context"]), offset=0),
    "PFAXSENDDOCUMENTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("FAX_JOB_PARAMW", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "JobParams", "CoverpageInfo", "FaxJobId"]), offset=0),
    "PFAXSETCONFIGURATIONA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_CONFIGURATIONA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]), offset=0),
    "PFAXSETCONFIGURATIONW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_CONFIGURATIONW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]), offset=0),
    "PFAXSETGLOBALROUTINGINFOA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_GLOBAL_ROUTING_INFOA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingInfo"]), offset=0),
    "PFAXSETGLOBALROUTINGINFOW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_GLOBAL_ROUTING_INFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingInfo"]), offset=0),
    "PFAXSETJOBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("FAX_JOB_ENTRYA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "Command", "JobEntry"]), offset=0),
    "PFAXSETJOBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("FAX_JOB_ENTRYW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "Command", "JobEntry"]), offset=0),
    "PFAXSETLOGGINGCATEGORIESA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_LOG_CATEGORYA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]), offset=0),
    "PFAXSETLOGGINGCATEGORIESW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_LOG_CATEGORYW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]), offset=0),
    "PFAXSETPORTA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_PORT_INFOA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]), offset=0),
    "PFAXSETPORTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FAX_PORT_INFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]), offset=0),
    "PFAXSETROUTINGINFOA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]), offset=0),
    "PFAXSETROUTINGINFOW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]), offset=0),
    "PFAXSTARTPRINTJOBA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("FAX_PRINT_INFOA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("FAX_CONTEXT_INFOA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrinterName", "PrintInfo", "FaxJobId", "FaxContextInfo"]), offset=0),
    "PFAXSTARTPRINTJOBW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("FAX_PRINT_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("FAX_CONTEXT_INFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrinterName", "PrintInfo", "FaxJobId", "FaxContextInfo"]), offset=0),
    "PFAXUNREGISTERSERVICEPROVIDERW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceProvider"]), offset=0),
    "PFAX_EXT_CONFIG_CHANGE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFAX_EXT_FREE_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFAX_EXT_GET_DATA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFAX_EXT_INITIALIZE_CONFIG": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFAX_EXT_REGISTER_FOR_EVENTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFAX_EXT_SET_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FAX_ENUM_DEVICE_ID_SOURCE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFAX_EXT_UNREGISTER_FOR_EVENTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PFAX_LINECALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["FaxHandle", "hDevice", "dwMessage", "dwInstance", "dwParam1", "dwParam2", "dwParam3"]), offset=0),
    "PFAX_RECIPIENT_CALLBACKA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FAX_JOB_PARAMA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RecipientNumber", "Context", "JobParams", "CoverpageInfo"]), offset=0),
    "PFAX_RECIPIENT_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FAX_JOB_PARAMW", SimStruct), offset=0), SimTypePointer(SimTypeRef("FAX_COVERPAGE_INFOW", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RecipientNumber", "Context", "JobParams", "CoverpageInfo"]), offset=0),
    "PFAX_ROUTING_INSTALLATION_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Context", "MethodName", "FriendlyName", "FunctionName", "Guid"]), offset=0),
    "PFAX_SEND_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "CallHandle", "Reserved1", "Reserved2"]), offset=0),
    "PFAX_SERVICE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "DeviceId", "Param1", "Param2", "Param3"]), offset=0),
    "PFD_FLAGS": SimTypeInt(signed=False, label="PFD_FLAGS"),
    "PFD_LAYER_TYPE": SimTypeInt(signed=False, label="PFD_LAYER_TYPE"),
    "PFD_PIXEL_TYPE": SimTypeInt(signed=False, label="PFD_PIXEL_TYPE"),
    "PFEATURE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "PFE_EXPORT_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pbData", "pvCallbackContext", "ulLength"]), offset=0),
    "PFE_IMPORT_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pbData", "pvCallbackContext", "ulLength"]), offset=0),
    "PFFORWARD_ACTION": SimTypeInt(signed=False, label="PFFORWARD_ACTION"),
    "PFFRAMETYPE": SimTypeInt(signed=False, label="PFFRAMETYPE"),
    "PFIBER_CALLOUT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["lpParameter"]), offset=0),
    "PFILTER_REPORT_CHANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["NotifyContext", "FilterContext"]), offset=0),
    "PFINDFILEINPATHCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["filename", "context"]), offset=0),
    "PFINDFILEINPATHCALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["filename", "context"]), offset=0),
    "PFIND_DEBUG_FILE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileHandle", "FileName", "CallerData"]), offset=0),
    "PFIND_DEBUG_FILE_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileHandle", "FileName", "CallerData"]), offset=0),
    "PFIND_EXE_FILE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileHandle", "FileName", "CallerData"]), offset=0),
    "PFIND_EXE_FILE_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FileHandle", "FileName", "CallerData"]), offset=0),
    "PFIND_FILELOCK_OWNERINFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("KDEXT_FILELOCK_OWNER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pFileLockOwner"]), offset=0),
    "PFIND_MATCHING_PROCESS": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("KDEXT_PROCESS_FIND_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "ProcessInfo", "Process"]), offset=0),
    "PFIND_MATCHING_THREAD": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("KDEXT_THREAD_FIND_PARAMS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "ThreadInfo"]), offset=0),
    "PFLOGFRAME": SimStruct(OrderedDict((("Timestamp", SimTypeLongLong(signed=True, label="Int64")), ("pfeTypeOfFrame", SimTypeInt(signed=False, label="PFFRAMETYPE")), ("dwTotalSizeUsed", SimTypeInt(signed=False, label="UInt32")), ("dwFilterRule", SimTypeInt(signed=False, label="UInt32")), ("wSizeOfAdditionalData", SimTypeShort(signed=False, label="UInt16")), ("wSizeOfIpHeader", SimTypeShort(signed=False, label="UInt16")), ("dwInterfaceName", SimTypeInt(signed=False, label="UInt32")), ("dwIPIndex", SimTypeInt(signed=False, label="UInt32")), ("bPacketData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PFLOGFRAME", pack=False, align=None),
    "PFLS_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFlsData"]), offset=0),
    "PFLTOPLOCK_PREPOST_CALLBACKDATA_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackData", "Context"]), offset=0),
    "PFLTOPLOCK_WAIT_COMPLETE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackData", "Context"]), offset=0),
    "PFLT_CALLBACK_DATA_QUEUE_ACQUIRE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Irql"]), offset=0),
    "PFLT_CALLBACK_DATA_QUEUE_COMPLETE_CANCELED_IO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Cbd"]), offset=0),
    "PFLT_CALLBACK_DATA_QUEUE_INSERT_IO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Cbdq", "Cbd", "InsertContext"]), offset=0),
    "PFLT_CALLBACK_DATA_QUEUE_PEEK_NEXT_IO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), arg_names=["Cbdq", "Cbd", "PeekContext"]), offset=0),
    "PFLT_CALLBACK_DATA_QUEUE_RELEASE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Irql"]), offset=0),
    "PFLT_CALLBACK_DATA_QUEUE_REMOVE_IO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA_QUEUE", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Cbdq", "Cbd"]), offset=0),
    "PFLT_COMPLETED_ASYNC_IO_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackData", "Context"]), offset=0),
    "PFLT_COMPLETE_CANCELED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackData"]), offset=0),
    "PFLT_COMPLETE_LOCK_CALLBACK_DATA_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "CallbackData"]), offset=0),
    "PFLT_CONNECT_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientPort", "ServerPortCookie", "ConnectionContext", "SizeOfContext", "ConnectionPortCookie"]), offset=0),
    "PFLT_CONTEXT": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "PFLT_CONTEXT_ALLOCATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="POOL_TYPE"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PoolType", "Size", "ContextType"]), offset=0),
    "PFLT_CONTEXT_CLEANUP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["Context", "ContextType"]), offset=0),
    "PFLT_CONTEXT_FREE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["Pool", "ContextType"]), offset=0),
    "PFLT_DEFERRED_IO_WORKITEM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFLT_DEFERRED_IO_WORKITEM_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["FltWorkItem", "CallbackData", "Context"]), offset=0),
    "PFLT_DISCONNECT_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ConnectionCookie"]), offset=0),
    "PFLT_FILTER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFLT_FILTER_UNLOAD_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Flags"]), offset=0),
    "PFLT_GENERATE_FILE_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("FLT_NAME_CONTROL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "FileObject", "CallbackData", "NameOptions", "CacheFileNameInformation", "FileName"]), offset=0),
    "PFLT_GENERIC_WORKITEM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFLT_GENERIC_WORKITEM_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["FltWorkItem", "FltObject", "Context"]), offset=0),
    "PFLT_GET_OPERATION_STATUS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_IO_PARAMETER_BLOCK", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["FltObjects", "IopbSnapshot", "OperationStatus", "RequesterContext"]), offset=0),
    "PFLT_INSTANCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFLT_INSTANCE_QUERY_TEARDOWN_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FltObjects", "Flags"]), offset=0),
    "PFLT_INSTANCE_SETUP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FLT_FILESYSTEM_TYPE")], SimTypeInt(signed=True, label="Int32"), arg_names=["FltObjects", "Flags", "VolumeDeviceType", "VolumeFilesystemType"]), offset=0),
    "PFLT_INSTANCE_TEARDOWN_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["FltObjects", "Reason"]), offset=0),
    "PFLT_MESSAGE_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PortCookie", "InputBuffer", "InputBufferLength", "OutputBuffer", "OutputBufferLength", "ReturnOutputBufferLength"]), offset=0),
    "PFLT_NORMALIZE_CONTEXT_CLEANUP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["NormalizationContext"]), offset=0),
    "PFLT_NORMALIZE_NAME_COMPONENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILE_NAMES_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "ParentDirectory", "VolumeNameLength", "Component", "ExpandComponentName", "ExpandComponentNameLength", "Flags", "NormalizationContext"]), offset=0),
    "PFLT_NORMALIZE_NAME_COMPONENT_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILE_NAMES_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "FileObject", "ParentDirectory", "VolumeNameLength", "Component", "ExpandComponentName", "ExpandComponentNameLength", "Flags", "NormalizationContext"]), offset=0),
    "PFLT_PORT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFLT_POST_OPERATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="FLT_POSTOP_CALLBACK_STATUS"), arg_names=["Data", "FltObjects", "CompletionContext", "Flags"]), offset=0),
    "PFLT_PRE_OPERATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="FLT_PREOP_CALLBACK_STATUS"), arg_names=["Data", "FltObjects", "CompletionContext"]), offset=0),
    "PFLT_SECTION_CONFLICT_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FLT_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Instance", "SectionContext", "Data"]), offset=0),
    "PFLT_TRANSACTION_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FLT_RELATED_OBJECTS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FltObjects", "TransactionContext", "NotificationMask"]), offset=0),
    "PFLT_VOLUME": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PFLUSH_ADAPTER_BUFFERS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "CurrentVa", "Length", "WriteToDevice"]), offset=0),
    "PFLUSH_ADAPTER_BUFFERS_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "Offset", "Length", "WriteToDevice"]), offset=0),
    "PFLUSH_DMA_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "ReadOperation"]), offset=0),
    "PFLUSH_TO_LSN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64")], SimTypeBottom(label="Void"), arg_names=["LogHandle", "Lsn"]), offset=0),
    "PFN": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0),
    "PFNALLOC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0),
    "PFNBLUETOOTH_GATT_EVENT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="BTH_LE_GATT_EVENT_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["EventType", "EventOutParameter", "Context"]), offset=0),
    "PFNCALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["wType", "wFmt", "hConv", "hsz1", "hsz2", "hData", "dwData1", "dwData2"]), offset=0),
    "PFNCANSHAREFOLDERW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszPath"]), offset=0),
    "PFNCFILTERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "pfInitialSelectedCert", "pvCallbackData"]), offset=0),
    "PFNCHECKCONNECTIONWIZARD": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0),
    "PFNCLOSE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hf"]), offset=0),
    "PFNCMFILTERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "param1", "param2", "param3"]), offset=0),
    "PFNCMHOOKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndDialog", "message", "wParam", "lParam"]), offset=0),
    "PFNCOLHEAPFREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFNCOMPROPSHEET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hComPropSheet", "Function", "lParam1", "lParam2"]), offset=0),
    "PFNCONTEXTCALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ComCallData", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pParam"]), offset=0),
    "PFNCryptStreamOutputCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackCtxt", "pbData", "cbData", "fFinal"]), offset=0),
    "PFNCryptStreamOutputCallbackEx": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackCtxt", "pbData", "cbData", "hDescriptor", "fFinal"]), offset=0),
    "PFND3DKMT_ACQUIREKEYEDMUTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ACQUIREKEYEDMUTEX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_ACQUIREKEYEDMUTEX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ACQUIREKEYEDMUTEX2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_ADJUSTFULLSCREENGAMMA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ADJUSTFULLSCREENGAMMA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_BUDGETCHANGENOTIFICATIONCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_BUDGETCHANGENOTIFICATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CANCELPRESENTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CANCEL_PRESENTS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHANGESURFACEPOINTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHANGESURFACEPOINTER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHANGEVIDEOMEMORYRESERVATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHANGEVIDEOMEMORYRESERVATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKEXCLUSIVEOWNERSHIP": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PFND3DKMT_CHECKMONITORPOWERSTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKMONITORPOWERSTATE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKMULTIPLANEOVERLAYSUPPORT3", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKOCCLUSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKOCCLUSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKSHAREDRESOURCEACCESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKSHAREDRESOURCEACCESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CLOSEADAPTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CLOSEADAPTER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CONFIGURESHAREDRESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CONFIGURESHAREDRESOURCE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CONNECTDOORBELL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CONNECT_DOORBELL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEALLOCATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEALLOCATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEALLOCATION2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEALLOCATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATECONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATECONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATECONTEXTVIRTUAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATECONTEXTVIRTUAL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEDCFROMMEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEDCFROMMEMORY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEDEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEDEVICE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEDOORBELL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATE_DOORBELL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEKEYEDMUTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEKEYEDMUTEX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEKEYEDMUTEX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEKEYEDMUTEX2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATENATIVEFENCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATENATIVEFENCE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEOUTPUTDUPL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATE_OUTPUTDUPL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEOVERLAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEPAGINGQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEPAGINGQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATEPROTECTEDSESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATEPROTECTEDSESSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATESYNCHRONIZATIONOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATESYNCHRONIZATIONOBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_CREATESYNCHRONIZATIONOBJECT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_CREATESYNCHRONIZATIONOBJECT2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYALLOCATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYALLOCATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYALLOCATION2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYALLOCATION2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYCONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYCONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYDCFROMMEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYDCFROMMEMORY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYDEVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYDEVICE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYDOORBELL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROY_DOORBELL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYKEYEDMUTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYKEYEDMUTEX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYOUTPUTDUPL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROY_OUTPUTDUPL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYOVERLAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYPAGINGQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DDDI_DESTROYPAGINGQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYPROTECTEDSESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYPROTECTEDSESSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_DESTROYSYNCHRONIZATIONOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_DESTROYSYNCHRONIZATIONOBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_ENUMADAPTERS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ENUMADAPTERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_ENUMADAPTERS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ENUMADAPTERS2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_ENUMADAPTERS3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ENUMADAPTERS3", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_ESCAPE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_ESCAPE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_EVICT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_EVICT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_FLIPOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_FLIPOVERLAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_FLUSHHEAPTRANSITIONS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_FLUSHHEAPTRANSITIONS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_FREEGPUVIRTUALADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_FREEGPUVIRTUALADDRESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETALLOCATIONPRIORITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETALLOCATIONPRIORITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETCONTEXTINPROCESSSCHEDULINGPRIORITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETCONTEXTINPROCESSSCHEDULINGPRIORITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETCONTEXTSCHEDULINGPRIORITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETCONTEXTSCHEDULINGPRIORITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETDEVICESTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETDEVICESTATE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETDISPLAYMODELIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETDISPLAYMODELIST", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETDWMVERTICALBLANKEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETVERTICALBLANKEVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETMULTIPLANEOVERLAYCAPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GET_MULTIPLANE_OVERLAY_CAPS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETMULTISAMPLEMETHODLIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETMULTISAMPLEMETHODLIST", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETOVERLAYSTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETOVERLAYSTATE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETPOSTCOMPOSITIONCAPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GET_POST_COMPOSITION_CAPS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETPRESENTHISTORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETPRESENTHISTORY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETPROCESSDEVICEREMOVALSUPPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETPROCESSDEVICEREMOVALSUPPORT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETPROCESSSCHEDULINGPRIORITYCLASS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3DKMT_SCHEDULINGPRIORITYCLASS"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFND3DKMT_GETRESOURCEPRESENTPRIVATEDRIVERDATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DDDI_GETRESOURCEPRESENTPRIVATEDRIVERDATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETRUNTIMEDATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETRUNTIMEDATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETSCANLINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETSCANLINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETSHAREDPRIMARYHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETSHAREDPRIMARYHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_GETSHAREDRESOURCEADAPTERLUID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_GETSHAREDRESOURCEADAPTERLUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_INVALIDATEACTIVEVIDPN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_INVALIDATEACTIVEVIDPN", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_INVALIDATECACHE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_INVALIDATECACHE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_LOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_LOCK", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_LOCK2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_LOCK2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_MAKERESIDENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DDDI_MAKERESIDENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_MAPGPUVIRTUALADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DDDI_MAPGPUVIRTUALADDRESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_MARKDEVICEASERROR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_MARKDEVICEASERROR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_NOTIFYWORKSUBMISSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_NOTIFY_WORK_SUBMISSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OFFERALLOCATIONS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OFFERALLOCATIONS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENADAPTERFROMDEVICENAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENADAPTERFROMDEVICENAME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENADAPTERFROMGDIDISPLAYNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENADAPTERFROMGDIDISPLAYNAME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENADAPTERFROMHDC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENADAPTERFROMHDC", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENADAPTERFROMLUID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENADAPTERFROMLUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENKEYEDMUTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENKEYEDMUTEX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENKEYEDMUTEX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENKEYEDMUTEX2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENKEYEDMUTEXFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENNATIVEFENCEFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENNATIVEFENCEFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENNTHANDLEFROMNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENNTHANDLEFROMNAME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENPROTECTEDSESSIONFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENPROTECTEDSESSIONFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENRESOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENRESOURCE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENRESOURCE2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENRESOURCE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENRESOURCEFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENRESOURCEFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENSYNCHRONIZATIONOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENSYNCHRONIZATIONOBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENSYNCOBJECTFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENSYNCOBJECTFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENSYNCOBJECTFROMNTHANDLE2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENSYNCOBJECTFROMNTHANDLE2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OPENSYNCOBJECTNTHANDLEFROMNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OPENSYNCOBJECTNTHANDLEFROMNAME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OUTPUTDUPLGETFRAMEINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OUTPUTDUPL_GET_FRAMEINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OUTPUTDUPLGETMETADATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OUTPUTDUPL_METADATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OUTPUTDUPLGETPOINTERSHAPEDATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OUTPUTDUPL_GET_POINTER_SHAPE_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OUTPUTDUPLPRESENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OUTPUTDUPLPRESENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OUTPUTDUPLPRESENTTOHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OUTPUTDUPLPRESENTTOHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_OUTPUTDUPLRELEASEFRAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_OUTPUTDUPL_RELEASE_FRAME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_PINDIRECTFLIPRESOURCES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_PINDIRECTFLIPRESOURCES", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_POLLDISPLAYCHILDREN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_POLLDISPLAYCHILDREN", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_PRESENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_PRESENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_PRESENTMULTIPLANEOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_PRESENT_MULTIPLANE_OVERLAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_PRESENTMULTIPLANEOVERLAY2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_PRESENT_MULTIPLANE_OVERLAY2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_PRESENTMULTIPLANEOVERLAY3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_PRESENT_MULTIPLANE_OVERLAY3", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYADAPTERINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYADAPTERINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYALLOCATIONRESIDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYALLOCATIONRESIDENCY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYCLOCKCALIBRATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYCLOCKCALIBRATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYFSEBLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYFSEBLOCK", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYHYBRIDLISTVALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_HYBRID_LIST", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYPROCESSOFFERINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYPROCESSOFFERINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYPROTECTEDSESSIONINFOFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYPROTECTEDSESSIONINFOFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYPROTECTEDSESSIONSTATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYPROTECTEDSESSIONSTATUS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYREMOTEVIDPNSOURCEFROMGDIDISPLAYNAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYREMOTEVIDPNSOURCEFROMGDIDISPLAYNAME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYRESOURCEINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYRESOURCEINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYRESOURCEINFOFROMNTHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYRESOURCEINFOFROMNTHANDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYSTATISTICS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYSTATISTICS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYVIDEOMEMORYINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYVIDEOMEMORYINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_QUERYVIDPNEXCLUSIVEOWNERSHIP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_QUERYVIDPNEXCLUSIVEOWNERSHIP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RECLAIMALLOCATIONS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_RECLAIMALLOCATIONS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RECLAIMALLOCATIONS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_RECLAIMALLOCATIONS2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_REGISTERBUDGETCHANGENOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_REGISTERBUDGETCHANGENOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_REGISTERTRIMNOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_REGISTERTRIMNOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RELEASEKEYEDMUTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_RELEASEKEYEDMUTEX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RELEASEKEYEDMUTEX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_RELEASEKEYEDMUTEX2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RELEASEPROCESSVIDPNSOURCEOWNERS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RENDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_RENDER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_RESERVEGPUVIRTUALADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DDDI_RESERVEGPUVIRTUALADDRESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETALLOCATIONPRIORITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETALLOCATIONPRIORITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETCONTEXTINPROCESSSCHEDULINGPRIORITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETCONTEXTINPROCESSSCHEDULINGPRIORITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETCONTEXTSCHEDULINGPRIORITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETCONTEXTSCHEDULINGPRIORITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETDISPLAYMODE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETDISPLAYMODE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETDISPLAYPRIVATEDRIVERFORMAT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETDISPLAYPRIVATEDRIVERFORMAT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETFSEBLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETFSEBLOCK", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETGAMMARAMP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETGAMMARAMP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETHWPROTECTIONTEARDOWNRECOVERY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETHWPROTECTIONTEARDOWNRECOVERY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETHYBRIDLISTVVALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_HYBRID_LIST", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETPROCESSSCHEDULINGPRIORITYCLASS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="D3DKMT_SCHEDULINGPRIORITYCLASS")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFND3DKMT_SETQUEUEDLIMIT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETQUEUEDLIMIT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETSTABLEPOWERSTATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETSTABLEPOWERSTATE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETSTEREOENABLED": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETSYNCREFRESHCOUNTWAITTARGET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETSYNCREFRESHCOUNTWAITTARGET", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETVIDPNSOURCEHWPROTECTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETVIDPNSOURCEHWPROTECTION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETVIDPNSOURCEOWNER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETVIDPNSOURCEOWNER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETVIDPNSOURCEOWNER1": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETVIDPNSOURCEOWNER1", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SETVIDPNSOURCEOWNER2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SETVIDPNSOURCEOWNER2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SHAREDPRIMARYLOCKNOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SHAREDPRIMARYLOCKNOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SHAREDPRIMARYUNLOCKNOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SHAREDPRIMARYUNLOCKNOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SHAREOBJECTS": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypePointer(SimTypeRef("OBJECT_ATTRIBUTES", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cObjects", "hObjects", "pObjectAttributes", "dwDesiredAccess", "phSharedNtHandle"]), offset=0),
    "PFND3DKMT_SIGNALSYNCHRONIZATIONOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SIGNALSYNCHRONIZATIONOBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SIGNALSYNCHRONIZATIONOBJECT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SIGNALSYNCHRONIZATIONOBJECT2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMGPU2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SUBMITCOMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SUBMITCOMMAND", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SUBMITCOMMANDTOHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SUBMITCOMMANDTOHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SUBMITPRESENTBLTTOHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SUBMITPRESENTBLTTOHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SUBMITPRESENTTOHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SUBMITPRESENTTOHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SUBMITSIGNALSYNCOBJECTSTOHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SUBMITSIGNALSYNCOBJECTSTOHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_SUBMITWAITFORSYNCOBJECTSTOHWQUEUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_SUBMITWAITFORSYNCOBJECTSTOHWQUEUE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_TRIMNOTIFICATIONCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_TRIMNOTIFICATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_TRIMPROCESSCOMMITMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_TRIMPROCESSCOMMITMENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UNLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UNLOCK", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UNLOCK2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UNLOCK2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UNPINDIRECTFLIPRESOURCES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UNPINDIRECTFLIPRESOURCES", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UNREGISTERBUDGETCHANGENOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UNREGISTERBUDGETCHANGENOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UNREGISTERTRIMNOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UNREGISTERTRIMNOTIFICATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UPDATEALLOCATIONPROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DDDI_UPDATEALLOCPROPERTY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UPDATEGPUVIRTUALADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UPDATEGPUVIRTUALADDRESS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_UPDATEOVERLAY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_UPDATEOVERLAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORIDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORIDLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORSYNCHRONIZATIONOBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORSYNCHRONIZATIONOBJECT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORSYNCHRONIZATIONOBJECT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORSYNCHRONIZATIONOBJECT2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMGPU": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMGPU", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORVERTICALBLANKEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORVERTICALBLANKEVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DKMT_WAITFORVERTICALBLANKEVENT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3DKMT_WAITFORVERTICALBLANKEVENT2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFND3DNTPARSEUNKNOWNCOMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpvCommands", "lplpvReturnedCommand"]), offset=0),
    "PFND3DPARSEUNKNOWNCOMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpvCommands", "lplpvReturnedCommand"]), offset=0),
    "PFNDACOMPARE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["p1", "p2", "lParam"]), offset=0),
    "PFNDACOMPARECONST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["p1", "p2", "lParam"]), offset=0),
    "PFNDAENUMCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["p", "pData"]), offset=0),
    "PFNDAENUMCALLBACKCONST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["p", "pData"]), offset=0),
    "PFNDAVAUTHCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DAV_CALLBACK_CRED", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="AUTHNEXTSTEP"), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pbuffer"]), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpwzServerName", "lpwzRemoteName", "dwAuthScheme", "dwFlags", "pCallbackCred", "NextStep", "pFreeCred"]), offset=0),
    "PFNDAVAUTHCALLBACK_FREECRED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pbuffer"]), offset=0),
    "PFNDPAMERGE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="DPAMM_MESSAGE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["uMsg", "pvDest", "pvSrc", "lParam"]), offset=0),
    "PFNDPAMERGECONST": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="DPAMM_MESSAGE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["uMsg", "pvDest", "pvSrc", "lParam"]), offset=0),
    "PFNDPASTREAM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DPASTREAMINFO", SimStruct), offset=0), SimTypeBottom(label="IStream"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pinfo", "pstream", "pvInstData"]), offset=0),
    "PFNDSCREATEISECINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="ISecurityInformation"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFNDSCREATEISECINFOEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="ISecurityInformation"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9"]), offset=0),
    "PFNDSCREATESECPAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFNDSEDITSECURITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFNFCIALLOC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0),
    "PFNFCICLOSE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hf", "err", "pv"]), offset=0),
    "PFNFCIDELETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszFile", "err", "pv"]), offset=0),
    "PFNFCIFILEPLACED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CCAB", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pccab", "pszFile", "cbFile", "fContinuation", "pv"]), offset=0),
    "PFNFCIFREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["memory"]), offset=0),
    "PFNFCIGETNEXTCABINET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CCAB", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pccab", "cbPrevCab", "pv"]), offset=0),
    "PFNFCIGETOPENINFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pszName", "pdate", "ptime", "pattribs", "err", "pv"]), offset=0),
    "PFNFCIGETTEMPFILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszTempName", "cbTempName", "pv"]), offset=0),
    "PFNFCIOPEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pszFile", "oflag", "pmode", "err", "pv"]), offset=0),
    "PFNFCIREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "memory", "cb", "err", "pv"]), offset=0),
    "PFNFCISEEK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hf", "dist", "seektype", "err", "pv"]), offset=0),
    "PFNFCISTATUS": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["typeStatus", "cb1", "cb2", "pv"]), offset=0),
    "PFNFCIWRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "memory", "cb", "err", "pv"]), offset=0),
    "PFNFDIDECRYPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FDIDECRYPT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pfdid"]), offset=0),
    "PFNFDINOTIFY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="FDINOTIFICATIONTYPE"), SimTypePointer(SimTypeRef("FDINOTIFICATION", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["fdint", "pfdin"]), offset=0),
    "PFNFILLTEXTBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("TEXT_SOURCE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTextSource"]), offset=0),
    "PFNFREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0),
    "PFNFTH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SYSTEM_FIRMWARE_TABLE_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SystemFirmwareTableInfo"]), offset=0),
    "PFNGENERATEGROUPPOLICY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("RSOP_TARGET", SimStruct), offset=0), SimTypePointer(SimTypeRef("RSOP_TARGET", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwFlags", "pbAbort", "pwszSite", "pComputerTarget", "pUserTarget"]), offset=0),
    "PFNGLADDSWAPHINTRECTWINPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["x", "y", "width", "height"]), offset=0),
    "PFNGLARRAYELEMENTARRAYEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["mode", "count", "pi"]), offset=0),
    "PFNGLARRAYELEMENTEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["i"]), offset=0),
    "PFNGLCOLORPOINTEREXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["size", "type", "stride", "count", "pointer"]), offset=0),
    "PFNGLCOLORSUBTABLEEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["target", "start", "count", "format", "type", "data"]), offset=0),
    "PFNGLCOLORTABLEEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["target", "internalFormat", "width", "format", "type", "data"]), offset=0),
    "PFNGLDRAWARRAYSEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["mode", "first", "count"]), offset=0),
    "PFNGLDRAWRANGEELEMENTSWINPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["mode", "start", "end", "count", "type", "indices"]), offset=0),
    "PFNGLEDGEFLAGPOINTEREXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["stride", "count", "pointer"]), offset=0),
    "PFNGLGETCOLORTABLEEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["target", "format", "type", "data"]), offset=0),
    "PFNGLGETCOLORTABLEPARAMETERFVEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFloat(size=32), offset=0)], SimTypeBottom(label="Void"), arg_names=["target", "pname", "params"]), offset=0),
    "PFNGLGETCOLORTABLEPARAMETERIVEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["target", "pname", "params"]), offset=0),
    "PFNGLGETPOINTERVEXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["pname", "params"]), offset=0),
    "PFNGLINDEXPOINTEREXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["type", "stride", "count", "pointer"]), offset=0),
    "PFNGLNORMALPOINTEREXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["type", "stride", "count", "pointer"]), offset=0),
    "PFNGLTEXCOORDPOINTEREXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["size", "type", "stride", "count", "pointer"]), offset=0),
    "PFNGLVERTEXPOINTEREXTPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["size", "type", "stride", "count", "pointer"]), offset=0),
    "PFNIDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFNLOG": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IMEDP", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFNLVCOMPARE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFNLVGROUPCOMPARE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFNMSGECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["bVerbose", "lpMessage"]), offset=0),
    "PFNOPEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pszFile", "oflag", "pmode"]), offset=0),
    "PFNPEER_FREE_SECURITY_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PEER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext", "pSecurityData"]), offset=0),
    "PFNPEER_ON_PASSWORD_AUTH_FAILED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext"]), offset=0),
    "PFNPEER_SECURE_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PEER_RECORD", SimStruct), offset=0), SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE"), SimTypePointer(SimTypePointer(SimTypeRef("PEER_DATA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext", "pRecord", "changeType", "ppSecurityData"]), offset=0),
    "PFNPEER_VALIDATE_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("PEER_RECORD", SimStruct), offset=0), SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE")], SimTypeInt(signed=True, label="Int32"), arg_names=["hGraph", "pvContext", "pRecord", "changeType"]), offset=0),
    "PFNPROCESSGROUPPOLICY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0), SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["bVerbose", "lpMessage"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwFlags", "hToken", "hKeyRoot", "pDeletedGPOList", "pChangedGPOList", "pHandle", "pbAbort", "pStatusCallback"]), offset=0),
    "PFNPROCESSGROUPPOLICYEX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0), SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["bVerbose", "lpMessage"]), offset=0), SimTypeBottom(label="IWbemServices"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwFlags", "hToken", "hKeyRoot", "pDeletedGPOList", "pChangedGPOList", "pHandle", "pbAbort", "pStatusCallback", "pWbemServices", "pRsopStatus"]), offset=0),
    "PFNPROPSHEETCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFNPROPSHEETUI": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PROPSHEETUI_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPSUIInfo", "lParam"]), offset=0),
    "PFNProgressNotification": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="WICProgressOperation"), SimTypeFloat(size=64)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvData", "uFrameNum", "operation", "dblProgress"]), offset=0),
    "PFNRASFREEBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pBufer"]), offset=0),
    "PFNRASGETBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppBuffer", "pdwSize"]), offset=0),
    "PFNRASRECEIVEBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pBuffer", "pdwSize", "dwTimeOut", "hEvent"]), offset=0),
    "PFNRASRETRIEVEBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pBuffer", "pdwSize"]), offset=0),
    "PFNRASSENDBUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pBuffer", "dwSize"]), offset=0),
    "PFNRASSETCOMMSETTINGS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RASCOMMSETTINGS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pRasCommSettings", "pvReserved"]), offset=0),
    "PFNREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "pv", "cb"]), offset=0),
    "PFNREADOBJECTSECURITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFNSEEK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hf", "dist", "seektype"]), offset=0),
    "PFNSETSHELLNEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PFNSHOWSHAREFOLDERUIW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pszPath"]), offset=0),
    "PFNSNMPCLEANUPEX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PFNSNMPEXTENSIONCLOSE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PFNSNMPEXTENSIONINIT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("AsnObjectIdentifier", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwUpTimeReference", "phSubagentTrapEvent", "pFirstSupportedRegion"]), offset=0),
    "PFNSNMPEXTENSIONINITEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("AsnObjectIdentifier", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pNextSupportedRegion"]), offset=0),
    "PFNSNMPEXTENSIONMONITOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAgentMgmtData"]), offset=0),
    "PFNSNMPEXTENSIONQUERY": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("SnmpVarBindList", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["bPduType", "pVarBindList", "pErrorStatus", "pErrorIndex"]), offset=0),
    "PFNSNMPEXTENSIONQUERYEX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SnmpVarBindList", SimStruct), offset=0), SimTypePointer(SimTypeRef("AsnOctetString", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["nRequestType", "nTransactionId", "pVarBindList", "pContextInfo", "pErrorStatus", "pErrorIndex"]), offset=0),
    "PFNSNMPEXTENSIONTRAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("AsnObjectIdentifier", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("SnmpVarBindList", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pEnterpriseOid", "pGenericTrapId", "pSpecificTrapId", "pTimeStamp", "pVarBindList"]), offset=0),
    "PFNSNMPSTARTUPEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFNSTATUSMESSAGECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["bVerbose", "lpMessage"]), offset=0),
    "PFNTRUSTHELPER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCertContext", "lCustData", "fLeafCertificate", "pbTrustBlob"]), offset=0),
    "PFNTVCOMPARE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lParam1", "lParam2", "lParamSort"]), offset=0),
    "PFNWRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "pv", "cb"]), offset=0),
    "PFNWRITEOBJECTSECURITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_ALLOCANDFILLDEFUSAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DEFUSAGE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszUsageOID", "psDefUsage"]), offset=0),
    "PFN_AUTHENTICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("BLUETOOTH_DEVICE_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvParam", "pDevice"]), offset=0),
    "PFN_AUTHENTICATION_CALLBACK_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("BLUETOOTH_AUTHENTICATION_CALLBACK_PARAMS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvParam", "pAuthCallbackParams"]), offset=0),
    "PFN_AUTHENTICODE_DIGEST_SIGN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSigningCert", "pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest"]), offset=0),
    "PFN_AUTHENTICODE_DIGEST_SIGN_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest", "ppSignerCert", "hCertChainStore"]), offset=0),
    "PFN_AUTHENTICODE_DIGEST_SIGN_EX_WITHFILEHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "hFile", "pSignedDigest", "ppSignerCert", "hCertChainStore"]), offset=0),
    "PFN_AUTHENTICODE_DIGEST_SIGN_WITHFILEHANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSigningCert", "pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "hFile", "pSignedDigest"]), offset=0),
    "PFN_AUTHZ_COMPUTE_DYNAMIC_GROUPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAuthzClientContext", "Args", "pSidAttrArray", "pSidCount", "pRestrictedSidAttrArray", "pRestrictedSidCount"]), offset=0),
    "PFN_AUTHZ_DYNAMIC_ACCESS_CHECK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ACE_HEADER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAuthzClientContext", "pAce", "pArgs", "pbAceApplicable"]), offset=0),
    "PFN_AUTHZ_FREE_CENTRAL_ACCESS_POLICY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pCentralAccessPolicy"]), offset=0),
    "PFN_AUTHZ_FREE_DYNAMIC_GROUPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pSidAttrArray"]), offset=0),
    "PFN_AUTHZ_GET_CENTRAL_ACCESS_POLICY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAuthzClientContext", "capid", "pArgs", "pCentralAccessPolicyApplicable", "ppCentralAccessPolicy"]), offset=0),
    "PFN_AUTH_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_BLUETOOTH_ENUM_ATTRIBUTES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["uAttribId", "pValueStream", "cbStreamSize", "pvParam"]), offset=0),
    "PFN_CANCEL_ASYNC_RETRIEVAL_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hAsyncRetrieve"]), offset=0),
    "PFN_CDF_PARSE_ERROR_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwErrorArea", "dwLocalError", "pwszLine"]), offset=0),
    "PFN_CERT_CHAIN_FIND_BY_ISSUER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCert", "pvFindArg"]), offset=0),
    "PFN_CERT_CREATE_CONTEXT_SORT_FUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cbTotalEncoded", "cbRemainEncoded", "cEntry", "pvSort"]), offset=0),
    "PFN_CERT_DLL_OPEN_STORE_PROV_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="CERT_OPEN_STORE_FLAGS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_STORE_PROV_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszStoreProvider", "dwEncodingType", "hCryptProv", "dwFlags", "pvPara", "hCertStore", "pStoreProvInfo"]), offset=0),
    "PFN_CERT_ENUM_PHYSICAL_STORE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("CERT_PHYSICAL_STORE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvSystemStore", "dwFlags", "pwszStoreName", "pStoreInfo", "pvReserved", "pvArg"]), offset=0),
    "PFN_CERT_ENUM_SYSTEM_STORE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CERT_SYSTEM_STORE_FLAGS"), SimTypePointer(SimTypeRef("CERT_SYSTEM_STORE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvSystemStore", "dwFlags", "pStoreInfo", "pvReserved", "pvArg"]), offset=0),
    "PFN_CERT_ENUM_SYSTEM_STORE_LOCATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszStoreLocation", "dwFlags", "pvReserved", "pvArg"]), offset=0),
    "PFN_CERT_IS_WEAK_HASH": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwHashUseType", "pwszCNGHashAlgid", "dwChainFlags", "pSignerChainContext", "pTimeStamp", "pwszFileName"]), offset=0),
    "PFN_CERT_SERVER_OCSP_RESPONSE_UPDATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CERT_SERVER_OCSP_RESPONSE_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pChainContext", "pServerOcspResponseContext", "pNewCrlContext", "pPrevCrlContext", "pvArg", "dwWriteOcspFileError"]), offset=0),
    "PFN_CERT_STORE_PROV_CLOSE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hStoreProv", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_CONTROL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "dwFlags", "dwCtrlType", "pvCtrlPara"]), offset=0),
    "PFN_CERT_STORE_PROV_DELETE_CERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCertContext", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_DELETE_CRL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCrlContext", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_DELETE_CTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCtlContext", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_FIND_CERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_STORE_PROV_FIND_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pFindInfo", "pPrevCertContext", "dwFlags", "ppvStoreProvFindInfo", "ppProvCertContext"]), offset=0),
    "PFN_CERT_STORE_PROV_FIND_CRL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_STORE_PROV_FIND_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pFindInfo", "pPrevCrlContext", "dwFlags", "ppvStoreProvFindInfo", "ppProvCrlContext"]), offset=0),
    "PFN_CERT_STORE_PROV_FIND_CTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_STORE_PROV_FIND_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pFindInfo", "pPrevCtlContext", "dwFlags", "ppvStoreProvFindInfo", "ppProvCtlContext"]), offset=0),
    "PFN_CERT_STORE_PROV_FREE_FIND_CERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCertContext", "pvStoreProvFindInfo", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_FREE_FIND_CRL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCrlContext", "pvStoreProvFindInfo", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_FREE_FIND_CTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCtlContext", "pvStoreProvFindInfo", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_GET_CERT_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCertContext", "dwPropId", "dwFlags", "pvData", "pcbData"]), offset=0),
    "PFN_CERT_STORE_PROV_GET_CRL_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCrlContext", "dwPropId", "dwFlags", "pvData", "pcbData"]), offset=0),
    "PFN_CERT_STORE_PROV_GET_CTL_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCtlContext", "dwPropId", "dwFlags", "pvData", "pcbData"]), offset=0),
    "PFN_CERT_STORE_PROV_READ_CERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pStoreCertContext", "dwFlags", "ppProvCertContext"]), offset=0),
    "PFN_CERT_STORE_PROV_READ_CRL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pStoreCrlContext", "dwFlags", "ppProvCrlContext"]), offset=0),
    "PFN_CERT_STORE_PROV_READ_CTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pStoreCtlContext", "dwFlags", "ppProvCtlContext"]), offset=0),
    "PFN_CERT_STORE_PROV_SET_CERT_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCertContext", "dwPropId", "dwFlags", "pvData"]), offset=0),
    "PFN_CERT_STORE_PROV_SET_CRL_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCrlContext", "dwPropId", "dwFlags", "pvData"]), offset=0),
    "PFN_CERT_STORE_PROV_SET_CTL_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCtlContext", "dwPropId", "dwFlags", "pvData"]), offset=0),
    "PFN_CERT_STORE_PROV_WRITE_CERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCertContext", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_WRITE_CRL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCrlContext", "dwFlags"]), offset=0),
    "PFN_CERT_STORE_PROV_WRITE_CTL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hStoreProv", "pCtlContext", "dwFlags"]), offset=0),
    "PFN_CMSG_ALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0),
    "PFN_CMSG_CNG_IMPORT_CONTENT_ENCRYPT_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CNG_CONTENT_DECRYPT_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCNGContentDecryptInfo", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_CNG_IMPORT_KEY_AGREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CNG_CONTENT_DECRYPT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_CTRL_KEY_AGREE_DECRYPT_PARA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCNGContentDecryptInfo", "pKeyAgreeDecryptPara", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_CNG_IMPORT_KEY_TRANS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CNG_CONTENT_DECRYPT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_CTRL_KEY_TRANS_DECRYPT_PARA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCNGContentDecryptInfo", "pKeyTransDecryptPara", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_EXPORT_ENCRYPT_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hCryptProv", "hEncryptKey", "pPublicKeyInfo", "pbData", "pcbData"]), offset=0),
    "PFN_CMSG_EXPORT_KEY_AGREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CONTENT_ENCRYPT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_KEY_AGREE_RECIPIENT_ENCODE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_KEY_AGREE_ENCRYPT_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptInfo", "pKeyAgreeEncodeInfo", "pKeyAgreeEncryptInfo", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_EXPORT_KEY_TRANS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CONTENT_ENCRYPT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_KEY_TRANS_RECIPIENT_ENCODE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_KEY_TRANS_ENCRYPT_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptInfo", "pKeyTransEncodeInfo", "pKeyTransEncryptInfo", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_EXPORT_MAIL_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CONTENT_ENCRYPT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_MAIL_LIST_RECIPIENT_ENCODE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_MAIL_LIST_ENCRYPT_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptInfo", "pMailListEncodeInfo", "pMailListEncryptInfo", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0),
    "PFN_CMSG_GEN_CONTENT_ENCRYPT_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CMSG_CONTENT_ENCRYPT_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptInfo", "dwFlags", "pvReserved"]), offset=0),
    "PFN_CMSG_GEN_ENCRYPT_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phCryptProv", "paiEncrypt", "pvEncryptAuxInfo", "pPublicKeyInfo", "pfnAlloc", "phEncryptKey", "ppbEncryptParameters", "pcbEncryptParameters"]), offset=0),
    "PFN_CMSG_IMPORT_ENCRYPT_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hCryptProv", "dwKeySpec", "paiEncrypt", "paiPubKey", "pbEncodedKey", "cbEncodedKey", "phEncryptKey"]), offset=0),
    "PFN_CMSG_IMPORT_KEY_AGREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_CTRL_KEY_AGREE_DECRYPT_PARA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptionAlgorithm", "pKeyAgreeDecryptPara", "dwFlags", "pvReserved", "phContentEncryptKey"]), offset=0),
    "PFN_CMSG_IMPORT_KEY_TRANS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_CTRL_KEY_TRANS_DECRYPT_PARA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptionAlgorithm", "pKeyTransDecryptPara", "dwFlags", "pvReserved", "phContentEncryptKey"]), offset=0),
    "PFN_CMSG_IMPORT_MAIL_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CMSG_CTRL_MAIL_LIST_DECRYPT_PARA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContentEncryptionAlgorithm", "pMailListDecryptPara", "dwFlags", "pvReserved", "phContentEncryptKey"]), offset=0),
    "PFN_CMSG_STREAM_OUTPUT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvArg", "pbData", "cbData", "fFinal"]), offset=0),
    "PFN_COMPRESS_ALLOCATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["UserContext", "Size"]), offset=0),
    "PFN_COMPRESS_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["UserContext", "Memory"]), offset=0),
    "PFN_CPD_ADD_CERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "idxSigner", "fCounterSigner", "idxCounterSigner", "pCert2Add"]), offset=0),
    "PFN_CPD_ADD_PRIVDATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_PRIVDATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "pPrivData2Add"]), offset=0),
    "PFN_CPD_ADD_SGNR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_SGNR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "fCounterSigner", "idxSigner", "pSgnr2Add"]), offset=0),
    "PFN_CPD_ADD_STORE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "hStore2Add"]), offset=0),
    "PFN_CPD_MEM_ALLOC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0),
    "PFN_CPD_MEM_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvMem2Free"]), offset=0),
    "PFN_CRYPT_ALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0),
    "PFN_CRYPT_ASYNC_PARAM_FREE_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pszParamOid", "pvParam"]), offset=0),
    "PFN_CRYPT_ASYNC_RETRIEVAL_COMPLETION_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvCompletion", "dwCompletionCode", "pszUrl", "pszObjectOid", "pvObject"]), offset=0),
    "PFN_CRYPT_CANCEL_RETRIEVAL": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwFlags", "pvArg"]), offset=0),
    "PFN_CRYPT_ENUM_KEYID_PROP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pKeyIdentifier", "dwFlags", "pvReserved", "pvArg", "cProp", "rgdwPropId", "rgpvData", "rgcbData"]), offset=0),
    "PFN_CRYPT_ENUM_OID_FUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwEncodingType", "pszFuncName", "pszOID", "cValue", "rgdwValueType", "rgpwszValueName", "rgpbValueData", "rgcbValueData", "pvArg"]), offset=0),
    "PFN_CRYPT_ENUM_OID_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_OID_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pInfo", "pvArg"]), offset=0),
    "PFN_CRYPT_EXPORT_PUBLIC_KEY_INFO_EX2_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNCryptKey", "dwCertEncodingType", "pszPublicKeyObjId", "dwFlags", "pvAuxInfo", "pInfo", "pcbInfo"]), offset=0),
    "PFN_CRYPT_EXPORT_PUBLIC_KEY_INFO_FROM_BCRYPT_HANDLE_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hBCryptKey", "dwCertEncodingType", "pszPublicKeyObjId", "dwFlags", "pvAuxInfo", "pInfo", "pcbInfo"]), offset=0),
    "PFN_CRYPT_EXTRACT_ENCODED_SIGNATURE_PARAMETERS_FUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCertEncodingType", "pSignatureAlgorithm", "ppvDecodedSignPara", "ppwszCNGHashAlgid"]), offset=0),
    "PFN_CRYPT_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0),
    "PFN_CRYPT_GET_SIGNER_CERTIFICATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeRef("CERT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), arg_names=["pvGetArg", "dwCertEncodingType", "pSignerId", "hMsgCertStore"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_FLUSH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "rgIdentifierOrNameList", "dwIdentifierOrNameListCount"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPluginContext", "pbData"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_FREE_IDENTIFIER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPluginContext", "pIdentifier"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_FREE_PASSWORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pPluginContext", "pwszPassword"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_GET": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPluginContext", "pIdentifier", "dwNameType", "pNameBlob", "ppbContent", "pcbContent", "ppwszPassword", "ppIdentifier"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_INITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "rgIdentifierOrNameList", "dwIdentifierOrNameListCount"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CRYPT_OBJECT_LOCATOR_PROVIDER_TABLE", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pfnFlush", "pContext", "pdwExpectedObjectCount", "ppFuncTable", "ppPluginContext"]), offset=0),
    "PFN_CRYPT_OBJECT_LOCATOR_PROVIDER_RELEASE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="CRYPT_OBJECT_LOCATOR_RELEASE_REASON"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwReason", "pPluginContext"]), offset=0),
    "PFN_CRYPT_SIGN_AND_ENCODE_HASH_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hKey", "dwCertEncodingType", "pSignatureAlgorithm", "pvDecodedSignPara", "pwszCNGPubKeyAlgid", "pwszCNGHashAlgid", "pbComputedHash", "cbComputedHash", "pbSignature", "pcbSignature"]), offset=0),
    "PFN_CRYPT_VERIFY_ENCODED_SIGNATURE_FUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCertEncodingType", "pPubKeyInfo", "pSignatureAlgorithm", "pvDecodedSignPara", "pwszCNGPubKeyAlgid", "pwszCNGHashAlgid", "pbComputedHash", "cbComputedHash", "pbSignature", "cbSignature"]), offset=0),
    "PFN_CRYPT_XML_CREATE_TRANSFORM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_XML_DATA_PROVIDER", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_XML_DATA_PROVIDER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTransform", "pProviderIn", "pProviderOut"]), offset=0),
    "PFN_CRYPT_XML_DATA_PROVIDER_CLOSE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState"]), offset=0),
    "PFN_CRYPT_XML_DATA_PROVIDER_READ": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState", "pbData", "cbData", "pcbRead"]), offset=0),
    "PFN_CRYPT_XML_ENUM_ALG_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_XML_ALGORITHM_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pInfo", "pvArg"]), offset=0),
    "PFN_CRYPT_XML_WRITE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvCallbackState", "pbData", "cbData"]), offset=0),
    "PFN_CUSTOM_HELP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hModule", "pwszCmdToken"]), offset=0),
    "PFN_D3D10_CREATE_DEVICE1": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDXGIAdapter"), SimTypeInt(signed=False, label="D3D10_DRIVER_TYPE"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="D3D10_FEATURE_LEVEL1"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="ID3D10Device1"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_D3D10_CREATE_DEVICE_AND_SWAP_CHAIN1": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDXGIAdapter"), SimTypeInt(signed=False, label="D3D10_DRIVER_TYPE"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="D3D10_FEATURE_LEVEL1"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXGI_SWAP_CHAIN_DESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="IDXGISwapChain"), offset=0), SimTypePointer(SimTypeBottom(label="ID3D10Device1"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "PFN_D3D11ON12_CREATE_DEVICE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="IUnknown"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="ID3D11Device"), offset=0), SimTypePointer(SimTypeBottom(label="ID3D11DeviceContext"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "FeatureLevels", "param4", "NumQueues", "param6", "param7", "param8", "param9"]), offset=0),
    "PFN_D3D11_CREATE_DEVICE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDXGIAdapter"), SimTypeInt(signed=False, label="D3D_DRIVER_TYPE"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="ID3D11Device"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), offset=0), SimTypePointer(SimTypeBottom(label="ID3D11DeviceContext"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "FeatureLevels", "param6", "param7", "param8", "param9"]), offset=0),
    "PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDXGIAdapter"), SimTypeInt(signed=False, label="D3D_DRIVER_TYPE"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DXGI_SWAP_CHAIN_DESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="IDXGISwapChain"), offset=0), SimTypePointer(SimTypeBottom(label="ID3D11Device"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), offset=0), SimTypePointer(SimTypeBottom(label="ID3D11DeviceContext"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "FeatureLevels", "param6", "param7", "param8", "param9", "param10", "param11"]), offset=0),
    "PFN_D3D12_CREATE_DEVICE": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeInt(signed=False, label="D3D_FEATURE_LEVEL"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_D3D12_CREATE_ROOT_SIGNATURE_DESERIALIZER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSizeInBytes", "pRootSignatureDeserializerInterface", "ppRootSignatureDeserializer"]), offset=0),
    "PFN_D3D12_CREATE_VERSIONED_ROOT_SIGNATURE_DESERIALIZER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSizeInBytes", "pRootSignatureDeserializerInterface", "ppRootSignatureDeserializer"]), offset=0),
    "PFN_D3D12_GET_DEBUG_INTERFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_D3D12_GET_INTERFACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_D3D12_SERIALIZE_ROOT_SIGNATURE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3D12_ROOT_SIGNATURE_DESC", SimStruct), offset=0), SimTypeInt(signed=False, label="D3D_ROOT_SIGNATURE_VERSION"), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pRootSignature", "Version", "ppBlob", "ppErrorBlob"]), offset=0),
    "PFN_D3D12_SERIALIZE_VERSIONED_ROOT_SIGNATURE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("D3D12_VERSIONED_ROOT_SIGNATURE_DESC", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pRootSignature", "ppBlob", "ppErrorBlob"]), offset=0),
    "PFN_DESTRUCTION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pData"]), offset=0),
    "PFN_DEVICE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("BLUETOOTH_DEVICE_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvParam", "pDevice"]), offset=0),
    "PFN_DIAL_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DXCORE_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="DXCoreNotificationType"), SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["notificationType", "object", "context"]), offset=0),
    "PFN_Direct3DCreate9On12": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("D3D9ON12_ARGS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="IDirect3D9"), arg_names=["SDKVersion", "pOverrideList", "NumOverrideEntries"]), offset=0),
    "PFN_Direct3DCreate9On12Ex": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("D3D9ON12_ARGS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="IDirect3D9Ex"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SDKVersion", "pOverrideList", "NumOverrideEntries", "ppOutputInterface"]), offset=0),
    "PFN_DrvAccumulateD3DDirtyRect": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CDDDXGK_REDIRBITMAPPRESENTINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvAlphaBlend": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("BLENDOBJ", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_DrvAssertMode": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvAssociateSharedSurface": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeRef("SIZE", SimStruct)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvBitBlt": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10"]), offset=0),
    "PFN_DrvCompletePDEV": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvCopyBits": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_DrvCreateDeviceBitmap": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeRef("SIZE", SimStruct), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvCreateDeviceBitmapEx": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeRef("SIZE", SimStruct), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_DrvDeleteDeviceBitmap": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_DrvDeleteDeviceBitmapEx": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_DrvDeriveSurface": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DD_DIRECTDRAW_GLOBAL", SimStruct), offset=0), SimTypePointer(SimTypeRef("DD_SURFACE_LOCAL", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvDescribePixelFormat": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("PIXELFORMATDESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvDestroyFont": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_DrvDisableDirectDraw": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_DrvDisableDriver": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PFN_DrvDisablePDEV": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_DrvDisableSurface": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_DrvDitherColor": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvDrawEscape": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_DrvEnableDirectDraw": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DD_CALLBACKS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DD_SURFACECALLBACKS", SimStruct), offset=0), SimTypePointer(SimTypeRef("DD_PALETTECALLBACKS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvEnableDriver": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DRVENABLEDATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvEnablePDEV": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("GDIINFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEVINFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10"]), offset=0),
    "PFN_DrvEnableSurface": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0"]), offset=0),
    "PFN_DrvEndDoc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pso", "fl"]), offset=0),
    "PFN_DrvEndDxInterop": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "KernelModeDeviceHandle"]), offset=0),
    "PFN_DrvEscape": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_DrvFillPath": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("PATHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_DrvFontManagement": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_DrvFree": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvGetDirectDrawInfo": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DD_HALINFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("VIDEOMEMORY", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_DrvGetDriverSetting": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pdriverobj", "Feature", "pOutput", "cbSize", "pcbNeeded", "pdwOptionsReturned"]), offset=0),
    "PFN_DrvGetGlyphMode": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dhpdev", "pfo"]), offset=0),
    "PFN_DrvGetModes": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvGetTrueTypeFile": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvGradientFill": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("TRIVERTEX", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9"]), offset=0),
    "PFN_DrvIcmCheckBitmapBits": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvIcmCreateColorTransform": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("LOGCOLORSPACEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "PFN_DrvIcmDeleteColorTransform": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvIcmSetDeviceGammaRamp": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvLineTo": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "PFN_DrvLoadFontFile": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("DESIGNVECTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_DrvLockDisplayArea": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvMovePointer": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pso", "x", "y", "prcl"]), offset=0),
    "PFN_DrvNextBand": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "ppointl"]), offset=0),
    "PFN_DrvNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvPaint": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_DrvPlgBlt": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("COLORADJUSTMENT", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTFIX", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10"]), offset=0),
    "PFN_DrvQueryAdvanceWidths": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_DrvQueryDeviceSupport": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XFORMOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_DrvQueryFont": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypePointer(SimTypeRef("IFIMETRICS", SimStruct), offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvQueryFontCaps": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvQueryFontData": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("GLYPHDATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_DrvQueryFontFile": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvQueryFontTree": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_DrvQueryGlyphAttrs": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeRef("FD_GLYPHATTR", SimStruct), offset=0), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvQueryPerBandInfo": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("PERBANDINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvQuerySpoolType": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dhpdev", "pwchType"]), offset=0),
    "PFN_DrvQueryTrueTypeOutline": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("GLYPHDATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("TTPOLYGONHEADER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_DrvQueryTrueTypeSection": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_DrvQueryTrueTypeTable": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_DrvRealizeBrush": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_DrvRenderHint": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dhpdev", "NotifyCode", "Length", "Data"]), offset=0),
    "PFN_DrvResetDevice": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvResetPDEV": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dhpdevOld", "dhpdevNew"]), offset=0),
    "PFN_DrvSaveScreenBits": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_DrvSendPage": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_DrvSetPalette": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("PALOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_DrvSetPixelFormat": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvSetPointerShape": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9"]), offset=0),
    "PFN_DrvStartBanding": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "ppointl"]), offset=0),
    "PFN_DrvStartDoc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pso", "pwszDocName", "dwJobId"]), offset=0),
    "PFN_DrvStartDxInterop": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "KernelModeDeviceHandle"]), offset=0),
    "PFN_DrvStartPage": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pso"]), offset=0),
    "PFN_DrvStretchBlt": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("COLORADJUSTMENT", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10"]), offset=0),
    "PFN_DrvStretchBltROP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("COLORADJUSTMENT", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10", "param11", "param12"]), offset=0),
    "PFN_DrvStrokeAndFillPath": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("PATHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XFORMOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("LINEATTRS", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9"]), offset=0),
    "PFN_DrvStrokePath": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("PATHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XFORMOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("LINEATTRS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_DrvSurfaceComplete": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvSwapBuffers": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("WNDOBJ", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvSynchronize": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvSynchronizeRedirectionBitmaps": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvSynchronizeSurface": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_DrvTextOut": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("STROBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("FONTOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("BRUSHOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("POINTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9"]), offset=0),
    "PFN_DrvTransparentBlt": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLIPOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("XLATEOBJ", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_DrvUnloadFontFile": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_DrvUnlockDisplayArea": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RECTL", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PFN_DrvUpdateUISetting": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pdriverobj", "pOptItem", "dwPreviousSelection", "dwMode"]), offset=0),
    "PFN_DrvUpgradeRegistrySetting": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPrinter", "pFeature", "pOption"]), offset=0),
    "PFN_EXPORT_PRIV_KEY_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CRYPT_PRIVATE_KEY_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hCryptProv", "dwKeySpec", "pszPrivateKeyObjId", "dwFlags", "pvAuxInfo", "pPrivateKeyInfo", "pcbPrivateKeyInfo"]), offset=0),
    "PFN_EngCombineRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hrgnTrg", "hrgnSrc1", "hrgnSrc2", "imode"]), offset=0),
    "PFN_EngCopyRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hrgnDst", "hrgnSrc"]), offset=0),
    "PFN_EngCreateRectRgn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["left", "top", "right", "bottom"]), offset=0),
    "PFN_EngDeleteRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hrgn"]), offset=0),
    "PFN_EngIntersectRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hrgnResult", "hRgnA", "hRgnB"]), offset=0),
    "PFN_EngSubtractRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hrgnResult", "hRgnA", "hRgnB"]), offset=0),
    "PFN_EngUnionRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hrgnResult", "hRgnA", "hRgnB"]), offset=0),
    "PFN_EngXorRgn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hrgnResult", "hRgnA", "hRgnB"]), offset=0),
    "PFN_FREEDEFUSAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DEFUSAGE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszUsageOID", "psDefUsage"]), offset=0),
    "PFN_FREE_ENCODED_OBJECT_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("CRYPT_BLOB_ARRAY", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pszObjectOid", "pObject", "pvFreeContext"]), offset=0),
    "PFN_FSRTLTEARDOWNPERSTREAMCONTEXTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FSRTL_ADVANCED_FCB_HEADER", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["AdvancedHeader"]), offset=0),
    "PFN_FWADDDYNAMICKEYWORDADDRESS0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FW_DYNAMIC_KEYWORD_ADDRESS0", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dynamicKeywordAddress"]), offset=0),
    "PFN_FWDELETEDYNAMICKEYWORDADDRESS0": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dynamicKeywordAddressId"]), offset=0),
    "PFN_FWENUMDYNAMICKEYWORDADDRESSBYID0": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypePointer(SimTypePointer(SimTypeRef("FW_DYNAMIC_KEYWORD_ADDRESS_DATA0", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dynamicKeywordAddressId", "dynamicKeywordAddressData"]), offset=0),
    "PFN_FWENUMDYNAMICKEYWORDADDRESSESBYTYPE0": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("FW_DYNAMIC_KEYWORD_ADDRESS_DATA0", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["flags", "dynamicKeywordAddressData"]), offset=0),
    "PFN_FWFREEDYNAMICKEYWORDADDRESSDATA0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FW_DYNAMIC_KEYWORD_ADDRESS_DATA0", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dynamicKeywordAddressData"]), offset=0),
    "PFN_FWUPDATEDYNAMICKEYWORDADDRESS0": SimTypePointer(SimTypeFunction([SimTypeBottom(label="Guid"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dynamicKeywordAddressId", "updatedAddresses", "append"]), offset=0),
    "PFN_GETEXTENSIONVERSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("HSE_VERSION_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVer"]), offset=0),
    "PFN_HANDLE_CMD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pwszMachine", "ppwcArguments", "dwCurrentIndex", "dwArgCount", "dwFlags", "pvData", "pbDone"]), offset=0),
    "PFN_HSE_CACHE_INVALIDATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszUrl"]), offset=0),
    "PFN_HSE_GET_PROTOCOL_MANAGER_CUSTOM_INTERFACE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszProtocolManagerDll", "pszProtocolManagerDllInitFunction", "dwCustomInterfaceId", "ppCustomInterface"]), offset=0),
    "PFN_HSE_IO_COMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXTENSION_CONTROL_BLOCK", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pECB", "pContext", "cbIO", "dwError"]), offset=0),
    "PFN_HTTPEXTENSIONPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXTENSION_CONTROL_BLOCK", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pECB"]), offset=0),
    "PFN_HidP_GetVersionInternal": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Version"]), offset=0),
    "PFN_IIS_GETSERVERVARIABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_IIS_READCLIENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_IIS_SERVERSUPPORTFUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_IIS_WRITECLIENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_IMPORT_PRIV_KEY_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_PRIVATE_KEY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hCryptProv", "pPrivateKeyInfo", "dwFlags", "pvAuxInfo"]), offset=0),
    "PFN_IMPORT_PUBLIC_KEY_INFO_EX2_FUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="CERT_QUERY_ENCODING_TYPE"), SimTypePointer(SimTypeRef("CERT_PUBLIC_KEY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCertEncodingType", "pInfo", "dwFlags", "pvAuxInfo", "phKey"]), offset=0),
    "PFN_IN_USE_PAGE_OFFLINE_NOTIFY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Page", "Flags", "Poisoned", "Context"]), offset=0),
    "PFN_IO_COMPLETION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FIO_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("FH_OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pContext", "lpo", "cb", "dwCompletionStatus"]), offset=0),
    "PFN_MAPPINGCALLBACKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MAPPING_PROPERTY_BAG", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["pBag", "data", "dwDataSize", "Result"]), offset=0),
    "PFN_NCRYPT_ALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cbSize"]), offset=0),
    "PFN_NCRYPT_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0),
    "PFN_NT_COMMIT_TRANSACTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["TransactionHandle", "Wait"]), offset=0),
    "PFN_NT_CREATE_TRANSACTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OBJECT_ATTRIBUTES", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["TransactionHandle", "DesiredAccess", "ObjectAttributes", "Uow", "TmHandle", "CreateOptions", "IsolationLevel", "IsolationFlags", "Timeout", "Description"]), offset=0),
    "PFN_NT_OPEN_TRANSACTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OBJECT_ATTRIBUTES", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["TransactionHandle", "DesiredAccess", "ObjectAttributes", "Uow", "TmHandle"]), offset=0),
    "PFN_NT_QUERY_INFORMATION_TRANSACTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="TRANSACTION_INFORMATION_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["TransactionHandle", "TransactionInformationClass", "TransactionInformation", "TransactionInformationLength", "ReturnLength"]), offset=0),
    "PFN_NT_ROLLBACK_TRANSACTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["TransactionHandle", "Wait"]), offset=0),
    "PFN_NT_SET_INFORMATION_TRANSACTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="TRANSACTION_INFORMATION_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["TransactionHandle", "TransactionInformationClass", "TransactionInformation", "TransactionInformationLength"]), offset=0),
    "PFN_PRINTING_ADDPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_ADDPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_ADDPORTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_ADDPORTEX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_PRINTING_CLOSEPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_CLOSEPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_CONFIGUREPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_CONFIGUREPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_DELETEPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_DELETEPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_ENDDOCPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_ENDDOCPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_ENUMPORTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_PRINTING_ENUMPORTS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_PRINTING_GETPRINTERDATAFROMPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_PRINTING_GETPRINTERDATAFROMPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PFN_PRINTING_NOTIFYUNUSEDPORTS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_NOTIFYUSEDPORTS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_OPENPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PFN_PRINTING_OPENPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_OPENPORTEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MONITOR2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_PRINTING_OPENPORTEX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("MONITOR2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PFN_PRINTING_POWEREVENT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("POWERBROADCAST_SETTING", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_READPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_READPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_SENDRECVBIDIDATAFROMPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("BIDI_REQUEST_CONTAINER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("BIDI_RESPONSE_CONTAINER", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_PRINTING_SETPORTTIMEOUTS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("COMMTIMEOUTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_SETPORTTIMEOUTS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("COMMTIMEOUTS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_SHUTDOWN2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_STARTDOCPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_PRINTING_STARTDOCPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PFN_PRINTING_WRITEPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_WRITEPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PRINTING_XCVCLOSEPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_XCVCLOSEPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PFN_PRINTING_XCVDATAPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_PRINTING_XCVDATAPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PFN_PRINTING_XCVOPENPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PFN_PRINTING_XCVOPENPORT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PFN_PROVIDER_CERTCHKPOLICY_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "idxSigner", "fCounterSignerChain", "idxCounterSigner"]), offset=0),
    "PFN_PROVIDER_CERTTRUST_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVIDER_CLEANUP_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVIDER_FINALPOLICY_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVIDER_INIT_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVIDER_OBJTRUST_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVIDER_SIGTRUST_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVIDER_TESTFINALPOLICY_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData"]), offset=0),
    "PFN_PROVUI_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndSecurityDialog", "pProvData"]), offset=0),
    "PFN_RPCNOTIFICATION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RPC_ASYNC_STATE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="RPC_ASYNC_EVENT")], SimTypeBottom(label="Void"), arg_names=["pAsync", "Context", "Event"]), offset=0),
    "PFN_RPC_ALLOCATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["param0"]), offset=0),
    "PFN_RPC_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PFN_RTL_IS_NTDDI_VERSION_AVAILABLE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["Version"]), offset=0),
    "PFN_RTL_IS_SERVICE_PACK_VERSION_INSTALLED": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["Version"]), offset=0),
    "PFN_SC_NOTIFY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pParameter"]), offset=0),
    "PFN_TERMINATEEXTENSION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["dwFlags"]), offset=0),
    "PFN_WDS_CLI_CALLBACK_MESSAGE_ID": SimTypeInt(signed=False, label="PFN_WDS_CLI_CALLBACK_MESSAGE_ID"),
    "PFN_WEB_CORE_ACTIVATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszAppHostConfigFile", "pszRootWebConfigFile", "pszInstanceName"]), offset=0),
    "PFN_WEB_CORE_SET_METADATA_DLL_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszMetadataType", "pszValue"]), offset=0),
    "PFN_WEB_CORE_SHUTDOWN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["fImmediate"]), offset=0),
    "PFN_WER_RUNTIME_EXCEPTION_DEBUGGER_LAUNCH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WER_RUNTIME_EXCEPTION_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "pExceptionInformation", "pbIsCustomDebugger", "pwszDebuggerLaunch", "pchDebuggerLaunch", "pbIsDebuggerAutolaunch"]), offset=0),
    "PFN_WER_RUNTIME_EXCEPTION_EVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WER_RUNTIME_EXCEPTION_INFORMATION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "pExceptionInformation", "pbOwnershipClaimed", "pwszEventName", "pchSize", "pdwSignatureCount"]), offset=0),
    "PFN_WER_RUNTIME_EXCEPTION_EVENT_SIGNATURE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WER_RUNTIME_EXCEPTION_INFORMATION", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContext", "pExceptionInformation", "dwIndex", "pwszName", "pchName", "pwszValue", "pchValue"]), offset=0),
    "PFN_WHEA_HIGH_IRQL_LOG_SEL_EVENT_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IPMI_OS_SEL_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "OsSelRecord"]), offset=0),
    "PFN_WTD_GENERIC_CHAIN_POLICY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("WTD_GENERIC_CHAIN_POLICY_SIGNER_INFO", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "dwStepError", "dwRegPolicySettings", "cSigner", "rgpSigner", "pvPolicyArg"]), offset=0),
    "PFN_WdsCliCallback": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="PFN_WDS_CLI_CALLBACK_MESSAGE_ID"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwMessageId", "wParam", "lParam", "pvUserData"]), offset=0),
    "PFN_WdsCliTraceFunction": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pwszFormat", "Params"]), offset=0),
    "PFN_WdsTransportClientReceiveContents": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "pContents", "ulSize", "pullContentOffset"]), offset=0),
    "PFN_WdsTransportClientReceiveMetadata": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "pMetadata", "ulSize"]), offset=0),
    "PFN_WdsTransportClientSessionComplete": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "dwError"]), offset=0),
    "PFN_WdsTransportClientSessionNegotiate": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TRANSPORTCLIENT_SESSION_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "pInfo", "hNegotiateKey"]), offset=0),
    "PFN_WdsTransportClientSessionStart": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "ullFileSize"]), offset=0),
    "PFN_WdsTransportClientSessionStartEx": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TRANSPORTCLIENT_SESSION_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "Info"]), offset=0),
    "PFPGA_BUS_SCAN": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PFPGA_CONTROL_CONFIG_SPACE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PFPGA_CONTROL_ERROR_REPORTING": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PFPGA_CONTROL_LINK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PFREEMEM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0),
    "PFREE_ADAPTER_CHANNEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter"]), offset=0),
    "PFREE_ADAPTER_OBJECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "AllocationAction"]), offset=0),
    "PFREE_CLUSTER_CRYPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pCryptInfo"]), offset=0),
    "PFREE_COMMON_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Length", "LogicalAddress", "VirtualAddress", "CacheEnabled"]), offset=0),
    "PFREE_COMMON_BUFFER_FROM_VECTOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Vector", "Index"]), offset=0),
    "PFREE_COMMON_BUFFER_VECTOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Vector"]), offset=0),
    "PFREE_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PFREE_FUNCTION_EX": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PFREE_MAP_REGISTERS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "MapRegisterBase", "NumberOfMapRegisters"]), offset=0),
    "PFREE_VIRTUAL_MEMORY_EX_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PFSCE_FREE_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pvServiceInfo"]), offset=0),
    "PFSCE_LOG_INFO": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SCE_LOG_ERR_LEVEL"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="SByte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ErrLevel", "Win32rc", "pErrFmt"]), offset=0),
    "PFSCE_QUERY_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SCESVC_INFO_TYPE"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["sceHandle", "sceType", "lpPrefix", "bExact", "ppvInfo", "psceEnumHandle"]), offset=0),
    "PFSCE_SET_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SCESVC_INFO_TYPE"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["sceHandle", "sceType", "lpPrefix", "bExact", "pvInfo"]), offset=0),
    "PFSRTL_EXTRA_CREATE_PARAMETER_CLEANUP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeBottom(label="Void"), arg_names=["EcpContext", "EcpType"]), offset=0),
    "PFSRTL_STACK_OVERFLOW_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("KEVENT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "Event"]), offset=0),
    "PFS_FILTER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data", "CompletionContext"]), offset=0),
    "PFS_FILTER_COMPLETION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FS_FILTER_CALLBACK_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Data", "OperationStatus", "CompletionContext"]), offset=0),
    "PFTASKDIALOGCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "msg", "wParam", "lParam", "lpRefData"]), offset=0),
    "PFUNCTION_TABLE_ACCESS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["hProcess", "AddrBase"]), offset=0),
    "PFUNCTION_TABLE_ACCESS_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ahProcess", "AddrBase"]), offset=0),
    "PFXExportOptions": SimTypeInt(signed=False, label="PFXExportOptions"),
    "PF_AddConnectNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("NOTIFYINFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("NOTIFYADD", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpNotifyInfo", "lpAddInfo"]), offset=0),
    "PF_CancelConnectNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("NOTIFYINFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("NOTIFYCANCEL", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpNotifyInfo", "lpCancelInfo"]), offset=0),
    "PF_ConfigAnalyzeService": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SCESVC_CALLBACK_INFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pSceCbInfo"]), offset=0),
    "PF_FILTER_DESCRIPTOR": SimStruct(OrderedDict((("dwFilterFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRule", SimTypeInt(signed=False, label="UInt32")), ("pfatType", SimTypeInt(signed=False, label="PFADDRESSTYPE")), ("SrcAddr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SrcMask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DstAddr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DstMask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("fLateBound", SimTypeInt(signed=False, label="UInt32")), ("wSrcPort", SimTypeShort(signed=False, label="UInt16")), ("wDstPort", SimTypeShort(signed=False, label="UInt16")), ("wSrcPortHighRange", SimTypeShort(signed=False, label="UInt16")), ("wDstPortHighRange", SimTypeShort(signed=False, label="UInt16")),)), name="PF_FILTER_DESCRIPTOR", pack=False, align=None),
    "PF_FILTER_STATS": SimStruct(OrderedDict((("dwNumPacketsFiltered", SimTypeInt(signed=False, label="UInt32")), ("info", SimTypeRef("PF_FILTER_DESCRIPTOR", SimStruct)),)), name="PF_FILTER_STATS", pack=False, align=None),
    "PF_INTERFACE_STATS": SimStruct(OrderedDict((("pvDriverContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwInDrops", SimTypeInt(signed=False, label="UInt32")), ("dwOutDrops", SimTypeInt(signed=False, label="UInt32")), ("eaInAction", SimTypeInt(signed=False, label="PFFORWARD_ACTION")), ("eaOutAction", SimTypeInt(signed=False, label="PFFORWARD_ACTION")), ("dwNumInFilters", SimTypeInt(signed=False, label="UInt32")), ("dwNumOutFilters", SimTypeInt(signed=False, label="UInt32")), ("dwFrag", SimTypeInt(signed=False, label="UInt32")), ("dwSpoof", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("liSYN", SimTypeLongLong(signed=True, label="Int64")), ("liTotalLogged", SimTypeLongLong(signed=True, label="Int64")), ("dwLostLogEntries", SimTypeInt(signed=False, label="UInt32")), ("FilterInfo", SimTypePointer(SimTypeRef("PF_FILTER_STATS", SimStruct), offset=0)),)), name="PF_INTERFACE_STATS", pack=False, align=None),
    "PF_LATEBIND_INFO": SimStruct(OrderedDict((("SrcAddr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DstAddr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Mask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PF_LATEBIND_INFO", pack=False, align=None),
    "PF_NPAddConnection": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpNetResource", "lpPassword", "lpUserName"]), offset=0),
    "PF_NPAddConnection3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndOwner", "lpNetResource", "lpPassword", "lpUserName", "dwFlags"]), offset=0),
    "PF_NPAddConnection4": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndOwner", "lpNetResource", "lpAuthBuffer", "cbAuthBuffer", "dwFlags", "lpUseOptions", "cbUseOptions"]), offset=0),
    "PF_NPCancelConnection": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpName", "fForce"]), offset=0),
    "PF_NPCancelConnection2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpName", "fForce", "dwFlags"]), offset=0),
    "PF_NPCloseEnum": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEnum"]), offset=0),
    "PF_NPDeviceMode": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hParent"]), offset=0),
    "PF_NPDirectoryNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "lpDir", "dwOper"]), offset=0),
    "PF_NPEnumResource": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEnum", "lpcCount", "lpBuffer", "lpBufferSize"]), offset=0),
    "PF_NPFMXEditPerm": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpDriveName", "hwndFMX", "nDialogType"]), offset=0),
    "PF_NPFMXGetPermCaps": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpDriveName"]), offset=0),
    "PF_NPFMXGetPermHelp": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpDriveName", "nDialogType", "fDirectory", "lpFileNameBuffer", "lpBufferSize", "lpnHelpContext"]), offset=0),
    "PF_NPFormatNetworkName": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpRemoteName", "lpFormattedName", "lpnLength", "dwFlags", "dwAveCharPerLine"]), offset=0),
    "PF_NPGetCaps": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ndex"]), offset=0),
    "PF_NPGetConnection": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpLocalName", "lpRemoteName", "lpnBufferLen"]), offset=0),
    "PF_NPGetConnection3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpLocalName", "dwLevel", "lpBuffer", "lpBufferSize"]), offset=0),
    "PF_NPGetConnectionPerformance": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("NETCONNECTINFOSTRUCT", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpRemoteName", "lpNetConnectInfo"]), offset=0),
    "PF_NPGetDirectoryType": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpName", "lpType", "bFlushCache"]), offset=0),
    "PF_NPGetPersistentUseOptionsForConnection": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpRemotePath", "lpReadUseOptions", "cbReadUseOptions", "lpWriteUseOptions", "lpSizeWriteUseOptions"]), offset=0),
    "PF_NPGetPropertyText": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["iButton", "nPropSel", "lpName", "lpButtonName", "nButtonNameLen", "nType"]), offset=0),
    "PF_NPGetResourceInformation": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpNetResource", "lpBuffer", "lpBufferSize", "lplpSystem"]), offset=0),
    "PF_NPGetResourceParent": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpNetResource", "lpBuffer", "lpBufferSize"]), offset=0),
    "PF_NPGetUniversalName": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpLocalPath", "dwInfoLevel", "lpBuffer", "lpnBufferSize"]), offset=0),
    "PF_NPGetUser": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpName", "lpUserName", "lpnBufferLen"]), offset=0),
    "PF_NPLogonNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpLogonId", "lpAuthentInfoType", "lpAuthentInfo", "lpPreviousAuthentInfoType", "lpPreviousAuthentInfo", "lpStationName", "StationHandle", "lpLogonScript"]), offset=0),
    "PF_NPOpenEnum": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwScope", "dwType", "dwUsage", "lpNetResource", "lphEnum"]), offset=0),
    "PF_NPPasswordChangeNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpAuthentInfoType", "lpAuthentInfo", "lpPreviousAuthentInfoType", "lpPreviousAuthentInfo", "lpStationName", "StationHandle", "dwChangeInfo"]), offset=0),
    "PF_NPPropertyDialog": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndParent", "iButtonDlg", "nPropSel", "lpFileName", "nType"]), offset=0),
    "PF_NPSearchDialog": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("NETRESOURCEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwndParent", "lpNetResource", "lpBuffer", "cbBuffer", "lpnFlags"]), offset=0),
    "PF_UpdateService": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SCESVC_CALLBACK_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCESVC_CONFIGURATION_INFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pSceCbInfo", "ServiceInfo"]), offset=0),
    "PGET_COMMON_BUFFER_FROM_VECTOR_BY_INDEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "Vector", "Index", "VirtualAddressOut", "LogicalAddressOut"]), offset=0),
    "PGET_CPU_MICROCODE_VERSION": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("DEBUG_CPU_MICROCODE_VERSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pCpuMicrocodeVersion"]), offset=0),
    "PGET_CPU_PSPEED_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("DEBUG_CPU_SPEED_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pCpuSpeedInfo"]), offset=0),
    "PGET_D3COLD_CAPABILITY": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PGET_D3COLD_LAST_TRANSITION_STATUS": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PGET_DEVICE_OBJECT_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("DEBUG_DEVICE_OBJECT_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "DeviceObject", "pDevObjInfo"]), offset=0),
    "PGET_DEVICE_RESET_STATUS": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PGET_DMA_ADAPTER": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)), offset=0),
    "PGET_DMA_ADAPTER_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DMA_ADAPTER_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "AdapterInfo"]), offset=0),
    "PGET_DMA_ALIGNMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["DmaAdapter"]), offset=0),
    "PGET_DMA_DOMAIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["DmaAdapter"]), offset=0),
    "PGET_DMA_TRANSFER_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("DMA_TRANSFER_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "Offset", "Length", "WriteOnly", "TransferInfo"]), offset=0),
    "PGET_DRIVER_OBJECT_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("DEBUG_DRIVER_OBJECT_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "DriverObject", "pDrvObjInfo"]), offset=0),
    "PGET_FULL_IMAGE_NAME": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Process", "FullImageName"]), offset=0),
    "PGET_IDLE_WAKE_INFO": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PGET_IRP_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("DEBUG_IRP_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Irp", "IrpInfo"]), offset=0),
    "PGET_LOCATION_STRING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "LocationStrings"]), offset=0),
    "PGET_MODULE_BASE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hProcess", "Address"]), offset=0),
    "PGET_MODULE_BASE_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeLongLong(signed=False, label="UInt64"), arg_names=["hProcess", "Address"]), offset=0),
    "PGET_MODULE_HANDLE_EXA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwFlags", "lpModuleName", "phModule"]), offset=0),
    "PGET_MODULE_HANDLE_EXW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwFlags", "lpModuleName", "phModule"]), offset=0),
    "PGET_PNP_TRIAGE_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("DEBUG_PNP_TRIAGE_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pPNPTriageInfo"]), offset=0),
    "PGET_POOL_DATA": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("DEBUG_POOL_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Pool", "PoolData"]), offset=0),
    "PGET_POOL_REGION": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeInt(signed=False, label="DEBUG_POOL_REGION"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Pool", "PoolRegion"]), offset=0),
    "PGET_POOL_TAG_DESCRIPTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DEBUG_POOLTAG_DESCRIPTION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PoolTag", "pDescription"]), offset=0),
    "PGET_PROCESS_COMMIT": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("PROCESS_COMMIT_USAGE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "TotalCommitCharge", "NumberOfProcesses", "CommitData"]), offset=0),
    "PGET_RESOURCE_STRING_FN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwMsgID", "lpBuffer", "nBufferMax"]), offset=0),
    "PGET_RUNTIME_FUNCTION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeRef("IMAGE_RUNTIME_FUNCTION_ENTRY", SimStruct), offset=0), arg_names=["ControlPc", "Context"]), offset=0),
    "PGET_SCATTER_GATHER_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "Mdl", "CurrentVa", "Length", "ExecutionRoutine", "Context", "WriteToDevice"]), offset=0),
    "PGET_SCATTER_GATHER_LIST_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "Irp", "ScatterGather", "Context"]), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DeviceObject", "DmaTransferContext", "Mdl", "Offset", "Length", "Flags", "ExecutionRoutine", "Context", "WriteToDevice", "DmaCompletionRoutine", "CompletionContext", "ScatterGatherList"]), offset=0),
    "PGET_SDEV_IDENTIFIER": SimTypePointer(SimTypeFunction([], SimTypeLongLong(signed=False, label="UInt64")), offset=0),
    "PGET_SET_DEVICE_DATA": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PGET_SMBIOS_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeRef("DEBUG_SMBIOS_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "pSmbiosInfo"]), offset=0),
    "PGET_SYSTEM_WOW64_DIRECTORY_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpBuffer", "uSize"]), offset=0),
    "PGET_SYSTEM_WOW64_DIRECTORY_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpBuffer", "uSize"]), offset=0),
    "PGET_TARGET_ATTRIBUTE_VALUE64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "Attribute", "AttributeData", "AttributeValue"]), offset=0),
    "PGET_UPDATED_BUS_RESOURCE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PGET_VIRTUAL_DEVICE_DATA": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PGET_VIRTUAL_DEVICE_LOCATION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PGET_VIRTUAL_DEVICE_RESOURCES": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PGET_VIRTUAL_FUNCTION_PROBED_BARS": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PGPE_CLEAR_STATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PGPE_CLEAR_STATUS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0),
    "PGPE_CONNECT_VECTOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="KINTERRUPT_MODE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PGPE_CONNECT_VECTOR2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="KINTERRUPT_MODE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "GpeNumber", "Mode", "Shareable", "ServiceRoutine", "ServiceContext", "ObjectContext"]), offset=0),
    "PGPE_DISABLE_EVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PGPE_DISABLE_EVENT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0),
    "PGPE_DISCONNECT_VECTOR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PGPE_DISCONNECT_VECTOR2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0),
    "PGPE_ENABLE_EVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PGPE_ENABLE_EVENT2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ObjectContext"]), offset=0),
    "PGPE_SERVICE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0", "param1"]), offset=0),
    "PGPE_SERVICE_ROUTINE2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["ObjectContext", "ServiceContext"]), offset=0),
    "PHALIOREADWRITEHANDLER": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["fRead", "dwAddr", "dwSize", "pdwData"]), offset=0),
    "PHALMCAINTERFACELOCK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PHALMCAINTERFACEREADREGISTER": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["BankNumber", "Exception"]), offset=0),
    "PHALMCAINTERFACEUNLOCK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PHAL_RESET_DISPLAY_PARAMETERS": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["Columns", "Rows"]), offset=0),
    "PHANDLER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CtrlType"]), offset=0),
    "PHIDP_INSERT_SCANCODES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["Context", "NewScanCodes", "Length"]), offset=0),
    "PHIDP_PREPARSED_DATA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PHONEBUTTONINFO": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwButtonMode", SimTypeInt(signed=False, label="UInt32")), ("dwButtonFunction", SimTypeInt(signed=False, label="UInt32")), ("dwButtonTextSize", SimTypeInt(signed=False, label="UInt32")), ("dwButtonTextOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwButtonState", SimTypeInt(signed=False, label="UInt32")),)), name="PHONEBUTTONINFO", pack=False, align=None),
    "PHONECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hDevice", "dwMessage", "dwInstance", "dwParam1", "dwParam2", "dwParam3"]), offset=0),
    "PHONECAPS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneInfoSize", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPermanentPhoneID", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwStringFormat", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneStates", SimTypeInt(signed=False, label="UInt32")), ("dwHookSwitchDevs", SimTypeInt(signed=False, label="UInt32")), ("dwHandsetHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwHeadsetHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwVolumeFlags", SimTypeInt(signed=False, label="UInt32")), ("dwGainFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayNumRows", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayNumColumns", SimTypeInt(signed=False, label="UInt32")), ("dwNumRingModes", SimTypeInt(signed=False, label="UInt32")), ("dwNumButtonLamps", SimTypeInt(signed=False, label="UInt32")), ("dwButtonModesSize", SimTypeInt(signed=False, label="UInt32")), ("dwButtonModesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwButtonFunctionsSize", SimTypeInt(signed=False, label="UInt32")), ("dwButtonFunctionsOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLampModesSize", SimTypeInt(signed=False, label="UInt32")), ("dwLampModesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwNumSetData", SimTypeInt(signed=False, label="UInt32")), ("dwSetDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwSetDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwNumGetData", SimTypeInt(signed=False, label="UInt32")), ("dwGetDataSize", SimTypeInt(signed=False, label="UInt32")), ("dwGetDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassesSize", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceClassesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneFeatures", SimTypeInt(signed=False, label="UInt32")), ("dwSettableHandsetHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwSettableSpeakerHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwSettableHeadsetHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwMonitoredHandsetHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwMonitoredSpeakerHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("dwMonitoredHeadsetHookSwitchModes", SimTypeInt(signed=False, label="UInt32")), ("PermanentPhoneGuid", SimTypeBottom(label="Guid")),)), name="PHONECAPS", pack=False, align=None),
    "PHONECAPS_BUFFER": SimTypeInt(signed=False, label="PHONECAPS_BUFFER"),
    "PHONECAPS_LONG": SimTypeInt(signed=False, label="PHONECAPS_LONG"),
    "PHONECAPS_STRING": SimTypeInt(signed=False, label="PHONECAPS_STRING"),
    "PHONEEVENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["htPhone", "dwMsg", "dwParam1", "dwParam2", "dwParam3"]), offset=0),
    "PHONEEXTENSIONID": SimStruct(OrderedDict((("dwExtensionID0", SimTypeInt(signed=False, label="UInt32")), ("dwExtensionID1", SimTypeInt(signed=False, label="UInt32")), ("dwExtensionID2", SimTypeInt(signed=False, label="UInt32")), ("dwExtensionID3", SimTypeInt(signed=False, label="UInt32")),)), name="PHONEEXTENSIONID", pack=False, align=None),
    "PHONEINITIALIZEEXPARAMS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("Handles", SimUnion({"hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hCompletionPort": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("dwCompletionKey", SimTypeInt(signed=False, label="UInt32")),)), name="PHONEINITIALIZEEXPARAMS", pack=False, align=None),
    "PHONEMESSAGE": SimStruct(OrderedDict((("hDevice", SimTypeInt(signed=False, label="UInt32")), ("dwMessageID", SimTypeInt(signed=False, label="UInt32")), ("dwCallbackInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwParam1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwParam2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwParam3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PHONEMESSAGE", pack=False, align=None),
    "PHONESTATUS": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwStatusFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumOwners", SimTypeInt(signed=False, label="UInt32")), ("dwNumMonitors", SimTypeInt(signed=False, label="UInt32")), ("dwRingMode", SimTypeInt(signed=False, label="UInt32")), ("dwRingVolume", SimTypeInt(signed=False, label="UInt32")), ("dwHandsetHookSwitchMode", SimTypeInt(signed=False, label="UInt32")), ("dwHandsetVolume", SimTypeInt(signed=False, label="UInt32")), ("dwHandsetGain", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerHookSwitchMode", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerVolume", SimTypeInt(signed=False, label="UInt32")), ("dwSpeakerGain", SimTypeInt(signed=False, label="UInt32")), ("dwHeadsetHookSwitchMode", SimTypeInt(signed=False, label="UInt32")), ("dwHeadsetVolume", SimTypeInt(signed=False, label="UInt32")), ("dwHeadsetGain", SimTypeInt(signed=False, label="UInt32")), ("dwDisplaySize", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayOffset", SimTypeInt(signed=False, label="UInt32")), ("dwLampModesSize", SimTypeInt(signed=False, label="UInt32")), ("dwLampModesOffset", SimTypeInt(signed=False, label="UInt32")), ("dwOwnerNameSize", SimTypeInt(signed=False, label="UInt32")), ("dwOwnerNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificSize", SimTypeInt(signed=False, label="UInt32")), ("dwDevSpecificOffset", SimTypeInt(signed=False, label="UInt32")), ("dwPhoneFeatures", SimTypeInt(signed=False, label="UInt32")),)), name="PHONESTATUS", pack=False, align=None),
    "PHONETICALPHABET": SimTypeInt(signed=False, label="PHONETICALPHABET"),
    "PHONE_BUTTON_FUNCTION": SimTypeInt(signed=False, label="PHONE_BUTTON_FUNCTION"),
    "PHONE_BUTTON_MODE": SimTypeInt(signed=False, label="PHONE_BUTTON_MODE"),
    "PHONE_BUTTON_STATE": SimTypeInt(signed=False, label="PHONE_BUTTON_STATE"),
    "PHONE_EVENT": SimTypeInt(signed=False, label="PHONE_EVENT"),
    "PHONE_HOOK_SWITCH_DEVICE": SimTypeInt(signed=False, label="PHONE_HOOK_SWITCH_DEVICE"),
    "PHONE_HOOK_SWITCH_STATE": SimTypeInt(signed=False, label="PHONE_HOOK_SWITCH_STATE"),
    "PHONE_LAMP_MODE": SimTypeInt(signed=False, label="PHONE_LAMP_MODE"),
    "PHONE_PRIVILEGE": SimTypeInt(signed=False, label="PHONE_PRIVILEGE"),
    "PHONE_TONE": SimTypeInt(signed=False, label="PHONE_TONE"),
    "PHVL_WHEA_ERROR_NOTIFICATION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PHYSICAL": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=False, label="UInt64")), ("BufLen", SimTypeInt(signed=False, label="UInt32")), ("Buf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PHYSICAL", pack=False, align=None),
    "PHYSICAL_COUNTER_EVENT_BUFFER_CONFIGURATION": SimStruct(OrderedDict((("OverflowHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["EventBuffer", "EntrySize", "NumberOfEntries", "OwningHandle"]), offset=0)), ("CustomEventBufferEntrySize", SimTypeInt(signed=False, label="UInt32")), ("EventThreshold", SimTypeInt(signed=False, label="UInt32")),)), name="PHYSICAL_COUNTER_EVENT_BUFFER_CONFIGURATION", pack=False, align=None),
    "PHYSICAL_COUNTER_RESOURCE_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="PHYSICAL_COUNTER_RESOURCE_DESCRIPTOR_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"CounterIndex": SimTypeInt(signed=False, label="UInt32"), "Range": SimTypeRef("_Range_e__Struct", SimStruct), "OverflowHandler": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["OverflowBits", "OwningHandle"]), offset=0), "EventBufferConfiguration": SimTypeRef("PHYSICAL_COUNTER_EVENT_BUFFER_CONFIGURATION", SimStruct), "IdentificationTag": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="PHYSICAL_COUNTER_RESOURCE_DESCRIPTOR", pack=False, align=None),
    "PHYSICAL_COUNTER_RESOURCE_DESCRIPTOR_TYPE": SimTypeInt(signed=False, label="PHYSICAL_COUNTER_RESOURCE_DESCRIPTOR_TYPE"),
    "PHYSICAL_COUNTER_RESOURCE_LIST": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Descriptors", SimTypePointer(SimTypeRef("PHYSICAL_COUNTER_RESOURCE_DESCRIPTOR", SimStruct), offset=0)),)), name="PHYSICAL_COUNTER_RESOURCE_LIST", pack=False, align=None),
    "PHYSICAL_ELEMENT_STATUS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DescriptorCount", SimTypeInt(signed=False, label="UInt32")), ("ReturnedDescriptorCount", SimTypeInt(signed=False, label="UInt32")), ("ElementIdentifierBeingDepoped", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Descriptors", SimTypePointer(SimTypeRef("PHYSICAL_ELEMENT_STATUS_DESCRIPTOR", SimStruct), offset=0)),)), name="PHYSICAL_ELEMENT_STATUS", pack=False, align=None),
    "PHYSICAL_ELEMENT_STATUS_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ElementIdentifier", SimTypeInt(signed=False, label="UInt32")), ("PhysicalElementType", SimTypeChar(label="Byte")), ("PhysicalElementHealth", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("AssociatedCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="PHYSICAL_ELEMENT_STATUS_DESCRIPTOR", pack=False, align=None),
    "PHYSICAL_ELEMENT_STATUS_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("StartingElement", SimTypeInt(signed=False, label="UInt32")), ("Filter", SimTypeChar(label="Byte")), ("ReportType", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="PHYSICAL_ELEMENT_STATUS_REQUEST", pack=False, align=None),
    "PHYSICAL_EXTENTS_DESCRIPTOR": SimStruct(OrderedDict((("NumberOfRuns", SimTypeInt(signed=False, label="UInt32")), ("NumberOfValidRuns", SimTypeInt(signed=False, label="UInt32")), ("Run", SimTypePointer(SimTypeRef("PHYSICAL_MEMORY_RUN", SimStruct), offset=0)),)), name="PHYSICAL_EXTENTS_DESCRIPTOR", pack=False, align=None),
    "PHYSICAL_MEMORY_DESCRIPTOR": SimStruct(OrderedDict((("NumberOfRuns", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPages", SimTypeInt(signed=False, label="UInt32")), ("Run", SimTypePointer(SimTypeRef("PHYSICAL_MEMORY_RUN", SimStruct), offset=0)),)), name="PHYSICAL_MEMORY_DESCRIPTOR", pack=False, align=None),
    "PHYSICAL_MEMORY_DESCRIPTOR32": SimStruct(OrderedDict((("NumberOfRuns", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPages", SimTypeInt(signed=False, label="UInt32")), ("Run", SimTypePointer(SimTypeRef("PHYSICAL_MEMORY_RUN32", SimStruct), offset=0)),)), name="PHYSICAL_MEMORY_DESCRIPTOR32", pack=False, align=None),
    "PHYSICAL_MEMORY_DESCRIPTOR64": SimStruct(OrderedDict((("NumberOfRuns", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPages", SimTypeLongLong(signed=False, label="UInt64")), ("Run", SimTypePointer(SimTypeRef("PHYSICAL_MEMORY_RUN64", SimStruct), offset=0)),)), name="PHYSICAL_MEMORY_DESCRIPTOR64", pack=False, align=None),
    "PHYSICAL_MEMORY_RANGE": SimStruct(OrderedDict((("BaseAddress", SimTypeLongLong(signed=True, label="Int64")), ("NumberOfBytes", SimTypeLongLong(signed=True, label="Int64")),)), name="PHYSICAL_MEMORY_RANGE", pack=False, align=None),
    "PHYSICAL_MEMORY_RUN": SimStruct(OrderedDict((("BasePage", SimTypeInt(signed=False, label="UInt32")), ("PageCount", SimTypeInt(signed=False, label="UInt32")),)), name="PHYSICAL_MEMORY_RUN", pack=False, align=None),
    "PHYSICAL_MEMORY_RUN32": SimStruct(OrderedDict((("BasePage", SimTypeInt(signed=False, label="UInt32")), ("PageCount", SimTypeInt(signed=False, label="UInt32")),)), name="PHYSICAL_MEMORY_RUN32", pack=False, align=None),
    "PHYSICAL_MEMORY_RUN64": SimStruct(OrderedDict((("BasePage", SimTypeLongLong(signed=False, label="UInt64")), ("PageCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="PHYSICAL_MEMORY_RUN64", pack=False, align=None),
    "PHYSICAL_MONITOR": SimStruct(OrderedDict((("hPhysicalMonitor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szPhysicalMonitorDescription", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="PHYSICAL_MONITOR", pack=False, align=None),
    "PHYSICAL_TO_VIRTUAL": SimStruct(OrderedDict((("Status", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PdeAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="PHYSICAL_TO_VIRTUAL", pack=False, align=None),
    "PHYSICAL_WITH_FLAGS": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=False, label="UInt64")), ("BufLen", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Buf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PHYSICAL_WITH_FLAGS", pack=False, align=None),
    "PIBIO_ENGINE_ACCEPT_PRIVATE_SENSOR_TYPE_INFO_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "TypeInfoBufferAddress", "TypeInfoBufferSize"]), offset=0),
    "PIBIO_ENGINE_ACCEPT_SAMPLE_DATA_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SampleBuffer", "SampleSize", "Purpose", "RejectDetail"]), offset=0),
    "PIBIO_ENGINE_ACTIVATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_ATTACH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_CHECK_FOR_DUPLICATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "Duplicate"]), offset=0),
    "PIBIO_ENGINE_CLEAR_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_COMMIT_ENROLLMENT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "PayloadBlob", "PayloadBlobSize"]), offset=0),
    "PIBIO_ENGINE_CONTROL_UNIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0),
    "PIBIO_ENGINE_CONTROL_UNIT_PRIVILEGED_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0),
    "PIBIO_ENGINE_CREATE_ENROLLMENT_AUTHENTICATED_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Nonce", "NonceSize"]), offset=0),
    "PIBIO_ENGINE_CREATE_ENROLLMENT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_CREATE_KEY_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Key", "KeySize", "KeyIdentifier", "KeyIdentifierSize", "ResultSize"]), offset=0),
    "PIBIO_ENGINE_DEACTIVATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_DETACH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_DISCARD_ENROLLMENT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_EXPORT_ENGINE_DATA_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Flags", "SampleBuffer", "SampleSize"]), offset=0),
    "PIBIO_ENGINE_GET_ENROLLMENT_HASH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "HashValue", "HashSize"]), offset=0),
    "PIBIO_ENGINE_GET_ENROLLMENT_STATUS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RejectDetail"]), offset=0),
    "PIBIO_ENGINE_IDENTIFY_ALL_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_PRESENCE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PresenceCount", "PresenceArray"]), offset=0),
    "PIBIO_ENGINE_IDENTIFY_FEATURE_SET_AUTHENTICATED_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Nonce", "NonceSize", "Identity", "SubFactor", "RejectDetail", "Authentication", "AuthenticationSize"]), offset=0),
    "PIBIO_ENGINE_IDENTIFY_FEATURE_SET_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "PayloadBlob", "PayloadBlobSize", "HashValue", "HashSize", "RejectDetail"]), offset=0),
    "PIBIO_ENGINE_IDENTIFY_FEATURE_SET_SECURE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Nonce", "NonceSize", "KeyIdentifier", "KeyIdentifierSize", "Identity", "SubFactor", "RejectDetail", "Authorization", "AuthorizationSize"]), offset=0),
    "PIBIO_ENGINE_NOTIFY_POWER_CHANGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0),
    "PIBIO_ENGINE_PIPELINE_CLEANUP_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_PIPELINE_INIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_QUERY_CALIBRATION_DATA_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DiscardAndRepeatCapture", "CalibrationBuffer", "CalibrationBufferSize", "MaxBufferSize"]), offset=0),
    "PIBIO_ENGINE_QUERY_EXTENDED_ENROLLMENT_STATUS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_ENROLLMENT_STATUS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "EnrollmentStatus", "EnrollmentStatusSize"]), offset=0),
    "PIBIO_ENGINE_QUERY_EXTENDED_INFO_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_ENGINE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "EngineInfo", "EngineInfoSize"]), offset=0),
    "PIBIO_ENGINE_QUERY_HASH_ALGORITHMS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AlgorithmCount", "AlgorithmBufferSize", "AlgorithmBuffer"]), offset=0),
    "PIBIO_ENGINE_QUERY_INDEX_VECTOR_SIZE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "IndexElementCount"]), offset=0),
    "PIBIO_ENGINE_QUERY_PREFERRED_FORMAT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "StandardFormat", "VendorFormat"]), offset=0),
    "PIBIO_ENGINE_QUERY_SAMPLE_HINT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SampleHint"]), offset=0),
    "PIBIO_ENGINE_REFRESH_CACHE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_ENGINE_RESERVED_1_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity"]), offset=0),
    "PIBIO_ENGINE_SELECT_CALIBRATION_FORMAT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "FormatArray", "FormatCount", "SelectedFormat", "MaxBufferSize"]), offset=0),
    "PIBIO_ENGINE_SET_ACCOUNT_POLICY_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_ACCOUNT_POLICY", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PolicyItemArray", "PolicyItemCount"]), offset=0),
    "PIBIO_ENGINE_SET_ENROLLMENT_PARAMETERS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_ENROLLMENT_PARAMETERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Parameters"]), offset=0),
    "PIBIO_ENGINE_SET_ENROLLMENT_SELECTOR_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SelectorValue"]), offset=0),
    "PIBIO_ENGINE_SET_HASH_ALGORITHM_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AlgorithmBufferSize", "AlgorithmBuffer"]), offset=0),
    "PIBIO_ENGINE_UPDATE_ENROLLMENT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RejectDetail"]), offset=0),
    "PIBIO_ENGINE_VERIFY_FEATURE_SET_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "Match", "PayloadBlob", "PayloadBlobSize", "HashValue", "HashSize", "RejectDetail"]), offset=0),
    "PIBIO_FRAMEWORK_ALLOCATE_MEMORY_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AllocationSize", "Address"]), offset=0),
    "PIBIO_FRAMEWORK_FREE_MEMORY_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Address"]), offset=0),
    "PIBIO_FRAMEWORK_GET_PROPERTY_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PropertyType", "PropertyId", "Identity", "SubFactor", "PropertyBuffer", "PropertyBufferSize"]), offset=0),
    "PIBIO_FRAMEWORK_LOCK_AND_VALIDATE_SECURE_BUFFER_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier", "SecureBufferAddress", "SecureBufferSize"]), offset=0),
    "PIBIO_FRAMEWORK_RELEASE_SECURE_BUFFER_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier"]), offset=0),
    "PIBIO_FRAMEWORK_SET_UNIT_STATUS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_UNIT_STATUS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ExtendedStatus", "ExtendedStatusSize"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_CLEAR_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_EXPORT_BEGIN_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RequiredCapacity", "MaxBufferSize"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_EXPORT_END_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_EXPORT_NEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "BufferAddress", "BufferSize", "ReturnedDataSize"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_IMPORT_BEGIN_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RequiredCapacity", "MaxBufferSize"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_IMPORT_END_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_CACHE_IMPORT_NEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "BufferAddress", "BufferSize"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_DECRYPT_SAMPLE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Authentication", "AuthenticationSize", "Iv", "IvSize", "EncryptedData", "EncryptedDataSize"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_QUERY_AUTHORIZED_ENROLLMENTS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SecureIdentityCount", "SecureIdentities"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_STORAGE_RESERVED_1_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Reserved1", "Reserved2"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_STORAGE_RESERVED_2_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Reserved1", "Reserved2"]), offset=0),
    "PIBIO_FRAMEWORK_VSM_STORAGE_RESERVED_3_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_ACCEPT_CALIBRATION_DATA_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "CalibrationBuffer", "CalibrationBufferSize"]), offset=0),
    "PIBIO_SENSOR_ACTIVATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_ASYNC_IMPORT_RAW_BUFFER_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RawBufferAddress", "RawBufferSize", "ResultBufferAddress", "ResultBufferSize"]), offset=0),
    "PIBIO_SENSOR_ASYNC_IMPORT_SECURE_BUFFER_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier", "MetadataBufferAddress", "MetadataBufferSize", "ResultBufferAddress", "ResultBufferSize"]), offset=0),
    "PIBIO_SENSOR_ATTACH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_CANCEL_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_CLEAR_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_CONNECT_SECURE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_SECURE_CONNECTION_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_SECURE_CONNECTION_DATA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ConnectionParams", "ConnectionData"]), offset=0),
    "PIBIO_SENSOR_CONTROL_UNIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0),
    "PIBIO_SENSOR_CONTROL_UNIT_PRIVILEGED_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0),
    "PIBIO_SENSOR_DEACTIVATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_DETACH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_EXPORT_SENSOR_DATA_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SampleBuffer", "SampleSize"]), offset=0),
    "PIBIO_SENSOR_FINISH_CAPTURE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RejectDetail"]), offset=0),
    "PIBIO_SENSOR_FINISH_NOTIFY_WAKE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Reason"]), offset=0),
    "PIBIO_SENSOR_GET_INDICATOR_STATUS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "IndicatorStatus"]), offset=0),
    "PIBIO_SENSOR_NOTIFY_POWER_CHANGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0),
    "PIBIO_SENSOR_PIPELINE_CLEANUP_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_PIPELINE_INIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_PUSH_DATA_TO_ENGINE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Flags", "RejectDetail"]), offset=0),
    "PIBIO_SENSOR_QUERY_CALIBRATION_FORMATS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "FormatArray", "FormatArraySize", "FormatCount"]), offset=0),
    "PIBIO_SENSOR_QUERY_EXTENDED_INFO_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_SENSOR_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SensorInfo", "SensorInfoSize"]), offset=0),
    "PIBIO_SENSOR_QUERY_PRIVATE_SENSOR_TYPE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "TypeInfoBufferAddress", "TypeInfoBufferSize", "TypeInfoDataSize"]), offset=0),
    "PIBIO_SENSOR_QUERY_STATUS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Status"]), offset=0),
    "PIBIO_SENSOR_RESET_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_SENSOR_SET_CALIBRATION_FORMAT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Format"]), offset=0),
    "PIBIO_SENSOR_SET_INDICATOR_STATUS_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "IndicatorStatus"]), offset=0),
    "PIBIO_SENSOR_SET_MODE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Mode"]), offset=0),
    "PIBIO_SENSOR_START_CAPTURE_EX_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Nonce", "NonceSize", "Flags", "Overlapped"]), offset=0),
    "PIBIO_SENSOR_START_CAPTURE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Overlapped"]), offset=0),
    "PIBIO_SENSOR_START_NOTIFY_WAKE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Overlapped"]), offset=0),
    "PIBIO_STORAGE_ACTIVATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_ADD_RECORD_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordContents"]), offset=0),
    "PIBIO_STORAGE_ATTACH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_CLEAR_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_CLOSE_DATABASE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_CONTROL_UNIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0),
    "PIBIO_STORAGE_CONTROL_UNIT_PRIVILEGED_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0),
    "PIBIO_STORAGE_CREATE_DATABASE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "Factor", "Format", "FilePath", "ConnectString", "IndexElementCount", "InitialSize"]), offset=0),
    "PIBIO_STORAGE_DEACTIVATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_DELETE_RECORD_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor"]), offset=0),
    "PIBIO_STORAGE_DETACH_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_ERASE_DATABASE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "FilePath", "ConnectString"]), offset=0),
    "PIBIO_STORAGE_FIRST_RECORD_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_GET_CURRENT_RECORD_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordContents"]), offset=0),
    "PIBIO_STORAGE_GET_DATABASE_SIZE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AvailableRecordCount", "TotalRecordCount"]), offset=0),
    "PIBIO_STORAGE_GET_DATA_FORMAT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WINBIO_VERSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Format", "Version"]), offset=0),
    "PIBIO_STORAGE_GET_RECORD_COUNT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordCount"]), offset=0),
    "PIBIO_STORAGE_NEXT_RECORD_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_NOTIFY_DATABASE_CHANGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordsAdded"]), offset=0),
    "PIBIO_STORAGE_NOTIFY_POWER_CHANGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0),
    "PIBIO_STORAGE_OPEN_DATABASE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "FilePath", "ConnectString"]), offset=0),
    "PIBIO_STORAGE_PIPELINE_CLEANUP_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_PIPELINE_INIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0),
    "PIBIO_STORAGE_QUERY_BY_CONTENT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SubFactor", "IndexVector", "IndexElementCount"]), offset=0),
    "PIBIO_STORAGE_QUERY_BY_SUBJECT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor"]), offset=0),
    "PIBIO_STORAGE_QUERY_EXTENDED_INFO_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_STORAGE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "StorageInfo", "StorageInfoSize"]), offset=0),
    "PIBIO_STORAGE_RESERVED_1_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "Reserved1", "Reserved2"]), offset=0),
    "PIBIO_STORAGE_RESERVED_2_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity"]), offset=0),
    "PIBIO_STORAGE_UPDATE_RECORD_BEGIN_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "RecordContents"]), offset=0),
    "PIBIO_STORAGE_UPDATE_RECORD_COMMIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordContents"]), offset=0),
    "PICTDESC": SimStruct(OrderedDict((("cbSizeofstruct", SimTypeInt(signed=False, label="UInt32")), ("picType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"bmp": SimTypeRef("_bmp_e__Struct", SimStruct), "wmf": SimTypeRef("_wmf_e__Struct", SimStruct), "icon": SimTypeRef("_icon_e__Struct", SimStruct), "emf": SimTypeRef("_emf_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PICTDESC", pack=False, align=None),
    "PICTUREATTRIBUTES": SimTypeInt(signed=False, label="PICTUREATTRIBUTES"),
    "PICTYPE": SimTypeInt(signed=False, label="PICTYPE"),
    "PIC_SEQ_SAMPLE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="PIC_SEQ_SAMPLE", pack=False, align=None),
    "PIDISF_FLAGS": SimTypeInt(signed=False, label="PIDISF_FLAGS"),
    "PIDISM_OPTIONS": SimTypeInt(signed=False, label="PIDISM_OPTIONS"),
    "PIDISR_INFO": SimTypeInt(signed=False, label="PIDISR_INFO"),
    "PIDListSpanningEvent": SimStruct(OrderedDict((("wPIDCount", SimTypeShort(signed=False, label="UInt16")), ("pulPIDs", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PIDListSpanningEvent", pack=False, align=None),
    "PIDMSI_STATUS_VALUE": SimTypeInt(signed=False, label="PIDMSI_STATUS_VALUE"),
    "PID_BITS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="PID_BITS", pack=False, align=None),
    "PID_BITS_MIDL": SimStruct(OrderedDict((("Bits", SimTypeShort(signed=False, label="UInt16")),)), name="PID_BITS_MIDL", pack=False, align=None),
    "PID_INTSITE": SimTypeInt(signed=False, label="PID_INTSITE"),
    "PID_IS": SimTypeInt(signed=False, label="PID_IS"),
    "PID_MAP": SimStruct(OrderedDict((("ulPID", SimTypeInt(signed=False, label="UInt32")), ("MediaSampleContent", SimTypeInt(signed=False, label="MEDIA_SAMPLE_CONTENT")),)), name="PID_MAP", pack=False, align=None),
    "PIMAGEHLP_STATUS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="IMAGEHLP_STATUS_REASON"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Reason", "ImageName", "DllName", "Va", "Parameter"]), offset=0),
    "PIMAGEHLP_STATUS_ROUTINE32": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="IMAGEHLP_STATUS_REASON"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Reason", "ImageName", "DllName", "Va", "Parameter"]), offset=0),
    "PIMAGEHLP_STATUS_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="IMAGEHLP_STATUS_REASON"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Reason", "ImageName", "DllName", "Va", "Parameter"]), offset=0),
    "PIMAGE_TLS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["DllHandle", "Reason", "Reserved"]), offset=0),
    "PINITIALIZE_DMA_TRANSFER_CONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DmaTransferContext"]), offset=0),
    "PINIT_ONCE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimUnion({"Ptr": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InitOnce", "Parameter", "Context"]), offset=0),
    "PINSTALLUI_HANDLER_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pvContext", "iMessageType", "hRecord"]), offset=0),
    "PINTERFACE_DEREFERENCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PINTERFACE_REFERENCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PINTERFACE_TIMESTAMP_CONFIG_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallerContext"]), offset=0),
    "PIN_DIRECTION": SimTypeInt(signed=False, label="PIN_DIRECTION"),
    "PIN_INFO": SimStruct(OrderedDict((("pFilter", SimTypeBottom(label="IBaseFilter")), ("dir", SimTypeInt(signed=False, label="PIN_DIRECTION")), ("achName", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="PIN_INFO", pack=False, align=None),
    "PIOMMU_DEVICE_CREATE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DEVICE_DELETE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DEVICE_FAULT_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIOMMU_DEVICE_QUERY_DOMAIN_TYPES": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIOMMU_DOMAIN_ATTACH_DEVICE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_ATTACH_DEVICE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_CONFIGURE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_CREATE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_CREATE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_DELETE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_DETACH_DEVICE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_DOMAIN_DETACH_DEVICE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_FLUSH_DOMAIN": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_FLUSH_DOMAIN_VA_LIST": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_FREE_RESERVED_LOGICAL_ADDRESS_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_INTERFACE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIOMMU_MAP_IDENTITY_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_MAP_IDENTITY_RANGE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_MAP_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_MAP_LOGICAL_RANGE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_MAP_RESERVED_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_QUERY_INPUT_MAPPINGS": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_REGISTER_INTERFACE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_RESERVE_LOGICAL_ADDRESS_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_SET_DEVICE_FAULT_REPORTING": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_SET_DEVICE_FAULT_REPORTING_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_UNMAP_IDENTITY_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_UNMAP_IDENTITY_RANGE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_UNMAP_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_UNMAP_RESERVED_LOGICAL_RANGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIOMMU_UNREGISTER_INTERFACE_STATE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIO_APC_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ApcContext", "IoStatusBlock", "Reserved"]), offset=0),
    "PIO_COMPLETION_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIO_CONTAINER_NOTIFICATION_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIO_CSQ_ACQUIRE_LOCK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_CSQ_COMPLETE_CANCELED_IRP": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_CSQ_INSERT_IRP": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_CSQ_INSERT_IRP_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIO_CSQ_PEEK_NEXT_IRP": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), offset=0),
    "PIO_CSQ_RELEASE_LOCK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_CSQ_REMOVE_IRP": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_DEVICE_EJECT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "Context"]), offset=0),
    "PIO_DPC_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_IRP_EXT_PROCESS_TRACKED_OFFSET_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("IO_IRP_EXT_TRACK_OFFSET_HEADER", SimStruct), offset=0), SimTypePointer(SimTypeRef("IO_IRP_EXT_TRACK_OFFSET_HEADER", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64")], SimTypeBottom(label="Void"), arg_names=["SourceContext", "TargetContext", "RelativeOffset"]), offset=0),
    "PIO_PERSISTED_MEMORY_ENUMERATION_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIO_QUERY_DEVICE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("KEY_VALUE_FULL_INFORMATION", SimStruct), offset=0), offset=0), SimTypeInt(signed=False, label="CONFIGURATION_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("KEY_VALUE_FULL_INFORMATION", SimStruct), offset=0), offset=0), SimTypeInt(signed=False, label="CONFIGURATION_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("KEY_VALUE_FULL_INFORMATION", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "PathName", "BusType", "BusNumber", "BusInformation", "ControllerType", "ControllerNumber", "ControllerInformation", "PeripheralType", "PeripheralNumber", "PeripheralInformation"]), offset=0),
    "PIO_REMOVE_LOCK_TRACKING_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PIO_SESSION_NOTIFICATION_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PIO_TIMER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PIO_TIMER_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_WORKITEM": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PIO_WORKITEM_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIO_WORKITEM_ROUTINE_EX": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PIPE_ALLOCATOR_PLACE": SimTypeInt(signed=False, label="PIPE_ALLOCATOR_PLACE"),
    "PIPE_DIMENSIONS": SimStruct(OrderedDict((("AllocatorPin", SimTypeRef("KS_COMPRESSION", SimStruct)), ("MaxExpansionPin", SimTypeRef("KS_COMPRESSION", SimStruct)), ("EndPin", SimTypeRef("KS_COMPRESSION", SimStruct)),)), name="PIPE_DIMENSIONS", pack=False, align=None),
    "PIPE_STATE": SimTypeInt(signed=False, label="PIPE_STATE"),
    "PIPE_TERMINATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("OutsideFactors", SimTypeInt(signed=False, label="UInt32")), ("Weigth", SimTypeInt(signed=False, label="UInt32")), ("PhysicalRange", SimTypeRef("KS_FRAMING_RANGE", SimStruct)), ("OptimalRange", SimTypeRef("KS_FRAMING_RANGE_WEIGHTED", SimStruct)), ("Compression", SimTypeRef("KS_COMPRESSION", SimStruct)),)), name="PIPE_TERMINATION", pack=False, align=None),
    "PIPE_TYPE": SimTypeInt(signed=False, label="PIPE_TYPE"),
    "PIPFORWARD_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MIB_IPFORWARD_ROW2", SimStruct), offset=0), SimTypeInt(signed=False, label="MIB_NOTIFICATION_TYPE")], SimTypeBottom(label="Void"), arg_names=["CallerContext", "Row", "NotificationType"]), offset=0),
    "PIPINTERFACE_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MIB_IPINTERFACE_ROW", SimStruct), offset=0), SimTypeInt(signed=False, label="MIB_NOTIFICATION_TYPE")], SimTypeBottom(label="Void"), arg_names=["CallerContext", "Row", "NotificationType"]), offset=0),
    "PIS_ALIVE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0),
    "PIXELFORMATDESCRIPTOR": SimStruct(OrderedDict((("nSize", SimTypeShort(signed=False, label="UInt16")), ("nVersion", SimTypeShort(signed=False, label="UInt16")), ("dwFlags", SimTypeInt(signed=False, label="PFD_FLAGS")), ("iPixelType", SimTypeInt(signed=False, label="PFD_PIXEL_TYPE")), ("cColorBits", SimTypeChar(label="Byte")), ("cRedBits", SimTypeChar(label="Byte")), ("cRedShift", SimTypeChar(label="Byte")), ("cGreenBits", SimTypeChar(label="Byte")), ("cGreenShift", SimTypeChar(label="Byte")), ("cBlueBits", SimTypeChar(label="Byte")), ("cBlueShift", SimTypeChar(label="Byte")), ("cAlphaBits", SimTypeChar(label="Byte")), ("cAlphaShift", SimTypeChar(label="Byte")), ("cAccumBits", SimTypeChar(label="Byte")), ("cAccumRedBits", SimTypeChar(label="Byte")), ("cAccumGreenBits", SimTypeChar(label="Byte")), ("cAccumBlueBits", SimTypeChar(label="Byte")), ("cAccumAlphaBits", SimTypeChar(label="Byte")), ("cDepthBits", SimTypeChar(label="Byte")), ("cStencilBits", SimTypeChar(label="Byte")), ("cAuxBuffers", SimTypeChar(label="Byte")), ("iLayerType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("dwLayerMask", SimTypeInt(signed=False, label="UInt32")), ("dwVisibleMask", SimTypeInt(signed=False, label="UInt32")), ("dwDamageMask", SimTypeInt(signed=False, label="UInt32")),)), name="PIXELFORMATDESCRIPTOR", pack=False, align=None),
    "PI_FLAGS": SimTypeInt(signed=False, label="PI_FLAGS"),
    "PJOIN_DMA_DOMAIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "DomainHandle"]), offset=0),
    "PKA_FLAGS": SimTypeInt(signed=False, label="PKA_FLAGS"),
    "PKBUGCHECK_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PKBUGCHECK_REASON_CALLBACK_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PKCS12_PBES2_EXPORT_PARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hNcryptDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pwszPbes2Alg", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PKCS12_PBES2_EXPORT_PARAMS", pack=False, align=None),
    "PKDEFERRED_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PKDEXTS_GET_PTE_INFO": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("KDEXTS_PTE_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Virtual", "PteInfo"]), offset=0),
    "PKINTERRUPT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PKIPI_BROADCAST_WORKER": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), offset=0),
    "PKMESSAGE_SERVICE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PKPROCESS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PKSEC_CREATE_CONTEXT_LIST": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="KSEC_CONTEXT_TYPE")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Type"]), offset=0),
    "PKSEC_DEREFERENCE_LIST_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KSEC_LIST_ENTRY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Entry", "Delete"]), offset=0),
    "PKSEC_INSERT_LIST_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("KSEC_LIST_ENTRY", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["List", "Entry"]), offset=0),
    "PKSEC_LOCATE_PKG_BY_ID": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PackageId"]), offset=0),
    "PKSEC_REFERENCE_LIST_ENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KSEC_LIST_ENTRY", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Entry", "Signature", "RemoveNoRef"]), offset=0),
    "PKSEC_SERIALIZE_SCHANNEL_AUTH_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvAuthData", "Size", "SerializedData"]), offset=0),
    "PKSEC_SERIALIZE_WINNT_AUTH_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvAuthData", "Size", "SerializedData"]), offset=0),
    "PKSERVICE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PKSTART_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PKSYNCHRONIZE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PKTHREAD": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PKU2U_CERTIFICATE_S4U_LOGON": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="PKU2U_LOGON_SUBMIT_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("UserPrincipalName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("CertificateLength", SimTypeInt(signed=False, label="UInt32")), ("Certificate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PKU2U_CERTIFICATE_S4U_LOGON", pack=False, align=None),
    "PKU2U_CERT_BLOB": SimStruct(OrderedDict((("CertOffset", SimTypeInt(signed=False, label="UInt32")), ("CertLength", SimTypeShort(signed=False, label="UInt16")),)), name="PKU2U_CERT_BLOB", pack=False, align=None),
    "PKU2U_CREDUI_CONTEXT": SimStruct(OrderedDict((("Version", SimTypeLongLong(signed=False, label="UInt64")), ("cbHeaderLength", SimTypeShort(signed=False, label="UInt16")), ("cbStructureLength", SimTypeInt(signed=False, label="UInt32")), ("CertArrayCount", SimTypeShort(signed=False, label="UInt16")), ("CertArrayOffset", SimTypeInt(signed=False, label="UInt32")),)), name="PKU2U_CREDUI_CONTEXT", pack=False, align=None),
    "PKU2U_LOGON_SUBMIT_TYPE": SimTypeInt(signed=False, label="PKU2U_LOGON_SUBMIT_TYPE"),
    "PLACEHOLDER_STATES": SimTypeInt(signed=False, label="PLACEHOLDER_STATES"),
    "PLACEMENT_OPTIONS": SimTypeInt(signed=False, label="PLACEMENT_OPTIONS"),
    "PLAYTO_SOURCE_CREATEFLAGS": SimTypeInt(signed=False, label="PLAYTO_SOURCE_CREATEFLAGS"),
    "PLA_CABEXTRACT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["FileName", "Context"]), offset=0),
    "PLDAPSearch": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PLEAVE_DMA_DOMAIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter"]), offset=0),
    "PLEX_READ_DATA_REQUEST": SimStruct(OrderedDict((("ByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("ByteLength", SimTypeInt(signed=False, label="UInt32")), ("PlexNumber", SimTypeInt(signed=False, label="UInt32")),)), name="PLEX_READ_DATA_REQUEST", pack=False, align=None),
    "PLOAD_IMAGE_NOTIFY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("IMAGE_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["FullImageName", "ProcessId", "ImageInfo"]), offset=0),
    "PLOG_EVENT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0),
    "PLOG_FULL_HANDLER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hLogFile", "dwError", "fLogIsPinned", "pvClientContext"]), offset=0),
    "PLOG_TAIL_ADVANCE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeRef("CLS_LSN", SimStruct), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hLogFile", "lsnTarget", "pvClientContext"]), offset=0),
    "PLOG_UNPINNED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hLogFile", "pvClientContext"]), offset=0),
    "PLOOKS_ALIVE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Resource"]), offset=0),
    "PLSA_ADD_CREDENTIAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "PrimaryKeyValue", "Credentials"]), offset=0),
    "PLSA_ALLOCATE_CLIENT_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LengthRequired", "ClientBaseAddress"]), offset=0),
    "PLSA_ALLOCATE_LSA_HEAP": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0),
    "PLSA_ALLOCATE_PRIVATE_HEAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0),
    "PLSA_ALLOCATE_SHARED_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["SharedMem", "Size"]), offset=0),
    "PLSA_AP_CALL_PACKAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0),
    "PLSA_AP_CALL_PACKAGE_PASSTHROUGH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0),
    "PLSA_AP_CALL_PACKAGE_UNTRUSTED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0),
    "PLSA_AP_INITIALIZE_PACKAGE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_DISPATCH_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackageId", "LsaDispatchTable", "Database", "Confidentiality", "AuthenticationPackageName"]), offset=0),
    "PLSA_AP_LOGON_TERMINATED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogonId"]), offset=0),
    "PLSA_AP_LOGON_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "AuthenticationInformation", "ClientAuthenticationBase", "AuthenticationInformationLength", "ProfileBuffer", "ProfileBufferLength", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority"]), offset=0),
    "PLSA_AP_LOGON_USER_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "AuthenticationInformation", "ClientAuthenticationBase", "AuthenticationInformationLength", "ProfileBuffer", "ProfileBufferLength", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName"]), offset=0),
    "PLSA_AP_LOGON_USER_EX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "ProfileBuffer", "ProfileBufferSize", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0),
    "PLSA_AP_LOGON_USER_EX3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "SurrogateLogon", "ProfileBuffer", "ProfileBufferSize", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0),
    "PLSA_AP_POST_LOGON_USER_SURROGATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "SurrogateLogon", "ProfileBuffer", "ProfileBufferSize", "LogonId", "Status", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0),
    "PLSA_AP_PRE_LOGON_USER_SURROGATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "SurrogateLogon", "SubStatus"]), offset=0),
    "PLSA_AUDIT_ACCOUNT_LOGON": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["AuditId", "Success", "Source", "ClientName", "MappedName", "Status"]), offset=0),
    "PLSA_AUDIT_LOGON": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "SubStatus", "AccountName", "AuthenticatingAuthority", "WorkstationName", "UserSid", "LogonType", "TokenSource", "LogonId"]), offset=0),
    "PLSA_AUDIT_LOGON_EX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "SubStatus", "AccountName", "AuthenticatingAuthority", "WorkstationName", "UserSid", "LogonType", "ImpersonationLevel", "TokenSource", "LogonId"]), offset=0),
    "PLSA_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Argument1", "Argument2", "InputBuffer", "OutputBuffer"]), offset=0),
    "PLSA_CALL_PACKAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackage", "ProtocolSubmitBuffer", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0),
    "PLSA_CALL_PACKAGEEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackage", "ClientBufferBase", "ProtocolSubmitBuffer", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0),
    "PLSA_CALL_PACKAGE_PASSTHROUGH": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackage", "ClientBufferBase", "ProtocolSubmitBuffer", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0),
    "PLSA_CANCEL_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["NotifyHandle"]), offset=0),
    "PLSA_CHECK_PROTECTED_USER_BY_TOKEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserToken", "ProtectedUser"]), offset=0),
    "PLSA_CLIENT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Callback", "Argument1", "Argument2", "Input", "Output"]), offset=0),
    "PLSA_CLOSE_SAM_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserHandle"]), offset=0),
    "PLSA_CONVERT_AUTH_DATA_TO_TOKEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserAuthData", "UserAuthDataSize", "ImpersonationLevel", "TokenSource", "LogonType", "AuthorityName", "Token", "LogonId", "AccountName", "SubStatus"]), offset=0),
    "PLSA_COPY_FROM_CLIENT_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "Length", "BufferToCopy", "ClientBaseAddress"]), offset=0),
    "PLSA_COPY_TO_CLIENT_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "Length", "ClientBaseAddress", "BufferToCopy"]), offset=0),
    "PLSA_CRACK_SINGLE_NAME": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FormatOffered", "PerformAtGC", "NameInput", "Prefix", "RequestedFormat", "CrackedName", "DnsDomainName", "SubStatus"]), offset=0),
    "PLSA_CREATE_LOGON_SESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0),
    "PLSA_CREATE_SHARED_MEMORY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["MaxSize", "InitialSize"]), offset=0),
    "PLSA_CREATE_THREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["SecurityAttributes", "StackSize", "StartFunction", "ThreadParameter", "CreationFlags", "ThreadId"]), offset=0),
    "PLSA_CREATE_TOKEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "TokenSource", "LogonType", "ImpersonationLevel", "TokenInformationType", "TokenInformation", "TokenGroups", "AccountName", "AuthorityName", "Workstation", "ProfilePath", "Token", "SubStatus"]), offset=0),
    "PLSA_CREATE_TOKEN_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeRef("TOKEN_SOURCE", SimStruct), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"), SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TOKEN_GROUPS", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SECPKG_SESSIONINFO_TYPE"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "TokenSource", "LogonType", "ImpersonationLevel", "TokenInformationType", "TokenInformation", "TokenGroups", "Workstation", "ProfilePath", "SessionInformation", "SessionInformationType", "Token", "SubStatus"]), offset=0),
    "PLSA_DELETE_CREDENTIAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "PrimaryKeyValue"]), offset=0),
    "PLSA_DELETE_LOGON_SESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0),
    "PLSA_DELETE_SHARED_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["SharedMem"]), offset=0),
    "PLSA_DUPLICATE_HANDLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SourceHandle", "DestionationHandle"]), offset=0),
    "PLSA_EXPAND_AUTH_DATA_FOR_DOMAIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserAuthData", "UserAuthDataSize", "Reserved", "ExpandedAuthData", "ExpandedAuthDataSize"]), offset=0),
    "PLSA_FREE_CLIENT_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ClientBaseAddress"]), offset=0),
    "PLSA_FREE_LSA_HEAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0),
    "PLSA_FREE_PRIVATE_HEAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0),
    "PLSA_FREE_SHARED_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["SharedMem", "Memory"]), offset=0),
    "PLSA_GET_APP_MODE_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserFunction", "Argument1", "Argument2", "UserData", "ReturnToLsa"]), offset=0),
    "PLSA_GET_AUTH_DATA_FOR_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="SECPKG_NAME_TYPE"), SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Name", "NameType", "Prefix", "UserAuthData", "UserAuthDataSize", "UserFlatName"]), offset=0),
    "PLSA_GET_CALL_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_CALL_INFO", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["Info"]), offset=0),
    "PLSA_GET_CLIENT_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_CLIENT_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientInfo"]), offset=0),
    "PLSA_GET_CLIENT_INFO_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_CLIENT_INFO_EX", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientInfo", "StructSize"]), offset=0),
    "PLSA_GET_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "AuthenticationPackage", "QueryContext", "RetrieveAllCredentials", "PrimaryKeyValue", "PrimaryKeyLength", "Credentials"]), offset=0),
    "PLSA_GET_EXTENDED_CALL_FLAGS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Flags"]), offset=0),
    "PLSA_GET_SERVICE_ACCOUNT_PASSWORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="CRED_FETCH"), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AccountName", "DomainName", "CredFetch", "FileTimeExpiry", "CurrentPassword", "PreviousPassword", "FileTimeCurrPwdValidForOutbound"]), offset=0),
    "PLSA_GET_USER_AUTH_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserHandle", "UserAuthData", "UserAuthDataSize"]), offset=0),
    "PLSA_GET_USER_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserHandle", "PrimaryCreds", "PrimaryCredsSize", "SupplementalCreds", "SupplementalCredsSize"]), offset=0),
    "PLSA_IMPERSONATE_CLIENT": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PLSA_LOCATE_PKG_BY_ID": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PackgeId"]), offset=0),
    "PLSA_MAP_BUFFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InputBuffer", "OutputBuffer"]), offset=0),
    "PLSA_OPEN_SAM_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="SECPKG_NAME_TYPE"), SimTypePointer(SimTypeRef("SECURITY_STRING", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Name", "NameType", "Prefix", "AllowGuest", "Reserved", "UserHandle"]), offset=0),
    "PLSA_OPEN_TOKEN_BY_LOGON_ID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "RetTokenHandle"]), offset=0),
    "PLSA_PROTECT_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferSize"]), offset=0),
    "PLSA_QUERY_CLIENT_REQUEST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "QueryType", "ReplyBuffer"]), offset=0),
    "PLSA_REDIRECTED_LOGON_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Buffer", "BufferLength", "ReturnBuffer", "ReturnBufferLength"]), offset=0),
    "PLSA_REDIRECTED_LOGON_CLEANUP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["RedirectedLogonHandle"]), offset=0),
    "PLSA_REDIRECTED_LOGON_GET_LOGON_CREDS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "LogonBuffer", "LogonBufferLength"]), offset=0),
    "PLSA_REDIRECTED_LOGON_GET_SID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Sid"]), offset=0),
    "PLSA_REDIRECTED_LOGON_GET_SUPP_CREDS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "SupplementalCredentials"]), offset=0),
    "PLSA_REDIRECTED_LOGON_INIT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "PackageName", "SessionId", "LogonId"]), offset=0),
    "PLSA_REGISTER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Argument1", "Argument2", "InputBuffer", "OutputBuffer"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackId", "Callback"]), offset=0),
    "PLSA_REGISTER_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["StartFunction", "Parameter", "NotificationType", "NotificationClass", "NotificationFlags", "IntervalMinutes", "WaitEvent"]), offset=0),
    "PLSA_SAVE_SUPPLEMENTAL_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "SupplementalCredSize", "SupplementalCreds", "Synchronous"]), offset=0),
    "PLSA_SET_APP_MODE_INFO": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["UserFunction", "Argument1", "Argument2", "UserData", "ReturnToLsa"]), offset=0),
    "PLSA_UNLOAD_PACKAGE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PLSA_UPDATE_PRIMARY_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrimaryCredentials", "Credentials"]), offset=0),
    "PLUGIN_TYPE": SimTypeInt(signed=False, label="PLUGIN_TYPE"),
    "PLUGPLAY_NOTIFICATION_HEADER": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Event", SimTypeBottom(label="Guid")),)), name="PLUGPLAY_NOTIFICATION_HEADER", pack=False, align=None),
    "PMAP_TRANSFER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte")], SimTypeLongLong(signed=True, label="Int64"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "CurrentVa", "Length", "WriteToDevice"]), offset=0),
    "PMAP_TRANSFER_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DmaAdapter", "Mdl", "MapRegisterBase", "Offset", "DeviceOffset", "Length", "WriteToDevice", "ScatterGatherBuffer", "ScatterGatherBufferLength", "DmaCompletionRoutine", "CompletionContext"]), offset=0),
    "PMDNS_QUERY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MDNS_QUERY_HANDLE", SimStruct), offset=0), SimTypePointer(SimTypeRef("DNS_QUERY_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pQueryContext", "pQueryHandle", "pQueryResults"]), offset=0),
    "PMF_FLAGS": SimTypeInt(signed=False, label="PMF_FLAGS"),
    "PMGM_CREATION_ALERT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MGM_IF_ENTRY", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwInIfIndex", "dwInIfNextHopAddr", "dwIfCount", "pmieOutIfList"]), offset=0),
    "PMGM_DISABLE_IGMP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwIfIndex", "dwIfNextHopAddr"]), offset=0),
    "PMGM_ENABLE_IGMP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwIfIndex", "dwIfNextHopAddr"]), offset=0),
    "PMGM_JOIN_ALERT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "bMemberUpdate"]), offset=0),
    "PMGM_LOCAL_JOIN_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwIfIndex", "dwIfNextHopAddr"]), offset=0),
    "PMGM_LOCAL_LEAVE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwIfIndex", "dwIfNextHopAddr"]), offset=0),
    "PMGM_PRUNE_ALERT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwIfIndex", "dwIfNextHopAddr", "bMemberDelete", "pdwTimeout"]), offset=0),
    "PMGM_RPF_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "pdwInIfIndex", "pdwInIfNextHopAddr", "pdwUpStreamNbr", "dwHdrSize", "pbPacketHdr", "pbRoute"]), offset=0),
    "PMGM_WRONG_IF_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwGroupAddr", "dwIfIndex", "dwIfNextHopAddr", "dwHdrSize", "pbPacketHdr"]), offset=0),
    "PMKID_CANDIDATE": SimStruct(OrderedDict((("BSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="PMKID_CANDIDATE", pack=False, align=None),
    "PMM_DLL_INITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RegistryPath"]), offset=0),
    "PMM_DLL_UNLOAD": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PMM_GET_SYSTEM_ROUTINE_ADDRESS_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ModuleName", "FunctionName"]), offset=0),
    "PMM_MDL_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PMM_ROTATE_COPY_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DestinationMdl", "SourceMdl", "Context"]), offset=0),
    "PMPRADMINACCEPTNEWCONNECTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PMPRADMINACCEPTNEWCONNECTION2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PMPRADMINACCEPTNEWCONNECTION3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_2", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_3", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PMPRADMINACCEPTNEWCONNECTIONEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PMPRADMINACCEPTNEWLINK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_PORT_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_PORT_1", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PMPRADMINACCEPTREAUTHENTICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_2", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_3", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PMPRADMINACCEPTREAUTHENTICATIONEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PMPRADMINACCEPTTUNNELENDPOINTCHANGEEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PMPRADMINCONNECTIONHANGUPNOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PMPRADMINCONNECTIONHANGUPNOTIFICATION2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_2", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PMPRADMINCONNECTIONHANGUPNOTIFICATION3": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_1", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_CONNECTION_2", SimStruct), offset=0), SimTypeRef("RAS_CONNECTION_3", SimStruct)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PMPRADMINCONNECTIONHANGUPNOTIFICATIONEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_CONNECTION_EX", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "PMPRADMINGETIPADDRESSFORUSER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PMPRADMINGETIPV6ADDRESSFORUSER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("IN6_ADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PMPRADMINLINKHANGUPNOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RAS_PORT_0", SimStruct), offset=0), SimTypePointer(SimTypeRef("RAS_PORT_1", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PMPRADMINRASVALIDATEPREAUTHENTICATEDCONNECTIONEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("AUTH_VALIDATION_EX", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "PMPRADMINRELEASEIPADRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PMPRADMINRELEASEIPV6ADDRESSFORUSER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("IN6_ADDR", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PMPRADMINTERMINATEDLL": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PMQRECEIVECALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MQMSGPROPS", SimStruct), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hrStatus", "hSource", "dwTimeout", "dwAction", "pMessageProps", "lpOverlapped", "hCursor"]), offset=0),
    "PMSIHANDLE": SimStruct(OrderedDict((("m_h", SimTypeInt(signed=False, label="UInt32")),)), name="PMSIHANDLE", pack=False, align=None),
    "PMTUD_STATE": SimTypeInt(signed=False, label="PMTUD_STATE"),
    "PM_ACTIVATION_POLICY": SimTypeInt(signed=False, label="PM_ACTIVATION_POLICY"),
    "PM_APPLICATION_HUBTYPE": SimTypeInt(signed=False, label="PM_APPLICATION_HUBTYPE"),
    "PM_APPLICATION_INSTALL_TYPE": SimTypeInt(signed=False, label="PM_APPLICATION_INSTALL_TYPE"),
    "PM_APPLICATION_STATE": SimTypeInt(signed=False, label="PM_APPLICATION_STATE"),
    "PM_APPTASKTYPE": SimStruct(OrderedDict((("ProductID", SimTypeBottom(label="Guid")), ("TaskType", SimTypeInt(signed=False, label="PM_TASK_TYPE")),)), name="PM_APPTASKTYPE", pack=False, align=None),
    "PM_APP_GENRE": SimTypeInt(signed=False, label="PM_APP_GENRE"),
    "PM_BSATASKID": SimStruct(OrderedDict((("ProductID", SimTypeBottom(label="Guid")), ("TaskID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PM_BSATASKID", pack=False, align=None),
    "PM_BWTASKID": SimStruct(OrderedDict((("ProductID", SimTypeBottom(label="Guid")), ("TaskID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PM_BWTASKID", pack=False, align=None),
    "PM_CLOSE_PROC": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PM_COLLECT_PROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pValueName", "ppData", "pcbTotalBytes", "pNumObjectTypes"]), offset=0),
    "PM_DISPATCH_TABLE": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="PM_DISPATCH_TABLE", pack=False, align=None),
    "PM_ENUM_APP_FILTER": SimTypeInt(signed=False, label="PM_ENUM_APP_FILTER"),
    "PM_ENUM_BSA_FILTER": SimTypeInt(signed=False, label="PM_ENUM_BSA_FILTER"),
    "PM_ENUM_BW_FILTER": SimTypeInt(signed=False, label="PM_ENUM_BW_FILTER"),
    "PM_ENUM_EXTENSION_FILTER": SimTypeInt(signed=False, label="PM_ENUM_EXTENSION_FILTER"),
    "PM_ENUM_FILTER": SimStruct(OrderedDict((("FilterType", SimTypeInt(signed=True, label="Int32")), ("FilterParameter", SimUnion({"Dummy": SimTypeInt(signed=True, label="Int32"), "Genre": SimTypeInt(signed=False, label="PM_APP_GENRE"), "AppHubType": SimTypeInt(signed=False, label="PM_APPLICATION_HUBTYPE"), "HubType": SimTypeInt(signed=False, label="PM_TILE_HUBTYPE"), "Tasktype": SimTypeInt(signed=False, label="PM_TASK_TYPE"), "TaskProductID": SimTypeBottom(label="Guid"), "TileProductID": SimTypeBottom(label="Guid"), "AppTaskType": SimTypeRef("PM_APPTASKTYPE", SimStruct), "Consumer": SimTypeRef("PM_EXTENSIONCONSUMER", SimStruct), "BSATask": SimTypeRef("PM_BSATASKID", SimStruct), "BSAProductID": SimTypeBottom(label="Guid"), "BWTask": SimTypeRef("PM_BWTASKID", SimStruct), "ProtocolName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "FileType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ContentType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "AppSupportedFileExtPID": SimTypeBottom(label="Guid"), "ShareTargetFileType": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="PM_ENUM_FILTER", pack=False, align=None),
    "PM_ENUM_TASK_FILTER": SimTypeInt(signed=False, label="PM_ENUM_TASK_FILTER"),
    "PM_ENUM_TILE_FILTER": SimTypeInt(signed=False, label="PM_ENUM_TILE_FILTER"),
    "PM_EXTENSIONCONSUMER": SimStruct(OrderedDict((("ConsumerPID", SimTypeBottom(label="Guid")), ("ExtensionID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PM_EXTENSIONCONSUMER", pack=False, align=None),
    "PM_INSTALLINFO": SimStruct(OrderedDict((("ProductID", SimTypeBottom(label="Guid")), ("PackagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InstanceID", SimTypeBottom(label="Guid")), ("pbLicense", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLicense", SimTypeInt(signed=False, label="UInt32")), ("IsUninstallDisabled", SimTypeInt(signed=True, label="Int32")), ("DeploymentOptions", SimTypeInt(signed=False, label="UInt32")), ("OfferID", SimTypeBottom(label="Guid")), ("MarketplaceAppVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PM_INSTALLINFO", pack=False, align=None),
    "PM_INVOCATIONINFO": SimStruct(OrderedDict((("URIBaseOrAUMID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("URIFragmentOrArgs", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PM_INVOCATIONINFO", pack=False, align=None),
    "PM_LIVETILE_RECURRENCE_TYPE": SimTypeInt(signed=False, label="PM_LIVETILE_RECURRENCE_TYPE"),
    "PM_LOGO_SIZE": SimTypeInt(signed=False, label="PM_LOGO_SIZE"),
    "PM_OPEN_PROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pContext"]), offset=0),
    "PM_STARTAPPBLOB": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ProductID", SimTypeBottom(label="Guid")), ("AppTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IconPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsUninstallable", SimTypeInt(signed=True, label="Int32")), ("AppInstallType", SimTypeInt(signed=False, label="PM_APPLICATION_INSTALL_TYPE")), ("InstanceID", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="PM_APPLICATION_STATE")), ("IsModern", SimTypeInt(signed=True, label="Int32")), ("IsModernLightUp", SimTypeInt(signed=True, label="Int32")), ("LightUpSupportMask", SimTypeShort(signed=False, label="UInt16")),)), name="PM_STARTAPPBLOB", pack=False, align=None),
    "PM_STARTTILEBLOB": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ProductID", SimTypeBottom(label="Guid")), ("TileID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TemplateType", SimTypeInt(signed=False, label="TILE_TEMPLATE_TYPE")), ("HubPosition", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("HubVisibilityBitmask", SimTypeInt(signed=False, label="UInt32")), ("IsDefault", SimTypeInt(signed=True, label="Int32")), ("TileType", SimTypeInt(signed=False, label="PM_STARTTILE_TYPE")), ("pbPropBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPropBlob", SimTypeInt(signed=False, label="UInt32")), ("IsRestoring", SimTypeInt(signed=True, label="Int32")), ("IsModern", SimTypeInt(signed=True, label="Int32")), ("InvocationInfo", SimTypeRef("PM_INVOCATIONINFO", SimStruct)),)), name="PM_STARTTILEBLOB", pack=False, align=None),
    "PM_STARTTILE_TYPE": SimTypeInt(signed=False, label="PM_STARTTILE_TYPE"),
    "PM_TASK_TRANSITION": SimTypeInt(signed=False, label="PM_TASK_TRANSITION"),
    "PM_TASK_TYPE": SimTypeInt(signed=False, label="PM_TASK_TYPE"),
    "PM_TILE_HUBTYPE": SimTypeInt(signed=False, label="PM_TILE_HUBTYPE"),
    "PM_TILE_SIZE": SimTypeInt(signed=False, label="PM_TILE_SIZE"),
    "PM_UPDATEINFO": SimStruct(OrderedDict((("ProductID", SimTypeBottom(label="Guid")), ("PackagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InstanceID", SimTypeBottom(label="Guid")), ("pbLicense", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLicense", SimTypeInt(signed=False, label="UInt32")), ("MarketplaceAppVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DeploymentOptions", SimTypeInt(signed=False, label="UInt32")),)), name="PM_UPDATEINFO", pack=False, align=None),
    "PM_UPDATEINFO_LEGACY": SimStruct(OrderedDict((("ProductID", SimTypeBottom(label="Guid")), ("PackagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InstanceID", SimTypeBottom(label="Guid")), ("pbLicense", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLicense", SimTypeInt(signed=False, label="UInt32")), ("MarketplaceAppVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PM_UPDATEINFO_LEGACY", pack=False, align=None),
    "PNDIS_TIMER_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PNDR_ASYNC_MESSAGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PNDR_CORRELATION_INFO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PNETISO_EDP_ID_CALLBACK_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["context", "wszEnterpriseId", "dwErr"]), offset=0),
    "PNETWORK_CONNECTIVITY_HINT_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeRef("NL_NETWORK_CONNECTIVITY_HINT", SimStruct)], SimTypeBottom(label="Void"), arg_names=["CallerContext", "ConnectivityHint"]), offset=0),
    "PNMI_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PNOTIFY_SYNC": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PNPEM_CONTROL_ENABLE_DISABLE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PNPEM_CONTROL_QUERY_CONTROL": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PNPEM_CONTROL_QUERY_STANDARD_CAPABILITIES": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PNPEM_CONTROL_SET_STANDARD_CONTROL": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PNP_BUS_INFORMATION": SimStruct(OrderedDict((("BusTypeGuid", SimTypeBottom(label="Guid")), ("LegacyBusType", SimTypeInt(signed=False, label="INTERFACE_TYPE")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")),)), name="PNP_BUS_INFORMATION", pack=False, align=None),
    "PNP_EXTENDED_ADDRESS_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("QueryExtendedAddress", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "ExtendedAddress"]), offset=0)),)), name="PNP_EXTENDED_ADDRESS_INTERFACE", pack=False, align=None),
    "PNP_LOCATION_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("GetLocationString", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "LocationStrings"]), offset=0)),)), name="PNP_LOCATION_INTERFACE", pack=False, align=None),
    "PNP_REPLACE_DRIVER_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Unload", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("BeginReplace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PNP_REPLACE_PARAMETERS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Parameters", "Context"]), offset=0)), ("EndReplace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0)), ("MirrorPhysicalMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "PhysicalAddress", "ByteCount"]), offset=0)), ("SetProcessorId", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ApicId", "Target"]), offset=0)), ("Swap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0)), ("InitiateHardwareMirror", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0)), ("MirrorPlatformMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0)), ("GetMemoryDestination", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "SourceAddress", "DestinationAddress"]), offset=0)), ("EnableDisableHardwareQuiesce", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Enable"]), offset=0)),)), name="PNP_REPLACE_DRIVER_INTERFACE", pack=False, align=None),
    "PNP_REPLACE_MEMORY_LIST": SimStruct(OrderedDict((("AllocatedCount", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("TotalLength", SimTypeLongLong(signed=False, label="UInt64")), ("Ranges", SimTypePointer(SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="PNP_REPLACE_MEMORY_LIST", pack=False, align=None),
    "PNP_REPLACE_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Target", SimTypeLongLong(signed=False, label="UInt64")), ("Spare", SimTypeLongLong(signed=False, label="UInt64")), ("TargetProcessors", SimTypePointer(SimTypeRef("PNP_REPLACE_PROCESSOR_LIST", SimStruct), offset=0)), ("SpareProcessors", SimTypePointer(SimTypeRef("PNP_REPLACE_PROCESSOR_LIST", SimStruct), offset=0)), ("TargetMemory", SimTypePointer(SimTypeRef("PNP_REPLACE_MEMORY_LIST", SimStruct), offset=0)), ("SpareMemory", SimTypePointer(SimTypeRef("PNP_REPLACE_MEMORY_LIST", SimStruct), offset=0)), ("MapMemory", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["TargetPhysicalAddress", "SparePhysicalAddress", "NumberOfBytes", "TargetAddress", "SpareAddress"]), offset=0)),)), name="PNP_REPLACE_PARAMETERS", pack=False, align=None),
    "PNP_REPLACE_PROCESSOR_LIST": SimStruct(OrderedDict((("Affinity", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)), ("GroupCount", SimTypeInt(signed=False, label="UInt32")), ("AllocatedCount", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("ApicIds", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PNP_REPLACE_PROCESSOR_LIST", pack=False, align=None),
    "PNP_REPLACE_PROCESSOR_LIST_V1": SimStruct(OrderedDict((("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AllocatedCount", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("ApicIds", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PNP_REPLACE_PROCESSOR_LIST_V1", pack=False, align=None),
    "PNP_VETO_TYPE": SimTypeInt(signed=False, label="PNP_VETO_TYPE"),
    "PNRPCLOUDINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Cloud", SimTypeRef("PNRP_CLOUD_ID", SimStruct)), ("enCloudState", SimTypeInt(signed=False, label="PNRP_CLOUD_STATE")), ("enCloudFlags", SimTypeInt(signed=False, label="PNRP_CLOUD_FLAGS")),)), name="PNRPCLOUDINFO", pack=False, align=None),
    "PNRPINFO_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpwszIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxResolve", SimTypeInt(signed=False, label="UInt32")), ("dwTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwLifetime", SimTypeInt(signed=False, label="UInt32")), ("enResolveCriteria", SimTypeInt(signed=False, label="PNRP_RESOLVE_CRITERIA")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("saHint", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("enNameState", SimTypeInt(signed=False, label="PNRP_REGISTERED_ID_STATE")),)), name="PNRPINFO_V1", pack=False, align=None),
    "PNRPINFO_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpwszIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMaxResolve", SimTypeInt(signed=False, label="UInt32")), ("dwTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwLifetime", SimTypeInt(signed=False, label="UInt32")), ("enResolveCriteria", SimTypeInt(signed=False, label="PNRP_RESOLVE_CRITERIA")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("saHint", SimTypeRef("SOCKET_ADDRESS", SimStruct)), ("enNameState", SimTypeInt(signed=False, label="PNRP_REGISTERED_ID_STATE")), ("enExtendedPayloadType", SimTypeInt(signed=False, label="PNRP_EXTENDED_PAYLOAD_TYPE")), ("Anonymous", SimUnion({"blobPayload": SimTypeRef("BLOB", SimStruct), "pwszPayload": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="PNRPINFO_V2", pack=False, align=None),
    "PNRP_CLOUD_FLAGS": SimTypeInt(signed=False, label="PNRP_CLOUD_FLAGS"),
    "PNRP_CLOUD_ID": SimStruct(OrderedDict((("AddressFamily", SimTypeInt(signed=True, label="Int32")), ("Scope", SimTypeInt(signed=False, label="PNRP_SCOPE")), ("ScopeId", SimTypeInt(signed=False, label="UInt32")),)), name="PNRP_CLOUD_ID", pack=False, align=None),
    "PNRP_CLOUD_STATE": SimTypeInt(signed=False, label="PNRP_CLOUD_STATE"),
    "PNRP_EXTENDED_PAYLOAD_TYPE": SimTypeInt(signed=False, label="PNRP_EXTENDED_PAYLOAD_TYPE"),
    "PNRP_REGISTERED_ID_STATE": SimTypeInt(signed=False, label="PNRP_REGISTERED_ID_STATE"),
    "PNRP_RESOLVE_CRITERIA": SimTypeInt(signed=False, label="PNRP_RESOLVE_CRITERIA"),
    "PNRP_SCOPE": SimTypeInt(signed=False, label="PNRP_SCOPE"),
    "PNS_CONTEXT_COMMIT_FN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwAction"]), offset=0),
    "PNS_CONTEXT_CONNECT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pwszMachine"]), offset=0),
    "PNS_CONTEXT_DUMP_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pwszRouter", "ppwcArguments", "dwArgCount", "pvData"]), offset=0),
    "PNS_DLL_INIT_FN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwNetshVersion", "pReserved"]), offset=0),
    "PNS_DLL_STOP_FN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwReserved"]), offset=0),
    "PNS_HELPER_START_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pguidParent", "dwVersion"]), offset=0),
    "PNS_HELPER_STOP_FN": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwReserved"]), offset=0),
    "PNS_OSVERSIONCHECK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CIMOSType", "CIMOSProductSuite", "CIMOSVersion", "CIMOSBuildNumber", "CIMServicePackMajorVersion", "CIMServicePackMinorVersion", "uiReserved", "dwReserved"]), offset=0),
    "PNTFS_DEREF_EXPORTED_SECURITY_DESCRIPTOR": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "POBJECT_TYPE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "POB_POST_OPERATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("OB_POST_OPERATION_INFORMATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["RegistrationContext", "OperationInformation"]), offset=0),
    "POB_PRE_OPERATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("OB_PRE_OPERATION_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=False, label="OB_PREOP_CALLBACK_STATUS"), arg_names=["RegistrationContext", "OperationInformation"]), offset=0),
    "POFFLINE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0),
    "POFFLINE_V2_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "DestinationNodeName", "OfflineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0),
    "POHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "POINT": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")),)), name="POINT", pack=False, align=None),
    "POINTE": SimStruct(OrderedDict((("x", SimTypeInt(signed=False, label="UInt32")), ("y", SimTypeInt(signed=False, label="UInt32")),)), name="POINTE", pack=False, align=None),
    "POINTERINACTIVE": SimTypeInt(signed=False, label="POINTERINACTIVE"),
    "POINTER_BUTTON_CHANGE_TYPE": SimTypeInt(signed=False, label="POINTER_BUTTON_CHANGE_TYPE"),
    "POINTER_DEVICE_CURSOR_INFO": SimStruct(OrderedDict((("cursorId", SimTypeInt(signed=False, label="UInt32")), ("cursor", SimTypeInt(signed=False, label="POINTER_DEVICE_CURSOR_TYPE")),)), name="POINTER_DEVICE_CURSOR_INFO", pack=False, align=None),
    "POINTER_DEVICE_CURSOR_TYPE": SimTypeInt(signed=False, label="POINTER_DEVICE_CURSOR_TYPE"),
    "POINTER_DEVICE_INFO": SimStruct(OrderedDict((("displayOrientation", SimTypeInt(signed=False, label="UInt32")), ("device", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pointerDeviceType", SimTypeInt(signed=False, label="POINTER_DEVICE_TYPE")), ("monitor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("startingCursorId", SimTypeInt(signed=False, label="UInt32")), ("maxActiveContacts", SimTypeShort(signed=False, label="UInt16")), ("productString", SimTypeArray(SimTypeChar(label="Char"), 520)),)), name="POINTER_DEVICE_INFO", pack=False, align=None),
    "POINTER_DEVICE_PROPERTY": SimStruct(OrderedDict((("logicalMin", SimTypeInt(signed=True, label="Int32")), ("logicalMax", SimTypeInt(signed=True, label="Int32")), ("physicalMin", SimTypeInt(signed=True, label="Int32")), ("physicalMax", SimTypeInt(signed=True, label="Int32")), ("unit", SimTypeInt(signed=False, label="UInt32")), ("unitExponent", SimTypeInt(signed=False, label="UInt32")), ("usagePageId", SimTypeShort(signed=False, label="UInt16")), ("usageId", SimTypeShort(signed=False, label="UInt16")),)), name="POINTER_DEVICE_PROPERTY", pack=False, align=None),
    "POINTER_DEVICE_TYPE": SimTypeInt(signed=False, label="POINTER_DEVICE_TYPE"),
    "POINTER_FEEDBACK_MODE": SimTypeInt(signed=False, label="POINTER_FEEDBACK_MODE"),
    "POINTER_FLAGS": SimTypeInt(signed=False, label="POINTER_FLAGS"),
    "POINTER_GRAVITY": SimTypeInt(signed=False, label="POINTER_GRAVITY"),
    "POINTER_INFO": SimStruct(OrderedDict((("pointerType", SimTypeInt(signed=False, label="POINTER_INPUT_TYPE")), ("pointerId", SimTypeInt(signed=False, label="UInt32")), ("frameId", SimTypeInt(signed=False, label="UInt32")), ("pointerFlags", SimTypeInt(signed=False, label="POINTER_FLAGS")), ("sourceDevice", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndTarget", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ptPixelLocation", SimTypeRef("POINT", SimStruct)), ("ptHimetricLocation", SimTypeRef("POINT", SimStruct)), ("ptPixelLocationRaw", SimTypeRef("POINT", SimStruct)), ("ptHimetricLocationRaw", SimTypeRef("POINT", SimStruct)), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("historyCount", SimTypeInt(signed=False, label="UInt32")), ("InputData", SimTypeInt(signed=True, label="Int32")), ("dwKeyStates", SimTypeInt(signed=False, label="UInt32")), ("PerformanceCount", SimTypeLongLong(signed=False, label="UInt64")), ("ButtonChangeType", SimTypeInt(signed=False, label="POINTER_BUTTON_CHANGE_TYPE")),)), name="POINTER_INFO", pack=False, align=None),
    "POINTER_INPUT_TYPE": SimTypeInt(signed=False, label="POINTER_INPUT_TYPE"),
    "POINTER_PEN_INFO": SimStruct(OrderedDict((("pointerInfo", SimTypeRef("POINTER_INFO", SimStruct)), ("penFlags", SimTypeInt(signed=False, label="UInt32")), ("penMask", SimTypeInt(signed=False, label="UInt32")), ("pressure", SimTypeInt(signed=False, label="UInt32")), ("rotation", SimTypeInt(signed=False, label="UInt32")), ("tiltX", SimTypeInt(signed=True, label="Int32")), ("tiltY", SimTypeInt(signed=True, label="Int32")),)), name="POINTER_PEN_INFO", pack=False, align=None),
    "POINTER_SEARCH_PHYSICAL": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")), ("PointerMin", SimTypeLongLong(signed=False, label="UInt64")), ("PointerMax", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MatchOffsets", SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)), ("MatchOffsetsSize", SimTypeInt(signed=False, label="UInt32")), ("MatchOffsetsCount", SimTypeInt(signed=False, label="UInt32")),)), name="POINTER_SEARCH_PHYSICAL", pack=False, align=None),
    "POINTER_TOUCH_INFO": SimStruct(OrderedDict((("pointerInfo", SimTypeRef("POINTER_INFO", SimStruct)), ("touchFlags", SimTypeInt(signed=False, label="UInt32")), ("touchMask", SimTypeInt(signed=False, label="UInt32")), ("rcContact", SimTypeRef("RECT", SimStruct)), ("rcContactRaw", SimTypeRef("RECT", SimStruct)), ("orientation", SimTypeInt(signed=False, label="UInt32")), ("pressure", SimTypeInt(signed=False, label="UInt32")),)), name="POINTER_TOUCH_INFO", pack=False, align=None),
    "POINTER_TYPE_INFO": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="POINTER_INPUT_TYPE")), ("Anonymous", SimUnion({"touchInfo": SimTypeRef("POINTER_TOUCH_INFO", SimStruct), "penInfo": SimTypeRef("POINTER_PEN_INFO", SimStruct)}, name="<anon>", label="None")),)), name="POINTER_TYPE_INFO", pack=False, align=None),
    "POINTF": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="POINTF", pack=False, align=None),
    "POINTFIX": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")),)), name="POINTFIX", pack=False, align=None),
    "POINTFLOAT": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="POINTFLOAT", pack=False, align=None),
    "POINTFX": SimStruct(OrderedDict((("x", SimTypeRef("FIXED", SimStruct)), ("y", SimTypeRef("FIXED", SimStruct)),)), name="POINTFX", pack=False, align=None),
    "POINTL": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")),)), name="POINTL", pack=False, align=None),
    "POINTQF": SimStruct(OrderedDict((("x", SimTypeLongLong(signed=True, label="Int64")), ("y", SimTypeLongLong(signed=True, label="Int64")),)), name="POINTQF", pack=False, align=None),
    "POINTS": SimStruct(OrderedDict((("x", SimTypeShort(signed=True, label="Int16")), ("y", SimTypeShort(signed=True, label="Int16")),)), name="POINTS", pack=False, align=None),
    "POLICYPROPERTIES": SimTypeInt(signed=False, label="POLICYPROPERTIES"),
    "POLICYSETTINGSTATUSINFO": SimStruct(OrderedDict((("szKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szEventSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szEventLogName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwEventID", SimTypeInt(signed=False, label="UInt32")), ("dwErrorCode", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="SETTINGSTATUS")), ("timeLogged", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="POLICYSETTINGSTATUSINFO", pack=False, align=None),
    "POLICY_ACCOUNT_DOMAIN_INFO": SimStruct(OrderedDict((("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="POLICY_ACCOUNT_DOMAIN_INFO", pack=False, align=None),
    "POLICY_AUDIT_CATEGORIES_INFO": SimStruct(OrderedDict((("MaximumCategoryCount", SimTypeInt(signed=False, label="UInt32")), ("SubCategoriesInfo", SimTypePointer(SimTypeRef("POLICY_AUDIT_SUBCATEGORIES_INFO", SimStruct), offset=0)),)), name="POLICY_AUDIT_CATEGORIES_INFO", pack=False, align=None),
    "POLICY_AUDIT_EVENTS_INFO": SimStruct(OrderedDict((("AuditingMode", SimTypeChar(label="Byte")), ("EventAuditingOptions", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("MaximumAuditEventCount", SimTypeInt(signed=False, label="UInt32")),)), name="POLICY_AUDIT_EVENTS_INFO", pack=False, align=None),
    "POLICY_AUDIT_EVENT_TYPE": SimTypeInt(signed=False, label="POLICY_AUDIT_EVENT_TYPE"),
    "POLICY_AUDIT_FULL_QUERY_INFO": SimStruct(OrderedDict((("ShutDownOnFull", SimTypeChar(label="Byte")), ("LogIsFull", SimTypeChar(label="Byte")),)), name="POLICY_AUDIT_FULL_QUERY_INFO", pack=False, align=None),
    "POLICY_AUDIT_FULL_SET_INFO": SimStruct(OrderedDict((("ShutDownOnFull", SimTypeChar(label="Byte")),)), name="POLICY_AUDIT_FULL_SET_INFO", pack=False, align=None),
    "POLICY_AUDIT_LOG_INFO": SimStruct(OrderedDict((("AuditLogPercentFull", SimTypeInt(signed=False, label="UInt32")), ("MaximumLogSize", SimTypeInt(signed=False, label="UInt32")), ("AuditRetentionPeriod", SimTypeLongLong(signed=True, label="Int64")), ("AuditLogFullShutdownInProgress", SimTypeChar(label="Byte")), ("TimeToShutdown", SimTypeLongLong(signed=True, label="Int64")), ("NextAuditRecordId", SimTypeInt(signed=False, label="UInt32")),)), name="POLICY_AUDIT_LOG_INFO", pack=False, align=None),
    "POLICY_AUDIT_SID_ARRAY": SimStruct(OrderedDict((("UsersCount", SimTypeInt(signed=False, label="UInt32")), ("UserSidArray", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="POLICY_AUDIT_SID_ARRAY", pack=False, align=None),
    "POLICY_AUDIT_SUBCATEGORIES_INFO": SimStruct(OrderedDict((("MaximumSubCategoryCount", SimTypeInt(signed=False, label="UInt32")), ("EventAuditingOptions", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="POLICY_AUDIT_SUBCATEGORIES_INFO", pack=False, align=None),
    "POLICY_DATA": SimStruct(OrderedDict((("PolicyObjHdr", SimTypeRef("RsvpObjHdr", SimStruct)), ("usPeOffset", SimTypeShort(signed=False, label="UInt16")), ("usReserved", SimTypeShort(signed=False, label="UInt16")),)), name="POLICY_DATA", pack=False, align=None),
    "POLICY_DECISION": SimStruct(OrderedDict((("lpvResult", SimTypeInt(signed=False, label="UInt32")), ("wPolicyErrCode", SimTypeShort(signed=False, label="UInt16")), ("wPolicyErrValue", SimTypeShort(signed=False, label="UInt16")),)), name="POLICY_DECISION", pack=False, align=None),
    "POLICY_DEFAULT_QUOTA_INFO": SimStruct(OrderedDict((("QuotaLimits", SimTypeRef("QUOTA_LIMITS", SimStruct)),)), name="POLICY_DEFAULT_QUOTA_INFO", pack=False, align=None),
    "POLICY_DNS_DOMAIN_INFO": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DnsDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DnsForestName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainGuid", SimTypeBottom(label="Guid")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="POLICY_DNS_DOMAIN_INFO", pack=False, align=None),
    "POLICY_DOMAIN_EFS_INFO": SimStruct(OrderedDict((("InfoLength", SimTypeInt(signed=False, label="UInt32")), ("EfsBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="POLICY_DOMAIN_EFS_INFO", pack=False, align=None),
    "POLICY_DOMAIN_INFORMATION_CLASS": SimTypeInt(signed=False, label="POLICY_DOMAIN_INFORMATION_CLASS"),
    "POLICY_DOMAIN_KERBEROS_TICKET_INFO": SimStruct(OrderedDict((("AuthenticationOptions", SimTypeInt(signed=False, label="UInt32")), ("MaxServiceTicketAge", SimTypeLongLong(signed=True, label="Int64")), ("MaxTicketAge", SimTypeLongLong(signed=True, label="Int64")), ("MaxRenewAge", SimTypeLongLong(signed=True, label="Int64")), ("MaxClockSkew", SimTypeLongLong(signed=True, label="Int64")), ("Reserved", SimTypeLongLong(signed=True, label="Int64")),)), name="POLICY_DOMAIN_KERBEROS_TICKET_INFO", pack=False, align=None),
    "POLICY_ELEMENT": SimStruct(OrderedDict((("targetEndpointAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("issuerEndpointAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("issuedTokenParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("privacyNoticeLink", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("privacyNoticeVersion", SimTypeInt(signed=False, label="UInt32")), ("useManagedPresentation", SimTypeInt(signed=True, label="Int32")),)), name="POLICY_ELEMENT", pack=False, align=None),
    "POLICY_INFORMATION_CLASS": SimTypeInt(signed=False, label="POLICY_INFORMATION_CLASS"),
    "POLICY_LSA_SERVER_ROLE": SimTypeInt(signed=False, label="POLICY_LSA_SERVER_ROLE"),
    "POLICY_LSA_SERVER_ROLE_INFO": SimStruct(OrderedDict((("LsaServerRole", SimTypeInt(signed=False, label="POLICY_LSA_SERVER_ROLE")),)), name="POLICY_LSA_SERVER_ROLE_INFO", pack=False, align=None),
    "POLICY_MACHINE_ACCT_INFO": SimStruct(OrderedDict((("Rid", SimTypeInt(signed=False, label="UInt32")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="POLICY_MACHINE_ACCT_INFO", pack=False, align=None),
    "POLICY_MACHINE_ACCT_INFO2": SimStruct(OrderedDict((("Rid", SimTypeInt(signed=False, label="UInt32")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectGuid", SimTypeBottom(label="Guid")),)), name="POLICY_MACHINE_ACCT_INFO2", pack=False, align=None),
    "POLICY_MODIFICATION_INFO": SimStruct(OrderedDict((("ModifiedId", SimTypeLongLong(signed=True, label="Int64")), ("DatabaseCreationTime", SimTypeLongLong(signed=True, label="Int64")),)), name="POLICY_MODIFICATION_INFO", pack=False, align=None),
    "POLICY_NOTIFICATION_INFORMATION_CLASS": SimTypeInt(signed=False, label="POLICY_NOTIFICATION_INFORMATION_CLASS"),
    "POLICY_PD_ACCOUNT_INFO": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="POLICY_PD_ACCOUNT_INFO", pack=False, align=None),
    "POLICY_PRIMARY_DOMAIN_INFO": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="POLICY_PRIMARY_DOMAIN_INFO", pack=False, align=None),
    "POLICY_REPLICA_SOURCE_INFO": SimStruct(OrderedDict((("ReplicaSource", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ReplicaAccountName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="POLICY_REPLICA_SOURCE_INFO", pack=False, align=None),
    "POLYTEXTA": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("n", SimTypeInt(signed=False, label="UInt32")), ("lpstr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("uiFlags", SimTypeInt(signed=False, label="UInt32")), ("rcl", SimTypeRef("RECT", SimStruct)), ("pdx", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="POLYTEXTA", pack=False, align=None),
    "POLYTEXTW": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("n", SimTypeInt(signed=False, label="UInt32")), ("lpstr", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("uiFlags", SimTypeInt(signed=False, label="UInt32")), ("rcl", SimTypeRef("RECT", SimStruct)), ("pdx", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="POLYTEXTW", pack=False, align=None),
    "PONLINE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "EventHandle"]), offset=0),
    "PONLINE_V2_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "EventHandle", "OnlineFlags", "InBuffer", "InBufferSize", "Reserved"]), offset=0),
    "POOLCUE": SimStruct(OrderedDict((("ulOffset", SimTypeInt(signed=False, label="UInt32")),)), name="POOLCUE", pack=False, align=None),
    "POOLED_USAGE_AND_LIMITS": SimStruct(OrderedDict((("PeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagedPoolLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("NonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("NonPagedPoolLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="POOLED_USAGE_AND_LIMITS", pack=False, align=None),
    "POOLTABLE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cCues", SimTypeInt(signed=False, label="UInt32")),)), name="POOLTABLE", pack=False, align=None),
    "POOL_CREATE_EXTENDED_PARAMS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")),)), name="POOL_CREATE_EXTENDED_PARAMS", pack=False, align=None),
    "POOL_EXTENDED_PARAMETER": SimStruct(OrderedDict((("Anonymous1", SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous1_e__Struct", pack=False, align=None)), ("Anonymous2", SimUnion({"Reserved2": SimTypeLongLong(signed=False, label="UInt64"), "Reserved3": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Priority": SimTypeInt(signed=False, label="EX_POOL_PRIORITY"), "SecurePoolParams": SimTypePointer(SimTypeRef("POOL_EXTENDED_PARAMS_SECURE_POOL", SimStruct), offset=0), "PreferredNode": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="POOL_EXTENDED_PARAMETER", pack=False, align=None),
    "POOL_EXTENDED_PARAMETER_TYPE": SimTypeInt(signed=False, label="POOL_EXTENDED_PARAMETER_TYPE"),
    "POOL_EXTENDED_PARAMS_SECURE_POOL": SimStruct(OrderedDict((("SecurePoolHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Cookie", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SecurePoolFlags", SimTypeInt(signed=False, label="UInt32")),)), name="POOL_EXTENDED_PARAMS_SECURE_POOL", pack=False, align=None),
    "POOL_TYPE": SimTypeInt(signed=False, label="POOL_TYPE"),
    "POPEN_CLUSTER_CRYPT_PROVIDER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpszResource", "lpszProvider", "dwType", "dwFlags"]), offset=0),
    "POPEN_CLUSTER_CRYPT_PROVIDEREX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpszResource", "lpszKeyname", "lpszProvider", "dwType", "dwFlags"]), offset=0),
    "POPEN_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ResourceName", "ResourceKey", "ResourceHandle"]), offset=0),
    "POPEN_V2_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["ResourceName", "ResourceKey", "ResourceHandle", "OpenFlags"]), offset=0),
    "POPLOCK_FS_PREPOST_IRP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "Irp"]), offset=0),
    "POPLOCK_NOTIFY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("OPLOCK_NOTIFY_PARAMS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["NotifyParams"]), offset=0),
    "POPLOCK_WAIT_COMPLETE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "Irp"]), offset=0),
    "POPUPCHECKBACKGROUNDSTATES": SimTypeInt(signed=False, label="POPUPCHECKBACKGROUNDSTATES"),
    "POPUPCHECKSTATES": SimTypeInt(signed=False, label="POPUPCHECKSTATES"),
    "POPUPITEMFOCUSABLESTATES": SimTypeInt(signed=False, label="POPUPITEMFOCUSABLESTATES"),
    "POPUPITEMKBFOCUSSTATES": SimTypeInt(signed=False, label="POPUPITEMKBFOCUSSTATES"),
    "POPUPITEMSTATES": SimTypeInt(signed=False, label="POPUPITEMSTATES"),
    "POPUPSUBMENUHCHOTSTATES": SimTypeInt(signed=False, label="POPUPSUBMENUHCHOTSTATES"),
    "POPUPSUBMENUSTATES": SimTypeInt(signed=False, label="POPUPSUBMENUSTATES"),
    "PORT_DATA_1": SimStruct(OrderedDict((("sztPortName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("sztHostAddress", SimTypeArray(SimTypeChar(label="Char"), 49)), ("sztSNMPCommunity", SimTypeArray(SimTypeChar(label="Char"), 33)), ("dwDoubleSpool", SimTypeInt(signed=False, label="UInt32")), ("sztQueue", SimTypeArray(SimTypeChar(label="Char"), 33)), ("sztIPAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 540)), ("dwPortNumber", SimTypeInt(signed=False, label="UInt32")), ("dwSNMPEnabled", SimTypeInt(signed=False, label="UInt32")), ("dwSNMPDevIndex", SimTypeInt(signed=False, label="UInt32")),)), name="PORT_DATA_1", pack=False, align=None),
    "PORT_DATA_2": SimStruct(OrderedDict((("sztPortName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("sztHostAddress", SimTypeArray(SimTypeChar(label="Char"), 128)), ("sztSNMPCommunity", SimTypeArray(SimTypeChar(label="Char"), 33)), ("dwDoubleSpool", SimTypeInt(signed=False, label="UInt32")), ("sztQueue", SimTypeArray(SimTypeChar(label="Char"), 33)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 514)), ("dwPortNumber", SimTypeInt(signed=False, label="UInt32")), ("dwSNMPEnabled", SimTypeInt(signed=False, label="UInt32")), ("dwSNMPDevIndex", SimTypeInt(signed=False, label="UInt32")), ("dwPortMonitorMibIndex", SimTypeInt(signed=False, label="UInt32")),)), name="PORT_DATA_2", pack=False, align=None),
    "PORT_DATA_LIST_1": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cPortData", SimTypeInt(signed=False, label="UInt32")), ("pPortData", SimTypePointer(SimTypeRef("PORT_DATA_2", SimStruct), offset=0)),)), name="PORT_DATA_LIST_1", pack=False, align=None),
    "PORT_INFO_1A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PORT_INFO_1A", pack=False, align=None),
    "PORT_INFO_1W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PORT_INFO_1W", pack=False, align=None),
    "PORT_INFO_2A": SimStruct(OrderedDict((("pPortName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fPortType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PORT_INFO_2A", pack=False, align=None),
    "PORT_INFO_2W": SimStruct(OrderedDict((("pPortName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMonitorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fPortType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PORT_INFO_2W", pack=False, align=None),
    "PORT_INFO_3A": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("pszStatus", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSeverity", SimTypeInt(signed=False, label="UInt32")),)), name="PORT_INFO_3A", pack=False, align=None),
    "PORT_INFO_3W": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")), ("pszStatus", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSeverity", SimTypeInt(signed=False, label="UInt32")),)), name="PORT_INFO_3W", pack=False, align=None),
    "POSTBL": SimStruct(OrderedDict((("nPos", SimTypeShort(signed=False, label="UInt16")), ("szName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="POSTBL", pack=False, align=None),
    "POST_PROCESS_PARAMETERS": SimStruct(OrderedDict((("pszSessionId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRemoteIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRemoteIpPort", SimTypeInt(signed=False, label="UInt32")), ("pszLocalIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLocalIpPort", SimTypeInt(signed=False, label="UInt32")), ("BytesSent", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("pszCommand", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCommandParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszFullPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPhysicalPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FtpStatus", SimTypeInt(signed=False, label="UInt32")), ("FtpSubStatus", SimTypeInt(signed=False, label="UInt32")), ("hrStatus", SimTypeInt(signed=True, label="Int32")), ("SessionStartTime", SimTypeRef("FILETIME", SimStruct)), ("BytesSentPerSession", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReceivedPerSession", SimTypeLongLong(signed=False, label="UInt64")),)), name="POST_PROCESS_PARAMETERS", pack=False, align=None),
    "POST_UPGRADE_VERSION_INFO": SimStruct(OrderedDict((("newMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("newUpgradeVersion", SimTypeInt(signed=False, label="UInt32")), ("oldMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("oldUpgradeVersion", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypeInt(signed=False, label="UInt32")),)), name="POST_UPGRADE_VERSION_INFO", pack=False, align=None),
    "POUT_OF_PROCESS_FUNCTION_TABLE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("IMAGE_RUNTIME_FUNCTION_ENTRY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Process", "TableAddress", "Entries", "Functions"]), offset=0),
    "POWERBROADCAST_SETTING": SimStruct(OrderedDict((("PowerSetting", SimTypeBottom(label="Guid")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="POWERBROADCAST_SETTING", pack=False, align=None),
    "POWER_ACTION": SimTypeInt(signed=False, label="POWER_ACTION"),
    "POWER_ACTION_POLICY": SimStruct(OrderedDict((("Action", SimTypeInt(signed=False, label="POWER_ACTION")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("EventCode", SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")),)), name="POWER_ACTION_POLICY", pack=False, align=None),
    "POWER_ACTION_POLICY_EVENT_CODE": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE"),
    "POWER_COOLING_MODE": SimTypeInt(signed=False, label="POWER_COOLING_MODE"),
    "POWER_DATA_ACCESSOR": SimTypeInt(signed=False, label="POWER_DATA_ACCESSOR"),
    "POWER_IDLE_RESILIENCY": SimStruct(OrderedDict((("CoalescingTimeout", SimTypeInt(signed=False, label="UInt32")), ("IdleResiliencyPeriod", SimTypeInt(signed=False, label="UInt32")),)), name="POWER_IDLE_RESILIENCY", pack=False, align=None),
    "POWER_INFORMATION_LEVEL": SimTypeInt(signed=False, label="POWER_INFORMATION_LEVEL"),
    "POWER_MONITOR_INVOCATION": SimStruct(OrderedDict((("Console", SimTypeChar(label="Byte")), ("RequestReason", SimTypeInt(signed=False, label="POWER_MONITOR_REQUEST_REASON")),)), name="POWER_MONITOR_INVOCATION", pack=False, align=None),
    "POWER_MONITOR_REQUEST_REASON": SimTypeInt(signed=False, label="POWER_MONITOR_REQUEST_REASON"),
    "POWER_MONITOR_REQUEST_TYPE": SimTypeInt(signed=False, label="POWER_MONITOR_REQUEST_TYPE"),
    "POWER_PLATFORM_INFORMATION": SimStruct(OrderedDict((("AoAc", SimTypeChar(label="Byte")),)), name="POWER_PLATFORM_INFORMATION", pack=False, align=None),
    "POWER_PLATFORM_ROLE": SimTypeInt(signed=False, label="POWER_PLATFORM_ROLE"),
    "POWER_PLATFORM_ROLE_VERSION": SimTypeInt(signed=False, label="POWER_PLATFORM_ROLE_VERSION"),
    "POWER_POLICY": SimStruct(OrderedDict((("user", SimTypeRef("USER_POWER_POLICY", SimStruct)), ("mach", SimTypeRef("MACHINE_POWER_POLICY", SimStruct)),)), name="POWER_POLICY", pack=False, align=None),
    "POWER_REQUEST_CONTEXT_FLAGS": SimTypeInt(signed=False, label="POWER_REQUEST_CONTEXT_FLAGS"),
    "POWER_REQUEST_TYPE": SimTypeInt(signed=False, label="POWER_REQUEST_TYPE"),
    "POWER_SEQUENCE": SimStruct(OrderedDict((("SequenceD1", SimTypeInt(signed=False, label="UInt32")), ("SequenceD2", SimTypeInt(signed=False, label="UInt32")), ("SequenceD3", SimTypeInt(signed=False, label="UInt32")),)), name="POWER_SEQUENCE", pack=False, align=None),
    "POWER_SESSION_ALLOW_EXTERNAL_DMA_DEVICES": SimStruct(OrderedDict((("IsAllowed", SimTypeChar(label="Byte")),)), name="POWER_SESSION_ALLOW_EXTERNAL_DMA_DEVICES", pack=False, align=None),
    "POWER_SESSION_CONNECT": SimStruct(OrderedDict((("Connected", SimTypeChar(label="Byte")), ("Console", SimTypeChar(label="Byte")),)), name="POWER_SESSION_CONNECT", pack=False, align=None),
    "POWER_SESSION_RIT_STATE": SimStruct(OrderedDict((("Active", SimTypeChar(label="Byte")), ("LastInputTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="POWER_SESSION_RIT_STATE", pack=False, align=None),
    "POWER_SESSION_TIMEOUTS": SimStruct(OrderedDict((("InputTimeout", SimTypeInt(signed=False, label="UInt32")), ("DisplayTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="POWER_SESSION_TIMEOUTS", pack=False, align=None),
    "POWER_SESSION_WINLOGON": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("Console", SimTypeChar(label="Byte")), ("Locked", SimTypeChar(label="Byte")),)), name="POWER_SESSION_WINLOGON", pack=False, align=None),
    "POWER_SETTING_ALTITUDE": SimTypeInt(signed=False, label="POWER_SETTING_ALTITUDE"),
    "POWER_SETTING_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SettingGuid", "Value", "ValueLength", "Context"]), offset=0),
    "POWER_STATE": SimUnion({"SystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DeviceState": SimTypeInt(signed=False, label="DEVICE_POWER_STATE")}, name="<anon>", label="None"),
    "POWER_STATE_TYPE": SimTypeInt(signed=False, label="POWER_STATE_TYPE"),
    "POWER_THROTTLING_PROCESS_STATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ControlMask", SimTypeInt(signed=False, label="UInt32")), ("StateMask", SimTypeInt(signed=False, label="UInt32")),)), name="POWER_THROTTLING_PROCESS_STATE", pack=False, align=None),
    "POWER_THROTTLING_THREAD_STATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ControlMask", SimTypeInt(signed=False, label="UInt32")), ("StateMask", SimTypeInt(signed=False, label="UInt32")),)), name="POWER_THROTTLING_THREAD_STATE", pack=False, align=None),
    "POWER_USER_PRESENCE": SimStruct(OrderedDict((("UserPresence", SimTypeInt(signed=False, label="POWER_USER_PRESENCE_TYPE")),)), name="POWER_USER_PRESENCE", pack=False, align=None),
    "POWER_USER_PRESENCE_TYPE": SimTypeInt(signed=False, label="POWER_USER_PRESENCE_TYPE"),
    "PO_FX_COMPONENT_ACTIVE_CONDITION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Context", "Component"]), offset=0),
    "PO_FX_COMPONENT_CRITICAL_TRANSITION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Context", "Component", "Active"]), offset=0),
    "PO_FX_COMPONENT_IDLE_CONDITION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Context", "Component"]), offset=0),
    "PO_FX_COMPONENT_IDLE_STATE": SimStruct(OrderedDict((("TransitionLatency", SimTypeLongLong(signed=False, label="UInt64")), ("ResidencyRequirement", SimTypeLongLong(signed=False, label="UInt64")), ("NominalPower", SimTypeInt(signed=False, label="UInt32")),)), name="PO_FX_COMPONENT_IDLE_STATE", pack=False, align=None),
    "PO_FX_COMPONENT_IDLE_STATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Context", "Component", "State"]), offset=0),
    "PO_FX_COMPONENT_PERF_INFO": SimStruct(OrderedDict((("PerfStateSetsCount", SimTypeInt(signed=False, label="UInt32")), ("PerfStateSets", SimTypePointer(SimTypeRef("PO_FX_COMPONENT_PERF_SET", SimStruct), offset=0)),)), name="PO_FX_COMPONENT_PERF_INFO", pack=False, align=None),
    "PO_FX_COMPONENT_PERF_SET": SimStruct(OrderedDict((("Name", SimTypeRef("UNICODE_STRING", SimStruct)), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("Unit", SimTypeInt(signed=False, label="PO_FX_PERF_STATE_UNIT")), ("Type", SimTypeInt(signed=False, label="PO_FX_PERF_STATE_TYPE")), ("Anonymous", SimUnion({"Discrete": SimTypeRef("_Discrete_e__Struct", SimStruct), "Range": SimTypeRef("_Range_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PO_FX_COMPONENT_PERF_SET", pack=False, align=None),
    "PO_FX_COMPONENT_PERF_STATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "Component", "Succeeded", "RequestContext"]), offset=0),
    "PO_FX_COMPONENT_V1": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("IdleStateCount", SimTypeInt(signed=False, label="UInt32")), ("DeepestWakeableIdleState", SimTypeInt(signed=False, label="UInt32")), ("IdleStates", SimTypePointer(SimTypeRef("PO_FX_COMPONENT_IDLE_STATE", SimStruct), offset=0)),)), name="PO_FX_COMPONENT_V1", pack=False, align=None),
    "PO_FX_COMPONENT_V2": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("DeepestWakeableIdleState", SimTypeInt(signed=False, label="UInt32")), ("IdleStateCount", SimTypeInt(signed=False, label="UInt32")), ("IdleStates", SimTypePointer(SimTypeRef("PO_FX_COMPONENT_IDLE_STATE", SimStruct), offset=0)), ("ProviderCount", SimTypeInt(signed=False, label="UInt32")), ("Providers", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PO_FX_COMPONENT_V2", pack=False, align=None),
    "PO_FX_DEVICE_POWER_NOT_REQUIRED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PO_FX_DEVICE_POWER_REQUIRED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PO_FX_DEVICE_V1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ComponentCount", SimTypeInt(signed=False, label="UInt32")), ("ComponentActiveConditionCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ComponentIdleConditionCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ComponentIdleStateCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DevicePowerRequiredCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DevicePowerNotRequiredCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("PowerControlCallback", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Components", SimTypePointer(SimTypeRef("PO_FX_COMPONENT_V1", SimStruct), offset=0)),)), name="PO_FX_DEVICE_V1", pack=False, align=None),
    "PO_FX_DEVICE_V2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("ComponentActiveConditionCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ComponentIdleConditionCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ComponentIdleStateCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DevicePowerRequiredCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DevicePowerNotRequiredCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("PowerControlCallback", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ComponentCount", SimTypeInt(signed=False, label="UInt32")), ("Components", SimTypePointer(SimTypeRef("PO_FX_COMPONENT_V2", SimStruct), offset=0)),)), name="PO_FX_DEVICE_V2", pack=False, align=None),
    "PO_FX_DEVICE_V3": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("ComponentActiveConditionCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ComponentIdleConditionCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("ComponentIdleStateCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DevicePowerRequiredCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DevicePowerNotRequiredCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("PowerControlCallback", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("DirectedPowerUpCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DirectedPowerDownCallback", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("DirectedFxTimeoutInSeconds", SimTypeInt(signed=False, label="UInt32")), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ComponentCount", SimTypeInt(signed=False, label="UInt32")), ("Components", SimTypePointer(SimTypeRef("PO_FX_COMPONENT_V2", SimStruct), offset=0)),)), name="PO_FX_DEVICE_V3", pack=False, align=None),
    "PO_FX_DIRECTED_POWER_DOWN_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Context", "Flags"]), offset=0),
    "PO_FX_DIRECTED_POWER_UP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Context", "Flags"]), offset=0),
    "PO_FX_DRIPS_WATCHDOG_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Context", "PhysicalDeviceObject", "UniqueId"]), offset=0),
    "PO_FX_PERF_STATE": SimStruct(OrderedDict((("Value", SimTypeLongLong(signed=False, label="UInt64")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PO_FX_PERF_STATE", pack=False, align=None),
    "PO_FX_PERF_STATE_CHANGE": SimStruct(OrderedDict((("Set", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"StateIndex": SimTypeInt(signed=False, label="UInt32"), "StateValue": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="PO_FX_PERF_STATE_CHANGE", pack=False, align=None),
    "PO_FX_PERF_STATE_TYPE": SimTypeInt(signed=False, label="PO_FX_PERF_STATE_TYPE"),
    "PO_FX_PERF_STATE_UNIT": SimTypeInt(signed=False, label="PO_FX_PERF_STATE_UNIT"),
    "PO_FX_POWER_CONTROL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceContext", "PowerControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0),
    "PO_THERMAL_REQUEST_TYPE": SimTypeInt(signed=False, label="PO_THERMAL_REQUEST_TYPE"),
    "PPATCH_PROGRESS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackContext", "CurrentPosition", "MaximumPosition"]), offset=0),
    "PPATCH_SYMLOAD_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["WhichFile", "SymbolFileName", "SymType", "SymbolFileCheckSum", "SymbolFileTimeDate", "ImageFileCheckSum", "ImageFileTimeDate", "CallbackContext"]), offset=0),
    "PPCI_EXPRESS_ENTER_LINK_QUIESCENT_MODE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_EXPRESS_EXIT_LINK_QUIESCENT_MODE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_EXPRESS_ROOT_PORT_READ_CONFIG_SPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Buffer", "Offset", "Length"]), offset=0),
    "PPCI_EXPRESS_ROOT_PORT_WRITE_CONFIG_SPACE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Buffer", "Offset", "Length"]), offset=0),
    "PPCI_EXPRESS_WAKE_CONTROL": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPCI_IS_DEVICE_PRESENT": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PPCI_IS_DEVICE_PRESENT_EX": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PPCI_LINE_TO_PIN": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPCI_MSIX_GET_ENTRY": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_MSIX_GET_TABLE_SIZE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_MSIX_MASKUNMASK_ENTRY": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_MSIX_SET_ENTRY": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_PIN_TO_LINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPCI_PREPARE_MULTISTAGE_RESUME": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPCI_READ_WRITE_CONFIG": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PPCI_ROOT_BUS_CAPABILITY": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPCI_SET_ACS": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_SET_ACS2": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCI_SET_ATS": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCW_BUFFER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PPCW_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPCW_INSTANCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PPCW_REGISTRATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PPHYSICAL_COUNTER_EVENT_BUFFER_OVERFLOW_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["EventBuffer", "EntrySize", "NumberOfEntries", "OwningHandle"]), offset=0),
    "PPHYSICAL_COUNTER_OVERFLOW_HANDLER": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["OverflowBits", "OwningHandle"]), offset=0),
    "PPM_IDLESTATE_EVENT": SimStruct(OrderedDict((("NewState", SimTypeInt(signed=False, label="UInt32")), ("OldState", SimTypeInt(signed=False, label="UInt32")), ("Processors", SimTypeLongLong(signed=False, label="UInt64")),)), name="PPM_IDLESTATE_EVENT", pack=False, align=None),
    "PPM_IDLE_ACCOUNTING": SimStruct(OrderedDict((("StateCount", SimTypeInt(signed=False, label="UInt32")), ("TotalTransitions", SimTypeInt(signed=False, label="UInt32")), ("ResetCount", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypePointer(SimTypeRef("PPM_IDLE_STATE_ACCOUNTING", SimStruct), offset=0)),)), name="PPM_IDLE_ACCOUNTING", pack=False, align=None),
    "PPM_IDLE_ACCOUNTING_EX": SimStruct(OrderedDict((("StateCount", SimTypeInt(signed=False, label="UInt32")), ("TotalTransitions", SimTypeInt(signed=False, label="UInt32")), ("ResetCount", SimTypeInt(signed=False, label="UInt32")), ("AbortCount", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypePointer(SimTypeRef("PPM_IDLE_STATE_ACCOUNTING_EX", SimStruct), offset=0)),)), name="PPM_IDLE_ACCOUNTING_EX", pack=False, align=None),
    "PPM_IDLE_STATE_ACCOUNTING": SimStruct(OrderedDict((("IdleTransitions", SimTypeInt(signed=False, label="UInt32")), ("FailedTransitions", SimTypeInt(signed=False, label="UInt32")), ("InvalidBucketIndex", SimTypeInt(signed=False, label="UInt32")), ("TotalTime", SimTypeLongLong(signed=False, label="UInt64")), ("IdleTimeBuckets", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)),)), name="PPM_IDLE_STATE_ACCOUNTING", pack=False, align=None),
    "PPM_IDLE_STATE_ACCOUNTING_EX": SimStruct(OrderedDict((("TotalTime", SimTypeLongLong(signed=False, label="UInt64")), ("IdleTransitions", SimTypeInt(signed=False, label="UInt32")), ("FailedTransitions", SimTypeInt(signed=False, label="UInt32")), ("InvalidBucketIndex", SimTypeInt(signed=False, label="UInt32")), ("MinTimeUs", SimTypeInt(signed=False, label="UInt32")), ("MaxTimeUs", SimTypeInt(signed=False, label="UInt32")), ("CancelledTransitions", SimTypeInt(signed=False, label="UInt32")), ("IdleTimeBuckets", SimTypeArray(SimTypeRef("PPM_IDLE_STATE_BUCKET_EX", SimStruct), 16)),)), name="PPM_IDLE_STATE_ACCOUNTING_EX", pack=False, align=None),
    "PPM_IDLE_STATE_BUCKET_EX": SimStruct(OrderedDict((("TotalTimeUs", SimTypeLongLong(signed=False, label="UInt64")), ("MinTimeUs", SimTypeInt(signed=False, label="UInt32")), ("MaxTimeUs", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")),)), name="PPM_IDLE_STATE_BUCKET_EX", pack=False, align=None),
    "PPM_PERFSTATE_DOMAIN_EVENT": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")), ("Latency", SimTypeInt(signed=False, label="UInt32")), ("Speed", SimTypeInt(signed=False, label="UInt32")), ("Processors", SimTypeLongLong(signed=False, label="UInt64")),)), name="PPM_PERFSTATE_DOMAIN_EVENT", pack=False, align=None),
    "PPM_PERFSTATE_EVENT": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("Latency", SimTypeInt(signed=False, label="UInt32")), ("Speed", SimTypeInt(signed=False, label="UInt32")), ("Processor", SimTypeInt(signed=False, label="UInt32")),)), name="PPM_PERFSTATE_EVENT", pack=False, align=None),
    "PPM_THERMALCHANGE_EVENT": SimStruct(OrderedDict((("ThermalConstraint", SimTypeInt(signed=False, label="UInt32")), ("Processors", SimTypeLongLong(signed=False, label="UInt64")),)), name="PPM_THERMALCHANGE_EVENT", pack=False, align=None),
    "PPM_THERMAL_POLICY_EVENT": SimStruct(OrderedDict((("Mode", SimTypeChar(label="Byte")), ("Processors", SimTypeLongLong(signed=False, label="UInt64")),)), name="PPM_THERMAL_POLICY_EVENT", pack=False, align=None),
    "PPM_WMI_IDLE_STATE": SimStruct(OrderedDict((("Latency", SimTypeInt(signed=False, label="UInt32")), ("Power", SimTypeInt(signed=False, label="UInt32")), ("TimeCheck", SimTypeInt(signed=False, label="UInt32")), ("PromotePercent", SimTypeChar(label="Byte")), ("DemotePercent", SimTypeChar(label="Byte")), ("StateType", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("StateFlags", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypeInt(signed=False, label="UInt32")), ("IdleHandler", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="PPM_WMI_IDLE_STATE", pack=False, align=None),
    "PPM_WMI_IDLE_STATES": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("TargetState", SimTypeInt(signed=False, label="UInt32")), ("OldState", SimTypeInt(signed=False, label="UInt32")), ("TargetProcessors", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypePointer(SimTypeRef("PPM_WMI_IDLE_STATE", SimStruct), offset=0)),)), name="PPM_WMI_IDLE_STATES", pack=False, align=None),
    "PPM_WMI_IDLE_STATES_EX": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("TargetState", SimTypeInt(signed=False, label="UInt32")), ("OldState", SimTypeInt(signed=False, label="UInt32")), ("TargetProcessors", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("State", SimTypePointer(SimTypeRef("PPM_WMI_IDLE_STATE", SimStruct), offset=0)),)), name="PPM_WMI_IDLE_STATES_EX", pack=False, align=None),
    "PPM_WMI_LEGACY_PERFSTATE": SimStruct(OrderedDict((("Frequency", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PercentFrequency", SimTypeInt(signed=False, label="UInt32")),)), name="PPM_WMI_LEGACY_PERFSTATE", pack=False, align=None),
    "PPM_WMI_PERF_STATE": SimStruct(OrderedDict((("Frequency", SimTypeInt(signed=False, label="UInt32")), ("Power", SimTypeInt(signed=False, label="UInt32")), ("PercentFrequency", SimTypeChar(label="Byte")), ("IncreaseLevel", SimTypeChar(label="Byte")), ("DecreaseLevel", SimTypeChar(label="Byte")), ("Type", SimTypeChar(label="Byte")), ("IncreaseTime", SimTypeInt(signed=False, label="UInt32")), ("DecreaseTime", SimTypeInt(signed=False, label="UInt32")), ("Control", SimTypeLongLong(signed=False, label="UInt64")), ("Status", SimTypeLongLong(signed=False, label="UInt64")), ("HitCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved3", SimTypeLongLong(signed=False, label="UInt64")),)), name="PPM_WMI_PERF_STATE", pack=False, align=None),
    "PPM_WMI_PERF_STATES": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("MaxFrequency", SimTypeInt(signed=False, label="UInt32")), ("CurrentState", SimTypeInt(signed=False, label="UInt32")), ("MaxPerfState", SimTypeInt(signed=False, label="UInt32")), ("MinPerfState", SimTypeInt(signed=False, label="UInt32")), ("LowestPerfState", SimTypeInt(signed=False, label="UInt32")), ("ThermalConstraint", SimTypeInt(signed=False, label="UInt32")), ("BusyAdjThreshold", SimTypeChar(label="Byte")), ("PolicyType", SimTypeChar(label="Byte")), ("Type", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TimerInterval", SimTypeInt(signed=False, label="UInt32")), ("TargetProcessors", SimTypeLongLong(signed=False, label="UInt64")), ("PStateHandler", SimTypeInt(signed=False, label="UInt32")), ("PStateContext", SimTypeInt(signed=False, label="UInt32")), ("TStateHandler", SimTypeInt(signed=False, label="UInt32")), ("TStateContext", SimTypeInt(signed=False, label="UInt32")), ("FeedbackHandler", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypePointer(SimTypeRef("PPM_WMI_PERF_STATE", SimStruct), offset=0)),)), name="PPM_WMI_PERF_STATES", pack=False, align=None),
    "PPM_WMI_PERF_STATES_EX": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("MaxFrequency", SimTypeInt(signed=False, label="UInt32")), ("CurrentState", SimTypeInt(signed=False, label="UInt32")), ("MaxPerfState", SimTypeInt(signed=False, label="UInt32")), ("MinPerfState", SimTypeInt(signed=False, label="UInt32")), ("LowestPerfState", SimTypeInt(signed=False, label="UInt32")), ("ThermalConstraint", SimTypeInt(signed=False, label="UInt32")), ("BusyAdjThreshold", SimTypeChar(label="Byte")), ("PolicyType", SimTypeChar(label="Byte")), ("Type", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TimerInterval", SimTypeInt(signed=False, label="UInt32")), ("TargetProcessors", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PStateHandler", SimTypeInt(signed=False, label="UInt32")), ("PStateContext", SimTypeInt(signed=False, label="UInt32")), ("TStateHandler", SimTypeInt(signed=False, label="UInt32")), ("TStateContext", SimTypeInt(signed=False, label="UInt32")), ("FeedbackHandler", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("State", SimTypePointer(SimTypeRef("PPM_WMI_PERF_STATE", SimStruct), offset=0)),)), name="PPM_WMI_PERF_STATES_EX", pack=False, align=None),
    "PPOWER_SETTING_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPO_FX_COMPONENT_ACTIVE_CONDITION_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_COMPONENT_CRITICAL_TRANSITION_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_COMPONENT_IDLE_CONDITION_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_COMPONENT_IDLE_STATE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_COMPONENT_PERF_STATE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_DEVICE_POWER_NOT_REQUIRED_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_DEVICE_POWER_REQUIRED_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_DIRECTED_POWER_DOWN_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_DIRECTED_POWER_UP_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_DRIPS_WATCHDOG_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPO_FX_POWER_CONTROL_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPP_ATCP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 33)),)), name="PPP_ATCP_INFO", pack=False, align=None),
    "PPP_CCP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteOptions", SimTypeInt(signed=False, label="UInt32")),)), name="PPP_CCP_INFO", pack=False, align=None),
    "PPP_EAP_ACTION": SimTypeInt(signed=False, label="PPP_EAP_ACTION"),
    "PPP_EAP_INFO": SimStruct(OrderedDict((("dwSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("RasEapInitialize", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RasEapBegin", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RasEapEnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RasEapMakeMessage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PPP_EAP_INFO", pack=False, align=None),
    "PPP_EAP_INPUT": SimStruct(OrderedDict((("dwSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("fFlags", SimTypeInt(signed=False, label="UInt32")), ("fAuthenticator", SimTypeInt(signed=True, label="Int32")), ("pwszIdentity", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bInitialId", SimTypeChar(label="Byte")), ("pUserAttributes", SimTypePointer(SimTypeRef("RAS_AUTH_ATTRIBUTE", SimStruct), offset=0)), ("fAuthenticationComplete", SimTypeInt(signed=True, label="Int32")), ("dwAuthResultCode", SimTypeInt(signed=False, label="UInt32")), ("hTokenImpersonateUser", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fSuccessPacketReceived", SimTypeInt(signed=True, label="Int32")), ("fDataReceivedFromInteractiveUI", SimTypeInt(signed=True, label="Int32")), ("pDataFromInteractiveUI", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeOfDataFromInteractiveUI", SimTypeInt(signed=False, label="UInt32")), ("pConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeOfConnectionData", SimTypeInt(signed=False, label="UInt32")), ("pUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeOfUserData", SimTypeInt(signed=False, label="UInt32")), ("hReserved", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("guidConnectionId", SimTypeBottom(label="Guid")), ("isVpn", SimTypeInt(signed=True, label="Int32")),)), name="PPP_EAP_INPUT", pack=False, align=None),
    "PPP_EAP_OUTPUT": SimStruct(OrderedDict((("dwSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="PPP_EAP_ACTION")), ("dwAuthResultCode", SimTypeInt(signed=False, label="UInt32")), ("pUserAttributes", SimTypePointer(SimTypeRef("RAS_AUTH_ATTRIBUTE", SimStruct), offset=0)), ("fInvokeInteractiveUI", SimTypeInt(signed=True, label="Int32")), ("pUIContextData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeOfUIContextData", SimTypeInt(signed=False, label="UInt32")), ("fSaveConnectionData", SimTypeInt(signed=True, label="Int32")), ("pConnectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeOfConnectionData", SimTypeInt(signed=False, label="UInt32")), ("fSaveUserData", SimTypeInt(signed=True, label="Int32")), ("pUserData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSizeOfUserData", SimTypeInt(signed=False, label="UInt32")), ("pNgcKerbTicket", SimTypePointer(SimTypeRef("NgcTicketContext", SimStruct), offset=0)), ("fSaveToCredMan", SimTypeInt(signed=True, label="Int32")),)), name="PPP_EAP_OUTPUT", pack=False, align=None),
    "PPP_EAP_PACKET": SimStruct(OrderedDict((("Code", SimTypeChar(label="Byte")), ("Id", SimTypeChar(label="Byte")), ("Length", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PPP_EAP_PACKET", pack=False, align=None),
    "PPP_INFO": SimStruct(OrderedDict((("nbf", SimTypeRef("PPP_NBFCP_INFO", SimStruct)), ("ip", SimTypeRef("PPP_IPCP_INFO", SimStruct)), ("ipx", SimTypeRef("PPP_IPXCP_INFO", SimStruct)), ("at", SimTypeRef("PPP_ATCP_INFO", SimStruct)),)), name="PPP_INFO", pack=False, align=None),
    "PPP_INFO_2": SimStruct(OrderedDict((("nbf", SimTypeRef("PPP_NBFCP_INFO", SimStruct)), ("ip", SimTypeRef("PPP_IPCP_INFO2", SimStruct)), ("ipx", SimTypeRef("PPP_IPXCP_INFO", SimStruct)), ("at", SimTypeRef("PPP_ATCP_INFO", SimStruct)), ("ccp", SimTypeRef("PPP_CCP_INFO", SimStruct)), ("lcp", SimTypeRef("PPP_LCP_INFO", SimStruct)),)), name="PPP_INFO_2", pack=False, align=None),
    "PPP_INFO_3": SimStruct(OrderedDict((("nbf", SimTypeRef("PPP_NBFCP_INFO", SimStruct)), ("ip", SimTypeRef("PPP_IPCP_INFO2", SimStruct)), ("ipv6", SimTypeRef("PPP_IPV6_CP_INFO", SimStruct)), ("ccp", SimTypeRef("PPP_CCP_INFO", SimStruct)), ("lcp", SimTypeRef("PPP_LCP_INFO", SimStruct)),)), name="PPP_INFO_3", pack=False, align=None),
    "PPP_IPCP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteAddress", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="PPP_IPCP_INFO", pack=False, align=None),
    "PPP_IPCP_INFO2": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteOptions", SimTypeInt(signed=False, label="UInt32")),)), name="PPP_IPCP_INFO2", pack=False, align=None),
    "PPP_IPV6_CP_INFO": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("bInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bRemoteInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteOptions", SimTypeInt(signed=False, label="UInt32")), ("bPrefix", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")),)), name="PPP_IPV6_CP_INFO", pack=False, align=None),
    "PPP_IPXCP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 23)),)), name="PPP_IPXCP_INFO", pack=False, align=None),
    "PPP_LCP": SimTypeInt(signed=False, label="PPP_LCP"),
    "PPP_LCP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="PPP_LCP")), ("dwAuthenticationData", SimTypeInt(signed=False, label="PPP_LCP_INFO_AUTH_DATA")), ("dwRemoteAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteAuthenticationData", SimTypeInt(signed=False, label="UInt32")), ("dwTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteOptions", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteEapTypeId", SimTypeInt(signed=False, label="UInt32")),)), name="PPP_LCP_INFO", pack=False, align=None),
    "PPP_LCP_INFO_AUTH_DATA": SimTypeInt(signed=False, label="PPP_LCP_INFO_AUTH_DATA"),
    "PPP_NBFCP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("wszWksta", SimTypeArray(SimTypeChar(label="Char"), 17)),)), name="PPP_NBFCP_INFO", pack=False, align=None),
    "PPP_PROJECTION_INFO": SimStruct(OrderedDict((("dwIPv4NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("dwIPv4Options", SimTypeInt(signed=False, label="UInt32")), ("dwIPv4RemoteOptions", SimTypeInt(signed=False, label="UInt32")), ("IPv4SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwIPv6NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("bInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bRemoteInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bPrefix", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")), ("IPv6SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwLcpError", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="PPP_LCP")), ("dwAuthenticationData", SimTypeInt(signed=False, label="PPP_LCP_INFO_AUTH_DATA")), ("dwRemoteAuthenticationProtocol", SimTypeInt(signed=False, label="PPP_LCP")), ("dwRemoteAuthenticationData", SimTypeInt(signed=False, label="PPP_LCP_INFO_AUTH_DATA")), ("dwLcpTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwLcpRemoteTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwLcpOptions", SimTypeInt(signed=False, label="UInt32")), ("dwLcpRemoteOptions", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwCcpError", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwCcpOptions", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwCcpRemoteOptions", SimTypeInt(signed=False, label="UInt32")),)), name="PPP_PROJECTION_INFO", pack=False, align=None),
    "PPP_PROJECTION_INFO2": SimStruct(OrderedDict((("dwIPv4NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("wszAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("dwIPv4Options", SimTypeInt(signed=False, label="UInt32")), ("dwIPv4RemoteOptions", SimTypeInt(signed=False, label="UInt32")), ("IPv4SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwIPv6NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("bInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bRemoteInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bPrefix", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")), ("IPv6SubInterfaceIndex", SimTypeLongLong(signed=False, label="UInt64")), ("dwLcpError", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="PPP_LCP")), ("dwAuthenticationData", SimTypeInt(signed=False, label="PPP_LCP_INFO_AUTH_DATA")), ("dwRemoteAuthenticationProtocol", SimTypeInt(signed=False, label="PPP_LCP")), ("dwRemoteAuthenticationData", SimTypeInt(signed=False, label="PPP_LCP_INFO_AUTH_DATA")), ("dwLcpTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwLcpRemoteTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwLcpOptions", SimTypeInt(signed=False, label="UInt32")), ("dwLcpRemoteOptions", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwEmbeddedEAPTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwCcpError", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwCcpOptions", SimTypeInt(signed=False, label="UInt32")), ("dwRemoteCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwCcpRemoteOptions", SimTypeInt(signed=False, label="UInt32")),)), name="PPP_PROJECTION_INFO2", pack=False, align=None),
    "PPROCESSOR_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPROCESSOR_HALT_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPS_POST_PROCESS_INIT_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PPTM_DEVICE_DISABLE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPTM_DEVICE_ENABLE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPTM_DEVICE_QUERY_GRANULARITY": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPTM_DEVICE_QUERY_TIME_SOURCE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PPTP_CONFIG_PARAMS": SimStruct(OrderedDict((("dwNumPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")),)), name="PPTP_CONFIG_PARAMS", pack=False, align=None),
    "PPUT_DMA_ADAPTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DmaAdapter"]), offset=0),
    "PPUT_SCATTER_GATHER_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), SimTypePointer(SimTypeRef("SCATTER_GATHER_LIST", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["DmaAdapter", "ScatterGather", "WriteToDevice"]), offset=0),
    "PQUERYACTCTXW_FUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwFlags", "hActCtx", "pvSubInstance", "ulInfoClass", "pvBuffer", "cbBuffer", "pcbWrittenOrRequired"]), offset=0),
    "PQUERYEXTENDEDADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "ExtendedAddress"]), offset=0),
    "PQUERYHANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("val_context", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["keycontext", "val_list", "num_vals", "outputbuffer", "total_outlen", "input_blen"]), offset=0),
    "PQUERY_APPINSTANCE_VERSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["AppInstanceId", "InstanceVersionHigh", "InstanceVersionLow", "VersionStatus"]), offset=0),
    "PQUERY_LOG_USAGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogHandle", "PercentageFull"]), offset=0),
    "PQUERY_VIRTUAL_MEMORY_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PQUORUM_RESOURCE_LOST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0),
    "PRADIUS_EXTENSION_FREE_ATTRIBUTES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RADIUS_ATTRIBUTE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pAttrs"]), offset=0),
    "PRADIUS_EXTENSION_INIT": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PRADIUS_EXTENSION_PROCESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RADIUS_ATTRIBUTE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="RADIUS_ACTION"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pAttrs", "pfAction"]), offset=0),
    "PRADIUS_EXTENSION_PROCESS_2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RADIUS_EXTENSION_CONTROL_BLOCK", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pECB"]), offset=0),
    "PRADIUS_EXTENSION_PROCESS_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RADIUS_ATTRIBUTE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("RADIUS_ATTRIBUTE", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="RADIUS_ACTION"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pInAttrs", "pOutAttrs", "pfAction"]), offset=0),
    "PRADIUS_EXTENSION_TERM": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PRAISE_RES_TYPE_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceType", "pPayload", "payloadSize"]), offset=0),
    "PREAD_DMA_COUNTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["DmaAdapter"]), offset=0),
    "PREAD_PROCESS_MEMORY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "lpBaseAddress", "lpBuffer", "nSize", "lpNumberOfBytesRead"]), offset=0),
    "PREAD_PROCESS_MEMORY_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "qwBaseAddress", "lpBuffer", "nSize", "lpNumberOfBytesRead"]), offset=0),
    "PREENUMERATE_SELF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PREFETCH_OPEN_ECP_CONTEXT": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PREFETCH_OPEN_ECP_CONTEXT", pack=False, align=None),
    "PREFIX_TABLE": SimStruct(OrderedDict((("NodeTypeCode", SimTypeShort(signed=True, label="Int16")), ("NameLength", SimTypeShort(signed=True, label="Int16")), ("NextPrefixTree", SimTypePointer(SimTypeRef("PREFIX_TABLE_ENTRY", SimStruct), offset=0)),)), name="PREFIX_TABLE", pack=False, align=None),
    "PREFIX_TABLE_ENTRY": SimStruct(OrderedDict((("NodeTypeCode", SimTypeShort(signed=True, label="Int16")), ("NameLength", SimTypeShort(signed=True, label="Int16")), ("NextPrefixTree", SimTypePointer(SimTypeRef("PREFIX_TABLE_ENTRY", SimStruct), offset=0)), ("Links", SimTypeRef("RTL_SPLAY_LINKS", SimStruct)), ("Prefix", SimTypePointer(SimTypeRef("STRING", SimStruct), offset=0)),)), name="PREFIX_TABLE_ENTRY", pack=False, align=None),
    "PREGISTERCLASSNAMEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0"]), offset=0),
    "PREGISTER_APPINSTANCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ProcessHandle", "AppInstanceId", "ChildrenInheritAppInstance"]), offset=0),
    "PREGISTER_APPINSTANCE_VERSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["AppInstanceId", "InstanceVersionHigh", "InstanceVersionLow"]), offset=0),
    "PREGISTER_FOR_DEVICE_NOTIFICATIONS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PREGISTER_FOR_DEVICE_NOTIFICATIONS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["NotificationContext", "NotifyCode"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "NotificationHandler", "NotificationContext"]), offset=0),
    "PRELEASE_FROM_LAZY_WRITE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PRELEASE_FROM_READ_AHEAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PRELEASE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource"]), offset=0),
    "PREPARE_TAPE_OPERATION": SimTypeInt(signed=False, label="PREPARE_TAPE_OPERATION"),
    "PREPLACE_BEGIN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PNP_REPLACE_PARAMETERS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Parameters", "Context"]), offset=0),
    "PREPLACE_DRIVER_INIT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PNP_REPLACE_DRIVER_INTERFACE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Interface", "Unused"]), offset=0),
    "PREPLACE_ENABLE_DISABLE_HARDWARE_QUIESCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Enable"]), offset=0),
    "PREPLACE_END": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PREPLACE_GET_MEMORY_DESTINATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "SourceAddress", "DestinationAddress"]), offset=0),
    "PREPLACE_INITIATE_HARDWARE_MIRROR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PREPLACE_MAP_MEMORY": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["TargetPhysicalAddress", "SparePhysicalAddress", "NumberOfBytes", "TargetAddress", "SpareAddress"]), offset=0),
    "PREPLACE_MIRROR_PHYSICAL_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "PhysicalAddress", "ByteCount"]), offset=0),
    "PREPLACE_MIRROR_PLATFORM_MEMORY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PREPLACE_SET_PROCESSOR_ID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ApicId", "Target"]), offset=0),
    "PREPLACE_SWAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PREPLACE_UNLOAD": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PREQUEST_DUMP_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "DumpDueToCallInProgress", "DumpDelayInMs"]), offset=0),
    "PREQUEST_POWER_COMPLETE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PRESET_ALL_APPINSTANCE_VERSIONS": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PRESOURCE_CONTROL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Resource", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0),
    "PRESOURCE_TYPE_CONTROL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceTypeName", "ControlCode", "InBuffer", "InBufferSize", "OutBuffer", "OutBufferSize", "BytesReturned"]), offset=0),
    "PRESUTIL_ADD_UNKNOWN_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "pOutPropertyList", "pcbOutPropertyListSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_CREATE_DIRECTORY_TREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszPath"]), offset=0),
    "PRESUTIL_DUP_PARAMETER_BLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pOutParams", "pInParams", "pPropertyTable"]), offset=0),
    "PRESUTIL_DUP_STRING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeChar(label="Char"), offset=0), arg_names=["pszInString"]), offset=0),
    "PRESUTIL_ENUM_PRIVATE_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszOutProperties", "cbOutPropertiesSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_ENUM_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyTable", "pszOutProperties", "cbOutPropertiesSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_ENUM_RESOURCES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hSelf", "lpszResTypeName", "pResCallBack", "pParameter"]), offset=0),
    "PRESUTIL_ENUM_RESOURCES_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "hSelf", "lpszResTypeName", "pResCallBack", "pParameter"]), offset=0),
    "PRESUTIL_ENUM_RESOURCES_EX2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "hSelf", "lpszResTypeName", "pResCallBack", "pParameter", "dwDesiredAccess"]), offset=0),
    "PRESUTIL_EXPAND_ENVIRONMENT_STRINGS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeChar(label="Char"), offset=0), arg_names=["pszSrc"]), offset=0),
    "PRESUTIL_FIND_BINARY_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pbPropertyValue", "pcbPropertyValueSize"]), offset=0),
    "PRESUTIL_FIND_DEPENDENT_DISK_RESOURCE_DRIVE_LETTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "hResource", "pszDriveLetter", "pcchDriveLetter"]), offset=0),
    "PRESUTIL_FIND_DWORD_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pdwPropertyValue"]), offset=0),
    "PRESUTIL_FIND_EXPANDED_SZ_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pszPropertyValue"]), offset=0),
    "PRESUTIL_FIND_EXPAND_SZ_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pszPropertyValue"]), offset=0),
    "PRESUTIL_FIND_FILETIME_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pftPropertyValue"]), offset=0),
    "PRESUTIL_FIND_LONG_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "plPropertyValue"]), offset=0),
    "PRESUTIL_FIND_MULTI_SZ_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pszPropertyValue", "pcbPropertyValueSize"]), offset=0),
    "PRESUTIL_FIND_SZ_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "pszPropertyValue"]), offset=0),
    "PRESUTIL_FIND_ULARGEINTEGER_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyList", "cbPropertyListSize", "pszPropertyName", "plPropertyValue"]), offset=0),
    "PRESUTIL_FREE_ENVIRONMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpEnvironment"]), offset=0),
    "PRESUTIL_FREE_PARAMETER_BLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pOutParams", "pInParams", "pPropertyTable"]), offset=0),
    "PRESUTIL_GET_ALL_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "pOutPropertyList", "cbOutPropertyListSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_GET_BINARY_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("CLUSPROP_BINARY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppbOutValue", "pcbOutValueSize", "pValueStruct", "pbOldValue", "cbOldValueSize", "ppPropertyList", "pcbPropertyListSize"]), offset=0),
    "PRESUTIL_GET_BINARY_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "ppbOutValue", "pcbOutValueSize"]), offset=0),
    "PRESUTIL_GET_CORE_CLUSTER_RESOURCES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hCluster", "phClusterNameResource", "phClusterIPAddressResource", "phClusterQuorumResource"]), offset=0),
    "PRESUTIL_GET_CORE_CLUSTER_RESOURCES_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hClusterIn", "phClusterNameResourceOut", "phClusterIPAddressResourceOut", "phClusterQuorumResourceOut", "dwDesiredAccess"]), offset=0),
    "PRESUTIL_GET_DWORD_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("CLUSPROP_DWORD", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pdwOutValue", "pValueStruct", "dwOldValue", "dwMinimum", "dwMaximum", "ppPropertyList", "pcbPropertyListSize"]), offset=0),
    "PRESUTIL_GET_DWORD_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "pdwOutValue", "dwDefaultValue"]), offset=0),
    "PRESUTIL_GET_ENVIRONMENT_WITH_NET_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["hResource"]), offset=0),
    "PRESUTIL_GET_EXPAND_SZ_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeChar(label="Char"), offset=0), arg_names=["hkeyClusterKey", "pszValueName", "bExpand"]), offset=0),
    "PRESUTIL_GET_FILETIME_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0), SimTypePointer(SimTypeRef("CLUSPROP_FILETIME", SimStruct), offset=0), SimTypeRef("FILETIME", SimStruct), SimTypeRef("FILETIME", SimStruct), SimTypeRef("FILETIME", SimStruct), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pftOutValue", "pValueStruct", "ftOldValue", "ftMinimum", "ftMaximum", "ppPropertyList", "pcbPropertyListSize"]), offset=0),
    "PRESUTIL_GET_LONG_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeRef("CLUSPROP_LONG", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["plOutValue", "pValueStruct", "lOldValue", "lMinimum", "lMaximum", "ppPropertyList", "pcbPropertyListSize"]), offset=0),
    "PRESUTIL_GET_MULTI_SZ_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("CLUSPROP_SZ", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppszOutValue", "pcbOutValueSize", "pValueStruct", "pszOldValue", "cbOldValueSize", "ppPropertyList", "pcbPropertyListSize"]), offset=0),
    "PRESUTIL_GET_PRIVATE_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pOutPropertyList", "cbOutPropertyListSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_GET_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "pOutPropertyList", "cbOutPropertyListSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_GET_PROPERTIES_TO_PARAMETER_BLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "pOutParams", "bCheckForRequiredProperties", "pszNameOfPropInError"]), offset=0),
    "PRESUTIL_GET_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTableItem", "pOutPropertyItem", "pcbOutPropertyItemSize"]), offset=0),
    "PRESUTIL_GET_PROPERTY_FORMATS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyTable", "pOutPropertyFormatList", "cbPropertyFormatListSize", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_GET_PROPERTY_SIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTableItem", "pcbOutPropertyListSize", "pnPropertyCount"]), offset=0),
    "PRESUTIL_GET_QWORD_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "pqwOutValue", "qwDefaultValue"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hSelf", "lpszResourceType"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENCY_BY_CLASS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CLUS_RESOURCE_CLASS_INFO", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "hSelf", "prci", "bRecurse"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENCY_BY_CLASS_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CLUS_RESOURCE_CLASS_INFO", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "hSelf", "prci", "bRecurse", "dwDesiredAccess"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENCY_BY_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "hSelf", "lpszResourceType", "bRecurse"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENCY_BY_NAME_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hCluster", "hSelf", "lpszResourceType", "bRecurse", "dwDesiredAccess"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hSelf", "lpszResourceType", "dwDesiredAccess"]), offset=0),
    "PRESUTIL_GET_RESOURCE_DEPENDENTIP_ADDRESS_PROPS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "pszAddress", "pcchAddress", "pszSubnetMask", "pcchSubnetMask", "pszNetwork", "pcchNetwork"]), offset=0),
    "PRESUTIL_GET_RESOURCE_NAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hResource", "pszResourceName", "pcchResourceNameInOut"]), offset=0),
    "PRESUTIL_GET_RESOURCE_NAME_DEPENDENCY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpszResourceName", "lpszResourceType"]), offset=0),
    "PRESUTIL_GET_RESOURCE_NAME_DEPENDENCY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpszResourceName", "lpszResourceType", "dwDesiredAccess"]), offset=0),
    "PRESUTIL_GET_SZ_PROPERTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypeRef("CLUSPROP_SZ", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppszOutValue", "pValueStruct", "pszOldValue", "ppPropertyList", "pcbPropertyListSize"]), offset=0),
    "PRESUTIL_GET_SZ_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeChar(label="Char"), offset=0), arg_names=["hkeyClusterKey", "pszValueName"]), offset=0),
    "PRESUTIL_IS_PATH_VALID": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszPath"]), offset=0),
    "PRESUTIL_IS_RESOURCE_CLASS_EQUAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUS_RESOURCE_CLASS_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["prci", "hResource"]), offset=0),
    "PRESUTIL_PROPERTY_LIST_FROM_PARAMETER_BLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyTable", "pOutPropertyList", "pcbOutPropertyListSize", "pInParams", "pcbBytesReturned", "pcbRequired"]), offset=0),
    "PRESUTIL_REMOVE_RESOURCE_SERVICE_ENVIRONMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName", "pfnLogEvent", "hResourceHandle"]), offset=0),
    "PRESUTIL_RESOURCES_EQUAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hSelf", "hResource"]), offset=0),
    "PRESUTIL_RESOURCE_TYPES_EQUAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszResourceTypeName", "hResource"]), offset=0),
    "PRESUTIL_SET_BINARY_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "pbNewValue", "cbNewValueSize", "ppbOutValue", "pcbOutValueSize"]), offset=0),
    "PRESUTIL_SET_DWORD_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "dwNewValue", "pdwOutValue"]), offset=0),
    "PRESUTIL_SET_EXPAND_SZ_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "pszNewValue", "ppszOutString"]), offset=0),
    "PRESUTIL_SET_MULTI_SZ_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "pszNewValue", "cbNewValueSize", "ppszOutValue", "pcbOutValueSize"]), offset=0),
    "PRESUTIL_SET_PRIVATE_PROPERTY_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pInPropertyList", "cbInPropertyListSize"]), offset=0),
    "PRESUTIL_SET_PROPERTY_PARAMETER_BLOCK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "Reserved", "pInParams", "pInPropertyList", "cbInPropertyListSize", "pOutParams"]), offset=0),
    "PRESUTIL_SET_PROPERTY_PARAMETER_BLOCK_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "Reserved", "pInParams", "pInPropertyList", "cbInPropertyListSize", "bForceWrite", "pOutParams"]), offset=0),
    "PRESUTIL_SET_PROPERTY_TABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "Reserved", "bAllowUnknownProperties", "pInPropertyList", "cbInPropertyListSize", "pOutParams"]), offset=0),
    "PRESUTIL_SET_PROPERTY_TABLE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "Reserved", "bAllowUnknownProperties", "pInPropertyList", "cbInPropertyListSize", "bForceWrite", "pOutParams"]), offset=0),
    "PRESUTIL_SET_QWORD_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "qwNewValue", "pqwOutValue"]), offset=0),
    "PRESUTIL_SET_RESOURCE_SERVICE_ENVIRONMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName", "hResource", "pfnLogEvent", "hResourceHandle"]), offset=0),
    "PRESUTIL_SET_RESOURCE_SERVICE_START_PARAMETERS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName", "schSCMHandle", "phService", "pfnLogEvent", "hResourceHandle"]), offset=0),
    "PRESUTIL_SET_RESOURCE_SERVICE_START_PARAMETERS_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName", "schSCMHandle", "phService", "dwDesiredAccess", "pfnLogEvent", "hResourceHandle"]), offset=0),
    "PRESUTIL_SET_SZ_VALUE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pszValueName", "pszNewValue", "ppszOutString"]), offset=0),
    "PRESUTIL_SET_UNKNOWN_PROPERTIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hkeyClusterKey", "pPropertyTable", "pInPropertyList", "cbInPropertyListSize"]), offset=0),
    "PRESUTIL_START_RESOURCE_SERVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName", "phServiceHandle"]), offset=0),
    "PRESUTIL_STOP_RESOURCE_SERVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName"]), offset=0),
    "PRESUTIL_STOP_SERVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hServiceHandle"]), offset=0),
    "PRESUTIL_TERMINATE_SERVICE_PROCESS_FROM_RES_DLL": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwServicePid", "bOffline", "pdwResourceState", "pfnLogEvent", "hResourceHandle"]), offset=0),
    "PRESUTIL_VERIFY_PRIVATE_PROPERTY_LIST": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pInPropertyList", "cbInPropertyListSize"]), offset=0),
    "PRESUTIL_VERIFY_PROPERTY_TABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RESUTIL_PROPERTY_ITEM", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pPropertyTable", "Reserved", "bAllowUnknownProperties", "pInPropertyList", "cbInPropertyListSize", "pOutParams"]), offset=0),
    "PRESUTIL_VERIFY_RESOURCE_SERVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pszServiceName"]), offset=0),
    "PRESUTIL_VERIFY_SERVICE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hServiceHandle"]), offset=0),
    "PRES_UTIL_VERIFY_SHUTDOWN_SAFE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["flags", "reason", "pResult"]), offset=0),
    "PREVENT_MEDIA_REMOVAL": SimStruct(OrderedDict((("PreventMediaRemoval", SimTypeChar(label="Byte")),)), name="PREVENT_MEDIA_REMOVAL", pack=False, align=None),
    "PREVIEWHANDLERFRAMEINFO": SimStruct(OrderedDict((("haccel", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cAccelEntries", SimTypeInt(signed=False, label="UInt32")),)), name="PREVIEWHANDLERFRAMEINFO", pack=False, align=None),
    "PRE_PROCESS_PARAMETERS": SimStruct(OrderedDict((("pszSessionId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSiteName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszRemoteIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRemoteIpPort", SimTypeInt(signed=False, label="UInt32")), ("pszLocalIpAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLocalIpPort", SimTypeInt(signed=False, label="UInt32")), ("pszCommand", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCommandParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SessionStartTime", SimTypeRef("FILETIME", SimStruct)), ("BytesSentPerSession", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReceivedPerSession", SimTypeLongLong(signed=False, label="UInt64")),)), name="PRE_PROCESS_PARAMETERS", pack=False, align=None),
    "PRF_FLAGS": SimTypeInt(signed=False, label="PRF_FLAGS"),
    "PRINTDLGA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDevNames", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="PRINTDLGEX_FLAGS")), ("nFromPage", SimTypeShort(signed=False, label="UInt16")), ("nToPage", SimTypeShort(signed=False, label="UInt16")), ("nMinPage", SimTypeShort(signed=False, label="UInt16")), ("nMaxPage", SimTypeShort(signed=False, label="UInt16")), ("nCopies", SimTypeShort(signed=False, label="UInt16")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnPrintHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnSetupHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpPrintTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpSetupTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hPrintTemplate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hSetupTemplate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PRINTDLGA", pack=False, align=None),
    "PRINTDLGEXA": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDevNames", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="PRINTDLGEX_FLAGS")), ("Flags2", SimTypeInt(signed=False, label="UInt32")), ("ExclusionFlags", SimTypeInt(signed=False, label="UInt32")), ("nPageRanges", SimTypeInt(signed=False, label="UInt32")), ("nMaxPageRanges", SimTypeInt(signed=False, label="UInt32")), ("lpPageRanges", SimTypePointer(SimTypeRef("PRINTPAGERANGE", SimStruct), offset=0)), ("nMinPage", SimTypeInt(signed=False, label="UInt32")), ("nMaxPage", SimTypeInt(signed=False, label="UInt32")), ("nCopies", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpPrintTemplateName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpCallback", SimTypeBottom(label="IUnknown")), ("nPropertyPages", SimTypeInt(signed=False, label="UInt32")), ("lphPropertyPages", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("nStartPage", SimTypeInt(signed=False, label="UInt32")), ("dwResultAction", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTDLGEXA", pack=False, align=None),
    "PRINTDLGEXW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDevNames", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="PRINTDLGEX_FLAGS")), ("Flags2", SimTypeInt(signed=False, label="UInt32")), ("ExclusionFlags", SimTypeInt(signed=False, label="UInt32")), ("nPageRanges", SimTypeInt(signed=False, label="UInt32")), ("nMaxPageRanges", SimTypeInt(signed=False, label="UInt32")), ("lpPageRanges", SimTypePointer(SimTypeRef("PRINTPAGERANGE", SimStruct), offset=0)), ("nMinPage", SimTypeInt(signed=False, label="UInt32")), ("nMaxPage", SimTypeInt(signed=False, label="UInt32")), ("nCopies", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpPrintTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpCallback", SimTypeBottom(label="IUnknown")), ("nPropertyPages", SimTypeInt(signed=False, label="UInt32")), ("lphPropertyPages", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("nStartPage", SimTypeInt(signed=False, label="UInt32")), ("dwResultAction", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTDLGEXW", pack=False, align=None),
    "PRINTDLGEX_FLAGS": SimTypeInt(signed=False, label="PRINTDLGEX_FLAGS"),
    "PRINTDLGW": SimStruct(OrderedDict((("lStructSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDevMode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDevNames", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hDC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="PRINTDLGEX_FLAGS")), ("nFromPage", SimTypeShort(signed=False, label="UInt16")), ("nToPage", SimTypeShort(signed=False, label="UInt16")), ("nMinPage", SimTypeShort(signed=False, label="UInt16")), ("nMaxPage", SimTypeShort(signed=False, label="UInt16")), ("nCopies", SimTypeShort(signed=False, label="UInt16")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lCustData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnPrintHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpfnSetupHook", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lpPrintTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpSetupTemplateName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hPrintTemplate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hSetupTemplate", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PRINTDLGW", pack=False, align=None),
    "PRINTER_ACCESS_RIGHTS": SimTypeInt(signed=False, label="PRINTER_ACCESS_RIGHTS"),
    "PRINTER_CONNECTION_INFO_1A": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszDriverName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PRINTER_CONNECTION_INFO_1A", pack=False, align=None),
    "PRINTER_CONNECTION_INFO_1W": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszDriverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PRINTER_CONNECTION_INFO_1W", pack=False, align=None),
    "PRINTER_DEFAULTSA": SimStruct(OrderedDict((("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="PRINTER_ACCESS_RIGHTS")),)), name="PRINTER_DEFAULTSA", pack=False, align=None),
    "PRINTER_DEFAULTSW": SimStruct(OrderedDict((("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="PRINTER_ACCESS_RIGHTS")),)), name="PRINTER_DEFAULTSW", pack=False, align=None),
    "PRINTER_DEVICE_CAPABILITIES": SimTypeInt(signed=False, label="PRINTER_DEVICE_CAPABILITIES"),
    "PRINTER_ENUM_VALUESA": SimStruct(OrderedDict((("pValueName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbValueName", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_ENUM_VALUESA", pack=False, align=None),
    "PRINTER_ENUM_VALUESW": SimStruct(OrderedDict((("pValueName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbValueName", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_ENUM_VALUESW", pack=False, align=None),
    "PRINTER_EVENT_ATTRIBUTES_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwOldAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwNewAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_EVENT_ATTRIBUTES_INFO", pack=False, align=None),
    "PRINTER_INFO_1A": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PRINTER_INFO_1A", pack=False, align=None),
    "PRINTER_INFO_1W": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PRINTER_INFO_1W", pack=False, align=None),
    "PRINTER_INFO_2A": SimStruct(OrderedDict((("pServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pShareName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPortName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDriverName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pLocation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pSepFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPrintProcessor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("DefaultPriority", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("UntilTime", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("cJobs", SimTypeInt(signed=False, label="UInt32")), ("AveragePPM", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_2A", pack=False, align=None),
    "PRINTER_INFO_2W": SimStruct(OrderedDict((("pServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pShareName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPortName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDriverName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDevMode", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)), ("pSepFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrintProcessor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("DefaultPriority", SimTypeInt(signed=False, label="UInt32")), ("StartTime", SimTypeInt(signed=False, label="UInt32")), ("UntilTime", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("cJobs", SimTypeInt(signed=False, label="UInt32")), ("AveragePPM", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_2W", pack=False, align=None),
    "PRINTER_INFO_3": SimStruct(OrderedDict((("pSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PRINTER_INFO_3", pack=False, align=None),
    "PRINTER_INFO_4A": SimStruct(OrderedDict((("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_4A", pack=False, align=None),
    "PRINTER_INFO_4W": SimStruct(OrderedDict((("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_4W", pack=False, align=None),
    "PRINTER_INFO_5A": SimStruct(OrderedDict((("pPrinterName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPortName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("DeviceNotSelectedTimeout", SimTypeInt(signed=False, label="UInt32")), ("TransmissionRetryTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_5A", pack=False, align=None),
    "PRINTER_INFO_5W": SimStruct(OrderedDict((("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPortName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("DeviceNotSelectedTimeout", SimTypeInt(signed=False, label="UInt32")), ("TransmissionRetryTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_5W", pack=False, align=None),
    "PRINTER_INFO_6": SimStruct(OrderedDict((("dwStatus", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_6", pack=False, align=None),
    "PRINTER_INFO_7A": SimStruct(OrderedDict((("pszObjectGUID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwAction", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_7A", pack=False, align=None),
    "PRINTER_INFO_7W": SimStruct(OrderedDict((("pszObjectGUID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAction", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_INFO_7W", pack=False, align=None),
    "PRINTER_INFO_8A": SimStruct(OrderedDict((("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)),)), name="PRINTER_INFO_8A", pack=False, align=None),
    "PRINTER_INFO_8W": SimStruct(OrderedDict((("pDevMode", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)),)), name="PRINTER_INFO_8W", pack=False, align=None),
    "PRINTER_INFO_9A": SimStruct(OrderedDict((("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)),)), name="PRINTER_INFO_9A", pack=False, align=None),
    "PRINTER_INFO_9W": SimStruct(OrderedDict((("pDevMode", SimTypePointer(SimTypeRef("DEVMODEW", SimStruct), offset=0)),)), name="PRINTER_INFO_9W", pack=False, align=None),
    "PRINTER_NOTIFY_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("aData", SimTypePointer(SimTypeRef("PRINTER_NOTIFY_INFO_DATA", SimStruct), offset=0)),)), name="PRINTER_NOTIFY_INFO", pack=False, align=None),
    "PRINTER_NOTIFY_INFO_DATA": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Field", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeInt(signed=False, label="UInt32")), ("NotifyData", SimUnion({"adwData": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2), "Data": SimTypeRef("_Data_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PRINTER_NOTIFY_INFO_DATA", pack=False, align=None),
    "PRINTER_NOTIFY_INIT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("PollTime", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_NOTIFY_INIT", pack=False, align=None),
    "PRINTER_NOTIFY_OPTIONS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("pTypes", SimTypePointer(SimTypeRef("PRINTER_NOTIFY_OPTIONS_TYPE", SimStruct), offset=0)),)), name="PRINTER_NOTIFY_OPTIONS", pack=False, align=None),
    "PRINTER_NOTIFY_OPTIONS_TYPE": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("pFields", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="PRINTER_NOTIFY_OPTIONS_TYPE", pack=False, align=None),
    "PRINTER_OPTIONSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_OPTIONSA", pack=False, align=None),
    "PRINTER_OPTIONSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTER_OPTIONSW", pack=False, align=None),
    "PRINTER_OPTION_FLAGS": SimTypeInt(signed=False, label="PRINTER_OPTION_FLAGS"),
    "PRINTFLAG": SimTypeInt(signed=False, label="PRINTFLAG"),
    "PRINTIFI32": SimStruct(OrderedDict((("cjThis", SimTypeInt(signed=False, label="UInt32")), ("cjIfiExtra", SimTypeInt(signed=False, label="UInt32")), ("dpwszFamilyName", SimTypeInt(signed=True, label="Int32")), ("dpwszStyleName", SimTypeInt(signed=True, label="Int32")), ("dpwszFaceName", SimTypeInt(signed=True, label="Int32")), ("dpwszUniqueName", SimTypeInt(signed=True, label="Int32")), ("dpFontSim", SimTypeInt(signed=True, label="Int32")), ("lEmbedId", SimTypeInt(signed=True, label="Int32")), ("lItalicAngle", SimTypeInt(signed=True, label="Int32")), ("lCharBias", SimTypeInt(signed=True, label="Int32")), ("dpCharSets", SimTypeInt(signed=True, label="Int32")), ("jWinCharSet", SimTypeChar(label="Byte")), ("jWinPitchAndFamily", SimTypeChar(label="Byte")), ("usWinWeight", SimTypeShort(signed=False, label="UInt16")), ("flInfo", SimTypeInt(signed=False, label="UInt32")), ("fsSelection", SimTypeShort(signed=False, label="UInt16")), ("fsType", SimTypeShort(signed=False, label="UInt16")), ("fwdUnitsPerEm", SimTypeShort(signed=True, label="Int16")), ("fwdLowestPPEm", SimTypeShort(signed=True, label="Int16")), ("fwdWinAscender", SimTypeShort(signed=True, label="Int16")), ("fwdWinDescender", SimTypeShort(signed=True, label="Int16")), ("fwdMacAscender", SimTypeShort(signed=True, label="Int16")), ("fwdMacDescender", SimTypeShort(signed=True, label="Int16")), ("fwdMacLineGap", SimTypeShort(signed=True, label="Int16")), ("fwdTypoAscender", SimTypeShort(signed=True, label="Int16")), ("fwdTypoDescender", SimTypeShort(signed=True, label="Int16")), ("fwdTypoLineGap", SimTypeShort(signed=True, label="Int16")), ("fwdAveCharWidth", SimTypeShort(signed=True, label="Int16")), ("fwdMaxCharInc", SimTypeShort(signed=True, label="Int16")), ("fwdCapHeight", SimTypeShort(signed=True, label="Int16")), ("fwdXHeight", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptXSize", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptYSize", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptXOffset", SimTypeShort(signed=True, label="Int16")), ("fwdSubscriptYOffset", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptXSize", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptYSize", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptXOffset", SimTypeShort(signed=True, label="Int16")), ("fwdSuperscriptYOffset", SimTypeShort(signed=True, label="Int16")), ("fwdUnderscoreSize", SimTypeShort(signed=True, label="Int16")), ("fwdUnderscorePosition", SimTypeShort(signed=True, label="Int16")), ("fwdStrikeoutSize", SimTypeShort(signed=True, label="Int16")), ("fwdStrikeoutPosition", SimTypeShort(signed=True, label="Int16")), ("chFirstChar", SimTypeChar(label="Byte")), ("chLastChar", SimTypeChar(label="Byte")), ("chDefaultChar", SimTypeChar(label="Byte")), ("chBreakChar", SimTypeChar(label="Byte")), ("wcFirstChar", SimTypeChar(label="Char")), ("wcLastChar", SimTypeChar(label="Char")), ("wcDefaultChar", SimTypeChar(label="Char")), ("wcBreakChar", SimTypeChar(label="Char")), ("ptlBaseline", SimTypeRef("POINTL", SimStruct)), ("ptlAspect", SimTypeRef("POINTL", SimStruct)), ("ptlCaret", SimTypeRef("POINTL", SimStruct)), ("rclFontBox", SimTypeRef("RECTL", SimStruct)), ("achVendId", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("cKerningPairs", SimTypeInt(signed=False, label="UInt32")), ("ulPanoseCulture", SimTypeInt(signed=False, label="UInt32")), ("panose", SimTypeRef("PANOSE", SimStruct)),)), name="PRINTIFI32", pack=False, align=None),
    "PRINTPAGERANGE": SimStruct(OrderedDict((("nFromPage", SimTypeInt(signed=False, label="UInt32")), ("nToPage", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTPAGERANGE", pack=False, align=None),
    "PRINTPROCESSOROPENDATA": SimStruct(OrderedDict((("pDevMode", SimTypePointer(SimTypeRef("DEVMODEA", SimStruct), offset=0)), ("pDatatype", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDocumentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("JobId", SimTypeInt(signed=False, label="UInt32")), ("pOutputFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pPrinterName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PRINTPROCESSOROPENDATA", pack=False, align=None),
    "PRINTPROCESSOR_CAPS_1": SimStruct(OrderedDict((("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwNupOptions", SimTypeInt(signed=False, label="UInt32")), ("dwPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfCopies", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTPROCESSOR_CAPS_1", pack=False, align=None),
    "PRINTPROCESSOR_CAPS_2": SimStruct(OrderedDict((("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwNupOptions", SimTypeInt(signed=False, label="UInt32")), ("dwPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwDuplexHandlingCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNupDirectionCaps", SimTypeInt(signed=False, label="UInt32")), ("dwNupBorderCaps", SimTypeInt(signed=False, label="UInt32")), ("dwBookletHandlingCaps", SimTypeInt(signed=False, label="UInt32")), ("dwScalingCaps", SimTypeInt(signed=False, label="UInt32")),)), name="PRINTPROCESSOR_CAPS_2", pack=False, align=None),
    "PRINTPROCESSOR_INFO_1A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PRINTPROCESSOR_INFO_1A", pack=False, align=None),
    "PRINTPROCESSOR_INFO_1W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PRINTPROCESSOR_INFO_1W", pack=False, align=None),
    "PRINTPROVIDOR": SimStruct(OrderedDict((("fpOpenPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetJob", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetJob", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumJobs", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrinters", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrinterDriver", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrinterDrivers", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrinterDriver", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrinterDriverDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterDriver", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrintProcessor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrintProcessors", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrintProcessorDirectory", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrintProcessor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrintProcessorDatatypes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpStartDocPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpStartPagePrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpWritePrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEndPagePrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAbortPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpReadPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEndDocPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddJob", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpScheduleJob", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrinterData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetPrinterData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpWaitForPrinterChange", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpClosePrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddForm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeleteForm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetForm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetForm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumForms", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumMonitors", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPorts", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpConfigurePort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpCreatePrinterIC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpPlayGdiScriptOnPrinterIC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterIC", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrinterConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpPrinterMessageBox", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddMonitor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeleteMonitor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpResetPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrinterDriverEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpFindFirstPrinterChangeNotification", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpFindClosePrinterChangeNotification", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPortEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpShutDown", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpRefreshPrinterChangeNotification", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpOpenPrinterEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrinterEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrinterData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpClusterSplOpen", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpClusterSplClose", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpClusterSplIsAlive", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetPrinterDataEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrinterDataEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrinterDataEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPrinterKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterDataEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSeekPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePrinterDriverEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPerMachineConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeletePerMachineConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumPerMachineConnections", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpXcvData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrinterDriverEx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSplReadPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDriverUnloadComplete", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetSpoolFileInfo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpCommitSpoolData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpCloseSpoolFileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpFlushPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSendRecvBidiData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpAddPrinterConnection2", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetPrintClassObject", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpReportJobProcessingProgress", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumAndLogProvidorObjects", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpInternalGetPrinterDriver", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpFindCompatibleDriver", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpInstallPrinterDriverPackageFromConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetJobNamedPropertyValue", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpSetJobNamedProperty", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpDeleteJobNamedProperty", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpEnumJobNamedProperties", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpPowerEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpGetUserPropertyBag", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpCanShutdown", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpLogJobInfoForBranchOffice", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpRegeneratePrintDeviceCapabilities", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpPrintSupportOperation", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpIppCreateJobOnPrinter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpIppGetJobAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpIppSetJobAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpIppGetPrinterAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpIppSetPrinterAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("fpIppCreateJobOnPrinterWithAttributes", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PRINTPROVIDOR", pack=False, align=None),
    "PRINT_EXECUTION_CONTEXT": SimTypeInt(signed=False, label="PRINT_EXECUTION_CONTEXT"),
    "PRINT_EXECUTION_DATA": SimStruct(OrderedDict((("context", SimTypeInt(signed=False, label="PRINT_EXECUTION_CONTEXT")), ("clientAppPID", SimTypeInt(signed=False, label="UInt32")),)), name="PRINT_EXECUTION_DATA", pack=False, align=None),
    "PRINT_FEATURE_OPTION": SimStruct(OrderedDict((("pszFeature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszOption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PRINT_FEATURE_OPTION", pack=False, align=None),
    "PRINT_OTHER_INFO": SimStruct(OrderedDict((("alrtpr_jobid", SimTypeInt(signed=False, label="UInt32")), ("alrtpr_status", SimTypeInt(signed=False, label="UInt32")), ("alrtpr_submitted", SimTypeInt(signed=False, label="UInt32")), ("alrtpr_size", SimTypeInt(signed=False, label="UInt32")),)), name="PRINT_OTHER_INFO", pack=False, align=None),
    "PRINT_WINDOW_FLAGS": SimTypeInt(signed=False, label="PRINT_WINDOW_FLAGS"),
    "PRIORITIZE_FLAGS": SimTypeInt(signed=False, label="PRIORITIZE_FLAGS"),
    "PRIORITY": SimTypeInt(signed=False, label="PRIORITY"),
    "PRIORITY_HINT": SimTypeInt(signed=False, label="PRIORITY_HINT"),
    "PRIORITY_LEVEL": SimTypeInt(signed=False, label="PRIORITY_LEVEL"),
    "PRIORITY_STATUS": SimStruct(OrderedDict((("Sender", SimTypeInt(signed=False, label="SOCKET_PRIORITY_HINT")), ("Receiver", SimTypeInt(signed=False, label="SOCKET_PRIORITY_HINT")),)), name="PRIORITY_STATUS", pack=False, align=None),
    "PRIVILEGE_SET": SimStruct(OrderedDict((("PrivilegeCount", SimTypeInt(signed=False, label="UInt32")), ("Control", SimTypeInt(signed=False, label="UInt32")), ("Privilege", SimTypePointer(SimTypeRef("LUID_AND_ATTRIBUTES", SimStruct), offset=0)),)), name="PRIVILEGE_SET", pack=False, align=None),
    "PRIVKEYVER3": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("bitlenP", SimTypeInt(signed=False, label="UInt32")), ("bitlenQ", SimTypeInt(signed=False, label="UInt32")), ("bitlenJ", SimTypeInt(signed=False, label="UInt32")), ("bitlenX", SimTypeInt(signed=False, label="UInt32")), ("DSSSeed", SimTypeRef("DSSSEED", SimStruct)),)), name="PRIVKEYVER3", pack=False, align=None),
    "PRJ_CALLBACKS": SimStruct(OrderedDict((("StartDirectoryEnumerationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId"]), offset=0)), ("EndDirectoryEnumerationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId"]), offset=0)), ("GetDirectoryEnumerationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId", "searchExpression", "dirEntryBufferHandle"]), offset=0)), ("GetPlaceholderInfoCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData"]), offset=0)), ("GetFileDataCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "byteOffset", "length"]), offset=0)), ("QueryFileNameCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData"]), offset=0)), ("NotificationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="PRJ_NOTIFICATION"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimUnion({"PostCreate": SimTypeRef("_PostCreate_e__Struct", SimStruct), "FileRenamed": SimTypeRef("_FileRenamed_e__Struct", SimStruct), "FileDeletedOnHandleClose": SimTypeRef("_FileDeletedOnHandleClose_e__Struct", SimStruct)}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "isDirectory", "notification", "destinationFileName", "operationParameters"]), offset=0)), ("CancelCommandCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["callbackData"]), offset=0)),)), name="PRJ_CALLBACKS", pack=False, align=None),
    "PRJ_CALLBACK_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS")), ("NamespaceVirtualizationContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CommandId", SimTypeInt(signed=True, label="Int32")), ("FileId", SimTypeBottom(label="Guid")), ("DataStreamId", SimTypeBottom(label="Guid")), ("FilePathName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("VersionInfo", SimTypePointer(SimTypeRef("PRJ_PLACEHOLDER_VERSION_INFO", SimStruct), offset=0)), ("TriggeringProcessId", SimTypeInt(signed=False, label="UInt32")), ("TriggeringProcessImageFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("InstanceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PRJ_CALLBACK_DATA", pack=False, align=None),
    "PRJ_CALLBACK_DATA_FLAGS": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"),
    "PRJ_CANCEL_COMMAND_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["callbackData"]), offset=0),
    "PRJ_COMPLETE_COMMAND_EXTENDED_PARAMETERS": SimStruct(OrderedDict((("CommandType", SimTypeInt(signed=False, label="PRJ_COMPLETE_COMMAND_TYPE")), ("Anonymous", SimUnion({"Notification": SimTypeRef("_Notification_e__Struct", SimStruct), "Enumeration": SimTypeRef("_Enumeration_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PRJ_COMPLETE_COMMAND_EXTENDED_PARAMETERS", pack=False, align=None),
    "PRJ_COMPLETE_COMMAND_TYPE": SimTypeInt(signed=False, label="PRJ_COMPLETE_COMMAND_TYPE"),
    "PRJ_DIR_ENTRY_BUFFER_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PRJ_END_DIRECTORY_ENUMERATION_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId"]), offset=0),
    "PRJ_EXTENDED_INFO": SimStruct(OrderedDict((("InfoType", SimTypeInt(signed=False, label="PRJ_EXT_INFO_TYPE")), ("NextInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Symlink": SimTypeRef("_Symlink_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PRJ_EXTENDED_INFO", pack=False, align=None),
    "PRJ_EXT_INFO_TYPE": SimTypeInt(signed=False, label="PRJ_EXT_INFO_TYPE"),
    "PRJ_FILE_BASIC_INFO": SimStruct(OrderedDict((("IsDirectory", SimTypeChar(label="Byte")), ("FileSize", SimTypeLongLong(signed=True, label="Int64")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="PRJ_FILE_BASIC_INFO", pack=False, align=None),
    "PRJ_FILE_STATE": SimTypeInt(signed=False, label="PRJ_FILE_STATE"),
    "PRJ_GET_DIRECTORY_ENUMERATION_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId", "searchExpression", "dirEntryBufferHandle"]), offset=0),
    "PRJ_GET_FILE_DATA_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "byteOffset", "length"]), offset=0),
    "PRJ_GET_PLACEHOLDER_INFO_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData"]), offset=0),
    "PRJ_NAMESPACE_VIRTUALIZATION_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PRJ_NOTIFICATION": SimTypeInt(signed=False, label="PRJ_NOTIFICATION"),
    "PRJ_NOTIFICATION_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="PRJ_NOTIFICATION"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimUnion({"PostCreate": SimTypeRef("_PostCreate_e__Struct", SimStruct), "FileRenamed": SimTypeRef("_FileRenamed_e__Struct", SimStruct), "FileDeletedOnHandleClose": SimTypeRef("_FileDeletedOnHandleClose_e__Struct", SimStruct)}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "isDirectory", "notification", "destinationFileName", "operationParameters"]), offset=0),
    "PRJ_NOTIFICATION_MAPPING": SimStruct(OrderedDict((("NotificationBitMask", SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")), ("NotificationRoot", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PRJ_NOTIFICATION_MAPPING", pack=False, align=None),
    "PRJ_NOTIFICATION_PARAMETERS": SimUnion({"PostCreate": SimTypeRef("_PostCreate_e__Struct", SimStruct), "FileRenamed": SimTypeRef("_FileRenamed_e__Struct", SimStruct), "FileDeletedOnHandleClose": SimTypeRef("_FileDeletedOnHandleClose_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "PRJ_NOTIFY_TYPES": SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES"),
    "PRJ_PLACEHOLDER_ID": SimTypeInt(signed=False, label="PRJ_PLACEHOLDER_ID"),
    "PRJ_PLACEHOLDER_INFO": SimStruct(OrderedDict((("FileBasicInfo", SimTypeRef("PRJ_FILE_BASIC_INFO", SimStruct)), ("EaInformation", SimTypeRef("_EaInformation_e__Struct", SimStruct)), ("SecurityInformation", SimTypeRef("_SecurityInformation_e__Struct", SimStruct)), ("StreamsInformation", SimTypeRef("_StreamsInformation_e__Struct", SimStruct)), ("VersionInfo", SimTypeRef("PRJ_PLACEHOLDER_VERSION_INFO", SimStruct)), ("VariableData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PRJ_PLACEHOLDER_INFO", pack=False, align=None),
    "PRJ_PLACEHOLDER_VERSION_INFO": SimStruct(OrderedDict((("ProviderID", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("ContentID", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None),
    "PRJ_QUERY_FILE_NAME_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData"]), offset=0),
    "PRJ_STARTVIRTUALIZING_FLAGS": SimTypeInt(signed=False, label="PRJ_STARTVIRTUALIZING_FLAGS"),
    "PRJ_STARTVIRTUALIZING_OPTIONS": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="PRJ_STARTVIRTUALIZING_FLAGS")), ("PoolThreadCount", SimTypeInt(signed=False, label="UInt32")), ("ConcurrentThreadCount", SimTypeInt(signed=False, label="UInt32")), ("NotificationMappings", SimTypePointer(SimTypeRef("PRJ_NOTIFICATION_MAPPING", SimStruct), offset=0)), ("NotificationMappingsCount", SimTypeInt(signed=False, label="UInt32")),)), name="PRJ_STARTVIRTUALIZING_OPTIONS", pack=False, align=None),
    "PRJ_START_DIRECTORY_ENUMERATION_CB": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PRJ_CALLBACK_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId"]), offset=0),
    "PRJ_UPDATE_FAILURE_CAUSES": SimTypeInt(signed=False, label="PRJ_UPDATE_FAILURE_CAUSES"),
    "PRJ_UPDATE_TYPES": SimTypeInt(signed=False, label="PRJ_UPDATE_TYPES"),
    "PRJ_VIRTUALIZATION_INSTANCE_INFO": SimStruct(OrderedDict((("InstanceID", SimTypeBottom(label="Guid")), ("WriteAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="PRJ_VIRTUALIZATION_INSTANCE_INFO", pack=False, align=None),
    "PRKPROCESS": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PRKTHREAD": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PROBLEM_TYPE": SimTypeInt(signed=False, label="PROBLEM_TYPE"),
    "PROC": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0),
    "PROCESSENTRY32": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cntUsage", SimTypeInt(signed=False, label="UInt32")), ("th32ProcessID", SimTypeInt(signed=False, label="UInt32")), ("th32DefaultHeapID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("th32ModuleID", SimTypeInt(signed=False, label="UInt32")), ("cntThreads", SimTypeInt(signed=False, label="UInt32")), ("th32ParentProcessID", SimTypeInt(signed=False, label="UInt32")), ("pcPriClassBase", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szExeFile", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="PROCESSENTRY32", pack=False, align=None),
    "PROCESSENTRY32W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cntUsage", SimTypeInt(signed=False, label="UInt32")), ("th32ProcessID", SimTypeInt(signed=False, label="UInt32")), ("th32DefaultHeapID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("th32ModuleID", SimTypeInt(signed=False, label="UInt32")), ("cntThreads", SimTypeInt(signed=False, label="UInt32")), ("th32ParentProcessID", SimTypeInt(signed=False, label="UInt32")), ("pcPriClassBase", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szExeFile", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="PROCESSENTRY32W", pack=False, align=None),
    "PROCESSENUMPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwProcessId", "dwAttributes", "lpUserDefined"]), offset=0),
    "PROCESSINFOCLASS": SimTypeInt(signed=False, label="PROCESSINFOCLASS"),
    "PROCESSORINFO": SimStruct(OrderedDict((("Processor", SimTypeShort(signed=False, label="UInt16")), ("NumberProcessors", SimTypeShort(signed=False, label="UInt16")),)), name="PROCESSORINFO", pack=False, align=None),
    "PROCESSOR_ARCHITECTURE": SimTypeInt(signed=False, label="PROCESSOR_ARCHITECTURE"),
    "PROCESSOR_CACHE_TYPE": SimTypeInt(signed=False, label="PROCESSOR_CACHE_TYPE"),
    "PROCESSOR_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("KE_PROCESSOR_CHANGE_NOTIFY_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackContext", "ChangeContext", "OperationStatus"]), offset=0),
    "PROCESSOR_FEATURE_ID": SimTypeInt(signed=False, label="PROCESSOR_FEATURE_ID"),
    "PROCESSOR_GROUP_INFO": SimStruct(OrderedDict((("MaximumProcessorCount", SimTypeChar(label="Byte")), ("ActiveProcessorCount", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 38)), ("ActiveProcessorMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROCESSOR_GROUP_INFO", pack=False, align=None),
    "PROCESSOR_HALT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PROCESSOR_IDLESTATE_INFO": SimStruct(OrderedDict((("TimeCheck", SimTypeInt(signed=False, label="UInt32")), ("DemotePercent", SimTypeChar(label="Byte")), ("PromotePercent", SimTypeChar(label="Byte")), ("Spare", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="PROCESSOR_IDLESTATE_INFO", pack=False, align=None),
    "PROCESSOR_IDLESTATE_POLICY": SimStruct(OrderedDict((("Revision", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"AsWORD": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("PolicyCount", SimTypeInt(signed=False, label="UInt32")), ("Policy", SimTypeArray(SimTypeRef("PROCESSOR_IDLESTATE_INFO", SimStruct), 3)),)), name="PROCESSOR_IDLESTATE_POLICY", pack=False, align=None),
    "PROCESSOR_NUMBER": SimStruct(OrderedDict((("Group", SimTypeShort(signed=False, label="UInt16")), ("Number", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="PROCESSOR_NUMBER", pack=False, align=None),
    "PROCESSOR_OBJECT_INFO": SimStruct(OrderedDict((("PhysicalID", SimTypeInt(signed=False, label="UInt32")), ("PBlkAddress", SimTypeInt(signed=False, label="UInt32")), ("PBlkLength", SimTypeChar(label="Byte")),)), name="PROCESSOR_OBJECT_INFO", pack=False, align=None),
    "PROCESSOR_OBJECT_INFO_EX": SimStruct(OrderedDict((("PhysicalID", SimTypeInt(signed=False, label="UInt32")), ("PBlkAddress", SimTypeInt(signed=False, label="UInt32")), ("PBlkLength", SimTypeChar(label="Byte")), ("InitialApicId", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESSOR_OBJECT_INFO_EX", pack=False, align=None),
    "PROCESSOR_PERFSTATE_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("MaxThrottle", SimTypeChar(label="Byte")), ("MinThrottle", SimTypeChar(label="Byte")), ("BusyAdjThreshold", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Spare": SimTypeChar(label="Byte"), "Flags": SimUnion({"AsBYTE": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None")), ("TimeCheck", SimTypeInt(signed=False, label="UInt32")), ("IncreaseTime", SimTypeInt(signed=False, label="UInt32")), ("DecreaseTime", SimTypeInt(signed=False, label="UInt32")), ("IncreasePercent", SimTypeInt(signed=False, label="UInt32")), ("DecreasePercent", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESSOR_PERFSTATE_POLICY", pack=False, align=None),
    "PROCESSOR_POWER_INFORMATION": SimStruct(OrderedDict((("Number", SimTypeInt(signed=False, label="UInt32")), ("MaxMhz", SimTypeInt(signed=False, label="UInt32")), ("CurrentMhz", SimTypeInt(signed=False, label="UInt32")), ("MhzLimit", SimTypeInt(signed=False, label="UInt32")), ("MaxIdleState", SimTypeInt(signed=False, label="UInt32")), ("CurrentIdleState", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESSOR_POWER_INFORMATION", pack=False, align=None),
    "PROCESSOR_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("DynamicThrottle", SimTypeChar(label="Byte")), ("Spare", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("PolicyCount", SimTypeInt(signed=False, label="UInt32")), ("Policy", SimTypeArray(SimTypeRef("PROCESSOR_POWER_POLICY_INFO", SimStruct), 3)),)), name="PROCESSOR_POWER_POLICY", pack=False, align=None),
    "PROCESSOR_POWER_POLICY_INFO": SimStruct(OrderedDict((("TimeCheck", SimTypeInt(signed=False, label="UInt32")), ("DemoteLimit", SimTypeInt(signed=False, label="UInt32")), ("PromoteLimit", SimTypeInt(signed=False, label="UInt32")), ("DemotePercent", SimTypeChar(label="Byte")), ("PromotePercent", SimTypeChar(label="Byte")), ("Spare", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESSOR_POWER_POLICY_INFO", pack=False, align=None),
    "PROCESSOR_RELATIONSHIP": SimStruct(OrderedDict((("Flags", SimTypeChar(label="Byte")), ("EfficiencyClass", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("GroupCount", SimTypeShort(signed=False, label="UInt16")), ("GroupMask", SimTypePointer(SimTypeRef("GROUP_AFFINITY", SimStruct), offset=0)),)), name="PROCESSOR_RELATIONSHIP", pack=False, align=None),
    "PROCESSTRACE_HANDLE": SimStruct(OrderedDict((("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="PROCESSTRACE_HANDLE", pack=False, align=None),
    "PROCESS_ACCESS_RIGHTS": SimTypeInt(signed=False, label="PROCESS_ACCESS_RIGHTS"),
    "PROCESS_ACCESS_TOKEN": SimStruct(OrderedDict((("Token", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Thread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PROCESS_ACCESS_TOKEN", pack=False, align=None),
    "PROCESS_AFFINITY_AUTO_UPDATE_FLAGS": SimTypeInt(signed=False, label="PROCESS_AFFINITY_AUTO_UPDATE_FLAGS"),
    "PROCESS_BASIC_INFORMATION": SimStruct(OrderedDict((("ExitStatus", SimTypeInt(signed=True, label="Int32")), ("PebBaseAddress", SimTypePointer(SimTypeRef("PEB", SimStruct), offset=0)), ("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("UniqueProcessId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InheritedFromUniqueProcessId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROCESS_BASIC_INFORMATION", pack=False, align=None),
    "PROCESS_COMMIT_USAGE": SimStruct(OrderedDict((("ImageFileName", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ClientId", SimTypeLongLong(signed=False, label="UInt64")), ("ProcessAddress", SimTypeLongLong(signed=False, label="UInt64")), ("CommitCharge", SimTypeLongLong(signed=False, label="UInt64")), ("SharedCommitCharge", SimTypeLongLong(signed=False, label="UInt64")), ("ReleasedCommitDebt", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="PROCESS_COMMIT_USAGE", pack=False, align=None),
    "PROCESS_CREATION_FLAGS": SimTypeInt(signed=False, label="PROCESS_CREATION_FLAGS"),
    "PROCESS_DEP_FLAGS": SimTypeInt(signed=False, label="PROCESS_DEP_FLAGS"),
    "PROCESS_DEVICEMAP_INFORMATION": SimStruct(OrderedDict((("Anonymous", SimUnion({"Set": SimTypeRef("_Set_e__Struct", SimStruct), "Query": SimTypeRef("_Query_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PROCESS_DEVICEMAP_INFORMATION", pack=False, align=None),
    "PROCESS_DEVICEMAP_INFORMATION_EX": SimStruct(OrderedDict((("Anonymous", SimUnion({"Set": SimTypeRef("_Set_e__Struct", SimStruct), "Query": SimTypeRef("_Query_e__Struct", SimStruct)}, name="<anon>", label="None")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_DEVICEMAP_INFORMATION_EX", pack=False, align=None),
    "PROCESS_DPI_AWARENESS": SimTypeInt(signed=False, label="PROCESS_DPI_AWARENESS"),
    "PROCESS_DYNAMIC_EH_CONTINUATION_TARGET": SimStruct(OrderedDict((("TargetAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Flags", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROCESS_DYNAMIC_EH_CONTINUATION_TARGET", pack=False, align=None),
    "PROCESS_DYNAMIC_EH_CONTINUATION_TARGETS_INFORMATION": SimStruct(OrderedDict((("NumberOfTargets", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Targets", SimTypePointer(SimTypeRef("PROCESS_DYNAMIC_EH_CONTINUATION_TARGET", SimStruct), offset=0)),)), name="PROCESS_DYNAMIC_EH_CONTINUATION_TARGETS_INFORMATION", pack=False, align=None),
    "PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGE": SimStruct(OrderedDict((("BaseAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGE", pack=False, align=None),
    "PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGES_INFORMATION": SimStruct(OrderedDict((("NumberOfRanges", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Ranges", SimTypePointer(SimTypeRef("PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGE", SimStruct), offset=0)),)), name="PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGES_INFORMATION", pack=False, align=None),
    "PROCESS_EXCEPTION_PORT": SimStruct(OrderedDict((("ExceptionPortHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("StateFlags", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_EXCEPTION_PORT", pack=False, align=None),
    "PROCESS_EXTENDED_BASIC_INFORMATION": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BasicInfo", SimTypeRef("PROCESS_BASIC_INFORMATION", SimStruct)), ("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_EXTENDED_BASIC_INFORMATION", pack=False, align=None),
    "PROCESS_HANDLE_TRACING_ENABLE": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_HANDLE_TRACING_ENABLE", pack=False, align=None),
    "PROCESS_HANDLE_TRACING_ENABLE_EX": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("TotalSlots", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_HANDLE_TRACING_ENABLE_EX", pack=False, align=None),
    "PROCESS_HANDLE_TRACING_ENTRY": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ClientId", SimTypeRef("CLIENT_ID", SimStruct)), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Stacks", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 16)),)), name="PROCESS_HANDLE_TRACING_ENTRY", pack=False, align=None),
    "PROCESS_HANDLE_TRACING_QUERY": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("TotalTraces", SimTypeInt(signed=False, label="UInt32")), ("HandleTrace", SimTypePointer(SimTypeRef("PROCESS_HANDLE_TRACING_ENTRY", SimStruct), offset=0)),)), name="PROCESS_HANDLE_TRACING_QUERY", pack=False, align=None),
    "PROCESS_HEAP_ENTRY": SimStruct(OrderedDict((("lpData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("cbOverhead", SimTypeChar(label="Byte")), ("iRegionIndex", SimTypeChar(label="Byte")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"Block": SimTypeRef("_Block_e__Struct", SimStruct), "Region": SimTypeRef("_Region_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PROCESS_HEAP_ENTRY", pack=False, align=None),
    "PROCESS_INFORMATION": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwThreadId", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_INFORMATION", pack=False, align=None),
    "PROCESS_INFORMATION_CLASS": SimTypeInt(signed=False, label="PROCESS_INFORMATION_CLASS"),
    "PROCESS_KEEPALIVE_COUNT_INFORMATION": SimStruct(OrderedDict((("WakeCount", SimTypeInt(signed=False, label="UInt32")), ("NoWakeCount", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_KEEPALIVE_COUNT_INFORMATION", pack=False, align=None),
    "PROCESS_LEAP_SECOND_INFO": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_LEAP_SECOND_INFO", pack=False, align=None),
    "PROCESS_LIST": SimStruct(OrderedDict((("lpLink", SimTypePointer(SimTypeRef("PROCESS_LIST", SimStruct), offset=0)), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwRefCnt", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaDepth", SimTypeInt(signed=False, label="UInt32")), ("dwZDepth", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_LIST", pack=False, align=None),
    "PROCESS_LOOPBACK_MODE": SimTypeInt(signed=False, label="PROCESS_LOOPBACK_MODE"),
    "PROCESS_MACHINE_INFORMATION": SimStruct(OrderedDict((("ProcessMachine", SimTypeInt(signed=False, label="IMAGE_FILE_MACHINE")), ("Res0", SimTypeShort(signed=False, label="UInt16")), ("MachineAttributes", SimTypeInt(signed=False, label="MACHINE_ATTRIBUTES")),)), name="PROCESS_MACHINE_INFORMATION", pack=False, align=None),
    "PROCESS_MEMBERSHIP_INFORMATION": SimStruct(OrderedDict((("ServerSiloId", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_MEMBERSHIP_INFORMATION", pack=False, align=None),
    "PROCESS_MEMORY_COUNTERS": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROCESS_MEMORY_COUNTERS", pack=False, align=None),
    "PROCESS_MEMORY_COUNTERS_EX": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrivateUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROCESS_MEMORY_COUNTERS_EX", pack=False, align=None),
    "PROCESS_MEMORY_COUNTERS_EX2": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrivateUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrivateWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedCommitUsage", SimTypeLongLong(signed=False, label="UInt64")),)), name="PROCESS_MEMORY_COUNTERS_EX2", pack=False, align=None),
    "PROCESS_MEMORY_EXHAUSTION_INFO": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeInt(signed=False, label="PROCESS_MEMORY_EXHAUSTION_TYPE")), ("Value", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROCESS_MEMORY_EXHAUSTION_INFO", pack=False, align=None),
    "PROCESS_MEMORY_EXHAUSTION_TYPE": SimTypeInt(signed=False, label="PROCESS_MEMORY_EXHAUSTION_TYPE"),
    "PROCESS_MITIGATION_ACTIVATION_CONTEXT_TRUST_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_ACTIVATION_CONTEXT_TRUST_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_ASLR_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_ASLR_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_BINARY_SIGNATURE_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_BINARY_SIGNATURE_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_CHILD_PROCESS_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_CHILD_PROCESS_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_CONTROL_FLOW_GUARD_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_CONTROL_FLOW_GUARD_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_DEP_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Permanent", SimTypeChar(label="Byte")),)), name="PROCESS_MITIGATION_DEP_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_DYNAMIC_CODE_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_DYNAMIC_CODE_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_EXTENSION_POINT_DISABLE_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_EXTENSION_POINT_DISABLE_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_FONT_DISABLE_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_FONT_DISABLE_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_IMAGE_LOAD_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_IMAGE_LOAD_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_PAYLOAD_RESTRICTION_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_PAYLOAD_RESTRICTION_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_POLICY": SimTypeInt(signed=False, label="PROCESS_MITIGATION_POLICY"),
    "PROCESS_MITIGATION_REDIRECTION_TRUST_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_REDIRECTION_TRUST_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_SEHOP_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_SEHOP_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_SIDE_CHANNEL_ISOLATION_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_SIDE_CHANNEL_ISOLATION_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_STRICT_HANDLE_CHECK_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_STRICT_HANDLE_CHECK_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_SYSTEM_CALL_DISABLE_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_SYSTEM_CALL_DISABLE_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_SYSTEM_CALL_FILTER_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_SYSTEM_CALL_FILTER_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_USER_POINTER_AUTH_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_USER_POINTER_AUTH_POLICY", pack=False, align=None),
    "PROCESS_MITIGATION_USER_SHADOW_STACK_POLICY": SimStruct(OrderedDict((("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="PROCESS_MITIGATION_USER_SHADOW_STACK_POLICY", pack=False, align=None),
    "PROCESS_NAME_ENTRY": SimStruct(OrderedDict((("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("NameSize", SimTypeInt(signed=False, label="UInt32")), ("NextEntry", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_NAME_ENTRY", pack=False, align=None),
    "PROCESS_NAME_FORMAT": SimTypeInt(signed=False, label="PROCESS_NAME_FORMAT"),
    "PROCESS_POWER_THROTTLING_STATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ControlMask", SimTypeInt(signed=False, label="UInt32")), ("StateMask", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_POWER_THROTTLING_STATE", pack=False, align=None),
    "PROCESS_PROTECTION_LEVEL": SimTypeInt(signed=False, label="PROCESS_PROTECTION_LEVEL"),
    "PROCESS_PROTECTION_LEVEL_INFORMATION": SimStruct(OrderedDict((("ProtectionLevel", SimTypeInt(signed=False, label="PROCESS_PROTECTION_LEVEL")),)), name="PROCESS_PROTECTION_LEVEL_INFORMATION", pack=False, align=None),
    "PROCESS_REVOKE_FILE_HANDLES_INFORMATION": SimStruct(OrderedDict((("TargetDevicePath", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="PROCESS_REVOKE_FILE_HANDLES_INFORMATION", pack=False, align=None),
    "PROCESS_SESSION_INFORMATION": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")),)), name="PROCESS_SESSION_INFORMATION", pack=False, align=None),
    "PROCESS_SYSCALL_PROVIDER_INFORMATION": SimStruct(OrderedDict((("ProviderId", SimTypeBottom(label="Guid")), ("Level", SimTypeChar(label="Byte")),)), name="PROCESS_SYSCALL_PROVIDER_INFORMATION", pack=False, align=None),
    "PROCESS_WS_WATCH_INFORMATION": SimStruct(OrderedDict((("FaultingPc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FaultingVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PROCESS_WS_WATCH_INFORMATION", pack=False, align=None),
    "PROC_THREAD_ATTRIBUTE_NUM": SimTypeInt(signed=False, label="PROC_THREAD_ATTRIBUTE_NUM"),
    "PRODUCT_INFOA": SimStruct(OrderedDict((("CompanyName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("ProductID", SimTypeArray(SimTypeChar(label="SByte"), 4)),)), name="PRODUCT_INFOA", pack=False, align=None),
    "PRODUCT_INFOW": SimStruct(OrderedDict((("CompanyName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ProductID", SimTypeArray(SimTypeChar(label="Char"), 4)),)), name="PRODUCT_INFOW", pack=False, align=None),
    "PROFILE": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("pProfileData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="PROFILE", pack=False, align=None),
    "PROFILEHEADER": SimStruct(OrderedDict((("phSize", SimTypeInt(signed=False, label="UInt32")), ("phCMMType", SimTypeInt(signed=False, label="UInt32")), ("phVersion", SimTypeInt(signed=False, label="UInt32")), ("phClass", SimTypeInt(signed=False, label="UInt32")), ("phDataColorSpace", SimTypeInt(signed=False, label="UInt32")), ("phConnectionSpace", SimTypeInt(signed=False, label="UInt32")), ("phDateTime", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("phSignature", SimTypeInt(signed=False, label="UInt32")), ("phPlatform", SimTypeInt(signed=False, label="UInt32")), ("phProfileFlags", SimTypeInt(signed=False, label="UInt32")), ("phManufacturer", SimTypeInt(signed=False, label="UInt32")), ("phModel", SimTypeInt(signed=False, label="UInt32")), ("phAttributes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("phRenderingIntent", SimTypeInt(signed=False, label="UInt32")), ("phIlluminant", SimTypeRef("CIEXYZ", SimStruct)), ("phCreator", SimTypeInt(signed=False, label="UInt32")), ("phReserved", SimTypeArray(SimTypeChar(label="Byte"), 44)),)), name="PROFILEHEADER", pack=False, align=None),
    "PROFILEINFOA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpProfilePath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDefaultPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpPolicyPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hProfile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PROFILEINFOA", pack=False, align=None),
    "PROFILEINFOW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpProfilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDefaultPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpPolicyPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hProfile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PROFILEINFOW", pack=False, align=None),
    "PROFILEPROPERTIES": SimTypeInt(signed=False, label="PROFILEPROPERTIES"),
    "PROFILER_EVENT_MASK": SimTypeInt(signed=False, label="PROFILER_EVENT_MASK"),
    "PROFILER_HEAP_ENUM_FLAGS": SimTypeInt(signed=False, label="PROFILER_HEAP_ENUM_FLAGS"),
    "PROFILER_HEAP_OBJECT": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"objectId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "externalObjectAddress": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("typeNameId", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("unused", SimTypeShort(signed=False, label="UInt16")), ("optionalInfoCount", SimTypeShort(signed=False, label="UInt16")),)), name="PROFILER_HEAP_OBJECT", pack=False, align=None),
    "PROFILER_HEAP_OBJECT_FLAGS": SimTypeInt(signed=False, label="PROFILER_HEAP_OBJECT_FLAGS"),
    "PROFILER_HEAP_OBJECT_OPTIONAL_INFO": SimStruct(OrderedDict((("infoType", SimTypeInt(signed=False, label="PROFILER_HEAP_OBJECT_OPTIONAL_INFO_TYPE")), ("Anonymous", SimUnion({"prototype": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "functionName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "elementAttributesSize": SimTypeInt(signed=False, label="UInt32"), "elementTextChildrenSize": SimTypeInt(signed=False, label="UInt32"), "scopeList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_SCOPE_LIST", SimStruct), offset=0), "internalProperty": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP", SimStruct), offset=0), "namePropertyList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0), "indexPropertyList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0), "relationshipList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0), "eventList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0), "weakMapCollectionList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0), "mapCollectionList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0), "setCollectionList": SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="PROFILER_HEAP_OBJECT_OPTIONAL_INFO", pack=False, align=None),
    "PROFILER_HEAP_OBJECT_OPTIONAL_INFO_TYPE": SimTypeInt(signed=False, label="PROFILER_HEAP_OBJECT_OPTIONAL_INFO_TYPE"),
    "PROFILER_HEAP_OBJECT_RELATIONSHIP": SimStruct(OrderedDict((("relationshipId", SimTypeInt(signed=False, label="UInt32")), ("relationshipInfo", SimTypeInt(signed=False, label="PROFILER_RELATIONSHIP_INFO")), ("Anonymous", SimUnion({"numberValue": SimTypeFloat(size=64), "stringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "bstrValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "objectId": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "externalObjectAddress": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "subString": SimTypePointer(SimTypeRef("PROFILER_PROPERTY_TYPE_SUBSTRING_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="PROFILER_HEAP_OBJECT_RELATIONSHIP", pack=False, align=None),
    "PROFILER_HEAP_OBJECT_RELATIONSHIP_FLAGS": SimTypeInt(signed=False, label="PROFILER_HEAP_OBJECT_RELATIONSHIP_FLAGS"),
    "PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("elements", SimTypePointer(SimTypeRef("PROFILER_HEAP_OBJECT_RELATIONSHIP", SimStruct), offset=0)),)), name="PROFILER_HEAP_OBJECT_RELATIONSHIP_LIST", pack=False, align=None),
    "PROFILER_HEAP_OBJECT_SCOPE_LIST": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("scopes", SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)),)), name="PROFILER_HEAP_OBJECT_SCOPE_LIST", pack=False, align=None),
    "PROFILER_HEAP_SUMMARY": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="PROFILER_HEAP_SUMMARY_VERSION")), ("totalHeapSize", SimTypeInt(signed=False, label="UInt32")),)), name="PROFILER_HEAP_SUMMARY", pack=False, align=None),
    "PROFILER_HEAP_SUMMARY_VERSION": SimTypeInt(signed=False, label="PROFILER_HEAP_SUMMARY_VERSION"),
    "PROFILER_PROPERTY_TYPE_SUBSTRING_INFO": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROFILER_PROPERTY_TYPE_SUBSTRING_INFO", pack=False, align=None),
    "PROFILER_RELATIONSHIP_INFO": SimTypeInt(signed=False, label="PROFILER_RELATIONSHIP_INFO"),
    "PROFILER_SCRIPT_TYPE": SimTypeInt(signed=False, label="PROFILER_SCRIPT_TYPE"),
    "PROFILE_SOURCE_INFO": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("Source", SimTypeInt(signed=False, label="UInt32")), ("MinInterval", SimTypeInt(signed=False, label="UInt32")), ("MaxInterval", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROFILE_SOURCE_INFO", pack=False, align=None),
    "PROGRESSPARTS": SimTypeInt(signed=False, label="PROGRESSPARTS"),
    "PROGRESS_DIALOG_CHECKBOX_ID": SimTypeInt(signed=False, label="PROGRESS_DIALOG_CHECKBOX_ID"),
    "PROGRESS_DIALOG_IMAGE_TYPE": SimTypeInt(signed=False, label="PROGRESS_DIALOG_IMAGE_TYPE"),
    "PROG_INVOKE_SETTING": SimTypeInt(signed=False, label="PROG_INVOKE_SETTING"),
    "PROJECTION_INFO": SimStruct(OrderedDict((("projectionInfoType", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"PppProjectionInfo": SimTypeRef("PPP_PROJECTION_INFO", SimStruct), "Ikev2ProjectionInfo": SimTypeRef("IKEV2_PROJECTION_INFO", SimStruct)}, name="<anon>", label="None")),)), name="PROJECTION_INFO", pack=False, align=None),
    "PROJECTION_INFO2": SimStruct(OrderedDict((("projectionInfoType", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"PppProjectionInfo": SimTypeRef("PPP_PROJECTION_INFO2", SimStruct), "Ikev2ProjectionInfo": SimTypeRef("IKEV2_PROJECTION_INFO2", SimStruct)}, name="<anon>", label="None")),)), name="PROJECTION_INFO2", pack=False, align=None),
    "PROPBAG2": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("vt", SimTypeInt(signed=False, label="VARENUM")), ("cfType", SimTypeShort(signed=False, label="UInt16")), ("dwHint", SimTypeInt(signed=False, label="UInt32")), ("pstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("clsid", SimTypeBottom(label="Guid")),)), name="PROPBAG2", pack=False, align=None),
    "PROPBAG2_TYPE": SimTypeInt(signed=False, label="PROPBAG2_TYPE"),
    "PROPDESC_AGGREGATION_TYPE": SimTypeInt(signed=False, label="PROPDESC_AGGREGATION_TYPE"),
    "PROPDESC_COLUMNINDEX_TYPE": SimTypeInt(signed=False, label="PROPDESC_COLUMNINDEX_TYPE"),
    "PROPDESC_CONDITION_TYPE": SimTypeInt(signed=False, label="PROPDESC_CONDITION_TYPE"),
    "PROPDESC_DISPLAYTYPE": SimTypeInt(signed=False, label="PROPDESC_DISPLAYTYPE"),
    "PROPDESC_ENUMFILTER": SimTypeInt(signed=False, label="PROPDESC_ENUMFILTER"),
    "PROPDESC_FORMAT_FLAGS": SimTypeInt(signed=False, label="PROPDESC_FORMAT_FLAGS"),
    "PROPDESC_GROUPING_RANGE": SimTypeInt(signed=False, label="PROPDESC_GROUPING_RANGE"),
    "PROPDESC_RELATIVEDESCRIPTION_TYPE": SimTypeInt(signed=False, label="PROPDESC_RELATIVEDESCRIPTION_TYPE"),
    "PROPDESC_SEARCHINFO_FLAGS": SimTypeInt(signed=False, label="PROPDESC_SEARCHINFO_FLAGS"),
    "PROPDESC_SORTDESCRIPTION": SimTypeInt(signed=False, label="PROPDESC_SORTDESCRIPTION"),
    "PROPDESC_TYPE_FLAGS": SimTypeInt(signed=False, label="PROPDESC_TYPE_FLAGS"),
    "PROPDESC_VIEW_FLAGS": SimTypeInt(signed=False, label="PROPDESC_VIEW_FLAGS"),
    "PROPENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PROPENUMPROCEXA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PROPENUMPROCEXW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PROPENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PROPENUMTYPE": SimTypeInt(signed=False, label="PROPENUMTYPE"),
    "PROPERTYKEY": SimStruct(OrderedDict((("fmtid", SimTypeBottom(label="Guid")), ("pid", SimTypeInt(signed=False, label="UInt32")),)), name="PROPERTYKEY", pack=False, align=None),
    "PROPERTYORIGIN": SimTypeInt(signed=False, label="PROPERTYORIGIN"),
    "PROPERTYRESTRICTION": SimStruct(OrderedDict((("rel", SimTypeInt(signed=False, label="UInt32")), ("prop", SimTypeRef("FULLPROPSPEC", SimStruct)), ("prval", SimTypeRef("PROPVARIANT", SimStruct)),)), name="PROPERTYRESTRICTION", pack=False, align=None),
    "PROPERTYUI_FLAGS": SimTypeInt(signed=False, label="PROPERTYUI_FLAGS"),
    "PROPERTYUI_FORMAT_FLAGS": SimTypeInt(signed=False, label="PROPERTYUI_FORMAT_FLAGS"),
    "PROPERTYUI_NAME_FLAGS": SimTypeInt(signed=False, label="PROPERTYUI_NAME_FLAGS"),
    "PROPERTY_DATA_DESCRIPTOR": SimStruct(OrderedDict((("PropertyName", SimTypeLongLong(signed=False, label="UInt64")), ("ArrayIndex", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="PROPERTY_DATA_DESCRIPTOR", pack=False, align=None),
    "PROPERTY_FLAGS": SimTypeInt(signed=False, label="PROPERTY_FLAGS"),
    "PROPERTY_METRICS": SimStruct(OrderedDict((("nLogicalMin", SimTypeInt(signed=True, label="Int32")), ("nLogicalMax", SimTypeInt(signed=True, label="Int32")), ("Units", SimTypeInt(signed=False, label="PROPERTY_UNITS")), ("fResolution", SimTypeFloat(size=32)),)), name="PROPERTY_METRICS", pack=False, align=None),
    "PROPERTY_UNITS": SimTypeInt(signed=False, label="PROPERTY_UNITS"),
    "PROPPAGEINFO": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("size", SimTypeRef("SIZE", SimStruct)), ("pszDocString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHelpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHelpContext", SimTypeInt(signed=False, label="UInt32")),)), name="PROPPAGEINFO", pack=False, align=None),
    "PROPPAGESTATUS": SimTypeInt(signed=False, label="PROPPAGESTATUS"),
    "PROPPRG": SimStruct(OrderedDict((("flPrg", SimTypeShort(signed=False, label="UInt16")), ("flPrgInit", SimTypeShort(signed=False, label="UInt16")), ("achTitle", SimTypeArray(SimTypeChar(label="SByte"), 30)), ("achCmdLine", SimTypeArray(SimTypeChar(label="SByte"), 128)), ("achWorkDir", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("wHotKey", SimTypeShort(signed=False, label="UInt16")), ("achIconFile", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("wIconIndex", SimTypeShort(signed=False, label="UInt16")), ("dwEnhModeFlags", SimTypeInt(signed=False, label="UInt32")), ("dwRealModeFlags", SimTypeInt(signed=False, label="UInt32")), ("achOtherFile", SimTypeArray(SimTypeChar(label="SByte"), 80)), ("achPIFFile", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="PROPPRG", pack=False, align=None),
    "PROPSHEETHEADERA_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("pszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nPages", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"nStartPage": SimTypeInt(signed=False, label="UInt32"), "pStartPage": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"ppsp": SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0), "phpage": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)),)), name="PROPSHEETHEADERA_V1", pack=False, align=None),
    "PROPSHEETHEADERA_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("pszCaption", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nPages", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"nStartPage": SimTypeInt(signed=False, label="UInt32"), "pStartPage": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"ppsp": SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0), "phpage": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("Anonymous4", SimUnion({"hbmWatermark": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmWatermark": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("hplWatermark", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous5", SimUnion({"hbmHeader": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmHeader": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="PROPSHEETHEADERA_V2", pack=False, align=None),
    "PROPSHEETHEADERW_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nPages", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"nStartPage": SimTypeInt(signed=False, label="UInt32"), "pStartPage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"ppsp": SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0), "phpage": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)),)), name="PROPSHEETHEADERW_V1", pack=False, align=None),
    "PROPSHEETHEADERW_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszCaption", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nPages", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"nStartPage": SimTypeInt(signed=False, label="UInt32"), "pStartPage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"ppsp": SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0), "phpage": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("Anonymous4", SimUnion({"hbmWatermark": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmWatermark": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("hplWatermark", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous5", SimUnion({"hbmHeader": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmHeader": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="PROPSHEETHEADERW_V2", pack=False, align=None),
    "PROPSHEETPAGEA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszHeaderTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHeaderSubTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous3", SimUnion({"hbmHeader": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmHeader": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="PROPSHEETPAGEA", pack=False, align=None),
    "PROPSHEETPAGEA_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PROPSHEETPAGEA_V1", pack=False, align=None),
    "PROPSHEETPAGEA_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszHeaderTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHeaderSubTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PROPSHEETPAGEA_V2", pack=False, align=None),
    "PROPSHEETPAGEA_V3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszHeaderTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszHeaderSubTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PROPSHEETPAGEA_V3", pack=False, align=None),
    "PROPSHEETPAGEW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszHeaderTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHeaderSubTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous3", SimUnion({"hbmHeader": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmHeader": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="PROPSHEETPAGEW", pack=False, align=None),
    "PROPSHEETPAGEW_V1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="PROPSHEETPAGEW_V1", pack=False, align=None),
    "PROPSHEETPAGEW_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszHeaderTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHeaderSubTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROPSHEETPAGEW_V2", pack=False, align=None),
    "PROPSHEETPAGEW_V3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous1", SimUnion({"pszTemplate": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pResource": SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfnDlgProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="PSPCB_MESSAGE"), SimTypePointer(SimTypeRef("PROPSHEETPAGEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "ppsp"]), offset=0)), ("pcRefParent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pszHeaderTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszHeaderSubTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hActCtx", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PROPSHEETPAGEW_V3", pack=False, align=None),
    "PROPSHEETUI_GETICON_INFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("cxIcon", SimTypeShort(signed=False, label="UInt16")), ("cyIcon", SimTypeShort(signed=False, label="UInt16")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PROPSHEETUI_GETICON_INFO", pack=False, align=None),
    "PROPSHEETUI_INFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Reason", SimTypeShort(signed=False, label="UInt16")), ("hComPropSheet", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnComPropSheet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hComPropSheet", "Function", "lParam1", "lParam2"]), offset=0)), ("lParamInit", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UserData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Result", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PROPSHEETUI_INFO", pack=False, align=None),
    "PROPSHEETUI_INFO_HEADER": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("pTitle", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "IconID": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")),)), name="PROPSHEETUI_INFO_HEADER", pack=False, align=None),
    "PROPSPEC": SimStruct(OrderedDict((("ulKind", SimTypeInt(signed=False, label="PROPSPEC_KIND")), ("Anonymous", SimUnion({"propid": SimTypeInt(signed=False, label="UInt32"), "lpwstr": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="PROPSPEC", pack=False, align=None),
    "PROPSPEC_KIND": SimTypeInt(signed=False, label="PROPSPEC_KIND"),
    "PROPVARIANT": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("vt", SimTypeInt(signed=False, label="VARENUM")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("wReserved2", SimTypeShort(signed=False, label="UInt16")), ("wReserved3", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"cVal": SimTypeChar(label="SByte"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "lVal": SimTypeInt(signed=True, label="Int32"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "hVal": SimTypeLongLong(signed=True, label="Int64"), "uhVal": SimTypeLongLong(signed=False, label="UInt64"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), "date": SimTypeFloat(size=64), "filetime": SimTypeRef("FILETIME", SimStruct), "puuid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "pclipdata": SimTypePointer(SimTypeRef("CLIPDATA", SimStruct), offset=0), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "bstrblobVal": SimTypeRef("BSTRBLOB", SimStruct), "blob": SimTypeRef("BLOB", SimStruct), "pszVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "pStream": SimTypeBottom(label="IStream"), "pStorage": SimTypeBottom(label="IStorage"), "pVersionedStream": SimTypePointer(SimTypeRef("VERSIONEDSTREAM", SimStruct), offset=0), "parray": SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0), "cac": SimTypeRef("CAC", SimStruct), "caub": SimTypeRef("CAUB", SimStruct), "cai": SimTypeRef("CAI", SimStruct), "caui": SimTypeRef("CAUI", SimStruct), "cal": SimTypeRef("CAL", SimStruct), "caul": SimTypeRef("CAUL", SimStruct), "cah": SimTypeRef("CAH", SimStruct), "cauh": SimTypeRef("CAUH", SimStruct), "caflt": SimTypeRef("CAFLT", SimStruct), "cadbl": SimTypeRef("CADBL", SimStruct), "cabool": SimTypeRef("CABOOL", SimStruct), "cascode": SimTypeRef("CASCODE", SimStruct), "cacy": SimTypeRef("CACY", SimStruct), "cadate": SimTypeRef("CADATE", SimStruct), "cafiletime": SimTypeRef("CAFILETIME", SimStruct), "cauuid": SimTypeRef("CACLSID", SimStruct), "caclipdata": SimTypeRef("CACLIPDATA", SimStruct), "cabstr": SimTypeRef("CABSTR", SimStruct), "cabstrblob": SimTypeRef("CABSTRBLOB", SimStruct), "calpstr": SimTypeRef("CALPSTR", SimStruct), "calpwstr": SimTypeRef("CALPWSTR", SimStruct), "capropvar": SimTypeRef("CAPROPVARIANT", SimStruct), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pdecVal": SimTypePointer(SimTypeRef("DECIMAL", SimStruct), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeRef("DECIMAL", SimStruct)}, name="<anon>", label="None")),)), name="PROPVARIANT", pack=False, align=None),
    "PROPVAR_CHANGE_FLAGS": SimTypeInt(signed=False, label="PROPVAR_CHANGE_FLAGS"),
    "PROPVAR_COMPARE_FLAGS": SimTypeInt(signed=False, label="PROPVAR_COMPARE_FLAGS"),
    "PROPVAR_COMPARE_UNIT": SimTypeInt(signed=False, label="PROPVAR_COMPARE_UNIT"),
    "PROP_INFO_FLAGS": SimTypeInt(signed=False, label="PROP_INFO_FLAGS"),
    "PROTCOL_CO_AF_REGISTER_NOTIFY": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PROTECTED_FILE_DATA": SimStruct(OrderedDict((("FileName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("FileNumber", SimTypeInt(signed=False, label="UInt32")),)), name="PROTECTED_FILE_DATA", pack=False, align=None),
    "PROTOCOLDATA": SimStruct(OrderedDict((("grfFlags", SimTypeInt(signed=False, label="UInt32")), ("dwState", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")),)), name="PROTOCOLDATA", pack=False, align=None),
    "PROTOCOLFILTERDATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pProtocolSink", SimTypeBottom(label="IInternetProtocolSink")), ("pProtocol", SimTypeBottom(label="IInternetProtocol")), ("pUnk", SimTypeBottom(label="IUnknown")), ("dwFilterFlags", SimTypeInt(signed=False, label="UInt32")),)), name="PROTOCOLFILTERDATA", pack=False, align=None),
    "PROTOCOLPROPERTIES": SimTypeInt(signed=False, label="PROTOCOLPROPERTIES"),
    "PROTOCOL_ARGUMENT": SimStruct(OrderedDict((("szMethod", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szTargetUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROTOCOL_ARGUMENT", pack=False, align=None),
    "PROTOCOL_CL_ADD_PARTY_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolPartyContext", "NdisPartyHandle", "CallParameters"]), offset=0),
    "PROTOCOL_CL_CALL_CONNECTED": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ProtocolVcContext"]), offset=0),
    "PROTOCOL_CL_CLOSE_AF_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolAfContext"]), offset=0),
    "PROTOCOL_CL_CLOSE_CALL_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolVcContext", "ProtocolPartyContext"]), offset=0),
    "PROTOCOL_CL_DEREGISTER_SAP_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolSapContext"]), offset=0),
    "PROTOCOL_CL_DROP_PARTY_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolPartyContext"]), offset=0),
    "PROTOCOL_CL_INCOMING_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ProtocolSapContext", "ProtocolVcContext", "CallParameters"]), offset=0),
    "PROTOCOL_CL_INCOMING_CALL_QOS_CHANGE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["ProtocolVcContext", "CallParameters"]), offset=0),
    "PROTOCOL_CL_INCOMING_CLOSE_CALL": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["CloseStatus", "ProtocolVcContext", "CloseData", "Size"]), offset=0),
    "PROTOCOL_CL_INCOMING_DROP_PARTY": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["DropStatus", "ProtocolPartyContext", "CloseData", "Size"]), offset=0),
    "PROTOCOL_CL_MAKE_CALL_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolVcContext", "NdisPartyHandle", "CallParameters"]), offset=0),
    "PROTOCOL_CL_MODIFY_CALL_QOS_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolVcContext", "CallParameters"]), offset=0),
    "PROTOCOL_CL_OPEN_AF_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolAfContext", "NdisAfHandle"]), offset=0),
    "PROTOCOL_CL_REGISTER_SAP_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CO_SAP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "ProtocolSapContext", "Sap", "NdisSapHandle"]), offset=0),
    "PROTOCOL_CM_ACTIVATE_VC_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "CallMgrVcContext", "CallParameters"]), offset=0),
    "PROTOCOL_CM_ADD_PARTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrVcContext", "CallParameters", "NdisPartyHandle", "CallMgrPartyContext"]), offset=0),
    "PROTOCOL_CM_CLOSE_AF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrAfContext"]), offset=0),
    "PROTOCOL_CM_CLOSE_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrVcContext", "CallMgrPartyContext", "CloseData", "Size"]), offset=0),
    "PROTOCOL_CM_DEACTIVATE_VC_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "CallMgrVcContext"]), offset=0),
    "PROTOCOL_CM_DEREGISTER_SAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrSapContext"]), offset=0),
    "PROTOCOL_CM_DROP_PARTY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrPartyContext", "CloseData", "Size"]), offset=0),
    "PROTOCOL_CM_INCOMING_CALL_COMPLETE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["Status", "CallMgrVcContext", "CallParameters"]), offset=0),
    "PROTOCOL_CM_MAKE_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrVcContext", "CallParameters", "NdisPartyHandle", "CallMgrPartyContext"]), offset=0),
    "PROTOCOL_CM_MODIFY_QOS_CALL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrVcContext", "CallParameters"]), offset=0),
    "PROTOCOL_CM_OPEN_AF": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CO_ADDRESS_FAMILY", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrBindingContext", "AddressFamily", "NdisAfHandle", "CallMgrAfContext"]), offset=0),
    "PROTOCOL_CM_REG_SAP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CO_SAP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallMgrAfContext", "Sap", "NdisSapHandle", "CallMgrSapContext"]), offset=0),
    "PROTOCOL_CO_AF_REGISTER_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CO_ADDRESS_FAMILY", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["ProtocolBindingContext", "AddressFamily"]), offset=0),
    "PROTOCOL_CO_CREATE_VC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ProtocolAfContext", "NdisVcHandle", "ProtocolVcContext"]), offset=0),
    "PROTOCOL_CO_DELETE_VC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ProtocolVcContext"]), offset=0),
    "PROTOCOL_INFOA": SimStruct(OrderedDict((("dwServiceFlags", SimTypeInt(signed=False, label="UInt32")), ("iAddressFamily", SimTypeInt(signed=True, label="Int32")), ("iMaxSockAddr", SimTypeInt(signed=True, label="Int32")), ("iMinSockAddr", SimTypeInt(signed=True, label="Int32")), ("iSocketType", SimTypeInt(signed=True, label="Int32")), ("iProtocol", SimTypeInt(signed=True, label="Int32")), ("dwMessageSize", SimTypeInt(signed=False, label="UInt32")), ("lpProtocol", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PROTOCOL_INFOA", pack=False, align=None),
    "PROTOCOL_INFOW": SimStruct(OrderedDict((("dwServiceFlags", SimTypeInt(signed=False, label="UInt32")), ("iAddressFamily", SimTypeInt(signed=True, label="Int32")), ("iMaxSockAddr", SimTypeInt(signed=True, label="Int32")), ("iMinSockAddr", SimTypeInt(signed=True, label="Int32")), ("iSocketType", SimTypeInt(signed=True, label="Int32")), ("iProtocol", SimTypeInt(signed=True, label="Int32")), ("dwMessageSize", SimTypeInt(signed=False, label="UInt32")), ("lpProtocol", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROTOCOL_INFOW", pack=False, align=None),
    "PROTOENT": SimStruct(OrderedDict((("p_name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("p_aliases", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("p_proto", SimTypeShort(signed=True, label="Int16")),)), name="PROTOENT", pack=False, align=None),
    "PROVDATA_SIP": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("gSubject", SimTypeBottom(label="Guid")), ("pSip", SimTypePointer(SimTypeRef("SIP_DISPATCH_INFO", SimStruct), offset=0)), ("pCATSip", SimTypePointer(SimTypeRef("SIP_DISPATCH_INFO", SimStruct), offset=0)), ("psSipSubjectInfo", SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0)), ("psSipCATSubjectInfo", SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0)), ("psIndirectData", SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)),)), name="PROVDATA_SIP", pack=False, align=None),
    "PROVIDER_ENUMERATION_INFO": SimStruct(OrderedDict((("NumberOfProviders", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("TraceProviderInfoArray", SimTypePointer(SimTypeRef("TRACE_PROVIDER_INFO", SimStruct), offset=0)),)), name="PROVIDER_ENUMERATION_INFO", pack=False, align=None),
    "PROVIDER_EVENT_INFO": SimStruct(OrderedDict((("NumberOfEvents", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("EventDescriptorsArray", SimTypePointer(SimTypeRef("EVENT_DESCRIPTOR", SimStruct), offset=0)),)), name="PROVIDER_EVENT_INFO", pack=False, align=None),
    "PROVIDER_FIELD_INFO": SimStruct(OrderedDict((("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("DescriptionOffset", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="PROVIDER_FIELD_INFO", pack=False, align=None),
    "PROVIDER_FIELD_INFOARRAY": SimStruct(OrderedDict((("NumberOfElements", SimTypeInt(signed=False, label="UInt32")), ("FieldType", SimTypeInt(signed=False, label="EVENT_FIELD_TYPE")), ("FieldInfoArray", SimTypePointer(SimTypeRef("PROVIDER_FIELD_INFO", SimStruct), offset=0)),)), name="PROVIDER_FIELD_INFOARRAY", pack=False, align=None),
    "PROVIDER_FILTER_INFO": SimStruct(OrderedDict((("Id", SimTypeChar(label="Byte")), ("Version", SimTypeChar(label="Byte")), ("MessageOffset", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("PropertyCount", SimTypeInt(signed=False, label="UInt32")), ("EventPropertyInfoArray", SimTypePointer(SimTypeRef("EVENT_PROPERTY_INFO", SimStruct), offset=0)),)), name="PROVIDER_FILTER_INFO", pack=False, align=None),
    "PROVIDOR_INFO_1A": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDLLName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PROVIDOR_INFO_1A", pack=False, align=None),
    "PROVIDOR_INFO_1W": SimStruct(OrderedDict((("pName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDLLName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROVIDOR_INFO_1W", pack=False, align=None),
    "PROVIDOR_INFO_2A": SimStruct(OrderedDict((("pOrder", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PROVIDOR_INFO_2A", pack=False, align=None),
    "PROVIDOR_INFO_2W": SimStruct(OrderedDict((("pOrder", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROVIDOR_INFO_2W", pack=False, align=None),
    "PROV_ENUMALGS": SimStruct(OrderedDict((("aiAlgid", SimTypeInt(signed=False, label="ALG_ID")), ("dwBitLen", SimTypeInt(signed=False, label="UInt32")), ("dwNameLen", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 20)),)), name="PROV_ENUMALGS", pack=False, align=None),
    "PROV_ENUMALGS_EX": SimStruct(OrderedDict((("aiAlgid", SimTypeInt(signed=False, label="ALG_ID")), ("dwDefaultLen", SimTypeInt(signed=False, label="UInt32")), ("dwMinLen", SimTypeInt(signed=False, label="UInt32")), ("dwMaxLen", SimTypeInt(signed=False, label="UInt32")), ("dwProtocols", SimTypeInt(signed=False, label="UInt32")), ("dwNameLen", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="SByte"), 20)), ("dwLongNameLen", SimTypeInt(signed=False, label="UInt32")), ("szLongName", SimTypeArray(SimTypeChar(label="SByte"), 40)),)), name="PROV_ENUMALGS_EX", pack=False, align=None),
    "PROXIMITY_SENSOR_CAPABILITIES": SimTypeInt(signed=False, label="PROXIMITY_SENSOR_CAPABILITIES"),
    "PROXIMITY_TYPE": SimTypeInt(signed=False, label="PROXIMITY_TYPE"),
    "PROXY_ACCESS": SimTypeInt(signed=False, label="PROXY_ACCESS"),
    "PROXY_AUTO_DETECT_TYPE": SimTypeInt(signed=False, label="PROXY_AUTO_DETECT_TYPE"),
    "PROXY_CONFIG_PARAMS": SimStruct(OrderedDict((("wcThreadsMax", SimTypeShort(signed=False, label="UInt16")),)), name="PROXY_CONFIG_PARAMS", pack=False, align=None),
    "PROXY_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pcwszUserAgent", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("paUseProxy", SimTypeInt(signed=False, label="PROXY_ACCESS")), ("fLocalBypass", SimTypeInt(signed=True, label="Int32")), ("dwPortNumber", SimTypeInt(signed=False, label="UInt32")), ("pcwszProxyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszBypassList", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PROXY_INFO", pack=False, align=None),
    "PROXY_PHASE": SimTypeInt(signed=False, label="PROXY_PHASE"),
    "PRPC_RUNDOWN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["AssociationContext"]), offset=0),
    "PRTL_ALLOCATE_STRING_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0),
    "PRTL_AVL_ALLOCATE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0),
    "PRTL_AVL_COMPARE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS")), offset=0),
    "PRTL_AVL_FREE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PRTL_AVL_MATCH_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PRTL_FREE_STRING_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PRTL_GENERIC_ALLOCATE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0),
    "PRTL_GENERIC_COMPARE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS")), offset=0),
    "PRTL_GENERIC_FREE_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PRTL_HEAP_COMMIT_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PRTL_QUERY_REGISTRY_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PRTL_REALLOCATE_STRING_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0),
    "PRTL_RUN_ONCE_INIT_FN": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PRTL_UMS_SCHEDULER_ENTRY_POINT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="RTL_UMS_SCHEDULER_REASON"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Reason", "ActivationPayload", "SchedulerParam"]), offset=0),
    "PSAM_CREDENTIAL_UPDATE_FREE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["p"]), offset=0),
    "PSAM_CREDENTIAL_UPDATE_NOTIFY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClearPassword", "OldCredentials", "OldCredentialSize", "UserAccountControl", "UPN", "UserName", "NetbiosDomainName", "DnsDomainName", "NewCredentials", "NewCredentialSize"]), offset=0),
    "PSAM_CREDENTIAL_UPDATE_REGISTER_MAPPED_ENTRYPOINTS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SAM_REGISTER_MAPPING_TABLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Table"]), offset=0),
    "PSAM_CREDENTIAL_UPDATE_REGISTER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)], SimTypeChar(label="Byte"), arg_names=["CredentialName"]), offset=0),
    "PSAM_INIT_NOTIFICATION_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PSAM_PASSWORD_FILTER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["AccountName", "FullName", "Password", "SetOperation"]), offset=0),
    "PSAM_PASSWORD_NOTIFICATION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UserName", "RelativeId", "NewPassword"]), offset=0),
    "PSAPI_WORKING_SET_BLOCK": SimUnion({"Flags": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "PSAPI_WORKING_SET_EX_BLOCK": SimUnion({"Flags": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "Invalid": SimTypeRef("_Invalid_e__Struct", SimStruct)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "PSAPI_WORKING_SET_EX_INFORMATION": SimStruct(OrderedDict((("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("VirtualAttributes", SimUnion({"Flags": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "Invalid": SimTypeRef("_Invalid_e__Struct", SimStruct)}, name="<anon>", label="None")}, name="<anon>", label="None")),)), name="PSAPI_WORKING_SET_EX_INFORMATION", pack=False, align=None),
    "PSAPI_WORKING_SET_INFORMATION": SimStruct(OrderedDict((("NumberOfEntries", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetInfo", SimTypePointer(SimUnion({"Flags": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), offset=0)),)), name="PSAPI_WORKING_SET_INFORMATION", pack=False, align=None),
    "PSAPI_WS_WATCH_INFORMATION": SimStruct(OrderedDict((("FaultingPc", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FaultingVa", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PSAPI_WS_WATCH_INFORMATION", pack=False, align=None),
    "PSAPI_WS_WATCH_INFORMATION_EX": SimStruct(OrderedDict((("BasicInfo", SimTypeRef("PSAPI_WS_WATCH_INFORMATION", SimStruct)), ("FaultingThreadId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Flags", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="PSAPI_WS_WATCH_INFORMATION_EX", pack=False, align=None),
    "PSCREATEPROCESSNOTIFYTYPE": SimTypeInt(signed=False, label="PSCREATEPROCESSNOTIFYTYPE"),
    "PSCREATETHREADNOTIFYTYPE": SimTypeInt(signed=False, label="PSCREATETHREADNOTIFYTYPE"),
    "PSCRIPT5_PRIVATE_DEVMODE": SimStruct(OrderedDict((("wReserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 57)), ("wSize", SimTypeShort(signed=False, label="UInt16")),)), name="PSCRIPT5_PRIVATE_DEVMODE", pack=False, align=None),
    "PSC_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNotify", "pCallbackContext"]), offset=0),
    "PSC_NOTIFICATION_REGISTRATION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PSC_STATE": SimTypeInt(signed=False, label="PSC_STATE"),
    "PSECURE_DRIVER_PROCESS_DEREFERENCE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PSECURE_DRIVER_PROCESS_REFERENCE": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0),
    "PSECURE_MEMORY_CACHE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["Addr", "Range"]), offset=0),
    "PSECURITY_DESCRIPTOR": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "PSERENUM_READPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["SerPortAddress"]), offset=0),
    "PSERENUM_WRITEPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["SerPortAddress", "Value"]), offset=0),
    "PSET_D3COLD_SUPPORT": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PSET_INTERNAL_STATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="CLUSTER_RESOURCE_APPLICATION_STATE"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "stateType", "active"]), offset=0),
    "PSET_RESOURCE_INMEMORY_NODELOCAL_PROPERTIES_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "propertyListBuffer", "propertyListBufferSize"]), offset=0),
    "PSET_RESOURCE_LOCKED_MODE_EX_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "LockedModeEnabled", "LockedModeReason", "LockedModeFlags"]), offset=0),
    "PSET_RESOURCE_LOCKED_MODE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "LockedModeEnabled", "LockedModeReason"]), offset=0),
    "PSET_RESOURCE_STATUS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESOURCE_STATUS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "ResourceStatus"]), offset=0),
    "PSET_RESOURCE_STATUS_ROUTINE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESOURCE_STATUS_EX", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "ResourceStatus"]), offset=0),
    "PSET_RESOURCE_WPR_POLICY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "WprPolicyFlags"]), offset=0),
    "PSET_VIRTUAL_DEVICE_DATA": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PSE_IMAGE_VERIFICATION_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PSE_LOGON_SESSION_TERMINATED_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PSE_LOGON_SESSION_TERMINATED_ROUTINE_EX": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PSFEATURE_CUSTPAPER": SimStruct(OrderedDict((("lOrientation", SimTypeInt(signed=True, label="Int32")), ("lWidth", SimTypeInt(signed=True, label="Int32")), ("lHeight", SimTypeInt(signed=True, label="Int32")), ("lWidthOffset", SimTypeInt(signed=True, label="Int32")), ("lHeightOffset", SimTypeInt(signed=True, label="Int32")),)), name="PSFEATURE_CUSTPAPER", pack=False, align=None),
    "PSFEATURE_OUTPUT": SimStruct(OrderedDict((("bPageIndependent", SimTypeInt(signed=True, label="Int32")), ("bSetPageDevice", SimTypeInt(signed=True, label="Int32")),)), name="PSFEATURE_OUTPUT", pack=False, align=None),
    "PSHED_PI_ATTEMPT_ERROR_RECOVERY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "BufferLength", "ErrorRecord"]), offset=0),
    "PSHED_PI_CLEAR_ERROR_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Flags", "ErrorRecordId"]), offset=0),
    "PSHED_PI_CLEAR_ERROR_STATUS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource", "BufferLength", "ErrorRecord"]), offset=0),
    "PSHED_PI_DISABLE_ERROR_SOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0),
    "PSHED_PI_ENABLE_ERROR_SOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0),
    "PSHED_PI_ERR_READING_PCIE_OVERRIDES": SimTypeInt(signed=False, label="PSHED_PI_ERR_READING_PCIE_OVERRIDES"),
    "PSHED_PI_FINALIZE_ERROR_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource", "BufferLength", "ErrorRecord"]), offset=0),
    "PSHED_PI_GET_ALL_ERROR_SOURCES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Count", "ErrorSrcs", "Length"]), offset=0),
    "PSHED_PI_GET_ERROR_SOURCE_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0),
    "PSHED_PI_GET_INJECTION_CAPABILITIES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Capabilities"]), offset=0),
    "PSHED_PI_INJECT_ERROR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorType", "Parameter1", "Parameter2", "Parameter3", "Parameter4"]), offset=0),
    "PSHED_PI_READ_ERROR_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Flags", "ErrorRecordId", "NextErrorRecordId", "RecordLength", "ErrorRecord"]), offset=0),
    "PSHED_PI_RETRIEVE_ERROR_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("WHEA_ERROR_PACKET_V2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource", "BufferLength", "Packet"]), offset=0),
    "PSHED_PI_SET_ERROR_SOURCE_INFO": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0),
    "PSHED_PI_WRITE_ERROR_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Flags", "RecordLength", "ErrorRecord"]), offset=0),
    "PSHNOTIFY": SimStruct(OrderedDict((("hdr", SimTypeRef("NMHDR", SimStruct)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PSHNOTIFY", pack=False, align=None),
    "PSID": SimTypePointer(SimTypeBottom(label="Void"), offset=0),
    "PSIGNAL_FAILURE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="FAILURE_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "FailureType", "ApplicationSpecificErrorCode"]), offset=0),
    "PSILO_MONITOR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PSINJECTDATA": SimStruct(OrderedDict((("DataBytes", SimTypeInt(signed=False, label="UInt32")), ("InjectionPoint", SimTypeInt(signed=False, label="PSINJECT_POINT")), ("PageNumber", SimTypeShort(signed=False, label="UInt16")),)), name="PSINJECTDATA", pack=False, align=None),
    "PSINJECT_POINT": SimTypeInt(signed=False, label="PSINJECT_POINT"),
    "PSPCB_MESSAGE": SimTypeInt(signed=False, label="PSPCB_MESSAGE"),
    "PSPINFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("hComPropSheet", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCPSUIPage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfnComPropSheet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hComPropSheet", "Function", "lParam1", "lParam2"]), offset=0)),)), name="PSPINFO", pack=False, align=None),
    "PSP_DETSIG_CMPPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("SP_DEVINFO_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("SP_DEVINFO_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["DeviceInfoSet", "NewDeviceData", "ExistingDeviceData", "CompareContext"]), offset=0),
    "PSP_FILE_CALLBACK_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Notification", "Param1", "Param2"]), offset=0),
    "PSP_FILE_CALLBACK_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Notification", "Param1", "Param2"]), offset=0),
    "PSS_ALLOCATOR": SimStruct(OrderedDict((("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocRoutine", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FreeRoutine", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PSS_ALLOCATOR", pack=False, align=None),
    "PSS_AUXILIARY_PAGES_INFORMATION": SimStruct(OrderedDict((("AuxPagesCaptured", SimTypeInt(signed=False, label="UInt32")),)), name="PSS_AUXILIARY_PAGES_INFORMATION", pack=False, align=None),
    "PSS_AUXILIARY_PAGE_ENTRY": SimStruct(OrderedDict((("Address", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BasicInformation", SimTypeRef("MEMORY_BASIC_INFORMATION", SimStruct)), ("CaptureTime", SimTypeRef("FILETIME", SimStruct)), ("PageContents", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PageSize", SimTypeInt(signed=False, label="UInt32")),)), name="PSS_AUXILIARY_PAGE_ENTRY", pack=False, align=None),
    "PSS_CAPTURE_FLAGS": SimTypeInt(signed=False, label="PSS_CAPTURE_FLAGS"),
    "PSS_DUPLICATE_FLAGS": SimTypeInt(signed=False, label="PSS_DUPLICATE_FLAGS"),
    "PSS_HANDLE_ENTRY": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="PSS_HANDLE_FLAGS")), ("ObjectType", SimTypeInt(signed=False, label="PSS_OBJECT_TYPE")), ("CaptureTime", SimTypeRef("FILETIME", SimStruct)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PointerCount", SimTypeInt(signed=False, label="UInt32")), ("PagedPoolCharge", SimTypeInt(signed=False, label="UInt32")), ("NonPagedPoolCharge", SimTypeInt(signed=False, label="UInt32")), ("CreationTime", SimTypeRef("FILETIME", SimStruct)), ("TypeNameLength", SimTypeShort(signed=False, label="UInt16")), ("TypeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ObjectNameLength", SimTypeShort(signed=False, label="UInt16")), ("ObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TypeSpecificInformation", SimUnion({"Process": SimTypeRef("_Process_e__Struct", SimStruct), "Thread": SimTypeRef("_Thread_e__Struct", SimStruct), "Mutant": SimTypeRef("_Mutant_e__Struct", SimStruct), "Event": SimTypeRef("_Event_e__Struct", SimStruct), "Section": SimTypeRef("_Section_e__Struct", SimStruct), "Semaphore": SimTypeRef("_Semaphore_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PSS_HANDLE_ENTRY", pack=False, align=None),
    "PSS_HANDLE_FLAGS": SimTypeInt(signed=False, label="PSS_HANDLE_FLAGS"),
    "PSS_HANDLE_INFORMATION": SimStruct(OrderedDict((("HandlesCaptured", SimTypeInt(signed=False, label="UInt32")),)), name="PSS_HANDLE_INFORMATION", pack=False, align=None),
    "PSS_HANDLE_TRACE_INFORMATION": SimStruct(OrderedDict((("SectionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="PSS_HANDLE_TRACE_INFORMATION", pack=False, align=None),
    "PSS_OBJECT_TYPE": SimTypeInt(signed=False, label="PSS_OBJECT_TYPE"),
    "PSS_PERFORMANCE_COUNTERS": SimStruct(OrderedDict((("TotalCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("TotalWallClockPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("VaCloneCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("VaCloneWallClockPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("VaSpaceCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("VaSpaceWallClockPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("AuxPagesCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("AuxPagesWallClockPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("HandlesCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("HandlesWallClockPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("ThreadsCycleCount", SimTypeLongLong(signed=False, label="UInt64")), ("ThreadsWallClockPeriod", SimTypeLongLong(signed=False, label="UInt64")),)), name="PSS_PERFORMANCE_COUNTERS", pack=False, align=None),
    "PSS_PROCESS_FLAGS": SimTypeInt(signed=False, label="PSS_PROCESS_FLAGS"),
    "PSS_PROCESS_INFORMATION": SimStruct(OrderedDict((("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("PebBaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ParentProcessId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="PSS_PROCESS_FLAGS")), ("CreateTime", SimTypeRef("FILETIME", SimStruct)), ("ExitTime", SimTypeRef("FILETIME", SimStruct)), ("KernelTime", SimTypeRef("FILETIME", SimStruct)), ("UserTime", SimTypeRef("FILETIME", SimStruct)), ("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("PeakVirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrivateUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ExecuteFlags", SimTypeInt(signed=False, label="UInt32")), ("ImageFileName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="PSS_PROCESS_INFORMATION", pack=False, align=None),
    "PSS_QUERY_INFORMATION_CLASS": SimTypeInt(signed=False, label="PSS_QUERY_INFORMATION_CLASS"),
    "PSS_THREAD_ENTRY": SimStruct(OrderedDict((("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("TebBaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Priority", SimTypeInt(signed=True, label="Int32")), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("LastSyscallFirstArgument", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LastSyscallNumber", SimTypeShort(signed=False, label="UInt16")), ("CreateTime", SimTypeRef("FILETIME", SimStruct)), ("ExitTime", SimTypeRef("FILETIME", SimStruct)), ("KernelTime", SimTypeRef("FILETIME", SimStruct)), ("UserTime", SimTypeRef("FILETIME", SimStruct)), ("Win32StartAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CaptureTime", SimTypeRef("FILETIME", SimStruct)), ("Flags", SimTypeInt(signed=False, label="PSS_THREAD_FLAGS")), ("SuspendCount", SimTypeShort(signed=False, label="UInt16")), ("SizeOfContextRecord", SimTypeShort(signed=False, label="UInt16")), ("ContextRecord", SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0)),)), name="PSS_THREAD_ENTRY", pack=False, align=None),
    "PSS_THREAD_FLAGS": SimTypeInt(signed=False, label="PSS_THREAD_FLAGS"),
    "PSS_THREAD_INFORMATION": SimStruct(OrderedDict((("ThreadsCaptured", SimTypeInt(signed=False, label="UInt32")), ("ContextLength", SimTypeInt(signed=False, label="UInt32")),)), name="PSS_THREAD_INFORMATION", pack=False, align=None),
    "PSS_VA_CLONE_INFORMATION": SimStruct(OrderedDict((("VaCloneHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="PSS_VA_CLONE_INFORMATION", pack=False, align=None),
    "PSS_VA_SPACE_ENTRY": SimStruct(OrderedDict((("BaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationProtect", SimTypeInt(signed=False, label="UInt32")), ("RegionSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("State", SimTypeInt(signed=False, label="UInt32")), ("Protect", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("TimeDateStamp", SimTypeInt(signed=False, label="UInt32")), ("SizeOfImage", SimTypeInt(signed=False, label="UInt32")), ("ImageBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("MappedFileNameLength", SimTypeShort(signed=False, label="UInt16")), ("MappedFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="PSS_VA_SPACE_ENTRY", pack=False, align=None),
    "PSS_VA_SPACE_INFORMATION": SimStruct(OrderedDict((("RegionCount", SimTypeInt(signed=False, label="UInt32")),)), name="PSS_VA_SPACE_INFORMATION", pack=False, align=None),
    "PSS_WALK_INFORMATION_CLASS": SimTypeInt(signed=False, label="PSS_WALK_INFORMATION_CLASS"),
    "PSTABLE_UNICAST_IPADDRESS_TABLE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MIB_UNICASTIPADDRESS_TABLE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallerContext", "AddressTable"]), offset=0),
    "PSTARTUP_EX_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CLRES_CALLBACK_FUNCTION_TABLE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CLRES_FUNCTION_TABLE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceType", "MinVersionSupported", "MaxVersionSupported", "MonitorCallbackFunctions", "ResourceDllInterfaceFunctions"]), offset=0),
    "PSTARTUP_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RESOURCE_STATUS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceHandle", "ResourceStatus"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="LOG_LEVEL"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ResourceHandle", "LogLevel", "FormatString"]), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CLRES_FUNCTION_TABLE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ResourceType", "MinVersionSupported", "MaxVersionSupported", "SetResourceStatus", "LogEvent", "FunctionTable"]), offset=0),
    "PSTIME_FLAGS": SimTypeInt(signed=False, label="PSTIME_FLAGS"),
    "PSTR": SimTypePointer(SimTypeChar(label="Byte"), offset=0),
    "PSUACTION": SimTypeInt(signed=False, label="PSUACTION"),
    "PSYMBOLSERVERBYINDEXPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PSYMBOLSERVERBYINDEXPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PSYMBOLSERVERBYINDEXPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "PSYMBOLSERVERCALLBACKPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["action", "data", "context"]), offset=0),
    "PSYMBOLSERVERCLOSEPROC": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PSYMBOLSERVERDELTANAME": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "PSYMBOLSERVERDELTANAMEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0),
    "PSYMBOLSERVERGETINDEXSTRING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PSYMBOLSERVERGETINDEXSTRINGW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PSYMBOLSERVERGETOPTIONDATAPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PSYMBOLSERVERGETOPTIONSPROC": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), offset=0),
    "PSYMBOLSERVERGETSUPPLEMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PSYMBOLSERVERGETSUPPLEMENTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "PSYMBOLSERVERGETVERSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("API_VERSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERISSTORE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERISSTOREW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERMESSAGEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["action", "data", "context"]), offset=0),
    "PSYMBOLSERVEROPENPROC": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PSYMBOLSERVERPINGPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERPINGPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERPINGPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERPINGPROCWEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "PSYMBOLSERVERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PSYMBOLSERVERPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PSYMBOLSERVERPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PSYMBOLSERVERSETHTTPAUTHHEADER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszAuthHeader"]), offset=0),
    "PSYMBOLSERVERSETOPTIONSPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PSYMBOLSERVERSETOPTIONSWPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "PSYMBOLSERVERSTOREFILE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PSYMBOLSERVERSTOREFILEW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "PSYMBOLSERVERSTORESUPPLEMENT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PSYMBOLSERVERSTORESUPPLEMENTW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PSYMBOLSERVERVERSION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PSYMBOLSERVERWEXPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("SYMSRV_EXTENDED_OUTPUT_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "PSYMBOL_FUNCENTRY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["hProcess", "AddrBase", "UserContext"]), offset=0),
    "PSYMBOL_FUNCENTRY_CALLBACK64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["hProcess", "AddrBase", "UserContext"]), offset=0),
    "PSYMBOL_REGISTERED_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "ActionCode", "CallbackData", "UserContext"]), offset=0),
    "PSYMBOL_REGISTERED_CALLBACK64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "ActionCode", "CallbackData", "UserContext"]), offset=0),
    "PSYM_DUMP_FIELD_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FIELD_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pField", "UserContext"]), offset=0),
    "PSYM_ENUMERATESYMBOLS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SYMBOL_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSymInfo", "SymbolSize", "UserContext"]), offset=0),
    "PSYM_ENUMERATESYMBOLS_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SYMBOL_INFOW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSymInfo", "SymbolSize", "UserContext"]), offset=0),
    "PSYM_ENUMLINES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SRCCODEINFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LineInfo", "UserContext"]), offset=0),
    "PSYM_ENUMLINES_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SRCCODEINFOW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LineInfo", "UserContext"]), offset=0),
    "PSYM_ENUMMODULES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ModuleName", "BaseOfDll", "UserContext"]), offset=0),
    "PSYM_ENUMMODULES_CALLBACK64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ModuleName", "BaseOfDll", "UserContext"]), offset=0),
    "PSYM_ENUMMODULES_CALLBACKW64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ModuleName", "BaseOfDll", "UserContext"]), offset=0),
    "PSYM_ENUMPROCESSES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "UserContext"]), offset=0),
    "PSYM_ENUMSOURCEFILES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SOURCEFILE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSourceFile", "UserContext"]), offset=0),
    "PSYM_ENUMSOURCEFILES_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SOURCEFILEW", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSourceFile", "UserContext"]), offset=0),
    "PSYM_ENUMSYMBOLS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SymbolName", "SymbolAddress", "SymbolSize", "UserContext"]), offset=0),
    "PSYM_ENUMSYMBOLS_CALLBACK64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SymbolName", "SymbolAddress", "SymbolSize", "UserContext"]), offset=0),
    "PSYM_ENUMSYMBOLS_CALLBACK64W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SymbolName", "SymbolAddress", "SymbolSize", "UserContext"]), offset=0),
    "PSYM_ENUMSYMBOLS_CALLBACKW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SymbolName", "SymbolAddress", "SymbolSize", "UserContext"]), offset=0),
    "PS_CREATE_NOTIFY_INFO": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("ParentProcessId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CreatingThreadId", SimTypeRef("CLIENT_ID", SimStruct)), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("ImageFileName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("CommandLine", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("CreationStatus", SimTypeInt(signed=True, label="Int32")),)), name="PS_CREATE_NOTIFY_INFO", pack=False, align=None),
    "PShutdownTextServices": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pTextServices"]), offset=0),
    "PTEREDO_PORT_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="MIB_NOTIFICATION_TYPE")], SimTypeBottom(label="Void"), arg_names=["CallerContext", "Port", "NotificationType"]), offset=0),
    "PTERMINATE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Resource"]), offset=0),
    "PTERMINATION_HANDLER": SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["_abnormal_termination", "EstablisherFrame"]), offset=0),
    "PTIMERAPCROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["lpArgToCompletionRoutine", "dwTimerLowValue", "dwTimerHighValue"]), offset=0),
    "PTIMER_APC_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["TimerContext", "TimerLowValue", "TimerHighValue"]), offset=0),
    "PTLS_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["__MIDL____MIDL_itf_mscoree_0000_00040005"]), offset=0),
    "PTM_CONTROL_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("QueryGranularity", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryTimeSource", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Enable", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Disable", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="PTM_CONTROL_INTERFACE", pack=False, align=None),
    "PTM_DEVICE_DISABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PTM_DEVICE_ENABLE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context"]), offset=0),
    "PTM_DEVICE_QUERY_GRANULARITY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Granularity"]), offset=0),
    "PTM_DEVICE_QUERY_TIME_SOURCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "TimeSource"]), offset=0),
    "PTM_PROPAGATE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeBottom(label="Guid")], SimTypeInt(signed=True, label="Int32"), arg_names=["PropagationCookie", "CallbackData", "PropagationStatus", "TransactionGuid"]), offset=0),
    "PTM_RM_NOTIFICATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnlistmentObject", "RMContext", "TransactionContext", "TransactionNotification", "TmVirtualClock", "ArgumentLength", "Argument"]), offset=0),
    "PTP_CALLBACK_INSTANCE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_CLEANUP_GROUP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_CLEANUP_GROUP_CANCEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ObjectContext", "CleanupContext"]), offset=0),
    "PTP_IO": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_POOL": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_SIMPLE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Instance", "Context"]), offset=0),
    "PTP_TIMER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_TIMER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Instance", "Context", "Timer"]), offset=0),
    "PTP_WAIT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_WAIT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["Instance", "Context", "Wait", "WaitResult"]), offset=0),
    "PTP_WIN32_IO_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Instance", "Context", "Overlapped", "IoResult", "NumberOfBytesTransferred", "Io"]), offset=0),
    "PTP_WORK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PTP_WORK_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Instance", "Context", "Work"]), offset=0),
    "PTRANSLATE_ADDRESS_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ADDRESS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hProcess", "hThread", "lpaddr"]), offset=0),
    "PTRANSLATE_ADDRESS_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("ADDRESS64", SimStruct), offset=0)], SimTypeLongLong(signed=False, label="UInt64"), arg_names=["hProcess", "hThread", "lpaddr"]), offset=0),
    "PTRANSLATE_BUS_ADDRESS": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "PTRANSLATE_RESOURCE_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="RESOURCE_TRANSLATION_DIRECTION"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Source", "Direction", "AlternativesCount", "Alternatives", "PhysicalDeviceObject", "Target"]), offset=0),
    "PTRANSLATE_RESOURCE_REQUIREMENTS_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Source", "PhysicalDeviceObject", "TargetCount", "Target"]), offset=0),
    "PUAF": SimTypeInt(signed=False, label="PUAF"),
    "PUAFOUT": SimTypeInt(signed=False, label="PUAFOUT"),
    "PUBAPPINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("pszSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("stAssigned", SimTypeRef("SYSTEMTIME", SimStruct)), ("stPublished", SimTypeRef("SYSTEMTIME", SimStruct)), ("stScheduled", SimTypeRef("SYSTEMTIME", SimStruct)), ("stExpire", SimTypeRef("SYSTEMTIME", SimStruct)),)), name="PUBAPPINFO", pack=False, align=None),
    "PUBAPPINFOFLAGS": SimTypeInt(signed=False, label="PUBAPPINFOFLAGS"),
    "PUBKEY": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("bitlen", SimTypeInt(signed=False, label="UInt32")),)), name="PUBKEY", pack=False, align=None),
    "PUBKEYVER3": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("bitlenP", SimTypeInt(signed=False, label="UInt32")), ("bitlenQ", SimTypeInt(signed=False, label="UInt32")), ("bitlenJ", SimTypeInt(signed=False, label="UInt32")), ("DSSSeed", SimTypeRef("DSSSEED", SimStruct)),)), name="PUBKEYVER3", pack=False, align=None),
    "PUBLICKEYSTRUC": SimStruct(OrderedDict((("bType", SimTypeChar(label="Byte")), ("bVersion", SimTypeChar(label="Byte")), ("reserved", SimTypeShort(signed=False, label="UInt16")), ("aiKeyAlg", SimTypeInt(signed=False, label="ALG_ID")),)), name="PUBLICKEYSTRUC", pack=False, align=None),
    "PUBLIC_BCB": SimStruct(OrderedDict((("NodeTypeCode", SimTypeShort(signed=True, label="Int16")), ("NodeByteSize", SimTypeShort(signed=True, label="Int16")), ("MappedLength", SimTypeInt(signed=False, label="UInt32")), ("MappedFileOffset", SimTypeLongLong(signed=True, label="Int64")),)), name="PUBLIC_BCB", pack=False, align=None),
    "PUBLIC_OBJECT_BASIC_INFORMATION": SimStruct(OrderedDict((("Attributes", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PointerCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 10)),)), name="PUBLIC_OBJECT_BASIC_INFORMATION", pack=False, align=None),
    "PUBLIC_OBJECT_TYPE_INFORMATION": SimStruct(OrderedDict((("TypeName", SimTypeRef("UNICODE_STRING", SimStruct)), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 22)),)), name="PUBLIC_OBJECT_TYPE_INFORMATION", pack=False, align=None),
    "PUBLISHERINFO": SimStruct(OrderedDict((("dwMode", SimTypeInt(signed=False, label="UInt32")), ("wMinoutlinePPEM", SimTypeShort(signed=False, label="UInt16")), ("wMaxbitmapPPEM", SimTypeShort(signed=False, label="UInt16")),)), name="PUBLISHERINFO", pack=False, align=None),
    "PUMS_SCHEDULER_ENTRY_POINT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="RTL_UMS_SCHEDULER_REASON"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Reason", "ActivationPayload", "SchedulerParam"]), offset=0),
    "PUNCTUATION": SimStruct(OrderedDict((("iSize", SimTypeInt(signed=False, label="UInt32")), ("szPunctuation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PUNCTUATION", pack=False, align=None),
    "PUNICAST_IPADDRESS_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("MIB_UNICASTIPADDRESS_ROW", SimStruct), offset=0), SimTypeInt(signed=False, label="MIB_NOTIFICATION_TYPE")], SimTypeBottom(label="Void"), arg_names=["CallerContext", "Row", "NotificationType"]), offset=0),
    "PUNLOCK_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("FILE_LOCK_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context", "FileLockInfo"]), offset=0),
    "PUNREGISTER_FOR_DEVICE_NOTIFICATIONS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "PUNREGISTER_FOR_DEVICE_NOTIFICATIONS2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "PURGE_COMM_FLAGS": SimTypeInt(signed=False, label="PURGE_COMM_FLAGS"),
    "PUSHBUTTONDROPDOWNSTATES": SimTypeInt(signed=False, label="PUSHBUTTONDROPDOWNSTATES"),
    "PUSHBUTTONSTATES": SimTypeInt(signed=False, label="PUSHBUTTONSTATES"),
    "PVALUEA": SimStruct(OrderedDict((("pv_valuename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pv_valuelen", SimTypeInt(signed=True, label="Int32")), ("pv_value_context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pv_type", SimTypeInt(signed=False, label="UInt32")),)), name="PVALUEA", pack=False, align=None),
    "PVALUEW": SimStruct(OrderedDict((("pv_valuename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pv_valuelen", SimTypeInt(signed=True, label="Int32")), ("pv_value_context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pv_type", SimTypeInt(signed=False, label="UInt32")),)), name="PVALUEW", pack=False, align=None),
    "PVECTORED_EXCEPTION_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ExceptionInfo"]), offset=0),
    "PVIDEO_WIN32K_CALLOUT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Params"]), offset=0),
    "PVIRTUALCHANNELCLOSE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["openHandle"]), offset=0),
    "PVIRTUALCHANNELENTRY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CHANNEL_ENTRY_POINTS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pEntryPoints"]), offset=0),
    "PVIRTUALCHANNELINIT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeRef("CHANNEL_DEF", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pInitHandle", "event", "pData", "dataLength"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppInitHandle", "pChannel", "channelCount", "versionRequested", "pChannelInitEventProc"]), offset=0),
    "PVIRTUALCHANNELOPEN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["openHandle", "event", "pData", "dataLength", "totalLength", "dataFlags"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pInitHandle", "pOpenHandle", "pChannelName", "pChannelOpenEventProc"]), offset=0),
    "PVIRTUALCHANNELWRITE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["openHandle", "pData", "dataLength", "pUserData"]), offset=0),
    "PWAITCHAINCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WAITCHAIN_NODE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["WctHandle", "Context", "CallbackStatus", "NodeCount", "NodeInfoArray", "IsCycle"]), offset=0),
    "PWINBIO_ASYNC_COMPLETION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_ASYNC_RESULT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["AsyncResult"]), offset=0),
    "PWINBIO_CAPTURE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["CaptureCallbackContext", "OperationStatus", "UnitId", "Sample", "SampleSize", "RejectDetail"]), offset=0),
    "PWINBIO_ENROLL_CAPTURE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["EnrollCallbackContext", "OperationStatus", "RejectDetail"]), offset=0),
    "PWINBIO_EVENT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WINBIO_EVENT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["EventCallbackContext", "OperationStatus", "Event"]), offset=0),
    "PWINBIO_IDENTIFY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["IdentifyCallbackContext", "OperationStatus", "UnitId", "Identity", "SubFactor", "RejectDetail"]), offset=0),
    "PWINBIO_LOCATE_SENSOR_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["LocateCallbackContext", "OperationStatus", "UnitId"]), offset=0),
    "PWINBIO_QUERY_ENGINE_INTERFACE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_ENGINE_INTERFACE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EngineInterface"]), offset=0),
    "PWINBIO_QUERY_SENSOR_INTERFACE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_SENSOR_INTERFACE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SensorInterface"]), offset=0),
    "PWINBIO_QUERY_STORAGE_INTERFACE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_STORAGE_INTERFACE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["StorageInterface"]), offset=0),
    "PWINBIO_VERIFY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["VerifyCallbackContext", "OperationStatus", "UnitId", "Match", "RejectDetail"]), offset=0),
    "PWINDBG_CHECK_CONTROL_C": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PWINDBG_CHECK_VERSION": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "PWINDBG_DISASM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0),
    "PWINDBG_DISASM32": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0),
    "PWINDBG_DISASM64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0),
    "PWINDBG_EXTENSION_API_VERSION": SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeRef("EXT_API_VERSION", SimStruct), offset=0)), offset=0),
    "PWINDBG_EXTENSION_DLL_INIT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINDBG_EXTENSION_APIS", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["lpExtensionApis", "MajorVersion", "MinorVersion"]), offset=0),
    "PWINDBG_EXTENSION_DLL_INIT32": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINDBG_EXTENSION_APIS32", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["lpExtensionApis", "MajorVersion", "MinorVersion"]), offset=0),
    "PWINDBG_EXTENSION_DLL_INIT64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINDBG_EXTENSION_APIS64", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16")], SimTypeBottom(label="Void"), arg_names=["lpExtensionApis", "MajorVersion", "MinorVersion"]), offset=0),
    "PWINDBG_EXTENSION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hCurrentProcess", "hCurrentThread", "dwCurrentPc", "dwProcessor", "lpArgumentString"]), offset=0),
    "PWINDBG_EXTENSION_ROUTINE32": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hCurrentProcess", "hCurrentThread", "dwCurrentPc", "dwProcessor", "lpArgumentString"]), offset=0),
    "PWINDBG_EXTENSION_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hCurrentProcess", "hCurrentThread", "dwCurrentPc", "dwProcessor", "lpArgumentString"]), offset=0),
    "PWINDBG_GET_EXPRESSION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["lpExpression"]), offset=0),
    "PWINDBG_GET_EXPRESSION32": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpExpression"]), offset=0),
    "PWINDBG_GET_EXPRESSION64": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeLongLong(signed=False, label="UInt64"), arg_names=["lpExpression"]), offset=0),
    "PWINDBG_GET_SYMBOL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0),
    "PWINDBG_GET_SYMBOL32": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0),
    "PWINDBG_GET_SYMBOL64": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0),
    "PWINDBG_GET_THREAD_CONTEXT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0),
    "PWINDBG_IOCTL_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["IoctlType", "lpvData", "cbSize"]), offset=0),
    "PWINDBG_OLDKD_EXTENSION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WINDBG_OLDKD_EXTENSION_APIS", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwCurrentPc", "lpExtensionApis", "lpArgumentString"]), offset=0),
    "PWINDBG_OLDKD_READ_PHYSICAL_MEMORY": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["address", "buffer", "count", "bytesread"]), offset=0),
    "PWINDBG_OLDKD_WRITE_PHYSICAL_MEMORY": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["address", "buffer", "length", "byteswritten"]), offset=0),
    "PWINDBG_OLD_EXTENSION_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WINDBG_EXTENSION_APIS", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwCurrentPc", "lpExtensionApis", "lpArgumentString"]), offset=0),
    "PWINDBG_OUTPUT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFormat"]), offset=0),
    "PWINDBG_READ_PROCESS_MEMORY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0),
    "PWINDBG_READ_PROCESS_MEMORY_ROUTINE32": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0),
    "PWINDBG_READ_PROCESS_MEMORY_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0),
    "PWINDBG_SET_THREAD_CONTEXT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0),
    "PWINDBG_STACKTRACE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("EXTSTACKTRACE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["FramePointer", "StackPointer", "ProgramCounter", "StackFrames", "Frames"]), offset=0),
    "PWINDBG_STACKTRACE_ROUTINE32": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("EXTSTACKTRACE32", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["FramePointer", "StackPointer", "ProgramCounter", "StackFrames", "Frames"]), offset=0),
    "PWINDBG_STACKTRACE_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("EXTSTACKTRACE64", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["FramePointer", "StackPointer", "ProgramCounter", "StackFrames", "Frames"]), offset=0),
    "PWINDBG_WRITE_PROCESS_MEMORY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0),
    "PWINDBG_WRITE_PROCESS_MEMORY_ROUTINE32": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0),
    "PWINDBG_WRITE_PROCESS_MEMORY_ROUTINE64": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0),
    "PWINSTATIONQUERYINFORMATIONW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="WINSTATIONINFOCLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "PWLDP_CANEXECUTEBUFFER_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="WLDP_EXECUTION_EVALUATION_OPTIONS"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WLDP_EXECUTION_POLICY"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["host", "options", "buffer", "bufferSize", "auditInfo", "result"]), offset=0),
    "PWLDP_CANEXECUTEFILE_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="WLDP_EXECUTION_EVALUATION_OPTIONS"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WLDP_EXECUTION_POLICY"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["host", "options", "fileHandle", "auditInfo", "result"]), offset=0),
    "PWLDP_CANEXECUTESTREAM_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="WLDP_EXECUTION_EVALUATION_OPTIONS"), SimTypeBottom(label="IStream"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WLDP_EXECUTION_POLICY"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["host", "options", "stream", "auditInfo", "result"]), offset=0),
    "PWLDP_ISAPPAPPROVEDBYPOLICY_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PackageFamilyName", "PackageVersion"]), offset=0),
    "PWLDP_ISDYNAMICCODEPOLICYENABLED_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pbEnabled"]), offset=0),
    "PWLDP_ISPRODUCTIONCONFIGURATION_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["IsProductionConfiguration"]), offset=0),
    "PWLDP_ISWCOSPRODUCTIONCONFIGURATION_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["IsProductionConfiguration"]), offset=0),
    "PWLDP_QUERYDEVICESECURITYINFORMATION_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLDP_DEVICE_SECURITY_INFORMATION", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["information", "informationLength", "returnLength"]), offset=0),
    "PWLDP_QUERYDYNAMICODETRUST_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["fileHandle", "baseImage", "imageSize"]), offset=0),
    "PWLDP_QUERYPOLICYSETTINGENABLED2_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Setting", "Enabled"]), offset=0),
    "PWLDP_QUERYPOLICYSETTINGENABLED_API": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WLDP_POLICY_SETTING"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Setting", "Enabled"]), offset=0),
    "PWLDP_QUERYWINDOWSLOCKDOWNMODE_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="WLDP_WINDOWS_LOCKDOWN_MODE"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lockdownMode"]), offset=0),
    "PWLDP_QUERYWINDOWSLOCKDOWNRESTRICTION_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="WLDP_WINDOWS_LOCKDOWN_RESTRICTION"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LockdownRestriction"]), offset=0),
    "PWLDP_RESETPRODUCTIONCONFIGURATION_API": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PWLDP_RESETWCOSPRODUCTIONCONFIGURATION_API": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PWLDP_SETDYNAMICCODETRUST_API": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hFileHandle"]), offset=0),
    "PWLDP_SETWINDOWSLOCKDOWNRESTRICTION_API": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WLDP_WINDOWS_LOCKDOWN_RESTRICTION")], SimTypeInt(signed=True, label="Int32"), arg_names=["LockdownRestriction"]), offset=0),
    "PWLX_ASSIGN_SHELL_PROTECTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "hProcess", "hThread"]), offset=0),
    "PWLX_CHANGE_PASSWORD_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo"]), offset=0),
    "PWLX_CHANGE_PASSWORD_NOTIFY_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo", "ProviderName", "Reserved"]), offset=0),
    "PWLX_CLOSE_USER_DESKTOP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop", "hToken"]), offset=0),
    "PWLX_CREATE_USER_DESKTOP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "Flags", "pszDesktopName", "ppDesktop"]), offset=0),
    "PWLX_DIALOG_BOX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc"]), offset=0),
    "PWLX_DIALOG_BOX_INDIRECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc"]), offset=0),
    "PWLX_DIALOG_BOX_INDIRECT_PARAM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0),
    "PWLX_DIALOG_BOX_PARAM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0),
    "PWLX_DISCONNECT": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "PWLX_GET_OPTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Option", "Value"]), offset=0),
    "PWLX_GET_SOURCE_DESKTOP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "ppDesktop"]), offset=0),
    "PWLX_MESSAGE_BOX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hwndOwner", "lpszText", "lpszTitle", "fuStyle"]), offset=0),
    "PWLX_QUERY_CLIENT_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0),
    "PWLX_QUERY_CONSOLESWITCH_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CONSOLESWITCH_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pCred"]), offset=0),
    "PWLX_QUERY_IC_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0),
    "PWLX_QUERY_TERMINAL_SERVICES_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_TERMINAL_SERVICES_DATA", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hWlx", "pTSData", "UserName", "Domain"]), offset=0),
    "PWLX_QUERY_TS_LOGON_CREDENTIALS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V2_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0),
    "PWLX_SAS_NOTIFY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWlx", "dwSasType"]), offset=0),
    "PWLX_SET_CONTEXT_POINTER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx", "pWlxContext"]), offset=0),
    "PWLX_SET_OPTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Option", "Value", "OldValue"]), offset=0),
    "PWLX_SET_RETURN_DESKTOP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop"]), offset=0),
    "PWLX_SET_TIMEOUT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Timeout"]), offset=0),
    "PWLX_SWITCH_DESKTOP_TO_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0),
    "PWLX_SWITCH_DESKTOP_TO_WINLOGON": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0),
    "PWLX_USE_CTRL_ALT_DEL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0),
    "PWLX_WIN31_MIGRATE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0),
    "PWMFRect16": SimStruct(OrderedDict((("Left", SimTypeShort(signed=True, label="Int16")), ("Top", SimTypeShort(signed=True, label="Int16")), ("Right", SimTypeShort(signed=True, label="Int16")), ("Bottom", SimTypeShort(signed=True, label="Int16")),)), name="PWMFRect16", pack=False, align=None),
    "PWM_CONTROLLER_GET_ACTUAL_PERIOD_OUTPUT": SimStruct(OrderedDict((("ActualPeriod", SimTypeLongLong(signed=False, label="UInt64")),)), name="PWM_CONTROLLER_GET_ACTUAL_PERIOD_OUTPUT", pack=False, align=None),
    "PWM_CONTROLLER_INFO": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PinCount", SimTypeInt(signed=False, label="UInt32")), ("MinimumPeriod", SimTypeLongLong(signed=False, label="UInt64")), ("MaximumPeriod", SimTypeLongLong(signed=False, label="UInt64")),)), name="PWM_CONTROLLER_INFO", pack=False, align=None),
    "PWM_CONTROLLER_SET_DESIRED_PERIOD_INPUT": SimStruct(OrderedDict((("DesiredPeriod", SimTypeLongLong(signed=False, label="UInt64")),)), name="PWM_CONTROLLER_SET_DESIRED_PERIOD_INPUT", pack=False, align=None),
    "PWM_CONTROLLER_SET_DESIRED_PERIOD_OUTPUT": SimStruct(OrderedDict((("ActualPeriod", SimTypeLongLong(signed=False, label="UInt64")),)), name="PWM_CONTROLLER_SET_DESIRED_PERIOD_OUTPUT", pack=False, align=None),
    "PWM_PIN_GET_ACTIVE_DUTY_CYCLE_PERCENTAGE_OUTPUT": SimStruct(OrderedDict((("Percentage", SimTypeLongLong(signed=False, label="UInt64")),)), name="PWM_PIN_GET_ACTIVE_DUTY_CYCLE_PERCENTAGE_OUTPUT", pack=False, align=None),
    "PWM_PIN_GET_POLARITY_OUTPUT": SimStruct(OrderedDict((("Polarity", SimTypeInt(signed=False, label="PWM_POLARITY")),)), name="PWM_PIN_GET_POLARITY_OUTPUT", pack=False, align=None),
    "PWM_PIN_IS_STARTED_OUTPUT": SimStruct(OrderedDict((("IsStarted", SimTypeChar(label="Byte")),)), name="PWM_PIN_IS_STARTED_OUTPUT", pack=False, align=None),
    "PWM_PIN_SET_ACTIVE_DUTY_CYCLE_PERCENTAGE_INPUT": SimStruct(OrderedDict((("Percentage", SimTypeLongLong(signed=False, label="UInt64")),)), name="PWM_PIN_SET_ACTIVE_DUTY_CYCLE_PERCENTAGE_INPUT", pack=False, align=None),
    "PWM_PIN_SET_POLARITY_INPUT": SimStruct(OrderedDict((("Polarity", SimTypeInt(signed=False, label="PWM_POLARITY")),)), name="PWM_PIN_SET_POLARITY_INPUT", pack=False, align=None),
    "PWM_POLARITY": SimTypeInt(signed=False, label="PWM_POLARITY"),
    "PWORKER_START_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CLUS_WORKER", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pWorker", "lpThreadParameter"]), offset=0),
    "PWORKER_THREAD_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "PWRSCHEMESENUMPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("POWER_POLICY", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["Index", "NameSize", "Name", "DescriptionSize", "Description", "Policy", "Context"]), offset=0),
    "PWRSCHEMESENUMPROC_V1": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeRef("POWER_POLICY", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["Index", "NameSize", "Name", "DescriptionSize", "Description", "Policy", "Context"]), offset=0),
    "PWSD_SOAP_MESSAGE_HANDLER": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeRef("WSD_EVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["thisUnknown", "event"]), offset=0),
    "PWSTR": SimTypePointer(SimTypeChar(label="Char"), offset=0),
    "PXE_ADDRESS": SimStruct(OrderedDict((("uFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"bAddress": SimTypeArray(SimTypeChar(label="Byte"), 16), "uIpAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("uAddrLen", SimTypeInt(signed=False, label="UInt32")), ("uPort", SimTypeShort(signed=False, label="UInt16")),)), name="PXE_ADDRESS", pack=False, align=None),
    "PXE_DHCPV6_MESSAGE": SimStruct(OrderedDict((("MessageType", SimTypeChar(label="Byte")), ("TransactionIDByte1", SimTypeChar(label="Byte")), ("TransactionIDByte2", SimTypeChar(label="Byte")), ("TransactionIDByte3", SimTypeChar(label="Byte")), ("Options", SimTypePointer(SimTypeRef("PXE_DHCPV6_OPTION", SimStruct), offset=0)),)), name="PXE_DHCPV6_MESSAGE", pack=False, align=None),
    "PXE_DHCPV6_MESSAGE_HEADER": SimStruct(OrderedDict((("MessageType", SimTypeChar(label="Byte")), ("Message", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PXE_DHCPV6_MESSAGE_HEADER", pack=False, align=None),
    "PXE_DHCPV6_NESTED_RELAY_MESSAGE": SimStruct(OrderedDict((("pRelayMessage", SimTypePointer(SimTypeRef("PXE_DHCPV6_RELAY_MESSAGE", SimStruct), offset=0)), ("cbRelayMessage", SimTypeInt(signed=False, label="UInt32")), ("pInterfaceIdOption", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbInterfaceIdOption", SimTypeShort(signed=False, label="UInt16")),)), name="PXE_DHCPV6_NESTED_RELAY_MESSAGE", pack=False, align=None),
    "PXE_DHCPV6_OPTION": SimStruct(OrderedDict((("OptionCode", SimTypeShort(signed=False, label="UInt16")), ("DataLength", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PXE_DHCPV6_OPTION", pack=False, align=None),
    "PXE_DHCPV6_RELAY_MESSAGE": SimStruct(OrderedDict((("MessageType", SimTypeChar(label="Byte")), ("HopCount", SimTypeChar(label="Byte")), ("LinkAddress", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PeerAddress", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Options", SimTypePointer(SimTypeRef("PXE_DHCPV6_OPTION", SimStruct), offset=0)),)), name="PXE_DHCPV6_RELAY_MESSAGE", pack=False, align=None),
    "PXE_DHCP_MESSAGE": SimStruct(OrderedDict((("Operation", SimTypeChar(label="Byte")), ("HardwareAddressType", SimTypeChar(label="Byte")), ("HardwareAddressLength", SimTypeChar(label="Byte")), ("HopCount", SimTypeChar(label="Byte")), ("TransactionID", SimTypeInt(signed=False, label="UInt32")), ("SecondsSinceBoot", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("ClientIpAddress", SimTypeInt(signed=False, label="UInt32")), ("YourIpAddress", SimTypeInt(signed=False, label="UInt32")), ("BootstrapServerAddress", SimTypeInt(signed=False, label="UInt32")), ("RelayAgentIpAddress", SimTypeInt(signed=False, label="UInt32")), ("HardwareAddress", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("HostName", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("BootFileName", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("Anonymous", SimUnion({"bMagicCookie": SimTypeArray(SimTypeChar(label="Byte"), 4), "uMagicCookie": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Option", SimTypeRef("PXE_DHCP_OPTION", SimStruct)),)), name="PXE_DHCP_MESSAGE", pack=False, align=None),
    "PXE_DHCP_OPTION": SimStruct(OrderedDict((("OptionType", SimTypeChar(label="Byte")), ("OptionLength", SimTypeChar(label="Byte")), ("OptionValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PXE_DHCP_OPTION", pack=False, align=None),
    "PXE_PROVIDER": SimStruct(OrderedDict((("uSizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bIsCritical", SimTypeInt(signed=True, label="Int32")), ("uIndex", SimTypeInt(signed=False, label="UInt32")),)), name="PXE_PROVIDER", pack=False, align=None),
    "PackageDependencyLifetimeKind": SimTypeInt(signed=False, label="PackageDependencyLifetimeKind"),
    "PackageDependencyProcessorArchitectures": SimTypeInt(signed=False, label="PackageDependencyProcessorArchitectures"),
    "PackageInfo3Type": SimTypeInt(signed=False, label="PackageInfo3Type"),
    "PackageOrigin": SimTypeInt(signed=False, label="PackageOrigin"),
    "PackagePathType": SimTypeInt(signed=False, label="PackagePathType"),
    "PaddingMode": SimTypeInt(signed=False, label="PaddingMode"),
    "PageCountType": SimTypeInt(signed=False, label="PageCountType"),
    "PaletteFlags": SimTypeInt(signed=False, label="PaletteFlags"),
    "PaletteType": SimTypeInt(signed=False, label="PaletteType"),
    "PanelInputArea": SimTypeInt(signed=False, label="PanelInputArea"),
    "PanelType": SimTypeInt(signed=False, label="PanelType"),
    "ParentChildRelation": SimStruct(OrderedDict((("Parent", SimTypeLongLong(signed=False, label="UInt64")), ("Child", SimTypeLongLong(signed=False, label="UInt64")), ("ChildIndex", SimTypeInt(signed=False, label="UInt32")),)), name="ParentChildRelation", pack=False, align=None),
    "PartType": SimTypeInt(signed=False, label="PartType"),
    "PasswordEncodingType": SimTypeInt(signed=False, label="PasswordEncodingType"),
    "PathData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "PathPointType": SimTypeInt(signed=False, label="PathPointType"),
    "PaxosTagCStruct": SimStruct(OrderedDict((("__padding__PaxosTagVtable", SimTypeLongLong(signed=False, label="UInt64")), ("__padding__NextEpochVtable", SimTypeLongLong(signed=False, label="UInt64")), ("__padding__NextEpoch_DateTimeVtable", SimTypeLongLong(signed=False, label="UInt64")), ("NextEpoch_DateTime_ticks", SimTypeLongLong(signed=False, label="UInt64")), ("NextEpoch_Value", SimTypeInt(signed=True, label="Int32")), ("__padding__BoundryNextEpoch", SimTypeInt(signed=False, label="UInt32")), ("__padding__EpochVtable", SimTypeLongLong(signed=False, label="UInt64")), ("__padding__Epoch_DateTimeVtable", SimTypeLongLong(signed=False, label="UInt64")), ("Epoch_DateTime_ticks", SimTypeLongLong(signed=False, label="UInt64")), ("Epoch_Value", SimTypeInt(signed=True, label="Int32")), ("__padding__BoundryEpoch", SimTypeInt(signed=False, label="UInt32")), ("Sequence", SimTypeInt(signed=True, label="Int32")), ("__padding__BoundrySequence", SimTypeInt(signed=False, label="UInt32")),)), name="PaxosTagCStruct", pack=False, align=None),
    "PciLine2Pin": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler", "RootHandler", "SlotNumber", "PciNewData", "PciOldData"]), offset=0),
    "PciPin2Line": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeRef("PCI_COMMON_CONFIG", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler", "RootHandler", "SlotNumber", "PciData"]), offset=0),
    "PciReadWriteConfig": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeRef("PCI_SLOT_NUMBER", SimStruct), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["BusHandler", "Slot", "Buffer", "Offset", "Length"]), offset=0),
    "PctPublicKey": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("cbKey", SimTypeInt(signed=False, label="UInt32")), ("pKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PctPublicKey", pack=False, align=None),
    "PenAlignment": SimTypeInt(signed=False, label="PenAlignment"),
    "PenType": SimTypeInt(signed=False, label="PenType"),
    "PerfCounterDataType": SimTypeInt(signed=False, label="PerfCounterDataType"),
    "PerfRegInfoType": SimTypeInt(signed=False, label="PerfRegInfoType"),
    "PfnRecoCallback": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "PhysicalConnectorType": SimTypeInt(signed=False, label="PhysicalConnectorType"),
    "Pilot": SimTypeInt(signed=False, label="Pilot"),
    "PixelOffsetMode": SimTypeInt(signed=False, label="PixelOffsetMode"),
    "Pkcs10AllowedSignatureTypes": SimTypeInt(signed=False, label="Pkcs10AllowedSignatureTypes"),
    "PlatformId": SimTypeInt(signed=False, label="PlatformId"),
    "PlayerState": SimTypeInt(signed=False, label="PlayerState"),
    "Point": SimStruct(OrderedDict((("X", SimTypeInt(signed=True, label="Int32")), ("Y", SimTypeInt(signed=True, label="Int32")),)), name="Point", pack=False, align=None),
    "PointF": SimStruct(OrderedDict((("X", SimTypeFloat(size=32)), ("Y", SimTypeFloat(size=32)),)), name="PointF", pack=False, align=None),
    "PointerKind": SimTypeInt(signed=False, label="PointerKind"),
    "Polarisation": SimTypeInt(signed=False, label="Polarisation"),
    "PolicyAttributeType": SimTypeInt(signed=False, label="PolicyAttributeType"),
    "PolicyQualifierType": SimTypeInt(signed=False, label="PolicyQualifierType"),
    "PolicyServerUrlFlags": SimTypeInt(signed=False, label="PolicyServerUrlFlags"),
    "PolicyServerUrlPropertyID": SimTypeInt(signed=False, label="PolicyServerUrlPropertyID"),
    "PositionModeList": SimTypeInt(signed=False, label="PositionModeList"),
    "PreferredFormat": SimTypeInt(signed=False, label="PreferredFormat"),
    "PresentStatisticsKind": SimTypeInt(signed=False, label="PresentStatisticsKind"),
    "PresentStatus": SimTypeInt(signed=False, label="PresentStatus"),
    "PresentationTransform": SimStruct(OrderedDict((("M11", SimTypeFloat(size=32)), ("M12", SimTypeFloat(size=32)), ("M21", SimTypeFloat(size=32)), ("M22", SimTypeFloat(size=32)), ("M31", SimTypeFloat(size=32)), ("M32", SimTypeFloat(size=32)),)), name="PresentationTransform", pack=False, align=None),
    "PrimaryJabColors": SimStruct(OrderedDict((("red", SimTypeRef("JabColorF", SimStruct)), ("yellow", SimTypeRef("JabColorF", SimStruct)), ("green", SimTypeRef("JabColorF", SimStruct)), ("cyan", SimTypeRef("JabColorF", SimStruct)), ("blue", SimTypeRef("JabColorF", SimStruct)), ("magenta", SimTypeRef("JabColorF", SimStruct)), ("black", SimTypeRef("JabColorF", SimStruct)), ("white", SimTypeRef("JabColorF", SimStruct)),)), name="PrimaryJabColors", pack=False, align=None),
    "PrimaryXYZColors": SimStruct(OrderedDict((("red", SimTypeRef("XYZColorF", SimStruct)), ("yellow", SimTypeRef("XYZColorF", SimStruct)), ("green", SimTypeRef("XYZColorF", SimStruct)), ("cyan", SimTypeRef("XYZColorF", SimStruct)), ("blue", SimTypeRef("XYZColorF", SimStruct)), ("magenta", SimTypeRef("XYZColorF", SimStruct)), ("black", SimTypeRef("XYZColorF", SimStruct)), ("white", SimTypeRef("XYZColorF", SimStruct)),)), name="PrimaryXYZColors", pack=False, align=None),
    "PrintAsyncNotifyConversationStyle": SimTypeInt(signed=False, label="PrintAsyncNotifyConversationStyle"),
    "PrintAsyncNotifyError": SimTypeInt(signed=False, label="PrintAsyncNotifyError"),
    "PrintAsyncNotifyUserFilter": SimTypeInt(signed=False, label="PrintAsyncNotifyUserFilter"),
    "PrintDocumentPackageCompletion": SimTypeInt(signed=False, label="PrintDocumentPackageCompletion"),
    "PrintDocumentPackageStatus": SimStruct(OrderedDict((("JobId", SimTypeInt(signed=False, label="UInt32")), ("CurrentDocument", SimTypeInt(signed=True, label="Int32")), ("CurrentPage", SimTypeInt(signed=True, label="Int32")), ("CurrentPageTotal", SimTypeInt(signed=True, label="Int32")), ("Completion", SimTypeInt(signed=False, label="PrintDocumentPackageCompletion")), ("PackageStatus", SimTypeInt(signed=True, label="Int32")),)), name="PrintDocumentPackageStatus", pack=False, align=None),
    "PrintJobStatus": SimTypeInt(signed=False, label="PrintJobStatus"),
    "PrintNamedProperty": SimStruct(OrderedDict((("propertyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("propertyValue", SimTypeRef("PrintPropertyValue", SimStruct)),)), name="PrintNamedProperty", pack=False, align=None),
    "PrintPropertiesCollection": SimStruct(OrderedDict((("numberOfProperties", SimTypeInt(signed=False, label="UInt32")), ("propertiesCollection", SimTypePointer(SimTypeRef("PrintNamedProperty", SimStruct), offset=0)),)), name="PrintPropertiesCollection", pack=False, align=None),
    "PrintPropertyValue": SimStruct(OrderedDict((("ePropertyType", SimTypeInt(signed=False, label="EPrintPropertyType")), ("value", SimUnion({"propertyByte": SimTypeChar(label="Byte"), "propertyString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "propertyInt32": SimTypeInt(signed=True, label="Int32"), "propertyInt64": SimTypeLongLong(signed=True, label="Int64"), "propertyBlob": SimTypeRef("_propertyBlob_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="PrintPropertyValue", pack=False, align=None),
    "PrintSchemaConstrainedSetting": SimTypeInt(signed=False, label="PrintSchemaConstrainedSetting"),
    "PrintSchemaParameterDataType": SimTypeInt(signed=False, label="PrintSchemaParameterDataType"),
    "PrintSchemaSelectionType": SimTypeInt(signed=False, label="PrintSchemaSelectionType"),
    "PrivateData": SimStruct(OrderedDict((("size", SimTypeShort(signed=False, label="UInt16")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="PrivateData", pack=False, align=None),
    "PrivateFontCollection": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ProgramElement": SimStruct(OrderedDict((("wProgramNumber", SimTypeShort(signed=False, label="UInt16")), ("wProgramMapPID", SimTypeShort(signed=False, label="UInt16")),)), name="ProgramElement", pack=False, align=None),
    "ProofOfPossessionCookieInfo": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("data", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("p3pHeader", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="ProofOfPossessionCookieInfo", pack=False, align=None),
    "Properties": SimTypeBottom(label="Properties"),
    "Property": SimTypeBottom(label="Property"),
    "PropertyChainSource": SimStruct(OrderedDict((("Handle", SimTypeLongLong(signed=False, label="UInt64")), ("TargetType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Source", SimTypeInt(signed=False, label="BaseValueSource")), ("SrcInfo", SimTypeRef("SourceInfo", SimStruct)),)), name="PropertyChainSource", pack=False, align=None),
    "PropertyChainValue": SimStruct(OrderedDict((("Index", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DeclaringType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ValueType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ItemType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Overridden", SimTypeInt(signed=True, label="Int32")), ("MetadataBits", SimTypeLongLong(signed=True, label="Int64")), ("PropertyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PropertyChainIndex", SimTypeInt(signed=False, label="UInt32")),)), name="PropertyChainValue", pack=False, align=None),
    "PropertyConditionFlags": SimTypeInt(signed=False, label="PropertyConditionFlags"),
    "PropertyConstraint": SimTypeInt(signed=False, label="PropertyConstraint"),
    "PropertyItem": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="UInt32")), ("length", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeShort(signed=False, label="UInt16")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="PropertyItem", pack=False, align=None),
    "ProtType": SimTypeInt(signed=False, label="ProtType"),
    "ProviderOptions": SimTypeInt(signed=False, label="ProviderOptions"),
    "ProviderType": SimTypeInt(signed=False, label="ProviderType"),
    "Q2931_IE": SimStruct(OrderedDict((("IEType", SimTypeInt(signed=False, label="Q2931_IE_TYPE")), ("IELength", SimTypeInt(signed=False, label="UInt32")), ("IE", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="Q2931_IE", pack=False, align=None),
    "Q2931_IE_TYPE": SimTypeInt(signed=False, label="Q2931_IE_TYPE"),
    "QACONTAINER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pClientSite", SimTypeBottom(label="IOleClientSite")), ("pAdviseSink", SimTypeBottom(label="IAdviseSinkEx")), ("pPropertyNotifySink", SimTypeBottom(label="IPropertyNotifySink")), ("pUnkEventSink", SimTypeBottom(label="IUnknown")), ("dwAmbientFlags", SimTypeInt(signed=False, label="UInt32")), ("colorFore", SimTypeInt(signed=False, label="UInt32")), ("colorBack", SimTypeInt(signed=False, label="UInt32")), ("pFont", SimTypeBottom(label="IFont")), ("pUndoMgr", SimTypeBottom(label="IOleUndoManager")), ("dwAppearance", SimTypeInt(signed=False, label="UInt32")), ("lcid", SimTypeInt(signed=True, label="Int32")), ("hpal", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pBindHost", SimTypeBottom(label="IBindHost")), ("pOleControlSite", SimTypeBottom(label="IOleControlSite")), ("pServiceProvider", SimTypeBottom(label="IServiceProvider")),)), name="QACONTAINER", pack=False, align=None),
    "QACONTAINERFLAGS": SimTypeInt(signed=False, label="QACONTAINERFLAGS"),
    "QACONTROL": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMiscStatus", SimTypeInt(signed=False, label="UInt32")), ("dwViewStatus", SimTypeInt(signed=False, label="UInt32")), ("dwEventCookie", SimTypeInt(signed=False, label="UInt32")), ("dwPropNotifyCookie", SimTypeInt(signed=False, label="UInt32")), ("dwPointerActivationPolicy", SimTypeInt(signed=False, label="UInt32")),)), name="QACONTROL", pack=False, align=None),
    "QCMINFO": SimStruct(OrderedDict((("hmenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("indexMenu", SimTypeInt(signed=False, label="UInt32")), ("idCmdFirst", SimTypeInt(signed=False, label="UInt32")), ("idCmdLast", SimTypeInt(signed=False, label="UInt32")), ("pIdMap", SimTypePointer(SimTypeRef("QCMINFO_IDMAP", SimStruct), offset=0)),)), name="QCMINFO", pack=False, align=None),
    "QCMINFO_IDMAP": SimStruct(OrderedDict((("nMaxIds", SimTypeInt(signed=False, label="UInt32")), ("pIdList", SimTypePointer(SimTypeRef("QCMINFO_IDMAP_PLACEMENT", SimStruct), offset=0)),)), name="QCMINFO_IDMAP", pack=False, align=None),
    "QCMINFO_IDMAP_PLACEMENT": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="UInt32")), ("fFlags", SimTypeInt(signed=False, label="UInt32")),)), name="QCMINFO_IDMAP_PLACEMENT", pack=False, align=None),
    "QITAB": SimStruct(OrderedDict((("piid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwOffset", SimTypeInt(signed=False, label="UInt32")),)), name="QITAB", pack=False, align=None),
    "QITIPF_FLAGS": SimTypeInt(signed=False, label="QITIPF_FLAGS"),
    "QOCINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwInSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwOutSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="QOCINFO", pack=False, align=None),
    "QOS": SimStruct(OrderedDict((("SendingFlowspec", SimTypeRef("FLOWSPEC", SimStruct)), ("ReceivingFlowspec", SimTypeRef("FLOWSPEC", SimStruct)), ("ProviderSpecific", SimTypeRef("WSABUF", SimStruct)),)), name="QOS", pack=False, align=None),
    "QOS_DESTADDR": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("SocketAddress", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("SocketAddressLength", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_DESTADDR", pack=False, align=None),
    "QOS_DIFFSERV": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("DSFieldCount", SimTypeInt(signed=False, label="UInt32")), ("DiffservRule", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="QOS_DIFFSERV", pack=False, align=None),
    "QOS_DIFFSERV_RULE": SimStruct(OrderedDict((("InboundDSField", SimTypeChar(label="Byte")), ("ConformingOutboundDSField", SimTypeChar(label="Byte")), ("NonConformingOutboundDSField", SimTypeChar(label="Byte")), ("ConformingUserPriority", SimTypeChar(label="Byte")), ("NonConformingUserPriority", SimTypeChar(label="Byte")),)), name="QOS_DIFFSERV_RULE", pack=False, align=None),
    "QOS_DS_CLASS": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("DSField", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_DS_CLASS", pack=False, align=None),
    "QOS_EVENT": SimTypeInt(signed=False, label="QOS_EVENT"),
    "QOS_FLOWRATE_OUTGOING": SimStruct(OrderedDict((("Bandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("ShapingBehavior", SimTypeInt(signed=False, label="QOS_SHAPING")), ("Reason", SimTypeInt(signed=False, label="QOS_FLOWRATE_REASON")),)), name="QOS_FLOWRATE_OUTGOING", pack=False, align=None),
    "QOS_FLOWRATE_REASON": SimTypeInt(signed=False, label="QOS_FLOWRATE_REASON"),
    "QOS_FLOW_FUNDAMENTALS": SimStruct(OrderedDict((("BottleneckBandwidthSet", SimTypeInt(signed=True, label="Int32")), ("BottleneckBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("AvailableBandwidthSet", SimTypeInt(signed=True, label="Int32")), ("AvailableBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("RTTSet", SimTypeInt(signed=True, label="Int32")), ("RTT", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_FLOW_FUNDAMENTALS", pack=False, align=None),
    "QOS_FRIENDLY_NAME": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("FriendlyName", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="QOS_FRIENDLY_NAME", pack=False, align=None),
    "QOS_NOTIFY_FLOW": SimTypeInt(signed=False, label="QOS_NOTIFY_FLOW"),
    "QOS_OBJECT_HDR": SimStruct(OrderedDict((("ObjectType", SimTypeInt(signed=False, label="UInt32")), ("ObjectLength", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_OBJECT_HDR", pack=False, align=None),
    "QOS_PACKET_PRIORITY": SimStruct(OrderedDict((("ConformantDSCPValue", SimTypeInt(signed=False, label="UInt32")), ("NonConformantDSCPValue", SimTypeInt(signed=False, label="UInt32")), ("ConformantL2Value", SimTypeInt(signed=False, label="UInt32")), ("NonConformantL2Value", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_PACKET_PRIORITY", pack=False, align=None),
    "QOS_QUERY_FLOW": SimTypeInt(signed=False, label="QOS_QUERY_FLOW"),
    "QOS_SD_MODE": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("ShapeDiscardMode", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_SD_MODE", pack=False, align=None),
    "QOS_SERVICE_LEVEL": SimTypeInt(signed=False, label="QOS_SERVICE_LEVEL"),
    "QOS_SET_FLOW": SimTypeInt(signed=False, label="QOS_SET_FLOW"),
    "QOS_SHAPING": SimTypeInt(signed=False, label="QOS_SHAPING"),
    "QOS_SHAPING_RATE": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("ShapingRate", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_SHAPING_RATE", pack=False, align=None),
    "QOS_TCP_TRAFFIC": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)),)), name="QOS_TCP_TRAFFIC", pack=False, align=None),
    "QOS_TRAFFIC_CLASS": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("TrafficClass", SimTypeInt(signed=False, label="UInt32")),)), name="QOS_TRAFFIC_CLASS", pack=False, align=None),
    "QOS_TRAFFIC_TYPE": SimTypeInt(signed=False, label="QOS_TRAFFIC_TYPE"),
    "QOS_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="QOS_VERSION", pack=False, align=None),
    "QStatus": SimTypeInt(signed=False, label="QStatus"),
    "QUAD": SimStruct(OrderedDict((("Anonymous", SimUnion({"UseThisFieldToCopy": SimTypeLongLong(signed=True, label="Int64"), "DoNotUseThisField": SimTypeFloat(size=64)}, name="<anon>", label="None")),)), name="QUAD", pack=False, align=None),
    "QUATERNION": SimStruct(OrderedDict((("X", SimTypeFloat(size=32)), ("Y", SimTypeFloat(size=32)), ("Z", SimTypeFloat(size=32)), ("W", SimTypeFloat(size=32)),)), name="QUATERNION", pack=False, align=None),
    "QUERYASMINFO_FLAGS": SimTypeInt(signed=False, label="QUERYASMINFO_FLAGS"),
    "QUERYCLIENTCERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecPkgContext_IssuerListInfoEx", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeChar(label="Byte"), arg_names=["Connection", "trusted_CAs", "ppCertificate"]), offset=0),
    "QUERYCONTEXT": SimStruct(OrderedDict((("dwContext", SimTypeInt(signed=False, label="UInt32")), ("Platform", SimTypeRef("CSPLATFORM", SimStruct)), ("Locale", SimTypeInt(signed=False, label="UInt32")), ("dwVersionHi", SimTypeInt(signed=False, label="UInt32")), ("dwVersionLo", SimTypeInt(signed=False, label="UInt32")),)), name="QUERYCONTEXT", pack=False, align=None),
    "QUERYFORCONNECTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PrimaryConnection", "ReferralFromConnection", "NewDN", "HostName", "PortNumber", "SecAuthIdentity", "CurrentUserToken", "ConnectionToUse"]), offset=0),
    "QUERYOPTION": SimTypeInt(signed=False, label="QUERYOPTION"),
    "QUERY_BAD_RANGES_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumRanges", SimTypeInt(signed=False, label="UInt32")), ("Ranges", SimTypePointer(SimTypeRef("QUERY_BAD_RANGES_INPUT_RANGE", SimStruct), offset=0)),)), name="QUERY_BAD_RANGES_INPUT", pack=False, align=None),
    "QUERY_BAD_RANGES_INPUT_RANGE": SimStruct(OrderedDict((("StartOffset", SimTypeLongLong(signed=False, label="UInt64")), ("LengthInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="QUERY_BAD_RANGES_INPUT_RANGE", pack=False, align=None),
    "QUERY_BAD_RANGES_OUTPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumBadRanges", SimTypeInt(signed=False, label="UInt32")), ("NextOffsetToLookUp", SimTypeLongLong(signed=False, label="UInt64")), ("BadRanges", SimTypePointer(SimTypeRef("QUERY_BAD_RANGES_OUTPUT_RANGE", SimStruct), offset=0)),)), name="QUERY_BAD_RANGES_OUTPUT", pack=False, align=None),
    "QUERY_BAD_RANGES_OUTPUT_RANGE": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("StartOffset", SimTypeLongLong(signed=False, label="UInt64")), ("LengthInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="QUERY_BAD_RANGES_OUTPUT_RANGE", pack=False, align=None),
    "QUERY_CHANGES_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="QUERY_CHANGES_VIRTUAL_DISK_FLAG"),
    "QUERY_CHANGES_VIRTUAL_DISK_RANGE": SimStruct(OrderedDict((("ByteOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ByteLength", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")),)), name="QUERY_CHANGES_VIRTUAL_DISK_RANGE", pack=False, align=None),
    "QUERY_CONTEXT_ATTRIBUTES_EX_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "QUERY_CONTEXT_ATTRIBUTES_EX_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "QUERY_CONTEXT_ATTRIBUTES_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "QUERY_CONTEXT_ATTRIBUTES_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "QUERY_CREDENTIALS_ATTRIBUTES_EX_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "QUERY_CREDENTIALS_ATTRIBUTES_EX_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "QUERY_CREDENTIALS_ATTRIBUTES_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "QUERY_CREDENTIALS_ATTRIBUTES_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "QUERY_DIRECT_ACCESS_EXTENTS": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_DIRECT_ACCESS_EXTENTS", pack=False, align=None),
    "QUERY_DISPLAY_CONFIG_FLAGS": SimTypeInt(signed=False, label="QUERY_DISPLAY_CONFIG_FLAGS"),
    "QUERY_FILE_LAYOUT_FILTER_TYPE": SimTypeInt(signed=False, label="QUERY_FILE_LAYOUT_FILTER_TYPE"),
    "QUERY_FILE_LAYOUT_INPUT": SimStruct(OrderedDict((("Anonymous", SimUnion({"FilterEntryCount": SimTypeInt(signed=False, label="UInt32"), "NumberOfPairs": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FilterType", SimTypeInt(signed=False, label="QUERY_FILE_LAYOUT_FILTER_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Filter", SimUnion({"ClusterRanges": SimTypePointer(SimTypeRef("CLUSTER_RANGE", SimStruct), offset=0), "FileReferenceRanges": SimTypePointer(SimTypeRef("FILE_REFERENCE_RANGE", SimStruct), offset=0), "StorageReserveIds": SimTypePointer(SimTypeInt(signed=False, label="STORAGE_RESERVE_ID"), offset=0)}, name="<anon>", label="None")),)), name="QUERY_FILE_LAYOUT_INPUT", pack=False, align=None),
    "QUERY_FILE_LAYOUT_OUTPUT": SimStruct(OrderedDict((("FileEntryCount", SimTypeInt(signed=False, label="UInt32")), ("FirstFileOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_FILE_LAYOUT_OUTPUT", pack=False, align=None),
    "QUERY_ON_CREATE_EA_INFORMATION": SimStruct(OrderedDict((("EaBufferSize", SimTypeInt(signed=False, label="UInt32")), ("EaBuffer", SimTypePointer(SimTypeRef("FILE_FULL_EA_INFORMATION", SimStruct), offset=0)),)), name="QUERY_ON_CREATE_EA_INFORMATION", pack=False, align=None),
    "QUERY_ON_CREATE_ECP_CONTEXT": SimStruct(OrderedDict((("RequestedClasses", SimTypeInt(signed=False, label="UInt32")), ("ClassesProcessed", SimTypeInt(signed=False, label="UInt32")), ("ClassesWithErrors", SimTypeInt(signed=False, label="UInt32")), ("ClassesWithNoData", SimTypeInt(signed=False, label="UInt32")), ("StatInformation", SimTypeRef("QUERY_ON_CREATE_FILE_STAT_INFORMATION", SimStruct)), ("LxInformation", SimTypeRef("QUERY_ON_CREATE_FILE_LX_INFORMATION", SimStruct)), ("EaInformation", SimTypeRef("QUERY_ON_CREATE_EA_INFORMATION", SimStruct)),)), name="QUERY_ON_CREATE_ECP_CONTEXT", pack=False, align=None),
    "QUERY_ON_CREATE_FILE_LX_INFORMATION": SimStruct(OrderedDict((("EffectiveAccess", SimTypeInt(signed=False, label="UInt32")), ("LxFlags", SimTypeInt(signed=False, label="UInt32")), ("LxUid", SimTypeInt(signed=False, label="UInt32")), ("LxGid", SimTypeInt(signed=False, label="UInt32")), ("LxMode", SimTypeInt(signed=False, label="UInt32")), ("LxDeviceIdMajor", SimTypeInt(signed=False, label="UInt32")), ("LxDeviceIdMinor", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_ON_CREATE_FILE_LX_INFORMATION", pack=False, align=None),
    "QUERY_ON_CREATE_FILE_STAT_INFORMATION": SimStruct(OrderedDict((("FileId", SimTypeLongLong(signed=True, label="Int64")), ("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ReparseTag", SimTypeInt(signed=False, label="UInt32")), ("NumberOfLinks", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_ON_CREATE_FILE_STAT_INFORMATION", pack=False, align=None),
    "QUERY_PARSER_MANAGER_OPTION": SimTypeInt(signed=False, label="QUERY_PARSER_MANAGER_OPTION"),
    "QUERY_PATH_REQUEST": SimStruct(OrderedDict((("PathNameLength", SimTypeInt(signed=False, label="UInt32")), ("SecurityContext", SimTypePointer(SimTypeRef("IO_SECURITY_CONTEXT", SimStruct), offset=0)), ("FilePathName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="QUERY_PATH_REQUEST", pack=False, align=None),
    "QUERY_PATH_REQUEST_EX": SimStruct(OrderedDict((("pSecurityContext", SimTypePointer(SimTypeRef("IO_SECURITY_CONTEXT", SimStruct), offset=0)), ("EaLength", SimTypeInt(signed=False, label="UInt32")), ("pEaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PathName", SimTypeRef("UNICODE_STRING", SimStruct)), ("DomainServiceName", SimTypeRef("UNICODE_STRING", SimStruct)), ("EcpList", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("Silo", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="QUERY_PATH_REQUEST_EX", pack=False, align=None),
    "QUERY_PATH_RESPONSE": SimStruct(OrderedDict((("LengthAccepted", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_PATH_RESPONSE", pack=False, align=None),
    "QUERY_SECURITY_CONTEXT_TOKEN_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "QUERY_SECURITY_PACKAGE_INFO_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "QUERY_SECURITY_PACKAGE_INFO_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "QUERY_SERVICE_CONFIGA": SimStruct(OrderedDict((("dwServiceType", SimTypeInt(signed=False, label="ENUM_SERVICE_TYPE")), ("dwStartType", SimTypeInt(signed=False, label="SERVICE_START_TYPE")), ("dwErrorControl", SimTypeInt(signed=False, label="SERVICE_ERROR")), ("lpBinaryPathName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpLoadOrderGroup", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwTagId", SimTypeInt(signed=False, label="UInt32")), ("lpDependencies", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpServiceStartName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="QUERY_SERVICE_CONFIGA", pack=False, align=None),
    "QUERY_SERVICE_CONFIGW": SimStruct(OrderedDict((("dwServiceType", SimTypeInt(signed=False, label="ENUM_SERVICE_TYPE")), ("dwStartType", SimTypeInt(signed=False, label="SERVICE_START_TYPE")), ("dwErrorControl", SimTypeInt(signed=False, label="SERVICE_ERROR")), ("lpBinaryPathName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpLoadOrderGroup", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwTagId", SimTypeInt(signed=False, label="UInt32")), ("lpDependencies", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpServiceStartName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="QUERY_SERVICE_CONFIGW", pack=False, align=None),
    "QUERY_SERVICE_LOCK_STATUSA": SimStruct(OrderedDict((("fIsLocked", SimTypeInt(signed=False, label="UInt32")), ("lpLockOwner", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwLockDuration", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_SERVICE_LOCK_STATUSA", pack=False, align=None),
    "QUERY_SERVICE_LOCK_STATUSW": SimStruct(OrderedDict((("fIsLocked", SimTypeInt(signed=False, label="UInt32")), ("lpLockOwner", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLockDuration", SimTypeInt(signed=False, label="UInt32")),)), name="QUERY_SERVICE_LOCK_STATUSW", pack=False, align=None),
    "QUERY_USER_NOTIFICATION_STATE": SimTypeInt(signed=False, label="QUERY_USER_NOTIFICATION_STATE"),
    "QUERY_VIRTUAL_MEMORY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="HEAP_MEMORY_INFO_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackContext", "ProcessHandle", "BaseAddress", "MemoryInformationClass", "MemoryInformation", "MemoryInformationLength", "ReturnLength"]), offset=0),
    "QUEUE_STATE": SimTypeInt(signed=False, label="QUEUE_STATE"),
    "QUEUE_STATUS_FLAGS": SimTypeInt(signed=False, label="QUEUE_STATUS_FLAGS"),
    "QUEUE_TYPE": SimTypeInt(signed=False, label="QUEUE_TYPE"),
    "QUEUE_USER_APC_FLAGS": SimTypeInt(signed=False, label="QUEUE_USER_APC_FLAGS"),
    "QUOTA_LIMITS": SimStruct(OrderedDict((("PagedPoolLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("NonPagedPoolLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MinimumWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaximumWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TimeLimit", SimTypeLongLong(signed=True, label="Int64")),)), name="QUOTA_LIMITS", pack=False, align=None),
    "QUOTA_LIMITS_EX": SimStruct(OrderedDict((("PagedPoolLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("NonPagedPoolLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MinimumWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaximumWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TimeLimit", SimTypeLongLong(signed=True, label="Int64")), ("WorkingSetLimit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved4", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CpuRateLimit", SimUnion({"RateData": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="QUOTA_LIMITS_EX", pack=False, align=None),
    "QualAppFlowSpec": SimStruct(OrderedDict((("Q_spec_serv_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("Q_spec_parm_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("Q_spec_parms", SimTypeRef("QualTspecParms", SimStruct)),)), name="QualAppFlowSpec", pack=False, align=None),
    "QualTspec": SimStruct(OrderedDict((("qual_Tspec_serv_hdr", SimTypeRef("IntServServiceHdr", SimStruct)), ("qual_Tspec_parm_hdr", SimTypeRef("IntServParmHdr", SimStruct)), ("qual_Tspec_parms", SimTypeRef("QualTspecParms", SimStruct)),)), name="QualTspec", pack=False, align=None),
    "QualTspecParms": SimStruct(OrderedDict((("TB_Tspec_M", SimTypeInt(signed=False, label="UInt32")),)), name="QualTspecParms", pack=False, align=None),
    "Quality": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="QualityMessageType")), ("Proportion", SimTypeInt(signed=True, label="Int32")), ("Late", SimTypeLongLong(signed=True, label="Int64")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")),)), name="Quality", pack=False, align=None),
    "QualityMessageType": SimTypeInt(signed=False, label="QualityMessageType"),
    "QualityMode": SimTypeInt(signed=False, label="QualityMode"),
    "QuarantineStatus": SimTypeInt(signed=False, label="QuarantineStatus"),
    "QueryCategoryType": SimTypeInt(signed=False, label="QueryCategoryType"),
    "QueryUpdateAction": SimTypeInt(signed=False, label="QueryUpdateAction"),
    "R2_MODE": SimTypeInt(signed=False, label="R2_MODE"),
    "RADIOBUTTONSTATES": SimTypeInt(signed=False, label="RADIOBUTTONSTATES"),
    "RADIUSPROPERTIES": SimTypeInt(signed=False, label="RADIUSPROPERTIES"),
    "RADIUSPROXYPROPERTIES": SimTypeInt(signed=False, label="RADIUSPROXYPROPERTIES"),
    "RADIUSSERVERGROUPPROPERTIES": SimTypeInt(signed=False, label="RADIUSSERVERGROUPPROPERTIES"),
    "RADIUSSERVERPROPERTIES": SimTypeInt(signed=False, label="RADIUSSERVERPROPERTIES"),
    "RADIUS_ACTION": SimTypeInt(signed=False, label="RADIUS_ACTION"),
    "RADIUS_ATTRIBUTE": SimStruct(OrderedDict((("dwAttrType", SimTypeInt(signed=False, label="UInt32")), ("fDataType", SimTypeInt(signed=False, label="RADIUS_DATA_TYPE")), ("cbDataLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwValue": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="RADIUS_ATTRIBUTE", pack=False, align=None),
    "RADIUS_ATTRIBUTE_ARRAY": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Add", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AttributeAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetSize", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InsertAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RemoveAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RADIUS_ATTRIBUTE_ARRAY", pack=False, align=None),
    "RADIUS_ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="RADIUS_ATTRIBUTE_TYPE"),
    "RADIUS_AUTHENTICATION_PROVIDER": SimTypeInt(signed=False, label="RADIUS_AUTHENTICATION_PROVIDER"),
    "RADIUS_CODE": SimTypeInt(signed=False, label="RADIUS_CODE"),
    "RADIUS_DATA_TYPE": SimTypeInt(signed=False, label="RADIUS_DATA_TYPE"),
    "RADIUS_EXTENSION_CONTROL_BLOCK": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("repPoint", SimTypeInt(signed=False, label="RADIUS_EXTENSION_POINT")), ("rcRequestType", SimTypeInt(signed=False, label="RADIUS_CODE")), ("rcResponseType", SimTypeInt(signed=False, label="RADIUS_CODE")), ("GetRequest", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("GetResponse", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SetResponseType", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RADIUS_EXTENSION_CONTROL_BLOCK", pack=False, align=None),
    "RADIUS_EXTENSION_POINT": SimTypeInt(signed=False, label="RADIUS_EXTENSION_POINT"),
    "RADIUS_REJECT_REASON_CODE": SimTypeInt(signed=False, label="RADIUS_REJECT_REASON_CODE"),
    "RADIUS_VSA_FORMAT": SimStruct(OrderedDict((("VendorId", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("VendorType", SimTypeChar(label="Byte")), ("VendorLength", SimTypeChar(label="Byte")), ("AttributeSpecific", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RADIUS_VSA_FORMAT", pack=False, align=None),
    "RANGECATEGORIZE": SimStruct(OrderedDict((("cRange", SimTypeInt(signed=False, label="UInt32")), ("aRangeBegin", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)),)), name="RANGECATEGORIZE", pack=False, align=None),
    "RANGEVALUE": SimStruct(OrderedDict((("lMin", SimTypeInt(signed=True, label="Int32")), ("lMax", SimTypeInt(signed=True, label="Int32")), ("lStep", SimTypeInt(signed=True, label="Int32")),)), name="RANGEVALUE", pack=False, align=None),
    "RASADFUNCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("RASADPARAMS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "RASADFUNCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("RASADPARAMS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "RASADPARAMS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("xDlg", SimTypeInt(signed=True, label="Int32")), ("yDlg", SimTypeInt(signed=True, label="Int32")),)), name="RASADPARAMS", pack=False, align=None),
    "RASAMBA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szNetBiosError", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("bLana", SimTypeChar(label="Byte")),)), name="RASAMBA", pack=False, align=None),
    "RASAMBW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szNetBiosError", SimTypeArray(SimTypeChar(label="Char"), 17)), ("bLana", SimTypeChar(label="Byte")),)), name="RASAMBW", pack=False, align=None),
    "RASAPIVERSION": SimTypeInt(signed=False, label="RASAPIVERSION"),
    "RASAUTODIALENTRYA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDialingLocation", SimTypeInt(signed=False, label="UInt32")), ("szEntry", SimTypeArray(SimTypeChar(label="SByte"), 257)),)), name="RASAUTODIALENTRYA", pack=False, align=None),
    "RASAUTODIALENTRYW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDialingLocation", SimTypeInt(signed=False, label="UInt32")), ("szEntry", SimTypeArray(SimTypeChar(label="Char"), 257)),)), name="RASAUTODIALENTRYW", pack=False, align=None),
    "RASCOMMSETTINGS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("bParity", SimTypeChar(label="Byte")), ("bStop", SimTypeChar(label="Byte")), ("bByteSize", SimTypeChar(label="Byte")), ("bAlign", SimTypeChar(label="Byte")),)), name="RASCOMMSETTINGS", pack=False, align=None),
    "RASCONNA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hrasconn", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szEntryName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szDeviceType", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("szPhonebook", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwSubEntry", SimTypeInt(signed=False, label="UInt32")), ("guidEntry", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("luid", SimTypeRef("LUID", SimStruct)), ("guidCorrelationId", SimTypeBottom(label="Guid")),)), name="RASCONNA", pack=False, align=None),
    "RASCONNSTATE": SimTypeInt(signed=False, label="RASCONNSTATE"),
    "RASCONNSTATUSA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("rasconnstate", SimTypeInt(signed=False, label="RASCONNSTATE")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("szPhoneNumber", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("localEndPoint", SimTypeRef("RASTUNNELENDPOINT", SimStruct)), ("remoteEndPoint", SimTypeRef("RASTUNNELENDPOINT", SimStruct)), ("rasconnsubstate", SimTypeInt(signed=False, label="RASCONNSUBSTATE")),)), name="RASCONNSTATUSA", pack=False, align=None),
    "RASCONNSTATUSW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("rasconnstate", SimTypeInt(signed=False, label="RASCONNSTATE")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("localEndPoint", SimTypeRef("RASTUNNELENDPOINT", SimStruct)), ("remoteEndPoint", SimTypeRef("RASTUNNELENDPOINT", SimStruct)), ("rasconnsubstate", SimTypeInt(signed=False, label="RASCONNSUBSTATE")),)), name="RASCONNSTATUSW", pack=False, align=None),
    "RASCONNSUBSTATE": SimTypeInt(signed=False, label="RASCONNSUBSTATE"),
    "RASCONNW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hrasconn", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szEntryName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szPhonebook", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwSubEntry", SimTypeInt(signed=False, label="UInt32")), ("guidEntry", SimTypeBottom(label="Guid")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("luid", SimTypeRef("LUID", SimStruct)), ("guidCorrelationId", SimTypeBottom(label="Guid")),)), name="RASCONNW", pack=False, align=None),
    "RASCON_IPUI": SimStruct(OrderedDict((("guidConnection", SimTypeBottom(label="Guid")), ("fIPv6Cfg", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszwIpAddr", SimTypeArray(SimTypeChar(label="Char"), 16)), ("pszwDnsAddr", SimTypeArray(SimTypeChar(label="Char"), 16)), ("pszwDns2Addr", SimTypeArray(SimTypeChar(label="Char"), 16)), ("pszwWinsAddr", SimTypeArray(SimTypeChar(label="Char"), 16)), ("pszwWins2Addr", SimTypeArray(SimTypeChar(label="Char"), 16)), ("pszwDnsSuffix", SimTypeArray(SimTypeChar(label="Char"), 256)), ("pszwIpv6Addr", SimTypeArray(SimTypeChar(label="Char"), 65)), ("dwIpv6PrefixLength", SimTypeInt(signed=False, label="UInt32")), ("pszwIpv6DnsAddr", SimTypeArray(SimTypeChar(label="Char"), 65)), ("pszwIpv6Dns2Addr", SimTypeArray(SimTypeChar(label="Char"), 65)), ("dwIPv4InfMetric", SimTypeInt(signed=False, label="UInt32")), ("dwIPv6InfMetric", SimTypeInt(signed=False, label="UInt32")),)), name="RASCON_IPUI", pack=False, align=None),
    "RASCON_UIINFO_FLAGS": SimTypeInt(signed=False, label="RASCON_UIINFO_FLAGS"),
    "RASCREDENTIALSA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("szUserName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szPassword", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szDomain", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="RASCREDENTIALSA", pack=False, align=None),
    "RASCREDENTIALSW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("szUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szPassword", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szDomain", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="RASCREDENTIALSW", pack=False, align=None),
    "RASCTRYINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwNextCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("dwCountryNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="RASCTRYINFO", pack=False, align=None),
    "RASCUSTOMSCRIPTEXTENSIONS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pfnRasSetCommSettings", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RASCOMMSETTINGS", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pRasCommSettings", "pvReserved"]), offset=0)),)), name="RASCUSTOMSCRIPTEXTENSIONS", pack=False, align=None),
    "RASDEVINFOA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 129)),)), name="RASDEVINFOA", pack=False, align=None),
    "RASDEVINFOW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)),)), name="RASDEVINFOW", pack=False, align=None),
    "RASDEVSPECIFICINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pbDevSpecificInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RASDEVSPECIFICINFO", pack=False, align=None),
    "RASDIALDLG": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("xDlg", SimTypeInt(signed=True, label="Int32")), ("yDlg", SimTypeInt(signed=True, label="Int32")), ("dwSubEntry", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RASDIALDLG", pack=False, align=None),
    "RASDIALEXTENSIONS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reserved1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("RasEapInfo", SimTypeRef("RASEAPINFO", SimStruct)), ("fSkipPppAuth", SimTypeInt(signed=True, label="Int32")), ("RasDevSpecificInfo", SimTypeRef("RASDEVSPECIFICINFO", SimStruct)),)), name="RASDIALEXTENSIONS", pack=False, align=None),
    "RASDIALFUNC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="RASCONNSTATE"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2"]), offset=0),
    "RASDIALFUNC1": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="RASCONNSTATE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "RASDIALFUNC2": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="RASCONNSTATE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "RASDIALPARAMSA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szEntryName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szPhoneNumber", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("szCallbackNumber", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("szUserName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szPassword", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szDomain", SimTypeArray(SimTypeChar(label="SByte"), 16)), ("dwSubEntry", SimTypeInt(signed=False, label="UInt32")), ("dwCallbackId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("szEncPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RASDIALPARAMSA", pack=False, align=None),
    "RASDIALPARAMSW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szEntryName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szCallbackNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szPassword", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szDomain", SimTypeArray(SimTypeChar(label="Char"), 16)), ("dwSubEntry", SimTypeInt(signed=False, label="UInt32")), ("dwCallbackId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("szEncPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="RASDIALPARAMSW", pack=False, align=None),
    "RASEAPINFO": SimStruct(OrderedDict((("dwSizeofEapInfo", SimTypeInt(signed=False, label="UInt32")), ("pbEapInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RASEAPINFO", pack=False, align=None),
    "RASEAPUSERIDENTITYA": SimStruct(OrderedDict((("szUserName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("dwSizeofEapInfo", SimTypeInt(signed=False, label="UInt32")), ("pbEapInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RASEAPUSERIDENTITYA", pack=False, align=None),
    "RASEAPUSERIDENTITYW": SimStruct(OrderedDict((("szUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwSizeofEapInfo", SimTypeInt(signed=False, label="UInt32")), ("pbEapInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RASEAPUSERIDENTITYW", pack=False, align=None),
    "RASENTRYA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions", SimTypeInt(signed=False, label="UInt32")), ("dwCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("szAreaCode", SimTypeArray(SimTypeChar(label="SByte"), 11)), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("dwAlternateOffset", SimTypeInt(signed=False, label="UInt32")), ("ipaddr", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrDns", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrDnsAlt", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrWins", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrWinsAlt", SimTypeRef("RASIPADDR", SimStruct)), ("dwFrameSize", SimTypeInt(signed=False, label="UInt32")), ("dwfNetProtocols", SimTypeInt(signed=False, label="UInt32")), ("dwFramingProtocol", SimTypeInt(signed=False, label="UInt32")), ("szScript", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szAutodialDll", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szAutodialFunc", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szDeviceType", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("szX25PadType", SimTypeArray(SimTypeChar(label="SByte"), 33)), ("szX25Address", SimTypeArray(SimTypeChar(label="SByte"), 201)), ("szX25Facilities", SimTypeArray(SimTypeChar(label="SByte"), 201)), ("szX25UserData", SimTypeArray(SimTypeChar(label="SByte"), 201)), ("dwChannels", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwSubEntries", SimTypeInt(signed=False, label="UInt32")), ("dwDialMode", SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE")), ("dwDialExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwDialExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwIdleDisconnectSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("dwCustomAuthKey", SimTypeInt(signed=False, label="UInt32")), ("guidId", SimTypeBottom(label="Guid")), ("szCustomDialDll", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("dwVpnStrategy", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions2", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions3", SimTypeInt(signed=False, label="UInt32")), ("szDnsSuffix", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("dwTcpWindowSize", SimTypeInt(signed=False, label="UInt32")), ("szPrerequisitePbk", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szPrerequisiteEntry", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("dwRedialCount", SimTypeInt(signed=False, label="UInt32")), ("dwRedialPause", SimTypeInt(signed=False, label="UInt32")), ("ipv6addrDns", SimTypeRef("IN6_ADDR", SimStruct)), ("ipv6addrDnsAlt", SimTypeRef("IN6_ADDR", SimStruct)), ("dwIPv4InterfaceMetric", SimTypeInt(signed=False, label="UInt32")), ("dwIPv6InterfaceMetric", SimTypeInt(signed=False, label="UInt32")), ("ipv6addr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwIPv6PrefixLength", SimTypeInt(signed=False, label="UInt32")), ("dwNetworkOutageTime", SimTypeInt(signed=False, label="UInt32")), ("szIDi", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szIDr", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("fIsImsConfig", SimTypeInt(signed=True, label="Int32")), ("IdiType", SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE")), ("IdrType", SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE")), ("fDisableIKEv2Fragmentation", SimTypeInt(signed=True, label="Int32")),)), name="RASENTRYA", pack=False, align=None),
    "RASENTRYDLGA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("xDlg", SimTypeInt(signed=True, label="Int32")), ("yDlg", SimTypeInt(signed=True, label="Int32")), ("szEntry", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RASENTRYDLGA", pack=False, align=None),
    "RASENTRYDLGW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("xDlg", SimTypeInt(signed=True, label="Int32")), ("yDlg", SimTypeInt(signed=True, label="Int32")), ("szEntry", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RASENTRYDLGW", pack=False, align=None),
    "RASENTRYNAMEA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szEntryName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szPhonebookPath", SimTypeArray(SimTypeChar(label="SByte"), 261)),)), name="RASENTRYNAMEA", pack=False, align=None),
    "RASENTRYNAMEW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("szEntryName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szPhonebookPath", SimTypeArray(SimTypeChar(label="Char"), 261)),)), name="RASENTRYNAMEW", pack=False, align=None),
    "RASENTRYW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions", SimTypeInt(signed=False, label="UInt32")), ("dwCountryID", SimTypeInt(signed=False, label="UInt32")), ("dwCountryCode", SimTypeInt(signed=False, label="UInt32")), ("szAreaCode", SimTypeArray(SimTypeChar(label="Char"), 11)), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("dwAlternateOffset", SimTypeInt(signed=False, label="UInt32")), ("ipaddr", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrDns", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrDnsAlt", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrWins", SimTypeRef("RASIPADDR", SimStruct)), ("ipaddrWinsAlt", SimTypeRef("RASIPADDR", SimStruct)), ("dwFrameSize", SimTypeInt(signed=False, label="UInt32")), ("dwfNetProtocols", SimTypeInt(signed=False, label="UInt32")), ("dwFramingProtocol", SimTypeInt(signed=False, label="UInt32")), ("szScript", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szAutodialDll", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szAutodialFunc", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szX25PadType", SimTypeArray(SimTypeChar(label="Char"), 33)), ("szX25Address", SimTypeArray(SimTypeChar(label="Char"), 201)), ("szX25Facilities", SimTypeArray(SimTypeChar(label="Char"), 201)), ("szX25UserData", SimTypeArray(SimTypeChar(label="Char"), 201)), ("dwChannels", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwSubEntries", SimTypeInt(signed=False, label="UInt32")), ("dwDialMode", SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE")), ("dwDialExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwDialExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraPercent", SimTypeInt(signed=False, label="UInt32")), ("dwHangUpExtraSampleSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwIdleDisconnectSeconds", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionType", SimTypeInt(signed=False, label="UInt32")), ("dwCustomAuthKey", SimTypeInt(signed=False, label="UInt32")), ("guidId", SimTypeBottom(label="Guid")), ("szCustomDialDll", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwVpnStrategy", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions2", SimTypeInt(signed=False, label="UInt32")), ("dwfOptions3", SimTypeInt(signed=False, label="UInt32")), ("szDnsSuffix", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwTcpWindowSize", SimTypeInt(signed=False, label="UInt32")), ("szPrerequisitePbk", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szPrerequisiteEntry", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwRedialCount", SimTypeInt(signed=False, label="UInt32")), ("dwRedialPause", SimTypeInt(signed=False, label="UInt32")), ("ipv6addrDns", SimTypeRef("IN6_ADDR", SimStruct)), ("ipv6addrDnsAlt", SimTypeRef("IN6_ADDR", SimStruct)), ("dwIPv4InterfaceMetric", SimTypeInt(signed=False, label="UInt32")), ("dwIPv6InterfaceMetric", SimTypeInt(signed=False, label="UInt32")), ("ipv6addr", SimTypeRef("IN6_ADDR", SimStruct)), ("dwIPv6PrefixLength", SimTypeInt(signed=False, label="UInt32")), ("dwNetworkOutageTime", SimTypeInt(signed=False, label="UInt32")), ("szIDi", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szIDr", SimTypeArray(SimTypeChar(label="Char"), 257)), ("fIsImsConfig", SimTypeInt(signed=True, label="Int32")), ("IdiType", SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE")), ("IdrType", SimTypeInt(signed=False, label="IKEV2_ID_PAYLOAD_TYPE")), ("fDisableIKEv2Fragmentation", SimTypeInt(signed=True, label="Int32")),)), name="RASENTRYW", pack=False, align=None),
    "RASENTRY_DIAL_MODE": SimTypeInt(signed=False, label="RASENTRY_DIAL_MODE"),
    "RASIKEV2_PROJECTION_INFO": SimStruct(OrderedDict((("dwIPv4NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("ipv4Address", SimTypeRef("IN_ADDR", SimStruct)), ("ipv4ServerAddress", SimTypeRef("IN_ADDR", SimStruct)), ("dwIPv6NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("ipv6Address", SimTypeRef("IN6_ADDR", SimStruct)), ("ipv6ServerAddress", SimTypeRef("IN6_ADDR", SimStruct)), ("dwPrefixLength", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="RASIKEV_PROJECTION_INFO_FLAGS")), ("dwEncryptionMethod", SimTypeInt(signed=False, label="UInt32")), ("numIPv4ServerAddresses", SimTypeInt(signed=False, label="UInt32")), ("ipv4ServerAddresses", SimTypePointer(SimTypeRef("IN_ADDR", SimStruct), offset=0)), ("numIPv6ServerAddresses", SimTypeInt(signed=False, label="UInt32")), ("ipv6ServerAddresses", SimTypePointer(SimTypeRef("IN6_ADDR", SimStruct), offset=0)),)), name="RASIKEV2_PROJECTION_INFO", pack=False, align=None),
    "RASIKEV_PROJECTION_INFO_FLAGS": SimTypeInt(signed=False, label="RASIKEV_PROJECTION_INFO_FLAGS"),
    "RASIPADDR": SimStruct(OrderedDict((("a", SimTypeChar(label="Byte")), ("b", SimTypeChar(label="Byte")), ("c", SimTypeChar(label="Byte")), ("d", SimTypeChar(label="Byte")),)), name="RASIPADDR", pack=False, align=None),
    "RASIPXW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szIpxAddress", SimTypeArray(SimTypeChar(label="Char"), 22)),)), name="RASIPXW", pack=False, align=None),
    "RASNOUSERA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutMs", SimTypeInt(signed=False, label="UInt32")), ("szUserName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szPassword", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szDomain", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="RASNOUSERA", pack=False, align=None),
    "RASNOUSERW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutMs", SimTypeInt(signed=False, label="UInt32")), ("szUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szPassword", SimTypeArray(SimTypeChar(label="Char"), 257)), ("szDomain", SimTypeArray(SimTypeChar(label="Char"), 16)),)), name="RASNOUSERW", pack=False, align=None),
    "RASPBDLGA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("xDlg", SimTypeInt(signed=True, label="Int32")), ("yDlg", SimTypeInt(signed=True, label="Int32")), ("dwCallbackId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RASPBDLGA", pack=False, align=None),
    "RASPBDLGFUNCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "RASPBDLGFUNCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "RASPBDLGW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hwndOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("xDlg", SimTypeInt(signed=True, label="Int32")), ("yDlg", SimTypeInt(signed=True, label="Int32")), ("dwCallbackId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("reserved2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RASPBDLGW", pack=False, align=None),
    "RASPPPCCP": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwServerCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwServerOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RASPPPCCP", pack=False, align=None),
    "RASPPPIPA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szIpAddress", SimTypeArray(SimTypeChar(label="SByte"), 16)), ("szServerIpAddress", SimTypeArray(SimTypeChar(label="SByte"), 16)), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwServerOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RASPPPIPA", pack=False, align=None),
    "RASPPPIPV6": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("bLocalInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bPeerInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bLocalCompressionProtocol", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("bPeerCompressionProtocol", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="RASPPPIPV6", pack=False, align=None),
    "RASPPPIPW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szIpAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("szServerIpAddress", SimTypeArray(SimTypeChar(label="Char"), 16)), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwServerOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RASPPPIPW", pack=False, align=None),
    "RASPPPIPXA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("szIpxAddress", SimTypeArray(SimTypeChar(label="SByte"), 22)),)), name="RASPPPIPXA", pack=False, align=None),
    "RASPPPLCPA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fBundled", SimTypeInt(signed=True, label="Int32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationData", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwServerAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwServerAuthenticationData", SimTypeInt(signed=False, label="UInt32")), ("dwServerEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("fMultilink", SimTypeInt(signed=True, label="Int32")), ("dwTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwServerTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("szReplyMessage", SimTypeArray(SimTypeChar(label="SByte"), 1024)), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwServerOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RASPPPLCPA", pack=False, align=None),
    "RASPPPLCPW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("fBundled", SimTypeInt(signed=True, label="Int32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwAuthenticationData", SimTypeInt(signed=False, label="UInt32")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwServerAuthenticationProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwServerAuthenticationData", SimTypeInt(signed=False, label="UInt32")), ("dwServerEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("fMultilink", SimTypeInt(signed=True, label="Int32")), ("dwTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("dwServerTerminateReason", SimTypeInt(signed=False, label="UInt32")), ("szReplyMessage", SimTypeArray(SimTypeChar(label="Char"), 1024)), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwServerOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RASPPPLCPW", pack=False, align=None),
    "RASPPPNBFA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwNetBiosError", SimTypeInt(signed=False, label="UInt32")), ("szNetBiosError", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("szWorkstationName", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("bLana", SimTypeChar(label="Byte")),)), name="RASPPPNBFA", pack=False, align=None),
    "RASPPPNBFW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwNetBiosError", SimTypeInt(signed=False, label="UInt32")), ("szNetBiosError", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szWorkstationName", SimTypeArray(SimTypeChar(label="Char"), 17)), ("bLana", SimTypeChar(label="Byte")),)), name="RASPPPNBFW", pack=False, align=None),
    "RASPPP_PROJECTION_INFO": SimStruct(OrderedDict((("dwIPv4NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("ipv4Address", SimTypeRef("IN_ADDR", SimStruct)), ("ipv4ServerAddress", SimTypeRef("IN_ADDR", SimStruct)), ("dwIPv4Options", SimTypeInt(signed=False, label="UInt32")), ("dwIPv4ServerOptions", SimTypeInt(signed=False, label="UInt32")), ("dwIPv6NegotiationError", SimTypeInt(signed=False, label="UInt32")), ("bInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("bServerInterfaceIdentifier", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("fBundled", SimTypeInt(signed=True, label="Int32")), ("fMultilink", SimTypeInt(signed=True, label="Int32")), ("dwAuthenticationProtocol", SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_PROTOCOL")), ("dwAuthenticationData", SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_DATA")), ("dwServerAuthenticationProtocol", SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_PROTOCOL")), ("dwServerAuthenticationData", SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_DATA")), ("dwEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwServerEapTypeId", SimTypeInt(signed=False, label="UInt32")), ("dwLcpOptions", SimTypeInt(signed=False, label="UInt32")), ("dwLcpServerOptions", SimTypeInt(signed=False, label="UInt32")), ("dwCcpError", SimTypeInt(signed=False, label="UInt32")), ("dwCcpCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwCcpServerCompressionAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("dwCcpOptions", SimTypeInt(signed=False, label="UInt32")), ("dwCcpServerOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RASPPP_PROJECTION_INFO", pack=False, align=None),
    "RASPPP_PROJECTION_INFO_SERVER_AUTH_DATA": SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_DATA"),
    "RASPPP_PROJECTION_INFO_SERVER_AUTH_PROTOCOL": SimTypeInt(signed=False, label="RASPPP_PROJECTION_INFO_SERVER_AUTH_PROTOCOL"),
    "RASPROJECTION": SimTypeInt(signed=False, label="RASPROJECTION"),
    "RASPROJECTION_INFO_TYPE": SimTypeInt(signed=False, label="RASPROJECTION_INFO_TYPE"),
    "RASSECURITYPROC": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0),
    "RASSUBENTRYA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwfFlags", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("dwAlternateOffset", SimTypeInt(signed=False, label="UInt32")),)), name="RASSUBENTRYA", pack=False, align=None),
    "RASSUBENTRYW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwfFlags", SimTypeInt(signed=False, label="UInt32")), ("szDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("szDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("szLocalPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("dwAlternateOffset", SimTypeInt(signed=False, label="UInt32")),)), name="RASSUBENTRYW", pack=False, align=None),
    "RASTERIZER_STATUS": SimStruct(OrderedDict((("nSize", SimTypeShort(signed=True, label="Int16")), ("wFlags", SimTypeShort(signed=True, label="Int16")), ("nLanguageID", SimTypeShort(signed=True, label="Int16")),)), name="RASTERIZER_STATUS", pack=False, align=None),
    "RASTUNNELENDPOINT": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ipv4": SimTypeRef("IN_ADDR", SimStruct), "ipv6": SimTypeRef("IN6_ADDR", SimStruct)}, name="<anon>", label="None")),)), name="RASTUNNELENDPOINT", pack=False, align=None),
    "RASUPDATECONN": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="RASAPIVERSION")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("localEndPoint", SimTypeRef("RASTUNNELENDPOINT", SimStruct)), ("remoteEndPoint", SimTypeRef("RASTUNNELENDPOINT", SimStruct)),)), name="RASUPDATECONN", pack=False, align=None),
    "RAS_AUTH_ATTRIBUTE": SimStruct(OrderedDict((("raaType", SimTypeInt(signed=False, label="RAS_AUTH_ATTRIBUTE_TYPE")), ("dwLength", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RAS_AUTH_ATTRIBUTE", pack=False, align=None),
    "RAS_AUTH_ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="RAS_AUTH_ATTRIBUTE_TYPE"),
    "RAS_CONNECTION_0": SimStruct(OrderedDict((("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwConnectDuration", SimTypeInt(signed=False, label="UInt32")), ("dwInterfaceType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionFlags", SimTypeInt(signed=False, label="RAS_FLAGS")), ("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszLogonDomain", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteComputer", SimTypeArray(SimTypeChar(label="Char"), 17)),)), name="RAS_CONNECTION_0", pack=False, align=None),
    "RAS_CONNECTION_1": SimStruct(OrderedDict((("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("PppInfo", SimTypeRef("PPP_INFO", SimStruct)), ("dwBytesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwFramesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwFramesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwCrcErr", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutErr", SimTypeInt(signed=False, label="UInt32")), ("dwAlignmentErr", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwFramingErr", SimTypeInt(signed=False, label="UInt32")), ("dwBufferOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioIn", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioOut", SimTypeInt(signed=False, label="UInt32")),)), name="RAS_CONNECTION_1", pack=False, align=None),
    "RAS_CONNECTION_2": SimStruct(OrderedDict((("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwInterfaceType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("guid", SimTypeBottom(label="Guid")), ("PppInfo2", SimTypeRef("PPP_INFO_2", SimStruct)),)), name="RAS_CONNECTION_2", pack=False, align=None),
    "RAS_CONNECTION_3": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwInterfaceType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("guid", SimTypeBottom(label="Guid")), ("PppInfo3", SimTypeRef("PPP_INFO_3", SimStruct)), ("rasQuarState", SimTypeInt(signed=False, label="RAS_QUARANTINE_STATE")), ("timer", SimTypeRef("FILETIME", SimStruct)),)), name="RAS_CONNECTION_3", pack=False, align=None),
    "RAS_CONNECTION_4": SimStruct(OrderedDict((("dwConnectDuration", SimTypeInt(signed=False, label="UInt32")), ("dwInterfaceType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionFlags", SimTypeInt(signed=False, label="RAS_FLAGS")), ("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszLogonDomain", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteComputer", SimTypeArray(SimTypeChar(label="Char"), 17)), ("guid", SimTypeBottom(label="Guid")), ("rasQuarState", SimTypeInt(signed=False, label="RAS_QUARANTINE_STATE")), ("probationTime", SimTypeRef("FILETIME", SimStruct)), ("connectionStartTime", SimTypeRef("FILETIME", SimStruct)), ("ullBytesXmited", SimTypeLongLong(signed=False, label="UInt64")), ("ullBytesRcved", SimTypeLongLong(signed=False, label="UInt64")), ("dwFramesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwFramesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwCrcErr", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutErr", SimTypeInt(signed=False, label="UInt32")), ("dwAlignmentErr", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwFramingErr", SimTypeInt(signed=False, label="UInt32")), ("dwBufferOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioIn", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioOut", SimTypeInt(signed=False, label="UInt32")), ("dwNumSwitchOvers", SimTypeInt(signed=False, label="UInt32")), ("wszRemoteEndpointAddress", SimTypeArray(SimTypeChar(label="Char"), 65)), ("wszLocalEndpointAddress", SimTypeArray(SimTypeChar(label="Char"), 65)), ("ProjectionInfo", SimTypeRef("PROJECTION_INFO2", SimStruct)), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwDeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="RAS_CONNECTION_4", pack=False, align=None),
    "RAS_CONNECTION_EX": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("dwConnectDuration", SimTypeInt(signed=False, label="UInt32")), ("dwInterfaceType", SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE")), ("dwConnectionFlags", SimTypeInt(signed=False, label="RAS_FLAGS")), ("wszInterfaceName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wszLogonDomain", SimTypeArray(SimTypeChar(label="Char"), 16)), ("wszRemoteComputer", SimTypeArray(SimTypeChar(label="Char"), 17)), ("guid", SimTypeBottom(label="Guid")), ("rasQuarState", SimTypeInt(signed=False, label="RAS_QUARANTINE_STATE")), ("probationTime", SimTypeRef("FILETIME", SimStruct)), ("dwBytesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwFramesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwFramesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwCrcErr", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutErr", SimTypeInt(signed=False, label="UInt32")), ("dwAlignmentErr", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwFramingErr", SimTypeInt(signed=False, label="UInt32")), ("dwBufferOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioIn", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioOut", SimTypeInt(signed=False, label="UInt32")), ("dwNumSwitchOvers", SimTypeInt(signed=False, label="UInt32")), ("wszRemoteEndpointAddress", SimTypeArray(SimTypeChar(label="Char"), 65)), ("wszLocalEndpointAddress", SimTypeArray(SimTypeChar(label="Char"), 65)), ("ProjectionInfo", SimTypeRef("PROJECTION_INFO", SimStruct)), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInterface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RAS_CONNECTION_EX", pack=False, align=None),
    "RAS_FLAGS": SimTypeInt(signed=False, label="RAS_FLAGS"),
    "RAS_HARDWARE_CONDITION": SimTypeInt(signed=False, label="RAS_HARDWARE_CONDITION"),
    "RAS_PORT_0": SimStruct(OrderedDict((("hPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwPortCondition", SimTypeInt(signed=False, label="RAS_PORT_CONDITION")), ("dwTotalNumberOfCalls", SimTypeInt(signed=False, label="UInt32")), ("dwConnectDuration", SimTypeInt(signed=False, label="UInt32")), ("wszPortName", SimTypeArray(SimTypeChar(label="Char"), 17)), ("wszMediaName", SimTypeArray(SimTypeChar(label="Char"), 17)), ("wszDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("wszDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)),)), name="RAS_PORT_0", pack=False, align=None),
    "RAS_PORT_1": SimStruct(OrderedDict((("hPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwHardwareCondition", SimTypeInt(signed=False, label="RAS_HARDWARE_CONDITION")), ("dwLineSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwBytesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwFramesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwFramesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwCrcErr", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutErr", SimTypeInt(signed=False, label="UInt32")), ("dwAlignmentErr", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwFramingErr", SimTypeInt(signed=False, label="UInt32")), ("dwBufferOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioIn", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioOut", SimTypeInt(signed=False, label="UInt32")),)), name="RAS_PORT_1", pack=False, align=None),
    "RAS_PORT_2": SimStruct(OrderedDict((("hPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hConnection", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwConn_State", SimTypeInt(signed=False, label="UInt32")), ("wszPortName", SimTypeArray(SimTypeChar(label="Char"), 17)), ("wszMediaName", SimTypeArray(SimTypeChar(label="Char"), 17)), ("wszDeviceName", SimTypeArray(SimTypeChar(label="Char"), 129)), ("wszDeviceType", SimTypeArray(SimTypeChar(label="Char"), 17)), ("dwHardwareCondition", SimTypeInt(signed=False, label="RAS_HARDWARE_CONDITION")), ("dwLineSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwCrcErr", SimTypeInt(signed=False, label="UInt32")), ("dwSerialOverRunErrs", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutErr", SimTypeInt(signed=False, label="UInt32")), ("dwAlignmentErr", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwFramingErr", SimTypeInt(signed=False, label="UInt32")), ("dwBufferOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioIn", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioOut", SimTypeInt(signed=False, label="UInt32")), ("dwTotalErrors", SimTypeInt(signed=False, label="UInt32")), ("ullBytesXmited", SimTypeLongLong(signed=False, label="UInt64")), ("ullBytesRcved", SimTypeLongLong(signed=False, label="UInt64")), ("ullFramesXmited", SimTypeLongLong(signed=False, label="UInt64")), ("ullFramesRcved", SimTypeLongLong(signed=False, label="UInt64")), ("ullBytesTxUncompressed", SimTypeLongLong(signed=False, label="UInt64")), ("ullBytesTxCompressed", SimTypeLongLong(signed=False, label="UInt64")), ("ullBytesRcvUncompressed", SimTypeLongLong(signed=False, label="UInt64")), ("ullBytesRcvCompressed", SimTypeLongLong(signed=False, label="UInt64")),)), name="RAS_PORT_2", pack=False, align=None),
    "RAS_PORT_CONDITION": SimTypeInt(signed=False, label="RAS_PORT_CONDITION"),
    "RAS_PROJECTION_INFO": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="RASAPIVERSION")), ("type", SimTypeInt(signed=False, label="RASPROJECTION_INFO_TYPE")), ("Anonymous", SimUnion({"ppp": SimTypeRef("RASPPP_PROJECTION_INFO", SimStruct), "ikev2": SimTypeRef("RASIKEV2_PROJECTION_INFO", SimStruct)}, name="<anon>", label="None")),)), name="RAS_PROJECTION_INFO", pack=False, align=None),
    "RAS_QUARANTINE_STATE": SimTypeInt(signed=False, label="RAS_QUARANTINE_STATE"),
    "RAS_SECURITY_INFO": SimStruct(OrderedDict((("LastError", SimTypeInt(signed=False, label="UInt32")), ("BytesReceived", SimTypeInt(signed=False, label="UInt32")), ("DeviceName", SimTypeArray(SimTypeChar(label="SByte"), 129)),)), name="RAS_SECURITY_INFO", pack=False, align=None),
    "RAS_STATS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwBytesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwFramesXmited", SimTypeInt(signed=False, label="UInt32")), ("dwFramesRcved", SimTypeInt(signed=False, label="UInt32")), ("dwCrcErr", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutErr", SimTypeInt(signed=False, label="UInt32")), ("dwAlignmentErr", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwFramingErr", SimTypeInt(signed=False, label="UInt32")), ("dwBufferOverrunErr", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioIn", SimTypeInt(signed=False, label="UInt32")), ("dwCompressionRatioOut", SimTypeInt(signed=False, label="UInt32")), ("dwBps", SimTypeInt(signed=False, label="UInt32")), ("dwConnectDuration", SimTypeInt(signed=False, label="UInt32")),)), name="RAS_STATS", pack=False, align=None),
    "RAS_UPDATE_CONNECTION": SimStruct(OrderedDict((("Header", SimTypeRef("MPRAPI_OBJECT_HEADER", SimStruct)), ("dwIfIndex", SimTypeInt(signed=False, label="UInt32")), ("wszLocalEndpointAddress", SimTypeArray(SimTypeChar(label="Char"), 65)), ("wszRemoteEndpointAddress", SimTypeArray(SimTypeChar(label="Char"), 65)),)), name="RAS_UPDATE_CONNECTION", pack=False, align=None),
    "RAS_USER_0": SimStruct(OrderedDict((("bfPrivilege", SimTypeChar(label="Byte")), ("wszPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)),)), name="RAS_USER_0", pack=False, align=None),
    "RAS_USER_1": SimStruct(OrderedDict((("bfPrivilege", SimTypeChar(label="Byte")), ("wszPhoneNumber", SimTypeArray(SimTypeChar(label="Char"), 129)), ("bfPrivilege2", SimTypeChar(label="Byte")),)), name="RAS_USER_1", pack=False, align=None),
    "RATE_QUOTA_LIMIT": SimUnion({"RateData": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "RATING_ATTRIBUTE": SimStruct(OrderedDict((("rating_attribute_id", SimTypeInt(signed=False, label="UInt32")), ("rating_attribute_value", SimTypeInt(signed=False, label="UInt32")),)), name="RATING_ATTRIBUTE", pack=False, align=None),
    "RATING_INFO": SimStruct(OrderedDict((("rating_system_count", SimTypeInt(signed=False, label="UInt32")), ("lpratingsystem", SimTypePointer(SimTypeRef("RATING_SYSTEM", SimStruct), offset=0)),)), name="RATING_INFO", pack=False, align=None),
    "RATING_SYSTEM": SimStruct(OrderedDict((("rating_system_id", SimTypeBottom(label="Guid")), ("_bitfield", SimTypeChar(label="Byte")), ("country_code", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("rating_attribute_count", SimTypeInt(signed=False, label="UInt32")), ("lpratingattrib", SimTypePointer(SimTypeRef("RATING_ATTRIBUTE", SimStruct), offset=0)),)), name="RATING_SYSTEM", pack=False, align=None),
    "RAWHID": SimStruct(OrderedDict((("dwSizeHid", SimTypeInt(signed=False, label="UInt32")), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("bRawData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RAWHID", pack=False, align=None),
    "RAWINPUT": SimStruct(OrderedDict((("header", SimTypeRef("RAWINPUTHEADER", SimStruct)), ("data", SimUnion({"mouse": SimTypeRef("RAWMOUSE", SimStruct), "keyboard": SimTypeRef("RAWKEYBOARD", SimStruct), "hid": SimTypeRef("RAWHID", SimStruct)}, name="<anon>", label="None")),)), name="RAWINPUT", pack=False, align=None),
    "RAWINPUTDEVICE": SimStruct(OrderedDict((("usUsagePage", SimTypeShort(signed=False, label="UInt16")), ("usUsage", SimTypeShort(signed=False, label="UInt16")), ("dwFlags", SimTypeInt(signed=False, label="RAWINPUTDEVICE_FLAGS")), ("hwndTarget", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RAWINPUTDEVICE", pack=False, align=None),
    "RAWINPUTDEVICELIST": SimStruct(OrderedDict((("hDevice", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwType", SimTypeInt(signed=False, label="RID_DEVICE_INFO_TYPE")),)), name="RAWINPUTDEVICELIST", pack=False, align=None),
    "RAWINPUTDEVICE_FLAGS": SimTypeInt(signed=False, label="RAWINPUTDEVICE_FLAGS"),
    "RAWINPUTHEADER": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hDevice", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RAWINPUTHEADER", pack=False, align=None),
    "RAWKEYBOARD": SimStruct(OrderedDict((("MakeCode", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("VKey", SimTypeShort(signed=False, label="UInt16")), ("Message", SimTypeInt(signed=False, label="UInt32")), ("ExtraInformation", SimTypeInt(signed=False, label="UInt32")),)), name="RAWKEYBOARD", pack=False, align=None),
    "RAWMOUSE": SimStruct(OrderedDict((("usFlags", SimTypeInt(signed=False, label="MOUSE_STATE")), ("Anonymous", SimUnion({"ulButtons": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("usButtonFlags", SimTypeShort(signed=False, label="UInt16")), ("usButtonData", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("ulRawButtons", SimTypeInt(signed=False, label="UInt32")), ("lLastX", SimTypeInt(signed=True, label="Int32")), ("lLastY", SimTypeInt(signed=True, label="Int32")), ("ulExtraInformation", SimTypeInt(signed=False, label="UInt32")),)), name="RAWMOUSE", pack=False, align=None),
    "RAW_INPUT_DATA_COMMAND_FLAGS": SimTypeInt(signed=False, label="RAW_INPUT_DATA_COMMAND_FLAGS"),
    "RAW_INPUT_DEVICE_INFO_COMMAND": SimTypeInt(signed=False, label="RAW_INPUT_DEVICE_INFO_COMMAND"),
    "RAW_PIPE_TYPE": SimTypeInt(signed=False, label="RAW_PIPE_TYPE"),
    "RAW_RESET_PORT_PARAMETERS": SimStruct(OrderedDict((("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("PortStatus", SimTypeShort(signed=False, label="UInt16")),)), name="RAW_RESET_PORT_PARAMETERS", pack=False, align=None),
    "RAW_ROOTPORT_FEATURE": SimStruct(OrderedDict((("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("PortFeature", SimTypeShort(signed=False, label="UInt16")), ("PortStatus", SimTypeShort(signed=False, label="UInt16")),)), name="RAW_ROOTPORT_FEATURE", pack=False, align=None),
    "RAW_ROOTPORT_PARAMETERS": SimStruct(OrderedDict((("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("PortStatus", SimTypeShort(signed=False, label="UInt16")),)), name="RAW_ROOTPORT_PARAMETERS", pack=False, align=None),
    "RAW_SCSI_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="RAW_SCSI_VIRTUAL_DISK_FLAG"),
    "RAW_SCSI_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="RAW_SCSI_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="RAW_SCSI_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "RAW_SCSI_VIRTUAL_DISK_RESPONSE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="RAW_SCSI_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="RAW_SCSI_VIRTUAL_DISK_RESPONSE", pack=False, align=None),
    "RAW_SCSI_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="RAW_SCSI_VIRTUAL_DISK_VERSION"),
    "RBHITTESTINFO": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("flags", SimTypeInt(signed=False, label="UInt32")), ("iBand", SimTypeInt(signed=True, label="Int32")),)), name="RBHITTESTINFO", pack=False, align=None),
    "RCVALL_IF": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="RCVALL_VALUE")), ("Interface", SimTypeInt(signed=False, label="UInt32")),)), name="RCVALL_IF", pack=False, align=None),
    "RCVALL_VALUE": SimTypeInt(signed=False, label="RCVALL_VALUE"),
    "RDCOMPARE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("nColumn", SimTypeInt(signed=True, label="Int32")), ("lUserParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("prdch1", SimTypePointer(SimTypeRef("RDITEMHDR", SimStruct), offset=0)), ("prdch2", SimTypePointer(SimTypeRef("RDITEMHDR", SimStruct), offset=0)),)), name="RDCOMPARE", pack=False, align=None),
    "RDC_ErrorCode": SimTypeInt(signed=False, label="RDC_ErrorCode"),
    "RDITEMHDR": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cookie", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpReserved", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RDITEMHDR", pack=False, align=None),
    "RDPENCOMAPI_ATTENDEE_FLAGS": SimTypeInt(signed=False, label="RDPENCOMAPI_ATTENDEE_FLAGS"),
    "RDPENCOMAPI_CONSTANTS": SimTypeInt(signed=False, label="RDPENCOMAPI_CONSTANTS"),
    "RDPSRAPI_APP_FLAGS": SimTypeInt(signed=False, label="RDPSRAPI_APP_FLAGS"),
    "RDPSRAPI_KBD_CODE_TYPE": SimTypeInt(signed=False, label="RDPSRAPI_KBD_CODE_TYPE"),
    "RDPSRAPI_KBD_SYNC_FLAG": SimTypeInt(signed=False, label="RDPSRAPI_KBD_SYNC_FLAG"),
    "RDPSRAPI_MOUSE_BUTTON_TYPE": SimTypeInt(signed=False, label="RDPSRAPI_MOUSE_BUTTON_TYPE"),
    "RDPSRAPI_WND_FLAGS": SimTypeInt(signed=False, label="RDPSRAPI_WND_FLAGS"),
    "RDR_CALLOUT_STATE": SimStruct(OrderedDict((("LastError", SimTypeInt(signed=False, label="RPC_STATUS")), ("LastEEInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LastCalledStage", SimTypeInt(signed=False, label="RPC_HTTP_REDIRECTOR_STAGE")), ("ServerName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ServerPort", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("RemoteUser", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("AuthType", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ResourceTypePresent", SimTypeChar(label="Byte")), ("SessionIdPresent", SimTypeChar(label="Byte")), ("InterfacePresent", SimTypeChar(label="Byte")), ("ResourceType", SimTypeBottom(label="Guid")), ("SessionId", SimTypeBottom(label="Guid")), ("Interface", SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct)), ("CertContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RDR_CALLOUT_STATE", pack=False, align=None),
    "RDV_TASK_STATUS": SimTypeInt(signed=False, label="RDV_TASK_STATUS"),
    "RD_FARM_TYPE": SimTypeInt(signed=False, label="RD_FARM_TYPE"),
    "READCONTROLSPACE": SimStruct(OrderedDict((("Processor", SimTypeShort(signed=False, label="UInt16")), ("Address", SimTypeInt(signed=False, label="UInt32")), ("BufLen", SimTypeInt(signed=False, label="UInt32")), ("Buf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="READCONTROLSPACE", pack=False, align=None),
    "READCONTROLSPACE32": SimStruct(OrderedDict((("Processor", SimTypeShort(signed=False, label="UInt16")), ("Address", SimTypeInt(signed=False, label="UInt32")), ("BufLen", SimTypeInt(signed=False, label="UInt32")), ("Buf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="READCONTROLSPACE32", pack=False, align=None),
    "READCONTROLSPACE64": SimStruct(OrderedDict((("Processor", SimTypeShort(signed=False, label="UInt16")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("BufLen", SimTypeInt(signed=False, label="UInt32")), ("Buf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="READCONTROLSPACE64", pack=False, align=None),
    "READEMBEDPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "READER_SEL_REQUEST": SimStruct(OrderedDict((("dwShareMode", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredProtocols", SimTypeInt(signed=False, label="UInt32")), ("MatchType", SimTypeInt(signed=False, label="READER_SEL_REQUEST_MATCH_TYPE")), ("Anonymous", SimUnion({"ReaderAndContainerParameter": SimTypeRef("_ReaderAndContainerParameter_e__Struct", SimStruct), "SerialNumberParameter": SimTypeRef("_SerialNumberParameter_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="READER_SEL_REQUEST", pack=False, align=None),
    "READER_SEL_REQUEST_MATCH_TYPE": SimTypeInt(signed=False, label="READER_SEL_REQUEST_MATCH_TYPE"),
    "READER_SEL_RESPONSE": SimStruct(OrderedDict((("cbReaderNameOffset", SimTypeInt(signed=False, label="UInt32")), ("cchReaderNameLength", SimTypeInt(signed=False, label="UInt32")), ("cbCardNameOffset", SimTypeInt(signed=False, label="UInt32")), ("cchCardNameLength", SimTypeInt(signed=False, label="UInt32")),)), name="READER_SEL_RESPONSE", pack=False, align=None),
    "READONLYSTATES": SimTypeInt(signed=False, label="READONLYSTATES"),
    "READYSTATE": SimTypeInt(signed=False, label="READYSTATE"),
    "READ_AHEAD_PARAMETERS": SimStruct(OrderedDict((("NodeByteSize", SimTypeShort(signed=True, label="Int16")), ("Granularity", SimTypeInt(signed=False, label="UInt32")), ("PipelinedRequestSize", SimTypeInt(signed=False, label="UInt32")), ("ReadAheadGrowthPercentage", SimTypeInt(signed=False, label="UInt32")),)), name="READ_AHEAD_PARAMETERS", pack=False, align=None),
    "READ_DIRECTORY_NOTIFY_INFORMATION_CLASS": SimTypeInt(signed=False, label="READ_DIRECTORY_NOTIFY_INFORMATION_CLASS"),
    "READ_ELEMENT_ADDRESS_INFO": SimStruct(OrderedDict((("NumberOfElements", SimTypeInt(signed=False, label="UInt32")), ("ElementStatus", SimTypePointer(SimTypeRef("CHANGER_ELEMENT_STATUS", SimStruct), offset=0)),)), name="READ_ELEMENT_ADDRESS_INFO", pack=False, align=None),
    "READ_EVENT_LOG_READ_FLAGS": SimTypeInt(signed=False, label="READ_EVENT_LOG_READ_FLAGS"),
    "READ_FILE_USN_DATA": SimStruct(OrderedDict((("MinMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="READ_FILE_USN_DATA", pack=False, align=None),
    "READ_LIST": SimStruct(OrderedDict((("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("NumberOfEntries", SimTypeInt(signed=False, label="UInt32")), ("IsImage", SimTypeInt(signed=False, label="UInt32")), ("List", SimTypePointer(SimUnion({"Buffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Alignment": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), offset=0)),)), name="READ_LIST", pack=False, align=None),
    "READ_USN_JOURNAL_DATA": SimStruct(OrderedDict((("StartUsn", SimTypeLongLong(signed=True, label="Int64")), ("ReasonMask", SimTypeInt(signed=False, label="UInt32")), ("ReturnOnlyOnClose", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeLongLong(signed=False, label="UInt64")), ("BytesToWaitFor", SimTypeLongLong(signed=False, label="UInt64")), ("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("MinMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="READ_USN_JOURNAL_DATA", pack=False, align=None),
    "READ_USN_JOURNAL_DATA_V0": SimStruct(OrderedDict((("StartUsn", SimTypeLongLong(signed=True, label="Int64")), ("ReasonMask", SimTypeInt(signed=False, label="UInt32")), ("ReturnOnlyOnClose", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeLongLong(signed=False, label="UInt64")), ("BytesToWaitFor", SimTypeLongLong(signed=False, label="UInt64")), ("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")),)), name="READ_USN_JOURNAL_DATA_V0", pack=False, align=None),
    "READ_USN_JOURNAL_DATA_V1": SimStruct(OrderedDict((("StartUsn", SimTypeLongLong(signed=True, label="Int64")), ("ReasonMask", SimTypeInt(signed=False, label="UInt32")), ("ReturnOnlyOnClose", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeLongLong(signed=False, label="UInt64")), ("BytesToWaitFor", SimTypeLongLong(signed=False, label="UInt64")), ("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("MinMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="READ_USN_JOURNAL_DATA_V1", pack=False, align=None),
    "READ_WRITE_MSR": SimStruct(OrderedDict((("Msr", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeLongLong(signed=True, label="Int64")),)), name="READ_WRITE_MSR", pack=False, align=None),
    "REAL_TIME_DATA_SOURCE_ID_FLAGS": SimTypeInt(signed=False, label="REAL_TIME_DATA_SOURCE_ID_FLAGS"),
    "REAL_TIME_NOTIFICATION_SETTING_INPUT": SimStruct(OrderedDict((("TransportSettingId", SimTypeRef("TRANSPORT_SETTING_ID", SimStruct)), ("BrokerEventGuid", SimTypeBottom(label="Guid")),)), name="REAL_TIME_NOTIFICATION_SETTING_INPUT", pack=False, align=None),
    "REAL_TIME_NOTIFICATION_SETTING_INPUT_EX": SimStruct(OrderedDict((("TransportSettingId", SimTypeRef("TRANSPORT_SETTING_ID", SimStruct)), ("BrokerEventGuid", SimTypeBottom(label="Guid")), ("Unmark", SimTypeChar(label="Byte")),)), name="REAL_TIME_NOTIFICATION_SETTING_INPUT_EX", pack=False, align=None),
    "REAL_TIME_NOTIFICATION_SETTING_OUTPUT": SimStruct(OrderedDict((("ChannelStatus", SimTypeInt(signed=False, label="CONTROL_CHANNEL_TRIGGER_STATUS")),)), name="REAL_TIME_NOTIFICATION_SETTING_OUTPUT", pack=False, align=None),
    "REARRANGE_FILE_DATA": SimStruct(OrderedDict((("SourceStartingOffset", SimTypeLongLong(signed=False, label="UInt64")), ("TargetOffset", SimTypeLongLong(signed=False, label="UInt64")), ("SourceFileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="REARRANGE_FILE_DATA", pack=False, align=None),
    "REARRANGE_FILE_DATA32": SimStruct(OrderedDict((("SourceStartingOffset", SimTypeLongLong(signed=False, label="UInt64")), ("TargetOffset", SimTypeLongLong(signed=False, label="UInt64")), ("SourceFileHandle", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="REARRANGE_FILE_DATA32", pack=False, align=None),
    "REASON_CONTEXT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="POWER_REQUEST_CONTEXT_FLAGS")), ("Reason", SimUnion({"Detailed": SimTypeRef("_Detailed_e__Struct", SimStruct), "SimpleReasonString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="REASON_CONTEXT", pack=False, align=None),
    "REASSIGN_BLOCKS": SimStruct(OrderedDict((("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeShort(signed=False, label="UInt16")), ("BlockNumber", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="REASSIGN_BLOCKS", pack=False, align=None),
    "REASSIGN_BLOCKS_EX": SimStruct(OrderedDict((("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeShort(signed=False, label="UInt16")), ("BlockNumber", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="REASSIGN_BLOCKS_EX", pack=False, align=None),
    "REBARBANDINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("fStyle", SimTypeInt(signed=False, label="UInt32")), ("clrFore", SimTypeInt(signed=False, label="UInt32")), ("clrBack", SimTypeInt(signed=False, label="UInt32")), ("lpText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cch", SimTypeInt(signed=False, label="UInt32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("hwndChild", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cxMinChild", SimTypeInt(signed=False, label="UInt32")), ("cyMinChild", SimTypeInt(signed=False, label="UInt32")), ("cx", SimTypeInt(signed=False, label="UInt32")), ("hbmBack", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wID", SimTypeInt(signed=False, label="UInt32")), ("cyChild", SimTypeInt(signed=False, label="UInt32")), ("cyMaxChild", SimTypeInt(signed=False, label="UInt32")), ("cyIntegral", SimTypeInt(signed=False, label="UInt32")), ("cxIdeal", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cxHeader", SimTypeInt(signed=False, label="UInt32")), ("rcChevronLocation", SimTypeRef("RECT", SimStruct)), ("uChevronState", SimTypeInt(signed=False, label="UInt32")),)), name="REBARBANDINFOA", pack=False, align=None),
    "REBARBANDINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("fStyle", SimTypeInt(signed=False, label="UInt32")), ("clrFore", SimTypeInt(signed=False, label="UInt32")), ("clrBack", SimTypeInt(signed=False, label="UInt32")), ("lpText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cch", SimTypeInt(signed=False, label="UInt32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("hwndChild", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cxMinChild", SimTypeInt(signed=False, label="UInt32")), ("cyMinChild", SimTypeInt(signed=False, label="UInt32")), ("cx", SimTypeInt(signed=False, label="UInt32")), ("hbmBack", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wID", SimTypeInt(signed=False, label="UInt32")), ("cyChild", SimTypeInt(signed=False, label="UInt32")), ("cyMaxChild", SimTypeInt(signed=False, label="UInt32")), ("cyIntegral", SimTypeInt(signed=False, label="UInt32")), ("cxIdeal", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cxHeader", SimTypeInt(signed=False, label="UInt32")), ("rcChevronLocation", SimTypeRef("RECT", SimStruct)), ("uChevronState", SimTypeInt(signed=False, label="UInt32")),)), name="REBARBANDINFOW", pack=False, align=None),
    "REBARINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("himl", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="REBARINFO", pack=False, align=None),
    "REBARPARTS": SimTypeInt(signed=False, label="REBARPARTS"),
    "RECIPIENTPOLICY": SimStruct(OrderedDict((("recipient", SimTypeRef("ENDPOINTADDRESS", SimStruct)), ("issuer", SimTypeRef("ENDPOINTADDRESS", SimStruct)), ("tokenType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("requiredClaims", SimTypeRef("CLAIMLIST", SimStruct)), ("optionalClaims", SimTypeRef("CLAIMLIST", SimStruct)), ("privacyUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("privacyVersion", SimTypeInt(signed=False, label="UInt32")),)), name="RECIPIENTPOLICY", pack=False, align=None),
    "RECIPIENTPOLICY2": SimStruct(OrderedDict((("recipient", SimTypeRef("ENDPOINTADDRESS2", SimStruct)), ("issuer", SimTypeRef("ENDPOINTADDRESS2", SimStruct)), ("tokenType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("requiredClaims", SimTypeRef("CLAIMLIST", SimStruct)), ("optionalClaims", SimTypeRef("CLAIMLIST", SimStruct)), ("privacyUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("privacyVersion", SimTypeInt(signed=False, label="UInt32")),)), name="RECIPIENTPOLICY2", pack=False, align=None),
    "RECONCILEF": SimTypeInt(signed=False, label="RECONCILEF"),
    "RECONVERTSTRING": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCompStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwCompStrOffset", SimTypeInt(signed=False, label="UInt32")), ("dwTargetStrLen", SimTypeInt(signed=False, label="UInt32")), ("dwTargetStrOffset", SimTypeInt(signed=False, label="UInt32")),)), name="RECONVERTSTRING", pack=False, align=None),
    "RECORDER_TYPES": SimTypeInt(signed=False, label="RECORDER_TYPES"),
    "RECORDING_TYPE": SimTypeInt(signed=False, label="RECORDING_TYPE"),
    "RECO_ATTRS": SimStruct(OrderedDict((("dwRecoCapabilityFlags", SimTypeInt(signed=False, label="UInt32")), ("awcVendorName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("awcFriendlyName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("awLanguageId", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 64)),)), name="RECO_ATTRS", pack=False, align=None),
    "RECO_FLAGS": SimTypeInt(signed=False, label="RECO_FLAGS"),
    "RECO_GUIDE": SimStruct(OrderedDict((("xOrigin", SimTypeInt(signed=True, label="Int32")), ("yOrigin", SimTypeInt(signed=True, label="Int32")), ("cxBox", SimTypeInt(signed=True, label="Int32")), ("cyBox", SimTypeInt(signed=True, label="Int32")), ("cxBase", SimTypeInt(signed=True, label="Int32")), ("cyBase", SimTypeInt(signed=True, label="Int32")), ("cHorzBox", SimTypeInt(signed=True, label="Int32")), ("cVertBox", SimTypeInt(signed=True, label="Int32")), ("cyMid", SimTypeInt(signed=True, label="Int32")),)), name="RECO_GUIDE", pack=False, align=None),
    "RECO_LATTICE": SimStruct(OrderedDict((("ulColumnCount", SimTypeInt(signed=False, label="UInt32")), ("pLatticeColumns", SimTypePointer(SimTypeRef("RECO_LATTICE_COLUMN", SimStruct), offset=0)), ("ulPropertyCount", SimTypeInt(signed=False, label="UInt32")), ("pGuidProperties", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ulBestResultColumnCount", SimTypeInt(signed=False, label="UInt32")), ("pulBestResultColumns", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pulBestResultIndexes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RECO_LATTICE", pack=False, align=None),
    "RECO_LATTICE_COLUMN": SimStruct(OrderedDict((("key", SimTypeInt(signed=False, label="UInt32")), ("cpProp", SimTypeRef("RECO_LATTICE_PROPERTIES", SimStruct)), ("cStrokes", SimTypeInt(signed=False, label="UInt32")), ("pStrokes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cLatticeElements", SimTypeInt(signed=False, label="UInt32")), ("pLatticeElements", SimTypePointer(SimTypeRef("RECO_LATTICE_ELEMENT", SimStruct), offset=0)),)), name="RECO_LATTICE_COLUMN", pack=False, align=None),
    "RECO_LATTICE_ELEMENT": SimStruct(OrderedDict((("score", SimTypeInt(signed=True, label="Int32")), ("type", SimTypeShort(signed=False, label="UInt16")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulNextColumn", SimTypeInt(signed=False, label="UInt32")), ("ulStrokeNumber", SimTypeInt(signed=False, label="UInt32")), ("epProp", SimTypeRef("RECO_LATTICE_PROPERTIES", SimStruct)),)), name="RECO_LATTICE_ELEMENT", pack=False, align=None),
    "RECO_LATTICE_PROPERTIES": SimStruct(OrderedDict((("cProperties", SimTypeInt(signed=False, label="UInt32")), ("apProps", SimTypePointer(SimTypePointer(SimTypeRef("RECO_LATTICE_PROPERTY", SimStruct), offset=0), offset=0)),)), name="RECO_LATTICE_PROPERTIES", pack=False, align=None),
    "RECO_LATTICE_PROPERTY": SimStruct(OrderedDict((("guidProperty", SimTypeBottom(label="Guid")), ("cbPropertyValue", SimTypeShort(signed=False, label="UInt16")), ("pPropertyValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RECO_LATTICE_PROPERTY", pack=False, align=None),
    "RECO_RANGE": SimStruct(OrderedDict((("iwcBegin", SimTypeInt(signed=False, label="UInt32")), ("cCount", SimTypeInt(signed=False, label="UInt32")),)), name="RECO_RANGE", pack=False, align=None),
    "RECO_TYPE": SimTypeInt(signed=False, label="RECO_TYPE"),
    "RECT": SimStruct(OrderedDict((("left", SimTypeInt(signed=True, label="Int32")), ("top", SimTypeInt(signed=True, label="Int32")), ("right", SimTypeInt(signed=True, label="Int32")), ("bottom", SimTypeInt(signed=True, label="Int32")),)), name="RECT", pack=False, align=None),
    "RECTFX": SimStruct(OrderedDict((("xLeft", SimTypeInt(signed=True, label="Int32")), ("yTop", SimTypeInt(signed=True, label="Int32")), ("xRight", SimTypeInt(signed=True, label="Int32")), ("yBottom", SimTypeInt(signed=True, label="Int32")),)), name="RECTFX", pack=False, align=None),
    "RECTL": SimStruct(OrderedDict((("left", SimTypeInt(signed=True, label="Int32")), ("top", SimTypeInt(signed=True, label="Int32")), ("right", SimTypeInt(signed=True, label="Int32")), ("bottom", SimTypeInt(signed=True, label="Int32")),)), name="RECTL", pack=False, align=None),
    "RECYCLE_INFO": SimStruct(OrderedDict((("guidCombaseProcessIdentifier", SimTypeBottom(label="Guid")), ("ProcessStartTime", SimTypeLongLong(signed=True, label="Int64")), ("dwRecycleLifetimeLimit", SimTypeInt(signed=False, label="UInt32")), ("dwRecycleMemoryLimit", SimTypeInt(signed=False, label="UInt32")), ("dwRecycleExpirationTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="RECYCLE_INFO", pack=False, align=None),
    "REDBOOK_DIGITAL_AUDIO_EXTRACTION_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Accurate", SimTypeInt(signed=False, label="UInt32")), ("Supported", SimTypeInt(signed=False, label="UInt32")), ("AccurateMask0", SimTypeInt(signed=False, label="UInt32")),)), name="REDBOOK_DIGITAL_AUDIO_EXTRACTION_INFO", pack=False, align=None),
    "REDIRECTION_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("FunctionCount", SimTypeInt(signed=False, label="UInt32")), ("Redirections", SimTypePointer(SimTypeRef("REDIRECTION_FUNCTION_DESCRIPTOR", SimStruct), offset=0)),)), name="REDIRECTION_DESCRIPTOR", pack=False, align=None),
    "REDIRECTION_FUNCTION_DESCRIPTOR": SimStruct(OrderedDict((("DllName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("FunctionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("RedirectionTarget", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REDIRECTION_FUNCTION_DESCRIPTOR", pack=False, align=None),
    "REDRAW_WINDOW_FLAGS": SimTypeInt(signed=False, label="REDRAW_WINDOW_FLAGS"),
    "REENUMERATE_SELF_INTERFACE_STANDARD": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("SurpriseRemoveAndReenumerateSelf", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)),)), name="REENUMERATE_SELF_INTERFACE_STANDARD", pack=False, align=None),
    "REFERENCE": SimStruct(OrderedDict((("SpinLock", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ReferenceCount", SimTypeShort(signed=False, label="UInt16")), ("Closing", SimTypeChar(label="Byte")),)), name="REFERENCE", pack=False, align=None),
    "REFS_COMPRESSION_FORMATS": SimTypeInt(signed=False, label="REFS_COMPRESSION_FORMATS"),
    "REFS_DEALLOCATE_RANGES_ALLOCATOR": SimTypeInt(signed=False, label="REFS_DEALLOCATE_RANGES_ALLOCATOR"),
    "REFS_DEALLOCATE_RANGES_INPUT_BUFFER": SimStruct(OrderedDict((("RangeCount", SimTypeInt(signed=False, label="UInt32")), ("Ranges", SimTypePointer(SimTypeRef("REFS_DEALLOCATE_RANGES_RANGE", SimStruct), offset=0)),)), name="REFS_DEALLOCATE_RANGES_INPUT_BUFFER", pack=False, align=None),
    "REFS_DEALLOCATE_RANGES_INPUT_BUFFER_EX": SimStruct(OrderedDict((("RangeCount", SimTypeInt(signed=False, label="UInt32")), ("Allocator", SimTypeInt(signed=False, label="REFS_DEALLOCATE_RANGES_ALLOCATOR")), ("StreamReserveUpdateCount", SimTypeLongLong(signed=True, label="Int64")), ("OffsetToRanges", SimTypeInt(signed=False, label="UInt32")), ("OffsetToLeakCounts", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="REFS_DEALLOCATE_RANGES_INPUT_BUFFER_EX", pack=False, align=None),
    "REFS_DEALLOCATE_RANGES_RANGE": SimStruct(OrderedDict((("StartOfRange", SimTypeLongLong(signed=False, label="UInt64")), ("CountOfRange", SimTypeLongLong(signed=False, label="UInt64")),)), name="REFS_DEALLOCATE_RANGES_RANGE", pack=False, align=None),
    "REFS_QUERY_VOLUME_COMPRESSION_INFO_OUTPUT_BUFFER": SimStruct(OrderedDict((("DefaultCompressionFormat", SimTypeInt(signed=False, label="REFS_COMPRESSION_FORMATS")), ("DefaultCompressionLevel", SimTypeShort(signed=True, label="Int16")), ("DefaultCompressionChunkSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("VolumeClusterSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("TotalVolumeClusters", SimTypeLongLong(signed=False, label="UInt64")), ("TotalAllocatedClusters", SimTypeLongLong(signed=False, label="UInt64")), ("TotalCompressibleClustersAllocated", SimTypeLongLong(signed=False, label="UInt64")), ("TotalCompressibleClustersInUse", SimTypeLongLong(signed=False, label="UInt64")), ("TotalCompressedClusters", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 6)),)), name="REFS_QUERY_VOLUME_COMPRESSION_INFO_OUTPUT_BUFFER", pack=False, align=None),
    "REFS_QUERY_VOLUME_DEDUP_INFO_OUTPUT_BUFFER": SimStruct(OrderedDict((("Enabled", SimTypeChar(label="Byte")),)), name="REFS_QUERY_VOLUME_DEDUP_INFO_OUTPUT_BUFFER", pack=False, align=None),
    "REFS_REMOVE_HARDLINK_BACKPOINTER": SimStruct(OrderedDict((("ParentDirectory", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REFS_REMOVE_HARDLINK_BACKPOINTER", pack=False, align=None),
    "REFS_SET_VOLUME_COMPRESSION_INFO_FLAGS": SimTypeInt(signed=False, label="REFS_SET_VOLUME_COMPRESSION_INFO_FLAGS"),
    "REFS_SET_VOLUME_COMPRESSION_INFO_INPUT_BUFFER": SimStruct(OrderedDict((("CompressionFormat", SimTypeInt(signed=False, label="REFS_COMPRESSION_FORMATS")), ("CompressionLevel", SimTypeShort(signed=True, label="Int16")), ("CompressionChunkSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="REFS_SET_VOLUME_COMPRESSION_INFO_FLAGS")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="REFS_SET_VOLUME_COMPRESSION_INFO_INPUT_BUFFER", pack=False, align=None),
    "REFS_SET_VOLUME_DEDUP_INFO_INPUT_BUFFER": SimStruct(OrderedDict((("Enable", SimTypeChar(label="Byte")),)), name="REFS_SET_VOLUME_DEDUP_INFO_INPUT_BUFFER", pack=False, align=None),
    "REFS_SMR_VOLUME_GC_ACTION": SimTypeInt(signed=False, label="REFS_SMR_VOLUME_GC_ACTION"),
    "REFS_SMR_VOLUME_GC_METHOD": SimTypeInt(signed=False, label="REFS_SMR_VOLUME_GC_METHOD"),
    "REFS_SMR_VOLUME_GC_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="REFS_SMR_VOLUME_GC_ACTION")), ("Method", SimTypeInt(signed=False, label="REFS_SMR_VOLUME_GC_METHOD")), ("IoGranularity", SimTypeInt(signed=False, label="UInt32")), ("CompressionFormat", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="REFS_SMR_VOLUME_GC_PARAMETERS", pack=False, align=None),
    "REFS_SMR_VOLUME_GC_STATE": SimTypeInt(signed=False, label="REFS_SMR_VOLUME_GC_STATE"),
    "REFS_SMR_VOLUME_INFO_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SizeOfRandomlyWritableTier", SimTypeLongLong(signed=True, label="Int64")), ("FreeSpaceInRandomlyWritableTier", SimTypeLongLong(signed=True, label="Int64")), ("SizeofSMRTier", SimTypeLongLong(signed=True, label="Int64")), ("FreeSpaceInSMRTier", SimTypeLongLong(signed=True, label="Int64")), ("UsableFreeSpaceInSMRTier", SimTypeLongLong(signed=True, label="Int64")), ("VolumeGcState", SimTypeInt(signed=False, label="REFS_SMR_VOLUME_GC_STATE")), ("VolumeGcLastStatus", SimTypeInt(signed=False, label="UInt32")), ("CurrentGcBandFillPercentage", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 6)),)), name="REFS_SMR_VOLUME_INFO_OUTPUT", pack=False, align=None),
    "REFS_STREAM_EXTENT": SimStruct(OrderedDict((("Vcn", SimTypeLongLong(signed=True, label="Int64")), ("Lcn", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("Properties", SimTypeShort(signed=False, label="UInt16")),)), name="REFS_STREAM_EXTENT", pack=False, align=None),
    "REFS_STREAM_SNAPSHOT_LIST_OUTPUT_BUFFER": SimStruct(OrderedDict((("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("BufferSizeRequiredForQuery", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("Entries", SimTypePointer(SimTypeRef("REFS_STREAM_SNAPSHOT_LIST_OUTPUT_BUFFER_ENTRY", SimStruct), offset=0)),)), name="REFS_STREAM_SNAPSHOT_LIST_OUTPUT_BUFFER", pack=False, align=None),
    "REFS_STREAM_SNAPSHOT_LIST_OUTPUT_BUFFER_ENTRY": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("SnapshotNameLength", SimTypeShort(signed=False, label="UInt16")), ("SnapshotCreationTime", SimTypeLongLong(signed=False, label="UInt64")), ("StreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("StreamAllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)), ("SnapshotName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REFS_STREAM_SNAPSHOT_LIST_OUTPUT_BUFFER_ENTRY", pack=False, align=None),
    "REFS_STREAM_SNAPSHOT_MANAGEMENT_INPUT_BUFFER": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="REFS_STREAM_SNAPSHOT_OPERATION")), ("SnapshotNameLength", SimTypeShort(signed=False, label="UInt16")), ("OperationInputBufferLength", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)), ("NameAndInputBuffer", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="REFS_STREAM_SNAPSHOT_MANAGEMENT_INPUT_BUFFER", pack=False, align=None),
    "REFS_STREAM_SNAPSHOT_OPERATION": SimTypeInt(signed=False, label="REFS_STREAM_SNAPSHOT_OPERATION"),
    "REFS_STREAM_SNAPSHOT_QUERY_DELTAS_INPUT_BUFFER": SimStruct(OrderedDict((("StartingVcn", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="REFS_STREAM_SNAPSHOT_QUERY_DELTAS_INPUT_BUFFER", pack=False, align=None),
    "REFS_STREAM_SNAPSHOT_QUERY_DELTAS_OUTPUT_BUFFER": SimStruct(OrderedDict((("ExtentCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("Extents", SimTypePointer(SimTypeRef("REFS_STREAM_EXTENT", SimStruct), offset=0)),)), name="REFS_STREAM_SNAPSHOT_QUERY_DELTAS_OUTPUT_BUFFER", pack=False, align=None),
    "REFS_VOLUME_COUNTER_INFO_INPUT_BUFFER": SimStruct(OrderedDict((("ResetCounters", SimTypeChar(label="Byte")),)), name="REFS_VOLUME_COUNTER_INFO_INPUT_BUFFER", pack=False, align=None),
    "REFS_VOLUME_DATA_BUFFER": SimStruct(OrderedDict((("ByteCount", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("BytesPerPhysicalSector", SimTypeInt(signed=False, label="UInt32")), ("VolumeSerialNumber", SimTypeLongLong(signed=True, label="Int64")), ("NumberSectors", SimTypeLongLong(signed=True, label="Int64")), ("TotalClusters", SimTypeLongLong(signed=True, label="Int64")), ("FreeClusters", SimTypeLongLong(signed=True, label="Int64")), ("TotalReserved", SimTypeLongLong(signed=True, label="Int64")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("BytesPerCluster", SimTypeInt(signed=False, label="UInt32")), ("MaximumSizeOfResidentFile", SimTypeLongLong(signed=True, label="Int64")), ("FastTierDataFillRatio", SimTypeShort(signed=False, label="UInt16")), ("SlowTierDataFillRatio", SimTypeShort(signed=False, label="UInt16")), ("DestagesFastTierToSlowTierRate", SimTypeInt(signed=False, label="UInt32")), ("MetadataChecksumType", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Reserved", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 8)),)), name="REFS_VOLUME_DATA_BUFFER", pack=False, align=None),
    "REGCLS": SimTypeInt(signed=False, label="REGCLS"),
    "REGFILTER": SimStruct(OrderedDict((("Clsid", SimTypeBottom(label="Guid")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REGFILTER", pack=False, align=None),
    "REGFILTER2": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMerit", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous1": SimStruct(OrderedDict((("cPins", SimTypeInt(signed=False, label="UInt32")), ("rgPins", SimTypePointer(SimTypeRef("REGFILTERPINS", SimStruct), offset=0)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("cPins2", SimTypeInt(signed=False, label="UInt32")), ("rgPins2", SimTypePointer(SimTypeRef("REGFILTERPINS2", SimStruct), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="REGFILTER2", pack=False, align=None),
    "REGFILTERPINS": SimStruct(OrderedDict((("strName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bRendered", SimTypeInt(signed=True, label="Int32")), ("bOutput", SimTypeInt(signed=True, label="Int32")), ("bZero", SimTypeInt(signed=True, label="Int32")), ("bMany", SimTypeInt(signed=True, label="Int32")), ("clsConnectsToFilter", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("strConnectsToPin", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nMediaTypes", SimTypeInt(signed=False, label="UInt32")), ("lpMediaType", SimTypePointer(SimTypeRef("REGPINTYPES", SimStruct), offset=0)),)), name="REGFILTERPINS", pack=False, align=None),
    "REGFILTERPINS2": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cInstances", SimTypeInt(signed=False, label="UInt32")), ("nMediaTypes", SimTypeInt(signed=False, label="UInt32")), ("lpMediaType", SimTypePointer(SimTypeRef("REGPINTYPES", SimStruct), offset=0)), ("nMediums", SimTypeInt(signed=False, label="UInt32")), ("lpMedium", SimTypePointer(SimTypeRef("REGPINMEDIUM", SimStruct), offset=0)), ("clsPinCategory", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="REGFILTERPINS2", pack=False, align=None),
    "REGINSTALLA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("STRTABLEA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hm", "pszSection", "pstTable"]), offset=0),
    "REGISTERWORDA": SimStruct(OrderedDict((("lpReading", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpWord", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="REGISTERWORDA", pack=False, align=None),
    "REGISTERWORDENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszReading", "param1", "lpszString", "param3"]), offset=0),
    "REGISTERWORDENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszReading", "param1", "lpszString", "param3"]), offset=0),
    "REGISTERWORDW": SimStruct(OrderedDict((("lpReading", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpWord", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REGISTERWORDW", pack=False, align=None),
    "REGISTER_APPLICATION_RESTART_FLAGS": SimTypeInt(signed=False, label="REGISTER_APPLICATION_RESTART_FLAGS"),
    "REGISTER_ID": SimTypeInt(signed=False, label="REGISTER_ID"),
    "REGISTER_NOTIFICATION_FLAGS": SimTypeInt(signed=False, label="REGISTER_NOTIFICATION_FLAGS"),
    "REGISTER_TOUCH_WINDOW_FLAGS": SimTypeInt(signed=False, label="REGISTER_TOUCH_WINDOW_FLAGS"),
    "REGISTRATION_INFORMATION_CLASS": SimTypeInt(signed=False, label="REGISTRATION_INFORMATION_CLASS"),
    "REGKIND": SimTypeInt(signed=False, label="REGKIND"),
    "REGPINMEDIUM": SimStruct(OrderedDict((("clsMedium", SimTypeBottom(label="Guid")), ("dw1", SimTypeInt(signed=False, label="UInt32")), ("dw2", SimTypeInt(signed=False, label="UInt32")),)), name="REGPINMEDIUM", pack=False, align=None),
    "REGPINTYPES": SimStruct(OrderedDict((("clsMajorType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("clsMinorType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="REGPINTYPES", pack=False, align=None),
    "REG_CALLBACK_CONTEXT_CLEANUP_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_CALLBACK_CONTEXT_CLEANUP_INFORMATION", pack=False, align=None),
    "REG_CREATE_KEY_DISPOSITION": SimTypeInt(signed=False, label="REG_CREATE_KEY_DISPOSITION"),
    "REG_CREATE_KEY_INFORMATION": SimStruct(OrderedDict((("CompleteName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("RootObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectType", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CreateOptions", SimTypeInt(signed=False, label="UInt32")), ("Class", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityQualityOfService", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("Disposition", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ResultObject", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RootObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Transaction", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_CREATE_KEY_INFORMATION", pack=False, align=None),
    "REG_CREATE_KEY_INFORMATION_V1": SimStruct(OrderedDict((("CompleteName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("RootObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectType", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Options", SimTypeInt(signed=False, label="UInt32")), ("Class", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityQualityOfService", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("Disposition", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ResultObject", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RootObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Transaction", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Version", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("RemainingName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("Wow64Flags", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("CheckAccessMode", SimTypeChar(label="SByte")),)), name="REG_CREATE_KEY_INFORMATION_V1", pack=False, align=None),
    "REG_DELETE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_DELETE_KEY_INFORMATION", pack=False, align=None),
    "REG_DELETE_VALUE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValueName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_DELETE_VALUE_KEY_INFORMATION", pack=False, align=None),
    "REG_ENUMERATE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Index", SimTypeInt(signed=False, label="UInt32")), ("KeyInformationClass", SimTypeInt(signed=False, label="KEY_INFORMATION_CLASS")), ("KeyInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ResultLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_ENUMERATE_KEY_INFORMATION", pack=False, align=None),
    "REG_ENUMERATE_VALUE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Index", SimTypeInt(signed=False, label="UInt32")), ("KeyValueInformationClass", SimTypeInt(signed=False, label="KEY_VALUE_INFORMATION_CLASS")), ("KeyValueInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ResultLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_ENUMERATE_VALUE_KEY_INFORMATION", pack=False, align=None),
    "REG_KEY_HANDLE_CLOSE_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_KEY_HANDLE_CLOSE_INFORMATION", pack=False, align=None),
    "REG_LOAD_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeyName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("SourceFile", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TrustClassObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UserEvent", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("RootHandle", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_LOAD_KEY_INFORMATION", pack=False, align=None),
    "REG_LOAD_KEY_INFORMATION_V2": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeyName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("SourceFile", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TrustClassObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UserEvent", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("RootHandle", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Version", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("FileAccessToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_LOAD_KEY_INFORMATION_V2", pack=False, align=None),
    "REG_NOTIFY_CLASS": SimTypeInt(signed=False, label="REG_NOTIFY_CLASS"),
    "REG_NOTIFY_FILTER": SimTypeInt(signed=False, label="REG_NOTIFY_FILTER"),
    "REG_OPEN_CREATE_OPTIONS": SimTypeInt(signed=False, label="REG_OPEN_CREATE_OPTIONS"),
    "REG_PINFLAG": SimTypeInt(signed=False, label="REG_PINFLAG"),
    "REG_POST_CREATE_KEY_INFORMATION": SimStruct(OrderedDict((("CompleteName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="REG_POST_CREATE_KEY_INFORMATION", pack=False, align=None),
    "REG_POST_OPERATION_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Status", SimTypeInt(signed=True, label="Int32")), ("PreInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ReturnStatus", SimTypeInt(signed=True, label="Int32")), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_POST_OPERATION_INFORMATION", pack=False, align=None),
    "REG_PRE_CREATE_KEY_INFORMATION": SimStruct(OrderedDict((("CompleteName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)),)), name="REG_PRE_CREATE_KEY_INFORMATION", pack=False, align=None),
    "REG_PROVIDER": SimStruct(OrderedDict((("pi_R0_1val", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("val_context", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["keycontext", "val_list", "num_vals", "outputbuffer", "total_outlen", "input_blen"]), offset=0)), ("pi_R0_allvals", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("val_context", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["keycontext", "val_list", "num_vals", "outputbuffer", "total_outlen", "input_blen"]), offset=0)), ("pi_R3_1val", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("val_context", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["keycontext", "val_list", "num_vals", "outputbuffer", "total_outlen", "input_blen"]), offset=0)), ("pi_R3_allvals", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("val_context", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["keycontext", "val_list", "num_vals", "outputbuffer", "total_outlen", "input_blen"]), offset=0)), ("pi_flags", SimTypeInt(signed=False, label="UInt32")), ("pi_key_context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_PROVIDER", pack=False, align=None),
    "REG_QUERY_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeyInformationClass", SimTypeInt(signed=False, label="KEY_INFORMATION_CLASS")), ("KeyInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ResultLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_QUERY_KEY_INFORMATION", pack=False, align=None),
    "REG_QUERY_KEY_NAME": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectNameInfo", SimTypePointer(SimTypeRef("OBJECT_NAME_INFORMATION", SimStruct), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ReturnLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_QUERY_KEY_NAME", pack=False, align=None),
    "REG_QUERY_KEY_SECURITY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityInformation", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_QUERY_KEY_SECURITY_INFORMATION", pack=False, align=None),
    "REG_QUERY_MULTIPLE_VALUE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValueEntries", SimTypePointer(SimTypeRef("KEY_VALUE_ENTRY", SimStruct), offset=0)), ("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("ValueBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("RequiredBufferLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_QUERY_MULTIPLE_VALUE_KEY_INFORMATION", pack=False, align=None),
    "REG_QUERY_VALUE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValueName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("KeyValueInformationClass", SimTypeInt(signed=False, label="KEY_VALUE_INFORMATION_CLASS")), ("KeyValueInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ResultLength", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_QUERY_VALUE_KEY_INFORMATION", pack=False, align=None),
    "REG_RENAME_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NewName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_RENAME_KEY_INFORMATION", pack=False, align=None),
    "REG_REPLACE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OldFileName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("NewFileName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_REPLACE_KEY_INFORMATION", pack=False, align=None),
    "REG_RESTORE_KEY_FLAGS": SimTypeInt(signed=False, label="REG_RESTORE_KEY_FLAGS"),
    "REG_RESTORE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_RESTORE_KEY_INFORMATION", pack=False, align=None),
    "REG_ROUTINE_FLAGS": SimTypeInt(signed=False, label="REG_ROUTINE_FLAGS"),
    "REG_SAM_FLAGS": SimTypeInt(signed=False, label="REG_SAM_FLAGS"),
    "REG_SAVE_FORMAT": SimTypeInt(signed=False, label="REG_SAVE_FORMAT"),
    "REG_SAVE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Format", SimTypeInt(signed=False, label="UInt32")), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_SAVE_KEY_INFORMATION", pack=False, align=None),
    "REG_SAVE_MERGED_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("HighKeyObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LowKeyObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_SAVE_MERGED_KEY_INFORMATION", pack=False, align=None),
    "REG_SET_INFORMATION_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeySetInformationClass", SimTypeInt(signed=False, label="KEY_SET_INFORMATION_CLASS")), ("KeySetInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("KeySetInformationLength", SimTypeInt(signed=False, label="UInt32")), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_SET_INFORMATION_KEY_INFORMATION", pack=False, align=None),
    "REG_SET_KEY_SECURITY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityInformation", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_SET_KEY_SECURITY_INFORMATION", pack=False, align=None),
    "REG_SET_VALUE_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ValueName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("TitleIndex", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_SET_VALUE_KEY_INFORMATION", pack=False, align=None),
    "REG_UNLOAD_KEY_INFORMATION": SimStruct(OrderedDict((("Object", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UserEvent", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CallContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ObjectContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="REG_UNLOAD_KEY_INFORMATION", pack=False, align=None),
    "REG_VALUE_TYPE": SimTypeInt(signed=False, label="REG_VALUE_TYPE"),
    "REINSTALLMODE": SimTypeInt(signed=False, label="REINSTALLMODE"),
    "RELOGSTREAM_HANDLE": SimStruct(OrderedDict((("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="RELOGSTREAM_HANDLE", pack=False, align=None),
    "RELOPS": SimTypeInt(signed=False, label="RELOPS"),
    "REMEDIATIONSERVERGROUPPROPERTIES": SimTypeInt(signed=False, label="REMEDIATIONSERVERGROUPPROPERTIES"),
    "REMEDIATIONSERVERPROPERTIES": SimTypeInt(signed=False, label="REMEDIATIONSERVERPROPERTIES"),
    "REMEDIATIONSERVERSPROPERTIES": SimTypeInt(signed=False, label="REMEDIATIONSERVERSPROPERTIES"),
    "REMOTE_LINK_TRACKING_INFORMATION": SimStruct(OrderedDict((("TargetFileObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TargetLinkTrackingInformationLength", SimTypeInt(signed=False, label="UInt32")), ("TargetLinkTrackingInformationBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="REMOTE_LINK_TRACKING_INFORMATION", pack=False, align=None),
    "REMOTE_NAME_INFOA": SimStruct(OrderedDict((("lpUniversalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpConnectionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpRemainingPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="REMOTE_NAME_INFOA", pack=False, align=None),
    "REMOTE_NAME_INFOW": SimStruct(OrderedDict((("lpUniversalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpConnectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpRemainingPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REMOTE_NAME_INFOW", pack=False, align=None),
    "REMOVE_ELEMENT_AND_TRUNCATE_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("RequestCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("ElementIdentifier", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="REMOVE_ELEMENT_AND_TRUNCATE_REQUEST", pack=False, align=None),
    "REMSECURITY_ATTRIBUTES": SimStruct(OrderedDict((("nLength", SimTypeInt(signed=False, label="UInt32")), ("lpSecurityDescriptor", SimTypeInt(signed=False, label="UInt32")), ("bInheritHandle", SimTypeInt(signed=True, label="Int32")),)), name="REMSECURITY_ATTRIBUTES", pack=False, align=None),
    "RENDDATA": SimStruct(OrderedDict((("atyp", SimTypeShort(signed=False, label="UInt16")), ("ulPosition", SimTypeInt(signed=False, label="UInt32")), ("dxWidth", SimTypeShort(signed=False, label="UInt16")), ("dyHeight", SimTypeShort(signed=False, label="UInt16")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="RENDDATA", pack=False, align=None),
    "RENDEZVOUS_SESSION_FLAGS": SimTypeInt(signed=False, label="RENDEZVOUS_SESSION_FLAGS"),
    "RENDEZVOUS_SESSION_STATE": SimTypeInt(signed=False, label="RENDEZVOUS_SESSION_STATE"),
    "REOBJECT": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("cp", SimTypeInt(signed=True, label="Int32")), ("clsid", SimTypeBottom(label="Guid")), ("poleobj", SimTypeBottom(label="IOleObject")), ("pstg", SimTypeBottom(label="IStorage")), ("polesite", SimTypeBottom(label="IOleClientSite")), ("sizel", SimTypeRef("SIZE", SimStruct)), ("dvaspect", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="REOBJECT_FLAGS")), ("dwUser", SimTypeInt(signed=False, label="UInt32")),)), name="REOBJECT", pack=False, align=None),
    "REOBJECT_FLAGS": SimTypeInt(signed=False, label="REOBJECT_FLAGS"),
    "REPAIR_COPIES_INPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("SourceCopy", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRepairCopies", SimTypeInt(signed=False, label="UInt32")), ("RepairCopies", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="REPAIR_COPIES_INPUT", pack=False, align=None),
    "REPAIR_COPIES_OUTPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("ResumeFileOffset", SimTypeLongLong(signed=True, label="Int64")),)), name="REPAIR_COPIES_OUTPUT", pack=False, align=None),
    "REPAIR_RISK": SimTypeInt(signed=False, label="REPAIR_RISK"),
    "REPAIR_SCOPE": SimTypeInt(signed=False, label="REPAIR_SCOPE"),
    "REPAIR_STATUS": SimTypeInt(signed=False, label="REPAIR_STATUS"),
    "REPARSE_DATA_BUFFER": SimStruct(OrderedDict((("ReparseTag", SimTypeInt(signed=False, label="UInt32")), ("ReparseDataLength", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"SymbolicLinkReparseBuffer": SimTypeRef("_SymbolicLinkReparseBuffer_e__Struct", SimStruct), "MountPointReparseBuffer": SimTypeRef("_MountPointReparseBuffer_e__Struct", SimStruct), "GenericReparseBuffer": SimTypeRef("_GenericReparseBuffer_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="REPARSE_DATA_BUFFER", pack=False, align=None),
    "REPARSE_DATA_BUFFER_EX": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ExistingReparseTag", SimTypeInt(signed=False, label="UInt32")), ("ExistingReparseGuid", SimTypeBottom(label="Guid")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"ReparseDataBuffer": SimTypeRef("REPARSE_DATA_BUFFER", SimStruct), "ReparseGuidDataBuffer": SimTypeRef("REPARSE_GUID_DATA_BUFFER", SimStruct)}, name="<anon>", label="None")),)), name="REPARSE_DATA_BUFFER_EX", pack=False, align=None),
    "REPARSE_GUID_DATA_BUFFER": SimStruct(OrderedDict((("ReparseTag", SimTypeInt(signed=False, label="UInt32")), ("ReparseDataLength", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("ReparseGuid", SimTypeBottom(label="Guid")), ("GenericReparseBuffer", SimTypeRef("_GenericReparseBuffer_e__Struct", SimStruct)),)), name="REPARSE_GUID_DATA_BUFFER", pack=False, align=None),
    "REPARSE_INDEX_KEY": SimStruct(OrderedDict((("FileReparseTag", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeLongLong(signed=True, label="Int64")),)), name="REPARSE_INDEX_KEY", pack=False, align=None),
    "REPASTESPECIAL": SimStruct(OrderedDict((("dwAspect", SimTypeInt(signed=False, label="DVASPECT")), ("dwParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="REPASTESPECIAL", pack=False, align=None),
    "REPLACE_FILE_FLAGS": SimTypeInt(signed=False, label="REPLACE_FILE_FLAGS"),
    "REPL_EDIR_INFO_0": SimStruct(OrderedDict((("rped0_dirname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REPL_EDIR_INFO_0", pack=False, align=None),
    "REPL_EDIR_INFO_1": SimStruct(OrderedDict((("rped1_dirname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rped1_integrity", SimTypeInt(signed=False, label="UInt32")), ("rped1_extent", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_EDIR_INFO_1", pack=False, align=None),
    "REPL_EDIR_INFO_1000": SimStruct(OrderedDict((("rped1000_integrity", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_EDIR_INFO_1000", pack=False, align=None),
    "REPL_EDIR_INFO_1001": SimStruct(OrderedDict((("rped1001_extent", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_EDIR_INFO_1001", pack=False, align=None),
    "REPL_EDIR_INFO_2": SimStruct(OrderedDict((("rped2_dirname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rped2_integrity", SimTypeInt(signed=False, label="UInt32")), ("rped2_extent", SimTypeInt(signed=False, label="UInt32")), ("rped2_lockcount", SimTypeInt(signed=False, label="UInt32")), ("rped2_locktime", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_EDIR_INFO_2", pack=False, align=None),
    "REPL_IDIR_INFO_0": SimStruct(OrderedDict((("rpid0_dirname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="REPL_IDIR_INFO_0", pack=False, align=None),
    "REPL_IDIR_INFO_1": SimStruct(OrderedDict((("rpid1_dirname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rpid1_state", SimTypeInt(signed=False, label="UInt32")), ("rpid1_mastername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rpid1_last_update_time", SimTypeInt(signed=False, label="UInt32")), ("rpid1_lockcount", SimTypeInt(signed=False, label="UInt32")), ("rpid1_locktime", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_IDIR_INFO_1", pack=False, align=None),
    "REPL_INFO_0": SimStruct(OrderedDict((("rp0_role", SimTypeInt(signed=False, label="UInt32")), ("rp0_exportpath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rp0_exportlist", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rp0_importpath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rp0_importlist", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rp0_logonusername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rp0_interval", SimTypeInt(signed=False, label="UInt32")), ("rp0_pulse", SimTypeInt(signed=False, label="UInt32")), ("rp0_guardtime", SimTypeInt(signed=False, label="UInt32")), ("rp0_random", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_INFO_0", pack=False, align=None),
    "REPL_INFO_1000": SimStruct(OrderedDict((("rp1000_interval", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_INFO_1000", pack=False, align=None),
    "REPL_INFO_1001": SimStruct(OrderedDict((("rp1001_pulse", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_INFO_1001", pack=False, align=None),
    "REPL_INFO_1002": SimStruct(OrderedDict((("rp1002_guardtime", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_INFO_1002", pack=False, align=None),
    "REPL_INFO_1003": SimStruct(OrderedDict((("rp1003_random", SimTypeInt(signed=False, label="UInt32")),)), name="REPL_INFO_1003", pack=False, align=None),
    "REPORT_EVENT_TYPE": SimTypeInt(signed=False, label="REPORT_EVENT_TYPE"),
    "REPORT_STORE_TYPES": SimTypeInt(signed=False, label="REPORT_STORE_TYPES"),
    "REQRESIZE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="REQRESIZE", pack=False, align=None),
    "REQUESTBODY_GetStatus": SimStruct(OrderedDict((("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="REQUESTBODY_GetStatus", pack=False, align=None),
    "REQUESTBODY_Renew": SimStruct(OrderedDict((("Expires", SimTypePointer(SimTypeRef("WSD_EVENTING_EXPIRES", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="REQUESTBODY_Renew", pack=False, align=None),
    "REQUESTBODY_Subscribe": SimStruct(OrderedDict((("EndTo", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Delivery", SimTypePointer(SimTypeRef("WSD_EVENTING_DELIVERY_MODE", SimStruct), offset=0)), ("Expires", SimTypePointer(SimTypeRef("WSD_EVENTING_EXPIRES", SimStruct), offset=0)), ("Filter", SimTypePointer(SimTypeRef("WSD_EVENTING_FILTER", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="REQUESTBODY_Subscribe", pack=False, align=None),
    "REQUESTBODY_Unsubscribe": SimStruct(OrderedDict((("any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="REQUESTBODY_Unsubscribe", pack=False, align=None),
    "REQUEST_OPLOCK_INPUT_BUFFER": SimStruct(OrderedDict((("StructureVersion", SimTypeShort(signed=False, label="UInt16")), ("StructureLength", SimTypeShort(signed=False, label="UInt16")), ("RequestedOplockLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="REQUEST_OPLOCK_INPUT_BUFFER", pack=False, align=None),
    "REQUEST_OPLOCK_OUTPUT_BUFFER": SimStruct(OrderedDict((("StructureVersion", SimTypeShort(signed=False, label="UInt16")), ("StructureLength", SimTypeShort(signed=False, label="UInt16")), ("OriginalOplockLevel", SimTypeInt(signed=False, label="UInt32")), ("NewOplockLevel", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AccessMode", SimTypeInt(signed=False, label="UInt32")), ("ShareMode", SimTypeShort(signed=False, label="UInt16")),)), name="REQUEST_OPLOCK_OUTPUT_BUFFER", pack=False, align=None),
    "REQUEST_POWER_COMPLETE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeChar(label="Byte"), SimUnion({"SystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DeviceState": SimTypeInt(signed=False, label="DEVICE_POWER_STATE")}, name="<anon>", label="None"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "MinorFunction", "PowerState", "Context", "IoStatus"]), offset=0),
    "REQUEST_RAW_ENCRYPTED_DATA": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="REQUEST_RAW_ENCRYPTED_DATA", pack=False, align=None),
    "REQUEST_TIMES": SimTypeInt(signed=False, label="REQUEST_TIMES"),
    "RESDLL_CONTEXT_OPERATION_TYPE": SimTypeInt(signed=False, label="RESDLL_CONTEXT_OPERATION_TYPE"),
    "RESIZE_VIRTUAL_DISK_FLAG": SimTypeInt(signed=False, label="RESIZE_VIRTUAL_DISK_FLAG"),
    "RESIZE_VIRTUAL_DISK_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="RESIZE_VIRTUAL_DISK_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="RESIZE_VIRTUAL_DISK_PARAMETERS", pack=False, align=None),
    "RESIZE_VIRTUAL_DISK_VERSION": SimTypeInt(signed=False, label="RESIZE_VIRTUAL_DISK_VERSION"),
    "RESOURCEMANAGER_BASIC_INFORMATION": SimStruct(OrderedDict((("ResourceManagerId", SimTypeBottom(label="Guid")), ("DescriptionLength", SimTypeInt(signed=False, label="UInt32")), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="RESOURCEMANAGER_BASIC_INFORMATION", pack=False, align=None),
    "RESOURCEMANAGER_COMPLETION_INFORMATION": SimStruct(OrderedDict((("IoCompletionPortHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CompletionKey", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RESOURCEMANAGER_COMPLETION_INFORMATION", pack=False, align=None),
    "RESOURCEMANAGER_INFORMATION_CLASS": SimTypeInt(signed=False, label="RESOURCEMANAGER_INFORMATION_CLASS"),
    "RESOURCE_DISPLAY_TYPE": SimTypeInt(signed=False, label="RESOURCE_DISPLAY_TYPE"),
    "RESOURCE_EXIT_STATE": SimTypeInt(signed=False, label="RESOURCE_EXIT_STATE"),
    "RESOURCE_FAILURE_INFO": SimStruct(OrderedDict((("dwRestartAttemptsRemaining", SimTypeInt(signed=False, label="UInt32")), ("dwRestartPeriodRemaining", SimTypeInt(signed=False, label="UInt32")),)), name="RESOURCE_FAILURE_INFO", pack=False, align=None),
    "RESOURCE_FAILURE_INFO_BUFFER": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("Info", SimTypeRef("RESOURCE_FAILURE_INFO", SimStruct)),)), name="RESOURCE_FAILURE_INFO_BUFFER", pack=False, align=None),
    "RESOURCE_HASH_ENTRY": SimStruct(OrderedDict((("ListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("Address", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ContentionCount", SimTypeInt(signed=False, label="UInt32")), ("Number", SimTypeInt(signed=False, label="UInt32")),)), name="RESOURCE_HASH_ENTRY", pack=False, align=None),
    "RESOURCE_MONITOR_STATE": SimTypeInt(signed=False, label="RESOURCE_MONITOR_STATE"),
    "RESOURCE_PERFORMANCE_DATA": SimStruct(OrderedDict((("ActiveResourceCount", SimTypeInt(signed=False, label="UInt32")), ("TotalResourceCount", SimTypeInt(signed=False, label="UInt32")), ("ExclusiveAcquire", SimTypeInt(signed=False, label="UInt32")), ("SharedFirstLevel", SimTypeInt(signed=False, label="UInt32")), ("SharedSecondLevel", SimTypeInt(signed=False, label="UInt32")), ("StarveFirstLevel", SimTypeInt(signed=False, label="UInt32")), ("StarveSecondLevel", SimTypeInt(signed=False, label="UInt32")), ("WaitForExclusive", SimTypeInt(signed=False, label="UInt32")), ("OwnerTableExpands", SimTypeInt(signed=False, label="UInt32")), ("MaximumTableExpand", SimTypeInt(signed=False, label="UInt32")), ("HashTable", SimTypeArray(SimTypeRef("LIST_ENTRY", SimStruct), 64)),)), name="RESOURCE_PERFORMANCE_DATA", pack=False, align=None),
    "RESOURCE_STATUS": SimStruct(OrderedDict((("ResourceState", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE")), ("CheckPoint", SimTypeInt(signed=False, label="UInt32")), ("WaitHint", SimTypeInt(signed=False, label="UInt32")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RESOURCE_STATUS", pack=False, align=None),
    "RESOURCE_STATUS_EX": SimStruct(OrderedDict((("ResourceState", SimTypeInt(signed=False, label="CLUSTER_RESOURCE_STATE")), ("CheckPoint", SimTypeInt(signed=False, label="UInt32")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ApplicationSpecificErrorCode", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("WaitHint", SimTypeInt(signed=False, label="UInt32")),)), name="RESOURCE_STATUS_EX", pack=False, align=None),
    "RESOURCE_TERMINAL_FAILURE_INFO_BUFFER": SimStruct(OrderedDict((("isTerminalFailure", SimTypeInt(signed=True, label="Int32")), ("restartPeriodRemaining", SimTypeInt(signed=False, label="UInt32")),)), name="RESOURCE_TERMINAL_FAILURE_INFO_BUFFER", pack=False, align=None),
    "RESOURCE_TRANSLATION_DIRECTION": SimTypeInt(signed=False, label="RESOURCE_TRANSLATION_DIRECTION"),
    "RESPONSEBODY_GetMetadata": SimStruct(OrderedDict((("Metadata", SimTypePointer(SimTypeRef("WSD_METADATA_SECTION_LIST", SimStruct), offset=0)),)), name="RESPONSEBODY_GetMetadata", pack=False, align=None),
    "RESPONSEBODY_GetStatus": SimStruct(OrderedDict((("expires", SimTypePointer(SimTypeRef("WSD_EVENTING_EXPIRES", SimStruct), offset=0)), ("any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="RESPONSEBODY_GetStatus", pack=False, align=None),
    "RESPONSEBODY_Renew": SimStruct(OrderedDict((("expires", SimTypePointer(SimTypeRef("WSD_EVENTING_EXPIRES", SimStruct), offset=0)), ("any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="RESPONSEBODY_Renew", pack=False, align=None),
    "RESPONSEBODY_Subscribe": SimStruct(OrderedDict((("SubscriptionManager", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("expires", SimTypePointer(SimTypeRef("WSD_EVENTING_EXPIRES", SimStruct), offset=0)), ("any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="RESPONSEBODY_Subscribe", pack=False, align=None),
    "RESPONSEBODY_SubscriptionEnd": SimStruct(OrderedDict((("SubscriptionManager", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Status", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Reason", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="RESPONSEBODY_SubscriptionEnd", pack=False, align=None),
    "RESTOREBUTTONSTATES": SimTypeInt(signed=False, label="RESTOREBUTTONSTATES"),
    "RESTOREPOINTINFOA": SimStruct(OrderedDict((("dwEventType", SimTypeInt(signed=False, label="RESTOREPOINTINFO_EVENT_TYPE")), ("dwRestorePtType", SimTypeInt(signed=False, label="RESTOREPOINTINFO_TYPE")), ("llSequenceNumber", SimTypeLongLong(signed=True, label="Int64")), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 64)),)), name="RESTOREPOINTINFOA", pack=False, align=None),
    "RESTOREPOINTINFOEX": SimStruct(OrderedDict((("ftCreation", SimTypeRef("FILETIME", SimStruct)), ("dwEventType", SimTypeInt(signed=False, label="UInt32")), ("dwRestorePtType", SimTypeInt(signed=False, label="UInt32")), ("dwRPNum", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="RESTOREPOINTINFOEX", pack=False, align=None),
    "RESTOREPOINTINFOW": SimStruct(OrderedDict((("dwEventType", SimTypeInt(signed=False, label="RESTOREPOINTINFO_EVENT_TYPE")), ("dwRestorePtType", SimTypeInt(signed=False, label="RESTOREPOINTINFO_TYPE")), ("llSequenceNumber", SimTypeLongLong(signed=True, label="Int64")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="RESTOREPOINTINFOW", pack=False, align=None),
    "RESTOREPOINTINFO_EVENT_TYPE": SimTypeInt(signed=False, label="RESTOREPOINTINFO_EVENT_TYPE"),
    "RESTOREPOINTINFO_TYPE": SimTypeInt(signed=False, label="RESTOREPOINTINFO_TYPE"),
    "RESTRICTION": SimStruct(OrderedDict((("rt", SimTypeInt(signed=False, label="UInt32")), ("weight", SimTypeInt(signed=False, label="UInt32")), ("res", SimUnion({"ar": SimTypeRef("NODERESTRICTION", SimStruct), "orRestriction": SimTypeRef("NODERESTRICTION", SimStruct), "pxr": SimTypeRef("NODERESTRICTION", SimStruct), "vr": SimTypeRef("VECTORRESTRICTION", SimStruct), "nr": SimTypeRef("NOTRESTRICTION", SimStruct), "cr": SimTypeRef("CONTENTRESTRICTION", SimStruct), "nlr": SimTypeRef("NATLANGUAGERESTRICTION", SimStruct), "pr": SimTypeRef("PROPERTYRESTRICTION", SimStruct)}, name="<anon>", label="None")),)), name="RESTRICTION", pack=False, align=None),
    "RESTRICTIONS": SimTypeInt(signed=False, label="RESTRICTIONS"),
    "RESULTDATAITEM": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="UInt32")), ("bScopeItem", SimTypeInt(signed=True, label="Int32")), ("itemID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nIndex", SimTypeInt(signed=True, label="Int32")), ("nCol", SimTypeInt(signed=True, label="Int32")), ("str", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nImage", SimTypeInt(signed=True, label="Int32")), ("nState", SimTypeInt(signed=False, label="UInt32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIndent", SimTypeInt(signed=True, label="Int32")),)), name="RESULTDATAITEM", pack=False, align=None),
    "RESULTFINDINFO": SimStruct(OrderedDict((("psz", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nStart", SimTypeInt(signed=True, label="Int32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")),)), name="RESULTFINDINFO", pack=False, align=None),
    "RESULTTYPES": SimTypeInt(signed=False, label="RESULTTYPES"),
    "RESULT_VIEW_TYPE_INFO": SimStruct(OrderedDict((("pstrPersistableViewDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("eViewType", SimTypeInt(signed=False, label="MMC_VIEW_TYPE")), ("dwMiscOptions", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwListOptions": SimTypeInt(signed=False, label="UInt32"), "Anonymous1": SimStruct(OrderedDict((("dwHTMLOptions", SimTypeInt(signed=False, label="UInt32")), ("pstrURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("dwOCXOptions", SimTypeInt(signed=False, label="UInt32")), ("pUnkControl", SimTypeBottom(label="IUnknown")),)), name="_Anonymous2_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="RESULT_VIEW_TYPE_INFO", pack=False, align=None),
    "RESUME_PERFORMANCE": SimStruct(OrderedDict((("PostTimeMs", SimTypeInt(signed=False, label="UInt32")), ("TotalResumeTimeMs", SimTypeLongLong(signed=False, label="UInt64")), ("ResumeCompleteTimestamp", SimTypeLongLong(signed=False, label="UInt64")),)), name="RESUME_PERFORMANCE", pack=False, align=None),
    "RESUTIL_FILETIME_DATA": SimStruct(OrderedDict((("Default", SimTypeRef("FILETIME", SimStruct)), ("Minimum", SimTypeRef("FILETIME", SimStruct)), ("Maximum", SimTypeRef("FILETIME", SimStruct)),)), name="RESUTIL_FILETIME_DATA", pack=False, align=None),
    "RESUTIL_LARGEINT_DATA": SimStruct(OrderedDict((("Default", SimTypeLongLong(signed=True, label="Int64")), ("Minimum", SimTypeLongLong(signed=True, label="Int64")), ("Maximum", SimTypeLongLong(signed=True, label="Int64")),)), name="RESUTIL_LARGEINT_DATA", pack=False, align=None),
    "RESUTIL_PROPERTY_ITEM": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("KeyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Format", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"DefaultPtr": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Default": SimTypeInt(signed=False, label="UInt32"), "lpDefault": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "LargeIntData": SimTypePointer(SimTypeRef("RESUTIL_LARGEINT_DATA", SimStruct), offset=0), "ULargeIntData": SimTypePointer(SimTypeRef("RESUTIL_ULARGEINT_DATA", SimStruct), offset=0), "FileTimeData": SimTypePointer(SimTypeRef("RESUTIL_FILETIME_DATA", SimStruct), offset=0)}, name="<anon>", label="None")), ("Minimum", SimTypeInt(signed=False, label="UInt32")), ("Maximum", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="RESUTIL_PROPERTY_ITEM", pack=False, align=None),
    "RESUTIL_ULARGEINT_DATA": SimStruct(OrderedDict((("Default", SimTypeLongLong(signed=False, label="UInt64")), ("Minimum", SimTypeLongLong(signed=False, label="UInt64")), ("Maximum", SimTypeLongLong(signed=False, label="UInt64")),)), name="RESUTIL_ULARGEINT_DATA", pack=False, align=None),
    "RESV_STYLE": SimStruct(OrderedDict((("style_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("style_word", SimTypeInt(signed=False, label="UInt32")),)), name="RESV_STYLE", pack=False, align=None),
    "RETRIEVAL_POINTERS_AND_REFCOUNT_BUFFER": SimStruct(OrderedDict((("ExtentCount", SimTypeInt(signed=False, label="UInt32")), ("StartingVcn", SimTypeLongLong(signed=True, label="Int64")), ("Extents", SimTypePointer(SimStruct(OrderedDict((("NextVcn", SimTypeLongLong(signed=True, label="Int64")), ("Lcn", SimTypeLongLong(signed=True, label="Int64")), ("ReferenceCount", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="RETRIEVAL_POINTERS_AND_REFCOUNT_BUFFER", pack=False, align=None),
    "RETRIEVAL_POINTERS_BUFFER": SimStruct(OrderedDict((("ExtentCount", SimTypeInt(signed=False, label="UInt32")), ("StartingVcn", SimTypeLongLong(signed=True, label="Int64")), ("Extents", SimTypePointer(SimStruct(OrderedDict((("NextVcn", SimTypeLongLong(signed=True, label="Int64")), ("Lcn", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="RETRIEVAL_POINTERS_BUFFER", pack=False, align=None),
    "RETRIEVAL_POINTER_BASE": SimStruct(OrderedDict((("FileAreaOffset", SimTypeLongLong(signed=True, label="Int64")),)), name="RETRIEVAL_POINTER_BASE", pack=False, align=None),
    "RETRIEVAL_POINTER_COUNT": SimStruct(OrderedDict((("ExtentCount", SimTypeInt(signed=False, label="UInt32")),)), name="RETRIEVAL_POINTER_COUNT", pack=False, align=None),
    "REVERT_SECURITY_CONTEXT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "RFC1766INFO": SimStruct(OrderedDict((("lcid", SimTypeInt(signed=False, label="UInt32")), ("wszRfc1766", SimTypeArray(SimTypeChar(label="Char"), 6)), ("wszLocaleName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="RFC1766INFO", pack=False, align=None),
    "RFCOMM_COMMAND": SimStruct(OrderedDict((("CmdType", SimTypeInt(signed=False, label="UInt32")), ("Data", SimUnion({"MSC": SimTypeRef("RFCOMM_MSC_DATA", SimStruct), "RLS": SimTypeRef("RFCOMM_RLS_DATA", SimStruct), "RPN": SimTypeRef("RFCOMM_RPN_DATA", SimStruct)}, name="<anon>", label="None")),)), name="RFCOMM_COMMAND", pack=False, align=None),
    "RFCOMM_MSC_DATA": SimStruct(OrderedDict((("Signals", SimTypeChar(label="Byte")), ("Break", SimTypeChar(label="Byte")),)), name="RFCOMM_MSC_DATA", pack=False, align=None),
    "RFCOMM_RLS_DATA": SimStruct(OrderedDict((("LineStatus", SimTypeChar(label="Byte")),)), name="RFCOMM_RLS_DATA", pack=False, align=None),
    "RFCOMM_RPN_DATA": SimStruct(OrderedDict((("Baud", SimTypeChar(label="Byte")), ("Data", SimTypeChar(label="Byte")), ("FlowControl", SimTypeChar(label="Byte")), ("XonChar", SimTypeChar(label="Byte")), ("XoffChar", SimTypeChar(label="Byte")), ("ParameterMask1", SimTypeChar(label="Byte")), ("ParameterMask2", SimTypeChar(label="Byte")),)), name="RFCOMM_RPN_DATA", pack=False, align=None),
    "RFX_GFX_MONITOR_INFO": SimStruct(OrderedDict((("left", SimTypeInt(signed=True, label="Int32")), ("top", SimTypeInt(signed=True, label="Int32")), ("right", SimTypeInt(signed=True, label="Int32")), ("bottom", SimTypeInt(signed=True, label="Int32")), ("physicalWidth", SimTypeInt(signed=False, label="UInt32")), ("physicalHeight", SimTypeInt(signed=False, label="UInt32")), ("orientation", SimTypeInt(signed=False, label="UInt32")), ("primary", SimTypeInt(signed=True, label="Int32")),)), name="RFX_GFX_MONITOR_INFO", pack=False, align=None),
    "RFX_GFX_MSG_CLIENT_DESKTOP_INFO_REQUEST": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)),)), name="RFX_GFX_MSG_CLIENT_DESKTOP_INFO_REQUEST", pack=False, align=None),
    "RFX_GFX_MSG_CLIENT_DESKTOP_INFO_RESPONSE": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)), ("reserved", SimTypeInt(signed=False, label="UInt32")), ("monitorCount", SimTypeInt(signed=False, label="UInt32")), ("MonitorData", SimTypeArray(SimTypeRef("RFX_GFX_MONITOR_INFO", SimStruct), 16)), ("clientUniqueId", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="RFX_GFX_MSG_CLIENT_DESKTOP_INFO_RESPONSE", pack=False, align=None),
    "RFX_GFX_MSG_DESKTOP_CONFIG_CHANGE_CONFIRM": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)),)), name="RFX_GFX_MSG_DESKTOP_CONFIG_CHANGE_CONFIRM", pack=False, align=None),
    "RFX_GFX_MSG_DESKTOP_CONFIG_CHANGE_NOTIFY": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)), ("ulWidth", SimTypeInt(signed=False, label="UInt32")), ("ulHeight", SimTypeInt(signed=False, label="UInt32")), ("ulBpp", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="RFX_GFX_MSG_DESKTOP_CONFIG_CHANGE_NOTIFY", pack=False, align=None),
    "RFX_GFX_MSG_DESKTOP_INPUT_RESET": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)), ("ulWidth", SimTypeInt(signed=False, label="UInt32")), ("ulHeight", SimTypeInt(signed=False, label="UInt32")),)), name="RFX_GFX_MSG_DESKTOP_INPUT_RESET", pack=False, align=None),
    "RFX_GFX_MSG_DESKTOP_RESEND_REQUEST": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)), ("RedrawRect", SimTypeRef("RFX_GFX_RECT", SimStruct)),)), name="RFX_GFX_MSG_DESKTOP_RESEND_REQUEST", pack=False, align=None),
    "RFX_GFX_MSG_DISCONNECT_NOTIFY": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)), ("DisconnectReason", SimTypeInt(signed=False, label="UInt32")),)), name="RFX_GFX_MSG_DISCONNECT_NOTIFY", pack=False, align=None),
    "RFX_GFX_MSG_HEADER": SimStruct(OrderedDict((("uMSGType", SimTypeShort(signed=False, label="UInt16")), ("cbSize", SimTypeShort(signed=False, label="UInt16")),)), name="RFX_GFX_MSG_HEADER", pack=False, align=None),
    "RFX_GFX_MSG_RDP_DATA": SimStruct(OrderedDict((("channelHdr", SimTypeRef("RFX_GFX_MSG_HEADER", SimStruct)), ("rdpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RFX_GFX_MSG_RDP_DATA", pack=False, align=None),
    "RFX_GFX_RECT": SimStruct(OrderedDict((("left", SimTypeInt(signed=True, label="Int32")), ("top", SimTypeInt(signed=True, label="Int32")), ("right", SimTypeInt(signed=True, label="Int32")), ("bottom", SimTypeInt(signed=True, label="Int32")),)), name="RFX_GFX_RECT", pack=False, align=None),
    "RGBCOLOR": SimStruct(OrderedDict((("red", SimTypeShort(signed=False, label="UInt16")), ("green", SimTypeShort(signed=False, label="UInt16")), ("blue", SimTypeShort(signed=False, label="UInt16")),)), name="RGBCOLOR", pack=False, align=None),
    "RGBQUAD": SimStruct(OrderedDict((("rgbBlue", SimTypeChar(label="Byte")), ("rgbGreen", SimTypeChar(label="Byte")), ("rgbRed", SimTypeChar(label="Byte")), ("rgbReserved", SimTypeChar(label="Byte")),)), name="RGBQUAD", pack=False, align=None),
    "RGBTRIPLE": SimStruct(OrderedDict((("rgbtBlue", SimTypeChar(label="Byte")), ("rgbtGreen", SimTypeChar(label="Byte")), ("rgbtRed", SimTypeChar(label="Byte")),)), name="RGBTRIPLE", pack=False, align=None),
    "RGNDATA": SimStruct(OrderedDict((("rdh", SimTypeRef("RGNDATAHEADER", SimStruct)), ("Buffer", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="RGNDATA", pack=False, align=None),
    "RGNDATAHEADER": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("iType", SimTypeInt(signed=False, label="UInt32")), ("nCount", SimTypeInt(signed=False, label="UInt32")), ("nRgnSize", SimTypeInt(signed=False, label="UInt32")), ("rcBound", SimTypeRef("RECT", SimStruct)),)), name="RGNDATAHEADER", pack=False, align=None),
    "RGNHEADER": SimStruct(OrderedDict((("RangeKey", SimTypeRef("RGNRANGE", SimStruct)), ("RangeVelocity", SimTypeRef("RGNRANGE", SimStruct)), ("fusOptions", SimTypeShort(signed=False, label="UInt16")), ("usKeyGroup", SimTypeShort(signed=False, label="UInt16")),)), name="RGNHEADER", pack=False, align=None),
    "RGNRANGE": SimStruct(OrderedDict((("usLow", SimTypeShort(signed=False, label="UInt16")), ("usHigh", SimTypeShort(signed=False, label="UInt16")),)), name="RGNRANGE", pack=False, align=None),
    "RGN_COMBINE_MODE": SimTypeInt(signed=False, label="RGN_COMBINE_MODE"),
    "RHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "RICHEDIT_IMAGE_PARAMETERS": SimStruct(OrderedDict((("xWidth", SimTypeInt(signed=True, label="Int32")), ("yHeight", SimTypeInt(signed=True, label="Int32")), ("Ascent", SimTypeInt(signed=True, label="Int32")), ("Type", SimTypeInt(signed=True, label="Int32")), ("pwszAlternateText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pIStream", SimTypeBottom(label="IStream")),)), name="RICHEDIT_IMAGE_PARAMETERS", pack=False, align=None),
    "RICH_EDIT_GET_CONTEXT_MENU_SEL_TYPE": SimTypeInt(signed=False, label="RICH_EDIT_GET_CONTEXT_MENU_SEL_TYPE"),
    "RICH_EDIT_GET_OBJECT_FLAGS": SimTypeInt(signed=False, label="RICH_EDIT_GET_OBJECT_FLAGS"),
    "RID_DEVICE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="RID_DEVICE_INFO_TYPE")), ("Anonymous", SimUnion({"mouse": SimTypeRef("RID_DEVICE_INFO_MOUSE", SimStruct), "keyboard": SimTypeRef("RID_DEVICE_INFO_KEYBOARD", SimStruct), "hid": SimTypeRef("RID_DEVICE_INFO_HID", SimStruct)}, name="<anon>", label="None")),)), name="RID_DEVICE_INFO", pack=False, align=None),
    "RID_DEVICE_INFO_HID": SimStruct(OrderedDict((("dwVendorId", SimTypeInt(signed=False, label="UInt32")), ("dwProductId", SimTypeInt(signed=False, label="UInt32")), ("dwVersionNumber", SimTypeInt(signed=False, label="UInt32")), ("usUsagePage", SimTypeShort(signed=False, label="UInt16")), ("usUsage", SimTypeShort(signed=False, label="UInt16")),)), name="RID_DEVICE_INFO_HID", pack=False, align=None),
    "RID_DEVICE_INFO_KEYBOARD": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwSubType", SimTypeInt(signed=False, label="UInt32")), ("dwKeyboardMode", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfFunctionKeys", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfIndicators", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfKeysTotal", SimTypeInt(signed=False, label="UInt32")),)), name="RID_DEVICE_INFO_KEYBOARD", pack=False, align=None),
    "RID_DEVICE_INFO_MOUSE": SimStruct(OrderedDict((("dwId", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfButtons", SimTypeInt(signed=False, label="UInt32")), ("dwSampleRate", SimTypeInt(signed=False, label="UInt32")), ("fHasHorizontalWheel", SimTypeInt(signed=True, label="Int32")),)), name="RID_DEVICE_INFO_MOUSE", pack=False, align=None),
    "RID_DEVICE_INFO_TYPE": SimTypeInt(signed=False, label="RID_DEVICE_INFO_TYPE"),
    "RIFFCHUNK": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")),)), name="RIFFCHUNK", pack=False, align=None),
    "RIFFLIST": SimStruct(OrderedDict((("fcc", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("fccListType", SimTypeInt(signed=False, label="UInt32")),)), name="RIFFLIST", pack=False, align=None),
    "RIORESULT": SimStruct(OrderedDict((("Status", SimTypeInt(signed=True, label="Int32")), ("BytesTransferred", SimTypeInt(signed=False, label="UInt32")), ("SocketContext", SimTypeLongLong(signed=False, label="UInt64")), ("RequestContext", SimTypeLongLong(signed=False, label="UInt64")),)), name="RIORESULT", pack=False, align=None),
    "RIO_BUF": SimStruct(OrderedDict((("BufferId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="RIO_BUF", pack=False, align=None),
    "RIO_BUFFERID": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "RIO_CMSG_BUFFER": SimStruct(OrderedDict((("TotalLength", SimTypeInt(signed=False, label="UInt32")),)), name="RIO_CMSG_BUFFER", pack=False, align=None),
    "RIO_CQ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "RIO_EXTENSION_FUNCTION_TABLE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("RIOReceive", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "Flags", "RequestContext"]), offset=0)), ("RIOReceiveEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "pLocalAddress", "pRemoteAddress", "pControlContext", "pFlags", "Flags", "RequestContext"]), offset=0)), ("RIOSend", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "Flags", "RequestContext"]), offset=0)), ("RIOSendEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("RIO_BUF", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SocketQueue", "pData", "DataBufferCount", "pLocalAddress", "pRemoteAddress", "pControlContext", "pFlags", "Flags", "RequestContext"]), offset=0)), ("RIOCloseCompletionQueue", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["CQ"]), offset=0)), ("RIOCreateCompletionQueue", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("RIO_NOTIFICATION_COMPLETION", SimStruct), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["QueueSize", "NotificationCompletion"]), offset=0)), ("RIOCreateRequestQueue", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["Socket", "MaxOutstandingReceive", "MaxReceiveDataBuffers", "MaxOutstandingSend", "MaxSendDataBuffers", "ReceiveCQ", "SendCQ", "SocketContext"]), offset=0)), ("RIODequeueCompletion", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RIORESULT", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["CQ", "Array", "ArraySize"]), offset=0)), ("RIODeregisterBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["BufferId"]), offset=0)), ("RIONotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CQ"]), offset=0)), ("RIORegisterBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["DataBuffer", "DataLength"]), offset=0)), ("RIOResizeCompletionQueue", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CQ", "QueueSize"]), offset=0)), ("RIOResizeRequestQueue", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["RQ", "MaxOutstandingReceive", "MaxOutstandingSend"]), offset=0)),)), name="RIO_EXTENSION_FUNCTION_TABLE", pack=False, align=None),
    "RIO_NOTIFICATION_COMPLETION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="RIO_NOTIFICATION_COMPLETION_TYPE")), ("Anonymous", SimUnion({"Event": SimTypeRef("_Event_e__Struct", SimStruct), "Iocp": SimTypeRef("_Iocp_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="RIO_NOTIFICATION_COMPLETION", pack=False, align=None),
    "RIO_NOTIFICATION_COMPLETION_TYPE": SimTypeInt(signed=False, label="RIO_NOTIFICATION_COMPLETION_TYPE"),
    "RIO_RQ": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "RIP_INFO": SimStruct(OrderedDict((("dwError", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="RIP_INFO_TYPE")),)), name="RIP_INFO", pack=False, align=None),
    "RIP_INFO_TYPE": SimTypeInt(signed=False, label="RIP_INFO_TYPE"),
    "RKF_BYPASS_ECP_CONTEXT": SimStruct(OrderedDict((("Reserved", SimTypeInt(signed=True, label="Int32")), ("Version", SimTypeInt(signed=True, label="Int32")),)), name="RKF_BYPASS_ECP_CONTEXT", pack=False, align=None),
    "RMTPACK": SimStruct(OrderedDict((("pISeqStream", SimTypeBottom(label="ISequentialStream")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("cBSTR", SimTypeInt(signed=False, label="UInt32")), ("rgBSTR", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("cVARIANT", SimTypeInt(signed=False, label="UInt32")), ("rgVARIANT", SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)), ("cIDISPATCH", SimTypeInt(signed=False, label="UInt32")), ("rgIDISPATCH", SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0)), ("cIUNKNOWN", SimTypeInt(signed=False, label="UInt32")), ("rgIUNKNOWN", SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)), ("cPROPVARIANT", SimTypeInt(signed=False, label="UInt32")), ("rgPROPVARIANT", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("cArray", SimTypeInt(signed=False, label="UInt32")), ("rgArray", SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)),)), name="RMTPACK", pack=False, align=None),
    "RM_APP_STATUS": SimTypeInt(signed=False, label="RM_APP_STATUS"),
    "RM_APP_TYPE": SimTypeInt(signed=False, label="RM_APP_TYPE"),
    "RM_FEC_INFO": SimStruct(OrderedDict((("FECBlockSize", SimTypeShort(signed=False, label="UInt16")), ("FECProActivePackets", SimTypeShort(signed=False, label="UInt16")), ("FECGroupSize", SimTypeChar(label="Byte")), ("fFECOnDemandParityEnabled", SimTypeChar(label="Byte")),)), name="RM_FEC_INFO", pack=False, align=None),
    "RM_FILTER_ACTION": SimTypeInt(signed=False, label="RM_FILTER_ACTION"),
    "RM_FILTER_INFO": SimStruct(OrderedDict((("FilterAction", SimTypeInt(signed=False, label="RM_FILTER_ACTION")), ("FilterTrigger", SimTypeInt(signed=False, label="RM_FILTER_TRIGGER")), ("cbNextOffset", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"strFilename": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Process": SimTypeRef("RM_UNIQUE_PROCESS", SimStruct), "strServiceShortName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="RM_FILTER_INFO", pack=False, align=None),
    "RM_FILTER_TRIGGER": SimTypeInt(signed=False, label="RM_FILTER_TRIGGER"),
    "RM_PROCESS_INFO": SimStruct(OrderedDict((("Process", SimTypeRef("RM_UNIQUE_PROCESS", SimStruct)), ("strAppName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("strServiceShortName", SimTypeArray(SimTypeChar(label="Char"), 64)), ("ApplicationType", SimTypeInt(signed=False, label="RM_APP_TYPE")), ("AppStatus", SimTypeInt(signed=False, label="UInt32")), ("TSSessionId", SimTypeInt(signed=False, label="UInt32")), ("bRestartable", SimTypeInt(signed=True, label="Int32")),)), name="RM_PROCESS_INFO", pack=False, align=None),
    "RM_REBOOT_REASON": SimTypeInt(signed=False, label="RM_REBOOT_REASON"),
    "RM_RECEIVER_STATS": SimStruct(OrderedDict((("NumODataPacketsReceived", SimTypeLongLong(signed=False, label="UInt64")), ("NumRDataPacketsReceived", SimTypeLongLong(signed=False, label="UInt64")), ("NumDuplicateDataPackets", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("TotalBytesReceived", SimTypeLongLong(signed=False, label="UInt64")), ("RateKBitsPerSecOverall", SimTypeLongLong(signed=False, label="UInt64")), ("RateKBitsPerSecLast", SimTypeLongLong(signed=False, label="UInt64")), ("TrailingEdgeSeqId", SimTypeLongLong(signed=False, label="UInt64")), ("LeadingEdgeSeqId", SimTypeLongLong(signed=False, label="UInt64")), ("AverageSequencesInWindow", SimTypeLongLong(signed=False, label="UInt64")), ("MinSequencesInWindow", SimTypeLongLong(signed=False, label="UInt64")), ("MaxSequencesInWindow", SimTypeLongLong(signed=False, label="UInt64")), ("FirstNakSequenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("NumPendingNaks", SimTypeLongLong(signed=False, label="UInt64")), ("NumOutstandingNaks", SimTypeLongLong(signed=False, label="UInt64")), ("NumDataPacketsBuffered", SimTypeLongLong(signed=False, label="UInt64")), ("TotalSelectiveNaksSent", SimTypeLongLong(signed=False, label="UInt64")), ("TotalParityNaksSent", SimTypeLongLong(signed=False, label="UInt64")),)), name="RM_RECEIVER_STATS", pack=False, align=None),
    "RM_SENDER_STATS": SimStruct(OrderedDict((("DataBytesSent", SimTypeLongLong(signed=False, label="UInt64")), ("TotalBytesSent", SimTypeLongLong(signed=False, label="UInt64")), ("NaksReceived", SimTypeLongLong(signed=False, label="UInt64")), ("NaksReceivedTooLate", SimTypeLongLong(signed=False, label="UInt64")), ("NumOutstandingNaks", SimTypeLongLong(signed=False, label="UInt64")), ("NumNaksAfterRData", SimTypeLongLong(signed=False, label="UInt64")), ("RepairPacketsSent", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSpaceAvailable", SimTypeLongLong(signed=False, label="UInt64")), ("TrailingEdgeSeqId", SimTypeLongLong(signed=False, label="UInt64")), ("LeadingEdgeSeqId", SimTypeLongLong(signed=False, label="UInt64")), ("RateKBitsPerSecOverall", SimTypeLongLong(signed=False, label="UInt64")), ("RateKBitsPerSecLast", SimTypeLongLong(signed=False, label="UInt64")), ("TotalODataPacketsSent", SimTypeLongLong(signed=False, label="UInt64")),)), name="RM_SENDER_STATS", pack=False, align=None),
    "RM_SEND_WINDOW": SimStruct(OrderedDict((("RateKbitsPerSec", SimTypeInt(signed=False, label="UInt32")), ("WindowSizeInMSecs", SimTypeInt(signed=False, label="UInt32")), ("WindowSizeInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="RM_SEND_WINDOW", pack=False, align=None),
    "RM_SHUTDOWN_TYPE": SimTypeInt(signed=False, label="RM_SHUTDOWN_TYPE"),
    "RM_UNIQUE_PROCESS": SimStruct(OrderedDict((("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("ProcessStartTime", SimTypeRef("FILETIME", SimStruct)),)), name="RM_UNIQUE_PROCESS", pack=False, align=None),
    "RM_WRITE_STATUS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["nPercentComplete"]), offset=0),
    "RND_ADVERTISING_SCOPE": SimTypeInt(signed=False, label="RND_ADVERTISING_SCOPE"),
    "ROI_AREA": SimStruct(OrderedDict((("rect", SimTypeRef("RECT", SimStruct)), ("QPDelta", SimTypeInt(signed=True, label="Int32")),)), name="ROI_AREA", pack=False, align=None),
    "ROOT_INFO_LUID": SimStruct(OrderedDict((("LowPart", SimTypeInt(signed=False, label="UInt32")), ("HighPart", SimTypeInt(signed=True, label="Int32")),)), name="ROOT_INFO_LUID", pack=False, align=None),
    "ROP_CODE": SimTypeInt(signed=False, label="ROP_CODE"),
    "ROT_FLAGS": SimTypeInt(signed=False, label="ROT_FLAGS"),
    "ROUTER_CONNECTION_STATE": SimTypeInt(signed=False, label="ROUTER_CONNECTION_STATE"),
    "ROUTER_CUSTOM_IKEv2_POLICY0": SimStruct(OrderedDict((("dwIntegrityMethod", SimTypeInt(signed=False, label="UInt32")), ("dwEncryptionMethod", SimTypeInt(signed=False, label="UInt32")), ("dwCipherTransformConstant", SimTypeInt(signed=False, label="UInt32")), ("dwAuthTransformConstant", SimTypeInt(signed=False, label="UInt32")), ("dwPfsGroup", SimTypeInt(signed=False, label="UInt32")), ("dwDhGroup", SimTypeInt(signed=False, label="UInt32")),)), name="ROUTER_CUSTOM_IKEv2_POLICY0", pack=False, align=None),
    "ROUTER_IKEv2_IF_CUSTOM_CONFIG0": SimStruct(OrderedDict((("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSize", SimTypeInt(signed=False, label="UInt32")), ("certificateName", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)),)), name="ROUTER_IKEv2_IF_CUSTOM_CONFIG0", pack=False, align=None),
    "ROUTER_IKEv2_IF_CUSTOM_CONFIG1": SimStruct(OrderedDict((("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSize", SimTypeInt(signed=False, label="UInt32")), ("certificateName", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)), ("certificateHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="ROUTER_IKEv2_IF_CUSTOM_CONFIG1", pack=False, align=None),
    "ROUTER_IKEv2_IF_CUSTOM_CONFIG2": SimStruct(OrderedDict((("dwSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("dwSaDataSize", SimTypeInt(signed=False, label="UInt32")), ("certificateName", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("customPolicy", SimTypePointer(SimTypeRef("ROUTER_CUSTOM_IKEv2_POLICY0", SimStruct), offset=0)), ("certificateHash", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("dwMmSaLifeTime", SimTypeInt(signed=False, label="UInt32")), ("vpnTrafficSelectors", SimTypeRef("MPR_VPN_TRAFFIC_SELECTORS", SimStruct)),)), name="ROUTER_IKEv2_IF_CUSTOM_CONFIG2", pack=False, align=None),
    "ROUTER_INTERFACE_TYPE": SimTypeInt(signed=False, label="ROUTER_INTERFACE_TYPE"),
    "ROUTER_NOTIFY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("PRINTER_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCommand", "pContext", "dwColor", "pNofityInfo", "fdwFlags", "pdwResult"]), offset=0),
    "ROUTING_PROTOCOL_CONFIG": SimStruct(OrderedDict((("dwCallbackFlags", SimTypeInt(signed=False, label="UInt32")), ("pfnRpfCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "pdwInIfIndex", "pdwInIfNextHopAddr", "pdwUpStreamNbr", "dwHdrSize", "pbPacketHdr", "pbRoute"]), offset=0)), ("pfnCreationAlertCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("MGM_IF_ENTRY", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwInIfIndex", "dwInIfNextHopAddr", "dwIfCount", "pmieOutIfList"]), offset=0)), ("pfnPruneAlertCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwIfIndex", "dwIfNextHopAddr", "bMemberDelete", "pdwTimeout"]), offset=0)), ("pfnJoinAlertCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "bMemberUpdate"]), offset=0)), ("pfnWrongIfCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwGroupAddr", "dwIfIndex", "dwIfNextHopAddr", "dwHdrSize", "pbPacketHdr"]), offset=0)), ("pfnLocalJoinCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwIfIndex", "dwIfNextHopAddr"]), offset=0)), ("pfnLocalLeaveCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSourceAddr", "dwSourceMask", "dwGroupAddr", "dwGroupMask", "dwIfIndex", "dwIfNextHopAddr"]), offset=0)), ("pfnDisableIgmpCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwIfIndex", "dwIfNextHopAddr"]), offset=0)), ("pfnEnableIgmpCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwIfIndex", "dwIfNextHopAddr"]), offset=0)),)), name="ROUTING_PROTOCOL_CONFIG", pack=False, align=None),
    "ROWSETEVENT_ITEMSTATE": SimTypeInt(signed=False, label="ROWSETEVENT_ITEMSTATE"),
    "ROWSETEVENT_TYPE": SimTypeInt(signed=False, label="ROWSETEVENT_TYPE"),
    "ROWSTATUS": SimStruct(OrderedDict((("lRowFirst", SimTypeInt(signed=True, label="Int32")), ("cRows", SimTypeInt(signed=True, label="Int32")), ("cProperties", SimTypeInt(signed=True, label="Int32")), ("cRowsTotal", SimTypeInt(signed=True, label="Int32")),)), name="ROWSTATUS", pack=False, align=None),
    "RPCLT_PDU_FILTER_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferLength", "fDatagram"]), offset=0),
    "RPCOLEMESSAGE": SimStruct(OrderedDict((("reserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dataRepresentation", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbBuffer", SimTypeInt(signed=False, label="UInt32")), ("iMethod", SimTypeInt(signed=False, label="UInt32")), ("reserved2", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 5)), ("rpcFlags", SimTypeInt(signed=False, label="UInt32")),)), name="RPCOLEMESSAGE", pack=False, align=None),
    "RPCOPT_PROPERTIES": SimTypeInt(signed=False, label="RPCOPT_PROPERTIES"),
    "RPCOPT_SERVER_LOCALITY_VALUES": SimTypeInt(signed=False, label="RPCOPT_SERVER_LOCALITY_VALUES"),
    "RPC_ADDRESS_CHANGE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["arg"]), offset=0),
    "RPC_ADDRESS_CHANGE_TYPE": SimTypeInt(signed=False, label="RPC_ADDRESS_CHANGE_TYPE"),
    "RPC_ASYNC_EVENT": SimTypeInt(signed=False, label="RPC_ASYNC_EVENT"),
    "RPC_ASYNC_NOTIFICATION_INFO": SimUnion({"APC": SimTypeRef("_APC_e__Struct", SimStruct), "IOC": SimTypeRef("_IOC_e__Struct", SimStruct), "IntPtr": SimTypeRef("_IntPtr_e__Struct", SimStruct), "hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "NotificationRoutine": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RPC_ASYNC_STATE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="RPC_ASYNC_EVENT")], SimTypeBottom(label="Void"), arg_names=["pAsync", "Context", "Event"]), offset=0)}, name="<anon>", label="None"),
    "RPC_ASYNC_STATE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Signature", SimTypeInt(signed=False, label="UInt32")), ("Lock", SimTypeInt(signed=True, label="Int32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("StubInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UserInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RuntimeInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Event", SimTypeInt(signed=False, label="RPC_ASYNC_EVENT")), ("NotificationType", SimTypeInt(signed=False, label="RPC_NOTIFICATION_TYPES")), ("u", SimUnion({"APC": SimTypeRef("_APC_e__Struct", SimStruct), "IOC": SimTypeRef("_IOC_e__Struct", SimStruct), "IntPtr": SimTypeRef("_IntPtr_e__Struct", SimStruct), "hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "NotificationRoutine": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RPC_ASYNC_STATE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="RPC_ASYNC_EVENT")], SimTypeBottom(label="Void"), arg_names=["pAsync", "Context", "Event"]), offset=0)}, name="<anon>", label="None")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 4)),)), name="RPC_ASYNC_STATE", pack=False, align=None),
    "RPC_AUTH_KEY_RETRIEVAL_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="RPC_STATUS"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "ServerPrincName", "KeyVer", "Key", "Status"]), offset=0),
    "RPC_BINDING_HANDLE_OPTIONS_FLAGS": SimTypeInt(signed=False, label="RPC_BINDING_HANDLE_OPTIONS_FLAGS"),
    "RPC_BINDING_HANDLE_OPTIONS_V1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="RPC_BINDING_HANDLE_OPTIONS_FLAGS")), ("ComTimeout", SimTypeInt(signed=False, label="UInt32")), ("CallTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_BINDING_HANDLE_OPTIONS_V1", pack=False, align=None),
    "RPC_BINDING_HANDLE_SECURITY_V1_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("AuthnLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthnSvc", SimTypeInt(signed=False, label="UInt32")), ("AuthIdentity", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_A", SimStruct), offset=0)), ("SecurityQos", SimTypePointer(SimTypeRef("RPC_SECURITY_QOS", SimStruct), offset=0)),)), name="RPC_BINDING_HANDLE_SECURITY_V1_A", pack=False, align=None),
    "RPC_BINDING_HANDLE_SECURITY_V1_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("AuthnLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthnSvc", SimTypeInt(signed=False, label="UInt32")), ("AuthIdentity", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_W", SimStruct), offset=0)), ("SecurityQos", SimTypePointer(SimTypeRef("RPC_SECURITY_QOS", SimStruct), offset=0)),)), name="RPC_BINDING_HANDLE_SECURITY_V1_W", pack=False, align=None),
    "RPC_BINDING_HANDLE_TEMPLATE_V1_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ProtocolSequence", SimTypeInt(signed=False, label="UInt32")), ("NetworkAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("StringEndpoint", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("u1", SimUnion({"Reserved": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("ObjectUuid", SimTypeBottom(label="Guid")),)), name="RPC_BINDING_HANDLE_TEMPLATE_V1_A", pack=False, align=None),
    "RPC_BINDING_HANDLE_TEMPLATE_V1_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ProtocolSequence", SimTypeInt(signed=False, label="UInt32")), ("NetworkAddress", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("StringEndpoint", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("u1", SimUnion({"Reserved": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="<anon>", label="None")), ("ObjectUuid", SimTypeBottom(label="Guid")),)), name="RPC_BINDING_HANDLE_TEMPLATE_V1_W", pack=False, align=None),
    "RPC_BINDING_VECTOR": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("BindingH", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="RPC_BINDING_VECTOR", pack=False, align=None),
    "RPC_BLOCKING_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["hWnd", "Context", "hSyncEvent"]), offset=0),
    "RPC_CALL_ATTRIBUTES_V1_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClientPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientPrincipalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("AuthenticationLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationService", SimTypeInt(signed=False, label="UInt32")), ("NullSession", SimTypeInt(signed=True, label="Int32")),)), name="RPC_CALL_ATTRIBUTES_V1_A", pack=False, align=None),
    "RPC_CALL_ATTRIBUTES_V1_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ClientPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientPrincipalName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("AuthenticationLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationService", SimTypeInt(signed=False, label="UInt32")), ("NullSession", SimTypeInt(signed=True, label="Int32")),)), name="RPC_CALL_ATTRIBUTES_V1_W", pack=False, align=None),
    "RPC_CALL_ATTRIBUTES_V2_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClientPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientPrincipalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("AuthenticationLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationService", SimTypeInt(signed=False, label="UInt32")), ("NullSession", SimTypeInt(signed=True, label="Int32")), ("KernelModeCaller", SimTypeInt(signed=True, label="Int32")), ("ProtocolSequence", SimTypeInt(signed=False, label="UInt32")), ("IsClientLocal", SimTypeInt(signed=False, label="UInt32")), ("ClientPID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallStatus", SimTypeInt(signed=False, label="UInt32")), ("CallType", SimTypeInt(signed=False, label="RpcCallType")), ("CallLocalAddress", SimTypePointer(SimTypeRef("RPC_CALL_LOCAL_ADDRESS_V1", SimStruct), offset=0)), ("OpNum", SimTypeShort(signed=False, label="UInt16")), ("InterfaceUuid", SimTypeBottom(label="Guid")),)), name="RPC_CALL_ATTRIBUTES_V2_A", pack=False, align=None),
    "RPC_CALL_ATTRIBUTES_V2_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ClientPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientPrincipalName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("AuthenticationLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationService", SimTypeInt(signed=False, label="UInt32")), ("NullSession", SimTypeInt(signed=True, label="Int32")), ("KernelModeCaller", SimTypeInt(signed=True, label="Int32")), ("ProtocolSequence", SimTypeInt(signed=False, label="UInt32")), ("IsClientLocal", SimTypeInt(signed=False, label="RpcCallClientLocality")), ("ClientPID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallStatus", SimTypeInt(signed=False, label="UInt32")), ("CallType", SimTypeInt(signed=False, label="RpcCallType")), ("CallLocalAddress", SimTypePointer(SimTypeRef("RPC_CALL_LOCAL_ADDRESS_V1", SimStruct), offset=0)), ("OpNum", SimTypeShort(signed=False, label="UInt16")), ("InterfaceUuid", SimTypeBottom(label="Guid")),)), name="RPC_CALL_ATTRIBUTES_V2_W", pack=False, align=None),
    "RPC_CALL_ATTRIBUTES_V3_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ClientPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientPrincipalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("AuthenticationLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationService", SimTypeInt(signed=False, label="UInt32")), ("NullSession", SimTypeInt(signed=True, label="Int32")), ("KernelModeCaller", SimTypeInt(signed=True, label="Int32")), ("ProtocolSequence", SimTypeInt(signed=False, label="UInt32")), ("IsClientLocal", SimTypeInt(signed=False, label="UInt32")), ("ClientPID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallStatus", SimTypeInt(signed=False, label="UInt32")), ("CallType", SimTypeInt(signed=False, label="RpcCallType")), ("CallLocalAddress", SimTypePointer(SimTypeRef("RPC_CALL_LOCAL_ADDRESS_V1", SimStruct), offset=0)), ("OpNum", SimTypeShort(signed=False, label="UInt16")), ("InterfaceUuid", SimTypeBottom(label="Guid")), ("ClientIdentifierBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RPC_CALL_ATTRIBUTES_V3_A", pack=False, align=None),
    "RPC_CALL_ATTRIBUTES_V3_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ServerPrincipalName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ClientPrincipalNameBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientPrincipalName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("AuthenticationLevel", SimTypeInt(signed=False, label="UInt32")), ("AuthenticationService", SimTypeInt(signed=False, label="UInt32")), ("NullSession", SimTypeInt(signed=True, label="Int32")), ("KernelModeCaller", SimTypeInt(signed=True, label="Int32")), ("ProtocolSequence", SimTypeInt(signed=False, label="UInt32")), ("IsClientLocal", SimTypeInt(signed=False, label="RpcCallClientLocality")), ("ClientPID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CallStatus", SimTypeInt(signed=False, label="UInt32")), ("CallType", SimTypeInt(signed=False, label="RpcCallType")), ("CallLocalAddress", SimTypePointer(SimTypeRef("RPC_CALL_LOCAL_ADDRESS_V1", SimStruct), offset=0)), ("OpNum", SimTypeShort(signed=False, label="UInt16")), ("InterfaceUuid", SimTypeBottom(label="Guid")), ("ClientIdentifierBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ClientIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RPC_CALL_ATTRIBUTES_V3_W", pack=False, align=None),
    "RPC_CALL_LOCAL_ADDRESS_V1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("AddressFormat", SimTypeInt(signed=False, label="RpcLocalAddressFormat")),)), name="RPC_CALL_LOCAL_ADDRESS_V1", pack=False, align=None),
    "RPC_CLIENT_ALLOC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Size"]), offset=0),
    "RPC_CLIENT_FREE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Ptr"]), offset=0),
    "RPC_CLIENT_INFORMATION1": SimStruct(OrderedDict((("UserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ComputerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Privilege", SimTypeShort(signed=False, label="UInt16")), ("AuthFlags", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_CLIENT_INFORMATION1", pack=False, align=None),
    "RPC_CLIENT_INTERFACE": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("InterfaceId", SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct)), ("TransferSyntax", SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct)), ("DispatchTable", SimTypePointer(SimTypeRef("RPC_DISPATCH_TABLE", SimStruct), offset=0)), ("RpcProtseqEndpointCount", SimTypeInt(signed=False, label="UInt32")), ("RpcProtseqEndpoint", SimTypePointer(SimTypeRef("RPC_PROTSEQ_ENDPOINT", SimStruct), offset=0)), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InterpreterInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_CLIENT_INTERFACE", pack=False, align=None),
    "RPC_C_AUTHN_INFO_TYPE": SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE"),
    "RPC_C_AUTHN_LEVEL": SimTypeInt(signed=False, label="RPC_C_AUTHN_LEVEL"),
    "RPC_C_HTTP_AUTHN_TARGET": SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET"),
    "RPC_C_HTTP_FLAGS": SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS"),
    "RPC_C_IMP_LEVEL": SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL"),
    "RPC_C_OPT_COOKIE_AUTH_DESCRIPTOR": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RPC_C_OPT_COOKIE_AUTH_DESCRIPTOR", pack=False, align=None),
    "RPC_C_QOS_CAPABILITIES": SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES"),
    "RPC_C_QOS_IDENTITY": SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY"),
    "RPC_DISPATCH_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RPC_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Message"]), offset=0),
    "RPC_DISPATCH_TABLE": SimStruct(OrderedDict((("DispatchTableCount", SimTypeInt(signed=False, label="UInt32")), ("DispatchTable", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RPC_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["Message"]), offset=0)), ("Reserved", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RPC_DISPATCH_TABLE", pack=False, align=None),
    "RPC_EE_INFO_PARAM": SimStruct(OrderedDict((("ParameterType", SimTypeInt(signed=False, label="ExtendedErrorParamTypes")), ("u", SimUnion({"AnsiString": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "UnicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "LVal": SimTypeInt(signed=True, label="Int32"), "SVal": SimTypeShort(signed=True, label="Int16"), "PVal": SimTypeLongLong(signed=False, label="UInt64"), "BVal": SimTypeRef("BinaryParam", SimStruct)}, name="<anon>", label="None")),)), name="RPC_EE_INFO_PARAM", pack=False, align=None),
    "RPC_ENDPOINT_TEMPLATEA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ProtSeq", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Endpoint", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Backlog", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_ENDPOINT_TEMPLATEA", pack=False, align=None),
    "RPC_ENDPOINT_TEMPLATEW": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ProtSeq", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Endpoint", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Backlog", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_ENDPOINT_TEMPLATEW", pack=False, align=None),
    "RPC_ERROR_ENUM_HANDLE": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("CurrentPos", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Head", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_ERROR_ENUM_HANDLE", pack=False, align=None),
    "RPC_EXTENDED_ERROR_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ComputerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProcessID", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"SystemTime": SimTypeRef("SYSTEMTIME", SimStruct), "FileTime": SimTypeRef("FILETIME", SimStruct)}, name="<anon>", label="None")), ("GeneratingComponent", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("DetectionLocation", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("NumberOfParameters", SimTypeInt(signed=True, label="Int32")), ("Parameters", SimTypeArray(SimTypeRef("RPC_EE_INFO_PARAM", SimStruct), 4)),)), name="RPC_EXTENDED_ERROR_INFO", pack=False, align=None),
    "RPC_FORWARD_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("RPC_VERSION", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InterfaceId", "InterfaceVersion", "ObjectId", "Rpcpro", "ppDestEndpoint"]), offset=0),
    "RPC_HTTP_PROXY_FREE_STRING": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["String"]), offset=0),
    "RPC_HTTP_REDIRECTOR_STAGE": SimTypeInt(signed=False, label="RPC_HTTP_REDIRECTOR_STAGE"),
    "RPC_HTTP_TRANSPORT_CREDENTIALS_A": SimStruct(OrderedDict((("TransportCredentials", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_A", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS")), ("AuthenticationTarget", SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET")), ("NumberOfAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("AuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ServerCertificateSubject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RPC_HTTP_TRANSPORT_CREDENTIALS_A", pack=False, align=None),
    "RPC_HTTP_TRANSPORT_CREDENTIALS_V2_A": SimStruct(OrderedDict((("TransportCredentials", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_A", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS")), ("AuthenticationTarget", SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET")), ("NumberOfAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("AuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ServerCertificateSubject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ProxyCredentials", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_A", SimStruct), offset=0)), ("NumberOfProxyAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("ProxyAuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RPC_HTTP_TRANSPORT_CREDENTIALS_V2_A", pack=False, align=None),
    "RPC_HTTP_TRANSPORT_CREDENTIALS_V2_W": SimStruct(OrderedDict((("TransportCredentials", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_W", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS")), ("AuthenticationTarget", SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET")), ("NumberOfAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("AuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ServerCertificateSubject", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ProxyCredentials", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_W", SimStruct), offset=0)), ("NumberOfProxyAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("ProxyAuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RPC_HTTP_TRANSPORT_CREDENTIALS_V2_W", pack=False, align=None),
    "RPC_HTTP_TRANSPORT_CREDENTIALS_V3_A": SimStruct(OrderedDict((("TransportCredentials", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS")), ("AuthenticationTarget", SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET")), ("NumberOfAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("AuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ServerCertificateSubject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ProxyCredentials", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfProxyAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("ProxyAuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RPC_HTTP_TRANSPORT_CREDENTIALS_V3_A", pack=False, align=None),
    "RPC_HTTP_TRANSPORT_CREDENTIALS_V3_W": SimStruct(OrderedDict((("TransportCredentials", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS")), ("AuthenticationTarget", SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET")), ("NumberOfAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("AuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ServerCertificateSubject", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ProxyCredentials", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfProxyAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("ProxyAuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RPC_HTTP_TRANSPORT_CREDENTIALS_V3_W", pack=False, align=None),
    "RPC_HTTP_TRANSPORT_CREDENTIALS_W": SimStruct(OrderedDict((("TransportCredentials", SimTypePointer(SimTypeRef("SEC_WINNT_AUTH_IDENTITY_W", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="RPC_C_HTTP_FLAGS")), ("AuthenticationTarget", SimTypeInt(signed=False, label="RPC_C_HTTP_AUTHN_TARGET")), ("NumberOfAuthnSchemes", SimTypeInt(signed=False, label="UInt32")), ("AuthnSchemes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("ServerCertificateSubject", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="RPC_HTTP_TRANSPORT_CREDENTIALS_W", pack=False, align=None),
    "RPC_IF_CALLBACK_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InterfaceUuid", "Context"]), offset=0),
    "RPC_IF_ID": SimStruct(OrderedDict((("Uuid", SimTypeBottom(label="Guid")), ("VersMajor", SimTypeShort(signed=False, label="UInt16")), ("VersMinor", SimTypeShort(signed=False, label="UInt16")),)), name="RPC_IF_ID", pack=False, align=None),
    "RPC_IF_ID_VECTOR": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("IfId", SimTypePointer(SimTypePointer(SimTypeRef("RPC_IF_ID", SimStruct), offset=0), offset=0)),)), name="RPC_IF_ID_VECTOR", pack=False, align=None),
    "RPC_IMPORT_CONTEXT_P": SimStruct(OrderedDict((("LookupContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ProposedHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Bindings", SimTypePointer(SimTypeRef("RPC_BINDING_VECTOR", SimStruct), offset=0)),)), name="RPC_IMPORT_CONTEXT_P", pack=False, align=None),
    "RPC_INTERFACE_GROUP_IDLE_CALLBACK_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["IfGroup", "IdleCallbackContext", "IsGroupIdle"]), offset=0),
    "RPC_INTERFACE_TEMPLATEA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("IfSpec", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MgrTypeUuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("MgrEpv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxCalls", SimTypeInt(signed=False, label="UInt32")), ("MaxRpcSize", SimTypeInt(signed=False, label="UInt32")), ("IfCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InterfaceUuid", "Context"]), offset=0)), ("UuidVector", SimTypePointer(SimTypeRef("UUID_VECTOR", SimStruct), offset=0)), ("Annotation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_INTERFACE_TEMPLATEA", pack=False, align=None),
    "RPC_INTERFACE_TEMPLATEW": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("IfSpec", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MgrTypeUuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("MgrEpv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxCalls", SimTypeInt(signed=False, label="UInt32")), ("MaxRpcSize", SimTypeInt(signed=False, label="UInt32")), ("IfCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InterfaceUuid", "Context"]), offset=0)), ("UuidVector", SimTypePointer(SimTypeRef("UUID_VECTOR", SimStruct), offset=0)), ("Annotation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_INTERFACE_TEMPLATEW", pack=False, align=None),
    "RPC_MESSAGE": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DataRepresentation", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferLength", SimTypeInt(signed=False, label="UInt32")), ("ProcNum", SimTypeInt(signed=False, label="UInt32")), ("TransferSyntax", SimTypePointer(SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct), offset=0)), ("RpcInterfaceInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ReservedForRuntime", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ManagerEpv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ImportContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RpcFlags", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_MESSAGE", pack=False, align=None),
    "RPC_MGMT_AUTHORIZATION_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="RPC_STATUS"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientBinding", "RequestedMgmtOperation", "Status"]), offset=0),
    "RPC_NEW_HTTP_PROXY_CHANNEL": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="RPC_HTTP_REDIRECTOR_STAGE"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["RedirectorStage", "ServerName", "ServerPort", "RemoteUser", "AuthType", "ResourceUuid", "SessionId", "Interface", "Reserved", "Flags", "NewServerName", "NewServerPort"]), offset=0),
    "RPC_NOTIFICATIONS": SimTypeInt(signed=False, label="RPC_NOTIFICATIONS"),
    "RPC_NOTIFICATION_TYPES": SimTypeInt(signed=False, label="RPC_NOTIFICATION_TYPES"),
    "RPC_OBJECT_INQ_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="RPC_STATUS"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ObjectUuid", "TypeUuid", "Status"]), offset=0),
    "RPC_POLICY": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("EndpointFlags", SimTypeInt(signed=False, label="UInt32")), ("NICFlags", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_POLICY", pack=False, align=None),
    "RPC_PROTSEQ_ENDPOINT": SimStruct(OrderedDict((("RpcProtocolSequence", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Endpoint", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RPC_PROTSEQ_ENDPOINT", pack=False, align=None),
    "RPC_PROTSEQ_VECTORA": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Protseq", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="RPC_PROTSEQ_VECTORA", pack=False, align=None),
    "RPC_PROTSEQ_VECTORW": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Protseq", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)),)), name="RPC_PROTSEQ_VECTORW", pack=False, align=None),
    "RPC_SECURITY_CALLBACK_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "RPC_SECURITY_QOS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")),)), name="RPC_SECURITY_QOS", pack=False, align=None),
    "RPC_SECURITY_QOS_V2_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_A", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="RPC_SECURITY_QOS_V2_A", pack=False, align=None),
    "RPC_SECURITY_QOS_V2_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_W", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="RPC_SECURITY_QOS_V2_W", pack=False, align=None),
    "RPC_SECURITY_QOS_V3_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_A", SimStruct), offset=0)}, name="<anon>", label="None")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_SECURITY_QOS_V3_A", pack=False, align=None),
    "RPC_SECURITY_QOS_V3_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_W", SimStruct), offset=0)}, name="<anon>", label="None")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_SECURITY_QOS_V3_W", pack=False, align=None),
    "RPC_SECURITY_QOS_V4_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_A", SimStruct), offset=0)}, name="<anon>", label="None")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EffectiveOnly", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_SECURITY_QOS_V4_A", pack=False, align=None),
    "RPC_SECURITY_QOS_V4_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_W", SimStruct), offset=0)}, name="<anon>", label="None")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EffectiveOnly", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_SECURITY_QOS_V4_W", pack=False, align=None),
    "RPC_SECURITY_QOS_V5_A": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_A", SimStruct), offset=0)}, name="<anon>", label="None")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EffectiveOnly", SimTypeInt(signed=False, label="UInt32")), ("ServerSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_SECURITY_QOS_V5_A", pack=False, align=None),
    "RPC_SECURITY_QOS_V5_W": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="RPC_C_QOS_CAPABILITIES")), ("IdentityTracking", SimTypeInt(signed=False, label="RPC_C_QOS_IDENTITY")), ("ImpersonationType", SimTypeInt(signed=False, label="RPC_C_IMP_LEVEL")), ("AdditionalSecurityInfoType", SimTypeInt(signed=False, label="RPC_C_AUTHN_INFO_TYPE")), ("u", SimUnion({"HttpCredentials": SimTypePointer(SimTypeRef("RPC_HTTP_TRANSPORT_CREDENTIALS_W", SimStruct), offset=0)}, name="<anon>", label="None")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EffectiveOnly", SimTypeInt(signed=False, label="UInt32")), ("ServerSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RPC_SECURITY_QOS_V5_W", pack=False, align=None),
    "RPC_SEC_CONTEXT_KEY_INFO": SimStruct(OrderedDict((("EncryptAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("KeySize", SimTypeInt(signed=False, label="UInt32")), ("SignatureAlgorithm", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_SEC_CONTEXT_KEY_INFO", pack=False, align=None),
    "RPC_SERVER_INTERFACE": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("InterfaceId", SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct)), ("TransferSyntax", SimTypeRef("RPC_SYNTAX_IDENTIFIER", SimStruct)), ("DispatchTable", SimTypePointer(SimTypeRef("RPC_DISPATCH_TABLE", SimStruct), offset=0)), ("RpcProtseqEndpointCount", SimTypeInt(signed=False, label="UInt32")), ("RpcProtseqEndpoint", SimTypePointer(SimTypeRef("RPC_PROTSEQ_ENDPOINT", SimStruct), offset=0)), ("DefaultManagerEpv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterpreterInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="RPC_SERVER_INTERFACE", pack=False, align=None),
    "RPC_SETFILTER_FUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["Buffer", "BufferLength", "fDatagram"]), offset=0)], SimTypeBottom(label="Void"), arg_names=["pfnFilter"]), offset=0),
    "RPC_STATS_VECTOR": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Stats", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RPC_STATS_VECTOR", pack=False, align=None),
    "RPC_STATUS": SimTypeInt(signed=False, label="RPC_STATUS"),
    "RPC_SYNTAX_IDENTIFIER": SimStruct(OrderedDict((("SyntaxGUID", SimTypeBottom(label="Guid")), ("SyntaxVersion", SimTypeRef("RPC_VERSION", SimStruct)),)), name="RPC_SYNTAX_IDENTIFIER", pack=False, align=None),
    "RPC_TRANSFER_SYNTAX": SimStruct(OrderedDict((("Uuid", SimTypeBottom(label="Guid")), ("VersMajor", SimTypeShort(signed=False, label="UInt16")), ("VersMinor", SimTypeShort(signed=False, label="UInt16")),)), name="RPC_TRANSFER_SYNTAX", pack=False, align=None),
    "RPC_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="RPC_VERSION", pack=False, align=None),
    "RSAPUBKEY": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("bitlen", SimTypeInt(signed=False, label="UInt32")), ("pubexp", SimTypeInt(signed=False, label="UInt32")),)), name="RSAPUBKEY", pack=False, align=None),
    "RSOP_TARGET": SimStruct(OrderedDict((("pwszAccountName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszNewSOM", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("psaSecurityGroups", SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0)), ("pRsopToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pGPOList", SimTypePointer(SimTypeRef("GROUP_POLICY_OBJECTA", SimStruct), offset=0)), ("pWbemServices", SimTypeBottom(label="IWbemServices")),)), name="RSOP_TARGET", pack=False, align=None),
    "RSS_SCALABILITY_INFO": SimStruct(OrderedDict((("RssEnabled", SimTypeChar(label="Byte")),)), name="RSS_SCALABILITY_INFO", pack=False, align=None),
    "RSVP_ADSPEC": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("GeneralParams", SimTypeRef("AD_GENERAL_PARAMS", SimStruct)), ("NumberOfServices", SimTypeInt(signed=False, label="UInt32")), ("Services", SimTypePointer(SimTypeRef("CONTROL_SERVICE", SimStruct), offset=0)),)), name="RSVP_ADSPEC", pack=False, align=None),
    "RSVP_FILTERSPEC": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="FilterType")), ("Anonymous", SimUnion({"FilterSpecV4": SimTypeRef("RSVP_FILTERSPEC_V4", SimStruct), "FilterSpecV6": SimTypeRef("RSVP_FILTERSPEC_V6", SimStruct), "FilterSpecV6Flow": SimTypeRef("RSVP_FILTERSPEC_V6_FLOW", SimStruct), "FilterSpecV4Gpi": SimTypeRef("RSVP_FILTERSPEC_V4_GPI", SimStruct), "FilterSpecV6Gpi": SimTypeRef("RSVP_FILTERSPEC_V6_GPI", SimStruct)}, name="<anon>", label="None")),)), name="RSVP_FILTERSPEC", pack=False, align=None),
    "RSVP_FILTERSPEC_V4": SimStruct(OrderedDict((("Address", SimUnion({"Addr": SimTypeInt(signed=False, label="UInt32"), "AddrBytes": SimTypeArray(SimTypeChar(label="Byte"), 4)}, name="<anon>", label="None")), ("Unused", SimTypeShort(signed=False, label="UInt16")), ("Port", SimTypeShort(signed=False, label="UInt16")),)), name="RSVP_FILTERSPEC_V4", pack=False, align=None),
    "RSVP_FILTERSPEC_V4_GPI": SimStruct(OrderedDict((("Address", SimUnion({"Addr": SimTypeInt(signed=False, label="UInt32"), "AddrBytes": SimTypeArray(SimTypeChar(label="Byte"), 4)}, name="<anon>", label="None")), ("GeneralPortId", SimTypeInt(signed=False, label="UInt32")),)), name="RSVP_FILTERSPEC_V4_GPI", pack=False, align=None),
    "RSVP_FILTERSPEC_V6": SimStruct(OrderedDict((("Address", SimTypeRef("IN_ADDR_IPV6", SimStruct)), ("UnUsed", SimTypeShort(signed=False, label="UInt16")), ("Port", SimTypeShort(signed=False, label="UInt16")),)), name="RSVP_FILTERSPEC_V6", pack=False, align=None),
    "RSVP_FILTERSPEC_V6_FLOW": SimStruct(OrderedDict((("Address", SimTypeRef("IN_ADDR_IPV6", SimStruct)), ("UnUsed", SimTypeChar(label="Byte")), ("FlowLabel", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="RSVP_FILTERSPEC_V6_FLOW", pack=False, align=None),
    "RSVP_FILTERSPEC_V6_GPI": SimStruct(OrderedDict((("Address", SimTypeRef("IN_ADDR_IPV6", SimStruct)), ("GeneralPortId", SimTypeInt(signed=False, label="UInt32")),)), name="RSVP_FILTERSPEC_V6_GPI", pack=False, align=None),
    "RSVP_HOP": SimStruct(OrderedDict((("hop_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("hop_u", SimUnion({"hop_ipv4": SimTypeRef("Rsvp_Hop_IPv4", SimStruct)}, name="<anon>", label="None")),)), name="RSVP_HOP", pack=False, align=None),
    "RSVP_MSG_OBJS": SimStruct(OrderedDict((("RsvpMsgType", SimTypeInt(signed=True, label="Int32")), ("pRsvpSession", SimTypePointer(SimTypeRef("RSVP_SESSION", SimStruct), offset=0)), ("pRsvpFromHop", SimTypePointer(SimTypeRef("RSVP_HOP", SimStruct), offset=0)), ("pRsvpToHop", SimTypePointer(SimTypeRef("RSVP_HOP", SimStruct), offset=0)), ("pResvStyle", SimTypePointer(SimTypeRef("RESV_STYLE", SimStruct), offset=0)), ("pRsvpScope", SimTypePointer(SimTypeRef("RSVP_SCOPE", SimStruct), offset=0)), ("FlowDescCount", SimTypeInt(signed=True, label="Int32")), ("pFlowDescs", SimTypePointer(SimTypeRef("FLOW_DESC", SimStruct), offset=0)), ("PdObjectCount", SimTypeInt(signed=True, label="Int32")), ("ppPdObjects", SimTypePointer(SimTypePointer(SimTypeRef("POLICY_DATA", SimStruct), offset=0), offset=0)), ("pErrorSpec", SimTypePointer(SimTypeRef("ERROR_SPEC", SimStruct), offset=0)), ("pAdspec", SimTypePointer(SimTypeRef("ADSPEC", SimStruct), offset=0)),)), name="RSVP_MSG_OBJS", pack=False, align=None),
    "RSVP_POLICY": SimStruct(OrderedDict((("Len", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeShort(signed=False, label="UInt16")), ("Info", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="RSVP_POLICY", pack=False, align=None),
    "RSVP_POLICY_INFO": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("NumPolicyElement", SimTypeInt(signed=False, label="UInt32")), ("PolicyElement", SimTypePointer(SimTypeRef("RSVP_POLICY", SimStruct), offset=0)),)), name="RSVP_POLICY_INFO", pack=False, align=None),
    "RSVP_RESERVE_INFO": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("Style", SimTypeInt(signed=False, label="UInt32")), ("ConfirmRequest", SimTypeInt(signed=False, label="UInt32")), ("PolicyElementList", SimTypePointer(SimTypeRef("RSVP_POLICY_INFO", SimStruct), offset=0)), ("NumFlowDesc", SimTypeInt(signed=False, label="UInt32")), ("FlowDescList", SimTypePointer(SimTypeRef("FLOWDESCRIPTOR", SimStruct), offset=0)),)), name="RSVP_RESERVE_INFO", pack=False, align=None),
    "RSVP_SCOPE": SimStruct(OrderedDict((("scopl_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("scope_u", SimUnion({"scopl_ipv4": SimTypeRef("Scope_list_ipv4", SimStruct)}, name="<anon>", label="None")),)), name="RSVP_SCOPE", pack=False, align=None),
    "RSVP_SESSION": SimStruct(OrderedDict((("sess_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("sess_u", SimUnion({"sess_ipv4": SimTypeRef("Session_IPv4", SimStruct)}, name="<anon>", label="None")),)), name="RSVP_SESSION", pack=False, align=None),
    "RSVP_STATUS_INFO": SimStruct(OrderedDict((("ObjectHdr", SimTypeRef("QOS_OBJECT_HDR", SimStruct)), ("StatusCode", SimTypeInt(signed=False, label="UInt32")), ("ExtendedStatus1", SimTypeInt(signed=False, label="UInt32")), ("ExtendedStatus2", SimTypeInt(signed=False, label="UInt32")),)), name="RSVP_STATUS_INFO", pack=False, align=None),
    "RTC_ACE_SCOPE": SimTypeInt(signed=False, label="RTC_ACE_SCOPE"),
    "RTC_ANSWER_MODE": SimTypeInt(signed=False, label="RTC_ANSWER_MODE"),
    "RTC_AUDIO_DEVICE": SimTypeInt(signed=False, label="RTC_AUDIO_DEVICE"),
    "RTC_BUDDY_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_BUDDY_EVENT_TYPE"),
    "RTC_BUDDY_SUBSCRIPTION_TYPE": SimTypeInt(signed=False, label="RTC_BUDDY_SUBSCRIPTION_TYPE"),
    "RTC_CLIENT_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_CLIENT_EVENT_TYPE"),
    "RTC_DTMF": SimTypeInt(signed=False, label="RTC_DTMF"),
    "RTC_EVENT": SimTypeInt(signed=False, label="RTC_EVENT"),
    "RTC_GROUP_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_GROUP_EVENT_TYPE"),
    "RTC_LISTEN_MODE": SimTypeInt(signed=False, label="RTC_LISTEN_MODE"),
    "RTC_MEDIA_EVENT_REASON": SimTypeInt(signed=False, label="RTC_MEDIA_EVENT_REASON"),
    "RTC_MEDIA_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_MEDIA_EVENT_TYPE"),
    "RTC_MESSAGING_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_MESSAGING_EVENT_TYPE"),
    "RTC_MESSAGING_USER_STATUS": SimTypeInt(signed=False, label="RTC_MESSAGING_USER_STATUS"),
    "RTC_OFFER_WATCHER_MODE": SimTypeInt(signed=False, label="RTC_OFFER_WATCHER_MODE"),
    "RTC_PARTICIPANT_STATE": SimTypeInt(signed=False, label="RTC_PARTICIPANT_STATE"),
    "RTC_PORT_TYPE": SimTypeInt(signed=False, label="RTC_PORT_TYPE"),
    "RTC_PRESENCE_PROPERTY": SimTypeInt(signed=False, label="RTC_PRESENCE_PROPERTY"),
    "RTC_PRESENCE_STATUS": SimTypeInt(signed=False, label="RTC_PRESENCE_STATUS"),
    "RTC_PRIVACY_MODE": SimTypeInt(signed=False, label="RTC_PRIVACY_MODE"),
    "RTC_PROFILE_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_PROFILE_EVENT_TYPE"),
    "RTC_PROVIDER_URI": SimTypeInt(signed=False, label="RTC_PROVIDER_URI"),
    "RTC_REGISTRATION_STATE": SimTypeInt(signed=False, label="RTC_REGISTRATION_STATE"),
    "RTC_REINVITE_STATE": SimTypeInt(signed=False, label="RTC_REINVITE_STATE"),
    "RTC_RING_TYPE": SimTypeInt(signed=False, label="RTC_RING_TYPE"),
    "RTC_ROAMING_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_ROAMING_EVENT_TYPE"),
    "RTC_SECURITY_LEVEL": SimTypeInt(signed=False, label="RTC_SECURITY_LEVEL"),
    "RTC_SECURITY_TYPE": SimTypeInt(signed=False, label="RTC_SECURITY_TYPE"),
    "RTC_SESSION_REFER_STATUS": SimTypeInt(signed=False, label="RTC_SESSION_REFER_STATUS"),
    "RTC_SESSION_STATE": SimTypeInt(signed=False, label="RTC_SESSION_STATE"),
    "RTC_SESSION_TYPE": SimTypeInt(signed=False, label="RTC_SESSION_TYPE"),
    "RTC_T120_APPLET": SimTypeInt(signed=False, label="RTC_T120_APPLET"),
    "RTC_TERMINATE_REASON": SimTypeInt(signed=False, label="RTC_TERMINATE_REASON"),
    "RTC_USER_SEARCH_COLUMN": SimTypeInt(signed=False, label="RTC_USER_SEARCH_COLUMN"),
    "RTC_USER_SEARCH_PREFERENCE": SimTypeInt(signed=False, label="RTC_USER_SEARCH_PREFERENCE"),
    "RTC_VIDEO_DEVICE": SimTypeInt(signed=False, label="RTC_VIDEO_DEVICE"),
    "RTC_WATCHER_EVENT_TYPE": SimTypeInt(signed=False, label="RTC_WATCHER_EVENT_TYPE"),
    "RTC_WATCHER_MATCH_MODE": SimTypeInt(signed=False, label="RTC_WATCHER_MATCH_MODE"),
    "RTC_WATCHER_STATE": SimTypeInt(signed=False, label="RTC_WATCHER_STATE"),
    "RTL_ALLOCATE_STRING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["NumberOfBytes"]), offset=0),
    "RTL_AVL_ALLOCATE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_AVL_TABLE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Table", "ByteSize"]), offset=0),
    "RTL_AVL_COMPARE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_AVL_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS"), arg_names=["Table", "FirstStruct", "SecondStruct"]), offset=0),
    "RTL_AVL_FREE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_AVL_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Table", "Buffer"]), offset=0),
    "RTL_AVL_MATCH_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_AVL_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Table", "UserData", "MatchData"]), offset=0),
    "RTL_AVL_TABLE": SimStruct(OrderedDict((("BalancedRoot", SimTypeRef("RTL_BALANCED_LINKS", SimStruct)), ("OrderedPointer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("WhichOrderedElement", SimTypeInt(signed=False, label="UInt32")), ("NumberGenericTableElements", SimTypeInt(signed=False, label="UInt32")), ("DepthOfTree", SimTypeInt(signed=False, label="UInt32")), ("RestartKey", SimTypePointer(SimTypeRef("RTL_BALANCED_LINKS", SimStruct), offset=0)), ("DeleteCount", SimTypeInt(signed=False, label="UInt32")), ("CompareRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS")), offset=0)), ("AllocateRoutine", SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0)), ("FreeRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("TableContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RTL_AVL_TABLE", pack=False, align=None),
    "RTL_BALANCED_LINKS": SimStruct(OrderedDict((("Parent", SimTypePointer(SimTypeRef("RTL_BALANCED_LINKS", SimStruct), offset=0)), ("LeftChild", SimTypePointer(SimTypeRef("RTL_BALANCED_LINKS", SimStruct), offset=0)), ("RightChild", SimTypePointer(SimTypeRef("RTL_BALANCED_LINKS", SimStruct), offset=0)), ("Balance", SimTypeChar(label="SByte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="RTL_BALANCED_LINKS", pack=False, align=None),
    "RTL_BALANCED_NODE": SimStruct(OrderedDict((("Anonymous1", SimUnion({"Children": SimTypeArray(SimTypePointer(SimTypeRef("RTL_BALANCED_NODE", SimStruct), offset=0), 2), "Anonymous": SimStruct(OrderedDict((("Left", SimTypePointer(SimTypeRef("RTL_BALANCED_NODE", SimStruct), offset=0)), ("Right", SimTypePointer(SimTypeRef("RTL_BALANCED_NODE", SimStruct), offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"_bitfield": SimTypeChar(label="Byte"), "ParentValue": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")),)), name="RTL_BALANCED_NODE", pack=False, align=None),
    "RTL_BITMAP": SimStruct(OrderedDict((("SizeOfBitMap", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="RTL_BITMAP", pack=False, align=None),
    "RTL_BITMAP_RUN": SimStruct(OrderedDict((("StartingIndex", SimTypeInt(signed=False, label="UInt32")), ("NumberOfBits", SimTypeInt(signed=False, label="UInt32")),)), name="RTL_BITMAP_RUN", pack=False, align=None),
    "RTL_DYNAMIC_HASH_TABLE": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Shift", SimTypeInt(signed=False, label="UInt32")), ("TableSize", SimTypeInt(signed=False, label="UInt32")), ("Pivot", SimTypeInt(signed=False, label="UInt32")), ("DivisorMask", SimTypeInt(signed=False, label="UInt32")), ("NumEntries", SimTypeInt(signed=False, label="UInt32")), ("NonEmptyBuckets", SimTypeInt(signed=False, label="UInt32")), ("NumEnumerators", SimTypeInt(signed=False, label="UInt32")), ("Directory", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RTL_DYNAMIC_HASH_TABLE", pack=False, align=None),
    "RTL_DYNAMIC_HASH_TABLE_CONTEXT": SimStruct(OrderedDict((("ChainHead", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("PrevLinkage", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("Signature", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RTL_DYNAMIC_HASH_TABLE_CONTEXT", pack=False, align=None),
    "RTL_DYNAMIC_HASH_TABLE_ENTRY": SimStruct(OrderedDict((("Linkage", SimTypeRef("LIST_ENTRY", SimStruct)), ("Signature", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RTL_DYNAMIC_HASH_TABLE_ENTRY", pack=False, align=None),
    "RTL_DYNAMIC_HASH_TABLE_ENUMERATOR": SimStruct(OrderedDict((("Anonymous", SimUnion({"HashEntry": SimTypeRef("RTL_DYNAMIC_HASH_TABLE_ENTRY", SimStruct), "CurEntry": SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)}, name="<anon>", label="None")), ("ChainHead", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("BucketIndex", SimTypeInt(signed=False, label="UInt32")),)), name="RTL_DYNAMIC_HASH_TABLE_ENUMERATOR", pack=False, align=None),
    "RTL_FREE_STRING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Buffer"]), offset=0),
    "RTL_GENERIC_ALLOCATE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_GENERIC_TABLE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Table", "ByteSize"]), offset=0),
    "RTL_GENERIC_COMPARE_RESULTS": SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS"),
    "RTL_GENERIC_COMPARE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_GENERIC_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS"), arg_names=["Table", "FirstStruct", "SecondStruct"]), offset=0),
    "RTL_GENERIC_FREE_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RTL_GENERIC_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Table", "Buffer"]), offset=0),
    "RTL_GENERIC_TABLE": SimStruct(OrderedDict((("TableRoot", SimTypePointer(SimTypeRef("RTL_SPLAY_LINKS", SimStruct), offset=0)), ("InsertOrderList", SimTypeRef("LIST_ENTRY", SimStruct)), ("OrderedPointer", SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0)), ("WhichOrderedElement", SimTypeInt(signed=False, label="UInt32")), ("NumberGenericTableElements", SimTypeInt(signed=False, label="UInt32")), ("CompareRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="RTL_GENERIC_COMPARE_RESULTS")), offset=0)), ("AllocateRoutine", SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeBottom(label="Void"), offset=0)), offset=0)), ("FreeRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("TableContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="RTL_GENERIC_TABLE", pack=False, align=None),
    "RTL_HEAP_COMMIT_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Base", "CommitAddress", "CommitSize"]), offset=0),
    "RTL_HEAP_MEMORY_LIMIT_DATA": SimStruct(OrderedDict((("CommitLimitBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CommitLimitFailureCode", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaxAllocationSizeBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AllocationLimitFailureCode", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="RTL_HEAP_MEMORY_LIMIT_DATA", pack=False, align=None),
    "RTL_HEAP_MEMORY_LIMIT_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeRef("RTL_HEAP_MEMORY_LIMIT_DATA", SimStruct)),)), name="RTL_HEAP_MEMORY_LIMIT_INFO", pack=False, align=None),
    "RTL_HEAP_PARAMETERS": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("SegmentReserve", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SegmentCommit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DeCommitFreeBlockThreshold", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DeCommitTotalFreeThreshold", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaximumAllocationSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VirtualMemoryThreshold", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InitialCommit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("InitialReserve", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CommitRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)),)), name="RTL_HEAP_PARAMETERS", pack=False, align=None),
    "RTL_MEMORY_TYPE": SimTypeInt(signed=False, label="RTL_MEMORY_TYPE"),
    "RTL_NLS_STATE": SimStruct(OrderedDict((("DefaultAcpTableInfo", SimTypeRef("CPTABLEINFO", SimStruct)), ("DefaultOemTableInfo", SimTypeRef("CPTABLEINFO", SimStruct)), ("ActiveCodePageData", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("OemCodePageData", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("LeadByteInfo", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("OemLeadByteInfo", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("CaseMappingData", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("UnicodeUpcaseTable844", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("UnicodeLowercaseTable844", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="RTL_NLS_STATE", pack=False, align=None),
    "RTL_QUERY_REGISTRY_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ValueName", "ValueType", "ValueData", "ValueLength", "Context", "EntryContext"]), offset=0),
    "RTL_QUERY_REGISTRY_TABLE": SimStruct(OrderedDict((("QueryRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("EntryContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DefaultType", SimTypeInt(signed=False, label="UInt32")), ("DefaultData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DefaultLength", SimTypeInt(signed=False, label="UInt32")),)), name="RTL_QUERY_REGISTRY_TABLE", pack=False, align=None),
    "RTL_REALLOCATE_STRING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["NumberOfBytes", "Buffer"]), offset=0),
    "RTL_RUN_ONCE_INIT_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimUnion({"Ptr": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RunOnce", "Parameter", "Context"]), offset=0),
    "RTL_SEGMENT_HEAP_MEMORY_SOURCE": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("MemoryTypeMask", SimTypeInt(signed=False, label="UInt32")), ("NumaNode", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"PartitionHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Callbacks": SimTypePointer(SimTypeRef("RTL_SEGMENT_HEAP_VA_CALLBACKS", SimStruct), offset=0)}, name="<anon>", label="None")), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)),)), name="RTL_SEGMENT_HEAP_MEMORY_SOURCE", pack=False, align=None),
    "RTL_SEGMENT_HEAP_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MemorySource", SimTypeRef("RTL_SEGMENT_HEAP_MEMORY_SOURCE", SimStruct)), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 4)),)), name="RTL_SEGMENT_HEAP_PARAMETERS", pack=False, align=None),
    "RTL_SEGMENT_HEAP_VA_CALLBACKS": SimStruct(OrderedDict((("CallbackContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AllocateVirtualMemory", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("FreeVirtualMemory", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("QueryVirtualMemory", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)),)), name="RTL_SEGMENT_HEAP_VA_CALLBACKS", pack=False, align=None),
    "RTL_SPLAY_LINKS": SimStruct(OrderedDict((("Parent", SimTypePointer(SimTypeRef("RTL_SPLAY_LINKS", SimStruct), offset=0)), ("LeftChild", SimTypePointer(SimTypeRef("RTL_SPLAY_LINKS", SimStruct), offset=0)), ("RightChild", SimTypePointer(SimTypeRef("RTL_SPLAY_LINKS", SimStruct), offset=0)),)), name="RTL_SPLAY_LINKS", pack=False, align=None),
    "RTL_SYSTEM_GLOBAL_DATA_ID": SimTypeInt(signed=False, label="RTL_SYSTEM_GLOBAL_DATA_ID"),
    "RTL_UMS_SCHEDULER_REASON": SimTypeInt(signed=False, label="RTL_UMS_SCHEDULER_REASON"),
    "RTL_USER_PROCESS_PARAMETERS": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Reserved2", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 10)), ("ImagePathName", SimTypeRef("UNICODE_STRING", SimStruct)), ("CommandLine", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="RTL_USER_PROCESS_PARAMETERS", pack=False, align=None),
    "RTL_VIRTUAL_UNWIND_HANDLER_TYPE": SimTypeInt(signed=False, label="RTL_VIRTUAL_UNWIND_HANDLER_TYPE"),
    "RTM_DEST_INFO": SimStruct(OrderedDict((("DestHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DestAddress", SimTypeRef("RTM_NET_ADDRESS", SimStruct)), ("LastChanged", SimTypeRef("FILETIME", SimStruct)), ("BelongsToViews", SimTypeInt(signed=False, label="UInt32")), ("NumberOfViews", SimTypeInt(signed=False, label="UInt32")), ("ViewInfo", SimTypePointer(SimStruct(OrderedDict((("ViewId", SimTypeInt(signed=True, label="Int32")), ("NumRoutes", SimTypeInt(signed=False, label="UInt32")), ("Route", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Owner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DestFlags", SimTypeInt(signed=False, label="UInt32")), ("HoldRoute", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="RTM_DEST_INFO", pack=False, align=None),
    "RTM_ENTITY_EXPORT_METHOD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RTM_ENTITY_METHOD_INPUT", SimStruct), offset=0), SimTypePointer(SimTypeRef("RTM_ENTITY_METHOD_OUTPUT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallerHandle", "CalleeHandle", "Input", "Output"]), offset=0),
    "RTM_ENTITY_EXPORT_METHODS": SimStruct(OrderedDict((("NumMethods", SimTypeInt(signed=False, label="UInt32")), ("Methods", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RTM_ENTITY_METHOD_INPUT", SimStruct), offset=0), SimTypePointer(SimTypeRef("RTM_ENTITY_METHOD_OUTPUT", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallerHandle", "CalleeHandle", "Input", "Output"]), offset=0), offset=0)),)), name="RTM_ENTITY_EXPORT_METHODS", pack=False, align=None),
    "RTM_ENTITY_ID": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("EntityProtocolId", SimTypeInt(signed=False, label="UInt32")), ("EntityInstanceId", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "EntityId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="RTM_ENTITY_ID", pack=False, align=None),
    "RTM_ENTITY_INFO": SimStruct(OrderedDict((("RtmInstanceId", SimTypeShort(signed=False, label="UInt16")), ("AddressFamily", SimTypeShort(signed=False, label="UInt16")), ("EntityId", SimTypeRef("RTM_ENTITY_ID", SimStruct)),)), name="RTM_ENTITY_INFO", pack=False, align=None),
    "RTM_ENTITY_METHOD_INPUT": SimStruct(OrderedDict((("MethodType", SimTypeInt(signed=False, label="UInt32")), ("InputSize", SimTypeInt(signed=False, label="UInt32")), ("InputData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RTM_ENTITY_METHOD_INPUT", pack=False, align=None),
    "RTM_ENTITY_METHOD_OUTPUT": SimStruct(OrderedDict((("MethodType", SimTypeInt(signed=False, label="UInt32")), ("MethodStatus", SimTypeInt(signed=False, label="UInt32")), ("OutputSize", SimTypeInt(signed=False, label="UInt32")), ("OutputData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RTM_ENTITY_METHOD_OUTPUT", pack=False, align=None),
    "RTM_EVENT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="RTM_EVENT_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RtmRegHandle", "EventType", "Context1", "Context2"]), offset=0),
    "RTM_EVENT_TYPE": SimTypeInt(signed=False, label="RTM_EVENT_TYPE"),
    "RTM_NET_ADDRESS": SimStruct(OrderedDict((("AddressFamily", SimTypeShort(signed=False, label="UInt16")), ("NumBits", SimTypeShort(signed=False, label="UInt16")), ("AddrBits", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="RTM_NET_ADDRESS", pack=False, align=None),
    "RTM_NEXTHOP_INFO": SimStruct(OrderedDict((("NextHopAddress", SimTypeRef("RTM_NET_ADDRESS", SimStruct)), ("NextHopOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("EntitySpecificInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("RemoteNextHop", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="RTM_NEXTHOP_INFO", pack=False, align=None),
    "RTM_NEXTHOP_LIST": SimStruct(OrderedDict((("NumNextHops", SimTypeShort(signed=False, label="UInt16")), ("NextHops", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="RTM_NEXTHOP_LIST", pack=False, align=None),
    "RTM_PREF_INFO": SimStruct(OrderedDict((("Metric", SimTypeInt(signed=False, label="UInt32")), ("Preference", SimTypeInt(signed=False, label="UInt32")),)), name="RTM_PREF_INFO", pack=False, align=None),
    "RTM_REGN_PROFILE": SimStruct(OrderedDict((("MaxNextHopsInRoute", SimTypeInt(signed=False, label="UInt32")), ("MaxHandlesInEnum", SimTypeInt(signed=False, label="UInt32")), ("ViewsSupported", SimTypeInt(signed=False, label="UInt32")), ("NumberOfViews", SimTypeInt(signed=False, label="UInt32")),)), name="RTM_REGN_PROFILE", pack=False, align=None),
    "RTM_ROUTE_INFO": SimStruct(OrderedDict((("DestHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RouteOwner", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Neighbour", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("State", SimTypeChar(label="Byte")), ("Flags1", SimTypeChar(label="Byte")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("PrefInfo", SimTypeRef("RTM_PREF_INFO", SimStruct)), ("BelongsToViews", SimTypeInt(signed=False, label="UInt32")), ("EntitySpecificInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NextHopsList", SimTypeRef("RTM_NEXTHOP_LIST", SimStruct)),)), name="RTM_ROUTE_INFO", pack=False, align=None),
    "RTR_INFO_BLOCK_HEADER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("TocEntriesCount", SimTypeInt(signed=False, label="UInt32")), ("TocEntry", SimTypePointer(SimTypeRef("RTR_TOC_ENTRY", SimStruct), offset=0)),)), name="RTR_INFO_BLOCK_HEADER", pack=False, align=None),
    "RTR_TOC_ENTRY": SimStruct(OrderedDict((("InfoType", SimTypeInt(signed=False, label="UInt32")), ("InfoSize", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="RTR_TOC_ENTRY", pack=False, align=None),
    "RTWQASYNCRESULT": SimTypeBottom(label="RTWQASYNCRESULT"),
    "RTWQPERIODICCALLBACK": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown")], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "RTWQ_WORKQUEUE_TYPE": SimTypeInt(signed=False, label="RTWQ_WORKQUEUE_TYPE"),
    "RUN": SimStruct(OrderedDict((("iStart", SimTypeInt(signed=True, label="Int32")), ("iStop", SimTypeInt(signed=True, label="Int32")),)), name="RUN", pack=False, align=None),
    "RUNTIME_INFO_FLAGS": SimTypeInt(signed=False, label="RUNTIME_INFO_FLAGS"),
    "RasCustomDeleteEntryNotifyFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpszPhonebook", "lpszEntry", "dwFlags"]), offset=0),
    "RasCustomDialDlgFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("RASDIALDLG", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInstDll", "dwFlags", "lpszPhonebook", "lpszEntry", "lpszPhoneNumber", "lpInfo", "pvInfo"]), offset=0),
    "RasCustomDialFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RASDIALEXTENSIONS", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("RASDIALPARAMSA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hInstDll", "lpRasDialExtensions", "lpszPhonebook", "lpRasDialParams", "dwNotifierType", "lpvNotifier", "lphRasConn", "dwFlags"]), offset=0),
    "RasCustomEntryDlgFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("RASENTRYDLGA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hInstDll", "lpszPhonebook", "lpszEntry", "lpInfo", "dwFlags"]), offset=0),
    "RasCustomHangUpFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hRasConn"]), offset=0),
    "RasCustomScriptExecuteFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ppBuffer", "pdwSize"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pBufer"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pBuffer", "dwSize"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pBuffer", "pdwSize", "dwTimeOut", "hEvent"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "pBuffer", "pdwSize"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("RASDIALPARAMSA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hPort", "lpszPhonebook", "lpszEntryName", "pfnRasGetBuffer", "pfnRasFreeBuffer", "pfnRasSendBuffer", "pfnRasReceiveBuffer", "pfnRasRetrieveBuffer", "hWnd", "pRasDialParams", "pvReserved"]), offset=0),
    "RawSearchFlags": SimTypeInt(signed=False, label="RawSearchFlags"),
    "RdcBufferPointer": SimStruct(OrderedDict((("m_Size", SimTypeInt(signed=False, label="UInt32")), ("m_Used", SimTypeInt(signed=False, label="UInt32")), ("m_Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RdcBufferPointer", pack=False, align=None),
    "RdcCreatedTables": SimTypeInt(signed=False, label="RdcCreatedTables"),
    "RdcMappingAccessMode": SimTypeInt(signed=False, label="RdcMappingAccessMode"),
    "RdcNeed": SimStruct(OrderedDict((("m_BlockType", SimTypeInt(signed=False, label="RdcNeedType")), ("m_FileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("m_BlockLength", SimTypeLongLong(signed=False, label="UInt64")),)), name="RdcNeed", pack=False, align=None),
    "RdcNeedPointer": SimStruct(OrderedDict((("m_Size", SimTypeInt(signed=False, label="UInt32")), ("m_Used", SimTypeInt(signed=False, label="UInt32")), ("m_Data", SimTypePointer(SimTypeRef("RdcNeed", SimStruct), offset=0)),)), name="RdcNeedPointer", pack=False, align=None),
    "RdcNeedType": SimTypeInt(signed=False, label="RdcNeedType"),
    "RdcSignature": SimStruct(OrderedDict((("m_Signature", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("m_BlockLength", SimTypeShort(signed=False, label="UInt16")),)), name="RdcSignature", pack=False, align=None),
    "RdcSignaturePointer": SimStruct(OrderedDict((("m_Size", SimTypeInt(signed=False, label="UInt32")), ("m_Used", SimTypeInt(signed=False, label="UInt32")), ("m_Data", SimTypePointer(SimTypeRef("RdcSignature", SimStruct), offset=0)),)), name="RdcSignaturePointer", pack=False, align=None),
    "RealTimeStylusDataInterest": SimTypeInt(signed=False, label="RealTimeStylusDataInterest"),
    "RealTimeStylusLockType": SimTypeInt(signed=False, label="RealTimeStylusLockType"),
    "RecordingType": SimTypeInt(signed=False, label="RecordingType"),
    "Rect": SimStruct(OrderedDict((("X", SimTypeInt(signed=True, label="Int32")), ("Y", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=True, label="Int32")), ("Height", SimTypeInt(signed=True, label="Int32")),)), name="Rect", pack=False, align=None),
    "RectF": SimStruct(OrderedDict((("X", SimTypeFloat(size=32)), ("Y", SimTypeFloat(size=32)), ("Width", SimTypeFloat(size=32)), ("Height", SimTypeFloat(size=32)),)), name="RectF", pack=False, align=None),
    "RedEyeCorrection": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="RedEyeCorrection", pack=False, align=None),
    "RedEyeCorrectionParams": SimStruct(OrderedDict((("numberOfAreas", SimTypeInt(signed=False, label="UInt32")), ("areas", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)),)), name="RedEyeCorrectionParams", pack=False, align=None),
    "RefreshConstants": SimTypeInt(signed=False, label="RefreshConstants"),
    "Region": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "RegisterAuthoringClientFunctionType": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IWebApplicationAuthoringMode"), SimTypeBottom(label="IWebApplicationHost")], SimTypeInt(signed=True, label="Int32"), arg_names=["authoringModeObject", "host"]), offset=0),
    "ReleaseModes": SimTypeInt(signed=False, label="ReleaseModes"),
    "RemBINDINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("szExtraInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("grfBindInfoF", SimTypeInt(signed=False, label="UInt32")), ("dwBindVerb", SimTypeInt(signed=False, label="UInt32")), ("szCustomVerb", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbstgmedData", SimTypeInt(signed=False, label="UInt32")), ("dwOptions", SimTypeInt(signed=False, label="UInt32")), ("dwOptionsFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCodePage", SimTypeInt(signed=False, label="UInt32")), ("securityAttributes", SimTypeRef("REMSECURITY_ATTRIBUTES", SimStruct)), ("iid", SimTypeBottom(label="Guid")), ("pUnk", SimTypeBottom(label="IUnknown")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="RemBINDINFO", pack=False, align=None),
    "RemFORMATETC": SimStruct(OrderedDict((("cfFormat", SimTypeInt(signed=False, label="UInt32")), ("ptd", SimTypeInt(signed=False, label="UInt32")), ("dwAspect", SimTypeInt(signed=False, label="UInt32")), ("lindex", SimTypeInt(signed=True, label="Int32")), ("tymed", SimTypeInt(signed=False, label="UInt32")),)), name="RemFORMATETC", pack=False, align=None),
    "RemHBITMAP": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemHBITMAP", pack=False, align=None),
    "RemHBRUSH": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemHBRUSH", pack=False, align=None),
    "RemHENHMETAFILE": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemHENHMETAFILE", pack=False, align=None),
    "RemHGLOBAL": SimStruct(OrderedDict((("fNullHGlobal", SimTypeInt(signed=True, label="Int32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemHGLOBAL", pack=False, align=None),
    "RemHMETAFILEPICT": SimStruct(OrderedDict((("mm", SimTypeInt(signed=True, label="Int32")), ("xExt", SimTypeInt(signed=True, label="Int32")), ("yExt", SimTypeInt(signed=True, label="Int32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemHMETAFILEPICT", pack=False, align=None),
    "RemHPALETTE": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemHPALETTE", pack=False, align=None),
    "RemSNB": SimStruct(OrderedDict((("ulCntStr", SimTypeInt(signed=False, label="UInt32")), ("ulCntChar", SimTypeInt(signed=False, label="UInt32")), ("rgString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="RemSNB", pack=False, align=None),
    "RemSTGMEDIUM": SimStruct(OrderedDict((("tymed", SimTypeInt(signed=False, label="UInt32")), ("dwHandleType", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypeInt(signed=False, label="UInt32")), ("pUnkForRelease", SimTypeInt(signed=False, label="UInt32")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="RemSTGMEDIUM", pack=False, align=None),
    "RemotableHandle": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="RemotableHandle", pack=False, align=None),
    "RemoteActionType": SimTypeInt(signed=False, label="RemoteActionType"),
    "RemoteConfigurationType": SimTypeInt(signed=False, label="RemoteConfigurationType"),
    "RenderTargetBitmapOptions": SimTypeInt(signed=False, label="RenderTargetBitmapOptions"),
    "RepairInfo": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("pwszClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sidType", SimTypeInt(signed=False, label="UInt32")), ("cost", SimTypeInt(signed=True, label="Int32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("scope", SimTypeInt(signed=False, label="REPAIR_SCOPE")), ("risk", SimTypeInt(signed=False, label="REPAIR_RISK")), ("UiInfo", SimTypeRef("UiInfo", SimStruct)), ("rootCauseIndex", SimTypeInt(signed=True, label="Int32")),)), name="RepairInfo", pack=False, align=None),
    "RepairInfoEx": SimStruct(OrderedDict((("repair", SimTypeRef("RepairInfo", SimStruct)), ("repairRank", SimTypeShort(signed=False, label="UInt16")),)), name="RepairInfoEx", pack=False, align=None),
    "ReplacesCorHdrNumericDefines": SimTypeInt(signed=False, label="ReplacesCorHdrNumericDefines"),
    "ReportValueTypeConstants": SimTypeInt(signed=False, label="ReportValueTypeConstants"),
    "RequestClientInfoClientId": SimTypeInt(signed=False, label="RequestClientInfoClientId"),
    "ResourcePolicy": SimTypeInt(signed=False, label="ResourcePolicy"),
    "ResourceType": SimTypeInt(signed=False, label="ResourceType"),
    "ResourceUtilizationInfoElement": SimStruct(OrderedDict((("PhysicalNumaId", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentMemory", SimTypeLongLong(signed=False, label="UInt64")),)), name="ResourceUtilizationInfoElement", pack=False, align=None),
    "ResultCodes": SimStruct(OrderedDict((("count", SimTypeShort(signed=False, label="UInt16")), ("results", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="ResultCodes", pack=False, align=None),
    "RevokedComponent": SimTypeInt(signed=False, label="RevokedComponent"),
    "RollOff": SimTypeInt(signed=False, label="RollOff"),
    "RootCauseInfo": SimStruct(OrderedDict((("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rootCauseID", SimTypeBottom(label="Guid")), ("rootCauseFlags", SimTypeInt(signed=False, label="UInt32")), ("networkInterfaceID", SimTypeBottom(label="Guid")), ("pRepairs", SimTypePointer(SimTypeRef("RepairInfoEx", SimStruct), offset=0)), ("repairCount", SimTypeShort(signed=False, label="UInt16")),)), name="RootCauseInfo", pack=False, align=None),
    "RotateFlipType": SimTypeInt(signed=False, label="RotateFlipType"),
    "RowOrColumnMajor": SimTypeInt(signed=False, label="RowOrColumnMajor"),
    "RpcCallClientLocality": SimTypeInt(signed=False, label="RpcCallClientLocality"),
    "RpcCallType": SimTypeInt(signed=False, label="RpcCallType"),
    "RpcLocalAddressFormat": SimTypeInt(signed=False, label="RpcLocalAddressFormat"),
    "RpcPerfCounters": SimTypeInt(signed=False, label="RpcPerfCounters"),
    "RsvpObjHdr": SimStruct(OrderedDict((("obj_length", SimTypeShort(signed=False, label="UInt16")), ("obj_class", SimTypeChar(label="Byte")), ("obj_ctype", SimTypeChar(label="Byte")),)), name="RsvpObjHdr", pack=False, align=None),
    "Rsvp_Hop_IPv4": SimStruct(OrderedDict((("hop_ipaddr", SimTypeRef("IN_ADDR", SimStruct)), ("hop_LIH", SimTypeInt(signed=False, label="UInt32")),)), name="Rsvp_Hop_IPv4", pack=False, align=None),
    "RuntimeLoadedCallbackFnPtr": SimTypePointer(SimTypeFunction([SimTypeBottom(label="ICLRRuntimeInfo"), SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0), SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)], SimTypeBottom(label="Void"), arg_names=["pRuntimeInfo", "pfnCallbackThreadSet", "pfnCallbackThreadUnset"]), offset=0),
    "SAFEARRAY": SimStruct(OrderedDict((("cDims", SimTypeShort(signed=False, label="UInt16")), ("fFeatures", SimTypeInt(signed=False, label="ADVANCED_FEATURE_FLAGS")), ("cbElements", SimTypeInt(signed=False, label="UInt32")), ("cLocks", SimTypeInt(signed=False, label="UInt32")), ("pvData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("rgsabound", SimTypePointer(SimTypeRef("SAFEARRAYBOUND", SimStruct), offset=0)),)), name="SAFEARRAY", pack=False, align=None),
    "SAFEARRAYBOUND": SimStruct(OrderedDict((("cElements", SimTypeInt(signed=False, label="UInt32")), ("lLbound", SimTypeInt(signed=True, label="Int32")),)), name="SAFEARRAYBOUND", pack=False, align=None),
    "SAFEARRAYUNION": SimStruct(OrderedDict((("sfType", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"BstrStr": SimTypeRef("SAFEARR_BSTR", SimStruct), "UnknownStr": SimTypeRef("SAFEARR_UNKNOWN", SimStruct), "DispatchStr": SimTypeRef("SAFEARR_DISPATCH", SimStruct), "VariantStr": SimTypeRef("SAFEARR_VARIANT", SimStruct), "RecordStr": SimTypeRef("SAFEARR_BRECORD", SimStruct), "HaveIidStr": SimTypeRef("SAFEARR_HAVEIID", SimStruct), "ByteStr": SimTypeRef("BYTE_SIZEDARR", SimStruct), "WordStr": SimTypeRef("WORD_SIZEDARR", SimStruct), "LongStr": SimTypeRef("DWORD_SIZEDARR", SimStruct), "HyperStr": SimTypeRef("HYPER_SIZEDARR", SimStruct)}, name="<anon>", label="None")),)), name="SAFEARRAYUNION", pack=False, align=None),
    "SAFEARR_BRECORD": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("aRecord", SimTypePointer(SimTypePointer(SimTypeRef("_wireBRECORD", SimStruct), offset=0), offset=0)),)), name="SAFEARR_BRECORD", pack=False, align=None),
    "SAFEARR_BSTR": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("aBstr", SimTypePointer(SimTypePointer(SimTypeRef("FLAGGED_WORD_BLOB", SimStruct), offset=0), offset=0)),)), name="SAFEARR_BSTR", pack=False, align=None),
    "SAFEARR_DISPATCH": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("apDispatch", SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0)),)), name="SAFEARR_DISPATCH", pack=False, align=None),
    "SAFEARR_HAVEIID": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("apUnknown", SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)), ("iid", SimTypeBottom(label="Guid")),)), name="SAFEARR_HAVEIID", pack=False, align=None),
    "SAFEARR_UNKNOWN": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("apUnknown", SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)),)), name="SAFEARR_UNKNOWN", pack=False, align=None),
    "SAFEARR_VARIANT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("aVariant", SimTypePointer(SimTypePointer(SimTypeRef("_wireVARIANT", SimStruct), offset=0), offset=0)),)), name="SAFEARR_VARIANT", pack=False, align=None),
    "SAFER_CODE_PROPERTIES_V1": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwCheckFlags", SimTypeInt(signed=False, label="UInt32")), ("ImagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hImageFileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UrlZoneId", SimTypeInt(signed=False, label="UInt32")), ("ImageHash", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("dwImageHashSize", SimTypeInt(signed=False, label="UInt32")), ("ImageSize", SimTypeLongLong(signed=True, label="Int64")), ("HashAlgorithm", SimTypeInt(signed=False, label="ALG_ID")), ("pByteBlock", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwWVTUIChoice", SimTypeInt(signed=False, label="UInt32")),)), name="SAFER_CODE_PROPERTIES_V1", pack=False, align=None),
    "SAFER_CODE_PROPERTIES_V2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwCheckFlags", SimTypeInt(signed=False, label="UInt32")), ("ImagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hImageFileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UrlZoneId", SimTypeInt(signed=False, label="UInt32")), ("ImageHash", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("dwImageHashSize", SimTypeInt(signed=False, label="UInt32")), ("ImageSize", SimTypeLongLong(signed=True, label="Int64")), ("HashAlgorithm", SimTypeInt(signed=False, label="ALG_ID")), ("pByteBlock", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hWndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwWVTUIChoice", SimTypeInt(signed=False, label="UInt32")), ("PackageMoniker", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PackagePublisher", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PackageVersion", SimTypeLongLong(signed=False, label="UInt64")), ("PackageIsFramework", SimTypeInt(signed=True, label="Int32")),)), name="SAFER_CODE_PROPERTIES_V2", pack=False, align=None),
    "SAFER_COMPUTE_TOKEN_FROM_LEVEL_FLAGS": SimTypeInt(signed=False, label="SAFER_COMPUTE_TOKEN_FROM_LEVEL_FLAGS"),
    "SAFER_HASH_IDENTIFICATION": SimStruct(OrderedDict((("header", SimTypeRef("SAFER_IDENTIFICATION_HEADER", SimStruct)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 256)), ("FriendlyName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("HashSize", SimTypeInt(signed=False, label="UInt32")), ("ImageHash", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("HashAlgorithm", SimTypeInt(signed=False, label="ALG_ID")), ("ImageSize", SimTypeLongLong(signed=True, label="Int64")), ("dwSaferFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SAFER_HASH_IDENTIFICATION", pack=False, align=None),
    "SAFER_HASH_IDENTIFICATION2": SimStruct(OrderedDict((("hashIdentification", SimTypeRef("SAFER_HASH_IDENTIFICATION", SimStruct)), ("HashSize", SimTypeInt(signed=False, label="UInt32")), ("ImageHash", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("HashAlgorithm", SimTypeInt(signed=False, label="ALG_ID")),)), name="SAFER_HASH_IDENTIFICATION2", pack=False, align=None),
    "SAFER_IDENTIFICATION_HEADER": SimStruct(OrderedDict((("dwIdentificationType", SimTypeInt(signed=False, label="SAFER_IDENTIFICATION_TYPES")), ("cbStructSize", SimTypeInt(signed=False, label="UInt32")), ("IdentificationGuid", SimTypeBottom(label="Guid")), ("lastModified", SimTypeRef("FILETIME", SimStruct)),)), name="SAFER_IDENTIFICATION_HEADER", pack=False, align=None),
    "SAFER_IDENTIFICATION_TYPES": SimTypeInt(signed=False, label="SAFER_IDENTIFICATION_TYPES"),
    "SAFER_LEVEL_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SAFER_OBJECT_INFO_CLASS": SimTypeInt(signed=False, label="SAFER_OBJECT_INFO_CLASS"),
    "SAFER_PATHNAME_IDENTIFICATION": SimStruct(OrderedDict((("header", SimTypeRef("SAFER_IDENTIFICATION_HEADER", SimStruct)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ImageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSaferFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SAFER_PATHNAME_IDENTIFICATION", pack=False, align=None),
    "SAFER_POLICY_INFO_CLASS": SimTypeInt(signed=False, label="SAFER_POLICY_INFO_CLASS"),
    "SAFER_URLZONE_IDENTIFICATION": SimStruct(OrderedDict((("header", SimTypeRef("SAFER_IDENTIFICATION_HEADER", SimStruct)), ("UrlZoneId", SimTypeInt(signed=False, label="UInt32")), ("dwSaferFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SAFER_URLZONE_IDENTIFICATION", pack=False, align=None),
    "SAMPLE_LIVE_STREAM_TIME": SimStruct(OrderedDict((("qwStreamTime", SimTypeLongLong(signed=False, label="UInt64")), ("qwLiveTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="SAMPLE_LIVE_STREAM_TIME", pack=False, align=None),
    "SAMPLE_PROTECTION_VERSION": SimTypeInt(signed=False, label="SAMPLE_PROTECTION_VERSION"),
    "SAMPLE_SEQ_OFFSET": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="SAMPLE_SEQ_OFFSET", pack=False, align=None),
    "SAMPR_ENCRYPTED_USER_PASSWORD": SimStruct(OrderedDict((("Buffer", SimTypeArray(SimTypeChar(label="Byte"), 516)),)), name="SAMPR_ENCRYPTED_USER_PASSWORD", pack=False, align=None),
    "SAM_REGISTER_MAPPING_ELEMENT": SimStruct(OrderedDict((("Original", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Mapped", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Continuable", SimTypeChar(label="Byte")),)), name="SAM_REGISTER_MAPPING_ELEMENT", pack=False, align=None),
    "SAM_REGISTER_MAPPING_LIST": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Elements", SimTypePointer(SimTypeRef("SAM_REGISTER_MAPPING_ELEMENT", SimStruct), offset=0)),)), name="SAM_REGISTER_MAPPING_LIST", pack=False, align=None),
    "SAM_REGISTER_MAPPING_TABLE": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Lists", SimTypePointer(SimTypeRef("SAM_REGISTER_MAPPING_LIST", SimStruct), offset=0)),)), name="SAM_REGISTER_MAPPING_TABLE", pack=False, align=None),
    "SASL_AUTHZID_STATE": SimTypeInt(signed=False, label="SASL_AUTHZID_STATE"),
    "SAVE_SEGMENTS_FLAGS": SimTypeInt(signed=False, label="SAVE_SEGMENTS_FLAGS"),
    "SAndRestriction": SimStruct(OrderedDict((("cRes", SimTypeInt(signed=False, label="UInt32")), ("lpRes", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)),)), name="SAndRestriction", pack=False, align=None),
    "SAppTimeArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpat", SimTypePointer(SimTypeFloat(size=64), offset=0)),)), name="SAppTimeArray", pack=False, align=None),
    "SBE2_STREAM_DESC": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("StreamId", SimTypeInt(signed=False, label="UInt32")), ("Default", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="SBE2_STREAM_DESC", pack=False, align=None),
    "SBE_PIN_DATA": SimStruct(OrderedDict((("cDataBytes", SimTypeLongLong(signed=False, label="UInt64")), ("cSamplesProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("cDiscontinuities", SimTypeLongLong(signed=False, label="UInt64")), ("cSyncPoints", SimTypeLongLong(signed=False, label="UInt64")), ("cTimestamps", SimTypeLongLong(signed=False, label="UInt64")),)), name="SBE_PIN_DATA", pack=False, align=None),
    "SBinary": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("lpb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SBinary", pack=False, align=None),
    "SBinaryArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpbin", SimTypePointer(SimTypeRef("SBinary", SimStruct), offset=0)),)), name="SBinaryArray", pack=False, align=None),
    "SBitMaskRestriction": SimStruct(OrderedDict((("relBMR", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("ulMask", SimTypeInt(signed=False, label="UInt32")),)), name="SBitMaskRestriction", pack=False, align=None),
    "SCALE_CHANGE_FLAGS": SimTypeInt(signed=False, label="SCALE_CHANGE_FLAGS"),
    "SCANINFO": SimStruct(OrderedDict((("ADF", SimTypeInt(signed=True, label="Int32")), ("TPA", SimTypeInt(signed=True, label="Int32")), ("Endorser", SimTypeInt(signed=True, label="Int32")), ("OpticalXResolution", SimTypeInt(signed=True, label="Int32")), ("OpticalYResolution", SimTypeInt(signed=True, label="Int32")), ("BedWidth", SimTypeInt(signed=True, label="Int32")), ("BedHeight", SimTypeInt(signed=True, label="Int32")), ("IntensityRange", SimTypeRef("RANGEVALUE", SimStruct)), ("ContrastRange", SimTypeRef("RANGEVALUE", SimStruct)), ("SupportedCompressionType", SimTypeInt(signed=True, label="Int32")), ("SupportedDataTypes", SimTypeInt(signed=True, label="Int32")), ("WidthPixels", SimTypeInt(signed=True, label="Int32")), ("WidthBytes", SimTypeInt(signed=True, label="Int32")), ("Lines", SimTypeInt(signed=True, label="Int32")), ("DataType", SimTypeInt(signed=True, label="Int32")), ("PixelBits", SimTypeInt(signed=True, label="Int32")), ("Intensity", SimTypeInt(signed=True, label="Int32")), ("Contrast", SimTypeInt(signed=True, label="Int32")), ("Xresolution", SimTypeInt(signed=True, label="Int32")), ("Yresolution", SimTypeInt(signed=True, label="Int32")), ("Window", SimTypeRef("SCANWINDOW", SimStruct)), ("DitherPattern", SimTypeInt(signed=True, label="Int32")), ("Negative", SimTypeInt(signed=True, label="Int32")), ("Mirror", SimTypeInt(signed=True, label="Int32")), ("AutoBack", SimTypeInt(signed=True, label="Int32")), ("ColorDitherPattern", SimTypeInt(signed=True, label="Int32")), ("ToneMap", SimTypeInt(signed=True, label="Int32")), ("Compression", SimTypeInt(signed=True, label="Int32")), ("RawDataFormat", SimTypeInt(signed=True, label="Int32")), ("RawPixelOrder", SimTypeInt(signed=True, label="Int32")), ("bNeedDataAlignment", SimTypeInt(signed=True, label="Int32")), ("DelayBetweenRead", SimTypeInt(signed=True, label="Int32")), ("MaxBufferSize", SimTypeInt(signed=True, label="Int32")), ("DeviceIOHandles", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 16)), ("lReserved", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 4)), ("pMicroDriverContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SCANINFO", pack=False, align=None),
    "SCANWINDOW": SimStruct(OrderedDict((("xPos", SimTypeInt(signed=True, label="Int32")), ("yPos", SimTypeInt(signed=True, label="Int32")), ("xExtent", SimTypeInt(signed=True, label="Int32")), ("yExtent", SimTypeInt(signed=True, label="Int32")),)), name="SCANWINDOW", pack=False, align=None),
    "SCARD_ATRMASK": SimStruct(OrderedDict((("cbAtr", SimTypeInt(signed=False, label="UInt32")), ("rgbAtr", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("rgbMask", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="SCARD_ATRMASK", pack=False, align=None),
    "SCARD_IO_REQUEST": SimStruct(OrderedDict((("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("cbPciLength", SimTypeInt(signed=False, label="UInt32")),)), name="SCARD_IO_REQUEST", pack=False, align=None),
    "SCARD_READERSTATEA": SimStruct(OrderedDict((("szReader", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCurrentState", SimTypeInt(signed=False, label="SCARD_STATE")), ("dwEventState", SimTypeInt(signed=False, label="SCARD_STATE")), ("cbAtr", SimTypeInt(signed=False, label="UInt32")), ("rgbAtr", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="SCARD_READERSTATEA", pack=False, align=None),
    "SCARD_READERSTATEW": SimStruct(OrderedDict((("szReader", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCurrentState", SimTypeInt(signed=False, label="SCARD_STATE")), ("dwEventState", SimTypeInt(signed=False, label="SCARD_STATE")), ("cbAtr", SimTypeInt(signed=False, label="UInt32")), ("rgbAtr", SimTypeArray(SimTypeChar(label="Byte"), 36)),)), name="SCARD_READERSTATEW", pack=False, align=None),
    "SCARD_SCOPE": SimTypeInt(signed=False, label="SCARD_SCOPE"),
    "SCARD_STATE": SimTypeInt(signed=False, label="SCARD_STATE"),
    "SCARD_T0_COMMAND": SimStruct(OrderedDict((("bCla", SimTypeChar(label="Byte")), ("bIns", SimTypeChar(label="Byte")), ("bP1", SimTypeChar(label="Byte")), ("bP2", SimTypeChar(label="Byte")), ("bP3", SimTypeChar(label="Byte")),)), name="SCARD_T0_COMMAND", pack=False, align=None),
    "SCARD_T0_REQUEST": SimStruct(OrderedDict((("ioRequest", SimTypeRef("SCARD_IO_REQUEST", SimStruct)), ("bSw1", SimTypeChar(label="Byte")), ("bSw2", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"CmdBytes": SimTypeRef("SCARD_T0_COMMAND", SimStruct), "rgbHeader": SimTypeArray(SimTypeChar(label="Byte"), 5)}, name="<anon>", label="None")),)), name="SCARD_T0_REQUEST", pack=False, align=None),
    "SCARD_T1_REQUEST": SimStruct(OrderedDict((("ioRequest", SimTypeRef("SCARD_IO_REQUEST", SimStruct)),)), name="SCARD_T1_REQUEST", pack=False, align=None),
    "SCATTER_GATHER_ELEMENT": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SCATTER_GATHER_ELEMENT", pack=False, align=None),
    "SCATTER_GATHER_LIST": SimStruct(OrderedDict((("NumberOfElements", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Elements", SimTypePointer(SimTypeRef("SCATTER_GATHER_ELEMENT", SimStruct), offset=0)),)), name="SCATTER_GATHER_LIST", pack=False, align=None),
    "SCESVC_ANALYSIS_INFO": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Lines", SimTypePointer(SimTypeRef("SCESVC_ANALYSIS_LINE", SimStruct), offset=0)),)), name="SCESVC_ANALYSIS_INFO", pack=False, align=None),
    "SCESVC_ANALYSIS_LINE": SimStruct(OrderedDict((("Key", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ValueLen", SimTypeInt(signed=False, label="UInt32")),)), name="SCESVC_ANALYSIS_LINE", pack=False, align=None),
    "SCESVC_CALLBACK_INFO": SimStruct(OrderedDict((("sceHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pfQueryInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SCESVC_INFO_TYPE"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["sceHandle", "sceType", "lpPrefix", "bExact", "ppvInfo", "psceEnumHandle"]), offset=0)), ("pfSetInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SCESVC_INFO_TYPE"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["sceHandle", "sceType", "lpPrefix", "bExact", "pvInfo"]), offset=0)), ("pfFreeInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pvServiceInfo"]), offset=0)), ("pfLogInfo", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SCE_LOG_ERR_LEVEL"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="SByte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ErrLevel", "Win32rc", "pErrFmt"]), offset=0)),)), name="SCESVC_CALLBACK_INFO", pack=False, align=None),
    "SCESVC_CONFIGURATION_INFO": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Lines", SimTypePointer(SimTypeRef("SCESVC_CONFIGURATION_LINE", SimStruct), offset=0)),)), name="SCESVC_CONFIGURATION_INFO", pack=False, align=None),
    "SCESVC_CONFIGURATION_LINE": SimStruct(OrderedDict((("Key", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("ValueLen", SimTypeInt(signed=False, label="UInt32")),)), name="SCESVC_CONFIGURATION_LINE", pack=False, align=None),
    "SCESVC_INFO_TYPE": SimTypeInt(signed=False, label="SCESVC_INFO_TYPE"),
    "SCE_LOG_ERR_LEVEL": SimTypeInt(signed=False, label="SCE_LOG_ERR_LEVEL"),
    "SCF_BUTTON_IDS": SimTypeInt(signed=False, label="SCF_BUTTON_IDS"),
    "SCF_CONTEXTMENU_EVENT": SimStruct(OrderedDict((("PreviousPage", SimTypeInt(signed=False, label="UInt32")), ("TargetPage", SimTypeInt(signed=False, label="UInt32")), ("PreviousItemId", SimTypeInt(signed=False, label="UInt32")), ("MenuPage", SimTypeInt(signed=False, label="UInt32")), ("MenuItemId", SimTypeInt(signed=False, label="UInt32")),)), name="SCF_CONTEXTMENU_EVENT", pack=False, align=None),
    "SCF_EVENT_HEADER": SimStruct(OrderedDict((("PreviousPage", SimTypeInt(signed=False, label="UInt32")), ("TargetPage", SimTypeInt(signed=False, label="UInt32")),)), name="SCF_EVENT_HEADER", pack=False, align=None),
    "SCF_EVENT_IDS": SimTypeInt(signed=False, label="SCF_EVENT_IDS"),
    "SCF_MENUACTION_EVENT": SimStruct(OrderedDict((("PreviousPage", SimTypeInt(signed=False, label="UInt32")), ("TargetPage", SimTypeInt(signed=False, label="UInt32")), ("Button", SimTypeInt(signed=False, label="UInt32")), ("ItemId", SimTypeInt(signed=False, label="UInt32")),)), name="SCF_MENUACTION_EVENT", pack=False, align=None),
    "SCF_NAVIGATION_EVENT": SimStruct(OrderedDict((("PreviousPage", SimTypeInt(signed=False, label="UInt32")), ("TargetPage", SimTypeInt(signed=False, label="UInt32")), ("Button", SimTypeInt(signed=False, label="UInt32")),)), name="SCF_NAVIGATION_EVENT", pack=False, align=None),
    "SCHANNEL_ALERT_TOKEN": SimStruct(OrderedDict((("dwTokenType", SimTypeInt(signed=False, label="UInt32")), ("dwAlertType", SimTypeInt(signed=False, label="SCHANNEL_ALERT_TOKEN_ALERT_TYPE")), ("dwAlertNumber", SimTypeInt(signed=False, label="UInt32")),)), name="SCHANNEL_ALERT_TOKEN", pack=False, align=None),
    "SCHANNEL_ALERT_TOKEN_ALERT_TYPE": SimTypeInt(signed=False, label="SCHANNEL_ALERT_TOKEN_ALERT_TYPE"),
    "SCHANNEL_ALG": SimStruct(OrderedDict((("dwUse", SimTypeInt(signed=False, label="UInt32")), ("Algid", SimTypeInt(signed=False, label="ALG_ID")), ("cBits", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="SCHANNEL_ALG", pack=False, align=None),
    "SCHANNEL_CERT_HASH": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ShaHash", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="SCHANNEL_CERT_HASH", pack=False, align=None),
    "SCHANNEL_CERT_HASH_STORE": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ShaHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("pwszStoreName", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="SCHANNEL_CERT_HASH_STORE", pack=False, align=None),
    "SCHANNEL_CLIENT_SIGNATURE": SimStruct(OrderedDict((("cbLength", SimTypeInt(signed=False, label="UInt32")), ("aiHash", SimTypeInt(signed=False, label="ALG_ID")), ("cbHash", SimTypeInt(signed=False, label="UInt32")), ("HashValue", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("CertThumbprint", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="SCHANNEL_CLIENT_SIGNATURE", pack=False, align=None),
    "SCHANNEL_CRED": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cCreds", SimTypeInt(signed=False, label="UInt32")), ("paCred", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("hRootStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cMappers", SimTypeInt(signed=False, label="UInt32")), ("aphMappers", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)), ("cSupportedAlgs", SimTypeInt(signed=False, label="UInt32")), ("palgSupportedAlgs", SimTypePointer(SimTypeInt(signed=False, label="ALG_ID"), offset=0)), ("grbitEnabledProtocols", SimTypeInt(signed=False, label="UInt32")), ("dwMinimumCipherStrength", SimTypeInt(signed=False, label="UInt32")), ("dwMaximumCipherStrength", SimTypeInt(signed=False, label="UInt32")), ("dwSessionLifespan", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="SCHANNEL_CRED_FLAGS")), ("dwCredFormat", SimTypeInt(signed=False, label="UInt32")),)), name="SCHANNEL_CRED", pack=False, align=None),
    "SCHANNEL_CRED_FLAGS": SimTypeInt(signed=False, label="SCHANNEL_CRED_FLAGS"),
    "SCHANNEL_SESSION_TOKEN": SimStruct(OrderedDict((("dwTokenType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="SCHANNEL_SESSION_TOKEN_FLAGS")),)), name="SCHANNEL_SESSION_TOKEN", pack=False, align=None),
    "SCHANNEL_SESSION_TOKEN_FLAGS": SimTypeInt(signed=False, label="SCHANNEL_SESSION_TOKEN_FLAGS"),
    "SCHEDULE": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Bandwidth", SimTypeInt(signed=False, label="UInt32")), ("NumberOfSchedules", SimTypeInt(signed=False, label="UInt32")), ("Schedules", SimTypePointer(SimTypeRef("SCHEDULE_HEADER", SimStruct), offset=0)),)), name="SCHEDULE", pack=False, align=None),
    "SCHEDULE_HEADER": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="SCHEDULE_HEADER", pack=False, align=None),
    "SCHEMACONTENTTYPE": SimTypeInt(signed=False, label="SCHEMACONTENTTYPE"),
    "SCHEMADERIVATIONMETHOD": SimTypeInt(signed=False, label="SCHEMADERIVATIONMETHOD"),
    "SCHEMAPROCESSCONTENTS": SimTypeInt(signed=False, label="SCHEMAPROCESSCONTENTS"),
    "SCHEMATYPEVARIETY": SimTypeInt(signed=False, label="SCHEMATYPEVARIETY"),
    "SCHEMAUSE": SimTypeInt(signed=False, label="SCHEMAUSE"),
    "SCHEMAWHITESPACE": SimTypeInt(signed=False, label="SCHEMAWHITESPACE"),
    "SCH_CRED": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cCreds", SimTypeInt(signed=False, label="UInt32")), ("paSecret", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("paPublic", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("cMappers", SimTypeInt(signed=False, label="UInt32")), ("aphMappers", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)),)), name="SCH_CRED", pack=False, align=None),
    "SCH_CREDENTIALS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwCredFormat", SimTypeInt(signed=False, label="UInt32")), ("cCreds", SimTypeInt(signed=False, label="UInt32")), ("paCred", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("hRootStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cMappers", SimTypeInt(signed=False, label="UInt32")), ("aphMappers", SimTypePointer(SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), offset=0)), ("dwSessionLifespan", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cTlsParameters", SimTypeInt(signed=False, label="UInt32")), ("pTlsParameters", SimTypePointer(SimTypeRef("TLS_PARAMETERS", SimStruct), offset=0)),)), name="SCH_CREDENTIALS", pack=False, align=None),
    "SCH_CRED_PUBLIC_CERTCHAIN": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("cbCertChain", SimTypeInt(signed=False, label="UInt32")), ("pCertChain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCH_CRED_PUBLIC_CERTCHAIN", pack=False, align=None),
    "SCH_CRED_SECRET_CAPI": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("hProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SCH_CRED_SECRET_CAPI", pack=False, align=None),
    "SCH_CRED_SECRET_PRIVKEY": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("pPrivateKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPrivateKey", SimTypeInt(signed=False, label="UInt32")), ("pszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCH_CRED_SECRET_PRIVKEY", pack=False, align=None),
    "SCH_EXTENSION_DATA": SimStruct(OrderedDict((("ExtensionType", SimTypeShort(signed=False, label="UInt16")), ("pExtData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbExtData", SimTypeInt(signed=False, label="UInt32")),)), name="SCH_EXTENSION_DATA", pack=False, align=None),
    "SCM_BUS_DEDICATED_MEMORY_DEVICES_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("Devices", SimTypePointer(SimTypeRef("SCM_BUS_DEDICATED_MEMORY_DEVICE_INFO", SimStruct), offset=0)),)), name="SCM_BUS_DEDICATED_MEMORY_DEVICES_INFO", pack=False, align=None),
    "SCM_BUS_DEDICATED_MEMORY_DEVICE_INFO": SimStruct(OrderedDict((("DeviceGuid", SimTypeBottom(label="Guid")), ("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("_Flags_e__Struct", SimStruct)), ("DeviceSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="SCM_BUS_DEDICATED_MEMORY_DEVICE_INFO", pack=False, align=None),
    "SCM_BUS_DEDICATED_MEMORY_STATE": SimStruct(OrderedDict((("ActivateState", SimTypeChar(label="Byte")),)), name="SCM_BUS_DEDICATED_MEMORY_STATE", pack=False, align=None),
    "SCM_BUS_FIRMWARE_ACTIVATION_STATE": SimTypeInt(signed=False, label="SCM_BUS_FIRMWARE_ACTIVATION_STATE"),
    "SCM_BUS_PROPERTY_ID": SimTypeInt(signed=False, label="SCM_BUS_PROPERTY_ID"),
    "SCM_BUS_PROPERTY_QUERY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PropertyId", SimTypeInt(signed=False, label="SCM_BUS_PROPERTY_ID")), ("QueryType", SimTypeInt(signed=False, label="SCM_BUS_QUERY_TYPE")), ("AdditionalParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_BUS_PROPERTY_QUERY", pack=False, align=None),
    "SCM_BUS_PROPERTY_SET": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PropertyId", SimTypeInt(signed=False, label="SCM_BUS_PROPERTY_ID")), ("SetType", SimTypeInt(signed=False, label="SCM_BUS_SET_TYPE")), ("AdditionalParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_BUS_PROPERTY_SET", pack=False, align=None),
    "SCM_BUS_QUERY_TYPE": SimTypeInt(signed=False, label="SCM_BUS_QUERY_TYPE"),
    "SCM_BUS_RUNTIME_FW_ACTIVATION_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("RuntimeFwActivationSupported", SimTypeChar(label="Byte")), ("FirmwareActivationState", SimTypeInt(signed=False, label="SCM_BUS_FIRMWARE_ACTIVATION_STATE")), ("FirmwareActivationCapability", SimTypeRef("_FirmwareActivationCapability_e__Struct", SimStruct)), ("EstimatedFirmwareActivationTimeInUSecs", SimTypeLongLong(signed=False, label="UInt64")), ("EstimatedProcessorAccessQuiesceTimeInUSecs", SimTypeLongLong(signed=False, label="UInt64")), ("EstimatedIOAccessQuiesceTimeInUSecs", SimTypeLongLong(signed=False, label="UInt64")), ("PlatformSupportedMaxIOAccessQuiesceTimeInUSecs", SimTypeLongLong(signed=False, label="UInt64")),)), name="SCM_BUS_RUNTIME_FW_ACTIVATION_INFO", pack=False, align=None),
    "SCM_BUS_SET_TYPE": SimTypeInt(signed=False, label="SCM_BUS_SET_TYPE"),
    "SCM_INTERLEAVED_PD_INFO": SimStruct(OrderedDict((("DeviceHandle", SimTypeInt(signed=False, label="UInt32")), ("DeviceGuid", SimTypeBottom(label="Guid")),)), name="SCM_INTERLEAVED_PD_INFO", pack=False, align=None),
    "SCM_LD_INTERLEAVE_SET_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("InterleaveSetSize", SimTypeInt(signed=False, label="UInt32")), ("InterleaveSet", SimTypePointer(SimTypeRef("SCM_INTERLEAVED_PD_INFO", SimStruct), offset=0)),)), name="SCM_LD_INTERLEAVE_SET_INFO", pack=False, align=None),
    "SCM_LOGICAL_DEVICES": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("Devices", SimTypePointer(SimTypeRef("SCM_LOGICAL_DEVICE_INSTANCE", SimStruct), offset=0)),)), name="SCM_LOGICAL_DEVICES", pack=False, align=None),
    "SCM_LOGICAL_DEVICE_INSTANCE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("SymbolicLink", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="SCM_LOGICAL_DEVICE_INSTANCE", pack=False, align=None),
    "SCM_PD_DESCRIPTOR_HEADER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="SCM_PD_DESCRIPTOR_HEADER", pack=False, align=None),
    "SCM_PD_DEVICE_HANDLE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("DeviceHandle", SimTypeInt(signed=False, label="UInt32")),)), name="SCM_PD_DEVICE_HANDLE", pack=False, align=None),
    "SCM_PD_DEVICE_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("UnsafeShutdownCount", SimTypeInt(signed=False, label="UInt32")), ("PersistentMemorySizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("VolatileMemorySizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("TotalMemorySizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("SlotNumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceHandle", SimTypeInt(signed=False, label="UInt32")), ("PhysicalId", SimTypeShort(signed=False, label="UInt16")), ("NumberOfFormatInterfaceCodes", SimTypeChar(label="Byte")), ("FormatInterfaceCodes", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)), ("VendorId", SimTypeInt(signed=False, label="UInt32")), ("ProductId", SimTypeInt(signed=False, label="UInt32")), ("SubsystemDeviceId", SimTypeInt(signed=False, label="UInt32")), ("SubsystemVendorId", SimTypeInt(signed=False, label="UInt32")), ("ManufacturingLocation", SimTypeChar(label="Byte")), ("ManufacturingWeek", SimTypeChar(label="Byte")), ("ManufacturingYear", SimTypeChar(label="Byte")), ("SerialNumber4Byte", SimTypeInt(signed=False, label="UInt32")), ("SerialNumberLengthInChars", SimTypeInt(signed=False, label="UInt32")), ("SerialNumber", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SCM_PD_DEVICE_INFO", pack=False, align=None),
    "SCM_PD_DEVICE_SPECIFIC_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumberOfProperties", SimTypeInt(signed=False, label="UInt32")), ("DeviceSpecificProperties", SimTypePointer(SimTypeRef("SCM_PD_DEVICE_SPECIFIC_PROPERTY", SimStruct), offset=0)),)), name="SCM_PD_DEVICE_SPECIFIC_INFO", pack=False, align=None),
    "SCM_PD_DEVICE_SPECIFIC_PROPERTY": SimStruct(OrderedDict((("Name", SimTypeArray(SimTypeChar(label="Char"), 128)), ("Value", SimTypeLongLong(signed=True, label="Int64")),)), name="SCM_PD_DEVICE_SPECIFIC_PROPERTY", pack=False, align=None),
    "SCM_PD_FIRMWARE_ACTIVATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeChar(label="Byte")),)), name="SCM_PD_FIRMWARE_ACTIVATE", pack=False, align=None),
    "SCM_PD_FIRMWARE_ACTIVATION_STATE": SimTypeInt(signed=False, label="SCM_PD_FIRMWARE_ACTIVATION_STATE"),
    "SCM_PD_FIRMWARE_DOWNLOAD": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("FirmwareImageSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("FirmwareImage", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_FIRMWARE_DOWNLOAD", pack=False, align=None),
    "SCM_PD_FIRMWARE_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ActiveSlot", SimTypeChar(label="Byte")), ("NextActiveSlot", SimTypeChar(label="Byte")), ("SlotCount", SimTypeChar(label="Byte")), ("Slots", SimTypePointer(SimTypeRef("SCM_PD_FIRMWARE_SLOT_INFO", SimStruct), offset=0)),)), name="SCM_PD_FIRMWARE_INFO", pack=False, align=None),
    "SCM_PD_FIRMWARE_SLOT_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SlotNumber", SimTypeChar(label="Byte")), ("_bitfield", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Revision", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="SCM_PD_FIRMWARE_SLOT_INFO", pack=False, align=None),
    "SCM_PD_FRU_ID_STRING": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("IdentifierSize", SimTypeInt(signed=False, label="UInt32")), ("Identifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_FRU_ID_STRING", pack=False, align=None),
    "SCM_PD_HEALTH_NOTIFICATION_DATA": SimStruct(OrderedDict((("DeviceGuid", SimTypeBottom(label="Guid")),)), name="SCM_PD_HEALTH_NOTIFICATION_DATA", pack=False, align=None),
    "SCM_PD_HEALTH_STATUS": SimTypeInt(signed=False, label="SCM_PD_HEALTH_STATUS"),
    "SCM_PD_LAST_FW_ACTIVATION_STATUS": SimTypeInt(signed=False, label="SCM_PD_LAST_FW_ACTIVATION_STATUS"),
    "SCM_PD_LOCATION_STRING": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Location", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SCM_PD_LOCATION_STRING", pack=False, align=None),
    "SCM_PD_MANAGEMENT_STATUS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Health", SimTypeInt(signed=False, label="SCM_PD_HEALTH_STATUS")), ("NumberOfOperationalStatus", SimTypeInt(signed=False, label="UInt32")), ("NumberOfAdditionalReasons", SimTypeInt(signed=False, label="UInt32")), ("OperationalStatus", SimTypeArray(SimTypeInt(signed=False, label="SCM_PD_OPERATIONAL_STATUS"), 16)), ("AdditionalReasons", SimTypePointer(SimTypeInt(signed=False, label="SCM_PD_OPERATIONAL_STATUS_REASON"), offset=0)),)), name="SCM_PD_MANAGEMENT_STATUS", pack=False, align=None),
    "SCM_PD_MEDIA_REINITIALIZATION_STATUS": SimTypeInt(signed=False, label="SCM_PD_MEDIA_REINITIALIZATION_STATUS"),
    "SCM_PD_OPERATIONAL_STATUS": SimTypeInt(signed=False, label="SCM_PD_OPERATIONAL_STATUS"),
    "SCM_PD_OPERATIONAL_STATUS_REASON": SimTypeInt(signed=False, label="SCM_PD_OPERATIONAL_STATUS_REASON"),
    "SCM_PD_PASSTHROUGH_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ProtocolGuid", SimTypeBottom(label="Guid")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_PASSTHROUGH_INPUT", pack=False, align=None),
    "SCM_PD_PASSTHROUGH_INVDIMM_INPUT": SimStruct(OrderedDict((("Opcode", SimTypeInt(signed=False, label="UInt32")), ("OpcodeParametersLength", SimTypeInt(signed=False, label="UInt32")), ("OpcodeParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_PASSTHROUGH_INVDIMM_INPUT", pack=False, align=None),
    "SCM_PD_PASSTHROUGH_INVDIMM_OUTPUT": SimStruct(OrderedDict((("GeneralStatus", SimTypeShort(signed=False, label="UInt16")), ("ExtendedStatus", SimTypeShort(signed=False, label="UInt16")), ("OutputDataLength", SimTypeInt(signed=False, label="UInt32")), ("OutputData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_PASSTHROUGH_INVDIMM_OUTPUT", pack=False, align=None),
    "SCM_PD_PASSTHROUGH_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ProtocolGuid", SimTypeBottom(label="Guid")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_PASSTHROUGH_OUTPUT", pack=False, align=None),
    "SCM_PD_PROPERTY_ID": SimTypeInt(signed=False, label="SCM_PD_PROPERTY_ID"),
    "SCM_PD_PROPERTY_QUERY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PropertyId", SimTypeInt(signed=False, label="SCM_PD_PROPERTY_ID")), ("QueryType", SimTypeInt(signed=False, label="SCM_PD_QUERY_TYPE")), ("AdditionalParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_PROPERTY_QUERY", pack=False, align=None),
    "SCM_PD_PROPERTY_SET": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PropertyId", SimTypeInt(signed=False, label="SCM_PD_PROPERTY_ID")), ("SetType", SimTypeInt(signed=False, label="SCM_PD_SET_TYPE")), ("AdditionalParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCM_PD_PROPERTY_SET", pack=False, align=None),
    "SCM_PD_QUERY_TYPE": SimTypeInt(signed=False, label="SCM_PD_QUERY_TYPE"),
    "SCM_PD_REINITIALIZE_MEDIA_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Options", SimTypeRef("_Options_e__Struct", SimStruct)),)), name="SCM_PD_REINITIALIZE_MEDIA_INPUT", pack=False, align=None),
    "SCM_PD_REINITIALIZE_MEDIA_OUTPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="SCM_PD_MEDIA_REINITIALIZATION_STATUS")),)), name="SCM_PD_REINITIALIZE_MEDIA_OUTPUT", pack=False, align=None),
    "SCM_PD_RUNTIME_FW_ACTIVATION_ARM_STATE": SimStruct(OrderedDict((("ArmState", SimTypeChar(label="Byte")),)), name="SCM_PD_RUNTIME_FW_ACTIVATION_ARM_STATE", pack=False, align=None),
    "SCM_PD_RUNTIME_FW_ACTIVATION_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("LastFirmwareActivationStatus", SimTypeInt(signed=False, label="SCM_PD_LAST_FW_ACTIVATION_STATUS")), ("FirmwareActivationState", SimTypeInt(signed=False, label="SCM_PD_FIRMWARE_ACTIVATION_STATE")),)), name="SCM_PD_RUNTIME_FW_ACTIVATION_INFO", pack=False, align=None),
    "SCM_PD_SET_TYPE": SimTypeInt(signed=False, label="SCM_PD_SET_TYPE"),
    "SCM_PHYSICAL_DEVICES": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("Devices", SimTypePointer(SimTypeRef("SCM_PHYSICAL_DEVICE_INSTANCE", SimStruct), offset=0)),)), name="SCM_PHYSICAL_DEVICES", pack=False, align=None),
    "SCM_PHYSICAL_DEVICE_INSTANCE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NfitHandle", SimTypeInt(signed=False, label="UInt32")), ("SymbolicLink", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="SCM_PHYSICAL_DEVICE_INSTANCE", pack=False, align=None),
    "SCM_REGION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NfitHandle", SimTypeInt(signed=False, label="UInt32")), ("LogicalDeviceGuid", SimTypeBottom(label="Guid")), ("AddressRangeType", SimTypeBottom(label="Guid")), ("AssociatedId", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeLongLong(signed=False, label="UInt64")), ("StartingDPA", SimTypeLongLong(signed=False, label="UInt64")), ("BaseSPA", SimTypeLongLong(signed=False, label="UInt64")), ("SPAOffset", SimTypeLongLong(signed=False, label="UInt64")), ("RegionOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="SCM_REGION", pack=False, align=None),
    "SCM_REGIONS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("RegionCount", SimTypeInt(signed=False, label="UInt32")), ("Regions", SimTypePointer(SimTypeRef("SCM_REGION", SimStruct), offset=0)),)), name="SCM_REGIONS", pack=False, align=None),
    "SCM_REGION_FLAG": SimTypeInt(signed=False, label="SCM_REGION_FLAG"),
    "SCNRT_STATUS": SimTypeInt(signed=False, label="SCNRT_STATUS"),
    "SCONTEXT_QUEUE": SimStruct(OrderedDict((("NumberOfObjects", SimTypeInt(signed=False, label="UInt32")), ("ArrayOfObjects", SimTypePointer(SimTypePointer(SimTypeRef("NDR_SCONTEXT", SimStruct), offset=0), offset=0)),)), name="SCONTEXT_QUEUE", pack=False, align=None),
    "SCOPEDATAITEM": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="UInt32")), ("displayname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nImage", SimTypeInt(signed=True, label="Int32")), ("nOpenImage", SimTypeInt(signed=True, label="Int32")), ("nState", SimTypeInt(signed=False, label="UInt32")), ("cChildren", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("relativeID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SCOPEDATAITEM", pack=False, align=None),
    "SCOPE_ID": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="SCOPE_ID", pack=False, align=None),
    "SCOPE_LEVEL": SimTypeInt(signed=False, label="SCOPE_LEVEL"),
    "SCOPE_MIB_INFO": SimStruct(OrderedDict((("Subnet", SimTypeInt(signed=False, label="UInt32")), ("NumAddressesInuse", SimTypeInt(signed=False, label="UInt32")), ("NumAddressesFree", SimTypeInt(signed=False, label="UInt32")), ("NumPendingOffers", SimTypeInt(signed=False, label="UInt32")),)), name="SCOPE_MIB_INFO", pack=False, align=None),
    "SCOPE_MIB_INFO_V5": SimStruct(OrderedDict((("Subnet", SimTypeInt(signed=False, label="UInt32")), ("NumAddressesInuse", SimTypeInt(signed=False, label="UInt32")), ("NumAddressesFree", SimTypeInt(signed=False, label="UInt32")), ("NumPendingOffers", SimTypeInt(signed=False, label="UInt32")),)), name="SCOPE_MIB_INFO_V5", pack=False, align=None),
    "SCOPE_MIB_INFO_V6": SimStruct(OrderedDict((("Subnet", SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct)), ("NumAddressesInuse", SimTypeLongLong(signed=False, label="UInt64")), ("NumAddressesFree", SimTypeLongLong(signed=False, label="UInt64")), ("NumPendingAdvertises", SimTypeLongLong(signed=False, label="UInt64")),)), name="SCOPE_MIB_INFO_V6", pack=False, align=None),
    "SCOPE_MIB_INFO_VQ": SimStruct(OrderedDict((("Subnet", SimTypeInt(signed=False, label="UInt32")), ("NumAddressesInuse", SimTypeInt(signed=False, label="UInt32")), ("NumAddressesFree", SimTypeInt(signed=False, label="UInt32")), ("NumPendingOffers", SimTypeInt(signed=False, label="UInt32")), ("QtnNumLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnPctQtnLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnProbationLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnNonQtnLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnExemptLeases", SimTypeInt(signed=False, label="UInt32")), ("QtnCapableClients", SimTypeInt(signed=False, label="UInt32")),)), name="SCOPE_MIB_INFO_VQ", pack=False, align=None),
    "SCOPE_TABLE_AMD64": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("ScopeRecord", SimTypePointer(SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")), ("HandlerAddress", SimTypeInt(signed=False, label="UInt32")), ("JumpTarget", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="SCOPE_TABLE_AMD64", pack=False, align=None),
    "SCOPE_TABLE_ARM": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("ScopeRecord", SimTypePointer(SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")), ("HandlerAddress", SimTypeInt(signed=False, label="UInt32")), ("JumpTarget", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="SCOPE_TABLE_ARM", pack=False, align=None),
    "SCOPE_TABLE_ARM64": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("ScopeRecord", SimTypePointer(SimStruct(OrderedDict((("BeginAddress", SimTypeInt(signed=False, label="UInt32")), ("EndAddress", SimTypeInt(signed=False, label="UInt32")), ("HandlerAddress", SimTypeInt(signed=False, label="UInt32")), ("JumpTarget", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="SCOPE_TABLE_ARM64", pack=False, align=None),
    "SCRIPTCONTF": SimTypeInt(signed=False, label="SCRIPTCONTF"),
    "SCRIPTFLAGS": SimTypeInt(signed=False, label="SCRIPTFLAGS"),
    "SCRIPTFONTCONTF": SimTypeInt(signed=False, label="SCRIPTFONTCONTF"),
    "SCRIPTFONTINFO": SimStruct(OrderedDict((("scripts", SimTypeLongLong(signed=True, label="Int64")), ("wszFont", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="SCRIPTFONTINFO", pack=False, align=None),
    "SCRIPTGCTYPE": SimTypeInt(signed=False, label="SCRIPTGCTYPE"),
    "SCRIPTINFO": SimStruct(OrderedDict((("ScriptId", SimTypeChar(label="Byte")), ("uiCodePage", SimTypeInt(signed=False, label="UInt32")), ("wszDescription", SimTypeArray(SimTypeChar(label="Char"), 48)), ("wszFixedWidthFont", SimTypeArray(SimTypeChar(label="Char"), 32)), ("wszProportionalFont", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="SCRIPTINFO", pack=False, align=None),
    "SCRIPTLANGUAGEVERSION": SimTypeInt(signed=False, label="SCRIPTLANGUAGEVERSION"),
    "SCRIPTSTATE": SimTypeInt(signed=False, label="SCRIPTSTATE"),
    "SCRIPTTHREADSTATE": SimTypeInt(signed=False, label="SCRIPTTHREADSTATE"),
    "SCRIPTTRACEINFO": SimTypeInt(signed=False, label="SCRIPTTRACEINFO"),
    "SCRIPTUICHANDLING": SimTypeInt(signed=False, label="SCRIPTUICHANDLING"),
    "SCRIPTUICITEM": SimTypeInt(signed=False, label="SCRIPTUICITEM"),
    "SCRIPT_ANALYSIS": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("s", SimTypeRef("SCRIPT_STATE", SimStruct)),)), name="SCRIPT_ANALYSIS", pack=False, align=None),
    "SCRIPT_CHARPROP": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="SCRIPT_CHARPROP", pack=False, align=None),
    "SCRIPT_CONTROL": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="SCRIPT_CONTROL", pack=False, align=None),
    "SCRIPT_DEBUGGER_OPTIONS": SimTypeInt(signed=False, label="SCRIPT_DEBUGGER_OPTIONS"),
    "SCRIPT_DIGITSUBSTITUTE": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="SCRIPT_DIGITSUBSTITUTE", pack=False, align=None),
    "SCRIPT_ERROR_DEBUG_EXCEPTION_THROWN_KIND": SimTypeInt(signed=False, label="SCRIPT_ERROR_DEBUG_EXCEPTION_THROWN_KIND"),
    "SCRIPT_FONTPROPERTIES": SimStruct(OrderedDict((("cBytes", SimTypeInt(signed=True, label="Int32")), ("wgBlank", SimTypeShort(signed=False, label="UInt16")), ("wgDefault", SimTypeShort(signed=False, label="UInt16")), ("wgInvalid", SimTypeShort(signed=False, label="UInt16")), ("wgKashida", SimTypeShort(signed=False, label="UInt16")), ("iKashidaWidth", SimTypeInt(signed=True, label="Int32")),)), name="SCRIPT_FONTPROPERTIES", pack=False, align=None),
    "SCRIPT_GLYPHPROP": SimStruct(OrderedDict((("sva", SimTypeRef("SCRIPT_VISATTR", SimStruct)), ("reserved", SimTypeShort(signed=False, label="UInt16")),)), name="SCRIPT_GLYPHPROP", pack=False, align=None),
    "SCRIPT_INVOCATION_CONTEXT_TYPE": SimTypeInt(signed=False, label="SCRIPT_INVOCATION_CONTEXT_TYPE"),
    "SCRIPT_IS_COMPLEX_FLAGS": SimTypeInt(signed=False, label="SCRIPT_IS_COMPLEX_FLAGS"),
    "SCRIPT_ITEM": SimStruct(OrderedDict((("iCharPos", SimTypeInt(signed=True, label="Int32")), ("a", SimTypeRef("SCRIPT_ANALYSIS", SimStruct)),)), name="SCRIPT_ITEM", pack=False, align=None),
    "SCRIPT_JUSTIFY": SimTypeInt(signed=False, label="SCRIPT_JUSTIFY"),
    "SCRIPT_LOGATTR": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="SCRIPT_LOGATTR", pack=False, align=None),
    "SCRIPT_PROPERTIES": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")),)), name="SCRIPT_PROPERTIES", pack=False, align=None),
    "SCRIPT_STATE": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="SCRIPT_STATE", pack=False, align=None),
    "SCRIPT_TABDEF": SimStruct(OrderedDict((("cTabStops", SimTypeInt(signed=True, label="Int32")), ("iScale", SimTypeInt(signed=True, label="Int32")), ("pTabStops", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("iTabOrigin", SimTypeInt(signed=True, label="Int32")),)), name="SCRIPT_TABDEF", pack=False, align=None),
    "SCRIPT_TIMER_TYPE": SimTypeInt(signed=False, label="SCRIPT_TIMER_TYPE"),
    "SCRIPT_VISATTR": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="SCRIPT_VISATTR", pack=False, align=None),
    "SCROLLABLECONTEXTMENU_PLACEMENT": SimTypeInt(signed=False, label="SCROLLABLECONTEXTMENU_PLACEMENT"),
    "SCROLLBARINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcScrollBar", SimTypeRef("RECT", SimStruct)), ("dxyLineButton", SimTypeInt(signed=True, label="Int32")), ("xyThumbTop", SimTypeInt(signed=True, label="Int32")), ("xyThumbBottom", SimTypeInt(signed=True, label="Int32")), ("reserved", SimTypeInt(signed=True, label="Int32")), ("rgstate", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)),)), name="SCROLLBARINFO", pack=False, align=None),
    "SCROLLBARPARTS": SimTypeInt(signed=False, label="SCROLLBARPARTS"),
    "SCROLLBARSTYLESTATES": SimTypeInt(signed=False, label="SCROLLBARSTYLESTATES"),
    "SCROLLBAR_COMMAND": SimTypeInt(signed=False, label="SCROLLBAR_COMMAND"),
    "SCROLLBAR_CONSTANTS": SimTypeInt(signed=False, label="SCROLLBAR_CONSTANTS"),
    "SCROLLDIRECTION": SimTypeInt(signed=False, label="SCROLLDIRECTION"),
    "SCROLLINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="SCROLLINFO_MASK")), ("nMin", SimTypeInt(signed=True, label="Int32")), ("nMax", SimTypeInt(signed=True, label="Int32")), ("nPage", SimTypeInt(signed=False, label="UInt32")), ("nPos", SimTypeInt(signed=True, label="Int32")), ("nTrackPos", SimTypeInt(signed=True, label="Int32")),)), name="SCROLLINFO", pack=False, align=None),
    "SCROLLINFO_MASK": SimTypeInt(signed=False, label="SCROLLINFO_MASK"),
    "SCROLL_WINDOW_FLAGS": SimTypeInt(signed=False, label="SCROLL_WINDOW_FLAGS"),
    "SCRUB_DATA_INPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaximumIos", SimTypeInt(signed=False, label="UInt32")), ("ObjectId", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 41)), ("ResumeContext", SimTypeArray(SimTypeChar(label="Byte"), 1040)),)), name="SCRUB_DATA_INPUT", pack=False, align=None),
    "SCRUB_DATA_OUTPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")), ("ErrorFileOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ErrorLength", SimTypeLongLong(signed=False, label="UInt64")), ("NumberOfBytesRepaired", SimTypeLongLong(signed=False, label="UInt64")), ("NumberOfBytesFailed", SimTypeLongLong(signed=False, label="UInt64")), ("InternalFileReference", SimTypeLongLong(signed=False, label="UInt64")), ("ResumeContextLength", SimTypeShort(signed=False, label="UInt16")), ("ParityExtentDataOffset", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 9)), ("NumberOfMetadataBytesProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("NumberOfDataBytesProcessed", SimTypeLongLong(signed=False, label="UInt64")), ("TotalNumberOfMetadataBytesInUse", SimTypeLongLong(signed=False, label="UInt64")), ("TotalNumberOfDataBytesInUse", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToNoAllocation", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToInvalidRun", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToIntegrityStream", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToRegionBeingClean", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToLockConflict", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToNoScrubDataFlag", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesSkippedDueToNoScrubNonIntegrityStreamFlag", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesScrubbed", SimTypeLongLong(signed=False, label="UInt64")), ("ResumeContext", SimTypeArray(SimTypeChar(label="Byte"), 1040)),)), name="SCRUB_DATA_OUTPUT", pack=False, align=None),
    "SCRUB_PARITY_EXTENT": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="SCRUB_PARITY_EXTENT", pack=False, align=None),
    "SCRUB_PARITY_EXTENT_DATA": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("NumberOfParityExtents", SimTypeShort(signed=False, label="UInt16")), ("MaximumNumberOfParityExtents", SimTypeShort(signed=False, label="UInt16")), ("ParityExtents", SimTypePointer(SimTypeRef("SCRUB_PARITY_EXTENT", SimStruct), offset=0)),)), name="SCRUB_PARITY_EXTENT_DATA", pack=False, align=None),
    "SCSI_ADAPTER_BUS_INFO": SimStruct(OrderedDict((("NumberOfBuses", SimTypeChar(label="Byte")), ("BusData", SimTypePointer(SimTypeRef("SCSI_BUS_DATA", SimStruct), offset=0)),)), name="SCSI_ADAPTER_BUS_INFO", pack=False, align=None),
    "SCSI_ADDRESS": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("PortNumber", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")),)), name="SCSI_ADDRESS", pack=False, align=None),
    "SCSI_BUS_DATA": SimStruct(OrderedDict((("NumberOfLogicalUnits", SimTypeChar(label="Byte")), ("InitiatorBusId", SimTypeChar(label="Byte")), ("InquiryDataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="SCSI_BUS_DATA", pack=False, align=None),
    "SCSI_INQUIRY_DATA": SimStruct(OrderedDict((("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("DeviceClaimed", SimTypeChar(label="Byte")), ("InquiryDataLength", SimTypeInt(signed=False, label="UInt32")), ("NextInquiryDataOffset", SimTypeInt(signed=False, label="UInt32")), ("InquiryData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCSI_INQUIRY_DATA", pack=False, align=None),
    "SCSI_LUN_LIST": SimStruct(OrderedDict((("OSLUN", SimTypeInt(signed=False, label="UInt32")), ("TargetLUN", SimTypeLongLong(signed=False, label="UInt64")),)), name="SCSI_LUN_LIST", pack=False, align=None),
    "SCSI_PASS_THROUGH": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("ScsiStatus", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("CdbLength", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataIn", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("DataBufferOffset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("Cdb", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SCSI_PASS_THROUGH", pack=False, align=None),
    "SCSI_PASS_THROUGH32": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("ScsiStatus", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("CdbLength", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataIn", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("DataBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("Cdb", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SCSI_PASS_THROUGH32", pack=False, align=None),
    "SCSI_PASS_THROUGH32_EX": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("CdbLength", SimTypeInt(signed=False, label="UInt32")), ("StorAddressLength", SimTypeInt(signed=False, label="UInt32")), ("ScsiStatus", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataDirection", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("StorAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("DataOutTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataInTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataOutBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("DataInBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("Cdb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCSI_PASS_THROUGH32_EX", pack=False, align=None),
    "SCSI_PASS_THROUGH_DIRECT": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("ScsiStatus", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("CdbLength", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataIn", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("Cdb", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SCSI_PASS_THROUGH_DIRECT", pack=False, align=None),
    "SCSI_PASS_THROUGH_DIRECT32": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("ScsiStatus", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")), ("CdbLength", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataIn", SimTypeChar(label="Byte")), ("DataTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("Cdb", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SCSI_PASS_THROUGH_DIRECT32", pack=False, align=None),
    "SCSI_PASS_THROUGH_DIRECT32_EX": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("CdbLength", SimTypeInt(signed=False, label="UInt32")), ("StorAddressLength", SimTypeInt(signed=False, label="UInt32")), ("ScsiStatus", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataDirection", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("StorAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("DataOutTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataInTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataOutBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DataInBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Cdb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCSI_PASS_THROUGH_DIRECT32_EX", pack=False, align=None),
    "SCSI_PASS_THROUGH_DIRECT_EX": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("CdbLength", SimTypeInt(signed=False, label="UInt32")), ("StorAddressLength", SimTypeInt(signed=False, label="UInt32")), ("ScsiStatus", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataDirection", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("StorAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("DataOutTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataInTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataOutBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DataInBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Cdb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCSI_PASS_THROUGH_DIRECT_EX", pack=False, align=None),
    "SCSI_PASS_THROUGH_EX": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("CdbLength", SimTypeInt(signed=False, label="UInt32")), ("StorAddressLength", SimTypeInt(signed=False, label="UInt32")), ("ScsiStatus", SimTypeChar(label="Byte")), ("SenseInfoLength", SimTypeChar(label="Byte")), ("DataDirection", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("StorAddressOffset", SimTypeInt(signed=False, label="UInt32")), ("SenseInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("DataOutTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataInTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataOutBufferOffset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DataInBufferOffset", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Cdb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SCSI_PASS_THROUGH_EX", pack=False, align=None),
    "SC_ACTION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="SC_ACTION_TYPE")), ("Delay", SimTypeInt(signed=False, label="UInt32")),)), name="SC_ACTION", pack=False, align=None),
    "SC_ACTION_TYPE": SimTypeInt(signed=False, label="SC_ACTION_TYPE"),
    "SC_ENUM_TYPE": SimTypeInt(signed=False, label="SC_ENUM_TYPE"),
    "SC_EVENT_TYPE": SimTypeInt(signed=False, label="SC_EVENT_TYPE"),
    "SC_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SC_STATUS_TYPE": SimTypeInt(signed=False, label="SC_STATUS_TYPE"),
    "SChannelHookCallInfo": SimStruct(OrderedDict((("iid", SimTypeBottom(label="Guid")), ("cbSize", SimTypeInt(signed=False, label="UInt32")), ("uCausality", SimTypeBottom(label="Guid")), ("dwServerPid", SimTypeInt(signed=False, label="UInt32")), ("iMethod", SimTypeInt(signed=False, label="UInt32")), ("pObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SChannelHookCallInfo", pack=False, align=None),
    "SColumnSetID": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cBytes", SimTypeInt(signed=False, label="UInt32")), ("id", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SColumnSetID", pack=False, align=None),
    "SCommentRestriction": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpRes", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)), ("lpProp", SimTypePointer(SimTypeRef("SPropValue", SimStruct), offset=0)),)), name="SCommentRestriction", pack=False, align=None),
    "SComparePropsRestriction": SimStruct(OrderedDict((("relop", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag1", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag2", SimTypeInt(signed=False, label="UInt32")),)), name="SComparePropsRestriction", pack=False, align=None),
    "SContentRestriction": SimStruct(OrderedDict((("ulFuzzyLevel", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("lpProp", SimTypePointer(SimTypeRef("SPropValue", SimStruct), offset=0)),)), name="SContentRestriction", pack=False, align=None),
    "SCurrencyArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpcur", SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), offset=0)),)), name="SCurrencyArray", pack=False, align=None),
    "SDEV_IDENTIFIER_INTERFACE": SimStruct(OrderedDict((("InterfaceHeader", SimTypeRef("INTERFACE", SimStruct)), ("GetIdentifier", SimTypePointer(SimTypeFunction([], SimTypeLongLong(signed=False, label="UInt64")), offset=0)),)), name="SDEV_IDENTIFIER_INTERFACE", pack=False, align=None),
    "SDP_ELEMENT_DATA": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="SDP_TYPE")), ("specificType", SimTypeInt(signed=False, label="SDP_SPECIFICTYPE")), ("data", SimUnion({"int128": SimTypeRef("SDP_LARGE_INTEGER_16", SimStruct), "int64": SimTypeLongLong(signed=True, label="Int64"), "int32": SimTypeInt(signed=True, label="Int32"), "int16": SimTypeShort(signed=True, label="Int16"), "int8": SimTypeChar(label="SByte"), "uint128": SimTypeRef("SDP_ULARGE_INTEGER_16", SimStruct), "uint64": SimTypeLongLong(signed=False, label="UInt64"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint8": SimTypeChar(label="Byte"), "booleanVal": SimTypeChar(label="Byte"), "uuid128": SimTypeBottom(label="Guid"), "uuid32": SimTypeInt(signed=False, label="UInt32"), "uuid16": SimTypeShort(signed=False, label="UInt16"), "string": SimTypeRef("_string_e__Struct", SimStruct), "url": SimTypeRef("_url_e__Struct", SimStruct), "sequence": SimTypeRef("_sequence_e__Struct", SimStruct), "alternative": SimTypeRef("_alternative_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="SDP_ELEMENT_DATA", pack=False, align=None),
    "SDP_LARGE_INTEGER_16": SimStruct(OrderedDict((("LowPart", SimTypeLongLong(signed=False, label="UInt64")), ("HighPart", SimTypeLongLong(signed=True, label="Int64")),)), name="SDP_LARGE_INTEGER_16", pack=False, align=None),
    "SDP_SPECIFICTYPE": SimTypeInt(signed=False, label="SDP_SPECIFICTYPE"),
    "SDP_STRING_TYPE_DATA": SimStruct(OrderedDict((("encoding", SimTypeShort(signed=False, label="UInt16")), ("mibeNum", SimTypeShort(signed=False, label="UInt16")), ("attributeId", SimTypeShort(signed=False, label="UInt16")),)), name="SDP_STRING_TYPE_DATA", pack=False, align=None),
    "SDP_TYPE": SimTypeInt(signed=False, label="SDP_TYPE"),
    "SDP_ULARGE_INTEGER_16": SimStruct(OrderedDict((("LowPart", SimTypeLongLong(signed=False, label="UInt64")), ("HighPart", SimTypeLongLong(signed=False, label="UInt64")),)), name="SDP_ULARGE_INTEGER_16", pack=False, align=None),
    "SD_CHANGE_MACHINE_SID_INPUT": SimStruct(OrderedDict((("CurrentMachineSIDOffset", SimTypeShort(signed=False, label="UInt16")), ("CurrentMachineSIDLength", SimTypeShort(signed=False, label="UInt16")), ("NewMachineSIDOffset", SimTypeShort(signed=False, label="UInt16")), ("NewMachineSIDLength", SimTypeShort(signed=False, label="UInt16")),)), name="SD_CHANGE_MACHINE_SID_INPUT", pack=False, align=None),
    "SD_CHANGE_MACHINE_SID_OUTPUT": SimStruct(OrderedDict((("NumSDChangedSuccess", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDChangedFail", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDUnused", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDTotal", SimTypeLongLong(signed=False, label="UInt64")), ("NumMftSDChangedSuccess", SimTypeLongLong(signed=False, label="UInt64")), ("NumMftSDChangedFail", SimTypeLongLong(signed=False, label="UInt64")), ("NumMftSDTotal", SimTypeLongLong(signed=False, label="UInt64")),)), name="SD_CHANGE_MACHINE_SID_OUTPUT", pack=False, align=None),
    "SD_ENUM_SDS_ENTRY": SimStruct(OrderedDict((("Hash", SimTypeInt(signed=False, label="UInt32")), ("SecurityId", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Descriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SD_ENUM_SDS_ENTRY", pack=False, align=None),
    "SD_ENUM_SDS_INPUT": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=False, label="UInt64")), ("MaxSDEntriesToReturn", SimTypeLongLong(signed=False, label="UInt64")),)), name="SD_ENUM_SDS_INPUT", pack=False, align=None),
    "SD_ENUM_SDS_OUTPUT": SimStruct(OrderedDict((("NextOffset", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDEntriesReturned", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDBytesReturned", SimTypeLongLong(signed=False, label="UInt64")), ("SDEntry", SimTypePointer(SimTypeRef("SD_ENUM_SDS_ENTRY", SimStruct), offset=0)),)), name="SD_ENUM_SDS_OUTPUT", pack=False, align=None),
    "SD_GLOBAL_CHANGE_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ChangeType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"SdChange": SimTypeRef("SD_CHANGE_MACHINE_SID_INPUT", SimStruct), "SdQueryStats": SimTypeRef("SD_QUERY_STATS_INPUT", SimStruct), "SdEnumSds": SimTypeRef("SD_ENUM_SDS_INPUT", SimStruct)}, name="<anon>", label="None")),)), name="SD_GLOBAL_CHANGE_INPUT", pack=False, align=None),
    "SD_GLOBAL_CHANGE_OUTPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ChangeType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"SdChange": SimTypeRef("SD_CHANGE_MACHINE_SID_OUTPUT", SimStruct), "SdQueryStats": SimTypeRef("SD_QUERY_STATS_OUTPUT", SimStruct), "SdEnumSds": SimTypeRef("SD_ENUM_SDS_OUTPUT", SimStruct)}, name="<anon>", label="None")),)), name="SD_GLOBAL_CHANGE_OUTPUT", pack=False, align=None),
    "SD_QUERY_STATS_INPUT": SimStruct(OrderedDict((("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="SD_QUERY_STATS_INPUT", pack=False, align=None),
    "SD_QUERY_STATS_OUTPUT": SimStruct(OrderedDict((("SdsStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("SdsAllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("SiiStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("SiiAllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("SdhStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("SdhAllocationSize", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDTotal", SimTypeLongLong(signed=False, label="UInt64")), ("NumSDUnused", SimTypeLongLong(signed=False, label="UInt64")),)), name="SD_QUERY_STATS_OUTPUT", pack=False, align=None),
    "SDateTimeArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpft", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)),)), name="SDateTimeArray", pack=False, align=None),
    "SDoubleArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpdbl", SimTypePointer(SimTypeFloat(size=64), offset=0)),)), name="SDoubleArray", pack=False, align=None),
    "SEALING_SIGNATURE_ATTRIBUTE": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("signerIndex", SimTypeInt(signed=False, label="UInt32")), ("signatureAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("encryptedDigest", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SEALING_SIGNATURE_ATTRIBUTE", pack=False, align=None),
    "SEALING_TIMESTAMP_ATTRIBUTE": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("signerIndex", SimTypeInt(signed=False, label="UInt32")), ("sealTimeStampToken", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SEALING_TIMESTAMP_ATTRIBUTE", pack=False, align=None),
    "SEARCHMEMORY": SimStruct(OrderedDict((("SearchAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SearchLength", SimTypeLongLong(signed=False, label="UInt64")), ("FoundAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PatternLength", SimTypeInt(signed=False, label="UInt32")), ("Pattern", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SEARCHMEMORY", pack=False, align=None),
    "SEARCH_COLUMN_PROPERTIES": SimStruct(OrderedDict((("Value", SimTypeRef("PROPVARIANT", SimStruct)), ("lcid", SimTypeInt(signed=False, label="UInt32")),)), name="SEARCH_COLUMN_PROPERTIES", pack=False, align=None),
    "SEARCH_INDEXING_PHASE": SimTypeInt(signed=False, label="SEARCH_INDEXING_PHASE"),
    "SEARCH_ITEM_CHANGE": SimStruct(OrderedDict((("Change", SimTypeInt(signed=False, label="SEARCH_KIND_OF_CHANGE")), ("Priority", SimTypeInt(signed=False, label="SEARCH_NOTIFICATION_PRIORITY")), ("pUserData", SimTypePointer(SimTypeRef("BLOB", SimStruct), offset=0)), ("lpwszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpwszOldURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SEARCH_ITEM_CHANGE", pack=False, align=None),
    "SEARCH_ITEM_INDEXING_STATUS": SimStruct(OrderedDict((("dwDocID", SimTypeInt(signed=False, label="UInt32")), ("hrIndexingStatus", SimTypeInt(signed=True, label="Int32")),)), name="SEARCH_ITEM_INDEXING_STATUS", pack=False, align=None),
    "SEARCH_ITEM_PERSISTENT_CHANGE": SimStruct(OrderedDict((("Change", SimTypeInt(signed=False, label="SEARCH_KIND_OF_CHANGE")), ("URL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("OldURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Priority", SimTypeInt(signed=False, label="SEARCH_NOTIFICATION_PRIORITY")),)), name="SEARCH_ITEM_PERSISTENT_CHANGE", pack=False, align=None),
    "SEARCH_KIND_OF_CHANGE": SimTypeInt(signed=False, label="SEARCH_KIND_OF_CHANGE"),
    "SEARCH_NOTIFICATION_PRIORITY": SimTypeInt(signed=False, label="SEARCH_NOTIFICATION_PRIORITY"),
    "SEARCH_QUERY_SYNTAX": SimTypeInt(signed=False, label="SEARCH_QUERY_SYNTAX"),
    "SEARCH_TERM_EXPANSION": SimTypeInt(signed=False, label="SEARCH_TERM_EXPANSION"),
    "SECPKG_APP_MODE_INFO": SimStruct(OrderedDict((("UserFunction", SimTypeInt(signed=False, label="UInt32")), ("Argument1", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Argument2", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("UserData", SimTypeRef("SecBuffer", SimStruct)), ("ReturnToLsa", SimTypeChar(label="Byte")),)), name="SECPKG_APP_MODE_INFO", pack=False, align=None),
    "SECPKG_ATTR": SimTypeInt(signed=False, label="SECPKG_ATTR"),
    "SECPKG_ATTR_LCT_STATUS": SimTypeInt(signed=False, label="SECPKG_ATTR_LCT_STATUS"),
    "SECPKG_BYTE_VECTOR": SimStruct(OrderedDict((("ByteArrayOffset", SimTypeInt(signed=False, label="UInt32")), ("ByteArrayLength", SimTypeShort(signed=False, label="UInt16")),)), name="SECPKG_BYTE_VECTOR", pack=False, align=None),
    "SECPKG_CALL_INFO": SimStruct(OrderedDict((("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("CallCount", SimTypeInt(signed=False, label="UInt32")), ("MechOid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECPKG_CALL_INFO", pack=False, align=None),
    "SECPKG_CALL_PACKAGE_MESSAGE_TYPE": SimTypeInt(signed=False, label="SECPKG_CALL_PACKAGE_MESSAGE_TYPE"),
    "SECPKG_CALL_PACKAGE_PIN_DC_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DcName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DcFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_CALL_PACKAGE_PIN_DC_REQUEST", pack=False, align=None),
    "SECPKG_CALL_PACKAGE_TRANSFER_CRED_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="UInt32")), ("OriginLogonId", SimTypeRef("LUID", SimStruct)), ("DestinationLogonId", SimTypeRef("LUID", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_CALL_PACKAGE_TRANSFER_CRED_REQUEST", pack=False, align=None),
    "SECPKG_CALL_PACKAGE_UNPIN_ALL_DCS_REQUEST": SimStruct(OrderedDict((("MessageType", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_CALL_PACKAGE_UNPIN_ALL_DCS_REQUEST", pack=False, align=None),
    "SECPKG_CLIENT_INFO": SimStruct(OrderedDict((("LogonId", SimTypeRef("LUID", SimStruct)), ("ProcessID", SimTypeInt(signed=False, label="UInt32")), ("ThreadID", SimTypeInt(signed=False, label="UInt32")), ("HasTcbPrivilege", SimTypeChar(label="Byte")), ("Impersonating", SimTypeChar(label="Byte")), ("Restricted", SimTypeChar(label="Byte")), ("ClientFlags", SimTypeChar(label="Byte")), ("ImpersonationLevel", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")), ("ClientToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SECPKG_CLIENT_INFO", pack=False, align=None),
    "SECPKG_CLIENT_INFO_EX": SimStruct(OrderedDict((("LogonId", SimTypeRef("LUID", SimStruct)), ("ProcessID", SimTypeInt(signed=False, label="UInt32")), ("ThreadID", SimTypeInt(signed=False, label="UInt32")), ("HasTcbPrivilege", SimTypeChar(label="Byte")), ("Impersonating", SimTypeChar(label="Byte")), ("Restricted", SimTypeChar(label="Byte")), ("ClientFlags", SimTypeChar(label="Byte")), ("ImpersonationLevel", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")), ("ClientToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IdentificationLogonId", SimTypeRef("LUID", SimStruct)), ("IdentificationToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SECPKG_CLIENT_INFO_EX", pack=False, align=None),
    "SECPKG_CONTEXT_THUNKS": SimStruct(OrderedDict((("InfoLevelCount", SimTypeInt(signed=False, label="UInt32")), ("Levels", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="SECPKG_CONTEXT_THUNKS", pack=False, align=None),
    "SECPKG_CRED": SimTypeInt(signed=False, label="SECPKG_CRED"),
    "SECPKG_CREDENTIAL": SimStruct(OrderedDict((("Version", SimTypeLongLong(signed=False, label="UInt64")), ("cbHeaderLength", SimTypeShort(signed=False, label="UInt16")), ("cbStructureLength", SimTypeInt(signed=False, label="UInt32")), ("ClientProcess", SimTypeInt(signed=False, label="UInt32")), ("ClientThread", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("ClientToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("ModifiedId", SimTypeRef("LUID", SimStruct)), ("fCredentials", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PrincipalName", SimTypeRef("SECPKG_BYTE_VECTOR", SimStruct)), ("PackageList", SimTypeRef("SECPKG_BYTE_VECTOR", SimStruct)), ("MarshaledSuppliedCreds", SimTypeRef("SECPKG_BYTE_VECTOR", SimStruct)),)), name="SECPKG_CREDENTIAL", pack=False, align=None),
    "SECPKG_CRED_CLASS": SimTypeInt(signed=False, label="SECPKG_CRED_CLASS"),
    "SECPKG_DLL_FUNCTIONS": SimStruct(OrderedDict((("AllocateHeap", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0)), ("FreeHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0)), ("RegisterCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Argument1", "Argument2", "InputBuffer", "OutputBuffer"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CallbackId", "Callback"]), offset=0)), ("LocatePackageById", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PackgeId"]), offset=0)),)), name="SECPKG_DLL_FUNCTIONS", pack=False, align=None),
    "SECPKG_EVENT_NOTIFY": SimStruct(OrderedDict((("EventClass", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("EventDataSize", SimTypeInt(signed=False, label="UInt32")), ("EventData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PackageParameter", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECPKG_EVENT_NOTIFY", pack=False, align=None),
    "SECPKG_EVENT_PACKAGE_CHANGE": SimStruct(OrderedDict((("ChangeType", SimTypeInt(signed=False, label="SECPKG_PACKAGE_CHANGE_TYPE")), ("PackageId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PackageName", SimTypeRef("SECURITY_STRING", SimStruct)),)), name="SECPKG_EVENT_PACKAGE_CHANGE", pack=False, align=None),
    "SECPKG_EVENT_ROLE_CHANGE": SimStruct(OrderedDict((("PreviousRole", SimTypeInt(signed=False, label="UInt32")), ("NewRole", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_EVENT_ROLE_CHANGE", pack=False, align=None),
    "SECPKG_EXTENDED_INFORMATION": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="SECPKG_EXTENDED_INFORMATION_CLASS")), ("Info", SimUnion({"GssInfo": SimTypeRef("SECPKG_GSS_INFO", SimStruct), "ContextThunks": SimTypeRef("SECPKG_CONTEXT_THUNKS", SimStruct), "MutualAuthLevel": SimTypeRef("SECPKG_MUTUAL_AUTH_LEVEL", SimStruct), "WowClientDll": SimTypeRef("SECPKG_WOW_CLIENT_DLL", SimStruct), "ExtraOids": SimTypeRef("SECPKG_EXTRA_OIDS", SimStruct), "Nego2Info": SimTypeRef("SECPKG_NEGO2_INFO", SimStruct)}, name="<anon>", label="None")),)), name="SECPKG_EXTENDED_INFORMATION", pack=False, align=None),
    "SECPKG_EXTENDED_INFORMATION_CLASS": SimTypeInt(signed=False, label="SECPKG_EXTENDED_INFORMATION_CLASS"),
    "SECPKG_EXTRA_OIDS": SimStruct(OrderedDict((("OidCount", SimTypeInt(signed=False, label="UInt32")), ("Oids", SimTypePointer(SimTypeRef("SECPKG_SERIALIZED_OID", SimStruct), offset=0)),)), name="SECPKG_EXTRA_OIDS", pack=False, align=None),
    "SECPKG_FUNCTION_TABLE": SimStruct(OrderedDict((("InitializePackage", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LSA_DISPATCH_TABLE", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_STRING", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AuthenticationPackageId", "LsaDispatchTable", "Database", "Confidentiality", "AuthenticationPackageName"]), offset=0)), ("LogonUserA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "AuthenticationInformation", "ClientAuthenticationBase", "AuthenticationInformationLength", "ProfileBuffer", "ProfileBufferLength", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority"]), offset=0)), ("CallPackage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0)), ("LogonTerminated", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["LogonId"]), offset=0)), ("CallPackageUntrusted", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0)), ("CallPackagePassthrough", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ProtocolReturnBuffer", "ReturnBufferLength", "ProtocolStatus"]), offset=0)), ("LogonUserExA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "AuthenticationInformation", "ClientAuthenticationBase", "AuthenticationInformationLength", "ProfileBuffer", "ProfileBufferLength", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName"]), offset=0)), ("LogonUserEx2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "ProfileBuffer", "ProfileBufferSize", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0)), ("Initialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SECPKG_PARAMETERS", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_SECPKG_FUNCTION_TABLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PackageId", "Parameters", "FunctionTable"]), offset=0)), ("Shutdown", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("GetInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PackageInfo"]), offset=0)), ("AcceptCredentials", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonType", "AccountName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0)), ("AcquireCredentialsHandleA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrincipalName", "CredentialUseFlags", "LogonId", "AuthorizationData", "GetKeyFunciton", "GetKeyArgument", "CredentialHandle", "ExpirationTime"]), offset=0)), ("QueryCredentialsAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "CredentialAttribute", "Buffer"]), offset=0)), ("FreeCredentialsHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle"]), offset=0)), ("SaveCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "Credentials"]), offset=0)), ("GetCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "Credentials"]), offset=0)), ("DeleteCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "Key"]), offset=0)), ("InitLsaModeContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "ContextHandle", "TargetName", "ContextRequirements", "TargetDataRep", "InputBuffers", "NewContextHandle", "OutputBuffers", "ContextAttributes", "ExpirationTime", "MappedContext", "ContextData"]), offset=0)), ("AcceptLsaModeContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "ContextHandle", "InputBuffer", "ContextRequirements", "TargetDataRep", "NewContextHandle", "OutputBuffer", "ContextAttributes", "ExpirationTime", "MappedContext", "ContextData"]), offset=0)), ("DeleteContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle"]), offset=0)), ("ApplyControlToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ControlToken"]), offset=0)), ("GetUserInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("SECURITY_USER_DATA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "Flags", "UserData"]), offset=0)), ("GetExtendedInformation", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SECPKG_EXTENDED_INFORMATION_CLASS"), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_EXTENDED_INFORMATION", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Class", "ppInformation"]), offset=0)), ("QueryContextAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ContextAttribute", "Buffer"]), offset=0)), ("AddCredentialsA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "PrincipalName", "Package", "CredentialUseFlags", "AuthorizationData", "GetKeyFunciton", "GetKeyArgument", "ExpirationTime"]), offset=0)), ("SetExtendedInformation", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SECPKG_EXTENDED_INFORMATION_CLASS"), SimTypePointer(SimTypeRef("SECPKG_EXTENDED_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Class", "Info"]), offset=0)), ("SetContextAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ContextAttribute", "Buffer", "BufferSize"]), offset=0)), ("SetCredentialsAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "CredentialAttribute", "Buffer", "BufferSize"]), offset=0)), ("ChangeAccountPasswordA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDomainName", "pAccountName", "pOldPassword", "pNewPassword", "Impersonating", "pOutput"]), offset=0)), ("QueryMetaData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "TargetName", "ContextRequirements", "MetaDataLength", "MetaData", "ContextHandle"]), offset=0)), ("ExchangeMetaData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "TargetName", "ContextRequirements", "MetaDataLength", "MetaData", "ContextHandle"]), offset=0)), ("GetCredUIContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "CredType", "FlatCredUIContextLength", "FlatCredUIContext"]), offset=0)), ("UpdateCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "CredType", "FlatCredUIContextLength", "FlatCredUIContext"]), offset=0)), ("ValidateTargetInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_TARGETINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "TargetInfo"]), offset=0)), ("PostLogonUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_POST_LOGON_USER_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PostLogonUserInfo"]), offset=0)), ("GetRemoteCredGuardLogonBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Buffer", "BufferLength", "ReturnBuffer", "ReturnBufferLength"]), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["RedirectedLogonHandle"]), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredHandle", "ContextHandle", "TargetName", "RedirectedLogonHandle", "Callback", "CleanupCallback", "LogonBufferSize", "LogonBuffer"]), offset=0)), ("GetRemoteCredGuardSupplementalCreds", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Buffer", "BufferLength", "ReturnBuffer", "ReturnBufferLength"]), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["RedirectedLogonHandle"]), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredHandle", "TargetName", "RedirectedLogonHandle", "Callback", "CleanupCallback", "SupplementalCredsSize", "SupplementalCreds"]), offset=0)), ("GetTbalSupplementalCreds", SimTypePointer(SimTypeFunction([SimTypeRef("LUID", SimStruct), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "SupplementalCredsSize", "SupplementalCreds"]), offset=0)), ("LogonUserEx3", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "SurrogateLogon", "ProfileBuffer", "ProfileBufferSize", "LogonId", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0)), ("PreLogonUserSurrogate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "SurrogateLogon", "SubStatus"]), offset=0)), ("PostLogonUserSurrogate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="LSA_TOKEN_INFORMATION_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "LogonType", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferSize", "SurrogateLogon", "ProfileBuffer", "ProfileBufferSize", "LogonId", "Status", "SubStatus", "TokenInformationType", "TokenInformation", "AccountName", "AuthenticatingAuthority", "MachineName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0)), ("ExtractTargetInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ppvTargetInfo", "pcbTargetInfo"]), offset=0)),)), name="SECPKG_FUNCTION_TABLE", pack=False, align=None),
    "SECPKG_GSS_INFO": SimStruct(OrderedDict((("EncodedIdLength", SimTypeInt(signed=False, label="UInt32")), ("EncodedId", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="SECPKG_GSS_INFO", pack=False, align=None),
    "SECPKG_KERNEL_FUNCTIONS": SimStruct(OrderedDict((("AllocateHeap", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Length"]), offset=0)), ("FreeHeap", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Base"]), offset=0)), ("CreateContextList", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="KSEC_CONTEXT_TYPE")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Type"]), offset=0)), ("InsertListEntry", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("KSEC_LIST_ENTRY", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["List", "Entry"]), offset=0)), ("ReferenceListEntry", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KSEC_LIST_ENTRY", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Entry", "Signature", "RemoveNoRef"]), offset=0)), ("DereferenceListEntry", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("KSEC_LIST_ENTRY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Entry", "Delete"]), offset=0)), ("SerializeWinntAuthData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvAuthData", "Size", "SerializedData"]), offset=0)), ("SerializeSchannelAuthData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvAuthData", "Size", "SerializedData"]), offset=0)), ("LocatePackageById", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PackageId"]), offset=0)),)), name="SECPKG_KERNEL_FUNCTIONS", pack=False, align=None),
    "SECPKG_KERNEL_FUNCTION_TABLE": SimStruct(OrderedDict((("Initialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SECPKG_KERNEL_FUNCTIONS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FunctionTable"]), offset=0)), ("DeleteContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "LsaContextId"]), offset=0)), ("InitContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "ContextData", "NewContextId"]), offset=0)), ("MapHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "LsaContextId"]), offset=0)), ("Sign", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "fQOP", "Message", "MessageSeqNo"]), offset=0)), ("Verify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Message", "MessageSeqNo", "pfQOP"]), offset=0)), ("Seal", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "fQOP", "Message", "MessageSeqNo"]), offset=0)), ("Unseal", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Message", "MessageSeqNo", "pfQOP"]), offset=0)), ("GetToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "ImpersonationToken", "RawToken"]), offset=0)), ("QueryAttributes", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Attribute", "Buffer"]), offset=0)), ("CompleteToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextId", "Token"]), offset=0)), ("ExportContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phContext", "fFlags", "pPackedContext", "pToken"]), offset=0)), ("ImportContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPackedContext", "Token", "phContext"]), offset=0)), ("SetPackagePagingMode", SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["PagingMode"]), offset=0)), ("SerializeAuthData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pvAuthData", "Size", "SerializedData"]), offset=0)),)), name="SECPKG_KERNEL_FUNCTION_TABLE", pack=False, align=None),
    "SECPKG_MUTUAL_AUTH_LEVEL": SimStruct(OrderedDict((("MutualAuthLevel", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_MUTUAL_AUTH_LEVEL", pack=False, align=None),
    "SECPKG_NAME_TYPE": SimTypeInt(signed=False, label="SECPKG_NAME_TYPE"),
    "SECPKG_NEGO2_INFO": SimStruct(OrderedDict((("AuthScheme", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PackageFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_NEGO2_INFO", pack=False, align=None),
    "SECPKG_NTLM_TARGETINFO": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("MsvAvNbComputerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MsvAvNbDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MsvAvDnsComputerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MsvAvDnsDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MsvAvDnsTreeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MsvAvFlags", SimTypeInt(signed=False, label="UInt32")), ("MsvAvTimestamp", SimTypeRef("FILETIME", SimStruct)), ("MsvAvTargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SECPKG_NTLM_TARGETINFO", pack=False, align=None),
    "SECPKG_PACKAGE_CHANGE_TYPE": SimTypeInt(signed=False, label="SECPKG_PACKAGE_CHANGE_TYPE"),
    "SECPKG_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("MachineState", SimTypeInt(signed=False, label="UInt32")), ("SetupMode", SimTypeInt(signed=False, label="UInt32")), ("DomainSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DnsDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainGuid", SimTypeBottom(label="Guid")),)), name="SECPKG_PARAMETERS", pack=False, align=None),
    "SECPKG_POST_LOGON_USER_INFO": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("LinkedLogonId", SimTypeRef("LUID", SimStruct)),)), name="SECPKG_POST_LOGON_USER_INFO", pack=False, align=None),
    "SECPKG_PRIMARY_CRED": SimStruct(OrderedDict((("LogonId", SimTypeRef("LUID", SimStruct)), ("DownlevelName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("OldPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DnsDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Upn", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare1", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare2", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare3", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare4", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="SECPKG_PRIMARY_CRED", pack=False, align=None),
    "SECPKG_PRIMARY_CRED_EX": SimStruct(OrderedDict((("LogonId", SimTypeRef("LUID", SimStruct)), ("DownlevelName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("OldPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DnsDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Upn", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare1", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare2", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare3", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Spare4", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("PackageId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrevLogonId", SimTypeRef("LUID", SimStruct)), ("FlagsEx", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_PRIMARY_CRED_EX", pack=False, align=None),
    "SECPKG_REDIRECTED_LOGON_BUFFER": SimStruct(OrderedDict((("RedirectedLogonGuid", SimTypeBottom(label="Guid")), ("RedirectedLogonHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Init", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "PackageName", "SessionId", "LogonId"]), offset=0)), ("Callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Buffer", "BufferLength", "ReturnBuffer", "ReturnBufferLength"]), offset=0)), ("CleanupCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["RedirectedLogonHandle"]), offset=0)), ("GetLogonCreds", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "LogonBuffer", "LogonBufferLength"]), offset=0)), ("GetSupplementalCreds", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED_ARRAY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "SupplementalCredentials"]), offset=0)), ("GetRedirectedLogonSid", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Sid"]), offset=0)),)), name="SECPKG_REDIRECTED_LOGON_BUFFER", pack=False, align=None),
    "SECPKG_SERIALIZED_OID": SimStruct(OrderedDict((("OidLength", SimTypeInt(signed=False, label="UInt32")), ("OidAttributes", SimTypeInt(signed=False, label="UInt32")), ("OidValue", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="SECPKG_SERIALIZED_OID", pack=False, align=None),
    "SECPKG_SESSIONINFO_TYPE": SimTypeInt(signed=False, label="SECPKG_SESSIONINFO_TYPE"),
    "SECPKG_SHORT_VECTOR": SimStruct(OrderedDict((("ShortArrayOffset", SimTypeInt(signed=False, label="UInt32")), ("ShortArrayCount", SimTypeShort(signed=False, label="UInt16")),)), name="SECPKG_SHORT_VECTOR", pack=False, align=None),
    "SECPKG_SUPPLEMENTAL_CRED": SimStruct(OrderedDict((("PackageName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("CredentialSize", SimTypeInt(signed=False, label="UInt32")), ("Credentials", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SECPKG_SUPPLEMENTAL_CRED", pack=False, align=None),
    "SECPKG_SUPPLEMENTAL_CRED_ARRAY": SimStruct(OrderedDict((("CredentialCount", SimTypeInt(signed=False, label="UInt32")), ("Credentials", SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED", SimStruct), offset=0)),)), name="SECPKG_SUPPLEMENTAL_CRED_ARRAY", pack=False, align=None),
    "SECPKG_SUPPLIED_CREDENTIAL": SimStruct(OrderedDict((("cbHeaderLength", SimTypeShort(signed=False, label="UInt16")), ("cbStructureLength", SimTypeShort(signed=False, label="UInt16")), ("UserName", SimTypeRef("SECPKG_SHORT_VECTOR", SimStruct)), ("DomainName", SimTypeRef("SECPKG_SHORT_VECTOR", SimStruct)), ("PackedCredentials", SimTypeRef("SECPKG_BYTE_VECTOR", SimStruct)), ("CredFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SECPKG_SUPPLIED_CREDENTIAL", pack=False, align=None),
    "SECPKG_SURROGATE_LOGON": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("SurrogateLogonID", SimTypeRef("LUID", SimStruct)), ("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("Entries", SimTypePointer(SimTypeRef("SECPKG_SURROGATE_LOGON_ENTRY", SimStruct), offset=0)),)), name="SECPKG_SURROGATE_LOGON", pack=False, align=None),
    "SECPKG_SURROGATE_LOGON_ENTRY": SimStruct(OrderedDict((("Type", SimTypeBottom(label="Guid")), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECPKG_SURROGATE_LOGON_ENTRY", pack=False, align=None),
    "SECPKG_TARGETINFO": SimStruct(OrderedDict((("DomainSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ComputerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SECPKG_TARGETINFO", pack=False, align=None),
    "SECPKG_USER_FUNCTION_TABLE": SimStruct(OrderedDict((("InstanceInit", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_DLL_FUNCTIONS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Version", "FunctionTable", "UserFunctions"]), offset=0)), ("InitUserModeContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "PackedContext"]), offset=0)), ("MakeSignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "QualityOfProtection", "MessageBuffers", "MessageSequenceNumber"]), offset=0)), ("VerifySignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "MessageBuffers", "MessageSequenceNumber", "QualityOfProtection"]), offset=0)), ("SealMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "QualityOfProtection", "MessageBuffers", "MessageSequenceNumber"]), offset=0)), ("UnsealMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "MessageBuffers", "MessageSequenceNumber", "QualityOfProtection"]), offset=0)), ("GetContextToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ImpersonationToken"]), offset=0)), ("QueryContextAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ContextAttribute", "Buffer"]), offset=0)), ("CompleteAuthToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "InputBuffer"]), offset=0)), ("DeleteUserModeContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle"]), offset=0)), ("FormatCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Credentials", "FormattedCredentials"]), offset=0)), ("MarshallSupplementalCreds", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialSize", "Credentials", "MarshalledCredSize", "MarshalledCreds"]), offset=0)), ("ExportContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phContext", "fFlags", "pPackedContext", "pToken"]), offset=0)), ("ImportContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPackedContext", "Token", "phContext"]), offset=0)), ("MarshalAttributeData", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AttributeInfo", "Attribute", "AttributeDataSize", "AttributeData", "MarshaledAttributeDataSize", "MarshaledAttributeData"]), offset=0)),)), name="SECPKG_USER_FUNCTION_TABLE", pack=False, align=None),
    "SECPKG_WOW_CLIENT_DLL": SimStruct(OrderedDict((("WowClientDllPath", SimTypeRef("SECURITY_STRING", SimStruct)),)), name="SECPKG_WOW_CLIENT_DLL", pack=False, align=None),
    "SECTION": SimStruct(OrderedDict((("TableId", SimTypeChar(label="Byte")), ("Header", SimUnion({"S": SimTypeRef("MPEG_HEADER_BITS_MIDL", SimStruct), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("SectionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SECTION", pack=False, align=None),
    "SECTIONTITLELINKSTATES": SimTypeInt(signed=False, label="SECTIONTITLELINKSTATES"),
    "SECTION_FLAGS": SimTypeInt(signed=False, label="SECTION_FLAGS"),
    "SECTION_INHERIT": SimTypeInt(signed=False, label="SECTION_INHERIT"),
    "SECTION_OBJECT_POINTERS": SimStruct(OrderedDict((("DataSectionObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SharedCacheMap", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ImageSectionObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECTION_OBJECT_POINTERS", pack=False, align=None),
    "SECURELOCKCODE": SimTypeInt(signed=False, label="SECURELOCKCODE"),
    "SECUREURLHOSTVALIDATE_FLAGS": SimTypeInt(signed=False, label="SECUREURLHOSTVALIDATE_FLAGS"),
    "SECURE_BUFFER_INFO": SimStruct(OrderedDict((("guidBufferIdentifier", SimTypeBottom(label="Guid")), ("cbBufferSize", SimTypeInt(signed=False, label="UInt32")), ("cbCaptured", SimTypeInt(signed=False, label="UInt32")), ("ullReserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)),)), name="SECURE_BUFFER_INFO", pack=False, align=None),
    "SECURE_DRIVER_INTERFACE": SimStruct(OrderedDict((("InterfaceHeader", SimTypeRef("INTERFACE", SimStruct)), ("ProcessReference", SimTypePointer(SimTypeFunction([], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), offset=0)), ("ProcessDereference", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="SECURE_DRIVER_INTERFACE", pack=False, align=None),
    "SECURE_DRIVER_PROCESS_DEREFERENCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["InterfaceContext", "Process"]), offset=0),
    "SECURE_DRIVER_PROCESS_REFERENCE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["InterfaceContext"]), offset=0),
    "SECURITY_ATTRIBUTES": SimStruct(OrderedDict((("nLength", SimTypeInt(signed=False, label="UInt32")), ("lpSecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("bInheritHandle", SimTypeInt(signed=True, label="Int32")),)), name="SECURITY_ATTRIBUTES", pack=False, align=None),
    "SECURITY_AUTO_INHERIT_FLAGS": SimTypeInt(signed=False, label="SECURITY_AUTO_INHERIT_FLAGS"),
    "SECURITY_CAPABILITIES": SimStruct(OrderedDict((("AppContainerSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Capabilities", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)), ("CapabilityCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="SECURITY_CAPABILITIES", pack=False, align=None),
    "SECURITY_CLIENT_CONTEXT": SimStruct(OrderedDict((("SecurityQos", SimTypeRef("SECURITY_QUALITY_OF_SERVICE", SimStruct)), ("ClientToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DirectlyAccessClientToken", SimTypeChar(label="Byte")), ("DirectAccessEffectiveOnly", SimTypeChar(label="Byte")), ("ServerIsRemote", SimTypeChar(label="Byte")), ("ClientTokenControl", SimTypeRef("TOKEN_CONTROL", SimStruct)),)), name="SECURITY_CLIENT_CONTEXT", pack=False, align=None),
    "SECURITY_CONTEXT_TRACKING_MODE": SimTypeChar(label="Byte"),
    "SECURITY_DESCRIPTOR": SimStruct(OrderedDict((("Revision", SimTypeChar(label="Byte")), ("Sbz1", SimTypeChar(label="Byte")), ("Control", SimTypeInt(signed=False, label="SECURITY_DESCRIPTOR_CONTROL")), ("Owner", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Group", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Sacl", SimTypePointer(SimTypeRef("ACL", SimStruct), offset=0)), ("Dacl", SimTypePointer(SimTypeRef("ACL", SimStruct), offset=0)),)), name="SECURITY_DESCRIPTOR", pack=False, align=None),
    "SECURITY_DESCRIPTOR_CONTROL": SimTypeInt(signed=False, label="SECURITY_DESCRIPTOR_CONTROL"),
    "SECURITY_DESCRIPTOR_RELATIVE": SimStruct(OrderedDict((("Revision", SimTypeChar(label="Byte")), ("Sbz1", SimTypeChar(label="Byte")), ("Control", SimTypeInt(signed=False, label="SECURITY_DESCRIPTOR_CONTROL")), ("Owner", SimTypeInt(signed=False, label="UInt32")), ("Group", SimTypeInt(signed=False, label="UInt32")), ("Sacl", SimTypeInt(signed=False, label="UInt32")), ("Dacl", SimTypeInt(signed=False, label="UInt32")),)), name="SECURITY_DESCRIPTOR_RELATIVE", pack=False, align=None),
    "SECURITY_IMPERSONATION_LEVEL": SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL"),
    "SECURITY_INFO_PAGE_FLAGS": SimTypeInt(signed=False, label="SECURITY_INFO_PAGE_FLAGS"),
    "SECURITY_LOGON_SESSION_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("LogonId", SimTypeRef("LUID", SimStruct)), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonDomain", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AuthenticationPackage", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogonType", SimTypeInt(signed=False, label="UInt32")), ("Session", SimTypeInt(signed=False, label="UInt32")), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("LogonServer", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("DnsDomainName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Upn", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserFlags", SimTypeInt(signed=False, label="UInt32")), ("LastLogonInfo", SimTypeRef("LSA_LAST_INTER_LOGON_INFO", SimStruct)), ("LogonScript", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProfilePath", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectory", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectoryDrive", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LogoffTime", SimTypeLongLong(signed=True, label="Int64")), ("KickOffTime", SimTypeLongLong(signed=True, label="Int64")), ("PasswordLastSet", SimTypeLongLong(signed=True, label="Int64")), ("PasswordCanChange", SimTypeLongLong(signed=True, label="Int64")), ("PasswordMustChange", SimTypeLongLong(signed=True, label="Int64")),)), name="SECURITY_LOGON_SESSION_DATA", pack=False, align=None),
    "SECURITY_LOGON_TYPE": SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"),
    "SECURITY_MESSAGE": SimStruct(OrderedDict((("dwMsgId", SimTypeInt(signed=False, label="SECURITY_MESSAGE_MSG_ID")), ("hPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("UserName", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("Domain", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="SECURITY_MESSAGE", pack=False, align=None),
    "SECURITY_MESSAGE_MSG_ID": SimTypeInt(signed=False, label="SECURITY_MESSAGE_MSG_ID"),
    "SECURITY_OBJECT": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pData2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData2", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeInt(signed=False, label="UInt32")), ("fWellKnown", SimTypeChar(label="Byte")),)), name="SECURITY_OBJECT", pack=False, align=None),
    "SECURITY_OBJECT_AI_PARAMS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ConstraintMask", SimTypeInt(signed=False, label="UInt32")),)), name="SECURITY_OBJECT_AI_PARAMS", pack=False, align=None),
    "SECURITY_OPERATION_CODE": SimTypeInt(signed=False, label="SECURITY_OPERATION_CODE"),
    "SECURITY_PACKAGE_OPTIONS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="SECURITY_PACKAGE_OPTIONS_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SignatureSize", SimTypeInt(signed=False, label="UInt32")), ("Signature", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECURITY_PACKAGE_OPTIONS", pack=False, align=None),
    "SECURITY_PACKAGE_OPTIONS_TYPE": SimTypeInt(signed=False, label="SECURITY_PACKAGE_OPTIONS_TYPE"),
    "SECURITY_PRODUCT_TYPE": SimTypeInt(signed=False, label="SECURITY_PRODUCT_TYPE"),
    "SECURITY_QUALITY_OF_SERVICE": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("ImpersonationLevel", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")), ("ContextTrackingMode", SimTypeChar(label="Byte")), ("EffectiveOnly", SimTypeChar(label="Byte")),)), name="SECURITY_QUALITY_OF_SERVICE", pack=False, align=None),
    "SECURITY_STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SECURITY_STRING", pack=False, align=None),
    "SECURITY_SUBJECT_CONTEXT": SimStruct(OrderedDict((("ClientToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ImpersonationLevel", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")), ("PrimaryToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ProcessAuditId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECURITY_SUBJECT_CONTEXT", pack=False, align=None),
    "SECURITY_USER_DATA": SimStruct(OrderedDict((("UserName", SimTypeRef("SECURITY_STRING", SimStruct)), ("LogonDomainName", SimTypeRef("SECURITY_STRING", SimStruct)), ("LogonServer", SimTypeRef("SECURITY_STRING", SimStruct)), ("pSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SECURITY_USER_DATA", pack=False, align=None),
    "SEC_APPLICATION_PROTOCOLS": SimStruct(OrderedDict((("ProtocolListsSize", SimTypeInt(signed=False, label="UInt32")), ("ProtocolLists", SimTypePointer(SimTypeRef("SEC_APPLICATION_PROTOCOL_LIST", SimStruct), offset=0)),)), name="SEC_APPLICATION_PROTOCOLS", pack=False, align=None),
    "SEC_APPLICATION_PROTOCOL_LIST": SimStruct(OrderedDict((("ProtoNegoExt", SimTypeInt(signed=False, label="SEC_APPLICATION_PROTOCOL_NEGOTIATION_EXT")), ("ProtocolListSize", SimTypeShort(signed=False, label="UInt16")), ("ProtocolList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_APPLICATION_PROTOCOL_LIST", pack=False, align=None),
    "SEC_APPLICATION_PROTOCOL_NEGOTIATION_EXT": SimTypeInt(signed=False, label="SEC_APPLICATION_PROTOCOL_NEGOTIATION_EXT"),
    "SEC_APPLICATION_PROTOCOL_NEGOTIATION_STATUS": SimTypeInt(signed=False, label="SEC_APPLICATION_PROTOCOL_NEGOTIATION_STATUS"),
    "SEC_CERTIFICATE_REQUEST_CONTEXT": SimStruct(OrderedDict((("cbCertificateRequestContext", SimTypeChar(label="Byte")), ("rgCertificateRequestContext", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_CERTIFICATE_REQUEST_CONTEXT", pack=False, align=None),
    "SEC_CHANNEL_BINDINGS": SimStruct(OrderedDict((("dwInitiatorAddrType", SimTypeInt(signed=False, label="UInt32")), ("cbInitiatorLength", SimTypeInt(signed=False, label="UInt32")), ("dwInitiatorOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAcceptorAddrType", SimTypeInt(signed=False, label="UInt32")), ("cbAcceptorLength", SimTypeInt(signed=False, label="UInt32")), ("dwAcceptorOffset", SimTypeInt(signed=False, label="UInt32")), ("cbApplicationDataLength", SimTypeInt(signed=False, label="UInt32")), ("dwApplicationDataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_CHANNEL_BINDINGS", pack=False, align=None),
    "SEC_CHANNEL_BINDINGS_EX": SimStruct(OrderedDict((("magicNumber", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("cbHeaderLength", SimTypeInt(signed=False, label="UInt32")), ("cbStructureLength", SimTypeInt(signed=False, label="UInt32")), ("dwInitiatorAddrType", SimTypeInt(signed=False, label="UInt32")), ("cbInitiatorLength", SimTypeInt(signed=False, label="UInt32")), ("dwInitiatorOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAcceptorAddrType", SimTypeInt(signed=False, label="UInt32")), ("cbAcceptorLength", SimTypeInt(signed=False, label="UInt32")), ("dwAcceptorOffset", SimTypeInt(signed=False, label="UInt32")), ("cbApplicationDataLength", SimTypeInt(signed=False, label="UInt32")), ("dwApplicationDataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_CHANNEL_BINDINGS_EX", pack=False, align=None),
    "SEC_CHANNEL_BINDINGS_RESULT": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_CHANNEL_BINDINGS_RESULT", pack=False, align=None),
    "SEC_DTLS_MTU": SimStruct(OrderedDict((("PathMTU", SimTypeShort(signed=False, label="UInt16")),)), name="SEC_DTLS_MTU", pack=False, align=None),
    "SEC_FLAGS": SimStruct(OrderedDict((("Flags", SimTypeLongLong(signed=False, label="UInt64")),)), name="SEC_FLAGS", pack=False, align=None),
    "SEC_GET_KEY_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0),
    "SEC_NEGOTIATION_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("NameLength", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SEC_NEGOTIATION_INFO", pack=False, align=None),
    "SEC_OBJECT": SimStruct(OrderedDict((("cObjects", SimTypeInt(signed=False, label="UInt32")), ("prgObjects", SimTypePointer(SimTypeRef("SEC_OBJECT_ELEMENT", SimStruct), offset=0)),)), name="SEC_OBJECT", pack=False, align=None),
    "SEC_OBJECT_ELEMENT": SimStruct(OrderedDict((("guidObjectType", SimTypeBottom(label="Guid")), ("ObjectID", SimTypeRef("DBID", SimStruct)),)), name="SEC_OBJECT_ELEMENT", pack=False, align=None),
    "SEC_PRESHAREDKEY": SimStruct(OrderedDict((("KeySize", SimTypeShort(signed=False, label="UInt16")), ("Key", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_PRESHAREDKEY", pack=False, align=None),
    "SEC_PRESHAREDKEY_IDENTITY": SimStruct(OrderedDict((("KeyIdentitySize", SimTypeShort(signed=False, label="UInt16")), ("KeyIdentity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_PRESHAREDKEY_IDENTITY", pack=False, align=None),
    "SEC_SRTP_MASTER_KEY_IDENTIFIER": SimStruct(OrderedDict((("MasterKeyIdentifierSize", SimTypeChar(label="Byte")), ("MasterKeyIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_SRTP_MASTER_KEY_IDENTIFIER", pack=False, align=None),
    "SEC_SRTP_PROTECTION_PROFILES": SimStruct(OrderedDict((("ProfilesSize", SimTypeShort(signed=False, label="UInt16")), ("ProfilesList", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SEC_SRTP_PROTECTION_PROFILES", pack=False, align=None),
    "SEC_THREAD_START": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpThreadParameter"]), offset=0),
    "SEC_TOKEN_BINDING": SimStruct(OrderedDict((("MajorVersion", SimTypeChar(label="Byte")), ("MinorVersion", SimTypeChar(label="Byte")), ("KeyParametersSize", SimTypeShort(signed=False, label="UInt16")), ("KeyParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_TOKEN_BINDING", pack=False, align=None),
    "SEC_TRAFFIC_SECRETS": SimStruct(OrderedDict((("SymmetricAlgId", SimTypeArray(SimTypeChar(label="Char"), 64)), ("ChainingMode", SimTypeArray(SimTypeChar(label="Char"), 64)), ("HashAlgId", SimTypeArray(SimTypeChar(label="Char"), 64)), ("KeySize", SimTypeShort(signed=False, label="UInt16")), ("IvSize", SimTypeShort(signed=False, label="UInt16")), ("MsgSequenceStart", SimTypeShort(signed=False, label="UInt16")), ("MsgSequenceEnd", SimTypeShort(signed=False, label="UInt16")), ("TrafficSecretType", SimTypeInt(signed=False, label="SEC_TRAFFIC_SECRET_TYPE")), ("TrafficSecretSize", SimTypeShort(signed=False, label="UInt16")), ("TrafficSecret", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEC_TRAFFIC_SECRETS", pack=False, align=None),
    "SEC_TRAFFIC_SECRET_TYPE": SimTypeInt(signed=False, label="SEC_TRAFFIC_SECRET_TYPE"),
    "SEC_WINNT_AUTH_IDENTITY": SimTypeInt(signed=False, label="SEC_WINNT_AUTH_IDENTITY"),
    "SEC_WINNT_AUTH_IDENTITY32": SimStruct(OrderedDict((("User", SimTypeInt(signed=False, label="UInt32")), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypeInt(signed=False, label="UInt32")), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypeInt(signed=False, label="UInt32")), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_WINNT_AUTH_IDENTITY32", pack=False, align=None),
    "SEC_WINNT_AUTH_IDENTITY_A": SimStruct(OrderedDict((("User", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SEC_WINNT_AUTH_IDENTITY")),)), name="SEC_WINNT_AUTH_IDENTITY_A", pack=False, align=None),
    "SEC_WINNT_AUTH_IDENTITY_EX2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("cbHeaderLength", SimTypeShort(signed=False, label="UInt16")), ("cbStructureLength", SimTypeInt(signed=False, label="UInt32")), ("UserOffset", SimTypeInt(signed=False, label="UInt32")), ("UserLength", SimTypeShort(signed=False, label="UInt16")), ("DomainOffset", SimTypeInt(signed=False, label="UInt32")), ("DomainLength", SimTypeShort(signed=False, label="UInt16")), ("PackedCredentialsOffset", SimTypeInt(signed=False, label="UInt32")), ("PackedCredentialsLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PackageListOffset", SimTypeInt(signed=False, label="UInt32")), ("PackageListLength", SimTypeShort(signed=False, label="UInt16")),)), name="SEC_WINNT_AUTH_IDENTITY_EX2", pack=False, align=None),
    "SEC_WINNT_AUTH_IDENTITY_EX32": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("User", SimTypeInt(signed=False, label="UInt32")), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypeInt(signed=False, label="UInt32")), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypeInt(signed=False, label="UInt32")), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PackageList", SimTypeInt(signed=False, label="UInt32")), ("PackageListLength", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_WINNT_AUTH_IDENTITY_EX32", pack=False, align=None),
    "SEC_WINNT_AUTH_IDENTITY_EXA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("User", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PackageList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PackageListLength", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_WINNT_AUTH_IDENTITY_EXA", pack=False, align=None),
    "SEC_WINNT_AUTH_IDENTITY_EXW": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("User", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PackageList", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("PackageListLength", SimTypeInt(signed=False, label="UInt32")),)), name="SEC_WINNT_AUTH_IDENTITY_EXW", pack=False, align=None),
    "SEC_WINNT_AUTH_IDENTITY_INFO": SimUnion({"AuthIdExw": SimTypeRef("SEC_WINNT_AUTH_IDENTITY_EXW", SimStruct), "AuthIdExa": SimTypeRef("SEC_WINNT_AUTH_IDENTITY_EXA", SimStruct), "AuthId_a": SimTypeRef("SEC_WINNT_AUTH_IDENTITY_A", SimStruct), "AuthId_w": SimTypeRef("SEC_WINNT_AUTH_IDENTITY_W", SimStruct), "AuthIdEx2": SimTypeRef("SEC_WINNT_AUTH_IDENTITY_EX2", SimStruct)}, name="<anon>", label="None"),
    "SEC_WINNT_AUTH_IDENTITY_W": SimStruct(OrderedDict((("User", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("UserLength", SimTypeInt(signed=False, label="UInt32")), ("Domain", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("DomainLength", SimTypeInt(signed=False, label="UInt32")), ("Password", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("PasswordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SEC_WINNT_AUTH_IDENTITY")),)), name="SEC_WINNT_AUTH_IDENTITY_W", pack=False, align=None),
    "SEEK_ORIGIN": SimTypeInt(signed=False, label="SEEK_ORIGIN"),
    "SEGMENT_NOTE": SimStruct(OrderedDict((("Selector1", SimTypeShort(signed=False, label="UInt16")), ("Selector2", SimTypeShort(signed=False, label="UInt16")), ("Segment", SimTypeShort(signed=False, label="UInt16")), ("Module", SimTypeArray(SimTypeChar(label="SByte"), 10)), ("FileName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Type", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="SEGMENT_NOTE", pack=False, align=None),
    "SELCHANGE": SimStruct(OrderedDict((("nmhdr", SimTypeRef("NMHDR", SimStruct)), ("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("seltyp", SimTypeInt(signed=False, label="RICH_EDIT_GET_CONTEXT_MENU_SEL_TYPE")),)), name="SELCHANGE", pack=False, align=None),
    "SELECTION_TYPE": SimTypeInt(signed=False, label="SELECTION_TYPE"),
    "SENDASYNCPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "SENDCMDINPARAMS": SimStruct(OrderedDict((("cBufferSize", SimTypeInt(signed=False, label="UInt32")), ("irDriveRegs", SimTypeRef("IDEREGS", SimStruct)), ("bDriveNumber", SimTypeChar(label="Byte")), ("bReserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("bBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SENDCMDINPARAMS", pack=False, align=None),
    "SENDCMDOUTPARAMS": SimStruct(OrderedDict((("cBufferSize", SimTypeInt(signed=False, label="UInt32")), ("DriverStatus", SimTypeRef("DRIVERSTATUS", SimStruct)), ("bBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SENDCMDOUTPARAMS", pack=False, align=None),
    "SENDER_TSPEC": SimStruct(OrderedDict((("stspec_header", SimTypeRef("RsvpObjHdr", SimStruct)), ("stspec_body", SimTypeRef("IntServTspecBody", SimStruct)),)), name="SENDER_TSPEC", pack=False, align=None),
    "SEND_GENERIC_TLS_EXTENSION": SimStruct(OrderedDict((("ExtensionType", SimTypeShort(signed=False, label="UInt16")), ("HandshakeType", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SEND_GENERIC_TLS_EXTENSION", pack=False, align=None),
    "SEND_MESSAGE_TIMEOUT_FLAGS": SimTypeInt(signed=False, label="SEND_MESSAGE_TIMEOUT_FLAGS"),
    "SEND_RECV_FLAGS": SimTypeInt(signed=False, label="SEND_RECV_FLAGS"),
    "SENSORPROFILEID": SimStruct(OrderedDict((("Type", SimTypeBottom(label="Guid")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeInt(signed=False, label="UInt32")),)), name="SENSORPROFILEID", pack=False, align=None),
    "SENSOR_COLLECTION_LIST": SimStruct(OrderedDict((("AllocatedSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("List", SimTypePointer(SimTypeRef("SENSOR_VALUE_PAIR", SimStruct), offset=0)),)), name="SENSOR_COLLECTION_LIST", pack=False, align=None),
    "SENSOR_CONNECTION_TYPES": SimTypeInt(signed=False, label="SENSOR_CONNECTION_TYPES"),
    "SENSOR_PROPERTY_LIST": SimStruct(OrderedDict((("AllocatedSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("List", SimTypePointer(SimTypeRef("PROPERTYKEY", SimStruct), offset=0)),)), name="SENSOR_PROPERTY_LIST", pack=False, align=None),
    "SENSOR_STATE": SimTypeInt(signed=False, label="SENSOR_STATE"),
    "SENSOR_VALUE_PAIR": SimStruct(OrderedDict((("Key", SimTypeRef("PROPERTYKEY", SimStruct)), ("Value", SimTypeRef("PROPVARIANT", SimStruct)),)), name="SENSOR_VALUE_PAIR", pack=False, align=None),
    "SENS_CONNECTION_TYPE": SimTypeInt(signed=False, label="SENS_CONNECTION_TYPE"),
    "SENS_QOCINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOutSpeed", SimTypeInt(signed=False, label="UInt32")), ("dwInSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="SENS_QOCINFO", pack=False, align=None),
    "SEQUENCE_INFO": SimStruct(OrderedDict((("SeqID", SimTypeLongLong(signed=True, label="Int64")), ("SeqNo", SimTypeInt(signed=False, label="UInt32")), ("PrevNo", SimTypeInt(signed=False, label="UInt32")),)), name="SEQUENCE_INFO", pack=False, align=None),
    "SERENUM_PORTION": SimTypeInt(signed=False, label="SERENUM_PORTION"),
    "SERENUM_PORT_DESC": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("PortHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PortAddress", SimTypeLongLong(signed=True, label="Int64")), ("Reserved", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SERENUM_PORT_DESC", pack=False, align=None),
    "SERENUM_PORT_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ReadAccessor", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeChar(label="Byte"), arg_names=["SerPortAddress"]), offset=0)), ("WriteAccessor", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["SerPortAddress", "Value"]), offset=0)), ("SerPortAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HardwareHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Portion", SimTypeInt(signed=False, label="SERENUM_PORTION")), ("NumberAxis", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)),)), name="SERENUM_PORT_PARAMETERS", pack=False, align=None),
    "SERIALCONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("SubType", SimTypeInt(signed=False, label="UInt32")), ("ProvOffset", SimTypeInt(signed=False, label="UInt32")), ("ProviderSize", SimTypeInt(signed=False, label="UInt32")), ("ProviderData", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERIALCONFIG", pack=False, align=None),
    "SERIALIZEDPROPERTYVALUE": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("rgb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERIALIZEDPROPERTYVALUE", pack=False, align=None),
    "SERIALIZEDPROPSTORAGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SERIALKEYSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="SERIALKEYS_FLAGS")), ("lpszActivePort", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszPort", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("iBaudRate", SimTypeInt(signed=False, label="UInt32")), ("iPortState", SimTypeInt(signed=False, label="UInt32")), ("iActive", SimTypeInt(signed=False, label="UInt32")),)), name="SERIALKEYSA", pack=False, align=None),
    "SERIALKEYSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="SERIALKEYS_FLAGS")), ("lpszActivePort", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszPort", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("iBaudRate", SimTypeInt(signed=False, label="UInt32")), ("iPortState", SimTypeInt(signed=False, label="UInt32")), ("iActive", SimTypeInt(signed=False, label="UInt32")),)), name="SERIALKEYSW", pack=False, align=None),
    "SERIALKEYS_FLAGS": SimTypeInt(signed=False, label="SERIALKEYS_FLAGS"),
    "SERIALPERF_STATS": SimStruct(OrderedDict((("ReceivedCount", SimTypeInt(signed=False, label="UInt32")), ("TransmittedCount", SimTypeInt(signed=False, label="UInt32")), ("FrameErrorCount", SimTypeInt(signed=False, label="UInt32")), ("SerialOverrunErrorCount", SimTypeInt(signed=False, label="UInt32")), ("BufferOverrunErrorCount", SimTypeInt(signed=False, label="UInt32")), ("ParityErrorCount", SimTypeInt(signed=False, label="UInt32")),)), name="SERIALPERF_STATS", pack=False, align=None),
    "SERIAL_BASIC_SETTINGS": SimStruct(OrderedDict((("Timeouts", SimTypeRef("SERIAL_TIMEOUTS", SimStruct)), ("HandFlow", SimTypeRef("SERIAL_HANDFLOW", SimStruct)), ("RxFifo", SimTypeInt(signed=False, label="UInt32")), ("TxFifo", SimTypeInt(signed=False, label="UInt32")),)), name="SERIAL_BASIC_SETTINGS", pack=False, align=None),
    "SERIAL_BAUD_RATE": SimStruct(OrderedDict((("BaudRate", SimTypeInt(signed=False, label="UInt32")),)), name="SERIAL_BAUD_RATE", pack=False, align=None),
    "SERIAL_CHARS": SimStruct(OrderedDict((("EofChar", SimTypeChar(label="Byte")), ("ErrorChar", SimTypeChar(label="Byte")), ("BreakChar", SimTypeChar(label="Byte")), ("EventChar", SimTypeChar(label="Byte")), ("XonChar", SimTypeChar(label="Byte")), ("XoffChar", SimTypeChar(label="Byte")),)), name="SERIAL_CHARS", pack=False, align=None),
    "SERIAL_COMMPROP": SimStruct(OrderedDict((("PacketLength", SimTypeShort(signed=False, label="UInt16")), ("PacketVersion", SimTypeShort(signed=False, label="UInt16")), ("ServiceMask", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("MaxTxQueue", SimTypeInt(signed=False, label="UInt32")), ("MaxRxQueue", SimTypeInt(signed=False, label="UInt32")), ("MaxBaud", SimTypeInt(signed=False, label="UInt32")), ("ProvSubType", SimTypeInt(signed=False, label="UInt32")), ("ProvCapabilities", SimTypeInt(signed=False, label="UInt32")), ("SettableParams", SimTypeInt(signed=False, label="UInt32")), ("SettableBaud", SimTypeInt(signed=False, label="UInt32")), ("SettableData", SimTypeShort(signed=False, label="UInt16")), ("SettableStopParity", SimTypeShort(signed=False, label="UInt16")), ("CurrentTxQueue", SimTypeInt(signed=False, label="UInt32")), ("CurrentRxQueue", SimTypeInt(signed=False, label="UInt32")), ("ProvSpec1", SimTypeInt(signed=False, label="UInt32")), ("ProvSpec2", SimTypeInt(signed=False, label="UInt32")), ("ProvChar", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERIAL_COMMPROP", pack=False, align=None),
    "SERIAL_HANDFLOW": SimStruct(OrderedDict((("ControlHandShake", SimTypeInt(signed=False, label="UInt32")), ("FlowReplace", SimTypeInt(signed=False, label="UInt32")), ("XonLimit", SimTypeInt(signed=True, label="Int32")), ("XoffLimit", SimTypeInt(signed=True, label="Int32")),)), name="SERIAL_HANDFLOW", pack=False, align=None),
    "SERIAL_LINE_CONTROL": SimStruct(OrderedDict((("StopBits", SimTypeChar(label="Byte")), ("Parity", SimTypeChar(label="Byte")), ("WordLength", SimTypeChar(label="Byte")),)), name="SERIAL_LINE_CONTROL", pack=False, align=None),
    "SERIAL_QUEUE_SIZE": SimStruct(OrderedDict((("InSize", SimTypeInt(signed=False, label="UInt32")), ("OutSize", SimTypeInt(signed=False, label="UInt32")),)), name="SERIAL_QUEUE_SIZE", pack=False, align=None),
    "SERIAL_STATUS": SimStruct(OrderedDict((("Errors", SimTypeInt(signed=False, label="UInt32")), ("HoldReasons", SimTypeInt(signed=False, label="UInt32")), ("AmountInInQueue", SimTypeInt(signed=False, label="UInt32")), ("AmountInOutQueue", SimTypeInt(signed=False, label="UInt32")), ("EofReceived", SimTypeChar(label="Byte")), ("WaitForImmediate", SimTypeChar(label="Byte")),)), name="SERIAL_STATUS", pack=False, align=None),
    "SERIAL_TIMEOUTS": SimStruct(OrderedDict((("ReadIntervalTimeout", SimTypeInt(signed=False, label="UInt32")), ("ReadTotalTimeoutMultiplier", SimTypeInt(signed=False, label="UInt32")), ("ReadTotalTimeoutConstant", SimTypeInt(signed=False, label="UInt32")), ("WriteTotalTimeoutMultiplier", SimTypeInt(signed=False, label="UInt32")), ("WriteTotalTimeoutConstant", SimTypeInt(signed=False, label="UInt32")),)), name="SERIAL_TIMEOUTS", pack=False, align=None),
    "SERIAL_XOFF_COUNTER": SimStruct(OrderedDict((("Timeout", SimTypeInt(signed=False, label="UInt32")), ("Counter", SimTypeInt(signed=True, label="Int32")), ("XoffChar", SimTypeChar(label="Byte")),)), name="SERIAL_XOFF_COUNTER", pack=False, align=None),
    "SERVENT": SimStruct(OrderedDict((("s_name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("s_aliases", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("s_port", SimTypeShort(signed=True, label="Int16")), ("s_proto", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVENT", pack=False, align=None),
    "SERVERCALL": SimTypeInt(signed=False, label="SERVERCALL"),
    "SERVERSILO_BASIC_INFORMATION": SimStruct(OrderedDict((("ServiceSessionId", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="SERVERSILO_STATE")), ("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("IsDownlevelContainer", SimTypeChar(label="Byte")), ("ApiSetSchema", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HostApiSetSchema", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SERVERSILO_BASIC_INFORMATION", pack=False, align=None),
    "SERVERSILO_STATE": SimTypeInt(signed=False, label="SERVERSILO_STATE"),
    "SERVERXMLHTTP_OPTION": SimTypeInt(signed=False, label="SERVERXMLHTTP_OPTION"),
    "SERVER_ALIAS_INFO_0": SimStruct(OrderedDict((("srvai0_alias", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvai0_target", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvai0_default", SimTypeChar(label="Byte")), ("srvai0_reserved", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_ALIAS_INFO_0", pack=False, align=None),
    "SERVER_CERTIFICATE_INFO_0": SimStruct(OrderedDict((("srvci0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_subject", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_issuer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_thumbprint", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_friendlyname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_notbefore", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_notafter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_storelocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_storename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_renewalchain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("srvci0_type", SimTypeInt(signed=False, label="UInt32")), ("srvci0_flags", SimTypeInt(signed=False, label="UInt32")), ("srvci0_mapping_status", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_CERTIFICATE_INFO_0", pack=False, align=None),
    "SERVER_CERTIFICATE_TYPE": SimTypeInt(signed=False, label="SERVER_CERTIFICATE_TYPE"),
    "SERVER_INFO_100": SimStruct(OrderedDict((("sv100_platform_id", SimTypeInt(signed=False, label="UInt32")), ("sv100_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_INFO_100", pack=False, align=None),
    "SERVER_INFO_1005": SimStruct(OrderedDict((("sv1005_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_INFO_1005", pack=False, align=None),
    "SERVER_INFO_101": SimStruct(OrderedDict((("sv101_platform_id", SimTypeInt(signed=False, label="UInt32")), ("sv101_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv101_version_major", SimTypeInt(signed=False, label="UInt32")), ("sv101_version_minor", SimTypeInt(signed=False, label="UInt32")), ("sv101_type", SimTypeInt(signed=False, label="NET_SERVER_TYPE")), ("sv101_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_INFO_101", pack=False, align=None),
    "SERVER_INFO_1010": SimStruct(OrderedDict((("sv1010_disc", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1010", pack=False, align=None),
    "SERVER_INFO_1016": SimStruct(OrderedDict((("sv1016_hidden", SimTypeInt(signed=False, label="SERVER_INFO_HIDDEN")),)), name="SERVER_INFO_1016", pack=False, align=None),
    "SERVER_INFO_1017": SimStruct(OrderedDict((("sv1017_announce", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1017", pack=False, align=None),
    "SERVER_INFO_1018": SimStruct(OrderedDict((("sv1018_anndelta", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1018", pack=False, align=None),
    "SERVER_INFO_102": SimStruct(OrderedDict((("sv102_platform_id", SimTypeInt(signed=False, label="UInt32")), ("sv102_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv102_version_major", SimTypeInt(signed=False, label="UInt32")), ("sv102_version_minor", SimTypeInt(signed=False, label="UInt32")), ("sv102_type", SimTypeInt(signed=False, label="NET_SERVER_TYPE")), ("sv102_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv102_users", SimTypeInt(signed=False, label="UInt32")), ("sv102_disc", SimTypeInt(signed=True, label="Int32")), ("sv102_hidden", SimTypeInt(signed=False, label="SERVER_INFO_HIDDEN")), ("sv102_announce", SimTypeInt(signed=False, label="UInt32")), ("sv102_anndelta", SimTypeInt(signed=False, label="UInt32")), ("sv102_licenses", SimTypeInt(signed=False, label="UInt32")), ("sv102_userpath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_INFO_102", pack=False, align=None),
    "SERVER_INFO_103": SimStruct(OrderedDict((("sv103_platform_id", SimTypeInt(signed=False, label="UInt32")), ("sv103_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv103_version_major", SimTypeInt(signed=False, label="UInt32")), ("sv103_version_minor", SimTypeInt(signed=False, label="UInt32")), ("sv103_type", SimTypeInt(signed=False, label="UInt32")), ("sv103_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv103_users", SimTypeInt(signed=False, label="UInt32")), ("sv103_disc", SimTypeInt(signed=True, label="Int32")), ("sv103_hidden", SimTypeInt(signed=True, label="Int32")), ("sv103_announce", SimTypeInt(signed=False, label="UInt32")), ("sv103_anndelta", SimTypeInt(signed=False, label="UInt32")), ("sv103_licenses", SimTypeInt(signed=False, label="UInt32")), ("sv103_userpath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv103_capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_103", pack=False, align=None),
    "SERVER_INFO_1107": SimStruct(OrderedDict((("sv1107_users", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1107", pack=False, align=None),
    "SERVER_INFO_1501": SimStruct(OrderedDict((("sv1501_sessopens", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1501", pack=False, align=None),
    "SERVER_INFO_1502": SimStruct(OrderedDict((("sv1502_sessvcs", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1502", pack=False, align=None),
    "SERVER_INFO_1503": SimStruct(OrderedDict((("sv1503_opensearch", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1503", pack=False, align=None),
    "SERVER_INFO_1506": SimStruct(OrderedDict((("sv1506_maxworkitems", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1506", pack=False, align=None),
    "SERVER_INFO_1509": SimStruct(OrderedDict((("sv1509_maxrawbuflen", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1509", pack=False, align=None),
    "SERVER_INFO_1510": SimStruct(OrderedDict((("sv1510_sessusers", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1510", pack=False, align=None),
    "SERVER_INFO_1511": SimStruct(OrderedDict((("sv1511_sessconns", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1511", pack=False, align=None),
    "SERVER_INFO_1512": SimStruct(OrderedDict((("sv1512_maxnonpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1512", pack=False, align=None),
    "SERVER_INFO_1513": SimStruct(OrderedDict((("sv1513_maxpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1513", pack=False, align=None),
    "SERVER_INFO_1514": SimStruct(OrderedDict((("sv1514_enablesoftcompat", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1514", pack=False, align=None),
    "SERVER_INFO_1515": SimStruct(OrderedDict((("sv1515_enableforcedlogoff", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1515", pack=False, align=None),
    "SERVER_INFO_1516": SimStruct(OrderedDict((("sv1516_timesource", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1516", pack=False, align=None),
    "SERVER_INFO_1518": SimStruct(OrderedDict((("sv1518_lmannounce", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1518", pack=False, align=None),
    "SERVER_INFO_1520": SimStruct(OrderedDict((("sv1520_maxcopyreadlen", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1520", pack=False, align=None),
    "SERVER_INFO_1521": SimStruct(OrderedDict((("sv1521_maxcopywritelen", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1521", pack=False, align=None),
    "SERVER_INFO_1522": SimStruct(OrderedDict((("sv1522_minkeepsearch", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1522", pack=False, align=None),
    "SERVER_INFO_1523": SimStruct(OrderedDict((("sv1523_maxkeepsearch", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1523", pack=False, align=None),
    "SERVER_INFO_1524": SimStruct(OrderedDict((("sv1524_minkeepcomplsearch", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1524", pack=False, align=None),
    "SERVER_INFO_1525": SimStruct(OrderedDict((("sv1525_maxkeepcomplsearch", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1525", pack=False, align=None),
    "SERVER_INFO_1528": SimStruct(OrderedDict((("sv1528_scavtimeout", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1528", pack=False, align=None),
    "SERVER_INFO_1529": SimStruct(OrderedDict((("sv1529_minrcvqueue", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1529", pack=False, align=None),
    "SERVER_INFO_1530": SimStruct(OrderedDict((("sv1530_minfreeworkitems", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1530", pack=False, align=None),
    "SERVER_INFO_1533": SimStruct(OrderedDict((("sv1533_maxmpxct", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1533", pack=False, align=None),
    "SERVER_INFO_1534": SimStruct(OrderedDict((("sv1534_oplockbreakwait", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1534", pack=False, align=None),
    "SERVER_INFO_1535": SimStruct(OrderedDict((("sv1535_oplockbreakresponsewait", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1535", pack=False, align=None),
    "SERVER_INFO_1536": SimStruct(OrderedDict((("sv1536_enableoplocks", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1536", pack=False, align=None),
    "SERVER_INFO_1537": SimStruct(OrderedDict((("sv1537_enableoplockforceclose", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1537", pack=False, align=None),
    "SERVER_INFO_1538": SimStruct(OrderedDict((("sv1538_enablefcbopens", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1538", pack=False, align=None),
    "SERVER_INFO_1539": SimStruct(OrderedDict((("sv1539_enableraw", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1539", pack=False, align=None),
    "SERVER_INFO_1540": SimStruct(OrderedDict((("sv1540_enablesharednetdrives", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1540", pack=False, align=None),
    "SERVER_INFO_1541": SimStruct(OrderedDict((("sv1541_minfreeconnections", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1541", pack=False, align=None),
    "SERVER_INFO_1542": SimStruct(OrderedDict((("sv1542_maxfreeconnections", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1542", pack=False, align=None),
    "SERVER_INFO_1543": SimStruct(OrderedDict((("sv1543_initsesstable", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1543", pack=False, align=None),
    "SERVER_INFO_1544": SimStruct(OrderedDict((("sv1544_initconntable", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1544", pack=False, align=None),
    "SERVER_INFO_1545": SimStruct(OrderedDict((("sv1545_initfiletable", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1545", pack=False, align=None),
    "SERVER_INFO_1546": SimStruct(OrderedDict((("sv1546_initsearchtable", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1546", pack=False, align=None),
    "SERVER_INFO_1547": SimStruct(OrderedDict((("sv1547_alertschedule", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1547", pack=False, align=None),
    "SERVER_INFO_1548": SimStruct(OrderedDict((("sv1548_errorthreshold", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1548", pack=False, align=None),
    "SERVER_INFO_1549": SimStruct(OrderedDict((("sv1549_networkerrorthreshold", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1549", pack=False, align=None),
    "SERVER_INFO_1550": SimStruct(OrderedDict((("sv1550_diskspacethreshold", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1550", pack=False, align=None),
    "SERVER_INFO_1552": SimStruct(OrderedDict((("sv1552_maxlinkdelay", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1552", pack=False, align=None),
    "SERVER_INFO_1553": SimStruct(OrderedDict((("sv1553_minlinkthroughput", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1553", pack=False, align=None),
    "SERVER_INFO_1554": SimStruct(OrderedDict((("sv1554_linkinfovalidtime", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1554", pack=False, align=None),
    "SERVER_INFO_1555": SimStruct(OrderedDict((("sv1555_scavqosinfoupdatetime", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1555", pack=False, align=None),
    "SERVER_INFO_1556": SimStruct(OrderedDict((("sv1556_maxworkitemidletime", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1556", pack=False, align=None),
    "SERVER_INFO_1557": SimStruct(OrderedDict((("sv1557_maxrawworkitems", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1557", pack=False, align=None),
    "SERVER_INFO_1560": SimStruct(OrderedDict((("sv1560_producttype", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1560", pack=False, align=None),
    "SERVER_INFO_1561": SimStruct(OrderedDict((("sv1561_serversize", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1561", pack=False, align=None),
    "SERVER_INFO_1562": SimStruct(OrderedDict((("sv1562_connectionlessautodisc", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1562", pack=False, align=None),
    "SERVER_INFO_1563": SimStruct(OrderedDict((("sv1563_sharingviolationretries", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1563", pack=False, align=None),
    "SERVER_INFO_1564": SimStruct(OrderedDict((("sv1564_sharingviolationdelay", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1564", pack=False, align=None),
    "SERVER_INFO_1565": SimStruct(OrderedDict((("sv1565_maxglobalopensearch", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1565", pack=False, align=None),
    "SERVER_INFO_1566": SimStruct(OrderedDict((("sv1566_removeduplicatesearches", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1566", pack=False, align=None),
    "SERVER_INFO_1567": SimStruct(OrderedDict((("sv1567_lockviolationretries", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1567", pack=False, align=None),
    "SERVER_INFO_1568": SimStruct(OrderedDict((("sv1568_lockviolationoffset", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1568", pack=False, align=None),
    "SERVER_INFO_1569": SimStruct(OrderedDict((("sv1569_lockviolationdelay", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1569", pack=False, align=None),
    "SERVER_INFO_1570": SimStruct(OrderedDict((("sv1570_mdlreadswitchover", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1570", pack=False, align=None),
    "SERVER_INFO_1571": SimStruct(OrderedDict((("sv1571_cachedopenlimit", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1571", pack=False, align=None),
    "SERVER_INFO_1572": SimStruct(OrderedDict((("sv1572_criticalthreads", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1572", pack=False, align=None),
    "SERVER_INFO_1573": SimStruct(OrderedDict((("sv1573_restrictnullsessaccess", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1573", pack=False, align=None),
    "SERVER_INFO_1574": SimStruct(OrderedDict((("sv1574_enablewfw311directipx", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1574", pack=False, align=None),
    "SERVER_INFO_1575": SimStruct(OrderedDict((("sv1575_otherqueueaffinity", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1575", pack=False, align=None),
    "SERVER_INFO_1576": SimStruct(OrderedDict((("sv1576_queuesamplesecs", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1576", pack=False, align=None),
    "SERVER_INFO_1577": SimStruct(OrderedDict((("sv1577_balancecount", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1577", pack=False, align=None),
    "SERVER_INFO_1578": SimStruct(OrderedDict((("sv1578_preferredaffinity", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1578", pack=False, align=None),
    "SERVER_INFO_1579": SimStruct(OrderedDict((("sv1579_maxfreerfcbs", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1579", pack=False, align=None),
    "SERVER_INFO_1580": SimStruct(OrderedDict((("sv1580_maxfreemfcbs", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1580", pack=False, align=None),
    "SERVER_INFO_1581": SimStruct(OrderedDict((("sv1581_maxfreemlcbs", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1581", pack=False, align=None),
    "SERVER_INFO_1582": SimStruct(OrderedDict((("sv1582_maxfreepagedpoolchunks", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1582", pack=False, align=None),
    "SERVER_INFO_1583": SimStruct(OrderedDict((("sv1583_minpagedpoolchunksize", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1583", pack=False, align=None),
    "SERVER_INFO_1584": SimStruct(OrderedDict((("sv1584_maxpagedpoolchunksize", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1584", pack=False, align=None),
    "SERVER_INFO_1585": SimStruct(OrderedDict((("sv1585_sendsfrompreferredprocessor", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1585", pack=False, align=None),
    "SERVER_INFO_1586": SimStruct(OrderedDict((("sv1586_maxthreadsperqueue", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1586", pack=False, align=None),
    "SERVER_INFO_1587": SimStruct(OrderedDict((("sv1587_cacheddirectorylimit", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1587", pack=False, align=None),
    "SERVER_INFO_1588": SimStruct(OrderedDict((("sv1588_maxcopylength", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1588", pack=False, align=None),
    "SERVER_INFO_1590": SimStruct(OrderedDict((("sv1590_enablecompression", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1590", pack=False, align=None),
    "SERVER_INFO_1591": SimStruct(OrderedDict((("sv1591_autosharewks", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1591", pack=False, align=None),
    "SERVER_INFO_1592": SimStruct(OrderedDict((("sv1592_autosharewks", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1592", pack=False, align=None),
    "SERVER_INFO_1593": SimStruct(OrderedDict((("sv1593_enablesecuritysignature", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1593", pack=False, align=None),
    "SERVER_INFO_1594": SimStruct(OrderedDict((("sv1594_requiresecuritysignature", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1594", pack=False, align=None),
    "SERVER_INFO_1595": SimStruct(OrderedDict((("sv1595_minclientbuffersize", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1595", pack=False, align=None),
    "SERVER_INFO_1596": SimStruct(OrderedDict((("sv1596_ConnectionNoSessionsTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1596", pack=False, align=None),
    "SERVER_INFO_1597": SimStruct(OrderedDict((("sv1597_IdleThreadTimeOut", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1597", pack=False, align=None),
    "SERVER_INFO_1598": SimStruct(OrderedDict((("sv1598_enableW9xsecuritysignature", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1598", pack=False, align=None),
    "SERVER_INFO_1599": SimStruct(OrderedDict((("sv1598_enforcekerberosreauthentication", SimTypeChar(label="Byte")),)), name="SERVER_INFO_1599", pack=False, align=None),
    "SERVER_INFO_1600": SimStruct(OrderedDict((("sv1598_disabledos", SimTypeChar(label="Byte")),)), name="SERVER_INFO_1600", pack=False, align=None),
    "SERVER_INFO_1601": SimStruct(OrderedDict((("sv1598_lowdiskspaceminimum", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_1601", pack=False, align=None),
    "SERVER_INFO_1602": SimStruct(OrderedDict((("sv_1598_disablestrictnamechecking", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_1602", pack=False, align=None),
    "SERVER_INFO_402": SimStruct(OrderedDict((("sv402_ulist_mtime", SimTypeInt(signed=False, label="UInt32")), ("sv402_glist_mtime", SimTypeInt(signed=False, label="UInt32")), ("sv402_alist_mtime", SimTypeInt(signed=False, label="UInt32")), ("sv402_alerts", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv402_security", SimTypeInt(signed=False, label="SERVER_INFO_SECURITY")), ("sv402_numadmin", SimTypeInt(signed=False, label="UInt32")), ("sv402_lanmask", SimTypeInt(signed=False, label="UInt32")), ("sv402_guestacct", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv402_chdevs", SimTypeInt(signed=False, label="UInt32")), ("sv402_chdevq", SimTypeInt(signed=False, label="UInt32")), ("sv402_chdevjobs", SimTypeInt(signed=False, label="UInt32")), ("sv402_connections", SimTypeInt(signed=False, label="UInt32")), ("sv402_shares", SimTypeInt(signed=False, label="UInt32")), ("sv402_openfiles", SimTypeInt(signed=False, label="UInt32")), ("sv402_sessopens", SimTypeInt(signed=False, label="UInt32")), ("sv402_sessvcs", SimTypeInt(signed=False, label="UInt32")), ("sv402_sessreqs", SimTypeInt(signed=False, label="UInt32")), ("sv402_opensearch", SimTypeInt(signed=False, label="UInt32")), ("sv402_activelocks", SimTypeInt(signed=False, label="UInt32")), ("sv402_numreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv402_sizreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv402_numbigbuf", SimTypeInt(signed=False, label="UInt32")), ("sv402_numfiletasks", SimTypeInt(signed=False, label="UInt32")), ("sv402_alertsched", SimTypeInt(signed=False, label="UInt32")), ("sv402_erroralert", SimTypeInt(signed=False, label="UInt32")), ("sv402_logonalert", SimTypeInt(signed=False, label="UInt32")), ("sv402_accessalert", SimTypeInt(signed=False, label="UInt32")), ("sv402_diskalert", SimTypeInt(signed=False, label="UInt32")), ("sv402_netioalert", SimTypeInt(signed=False, label="UInt32")), ("sv402_maxauditsz", SimTypeInt(signed=False, label="UInt32")), ("sv402_srvheuristics", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_INFO_402", pack=False, align=None),
    "SERVER_INFO_403": SimStruct(OrderedDict((("sv403_ulist_mtime", SimTypeInt(signed=False, label="UInt32")), ("sv403_glist_mtime", SimTypeInt(signed=False, label="UInt32")), ("sv403_alist_mtime", SimTypeInt(signed=False, label="UInt32")), ("sv403_alerts", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv403_security", SimTypeInt(signed=False, label="SERVER_INFO_SECURITY")), ("sv403_numadmin", SimTypeInt(signed=False, label="UInt32")), ("sv403_lanmask", SimTypeInt(signed=False, label="UInt32")), ("sv403_guestacct", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv403_chdevs", SimTypeInt(signed=False, label="UInt32")), ("sv403_chdevq", SimTypeInt(signed=False, label="UInt32")), ("sv403_chdevjobs", SimTypeInt(signed=False, label="UInt32")), ("sv403_connections", SimTypeInt(signed=False, label="UInt32")), ("sv403_shares", SimTypeInt(signed=False, label="UInt32")), ("sv403_openfiles", SimTypeInt(signed=False, label="UInt32")), ("sv403_sessopens", SimTypeInt(signed=False, label="UInt32")), ("sv403_sessvcs", SimTypeInt(signed=False, label="UInt32")), ("sv403_sessreqs", SimTypeInt(signed=False, label="UInt32")), ("sv403_opensearch", SimTypeInt(signed=False, label="UInt32")), ("sv403_activelocks", SimTypeInt(signed=False, label="UInt32")), ("sv403_numreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv403_sizreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv403_numbigbuf", SimTypeInt(signed=False, label="UInt32")), ("sv403_numfiletasks", SimTypeInt(signed=False, label="UInt32")), ("sv403_alertsched", SimTypeInt(signed=False, label="UInt32")), ("sv403_erroralert", SimTypeInt(signed=False, label="UInt32")), ("sv403_logonalert", SimTypeInt(signed=False, label="UInt32")), ("sv403_accessalert", SimTypeInt(signed=False, label="UInt32")), ("sv403_diskalert", SimTypeInt(signed=False, label="UInt32")), ("sv403_netioalert", SimTypeInt(signed=False, label="UInt32")), ("sv403_maxauditsz", SimTypeInt(signed=False, label="UInt32")), ("sv403_srvheuristics", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv403_auditedevents", SimTypeInt(signed=False, label="UInt32")), ("sv403_autoprofile", SimTypeInt(signed=False, label="UInt32")), ("sv403_autopath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_INFO_403", pack=False, align=None),
    "SERVER_INFO_502": SimStruct(OrderedDict((("sv502_sessopens", SimTypeInt(signed=False, label="UInt32")), ("sv502_sessvcs", SimTypeInt(signed=False, label="UInt32")), ("sv502_opensearch", SimTypeInt(signed=False, label="UInt32")), ("sv502_sizreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv502_initworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv502_maxworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv502_rawworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv502_irpstacksize", SimTypeInt(signed=False, label="UInt32")), ("sv502_maxrawbuflen", SimTypeInt(signed=False, label="UInt32")), ("sv502_sessusers", SimTypeInt(signed=False, label="UInt32")), ("sv502_sessconns", SimTypeInt(signed=False, label="UInt32")), ("sv502_maxpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")), ("sv502_maxnonpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")), ("sv502_enablesoftcompat", SimTypeInt(signed=True, label="Int32")), ("sv502_enableforcedlogoff", SimTypeInt(signed=True, label="Int32")), ("sv502_timesource", SimTypeInt(signed=True, label="Int32")), ("sv502_acceptdownlevelapis", SimTypeInt(signed=True, label="Int32")), ("sv502_lmannounce", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_502", pack=False, align=None),
    "SERVER_INFO_503": SimStruct(OrderedDict((("sv503_sessopens", SimTypeInt(signed=False, label="UInt32")), ("sv503_sessvcs", SimTypeInt(signed=False, label="UInt32")), ("sv503_opensearch", SimTypeInt(signed=False, label="UInt32")), ("sv503_sizreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv503_initworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv503_rawworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv503_irpstacksize", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxrawbuflen", SimTypeInt(signed=False, label="UInt32")), ("sv503_sessusers", SimTypeInt(signed=False, label="UInt32")), ("sv503_sessconns", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxnonpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")), ("sv503_enablesoftcompat", SimTypeInt(signed=True, label="Int32")), ("sv503_enableforcedlogoff", SimTypeInt(signed=True, label="Int32")), ("sv503_timesource", SimTypeInt(signed=True, label="Int32")), ("sv503_acceptdownlevelapis", SimTypeInt(signed=True, label="Int32")), ("sv503_lmannounce", SimTypeInt(signed=True, label="Int32")), ("sv503_domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv503_maxcopyreadlen", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxcopywritelen", SimTypeInt(signed=False, label="UInt32")), ("sv503_minkeepsearch", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxkeepsearch", SimTypeInt(signed=False, label="UInt32")), ("sv503_minkeepcomplsearch", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxkeepcomplsearch", SimTypeInt(signed=False, label="UInt32")), ("sv503_threadcountadd", SimTypeInt(signed=False, label="UInt32")), ("sv503_numblockthreads", SimTypeInt(signed=False, label="UInt32")), ("sv503_scavtimeout", SimTypeInt(signed=False, label="UInt32")), ("sv503_minrcvqueue", SimTypeInt(signed=False, label="UInt32")), ("sv503_minfreeworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv503_xactmemsize", SimTypeInt(signed=False, label="UInt32")), ("sv503_threadpriority", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxmpxct", SimTypeInt(signed=False, label="UInt32")), ("sv503_oplockbreakwait", SimTypeInt(signed=False, label="UInt32")), ("sv503_oplockbreakresponsewait", SimTypeInt(signed=False, label="UInt32")), ("sv503_enableoplocks", SimTypeInt(signed=True, label="Int32")), ("sv503_enableoplockforceclose", SimTypeInt(signed=True, label="Int32")), ("sv503_enablefcbopens", SimTypeInt(signed=True, label="Int32")), ("sv503_enableraw", SimTypeInt(signed=True, label="Int32")), ("sv503_enablesharednetdrives", SimTypeInt(signed=True, label="Int32")), ("sv503_minfreeconnections", SimTypeInt(signed=False, label="UInt32")), ("sv503_maxfreeconnections", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_503", pack=False, align=None),
    "SERVER_INFO_598": SimStruct(OrderedDict((("sv598_maxrawworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxthreadsperqueue", SimTypeInt(signed=False, label="UInt32")), ("sv598_producttype", SimTypeInt(signed=False, label="UInt32")), ("sv598_serversize", SimTypeInt(signed=False, label="UInt32")), ("sv598_connectionlessautodisc", SimTypeInt(signed=False, label="UInt32")), ("sv598_sharingviolationretries", SimTypeInt(signed=False, label="UInt32")), ("sv598_sharingviolationdelay", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxglobalopensearch", SimTypeInt(signed=False, label="UInt32")), ("sv598_removeduplicatesearches", SimTypeInt(signed=False, label="UInt32")), ("sv598_lockviolationoffset", SimTypeInt(signed=False, label="UInt32")), ("sv598_lockviolationdelay", SimTypeInt(signed=False, label="UInt32")), ("sv598_mdlreadswitchover", SimTypeInt(signed=False, label="UInt32")), ("sv598_cachedopenlimit", SimTypeInt(signed=False, label="UInt32")), ("sv598_otherqueueaffinity", SimTypeInt(signed=False, label="UInt32")), ("sv598_restrictnullsessaccess", SimTypeInt(signed=True, label="Int32")), ("sv598_enablewfw311directipx", SimTypeInt(signed=True, label="Int32")), ("sv598_queuesamplesecs", SimTypeInt(signed=False, label="UInt32")), ("sv598_balancecount", SimTypeInt(signed=False, label="UInt32")), ("sv598_preferredaffinity", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxfreerfcbs", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxfreemfcbs", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxfreelfcbs", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxfreepagedpoolchunks", SimTypeInt(signed=False, label="UInt32")), ("sv598_minpagedpoolchunksize", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxpagedpoolchunksize", SimTypeInt(signed=False, label="UInt32")), ("sv598_sendsfrompreferredprocessor", SimTypeInt(signed=True, label="Int32")), ("sv598_cacheddirectorylimit", SimTypeInt(signed=False, label="UInt32")), ("sv598_maxcopylength", SimTypeInt(signed=False, label="UInt32")), ("sv598_enablecompression", SimTypeInt(signed=True, label="Int32")), ("sv598_autosharewks", SimTypeInt(signed=True, label="Int32")), ("sv598_autoshareserver", SimTypeInt(signed=True, label="Int32")), ("sv598_enablesecuritysignature", SimTypeInt(signed=True, label="Int32")), ("sv598_requiresecuritysignature", SimTypeInt(signed=True, label="Int32")), ("sv598_minclientbuffersize", SimTypeInt(signed=False, label="UInt32")), ("sv598_serverguid", SimTypeBottom(label="Guid")), ("sv598_ConnectionNoSessionsTimeout", SimTypeInt(signed=False, label="UInt32")), ("sv598_IdleThreadTimeOut", SimTypeInt(signed=False, label="UInt32")), ("sv598_enableW9xsecuritysignature", SimTypeInt(signed=True, label="Int32")), ("sv598_enforcekerberosreauthentication", SimTypeInt(signed=True, label="Int32")), ("sv598_disabledos", SimTypeInt(signed=True, label="Int32")), ("sv598_lowdiskspaceminimum", SimTypeInt(signed=False, label="UInt32")), ("sv598_disablestrictnamechecking", SimTypeInt(signed=True, label="Int32")), ("sv598_enableauthenticateusersharing", SimTypeInt(signed=True, label="Int32")),)), name="SERVER_INFO_598", pack=False, align=None),
    "SERVER_INFO_599": SimStruct(OrderedDict((("sv599_sessopens", SimTypeInt(signed=False, label="UInt32")), ("sv599_sessvcs", SimTypeInt(signed=False, label="UInt32")), ("sv599_opensearch", SimTypeInt(signed=False, label="UInt32")), ("sv599_sizreqbuf", SimTypeInt(signed=False, label="UInt32")), ("sv599_initworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv599_rawworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv599_irpstacksize", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxrawbuflen", SimTypeInt(signed=False, label="UInt32")), ("sv599_sessusers", SimTypeInt(signed=False, label="UInt32")), ("sv599_sessconns", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxnonpagedmemoryusage", SimTypeInt(signed=False, label="UInt32")), ("sv599_enablesoftcompat", SimTypeInt(signed=True, label="Int32")), ("sv599_enableforcedlogoff", SimTypeInt(signed=True, label="Int32")), ("sv599_timesource", SimTypeInt(signed=True, label="Int32")), ("sv599_acceptdownlevelapis", SimTypeInt(signed=True, label="Int32")), ("sv599_lmannounce", SimTypeInt(signed=True, label="Int32")), ("sv599_domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sv599_maxcopyreadlen", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxcopywritelen", SimTypeInt(signed=False, label="UInt32")), ("sv599_minkeepsearch", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxkeepsearch", SimTypeInt(signed=False, label="UInt32")), ("sv599_minkeepcomplsearch", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxkeepcomplsearch", SimTypeInt(signed=False, label="UInt32")), ("sv599_threadcountadd", SimTypeInt(signed=False, label="UInt32")), ("sv599_numblockthreads", SimTypeInt(signed=False, label="UInt32")), ("sv599_scavtimeout", SimTypeInt(signed=False, label="UInt32")), ("sv599_minrcvqueue", SimTypeInt(signed=False, label="UInt32")), ("sv599_minfreeworkitems", SimTypeInt(signed=False, label="UInt32")), ("sv599_xactmemsize", SimTypeInt(signed=False, label="UInt32")), ("sv599_threadpriority", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxmpxct", SimTypeInt(signed=False, label="UInt32")), ("sv599_oplockbreakwait", SimTypeInt(signed=False, label="UInt32")), ("sv599_oplockbreakresponsewait", SimTypeInt(signed=False, label="UInt32")), ("sv599_enableoplocks", SimTypeInt(signed=True, label="Int32")), ("sv599_enableoplockforceclose", SimTypeInt(signed=True, label="Int32")), ("sv599_enablefcbopens", SimTypeInt(signed=True, label="Int32")), ("sv599_enableraw", SimTypeInt(signed=True, label="Int32")), ("sv599_enablesharednetdrives", SimTypeInt(signed=True, label="Int32")), ("sv599_minfreeconnections", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxfreeconnections", SimTypeInt(signed=False, label="UInt32")), ("sv599_initsesstable", SimTypeInt(signed=False, label="UInt32")), ("sv599_initconntable", SimTypeInt(signed=False, label="UInt32")), ("sv599_initfiletable", SimTypeInt(signed=False, label="UInt32")), ("sv599_initsearchtable", SimTypeInt(signed=False, label="UInt32")), ("sv599_alertschedule", SimTypeInt(signed=False, label="UInt32")), ("sv599_errorthreshold", SimTypeInt(signed=False, label="UInt32")), ("sv599_networkerrorthreshold", SimTypeInt(signed=False, label="UInt32")), ("sv599_diskspacethreshold", SimTypeInt(signed=False, label="UInt32")), ("sv599_reserved", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxlinkdelay", SimTypeInt(signed=False, label="UInt32")), ("sv599_minlinkthroughput", SimTypeInt(signed=False, label="UInt32")), ("sv599_linkinfovalidtime", SimTypeInt(signed=False, label="UInt32")), ("sv599_scavqosinfoupdatetime", SimTypeInt(signed=False, label="UInt32")), ("sv599_maxworkitemidletime", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_INFO_599", pack=False, align=None),
    "SERVER_INFO_HIDDEN": SimTypeInt(signed=False, label="SERVER_INFO_HIDDEN"),
    "SERVER_INFO_SECURITY": SimTypeInt(signed=False, label="SERVER_INFO_SECURITY"),
    "SERVER_ROUTINE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "SERVER_TRANSPORT_INFO_0": SimStruct(OrderedDict((("svti0_numberofvcs", SimTypeInt(signed=False, label="UInt32")), ("svti0_transportname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti0_transportaddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("svti0_transportaddresslength", SimTypeInt(signed=False, label="UInt32")), ("svti0_networkaddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_TRANSPORT_INFO_0", pack=False, align=None),
    "SERVER_TRANSPORT_INFO_1": SimStruct(OrderedDict((("svti1_numberofvcs", SimTypeInt(signed=False, label="UInt32")), ("svti1_transportname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti1_transportaddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("svti1_transportaddresslength", SimTypeInt(signed=False, label="UInt32")), ("svti1_networkaddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti1_domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVER_TRANSPORT_INFO_1", pack=False, align=None),
    "SERVER_TRANSPORT_INFO_2": SimStruct(OrderedDict((("svti2_numberofvcs", SimTypeInt(signed=False, label="UInt32")), ("svti2_transportname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti2_transportaddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("svti2_transportaddresslength", SimTypeInt(signed=False, label="UInt32")), ("svti2_networkaddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti2_domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti2_flags", SimTypeInt(signed=False, label="UInt32")),)), name="SERVER_TRANSPORT_INFO_2", pack=False, align=None),
    "SERVER_TRANSPORT_INFO_3": SimStruct(OrderedDict((("svti3_numberofvcs", SimTypeInt(signed=False, label="UInt32")), ("svti3_transportname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti3_transportaddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("svti3_transportaddresslength", SimTypeInt(signed=False, label="UInt32")), ("svti3_networkaddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti3_domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svti3_flags", SimTypeInt(signed=False, label="UInt32")), ("svti3_passwordlength", SimTypeInt(signed=False, label="UInt32")), ("svti3_password", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="SERVER_TRANSPORT_INFO_3", pack=False, align=None),
    "SERVICE_ADDRESS": SimStruct(OrderedDict((("dwAddressType", SimTypeInt(signed=False, label="UInt32")), ("dwAddressFlags", SimTypeInt(signed=False, label="UInt32")), ("dwAddressLength", SimTypeInt(signed=False, label="UInt32")), ("dwPrincipalLength", SimTypeInt(signed=False, label="UInt32")), ("lpAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpPrincipal", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVICE_ADDRESS", pack=False, align=None),
    "SERVICE_ADDRESSES": SimStruct(OrderedDict((("dwAddressCount", SimTypeInt(signed=False, label="UInt32")), ("Addresses", SimTypePointer(SimTypeRef("SERVICE_ADDRESS", SimStruct), offset=0)),)), name="SERVICE_ADDRESSES", pack=False, align=None),
    "SERVICE_ASYNC_INFO": SimStruct(OrderedDict((("lpServiceCallbackProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["lParam", "hAsyncTaskHandle"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hAsyncTaskHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SERVICE_ASYNC_INFO", pack=False, align=None),
    "SERVICE_CONFIG": SimTypeInt(signed=False, label="SERVICE_CONFIG"),
    "SERVICE_CONTROL_STATUS_REASON_PARAMSA": SimStruct(OrderedDict((("dwReason", SimTypeInt(signed=False, label="UInt32")), ("pszComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ServiceStatus", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)),)), name="SERVICE_CONTROL_STATUS_REASON_PARAMSA", pack=False, align=None),
    "SERVICE_CONTROL_STATUS_REASON_PARAMSW": SimStruct(OrderedDict((("dwReason", SimTypeInt(signed=False, label="UInt32")), ("pszComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ServiceStatus", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)),)), name="SERVICE_CONTROL_STATUS_REASON_PARAMSW", pack=False, align=None),
    "SERVICE_CUSTOM_SYSTEM_STATE_CHANGE_DATA_ITEM": SimStruct(OrderedDict((("u", SimUnion({"CustomStateId": SimTypeRef("SERVICE_TRIGGER_CUSTOM_STATE_ID", SimStruct), "s": SimTypeRef("_s_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="SERVICE_CUSTOM_SYSTEM_STATE_CHANGE_DATA_ITEM", pack=False, align=None),
    "SERVICE_DELAYED_AUTO_START_INFO": SimStruct(OrderedDict((("fDelayedAutostart", SimTypeInt(signed=True, label="Int32")),)), name="SERVICE_DELAYED_AUTO_START_INFO", pack=False, align=None),
    "SERVICE_DESCRIPTIONA": SimStruct(OrderedDict((("lpDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVICE_DESCRIPTIONA", pack=False, align=None),
    "SERVICE_DESCRIPTIONW": SimStruct(OrderedDict((("lpDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVICE_DESCRIPTIONW", pack=False, align=None),
    "SERVICE_DIRECTORY_TYPE": SimTypeInt(signed=False, label="SERVICE_DIRECTORY_TYPE"),
    "SERVICE_ERROR": SimTypeInt(signed=False, label="SERVICE_ERROR"),
    "SERVICE_ERROR_TYPE": SimTypeInt(signed=False, label="SERVICE_ERROR_TYPE"),
    "SERVICE_FAILURE_ACTIONSA": SimStruct(OrderedDict((("dwResetPeriod", SimTypeInt(signed=False, label="UInt32")), ("lpRebootMsg", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpCommand", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cActions", SimTypeInt(signed=False, label="UInt32")), ("lpsaActions", SimTypePointer(SimTypeRef("SC_ACTION", SimStruct), offset=0)),)), name="SERVICE_FAILURE_ACTIONSA", pack=False, align=None),
    "SERVICE_FAILURE_ACTIONSW": SimStruct(OrderedDict((("dwResetPeriod", SimTypeInt(signed=False, label="UInt32")), ("lpRebootMsg", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpCommand", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cActions", SimTypeInt(signed=False, label="UInt32")), ("lpsaActions", SimTypePointer(SimTypeRef("SC_ACTION", SimStruct), offset=0)),)), name="SERVICE_FAILURE_ACTIONSW", pack=False, align=None),
    "SERVICE_FAILURE_ACTIONS_FLAG": SimStruct(OrderedDict((("fFailureActionsOnNonCrashFailures", SimTypeInt(signed=True, label="Int32")),)), name="SERVICE_FAILURE_ACTIONS_FLAG", pack=False, align=None),
    "SERVICE_INFOA": SimStruct(OrderedDict((("lpServiceType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpServiceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpLocale", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwDisplayHint", SimTypeInt(signed=False, label="RESOURCE_DISPLAY_TYPE")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("lpMachineName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpServiceAddress", SimTypePointer(SimTypeRef("SERVICE_ADDRESSES", SimStruct), offset=0)), ("ServiceSpecificInfo", SimTypeRef("BLOB", SimStruct)),)), name="SERVICE_INFOA", pack=False, align=None),
    "SERVICE_INFOW": SimStruct(OrderedDict((("lpServiceType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpServiceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpLocale", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwDisplayHint", SimTypeInt(signed=False, label="RESOURCE_DISPLAY_TYPE")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("lpMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpServiceAddress", SimTypePointer(SimTypeRef("SERVICE_ADDRESSES", SimStruct), offset=0)), ("ServiceSpecificInfo", SimTypeRef("BLOB", SimStruct)),)), name="SERVICE_INFOW", pack=False, align=None),
    "SERVICE_INFO_0": SimStruct(OrderedDict((("svci0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVICE_INFO_0", pack=False, align=None),
    "SERVICE_INFO_1": SimStruct(OrderedDict((("svci1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svci1_status", SimTypeInt(signed=False, label="UInt32")), ("svci1_code", SimTypeInt(signed=False, label="UInt32")), ("svci1_pid", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_INFO_1", pack=False, align=None),
    "SERVICE_INFO_2": SimStruct(OrderedDict((("svci2_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svci2_status", SimTypeInt(signed=False, label="UInt32")), ("svci2_code", SimTypeInt(signed=False, label="UInt32")), ("svci2_pid", SimTypeInt(signed=False, label="UInt32")), ("svci2_text", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("svci2_specific_error", SimTypeInt(signed=False, label="UInt32")), ("svci2_display_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVICE_INFO_2", pack=False, align=None),
    "SERVICE_LAUNCH_PROTECTED_INFO": SimStruct(OrderedDict((("dwLaunchProtected", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_LAUNCH_PROTECTED_INFO", pack=False, align=None),
    "SERVICE_LOAD_TYPE": SimTypeInt(signed=False, label="SERVICE_LOAD_TYPE"),
    "SERVICE_MAIN_FUNCTIONA": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNumServicesArgs", "lpServiceArgVectors"]), offset=0),
    "SERVICE_MAIN_FUNCTIONW": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNumServicesArgs", "lpServiceArgVectors"]), offset=0),
    "SERVICE_NODE_TYPE": SimTypeInt(signed=False, label="SERVICE_NODE_TYPE"),
    "SERVICE_NOTIFY": SimTypeInt(signed=False, label="SERVICE_NOTIFY"),
    "SERVICE_NOTIFY_1": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pfnNotifyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pParameter"]), offset=0)), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwNotificationStatus", SimTypeInt(signed=False, label="UInt32")), ("ServiceStatus", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)),)), name="SERVICE_NOTIFY_1", pack=False, align=None),
    "SERVICE_NOTIFY_2A": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pfnNotifyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pParameter"]), offset=0)), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwNotificationStatus", SimTypeInt(signed=False, label="UInt32")), ("ServiceStatus", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)), ("dwNotificationTriggered", SimTypeInt(signed=False, label="UInt32")), ("pszServiceNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVICE_NOTIFY_2A", pack=False, align=None),
    "SERVICE_NOTIFY_2W": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pfnNotifyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pParameter"]), offset=0)), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwNotificationStatus", SimTypeInt(signed=False, label="UInt32")), ("ServiceStatus", SimTypeRef("SERVICE_STATUS_PROCESS", SimStruct)), ("dwNotificationTriggered", SimTypeInt(signed=False, label="UInt32")), ("pszServiceNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVICE_NOTIFY_2W", pack=False, align=None),
    "SERVICE_PREFERRED_NODE_INFO": SimStruct(OrderedDict((("usPreferredNode", SimTypeShort(signed=False, label="UInt16")), ("fDelete", SimTypeChar(label="Byte")),)), name="SERVICE_PREFERRED_NODE_INFO", pack=False, align=None),
    "SERVICE_PRESHUTDOWN_INFO": SimStruct(OrderedDict((("dwPreshutdownTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_PRESHUTDOWN_INFO", pack=False, align=None),
    "SERVICE_REGISTRY_STATE_TYPE": SimTypeInt(signed=False, label="SERVICE_REGISTRY_STATE_TYPE"),
    "SERVICE_REQUIRED_PRIVILEGES_INFOA": SimStruct(OrderedDict((("pmszRequiredPrivileges", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVICE_REQUIRED_PRIVILEGES_INFOA", pack=False, align=None),
    "SERVICE_REQUIRED_PRIVILEGES_INFOW": SimStruct(OrderedDict((("pmszRequiredPrivileges", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SERVICE_REQUIRED_PRIVILEGES_INFOW", pack=False, align=None),
    "SERVICE_RUNS_IN_PROCESS": SimTypeInt(signed=False, label="SERVICE_RUNS_IN_PROCESS"),
    "SERVICE_SHARED_DIRECTORY_TYPE": SimTypeInt(signed=False, label="SERVICE_SHARED_DIRECTORY_TYPE"),
    "SERVICE_SHARED_REGISTRY_STATE_TYPE": SimTypeInt(signed=False, label="SERVICE_SHARED_REGISTRY_STATE_TYPE"),
    "SERVICE_SID_INFO": SimStruct(OrderedDict((("dwServiceSidType", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_SID_INFO", pack=False, align=None),
    "SERVICE_START_REASON": SimStruct(OrderedDict((("dwReason", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_START_REASON", pack=False, align=None),
    "SERVICE_START_TYPE": SimTypeInt(signed=False, label="SERVICE_START_TYPE"),
    "SERVICE_STATUS": SimStruct(OrderedDict((("dwServiceType", SimTypeInt(signed=False, label="ENUM_SERVICE_TYPE")), ("dwCurrentState", SimTypeInt(signed=False, label="SERVICE_STATUS_CURRENT_STATE")), ("dwControlsAccepted", SimTypeInt(signed=False, label="UInt32")), ("dwWin32ExitCode", SimTypeInt(signed=False, label="UInt32")), ("dwServiceSpecificExitCode", SimTypeInt(signed=False, label="UInt32")), ("dwCheckPoint", SimTypeInt(signed=False, label="UInt32")), ("dwWaitHint", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_STATUS", pack=False, align=None),
    "SERVICE_STATUS_CURRENT_STATE": SimTypeInt(signed=False, label="SERVICE_STATUS_CURRENT_STATE"),
    "SERVICE_STATUS_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SERVICE_STATUS_PROCESS": SimStruct(OrderedDict((("dwServiceType", SimTypeInt(signed=False, label="ENUM_SERVICE_TYPE")), ("dwCurrentState", SimTypeInt(signed=False, label="SERVICE_STATUS_CURRENT_STATE")), ("dwControlsAccepted", SimTypeInt(signed=False, label="UInt32")), ("dwWin32ExitCode", SimTypeInt(signed=False, label="UInt32")), ("dwServiceSpecificExitCode", SimTypeInt(signed=False, label="UInt32")), ("dwCheckPoint", SimTypeInt(signed=False, label="UInt32")), ("dwWaitHint", SimTypeInt(signed=False, label="UInt32")), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags", SimTypeInt(signed=False, label="SERVICE_RUNS_IN_PROCESS")),)), name="SERVICE_STATUS_PROCESS", pack=False, align=None),
    "SERVICE_TABLE_ENTRYA": SimStruct(OrderedDict((("lpServiceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpServiceProc", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNumServicesArgs", "lpServiceArgVectors"]), offset=0)),)), name="SERVICE_TABLE_ENTRYA", pack=False, align=None),
    "SERVICE_TABLE_ENTRYW": SimStruct(OrderedDict((("lpServiceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpServiceProc", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwNumServicesArgs", "lpServiceArgVectors"]), offset=0)),)), name="SERVICE_TABLE_ENTRYW", pack=False, align=None),
    "SERVICE_TIMECHANGE_INFO": SimStruct(OrderedDict((("liNewTime", SimTypeLongLong(signed=True, label="Int64")), ("liOldTime", SimTypeLongLong(signed=True, label="Int64")),)), name="SERVICE_TIMECHANGE_INFO", pack=False, align=None),
    "SERVICE_TRIGGER": SimStruct(OrderedDict((("dwTriggerType", SimTypeInt(signed=False, label="SERVICE_TRIGGER_TYPE")), ("dwAction", SimTypeInt(signed=False, label="SERVICE_TRIGGER_ACTION")), ("pTriggerSubtype", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cDataItems", SimTypeInt(signed=False, label="UInt32")), ("pDataItems", SimTypePointer(SimTypeRef("SERVICE_TRIGGER_SPECIFIC_DATA_ITEM", SimStruct), offset=0)),)), name="SERVICE_TRIGGER", pack=False, align=None),
    "SERVICE_TRIGGER_ACTION": SimTypeInt(signed=False, label="SERVICE_TRIGGER_ACTION"),
    "SERVICE_TRIGGER_CUSTOM_STATE_ID": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="SERVICE_TRIGGER_CUSTOM_STATE_ID", pack=False, align=None),
    "SERVICE_TRIGGER_INFO": SimStruct(OrderedDict((("cTriggers", SimTypeInt(signed=False, label="UInt32")), ("pTriggers", SimTypePointer(SimTypeRef("SERVICE_TRIGGER", SimStruct), offset=0)), ("pReserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVICE_TRIGGER_INFO", pack=False, align=None),
    "SERVICE_TRIGGER_SPECIFIC_DATA_ITEM": SimStruct(OrderedDict((("dwDataType", SimTypeInt(signed=False, label="SERVICE_TRIGGER_SPECIFIC_DATA_ITEM_DATA_TYPE")), ("cbData", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SERVICE_TRIGGER_SPECIFIC_DATA_ITEM", pack=False, align=None),
    "SERVICE_TRIGGER_SPECIFIC_DATA_ITEM_DATA_TYPE": SimTypeInt(signed=False, label="SERVICE_TRIGGER_SPECIFIC_DATA_ITEM_DATA_TYPE"),
    "SERVICE_TRIGGER_TYPE": SimTypeInt(signed=False, label="SERVICE_TRIGGER_TYPE"),
    "SERVICE_TYPE": SimTypeInt(signed=False, label="SERVICE_TYPE"),
    "SERVICE_TYPE_INFO": SimStruct(OrderedDict((("dwTypeNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwValueCount", SimTypeInt(signed=False, label="UInt32")), ("Values", SimTypePointer(SimTypeRef("SERVICE_TYPE_VALUE", SimStruct), offset=0)),)), name="SERVICE_TYPE_INFO", pack=False, align=None),
    "SERVICE_TYPE_INFO_ABSA": SimStruct(OrderedDict((("lpTypeName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwValueCount", SimTypeInt(signed=False, label="UInt32")), ("Values", SimTypePointer(SimTypeRef("SERVICE_TYPE_VALUE_ABSA", SimStruct), offset=0)),)), name="SERVICE_TYPE_INFO_ABSA", pack=False, align=None),
    "SERVICE_TYPE_INFO_ABSW": SimStruct(OrderedDict((("lpTypeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwValueCount", SimTypeInt(signed=False, label="UInt32")), ("Values", SimTypePointer(SimTypeRef("SERVICE_TYPE_VALUE_ABSW", SimStruct), offset=0)),)), name="SERVICE_TYPE_INFO_ABSW", pack=False, align=None),
    "SERVICE_TYPE_VALUE": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("dwValueSize", SimTypeInt(signed=False, label="UInt32")), ("dwValueNameOffset", SimTypeInt(signed=False, label="UInt32")), ("dwValueOffset", SimTypeInt(signed=False, label="UInt32")),)), name="SERVICE_TYPE_VALUE", pack=False, align=None),
    "SERVICE_TYPE_VALUE_ABSA": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("dwValueSize", SimTypeInt(signed=False, label="UInt32")), ("lpValueName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SERVICE_TYPE_VALUE_ABSA", pack=False, align=None),
    "SERVICE_TYPE_VALUE_ABSW": SimStruct(OrderedDict((("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("dwValueSize", SimTypeInt(signed=False, label="UInt32")), ("lpValueName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SERVICE_TYPE_VALUE_ABSW", pack=False, align=None),
    "SESSION_BUFFER": SimStruct(OrderedDict((("lsn", SimTypeChar(label="Byte")), ("state", SimTypeChar(label="Byte")), ("local_name", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("remote_name", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("rcvs_outstanding", SimTypeChar(label="Byte")), ("sends_outstanding", SimTypeChar(label="Byte")),)), name="SESSION_BUFFER", pack=False, align=None),
    "SESSION_HEADER": SimStruct(OrderedDict((("sess_name", SimTypeChar(label="Byte")), ("num_sess", SimTypeChar(label="Byte")), ("rcv_dg_outstanding", SimTypeChar(label="Byte")), ("rcv_any_outstanding", SimTypeChar(label="Byte")),)), name="SESSION_HEADER", pack=False, align=None),
    "SESSION_INFO_0": SimStruct(OrderedDict((("sesi0_cname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SESSION_INFO_0", pack=False, align=None),
    "SESSION_INFO_1": SimStruct(OrderedDict((("sesi1_cname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi1_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi1_num_opens", SimTypeInt(signed=False, label="UInt32")), ("sesi1_time", SimTypeInt(signed=False, label="UInt32")), ("sesi1_idle_time", SimTypeInt(signed=False, label="UInt32")), ("sesi1_user_flags", SimTypeInt(signed=False, label="SESSION_INFO_USER_FLAGS")),)), name="SESSION_INFO_1", pack=False, align=None),
    "SESSION_INFO_10": SimStruct(OrderedDict((("sesi10_cname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi10_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi10_time", SimTypeInt(signed=False, label="UInt32")), ("sesi10_idle_time", SimTypeInt(signed=False, label="UInt32")),)), name="SESSION_INFO_10", pack=False, align=None),
    "SESSION_INFO_2": SimStruct(OrderedDict((("sesi2_cname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi2_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi2_num_opens", SimTypeInt(signed=False, label="UInt32")), ("sesi2_time", SimTypeInt(signed=False, label="UInt32")), ("sesi2_idle_time", SimTypeInt(signed=False, label="UInt32")), ("sesi2_user_flags", SimTypeInt(signed=False, label="SESSION_INFO_USER_FLAGS")), ("sesi2_cltype_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SESSION_INFO_2", pack=False, align=None),
    "SESSION_INFO_502": SimStruct(OrderedDict((("sesi502_cname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi502_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi502_num_opens", SimTypeInt(signed=False, label="UInt32")), ("sesi502_time", SimTypeInt(signed=False, label="UInt32")), ("sesi502_idle_time", SimTypeInt(signed=False, label="UInt32")), ("sesi502_user_flags", SimTypeInt(signed=False, label="SESSION_INFO_USER_FLAGS")), ("sesi502_cltype_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("sesi502_transport", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SESSION_INFO_502", pack=False, align=None),
    "SESSION_INFO_USER_FLAGS": SimTypeInt(signed=False, label="SESSION_INFO_USER_FLAGS"),
    "SESSION_TIMEOUT_ACTION_TYPE": SimTypeInt(signed=False, label="SESSION_TIMEOUT_ACTION_TYPE"),
    "SETPROCESSWORKINGSETSIZEEX_FLAGS": SimTypeInt(signed=False, label="SETPROCESSWORKINGSETSIZEEX_FLAGS"),
    "SETRESULT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("hSetResult", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Result", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SETRESULT_INFO", pack=False, align=None),
    "SETTEXTEX": SimStruct(OrderedDict((("flags", SimTypeInt(signed=False, label="UInt32")), ("codepage", SimTypeInt(signed=False, label="UInt32")),)), name="SETTEXTEX", pack=False, align=None),
    "SETTINGSTATUS": SimTypeInt(signed=False, label="SETTINGSTATUS"),
    "SETUPSCANFILEQUEUE_FLAGS": SimTypeInt(signed=False, label="SETUPSCANFILEQUEUE_FLAGS"),
    "SETUP_DI_DEVICE_CONFIGURATION_FLAGS": SimTypeInt(signed=False, label="SETUP_DI_DEVICE_CONFIGURATION_FLAGS"),
    "SETUP_DI_DEVICE_CREATION_FLAGS": SimTypeInt(signed=False, label="SETUP_DI_DEVICE_CREATION_FLAGS"),
    "SETUP_DI_DEVICE_INSTALL_FLAGS": SimTypeInt(signed=False, label="SETUP_DI_DEVICE_INSTALL_FLAGS"),
    "SETUP_DI_DEVICE_INSTALL_FLAGS_EX": SimTypeInt(signed=False, label="SETUP_DI_DEVICE_INSTALL_FLAGS_EX"),
    "SETUP_DI_DRIVER_INSTALL_FLAGS": SimTypeInt(signed=False, label="SETUP_DI_DRIVER_INSTALL_FLAGS"),
    "SETUP_DI_DRIVER_TYPE": SimTypeInt(signed=False, label="SETUP_DI_DRIVER_TYPE"),
    "SETUP_DI_GET_CLASS_DEVS_FLAGS": SimTypeInt(signed=False, label="SETUP_DI_GET_CLASS_DEVS_FLAGS"),
    "SETUP_DI_PROPERTY_CHANGE_SCOPE": SimTypeInt(signed=False, label="SETUP_DI_PROPERTY_CHANGE_SCOPE"),
    "SETUP_DI_REGISTRY_PROPERTY": SimTypeInt(signed=False, label="SETUP_DI_REGISTRY_PROPERTY"),
    "SETUP_DI_REMOVE_DEVICE_SCOPE": SimTypeInt(signed=False, label="SETUP_DI_REMOVE_DEVICE_SCOPE"),
    "SETUP_DI_STATE_CHANGE": SimTypeInt(signed=False, label="SETUP_DI_STATE_CHANGE"),
    "SETUP_FILE_OPERATION": SimTypeInt(signed=False, label="SETUP_FILE_OPERATION"),
    "SET_ACTIVE_COLOR_PROFILE_NAME": SimStruct(OrderedDict((("ColorProfileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SET_ACTIVE_COLOR_PROFILE_NAME", pack=False, align=None),
    "SET_APP_INSTANCE_CSV_FLAGS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ProcessHandle", "Mask", "Flags"]), offset=0),
    "SET_BOUNDS_RECT_FLAGS": SimTypeInt(signed=False, label="SET_BOUNDS_RECT_FLAGS"),
    "SET_CACHED_RUNS_STATE_INPUT_BUFFER": SimStruct(OrderedDict((("Enable", SimTypeChar(label="Byte")),)), name="SET_CACHED_RUNS_STATE_INPUT_BUFFER", pack=False, align=None),
    "SET_COMPOSITION_STRING_TYPE": SimTypeInt(signed=False, label="SET_COMPOSITION_STRING_TYPE"),
    "SET_CONTEXT_ATTRIBUTES_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "SET_CONTEXT_ATTRIBUTES_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "SET_CREDENTIALS_ATTRIBUTES_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "SET_CREDENTIALS_ATTRIBUTES_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "SET_D3COLD_SUPPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["Context", "D3ColdSupport"]), offset=0),
    "SET_DAX_ALLOC_ALIGNMENT_HINT_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("AlignmentShift", SimTypeInt(signed=False, label="UInt32")), ("FileOffsetToAlign", SimTypeLongLong(signed=False, label="UInt64")), ("FallbackAlignmentShift", SimTypeInt(signed=False, label="UInt32")),)), name="SET_DAX_ALLOC_ALIGNMENT_HINT_INPUT", pack=False, align=None),
    "SET_DISK_ATTRIBUTES": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Persist", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Attributes", SimTypeLongLong(signed=False, label="UInt64")), ("AttributesMask", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="SET_DISK_ATTRIBUTES", pack=False, align=None),
    "SET_DISPLAY_CONFIG_FLAGS": SimTypeInt(signed=False, label="SET_DISPLAY_CONFIG_FLAGS"),
    "SET_FILE_POINTER_MOVE_METHOD": SimTypeInt(signed=False, label="SET_FILE_POINTER_MOVE_METHOD"),
    "SET_PARTITION_INFORMATION": SimStruct(OrderedDict((("PartitionType", SimTypeChar(label="Byte")),)), name="SET_PARTITION_INFORMATION", pack=False, align=None),
    "SET_PARTITION_INFORMATION_EX": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="PARTITION_STYLE")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("SET_PARTITION_INFORMATION", SimStruct), "Gpt": SimTypeRef("PARTITION_INFORMATION_GPT", SimStruct)}, name="<anon>", label="None")),)), name="SET_PARTITION_INFORMATION_EX", pack=False, align=None),
    "SET_POWER_SETTING_VALUE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Guid", SimTypeBottom(label="Guid")), ("PowerCondition", SimTypeInt(signed=False, label="SYSTEM_POWER_CONDITION")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SET_POWER_SETTING_VALUE", pack=False, align=None),
    "SET_PURGE_FAILURE_MODE_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SET_PURGE_FAILURE_MODE_INPUT", pack=False, align=None),
    "SET_SERVICE_OPERATION": SimTypeInt(signed=False, label="SET_SERVICE_OPERATION"),
    "SET_THEME_APP_PROPERTIES_FLAGS": SimTypeInt(signed=False, label="SET_THEME_APP_PROPERTIES_FLAGS"),
    "SET_VIRTUAL_DEVICE_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "VirtualFunction", "Buffer", "Offset", "Length"]), offset=0),
    "SET_VIRTUAL_DISK_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="SET_VIRTUAL_DISK_INFO_VERSION")), ("Anonymous", SimUnion({"ParentFilePath": SimTypePointer(SimTypeChar(label="Char"), offset=0), "UniqueIdentifier": SimTypeBottom(label="Guid"), "ParentPathWithDepthInfo": SimTypeRef("_ParentPathWithDepthInfo_e__Struct", SimStruct), "VhdPhysicalSectorSize": SimTypeInt(signed=False, label="UInt32"), "VirtualDiskId": SimTypeBottom(label="Guid"), "ChangeTrackingEnabled": SimTypeInt(signed=True, label="Int32"), "ParentLocator": SimTypeRef("_ParentLocator_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="SET_VIRTUAL_DISK_INFO", pack=False, align=None),
    "SET_VIRTUAL_DISK_INFO_VERSION": SimTypeInt(signed=False, label="SET_VIRTUAL_DISK_INFO_VERSION"),
    "SET_WINDOW_POS_FLAGS": SimTypeInt(signed=False, label="SET_WINDOW_POS_FLAGS"),
    "SE_ACCESS_REPLY": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ResultListCount", SimTypeInt(signed=False, label="UInt32")), ("GrantedAccess", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AccessStatus", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("AccessReason", SimTypePointer(SimTypeRef("ACCESS_REASONS", SimStruct), offset=0)), ("Privileges", SimTypePointer(SimTypePointer(SimTypeRef("PRIVILEGE_SET", SimStruct), offset=0), offset=0)),)), name="SE_ACCESS_REPLY", pack=False, align=None),
    "SE_ACCESS_REQUEST": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("SeSecurityDescriptor", SimTypePointer(SimTypeRef("SE_SECURITY_DESCRIPTOR", SimStruct), offset=0)), ("DesiredAccess", SimTypeInt(signed=False, label="UInt32")), ("PreviouslyGrantedAccess", SimTypeInt(signed=False, label="UInt32")), ("PrincipalSelfSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("GenericMapping", SimTypePointer(SimTypeRef("GENERIC_MAPPING", SimStruct), offset=0)), ("ObjectTypeListCount", SimTypeInt(signed=False, label="UInt32")), ("ObjectTypeList", SimTypePointer(SimTypeRef("OBJECT_TYPE_LIST", SimStruct), offset=0)),)), name="SE_ACCESS_REQUEST", pack=False, align=None),
    "SE_ADT_ACCESS_REASON": SimStruct(OrderedDict((("AccessMask", SimTypeInt(signed=False, label="UInt32")), ("AccessReasons", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("ObjectTypeIndex", SimTypeInt(signed=False, label="UInt32")), ("AccessGranted", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SE_ADT_ACCESS_REASON", pack=False, align=None),
    "SE_ADT_CLAIMS": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Claims", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SE_ADT_CLAIMS", pack=False, align=None),
    "SE_ADT_OBJECT_TYPE": SimStruct(OrderedDict((("ObjectType", SimTypeBottom(label="Guid")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Level", SimTypeShort(signed=False, label="UInt16")), ("AccessMask", SimTypeInt(signed=False, label="UInt32")),)), name="SE_ADT_OBJECT_TYPE", pack=False, align=None),
    "SE_ADT_PARAMETER_ARRAY": SimStruct(OrderedDict((("CategoryId", SimTypeInt(signed=False, label="UInt32")), ("AuditId", SimTypeInt(signed=False, label="UInt32")), ("ParameterCount", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("FlatSubCategoryId", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Parameters", SimTypeArray(SimTypeRef("SE_ADT_PARAMETER_ARRAY_ENTRY", SimStruct), 32)),)), name="SE_ADT_PARAMETER_ARRAY", pack=False, align=None),
    "SE_ADT_PARAMETER_ARRAY_ENTRY": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="SE_ADT_PARAMETER_TYPE")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)), ("Address", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SE_ADT_PARAMETER_ARRAY_ENTRY", pack=False, align=None),
    "SE_ADT_PARAMETER_ARRAY_EX": SimStruct(OrderedDict((("CategoryId", SimTypeInt(signed=False, label="UInt32")), ("AuditId", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("ParameterCount", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("FlatSubCategoryId", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Parameters", SimTypeArray(SimTypeRef("SE_ADT_PARAMETER_ARRAY_ENTRY", SimStruct), 32)),)), name="SE_ADT_PARAMETER_ARRAY_EX", pack=False, align=None),
    "SE_ADT_PARAMETER_TYPE": SimTypeInt(signed=False, label="SE_ADT_PARAMETER_TYPE"),
    "SE_AUDIT_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("AuditType", SimTypeInt(signed=False, label="AUDIT_EVENT_TYPE")), ("AuditOperation", SimTypeInt(signed=False, label="SE_AUDIT_OPERATION")), ("AuditFlags", SimTypeInt(signed=False, label="UInt32")), ("SubsystemName", SimTypeRef("UNICODE_STRING", SimStruct)), ("ObjectTypeName", SimTypeRef("UNICODE_STRING", SimStruct)), ("ObjectName", SimTypeRef("UNICODE_STRING", SimStruct)), ("HandleId", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransactionId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("OperationId", SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)), ("ObjectCreation", SimTypeChar(label="Byte")), ("GenerateOnClose", SimTypeChar(label="Byte")),)), name="SE_AUDIT_INFO", pack=False, align=None),
    "SE_AUDIT_OPERATION": SimTypeInt(signed=False, label="SE_AUDIT_OPERATION"),
    "SE_EXPORTS": SimStruct(OrderedDict((("SeCreateTokenPrivilege", SimTypeRef("LUID", SimStruct)), ("SeAssignPrimaryTokenPrivilege", SimTypeRef("LUID", SimStruct)), ("SeLockMemoryPrivilege", SimTypeRef("LUID", SimStruct)), ("SeIncreaseQuotaPrivilege", SimTypeRef("LUID", SimStruct)), ("SeUnsolicitedInputPrivilege", SimTypeRef("LUID", SimStruct)), ("SeTcbPrivilege", SimTypeRef("LUID", SimStruct)), ("SeSecurityPrivilege", SimTypeRef("LUID", SimStruct)), ("SeTakeOwnershipPrivilege", SimTypeRef("LUID", SimStruct)), ("SeLoadDriverPrivilege", SimTypeRef("LUID", SimStruct)), ("SeCreatePagefilePrivilege", SimTypeRef("LUID", SimStruct)), ("SeIncreaseBasePriorityPrivilege", SimTypeRef("LUID", SimStruct)), ("SeSystemProfilePrivilege", SimTypeRef("LUID", SimStruct)), ("SeSystemtimePrivilege", SimTypeRef("LUID", SimStruct)), ("SeProfileSingleProcessPrivilege", SimTypeRef("LUID", SimStruct)), ("SeCreatePermanentPrivilege", SimTypeRef("LUID", SimStruct)), ("SeBackupPrivilege", SimTypeRef("LUID", SimStruct)), ("SeRestorePrivilege", SimTypeRef("LUID", SimStruct)), ("SeShutdownPrivilege", SimTypeRef("LUID", SimStruct)), ("SeDebugPrivilege", SimTypeRef("LUID", SimStruct)), ("SeAuditPrivilege", SimTypeRef("LUID", SimStruct)), ("SeSystemEnvironmentPrivilege", SimTypeRef("LUID", SimStruct)), ("SeChangeNotifyPrivilege", SimTypeRef("LUID", SimStruct)), ("SeRemoteShutdownPrivilege", SimTypeRef("LUID", SimStruct)), ("SeNullSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeWorldSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeLocalSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeCreatorOwnerSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeCreatorGroupSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeNtAuthoritySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeDialupSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeNetworkSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeBatchSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeInteractiveSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeLocalSystemSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasAdminsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasUsersSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasGuestsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasPowerUsersSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasAccountOpsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasSystemOpsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasPrintOpsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAliasBackupOpsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAuthenticatedUsersSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeRestrictedSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAnonymousLogonSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeUndockPrivilege", SimTypeRef("LUID", SimStruct)), ("SeSyncAgentPrivilege", SimTypeRef("LUID", SimStruct)), ("SeEnableDelegationPrivilege", SimTypeRef("LUID", SimStruct)), ("SeLocalServiceSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeNetworkServiceSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeManageVolumePrivilege", SimTypeRef("LUID", SimStruct)), ("SeImpersonatePrivilege", SimTypeRef("LUID", SimStruct)), ("SeCreateGlobalPrivilege", SimTypeRef("LUID", SimStruct)), ("SeTrustedCredManAccessPrivilege", SimTypeRef("LUID", SimStruct)), ("SeRelabelPrivilege", SimTypeRef("LUID", SimStruct)), ("SeIncreaseWorkingSetPrivilege", SimTypeRef("LUID", SimStruct)), ("SeTimeZonePrivilege", SimTypeRef("LUID", SimStruct)), ("SeCreateSymbolicLinkPrivilege", SimTypeRef("LUID", SimStruct)), ("SeIUserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeUntrustedMandatorySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeLowMandatorySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeMediumMandatorySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeHighMandatorySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeSystemMandatorySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeOwnerRightsSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAllAppPackagesSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeUserModeDriversSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeProcTrustWinTcbSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeTrustedInstallerSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeDelegateSessionUserImpersonatePrivilege", SimTypeRef("LUID", SimStruct)), ("SeAppSiloSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAppSiloVolumeRootMinimalCapabilitySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SeAppSiloProfilesRootMinimalCapabilitySid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SE_EXPORTS", pack=False, align=None),
    "SE_IMAGE_SIGNATURE_TYPE": SimTypeInt(signed=False, label="SE_IMAGE_SIGNATURE_TYPE"),
    "SE_IMAGE_TYPE": SimTypeInt(signed=False, label="SE_IMAGE_TYPE"),
    "SE_IMAGE_VERIFICATION_CALLBACK_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SE_IMAGE_TYPE"), SimTypePointer(SimTypeRef("BDCB_IMAGE_INFORMATION", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["CallbackContext", "ImageType", "ImageInformation"]), offset=0),
    "SE_IMAGE_VERIFICATION_CALLBACK_TYPE": SimTypeInt(signed=False, label="SE_IMAGE_VERIFICATION_CALLBACK_TYPE"),
    "SE_IMPERSONATION_STATE": SimStruct(OrderedDict((("Token", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CopyOnOpen", SimTypeChar(label="Byte")), ("EffectiveOnly", SimTypeChar(label="Byte")), ("Level", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")),)), name="SE_IMPERSONATION_STATE", pack=False, align=None),
    "SE_LOGON_SESSION_TERMINATED_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId"]), offset=0),
    "SE_LOGON_SESSION_TERMINATED_ROUTINE_EX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "pServerSilo", "Context"]), offset=0),
    "SE_OBJECT_TYPE": SimTypeInt(signed=False, label="SE_OBJECT_TYPE"),
    "SE_SECURITY_DESCRIPTOR": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SE_SECURITY_DESCRIPTOR", pack=False, align=None),
    "SE_SID": SimUnion({"Sid": SimTypeRef("SID", SimStruct), "Buffer": SimTypeArray(SimTypeChar(label="Byte"), 68)}, name="<anon>", label="None"),
    "SE_TOKEN_USER": SimStruct(OrderedDict((("Anonymous1", SimUnion({"TokenUser": SimTypeRef("TOKEN_USER", SimStruct), "User": SimTypeRef("SID_AND_ATTRIBUTES", SimStruct)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Sid": SimTypeRef("SID", SimStruct), "Buffer": SimTypeArray(SimTypeChar(label="Byte"), 68)}, name="<anon>", label="None")),)), name="SE_TOKEN_USER", pack=False, align=None),
    "SExistRestriction": SimStruct(OrderedDict((("ulReserved1", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="SExistRestriction", pack=False, align=None),
    "SFBS_FLAGS": SimTypeInt(signed=False, label="SFBS_FLAGS"),
    "SFGAO_FLAGS": SimTypeInt(signed=False, label="SFGAO_FLAGS"),
    "SFVM_HELPTOPIC_DATA": SimStruct(OrderedDict((("wszHelpFile", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wszHelpTopic", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SFVM_HELPTOPIC_DATA", pack=False, align=None),
    "SFVM_MESSAGE_ID": SimTypeInt(signed=False, label="SFVM_MESSAGE_ID"),
    "SFVM_PROPPAGE_DATA": SimStruct(OrderedDict((("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pfn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SFVM_PROPPAGE_DATA", pack=False, align=None),
    "SFVS_SELECT": SimTypeInt(signed=False, label="SFVS_SELECT"),
    "SFV_CREATE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pshf", SimTypeBottom(label="IShellFolder")), ("psvOuter", SimTypeBottom(label="IShellView")), ("psfvcb", SimTypeBottom(label="IShellFolderViewCB")),)), name="SFV_CREATE", pack=False, align=None),
    "SFV_SETITEMPOS": SimStruct(OrderedDict((("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("pt", SimTypeRef("POINT", SimStruct)),)), name="SFV_SETITEMPOS", pack=False, align=None),
    "SF_PROPERTY_IIS": SimTypeInt(signed=False, label="SF_PROPERTY_IIS"),
    "SF_REQ_TYPE": SimTypeInt(signed=False, label="SF_REQ_TYPE"),
    "SF_STATUS_TYPE": SimTypeInt(signed=False, label="SF_STATUS_TYPE"),
    "SF_TYPE": SimTypeInt(signed=False, label="SF_TYPE"),
    "SGuidArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpguid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="SGuidArray", pack=False, align=None),
    "SHANDLE_PTR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SHARD": SimTypeInt(signed=False, label="SHARD"),
    "SHARDAPPIDINFO": SimStruct(OrderedDict((("psi", SimTypeBottom(label="IShellItem")), ("pszAppID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARDAPPIDINFO", pack=False, align=None),
    "SHARDAPPIDINFOIDLIST": SimStruct(OrderedDict((("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("pszAppID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARDAPPIDINFOIDLIST", pack=False, align=None),
    "SHARDAPPIDINFOLINK": SimStruct(OrderedDict((("psl", SimTypeBottom(label="IShellLinkA")), ("pszAppID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARDAPPIDINFOLINK", pack=False, align=None),
    "SHAREDSECRETPROPERTIES": SimTypeInt(signed=False, label="SHAREDSECRETPROPERTIES"),
    "SHARED_VIRTUAL_DISK_SUPPORT": SimStruct(OrderedDict((("SharedVirtualDiskSupport", SimTypeInt(signed=False, label="SharedVirtualDiskSupportType")), ("HandleState", SimTypeInt(signed=False, label="SharedVirtualDiskHandleState")),)), name="SHARED_VIRTUAL_DISK_SUPPORT", pack=False, align=None),
    "SHARE_ACCESS": SimStruct(OrderedDict((("OpenCount", SimTypeInt(signed=False, label="UInt32")), ("Readers", SimTypeInt(signed=False, label="UInt32")), ("Writers", SimTypeInt(signed=False, label="UInt32")), ("Deleters", SimTypeInt(signed=False, label="UInt32")), ("SharedRead", SimTypeInt(signed=False, label="UInt32")), ("SharedWrite", SimTypeInt(signed=False, label="UInt32")), ("SharedDelete", SimTypeInt(signed=False, label="UInt32")),)), name="SHARE_ACCESS", pack=False, align=None),
    "SHARE_INFO_0": SimStruct(OrderedDict((("shi0_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARE_INFO_0", pack=False, align=None),
    "SHARE_INFO_1": SimStruct(OrderedDict((("shi1_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi1_type", SimTypeInt(signed=False, label="SHARE_TYPE")), ("shi1_remark", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARE_INFO_1", pack=False, align=None),
    "SHARE_INFO_1004": SimStruct(OrderedDict((("shi1004_remark", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARE_INFO_1004", pack=False, align=None),
    "SHARE_INFO_1005": SimStruct(OrderedDict((("shi1005_flags", SimTypeInt(signed=False, label="UInt32")),)), name="SHARE_INFO_1005", pack=False, align=None),
    "SHARE_INFO_1006": SimStruct(OrderedDict((("shi1006_max_uses", SimTypeInt(signed=False, label="UInt32")),)), name="SHARE_INFO_1006", pack=False, align=None),
    "SHARE_INFO_1501": SimStruct(OrderedDict((("shi1501_reserved", SimTypeInt(signed=False, label="UInt32")), ("shi1501_security_descriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SHARE_INFO_1501", pack=False, align=None),
    "SHARE_INFO_1503": SimStruct(OrderedDict((("shi1503_sharefilter", SimTypeBottom(label="Guid")),)), name="SHARE_INFO_1503", pack=False, align=None),
    "SHARE_INFO_2": SimStruct(OrderedDict((("shi2_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi2_type", SimTypeInt(signed=False, label="SHARE_TYPE")), ("shi2_remark", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi2_permissions", SimTypeInt(signed=False, label="SHARE_INFO_PERMISSIONS")), ("shi2_max_uses", SimTypeInt(signed=False, label="UInt32")), ("shi2_current_uses", SimTypeInt(signed=False, label="UInt32")), ("shi2_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi2_passwd", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHARE_INFO_2", pack=False, align=None),
    "SHARE_INFO_501": SimStruct(OrderedDict((("shi501_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi501_type", SimTypeInt(signed=False, label="SHARE_TYPE")), ("shi501_remark", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi501_flags", SimTypeInt(signed=False, label="UInt32")),)), name="SHARE_INFO_501", pack=False, align=None),
    "SHARE_INFO_502": SimStruct(OrderedDict((("shi502_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi502_type", SimTypeInt(signed=False, label="SHARE_TYPE")), ("shi502_remark", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi502_permissions", SimTypeInt(signed=False, label="SHARE_INFO_PERMISSIONS")), ("shi502_max_uses", SimTypeInt(signed=False, label="UInt32")), ("shi502_current_uses", SimTypeInt(signed=False, label="UInt32")), ("shi502_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi502_passwd", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi502_reserved", SimTypeInt(signed=False, label="UInt32")), ("shi502_security_descriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SHARE_INFO_502", pack=False, align=None),
    "SHARE_INFO_503": SimStruct(OrderedDict((("shi503_netname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi503_type", SimTypeInt(signed=False, label="SHARE_TYPE")), ("shi503_remark", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi503_permissions", SimTypeInt(signed=False, label="SHARE_INFO_PERMISSIONS")), ("shi503_max_uses", SimTypeInt(signed=False, label="UInt32")), ("shi503_current_uses", SimTypeInt(signed=False, label="UInt32")), ("shi503_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi503_passwd", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi503_servername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("shi503_reserved", SimTypeInt(signed=False, label="UInt32")), ("shi503_security_descriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SHARE_INFO_503", pack=False, align=None),
    "SHARE_INFO_PERMISSIONS": SimTypeInt(signed=False, label="SHARE_INFO_PERMISSIONS"),
    "SHARE_ROLE": SimTypeInt(signed=False, label="SHARE_ROLE"),
    "SHARE_TYPE": SimTypeInt(signed=False, label="SHARE_TYPE"),
    "SHARINGCONNECTIONTYPE": SimTypeInt(signed=False, label="SHARINGCONNECTIONTYPE"),
    "SHARINGCONNECTION_ENUM_FLAGS": SimTypeInt(signed=False, label="SHARINGCONNECTION_ENUM_FLAGS"),
    "SHCNE_ID": SimTypeInt(signed=False, label="SHCNE_ID"),
    "SHCNF_FLAGS": SimTypeInt(signed=False, label="SHCNF_FLAGS"),
    "SHCNRF_SOURCE": SimTypeInt(signed=False, label="SHCNRF_SOURCE"),
    "SHCOLSTATE": SimTypeInt(signed=False, label="SHCOLSTATE"),
    "SHCOLUMNDATA": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pwszExt", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszFile", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SHCOLUMNDATA", pack=False, align=None),
    "SHCOLUMNINFO": SimStruct(OrderedDict((("scid", SimTypeRef("PROPERTYKEY", SimStruct)), ("vt", SimTypeInt(signed=False, label="VARENUM")), ("fmt", SimTypeInt(signed=False, label="UInt32")), ("cChars", SimTypeInt(signed=False, label="UInt32")), ("csFlags", SimTypeInt(signed=False, label="UInt32")), ("wszTitle", SimTypeArray(SimTypeChar(label="Char"), 80)), ("wszDescription", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="SHCOLUMNINFO", pack=False, align=None),
    "SHCOLUMNINIT": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("wszFolder", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SHCOLUMNINIT", pack=False, align=None),
    "SHCREATEPROCESSINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCurrentDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hUserToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpProcessAttributes", SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0)), ("lpThreadAttributes", SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0)), ("bInheritHandles", SimTypeInt(signed=True, label="Int32")), ("dwCreationFlags", SimTypeInt(signed=False, label="UInt32")), ("lpStartupInfo", SimTypePointer(SimTypeRef("STARTUPINFOW", SimStruct), offset=0)), ("lpProcessInformation", SimTypePointer(SimTypeRef("PROCESS_INFORMATION", SimStruct), offset=0)),)), name="SHCREATEPROCESSINFOW", pack=False, align=None),
    "SHChangeDWORDAsIDList": SimStruct(OrderedDict((("cb", SimTypeShort(signed=False, label="UInt16")), ("dwItem1", SimTypeInt(signed=False, label="UInt32")), ("dwItem2", SimTypeInt(signed=False, label="UInt32")), ("cbZero", SimTypeShort(signed=False, label="UInt16")),)), name="SHChangeDWORDAsIDList", pack=False, align=None),
    "SHChangeNotifyEntry": SimStruct(OrderedDict((("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("fRecursive", SimTypeInt(signed=True, label="Int32")),)), name="SHChangeNotifyEntry", pack=False, align=None),
    "SHChangeProductKeyAsIDList": SimStruct(OrderedDict((("cb", SimTypeShort(signed=False, label="UInt16")), ("wszProductKey", SimTypeArray(SimTypeChar(label="Char"), 39)), ("cbZero", SimTypeShort(signed=False, label="UInt16")),)), name="SHChangeProductKeyAsIDList", pack=False, align=None),
    "SHChangeUpdateImageIDList": SimStruct(OrderedDict((("cb", SimTypeShort(signed=False, label="UInt16")), ("iIconIndex", SimTypeInt(signed=True, label="Int32")), ("iCurIndex", SimTypeInt(signed=True, label="Int32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")), ("dwProcessID", SimTypeInt(signed=False, label="UInt32")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("cbZero", SimTypeShort(signed=False, label="UInt16")),)), name="SHChangeUpdateImageIDList", pack=False, align=None),
    "SHDESCRIPTIONID": SimStruct(OrderedDict((("dwDescriptionId", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")),)), name="SHDESCRIPTIONID", pack=False, align=None),
    "SHDID_ID": SimTypeInt(signed=False, label="SHDID_ID"),
    "SHDRAGIMAGE": SimStruct(OrderedDict((("sizeDragImage", SimTypeRef("SIZE", SimStruct)), ("ptOffset", SimTypeRef("POINT", SimStruct)), ("hbmpDragImage", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("crColorKey", SimTypeInt(signed=False, label="UInt32")),)), name="SHDRAGIMAGE", pack=False, align=None),
    "SHELLBROWSERSHOWCONTROL": SimTypeInt(signed=False, label="SHELLBROWSERSHOWCONTROL"),
    "SHELLDETAILS": SimStruct(OrderedDict((("fmt", SimTypeInt(signed=True, label="Int32")), ("cxChar", SimTypeInt(signed=True, label="Int32")), ("str", SimTypeRef("STRRET", SimStruct)),)), name="SHELLDETAILS", pack=False, align=None),
    "SHELLEXECUTEINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpVerb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("nShow", SimTypeInt(signed=True, label="Int32")), ("hInstApp", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpIDList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpClass", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hkeyClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwHotKey", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hMonitor": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SHELLEXECUTEINFOA", pack=False, align=None),
    "SHELLEXECUTEINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fMask", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpVerb", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nShow", SimTypeInt(signed=True, label="Int32")), ("hInstApp", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpIDList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hkeyClass", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwHotKey", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"hIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hMonitor": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SHELLEXECUTEINFOW", pack=False, align=None),
    "SHELLFLAGSTATE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")),)), name="SHELLFLAGSTATE", pack=False, align=None),
    "SHELLHOOKINFO": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("rc", SimTypeRef("RECT", SimStruct)),)), name="SHELLHOOKINFO", pack=False, align=None),
    "SHELLSTATEA": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=True, label="Int32")), ("dwWin95Unused", SimTypeInt(signed=False, label="UInt32")), ("uWin95Unused", SimTypeInt(signed=False, label="UInt32")), ("lParamSort", SimTypeInt(signed=True, label="Int32")), ("iSortDirection", SimTypeInt(signed=True, label="Int32")), ("version", SimTypeInt(signed=False, label="UInt32")), ("uNotUsed", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=True, label="Int32")),)), name="SHELLSTATEA", pack=False, align=None),
    "SHELLSTATEW": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=True, label="Int32")), ("dwWin95Unused", SimTypeInt(signed=False, label="UInt32")), ("uWin95Unused", SimTypeInt(signed=False, label="UInt32")), ("lParamSort", SimTypeInt(signed=True, label="Int32")), ("iSortDirection", SimTypeInt(signed=True, label="Int32")), ("version", SimTypeInt(signed=False, label="UInt32")), ("uNotUsed", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=True, label="Int32")),)), name="SHELLSTATEW", pack=False, align=None),
    "SHELL_AUTOCOMPLETE_FLAGS": SimTypeInt(signed=False, label="SHELL_AUTOCOMPLETE_FLAGS"),
    "SHELL_ITEM_RESOURCE": SimStruct(OrderedDict((("guidType", SimTypeBottom(label="Guid")), ("szName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SHELL_ITEM_RESOURCE", pack=False, align=None),
    "SHELL_LINK_DATA_FLAGS": SimTypeInt(signed=False, label="SHELL_LINK_DATA_FLAGS"),
    "SHELL_UI_COMPONENT": SimTypeInt(signed=False, label="SHELL_UI_COMPONENT"),
    "SHFILEINFOA": SimStruct(OrderedDict((("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIcon", SimTypeInt(signed=True, label="Int32")), ("dwAttributes", SimTypeInt(signed=False, label="UInt32")), ("szDisplayName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("szTypeName", SimTypeArray(SimTypeChar(label="SByte"), 80)),)), name="SHFILEINFOA", pack=False, align=None),
    "SHFILEINFOW": SimStruct(OrderedDict((("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIcon", SimTypeInt(signed=True, label="Int32")), ("dwAttributes", SimTypeInt(signed=False, label="UInt32")), ("szDisplayName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szTypeName", SimTypeArray(SimTypeChar(label="Char"), 80)),)), name="SHFILEINFOW", pack=False, align=None),
    "SHFILEOPSTRUCTA": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wFunc", SimTypeInt(signed=False, label="UInt32")), ("pFrom", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("pTo", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("fFlags", SimTypeShort(signed=False, label="UInt16")), ("fAnyOperationsAborted", SimTypeInt(signed=True, label="Int32")), ("hNameMappings", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpszProgressTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SHFILEOPSTRUCTA", pack=False, align=None),
    "SHFILEOPSTRUCTW": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wFunc", SimTypeInt(signed=False, label="UInt32")), ("pFrom", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pTo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fFlags", SimTypeShort(signed=False, label="UInt16")), ("fAnyOperationsAborted", SimTypeInt(signed=True, label="Int32")), ("hNameMappings", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpszProgressTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SHFILEOPSTRUCTW", pack=False, align=None),
    "SHFMT_ID": SimTypeInt(signed=False, label="SHFMT_ID"),
    "SHFMT_OPT": SimTypeInt(signed=False, label="SHFMT_OPT"),
    "SHFMT_RET": SimTypeInt(signed=False, label="SHFMT_RET"),
    "SHFOLDERCUSTOMSETTINGS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("pvid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pszWebViewTemplate", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchWebViewTemplate", SimTypeInt(signed=False, label="UInt32")), ("pszWebViewTemplateVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszInfoTip", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchInfoTip", SimTypeInt(signed=False, label="UInt32")), ("pclsid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pszIconFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchIconFile", SimTypeInt(signed=False, label="UInt32")), ("iIconIndex", SimTypeInt(signed=True, label="Int32")), ("pszLogo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchLogo", SimTypeInt(signed=False, label="UInt32")),)), name="SHFOLDERCUSTOMSETTINGS", pack=False, align=None),
    "SHGDFIL_FORMAT": SimTypeInt(signed=False, label="SHGDFIL_FORMAT"),
    "SHGDNF": SimTypeInt(signed=False, label="SHGDNF"),
    "SHGFI_FLAGS": SimTypeInt(signed=False, label="SHGFI_FLAGS"),
    "SHGFP_TYPE": SimTypeInt(signed=False, label="SHGFP_TYPE"),
    "SHGLOBALCOUNTER": SimTypeInt(signed=False, label="SHGLOBALCOUNTER"),
    "SHGSI_FLAGS": SimTypeInt(signed=False, label="SHGSI_FLAGS"),
    "SHIMOPTS": SimTypeInt(signed=False, label="SHIMOPTS"),
    "SHITEMID": SimStruct(OrderedDict((("cb", SimTypeShort(signed=False, label="UInt16")), ("abID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SHITEMID", pack=False, align=None),
    "SHNAMEMAPPINGA": SimStruct(OrderedDict((("pszOldPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszNewPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchOldPath", SimTypeInt(signed=True, label="Int32")), ("cchNewPath", SimTypeInt(signed=True, label="Int32")),)), name="SHNAMEMAPPINGA", pack=False, align=None),
    "SHNAMEMAPPINGW": SimStruct(OrderedDict((("pszOldPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszNewPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchOldPath", SimTypeInt(signed=True, label="Int32")), ("cchNewPath", SimTypeInt(signed=True, label="Int32")),)), name="SHNAMEMAPPINGW", pack=False, align=None),
    "SHOP_TYPE": SimTypeInt(signed=False, label="SHOP_TYPE"),
    "SHOWCALENDARBUTTONRIGHTSTATES": SimTypeInt(signed=False, label="SHOWCALENDARBUTTONRIGHTSTATES"),
    "SHOWHTMLDIALOGEXFN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IMoniker"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pmk", "dwDialogFlags", "pvarArgIn", "pchOptions", "pvArgOut"]), offset=0),
    "SHOWHTMLDIALOGFN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pmk", "pvarArgIn", "pchOptions", "pvArgOut"]), offset=0),
    "SHOWMODELESSHTMLDIALOGFN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0), SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="IHTMLWindow2"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "pmk", "pvarArgIn", "pvarOptions", "ppWindow"]), offset=0),
    "SHOWUIPARAMS": SimStruct(OrderedDict((("UIType", SimTypeInt(signed=False, label="UI_TYPE")), ("MessageBoxParams", SimTypeRef("MESSAGEBOX_PARAMS", SimStruct)),)), name="SHOWUIPARAMS", pack=False, align=None),
    "SHOW_WINDOW_CMD": SimTypeInt(signed=False, label="SHOW_WINDOW_CMD"),
    "SHOW_WINDOW_STATUS": SimTypeInt(signed=False, label="SHOW_WINDOW_STATUS"),
    "SHQUERYRBINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("i64Size", SimTypeLongLong(signed=True, label="Int64")), ("i64NumItems", SimTypeLongLong(signed=True, label="Int64")),)), name="SHQUERYRBINFO", pack=False, align=None),
    "SHREGDEL_FLAGS": SimTypeInt(signed=False, label="SHREGDEL_FLAGS"),
    "SHREGENUM_FLAGS": SimTypeInt(signed=False, label="SHREGENUM_FLAGS"),
    "SHRINK_VOLUME_INFORMATION": SimStruct(OrderedDict((("ShrinkRequestType", SimTypeInt(signed=False, label="SHRINK_VOLUME_REQUEST_TYPES")), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("NewNumberOfSectors", SimTypeLongLong(signed=True, label="Int64")),)), name="SHRINK_VOLUME_INFORMATION", pack=False, align=None),
    "SHRINK_VOLUME_REQUEST_TYPES": SimTypeInt(signed=False, label="SHRINK_VOLUME_REQUEST_TYPES"),
    "SHSTOCKICONID": SimTypeInt(signed=False, label="SHSTOCKICONID"),
    "SHSTOCKICONINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iSysImageIndex", SimTypeInt(signed=True, label="Int32")), ("iIcon", SimTypeInt(signed=True, label="Int32")), ("szPath", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SHSTOCKICONINFO", pack=False, align=None),
    "SHUFFLE_FILE_DATA": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SHUFFLE_FILE_DATA", pack=False, align=None),
    "SHUTDOWN_FLAGS": SimTypeInt(signed=False, label="SHUTDOWN_FLAGS"),
    "SHUTDOWN_REASON": SimTypeInt(signed=False, label="SHUTDOWN_REASON"),
    "SHVTEMPLATEPROPERTIES": SimTypeInt(signed=False, label="SHVTEMPLATEPROPERTIES"),
    "SHV_COMBINATION_TYPE": SimTypeInt(signed=False, label="SHV_COMBINATION_TYPE"),
    "SIATTRIBFLAGS": SimTypeInt(signed=False, label="SIATTRIBFLAGS"),
    "SID": SimStruct(OrderedDict((("Revision", SimTypeChar(label="Byte")), ("SubAuthorityCount", SimTypeChar(label="Byte")), ("IdentifierAuthority", SimTypeRef("SID_IDENTIFIER_AUTHORITY", SimStruct)), ("SubAuthority", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="SID", pack=False, align=None),
    "SIDESHOW_COLOR_TYPE": SimTypeInt(signed=False, label="SIDESHOW_COLOR_TYPE"),
    "SIDESHOW_SCREEN_TYPE": SimTypeInt(signed=False, label="SIDESHOW_SCREEN_TYPE"),
    "SID_AND_ATTRIBUTES": SimStruct(OrderedDict((("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")),)), name="SID_AND_ATTRIBUTES", pack=False, align=None),
    "SID_AND_ATTRIBUTES_HASH": SimStruct(OrderedDict((("SidCount", SimTypeInt(signed=False, label="UInt32")), ("SidAttr", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)), ("Hash", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 32)),)), name="SID_AND_ATTRIBUTES_HASH", pack=False, align=None),
    "SID_IDENTIFIER_AUTHORITY": SimStruct(OrderedDict((("Value", SimTypeArray(SimTypeChar(label="Byte"), 6)),)), name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None),
    "SID_INFO": SimStruct(OrderedDict((("pSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pwzCommonName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwzUPN", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SID_INFO", pack=False, align=None),
    "SID_INFO_LIST": SimStruct(OrderedDict((("cItems", SimTypeInt(signed=False, label="UInt32")), ("aSidInfo", SimTypePointer(SimTypeRef("SID_INFO", SimStruct), offset=0)),)), name="SID_INFO_LIST", pack=False, align=None),
    "SID_NAME_USE": SimTypeInt(signed=False, label="SID_NAME_USE"),
    "SIERRAADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")),)), name="SIERRAADPCMWAVEFORMAT", pack=False, align=None),
    "SIGDN": SimTypeInt(signed=False, label="SIGDN"),
    "SIGNAL_REG_VALUE": SimStruct(OrderedDict((("RegName", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("MsrAddr", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="SIGNAL_REG_VALUE", pack=False, align=None),
    "SIGNER_ATTR_AUTHCODE": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fCommercial", SimTypeInt(signed=True, label="Int32")), ("fIndividual", SimTypeInt(signed=True, label="Int32")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SIGNER_ATTR_AUTHCODE", pack=False, align=None),
    "SIGNER_BLOB_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pGuidSubject", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("cbBlob", SimTypeInt(signed=False, label="UInt32")), ("pbBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SIGNER_BLOB_INFO", pack=False, align=None),
    "SIGNER_CERT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwCertChoice", SimTypeInt(signed=False, label="SIGNER_CERT_CHOICE")), ("Anonymous", SimUnion({"pwszSpcFile": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pCertStoreInfo": SimTypePointer(SimTypeRef("SIGNER_CERT_STORE_INFO", SimStruct), offset=0), "pSpcChainInfo": SimTypePointer(SimTypeRef("SIGNER_SPC_CHAIN_INFO", SimStruct), offset=0)}, name="<anon>", label="None")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SIGNER_CERT", pack=False, align=None),
    "SIGNER_CERT_CHOICE": SimTypeInt(signed=False, label="SIGNER_CERT_CHOICE"),
    "SIGNER_CERT_POLICY": SimTypeInt(signed=False, label="SIGNER_CERT_POLICY"),
    "SIGNER_CERT_STORE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pSigningCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("dwCertPolicy", SimTypeInt(signed=False, label="SIGNER_CERT_POLICY")), ("hCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SIGNER_CERT_STORE_INFO", pack=False, align=None),
    "SIGNER_CONTEXT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("cbBlob", SimTypeInt(signed=False, label="UInt32")), ("pbBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SIGNER_CONTEXT", pack=False, align=None),
    "SIGNER_DIGEST_SIGN_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwDigestSignChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pfnAuthenticodeDigestSign": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSigningCert", "pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest"]), offset=0), "pfnAuthenticodeDigestSignWithFileHandle": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSigningCert", "pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "hFile", "pSignedDigest"]), offset=0), "pfnAuthenticodeDigestSignEx": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest", "ppSignerCert", "hCertChainStore"]), offset=0), "pfnAuthenticodeDigestSignExWithFileHandle": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "hFile", "pSignedDigest", "ppSignerCert", "hCertChainStore"]), offset=0)}, name="<anon>", label="None")), ("pMetadataBlob", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="SIGNER_DIGEST_SIGN_INFO", pack=False, align=None),
    "SIGNER_DIGEST_SIGN_INFO_V1": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnAuthenticodeDigestSign", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSigningCert", "pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest"]), offset=0)), ("pMetadataBlob", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="SIGNER_DIGEST_SIGN_INFO_V1", pack=False, align=None),
    "SIGNER_DIGEST_SIGN_INFO_V2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pfnAuthenticodeDigestSign", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSigningCert", "pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest"]), offset=0)), ("pfnAuthenticodeDigestSignEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypeInt(signed=False, label="ALG_ID"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMetadataBlob", "digestAlgId", "pbToBeSignedDigest", "cbToBeSignedDigest", "pSignedDigest", "ppSignerCert", "hCertChainStore"]), offset=0)), ("pMetadataBlob", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)),)), name="SIGNER_DIGEST_SIGN_INFO_V2", pack=False, align=None),
    "SIGNER_FILE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pwszFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SIGNER_FILE_INFO", pack=False, align=None),
    "SIGNER_PRIVATE_KEY_CHOICE": SimTypeInt(signed=False, label="SIGNER_PRIVATE_KEY_CHOICE"),
    "SIGNER_PROVIDER_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pwszProviderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwProviderType", SimTypeInt(signed=False, label="UInt32")), ("dwKeySpec", SimTypeInt(signed=False, label="UInt32")), ("dwPvkChoice", SimTypeInt(signed=False, label="SIGNER_PRIVATE_KEY_CHOICE")), ("Anonymous", SimUnion({"pwszPvkFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszKeyContainer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="SIGNER_PROVIDER_INFO", pack=False, align=None),
    "SIGNER_SIGNATURE_ATTRIBUTE_CHOICE": SimTypeInt(signed=False, label="SIGNER_SIGNATURE_ATTRIBUTE_CHOICE"),
    "SIGNER_SIGNATURE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("algidHash", SimTypeInt(signed=False, label="ALG_ID")), ("dwAttrChoice", SimTypeInt(signed=False, label="SIGNER_SIGNATURE_ATTRIBUTE_CHOICE")), ("Anonymous", SimUnion({"pAttrAuthcode": SimTypePointer(SimTypeRef("SIGNER_ATTR_AUTHCODE", SimStruct), offset=0)}, name="<anon>", label="None")), ("psAuthenticated", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTES", SimStruct), offset=0)), ("psUnauthenticated", SimTypePointer(SimTypeRef("CRYPT_ATTRIBUTES", SimStruct), offset=0)),)), name="SIGNER_SIGNATURE_INFO", pack=False, align=None),
    "SIGNER_SIGN_FLAGS": SimTypeInt(signed=False, label="SIGNER_SIGN_FLAGS"),
    "SIGNER_SPC_CHAIN_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pwszSpcFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwCertPolicy", SimTypeInt(signed=False, label="UInt32")), ("hCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SIGNER_SPC_CHAIN_INFO", pack=False, align=None),
    "SIGNER_SUBJECT_CHOICE": SimTypeInt(signed=False, label="SIGNER_SUBJECT_CHOICE"),
    "SIGNER_SUBJECT_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pdwIndex", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("dwSubjectChoice", SimTypeInt(signed=False, label="SIGNER_SUBJECT_CHOICE")), ("Anonymous", SimUnion({"pSignerFileInfo": SimTypePointer(SimTypeRef("SIGNER_FILE_INFO", SimStruct), offset=0), "pSignerBlobInfo": SimTypePointer(SimTypeRef("SIGNER_BLOB_INFO", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="SIGNER_SUBJECT_INFO", pack=False, align=None),
    "SIGNER_TIMESTAMP_FLAGS": SimTypeInt(signed=False, label="SIGNER_TIMESTAMP_FLAGS"),
    "SIIGBF": SimTypeInt(signed=False, label="SIIGBF"),
    "SILOOBJECT_BASIC_INFORMATION": SimStruct(OrderedDict((("SiloId", SimTypeInt(signed=False, label="UInt32")), ("SiloParentId", SimTypeInt(signed=False, label="UInt32")), ("NumberOfProcesses", SimTypeInt(signed=False, label="UInt32")), ("IsInServerSilo", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="SILOOBJECT_BASIC_INFORMATION", pack=False, align=None),
    "SILO_CONTEXT_CLEANUP_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["SiloContext"]), offset=0),
    "SILO_INFO": SimStruct(OrderedDict((("ulSTID", SimTypeInt(signed=False, label="UInt32")), ("SpecificationMajor", SimTypeChar(label="Byte")), ("SpecificationMinor", SimTypeChar(label="Byte")), ("ImplementationMajor", SimTypeChar(label="Byte")), ("ImplementationMinor", SimTypeChar(label="Byte")), ("type", SimTypeChar(label="Byte")), ("capabilities", SimTypeChar(label="Byte")),)), name="SILO_INFO", pack=False, align=None),
    "SILO_MONITOR_CREATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Silo"]), offset=0),
    "SILO_MONITOR_REGISTRATION": SimStruct(OrderedDict((("Version", SimTypeChar(label="Byte")), ("MonitorHost", SimTypeChar(label="Byte")), ("MonitorExistingSilos", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("Anonymous", SimUnion({"DriverObjectName": SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), "ComponentName": SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)}, name="<anon>", label="None")), ("CreateCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Silo"]), offset=0)), ("TerminateCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Silo"]), offset=0)),)), name="SILO_MONITOR_REGISTRATION", pack=False, align=None),
    "SILO_MONITOR_TERMINATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["Silo"]), offset=0),
    "SIMPLE_DEVICE_ORIENTATION": SimTypeInt(signed=False, label="SIMPLE_DEVICE_ORIENTATION"),
    "SIMULATE_CAPS_1": SimStruct(OrderedDict((("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwPageOrderFlags", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfCopies", SimTypeInt(signed=False, label="UInt32")), ("dwCollate", SimTypeInt(signed=False, label="UInt32")), ("dwNupOptions", SimTypeInt(signed=False, label="UInt32")),)), name="SIMULATE_CAPS_1", pack=False, align=None),
    "SINGLE_LIST_ENTRY": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("SINGLE_LIST_ENTRY", SimStruct), offset=0)),)), name="SINGLE_LIST_ENTRY", pack=False, align=None),
    "SINGLE_LIST_ENTRY32": SimStruct(OrderedDict((("Next", SimTypeInt(signed=False, label="UInt32")),)), name="SINGLE_LIST_ENTRY32", pack=False, align=None),
    "SIPAEVENT_KSR_SIGNATURE_PAYLOAD": SimStruct(OrderedDict((("SignAlgID", SimTypeInt(signed=False, label="UInt32")), ("SignatureLength", SimTypeInt(signed=False, label="UInt32")), ("Signature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SIPAEVENT_KSR_SIGNATURE_PAYLOAD", pack=False, align=None),
    "SIPAEVENT_REVOCATION_LIST_PAYLOAD": SimStruct(OrderedDict((("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("DigestLength", SimTypeInt(signed=False, label="UInt32")), ("HashAlgID", SimTypeShort(signed=False, label="UInt16")), ("Digest", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SIPAEVENT_REVOCATION_LIST_PAYLOAD", pack=False, align=None),
    "SIPAEVENT_SBCP_INFO_PAYLOAD_V1": SimStruct(OrderedDict((("PayloadVersion", SimTypeInt(signed=False, label="UInt32")), ("VarDataOffset", SimTypeInt(signed=False, label="UInt32")), ("HashAlgID", SimTypeShort(signed=False, label="UInt16")), ("DigestLength", SimTypeShort(signed=False, label="UInt16")), ("Options", SimTypeInt(signed=False, label="UInt32")), ("SignersCount", SimTypeInt(signed=False, label="UInt32")), ("VarData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SIPAEVENT_SBCP_INFO_PAYLOAD_V1", pack=False, align=None),
    "SIPAEVENT_SI_POLICY_PAYLOAD": SimStruct(OrderedDict((("PolicyVersion", SimTypeLongLong(signed=False, label="UInt64")), ("PolicyNameLength", SimTypeShort(signed=False, label="UInt16")), ("HashAlgID", SimTypeShort(signed=False, label="UInt16")), ("DigestLength", SimTypeInt(signed=False, label="UInt32")), ("VarLengthData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SIPAEVENT_SI_POLICY_PAYLOAD", pack=False, align=None),
    "SIPAEVENT_VSM_IDK_INFO_PAYLOAD": SimStruct(OrderedDict((("KeyAlgID", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"RsaKeyInfo": SimTypeRef("SIPAEVENT_VSM_IDK_RSA_INFO", SimStruct)}, name="<anon>", label="None")),)), name="SIPAEVENT_VSM_IDK_INFO_PAYLOAD", pack=False, align=None),
    "SIPAEVENT_VSM_IDK_RSA_INFO": SimStruct(OrderedDict((("KeyBitLength", SimTypeInt(signed=False, label="UInt32")), ("PublicExpLengthBytes", SimTypeInt(signed=False, label="UInt32")), ("ModulusSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("PublicKeyData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SIPAEVENT_VSM_IDK_RSA_INFO", pack=False, align=None),
    "SIP_ADD_NEWPROVIDER": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pgSubject", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pwszDLLFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszMagicNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIsFunctionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszGetFuncName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPutFuncName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszCreateFuncName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszVerifyFuncName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszRemoveFuncName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIsFunctionNameFmt2", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszGetCapFuncName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SIP_ADD_NEWPROVIDER", pack=False, align=None),
    "SIP_CAP_SET_V2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("isMultiSign", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="SIP_CAP_SET_V2", pack=False, align=None),
    "SIP_CAP_SET_V3": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("isMultiSign", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="SIP_CAP_SET_V3", pack=False, align=None),
    "SIP_DISPATCH_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hSIP", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pfGet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pdwEncodingType", "dwIndex", "pcbSignedDataMsg", "pbSignedDataMsg"]), offset=0)), ("pfPut", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "dwEncodingType", "pdwIndex", "cbSignedDataMsg", "pbSignedDataMsg"]), offset=0)), ("pfCreate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pcbIndirectData", "pIndirectData"]), offset=0)), ("pfVerify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pIndirectData"]), offset=0)), ("pfRemove", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "dwIndex"]), offset=0)),)), name="SIP_DISPATCH_INFO", pack=False, align=None),
    "SIP_INDIRECT_DATA": SimStruct(OrderedDict((("Data", SimTypeRef("CRYPT_ATTRIBUTE_TYPE_VALUE", SimStruct)), ("DigestAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Digest", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SIP_INDIRECT_DATA", pack=False, align=None),
    "SIP_SUBJECTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pgSubjectType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwsFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwsDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwIntVersion", SimTypeInt(signed=False, label="UInt32")), ("hProv", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DigestAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwEncodingType", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("fdwCAPISettings", SimTypeInt(signed=False, label="UInt32")), ("fdwSecuritySettings", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwUnionChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"psFlat": SimTypePointer(SimTypeRef("MS_ADDINFO_FLAT", SimStruct), offset=0), "psCatMember": SimTypePointer(SimTypeRef("MS_ADDINFO_CATALOGMEMBER", SimStruct), offset=0), "psBlob": SimTypePointer(SimTypeRef("MS_ADDINFO_BLOB", SimStruct), offset=0)}, name="<anon>", label="None")), ("pClientData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SIP_SUBJECTINFO", pack=False, align=None),
    "SIZE": SimStruct(OrderedDict((("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")),)), name="SIZE", pack=False, align=None),
    "SIZEBOXSTATES": SimTypeInt(signed=False, label="SIZEBOXSTATES"),
    "SIZINGTYPE": SimTypeInt(signed=False, label="SIZINGTYPE"),
    "SI_ACCESS": SimStruct(OrderedDict((("pguid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("mask", SimTypeInt(signed=False, label="UInt32")), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SI_ACCESS", pack=False, align=None),
    "SI_COPYFILE": SimStruct(OrderedDict((("SourceFileNameLength", SimTypeInt(signed=False, label="UInt32")), ("DestinationFileNameLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FileNameBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SI_COPYFILE", pack=False, align=None),
    "SI_INHERIT_TYPE": SimStruct(OrderedDict((("pguid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="ACE_FLAGS")), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SI_INHERIT_TYPE", pack=False, align=None),
    "SI_OBJECT_INFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="SI_OBJECT_INFO_FLAGS")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPageTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("guidObjectType", SimTypeBottom(label="Guid")),)), name="SI_OBJECT_INFO", pack=False, align=None),
    "SI_OBJECT_INFO_FLAGS": SimTypeInt(signed=False, label="SI_OBJECT_INFO_FLAGS"),
    "SI_PAGE_ACTIVATED": SimTypeInt(signed=False, label="SI_PAGE_ACTIVATED"),
    "SI_PAGE_TYPE": SimTypeInt(signed=False, label="SI_PAGE_TYPE"),
    "SLDATATYPE": SimTypeInt(signed=False, label="SLDATATYPE"),
    "SLGP_FLAGS": SimTypeInt(signed=False, label="SLGP_FLAGS"),
    "SLIDTYPE": SimTypeInt(signed=False, label="SLIDTYPE"),
    "SLIST_ENTRY": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("SLIST_ENTRY", SimStruct), offset=0)),)), name="SLIST_ENTRY", pack=False, align=None),
    "SLIST_HEADER": SimUnion({"Alignment": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Next", SimTypeRef("SINGLE_LIST_ENTRY", SimStruct)), ("Depth", SimTypeShort(signed=False, label="UInt16")), ("CpuId", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "SLLICENSINGSTATUS": SimTypeInt(signed=False, label="SLLICENSINGSTATUS"),
    "SLOWAPPINFO": SimStruct(OrderedDict((("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("ftLastUsed", SimTypeRef("FILETIME", SimStruct)), ("iTimesUsed", SimTypeInt(signed=True, label="Int32")), ("pszImage", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SLOWAPPINFO", pack=False, align=None),
    "SLPSTRArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lppszA", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="SLPSTRArray", pack=False, align=None),
    "SLREFERRALTYPE": SimTypeInt(signed=False, label="SLREFERRALTYPE"),
    "SLR_FLAGS": SimTypeInt(signed=False, label="SLR_FLAGS"),
    "SL_ACTIVATION_INFO_HEADER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="SL_ACTIVATION_TYPE")),)), name="SL_ACTIVATION_INFO_HEADER", pack=False, align=None),
    "SL_ACTIVATION_TYPE": SimTypeInt(signed=False, label="SL_ACTIVATION_TYPE"),
    "SL_AD_ACTIVATION_INFO": SimStruct(OrderedDict((("header", SimTypeRef("SL_ACTIVATION_INFO_HEADER", SimStruct)), ("pwszProductKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszActivationObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SL_AD_ACTIVATION_INFO", pack=False, align=None),
    "SL_GENUINE_STATE": SimTypeInt(signed=False, label="SL_GENUINE_STATE"),
    "SL_LICENSING_STATUS": SimStruct(OrderedDict((("SkuId", SimTypeBottom(label="Guid")), ("eStatus", SimTypeInt(signed=False, label="SLLICENSINGSTATUS")), ("dwGraceTime", SimTypeInt(signed=False, label="UInt32")), ("dwTotalGraceDays", SimTypeInt(signed=False, label="UInt32")), ("hrReason", SimTypeInt(signed=True, label="Int32")), ("qwValidityExpiration", SimTypeLongLong(signed=False, label="UInt64")),)), name="SL_LICENSING_STATUS", pack=False, align=None),
    "SL_NONGENUINE_UI_OPTIONS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pComponentId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("hResultUI", SimTypeInt(signed=True, label="Int32")),)), name="SL_NONGENUINE_UI_OPTIONS", pack=False, align=None),
    "SL_SYSTEM_POLICY_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="SL_SYSTEM_POLICY_INFORMATION", pack=False, align=None),
    "SLargeIntegerArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpli", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="SLargeIntegerArray", pack=False, align=None),
    "SLongArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpl", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="SLongArray", pack=False, align=None),
    "SMALLCAPTIONSTATES": SimTypeInt(signed=False, label="SMALLCAPTIONSTATES"),
    "SMALLCLOSEBUTTONSTATES": SimTypeInt(signed=False, label="SMALLCLOSEBUTTONSTATES"),
    "SMALLFRAMEBOTTOMSTATES": SimTypeInt(signed=False, label="SMALLFRAMEBOTTOMSTATES"),
    "SMALLFRAMELEFTSTATES": SimTypeInt(signed=False, label="SMALLFRAMELEFTSTATES"),
    "SMALLFRAMERIGHTSTATES": SimTypeInt(signed=False, label="SMALLFRAMERIGHTSTATES"),
    "SMALL_RECT": SimStruct(OrderedDict((("Left", SimTypeShort(signed=True, label="Int16")), ("Top", SimTypeShort(signed=True, label="Int16")), ("Right", SimTypeShort(signed=True, label="Int16")), ("Bottom", SimTypeShort(signed=True, label="Int16")),)), name="SMALL_RECT", pack=False, align=None),
    "SMB_COMPRESSION_INFO": SimStruct(OrderedDict((("Switch", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")),)), name="SMB_COMPRESSION_INFO", pack=False, align=None),
    "SMB_SHARE_FLUSH_AND_PURGE_INPUT": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")),)), name="SMB_SHARE_FLUSH_AND_PURGE_INPUT", pack=False, align=None),
    "SMB_SHARE_FLUSH_AND_PURGE_OUTPUT": SimStruct(OrderedDict((("cEntriesPurged", SimTypeInt(signed=False, label="UInt32")),)), name="SMB_SHARE_FLUSH_AND_PURGE_OUTPUT", pack=False, align=None),
    "SMB_TREE_CONNECT_PARAMETERS": SimStruct(OrderedDict((("EABufferOffset", SimTypeInt(signed=False, label="UInt32")), ("EABufferLen", SimTypeInt(signed=False, label="UInt32")), ("CreateOptions", SimTypeInt(signed=False, label="UInt32")), ("TreeConnectAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="SMB_TREE_CONNECT_PARAMETERS", pack=False, align=None),
    "SMB_USE_OPTION_COMPRESSION_PARAMETERS": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="SMB_USE_OPTION_COMPRESSION_PARAMETERS", pack=False, align=None),
    "SMCSHCHANGENOTIFYSTRUCT": SimStruct(OrderedDict((("lEvent", SimTypeInt(signed=True, label="Int32")), ("pidl1", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("pidl2", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)),)), name="SMCSHCHANGENOTIFYSTRUCT", pack=False, align=None),
    "SMDATA": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hmenu", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uId", SimTypeInt(signed=False, label="UInt32")), ("uIdParent", SimTypeInt(signed=False, label="UInt32")), ("uIdAncestor", SimTypeInt(signed=False, label="UInt32")), ("punk", SimTypeBottom(label="IUnknown")), ("pidlFolder", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("pidlItem", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("psf", SimTypeBottom(label="IShellFolder")), ("pvUserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SMDATA", pack=False, align=None),
    "SMINFO": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("iIcon", SimTypeInt(signed=True, label="Int32")),)), name="SMINFO", pack=False, align=None),
    "SMINFOFLAGS": SimTypeInt(signed=False, label="SMINFOFLAGS"),
    "SMINFOMASK": SimTypeInt(signed=False, label="SMINFOMASK"),
    "SMINFOTYPE": SimTypeInt(signed=False, label="SMINFOTYPE"),
    "SMMCDataObjects": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("lpDataObject", SimTypePointer(SimTypeBottom(label="IDataObject"), offset=0)),)), name="SMMCDataObjects", pack=False, align=None),
    "SMMCObjectTypes": SimStruct(OrderedDict((("count", SimTypeInt(signed=False, label="UInt32")), ("guid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="SMMCObjectTypes", pack=False, align=None),
    "SMS_MESSAGE_TYPES": SimTypeInt(signed=False, label="SMS_MESSAGE_TYPES"),
    "SNAP_HEADER": SimStruct(OrderedDict((("Dsap", SimTypeChar(label="Byte")), ("Ssap", SimTypeChar(label="Byte")), ("Control", SimTypeChar(label="Byte")), ("Oui", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Type", SimTypeShort(signed=False, label="UInt16")),)), name="SNAP_HEADER", pack=False, align=None),
    "SNDDEV_ERR": SimTypeInt(signed=False, label="SNDDEV_ERR"),
    "SND_FLAGS": SimTypeInt(signed=False, label="SND_FLAGS"),
    "SNMPAPI_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hSession", "hWnd", "wMsg", "wParam", "lParam", "lpClientData"]), offset=0),
    "SNMP_API_TRANSLATE_MODE": SimTypeInt(signed=False, label="SNMP_API_TRANSLATE_MODE"),
    "SNMP_ERROR": SimTypeInt(signed=False, label="SNMP_ERROR"),
    "SNMP_ERROR_STATUS": SimTypeInt(signed=False, label="SNMP_ERROR_STATUS"),
    "SNMP_EXTENSION_REQUEST_TYPE": SimTypeInt(signed=False, label="SNMP_EXTENSION_REQUEST_TYPE"),
    "SNMP_GENERICTRAP": SimTypeInt(signed=False, label="SNMP_GENERICTRAP"),
    "SNMP_LOG": SimTypeInt(signed=False, label="SNMP_LOG"),
    "SNMP_OUTPUT_LOG_TYPE": SimTypeInt(signed=False, label="SNMP_OUTPUT_LOG_TYPE"),
    "SNMP_PDU_TYPE": SimTypeInt(signed=False, label="SNMP_PDU_TYPE"),
    "SNMP_STATUS": SimTypeInt(signed=False, label="SNMP_STATUS"),
    "SNodeID": SimStruct(OrderedDict((("cBytes", SimTypeInt(signed=False, label="UInt32")), ("id", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SNodeID", pack=False, align=None),
    "SNodeID2": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cBytes", SimTypeInt(signed=False, label="UInt32")), ("id", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SNodeID2", pack=False, align=None),
    "SNotRestriction": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")), ("lpRes", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)),)), name="SNotRestriction", pack=False, align=None),
    "SOCKADDR": SimStruct(OrderedDict((("sa_family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("sa_data", SimTypeArray(SimTypeChar(label="SByte"), 14)),)), name="SOCKADDR", pack=False, align=None),
    "SOCKADDR_ATM": SimStruct(OrderedDict((("satm_family", SimTypeShort(signed=False, label="UInt16")), ("satm_number", SimTypeRef("ATM_ADDRESS", SimStruct)), ("satm_blli", SimTypeRef("ATM_BLLI", SimStruct)), ("satm_bhli", SimTypeRef("ATM_BHLI", SimStruct)),)), name="SOCKADDR_ATM", pack=False, align=None),
    "SOCKADDR_BTH": SimStruct(OrderedDict((("addressFamily", SimTypeShort(signed=False, label="UInt16")), ("btAddr", SimTypeLongLong(signed=False, label="UInt64")), ("serviceClassId", SimTypeBottom(label="Guid")), ("port", SimTypeInt(signed=False, label="UInt32")),)), name="SOCKADDR_BTH", pack=False, align=None),
    "SOCKADDR_DL": SimStruct(OrderedDict((("sdl_family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("sdl_data", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("sdl_zero", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="SOCKADDR_DL", pack=False, align=None),
    "SOCKADDR_HV": SimStruct(OrderedDict((("Family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("VmId", SimTypeBottom(label="Guid")), ("ServiceId", SimTypeBottom(label="Guid")),)), name="SOCKADDR_HV", pack=False, align=None),
    "SOCKADDR_IN": SimStruct(OrderedDict((("sin_family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("sin_port", SimTypeShort(signed=False, label="UInt16")), ("sin_addr", SimTypeRef("IN_ADDR", SimStruct)), ("sin_zero", SimTypeArray(SimTypeChar(label="SByte"), 8)),)), name="SOCKADDR_IN", pack=False, align=None),
    "SOCKADDR_IN6": SimStruct(OrderedDict((("sin6_family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("sin6_port", SimTypeShort(signed=False, label="UInt16")), ("sin6_flowinfo", SimTypeInt(signed=False, label="UInt32")), ("sin6_addr", SimTypeRef("IN6_ADDR", SimStruct)), ("Anonymous", SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimTypeRef("SCOPE_ID", SimStruct)}, name="<anon>", label="None")),)), name="SOCKADDR_IN6", pack=False, align=None),
    "SOCKADDR_IN6_PAIR": SimStruct(OrderedDict((("SourceAddress", SimTypePointer(SimTypeRef("SOCKADDR_IN6", SimStruct), offset=0)), ("DestinationAddress", SimTypePointer(SimTypeRef("SOCKADDR_IN6", SimStruct), offset=0)),)), name="SOCKADDR_IN6_PAIR", pack=False, align=None),
    "SOCKADDR_IN6_W2KSP1": SimStruct(OrderedDict((("sin6_family", SimTypeShort(signed=True, label="Int16")), ("sin6_port", SimTypeShort(signed=False, label="UInt16")), ("sin6_flowinfo", SimTypeInt(signed=False, label="UInt32")), ("sin6_addr", SimTypeRef("IN6_ADDR", SimStruct)), ("sin6_scope_id", SimTypeInt(signed=False, label="UInt32")),)), name="SOCKADDR_IN6_W2KSP1", pack=False, align=None),
    "SOCKADDR_INET": SimUnion({"Ipv4": SimTypeRef("SOCKADDR_IN", SimStruct), "Ipv6": SimTypeRef("SOCKADDR_IN6", SimStruct), "si_family": SimTypeInt(signed=False, label="ADDRESS_FAMILY")}, name="<anon>", label="None"),
    "SOCKADDR_IPX": SimStruct(OrderedDict((("sa_family", SimTypeShort(signed=True, label="Int16")), ("sa_netnum", SimTypeArray(SimTypeChar(label="SByte"), 4)), ("sa_nodenum", SimTypeArray(SimTypeChar(label="SByte"), 6)), ("sa_socket", SimTypeShort(signed=False, label="UInt16")),)), name="SOCKADDR_IPX", pack=False, align=None),
    "SOCKADDR_IRDA": SimStruct(OrderedDict((("irdaAddressFamily", SimTypeShort(signed=False, label="UInt16")), ("irdaDeviceID", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("irdaServiceName", SimTypeArray(SimTypeChar(label="SByte"), 25)),)), name="SOCKADDR_IRDA", pack=False, align=None),
    "SOCKADDR_NB": SimStruct(OrderedDict((("snb_family", SimTypeShort(signed=True, label="Int16")), ("snb_type", SimTypeShort(signed=False, label="UInt16")), ("snb_name", SimTypeArray(SimTypeChar(label="SByte"), 16)),)), name="SOCKADDR_NB", pack=False, align=None),
    "SOCKADDR_STORAGE": SimStruct(OrderedDict((("ss_family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("__ss_pad1", SimTypeArray(SimTypeChar(label="SByte"), 6)), ("__ss_align", SimTypeLongLong(signed=True, label="Int64")), ("__ss_pad2", SimTypeArray(SimTypeChar(label="SByte"), 112)),)), name="SOCKADDR_STORAGE", pack=False, align=None),
    "SOCKADDR_STORAGE_XP": SimStruct(OrderedDict((("ss_family", SimTypeShort(signed=True, label="Int16")), ("__ss_pad1", SimTypeArray(SimTypeChar(label="SByte"), 6)), ("__ss_align", SimTypeLongLong(signed=True, label="Int64")), ("__ss_pad2", SimTypeArray(SimTypeChar(label="SByte"), 112)),)), name="SOCKADDR_STORAGE_XP", pack=False, align=None),
    "SOCKADDR_TP": SimStruct(OrderedDict((("tp_family", SimTypeShort(signed=False, label="UInt16")), ("tp_addr_type", SimTypeShort(signed=False, label="UInt16")), ("tp_taddr_len", SimTypeShort(signed=False, label="UInt16")), ("tp_tsel_len", SimTypeShort(signed=False, label="UInt16")), ("tp_addr", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="SOCKADDR_TP", pack=False, align=None),
    "SOCKADDR_UN": SimStruct(OrderedDict((("sun_family", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("sun_path", SimTypeArray(SimTypeChar(label="SByte"), 108)),)), name="SOCKADDR_UN", pack=False, align=None),
    "SOCKADDR_VNS": SimStruct(OrderedDict((("sin_family", SimTypeShort(signed=False, label="UInt16")), ("net_address", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("subnet_addr", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("port", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("hops", SimTypeChar(label="Byte")), ("filler", SimTypeArray(SimTypeChar(label="Byte"), 5)),)), name="SOCKADDR_VNS", pack=False, align=None),
    "SOCKET": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "SOCKET_ADDRESS": SimStruct(OrderedDict((("lpSockaddr", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("iSockaddrLength", SimTypeInt(signed=True, label="Int32")),)), name="SOCKET_ADDRESS", pack=False, align=None),
    "SOCKET_ADDRESS_LIST": SimStruct(OrderedDict((("iAddressCount", SimTypeInt(signed=True, label="Int32")), ("Address", SimTypePointer(SimTypeRef("SOCKET_ADDRESS", SimStruct), offset=0)),)), name="SOCKET_ADDRESS_LIST", pack=False, align=None),
    "SOCKET_PEER_TARGET_NAME": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("PeerAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("PeerTargetNameStringLen", SimTypeInt(signed=False, label="UInt32")), ("AllStrings", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SOCKET_PEER_TARGET_NAME", pack=False, align=None),
    "SOCKET_PRIORITY_HINT": SimTypeInt(signed=False, label="SOCKET_PRIORITY_HINT"),
    "SOCKET_PROCESSOR_AFFINITY": SimStruct(OrderedDict((("Processor", SimTypeRef("PROCESSOR_NUMBER", SimStruct)), ("NumaNodeId", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="SOCKET_PROCESSOR_AFFINITY", pack=False, align=None),
    "SOCKET_SECURITY_PROTOCOL": SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL"),
    "SOCKET_SECURITY_QUERY_INFO": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PeerApplicationAccessTokenHandle", SimTypeLongLong(signed=False, label="UInt64")), ("PeerMachineAccessTokenHandle", SimTypeLongLong(signed=False, label="UInt64")),)), name="SOCKET_SECURITY_QUERY_INFO", pack=False, align=None),
    "SOCKET_SECURITY_QUERY_INFO_IPSEC2": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PeerApplicationAccessTokenHandle", SimTypeLongLong(signed=False, label="UInt64")), ("PeerMachineAccessTokenHandle", SimTypeLongLong(signed=False, label="UInt64")), ("MmSaId", SimTypeLongLong(signed=False, label="UInt64")), ("QmSaId", SimTypeLongLong(signed=False, label="UInt64")), ("NegotiationWinerr", SimTypeInt(signed=False, label="UInt32")), ("SaLookupContext", SimTypeBottom(label="Guid")),)), name="SOCKET_SECURITY_QUERY_INFO_IPSEC2", pack=False, align=None),
    "SOCKET_SECURITY_QUERY_TEMPLATE": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("PeerAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("PeerTokenAccessMask", SimTypeInt(signed=False, label="UInt32")),)), name="SOCKET_SECURITY_QUERY_TEMPLATE", pack=False, align=None),
    "SOCKET_SECURITY_QUERY_TEMPLATE_IPSEC2": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("PeerAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("PeerTokenAccessMask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("FieldMask", SimTypeInt(signed=False, label="UInt32")),)), name="SOCKET_SECURITY_QUERY_TEMPLATE_IPSEC2", pack=False, align=None),
    "SOCKET_SECURITY_SETTINGS": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("SecurityFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SOCKET_SECURITY_SETTINGS", pack=False, align=None),
    "SOCKET_SECURITY_SETTINGS_IPSEC": SimStruct(OrderedDict((("SecurityProtocol", SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL")), ("SecurityFlags", SimTypeInt(signed=False, label="UInt32")), ("IpsecFlags", SimTypeInt(signed=False, label="UInt32")), ("AuthipMMPolicyKey", SimTypeBottom(label="Guid")), ("AuthipQMPolicyKey", SimTypeBottom(label="Guid")), ("Reserved", SimTypeBottom(label="Guid")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("UserNameStringLen", SimTypeInt(signed=False, label="UInt32")), ("DomainNameStringLen", SimTypeInt(signed=False, label="UInt32")), ("PasswordStringLen", SimTypeInt(signed=False, label="UInt32")), ("AllStrings", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SOCKET_SECURITY_SETTINGS_IPSEC", pack=False, align=None),
    "SOCKET_USAGE_TYPE": SimTypeInt(signed=False, label="SOCKET_USAGE_TYPE"),
    "SOCK_NOTIFY_REGISTRATION": SimStruct(OrderedDict((("socket", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("completionKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("eventFilter", SimTypeShort(signed=False, label="UInt16")), ("operation", SimTypeChar(label="Byte")), ("triggerFlags", SimTypeChar(label="Byte")), ("registrationResult", SimTypeInt(signed=False, label="UInt32")),)), name="SOCK_NOTIFY_REGISTRATION", pack=False, align=None),
    "SOC_SUBSYSTEM_FAILURE_DETAILS": SimStruct(OrderedDict((("SubsysType", SimTypeInt(signed=False, label="SOC_SUBSYSTEM_TYPE")), ("FirmwareVersion", SimTypeLongLong(signed=False, label="UInt64")), ("HardwareVersion", SimTypeLongLong(signed=False, label="UInt64")), ("UnifiedFailureRegionSize", SimTypeInt(signed=False, label="UInt32")), ("UnifiedFailureRegion", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SOC_SUBSYSTEM_FAILURE_DETAILS", pack=False, align=None),
    "SOC_SUBSYSTEM_TYPE": SimTypeInt(signed=False, label="SOC_SUBSYSTEM_TYPE"),
    "SOFTDISTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwAdState", SimTypeInt(signed=False, label="UInt32")), ("szTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szAbstract", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("szHREF", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwInstalledVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwInstalledVersionLS", SimTypeInt(signed=False, label="UInt32")), ("dwUpdateVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwUpdateVersionLS", SimTypeInt(signed=False, label="UInt32")), ("dwAdvertisedVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwAdvertisedVersionLS", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="SOFTDISTINFO", pack=False, align=None),
    "SOFTKBDDATA": SimStruct(OrderedDict((("uCount", SimTypeInt(signed=False, label="UInt32")), ("wCode", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="SOFTKBDDATA", pack=False, align=None),
    "SOFTWAREEXPLORERSTATES": SimTypeInt(signed=False, label="SOFTWAREEXPLORERSTATES"),
    "SOLE_AUTHENTICATION_INFO": SimStruct(OrderedDict((("dwAuthnSvc", SimTypeInt(signed=False, label="UInt32")), ("dwAuthzSvc", SimTypeInt(signed=False, label="UInt32")), ("pAuthInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SOLE_AUTHENTICATION_INFO", pack=False, align=None),
    "SOLE_AUTHENTICATION_LIST": SimStruct(OrderedDict((("cAuthInfo", SimTypeInt(signed=False, label="UInt32")), ("aAuthInfo", SimTypePointer(SimTypeRef("SOLE_AUTHENTICATION_INFO", SimStruct), offset=0)),)), name="SOLE_AUTHENTICATION_LIST", pack=False, align=None),
    "SOLE_AUTHENTICATION_SERVICE": SimStruct(OrderedDict((("dwAuthnSvc", SimTypeInt(signed=False, label="UInt32")), ("dwAuthzSvc", SimTypeInt(signed=False, label="UInt32")), ("pPrincipalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hr", SimTypeInt(signed=True, label="Int32")),)), name="SOLE_AUTHENTICATION_SERVICE", pack=False, align=None),
    "SOMITEMTYPE": SimTypeInt(signed=False, label="SOMITEMTYPE"),
    "SONARCWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wCompType", SimTypeShort(signed=False, label="UInt16")),)), name="SONARCWAVEFORMAT", pack=False, align=None),
    "SORTCOLUMN": SimStruct(OrderedDict((("propkey", SimTypeRef("PROPERTYKEY", SimStruct)), ("direction", SimTypeInt(signed=False, label="SORTDIRECTION")),)), name="SORTCOLUMN", pack=False, align=None),
    "SORTCOMP": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pv1", "pv2"]), offset=0),
    "SORTDIRECTION": SimTypeInt(signed=False, label="SORTDIRECTION"),
    "SORTKEY": SimStruct(OrderedDict((("propColumn", SimTypeRef("FULLPROPSPEC", SimStruct)), ("dwOrder", SimTypeInt(signed=False, label="UInt32")), ("locale", SimTypeInt(signed=False, label="UInt32")),)), name="SORTKEY", pack=False, align=None),
    "SORTSET": SimStruct(OrderedDict((("cCol", SimTypeInt(signed=False, label="UInt32")), ("aCol", SimTypePointer(SimTypeRef("SORTKEY", SimStruct), offset=0)),)), name="SORTSET", pack=False, align=None),
    "SORT_ORDER_TYPE": SimTypeInt(signed=False, label="SORT_ORDER_TYPE"),
    "SOUNDDETECTOR_PATTERNHEADER": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("PatternType", SimTypeBottom(label="Guid")),)), name="SOUNDDETECTOR_PATTERNHEADER", pack=False, align=None),
    "SOUNDSENTRYA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="SOUNDSENTRY_FLAGS")), ("iFSTextEffect", SimTypeInt(signed=False, label="SOUNDSENTRY_TEXT_EFFECT")), ("iFSTextEffectMSec", SimTypeInt(signed=False, label="UInt32")), ("iFSTextEffectColorBits", SimTypeInt(signed=False, label="UInt32")), ("iFSGrafEffect", SimTypeInt(signed=False, label="SOUND_SENTRY_GRAPHICS_EFFECT")), ("iFSGrafEffectMSec", SimTypeInt(signed=False, label="UInt32")), ("iFSGrafEffectColor", SimTypeInt(signed=False, label="UInt32")), ("iWindowsEffect", SimTypeInt(signed=False, label="SOUNDSENTRY_WINDOWS_EFFECT")), ("iWindowsEffectMSec", SimTypeInt(signed=False, label="UInt32")), ("lpszWindowsEffectDLL", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("iWindowsEffectOrdinal", SimTypeInt(signed=False, label="UInt32")),)), name="SOUNDSENTRYA", pack=False, align=None),
    "SOUNDSENTRYW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="SOUNDSENTRY_FLAGS")), ("iFSTextEffect", SimTypeInt(signed=False, label="SOUNDSENTRY_TEXT_EFFECT")), ("iFSTextEffectMSec", SimTypeInt(signed=False, label="UInt32")), ("iFSTextEffectColorBits", SimTypeInt(signed=False, label="UInt32")), ("iFSGrafEffect", SimTypeInt(signed=False, label="SOUND_SENTRY_GRAPHICS_EFFECT")), ("iFSGrafEffectMSec", SimTypeInt(signed=False, label="UInt32")), ("iFSGrafEffectColor", SimTypeInt(signed=False, label="UInt32")), ("iWindowsEffect", SimTypeInt(signed=False, label="SOUNDSENTRY_WINDOWS_EFFECT")), ("iWindowsEffectMSec", SimTypeInt(signed=False, label="UInt32")), ("lpszWindowsEffectDLL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("iWindowsEffectOrdinal", SimTypeInt(signed=False, label="UInt32")),)), name="SOUNDSENTRYW", pack=False, align=None),
    "SOUNDSENTRY_FLAGS": SimTypeInt(signed=False, label="SOUNDSENTRY_FLAGS"),
    "SOUNDSENTRY_TEXT_EFFECT": SimTypeInt(signed=False, label="SOUNDSENTRY_TEXT_EFFECT"),
    "SOUNDSENTRY_WINDOWS_EFFECT": SimTypeInt(signed=False, label="SOUNDSENTRY_WINDOWS_EFFECT"),
    "SOUND_SENTRY_GRAPHICS_EFFECT": SimTypeInt(signed=False, label="SOUND_SENTRY_GRAPHICS_EFFECT"),
    "SOURCEFILE": SimStruct(OrderedDict((("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("FileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SOURCEFILE", pack=False, align=None),
    "SOURCEFILEW": SimStruct(OrderedDict((("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SOURCEFILEW", pack=False, align=None),
    "SOURCE_GROUP_ENTRY": SimStruct(OrderedDict((("dwSourceAddr", SimTypeInt(signed=False, label="UInt32")), ("dwSourceMask", SimTypeInt(signed=False, label="UInt32")), ("dwGroupAddr", SimTypeInt(signed=False, label="UInt32")), ("dwGroupMask", SimTypeInt(signed=False, label="UInt32")),)), name="SOURCE_GROUP_ENTRY", pack=False, align=None),
    "SOURCE_MEDIA_A": SimStruct(OrderedDict((("Reserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Tagfile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SourcePath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SourceFile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SOURCE_MEDIA_A", pack=False, align=None),
    "SOURCE_MEDIA_W": SimStruct(OrderedDict((("Reserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Tagfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourcePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourceFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SOURCE_MEDIA_W", pack=False, align=None),
    "SOrRestriction": SimStruct(OrderedDict((("cRes", SimTypeInt(signed=False, label="UInt32")), ("lpRes", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)),)), name="SOrRestriction", pack=False, align=None),
    "SPACTION": SimTypeInt(signed=False, label="SPACTION"),
    "SPADAPTATIONRELEVANCE": SimTypeInt(signed=False, label="SPADAPTATIONRELEVANCE"),
    "SPADAPTATIONSETTINGS": SimTypeInt(signed=False, label="SPADAPTATIONSETTINGS"),
    "SPATIAL_AUDIO_STREAM_OPTIONS": SimTypeInt(signed=False, label="SPATIAL_AUDIO_STREAM_OPTIONS"),
    "SPAUDIOBUFFERINFO": SimStruct(OrderedDict((("ulMsMinNotification", SimTypeInt(signed=False, label="UInt32")), ("ulMsBufferSize", SimTypeInt(signed=False, label="UInt32")), ("ulMsEventBias", SimTypeInt(signed=False, label="UInt32")),)), name="SPAUDIOBUFFERINFO", pack=False, align=None),
    "SPAUDIOOPTIONS": SimTypeInt(signed=False, label="SPAUDIOOPTIONS"),
    "SPAUDIOSTATE": SimTypeInt(signed=False, label="SPAUDIOSTATE"),
    "SPAUDIOSTATUS": SimStruct(OrderedDict((("cbFreeBuffSpace", SimTypeInt(signed=True, label="Int32")), ("cbNonBlockingIO", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="SPAUDIOSTATE")), ("CurSeekPos", SimTypeLongLong(signed=False, label="UInt64")), ("CurDevicePos", SimTypeLongLong(signed=False, label="UInt64")), ("dwAudioLevel", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="SPAUDIOSTATUS", pack=False, align=None),
    "SPBINARYGRAMMAR": SimStruct(OrderedDict((("ulTotalSerializedSize", SimTypeInt(signed=False, label="UInt32")),)), name="SPBINARYGRAMMAR", pack=False, align=None),
    "SPBOOKMARKOPTIONS": SimTypeInt(signed=False, label="SPBOOKMARKOPTIONS"),
    "SPCFGNOTIFY": SimTypeInt(signed=False, label="SPCFGNOTIFY"),
    "SPCFGRULEATTRIBUTES": SimTypeInt(signed=False, label="SPCFGRULEATTRIBUTES"),
    "SPCOMMITFLAGS": SimTypeInt(signed=False, label="SPCOMMITFLAGS"),
    "SPCONTEXTSTATE": SimTypeInt(signed=False, label="SPCONTEXTSTATE"),
    "SPC_FINANCIAL_CRITERIA": SimStruct(OrderedDict((("fFinancialInfoAvailable", SimTypeInt(signed=True, label="Int32")), ("fMeetsCriteria", SimTypeInt(signed=True, label="Int32")),)), name="SPC_FINANCIAL_CRITERIA", pack=False, align=None),
    "SPC_IMAGE": SimStruct(OrderedDict((("pImageLink", SimTypePointer(SimTypeRef("SPC_LINK", SimStruct), offset=0)), ("Bitmap", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("Metafile", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("EnhancedMetafile", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)), ("GifFile", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SPC_IMAGE", pack=False, align=None),
    "SPC_INDIRECT_DATA_CONTENT": SimStruct(OrderedDict((("Data", SimTypeRef("CRYPT_ATTRIBUTE_TYPE_VALUE", SimStruct)), ("DigestAlgorithm", SimTypeRef("CRYPT_ALGORITHM_IDENTIFIER", SimStruct)), ("Digest", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SPC_INDIRECT_DATA_CONTENT", pack=False, align=None),
    "SPC_LINK": SimStruct(OrderedDict((("dwLinkChoice", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pwszUrl": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Moniker": SimTypeRef("SPC_SERIALIZED_OBJECT", SimStruct), "pwszFile": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="SPC_LINK", pack=False, align=None),
    "SPC_PE_IMAGE_DATA": SimStruct(OrderedDict((("Flags", SimTypeRef("CRYPT_BIT_BLOB", SimStruct)), ("pFile", SimTypePointer(SimTypeRef("SPC_LINK", SimStruct), offset=0)),)), name="SPC_PE_IMAGE_DATA", pack=False, align=None),
    "SPC_SERIALIZED_OBJECT": SimStruct(OrderedDict((("ClassId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("SerializedData", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SPC_SERIALIZED_OBJECT", pack=False, align=None),
    "SPC_SIGINFO": SimStruct(OrderedDict((("dwSipVersion", SimTypeInt(signed=False, label="UInt32")), ("gSIPGuid", SimTypeBottom(label="Guid")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("dwReserved3", SimTypeInt(signed=False, label="UInt32")), ("dwReserved4", SimTypeInt(signed=False, label="UInt32")), ("dwReserved5", SimTypeInt(signed=False, label="UInt32")),)), name="SPC_SIGINFO", pack=False, align=None),
    "SPC_SP_AGENCY_INFO": SimStruct(OrderedDict((("pPolicyInformation", SimTypePointer(SimTypeRef("SPC_LINK", SimStruct), offset=0)), ("pwszPolicyDisplayText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pLogoImage", SimTypePointer(SimTypeRef("SPC_IMAGE", SimStruct), offset=0)), ("pLogoLink", SimTypePointer(SimTypeRef("SPC_LINK", SimStruct), offset=0)),)), name="SPC_SP_AGENCY_INFO", pack=False, align=None),
    "SPC_SP_OPUS_INFO": SimStruct(OrderedDict((("pwszProgramName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pMoreInfo", SimTypePointer(SimTypeRef("SPC_LINK", SimStruct), offset=0)), ("pPublisherInfo", SimTypePointer(SimTypeRef("SPC_LINK", SimStruct), offset=0)),)), name="SPC_SP_OPUS_INFO", pack=False, align=None),
    "SPC_STATEMENT_TYPE": SimStruct(OrderedDict((("cKeyPurposeId", SimTypeInt(signed=False, label="UInt32")), ("rgpszKeyPurposeId", SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)),)), name="SPC_STATEMENT_TYPE", pack=False, align=None),
    "SPDATAKEYLOCATION": SimTypeInt(signed=False, label="SPDATAKEYLOCATION"),
    "SPDISPLAYATTRIBUTES": SimTypeInt(signed=False, label="SPDISPLAYATTRIBUTES"),
    "SPDISPLAYPHRASE": SimStruct(OrderedDict((("ulNumTokens", SimTypeInt(signed=False, label="UInt32")), ("pTokens", SimTypePointer(SimTypeRef("SPDISPLAYTOKEN", SimStruct), offset=0)),)), name="SPDISPLAYPHRASE", pack=False, align=None),
    "SPDISPLAYTOKEN": SimStruct(OrderedDict((("pszLexical", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDisplay", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bDisplayAttributes", SimTypeChar(label="Byte")),)), name="SPDISPLAYTOKEN", pack=False, align=None),
    "SPEAKFLAGS": SimTypeInt(signed=False, label="SPEAKFLAGS"),
    "SPECIALGROUPCOLLAPSESTATES": SimTypeInt(signed=False, label="SPECIALGROUPCOLLAPSESTATES"),
    "SPECIALGROUPEXPANDSTATES": SimTypeInt(signed=False, label="SPECIALGROUPEXPANDSTATES"),
    "SPENDSRSTREAMFLAGS": SimTypeInt(signed=False, label="SPENDSRSTREAMFLAGS"),
    "SPEVENT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")), ("ulStreamNum", SimTypeInt(signed=False, label="UInt32")), ("ullAudioStreamOffset", SimTypeLongLong(signed=False, label="UInt64")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SPEVENT", pack=False, align=None),
    "SPEVENTENUM": SimTypeInt(signed=False, label="SPEVENTENUM"),
    "SPEVENTEX": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")), ("ulStreamNum", SimTypeInt(signed=False, label="UInt32")), ("ullAudioStreamOffset", SimTypeLongLong(signed=False, label="UInt64")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ullAudioTimeOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="SPEVENTEX", pack=False, align=None),
    "SPEVENTLPARAMTYPE": SimTypeInt(signed=False, label="SPEVENTLPARAMTYPE"),
    "SPEVENTSOURCEINFO": SimStruct(OrderedDict((("ullEventInterest", SimTypeLongLong(signed=False, label="UInt64")), ("ullQueuedInterest", SimTypeLongLong(signed=False, label="UInt64")), ("ulCount", SimTypeInt(signed=False, label="UInt32")),)), name="SPEVENTSOURCEINFO", pack=False, align=None),
    "SPFILEMODE": SimTypeInt(signed=False, label="SPFILEMODE"),
    "SPGRAMMARHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPGRAMMAROPTIONS": SimTypeInt(signed=False, label="SPGRAMMAROPTIONS"),
    "SPGRAMMARSTATE": SimTypeInt(signed=False, label="SPGRAMMARSTATE"),
    "SPGRAMMARWORDTYPE": SimTypeInt(signed=False, label="SPGRAMMARWORDTYPE"),
    "SPINPARTS": SimTypeInt(signed=False, label="SPINPARTS"),
    "SPINTERFERENCE": SimTypeInt(signed=False, label="SPINTERFERENCE"),
    "SPLCLIENT_INFO_1": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwBuildNum", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("wProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")),)), name="SPLCLIENT_INFO_1", pack=False, align=None),
    "SPLCLIENT_INFO_2_W2K": SimStruct(OrderedDict((("hSplPrinter", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SPLCLIENT_INFO_2_W2K", pack=False, align=None),
    "SPLCLIENT_INFO_2_WINXP": SimStruct(OrderedDict((("hSplPrinter", SimTypeInt(signed=False, label="UInt32")),)), name="SPLCLIENT_INFO_2_WINXP", pack=False, align=None),
    "SPLCLIENT_INFO_3_VISTA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwBuildNum", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("wProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("hSplPrinter", SimTypeLongLong(signed=False, label="UInt64")),)), name="SPLCLIENT_INFO_3_VISTA", pack=False, align=None),
    "SPLCLIENT_INFO_INTERNAL": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("pMachineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwBuildNum", SimTypeInt(signed=False, label="UInt32")), ("dwMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("dwMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("wProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("hSplPrinter", SimTypeLongLong(signed=False, label="UInt64")), ("dwProcessId", SimTypeInt(signed=False, label="UInt32")), ("dwSessionId", SimTypeInt(signed=False, label="UInt32")),)), name="SPLCLIENT_INFO_INTERNAL", pack=False, align=None),
    "SPLEXICONTYPE": SimTypeInt(signed=False, label="SPLEXICONTYPE"),
    "SPLITTERSTATES": SimTypeInt(signed=False, label="SPLITTERSTATES"),
    "SPLITTERVERTSTATES": SimTypeInt(signed=False, label="SPLITTERVERTSTATES"),
    "SPLOADOPTIONS": SimTypeInt(signed=False, label="SPLOADOPTIONS"),
    "SPMATCHINGMODE": SimTypeInt(signed=False, label="SPMATCHINGMODE"),
    "SPNORMALIZATIONLIST": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("ppszzNormalizedList", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)),)), name="SPNORMALIZATIONLIST", pack=False, align=None),
    "SPNOTIFYCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["wParam", "lParam"]), offset=0),
    "SPPARSEINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hRule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ullAudioStreamPosition", SimTypeLongLong(signed=False, label="UInt64")), ("ulAudioSize", SimTypeInt(signed=False, label="UInt32")), ("cTransitions", SimTypeInt(signed=False, label="UInt32")), ("pPath", SimTypePointer(SimTypeRef("SPPATHENTRY", SimStruct), offset=0)), ("SREngineID", SimTypeBottom(label="Guid")), ("ulSREnginePrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("pSREnginePrivateData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("fHypothesis", SimTypeInt(signed=True, label="Int32")),)), name="SPPARSEINFO", pack=False, align=None),
    "SPPARTOFSPEECH": SimTypeInt(signed=False, label="SPPARTOFSPEECH"),
    "SPPATHENTRY": SimStruct(OrderedDict((("hTransition", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("elem", SimTypeRef("SPPHRASEELEMENT", SimStruct)),)), name="SPPATHENTRY", pack=False, align=None),
    "SPPHRASE": SimStruct(OrderedDict((("Base", SimTypeRef("SPPHRASE_50", SimStruct)), ("pSML", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSemanticErrorInfo", SimTypePointer(SimTypeRef("SPSEMANTICERRORINFO", SimStruct), offset=0)),)), name="SPPHRASE", pack=False, align=None),
    "SPPHRASEALT": SimStruct(OrderedDict((("pPhrase", SimTypeBottom(label="ISpPhraseBuilder")), ("ulStartElementInParent", SimTypeInt(signed=False, label="UInt32")), ("cElementsInParent", SimTypeInt(signed=False, label="UInt32")), ("cElementsInAlternate", SimTypeInt(signed=False, label="UInt32")), ("pvAltExtra", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbAltExtra", SimTypeInt(signed=False, label="UInt32")),)), name="SPPHRASEALT", pack=False, align=None),
    "SPPHRASEALTREQUEST": SimStruct(OrderedDict((("ulStartElement", SimTypeInt(signed=False, label="UInt32")), ("cElements", SimTypeInt(signed=False, label="UInt32")), ("ulRequestAltCount", SimTypeInt(signed=False, label="UInt32")), ("pvResultExtra", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbResultExtra", SimTypeInt(signed=False, label="UInt32")), ("pPhrase", SimTypeBottom(label="ISpPhrase")), ("pRecoContext", SimTypeBottom(label="ISpRecoContext")),)), name="SPPHRASEALTREQUEST", pack=False, align=None),
    "SPPHRASEELEMENT": SimStruct(OrderedDict((("ulAudioTimeOffset", SimTypeInt(signed=False, label="UInt32")), ("ulAudioSizeTime", SimTypeInt(signed=False, label="UInt32")), ("ulAudioStreamOffset", SimTypeInt(signed=False, label="UInt32")), ("ulAudioSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("ulRetainedStreamOffset", SimTypeInt(signed=False, label="UInt32")), ("ulRetainedSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("pszDisplayText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLexicalForm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPronunciation", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("bDisplayAttributes", SimTypeChar(label="Byte")), ("RequiredConfidence", SimTypeChar(label="SByte")), ("ActualConfidence", SimTypeChar(label="SByte")), ("Reserved", SimTypeChar(label="Byte")), ("SREngineConfidence", SimTypeFloat(size=32)),)), name="SPPHRASEELEMENT", pack=False, align=None),
    "SPPHRASEPROPERTY": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"ulId": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("bType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("usArrayIndex", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("pszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("vValue", SimTypeRef("VARIANT", SimStruct)), ("ulFirstElement", SimTypeInt(signed=False, label="UInt32")), ("ulCountOfElements", SimTypeInt(signed=False, label="UInt32")), ("pNextSibling", SimTypePointer(SimTypeRef("SPPHRASEPROPERTY", SimStruct), offset=0)), ("pFirstChild", SimTypePointer(SimTypeRef("SPPHRASEPROPERTY", SimStruct), offset=0)), ("SREngineConfidence", SimTypeFloat(size=32)), ("Confidence", SimTypeChar(label="SByte")),)), name="SPPHRASEPROPERTY", pack=False, align=None),
    "SPPHRASEPROPERTYHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPPHRASEPROPERTYUNIONTYPE": SimTypeInt(signed=False, label="SPPHRASEPROPERTYUNIONTYPE"),
    "SPPHRASEREPLACEMENT": SimStruct(OrderedDict((("bDisplayAttributes", SimTypeChar(label="Byte")), ("pszReplacementText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFirstElement", SimTypeInt(signed=False, label="UInt32")), ("ulCountOfElements", SimTypeInt(signed=False, label="UInt32")),)), name="SPPHRASEREPLACEMENT", pack=False, align=None),
    "SPPHRASERNG": SimTypeInt(signed=False, label="SPPHRASERNG"),
    "SPPHRASERULE": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulId", SimTypeInt(signed=False, label="UInt32")), ("ulFirstElement", SimTypeInt(signed=False, label="UInt32")), ("ulCountOfElements", SimTypeInt(signed=False, label="UInt32")), ("pNextSibling", SimTypePointer(SimTypeRef("SPPHRASERULE", SimStruct), offset=0)), ("pFirstChild", SimTypePointer(SimTypeRef("SPPHRASERULE", SimStruct), offset=0)), ("SREngineConfidence", SimTypeFloat(size=32)), ("Confidence", SimTypeChar(label="SByte")),)), name="SPPHRASERULE", pack=False, align=None),
    "SPPHRASERULEHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPPHRASE_50": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("LangID", SimTypeShort(signed=False, label="UInt16")), ("wHomophoneGroupId", SimTypeShort(signed=False, label="UInt16")), ("ullGrammarID", SimTypeLongLong(signed=False, label="UInt64")), ("ftStartTime", SimTypeLongLong(signed=False, label="UInt64")), ("ullAudioStreamPosition", SimTypeLongLong(signed=False, label="UInt64")), ("ulAudioSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("ulRetainedSizeBytes", SimTypeInt(signed=False, label="UInt32")), ("ulAudioSizeTime", SimTypeInt(signed=False, label="UInt32")), ("Rule", SimTypeRef("SPPHRASERULE", SimStruct)), ("pProperties", SimTypePointer(SimTypeRef("SPPHRASEPROPERTY", SimStruct), offset=0)), ("pElements", SimTypePointer(SimTypeRef("SPPHRASEELEMENT", SimStruct), offset=0)), ("cReplacements", SimTypeInt(signed=False, label="UInt32")), ("pReplacements", SimTypePointer(SimTypeRef("SPPHRASEREPLACEMENT", SimStruct), offset=0)), ("SREngineID", SimTypeBottom(label="Guid")), ("ulSREnginePrivateDataSize", SimTypeInt(signed=False, label="UInt32")), ("pSREnginePrivateData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SPPHRASE_50", pack=False, align=None),
    "SPPRONUNCIATIONFLAGS": SimTypeInt(signed=False, label="SPPRONUNCIATIONFLAGS"),
    "SPPROPERTYINFO": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulId", SimTypeInt(signed=False, label="UInt32")), ("pszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("vValue", SimTypeRef("VARIANT", SimStruct)),)), name="SPPROPERTYINFO", pack=False, align=None),
    "SPPROPSRC": SimTypeInt(signed=False, label="SPPROPSRC"),
    "SPRECOCONTEXTHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPRECOCONTEXTSTATUS": SimStruct(OrderedDict((("eInterference", SimTypeInt(signed=False, label="SPINTERFERENCE")), ("szRequestTypeOfUI", SimTypeArray(SimTypeChar(label="Char"), 255)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="SPRECOCONTEXTSTATUS", pack=False, align=None),
    "SPRECOEVENTFLAGS": SimTypeInt(signed=False, label="SPRECOEVENTFLAGS"),
    "SPRECOGNIZERSTATUS": SimStruct(OrderedDict((("AudioStatus", SimTypeRef("SPAUDIOSTATUS", SimStruct)), ("ullRecognitionStreamPos", SimTypeLongLong(signed=False, label="UInt64")), ("ulStreamNumber", SimTypeInt(signed=False, label="UInt32")), ("ulNumActive", SimTypeInt(signed=False, label="UInt32")), ("clsidEngine", SimTypeBottom(label="Guid")), ("cLangIDs", SimTypeInt(signed=False, label="UInt32")), ("aLangID", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 20)), ("ullRecognitionStreamTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="SPRECOGNIZERSTATUS", pack=False, align=None),
    "SPRECORESULTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("eResultType", SimTypeInt(signed=False, label="SPRESULTTYPE")), ("fHypothesis", SimTypeInt(signed=True, label="Int32")), ("fProprietaryAutoPause", SimTypeInt(signed=True, label="Int32")), ("ullStreamPosStart", SimTypeLongLong(signed=False, label="UInt64")), ("ullStreamPosEnd", SimTypeLongLong(signed=False, label="UInt64")), ("hGrammar", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ulSizeEngineData", SimTypeInt(signed=False, label="UInt32")), ("pvEngineData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pPhrase", SimTypeBottom(label="ISpPhraseBuilder")), ("aPhraseAlts", SimTypePointer(SimTypeRef("SPPHRASEALT", SimStruct), offset=0)), ("ulNumAlts", SimTypeInt(signed=False, label="UInt32")),)), name="SPRECORESULTINFO", pack=False, align=None),
    "SPRECORESULTINFOEX": SimStruct(OrderedDict((("Base", SimTypeRef("SPRECORESULTINFO", SimStruct)), ("ullStreamTimeStart", SimTypeLongLong(signed=False, label="UInt64")), ("ullStreamTimeEnd", SimTypeLongLong(signed=False, label="UInt64")),)), name="SPRECORESULTINFOEX", pack=False, align=None),
    "SPRECORESULTTIMES": SimStruct(OrderedDict((("ftStreamTime", SimTypeRef("FILETIME", SimStruct)), ("ullLength", SimTypeLongLong(signed=False, label="UInt64")), ("dwTickCount", SimTypeInt(signed=False, label="UInt32")), ("ullStart", SimTypeLongLong(signed=False, label="UInt64")),)), name="SPRECORESULTTIMES", pack=False, align=None),
    "SPRECOSTATE": SimTypeInt(signed=False, label="SPRECOSTATE"),
    "SPRESULTTYPE": SimTypeInt(signed=False, label="SPRESULTTYPE"),
    "SPRULE": SimStruct(OrderedDict((("pszRuleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulRuleId", SimTypeInt(signed=False, label="UInt32")), ("dwAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="SPRULE", pack=False, align=None),
    "SPRULEENTRY": SimStruct(OrderedDict((("hRule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInitialState", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("pvClientRuleContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pvClientGrammarContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SPRULEENTRY", pack=False, align=None),
    "SPRULEHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPRULEINFOOPT": SimTypeInt(signed=False, label="SPRULEINFOOPT"),
    "SPRULESTATE": SimTypeInt(signed=False, label="SPRULESTATE"),
    "SPRUNSTATE": SimTypeInt(signed=False, label="SPRUNSTATE"),
    "SPSEMANTICERRORINFO": SimStruct(OrderedDict((("ulLineNumber", SimTypeInt(signed=False, label="UInt32")), ("pszScriptLine", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hrResultCode", SimTypeInt(signed=True, label="Int32")),)), name="SPSEMANTICERRORINFO", pack=False, align=None),
    "SPSEMANTICFORMAT": SimTypeInt(signed=False, label="SPSEMANTICFORMAT"),
    "SPSERIALIZEDEVENT": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")), ("ulStreamNum", SimTypeInt(signed=False, label="UInt32")), ("ullAudioStreamOffset", SimTypeLongLong(signed=False, label="UInt64")), ("SerializedwParam", SimTypeInt(signed=False, label="UInt32")), ("SerializedlParam", SimTypeInt(signed=True, label="Int32")),)), name="SPSERIALIZEDEVENT", pack=False, align=None),
    "SPSERIALIZEDEVENT64": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=True, label="Int32")), ("ulStreamNum", SimTypeInt(signed=False, label="UInt32")), ("ullAudioStreamOffset", SimTypeLongLong(signed=False, label="UInt64")), ("SerializedwParam", SimTypeLongLong(signed=False, label="UInt64")), ("SerializedlParam", SimTypeLongLong(signed=True, label="Int64")),)), name="SPSERIALIZEDEVENT64", pack=False, align=None),
    "SPSERIALIZEDPHRASE": SimStruct(OrderedDict((("ulSerializedSize", SimTypeInt(signed=False, label="UInt32")),)), name="SPSERIALIZEDPHRASE", pack=False, align=None),
    "SPSERIALIZEDRESULT": SimStruct(OrderedDict((("ulSerializedSize", SimTypeInt(signed=False, label="UInt32")),)), name="SPSERIALIZEDRESULT", pack=False, align=None),
    "SPSHORTCUTPAIR": SimStruct(OrderedDict((("pNextSHORTCUTPAIR", SimTypePointer(SimTypeRef("SPSHORTCUTPAIR", SimStruct), offset=0)), ("LangID", SimTypeShort(signed=False, label="UInt16")), ("shType", SimTypeInt(signed=False, label="SPSHORTCUTTYPE")), ("pszDisplay", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszSpoken", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SPSHORTCUTPAIR", pack=False, align=None),
    "SPSHORTCUTPAIRLIST": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("pvBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pFirstShortcutPair", SimTypePointer(SimTypeRef("SPSHORTCUTPAIR", SimStruct), offset=0)),)), name="SPSHORTCUTPAIRLIST", pack=False, align=None),
    "SPSHORTCUTTYPE": SimTypeInt(signed=False, label="SPSHORTCUTTYPE"),
    "SPSTATEHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPSTATEINFO": SimStruct(OrderedDict((("cAllocatedEntries", SimTypeInt(signed=False, label="UInt32")), ("pTransitions", SimTypePointer(SimTypeRef("SPTRANSITIONENTRY", SimStruct), offset=0)), ("cEpsilons", SimTypeInt(signed=False, label="UInt32")), ("cRules", SimTypeInt(signed=False, label="UInt32")), ("cWords", SimTypeInt(signed=False, label="UInt32")), ("cSpecialTransitions", SimTypeInt(signed=False, label="UInt32")),)), name="SPSTATEINFO", pack=False, align=None),
    "SPSTREAMFORMAT": SimTypeInt(signed=False, label="SPSTREAMFORMAT"),
    "SPSTREAMFORMATTYPE": SimTypeInt(signed=False, label="SPSTREAMFORMATTYPE"),
    "SPSVCINST_FLAGS": SimTypeInt(signed=False, label="SPSVCINST_FLAGS"),
    "SPTEXT": SimTypeInt(signed=False, label="SPTEXT"),
    "SPTEXTSELECTIONINFO": SimStruct(OrderedDict((("ulStartActiveOffset", SimTypeInt(signed=False, label="UInt32")), ("cchActiveChars", SimTypeInt(signed=False, label="UInt32")), ("ulStartSelection", SimTypeInt(signed=False, label="UInt32")), ("cchSelection", SimTypeInt(signed=False, label="UInt32")),)), name="SPTEXTSELECTIONINFO", pack=False, align=None),
    "SPTMTHREADINFO": SimStruct(OrderedDict((("lPoolSize", SimTypeInt(signed=True, label="Int32")), ("lPriority", SimTypeInt(signed=True, label="Int32")), ("ulConcurrencyLimit", SimTypeInt(signed=False, label="UInt32")), ("ulMaxQuickAllocThreads", SimTypeInt(signed=False, label="UInt32")),)), name="SPTMTHREADINFO", pack=False, align=None),
    "SPTRANSITIONENTRY": SimStruct(OrderedDict((("ID", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hNextState", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Type", SimTypeChar(label="Byte")), ("RequiredConfidence", SimTypeChar(label="Byte")), ("Anonymous1", SimStruct(OrderedDict((("fHasProperty", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous1_e__Struct", pack=False, align=None)), ("Weight", SimTypeFloat(size=32)), ("Anonymous2", SimUnion({"Anonymous1": SimStruct(OrderedDict((("hRuleInitialState", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hRule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pvClientRuleContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("hWord", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pvClientWordContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct(OrderedDict((("pvGrammarCookie", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="SPTRANSITIONENTRY", pack=False, align=None),
    "SPTRANSITIONID": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPTRANSITIONPROPERTY": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulId", SimTypeInt(signed=False, label="UInt32")), ("pszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("vValue", SimTypeRef("VARIANT", SimStruct)),)), name="SPTRANSITIONPROPERTY", pack=False, align=None),
    "SPTRANSITIONTYPE": SimTypeInt(signed=False, label="SPTRANSITIONTYPE"),
    "SPVACTIONS": SimTypeInt(signed=False, label="SPVACTIONS"),
    "SPVALUETYPE": SimTypeInt(signed=False, label="SPVALUETYPE"),
    "SPVCONTEXT": SimStruct(OrderedDict((("pCategory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pBefore", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pAfter", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SPVCONTEXT", pack=False, align=None),
    "SPVESACTIONS": SimTypeInt(signed=False, label="SPVESACTIONS"),
    "SPVFEATURE": SimTypeInt(signed=False, label="SPVFEATURE"),
    "SPVISEMES": SimTypeInt(signed=False, label="SPVISEMES"),
    "SPVLIMITS": SimTypeInt(signed=False, label="SPVLIMITS"),
    "SPVOICESTATUS": SimStruct(OrderedDict((("ulCurrentStream", SimTypeInt(signed=False, label="UInt32")), ("ulLastStreamQueued", SimTypeInt(signed=False, label="UInt32")), ("hrLastResult", SimTypeInt(signed=True, label="Int32")), ("dwRunningState", SimTypeInt(signed=False, label="UInt32")), ("ulInputWordPos", SimTypeInt(signed=False, label="UInt32")), ("ulInputWordLen", SimTypeInt(signed=False, label="UInt32")), ("ulInputSentPos", SimTypeInt(signed=False, label="UInt32")), ("ulInputSentLen", SimTypeInt(signed=False, label="UInt32")), ("lBookmarkId", SimTypeInt(signed=True, label="Int32")), ("PhonemeId", SimTypeShort(signed=False, label="UInt16")), ("VisemeId", SimTypeInt(signed=False, label="SPVISEMES")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="SPVOICESTATUS", pack=False, align=None),
    "SPVPITCH": SimStruct(OrderedDict((("MiddleAdj", SimTypeInt(signed=True, label="Int32")), ("RangeAdj", SimTypeInt(signed=True, label="Int32")),)), name="SPVPITCH", pack=False, align=None),
    "SPVPRIORITY": SimTypeInt(signed=False, label="SPVPRIORITY"),
    "SPVSKIPTYPE": SimTypeInt(signed=False, label="SPVSKIPTYPE"),
    "SPVSTATE": SimStruct(OrderedDict((("eAction", SimTypeInt(signed=False, label="SPVACTIONS")), ("LangID", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("EmphAdj", SimTypeInt(signed=True, label="Int32")), ("RateAdj", SimTypeInt(signed=True, label="Int32")), ("Volume", SimTypeInt(signed=False, label="UInt32")), ("PitchAdj", SimTypeRef("SPVPITCH", SimStruct)), ("SilenceMSecs", SimTypeInt(signed=False, label="UInt32")), ("pPhoneIds", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ePartOfSpeech", SimTypeInt(signed=False, label="SPPARTOFSPEECH")), ("Context", SimTypeRef("SPVCONTEXT", SimStruct)),)), name="SPVSTATE", pack=False, align=None),
    "SPVTEXTFRAG": SimStruct(OrderedDict((("pNext", SimTypePointer(SimTypeRef("SPVTEXTFRAG", SimStruct), offset=0)), ("State", SimTypeRef("SPVSTATE", SimStruct)), ("pTextStart", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulTextLen", SimTypeInt(signed=False, label="UInt32")), ("ulTextSrcOffset", SimTypeInt(signed=False, label="UInt32")),)), name="SPVTEXTFRAG", pack=False, align=None),
    "SPWORD": SimStruct(OrderedDict((("pNextWord", SimTypePointer(SimTypeRef("SPWORD", SimStruct), offset=0)), ("LangID", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("eWordType", SimTypeInt(signed=False, label="SPWORDTYPE")), ("pszWord", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pFirstWordPronunciation", SimTypePointer(SimTypeRef("SPWORDPRONUNCIATION", SimStruct), offset=0)),)), name="SPWORD", pack=False, align=None),
    "SPWORDENTRY": SimStruct(OrderedDict((("hWord", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("LangID", SimTypeShort(signed=False, label="UInt16")), ("pszDisplayText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLexicalForm", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("aPhoneId", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("pvClientContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SPWORDENTRY", pack=False, align=None),
    "SPWORDHANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SPWORDINFOOPT": SimTypeInt(signed=False, label="SPWORDINFOOPT"),
    "SPWORDLIST": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("pvBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pFirstWord", SimTypePointer(SimTypeRef("SPWORD", SimStruct), offset=0)),)), name="SPWORDLIST", pack=False, align=None),
    "SPWORDPRONOUNCEABLE": SimTypeInt(signed=False, label="SPWORDPRONOUNCEABLE"),
    "SPWORDPRONUNCIATION": SimStruct(OrderedDict((("pNextWordPronunciation", SimTypePointer(SimTypeRef("SPWORDPRONUNCIATION", SimStruct), offset=0)), ("eLexiconType", SimTypeInt(signed=False, label="SPLEXICONTYPE")), ("LangID", SimTypeShort(signed=False, label="UInt16")), ("wPronunciationFlags", SimTypeShort(signed=False, label="UInt16")), ("ePartOfSpeech", SimTypeInt(signed=False, label="SPPARTOFSPEECH")), ("szPronunciation", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SPWORDPRONUNCIATION", pack=False, align=None),
    "SPWORDPRONUNCIATIONLIST": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("pvBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pFirstWordPronunciation", SimTypePointer(SimTypeRef("SPWORDPRONUNCIATION", SimStruct), offset=0)),)), name="SPWORDPRONUNCIATIONLIST", pack=False, align=None),
    "SPWORDTYPE": SimTypeInt(signed=False, label="SPWORDTYPE"),
    "SPXMLRESULTOPTIONS": SimTypeInt(signed=False, label="SPXMLRESULTOPTIONS"),
    "SP_ALTPLATFORM_INFO_V1": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Platform", SimTypeInt(signed=False, label="VER_PLATFORM")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="SP_ALTPLATFORM_INFO_V1", pack=False, align=None),
    "SP_ALTPLATFORM_INFO_V2": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Platform", SimTypeInt(signed=False, label="VER_PLATFORM")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ProcessorArchitecture", SimTypeInt(signed=False, label="PROCESSOR_ARCHITECTURE")), ("Anonymous", SimUnion({"Reserved": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("FirstValidatedMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("FirstValidatedMinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="SP_ALTPLATFORM_INFO_V2", pack=False, align=None),
    "SP_ALTPLATFORM_INFO_V3": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Platform", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ProcessorArchitecture", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"Reserved": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("FirstValidatedMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("FirstValidatedMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ProductType", SimTypeChar(label="Byte")), ("SuiteMask", SimTypeShort(signed=False, label="UInt16")), ("BuildNumber", SimTypeInt(signed=False, label="UInt32")),)), name="SP_ALTPLATFORM_INFO_V3", pack=False, align=None),
    "SP_BACKUP_QUEUE_PARAMS_V1_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("FullInfPath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("FilenameOffset", SimTypeInt(signed=True, label="Int32")),)), name="SP_BACKUP_QUEUE_PARAMS_V1_A", pack=False, align=None),
    "SP_BACKUP_QUEUE_PARAMS_V1_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("FullInfPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("FilenameOffset", SimTypeInt(signed=True, label="Int32")),)), name="SP_BACKUP_QUEUE_PARAMS_V1_W", pack=False, align=None),
    "SP_BACKUP_QUEUE_PARAMS_V2_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("FullInfPath", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("FilenameOffset", SimTypeInt(signed=True, label="Int32")), ("ReinstallInstance", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="SP_BACKUP_QUEUE_PARAMS_V2_A", pack=False, align=None),
    "SP_BACKUP_QUEUE_PARAMS_V2_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("FullInfPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("FilenameOffset", SimTypeInt(signed=True, label="Int32")), ("ReinstallInstance", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SP_BACKUP_QUEUE_PARAMS_V2_W", pack=False, align=None),
    "SP_CLASSIMAGELIST_DATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ImageList", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SP_CLASSIMAGELIST_DATA", pack=False, align=None),
    "SP_CLASSINSTALL_HEADER": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("InstallFunction", SimTypeInt(signed=False, label="DI_FUNCTION")),)), name="SP_CLASSINSTALL_HEADER", pack=False, align=None),
    "SP_COPY_STYLE": SimTypeInt(signed=False, label="SP_COPY_STYLE"),
    "SP_DETECTDEVICE_PARAMS": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("DetectProgressNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ProgressNotifyParam", "DetectComplete"]), offset=0)), ("ProgressNotifyParam", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SP_DETECTDEVICE_PARAMS", pack=False, align=None),
    "SP_DEVICE_INTERFACE_DATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("InterfaceClassGuid", SimTypeBottom(label="Guid")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SP_DEVICE_INTERFACE_DATA", pack=False, align=None),
    "SP_DEVICE_INTERFACE_DETAIL_DATA_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("DevicePath", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SP_DEVICE_INTERFACE_DETAIL_DATA_A", pack=False, align=None),
    "SP_DEVICE_INTERFACE_DETAIL_DATA_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("DevicePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SP_DEVICE_INTERFACE_DETAIL_DATA_W", pack=False, align=None),
    "SP_DEVINFO_DATA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ClassGuid", SimTypeBottom(label="Guid")), ("DevInst", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SP_DEVINFO_DATA", pack=False, align=None),
    "SP_DEVINFO_LIST_DETAIL_DATA_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ClassGuid", SimTypeBottom(label="Guid")), ("RemoteMachineHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RemoteMachineName", SimTypeArray(SimTypeChar(label="SByte"), 263)),)), name="SP_DEVINFO_LIST_DETAIL_DATA_A", pack=False, align=None),
    "SP_DEVINFO_LIST_DETAIL_DATA_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ClassGuid", SimTypeBottom(label="Guid")), ("RemoteMachineHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("RemoteMachineName", SimTypeArray(SimTypeChar(label="Char"), 263)),)), name="SP_DEVINFO_LIST_DETAIL_DATA_W", pack=False, align=None),
    "SP_DEVINSTALL_PARAMS_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SETUP_DI_DEVICE_INSTALL_FLAGS")), ("FlagsEx", SimTypeInt(signed=False, label="SETUP_DI_DEVICE_INSTALL_FLAGS_EX")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InstallMsgHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Notification", "Param1", "Param2"]), offset=0)), ("InstallMsgHandlerContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileQueue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ClassInstallReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DriverPath", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="SP_DEVINSTALL_PARAMS_A", pack=False, align=None),
    "SP_DEVINSTALL_PARAMS_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SETUP_DI_DEVICE_INSTALL_FLAGS")), ("FlagsEx", SimTypeInt(signed=False, label="SETUP_DI_DEVICE_INSTALL_FLAGS_EX")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InstallMsgHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Context", "Notification", "Param1", "Param2"]), offset=0)), ("InstallMsgHandlerContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileQueue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ClassInstallReserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DriverPath", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SP_DEVINSTALL_PARAMS_W", pack=False, align=None),
    "SP_DRVINFO_DATA_V1_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("DriverType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Description", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("MfgName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("ProviderName", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="SP_DRVINFO_DATA_V1_A", pack=False, align=None),
    "SP_DRVINFO_DATA_V1_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("DriverType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 256)), ("MfgName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ProviderName", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="SP_DRVINFO_DATA_V1_W", pack=False, align=None),
    "SP_DRVINFO_DATA_V2_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("DriverType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Description", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("MfgName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("ProviderName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("DriverDate", SimTypeRef("FILETIME", SimStruct)), ("DriverVersion", SimTypeLongLong(signed=False, label="UInt64")),)), name="SP_DRVINFO_DATA_V2_A", pack=False, align=None),
    "SP_DRVINFO_DATA_V2_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("DriverType", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 256)), ("MfgName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ProviderName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("DriverDate", SimTypeRef("FILETIME", SimStruct)), ("DriverVersion", SimTypeLongLong(signed=False, label="UInt64")),)), name="SP_DRVINFO_DATA_V2_W", pack=False, align=None),
    "SP_DRVINFO_DETAIL_DATA_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("InfDate", SimTypeRef("FILETIME", SimStruct)), ("CompatIDsOffset", SimTypeInt(signed=False, label="UInt32")), ("CompatIDsLength", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SectionName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("InfFileName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("DrvDescription", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("HardwareID", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SP_DRVINFO_DETAIL_DATA_A", pack=False, align=None),
    "SP_DRVINFO_DETAIL_DATA_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("InfDate", SimTypeRef("FILETIME", SimStruct)), ("CompatIDsOffset", SimTypeInt(signed=False, label="UInt32")), ("CompatIDsLength", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SectionName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("InfFileName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DrvDescription", SimTypeArray(SimTypeChar(label="Char"), 256)), ("HardwareID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SP_DRVINFO_DETAIL_DATA_W", pack=False, align=None),
    "SP_DRVINSTALL_PARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("Rank", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SETUP_DI_DRIVER_INSTALL_FLAGS")), ("PrivateData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="SP_DRVINSTALL_PARAMS", pack=False, align=None),
    "SP_ENABLECLASS_PARAMS": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("ClassGuid", SimTypeBottom(label="Guid")), ("EnableMessage", SimTypeInt(signed=False, label="UInt32")),)), name="SP_ENABLECLASS_PARAMS", pack=False, align=None),
    "SP_FILE_COPY_PARAMS_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("QueueHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SourceRootPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SourcePath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SourceFilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SourceDescription", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SourceTagfile", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TargetDirectory", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TargetFilename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("CopyStyle", SimTypeInt(signed=False, label="UInt32")), ("LayoutInf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SP_FILE_COPY_PARAMS_A", pack=False, align=None),
    "SP_FILE_COPY_PARAMS_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("QueueHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SourceRootPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourcePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourceFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourceDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourceTagfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TargetDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TargetFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("CopyStyle", SimTypeInt(signed=False, label="UInt32")), ("LayoutInf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityDescriptor", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SP_FILE_COPY_PARAMS_W", pack=False, align=None),
    "SP_INF_INFORMATION": SimStruct(OrderedDict((("InfStyle", SimTypeInt(signed=False, label="INF_STYLE")), ("InfCount", SimTypeInt(signed=False, label="UInt32")), ("VersionData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SP_INF_INFORMATION", pack=False, align=None),
    "SP_INF_SIGNER_INFO_V1_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("CatalogFile", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("DigitalSigner", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("DigitalSignerVersion", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="SP_INF_SIGNER_INFO_V1_A", pack=False, align=None),
    "SP_INF_SIGNER_INFO_V1_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("CatalogFile", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DigitalSigner", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DigitalSignerVersion", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SP_INF_SIGNER_INFO_V1_W", pack=False, align=None),
    "SP_INF_SIGNER_INFO_V2_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("CatalogFile", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("DigitalSigner", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("DigitalSignerVersion", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("SignerScore", SimTypeInt(signed=False, label="UInt32")),)), name="SP_INF_SIGNER_INFO_V2_A", pack=False, align=None),
    "SP_INF_SIGNER_INFO_V2_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("CatalogFile", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DigitalSigner", SimTypeArray(SimTypeChar(label="Char"), 260)), ("DigitalSignerVersion", SimTypeArray(SimTypeChar(label="Char"), 260)), ("SignerScore", SimTypeInt(signed=False, label="UInt32")),)), name="SP_INF_SIGNER_INFO_V2_W", pack=False, align=None),
    "SP_INSTALLWIZARD_DATA": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DynamicPages", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 20)), ("NumDynamicPages", SimTypeInt(signed=False, label="UInt32")), ("DynamicPageFlags", SimTypeInt(signed=False, label="UInt32")), ("PrivateFlags", SimTypeInt(signed=False, label="UInt32")), ("PrivateData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndWizardDlg", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SP_INSTALLWIZARD_DATA", pack=False, align=None),
    "SP_NEWDEVICEWIZARD_DATA": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DynamicPages", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 20)), ("NumDynamicPages", SimTypeInt(signed=False, label="UInt32")), ("hwndWizardDlg", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SP_NEWDEVICEWIZARD_DATA", pack=False, align=None),
    "SP_ORIGINAL_FILE_INFO_A": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("OriginalInfName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("OriginalCatalogName", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="SP_ORIGINAL_FILE_INFO_A", pack=False, align=None),
    "SP_ORIGINAL_FILE_INFO_W": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("OriginalInfName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("OriginalCatalogName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SP_ORIGINAL_FILE_INFO_W", pack=False, align=None),
    "SP_POWERMESSAGEWAKE_PARAMS_A": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("PowerMessageWake", SimTypeArray(SimTypeChar(label="SByte"), 512)),)), name="SP_POWERMESSAGEWAKE_PARAMS_A", pack=False, align=None),
    "SP_POWERMESSAGEWAKE_PARAMS_W": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("PowerMessageWake", SimTypeArray(SimTypeChar(label="Char"), 512)),)), name="SP_POWERMESSAGEWAKE_PARAMS_W", pack=False, align=None),
    "SP_PROPCHANGE_PARAMS": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("StateChange", SimTypeInt(signed=False, label="SETUP_DI_STATE_CHANGE")), ("Scope", SimTypeInt(signed=False, label="SETUP_DI_PROPERTY_CHANGE_SCOPE")), ("HwProfile", SimTypeInt(signed=False, label="UInt32")),)), name="SP_PROPCHANGE_PARAMS", pack=False, align=None),
    "SP_PROPSHEETPAGE_REQUEST": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("PageRequested", SimTypeInt(signed=False, label="UInt32")), ("DeviceInfoSet", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DeviceInfoData", SimTypePointer(SimTypeRef("SP_DEVINFO_DATA", SimStruct), offset=0)),)), name="SP_PROPSHEETPAGE_REQUEST", pack=False, align=None),
    "SP_REGISTER_CONTROL_STATUSA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("FailureCode", SimTypeInt(signed=False, label="UInt32")),)), name="SP_REGISTER_CONTROL_STATUSA", pack=False, align=None),
    "SP_REGISTER_CONTROL_STATUSW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Win32Error", SimTypeInt(signed=False, label="UInt32")), ("FailureCode", SimTypeInt(signed=False, label="UInt32")),)), name="SP_REGISTER_CONTROL_STATUSW", pack=False, align=None),
    "SP_REMOVEDEVICE_PARAMS": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("Scope", SimTypeInt(signed=False, label="SETUP_DI_REMOVE_DEVICE_SCOPE")), ("HwProfile", SimTypeInt(signed=False, label="UInt32")),)), name="SP_REMOVEDEVICE_PARAMS", pack=False, align=None),
    "SP_SELECTDEVICE_PARAMS_A": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("Title", SimTypeArray(SimTypeChar(label="SByte"), 60)), ("Instructions", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("ListLabel", SimTypeArray(SimTypeChar(label="SByte"), 30)), ("SubTitle", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="SP_SELECTDEVICE_PARAMS_A", pack=False, align=None),
    "SP_SELECTDEVICE_PARAMS_W": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("Title", SimTypeArray(SimTypeChar(label="Char"), 60)), ("Instructions", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ListLabel", SimTypeArray(SimTypeChar(label="Char"), 30)), ("SubTitle", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="SP_SELECTDEVICE_PARAMS_W", pack=False, align=None),
    "SP_TROUBLESHOOTER_PARAMS_A": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("ChmFile", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("HtmlTroubleShooter", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="SP_TROUBLESHOOTER_PARAMS_A", pack=False, align=None),
    "SP_TROUBLESHOOTER_PARAMS_W": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("ChmFile", SimTypeArray(SimTypeChar(label="Char"), 260)), ("HtmlTroubleShooter", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SP_TROUBLESHOOTER_PARAMS_W", pack=False, align=None),
    "SP_UNREMOVEDEVICE_PARAMS": SimStruct(OrderedDict((("ClassInstallHeader", SimTypeRef("SP_CLASSINSTALL_HEADER", SimStruct)), ("Scope", SimTypeInt(signed=False, label="UInt32")), ("HwProfile", SimTypeInt(signed=False, label="UInt32")),)), name="SP_UNREMOVEDEVICE_PARAMS", pack=False, align=None),
    "SPropAttrArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("aPropAttr", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="SPropAttrArray", pack=False, align=None),
    "SPropProblem": SimStruct(OrderedDict((("ulIndex", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("scode", SimTypeInt(signed=True, label="Int32")),)), name="SPropProblem", pack=False, align=None),
    "SPropProblemArray": SimStruct(OrderedDict((("cProblem", SimTypeInt(signed=False, label="UInt32")), ("aProblem", SimTypePointer(SimTypeRef("SPropProblem", SimStruct), offset=0)),)), name="SPropProblemArray", pack=False, align=None),
    "SPropTagArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("aulPropTag", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="SPropTagArray", pack=False, align=None),
    "SPropValue": SimStruct(OrderedDict((("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("dwAlignPad", SimTypeInt(signed=False, label="UInt32")), ("Value", SimUnion({"i": SimTypeShort(signed=True, label="Int16"), "l": SimTypeInt(signed=True, label="Int32"), "ul": SimTypeInt(signed=False, label="UInt32"), "flt": SimTypeFloat(size=32), "dbl": SimTypeFloat(size=64), "b": SimTypeShort(signed=False, label="UInt16"), "cur": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), "at": SimTypeFloat(size=64), "ft": SimTypeRef("FILETIME", SimStruct), "lpszA": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "bin": SimTypeRef("SBinary", SimStruct), "lpszW": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lpguid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "li": SimTypeLongLong(signed=True, label="Int64"), "MVi": SimTypeRef("SShortArray", SimStruct), "MVl": SimTypeRef("SLongArray", SimStruct), "MVflt": SimTypeRef("SRealArray", SimStruct), "MVdbl": SimTypeRef("SDoubleArray", SimStruct), "MVcur": SimTypeRef("SCurrencyArray", SimStruct), "MVat": SimTypeRef("SAppTimeArray", SimStruct), "MVft": SimTypeRef("SDateTimeArray", SimStruct), "MVbin": SimTypeRef("SBinaryArray", SimStruct), "MVszA": SimTypeRef("SLPSTRArray", SimStruct), "MVszW": SimTypeRef("SWStringArray", SimStruct), "MVguid": SimTypeRef("SGuidArray", SimStruct), "MVli": SimTypeRef("SLargeIntegerArray", SimStruct), "err": SimTypeInt(signed=True, label="Int32"), "x": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")),)), name="SPropValue", pack=False, align=None),
    "SPropertyRestriction": SimStruct(OrderedDict((("relop", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("lpProp", SimTypePointer(SimTypeRef("SPropValue", SimStruct), offset=0)),)), name="SPropertyRestriction", pack=False, align=None),
    "SQLINTERVAL": SimTypeInt(signed=False, label="SQLINTERVAL"),
    "SQLPERF": SimStruct(OrderedDict((("TimerResolution", SimTypeInt(signed=False, label="UInt32")), ("SQLidu", SimTypeInt(signed=False, label="UInt32")), ("SQLiduRows", SimTypeInt(signed=False, label="UInt32")), ("SQLSelects", SimTypeInt(signed=False, label="UInt32")), ("SQLSelectRows", SimTypeInt(signed=False, label="UInt32")), ("Transactions", SimTypeInt(signed=False, label="UInt32")), ("SQLPrepares", SimTypeInt(signed=False, label="UInt32")), ("ExecDirects", SimTypeInt(signed=False, label="UInt32")), ("SQLExecutes", SimTypeInt(signed=False, label="UInt32")), ("CursorOpens", SimTypeInt(signed=False, label="UInt32")), ("CursorSize", SimTypeInt(signed=False, label="UInt32")), ("CursorUsed", SimTypeInt(signed=False, label="UInt32")), ("PercentCursorUsed", SimTypeFloat(size=64)), ("AvgFetchTime", SimTypeFloat(size=64)), ("AvgCursorSize", SimTypeFloat(size=64)), ("AvgCursorUsed", SimTypeFloat(size=64)), ("SQLFetchTime", SimTypeInt(signed=False, label="UInt32")), ("SQLFetchCount", SimTypeInt(signed=False, label="UInt32")), ("CurrentStmtCount", SimTypeInt(signed=False, label="UInt32")), ("MaxOpenStmt", SimTypeInt(signed=False, label="UInt32")), ("SumOpenStmt", SimTypeInt(signed=False, label="UInt32")), ("CurrentConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("MaxConnectionsOpened", SimTypeInt(signed=False, label="UInt32")), ("SumConnectionsOpened", SimTypeInt(signed=False, label="UInt32")), ("SumConnectiontime", SimTypeInt(signed=False, label="UInt32")), ("AvgTimeOpened", SimTypeFloat(size=64)), ("ServerRndTrips", SimTypeInt(signed=False, label="UInt32")), ("BuffersSent", SimTypeInt(signed=False, label="UInt32")), ("BuffersRec", SimTypeInt(signed=False, label="UInt32")), ("BytesSent", SimTypeInt(signed=False, label="UInt32")), ("BytesRec", SimTypeInt(signed=False, label="UInt32")), ("msExecutionTime", SimTypeInt(signed=False, label="UInt32")), ("msNetWorkServerTime", SimTypeInt(signed=False, label="UInt32")),)), name="SQLPERF", pack=False, align=None),
    "SQLVARENUM": SimTypeInt(signed=False, label="SQLVARENUM"),
    "SQL_ASYNC_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeShort(signed=True, label="Int16"), arg_names=["pContext", "fLast"]), offset=0),
    "SQL_DAY_SECOND_STRUCT": SimStruct(OrderedDict((("day", SimTypeInt(signed=False, label="UInt32")), ("hour", SimTypeInt(signed=False, label="UInt32")), ("minute", SimTypeInt(signed=False, label="UInt32")), ("second", SimTypeInt(signed=False, label="UInt32")), ("fraction", SimTypeInt(signed=False, label="UInt32")),)), name="SQL_DAY_SECOND_STRUCT", pack=False, align=None),
    "SQL_INTERVAL_STRUCT": SimStruct(OrderedDict((("interval_type", SimTypeInt(signed=False, label="SQLINTERVAL")), ("interval_sign", SimTypeShort(signed=True, label="Int16")), ("intval", SimUnion({"year_month": SimTypeRef("SQL_YEAR_MONTH_STRUCT", SimStruct), "day_second": SimTypeRef("SQL_DAY_SECOND_STRUCT", SimStruct)}, name="<anon>", label="None")),)), name="SQL_INTERVAL_STRUCT", pack=False, align=None),
    "SQL_NUMERIC_STRUCT": SimStruct(OrderedDict((("precision", SimTypeChar(label="Byte")), ("scale", SimTypeChar(label="SByte")), ("sign", SimTypeChar(label="Byte")), ("val", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SQL_NUMERIC_STRUCT", pack=False, align=None),
    "SQL_YEAR_MONTH_STRUCT": SimStruct(OrderedDict((("year", SimTypeInt(signed=False, label="UInt32")), ("month", SimTypeInt(signed=False, label="UInt32")),)), name="SQL_YEAR_MONTH_STRUCT", pack=False, align=None),
    "SRB_IO_CONTROL": SimStruct(OrderedDict((("HeaderLength", SimTypeInt(signed=False, label="UInt32")), ("Signature", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("ControlCode", SimTypeInt(signed=False, label="UInt32")), ("ReturnCode", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="SRB_IO_CONTROL", pack=False, align=None),
    "SRCCODEINFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("Obj", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("FileName", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="SRCCODEINFO", pack=False, align=None),
    "SRCCODEINFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("Obj", SimTypeArray(SimTypeChar(label="Char"), 261)), ("FileName", SimTypeArray(SimTypeChar(label="Char"), 261)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="SRCCODEINFOW", pack=False, align=None),
    "SRPHOSTING_TYPE": SimTypeInt(signed=False, label="SRPHOSTING_TYPE"),
    "SRPHOSTING_VERSION": SimTypeInt(signed=False, label="SRPHOSTING_VERSION"),
    "SRV_INSTANCE_TYPE": SimTypeInt(signed=False, label="SRV_INSTANCE_TYPE"),
    "SRV_OPEN_ECP_CONTEXT": SimStruct(OrderedDict((("ShareName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("SocketAddress", SimTypePointer(SimTypeRef("SOCKADDR_STORAGE", SimStruct), offset=0)), ("OplockBlockState", SimTypeChar(label="Byte")), ("OplockAppState", SimTypeChar(label="Byte")), ("OplockFinalState", SimTypeChar(label="Byte")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("InstanceType", SimTypeInt(signed=False, label="SRV_INSTANCE_TYPE")),)), name="SRV_OPEN_ECP_CONTEXT", pack=False, align=None),
    "SRWLOCK": SimStruct(OrderedDict((("Ptr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SRWLOCK", pack=False, align=None),
    "SR_DISK_REPLICATION_ELIGIBLE": SimTypeInt(signed=False, label="SR_DISK_REPLICATION_ELIGIBLE"),
    "SR_REPLICATED_DISK_TYPE": SimTypeInt(signed=False, label="SR_REPLICATED_DISK_TYPE"),
    "SR_RESOURCE_TYPE_ADD_REPLICATION_GROUP": SimStruct(OrderedDict((("ReplicationGroupName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 260)), ("LogPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("MaxLogSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("LogType", SimTypeShort(signed=False, label="UInt16")), ("ReplicationMode", SimTypeInt(signed=False, label="UInt32")), ("MinimumPartnersInSync", SimTypeInt(signed=False, label="UInt32")), ("EnableWriteConsistency", SimTypeChar(label="Byte")), ("EnableEncryption", SimTypeChar(label="Byte")), ("EnableCompression", SimTypeChar(label="Byte")), ("CertificateThumbprint", SimTypeArray(SimTypeChar(label="Char"), 260)), ("VolumeNameCount", SimTypeInt(signed=False, label="UInt32")), ("VolumeNames", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SR_RESOURCE_TYPE_ADD_REPLICATION_GROUP", pack=False, align=None),
    "SR_RESOURCE_TYPE_ADD_REPLICATION_GROUP_RESULT": SimStruct(OrderedDict((("Result", SimTypeInt(signed=False, label="UInt32")), ("ErrorString", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SR_RESOURCE_TYPE_ADD_REPLICATION_GROUP_RESULT", pack=False, align=None),
    "SR_RESOURCE_TYPE_DISK_INFO": SimStruct(OrderedDict((("Reason", SimTypeInt(signed=False, label="SR_DISK_REPLICATION_ELIGIBLE")), ("DiskGuid", SimTypeBottom(label="Guid")),)), name="SR_RESOURCE_TYPE_DISK_INFO", pack=False, align=None),
    "SR_RESOURCE_TYPE_ELIGIBLE_DISKS_RESULT": SimStruct(OrderedDict((("Count", SimTypeShort(signed=False, label="UInt16")), ("DiskInfo", SimTypePointer(SimTypeRef("SR_RESOURCE_TYPE_DISK_INFO", SimStruct), offset=0)),)), name="SR_RESOURCE_TYPE_ELIGIBLE_DISKS_RESULT", pack=False, align=None),
    "SR_RESOURCE_TYPE_QUERY_ELIGIBLE_LOGDISKS": SimStruct(OrderedDict((("DataDiskGuid", SimTypeBottom(label="Guid")), ("IncludeOfflineDisks", SimTypeChar(label="Byte")),)), name="SR_RESOURCE_TYPE_QUERY_ELIGIBLE_LOGDISKS", pack=False, align=None),
    "SR_RESOURCE_TYPE_QUERY_ELIGIBLE_SOURCE_DATADISKS": SimStruct(OrderedDict((("DataDiskGuid", SimTypeBottom(label="Guid")), ("IncludeAvailableStoargeDisks", SimTypeChar(label="Byte")),)), name="SR_RESOURCE_TYPE_QUERY_ELIGIBLE_SOURCE_DATADISKS", pack=False, align=None),
    "SR_RESOURCE_TYPE_QUERY_ELIGIBLE_TARGET_DATADISKS": SimStruct(OrderedDict((("SourceDataDiskGuid", SimTypeBottom(label="Guid")), ("TargetReplicationGroupGuid", SimTypeBottom(label="Guid")), ("SkipConnectivityCheck", SimTypeChar(label="Byte")), ("IncludeOfflineDisks", SimTypeChar(label="Byte")),)), name="SR_RESOURCE_TYPE_QUERY_ELIGIBLE_TARGET_DATADISKS", pack=False, align=None),
    "SR_RESOURCE_TYPE_REPLICATED_DISK": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="SR_REPLICATED_DISK_TYPE")), ("ClusterDiskResourceGuid", SimTypeBottom(label="Guid")), ("ReplicationGroupId", SimTypeBottom(label="Guid")), ("ReplicationGroupName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="SR_RESOURCE_TYPE_REPLICATED_DISK", pack=False, align=None),
    "SR_RESOURCE_TYPE_REPLICATED_DISKS_RESULT": SimStruct(OrderedDict((("Count", SimTypeShort(signed=False, label="UInt16")), ("ReplicatedDisks", SimTypePointer(SimTypeRef("SR_RESOURCE_TYPE_REPLICATED_DISK", SimStruct), offset=0)),)), name="SR_RESOURCE_TYPE_REPLICATED_DISKS_RESULT", pack=False, align=None),
    "SR_RESOURCE_TYPE_REPLICATED_PARTITION_ARRAY": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("PartitionArray", SimTypePointer(SimTypeRef("SR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO", SimStruct), offset=0)),)), name="SR_RESOURCE_TYPE_REPLICATED_PARTITION_ARRAY", pack=False, align=None),
    "SR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO": SimStruct(OrderedDict((("PartitionOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="SR_RESOURCE_TYPE_REPLICATED_PARTITION_INFO", pack=False, align=None),
    "SR_SECURITY_DESCRIPTOR": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SR_SECURITY_DESCRIPTOR", pack=False, align=None),
    "SRealArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpflt", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="SRealArray", pack=False, align=None),
    "SRestriction": SimStruct(OrderedDict((("rt", SimTypeInt(signed=False, label="UInt32")), ("res", SimUnion({"resCompareProps": SimTypeRef("SComparePropsRestriction", SimStruct), "resAnd": SimTypeRef("SAndRestriction", SimStruct), "resOr": SimTypeRef("SOrRestriction", SimStruct), "resNot": SimTypeRef("SNotRestriction", SimStruct), "resContent": SimTypeRef("SContentRestriction", SimStruct), "resProperty": SimTypeRef("SPropertyRestriction", SimStruct), "resBitMask": SimTypeRef("SBitMaskRestriction", SimStruct), "resSize": SimTypeRef("SSizeRestriction", SimStruct), "resExist": SimTypeRef("SExistRestriction", SimStruct), "resSub": SimTypeRef("SSubRestriction", SimStruct), "resComment": SimTypeRef("SCommentRestriction", SimStruct)}, name="<anon>", label="None")),)), name="SRestriction", pack=False, align=None),
    "SRow": SimStruct(OrderedDict((("ulAdrEntryPad", SimTypeInt(signed=False, label="UInt32")), ("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpProps", SimTypePointer(SimTypeRef("SPropValue", SimStruct), offset=0)),)), name="SRow", pack=False, align=None),
    "SRowSet": SimStruct(OrderedDict((("cRows", SimTypeInt(signed=False, label="UInt32")), ("aRow", SimTypePointer(SimTypeRef("SRow", SimStruct), offset=0)),)), name="SRowSet", pack=False, align=None),
    "SSERRORINFO": SimStruct(OrderedDict((("pwszMessage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszProcedure", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lNative", SimTypeInt(signed=True, label="Int32")), ("bState", SimTypeChar(label="Byte")), ("bClass", SimTypeChar(label="Byte")), ("wLineNumber", SimTypeShort(signed=False, label="UInt16")),)), name="SSERRORINFO", pack=False, align=None),
    "SSF_MASK": SimTypeInt(signed=False, label="SSF_MASK"),
    "SSL_CRACK_CERTIFICATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeRef("X509Certificate", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pbCertificate", "cbCertificate", "VerifySignature", "ppCertificate"]), offset=0),
    "SSL_CREDENTIAL_CERTIFICATE": SimStruct(OrderedDict((("cbPrivateKey", SimTypeInt(signed=False, label="UInt32")), ("pPrivateKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCertificate", SimTypeInt(signed=False, label="UInt32")), ("pCertificate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SSL_CREDENTIAL_CERTIFICATE", pack=False, align=None),
    "SSL_ECCKEY_BLOB": SimStruct(OrderedDict((("dwCurveType", SimTypeInt(signed=False, label="UInt32")), ("cbKey", SimTypeInt(signed=False, label="UInt32")),)), name="SSL_ECCKEY_BLOB", pack=False, align=None),
    "SSL_EMPTY_CACHE_FN_A": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszTargetName", "dwFlags"]), offset=0),
    "SSL_EMPTY_CACHE_FN_W": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszTargetName", "dwFlags"]), offset=0),
    "SSL_F12_EXTRA_CERT_CHAIN_POLICY_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwErrorLevel", SimTypeInt(signed=False, label="UInt32")), ("dwErrorCategory", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("wszErrorText", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="SSL_F12_EXTRA_CERT_CHAIN_POLICY_STATUS", pack=False, align=None),
    "SSL_FREE_CERTIFICATE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("X509Certificate", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pCertificate"]), offset=0),
    "SSL_HPKP_HEADER_EXTRA_CERT_CHAIN_POLICY_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pwszServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rgpszHpkpValue", SimTypeArray(SimTypePointer(SimTypeChar(label="Byte"), offset=0), 2)),)), name="SSL_HPKP_HEADER_EXTRA_CERT_CHAIN_POLICY_PARA", pack=False, align=None),
    "SSL_KEY_PIN_EXTRA_CERT_CHAIN_POLICY_PARA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pwszServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SSL_KEY_PIN_EXTRA_CERT_CHAIN_POLICY_PARA", pack=False, align=None),
    "SSL_KEY_PIN_EXTRA_CERT_CHAIN_POLICY_STATUS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("lError", SimTypeInt(signed=True, label="Int32")), ("wszErrorText", SimTypeArray(SimTypeChar(label="Char"), 512)),)), name="SSL_KEY_PIN_EXTRA_CERT_CHAIN_POLICY_STATUS", pack=False, align=None),
    "SSTP_CERT_INFO": SimStruct(OrderedDict((("isDefault", SimTypeInt(signed=True, label="Int32")), ("certBlob", SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct)),)), name="SSTP_CERT_INFO", pack=False, align=None),
    "SSTP_CONFIG_PARAMS": SimStruct(OrderedDict((("dwNumPorts", SimTypeInt(signed=False, label="UInt32")), ("dwPortFlags", SimTypeInt(signed=False, label="UInt32")), ("isUseHttps", SimTypeInt(signed=True, label="Int32")), ("certAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("sstpCertDetails", SimTypeRef("SSTP_CERT_INFO", SimStruct)),)), name="SSTP_CONFIG_PARAMS", pack=False, align=None),
    "SSUPDATE_TYPE": SimTypeInt(signed=False, label="SSUPDATE_TYPE"),
    "SSVARIANT": SimStruct(OrderedDict((("vt", SimTypeShort(signed=False, label="UInt16")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"bTinyIntVal": SimTypeChar(label="Byte"), "sShortIntVal": SimTypeShort(signed=True, label="Int16"), "lIntVal": SimTypeInt(signed=True, label="Int32"), "llBigIntVal": SimTypeLongLong(signed=True, label="Int64"), "fltRealVal": SimTypeFloat(size=32), "dblFloatVal": SimTypeFloat(size=64), "cyMoneyVal": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), "NCharVal": SimTypeRef("_NCharVal", SimStruct), "CharVal": SimTypeRef("_CharVal", SimStruct), "fBitVal": SimTypeShort(signed=True, label="Int16"), "rgbGuidVal": SimTypeArray(SimTypeChar(label="Byte"), 16), "numNumericVal": SimTypeRef("DB_NUMERIC", SimStruct), "BinaryVal": SimTypeRef("_BinaryVal", SimStruct), "tsDateTimeVal": SimTypeRef("DBTIMESTAMP", SimStruct), "UnknownType": SimTypeRef("_UnknownType", SimStruct), "BLOBType": SimTypeRef("_BLOBType", SimStruct)}, name="<anon>", label="None")),)), name="SSVARIANT", pack=False, align=None),
    "SShortArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpi", SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)),)), name="SShortArray", pack=False, align=None),
    "SSizeRestriction": SimStruct(OrderedDict((("relop", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("cb", SimTypeInt(signed=False, label="UInt32")),)), name="SSizeRestriction", pack=False, align=None),
    "SSortOrder": SimStruct(OrderedDict((("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("ulOrder", SimTypeInt(signed=False, label="UInt32")),)), name="SSortOrder", pack=False, align=None),
    "SSortOrderSet": SimStruct(OrderedDict((("cSorts", SimTypeInt(signed=False, label="UInt32")), ("cCategories", SimTypeInt(signed=False, label="UInt32")), ("cExpanded", SimTypeInt(signed=False, label="UInt32")), ("aSort", SimTypePointer(SimTypeRef("SSortOrder", SimStruct), offset=0)),)), name="SSortOrderSet", pack=False, align=None),
    "SSubRestriction": SimStruct(OrderedDict((("ulSubObject", SimTypeInt(signed=False, label="UInt32")), ("lpRes", SimTypePointer(SimTypeRef("SRestriction", SimStruct), offset=0)),)), name="SSubRestriction", pack=False, align=None),
    "STACKFRAME": SimStruct(OrderedDict((("AddrPC", SimTypeRef("ADDRESS", SimStruct)), ("AddrReturn", SimTypeRef("ADDRESS", SimStruct)), ("AddrFrame", SimTypeRef("ADDRESS", SimStruct)), ("AddrStack", SimTypeRef("ADDRESS", SimStruct)), ("FuncTableEntry", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Params", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("Far", SimTypeInt(signed=True, label="Int32")), ("Virtual", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("KdHelp", SimTypeRef("KDHELP", SimStruct)), ("AddrBStore", SimTypeRef("ADDRESS", SimStruct)),)), name="STACKFRAME", pack=False, align=None),
    "STACKFRAME64": SimStruct(OrderedDict((("AddrPC", SimTypeRef("ADDRESS64", SimStruct)), ("AddrReturn", SimTypeRef("ADDRESS64", SimStruct)), ("AddrFrame", SimTypeRef("ADDRESS64", SimStruct)), ("AddrStack", SimTypeRef("ADDRESS64", SimStruct)), ("AddrBStore", SimTypeRef("ADDRESS64", SimStruct)), ("FuncTableEntry", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Params", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("Far", SimTypeInt(signed=True, label="Int32")), ("Virtual", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 3)), ("KdHelp", SimTypeRef("KDHELP64", SimStruct)),)), name="STACKFRAME64", pack=False, align=None),
    "STACKFRAME_EX": SimStruct(OrderedDict((("AddrPC", SimTypeRef("ADDRESS64", SimStruct)), ("AddrReturn", SimTypeRef("ADDRESS64", SimStruct)), ("AddrFrame", SimTypeRef("ADDRESS64", SimStruct)), ("AddrStack", SimTypeRef("ADDRESS64", SimStruct)), ("AddrBStore", SimTypeRef("ADDRESS64", SimStruct)), ("FuncTableEntry", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Params", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("Far", SimTypeInt(signed=True, label="Int32")), ("Virtual", SimTypeInt(signed=True, label="Int32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 3)), ("KdHelp", SimTypeRef("KDHELP64", SimStruct)), ("StackFrameSize", SimTypeInt(signed=False, label="UInt32")), ("InlineFrameContext", SimTypeInt(signed=False, label="UInt32")),)), name="STACKFRAME_EX", pack=False, align=None),
    "STACK_SRC_INFO": SimStruct(OrderedDict((("ImagePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ModuleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Function", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Displacement", SimTypeInt(signed=False, label="UInt32")), ("Row", SimTypeInt(signed=False, label="UInt32")), ("Column", SimTypeInt(signed=False, label="UInt32")),)), name="STACK_SRC_INFO", pack=False, align=None),
    "STACK_SYM_FRAME_INFO": SimStruct(OrderedDict((("StackFrameEx", SimTypeRef("DEBUG_STACK_FRAME_EX", SimStruct)), ("SrcInfo", SimTypeRef("STACK_SRC_INFO", SimStruct)),)), name="STACK_SYM_FRAME_INFO", pack=False, align=None),
    "STANDARDSTATES": SimTypeInt(signed=False, label="STANDARDSTATES"),
    "STARTING_LCN_INPUT_BUFFER": SimStruct(OrderedDict((("StartingLcn", SimTypeLongLong(signed=True, label="Int64")),)), name="STARTING_LCN_INPUT_BUFFER", pack=False, align=None),
    "STARTING_LCN_INPUT_BUFFER_EX": SimStruct(OrderedDict((("StartingLcn", SimTypeLongLong(signed=True, label="Int64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="STARTING_LCN_INPUT_BUFFER_EX", pack=False, align=None),
    "STARTING_VCN_INPUT_BUFFER": SimStruct(OrderedDict((("StartingVcn", SimTypeLongLong(signed=True, label="Int64")),)), name="STARTING_VCN_INPUT_BUFFER", pack=False, align=None),
    "STARTPANELPARTS": SimTypeInt(signed=False, label="STARTPANELPARTS"),
    "STARTUPINFOA": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("lpReserved", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpDesktop", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpTitle", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwXSize", SimTypeInt(signed=False, label="UInt32")), ("dwYSize", SimTypeInt(signed=False, label="UInt32")), ("dwXCountChars", SimTypeInt(signed=False, label="UInt32")), ("dwYCountChars", SimTypeInt(signed=False, label="UInt32")), ("dwFillAttribute", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="STARTUPINFOW_FLAGS")), ("wShowWindow", SimTypeShort(signed=False, label="UInt16")), ("cbReserved2", SimTypeShort(signed=False, label="UInt16")), ("lpReserved2", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hStdInput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hStdOutput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hStdError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="STARTUPINFOA", pack=False, align=None),
    "STARTUPINFOEXA": SimStruct(OrderedDict((("StartupInfo", SimTypeRef("STARTUPINFOA", SimStruct)), ("lpAttributeList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="STARTUPINFOEXA", pack=False, align=None),
    "STARTUPINFOEXW": SimStruct(OrderedDict((("StartupInfo", SimTypeRef("STARTUPINFOW", SimStruct)), ("lpAttributeList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="STARTUPINFOEXW", pack=False, align=None),
    "STARTUPINFOW": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("lpReserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDesktop", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwX", SimTypeInt(signed=False, label="UInt32")), ("dwY", SimTypeInt(signed=False, label="UInt32")), ("dwXSize", SimTypeInt(signed=False, label="UInt32")), ("dwYSize", SimTypeInt(signed=False, label="UInt32")), ("dwXCountChars", SimTypeInt(signed=False, label="UInt32")), ("dwYCountChars", SimTypeInt(signed=False, label="UInt32")), ("dwFillAttribute", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="STARTUPINFOW_FLAGS")), ("wShowWindow", SimTypeShort(signed=False, label="UInt16")), ("cbReserved2", SimTypeShort(signed=False, label="UInt16")), ("lpReserved2", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hStdInput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hStdOutput", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hStdError", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="STARTUPINFOW", pack=False, align=None),
    "STARTUPINFOW_FLAGS": SimTypeInt(signed=False, label="STARTUPINFOW_FLAGS"),
    "STARTUP_FLAGS": SimTypeInt(signed=False, label="STARTUP_FLAGS"),
    "STATDATA": SimStruct(OrderedDict((("formatetc", SimTypeRef("FORMATETC", SimStruct)), ("advf", SimTypeInt(signed=False, label="UInt32")), ("pAdvSink", SimTypeBottom(label="IAdviseSink")), ("dwConnection", SimTypeInt(signed=False, label="UInt32")),)), name="STATDATA", pack=False, align=None),
    "STATEMGRSTATUS": SimStruct(OrderedDict((("nStatus", SimTypeInt(signed=False, label="WIN32_ERROR")), ("llSequenceNumber", SimTypeLongLong(signed=True, label="Int64")),)), name="STATEMGRSTATUS", pack=False, align=None),
    "STATE_LOCATION_TYPE": SimTypeInt(signed=False, label="STATE_LOCATION_TYPE"),
    "STATFLAG": SimTypeInt(signed=False, label="STATFLAG"),
    "STATICPARTS": SimTypeInt(signed=False, label="STATICPARTS"),
    "STATIC_STYLES": SimTypeInt(signed=False, label="STATIC_STYLES"),
    "STATPROPSETSTG": SimStruct(OrderedDict((("fmtid", SimTypeBottom(label="Guid")), ("clsid", SimTypeBottom(label="Guid")), ("grfFlags", SimTypeInt(signed=False, label="UInt32")), ("mtime", SimTypeRef("FILETIME", SimStruct)), ("ctime", SimTypeRef("FILETIME", SimStruct)), ("atime", SimTypeRef("FILETIME", SimStruct)), ("dwOSVersion", SimTypeInt(signed=False, label="UInt32")),)), name="STATPROPSETSTG", pack=False, align=None),
    "STATPROPSTG": SimStruct(OrderedDict((("lpwstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("propid", SimTypeInt(signed=False, label="UInt32")), ("vt", SimTypeInt(signed=False, label="VARENUM")),)), name="STATPROPSTG", pack=False, align=None),
    "STATSTG": SimStruct(OrderedDict((("pwcsName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("type", SimTypeInt(signed=False, label="UInt32")), ("cbSize", SimTypeLongLong(signed=False, label="UInt64")), ("mtime", SimTypeRef("FILETIME", SimStruct)), ("ctime", SimTypeRef("FILETIME", SimStruct)), ("atime", SimTypeRef("FILETIME", SimStruct)), ("grfMode", SimTypeInt(signed=False, label="STGM")), ("grfLocksSupported", SimTypeInt(signed=False, label="UInt32")), ("clsid", SimTypeBottom(label="Guid")), ("grfStateBits", SimTypeInt(signed=False, label="UInt32")), ("reserved", SimTypeInt(signed=False, label="UInt32")),)), name="STATSTG", pack=False, align=None),
    "STATURL": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pwcsUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwcsTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftLastVisited", SimTypeRef("FILETIME", SimStruct)), ("ftLastUpdated", SimTypeRef("FILETIME", SimStruct)), ("ftExpires", SimTypeRef("FILETIME", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="STATURL", pack=False, align=None),
    "STATUSPARTS": SimTypeInt(signed=False, label="STATUSPARTS"),
    "STATUSTYPES": SimTypeInt(signed=False, label="STATUSTYPES"),
    "STATUS_OBJECT_NOTIFICATION": SimStruct(OrderedDict((("cbEntryID", SimTypeInt(signed=False, label="UInt32")), ("lpEntryID", SimTypePointer(SimTypeRef("ENTRYID", SimStruct), offset=0)), ("cValues", SimTypeInt(signed=False, label="UInt32")), ("lpPropVals", SimTypePointer(SimTypeRef("SPropValue", SimStruct), offset=0)),)), name="STATUS_OBJECT_NOTIFICATION", pack=False, align=None),
    "STAT_CHUNK": SimStruct(OrderedDict((("idChunk", SimTypeInt(signed=False, label="UInt32")), ("breakType", SimTypeInt(signed=False, label="CHUNK_BREAKTYPE")), ("flags", SimTypeInt(signed=False, label="CHUNKSTATE")), ("locale", SimTypeInt(signed=False, label="UInt32")), ("attribute", SimTypeRef("FULLPROPSPEC", SimStruct)), ("idChunkSource", SimTypeInt(signed=False, label="UInt32")), ("cwcStartSource", SimTypeInt(signed=False, label="UInt32")), ("cwcLenSource", SimTypeInt(signed=False, label="UInt32")),)), name="STAT_CHUNK", pack=False, align=None),
    "STAT_SERVER_0": SimStruct(OrderedDict((("sts0_start", SimTypeInt(signed=False, label="UInt32")), ("sts0_fopens", SimTypeInt(signed=False, label="UInt32")), ("sts0_devopens", SimTypeInt(signed=False, label="UInt32")), ("sts0_jobsqueued", SimTypeInt(signed=False, label="UInt32")), ("sts0_sopens", SimTypeInt(signed=False, label="UInt32")), ("sts0_stimedout", SimTypeInt(signed=False, label="UInt32")), ("sts0_serrorout", SimTypeInt(signed=False, label="UInt32")), ("sts0_pwerrors", SimTypeInt(signed=False, label="UInt32")), ("sts0_permerrors", SimTypeInt(signed=False, label="UInt32")), ("sts0_syserrors", SimTypeInt(signed=False, label="UInt32")), ("sts0_bytessent_low", SimTypeInt(signed=False, label="UInt32")), ("sts0_bytessent_high", SimTypeInt(signed=False, label="UInt32")), ("sts0_bytesrcvd_low", SimTypeInt(signed=False, label="UInt32")), ("sts0_bytesrcvd_high", SimTypeInt(signed=False, label="UInt32")), ("sts0_avresponse", SimTypeInt(signed=False, label="UInt32")), ("sts0_reqbufneed", SimTypeInt(signed=False, label="UInt32")), ("sts0_bigbufneed", SimTypeInt(signed=False, label="UInt32")),)), name="STAT_SERVER_0", pack=False, align=None),
    "STAT_WORKSTATION_0": SimStruct(OrderedDict((("StatisticsStartTime", SimTypeLongLong(signed=True, label="Int64")), ("BytesReceived", SimTypeLongLong(signed=True, label="Int64")), ("SmbsReceived", SimTypeLongLong(signed=True, label="Int64")), ("PagingReadBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("NonPagingReadBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("CacheReadBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("NetworkReadBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("BytesTransmitted", SimTypeLongLong(signed=True, label="Int64")), ("SmbsTransmitted", SimTypeLongLong(signed=True, label="Int64")), ("PagingWriteBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("NonPagingWriteBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("CacheWriteBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("NetworkWriteBytesRequested", SimTypeLongLong(signed=True, label="Int64")), ("InitiallyFailedOperations", SimTypeInt(signed=False, label="UInt32")), ("FailedCompletionOperations", SimTypeInt(signed=False, label="UInt32")), ("ReadOperations", SimTypeInt(signed=False, label="UInt32")), ("RandomReadOperations", SimTypeInt(signed=False, label="UInt32")), ("ReadSmbs", SimTypeInt(signed=False, label="UInt32")), ("LargeReadSmbs", SimTypeInt(signed=False, label="UInt32")), ("SmallReadSmbs", SimTypeInt(signed=False, label="UInt32")), ("WriteOperations", SimTypeInt(signed=False, label="UInt32")), ("RandomWriteOperations", SimTypeInt(signed=False, label="UInt32")), ("WriteSmbs", SimTypeInt(signed=False, label="UInt32")), ("LargeWriteSmbs", SimTypeInt(signed=False, label="UInt32")), ("SmallWriteSmbs", SimTypeInt(signed=False, label="UInt32")), ("RawReadsDenied", SimTypeInt(signed=False, label="UInt32")), ("RawWritesDenied", SimTypeInt(signed=False, label="UInt32")), ("NetworkErrors", SimTypeInt(signed=False, label="UInt32")), ("Sessions", SimTypeInt(signed=False, label="UInt32")), ("FailedSessions", SimTypeInt(signed=False, label="UInt32")), ("Reconnects", SimTypeInt(signed=False, label="UInt32")), ("CoreConnects", SimTypeInt(signed=False, label="UInt32")), ("Lanman20Connects", SimTypeInt(signed=False, label="UInt32")), ("Lanman21Connects", SimTypeInt(signed=False, label="UInt32")), ("LanmanNtConnects", SimTypeInt(signed=False, label="UInt32")), ("ServerDisconnects", SimTypeInt(signed=False, label="UInt32")), ("HungSessions", SimTypeInt(signed=False, label="UInt32")), ("UseCount", SimTypeInt(signed=False, label="UInt32")), ("FailedUseCount", SimTypeInt(signed=False, label="UInt32")), ("CurrentCommands", SimTypeInt(signed=False, label="UInt32")),)), name="STAT_WORKSTATION_0", pack=False, align=None),
    "STDMSHLFLAGS": SimTypeInt(signed=False, label="STDMSHLFLAGS"),
    "STD_ALERT": SimStruct(OrderedDict((("alrt_timestamp", SimTypeInt(signed=False, label="UInt32")), ("alrt_eventname", SimTypeArray(SimTypeChar(label="Char"), 17)), ("alrt_servicename", SimTypeArray(SimTypeChar(label="Char"), 81)),)), name="STD_ALERT", pack=False, align=None),
    "STD_HANDLE": SimTypeInt(signed=False, label="STD_HANDLE"),
    "STGC": SimTypeInt(signed=False, label="STGC"),
    "STGFMT": SimTypeInt(signed=False, label="STGFMT"),
    "STGM": SimTypeInt(signed=False, label="STGM"),
    "STGMEDIUM": SimStruct(OrderedDict((("tymed", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"hBitmap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hMetaFilePict": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "hEnhMetaFile": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hGlobal": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "lpszFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pstm": SimTypeBottom(label="IStream"), "pstg": SimTypeBottom(label="IStorage")}, name="<anon>", label="None")), ("pUnkForRelease", SimTypeBottom(label="IUnknown")),)), name="STGMEDIUM", pack=False, align=None),
    "STGMOVE": SimTypeInt(signed=False, label="STGMOVE"),
    "STGOP": SimTypeInt(signed=False, label="STGOP"),
    "STGOPTIONS": SimStruct(OrderedDict((("usVersion", SimTypeShort(signed=False, label="UInt16")), ("reserved", SimTypeShort(signed=False, label="UInt16")), ("ulSectorSize", SimTypeInt(signed=False, label="UInt32")), ("pwcsTemplateFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STGOPTIONS", pack=False, align=None),
    "STGTY": SimTypeInt(signed=False, label="STGTY"),
    "STICKYKEYS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="STICKYKEYS_FLAGS")),)), name="STICKYKEYS", pack=False, align=None),
    "STICKYKEYS_FLAGS": SimTypeInt(signed=False, label="STICKYKEYS_FLAGS"),
    "STINOTIFY": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("guidNotificationCode", SimTypeBottom(label="Guid")), ("abNotificationData", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="STINOTIFY", pack=False, align=None),
    "STISUBSCRIBE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwFilter", SimTypeInt(signed=False, label="UInt32")), ("hWndNotify", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hEvent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uiNotificationMessage", SimTypeInt(signed=False, label="UInt32")),)), name="STISUBSCRIBE", pack=False, align=None),
    "STI_DEVICE_INFORMATIONW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("szDeviceInternalName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("DeviceCapabilitiesA", SimTypeRef("STI_DEV_CAPS", SimStruct)), ("dwHardwareConfiguration", SimTypeInt(signed=False, label="UInt32")), ("pszVendorDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDeviceDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPortName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPropProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STI_DEVICE_INFORMATIONW", pack=False, align=None),
    "STI_DEVICE_MJ_TYPE": SimTypeInt(signed=False, label="STI_DEVICE_MJ_TYPE"),
    "STI_DEVICE_STATUS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("StatusMask", SimTypeInt(signed=False, label="UInt32")), ("dwOnlineState", SimTypeInt(signed=False, label="UInt32")), ("dwHardwareStatusCode", SimTypeInt(signed=False, label="UInt32")), ("dwEventHandlingState", SimTypeInt(signed=False, label="UInt32")), ("dwPollingInterval", SimTypeInt(signed=False, label="UInt32")),)), name="STI_DEVICE_STATUS", pack=False, align=None),
    "STI_DEV_CAPS": SimStruct(OrderedDict((("dwGeneric", SimTypeInt(signed=False, label="UInt32")),)), name="STI_DEV_CAPS", pack=False, align=None),
    "STI_DIAG": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwBasicDiagCode", SimTypeInt(signed=False, label="UInt32")), ("dwVendorDiagCode", SimTypeInt(signed=False, label="UInt32")), ("dwStatusMask", SimTypeInt(signed=False, label="UInt32")), ("sErrorInfo", SimTypeRef("_ERROR_INFOW", SimStruct)),)), name="STI_DIAG", pack=False, align=None),
    "STI_USD_CAPS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwGenericCaps", SimTypeInt(signed=False, label="UInt32")),)), name="STI_USD_CAPS", pack=False, align=None),
    "STI_WIA_DEVICE_INFORMATIONW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("szDeviceInternalName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("DeviceCapabilitiesA", SimTypeRef("STI_DEV_CAPS", SimStruct)), ("dwHardwareConfiguration", SimTypeInt(signed=False, label="UInt32")), ("pszVendorDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDeviceDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPortName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPropProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszLocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszUiDll", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STI_WIA_DEVICE_INFORMATIONW", pack=False, align=None),
    "STORAGE_ACCESS_ALIGNMENT_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("BytesPerCacheLine", SimTypeInt(signed=False, label="UInt32")), ("BytesOffsetForCacheAlignment", SimTypeInt(signed=False, label="UInt32")), ("BytesPerLogicalSector", SimTypeInt(signed=False, label="UInt32")), ("BytesPerPhysicalSector", SimTypeInt(signed=False, label="UInt32")), ("BytesOffsetForSectorAlignment", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_ACCESS_ALIGNMENT_DESCRIPTOR", pack=False, align=None),
    "STORAGE_ADAPTER_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("MaximumTransferLength", SimTypeInt(signed=False, label="UInt32")), ("MaximumPhysicalPages", SimTypeInt(signed=False, label="UInt32")), ("AlignmentMask", SimTypeInt(signed=False, label="UInt32")), ("AdapterUsesPio", SimTypeChar(label="Byte")), ("AdapterScansDown", SimTypeChar(label="Byte")), ("CommandQueueing", SimTypeChar(label="Byte")), ("AcceleratedTransfer", SimTypeChar(label="Byte")), ("BusType", SimTypeChar(label="Byte")), ("BusMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("BusMinorVersion", SimTypeShort(signed=False, label="UInt16")), ("SrbType", SimTypeChar(label="Byte")), ("AddressType", SimTypeChar(label="Byte")),)), name="STORAGE_ADAPTER_DESCRIPTOR", pack=False, align=None),
    "STORAGE_ADAPTER_SERIAL_NUMBER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SerialNumber", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="STORAGE_ADAPTER_SERIAL_NUMBER", pack=False, align=None),
    "STORAGE_ALLOCATE_BC_STREAM_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestsPerPeriod", SimTypeInt(signed=False, label="UInt32")), ("Period", SimTypeInt(signed=False, label="UInt32")), ("RetryFailures", SimTypeChar(label="Byte")), ("Discardable", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("AccessType", SimTypeInt(signed=False, label="UInt32")), ("AccessMode", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_ALLOCATE_BC_STREAM_INPUT", pack=False, align=None),
    "STORAGE_ALLOCATE_BC_STREAM_OUTPUT": SimStruct(OrderedDict((("RequestSize", SimTypeLongLong(signed=False, label="UInt64")), ("NumOutStandingRequests", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_ALLOCATE_BC_STREAM_OUTPUT", pack=False, align=None),
    "STORAGE_ASSOCIATION_TYPE": SimTypeInt(signed=False, label="STORAGE_ASSOCIATION_TYPE"),
    "STORAGE_ATTRIBUTE_MGMT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="STORAGE_ATTRIBUTE_MGMT_ACTION")), ("Attribute", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_ATTRIBUTE_MGMT", pack=False, align=None),
    "STORAGE_ATTRIBUTE_MGMT_ACTION": SimTypeInt(signed=False, label="STORAGE_ATTRIBUTE_MGMT_ACTION"),
    "STORAGE_BREAK_RESERVATION_REQUEST": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("_unused", SimTypeChar(label="Byte")), ("PathId", SimTypeChar(label="Byte")), ("TargetId", SimTypeChar(label="Byte")), ("Lun", SimTypeChar(label="Byte")),)), name="STORAGE_BREAK_RESERVATION_REQUEST", pack=False, align=None),
    "STORAGE_BUS_RESET_REQUEST": SimStruct(OrderedDict((("PathId", SimTypeChar(label="Byte")),)), name="STORAGE_BUS_RESET_REQUEST", pack=False, align=None),
    "STORAGE_BUS_TYPE": SimTypeInt(signed=False, label="STORAGE_BUS_TYPE"),
    "STORAGE_COMPONENT_HEALTH_STATUS": SimTypeInt(signed=False, label="STORAGE_COMPONENT_HEALTH_STATUS"),
    "STORAGE_COUNTER": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="STORAGE_COUNTER_TYPE")), ("Value", SimUnion({"ManufactureDate": SimTypeRef("_ManufactureDate_e__Struct", SimStruct), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="STORAGE_COUNTER", pack=False, align=None),
    "STORAGE_COUNTERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumberOfCounters", SimTypeInt(signed=False, label="UInt32")), ("Counters", SimTypePointer(SimTypeRef("STORAGE_COUNTER", SimStruct), offset=0)),)), name="STORAGE_COUNTERS", pack=False, align=None),
    "STORAGE_COUNTER_TYPE": SimTypeInt(signed=False, label="STORAGE_COUNTER_TYPE"),
    "STORAGE_CRYPTO_ALGORITHM_ID": SimTypeInt(signed=False, label="STORAGE_CRYPTO_ALGORITHM_ID"),
    "STORAGE_CRYPTO_CAPABILITY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("CryptoCapabilityIndex", SimTypeInt(signed=False, label="UInt32")), ("AlgorithmId", SimTypeInt(signed=False, label="STORAGE_CRYPTO_ALGORITHM_ID")), ("KeySize", SimTypeInt(signed=False, label="STORAGE_CRYPTO_KEY_SIZE")), ("DataUnitSizeBitmask", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_CRYPTO_CAPABILITY", pack=False, align=None),
    "STORAGE_CRYPTO_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumKeysSupported", SimTypeInt(signed=False, label="UInt32")), ("NumCryptoCapabilities", SimTypeInt(signed=False, label="UInt32")), ("CryptoCapabilities", SimTypePointer(SimTypeRef("STORAGE_CRYPTO_CAPABILITY", SimStruct), offset=0)),)), name="STORAGE_CRYPTO_DESCRIPTOR", pack=False, align=None),
    "STORAGE_CRYPTO_KEY_SIZE": SimTypeInt(signed=False, label="STORAGE_CRYPTO_KEY_SIZE"),
    "STORAGE_DEPENDENCY_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="STORAGE_DEPENDENCY_INFO_VERSION")), ("NumberEntries", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Version1Entries": SimTypePointer(SimTypeRef("STORAGE_DEPENDENCY_INFO_TYPE_1", SimStruct), offset=0), "Version2Entries": SimTypePointer(SimTypeRef("STORAGE_DEPENDENCY_INFO_TYPE_2", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="STORAGE_DEPENDENCY_INFO", pack=False, align=None),
    "STORAGE_DEPENDENCY_INFO_TYPE_1": SimStruct(OrderedDict((("DependencyTypeFlags", SimTypeInt(signed=False, label="DEPENDENT_DISK_FLAG")), ("ProviderSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("VirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)),)), name="STORAGE_DEPENDENCY_INFO_TYPE_1", pack=False, align=None),
    "STORAGE_DEPENDENCY_INFO_TYPE_2": SimStruct(OrderedDict((("DependencyTypeFlags", SimTypeInt(signed=False, label="DEPENDENT_DISK_FLAG")), ("ProviderSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("VirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("AncestorLevel", SimTypeInt(signed=False, label="UInt32")), ("DependencyDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HostVolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DependentVolumeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DependentVolumeRelativePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STORAGE_DEPENDENCY_INFO_TYPE_2", pack=False, align=None),
    "STORAGE_DEPENDENCY_INFO_VERSION": SimTypeInt(signed=False, label="STORAGE_DEPENDENCY_INFO_VERSION"),
    "STORAGE_DESCRIPTOR_HEADER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DESCRIPTOR_HEADER", pack=False, align=None),
    "STORAGE_DEVICE_ATTRIBUTES_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Attributes", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_DEVICE_ATTRIBUTES_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeChar(label="Byte")), ("DeviceTypeModifier", SimTypeChar(label="Byte")), ("RemovableMedia", SimTypeChar(label="Byte")), ("CommandQueueing", SimTypeChar(label="Byte")), ("VendorIdOffset", SimTypeInt(signed=False, label="UInt32")), ("ProductIdOffset", SimTypeInt(signed=False, label="UInt32")), ("ProductRevisionOffset", SimTypeInt(signed=False, label="UInt32")), ("SerialNumberOffset", SimTypeInt(signed=False, label="UInt32")), ("BusType", SimTypeInt(signed=False, label="STORAGE_BUS_TYPE")), ("RawPropertiesLength", SimTypeInt(signed=False, label="UInt32")), ("RawDeviceProperties", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_DEVICE_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_FAULT_DOMAIN_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumberOfFaultDomains", SimTypeInt(signed=False, label="UInt32")), ("FaultDomainIds", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="STORAGE_DEVICE_FAULT_DOMAIN_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_FORM_FACTOR": SimTypeInt(signed=False, label="STORAGE_DEVICE_FORM_FACTOR"),
    "STORAGE_DEVICE_ID_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumberOfIdentifiers", SimTypeInt(signed=False, label="UInt32")), ("Identifiers", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_DEVICE_ID_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_IO_CAPABILITY_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("LunMaxIoCount", SimTypeInt(signed=False, label="UInt32")), ("AdapterMaxIoCount", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_IO_CAPABILITY_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_LED_STATE_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_DEVICE_LED_STATE_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_LOCATION_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Location", SimTypeRef("DEVICE_LOCATION", SimStruct)), ("StringOffset", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_LOCATION_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_MANAGEMENT_STATUS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Health", SimTypeInt(signed=False, label="STORAGE_DISK_HEALTH_STATUS")), ("NumberOfOperationalStatus", SimTypeInt(signed=False, label="UInt32")), ("NumberOfAdditionalReasons", SimTypeInt(signed=False, label="UInt32")), ("OperationalStatus", SimTypeArray(SimTypeInt(signed=False, label="STORAGE_DISK_OPERATIONAL_STATUS"), 16)), ("AdditionalReasons", SimTypePointer(SimTypeRef("STORAGE_OPERATIONAL_REASON", SimStruct), offset=0)),)), name="STORAGE_DEVICE_MANAGEMENT_STATUS", pack=False, align=None),
    "STORAGE_DEVICE_NUMA_PROPERTY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumaNode", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_NUMA_PROPERTY", pack=False, align=None),
    "STORAGE_DEVICE_NUMBER": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_NUMBER", pack=False, align=None),
    "STORAGE_DEVICE_NUMBERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NumberOfDevices", SimTypeInt(signed=False, label="UInt32")), ("Devices", SimTypePointer(SimTypeRef("STORAGE_DEVICE_NUMBER", SimStruct), offset=0)),)), name="STORAGE_DEVICE_NUMBERS", pack=False, align=None),
    "STORAGE_DEVICE_NUMBER_EX": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="UInt32")), ("DeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceGuid", SimTypeBottom(label="Guid")), ("PartitionNumber", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_NUMBER_EX", pack=False, align=None),
    "STORAGE_DEVICE_POWER_CAP": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Units", SimTypeInt(signed=False, label="STORAGE_DEVICE_POWER_CAP_UNITS")), ("MaxPower", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_DEVICE_POWER_CAP", pack=False, align=None),
    "STORAGE_DEVICE_POWER_CAP_UNITS": SimTypeInt(signed=False, label="STORAGE_DEVICE_POWER_CAP_UNITS"),
    "STORAGE_DEVICE_RESILIENCY_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NameOffset", SimTypeInt(signed=False, label="UInt32")), ("NumberOfLogicalCopies", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPhysicalCopies", SimTypeInt(signed=False, label="UInt32")), ("PhysicalDiskRedundancy", SimTypeInt(signed=False, label="UInt32")), ("NumberOfColumns", SimTypeInt(signed=False, label="UInt32")), ("Interleave", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_RESILIENCY_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_SELF_ENCRYPTION_PROPERTY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SupportsSelfEncryption", SimTypeChar(label="Byte")),)), name="STORAGE_DEVICE_SELF_ENCRYPTION_PROPERTY", pack=False, align=None),
    "STORAGE_DEVICE_SELF_ENCRYPTION_PROPERTY_V2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SupportsSelfEncryption", SimTypeChar(label="Byte")), ("EncryptionType", SimTypeInt(signed=False, label="STORAGE_ENCRYPTION_TYPE")),)), name="STORAGE_DEVICE_SELF_ENCRYPTION_PROPERTY_V2", pack=False, align=None),
    "STORAGE_DEVICE_TIERING_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TotalNumberOfTiers", SimTypeInt(signed=False, label="UInt32")), ("NumberOfTiersReturned", SimTypeInt(signed=False, label="UInt32")), ("Tiers", SimTypePointer(SimTypeRef("STORAGE_TIER", SimStruct), offset=0)),)), name="STORAGE_DEVICE_TIERING_DESCRIPTOR", pack=False, align=None),
    "STORAGE_DEVICE_UNSAFE_SHUTDOWN_COUNT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("UnsafeShutdownCount", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_DEVICE_UNSAFE_SHUTDOWN_COUNT", pack=False, align=None),
    "STORAGE_DIAGNOSTIC_DATA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ProviderId", SimTypeBottom(label="Guid")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DiagnosticDataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_DIAGNOSTIC_DATA", pack=False, align=None),
    "STORAGE_DIAGNOSTIC_LEVEL": SimTypeInt(signed=False, label="STORAGE_DIAGNOSTIC_LEVEL"),
    "STORAGE_DIAGNOSTIC_MP_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("TargetType", SimTypeInt(signed=False, label="MP_STORAGE_DIAGNOSTIC_TARGET_TYPE")), ("Level", SimTypeInt(signed=False, label="MP_STORAGE_DIAGNOSTIC_LEVEL")), ("ProviderId", SimTypeBottom(label="Guid")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_DIAGNOSTIC_MP_REQUEST", pack=False, align=None),
    "STORAGE_DIAGNOSTIC_REQUEST": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TargetType", SimTypeInt(signed=False, label="STORAGE_DIAGNOSTIC_TARGET_TYPE")), ("Level", SimTypeInt(signed=False, label="STORAGE_DIAGNOSTIC_LEVEL")),)), name="STORAGE_DIAGNOSTIC_REQUEST", pack=False, align=None),
    "STORAGE_DIAGNOSTIC_TARGET_TYPE": SimTypeInt(signed=False, label="STORAGE_DIAGNOSTIC_TARGET_TYPE"),
    "STORAGE_DISK_HEALTH_STATUS": SimTypeInt(signed=False, label="STORAGE_DISK_HEALTH_STATUS"),
    "STORAGE_DISK_OPERATIONAL_STATUS": SimTypeInt(signed=False, label="STORAGE_DISK_OPERATIONAL_STATUS"),
    "STORAGE_ENCRYPTION_TYPE": SimTypeInt(signed=False, label="STORAGE_ENCRYPTION_TYPE"),
    "STORAGE_ENDURANCE_DATA_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("EnduranceInfo", SimTypeRef("STORAGE_ENDURANCE_INFO", SimStruct)),)), name="STORAGE_ENDURANCE_DATA_DESCRIPTOR", pack=False, align=None),
    "STORAGE_ENDURANCE_INFO": SimStruct(OrderedDict((("ValidFields", SimTypeInt(signed=False, label="UInt32")), ("GroupId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("_Flags_e__Struct", SimStruct)), ("LifePercentage", SimTypeInt(signed=False, label="UInt32")), ("BytesReadCount", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ByteWriteCount", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="STORAGE_ENDURANCE_INFO", pack=False, align=None),
    "STORAGE_EVENT_NOTIFICATION": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Events", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_EVENT_NOTIFICATION", pack=False, align=None),
    "STORAGE_FAILURE_PREDICTION_CONFIG": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Set", SimTypeChar(label="Byte")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="STORAGE_FAILURE_PREDICTION_CONFIG", pack=False, align=None),
    "STORAGE_FIRMWARE_ACTIVATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SlotToActivate", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="STORAGE_FIRMWARE_ACTIVATE", pack=False, align=None),
    "STORAGE_FIRMWARE_DOWNLOAD": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSize", SimTypeLongLong(signed=False, label="UInt64")), ("ImageBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_FIRMWARE_DOWNLOAD", pack=False, align=None),
    "STORAGE_FIRMWARE_DOWNLOAD_V2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSize", SimTypeLongLong(signed=False, label="UInt64")), ("Slot", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("ImageBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_FIRMWARE_DOWNLOAD_V2", pack=False, align=None),
    "STORAGE_FIRMWARE_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("UpgradeSupport", SimTypeChar(label="Byte")), ("SlotCount", SimTypeChar(label="Byte")), ("ActiveSlot", SimTypeChar(label="Byte")), ("PendingActivateSlot", SimTypeChar(label="Byte")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypePointer(SimTypeRef("STORAGE_FIRMWARE_SLOT_INFO", SimStruct), offset=0)),)), name="STORAGE_FIRMWARE_INFO", pack=False, align=None),
    "STORAGE_FIRMWARE_INFO_V2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("UpgradeSupport", SimTypeChar(label="Byte")), ("SlotCount", SimTypeChar(label="Byte")), ("ActiveSlot", SimTypeChar(label="Byte")), ("PendingActivateSlot", SimTypeChar(label="Byte")), ("FirmwareShared", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ImagePayloadAlignment", SimTypeInt(signed=False, label="UInt32")), ("ImagePayloadMaxSize", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypePointer(SimTypeRef("STORAGE_FIRMWARE_SLOT_INFO_V2", SimStruct), offset=0)),)), name="STORAGE_FIRMWARE_INFO_V2", pack=False, align=None),
    "STORAGE_FIRMWARE_SLOT_INFO": SimStruct(OrderedDict((("SlotNumber", SimTypeChar(label="Byte")), ("ReadOnly", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Revision", SimUnion({"Info": SimTypeArray(SimTypeChar(label="Byte"), 8), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="STORAGE_FIRMWARE_SLOT_INFO", pack=False, align=None),
    "STORAGE_FIRMWARE_SLOT_INFO_V2": SimStruct(OrderedDict((("SlotNumber", SimTypeChar(label="Byte")), ("ReadOnly", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Revision", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="STORAGE_FIRMWARE_SLOT_INFO_V2", pack=False, align=None),
    "STORAGE_FRU_ID_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("IdentifierSize", SimTypeInt(signed=False, label="UInt32")), ("Identifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_FRU_ID_DESCRIPTOR", pack=False, align=None),
    "STORAGE_GET_BC_PROPERTIES_OUTPUT": SimStruct(OrderedDict((("MaximumRequestsPerPeriod", SimTypeInt(signed=False, label="UInt32")), ("MinimumPeriod", SimTypeInt(signed=False, label="UInt32")), ("MaximumRequestSize", SimTypeLongLong(signed=False, label="UInt64")), ("EstimatedTimePerRequest", SimTypeInt(signed=False, label="UInt32")), ("NumOutStandingRequests", SimTypeInt(signed=False, label="UInt32")), ("RequestSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_GET_BC_PROPERTIES_OUTPUT", pack=False, align=None),
    "STORAGE_HOTPLUG_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("MediaRemovable", SimTypeChar(label="Byte")), ("MediaHotplug", SimTypeChar(label="Byte")), ("DeviceHotplug", SimTypeChar(label="Byte")), ("WriteCacheEnableOverride", SimTypeChar(label="Byte")),)), name="STORAGE_HOTPLUG_INFO", pack=False, align=None),
    "STORAGE_HW_ENDURANCE_DATA_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("EnduranceInfo", SimTypeRef("STORAGE_HW_ENDURANCE_INFO", SimStruct)),)), name="STORAGE_HW_ENDURANCE_DATA_DESCRIPTOR", pack=False, align=None),
    "STORAGE_HW_ENDURANCE_INFO": SimStruct(OrderedDict((("ValidFields", SimTypeInt(signed=False, label="UInt32")), ("GroupId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeRef("_Flags_e__Struct", SimStruct)), ("LifePercentage", SimTypeInt(signed=False, label="UInt32")), ("BytesReadCount", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ByteWriteCount", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="STORAGE_HW_ENDURANCE_INFO", pack=False, align=None),
    "STORAGE_HW_FIRMWARE_ACTIVATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="STORAGE_HW_FIRMWARE_ACTIVATE", pack=False, align=None),
    "STORAGE_HW_FIRMWARE_DOWNLOAD": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSize", SimTypeLongLong(signed=False, label="UInt64")), ("ImageBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_HW_FIRMWARE_DOWNLOAD", pack=False, align=None),
    "STORAGE_HW_FIRMWARE_DOWNLOAD_V2": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSize", SimTypeLongLong(signed=False, label="UInt64")), ("ImageSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("ImageBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_HW_FIRMWARE_DOWNLOAD_V2", pack=False, align=None),
    "STORAGE_HW_FIRMWARE_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeChar(label="Byte")), ("SlotCount", SimTypeChar(label="Byte")), ("ActiveSlot", SimTypeChar(label="Byte")), ("PendingActivateSlot", SimTypeChar(label="Byte")), ("FirmwareShared", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ImagePayloadAlignment", SimTypeInt(signed=False, label="UInt32")), ("ImagePayloadMaxSize", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypePointer(SimTypeRef("STORAGE_HW_FIRMWARE_SLOT_INFO", SimStruct), offset=0)),)), name="STORAGE_HW_FIRMWARE_INFO", pack=False, align=None),
    "STORAGE_HW_FIRMWARE_INFO_QUERY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_HW_FIRMWARE_INFO_QUERY", pack=False, align=None),
    "STORAGE_HW_FIRMWARE_SLOT_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SlotNumber", SimTypeChar(label="Byte")), ("_bitfield", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Revision", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="STORAGE_HW_FIRMWARE_SLOT_INFO", pack=False, align=None),
    "STORAGE_IDENTIFIER": SimStruct(OrderedDict((("CodeSet", SimTypeInt(signed=False, label="STORAGE_IDENTIFIER_CODE_SET")), ("Type", SimTypeInt(signed=False, label="STORAGE_IDENTIFIER_TYPE")), ("IdentifierSize", SimTypeShort(signed=False, label="UInt16")), ("NextOffset", SimTypeShort(signed=False, label="UInt16")), ("Association", SimTypeInt(signed=False, label="STORAGE_ASSOCIATION_TYPE")), ("Identifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_IDENTIFIER", pack=False, align=None),
    "STORAGE_IDENTIFIER_CODE_SET": SimTypeInt(signed=False, label="STORAGE_IDENTIFIER_CODE_SET"),
    "STORAGE_IDENTIFIER_TYPE": SimTypeInt(signed=False, label="STORAGE_IDENTIFIER_TYPE"),
    "STORAGE_IDLE_POWER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("D3IdleTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_IDLE_POWER", pack=False, align=None),
    "STORAGE_IDLE_POWERUP_REASON": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PowerupReason", SimTypeInt(signed=False, label="STORAGE_POWERUP_REASON_TYPE")),)), name="STORAGE_IDLE_POWERUP_REASON", pack=False, align=None),
    "STORAGE_ID_NAA_FORMAT": SimTypeInt(signed=False, label="STORAGE_ID_NAA_FORMAT"),
    "STORAGE_LB_PROVISIONING_MAP_RESOURCES": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("_bitfield1", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("_bitfield2", SimTypeChar(label="Byte")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("AvailableMappingResources", SimTypeLongLong(signed=False, label="UInt64")), ("UsedMappingResources", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_LB_PROVISIONING_MAP_RESOURCES", pack=False, align=None),
    "STORAGE_MEDIA_SERIAL_NUMBER_DATA": SimStruct(OrderedDict((("Reserved", SimTypeShort(signed=False, label="UInt16")), ("SerialNumberLength", SimTypeShort(signed=False, label="UInt16")), ("SerialNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_MEDIA_SERIAL_NUMBER_DATA", pack=False, align=None),
    "STORAGE_MEDIA_TYPE": SimTypeInt(signed=False, label="STORAGE_MEDIA_TYPE"),
    "STORAGE_MEDIUM_PRODUCT_TYPE_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("MediumProductType", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_MEDIUM_PRODUCT_TYPE_DESCRIPTOR", pack=False, align=None),
    "STORAGE_MINIPORT_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Portdriver", SimTypeInt(signed=False, label="STORAGE_PORT_CODE_SET")), ("LUNResetSupported", SimTypeChar(label="Byte")), ("TargetResetSupported", SimTypeChar(label="Byte")), ("IoTimeoutValue", SimTypeShort(signed=False, label="UInt16")), ("ExtraIoInfoSupported", SimTypeChar(label="Byte")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsBYTE": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_MINIPORT_DESCRIPTOR", pack=False, align=None),
    "STORAGE_OFFLOAD_READ_OUTPUT": SimStruct(OrderedDict((("OffloadReadFlags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("LengthProtected", SimTypeLongLong(signed=False, label="UInt64")), ("TokenLength", SimTypeInt(signed=False, label="UInt32")), ("Token", SimTypeRef("STORAGE_OFFLOAD_TOKEN", SimStruct)),)), name="STORAGE_OFFLOAD_READ_OUTPUT", pack=False, align=None),
    "STORAGE_OFFLOAD_TOKEN": SimStruct(OrderedDict((("TokenType", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("TokenIdLength", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Anonymous", SimUnion({"StorageOffloadZeroDataToken": SimTypeRef("_StorageOffloadZeroDataToken_e__Struct", SimStruct), "Token": SimTypeArray(SimTypeChar(label="Byte"), 504)}, name="<anon>", label="None")),)), name="STORAGE_OFFLOAD_TOKEN", pack=False, align=None),
    "STORAGE_OFFLOAD_WRITE_OUTPUT": SimStruct(OrderedDict((("OffloadWriteFlags", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("LengthCopied", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_OFFLOAD_WRITE_OUTPUT", pack=False, align=None),
    "STORAGE_OPERATIONAL_REASON": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Reason", SimTypeInt(signed=False, label="STORAGE_OPERATIONAL_STATUS_REASON")), ("RawBytes", SimUnion({"ScsiSenseKey": SimTypeRef("_ScsiSenseKey_e__Struct", SimStruct), "NVDIMM_N": SimTypeRef("_NVDIMM_N_e__Struct", SimStruct), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="STORAGE_OPERATIONAL_REASON", pack=False, align=None),
    "STORAGE_OPERATIONAL_STATUS_REASON": SimTypeInt(signed=False, label="STORAGE_OPERATIONAL_STATUS_REASON"),
    "STORAGE_PHYSICAL_ADAPTER_DATA": SimStruct(OrderedDict((("AdapterId", SimTypeInt(signed=False, label="UInt32")), ("HealthStatus", SimTypeInt(signed=False, label="STORAGE_COMPONENT_HEALTH_STATUS")), ("CommandProtocol", SimTypeInt(signed=False, label="STORAGE_PROTOCOL_TYPE")), ("SpecVersion", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorVersion", SimUnion({"Anonymous": SimStruct(OrderedDict((("SubMinor", SimTypeChar(label="Byte")), ("Minor", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Vendor", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Model", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("FirmwareRevision", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PhysicalLocation", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("ExpanderConnected", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Reserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="STORAGE_PHYSICAL_ADAPTER_DATA", pack=False, align=None),
    "STORAGE_PHYSICAL_DEVICE_DATA": SimStruct(OrderedDict((("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("Role", SimTypeInt(signed=False, label="UInt32")), ("HealthStatus", SimTypeInt(signed=False, label="STORAGE_COMPONENT_HEALTH_STATUS")), ("CommandProtocol", SimTypeInt(signed=False, label="STORAGE_PROTOCOL_TYPE")), ("SpecVersion", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorVersion", SimUnion({"Anonymous": SimStruct(OrderedDict((("SubMinor", SimTypeChar(label="Byte")), ("Minor", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("FormFactor", SimTypeInt(signed=False, label="STORAGE_DEVICE_FORM_FACTOR")), ("Vendor", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Model", SimTypeArray(SimTypeChar(label="Byte"), 40)), ("FirmwareRevision", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Capacity", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalLocation", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="STORAGE_PHYSICAL_DEVICE_DATA", pack=False, align=None),
    "STORAGE_PHYSICAL_NODE_DATA": SimStruct(OrderedDict((("NodeId", SimTypeInt(signed=False, label="UInt32")), ("AdapterCount", SimTypeInt(signed=False, label="UInt32")), ("AdapterDataLength", SimTypeInt(signed=False, label="UInt32")), ("AdapterDataOffset", SimTypeInt(signed=False, label="UInt32")), ("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("DeviceDataLength", SimTypeInt(signed=False, label="UInt32")), ("DeviceDataOffset", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="STORAGE_PHYSICAL_NODE_DATA", pack=False, align=None),
    "STORAGE_PHYSICAL_TOPOLOGY_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("NodeCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Node", SimTypePointer(SimTypeRef("STORAGE_PHYSICAL_NODE_DATA", SimStruct), offset=0)),)), name="STORAGE_PHYSICAL_TOPOLOGY_DESCRIPTOR", pack=False, align=None),
    "STORAGE_PORT_CODE_SET": SimTypeInt(signed=False, label="STORAGE_PORT_CODE_SET"),
    "STORAGE_POWERUP_REASON_TYPE": SimTypeInt(signed=False, label="STORAGE_POWERUP_REASON_TYPE"),
    "STORAGE_PREDICT_FAILURE": SimStruct(OrderedDict((("PredictFailure", SimTypeInt(signed=False, label="UInt32")), ("VendorSpecific", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="STORAGE_PREDICT_FAILURE", pack=False, align=None),
    "STORAGE_PRIORITY_HINT_SUPPORT": SimStruct(OrderedDict((("SupportFlags", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_PRIORITY_HINT_SUPPORT", pack=False, align=None),
    "STORAGE_PROPERTY_ID": SimTypeInt(signed=False, label="STORAGE_PROPERTY_ID"),
    "STORAGE_PROPERTY_QUERY": SimStruct(OrderedDict((("PropertyId", SimTypeInt(signed=False, label="STORAGE_PROPERTY_ID")), ("QueryType", SimTypeInt(signed=False, label="STORAGE_QUERY_TYPE")), ("AdditionalParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_PROPERTY_QUERY", pack=False, align=None),
    "STORAGE_PROPERTY_SET": SimStruct(OrderedDict((("PropertyId", SimTypeInt(signed=False, label="STORAGE_PROPERTY_ID")), ("SetType", SimTypeInt(signed=False, label="STORAGE_SET_TYPE")), ("AdditionalParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_PROPERTY_SET", pack=False, align=None),
    "STORAGE_PROTOCOL_ATA_DATA_TYPE": SimTypeInt(signed=False, label="STORAGE_PROTOCOL_ATA_DATA_TYPE"),
    "STORAGE_PROTOCOL_COMMAND": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("ProtocolType", SimTypeInt(signed=False, label="STORAGE_PROTOCOL_TYPE")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ReturnStatus", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("CommandLength", SimTypeInt(signed=False, label="UInt32")), ("ErrorInfoLength", SimTypeInt(signed=False, label="UInt32")), ("DataToDeviceTransferLength", SimTypeInt(signed=False, label="UInt32")), ("DataFromDeviceTransferLength", SimTypeInt(signed=False, label="UInt32")), ("TimeOutValue", SimTypeInt(signed=False, label="UInt32")), ("ErrorInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("DataToDeviceBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("DataFromDeviceBufferOffset", SimTypeInt(signed=False, label="UInt32")), ("CommandSpecific", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("FixedProtocolReturnData", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("Command", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STORAGE_PROTOCOL_COMMAND", pack=False, align=None),
    "STORAGE_PROTOCOL_DATA_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ProtocolSpecificData", SimTypeRef("STORAGE_PROTOCOL_SPECIFIC_DATA", SimStruct)),)), name="STORAGE_PROTOCOL_DATA_DESCRIPTOR", pack=False, align=None),
    "STORAGE_PROTOCOL_DATA_DESCRIPTOR_EXT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ProtocolSpecificData", SimTypeRef("STORAGE_PROTOCOL_SPECIFIC_DATA_EXT", SimStruct)),)), name="STORAGE_PROTOCOL_DATA_DESCRIPTOR_EXT", pack=False, align=None),
    "STORAGE_PROTOCOL_DATA_SUBVALUE_GET_LOG_PAGE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "STORAGE_PROTOCOL_NVME_DATA_TYPE": SimTypeInt(signed=False, label="STORAGE_PROTOCOL_NVME_DATA_TYPE"),
    "STORAGE_PROTOCOL_SPECIFIC_DATA": SimStruct(OrderedDict((("ProtocolType", SimTypeInt(signed=False, label="STORAGE_PROTOCOL_TYPE")), ("DataType", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataRequestValue", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataRequestSubValue", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataOffset", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataLength", SimTypeInt(signed=False, label="UInt32")), ("FixedProtocolReturnData", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataRequestSubValue2", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataRequestSubValue3", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataRequestSubValue4", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_PROTOCOL_SPECIFIC_DATA", pack=False, align=None),
    "STORAGE_PROTOCOL_SPECIFIC_DATA_EXT": SimStruct(OrderedDict((("ProtocolType", SimTypeInt(signed=False, label="STORAGE_PROTOCOL_TYPE")), ("DataType", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataValue", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataSubValue", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataOffset", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataLength", SimTypeInt(signed=False, label="UInt32")), ("FixedProtocolReturnData", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataSubValue2", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataSubValue3", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataSubValue4", SimTypeInt(signed=False, label="UInt32")), ("ProtocolDataSubValue5", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)),)), name="STORAGE_PROTOCOL_SPECIFIC_DATA_EXT", pack=False, align=None),
    "STORAGE_PROTOCOL_TYPE": SimTypeInt(signed=False, label="STORAGE_PROTOCOL_TYPE"),
    "STORAGE_PROTOCOL_UFS_DATA_TYPE": SimTypeInt(signed=False, label="STORAGE_PROTOCOL_UFS_DATA_TYPE"),
    "STORAGE_PROVIDER_FILE_FLAGS": SimTypeInt(signed=False, label="STORAGE_PROVIDER_FILE_FLAGS"),
    "STORAGE_QUERY_DEPENDENT_VOLUME_LEV1_ENTRY": SimStruct(OrderedDict((("EntryLength", SimTypeInt(signed=False, label="UInt32")), ("DependencyTypeFlags", SimTypeInt(signed=False, label="UInt32")), ("ProviderSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("VirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)),)), name="STORAGE_QUERY_DEPENDENT_VOLUME_LEV1_ENTRY", pack=False, align=None),
    "STORAGE_QUERY_DEPENDENT_VOLUME_LEV2_ENTRY": SimStruct(OrderedDict((("EntryLength", SimTypeInt(signed=False, label="UInt32")), ("DependencyTypeFlags", SimTypeInt(signed=False, label="UInt32")), ("ProviderSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("VirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("AncestorLevel", SimTypeInt(signed=False, label="UInt32")), ("HostVolumeNameOffset", SimTypeInt(signed=False, label="UInt32")), ("HostVolumeNameSize", SimTypeInt(signed=False, label="UInt32")), ("DependentVolumeNameOffset", SimTypeInt(signed=False, label="UInt32")), ("DependentVolumeNameSize", SimTypeInt(signed=False, label="UInt32")), ("RelativePathOffset", SimTypeInt(signed=False, label="UInt32")), ("RelativePathSize", SimTypeInt(signed=False, label="UInt32")), ("DependentDeviceNameOffset", SimTypeInt(signed=False, label="UInt32")), ("DependentDeviceNameSize", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_QUERY_DEPENDENT_VOLUME_LEV2_ENTRY", pack=False, align=None),
    "STORAGE_QUERY_DEPENDENT_VOLUME_REQUEST": SimStruct(OrderedDict((("RequestLevel", SimTypeInt(signed=False, label="UInt32")), ("RequestFlags", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_QUERY_DEPENDENT_VOLUME_REQUEST", pack=False, align=None),
    "STORAGE_QUERY_DEPENDENT_VOLUME_RESPONSE": SimStruct(OrderedDict((("ResponseLevel", SimTypeInt(signed=False, label="UInt32")), ("NumberEntries", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Lev1Depends": SimTypePointer(SimTypeRef("STORAGE_QUERY_DEPENDENT_VOLUME_LEV1_ENTRY", SimStruct), offset=0), "Lev2Depends": SimTypePointer(SimTypeRef("STORAGE_QUERY_DEPENDENT_VOLUME_LEV2_ENTRY", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="STORAGE_QUERY_DEPENDENT_VOLUME_RESPONSE", pack=False, align=None),
    "STORAGE_QUERY_TYPE": SimTypeInt(signed=False, label="STORAGE_QUERY_TYPE"),
    "STORAGE_READ_CAPACITY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("BlockLength", SimTypeInt(signed=False, label="UInt32")), ("NumberOfBlocks", SimTypeLongLong(signed=True, label="Int64")), ("DiskLength", SimTypeLongLong(signed=True, label="Int64")),)), name="STORAGE_READ_CAPACITY", pack=False, align=None),
    "STORAGE_REINITIALIZE_MEDIA": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("TimeoutInSeconds", SimTypeInt(signed=False, label="UInt32")), ("SanitizeOption", SimTypeRef("_SanitizeOption_e__Struct", SimStruct)),)), name="STORAGE_REINITIALIZE_MEDIA", pack=False, align=None),
    "STORAGE_RESERVE_ID": SimTypeInt(signed=False, label="STORAGE_RESERVE_ID"),
    "STORAGE_RPMB_COMMAND_TYPE": SimTypeInt(signed=False, label="STORAGE_RPMB_COMMAND_TYPE"),
    "STORAGE_RPMB_DATA_FRAME": SimStruct(OrderedDict((("Stuff", SimTypeArray(SimTypeChar(label="Byte"), 196)), ("KeyOrMAC", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("Data", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("Nonce", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("WriteCounter", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("BlockCount", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("OperationResult", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("RequestOrResponseType", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="STORAGE_RPMB_DATA_FRAME", pack=False, align=None),
    "STORAGE_RPMB_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("SizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("MaxReliableWriteSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("FrameFormat", SimTypeInt(signed=False, label="STORAGE_RPMB_FRAME_TYPE")),)), name="STORAGE_RPMB_DESCRIPTOR", pack=False, align=None),
    "STORAGE_RPMB_FRAME_TYPE": SimTypeInt(signed=False, label="STORAGE_RPMB_FRAME_TYPE"),
    "STORAGE_SANITIZE_METHOD": SimTypeInt(signed=False, label="STORAGE_SANITIZE_METHOD"),
    "STORAGE_SET_TYPE": SimTypeInt(signed=False, label="STORAGE_SET_TYPE"),
    "STORAGE_SPEC_VERSION": SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorVersion", SimUnion({"Anonymous": SimStruct(OrderedDict((("SubMinor", SimTypeChar(label="Byte")), ("Minor", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "STORAGE_TEMPERATURE_DATA_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("CriticalTemperature", SimTypeShort(signed=True, label="Int16")), ("WarningTemperature", SimTypeShort(signed=True, label="Int16")), ("InfoCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("Reserved1", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("TemperatureInfo", SimTypePointer(SimTypeRef("STORAGE_TEMPERATURE_INFO", SimStruct), offset=0)),)), name="STORAGE_TEMPERATURE_DATA_DESCRIPTOR", pack=False, align=None),
    "STORAGE_TEMPERATURE_INFO": SimStruct(OrderedDict((("Index", SimTypeShort(signed=False, label="UInt16")), ("Temperature", SimTypeShort(signed=True, label="Int16")), ("OverThreshold", SimTypeShort(signed=True, label="Int16")), ("UnderThreshold", SimTypeShort(signed=True, label="Int16")), ("OverThresholdChangable", SimTypeChar(label="Byte")), ("UnderThresholdChangable", SimTypeChar(label="Byte")), ("EventGenerated", SimTypeChar(label="Byte")), ("Reserved0", SimTypeChar(label="Byte")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")),)), name="STORAGE_TEMPERATURE_INFO", pack=False, align=None),
    "STORAGE_TEMPERATURE_THRESHOLD": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Index", SimTypeShort(signed=False, label="UInt16")), ("Threshold", SimTypeShort(signed=True, label="Int16")), ("OverThreshold", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="STORAGE_TEMPERATURE_THRESHOLD", pack=False, align=None),
    "STORAGE_TIER": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("Name", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Flags", SimTypeLongLong(signed=False, label="UInt64")), ("ProvisionedCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("MediaType", SimTypeInt(signed=False, label="STORAGE_TIER_MEDIA_TYPE")), ("Class", SimTypeInt(signed=False, label="STORAGE_TIER_CLASS")),)), name="STORAGE_TIER", pack=False, align=None),
    "STORAGE_TIER_CLASS": SimTypeInt(signed=False, label="STORAGE_TIER_CLASS"),
    "STORAGE_TIER_MEDIA_TYPE": SimTypeInt(signed=False, label="STORAGE_TIER_MEDIA_TYPE"),
    "STORAGE_TIER_REGION": SimStruct(OrderedDict((("TierId", SimTypeBottom(label="Guid")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_TIER_REGION", pack=False, align=None),
    "STORAGE_WRITE_CACHE_PROPERTY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("WriteCacheType", SimTypeInt(signed=False, label="WRITE_CACHE_TYPE")), ("WriteCacheEnabled", SimTypeInt(signed=False, label="WRITE_CACHE_ENABLE")), ("WriteCacheChangeable", SimTypeInt(signed=False, label="WRITE_CACHE_CHANGE")), ("WriteThroughSupported", SimTypeInt(signed=False, label="WRITE_THROUGH")), ("FlushCacheSupported", SimTypeChar(label="Byte")), ("UserDefinedPowerProtection", SimTypeChar(label="Byte")), ("NVCacheEnabled", SimTypeChar(label="Byte")),)), name="STORAGE_WRITE_CACHE_PROPERTY", pack=False, align=None),
    "STORAGE_ZONED_DEVICE_DESCRIPTOR": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("DeviceType", SimTypeInt(signed=False, label="STORAGE_ZONED_DEVICE_TYPES")), ("ZoneCount", SimTypeInt(signed=False, label="UInt32")), ("ZoneAttributes", SimUnion({"SequentialRequiredZone": SimTypeRef("_SequentialRequiredZone_e__Struct", SimStruct), "SequentialPreferredZone": SimTypeRef("_SequentialPreferredZone_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ZoneGroupCount", SimTypeInt(signed=False, label="UInt32")), ("ZoneGroup", SimTypePointer(SimTypeRef("STORAGE_ZONE_GROUP", SimStruct), offset=0)),)), name="STORAGE_ZONED_DEVICE_DESCRIPTOR", pack=False, align=None),
    "STORAGE_ZONED_DEVICE_TYPES": SimTypeInt(signed=False, label="STORAGE_ZONED_DEVICE_TYPES"),
    "STORAGE_ZONES_ATTRIBUTES": SimTypeInt(signed=False, label="STORAGE_ZONES_ATTRIBUTES"),
    "STORAGE_ZONE_CONDITION": SimTypeInt(signed=False, label="STORAGE_ZONE_CONDITION"),
    "STORAGE_ZONE_DESCRIPTOR": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("ZoneType", SimTypeInt(signed=False, label="STORAGE_ZONE_TYPES")), ("ZoneCondition", SimTypeInt(signed=False, label="STORAGE_ZONE_CONDITION")), ("ResetWritePointerRecommend", SimTypeChar(label="Byte")), ("Reserved0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ZoneSize", SimTypeLongLong(signed=False, label="UInt64")), ("WritePointerOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_ZONE_DESCRIPTOR", pack=False, align=None),
    "STORAGE_ZONE_GROUP": SimStruct(OrderedDict((("ZoneCount", SimTypeInt(signed=False, label="UInt32")), ("ZoneType", SimTypeInt(signed=False, label="STORAGE_ZONE_TYPES")), ("ZoneSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="STORAGE_ZONE_GROUP", pack=False, align=None),
    "STORAGE_ZONE_TYPES": SimTypeInt(signed=False, label="STORAGE_ZONE_TYPES"),
    "STPFLAG": SimTypeInt(signed=False, label="STPFLAG"),
    "STREAMBUFFER_ATTRIBUTE": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("StreamBufferAttributeType", SimTypeInt(signed=False, label="STREAMBUFFER_ATTR_DATATYPE")), ("pbAttribute", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbLength", SimTypeShort(signed=False, label="UInt16")),)), name="STREAMBUFFER_ATTRIBUTE", pack=False, align=None),
    "STREAMBUFFER_ATTR_DATATYPE": SimTypeInt(signed=False, label="STREAMBUFFER_ATTR_DATATYPE"),
    "STREAMIF_CONSTANTS": SimTypeInt(signed=False, label="STREAMIF_CONSTANTS"),
    "STREAMS_ASSOCIATE_ID_INPUT_BUFFER": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("StreamId", SimTypeInt(signed=False, label="UInt32")),)), name="STREAMS_ASSOCIATE_ID_INPUT_BUFFER", pack=False, align=None),
    "STREAMS_QUERY_ID_OUTPUT_BUFFER": SimStruct(OrderedDict((("StreamId", SimTypeInt(signed=False, label="UInt32")),)), name="STREAMS_QUERY_ID_OUTPUT_BUFFER", pack=False, align=None),
    "STREAMS_QUERY_PARAMETERS_OUTPUT_BUFFER": SimStruct(OrderedDict((("OptimalWriteSize", SimTypeInt(signed=False, label="UInt32")), ("StreamGranularitySize", SimTypeInt(signed=False, label="UInt32")), ("StreamIdMin", SimTypeInt(signed=False, label="UInt32")), ("StreamIdMax", SimTypeInt(signed=False, label="UInt32")),)), name="STREAMS_QUERY_PARAMETERS_OUTPUT_BUFFER", pack=False, align=None),
    "STREAM_EXTENT_ENTRY": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("ExtentInformation", SimUnion({"RetrievalPointers": SimTypeRef("RETRIEVAL_POINTERS_BUFFER", SimStruct)}, name="<anon>", label="None")),)), name="STREAM_EXTENT_ENTRY", pack=False, align=None),
    "STREAM_ID_MAP": SimStruct(OrderedDict((("stream_id", SimTypeInt(signed=False, label="UInt32")), ("dwMediaSampleContent", SimTypeInt(signed=False, label="UInt32")), ("ulSubstreamFilterValue", SimTypeInt(signed=False, label="UInt32")), ("iDataOffset", SimTypeInt(signed=True, label="Int32")),)), name="STREAM_ID_MAP", pack=False, align=None),
    "STREAM_INFORMATION_ENTRY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("StreamInformation", SimUnion({"DesiredStorageClass": SimTypeRef("_DesiredStorageClass", SimStruct), "DataStream": SimTypeRef("_DataStream", SimStruct), "Reparse": SimTypeRef("_Reparse", SimStruct), "Ea": SimTypeRef("_Ea", SimStruct)}, name="<anon>", label="None")),)), name="STREAM_INFORMATION_ENTRY", pack=False, align=None),
    "STREAM_INFO_LEVELS": SimTypeInt(signed=False, label="STREAM_INFO_LEVELS"),
    "STREAM_LAYOUT_ENTRY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("NextStreamOffset", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ExtentInformationOffset", SimTypeInt(signed=False, label="UInt32")), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")), ("EndOfFile", SimTypeLongLong(signed=True, label="Int64")), ("StreamInformationOffset", SimTypeInt(signed=False, label="UInt32")), ("AttributeTypeCode", SimTypeInt(signed=False, label="UInt32")), ("AttributeFlags", SimTypeInt(signed=False, label="UInt32")), ("StreamIdentifierLength", SimTypeInt(signed=False, label="UInt32")), ("StreamIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STREAM_LAYOUT_ENTRY", pack=False, align=None),
    "STREAM_MEDIUM": SimStruct(OrderedDict((("gidMedium", SimTypeBottom(label="Guid")), ("unMediumInstance", SimTypeInt(signed=False, label="UInt32")),)), name="STREAM_MEDIUM", pack=False, align=None),
    "STREAM_SEEK": SimTypeInt(signed=False, label="STREAM_SEEK"),
    "STREAM_STATE": SimTypeInt(signed=False, label="STREAM_STATE"),
    "STREAM_TYPE": SimTypeInt(signed=False, label="STREAM_TYPE"),
    "STRENTRYA": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STRENTRYA", pack=False, align=None),
    "STRENTRYW": SimStruct(OrderedDict((("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STRENTRYW", pack=False, align=None),
    "STRETCH_BLT_MODE": SimTypeInt(signed=False, label="STRETCH_BLT_MODE"),
    "STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="STRING", pack=False, align=None),
    "STRING32": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypeInt(signed=False, label="UInt32")),)), name="STRING32", pack=False, align=None),
    "STRING64": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypeLongLong(signed=False, label="UInt64")),)), name="STRING64", pack=False, align=None),
    "STRINGEXSTRUCT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("uDeterminePos", SimTypeInt(signed=False, label="UInt32")), ("uDetermineDelimPos", SimTypeInt(signed=False, label="UInt32")), ("uYomiPos", SimTypeInt(signed=False, label="UInt32")), ("uYomiDelimPos", SimTypeInt(signed=False, label="UInt32")),)), name="STRINGEXSTRUCT", pack=False, align=None),
    "STROBJ": SimStruct(OrderedDict((("cGlyphs", SimTypeInt(signed=False, label="UInt32")), ("flAccel", SimTypeInt(signed=False, label="UInt32")), ("ulCharInc", SimTypeInt(signed=False, label="UInt32")), ("rclBkGround", SimTypeRef("RECTL", SimStruct)), ("pgp", SimTypePointer(SimTypeRef("GLYPHPOS", SimStruct), offset=0)), ("pwszOrg", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="STROBJ", pack=False, align=None),
    "STROKE_RANGE": SimStruct(OrderedDict((("iStrokeBegin", SimTypeInt(signed=False, label="UInt32")), ("iStrokeEnd", SimTypeInt(signed=False, label="UInt32")),)), name="STROKE_RANGE", pack=False, align=None),
    "STRRET": SimStruct(OrderedDict((("uType", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"pOleStr": SimTypePointer(SimTypeChar(label="Char"), offset=0), "uOffset": SimTypeInt(signed=False, label="UInt32"), "cStr": SimTypeArray(SimTypeChar(label="Byte"), 260)}, name="<anon>", label="None")),)), name="STRRET", pack=False, align=None),
    "STRRET_TYPE": SimTypeInt(signed=False, label="STRRET_TYPE"),
    "STRTABLEA": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pse", SimTypePointer(SimTypeRef("STRENTRYA", SimStruct), offset=0)),)), name="STRTABLEA", pack=False, align=None),
    "STRTABLEW": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pse", SimTypePointer(SimTypeRef("STRENTRYW", SimStruct), offset=0)),)), name="STRTABLEW", pack=False, align=None),
    "STRUCTURED_QUERY_MULTIOPTION": SimTypeInt(signed=False, label="STRUCTURED_QUERY_MULTIOPTION"),
    "STRUCTURED_QUERY_PARSE_ERROR": SimTypeInt(signed=False, label="STRUCTURED_QUERY_PARSE_ERROR"),
    "STRUCTURED_QUERY_RESOLVE_OPTION": SimTypeInt(signed=False, label="STRUCTURED_QUERY_RESOLVE_OPTION"),
    "STRUCTURED_QUERY_SINGLE_OPTION": SimTypeInt(signed=False, label="STRUCTURED_QUERY_SINGLE_OPTION"),
    "STRUCTURED_QUERY_SYNTAX": SimTypeInt(signed=False, label="STRUCTURED_QUERY_SYNTAX"),
    "STUB_PHASE": SimTypeInt(signed=False, label="STUB_PHASE"),
    "STUB_THUNK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "STYLEBUFA": SimStruct(OrderedDict((("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="STYLEBUFA", pack=False, align=None),
    "STYLEBUFW": SimStruct(OrderedDict((("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="STYLEBUFW", pack=False, align=None),
    "STYLESTRUCT": SimStruct(OrderedDict((("styleOld", SimTypeInt(signed=False, label="UInt32")), ("styleNew", SimTypeInt(signed=False, label="UInt32")),)), name="STYLESTRUCT", pack=False, align=None),
    "STnefProblem": SimStruct(OrderedDict((("ulComponent", SimTypeInt(signed=False, label="UInt32")), ("ulAttribute", SimTypeInt(signed=False, label="UInt32")), ("ulPropTag", SimTypeInt(signed=False, label="UInt32")), ("scode", SimTypeInt(signed=True, label="Int32")),)), name="STnefProblem", pack=False, align=None),
    "STnefProblemArray": SimStruct(OrderedDict((("cProblem", SimTypeInt(signed=False, label="UInt32")), ("aProblem", SimTypePointer(SimTypeRef("STnefProblem", SimStruct), offset=0)),)), name="STnefProblemArray", pack=False, align=None),
    "SUBCLASSPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "uMsg", "wParam", "lParam", "uIdSubclass", "dwRefData"]), offset=0),
    "SUBSCRIBE_GENERIC_TLS_EXTENSION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("SubscriptionsCount", SimTypeInt(signed=False, label="UInt32")), ("Subscriptions", SimTypePointer(SimTypeRef("TLS_EXTENSION_SUBSCRIPTION", SimStruct), offset=0)),)), name="SUBSCRIBE_GENERIC_TLS_EXTENSION", pack=False, align=None),
    "SUBSCRIPTIONINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("fUpdateFlags", SimTypeInt(signed=False, label="UInt32")), ("schedule", SimTypeInt(signed=False, label="SUBSCRIPTIONSCHEDULE")), ("customGroupCookie", SimTypeBottom(label="Guid")), ("pTrigger", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwRecurseLevels", SimTypeInt(signed=False, label="UInt32")), ("fWebcrawlerFlags", SimTypeInt(signed=False, label="UInt32")), ("bMailNotification", SimTypeInt(signed=True, label="Int32")), ("bGleam", SimTypeInt(signed=True, label="Int32")), ("bChangesOnly", SimTypeInt(signed=True, label="Int32")), ("bNeedPassword", SimTypeInt(signed=True, label="Int32")), ("fChannelFlags", SimTypeInt(signed=False, label="UInt32")), ("bstrUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwMaxSizeKB", SimTypeInt(signed=False, label="UInt32")), ("subType", SimTypeInt(signed=False, label="SUBSCRIPTIONTYPE")), ("fTaskFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="SUBSCRIPTIONINFO", pack=False, align=None),
    "SUBSCRIPTIONINFOFLAGS": SimTypeInt(signed=False, label="SUBSCRIPTIONINFOFLAGS"),
    "SUBSCRIPTIONITEMINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPriority", SimTypeInt(signed=False, label="UInt32")), ("ScheduleGroup", SimTypeBottom(label="Guid")), ("clsidAgent", SimTypeBottom(label="Guid")),)), name="SUBSCRIPTIONITEMINFO", pack=False, align=None),
    "SUBSCRIPTIONSCHEDULE": SimTypeInt(signed=False, label="SUBSCRIPTIONSCHEDULE"),
    "SUBSCRIPTIONTYPE": SimTypeInt(signed=False, label="SUBSCRIPTIONTYPE"),
    "SUBSYSTEM_INFORMATION_TYPE": SimTypeInt(signed=False, label="SUBSYSTEM_INFORMATION_TYPE"),
    "SUITE_TYPE": SimTypeInt(signed=False, label="SUITE_TYPE"),
    "SUPPORTED_OS_INFO": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="SUPPORTED_OS_INFO", pack=False, align=None),
    "SUPPORTS_BINDING_INTERFACE_FLAGS": SimTypeInt(signed=False, label="SUPPORTS_BINDING_INTERFACE_FLAGS"),
    "SURFACEALIGNMENT": SimStruct(OrderedDict((("Anonymous", SimUnion({"Linear": SimTypeRef("_Linear_e__Struct", SimStruct), "Rectangular": SimTypeRef("_Rectangular_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="SURFACEALIGNMENT", pack=False, align=None),
    "SURFOBJ": SimStruct(OrderedDict((("dhsurf", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hsurf", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dhpdev", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hdev", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("sizlBitmap", SimTypeRef("SIZE", SimStruct)), ("cjBits", SimTypeInt(signed=False, label="UInt32")), ("pvBits", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pvScan0", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lDelta", SimTypeInt(signed=True, label="Int32")), ("iUniq", SimTypeInt(signed=False, label="UInt32")), ("iBitmapFormat", SimTypeInt(signed=False, label="UInt32")), ("iType", SimTypeShort(signed=False, label="UInt16")), ("fjBitmap", SimTypeShort(signed=False, label="UInt16")),)), name="SURFOBJ", pack=False, align=None),
    "SV2CVW2_PARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("psvPrev", SimTypeBottom(label="IShellView")), ("pfs", SimTypePointer(SimTypeRef("FOLDERSETTINGS", SimStruct), offset=0)), ("psbOwner", SimTypeBottom(label="IShellBrowser")), ("prcView", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("pvid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("hwndView", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SV2CVW2_PARAMS", pack=False, align=None),
    "SVUIA_STATUS": SimTypeInt(signed=False, label="SVUIA_STATUS"),
    "SWStringArray": SimStruct(OrderedDict((("cValues", SimTypeInt(signed=False, label="UInt32")), ("lppszW", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="SWStringArray", pack=False, align=None),
    "SW_DEVICE_CAPABILITIES": SimTypeInt(signed=False, label="SW_DEVICE_CAPABILITIES"),
    "SW_DEVICE_CREATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSwDevice", "CreateResult", "pContext", "pszDeviceInstanceId"]), offset=0),
    "SW_DEVICE_CREATE_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pszInstanceId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszzHardwareIds", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszzCompatibleIds", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pContainerId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("CapabilityFlags", SimTypeInt(signed=False, label="UInt32")), ("pszDeviceDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDeviceLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSecurityDescriptor", SimTypePointer(SimTypeRef("SECURITY_DESCRIPTOR", SimStruct), offset=0)),)), name="SW_DEVICE_CREATE_INFO", pack=False, align=None),
    "SW_DEVICE_LIFETIME": SimTypeInt(signed=False, label="SW_DEVICE_LIFETIME"),
    "SWbemAnalysisMatrix": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uMatrixType", SimTypeInt(signed=False, label="UInt32")), ("m_pszProperty", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_uPropertyType", SimTypeInt(signed=False, label="UInt32")), ("m_uEntries", SimTypeInt(signed=False, label="UInt32")), ("m_pValues", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("m_pbTruthTable", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="SWbemAnalysisMatrix", pack=False, align=None),
    "SWbemAnalysisMatrixList": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uMatrixType", SimTypeInt(signed=False, label="UInt32")), ("m_uNumMatrices", SimTypeInt(signed=False, label="UInt32")), ("m_pMatrices", SimTypePointer(SimTypeRef("SWbemAnalysisMatrix", SimStruct), offset=0)),)), name="SWbemAnalysisMatrixList", pack=False, align=None),
    "SWbemAssocQueryInf": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uAnalysisType", SimTypeInt(signed=False, label="UInt32")), ("m_uFeatureMask", SimTypeInt(signed=False, label="UInt32")), ("m_pPath", SimTypeBottom(label="IWbemPath")), ("m_pszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszQueryText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszResultClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszAssocClass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszRole", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszResultRole", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszRequiredQualifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszRequiredAssocQualifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SWbemAssocQueryInf", pack=False, align=None),
    "SWbemQueryQualifiedName": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uTokenType", SimTypeInt(signed=False, label="UInt32")), ("m_uNameListSize", SimTypeInt(signed=False, label="UInt32")), ("m_ppszNameList", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("m_bArraysUsed", SimTypeInt(signed=True, label="Int32")), ("m_pbArrayElUsed", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("m_puArrayIndex", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="SWbemQueryQualifiedName", pack=False, align=None),
    "SWbemRpnConst": SimUnion({"m_pszStrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "m_bBoolVal": SimTypeInt(signed=True, label="Int32"), "m_lLongVal": SimTypeInt(signed=True, label="Int32"), "m_uLongVal": SimTypeInt(signed=False, label="UInt32"), "m_dblVal": SimTypeFloat(size=64), "m_lVal64": SimTypeLongLong(signed=True, label="Int64"), "m_uVal64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"),
    "SWbemRpnEncodedQuery": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uTokenType", SimTypeInt(signed=False, label="UInt32")), ("m_uParsedFeatureMask", SimTypeLongLong(signed=False, label="UInt64")), ("m_uDetectedArraySize", SimTypeInt(signed=False, label="UInt32")), ("m_puDetectedFeatures", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("m_uSelectListSize", SimTypeInt(signed=False, label="UInt32")), ("m_ppSelectList", SimTypePointer(SimTypePointer(SimTypeRef("SWbemQueryQualifiedName", SimStruct), offset=0), offset=0)), ("m_uFromTargetType", SimTypeInt(signed=False, label="UInt32")), ("m_pszOptionalFromPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_uFromListSize", SimTypeInt(signed=False, label="UInt32")), ("m_ppszFromList", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("m_uWhereClauseSize", SimTypeInt(signed=False, label="UInt32")), ("m_ppRpnWhereClause", SimTypePointer(SimTypePointer(SimTypeRef("SWbemRpnQueryToken", SimStruct), offset=0), offset=0)), ("m_dblWithinPolling", SimTypeFloat(size=64)), ("m_dblWithinWindow", SimTypeFloat(size=64)), ("m_uOrderByListSize", SimTypeInt(signed=False, label="UInt32")), ("m_ppszOrderByList", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("m_uOrderDirectionEl", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="SWbemRpnEncodedQuery", pack=False, align=None),
    "SWbemRpnQueryToken": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uTokenType", SimTypeInt(signed=False, label="UInt32")), ("m_uSubexpressionShape", SimTypeInt(signed=False, label="UInt32")), ("m_uOperator", SimTypeInt(signed=False, label="UInt32")), ("m_pRightIdent", SimTypePointer(SimTypeRef("SWbemQueryQualifiedName", SimStruct), offset=0)), ("m_pLeftIdent", SimTypePointer(SimTypeRef("SWbemQueryQualifiedName", SimStruct), offset=0)), ("m_uConstApparentType", SimTypeInt(signed=False, label="UInt32")), ("m_Const", SimUnion({"m_pszStrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "m_bBoolVal": SimTypeInt(signed=True, label="Int32"), "m_lLongVal": SimTypeInt(signed=True, label="Int32"), "m_uLongVal": SimTypeInt(signed=False, label="UInt32"), "m_dblVal": SimTypeFloat(size=64), "m_lVal64": SimTypeLongLong(signed=True, label="Int64"), "m_uVal64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("m_uConst2ApparentType", SimTypeInt(signed=False, label="UInt32")), ("m_Const2", SimUnion({"m_pszStrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "m_bBoolVal": SimTypeInt(signed=True, label="Int32"), "m_lLongVal": SimTypeInt(signed=True, label="Int32"), "m_uLongVal": SimTypeInt(signed=False, label="UInt32"), "m_dblVal": SimTypeFloat(size=64), "m_lVal64": SimTypeLongLong(signed=True, label="Int64"), "m_uVal64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("m_pszRightFunc", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_pszLeftFunc", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SWbemRpnQueryToken", pack=False, align=None),
    "SWbemRpnTokenList": SimStruct(OrderedDict((("m_uVersion", SimTypeInt(signed=False, label="UInt32")), ("m_uTokenType", SimTypeInt(signed=False, label="UInt32")), ("m_uNumTokens", SimTypeInt(signed=False, label="UInt32")),)), name="SWbemRpnTokenList", pack=False, align=None),
    "SXH_PROXY_SETTING": SimTypeInt(signed=False, label="SXH_PROXY_SETTING"),
    "SXH_SERVER_CERT_OPTION": SimTypeInt(signed=False, label="SXH_SERVER_CERT_OPTION"),
    "SYMADDSOURCESTREAM": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "SYMADDSOURCESTREAMA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "SYMBOLIC_LINK_FLAGS": SimTypeInt(signed=False, label="SYMBOLIC_LINK_FLAGS"),
    "SYMBOL_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("TypeIndex", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="SYMBOL_INFO_FLAGS")), ("Value", SimTypeLongLong(signed=False, label="UInt64")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("Scope", SimTypeInt(signed=False, label="UInt32")), ("Tag", SimTypeInt(signed=False, label="UInt32")), ("NameLen", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLen", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SYMBOL_INFO", pack=False, align=None),
    "SYMBOL_INFOW": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("TypeIndex", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="SYMBOL_INFO_FLAGS")), ("Value", SimTypeLongLong(signed=False, label="UInt64")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Register", SimTypeInt(signed=False, label="UInt32")), ("Scope", SimTypeInt(signed=False, label="UInt32")), ("Tag", SimTypeInt(signed=False, label="UInt32")), ("NameLen", SimTypeInt(signed=False, label="UInt32")), ("MaxNameLen", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SYMBOL_INFOW", pack=False, align=None),
    "SYMBOL_INFO_EX": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("TypeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Line", SimTypeInt(signed=False, label="UInt32")), ("Displacement", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="SYMBOL_INFO_EX", pack=False, align=None),
    "SYMBOL_INFO_FLAGS": SimTypeInt(signed=False, label="SYMBOL_INFO_FLAGS"),
    "SYMBOL_INFO_PACKAGE": SimStruct(OrderedDict((("si", SimTypeRef("SYMBOL_INFO", SimStruct)), ("name", SimTypeArray(SimTypeChar(label="SByte"), 2001)),)), name="SYMBOL_INFO_PACKAGE", pack=False, align=None),
    "SYMBOL_INFO_PACKAGEW": SimStruct(OrderedDict((("si", SimTypeRef("SYMBOL_INFOW", SimStruct)), ("name", SimTypeArray(SimTypeChar(label="Char"), 2001)),)), name="SYMBOL_INFO_PACKAGEW", pack=False, align=None),
    "SYMSRV_EXTENDED_OUTPUT_DATA": SimStruct(OrderedDict((("sizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("version", SimTypeInt(signed=False, label="UInt32")), ("filePtrMsg", SimTypeArray(SimTypeChar(label="Char"), 261)),)), name="SYMSRV_EXTENDED_OUTPUT_DATA", pack=False, align=None),
    "SYMSRV_INDEX_INFO": SimStruct(OrderedDict((("sizeofstruct", SimTypeInt(signed=False, label="UInt32")), ("file", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("stripped", SimTypeInt(signed=True, label="Int32")), ("timestamp", SimTypeInt(signed=False, label="UInt32")), ("size", SimTypeInt(signed=False, label="UInt32")), ("dbgfile", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("pdbfile", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("guid", SimTypeBottom(label="Guid")), ("sig", SimTypeInt(signed=False, label="UInt32")), ("age", SimTypeInt(signed=False, label="UInt32")),)), name="SYMSRV_INDEX_INFO", pack=False, align=None),
    "SYMSRV_INDEX_INFOW": SimStruct(OrderedDict((("sizeofstruct", SimTypeInt(signed=False, label="UInt32")), ("file", SimTypeArray(SimTypeChar(label="Char"), 261)), ("stripped", SimTypeInt(signed=True, label="Int32")), ("timestamp", SimTypeInt(signed=False, label="UInt32")), ("size", SimTypeInt(signed=False, label="UInt32")), ("dbgfile", SimTypeArray(SimTypeChar(label="Char"), 261)), ("pdbfile", SimTypeArray(SimTypeChar(label="Char"), 261)), ("guid", SimTypeBottom(label="Guid")), ("sig", SimTypeInt(signed=False, label="UInt32")), ("age", SimTypeInt(signed=False, label="UInt32")),)), name="SYMSRV_INDEX_INFOW", pack=False, align=None),
    "SYM_DUMP_PARAM": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("sName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Options", SimTypeInt(signed=False, label="UInt32")), ("addr", SimTypeLongLong(signed=False, label="UInt64")), ("listLink", SimTypePointer(SimTypeRef("FIELD_INFO", SimStruct), offset=0)), ("Anonymous", SimUnion({"Context": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pBuffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("CallbackRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("FIELD_INFO", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pField", "UserContext"]), offset=0)), ("nFields", SimTypeInt(signed=False, label="UInt32")), ("Fields", SimTypePointer(SimTypeRef("FIELD_INFO", SimStruct), offset=0)), ("ModBase", SimTypeLongLong(signed=False, label="UInt64")), ("TypeId", SimTypeInt(signed=False, label="UInt32")), ("TypeSize", SimTypeInt(signed=False, label="UInt32")), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="SYM_DUMP_PARAM", pack=False, align=None),
    "SYM_FIND_ID_OPTION": SimTypeInt(signed=False, label="SYM_FIND_ID_OPTION"),
    "SYM_LOAD_FLAGS": SimTypeInt(signed=False, label="SYM_LOAD_FLAGS"),
    "SYM_SRV_STORE_FILE_FLAGS": SimTypeInt(signed=False, label="SYM_SRV_STORE_FILE_FLAGS"),
    "SYM_TYPE": SimTypeInt(signed=False, label="SYM_TYPE"),
    "SYNCHRONIZATION_ACCESS_RIGHTS": SimTypeInt(signed=False, label="SYNCHRONIZATION_ACCESS_RIGHTS"),
    "SYNCHRONIZATION_BARRIER": SimStruct(OrderedDict((("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)), ("Reserved4", SimTypeInt(signed=False, label="UInt32")), ("Reserved5", SimTypeInt(signed=False, label="UInt32")),)), name="SYNCHRONIZATION_BARRIER", pack=False, align=None),
    "SYNCMGRERRORFLAGS": SimTypeInt(signed=False, label="SYNCMGRERRORFLAGS"),
    "SYNCMGRFLAG": SimTypeInt(signed=False, label="SYNCMGRFLAG"),
    "SYNCMGRHANDLERFLAGS": SimTypeInt(signed=False, label="SYNCMGRHANDLERFLAGS"),
    "SYNCMGRHANDLERINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SyncMgrHandlerFlags", SimTypeInt(signed=False, label="UInt32")), ("wszHandlerName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="SYNCMGRHANDLERINFO", pack=False, align=None),
    "SYNCMGRINVOKEFLAGS": SimTypeInt(signed=False, label="SYNCMGRINVOKEFLAGS"),
    "SYNCMGRITEM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("ItemID", SimTypeBottom(label="Guid")), ("dwItemState", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wszItemName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("ftLastUpdate", SimTypeRef("FILETIME", SimStruct)),)), name="SYNCMGRITEM", pack=False, align=None),
    "SYNCMGRITEMFLAGS": SimTypeInt(signed=False, label="SYNCMGRITEMFLAGS"),
    "SYNCMGRITEMSTATE": SimTypeInt(signed=False, label="SYNCMGRITEMSTATE"),
    "SYNCMGRLOGERRORINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("mask", SimTypeInt(signed=False, label="UInt32")), ("dwSyncMgrErrorFlags", SimTypeInt(signed=False, label="UInt32")), ("ErrorID", SimTypeBottom(label="Guid")), ("ItemID", SimTypeBottom(label="Guid")),)), name="SYNCMGRLOGERRORINFO", pack=False, align=None),
    "SYNCMGRLOGLEVEL": SimTypeInt(signed=False, label="SYNCMGRLOGLEVEL"),
    "SYNCMGRPROGRESSITEM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("mask", SimTypeInt(signed=False, label="UInt32")), ("lpcStatusText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwStatusType", SimTypeInt(signed=False, label="UInt32")), ("iProgValue", SimTypeInt(signed=True, label="Int32")), ("iMaxValue", SimTypeInt(signed=True, label="Int32")),)), name="SYNCMGRPROGRESSITEM", pack=False, align=None),
    "SYNCMGRREGISTERFLAGS": SimTypeInt(signed=False, label="SYNCMGRREGISTERFLAGS"),
    "SYNCMGRSTATUS": SimTypeInt(signed=False, label="SYNCMGRSTATUS"),
    "SYNCMGR_CANCEL_REQUEST": SimTypeInt(signed=False, label="SYNCMGR_CANCEL_REQUEST"),
    "SYNCMGR_CONFLICT_ID_INFO": SimStruct(OrderedDict((("pblobID", SimTypePointer(SimTypeRef("BYTE_BLOB", SimStruct), offset=0)), ("pblobExtra", SimTypePointer(SimTypeRef("BYTE_BLOB", SimStruct), offset=0)),)), name="SYNCMGR_CONFLICT_ID_INFO", pack=False, align=None),
    "SYNCMGR_CONFLICT_ITEM_TYPE": SimTypeInt(signed=False, label="SYNCMGR_CONFLICT_ITEM_TYPE"),
    "SYNCMGR_CONTROL_FLAGS": SimTypeInt(signed=False, label="SYNCMGR_CONTROL_FLAGS"),
    "SYNCMGR_EVENT_FLAGS": SimTypeInt(signed=False, label="SYNCMGR_EVENT_FLAGS"),
    "SYNCMGR_EVENT_LEVEL": SimTypeInt(signed=False, label="SYNCMGR_EVENT_LEVEL"),
    "SYNCMGR_HANDLER_CAPABILITIES": SimTypeInt(signed=False, label="SYNCMGR_HANDLER_CAPABILITIES"),
    "SYNCMGR_HANDLER_POLICIES": SimTypeInt(signed=False, label="SYNCMGR_HANDLER_POLICIES"),
    "SYNCMGR_HANDLER_TYPE": SimTypeInt(signed=False, label="SYNCMGR_HANDLER_TYPE"),
    "SYNCMGR_ITEM_CAPABILITIES": SimTypeInt(signed=False, label="SYNCMGR_ITEM_CAPABILITIES"),
    "SYNCMGR_ITEM_POLICIES": SimTypeInt(signed=False, label="SYNCMGR_ITEM_POLICIES"),
    "SYNCMGR_PRESENTER_CHOICE": SimTypeInt(signed=False, label="SYNCMGR_PRESENTER_CHOICE"),
    "SYNCMGR_PRESENTER_NEXT_STEP": SimTypeInt(signed=False, label="SYNCMGR_PRESENTER_NEXT_STEP"),
    "SYNCMGR_PROGRESS_STATUS": SimTypeInt(signed=False, label="SYNCMGR_PROGRESS_STATUS"),
    "SYNCMGR_RESOLUTION_ABILITIES": SimTypeInt(signed=False, label="SYNCMGR_RESOLUTION_ABILITIES"),
    "SYNCMGR_RESOLUTION_FEEDBACK": SimTypeInt(signed=False, label="SYNCMGR_RESOLUTION_FEEDBACK"),
    "SYNCMGR_SYNC_CONTROL_FLAGS": SimTypeInt(signed=False, label="SYNCMGR_SYNC_CONTROL_FLAGS"),
    "SYNCMGR_UPDATE_REASON": SimTypeInt(signed=False, label="SYNCMGR_UPDATE_REASON"),
    "SYNC_CONSTRAINT_RESOLVE_ACTION": SimTypeInt(signed=False, label="SYNC_CONSTRAINT_RESOLVE_ACTION"),
    "SYNC_ENGINE_STATE_FLAGS": SimTypeInt(signed=False, label="SYNC_ENGINE_STATE_FLAGS"),
    "SYNC_FILTER_CHANGE": SimStruct(OrderedDict((("fMoveIn", SimTypeInt(signed=True, label="Int32")), ("moveVersion", SimTypeRef("SYNC_VERSION", SimStruct)),)), name="SYNC_FILTER_CHANGE", pack=False, align=None),
    "SYNC_FULL_ENUMERATION_ACTION": SimTypeInt(signed=False, label="SYNC_FULL_ENUMERATION_ACTION"),
    "SYNC_PROGRESS_STAGE": SimTypeInt(signed=False, label="SYNC_PROGRESS_STAGE"),
    "SYNC_PROVIDER_ROLE": SimTypeInt(signed=False, label="SYNC_PROVIDER_ROLE"),
    "SYNC_RANGE": SimStruct(OrderedDict((("pbClosedLowerBound", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pbClosedUpperBound", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SYNC_RANGE", pack=False, align=None),
    "SYNC_REGISTRATION_EVENT": SimTypeInt(signed=False, label="SYNC_REGISTRATION_EVENT"),
    "SYNC_RESOLVE_ACTION": SimTypeInt(signed=False, label="SYNC_RESOLVE_ACTION"),
    "SYNC_SERIALIZATION_VERSION": SimTypeInt(signed=False, label="SYNC_SERIALIZATION_VERSION"),
    "SYNC_SESSION_STATISTICS": SimStruct(OrderedDict((("dwChangesApplied", SimTypeInt(signed=False, label="UInt32")), ("dwChangesFailed", SimTypeInt(signed=False, label="UInt32")),)), name="SYNC_SESSION_STATISTICS", pack=False, align=None),
    "SYNC_STATISTICS": SimTypeInt(signed=False, label="SYNC_STATISTICS"),
    "SYNC_TIME": SimStruct(OrderedDict((("dwDate", SimTypeInt(signed=False, label="UInt32")), ("dwTime", SimTypeInt(signed=False, label="UInt32")),)), name="SYNC_TIME", pack=False, align=None),
    "SYNC_TRANSFER_STATUS": SimTypeInt(signed=False, label="SYNC_TRANSFER_STATUS"),
    "SYNC_VERSION": SimStruct(OrderedDict((("dwLastUpdatingReplicaKey", SimTypeInt(signed=False, label="UInt32")), ("ullTickCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="SYNC_VERSION", pack=False, align=None),
    "SYSBUTTONSTATES": SimTypeInt(signed=False, label="SYSBUTTONSTATES"),
    "SYSGEOCLASS": SimTypeInt(signed=False, label="SYSGEOCLASS"),
    "SYSGEOTYPE": SimTypeInt(signed=False, label="SYSGEOTYPE"),
    "SYSKIND": SimTypeInt(signed=False, label="SYSKIND"),
    "SYSNLS_FUNCTION": SimTypeInt(signed=False, label="SYSNLS_FUNCTION"),
    "SYSTEMCLOSEHCHOTSTATES": SimTypeInt(signed=False, label="SYSTEMCLOSEHCHOTSTATES"),
    "SYSTEMCLOSESTATES": SimTypeInt(signed=False, label="SYSTEMCLOSESTATES"),
    "SYSTEMMAXIMIZEHCHOTSTATES": SimTypeInt(signed=False, label="SYSTEMMAXIMIZEHCHOTSTATES"),
    "SYSTEMMAXIMIZESTATES": SimTypeInt(signed=False, label="SYSTEMMAXIMIZESTATES"),
    "SYSTEMMINIMIZEHCHOTSTATES": SimTypeInt(signed=False, label="SYSTEMMINIMIZEHCHOTSTATES"),
    "SYSTEMMINIMIZESTATES": SimTypeInt(signed=False, label="SYSTEMMINIMIZESTATES"),
    "SYSTEMRESTOREHCHOTSTATES": SimTypeInt(signed=False, label="SYSTEMRESTOREHCHOTSTATES"),
    "SYSTEMRESTORESTATES": SimTypeInt(signed=False, label="SYSTEMRESTORESTATES"),
    "SYSTEMTIME": SimStruct(OrderedDict((("wYear", SimTypeShort(signed=False, label="UInt16")), ("wMonth", SimTypeShort(signed=False, label="UInt16")), ("wDayOfWeek", SimTypeShort(signed=False, label="UInt16")), ("wDay", SimTypeShort(signed=False, label="UInt16")), ("wHour", SimTypeShort(signed=False, label="UInt16")), ("wMinute", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")), ("wMilliseconds", SimTypeShort(signed=False, label="UInt16")),)), name="SYSTEMTIME", pack=False, align=None),
    "SYSTEM_ACCESS_FILTER_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_ACCESS_FILTER_ACE", pack=False, align=None),
    "SYSTEM_ALARM_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_ALARM_ACE", pack=False, align=None),
    "SYSTEM_ALARM_CALLBACK_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_ALARM_CALLBACK_ACE", pack=False, align=None),
    "SYSTEM_ALARM_CALLBACK_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_ALARM_CALLBACK_OBJECT_ACE", pack=False, align=None),
    "SYSTEM_ALARM_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_ALARM_OBJECT_ACE", pack=False, align=None),
    "SYSTEM_AUDIT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_AUDIT_ACE", pack=False, align=None),
    "SYSTEM_AUDIT_CALLBACK_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_AUDIT_CALLBACK_ACE", pack=False, align=None),
    "SYSTEM_AUDIT_CALLBACK_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_AUDIT_CALLBACK_OBJECT_ACE", pack=False, align=None),
    "SYSTEM_AUDIT_OBJECT_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS")), ("ObjectType", SimTypeBottom(label="Guid")), ("InheritedObjectType", SimTypeBottom(label="Guid")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_AUDIT_OBJECT_ACE", pack=False, align=None),
    "SYSTEM_AUDIT_OBJECT_ACE_FLAGS": SimTypeInt(signed=False, label="SYSTEM_AUDIT_OBJECT_ACE_FLAGS"),
    "SYSTEM_BASIC_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 24)), ("Reserved2", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 4)), ("NumberOfProcessors", SimTypeChar(label="SByte")),)), name="SYSTEM_BASIC_INFORMATION", pack=False, align=None),
    "SYSTEM_BATTERY_STATE": SimStruct(OrderedDict((("AcOnLine", SimTypeChar(label="Byte")), ("BatteryPresent", SimTypeChar(label="Byte")), ("Charging", SimTypeChar(label="Byte")), ("Discharging", SimTypeChar(label="Byte")), ("Spare1", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Tag", SimTypeChar(label="Byte")), ("MaxCapacity", SimTypeInt(signed=False, label="UInt32")), ("RemainingCapacity", SimTypeInt(signed=False, label="UInt32")), ("Rate", SimTypeInt(signed=False, label="UInt32")), ("EstimatedTime", SimTypeInt(signed=False, label="UInt32")), ("DefaultAlert1", SimTypeInt(signed=False, label="UInt32")), ("DefaultAlert2", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_BATTERY_STATE", pack=False, align=None),
    "SYSTEM_CODEINTEGRITY_INFORMATION": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("CodeIntegrityOptions", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_CODEINTEGRITY_INFORMATION", pack=False, align=None),
    "SYSTEM_CPU_SET_INFORMATION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="CPU_SET_INFORMATION_TYPE")), ("Anonymous", SimUnion({"CpuSet": SimTypeRef("_CpuSet_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="SYSTEM_CPU_SET_INFORMATION", pack=False, align=None),
    "SYSTEM_CURSOR_ID": SimTypeInt(signed=False, label="SYSTEM_CURSOR_ID"),
    "SYSTEM_EVENT_DATA": SimStruct(OrderedDict((("bModifier", SimTypeChar(label="Byte")), ("wKey", SimTypeChar(label="Char")), ("xPos", SimTypeInt(signed=True, label="Int32")), ("yPos", SimTypeInt(signed=True, label="Int32")), ("bCursorMode", SimTypeChar(label="Byte")), ("dwButtonState", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_EVENT_DATA", pack=False, align=None),
    "SYSTEM_EXCEPTION_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SYSTEM_EXCEPTION_INFORMATION", pack=False, align=None),
    "SYSTEM_FIRMWARE_TABLE_ACTION": SimTypeInt(signed=False, label="SYSTEM_FIRMWARE_TABLE_ACTION"),
    "SYSTEM_FIRMWARE_TABLE_HANDLER": SimStruct(OrderedDict((("ProviderSignature", SimTypeInt(signed=False, label="UInt32")), ("Register", SimTypeChar(label="Byte")), ("FirmwareTableHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SYSTEM_FIRMWARE_TABLE_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SystemFirmwareTableInfo"]), offset=0)), ("DriverObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SYSTEM_FIRMWARE_TABLE_HANDLER", pack=False, align=None),
    "SYSTEM_FIRMWARE_TABLE_INFORMATION": SimStruct(OrderedDict((("ProviderSignature", SimTypeInt(signed=False, label="UInt32")), ("Action", SimTypeInt(signed=False, label="SYSTEM_FIRMWARE_TABLE_ACTION")), ("TableID", SimTypeInt(signed=False, label="UInt32")), ("TableBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TableBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SYSTEM_FIRMWARE_TABLE_INFORMATION", pack=False, align=None),
    "SYSTEM_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"dwOemId": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("wProcessorArchitecture", SimTypeInt(signed=False, label="PROCESSOR_ARCHITECTURE")), ("wReserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("dwPageSize", SimTypeInt(signed=False, label="UInt32")), ("lpMinimumApplicationAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpMaximumApplicationAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwActiveProcessorMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwNumberOfProcessors", SimTypeInt(signed=False, label="UInt32")), ("dwProcessorType", SimTypeInt(signed=False, label="UInt32")), ("dwAllocationGranularity", SimTypeInt(signed=False, label="UInt32")), ("wProcessorLevel", SimTypeShort(signed=False, label="UInt16")), ("wProcessorRevision", SimTypeShort(signed=False, label="UInt16")),)), name="SYSTEM_INFO", pack=False, align=None),
    "SYSTEM_INFORMATION_CLASS": SimTypeInt(signed=False, label="SYSTEM_INFORMATION_CLASS"),
    "SYSTEM_INTERRUPT_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 24)),)), name="SYSTEM_INTERRUPT_INFORMATION", pack=False, align=None),
    "SYSTEM_LOGICAL_PROCESSOR_INFORMATION": SimStruct(OrderedDict((("ProcessorMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Relationship", SimTypeInt(signed=False, label="LOGICAL_PROCESSOR_RELATIONSHIP")), ("Anonymous", SimUnion({"ProcessorCore": SimTypeRef("_ProcessorCore_e__Struct", SimStruct), "NumaNode": SimTypeRef("_NumaNode_e__Struct", SimStruct), "Cache": SimTypeRef("CACHE_DESCRIPTOR", SimStruct), "Reserved": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)}, name="<anon>", label="None")),)), name="SYSTEM_LOGICAL_PROCESSOR_INFORMATION", pack=False, align=None),
    "SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX": SimStruct(OrderedDict((("Relationship", SimTypeInt(signed=False, label="LOGICAL_PROCESSOR_RELATIONSHIP")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Processor": SimTypeRef("PROCESSOR_RELATIONSHIP", SimStruct), "NumaNode": SimTypeRef("NUMA_NODE_RELATIONSHIP", SimStruct), "Cache": SimTypeRef("CACHE_RELATIONSHIP", SimStruct), "Group": SimTypeRef("GROUP_RELATIONSHIP", SimStruct)}, name="<anon>", label="None")),)), name="SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX", pack=False, align=None),
    "SYSTEM_LOOKASIDE_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="SYSTEM_LOOKASIDE_INFORMATION", pack=False, align=None),
    "SYSTEM_MANDATORY_LABEL_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_MANDATORY_LABEL_ACE", pack=False, align=None),
    "SYSTEM_METRICS_INDEX": SimTypeInt(signed=False, label="SYSTEM_METRICS_INDEX"),
    "SYSTEM_PALETTE_USE": SimTypeInt(signed=False, label="SYSTEM_PALETTE_USE"),
    "SYSTEM_PARAMETERS_INFO_ACTION": SimTypeInt(signed=False, label="SYSTEM_PARAMETERS_INFO_ACTION"),
    "SYSTEM_PARAMETERS_INFO_UPDATE_FLAGS": SimTypeInt(signed=False, label="SYSTEM_PARAMETERS_INFO_UPDATE_FLAGS"),
    "SYSTEM_PERFORMANCE_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 312)),)), name="SYSTEM_PERFORMANCE_INFORMATION", pack=False, align=None),
    "SYSTEM_POLICY_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 2)), ("Reserved2", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="SYSTEM_POLICY_INFORMATION", pack=False, align=None),
    "SYSTEM_POOL_ZEROING_INFORMATION": SimStruct(OrderedDict((("PoolZeroingSupportPresent", SimTypeChar(label="Byte")),)), name="SYSTEM_POOL_ZEROING_INFORMATION", pack=False, align=None),
    "SYSTEM_POWER_CAPABILITIES": SimStruct(OrderedDict((("PowerButtonPresent", SimTypeChar(label="Byte")), ("SleepButtonPresent", SimTypeChar(label="Byte")), ("LidPresent", SimTypeChar(label="Byte")), ("SystemS1", SimTypeChar(label="Byte")), ("SystemS2", SimTypeChar(label="Byte")), ("SystemS3", SimTypeChar(label="Byte")), ("SystemS4", SimTypeChar(label="Byte")), ("SystemS5", SimTypeChar(label="Byte")), ("HiberFilePresent", SimTypeChar(label="Byte")), ("FullWake", SimTypeChar(label="Byte")), ("VideoDimPresent", SimTypeChar(label="Byte")), ("ApmPresent", SimTypeChar(label="Byte")), ("UpsPresent", SimTypeChar(label="Byte")), ("ThermalControl", SimTypeChar(label="Byte")), ("ProcessorThrottle", SimTypeChar(label="Byte")), ("ProcessorMinThrottle", SimTypeChar(label="Byte")), ("ProcessorMaxThrottle", SimTypeChar(label="Byte")), ("FastSystemS4", SimTypeChar(label="Byte")), ("Hiberboot", SimTypeChar(label="Byte")), ("WakeAlarmPresent", SimTypeChar(label="Byte")), ("AoAc", SimTypeChar(label="Byte")), ("DiskSpinDown", SimTypeChar(label="Byte")), ("HiberFileType", SimTypeChar(label="Byte")), ("AoAcConnectivitySupported", SimTypeChar(label="Byte")), ("spare3", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("SystemBatteriesPresent", SimTypeChar(label="Byte")), ("BatteriesAreShortTerm", SimTypeChar(label="Byte")), ("BatteryScale", SimTypeArray(SimTypeRef("BATTERY_REPORTING_SCALE", SimStruct), 3)), ("AcOnLineWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("SoftLidWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("RtcWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("MinDeviceWakeState", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("DefaultLowLatencyWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")),)), name="SYSTEM_POWER_CAPABILITIES", pack=False, align=None),
    "SYSTEM_POWER_CONDITION": SimTypeInt(signed=False, label="SYSTEM_POWER_CONDITION"),
    "SYSTEM_POWER_INFORMATION": SimStruct(OrderedDict((("MaxIdlenessAllowed", SimTypeInt(signed=False, label="UInt32")), ("Idleness", SimTypeInt(signed=False, label="UInt32")), ("TimeRemaining", SimTypeInt(signed=False, label="UInt32")), ("CoolingMode", SimTypeInt(signed=False, label="POWER_COOLING_MODE")),)), name="SYSTEM_POWER_INFORMATION", pack=False, align=None),
    "SYSTEM_POWER_LEVEL": SimStruct(OrderedDict((("Enable", SimTypeChar(label="Byte")), ("Spare", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("BatteryLevel", SimTypeInt(signed=False, label="UInt32")), ("PowerPolicy", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("MinSystemState", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")),)), name="SYSTEM_POWER_LEVEL", pack=False, align=None),
    "SYSTEM_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("PowerButton", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("SleepButton", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("LidClose", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("LidOpenWake", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Idle", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("IdleTimeout", SimTypeInt(signed=False, label="UInt32")), ("IdleSensitivity", SimTypeChar(label="Byte")), ("DynamicThrottle", SimTypeChar(label="Byte")), ("Spare2", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("MinSleep", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("MaxSleep", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("ReducedLatencySleep", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("WinLogonFlags", SimTypeInt(signed=False, label="UInt32")), ("Spare3", SimTypeInt(signed=False, label="UInt32")), ("DozeS4Timeout", SimTypeInt(signed=False, label="UInt32")), ("BroadcastCapacityResolution", SimTypeInt(signed=False, label="UInt32")), ("DischargePolicy", SimTypeArray(SimTypeRef("SYSTEM_POWER_LEVEL", SimStruct), 4)), ("VideoTimeout", SimTypeInt(signed=False, label="UInt32")), ("VideoDimDisplay", SimTypeChar(label="Byte")), ("VideoReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("SpindownTimeout", SimTypeInt(signed=False, label="UInt32")), ("OptimizeForPower", SimTypeChar(label="Byte")), ("FanThrottleTolerance", SimTypeChar(label="Byte")), ("ForcedThrottle", SimTypeChar(label="Byte")), ("MinThrottle", SimTypeChar(label="Byte")), ("OverThrottled", SimTypeRef("POWER_ACTION_POLICY", SimStruct)),)), name="SYSTEM_POWER_POLICY", pack=False, align=None),
    "SYSTEM_POWER_STATE": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"),
    "SYSTEM_POWER_STATE_CONTEXT": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ContextAsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="SYSTEM_POWER_STATE_CONTEXT", pack=False, align=None),
    "SYSTEM_POWER_STATUS": SimStruct(OrderedDict((("ACLineStatus", SimTypeChar(label="Byte")), ("BatteryFlag", SimTypeChar(label="Byte")), ("BatteryLifePercent", SimTypeChar(label="Byte")), ("SystemStatusFlag", SimTypeChar(label="Byte")), ("BatteryLifeTime", SimTypeInt(signed=False, label="UInt32")), ("BatteryFullLifeTime", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_POWER_STATUS", pack=False, align=None),
    "SYSTEM_PROCESSOR_CYCLE_TIME_INFORMATION": SimStruct(OrderedDict((("CycleTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="SYSTEM_PROCESSOR_CYCLE_TIME_INFORMATION", pack=False, align=None),
    "SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION": SimStruct(OrderedDict((("IdleTime", SimTypeLongLong(signed=True, label="Int64")), ("KernelTime", SimTypeLongLong(signed=True, label="Int64")), ("UserTime", SimTypeLongLong(signed=True, label="Int64")), ("Reserved1", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 2)), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION", pack=False, align=None),
    "SYSTEM_PROCESS_INFORMATION": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("NumberOfThreads", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 48)), ("ImageName", SimTypeRef("UNICODE_STRING", SimStruct)), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("UniqueProcessId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("PeakVirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved4", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved5", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved6", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrivatePageCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved7", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 6)),)), name="SYSTEM_PROCESS_INFORMATION", pack=False, align=None),
    "SYSTEM_PROCESS_TRUST_LABEL_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_PROCESS_TRUST_LABEL_ACE", pack=False, align=None),
    "SYSTEM_RADIO_STATE": SimTypeInt(signed=False, label="SYSTEM_RADIO_STATE"),
    "SYSTEM_REGISTRY_QUOTA_INFORMATION": SimStruct(OrderedDict((("RegistryQuotaAllowed", SimTypeInt(signed=False, label="UInt32")), ("RegistryQuotaUsed", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SYSTEM_REGISTRY_QUOTA_INFORMATION", pack=False, align=None),
    "SYSTEM_RESOURCE_ATTRIBUTE_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_RESOURCE_ATTRIBUTE_ACE", pack=False, align=None),
    "SYSTEM_SCOPED_POLICY_ID_ACE": SimStruct(OrderedDict((("Header", SimTypeRef("ACE_HEADER", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("SidStart", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_SCOPED_POLICY_ID_ACE", pack=False, align=None),
    "SYSTEM_SUPPORTED_PROCESSOR_ARCHITECTURES_INFORMATION": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_SUPPORTED_PROCESSOR_ARCHITECTURES_INFORMATION", pack=False, align=None),
    "SYSTEM_THREAD_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 3)), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("StartAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ClientId", SimTypeRef("CLIENT_ID", SimStruct)), ("Priority", SimTypeInt(signed=True, label="Int32")), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("ThreadState", SimTypeInt(signed=False, label="UInt32")), ("WaitReason", SimTypeInt(signed=False, label="UInt32")),)), name="SYSTEM_THREAD_INFORMATION", pack=False, align=None),
    "SYSTEM_TIMEOFDAY_INFORMATION": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 48)),)), name="SYSTEM_TIMEOFDAY_INFORMATION", pack=False, align=None),
    "SYS_COLOR_INDEX": SimTypeInt(signed=False, label="SYS_COLOR_INDEX"),
    "SZM_FLAGS": SimTypeInt(signed=False, label="SZM_FLAGS"),
    "SayAsInterpretAs": SimTypeInt(signed=False, label="SayAsInterpretAs"),
    "ScanModulationTypes": SimTypeInt(signed=False, label="ScanModulationTypes"),
    "SchGetExtensionsOptions": SimTypeInt(signed=False, label="SchGetExtensionsOptions"),
    "ScopeNamespace": SimTypeBottom(label="ScopeNamespace"),
    "Scope_list_ipv4": SimStruct(OrderedDict((("scopl_ipaddr", SimTypePointer(SimTypeRef("IN_ADDR", SimStruct), offset=0)),)), name="Scope_list_ipv4", pack=False, align=None),
    "ScriptChangeKind": SimTypeInt(signed=False, label="ScriptChangeKind"),
    "ScriptDebugEvent": SimTypeInt(signed=False, label="ScriptDebugEvent"),
    "ScriptDebugEventFilter": SimTypeInt(signed=False, label="ScriptDebugEventFilter"),
    "ScriptDebugEventInformation": SimStruct(OrderedDict((("DebugEvent", SimTypeInt(signed=False, label="ScriptDebugEvent")), ("EventPosition", SimTypeRef("ScriptDebugPosition", SimStruct)), ("EventSpanEnd", SimTypeRef("ScriptDebugPosition", SimStruct)), ("u", SimUnion({"ExceptionInformation": SimTypeRef("_ExceptionInformation_e__Struct", SimStruct), "BreakpointInformation": SimTypeRef("_BreakpointInformation_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="ScriptDebugEventInformation", pack=False, align=None),
    "ScriptDebugPosition": SimStruct(OrderedDict((("Line", SimTypeInt(signed=False, label="UInt32")), ("Column", SimTypeInt(signed=False, label="UInt32")),)), name="ScriptDebugPosition", pack=False, align=None),
    "ScriptDebugState": SimTypeInt(signed=False, label="ScriptDebugState"),
    "ScriptExecutionKind": SimTypeInt(signed=False, label="ScriptExecutionKind"),
    "ScrollAmount": SimTypeInt(signed=False, label="ScrollAmount"),
    "ScrollBarsConstants": SimTypeInt(signed=False, label="ScrollBarsConstants"),
    "SdpAttributeRange": SimStruct(OrderedDict((("minAttribute", SimTypeShort(signed=False, label="UInt16")), ("maxAttribute", SimTypeShort(signed=False, label="UInt16")),)), name="SdpAttributeRange", pack=False, align=None),
    "SdpQueryUuid": SimStruct(OrderedDict((("u", SimUnion({"uuid128": SimTypeBottom(label="Guid"), "uuid32": SimTypeInt(signed=False, label="UInt32"), "uuid16": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("uuidType", SimTypeShort(signed=False, label="UInt16")),)), name="SdpQueryUuid", pack=False, align=None),
    "SdpQueryUuidUnion": SimUnion({"uuid128": SimTypeBottom(label="Guid"), "uuid32": SimTypeInt(signed=False, label="UInt32"), "uuid16": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "SearchScope": SimTypeInt(signed=False, label="SearchScope"),
    "SecBuffer": SimStruct(OrderedDict((("cbBuffer", SimTypeInt(signed=False, label="UInt32")), ("BufferType", SimTypeInt(signed=False, label="UInt32")), ("pvBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SecBuffer", pack=False, align=None),
    "SecBufferDesc": SimStruct(OrderedDict((("ulVersion", SimTypeInt(signed=False, label="UInt32")), ("cBuffers", SimTypeInt(signed=False, label="UInt32")), ("pBuffers", SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)),)), name="SecBufferDesc", pack=False, align=None),
    "SecDelegationType": SimTypeInt(signed=False, label="SecDelegationType"),
    "SecHandle": SimStruct(OrderedDict((("dwLower", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwUpper", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="SecHandle", pack=False, align=None),
    "SecPkgContext_AccessToken": SimStruct(OrderedDict((("AccessToken", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SecPkgContext_AccessToken", pack=False, align=None),
    "SecPkgContext_ApplicationProtocol": SimStruct(OrderedDict((("ProtoNegoStatus", SimTypeInt(signed=False, label="SEC_APPLICATION_PROTOCOL_NEGOTIATION_STATUS")), ("ProtoNegoExt", SimTypeInt(signed=False, label="SEC_APPLICATION_PROTOCOL_NEGOTIATION_EXT")), ("ProtocolIdSize", SimTypeChar(label="Byte")), ("ProtocolId", SimTypeArray(SimTypeChar(label="Byte"), 255)),)), name="SecPkgContext_ApplicationProtocol", pack=False, align=None),
    "SecPkgContext_AuthorityA": SimStruct(OrderedDict((("sAuthorityName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SecPkgContext_AuthorityA", pack=False, align=None),
    "SecPkgContext_AuthorityW": SimStruct(OrderedDict((("sAuthorityName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_AuthorityW", pack=False, align=None),
    "SecPkgContext_AuthzID": SimStruct(OrderedDict((("AuthzIDLength", SimTypeInt(signed=False, label="UInt32")), ("AuthzID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_AuthzID", pack=False, align=None),
    "SecPkgContext_Bindings": SimStruct(OrderedDict((("BindingsLength", SimTypeInt(signed=False, label="UInt32")), ("Bindings", SimTypePointer(SimTypeRef("SEC_CHANNEL_BINDINGS", SimStruct), offset=0)),)), name="SecPkgContext_Bindings", pack=False, align=None),
    "SecPkgContext_CertInfo": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbSubjectName", SimTypeInt(signed=False, label="UInt32")), ("pwszSubjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbIssuerName", SimTypeInt(signed=False, label="UInt32")), ("pwszIssuerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwKeySize", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_CertInfo", pack=False, align=None),
    "SecPkgContext_CertificateValidationResult": SimStruct(OrderedDict((("dwChainErrorStatus", SimTypeInt(signed=False, label="UInt32")), ("hrVerifyChainStatus", SimTypeInt(signed=True, label="Int32")),)), name="SecPkgContext_CertificateValidationResult", pack=False, align=None),
    "SecPkgContext_Certificates": SimStruct(OrderedDict((("cCertificates", SimTypeInt(signed=False, label="UInt32")), ("cbCertificateChain", SimTypeInt(signed=False, label="UInt32")), ("pbCertificateChain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_Certificates", pack=False, align=None),
    "SecPkgContext_CipherInfo": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("dwCipherSuite", SimTypeInt(signed=False, label="UInt32")), ("dwBaseCipherSuite", SimTypeInt(signed=False, label="UInt32")), ("szCipherSuite", SimTypeArray(SimTypeChar(label="Char"), 64)), ("szCipher", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwCipherLen", SimTypeInt(signed=False, label="UInt32")), ("dwCipherBlockLen", SimTypeInt(signed=False, label="UInt32")), ("szHash", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwHashLen", SimTypeInt(signed=False, label="UInt32")), ("szExchange", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwMinExchangeLen", SimTypeInt(signed=False, label="UInt32")), ("dwMaxExchangeLen", SimTypeInt(signed=False, label="UInt32")), ("szCertificate", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwKeyType", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_CipherInfo", pack=False, align=None),
    "SecPkgContext_ClientCertPolicyResult": SimStruct(OrderedDict((("dwPolicyResult", SimTypeInt(signed=True, label="Int32")), ("guidPolicyId", SimTypeBottom(label="Guid")),)), name="SecPkgContext_ClientCertPolicyResult", pack=False, align=None),
    "SecPkgContext_ClientCreds": SimStruct(OrderedDict((("AuthBufferLen", SimTypeInt(signed=False, label="UInt32")), ("AuthBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_ClientCreds", pack=False, align=None),
    "SecPkgContext_ClientSpecifiedTarget": SimStruct(OrderedDict((("sTargetName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_ClientSpecifiedTarget", pack=False, align=None),
    "SecPkgContext_ConnectionInfo": SimStruct(OrderedDict((("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("aiCipher", SimTypeInt(signed=False, label="ALG_ID")), ("dwCipherStrength", SimTypeInt(signed=False, label="UInt32")), ("aiHash", SimTypeInt(signed=False, label="ALG_ID")), ("dwHashStrength", SimTypeInt(signed=False, label="UInt32")), ("aiExch", SimTypeInt(signed=False, label="ALG_ID")), ("dwExchStrength", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_ConnectionInfo", pack=False, align=None),
    "SecPkgContext_ConnectionInfoEx": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwProtocol", SimTypeInt(signed=False, label="UInt32")), ("szCipher", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwCipherStrength", SimTypeInt(signed=False, label="UInt32")), ("szHash", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwHashStrength", SimTypeInt(signed=False, label="UInt32")), ("szExchange", SimTypeArray(SimTypeChar(label="Char"), 64)), ("dwExchStrength", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_ConnectionInfoEx", pack=False, align=None),
    "SecPkgContext_CredInfo": SimStruct(OrderedDict((("CredClass", SimTypeInt(signed=False, label="SECPKG_CRED_CLASS")), ("IsPromptingNeeded", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_CredInfo", pack=False, align=None),
    "SecPkgContext_CredentialNameA": SimStruct(OrderedDict((("CredentialType", SimTypeInt(signed=False, label="UInt32")), ("sCredentialName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SecPkgContext_CredentialNameA", pack=False, align=None),
    "SecPkgContext_CredentialNameW": SimStruct(OrderedDict((("CredentialType", SimTypeInt(signed=False, label="UInt32")), ("sCredentialName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_CredentialNameW", pack=False, align=None),
    "SecPkgContext_DceInfo": SimStruct(OrderedDict((("AuthzSvc", SimTypeInt(signed=False, label="UInt32")), ("pPac", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SecPkgContext_DceInfo", pack=False, align=None),
    "SecPkgContext_EapKeyBlock": SimStruct(OrderedDict((("rgbKeys", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("rgbIVs", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="SecPkgContext_EapKeyBlock", pack=False, align=None),
    "SecPkgContext_EapPrfInfo": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbPrfData", SimTypeInt(signed=False, label="UInt32")), ("pbPrfData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_EapPrfInfo", pack=False, align=None),
    "SecPkgContext_EarlyStart": SimStruct(OrderedDict((("dwEarlyStartFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_EarlyStart", pack=False, align=None),
    "SecPkgContext_Flags": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_Flags", pack=False, align=None),
    "SecPkgContext_IssuerListInfoEx": SimStruct(OrderedDict((("aIssuers", SimTypePointer(SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), offset=0)), ("cIssuers", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_IssuerListInfoEx", pack=False, align=None),
    "SecPkgContext_KeyInfoA": SimStruct(OrderedDict((("sSignatureAlgorithmName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("sEncryptAlgorithmName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("KeySize", SimTypeInt(signed=False, label="UInt32")), ("SignatureAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("EncryptAlgorithm", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_KeyInfoA", pack=False, align=None),
    "SecPkgContext_KeyInfoW": SimStruct(OrderedDict((("sSignatureAlgorithmName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("sEncryptAlgorithmName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("KeySize", SimTypeInt(signed=False, label="UInt32")), ("SignatureAlgorithm", SimTypeInt(signed=False, label="UInt32")), ("EncryptAlgorithm", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_KeyInfoW", pack=False, align=None),
    "SecPkgContext_KeyingMaterial": SimStruct(OrderedDict((("cbKeyingMaterial", SimTypeInt(signed=False, label="UInt32")), ("pbKeyingMaterial", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_KeyingMaterial", pack=False, align=None),
    "SecPkgContext_KeyingMaterialInfo": SimStruct(OrderedDict((("cbLabel", SimTypeShort(signed=False, label="UInt16")), ("pszLabel", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbContextValue", SimTypeShort(signed=False, label="UInt16")), ("pbContextValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKeyingMaterial", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_KeyingMaterialInfo", pack=False, align=None),
    "SecPkgContext_KeyingMaterial_Inproc": SimStruct(OrderedDict((("cbLabel", SimTypeShort(signed=False, label="UInt16")), ("pszLabel", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbContextValue", SimTypeShort(signed=False, label="UInt16")), ("pbContextValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbKeyingMaterial", SimTypeInt(signed=False, label="UInt32")), ("pbKeyingMaterial", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_KeyingMaterial_Inproc", pack=False, align=None),
    "SecPkgContext_LastClientTokenStatus": SimStruct(OrderedDict((("LastClientTokenStatus", SimTypeInt(signed=False, label="SECPKG_ATTR_LCT_STATUS")),)), name="SecPkgContext_LastClientTokenStatus", pack=False, align=None),
    "SecPkgContext_Lifespan": SimStruct(OrderedDict((("tsStart", SimTypeLongLong(signed=True, label="Int64")), ("tsExpiry", SimTypeLongLong(signed=True, label="Int64")),)), name="SecPkgContext_Lifespan", pack=False, align=None),
    "SecPkgContext_LocalCredentialInfo": SimStruct(OrderedDict((("cbCertificateChain", SimTypeInt(signed=False, label="UInt32")), ("pbCertificateChain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cCertificates", SimTypeInt(signed=False, label="UInt32")), ("fFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBits", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_LocalCredentialInfo", pack=False, align=None),
    "SecPkgContext_LogoffTime": SimStruct(OrderedDict((("tsLogoffTime", SimTypeLongLong(signed=True, label="Int64")),)), name="SecPkgContext_LogoffTime", pack=False, align=None),
    "SecPkgContext_MappedCredAttr": SimStruct(OrderedDict((("dwAttribute", SimTypeInt(signed=False, label="UInt32")), ("pvBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SecPkgContext_MappedCredAttr", pack=False, align=None),
    "SecPkgContext_NamesA": SimStruct(OrderedDict((("sUserName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SecPkgContext_NamesA", pack=False, align=None),
    "SecPkgContext_NamesW": SimStruct(OrderedDict((("sUserName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_NamesW", pack=False, align=None),
    "SecPkgContext_NativeNamesA": SimStruct(OrderedDict((("sClientName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("sServerName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SecPkgContext_NativeNamesA", pack=False, align=None),
    "SecPkgContext_NativeNamesW": SimStruct(OrderedDict((("sClientName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("sServerName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_NativeNamesW", pack=False, align=None),
    "SecPkgContext_NegoKeys": SimStruct(OrderedDict((("KeyType", SimTypeInt(signed=False, label="UInt32")), ("KeyLength", SimTypeShort(signed=False, label="UInt16")), ("KeyValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("VerifyKeyType", SimTypeInt(signed=False, label="UInt32")), ("VerifyKeyLength", SimTypeShort(signed=False, label="UInt16")), ("VerifyKeyValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_NegoKeys", pack=False, align=None),
    "SecPkgContext_NegoPackageInfo": SimStruct(OrderedDict((("PackageMask", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_NegoPackageInfo", pack=False, align=None),
    "SecPkgContext_NegoStatus": SimStruct(OrderedDict((("LastStatus", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_NegoStatus", pack=False, align=None),
    "SecPkgContext_NegotiatedTlsExtensions": SimStruct(OrderedDict((("ExtensionsCount", SimTypeInt(signed=False, label="UInt32")), ("Extensions", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_NegotiatedTlsExtensions", pack=False, align=None),
    "SecPkgContext_NegotiationInfoA": SimStruct(OrderedDict((("PackageInfo", SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0)), ("NegotiationState", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_NegotiationInfoA", pack=False, align=None),
    "SecPkgContext_NegotiationInfoW": SimStruct(OrderedDict((("PackageInfo", SimTypePointer(SimTypeRef("SecPkgInfoW", SimStruct), offset=0)), ("NegotiationState", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_NegotiationInfoW", pack=False, align=None),
    "SecPkgContext_PackageInfoA": SimStruct(OrderedDict((("PackageInfo", SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0)),)), name="SecPkgContext_PackageInfoA", pack=False, align=None),
    "SecPkgContext_PackageInfoW": SimStruct(OrderedDict((("PackageInfo", SimTypePointer(SimTypeRef("SecPkgInfoW", SimStruct), offset=0)),)), name="SecPkgContext_PackageInfoW", pack=False, align=None),
    "SecPkgContext_PasswordExpiry": SimStruct(OrderedDict((("tsPasswordExpires", SimTypeLongLong(signed=True, label="Int64")),)), name="SecPkgContext_PasswordExpiry", pack=False, align=None),
    "SecPkgContext_ProtoInfoA": SimStruct(OrderedDict((("sProtocolName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("majorVersion", SimTypeInt(signed=False, label="UInt32")), ("minorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_ProtoInfoA", pack=False, align=None),
    "SecPkgContext_ProtoInfoW": SimStruct(OrderedDict((("sProtocolName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("majorVersion", SimTypeInt(signed=False, label="UInt32")), ("minorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_ProtoInfoW", pack=False, align=None),
    "SecPkgContext_RemoteCredentialInfo": SimStruct(OrderedDict((("cbCertificateChain", SimTypeInt(signed=False, label="UInt32")), ("pbCertificateChain", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cCertificates", SimTypeInt(signed=False, label="UInt32")), ("fFlags", SimTypeInt(signed=False, label="UInt32")), ("dwBits", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_RemoteCredentialInfo", pack=False, align=None),
    "SecPkgContext_SaslContext": SimStruct(OrderedDict((("SaslContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SecPkgContext_SaslContext", pack=False, align=None),
    "SecPkgContext_SessionAppData": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbAppData", SimTypeInt(signed=False, label="UInt32")), ("pbAppData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_SessionAppData", pack=False, align=None),
    "SecPkgContext_SessionInfo": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("cbSessionId", SimTypeInt(signed=False, label="UInt32")), ("rgbSessionId", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="SecPkgContext_SessionInfo", pack=False, align=None),
    "SecPkgContext_SessionKey": SimStruct(OrderedDict((("SessionKeyLength", SimTypeInt(signed=False, label="UInt32")), ("SessionKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_SessionKey", pack=False, align=None),
    "SecPkgContext_Sizes": SimStruct(OrderedDict((("cbMaxToken", SimTypeInt(signed=False, label="UInt32")), ("cbMaxSignature", SimTypeInt(signed=False, label="UInt32")), ("cbBlockSize", SimTypeInt(signed=False, label="UInt32")), ("cbSecurityTrailer", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_Sizes", pack=False, align=None),
    "SecPkgContext_SrtpParameters": SimStruct(OrderedDict((("ProtectionProfile", SimTypeShort(signed=False, label="UInt16")), ("MasterKeyIdentifierSize", SimTypeChar(label="Byte")), ("MasterKeyIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_SrtpParameters", pack=False, align=None),
    "SecPkgContext_StreamSizes": SimStruct(OrderedDict((("cbHeader", SimTypeInt(signed=False, label="UInt32")), ("cbTrailer", SimTypeInt(signed=False, label="UInt32")), ("cbMaximumMessage", SimTypeInt(signed=False, label="UInt32")), ("cBuffers", SimTypeInt(signed=False, label="UInt32")), ("cbBlockSize", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_StreamSizes", pack=False, align=None),
    "SecPkgContext_SubjectAttributes": SimStruct(OrderedDict((("AttributeInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="SecPkgContext_SubjectAttributes", pack=False, align=None),
    "SecPkgContext_SupportedSignatures": SimStruct(OrderedDict((("cSignatureAndHashAlgorithms", SimTypeShort(signed=False, label="UInt16")), ("pSignatureAndHashAlgorithms", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgContext_SupportedSignatures", pack=False, align=None),
    "SecPkgContext_Target": SimStruct(OrderedDict((("TargetLength", SimTypeInt(signed=False, label="UInt32")), ("Target", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_Target", pack=False, align=None),
    "SecPkgContext_TargetInformation": SimStruct(OrderedDict((("MarshalledTargetInfoLength", SimTypeInt(signed=False, label="UInt32")), ("MarshalledTargetInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_TargetInformation", pack=False, align=None),
    "SecPkgContext_TokenBinding": SimStruct(OrderedDict((("MajorVersion", SimTypeChar(label="Byte")), ("MinorVersion", SimTypeChar(label="Byte")), ("KeyParametersSize", SimTypeShort(signed=False, label="UInt16")), ("KeyParameters", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgContext_TokenBinding", pack=False, align=None),
    "SecPkgContext_UiInfo": SimStruct(OrderedDict((("hParentWindow", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="SecPkgContext_UiInfo", pack=False, align=None),
    "SecPkgContext_UserFlags": SimStruct(OrderedDict((("UserFlags", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgContext_UserFlags", pack=False, align=None),
    "SecPkgCred_CipherStrengths": SimStruct(OrderedDict((("dwMinimumCipherStrength", SimTypeInt(signed=False, label="UInt32")), ("dwMaximumCipherStrength", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgCred_CipherStrengths", pack=False, align=None),
    "SecPkgCred_ClientCertPolicy": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("guidPolicyId", SimTypeBottom(label="Guid")), ("dwCertFlags", SimTypeInt(signed=False, label="UInt32")), ("dwUrlRetrievalTimeout", SimTypeInt(signed=False, label="UInt32")), ("fCheckRevocationFreshnessTime", SimTypeInt(signed=True, label="Int32")), ("dwRevocationFreshnessTime", SimTypeInt(signed=False, label="UInt32")), ("fOmitUsageCheck", SimTypeInt(signed=True, label="Int32")), ("pwszSslCtlStoreName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszSslCtlIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SecPkgCred_ClientCertPolicy", pack=False, align=None),
    "SecPkgCred_SessionTicketKey": SimStruct(OrderedDict((("TicketInfoVersion", SimTypeInt(signed=False, label="UInt32")), ("KeyId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("KeyingMaterial", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("KeyingMaterialSize", SimTypeChar(label="Byte")),)), name="SecPkgCred_SessionTicketKey", pack=False, align=None),
    "SecPkgCred_SessionTicketKeys": SimStruct(OrderedDict((("cSessionTicketKeys", SimTypeInt(signed=False, label="UInt32")), ("pSessionTicketKeys", SimTypePointer(SimTypeRef("SecPkgCred_SessionTicketKey", SimStruct), offset=0)),)), name="SecPkgCred_SessionTicketKeys", pack=False, align=None),
    "SecPkgCred_SupportedAlgs": SimStruct(OrderedDict((("cSupportedAlgs", SimTypeInt(signed=False, label="UInt32")), ("palgSupportedAlgs", SimTypePointer(SimTypeInt(signed=False, label="ALG_ID"), offset=0)),)), name="SecPkgCred_SupportedAlgs", pack=False, align=None),
    "SecPkgCred_SupportedProtocols": SimStruct(OrderedDict((("grbitProtocol", SimTypeInt(signed=False, label="UInt32")),)), name="SecPkgCred_SupportedProtocols", pack=False, align=None),
    "SecPkgCredentials_Cert": SimStruct(OrderedDict((("EncodedCertSize", SimTypeInt(signed=False, label="UInt32")), ("EncodedCert", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgCredentials_Cert", pack=False, align=None),
    "SecPkgCredentials_KdcProxySettingsW": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ProxyServerOffset", SimTypeShort(signed=False, label="UInt16")), ("ProxyServerLength", SimTypeShort(signed=False, label="UInt16")), ("ClientTlsCredOffset", SimTypeShort(signed=False, label="UInt16")), ("ClientTlsCredLength", SimTypeShort(signed=False, label="UInt16")),)), name="SecPkgCredentials_KdcProxySettingsW", pack=False, align=None),
    "SecPkgCredentials_NamesA": SimStruct(OrderedDict((("sUserName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SecPkgCredentials_NamesA", pack=False, align=None),
    "SecPkgCredentials_NamesW": SimStruct(OrderedDict((("sUserName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgCredentials_NamesW", pack=False, align=None),
    "SecPkgCredentials_SSIProviderA": SimStruct(OrderedDict((("sProviderName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("ProviderInfoLength", SimTypeInt(signed=False, label="UInt32")), ("ProviderInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgCredentials_SSIProviderA", pack=False, align=None),
    "SecPkgCredentials_SSIProviderW": SimStruct(OrderedDict((("sProviderName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("ProviderInfoLength", SimTypeInt(signed=False, label="UInt32")), ("ProviderInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SecPkgCredentials_SSIProviderW", pack=False, align=None),
    "SecPkgInfoA": SimStruct(OrderedDict((("fCapabilities", SimTypeInt(signed=False, label="UInt32")), ("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wRPCID", SimTypeShort(signed=False, label="UInt16")), ("cbMaxToken", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("Comment", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="SecPkgInfoA", pack=False, align=None),
    "SecPkgInfoW": SimStruct(OrderedDict((("fCapabilities", SimTypeInt(signed=False, label="UInt32")), ("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wRPCID", SimTypeShort(signed=False, label="UInt16")), ("cbMaxToken", SimTypeInt(signed=False, label="UInt32")), ("Name", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("Comment", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="SecPkgInfoW", pack=False, align=None),
    "SecureLockIconConstants": SimTypeInt(signed=False, label="SecureLockIconConstants"),
    "SecurityFunctionTableA": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("EnumerateSecurityPackagesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("QueryCredentialsAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("AcquireCredentialsHandleA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0)), ("FreeCredentialsHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("Reserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InitializeSecurityContextA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10", "param11"]), offset=0)), ("AcceptSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0)), ("CompleteAuthToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("DeleteSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("ApplyControlToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("QueryContextAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("ImpersonateSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("RevertSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("MakeSignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("VerifySignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("FreeContextBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("QuerySecurityPackageInfoA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("Reserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ExportSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("ImportSecurityContextA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("AddCredentialsA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0)), ("Reserved8", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QuerySecurityContextToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("EncryptMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("DecryptMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("SetContextAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("SetCredentialsAttributesA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("ChangeAccountPasswordA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0)), ("QueryContextAttributesExA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("QueryCredentialsAttributesExA", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)),)), name="SecurityFunctionTableA", pack=False, align=None),
    "SecurityFunctionTableW": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("EnumerateSecurityPackagesW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("QueryCredentialsAttributesW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("AcquireCredentialsHandleW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0)), ("FreeCredentialsHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("Reserved2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InitializeSecurityContextW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8", "param9", "param10", "param11"]), offset=0)), ("AcceptSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7", "param8"]), offset=0)), ("CompleteAuthToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("DeleteSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("ApplyControlToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("QueryContextAttributesW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("ImpersonateSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("RevertSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("MakeSignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("VerifySignature", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("FreeContextBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0)), ("QuerySecurityPackageInfoW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SecPkgInfoW", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("Reserved3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ExportSecurityContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("ImportSecurityContextW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("AddCredentialsW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Arg", "Principal", "KeyVer", "Key", "Status"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0)), ("Reserved8", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("QuerySecurityContextToken", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0)), ("EncryptMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("DecryptMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("SetContextAttributesW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("SetCredentialsAttributesW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("ChangeAccountPasswordW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0)), ("QueryContextAttributesExW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("QueryCredentialsAttributesExW", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0)),)), name="SecurityFunctionTableW", pack=False, align=None),
    "SecurityProperty": SimTypeBottom(label="SecurityProperty"),
    "SegDispidList": SimTypeInt(signed=False, label="SegDispidList"),
    "SegEventidList": SimTypeInt(signed=False, label="SegEventidList"),
    "SelAlignmentConstants": SimTypeInt(signed=False, label="SelAlignmentConstants"),
    "SelectionHitResult": SimTypeInt(signed=False, label="SelectionHitResult"),
    "SendToMode": SimTypeInt(signed=False, label="SendToMode"),
    "SensorConnectionType": SimTypeInt(signed=False, label="SensorConnectionType"),
    "SensorState": SimTypeInt(signed=False, label="SensorState"),
    "ServerSelection": SimTypeInt(signed=False, label="ServerSelection"),
    "Session_IPv4": SimStruct(OrderedDict((("sess_destaddr", SimTypeRef("IN_ADDR", SimStruct)), ("sess_protid", SimTypeChar(label="Byte")), ("sess_flags", SimTypeChar(label="Byte")), ("sess_destport", SimTypeShort(signed=False, label="UInt16")),)), name="Session_IPv4", pack=False, align=None),
    "SetupFileLogInfo": SimTypeInt(signed=False, label="SetupFileLogInfo"),
    "SharedVirtualDiskHandleState": SimTypeInt(signed=False, label="SharedVirtualDiskHandleState"),
    "SharedVirtualDiskSupportType": SimTypeInt(signed=False, label="SharedVirtualDiskSupportType"),
    "Sharpen": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="Sharpen", pack=False, align=None),
    "SharpenParams": SimStruct(OrderedDict((("radius", SimTypeFloat(size=32)), ("amount", SimTypeFloat(size=32)),)), name="SharpenParams", pack=False, align=None),
    "ShellCommandInfo": SimStruct(OrderedDict((("pwszOperation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszParameters", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nShowCmd", SimTypeInt(signed=False, label="UInt32")),)), name="ShellCommandInfo", pack=False, align=None),
    "ShellFolderViewOptions": SimTypeInt(signed=False, label="ShellFolderViewOptions"),
    "ShellSpecialFolderConstants": SimTypeInt(signed=False, label="ShellSpecialFolderConstants"),
    "ShellWindowFindWindowOptions": SimTypeInt(signed=False, label="ShellWindowFindWindowOptions"),
    "ShellWindowTypeConstants": SimTypeInt(signed=False, label="ShellWindowTypeConstants"),
    "ShutdownType": SimTypeInt(signed=False, label="ShutdownType"),
    "SignalAndServiceStatusSpanningEvent_State": SimTypeInt(signed=False, label="SignalAndServiceStatusSpanningEvent_State"),
    "SignatureComparison": SimTypeInt(signed=False, label="SignatureComparison"),
    "SimilarityData": SimStruct(OrderedDict((("m_Data", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="SimilarityData", pack=False, align=None),
    "SimilarityDumpData": SimStruct(OrderedDict((("m_FileIndex", SimTypeInt(signed=False, label="UInt32")), ("m_Data", SimTypeRef("SimilarityData", SimStruct)),)), name="SimilarityDumpData", pack=False, align=None),
    "SimilarityFileId": SimStruct(OrderedDict((("m_FileId", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="SimilarityFileId", pack=False, align=None),
    "SimilarityMappedViewInfo": SimStruct(OrderedDict((("m_Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("m_Length", SimTypeInt(signed=False, label="UInt32")),)), name="SimilarityMappedViewInfo", pack=False, align=None),
    "SimpleDeviceOrientation": SimTypeInt(signed=False, label="SimpleDeviceOrientation"),
    "Size": SimStruct(OrderedDict((("Width", SimTypeInt(signed=True, label="Int32")), ("Height", SimTypeInt(signed=True, label="Int32")),)), name="Size", pack=False, align=None),
    "SizeF": SimStruct(OrderedDict((("Width", SimTypeFloat(size=32)), ("Height", SimTypeFloat(size=32)),)), name="SizeF", pack=False, align=None),
    "SmartCardApplication": SimStruct(OrderedDict((("ApplicationType", SimTypeInt(signed=False, label="ApplicationTypeType")), ("ApplicationVersion", SimTypeShort(signed=False, label="UInt16")), ("pbstrApplicationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pbstrApplicationURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SmartCardApplication", pack=False, align=None),
    "SmartCardAssociationType": SimTypeInt(signed=False, label="SmartCardAssociationType"),
    "SmartCardStatusType": SimTypeInt(signed=False, label="SmartCardStatusType"),
    "SmoothingMode": SimTypeInt(signed=False, label="SmoothingMode"),
    "SnapIn": SimTypeBottom(label="SnapIn"),
    "SnapIns": SimTypeBottom(label="SnapIns"),
    "SnapshotEncodingType": SimTypeInt(signed=False, label="SnapshotEncodingType"),
    "SnapshotFormatType": SimTypeInt(signed=False, label="SnapshotFormatType"),
    "SnmpVarBind": SimStruct(OrderedDict((("name", SimTypeRef("AsnObjectIdentifier", SimStruct)), ("value", SimTypeRef("AsnAny", SimStruct)),)), name="SnmpVarBind", pack=False, align=None),
    "SnmpVarBindList": SimStruct(OrderedDict((("list", SimTypePointer(SimTypeRef("SnmpVarBind", SimStruct), offset=0)), ("len", SimTypeInt(signed=False, label="UInt32")),)), name="SnmpVarBindList", pack=False, align=None),
    "SoH": SimStruct(OrderedDict((("count", SimTypeShort(signed=False, label="UInt16")), ("attributes", SimTypePointer(SimTypeRef("SoHAttribute", SimStruct), offset=0)),)), name="SoH", pack=False, align=None),
    "SoHAttribute": SimStruct(OrderedDict((("type", SimTypeShort(signed=False, label="UInt16")), ("size", SimTypeShort(signed=False, label="UInt16")), ("value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SoHAttribute", pack=False, align=None),
    "SourceInfo": SimStruct(OrderedDict((("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LineNumber", SimTypeInt(signed=False, label="UInt32")), ("ColumnNumber", SimTypeInt(signed=False, label="UInt32")), ("CharPosition", SimTypeInt(signed=False, label="UInt32")), ("Hash", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SourceInfo", pack=False, align=None),
    "SourceSizeList": SimTypeInt(signed=False, label="SourceSizeList"),
    "Sources": SimStruct(OrderedDict((("sourceId", SimTypeInt(signed=False, label="UInt32")), ("numTargets", SimTypeInt(signed=True, label="Int32")), ("aTargets", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="Sources", pack=False, align=None),
    "SpAcceptCredentialsFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SECURITY_LOGON_TYPE"), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_PRIMARY_CRED", SimStruct), offset=0), SimTypePointer(SimTypeRef("SECPKG_SUPPLEMENTAL_CRED", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonType", "AccountName", "PrimaryCredentials", "SupplementalCredentials"]), offset=0),
    "SpAcceptLsaModeContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "ContextHandle", "InputBuffer", "ContextRequirements", "TargetDataRep", "NewContextHandle", "OutputBuffer", "ContextAttributes", "ExpirationTime", "MappedContext", "ContextData"]), offset=0),
    "SpAcquireCredentialsHandleFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrincipalName", "CredentialUseFlags", "LogonId", "AuthorizationData", "GetKeyFunciton", "GetKeyArgument", "CredentialHandle", "ExpirationTime"]), offset=0),
    "SpAddCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "PrincipalName", "Package", "CredentialUseFlags", "AuthorizationData", "GetKeyFunciton", "GetKeyArgument", "ExpirationTime"]), offset=0),
    "SpApplyControlTokenFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ControlToken"]), offset=0),
    "SpChangeAccountPasswordFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDomainName", "pAccountName", "pOldPassword", "pNewPassword", "Impersonating", "pOutput"]), offset=0),
    "SpCompleteAuthTokenFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "InputBuffer"]), offset=0),
    "SpDeleteContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle"]), offset=0),
    "SpDeleteCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "Key"]), offset=0),
    "SpExchangeMetaDataFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "TargetName", "ContextRequirements", "MetaDataLength", "MetaData", "ContextHandle"]), offset=0),
    "SpExportSecurityContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phContext", "fFlags", "pPackedContext", "pToken"]), offset=0),
    "SpExtractTargetInfoFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "ppvTargetInfo", "pcbTargetInfo"]), offset=0),
    "SpFormatCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Credentials", "FormattedCredentials"]), offset=0),
    "SpFreeCredentialsHandleFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle"]), offset=0),
    "SpGetContextTokenFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ImpersonationToken"]), offset=0),
    "SpGetCredUIContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "CredType", "FlatCredUIContextLength", "FlatCredUIContext"]), offset=0),
    "SpGetCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "Credentials"]), offset=0),
    "SpGetExtendedInformationFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SECPKG_EXTENDED_INFORMATION_CLASS"), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_EXTENDED_INFORMATION", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Class", "ppInformation"]), offset=0),
    "SpGetInfoFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecPkgInfoA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PackageInfo"]), offset=0),
    "SpGetRemoteCredGuardLogonBufferFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Buffer", "BufferLength", "ReturnBuffer", "ReturnBufferLength"]), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["RedirectedLogonHandle"]), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredHandle", "ContextHandle", "TargetName", "RedirectedLogonHandle", "Callback", "CleanupCallback", "LogonBufferSize", "LogonBuffer"]), offset=0),
    "SpGetRemoteCredGuardSupplementalCredsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RedirectedLogonHandle", "Buffer", "BufferLength", "ReturnBuffer", "ReturnBufferLength"]), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["RedirectedLogonHandle"]), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredHandle", "TargetName", "RedirectedLogonHandle", "Callback", "CleanupCallback", "SupplementalCredsSize", "SupplementalCreds"]), offset=0),
    "SpGetTbalSupplementalCredsFn": SimTypePointer(SimTypeFunction([SimTypeRef("LUID", SimStruct), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "SupplementalCredsSize", "SupplementalCreds"]), offset=0),
    "SpGetUserInfoFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LUID", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("SECURITY_USER_DATA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LogonId", "Flags", "UserData"]), offset=0),
    "SpImportSecurityContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPackedContext", "Token", "phContext"]), offset=0),
    "SpInitLsaModeContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "ContextHandle", "TargetName", "ContextRequirements", "TargetDataRep", "InputBuffers", "NewContextHandle", "OutputBuffers", "ContextAttributes", "ExpirationTime", "MappedContext", "ContextData"]), offset=0),
    "SpInitUserModeContextFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "PackedContext"]), offset=0),
    "SpInitializeFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SECPKG_PARAMETERS", SimStruct), offset=0), SimTypePointer(SimTypeRef("LSA_SECPKG_FUNCTION_TABLE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PackageId", "Parameters", "FunctionTable"]), offset=0),
    "SpInstanceInitFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_DLL_FUNCTIONS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Version", "FunctionTable", "UserFunctions"]), offset=0),
    "SpLsaModeInitializeFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_FUNCTION_TABLE", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LsaVersion", "PackageVersion", "ppTables", "pcTables"]), offset=0),
    "SpMakeSignatureFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "QualityOfProtection", "MessageBuffers", "MessageSequenceNumber"]), offset=0),
    "SpMarshalAttributeDataFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AttributeInfo", "Attribute", "AttributeDataSize", "AttributeData", "MarshaledAttributeDataSize", "MarshaledAttributeData"]), offset=0),
    "SpMarshallSupplementalCredsFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialSize", "Credentials", "MarshalledCredSize", "MarshalledCreds"]), offset=0),
    "SpQueryContextAttributesFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ContextAttribute", "Buffer"]), offset=0),
    "SpQueryCredentialsAttributesFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "CredentialAttribute", "Buffer"]), offset=0),
    "SpQueryMetaDataFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "TargetName", "ContextRequirements", "MetaDataLength", "MetaData", "ContextHandle"]), offset=0),
    "SpSaveCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBuffer", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "Credentials"]), offset=0),
    "SpSealMessageFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "QualityOfProtection", "MessageBuffers", "MessageSequenceNumber"]), offset=0),
    "SpSetContextAttributesFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "ContextAttribute", "Buffer", "BufferSize"]), offset=0),
    "SpSetCredentialsAttributesFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["CredentialHandle", "CredentialAttribute", "Buffer", "BufferSize"]), offset=0),
    "SpSetExtendedInformationFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="SECPKG_EXTENDED_INFORMATION_CLASS"), SimTypePointer(SimTypeRef("SECPKG_EXTENDED_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Class", "Info"]), offset=0),
    "SpShutdownFn": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "SpUnsealMessageFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "MessageBuffers", "MessageSequenceNumber", "QualityOfProtection"]), offset=0),
    "SpUpdateCredentialsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "CredType", "FlatCredUIContextLength", "FlatCredUIContext"]), offset=0),
    "SpUserModeInitializeFn": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("SECPKG_USER_FUNCTION_TABLE", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LsaVersion", "PackageVersion", "ppTables", "pcTables"]), offset=0),
    "SpValidateTargetInfoFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SECPKG_TARGETINFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientRequest", "ProtocolSubmitBuffer", "ClientBufferBase", "SubmitBufferLength", "TargetInfo"]), offset=0),
    "SpVerifySignatureFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ContextHandle", "MessageBuffers", "MessageSequenceNumber", "QualityOfProtection"]), offset=0),
    "SpanningEventDescriptor": SimStruct(OrderedDict((("wDataLen", SimTypeShort(signed=False, label="UInt16")), ("wProgNumber", SimTypeShort(signed=False, label="UInt16")), ("wSID", SimTypeShort(signed=False, label="UInt16")), ("bDescriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="SpanningEventDescriptor", pack=False, align=None),
    "SpanningEventEmmMessage": SimStruct(OrderedDict((("bCAbroadcasterGroupId", SimTypeChar(label="Byte")), ("bMessageControl", SimTypeChar(label="Byte")), ("wServiceId", SimTypeShort(signed=False, label="UInt16")), ("wTableIdExtension", SimTypeShort(signed=False, label="UInt16")), ("bDeletionStatus", SimTypeChar(label="Byte")), ("bDisplayingDuration1", SimTypeChar(label="Byte")), ("bDisplayingDuration2", SimTypeChar(label="Byte")), ("bDisplayingDuration3", SimTypeChar(label="Byte")), ("bDisplayingCycle", SimTypeChar(label="Byte")), ("bFormatVersion", SimTypeChar(label="Byte")), ("bDisplayPosition", SimTypeChar(label="Byte")), ("wMessageLength", SimTypeShort(signed=False, label="UInt16")), ("szMessageArea", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="SpanningEventEmmMessage", pack=False, align=None),
    "SpatialAudioClientActivationParams": SimStruct(OrderedDict((("tracingContextId", SimTypeBottom(label="Guid")), ("appId", SimTypeBottom(label="Guid")), ("majorVersion", SimTypeInt(signed=True, label="Int32")), ("minorVersion1", SimTypeInt(signed=True, label="Int32")), ("minorVersion2", SimTypeInt(signed=True, label="Int32")), ("minorVersion3", SimTypeInt(signed=True, label="Int32")),)), name="SpatialAudioClientActivationParams", pack=False, align=None),
    "SpatialAudioHrtfActivationParams": SimStruct(OrderedDict((("ObjectFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("StaticObjectTypeMask", SimTypeInt(signed=False, label="AudioObjectType")), ("MinDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("Category", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NotifyObject", SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify")), ("DistanceDecay", SimTypePointer(SimTypeRef("SpatialAudioHrtfDistanceDecay", SimStruct), offset=0)), ("Directivity", SimTypePointer(SimUnion({"Cone": SimTypeRef("SpatialAudioHrtfDirectivityCone", SimStruct), "Cardiod": SimTypeRef("SpatialAudioHrtfDirectivityCardioid", SimStruct), "Omni": SimTypeRef("SpatialAudioHrtfDirectivity", SimStruct)}, name="<anon>", label="None"), offset=0)), ("Environment", SimTypePointer(SimTypeInt(signed=False, label="SpatialAudioHrtfEnvironmentType"), offset=0)), ("Orientation", SimTypePointer(SimTypeFloat(size=32), offset=0)),)), name="SpatialAudioHrtfActivationParams", pack=False, align=None),
    "SpatialAudioHrtfActivationParams2": SimStruct(OrderedDict((("ObjectFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("StaticObjectTypeMask", SimTypeInt(signed=False, label="AudioObjectType")), ("MinDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("Category", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NotifyObject", SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify")), ("DistanceDecay", SimTypePointer(SimTypeRef("SpatialAudioHrtfDistanceDecay", SimStruct), offset=0)), ("Directivity", SimTypePointer(SimUnion({"Cone": SimTypeRef("SpatialAudioHrtfDirectivityCone", SimStruct), "Cardiod": SimTypeRef("SpatialAudioHrtfDirectivityCardioid", SimStruct), "Omni": SimTypeRef("SpatialAudioHrtfDirectivity", SimStruct)}, name="<anon>", label="None"), offset=0)), ("Environment", SimTypePointer(SimTypeInt(signed=False, label="SpatialAudioHrtfEnvironmentType"), offset=0)), ("Orientation", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("Options", SimTypeInt(signed=False, label="SPATIAL_AUDIO_STREAM_OPTIONS")),)), name="SpatialAudioHrtfActivationParams2", pack=False, align=None),
    "SpatialAudioHrtfDirectivity": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="SpatialAudioHrtfDirectivityType")), ("Scaling", SimTypeFloat(size=32)),)), name="SpatialAudioHrtfDirectivity", pack=False, align=None),
    "SpatialAudioHrtfDirectivityCardioid": SimStruct(OrderedDict((("directivity", SimTypeRef("SpatialAudioHrtfDirectivity", SimStruct)), ("Order", SimTypeFloat(size=32)),)), name="SpatialAudioHrtfDirectivityCardioid", pack=False, align=None),
    "SpatialAudioHrtfDirectivityCone": SimStruct(OrderedDict((("directivity", SimTypeRef("SpatialAudioHrtfDirectivity", SimStruct)), ("InnerAngle", SimTypeFloat(size=32)), ("OuterAngle", SimTypeFloat(size=32)),)), name="SpatialAudioHrtfDirectivityCone", pack=False, align=None),
    "SpatialAudioHrtfDirectivityType": SimTypeInt(signed=False, label="SpatialAudioHrtfDirectivityType"),
    "SpatialAudioHrtfDirectivityUnion": SimUnion({"Cone": SimTypeRef("SpatialAudioHrtfDirectivityCone", SimStruct), "Cardiod": SimTypeRef("SpatialAudioHrtfDirectivityCardioid", SimStruct), "Omni": SimTypeRef("SpatialAudioHrtfDirectivity", SimStruct)}, name="<anon>", label="None"),
    "SpatialAudioHrtfDistanceDecay": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="SpatialAudioHrtfDistanceDecayType")), ("MaxGain", SimTypeFloat(size=32)), ("MinGain", SimTypeFloat(size=32)), ("UnityGainDistance", SimTypeFloat(size=32)), ("CutoffDistance", SimTypeFloat(size=32)),)), name="SpatialAudioHrtfDistanceDecay", pack=False, align=None),
    "SpatialAudioHrtfDistanceDecayType": SimTypeInt(signed=False, label="SpatialAudioHrtfDistanceDecayType"),
    "SpatialAudioHrtfEnvironmentType": SimTypeInt(signed=False, label="SpatialAudioHrtfEnvironmentType"),
    "SpatialAudioMetadataCopyMode": SimTypeInt(signed=False, label="SpatialAudioMetadataCopyMode"),
    "SpatialAudioMetadataItemsInfo": SimStruct(OrderedDict((("FrameCount", SimTypeShort(signed=False, label="UInt16")), ("ItemCount", SimTypeShort(signed=False, label="UInt16")), ("MaxItemCount", SimTypeShort(signed=False, label="UInt16")), ("MaxValueBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="SpatialAudioMetadataItemsInfo", pack=False, align=None),
    "SpatialAudioMetadataWriterOverflowMode": SimTypeInt(signed=False, label="SpatialAudioMetadataWriterOverflowMode"),
    "SpatialAudioObjectRenderStreamActivationParams": SimStruct(OrderedDict((("ObjectFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("StaticObjectTypeMask", SimTypeInt(signed=False, label="AudioObjectType")), ("MinDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("Category", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NotifyObject", SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify")),)), name="SpatialAudioObjectRenderStreamActivationParams", pack=False, align=None),
    "SpatialAudioObjectRenderStreamActivationParams2": SimStruct(OrderedDict((("ObjectFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("StaticObjectTypeMask", SimTypeInt(signed=False, label="AudioObjectType")), ("MinDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("Category", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("NotifyObject", SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify")), ("Options", SimTypeInt(signed=False, label="SPATIAL_AUDIO_STREAM_OPTIONS")),)), name="SpatialAudioObjectRenderStreamActivationParams2", pack=False, align=None),
    "SpatialAudioObjectRenderStreamForMetadataActivationParams": SimStruct(OrderedDict((("ObjectFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("StaticObjectTypeMask", SimTypeInt(signed=False, label="AudioObjectType")), ("MinDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("Category", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("MetadataFormatId", SimTypeBottom(label="Guid")), ("MaxMetadataItemCount", SimTypeShort(signed=False, label="UInt16")), ("MetadataActivationParams", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("NotifyObject", SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify")),)), name="SpatialAudioObjectRenderStreamForMetadataActivationParams", pack=False, align=None),
    "SpatialAudioObjectRenderStreamForMetadataActivationParams2": SimStruct(OrderedDict((("ObjectFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("StaticObjectTypeMask", SimTypeInt(signed=False, label="AudioObjectType")), ("MinDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("MaxDynamicObjectCount", SimTypeInt(signed=False, label="UInt32")), ("Category", SimTypeInt(signed=False, label="AUDIO_STREAM_CATEGORY")), ("EventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("MetadataFormatId", SimTypeBottom(label="Guid")), ("MaxMetadataItemCount", SimTypeInt(signed=False, label="UInt32")), ("MetadataActivationParams", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)), ("NotifyObject", SimTypeBottom(label="ISpatialAudioObjectRenderStreamNotify")), ("Options", SimTypeInt(signed=False, label="SPATIAL_AUDIO_STREAM_OPTIONS")),)), name="SpatialAudioObjectRenderStreamForMetadataActivationParams2", pack=False, align=None),
    "SpectralInversion": SimTypeInt(signed=False, label="SpectralInversion"),
    "SpeechAudioFormatType": SimTypeInt(signed=False, label="SpeechAudioFormatType"),
    "SpeechAudioState": SimTypeInt(signed=False, label="SpeechAudioState"),
    "SpeechBookmarkOptions": SimTypeInt(signed=False, label="SpeechBookmarkOptions"),
    "SpeechDataKeyLocation": SimTypeInt(signed=False, label="SpeechDataKeyLocation"),
    "SpeechDiscardType": SimTypeInt(signed=False, label="SpeechDiscardType"),
    "SpeechDisplayAttributes": SimTypeInt(signed=False, label="SpeechDisplayAttributes"),
    "SpeechEmulationCompareFlags": SimTypeInt(signed=False, label="SpeechEmulationCompareFlags"),
    "SpeechEngineConfidence": SimTypeInt(signed=False, label="SpeechEngineConfidence"),
    "SpeechFormatType": SimTypeInt(signed=False, label="SpeechFormatType"),
    "SpeechGrammarRuleStateTransitionType": SimTypeInt(signed=False, label="SpeechGrammarRuleStateTransitionType"),
    "SpeechGrammarState": SimTypeInt(signed=False, label="SpeechGrammarState"),
    "SpeechGrammarWordType": SimTypeInt(signed=False, label="SpeechGrammarWordType"),
    "SpeechInterference": SimTypeInt(signed=False, label="SpeechInterference"),
    "SpeechLexiconType": SimTypeInt(signed=False, label="SpeechLexiconType"),
    "SpeechLoadOption": SimTypeInt(signed=False, label="SpeechLoadOption"),
    "SpeechPartOfSpeech": SimTypeInt(signed=False, label="SpeechPartOfSpeech"),
    "SpeechRecoContextState": SimTypeInt(signed=False, label="SpeechRecoContextState"),
    "SpeechRecoEvents": SimTypeInt(signed=False, label="SpeechRecoEvents"),
    "SpeechRecognitionType": SimTypeInt(signed=False, label="SpeechRecognitionType"),
    "SpeechRecognizerState": SimTypeInt(signed=False, label="SpeechRecognizerState"),
    "SpeechRetainedAudioOptions": SimTypeInt(signed=False, label="SpeechRetainedAudioOptions"),
    "SpeechRuleAttributes": SimTypeInt(signed=False, label="SpeechRuleAttributes"),
    "SpeechRuleState": SimTypeInt(signed=False, label="SpeechRuleState"),
    "SpeechRunState": SimTypeInt(signed=False, label="SpeechRunState"),
    "SpeechSpecialTransitionType": SimTypeInt(signed=False, label="SpeechSpecialTransitionType"),
    "SpeechStreamFileMode": SimTypeInt(signed=False, label="SpeechStreamFileMode"),
    "SpeechStreamSeekPositionType": SimTypeInt(signed=False, label="SpeechStreamSeekPositionType"),
    "SpeechTokenContext": SimTypeInt(signed=False, label="SpeechTokenContext"),
    "SpeechTokenShellFolder": SimTypeInt(signed=False, label="SpeechTokenShellFolder"),
    "SpeechVisemeFeature": SimTypeInt(signed=False, label="SpeechVisemeFeature"),
    "SpeechVisemeType": SimTypeInt(signed=False, label="SpeechVisemeType"),
    "SpeechVoiceEvents": SimTypeInt(signed=False, label="SpeechVoiceEvents"),
    "SpeechVoicePriority": SimTypeInt(signed=False, label="SpeechVoicePriority"),
    "SpeechVoiceSpeakFlags": SimTypeInt(signed=False, label="SpeechVoiceSpeakFlags"),
    "SpeechWordPronounceable": SimTypeInt(signed=False, label="SpeechWordPronounceable"),
    "SpeechWordType": SimTypeInt(signed=False, label="SpeechWordType"),
    "SslDeserializeCertificateStoreFn": SimTypePointer(SimTypeFunction([SimTypeRef("CRYPT_INTEGER_BLOB", SimStruct), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SerializedCertificateStore", "ppCertContext"]), offset=0),
    "SslGetExtensionsFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SCH_EXTENSION_DATA", SimStruct), label="LPArray", offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="SchGetExtensionsOptions")], SimTypeInt(signed=True, label="Int32"), arg_names=["clientHello", "clientHelloByteSize", "genericExtensions", "genericExtensionsCount", "bytesToRead", "flags"]), offset=0),
    "SslGetServerIdentityFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ClientHello", "ClientHelloSize", "ServerIdentity", "ServerIdentitySize", "Flags"]), offset=0),
    "SspiAsyncContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "SspiAsyncNotifyCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Handle", "CallbackData"]), offset=0),
    "StackOverflowInfo": SimStruct(OrderedDict((("soType", SimTypeInt(signed=False, label="StackOverflowType")), ("pExceptionInfo", SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0)),)), name="StackOverflowInfo", pack=False, align=None),
    "StackOverflowType": SimTypeInt(signed=False, label="StackOverflowType"),
    "StackSnapshotCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["funcId", "ip", "frameInfo", "contextSize", "context", "clientData"]), offset=0),
    "StartParam": SimStruct(OrderedDict((("iid", SimTypeBottom(label="Guid")), ("pIBindCtx", SimTypeBottom(label="IBindCtx")), ("pItf", SimTypeBottom(label="IUnknown")),)), name="StartParam", pack=False, align=None),
    "Status": SimTypeInt(signed=False, label="Status"),
    "StatusCode": SimTypeInt(signed=False, label="StatusCode"),
    "StorageLayout": SimStruct(OrderedDict((("LayoutType", SimTypeInt(signed=False, label="UInt32")), ("pwcsElementName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cOffset", SimTypeLongLong(signed=True, label="Int64")), ("cBytes", SimTypeLongLong(signed=True, label="Int64")),)), name="StorageLayout", pack=False, align=None),
    "StreamMode": SimTypeInt(signed=False, label="StreamMode"),
    "StringAlignment": SimTypeInt(signed=False, label="StringAlignment"),
    "StringDigitSubstitute": SimTypeInt(signed=False, label="StringDigitSubstitute"),
    "StringFormatFlags": SimTypeInt(signed=False, label="StringFormatFlags"),
    "StringTrimming": SimTypeInt(signed=False, label="StringTrimming"),
    "StructureChangeType": SimTypeInt(signed=False, label="StructureChangeType"),
    "StylusInfo": SimStruct(OrderedDict((("tcid", SimTypeInt(signed=False, label="UInt32")), ("cid", SimTypeInt(signed=False, label="UInt32")), ("bIsInvertedCursor", SimTypeInt(signed=True, label="Int32")),)), name="StylusInfo", pack=False, align=None),
    "StylusQueue": SimTypeInt(signed=False, label="StylusQueue"),
    "SupportedTextSelection": SimTypeInt(signed=False, label="SupportedTextSelection"),
    "SymbolKind": SimTypeInt(signed=False, label="SymbolKind"),
    "SymbolSearchOptions": SimTypeInt(signed=False, label="SymbolSearchOptions"),
    "SyncProviderConfigUIConfiguration": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("guidInstanceId", SimTypeBottom(label="Guid")), ("clsidConfigUI", SimTypeBottom(label="Guid")), ("guidContentType", SimTypeBottom(label="Guid")), ("dwCapabilities", SimTypeInt(signed=False, label="UInt32")), ("dwSupportedArchitecture", SimTypeInt(signed=False, label="UInt32")), ("fIsGlobal", SimTypeInt(signed=True, label="Int32")),)), name="SyncProviderConfigUIConfiguration", pack=False, align=None),
    "SyncProviderConfiguration": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("guidInstanceId", SimTypeBottom(label="Guid")), ("clsidProvider", SimTypeBottom(label="Guid")), ("guidConfigUIInstanceId", SimTypeBottom(label="Guid")), ("guidContentType", SimTypeBottom(label="Guid")), ("dwCapabilities", SimTypeInt(signed=False, label="UInt32")), ("dwSupportedArchitecture", SimTypeInt(signed=False, label="UInt32")),)), name="SyncProviderConfiguration", pack=False, align=None),
    "SynchronizedInputType": SimTypeInt(signed=False, label="SynchronizedInputType"),
    "SysmonBatchReason": SimTypeInt(signed=False, label="SysmonBatchReason"),
    "SysmonDataType": SimTypeInt(signed=False, label="SysmonDataType"),
    "SysmonFileType": SimTypeInt(signed=False, label="SysmonFileType"),
    "SystemHealthAgentState": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="UInt32")), ("shaResultCodes", SimTypeRef("ResultCodes", SimStruct)), ("failureCategory", SimTypeInt(signed=False, label="FailureCategory")), ("fixupInfo", SimTypeRef("FixupInfo", SimStruct)),)), name="SystemHealthAgentState", pack=False, align=None),
    "SystemInterruptTime": SimStruct(OrderedDict((("value", SimTypeLongLong(signed=False, label="UInt64")),)), name="SystemInterruptTime", pack=False, align=None),
    "SystemVisibilityFlags": SimTypeInt(signed=False, label="SystemVisibilityFlags"),
    "TABITEMBOTHEDGESTATES": SimTypeInt(signed=False, label="TABITEMBOTHEDGESTATES"),
    "TABITEMLEFTEDGESTATES": SimTypeInt(signed=False, label="TABITEMLEFTEDGESTATES"),
    "TABITEMRIGHTEDGESTATES": SimTypeInt(signed=False, label="TABITEMRIGHTEDGESTATES"),
    "TABITEMSTATES": SimTypeInt(signed=False, label="TABITEMSTATES"),
    "TABLECELLPARMS": SimStruct(OrderedDict((("dxWidth", SimTypeInt(signed=True, label="Int32")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("wShading", SimTypeShort(signed=False, label="UInt16")), ("dxBrdrLeft", SimTypeShort(signed=True, label="Int16")), ("dyBrdrTop", SimTypeShort(signed=True, label="Int16")), ("dxBrdrRight", SimTypeShort(signed=True, label="Int16")), ("dyBrdrBottom", SimTypeShort(signed=True, label="Int16")), ("crBrdrLeft", SimTypeInt(signed=False, label="UInt32")), ("crBrdrTop", SimTypeInt(signed=False, label="UInt32")), ("crBrdrRight", SimTypeInt(signed=False, label="UInt32")), ("crBrdrBottom", SimTypeInt(signed=False, label="UInt32")), ("crBackPat", SimTypeInt(signed=False, label="UInt32")), ("crForePat", SimTypeInt(signed=False, label="UInt32")),)), name="TABLECELLPARMS", pack=False, align=None),
    "TABLEROWPARMS": SimStruct(OrderedDict((("cbRow", SimTypeChar(label="Byte")), ("cbCell", SimTypeChar(label="Byte")), ("cCell", SimTypeChar(label="Byte")), ("cRow", SimTypeChar(label="Byte")), ("dxCellMargin", SimTypeInt(signed=True, label="Int32")), ("dxIndent", SimTypeInt(signed=True, label="Int32")), ("dyHeight", SimTypeInt(signed=True, label="Int32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("cpStartRow", SimTypeInt(signed=True, label="Int32")), ("bTableLevel", SimTypeChar(label="Byte")), ("iCell", SimTypeChar(label="Byte")),)), name="TABLEROWPARMS", pack=False, align=None),
    "TABLE_NOTIFICATION": SimStruct(OrderedDict((("ulTableEvent", SimTypeInt(signed=False, label="UInt32")), ("hResult", SimTypeInt(signed=True, label="Int32")), ("propIndex", SimTypeRef("SPropValue", SimStruct)), ("propPrior", SimTypeRef("SPropValue", SimStruct)), ("row", SimTypeRef("SRow", SimStruct)), ("ulPad", SimTypeInt(signed=False, label="UInt32")),)), name="TABLE_NOTIFICATION", pack=False, align=None),
    "TABLE_SEARCH_RESULT": SimTypeInt(signed=False, label="TABLE_SEARCH_RESULT"),
    "TABPARTS": SimTypeInt(signed=False, label="TABPARTS"),
    "TABSTATES": SimTypeInt(signed=False, label="TABSTATES"),
    "TAB_CONTROL_ITEM_STATE": SimTypeInt(signed=False, label="TAB_CONTROL_ITEM_STATE"),
    "TAG_TYPE": SimStruct(OrderedDict((("pwszTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwRequired", SimTypeInt(signed=False, label="UInt32")), ("bPresent", SimTypeInt(signed=True, label="Int32")),)), name="TAG_TYPE", pack=False, align=None),
    "TAKE_SNAPSHOT_VHDSET_FLAG": SimTypeInt(signed=False, label="TAKE_SNAPSHOT_VHDSET_FLAG"),
    "TAKE_SNAPSHOT_VHDSET_PARAMETERS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="TAKE_SNAPSHOT_VHDSET_VERSION")), ("Anonymous", SimUnion({"Version1": SimTypeRef("_Version1_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="TAKE_SNAPSHOT_VHDSET_PARAMETERS", pack=False, align=None),
    "TAKE_SNAPSHOT_VHDSET_VERSION": SimTypeInt(signed=False, label="TAKE_SNAPSHOT_VHDSET_VERSION"),
    "TANALYZE_RETURN": SimTypeInt(signed=False, label="TANALYZE_RETURN"),
    "TAPEMARK_TYPE": SimTypeInt(signed=False, label="TAPEMARK_TYPE"),
    "TAPE_CREATE_PARTITION": SimStruct(OrderedDict((("Method", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="TAPE_CREATE_PARTITION", pack=False, align=None),
    "TAPE_DRIVE_PROBLEM_TYPE": SimTypeInt(signed=False, label="TAPE_DRIVE_PROBLEM_TYPE"),
    "TAPE_ERASE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="ERASE_TAPE_TYPE")), ("Immediate", SimTypeChar(label="Byte")),)), name="TAPE_ERASE", pack=False, align=None),
    "TAPE_GET_DRIVE_PARAMETERS": SimStruct(OrderedDict((("ECC", SimTypeChar(label="Byte")), ("Compression", SimTypeChar(label="Byte")), ("DataPadding", SimTypeChar(label="Byte")), ("ReportSetmarks", SimTypeChar(label="Byte")), ("DefaultBlockSize", SimTypeInt(signed=False, label="UInt32")), ("MaximumBlockSize", SimTypeInt(signed=False, label="UInt32")), ("MinimumBlockSize", SimTypeInt(signed=False, label="UInt32")), ("MaximumPartitionCount", SimTypeInt(signed=False, label="UInt32")), ("FeaturesLow", SimTypeInt(signed=False, label="UInt32")), ("FeaturesHigh", SimTypeInt(signed=False, label="TAPE_GET_DRIVE_PARAMETERS_FEATURES_HIGH")), ("EOTWarningZoneSize", SimTypeInt(signed=False, label="UInt32")),)), name="TAPE_GET_DRIVE_PARAMETERS", pack=False, align=None),
    "TAPE_GET_DRIVE_PARAMETERS_FEATURES_HIGH": SimTypeInt(signed=False, label="TAPE_GET_DRIVE_PARAMETERS_FEATURES_HIGH"),
    "TAPE_GET_MEDIA_PARAMETERS": SimStruct(OrderedDict((("Capacity", SimTypeLongLong(signed=True, label="Int64")), ("Remaining", SimTypeLongLong(signed=True, label="Int64")), ("BlockSize", SimTypeInt(signed=False, label="UInt32")), ("PartitionCount", SimTypeInt(signed=False, label="UInt32")), ("WriteProtected", SimTypeChar(label="Byte")),)), name="TAPE_GET_MEDIA_PARAMETERS", pack=False, align=None),
    "TAPE_GET_POSITION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="TAPE_POSITION_TYPE")), ("Partition", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=True, label="Int64")),)), name="TAPE_GET_POSITION", pack=False, align=None),
    "TAPE_GET_STATISTICS": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="UInt32")),)), name="TAPE_GET_STATISTICS", pack=False, align=None),
    "TAPE_INFORMATION_TYPE": SimTypeInt(signed=False, label="TAPE_INFORMATION_TYPE"),
    "TAPE_POSITION_METHOD": SimTypeInt(signed=False, label="TAPE_POSITION_METHOD"),
    "TAPE_POSITION_TYPE": SimTypeInt(signed=False, label="TAPE_POSITION_TYPE"),
    "TAPE_PREPARE": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="PREPARE_TAPE_OPERATION")), ("Immediate", SimTypeChar(label="Byte")),)), name="TAPE_PREPARE", pack=False, align=None),
    "TAPE_SET_DRIVE_PARAMETERS": SimStruct(OrderedDict((("ECC", SimTypeChar(label="Byte")), ("Compression", SimTypeChar(label="Byte")), ("DataPadding", SimTypeChar(label="Byte")), ("ReportSetmarks", SimTypeChar(label="Byte")), ("EOTWarningZoneSize", SimTypeInt(signed=False, label="UInt32")),)), name="TAPE_SET_DRIVE_PARAMETERS", pack=False, align=None),
    "TAPE_SET_MEDIA_PARAMETERS": SimStruct(OrderedDict((("BlockSize", SimTypeInt(signed=False, label="UInt32")),)), name="TAPE_SET_MEDIA_PARAMETERS", pack=False, align=None),
    "TAPE_SET_POSITION": SimStruct(OrderedDict((("Method", SimTypeInt(signed=False, label="TAPE_POSITION_METHOD")), ("Partition", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Immediate", SimTypeChar(label="Byte")),)), name="TAPE_SET_POSITION", pack=False, align=None),
    "TAPE_STATISTICS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("RecoveredWrites", SimTypeLongLong(signed=True, label="Int64")), ("UnrecoveredWrites", SimTypeLongLong(signed=True, label="Int64")), ("RecoveredReads", SimTypeLongLong(signed=True, label="Int64")), ("UnrecoveredReads", SimTypeLongLong(signed=True, label="Int64")), ("CompressionRatioReads", SimTypeChar(label="Byte")), ("CompressionRatioWrites", SimTypeChar(label="Byte")),)), name="TAPE_STATISTICS", pack=False, align=None),
    "TAPE_WMI_OPERATIONS": SimStruct(OrderedDict((("Method", SimTypeInt(signed=False, label="UInt32")), ("DataBufferSize", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TAPE_WMI_OPERATIONS", pack=False, align=None),
    "TAPE_WRITE_MARKS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="TAPEMARK_TYPE")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Immediate", SimTypeChar(label="Byte")),)), name="TAPE_WRITE_MARKS", pack=False, align=None),
    "TAPIOBJECT_EVENT": SimTypeInt(signed=False, label="TAPIOBJECT_EVENT"),
    "TAPI_CUSTOMTONE": SimStruct(OrderedDict((("dwFrequency", SimTypeInt(signed=False, label="UInt32")), ("dwCadenceOn", SimTypeInt(signed=False, label="UInt32")), ("dwCadenceOff", SimTypeInt(signed=False, label="UInt32")), ("dwVolume", SimTypeInt(signed=False, label="UInt32")),)), name="TAPI_CUSTOMTONE", pack=False, align=None),
    "TAPI_DETECTTONE": SimStruct(OrderedDict((("dwAppSpecific", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency1", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency2", SimTypeInt(signed=False, label="UInt32")), ("dwFrequency3", SimTypeInt(signed=False, label="UInt32")),)), name="TAPI_DETECTTONE", pack=False, align=None),
    "TAPI_EVENT": SimTypeInt(signed=False, label="TAPI_EVENT"),
    "TAPI_GATHERTERM": SimTypeInt(signed=False, label="TAPI_GATHERTERM"),
    "TAPI_OBJECT_TYPE": SimTypeInt(signed=False, label="TAPI_OBJECT_TYPE"),
    "TAPI_TONEMODE": SimTypeInt(signed=False, label="TAPI_TONEMODE"),
    "TAP_PARAMETER": SimTypeInt(signed=False, label="TAP_PARAMETER"),
    "TARGETPROTOCOLTYPE": SimTypeInt(signed=False, label="TARGETPROTOCOLTYPE"),
    "TARGET_CHANGE_TYPE": SimTypeInt(signed=False, label="TARGET_CHANGE_TYPE"),
    "TARGET_DEBUG_INFO": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("EntryDate", SimTypeLongLong(signed=False, label="UInt64")), ("DebugeeClass", SimTypeInt(signed=False, label="UInt32")), ("SysUpTime", SimTypeLongLong(signed=False, label="UInt64")), ("AppUpTime", SimTypeLongLong(signed=False, label="UInt64")), ("CrashTime", SimTypeLongLong(signed=False, label="UInt64")), ("OsInfo", SimTypeRef("OS_INFO", SimStruct)), ("Cpu", SimTypeRef("CPU_INFO", SimStruct)), ("DumpFile", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="TARGET_DEBUG_INFO", pack=False, align=None),
    "TARGET_DEBUG_INFO_v1": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeLongLong(signed=False, label="UInt64")), ("Source", SimTypeLongLong(signed=False, label="UInt64")), ("EntryDate", SimTypeLongLong(signed=False, label="UInt64")), ("SysUpTime", SimTypeLongLong(signed=False, label="UInt64")), ("AppUpTime", SimTypeLongLong(signed=False, label="UInt64")), ("CrashTime", SimTypeLongLong(signed=False, label="UInt64")), ("Mode", SimTypeLongLong(signed=False, label="UInt64")), ("OsInfo", SimTypeRef("OS_INFO_v1", SimStruct)), ("Cpu", SimTypeRef("CPU_INFO_v1", SimStruct)), ("DumpFile", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("FailureData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StackTr", SimTypeArray(SimTypeChar(label="SByte"), 4096)),)), name="TARGET_DEBUG_INFO_v1", pack=False, align=None),
    "TARGET_DEBUG_INFO_v2": SimStruct(OrderedDict((("SizeOfStruct", SimTypeInt(signed=False, label="UInt32")), ("EntryDate", SimTypeLongLong(signed=False, label="UInt64")), ("DebugeeClass", SimTypeInt(signed=False, label="UInt32")), ("SysUpTime", SimTypeLongLong(signed=False, label="UInt64")), ("AppUpTime", SimTypeLongLong(signed=False, label="UInt64")), ("CrashTime", SimTypeLongLong(signed=False, label="UInt64")), ("OsInfo", SimTypeRef("OS_INFO", SimStruct)), ("Cpu", SimTypeRef("CPU_INFO_v2", SimStruct)), ("DumpFile", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="TARGET_DEBUG_INFO_v2", pack=False, align=None),
    "TARGET_DEVICE_CUSTOM_NOTIFICATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Event", SimTypeBottom(label="Guid")), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("NameBufferOffset", SimTypeInt(signed=True, label="Int32")), ("CustomDataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TARGET_DEVICE_CUSTOM_NOTIFICATION", pack=False, align=None),
    "TARGET_DEVICE_REMOVAL_NOTIFICATION": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Event", SimTypeBottom(label="Guid")), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)),)), name="TARGET_DEVICE_REMOVAL_NOTIFICATION", pack=False, align=None),
    "TARGET_INFORMATION_CLASS": SimTypeInt(signed=False, label="TARGET_INFORMATION_CLASS"),
    "TARGET_OWNER": SimTypeInt(signed=False, label="TARGET_OWNER"),
    "TARGET_PATCH_STATE": SimTypeInt(signed=False, label="TARGET_PATCH_STATE"),
    "TARGET_STATE": SimTypeInt(signed=False, label="TARGET_STATE"),
    "TARGET_TYPE": SimTypeInt(signed=False, label="TARGET_TYPE"),
    "TASKBANDPARTS": SimTypeInt(signed=False, label="TASKBANDPARTS"),
    "TASKBARPARTS": SimTypeInt(signed=False, label="TASKBARPARTS"),
    "TASKDIALOGCONFIG": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="TASKDIALOG_FLAGS")), ("dwCommonButtons", SimTypeInt(signed=False, label="TASKDIALOG_COMMON_BUTTON_FLAGS")), ("pszWindowTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous1", SimUnion({"hMainIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszMainIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszMainInstruction", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszContent", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cButtons", SimTypeInt(signed=False, label="UInt32")), ("pButtons", SimTypePointer(SimTypeRef("TASKDIALOG_BUTTON", SimStruct), offset=0)), ("nDefaultButton", SimTypeInt(signed=True, label="Int32")), ("cRadioButtons", SimTypeInt(signed=False, label="UInt32")), ("pRadioButtons", SimTypePointer(SimTypeRef("TASKDIALOG_BUTTON", SimStruct), offset=0)), ("nDefaultRadioButton", SimTypeInt(signed=True, label="Int32")), ("pszVerificationText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszExpandedInformation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszExpandedControlText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCollapsedControlText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous2", SimUnion({"hFooterIcon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszFooterIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")), ("pszFooter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pfCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwnd", "msg", "wParam", "lParam", "lpRefData"]), offset=0)), ("lpCallbackData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("cxWidth", SimTypeInt(signed=False, label="UInt32")),)), name="TASKDIALOGCONFIG", pack=False, align=None),
    "TASKDIALOGPARTS": SimTypeInt(signed=False, label="TASKDIALOGPARTS"),
    "TASKDIALOG_BUTTON": SimStruct(OrderedDict((("nButtonID", SimTypeInt(signed=True, label="Int32")), ("pszButtonText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TASKDIALOG_BUTTON", pack=False, align=None),
    "TASKDIALOG_COMMON_BUTTON_FLAGS": SimTypeInt(signed=False, label="TASKDIALOG_COMMON_BUTTON_FLAGS"),
    "TASKDIALOG_ELEMENTS": SimTypeInt(signed=False, label="TASKDIALOG_ELEMENTS"),
    "TASKDIALOG_FLAGS": SimTypeInt(signed=False, label="TASKDIALOG_FLAGS"),
    "TASKDIALOG_ICON_ELEMENTS": SimTypeInt(signed=False, label="TASKDIALOG_ICON_ELEMENTS"),
    "TASKDIALOG_MESSAGES": SimTypeInt(signed=False, label="TASKDIALOG_MESSAGES"),
    "TASKDIALOG_NOTIFICATIONS": SimTypeInt(signed=False, label="TASKDIALOG_NOTIFICATIONS"),
    "TASKENUMPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwThreadId", "hMod16", "hTask16", "lpUserDefined"]), offset=0),
    "TASKENUMPROCEX": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwThreadId", "hMod16", "hTask16", "pszModName", "pszFileName", "lpUserDefined"]), offset=0),
    "TASKLINKSTATES": SimTypeInt(signed=False, label="TASKLINKSTATES"),
    "TASKPAGE": SimTypeInt(signed=False, label="TASKPAGE"),
    "TASK_ACTION_TYPE": SimTypeInt(signed=False, label="TASK_ACTION_TYPE"),
    "TASK_COMPATIBILITY": SimTypeInt(signed=False, label="TASK_COMPATIBILITY"),
    "TASK_CREATION": SimTypeInt(signed=False, label="TASK_CREATION"),
    "TASK_ENUM_FLAGS": SimTypeInt(signed=False, label="TASK_ENUM_FLAGS"),
    "TASK_INSTANCES_POLICY": SimTypeInt(signed=False, label="TASK_INSTANCES_POLICY"),
    "TASK_LOGON_TYPE": SimTypeInt(signed=False, label="TASK_LOGON_TYPE"),
    "TASK_PROCESSTOKENSID_TYPE": SimTypeInt(signed=False, label="TASK_PROCESSTOKENSID_TYPE"),
    "TASK_RUNLEVEL_TYPE": SimTypeInt(signed=False, label="TASK_RUNLEVEL_TYPE"),
    "TASK_RUN_FLAGS": SimTypeInt(signed=False, label="TASK_RUN_FLAGS"),
    "TASK_SESSION_STATE_CHANGE_TYPE": SimTypeInt(signed=False, label="TASK_SESSION_STATE_CHANGE_TYPE"),
    "TASK_STATE": SimTypeInt(signed=False, label="TASK_STATE"),
    "TASK_TRIGGER": SimStruct(OrderedDict((("cbTriggerSize", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("wBeginYear", SimTypeShort(signed=False, label="UInt16")), ("wBeginMonth", SimTypeShort(signed=False, label="UInt16")), ("wBeginDay", SimTypeShort(signed=False, label="UInt16")), ("wEndYear", SimTypeShort(signed=False, label="UInt16")), ("wEndMonth", SimTypeShort(signed=False, label="UInt16")), ("wEndDay", SimTypeShort(signed=False, label="UInt16")), ("wStartHour", SimTypeShort(signed=False, label="UInt16")), ("wStartMinute", SimTypeShort(signed=False, label="UInt16")), ("MinutesDuration", SimTypeInt(signed=False, label="UInt32")), ("MinutesInterval", SimTypeInt(signed=False, label="UInt32")), ("rgFlags", SimTypeInt(signed=False, label="UInt32")), ("TriggerType", SimTypeInt(signed=False, label="TASK_TRIGGER_TYPE")), ("Type", SimUnion({"Daily": SimTypeRef("DAILY", SimStruct), "Weekly": SimTypeRef("WEEKLY", SimStruct), "MonthlyDate": SimTypeRef("MONTHLYDATE", SimStruct), "MonthlyDOW": SimTypeRef("MONTHLYDOW", SimStruct)}, name="<anon>", label="None")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("wRandomMinutesInterval", SimTypeShort(signed=False, label="UInt16")),)), name="TASK_TRIGGER", pack=False, align=None),
    "TASK_TRIGGER_TYPE": SimTypeInt(signed=False, label="TASK_TRIGGER_TYPE"),
    "TASK_TRIGGER_TYPE2": SimTypeInt(signed=False, label="TASK_TRIGGER_TYPE2"),
    "TA_CUBIC_BEZIER": SimStruct(OrderedDict((("header", SimTypeRef("TA_TIMINGFUNCTION", SimStruct)), ("rX0", SimTypeFloat(size=32)), ("rY0", SimTypeFloat(size=32)), ("rX1", SimTypeFloat(size=32)), ("rY1", SimTypeFloat(size=32)),)), name="TA_CUBIC_BEZIER", pack=False, align=None),
    "TA_PROPERTY": SimTypeInt(signed=False, label="TA_PROPERTY"),
    "TA_PROPERTY_FLAG": SimTypeInt(signed=False, label="TA_PROPERTY_FLAG"),
    "TA_TIMINGFUNCTION": SimStruct(OrderedDict((("eTimingFunctionType", SimTypeInt(signed=False, label="TA_TIMINGFUNCTION_TYPE")),)), name="TA_TIMINGFUNCTION", pack=False, align=None),
    "TA_TIMINGFUNCTION_TYPE": SimTypeInt(signed=False, label="TA_TIMINGFUNCTION_TYPE"),
    "TA_TRANSFORM": SimStruct(OrderedDict((("eTransformType", SimTypeInt(signed=False, label="TA_TRANSFORM_TYPE")), ("dwTimingFunctionId", SimTypeInt(signed=False, label="UInt32")), ("dwStartTime", SimTypeInt(signed=False, label="UInt32")), ("dwDurationTime", SimTypeInt(signed=False, label="UInt32")), ("eFlags", SimTypeInt(signed=False, label="TA_TRANSFORM_FLAG")),)), name="TA_TRANSFORM", pack=False, align=None),
    "TA_TRANSFORM_2D": SimStruct(OrderedDict((("header", SimTypeRef("TA_TRANSFORM", SimStruct)), ("rX", SimTypeFloat(size=32)), ("rY", SimTypeFloat(size=32)), ("rInitialX", SimTypeFloat(size=32)), ("rInitialY", SimTypeFloat(size=32)), ("rOriginX", SimTypeFloat(size=32)), ("rOriginY", SimTypeFloat(size=32)),)), name="TA_TRANSFORM_2D", pack=False, align=None),
    "TA_TRANSFORM_CLIP": SimStruct(OrderedDict((("header", SimTypeRef("TA_TRANSFORM", SimStruct)), ("rLeft", SimTypeFloat(size=32)), ("rTop", SimTypeFloat(size=32)), ("rRight", SimTypeFloat(size=32)), ("rBottom", SimTypeFloat(size=32)), ("rInitialLeft", SimTypeFloat(size=32)), ("rInitialTop", SimTypeFloat(size=32)), ("rInitialRight", SimTypeFloat(size=32)), ("rInitialBottom", SimTypeFloat(size=32)),)), name="TA_TRANSFORM_CLIP", pack=False, align=None),
    "TA_TRANSFORM_FLAG": SimTypeInt(signed=False, label="TA_TRANSFORM_FLAG"),
    "TA_TRANSFORM_OPACITY": SimStruct(OrderedDict((("header", SimTypeRef("TA_TRANSFORM", SimStruct)), ("rOpacity", SimTypeFloat(size=32)), ("rInitialOpacity", SimTypeFloat(size=32)),)), name="TA_TRANSFORM_OPACITY", pack=False, align=None),
    "TA_TRANSFORM_TYPE": SimTypeInt(signed=False, label="TA_TRANSFORM_TYPE"),
    "TBADDBITMAP": SimStruct(OrderedDict((("hInst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="TBADDBITMAP", pack=False, align=None),
    "TBBUTTON": SimStruct(OrderedDict((("iBitmap", SimTypeInt(signed=True, label="Int32")), ("idCommand", SimTypeInt(signed=True, label="Int32")), ("fsState", SimTypeChar(label="Byte")), ("fsStyle", SimTypeChar(label="Byte")), ("bReserved", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("dwData", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("iString", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TBBUTTON", pack=False, align=None),
    "TBBUTTONINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="TBBUTTONINFOW_MASK")), ("idCommand", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("fsState", SimTypeChar(label="Byte")), ("fsStyle", SimTypeChar(label="Byte")), ("cx", SimTypeShort(signed=False, label="UInt16")), ("lParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchText", SimTypeInt(signed=True, label="Int32")),)), name="TBBUTTONINFOA", pack=False, align=None),
    "TBBUTTONINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="TBBUTTONINFOW_MASK")), ("idCommand", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("fsState", SimTypeChar(label="Byte")), ("fsStyle", SimTypeChar(label="Byte")), ("cx", SimTypeShort(signed=False, label="UInt16")), ("lParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchText", SimTypeInt(signed=True, label="Int32")),)), name="TBBUTTONINFOW", pack=False, align=None),
    "TBBUTTONINFOW_MASK": SimTypeInt(signed=False, label="TBBUTTONINFOW_MASK"),
    "TBINFO": SimStruct(OrderedDict((("cbuttons", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="UInt32")),)), name="TBINFO", pack=False, align=None),
    "TBINSERTMARK": SimStruct(OrderedDict((("iButton", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="TBINSERTMARK_FLAGS")),)), name="TBINSERTMARK", pack=False, align=None),
    "TBINSERTMARK_FLAGS": SimTypeInt(signed=False, label="TBINSERTMARK_FLAGS"),
    "TBMETRICS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("cxPad", SimTypeInt(signed=True, label="Int32")), ("cyPad", SimTypeInt(signed=True, label="Int32")), ("cxBarPad", SimTypeInt(signed=True, label="Int32")), ("cyBarPad", SimTypeInt(signed=True, label="Int32")), ("cxButtonSpacing", SimTypeInt(signed=True, label="Int32")), ("cyButtonSpacing", SimTypeInt(signed=True, label="Int32")),)), name="TBMETRICS", pack=False, align=None),
    "TBPFLAG": SimTypeInt(signed=False, label="TBPFLAG"),
    "TBREPLACEBITMAP": SimStruct(OrderedDict((("hInstOld", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nIDOld", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("hInstNew", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("nIDNew", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("nButtons", SimTypeInt(signed=True, label="Int32")),)), name="TBREPLACEBITMAP", pack=False, align=None),
    "TBSAVEPARAMSA": SimStruct(OrderedDict((("hkr", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszSubKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszValueName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TBSAVEPARAMSA", pack=False, align=None),
    "TBSAVEPARAMSW": SimStruct(OrderedDict((("hkr", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszSubKey", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszValueName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TBSAVEPARAMSW", pack=False, align=None),
    "TBS_COMMAND_LOCALITY": SimTypeInt(signed=False, label="TBS_COMMAND_LOCALITY"),
    "TBS_COMMAND_PRIORITY": SimTypeInt(signed=False, label="TBS_COMMAND_PRIORITY"),
    "TBS_CONTEXT_PARAMS": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")),)), name="TBS_CONTEXT_PARAMS", pack=False, align=None),
    "TBS_CONTEXT_PARAMS2": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "asUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="TBS_CONTEXT_PARAMS2", pack=False, align=None),
    "TCG_ACTIVATE_METHOD_SPECIFIC": SimStruct(OrderedDict((("RangeStartLengthPolicy", SimTypeChar(label="Byte")),)), name="TCG_ACTIVATE_METHOD_SPECIFIC", pack=False, align=None),
    "TCG_ASSIGN_METHOD_SPECIFIC": SimStruct(OrderedDict((("NamespaceId", SimTypeInt(signed=False, label="UInt32")),)), name="TCG_ASSIGN_METHOD_SPECIFIC", pack=False, align=None),
    "TCG_AUTH_METHOD_SPECIFIC": SimStruct(OrderedDict((("AuthorityId", SimTypeLongLong(signed=False, label="UInt64")), ("TriesCount", SimTypeChar(label="Byte")),)), name="TCG_AUTH_METHOD_SPECIFIC", pack=False, align=None),
    "TCG_BLOCKSID_METHOD_SPECIFIC": SimStruct(OrderedDict((("ClearEvents", SimTypeChar(label="Byte")),)), name="TCG_BLOCKSID_METHOD_SPECIFIC", pack=False, align=None),
    "TCG_HISTORY_ENTRY": SimStruct(OrderedDict((("VersionNumber", SimTypeChar(label="Byte")), ("EntryLength", SimTypeChar(label="Byte")), ("PowerCycleCount", SimTypeShort(signed=False, label="UInt16")), ("TcgCommandCount", SimTypeInt(signed=False, label="UInt32")), ("TcgCommandCompletionTS", SimTypeLongLong(signed=False, label="UInt64")), ("InvokingId", SimTypeLongLong(signed=False, label="UInt64")), ("MethodId", SimTypeLongLong(signed=False, label="UInt64")), ("ComId", SimTypeShort(signed=False, label="UInt16")), ("ProtocolId", SimTypeChar(label="Byte")), ("TcgStatus", SimTypeChar(label="Byte")), ("ProcessTime", SimTypeShort(signed=False, label="UInt16")), ("CommandSpecific", SimTypeArray(SimTypeChar(label="Byte"), 10)),)), name="TCG_HISTORY_ENTRY", pack=False, align=None),
    "TCG_PCClientPCREventStruct": SimStruct(OrderedDict((("pcrIndex", SimTypeInt(signed=False, label="UInt32")), ("eventType", SimTypeInt(signed=False, label="UInt32")), ("digest", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("eventDataSize", SimTypeInt(signed=False, label="UInt32")), ("event", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TCG_PCClientPCREventStruct", pack=False, align=None),
    "TCG_PCClientTaggedEventStruct": SimStruct(OrderedDict((("EventID", SimTypeInt(signed=False, label="UInt32")), ("EventDataSize", SimTypeInt(signed=False, label="UInt32")), ("EventData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TCG_PCClientTaggedEventStruct", pack=False, align=None),
    "TCG_REACTIVATE_METHOD_SPECIFIC": SimStruct(OrderedDict((("RangeStartLengthPolicy", SimTypeChar(label="Byte")),)), name="TCG_REACTIVATE_METHOD_SPECIFIC", pack=False, align=None),
    "TCHITTESTINFO": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("flags", SimTypeInt(signed=False, label="TCHITTESTINFO_FLAGS")),)), name="TCHITTESTINFO", pack=False, align=None),
    "TCHITTESTINFO_FLAGS": SimTypeInt(signed=False, label="TCHITTESTINFO_FLAGS"),
    "TCITEMA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TCITEMHEADERA_MASK")), ("dwState", SimTypeInt(signed=False, label="TAB_CONTROL_ITEM_STATE")), ("dwStateMask", SimTypeInt(signed=False, label="TAB_CONTROL_ITEM_STATE")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TCITEMA", pack=False, align=None),
    "TCITEMHEADERA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TCITEMHEADERA_MASK")), ("lpReserved1", SimTypeInt(signed=False, label="UInt32")), ("lpReserved2", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")),)), name="TCITEMHEADERA", pack=False, align=None),
    "TCITEMHEADERA_MASK": SimTypeInt(signed=False, label="TCITEMHEADERA_MASK"),
    "TCITEMHEADERW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TCITEMHEADERA_MASK")), ("lpReserved1", SimTypeInt(signed=False, label="UInt32")), ("lpReserved2", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")),)), name="TCITEMHEADERW", pack=False, align=None),
    "TCITEMW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TCITEMHEADERA_MASK")), ("dwState", SimTypeInt(signed=False, label="TAB_CONTROL_ITEM_STATE")), ("dwStateMask", SimTypeInt(signed=False, label="TAB_CONTROL_ITEM_STATE")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TCITEMW", pack=False, align=None),
    "TCI_ADD_FLOW_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0),
    "TCI_CLIENT_FUNC_LIST": SimStruct(OrderedDict((("ClNotifyHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ClRegCtx", "ClIfcCtx", "Event", "SubCode", "BufSize", "Buffer"]), offset=0)), ("ClAddFlowCompleteHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0)), ("ClModifyFlowCompleteHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0)), ("ClDeleteFlowCompleteHandler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0)),)), name="TCI_CLIENT_FUNC_LIST", pack=False, align=None),
    "TCI_DEL_FLOW_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0),
    "TCI_MOD_FLOW_COMPLETE_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0),
    "TCI_NOTIFY_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ClRegCtx", "ClIfcCtx", "Event", "SubCode", "BufSize", "Buffer"]), offset=0),
    "TCPIP_OWNER_MODULE_BASIC_INFO": SimStruct(OrderedDict((("pModuleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pModulePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TCPIP_OWNER_MODULE_BASIC_INFO", pack=False, align=None),
    "TCPIP_OWNER_MODULE_INFO_CLASS": SimTypeInt(signed=False, label="TCPIP_OWNER_MODULE_INFO_CLASS"),
    "TCPSTATE": SimTypeInt(signed=False, label="TCPSTATE"),
    "TCP_ACK_FREQUENCY_PARAMETERS": SimStruct(OrderedDict((("TcpDelayedAckFrequency", SimTypeChar(label="Byte")),)), name="TCP_ACK_FREQUENCY_PARAMETERS", pack=False, align=None),
    "TCP_BOOLEAN_OPTIONAL": SimTypeInt(signed=False, label="TCP_BOOLEAN_OPTIONAL"),
    "TCP_CONNECTION_OFFLOAD_STATE": SimTypeInt(signed=False, label="TCP_CONNECTION_OFFLOAD_STATE"),
    "TCP_ESTATS_BANDWIDTH_ROD_v0": SimStruct(OrderedDict((("OutboundBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("InboundBandwidth", SimTypeLongLong(signed=False, label="UInt64")), ("OutboundInstability", SimTypeLongLong(signed=False, label="UInt64")), ("InboundInstability", SimTypeLongLong(signed=False, label="UInt64")), ("OutboundBandwidthPeaked", SimTypeChar(label="Byte")), ("InboundBandwidthPeaked", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_BANDWIDTH_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_BANDWIDTH_RW_v0": SimStruct(OrderedDict((("EnableCollectionOutbound", SimTypeInt(signed=False, label="TCP_BOOLEAN_OPTIONAL")), ("EnableCollectionInbound", SimTypeInt(signed=False, label="TCP_BOOLEAN_OPTIONAL")),)), name="TCP_ESTATS_BANDWIDTH_RW_v0", pack=False, align=None),
    "TCP_ESTATS_DATA_ROD_v0": SimStruct(OrderedDict((("DataBytesOut", SimTypeLongLong(signed=False, label="UInt64")), ("DataSegsOut", SimTypeLongLong(signed=False, label="UInt64")), ("DataBytesIn", SimTypeLongLong(signed=False, label="UInt64")), ("DataSegsIn", SimTypeLongLong(signed=False, label="UInt64")), ("SegsOut", SimTypeLongLong(signed=False, label="UInt64")), ("SegsIn", SimTypeLongLong(signed=False, label="UInt64")), ("SoftErrors", SimTypeInt(signed=False, label="UInt32")), ("SoftErrorReason", SimTypeInt(signed=False, label="UInt32")), ("SndUna", SimTypeInt(signed=False, label="UInt32")), ("SndNxt", SimTypeInt(signed=False, label="UInt32")), ("SndMax", SimTypeInt(signed=False, label="UInt32")), ("ThruBytesAcked", SimTypeLongLong(signed=False, label="UInt64")), ("RcvNxt", SimTypeInt(signed=False, label="UInt32")), ("ThruBytesReceived", SimTypeLongLong(signed=False, label="UInt64")),)), name="TCP_ESTATS_DATA_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_DATA_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_DATA_RW_v0", pack=False, align=None),
    "TCP_ESTATS_FINE_RTT_ROD_v0": SimStruct(OrderedDict((("RttVar", SimTypeInt(signed=False, label="UInt32")), ("MaxRtt", SimTypeInt(signed=False, label="UInt32")), ("MinRtt", SimTypeInt(signed=False, label="UInt32")), ("SumRtt", SimTypeInt(signed=False, label="UInt32")),)), name="TCP_ESTATS_FINE_RTT_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_FINE_RTT_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_FINE_RTT_RW_v0", pack=False, align=None),
    "TCP_ESTATS_OBS_REC_ROD_v0": SimStruct(OrderedDict((("CurRwinRcvd", SimTypeInt(signed=False, label="UInt32")), ("MaxRwinRcvd", SimTypeInt(signed=False, label="UInt32")), ("MinRwinRcvd", SimTypeInt(signed=False, label="UInt32")), ("WinScaleRcvd", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_OBS_REC_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_OBS_REC_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_OBS_REC_RW_v0", pack=False, align=None),
    "TCP_ESTATS_PATH_ROD_v0": SimStruct(OrderedDict((("FastRetran", SimTypeInt(signed=False, label="UInt32")), ("Timeouts", SimTypeInt(signed=False, label="UInt32")), ("SubsequentTimeouts", SimTypeInt(signed=False, label="UInt32")), ("CurTimeoutCount", SimTypeInt(signed=False, label="UInt32")), ("AbruptTimeouts", SimTypeInt(signed=False, label="UInt32")), ("PktsRetrans", SimTypeInt(signed=False, label="UInt32")), ("BytesRetrans", SimTypeInt(signed=False, label="UInt32")), ("DupAcksIn", SimTypeInt(signed=False, label="UInt32")), ("SacksRcvd", SimTypeInt(signed=False, label="UInt32")), ("SackBlocksRcvd", SimTypeInt(signed=False, label="UInt32")), ("CongSignals", SimTypeInt(signed=False, label="UInt32")), ("PreCongSumCwnd", SimTypeInt(signed=False, label="UInt32")), ("PreCongSumRtt", SimTypeInt(signed=False, label="UInt32")), ("PostCongSumRtt", SimTypeInt(signed=False, label="UInt32")), ("PostCongCountRtt", SimTypeInt(signed=False, label="UInt32")), ("EcnSignals", SimTypeInt(signed=False, label="UInt32")), ("EceRcvd", SimTypeInt(signed=False, label="UInt32")), ("SendStall", SimTypeInt(signed=False, label="UInt32")), ("QuenchRcvd", SimTypeInt(signed=False, label="UInt32")), ("RetranThresh", SimTypeInt(signed=False, label="UInt32")), ("SndDupAckEpisodes", SimTypeInt(signed=False, label="UInt32")), ("SumBytesReordered", SimTypeInt(signed=False, label="UInt32")), ("NonRecovDa", SimTypeInt(signed=False, label="UInt32")), ("NonRecovDaEpisodes", SimTypeInt(signed=False, label="UInt32")), ("AckAfterFr", SimTypeInt(signed=False, label="UInt32")), ("DsackDups", SimTypeInt(signed=False, label="UInt32")), ("SampleRtt", SimTypeInt(signed=False, label="UInt32")), ("SmoothedRtt", SimTypeInt(signed=False, label="UInt32")), ("RttVar", SimTypeInt(signed=False, label="UInt32")), ("MaxRtt", SimTypeInt(signed=False, label="UInt32")), ("MinRtt", SimTypeInt(signed=False, label="UInt32")), ("SumRtt", SimTypeInt(signed=False, label="UInt32")), ("CountRtt", SimTypeInt(signed=False, label="UInt32")), ("CurRto", SimTypeInt(signed=False, label="UInt32")), ("MaxRto", SimTypeInt(signed=False, label="UInt32")), ("MinRto", SimTypeInt(signed=False, label="UInt32")), ("CurMss", SimTypeInt(signed=False, label="UInt32")), ("MaxMss", SimTypeInt(signed=False, label="UInt32")), ("MinMss", SimTypeInt(signed=False, label="UInt32")), ("SpuriousRtoDetections", SimTypeInt(signed=False, label="UInt32")),)), name="TCP_ESTATS_PATH_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_PATH_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_PATH_RW_v0", pack=False, align=None),
    "TCP_ESTATS_REC_ROD_v0": SimStruct(OrderedDict((("CurRwinSent", SimTypeInt(signed=False, label="UInt32")), ("MaxRwinSent", SimTypeInt(signed=False, label="UInt32")), ("MinRwinSent", SimTypeInt(signed=False, label="UInt32")), ("LimRwin", SimTypeInt(signed=False, label="UInt32")), ("DupAckEpisodes", SimTypeInt(signed=False, label="UInt32")), ("DupAcksOut", SimTypeInt(signed=False, label="UInt32")), ("CeRcvd", SimTypeInt(signed=False, label="UInt32")), ("EcnSent", SimTypeInt(signed=False, label="UInt32")), ("EcnNoncesRcvd", SimTypeInt(signed=False, label="UInt32")), ("CurReasmQueue", SimTypeInt(signed=False, label="UInt32")), ("MaxReasmQueue", SimTypeInt(signed=False, label="UInt32")), ("CurAppRQueue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaxAppRQueue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WinScaleSent", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_REC_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_REC_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_REC_RW_v0", pack=False, align=None),
    "TCP_ESTATS_SEND_BUFF_ROD_v0": SimStruct(OrderedDict((("CurRetxQueue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaxRetxQueue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CurAppWQueue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("MaxAppWQueue", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="TCP_ESTATS_SEND_BUFF_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_SEND_BUFF_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_SEND_BUFF_RW_v0", pack=False, align=None),
    "TCP_ESTATS_SND_CONG_ROD_v0": SimStruct(OrderedDict((("SndLimTransRwin", SimTypeInt(signed=False, label="UInt32")), ("SndLimTimeRwin", SimTypeInt(signed=False, label="UInt32")), ("SndLimBytesRwin", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SndLimTransCwnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimTimeCwnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimBytesCwnd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SndLimTransSnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimTimeSnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimBytesSnd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SlowStart", SimTypeInt(signed=False, label="UInt32")), ("CongAvoid", SimTypeInt(signed=False, label="UInt32")), ("OtherReductions", SimTypeInt(signed=False, label="UInt32")), ("CurCwnd", SimTypeInt(signed=False, label="UInt32")), ("MaxSsCwnd", SimTypeInt(signed=False, label="UInt32")), ("MaxCaCwnd", SimTypeInt(signed=False, label="UInt32")), ("CurSsthresh", SimTypeInt(signed=False, label="UInt32")), ("MaxSsthresh", SimTypeInt(signed=False, label="UInt32")), ("MinSsthresh", SimTypeInt(signed=False, label="UInt32")),)), name="TCP_ESTATS_SND_CONG_ROD_v0", pack=False, align=None),
    "TCP_ESTATS_SND_CONG_ROS_v0": SimStruct(OrderedDict((("LimCwnd", SimTypeInt(signed=False, label="UInt32")),)), name="TCP_ESTATS_SND_CONG_ROS_v0", pack=False, align=None),
    "TCP_ESTATS_SND_CONG_RW_v0": SimStruct(OrderedDict((("EnableCollection", SimTypeChar(label="Byte")),)), name="TCP_ESTATS_SND_CONG_RW_v0", pack=False, align=None),
    "TCP_ESTATS_SYN_OPTS_ROS_v0": SimStruct(OrderedDict((("ActiveOpen", SimTypeChar(label="Byte")), ("MssRcvd", SimTypeInt(signed=False, label="UInt32")), ("MssSent", SimTypeInt(signed=False, label="UInt32")),)), name="TCP_ESTATS_SYN_OPTS_ROS_v0", pack=False, align=None),
    "TCP_ESTATS_TYPE": SimTypeInt(signed=False, label="TCP_ESTATS_TYPE"),
    "TCP_HDR": SimStruct(OrderedDict((("th_sport", SimTypeShort(signed=False, label="UInt16")), ("th_dport", SimTypeShort(signed=False, label="UInt16")), ("th_seq", SimTypeInt(signed=False, label="UInt32")), ("th_ack", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeChar(label="Byte")), ("th_flags", SimTypeChar(label="Byte")), ("th_win", SimTypeShort(signed=False, label="UInt16")), ("th_sum", SimTypeShort(signed=False, label="UInt16")), ("th_urp", SimTypeShort(signed=False, label="UInt16")),)), name="TCP_HDR", pack=False, align=None),
    "TCP_ICW_LEVEL": SimTypeInt(signed=False, label="TCP_ICW_LEVEL"),
    "TCP_ICW_PARAMETERS": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="TCP_ICW_LEVEL")),)), name="TCP_ICW_PARAMETERS", pack=False, align=None),
    "TCP_INFO_v0": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="TCPSTATE")), ("Mss", SimTypeInt(signed=False, label="UInt32")), ("ConnectionTimeMs", SimTypeLongLong(signed=False, label="UInt64")), ("TimestampsEnabled", SimTypeChar(label="Byte")), ("RttUs", SimTypeInt(signed=False, label="UInt32")), ("MinRttUs", SimTypeInt(signed=False, label="UInt32")), ("BytesInFlight", SimTypeInt(signed=False, label="UInt32")), ("Cwnd", SimTypeInt(signed=False, label="UInt32")), ("SndWnd", SimTypeInt(signed=False, label="UInt32")), ("RcvWnd", SimTypeInt(signed=False, label="UInt32")), ("RcvBuf", SimTypeInt(signed=False, label="UInt32")), ("BytesOut", SimTypeLongLong(signed=False, label="UInt64")), ("BytesIn", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReordered", SimTypeInt(signed=False, label="UInt32")), ("BytesRetrans", SimTypeInt(signed=False, label="UInt32")), ("FastRetrans", SimTypeInt(signed=False, label="UInt32")), ("DupAcksIn", SimTypeInt(signed=False, label="UInt32")), ("TimeoutEpisodes", SimTypeInt(signed=False, label="UInt32")), ("SynRetrans", SimTypeChar(label="Byte")),)), name="TCP_INFO_v0", pack=False, align=None),
    "TCP_INFO_v1": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="TCPSTATE")), ("Mss", SimTypeInt(signed=False, label="UInt32")), ("ConnectionTimeMs", SimTypeLongLong(signed=False, label="UInt64")), ("TimestampsEnabled", SimTypeChar(label="Byte")), ("RttUs", SimTypeInt(signed=False, label="UInt32")), ("MinRttUs", SimTypeInt(signed=False, label="UInt32")), ("BytesInFlight", SimTypeInt(signed=False, label="UInt32")), ("Cwnd", SimTypeInt(signed=False, label="UInt32")), ("SndWnd", SimTypeInt(signed=False, label="UInt32")), ("RcvWnd", SimTypeInt(signed=False, label="UInt32")), ("RcvBuf", SimTypeInt(signed=False, label="UInt32")), ("BytesOut", SimTypeLongLong(signed=False, label="UInt64")), ("BytesIn", SimTypeLongLong(signed=False, label="UInt64")), ("BytesReordered", SimTypeInt(signed=False, label="UInt32")), ("BytesRetrans", SimTypeInt(signed=False, label="UInt32")), ("FastRetrans", SimTypeInt(signed=False, label="UInt32")), ("DupAcksIn", SimTypeInt(signed=False, label="UInt32")), ("TimeoutEpisodes", SimTypeInt(signed=False, label="UInt32")), ("SynRetrans", SimTypeChar(label="Byte")), ("SndLimTransRwin", SimTypeInt(signed=False, label="UInt32")), ("SndLimTimeRwin", SimTypeInt(signed=False, label="UInt32")), ("SndLimBytesRwin", SimTypeLongLong(signed=False, label="UInt64")), ("SndLimTransCwnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimTimeCwnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimBytesCwnd", SimTypeLongLong(signed=False, label="UInt64")), ("SndLimTransSnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimTimeSnd", SimTypeInt(signed=False, label="UInt32")), ("SndLimBytesSnd", SimTypeLongLong(signed=False, label="UInt64")),)), name="TCP_INFO_v1", pack=False, align=None),
    "TCP_INITIAL_RTO_PARAMETERS": SimStruct(OrderedDict((("Rtt", SimTypeShort(signed=False, label="UInt16")), ("MaxSynRetransmissions", SimTypeChar(label="Byte")),)), name="TCP_INITIAL_RTO_PARAMETERS", pack=False, align=None),
    "TCP_OPT_FASTOPEN": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("Cookie", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TCP_OPT_FASTOPEN", pack=False, align=None),
    "TCP_OPT_MSS": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("Mss", SimTypeShort(signed=False, label="UInt16")),)), name="TCP_OPT_MSS", pack=False, align=None),
    "TCP_OPT_SACK": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("Block", SimTypePointer(SimTypeRef("tcp_opt_sack_block", SimStruct), offset=0)),)), name="TCP_OPT_SACK", pack=False, align=None),
    "TCP_OPT_SACK_PERMITTED": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")),)), name="TCP_OPT_SACK_PERMITTED", pack=False, align=None),
    "TCP_OPT_TS": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("Val", SimTypeInt(signed=False, label="UInt32")), ("EcR", SimTypeInt(signed=False, label="UInt32")),)), name="TCP_OPT_TS", pack=False, align=None),
    "TCP_OPT_UNKNOWN": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")),)), name="TCP_OPT_UNKNOWN", pack=False, align=None),
    "TCP_OPT_WS": SimStruct(OrderedDict((("Kind", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("ShiftCnt", SimTypeChar(label="Byte")),)), name="TCP_OPT_WS", pack=False, align=None),
    "TCP_REQUEST_QUERY_INFORMATION_EX32_XP": SimStruct(OrderedDict((("ID", SimTypeRef("TDIObjectID", SimStruct)), ("Context", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="TCP_REQUEST_QUERY_INFORMATION_EX32_XP", pack=False, align=None),
    "TCP_REQUEST_QUERY_INFORMATION_EX_W2K": SimStruct(OrderedDict((("ID", SimTypeRef("TDIObjectID", SimStruct)), ("Context", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="TCP_REQUEST_QUERY_INFORMATION_EX_W2K", pack=False, align=None),
    "TCP_REQUEST_QUERY_INFORMATION_EX_XP": SimStruct(OrderedDict((("ID", SimTypeRef("TDIObjectID", SimStruct)), ("Context", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 4)),)), name="TCP_REQUEST_QUERY_INFORMATION_EX_XP", pack=False, align=None),
    "TCP_REQUEST_SET_INFORMATION_EX": SimStruct(OrderedDict((("ID", SimTypeRef("TDIObjectID", SimStruct)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TCP_REQUEST_SET_INFORMATION_EX", pack=False, align=None),
    "TCP_RESERVE_PORT_RANGE": SimStruct(OrderedDict((("UpperRange", SimTypeShort(signed=False, label="UInt16")), ("LowerRange", SimTypeShort(signed=False, label="UInt16")),)), name="TCP_RESERVE_PORT_RANGE", pack=False, align=None),
    "TCP_RTO_ALGORITHM": SimTypeInt(signed=False, label="TCP_RTO_ALGORITHM"),
    "TCP_SOFT_ERROR": SimTypeInt(signed=False, label="TCP_SOFT_ERROR"),
    "TCP_TABLE_CLASS": SimTypeInt(signed=False, label="TCP_TABLE_CLASS"),
    "TC_GEN_FILTER": SimStruct(OrderedDict((("AddressType", SimTypeShort(signed=False, label="UInt16")), ("PatternSize", SimTypeInt(signed=False, label="UInt32")), ("Pattern", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Mask", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TC_GEN_FILTER", pack=False, align=None),
    "TC_GEN_FLOW": SimStruct(OrderedDict((("SendingFlowspec", SimTypeRef("FLOWSPEC", SimStruct)), ("ReceivingFlowspec", SimTypeRef("FLOWSPEC", SimStruct)), ("TcObjectsLength", SimTypeInt(signed=False, label="UInt32")), ("TcObjects", SimTypePointer(SimTypeRef("QOS_OBJECT_HDR", SimStruct), offset=0)),)), name="TC_GEN_FLOW", pack=False, align=None),
    "TC_IFC_DESCRIPTOR": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("pInterfaceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pInterfaceID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("AddressListDesc", SimTypeRef("ADDRESS_LIST_DESCRIPTOR", SimStruct)),)), name="TC_IFC_DESCRIPTOR", pack=False, align=None),
    "TC_SUPPORTED_INFO_BUFFER": SimStruct(OrderedDict((("InstanceIDLength", SimTypeShort(signed=False, label="UInt16")), ("InstanceID", SimTypeArray(SimTypeChar(label="Char"), 256)), ("InterfaceLuid", SimTypeLongLong(signed=False, label="UInt64")), ("AddrListDesc", SimTypeRef("ADDRESS_LIST_DESCRIPTOR", SimStruct)),)), name="TC_SUPPORTED_INFO_BUFFER", pack=False, align=None),
    "TDH_CONTEXT": SimStruct(OrderedDict((("ParameterValue", SimTypeLongLong(signed=False, label="UInt64")), ("ParameterType", SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE")), ("ParameterSize", SimTypeInt(signed=False, label="UInt32")),)), name="TDH_CONTEXT", pack=False, align=None),
    "TDH_CONTEXT_TYPE": SimTypeInt(signed=False, label="TDH_CONTEXT_TYPE"),
    "TDH_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "TDIENTITY_ENTITY_TYPE": SimTypeInt(signed=False, label="TDIENTITY_ENTITY_TYPE"),
    "TDIEntityID": SimStruct(OrderedDict((("tei_entity", SimTypeInt(signed=False, label="TDIENTITY_ENTITY_TYPE")), ("tei_instance", SimTypeInt(signed=False, label="UInt32")),)), name="TDIEntityID", pack=False, align=None),
    "TDIObjectID": SimStruct(OrderedDict((("toi_entity", SimTypeRef("TDIEntityID", SimStruct)), ("toi_class", SimTypeInt(signed=False, label="UInt32")), ("toi_type", SimTypeInt(signed=False, label="UInt32")), ("toi_id", SimTypeInt(signed=False, label="UInt32")),)), name="TDIObjectID", pack=False, align=None),
    "TDI_PNP_HANDLER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["UpperComponent", "LowerComponent", "BindList", "ReconfigBuffer", "ReconfigBufferSize", "Operation"]), offset=0),
    "TDI_REGISTER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceName", "TdiHandle"]), offset=0),
    "TDI_TL_IO_CONTROL_ENDPOINT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="TDI_TL_IO_CONTROL_TYPE")), ("Level", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"IoControlCode": SimTypeInt(signed=False, label="UInt32"), "OptionName": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("InputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("OutputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="TDI_TL_IO_CONTROL_ENDPOINT", pack=False, align=None),
    "TDI_TL_IO_CONTROL_TYPE": SimTypeInt(signed=False, label="TDI_TL_IO_CONTROL_TYPE"),
    "TEB": SimStruct(OrderedDict((("Reserved1", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 12)), ("ProcessEnvironmentBlock", SimTypePointer(SimTypeRef("PEB", SimStruct), offset=0)), ("Reserved2", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 399)), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 1952)), ("TlsSlots", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 64)), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Reserved5", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 26)), ("ReservedForOle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved6", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 4)), ("TlsExpansionSlots", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TEB", pack=False, align=None),
    "TELEPHONY_CALLCONTROLOP": SimTypeInt(signed=False, label="TELEPHONY_CALLCONTROLOP"),
    "TELEPHONY_CALLSTATE": SimTypeInt(signed=False, label="TELEPHONY_CALLSTATE"),
    "TELEPHONY_CALLTYPE": SimTypeInt(signed=False, label="TELEPHONY_CALLTYPE"),
    "TELEPHONY_PROVIDERCHANGEOP": SimTypeInt(signed=False, label="TELEPHONY_PROVIDERCHANGEOP"),
    "TEMPLATESPROPERTIES": SimTypeInt(signed=False, label="TEMPLATESPROPERTIES"),
    "TEMPLATE_FLAGS": SimTypeInt(signed=False, label="TEMPLATE_FLAGS"),
    "TEMP_BP_NOTE": SimStruct(OrderedDict((("Seg", SimTypeShort(signed=False, label="UInt16")), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("bPM", SimTypeInt(signed=True, label="Int32")),)), name="TEMP_BP_NOTE", pack=False, align=None),
    "TERMINAL_DIRECTION": SimTypeInt(signed=False, label="TERMINAL_DIRECTION"),
    "TERMINAL_MEDIA_STATE": SimTypeInt(signed=False, label="TERMINAL_MEDIA_STATE"),
    "TERMINAL_STATE": SimTypeInt(signed=False, label="TERMINAL_STATE"),
    "TERMINAL_TYPE": SimTypeInt(signed=False, label="TERMINAL_TYPE"),
    "TEXTMETRICA": SimStruct(OrderedDict((("tmHeight", SimTypeInt(signed=True, label="Int32")), ("tmAscent", SimTypeInt(signed=True, label="Int32")), ("tmDescent", SimTypeInt(signed=True, label="Int32")), ("tmInternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmExternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmAveCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmMaxCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmWeight", SimTypeInt(signed=True, label="Int32")), ("tmOverhang", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectX", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectY", SimTypeInt(signed=True, label="Int32")), ("tmFirstChar", SimTypeChar(label="Byte")), ("tmLastChar", SimTypeChar(label="Byte")), ("tmDefaultChar", SimTypeChar(label="Byte")), ("tmBreakChar", SimTypeChar(label="Byte")), ("tmItalic", SimTypeChar(label="Byte")), ("tmUnderlined", SimTypeChar(label="Byte")), ("tmStruckOut", SimTypeChar(label="Byte")), ("tmPitchAndFamily", SimTypeInt(signed=False, label="TMPF_FLAGS")), ("tmCharSet", SimTypeChar(label="Byte")),)), name="TEXTMETRICA", pack=False, align=None),
    "TEXTMETRICW": SimStruct(OrderedDict((("tmHeight", SimTypeInt(signed=True, label="Int32")), ("tmAscent", SimTypeInt(signed=True, label="Int32")), ("tmDescent", SimTypeInt(signed=True, label="Int32")), ("tmInternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmExternalLeading", SimTypeInt(signed=True, label="Int32")), ("tmAveCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmMaxCharWidth", SimTypeInt(signed=True, label="Int32")), ("tmWeight", SimTypeInt(signed=True, label="Int32")), ("tmOverhang", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectX", SimTypeInt(signed=True, label="Int32")), ("tmDigitizedAspectY", SimTypeInt(signed=True, label="Int32")), ("tmFirstChar", SimTypeChar(label="Char")), ("tmLastChar", SimTypeChar(label="Char")), ("tmDefaultChar", SimTypeChar(label="Char")), ("tmBreakChar", SimTypeChar(label="Char")), ("tmItalic", SimTypeChar(label="Byte")), ("tmUnderlined", SimTypeChar(label="Byte")), ("tmStruckOut", SimTypeChar(label="Byte")), ("tmPitchAndFamily", SimTypeInt(signed=False, label="TMPF_FLAGS")), ("tmCharSet", SimTypeChar(label="Byte")),)), name="TEXTMETRICW", pack=False, align=None),
    "TEXTMODE": SimTypeInt(signed=False, label="TEXTMODE"),
    "TEXTRANGEA": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("lpstrText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TEXTRANGEA", pack=False, align=None),
    "TEXTRANGEW": SimStruct(OrderedDict((("chrg", SimTypeRef("CHARRANGE", SimStruct)), ("lpstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TEXTRANGEW", pack=False, align=None),
    "TEXTRANGE_PROPERTIES": SimStruct(OrderedDict((("potfRecords", SimTypePointer(SimTypeRef("OPENTYPE_FEATURE_RECORD", SimStruct), offset=0)), ("cotfRecords", SimTypeInt(signed=True, label="Int32")),)), name="TEXTRANGE_PROPERTIES", pack=False, align=None),
    "TEXTSELECTIONGRIPPERPARTS": SimTypeInt(signed=False, label="TEXTSELECTIONGRIPPERPARTS"),
    "TEXTSHADOWTYPE": SimTypeInt(signed=False, label="TEXTSHADOWTYPE"),
    "TEXTSTYLEPARTS": SimTypeInt(signed=False, label="TEXTSTYLEPARTS"),
    "TEXT_ALIGN_OPTIONS": SimTypeInt(signed=False, label="TEXT_ALIGN_OPTIONS"),
    "TEXT_DOCUMENT_ARRAY": SimStruct(OrderedDict((("dwCount", SimTypeInt(signed=False, label="UInt32")), ("Members", SimTypePointer(SimTypeBottom(label="IDebugDocumentText"), offset=0)),)), name="TEXT_DOCUMENT_ARRAY", pack=False, align=None),
    "TEXT_SOURCE": SimStruct(OrderedDict((("pfnFillTextBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("TEXT_SOURCE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTextSource"]), offset=0)), ("awcBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("iEnd", SimTypeInt(signed=False, label="UInt32")), ("iCur", SimTypeInt(signed=False, label="UInt32")),)), name="TEXT_SOURCE", pack=False, align=None),
    "TEXT_STORE_CHANGE_FLAGS": SimTypeInt(signed=False, label="TEXT_STORE_CHANGE_FLAGS"),
    "TEXT_STORE_LOCK_FLAGS": SimTypeInt(signed=False, label="TEXT_STORE_LOCK_FLAGS"),
    "TEXT_STORE_TEXT_CHANGE_FLAGS": SimTypeInt(signed=False, label="TEXT_STORE_TEXT_CHANGE_FLAGS"),
    "TF_CONTEXT_EDIT_CONTEXT_FLAGS": SimTypeInt(signed=False, label="TF_CONTEXT_EDIT_CONTEXT_FLAGS"),
    "TF_DA_ATTR_INFO": SimTypeInt(signed=False, label="TF_DA_ATTR_INFO"),
    "TF_DA_COLOR": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="TF_DA_COLORTYPE")), ("Anonymous", SimUnion({"nIndex": SimTypeInt(signed=True, label="Int32"), "cr": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="TF_DA_COLOR", pack=False, align=None),
    "TF_DA_COLORTYPE": SimTypeInt(signed=False, label="TF_DA_COLORTYPE"),
    "TF_DA_LINESTYLE": SimTypeInt(signed=False, label="TF_DA_LINESTYLE"),
    "TF_DISPLAYATTRIBUTE": SimStruct(OrderedDict((("crText", SimTypeRef("TF_DA_COLOR", SimStruct)), ("crBk", SimTypeRef("TF_DA_COLOR", SimStruct)), ("lsStyle", SimTypeInt(signed=False, label="TF_DA_LINESTYLE")), ("fBoldLine", SimTypeInt(signed=True, label="Int32")), ("crLine", SimTypeRef("TF_DA_COLOR", SimStruct)), ("bAttr", SimTypeInt(signed=False, label="TF_DA_ATTR_INFO")),)), name="TF_DISPLAYATTRIBUTE", pack=False, align=None),
    "TF_HALTCOND": SimStruct(OrderedDict((("pHaltRange", SimTypeBottom(label="ITfRange")), ("aHaltPos", SimTypeInt(signed=False, label="TfAnchor")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="TF_HALTCOND", pack=False, align=None),
    "TF_INPUTPROCESSORPROFILE": SimStruct(OrderedDict((("dwProfileType", SimTypeInt(signed=False, label="UInt32")), ("langid", SimTypeShort(signed=False, label="UInt16")), ("clsid", SimTypeBottom(label="Guid")), ("guidProfile", SimTypeBottom(label="Guid")), ("catid", SimTypeBottom(label="Guid")), ("hklSubstitute", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwCaps", SimTypeInt(signed=False, label="UInt32")), ("hkl", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="TF_INPUTPROCESSORPROFILE", pack=False, align=None),
    "TF_LANGBARITEMINFO": SimStruct(OrderedDict((("clsidService", SimTypeBottom(label="Guid")), ("guidItem", SimTypeBottom(label="Guid")), ("dwStyle", SimTypeInt(signed=False, label="UInt32")), ("ulSort", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="TF_LANGBARITEMINFO", pack=False, align=None),
    "TF_LANGUAGEPROFILE": SimStruct(OrderedDict((("clsid", SimTypeBottom(label="Guid")), ("langid", SimTypeShort(signed=False, label="UInt16")), ("catid", SimTypeBottom(label="Guid")), ("fActive", SimTypeInt(signed=True, label="Int32")), ("guidProfile", SimTypeBottom(label="Guid")),)), name="TF_LANGUAGEPROFILE", pack=False, align=None),
    "TF_LBBALLOONINFO": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="TfLBBalloonStyle")), ("bstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TF_LBBALLOONINFO", pack=False, align=None),
    "TF_LMLATTELEMENT": SimStruct(OrderedDict((("dwFrameStart", SimTypeInt(signed=False, label="UInt32")), ("dwFrameLen", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"iCost": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")), ("bstrText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TF_LMLATTELEMENT", pack=False, align=None),
    "TF_PERSISTENT_PROPERTY_HEADER_ACP": SimStruct(OrderedDict((("guidType", SimTypeBottom(label="Guid")), ("ichStart", SimTypeInt(signed=True, label="Int32")), ("cch", SimTypeInt(signed=True, label="Int32")), ("cb", SimTypeInt(signed=False, label="UInt32")), ("dwPrivate", SimTypeInt(signed=False, label="UInt32")), ("clsidTIP", SimTypeBottom(label="Guid")),)), name="TF_PERSISTENT_PROPERTY_HEADER_ACP", pack=False, align=None),
    "TF_PRESERVEDKEY": SimStruct(OrderedDict((("uVKey", SimTypeInt(signed=False, label="UInt32")), ("uModifiers", SimTypeInt(signed=False, label="UInt32")),)), name="TF_PRESERVEDKEY", pack=False, align=None),
    "TF_PROPERTYVAL": SimStruct(OrderedDict((("guidId", SimTypeBottom(label="Guid")), ("varValue", SimTypeRef("VARIANT", SimStruct)),)), name="TF_PROPERTYVAL", pack=False, align=None),
    "TF_SELECTION": SimStruct(OrderedDict((("range", SimTypeBottom(label="ITfRange")), ("style", SimTypeRef("TF_SELECTIONSTYLE", SimStruct)),)), name="TF_SELECTION", pack=False, align=None),
    "TF_SELECTIONSTYLE": SimStruct(OrderedDict((("ase", SimTypeInt(signed=False, label="TfActiveSelEnd")), ("fInterimChar", SimTypeInt(signed=True, label="Int32")),)), name="TF_SELECTIONSTYLE", pack=False, align=None),
    "THDTYPE": SimTypeInt(signed=False, label="THDTYPE"),
    "THEMESIZE": SimTypeInt(signed=False, label="THEMESIZE"),
    "THEME_PROPERTY_SYMBOL_ID": SimTypeInt(signed=False, label="THEME_PROPERTY_SYMBOL_ID"),
    "THERMAL_EVENT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Temperature", SimTypeInt(signed=False, label="UInt32")), ("TripPointTemperature", SimTypeInt(signed=False, label="UInt32")), ("Initiator", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="THERMAL_EVENT", pack=False, align=None),
    "THERMAL_INFORMATION": SimStruct(OrderedDict((("ThermalStamp", SimTypeInt(signed=False, label="UInt32")), ("ThermalConstant1", SimTypeInt(signed=False, label="UInt32")), ("ThermalConstant2", SimTypeInt(signed=False, label="UInt32")), ("Processors", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SamplingPeriod", SimTypeInt(signed=False, label="UInt32")), ("CurrentTemperature", SimTypeInt(signed=False, label="UInt32")), ("PassiveTripPoint", SimTypeInt(signed=False, label="UInt32")), ("CriticalTripPoint", SimTypeInt(signed=False, label="UInt32")), ("ActiveTripPointCount", SimTypeChar(label="Byte")), ("ActiveTripPoint", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 10)),)), name="THERMAL_INFORMATION", pack=False, align=None),
    "THERMAL_POLICY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("WaitForUpdate", SimTypeChar(label="Byte")), ("Hibernate", SimTypeChar(label="Byte")), ("Critical", SimTypeChar(label="Byte")), ("ThermalStandby", SimTypeChar(label="Byte")), ("ActivationReasons", SimTypeInt(signed=False, label="UInt32")), ("PassiveLimit", SimTypeInt(signed=False, label="UInt32")), ("ActiveLevel", SimTypeInt(signed=False, label="UInt32")), ("OverThrottled", SimTypeChar(label="Byte")),)), name="THERMAL_POLICY", pack=False, align=None),
    "THERMAL_WAIT_READ": SimStruct(OrderedDict((("Timeout", SimTypeInt(signed=False, label="UInt32")), ("LowTemperature", SimTypeInt(signed=False, label="UInt32")), ("HighTemperature", SimTypeInt(signed=False, label="UInt32")),)), name="THERMAL_WAIT_READ", pack=False, align=None),
    "THREADENTRY32": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("cntUsage", SimTypeInt(signed=False, label="UInt32")), ("th32ThreadID", SimTypeInt(signed=False, label="UInt32")), ("th32OwnerProcessID", SimTypeInt(signed=False, label="UInt32")), ("tpBasePri", SimTypeInt(signed=True, label="Int32")), ("tpDeltaPri", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="THREADENTRY32", pack=False, align=None),
    "THREADINFOCLASS": SimTypeInt(signed=False, label="THREADINFOCLASS"),
    "THREAD_ACCESS_RIGHTS": SimTypeInt(signed=False, label="THREAD_ACCESS_RIGHTS"),
    "THREAD_CREATION_FLAGS": SimTypeInt(signed=False, label="THREAD_CREATION_FLAGS"),
    "THREAD_ERROR_MODE": SimTypeInt(signed=False, label="THREAD_ERROR_MODE"),
    "THREAD_INFORMATION_CLASS": SimTypeInt(signed=False, label="THREAD_INFORMATION_CLASS"),
    "THREAD_NAME_INFORMATION": SimStruct(OrderedDict((("ThreadName", SimTypeRef("UNICODE_STRING", SimStruct)),)), name="THREAD_NAME_INFORMATION", pack=False, align=None),
    "THREAD_POWER_THROTTLING_STATE": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("ControlMask", SimTypeInt(signed=False, label="UInt32")), ("StateMask", SimTypeInt(signed=False, label="UInt32")),)), name="THREAD_POWER_THROTTLING_STATE", pack=False, align=None),
    "THREAD_PRIORITY": SimTypeInt(signed=False, label="THREAD_PRIORITY"),
    "THREAD_WRITE_FLAGS": SimTypeInt(signed=False, label="THREAD_WRITE_FLAGS"),
    "THUMBBOTTOMSTATES": SimTypeInt(signed=False, label="THUMBBOTTOMSTATES"),
    "THUMBBUTTON": SimStruct(OrderedDict((("dwMask", SimTypeInt(signed=False, label="THUMBBUTTONMASK")), ("iId", SimTypeInt(signed=False, label="UInt32")), ("iBitmap", SimTypeInt(signed=False, label="UInt32")), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szTip", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwFlags", SimTypeInt(signed=False, label="THUMBBUTTONFLAGS")),)), name="THUMBBUTTON", pack=False, align=None),
    "THUMBBUTTONFLAGS": SimTypeInt(signed=False, label="THUMBBUTTONFLAGS"),
    "THUMBBUTTONMASK": SimTypeInt(signed=False, label="THUMBBUTTONMASK"),
    "THUMBLEFTSTATES": SimTypeInt(signed=False, label="THUMBLEFTSTATES"),
    "THUMBRIGHTSTATES": SimTypeInt(signed=False, label="THUMBRIGHTSTATES"),
    "THUMBSTATES": SimTypeInt(signed=False, label="THUMBSTATES"),
    "THUMBTOPSTATES": SimTypeInt(signed=False, label="THUMBTOPSTATES"),
    "THUMBVERTSTATES": SimTypeInt(signed=False, label="THUMBVERTSTATES"),
    "TICSSTATES": SimTypeInt(signed=False, label="TICSSTATES"),
    "TICSVERTSTATES": SimTypeInt(signed=False, label="TICSVERTSTATES"),
    "TID_EXTENSION": SimStruct(OrderedDict((("wTidExt", SimTypeShort(signed=False, label="UInt16")), ("wCount", SimTypeShort(signed=False, label="UInt16")),)), name="TID_EXTENSION", pack=False, align=None),
    "TILE_TEMPLATE_TYPE": SimTypeInt(signed=False, label="TILE_TEMPLATE_TYPE"),
    "TILE_WINDOWS_HOW": SimTypeInt(signed=False, label="TILE_WINDOWS_HOW"),
    "TIMECAPS": SimStruct(OrderedDict((("wPeriodMin", SimTypeInt(signed=False, label="UInt32")), ("wPeriodMax", SimTypeInt(signed=False, label="UInt32")),)), name="TIMECAPS", pack=False, align=None),
    "TIMECODE": SimUnion({"Anonymous": SimStruct(OrderedDict((("wFrameRate", SimTypeShort(signed=False, label="UInt16")), ("wFrameFract", SimTypeShort(signed=False, label="UInt16")), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "qw": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "TIMECODEDATA": SimStruct(OrderedDict((("time", SimUnion({"Anonymous": SimStruct(OrderedDict((("wFrameRate", SimTypeShort(signed=False, label="UInt16")), ("wFrameFract", SimTypeShort(signed=False, label="UInt16")), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "qw": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("dwSMPTEflags", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypeInt(signed=False, label="UInt32")),)), name="TIMECODEDATA", pack=False, align=None),
    "TIMECODE_SAMPLE": SimStruct(OrderedDict((("qwTick", SimTypeLongLong(signed=True, label="Int64")), ("timecode", SimUnion({"Anonymous": SimStruct(OrderedDict((("wFrameRate", SimTypeShort(signed=False, label="UInt16")), ("wFrameFract", SimTypeShort(signed=False, label="UInt16")), ("dwFrames", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "qw": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("dwUser", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="TIMECODE_SAMPLE_FLAGS")),)), name="TIMECODE_SAMPLE", pack=False, align=None),
    "TIMECODE_SAMPLE_FLAGS": SimTypeInt(signed=False, label="TIMECODE_SAMPLE_FLAGS"),
    "TIMEFMT_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "TIMEFMT_ENUMPROCEX": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "TIMEFMT_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "TIMEOUT_INFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwConnectTimeout", SimTypeInt(signed=False, label="UInt32")), ("dwDataTimeout", SimTypeInt(signed=False, label="UInt32")),)), name="TIMEOUT_INFO", pack=False, align=None),
    "TIMEREVENT": SimStruct(OrderedDict((("wDelay", SimTypeShort(signed=False, label="UInt16")), ("wResolution", SimTypeShort(signed=False, label="UInt16")), ("lpFunction", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["uTimerID", "uMsg", "dwUser", "dw1", "dw2"]), offset=0)), ("dwUser", SimTypeInt(signed=False, label="UInt32")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")),)), name="TIMEREVENT", pack=False, align=None),
    "TIMERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "TIMER_SET_COALESCABLE_TIMER_INFO": SimStruct(OrderedDict((("DueTime", SimTypeLongLong(signed=True, label="Int64")), ("TimerApcRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["TimerContext", "TimerLowValue", "TimerHighValue"]), offset=0)), ("TimerContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("WakeContext", SimTypePointer(SimTypeRef("COUNTED_REASON_CONTEXT", SimStruct), offset=0)), ("Period", SimTypeInt(signed=False, label="UInt32")), ("TolerableDelay", SimTypeInt(signed=False, label="UInt32")), ("PreviousState", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TIMER_SET_COALESCABLE_TIMER_INFO", pack=False, align=None),
    "TIMER_SET_INFORMATION_CLASS": SimTypeInt(signed=False, label="TIMER_SET_INFORMATION_CLASS"),
    "TIMER_TYPE": SimTypeInt(signed=False, label="TIMER_TYPE"),
    "TIMESTAMPING_CONFIG": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("TxTimestampsBuffered", SimTypeShort(signed=False, label="UInt16")),)), name="TIMESTAMPING_CONFIG", pack=False, align=None),
    "TIMESTAMP_STRUCT": SimStruct(OrderedDict((("year", SimTypeShort(signed=True, label="Int16")), ("month", SimTypeShort(signed=False, label="UInt16")), ("day", SimTypeShort(signed=False, label="UInt16")), ("hour", SimTypeShort(signed=False, label="UInt16")), ("minute", SimTypeShort(signed=False, label="UInt16")), ("second", SimTypeShort(signed=False, label="UInt16")), ("fraction", SimTypeInt(signed=False, label="UInt32")),)), name="TIMESTAMP_STRUCT", pack=False, align=None),
    "TIMEVAL": SimStruct(OrderedDict((("tv_sec", SimTypeInt(signed=True, label="Int32")), ("tv_usec", SimTypeInt(signed=True, label="Int32")),)), name="TIMEVAL", pack=False, align=None),
    "TIME_FIELDS": SimStruct(OrderedDict((("Year", SimTypeShort(signed=True, label="Int16")), ("Month", SimTypeShort(signed=True, label="Int16")), ("Day", SimTypeShort(signed=True, label="Int16")), ("Hour", SimTypeShort(signed=True, label="Int16")), ("Minute", SimTypeShort(signed=True, label="Int16")), ("Second", SimTypeShort(signed=True, label="Int16")), ("Milliseconds", SimTypeShort(signed=True, label="Int16")), ("Weekday", SimTypeShort(signed=True, label="Int16")),)), name="TIME_FIELDS", pack=False, align=None),
    "TIME_FORMAT_FLAGS": SimTypeInt(signed=False, label="TIME_FORMAT_FLAGS"),
    "TIME_OF_DAY_INFO": SimStruct(OrderedDict((("tod_elapsedt", SimTypeInt(signed=False, label="UInt32")), ("tod_msecs", SimTypeInt(signed=False, label="UInt32")), ("tod_hours", SimTypeInt(signed=False, label="UInt32")), ("tod_mins", SimTypeInt(signed=False, label="UInt32")), ("tod_secs", SimTypeInt(signed=False, label="UInt32")), ("tod_hunds", SimTypeInt(signed=False, label="UInt32")), ("tod_timezone", SimTypeInt(signed=True, label="Int32")), ("tod_tinterval", SimTypeInt(signed=False, label="UInt32")), ("tod_day", SimTypeInt(signed=False, label="UInt32")), ("tod_month", SimTypeInt(signed=False, label="UInt32")), ("tod_year", SimTypeInt(signed=False, label="UInt32")), ("tod_weekday", SimTypeInt(signed=False, label="UInt32")),)), name="TIME_OF_DAY_INFO", pack=False, align=None),
    "TIME_STRUCT": SimStruct(OrderedDict((("hour", SimTypeShort(signed=False, label="UInt16")), ("minute", SimTypeShort(signed=False, label="UInt16")), ("second", SimTypeShort(signed=False, label="UInt16")),)), name="TIME_STRUCT", pack=False, align=None),
    "TIME_ZONE_INFORMATION": SimStruct(OrderedDict((("Bias", SimTypeInt(signed=True, label="Int32")), ("StandardName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("StandardDate", SimTypeRef("SYSTEMTIME", SimStruct)), ("StandardBias", SimTypeInt(signed=True, label="Int32")), ("DaylightName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DaylightDate", SimTypeRef("SYSTEMTIME", SimStruct)), ("DaylightBias", SimTypeInt(signed=True, label="Int32")),)), name="TIME_ZONE_INFORMATION", pack=False, align=None),
    "TITLEBARINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcTitleBar", SimTypeRef("RECT", SimStruct)), ("rgstate", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)),)), name="TITLEBARINFO", pack=False, align=None),
    "TITLEBARINFOEX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcTitleBar", SimTypeRef("RECT", SimStruct)), ("rgstate", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)), ("rgrect", SimTypeArray(SimTypeRef("RECT", SimStruct), 6)),)), name="TITLEBARINFOEX", pack=False, align=None),
    "TITLEBARSTATES": SimTypeInt(signed=False, label="TITLEBARSTATES"),
    "TI_FINDCHILDREN_PARAMS": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Start", SimTypeInt(signed=False, label="UInt32")), ("ChildId", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="TI_FINDCHILDREN_PARAMS", pack=False, align=None),
    "TI_FLAGS": SimTypeInt(signed=False, label="TI_FLAGS"),
    "TKBLayoutType": SimTypeInt(signed=False, label="TKBLayoutType"),
    "TLENUMF": SimTypeInt(signed=False, label="TLENUMF"),
    "TLIBATTR": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("syskind", SimTypeInt(signed=False, label="SYSKIND")), ("wMajorVerNum", SimTypeShort(signed=False, label="UInt16")), ("wMinorVerNum", SimTypeShort(signed=False, label="UInt16")), ("wLibFlags", SimTypeShort(signed=False, label="UInt16")),)), name="TLIBATTR", pack=False, align=None),
    "TLS_EXTENSION_SUBSCRIPTION": SimStruct(OrderedDict((("ExtensionType", SimTypeShort(signed=False, label="UInt16")), ("HandshakeType", SimTypeShort(signed=False, label="UInt16")),)), name="TLS_EXTENSION_SUBSCRIPTION", pack=False, align=None),
    "TLS_PARAMETERS": SimStruct(OrderedDict((("cAlpnIds", SimTypeInt(signed=False, label="UInt32")), ("rgstrAlpnIds", SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)), ("grbitDisabledProtocols", SimTypeInt(signed=False, label="UInt32")), ("cDisabledCrypto", SimTypeInt(signed=False, label="UInt32")), ("pDisabledCrypto", SimTypePointer(SimTypeRef("CRYPTO_SETTINGS", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="TLS_PARAMETERS", pack=False, align=None),
    "TMPF_FLAGS": SimTypeInt(signed=False, label="TMPF_FLAGS"),
    "TOC_DESCRIPTOR": SimStruct(OrderedDict((("guidID", SimTypeBottom(label="Guid")), ("wStreamNumber", SimTypeShort(signed=False, label="UInt16")), ("guidType", SimTypeBottom(label="Guid")), ("wLanguageIndex", SimTypeShort(signed=False, label="UInt16")),)), name="TOC_DESCRIPTOR", pack=False, align=None),
    "TOC_ENTRY_DESCRIPTOR": SimStruct(OrderedDict((("qwStartTime", SimTypeLongLong(signed=False, label="UInt64")), ("qwEndTime", SimTypeLongLong(signed=False, label="UInt64")), ("qwStartPacketOffset", SimTypeLongLong(signed=False, label="UInt64")), ("qwEndPacketOffset", SimTypeLongLong(signed=False, label="UInt64")), ("qwRepresentativeFrameTime", SimTypeLongLong(signed=False, label="UInt64")),)), name="TOC_ENTRY_DESCRIPTOR", pack=False, align=None),
    "TOC_POS_TYPE": SimTypeInt(signed=False, label="TOC_POS_TYPE"),
    "TOGGLEKEYS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="TOGGLEKEYS", pack=False, align=None),
    "TOKENBINDING_EXTENSION_FORMAT": SimTypeInt(signed=False, label="TOKENBINDING_EXTENSION_FORMAT"),
    "TOKENBINDING_IDENTIFIER": SimStruct(OrderedDict((("keyType", SimTypeChar(label="Byte")),)), name="TOKENBINDING_IDENTIFIER", pack=False, align=None),
    "TOKENBINDING_KEY_PARAMETERS_TYPE": SimTypeInt(signed=False, label="TOKENBINDING_KEY_PARAMETERS_TYPE"),
    "TOKENBINDING_KEY_TYPES": SimStruct(OrderedDict((("keyCount", SimTypeInt(signed=False, label="UInt32")), ("keyType", SimTypePointer(SimTypeInt(signed=False, label="TOKENBINDING_KEY_PARAMETERS_TYPE"), offset=0)),)), name="TOKENBINDING_KEY_TYPES", pack=False, align=None),
    "TOKENBINDING_RESULT_DATA": SimStruct(OrderedDict((("bindingType", SimTypeInt(signed=False, label="TOKENBINDING_TYPE")), ("identifierSize", SimTypeInt(signed=False, label="UInt32")), ("identifierData", SimTypePointer(SimTypeRef("TOKENBINDING_IDENTIFIER", SimStruct), offset=0)), ("extensionFormat", SimTypeInt(signed=False, label="TOKENBINDING_EXTENSION_FORMAT")), ("extensionSize", SimTypeInt(signed=False, label="UInt32")), ("extensionData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKENBINDING_RESULT_DATA", pack=False, align=None),
    "TOKENBINDING_RESULT_LIST": SimStruct(OrderedDict((("resultCount", SimTypeInt(signed=False, label="UInt32")), ("resultData", SimTypePointer(SimTypeRef("TOKENBINDING_RESULT_DATA", SimStruct), offset=0)),)), name="TOKENBINDING_RESULT_LIST", pack=False, align=None),
    "TOKENBINDING_TYPE": SimTypeInt(signed=False, label="TOKENBINDING_TYPE"),
    "TOKEN_ACCESS_INFORMATION": SimStruct(OrderedDict((("SidHash", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES_HASH", SimStruct), offset=0)), ("RestrictedSidHash", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES_HASH", SimStruct), offset=0)), ("Privileges", SimTypePointer(SimTypeRef("TOKEN_PRIVILEGES", SimStruct), offset=0)), ("AuthenticationId", SimTypeRef("LUID", SimStruct)), ("TokenType", SimTypeInt(signed=False, label="TOKEN_TYPE")), ("ImpersonationLevel", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")), ("MandatoryPolicy", SimTypeRef("TOKEN_MANDATORY_POLICY", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AppContainerNumber", SimTypeInt(signed=False, label="UInt32")), ("PackageSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CapabilitiesHash", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES_HASH", SimStruct), offset=0)), ("TrustLevelSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SecurityAttributes", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_ACCESS_INFORMATION", pack=False, align=None),
    "TOKEN_ACCESS_MASK": SimTypeInt(signed=False, label="TOKEN_ACCESS_MASK"),
    "TOKEN_APPCONTAINER_INFORMATION": SimStruct(OrderedDict((("TokenAppContainer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_APPCONTAINER_INFORMATION", pack=False, align=None),
    "TOKEN_AUDIT_POLICY": SimStruct(OrderedDict((("PerUserPolicy", SimTypeArray(SimTypeChar(label="Byte"), 30)),)), name="TOKEN_AUDIT_POLICY", pack=False, align=None),
    "TOKEN_BNO_ISOLATION_INFORMATION": SimStruct(OrderedDict((("IsolationPrefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("IsolationEnabled", SimTypeChar(label="Byte")),)), name="TOKEN_BNO_ISOLATION_INFORMATION", pack=False, align=None),
    "TOKEN_CONTROL": SimStruct(OrderedDict((("TokenId", SimTypeRef("LUID", SimStruct)), ("AuthenticationId", SimTypeRef("LUID", SimStruct)), ("ModifiedId", SimTypeRef("LUID", SimStruct)), ("TokenSource", SimTypeRef("TOKEN_SOURCE", SimStruct)),)), name="TOKEN_CONTROL", pack=False, align=None),
    "TOKEN_DEFAULT_DACL": SimStruct(OrderedDict((("DefaultDacl", SimTypePointer(SimTypeRef("ACL", SimStruct), offset=0)),)), name="TOKEN_DEFAULT_DACL", pack=False, align=None),
    "TOKEN_DEVICE_CLAIMS": SimStruct(OrderedDict((("DeviceClaims", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_DEVICE_CLAIMS", pack=False, align=None),
    "TOKEN_ELEVATION": SimStruct(OrderedDict((("TokenIsElevated", SimTypeInt(signed=False, label="UInt32")),)), name="TOKEN_ELEVATION", pack=False, align=None),
    "TOKEN_ELEVATION_TYPE": SimTypeInt(signed=False, label="TOKEN_ELEVATION_TYPE"),
    "TOKEN_GROUPS": SimStruct(OrderedDict((("GroupCount", SimTypeInt(signed=False, label="UInt32")), ("Groups", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)),)), name="TOKEN_GROUPS", pack=False, align=None),
    "TOKEN_GROUPS_AND_PRIVILEGES": SimStruct(OrderedDict((("SidCount", SimTypeInt(signed=False, label="UInt32")), ("SidLength", SimTypeInt(signed=False, label="UInt32")), ("Sids", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)), ("RestrictedSidCount", SimTypeInt(signed=False, label="UInt32")), ("RestrictedSidLength", SimTypeInt(signed=False, label="UInt32")), ("RestrictedSids", SimTypePointer(SimTypeRef("SID_AND_ATTRIBUTES", SimStruct), offset=0)), ("PrivilegeCount", SimTypeInt(signed=False, label="UInt32")), ("PrivilegeLength", SimTypeInt(signed=False, label="UInt32")), ("Privileges", SimTypePointer(SimTypeRef("LUID_AND_ATTRIBUTES", SimStruct), offset=0)), ("AuthenticationId", SimTypeRef("LUID", SimStruct)),)), name="TOKEN_GROUPS_AND_PRIVILEGES", pack=False, align=None),
    "TOKEN_INFORMATION_CLASS": SimTypeInt(signed=False, label="TOKEN_INFORMATION_CLASS"),
    "TOKEN_LINKED_TOKEN": SimStruct(OrderedDict((("LinkedToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TOKEN_LINKED_TOKEN", pack=False, align=None),
    "TOKEN_MANDATORY_LABEL": SimStruct(OrderedDict((("Label", SimTypeRef("SID_AND_ATTRIBUTES", SimStruct)),)), name="TOKEN_MANDATORY_LABEL", pack=False, align=None),
    "TOKEN_MANDATORY_POLICY": SimStruct(OrderedDict((("Policy", SimTypeInt(signed=False, label="TOKEN_MANDATORY_POLICY_ID")),)), name="TOKEN_MANDATORY_POLICY", pack=False, align=None),
    "TOKEN_MANDATORY_POLICY_ID": SimTypeInt(signed=False, label="TOKEN_MANDATORY_POLICY_ID"),
    "TOKEN_ORIGIN": SimStruct(OrderedDict((("OriginatingLogonSession", SimTypeRef("LUID", SimStruct)),)), name="TOKEN_ORIGIN", pack=False, align=None),
    "TOKEN_OWNER": SimStruct(OrderedDict((("Owner", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_OWNER", pack=False, align=None),
    "TOKEN_PRIMARY_GROUP": SimStruct(OrderedDict((("PrimaryGroup", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_PRIMARY_GROUP", pack=False, align=None),
    "TOKEN_PRIVILEGES": SimStruct(OrderedDict((("PrivilegeCount", SimTypeInt(signed=False, label="UInt32")), ("Privileges", SimTypePointer(SimTypeRef("LUID_AND_ATTRIBUTES", SimStruct), offset=0)),)), name="TOKEN_PRIVILEGES", pack=False, align=None),
    "TOKEN_PRIVILEGES_ATTRIBUTES": SimTypeInt(signed=False, label="TOKEN_PRIVILEGES_ATTRIBUTES"),
    "TOKEN_SID_INFORMATION": SimStruct(OrderedDict((("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_SID_INFORMATION", pack=False, align=None),
    "TOKEN_SOURCE": SimStruct(OrderedDict((("SourceName", SimTypeArray(SimTypeChar(label="SByte"), 8)), ("SourceIdentifier", SimTypeRef("LUID", SimStruct)),)), name="TOKEN_SOURCE", pack=False, align=None),
    "TOKEN_STATISTICS": SimStruct(OrderedDict((("TokenId", SimTypeRef("LUID", SimStruct)), ("AuthenticationId", SimTypeRef("LUID", SimStruct)), ("ExpirationTime", SimTypeLongLong(signed=True, label="Int64")), ("TokenType", SimTypeInt(signed=False, label="TOKEN_TYPE")), ("ImpersonationLevel", SimTypeInt(signed=False, label="SECURITY_IMPERSONATION_LEVEL")), ("DynamicCharged", SimTypeInt(signed=False, label="UInt32")), ("DynamicAvailable", SimTypeInt(signed=False, label="UInt32")), ("GroupCount", SimTypeInt(signed=False, label="UInt32")), ("PrivilegeCount", SimTypeInt(signed=False, label="UInt32")), ("ModifiedId", SimTypeRef("LUID", SimStruct)),)), name="TOKEN_STATISTICS", pack=False, align=None),
    "TOKEN_TYPE": SimTypeInt(signed=False, label="TOKEN_TYPE"),
    "TOKEN_USER": SimStruct(OrderedDict((("User", SimTypeRef("SID_AND_ATTRIBUTES", SimStruct)),)), name="TOKEN_USER", pack=False, align=None),
    "TOKEN_USER_CLAIMS": SimStruct(OrderedDict((("UserClaims", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TOKEN_USER_CLAIMS", pack=False, align=None),
    "TOKEN_VALUE": SimStruct(OrderedDict((("pwszToken", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwValue", SimTypeInt(signed=False, label="UInt32")),)), name="TOKEN_VALUE", pack=False, align=None),
    "TOOLBARITEM": SimStruct(OrderedDict((("ptbar", SimTypeBottom(label="IDockingWindow")), ("rcBorderTool", SimTypeRef("RECT", SimStruct)), ("pwszItem", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fShow", SimTypeInt(signed=True, label="Int32")), ("hMon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TOOLBARITEM", pack=False, align=None),
    "TOOLBARPARTS": SimTypeInt(signed=False, label="TOOLBARPARTS"),
    "TOOLBARSTYLESTATES": SimTypeInt(signed=False, label="TOOLBARSTYLESTATES"),
    "TOOLTIPPARTS": SimTypeInt(signed=False, label="TOOLTIPPARTS"),
    "TOOLTIP_DISMISS_FLAGS": SimTypeInt(signed=False, label="TOOLTIP_DISMISS_FLAGS"),
    "TOOLTIP_FLAGS": SimTypeInt(signed=False, label="TOOLTIP_FLAGS"),
    "TOPTABITEMBOTHEDGESTATES": SimTypeInt(signed=False, label="TOPTABITEMBOTHEDGESTATES"),
    "TOPTABITEMLEFTEDGESTATES": SimTypeInt(signed=False, label="TOPTABITEMLEFTEDGESTATES"),
    "TOPTABITEMRIGHTEDGESTATES": SimTypeInt(signed=False, label="TOPTABITEMRIGHTEDGESTATES"),
    "TOPTABITEMSTATES": SimTypeInt(signed=False, label="TOPTABITEMSTATES"),
    "TOUCHEVENTF_FLAGS": SimTypeInt(signed=False, label="TOUCHEVENTF_FLAGS"),
    "TOUCHINPUT": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("hSource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwID", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="TOUCHEVENTF_FLAGS")), ("dwMask", SimTypeInt(signed=False, label="TOUCHINPUTMASKF_MASK")), ("dwTime", SimTypeInt(signed=False, label="UInt32")), ("dwExtraInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("cxContact", SimTypeInt(signed=False, label="UInt32")), ("cyContact", SimTypeInt(signed=False, label="UInt32")),)), name="TOUCHINPUT", pack=False, align=None),
    "TOUCHINPUTMASKF_MASK": SimTypeInt(signed=False, label="TOUCHINPUTMASKF_MASK"),
    "TOUCHPREDICTIONPARAMETERS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwLatency", SimTypeInt(signed=False, label="UInt32")), ("dwSampleTime", SimTypeInt(signed=False, label="UInt32")), ("bUseHWTimeStamp", SimTypeInt(signed=False, label="UInt32")),)), name="TOUCHPREDICTIONPARAMETERS", pack=False, align=None),
    "TOUCH_FEEDBACK_MODE": SimTypeInt(signed=False, label="TOUCH_FEEDBACK_MODE"),
    "TOUCH_HIT_TESTING_INPUT": SimStruct(OrderedDict((("pointerId", SimTypeInt(signed=False, label="UInt32")), ("point", SimTypeRef("POINT", SimStruct)), ("boundingBox", SimTypeRef("RECT", SimStruct)), ("nonOccludedBoundingBox", SimTypeRef("RECT", SimStruct)), ("orientation", SimTypeInt(signed=False, label="UInt32")),)), name="TOUCH_HIT_TESTING_INPUT", pack=False, align=None),
    "TOUCH_HIT_TESTING_PROXIMITY_EVALUATION": SimStruct(OrderedDict((("score", SimTypeShort(signed=False, label="UInt16")), ("adjustedPoint", SimTypeRef("POINT", SimStruct)),)), name="TOUCH_HIT_TESTING_PROXIMITY_EVALUATION", pack=False, align=None),
    "TPMPARAMS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcExclude", SimTypeRef("RECT", SimStruct)),)), name="TPMPARAMS", pack=False, align=None),
    "TPMVSCMGR_ERROR": SimTypeInt(signed=False, label="TPMVSCMGR_ERROR"),
    "TPMVSCMGR_STATUS": SimTypeInt(signed=False, label="TPMVSCMGR_STATUS"),
    "TPMVSC_ATTESTATION_TYPE": SimTypeInt(signed=False, label="TPMVSC_ATTESTATION_TYPE"),
    "TPM_DEVICE_INFO": SimStruct(OrderedDict((("structVersion", SimTypeInt(signed=False, label="UInt32")), ("tpmVersion", SimTypeInt(signed=False, label="UInt32")), ("tpmInterfaceType", SimTypeInt(signed=False, label="UInt32")), ("tpmImpRevision", SimTypeInt(signed=False, label="UInt32")),)), name="TPM_DEVICE_INFO", pack=False, align=None),
    "TPM_WNF_PROVISIONING": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="UInt32")), ("message", SimTypeArray(SimTypeChar(label="Byte"), 28)),)), name="TPM_WNF_PROVISIONING", pack=False, align=None),
    "TP_CALLBACK_ENVIRON_V3": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Pool", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CleanupGroup", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CleanupGroupCancelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ObjectContext", "CleanupContext"]), offset=0)), ("RaceDll", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ActivationContext", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FinalizationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Instance", "Context"]), offset=0)), ("u", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "s": SimTypeRef("_s_e__Struct", SimStruct)}, name="<anon>", label="None")), ("CallbackPriority", SimTypeInt(signed=False, label="TP_CALLBACK_PRIORITY")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="TP_CALLBACK_ENVIRON_V3", pack=False, align=None),
    "TP_CALLBACK_PRIORITY": SimTypeInt(signed=False, label="TP_CALLBACK_PRIORITY"),
    "TP_POOL_STACK_INFORMATION": SimStruct(OrderedDict((("StackReserve", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("StackCommit", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="TP_POOL_STACK_INFORMATION", pack=False, align=None),
    "TRACE_ENABLE_INFO": SimStruct(OrderedDict((("IsEnabled", SimTypeInt(signed=False, label="UInt32")), ("Level", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("LoggerId", SimTypeShort(signed=False, label="UInt16")), ("EnableProperty", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("MatchAnyKeyword", SimTypeLongLong(signed=False, label="UInt64")), ("MatchAllKeyword", SimTypeLongLong(signed=False, label="UInt64")),)), name="TRACE_ENABLE_INFO", pack=False, align=None),
    "TRACE_EVENT_INFO": SimStruct(OrderedDict((("ProviderGuid", SimTypeBottom(label="Guid")), ("EventGuid", SimTypeBottom(label="Guid")), ("EventDescriptor", SimTypeRef("EVENT_DESCRIPTOR", SimStruct)), ("DecodingSource", SimTypeInt(signed=False, label="DECODING_SOURCE")), ("ProviderNameOffset", SimTypeInt(signed=False, label="UInt32")), ("LevelNameOffset", SimTypeInt(signed=False, label="UInt32")), ("ChannelNameOffset", SimTypeInt(signed=False, label="UInt32")), ("KeywordsNameOffset", SimTypeInt(signed=False, label="UInt32")), ("TaskNameOffset", SimTypeInt(signed=False, label="UInt32")), ("OpcodeNameOffset", SimTypeInt(signed=False, label="UInt32")), ("EventMessageOffset", SimTypeInt(signed=False, label="UInt32")), ("ProviderMessageOffset", SimTypeInt(signed=False, label="UInt32")), ("BinaryXMLOffset", SimTypeInt(signed=False, label="UInt32")), ("BinaryXMLSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"EventNameOffset": SimTypeInt(signed=False, label="UInt32"), "ActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"EventAttributesOffset": SimTypeInt(signed=False, label="UInt32"), "RelatedActivityIDNameOffset": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("PropertyCount", SimTypeInt(signed=False, label="UInt32")), ("TopLevelPropertyCount", SimTypeInt(signed=False, label="UInt32")), ("Anonymous3", SimUnion({"Flags": SimTypeInt(signed=False, label="TEMPLATE_FLAGS"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("EventPropertyInfoArray", SimTypePointer(SimTypeRef("EVENT_PROPERTY_INFO", SimStruct), offset=0)),)), name="TRACE_EVENT_INFO", pack=False, align=None),
    "TRACE_GUID_INFO": SimStruct(OrderedDict((("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_GUID_INFO", pack=False, align=None),
    "TRACE_GUID_PROPERTIES": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("GuidType", SimTypeInt(signed=False, label="UInt32")), ("LoggerId", SimTypeInt(signed=False, label="UInt32")), ("EnableLevel", SimTypeInt(signed=False, label="UInt32")), ("EnableFlags", SimTypeInt(signed=False, label="UInt32")), ("IsEnable", SimTypeChar(label="Byte")),)), name="TRACE_GUID_PROPERTIES", pack=False, align=None),
    "TRACE_GUID_REGISTRATION": SimStruct(OrderedDict((("Guid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("RegHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TRACE_GUID_REGISTRATION", pack=False, align=None),
    "TRACE_INFORMATION_CLASS": SimTypeInt(signed=False, label="TRACE_INFORMATION_CLASS"),
    "TRACE_LOGFILE_HEADER": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"Version": SimTypeInt(signed=False, label="UInt32"), "VersionDetail": SimTypeRef("_VersionDetail_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ProviderVersion", SimTypeInt(signed=False, label="UInt32")), ("NumberOfProcessors", SimTypeInt(signed=False, label="UInt32")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("TimerResolution", SimTypeInt(signed=False, label="UInt32")), ("MaximumFileSize", SimTypeInt(signed=False, label="UInt32")), ("LogFileMode", SimTypeInt(signed=False, label="UInt32")), ("BuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"LogInstanceGuid": SimTypeBottom(label="Guid"), "Anonymous": SimStruct(OrderedDict((("StartBuffers", SimTypeInt(signed=False, label="UInt32")), ("PointerSize", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("CpuSpeedInMHz", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("LoggerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LogFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("TimeZone", SimTypeRef("TIME_ZONE_INFORMATION", SimStruct)), ("BootTime", SimTypeLongLong(signed=True, label="Int64")), ("PerfFreq", SimTypeLongLong(signed=True, label="Int64")), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("ReservedFlags", SimTypeInt(signed=False, label="UInt32")), ("BuffersLost", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_LOGFILE_HEADER", pack=False, align=None),
    "TRACE_LOGFILE_HEADER32": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"Version": SimTypeInt(signed=False, label="UInt32"), "VersionDetail": SimTypeRef("_VersionDetail_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ProviderVersion", SimTypeInt(signed=False, label="UInt32")), ("NumberOfProcessors", SimTypeInt(signed=False, label="UInt32")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("TimerResolution", SimTypeInt(signed=False, label="UInt32")), ("MaximumFileSize", SimTypeInt(signed=False, label="UInt32")), ("LogFileMode", SimTypeInt(signed=False, label="UInt32")), ("BuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"LogInstanceGuid": SimTypeBottom(label="Guid"), "Anonymous": SimStruct(OrderedDict((("StartBuffers", SimTypeInt(signed=False, label="UInt32")), ("PointerSize", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("CpuSpeedInMHz", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("LoggerName", SimTypeInt(signed=False, label="UInt32")), ("LogFileName", SimTypeInt(signed=False, label="UInt32")), ("TimeZone", SimTypeRef("TIME_ZONE_INFORMATION", SimStruct)), ("BootTime", SimTypeLongLong(signed=True, label="Int64")), ("PerfFreq", SimTypeLongLong(signed=True, label="Int64")), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("ReservedFlags", SimTypeInt(signed=False, label="UInt32")), ("BuffersLost", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_LOGFILE_HEADER32", pack=False, align=None),
    "TRACE_LOGFILE_HEADER64": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"Version": SimTypeInt(signed=False, label="UInt32"), "VersionDetail": SimTypeRef("_VersionDetail_e__Struct", SimStruct)}, name="<anon>", label="None")), ("ProviderVersion", SimTypeInt(signed=False, label="UInt32")), ("NumberOfProcessors", SimTypeInt(signed=False, label="UInt32")), ("EndTime", SimTypeLongLong(signed=True, label="Int64")), ("TimerResolution", SimTypeInt(signed=False, label="UInt32")), ("MaximumFileSize", SimTypeInt(signed=False, label="UInt32")), ("LogFileMode", SimTypeInt(signed=False, label="UInt32")), ("BuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("Anonymous2", SimUnion({"LogInstanceGuid": SimTypeBottom(label="Guid"), "Anonymous": SimStruct(OrderedDict((("StartBuffers", SimTypeInt(signed=False, label="UInt32")), ("PointerSize", SimTypeInt(signed=False, label="UInt32")), ("EventsLost", SimTypeInt(signed=False, label="UInt32")), ("CpuSpeedInMHz", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("LoggerName", SimTypeLongLong(signed=False, label="UInt64")), ("LogFileName", SimTypeLongLong(signed=False, label="UInt64")), ("TimeZone", SimTypeRef("TIME_ZONE_INFORMATION", SimStruct)), ("BootTime", SimTypeLongLong(signed=True, label="Int64")), ("PerfFreq", SimTypeLongLong(signed=True, label="Int64")), ("StartTime", SimTypeLongLong(signed=True, label="Int64")), ("ReservedFlags", SimTypeInt(signed=False, label="UInt32")), ("BuffersLost", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_LOGFILE_HEADER64", pack=False, align=None),
    "TRACE_MESSAGE_FLAGS": SimTypeInt(signed=False, label="TRACE_MESSAGE_FLAGS"),
    "TRACE_PERIODIC_CAPTURE_STATE_INFO": SimStruct(OrderedDict((("CaptureStateFrequencyInSeconds", SimTypeInt(signed=False, label="UInt32")), ("ProviderCount", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="TRACE_PERIODIC_CAPTURE_STATE_INFO", pack=False, align=None),
    "TRACE_PROFILE_INTERVAL": SimStruct(OrderedDict((("Source", SimTypeInt(signed=False, label="UInt32")), ("Interval", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_PROFILE_INTERVAL", pack=False, align=None),
    "TRACE_PROVIDER_INFO": SimStruct(OrderedDict((("ProviderGuid", SimTypeBottom(label="Guid")), ("SchemaSource", SimTypeInt(signed=False, label="UInt32")), ("ProviderNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_PROVIDER_INFO", pack=False, align=None),
    "TRACE_PROVIDER_INSTANCE_INFO": SimStruct(OrderedDict((("NextOffset", SimTypeInt(signed=False, label="UInt32")), ("EnableCount", SimTypeInt(signed=False, label="UInt32")), ("Pid", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_PROVIDER_INSTANCE_INFO", pack=False, align=None),
    "TRACE_QUERY_INFO_CLASS": SimTypeInt(signed=False, label="TRACE_QUERY_INFO_CLASS"),
    "TRACE_STACK_CACHING_INFO": SimStruct(OrderedDict((("Enabled", SimTypeChar(label="Byte")), ("CacheSize", SimTypeInt(signed=False, label="UInt32")), ("BucketCount", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_STACK_CACHING_INFO", pack=False, align=None),
    "TRACE_VERSION_INFO": SimStruct(OrderedDict((("EtwTraceProcessingVersion", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="TRACE_VERSION_INFO", pack=False, align=None),
    "TRACKBARPARTS": SimTypeInt(signed=False, label="TRACKBARPARTS"),
    "TRACKBARSTYLESTATES": SimTypeInt(signed=False, label="TRACKBARSTYLESTATES"),
    "TRACKING_COLL_TYPE": SimTypeInt(signed=False, label="TRACKING_COLL_TYPE"),
    "TRACKMOUSEEVENT": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="TRACKMOUSEEVENT_FLAGS")), ("hwndTrack", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwHoverTime", SimTypeInt(signed=False, label="UInt32")),)), name="TRACKMOUSEEVENT", pack=False, align=None),
    "TRACKMOUSEEVENT_FLAGS": SimTypeInt(signed=False, label="TRACKMOUSEEVENT_FLAGS"),
    "TRACKSTATES": SimTypeInt(signed=False, label="TRACKSTATES"),
    "TRACKVERTSTATES": SimTypeInt(signed=False, label="TRACKVERTSTATES"),
    "TRACK_POPUP_MENU_FLAGS": SimTypeInt(signed=False, label="TRACK_POPUP_MENU_FLAGS"),
    "TRAILINGGRIDCELLSTATES": SimTypeInt(signed=False, label="TRAILINGGRIDCELLSTATES"),
    "TRAILINGGRIDCELLUPPERSTATES": SimTypeInt(signed=False, label="TRAILINGGRIDCELLUPPERSTATES"),
    "TRANSACTIONMANAGER_BASIC_INFORMATION": SimStruct(OrderedDict((("TmIdentity", SimTypeBottom(label="Guid")), ("VirtualClock", SimTypeLongLong(signed=True, label="Int64")),)), name="TRANSACTIONMANAGER_BASIC_INFORMATION", pack=False, align=None),
    "TRANSACTIONMANAGER_INFORMATION_CLASS": SimTypeInt(signed=False, label="TRANSACTIONMANAGER_INFORMATION_CLASS"),
    "TRANSACTIONMANAGER_LOGPATH_INFORMATION": SimStruct(OrderedDict((("LogPathLength", SimTypeInt(signed=False, label="UInt32")), ("LogPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TRANSACTIONMANAGER_LOGPATH_INFORMATION", pack=False, align=None),
    "TRANSACTIONMANAGER_LOG_INFORMATION": SimStruct(OrderedDict((("LogIdentity", SimTypeBottom(label="Guid")),)), name="TRANSACTIONMANAGER_LOG_INFORMATION", pack=False, align=None),
    "TRANSACTIONMANAGER_OLDEST_INFORMATION": SimStruct(OrderedDict((("OldestTransactionGuid", SimTypeBottom(label="Guid")),)), name="TRANSACTIONMANAGER_OLDEST_INFORMATION", pack=False, align=None),
    "TRANSACTIONMANAGER_RECOVERY_INFORMATION": SimStruct(OrderedDict((("LastRecoveredLsn", SimTypeLongLong(signed=False, label="UInt64")),)), name="TRANSACTIONMANAGER_RECOVERY_INFORMATION", pack=False, align=None),
    "TRANSACTION_BASIC_INFORMATION": SimStruct(OrderedDict((("TransactionId", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="UInt32")), ("Outcome", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSACTION_BASIC_INFORMATION", pack=False, align=None),
    "TRANSACTION_BIND_INFORMATION": SimStruct(OrderedDict((("TmHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TRANSACTION_BIND_INFORMATION", pack=False, align=None),
    "TRANSACTION_ENLISTMENTS_INFORMATION": SimStruct(OrderedDict((("NumberOfEnlistments", SimTypeInt(signed=False, label="UInt32")), ("EnlistmentPair", SimTypePointer(SimTypeRef("TRANSACTION_ENLISTMENT_PAIR", SimStruct), offset=0)),)), name="TRANSACTION_ENLISTMENTS_INFORMATION", pack=False, align=None),
    "TRANSACTION_ENLISTMENT_PAIR": SimStruct(OrderedDict((("EnlistmentId", SimTypeBottom(label="Guid")), ("ResourceManagerId", SimTypeBottom(label="Guid")),)), name="TRANSACTION_ENLISTMENT_PAIR", pack=False, align=None),
    "TRANSACTION_INFORMATION_CLASS": SimTypeInt(signed=False, label="TRANSACTION_INFORMATION_CLASS"),
    "TRANSACTION_LIST_ENTRY": SimStruct(OrderedDict((("UOW", SimTypeBottom(label="Guid")),)), name="TRANSACTION_LIST_ENTRY", pack=False, align=None),
    "TRANSACTION_LIST_INFORMATION": SimStruct(OrderedDict((("NumberOfTransactions", SimTypeInt(signed=False, label="UInt32")), ("TransactionInformation", SimTypePointer(SimTypeRef("TRANSACTION_LIST_ENTRY", SimStruct), offset=0)),)), name="TRANSACTION_LIST_INFORMATION", pack=False, align=None),
    "TRANSACTION_NOTIFICATION": SimStruct(OrderedDict((("TransactionKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransactionNotification", SimTypeInt(signed=False, label="UInt32")), ("TmVirtualClock", SimTypeLongLong(signed=True, label="Int64")), ("ArgumentLength", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSACTION_NOTIFICATION", pack=False, align=None),
    "TRANSACTION_NOTIFICATION_MARSHAL_ARGUMENT": SimStruct(OrderedDict((("MarshalCookie", SimTypeInt(signed=False, label="UInt32")), ("UOW", SimTypeBottom(label="Guid")),)), name="TRANSACTION_NOTIFICATION_MARSHAL_ARGUMENT", pack=False, align=None),
    "TRANSACTION_NOTIFICATION_PROPAGATE_ARGUMENT": SimStruct(OrderedDict((("PropagationCookie", SimTypeInt(signed=False, label="UInt32")), ("UOW", SimTypeBottom(label="Guid")), ("TmIdentity", SimTypeBottom(label="Guid")), ("BufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSACTION_NOTIFICATION_PROPAGATE_ARGUMENT", pack=False, align=None),
    "TRANSACTION_NOTIFICATION_RECOVERY_ARGUMENT": SimStruct(OrderedDict((("EnlistmentId", SimTypeBottom(label="Guid")), ("UOW", SimTypeBottom(label="Guid")),)), name="TRANSACTION_NOTIFICATION_RECOVERY_ARGUMENT", pack=False, align=None),
    "TRANSACTION_NOTIFICATION_SAVEPOINT_ARGUMENT": SimStruct(OrderedDict((("SavepointId", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSACTION_NOTIFICATION_SAVEPOINT_ARGUMENT", pack=False, align=None),
    "TRANSACTION_NOTIFICATION_TM_ONLINE_ARGUMENT": SimStruct(OrderedDict((("TmIdentity", SimTypeBottom(label="Guid")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSACTION_NOTIFICATION_TM_ONLINE_ARGUMENT", pack=False, align=None),
    "TRANSACTION_OUTCOME": SimTypeInt(signed=False, label="TRANSACTION_OUTCOME"),
    "TRANSACTION_PROPERTIES_INFORMATION": SimStruct(OrderedDict((("IsolationLevel", SimTypeInt(signed=False, label="UInt32")), ("IsolationFlags", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeLongLong(signed=True, label="Int64")), ("Outcome", SimTypeInt(signed=False, label="UInt32")), ("DescriptionLength", SimTypeInt(signed=False, label="UInt32")), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TRANSACTION_PROPERTIES_INFORMATION", pack=False, align=None),
    "TRANSACTION_STATE": SimTypeInt(signed=False, label="TRANSACTION_STATE"),
    "TRANSACTION_SUPERIOR_ENLISTMENT_INFORMATION": SimStruct(OrderedDict((("SuperiorEnlistmentPair", SimTypeRef("TRANSACTION_ENLISTMENT_PAIR", SimStruct)),)), name="TRANSACTION_SUPERIOR_ENLISTMENT_INFORMATION", pack=False, align=None),
    "TRANSDATA": SimStruct(OrderedDict((("ubCodePageID", SimTypeChar(label="Byte")), ("ubType", SimTypeChar(label="Byte")), ("uCode", SimUnion({"sCode": SimTypeShort(signed=True, label="Int16"), "ubCode": SimTypeChar(label="Byte"), "ubPairs": SimTypeArray(SimTypeChar(label="Byte"), 2)}, name="<anon>", label="None")),)), name="TRANSDATA", pack=False, align=None),
    "TRANSLATEURL_IN_FLAGS": SimTypeInt(signed=False, label="TRANSLATEURL_IN_FLAGS"),
    "TRANSLATE_BUS_ADDRESS": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeChar(label="Byte"), arg_names=["Context", "BusAddress", "Length", "AddressSpace", "TranslatedAddress"]), offset=0),
    "TRANSLATE_CHARSET_INFO_FLAGS": SimTypeInt(signed=False, label="TRANSLATE_CHARSET_INFO_FLAGS"),
    "TRANSLATE_VIRTUAL_TO_PHYSICAL": SimStruct(OrderedDict((("Virtual", SimTypeLongLong(signed=False, label="UInt64")), ("Physical", SimTypeLongLong(signed=False, label="UInt64")),)), name="TRANSLATE_VIRTUAL_TO_PHYSICAL", pack=False, align=None),
    "TRANSLATION_PARAMETER": SimTypeInt(signed=False, label="TRANSLATION_PARAMETER"),
    "TRANSLATOR_INTERFACE": SimStruct(OrderedDict((("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("InterfaceReference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("InterfaceDereference", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("TranslateResources", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="RESOURCE_TRANSLATION_DIRECTION"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("CM_PARTIAL_RESOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Source", "Direction", "AlternativesCount", "Alternatives", "PhysicalDeviceObject", "Target"]), offset=0)), ("TranslateResourceRequirements", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("IO_RESOURCE_DESCRIPTOR", SimStruct), offset=0), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Source", "PhysicalDeviceObject", "TargetCount", "Target"]), offset=0)),)), name="TRANSLATOR_INTERFACE", pack=False, align=None),
    "TRANSMIT_FILE_BUFFERS": SimStruct(OrderedDict((("Head", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("HeadLength", SimTypeInt(signed=False, label="UInt32")), ("Tail", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TailLength", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSMIT_FILE_BUFFERS", pack=False, align=None),
    "TRANSMIT_PACKETS_ELEMENT": SimStruct(OrderedDict((("dwElFlags", SimTypeInt(signed=False, label="UInt32")), ("cLength", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("nFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_Anonymous_e__Struct", pack=False, align=None), "pBuffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")),)), name="TRANSMIT_PACKETS_ELEMENT", pack=False, align=None),
    "TRANSMSG": SimStruct(OrderedDict((("message", SimTypeInt(signed=False, label="UInt32")), ("wParam", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TRANSMSG", pack=False, align=None),
    "TRANSMSGLIST": SimStruct(OrderedDict((("uMsgCount", SimTypeInt(signed=False, label="UInt32")), ("TransMsg", SimTypePointer(SimTypeRef("TRANSMSG", SimStruct), offset=0)),)), name="TRANSMSGLIST", pack=False, align=None),
    "TRANSPARENTBACKGROUNDSTATES": SimTypeInt(signed=False, label="TRANSPARENTBACKGROUNDSTATES"),
    "TRANSPARENTBARSTATES": SimTypeInt(signed=False, label="TRANSPARENTBARSTATES"),
    "TRANSPARENTBARVERTSTATES": SimTypeInt(signed=False, label="TRANSPARENTBARVERTSTATES"),
    "TRANSPORTAUDIOPARMS": SimStruct(OrderedDict((("EnableOutput", SimTypeInt(signed=True, label="Int32")), ("EnableRecord", SimTypeInt(signed=True, label="Int32")), ("EnableSelsync", SimTypeInt(signed=True, label="Int32")), ("Input", SimTypeInt(signed=True, label="Int32")), ("MonitorSource", SimTypeInt(signed=True, label="Int32")),)), name="TRANSPORTAUDIOPARMS", pack=False, align=None),
    "TRANSPORTBASICPARMS": SimStruct(OrderedDict((("TimeFormat", SimTypeInt(signed=True, label="Int32")), ("TimeReference", SimTypeInt(signed=True, label="Int32")), ("Superimpose", SimTypeInt(signed=True, label="Int32")), ("EndStopAction", SimTypeInt(signed=True, label="Int32")), ("RecordFormat", SimTypeInt(signed=True, label="Int32")), ("StepFrames", SimTypeInt(signed=True, label="Int32")), ("SetpField", SimTypeInt(signed=True, label="Int32")), ("Preroll", SimTypeInt(signed=True, label="Int32")), ("RecPreroll", SimTypeInt(signed=True, label="Int32")), ("Postroll", SimTypeInt(signed=True, label="Int32")), ("EditDelay", SimTypeInt(signed=True, label="Int32")), ("PlayTCDelay", SimTypeInt(signed=True, label="Int32")), ("RecTCDelay", SimTypeInt(signed=True, label="Int32")), ("EditField", SimTypeInt(signed=True, label="Int32")), ("FrameServo", SimTypeInt(signed=True, label="Int32")), ("ColorFrameServo", SimTypeInt(signed=True, label="Int32")), ("ServoRef", SimTypeInt(signed=True, label="Int32")), ("WarnGenlock", SimTypeInt(signed=True, label="Int32")), ("SetTracking", SimTypeInt(signed=True, label="Int32")), ("VolumeName", SimTypeArray(SimTypeChar(label="SByte"), 40)), ("Ballistic", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 20)), ("Speed", SimTypeInt(signed=True, label="Int32")), ("CounterFormat", SimTypeInt(signed=True, label="Int32")), ("TunerChannel", SimTypeInt(signed=True, label="Int32")), ("TunerNumber", SimTypeInt(signed=True, label="Int32")), ("TimerEvent", SimTypeInt(signed=True, label="Int32")), ("TimerStartDay", SimTypeInt(signed=True, label="Int32")), ("TimerStartTime", SimTypeInt(signed=True, label="Int32")), ("TimerStopDay", SimTypeInt(signed=True, label="Int32")), ("TimerStopTime", SimTypeInt(signed=True, label="Int32")),)), name="TRANSPORTBASICPARMS", pack=False, align=None),
    "TRANSPORTCLIENT_CALLBACK_ID": SimTypeInt(signed=False, label="TRANSPORTCLIENT_CALLBACK_ID"),
    "TRANSPORTCLIENT_SESSION_INFO": SimStruct(OrderedDict((("ulStructureLength", SimTypeInt(signed=False, label="UInt32")), ("ullFileSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulBlockSize", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSPORTCLIENT_SESSION_INFO", pack=False, align=None),
    "TRANSPORTPROVIDER_CALLBACK_ID": SimTypeInt(signed=False, label="TRANSPORTPROVIDER_CALLBACK_ID"),
    "TRANSPORTSTATUS": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=True, label="Int32")), ("LastError", SimTypeInt(signed=True, label="Int32")), ("RecordInhibit", SimTypeInt(signed=True, label="Int32")), ("ServoLock", SimTypeInt(signed=True, label="Int32")), ("MediaPresent", SimTypeInt(signed=True, label="Int32")), ("MediaLength", SimTypeInt(signed=True, label="Int32")), ("MediaSize", SimTypeInt(signed=True, label="Int32")), ("MediaTrackCount", SimTypeInt(signed=True, label="Int32")), ("MediaTrackLength", SimTypeInt(signed=True, label="Int32")), ("MediaTrackSide", SimTypeInt(signed=True, label="Int32")), ("MediaType", SimTypeInt(signed=True, label="Int32")), ("LinkMode", SimTypeInt(signed=True, label="Int32")), ("NotifyOn", SimTypeInt(signed=True, label="Int32")),)), name="TRANSPORTSTATUS", pack=False, align=None),
    "TRANSPORTVIDEOPARMS": SimStruct(OrderedDict((("OutputMode", SimTypeInt(signed=True, label="Int32")), ("Input", SimTypeInt(signed=True, label="Int32")),)), name="TRANSPORTVIDEOPARMS", pack=False, align=None),
    "TRANSPORT_HEADER_OFFSET": SimStruct(OrderedDict((("ProtocolType", SimTypeShort(signed=False, label="UInt16")), ("HeaderOffset", SimTypeShort(signed=False, label="UInt16")),)), name="TRANSPORT_HEADER_OFFSET", pack=False, align=None),
    "TRANSPORT_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="TRANSPORT_TYPE")), ("SkipCertificateCheck", SimTypeChar(label="Byte")),)), name="TRANSPORT_INFO", pack=False, align=None),
    "TRANSPORT_PROPERTIES": SimStruct(OrderedDict((("PID", SimTypeInt(signed=False, label="UInt32")), ("PCR", SimTypeLongLong(signed=True, label="Int64")), ("Fields", SimUnion({"Others": SimTypeRef("_Others", SimStruct), "Value": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="TRANSPORT_PROPERTIES", pack=False, align=None),
    "TRANSPORT_SETTING": SimStruct(OrderedDict((("SettingId", SimTypeRef("TRANSPORT_SETTING_ID", SimStruct)), ("Length", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TRANSPORT_SETTING", pack=False, align=None),
    "TRANSPORT_SETTING_ID": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")),)), name="TRANSPORT_SETTING_ID", pack=False, align=None),
    "TRANSPORT_STATE": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="UInt32")),)), name="TRANSPORT_STATE", pack=False, align=None),
    "TRANSPORT_TYPE": SimTypeInt(signed=False, label="TRANSPORT_TYPE"),
    "TRAYNOTIFYPARTS": SimTypeInt(signed=False, label="TRAYNOTIFYPARTS"),
    "TREEITEMSTATES": SimTypeInt(signed=False, label="TREEITEMSTATES"),
    "TREEVIEWPARTS": SimTypeInt(signed=False, label="TREEVIEWPARTS"),
    "TREE_SEC_INFO": SimTypeInt(signed=False, label="TREE_SEC_INFO"),
    "TREE_VIEW_ITEM_STATE_FLAGS": SimTypeInt(signed=False, label="TREE_VIEW_ITEM_STATE_FLAGS"),
    "TRIGGER_TYPE_UNION": SimUnion({"Daily": SimTypeRef("DAILY", SimStruct), "Weekly": SimTypeRef("WEEKLY", SimStruct), "MonthlyDate": SimTypeRef("MONTHLYDATE", SimStruct), "MonthlyDOW": SimTypeRef("MONTHLYDOW", SimStruct)}, name="<anon>", label="None"),
    "TRIVERTEX": SimStruct(OrderedDict((("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("Red", SimTypeShort(signed=False, label="UInt16")), ("Green", SimTypeShort(signed=False, label="UInt16")), ("Blue", SimTypeShort(signed=False, label="UInt16")), ("Alpha", SimTypeShort(signed=False, label="UInt16")),)), name="TRIVERTEX", pack=False, align=None),
    "TRP": SimStruct(OrderedDict((("trpid", SimTypeShort(signed=False, label="UInt16")), ("cbgrtrp", SimTypeShort(signed=False, label="UInt16")), ("cch", SimTypeShort(signed=False, label="UInt16")), ("cbRgb", SimTypeShort(signed=False, label="UInt16")),)), name="TRP", pack=False, align=None),
    "TRUECOLORINFO": SimStruct(OrderedDict((("dwBitMasks", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("bmiColors", SimTypeArray(SimTypeRef("RGBQUAD", SimStruct), 256)),)), name="TRUECOLORINFO", pack=False, align=None),
    "TRUESIZESCALINGTYPE": SimTypeInt(signed=False, label="TRUESIZESCALINGTYPE"),
    "TRUESPEECHWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wRevision", SimTypeShort(signed=False, label="UInt16")), ("nSamplesPerBlock", SimTypeShort(signed=False, label="UInt16")), ("abReserved", SimTypeArray(SimTypeChar(label="Byte"), 28)),)), name="TRUESPEECHWAVEFORMAT", pack=False, align=None),
    "TRUSTED_CONTROLLERS_INFO": SimStruct(OrderedDict((("Entries", SimTypeInt(signed=False, label="UInt32")), ("Names", SimTypePointer(SimTypeRef("LSA_UNICODE_STRING", SimStruct), offset=0)),)), name="TRUSTED_CONTROLLERS_INFO", pack=False, align=None),
    "TRUSTED_DOMAIN_AUTH_INFORMATION": SimStruct(OrderedDict((("IncomingAuthInfos", SimTypeInt(signed=False, label="UInt32")), ("IncomingAuthenticationInformation", SimTypePointer(SimTypeRef("LSA_AUTH_INFORMATION", SimStruct), offset=0)), ("IncomingPreviousAuthenticationInformation", SimTypePointer(SimTypeRef("LSA_AUTH_INFORMATION", SimStruct), offset=0)), ("OutgoingAuthInfos", SimTypeInt(signed=False, label="UInt32")), ("OutgoingAuthenticationInformation", SimTypePointer(SimTypeRef("LSA_AUTH_INFORMATION", SimStruct), offset=0)), ("OutgoingPreviousAuthenticationInformation", SimTypePointer(SimTypeRef("LSA_AUTH_INFORMATION", SimStruct), offset=0)),)), name="TRUSTED_DOMAIN_AUTH_INFORMATION", pack=False, align=None),
    "TRUSTED_DOMAIN_FULL_INFORMATION": SimStruct(OrderedDict((("Information", SimTypeRef("TRUSTED_DOMAIN_INFORMATION_EX", SimStruct)), ("PosixOffset", SimTypeRef("TRUSTED_POSIX_OFFSET_INFO", SimStruct)), ("AuthInformation", SimTypeRef("TRUSTED_DOMAIN_AUTH_INFORMATION", SimStruct)),)), name="TRUSTED_DOMAIN_FULL_INFORMATION", pack=False, align=None),
    "TRUSTED_DOMAIN_FULL_INFORMATION2": SimStruct(OrderedDict((("Information", SimTypeRef("TRUSTED_DOMAIN_INFORMATION_EX2", SimStruct)), ("PosixOffset", SimTypeRef("TRUSTED_POSIX_OFFSET_INFO", SimStruct)), ("AuthInformation", SimTypeRef("TRUSTED_DOMAIN_AUTH_INFORMATION", SimStruct)),)), name="TRUSTED_DOMAIN_FULL_INFORMATION2", pack=False, align=None),
    "TRUSTED_DOMAIN_INFORMATION_EX": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("FlatName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TrustDirection", SimTypeInt(signed=False, label="TRUSTED_DOMAIN_TRUST_DIRECTION")), ("TrustType", SimTypeInt(signed=False, label="TRUSTED_DOMAIN_TRUST_TYPE")), ("TrustAttributes", SimTypeInt(signed=False, label="TRUSTED_DOMAIN_TRUST_ATTRIBUTES")),)), name="TRUSTED_DOMAIN_INFORMATION_EX", pack=False, align=None),
    "TRUSTED_DOMAIN_INFORMATION_EX2": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("FlatName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TrustDirection", SimTypeInt(signed=False, label="UInt32")), ("TrustType", SimTypeInt(signed=False, label="UInt32")), ("TrustAttributes", SimTypeInt(signed=False, label="UInt32")), ("ForestTrustLength", SimTypeInt(signed=False, label="UInt32")), ("ForestTrustInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TRUSTED_DOMAIN_INFORMATION_EX2", pack=False, align=None),
    "TRUSTED_DOMAIN_NAME_INFO": SimStruct(OrderedDict((("Name", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="TRUSTED_DOMAIN_NAME_INFO", pack=False, align=None),
    "TRUSTED_DOMAIN_SUPPORTED_ENCRYPTION_TYPES": SimStruct(OrderedDict((("SupportedEncryptionTypes", SimTypeInt(signed=False, label="UInt32")),)), name="TRUSTED_DOMAIN_SUPPORTED_ENCRYPTION_TYPES", pack=False, align=None),
    "TRUSTED_DOMAIN_TRUST_ATTRIBUTES": SimTypeInt(signed=False, label="TRUSTED_DOMAIN_TRUST_ATTRIBUTES"),
    "TRUSTED_DOMAIN_TRUST_DIRECTION": SimTypeInt(signed=False, label="TRUSTED_DOMAIN_TRUST_DIRECTION"),
    "TRUSTED_DOMAIN_TRUST_TYPE": SimTypeInt(signed=False, label="TRUSTED_DOMAIN_TRUST_TYPE"),
    "TRUSTED_INFORMATION_CLASS": SimTypeInt(signed=False, label="TRUSTED_INFORMATION_CLASS"),
    "TRUSTED_PASSWORD_INFO": SimStruct(OrderedDict((("Password", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("OldPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)),)), name="TRUSTED_PASSWORD_INFO", pack=False, align=None),
    "TRUSTED_POSIX_OFFSET_INFO": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="TRUSTED_POSIX_OFFSET_INFO", pack=False, align=None),
    "TRUSTEE_A": SimStruct(OrderedDict((("pMultipleTrustee", SimTypePointer(SimTypeRef("TRUSTEE_A", SimStruct), offset=0)), ("MultipleTrusteeOperation", SimTypeInt(signed=False, label="MULTIPLE_TRUSTEE_OPERATION")), ("TrusteeForm", SimTypeInt(signed=False, label="TRUSTEE_FORM")), ("TrusteeType", SimTypeInt(signed=False, label="TRUSTEE_TYPE")), ("ptstrName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TRUSTEE_A", pack=False, align=None),
    "TRUSTEE_ACCESSA": SimStruct(OrderedDict((("lpProperty", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Access", SimTypeInt(signed=False, label="UInt32")), ("fAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("fReturnedAccess", SimTypeInt(signed=False, label="UInt32")),)), name="TRUSTEE_ACCESSA", pack=False, align=None),
    "TRUSTEE_ACCESSW": SimStruct(OrderedDict((("lpProperty", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Access", SimTypeInt(signed=False, label="UInt32")), ("fAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("fReturnedAccess", SimTypeInt(signed=False, label="UInt32")),)), name="TRUSTEE_ACCESSW", pack=False, align=None),
    "TRUSTEE_FORM": SimTypeInt(signed=False, label="TRUSTEE_FORM"),
    "TRUSTEE_TYPE": SimTypeInt(signed=False, label="TRUSTEE_TYPE"),
    "TRUSTEE_W": SimStruct(OrderedDict((("pMultipleTrustee", SimTypePointer(SimTypeRef("TRUSTEE_W", SimStruct), offset=0)), ("MultipleTrusteeOperation", SimTypeInt(signed=False, label="MULTIPLE_TRUSTEE_OPERATION")), ("TrusteeForm", SimTypeInt(signed=False, label="TRUSTEE_FORM")), ("TrusteeType", SimTypeInt(signed=False, label="TRUSTEE_TYPE")), ("ptstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TRUSTEE_W", pack=False, align=None),
    "TR_FILTER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "TSPUB_PLUGIN_PD_ASSIGNMENT_TYPE": SimTypeInt(signed=False, label="TSPUB_PLUGIN_PD_ASSIGNMENT_TYPE"),
    "TSPUB_PLUGIN_PD_RESOLUTION_TYPE": SimTypeInt(signed=False, label="TSPUB_PLUGIN_PD_RESOLUTION_TYPE"),
    "TSSB_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="TSSB_NOTIFICATION_TYPE"),
    "TSSD_AddrV46Type": SimTypeInt(signed=False, label="TSSD_AddrV46Type"),
    "TSSD_ConnectionPoint": SimStruct(OrderedDict((("ServerAddressB", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("AddressType", SimTypeInt(signed=False, label="TSSD_AddrV46Type")), ("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("AddressScope", SimTypeInt(signed=False, label="UInt32")),)), name="TSSD_ConnectionPoint", pack=False, align=None),
    "TSSESSION_STATE": SimTypeInt(signed=False, label="TSSESSION_STATE"),
    "TS_ATTRVAL": SimStruct(OrderedDict((("idAttr", SimTypeBottom(label="Guid")), ("dwOverlapId", SimTypeInt(signed=False, label="UInt32")), ("varValue", SimTypeRef("VARIANT", SimStruct)),)), name="TS_ATTRVAL", pack=False, align=None),
    "TS_RUNINFO": SimStruct(OrderedDict((("uCount", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="TsRunType")),)), name="TS_RUNINFO", pack=False, align=None),
    "TS_SB_SORT_BY": SimTypeInt(signed=False, label="TS_SB_SORT_BY"),
    "TS_SELECTIONSTYLE": SimStruct(OrderedDict((("ase", SimTypeInt(signed=False, label="TsActiveSelEnd")), ("fInterimChar", SimTypeInt(signed=True, label="Int32")),)), name="TS_SELECTIONSTYLE", pack=False, align=None),
    "TS_SELECTION_ACP": SimStruct(OrderedDict((("acpStart", SimTypeInt(signed=True, label="Int32")), ("acpEnd", SimTypeInt(signed=True, label="Int32")), ("style", SimTypeRef("TS_SELECTIONSTYLE", SimStruct)),)), name="TS_SELECTION_ACP", pack=False, align=None),
    "TS_SELECTION_ANCHOR": SimStruct(OrderedDict((("paStart", SimTypeBottom(label="IAnchor")), ("paEnd", SimTypeBottom(label="IAnchor")), ("style", SimTypeRef("TS_SELECTIONSTYLE", SimStruct)),)), name="TS_SELECTION_ANCHOR", pack=False, align=None),
    "TS_STATUS": SimStruct(OrderedDict((("dwDynamicFlags", SimTypeInt(signed=False, label="UInt32")), ("dwStaticFlags", SimTypeInt(signed=False, label="UInt32")),)), name="TS_STATUS", pack=False, align=None),
    "TS_TEXTCHANGE": SimStruct(OrderedDict((("acpStart", SimTypeInt(signed=True, label="Int32")), ("acpOldEnd", SimTypeInt(signed=True, label="Int32")), ("acpNewEnd", SimTypeInt(signed=True, label="Int32")),)), name="TS_TEXTCHANGE", pack=False, align=None),
    "TTEMBEDINFO": SimStruct(OrderedDict((("usStructSize", SimTypeShort(signed=False, label="UInt16")), ("usRootStrSize", SimTypeShort(signed=False, label="UInt16")), ("pusRootStr", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="TTEMBEDINFO", pack=False, align=None),
    "TTEMBED_FLAGS": SimTypeInt(signed=False, label="TTEMBED_FLAGS"),
    "TTGETTITLE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("uTitleBitmap", SimTypeInt(signed=False, label="UInt32")), ("cch", SimTypeInt(signed=False, label="UInt32")), ("pszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TTGETTITLE", pack=False, align=None),
    "TTHITTESTINFOA": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pt", SimTypeRef("POINT", SimStruct)), ("ti", SimTypeRef("TTTOOLINFOA", SimStruct)),)), name="TTHITTESTINFOA", pack=False, align=None),
    "TTHITTESTINFOW": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pt", SimTypeRef("POINT", SimStruct)), ("ti", SimTypeRef("TTTOOLINFOW", SimStruct)),)), name="TTHITTESTINFOW", pack=False, align=None),
    "TTLOADINFO": SimStruct(OrderedDict((("usStructSize", SimTypeShort(signed=False, label="UInt16")), ("usRefStrSize", SimTypeShort(signed=False, label="UInt16")), ("pusRefStr", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="TTLOADINFO", pack=False, align=None),
    "TTLOAD_EMBEDDED_FONT_STATUS": SimTypeInt(signed=False, label="TTLOAD_EMBEDDED_FONT_STATUS"),
    "TTPOLYCURVE": SimStruct(OrderedDict((("wType", SimTypeShort(signed=False, label="UInt16")), ("cpfx", SimTypeShort(signed=False, label="UInt16")), ("apfx", SimTypePointer(SimTypeRef("POINTFX", SimStruct), offset=0)),)), name="TTPOLYCURVE", pack=False, align=None),
    "TTPOLYGONHEADER": SimStruct(OrderedDict((("cb", SimTypeInt(signed=False, label="UInt32")), ("dwType", SimTypeInt(signed=False, label="UInt32")), ("pfxStart", SimTypeRef("POINTFX", SimStruct)),)), name="TTPOLYGONHEADER", pack=False, align=None),
    "TTTOOLINFOA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="TOOLTIP_FLAGS")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rect", SimTypeRef("RECT", SimStruct)), ("hinst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TTTOOLINFOA", pack=False, align=None),
    "TTTOOLINFOW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("uFlags", SimTypeInt(signed=False, label="TOOLTIP_FLAGS")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("rect", SimTypeRef("RECT", SimStruct)), ("hinst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TTTOOLINFOW", pack=False, align=None),
    "TTVALIDATIONTESTSPARAMS": SimStruct(OrderedDict((("ulStructSize", SimTypeInt(signed=False, label="UInt32")), ("lTestFromSize", SimTypeInt(signed=True, label="Int32")), ("lTestToSize", SimTypeInt(signed=True, label="Int32")), ("ulCharSet", SimTypeInt(signed=False, label="UInt32")), ("usReserved1", SimTypeShort(signed=False, label="UInt16")), ("usCharCodeCount", SimTypeShort(signed=False, label="UInt16")), ("pusCharCodeSet", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="TTVALIDATIONTESTSPARAMS", pack=False, align=None),
    "TTVALIDATIONTESTSPARAMSEX": SimStruct(OrderedDict((("ulStructSize", SimTypeInt(signed=False, label="UInt32")), ("lTestFromSize", SimTypeInt(signed=True, label="Int32")), ("lTestToSize", SimTypeInt(signed=True, label="Int32")), ("ulCharSet", SimTypeInt(signed=False, label="UInt32")), ("usReserved1", SimTypeShort(signed=False, label="UInt16")), ("usCharCodeCount", SimTypeShort(signed=False, label="UInt16")), ("pulCharCodeSet", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="TTVALIDATIONTESTSPARAMSEX", pack=False, align=None),
    "TUISPICREATEDIALOGINSTANCEPARAMS": SimStruct(OrderedDict((("dwRequestID", SimTypeInt(signed=False, label="UInt32")), ("hdDlgInst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("htDlgInst", SimTypeInt(signed=False, label="UInt32")), ("lpszUIDLLName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpParams", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="TUISPICREATEDIALOGINSTANCEPARAMS", pack=False, align=None),
    "TUISPIDLLCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["dwObjectID", "dwObjectType", "lpParams", "dwSize"]), offset=0),
    "TUNER_ANALOG_CAPS_S": SimStruct(OrderedDict((("Mode", SimTypeInt(signed=False, label="UInt32")), ("StandardsSupported", SimTypeInt(signed=False, label="UInt32")), ("MinFrequency", SimTypeInt(signed=False, label="UInt32")), ("MaxFrequency", SimTypeInt(signed=False, label="UInt32")), ("TuningGranularity", SimTypeInt(signed=False, label="UInt32")), ("SettlingTime", SimTypeInt(signed=False, label="UInt32")), ("ScanSensingRange", SimTypeInt(signed=False, label="UInt32")), ("FineTuneSensingRange", SimTypeInt(signed=False, label="UInt32")),)), name="TUNER_ANALOG_CAPS_S", pack=False, align=None),
    "TUNNEL": SimStruct(OrderedDict((("Mutex", SimTypeRef("FAST_MUTEX", SimStruct)), ("Cache", SimTypePointer(SimTypeRef("RTL_SPLAY_LINKS", SimStruct), offset=0)), ("TimerQueue", SimTypeRef("LIST_ENTRY", SimStruct)), ("NumEntries", SimTypeShort(signed=False, label="UInt16")),)), name="TUNNEL", pack=False, align=None),
    "TUNNEL_SUB_TYPE": SimTypeInt(signed=False, label="TUNNEL_SUB_TYPE"),
    "TUNNEL_TYPE": SimTypeInt(signed=False, label="TUNNEL_TYPE"),
    "TVAudioMode": SimTypeInt(signed=False, label="TVAudioMode"),
    "TVGETITEMPARTRECTINFO": SimStruct(OrderedDict((("hti", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("prc", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)), ("partID", SimTypeInt(signed=False, label="TVITEMPART")),)), name="TVGETITEMPARTRECTINFO", pack=False, align=None),
    "TVHITTESTINFO": SimStruct(OrderedDict((("pt", SimTypeRef("POINT", SimStruct)), ("flags", SimTypeInt(signed=False, label="TVHITTESTINFO_FLAGS")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TVHITTESTINFO", pack=False, align=None),
    "TVHITTESTINFO_FLAGS": SimTypeInt(signed=False, label="TVHITTESTINFO_FLAGS"),
    "TVINSERTSTRUCTA": SimStruct(OrderedDict((("hParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInsertAfter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous", SimUnion({"itemex": SimTypeRef("TVITEMEXA", SimStruct), "item": SimTypeRef("TVITEMA", SimStruct)}, name="<anon>", label="None")),)), name="TVINSERTSTRUCTA", pack=False, align=None),
    "TVINSERTSTRUCTW": SimStruct(OrderedDict((("hParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hInsertAfter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Anonymous", SimUnion({"itemex": SimTypeRef("TVITEMEXW", SimStruct), "item": SimTypeRef("TVITEMW", SimStruct)}, name="<anon>", label="None")),)), name="TVINSERTSTRUCTW", pack=False, align=None),
    "TVITEMA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TVITEM_MASK")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("state", SimTypeInt(signed=False, label="TREE_VIEW_ITEM_STATE_FLAGS")), ("stateMask", SimTypeInt(signed=False, label="TREE_VIEW_ITEM_STATE_FLAGS")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iSelectedImage", SimTypeInt(signed=True, label="Int32")), ("cChildren", SimTypeInt(signed=False, label="TVITEMEXW_CHILDREN")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TVITEMA", pack=False, align=None),
    "TVITEMEXA": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TVITEM_MASK")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("state", SimTypeInt(signed=False, label="UInt32")), ("stateMask", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iSelectedImage", SimTypeInt(signed=True, label="Int32")), ("cChildren", SimTypeInt(signed=False, label="TVITEMEXW_CHILDREN")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIntegral", SimTypeInt(signed=True, label="Int32")), ("uStateEx", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iExpandedImage", SimTypeInt(signed=True, label="Int32")), ("iReserved", SimTypeInt(signed=True, label="Int32")),)), name="TVITEMEXA", pack=False, align=None),
    "TVITEMEXW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TVITEM_MASK")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("state", SimTypeInt(signed=False, label="UInt32")), ("stateMask", SimTypeInt(signed=False, label="UInt32")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iSelectedImage", SimTypeInt(signed=True, label="Int32")), ("cChildren", SimTypeInt(signed=False, label="TVITEMEXW_CHILDREN")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iIntegral", SimTypeInt(signed=True, label="Int32")), ("uStateEx", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("iExpandedImage", SimTypeInt(signed=True, label="Int32")), ("iReserved", SimTypeInt(signed=True, label="Int32")),)), name="TVITEMEXW", pack=False, align=None),
    "TVITEMEXW_CHILDREN": SimTypeInt(signed=False, label="TVITEMEXW_CHILDREN"),
    "TVITEMPART": SimTypeInt(signed=False, label="TVITEMPART"),
    "TVITEMW": SimStruct(OrderedDict((("mask", SimTypeInt(signed=False, label="TVITEM_MASK")), ("hItem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("state", SimTypeInt(signed=False, label="TREE_VIEW_ITEM_STATE_FLAGS")), ("stateMask", SimTypeInt(signed=False, label="TREE_VIEW_ITEM_STATE_FLAGS")), ("pszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cchTextMax", SimTypeInt(signed=True, label="Int32")), ("iImage", SimTypeInt(signed=True, label="Int32")), ("iSelectedImage", SimTypeInt(signed=True, label="Int32")), ("cChildren", SimTypeInt(signed=False, label="TVITEMEXW_CHILDREN")), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TVITEMW", pack=False, align=None),
    "TVITEM_MASK": SimTypeInt(signed=False, label="TVITEM_MASK"),
    "TVSORTCB": SimStruct(OrderedDict((("hParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpfnCompare", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lParam1", "lParam2", "lParamSort"]), offset=0)), ("lParam", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="TVSORTCB", pack=False, align=None),
    "TWAIN_CAPABILITY": SimStruct(OrderedDict((("lSize", SimTypeInt(signed=True, label="Int32")), ("lMSG", SimTypeInt(signed=True, label="Int32")), ("lCapID", SimTypeInt(signed=True, label="Int32")), ("lConType", SimTypeInt(signed=True, label="Int32")), ("lRC", SimTypeInt(signed=True, label="Int32")), ("lCC", SimTypeInt(signed=True, label="Int32")), ("lDataSize", SimTypeInt(signed=True, label="Int32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TWAIN_CAPABILITY", pack=False, align=None),
    "TXFS_CREATE_MINIVERSION_INFO": SimStruct(OrderedDict((("StructureVersion", SimTypeShort(signed=False, label="UInt16")), ("StructureLength", SimTypeShort(signed=False, label="UInt16")), ("BaseVersion", SimTypeInt(signed=False, label="UInt32")), ("MiniVersion", SimTypeShort(signed=False, label="UInt16")),)), name="TXFS_CREATE_MINIVERSION_INFO", pack=False, align=None),
    "TXFS_GET_METADATA_INFO_OUT": SimStruct(OrderedDict((("TxfFileId", SimTypeRef("_TxfFileId_e__Struct", SimStruct)), ("LockingTransaction", SimTypeBottom(label="Guid")), ("LastLsn", SimTypeLongLong(signed=False, label="UInt64")), ("TransactionState", SimTypeInt(signed=False, label="UInt32")),)), name="TXFS_GET_METADATA_INFO_OUT", pack=False, align=None),
    "TXFS_GET_TRANSACTED_VERSION": SimStruct(OrderedDict((("ThisBaseVersion", SimTypeInt(signed=False, label="UInt32")), ("LatestVersion", SimTypeInt(signed=False, label="UInt32")), ("ThisMiniVersion", SimTypeShort(signed=False, label="UInt16")), ("FirstMiniVersion", SimTypeShort(signed=False, label="UInt16")), ("LatestMiniVersion", SimTypeShort(signed=False, label="UInt16")),)), name="TXFS_GET_TRANSACTED_VERSION", pack=False, align=None),
    "TXFS_LIST_TRANSACTIONS": SimStruct(OrderedDict((("NumberOfTransactions", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSizeRequired", SimTypeLongLong(signed=False, label="UInt64")),)), name="TXFS_LIST_TRANSACTIONS", pack=False, align=None),
    "TXFS_LIST_TRANSACTIONS_ENTRY": SimStruct(OrderedDict((("TransactionId", SimTypeBottom(label="Guid")), ("TransactionState", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeLongLong(signed=True, label="Int64")),)), name="TXFS_LIST_TRANSACTIONS_ENTRY", pack=False, align=None),
    "TXFS_LIST_TRANSACTION_LOCKED_FILES": SimStruct(OrderedDict((("KtmTransaction", SimTypeBottom(label="Guid")), ("NumberOfFiles", SimTypeLongLong(signed=False, label="UInt64")), ("BufferSizeRequired", SimTypeLongLong(signed=False, label="UInt64")), ("Offset", SimTypeLongLong(signed=False, label="UInt64")),)), name="TXFS_LIST_TRANSACTION_LOCKED_FILES", pack=False, align=None),
    "TXFS_LIST_TRANSACTION_LOCKED_FILES_ENTRY": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("NameFlags", SimTypeInt(signed=False, label="UInt32")), ("FileId", SimTypeLongLong(signed=True, label="Int64")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeLongLong(signed=True, label="Int64")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TXFS_LIST_TRANSACTION_LOCKED_FILES_ENTRY", pack=False, align=None),
    "TXFS_MINIVERSION": SimTypeInt(signed=False, label="TXFS_MINIVERSION"),
    "TXFS_MODIFY_RM": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="TXFS_RMF_LAGS")), ("LogContainerCountMax", SimTypeInt(signed=False, label="UInt32")), ("LogContainerCountMin", SimTypeInt(signed=False, label="UInt32")), ("LogContainerCount", SimTypeInt(signed=False, label="UInt32")), ("LogGrowthIncrement", SimTypeInt(signed=False, label="UInt32")), ("LogAutoShrinkPercentage", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeLongLong(signed=False, label="UInt64")), ("LoggingMode", SimTypeShort(signed=False, label="UInt16")),)), name="TXFS_MODIFY_RM", pack=False, align=None),
    "TXFS_QUERY_RM_INFORMATION": SimStruct(OrderedDict((("BytesRequired", SimTypeInt(signed=False, label="UInt32")), ("TailLsn", SimTypeLongLong(signed=False, label="UInt64")), ("CurrentLsn", SimTypeLongLong(signed=False, label="UInt64")), ("ArchiveTailLsn", SimTypeLongLong(signed=False, label="UInt64")), ("LogContainerSize", SimTypeLongLong(signed=False, label="UInt64")), ("HighestVirtualClock", SimTypeLongLong(signed=True, label="Int64")), ("LogContainerCount", SimTypeInt(signed=False, label="UInt32")), ("LogContainerCountMax", SimTypeInt(signed=False, label="UInt32")), ("LogContainerCountMin", SimTypeInt(signed=False, label="UInt32")), ("LogGrowthIncrement", SimTypeInt(signed=False, label="UInt32")), ("LogAutoShrinkPercentage", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="TXFS_RMF_LAGS")), ("LoggingMode", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("RmState", SimTypeInt(signed=False, label="UInt32")), ("LogCapacity", SimTypeLongLong(signed=False, label="UInt64")), ("LogFree", SimTypeLongLong(signed=False, label="UInt64")), ("TopsSize", SimTypeLongLong(signed=False, label="UInt64")), ("TopsUsed", SimTypeLongLong(signed=False, label="UInt64")), ("TransactionCount", SimTypeLongLong(signed=False, label="UInt64")), ("OnePCCount", SimTypeLongLong(signed=False, label="UInt64")), ("TwoPCCount", SimTypeLongLong(signed=False, label="UInt64")), ("NumberLogFileFull", SimTypeLongLong(signed=False, label="UInt64")), ("OldestTransactionAge", SimTypeLongLong(signed=False, label="UInt64")), ("RMName", SimTypeBottom(label="Guid")), ("TmLogPathOffset", SimTypeInt(signed=False, label="UInt32")),)), name="TXFS_QUERY_RM_INFORMATION", pack=False, align=None),
    "TXFS_READ_BACKUP_INFORMATION_OUT": SimStruct(OrderedDict((("Anonymous", SimUnion({"BufferLength": SimTypeInt(signed=False, label="UInt32"), "Buffer": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="TXFS_READ_BACKUP_INFORMATION_OUT", pack=False, align=None),
    "TXFS_RMF_LAGS": SimTypeInt(signed=False, label="TXFS_RMF_LAGS"),
    "TXFS_ROLLFORWARD_REDO_INFORMATION": SimStruct(OrderedDict((("LastVirtualClock", SimTypeLongLong(signed=True, label="Int64")), ("LastRedoLsn", SimTypeLongLong(signed=False, label="UInt64")), ("HighestRecoveryLsn", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="TXFS_ROLLFORWARD_REDO_INFORMATION", pack=False, align=None),
    "TXFS_SAVEPOINT_INFORMATION": SimStruct(OrderedDict((("KtmTransaction", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ActionCode", SimTypeInt(signed=False, label="UInt32")), ("SavepointId", SimTypeInt(signed=False, label="UInt32")),)), name="TXFS_SAVEPOINT_INFORMATION", pack=False, align=None),
    "TXFS_START_RM_INFORMATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("LogContainerSize", SimTypeLongLong(signed=False, label="UInt64")), ("LogContainerCountMin", SimTypeInt(signed=False, label="UInt32")), ("LogContainerCountMax", SimTypeInt(signed=False, label="UInt32")), ("LogGrowthIncrement", SimTypeInt(signed=False, label="UInt32")), ("LogAutoShrinkPercentage", SimTypeInt(signed=False, label="UInt32")), ("TmLogPathOffset", SimTypeInt(signed=False, label="UInt32")), ("TmLogPathLength", SimTypeShort(signed=False, label="UInt16")), ("LoggingMode", SimTypeShort(signed=False, label="UInt16")), ("LogPathLength", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("LogPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="TXFS_START_RM_INFORMATION", pack=False, align=None),
    "TXFS_TRANSACTION_ACTIVE_INFO": SimStruct(OrderedDict((("TransactionsActiveAtSnapshot", SimTypeChar(label="Byte")),)), name="TXFS_TRANSACTION_ACTIVE_INFO", pack=False, align=None),
    "TXFS_WRITE_BACKUP_INFORMATION": SimStruct(OrderedDict((("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="TXFS_WRITE_BACKUP_INFORMATION", pack=False, align=None),
    "TXF_ID": SimStruct(OrderedDict((("Anonymous", SimStruct(OrderedDict((("LowPart", SimTypeLongLong(signed=True, label="Int64")), ("HighPart", SimTypeLongLong(signed=True, label="Int64")),)), name="_Anonymous_e__Struct", pack=False, align=None)),)), name="TXF_ID", pack=False, align=None),
    "TXF_LOG_RECORD_AFFECTED_FILE": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TxfFileId", SimTypeRef("TXF_ID", SimStruct)), ("KtmGuid", SimTypeBottom(label="Guid")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileNameByteOffsetInStructure", SimTypeInt(signed=False, label="UInt32")),)), name="TXF_LOG_RECORD_AFFECTED_FILE", pack=False, align=None),
    "TXF_LOG_RECORD_BASE": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("RecordType", SimTypeInt(signed=False, label="TXF_LOG_RECORD_TYPE")), ("RecordLength", SimTypeInt(signed=False, label="UInt32")),)), name="TXF_LOG_RECORD_BASE", pack=False, align=None),
    "TXF_LOG_RECORD_TRUNCATE": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("RecordType", SimTypeShort(signed=False, label="UInt16")), ("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TxfFileId", SimTypeRef("TXF_ID", SimStruct)), ("KtmGuid", SimTypeBottom(label="Guid")), ("NewFileSize", SimTypeLongLong(signed=True, label="Int64")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileNameByteOffsetInStructure", SimTypeInt(signed=False, label="UInt32")),)), name="TXF_LOG_RECORD_TRUNCATE", pack=False, align=None),
    "TXF_LOG_RECORD_TYPE": SimTypeInt(signed=False, label="TXF_LOG_RECORD_TYPE"),
    "TXF_LOG_RECORD_WRITE": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("RecordType", SimTypeShort(signed=False, label="UInt16")), ("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("TxfFileId", SimTypeRef("TXF_ID", SimStruct)), ("KtmGuid", SimTypeBottom(label="Guid")), ("ByteOffsetInFile", SimTypeLongLong(signed=True, label="Int64")), ("NumBytesWritten", SimTypeInt(signed=False, label="UInt32")), ("ByteOffsetInStructure", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeInt(signed=False, label="UInt32")), ("FileNameByteOffsetInStructure", SimTypeInt(signed=False, label="UInt32")),)), name="TXF_LOG_RECORD_WRITE", pack=False, align=None),
    "TXN_PARAMETER_BLOCK": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("TxFsContext", SimTypeShort(signed=False, label="UInt16")), ("TransactionObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="TXN_PARAMETER_BLOCK", pack=False, align=None),
    "TXTBACKSTYLE": SimTypeInt(signed=False, label="TXTBACKSTYLE"),
    "TXTHITRESULT": SimTypeInt(signed=False, label="TXTHITRESULT"),
    "TXTNATURALSIZE": SimTypeInt(signed=False, label="TXTNATURALSIZE"),
    "TXTVIEW": SimTypeInt(signed=False, label="TXTVIEW"),
    "TX_MISC_CONSTANTS": SimTypeInt(signed=False, label="TX_MISC_CONSTANTS"),
    "TYMED": SimTypeInt(signed=False, label="TYMED"),
    "TYPE1_FONT": SimStruct(OrderedDict((("hPFM", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hPFB", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ulIdentifier", SimTypeInt(signed=False, label="UInt32")),)), name="TYPE1_FONT", pack=False, align=None),
    "TYPEATTR": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("lcid", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("memidConstructor", SimTypeInt(signed=True, label="Int32")), ("memidDestructor", SimTypeInt(signed=True, label="Int32")), ("lpstrSchema", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbSizeInstance", SimTypeInt(signed=False, label="UInt32")), ("typekind", SimTypeInt(signed=False, label="TYPEKIND")), ("cFuncs", SimTypeShort(signed=False, label="UInt16")), ("cVars", SimTypeShort(signed=False, label="UInt16")), ("cImplTypes", SimTypeShort(signed=False, label="UInt16")), ("cbSizeVft", SimTypeShort(signed=False, label="UInt16")), ("cbAlignment", SimTypeShort(signed=False, label="UInt16")), ("wTypeFlags", SimTypeShort(signed=False, label="UInt16")), ("wMajorVerNum", SimTypeShort(signed=False, label="UInt16")), ("wMinorVerNum", SimTypeShort(signed=False, label="UInt16")), ("tdescAlias", SimTypeRef("TYPEDESC", SimStruct)), ("idldescType", SimTypeRef("IDLDESC", SimStruct)),)), name="TYPEATTR", pack=False, align=None),
    "TYPEDESC": SimStruct(OrderedDict((("Anonymous", SimUnion({"lptdesc": SimTypePointer(SimTypeRef("TYPEDESC", SimStruct), offset=0), "lpadesc": SimTypePointer(SimTypeRef("ARRAYDESC", SimStruct), offset=0), "hreftype": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("vt", SimTypeInt(signed=False, label="VARENUM")),)), name="TYPEDESC", pack=False, align=None),
    "TYPEFLAGS": SimTypeInt(signed=False, label="TYPEFLAGS"),
    "TYPEKIND": SimTypeInt(signed=False, label="TYPEKIND"),
    "TYSPEC": SimTypeInt(signed=False, label="TYSPEC"),
    "TabletDeviceKind": SimTypeInt(signed=False, label="TabletDeviceKind"),
    "TabletHardwareCapabilities": SimTypeInt(signed=False, label="TabletHardwareCapabilities"),
    "TabletPropertyMetricUnit": SimTypeInt(signed=False, label="TabletPropertyMetricUnit"),
    "TextDecorationLineStyle": SimTypeInt(signed=False, label="TextDecorationLineStyle"),
    "TextEditChangeType": SimTypeInt(signed=False, label="TextEditChangeType"),
    "TextPatternRangeEndpoint": SimTypeInt(signed=False, label="TextPatternRangeEndpoint"),
    "TextRenderingHint": SimTypeInt(signed=False, label="TextRenderingHint"),
    "TextUnit": SimTypeInt(signed=False, label="TextUnit"),
    "TfActiveSelEnd": SimTypeInt(signed=False, label="TfActiveSelEnd"),
    "TfAnchor": SimTypeInt(signed=False, label="TfAnchor"),
    "TfCandidateResult": SimTypeInt(signed=False, label="TfCandidateResult"),
    "TfGravity": SimTypeInt(signed=False, label="TfGravity"),
    "TfIntegratableCandidateListSelectionStyle": SimTypeInt(signed=False, label="TfIntegratableCandidateListSelectionStyle"),
    "TfLBBalloonStyle": SimTypeInt(signed=False, label="TfLBBalloonStyle"),
    "TfLBIClick": SimTypeInt(signed=False, label="TfLBIClick"),
    "TfLayoutCode": SimTypeInt(signed=False, label="TfLayoutCode"),
    "TfSapiObject": SimTypeInt(signed=False, label="TfSapiObject"),
    "TfShiftDir": SimTypeInt(signed=False, label="TfShiftDir"),
    "ThumbnailStreamCacheOptions": SimTypeInt(signed=False, label="ThumbnailStreamCacheOptions"),
    "TimedLevel": SimStruct(OrderedDict((("frequency", SimTypeArray(SimTypeChar(label="Byte"), 2048)), ("waveform", SimTypeArray(SimTypeChar(label="Byte"), 2048)), ("state", SimTypeInt(signed=True, label="Int32")), ("timeStamp", SimTypeLongLong(signed=True, label="Int64")),)), name="TimedLevel", pack=False, align=None),
    "Tint": SimStruct(OrderedDict((("Base", SimTypeRef("Effect", SimStruct)),)), name="Tint", pack=False, align=None),
    "TintParams": SimStruct(OrderedDict((("hue", SimTypeInt(signed=True, label="Int32")), ("amount", SimTypeInt(signed=True, label="Int32")),)), name="TintParams", pack=False, align=None),
    "ToggleState": SimTypeInt(signed=False, label="ToggleState"),
    "TransactionVote": SimTypeInt(signed=False, label="TransactionVote"),
    "TransmissionMode": SimTypeInt(signed=False, label="TransmissionMode"),
    "TreeScope": SimTypeInt(signed=False, label="TreeScope"),
    "TreeTraversalOptions": SimTypeInt(signed=False, label="TreeTraversalOptions"),
    "TsActiveSelEnd": SimTypeInt(signed=False, label="TsActiveSelEnd"),
    "TsGravity": SimTypeInt(signed=False, label="TsGravity"),
    "TsLayoutCode": SimTypeInt(signed=False, label="TsLayoutCode"),
    "TsRunType": SimTypeInt(signed=False, label="TsRunType"),
    "TsShiftDir": SimTypeInt(signed=False, label="TsShiftDir"),
    "TunerInputType": SimTypeInt(signed=False, label="TunerInputType"),
    "TunerLockType": SimTypeInt(signed=False, label="TunerLockType"),
    "TypeKind": SimTypeInt(signed=False, label="TypeKind"),
    "UAL_DATA_BLOB": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("RoleGuid", SimTypeBottom(label="Guid")), ("TenantId", SimTypeBottom(label="Guid")), ("Address", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("UserName", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="UAL_DATA_BLOB", pack=False, align=None),
    "UASFLAGS": SimTypeInt(signed=False, label="UASFLAGS"),
    "UAcceptResult": SimTypeInt(signed=False, label="UAcceptResult"),
    "UAlphabeticIndexLabelType": SimTypeInt(signed=False, label="UAlphabeticIndexLabelType"),
    "UBiDi": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UBiDiClassCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UCharDirection"), arg_names=["context", "c"]), offset=0),
    "UBiDiDirection": SimTypeInt(signed=False, label="UBiDiDirection"),
    "UBiDiMirroring": SimTypeInt(signed=False, label="UBiDiMirroring"),
    "UBiDiOrder": SimTypeInt(signed=False, label="UBiDiOrder"),
    "UBiDiReorderingMode": SimTypeInt(signed=False, label="UBiDiReorderingMode"),
    "UBiDiReorderingOption": SimTypeInt(signed=False, label="UBiDiReorderingOption"),
    "UBiDiTransform": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UBidiPairedBracketType": SimTypeInt(signed=False, label="UBidiPairedBracketType"),
    "UBlockCode": SimTypeInt(signed=False, label="UBlockCode"),
    "UBreakIterator": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UBreakIteratorType": SimTypeInt(signed=False, label="UBreakIteratorType"),
    "UCPMap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UCPMapRangeOption": SimTypeInt(signed=False, label="UCPMapRangeOption"),
    "UCPMapValueFilter": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "value"]), offset=0),
    "UCPTrie": SimStruct(OrderedDict((("index", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("data", SimUnion({"ptr0": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ptr16": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ptr32": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "ptr8": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("indexLength", SimTypeInt(signed=True, label="Int32")), ("dataLength", SimTypeInt(signed=True, label="Int32")), ("highStart", SimTypeInt(signed=True, label="Int32")), ("shifted12HighStart", SimTypeShort(signed=False, label="UInt16")), ("type", SimTypeChar(label="SByte")), ("valueWidth", SimTypeChar(label="SByte")), ("reserved32", SimTypeInt(signed=False, label="UInt32")), ("reserved16", SimTypeShort(signed=False, label="UInt16")), ("index3NullOffset", SimTypeShort(signed=False, label="UInt16")), ("dataNullOffset", SimTypeInt(signed=True, label="Int32")), ("nullValue", SimTypeInt(signed=False, label="UInt32")),)), name="UCPTrie", pack=False, align=None),
    "UCPTrieData": SimUnion({"ptr0": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ptr16": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ptr32": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "ptr8": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None"),
    "UCPTrieType": SimTypeInt(signed=False, label="UCPTrieType"),
    "UCPTrieValueWidth": SimTypeInt(signed=False, label="UCPTrieValueWidth"),
    "UCalendarAMPMs": SimTypeInt(signed=False, label="UCalendarAMPMs"),
    "UCalendarAttribute": SimTypeInt(signed=False, label="UCalendarAttribute"),
    "UCalendarDateFields": SimTypeInt(signed=False, label="UCalendarDateFields"),
    "UCalendarDaysOfWeek": SimTypeInt(signed=False, label="UCalendarDaysOfWeek"),
    "UCalendarDisplayNameType": SimTypeInt(signed=False, label="UCalendarDisplayNameType"),
    "UCalendarLimitType": SimTypeInt(signed=False, label="UCalendarLimitType"),
    "UCalendarMonths": SimTypeInt(signed=False, label="UCalendarMonths"),
    "UCalendarType": SimTypeInt(signed=False, label="UCalendarType"),
    "UCalendarWallTimeOption": SimTypeInt(signed=False, label="UCalendarWallTimeOption"),
    "UCalendarWeekdayType": SimTypeInt(signed=False, label="UCalendarWeekdayType"),
    "UCaseMap": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UCharCategory": SimTypeInt(signed=False, label="UCharCategory"),
    "UCharDirection": SimTypeInt(signed=False, label="UCharDirection"),
    "UCharEnumTypeRange": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UCharCategory")], SimTypeChar(label="SByte"), arg_names=["context", "start", "limit", "type"]), offset=0),
    "UCharIterator": SimStruct(OrderedDict((("context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("length", SimTypeInt(signed=True, label="Int32")), ("start", SimTypeInt(signed=True, label="Int32")), ("index", SimTypeInt(signed=True, label="Int32")), ("limit", SimTypeInt(signed=True, label="Int32")), ("reservedField", SimTypeInt(signed=True, label="Int32")), ("getIndex", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=False, label="UCharIteratorOrigin")], SimTypeInt(signed=True, label="Int32"), arg_names=["iter", "origin"]), offset=0)), ("move", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UCharIteratorOrigin")], SimTypeInt(signed=True, label="Int32"), arg_names=["iter", "delta", "origin"]), offset=0)), ("hasNext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeChar(label="SByte"), arg_names=["iter"]), offset=0)), ("hasPrevious", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeChar(label="SByte"), arg_names=["iter"]), offset=0)), ("current", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iter"]), offset=0)), ("next", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iter"]), offset=0)), ("previous", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iter"]), offset=0)), ("reservedFn", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["iter", "something"]), offset=0)), ("getState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["iter"]), offset=0)), ("setState", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeBottom(label="Void"), arg_names=["iter", "state", "pErrorCode"]), offset=0)),)), name="UCharIterator", pack=False, align=None),
    "UCharIteratorCurrent": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iter"]), offset=0),
    "UCharIteratorGetIndex": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=False, label="UCharIteratorOrigin")], SimTypeInt(signed=True, label="Int32"), arg_names=["iter", "origin"]), offset=0),
    "UCharIteratorGetState": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["iter"]), offset=0),
    "UCharIteratorHasNext": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeChar(label="SByte"), arg_names=["iter"]), offset=0),
    "UCharIteratorHasPrevious": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeChar(label="SByte"), arg_names=["iter"]), offset=0),
    "UCharIteratorMove": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UCharIteratorOrigin")], SimTypeInt(signed=True, label="Int32"), arg_names=["iter", "delta", "origin"]), offset=0),
    "UCharIteratorNext": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iter"]), offset=0),
    "UCharIteratorOrigin": SimTypeInt(signed=False, label="UCharIteratorOrigin"),
    "UCharIteratorPrevious": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["iter"]), offset=0),
    "UCharIteratorReserved": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["iter", "something"]), offset=0),
    "UCharIteratorSetState": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UCharIterator", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeBottom(label="Void"), arg_names=["iter", "state", "pErrorCode"]), offset=0),
    "UCharNameChoice": SimTypeInt(signed=False, label="UCharNameChoice"),
    "UCharsetDetector": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UCharsetMatch": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UColAttribute": SimTypeInt(signed=False, label="UColAttribute"),
    "UColAttributeValue": SimTypeInt(signed=False, label="UColAttributeValue"),
    "UColBoundMode": SimTypeInt(signed=False, label="UColBoundMode"),
    "UColReorderCode": SimTypeInt(signed=False, label="UColReorderCode"),
    "UColRuleOption": SimTypeInt(signed=False, label="UColRuleOption"),
    "UCollationElements": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UCollationResult": SimTypeInt(signed=False, label="UCollationResult"),
    "UCollator": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UConstrainedFieldPosition": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UConverter": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UConverterCallbackReason": SimTypeInt(signed=False, label="UConverterCallbackReason"),
    "UConverterFromUCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("UConverterFromUnicodeArgs", SimStruct), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UConverterCallbackReason"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "args", "codeUnits", "length", "codePoint", "reason", "pErrorCode"]), offset=0),
    "UConverterFromUnicodeArgs": SimStruct(OrderedDict((("size", SimTypeShort(signed=False, label="UInt16")), ("flush", SimTypeChar(label="SByte")), ("converter", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("source", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("sourceLimit", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("target", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("targetLimit", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("offsets", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="UConverterFromUnicodeArgs", pack=False, align=None),
    "UConverterPlatform": SimTypeInt(signed=False, label="UConverterPlatform"),
    "UConverterSelector": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UConverterToUCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("UConverterToUnicodeArgs", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UConverterCallbackReason"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "args", "codeUnits", "length", "reason", "pErrorCode"]), offset=0),
    "UConverterToUnicodeArgs": SimStruct(OrderedDict((("size", SimTypeShort(signed=False, label="UInt16")), ("flush", SimTypeChar(label="SByte")), ("converter", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("source", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("sourceLimit", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("target", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("targetLimit", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("offsets", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="UConverterToUnicodeArgs", pack=False, align=None),
    "UConverterType": SimTypeInt(signed=False, label="UConverterType"),
    "UConverterUnicodeSet": SimTypeInt(signed=False, label="UConverterUnicodeSet"),
    "UCurrCurrencyType": SimTypeInt(signed=False, label="UCurrCurrencyType"),
    "UCurrNameStyle": SimTypeInt(signed=False, label="UCurrNameStyle"),
    "UCurrencySpacing": SimTypeInt(signed=False, label="UCurrencySpacing"),
    "UCurrencyUsage": SimTypeInt(signed=False, label="UCurrencyUsage"),
    "UDACCEL": SimStruct(OrderedDict((("nSec", SimTypeInt(signed=False, label="UInt32")), ("nInc", SimTypeInt(signed=False, label="UInt32")),)), name="UDACCEL", pack=False, align=None),
    "UDATE": SimStruct(OrderedDict((("st", SimTypeRef("SYSTEMTIME", SimStruct)), ("wDayOfYear", SimTypeShort(signed=False, label="UInt16")),)), name="UDATE", pack=False, align=None),
    "UDCR_TAG": SimStruct(OrderedDict((("bVersion", SimTypeChar(label="Byte")), ("KID", SimTypeArray(SimTypeChar(label="Byte"), 25)), ("ullBaseCounter", SimTypeLongLong(signed=False, label="UInt64")), ("ullBaseCounterRange", SimTypeLongLong(signed=False, label="UInt64")), ("fScrambled", SimTypeInt(signed=True, label="Int32")), ("bStreamMark", SimTypeChar(label="Byte")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="UDCR_TAG", pack=False, align=None),
    "UDP_ENCAP_TYPE": SimTypeInt(signed=False, label="UDP_ENCAP_TYPE"),
    "UDP_TABLE_CLASS": SimTypeInt(signed=False, label="UDP_TABLE_CLASS"),
    "UDateAbsoluteUnit": SimTypeInt(signed=False, label="UDateAbsoluteUnit"),
    "UDateDirection": SimTypeInt(signed=False, label="UDateDirection"),
    "UDateFormatBooleanAttribute": SimTypeInt(signed=False, label="UDateFormatBooleanAttribute"),
    "UDateFormatField": SimTypeInt(signed=False, label="UDateFormatField"),
    "UDateFormatStyle": SimTypeInt(signed=False, label="UDateFormatStyle"),
    "UDateFormatSymbolType": SimTypeInt(signed=False, label="UDateFormatSymbolType"),
    "UDateFormatSymbols": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UDateIntervalFormat": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UDateRelativeDateTimeFormatterStyle": SimTypeInt(signed=False, label="UDateRelativeDateTimeFormatterStyle"),
    "UDateRelativeUnit": SimTypeInt(signed=False, label="UDateRelativeUnit"),
    "UDateTimePGDisplayWidth": SimTypeInt(signed=False, label="UDateTimePGDisplayWidth"),
    "UDateTimePatternConflict": SimTypeInt(signed=False, label="UDateTimePatternConflict"),
    "UDateTimePatternField": SimTypeInt(signed=False, label="UDateTimePatternField"),
    "UDateTimePatternMatchOptions": SimTypeInt(signed=False, label="UDateTimePatternMatchOptions"),
    "UDateTimeScale": SimTypeInt(signed=False, label="UDateTimeScale"),
    "UDecompositionType": SimTypeInt(signed=False, label="UDecompositionType"),
    "UDialectHandling": SimTypeInt(signed=False, label="UDialectHandling"),
    "UDisplayContext": SimTypeInt(signed=False, label="UDisplayContext"),
    "UDisplayContextType": SimTypeInt(signed=False, label="UDisplayContextType"),
    "UEastAsianWidth": SimTypeInt(signed=False, label="UEastAsianWidth"),
    "UEnumCharNamesFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UCharNameChoice"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeChar(label="SByte"), arg_names=["context", "code", "nameChoice", "name", "length"]), offset=0),
    "UEnumeration": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UErrorCode": SimTypeInt(signed=False, label="UErrorCode"),
    "UFF_FILEHEADER": SimStruct(OrderedDict((("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("nFonts", SimTypeInt(signed=False, label="UInt32")), ("nGlyphSets", SimTypeInt(signed=False, label="UInt32")), ("nVarData", SimTypeInt(signed=False, label="UInt32")), ("offFontDir", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)),)), name="UFF_FILEHEADER", pack=False, align=None),
    "UFF_FONTDIRECTORY": SimStruct(OrderedDict((("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("wSize", SimTypeShort(signed=False, label="UInt16")), ("wFontID", SimTypeShort(signed=False, label="UInt16")), ("sGlyphID", SimTypeShort(signed=True, label="Int16")), ("wFlags", SimTypeShort(signed=False, label="UInt16")), ("dwInstallerSig", SimTypeInt(signed=False, label="UInt32")), ("offFontName", SimTypeInt(signed=False, label="UInt32")), ("offCartridgeName", SimTypeInt(signed=False, label="UInt32")), ("offFontData", SimTypeInt(signed=False, label="UInt32")), ("offGlyphData", SimTypeInt(signed=False, label="UInt32")), ("offVarData", SimTypeInt(signed=False, label="UInt32")),)), name="UFF_FONTDIRECTORY", pack=False, align=None),
    "UFieldCategory": SimTypeInt(signed=False, label="UFieldCategory"),
    "UFieldPosition": SimStruct(OrderedDict((("field", SimTypeInt(signed=True, label="Int32")), ("beginIndex", SimTypeInt(signed=True, label="Int32")), ("endIndex", SimTypeInt(signed=True, label="Int32")),)), name="UFieldPosition", pack=False, align=None),
    "UFieldPositionIterator": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UFormattableType": SimTypeInt(signed=False, label="UFormattableType"),
    "UFormattedDateInterval": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UFormattedList": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UFormattedNumber": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UFormattedNumberRange": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UFormattedRelativeDateTime": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UFormattedValue": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UGender": SimTypeInt(signed=False, label="UGender"),
    "UGenderInfo": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UGraphemeClusterBreak": SimTypeInt(signed=False, label="UGraphemeClusterBreak"),
    "UHangulSyllableType": SimTypeInt(signed=False, label="UHangulSyllableType"),
    "UHashtable": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UIA_ANNOTATIONTYPE": SimTypeInt(signed=False, label="UIA_ANNOTATIONTYPE"),
    "UIA_CHANGE_ID": SimTypeInt(signed=False, label="UIA_CHANGE_ID"),
    "UIA_CONTROLTYPE_ID": SimTypeInt(signed=False, label="UIA_CONTROLTYPE_ID"),
    "UIA_EVENT_ID": SimTypeInt(signed=False, label="UIA_EVENT_ID"),
    "UIA_HEADINGLEVEL_ID": SimTypeInt(signed=False, label="UIA_HEADINGLEVEL_ID"),
    "UIA_LANDMARKTYPE_ID": SimTypeInt(signed=False, label="UIA_LANDMARKTYPE_ID"),
    "UIA_METADATA_ID": SimTypeInt(signed=False, label="UIA_METADATA_ID"),
    "UIA_PATTERN_ID": SimTypeInt(signed=False, label="UIA_PATTERN_ID"),
    "UIA_PROPERTY_ID": SimTypeInt(signed=False, label="UIA_PROPERTY_ID"),
    "UIA_STYLE_ID": SimTypeInt(signed=False, label="UIA_STYLE_ID"),
    "UIA_TEXTATTRIBUTE_ID": SimTypeInt(signed=False, label="UIA_TEXTATTRIBUTE_ID"),
    "UIAutomationEventInfo": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("pProgrammaticName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="UIAutomationEventInfo", pack=False, align=None),
    "UIAutomationMethodInfo": SimStruct(OrderedDict((("pProgrammaticName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("doSetFocus", SimTypeInt(signed=True, label="Int32")), ("cInParameters", SimTypeInt(signed=False, label="UInt32")), ("cOutParameters", SimTypeInt(signed=False, label="UInt32")), ("pParameterTypes", SimTypePointer(SimTypeInt(signed=False, label="UIAutomationType"), offset=0)), ("pParameterNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="UIAutomationMethodInfo", pack=False, align=None),
    "UIAutomationParameter": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="UIAutomationType")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="UIAutomationParameter", pack=False, align=None),
    "UIAutomationPatternInfo": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("pProgrammaticName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("providerInterfaceId", SimTypeBottom(label="Guid")), ("clientInterfaceId", SimTypeBottom(label="Guid")), ("cProperties", SimTypeInt(signed=False, label="UInt32")), ("pProperties", SimTypePointer(SimTypeRef("UIAutomationPropertyInfo", SimStruct), offset=0)), ("cMethods", SimTypeInt(signed=False, label="UInt32")), ("pMethods", SimTypePointer(SimTypeRef("UIAutomationMethodInfo", SimStruct), offset=0)), ("cEvents", SimTypeInt(signed=False, label="UInt32")), ("pEvents", SimTypePointer(SimTypeRef("UIAutomationEventInfo", SimStruct), offset=0)), ("pPatternHandler", SimTypeBottom(label="IUIAutomationPatternHandler")),)), name="UIAutomationPatternInfo", pack=False, align=None),
    "UIAutomationPropertyInfo": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("pProgrammaticName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("type", SimTypeInt(signed=False, label="UIAutomationType")),)), name="UIAutomationPropertyInfo", pack=False, align=None),
    "UIAutomationType": SimTypeInt(signed=False, label="UIAutomationType"),
    "UICloseReasonType": SimTypeInt(signed=False, label="UICloseReasonType"),
    "UIDNA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UIDNAInfo": SimStruct(OrderedDict((("size", SimTypeShort(signed=True, label="Int16")), ("isTransitionalDifferent", SimTypeChar(label="SByte")), ("reservedB3", SimTypeChar(label="SByte")), ("errors", SimTypeInt(signed=False, label="UInt32")), ("reservedI2", SimTypeInt(signed=True, label="Int32")), ("reservedI3", SimTypeInt(signed=True, label="Int32")),)), name="UIDNAInfo", pack=False, align=None),
    "UILANGUAGE_ENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "UILANGUAGE_ENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "UI_ANIMATION_DEPENDENCIES": SimTypeInt(signed=False, label="UI_ANIMATION_DEPENDENCIES"),
    "UI_ANIMATION_IDLE_BEHAVIOR": SimTypeInt(signed=False, label="UI_ANIMATION_IDLE_BEHAVIOR"),
    "UI_ANIMATION_KEYFRAME": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UI_ANIMATION_MANAGER_STATUS": SimTypeInt(signed=False, label="UI_ANIMATION_MANAGER_STATUS"),
    "UI_ANIMATION_MODE": SimTypeInt(signed=False, label="UI_ANIMATION_MODE"),
    "UI_ANIMATION_PRIORITY_EFFECT": SimTypeInt(signed=False, label="UI_ANIMATION_PRIORITY_EFFECT"),
    "UI_ANIMATION_REPEAT_MODE": SimTypeInt(signed=False, label="UI_ANIMATION_REPEAT_MODE"),
    "UI_ANIMATION_ROUNDING_MODE": SimTypeInt(signed=False, label="UI_ANIMATION_ROUNDING_MODE"),
    "UI_ANIMATION_SCHEDULING_RESULT": SimTypeInt(signed=False, label="UI_ANIMATION_SCHEDULING_RESULT"),
    "UI_ANIMATION_SLOPE": SimTypeInt(signed=False, label="UI_ANIMATION_SLOPE"),
    "UI_ANIMATION_STORYBOARD_STATUS": SimTypeInt(signed=False, label="UI_ANIMATION_STORYBOARD_STATUS"),
    "UI_ANIMATION_TIMER_CLIENT_STATUS": SimTypeInt(signed=False, label="UI_ANIMATION_TIMER_CLIENT_STATUS"),
    "UI_ANIMATION_UPDATE_RESULT": SimTypeInt(signed=False, label="UI_ANIMATION_UPDATE_RESULT"),
    "UI_COLLECTIONCHANGE": SimTypeInt(signed=False, label="UI_COLLECTIONCHANGE"),
    "UI_COMMANDTYPE": SimTypeInt(signed=False, label="UI_COMMANDTYPE"),
    "UI_CONTEXTAVAILABILITY": SimTypeInt(signed=False, label="UI_CONTEXTAVAILABILITY"),
    "UI_CONTROLDOCK": SimTypeInt(signed=False, label="UI_CONTROLDOCK"),
    "UI_CONVERT_FLAGS": SimTypeInt(signed=False, label="UI_CONVERT_FLAGS"),
    "UI_EVENTLOCATION": SimTypeInt(signed=False, label="UI_EVENTLOCATION"),
    "UI_EVENTPARAMS": SimStruct(OrderedDict((("EventType", SimTypeInt(signed=False, label="UI_EVENTTYPE")), ("Anonymous", SimUnion({"Modes": SimTypeInt(signed=True, label="Int32"), "Params": SimTypeRef("UI_EVENTPARAMS_COMMAND", SimStruct)}, name="<anon>", label="None")),)), name="UI_EVENTPARAMS", pack=False, align=None),
    "UI_EVENTPARAMS_COMMAND": SimStruct(OrderedDict((("CommandID", SimTypeInt(signed=False, label="UInt32")), ("CommandName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ParentCommandID", SimTypeInt(signed=False, label="UInt32")), ("ParentCommandName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SelectionIndex", SimTypeInt(signed=False, label="UInt32")), ("Location", SimTypeInt(signed=False, label="UI_EVENTLOCATION")),)), name="UI_EVENTPARAMS_COMMAND", pack=False, align=None),
    "UI_EVENTTYPE": SimTypeInt(signed=False, label="UI_EVENTTYPE"),
    "UI_EXECUTIONVERB": SimTypeInt(signed=False, label="UI_EXECUTIONVERB"),
    "UI_FONTDELTASIZE": SimTypeInt(signed=False, label="UI_FONTDELTASIZE"),
    "UI_FONTPROPERTIES": SimTypeInt(signed=False, label="UI_FONTPROPERTIES"),
    "UI_FONTUNDERLINE": SimTypeInt(signed=False, label="UI_FONTUNDERLINE"),
    "UI_FONTVERTICALPOSITION": SimTypeInt(signed=False, label="UI_FONTVERTICALPOSITION"),
    "UI_INFO_TYPE": SimTypeInt(signed=False, label="UI_INFO_TYPE"),
    "UI_INVALIDATIONS": SimTypeInt(signed=False, label="UI_INVALIDATIONS"),
    "UI_OWNERSHIP": SimTypeInt(signed=False, label="UI_OWNERSHIP"),
    "UI_SWATCHCOLORMODE": SimTypeInt(signed=False, label="UI_SWATCHCOLORMODE"),
    "UI_SWATCHCOLORTYPE": SimTypeInt(signed=False, label="UI_SWATCHCOLORTYPE"),
    "UI_TYPE": SimTypeInt(signed=False, label="UI_TYPE"),
    "UI_VIEWTYPE": SimTypeInt(signed=False, label="UI_VIEWTYPE"),
    "UI_VIEWVERB": SimTypeInt(signed=False, label="UI_VIEWVERB"),
    "UIndicPositionalCategory": SimTypeInt(signed=False, label="UIndicPositionalCategory"),
    "UIndicSyllabicCategory": SimTypeInt(signed=False, label="UIndicSyllabicCategory"),
    "UJoiningGroup": SimTypeInt(signed=False, label="UJoiningGroup"),
    "UJoiningType": SimTypeInt(signed=False, label="UJoiningType"),
    "ULayoutType": SimTypeInt(signed=False, label="ULayoutType"),
    "ULineBreak": SimTypeInt(signed=False, label="ULineBreak"),
    "ULineBreakTag": SimTypeInt(signed=False, label="ULineBreakTag"),
    "UListFormatter": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UListFormatterField": SimTypeInt(signed=False, label="UListFormatterField"),
    "UListFormatterType": SimTypeInt(signed=False, label="UListFormatterType"),
    "UListFormatterWidth": SimTypeInt(signed=False, label="UListFormatterWidth"),
    "ULocAvailableType": SimTypeInt(signed=False, label="ULocAvailableType"),
    "ULocDataLocaleType": SimTypeInt(signed=False, label="ULocDataLocaleType"),
    "ULocaleData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "ULocaleDataDelimiterType": SimTypeInt(signed=False, label="ULocaleDataDelimiterType"),
    "ULocaleDataExemplarSetType": SimTypeInt(signed=False, label="ULocaleDataExemplarSetType"),
    "ULocaleDisplayNames": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UMS_CREATE_THREAD_ATTRIBUTES": SimStruct(OrderedDict((("UmsVersion", SimTypeInt(signed=False, label="UInt32")), ("UmsContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UmsCompletionList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="UMS_CREATE_THREAD_ATTRIBUTES", pack=False, align=None),
    "UMS_SCHEDULER_STARTUP_INFO": SimStruct(OrderedDict((("UmsVersion", SimTypeInt(signed=False, label="UInt32")), ("CompletionList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SchedulerProc", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="RTL_UMS_SCHEDULER_REASON"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Reason", "ActivationPayload", "SchedulerParam"]), offset=0)), ("SchedulerParam", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="UMS_SCHEDULER_STARTUP_INFO", pack=False, align=None),
    "UMS_SYSTEM_THREAD_INFORMATION": SimStruct(OrderedDict((("UmsVersion", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ThreadUmsFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="UMS_SYSTEM_THREAD_INFORMATION", pack=False, align=None),
    "UMS_THREAD_INFO_CLASS": SimTypeInt(signed=False, label="UMS_THREAD_INFO_CLASS"),
    "UMeasureFormatWidth": SimTypeInt(signed=False, label="UMeasureFormatWidth"),
    "UMeasurementSystem": SimTypeInt(signed=False, label="UMeasurementSystem"),
    "UMemAllocFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["context", "size"]), offset=0),
    "UMemFreeFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "mem"]), offset=0),
    "UMemReallocFn": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["context", "mem", "size"]), offset=0),
    "UMessagePatternApostropheMode": SimTypeInt(signed=False, label="UMessagePatternApostropheMode"),
    "UMessagePatternArgType": SimTypeInt(signed=False, label="UMessagePatternArgType"),
    "UMessagePatternPartType": SimTypeInt(signed=False, label="UMessagePatternPartType"),
    "UMutableCPTrie": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UNCOMPRESSEDAUDIOFORMAT": SimStruct(OrderedDict((("guidFormatType", SimTypeBottom(label="Guid")), ("dwSamplesPerFrame", SimTypeInt(signed=False, label="UInt32")), ("dwBytesPerSampleContainer", SimTypeInt(signed=False, label="UInt32")), ("dwValidBitsPerSample", SimTypeInt(signed=False, label="UInt32")), ("fFramesPerSecond", SimTypeFloat(size=32)), ("dwChannelMask", SimTypeInt(signed=False, label="UInt32")),)), name="UNCOMPRESSEDAUDIOFORMAT", pack=False, align=None),
    "UNC_INFO_LEVEL": SimTypeInt(signed=False, label="UNC_INFO_LEVEL"),
    "UNDETERMINESTRUCT": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("uDefIMESize", SimTypeInt(signed=False, label="UInt32")), ("uDefIMEPos", SimTypeInt(signed=False, label="UInt32")), ("uUndetTextLen", SimTypeInt(signed=False, label="UInt32")), ("uUndetTextPos", SimTypeInt(signed=False, label="UInt32")), ("uUndetAttrPos", SimTypeInt(signed=False, label="UInt32")), ("uCursorPos", SimTypeInt(signed=False, label="UInt32")), ("uDeltaStart", SimTypeInt(signed=False, label="UInt32")), ("uDetermineTextLen", SimTypeInt(signed=False, label="UInt32")), ("uDetermineTextPos", SimTypeInt(signed=False, label="UInt32")), ("uDetermineDelimPos", SimTypeInt(signed=False, label="UInt32")), ("uYomiTextLen", SimTypeInt(signed=False, label="UInt32")), ("uYomiTextPos", SimTypeInt(signed=False, label="UInt32")), ("uYomiDelimPos", SimTypeInt(signed=False, label="UInt32")),)), name="UNDETERMINESTRUCT", pack=False, align=None),
    "UNDOCK_REASON": SimTypeInt(signed=False, label="UNDOCK_REASON"),
    "UNDONAMEID": SimTypeInt(signed=False, label="UNDONAMEID"),
    "UNESCAPE_CHAR_AT": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeShort(signed=False, label="UInt16"), arg_names=["offset", "context"]), offset=0),
    "UNICODERANGE": SimStruct(OrderedDict((("wcFrom", SimTypeChar(label="Char")), ("wcTo", SimTypeChar(label="Char")),)), name="UNICODERANGE", pack=False, align=None),
    "UNICODE_PREFIX_TABLE": SimStruct(OrderedDict((("NodeTypeCode", SimTypeShort(signed=True, label="Int16")), ("NameLength", SimTypeShort(signed=True, label="Int16")), ("NextPrefixTree", SimTypePointer(SimTypeRef("UNICODE_PREFIX_TABLE_ENTRY", SimStruct), offset=0)), ("LastNextEntry", SimTypePointer(SimTypeRef("UNICODE_PREFIX_TABLE_ENTRY", SimStruct), offset=0)),)), name="UNICODE_PREFIX_TABLE", pack=False, align=None),
    "UNICODE_PREFIX_TABLE_ENTRY": SimStruct(OrderedDict((("NodeTypeCode", SimTypeShort(signed=True, label="Int16")), ("NameLength", SimTypeShort(signed=True, label="Int16")), ("NextPrefixTree", SimTypePointer(SimTypeRef("UNICODE_PREFIX_TABLE_ENTRY", SimStruct), offset=0)), ("CaseMatch", SimTypePointer(SimTypeRef("UNICODE_PREFIX_TABLE_ENTRY", SimStruct), offset=0)), ("Links", SimTypeRef("RTL_SPLAY_LINKS", SimStruct)), ("Prefix", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)),)), name="UNICODE_PREFIX_TABLE_ENTRY", pack=False, align=None),
    "UNICODE_STRING": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="UNICODE_STRING", pack=False, align=None),
    "UNIDRVINFO": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("flGenFlags", SimTypeInt(signed=False, label="UInt32")), ("wType", SimTypeShort(signed=False, label="UInt16")), ("fCaps", SimTypeShort(signed=False, label="UInt16")), ("wXRes", SimTypeShort(signed=False, label="UInt16")), ("wYRes", SimTypeShort(signed=False, label="UInt16")), ("sYAdjust", SimTypeShort(signed=True, label="Int16")), ("sYMoved", SimTypeShort(signed=True, label="Int16")), ("wPrivateData", SimTypeShort(signed=False, label="UInt16")), ("sShift", SimTypeShort(signed=True, label="Int16")), ("SelectFont", SimTypeRef("INVOC", SimStruct)), ("UnSelectFont", SimTypeRef("INVOC", SimStruct)), ("wReserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 4)),)), name="UNIDRVINFO", pack=False, align=None),
    "UNIDRV_PRIVATE_DEVMODE": SimStruct(OrderedDict((("wReserved", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 4)), ("wSize", SimTypeShort(signed=False, label="UInt16")),)), name="UNIDRV_PRIVATE_DEVMODE", pack=False, align=None),
    "UNIFM_HDR": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ulDefaultCodepage", SimTypeInt(signed=False, label="UInt32")), ("lGlyphSetDataRCID", SimTypeInt(signed=True, label="Int32")), ("loUnidrvInfo", SimTypeInt(signed=False, label="UInt32")), ("loIFIMetrics", SimTypeInt(signed=False, label="UInt32")), ("loExtTextMetric", SimTypeInt(signed=False, label="UInt32")), ("loWidthTable", SimTypeInt(signed=False, label="UInt32")), ("loKernPair", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="UNIFM_HDR", pack=False, align=None),
    "UNIVERSAL_NAME_INFOA": SimStruct(OrderedDict((("lpUniversalName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="UNIVERSAL_NAME_INFOA", pack=False, align=None),
    "UNIVERSAL_NAME_INFOW": SimStruct(OrderedDict((("lpUniversalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="UNIVERSAL_NAME_INFOW", pack=False, align=None),
    "UNI_CODEPAGEINFO": SimStruct(OrderedDict((("dwCodePage", SimTypeInt(signed=False, label="UInt32")), ("SelectSymbolSet", SimTypeRef("INVOC", SimStruct)), ("UnSelectSymbolSet", SimTypeRef("INVOC", SimStruct)),)), name="UNI_CODEPAGEINFO", pack=False, align=None),
    "UNI_GLYPHSETDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lPredefinedID", SimTypeInt(signed=True, label="Int32")), ("dwGlyphCount", SimTypeInt(signed=False, label="UInt32")), ("dwRunCount", SimTypeInt(signed=False, label="UInt32")), ("loRunOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCodePageCount", SimTypeInt(signed=False, label="UInt32")), ("loCodePageOffset", SimTypeInt(signed=False, label="UInt32")), ("loMapTableOffset", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="UNI_GLYPHSETDATA", pack=False, align=None),
    "UNLOAD_DLL_DEBUG_INFO": SimStruct(OrderedDict((("lpBaseOfDll", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="UNLOAD_DLL_DEBUG_INFO", pack=False, align=None),
    "UNMAP_VIEW_OF_FILE_FLAGS": SimTypeInt(signed=False, label="UNMAP_VIEW_OF_FILE_FLAGS"),
    "UNSIGNED_RATIO": SimStruct(OrderedDict((("uiNumerator", SimTypeInt(signed=False, label="UInt32")), ("uiDenominator", SimTypeInt(signed=False, label="UInt32")),)), name="UNSIGNED_RATIO", pack=False, align=None),
    "UNSUPPORTED_REQUIREMENT": SimStruct(OrderedDict((("ReqId", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="UNSUPPORTED_REQUIREMENT", pack=False, align=None),
    "UNWIND_HISTORY_TABLE": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("LocalHint", SimTypeChar(label="Byte")), ("GlobalHint", SimTypeChar(label="Byte")), ("Search", SimTypeChar(label="Byte")), ("Once", SimTypeChar(label="Byte")), ("LowAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("HighAddress", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Entry", SimTypeArray(SimTypeRef("UNWIND_HISTORY_TABLE_ENTRY", SimStruct), 12)),)), name="UNWIND_HISTORY_TABLE", pack=False, align=None),
    "UNWIND_HISTORY_TABLE_ENTRY": SimStruct(OrderedDict((("ImageBase", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("FunctionEntry", SimTypePointer(SimTypeRef("IMAGE_RUNTIME_FUNCTION_ENTRY", SimStruct), offset=0)),)), name="UNWIND_HISTORY_TABLE_ENTRY", pack=False, align=None),
    "UNormalization2Mode": SimTypeInt(signed=False, label="UNormalization2Mode"),
    "UNormalizationCheckResult": SimTypeInt(signed=False, label="UNormalizationCheckResult"),
    "UNormalizationMode": SimTypeInt(signed=False, label="UNormalizationMode"),
    "UNormalizer2": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UNumberCompactStyle": SimTypeInt(signed=False, label="UNumberCompactStyle"),
    "UNumberDecimalSeparatorDisplay": SimTypeInt(signed=False, label="UNumberDecimalSeparatorDisplay"),
    "UNumberFormatAttribute": SimTypeInt(signed=False, label="UNumberFormatAttribute"),
    "UNumberFormatAttributeValue": SimTypeInt(signed=False, label="UNumberFormatAttributeValue"),
    "UNumberFormatFields": SimTypeInt(signed=False, label="UNumberFormatFields"),
    "UNumberFormatPadPosition": SimTypeInt(signed=False, label="UNumberFormatPadPosition"),
    "UNumberFormatRoundingMode": SimTypeInt(signed=False, label="UNumberFormatRoundingMode"),
    "UNumberFormatStyle": SimTypeInt(signed=False, label="UNumberFormatStyle"),
    "UNumberFormatSymbol": SimTypeInt(signed=False, label="UNumberFormatSymbol"),
    "UNumberFormatTextAttribute": SimTypeInt(signed=False, label="UNumberFormatTextAttribute"),
    "UNumberFormatter": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UNumberGroupingStrategy": SimTypeInt(signed=False, label="UNumberGroupingStrategy"),
    "UNumberRangeCollapse": SimTypeInt(signed=False, label="UNumberRangeCollapse"),
    "UNumberRangeIdentityFallback": SimTypeInt(signed=False, label="UNumberRangeIdentityFallback"),
    "UNumberRangeIdentityResult": SimTypeInt(signed=False, label="UNumberRangeIdentityResult"),
    "UNumberSignDisplay": SimTypeInt(signed=False, label="UNumberSignDisplay"),
    "UNumberUnitWidth": SimTypeInt(signed=False, label="UNumberUnitWidth"),
    "UNumberingSystem": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UNumericType": SimTypeInt(signed=False, label="UNumericType"),
    "UPDATEDRIVERFORPLUGANDPLAYDEVICES_FLAGS": SimTypeInt(signed=False, label="UPDATEDRIVERFORPLUGANDPLAYDEVICES_FLAGS"),
    "UPDATELAYEREDWINDOWINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hdcDst", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pptDst", SimTypePointer(SimTypeRef("POINT", SimStruct), offset=0)), ("psize", SimTypePointer(SimTypeRef("SIZE", SimStruct), offset=0)), ("hdcSrc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pptSrc", SimTypePointer(SimTypeRef("POINT", SimStruct), offset=0)), ("crKey", SimTypeInt(signed=False, label="UInt32")), ("pblend", SimTypePointer(SimTypeRef("BLENDFUNCTION", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UPDATE_LAYERED_WINDOW_FLAGS")), ("prcDirty", SimTypePointer(SimTypeRef("RECT", SimStruct), offset=0)),)), name="UPDATELAYEREDWINDOWINFO", pack=False, align=None),
    "UPDATEMETADATASTATES": SimTypeInt(signed=False, label="UPDATEMETADATASTATES"),
    "UPDATE_LAYERED_WINDOW_FLAGS": SimTypeInt(signed=False, label="UPDATE_LAYERED_WINDOW_FLAGS"),
    "UPDFCACHE_FLAGS": SimTypeInt(signed=False, label="UPDFCACHE_FLAGS"),
    "UPHORZSTATES": SimTypeInt(signed=False, label="UPHORZSTATES"),
    "UPSTATES": SimTypeInt(signed=False, label="UPSTATES"),
    "UParseError": SimStruct(OrderedDict((("line", SimTypeInt(signed=True, label="Int32")), ("offset", SimTypeInt(signed=True, label="Int32")), ("preContext", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)), ("postContext", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 16)),)), name="UParseError", pack=False, align=None),
    "UPluralRules": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UPluralType": SimTypeInt(signed=False, label="UPluralType"),
    "UProperty": SimTypeInt(signed=False, label="UProperty"),
    "UPropertyNameChoice": SimTypeInt(signed=False, label="UPropertyNameChoice"),
    "URB": SimStruct(OrderedDict((("Anonymous", SimUnion({"UrbHeader": SimTypeRef("_URB_HEADER", SimStruct), "UrbSelectInterface": SimTypeRef("_URB_SELECT_INTERFACE", SimStruct), "UrbSelectConfiguration": SimTypeRef("_URB_SELECT_CONFIGURATION", SimStruct), "UrbPipeRequest": SimTypeRef("_URB_PIPE_REQUEST", SimStruct), "UrbFrameLengthControl": SimTypeRef("_URB_FRAME_LENGTH_CONTROL", SimStruct), "UrbGetFrameLength": SimTypeRef("_URB_GET_FRAME_LENGTH", SimStruct), "UrbSetFrameLength": SimTypeRef("_URB_SET_FRAME_LENGTH", SimStruct), "UrbGetCurrentFrameNumber": SimTypeRef("_URB_GET_CURRENT_FRAME_NUMBER", SimStruct), "UrbControlTransfer": SimTypeRef("_URB_CONTROL_TRANSFER", SimStruct), "UrbControlTransferEx": SimTypeRef("_URB_CONTROL_TRANSFER_EX", SimStruct), "UrbBulkOrInterruptTransfer": SimTypeRef("_URB_BULK_OR_INTERRUPT_TRANSFER", SimStruct), "UrbIsochronousTransfer": SimTypeRef("_URB_ISOCH_TRANSFER", SimStruct), "UrbControlDescriptorRequest": SimTypeRef("_URB_CONTROL_DESCRIPTOR_REQUEST", SimStruct), "UrbControlGetStatusRequest": SimTypeRef("_URB_CONTROL_GET_STATUS_REQUEST", SimStruct), "UrbControlFeatureRequest": SimTypeRef("_URB_CONTROL_FEATURE_REQUEST", SimStruct), "UrbControlVendorClassRequest": SimTypeRef("_URB_CONTROL_VENDOR_OR_CLASS_REQUEST", SimStruct), "UrbControlGetInterfaceRequest": SimTypeRef("_URB_CONTROL_GET_INTERFACE_REQUEST", SimStruct), "UrbControlGetConfigurationRequest": SimTypeRef("_URB_CONTROL_GET_CONFIGURATION_REQUEST", SimStruct), "UrbOSFeatureDescriptorRequest": SimTypeRef("_URB_OS_FEATURE_DESCRIPTOR_REQUEST", SimStruct), "UrbOpenStaticStreams": SimTypeRef("_URB_OPEN_STATIC_STREAMS", SimStruct), "UrbGetIsochPipeTransferPathDelays": SimTypeRef("_URB_GET_ISOCH_PIPE_TRANSFER_PATH_DELAYS", SimStruct)}, name="<anon>", label="None")),)), name="URB", pack=False, align=None),
    "URI_CREATE_FLAGS": SimTypeInt(signed=False, label="URI_CREATE_FLAGS"),
    "URLASSOCIATIONDIALOG_IN_FLAGS": SimTypeInt(signed=False, label="URLASSOCIATIONDIALOG_IN_FLAGS"),
    "URLCACHE_ENTRY_INFO": SimStruct(OrderedDict((("pwszSourceUrlName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszLocalFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwCacheEntryType", SimTypeInt(signed=False, label="UInt32")), ("dwUseCount", SimTypeInt(signed=False, label="UInt32")), ("dwHitRate", SimTypeInt(signed=False, label="UInt32")), ("dwSizeLow", SimTypeInt(signed=False, label="UInt32")), ("dwSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("ftLastModifiedTime", SimTypeRef("FILETIME", SimStruct)), ("ftExpireTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastSyncTime", SimTypeRef("FILETIME", SimStruct)), ("pbHeaderInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbHeaderInfoSize", SimTypeInt(signed=False, label="UInt32")), ("pbExtraData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbExtraDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="URLCACHE_ENTRY_INFO", pack=False, align=None),
    "URLINVOKECOMMANDINFOA": SimStruct(OrderedDict((("dwcbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pcszVerb", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="URLINVOKECOMMANDINFOA", pack=False, align=None),
    "URLINVOKECOMMANDINFOW": SimStruct(OrderedDict((("dwcbSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pcszVerb", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="URLINVOKECOMMANDINFOW", pack=False, align=None),
    "URLIS": SimTypeInt(signed=False, label="URLIS"),
    "URLTEMPLATE": SimTypeInt(signed=False, label="URLTEMPLATE"),
    "URLZONE": SimTypeInt(signed=False, label="URLZONE"),
    "URLZONEREG": SimTypeInt(signed=False, label="URLZONEREG"),
    "URL_CACHE_LIMIT_TYPE": SimTypeInt(signed=False, label="URL_CACHE_LIMIT_TYPE"),
    "URL_COMPONENTS": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszScheme", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSchemeLength", SimTypeInt(signed=False, label="UInt32")), ("nScheme", SimTypeInt(signed=False, label="WINHTTP_INTERNET_SCHEME")), ("lpszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHostNameLength", SimTypeInt(signed=False, label="UInt32")), ("nPort", SimTypeShort(signed=False, label="UInt16")), ("lpszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwUserNameLength", SimTypeInt(signed=False, label="UInt32")), ("lpszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPasswordLength", SimTypeInt(signed=False, label="UInt32")), ("lpszUrlPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwUrlPathLength", SimTypeInt(signed=False, label="UInt32")), ("lpszExtraInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwExtraInfoLength", SimTypeInt(signed=False, label="UInt32")),)), name="URL_COMPONENTS", pack=False, align=None),
    "URL_COMPONENTSA": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszScheme", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwSchemeLength", SimTypeInt(signed=False, label="UInt32")), ("nScheme", SimTypeInt(signed=False, label="INTERNET_SCHEME")), ("lpszHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwHostNameLength", SimTypeInt(signed=False, label="UInt32")), ("nPort", SimTypeShort(signed=False, label="UInt16")), ("lpszUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwUserNameLength", SimTypeInt(signed=False, label="UInt32")), ("lpszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwPasswordLength", SimTypeInt(signed=False, label="UInt32")), ("lpszUrlPath", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwUrlPathLength", SimTypeInt(signed=False, label="UInt32")), ("lpszExtraInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwExtraInfoLength", SimTypeInt(signed=False, label="UInt32")),)), name="URL_COMPONENTSA", pack=False, align=None),
    "URL_COMPONENTSW": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("lpszScheme", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwSchemeLength", SimTypeInt(signed=False, label="UInt32")), ("nScheme", SimTypeInt(signed=False, label="INTERNET_SCHEME")), ("lpszHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwHostNameLength", SimTypeInt(signed=False, label="UInt32")), ("nPort", SimTypeShort(signed=False, label="UInt16")), ("lpszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwUserNameLength", SimTypeInt(signed=False, label="UInt32")), ("lpszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwPasswordLength", SimTypeInt(signed=False, label="UInt32")), ("lpszUrlPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwUrlPathLength", SimTypeInt(signed=False, label="UInt32")), ("lpszExtraInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwExtraInfoLength", SimTypeInt(signed=False, label="UInt32")),)), name="URL_COMPONENTSW", pack=False, align=None),
    "URL_ENCODING": SimTypeInt(signed=False, label="URL_ENCODING"),
    "URL_PART": SimTypeInt(signed=False, label="URL_PART"),
    "URL_SCHEME": SimTypeInt(signed=False, label="URL_SCHEME"),
    "URegexFindProgressCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=True, label="Int64")], SimTypeChar(label="SByte"), arg_names=["context", "matchIndex"]), offset=0),
    "URegexMatchCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeChar(label="SByte"), arg_names=["context", "steps"]), offset=0),
    "URegexpFlag": SimTypeInt(signed=False, label="URegexpFlag"),
    "URegion": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "URegionType": SimTypeInt(signed=False, label="URegionType"),
    "URegularExpression": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "URelativeDateTimeFormatter": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "URelativeDateTimeFormatterField": SimTypeInt(signed=False, label="URelativeDateTimeFormatterField"),
    "URelativeDateTimeUnit": SimTypeInt(signed=False, label="URelativeDateTimeUnit"),
    "UReplaceableCallbacks": SimStruct(OrderedDict((("length", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("charAt", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("char32At", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("replace", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("extract", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("copy", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="UReplaceableCallbacks", pack=False, align=None),
    "UResType": SimTypeInt(signed=False, label="UResType"),
    "UResourceBundle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "URestrictionLevel": SimTypeInt(signed=False, label="URestrictionLevel"),
    "USAGE_AND_PAGE": SimStruct(OrderedDict((("Usage", SimTypeShort(signed=False, label="UInt16")), ("UsagePage", SimTypeShort(signed=False, label="UInt16")),)), name="USAGE_AND_PAGE", pack=False, align=None),
    "USAGE_PROPERTIES": SimStruct(OrderedDict((("level", SimTypeShort(signed=False, label="UInt16")), ("page", SimTypeShort(signed=False, label="UInt16")), ("usage", SimTypeShort(signed=False, label="UInt16")), ("logicalMinimum", SimTypeInt(signed=True, label="Int32")), ("logicalMaximum", SimTypeInt(signed=True, label="Int32")), ("unit", SimTypeShort(signed=False, label="UInt16")), ("exponent", SimTypeShort(signed=False, label="UInt16")), ("count", SimTypeChar(label="Byte")), ("physicalMinimum", SimTypeInt(signed=True, label="Int32")), ("physicalMaximum", SimTypeInt(signed=True, label="Int32")),)), name="USAGE_PROPERTIES", pack=False, align=None),
    "USBD_DEVICE_INFORMATION": SimStruct(OrderedDict((("OffsetNext", SimTypeInt(signed=False, label="UInt32")), ("UsbdDeviceHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DeviceDescriptor", SimTypeRef("USB_DEVICE_DESCRIPTOR", SimStruct)),)), name="USBD_DEVICE_INFORMATION", pack=False, align=None),
    "USBD_ENDPOINT_OFFLOAD_INFORMATION": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("EndpointAddress", SimTypeShort(signed=False, label="UInt16")), ("ResourceId", SimTypeInt(signed=False, label="UInt32")), ("Mode", SimTypeInt(signed=False, label="USBD_ENDPOINT_OFFLOAD_MODE")), ("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")), ("TransferSegmentLA", SimTypeLongLong(signed=True, label="Int64")), ("TransferSegmentVA", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferRingSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TransferRingInitialCycleBit", SimTypeInt(signed=False, label="UInt32")), ("MessageNumber", SimTypeInt(signed=False, label="UInt32")), ("EventRingSegmentLA", SimTypeLongLong(signed=True, label="Int64")), ("EventRingSegmentVA", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EventRingSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("EventRingInitialCycleBit", SimTypeInt(signed=False, label="UInt32")),)), name="USBD_ENDPOINT_OFFLOAD_INFORMATION", pack=False, align=None),
    "USBD_ENDPOINT_OFFLOAD_MODE": SimTypeInt(signed=False, label="USBD_ENDPOINT_OFFLOAD_MODE"),
    "USBD_INTERFACE_INFORMATION": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("InterfaceNumber", SimTypeChar(label="Byte")), ("AlternateSetting", SimTypeChar(label="Byte")), ("Class", SimTypeChar(label="Byte")), ("SubClass", SimTypeChar(label="Byte")), ("Protocol", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("InterfaceHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfPipes", SimTypeInt(signed=False, label="UInt32")), ("Pipes", SimTypePointer(SimTypeRef("USBD_PIPE_INFORMATION", SimStruct), offset=0)),)), name="USBD_INTERFACE_INFORMATION", pack=False, align=None),
    "USBD_ISO_PACKET_DESCRIPTOR": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="USBD_ISO_PACKET_DESCRIPTOR", pack=False, align=None),
    "USBD_PIPE_INFORMATION": SimStruct(OrderedDict((("MaximumPacketSize", SimTypeShort(signed=False, label="UInt16")), ("EndpointAddress", SimTypeChar(label="Byte")), ("Interval", SimTypeChar(label="Byte")), ("PipeType", SimTypeInt(signed=False, label="USBD_PIPE_TYPE")), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MaximumTransferSize", SimTypeInt(signed=False, label="UInt32")), ("PipeFlags", SimTypeInt(signed=False, label="UInt32")),)), name="USBD_PIPE_INFORMATION", pack=False, align=None),
    "USBD_PIPE_TYPE": SimTypeInt(signed=False, label="USBD_PIPE_TYPE"),
    "USBD_STREAM_INFORMATION": SimStruct(OrderedDict((("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("StreamID", SimTypeInt(signed=False, label="UInt32")), ("MaximumTransferSize", SimTypeInt(signed=False, label="UInt32")), ("PipeFlags", SimTypeInt(signed=False, label="UInt32")),)), name="USBD_STREAM_INFORMATION", pack=False, align=None),
    "USBD_VERSION_INFORMATION": SimStruct(OrderedDict((("USBDI_Version", SimTypeInt(signed=False, label="UInt32")), ("Supported_USB_Version", SimTypeInt(signed=False, label="UInt32")),)), name="USBD_VERSION_INFORMATION", pack=False, align=None),
    "USBFN_BUS_CONFIGURATION_INFO": SimStruct(OrderedDict((("ConfigurationName", SimTypeArray(SimTypeChar(label="Char"), 40)), ("IsCurrent", SimTypeChar(label="Byte")), ("IsActive", SimTypeChar(label="Byte")),)), name="USBFN_BUS_CONFIGURATION_INFO", pack=False, align=None),
    "USBFN_BUS_SPEED": SimTypeInt(signed=False, label="USBFN_BUS_SPEED"),
    "USBFN_CLASS_INFORMATION_PACKET": SimStruct(OrderedDict((("FullSpeedClassInterface", SimTypeRef("USBFN_CLASS_INTERFACE", SimStruct)), ("HighSpeedClassInterface", SimTypeRef("USBFN_CLASS_INTERFACE", SimStruct)), ("InterfaceName", SimTypeArray(SimTypeChar(label="Char"), 40)), ("InterfaceGuid", SimTypeArray(SimTypeChar(label="Char"), 39)), ("HasInterfaceGuid", SimTypeChar(label="Byte")), ("SuperSpeedClassInterface", SimTypeRef("USBFN_CLASS_INTERFACE", SimStruct)),)), name="USBFN_CLASS_INFORMATION_PACKET", pack=False, align=None),
    "USBFN_CLASS_INFORMATION_PACKET_EX": SimStruct(OrderedDict((("FullSpeedClassInterfaceEx", SimTypeRef("USBFN_CLASS_INTERFACE_EX", SimStruct)), ("HighSpeedClassInterfaceEx", SimTypeRef("USBFN_CLASS_INTERFACE_EX", SimStruct)), ("SuperSpeedClassInterfaceEx", SimTypeRef("USBFN_CLASS_INTERFACE_EX", SimStruct)), ("InterfaceName", SimTypeArray(SimTypeChar(label="Char"), 40)), ("InterfaceGuid", SimTypeArray(SimTypeChar(label="Char"), 39)), ("HasInterfaceGuid", SimTypeChar(label="Byte")),)), name="USBFN_CLASS_INFORMATION_PACKET_EX", pack=False, align=None),
    "USBFN_CLASS_INTERFACE": SimStruct(OrderedDict((("InterfaceNumber", SimTypeChar(label="Byte")), ("PipeCount", SimTypeChar(label="Byte")), ("PipeArr", SimTypeArray(SimTypeRef("USBFN_PIPE_INFORMATION", SimStruct), 16)),)), name="USBFN_CLASS_INTERFACE", pack=False, align=None),
    "USBFN_CLASS_INTERFACE_EX": SimStruct(OrderedDict((("BaseInterfaceNumber", SimTypeChar(label="Byte")), ("InterfaceCount", SimTypeChar(label="Byte")), ("PipeCount", SimTypeChar(label="Byte")), ("PipeArr", SimTypeArray(SimTypeRef("USBFN_PIPE_INFORMATION", SimStruct), 16)),)), name="USBFN_CLASS_INTERFACE_EX", pack=False, align=None),
    "USBFN_DEVICE_STATE": SimTypeInt(signed=False, label="USBFN_DEVICE_STATE"),
    "USBFN_DIRECTION": SimTypeInt(signed=False, label="USBFN_DIRECTION"),
    "USBFN_EVENT": SimTypeInt(signed=False, label="USBFN_EVENT"),
    "USBFN_INTERFACE_INFO": SimStruct(OrderedDict((("InterfaceNumber", SimTypeChar(label="Byte")), ("Speed", SimTypeInt(signed=False, label="USBFN_BUS_SPEED")), ("Size", SimTypeShort(signed=False, label="UInt16")), ("InterfaceDescriptorSet", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USBFN_INTERFACE_INFO", pack=False, align=None),
    "USBFN_NOTIFICATION": SimStruct(OrderedDict((("Event", SimTypeInt(signed=False, label="USBFN_EVENT")), ("u", SimUnion({"BusSpeed": SimTypeInt(signed=False, label="USBFN_BUS_SPEED"), "SetupPacket": SimTypeRef("USB_DEFAULT_PIPE_SETUP_PACKET", SimStruct), "ConfigurationValue": SimTypeShort(signed=False, label="UInt16"), "PortType": SimTypeInt(signed=False, label="USBFN_PORT_TYPE"), "AlternateInterface": SimTypeRef("ALTERNATE_INTERFACE", SimStruct)}, name="<anon>", label="None")),)), name="USBFN_NOTIFICATION", pack=False, align=None),
    "USBFN_PIPE_INFORMATION": SimStruct(OrderedDict((("EpDesc", SimTypeRef("USB_ENDPOINT_DESCRIPTOR", SimStruct)), ("PipeId", SimTypeInt(signed=False, label="UInt32")),)), name="USBFN_PIPE_INFORMATION", pack=False, align=None),
    "USBFN_PORT_TYPE": SimTypeInt(signed=False, label="USBFN_PORT_TYPE"),
    "USBFN_USB_STRING": SimStruct(OrderedDict((("StringIndex", SimTypeChar(label="Byte")), ("UsbString", SimTypeArray(SimTypeChar(label="Char"), 255)),)), name="USBFN_USB_STRING", pack=False, align=None),
    "USBSCAN_GET_DESCRIPTOR": SimStruct(OrderedDict((("DescriptorType", SimTypeChar(label="Byte")), ("Index", SimTypeChar(label="Byte")), ("LanguageId", SimTypeShort(signed=False, label="UInt16")),)), name="USBSCAN_GET_DESCRIPTOR", pack=False, align=None),
    "USBSCAN_PIPE_CONFIGURATION": SimStruct(OrderedDict((("NumberOfPipes", SimTypeInt(signed=False, label="UInt32")), ("PipeInfo", SimTypeArray(SimTypeRef("USBSCAN_PIPE_INFORMATION", SimStruct), 8)),)), name="USBSCAN_PIPE_CONFIGURATION", pack=False, align=None),
    "USBSCAN_PIPE_INFORMATION": SimStruct(OrderedDict((("MaximumPacketSize", SimTypeShort(signed=False, label="UInt16")), ("EndpointAddress", SimTypeChar(label="Byte")), ("Interval", SimTypeChar(label="Byte")), ("PipeType", SimTypeInt(signed=False, label="RAW_PIPE_TYPE")),)), name="USBSCAN_PIPE_INFORMATION", pack=False, align=None),
    "USBSCAN_TIMEOUT": SimStruct(OrderedDict((("TimeoutRead", SimTypeInt(signed=False, label="UInt32")), ("TimeoutWrite", SimTypeInt(signed=False, label="UInt32")), ("TimeoutEvent", SimTypeInt(signed=False, label="UInt32")),)), name="USBSCAN_TIMEOUT", pack=False, align=None),
    "USBUSER_BANDWIDTH_INFO_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("BandwidthInformation", SimTypeRef("USB_BANDWIDTH_INFO", SimStruct)),)), name="USBUSER_BANDWIDTH_INFO_REQUEST", pack=False, align=None),
    "USBUSER_BUS_STATISTICS_0_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("BusStatistics0", SimTypeRef("USB_BUS_STATISTICS_0", SimStruct)),)), name="USBUSER_BUS_STATISTICS_0_REQUEST", pack=False, align=None),
    "USBUSER_CLOSE_RAW_DEVICE": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("USB_CLOSE_RAW_DEVICE_PARAMETERS", SimStruct)),)), name="USBUSER_CLOSE_RAW_DEVICE", pack=False, align=None),
    "USBUSER_CONTROLLER_INFO_0": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Info0", SimTypeRef("USB_CONTROLLER_INFO_0", SimStruct)),)), name="USBUSER_CONTROLLER_INFO_0", pack=False, align=None),
    "USBUSER_CONTROLLER_UNICODE_NAME": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("UnicodeName", SimTypeRef("USB_UNICODE_NAME", SimStruct)),)), name="USBUSER_CONTROLLER_UNICODE_NAME", pack=False, align=None),
    "USBUSER_GET_DRIVER_VERSION": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("USB_DRIVER_VERSION_PARAMETERS", SimStruct)),)), name="USBUSER_GET_DRIVER_VERSION", pack=False, align=None),
    "USBUSER_GET_USB2HW_VERSION": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("USB_USB2HW_VERSION_PARAMETERS", SimStruct)),)), name="USBUSER_GET_USB2HW_VERSION", pack=False, align=None),
    "USBUSER_OPEN_RAW_DEVICE": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("USB_OPEN_RAW_DEVICE_PARAMETERS", SimStruct)),)), name="USBUSER_OPEN_RAW_DEVICE", pack=False, align=None),
    "USBUSER_PASS_THRU_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("PassThru", SimTypeRef("USB_PASS_THRU_PARAMETERS", SimStruct)),)), name="USBUSER_PASS_THRU_REQUEST", pack=False, align=None),
    "USBUSER_POWER_INFO_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("PowerInformation", SimTypeRef("USB_POWER_INFO", SimStruct)),)), name="USBUSER_POWER_INFO_REQUEST", pack=False, align=None),
    "USBUSER_RAW_RESET_ROOT_PORT": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("RAW_RESET_PORT_PARAMETERS", SimStruct)),)), name="USBUSER_RAW_RESET_ROOT_PORT", pack=False, align=None),
    "USBUSER_REFRESH_HCT_REG": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="USBUSER_REFRESH_HCT_REG", pack=False, align=None),
    "USBUSER_REQUEST_HEADER": SimStruct(OrderedDict((("UsbUserRequest", SimTypeInt(signed=False, label="UInt32")), ("UsbUserStatusCode", SimTypeInt(signed=False, label="USB_USER_ERROR_CODE")), ("RequestBufferLength", SimTypeInt(signed=False, label="UInt32")), ("ActualBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="USBUSER_REQUEST_HEADER", pack=False, align=None),
    "USBUSER_ROOTPORT_FEATURE_REQUEST": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("RAW_ROOTPORT_FEATURE", SimStruct)),)), name="USBUSER_ROOTPORT_FEATURE_REQUEST", pack=False, align=None),
    "USBUSER_ROOTPORT_PARAMETERS": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("RAW_ROOTPORT_PARAMETERS", SimStruct)),)), name="USBUSER_ROOTPORT_PARAMETERS", pack=False, align=None),
    "USBUSER_SEND_ONE_PACKET": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("PacketParameters", SimTypeRef("PACKET_PARAMETERS", SimStruct)),)), name="USBUSER_SEND_ONE_PACKET", pack=False, align=None),
    "USBUSER_SEND_RAW_COMMAND": SimStruct(OrderedDict((("Header", SimTypeRef("USBUSER_REQUEST_HEADER", SimStruct)), ("Parameters", SimTypeRef("USB_SEND_RAW_COMMAND_PARAMETERS", SimStruct)),)), name="USBUSER_SEND_RAW_COMMAND", pack=False, align=None),
    "USB_20_PORT_CHANGE": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_20_PORT_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_30_HUB_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bNumberOfPorts", SimTypeChar(label="Byte")), ("wHubCharacteristics", SimTypeShort(signed=False, label="UInt16")), ("bPowerOnToPowerGood", SimTypeChar(label="Byte")), ("bHubControlCurrent", SimTypeChar(label="Byte")), ("bHubHdrDecLat", SimTypeChar(label="Byte")), ("wHubDelay", SimTypeShort(signed=False, label="UInt16")), ("DeviceRemovable", SimTypeShort(signed=False, label="UInt16")),)), name="USB_30_HUB_DESCRIPTOR", pack=False, align=None),
    "USB_30_PORT_CHANGE": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_30_PORT_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_ACQUIRE_INFO": SimStruct(OrderedDict((("NotificationType", SimTypeInt(signed=False, label="USB_NOTIFICATION_TYPE")), ("TotalSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_ACQUIRE_INFO", pack=False, align=None),
    "USB_BANDWIDTH_INFO": SimStruct(OrderedDict((("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("TotalBusBandwidth", SimTypeInt(signed=False, label="UInt32")), ("Total32secBandwidth", SimTypeInt(signed=False, label="UInt32")), ("AllocedBulkAndControl", SimTypeInt(signed=False, label="UInt32")), ("AllocedIso", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt_1ms", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt_2ms", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt_4ms", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt_8ms", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt_16ms", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt_32ms", SimTypeInt(signed=False, label="UInt32")),)), name="USB_BANDWIDTH_INFO", pack=False, align=None),
    "USB_BOS_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("wTotalLength", SimTypeShort(signed=False, label="UInt16")), ("bNumDeviceCaps", SimTypeChar(label="Byte")),)), name="USB_BOS_DESCRIPTOR", pack=False, align=None),
    "USB_BUS_NOTIFICATION": SimStruct(OrderedDict((("NotificationType", SimTypeInt(signed=False, label="USB_NOTIFICATION_TYPE")), ("TotalBandwidth", SimTypeInt(signed=False, label="UInt32")), ("ConsumedBandwidth", SimTypeInt(signed=False, label="UInt32")), ("ControllerNameLength", SimTypeInt(signed=False, label="UInt32")),)), name="USB_BUS_NOTIFICATION", pack=False, align=None),
    "USB_BUS_STATISTICS_0": SimStruct(OrderedDict((("DeviceCount", SimTypeInt(signed=False, label="UInt32")), ("CurrentSystemTime", SimTypeLongLong(signed=True, label="Int64")), ("CurrentUsbFrame", SimTypeInt(signed=False, label="UInt32")), ("BulkBytes", SimTypeInt(signed=False, label="UInt32")), ("IsoBytes", SimTypeInt(signed=False, label="UInt32")), ("InterruptBytes", SimTypeInt(signed=False, label="UInt32")), ("ControlDataBytes", SimTypeInt(signed=False, label="UInt32")), ("PciInterruptCount", SimTypeInt(signed=False, label="UInt32")), ("HardResetCount", SimTypeInt(signed=False, label="UInt32")), ("WorkerSignalCount", SimTypeInt(signed=False, label="UInt32")), ("CommonBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("WorkerIdleTimeMs", SimTypeInt(signed=False, label="UInt32")), ("RootHubEnabled", SimTypeChar(label="Byte")), ("RootHubDevicePowerState", SimTypeChar(label="Byte")), ("Unused", SimTypeChar(label="Byte")), ("NameIndex", SimTypeChar(label="Byte")),)), name="USB_BUS_STATISTICS_0", pack=False, align=None),
    "USB_CHANGE_REGISTRATION_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "USB_CHARGER_PORT": SimTypeInt(signed=False, label="USB_CHARGER_PORT"),
    "USB_CLOSE_RAW_DEVICE_PARAMETERS": SimStruct(OrderedDict((("xxx", SimTypeInt(signed=False, label="UInt32")),)), name="USB_CLOSE_RAW_DEVICE_PARAMETERS", pack=False, align=None),
    "USB_COMMON_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")),)), name="USB_COMMON_DESCRIPTOR", pack=False, align=None),
    "USB_COMPOSITE_DEVICE_INFO": SimStruct(OrderedDict((("DeviceDescriptor", SimTypeRef("USB_DEVICE_DESCRIPTOR", SimStruct)), ("CurrentConfigDescriptor", SimTypeRef("USB_CONFIGURATION_DESCRIPTOR", SimStruct)), ("CurrentConfigurationValue", SimTypeChar(label="Byte")), ("NumberOfFunctions", SimTypeChar(label="Byte")), ("FunctionInfo", SimTypePointer(SimTypeRef("USB_COMPOSITE_FUNCTION_INFO", SimStruct), offset=0)),)), name="USB_COMPOSITE_DEVICE_INFO", pack=False, align=None),
    "USB_COMPOSITE_FUNCTION_INFO": SimStruct(OrderedDict((("FunctionNumber", SimTypeChar(label="Byte")), ("BaseInterfaceNumber", SimTypeChar(label="Byte")), ("NumberOfInterfaces", SimTypeChar(label="Byte")), ("FunctionIsIdle", SimTypeChar(label="Byte")),)), name="USB_COMPOSITE_FUNCTION_INFO", pack=False, align=None),
    "USB_CONFIGURATION_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("wTotalLength", SimTypeShort(signed=False, label="UInt16")), ("bNumInterfaces", SimTypeChar(label="Byte")), ("bConfigurationValue", SimTypeChar(label="Byte")), ("iConfiguration", SimTypeChar(label="Byte")), ("bmAttributes", SimTypeChar(label="Byte")), ("MaxPower", SimTypeChar(label="Byte")),)), name="USB_CONFIGURATION_DESCRIPTOR", pack=False, align=None),
    "USB_CONFIGURATION_POWER_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("SelfPowerConsumedD0", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("bPowerSummaryId", SimTypeChar(label="Byte")), ("bBusPowerSavingD1", SimTypeChar(label="Byte")), ("bSelfPowerSavingD1", SimTypeChar(label="Byte")), ("bBusPowerSavingD2", SimTypeChar(label="Byte")), ("bSelfPowerSavingD2", SimTypeChar(label="Byte")), ("bBusPowerSavingD3", SimTypeChar(label="Byte")), ("bSelfPowerSavingD3", SimTypeChar(label="Byte")), ("TransitionTimeFromD1", SimTypeShort(signed=False, label="UInt16")), ("TransitionTimeFromD2", SimTypeShort(signed=False, label="UInt16")), ("TransitionTimeFromD3", SimTypeShort(signed=False, label="UInt16")),)), name="USB_CONFIGURATION_POWER_DESCRIPTOR", pack=False, align=None),
    "USB_CONNECTION_NOTIFICATION": SimStruct(OrderedDict((("NotificationType", SimTypeInt(signed=False, label="USB_NOTIFICATION_TYPE")), ("ConnectionNumber", SimTypeInt(signed=False, label="UInt32")), ("RequestedBandwidth", SimTypeInt(signed=False, label="UInt32")), ("EnumerationFailReason", SimTypeInt(signed=False, label="UInt32")), ("PowerRequested", SimTypeInt(signed=False, label="UInt32")), ("HubNameLength", SimTypeInt(signed=False, label="UInt32")),)), name="USB_CONNECTION_NOTIFICATION", pack=False, align=None),
    "USB_CONNECTION_STATUS": SimTypeInt(signed=False, label="USB_CONNECTION_STATUS"),
    "USB_CONTROLLER_DEVICE_INFO": SimStruct(OrderedDict((("PciVendorId", SimTypeInt(signed=False, label="UInt32")), ("PciDeviceId", SimTypeInt(signed=False, label="UInt32")), ("PciRevision", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRootPorts", SimTypeInt(signed=False, label="UInt32")), ("HcFeatureFlags", SimTypeInt(signed=False, label="UInt32")),)), name="USB_CONTROLLER_DEVICE_INFO", pack=False, align=None),
    "USB_CONTROLLER_FLAVOR": SimTypeInt(signed=False, label="USB_CONTROLLER_FLAVOR"),
    "USB_CONTROLLER_INFO_0": SimStruct(OrderedDict((("PciVendorId", SimTypeInt(signed=False, label="UInt32")), ("PciDeviceId", SimTypeInt(signed=False, label="UInt32")), ("PciRevision", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRootPorts", SimTypeInt(signed=False, label="UInt32")), ("ControllerFlavor", SimTypeInt(signed=False, label="USB_CONTROLLER_FLAVOR")), ("HcFeatureFlags", SimTypeInt(signed=False, label="UInt32")),)), name="USB_CONTROLLER_INFO_0", pack=False, align=None),
    "USB_CYCLE_PORT_PARAMS": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("StatusReturned", SimTypeInt(signed=False, label="UInt32")),)), name="USB_CYCLE_PORT_PARAMS", pack=False, align=None),
    "USB_DEFAULT_PIPE_SETUP_PACKET": SimStruct(OrderedDict((("bmRequestType", SimUnion({"s": SimTypeRef("_BM", SimStruct), "B": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("bRequest", SimTypeChar(label="Byte")), ("wValue", SimUnion({"Anonymous": SimStruct(OrderedDict((("LowByte", SimTypeChar(label="Byte")), ("HiByte", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("wIndex", SimUnion({"Anonymous": SimStruct(OrderedDict((("LowByte", SimTypeChar(label="Byte")), ("HiByte", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("wLength", SimTypeShort(signed=False, label="UInt16")),)), name="USB_DEFAULT_PIPE_SETUP_PACKET", pack=False, align=None),
    "USB_DESCRIPTOR_REQUEST": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("SetupPacket", SimTypeRef("_SetupPacket_e__Struct", SimStruct)), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USB_DESCRIPTOR_REQUEST", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_BILLBOARD_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("iAddtionalInfoURL", SimTypeChar(label="Byte")), ("bNumberOfAlternateModes", SimTypeChar(label="Byte")), ("bPreferredAlternateMode", SimTypeChar(label="Byte")), ("VconnPower", SimUnion({"AsUshort": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("bmConfigured", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("bReserved", SimTypeInt(signed=False, label="UInt32")), ("AlternateMode", SimTypePointer(SimStruct(OrderedDict((("wSVID", SimTypeShort(signed=False, label="UInt16")), ("bAlternateMode", SimTypeChar(label="Byte")), ("iAlternateModeSetting", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="USB_DEVICE_CAPABILITY_BILLBOARD_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_CONTAINER_ID_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("ContainerID", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="USB_DEVICE_CAPABILITY_CONTAINER_ID_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")),)), name="USB_DEVICE_CAPABILITY_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_FIRMWARE_STATUS_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bcdDescriptorVersion", SimTypeChar(label="Byte")), ("bmAttributes", SimUnion({"AsUlong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="USB_DEVICE_CAPABILITY_FIRMWARE_STATUS_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_PD_CONSUMER_PORT_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("bmCapabilities", SimUnion({"AsUshort": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("wMinVoltage", SimTypeShort(signed=False, label="UInt16")), ("wMaxVoltage", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("dwMaxOperatingPower", SimTypeInt(signed=False, label="UInt32")), ("dwMaxPeakPower", SimTypeInt(signed=False, label="UInt32")), ("dwMaxPeakPowerTime", SimTypeInt(signed=False, label="UInt32")),)), name="USB_DEVICE_CAPABILITY_PD_CONSUMER_PORT_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_PLATFORM_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("PlatformCapabilityUuid", SimTypeBottom(label="Guid")), ("CapabililityData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USB_DEVICE_CAPABILITY_PLATFORM_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_POWER_DELIVERY_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("bmAttributes", SimUnion({"AsUlong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("bmProviderPorts", SimTypeShort(signed=False, label="UInt16")), ("bmConsumerPorts", SimTypeShort(signed=False, label="UInt16")), ("bcdBCVersion", SimTypeShort(signed=False, label="UInt16")), ("bcdPDVersion", SimTypeShort(signed=False, label="UInt16")), ("bcdUSBTypeCVersion", SimTypeShort(signed=False, label="UInt16")),)), name="USB_DEVICE_CAPABILITY_POWER_DELIVERY_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_SUPERSPEEDPLUS_SPEED": SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_DEVICE_CAPABILITY_SUPERSPEEDPLUS_USB_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("bmAttributes", SimUnion({"AsUlong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("wFunctionalitySupport", SimUnion({"AsUshort": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("bmSublinkSpeedAttr", SimTypePointer(SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), offset=0)),)), name="USB_DEVICE_CAPABILITY_SUPERSPEEDPLUS_USB_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_SUPERSPEED_USB_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bmAttributes", SimTypeChar(label="Byte")), ("wSpeedsSupported", SimTypeShort(signed=False, label="UInt16")), ("bFunctionalitySupport", SimTypeChar(label="Byte")), ("bU1DevExitLat", SimTypeChar(label="Byte")), ("wU2DevExitLat", SimTypeShort(signed=False, label="UInt16")),)), name="USB_DEVICE_CAPABILITY_SUPERSPEED_USB_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CAPABILITY_USB20_EXTENSION_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bDevCapabilityType", SimTypeChar(label="Byte")), ("bmAttributes", SimUnion({"AsUlong": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="USB_DEVICE_CAPABILITY_USB20_EXTENSION_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_CHARACTERISTICS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("UsbDeviceCharacteristicsFlags", SimTypeInt(signed=False, label="UInt32")), ("MaximumSendPathDelayInMilliSeconds", SimTypeInt(signed=False, label="UInt32")), ("MaximumCompletionPathDelayInMilliSeconds", SimTypeInt(signed=False, label="UInt32")),)), name="USB_DEVICE_CHARACTERISTICS", pack=False, align=None),
    "USB_DEVICE_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bcdUSB", SimTypeShort(signed=False, label="UInt16")), ("bDeviceClass", SimTypeChar(label="Byte")), ("bDeviceSubClass", SimTypeChar(label="Byte")), ("bDeviceProtocol", SimTypeChar(label="Byte")), ("bMaxPacketSize0", SimTypeChar(label="Byte")), ("idVendor", SimTypeShort(signed=False, label="UInt16")), ("idProduct", SimTypeShort(signed=False, label="UInt16")), ("bcdDevice", SimTypeShort(signed=False, label="UInt16")), ("iManufacturer", SimTypeChar(label="Byte")), ("iProduct", SimTypeChar(label="Byte")), ("iSerialNumber", SimTypeChar(label="Byte")), ("bNumConfigurations", SimTypeChar(label="Byte")),)), name="USB_DEVICE_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_INFO": SimStruct(OrderedDict((("DeviceState", SimTypeRef("USB_DEVICE_STATE", SimStruct)), ("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("DeviceDescriptor", SimTypeRef("USB_DEVICE_DESCRIPTOR", SimStruct)), ("CurrentConfigurationValue", SimTypeChar(label="Byte")), ("Speed", SimTypeInt(signed=False, label="USB_DEVICE_SPEED")), ("DeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("ConnectionStatus", SimTypeInt(signed=False, label="USB_CONNECTION_STATUS")), ("PnpHardwareId", SimTypeArray(SimTypeChar(label="Char"), 128)), ("PnpCompatibleId", SimTypeArray(SimTypeChar(label="Char"), 128)), ("SerialNumberId", SimTypeArray(SimTypeChar(label="Char"), 128)), ("PnpDeviceDescription", SimTypeArray(SimTypeChar(label="Char"), 128)), ("NumberOfOpenPipes", SimTypeInt(signed=False, label="UInt32")), ("PipeList", SimTypePointer(SimTypeRef("USB_PIPE_INFO", SimStruct), offset=0)),)), name="USB_DEVICE_INFO", pack=False, align=None),
    "USB_DEVICE_NODE_INFO": SimStruct(OrderedDict((("Sig", SimTypeInt(signed=False, label="UInt32")), ("LengthInBytes", SimTypeInt(signed=False, label="UInt32")), ("DeviceDescription", SimTypeArray(SimTypeChar(label="Char"), 40)), ("NodeType", SimTypeInt(signed=False, label="USB_WMI_DEVICE_NODE_TYPE")), ("BusAddress", SimTypeRef("USB_TOPOLOGY_ADDRESS", SimStruct)), ("Anonymous", SimUnion({"UsbDeviceInfo": SimTypeRef("USB_DEVICE_INFO", SimStruct), "HubDeviceInfo": SimTypeRef("USB_HUB_DEVICE_INFO", SimStruct), "CompositeDeviceInfo": SimTypeRef("USB_COMPOSITE_DEVICE_INFO", SimStruct), "ControllerDeviceInfo": SimTypeRef("USB_CONTROLLER_DEVICE_INFO", SimStruct), "DeviceInformation": SimTypeArray(SimTypeChar(label="Byte"), 4)}, name="<anon>", label="None")),)), name="USB_DEVICE_NODE_INFO", pack=False, align=None),
    "USB_DEVICE_PERFORMANCE_INFO": SimStruct(OrderedDict((("BulkBytes", SimTypeInt(signed=False, label="UInt32")), ("ControlDataBytes", SimTypeInt(signed=False, label="UInt32")), ("IsoBytes", SimTypeInt(signed=False, label="UInt32")), ("InterruptBytes", SimTypeInt(signed=False, label="UInt32")), ("BulkUrbCount", SimTypeInt(signed=False, label="UInt32")), ("ControlUrbCount", SimTypeInt(signed=False, label="UInt32")), ("IsoUrbCount", SimTypeInt(signed=False, label="UInt32")), ("InterruptUrbCount", SimTypeInt(signed=False, label="UInt32")), ("AllocedInterrupt", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)), ("AllocedIso", SimTypeInt(signed=False, label="UInt32")), ("Total32secBandwidth", SimTypeInt(signed=False, label="UInt32")), ("TotalTtBandwidth", SimTypeInt(signed=False, label="UInt32")), ("DeviceDescription", SimTypeArray(SimTypeChar(label="Char"), 60)), ("DeviceSpeed", SimTypeInt(signed=False, label="USB_DEVICE_SPEED")), ("TotalIsoLatency", SimTypeInt(signed=False, label="UInt32")), ("DroppedIsoPackets", SimTypeInt(signed=False, label="UInt32")), ("TransferErrors", SimTypeInt(signed=False, label="UInt32")), ("PciInterruptCount", SimTypeInt(signed=False, label="UInt32")), ("HcIdleState", SimTypeInt(signed=False, label="UInt32")), ("HcAsyncIdleState", SimTypeInt(signed=False, label="UInt32")), ("HcAsyncCacheFlushCount", SimTypeInt(signed=False, label="UInt32")), ("HcPeriodicIdleState", SimTypeInt(signed=False, label="UInt32")), ("HcPeriodicCacheFlushCount", SimTypeInt(signed=False, label="UInt32")),)), name="USB_DEVICE_PERFORMANCE_INFO", pack=False, align=None),
    "USB_DEVICE_QUALIFIER_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bcdUSB", SimTypeShort(signed=False, label="UInt16")), ("bDeviceClass", SimTypeChar(label="Byte")), ("bDeviceSubClass", SimTypeChar(label="Byte")), ("bDeviceProtocol", SimTypeChar(label="Byte")), ("bMaxPacketSize0", SimTypeChar(label="Byte")), ("bNumConfigurations", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")),)), name="USB_DEVICE_QUALIFIER_DESCRIPTOR", pack=False, align=None),
    "USB_DEVICE_SPEED": SimTypeInt(signed=False, label="USB_DEVICE_SPEED"),
    "USB_DEVICE_STATE": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="USB_DEVICE_STATE", pack=False, align=None),
    "USB_DEVICE_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_DEVICE_TYPE": SimTypeInt(signed=False, label="USB_DEVICE_TYPE"),
    "USB_DRIVER_VERSION_PARAMETERS": SimStruct(OrderedDict((("DriverTrackingCode", SimTypeInt(signed=False, label="UInt32")), ("USBDI_Version", SimTypeInt(signed=False, label="UInt32")), ("USBUSER_Version", SimTypeInt(signed=False, label="UInt32")), ("CheckedPortDriver", SimTypeChar(label="Byte")), ("CheckedMiniportDriver", SimTypeChar(label="Byte")), ("USB_Version", SimTypeShort(signed=False, label="UInt16")),)), name="USB_DRIVER_VERSION_PARAMETERS", pack=False, align=None),
    "USB_ENDPOINT_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bEndpointAddress", SimTypeChar(label="Byte")), ("bmAttributes", SimTypeChar(label="Byte")), ("wMaxPacketSize", SimTypeShort(signed=False, label="UInt16")), ("bInterval", SimTypeChar(label="Byte")),)), name="USB_ENDPOINT_DESCRIPTOR", pack=False, align=None),
    "USB_ENDPOINT_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_FRAME_NUMBER_AND_QPC_FOR_TIME_SYNC_INFORMATION": SimStruct(OrderedDict((("TimeTrackingHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("InputFrameNumber", SimTypeInt(signed=False, label="UInt32")), ("InputMicroFrameNumber", SimTypeInt(signed=False, label="UInt32")), ("QueryPerformanceCounterAtInputFrameOrMicroFrame", SimTypeLongLong(signed=True, label="Int64")), ("QueryPerformanceCounterFrequency", SimTypeLongLong(signed=True, label="Int64")), ("PredictedAccuracyInMicroSeconds", SimTypeInt(signed=False, label="UInt32")), ("CurrentGenerationID", SimTypeInt(signed=False, label="UInt32")), ("CurrentQueryPerformanceCounter", SimTypeLongLong(signed=True, label="Int64")), ("CurrentHardwareFrameNumber", SimTypeInt(signed=False, label="UInt32")), ("CurrentHardwareMicroFrameNumber", SimTypeInt(signed=False, label="UInt32")), ("CurrentUSBFrameNumber", SimTypeInt(signed=False, label="UInt32")),)), name="USB_FRAME_NUMBER_AND_QPC_FOR_TIME_SYNC_INFORMATION", pack=False, align=None),
    "USB_FUNCTION_SUSPEND_OPTIONS": SimUnion({"AsUchar": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_HCD_DRIVERKEY_NAME": SimStruct(OrderedDict((("ActualLength", SimTypeInt(signed=False, label="UInt32")), ("DriverKeyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_HCD_DRIVERKEY_NAME", pack=False, align=None),
    "USB_HIGH_SPEED_MAXPACKET": SimUnion({"us": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "USB_HUB_30_PORT_REMOTE_WAKE_MASK": SimUnion({"AsUchar8": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_HUB_CAPABILITIES": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="USB_HUB_CAPABILITIES", pack=False, align=None),
    "USB_HUB_CAPABILITIES_EX": SimStruct(OrderedDict((("CapabilityFlags", SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="USB_HUB_CAPABILITIES_EX", pack=False, align=None),
    "USB_HUB_CAP_FLAGS": SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_HUB_CHANGE": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_HUB_DESCRIPTOR": SimStruct(OrderedDict((("bDescriptorLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bNumberOfPorts", SimTypeChar(label="Byte")), ("wHubCharacteristics", SimTypeShort(signed=False, label="UInt16")), ("bPowerOnToPowerGood", SimTypeChar(label="Byte")), ("bHubControlCurrent", SimTypeChar(label="Byte")), ("bRemoveAndPowerMask", SimTypeArray(SimTypeChar(label="Byte"), 64)),)), name="USB_HUB_DESCRIPTOR", pack=False, align=None),
    "USB_HUB_DEVICE_INFO": SimStruct(OrderedDict((("HubDescriptor", SimTypeRef("USB_HUB_DESCRIPTOR", SimStruct)), ("HubNumber", SimTypeInt(signed=False, label="UInt32")), ("DeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("HubIsSelfPowered", SimTypeChar(label="Byte")), ("HubIsRootHub", SimTypeChar(label="Byte")), ("HubCapabilities", SimTypeRef("USB_HUB_CAPABILITIES", SimStruct)), ("NumberOfHubPorts", SimTypeInt(signed=False, label="UInt32")), ("PortInfo", SimTypePointer(SimTypeRef("USB_HUB_PORT_INFORMATION", SimStruct), offset=0)),)), name="USB_HUB_DEVICE_INFO", pack=False, align=None),
    "USB_HUB_DEVICE_UXD_SETTINGS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("PnpGuid", SimTypeBottom(label="Guid")), ("OwnerGuid", SimTypeBottom(label="Guid")), ("DeleteOnShutdown", SimTypeInt(signed=False, label="UInt32")), ("DeleteOnReload", SimTypeInt(signed=False, label="UInt32")), ("DeleteOnDisconnect", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)),)), name="USB_HUB_DEVICE_UXD_SETTINGS", pack=False, align=None),
    "USB_HUB_INFORMATION": SimStruct(OrderedDict((("HubDescriptor", SimTypeRef("USB_HUB_DESCRIPTOR", SimStruct)), ("HubIsBusPowered", SimTypeChar(label="Byte")),)), name="USB_HUB_INFORMATION", pack=False, align=None),
    "USB_HUB_INFORMATION_EX": SimStruct(OrderedDict((("HubType", SimTypeInt(signed=False, label="USB_HUB_TYPE")), ("HighestPortNumber", SimTypeShort(signed=False, label="UInt16")), ("u", SimUnion({"UsbHubDescriptor": SimTypeRef("USB_HUB_DESCRIPTOR", SimStruct), "Usb30HubDescriptor": SimTypeRef("USB_30_HUB_DESCRIPTOR", SimStruct)}, name="<anon>", label="None")),)), name="USB_HUB_INFORMATION_EX", pack=False, align=None),
    "USB_HUB_NAME": SimStruct(OrderedDict((("ActualLength", SimTypeInt(signed=False, label="UInt32")), ("HubName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_HUB_NAME", pack=False, align=None),
    "USB_HUB_NODE": SimTypeInt(signed=False, label="USB_HUB_NODE"),
    "USB_HUB_PORT_INFORMATION": SimStruct(OrderedDict((("DeviceState", SimTypeRef("USB_DEVICE_STATE", SimStruct)), ("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("DeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("ConnectionStatus", SimTypeInt(signed=False, label="USB_CONNECTION_STATUS")),)), name="USB_HUB_PORT_INFORMATION", pack=False, align=None),
    "USB_HUB_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_HUB_STATUS_AND_CHANGE": SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("HubStatus", SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("HubChange", SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_HUB_TYPE": SimTypeInt(signed=False, label="USB_HUB_TYPE"),
    "USB_IDLE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "USB_IDLE_CALLBACK_INFO": SimStruct(OrderedDict((("IdleCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("IdleContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="USB_IDLE_CALLBACK_INFO", pack=False, align=None),
    "USB_ID_STRING": SimStruct(OrderedDict((("LanguageId", SimTypeShort(signed=False, label="UInt16")), ("Pad", SimTypeShort(signed=False, label="UInt16")), ("LengthInBytes", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_ID_STRING", pack=False, align=None),
    "USB_INTERFACE_ASSOCIATION_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bFirstInterface", SimTypeChar(label="Byte")), ("bInterfaceCount", SimTypeChar(label="Byte")), ("bFunctionClass", SimTypeChar(label="Byte")), ("bFunctionSubClass", SimTypeChar(label="Byte")), ("bFunctionProtocol", SimTypeChar(label="Byte")), ("iFunction", SimTypeChar(label="Byte")),)), name="USB_INTERFACE_ASSOCIATION_DESCRIPTOR", pack=False, align=None),
    "USB_INTERFACE_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bInterfaceNumber", SimTypeChar(label="Byte")), ("bAlternateSetting", SimTypeChar(label="Byte")), ("bNumEndpoints", SimTypeChar(label="Byte")), ("bInterfaceClass", SimTypeChar(label="Byte")), ("bInterfaceSubClass", SimTypeChar(label="Byte")), ("bInterfaceProtocol", SimTypeChar(label="Byte")), ("iInterface", SimTypeChar(label="Byte")),)), name="USB_INTERFACE_DESCRIPTOR", pack=False, align=None),
    "USB_INTERFACE_POWER_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bmCapabilitiesFlags", SimTypeChar(label="Byte")), ("bBusPowerSavingD1", SimTypeChar(label="Byte")), ("bSelfPowerSavingD1", SimTypeChar(label="Byte")), ("bBusPowerSavingD2", SimTypeChar(label="Byte")), ("bSelfPowerSavingD2", SimTypeChar(label="Byte")), ("bBusPowerSavingD3", SimTypeChar(label="Byte")), ("bSelfPowerSavingD3", SimTypeChar(label="Byte")), ("TransitionTimeFromD1", SimTypeShort(signed=False, label="UInt16")), ("TransitionTimeFromD2", SimTypeShort(signed=False, label="UInt16")), ("TransitionTimeFromD3", SimTypeShort(signed=False, label="UInt16")),)), name="USB_INTERFACE_POWER_DESCRIPTOR", pack=False, align=None),
    "USB_INTERFACE_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_MI_PARENT_INFORMATION": SimStruct(OrderedDict((("NumberOfInterfaces", SimTypeInt(signed=False, label="UInt32")),)), name="USB_MI_PARENT_INFORMATION", pack=False, align=None),
    "USB_NODE_CONNECTION_ATTRIBUTES": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("ConnectionStatus", SimTypeInt(signed=False, label="USB_CONNECTION_STATUS")), ("PortAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="USB_NODE_CONNECTION_ATTRIBUTES", pack=False, align=None),
    "USB_NODE_CONNECTION_DRIVERKEY_NAME": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("ActualLength", SimTypeInt(signed=False, label="UInt32")), ("DriverKeyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_NODE_CONNECTION_DRIVERKEY_NAME", pack=False, align=None),
    "USB_NODE_CONNECTION_INFORMATION": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("DeviceDescriptor", SimTypeRef("USB_DEVICE_DESCRIPTOR", SimStruct)), ("CurrentConfigurationValue", SimTypeChar(label="Byte")), ("LowSpeed", SimTypeChar(label="Byte")), ("DeviceIsHub", SimTypeChar(label="Byte")), ("DeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("NumberOfOpenPipes", SimTypeInt(signed=False, label="UInt32")), ("ConnectionStatus", SimTypeInt(signed=False, label="USB_CONNECTION_STATUS")), ("PipeList", SimTypePointer(SimTypeRef("USB_PIPE_INFO", SimStruct), offset=0)),)), name="USB_NODE_CONNECTION_INFORMATION", pack=False, align=None),
    "USB_NODE_CONNECTION_INFORMATION_EX": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("DeviceDescriptor", SimTypeRef("USB_DEVICE_DESCRIPTOR", SimStruct)), ("CurrentConfigurationValue", SimTypeChar(label="Byte")), ("Speed", SimTypeChar(label="Byte")), ("DeviceIsHub", SimTypeChar(label="Byte")), ("DeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("NumberOfOpenPipes", SimTypeInt(signed=False, label="UInt32")), ("ConnectionStatus", SimTypeInt(signed=False, label="USB_CONNECTION_STATUS")), ("PipeList", SimTypePointer(SimTypeRef("USB_PIPE_INFO", SimStruct), offset=0)),)), name="USB_NODE_CONNECTION_INFORMATION_EX", pack=False, align=None),
    "USB_NODE_CONNECTION_INFORMATION_EX_V2": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("SupportedUsbProtocols", SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Flags", SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="USB_NODE_CONNECTION_INFORMATION_EX_V2", pack=False, align=None),
    "USB_NODE_CONNECTION_INFORMATION_EX_V2_FLAGS": SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_NODE_CONNECTION_NAME": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("ActualLength", SimTypeInt(signed=False, label="UInt32")), ("NodeName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_NODE_CONNECTION_NAME", pack=False, align=None),
    "USB_NODE_INFORMATION": SimStruct(OrderedDict((("NodeType", SimTypeInt(signed=False, label="USB_HUB_NODE")), ("u", SimUnion({"HubInformation": SimTypeRef("USB_HUB_INFORMATION", SimStruct), "MiParentInformation": SimTypeRef("USB_MI_PARENT_INFORMATION", SimStruct)}, name="<anon>", label="None")),)), name="USB_NODE_INFORMATION", pack=False, align=None),
    "USB_NOTIFICATION": SimStruct(OrderedDict((("NotificationType", SimTypeInt(signed=False, label="USB_NOTIFICATION_TYPE")),)), name="USB_NOTIFICATION", pack=False, align=None),
    "USB_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="USB_NOTIFICATION_TYPE"),
    "USB_OPEN_RAW_DEVICE_PARAMETERS": SimStruct(OrderedDict((("PortStatus", SimTypeShort(signed=False, label="UInt16")), ("MaxPacketEp0", SimTypeShort(signed=False, label="UInt16")),)), name="USB_OPEN_RAW_DEVICE_PARAMETERS", pack=False, align=None),
    "USB_PASS_THRU_PARAMETERS": SimStruct(OrderedDict((("FunctionGUID", SimTypeBottom(label="Guid")), ("ParameterLength", SimTypeInt(signed=False, label="UInt32")), ("Parameters", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="USB_PASS_THRU_PARAMETERS", pack=False, align=None),
    "USB_PIPE_INFO": SimStruct(OrderedDict((("EndpointDescriptor", SimTypeRef("USB_ENDPOINT_DESCRIPTOR", SimStruct)), ("ScheduleOffset", SimTypeInt(signed=False, label="UInt32")),)), name="USB_PIPE_INFO", pack=False, align=None),
    "USB_PORT_CHANGE": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Usb20PortChange": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Usb30PortChange": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "USB_PORT_CONNECTOR_PROPERTIES": SimStruct(OrderedDict((("ConnectionIndex", SimTypeInt(signed=False, label="UInt32")), ("ActualLength", SimTypeInt(signed=False, label="UInt32")), ("UsbPortProperties", SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("CompanionIndex", SimTypeShort(signed=False, label="UInt16")), ("CompanionPortNumber", SimTypeShort(signed=False, label="UInt16")), ("CompanionHubSymbolicLinkName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_PORT_CONNECTOR_PROPERTIES", pack=False, align=None),
    "USB_PORT_EXT_STATUS": SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_PORT_EXT_STATUS_AND_CHANGE": SimUnion({"AsUlong64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("PortStatusChange", SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("PortStatus", SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Usb20PortStatus": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Usb30PortStatus": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None")), ("PortChange", SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Usb20PortChange": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Usb30PortChange": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("PortExtStatus", SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_PORT_PROPERTIES": SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_PORT_STATUS": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Usb20PortStatus": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Usb30PortStatus": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "USB_PORT_STATUS_AND_CHANGE": SimUnion({"AsUlong32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("PortStatus", SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Usb20PortStatus": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Usb30PortStatus": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None")), ("PortChange", SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Usb20PortChange": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "Usb30PortChange": SimUnion({"AsUshort16": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_POWER_INFO": SimStruct(OrderedDict((("SystemState", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("HcDevicePowerState", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("HcDeviceWake", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("HcSystemWake", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("RhDevicePowerState", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("RhDeviceWake", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("RhSystemWake", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("LastSystemSleepState", SimTypeInt(signed=False, label="WDMUSB_POWER_STATE")), ("CanWakeup", SimTypeChar(label="Byte")), ("IsPowered", SimTypeChar(label="Byte")),)), name="USB_POWER_INFO", pack=False, align=None),
    "USB_PROTOCOLS": SimUnion({"ul": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "USB_ROOT_HUB_NAME": SimStruct(OrderedDict((("ActualLength", SimTypeInt(signed=False, label="UInt32")), ("RootHubName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_ROOT_HUB_NAME", pack=False, align=None),
    "USB_SEND_RAW_COMMAND_PARAMETERS": SimStruct(OrderedDict((("Usb_bmRequest", SimTypeChar(label="Byte")), ("Usb_bRequest", SimTypeChar(label="Byte")), ("Usb_wVlaue", SimTypeShort(signed=False, label="UInt16")), ("Usb_wIndex", SimTypeShort(signed=False, label="UInt16")), ("Usb_wLength", SimTypeShort(signed=False, label="UInt16")), ("DeviceAddress", SimTypeShort(signed=False, label="UInt16")), ("MaximumPacketSize", SimTypeShort(signed=False, label="UInt16")), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("UsbdStatusCode", SimTypeInt(signed=True, label="Int32")), ("Data", SimTypeArray(SimTypeChar(label="Byte"), 4)),)), name="USB_SEND_RAW_COMMAND_PARAMETERS", pack=False, align=None),
    "USB_START_TRACKING_FOR_TIME_SYNC_INFORMATION": SimStruct(OrderedDict((("TimeTrackingHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("IsStartupDelayTolerable", SimTypeChar(label="Byte")),)), name="USB_START_TRACKING_FOR_TIME_SYNC_INFORMATION", pack=False, align=None),
    "USB_STOP_TRACKING_FOR_TIME_SYNC_INFORMATION": SimStruct(OrderedDict((("TimeTrackingHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="USB_STOP_TRACKING_FOR_TIME_SYNC_INFORMATION", pack=False, align=None),
    "USB_STRING_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_STRING_DESCRIPTOR", pack=False, align=None),
    "USB_SUPERSPEEDPLUS_ISOCH_ENDPOINT_COMPANION_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("dwBytesPerInterval", SimTypeInt(signed=False, label="UInt32")),)), name="USB_SUPERSPEEDPLUS_ISOCH_ENDPOINT_COMPANION_DESCRIPTOR", pack=False, align=None),
    "USB_SUPERSPEED_ENDPOINT_COMPANION_DESCRIPTOR": SimStruct(OrderedDict((("bLength", SimTypeChar(label="Byte")), ("bDescriptorType", SimTypeChar(label="Byte")), ("bMaxBurst", SimTypeChar(label="Byte")), ("bmAttributes", SimUnion({"AsUchar": SimTypeChar(label="Byte"), "Bulk": SimTypeRef("_Bulk_e__Struct", SimStruct), "Isochronous": SimTypeRef("_Isochronous_e__Struct", SimStruct)}, name="<anon>", label="None")), ("wBytesPerInterval", SimTypeShort(signed=False, label="UInt16")),)), name="USB_SUPERSPEED_ENDPOINT_COMPANION_DESCRIPTOR", pack=False, align=None),
    "USB_TOPOLOGY_ADDRESS": SimStruct(OrderedDict((("PciBusNumber", SimTypeInt(signed=False, label="UInt32")), ("PciDeviceNumber", SimTypeInt(signed=False, label="UInt32")), ("PciFunctionNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("RootHubPortNumber", SimTypeShort(signed=False, label="UInt16")), ("HubPortNumber", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 5)), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="USB_TOPOLOGY_ADDRESS", pack=False, align=None),
    "USB_TRANSPORT_CHARACTERISTICS": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("TransportCharacteristicsFlags", SimTypeInt(signed=False, label="UInt32")), ("CurrentRoundtripLatencyInMilliSeconds", SimTypeLongLong(signed=False, label="UInt64")), ("MaxPotentialBandwidth", SimTypeLongLong(signed=False, label="UInt64")),)), name="USB_TRANSPORT_CHARACTERISTICS", pack=False, align=None),
    "USB_TRANSPORT_CHARACTERISTICS_CHANGE_NOTIFICATION": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UsbTransportCharacteristics", SimTypeRef("USB_TRANSPORT_CHARACTERISTICS", SimStruct)),)), name="USB_TRANSPORT_CHARACTERISTICS_CHANGE_NOTIFICATION", pack=False, align=None),
    "USB_TRANSPORT_CHARACTERISTICS_CHANGE_REGISTRATION": SimStruct(OrderedDict((("ChangeNotificationInputFlags", SimTypeInt(signed=False, label="UInt32")), ("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("UsbTransportCharacteristics", SimTypeRef("USB_TRANSPORT_CHARACTERISTICS", SimStruct)),)), name="USB_TRANSPORT_CHARACTERISTICS_CHANGE_REGISTRATION", pack=False, align=None),
    "USB_TRANSPORT_CHARACTERISTICS_CHANGE_UNREGISTRATION": SimStruct(OrderedDict((("Handle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="USB_TRANSPORT_CHARACTERISTICS_CHANGE_UNREGISTRATION", pack=False, align=None),
    "USB_UNICODE_NAME": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("String", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USB_UNICODE_NAME", pack=False, align=None),
    "USB_USB2HW_VERSION_PARAMETERS": SimStruct(OrderedDict((("Usb2HwRevision", SimTypeChar(label="Byte")),)), name="USB_USB2HW_VERSION_PARAMETERS", pack=False, align=None),
    "USB_USER_ERROR_CODE": SimTypeInt(signed=False, label="USB_USER_ERROR_CODE"),
    "USB_WMI_DEVICE_NODE_TYPE": SimTypeInt(signed=False, label="USB_WMI_DEVICE_NODE_TYPE"),
    "USERCLASSTYPE": SimTypeInt(signed=False, label="USERCLASSTYPE"),
    "USERDATA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwItemID", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("pKeyWordName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="USERDATA", pack=False, align=None),
    "USERINFOSTATE": SimTypeInt(signed=False, label="USERINFOSTATE"),
    "USERNAME_TARGET_CREDENTIAL_INFO": SimStruct(OrderedDict((("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USERNAME_TARGET_CREDENTIAL_INFO", pack=False, align=None),
    "USEROBJECTFLAGS": SimStruct(OrderedDict((("fInherit", SimTypeInt(signed=True, label="Int32")), ("fReserved", SimTypeInt(signed=True, label="Int32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="USEROBJECTFLAGS", pack=False, align=None),
    "USERPROPERTIES": SimTypeInt(signed=False, label="USERPROPERTIES"),
    "USERTILEPARTS": SimTypeInt(signed=False, label="USERTILEPARTS"),
    "USER_ACCOUNT_FLAGS": SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS"),
    "USER_ACTIVITY_PRESENCE": SimTypeInt(signed=False, label="USER_ACTIVITY_PRESENCE"),
    "USER_ALL_INFORMATION": SimStruct(OrderedDict((("LastLogon", SimTypeLongLong(signed=True, label="Int64")), ("LastLogoff", SimTypeLongLong(signed=True, label="Int64")), ("PasswordLastSet", SimTypeLongLong(signed=True, label="Int64")), ("AccountExpires", SimTypeLongLong(signed=True, label="Int64")), ("PasswordCanChange", SimTypeLongLong(signed=True, label="Int64")), ("PasswordMustChange", SimTypeLongLong(signed=True, label="Int64")), ("UserName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("FullName", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectory", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("HomeDirectoryDrive", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ScriptPath", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("ProfilePath", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("AdminComment", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("WorkStations", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("UserComment", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("Parameters", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("LmPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("NtPassword", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("PrivateData", SimTypeRef("LSA_UNICODE_STRING", SimStruct)), ("SecurityDescriptor", SimTypeRef("SR_SECURITY_DESCRIPTOR", SimStruct)), ("UserId", SimTypeInt(signed=False, label="UInt32")), ("PrimaryGroupId", SimTypeInt(signed=False, label="UInt32")), ("UserAccountControl", SimTypeInt(signed=False, label="UInt32")), ("WhichFields", SimTypeInt(signed=False, label="UInt32")), ("LogonHours", SimTypeRef("LOGON_HOURS", SimStruct)), ("BadPasswordCount", SimTypeShort(signed=False, label="UInt16")), ("LogonCount", SimTypeShort(signed=False, label="UInt16")), ("CountryCode", SimTypeShort(signed=False, label="UInt16")), ("CodePage", SimTypeShort(signed=False, label="UInt16")), ("LmPasswordPresent", SimTypeChar(label="Byte")), ("NtPasswordPresent", SimTypeChar(label="Byte")), ("PasswordExpired", SimTypeChar(label="Byte")), ("PrivateDataSensitive", SimTypeChar(label="Byte")),)), name="USER_ALL_INFORMATION", pack=False, align=None),
    "USER_CET_ENVIRONMENT": SimTypeInt(signed=False, label="USER_CET_ENVIRONMENT"),
    "USER_INFO_0": SimStruct(OrderedDict((("usri0_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_0", pack=False, align=None),
    "USER_INFO_1": SimStruct(OrderedDict((("usri1_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_password_age", SimTypeInt(signed=False, label="UInt32")), ("usri1_priv", SimTypeInt(signed=False, label="USER_PRIV")), ("usri1_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri1_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri1_script_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1", pack=False, align=None),
    "USER_INFO_10": SimStruct(OrderedDict((("usri10_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri10_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri10_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri10_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_10", pack=False, align=None),
    "USER_INFO_1003": SimStruct(OrderedDict((("usri1003_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1003", pack=False, align=None),
    "USER_INFO_1005": SimStruct(OrderedDict((("usri1005_priv", SimTypeInt(signed=False, label="USER_PRIV")),)), name="USER_INFO_1005", pack=False, align=None),
    "USER_INFO_1006": SimStruct(OrderedDict((("usri1006_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1006", pack=False, align=None),
    "USER_INFO_1007": SimStruct(OrderedDict((("usri1007_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1007", pack=False, align=None),
    "USER_INFO_1008": SimStruct(OrderedDict((("usri1008_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")),)), name="USER_INFO_1008", pack=False, align=None),
    "USER_INFO_1009": SimStruct(OrderedDict((("usri1009_script_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1009", pack=False, align=None),
    "USER_INFO_1010": SimStruct(OrderedDict((("usri1010_auth_flags", SimTypeInt(signed=False, label="AF_OP")),)), name="USER_INFO_1010", pack=False, align=None),
    "USER_INFO_1011": SimStruct(OrderedDict((("usri1011_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1011", pack=False, align=None),
    "USER_INFO_1012": SimStruct(OrderedDict((("usri1012_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1012", pack=False, align=None),
    "USER_INFO_1013": SimStruct(OrderedDict((("usri1013_parms", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1013", pack=False, align=None),
    "USER_INFO_1014": SimStruct(OrderedDict((("usri1014_workstations", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1014", pack=False, align=None),
    "USER_INFO_1017": SimStruct(OrderedDict((("usri1017_acct_expires", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_1017", pack=False, align=None),
    "USER_INFO_1018": SimStruct(OrderedDict((("usri1018_max_storage", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_1018", pack=False, align=None),
    "USER_INFO_1020": SimStruct(OrderedDict((("usri1020_units_per_week", SimTypeInt(signed=False, label="UInt32")), ("usri1020_logon_hours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USER_INFO_1020", pack=False, align=None),
    "USER_INFO_1023": SimStruct(OrderedDict((("usri1023_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1023", pack=False, align=None),
    "USER_INFO_1024": SimStruct(OrderedDict((("usri1024_country_code", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_1024", pack=False, align=None),
    "USER_INFO_1025": SimStruct(OrderedDict((("usri1025_code_page", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_1025", pack=False, align=None),
    "USER_INFO_1051": SimStruct(OrderedDict((("usri1051_primary_group_id", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_1051", pack=False, align=None),
    "USER_INFO_1052": SimStruct(OrderedDict((("usri1052_profile", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1052", pack=False, align=None),
    "USER_INFO_1053": SimStruct(OrderedDict((("usri1053_home_dir_drive", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_INFO_1053", pack=False, align=None),
    "USER_INFO_11": SimStruct(OrderedDict((("usri11_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_priv", SimTypeInt(signed=False, label="USER_PRIV")), ("usri11_auth_flags", SimTypeInt(signed=False, label="AF_OP")), ("usri11_password_age", SimTypeInt(signed=False, label="UInt32")), ("usri11_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_parms", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_last_logon", SimTypeInt(signed=False, label="UInt32")), ("usri11_last_logoff", SimTypeInt(signed=False, label="UInt32")), ("usri11_bad_pw_count", SimTypeInt(signed=False, label="UInt32")), ("usri11_num_logons", SimTypeInt(signed=False, label="UInt32")), ("usri11_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_country_code", SimTypeInt(signed=False, label="UInt32")), ("usri11_workstations", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri11_max_storage", SimTypeInt(signed=False, label="UInt32")), ("usri11_units_per_week", SimTypeInt(signed=False, label="UInt32")), ("usri11_logon_hours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("usri11_code_page", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_11", pack=False, align=None),
    "USER_INFO_2": SimStruct(OrderedDict((("usri2_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_password_age", SimTypeInt(signed=False, label="UInt32")), ("usri2_priv", SimTypeInt(signed=False, label="USER_PRIV")), ("usri2_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri2_script_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_auth_flags", SimTypeInt(signed=False, label="AF_OP")), ("usri2_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_parms", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_workstations", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_last_logon", SimTypeInt(signed=False, label="UInt32")), ("usri2_last_logoff", SimTypeInt(signed=False, label="UInt32")), ("usri2_acct_expires", SimTypeInt(signed=False, label="UInt32")), ("usri2_max_storage", SimTypeInt(signed=False, label="UInt32")), ("usri2_units_per_week", SimTypeInt(signed=False, label="UInt32")), ("usri2_logon_hours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("usri2_bad_pw_count", SimTypeInt(signed=False, label="UInt32")), ("usri2_num_logons", SimTypeInt(signed=False, label="UInt32")), ("usri2_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri2_country_code", SimTypeInt(signed=False, label="UInt32")), ("usri2_code_page", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_2", pack=False, align=None),
    "USER_INFO_20": SimStruct(OrderedDict((("usri20_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri20_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri20_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri20_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri20_user_id", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_20", pack=False, align=None),
    "USER_INFO_21": SimStruct(OrderedDict((("usri21_password", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="USER_INFO_21", pack=False, align=None),
    "USER_INFO_22": SimStruct(OrderedDict((("usri22_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_password", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("usri22_password_age", SimTypeInt(signed=False, label="UInt32")), ("usri22_priv", SimTypeInt(signed=False, label="USER_PRIV")), ("usri22_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri22_script_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_auth_flags", SimTypeInt(signed=False, label="AF_OP")), ("usri22_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_parms", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_workstations", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_last_logon", SimTypeInt(signed=False, label="UInt32")), ("usri22_last_logoff", SimTypeInt(signed=False, label="UInt32")), ("usri22_acct_expires", SimTypeInt(signed=False, label="UInt32")), ("usri22_max_storage", SimTypeInt(signed=False, label="UInt32")), ("usri22_units_per_week", SimTypeInt(signed=False, label="UInt32")), ("usri22_logon_hours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("usri22_bad_pw_count", SimTypeInt(signed=False, label="UInt32")), ("usri22_num_logons", SimTypeInt(signed=False, label="UInt32")), ("usri22_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri22_country_code", SimTypeInt(signed=False, label="UInt32")), ("usri22_code_page", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_22", pack=False, align=None),
    "USER_INFO_23": SimStruct(OrderedDict((("usri23_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri23_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri23_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri23_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri23_user_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="USER_INFO_23", pack=False, align=None),
    "USER_INFO_24": SimStruct(OrderedDict((("usri24_internet_identity", SimTypeInt(signed=True, label="Int32")), ("usri24_flags", SimTypeInt(signed=False, label="UInt32")), ("usri24_internet_provider_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri24_internet_principal_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri24_user_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="USER_INFO_24", pack=False, align=None),
    "USER_INFO_3": SimStruct(OrderedDict((("usri3_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_password_age", SimTypeInt(signed=False, label="UInt32")), ("usri3_priv", SimTypeInt(signed=False, label="USER_PRIV")), ("usri3_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri3_script_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_auth_flags", SimTypeInt(signed=False, label="AF_OP")), ("usri3_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_parms", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_workstations", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_last_logon", SimTypeInt(signed=False, label="UInt32")), ("usri3_last_logoff", SimTypeInt(signed=False, label="UInt32")), ("usri3_acct_expires", SimTypeInt(signed=False, label="UInt32")), ("usri3_max_storage", SimTypeInt(signed=False, label="UInt32")), ("usri3_units_per_week", SimTypeInt(signed=False, label="UInt32")), ("usri3_logon_hours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("usri3_bad_pw_count", SimTypeInt(signed=False, label="UInt32")), ("usri3_num_logons", SimTypeInt(signed=False, label="UInt32")), ("usri3_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_country_code", SimTypeInt(signed=False, label="UInt32")), ("usri3_code_page", SimTypeInt(signed=False, label="UInt32")), ("usri3_user_id", SimTypeInt(signed=False, label="UInt32")), ("usri3_primary_group_id", SimTypeInt(signed=False, label="UInt32")), ("usri3_profile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_home_dir_drive", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri3_password_expired", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_3", pack=False, align=None),
    "USER_INFO_4": SimStruct(OrderedDict((("usri4_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_password_age", SimTypeInt(signed=False, label="UInt32")), ("usri4_priv", SimTypeInt(signed=False, label="USER_PRIV")), ("usri4_home_dir", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_flags", SimTypeInt(signed=False, label="USER_ACCOUNT_FLAGS")), ("usri4_script_path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_auth_flags", SimTypeInt(signed=False, label="AF_OP")), ("usri4_full_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_usr_comment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_parms", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_workstations", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_last_logon", SimTypeInt(signed=False, label="UInt32")), ("usri4_last_logoff", SimTypeInt(signed=False, label="UInt32")), ("usri4_acct_expires", SimTypeInt(signed=False, label="UInt32")), ("usri4_max_storage", SimTypeInt(signed=False, label="UInt32")), ("usri4_units_per_week", SimTypeInt(signed=False, label="UInt32")), ("usri4_logon_hours", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("usri4_bad_pw_count", SimTypeInt(signed=False, label="UInt32")), ("usri4_num_logons", SimTypeInt(signed=False, label="UInt32")), ("usri4_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_country_code", SimTypeInt(signed=False, label="UInt32")), ("usri4_code_page", SimTypeInt(signed=False, label="UInt32")), ("usri4_user_sid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("usri4_primary_group_id", SimTypeInt(signed=False, label="UInt32")), ("usri4_profile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_home_dir_drive", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usri4_password_expired", SimTypeInt(signed=False, label="UInt32")),)), name="USER_INFO_4", pack=False, align=None),
    "USER_INPUT_STRING_TYPE": SimTypeInt(signed=False, label="USER_INPUT_STRING_TYPE"),
    "USER_MARSHAL_CB": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pStubMsg", SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)), ("pReserve", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Signature", SimTypeInt(signed=False, label="UInt32")), ("CBType", SimTypeInt(signed=False, label="USER_MARSHAL_CB_TYPE")), ("pFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pTypeFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USER_MARSHAL_CB", pack=False, align=None),
    "USER_MARSHAL_CB_TYPE": SimTypeInt(signed=False, label="USER_MARSHAL_CB_TYPE"),
    "USER_MARSHAL_FREEING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "USER_MARSHAL_MARSHALLING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeChar(label="Byte"), offset=0), arg_names=["param0", "param1", "param2"]), offset=0),
    "USER_MARSHAL_ROUTINE_QUADRUPLE": SimStruct(OrderedDict((("pfnBufferSize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnMarshall", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeChar(label="Byte"), offset=0), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnUnmarshall", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeChar(label="Byte"), offset=0), arg_names=["param0", "param1", "param2"]), offset=0)), ("pfnFree", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0)),)), name="USER_MARSHAL_ROUTINE_QUADRUPLE", pack=False, align=None),
    "USER_MARSHAL_SIZING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "USER_MARSHAL_UNMARSHALLING_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeChar(label="Byte"), offset=0), arg_names=["param0", "param1", "param2"]), offset=0),
    "USER_MODALS_INFO_0": SimStruct(OrderedDict((("usrmod0_min_passwd_len", SimTypeInt(signed=False, label="UInt32")), ("usrmod0_max_passwd_age", SimTypeInt(signed=False, label="UInt32")), ("usrmod0_min_passwd_age", SimTypeInt(signed=False, label="UInt32")), ("usrmod0_force_logoff", SimTypeInt(signed=False, label="UInt32")), ("usrmod0_password_hist_len", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_0", pack=False, align=None),
    "USER_MODALS_INFO_1": SimStruct(OrderedDict((("usrmod1_role", SimTypeInt(signed=False, label="UInt32")), ("usrmod1_primary", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_MODALS_INFO_1", pack=False, align=None),
    "USER_MODALS_INFO_1001": SimStruct(OrderedDict((("usrmod1001_min_passwd_len", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_1001", pack=False, align=None),
    "USER_MODALS_INFO_1002": SimStruct(OrderedDict((("usrmod1002_max_passwd_age", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_1002", pack=False, align=None),
    "USER_MODALS_INFO_1003": SimStruct(OrderedDict((("usrmod1003_min_passwd_age", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_1003", pack=False, align=None),
    "USER_MODALS_INFO_1004": SimStruct(OrderedDict((("usrmod1004_force_logoff", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_1004", pack=False, align=None),
    "USER_MODALS_INFO_1005": SimStruct(OrderedDict((("usrmod1005_password_hist_len", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_1005", pack=False, align=None),
    "USER_MODALS_INFO_1006": SimStruct(OrderedDict((("usrmod1006_role", SimTypeInt(signed=False, label="USER_MODALS_ROLES")),)), name="USER_MODALS_INFO_1006", pack=False, align=None),
    "USER_MODALS_INFO_1007": SimStruct(OrderedDict((("usrmod1007_primary", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USER_MODALS_INFO_1007", pack=False, align=None),
    "USER_MODALS_INFO_2": SimStruct(OrderedDict((("usrmod2_domain_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usrmod2_domain_id", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="USER_MODALS_INFO_2", pack=False, align=None),
    "USER_MODALS_INFO_3": SimStruct(OrderedDict((("usrmod3_lockout_duration", SimTypeInt(signed=False, label="UInt32")), ("usrmod3_lockout_observation_window", SimTypeInt(signed=False, label="UInt32")), ("usrmod3_lockout_threshold", SimTypeInt(signed=False, label="UInt32")),)), name="USER_MODALS_INFO_3", pack=False, align=None),
    "USER_MODALS_ROLES": SimTypeInt(signed=False, label="USER_MODALS_ROLES"),
    "USER_OBJECT_INFORMATION_INDEX": SimTypeInt(signed=False, label="USER_OBJECT_INFORMATION_INDEX"),
    "USER_OTHER_INFO": SimStruct(OrderedDict((("alrtus_errcode", SimTypeInt(signed=False, label="UInt32")), ("alrtus_numstrings", SimTypeInt(signed=False, label="UInt32")),)), name="USER_OTHER_INFO", pack=False, align=None),
    "USER_POWER_POLICY": SimStruct(OrderedDict((("Revision", SimTypeInt(signed=False, label="UInt32")), ("IdleAc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("IdleDc", SimTypeRef("POWER_ACTION_POLICY", SimStruct)), ("IdleTimeoutAc", SimTypeInt(signed=False, label="UInt32")), ("IdleTimeoutDc", SimTypeInt(signed=False, label="UInt32")), ("IdleSensitivityAc", SimTypeChar(label="Byte")), ("IdleSensitivityDc", SimTypeChar(label="Byte")), ("ThrottlePolicyAc", SimTypeChar(label="Byte")), ("ThrottlePolicyDc", SimTypeChar(label="Byte")), ("MaxSleepAc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("MaxSleepDc", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("VideoTimeoutAc", SimTypeInt(signed=False, label="UInt32")), ("VideoTimeoutDc", SimTypeInt(signed=False, label="UInt32")), ("SpindownTimeoutAc", SimTypeInt(signed=False, label="UInt32")), ("SpindownTimeoutDc", SimTypeInt(signed=False, label="UInt32")), ("OptimizeForPowerAc", SimTypeChar(label="Byte")), ("OptimizeForPowerDc", SimTypeChar(label="Byte")), ("FanThrottleToleranceAc", SimTypeChar(label="Byte")), ("FanThrottleToleranceDc", SimTypeChar(label="Byte")), ("ForcedThrottleAc", SimTypeChar(label="Byte")), ("ForcedThrottleDc", SimTypeChar(label="Byte")),)), name="USER_POWER_POLICY", pack=False, align=None),
    "USER_PRIV": SimTypeInt(signed=False, label="USER_PRIV"),
    "USER_SESSION_KEY": SimStruct(OrderedDict((("data", SimTypeArray(SimTypeRef("CYPHER_BLOCK", SimStruct), 2)),)), name="USER_SESSION_KEY", pack=False, align=None),
    "USE_INFO_0": SimStruct(OrderedDict((("ui0_local", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui0_remote", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USE_INFO_0", pack=False, align=None),
    "USE_INFO_1": SimStruct(OrderedDict((("ui1_local", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui1_remote", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui1_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui1_status", SimTypeInt(signed=False, label="UInt32")), ("ui1_asg_type", SimTypeInt(signed=False, label="USE_INFO_ASG_TYPE")), ("ui1_refcount", SimTypeInt(signed=False, label="UInt32")), ("ui1_usecount", SimTypeInt(signed=False, label="UInt32")),)), name="USE_INFO_1", pack=False, align=None),
    "USE_INFO_2": SimStruct(OrderedDict((("ui2_local", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui2_remote", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui2_password", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui2_status", SimTypeInt(signed=False, label="UInt32")), ("ui2_asg_type", SimTypeInt(signed=False, label="USE_INFO_ASG_TYPE")), ("ui2_refcount", SimTypeInt(signed=False, label="UInt32")), ("ui2_usecount", SimTypeInt(signed=False, label="UInt32")), ("ui2_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ui2_domainname", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USE_INFO_2", pack=False, align=None),
    "USE_INFO_3": SimStruct(OrderedDict((("ui3_ui2", SimTypeRef("USE_INFO_2", SimStruct)), ("ui3_flags", SimTypeInt(signed=False, label="UInt32")),)), name="USE_INFO_3", pack=False, align=None),
    "USE_INFO_4": SimStruct(OrderedDict((("ui4_ui3", SimTypeRef("USE_INFO_3", SimStruct)), ("ui4_auth_identity_length", SimTypeInt(signed=False, label="UInt32")), ("ui4_auth_identity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USE_INFO_4", pack=False, align=None),
    "USE_INFO_5": SimStruct(OrderedDict((("ui4_ui3", SimTypeRef("USE_INFO_3", SimStruct)), ("ui4_auth_identity_length", SimTypeInt(signed=False, label="UInt32")), ("ui4_auth_identity", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ui5_security_descriptor_length", SimTypeInt(signed=False, label="UInt32")), ("ui5_security_descriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ui5_use_options_length", SimTypeInt(signed=False, label="UInt32")), ("ui5_use_options", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="USE_INFO_5", pack=False, align=None),
    "USE_INFO_ASG_TYPE": SimTypeInt(signed=False, label="USE_INFO_ASG_TYPE"),
    "USE_OPTION_DEFERRED_CONNECTION_PARAMETERS": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="USE_OPTION_DEFERRED_CONNECTION_PARAMETERS", pack=False, align=None),
    "USE_OPTION_GENERIC": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="USE_OPTION_GENERIC", pack=False, align=None),
    "USE_OPTION_PROPERTIES": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("pInfo", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="USE_OPTION_PROPERTIES", pack=False, align=None),
    "USE_OPTION_TRANSPORT_PARAMETERS": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="USE_OPTION_TRANSPORT_PARAMETERS", pack=False, align=None),
    "USN_DELETE_FLAGS": SimTypeInt(signed=False, label="USN_DELETE_FLAGS"),
    "USN_JOURNAL_DATA": SimStruct(OrderedDict((("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("FirstUsn", SimTypeLongLong(signed=True, label="Int64")), ("NextUsn", SimTypeLongLong(signed=True, label="Int64")), ("LowestValidUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaxUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationDelta", SimTypeLongLong(signed=False, label="UInt64")), ("MinSupportedMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxSupportedMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="USN_JOURNAL_DATA", pack=False, align=None),
    "USN_JOURNAL_DATA_V0": SimStruct(OrderedDict((("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("FirstUsn", SimTypeLongLong(signed=True, label="Int64")), ("NextUsn", SimTypeLongLong(signed=True, label="Int64")), ("LowestValidUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaxUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationDelta", SimTypeLongLong(signed=False, label="UInt64")),)), name="USN_JOURNAL_DATA_V0", pack=False, align=None),
    "USN_JOURNAL_DATA_V1": SimStruct(OrderedDict((("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("FirstUsn", SimTypeLongLong(signed=True, label="Int64")), ("NextUsn", SimTypeLongLong(signed=True, label="Int64")), ("LowestValidUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaxUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationDelta", SimTypeLongLong(signed=False, label="UInt64")), ("MinSupportedMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxSupportedMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="USN_JOURNAL_DATA_V1", pack=False, align=None),
    "USN_JOURNAL_DATA_V2": SimStruct(OrderedDict((("UsnJournalID", SimTypeLongLong(signed=False, label="UInt64")), ("FirstUsn", SimTypeLongLong(signed=True, label="Int64")), ("NextUsn", SimTypeLongLong(signed=True, label="Int64")), ("LowestValidUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaxUsn", SimTypeLongLong(signed=True, label="Int64")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationDelta", SimTypeLongLong(signed=False, label="UInt64")), ("MinSupportedMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MaxSupportedMajorVersion", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("RangeTrackChunkSize", SimTypeLongLong(signed=False, label="UInt64")), ("RangeTrackFileSizeThreshold", SimTypeLongLong(signed=True, label="Int64")),)), name="USN_JOURNAL_DATA_V2", pack=False, align=None),
    "USN_RANGE_TRACK_OUTPUT": SimStruct(OrderedDict((("Usn", SimTypeLongLong(signed=True, label="Int64")),)), name="USN_RANGE_TRACK_OUTPUT", pack=False, align=None),
    "USN_RECORD": SimStruct(OrderedDict((("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("FileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("ParentFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("Usn", SimTypeLongLong(signed=True, label="Int64")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("Reason", SimTypeInt(signed=False, label="UInt32")), ("SourceInfo", SimTypeInt(signed=False, label="UInt32")), ("SecurityId", SimTypeInt(signed=False, label="UInt32")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeShort(signed=False, label="UInt16")), ("FileNameOffset", SimTypeShort(signed=False, label="UInt16")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USN_RECORD", pack=False, align=None),
    "USN_RECORD_COMMON_HEADER": SimStruct(OrderedDict((("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="USN_RECORD_COMMON_HEADER", pack=False, align=None),
    "USN_RECORD_EXTENT": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="USN_RECORD_EXTENT", pack=False, align=None),
    "USN_RECORD_UNION": SimUnion({"Header": SimTypeRef("USN_RECORD_COMMON_HEADER", SimStruct), "V2": SimTypeRef("USN_RECORD_V2", SimStruct), "V3": SimTypeRef("USN_RECORD_V3", SimStruct), "V4": SimTypeRef("USN_RECORD_V4", SimStruct)}, name="<anon>", label="None"),
    "USN_RECORD_V2": SimStruct(OrderedDict((("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("FileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("ParentFileReferenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("Usn", SimTypeLongLong(signed=True, label="Int64")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("Reason", SimTypeInt(signed=False, label="UInt32")), ("SourceInfo", SimTypeInt(signed=False, label="UInt32")), ("SecurityId", SimTypeInt(signed=False, label="UInt32")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeShort(signed=False, label="UInt16")), ("FileNameOffset", SimTypeShort(signed=False, label="UInt16")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USN_RECORD_V2", pack=False, align=None),
    "USN_RECORD_V3": SimStruct(OrderedDict((("RecordLength", SimTypeInt(signed=False, label="UInt32")), ("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")), ("FileReferenceNumber", SimTypeRef("FILE_ID_128", SimStruct)), ("ParentFileReferenceNumber", SimTypeRef("FILE_ID_128", SimStruct)), ("Usn", SimTypeLongLong(signed=True, label="Int64")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("Reason", SimTypeInt(signed=False, label="UInt32")), ("SourceInfo", SimTypeInt(signed=False, label="UInt32")), ("SecurityId", SimTypeInt(signed=False, label="UInt32")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")), ("FileNameLength", SimTypeShort(signed=False, label="UInt16")), ("FileNameOffset", SimTypeShort(signed=False, label="UInt16")), ("FileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="USN_RECORD_V3", pack=False, align=None),
    "USN_RECORD_V4": SimStruct(OrderedDict((("Header", SimTypeRef("USN_RECORD_COMMON_HEADER", SimStruct)), ("FileReferenceNumber", SimTypeRef("FILE_ID_128", SimStruct)), ("ParentFileReferenceNumber", SimTypeRef("FILE_ID_128", SimStruct)), ("Usn", SimTypeLongLong(signed=True, label="Int64")), ("Reason", SimTypeInt(signed=False, label="UInt32")), ("SourceInfo", SimTypeInt(signed=False, label="USN_SOURCE_INFO_ID")), ("RemainingExtents", SimTypeInt(signed=False, label="UInt32")), ("NumberOfExtents", SimTypeShort(signed=False, label="UInt16")), ("ExtentSize", SimTypeShort(signed=False, label="UInt16")), ("Extents", SimTypePointer(SimTypeRef("USN_RECORD_EXTENT", SimStruct), offset=0)),)), name="USN_RECORD_V4", pack=False, align=None),
    "USN_SOURCE_INFO_ID": SimTypeInt(signed=False, label="USN_SOURCE_INFO_ID"),
    "USN_TRACK_MODIFIED_RANGES": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Unused", SimTypeInt(signed=False, label="UInt32")), ("ChunkSize", SimTypeLongLong(signed=False, label="UInt64")), ("FileSizeThreshold", SimTypeLongLong(signed=True, label="Int64")),)), name="USN_TRACK_MODIFIED_RANGES", pack=False, align=None),
    "UScriptCode": SimTypeInt(signed=False, label="UScriptCode"),
    "UScriptUsage": SimTypeInt(signed=False, label="UScriptUsage"),
    "USearch": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "USearchAttribute": SimTypeInt(signed=False, label="USearchAttribute"),
    "USearchAttributeValue": SimTypeInt(signed=False, label="USearchAttributeValue"),
    "USentenceBreak": SimTypeInt(signed=False, label="USentenceBreak"),
    "USentenceBreakTag": SimTypeInt(signed=False, label="USentenceBreakTag"),
    "USerializedSet": SimStruct(OrderedDict((("array", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("bmpLength", SimTypeInt(signed=True, label="Int32")), ("length", SimTypeInt(signed=True, label="Int32")), ("staticArray", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)),)), name="USerializedSet", pack=False, align=None),
    "USet": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "USetSpanCondition": SimTypeInt(signed=False, label="USetSpanCondition"),
    "USpoofCheckResult": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "USpoofChecker": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "USpoofChecks": SimTypeInt(signed=False, label="USpoofChecks"),
    "UStringCaseMapper": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["csm", "dest", "destCapacity", "src", "srcLength", "pErrorCode"]), offset=0),
    "UStringPrepProfile": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UStringPrepProfileType": SimTypeInt(signed=False, label="UStringPrepProfileType"),
    "UStringSearch": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "UStringTrieBuildOption": SimTypeInt(signed=False, label="UStringTrieBuildOption"),
    "UStringTrieResult": SimTypeInt(signed=False, label="UStringTrieResult"),
    "USystemTimeZoneType": SimTypeInt(signed=False, label="USystemTimeZoneType"),
    "UText": SimStruct(OrderedDict((("magic", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=True, label="Int32")), ("providerProperties", SimTypeInt(signed=True, label="Int32")), ("sizeOfStruct", SimTypeInt(signed=True, label="Int32")), ("chunkNativeLimit", SimTypeLongLong(signed=True, label="Int64")), ("extraSize", SimTypeInt(signed=True, label="Int32")), ("nativeIndexingLimit", SimTypeInt(signed=True, label="Int32")), ("chunkNativeStart", SimTypeLongLong(signed=True, label="Int64")), ("chunkOffset", SimTypeInt(signed=True, label="Int32")), ("chunkLength", SimTypeInt(signed=True, label="Int32")), ("chunkContents", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("pFuncs", SimTypePointer(SimTypeRef("UTextFuncs", SimStruct), offset=0)), ("pExtra", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("p", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("q", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("r", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("privP", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("a", SimTypeLongLong(signed=True, label="Int64")), ("b", SimTypeInt(signed=True, label="Int32")), ("c", SimTypeInt(signed=True, label="Int32")), ("privA", SimTypeLongLong(signed=True, label="Int64")), ("privB", SimTypeInt(signed=True, label="Int32")), ("privC", SimTypeInt(signed=True, label="Int32")),)), name="UText", pack=False, align=None),
    "UTextAccess": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeChar(label="SByte")], SimTypeChar(label="SByte"), arg_names=["ut", "nativeIndex", "forward"]), offset=0),
    "UTextClone": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeChar(label="SByte"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), arg_names=["dest", "src", "deep", "status"]), offset=0),
    "UTextClose": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut"]), offset=0),
    "UTextCopy": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeChar(label="SByte"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut", "nativeStart", "nativeLimit", "nativeDest", "move", "status"]), offset=0),
    "UTextExtract": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ut", "nativeStart", "nativeLimit", "dest", "destCapacity", "status"]), offset=0),
    "UTextFuncs": SimStruct(OrderedDict((("tableSize", SimTypeInt(signed=True, label="Int32")), ("reserved1", SimTypeInt(signed=True, label="Int32")), ("reserved2", SimTypeInt(signed=True, label="Int32")), ("reserved3", SimTypeInt(signed=True, label="Int32")), ("clone", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeChar(label="SByte"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), arg_names=["dest", "src", "deep", "status"]), offset=0)), ("nativeLength", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeLongLong(signed=True, label="Int64"), arg_names=["ut"]), offset=0)), ("access", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeChar(label="SByte")], SimTypeChar(label="SByte"), arg_names=["ut", "nativeIndex", "forward"]), offset=0)), ("extract", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ut", "nativeStart", "nativeLimit", "dest", "destCapacity", "status"]), offset=0)), ("replace", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ut", "nativeStart", "nativeLimit", "replacementText", "replacmentLength", "status"]), offset=0)), ("copy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypeChar(label="SByte"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut", "nativeStart", "nativeLimit", "nativeDest", "move", "status"]), offset=0)), ("mapOffsetToNative", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeLongLong(signed=True, label="Int64"), arg_names=["ut"]), offset=0)), ("mapNativeIndexToUTF16", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["ut", "nativeIndex"]), offset=0)), ("close", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut"]), offset=0)), ("spare1", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut"]), offset=0)), ("spare2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut"]), offset=0)), ("spare3", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["ut"]), offset=0)),)), name="UTextFuncs", pack=False, align=None),
    "UTextMapNativeIndexToUTF16": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["ut", "nativeIndex"]), offset=0),
    "UTextMapOffsetToNative": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeLongLong(signed=True, label="Int64"), arg_names=["ut"]), offset=0),
    "UTextNativeLength": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0)], SimTypeLongLong(signed=True, label="Int64"), arg_names=["ut"]), offset=0),
    "UTextReplace": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UText", SimStruct), offset=0), SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UErrorCode"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ut", "nativeStart", "nativeLimit", "replacementText", "replacmentLength", "status"]), offset=0),
    "UTimeScaleValue": SimTypeInt(signed=False, label="UTimeScaleValue"),
    "UTimeZoneFormatGMTOffsetPatternType": SimTypeInt(signed=False, label="UTimeZoneFormatGMTOffsetPatternType"),
    "UTimeZoneFormatParseOption": SimTypeInt(signed=False, label="UTimeZoneFormatParseOption"),
    "UTimeZoneFormatStyle": SimTypeInt(signed=False, label="UTimeZoneFormatStyle"),
    "UTimeZoneFormatTimeType": SimTypeInt(signed=False, label="UTimeZoneFormatTimeType"),
    "UTimeZoneNameType": SimTypeInt(signed=False, label="UTimeZoneNameType"),
    "UTimeZoneTransitionType": SimTypeInt(signed=False, label="UTimeZoneTransitionType"),
    "UTraceData": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "fnNumber", "level", "fmt", "args"]), offset=0),
    "UTraceEntry": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["context", "fnNumber"]), offset=0),
    "UTraceExit": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "fnNumber", "fmt", "args"]), offset=0),
    "UTraceFunctionNumber": SimTypeInt(signed=False, label="UTraceFunctionNumber"),
    "UTraceLevel": SimTypeInt(signed=False, label="UTraceLevel"),
    "UTransDirection": SimTypeInt(signed=False, label="UTransDirection"),
    "UTransPosition": SimStruct(OrderedDict((("contextStart", SimTypeInt(signed=True, label="Int32")), ("contextLimit", SimTypeInt(signed=True, label="Int32")), ("start", SimTypeInt(signed=True, label="Int32")), ("limit", SimTypeInt(signed=True, label="Int32")),)), name="UTransPosition", pack=False, align=None),
    "UUID_VECTOR": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Uuid", SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)),)), name="UUID_VECTOR", pack=False, align=None),
    "UVerticalOrientation": SimTypeInt(signed=False, label="UVerticalOrientation"),
    "UWordBreak": SimTypeInt(signed=False, label="UWordBreak"),
    "UWordBreakValues": SimTypeInt(signed=False, label="UWordBreakValues"),
    "UiInfo": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="UI_INFO_TYPE")), ("Anonymous", SimUnion({"pwzNull": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ShellInfo": SimTypeRef("ShellCommandInfo", SimStruct), "pwzHelpUrl": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDui": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="UiInfo", pack=False, align=None),
    "UiaAndOrCondition": SimStruct(OrderedDict((("ConditionType", SimTypeInt(signed=False, label="ConditionType")), ("ppConditions", SimTypePointer(SimTypePointer(SimTypeRef("UiaCondition", SimStruct), offset=0), offset=0)), ("cConditions", SimTypeInt(signed=True, label="Int32")),)), name="UiaAndOrCondition", pack=False, align=None),
    "UiaAsyncContentLoadedEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=True, label="Int32")), ("AsyncContentLoadedState", SimTypeInt(signed=False, label="AsyncContentLoadedState")), ("PercentComplete", SimTypeFloat(size=64)),)), name="UiaAsyncContentLoadedEventArgs", pack=False, align=None),
    "UiaCacheRequest": SimStruct(OrderedDict((("pViewCondition", SimTypePointer(SimTypeRef("UiaCondition", SimStruct), offset=0)), ("Scope", SimTypeInt(signed=False, label="TreeScope")), ("pProperties", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("cProperties", SimTypeInt(signed=True, label="Int32")), ("pPatterns", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("cPatterns", SimTypeInt(signed=True, label="Int32")), ("automationElementMode", SimTypeInt(signed=False, label="AutomationElementMode")),)), name="UiaCacheRequest", pack=False, align=None),
    "UiaChangeInfo": SimStruct(OrderedDict((("uiaId", SimTypeInt(signed=True, label="Int32")), ("payload", SimTypeRef("VARIANT", SimStruct)), ("extraInfo", SimTypeRef("VARIANT", SimStruct)),)), name="UiaChangeInfo", pack=False, align=None),
    "UiaChangesEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=True, label="Int32")), ("EventIdCount", SimTypeInt(signed=True, label="Int32")), ("pUiaChanges", SimTypePointer(SimTypeRef("UiaChangeInfo", SimStruct), offset=0)),)), name="UiaChangesEventArgs", pack=False, align=None),
    "UiaCondition": SimStruct(OrderedDict((("ConditionType", SimTypeInt(signed=False, label="ConditionType")),)), name="UiaCondition", pack=False, align=None),
    "UiaEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=True, label="Int32")),)), name="UiaEventArgs", pack=False, align=None),
    "UiaEventCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UiaEventArgs", SimStruct), offset=0), SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pArgs", "pRequestedData", "pTreeStructure"]), offset=0),
    "UiaFindParams": SimStruct(OrderedDict((("MaxDepth", SimTypeInt(signed=True, label="Int32")), ("FindFirst", SimTypeInt(signed=True, label="Int32")), ("ExcludeRoot", SimTypeInt(signed=True, label="Int32")), ("pFindCondition", SimTypePointer(SimTypeRef("UiaCondition", SimStruct), offset=0)),)), name="UiaFindParams", pack=False, align=None),
    "UiaNotCondition": SimStruct(OrderedDict((("ConditionType", SimTypeInt(signed=False, label="ConditionType")), ("pCondition", SimTypePointer(SimTypeRef("UiaCondition", SimStruct), offset=0)),)), name="UiaNotCondition", pack=False, align=None),
    "UiaPoint": SimStruct(OrderedDict((("x", SimTypeFloat(size=64)), ("y", SimTypeFloat(size=64)),)), name="UiaPoint", pack=False, align=None),
    "UiaPropertyChangedEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=False, label="UIA_EVENT_ID")), ("PropertyId", SimTypeInt(signed=True, label="Int32")), ("OldValue", SimTypeRef("VARIANT", SimStruct)), ("NewValue", SimTypeRef("VARIANT", SimStruct)),)), name="UiaPropertyChangedEventArgs", pack=False, align=None),
    "UiaPropertyCondition": SimStruct(OrderedDict((("ConditionType", SimTypeInt(signed=False, label="ConditionType")), ("PropertyId", SimTypeInt(signed=False, label="UIA_PROPERTY_ID")), ("Value", SimTypeRef("VARIANT", SimStruct)), ("Flags", SimTypeInt(signed=False, label="PropertyConditionFlags")),)), name="UiaPropertyCondition", pack=False, align=None),
    "UiaProviderCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="ProviderType")], SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0), arg_names=["hwnd", "providerType"]), offset=0),
    "UiaRect": SimStruct(OrderedDict((("left", SimTypeFloat(size=64)), ("top", SimTypeFloat(size=64)), ("width", SimTypeFloat(size=64)), ("height", SimTypeFloat(size=64)),)), name="UiaRect", pack=False, align=None),
    "UiaStructureChangedEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=True, label="Int32")), ("StructureChangeType", SimTypeInt(signed=False, label="StructureChangeType")), ("pRuntimeId", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("cRuntimeIdLen", SimTypeInt(signed=True, label="Int32")),)), name="UiaStructureChangedEventArgs", pack=False, align=None),
    "UiaTextEditTextChangedEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=True, label="Int32")), ("TextEditChangeType", SimTypeInt(signed=False, label="TextEditChangeType")), ("pTextChange", SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0)),)), name="UiaTextEditTextChangedEventArgs", pack=False, align=None),
    "UiaWindowClosedEventArgs": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="EventArgsType")), ("EventId", SimTypeInt(signed=True, label="Int32")), ("pRuntimeId", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("cRuntimeIdLen", SimTypeInt(signed=True, label="Int32")),)), name="UiaWindowClosedEventArgs", pack=False, align=None),
    "Unit": SimTypeInt(signed=False, label="Unit"),
    "UnregisterAuthoringClientFunctionType": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IWebApplicationHost")], SimTypeInt(signed=True, label="Int32"), arg_names=["host"]), offset=0),
    "UpdateAssessment": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="UpdateAssessmentStatus")), ("impact", SimTypeInt(signed=False, label="UpdateImpactLevel")), ("daysOutOfDate", SimTypeInt(signed=False, label="UInt32")),)), name="UpdateAssessment", pack=False, align=None),
    "UpdateAssessmentStatus": SimTypeInt(signed=False, label="UpdateAssessmentStatus"),
    "UpdateExceptionContext": SimTypeInt(signed=False, label="UpdateExceptionContext"),
    "UpdateImpactLevel": SimTypeInt(signed=False, label="UpdateImpactLevel"),
    "UpdateLockdownOption": SimTypeInt(signed=False, label="UpdateLockdownOption"),
    "UpdateOperation": SimTypeInt(signed=False, label="UpdateOperation"),
    "UpdateServiceOption": SimTypeInt(signed=False, label="UpdateServiceOption"),
    "UpdateServiceRegistrationState": SimTypeInt(signed=False, label="UpdateServiceRegistrationState"),
    "UpdateType": SimTypeInt(signed=False, label="UpdateType"),
    "Uri_HOST_TYPE": SimTypeInt(signed=False, label="Uri_HOST_TYPE"),
    "Uri_PROPERTY": SimTypeInt(signed=False, label="Uri_PROPERTY"),
    "VAL": SimStruct(OrderedDict((("lVal", SimTypeInt(signed=True, label="Int32")), ("dblVal", SimTypeFloat(size=64)), ("pGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pScanInfo", SimTypePointer(SimTypeRef("SCANINFO", SimStruct), offset=0)), ("handle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ppButtonNames", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)), ("pHandle", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("lReserved", SimTypeInt(signed=True, label="Int32")), ("szVal", SimTypeArray(SimTypeChar(label="SByte"), 255)),)), name="VAL", pack=False, align=None),
    "VALENTA": SimStruct(OrderedDict((("ve_valuename", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ve_valuelen", SimTypeInt(signed=False, label="UInt32")), ("ve_valueptr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ve_type", SimTypeInt(signed=False, label="REG_VALUE_TYPE")),)), name="VALENTA", pack=False, align=None),
    "VALENTW": SimStruct(OrderedDict((("ve_valuename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ve_valuelen", SimTypeInt(signed=False, label="UInt32")), ("ve_valueptr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ve_type", SimTypeInt(signed=False, label="REG_VALUE_TYPE")),)), name="VALENTW", pack=False, align=None),
    "VALIDATEUNC_OPTION": SimTypeInt(signed=False, label="VALIDATEUNC_OPTION"),
    "VALID_UOP_FLAG": SimTypeInt(signed=False, label="VALID_UOP_FLAG"),
    "VALIGN": SimTypeInt(signed=False, label="VALIGN"),
    "VALUENAME": SimTypeInt(signed=False, label="VALUENAME"),
    "VARCMP": SimTypeInt(signed=False, label="VARCMP"),
    "VARDESC": SimStruct(OrderedDict((("memid", SimTypeInt(signed=True, label="Int32")), ("lpstrSchema", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Anonymous", SimUnion({"oInst": SimTypeInt(signed=False, label="UInt32"), "lpvarValue": SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0)}, name="<anon>", label="None")), ("elemdescVar", SimTypeRef("ELEMDESC", SimStruct)), ("wVarFlags", SimTypeInt(signed=False, label="VARFLAGS")), ("varkind", SimTypeInt(signed=False, label="VARKIND")),)), name="VARDESC", pack=False, align=None),
    "VARENUM": SimTypeInt(signed=False, label="VARENUM"),
    "VARFLAGS": SimTypeInt(signed=False, label="VARFLAGS"),
    "VARFORMAT_FIRST_DAY": SimTypeInt(signed=False, label="VARFORMAT_FIRST_DAY"),
    "VARFORMAT_FIRST_WEEK": SimTypeInt(signed=False, label="VARFORMAT_FIRST_WEEK"),
    "VARFORMAT_GROUP": SimTypeInt(signed=False, label="VARFORMAT_GROUP"),
    "VARFORMAT_LEADING_DIGIT": SimTypeInt(signed=False, label="VARFORMAT_LEADING_DIGIT"),
    "VARFORMAT_NAMED_FORMAT": SimTypeInt(signed=False, label="VARFORMAT_NAMED_FORMAT"),
    "VARFORMAT_PARENTHESES": SimTypeInt(signed=False, label="VARFORMAT_PARENTHESES"),
    "VARIANT": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("vt", SimTypeInt(signed=False, label="VARENUM")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("wReserved2", SimTypeShort(signed=False, label="UInt16")), ("wReserved3", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimTypeRef("SAFEARRAY", SimStruct), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeRef("VARIANT", SimStruct), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeChar(label="SByte"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeRef("DECIMAL", SimStruct), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct(OrderedDict((("pvRecord", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pRecInfo", SimTypeBottom(label="IRecordInfo")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeRef("DECIMAL", SimStruct)}, name="<anon>", label="None")),)), name="VARIANT", pack=False, align=None),
    "VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"),
    "VARKIND": SimTypeInt(signed=False, label="VARKIND"),
    "VARSTRING": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNeededSize", SimTypeInt(signed=False, label="UInt32")), ("dwUsedSize", SimTypeInt(signed=False, label="UInt32")), ("dwStringFormat", SimTypeInt(signed=False, label="UInt32")), ("dwStringSize", SimTypeInt(signed=False, label="UInt32")), ("dwStringOffset", SimTypeInt(signed=False, label="UInt32")),)), name="VARSTRING", pack=False, align=None),
    "VAR_CHANGE_FLAGS": SimTypeInt(signed=False, label="VAR_CHANGE_FLAGS"),
    "VAR_STRING": SimStruct(OrderedDict((("ulTotalSize", SimTypeInt(signed=False, label="UInt32")), ("ulNeededSize", SimTypeInt(signed=False, label="UInt32")), ("ulUsedSize", SimTypeInt(signed=False, label="UInt32")), ("ulStringFormat", SimTypeInt(signed=False, label="UInt32")), ("ulStringSize", SimTypeInt(signed=False, label="UInt32")), ("ulStringOffset", SimTypeInt(signed=False, label="UInt32")),)), name="VAR_STRING", pack=False, align=None),
    "VA_COLOR_PRIMARIES": SimTypeInt(signed=False, label="VA_COLOR_PRIMARIES"),
    "VA_MATRIX_COEFFICIENTS": SimTypeInt(signed=False, label="VA_MATRIX_COEFFICIENTS"),
    "VA_OPTIONAL_VIDEO_PROPERTIES": SimStruct(OrderedDict((("dwPictureHeight", SimTypeShort(signed=False, label="UInt16")), ("dwPictureWidth", SimTypeShort(signed=False, label="UInt16")), ("dwAspectRatioX", SimTypeShort(signed=False, label="UInt16")), ("dwAspectRatioY", SimTypeShort(signed=False, label="UInt16")), ("VAVideoFormat", SimTypeInt(signed=False, label="VA_VIDEO_FORMAT")), ("VAColorPrimaries", SimTypeInt(signed=False, label="VA_COLOR_PRIMARIES")), ("VATransferCharacteristics", SimTypeInt(signed=False, label="VA_TRANSFER_CHARACTERISTICS")), ("VAMatrixCoefficients", SimTypeInt(signed=False, label="VA_MATRIX_COEFFICIENTS")),)), name="VA_OPTIONAL_VIDEO_PROPERTIES", pack=False, align=None),
    "VA_TRANSFER_CHARACTERISTICS": SimTypeInt(signed=False, label="VA_TRANSFER_CHARACTERISTICS"),
    "VA_VIDEO_FORMAT": SimTypeInt(signed=False, label="VA_VIDEO_FORMAT"),
    "VBICAP_PROPERTIES_PROTECTION_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("StreamIndex", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=False, label="UInt32")),)), name="VBICAP_PROPERTIES_PROTECTION_S", pack=False, align=None),
    "VBICODECFILTERING_CC_SUBSTREAMS": SimStruct(OrderedDict((("SubstreamMask", SimTypeInt(signed=False, label="UInt32")),)), name="VBICODECFILTERING_CC_SUBSTREAMS", pack=False, align=None),
    "VBICODECFILTERING_NABTS_SUBSTREAMS": SimStruct(OrderedDict((("SubstreamMask", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 128)),)), name="VBICODECFILTERING_NABTS_SUBSTREAMS", pack=False, align=None),
    "VBICODECFILTERING_SCANLINES": SimStruct(OrderedDict((("DwordBitArray", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)),)), name="VBICODECFILTERING_SCANLINES", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_CC": SimStruct(OrderedDict((("Common", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON", SimStruct)),)), name="VBICODECFILTERING_STATISTICS_CC", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_CC_PIN": SimStruct(OrderedDict((("Common", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON_PIN", SimStruct)),)), name="VBICODECFILTERING_STATISTICS_CC_PIN", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_COMMON": SimStruct(OrderedDict((("InputSRBsProcessed", SimTypeInt(signed=False, label="UInt32")), ("OutputSRBsProcessed", SimTypeInt(signed=False, label="UInt32")), ("SRBsIgnored", SimTypeInt(signed=False, label="UInt32")), ("InputSRBsMissing", SimTypeInt(signed=False, label="UInt32")), ("OutputSRBsMissing", SimTypeInt(signed=False, label="UInt32")), ("OutputFailures", SimTypeInt(signed=False, label="UInt32")), ("InternalErrors", SimTypeInt(signed=False, label="UInt32")), ("ExternalErrors", SimTypeInt(signed=False, label="UInt32")), ("InputDiscontinuities", SimTypeInt(signed=False, label="UInt32")), ("DSPFailures", SimTypeInt(signed=False, label="UInt32")), ("TvTunerChanges", SimTypeInt(signed=False, label="UInt32")), ("VBIHeaderChanges", SimTypeInt(signed=False, label="UInt32")), ("LineConfidenceAvg", SimTypeInt(signed=False, label="UInt32")), ("BytesOutput", SimTypeInt(signed=False, label="UInt32")),)), name="VBICODECFILTERING_STATISTICS_COMMON", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_COMMON_PIN": SimStruct(OrderedDict((("SRBsProcessed", SimTypeInt(signed=False, label="UInt32")), ("SRBsIgnored", SimTypeInt(signed=False, label="UInt32")), ("SRBsMissing", SimTypeInt(signed=False, label="UInt32")), ("InternalErrors", SimTypeInt(signed=False, label="UInt32")), ("ExternalErrors", SimTypeInt(signed=False, label="UInt32")), ("Discontinuities", SimTypeInt(signed=False, label="UInt32")), ("LineConfidenceAvg", SimTypeInt(signed=False, label="UInt32")), ("BytesOutput", SimTypeInt(signed=False, label="UInt32")),)), name="VBICODECFILTERING_STATISTICS_COMMON_PIN", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_NABTS": SimStruct(OrderedDict((("Common", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON", SimStruct)), ("FECBundleBadLines", SimTypeInt(signed=False, label="UInt32")), ("FECQueueOverflows", SimTypeInt(signed=False, label="UInt32")), ("FECCorrectedLines", SimTypeInt(signed=False, label="UInt32")), ("FECUncorrectableLines", SimTypeInt(signed=False, label="UInt32")), ("BundlesProcessed", SimTypeInt(signed=False, label="UInt32")), ("BundlesSent2IP", SimTypeInt(signed=False, label="UInt32")), ("FilteredLines", SimTypeInt(signed=False, label="UInt32")),)), name="VBICODECFILTERING_STATISTICS_NABTS", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_NABTS_PIN": SimStruct(OrderedDict((("Common", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON_PIN", SimStruct)),)), name="VBICODECFILTERING_STATISTICS_NABTS_PIN", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_TELETEXT": SimStruct(OrderedDict((("Common", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON", SimStruct)),)), name="VBICODECFILTERING_STATISTICS_TELETEXT", pack=False, align=None),
    "VBICODECFILTERING_STATISTICS_TELETEXT_PIN": SimStruct(OrderedDict((("Common", SimTypeRef("VBICODECFILTERING_STATISTICS_COMMON_PIN", SimStruct)),)), name="VBICODECFILTERING_STATISTICS_TELETEXT_PIN", pack=False, align=None),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_COMMIT_PAGES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveAddress", "NumberOfBytes", "SourceAddress", "PageProtection"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_CREATE_THREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ThreadDescriptor"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_DECOMMIT_PAGES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveAddress", "NumberOfBytes"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_GENERATE_KEY": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ENCLAVE_VBS_BASIC_KEY_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["KeyRequest", "RequestedKeySize", "ReturnedKey"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_GENERATE_RANDOM_DATA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Buffer", "NumberOfBytes", "Generation"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_GENERATE_REPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveData", "Report", "BufferSize", "OutputSize"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_GET_ENCLAVE_INFORMATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ENCLAVE_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveInfo"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_INTERRUPT_THREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ThreadDescriptor"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_PROTECT_PAGES": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveAddress", "NumberOfytes", "PageProtection"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_RETURN_FROM_ENCLAVE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["ReturnValue"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_RETURN_FROM_EXCEPTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ExceptionRecord"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_TERMINATE_THREAD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ThreadDescriptor"]), offset=0),
    "VBS_BASIC_ENCLAVE_BASIC_CALL_VERIFY_REPORT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Report", "ReportSize"]), offset=0),
    "VBS_BASIC_ENCLAVE_EXCEPTION_AMD64": SimStruct(OrderedDict((("ExceptionCode", SimTypeInt(signed=False, label="UInt32")), ("NumberParameters", SimTypeInt(signed=False, label="UInt32")), ("ExceptionInformation", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)), ("ExceptionRAX", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ExceptionRCX", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ExceptionRIP", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ExceptionRFLAGS", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ExceptionRSP", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="VBS_BASIC_ENCLAVE_EXCEPTION_AMD64", pack=False, align=None),
    "VBS_BASIC_ENCLAVE_SYSCALL_PAGE": SimStruct(OrderedDict((("ReturnFromEnclave", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["ReturnValue"]), offset=0)), ("ReturnFromException", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ExceptionRecord"]), offset=0)), ("TerminateThread", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ThreadDescriptor"]), offset=0)), ("InterruptThread", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ThreadDescriptor"]), offset=0)), ("CommitPages", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveAddress", "NumberOfBytes", "SourceAddress", "PageProtection"]), offset=0)), ("DecommitPages", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveAddress", "NumberOfBytes"]), offset=0)), ("ProtectPages", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveAddress", "NumberOfytes", "PageProtection"]), offset=0)), ("CreateThread", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ThreadDescriptor"]), offset=0)), ("GetEnclaveInformation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ENCLAVE_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveInfo"]), offset=0)), ("GenerateKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("ENCLAVE_VBS_BASIC_KEY_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["KeyRequest", "RequestedKeySize", "ReturnedKey"]), offset=0)), ("GenerateReport", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EnclaveData", "Report", "BufferSize", "OutputSize"]), offset=0)), ("VerifyReport", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Report", "ReportSize"]), offset=0)), ("GenerateRandomData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Buffer", "NumberOfBytes", "Generation"]), offset=0)),)), name="VBS_BASIC_ENCLAVE_SYSCALL_PAGE", pack=False, align=None),
    "VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32": SimStruct(OrderedDict((("ThreadContext", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("EntryPoint", SimTypeInt(signed=False, label="UInt32")), ("StackPointer", SimTypeInt(signed=False, label="UInt32")), ("ExceptionEntryPoint", SimTypeInt(signed=False, label="UInt32")), ("ExceptionStack", SimTypeInt(signed=False, label="UInt32")), ("ExceptionActive", SimTypeInt(signed=False, label="UInt32")),)), name="VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR32", pack=False, align=None),
    "VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR64": SimStruct(OrderedDict((("ThreadContext", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 4)), ("EntryPoint", SimTypeLongLong(signed=False, label="UInt64")), ("StackPointer", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionEntryPoint", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionStack", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionActive", SimTypeInt(signed=False, label="UInt32")),)), name="VBS_BASIC_ENCLAVE_THREAD_DESCRIPTOR64", pack=False, align=None),
    "VBS_ENCLAVE_REPORT": SimStruct(OrderedDict((("ReportSize", SimTypeInt(signed=False, label="UInt32")), ("ReportVersion", SimTypeInt(signed=False, label="UInt32")), ("EnclaveData", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("EnclaveIdentity", SimTypeRef("ENCLAVE_IDENTITY", SimStruct)),)), name="VBS_ENCLAVE_REPORT", pack=False, align=None),
    "VBS_ENCLAVE_REPORT_MODULE": SimStruct(OrderedDict((("Header", SimTypeRef("VBS_ENCLAVE_REPORT_VARDATA_HEADER", SimStruct)), ("UniqueId", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("AuthorId", SimTypeArray(SimTypeChar(label="Byte"), 32)), ("FamilyId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ImageId", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("Svn", SimTypeInt(signed=False, label="UInt32")), ("ModuleName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VBS_ENCLAVE_REPORT_MODULE", pack=False, align=None),
    "VBS_ENCLAVE_REPORT_PKG_HEADER": SimStruct(OrderedDict((("PackageSize", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("SignatureScheme", SimTypeInt(signed=False, label="UInt32")), ("SignedStatementSize", SimTypeInt(signed=False, label="UInt32")), ("SignatureSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="VBS_ENCLAVE_REPORT_PKG_HEADER", pack=False, align=None),
    "VBS_ENCLAVE_REPORT_VARDATA_HEADER": SimStruct(OrderedDict((("DataType", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="VBS_ENCLAVE_REPORT_VARDATA_HEADER", pack=False, align=None),
    "VCN_RANGE_INPUT_BUFFER": SimStruct(OrderedDict((("StartingVcn", SimTypeLongLong(signed=True, label="Int64")), ("ClusterCount", SimTypeLongLong(signed=True, label="Int64")),)), name="VCN_RANGE_INPUT_BUFFER", pack=False, align=None),
    "VDMBREAKTHREADPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "VDMCONTEXT": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=False, label="UInt32")), ("Dr0", SimTypeInt(signed=False, label="UInt32")), ("Dr1", SimTypeInt(signed=False, label="UInt32")), ("Dr2", SimTypeInt(signed=False, label="UInt32")), ("Dr3", SimTypeInt(signed=False, label="UInt32")), ("Dr6", SimTypeInt(signed=False, label="UInt32")), ("Dr7", SimTypeInt(signed=False, label="UInt32")), ("FloatSave", SimTypeRef("FLOATING_SAVE_AREA", SimStruct)), ("SegGs", SimTypeInt(signed=False, label="UInt32")), ("SegFs", SimTypeInt(signed=False, label="UInt32")), ("SegEs", SimTypeInt(signed=False, label="UInt32")), ("SegDs", SimTypeInt(signed=False, label="UInt32")), ("Edi", SimTypeInt(signed=False, label="UInt32")), ("Esi", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebp", SimTypeInt(signed=False, label="UInt32")), ("Eip", SimTypeInt(signed=False, label="UInt32")), ("SegCs", SimTypeInt(signed=False, label="UInt32")), ("EFlags", SimTypeInt(signed=False, label="UInt32")), ("Esp", SimTypeInt(signed=False, label="UInt32")), ("SegSs", SimTypeInt(signed=False, label="UInt32")), ("ExtendedRegisters", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="VDMCONTEXT", pack=False, align=None),
    "VDMCONTEXT_WITHOUT_XSAVE": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=False, label="UInt32")), ("Dr0", SimTypeInt(signed=False, label="UInt32")), ("Dr1", SimTypeInt(signed=False, label="UInt32")), ("Dr2", SimTypeInt(signed=False, label="UInt32")), ("Dr3", SimTypeInt(signed=False, label="UInt32")), ("Dr6", SimTypeInt(signed=False, label="UInt32")), ("Dr7", SimTypeInt(signed=False, label="UInt32")), ("FloatSave", SimTypeRef("FLOATING_SAVE_AREA", SimStruct)), ("SegGs", SimTypeInt(signed=False, label="UInt32")), ("SegFs", SimTypeInt(signed=False, label="UInt32")), ("SegEs", SimTypeInt(signed=False, label="UInt32")), ("SegDs", SimTypeInt(signed=False, label="UInt32")), ("Edi", SimTypeInt(signed=False, label="UInt32")), ("Esi", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebp", SimTypeInt(signed=False, label="UInt32")), ("Eip", SimTypeInt(signed=False, label="UInt32")), ("SegCs", SimTypeInt(signed=False, label="UInt32")), ("EFlags", SimTypeInt(signed=False, label="UInt32")), ("Esp", SimTypeInt(signed=False, label="UInt32")), ("SegSs", SimTypeInt(signed=False, label="UInt32")),)), name="VDMCONTEXT_WITHOUT_XSAVE", pack=False, align=None),
    "VDMDETECTWOWPROC": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "VDMENUMPROCESSWOWPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwProcessId", "dwAttributes", "lpUserDefined"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "VDMENUMTASKWOWEXPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwThreadId", "hMod16", "hTask16", "pszModName", "pszFileName", "lpUserDefined"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "VDMENUMTASKWOWPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwThreadId", "hMod16", "hTask16", "lpUserDefined"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "VDMGETADDREXPRESSIONPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "VDMGETCONTEXTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "VDMGETDBGFLAGSPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]), offset=0),
    "VDMGETMODULESELECTORPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "VDMGETPOINTERPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "VDMGETSEGMENTINFOPROC": SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeRef("VDM_SEGINFO", SimStruct)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "VDMGETSELECTORMODULEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6", "param7"]), offset=0),
    "VDMGETSYMBOLPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5", "param6"]), offset=0),
    "VDMGETTHREADSELECTORENTRYPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("LDT_ENTRY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "VDMGLOBALFIRSTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("GLOBALENTRY", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "VDMGLOBALNEXTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("GLOBALENTRY", SimStruct), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4", "param5"]), offset=0),
    "VDMISMODULELOADEDPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "VDMKILLWOWPROC": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "VDMLDT_ENTRY": SimStruct(OrderedDict((("LimitLow", SimTypeShort(signed=False, label="UInt16")), ("BaseLow", SimTypeShort(signed=False, label="UInt16")), ("HighWord", SimUnion({"Bytes": SimTypeRef("_Bytes_e__Struct", SimStruct), "Bits": SimTypeRef("_Bits_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="VDMLDT_ENTRY", pack=False, align=None),
    "VDMMODULEFIRSTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("MODULEENTRY", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "VDMMODULENEXTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("MODULEENTRY", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_EVENT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3", "param4"]), offset=0),
    "VDMPROCESSEXCEPTIONPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_EVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0"]), offset=0),
    "VDMSETCONTEXTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "VDMSETDBGFLAGSPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "VDMSTARTTASKINWOWPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "VDMTERMINATETASKINWOWPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "VDM_SEGINFO": SimStruct(OrderedDict((("Selector", SimTypeShort(signed=False, label="UInt16")), ("SegNumber", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeShort(signed=False, label="UInt16")), ("ModuleName", SimTypeArray(SimTypeChar(label="SByte"), 9)), ("FileName", SimTypeArray(SimTypeChar(label="SByte"), 255)),)), name="VDM_SEGINFO", pack=False, align=None),
    "VDS_ADVANCEDDISK_PROP": SimStruct(OrderedDict((("pwszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPathname", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pswzIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("usIdentifierFormat", SimTypeShort(signed=False, label="UInt16")), ("ulNumber", SimTypeInt(signed=False, label="UInt32")), ("pwszSerialNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFirmwareVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszManufacturer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszModel", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ullTotalSize", SimTypeLongLong(signed=False, label="UInt64")), ("ullAllocatedSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulLogicalSectorSize", SimTypeInt(signed=False, label="UInt32")), ("ulPhysicalSectorSize", SimTypeInt(signed=False, label="UInt32")), ("ulPartitionCount", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="VDS_DISK_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("BusType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("PartitionStyle", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("Anonymous", SimUnion({"dwSignature": SimTypeInt(signed=False, label="UInt32"), "DiskGuid": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_ADVANCEDDISK_PROP", pack=False, align=None),
    "VDS_ASYNC_OUTPUT": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="VDS_ASYNC_OUTPUT_TYPE")), ("Anonymous", SimUnion({"cp": SimTypeRef("_cp", SimStruct), "cv": SimTypeRef("_cv", SimStruct), "bvp": SimTypeRef("_bvp", SimStruct), "sv": SimTypeRef("_sv", SimStruct), "cl": SimTypeRef("_cl", SimStruct), "ct": SimTypeRef("_ct", SimStruct), "cpg": SimTypeRef("_cpg", SimStruct), "cvd": SimTypeRef("_cvd", SimStruct)}, name="<anon>", label="None")),)), name="VDS_ASYNC_OUTPUT", pack=False, align=None),
    "VDS_ASYNC_OUTPUT_TYPE": SimTypeInt(signed=False, label="VDS_ASYNC_OUTPUT_TYPE"),
    "VDS_CONTROLLER_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_CONTROLLER")), ("controllerId", SimTypeBottom(label="Guid")),)), name="VDS_CONTROLLER_NOTIFICATION", pack=False, align=None),
    "VDS_CONTROLLER_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("status", SimTypeInt(signed=False, label="VDS_CONTROLLER_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("sNumberOfPorts", SimTypeShort(signed=True, label="Int16")),)), name="VDS_CONTROLLER_PROP", pack=False, align=None),
    "VDS_CONTROLLER_STATUS": SimTypeInt(signed=False, label="VDS_CONTROLLER_STATUS"),
    "VDS_CREATE_VDISK_PARAMETERS": SimStruct(OrderedDict((("UniqueId", SimTypeBottom(label="Guid")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("BlockSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("SectorSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("pParentPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pSourcePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_CREATE_VDISK_PARAMETERS", pack=False, align=None),
    "VDS_DISK_EXTENT": SimStruct(OrderedDict((("diskId", SimTypeBottom(label="Guid")), ("type", SimTypeInt(signed=False, label="VDS_DISK_EXTENT_TYPE")), ("ullOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("volumeId", SimTypeBottom(label="Guid")), ("plexId", SimTypeBottom(label="Guid")), ("memberIdx", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_DISK_EXTENT", pack=False, align=None),
    "VDS_DISK_EXTENT_TYPE": SimTypeInt(signed=False, label="VDS_DISK_EXTENT_TYPE"),
    "VDS_DISK_FLAG": SimTypeInt(signed=False, label="VDS_DISK_FLAG"),
    "VDS_DISK_FREE_EXTENT": SimStruct(OrderedDict((("diskId", SimTypeBottom(label="Guid")), ("ullOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="VDS_DISK_FREE_EXTENT", pack=False, align=None),
    "VDS_DISK_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_DISK")), ("diskId", SimTypeBottom(label="Guid")),)), name="VDS_DISK_NOTIFICATION", pack=False, align=None),
    "VDS_DISK_OFFLINE_REASON": SimTypeInt(signed=False, label="VDS_DISK_OFFLINE_REASON"),
    "VDS_DISK_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("status", SimTypeInt(signed=False, label="VDS_DISK_STATUS")), ("ReserveMode", SimTypeInt(signed=False, label="VDS_LUN_RESERVE_MODE")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("dwDeviceType", SimTypeInt(signed=False, label="UInt32")), ("dwMediaType", SimTypeInt(signed=False, label="UInt32")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulBytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("ulSectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("ulTracksPerCylinder", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("BusType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("PartitionStyle", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("Anonymous", SimUnion({"dwSignature": SimTypeInt(signed=False, label="UInt32"), "DiskGuid": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("pwszDiskAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszAdaptorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDevicePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_DISK_PROP", pack=False, align=None),
    "VDS_DISK_PROP2": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("status", SimTypeInt(signed=False, label="VDS_DISK_STATUS")), ("OfflineReason", SimTypeInt(signed=False, label="VDS_DISK_OFFLINE_REASON")), ("ReserveMode", SimTypeInt(signed=False, label="VDS_LUN_RESERVE_MODE")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("dwDeviceType", SimTypeInt(signed=False, label="UInt32")), ("dwMediaType", SimTypeInt(signed=False, label="UInt32")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulBytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("ulSectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("ulTracksPerCylinder", SimTypeInt(signed=False, label="UInt32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("BusType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("PartitionStyle", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("Anonymous", SimUnion({"dwSignature": SimTypeInt(signed=False, label="UInt32"), "DiskGuid": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("pwszDiskAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszAdaptorName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDevicePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszLocationPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_DISK_PROP2", pack=False, align=None),
    "VDS_DISK_STATUS": SimTypeInt(signed=False, label="VDS_DISK_STATUS"),
    "VDS_DRIVE_EXTENT": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("LunId", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("bUsed", SimTypeInt(signed=True, label="Int32")),)), name="VDS_DRIVE_EXTENT", pack=False, align=None),
    "VDS_DRIVE_FLAG": SimTypeInt(signed=False, label="VDS_DRIVE_FLAG"),
    "VDS_DRIVE_LETTER_FLAG": SimTypeInt(signed=False, label="VDS_DRIVE_LETTER_FLAG"),
    "VDS_DRIVE_LETTER_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("wcLetter", SimTypeChar(label="Char")), ("volumeId", SimTypeBottom(label="Guid")),)), name="VDS_DRIVE_LETTER_NOTIFICATION", pack=False, align=None),
    "VDS_DRIVE_LETTER_PROP": SimStruct(OrderedDict((("wcLetter", SimTypeChar(label="Char")), ("volumeId", SimTypeBottom(label="Guid")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("bUsed", SimTypeInt(signed=True, label="Int32")),)), name="VDS_DRIVE_LETTER_PROP", pack=False, align=None),
    "VDS_DRIVE_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_DRIVE")), ("driveId", SimTypeBottom(label="Guid")),)), name="VDS_DRIVE_NOTIFICATION", pack=False, align=None),
    "VDS_DRIVE_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="VDS_DRIVE_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("sInternalBusNumber", SimTypeShort(signed=True, label="Int16")), ("sSlotNumber", SimTypeShort(signed=True, label="Int16")),)), name="VDS_DRIVE_PROP", pack=False, align=None),
    "VDS_DRIVE_PROP2": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="VDS_DRIVE_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("sInternalBusNumber", SimTypeShort(signed=True, label="Int16")), ("sSlotNumber", SimTypeShort(signed=True, label="Int16")), ("ulEnclosureNumber", SimTypeInt(signed=False, label="UInt32")), ("busType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("ulSpindleSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_DRIVE_PROP2", pack=False, align=None),
    "VDS_DRIVE_STATUS": SimTypeInt(signed=False, label="VDS_DRIVE_STATUS"),
    "VDS_FILE_SYSTEM_FLAG": SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_FLAG"),
    "VDS_FILE_SYSTEM_FORMAT_SUPPORT_FLAG": SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_FORMAT_SUPPORT_FLAG"),
    "VDS_FILE_SYSTEM_FORMAT_SUPPORT_PROP": SimStruct(OrderedDict((("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("usRevision", SimTypeShort(signed=False, label="UInt16")), ("ulDefaultUnitAllocationSize", SimTypeInt(signed=False, label="UInt32")), ("rgulAllowedUnitAllocationSizes", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)), ("wszName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="VDS_FILE_SYSTEM_FORMAT_SUPPORT_PROP", pack=False, align=None),
    "VDS_FILE_SYSTEM_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_FILE_SYSTEM")), ("volumeId", SimTypeBottom(label="Guid")), ("dwPercentCompleted", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_FILE_SYSTEM_NOTIFICATION", pack=False, align=None),
    "VDS_FILE_SYSTEM_PROP": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_TYPE")), ("volumeId", SimTypeBottom(label="Guid")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ullTotalAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("ullAvailableAllocationUnits", SimTypeLongLong(signed=False, label="UInt64")), ("ulAllocationUnitSize", SimTypeInt(signed=False, label="UInt32")), ("pwszLabel", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_FILE_SYSTEM_PROP", pack=False, align=None),
    "VDS_FILE_SYSTEM_PROP_FLAG": SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_PROP_FLAG"),
    "VDS_FILE_SYSTEM_TYPE": SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_TYPE"),
    "VDS_FILE_SYSTEM_TYPE_PROP": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_TYPE")), ("wszName", SimTypeArray(SimTypeChar(label="Char"), 8)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulCompressionFlags", SimTypeInt(signed=False, label="UInt32")), ("ulMaxLableLength", SimTypeInt(signed=False, label="UInt32")), ("pwszIllegalLabelCharSet", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_FILE_SYSTEM_TYPE_PROP", pack=False, align=None),
    "VDS_FORMAT_OPTION_FLAGS": SimTypeInt(signed=False, label="VDS_FORMAT_OPTION_FLAGS"),
    "VDS_HBAPORT_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("wwnNode", SimTypeRef("VDS_WWN", SimStruct)), ("wwnPort", SimTypeRef("VDS_WWN", SimStruct)), ("type", SimTypeInt(signed=False, label="VDS_HBAPORT_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_HBAPORT_STATUS")), ("ulPortSpeed", SimTypeInt(signed=False, label="UInt32")), ("ulSupportedPortSpeed", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_HBAPORT_PROP", pack=False, align=None),
    "VDS_HBAPORT_SPEED_FLAG": SimTypeInt(signed=False, label="VDS_HBAPORT_SPEED_FLAG"),
    "VDS_HBAPORT_STATUS": SimTypeInt(signed=False, label="VDS_HBAPORT_STATUS"),
    "VDS_HBAPORT_TYPE": SimTypeInt(signed=False, label="VDS_HBAPORT_TYPE"),
    "VDS_HEALTH": SimTypeInt(signed=False, label="VDS_HEALTH"),
    "VDS_HINTS": SimStruct(OrderedDict((("ullHintMask", SimTypeLongLong(signed=False, label="UInt64")), ("ullExpectedMaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulOptimalReadSize", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalReadAlignment", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalWriteSize", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalWriteAlignment", SimTypeInt(signed=False, label="UInt32")), ("ulMaximumDriveCount", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSize", SimTypeInt(signed=False, label="UInt32")), ("bFastCrashRecoveryRequired", SimTypeInt(signed=True, label="Int32")), ("bMostlyReads", SimTypeInt(signed=True, label="Int32")), ("bOptimizeForSequentialReads", SimTypeInt(signed=True, label="Int32")), ("bOptimizeForSequentialWrites", SimTypeInt(signed=True, label="Int32")), ("bRemapEnabled", SimTypeInt(signed=True, label="Int32")), ("bReadBackVerifyEnabled", SimTypeInt(signed=True, label="Int32")), ("bWriteThroughCachingEnabled", SimTypeInt(signed=True, label="Int32")), ("bHardwareChecksumEnabled", SimTypeInt(signed=True, label="Int32")), ("bIsYankable", SimTypeInt(signed=True, label="Int32")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")),)), name="VDS_HINTS", pack=False, align=None),
    "VDS_HINTS2": SimStruct(OrderedDict((("ullHintMask", SimTypeLongLong(signed=False, label="UInt64")), ("ullExpectedMaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulOptimalReadSize", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalReadAlignment", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalWriteSize", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalWriteAlignment", SimTypeInt(signed=False, label="UInt32")), ("ulMaximumDriveCount", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSize", SimTypeInt(signed=False, label="UInt32")), ("ulReserved1", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeInt(signed=False, label="UInt32")), ("ulReserved3", SimTypeInt(signed=False, label="UInt32")), ("bFastCrashRecoveryRequired", SimTypeInt(signed=True, label="Int32")), ("bMostlyReads", SimTypeInt(signed=True, label="Int32")), ("bOptimizeForSequentialReads", SimTypeInt(signed=True, label="Int32")), ("bOptimizeForSequentialWrites", SimTypeInt(signed=True, label="Int32")), ("bRemapEnabled", SimTypeInt(signed=True, label="Int32")), ("bReadBackVerifyEnabled", SimTypeInt(signed=True, label="Int32")), ("bWriteThroughCachingEnabled", SimTypeInt(signed=True, label="Int32")), ("bHardwareChecksumEnabled", SimTypeInt(signed=True, label="Int32")), ("bIsYankable", SimTypeInt(signed=True, label="Int32")), ("bAllocateHotSpare", SimTypeInt(signed=True, label="Int32")), ("bUseMirroredCache", SimTypeInt(signed=True, label="Int32")), ("bReadCachingEnabled", SimTypeInt(signed=True, label="Int32")), ("bWriteCachingEnabled", SimTypeInt(signed=True, label="Int32")), ("bMediaScanEnabled", SimTypeInt(signed=True, label="Int32")), ("bConsistencyCheckEnabled", SimTypeInt(signed=True, label="Int32")), ("BusType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("bReserved1", SimTypeInt(signed=True, label="Int32")), ("bReserved2", SimTypeInt(signed=True, label="Int32")), ("bReserved3", SimTypeInt(signed=True, label="Int32")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")),)), name="VDS_HINTS2", pack=False, align=None),
    "VDS_HWPROVIDER_TYPE": SimTypeInt(signed=False, label="VDS_HWPROVIDER_TYPE"),
    "VDS_INPUT_DISK": SimStruct(OrderedDict((("diskId", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("plexId", SimTypeBottom(label="Guid")), ("memberIdx", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_INPUT_DISK", pack=False, align=None),
    "VDS_INTERCONNECT": SimStruct(OrderedDict((("m_addressType", SimTypeInt(signed=False, label="VDS_INTERCONNECT_ADDRESS_TYPE")), ("m_cbPort", SimTypeInt(signed=False, label="UInt32")), ("m_pbPort", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("m_cbAddress", SimTypeInt(signed=False, label="UInt32")), ("m_pbAddress", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VDS_INTERCONNECT", pack=False, align=None),
    "VDS_INTERCONNECT_ADDRESS_TYPE": SimTypeInt(signed=False, label="VDS_INTERCONNECT_ADDRESS_TYPE"),
    "VDS_INTERCONNECT_FLAG": SimTypeInt(signed=False, label="VDS_INTERCONNECT_FLAG"),
    "VDS_IPADDRESS": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="VDS_IPADDRESS_TYPE")), ("ipv4Address", SimTypeInt(signed=False, label="UInt32")), ("ipv6Address", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ulIpv6FlowInfo", SimTypeInt(signed=False, label="UInt32")), ("ulIpv6ScopeId", SimTypeInt(signed=False, label="UInt32")), ("wszTextAddress", SimTypeArray(SimTypeChar(label="Char"), 257)), ("ulPort", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_IPADDRESS", pack=False, align=None),
    "VDS_IPADDRESS_TYPE": SimTypeInt(signed=False, label="VDS_IPADDRESS_TYPE"),
    "VDS_ISCSI_AUTH_TYPE": SimTypeInt(signed=False, label="VDS_ISCSI_AUTH_TYPE"),
    "VDS_ISCSI_INITIATOR_ADAPTER_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_ISCSI_INITIATOR_ADAPTER_PROP", pack=False, align=None),
    "VDS_ISCSI_INITIATOR_PORTAL_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("address", SimTypeRef("VDS_IPADDRESS", SimStruct)), ("ulPortIndex", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_ISCSI_INITIATOR_PORTAL_PROP", pack=False, align=None),
    "VDS_ISCSI_IPSEC_FLAG": SimTypeInt(signed=False, label="VDS_ISCSI_IPSEC_FLAG"),
    "VDS_ISCSI_IPSEC_KEY": SimStruct(OrderedDict((("pKey", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulKeySize", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_ISCSI_IPSEC_KEY", pack=False, align=None),
    "VDS_ISCSI_LOGIN_FLAG": SimTypeInt(signed=False, label="VDS_ISCSI_LOGIN_FLAG"),
    "VDS_ISCSI_LOGIN_TYPE": SimTypeInt(signed=False, label="VDS_ISCSI_LOGIN_TYPE"),
    "VDS_ISCSI_PORTALGROUP_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("tag", SimTypeShort(signed=False, label="UInt16")),)), name="VDS_ISCSI_PORTALGROUP_PROP", pack=False, align=None),
    "VDS_ISCSI_PORTAL_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("address", SimTypeRef("VDS_IPADDRESS", SimStruct)), ("status", SimTypeInt(signed=False, label="VDS_ISCSI_PORTAL_STATUS")),)), name="VDS_ISCSI_PORTAL_PROP", pack=False, align=None),
    "VDS_ISCSI_PORTAL_STATUS": SimTypeInt(signed=False, label="VDS_ISCSI_PORTAL_STATUS"),
    "VDS_ISCSI_SHARED_SECRET": SimStruct(OrderedDict((("pSharedSecret", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulSharedSecretSize", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_ISCSI_SHARED_SECRET", pack=False, align=None),
    "VDS_ISCSI_TARGET_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszIscsiName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bChapEnabled", SimTypeInt(signed=True, label="Int32")),)), name="VDS_ISCSI_TARGET_PROP", pack=False, align=None),
    "VDS_LOADBALANCE_POLICY_ENUM": SimTypeInt(signed=False, label="VDS_LOADBALANCE_POLICY_ENUM"),
    "VDS_LUN_FLAG": SimTypeInt(signed=False, label="VDS_LUN_FLAG"),
    "VDS_LUN_INFORMATION": SimStruct(OrderedDict((("m_version", SimTypeInt(signed=False, label="UInt32")), ("m_DeviceType", SimTypeChar(label="Byte")), ("m_DeviceTypeModifier", SimTypeChar(label="Byte")), ("m_bCommandQueueing", SimTypeInt(signed=True, label="Int32")), ("m_BusType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("m_szVendorId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("m_szProductId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("m_szProductRevision", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("m_szSerialNumber", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("m_diskSignature", SimTypeBottom(label="Guid")), ("m_deviceIdDescriptor", SimTypeRef("VDS_STORAGE_DEVICE_ID_DESCRIPTOR", SimStruct)), ("m_cInterconnects", SimTypeInt(signed=False, label="UInt32")), ("m_rgInterconnects", SimTypePointer(SimTypeRef("VDS_INTERCONNECT", SimStruct), offset=0)),)), name="VDS_LUN_INFORMATION", pack=False, align=None),
    "VDS_LUN_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_LUN")), ("LunId", SimTypeBottom(label="Guid")),)), name="VDS_LUN_NOTIFICATION", pack=False, align=None),
    "VDS_LUN_PLEX_FLAG": SimTypeInt(signed=False, label="VDS_LUN_PLEX_FLAG"),
    "VDS_LUN_PLEX_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("type", SimTypeInt(signed=False, label="VDS_LUN_PLEX_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_LUN_PLEX_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("TransitionState", SimTypeInt(signed=False, label="VDS_TRANSITION_STATE")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSize", SimTypeInt(signed=False, label="UInt32")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")),)), name="VDS_LUN_PLEX_PROP", pack=False, align=None),
    "VDS_LUN_PLEX_STATUS": SimTypeInt(signed=False, label="VDS_LUN_PLEX_STATUS"),
    "VDS_LUN_PLEX_TYPE": SimTypeInt(signed=False, label="VDS_LUN_PLEX_TYPE"),
    "VDS_LUN_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszUnmaskingList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="VDS_LUN_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_LUN_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("TransitionState", SimTypeInt(signed=False, label="VDS_TRANSITION_STATE")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")),)), name="VDS_LUN_PROP", pack=False, align=None),
    "VDS_LUN_RESERVE_MODE": SimTypeInt(signed=False, label="VDS_LUN_RESERVE_MODE"),
    "VDS_LUN_STATUS": SimTypeInt(signed=False, label="VDS_LUN_STATUS"),
    "VDS_LUN_TYPE": SimTypeInt(signed=False, label="VDS_LUN_TYPE"),
    "VDS_MAINTENANCE_OPERATION": SimTypeInt(signed=False, label="VDS_MAINTENANCE_OPERATION"),
    "VDS_MOUNT_POINT_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("volumeId", SimTypeBottom(label="Guid")),)), name="VDS_MOUNT_POINT_NOTIFICATION", pack=False, align=None),
    "VDS_NF_CONTROLLER": SimTypeInt(signed=False, label="VDS_NF_CONTROLLER"),
    "VDS_NF_DISK": SimTypeInt(signed=False, label="VDS_NF_DISK"),
    "VDS_NF_DRIVE": SimTypeInt(signed=False, label="VDS_NF_DRIVE"),
    "VDS_NF_FILE_SYSTEM": SimTypeInt(signed=False, label="VDS_NF_FILE_SYSTEM"),
    "VDS_NF_LUN": SimTypeInt(signed=False, label="VDS_NF_LUN"),
    "VDS_NF_PACK": SimTypeInt(signed=False, label="VDS_NF_PACK"),
    "VDS_NF_PORT": SimTypeInt(signed=False, label="VDS_NF_PORT"),
    "VDS_NOTIFICATION": SimStruct(OrderedDict((("objectType", SimTypeInt(signed=False, label="VDS_NOTIFICATION_TARGET_TYPE")), ("Anonymous", SimUnion({"Pack": SimTypeRef("VDS_PACK_NOTIFICATION", SimStruct), "Disk": SimTypeRef("VDS_DISK_NOTIFICATION", SimStruct), "Volume": SimTypeRef("VDS_VOLUME_NOTIFICATION", SimStruct), "Partition": SimTypeRef("VDS_PARTITION_NOTIFICATION", SimStruct), "Letter": SimTypeRef("VDS_DRIVE_LETTER_NOTIFICATION", SimStruct), "FileSystem": SimTypeRef("VDS_FILE_SYSTEM_NOTIFICATION", SimStruct), "MountPoint": SimTypeRef("VDS_MOUNT_POINT_NOTIFICATION", SimStruct), "SubSystem": SimTypeRef("VDS_SUB_SYSTEM_NOTIFICATION", SimStruct), "Controller": SimTypeRef("VDS_CONTROLLER_NOTIFICATION", SimStruct), "Drive": SimTypeRef("VDS_DRIVE_NOTIFICATION", SimStruct), "Lun": SimTypeRef("VDS_LUN_NOTIFICATION", SimStruct), "Port": SimTypeRef("VDS_PORT_NOTIFICATION", SimStruct), "Portal": SimTypeRef("VDS_PORTAL_NOTIFICATION", SimStruct), "Target": SimTypeRef("VDS_TARGET_NOTIFICATION", SimStruct), "PortalGroup": SimTypeRef("VDS_PORTAL_GROUP_NOTIFICATION", SimStruct), "Service": SimTypeRef("VDS_SERVICE_NOTIFICATION", SimStruct)}, name="<anon>", label="None")),)), name="VDS_NOTIFICATION", pack=False, align=None),
    "VDS_NOTIFICATION_TARGET_TYPE": SimTypeInt(signed=False, label="VDS_NOTIFICATION_TARGET_TYPE"),
    "VDS_OBJECT_TYPE": SimTypeInt(signed=False, label="VDS_OBJECT_TYPE"),
    "VDS_PACK_FLAG": SimTypeInt(signed=False, label="VDS_PACK_FLAG"),
    "VDS_PACK_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_PACK")), ("packId", SimTypeBottom(label="Guid")),)), name="VDS_PACK_NOTIFICATION", pack=False, align=None),
    "VDS_PACK_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("status", SimTypeInt(signed=False, label="VDS_PACK_STATUS")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_PACK_PROP", pack=False, align=None),
    "VDS_PACK_STATUS": SimTypeInt(signed=False, label="VDS_PACK_STATUS"),
    "VDS_PARTITION_FLAG": SimTypeInt(signed=False, label="VDS_PARTITION_FLAG"),
    "VDS_PARTITION_INFORMATION_EX": SimStruct(OrderedDict((("dwPartitionStyle", SimTypeInt(signed=False, label="__VDS_PARTITION_STYLE")), ("ullStartingOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ullPartitionLength", SimTypeLongLong(signed=False, label="UInt64")), ("dwPartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("bRewritePartition", SimTypeChar(label="Byte")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("VDS_PARTITION_INFO_MBR", SimStruct), "Gpt": SimTypeRef("VDS_PARTITION_INFO_GPT", SimStruct)}, name="<anon>", label="None")),)), name="VDS_PARTITION_INFORMATION_EX", pack=False, align=None),
    "VDS_PARTITION_INFO_GPT": SimStruct(OrderedDict((("partitionType", SimTypeBottom(label="Guid")), ("partitionId", SimTypeBottom(label="Guid")), ("attributes", SimTypeLongLong(signed=False, label="UInt64")), ("name", SimTypeArray(SimTypeChar(label="Char"), 36)),)), name="VDS_PARTITION_INFO_GPT", pack=False, align=None),
    "VDS_PARTITION_INFO_MBR": SimStruct(OrderedDict((("partitionType", SimTypeChar(label="Byte")), ("bootIndicator", SimTypeChar(label="Byte")), ("recognizedPartition", SimTypeChar(label="Byte")), ("hiddenSectors", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_PARTITION_INFO_MBR", pack=False, align=None),
    "VDS_PARTITION_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("diskId", SimTypeBottom(label="Guid")), ("ullOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="VDS_PARTITION_NOTIFICATION", pack=False, align=None),
    "VDS_PARTITION_PROP": SimStruct(OrderedDict((("PartitionStyle", SimTypeInt(signed=False, label="VDS_PARTITION_STYLE")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulPartitionNumber", SimTypeInt(signed=False, label="UInt32")), ("ullOffset", SimTypeLongLong(signed=False, label="UInt64")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("Anonymous", SimUnion({"Mbr": SimTypeRef("VDS_PARTITION_INFO_MBR", SimStruct), "Gpt": SimTypeRef("VDS_PARTITION_INFO_GPT", SimStruct)}, name="<anon>", label="None")),)), name="VDS_PARTITION_PROP", pack=False, align=None),
    "VDS_PARTITION_STYLE": SimTypeInt(signed=False, label="VDS_PARTITION_STYLE"),
    "VDS_PATH_ID": SimStruct(OrderedDict((("ullSourceId", SimTypeLongLong(signed=False, label="UInt64")), ("ullPathId", SimTypeLongLong(signed=False, label="UInt64")),)), name="VDS_PATH_ID", pack=False, align=None),
    "VDS_PATH_INFO": SimStruct(OrderedDict((("pathId", SimTypeRef("VDS_PATH_ID", SimStruct)), ("type", SimTypeInt(signed=False, label="VDS_HWPROVIDER_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_PATH_STATUS")), ("Anonymous1", SimUnion({"controllerPortId": SimTypeBottom(label="Guid"), "targetPortalId": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"hbaPortId": SimTypeBottom(label="Guid"), "initiatorAdapterId": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("Anonymous3", SimUnion({"pHbaPortProp": SimTypePointer(SimTypeRef("VDS_HBAPORT_PROP", SimStruct), offset=0), "pInitiatorPortalIpAddr": SimTypePointer(SimTypeRef("VDS_IPADDRESS", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="VDS_PATH_INFO", pack=False, align=None),
    "VDS_PATH_POLICY": SimStruct(OrderedDict((("pathId", SimTypeRef("VDS_PATH_ID", SimStruct)), ("bPrimaryPath", SimTypeInt(signed=True, label="Int32")), ("ulWeight", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_PATH_POLICY", pack=False, align=None),
    "VDS_PATH_STATUS": SimTypeInt(signed=False, label="VDS_PATH_STATUS"),
    "VDS_POOL_ATTRIBUTES": SimStruct(OrderedDict((("ullAttributeMask", SimTypeLongLong(signed=False, label="UInt64")), ("raidType", SimTypeInt(signed=False, label="VDS_RAID_TYPE")), ("busType", SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE")), ("pwszIntendedUsage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bSpinDown", SimTypeInt(signed=True, label="Int32")), ("bIsThinProvisioned", SimTypeInt(signed=True, label="Int32")), ("ullProvisionedSpace", SimTypeLongLong(signed=False, label="UInt64")), ("bNoSinglePointOfFailure", SimTypeInt(signed=True, label="Int32")), ("ulDataRedundancyMax", SimTypeInt(signed=False, label="UInt32")), ("ulDataRedundancyMin", SimTypeInt(signed=False, label="UInt32")), ("ulDataRedundancyDefault", SimTypeInt(signed=False, label="UInt32")), ("ulPackageRedundancyMax", SimTypeInt(signed=False, label="UInt32")), ("ulPackageRedundancyMin", SimTypeInt(signed=False, label="UInt32")), ("ulPackageRedundancyDefault", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSize", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSizeMax", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSizeMin", SimTypeInt(signed=False, label="UInt32")), ("ulDefaultStripeSize", SimTypeInt(signed=False, label="UInt32")), ("ulNumberOfColumns", SimTypeInt(signed=False, label="UInt32")), ("ulNumberOfColumnsMax", SimTypeInt(signed=False, label="UInt32")), ("ulNumberOfColumnsMin", SimTypeInt(signed=False, label="UInt32")), ("ulDefaultNumberofColumns", SimTypeInt(signed=False, label="UInt32")), ("ulDataAvailabilityHint", SimTypeInt(signed=False, label="UInt32")), ("ulAccessRandomnessHint", SimTypeInt(signed=False, label="UInt32")), ("ulAccessDirectionHint", SimTypeInt(signed=False, label="UInt32")), ("ulAccessSizeHint", SimTypeInt(signed=False, label="UInt32")), ("ulAccessLatencyHint", SimTypeInt(signed=False, label="UInt32")), ("ulAccessBandwidthWeightHint", SimTypeInt(signed=False, label="UInt32")), ("ulStorageCostHint", SimTypeInt(signed=False, label="UInt32")), ("ulStorageEfficiencyHint", SimTypeInt(signed=False, label="UInt32")), ("ulNumOfCustomAttributes", SimTypeInt(signed=False, label="UInt32")), ("pPoolCustomAttributes", SimTypePointer(SimTypeRef("VDS_POOL_CUSTOM_ATTRIBUTES", SimStruct), offset=0)), ("bReserved1", SimTypeInt(signed=True, label="Int32")), ("bReserved2", SimTypeInt(signed=True, label="Int32")), ("ulReserved1", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeInt(signed=False, label="UInt32")), ("ullReserved1", SimTypeLongLong(signed=False, label="UInt64")), ("ullReserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="VDS_POOL_ATTRIBUTES", pack=False, align=None),
    "VDS_POOL_CUSTOM_ATTRIBUTES": SimStruct(OrderedDict((("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_POOL_CUSTOM_ATTRIBUTES", pack=False, align=None),
    "VDS_PORTAL_GROUP_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("portalGroupId", SimTypeBottom(label="Guid")),)), name="VDS_PORTAL_GROUP_NOTIFICATION", pack=False, align=None),
    "VDS_PORTAL_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("portalId", SimTypeBottom(label="Guid")),)), name="VDS_PORTAL_NOTIFICATION", pack=False, align=None),
    "VDS_PORT_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="VDS_NF_PORT")), ("portId", SimTypeBottom(label="Guid")),)), name="VDS_PORT_NOTIFICATION", pack=False, align=None),
    "VDS_PORT_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("status", SimTypeInt(signed=False, label="VDS_PORT_STATUS")),)), name="VDS_PORT_PROP", pack=False, align=None),
    "VDS_PORT_STATUS": SimTypeInt(signed=False, label="VDS_PORT_STATUS"),
    "VDS_PROVIDER_FLAG": SimTypeInt(signed=False, label="VDS_PROVIDER_FLAG"),
    "VDS_PROVIDER_LBSUPPORT_FLAG": SimTypeInt(signed=False, label="VDS_PROVIDER_LBSUPPORT_FLAG"),
    "VDS_PROVIDER_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("guidVersionId", SimTypeBottom(label="Guid")), ("pwszVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("type", SimTypeInt(signed=False, label="VDS_PROVIDER_TYPE")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSizeFlags", SimTypeInt(signed=False, label="UInt32")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")),)), name="VDS_PROVIDER_PROP", pack=False, align=None),
    "VDS_PROVIDER_TYPE": SimTypeInt(signed=False, label="VDS_PROVIDER_TYPE"),
    "VDS_QUERY_PROVIDER_FLAG": SimTypeInt(signed=False, label="VDS_QUERY_PROVIDER_FLAG"),
    "VDS_RAID_TYPE": SimTypeInt(signed=False, label="VDS_RAID_TYPE"),
    "VDS_RECOVER_ACTION": SimTypeInt(signed=False, label="VDS_RECOVER_ACTION"),
    "VDS_REPARSE_POINT_PROP": SimStruct(OrderedDict((("SourceVolumeId", SimTypeBottom(label="Guid")), ("pwszPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_REPARSE_POINT_PROP", pack=False, align=None),
    "VDS_SAN_POLICY": SimTypeInt(signed=False, label="VDS_SAN_POLICY"),
    "VDS_SERVICE_FLAG": SimTypeInt(signed=False, label="VDS_SERVICE_FLAG"),
    "VDS_SERVICE_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("action", SimTypeInt(signed=False, label="VDS_RECOVER_ACTION")),)), name="VDS_SERVICE_NOTIFICATION", pack=False, align=None),
    "VDS_SERVICE_PROP": SimStruct(OrderedDict((("pwszVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_SERVICE_PROP", pack=False, align=None),
    "VDS_STORAGE_BUS_TYPE": SimTypeInt(signed=False, label="VDS_STORAGE_BUS_TYPE"),
    "VDS_STORAGE_DEVICE_ID_DESCRIPTOR": SimStruct(OrderedDict((("m_version", SimTypeInt(signed=False, label="UInt32")), ("m_cIdentifiers", SimTypeInt(signed=False, label="UInt32")), ("m_rgIdentifiers", SimTypePointer(SimTypeRef("VDS_STORAGE_IDENTIFIER", SimStruct), offset=0)),)), name="VDS_STORAGE_DEVICE_ID_DESCRIPTOR", pack=False, align=None),
    "VDS_STORAGE_IDENTIFIER": SimStruct(OrderedDict((("m_CodeSet", SimTypeInt(signed=False, label="VDS_STORAGE_IDENTIFIER_CODE_SET")), ("m_Type", SimTypeInt(signed=False, label="VDS_STORAGE_IDENTIFIER_TYPE")), ("m_cbIdentifier", SimTypeInt(signed=False, label="UInt32")), ("m_rgbIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VDS_STORAGE_IDENTIFIER", pack=False, align=None),
    "VDS_STORAGE_IDENTIFIER_CODE_SET": SimTypeInt(signed=False, label="VDS_STORAGE_IDENTIFIER_CODE_SET"),
    "VDS_STORAGE_IDENTIFIER_TYPE": SimTypeInt(signed=False, label="VDS_STORAGE_IDENTIFIER_TYPE"),
    "VDS_STORAGE_POOL_DRIVE_EXTENT": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("bUsed", SimTypeInt(signed=True, label="Int32")),)), name="VDS_STORAGE_POOL_DRIVE_EXTENT", pack=False, align=None),
    "VDS_STORAGE_POOL_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("status", SimTypeInt(signed=False, label="VDS_STORAGE_POOL_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("type", SimTypeInt(signed=False, label="VDS_STORAGE_POOL_TYPE")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ullTotalConsumedSpace", SimTypeLongLong(signed=False, label="UInt64")), ("ullTotalManagedSpace", SimTypeLongLong(signed=False, label="UInt64")), ("ullRemainingFreeSpace", SimTypeLongLong(signed=False, label="UInt64")),)), name="VDS_STORAGE_POOL_PROP", pack=False, align=None),
    "VDS_STORAGE_POOL_STATUS": SimTypeInt(signed=False, label="VDS_STORAGE_POOL_STATUS"),
    "VDS_STORAGE_POOL_TYPE": SimTypeInt(signed=False, label="VDS_STORAGE_POOL_TYPE"),
    "VDS_SUB_SYSTEM_FLAG": SimTypeInt(signed=False, label="VDS_SUB_SYSTEM_FLAG"),
    "VDS_SUB_SYSTEM_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("subSystemId", SimTypeBottom(label="Guid")),)), name="VDS_SUB_SYSTEM_NOTIFICATION", pack=False, align=None),
    "VDS_SUB_SYSTEM_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSizeFlags", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="VDS_SUB_SYSTEM_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("sNumberOfInternalBuses", SimTypeShort(signed=True, label="Int16")), ("sMaxNumberOfSlotsEachBus", SimTypeShort(signed=True, label="Int16")), ("sMaxNumberOfControllers", SimTypeShort(signed=True, label="Int16")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")),)), name="VDS_SUB_SYSTEM_PROP", pack=False, align=None),
    "VDS_SUB_SYSTEM_PROP2": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("pwszFriendlyName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIdentification", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("ulStripeSizeFlags", SimTypeInt(signed=False, label="UInt32")), ("ulSupportedRaidTypeFlags", SimTypeInt(signed=False, label="UInt32")), ("status", SimTypeInt(signed=False, label="VDS_SUB_SYSTEM_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("sNumberOfInternalBuses", SimTypeShort(signed=True, label="Int16")), ("sMaxNumberOfSlotsEachBus", SimTypeShort(signed=True, label="Int16")), ("sMaxNumberOfControllers", SimTypeShort(signed=True, label="Int16")), ("sRebuildPriority", SimTypeShort(signed=True, label="Int16")), ("ulNumberOfEnclosures", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_SUB_SYSTEM_PROP2", pack=False, align=None),
    "VDS_SUB_SYSTEM_STATUS": SimTypeInt(signed=False, label="VDS_SUB_SYSTEM_STATUS"),
    "VDS_SUB_SYSTEM_SUPPORTED_RAID_TYPE_FLAG": SimTypeInt(signed=False, label="VDS_SUB_SYSTEM_SUPPORTED_RAID_TYPE_FLAG"),
    "VDS_TARGET_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("targetId", SimTypeBottom(label="Guid")),)), name="VDS_TARGET_NOTIFICATION", pack=False, align=None),
    "VDS_TRANSITION_STATE": SimTypeInt(signed=False, label="VDS_TRANSITION_STATE"),
    "VDS_VDISK_PROPERTIES": SimStruct(OrderedDict((("Id", SimTypeBottom(label="Guid")), ("State", SimTypeInt(signed=False, label="VDS_VDISK_STATE")), ("VirtualDeviceType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("VirtualSize", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalSize", SimTypeLongLong(signed=False, label="UInt64")), ("pPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDeviceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("DiskFlag", SimTypeInt(signed=False, label="DEPENDENT_DISK_FLAG")), ("bIsChild", SimTypeInt(signed=True, label="Int32")), ("pParentPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_VDISK_PROPERTIES", pack=False, align=None),
    "VDS_VDISK_STATE": SimTypeInt(signed=False, label="VDS_VDISK_STATE"),
    "VDS_VERSION_SUPPORT_FLAG": SimTypeInt(signed=False, label="VDS_VERSION_SUPPORT_FLAG"),
    "VDS_VOLUME_FLAG": SimTypeInt(signed=False, label="VDS_VOLUME_FLAG"),
    "VDS_VOLUME_NOTIFICATION": SimStruct(OrderedDict((("ulEvent", SimTypeInt(signed=False, label="UInt32")), ("volumeId", SimTypeBottom(label="Guid")), ("plexId", SimTypeBottom(label="Guid")), ("ulPercentCompleted", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_VOLUME_NOTIFICATION", pack=False, align=None),
    "VDS_VOLUME_PLEX_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("type", SimTypeInt(signed=False, label="VDS_VOLUME_PLEX_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_VOLUME_PLEX_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("TransitionState", SimTypeInt(signed=False, label="VDS_TRANSITION_STATE")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulStripeSize", SimTypeInt(signed=False, label="UInt32")), ("ulNumberOfMembers", SimTypeInt(signed=False, label="UInt32")),)), name="VDS_VOLUME_PLEX_PROP", pack=False, align=None),
    "VDS_VOLUME_PLEX_STATUS": SimTypeInt(signed=False, label="VDS_VOLUME_PLEX_STATUS"),
    "VDS_VOLUME_PLEX_TYPE": SimTypeInt(signed=False, label="VDS_VOLUME_PLEX_TYPE"),
    "VDS_VOLUME_PROP": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("type", SimTypeInt(signed=False, label="VDS_VOLUME_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_VOLUME_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("TransitionState", SimTypeInt(signed=False, label="VDS_TRANSITION_STATE")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("RecommendedFileSystemType", SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_TYPE")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VDS_VOLUME_PROP", pack=False, align=None),
    "VDS_VOLUME_PROP2": SimStruct(OrderedDict((("id", SimTypeBottom(label="Guid")), ("type", SimTypeInt(signed=False, label="VDS_VOLUME_TYPE")), ("status", SimTypeInt(signed=False, label="VDS_VOLUME_STATUS")), ("health", SimTypeInt(signed=False, label="VDS_HEALTH")), ("TransitionState", SimTypeInt(signed=False, label="VDS_TRANSITION_STATE")), ("ullSize", SimTypeLongLong(signed=False, label="UInt64")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("RecommendedFileSystemType", SimTypeInt(signed=False, label="VDS_FILE_SYSTEM_TYPE")), ("cbUniqueId", SimTypeInt(signed=False, label="UInt32")), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUniqueId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VDS_VOLUME_PROP2", pack=False, align=None),
    "VDS_VOLUME_STATUS": SimTypeInt(signed=False, label="VDS_VOLUME_STATUS"),
    "VDS_VOLUME_TYPE": SimTypeInt(signed=False, label="VDS_VOLUME_TYPE"),
    "VDS_WWN": SimStruct(OrderedDict((("rguchWwn", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="VDS_WWN", pack=False, align=None),
    "VEC3D": SimStruct(OrderedDict((("X", SimTypeFloat(size=32)), ("Y", SimTypeFloat(size=32)), ("Z", SimTypeFloat(size=32)),)), name="VEC3D", pack=False, align=None),
    "VECTORRESTRICTION": SimStruct(OrderedDict((("Node", SimTypeRef("NODERESTRICTION", SimStruct)), ("RankMethod", SimTypeInt(signed=False, label="UInt32")),)), name="VECTORRESTRICTION", pack=False, align=None),
    "VENDORPROPERTIES": SimTypeInt(signed=False, label="VENDORPROPERTIES"),
    "VERIFIER_ENUM_RESOURCE_FLAGS": SimTypeInt(signed=False, label="VERIFIER_ENUM_RESOURCE_FLAGS"),
    "VERIFYSERVERCERT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("LDAP", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)], SimTypeChar(label="Byte"), arg_names=["Connection", "pServerCert"]), offset=0),
    "VERIFY_INFORMATION": SimStruct(OrderedDict((("StartingOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="VERIFY_INFORMATION", pack=False, align=None),
    "VERIFY_SIGNATURE_FN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SecHandle", SimStruct), offset=0), SimTypePointer(SimTypeRef("SecBufferDesc", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "VERSIONEDSTREAM": SimStruct(OrderedDict((("guidVersion", SimTypeBottom(label="Guid")), ("pStream", SimTypeBottom(label="IStream")),)), name="VERSIONEDSTREAM", pack=False, align=None),
    "VERTSCROLLSTATES": SimTypeInt(signed=False, label="VERTSCROLLSTATES"),
    "VERTTHUMBSTATES": SimTypeInt(signed=False, label="VERTTHUMBSTATES"),
    "VER_FIND_FILE_FLAGS": SimTypeInt(signed=False, label="VER_FIND_FILE_FLAGS"),
    "VER_FIND_FILE_STATUS": SimTypeInt(signed=False, label="VER_FIND_FILE_STATUS"),
    "VER_FLAGS": SimTypeInt(signed=False, label="VER_FLAGS"),
    "VER_INSTALL_FILE_FLAGS": SimTypeInt(signed=False, label="VER_INSTALL_FILE_FLAGS"),
    "VER_INSTALL_FILE_STATUS": SimTypeInt(signed=False, label="VER_INSTALL_FILE_STATUS"),
    "VER_PLATFORM": SimTypeInt(signed=False, label="VER_PLATFORM"),
    "VFWWDMExtensionProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pfnDeviceIoControl", "pfnAddPropertyPage", "lParam"]), offset=0),
    "VFW_FILTERLIST": SimStruct(OrderedDict((("cFilters", SimTypeInt(signed=False, label="UInt32")), ("aClsId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="VFW_FILTERLIST", pack=False, align=None),
    "VGA_CHAR": SimStruct(OrderedDict((("Char", SimTypeChar(label="SByte")), ("Attributes", SimTypeChar(label="SByte")),)), name="VGA_CHAR", pack=False, align=None),
    "VHF_CONFIG": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("VhfClientContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OperationContextSize", SimTypeInt(signed=False, label="UInt32")), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("VendorID", SimTypeShort(signed=False, label="UInt16")), ("ProductID", SimTypeShort(signed=False, label="UInt16")), ("VersionNumber", SimTypeShort(signed=False, label="UInt16")), ("ContainerID", SimTypeBottom(label="Guid")), ("InstanceIDLength", SimTypeShort(signed=False, label="UInt16")), ("InstanceID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReportDescriptorLength", SimTypeShort(signed=False, label="UInt16")), ("ReportDescriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("EvtVhfReadyForNextReadReport", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("EvtVhfAsyncOperationGetFeature", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("EvtVhfAsyncOperationSetFeature", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("EvtVhfAsyncOperationWriteReport", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("EvtVhfAsyncOperationGetInputReport", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("EvtVhfCleanup", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("HardwareIDsLength", SimTypeShort(signed=False, label="UInt16")), ("HardwareIDs", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VHF_CONFIG", pack=False, align=None),
    "VIDEOENCODER_BITRATE_MODE": SimTypeInt(signed=False, label="VIDEOENCODER_BITRATE_MODE"),
    "VIDEOHDR": SimStruct(OrderedDict((("lpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwBufferLength", SimTypeInt(signed=False, label="UInt32")), ("dwBytesUsed", SimTypeInt(signed=False, label="UInt32")), ("dwTimeCaptured", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 4)),)), name="VIDEOHDR", pack=False, align=None),
    "VIDEOINFO": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("bmiHeader", SimTypeRef("BITMAPINFOHEADER", SimStruct)), ("Anonymous", SimUnion({"bmiColors": SimTypeArray(SimTypeRef("RGBQUAD", SimStruct), 256), "dwBitMasks": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3), "TrueColorInfo": SimTypeRef("TRUECOLORINFO", SimStruct)}, name="<anon>", label="None")),)), name="VIDEOINFO", pack=False, align=None),
    "VIDEOINFOHEADER": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("bmiHeader", SimTypeRef("BITMAPINFOHEADER", SimStruct)),)), name="VIDEOINFOHEADER", pack=False, align=None),
    "VIDEOINFOHEADER2": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("dwInterlaceFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCopyProtectFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioX", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioY", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"dwControlFlags": SimTypeInt(signed=False, label="UInt32"), "dwReserved1": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("bmiHeader", SimTypeRef("BITMAPINFOHEADER", SimStruct)),)), name="VIDEOINFOHEADER2", pack=False, align=None),
    "VIDEOMEMORY": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fpStart", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous1", SimUnion({"fpEnd": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwWidth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("ddsCapsAlt", SimTypeRef("DDSCAPS", SimStruct)), ("Anonymous2", SimUnion({"lpHeap": SimTypePointer(SimTypeRef("VMEMHEAP", SimStruct), offset=0), "dwHeight": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="VIDEOMEMORY", pack=False, align=None),
    "VIDEOMEMORYINFO": SimStruct(OrderedDict((("fpPrimary", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayWidth", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayHeight", SimTypeInt(signed=False, label="UInt32")), ("lDisplayPitch", SimTypeInt(signed=True, label="Int32")), ("ddpfDisplay", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwOffscreenAlign", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayAlign", SimTypeInt(signed=False, label="UInt32")), ("dwTextureAlign", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferAlign", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaAlign", SimTypeInt(signed=False, label="UInt32")), ("pvPrimary", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="VIDEOMEMORYINFO", pack=False, align=None),
    "VIDEOPARAMETERS": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwCommand", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwMode", SimTypeInt(signed=False, label="UInt32")), ("dwTVStandard", SimTypeInt(signed=False, label="UInt32")), ("dwAvailableModes", SimTypeInt(signed=False, label="UInt32")), ("dwAvailableTVStandard", SimTypeInt(signed=False, label="UInt32")), ("dwFlickerFilter", SimTypeInt(signed=False, label="UInt32")), ("dwOverScanX", SimTypeInt(signed=False, label="UInt32")), ("dwOverScanY", SimTypeInt(signed=False, label="UInt32")), ("dwMaxUnscaledX", SimTypeInt(signed=False, label="UInt32")), ("dwMaxUnscaledY", SimTypeInt(signed=False, label="UInt32")), ("dwPositionX", SimTypeInt(signed=False, label="UInt32")), ("dwPositionY", SimTypeInt(signed=False, label="UInt32")), ("dwBrightness", SimTypeInt(signed=False, label="UInt32")), ("dwContrast", SimTypeInt(signed=False, label="UInt32")), ("dwCPType", SimTypeInt(signed=False, label="UInt32")), ("dwCPCommand", SimTypeInt(signed=False, label="UInt32")), ("dwCPStandard", SimTypeInt(signed=False, label="UInt32")), ("dwCPKey", SimTypeInt(signed=False, label="UInt32")), ("bCP_APSTriggerBits", SimTypeInt(signed=False, label="UInt32")), ("bOEMCopyProtection", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="VIDEOPARAMETERS", pack=False, align=None),
    "VIDEO_BANK_SELECT": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("BankingFlags", SimTypeInt(signed=False, label="UInt32")), ("BankingType", SimTypeInt(signed=False, label="UInt32")), ("PlanarHCBankingType", SimTypeInt(signed=False, label="UInt32")), ("BitmapWidthInBytes", SimTypeInt(signed=False, label="UInt32")), ("BitmapSize", SimTypeInt(signed=False, label="UInt32")), ("Granularity", SimTypeInt(signed=False, label="UInt32")), ("PlanarHCGranularity", SimTypeInt(signed=False, label="UInt32")), ("CodeOffset", SimTypeInt(signed=False, label="UInt32")), ("PlanarHCBankCodeOffset", SimTypeInt(signed=False, label="UInt32")), ("PlanarHCEnableCodeOffset", SimTypeInt(signed=False, label="UInt32")), ("PlanarHCDisableCodeOffset", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_BANK_SELECT", pack=False, align=None),
    "VIDEO_BANK_TYPE": SimTypeInt(signed=False, label="VIDEO_BANK_TYPE"),
    "VIDEO_BRIGHTNESS_POLICY": SimStruct(OrderedDict((("DefaultToBiosPolicy", SimTypeChar(label="Byte")), ("LevelCount", SimTypeChar(label="Byte")), ("Level", SimTypePointer(SimStruct(OrderedDict((("BatteryLevel", SimTypeChar(label="Byte")), ("Brightness", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="VIDEO_BRIGHTNESS_POLICY", pack=False, align=None),
    "VIDEO_CLUT": SimStruct(OrderedDict((("NumEntries", SimTypeShort(signed=False, label="UInt16")), ("FirstEntry", SimTypeShort(signed=False, label="UInt16")), ("LookupTable", SimTypePointer(SimUnion({"RgbArray": SimTypeRef("VIDEO_CLUTDATA", SimStruct), "RgbLong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), offset=0)),)), name="VIDEO_CLUT", pack=False, align=None),
    "VIDEO_CLUTDATA": SimStruct(OrderedDict((("Red", SimTypeChar(label="Byte")), ("Green", SimTypeChar(label="Byte")), ("Blue", SimTypeChar(label="Byte")), ("Unused", SimTypeChar(label="Byte")),)), name="VIDEO_CLUTDATA", pack=False, align=None),
    "VIDEO_COLOR_CAPABILITIES": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("AttributeFlags", SimTypeInt(signed=False, label="UInt32")), ("RedPhosphoreDecay", SimTypeInt(signed=True, label="Int32")), ("GreenPhosphoreDecay", SimTypeInt(signed=True, label="Int32")), ("BluePhosphoreDecay", SimTypeInt(signed=True, label="Int32")), ("WhiteChromaticity_x", SimTypeInt(signed=True, label="Int32")), ("WhiteChromaticity_y", SimTypeInt(signed=True, label="Int32")), ("WhiteChromaticity_Y", SimTypeInt(signed=True, label="Int32")), ("RedChromaticity_x", SimTypeInt(signed=True, label="Int32")), ("RedChromaticity_y", SimTypeInt(signed=True, label="Int32")), ("GreenChromaticity_x", SimTypeInt(signed=True, label="Int32")), ("GreenChromaticity_y", SimTypeInt(signed=True, label="Int32")), ("BlueChromaticity_x", SimTypeInt(signed=True, label="Int32")), ("BlueChromaticity_y", SimTypeInt(signed=True, label="Int32")), ("WhiteGamma", SimTypeInt(signed=True, label="Int32")), ("RedGamma", SimTypeInt(signed=True, label="Int32")), ("GreenGamma", SimTypeInt(signed=True, label="Int32")), ("BlueGamma", SimTypeInt(signed=True, label="Int32")),)), name="VIDEO_COLOR_CAPABILITIES", pack=False, align=None),
    "VIDEO_COLOR_LUT_DATA": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("LutDataFormat", SimTypeInt(signed=False, label="UInt32")), ("LutData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VIDEO_COLOR_LUT_DATA", pack=False, align=None),
    "VIDEO_CURSOR_ATTRIBUTES": SimStruct(OrderedDict((("Width", SimTypeShort(signed=False, label="UInt16")), ("Height", SimTypeShort(signed=False, label="UInt16")), ("Column", SimTypeShort(signed=True, label="Int16")), ("Row", SimTypeShort(signed=True, label="Int16")), ("Rate", SimTypeChar(label="Byte")), ("Enable", SimTypeChar(label="Byte")),)), name="VIDEO_CURSOR_ATTRIBUTES", pack=False, align=None),
    "VIDEO_CURSOR_POSITION": SimStruct(OrderedDict((("Column", SimTypeShort(signed=True, label="Int16")), ("Row", SimTypeShort(signed=True, label="Int16")),)), name="VIDEO_CURSOR_POSITION", pack=False, align=None),
    "VIDEO_DEVICE_SESSION_STATUS": SimStruct(OrderedDict((("bEnable", SimTypeInt(signed=False, label="UInt32")), ("bSuccess", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_DEVICE_SESSION_STATUS", pack=False, align=None),
    "VIDEO_HARDWARE_STATE": SimStruct(OrderedDict((("StateHeader", SimTypePointer(SimTypeRef("VIDEO_HARDWARE_STATE_HEADER", SimStruct), offset=0)), ("StateLength", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_HARDWARE_STATE", pack=False, align=None),
    "VIDEO_HARDWARE_STATE_HEADER": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("PortValue", SimTypeArray(SimTypeChar(label="Byte"), 48)), ("AttribIndexDataState", SimTypeInt(signed=False, label="UInt32")), ("BasicSequencerOffset", SimTypeInt(signed=False, label="UInt32")), ("BasicCrtContOffset", SimTypeInt(signed=False, label="UInt32")), ("BasicGraphContOffset", SimTypeInt(signed=False, label="UInt32")), ("BasicAttribContOffset", SimTypeInt(signed=False, label="UInt32")), ("BasicDacOffset", SimTypeInt(signed=False, label="UInt32")), ("BasicLatchesOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedSequencerOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedCrtContOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedGraphContOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedAttribContOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedDacOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedValidatorStateOffset", SimTypeInt(signed=False, label="UInt32")), ("ExtendedMiscDataOffset", SimTypeInt(signed=False, label="UInt32")), ("PlaneLength", SimTypeInt(signed=False, label="UInt32")), ("Plane1Offset", SimTypeInt(signed=False, label="UInt32")), ("Plane2Offset", SimTypeInt(signed=False, label="UInt32")), ("Plane3Offset", SimTypeInt(signed=False, label="UInt32")), ("Plane4Offset", SimTypeInt(signed=False, label="UInt32")), ("VGAStateFlags", SimTypeInt(signed=False, label="UInt32")), ("DIBOffset", SimTypeInt(signed=False, label="UInt32")), ("DIBBitsPerPixel", SimTypeInt(signed=False, label="UInt32")), ("DIBXResolution", SimTypeInt(signed=False, label="UInt32")), ("DIBYResolution", SimTypeInt(signed=False, label="UInt32")), ("DIBXlatOffset", SimTypeInt(signed=False, label="UInt32")), ("DIBXlatLength", SimTypeInt(signed=False, label="UInt32")), ("VesaInfoOffset", SimTypeInt(signed=False, label="UInt32")), ("FrameBufferData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="VIDEO_HARDWARE_STATE_HEADER", pack=False, align=None),
    "VIDEO_LOAD_FONT_INFORMATION": SimStruct(OrderedDict((("WidthInPixels", SimTypeShort(signed=False, label="UInt16")), ("HeightInPixels", SimTypeShort(signed=False, label="UInt16")), ("FontSize", SimTypeInt(signed=False, label="UInt32")), ("Font", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VIDEO_LOAD_FONT_INFORMATION", pack=False, align=None),
    "VIDEO_LUT_RGB256WORDS": SimStruct(OrderedDict((("Red", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Green", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Blue", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="VIDEO_LUT_RGB256WORDS", pack=False, align=None),
    "VIDEO_MEMORY": SimStruct(OrderedDict((("RequestedVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="VIDEO_MEMORY", pack=False, align=None),
    "VIDEO_MEMORY_INFORMATION": SimStruct(OrderedDict((("VideoRamBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("VideoRamLength", SimTypeInt(signed=False, label="UInt32")), ("FrameBufferBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FrameBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_MEMORY_INFORMATION", pack=False, align=None),
    "VIDEO_MODE": SimStruct(OrderedDict((("RequestedMode", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_MODE", pack=False, align=None),
    "VIDEO_MODE_INFORMATION": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("ModeIndex", SimTypeInt(signed=False, label="UInt32")), ("VisScreenWidth", SimTypeInt(signed=False, label="UInt32")), ("VisScreenHeight", SimTypeInt(signed=False, label="UInt32")), ("ScreenStride", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPlanes", SimTypeInt(signed=False, label="UInt32")), ("BitsPerPlane", SimTypeInt(signed=False, label="UInt32")), ("Frequency", SimTypeInt(signed=False, label="UInt32")), ("XMillimeter", SimTypeInt(signed=False, label="UInt32")), ("YMillimeter", SimTypeInt(signed=False, label="UInt32")), ("NumberRedBits", SimTypeInt(signed=False, label="UInt32")), ("NumberGreenBits", SimTypeInt(signed=False, label="UInt32")), ("NumberBlueBits", SimTypeInt(signed=False, label="UInt32")), ("RedMask", SimTypeInt(signed=False, label="UInt32")), ("GreenMask", SimTypeInt(signed=False, label="UInt32")), ("BlueMask", SimTypeInt(signed=False, label="UInt32")), ("AttributeFlags", SimTypeInt(signed=False, label="UInt32")), ("VideoMemoryBitmapWidth", SimTypeInt(signed=False, label="UInt32")), ("VideoMemoryBitmapHeight", SimTypeInt(signed=False, label="UInt32")), ("DriverSpecificAttributeFlags", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_MODE_INFORMATION", pack=False, align=None),
    "VIDEO_MONITOR_DESCRIPTOR": SimStruct(OrderedDict((("DescriptorSize", SimTypeInt(signed=False, label="UInt32")), ("Descriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VIDEO_MONITOR_DESCRIPTOR", pack=False, align=None),
    "VIDEO_NUM_MODES": SimStruct(OrderedDict((("NumModes", SimTypeInt(signed=False, label="UInt32")), ("ModeInformationLength", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_NUM_MODES", pack=False, align=None),
    "VIDEO_PALETTE_DATA": SimStruct(OrderedDict((("NumEntries", SimTypeShort(signed=False, label="UInt16")), ("FirstEntry", SimTypeShort(signed=False, label="UInt16")), ("Colors", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="VIDEO_PALETTE_DATA", pack=False, align=None),
    "VIDEO_PERFORMANCE_COUNTER": SimStruct(OrderedDict((("NbOfAllocationEvicted", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 10)), ("NbOfAllocationMarked", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 10)), ("NbOfAllocationRestored", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 10)), ("KBytesEvicted", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 10)), ("KBytesMarked", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 10)), ("KBytesRestored", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 10)), ("NbProcessCommited", SimTypeLongLong(signed=False, label="UInt64")), ("NbAllocationCommited", SimTypeLongLong(signed=False, label="UInt64")), ("NbAllocationMarked", SimTypeLongLong(signed=False, label="UInt64")), ("KBytesAllocated", SimTypeLongLong(signed=False, label="UInt64")), ("KBytesAvailable", SimTypeLongLong(signed=False, label="UInt64")), ("KBytesCurMarked", SimTypeLongLong(signed=False, label="UInt64")), ("Reference", SimTypeLongLong(signed=False, label="UInt64")), ("Unreference", SimTypeLongLong(signed=False, label="UInt64")), ("TrueReference", SimTypeLongLong(signed=False, label="UInt64")), ("NbOfPageIn", SimTypeLongLong(signed=False, label="UInt64")), ("KBytesPageIn", SimTypeLongLong(signed=False, label="UInt64")), ("NbOfPageOut", SimTypeLongLong(signed=False, label="UInt64")), ("KBytesPageOut", SimTypeLongLong(signed=False, label="UInt64")), ("NbOfRotateOut", SimTypeLongLong(signed=False, label="UInt64")), ("KBytesRotateOut", SimTypeLongLong(signed=False, label="UInt64")),)), name="VIDEO_PERFORMANCE_COUNTER", pack=False, align=None),
    "VIDEO_POINTER_ATTRIBUTES": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("WidthInBytes", SimTypeInt(signed=False, label="UInt32")), ("Enable", SimTypeInt(signed=False, label="UInt32")), ("Column", SimTypeShort(signed=True, label="Int16")), ("Row", SimTypeShort(signed=True, label="Int16")), ("Pixels", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VIDEO_POINTER_ATTRIBUTES", pack=False, align=None),
    "VIDEO_POINTER_CAPABILITIES": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("MaxWidth", SimTypeInt(signed=False, label="UInt32")), ("MaxHeight", SimTypeInt(signed=False, label="UInt32")), ("HWPtrBitmapStart", SimTypeInt(signed=False, label="UInt32")), ("HWPtrBitmapEnd", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_POINTER_CAPABILITIES", pack=False, align=None),
    "VIDEO_POINTER_POSITION": SimStruct(OrderedDict((("Column", SimTypeShort(signed=True, label="Int16")), ("Row", SimTypeShort(signed=True, label="Int16")),)), name="VIDEO_POINTER_POSITION", pack=False, align=None),
    "VIDEO_POWER_MANAGEMENT": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("DPMSVersion", SimTypeInt(signed=False, label="UInt32")), ("PowerState", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_POWER_MANAGEMENT", pack=False, align=None),
    "VIDEO_POWER_STATE": SimTypeInt(signed=False, label="VIDEO_POWER_STATE"),
    "VIDEO_PUBLIC_ACCESS_RANGES": SimStruct(OrderedDict((("InIoSpace", SimTypeInt(signed=False, label="UInt32")), ("MappedInIoSpace", SimTypeInt(signed=False, label="UInt32")), ("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="VIDEO_PUBLIC_ACCESS_RANGES", pack=False, align=None),
    "VIDEO_QUERY_PERFORMANCE_COUNTER": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeRef("VIDEO_PERFORMANCE_COUNTER", SimStruct), offset=0)),)), name="VIDEO_QUERY_PERFORMANCE_COUNTER", pack=False, align=None),
    "VIDEO_REGISTER_VDM": SimStruct(OrderedDict((("MinimumStateSize", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_REGISTER_VDM", pack=False, align=None),
    "VIDEO_SHARE_MEMORY": SimStruct(OrderedDict((("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ViewOffset", SimTypeInt(signed=False, label="UInt32")), ("ViewSize", SimTypeInt(signed=False, label="UInt32")), ("RequestedVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="VIDEO_SHARE_MEMORY", pack=False, align=None),
    "VIDEO_SHARE_MEMORY_INFORMATION": SimStruct(OrderedDict((("SharedViewOffset", SimTypeInt(signed=False, label="UInt32")), ("SharedViewSize", SimTypeInt(signed=False, label="UInt32")), ("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="VIDEO_SHARE_MEMORY_INFORMATION", pack=False, align=None),
    "VIDEO_STREAM_CONFIG_CAPS": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("VideoStandard", SimTypeInt(signed=False, label="UInt32")), ("InputSize", SimTypeRef("SIZE", SimStruct)), ("MinCroppingSize", SimTypeRef("SIZE", SimStruct)), ("MaxCroppingSize", SimTypeRef("SIZE", SimStruct)), ("CropGranularityX", SimTypeInt(signed=True, label="Int32")), ("CropGranularityY", SimTypeInt(signed=True, label="Int32")), ("CropAlignX", SimTypeInt(signed=True, label="Int32")), ("CropAlignY", SimTypeInt(signed=True, label="Int32")), ("MinOutputSize", SimTypeRef("SIZE", SimStruct)), ("MaxOutputSize", SimTypeRef("SIZE", SimStruct)), ("OutputGranularityX", SimTypeInt(signed=True, label="Int32")), ("OutputGranularityY", SimTypeInt(signed=True, label="Int32")), ("StretchTapsX", SimTypeInt(signed=True, label="Int32")), ("StretchTapsY", SimTypeInt(signed=True, label="Int32")), ("ShrinkTapsX", SimTypeInt(signed=True, label="Int32")), ("ShrinkTapsY", SimTypeInt(signed=True, label="Int32")), ("MinFrameInterval", SimTypeLongLong(signed=True, label="Int64")), ("MaxFrameInterval", SimTypeLongLong(signed=True, label="Int64")), ("MinBitsPerSecond", SimTypeInt(signed=True, label="Int32")), ("MaxBitsPerSecond", SimTypeInt(signed=True, label="Int32")),)), name="VIDEO_STREAM_CONFIG_CAPS", pack=False, align=None),
    "VIDEO_VDM": SimStruct(OrderedDict((("ProcessHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="VIDEO_VDM", pack=False, align=None),
    "VIDEO_WIN32K_CALLBACKS": SimStruct(OrderedDict((("PhysDisp", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Callout", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Params"]), offset=0)), ("bACPI", SimTypeInt(signed=False, label="UInt32")), ("pPhysDeviceObject", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("DualviewFlags", SimTypeInt(signed=False, label="UInt32")),)), name="VIDEO_WIN32K_CALLBACKS", pack=False, align=None),
    "VIDEO_WIN32K_CALLBACKS_PARAMS": SimStruct(OrderedDict((("CalloutType", SimTypeInt(signed=False, label="VIDEO_WIN32K_CALLBACKS_PARAMS_TYPE")), ("PhysDisp", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Param", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Status", SimTypeInt(signed=True, label="Int32")), ("LockUserSession", SimTypeChar(label="Byte")), ("IsPostDevice", SimTypeChar(label="Byte")), ("SurpriseRemoval", SimTypeChar(label="Byte")), ("WaitForQueueReady", SimTypeChar(label="Byte")),)), name="VIDEO_WIN32K_CALLBACKS_PARAMS", pack=False, align=None),
    "VIDEO_WIN32K_CALLBACKS_PARAMS_TYPE": SimTypeInt(signed=False, label="VIDEO_WIN32K_CALLBACKS_PARAMS_TYPE"),
    "VIDMEM": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("fpStart", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Anonymous1", SimUnion({"fpEnd": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwWidth": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ddsCaps", SimTypeRef("DDSCAPS", SimStruct)), ("ddsCapsAlt", SimTypeRef("DDSCAPS", SimStruct)), ("Anonymous2", SimUnion({"lpHeap": SimTypePointer(SimTypeRef("VMEMHEAP", SimStruct), offset=0), "dwHeight": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="VIDMEM", pack=False, align=None),
    "VIDMEMINFO": SimStruct(OrderedDict((("fpPrimary", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayWidth", SimTypeInt(signed=False, label="UInt32")), ("dwDisplayHeight", SimTypeInt(signed=False, label="UInt32")), ("lDisplayPitch", SimTypeInt(signed=True, label="Int32")), ("ddpfDisplay", SimTypeRef("DDPIXELFORMAT", SimStruct)), ("dwOffscreenAlign", SimTypeInt(signed=False, label="UInt32")), ("dwOverlayAlign", SimTypeInt(signed=False, label="UInt32")), ("dwTextureAlign", SimTypeInt(signed=False, label="UInt32")), ("dwZBufferAlign", SimTypeInt(signed=False, label="UInt32")), ("dwAlphaAlign", SimTypeInt(signed=False, label="UInt32")), ("dwNumHeaps", SimTypeInt(signed=False, label="UInt32")), ("pvmList", SimTypePointer(SimTypeRef("VIDMEM", SimStruct), offset=0)),)), name="VIDMEMINFO", pack=False, align=None),
    "VIEWSTATUS": SimTypeInt(signed=False, label="VIEWSTATUS"),
    "VIEW_OBJECT_ALPHA_MODE": SimTypeInt(signed=False, label="VIEW_OBJECT_ALPHA_MODE"),
    "VIEW_OBJECT_COMPOSITION_MODE": SimTypeInt(signed=False, label="VIEW_OBJECT_COMPOSITION_MODE"),
    "VIEW_OBJECT_PROPERTIES_FLAGS": SimTypeInt(signed=False, label="VIEW_OBJECT_PROPERTIES_FLAGS"),
    "VIRTUALIZATION_INSTANCE_INFO_INPUT": SimStruct(OrderedDict((("NumberOfWorkerThreads", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="VIRTUALIZATION_INSTANCE_INFO_INPUT", pack=False, align=None),
    "VIRTUALIZATION_INSTANCE_INFO_INPUT_EX": SimStruct(OrderedDict((("HeaderSize", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("NotificationInfoSize", SimTypeInt(signed=False, label="UInt32")), ("NotificationInfoOffset", SimTypeShort(signed=False, label="UInt16")), ("ProviderMajorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="VIRTUALIZATION_INSTANCE_INFO_INPUT_EX", pack=False, align=None),
    "VIRTUALIZATION_INSTANCE_INFO_OUTPUT": SimStruct(OrderedDict((("VirtualizationInstanceID", SimTypeBottom(label="Guid")),)), name="VIRTUALIZATION_INSTANCE_INFO_OUTPUT", pack=False, align=None),
    "VIRTUAL_ALLOCATION_TYPE": SimTypeInt(signed=False, label="VIRTUAL_ALLOCATION_TYPE"),
    "VIRTUAL_CHANNEL_CAPABILITIES1": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "VIRTUAL_CHANNEL_CAPABILITIES2": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "VIRTUAL_CHANNEL_CONTROL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "VIRTUAL_CHANNEL_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "VIRTUAL_DISK_ACCESS_MASK": SimTypeInt(signed=False, label="VIRTUAL_DISK_ACCESS_MASK"),
    "VIRTUAL_DISK_PROGRESS": SimStruct(OrderedDict((("OperationStatus", SimTypeInt(signed=False, label="UInt32")), ("CurrentValue", SimTypeLongLong(signed=False, label="UInt64")), ("CompletionValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="VIRTUAL_DISK_PROGRESS", pack=False, align=None),
    "VIRTUAL_FREE_TYPE": SimTypeInt(signed=False, label="VIRTUAL_FREE_TYPE"),
    "VIRTUAL_KEY": SimTypeInt(signed=False, label="VIRTUAL_KEY"),
    "VIRTUAL_MEMORY_INFORMATION_CLASS": SimTypeInt(signed=False, label="VIRTUAL_MEMORY_INFORMATION_CLASS"),
    "VIRTUAL_PROCESSOR_ARCH": SimTypeInt(signed=False, label="VIRTUAL_PROCESSOR_ARCH"),
    "VIRTUAL_PROCESSOR_REGISTER": SimUnion({"Reg64": SimTypeLongLong(signed=False, label="UInt64"), "Reg32": SimTypeInt(signed=False, label="UInt32"), "Reg16": SimTypeShort(signed=False, label="UInt16"), "Reg8": SimTypeChar(label="Byte"), "Reg128": SimTypeRef("_Reg128_e__Struct", SimStruct), "X64": SimUnion({"Segment": SimTypeRef("_Segment_e__Struct", SimStruct), "Table": SimTypeRef("_Table_e__Struct", SimStruct), "FpControlStatus": SimTypeRef("_FpControlStatus_e__Struct", SimStruct), "XmmControlStatus": SimTypeRef("_XmmControlStatus_e__Struct", SimStruct)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "VIRTUAL_PROCESSOR_VENDOR": SimTypeInt(signed=False, label="VIRTUAL_PROCESSOR_VENDOR"),
    "VIRTUAL_RESOURCE": SimStruct(OrderedDict((("Capability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Control", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RsvdP", SimTypeShort(signed=False, label="UInt16")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="VIRTUAL_RESOURCE", pack=False, align=None),
    "VIRTUAL_RESOURCE_CAPABILITY": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "VIRTUAL_RESOURCE_CONTROL": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "VIRTUAL_RESOURCE_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "VIRTUAL_STORAGE_BEHAVIOR_CODE": SimTypeInt(signed=False, label="VIRTUAL_STORAGE_BEHAVIOR_CODE"),
    "VIRTUAL_STORAGE_SET_BEHAVIOR_INPUT": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("BehaviorCode", SimTypeInt(signed=False, label="VIRTUAL_STORAGE_BEHAVIOR_CODE")),)), name="VIRTUAL_STORAGE_SET_BEHAVIOR_INPUT", pack=False, align=None),
    "VIRTUAL_STORAGE_TYPE": SimStruct(OrderedDict((("DeviceId", SimTypeInt(signed=False, label="UInt32")), ("VendorId", SimTypeBottom(label="Guid")),)), name="VIRTUAL_STORAGE_TYPE", pack=False, align=None),
    "VIRTUAL_TO_PHYSICAL": SimStruct(OrderedDict((("Status", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("PdeAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Virtual", SimTypeLongLong(signed=False, label="UInt64")), ("Physical", SimTypeLongLong(signed=False, label="UInt64")),)), name="VIRTUAL_TO_PHYSICAL", pack=False, align=None),
    "VK_F": SimStruct(OrderedDict((("Vk", SimTypeChar(label="Byte")), ("NLSFEProcType", SimTypeChar(label="Byte")), ("NLSFEProcCurrent", SimTypeChar(label="Byte")), ("NLSFEProcSwitch", SimTypeChar(label="Byte")), ("NLSFEProc", SimTypeArray(SimTypeRef("VK_FPARAM", SimStruct), 8)), ("NLSFEProcAlt", SimTypeArray(SimTypeRef("VK_FPARAM", SimStruct), 8)),)), name="VK_F", pack=False, align=None),
    "VK_FPARAM": SimStruct(OrderedDict((("NLSFEProcIndex", SimTypeChar(label="Byte")), ("NLSFEProcParam", SimTypeInt(signed=False, label="UInt32")),)), name="VK_FPARAM", pack=False, align=None),
    "VK_TO_BIT": SimStruct(OrderedDict((("Vk", SimTypeChar(label="Byte")), ("ModBits", SimTypeChar(label="Byte")),)), name="VK_TO_BIT", pack=False, align=None),
    "VK_TO_WCHARS1": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VK_TO_WCHARS1", pack=False, align=None),
    "VK_TO_WCHARS10": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 10)),)), name="VK_TO_WCHARS10", pack=False, align=None),
    "VK_TO_WCHARS2": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 2)),)), name="VK_TO_WCHARS2", pack=False, align=None),
    "VK_TO_WCHARS3": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 3)),)), name="VK_TO_WCHARS3", pack=False, align=None),
    "VK_TO_WCHARS4": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 4)),)), name="VK_TO_WCHARS4", pack=False, align=None),
    "VK_TO_WCHARS5": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 5)),)), name="VK_TO_WCHARS5", pack=False, align=None),
    "VK_TO_WCHARS6": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 6)),)), name="VK_TO_WCHARS6", pack=False, align=None),
    "VK_TO_WCHARS7": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 7)),)), name="VK_TO_WCHARS7", pack=False, align=None),
    "VK_TO_WCHARS8": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 8)),)), name="VK_TO_WCHARS8", pack=False, align=None),
    "VK_TO_WCHARS9": SimStruct(OrderedDict((("VirtualKey", SimTypeChar(label="Byte")), ("Attributes", SimTypeChar(label="Byte")), ("wch", SimTypeArray(SimTypeChar(label="Char"), 9)),)), name="VK_TO_WCHARS9", pack=False, align=None),
    "VK_TO_WCHAR_TABLE": SimStruct(OrderedDict((("pVkToWchars", SimTypePointer(SimTypeRef("VK_TO_WCHARS1", SimStruct), offset=0)), ("nModifications", SimTypeChar(label="Byte")), ("cbSize", SimTypeChar(label="Byte")),)), name="VK_TO_WCHAR_TABLE", pack=False, align=None),
    "VK_VSC": SimStruct(OrderedDict((("Vk", SimTypeChar(label="Byte")), ("Vsc", SimTypeChar(label="Byte")),)), name="VK_VSC", pack=False, align=None),
    "VLAN_TAG": SimStruct(OrderedDict((("Anonymous", SimUnion({"Tag": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Type", SimTypeShort(signed=False, label="UInt16")),)), name="VLAN_TAG", pack=False, align=None),
    "VMEMHEAP": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("stride", SimTypeInt(signed=False, label="UInt32")), ("freeList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("allocList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("fpGARTLin", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("fpGARTDev", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwCommitedSize", SimTypeInt(signed=False, label="UInt32")), ("dwCoalesceCount", SimTypeInt(signed=False, label="UInt32")), ("Alignment", SimTypeRef("HEAPALIGNMENT", SimStruct)), ("ddsCapsEx", SimTypeRef("DDSCAPSEX", SimStruct)), ("ddsCapsExAlt", SimTypeRef("DDSCAPSEX", SimStruct)), ("liPhysAGPBase", SimTypeLongLong(signed=True, label="Int64")), ("hdevAGP", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pvPhysRsrv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pAgpCommitMask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwAgpCommitMaskSize", SimTypeInt(signed=False, label="UInt32")),)), name="VMEMHEAP", pack=False, align=None),
    "VMEML": SimStruct(OrderedDict((("next", SimTypePointer(SimTypeRef("VMEML", SimStruct), offset=0)), ("ptr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")), ("bDiscardable", SimTypeInt(signed=True, label="Int32")),)), name="VMEML", pack=False, align=None),
    "VMEMR": SimStruct(OrderedDict((("next", SimTypePointer(SimTypeRef("VMEMR", SimStruct), offset=0)), ("prev", SimTypePointer(SimTypeRef("VMEMR", SimStruct), offset=0)), ("pUp", SimTypePointer(SimTypeRef("VMEMR", SimStruct), offset=0)), ("pDown", SimTypePointer(SimTypeRef("VMEMR", SimStruct), offset=0)), ("pLeft", SimTypePointer(SimTypeRef("VMEMR", SimStruct), offset=0)), ("pRight", SimTypePointer(SimTypeRef("VMEMR", SimStruct), offset=0)), ("ptr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("size", SimTypeInt(signed=False, label="UInt32")), ("x", SimTypeInt(signed=False, label="UInt32")), ("y", SimTypeInt(signed=False, label="UInt32")), ("cx", SimTypeInt(signed=False, label="UInt32")), ("cy", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="UInt32")), ("pBits", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("bDiscardable", SimTypeInt(signed=True, label="Int32")),)), name="VMEMR", pack=False, align=None),
    "VMR9AllocationInfo": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("Format", SimTypeInt(signed=False, label="D3DFORMAT")), ("Pool", SimTypeInt(signed=False, label="D3DPOOL")), ("MinBuffers", SimTypeInt(signed=False, label="UInt32")), ("szAspectRatio", SimTypeRef("SIZE", SimStruct)), ("szNativeSize", SimTypeRef("SIZE", SimStruct)),)), name="VMR9AllocationInfo", pack=False, align=None),
    "VMR9AlphaBitmap": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pDDS", SimTypeBottom(label="IDirect3DSurface9")), ("rSrc", SimTypeRef("RECT", SimStruct)), ("rDest", SimTypeRef("VMR9NormalizedRect", SimStruct)), ("fAlpha", SimTypeFloat(size=32)), ("clrSrcKey", SimTypeInt(signed=False, label="UInt32")), ("dwFilterMode", SimTypeInt(signed=False, label="UInt32")),)), name="VMR9AlphaBitmap", pack=False, align=None),
    "VMR9AlphaBitmapFlags": SimTypeInt(signed=False, label="VMR9AlphaBitmapFlags"),
    "VMR9AspectRatioMode": SimTypeInt(signed=False, label="VMR9AspectRatioMode"),
    "VMR9DeinterlaceCaps": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumPreviousOutputFrames", SimTypeInt(signed=False, label="UInt32")), ("dwNumForwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("dwNumBackwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("DeinterlaceTechnology", SimTypeInt(signed=False, label="VMR9DeinterlaceTech")),)), name="VMR9DeinterlaceCaps", pack=False, align=None),
    "VMR9DeinterlacePrefs": SimTypeInt(signed=False, label="VMR9DeinterlacePrefs"),
    "VMR9DeinterlaceTech": SimTypeInt(signed=False, label="VMR9DeinterlaceTech"),
    "VMR9Frequency": SimStruct(OrderedDict((("dwNumerator", SimTypeInt(signed=False, label="UInt32")), ("dwDenominator", SimTypeInt(signed=False, label="UInt32")),)), name="VMR9Frequency", pack=False, align=None),
    "VMR9MixerPrefs": SimTypeInt(signed=False, label="VMR9MixerPrefs"),
    "VMR9Mode": SimTypeInt(signed=False, label="VMR9Mode"),
    "VMR9MonitorInfo": SimStruct(OrderedDict((("uDevID", SimTypeInt(signed=False, label="UInt32")), ("rcMonitor", SimTypeRef("RECT", SimStruct)), ("hMon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szDevice", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 512)), ("liDriverVersion", SimTypeLongLong(signed=True, label="Int64")), ("dwVendorId", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceId", SimTypeInt(signed=False, label="UInt32")), ("dwSubSysId", SimTypeInt(signed=False, label="UInt32")), ("dwRevision", SimTypeInt(signed=False, label="UInt32")),)), name="VMR9MonitorInfo", pack=False, align=None),
    "VMR9NormalizedRect": SimStruct(OrderedDict((("left", SimTypeFloat(size=32)), ("top", SimTypeFloat(size=32)), ("right", SimTypeFloat(size=32)), ("bottom", SimTypeFloat(size=32)),)), name="VMR9NormalizedRect", pack=False, align=None),
    "VMR9PresentationFlags": SimTypeInt(signed=False, label="VMR9PresentationFlags"),
    "VMR9PresentationInfo": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpSurf", SimTypeBottom(label="IDirect3DSurface9")), ("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("szAspectRatio", SimTypeRef("SIZE", SimStruct)), ("rcSrc", SimTypeRef("RECT", SimStruct)), ("rcDst", SimTypeRef("RECT", SimStruct)), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="VMR9PresentationInfo", pack=False, align=None),
    "VMR9ProcAmpControl": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("Brightness", SimTypeFloat(size=32)), ("Contrast", SimTypeFloat(size=32)), ("Hue", SimTypeFloat(size=32)), ("Saturation", SimTypeFloat(size=32)),)), name="VMR9ProcAmpControl", pack=False, align=None),
    "VMR9ProcAmpControlFlags": SimTypeInt(signed=False, label="VMR9ProcAmpControlFlags"),
    "VMR9ProcAmpControlRange": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwProperty", SimTypeInt(signed=False, label="VMR9ProcAmpControlFlags")), ("MinValue", SimTypeFloat(size=32)), ("MaxValue", SimTypeFloat(size=32)), ("DefaultValue", SimTypeFloat(size=32)), ("StepSize", SimTypeFloat(size=32)),)), name="VMR9ProcAmpControlRange", pack=False, align=None),
    "VMR9RenderPrefs": SimTypeInt(signed=False, label="VMR9RenderPrefs"),
    "VMR9SurfaceAllocationFlags": SimTypeInt(signed=False, label="VMR9SurfaceAllocationFlags"),
    "VMR9VideoDesc": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSampleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwSampleHeight", SimTypeInt(signed=False, label="UInt32")), ("SampleFormat", SimTypeInt(signed=False, label="VMR9_SampleFormat")), ("dwFourCC", SimTypeInt(signed=False, label="UInt32")), ("InputSampleFreq", SimTypeRef("VMR9Frequency", SimStruct)), ("OutputFrameFreq", SimTypeRef("VMR9Frequency", SimStruct)),)), name="VMR9VideoDesc", pack=False, align=None),
    "VMR9VideoStreamInfo": SimStruct(OrderedDict((("pddsVideoSurface", SimTypeBottom(label="IDirect3DSurface9")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwStrmID", SimTypeInt(signed=False, label="UInt32")), ("fAlpha", SimTypeFloat(size=32)), ("rNormal", SimTypeRef("VMR9NormalizedRect", SimStruct)), ("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("SampleFormat", SimTypeInt(signed=False, label="VMR9_SampleFormat")),)), name="VMR9VideoStreamInfo", pack=False, align=None),
    "VMR9_SampleFormat": SimTypeInt(signed=False, label="VMR9_SampleFormat"),
    "VMRALLOCATIONINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpHdr", SimTypePointer(SimTypeRef("BITMAPINFOHEADER", SimStruct), offset=0)), ("lpPixFmt", SimTypePointer(SimTypeRef("DDPIXELFORMAT", SimStruct), offset=0)), ("szAspectRatio", SimTypeRef("SIZE", SimStruct)), ("dwMinBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwMaxBuffers", SimTypeInt(signed=False, label="UInt32")), ("dwInterlaceFlags", SimTypeInt(signed=False, label="UInt32")), ("szNativeSize", SimTypeRef("SIZE", SimStruct)),)), name="VMRALLOCATIONINFO", pack=False, align=None),
    "VMRALPHABITMAP": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pDDS", SimTypeBottom(label="IDirectDrawSurface7")), ("rSrc", SimTypeRef("RECT", SimStruct)), ("rDest", SimTypeRef("NORMALIZEDRECT", SimStruct)), ("fAlpha", SimTypeFloat(size=32)), ("clrSrcKey", SimTypeInt(signed=False, label="UInt32")),)), name="VMRALPHABITMAP", pack=False, align=None),
    "VMRDeinterlaceCaps": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumPreviousOutputFrames", SimTypeInt(signed=False, label="UInt32")), ("dwNumForwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("dwNumBackwardRefSamples", SimTypeInt(signed=False, label="UInt32")), ("DeinterlaceTechnology", SimTypeInt(signed=False, label="VMRDeinterlaceTech")),)), name="VMRDeinterlaceCaps", pack=False, align=None),
    "VMRDeinterlacePrefs": SimTypeInt(signed=False, label="VMRDeinterlacePrefs"),
    "VMRDeinterlaceTech": SimTypeInt(signed=False, label="VMRDeinterlaceTech"),
    "VMRFrequency": SimStruct(OrderedDict((("dwNumerator", SimTypeInt(signed=False, label="UInt32")), ("dwDenominator", SimTypeInt(signed=False, label="UInt32")),)), name="VMRFrequency", pack=False, align=None),
    "VMRGUID": SimStruct(OrderedDict((("pGUID", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("GUID", SimTypeBottom(label="Guid")),)), name="VMRGUID", pack=False, align=None),
    "VMRMONITORINFO": SimStruct(OrderedDict((("guid", SimTypeRef("VMRGUID", SimStruct)), ("rcMonitor", SimTypeRef("RECT", SimStruct)), ("hMon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("szDevice", SimTypeArray(SimTypeChar(label="Char"), 32)), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 256)), ("liDriverVersion", SimTypeLongLong(signed=True, label="Int64")), ("dwVendorId", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceId", SimTypeInt(signed=False, label="UInt32")), ("dwSubSysId", SimTypeInt(signed=False, label="UInt32")), ("dwRevision", SimTypeInt(signed=False, label="UInt32")),)), name="VMRMONITORINFO", pack=False, align=None),
    "VMRMixerPrefs": SimTypeInt(signed=False, label="VMRMixerPrefs"),
    "VMRMode": SimTypeInt(signed=False, label="VMRMode"),
    "VMRPRESENTATIONINFO": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpSurf", SimTypeBottom(label="IDirectDrawSurface7")), ("rtStart", SimTypeLongLong(signed=True, label="Int64")), ("rtEnd", SimTypeLongLong(signed=True, label="Int64")), ("szAspectRatio", SimTypeRef("SIZE", SimStruct)), ("rcSrc", SimTypeRef("RECT", SimStruct)), ("rcDst", SimTypeRef("RECT", SimStruct)), ("dwTypeSpecificFlags", SimTypeInt(signed=False, label="UInt32")), ("dwInterlaceFlags", SimTypeInt(signed=False, label="UInt32")),)), name="VMRPRESENTATIONINFO", pack=False, align=None),
    "VMRPresentationFlags": SimTypeInt(signed=False, label="VMRPresentationFlags"),
    "VMRRenderPrefs": SimTypeInt(signed=False, label="VMRRenderPrefs"),
    "VMRSurfaceAllocationFlags": SimTypeInt(signed=False, label="VMRSurfaceAllocationFlags"),
    "VMRVIDEOSTREAMINFO": SimStruct(OrderedDict((("pddsVideoSurface", SimTypeBottom(label="IDirectDrawSurface7")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwStrmID", SimTypeInt(signed=False, label="UInt32")), ("fAlpha", SimTypeFloat(size=32)), ("ddClrKey", SimTypeRef("DDCOLORKEY", SimStruct)), ("rNormal", SimTypeRef("NORMALIZEDRECT", SimStruct)),)), name="VMRVIDEOSTREAMINFO", pack=False, align=None),
    "VMRVideoDesc": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwSampleWidth", SimTypeInt(signed=False, label="UInt32")), ("dwSampleHeight", SimTypeInt(signed=False, label="UInt32")), ("SingleFieldPerSample", SimTypeInt(signed=True, label="Int32")), ("dwFourCC", SimTypeInt(signed=False, label="UInt32")), ("InputSampleFreq", SimTypeRef("VMRFrequency", SimStruct)), ("OutputFrameFreq", SimTypeRef("VMRFrequency", SimStruct)),)), name="VMRVideoDesc", pack=False, align=None),
    "VMR_ASPECT_RATIO_MODE": SimTypeInt(signed=False, label="VMR_ASPECT_RATIO_MODE"),
    "VM_COUNTERS": SimStruct(OrderedDict((("PeakVirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="VM_COUNTERS", pack=False, align=None),
    "VM_COUNTERS_EX": SimStruct(OrderedDict((("PeakVirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VirtualSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PageFaultCount", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("WorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaPeakNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("QuotaNonPagedPoolUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PeakPagefileUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PrivateUsage", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="VM_COUNTERS_EX", pack=False, align=None),
    "VM_COUNTERS_EX2": SimStruct(OrderedDict((("CountersEx", SimTypeRef("VM_COUNTERS_EX", SimStruct)), ("PrivateWorkingSetSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SharedCommitUsage", SimTypeLongLong(signed=False, label="UInt64")),)), name="VM_COUNTERS_EX2", pack=False, align=None),
    "VM_GENCOUNTER": SimStruct(OrderedDict((("GenerationCount", SimTypeLongLong(signed=False, label="UInt64")), ("GenerationCountHigh", SimTypeLongLong(signed=False, label="UInt64")),)), name="VM_GENCOUNTER", pack=False, align=None),
    "VM_HOST_NOTIFY_STATUS": SimTypeInt(signed=False, label="VM_HOST_NOTIFY_STATUS"),
    "VM_NOTIFY_ENTRY": SimStruct(OrderedDict((("VmName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("VmHost", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="VM_NOTIFY_ENTRY", pack=False, align=None),
    "VM_NOTIFY_INFO": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("ppVmEntries", SimTypePointer(SimTypePointer(SimTypeRef("VM_NOTIFY_ENTRY", SimStruct), offset=0), offset=0)),)), name="VM_NOTIFY_INFO", pack=False, align=None),
    "VM_NOTIFY_STATUS": SimTypeInt(signed=False, label="VM_NOTIFY_STATUS"),
    "VM_PATCH_INFO": SimStruct(OrderedDict((("dwNumEntries", SimTypeInt(signed=False, label="UInt32")), ("pVmNames", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="VM_PATCH_INFO", pack=False, align=None),
    "VM_RESDLL_CONTEXT": SimTypeInt(signed=False, label="VM_RESDLL_CONTEXT"),
    "VOLUMEWAVEFILTER": SimStruct(OrderedDict((("wfltr", SimTypeRef("WAVEFILTER", SimStruct)), ("dwVolume", SimTypeInt(signed=False, label="UInt32")),)), name="VOLUMEWAVEFILTER", pack=False, align=None),
    "VOLUME_ALLOCATE_BC_STREAM_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("RequestsPerPeriod", SimTypeInt(signed=False, label="UInt32")), ("Period", SimTypeInt(signed=False, label="UInt32")), ("RetryFailures", SimTypeChar(label="Byte")), ("Discardable", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("LowestByteOffset", SimTypeLongLong(signed=False, label="UInt64")), ("HighestByteOffset", SimTypeLongLong(signed=False, label="UInt64")), ("AccessType", SimTypeInt(signed=False, label="UInt32")), ("AccessMode", SimTypeInt(signed=False, label="UInt32")),)), name="VOLUME_ALLOCATE_BC_STREAM_INPUT", pack=False, align=None),
    "VOLUME_ALLOCATE_BC_STREAM_OUTPUT": SimStruct(OrderedDict((("RequestSize", SimTypeLongLong(signed=False, label="UInt64")), ("NumOutStandingRequests", SimTypeInt(signed=False, label="UInt32")),)), name="VOLUME_ALLOCATE_BC_STREAM_OUTPUT", pack=False, align=None),
    "VOLUME_ALLOCATION_HINT_INPUT": SimStruct(OrderedDict((("ClusterSize", SimTypeInt(signed=False, label="UInt32")), ("NumberOfClusters", SimTypeInt(signed=False, label="UInt32")), ("StartingClusterNumber", SimTypeLongLong(signed=True, label="Int64")),)), name="VOLUME_ALLOCATION_HINT_INPUT", pack=False, align=None),
    "VOLUME_ALLOCATION_HINT_OUTPUT": SimStruct(OrderedDict((("Bitmap", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="VOLUME_ALLOCATION_HINT_OUTPUT", pack=False, align=None),
    "VOLUME_BITMAP_BUFFER": SimStruct(OrderedDict((("StartingLcn", SimTypeLongLong(signed=True, label="Int64")), ("BitmapSize", SimTypeLongLong(signed=True, label="Int64")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="VOLUME_BITMAP_BUFFER", pack=False, align=None),
    "VOLUME_CRITICAL_IO": SimStruct(OrderedDict((("AccessType", SimTypeInt(signed=False, label="UInt32")), ("ExtentsCount", SimTypeInt(signed=False, label="UInt32")), ("Extents", SimTypePointer(SimTypeRef("FILE_EXTENT", SimStruct), offset=0)),)), name="VOLUME_CRITICAL_IO", pack=False, align=None),
    "VOLUME_DISK_EXTENTS": SimStruct(OrderedDict((("NumberOfDiskExtents", SimTypeInt(signed=False, label="UInt32")), ("Extents", SimTypePointer(SimTypeRef("DISK_EXTENT", SimStruct), offset=0)),)), name="VOLUME_DISK_EXTENTS", pack=False, align=None),
    "VOLUME_FAILOVER_SET": SimStruct(OrderedDict((("NumberOfDisks", SimTypeInt(signed=False, label="UInt32")), ("DiskNumbers", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="VOLUME_FAILOVER_SET", pack=False, align=None),
    "VOLUME_GET_BC_PROPERTIES_INPUT": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("LowestByteOffset", SimTypeLongLong(signed=False, label="UInt64")), ("HighestByteOffset", SimTypeLongLong(signed=False, label="UInt64")), ("AccessType", SimTypeInt(signed=False, label="UInt32")), ("AccessMode", SimTypeInt(signed=False, label="UInt32")),)), name="VOLUME_GET_BC_PROPERTIES_INPUT", pack=False, align=None),
    "VOLUME_GET_BC_PROPERTIES_OUTPUT": SimStruct(OrderedDict((("MaximumRequestsPerPeriod", SimTypeInt(signed=False, label="UInt32")), ("MinimumPeriod", SimTypeInt(signed=False, label="UInt32")), ("MaximumRequestSize", SimTypeLongLong(signed=False, label="UInt64")), ("EstimatedTimePerRequest", SimTypeInt(signed=False, label="UInt32")), ("NumOutStandingRequests", SimTypeInt(signed=False, label="UInt32")), ("RequestSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="VOLUME_GET_BC_PROPERTIES_OUTPUT", pack=False, align=None),
    "VOLUME_GET_GPT_ATTRIBUTES_INFORMATION": SimStruct(OrderedDict((("GptAttributes", SimTypeLongLong(signed=False, label="UInt64")),)), name="VOLUME_GET_GPT_ATTRIBUTES_INFORMATION", pack=False, align=None),
    "VOLUME_LOGICAL_OFFSET": SimStruct(OrderedDict((("LogicalOffset", SimTypeLongLong(signed=True, label="Int64")),)), name="VOLUME_LOGICAL_OFFSET", pack=False, align=None),
    "VOLUME_NUMBER": SimStruct(OrderedDict((("VolumeNumber", SimTypeInt(signed=False, label="UInt32")), ("VolumeManagerName", SimTypeArray(SimTypeChar(label="Char"), 8)),)), name="VOLUME_NUMBER", pack=False, align=None),
    "VOLUME_PHYSICAL_OFFSET": SimStruct(OrderedDict((("DiskNumber", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeLongLong(signed=True, label="Int64")),)), name="VOLUME_PHYSICAL_OFFSET", pack=False, align=None),
    "VOLUME_PHYSICAL_OFFSETS": SimStruct(OrderedDict((("NumberOfPhysicalOffsets", SimTypeInt(signed=False, label="UInt32")), ("PhysicalOffset", SimTypePointer(SimTypeRef("VOLUME_PHYSICAL_OFFSET", SimStruct), offset=0)),)), name="VOLUME_PHYSICAL_OFFSETS", pack=False, align=None),
    "VOLUME_READ_PLEX_INPUT": SimStruct(OrderedDict((("ByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("PlexNumber", SimTypeInt(signed=False, label="UInt32")),)), name="VOLUME_READ_PLEX_INPUT", pack=False, align=None),
    "VOLUME_REFS_INFO_BUFFER": SimStruct(OrderedDict((("CacheSizeInBytes", SimTypeLongLong(signed=True, label="Int64")), ("AllocatedCacheInBytes", SimTypeLongLong(signed=True, label="Int64")), ("PopulatedCacheInBytes", SimTypeLongLong(signed=True, label="Int64")), ("InErrorCacheInBytes", SimTypeLongLong(signed=True, label="Int64")), ("MemoryUsedForCacheMetadata", SimTypeLongLong(signed=True, label="Int64")), ("CacheLineSize", SimTypeInt(signed=False, label="UInt32")), ("CacheTransactionsOutstanding", SimTypeInt(signed=True, label="Int32")), ("CacheLinesFree", SimTypeInt(signed=True, label="Int32")), ("CacheLinesInError", SimTypeInt(signed=True, label="Int32")), ("CacheHitsInBytes", SimTypeLongLong(signed=True, label="Int64")), ("CacheMissesInBytes", SimTypeLongLong(signed=True, label="Int64")), ("CachePopulationUpdatesInBytes", SimTypeLongLong(signed=True, label="Int64")), ("CacheWriteThroughUpdatesInBytes", SimTypeLongLong(signed=True, label="Int64")), ("CacheInvalidationsInBytes", SimTypeLongLong(signed=True, label="Int64")), ("CacheOverReadsInBytes", SimTypeLongLong(signed=True, label="Int64")), ("MetadataWrittenBytes", SimTypeLongLong(signed=True, label="Int64")), ("CacheHitCounter", SimTypeInt(signed=True, label="Int32")), ("CacheMissCounter", SimTypeInt(signed=True, label="Int32")), ("CacheLineAllocationCounter", SimTypeInt(signed=True, label="Int32")), ("CacheInvalidationsCounter", SimTypeInt(signed=True, label="Int32")), ("CachePopulationUpdatesCounter", SimTypeInt(signed=True, label="Int32")), ("CacheWriteThroughUpdatesCounter", SimTypeInt(signed=True, label="Int32")), ("MaxCacheTransactionsOutstanding", SimTypeInt(signed=True, label="Int32")), ("DataWritesReallocationCount", SimTypeInt(signed=True, label="Int32")), ("DataInPlaceWriteCount", SimTypeInt(signed=True, label="Int32")), ("MetadataAllocationsFastTierCount", SimTypeInt(signed=True, label="Int32")), ("MetadataAllocationsSlowTierCount", SimTypeInt(signed=True, label="Int32")), ("DataAllocationsFastTierCount", SimTypeInt(signed=True, label="Int32")), ("DataAllocationsSlowTierCount", SimTypeInt(signed=True, label="Int32")), ("DestagesSlowTierToFastTier", SimTypeInt(signed=True, label="Int32")), ("DestagesFastTierToSlowTier", SimTypeInt(signed=True, label="Int32")), ("SlowTierDataFillRatio", SimTypeInt(signed=True, label="Int32")), ("FastTierDataFillRatio", SimTypeInt(signed=True, label="Int32")), ("SlowTierMetadataFillRatio", SimTypeInt(signed=True, label="Int32")), ("FastTierMetadataFillRatio", SimTypeInt(signed=True, label="Int32")), ("SlowToFastDestageReadLatency", SimTypeInt(signed=True, label="Int32")), ("SlowToFastDestageReadLatencyBase", SimTypeInt(signed=True, label="Int32")), ("SlowToFastDestageWriteLatency", SimTypeInt(signed=True, label="Int32")), ("SlowToFastDestageWriteLatencyBase", SimTypeInt(signed=True, label="Int32")), ("FastToSlowDestageReadLatency", SimTypeInt(signed=True, label="Int32")), ("FastToSlowDestageReadLatencyBase", SimTypeInt(signed=True, label="Int32")), ("FastToSlowDestageWriteLatency", SimTypeInt(signed=True, label="Int32")), ("FastToSlowDestageWriteLatencyBase", SimTypeInt(signed=True, label="Int32")), ("SlowTierContainerFillRatio", SimTypeInt(signed=True, label="Int32")), ("SlowTierContainerFillRatioBase", SimTypeInt(signed=True, label="Int32")), ("FastTierContainerFillRatio", SimTypeInt(signed=True, label="Int32")), ("FastTierContainerFillRatioBase", SimTypeInt(signed=True, label="Int32")), ("TreeUpdateLatency", SimTypeInt(signed=True, label="Int32")), ("TreeUpdateLatencyBase", SimTypeInt(signed=True, label="Int32")), ("CheckpointLatency", SimTypeInt(signed=True, label="Int32")), ("CheckpointLatencyBase", SimTypeInt(signed=True, label="Int32")), ("TreeUpdateCount", SimTypeInt(signed=True, label="Int32")), ("CheckpointCount", SimTypeInt(signed=True, label="Int32")), ("LogWriteCount", SimTypeInt(signed=True, label="Int32")), ("LogFillRatio", SimTypeInt(signed=True, label="Int32")), ("ReadCacheInvalidationsForOverwrite", SimTypeInt(signed=True, label="Int32")), ("ReadCacheInvalidationsForReuse", SimTypeInt(signed=True, label="Int32")), ("ReadCacheInvalidationsGeneral", SimTypeInt(signed=True, label="Int32")), ("ReadCacheChecksOnMount", SimTypeInt(signed=True, label="Int32")), ("ReadCacheIssuesOnMount", SimTypeInt(signed=True, label="Int32")), ("TrimLatency", SimTypeInt(signed=True, label="Int32")), ("TrimLatencyBase", SimTypeInt(signed=True, label="Int32")), ("DataCompactionCount", SimTypeInt(signed=True, label="Int32")), ("CompactionReadLatency", SimTypeInt(signed=True, label="Int32")), ("CompactionReadLatencyBase", SimTypeInt(signed=True, label="Int32")), ("CompactionWriteLatency", SimTypeInt(signed=True, label="Int32")), ("CompactionWriteLatencyBase", SimTypeInt(signed=True, label="Int32")), ("DataInPlaceWriteClusterCount", SimTypeLongLong(signed=True, label="Int64")), ("CompactionFailedDueToIneligibleContainer", SimTypeInt(signed=True, label="Int32")), ("CompactionFailedDueToMaxFragmentation", SimTypeInt(signed=True, label="Int32")), ("CompactedContainerFillRatio", SimTypeInt(signed=True, label="Int32")), ("CompactedContainerFillRatioBase", SimTypeInt(signed=True, label="Int32")), ("ContainerMoveRetryCount", SimTypeInt(signed=True, label="Int32")), ("ContainerMoveFailedDueToIneligibleContainer", SimTypeInt(signed=True, label="Int32")), ("CompactionFailureCount", SimTypeInt(signed=True, label="Int32")), ("ContainerMoveFailureCount", SimTypeInt(signed=True, label="Int32")), ("NumberOfDirtyMetadataPages", SimTypeLongLong(signed=True, label="Int64")), ("NumberOfDirtyTableListEntries", SimTypeInt(signed=True, label="Int32")), ("NumberOfDeleteQueueEntries", SimTypeInt(signed=True, label="Int32")),)), name="VOLUME_REFS_INFO_BUFFER", pack=False, align=None),
    "VOLUME_SET_GPT_ATTRIBUTES_INFORMATION": SimStruct(OrderedDict((("GptAttributes", SimTypeLongLong(signed=False, label="UInt64")), ("RevertOnClose", SimTypeChar(label="Byte")), ("ApplyToAllConnectedVolumes", SimTypeChar(label="Byte")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="VOLUME_SET_GPT_ATTRIBUTES_INFORMATION", pack=False, align=None),
    "VOLUME_SHRINK_INFO": SimStruct(OrderedDict((("VolumeSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="VOLUME_SHRINK_INFO", pack=False, align=None),
    "VPB": SimStruct(OrderedDict((("Type", SimTypeShort(signed=True, label="Int16")), ("Size", SimTypeShort(signed=True, label="Int16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("VolumeLabelLength", SimTypeShort(signed=False, label="UInt16")), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("RealDevice", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)), ("SerialNumber", SimTypeInt(signed=False, label="UInt32")), ("ReferenceCount", SimTypeInt(signed=False, label="UInt32")), ("VolumeLabel", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="VPB", pack=False, align=None),
    "VPCOLORFLAGS": SimTypeInt(signed=False, label="VPCOLORFLAGS"),
    "VPN_TS_IP_ADDRESS": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"v4": SimTypeRef("IN_ADDR", SimStruct), "v6": SimTypeRef("IN6_ADDR", SimStruct)}, name="<anon>", label="None")),)), name="VPN_TS_IP_ADDRESS", pack=False, align=None),
    "VPWATERMARKFLAGS": SimTypeInt(signed=False, label="VPWATERMARKFLAGS"),
    "VRAM_SURFACE_INFO": SimStruct(OrderedDict((("hSurface", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("VramPhysicalAddress", SimTypeLongLong(signed=True, label="Int64")), ("cbCaptured", SimTypeInt(signed=False, label="UInt32")), ("dwWidth", SimTypeInt(signed=False, label="UInt32")), ("dwHeight", SimTypeInt(signed=False, label="UInt32")), ("dwLinearSize", SimTypeInt(signed=False, label="UInt32")), ("lPitch", SimTypeInt(signed=True, label="Int32")), ("ullReserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)),)), name="VRAM_SURFACE_INFO", pack=False, align=None),
    "VRAM_SURFACE_INFO_PROPERTY_S": SimStruct(OrderedDict((("Property", SimTypeRef("KSIDENTIFIER", SimStruct)), ("pVramSurfaceInfo", SimTypePointer(SimTypeRef("VRAM_SURFACE_INFO", SimStruct), offset=0)),)), name="VRAM_SURFACE_INFO_PROPERTY_S", pack=False, align=None),
    "VSC_LPWSTR": SimStruct(OrderedDict((("vsc", SimTypeChar(label="Byte")), ("pwsz", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="VSC_LPWSTR", pack=False, align=None),
    "VSC_VK": SimStruct(OrderedDict((("Vsc", SimTypeChar(label="Byte")), ("Vk", SimTypeShort(signed=False, label="UInt16")),)), name="VSC_VK", pack=False, align=None),
    "VSS_ALTERNATE_WRITER_STATE": SimTypeInt(signed=False, label="VSS_ALTERNATE_WRITER_STATE"),
    "VSS_APPLICATION_LEVEL": SimTypeInt(signed=False, label="VSS_APPLICATION_LEVEL"),
    "VSS_BACKUP_SCHEMA": SimTypeInt(signed=False, label="VSS_BACKUP_SCHEMA"),
    "VSS_BACKUP_TYPE": SimTypeInt(signed=False, label="VSS_BACKUP_TYPE"),
    "VSS_COMPONENT_FLAGS": SimTypeInt(signed=False, label="VSS_COMPONENT_FLAGS"),
    "VSS_COMPONENT_TYPE": SimTypeInt(signed=False, label="VSS_COMPONENT_TYPE"),
    "VSS_DIFF_AREA_PROP": SimStruct(OrderedDict((("m_pwszVolumeName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszDiffAreaVolumeName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_llMaximumDiffSpace", SimTypeLongLong(signed=True, label="Int64")), ("m_llAllocatedDiffSpace", SimTypeLongLong(signed=True, label="Int64")), ("m_llUsedDiffSpace", SimTypeLongLong(signed=True, label="Int64")),)), name="VSS_DIFF_AREA_PROP", pack=False, align=None),
    "VSS_DIFF_VOLUME_PROP": SimStruct(OrderedDict((("m_pwszVolumeName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszVolumeDisplayName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_llVolumeFreeSpace", SimTypeLongLong(signed=True, label="Int64")), ("m_llVolumeTotalSpace", SimTypeLongLong(signed=True, label="Int64")),)), name="VSS_DIFF_VOLUME_PROP", pack=False, align=None),
    "VSS_FILE_RESTORE_STATUS": SimTypeInt(signed=False, label="VSS_FILE_RESTORE_STATUS"),
    "VSS_FILE_SPEC_BACKUP_TYPE": SimTypeInt(signed=False, label="VSS_FILE_SPEC_BACKUP_TYPE"),
    "VSS_HARDWARE_OPTIONS": SimTypeInt(signed=False, label="VSS_HARDWARE_OPTIONS"),
    "VSS_MGMT_OBJECT_PROP": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="VSS_MGMT_OBJECT_TYPE")), ("Obj", SimUnion({"Vol": SimTypeRef("VSS_VOLUME_PROP", SimStruct), "DiffVol": SimTypeRef("VSS_DIFF_VOLUME_PROP", SimStruct), "DiffArea": SimTypeRef("VSS_DIFF_AREA_PROP", SimStruct)}, name="<anon>", label="None")),)), name="VSS_MGMT_OBJECT_PROP", pack=False, align=None),
    "VSS_MGMT_OBJECT_TYPE": SimTypeInt(signed=False, label="VSS_MGMT_OBJECT_TYPE"),
    "VSS_MGMT_OBJECT_UNION": SimUnion({"Vol": SimTypeRef("VSS_VOLUME_PROP", SimStruct), "DiffVol": SimTypeRef("VSS_DIFF_VOLUME_PROP", SimStruct), "DiffArea": SimTypeRef("VSS_DIFF_AREA_PROP", SimStruct)}, name="<anon>", label="None"),
    "VSS_OBJECT_PROP": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="VSS_OBJECT_TYPE")), ("Obj", SimUnion({"Snap": SimTypeRef("VSS_SNAPSHOT_PROP", SimStruct), "Prov": SimTypeRef("VSS_PROVIDER_PROP", SimStruct)}, name="<anon>", label="None")),)), name="VSS_OBJECT_PROP", pack=False, align=None),
    "VSS_OBJECT_TYPE": SimTypeInt(signed=False, label="VSS_OBJECT_TYPE"),
    "VSS_OBJECT_UNION": SimUnion({"Snap": SimTypeRef("VSS_SNAPSHOT_PROP", SimStruct), "Prov": SimTypeRef("VSS_PROVIDER_PROP", SimStruct)}, name="<anon>", label="None"),
    "VSS_PROTECTION_FAULT": SimTypeInt(signed=False, label="VSS_PROTECTION_FAULT"),
    "VSS_PROTECTION_LEVEL": SimTypeInt(signed=False, label="VSS_PROTECTION_LEVEL"),
    "VSS_PROVIDER_CAPABILITIES": SimTypeInt(signed=False, label="VSS_PROVIDER_CAPABILITIES"),
    "VSS_PROVIDER_PROP": SimStruct(OrderedDict((("m_ProviderId", SimTypeBottom(label="Guid")), ("m_pwszProviderName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_eProviderType", SimTypeInt(signed=False, label="VSS_PROVIDER_TYPE")), ("m_pwszProviderVersion", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_ProviderVersionId", SimTypeBottom(label="Guid")), ("m_ClassId", SimTypeBottom(label="Guid")),)), name="VSS_PROVIDER_PROP", pack=False, align=None),
    "VSS_PROVIDER_TYPE": SimTypeInt(signed=False, label="VSS_PROVIDER_TYPE"),
    "VSS_RECOVERY_OPTIONS": SimTypeInt(signed=False, label="VSS_RECOVERY_OPTIONS"),
    "VSS_RESTOREMETHOD_ENUM": SimTypeInt(signed=False, label="VSS_RESTOREMETHOD_ENUM"),
    "VSS_RESTORE_TARGET": SimTypeInt(signed=False, label="VSS_RESTORE_TARGET"),
    "VSS_RESTORE_TYPE": SimTypeInt(signed=False, label="VSS_RESTORE_TYPE"),
    "VSS_ROLLFORWARD_TYPE": SimTypeInt(signed=False, label="VSS_ROLLFORWARD_TYPE"),
    "VSS_SNAPSHOT_COMPATIBILITY": SimTypeInt(signed=False, label="VSS_SNAPSHOT_COMPATIBILITY"),
    "VSS_SNAPSHOT_CONTEXT": SimTypeInt(signed=False, label="VSS_SNAPSHOT_CONTEXT"),
    "VSS_SNAPSHOT_PROP": SimStruct(OrderedDict((("m_SnapshotId", SimTypeBottom(label="Guid")), ("m_SnapshotSetId", SimTypeBottom(label="Guid")), ("m_lSnapshotsCount", SimTypeInt(signed=True, label="Int32")), ("m_pwszSnapshotDeviceObject", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszOriginalVolumeName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszOriginatingMachine", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszServiceMachine", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszExposedName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszExposedPath", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_ProviderId", SimTypeBottom(label="Guid")), ("m_lSnapshotAttributes", SimTypeInt(signed=True, label="Int32")), ("m_tsCreationTimestamp", SimTypeLongLong(signed=True, label="Int64")), ("m_eStatus", SimTypeInt(signed=False, label="VSS_SNAPSHOT_STATE")),)), name="VSS_SNAPSHOT_PROP", pack=False, align=None),
    "VSS_SNAPSHOT_PROPERTY_ID": SimTypeInt(signed=False, label="VSS_SNAPSHOT_PROPERTY_ID"),
    "VSS_SNAPSHOT_STATE": SimTypeInt(signed=False, label="VSS_SNAPSHOT_STATE"),
    "VSS_SOURCE_TYPE": SimTypeInt(signed=False, label="VSS_SOURCE_TYPE"),
    "VSS_SUBSCRIBE_MASK": SimTypeInt(signed=False, label="VSS_SUBSCRIBE_MASK"),
    "VSS_USAGE_TYPE": SimTypeInt(signed=False, label="VSS_USAGE_TYPE"),
    "VSS_VOLUME_PROP": SimStruct(OrderedDict((("m_pwszVolumeName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)), ("m_pwszVolumeDisplayName", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="VSS_VOLUME_PROP", pack=False, align=None),
    "VSS_VOLUME_PROTECTION_INFO": SimStruct(OrderedDict((("m_protectionLevel", SimTypeInt(signed=False, label="VSS_PROTECTION_LEVEL")), ("m_volumeIsOfflineForProtection", SimTypeInt(signed=True, label="Int32")), ("m_protectionFault", SimTypeInt(signed=False, label="VSS_PROTECTION_FAULT")), ("m_failureStatus", SimTypeInt(signed=True, label="Int32")), ("m_volumeHasUnusedDiffArea", SimTypeInt(signed=True, label="Int32")), ("m_reserved", SimTypeInt(signed=False, label="UInt32")),)), name="VSS_VOLUME_PROTECTION_INFO", pack=False, align=None),
    "VSS_VOLUME_SNAPSHOT_ATTRIBUTES": SimTypeInt(signed=False, label="VSS_VOLUME_SNAPSHOT_ATTRIBUTES"),
    "VSS_WRITERRESTORE_ENUM": SimTypeInt(signed=False, label="VSS_WRITERRESTORE_ENUM"),
    "VSS_WRITER_STATE": SimTypeInt(signed=False, label="VSS_WRITER_STATE"),
    "VS_FIXEDFILEINFO": SimStruct(OrderedDict((("dwSignature", SimTypeInt(signed=False, label="UInt32")), ("dwStrucVersion", SimTypeInt(signed=False, label="UInt32")), ("dwFileVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwFileVersionLS", SimTypeInt(signed=False, label="UInt32")), ("dwProductVersionMS", SimTypeInt(signed=False, label="UInt32")), ("dwProductVersionLS", SimTypeInt(signed=False, label="UInt32")), ("dwFileFlagsMask", SimTypeInt(signed=False, label="UInt32")), ("dwFileFlags", SimTypeInt(signed=False, label="VS_FIXEDFILEINFO_FILE_FLAGS")), ("dwFileOS", SimTypeInt(signed=False, label="VS_FIXEDFILEINFO_FILE_OS")), ("dwFileType", SimTypeInt(signed=False, label="UInt32")), ("dwFileSubtype", SimTypeInt(signed=False, label="UInt32")), ("dwFileDateMS", SimTypeInt(signed=False, label="UInt32")), ("dwFileDateLS", SimTypeInt(signed=False, label="UInt32")),)), name="VS_FIXEDFILEINFO", pack=False, align=None),
    "VS_FIXEDFILEINFO_FILE_FLAGS": SimTypeInt(signed=False, label="VS_FIXEDFILEINFO_FILE_FLAGS"),
    "VS_FIXEDFILEINFO_FILE_OS": SimTypeInt(signed=False, label="VS_FIXEDFILEINFO_FILE_OS"),
    "VS_FIXEDFILEINFO_FILE_SUBTYPE": SimTypeInt(signed=False, label="VS_FIXEDFILEINFO_FILE_SUBTYPE"),
    "VS_FIXEDFILEINFO_FILE_TYPE": SimTypeInt(signed=False, label="VS_FIXEDFILEINFO_FILE_TYPE"),
    "ValueMapType": SimTypeInt(signed=False, label="ValueMapType"),
    "VarArgsKind": SimTypeInt(signed=False, label="VarArgsKind"),
    "VfwCaptureDialogs": SimTypeInt(signed=False, label="VfwCaptureDialogs"),
    "VfwCompressDialogs": SimTypeInt(signed=False, label="VfwCompressDialogs"),
    "VideoControlFlags": SimTypeInt(signed=False, label="VideoControlFlags"),
    "VideoCopyProtectionType": SimTypeInt(signed=False, label="VideoCopyProtectionType"),
    "VideoProcAmpFlags": SimTypeInt(signed=False, label="VideoProcAmpFlags"),
    "VideoProcAmpProperty": SimTypeInt(signed=False, label="VideoProcAmpProperty"),
    "View": SimTypeBottom(label="View"),
    "Views": SimTypeBottom(label="Views"),
    "VisualEffects": SimTypeInt(signed=False, label="VisualEffects"),
    "VisualElement": SimStruct(OrderedDict((("Handle", SimTypeLongLong(signed=False, label="UInt64")), ("SrcInfo", SimTypeRef("SourceInfo", SimStruct)), ("Type", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("NumChildren", SimTypeInt(signed=False, label="UInt32")),)), name="VisualElement", pack=False, align=None),
    "VisualElementState": SimTypeInt(signed=False, label="VisualElementState"),
    "VisualMutationType": SimTypeInt(signed=False, label="VisualMutationType"),
    "VisualState": SimTypeInt(signed=False, label="VisualState"),
    "VolLockBroadcast": SimStruct(OrderedDict((("vlb_dbh", SimTypeRef("DEV_BROADCAST_HDR", SimStruct)), ("vlb_owner", SimTypeInt(signed=False, label="UInt32")), ("vlb_perms", SimTypeChar(label="Byte")), ("vlb_lockType", SimTypeChar(label="Byte")), ("vlb_drive", SimTypeChar(label="Byte")), ("vlb_flags", SimTypeChar(label="Byte")),)), name="VolLockBroadcast", pack=False, align=None),
    "WABEXTDISPLAY": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("lpWABObject", SimTypeBottom(label="IWABObject")), ("lpAdrBook", SimTypeBottom(label="IAddrBook")), ("lpPropObj", SimTypeBottom(label="IMAPIProp")), ("fReadOnly", SimTypeInt(signed=True, label="Int32")), ("fDataChanged", SimTypeInt(signed=True, label="Int32")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpv", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpsz", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="WABEXTDISPLAY", pack=False, align=None),
    "WABIMPORTPARAM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("lpAdrBook", SimTypeBottom(label="IAddrBook")), ("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("lpszFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WABIMPORTPARAM", pack=False, align=None),
    "WAB_PARAM": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("szFileName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("guidPSExt", SimTypeBottom(label="Guid")),)), name="WAB_PARAM", pack=False, align=None),
    "WAITCHAIN_NODE_INFO": SimStruct(OrderedDict((("ObjectType", SimTypeInt(signed=False, label="WCT_OBJECT_TYPE")), ("ObjectStatus", SimTypeInt(signed=False, label="WCT_OBJECT_STATUS")), ("Anonymous", SimUnion({"LockObject": SimTypeRef("_LockObject_e__Struct", SimStruct), "ThreadObject": SimTypeRef("_ThreadObject_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WAITCHAIN_NODE_INFO", pack=False, align=None),
    "WAITORTIMERCALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "WAIT_CHAIN_THREAD_OPTIONS": SimTypeInt(signed=False, label="WAIT_CHAIN_THREAD_OPTIONS"),
    "WAIT_CONTEXT_BLOCK": SimStruct(OrderedDict((("Anonymous", SimUnion({"WaitQueueEntry": SimTypeRef("KDEVICE_QUEUE_ENTRY", SimStruct), "Anonymous": SimStruct(OrderedDict((("DmaWaitEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("NumberOfChannels", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("DeviceRoutine", SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="IO_ALLOCATION_ACTION"), arg_names=["DeviceObject", "Irp", "MapRegisterBase", "Context"]), offset=0), offset=0)), ("DeviceContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfMapRegisters", SimTypeInt(signed=False, label="UInt32")), ("DeviceObject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("CurrentIrp", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferChainingDpc", SimTypePointer(SimTypeRef("KDPC", SimStruct), offset=0)),)), name="WAIT_CONTEXT_BLOCK", pack=False, align=None),
    "WAIT_EVENT": SimTypeInt(signed=False, label="WAIT_EVENT"),
    "WAIT_OPTION": SimTypeInt(signed=False, label="WAIT_OPTION"),
    "WAIT_TYPE": SimTypeInt(signed=False, label="WAIT_TYPE"),
    "WAKE_ALARM_INFORMATION": SimStruct(OrderedDict((("TimerIdentifier", SimTypeInt(signed=False, label="UInt32")), ("Timeout", SimTypeInt(signed=False, label="UInt32")),)), name="WAKE_ALARM_INFORMATION", pack=False, align=None),
    "WARNINGSTATES": SimTypeInt(signed=False, label="WARNINGSTATES"),
    "WAVEFILTER": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFilterTag", SimTypeInt(signed=False, label="UInt32")), ("fdwFilter", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 5)),)), name="WAVEFILTER", pack=False, align=None),
    "WAVEFORMAT": SimStruct(OrderedDict((("wFormatTag", SimTypeShort(signed=False, label="UInt16")), ("nChannels", SimTypeShort(signed=False, label="UInt16")), ("nSamplesPerSec", SimTypeInt(signed=False, label="UInt32")), ("nAvgBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("nBlockAlign", SimTypeShort(signed=False, label="UInt16")),)), name="WAVEFORMAT", pack=False, align=None),
    "WAVEFORMATEX": SimStruct(OrderedDict((("wFormatTag", SimTypeShort(signed=False, label="UInt16")), ("nChannels", SimTypeShort(signed=False, label="UInt16")), ("nSamplesPerSec", SimTypeInt(signed=False, label="UInt32")), ("nAvgBytesPerSec", SimTypeInt(signed=False, label="UInt32")), ("nBlockAlign", SimTypeShort(signed=False, label="UInt16")), ("wBitsPerSample", SimTypeShort(signed=False, label="UInt16")), ("cbSize", SimTypeShort(signed=False, label="UInt16")),)), name="WAVEFORMATEX", pack=False, align=None),
    "WAVEFORMATEXTENSIBLE": SimStruct(OrderedDict((("Format", SimTypeRef("WAVEFORMATEX", SimStruct)), ("Samples", SimUnion({"wValidBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "wSamplesPerBlock": SimTypeShort(signed=False, label="UInt16"), "wReserved": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("dwChannelMask", SimTypeInt(signed=False, label="UInt32")), ("SubFormat", SimTypeBottom(label="Guid")),)), name="WAVEFORMATEXTENSIBLE", pack=False, align=None),
    "WAVEHDR": SimStruct(OrderedDict((("lpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwBufferLength", SimTypeInt(signed=False, label="UInt32")), ("dwBytesRecorded", SimTypeInt(signed=False, label="UInt32")), ("dwUser", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwLoops", SimTypeInt(signed=False, label="UInt32")), ("lpNext", SimTypePointer(SimTypeRef("WAVEHDR", SimStruct), offset=0)), ("reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WAVEHDR", pack=False, align=None),
    "WAVEINCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="WAVEINCAPS2A", pack=False, align=None),
    "WAVEINCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="WAVEINCAPS2W", pack=False, align=None),
    "WAVEINCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")),)), name="WAVEINCAPSA", pack=False, align=None),
    "WAVEINCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")),)), name="WAVEINCAPSW", pack=False, align=None),
    "WAVELINK": SimStruct(OrderedDict((("fusOptions", SimTypeShort(signed=False, label="UInt16")), ("usPhaseGroup", SimTypeShort(signed=False, label="UInt16")), ("ulChannel", SimTypeInt(signed=False, label="UInt32")), ("ulTableIndex", SimTypeInt(signed=False, label="UInt32")),)), name="WAVELINK", pack=False, align=None),
    "WAVEOPENDESC": SimStruct(OrderedDict((("hWave", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpFormat", SimTypePointer(SimTypeRef("WAVEFORMAT", SimStruct), offset=0)), ("dwCallback", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwInstance", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("uMappedDeviceID", SimTypeInt(signed=False, label="UInt32")), ("dnDevNode", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WAVEOPENDESC", pack=False, align=None),
    "WAVEOUTCAPS2A": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="WAVEOUTCAPS2A", pack=False, align=None),
    "WAVEOUTCAPS2W": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerGuid", SimTypeBottom(label="Guid")), ("ProductGuid", SimTypeBottom(label="Guid")), ("NameGuid", SimTypeBottom(label="Guid")),)), name="WAVEOUTCAPS2W", pack=False, align=None),
    "WAVEOUTCAPSA": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="WAVEOUTCAPSA", pack=False, align=None),
    "WAVEOUTCAPSW": SimStruct(OrderedDict((("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)), ("dwFormats", SimTypeInt(signed=False, label="UInt32")), ("wChannels", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("dwSupport", SimTypeInt(signed=False, label="UInt32")),)), name="WAVEOUTCAPSW", pack=False, align=None),
    "WBCL_Iterator": SimStruct(OrderedDict((("firstElementPtr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("logSize", SimTypeInt(signed=False, label="UInt32")), ("currentElementPtr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("currentElementSize", SimTypeInt(signed=False, label="UInt32")), ("digestSize", SimTypeShort(signed=False, label="UInt16")), ("logFormat", SimTypeShort(signed=False, label="UInt16")), ("numberOfDigests", SimTypeInt(signed=False, label="UInt32")), ("digestSizes", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("supportedAlgorithms", SimTypeInt(signed=False, label="UInt32")), ("hashAlgorithm", SimTypeShort(signed=False, label="UInt16")),)), name="WBCL_Iterator", pack=False, align=None),
    "WBCL_LogHdr": SimStruct(OrderedDict((("signature", SimTypeInt(signed=False, label="UInt32")), ("version", SimTypeInt(signed=False, label="UInt32")), ("entries", SimTypeInt(signed=False, label="UInt32")), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="WBCL_LogHdr", pack=False, align=None),
    "WBEMSTATUS": SimTypeInt(signed=False, label="WBEMSTATUS"),
    "WBEMSTATUS_FORMAT": SimTypeInt(signed=False, label="WBEMSTATUS_FORMAT"),
    "WBEM_BACKUP_RESTORE_FLAGS": SimTypeInt(signed=False, label="WBEM_BACKUP_RESTORE_FLAGS"),
    "WBEM_BATCH_TYPE": SimTypeInt(signed=False, label="WBEM_BATCH_TYPE"),
    "WBEM_CHANGE_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_CHANGE_FLAG_TYPE"),
    "WBEM_COMPARISON_FLAG": SimTypeInt(signed=False, label="WBEM_COMPARISON_FLAG"),
    "WBEM_COMPILER_OPTIONS": SimTypeInt(signed=False, label="WBEM_COMPILER_OPTIONS"),
    "WBEM_COMPILE_STATUS_INFO": SimStruct(OrderedDict((("lPhaseError", SimTypeInt(signed=True, label="Int32")), ("hRes", SimTypeInt(signed=True, label="Int32")), ("ObjectNum", SimTypeInt(signed=True, label="Int32")), ("FirstLine", SimTypeInt(signed=True, label="Int32")), ("LastLine", SimTypeInt(signed=True, label="Int32")), ("dwOutFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WBEM_COMPILE_STATUS_INFO", pack=False, align=None),
    "WBEM_CONDITION_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_CONDITION_FLAG_TYPE"),
    "WBEM_CONNECT_OPTIONS": SimTypeInt(signed=False, label="WBEM_CONNECT_OPTIONS"),
    "WBEM_EXTRA_RETURN_CODES": SimTypeInt(signed=False, label="WBEM_EXTRA_RETURN_CODES"),
    "WBEM_FLAVOR_TYPE": SimTypeInt(signed=False, label="WBEM_FLAVOR_TYPE"),
    "WBEM_GENERIC_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_GENERIC_FLAG_TYPE"),
    "WBEM_GENUS_TYPE": SimTypeInt(signed=False, label="WBEM_GENUS_TYPE"),
    "WBEM_GET_KEY_FLAGS": SimTypeInt(signed=False, label="WBEM_GET_KEY_FLAGS"),
    "WBEM_GET_TEXT_FLAGS": SimTypeInt(signed=False, label="WBEM_GET_TEXT_FLAGS"),
    "WBEM_INFORMATION_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_INFORMATION_FLAG_TYPE"),
    "WBEM_LIMITATION_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_LIMITATION_FLAG_TYPE"),
    "WBEM_LIMITS": SimTypeInt(signed=False, label="WBEM_LIMITS"),
    "WBEM_LOCKING_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_LOCKING_FLAG_TYPE"),
    "WBEM_LOGIN_TYPE": SimTypeInt(signed=False, label="WBEM_LOGIN_TYPE"),
    "WBEM_PATH_CREATE_FLAG": SimTypeInt(signed=False, label="WBEM_PATH_CREATE_FLAG"),
    "WBEM_PATH_STATUS_FLAG": SimTypeInt(signed=False, label="WBEM_PATH_STATUS_FLAG"),
    "WBEM_PROVIDER_FLAGS": SimTypeInt(signed=False, label="WBEM_PROVIDER_FLAGS"),
    "WBEM_PROVIDER_REQUIREMENTS_TYPE": SimTypeInt(signed=False, label="WBEM_PROVIDER_REQUIREMENTS_TYPE"),
    "WBEM_QUERY_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_QUERY_FLAG_TYPE"),
    "WBEM_REFRESHER_FLAGS": SimTypeInt(signed=False, label="WBEM_REFRESHER_FLAGS"),
    "WBEM_SECURITY_FLAGS": SimTypeInt(signed=False, label="WBEM_SECURITY_FLAGS"),
    "WBEM_SHUTDOWN_FLAGS": SimTypeInt(signed=False, label="WBEM_SHUTDOWN_FLAGS"),
    "WBEM_STATUS_TYPE": SimTypeInt(signed=False, label="WBEM_STATUS_TYPE"),
    "WBEM_TEXT_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_TEXT_FLAG_TYPE"),
    "WBEM_UNSECAPP_FLAG_TYPE": SimTypeInt(signed=False, label="WBEM_UNSECAPP_FLAG_TYPE"),
    "WCE_DEVICELIST": SimStruct(OrderedDict((("numDevice", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypePointer(SimTypeRef("WCE_IRDA_DEVICE_INFO", SimStruct), offset=0)),)), name="WCE_DEVICELIST", pack=False, align=None),
    "WCE_IRDA_DEVICE_INFO": SimStruct(OrderedDict((("irdaDeviceID", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("irdaDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 22)), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="WCE_IRDA_DEVICE_INFO", pack=False, align=None),
    "WCM_BILLING_CYCLE_INFO": SimStruct(OrderedDict((("StartDate", SimTypeRef("FILETIME", SimStruct)), ("Duration", SimTypeRef("WCM_TIME_INTERVAL", SimStruct)), ("Reset", SimTypeInt(signed=True, label="Int32")),)), name="WCM_BILLING_CYCLE_INFO", pack=False, align=None),
    "WCM_CONNECTION_COST": SimTypeInt(signed=False, label="WCM_CONNECTION_COST"),
    "WCM_CONNECTION_COST_DATA": SimStruct(OrderedDict((("ConnectionCost", SimTypeInt(signed=False, label="UInt32")), ("CostSource", SimTypeInt(signed=False, label="WCM_CONNECTION_COST_SOURCE")),)), name="WCM_CONNECTION_COST_DATA", pack=False, align=None),
    "WCM_CONNECTION_COST_SOURCE": SimTypeInt(signed=False, label="WCM_CONNECTION_COST_SOURCE"),
    "WCM_DATAPLAN_STATUS": SimStruct(OrderedDict((("UsageData", SimTypeRef("WCM_USAGE_DATA", SimStruct)), ("DataLimitInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("InboundBandwidthInKbps", SimTypeInt(signed=False, label="UInt32")), ("OutboundBandwidthInKbps", SimTypeInt(signed=False, label="UInt32")), ("BillingCycle", SimTypeRef("WCM_BILLING_CYCLE_INFO", SimStruct)), ("MaxTransferSizeInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="WCM_DATAPLAN_STATUS", pack=False, align=None),
    "WCM_MEDIA_TYPE": SimTypeInt(signed=False, label="WCM_MEDIA_TYPE"),
    "WCM_POLICY_VALUE": SimStruct(OrderedDict((("fValue", SimTypeInt(signed=True, label="Int32")), ("fIsGroupPolicy", SimTypeInt(signed=True, label="Int32")),)), name="WCM_POLICY_VALUE", pack=False, align=None),
    "WCM_PROFILE_INFO": SimStruct(OrderedDict((("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("AdapterGUID", SimTypeBottom(label="Guid")), ("Media", SimTypeInt(signed=False, label="WCM_MEDIA_TYPE")),)), name="WCM_PROFILE_INFO", pack=False, align=None),
    "WCM_PROFILE_INFO_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("ProfileInfo", SimTypePointer(SimTypeRef("WCM_PROFILE_INFO", SimStruct), offset=0)),)), name="WCM_PROFILE_INFO_LIST", pack=False, align=None),
    "WCM_PROPERTY": SimTypeInt(signed=False, label="WCM_PROPERTY"),
    "WCM_TIME_INTERVAL": SimStruct(OrderedDict((("wYear", SimTypeShort(signed=False, label="UInt16")), ("wMonth", SimTypeShort(signed=False, label="UInt16")), ("wDay", SimTypeShort(signed=False, label="UInt16")), ("wHour", SimTypeShort(signed=False, label="UInt16")), ("wMinute", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")), ("wMilliseconds", SimTypeShort(signed=False, label="UInt16")),)), name="WCM_TIME_INTERVAL", pack=False, align=None),
    "WCM_USAGE_DATA": SimStruct(OrderedDict((("UsageInMegabytes", SimTypeInt(signed=False, label="UInt32")), ("LastSyncTime", SimTypeRef("FILETIME", SimStruct)),)), name="WCM_USAGE_DATA", pack=False, align=None),
    "WCN_ATTRIBUTE_TYPE": SimTypeInt(signed=False, label="WCN_ATTRIBUTE_TYPE"),
    "WCN_PASSWORD_TYPE": SimTypeInt(signed=False, label="WCN_PASSWORD_TYPE"),
    "WCN_SESSION_STATUS": SimTypeInt(signed=False, label="WCN_SESSION_STATUS"),
    "WCN_VALUE_TYPE_ASSOCIATION_STATE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_ASSOCIATION_STATE"),
    "WCN_VALUE_TYPE_AUTHENTICATION_TYPE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_AUTHENTICATION_TYPE"),
    "WCN_VALUE_TYPE_BOOLEAN": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_BOOLEAN"),
    "WCN_VALUE_TYPE_CONFIGURATION_ERROR": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_CONFIGURATION_ERROR"),
    "WCN_VALUE_TYPE_CONFIG_METHODS": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_CONFIG_METHODS"),
    "WCN_VALUE_TYPE_CONNECTION_TYPE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_CONNECTION_TYPE"),
    "WCN_VALUE_TYPE_DEVICE_PASSWORD_ID": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_DEVICE_PASSWORD_ID"),
    "WCN_VALUE_TYPE_ENCRYPTION_TYPE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_ENCRYPTION_TYPE"),
    "WCN_VALUE_TYPE_MESSAGE_TYPE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_MESSAGE_TYPE"),
    "WCN_VALUE_TYPE_PRIMARY_DEVICE_TYPE": SimStruct(OrderedDict((("Category", SimTypeShort(signed=False, label="UInt16")), ("SubCategoryOUI", SimTypeInt(signed=False, label="UInt32")), ("SubCategory", SimTypeShort(signed=False, label="UInt16")),)), name="WCN_VALUE_TYPE_PRIMARY_DEVICE_TYPE", pack=False, align=None),
    "WCN_VALUE_TYPE_REQUEST_TYPE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_REQUEST_TYPE"),
    "WCN_VALUE_TYPE_RESPONSE_TYPE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_RESPONSE_TYPE"),
    "WCN_VALUE_TYPE_RF_BANDS": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_RF_BANDS"),
    "WCN_VALUE_TYPE_VERSION": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_VERSION"),
    "WCN_VALUE_TYPE_WI_FI_PROTECTED_SETUP_STATE": SimTypeInt(signed=False, label="WCN_VALUE_TYPE_WI_FI_PROTECTED_SETUP_STATE"),
    "WCN_VENDOR_EXTENSION_SPEC": SimStruct(OrderedDict((("VendorId", SimTypeInt(signed=False, label="UInt32")), ("SubType", SimTypeInt(signed=False, label="UInt32")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="WCN_VENDOR_EXTENSION_SPEC", pack=False, align=None),
    "WCRANGE": SimStruct(OrderedDict((("wcLow", SimTypeChar(label="Char")), ("cGlyphs", SimTypeShort(signed=False, label="UInt16")),)), name="WCRANGE", pack=False, align=None),
    "WCRUN": SimStruct(OrderedDict((("wcLow", SimTypeChar(label="Char")), ("cGlyphs", SimTypeShort(signed=False, label="UInt16")), ("phg", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="WCRUN", pack=False, align=None),
    "WCS_DEVICE_CAPABILITIES_TYPE": SimTypeInt(signed=False, label="WCS_DEVICE_CAPABILITIES_TYPE"),
    "WCS_DEVICE_MHC2_CAPABILITIES": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("SupportsMhc2", SimTypeInt(signed=True, label="Int32")), ("RegammaLutEntryCount", SimTypeInt(signed=False, label="UInt32")), ("CscXyzMatrixRows", SimTypeInt(signed=False, label="UInt32")), ("CscXyzMatrixColumns", SimTypeInt(signed=False, label="UInt32")),)), name="WCS_DEVICE_MHC2_CAPABILITIES", pack=False, align=None),
    "WCS_DEVICE_VCGT_CAPABILITIES": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("SupportsVcgt", SimTypeInt(signed=True, label="Int32")),)), name="WCS_DEVICE_VCGT_CAPABILITIES", pack=False, align=None),
    "WCS_PROFILE_MANAGEMENT_SCOPE": SimTypeInt(signed=False, label="WCS_PROFILE_MANAGEMENT_SCOPE"),
    "WCT_OBJECT_STATUS": SimTypeInt(signed=False, label="WCT_OBJECT_STATUS"),
    "WCT_OBJECT_TYPE": SimTypeInt(signed=False, label="WCT_OBJECT_TYPE"),
    "WDBGEXTS_CLR_DATA_INTERFACE": SimStruct(OrderedDict((("Iid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("Iface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WDBGEXTS_CLR_DATA_INTERFACE", pack=False, align=None),
    "WDBGEXTS_DISASSEMBLE_BUFFER": SimStruct(OrderedDict((("InOffset", SimTypeLongLong(signed=False, label="UInt64")), ("OutOffset", SimTypeLongLong(signed=False, label="UInt64")), ("AddrFlags", SimTypeInt(signed=False, label="UInt32")), ("FormatFlags", SimTypeInt(signed=False, label="UInt32")), ("DataBufferBytes", SimTypeInt(signed=False, label="UInt32")), ("DisasmBufferChars", SimTypeInt(signed=False, label="UInt32")), ("DataBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("DisasmBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Reserved0", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 3)),)), name="WDBGEXTS_DISASSEMBLE_BUFFER", pack=False, align=None),
    "WDBGEXTS_MODULE_IN_RANGE": SimStruct(OrderedDict((("Start", SimTypeLongLong(signed=False, label="UInt64")), ("End", SimTypeLongLong(signed=False, label="UInt64")), ("FoundModBase", SimTypeLongLong(signed=False, label="UInt64")), ("FoundModSize", SimTypeInt(signed=False, label="UInt32")),)), name="WDBGEXTS_MODULE_IN_RANGE", pack=False, align=None),
    "WDBGEXTS_QUERY_INTERFACE": SimStruct(OrderedDict((("Iid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("Iface", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WDBGEXTS_QUERY_INTERFACE", pack=False, align=None),
    "WDBGEXTS_THREAD_OS_INFO": SimStruct(OrderedDict((("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("PriorityClass", SimTypeInt(signed=False, label="UInt32")), ("Priority", SimTypeInt(signed=False, label="UInt32")), ("CreateTime", SimTypeLongLong(signed=False, label="UInt64")), ("ExitTime", SimTypeLongLong(signed=False, label="UInt64")), ("KernelTime", SimTypeLongLong(signed=False, label="UInt64")), ("UserTime", SimTypeLongLong(signed=False, label="UInt64")), ("StartOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Affinity", SimTypeLongLong(signed=False, label="UInt64")),)), name="WDBGEXTS_THREAD_OS_INFO", pack=False, align=None),
    "WDD": SimStruct(OrderedDict((("wDispPos", SimTypeShort(signed=False, label="UInt16")), ("Anonymous1", SimUnion({"wReadPos": SimTypeShort(signed=False, label="UInt16"), "wCompPos": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("cchDisp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous2", SimUnion({"cchRead": SimTypeShort(signed=False, label="UInt16"), "cchComp": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("WDD_nReserve1", SimTypeInt(signed=False, label="UInt32")), ("nPos", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeShort(signed=False, label="UInt16")), ("pReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WDD", pack=False, align=None),
    "WDIAG_IHV_WLAN_ID": SimStruct(OrderedDict((("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReasonCode", SimTypeInt(signed=False, label="UInt32")),)), name="WDIAG_IHV_WLAN_ID", pack=False, align=None),
    "WDMUSB_POWER_STATE": SimTypeInt(signed=False, label="WDMUSB_POWER_STATE"),
    "WDSTRANSPORT_DIAGNOSTICS_COMPONENT_FLAGS": SimTypeInt(signed=False, label="WDSTRANSPORT_DIAGNOSTICS_COMPONENT_FLAGS"),
    "WDSTRANSPORT_DISCONNECT_TYPE": SimTypeInt(signed=False, label="WDSTRANSPORT_DISCONNECT_TYPE"),
    "WDSTRANSPORT_FEATURE_FLAGS": SimTypeInt(signed=False, label="WDSTRANSPORT_FEATURE_FLAGS"),
    "WDSTRANSPORT_IP_ADDRESS_SOURCE_TYPE": SimTypeInt(signed=False, label="WDSTRANSPORT_IP_ADDRESS_SOURCE_TYPE"),
    "WDSTRANSPORT_IP_ADDRESS_TYPE": SimTypeInt(signed=False, label="WDSTRANSPORT_IP_ADDRESS_TYPE"),
    "WDSTRANSPORT_NAMESPACE_TYPE": SimTypeInt(signed=False, label="WDSTRANSPORT_NAMESPACE_TYPE"),
    "WDSTRANSPORT_NETWORK_PROFILE_TYPE": SimTypeInt(signed=False, label="WDSTRANSPORT_NETWORK_PROFILE_TYPE"),
    "WDSTRANSPORT_PROTOCOL_FLAGS": SimTypeInt(signed=False, label="WDSTRANSPORT_PROTOCOL_FLAGS"),
    "WDSTRANSPORT_SERVICE_NOTIFICATION": SimTypeInt(signed=False, label="WDSTRANSPORT_SERVICE_NOTIFICATION"),
    "WDSTRANSPORT_SLOW_CLIENT_HANDLING_TYPE": SimTypeInt(signed=False, label="WDSTRANSPORT_SLOW_CLIENT_HANDLING_TYPE"),
    "WDSTRANSPORT_TFTP_CAPABILITY": SimTypeInt(signed=False, label="WDSTRANSPORT_TFTP_CAPABILITY"),
    "WDSTRANSPORT_UDP_PORT_POLICY": SimTypeInt(signed=False, label="WDSTRANSPORT_UDP_PORT_POLICY"),
    "WDS_CLI_CRED": SimStruct(OrderedDict((("pwszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WDS_CLI_CRED", pack=False, align=None),
    "WDS_CLI_FIRMWARE_TYPE": SimTypeInt(signed=False, label="WDS_CLI_FIRMWARE_TYPE"),
    "WDS_CLI_IMAGE_PARAM_TYPE": SimTypeInt(signed=False, label="WDS_CLI_IMAGE_PARAM_TYPE"),
    "WDS_CLI_IMAGE_TYPE": SimTypeInt(signed=False, label="WDS_CLI_IMAGE_TYPE"),
    "WDS_TRANSPORTCLIENT_CALLBACKS": SimStruct(OrderedDict((("SessionStart", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "ullFileSize"]), offset=0)), ("SessionStartEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TRANSPORTCLIENT_SESSION_INFO", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "Info"]), offset=0)), ("ReceiveContents", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "pContents", "ulSize", "pullContentOffset"]), offset=0)), ("ReceiveMetadata", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "pMetadata", "ulSize"]), offset=0)), ("SessionComplete", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "dwError"]), offset=0)), ("SessionNegotiate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("TRANSPORTCLIENT_SESSION_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hSessionKey", "pCallerData", "pInfo", "hNegotiateKey"]), offset=0)),)), name="WDS_TRANSPORTCLIENT_CALLBACKS", pack=False, align=None),
    "WDS_TRANSPORTCLIENT_REQUEST": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("ulApiVersion", SimTypeInt(signed=False, label="UInt32")), ("ulAuthLevel", SimTypeInt(signed=False, label="WDS_TRANSPORTCLIENT_REQUEST_AUTH_LEVEL")), ("pwszServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszNamespace", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszObjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ulCacheSize", SimTypeInt(signed=False, label="UInt32")), ("ulProtocol", SimTypeInt(signed=False, label="UInt32")), ("pvProtocolData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulProtocolDataLength", SimTypeInt(signed=False, label="UInt32")),)), name="WDS_TRANSPORTCLIENT_REQUEST", pack=False, align=None),
    "WDS_TRANSPORTCLIENT_REQUEST_AUTH_LEVEL": SimTypeInt(signed=False, label="WDS_TRANSPORTCLIENT_REQUEST_AUTH_LEVEL"),
    "WDS_TRANSPORTPROVIDER_INIT_PARAMS": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("ulMcServerVersion", SimTypeInt(signed=False, label="UInt32")), ("hRegistryKey", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hProvider", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WDS_TRANSPORTPROVIDER_INIT_PARAMS", pack=False, align=None),
    "WDS_TRANSPORTPROVIDER_SETTINGS": SimStruct(OrderedDict((("ulLength", SimTypeInt(signed=False, label="UInt32")), ("ulProviderVersion", SimTypeInt(signed=False, label="UInt32")),)), name="WDS_TRANSPORTPROVIDER_SETTINGS", pack=False, align=None),
    "WEBAUTHN_ASSERTION": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbAuthenticatorData", SimTypeInt(signed=False, label="UInt32")), ("pbAuthenticatorData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbSignature", SimTypeInt(signed=False, label="UInt32")), ("pbSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Credential", SimTypeRef("WEBAUTHN_CREDENTIAL", SimStruct)), ("cbUserId", SimTypeInt(signed=False, label="UInt32")), ("pbUserId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Extensions", SimTypeRef("WEBAUTHN_EXTENSIONS", SimStruct)), ("cbCredLargeBlob", SimTypeInt(signed=False, label="UInt32")), ("pbCredLargeBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwCredLargeBlobStatus", SimTypeInt(signed=False, label="UInt32")), ("pHmacSecret", SimTypePointer(SimTypeRef("WEBAUTHN_HMAC_SECRET_SALT", SimStruct), offset=0)), ("dwUsedTransport", SimTypeInt(signed=False, label="UInt32")), ("cbUnsignedExtensionOutputs", SimTypeInt(signed=False, label="UInt32")), ("pbUnsignedExtensionOutputs", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_ASSERTION", pack=False, align=None),
    "WEBAUTHN_AUTHENTICATOR_GET_ASSERTION_OPTIONS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutMilliseconds", SimTypeInt(signed=False, label="UInt32")), ("CredentialList", SimTypeRef("WEBAUTHN_CREDENTIALS", SimStruct)), ("Extensions", SimTypeRef("WEBAUTHN_EXTENSIONS", SimStruct)), ("dwAuthenticatorAttachment", SimTypeInt(signed=False, label="UInt32")), ("dwUserVerificationRequirement", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pwszU2fAppId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pbU2fAppId", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)), ("pCancellationId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pAllowCredentialList", SimTypePointer(SimTypeRef("WEBAUTHN_CREDENTIAL_LIST", SimStruct), offset=0)), ("dwCredLargeBlobOperation", SimTypeInt(signed=False, label="UInt32")), ("cbCredLargeBlob", SimTypeInt(signed=False, label="UInt32")), ("pbCredLargeBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHmacSecretSaltValues", SimTypePointer(SimTypeRef("WEBAUTHN_HMAC_SECRET_SALT_VALUES", SimStruct), offset=0)), ("bBrowserInPrivateMode", SimTypeInt(signed=True, label="Int32")), ("pLinkedDevice", SimTypePointer(SimTypeRef("CTAPCBOR_HYBRID_STORAGE_LINKED_DATA", SimStruct), offset=0)), ("bAutoFill", SimTypeInt(signed=True, label="Int32")), ("cbJsonExt", SimTypeInt(signed=False, label="UInt32")), ("pbJsonExt", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_AUTHENTICATOR_GET_ASSERTION_OPTIONS", pack=False, align=None),
    "WEBAUTHN_AUTHENTICATOR_MAKE_CREDENTIAL_OPTIONS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("dwTimeoutMilliseconds", SimTypeInt(signed=False, label="UInt32")), ("CredentialList", SimTypeRef("WEBAUTHN_CREDENTIALS", SimStruct)), ("Extensions", SimTypeRef("WEBAUTHN_EXTENSIONS", SimStruct)), ("dwAuthenticatorAttachment", SimTypeInt(signed=False, label="UInt32")), ("bRequireResidentKey", SimTypeInt(signed=True, label="Int32")), ("dwUserVerificationRequirement", SimTypeInt(signed=False, label="UInt32")), ("dwAttestationConveyancePreference", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pCancellationId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("pExcludeCredentialList", SimTypePointer(SimTypeRef("WEBAUTHN_CREDENTIAL_LIST", SimStruct), offset=0)), ("dwEnterpriseAttestation", SimTypeInt(signed=False, label="UInt32")), ("dwLargeBlobSupport", SimTypeInt(signed=False, label="UInt32")), ("bPreferResidentKey", SimTypeInt(signed=True, label="Int32")), ("bBrowserInPrivateMode", SimTypeInt(signed=True, label="Int32")), ("bEnablePrf", SimTypeInt(signed=True, label="Int32")), ("pLinkedDevice", SimTypePointer(SimTypeRef("CTAPCBOR_HYBRID_STORAGE_LINKED_DATA", SimStruct), offset=0)), ("cbJsonExt", SimTypeInt(signed=False, label="UInt32")), ("pbJsonExt", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_AUTHENTICATOR_MAKE_CREDENTIAL_OPTIONS", pack=False, align=None),
    "WEBAUTHN_CLIENT_DATA": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbClientDataJSON", SimTypeInt(signed=False, label="UInt32")), ("pbClientDataJSON", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszHashAlgId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WEBAUTHN_CLIENT_DATA", pack=False, align=None),
    "WEBAUTHN_COMMON_ATTESTATION": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszAlg", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lAlg", SimTypeInt(signed=True, label="Int32")), ("cbSignature", SimTypeInt(signed=False, label="UInt32")), ("pbSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cX5c", SimTypeInt(signed=False, label="UInt32")), ("pX5c", SimTypePointer(SimTypeRef("WEBAUTHN_X5C", SimStruct), offset=0)), ("pwszVer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbCertInfo", SimTypeInt(signed=False, label="UInt32")), ("pbCertInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbPubArea", SimTypeInt(signed=False, label="UInt32")), ("pbPubArea", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_COMMON_ATTESTATION", pack=False, align=None),
    "WEBAUTHN_COSE_CREDENTIAL_PARAMETER": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszCredentialType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lAlg", SimTypeInt(signed=True, label="Int32")),)), name="WEBAUTHN_COSE_CREDENTIAL_PARAMETER", pack=False, align=None),
    "WEBAUTHN_COSE_CREDENTIAL_PARAMETERS": SimStruct(OrderedDict((("cCredentialParameters", SimTypeInt(signed=False, label="UInt32")), ("pCredentialParameters", SimTypePointer(SimTypeRef("WEBAUTHN_COSE_CREDENTIAL_PARAMETER", SimStruct), offset=0)),)), name="WEBAUTHN_COSE_CREDENTIAL_PARAMETERS", pack=False, align=None),
    "WEBAUTHN_CREDENTIAL": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbId", SimTypeInt(signed=False, label="UInt32")), ("pbId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszCredentialType", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WEBAUTHN_CREDENTIAL", pack=False, align=None),
    "WEBAUTHN_CREDENTIALS": SimStruct(OrderedDict((("cCredentials", SimTypeInt(signed=False, label="UInt32")), ("pCredentials", SimTypePointer(SimTypeRef("WEBAUTHN_CREDENTIAL", SimStruct), offset=0)),)), name="WEBAUTHN_CREDENTIALS", pack=False, align=None),
    "WEBAUTHN_CREDENTIAL_ATTESTATION": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszFormatType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbAuthenticatorData", SimTypeInt(signed=False, label="UInt32")), ("pbAuthenticatorData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbAttestation", SimTypeInt(signed=False, label="UInt32")), ("pbAttestation", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwAttestationDecodeType", SimTypeInt(signed=False, label="UInt32")), ("pvAttestationDecode", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbAttestationObject", SimTypeInt(signed=False, label="UInt32")), ("pbAttestationObject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCredentialId", SimTypeInt(signed=False, label="UInt32")), ("pbCredentialId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Extensions", SimTypeRef("WEBAUTHN_EXTENSIONS", SimStruct)), ("dwUsedTransport", SimTypeInt(signed=False, label="UInt32")), ("bEpAtt", SimTypeInt(signed=True, label="Int32")), ("bLargeBlobSupported", SimTypeInt(signed=True, label="Int32")), ("bResidentKey", SimTypeInt(signed=True, label="Int32")), ("bPrfEnabled", SimTypeInt(signed=True, label="Int32")), ("cbUnsignedExtensionOutputs", SimTypeInt(signed=False, label="UInt32")), ("pbUnsignedExtensionOutputs", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_CREDENTIAL_ATTESTATION", pack=False, align=None),
    "WEBAUTHN_CREDENTIAL_DETAILS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbCredentialID", SimTypeInt(signed=False, label="UInt32")), ("pbCredentialID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pRpInformation", SimTypePointer(SimTypeRef("WEBAUTHN_RP_ENTITY_INFORMATION", SimStruct), offset=0)), ("pUserInformation", SimTypePointer(SimTypeRef("WEBAUTHN_USER_ENTITY_INFORMATION", SimStruct), offset=0)), ("bRemovable", SimTypeInt(signed=True, label="Int32")), ("bBackedUp", SimTypeInt(signed=True, label="Int32")),)), name="WEBAUTHN_CREDENTIAL_DETAILS", pack=False, align=None),
    "WEBAUTHN_CREDENTIAL_DETAILS_LIST": SimStruct(OrderedDict((("cCredentialDetails", SimTypeInt(signed=False, label="UInt32")), ("ppCredentialDetails", SimTypePointer(SimTypePointer(SimTypeRef("WEBAUTHN_CREDENTIAL_DETAILS", SimStruct), offset=0), offset=0)),)), name="WEBAUTHN_CREDENTIAL_DETAILS_LIST", pack=False, align=None),
    "WEBAUTHN_CREDENTIAL_EX": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbId", SimTypeInt(signed=False, label="UInt32")), ("pbId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszCredentialType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwTransports", SimTypeInt(signed=False, label="UInt32")),)), name="WEBAUTHN_CREDENTIAL_EX", pack=False, align=None),
    "WEBAUTHN_CREDENTIAL_LIST": SimStruct(OrderedDict((("cCredentials", SimTypeInt(signed=False, label="UInt32")), ("ppCredentials", SimTypePointer(SimTypePointer(SimTypeRef("WEBAUTHN_CREDENTIAL_EX", SimStruct), offset=0), offset=0)),)), name="WEBAUTHN_CREDENTIAL_LIST", pack=False, align=None),
    "WEBAUTHN_CRED_BLOB_EXTENSION": SimStruct(OrderedDict((("cbCredBlob", SimTypeInt(signed=False, label="UInt32")), ("pbCredBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_CRED_BLOB_EXTENSION", pack=False, align=None),
    "WEBAUTHN_CRED_PROTECT_EXTENSION_IN": SimStruct(OrderedDict((("dwCredProtect", SimTypeInt(signed=False, label="UInt32")), ("bRequireCredProtect", SimTypeInt(signed=True, label="Int32")),)), name="WEBAUTHN_CRED_PROTECT_EXTENSION_IN", pack=False, align=None),
    "WEBAUTHN_CRED_WITH_HMAC_SECRET_SALT": SimStruct(OrderedDict((("cbCredID", SimTypeInt(signed=False, label="UInt32")), ("pbCredID", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHmacSecretSalt", SimTypePointer(SimTypeRef("WEBAUTHN_HMAC_SECRET_SALT", SimStruct), offset=0)),)), name="WEBAUTHN_CRED_WITH_HMAC_SECRET_SALT", pack=False, align=None),
    "WEBAUTHN_EXTENSION": SimStruct(OrderedDict((("pwszExtensionIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbExtension", SimTypeInt(signed=False, label="UInt32")), ("pvExtension", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WEBAUTHN_EXTENSION", pack=False, align=None),
    "WEBAUTHN_EXTENSIONS": SimStruct(OrderedDict((("cExtensions", SimTypeInt(signed=False, label="UInt32")), ("pExtensions", SimTypePointer(SimTypeRef("WEBAUTHN_EXTENSION", SimStruct), offset=0)),)), name="WEBAUTHN_EXTENSIONS", pack=False, align=None),
    "WEBAUTHN_GET_CREDENTIALS_OPTIONS": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszRpId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bBrowserInPrivateMode", SimTypeInt(signed=True, label="Int32")),)), name="WEBAUTHN_GET_CREDENTIALS_OPTIONS", pack=False, align=None),
    "WEBAUTHN_HMAC_SECRET_SALT": SimStruct(OrderedDict((("cbFirst", SimTypeInt(signed=False, label="UInt32")), ("pbFirst", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbSecond", SimTypeInt(signed=False, label="UInt32")), ("pbSecond", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_HMAC_SECRET_SALT", pack=False, align=None),
    "WEBAUTHN_HMAC_SECRET_SALT_VALUES": SimStruct(OrderedDict((("pGlobalHmacSalt", SimTypePointer(SimTypeRef("WEBAUTHN_HMAC_SECRET_SALT", SimStruct), offset=0)), ("cCredWithHmacSecretSaltList", SimTypeInt(signed=False, label="UInt32")), ("pCredWithHmacSecretSaltList", SimTypePointer(SimTypeRef("WEBAUTHN_CRED_WITH_HMAC_SECRET_SALT", SimStruct), offset=0)),)), name="WEBAUTHN_HMAC_SECRET_SALT_VALUES", pack=False, align=None),
    "WEBAUTHN_RP_ENTITY_INFORMATION": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIcon", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WEBAUTHN_RP_ENTITY_INFORMATION", pack=False, align=None),
    "WEBAUTHN_USER_ENTITY_INFORMATION": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbId", SimTypeInt(signed=False, label="UInt32")), ("pbId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszIcon", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WEBAUTHN_USER_ENTITY_INFORMATION", pack=False, align=None),
    "WEBAUTHN_X5C": SimStruct(OrderedDict((("cbData", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WEBAUTHN_X5C", pack=False, align=None),
    "WEBCRAWL_RECURSEFLAGS": SimTypeInt(signed=False, label="WEBCRAWL_RECURSEFLAGS"),
    "WEBSTREAM_SAMPLE_TYPE": SimTypeInt(signed=False, label="WEBSTREAM_SAMPLE_TYPE"),
    "WEB_SOCKET_ACTION": SimTypeInt(signed=False, label="WEB_SOCKET_ACTION"),
    "WEB_SOCKET_ACTION_QUEUE": SimTypeInt(signed=False, label="WEB_SOCKET_ACTION_QUEUE"),
    "WEB_SOCKET_BUFFER": SimUnion({"Data": SimTypeRef("_Data_e__Struct", SimStruct), "CloseStatus": SimTypeRef("_CloseStatus_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "WEB_SOCKET_BUFFER_TYPE": SimTypeInt(signed=False, label="WEB_SOCKET_BUFFER_TYPE"),
    "WEB_SOCKET_CLOSE_STATUS": SimTypeInt(signed=False, label="WEB_SOCKET_CLOSE_STATUS"),
    "WEB_SOCKET_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WEB_SOCKET_HTTP_HEADER": SimStruct(OrderedDict((("pcName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulNameLength", SimTypeInt(signed=False, label="UInt32")), ("pcValue", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulValueLength", SimTypeInt(signed=False, label="UInt32")),)), name="WEB_SOCKET_HTTP_HEADER", pack=False, align=None),
    "WEB_SOCKET_PROPERTY": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="WEB_SOCKET_PROPERTY_TYPE")), ("pvValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ulValueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WEB_SOCKET_PROPERTY", pack=False, align=None),
    "WEB_SOCKET_PROPERTY_TYPE": SimTypeInt(signed=False, label="WEB_SOCKET_PROPERTY_TYPE"),
    "WEEKLY": SimStruct(OrderedDict((("WeeksInterval", SimTypeShort(signed=False, label="UInt16")), ("rgfDaysOfTheWeek", SimTypeShort(signed=False, label="UInt16")),)), name="WEEKLY", pack=False, align=None),
    "WELL_KNOWN_SID_TYPE": SimTypeInt(signed=False, label="WELL_KNOWN_SID_TYPE"),
    "WER_CONSENT": SimTypeInt(signed=False, label="WER_CONSENT"),
    "WER_DUMP_CUSTOM_OPTIONS": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwDumpFlags", SimTypeInt(signed=False, label="UInt32")), ("bOnlyThisThread", SimTypeInt(signed=True, label="Int32")), ("dwExceptionThreadFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherThreadFlags", SimTypeInt(signed=False, label="UInt32")), ("dwExceptionThreadExFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherThreadExFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredModuleFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherModuleFlags", SimTypeInt(signed=False, label="UInt32")), ("wzPreferredModuleList", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="WER_DUMP_CUSTOM_OPTIONS", pack=False, align=None),
    "WER_DUMP_CUSTOM_OPTIONS_V2": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwDumpFlags", SimTypeInt(signed=False, label="UInt32")), ("bOnlyThisThread", SimTypeInt(signed=True, label="Int32")), ("dwExceptionThreadFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherThreadFlags", SimTypeInt(signed=False, label="UInt32")), ("dwExceptionThreadExFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherThreadExFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredModuleFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherModuleFlags", SimTypeInt(signed=False, label="UInt32")), ("wzPreferredModuleList", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwPreferredModuleResetFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherModuleResetFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WER_DUMP_CUSTOM_OPTIONS_V2", pack=False, align=None),
    "WER_DUMP_CUSTOM_OPTIONS_V3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")), ("dwDumpFlags", SimTypeInt(signed=False, label="UInt32")), ("bOnlyThisThread", SimTypeInt(signed=True, label="Int32")), ("dwExceptionThreadFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherThreadFlags", SimTypeInt(signed=False, label="UInt32")), ("dwExceptionThreadExFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherThreadExFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPreferredModuleFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherModuleFlags", SimTypeInt(signed=False, label="UInt32")), ("wzPreferredModuleList", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwPreferredModuleResetFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOtherModuleResetFlags", SimTypeInt(signed=False, label="UInt32")), ("pvDumpKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hSnapshot", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwThreadID", SimTypeInt(signed=False, label="UInt32")),)), name="WER_DUMP_CUSTOM_OPTIONS_V3", pack=False, align=None),
    "WER_DUMP_TYPE": SimTypeInt(signed=False, label="WER_DUMP_TYPE"),
    "WER_EXCEPTION_INFORMATION": SimStruct(OrderedDict((("pExceptionPointers", SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0)), ("bClientPointers", SimTypeInt(signed=True, label="Int32")),)), name="WER_EXCEPTION_INFORMATION", pack=False, align=None),
    "WER_FAULT_REPORTING": SimTypeInt(signed=False, label="WER_FAULT_REPORTING"),
    "WER_FILE": SimTypeInt(signed=False, label="WER_FILE"),
    "WER_FILE_TYPE": SimTypeInt(signed=False, label="WER_FILE_TYPE"),
    "WER_REGISTER_FILE_TYPE": SimTypeInt(signed=False, label="WER_REGISTER_FILE_TYPE"),
    "WER_REPORT_INFORMATION": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzConsentKey", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzFriendlyEventName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wzDescription", SimTypeArray(SimTypeChar(label="Char"), 512)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WER_REPORT_INFORMATION", pack=False, align=None),
    "WER_REPORT_INFORMATION_V3": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzConsentKey", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzFriendlyEventName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wzDescription", SimTypeArray(SimTypeChar(label="Char"), 512)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzNamespacePartner", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzNamespaceGroup", SimTypeArray(SimTypeChar(label="Char"), 64)),)), name="WER_REPORT_INFORMATION_V3", pack=False, align=None),
    "WER_REPORT_INFORMATION_V4": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzConsentKey", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzFriendlyEventName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wzDescription", SimTypeArray(SimTypeChar(label="Char"), 512)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzNamespacePartner", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzNamespaceGroup", SimTypeArray(SimTypeChar(label="Char"), 64)), ("rgbApplicationIdentity", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("hSnapshot", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDeleteFilesImpersonationToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WER_REPORT_INFORMATION_V4", pack=False, align=None),
    "WER_REPORT_INFORMATION_V5": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzConsentKey", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzFriendlyEventName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("wzApplicationPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("wzDescription", SimTypeArray(SimTypeChar(label="Char"), 512)), ("hwndParent", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("wzNamespacePartner", SimTypeArray(SimTypeChar(label="Char"), 64)), ("wzNamespaceGroup", SimTypeArray(SimTypeChar(label="Char"), 64)), ("rgbApplicationIdentity", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("hSnapshot", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDeleteFilesImpersonationToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("submitResultMax", SimTypeInt(signed=False, label="WER_SUBMIT_RESULT")),)), name="WER_REPORT_INFORMATION_V5", pack=False, align=None),
    "WER_REPORT_METADATA_V1": SimStruct(OrderedDict((("Signature", SimTypeRef("WER_REPORT_SIGNATURE", SimStruct)), ("BucketId", SimTypeBottom(label="Guid")), ("ReportId", SimTypeBottom(label="Guid")), ("CreationTime", SimTypeRef("FILETIME", SimStruct)), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="WER_REPORT_METADATA_V1", pack=False, align=None),
    "WER_REPORT_METADATA_V2": SimStruct(OrderedDict((("Signature", SimTypeRef("WER_REPORT_SIGNATURE", SimStruct)), ("BucketId", SimTypeBottom(label="Guid")), ("ReportId", SimTypeBottom(label="Guid")), ("CreationTime", SimTypeRef("FILETIME", SimStruct)), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("CabId", SimTypeArray(SimTypeChar(label="Char"), 260)), ("ReportStatus", SimTypeInt(signed=False, label="UInt32")), ("ReportIntegratorId", SimTypeBottom(label="Guid")), ("NumberOfFiles", SimTypeInt(signed=False, label="UInt32")), ("SizeOfFileNames", SimTypeInt(signed=False, label="UInt32")), ("FileNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WER_REPORT_METADATA_V2", pack=False, align=None),
    "WER_REPORT_METADATA_V3": SimStruct(OrderedDict((("Signature", SimTypeRef("WER_REPORT_SIGNATURE", SimStruct)), ("BucketId", SimTypeBottom(label="Guid")), ("ReportId", SimTypeBottom(label="Guid")), ("CreationTime", SimTypeRef("FILETIME", SimStruct)), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("CabId", SimTypeArray(SimTypeChar(label="Char"), 260)), ("ReportStatus", SimTypeInt(signed=False, label="UInt32")), ("ReportIntegratorId", SimTypeBottom(label="Guid")), ("NumberOfFiles", SimTypeInt(signed=False, label="UInt32")), ("SizeOfFileNames", SimTypeInt(signed=False, label="UInt32")), ("FileNames", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FriendlyEventName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("ApplicationName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("ApplicationPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("Description", SimTypeArray(SimTypeChar(label="Char"), 512)), ("BucketIdString", SimTypeArray(SimTypeChar(label="Char"), 260)), ("LegacyBucketId", SimTypeLongLong(signed=False, label="UInt64")),)), name="WER_REPORT_METADATA_V3", pack=False, align=None),
    "WER_REPORT_PARAMETER": SimStruct(OrderedDict((("Name", SimTypeArray(SimTypeChar(label="Char"), 129)), ("Value", SimTypeArray(SimTypeChar(label="Char"), 260)),)), name="WER_REPORT_PARAMETER", pack=False, align=None),
    "WER_REPORT_SIGNATURE": SimStruct(OrderedDict((("EventName", SimTypeArray(SimTypeChar(label="Char"), 65)), ("Parameters", SimTypeArray(SimTypeRef("WER_REPORT_PARAMETER", SimStruct), 10)),)), name="WER_REPORT_SIGNATURE", pack=False, align=None),
    "WER_REPORT_TYPE": SimTypeInt(signed=False, label="WER_REPORT_TYPE"),
    "WER_REPORT_UI": SimTypeInt(signed=False, label="WER_REPORT_UI"),
    "WER_RUNTIME_EXCEPTION_INFORMATION": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("exceptionRecord", SimTypeRef("EXCEPTION_RECORD", SimStruct)), ("context", SimTypeRef("CONTEXT", SimStruct)), ("pwszReportId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bIsFatal", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="WER_RUNTIME_EXCEPTION_INFORMATION", pack=False, align=None),
    "WER_SUBMIT_FLAGS": SimTypeInt(signed=False, label="WER_SUBMIT_FLAGS"),
    "WER_SUBMIT_RESULT": SimTypeInt(signed=False, label="WER_SUBMIT_RESULT"),
    "WFDSVC_CONNECTION_CAPABILITY": SimStruct(OrderedDict((("bNew", SimTypeChar(label="Byte")), ("bClient", SimTypeChar(label="Byte")), ("bGO", SimTypeChar(label="Byte")),)), name="WFDSVC_CONNECTION_CAPABILITY", pack=False, align=None),
    "WFD_GROUP_ID": SimStruct(OrderedDict((("DeviceAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("GroupSSID", SimTypeRef("DOT11_SSID", SimStruct)),)), name="WFD_GROUP_ID", pack=False, align=None),
    "WFD_OPEN_SESSION_COMPLETE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeBottom(label="Guid"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hSessionHandle", "pvContext", "guidSessionInterface", "dwError", "dwReasonCode"]), offset=0),
    "WFD_ROLE_TYPE": SimTypeInt(signed=False, label="WFD_ROLE_TYPE"),
    "WGLSWAP": SimStruct(OrderedDict((("hdc", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uiFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WGLSWAP", pack=False, align=None),
    "WHEA128A": SimStruct(OrderedDict((("Low", SimTypeLongLong(signed=False, label="UInt64")), ("High", SimTypeLongLong(signed=True, label="Int64")),)), name="WHEA128A", pack=False, align=None),
    "WHEAP_ACPI_TIMEOUT_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("TableType", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("TableRequest", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="WHEAP_ACPI_TIMEOUT_EVENT", pack=False, align=None),
    "WHEAP_ADD_REMOVE_ERROR_SOURCE_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Descriptor", SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct)), ("Status", SimTypeInt(signed=True, label="Int32")), ("IsRemove", SimTypeChar(label="Byte")),)), name="WHEAP_ADD_REMOVE_ERROR_SOURCE_EVENT", pack=False, align=None),
    "WHEAP_ATTEMPT_RECOVERY_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrorHeader", SimTypeRef("WHEA_ERROR_RECORD_HEADER", SimStruct)), ("ArchitecturalRecovery", SimTypeChar(label="Byte")), ("PshedRecovery", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEAP_ATTEMPT_RECOVERY_EVENT", pack=False, align=None),
    "WHEAP_BAD_HEST_NOTIFY_DATA_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("SourceId", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("NotifyDesc", SimTypeRef("WHEA_NOTIFICATION_DESCRIPTOR", SimStruct)),)), name="WHEAP_BAD_HEST_NOTIFY_DATA_EVENT", pack=False, align=None),
    "WHEAP_CLEARED_POISON_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("PhysicalAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEAP_CLEARED_POISON_EVENT", pack=False, align=None),
    "WHEAP_CMCI_IMPLEMENTED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("CmciAvailable", SimTypeChar(label="Byte")),)), name="WHEAP_CMCI_IMPLEMENTED_EVENT", pack=False, align=None),
    "WHEAP_CMCI_INITERR_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Msr", SimTypeLongLong(signed=False, label="UInt64")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Bank", SimTypeInt(signed=False, label="UInt32")), ("EpIndex", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_CMCI_INITERR_EVENT", pack=False, align=None),
    "WHEAP_CMCI_RESTART_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("CmciRestoreAttempts", SimTypeInt(signed=False, label="UInt32")), ("MaxCmciRestoreLimit", SimTypeInt(signed=False, label="UInt32")), ("MaxCorrectedErrorsFound", SimTypeInt(signed=False, label="UInt32")), ("MaxCorrectedErrorLimit", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_CMCI_RESTART_EVENT", pack=False, align=None),
    "WHEAP_CREATE_GENERIC_RECORD_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Error", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("EntryCount", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEAP_CREATE_GENERIC_RECORD_EVENT", pack=False, align=None),
    "WHEAP_DEFERRED_EVENT": SimStruct(OrderedDict((("ListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEAP_DEFERRED_EVENT", pack=False, align=None),
    "WHEAP_DEVICE_DRV_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Function", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="WHEAP_DEVICE_DRV_EVENT", pack=False, align=None),
    "WHEAP_DPC_ERROR_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrType", SimTypeInt(signed=False, label="WHEAP_DPC_ERROR_EVENT_TYPE")), ("Bus", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeShort(signed=False, label="UInt16")), ("VendorId", SimTypeShort(signed=False, label="UInt16")),)), name="WHEAP_DPC_ERROR_EVENT", pack=False, align=None),
    "WHEAP_DPC_ERROR_EVENT_TYPE": SimTypeInt(signed=False, label="WHEAP_DPC_ERROR_EVENT_TYPE"),
    "WHEAP_DROPPED_CORRECTED_ERROR_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrorSourceType", SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_TYPE")), ("ErrorSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_DROPPED_CORRECTED_ERROR_EVENT", pack=False, align=None),
    "WHEAP_EDPC_ENABLED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("eDPCEnabled", SimTypeChar(label="Byte")), ("eDPCRecovEnabled", SimTypeChar(label="Byte")),)), name="WHEAP_EDPC_ENABLED_EVENT", pack=False, align=None),
    "WHEAP_ERROR_CLEARED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("EpIndex", SimTypeInt(signed=False, label="UInt32")), ("Bank", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_ERROR_CLEARED_EVENT", pack=False, align=None),
    "WHEAP_ERROR_RECORD_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Record", SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)),)), name="WHEAP_ERROR_RECORD_EVENT", pack=False, align=None),
    "WHEAP_ERR_SRC_ARRAY_INVALID_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrorSourceCount", SimTypeInt(signed=False, label="UInt32")), ("ReportedLength", SimTypeInt(signed=False, label="UInt32")), ("ExpectedLength", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_ERR_SRC_ARRAY_INVALID_EVENT", pack=False, align=None),
    "WHEAP_ERR_SRC_INVALID_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrDescriptor", SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct)), ("Error", SimTypeArray(SimTypeChar(label="SByte"), 32)),)), name="WHEAP_ERR_SRC_INVALID_EVENT", pack=False, align=None),
    "WHEAP_FOUND_ERROR_IN_BANK_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("EpIndex", SimTypeInt(signed=False, label="UInt32")), ("Bank", SimTypeInt(signed=False, label="UInt32")), ("MciStatus", SimTypeLongLong(signed=False, label="UInt64")), ("ErrorType", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_FOUND_ERROR_IN_BANK_EVENT", pack=False, align=None),
    "WHEAP_GENERIC_ERR_MEM_MAP_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("MapReason", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("PhysicalAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEAP_GENERIC_ERR_MEM_MAP_EVENT", pack=False, align=None),
    "WHEAP_OSC_IMPLEMENTED": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("OscImplemented", SimTypeChar(label="Byte")), ("DebugChecked", SimTypeChar(label="Byte")),)), name="WHEAP_OSC_IMPLEMENTED", pack=False, align=None),
    "WHEAP_PCIE_CONFIG_INFO": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Segment", SimTypeInt(signed=False, label="UInt32")), ("Bus", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Value", SimTypeLongLong(signed=False, label="UInt64")), ("Succeeded", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="WHEAP_PCIE_CONFIG_INFO", pack=False, align=None),
    "WHEAP_PCIE_OVERRIDE_INFO": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Segment", SimTypeInt(signed=False, label="UInt32")), ("Bus", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeInt(signed=False, label="UInt32")), ("ValidBits", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("UncorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("UncorrectableErrorSeverity", SimTypeInt(signed=False, label="UInt32")), ("CorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("CapAndControl", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_PCIE_OVERRIDE_INFO", pack=False, align=None),
    "WHEAP_PCIE_READ_OVERRIDES_ERR": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("FailureReason", SimTypeInt(signed=False, label="UInt32")), ("FailureStatus", SimTypeInt(signed=True, label="Int32")),)), name="WHEAP_PCIE_READ_OVERRIDES_ERR", pack=False, align=None),
    "WHEAP_PFA_MEMORY_OFFLINED": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("DecisionType", SimTypeInt(signed=False, label="WHEAP_PFA_OFFLINE_DECISION_TYPE")), ("ImmediateSuccess", SimTypeChar(label="Byte")), ("Page", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_PFA_MEMORY_OFFLINED", pack=False, align=None),
    "WHEAP_PFA_MEMORY_POLICY": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("RegistryKeysPresent", SimTypeInt(signed=False, label="UInt32")), ("DisableOffline", SimTypeChar(label="Byte")), ("PersistOffline", SimTypeChar(label="Byte")), ("PfaDisabled", SimTypeChar(label="Byte")), ("PageCount", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("TimeOut", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_PFA_MEMORY_POLICY", pack=False, align=None),
    "WHEAP_PFA_MEMORY_REMOVE_MONITOR": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("RemoveTrigger", SimTypeInt(signed=False, label="WHEA_PFA_REMOVE_TRIGGER")), ("TimeInList", SimTypeInt(signed=False, label="UInt32")), ("ErrorCount", SimTypeInt(signed=False, label="UInt32")), ("Page", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_PFA_MEMORY_REMOVE_MONITOR", pack=False, align=None),
    "WHEAP_PFA_OFFLINE_DECISION_TYPE": SimTypeInt(signed=False, label="WHEAP_PFA_OFFLINE_DECISION_TYPE"),
    "WHEAP_PLUGIN_DEFECT_LIST_CORRUPT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEAP_PLUGIN_DEFECT_LIST_CORRUPT", pack=False, align=None),
    "WHEAP_PLUGIN_DEFECT_LIST_FULL_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEAP_PLUGIN_DEFECT_LIST_FULL_EVENT", pack=False, align=None),
    "WHEAP_PLUGIN_DEFECT_LIST_UEFI_VAR_FAILED": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEAP_PLUGIN_DEFECT_LIST_UEFI_VAR_FAILED", pack=False, align=None),
    "WHEAP_PLUGIN_PFA_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("NoFurtherPfa", SimTypeChar(label="Byte")),)), name="WHEAP_PLUGIN_PFA_EVENT", pack=False, align=None),
    "WHEAP_PROCESS_EINJ_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Error", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("InjectionActionTableValid", SimTypeChar(label="Byte")), ("BeginInjectionInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GetTriggerErrorActionTableInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("SetErrorTypeInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GetErrorTypeInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("EndOperationInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("ExecuteOperationInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("CheckBusyStatusInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GetCommandStatusInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("SetErrorTypeWithAddressInstructionCount", SimTypeInt(signed=False, label="UInt32")), ("GetExecuteOperationTimingsInstructionCount", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_PROCESS_EINJ_EVENT", pack=False, align=None),
    "WHEAP_PROCESS_HEST_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Error", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("EntryType", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("EntryIndex", SimTypeInt(signed=False, label="UInt32")), ("HestValid", SimTypeChar(label="Byte")), ("CmcCount", SimTypeInt(signed=False, label="UInt32")), ("MceCount", SimTypeInt(signed=False, label="UInt32")), ("NmiCount", SimTypeInt(signed=False, label="UInt32")), ("AerRootCount", SimTypeInt(signed=False, label="UInt32")), ("AerBridgeCount", SimTypeInt(signed=False, label="UInt32")), ("AerEndPointCount", SimTypeInt(signed=False, label="UInt32")), ("GenericV1Count", SimTypeInt(signed=False, label="UInt32")), ("GenericV2Count", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_PROCESS_HEST_EVENT", pack=False, align=None),
    "WHEAP_PSHED_INJECT_ERROR": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrorType", SimTypeInt(signed=False, label="UInt32")), ("Parameter1", SimTypeLongLong(signed=False, label="UInt64")), ("Parameter2", SimTypeLongLong(signed=False, label="UInt64")), ("Parameter3", SimTypeLongLong(signed=False, label="UInt64")), ("Parameter4", SimTypeLongLong(signed=False, label="UInt64")), ("InjectionStatus", SimTypeInt(signed=True, label="Int32")), ("InjectionAttempted", SimTypeChar(label="Byte")), ("InjectionByPlugin", SimTypeChar(label="Byte")),)), name="WHEAP_PSHED_INJECT_ERROR", pack=False, align=None),
    "WHEAP_PSHED_PLUGIN_REGISTER": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("FunctionalAreaMask", SimTypeInt(signed=False, label="UInt32")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEAP_PSHED_PLUGIN_REGISTER", pack=False, align=None),
    "WHEAP_ROW_FAILURE_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("LowOrderPage", SimTypeInt(signed=False, label="UInt32")), ("HighOrderPage", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_ROW_FAILURE_EVENT", pack=False, align=None),
    "WHEAP_SPURIOUS_AER_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrorSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("ErrorHandlerType", SimTypeInt(signed=False, label="UInt32")), ("SpuriousErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("RootErrorCommand", SimTypeInt(signed=False, label="UInt32")), ("RootErrorStatus", SimTypeInt(signed=False, label="UInt32")), ("DeviceAssociationBitmap", SimTypeInt(signed=False, label="UInt32")),)), name="WHEAP_SPURIOUS_AER_EVENT", pack=False, align=None),
    "WHEAP_STARTED_REPORT_HW_ERROR": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ErrorPacket", SimTypePointer(SimTypeRef("WHEA_ERROR_PACKET_V2", SimStruct), offset=0)),)), name="WHEAP_STARTED_REPORT_HW_ERROR", pack=False, align=None),
    "WHEAP_STUCK_ERROR_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("EpIndex", SimTypeInt(signed=False, label="UInt32")), ("Bank", SimTypeInt(signed=False, label="UInt32")), ("MciStatus", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEAP_STUCK_ERROR_EVENT", pack=False, align=None),
    "WHEA_ACPI_HEADER": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimTypeChar(label="Byte")), ("Checksum", SimTypeChar(label="Byte")), ("OemId", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("OemTableId", SimTypeLongLong(signed=False, label="UInt64")), ("OemRevision", SimTypeInt(signed=False, label="UInt32")), ("CreatorId", SimTypeInt(signed=False, label="UInt32")), ("CreatorRevision", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ACPI_HEADER", pack=False, align=None),
    "WHEA_AER_BRIDGE_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeRef("WHEA_PCI_SLOT_NUMBER", SimStruct)), ("DeviceControl", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("UncorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("UncorrectableErrorSeverity", SimTypeInt(signed=False, label="UInt32")), ("CorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("AdvancedCapsAndControl", SimTypeInt(signed=False, label="UInt32")), ("SecondaryUncorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("SecondaryUncorrectableErrorSev", SimTypeInt(signed=False, label="UInt32")), ("SecondaryCapsAndControl", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_AER_BRIDGE_DESCRIPTOR", pack=False, align=None),
    "WHEA_AER_ENDPOINT_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeRef("WHEA_PCI_SLOT_NUMBER", SimStruct)), ("DeviceControl", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("UncorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("UncorrectableErrorSeverity", SimTypeInt(signed=False, label="UInt32")), ("CorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("AdvancedCapsAndControl", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_AER_ENDPOINT_DESCRIPTOR", pack=False, align=None),
    "WHEA_AER_ROOTPORT_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("BusNumber", SimTypeInt(signed=False, label="UInt32")), ("Slot", SimTypeRef("WHEA_PCI_SLOT_NUMBER", SimStruct)), ("DeviceControl", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("UncorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("UncorrectableErrorSeverity", SimTypeInt(signed=False, label="UInt32")), ("CorrectableErrorMask", SimTypeInt(signed=False, label="UInt32")), ("AdvancedCapsAndControl", SimTypeInt(signed=False, label="UInt32")), ("RootErrorCommand", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_AER_ROOTPORT_DESCRIPTOR", pack=False, align=None),
    "WHEA_AMD_EXTENDED_REGISTERS": SimStruct(OrderedDict((("IPID", SimTypeLongLong(signed=False, label="UInt64")), ("SYND", SimTypeLongLong(signed=False, label="UInt64")), ("CONFIG", SimTypeLongLong(signed=False, label="UInt64")), ("DESTAT", SimTypeLongLong(signed=False, label="UInt64")), ("DEADDR", SimTypeLongLong(signed=False, label="UInt64")), ("MISC1", SimTypeLongLong(signed=False, label="UInt64")), ("MISC2", SimTypeLongLong(signed=False, label="UInt64")), ("MISC3", SimTypeLongLong(signed=False, label="UInt64")), ("MISC4", SimTypeLongLong(signed=False, label="UInt64")), ("RasCap", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 14)),)), name="WHEA_AMD_EXTENDED_REGISTERS", pack=False, align=None),
    "WHEA_ARMV8_AARCH32_GPRS": SimStruct(OrderedDict((("R0", SimTypeInt(signed=False, label="UInt32")), ("R1", SimTypeInt(signed=False, label="UInt32")), ("R2", SimTypeInt(signed=False, label="UInt32")), ("R3", SimTypeInt(signed=False, label="UInt32")), ("R4", SimTypeInt(signed=False, label="UInt32")), ("R5", SimTypeInt(signed=False, label="UInt32")), ("R6", SimTypeInt(signed=False, label="UInt32")), ("R7", SimTypeInt(signed=False, label="UInt32")), ("R8", SimTypeInt(signed=False, label="UInt32")), ("R9", SimTypeInt(signed=False, label="UInt32")), ("R10", SimTypeInt(signed=False, label="UInt32")), ("R11", SimTypeInt(signed=False, label="UInt32")), ("R12", SimTypeInt(signed=False, label="UInt32")), ("R13", SimTypeInt(signed=False, label="UInt32")), ("R14", SimTypeInt(signed=False, label="UInt32")), ("R15", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ARMV8_AARCH32_GPRS", pack=False, align=None),
    "WHEA_ARMV8_AARCH64_EL3_CSR": SimStruct(OrderedDict((("ELR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("ESR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("FAR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("MAIR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("SCTLR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("SP_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("SPSR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("TCR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("TPIDR_EL3", SimTypeLongLong(signed=False, label="UInt64")), ("TTBR0_EL3", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARMV8_AARCH64_EL3_CSR", pack=False, align=None),
    "WHEA_ARMV8_AARCH64_GPRS": SimStruct(OrderedDict((("X0", SimTypeLongLong(signed=False, label="UInt64")), ("X1", SimTypeLongLong(signed=False, label="UInt64")), ("X2", SimTypeLongLong(signed=False, label="UInt64")), ("X3", SimTypeLongLong(signed=False, label="UInt64")), ("X4", SimTypeLongLong(signed=False, label="UInt64")), ("X5", SimTypeLongLong(signed=False, label="UInt64")), ("X6", SimTypeLongLong(signed=False, label="UInt64")), ("X7", SimTypeLongLong(signed=False, label="UInt64")), ("X8", SimTypeLongLong(signed=False, label="UInt64")), ("X9", SimTypeLongLong(signed=False, label="UInt64")), ("X10", SimTypeLongLong(signed=False, label="UInt64")), ("X11", SimTypeLongLong(signed=False, label="UInt64")), ("X12", SimTypeLongLong(signed=False, label="UInt64")), ("X13", SimTypeLongLong(signed=False, label="UInt64")), ("X14", SimTypeLongLong(signed=False, label="UInt64")), ("X15", SimTypeLongLong(signed=False, label="UInt64")), ("X16", SimTypeLongLong(signed=False, label="UInt64")), ("X17", SimTypeLongLong(signed=False, label="UInt64")), ("X18", SimTypeLongLong(signed=False, label="UInt64")), ("X19", SimTypeLongLong(signed=False, label="UInt64")), ("X20", SimTypeLongLong(signed=False, label="UInt64")), ("X21", SimTypeLongLong(signed=False, label="UInt64")), ("X22", SimTypeLongLong(signed=False, label="UInt64")), ("X23", SimTypeLongLong(signed=False, label="UInt64")), ("X24", SimTypeLongLong(signed=False, label="UInt64")), ("X25", SimTypeLongLong(signed=False, label="UInt64")), ("X26", SimTypeLongLong(signed=False, label="UInt64")), ("X27", SimTypeLongLong(signed=False, label="UInt64")), ("X28", SimTypeLongLong(signed=False, label="UInt64")), ("X29", SimTypeLongLong(signed=False, label="UInt64")), ("X30", SimTypeLongLong(signed=False, label="UInt64")), ("SP", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARMV8_AARCH64_GPRS", pack=False, align=None),
    "WHEA_ARM_AARCH32_EL1_CSR": SimStruct(OrderedDict((("DFAR", SimTypeInt(signed=False, label="UInt32")), ("DFSR", SimTypeInt(signed=False, label="UInt32")), ("IFAR", SimTypeInt(signed=False, label="UInt32")), ("ISR", SimTypeInt(signed=False, label="UInt32")), ("MAIR0", SimTypeInt(signed=False, label="UInt32")), ("MAIR1", SimTypeInt(signed=False, label="UInt32")), ("MIDR", SimTypeInt(signed=False, label="UInt32")), ("MPIDR", SimTypeInt(signed=False, label="UInt32")), ("NMRR", SimTypeInt(signed=False, label="UInt32")), ("PRRR", SimTypeInt(signed=False, label="UInt32")), ("SCTLR", SimTypeInt(signed=False, label="UInt32")), ("SPSR", SimTypeInt(signed=False, label="UInt32")), ("SPSR_abt", SimTypeInt(signed=False, label="UInt32")), ("SPSR_fiq", SimTypeInt(signed=False, label="UInt32")), ("SPSR_irq", SimTypeInt(signed=False, label="UInt32")), ("SPSR_svc", SimTypeInt(signed=False, label="UInt32")), ("SPSR_und", SimTypeInt(signed=False, label="UInt32")), ("TPIDRPRW", SimTypeInt(signed=False, label="UInt32")), ("TPIDRURO", SimTypeInt(signed=False, label="UInt32")), ("TPIDRURW", SimTypeInt(signed=False, label="UInt32")), ("TTBCR", SimTypeInt(signed=False, label="UInt32")), ("TTBR0", SimTypeInt(signed=False, label="UInt32")), ("TTBR1", SimTypeInt(signed=False, label="UInt32")), ("DACR", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ARM_AARCH32_EL1_CSR", pack=False, align=None),
    "WHEA_ARM_AARCH32_EL2_CSR": SimStruct(OrderedDict((("ELR_hyp", SimTypeInt(signed=False, label="UInt32")), ("HAMAIR0", SimTypeInt(signed=False, label="UInt32")), ("HAMAIR1", SimTypeInt(signed=False, label="UInt32")), ("HCR", SimTypeInt(signed=False, label="UInt32")), ("HCR2", SimTypeInt(signed=False, label="UInt32")), ("HDFAR", SimTypeInt(signed=False, label="UInt32")), ("HIFAR", SimTypeInt(signed=False, label="UInt32")), ("HPFAR", SimTypeInt(signed=False, label="UInt32")), ("HSR", SimTypeInt(signed=False, label="UInt32")), ("HTCR", SimTypeInt(signed=False, label="UInt32")), ("HTPIDR", SimTypeInt(signed=False, label="UInt32")), ("HTTBR", SimTypeInt(signed=False, label="UInt32")), ("SPSR_hyp", SimTypeInt(signed=False, label="UInt32")), ("VTCR", SimTypeInt(signed=False, label="UInt32")), ("VTTBR", SimTypeInt(signed=False, label="UInt32")), ("DACR32_EL2", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ARM_AARCH32_EL2_CSR", pack=False, align=None),
    "WHEA_ARM_AARCH32_SECURE_CSR": SimStruct(OrderedDict((("SCTLR", SimTypeInt(signed=False, label="UInt32")), ("SPSR_mon", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ARM_AARCH32_SECURE_CSR", pack=False, align=None),
    "WHEA_ARM_AARCH64_EL1_CSR": SimStruct(OrderedDict((("ELR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("ESR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("FAR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("ISR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("MAIR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("MIDR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("MPIDR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("SCTLR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("SP_EL0", SimTypeLongLong(signed=False, label="UInt64")), ("SP_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("SPSR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("TCR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("TPIDR_EL0", SimTypeLongLong(signed=False, label="UInt64")), ("TPIDR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("TPIDRRO_EL0", SimTypeLongLong(signed=False, label="UInt64")), ("TTBR0_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("TTBR1_EL1", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARM_AARCH64_EL1_CSR", pack=False, align=None),
    "WHEA_ARM_AARCH64_EL2_CSR": SimStruct(OrderedDict((("ELR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("ESR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("FAR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("HACR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("HCR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("HPFAR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("MAIR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("SCTLR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("SP_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("SPSR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("TCR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("TPIDR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("TTBR0_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("VTCR_EL2", SimTypeLongLong(signed=False, label="UInt64")), ("VTTBR_EL2", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARM_AARCH64_EL2_CSR", pack=False, align=None),
    "WHEA_ARM_BUS_ERROR": SimStruct(OrderedDict((("ValidationBit", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("_bitfield3", SimTypeChar(label="Byte")), ("_bitfield4", SimTypeShort(signed=False, label="UInt16")), ("_bitfield5", SimTypeChar(label="Byte")), ("_bitfield6", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ARM_BUS_ERROR", pack=False, align=None),
    "WHEA_ARM_BUS_ERROR_VALID_BITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_ARM_CACHE_ERROR": SimStruct(OrderedDict((("ValidationBit", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("_bitfield3", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARM_CACHE_ERROR", pack=False, align=None),
    "WHEA_ARM_CACHE_ERROR_VALID_BITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_ARM_MISC_CSR": SimStruct(OrderedDict((("MRSEncoding", SimTypeShort(signed=False, label="UInt16")), ("Value", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARM_MISC_CSR", pack=False, align=None),
    "WHEA_ARM_PROCESSOR_ERROR": SimUnion({"CacheError": SimTypeRef("WHEA_ARM_CACHE_ERROR", SimStruct), "TlbError": SimTypeRef("WHEA_ARM_TLB_ERROR", SimStruct), "BusError": SimTypeRef("WHEA_ARM_BUS_ERROR", SimStruct), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_ARM_PROCESSOR_ERROR_CONTEXT_INFORMATION_HEADER": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("RegisterContextType", SimTypeShort(signed=False, label="UInt16")), ("RegisterArraySize", SimTypeInt(signed=False, label="UInt32")), ("RegisterArray", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_ARM_PROCESSOR_ERROR_CONTEXT_INFORMATION_HEADER", pack=False, align=None),
    "WHEA_ARM_PROCESSOR_ERROR_CONTEXT_INFORMATION_HEADER_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ARM_PROCESSOR_ERROR_INFORMATION": SimStruct(OrderedDict((("Version", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("ValidationBit", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Type", SimTypeChar(label="Byte")), ("MultipleError", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeChar(label="Byte")), ("ErrorInformation", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualFaultAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalFaultAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARM_PROCESSOR_ERROR_INFORMATION", pack=False, align=None),
    "WHEA_ARM_PROCESSOR_ERROR_INFORMATION_VALID_BITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_ARM_PROCESSOR_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ErrorInformationStructures", SimTypeShort(signed=False, label="UInt16")), ("ContextInformationStructures", SimTypeShort(signed=False, label="UInt16")), ("SectionLength", SimTypeInt(signed=False, label="UInt32")), ("ErrorAffinityLevel", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("MPIDR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("MIDR_EL1", SimTypeLongLong(signed=False, label="UInt64")), ("RunningState", SimTypeInt(signed=False, label="UInt32")), ("PSCIState", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_ARM_PROCESSOR_ERROR_SECTION", pack=False, align=None),
    "WHEA_ARM_PROCESSOR_ERROR_SECTION_VALID_BITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ARM_TLB_ERROR": SimStruct(OrderedDict((("ValidationBit", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("_bitfield3", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ARM_TLB_ERROR", pack=False, align=None),
    "WHEA_ARM_TLB_ERROR_VALID_BITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_AZCC_ROOT_BUS_ERR_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("MaxBusCountPassed", SimTypeChar(label="Byte")), ("InvalidBusMSR", SimTypeChar(label="Byte")),)), name="WHEA_AZCC_ROOT_BUS_ERR_EVENT", pack=False, align=None),
    "WHEA_AZCC_ROOT_BUS_LIST_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("RootBusCount", SimTypeInt(signed=False, label="UInt32")), ("RootBuses", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="WHEA_AZCC_ROOT_BUS_LIST_EVENT", pack=False, align=None),
    "WHEA_AZCC_SET_POISON_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Bus", SimTypeInt(signed=False, label="UInt32")), ("ReadSuccess", SimTypeChar(label="Byte")), ("WriteSuccess", SimTypeChar(label="Byte")), ("IsEnable", SimTypeChar(label="Byte")),)), name="WHEA_AZCC_SET_POISON_EVENT", pack=False, align=None),
    "WHEA_BUGCHECK_RECOVERY_LOG_TYPE": SimTypeInt(signed=False, label="WHEA_BUGCHECK_RECOVERY_LOG_TYPE"),
    "WHEA_CPU_VENDOR": SimTypeInt(signed=False, label="WHEA_CPU_VENDOR"),
    "WHEA_DEVICE_DRIVER_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("SourceGuid", SimTypeBottom(label="Guid")), ("LogTag", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("PacketLength", SimTypeInt(signed=False, label="UInt32")), ("PacketCount", SimTypeInt(signed=False, label="UInt32")), ("PacketBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Config", SimTypeRef("WHEA_ERROR_SOURCE_CONFIGURATION_DD", SimStruct)), ("CreatorId", SimTypeBottom(label="Guid")), ("PartitionId", SimTypeBottom(label="Guid")), ("MaxSectionDataLength", SimTypeInt(signed=False, label="UInt32")), ("MaxSectionsPerRecord", SimTypeInt(signed=False, label="UInt32")), ("PacketStateBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("OpenHandles", SimTypeInt(signed=True, label="Int32")),)), name="WHEA_DEVICE_DRIVER_DESCRIPTOR", pack=False, align=None),
    "WHEA_DRIVER_BUFFER_SET": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("SectionTypeGuid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("SectionFriendlyName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Flags", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_DRIVER_BUFFER_SET", pack=False, align=None),
    "WHEA_ERROR_INJECTION_CAPABILITIES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ERROR_PACKET_DATA_FORMAT": SimTypeInt(signed=False, label="WHEA_ERROR_PACKET_DATA_FORMAT"),
    "WHEA_ERROR_PACKET_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ERROR_PACKET_V1": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("RawDataLength", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeLongLong(signed=False, label="UInt64")), ("Context", SimTypeLongLong(signed=False, label="UInt64")), ("ErrorType", SimTypeInt(signed=False, label="WHEA_ERROR_TYPE")), ("ErrorSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("ErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("ErrorSourceType", SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_TYPE")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Cpu", SimTypeLongLong(signed=False, label="UInt64")), ("u", SimUnion({"ProcessorError": SimTypeRef("WHEA_PROCESSOR_GENERIC_ERROR_SECTION", SimStruct), "MemoryError": SimTypeRef("WHEA_MEMORY_ERROR_SECTION", SimStruct), "NmiError": SimTypeRef("WHEA_NMI_ERROR_SECTION", SimStruct), "PciExpressError": SimTypeRef("WHEA_PCIEXPRESS_ERROR_SECTION", SimStruct), "PciXBusError": SimTypeRef("WHEA_PCIXBUS_ERROR_SECTION", SimStruct), "PciXDeviceError": SimTypeRef("WHEA_PCIXDEVICE_ERROR_SECTION", SimStruct), "PmemError": SimTypeRef("WHEA_PMEM_ERROR_SECTION", SimStruct)}, name="<anon>", label="None")), ("RawDataFormat", SimTypeInt(signed=False, label="WHEA_RAW_DATA_FORMAT")), ("RawDataOffset", SimTypeInt(signed=False, label="UInt32")), ("RawData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_ERROR_PACKET_V1", pack=False, align=None),
    "WHEA_ERROR_PACKET_V2": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ErrorType", SimTypeInt(signed=False, label="WHEA_ERROR_TYPE")), ("ErrorSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("ErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("ErrorSourceType", SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_TYPE")), ("NotifyType", SimTypeBottom(label="Guid")), ("Context", SimTypeLongLong(signed=False, label="UInt64")), ("DataFormat", SimTypeInt(signed=False, label="WHEA_ERROR_PACKET_DATA_FORMAT")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("PshedDataOffset", SimTypeInt(signed=False, label="UInt32")), ("PshedDataLength", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ERROR_PACKET_V2", pack=False, align=None),
    "WHEA_ERROR_RECORD": SimStruct(OrderedDict((("Header", SimTypeRef("WHEA_ERROR_RECORD_HEADER", SimStruct)), ("SectionDescriptor", SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD_SECTION_DESCRIPTOR", SimStruct), offset=0)),)), name="WHEA_ERROR_RECORD", pack=False, align=None),
    "WHEA_ERROR_RECORD_HEADER": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorRevision", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("SignatureEnd", SimTypeInt(signed=False, label="UInt32")), ("SectionCount", SimTypeShort(signed=False, label="UInt16")), ("Severity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Timestamp", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsLARGE_INTEGER": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("PlatformId", SimTypeBottom(label="Guid")), ("PartitionId", SimTypeBottom(label="Guid")), ("CreatorId", SimTypeBottom(label="Guid")), ("NotifyType", SimTypeBottom(label="Guid")), ("RecordId", SimTypeLongLong(signed=False, label="UInt64")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("PersistenceInfo", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("OsBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="_Anonymous_e__Struct", pack=False, align=None), "Reserved": SimTypeArray(SimTypeChar(label="Byte"), 12)}, name="<anon>", label="None")),)), name="WHEA_ERROR_RECORD_HEADER", pack=False, align=None),
    "WHEA_ERROR_RECORD_HEADER_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ERROR_RECORD_HEADER_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ERROR_RECORD_SECTION_DESCRIPTOR": SimStruct(OrderedDict((("SectionOffset", SimTypeInt(signed=False, label="UInt32")), ("SectionLength", SimTypeInt(signed=False, label="UInt32")), ("Revision", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorRevision", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("Reserved", SimTypeChar(label="Byte")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("SectionType", SimTypeBottom(label="Guid")), ("FRUId", SimTypeBottom(label="Guid")), ("SectionSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("FRUText", SimTypeArray(SimTypeChar(label="SByte"), 20)),)), name="WHEA_ERROR_RECORD_SECTION_DESCRIPTOR", pack=False, align=None),
    "WHEA_ERROR_RECORD_SECTION_DESCRIPTOR_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_ERROR_RECORD_SECTION_DESCRIPTOR_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "WHEA_ERROR_RECOVERY_INFO_SECTION": SimStruct(OrderedDict((("RecoveryKernel", SimTypeChar(label="Byte")), ("RecoveryAction", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RecoveryType", SimTypeInt(signed=False, label="WHEA_RECOVERY_TYPE")), ("Irql", SimTypeChar(label="Byte")), ("RecoverySucceeded", SimTypeChar(label="Byte")), ("FailureReason", SimTypeInt(signed=False, label="WHEA_RECOVERY_FAILURE_REASON")), ("ProcessName", SimTypeArray(SimTypeChar(label="SByte"), 20)),)), name="WHEA_ERROR_RECOVERY_INFO_SECTION", pack=False, align=None),
    "WHEA_ERROR_SEVERITY": SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY"),
    "WHEA_ERROR_SOURCE_CONFIGURATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("Correct", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Initialize", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("CreateRecord", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Recover", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("Uninitialize", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WHEA_ERROR_SOURCE_CONFIGURATION", pack=False, align=None),
    "WHEA_ERROR_SOURCE_CONFIGURATION_DD": SimStruct(OrderedDict((("Initialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ErrorSourceId"]), offset=0)), ("Uninitialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("Correct", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ErrorSourceDesc", "MaximumSectionLength"]), offset=0)),)), name="WHEA_ERROR_SOURCE_CONFIGURATION_DD", pack=False, align=None),
    "WHEA_ERROR_SOURCE_CONFIGURATION_DEVICE_DRIVER": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("SourceGuid", SimTypeBottom(label="Guid")), ("LogTag", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Initialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ErrorSourceId"]), offset=0)), ("Uninitialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)), ("MaxSectionDataLength", SimTypeInt(signed=False, label="UInt32")), ("MaxSectionsPerReport", SimTypeInt(signed=False, label="UInt32")), ("CreatorId", SimTypeBottom(label="Guid")), ("PartitionId", SimTypeBottom(label="Guid")),)), name="WHEA_ERROR_SOURCE_CONFIGURATION_DEVICE_DRIVER", pack=False, align=None),
    "WHEA_ERROR_SOURCE_CONFIGURATION_DEVICE_DRIVER_V1": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("SourceGuid", SimTypeBottom(label="Guid")), ("LogTag", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Initialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ErrorSourceId"]), offset=0)), ("Uninitialize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0)),)), name="WHEA_ERROR_SOURCE_CONFIGURATION_DEVICE_DRIVER_V1", pack=False, align=None),
    "WHEA_ERROR_SOURCE_CORRECT": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "WHEA_ERROR_SOURCE_CORRECT_DEVICE_DRIVER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ErrorSourceDesc", "MaximumSectionLength"]), offset=0),
    "WHEA_ERROR_SOURCE_CREATE_RECORD": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "WHEA_ERROR_SOURCE_DESCRIPTOR": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_TYPE")), ("State", SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_STATE")), ("MaxRawDataLength", SimTypeInt(signed=False, label="UInt32")), ("NumRecordsToPreallocate", SimTypeInt(signed=False, label="UInt32")), ("MaxSectionsPerRecord", SimTypeInt(signed=False, label="UInt32")), ("ErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("PlatformErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("Info", SimUnion({"XpfMceDescriptor": SimTypeRef("WHEA_XPF_MCE_DESCRIPTOR", SimStruct), "XpfCmcDescriptor": SimTypeRef("WHEA_XPF_CMC_DESCRIPTOR", SimStruct), "XpfNmiDescriptor": SimTypeRef("WHEA_XPF_NMI_DESCRIPTOR", SimStruct), "IpfMcaDescriptor": SimTypeRef("WHEA_IPF_MCA_DESCRIPTOR", SimStruct), "IpfCmcDescriptor": SimTypeRef("WHEA_IPF_CMC_DESCRIPTOR", SimStruct), "IpfCpeDescriptor": SimTypeRef("WHEA_IPF_CPE_DESCRIPTOR", SimStruct), "AerRootportDescriptor": SimTypeRef("WHEA_AER_ROOTPORT_DESCRIPTOR", SimStruct), "AerEndpointDescriptor": SimTypeRef("WHEA_AER_ENDPOINT_DESCRIPTOR", SimStruct), "AerBridgeDescriptor": SimTypeRef("WHEA_AER_BRIDGE_DESCRIPTOR", SimStruct), "GenErrDescriptor": SimTypeRef("WHEA_GENERIC_ERROR_DESCRIPTOR", SimStruct), "GenErrDescriptorV2": SimTypeRef("WHEA_GENERIC_ERROR_DESCRIPTOR_V2", SimStruct), "DeviceDriverDescriptor": SimTypeRef("WHEA_DEVICE_DRIVER_DESCRIPTOR", SimStruct)}, name="<anon>", label="None")),)), name="WHEA_ERROR_SOURCE_DESCRIPTOR", pack=False, align=None),
    "WHEA_ERROR_SOURCE_INITIALIZE": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "WHEA_ERROR_SOURCE_INITIALIZE_DEVICE_DRIVER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "ErrorSourceId"]), offset=0),
    "WHEA_ERROR_SOURCE_OVERRIDE_SETTINGS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_TYPE")), ("MaxRawDataLength", SimTypeInt(signed=False, label="UInt32")), ("NumRecordsToPreallocate", SimTypeInt(signed=False, label="UInt32")), ("MaxSectionsPerRecord", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_ERROR_SOURCE_OVERRIDE_SETTINGS", pack=False, align=None),
    "WHEA_ERROR_SOURCE_RECOVER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "WHEA_ERROR_SOURCE_STATE": SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_STATE"),
    "WHEA_ERROR_SOURCE_TYPE": SimTypeInt(signed=False, label="WHEA_ERROR_SOURCE_TYPE"),
    "WHEA_ERROR_SOURCE_UNINITIALIZE": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "WHEA_ERROR_SOURCE_UNINITIALIZE_DEVICE_DRIVER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "WHEA_ERROR_STATUS": SimUnion({"ErrorStatus": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "WHEA_ERROR_TYPE": SimTypeInt(signed=False, label="WHEA_ERROR_TYPE"),
    "WHEA_ETW_OVERFLOW_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("RecordId", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_ETW_OVERFLOW_EVENT", pack=False, align=None),
    "WHEA_EVENT_LOG_ENTRY": SimStruct(OrderedDict((("Header", SimTypeRef("WHEA_EVENT_LOG_ENTRY_HEADER", SimStruct)),)), name="WHEA_EVENT_LOG_ENTRY", pack=False, align=None),
    "WHEA_EVENT_LOG_ENTRY_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_EVENT_LOG_ENTRY_HEADER": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="WHEA_EVENT_LOG_ENTRY_TYPE")), ("OwnerTag", SimTypeInt(signed=False, label="UInt32")), ("Id", SimTypeInt(signed=False, label="WHEA_EVENT_LOG_ENTRY_ID")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("PayloadLength", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_EVENT_LOG_ENTRY_HEADER", pack=False, align=None),
    "WHEA_EVENT_LOG_ENTRY_ID": SimTypeInt(signed=False, label="WHEA_EVENT_LOG_ENTRY_ID"),
    "WHEA_EVENT_LOG_ENTRY_TYPE": SimTypeInt(signed=False, label="WHEA_EVENT_LOG_ENTRY_TYPE"),
    "WHEA_FAILED_ADD_DEFECT_LIST_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEA_FAILED_ADD_DEFECT_LIST_EVENT", pack=False, align=None),
    "WHEA_FIRMWARE_ERROR_RECORD_REFERENCE": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("FirmwareRecordId", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_FIRMWARE_ERROR_RECORD_REFERENCE", pack=False, align=None),
    "WHEA_GENERIC_ERROR": SimStruct(OrderedDict((("BlockStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("RawDataOffset", SimTypeInt(signed=False, label="UInt32")), ("RawDataLength", SimTypeInt(signed=False, label="UInt32")), ("DataLength", SimTypeInt(signed=False, label="UInt32")), ("ErrorSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_GENERIC_ERROR", pack=False, align=None),
    "WHEA_GENERIC_ERROR_BLOCKSTATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_GENERIC_ERROR_DATA_ENTRY_V1": SimStruct(OrderedDict((("SectionType", SimTypeBottom(label="Guid")), ("ErrorSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("Revision", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorRevision", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("ValidBits", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("ErrorDataLength", SimTypeInt(signed=False, label="UInt32")), ("FRUId", SimTypeBottom(label="Guid")), ("FRUText", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_GENERIC_ERROR_DATA_ENTRY_V1", pack=False, align=None),
    "WHEA_GENERIC_ERROR_DATA_ENTRY_V2": SimStruct(OrderedDict((("SectionType", SimTypeBottom(label="Guid")), ("ErrorSeverity", SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY")), ("Revision", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorRevision", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("ValidBits", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("ErrorDataLength", SimTypeInt(signed=False, label="UInt32")), ("FRUId", SimTypeBottom(label="Guid")), ("FRUText", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("Timestamp", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsLARGE_INTEGER": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_GENERIC_ERROR_DATA_ENTRY_V2", pack=False, align=None),
    "WHEA_GENERIC_ERROR_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeChar(label="Byte")), ("Enabled", SimTypeChar(label="Byte")), ("ErrStatusBlockLength", SimTypeInt(signed=False, label="UInt32")), ("RelatedErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("ErrStatusAddressSpaceID", SimTypeChar(label="Byte")), ("ErrStatusAddressBitWidth", SimTypeChar(label="Byte")), ("ErrStatusAddressBitOffset", SimTypeChar(label="Byte")), ("ErrStatusAddressAccessSize", SimTypeChar(label="Byte")), ("ErrStatusAddress", SimTypeLongLong(signed=True, label="Int64")), ("Notify", SimTypeRef("WHEA_NOTIFICATION_DESCRIPTOR", SimStruct)),)), name="WHEA_GENERIC_ERROR_DESCRIPTOR", pack=False, align=None),
    "WHEA_GENERIC_ERROR_DESCRIPTOR_V2": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeChar(label="Byte")), ("Enabled", SimTypeChar(label="Byte")), ("ErrStatusBlockLength", SimTypeInt(signed=False, label="UInt32")), ("RelatedErrorSourceId", SimTypeInt(signed=False, label="UInt32")), ("ErrStatusAddressSpaceID", SimTypeChar(label="Byte")), ("ErrStatusAddressBitWidth", SimTypeChar(label="Byte")), ("ErrStatusAddressBitOffset", SimTypeChar(label="Byte")), ("ErrStatusAddressAccessSize", SimTypeChar(label="Byte")), ("ErrStatusAddress", SimTypeLongLong(signed=True, label="Int64")), ("Notify", SimTypeRef("WHEA_NOTIFICATION_DESCRIPTOR", SimStruct)), ("ReadAckAddressSpaceID", SimTypeChar(label="Byte")), ("ReadAckAddressBitWidth", SimTypeChar(label="Byte")), ("ReadAckAddressBitOffset", SimTypeChar(label="Byte")), ("ReadAckAddressAccessSize", SimTypeChar(label="Byte")), ("ReadAckAddress", SimTypeLongLong(signed=True, label="Int64")), ("ReadAckPreserveMask", SimTypeLongLong(signed=False, label="UInt64")), ("ReadAckWriteMask", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_GENERIC_ERROR_DESCRIPTOR_V2", pack=False, align=None),
    "WHEA_IN_USE_PAGE_NOTIFY_FLAGS": SimUnion({"Bits": SimTypeRef("_Bits_e__Struct", SimStruct), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "WHEA_IPF_CMC_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="WHEA_IPF_CMC_DESCRIPTOR", pack=False, align=None),
    "WHEA_IPF_CPE_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="WHEA_IPF_CPE_DESCRIPTOR", pack=False, align=None),
    "WHEA_IPF_MCA_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="WHEA_IPF_MCA_DESCRIPTOR", pack=False, align=None),
    "WHEA_MEMORY_CORRECTABLE_ERROR_DATA": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("SocketId", SimTypeInt(signed=False, label="UInt32")), ("ChannelId", SimTypeInt(signed=False, label="UInt32")), ("DimmSlot", SimTypeInt(signed=False, label="UInt32")), ("CorrectableErrorCount", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_MEMORY_CORRECTABLE_ERROR_DATA", pack=False, align=None),
    "WHEA_MEMORY_CORRECTABLE_ERROR_HEADER": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeShort(signed=False, label="UInt16")),)), name="WHEA_MEMORY_CORRECTABLE_ERROR_HEADER", pack=False, align=None),
    "WHEA_MEMORY_CORRECTABLE_ERROR_SECTION": SimStruct(OrderedDict((("Header", SimTypeRef("WHEA_MEMORY_CORRECTABLE_ERROR_HEADER", SimStruct)), ("Data", SimTypePointer(SimTypeRef("WHEA_MEMORY_CORRECTABLE_ERROR_DATA", SimStruct), offset=0)),)), name="WHEA_MEMORY_CORRECTABLE_ERROR_SECTION", pack=False, align=None),
    "WHEA_MEMORY_CORRECTABLE_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_MEMORY_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ErrorStatus", SimUnion({"ErrorStatus": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("PhysicalAddress", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalAddressMask", SimTypeLongLong(signed=False, label="UInt64")), ("Node", SimTypeShort(signed=False, label="UInt16")), ("Card", SimTypeShort(signed=False, label="UInt16")), ("Module", SimTypeShort(signed=False, label="UInt16")), ("Bank", SimTypeShort(signed=False, label="UInt16")), ("Device", SimTypeShort(signed=False, label="UInt16")), ("Row", SimTypeShort(signed=False, label="UInt16")), ("Column", SimTypeShort(signed=False, label="UInt16")), ("BitPosition", SimTypeShort(signed=False, label="UInt16")), ("RequesterId", SimTypeLongLong(signed=False, label="UInt64")), ("ResponderId", SimTypeLongLong(signed=False, label="UInt64")), ("TargetId", SimTypeLongLong(signed=False, label="UInt64")), ("ErrorType", SimTypeChar(label="Byte")), ("Extended", SimTypeChar(label="Byte")), ("RankNumber", SimTypeShort(signed=False, label="UInt16")), ("CardHandle", SimTypeShort(signed=False, label="UInt16")), ("ModuleHandle", SimTypeShort(signed=False, label="UInt16")),)), name="WHEA_MEMORY_ERROR_SECTION", pack=False, align=None),
    "WHEA_MEMORY_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_MEMORY_THROTTLE_SUMMARY_FAILED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEA_MEMORY_THROTTLE_SUMMARY_FAILED_EVENT", pack=False, align=None),
    "WHEA_MSR_DUMP_SECTION": SimStruct(OrderedDict((("MsrDumpBuffer", SimTypeChar(label="Byte")), ("MsrDumpLength", SimTypeInt(signed=False, label="UInt32")), ("MsrDumpData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_MSR_DUMP_SECTION", pack=False, align=None),
    "WHEA_NMI_ERROR_SECTION": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="WHEA_NMI_ERROR_SECTION", pack=False, align=None),
    "WHEA_NMI_ERROR_SECTION_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_NOTIFICATION_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Length", SimTypeChar(label="Byte")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("u", SimUnion({"Polled": SimTypeRef("_Polled_e__Struct", SimStruct), "Interrupt": SimTypeRef("_Interrupt_e__Struct", SimStruct), "LocalInterrupt": SimTypeRef("_LocalInterrupt_e__Struct", SimStruct), "Sci": SimTypeRef("_Sci_e__Struct", SimStruct), "Nmi": SimTypeRef("_Nmi_e__Struct", SimStruct), "Sea": SimTypeRef("_Sea_e__Struct", SimStruct), "Sei": SimTypeRef("_Sei_e__Struct", SimStruct), "Gsiv": SimTypeRef("_Gsiv_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WHEA_NOTIFICATION_DESCRIPTOR", pack=False, align=None),
    "WHEA_NOTIFICATION_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_OFFLINE_DONE_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Address", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_OFFLINE_DONE_EVENT", pack=False, align=None),
    "WHEA_PACKET_LOG_DATA": SimStruct(OrderedDict((("LogData", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("ExtraBytes", SimTypeArray(SimTypeChar(label="Byte"), 36)), ("BcParam3", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BcParam4", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("LogDataLength", SimTypeInt(signed=False, label="UInt32")), ("LogTag", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="WHEA_PACKET_LOG_DATA", pack=False, align=None),
    "WHEA_PCIEXPRESS_BRIDGE_CONTROL_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("BridgeSecondaryStatus", SimTypeShort(signed=False, label="UInt16")), ("BridgeControl", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_PCIEXPRESS_COMMAND_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("Command", SimTypeShort(signed=False, label="UInt16")), ("Status", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_PCIEXPRESS_DEVICE_ID": SimStruct(OrderedDict((("VendorID", SimTypeShort(signed=False, label="UInt16")), ("DeviceID", SimTypeShort(signed=False, label="UInt16")), ("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")), ("_bitfield3", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_PCIEXPRESS_DEVICE_ID", pack=False, align=None),
    "WHEA_PCIEXPRESS_DEVICE_TYPE": SimTypeInt(signed=False, label="WHEA_PCIEXPRESS_DEVICE_TYPE"),
    "WHEA_PCIEXPRESS_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("PortType", SimTypeInt(signed=False, label="WHEA_PCIEXPRESS_DEVICE_TYPE")), ("Version", SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorVersion", SimTypeChar(label="Byte")), ("MajorVersion", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CommandStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("Command", SimTypeShort(signed=False, label="UInt16")), ("Status", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("DeviceId", SimTypeRef("WHEA_PCIEXPRESS_DEVICE_ID", SimStruct)), ("DeviceSerialNumber", SimTypeLongLong(signed=False, label="UInt64")), ("BridgeControlStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("BridgeSecondaryStatus", SimTypeShort(signed=False, label="UInt16")), ("BridgeControl", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ExpressCapability", SimTypeArray(SimTypeChar(label="Byte"), 60)), ("AerInfo", SimTypeArray(SimTypeChar(label="Byte"), 96)),)), name="WHEA_PCIEXPRESS_ERROR_SECTION", pack=False, align=None),
    "WHEA_PCIEXPRESS_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PCIEXPRESS_VERSION": SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorVersion", SimTypeChar(label="Byte")), ("MajorVersion", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_PCIE_ADDRESS": SimStruct(OrderedDict((("Segment", SimTypeInt(signed=False, label="UInt32")), ("Bus", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypeInt(signed=False, label="UInt32")), ("Function", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_PCIE_ADDRESS", pack=False, align=None),
    "WHEA_PCIE_CORRECTABLE_ERROR_DEVICES": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Address", SimTypeRef("WHEA_PCIE_ADDRESS", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("CorrectableErrorCount", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 32)),)), name="WHEA_PCIE_CORRECTABLE_ERROR_DEVICES", pack=False, align=None),
    "WHEA_PCIE_CORRECTABLE_ERROR_DEVICES_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PCIE_CORRECTABLE_ERROR_SECTION": SimStruct(OrderedDict((("Header", SimTypeRef("WHEA_PCIE_CORRECTABLE_ERROR_SECTION_HEADER", SimStruct)), ("Devices", SimTypePointer(SimTypeRef("WHEA_PCIE_CORRECTABLE_ERROR_DEVICES", SimStruct), offset=0)),)), name="WHEA_PCIE_CORRECTABLE_ERROR_SECTION", pack=False, align=None),
    "WHEA_PCIE_CORRECTABLE_ERROR_SECTION_HEADER": SimStruct(OrderedDict((("Version", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeShort(signed=False, label="UInt16")),)), name="WHEA_PCIE_CORRECTABLE_ERROR_SECTION_HEADER", pack=False, align=None),
    "WHEA_PCIXBUS_COMMAND": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PCIXBUS_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ErrorStatus", SimUnion({"ErrorStatus": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("ErrorType", SimTypeShort(signed=False, label="UInt16")), ("BusId", SimUnion({"Anonymous": SimStruct(OrderedDict((("BusNumber", SimTypeChar(label="Byte")), ("BusSegment", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("BusAddress", SimTypeLongLong(signed=False, label="UInt64")), ("BusData", SimTypeLongLong(signed=False, label="UInt64")), ("BusCommand", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("RequesterId", SimTypeLongLong(signed=False, label="UInt64")), ("CompleterId", SimTypeLongLong(signed=False, label="UInt64")), ("TargetId", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_PCIXBUS_ERROR_SECTION", pack=False, align=None),
    "WHEA_PCIXBUS_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PCIXBUS_ID": SimUnion({"Anonymous": SimStruct(OrderedDict((("BusNumber", SimTypeChar(label="Byte")), ("BusSegment", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_PCIXDEVICE_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ErrorStatus", SimUnion({"ErrorStatus": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("IdInfo", SimTypeRef("WHEA_PCIXDEVICE_ID", SimStruct)), ("MemoryNumber", SimTypeInt(signed=False, label="UInt32")), ("IoNumber", SimTypeInt(signed=False, label="UInt32")), ("RegisterDataPairs", SimTypePointer(SimTypeRef("WHEA_PCIXDEVICE_REGISTER_PAIR", SimStruct), offset=0)),)), name="WHEA_PCIXDEVICE_ERROR_SECTION", pack=False, align=None),
    "WHEA_PCIXDEVICE_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PCIXDEVICE_ID": SimStruct(OrderedDict((("VendorId", SimTypeShort(signed=False, label="UInt16")), ("DeviceId", SimTypeShort(signed=False, label="UInt16")), ("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_PCIXDEVICE_ID", pack=False, align=None),
    "WHEA_PCIXDEVICE_REGISTER_PAIR": SimStruct(OrderedDict((("Register", SimTypeLongLong(signed=False, label="UInt64")), ("Data", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_PCIXDEVICE_REGISTER_PAIR", pack=False, align=None),
    "WHEA_PCI_RECOVERY_SECTION": SimStruct(OrderedDict((("SignalType", SimTypeChar(label="Byte")), ("RecoveryAttempted", SimTypeChar(label="Byte")), ("RecoveryStatus", SimTypeChar(label="Byte")),)), name="WHEA_PCI_RECOVERY_SECTION", pack=False, align=None),
    "WHEA_PCI_RECOVERY_SIGNAL": SimTypeInt(signed=False, label="WHEA_PCI_RECOVERY_SIGNAL"),
    "WHEA_PCI_RECOVERY_STATUS": SimTypeInt(signed=False, label="WHEA_PCI_RECOVERY_STATUS"),
    "WHEA_PCI_SLOT_NUMBER": SimStruct(OrderedDict((("u", SimUnion({"bits": SimTypeRef("_bits_e__Struct", SimStruct), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="WHEA_PCI_SLOT_NUMBER", pack=False, align=None),
    "WHEA_PERSISTENCE_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PFA_REMOVE_TRIGGER": SimTypeInt(signed=False, label="WHEA_PFA_REMOVE_TRIGGER"),
    "WHEA_PMEM_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("LocationInfo", SimTypeArray(SimTypeChar(label="Byte"), 64)), ("ErrorStatus", SimUnion({"ErrorStatus": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("NFITHandle", SimTypeInt(signed=False, label="UInt32")), ("PageRangeCount", SimTypeInt(signed=False, label="UInt32")), ("PageRange", SimTypePointer(SimTypeRef("WHEA_PMEM_PAGE_RANGE", SimStruct), offset=0)),)), name="WHEA_PMEM_ERROR_SECTION", pack=False, align=None),
    "WHEA_PMEM_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PMEM_PAGE_RANGE": SimStruct(OrderedDict((("StartingPfn", SimTypeLongLong(signed=False, label="UInt64")), ("PageCount", SimTypeLongLong(signed=False, label="UInt64")), ("MarkedBadBitmap", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_PMEM_PAGE_RANGE", pack=False, align=None),
    "WHEA_PROCESSOR_FAMILY_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("NativeModelId", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PROCESSOR_GENERIC_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("ProcessorType", SimTypeChar(label="Byte")), ("InstructionSet", SimTypeChar(label="Byte")), ("ErrorType", SimTypeChar(label="Byte")), ("Operation", SimTypeChar(label="Byte")), ("Flags", SimTypeChar(label="Byte")), ("Level", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("CPUVersion", SimTypeLongLong(signed=False, label="UInt64")), ("CPUBrandString", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("ProcessorId", SimTypeLongLong(signed=False, label="UInt64")), ("TargetAddress", SimTypeLongLong(signed=False, label="UInt64")), ("RequesterId", SimTypeLongLong(signed=False, label="UInt64")), ("ResponderId", SimTypeLongLong(signed=False, label="UInt64")), ("InstructionPointer", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_PROCESSOR_GENERIC_ERROR_SECTION", pack=False, align=None),
    "WHEA_PROCESSOR_GENERIC_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_PSHED_PI_CPU_BUSES_INIT_FAILED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEA_PSHED_PI_CPU_BUSES_INIT_FAILED_EVENT", pack=False, align=None),
    "WHEA_PSHED_PI_TRACE_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Buffer", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="WHEA_PSHED_PI_TRACE_EVENT", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_CALLBACKS": SimStruct(OrderedDict((("GetAllErrorSources", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Count", "ErrorSrcs", "Length"]), offset=0)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("GetErrorSourceInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0)), ("SetErrorSourceInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0)), ("EnableErrorSource", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0)), ("DisableErrorSource", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource"]), offset=0)), ("WriteErrorRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Flags", "RecordLength", "ErrorRecord"]), offset=0)), ("ReadErrorRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Flags", "ErrorRecordId", "NextErrorRecordId", "RecordLength", "ErrorRecord"]), offset=0)), ("ClearErrorRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Flags", "ErrorRecordId"]), offset=0)), ("RetrieveErrorInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("WHEA_ERROR_PACKET_V2", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource", "BufferLength", "Packet"]), offset=0)), ("FinalizeErrorRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource", "BufferLength", "ErrorRecord"]), offset=0)), ("ClearErrorStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorSource", "BufferLength", "ErrorRecord"]), offset=0)), ("AttemptRecovery", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "BufferLength", "ErrorRecord"]), offset=0)), ("GetInjectionCapabilities", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "Capabilities"]), offset=0)), ("InjectError", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PluginContext", "ErrorType", "Parameter1", "Parameter2", "Parameter3", "Parameter4"]), offset=0)),)), name="WHEA_PSHED_PLUGIN_CALLBACKS", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_DIMM_MISMATCH": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("FirmwareBank", SimTypeShort(signed=False, label="UInt16")), ("FirmwareCol", SimTypeShort(signed=False, label="UInt16")), ("FirmwareRow", SimTypeShort(signed=False, label="UInt16")), ("RetryRdBank", SimTypeShort(signed=False, label="UInt16")), ("RetryRdCol", SimTypeShort(signed=False, label="UInt16")), ("RetryRdRow", SimTypeShort(signed=False, label="UInt16")), ("TaBank", SimTypeShort(signed=False, label="UInt16")), ("TaCol", SimTypeShort(signed=False, label="UInt16")), ("TaRow", SimTypeShort(signed=False, label="UInt16")),)), name="WHEA_PSHED_PLUGIN_DIMM_MISMATCH", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_ENABLE_NOTIFY_ERRORS": SimTypeInt(signed=False, label="WHEA_PSHED_PLUGIN_ENABLE_NOTIFY_ERRORS"),
    "WHEA_PSHED_PLUGIN_ENABLE_NOTIFY_FAILED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("EnableError", SimTypeInt(signed=False, label="WHEA_PSHED_PLUGIN_ENABLE_NOTIFY_ERRORS")),)), name="WHEA_PSHED_PLUGIN_ENABLE_NOTIFY_FAILED_EVENT", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_HEARTBEAT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEA_PSHED_PLUGIN_HEARTBEAT", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_INIT_FAILED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEA_PSHED_PLUGIN_INIT_FAILED_EVENT", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_LOAD_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("PluginName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_PSHED_PLUGIN_LOAD_EVENT", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_PLATFORM_SUPPORT_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("PluginName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("Supported", SimTypeChar(label="Byte")),)), name="WHEA_PSHED_PLUGIN_PLATFORM_SUPPORT_EVENT", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_REGISTRATION_PACKET_V1": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FunctionalAreaMask", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Callbacks", SimTypeRef("WHEA_PSHED_PLUGIN_CALLBACKS", SimStruct)),)), name="WHEA_PSHED_PLUGIN_REGISTRATION_PACKET_V1", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_REGISTRATION_PACKET_V2": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FunctionalAreaMask", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Callbacks", SimTypeRef("WHEA_PSHED_PLUGIN_CALLBACKS", SimStruct)), ("PluginHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WHEA_PSHED_PLUGIN_REGISTRATION_PACKET_V2", pack=False, align=None),
    "WHEA_PSHED_PLUGIN_UNLOAD_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("PluginName", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="WHEA_PSHED_PLUGIN_UNLOAD_EVENT", pack=False, align=None),
    "WHEA_RAW_DATA_FORMAT": SimTypeInt(signed=False, label="WHEA_RAW_DATA_FORMAT"),
    "WHEA_RECOVERY_ACTION": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield1", SimTypeInt(signed=False, label="UInt32")), ("_bitfield2", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_RECOVERY_CONTEXT": SimStruct(OrderedDict((("Anonymous", SimUnion({"MemoryError": SimTypeRef("_MemoryError_e__Struct", SimStruct), "PmemError": SimTypeRef("_PmemError_e__Struct", SimStruct)}, name="<anon>", label="None")), ("PartitionId", SimTypeLongLong(signed=False, label="UInt64")), ("VpIndex", SimTypeInt(signed=False, label="UInt32")), ("ErrorType", SimTypeInt(signed=False, label="WHEA_RECOVERY_CONTEXT_ERROR_TYPE")),)), name="WHEA_RECOVERY_CONTEXT", pack=False, align=None),
    "WHEA_RECOVERY_CONTEXT_ERROR_TYPE": SimTypeInt(signed=False, label="WHEA_RECOVERY_CONTEXT_ERROR_TYPE"),
    "WHEA_RECOVERY_FAILURE_REASON": SimTypeInt(signed=False, label="WHEA_RECOVERY_FAILURE_REASON"),
    "WHEA_RECOVERY_TYPE": SimTypeInt(signed=False, label="WHEA_RECOVERY_TYPE"),
    "WHEA_REGISTER_KEY_NOTIFICATION_FAILED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEA_REGISTER_KEY_NOTIFICATION_FAILED_EVENT", pack=False, align=None),
    "WHEA_REPORT_HW_ERROR_DEVICE_DRIVER_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHEA_REVISION": SimUnion({"Anonymous": SimStruct(OrderedDict((("MinorRevision", SimTypeChar(label="Byte")), ("MajorRevision", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHEA_SEA_SECTION": SimStruct(OrderedDict((("Esr", SimTypeInt(signed=False, label="UInt32")), ("Far", SimTypeLongLong(signed=False, label="UInt64")), ("Par", SimTypeLongLong(signed=False, label="UInt64")), ("WasKernel", SimTypeChar(label="Byte")),)), name="WHEA_SEA_SECTION", pack=False, align=None),
    "WHEA_SEI_SECTION": SimStruct(OrderedDict((("Esr", SimTypeInt(signed=False, label="UInt32")), ("Far", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_SEI_SECTION", pack=False, align=None),
    "WHEA_SEL_BUGCHECK_PROGRESS": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("BugCheckCode", SimTypeInt(signed=False, label="UInt32")), ("BugCheckProgressSummary", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_SEL_BUGCHECK_PROGRESS", pack=False, align=None),
    "WHEA_SEL_BUGCHECK_RECOVERY_STATUS_MULTIPLE_BUGCHECK_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("IsBugcheckOwner", SimTypeChar(label="Byte")), ("RecursionCount", SimTypeChar(label="Byte")), ("IsBugcheckRecoveryOwner", SimTypeChar(label="Byte")),)), name="WHEA_SEL_BUGCHECK_RECOVERY_STATUS_MULTIPLE_BUGCHECK_EVENT", pack=False, align=None),
    "WHEA_SEL_BUGCHECK_RECOVERY_STATUS_PHASE1_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Success", SimTypeChar(label="Byte")), ("Version", SimTypeChar(label="Byte")), ("EntryCount", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypeRef("_Data_e__Struct", SimStruct)),)), name="WHEA_SEL_BUGCHECK_RECOVERY_STATUS_PHASE1_EVENT", pack=False, align=None),
    "WHEA_SEL_BUGCHECK_RECOVERY_STATUS_PHASE2_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("BootId", SimTypeInt(signed=False, label="UInt32")), ("Success", SimTypeChar(label="Byte")),)), name="WHEA_SEL_BUGCHECK_RECOVERY_STATUS_PHASE2_EVENT", pack=False, align=None),
    "WHEA_SEL_BUGCHECK_RECOVERY_STATUS_START_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("StartingIrql", SimTypeChar(label="Byte")),)), name="WHEA_SEL_BUGCHECK_RECOVERY_STATUS_START_EVENT", pack=False, align=None),
    "WHEA_SIGNAL_HANDLER_OVERRIDE_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "WHEA_SRAR_DETAIL_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("RecoveryContextFlags", SimTypeInt(signed=False, label="UInt32")), ("RecoveryContextPa", SimTypeLongLong(signed=False, label="UInt64")), ("PageOfflineStatus", SimTypeInt(signed=True, label="Int32")), ("KernelConsumerError", SimTypeChar(label="Byte")),)), name="WHEA_SRAR_DETAIL_EVENT", pack=False, align=None),
    "WHEA_SRAS_TABLE_ENTRIES_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("LogNumber", SimTypeInt(signed=False, label="UInt32")), ("NumberSignals", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_SRAS_TABLE_ENTRIES_EVENT", pack=False, align=None),
    "WHEA_SRAS_TABLE_ERROR": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEA_SRAS_TABLE_ERROR", pack=False, align=None),
    "WHEA_SRAS_TABLE_NOT_FOUND": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEA_SRAS_TABLE_NOT_FOUND", pack=False, align=None),
    "WHEA_THROTTLE_ADD_ERR_SRC_FAILED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)),)), name="WHEA_THROTTLE_ADD_ERR_SRC_FAILED_EVENT", pack=False, align=None),
    "WHEA_THROTTLE_MEMORY_ADD_OR_REMOVE_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("SocketId", SimTypeInt(signed=False, label="UInt32")), ("ChannelId", SimTypeInt(signed=False, label="UInt32")), ("DimmSlot", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_THROTTLE_MEMORY_ADD_OR_REMOVE_EVENT", pack=False, align=None),
    "WHEA_THROTTLE_PCIE_ADD_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Address", SimTypeRef("WHEA_PCIE_ADDRESS", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")), ("Updated", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="WHEA_THROTTLE_PCIE_ADD_EVENT", pack=False, align=None),
    "WHEA_THROTTLE_PCIE_REMOVE_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("Address", SimTypeRef("WHEA_PCIE_ADDRESS", SimStruct)), ("Mask", SimTypeInt(signed=False, label="UInt32")),)), name="WHEA_THROTTLE_PCIE_REMOVE_EVENT", pack=False, align=None),
    "WHEA_THROTTLE_REGISTRY_CORRUPT_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ThrottleType", SimTypeInt(signed=False, label="WHEA_THROTTLE_TYPE")),)), name="WHEA_THROTTLE_REGISTRY_CORRUPT_EVENT", pack=False, align=None),
    "WHEA_THROTTLE_REG_DATA_IGNORED_EVENT": SimStruct(OrderedDict((("WheaEventLogEntry", SimTypeRef("WHEA_EVENT_LOG_ENTRY", SimStruct)), ("ThrottleType", SimTypeInt(signed=False, label="WHEA_THROTTLE_TYPE")),)), name="WHEA_THROTTLE_REG_DATA_IGNORED_EVENT", pack=False, align=None),
    "WHEA_THROTTLE_TYPE": SimTypeInt(signed=False, label="WHEA_THROTTLE_TYPE"),
    "WHEA_TIMESTAMP": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsLARGE_INTEGER": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"),
    "WHEA_X64_REGISTER_STATE": SimStruct(OrderedDict((("Rax", SimTypeLongLong(signed=False, label="UInt64")), ("Rbx", SimTypeLongLong(signed=False, label="UInt64")), ("Rcx", SimTypeLongLong(signed=False, label="UInt64")), ("Rdx", SimTypeLongLong(signed=False, label="UInt64")), ("Rsi", SimTypeLongLong(signed=False, label="UInt64")), ("Rdi", SimTypeLongLong(signed=False, label="UInt64")), ("Rbp", SimTypeLongLong(signed=False, label="UInt64")), ("Rsp", SimTypeLongLong(signed=False, label="UInt64")), ("R8", SimTypeLongLong(signed=False, label="UInt64")), ("R9", SimTypeLongLong(signed=False, label="UInt64")), ("R10", SimTypeLongLong(signed=False, label="UInt64")), ("R11", SimTypeLongLong(signed=False, label="UInt64")), ("R12", SimTypeLongLong(signed=False, label="UInt64")), ("R13", SimTypeLongLong(signed=False, label="UInt64")), ("R14", SimTypeLongLong(signed=False, label="UInt64")), ("R15", SimTypeLongLong(signed=False, label="UInt64")), ("Cs", SimTypeShort(signed=False, label="UInt16")), ("Ds", SimTypeShort(signed=False, label="UInt16")), ("Ss", SimTypeShort(signed=False, label="UInt16")), ("Es", SimTypeShort(signed=False, label="UInt16")), ("Fs", SimTypeShort(signed=False, label="UInt16")), ("Gs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Rflags", SimTypeLongLong(signed=False, label="UInt64")), ("Eip", SimTypeLongLong(signed=False, label="UInt64")), ("Cr0", SimTypeLongLong(signed=False, label="UInt64")), ("Cr1", SimTypeLongLong(signed=False, label="UInt64")), ("Cr2", SimTypeLongLong(signed=False, label="UInt64")), ("Cr3", SimTypeLongLong(signed=False, label="UInt64")), ("Cr4", SimTypeLongLong(signed=False, label="UInt64")), ("Cr8", SimTypeLongLong(signed=False, label="UInt64")), ("Gdtr", SimTypeRef("WHEA128A", SimStruct)), ("Idtr", SimTypeRef("WHEA128A", SimStruct)), ("Ldtr", SimTypeShort(signed=False, label="UInt16")), ("Tr", SimTypeShort(signed=False, label="UInt16")),)), name="WHEA_X64_REGISTER_STATE", pack=False, align=None),
    "WHEA_X86_REGISTER_STATE": SimStruct(OrderedDict((("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")), ("Esi", SimTypeInt(signed=False, label="UInt32")), ("Edi", SimTypeInt(signed=False, label="UInt32")), ("Ebp", SimTypeInt(signed=False, label="UInt32")), ("Esp", SimTypeInt(signed=False, label="UInt32")), ("Cs", SimTypeShort(signed=False, label="UInt16")), ("Ds", SimTypeShort(signed=False, label="UInt16")), ("Ss", SimTypeShort(signed=False, label="UInt16")), ("Es", SimTypeShort(signed=False, label="UInt16")), ("Fs", SimTypeShort(signed=False, label="UInt16")), ("Gs", SimTypeShort(signed=False, label="UInt16")), ("Eflags", SimTypeInt(signed=False, label="UInt32")), ("Eip", SimTypeInt(signed=False, label="UInt32")), ("Cr0", SimTypeInt(signed=False, label="UInt32")), ("Cr1", SimTypeInt(signed=False, label="UInt32")), ("Cr2", SimTypeInt(signed=False, label="UInt32")), ("Cr3", SimTypeInt(signed=False, label="UInt32")), ("Cr4", SimTypeInt(signed=False, label="UInt32")), ("Gdtr", SimTypeLongLong(signed=False, label="UInt64")), ("Idtr", SimTypeLongLong(signed=False, label="UInt64")), ("Ldtr", SimTypeShort(signed=False, label="UInt16")), ("Tr", SimTypeShort(signed=False, label="UInt16")),)), name="WHEA_X86_REGISTER_STATE", pack=False, align=None),
    "WHEA_XPF_BUS_CHECK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfBusCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_XPF_CACHE_CHECK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfCacheCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_XPF_CMC_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("NumberOfBanks", SimTypeChar(label="Byte")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Notify", SimTypeRef("WHEA_NOTIFICATION_DESCRIPTOR", SimStruct)), ("Banks", SimTypeArray(SimTypeRef("WHEA_XPF_MC_BANK_DESCRIPTOR", SimStruct), 32)),)), name="WHEA_XPF_CMC_DESCRIPTOR", pack=False, align=None),
    "WHEA_XPF_CONTEXT_INFO": SimStruct(OrderedDict((("RegisterContextType", SimTypeShort(signed=False, label="UInt16")), ("RegisterDataSize", SimTypeShort(signed=False, label="UInt16")), ("MSRAddress", SimTypeInt(signed=False, label="UInt32")), ("MmRegisterAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_XPF_CONTEXT_INFO", pack=False, align=None),
    "WHEA_XPF_MCA_SECTION": SimStruct(OrderedDict((("VersionNumber", SimTypeInt(signed=False, label="UInt32")), ("CpuVendor", SimTypeInt(signed=False, label="WHEA_CPU_VENDOR")), ("Timestamp", SimTypeLongLong(signed=True, label="Int64")), ("ProcessorNumber", SimTypeInt(signed=False, label="UInt32")), ("GlobalStatus", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("InstructionPointer", SimTypeLongLong(signed=False, label="UInt64")), ("BankNumber", SimTypeInt(signed=False, label="UInt32")), ("Status", SimUnion({"CommonBits": SimTypeRef("MCI_STATUS_BITS_COMMON", SimStruct), "AmdBits": SimTypeRef("MCI_STATUS_AMD_BITS", SimStruct), "IntelBits": SimTypeRef("MCI_STATUS_INTEL_BITS", SimStruct), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Misc", SimTypeLongLong(signed=False, label="UInt64")), ("ExtendedRegisterCount", SimTypeInt(signed=False, label="UInt32")), ("ApicId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ExtendedRegisters": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 24), "AMDExtendedRegisters": SimTypeRef("WHEA_AMD_EXTENDED_REGISTERS", SimStruct)}, name="<anon>", label="None")), ("GlobalCapability", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("RecoveryInfo", SimTypeRef("XPF_RECOVERY_INFO", SimStruct)),)), name="WHEA_XPF_MCA_SECTION", pack=False, align=None),
    "WHEA_XPF_MCE_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")), ("NumberOfBanks", SimTypeChar(label="Byte")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MCG_Capability", SimTypeLongLong(signed=False, label="UInt64")), ("MCG_GlobalControl", SimTypeLongLong(signed=False, label="UInt64")), ("Banks", SimTypeArray(SimTypeRef("WHEA_XPF_MC_BANK_DESCRIPTOR", SimStruct), 32)),)), name="WHEA_XPF_MCE_DESCRIPTOR", pack=False, align=None),
    "WHEA_XPF_MC_BANK_DESCRIPTOR": SimStruct(OrderedDict((("BankNumber", SimTypeChar(label="Byte")), ("ClearOnInitialization", SimTypeChar(label="Byte")), ("StatusDataFormat", SimTypeChar(label="Byte")), ("Flags", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("ControlMsr", SimTypeInt(signed=False, label="UInt32")), ("StatusMsr", SimTypeInt(signed=False, label="UInt32")), ("AddressMsr", SimTypeInt(signed=False, label="UInt32")), ("MiscMsr", SimTypeInt(signed=False, label="UInt32")), ("ControlData", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_XPF_MC_BANK_DESCRIPTOR", pack=False, align=None),
    "WHEA_XPF_MS_CHECK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfMsCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_XPF_NMI_DESCRIPTOR": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("Enabled", SimTypeChar(label="Byte")),)), name="WHEA_XPF_NMI_DESCRIPTOR", pack=False, align=None),
    "WHEA_XPF_PROCESSOR_ERROR_SECTION": SimStruct(OrderedDict((("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("LocalAPICId", SimTypeLongLong(signed=False, label="UInt64")), ("CpuId", SimTypeArray(SimTypeChar(label="Byte"), 48)), ("VariableInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WHEA_XPF_PROCESSOR_ERROR_SECTION", pack=False, align=None),
    "WHEA_XPF_PROCESSOR_ERROR_SECTION_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_XPF_PROCINFO": SimStruct(OrderedDict((("CheckInfoId", SimTypeBottom(label="Guid")), ("ValidBits", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("CheckInfo", SimUnion({"CacheCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfCacheCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "TlbCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfTLBCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "BusCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfBusCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "MsCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfMsCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("TargetId", SimTypeLongLong(signed=False, label="UInt64")), ("RequesterId", SimTypeLongLong(signed=False, label="UInt64")), ("ResponderId", SimTypeLongLong(signed=False, label="UInt64")), ("InstructionPointer", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHEA_XPF_PROCINFO", pack=False, align=None),
    "WHEA_XPF_PROCINFO_VALIDBITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ValidBits": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHEA_XPF_TLB_CHECK": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfTLBCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_ACCESS_GPA_CONTROLS": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("CacheType", SimTypeInt(signed=False, label="WHV_CACHE_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "WHV_ADVISE_GPA_RANGE": SimUnion({"Populate": SimTypeRef("WHV_ADVISE_GPA_RANGE_POPULATE", SimStruct)}, name="<anon>", label="None"),
    "WHV_ADVISE_GPA_RANGE_CODE": SimTypeInt(signed=False, label="WHV_ADVISE_GPA_RANGE_CODE"),
    "WHV_ADVISE_GPA_RANGE_POPULATE": SimStruct(OrderedDict((("Flags", SimUnion({"AsUINT32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("AccessType", SimTypeInt(signed=False, label="WHV_MEMORY_ACCESS_TYPE")),)), name="WHV_ADVISE_GPA_RANGE_POPULATE", pack=False, align=None),
    "WHV_ADVISE_GPA_RANGE_POPULATE_FLAGS": SimUnion({"AsUINT32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "WHV_ALLOCATE_VPCI_RESOURCE_FLAGS": SimTypeInt(signed=False, label="WHV_ALLOCATE_VPCI_RESOURCE_FLAGS"),
    "WHV_CACHE_TYPE": SimTypeInt(signed=False, label="WHV_CACHE_TYPE"),
    "WHV_CAPABILITY": SimUnion({"HypervisorPresent": SimTypeInt(signed=True, label="Int32"), "Features": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ExtendedVmExits": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ProcessorVendor": SimTypeInt(signed=False, label="WHV_PROCESSOR_VENDOR"), "ProcessorFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "SyntheticProcessorFeaturesBanks": SimTypeRef("WHV_SYNTHETIC_PROCESSOR_FEATURES_BANKS", SimStruct), "ProcessorXsaveFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ProcessorClFlushSize": SimTypeChar(label="Byte"), "ExceptionExitBitmap": SimTypeLongLong(signed=False, label="UInt64"), "X64MsrExitBitmap": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "ProcessorClockFrequency": SimTypeLongLong(signed=False, label="UInt64"), "InterruptClockFrequency": SimTypeLongLong(signed=False, label="UInt64"), "ProcessorFeaturesBanks": SimTypeRef("WHV_PROCESSOR_FEATURES_BANKS", SimStruct), "GpaRangePopulateFlags": SimUnion({"AsUINT32": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "ProcessorFrequencyCap": SimTypeRef("WHV_CAPABILITY_PROCESSOR_FREQUENCY_CAP", SimStruct), "ProcessorPerfmonFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "SchedulerFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WHV_CAPABILITY_CODE": SimTypeInt(signed=False, label="WHV_CAPABILITY_CODE"),
    "WHV_CAPABILITY_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_CAPABILITY_PROCESSOR_FREQUENCY_CAP": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("HighestFrequencyMhz", SimTypeInt(signed=False, label="UInt32")), ("NominalFrequencyMhz", SimTypeInt(signed=False, label="UInt32")), ("LowestFrequencyMhz", SimTypeInt(signed=False, label="UInt32")), ("FrequencyStepMhz", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_CAPABILITY_PROCESSOR_FREQUENCY_CAP", pack=False, align=None),
    "WHV_CPUID_OUTPUT": SimStruct(OrderedDict((("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_CPUID_OUTPUT", pack=False, align=None),
    "WHV_CREATE_VPCI_DEVICE_FLAGS": SimTypeInt(signed=False, label="WHV_CREATE_VPCI_DEVICE_FLAGS"),
    "WHV_DOORBELL_MATCH_DATA": SimStruct(OrderedDict((("GuestAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Value", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_DOORBELL_MATCH_DATA", pack=False, align=None),
    "WHV_EMULATOR_CALLBACKS": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("WHvEmulatorIoPortCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHV_EMULATOR_IO_ACCESS_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "IoAccess"]), offset=0)), ("WHvEmulatorMemoryCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHV_EMULATOR_MEMORY_ACCESS_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "MemoryAccess"]), offset=0)), ("WHvEmulatorGetVirtualProcessorRegisters", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WHV_REGISTER_NAME"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimUnion({"Reg128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"), "Reg64": SimTypeLongLong(signed=False, label="UInt64"), "Reg32": SimTypeInt(signed=False, label="UInt32"), "Reg16": SimTypeShort(signed=False, label="UInt16"), "Reg8": SimTypeChar(label="Byte"), "Fp": SimUnion({"Anonymous": SimStruct(OrderedDict((("Mantissa", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "FpControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "XmmControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "Segment": SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct), "Table": SimTypeRef("WHV_X64_TABLE_REGISTER", SimStruct), "InterruptState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingInterruption": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "DeliverabilityNotifications": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ExceptionEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "ExtIntEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "InternalActivity": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingDebugException": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "RegisterNames", "RegisterCount", "RegisterValues"]), offset=0)), ("WHvEmulatorSetVirtualProcessorRegisters", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WHV_REGISTER_NAME"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimUnion({"Reg128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"), "Reg64": SimTypeLongLong(signed=False, label="UInt64"), "Reg32": SimTypeInt(signed=False, label="UInt32"), "Reg16": SimTypeShort(signed=False, label="UInt16"), "Reg8": SimTypeChar(label="Byte"), "Fp": SimUnion({"Anonymous": SimStruct(OrderedDict((("Mantissa", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "FpControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "XmmControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "Segment": SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct), "Table": SimTypeRef("WHV_X64_TABLE_REGISTER", SimStruct), "InterruptState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingInterruption": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "DeliverabilityNotifications": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ExceptionEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "ExtIntEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "InternalActivity": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingDebugException": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "RegisterNames", "RegisterCount", "RegisterValues"]), offset=0)), ("WHvEmulatorTranslateGvaPage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_FLAGS"), SimTypePointer(SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_RESULT_CODE"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Gva", "TranslateFlags", "TranslationResult", "Gpa"]), offset=0)),)), name="WHV_EMULATOR_CALLBACKS", pack=False, align=None),
    "WHV_EMULATOR_GET_VIRTUAL_PROCESSOR_REGISTERS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WHV_REGISTER_NAME"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimUnion({"Reg128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"), "Reg64": SimTypeLongLong(signed=False, label="UInt64"), "Reg32": SimTypeInt(signed=False, label="UInt32"), "Reg16": SimTypeShort(signed=False, label="UInt16"), "Reg8": SimTypeChar(label="Byte"), "Fp": SimUnion({"Anonymous": SimStruct(OrderedDict((("Mantissa", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "FpControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "XmmControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "Segment": SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct), "Table": SimTypeRef("WHV_X64_TABLE_REGISTER", SimStruct), "InterruptState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingInterruption": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "DeliverabilityNotifications": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ExceptionEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "ExtIntEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "InternalActivity": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingDebugException": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "RegisterNames", "RegisterCount", "RegisterValues"]), offset=0),
    "WHV_EMULATOR_IO_ACCESS_INFO": SimStruct(OrderedDict((("Direction", SimTypeChar(label="Byte")), ("Port", SimTypeShort(signed=False, label="UInt16")), ("AccessSize", SimTypeShort(signed=False, label="UInt16")), ("Data", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_EMULATOR_IO_ACCESS_INFO", pack=False, align=None),
    "WHV_EMULATOR_IO_PORT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHV_EMULATOR_IO_ACCESS_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "IoAccess"]), offset=0),
    "WHV_EMULATOR_MEMORY_ACCESS_INFO": SimStruct(OrderedDict((("GpaAddress", SimTypeLongLong(signed=False, label="UInt64")), ("Direction", SimTypeChar(label="Byte")), ("AccessSize", SimTypeChar(label="Byte")), ("Data", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="WHV_EMULATOR_MEMORY_ACCESS_INFO", pack=False, align=None),
    "WHV_EMULATOR_MEMORY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WHV_EMULATOR_MEMORY_ACCESS_INFO", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "MemoryAccess"]), offset=0),
    "WHV_EMULATOR_SET_VIRTUAL_PROCESSOR_REGISTERS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WHV_REGISTER_NAME"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimUnion({"Reg128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"), "Reg64": SimTypeLongLong(signed=False, label="UInt64"), "Reg32": SimTypeInt(signed=False, label="UInt32"), "Reg16": SimTypeShort(signed=False, label="UInt16"), "Reg8": SimTypeChar(label="Byte"), "Fp": SimUnion({"Anonymous": SimStruct(OrderedDict((("Mantissa", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "FpControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "XmmControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "Segment": SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct), "Table": SimTypeRef("WHV_X64_TABLE_REGISTER", SimStruct), "InterruptState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingInterruption": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "DeliverabilityNotifications": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ExceptionEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "ExtIntEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "InternalActivity": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingDebugException": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "RegisterNames", "RegisterCount", "RegisterValues"]), offset=0),
    "WHV_EMULATOR_STATUS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHV_EMULATOR_TRANSLATE_GVA_PAGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_FLAGS"), SimTypePointer(SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_RESULT_CODE"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Context", "Gva", "TranslateFlags", "TranslationResult", "Gpa"]), offset=0),
    "WHV_EXCEPTION_TYPE": SimTypeInt(signed=False, label="WHV_EXCEPTION_TYPE"),
    "WHV_EXTENDED_VM_EXITS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_HYPERCALL_CONTEXT": SimStruct(OrderedDict((("Rax", SimTypeLongLong(signed=False, label="UInt64")), ("Rbx", SimTypeLongLong(signed=False, label="UInt64")), ("Rcx", SimTypeLongLong(signed=False, label="UInt64")), ("Rdx", SimTypeLongLong(signed=False, label="UInt64")), ("R8", SimTypeLongLong(signed=False, label="UInt64")), ("Rsi", SimTypeLongLong(signed=False, label="UInt64")), ("Rdi", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved0", SimTypeLongLong(signed=False, label="UInt64")), ("XmmRegisters", SimTypeArray(SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"), 6)), ("Reserved1", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="WHV_HYPERCALL_CONTEXT", pack=False, align=None),
    "WHV_INTERNAL_ACTIVITY_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_INTERRUPT_CONTROL": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Destination", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_INTERRUPT_CONTROL", pack=False, align=None),
    "WHV_INTERRUPT_DESTINATION_MODE": SimTypeInt(signed=False, label="WHV_INTERRUPT_DESTINATION_MODE"),
    "WHV_INTERRUPT_TRIGGER_MODE": SimTypeInt(signed=False, label="WHV_INTERRUPT_TRIGGER_MODE"),
    "WHV_INTERRUPT_TYPE": SimTypeInt(signed=False, label="WHV_INTERRUPT_TYPE"),
    "WHV_MAP_GPA_RANGE_FLAGS": SimTypeInt(signed=False, label="WHV_MAP_GPA_RANGE_FLAGS"),
    "WHV_MEMORY_ACCESS_CONTEXT": SimStruct(OrderedDict((("InstructionByteCount", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("InstructionBytes", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("AccessInfo", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("Gpa", SimTypeLongLong(signed=False, label="UInt64")), ("Gva", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_MEMORY_ACCESS_CONTEXT", pack=False, align=None),
    "WHV_MEMORY_ACCESS_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHV_MEMORY_ACCESS_TYPE": SimTypeInt(signed=False, label="WHV_MEMORY_ACCESS_TYPE"),
    "WHV_MEMORY_RANGE_ENTRY": SimStruct(OrderedDict((("GuestAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_MEMORY_RANGE_ENTRY", pack=False, align=None),
    "WHV_MSR_ACTION": SimTypeInt(signed=False, label="WHV_MSR_ACTION"),
    "WHV_MSR_ACTION_ENTRY": SimStruct(OrderedDict((("Index", SimTypeInt(signed=False, label="UInt32")), ("ReadAction", SimTypeChar(label="Byte")), ("WriteAction", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="WHV_MSR_ACTION_ENTRY", pack=False, align=None),
    "WHV_NOTIFICATION_PORT_PARAMETERS": SimStruct(OrderedDict((("NotificationPortType", SimTypeInt(signed=False, label="WHV_NOTIFICATION_PORT_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Doorbell": SimTypeRef("WHV_DOORBELL_MATCH_DATA", SimStruct), "Event": SimTypeRef("_Event_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WHV_NOTIFICATION_PORT_PARAMETERS", pack=False, align=None),
    "WHV_NOTIFICATION_PORT_PROPERTY_CODE": SimTypeInt(signed=False, label="WHV_NOTIFICATION_PORT_PROPERTY_CODE"),
    "WHV_NOTIFICATION_PORT_TYPE": SimTypeInt(signed=False, label="WHV_NOTIFICATION_PORT_TYPE"),
    "WHV_PARTITION_COUNTER_SET": SimTypeInt(signed=False, label="WHV_PARTITION_COUNTER_SET"),
    "WHV_PARTITION_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WHV_PARTITION_MEMORY_COUNTERS": SimStruct(OrderedDict((("Mapped4KPageCount", SimTypeLongLong(signed=False, label="UInt64")), ("Mapped2MPageCount", SimTypeLongLong(signed=False, label="UInt64")), ("Mapped1GPageCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_PARTITION_MEMORY_COUNTERS", pack=False, align=None),
    "WHV_PARTITION_PROPERTY": SimUnion({"ExtendedVmExits": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ProcessorFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "SyntheticProcessorFeaturesBanks": SimTypeRef("WHV_SYNTHETIC_PROCESSOR_FEATURES_BANKS", SimStruct), "ProcessorXsaveFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ProcessorClFlushSize": SimTypeChar(label="Byte"), "ProcessorCount": SimTypeInt(signed=False, label="UInt32"), "CpuidExitList": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "CpuidResultList": SimTypePointer(SimTypeRef("WHV_X64_CPUID_RESULT", SimStruct), offset=0), "CpuidResultList2": SimTypePointer(SimTypeRef("WHV_X64_CPUID_RESULT2", SimStruct), offset=0), "MsrActionList": SimTypePointer(SimTypeRef("WHV_MSR_ACTION_ENTRY", SimStruct), offset=0), "UnimplementedMsrAction": SimTypeInt(signed=False, label="WHV_MSR_ACTION"), "ExceptionExitBitmap": SimTypeLongLong(signed=False, label="UInt64"), "LocalApicEmulationMode": SimTypeInt(signed=False, label="WHV_X64_LOCAL_APIC_EMULATION_MODE"), "SeparateSecurityDomain": SimTypeInt(signed=True, label="Int32"), "NestedVirtualization": SimTypeInt(signed=True, label="Int32"), "X64MsrExitBitmap": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "ProcessorClockFrequency": SimTypeLongLong(signed=False, label="UInt64"), "InterruptClockFrequency": SimTypeLongLong(signed=False, label="UInt64"), "ApicRemoteRead": SimTypeInt(signed=True, label="Int32"), "ProcessorFeaturesBanks": SimTypeRef("WHV_PROCESSOR_FEATURES_BANKS", SimStruct), "ReferenceTime": SimTypeLongLong(signed=False, label="UInt64"), "PrimaryNumaNode": SimTypeShort(signed=False, label="UInt16"), "CpuReserve": SimTypeInt(signed=False, label="UInt32"), "CpuCap": SimTypeInt(signed=False, label="UInt32"), "CpuWeight": SimTypeInt(signed=False, label="UInt32"), "CpuGroupId": SimTypeLongLong(signed=False, label="UInt64"), "ProcessorFrequencyCap": SimTypeInt(signed=False, label="UInt32"), "AllowDeviceAssignment": SimTypeInt(signed=True, label="Int32"), "ProcessorPerfmonFeatures": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "DisableSmt": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"),
    "WHV_PARTITION_PROPERTY_CODE": SimTypeInt(signed=False, label="WHV_PARTITION_PROPERTY_CODE"),
    "WHV_PROCESSOR_APIC_COUNTERS": SimStruct(OrderedDict((("MmioAccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("EoiAccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("TprAccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("SentIpiCount", SimTypeLongLong(signed=False, label="UInt64")), ("SelfIpiCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_PROCESSOR_APIC_COUNTERS", pack=False, align=None),
    "WHV_PROCESSOR_COUNTER_SET": SimTypeInt(signed=False, label="WHV_PROCESSOR_COUNTER_SET"),
    "WHV_PROCESSOR_EVENT_COUNTERS": SimStruct(OrderedDict((("PageFaultCount", SimTypeLongLong(signed=False, label="UInt64")), ("ExceptionCount", SimTypeLongLong(signed=False, label="UInt64")), ("InterruptCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_PROCESSOR_EVENT_COUNTERS", pack=False, align=None),
    "WHV_PROCESSOR_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_PROCESSOR_FEATURES1": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_PROCESSOR_FEATURES_BANKS": SimStruct(OrderedDict((("BanksCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Bank0", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Bank1", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)}, name="<anon>", label="None")),)), name="WHV_PROCESSOR_FEATURES_BANKS", pack=False, align=None),
    "WHV_PROCESSOR_INTERCEPT_COUNTER": SimStruct(OrderedDict((("Count", SimTypeLongLong(signed=False, label="UInt64")), ("Time100ns", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_PROCESSOR_INTERCEPT_COUNTER", pack=False, align=None),
    "WHV_PROCESSOR_INTERCEPT_COUNTERS": SimStruct(OrderedDict((("PageInvalidations", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("ControlRegisterAccesses", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("IoInstructions", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("HaltInstructions", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("CpuidInstructions", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("MsrAccesses", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("OtherIntercepts", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("PendingInterrupts", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("EmulatedInstructions", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("DebugRegisterAccesses", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("PageFaultIntercepts", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("NestedPageFaultIntercepts", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("Hypercalls", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)), ("RdpmcInstructions", SimTypeRef("WHV_PROCESSOR_INTERCEPT_COUNTER", SimStruct)),)), name="WHV_PROCESSOR_INTERCEPT_COUNTERS", pack=False, align=None),
    "WHV_PROCESSOR_PERFMON_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_PROCESSOR_RUNTIME_COUNTERS": SimStruct(OrderedDict((("TotalRuntime100ns", SimTypeLongLong(signed=False, label="UInt64")), ("HypervisorRuntime100ns", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_PROCESSOR_RUNTIME_COUNTERS", pack=False, align=None),
    "WHV_PROCESSOR_SYNTHETIC_FEATURES_COUNTERS": SimStruct(OrderedDict((("SyntheticInterruptsCount", SimTypeLongLong(signed=False, label="UInt64")), ("LongSpinWaitHypercallsCount", SimTypeLongLong(signed=False, label="UInt64")), ("OtherHypercallsCount", SimTypeLongLong(signed=False, label="UInt64")), ("SyntheticInterruptHypercallsCount", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualInterruptHypercallsCount", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualMmuHypercallsCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_PROCESSOR_SYNTHETIC_FEATURES_COUNTERS", pack=False, align=None),
    "WHV_PROCESSOR_VENDOR": SimTypeInt(signed=False, label="WHV_PROCESSOR_VENDOR"),
    "WHV_PROCESSOR_XSAVE_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_REGISTER_NAME": SimTypeInt(signed=False, label="WHV_REGISTER_NAME"),
    "WHV_REGISTER_VALUE": SimUnion({"Reg128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"), "Reg64": SimTypeLongLong(signed=False, label="UInt64"), "Reg32": SimTypeInt(signed=False, label="UInt32"), "Reg16": SimTypeShort(signed=False, label="UInt16"), "Reg8": SimTypeChar(label="Byte"), "Fp": SimUnion({"Anonymous": SimStruct(OrderedDict((("Mantissa", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "FpControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "XmmControlStatus": SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "Segment": SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct), "Table": SimTypeRef("WHV_X64_TABLE_REGISTER", SimStruct), "InterruptState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingInterruption": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "DeliverabilityNotifications": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "ExceptionEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "ExtIntEvent": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"), "InternalActivity": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "PendingDebugException": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WHV_RUN_VP_CANCELED_CONTEXT": SimStruct(OrderedDict((("CancelReason", SimTypeInt(signed=False, label="WHV_RUN_VP_CANCEL_REASON")),)), name="WHV_RUN_VP_CANCELED_CONTEXT", pack=False, align=None),
    "WHV_RUN_VP_CANCEL_REASON": SimTypeInt(signed=False, label="WHV_RUN_VP_CANCEL_REASON"),
    "WHV_RUN_VP_EXIT_CONTEXT": SimStruct(OrderedDict((("ExitReason", SimTypeInt(signed=False, label="WHV_RUN_VP_EXIT_REASON")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("VpContext", SimTypeRef("WHV_VP_EXIT_CONTEXT", SimStruct)), ("Anonymous", SimUnion({"MemoryAccess": SimTypeRef("WHV_MEMORY_ACCESS_CONTEXT", SimStruct), "IoPortAccess": SimTypeRef("WHV_X64_IO_PORT_ACCESS_CONTEXT", SimStruct), "MsrAccess": SimTypeRef("WHV_X64_MSR_ACCESS_CONTEXT", SimStruct), "CpuidAccess": SimTypeRef("WHV_X64_CPUID_ACCESS_CONTEXT", SimStruct), "VpException": SimTypeRef("WHV_VP_EXCEPTION_CONTEXT", SimStruct), "InterruptWindow": SimTypeRef("WHV_X64_INTERRUPTION_DELIVERABLE_CONTEXT", SimStruct), "UnsupportedFeature": SimTypeRef("WHV_X64_UNSUPPORTED_FEATURE_CONTEXT", SimStruct), "CancelReason": SimTypeRef("WHV_RUN_VP_CANCELED_CONTEXT", SimStruct), "ApicEoi": SimTypeRef("WHV_X64_APIC_EOI_CONTEXT", SimStruct), "ReadTsc": SimTypeRef("WHV_X64_RDTSC_CONTEXT", SimStruct), "ApicSmi": SimTypeRef("WHV_X64_APIC_SMI_CONTEXT", SimStruct), "Hypercall": SimTypeRef("WHV_HYPERCALL_CONTEXT", SimStruct), "ApicInitSipi": SimTypeRef("WHV_X64_APIC_INIT_SIPI_CONTEXT", SimStruct), "ApicWrite": SimTypeRef("WHV_X64_APIC_WRITE_CONTEXT", SimStruct), "SynicSintDeliverable": SimTypeRef("WHV_SYNIC_SINT_DELIVERABLE_CONTEXT", SimStruct)}, name="<anon>", label="None")),)), name="WHV_RUN_VP_EXIT_CONTEXT", pack=False, align=None),
    "WHV_RUN_VP_EXIT_REASON": SimTypeInt(signed=False, label="WHV_RUN_VP_EXIT_REASON"),
    "WHV_SCHEDULER_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_SRIOV_RESOURCE_DESCRIPTOR": SimStruct(OrderedDict((("PnpInstanceId", SimTypeArray(SimTypeChar(label="Char"), 200)), ("VirtualFunctionId", SimTypeRef("LUID", SimStruct)), ("VirtualFunctionIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="WHV_SRIOV_RESOURCE_DESCRIPTOR", pack=False, align=None),
    "WHV_SYNIC_EVENT_PARAMETERS": SimStruct(OrderedDict((("VpIndex", SimTypeInt(signed=False, label="UInt32")), ("TargetSint", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("FlagNumber", SimTypeShort(signed=False, label="UInt16")),)), name="WHV_SYNIC_EVENT_PARAMETERS", pack=False, align=None),
    "WHV_SYNIC_SINT_DELIVERABLE_CONTEXT": SimStruct(OrderedDict((("DeliverableSints", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_SYNIC_SINT_DELIVERABLE_CONTEXT", pack=False, align=None),
    "WHV_SYNTHETIC_PROCESSOR_FEATURES": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_SYNTHETIC_PROCESSOR_FEATURES_BANKS": SimStruct(OrderedDict((("BanksCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Bank0", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)}, name="<anon>", label="None")),)), name="WHV_SYNTHETIC_PROCESSOR_FEATURES_BANKS", pack=False, align=None),
    "WHV_TRANSLATE_GVA_FLAGS": SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_FLAGS"),
    "WHV_TRANSLATE_GVA_RESULT": SimStruct(OrderedDict((("ResultCode", SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_RESULT_CODE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_TRANSLATE_GVA_RESULT", pack=False, align=None),
    "WHV_TRANSLATE_GVA_RESULT_CODE": SimTypeInt(signed=False, label="WHV_TRANSLATE_GVA_RESULT_CODE"),
    "WHV_TRIGGER_PARAMETERS": SimStruct(OrderedDict((("TriggerType", SimTypeInt(signed=False, label="WHV_TRIGGER_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Interrupt": SimTypeRef("WHV_INTERRUPT_CONTROL", SimStruct), "SynicEvent": SimTypeRef("WHV_SYNIC_EVENT_PARAMETERS", SimStruct), "DeviceInterrupt": SimTypeRef("_DeviceInterrupt_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WHV_TRIGGER_PARAMETERS", pack=False, align=None),
    "WHV_TRIGGER_TYPE": SimTypeInt(signed=False, label="WHV_TRIGGER_TYPE"),
    "WHV_UINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None"),
    "WHV_VIRTUAL_PROCESSOR_PROPERTY": SimStruct(OrderedDict((("PropertyCode", SimTypeInt(signed=False, label="WHV_VIRTUAL_PROCESSOR_PROPERTY_CODE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"NumaNode": SimTypeShort(signed=False, label="UInt16"), "Padding": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="WHV_VIRTUAL_PROCESSOR_PROPERTY", pack=False, align=None),
    "WHV_VIRTUAL_PROCESSOR_PROPERTY_CODE": SimTypeInt(signed=False, label="WHV_VIRTUAL_PROCESSOR_PROPERTY_CODE"),
    "WHV_VIRTUAL_PROCESSOR_STATE_TYPE": SimTypeInt(signed=False, label="WHV_VIRTUAL_PROCESSOR_STATE_TYPE"),
    "WHV_VPCI_DEVICE_NOTIFICATION": SimStruct(OrderedDict((("NotificationType", SimTypeInt(signed=False, label="WHV_VPCI_DEVICE_NOTIFICATION_TYPE")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Reserved2": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="WHV_VPCI_DEVICE_NOTIFICATION", pack=False, align=None),
    "WHV_VPCI_DEVICE_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="WHV_VPCI_DEVICE_NOTIFICATION_TYPE"),
    "WHV_VPCI_DEVICE_PROPERTY_CODE": SimTypeInt(signed=False, label="WHV_VPCI_DEVICE_PROPERTY_CODE"),
    "WHV_VPCI_DEVICE_REGISTER": SimStruct(OrderedDict((("Location", SimTypeInt(signed=False, label="WHV_VPCI_DEVICE_REGISTER_SPACE")), ("SizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("OffsetInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_VPCI_DEVICE_REGISTER", pack=False, align=None),
    "WHV_VPCI_DEVICE_REGISTER_SPACE": SimTypeInt(signed=False, label="WHV_VPCI_DEVICE_REGISTER_SPACE"),
    "WHV_VPCI_HARDWARE_IDS": SimStruct(OrderedDict((("VendorID", SimTypeShort(signed=False, label="UInt16")), ("DeviceID", SimTypeShort(signed=False, label="UInt16")), ("RevisionID", SimTypeChar(label="Byte")), ("ProgIf", SimTypeChar(label="Byte")), ("SubClass", SimTypeChar(label="Byte")), ("BaseClass", SimTypeChar(label="Byte")), ("SubVendorID", SimTypeShort(signed=False, label="UInt16")), ("SubSystemID", SimTypeShort(signed=False, label="UInt16")),)), name="WHV_VPCI_HARDWARE_IDS", pack=False, align=None),
    "WHV_VPCI_INTERRUPT_TARGET": SimStruct(OrderedDict((("Vector", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="WHV_VPCI_INTERRUPT_TARGET_FLAGS")), ("ProcessorCount", SimTypeInt(signed=False, label="UInt32")), ("Processors", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="WHV_VPCI_INTERRUPT_TARGET", pack=False, align=None),
    "WHV_VPCI_INTERRUPT_TARGET_FLAGS": SimTypeInt(signed=False, label="WHV_VPCI_INTERRUPT_TARGET_FLAGS"),
    "WHV_VPCI_MMIO_MAPPING": SimStruct(OrderedDict((("Location", SimTypeInt(signed=False, label="WHV_VPCI_DEVICE_REGISTER_SPACE")), ("Flags", SimTypeInt(signed=False, label="WHV_VPCI_MMIO_RANGE_FLAGS")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("OffsetInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WHV_VPCI_MMIO_MAPPING", pack=False, align=None),
    "WHV_VPCI_MMIO_RANGE_FLAGS": SimTypeInt(signed=False, label="WHV_VPCI_MMIO_RANGE_FLAGS"),
    "WHV_VPCI_PROBED_BARS": SimStruct(OrderedDict((("Value", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)),)), name="WHV_VPCI_PROBED_BARS", pack=False, align=None),
    "WHV_VP_EXCEPTION_CONTEXT": SimStruct(OrderedDict((("InstructionByteCount", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("InstructionBytes", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("ExceptionInfo", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("ExceptionType", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_VP_EXCEPTION_CONTEXT", pack=False, align=None),
    "WHV_VP_EXCEPTION_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHV_VP_EXIT_CONTEXT": SimStruct(OrderedDict((("ExecutionState", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT16": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")), ("_bitfield", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Cs", SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct)), ("Rip", SimTypeLongLong(signed=False, label="UInt64")), ("Rflags", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_VP_EXIT_CONTEXT", pack=False, align=None),
    "WHV_X64_APIC_EOI_CONTEXT": SimStruct(OrderedDict((("InterruptVector", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_X64_APIC_EOI_CONTEXT", pack=False, align=None),
    "WHV_X64_APIC_INIT_SIPI_CONTEXT": SimStruct(OrderedDict((("ApicIcr", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_APIC_INIT_SIPI_CONTEXT", pack=False, align=None),
    "WHV_X64_APIC_SMI_CONTEXT": SimStruct(OrderedDict((("ApicIcr", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_APIC_SMI_CONTEXT", pack=False, align=None),
    "WHV_X64_APIC_WRITE_CONTEXT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="WHV_X64_APIC_WRITE_TYPE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("WriteValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_APIC_WRITE_CONTEXT", pack=False, align=None),
    "WHV_X64_APIC_WRITE_TYPE": SimTypeInt(signed=False, label="WHV_X64_APIC_WRITE_TYPE"),
    "WHV_X64_CPUID_ACCESS_CONTEXT": SimStruct(OrderedDict((("Rax", SimTypeLongLong(signed=False, label="UInt64")), ("Rcx", SimTypeLongLong(signed=False, label="UInt64")), ("Rdx", SimTypeLongLong(signed=False, label="UInt64")), ("Rbx", SimTypeLongLong(signed=False, label="UInt64")), ("DefaultResultRax", SimTypeLongLong(signed=False, label="UInt64")), ("DefaultResultRcx", SimTypeLongLong(signed=False, label="UInt64")), ("DefaultResultRdx", SimTypeLongLong(signed=False, label="UInt64")), ("DefaultResultRbx", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_CPUID_ACCESS_CONTEXT", pack=False, align=None),
    "WHV_X64_CPUID_RESULT": SimStruct(OrderedDict((("Function", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")),)), name="WHV_X64_CPUID_RESULT", pack=False, align=None),
    "WHV_X64_CPUID_RESULT2": SimStruct(OrderedDict((("Function", SimTypeInt(signed=False, label="UInt32")), ("Index", SimTypeInt(signed=False, label="UInt32")), ("VpIndex", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="WHV_X64_CPUID_RESULT2_FLAGS")), ("Output", SimTypeRef("WHV_CPUID_OUTPUT", SimStruct)), ("Mask", SimTypeRef("WHV_CPUID_OUTPUT", SimStruct)),)), name="WHV_X64_CPUID_RESULT2", pack=False, align=None),
    "WHV_X64_CPUID_RESULT2_FLAGS": SimTypeInt(signed=False, label="WHV_X64_CPUID_RESULT2_FLAGS"),
    "WHV_X64_DELIVERABILITY_NOTIFICATIONS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_X64_FP_CONTROL_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WHV_X64_FP_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("Mantissa", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WHV_X64_INTERRUPTION_DELIVERABLE_CONTEXT": SimStruct(OrderedDict((("DeliverableType", SimTypeInt(signed=False, label="WHV_X64_PENDING_INTERRUPTION_TYPE")),)), name="WHV_X64_INTERRUPTION_DELIVERABLE_CONTEXT", pack=False, align=None),
    "WHV_X64_INTERRUPT_STATE_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_X64_IO_PORT_ACCESS_CONTEXT": SimStruct(OrderedDict((("InstructionByteCount", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("InstructionBytes", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("AccessInfo", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)), ("Rax", SimTypeLongLong(signed=False, label="UInt64")), ("Rcx", SimTypeLongLong(signed=False, label="UInt64")), ("Rsi", SimTypeLongLong(signed=False, label="UInt64")), ("Rdi", SimTypeLongLong(signed=False, label="UInt64")), ("Ds", SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct)), ("Es", SimTypeRef("WHV_X64_SEGMENT_REGISTER", SimStruct)),)), name="WHV_X64_IO_PORT_ACCESS_CONTEXT", pack=False, align=None),
    "WHV_X64_IO_PORT_ACCESS_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHV_X64_LOCAL_APIC_EMULATION_MODE": SimTypeInt(signed=False, label="WHV_X64_LOCAL_APIC_EMULATION_MODE"),
    "WHV_X64_MSR_ACCESS_CONTEXT": SimStruct(OrderedDict((("AccessInfo", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("MsrNumber", SimTypeInt(signed=False, label="UInt32")), ("Rax", SimTypeLongLong(signed=False, label="UInt64")), ("Rdx", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_MSR_ACCESS_CONTEXT", pack=False, align=None),
    "WHV_X64_MSR_ACCESS_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT32": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "WHV_X64_MSR_EXIT_BITMAP": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "WHV_X64_PENDING_DEBUG_EXCEPTION": SimUnion({"AsUINT64": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "WHV_X64_PENDING_EVENT_TYPE": SimTypeInt(signed=False, label="WHV_X64_PENDING_EVENT_TYPE"),
    "WHV_X64_PENDING_EXCEPTION_EVENT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")), ("ExceptionParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WHV_X64_PENDING_EXT_INT_EVENT": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WHV_X64_PENDING_INTERRUPTION_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("ErrorCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_X64_PENDING_INTERRUPTION_TYPE": SimTypeInt(signed=False, label="WHV_X64_PENDING_INTERRUPTION_TYPE"),
    "WHV_X64_RDTSC_CONTEXT": SimStruct(OrderedDict((("TscAux", SimTypeLongLong(signed=False, label="UInt64")), ("VirtualOffset", SimTypeLongLong(signed=False, label="UInt64")), ("Tsc", SimTypeLongLong(signed=False, label="UInt64")), ("ReferenceTime", SimTypeLongLong(signed=False, label="UInt64")), ("RdtscInfo", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")),)), name="WHV_X64_RDTSC_CONTEXT", pack=False, align=None),
    "WHV_X64_RDTSC_INFO": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT64": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "WHV_X64_SEGMENT_REGISTER": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=False, label="UInt64")), ("Limit", SimTypeInt(signed=False, label="UInt32")), ("Selector", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Attributes": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="WHV_X64_SEGMENT_REGISTER", pack=False, align=None),
    "WHV_X64_TABLE_REGISTER": SimStruct(OrderedDict((("Pad", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 3)), ("Limit", SimTypeShort(signed=False, label="UInt16")), ("Base", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_TABLE_REGISTER", pack=False, align=None),
    "WHV_X64_UNSUPPORTED_FEATURE_CODE": SimTypeInt(signed=False, label="WHV_X64_UNSUPPORTED_FEATURE_CODE"),
    "WHV_X64_UNSUPPORTED_FEATURE_CONTEXT": SimStruct(OrderedDict((("FeatureCode", SimTypeInt(signed=False, label="WHV_X64_UNSUPPORTED_FEATURE_CODE")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("FeatureParameter", SimTypeLongLong(signed=False, label="UInt64")),)), name="WHV_X64_UNSUPPORTED_FEATURE_CONTEXT", pack=False, align=None),
    "WHV_X64_VP_EXECUTION_STATE": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT16": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "WHV_X64_XMM_CONTROL_STATUS_REGISTER": SimUnion({"Anonymous": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUINT128": SimUnion({"Anonymous": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Dword": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)}, name="<anon>", label="None")}, name="<anon>", label="None"),
    "WIAS_CHANGED_VALUE_INFO": SimStruct(OrderedDict((("bChanged", SimTypeInt(signed=True, label="Int32")), ("vt", SimTypeInt(signed=True, label="Int32")), ("Old", SimUnion({"lVal": SimTypeInt(signed=True, label="Int32"), "fltVal": SimTypeFloat(size=32), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "guidVal": SimTypeBottom(label="Guid")}, name="<anon>", label="None")), ("Current", SimUnion({"lVal": SimTypeInt(signed=True, label="Int32"), "fltVal": SimTypeFloat(size=32), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "guidVal": SimTypeBottom(label="Guid")}, name="<anon>", label="None")),)), name="WIAS_CHANGED_VALUE_INFO", pack=False, align=None),
    "WIAS_DOWN_SAMPLE_INFO": SimStruct(OrderedDict((("ulOriginalWidth", SimTypeInt(signed=False, label="UInt32")), ("ulOriginalHeight", SimTypeInt(signed=False, label="UInt32")), ("ulBitsPerPixel", SimTypeInt(signed=False, label="UInt32")), ("ulXRes", SimTypeInt(signed=False, label="UInt32")), ("ulYRes", SimTypeInt(signed=False, label="UInt32")), ("ulDownSampledWidth", SimTypeInt(signed=False, label="UInt32")), ("ulDownSampledHeight", SimTypeInt(signed=False, label="UInt32")), ("ulActualSize", SimTypeInt(signed=False, label="UInt32")), ("ulDestBufSize", SimTypeInt(signed=False, label="UInt32")), ("ulSrcBufSize", SimTypeInt(signed=False, label="UInt32")), ("pSrcBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDestBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WIAS_DOWN_SAMPLE_INFO", pack=False, align=None),
    "WIAS_ENDORSER_INFO": SimStruct(OrderedDict((("ulPageCount", SimTypeInt(signed=False, label="UInt32")), ("ulNumEndorserValues", SimTypeInt(signed=False, label="UInt32")), ("pEndorserValues", SimTypePointer(SimTypeRef("WIAS_ENDORSER_VALUE", SimStruct), offset=0)),)), name="WIAS_ENDORSER_INFO", pack=False, align=None),
    "WIAS_ENDORSER_VALUE": SimStruct(OrderedDict((("wszTokenName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIAS_ENDORSER_VALUE", pack=False, align=None),
    "WIAVIDEO_STATE": SimTypeInt(signed=False, label="WIAVIDEO_STATE"),
    "WIA_BARCODES": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Barcodes", SimTypePointer(SimTypeRef("WIA_BARCODE_INFO", SimStruct), offset=0)),)), name="WIA_BARCODES", pack=False, align=None),
    "WIA_BARCODE_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Page", SimTypeInt(signed=False, label="UInt32")), ("Confidence", SimTypeInt(signed=False, label="UInt32")), ("XOffset", SimTypeInt(signed=False, label="UInt32")), ("YOffset", SimTypeInt(signed=False, label="UInt32")), ("Rotation", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Text", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIA_BARCODE_INFO", pack=False, align=None),
    "WIA_DATA_CALLBACK_HEADER": SimStruct(OrderedDict((("lSize", SimTypeInt(signed=True, label="Int32")), ("guidFormatID", SimTypeBottom(label="Guid")), ("lBufferSize", SimTypeInt(signed=True, label="Int32")), ("lPageCount", SimTypeInt(signed=True, label="Int32")),)), name="WIA_DATA_CALLBACK_HEADER", pack=False, align=None),
    "WIA_DATA_TRANSFER_INFO": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("ulSection", SimTypeInt(signed=False, label="UInt32")), ("ulBufferSize", SimTypeInt(signed=False, label="UInt32")), ("bDoubleBuffer", SimTypeInt(signed=True, label="Int32")), ("ulReserved1", SimTypeInt(signed=False, label="UInt32")), ("ulReserved2", SimTypeInt(signed=False, label="UInt32")), ("ulReserved3", SimTypeInt(signed=False, label="UInt32")),)), name="WIA_DATA_TRANSFER_INFO", pack=False, align=None),
    "WIA_DEV_CAP": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("bstrName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrIcon", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrCommandline", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIA_DEV_CAP", pack=False, align=None),
    "WIA_DEV_CAP_DRV": SimStruct(OrderedDict((("guid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("ulFlags", SimTypeInt(signed=False, label="UInt32")), ("wszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszIcon", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIA_DEV_CAP_DRV", pack=False, align=None),
    "WIA_DITHER_PATTERN_DATA": SimStruct(OrderedDict((("lSize", SimTypeInt(signed=True, label="Int32")), ("bstrPatternName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lPatternWidth", SimTypeInt(signed=True, label="Int32")), ("lPatternLength", SimTypeInt(signed=True, label="Int32")), ("cbPattern", SimTypeInt(signed=True, label="Int32")), ("pbPattern", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WIA_DITHER_PATTERN_DATA", pack=False, align=None),
    "WIA_EXTENDED_TRANSFER_INFO": SimStruct(OrderedDict((("ulSize", SimTypeInt(signed=False, label="UInt32")), ("ulMinBufferSize", SimTypeInt(signed=False, label="UInt32")), ("ulOptimalBufferSize", SimTypeInt(signed=False, label="UInt32")), ("ulMaxBufferSize", SimTypeInt(signed=False, label="UInt32")), ("ulNumBuffers", SimTypeInt(signed=False, label="UInt32")),)), name="WIA_EXTENDED_TRANSFER_INFO", pack=False, align=None),
    "WIA_FORMAT_INFO": SimStruct(OrderedDict((("guidFormatID", SimTypeBottom(label="Guid")), ("lTymed", SimTypeInt(signed=True, label="Int32")),)), name="WIA_FORMAT_INFO", pack=False, align=None),
    "WIA_MICR": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Placeholder", SimTypeChar(label="Char")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("Micr", SimTypePointer(SimTypeRef("WIA_MICR_INFO", SimStruct), offset=0)),)), name="WIA_MICR", pack=False, align=None),
    "WIA_MICR_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Page", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Text", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIA_MICR_INFO", pack=False, align=None),
    "WIA_PATCH_CODES": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Count", SimTypeInt(signed=False, label="UInt32")), ("PatchCodes", SimTypePointer(SimTypeRef("WIA_PATCH_CODE_INFO", SimStruct), offset=0)),)), name="WIA_PATCH_CODES", pack=False, align=None),
    "WIA_PATCH_CODE_INFO": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")),)), name="WIA_PATCH_CODE_INFO", pack=False, align=None),
    "WIA_PROPERTY_CONTEXT": SimStruct(OrderedDict((("cProps", SimTypeInt(signed=False, label="UInt32")), ("pProps", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("pChanged", SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)),)), name="WIA_PROPERTY_CONTEXT", pack=False, align=None),
    "WIA_PROPERTY_INFO": SimStruct(OrderedDict((("lAccessFlags", SimTypeInt(signed=False, label="UInt32")), ("vt", SimTypeInt(signed=False, label="VARENUM")), ("ValidVal", SimUnion({"Range": SimTypeRef("_Range_e__Struct", SimStruct), "RangeFloat": SimTypeRef("_RangeFloat_e__Struct", SimStruct), "List": SimTypeRef("_List_e__Struct", SimStruct), "ListFloat": SimTypeRef("_ListFloat_e__Struct", SimStruct), "ListGuid": SimTypeRef("_ListGuid_e__Struct", SimStruct), "ListBStr": SimTypeRef("_ListBStr_e__Struct", SimStruct), "Flag": SimTypeRef("_Flag_e__Struct", SimStruct), "None": SimTypeRef("_None_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WIA_PROPERTY_INFO", pack=False, align=None),
    "WIA_PROPID_TO_NAME": SimStruct(OrderedDict((("propid", SimTypeInt(signed=False, label="UInt32")), ("pszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIA_PROPID_TO_NAME", pack=False, align=None),
    "WIA_RAW_HEADER": SimStruct(OrderedDict((("Tag", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeInt(signed=False, label="UInt32")), ("HeaderSize", SimTypeInt(signed=False, label="UInt32")), ("XRes", SimTypeInt(signed=False, label="UInt32")), ("YRes", SimTypeInt(signed=False, label="UInt32")), ("XExtent", SimTypeInt(signed=False, label="UInt32")), ("YExtent", SimTypeInt(signed=False, label="UInt32")), ("BytesPerLine", SimTypeInt(signed=False, label="UInt32")), ("BitsPerPixel", SimTypeInt(signed=False, label="UInt32")), ("ChannelsPerPixel", SimTypeInt(signed=False, label="UInt32")), ("DataType", SimTypeInt(signed=False, label="UInt32")), ("BitsPerChannel", SimTypeArray(SimTypeChar(label="Byte"), 8)), ("Compression", SimTypeInt(signed=False, label="UInt32")), ("PhotometricInterp", SimTypeInt(signed=False, label="UInt32")), ("LineOrder", SimTypeInt(signed=False, label="UInt32")), ("RawDataOffset", SimTypeInt(signed=False, label="UInt32")), ("RawDataSize", SimTypeInt(signed=False, label="UInt32")), ("PaletteOffset", SimTypeInt(signed=False, label="UInt32")), ("PaletteSize", SimTypeInt(signed=False, label="UInt32")),)), name="WIA_RAW_HEADER", pack=False, align=None),
    "WIC8BIMIptcDigestProperties": SimTypeInt(signed=False, label="WIC8BIMIptcDigestProperties"),
    "WIC8BIMIptcProperties": SimTypeInt(signed=False, label="WIC8BIMIptcProperties"),
    "WIC8BIMResolutionInfoProperties": SimTypeInt(signed=False, label="WIC8BIMResolutionInfoProperties"),
    "WICBitmapAlphaChannelOption": SimTypeInt(signed=False, label="WICBitmapAlphaChannelOption"),
    "WICBitmapCreateCacheOption": SimTypeInt(signed=False, label="WICBitmapCreateCacheOption"),
    "WICBitmapDecoderCapabilities": SimTypeInt(signed=False, label="WICBitmapDecoderCapabilities"),
    "WICBitmapDitherType": SimTypeInt(signed=False, label="WICBitmapDitherType"),
    "WICBitmapEncoderCacheOption": SimTypeInt(signed=False, label="WICBitmapEncoderCacheOption"),
    "WICBitmapInterpolationMode": SimTypeInt(signed=False, label="WICBitmapInterpolationMode"),
    "WICBitmapLockFlags": SimTypeInt(signed=False, label="WICBitmapLockFlags"),
    "WICBitmapPaletteType": SimTypeInt(signed=False, label="WICBitmapPaletteType"),
    "WICBitmapPattern": SimStruct(OrderedDict((("Position", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Pattern", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Mask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("EndOfStream", SimTypeInt(signed=True, label="Int32")),)), name="WICBitmapPattern", pack=False, align=None),
    "WICBitmapPlane": SimStruct(OrderedDict((("Format", SimTypeBottom(label="Guid")), ("pbBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbStride", SimTypeInt(signed=False, label="UInt32")), ("cbBufferSize", SimTypeInt(signed=False, label="UInt32")),)), name="WICBitmapPlane", pack=False, align=None),
    "WICBitmapPlaneDescription": SimStruct(OrderedDict((("Format", SimTypeBottom(label="Guid")), ("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")),)), name="WICBitmapPlaneDescription", pack=False, align=None),
    "WICBitmapTransformOptions": SimTypeInt(signed=False, label="WICBitmapTransformOptions"),
    "WICColorContextType": SimTypeInt(signed=False, label="WICColorContextType"),
    "WICComponentEnumerateOptions": SimTypeInt(signed=False, label="WICComponentEnumerateOptions"),
    "WICComponentSigning": SimTypeInt(signed=False, label="WICComponentSigning"),
    "WICComponentType": SimTypeInt(signed=False, label="WICComponentType"),
    "WICDdsAlphaMode": SimTypeInt(signed=False, label="WICDdsAlphaMode"),
    "WICDdsDimension": SimTypeInt(signed=False, label="WICDdsDimension"),
    "WICDdsFormatInfo": SimStruct(OrderedDict((("DxgiFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("BytesPerBlock", SimTypeInt(signed=False, label="UInt32")), ("BlockWidth", SimTypeInt(signed=False, label="UInt32")), ("BlockHeight", SimTypeInt(signed=False, label="UInt32")),)), name="WICDdsFormatInfo", pack=False, align=None),
    "WICDdsParameters": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("Depth", SimTypeInt(signed=False, label="UInt32")), ("MipLevels", SimTypeInt(signed=False, label="UInt32")), ("ArraySize", SimTypeInt(signed=False, label="UInt32")), ("DxgiFormat", SimTypeInt(signed=False, label="DXGI_FORMAT")), ("Dimension", SimTypeInt(signed=False, label="WICDdsDimension")), ("AlphaMode", SimTypeInt(signed=False, label="WICDdsAlphaMode")),)), name="WICDdsParameters", pack=False, align=None),
    "WICDecodeOptions": SimTypeInt(signed=False, label="WICDecodeOptions"),
    "WICGifApplicationExtensionProperties": SimTypeInt(signed=False, label="WICGifApplicationExtensionProperties"),
    "WICGifCommentExtensionProperties": SimTypeInt(signed=False, label="WICGifCommentExtensionProperties"),
    "WICGifGraphicControlExtensionProperties": SimTypeInt(signed=False, label="WICGifGraphicControlExtensionProperties"),
    "WICGifImageDescriptorProperties": SimTypeInt(signed=False, label="WICGifImageDescriptorProperties"),
    "WICGifLogicalScreenDescriptorProperties": SimTypeInt(signed=False, label="WICGifLogicalScreenDescriptorProperties"),
    "WICHeifHdrProperties": SimTypeInt(signed=False, label="WICHeifHdrProperties"),
    "WICHeifProperties": SimTypeInt(signed=False, label="WICHeifProperties"),
    "WICImageParameters": SimStruct(OrderedDict((("PixelFormat", SimTypeRef("D2D1_PIXEL_FORMAT", SimStruct)), ("DpiX", SimTypeFloat(size=32)), ("DpiY", SimTypeFloat(size=32)), ("Top", SimTypeFloat(size=32)), ("Left", SimTypeFloat(size=32)), ("PixelWidth", SimTypeInt(signed=False, label="UInt32")), ("PixelHeight", SimTypeInt(signed=False, label="UInt32")),)), name="WICImageParameters", pack=False, align=None),
    "WICJpegChrominanceProperties": SimTypeInt(signed=False, label="WICJpegChrominanceProperties"),
    "WICJpegCommentProperties": SimTypeInt(signed=False, label="WICJpegCommentProperties"),
    "WICJpegFrameHeader": SimStruct(OrderedDict((("Width", SimTypeInt(signed=False, label="UInt32")), ("Height", SimTypeInt(signed=False, label="UInt32")), ("TransferMatrix", SimTypeInt(signed=False, label="WICJpegTransferMatrix")), ("ScanType", SimTypeInt(signed=False, label="WICJpegScanType")), ("cComponents", SimTypeInt(signed=False, label="UInt32")), ("ComponentIdentifiers", SimTypeInt(signed=False, label="UInt32")), ("SampleFactors", SimTypeInt(signed=False, label="UInt32")), ("QuantizationTableIndices", SimTypeInt(signed=False, label="UInt32")),)), name="WICJpegFrameHeader", pack=False, align=None),
    "WICJpegIndexingOptions": SimTypeInt(signed=False, label="WICJpegIndexingOptions"),
    "WICJpegLuminanceProperties": SimTypeInt(signed=False, label="WICJpegLuminanceProperties"),
    "WICJpegScanHeader": SimStruct(OrderedDict((("cComponents", SimTypeInt(signed=False, label="UInt32")), ("RestartInterval", SimTypeInt(signed=False, label="UInt32")), ("ComponentSelectors", SimTypeInt(signed=False, label="UInt32")), ("HuffmanTableIndices", SimTypeInt(signed=False, label="UInt32")), ("StartSpectralSelection", SimTypeChar(label="Byte")), ("EndSpectralSelection", SimTypeChar(label="Byte")), ("SuccessiveApproximationHigh", SimTypeChar(label="Byte")), ("SuccessiveApproximationLow", SimTypeChar(label="Byte")),)), name="WICJpegScanHeader", pack=False, align=None),
    "WICJpegScanType": SimTypeInt(signed=False, label="WICJpegScanType"),
    "WICJpegTransferMatrix": SimTypeInt(signed=False, label="WICJpegTransferMatrix"),
    "WICJpegYCrCbSubsamplingOption": SimTypeInt(signed=False, label="WICJpegYCrCbSubsamplingOption"),
    "WICMetadataCreationOptions": SimTypeInt(signed=False, label="WICMetadataCreationOptions"),
    "WICMetadataHeader": SimStruct(OrderedDict((("Position", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Header", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DataOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="WICMetadataHeader", pack=False, align=None),
    "WICMetadataPattern": SimStruct(OrderedDict((("Position", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Pattern", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Mask", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("DataOffset", SimTypeLongLong(signed=False, label="UInt64")),)), name="WICMetadataPattern", pack=False, align=None),
    "WICNamedWhitePoint": SimTypeInt(signed=False, label="WICNamedWhitePoint"),
    "WICPersistOptions": SimTypeInt(signed=False, label="WICPersistOptions"),
    "WICPixelFormatNumericRepresentation": SimTypeInt(signed=False, label="WICPixelFormatNumericRepresentation"),
    "WICPlanarOptions": SimTypeInt(signed=False, label="WICPlanarOptions"),
    "WICPngBkgdProperties": SimTypeInt(signed=False, label="WICPngBkgdProperties"),
    "WICPngChrmProperties": SimTypeInt(signed=False, label="WICPngChrmProperties"),
    "WICPngFilterOption": SimTypeInt(signed=False, label="WICPngFilterOption"),
    "WICPngGamaProperties": SimTypeInt(signed=False, label="WICPngGamaProperties"),
    "WICPngHistProperties": SimTypeInt(signed=False, label="WICPngHistProperties"),
    "WICPngIccpProperties": SimTypeInt(signed=False, label="WICPngIccpProperties"),
    "WICPngItxtProperties": SimTypeInt(signed=False, label="WICPngItxtProperties"),
    "WICPngSrgbProperties": SimTypeInt(signed=False, label="WICPngSrgbProperties"),
    "WICPngTimeProperties": SimTypeInt(signed=False, label="WICPngTimeProperties"),
    "WICProgressNotification": SimTypeInt(signed=False, label="WICProgressNotification"),
    "WICProgressOperation": SimTypeInt(signed=False, label="WICProgressOperation"),
    "WICRawCapabilities": SimTypeInt(signed=False, label="WICRawCapabilities"),
    "WICRawCapabilitiesInfo": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("CodecMajorVersion", SimTypeInt(signed=False, label="UInt32")), ("CodecMinorVersion", SimTypeInt(signed=False, label="UInt32")), ("ExposureCompensationSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("ContrastSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("RGBWhitePointSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("NamedWhitePointSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("NamedWhitePointSupportMask", SimTypeInt(signed=False, label="UInt32")), ("KelvinWhitePointSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("GammaSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("TintSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("SaturationSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("SharpnessSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("NoiseReductionSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("DestinationColorProfileSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("ToneCurveSupport", SimTypeInt(signed=False, label="WICRawCapabilities")), ("RotationSupport", SimTypeInt(signed=False, label="WICRawRotationCapabilities")), ("RenderModeSupport", SimTypeInt(signed=False, label="WICRawCapabilities")),)), name="WICRawCapabilitiesInfo", pack=False, align=None),
    "WICRawParameterSet": SimTypeInt(signed=False, label="WICRawParameterSet"),
    "WICRawRenderMode": SimTypeInt(signed=False, label="WICRawRenderMode"),
    "WICRawRotationCapabilities": SimTypeInt(signed=False, label="WICRawRotationCapabilities"),
    "WICRawToneCurve": SimStruct(OrderedDict((("cPoints", SimTypeInt(signed=False, label="UInt32")), ("aPoints", SimTypePointer(SimTypeRef("WICRawToneCurvePoint", SimStruct), offset=0)),)), name="WICRawToneCurve", pack=False, align=None),
    "WICRawToneCurvePoint": SimStruct(OrderedDict((("Input", SimTypeFloat(size=64)), ("Output", SimTypeFloat(size=64)),)), name="WICRawToneCurvePoint", pack=False, align=None),
    "WICRect": SimStruct(OrderedDict((("X", SimTypeInt(signed=True, label="Int32")), ("Y", SimTypeInt(signed=True, label="Int32")), ("Width", SimTypeInt(signed=True, label="Int32")), ("Height", SimTypeInt(signed=True, label="Int32")),)), name="WICRect", pack=False, align=None),
    "WICSectionAccessLevel": SimTypeInt(signed=False, label="WICSectionAccessLevel"),
    "WICTiffCompressionOption": SimTypeInt(signed=False, label="WICTiffCompressionOption"),
    "WICWebpAnimProperties": SimTypeInt(signed=False, label="WICWebpAnimProperties"),
    "WICWebpAnmfProperties": SimTypeInt(signed=False, label="WICWebpAnmfProperties"),
    "WIDTHRUN": SimStruct(OrderedDict((("wStartGlyph", SimTypeShort(signed=False, label="UInt16")), ("wGlyphCount", SimTypeShort(signed=False, label="UInt16")), ("loCharWidthOffset", SimTypeInt(signed=False, label="UInt32")),)), name="WIDTHRUN", pack=False, align=None),
    "WIDTHTABLE": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwRunNum", SimTypeInt(signed=False, label="UInt32")), ("WidthRun", SimTypePointer(SimTypeRef("WIDTHRUN", SimStruct), offset=0)),)), name="WIDTHTABLE", pack=False, align=None),
    "WIM_ENTRY_INFO": SimStruct(OrderedDict((("WimEntryInfoSize", SimTypeInt(signed=False, label="UInt32")), ("WimType", SimTypeInt(signed=False, label="UInt32")), ("DataSourceId", SimTypeLongLong(signed=True, label="Int64")), ("WimGuid", SimTypeBottom(label="Guid")), ("WimPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("WimIndex", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="WIM_ENTRY_INFO", pack=False, align=None),
    "WIM_EXTERNAL_FILE_INFO": SimStruct(OrderedDict((("DataSourceId", SimTypeLongLong(signed=True, label="Int64")), ("ResourceHash", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="WIM_EXTERNAL_FILE_INFO", pack=False, align=None),
    "WIM_PROVIDER_ADD_OVERLAY_INPUT": SimStruct(OrderedDict((("WimType", SimTypeInt(signed=False, label="UInt32")), ("WimIndex", SimTypeInt(signed=False, label="UInt32")), ("WimFileNameOffset", SimTypeInt(signed=False, label="UInt32")), ("WimFileNameLength", SimTypeInt(signed=False, label="UInt32")),)), name="WIM_PROVIDER_ADD_OVERLAY_INPUT", pack=False, align=None),
    "WIM_PROVIDER_EXTERNAL_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("DataSourceId", SimTypeLongLong(signed=True, label="Int64")), ("ResourceHash", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="WIM_PROVIDER_EXTERNAL_INFO", pack=False, align=None),
    "WIM_PROVIDER_OVERLAY_ENTRY": SimStruct(OrderedDict((("NextEntryOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSourceId", SimTypeLongLong(signed=True, label="Int64")), ("WimGuid", SimTypeBottom(label="Guid")), ("WimFileNameOffset", SimTypeInt(signed=False, label="UInt32")), ("WimType", SimTypeInt(signed=False, label="UInt32")), ("WimIndex", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="WIM_PROVIDER_OVERLAY_ENTRY", pack=False, align=None),
    "WIM_PROVIDER_REMOVE_OVERLAY_INPUT": SimStruct(OrderedDict((("DataSourceId", SimTypeLongLong(signed=True, label="Int64")),)), name="WIM_PROVIDER_REMOVE_OVERLAY_INPUT", pack=False, align=None),
    "WIM_PROVIDER_SUSPEND_OVERLAY_INPUT": SimStruct(OrderedDict((("DataSourceId", SimTypeLongLong(signed=True, label="Int64")),)), name="WIM_PROVIDER_SUSPEND_OVERLAY_INPUT", pack=False, align=None),
    "WIM_PROVIDER_UPDATE_OVERLAY_INPUT": SimStruct(OrderedDict((("DataSourceId", SimTypeLongLong(signed=True, label="Int64")), ("WimFileNameOffset", SimTypeInt(signed=False, label="UInt32")), ("WimFileNameLength", SimTypeInt(signed=False, label="UInt32")),)), name="WIM_PROVIDER_UPDATE_OVERLAY_INPUT", pack=False, align=None),
    "WIN32_ERROR": SimTypeInt(signed=False, label="WIN32_ERROR"),
    "WIN32_FILE_ATTRIBUTE_DATA": SimStruct(OrderedDict((("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ftCreationTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastWriteTime", SimTypeRef("FILETIME", SimStruct)), ("nFileSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeLow", SimTypeInt(signed=False, label="UInt32")),)), name="WIN32_FILE_ATTRIBUTE_DATA", pack=False, align=None),
    "WIN32_FIND_DATAA": SimStruct(OrderedDict((("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ftCreationTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastWriteTime", SimTypeRef("FILETIME", SimStruct)), ("nFileSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeLow", SimTypeInt(signed=False, label="UInt32")), ("dwReserved0", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("cFileName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("cAlternateFileName", SimTypeArray(SimTypeChar(label="SByte"), 14)),)), name="WIN32_FIND_DATAA", pack=False, align=None),
    "WIN32_FIND_DATAW": SimStruct(OrderedDict((("dwFileAttributes", SimTypeInt(signed=False, label="UInt32")), ("ftCreationTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastAccessTime", SimTypeRef("FILETIME", SimStruct)), ("ftLastWriteTime", SimTypeRef("FILETIME", SimStruct)), ("nFileSizeHigh", SimTypeInt(signed=False, label="UInt32")), ("nFileSizeLow", SimTypeInt(signed=False, label="UInt32")), ("dwReserved0", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("cFileName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("cAlternateFileName", SimTypeArray(SimTypeChar(label="Char"), 14)),)), name="WIN32_FIND_DATAW", pack=False, align=None),
    "WIN32_FIND_STREAM_DATA": SimStruct(OrderedDict((("StreamSize", SimTypeLongLong(signed=True, label="Int64")), ("cStreamName", SimTypeArray(SimTypeChar(label="Char"), 296)),)), name="WIN32_FIND_STREAM_DATA", pack=False, align=None),
    "WIN32_MEMORY_INFORMATION_CLASS": SimTypeInt(signed=False, label="WIN32_MEMORY_INFORMATION_CLASS"),
    "WIN32_MEMORY_PARTITION_INFORMATION": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("NumaNode", SimTypeInt(signed=False, label="UInt32")), ("Channel", SimTypeInt(signed=False, label="UInt32")), ("NumberOfNumaNodes", SimTypeInt(signed=False, label="UInt32")), ("ResidentAvailablePages", SimTypeLongLong(signed=False, label="UInt64")), ("CommittedPages", SimTypeLongLong(signed=False, label="UInt64")), ("CommitLimit", SimTypeLongLong(signed=False, label="UInt64")), ("PeakCommitment", SimTypeLongLong(signed=False, label="UInt64")), ("TotalNumberOfPages", SimTypeLongLong(signed=False, label="UInt64")), ("AvailablePages", SimTypeLongLong(signed=False, label="UInt64")), ("ZeroPages", SimTypeLongLong(signed=False, label="UInt64")), ("FreePages", SimTypeLongLong(signed=False, label="UInt64")), ("StandbyPages", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 16)), ("MaximumCommitLimit", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeLongLong(signed=False, label="UInt64")), ("PartitionId", SimTypeInt(signed=False, label="UInt32")),)), name="WIN32_MEMORY_PARTITION_INFORMATION", pack=False, align=None),
    "WIN32_MEMORY_PARTITION_INFORMATION_CLASS": SimTypeInt(signed=False, label="WIN32_MEMORY_PARTITION_INFORMATION_CLASS"),
    "WIN32_MEMORY_RANGE_ENTRY": SimStruct(OrderedDict((("VirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WIN32_MEMORY_RANGE_ENTRY", pack=False, align=None),
    "WIN32_MEMORY_REGION_INFORMATION": SimStruct(OrderedDict((("AllocationBase", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationProtect", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"Flags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("RegionSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("CommitSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WIN32_MEMORY_REGION_INFORMATION", pack=False, align=None),
    "WIN32_STREAM_ID": SimStruct(OrderedDict((("dwStreamId", SimTypeInt(signed=False, label="WIN_STREAM_ID")), ("dwStreamAttributes", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeLongLong(signed=True, label="Int64")), ("dwStreamNameSize", SimTypeInt(signed=False, label="UInt32")), ("cStreamName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIN32_STREAM_ID", pack=False, align=None),
    "WINBIO_ACCOUNT_POLICY": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("AntiSpoofBehavior", SimTypeInt(signed=False, label="WINBIO_ANTI_SPOOF_POLICY_ACTION")),)), name="WINBIO_ACCOUNT_POLICY", pack=False, align=None),
    "WINBIO_ADAPTER_INTERFACE_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeShort(signed=False, label="UInt16")), ("MinorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="WINBIO_ADAPTER_INTERFACE_VERSION", pack=False, align=None),
    "WINBIO_ANTI_SPOOF_POLICY": SimStruct(OrderedDict((("Action", SimTypeInt(signed=False, label="WINBIO_ANTI_SPOOF_POLICY_ACTION")), ("Source", SimTypeInt(signed=False, label="WINBIO_POLICY_SOURCE")),)), name="WINBIO_ANTI_SPOOF_POLICY", pack=False, align=None),
    "WINBIO_ANTI_SPOOF_POLICY_ACTION": SimTypeInt(signed=False, label="WINBIO_ANTI_SPOOF_POLICY_ACTION"),
    "WINBIO_ASYNC_NOTIFICATION_METHOD": SimTypeInt(signed=False, label="WINBIO_ASYNC_NOTIFICATION_METHOD"),
    "WINBIO_ASYNC_RESULT": SimStruct(OrderedDict((("SessionHandle", SimTypeInt(signed=False, label="UInt32")), ("Operation", SimTypeInt(signed=False, label="UInt32")), ("SequenceNumber", SimTypeLongLong(signed=False, label="UInt64")), ("TimeStamp", SimTypeLongLong(signed=True, label="Int64")), ("ApiStatus", SimTypeInt(signed=True, label="Int32")), ("UnitId", SimTypeInt(signed=False, label="UInt32")), ("UserData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Parameters", SimUnion({"Verify": SimTypeRef("_Verify_e__Struct", SimStruct), "Identify": SimTypeRef("_Identify_e__Struct", SimStruct), "EnrollBegin": SimTypeRef("_EnrollBegin_e__Struct", SimStruct), "EnrollCapture": SimTypeRef("_EnrollCapture_e__Struct", SimStruct), "EnrollCommit": SimTypeRef("_EnrollCommit_e__Struct", SimStruct), "EnumEnrollments": SimTypeRef("_EnumEnrollments_e__Struct", SimStruct), "CaptureSample": SimTypeRef("_CaptureSample_e__Struct", SimStruct), "DeleteTemplate": SimTypeRef("_DeleteTemplate_e__Struct", SimStruct), "GetProperty": SimTypeRef("_GetProperty_e__Struct", SimStruct), "SetProperty": SimTypeRef("_SetProperty_e__Struct", SimStruct), "GetEvent": SimTypeRef("_GetEvent_e__Struct", SimStruct), "ControlUnit": SimTypeRef("_ControlUnit_e__Struct", SimStruct), "EnumServiceProviders": SimTypeRef("_EnumServiceProviders_e__Struct", SimStruct), "EnumBiometricUnits": SimTypeRef("_EnumBiometricUnits_e__Struct", SimStruct), "EnumDatabases": SimTypeRef("_EnumDatabases_e__Struct", SimStruct), "VerifyAndReleaseTicket": SimTypeRef("_VerifyAndReleaseTicket_e__Struct", SimStruct), "IdentifyAndReleaseTicket": SimTypeRef("_IdentifyAndReleaseTicket_e__Struct", SimStruct), "EnrollSelect": SimTypeRef("_EnrollSelect_e__Struct", SimStruct), "MonitorPresence": SimTypeRef("_MonitorPresence_e__Struct", SimStruct), "GetProtectionPolicy": SimTypeRef("_GetProtectionPolicy_e__Struct", SimStruct), "NotifyUnitStatusChange": SimTypeRef("_NotifyUnitStatusChange_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINBIO_ASYNC_RESULT", pack=False, align=None),
    "WINBIO_BDB_ANSI_381_HEADER": SimStruct(OrderedDict((("RecordLength", SimTypeLongLong(signed=False, label="UInt64")), ("FormatIdentifier", SimTypeInt(signed=False, label="UInt32")), ("VersionNumber", SimTypeInt(signed=False, label="UInt32")), ("ProductId", SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct)), ("CaptureDeviceId", SimTypeShort(signed=False, label="UInt16")), ("ImageAcquisitionLevel", SimTypeShort(signed=False, label="UInt16")), ("HorizontalScanResolution", SimTypeShort(signed=False, label="UInt16")), ("VerticalScanResolution", SimTypeShort(signed=False, label="UInt16")), ("HorizontalImageResolution", SimTypeShort(signed=False, label="UInt16")), ("VerticalImageResolution", SimTypeShort(signed=False, label="UInt16")), ("ElementCount", SimTypeChar(label="Byte")), ("ScaleUnits", SimTypeChar(label="Byte")), ("PixelDepth", SimTypeChar(label="Byte")), ("ImageCompressionAlg", SimTypeChar(label="Byte")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="WINBIO_BDB_ANSI_381_HEADER", pack=False, align=None),
    "WINBIO_BDB_ANSI_381_RECORD": SimStruct(OrderedDict((("BlockLength", SimTypeInt(signed=False, label="UInt32")), ("HorizontalLineLength", SimTypeShort(signed=False, label="UInt16")), ("VerticalLineLength", SimTypeShort(signed=False, label="UInt16")), ("Position", SimTypeChar(label="Byte")), ("CountOfViews", SimTypeChar(label="Byte")), ("ViewNumber", SimTypeChar(label="Byte")), ("ImageQuality", SimTypeChar(label="Byte")), ("ImpressionType", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="WINBIO_BDB_ANSI_381_RECORD", pack=False, align=None),
    "WINBIO_BIR": SimStruct(OrderedDict((("HeaderBlock", SimTypeRef("WINBIO_BIR_DATA", SimStruct)), ("StandardDataBlock", SimTypeRef("WINBIO_BIR_DATA", SimStruct)), ("VendorDataBlock", SimTypeRef("WINBIO_BIR_DATA", SimStruct)), ("SignatureBlock", SimTypeRef("WINBIO_BIR_DATA", SimStruct)),)), name="WINBIO_BIR", pack=False, align=None),
    "WINBIO_BIR_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Offset", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_BIR_DATA", pack=False, align=None),
    "WINBIO_BIR_HEADER": SimStruct(OrderedDict((("ValidFields", SimTypeShort(signed=False, label="UInt16")), ("HeaderVersion", SimTypeChar(label="Byte")), ("PatronHeaderVersion", SimTypeChar(label="Byte")), ("DataFlags", SimTypeChar(label="Byte")), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Subtype", SimTypeChar(label="Byte")), ("Purpose", SimTypeChar(label="Byte")), ("DataQuality", SimTypeChar(label="SByte")), ("CreationDate", SimTypeLongLong(signed=True, label="Int64")), ("ValidityPeriod", SimTypeRef("_ValidityPeriod_e__Struct", SimStruct)), ("BiometricDataFormat", SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct)), ("ProductId", SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct)),)), name="WINBIO_BIR_HEADER", pack=False, align=None),
    "WINBIO_BLANK_PAYLOAD": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")),)), name="WINBIO_BLANK_PAYLOAD", pack=False, align=None),
    "WINBIO_BSP_SCHEMA": SimStruct(OrderedDict((("BiometricFactor", SimTypeInt(signed=False, label="UInt32")), ("BspId", SimTypeBottom(label="Guid")), ("Description", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Vendor", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Version", SimTypeRef("WINBIO_VERSION", SimStruct)),)), name="WINBIO_BSP_SCHEMA", pack=False, align=None),
    "WINBIO_CALIBRATION_INFO": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("CalibrationData", SimTypeRef("WINBIO_DATA", SimStruct)),)), name="WINBIO_CALIBRATION_INFO", pack=False, align=None),
    "WINBIO_CAPTURE_DATA": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("SensorStatus", SimTypeInt(signed=False, label="UInt32")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")), ("CaptureData", SimTypeRef("WINBIO_DATA", SimStruct)),)), name="WINBIO_CAPTURE_DATA", pack=False, align=None),
    "WINBIO_CAPTURE_PARAMETERS": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("Purpose", SimTypeChar(label="Byte")), ("Format", SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct)), ("VendorFormat", SimTypeBottom(label="Guid")), ("Flags", SimTypeChar(label="Byte")),)), name="WINBIO_CAPTURE_PARAMETERS", pack=False, align=None),
    "WINBIO_COMPONENT": SimTypeInt(signed=False, label="WINBIO_COMPONENT"),
    "WINBIO_CREDENTIAL_FORMAT": SimTypeInt(signed=False, label="WINBIO_CREDENTIAL_FORMAT"),
    "WINBIO_CREDENTIAL_STATE": SimTypeInt(signed=False, label="WINBIO_CREDENTIAL_STATE"),
    "WINBIO_CREDENTIAL_TYPE": SimTypeInt(signed=False, label="WINBIO_CREDENTIAL_TYPE"),
    "WINBIO_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WINBIO_DATA", pack=False, align=None),
    "WINBIO_DIAGNOSTICS": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("SensorStatus", SimTypeInt(signed=False, label="UInt32")), ("VendorDiagnostics", SimTypeRef("WINBIO_DATA", SimStruct)),)), name="WINBIO_DIAGNOSTICS", pack=False, align=None),
    "WINBIO_ENCRYPTED_CAPTURE_PARAMS": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("Purpose", SimTypeChar(label="Byte")), ("Format", SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct)), ("VendorFormat", SimTypeBottom(label="Guid")), ("Flags", SimTypeChar(label="Byte")), ("NonceSize", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_ENCRYPTED_CAPTURE_PARAMS", pack=False, align=None),
    "WINBIO_ENGINE_INTERFACE": SimStruct(OrderedDict((("Version", SimTypeRef("WINBIO_ADAPTER_INTERFACE_VERSION", SimStruct)), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterId", SimTypeBottom(label="Guid")), ("Attach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Detach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("ClearContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("QueryPreferredFormat", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "StandardFormat", "VendorFormat"]), offset=0)), ("QueryIndexVectorSize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "IndexElementCount"]), offset=0)), ("QueryHashAlgorithms", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AlgorithmCount", "AlgorithmBufferSize", "AlgorithmBuffer"]), offset=0)), ("SetHashAlgorithm", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AlgorithmBufferSize", "AlgorithmBuffer"]), offset=0)), ("QuerySampleHint", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SampleHint"]), offset=0)), ("AcceptSampleData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SampleBuffer", "SampleSize", "Purpose", "RejectDetail"]), offset=0)), ("ExportEngineData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Flags", "SampleBuffer", "SampleSize"]), offset=0)), ("VerifyFeatureSet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "Match", "PayloadBlob", "PayloadBlobSize", "HashValue", "HashSize", "RejectDetail"]), offset=0)), ("IdentifyFeatureSet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "PayloadBlob", "PayloadBlobSize", "HashValue", "HashSize", "RejectDetail"]), offset=0)), ("CreateEnrollment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("UpdateEnrollment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RejectDetail"]), offset=0)), ("GetEnrollmentStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RejectDetail"]), offset=0)), ("GetEnrollmentHash", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "HashValue", "HashSize"]), offset=0)), ("CheckForDuplicate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "Duplicate"]), offset=0)), ("CommitEnrollment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "PayloadBlob", "PayloadBlobSize"]), offset=0)), ("DiscardEnrollment", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("ControlUnit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0)), ("ControlUnitPrivileged", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0)), ("NotifyPowerChange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0)), ("Reserved_1", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity"]), offset=0)), ("PipelineInit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("PipelineCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Activate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Deactivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("QueryExtendedInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_ENGINE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "EngineInfo", "EngineInfoSize"]), offset=0)), ("IdentifyAll", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_PRESENCE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PresenceCount", "PresenceArray"]), offset=0)), ("SetEnrollmentSelector", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SelectorValue"]), offset=0)), ("SetEnrollmentParameters", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_ENROLLMENT_PARAMETERS", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Parameters"]), offset=0)), ("QueryExtendedEnrollmentStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_ENROLLMENT_STATUS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "EnrollmentStatus", "EnrollmentStatusSize"]), offset=0)), ("RefreshCache", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("SelectCalibrationFormat", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "FormatArray", "FormatCount", "SelectedFormat", "MaxBufferSize"]), offset=0)), ("QueryCalibrationData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DiscardAndRepeatCapture", "CalibrationBuffer", "CalibrationBufferSize", "MaxBufferSize"]), offset=0)), ("SetAccountPolicy", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_ACCOUNT_POLICY", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PolicyItemArray", "PolicyItemCount"]), offset=0)), ("CreateKey", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Key", "KeySize", "KeyIdentifier", "KeyIdentifierSize", "ResultSize"]), offset=0)), ("IdentifyFeatureSetSecure", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Nonce", "NonceSize", "KeyIdentifier", "KeyIdentifierSize", "Identity", "SubFactor", "RejectDetail", "Authorization", "AuthorizationSize"]), offset=0)), ("AcceptPrivateSensorTypeInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "TypeInfoBufferAddress", "TypeInfoBufferSize"]), offset=0)), ("CreateEnrollmentAuthenticated", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Nonce", "NonceSize"]), offset=0)), ("IdentifyFeatureSetAuthenticated", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Nonce", "NonceSize", "Identity", "SubFactor", "RejectDetail", "Authentication", "AuthenticationSize"]), offset=0)),)), name="WINBIO_ENGINE_INTERFACE", pack=False, align=None),
    "WINBIO_EVENT": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Parameters", SimUnion({"Unclaimed": SimTypeRef("_Unclaimed_e__Struct", SimStruct), "UnclaimedIdentify": SimTypeRef("_UnclaimedIdentify_e__Struct", SimStruct), "Error": SimTypeRef("_Error_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINBIO_EVENT", pack=False, align=None),
    "WINBIO_EXTENDED_ENGINE_INFO": SimStruct(OrderedDict((("GenericEngineCapabilities", SimTypeInt(signed=False, label="UInt32")), ("Factor", SimTypeInt(signed=False, label="UInt32")), ("Specific", SimUnion({"Null": SimTypeInt(signed=False, label="UInt32"), "FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Fingerprint": SimTypeRef("_Fingerprint_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct), "Voice": SimTypeRef("_Voice_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINBIO_EXTENDED_ENGINE_INFO", pack=False, align=None),
    "WINBIO_EXTENDED_ENROLLMENT_PARAMETERS": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SubFactor", SimTypeChar(label="Byte")),)), name="WINBIO_EXTENDED_ENROLLMENT_PARAMETERS", pack=False, align=None),
    "WINBIO_EXTENDED_ENROLLMENT_STATUS": SimStruct(OrderedDict((("TemplateStatus", SimTypeInt(signed=True, label="Int32")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")), ("PercentComplete", SimTypeInt(signed=False, label="UInt32")), ("Factor", SimTypeInt(signed=False, label="UInt32")), ("SubFactor", SimTypeChar(label="Byte")), ("Specific", SimUnion({"Null": SimTypeInt(signed=False, label="UInt32"), "FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Fingerprint": SimTypeRef("_Fingerprint_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct), "Voice": SimTypeRef("_Voice_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINBIO_EXTENDED_ENROLLMENT_STATUS", pack=False, align=None),
    "WINBIO_EXTENDED_SENSOR_INFO": SimStruct(OrderedDict((("GenericSensorCapabilities", SimTypeInt(signed=False, label="UInt32")), ("Factor", SimTypeInt(signed=False, label="UInt32")), ("Specific", SimUnion({"Null": SimTypeInt(signed=False, label="UInt32"), "FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Fingerprint": SimTypeRef("_Fingerprint_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct), "Voice": SimTypeRef("_Voice_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINBIO_EXTENDED_SENSOR_INFO", pack=False, align=None),
    "WINBIO_EXTENDED_STORAGE_INFO": SimStruct(OrderedDict((("GenericStorageCapabilities", SimTypeInt(signed=False, label="UInt32")), ("Factor", SimTypeInt(signed=False, label="UInt32")), ("Specific", SimUnion({"Null": SimTypeInt(signed=False, label="UInt32"), "FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Fingerprint": SimTypeRef("_Fingerprint_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct), "Voice": SimTypeRef("_Voice_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINBIO_EXTENDED_STORAGE_INFO", pack=False, align=None),
    "WINBIO_EXTENDED_UNIT_STATUS": SimStruct(OrderedDict((("Availability", SimTypeInt(signed=False, label="UInt32")), ("ReasonCode", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_EXTENDED_UNIT_STATUS", pack=False, align=None),
    "WINBIO_FP_BU_STATE": SimStruct(OrderedDict((("SensorAttached", SimTypeInt(signed=True, label="Int32")), ("CreationResult", SimTypeInt(signed=True, label="Int32")),)), name="WINBIO_FP_BU_STATE", pack=False, align=None),
    "WINBIO_FRAMEWORK_INTERFACE": SimStruct(OrderedDict((("Version", SimTypeRef("WINBIO_ADAPTER_INTERFACE_VERSION", SimStruct)), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterId", SimTypeBottom(label="Guid")), ("SetUnitStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_UNIT_STATUS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ExtendedStatus", "ExtendedStatusSize"]), offset=0)), ("VsmStorageAttach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageDetach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageClearContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageCreateDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "Factor", "Format", "FilePath", "ConnectString", "IndexElementCount", "InitialSize"]), offset=0)), ("VsmStorageOpenDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "FilePath", "ConnectString"]), offset=0)), ("VsmStorageCloseDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageDeleteRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor"]), offset=0)), ("VsmStorageNotifyPowerChange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0)), ("VsmStoragePipelineInit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStoragePipelineCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageActivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageDeactivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageQueryExtendedInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_STORAGE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "StorageInfo", "StorageInfoSize"]), offset=0)), ("VsmStorageCacheClear", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageCacheImportBegin", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RequiredCapacity", "MaxBufferSize"]), offset=0)), ("VsmStorageCacheImportNext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "BufferAddress", "BufferSize"]), offset=0)), ("VsmStorageCacheImportEnd", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmStorageCacheExportBegin", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RequiredCapacity", "MaxBufferSize"]), offset=0)), ("VsmStorageCacheExportNext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "BufferAddress", "BufferSize", "ReturnedDataSize"]), offset=0)), ("VsmStorageCacheExportEnd", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorAttach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorDetach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorClearContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorPushDataToEngine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Flags", "RejectDetail"]), offset=0)), ("VsmSensorNotifyPowerChange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0)), ("VsmSensorPipelineInit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorPipelineCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorActivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorDeactivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("VsmSensorAsyncImportRawBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RawBufferAddress", "RawBufferSize", "ResultBufferAddress", "ResultBufferSize"]), offset=0)), ("VsmSensorAsyncImportSecureBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier", "MetadataBufferAddress", "MetadataBufferSize", "ResultBufferAddress", "ResultBufferSize"]), offset=0)), ("Reserved1", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Reserved1", "Reserved2"]), offset=0)), ("Reserved2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Reserved1", "Reserved2"]), offset=0)), ("Reserved3", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Reserved4", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "Reserved1", "Reserved2"]), offset=0)), ("Reserved5", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity"]), offset=0)), ("AllocateMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AllocationSize", "Address"]), offset=0)), ("FreeMemory", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Address"]), offset=0)), ("GetProperty", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PropertyType", "PropertyId", "Identity", "SubFactor", "PropertyBuffer", "PropertyBufferSize"]), offset=0)), ("LockAndValidateSecureBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier", "SecureBufferAddress", "SecureBufferSize"]), offset=0)), ("ReleaseSecureBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier"]), offset=0)), ("QueryAuthorizedEnrollments", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SecureIdentityCount", "SecureIdentities"]), offset=0)), ("DecryptSample", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Authentication", "AuthenticationSize", "Iv", "IvSize", "EncryptedData", "EncryptedDataSize"]), offset=0)),)), name="WINBIO_FRAMEWORK_INTERFACE", pack=False, align=None),
    "WINBIO_GESTURE_METADATA": SimStruct(OrderedDict((("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BiometricType", SimTypeInt(signed=False, label="UInt32")), ("MatchType", SimTypeInt(signed=False, label="UInt32")), ("ProtectionType", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_GESTURE_METADATA", pack=False, align=None),
    "WINBIO_GET_INDICATOR": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("IndicatorStatus", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_GET_INDICATOR", pack=False, align=None),
    "WINBIO_IDENTITY": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Value", SimUnion({"Null": SimTypeInt(signed=False, label="UInt32"), "Wildcard": SimTypeInt(signed=False, label="UInt32"), "TemplateGuid": SimTypeBottom(label="Guid"), "AccountSid": SimTypeRef("_AccountSid_e__Struct", SimStruct), "SecureId": SimTypeArray(SimTypeChar(label="Byte"), 32)}, name="<anon>", label="None")),)), name="WINBIO_IDENTITY", pack=False, align=None),
    "WINBIO_NOTIFY_WAKE": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("Reason", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_NOTIFY_WAKE", pack=False, align=None),
    "WINBIO_PIPELINE": SimStruct(OrderedDict((("SensorHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("EngineHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("StorageHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SensorInterface", SimTypePointer(SimTypeRef("WINBIO_SENSOR_INTERFACE", SimStruct), offset=0)), ("EngineInterface", SimTypePointer(SimTypeRef("WINBIO_ENGINE_INTERFACE", SimStruct), offset=0)), ("StorageInterface", SimTypePointer(SimTypeRef("WINBIO_STORAGE_INTERFACE", SimStruct), offset=0)), ("SensorContext", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("EngineContext", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("StorageContext", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("FrameworkInterface", SimTypePointer(SimTypeRef("WINBIO_FRAMEWORK_INTERFACE", SimStruct), offset=0)),)), name="WINBIO_PIPELINE", pack=False, align=None),
    "WINBIO_POLICY_SOURCE": SimTypeInt(signed=False, label="WINBIO_POLICY_SOURCE"),
    "WINBIO_POOL": SimTypeInt(signed=False, label="WINBIO_POOL"),
    "WINBIO_PRESENCE": SimStruct(OrderedDict((("Factor", SimTypeInt(signed=False, label="UInt32")), ("SubFactor", SimTypeChar(label="Byte")), ("Status", SimTypeInt(signed=True, label="Int32")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")), ("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("TrackingId", SimTypeLongLong(signed=False, label="UInt64")), ("Ticket", SimTypeLongLong(signed=False, label="UInt64")), ("Properties", SimUnion({"FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct)}, name="<anon>", label="None")), ("Authorization", SimTypeRef("_Authorization_e__Struct", SimStruct)),)), name="WINBIO_PRESENCE", pack=False, align=None),
    "WINBIO_PRESENCE_PROPERTIES": SimUnion({"FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "WINBIO_PRIVATE_SENSOR_TYPE_INFO": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("PrivateSensorTypeInfo", SimTypeRef("WINBIO_DATA", SimStruct)),)), name="WINBIO_PRIVATE_SENSOR_TYPE_INFO", pack=False, align=None),
    "WINBIO_PROTECTION_POLICY": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("DatabaseId", SimTypeBottom(label="Guid")), ("UserState", SimTypeLongLong(signed=False, label="UInt64")), ("PolicySize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Policy", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="WINBIO_PROTECTION_POLICY", pack=False, align=None),
    "WINBIO_REGISTERED_FORMAT": SimStruct(OrderedDict((("Owner", SimTypeShort(signed=False, label="UInt16")), ("Type", SimTypeShort(signed=False, label="UInt16")),)), name="WINBIO_REGISTERED_FORMAT", pack=False, align=None),
    "WINBIO_SECURE_BUFFER_HEADER_V1": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("ValidationTag", SimTypeLongLong(signed=False, label="UInt64")),)), name="WINBIO_SECURE_BUFFER_HEADER_V1", pack=False, align=None),
    "WINBIO_SECURE_CONNECTION_DATA": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("ModelCertificateSize", SimTypeInt(signed=False, label="UInt32")), ("IntermediateCA1Size", SimTypeInt(signed=False, label="UInt32")), ("IntermediateCA2Size", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_SECURE_CONNECTION_DATA", pack=False, align=None),
    "WINBIO_SECURE_CONNECTION_PARAMS": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")),)), name="WINBIO_SECURE_CONNECTION_PARAMS", pack=False, align=None),
    "WINBIO_SENSOR_ATTRIBUTES": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("WinBioVersion", SimTypeRef("WINBIO_VERSION", SimStruct)), ("SensorType", SimTypeInt(signed=False, label="UInt32")), ("SensorSubType", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerName", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("ModelName", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("SerialNumber", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("FirmwareVersion", SimTypeRef("WINBIO_VERSION", SimStruct)), ("SupportedFormatEntries", SimTypeInt(signed=False, label="UInt32")), ("SupportedFormat", SimTypePointer(SimTypeRef("WINBIO_REGISTERED_FORMAT", SimStruct), offset=0)),)), name="WINBIO_SENSOR_ATTRIBUTES", pack=False, align=None),
    "WINBIO_SENSOR_INTERFACE": SimStruct(OrderedDict((("Version", SimTypeRef("WINBIO_ADAPTER_INTERFACE_VERSION", SimStruct)), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterId", SimTypeBottom(label="Guid")), ("Attach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Detach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("ClearContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("QueryStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Status"]), offset=0)), ("Reset", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("SetMode", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Mode"]), offset=0)), ("SetIndicatorStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "IndicatorStatus"]), offset=0)), ("GetIndicatorStatus", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "IndicatorStatus"]), offset=0)), ("StartCapture", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Overlapped"]), offset=0)), ("FinishCapture", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RejectDetail"]), offset=0)), ("ExportSensorData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SampleBuffer", "SampleSize"]), offset=0)), ("Cancel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("PushDataToEngine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Flags", "RejectDetail"]), offset=0)), ("ControlUnit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0)), ("ControlUnitPrivileged", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0)), ("NotifyPowerChange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0)), ("PipelineInit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("PipelineCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Activate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Deactivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("QueryExtendedInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_SENSOR_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SensorInfo", "SensorInfoSize"]), offset=0)), ("QueryCalibrationFormats", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "FormatArray", "FormatArraySize", "FormatCount"]), offset=0)), ("SetCalibrationFormat", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Format"]), offset=0)), ("AcceptCalibrationData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "CalibrationBuffer", "CalibrationBufferSize"]), offset=0)), ("AsyncImportRawBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RawBufferAddress", "RawBufferSize", "ResultBufferAddress", "ResultBufferSize"]), offset=0)), ("AsyncImportSecureBuffer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeBottom(label="Guid"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SecureBufferIdentifier", "MetadataBufferAddress", "MetadataBufferSize", "ResultBufferAddress", "ResultBufferSize"]), offset=0)), ("QueryPrivateSensorType", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "TypeInfoBufferAddress", "TypeInfoBufferSize", "TypeInfoDataSize"]), offset=0)), ("ConnectSecure", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_SECURE_CONNECTION_PARAMS", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WINBIO_SECURE_CONNECTION_DATA", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ConnectionParams", "ConnectionData"]), offset=0)), ("StartCaptureEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Purpose", "Nonce", "NonceSize", "Flags", "Overlapped"]), offset=0)), ("StartNotifyWake", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Overlapped"]), offset=0)), ("FinishNotifyWake", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Reason"]), offset=0)),)), name="WINBIO_SENSOR_INTERFACE", pack=False, align=None),
    "WINBIO_SETTING_SOURCE": SimTypeInt(signed=False, label="WINBIO_SETTING_SOURCE"),
    "WINBIO_SET_INDICATOR": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("IndicatorStatus", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_SET_INDICATOR", pack=False, align=None),
    "WINBIO_STORAGE_INTERFACE": SimStruct(OrderedDict((("Version", SimTypeRef("WINBIO_ADAPTER_INTERFACE_VERSION", SimStruct)), ("Type", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("AdapterId", SimTypeBottom(label="Guid")), ("Attach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Detach", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("ClearContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("CreateDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "Factor", "Format", "FilePath", "ConnectString", "IndexElementCount", "InitialSize"]), offset=0)), ("EraseDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "FilePath", "ConnectString"]), offset=0)), ("OpenDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "DatabaseId", "FilePath", "ConnectString"]), offset=0)), ("CloseDatabase", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("GetDataFormat", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeRef("WINBIO_VERSION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Format", "Version"]), offset=0)), ("GetDatabaseSize", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "AvailableRecordCount", "TotalRecordCount"]), offset=0)), ("AddRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordContents"]), offset=0)), ("DeleteRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor"]), offset=0)), ("QueryBySubject", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor"]), offset=0)), ("QueryByContent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "SubFactor", "IndexVector", "IndexElementCount"]), offset=0)), ("GetRecordCount", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordCount"]), offset=0)), ("FirstRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("NextRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("GetCurrentRecord", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordContents"]), offset=0)), ("ControlUnit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0)), ("ControlUnitPrivileged", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "ControlCode", "SendBuffer", "SendBufferSize", "ReceiveBuffer", "ReceiveBufferSize", "ReceiveDataSize", "OperationStatus"]), offset=0)), ("NotifyPowerChange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "PowerEventType"]), offset=0)), ("PipelineInit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("PipelineCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Activate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("Deactivate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline"]), offset=0)), ("QueryExtendedInfo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_EXTENDED_STORAGE_INFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "StorageInfo", "StorageInfoSize"]), offset=0)), ("NotifyDatabaseChange", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypeChar(label="Byte")], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordsAdded"]), offset=0)), ("Reserved1", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "Reserved1", "Reserved2"]), offset=0)), ("Reserved2", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity"]), offset=0)), ("UpdateRecordBegin", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "Identity", "SubFactor", "RecordContents"]), offset=0)), ("UpdateRecordCommit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WINBIO_PIPELINE", SimStruct), offset=0), SimTypePointer(SimTypeRef("WINBIO_STORAGE_RECORD", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Pipeline", "RecordContents"]), offset=0)),)), name="WINBIO_STORAGE_INTERFACE", pack=False, align=None),
    "WINBIO_STORAGE_RECORD": SimStruct(OrderedDict((("Identity", SimTypePointer(SimTypeRef("WINBIO_IDENTITY", SimStruct), offset=0)), ("SubFactor", SimTypeChar(label="Byte")), ("IndexVector", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("IndexElementCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("TemplateBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("TemplateBlobSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PayloadBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PayloadBlobSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WINBIO_STORAGE_RECORD", pack=False, align=None),
    "WINBIO_STORAGE_SCHEMA": SimStruct(OrderedDict((("BiometricFactor", SimTypeInt(signed=False, label="UInt32")), ("DatabaseId", SimTypeBottom(label="Guid")), ("DataFormat", SimTypeBottom(label="Guid")), ("Attributes", SimTypeInt(signed=False, label="UInt32")), ("FilePath", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("ConnectionString", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)),)), name="WINBIO_STORAGE_SCHEMA", pack=False, align=None),
    "WINBIO_SUPPORTED_ALGORITHMS": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("WinBioHresult", SimTypeInt(signed=True, label="Int32")), ("NumberOfAlgorithms", SimTypeInt(signed=False, label="UInt32")), ("AlgorithmData", SimTypeRef("WINBIO_DATA", SimStruct)),)), name="WINBIO_SUPPORTED_ALGORITHMS", pack=False, align=None),
    "WINBIO_UNIT_SCHEMA": SimStruct(OrderedDict((("UnitId", SimTypeInt(signed=False, label="UInt32")), ("PoolType", SimTypeInt(signed=False, label="UInt32")), ("BiometricFactor", SimTypeInt(signed=False, label="UInt32")), ("SensorSubType", SimTypeInt(signed=False, label="UInt32")), ("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("DeviceInstanceId", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Description", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Manufacturer", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("Model", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("SerialNumber", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 256)), ("FirmwareVersion", SimTypeRef("WINBIO_VERSION", SimStruct)),)), name="WINBIO_UNIT_SCHEMA", pack=False, align=None),
    "WINBIO_UPDATE_FIRMWARE": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("FirmwareData", SimTypeRef("WINBIO_DATA", SimStruct)),)), name="WINBIO_UPDATE_FIRMWARE", pack=False, align=None),
    "WINBIO_VERSION": SimStruct(OrderedDict((("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")),)), name="WINBIO_VERSION", pack=False, align=None),
    "WINDBG_EXTENSION_APIS": SimStruct(OrderedDict((("nSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFormat"]), offset=0)), ("lpGetExpressionRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["lpExpression"]), offset=0)), ("lpGetSymbolRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0)), ("lpDisasmRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0)), ("lpCheckControlCRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("lpReadProcessMemoryRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0)), ("lpWriteProcessMemoryRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0)), ("lpGetThreadContextRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0)), ("lpSetThreadContextRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0)), ("lpIoctlRoutine", SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["IoctlType", "lpvData", "cbSize"]), offset=0)), ("lpStackTraceRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("EXTSTACKTRACE", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["FramePointer", "StackPointer", "ProgramCounter", "StackFrames", "Frames"]), offset=0)),)), name="WINDBG_EXTENSION_APIS", pack=False, align=None),
    "WINDBG_EXTENSION_APIS32": SimStruct(OrderedDict((("nSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFormat"]), offset=0)), ("lpGetExpressionRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpExpression"]), offset=0)), ("lpGetSymbolRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0)), ("lpDisasmRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0)), ("lpCheckControlCRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("lpReadProcessMemoryRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0)), ("lpWriteProcessMemoryRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0)), ("lpGetThreadContextRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0)), ("lpSetThreadContextRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0)), ("lpIoctlRoutine", SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["IoctlType", "lpvData", "cbSize"]), offset=0)), ("lpStackTraceRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("EXTSTACKTRACE32", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["FramePointer", "StackPointer", "ProgramCounter", "StackFrames", "Frames"]), offset=0)),)), name="WINDBG_EXTENSION_APIS32", pack=False, align=None),
    "WINDBG_EXTENSION_APIS64": SimStruct(OrderedDict((("nSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFormat"]), offset=0)), ("lpGetExpressionRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeLongLong(signed=False, label="UInt64"), arg_names=["lpExpression"]), offset=0)), ("lpGetSymbolRoutine", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0)), ("lpDisasmRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0)), ("lpCheckControlCRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("lpReadProcessMemoryRoutine", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0)), ("lpWriteProcessMemoryRoutine", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0)), ("lpGetThreadContextRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0)), ("lpSetThreadContextRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("CONTEXT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Processor", "lpContext", "cbSizeOfContext"]), offset=0)), ("lpIoctlRoutine", SimTypePointer(SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["IoctlType", "lpvData", "cbSize"]), offset=0)), ("lpStackTraceRoutine", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("EXTSTACKTRACE64", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["FramePointer", "StackPointer", "ProgramCounter", "StackFrames", "Frames"]), offset=0)),)), name="WINDBG_EXTENSION_APIS64", pack=False, align=None),
    "WINDBG_OLDKD_EXTENSION_APIS": SimStruct(OrderedDict((("nSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFormat"]), offset=0)), ("lpGetExpressionRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpExpression"]), offset=0)), ("lpGetSymbolRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0)), ("lpDisasmRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0)), ("lpCheckControlCRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)), ("lpReadVirtualMemRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesRead"]), offset=0)), ("lpWriteVirtualMemRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["offset", "lpBuffer", "cb", "lpcbBytesWritten"]), offset=0)), ("lpReadPhysicalMemRoutine", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["address", "buffer", "count", "bytesread"]), offset=0)), ("lpWritePhysicalMemRoutine", SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["address", "buffer", "length", "byteswritten"]), offset=0)),)), name="WINDBG_OLDKD_EXTENSION_APIS", pack=False, align=None),
    "WINDBG_OLD_EXTENSION_APIS": SimStruct(OrderedDict((("nSize", SimTypeInt(signed=False, label="UInt32")), ("lpOutputRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["lpFormat"]), offset=0)), ("lpGetExpressionRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["lpExpression"]), offset=0)), ("lpGetSymbolRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["offset", "pchBuffer", "pDisplacement"]), offset=0)), ("lpDisasmRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["lpOffset", "lpBuffer", "fShowEffectiveAddress"]), offset=0)), ("lpCheckControlCRoutine", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")), offset=0)),)), name="WINDBG_OLD_EXTENSION_APIS", pack=False, align=None),
    "WINDOWDATA": SimStruct(OrderedDict((("dwWindowID", SimTypeInt(signed=False, label="UInt32")), ("uiCP", SimTypeInt(signed=False, label="UInt32")), ("pidl", SimTypePointer(SimTypeRef("ITEMIDLIST", SimStruct), offset=0)), ("lpszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszUrlLocation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszTitle", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WINDOWDATA", pack=False, align=None),
    "WINDOWINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("rcWindow", SimTypeRef("RECT", SimStruct)), ("rcClient", SimTypeRef("RECT", SimStruct)), ("dwStyle", SimTypeInt(signed=False, label="WINDOW_STYLE")), ("dwExStyle", SimTypeInt(signed=False, label="WINDOW_EX_STYLE")), ("dwWindowStatus", SimTypeInt(signed=False, label="UInt32")), ("cxWindowBorders", SimTypeInt(signed=False, label="UInt32")), ("cyWindowBorders", SimTypeInt(signed=False, label="UInt32")), ("atomWindowType", SimTypeShort(signed=False, label="UInt16")), ("wCreatorVersion", SimTypeShort(signed=False, label="UInt16")),)), name="WINDOWINFO", pack=False, align=None),
    "WINDOWPARTS": SimTypeInt(signed=False, label="WINDOWPARTS"),
    "WINDOWPLACEMENT": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("flags", SimTypeInt(signed=False, label="WINDOWPLACEMENT_FLAGS")), ("showCmd", SimTypeInt(signed=False, label="UInt32")), ("ptMinPosition", SimTypeRef("POINT", SimStruct)), ("ptMaxPosition", SimTypeRef("POINT", SimStruct)), ("rcNormalPosition", SimTypeRef("RECT", SimStruct)),)), name="WINDOWPLACEMENT", pack=False, align=None),
    "WINDOWPLACEMENT_FLAGS": SimTypeInt(signed=False, label="WINDOWPLACEMENT_FLAGS"),
    "WINDOWPOS": SimStruct(OrderedDict((("hwnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hwndInsertAfter", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("x", SimTypeInt(signed=True, label="Int32")), ("y", SimTypeInt(signed=True, label="Int32")), ("cx", SimTypeInt(signed=True, label="Int32")), ("cy", SimTypeInt(signed=True, label="Int32")), ("flags", SimTypeInt(signed=False, label="SET_WINDOW_POS_FLAGS")),)), name="WINDOWPOS", pack=False, align=None),
    "WINDOWS_DEVICELIST": SimStruct(OrderedDict((("numDevice", SimTypeInt(signed=False, label="UInt32")), ("Device", SimTypePointer(SimTypeRef("WINDOWS_IRDA_DEVICE_INFO", SimStruct), offset=0)),)), name="WINDOWS_DEVICELIST", pack=False, align=None),
    "WINDOWS_HOOK_ID": SimTypeInt(signed=False, label="WINDOWS_HOOK_ID"),
    "WINDOWS_IAS_QUERY": SimStruct(OrderedDict((("irdaDeviceID", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("irdaClassName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("irdaAttribName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("irdaAttribType", SimTypeInt(signed=False, label="UInt32")), ("irdaAttribute", SimUnion({"irdaAttribInt": SimTypeInt(signed=True, label="Int32"), "irdaAttribOctetSeq": SimTypeRef("_irdaAttribOctetSeq_e__Struct", SimStruct), "irdaAttribUsrStr": SimTypeRef("_irdaAttribUsrStr_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINDOWS_IAS_QUERY", pack=False, align=None),
    "WINDOWS_IAS_SET": SimStruct(OrderedDict((("irdaClassName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("irdaAttribName", SimTypeArray(SimTypeChar(label="SByte"), 256)), ("irdaAttribType", SimTypeInt(signed=False, label="UInt32")), ("irdaAttribute", SimUnion({"irdaAttribInt": SimTypeInt(signed=True, label="Int32"), "irdaAttribOctetSeq": SimTypeRef("_irdaAttribOctetSeq_e__Struct", SimStruct), "irdaAttribUsrStr": SimTypeRef("_irdaAttribUsrStr_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WINDOWS_IAS_SET", pack=False, align=None),
    "WINDOWS_IRDA_DEVICE_INFO": SimStruct(OrderedDict((("irdaDeviceID", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("irdaDeviceName", SimTypeArray(SimTypeChar(label="SByte"), 22)), ("irdaDeviceHints1", SimTypeChar(label="Byte")), ("irdaDeviceHints2", SimTypeChar(label="Byte")), ("irdaCharSet", SimTypeChar(label="Byte")),)), name="WINDOWS_IRDA_DEVICE_INFO", pack=False, align=None),
    "WINDOWTHEMEATTRIBUTETYPE": SimTypeInt(signed=False, label="WINDOWTHEMEATTRIBUTETYPE"),
    "WINDOW_BUFFER_SIZE_RECORD": SimStruct(OrderedDict((("dwSize", SimTypeRef("COORD", SimStruct)),)), name="WINDOW_BUFFER_SIZE_RECORD", pack=False, align=None),
    "WINDOW_DISPLAY_AFFINITY": SimTypeInt(signed=False, label="WINDOW_DISPLAY_AFFINITY"),
    "WINDOW_EX_STYLE": SimTypeInt(signed=False, label="WINDOW_EX_STYLE"),
    "WINDOW_LONG_PTR_INDEX": SimTypeInt(signed=False, label="WINDOW_LONG_PTR_INDEX"),
    "WINDOW_MESSAGE_FILTER_ACTION": SimTypeInt(signed=False, label="WINDOW_MESSAGE_FILTER_ACTION"),
    "WINDOW_STYLE": SimTypeInt(signed=False, label="WINDOW_STYLE"),
    "WINEVENTPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWinEventHook", "event", "hwnd", "idObject", "idChild", "idEventThread", "dwmsEventTime"]), offset=0),
    "WINHTTP_ACCESS_TYPE": SimTypeInt(signed=False, label="WINHTTP_ACCESS_TYPE"),
    "WINHTTP_ASYNC_RESULT": SimStruct(OrderedDict((("dwResult", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")),)), name="WINHTTP_ASYNC_RESULT", pack=False, align=None),
    "WINHTTP_AUTOPROXY_OPTIONS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwAutoDetectFlags", SimTypeInt(signed=False, label="UInt32")), ("lpszAutoConfigUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("fAutoLogonIfChallenged", SimTypeInt(signed=True, label="Int32")),)), name="WINHTTP_AUTOPROXY_OPTIONS", pack=False, align=None),
    "WINHTTP_CERTIFICATE_INFO": SimStruct(OrderedDict((("ftExpiry", SimTypeRef("FILETIME", SimStruct)), ("ftStart", SimTypeRef("FILETIME", SimStruct)), ("lpszSubjectInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszIssuerInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszProtocolName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszSignatureAlgName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszEncryptionAlgName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwKeySize", SimTypeInt(signed=False, label="UInt32")),)), name="WINHTTP_CERTIFICATE_INFO", pack=False, align=None),
    "WINHTTP_CONNECTION_GROUP": SimStruct(OrderedDict((("cConnections", SimTypeInt(signed=False, label="UInt32")), ("guidGroup", SimTypeBottom(label="Guid")),)), name="WINHTTP_CONNECTION_GROUP", pack=False, align=None),
    "WINHTTP_CONNECTION_INFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("LocalAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)), ("RemoteAddress", SimTypeRef("SOCKADDR_STORAGE", SimStruct)),)), name="WINHTTP_CONNECTION_INFO", pack=False, align=None),
    "WINHTTP_CREDS": SimStruct(OrderedDict((("lpszUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszRealm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwAuthScheme", SimTypeInt(signed=False, label="WINHTTP_CREDS_AUTHSCHEME")), ("lpszHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwPort", SimTypeInt(signed=False, label="UInt32")),)), name="WINHTTP_CREDS", pack=False, align=None),
    "WINHTTP_CREDS_AUTHSCHEME": SimTypeInt(signed=False, label="WINHTTP_CREDS_AUTHSCHEME"),
    "WINHTTP_CREDS_EX": SimStruct(OrderedDict((("lpszUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszPassword", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszRealm", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwAuthScheme", SimTypeInt(signed=False, label="WINHTTP_CREDS_AUTHSCHEME")), ("lpszHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwPort", SimTypeInt(signed=False, label="UInt32")), ("lpszUrl", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WINHTTP_CREDS_EX", pack=False, align=None),
    "WINHTTP_CURRENT_USER_IE_PROXY_CONFIG": SimStruct(OrderedDict((("fAutoDetect", SimTypeInt(signed=True, label="Int32")), ("lpszAutoConfigUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszProxyBypass", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WINHTTP_CURRENT_USER_IE_PROXY_CONFIG", pack=False, align=None),
    "WINHTTP_EXTENDED_HEADER": SimStruct(OrderedDict((("Anonymous1", SimUnion({"pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszName": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"pwszValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None")),)), name="WINHTTP_EXTENDED_HEADER", pack=False, align=None),
    "WINHTTP_FAILED_CONNECTION_RETRIES": SimStruct(OrderedDict((("dwMaxRetries", SimTypeInt(signed=False, label="UInt32")), ("dwAllowedRetryConditions", SimTypeInt(signed=False, label="UInt32")),)), name="WINHTTP_FAILED_CONNECTION_RETRIES", pack=False, align=None),
    "WINHTTP_HEADER_NAME": SimUnion({"pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszName": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="<anon>", label="None"),
    "WINHTTP_HOST_CONNECTION_GROUP": SimStruct(OrderedDict((("pwszHost", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cConnectionGroups", SimTypeInt(signed=False, label="UInt32")), ("pConnectionGroups", SimTypePointer(SimTypeRef("WINHTTP_CONNECTION_GROUP", SimStruct), offset=0)),)), name="WINHTTP_HOST_CONNECTION_GROUP", pack=False, align=None),
    "WINHTTP_HTTP2_RECEIVE_WINDOW": SimStruct(OrderedDict((("ulStreamWindow", SimTypeInt(signed=False, label="UInt32")), ("ulStreamWindowUpdateDelta", SimTypeInt(signed=False, label="UInt32")),)), name="WINHTTP_HTTP2_RECEIVE_WINDOW", pack=False, align=None),
    "WINHTTP_INTERNET_SCHEME": SimTypeInt(signed=False, label="WINHTTP_INTERNET_SCHEME"),
    "WINHTTP_MATCH_CONNECTION_GUID": SimStruct(OrderedDict((("ConnectionGuid", SimTypeBottom(label="Guid")), ("ullFlags", SimTypeLongLong(signed=False, label="UInt64")),)), name="WINHTTP_MATCH_CONNECTION_GUID", pack=False, align=None),
    "WINHTTP_OPEN_REQUEST_FLAGS": SimTypeInt(signed=False, label="WINHTTP_OPEN_REQUEST_FLAGS"),
    "WINHTTP_PROXY_CHANGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ullFlags", "pvContext"]), offset=0),
    "WINHTTP_PROXY_INFO": SimStruct(OrderedDict((("dwAccessType", SimTypeInt(signed=False, label="WINHTTP_ACCESS_TYPE")), ("lpszProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszProxyBypass", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WINHTTP_PROXY_INFO", pack=False, align=None),
    "WINHTTP_PROXY_NETWORKING_KEY": SimStruct(OrderedDict((("pbBuffer", SimTypeArray(SimTypeChar(label="Byte"), 128)),)), name="WINHTTP_PROXY_NETWORKING_KEY", pack=False, align=None),
    "WINHTTP_PROXY_RESULT": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pEntries", SimTypePointer(SimTypeRef("WINHTTP_PROXY_RESULT_ENTRY", SimStruct), offset=0)),)), name="WINHTTP_PROXY_RESULT", pack=False, align=None),
    "WINHTTP_PROXY_RESULT_ENTRY": SimStruct(OrderedDict((("fProxy", SimTypeInt(signed=True, label="Int32")), ("fBypass", SimTypeInt(signed=True, label="Int32")), ("ProxyScheme", SimTypeInt(signed=False, label="WINHTTP_INTERNET_SCHEME")), ("pwszProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProxyPort", SimTypeShort(signed=False, label="UInt16")),)), name="WINHTTP_PROXY_RESULT_ENTRY", pack=False, align=None),
    "WINHTTP_PROXY_RESULT_EX": SimStruct(OrderedDict((("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pEntries", SimTypePointer(SimTypeRef("WINHTTP_PROXY_RESULT_ENTRY", SimStruct), offset=0)), ("hProxyDetectionHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwProxyInterfaceAffinity", SimTypeInt(signed=False, label="UInt32")),)), name="WINHTTP_PROXY_RESULT_EX", pack=False, align=None),
    "WINHTTP_PROXY_SETTINGS": SimStruct(OrderedDict((("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentSettingsVersion", SimTypeInt(signed=False, label="UInt32")), ("pwszConnectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszProxyBypass", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszAutoconfigUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszAutoconfigSecondaryUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAutoDiscoveryFlags", SimTypeInt(signed=False, label="UInt32")), ("pwszLastKnownGoodAutoConfigUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwAutoconfigReloadDelayMins", SimTypeInt(signed=False, label="UInt32")), ("ftLastKnownDetectTime", SimTypeRef("FILETIME", SimStruct)), ("dwDetectedInterfaceIpCount", SimTypeInt(signed=False, label="UInt32")), ("pdwDetectedInterfaceIp", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("cNetworkKeys", SimTypeInt(signed=False, label="UInt32")), ("pNetworkKeys", SimTypePointer(SimTypeRef("WINHTTP_PROXY_NETWORKING_KEY", SimStruct), offset=0)),)), name="WINHTTP_PROXY_SETTINGS", pack=False, align=None),
    "WINHTTP_PROXY_SETTINGS_EX": SimStruct(OrderedDict((("ullGenerationId", SimTypeLongLong(signed=False, label="UInt64")), ("ullFlags", SimTypeLongLong(signed=False, label="UInt64")), ("pcwszAutoconfigUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszSecureProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cProxyBypasses", SimTypeInt(signed=False, label="UInt32")), ("rgpcwszProxyBypasses", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)), ("dwInterfaceIndex", SimTypeInt(signed=False, label="UInt32")), ("pcwszConnectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WINHTTP_PROXY_SETTINGS_EX", pack=False, align=None),
    "WINHTTP_PROXY_SETTINGS_PARAM": SimStruct(OrderedDict((("ullFlags", SimTypeLongLong(signed=False, label="UInt64")), ("pcwszConnectionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszProbeHost", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WINHTTP_PROXY_SETTINGS_PARAM", pack=False, align=None),
    "WINHTTP_PROXY_SETTINGS_TYPE": SimTypeInt(signed=False, label="WINHTTP_PROXY_SETTINGS_TYPE"),
    "WINHTTP_QUERY_CONNECTION_GROUP_RESULT": SimStruct(OrderedDict((("cHosts", SimTypeInt(signed=False, label="UInt32")), ("pHostConnectionGroups", SimTypePointer(SimTypeRef("WINHTTP_HOST_CONNECTION_GROUP", SimStruct), offset=0)),)), name="WINHTTP_QUERY_CONNECTION_GROUP_RESULT", pack=False, align=None),
    "WINHTTP_REQUEST_STATS": SimStruct(OrderedDict((("ullFlags", SimTypeLongLong(signed=False, label="UInt64")), ("ulIndex", SimTypeInt(signed=False, label="UInt32")), ("cStats", SimTypeInt(signed=False, label="UInt32")), ("rgullStats", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 32)),)), name="WINHTTP_REQUEST_STATS", pack=False, align=None),
    "WINHTTP_REQUEST_STAT_ENTRY": SimTypeInt(signed=False, label="WINHTTP_REQUEST_STAT_ENTRY"),
    "WINHTTP_REQUEST_TIMES": SimStruct(OrderedDict((("cTimes", SimTypeInt(signed=False, label="UInt32")), ("rgullTimes", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 64)),)), name="WINHTTP_REQUEST_TIMES", pack=False, align=None),
    "WINHTTP_REQUEST_TIME_ENTRY": SimTypeInt(signed=False, label="WINHTTP_REQUEST_TIME_ENTRY"),
    "WINHTTP_RESOLVER_CACHE_CONFIG": SimStruct(OrderedDict((("ulMaxResolverCacheEntries", SimTypeInt(signed=False, label="UInt32")), ("ulMaxCacheEntryAge", SimTypeInt(signed=False, label="UInt32")), ("ulMinCacheEntryTtl", SimTypeInt(signed=False, label="UInt32")), ("SecureDnsSetting", SimTypeInt(signed=False, label="WINHTTP_SECURE_DNS_SETTING")), ("ullConnResolutionWaitTime", SimTypeLongLong(signed=False, label="UInt64")), ("ullFlags", SimTypeLongLong(signed=False, label="UInt64")),)), name="WINHTTP_RESOLVER_CACHE_CONFIG", pack=False, align=None),
    "WINHTTP_SECURE_DNS_SETTING": SimTypeInt(signed=False, label="WINHTTP_SECURE_DNS_SETTING"),
    "WINHTTP_STATUS_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hInternet", "dwContext", "dwInternetStatus", "lpvStatusInformation", "dwStatusInformationLength"]), offset=0),
    "WINHTTP_WEB_SOCKET_ASYNC_RESULT": SimStruct(OrderedDict((("AsyncResult", SimTypeRef("WINHTTP_ASYNC_RESULT", SimStruct)), ("Operation", SimTypeInt(signed=False, label="WINHTTP_WEB_SOCKET_OPERATION")),)), name="WINHTTP_WEB_SOCKET_ASYNC_RESULT", pack=False, align=None),
    "WINHTTP_WEB_SOCKET_BUFFER_TYPE": SimTypeInt(signed=False, label="WINHTTP_WEB_SOCKET_BUFFER_TYPE"),
    "WINHTTP_WEB_SOCKET_CLOSE_STATUS": SimTypeInt(signed=False, label="WINHTTP_WEB_SOCKET_CLOSE_STATUS"),
    "WINHTTP_WEB_SOCKET_OPERATION": SimTypeInt(signed=False, label="WINHTTP_WEB_SOCKET_OPERATION"),
    "WINHTTP_WEB_SOCKET_STATUS": SimStruct(OrderedDict((("dwBytesTransferred", SimTypeInt(signed=False, label="UInt32")), ("eBufferType", SimTypeInt(signed=False, label="WINHTTP_WEB_SOCKET_BUFFER_TYPE")),)), name="WINHTTP_WEB_SOCKET_STATUS", pack=False, align=None),
    "WINIBIO_ENGINE_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WINIBIO_SENSOR_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WINIBIO_STORAGE_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WININET_PROXY_INFO": SimStruct(OrderedDict((("fProxy", SimTypeInt(signed=True, label="Int32")), ("fBypass", SimTypeInt(signed=True, label="Int32")), ("ProxyScheme", SimTypeInt(signed=False, label="INTERNET_SCHEME")), ("pwszProxy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProxyPort", SimTypeShort(signed=False, label="UInt16")),)), name="WININET_PROXY_INFO", pack=False, align=None),
    "WININET_PROXY_INFO_LIST": SimStruct(OrderedDict((("dwProxyInfoCount", SimTypeInt(signed=False, label="UInt32")), ("pProxyInfo", SimTypePointer(SimTypeRef("WININET_PROXY_INFO", SimStruct), offset=0)),)), name="WININET_PROXY_INFO_LIST", pack=False, align=None),
    "WININET_SYNC_MODE": SimTypeInt(signed=False, label="WININET_SYNC_MODE"),
    "WINML_BINDING_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BindType", SimTypeInt(signed=False, label="WINML_BINDING_TYPE")), ("Anonymous", SimUnion({"Tensor": SimTypeRef("WINML_TENSOR_BINDING_DESC", SimStruct), "Sequence": SimTypeRef("WINML_SEQUENCE_BINDING_DESC", SimStruct), "Map": SimTypeRef("WINML_MAP_BINDING_DESC", SimStruct), "Image": SimTypeRef("WINML_IMAGE_BINDING_DESC", SimStruct), "Resource": SimTypeRef("WINML_RESOURCE_BINDING_DESC", SimStruct)}, name="<anon>", label="None")),)), name="WINML_BINDING_DESC", pack=False, align=None),
    "WINML_BINDING_TYPE": SimTypeInt(signed=False, label="WINML_BINDING_TYPE"),
    "WINML_FEATURE_TYPE": SimTypeInt(signed=False, label="WINML_FEATURE_TYPE"),
    "WINML_IMAGE_BINDING_DESC": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("NumDimensions", SimTypeInt(signed=False, label="UInt32")), ("pShape", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WINML_IMAGE_BINDING_DESC", pack=False, align=None),
    "WINML_IMAGE_VARIABLE_DESC": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("NumDimensions", SimTypeInt(signed=False, label="UInt32")), ("pShape", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="WINML_IMAGE_VARIABLE_DESC", pack=False, align=None),
    "WINML_MAP_BINDING_DESC": SimStruct(OrderedDict((("ElementCount", SimTypeInt(signed=False, label="UInt32")), ("KeyType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("Anonymous1", SimUnion({"pStringKeys": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "pIntKeys": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)}, name="<anon>", label="None")), ("Fields", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("Anonymous2", SimUnion({"pStringFields": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "pIntFields": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pFloatFields": SimTypePointer(SimTypeFloat(size=32), offset=0), "pDoubleFields": SimTypePointer(SimTypeFloat(size=64), offset=0)}, name="<anon>", label="None")),)), name="WINML_MAP_BINDING_DESC", pack=False, align=None),
    "WINML_MAP_VARIABLE_DESC": SimStruct(OrderedDict((("KeyType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("Fields", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")),)), name="WINML_MAP_VARIABLE_DESC", pack=False, align=None),
    "WINML_MODEL_DESC": SimStruct(OrderedDict((("Author", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Version", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WINML_MODEL_DESC", pack=False, align=None),
    "WINML_RESOURCE_BINDING_DESC": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("NumDimensions", SimTypeInt(signed=False, label="UInt32")), ("pShape", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("pResource", SimTypeBottom(label="ID3D12Resource")),)), name="WINML_RESOURCE_BINDING_DESC", pack=False, align=None),
    "WINML_RUNTIME_TYPE": SimTypeInt(signed=False, label="WINML_RUNTIME_TYPE"),
    "WINML_SEQUENCE_BINDING_DESC": SimStruct(OrderedDict((("ElementCount", SimTypeInt(signed=False, label="UInt32")), ("ElementType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("Anonymous", SimUnion({"pStrings": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "pInts": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pFloats": SimTypePointer(SimTypeFloat(size=32), offset=0), "pDoubles": SimTypePointer(SimTypeFloat(size=64), offset=0)}, name="<anon>", label="None")),)), name="WINML_SEQUENCE_BINDING_DESC", pack=False, align=None),
    "WINML_SEQUENCE_VARIABLE_DESC": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")),)), name="WINML_SEQUENCE_VARIABLE_DESC", pack=False, align=None),
    "WINML_TENSOR_BINDING_DESC": SimStruct(OrderedDict((("DataType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("NumDimensions", SimTypeInt(signed=False, label="UInt32")), ("pShape", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WINML_TENSOR_BINDING_DESC", pack=False, align=None),
    "WINML_TENSOR_DATA_TYPE": SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE"),
    "WINML_TENSOR_VARIABLE_DESC": SimStruct(OrderedDict((("ElementType", SimTypeInt(signed=False, label="WINML_TENSOR_DATA_TYPE")), ("NumDimensions", SimTypeInt(signed=False, label="UInt32")), ("pShape", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)),)), name="WINML_TENSOR_VARIABLE_DESC", pack=False, align=None),
    "WINML_VARIABLE_DESC": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Description", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FeatureType", SimTypeInt(signed=False, label="WINML_FEATURE_TYPE")), ("Required", SimTypeInt(signed=True, label="Int32")), ("Anonymous", SimUnion({"Tensor": SimTypeRef("WINML_TENSOR_VARIABLE_DESC", SimStruct), "Sequence": SimTypeRef("WINML_SEQUENCE_VARIABLE_DESC", SimStruct), "Map": SimTypeRef("WINML_MAP_VARIABLE_DESC", SimStruct), "Image": SimTypeRef("WINML_IMAGE_VARIABLE_DESC", SimStruct)}, name="<anon>", label="None")),)), name="WINML_VARIABLE_DESC", pack=False, align=None),
    "WINSAT_ASSESSMENT_STATE": SimTypeInt(signed=False, label="WINSAT_ASSESSMENT_STATE"),
    "WINSAT_ASSESSMENT_TYPE": SimTypeInt(signed=False, label="WINSAT_ASSESSMENT_TYPE"),
    "WINSAT_BITMAP_SIZE": SimTypeInt(signed=False, label="WINSAT_BITMAP_SIZE"),
    "WINSAT_OEM_CUSTOMIZATION_STATE": SimTypeInt(signed=False, label="WINSAT_OEM_CUSTOMIZATION_STATE"),
    "WINSOCK_SHUTDOWN_HOW": SimTypeInt(signed=False, label="WINSOCK_SHUTDOWN_HOW"),
    "WINSOCK_SOCKET_TYPE": SimTypeInt(signed=False, label="WINSOCK_SOCKET_TYPE"),
    "WINSTAENUMPROCA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "WINSTAENUMPROCW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "WINSTATIONINFOCLASS": SimTypeInt(signed=False, label="WINSTATIONINFOCLASS"),
    "WINSTATIONINFORMATIONW": SimStruct(OrderedDict((("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 70)), ("LogonId", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeArray(SimTypeChar(label="Byte"), 1140)),)), name="WINSTATIONINFORMATIONW", pack=False, align=None),
    "WINTRUST_BLOB_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("gSubject", SimTypeBottom(label="Guid")), ("pcwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("cbMemObject", SimTypeInt(signed=False, label="UInt32")), ("pbMemObject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbMemSignedMsg", SimTypeInt(signed=False, label="UInt32")), ("pbMemSignedMsg", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WINTRUST_BLOB_INFO", pack=False, align=None),
    "WINTRUST_CATALOG_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwCatalogVersion", SimTypeInt(signed=False, label="UInt32")), ("pcwszCatalogFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszMemberTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pcwszMemberFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hMemberFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pbCalculatedFileHash", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbCalculatedFileHash", SimTypeInt(signed=False, label="UInt32")), ("pcCatalogContext", SimTypePointer(SimTypeRef("CTL_CONTEXT", SimStruct), offset=0)), ("hCatAdmin", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WINTRUST_CATALOG_INFO", pack=False, align=None),
    "WINTRUST_CERT_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pcwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("psCertContext", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("chStores", SimTypeInt(signed=False, label="UInt32")), ("pahStores", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("psftVerifyAsOf", SimTypePointer(SimTypeRef("FILETIME", SimStruct), offset=0)),)), name="WINTRUST_CERT_INFO", pack=False, align=None),
    "WINTRUST_DATA": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pPolicyCallbackData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("pSIPClientData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwUIChoice", SimTypeInt(signed=False, label="WINTRUST_DATA_UICHOICE")), ("fdwRevocationChecks", SimTypeInt(signed=False, label="WINTRUST_DATA_REVOCATION_CHECKS")), ("dwUnionChoice", SimTypeInt(signed=False, label="WINTRUST_DATA_UNION_CHOICE")), ("Anonymous", SimUnion({"pFile": SimTypePointer(SimTypeRef("WINTRUST_FILE_INFO", SimStruct), offset=0), "pCatalog": SimTypePointer(SimTypeRef("WINTRUST_CATALOG_INFO", SimStruct), offset=0), "pBlob": SimTypePointer(SimTypeRef("WINTRUST_BLOB_INFO", SimStruct), offset=0), "pSgnr": SimTypePointer(SimTypeRef("WINTRUST_SGNR_INFO", SimStruct), offset=0), "pCert": SimTypePointer(SimTypeRef("WINTRUST_CERT_INFO", SimStruct), offset=0)}, name="<anon>", label="None")), ("dwStateAction", SimTypeInt(signed=False, label="WINTRUST_DATA_STATE_ACTION")), ("hWVTStateData", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pwszURLReference", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwProvFlags", SimTypeInt(signed=False, label="WINTRUST_DATA_PROVIDER_FLAGS")), ("dwUIContext", SimTypeInt(signed=False, label="WINTRUST_DATA_UICONTEXT")), ("pSignatureSettings", SimTypePointer(SimTypeRef("WINTRUST_SIGNATURE_SETTINGS", SimStruct), offset=0)),)), name="WINTRUST_DATA", pack=False, align=None),
    "WINTRUST_DATA_PROVIDER_FLAGS": SimTypeInt(signed=False, label="WINTRUST_DATA_PROVIDER_FLAGS"),
    "WINTRUST_DATA_REVOCATION_CHECKS": SimTypeInt(signed=False, label="WINTRUST_DATA_REVOCATION_CHECKS"),
    "WINTRUST_DATA_STATE_ACTION": SimTypeInt(signed=False, label="WINTRUST_DATA_STATE_ACTION"),
    "WINTRUST_DATA_UICHOICE": SimTypeInt(signed=False, label="WINTRUST_DATA_UICHOICE"),
    "WINTRUST_DATA_UICONTEXT": SimTypeInt(signed=False, label="WINTRUST_DATA_UICONTEXT"),
    "WINTRUST_DATA_UNION_CHOICE": SimTypeInt(signed=False, label="WINTRUST_DATA_UNION_CHOICE"),
    "WINTRUST_FILE_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pcwszFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pgKnownSubject", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="WINTRUST_FILE_INFO", pack=False, align=None),
    "WINTRUST_GET_DEFAULT_FOR_USAGE_ACTION": SimTypeInt(signed=False, label="WINTRUST_GET_DEFAULT_FOR_USAGE_ACTION"),
    "WINTRUST_POLICY_FLAGS": SimTypeInt(signed=False, label="WINTRUST_POLICY_FLAGS"),
    "WINTRUST_SGNR_INFO": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("pcwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("psSignerInfo", SimTypePointer(SimTypeRef("CMSG_SIGNER_INFO", SimStruct), offset=0)), ("chStores", SimTypeInt(signed=False, label="UInt32")), ("pahStores", SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)),)), name="WINTRUST_SGNR_INFO", pack=False, align=None),
    "WINTRUST_SIGNATURE_SETTINGS": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="WINTRUST_SIGNATURE_SETTINGS_FLAGS")), ("cSecondarySigs", SimTypeInt(signed=False, label="UInt32")), ("dwVerifiedSigIndex", SimTypeInt(signed=False, label="UInt32")), ("pCryptoPolicy", SimTypePointer(SimTypeRef("CERT_STRONG_SIGN_PARA", SimStruct), offset=0)),)), name="WINTRUST_SIGNATURE_SETTINGS", pack=False, align=None),
    "WINTRUST_SIGNATURE_SETTINGS_FLAGS": SimTypeInt(signed=False, label="WINTRUST_SIGNATURE_SETTINGS_FLAGS"),
    "WINUSB_INTERFACE_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WINUSB_PIPE_INFORMATION": SimStruct(OrderedDict((("PipeType", SimTypeInt(signed=False, label="USBD_PIPE_TYPE")), ("PipeId", SimTypeChar(label="Byte")), ("MaximumPacketSize", SimTypeShort(signed=False, label="UInt16")), ("Interval", SimTypeChar(label="Byte")),)), name="WINUSB_PIPE_INFORMATION", pack=False, align=None),
    "WINUSB_PIPE_INFORMATION_EX": SimStruct(OrderedDict((("PipeType", SimTypeInt(signed=False, label="USBD_PIPE_TYPE")), ("PipeId", SimTypeChar(label="Byte")), ("MaximumPacketSize", SimTypeShort(signed=False, label="UInt16")), ("Interval", SimTypeChar(label="Byte")), ("MaximumBytesPerInterval", SimTypeInt(signed=False, label="UInt32")),)), name="WINUSB_PIPE_INFORMATION_EX", pack=False, align=None),
    "WINUSB_PIPE_POLICY": SimTypeInt(signed=False, label="WINUSB_PIPE_POLICY"),
    "WINUSB_POWER_POLICY": SimTypeInt(signed=False, label="WINUSB_POWER_POLICY"),
    "WINUSB_SETUP_PACKET": SimStruct(OrderedDict((("RequestType", SimTypeChar(label="Byte")), ("Request", SimTypeChar(label="Byte")), ("Value", SimTypeShort(signed=False, label="UInt16")), ("Index", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeShort(signed=False, label="UInt16")),)), name="WINUSB_SETUP_PACKET", pack=False, align=None),
    "WINWATCHNOTIFYPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hww", "hwnd", "code", "lParam"]), offset=0),
    "WIN_CERTIFICATE": SimStruct(OrderedDict((("dwLength", SimTypeInt(signed=False, label="UInt32")), ("wRevision", SimTypeShort(signed=False, label="UInt16")), ("wCertificateType", SimTypeShort(signed=False, label="UInt16")), ("bCertificate", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WIN_CERTIFICATE", pack=False, align=None),
    "WIN_HTTP_CREATE_URL_FLAGS": SimTypeInt(signed=False, label="WIN_HTTP_CREATE_URL_FLAGS"),
    "WIN_SPUB_TRUSTED_PUBLISHER_DATA": SimStruct(OrderedDict((("hClientToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpCertificate", SimTypePointer(SimTypeRef("WIN_CERTIFICATE", SimStruct), offset=0)),)), name="WIN_SPUB_TRUSTED_PUBLISHER_DATA", pack=False, align=None),
    "WIN_STREAM_ID": SimTypeInt(signed=False, label="WIN_STREAM_ID"),
    "WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT": SimStruct(OrderedDict((("hClientToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("SubjectType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("Subject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT", pack=False, align=None),
    "WIN_TRUST_ACTDATA_SUBJECT_ONLY": SimStruct(OrderedDict((("SubjectType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("Subject", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WIN_TRUST_ACTDATA_SUBJECT_ONLY", pack=False, align=None),
    "WIN_TRUST_SUBJECT_FILE": SimStruct(OrderedDict((("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIN_TRUST_SUBJECT_FILE", pack=False, align=None),
    "WIN_TRUST_SUBJECT_FILE_AND_DISPLAY": SimStruct(OrderedDict((("hFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WIN_TRUST_SUBJECT_FILE_AND_DISPLAY", pack=False, align=None),
    "WKSTA_INFO_100": SimStruct(OrderedDict((("wki100_platform_id", SimTypeInt(signed=False, label="UInt32")), ("wki100_computername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki100_langroup", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki100_ver_major", SimTypeInt(signed=False, label="UInt32")), ("wki100_ver_minor", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_100", pack=False, align=None),
    "WKSTA_INFO_101": SimStruct(OrderedDict((("wki101_platform_id", SimTypeInt(signed=False, label="UInt32")), ("wki101_computername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki101_langroup", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki101_ver_major", SimTypeInt(signed=False, label="UInt32")), ("wki101_ver_minor", SimTypeInt(signed=False, label="UInt32")), ("wki101_lanroot", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WKSTA_INFO_101", pack=False, align=None),
    "WKSTA_INFO_1010": SimStruct(OrderedDict((("wki1010_char_wait", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1010", pack=False, align=None),
    "WKSTA_INFO_1011": SimStruct(OrderedDict((("wki1011_collection_time", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1011", pack=False, align=None),
    "WKSTA_INFO_1012": SimStruct(OrderedDict((("wki1012_maximum_collection_count", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1012", pack=False, align=None),
    "WKSTA_INFO_1013": SimStruct(OrderedDict((("wki1013_keep_conn", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1013", pack=False, align=None),
    "WKSTA_INFO_1018": SimStruct(OrderedDict((("wki1018_sess_timeout", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1018", pack=False, align=None),
    "WKSTA_INFO_102": SimStruct(OrderedDict((("wki102_platform_id", SimTypeInt(signed=False, label="UInt32")), ("wki102_computername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki102_langroup", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki102_ver_major", SimTypeInt(signed=False, label="UInt32")), ("wki102_ver_minor", SimTypeInt(signed=False, label="UInt32")), ("wki102_lanroot", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki102_logged_on_users", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_102", pack=False, align=None),
    "WKSTA_INFO_1023": SimStruct(OrderedDict((("wki1023_siz_char_buf", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1023", pack=False, align=None),
    "WKSTA_INFO_1027": SimStruct(OrderedDict((("wki1027_errlog_sz", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1027", pack=False, align=None),
    "WKSTA_INFO_1028": SimStruct(OrderedDict((("wki1028_print_buf_time", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1028", pack=False, align=None),
    "WKSTA_INFO_1032": SimStruct(OrderedDict((("wki1032_wrk_heuristics", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1032", pack=False, align=None),
    "WKSTA_INFO_1033": SimStruct(OrderedDict((("wki1033_max_threads", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1033", pack=False, align=None),
    "WKSTA_INFO_1041": SimStruct(OrderedDict((("wki1041_lock_quota", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1041", pack=False, align=None),
    "WKSTA_INFO_1042": SimStruct(OrderedDict((("wki1042_lock_increment", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1042", pack=False, align=None),
    "WKSTA_INFO_1043": SimStruct(OrderedDict((("wki1043_lock_maximum", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1043", pack=False, align=None),
    "WKSTA_INFO_1044": SimStruct(OrderedDict((("wki1044_pipe_increment", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1044", pack=False, align=None),
    "WKSTA_INFO_1045": SimStruct(OrderedDict((("wki1045_pipe_maximum", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1045", pack=False, align=None),
    "WKSTA_INFO_1046": SimStruct(OrderedDict((("wki1046_dormant_file_limit", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1046", pack=False, align=None),
    "WKSTA_INFO_1047": SimStruct(OrderedDict((("wki1047_cache_file_timeout", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1047", pack=False, align=None),
    "WKSTA_INFO_1048": SimStruct(OrderedDict((("wki1048_use_opportunistic_locking", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1048", pack=False, align=None),
    "WKSTA_INFO_1049": SimStruct(OrderedDict((("wki1049_use_unlock_behind", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1049", pack=False, align=None),
    "WKSTA_INFO_1050": SimStruct(OrderedDict((("wki1050_use_close_behind", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1050", pack=False, align=None),
    "WKSTA_INFO_1051": SimStruct(OrderedDict((("wki1051_buf_named_pipes", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1051", pack=False, align=None),
    "WKSTA_INFO_1052": SimStruct(OrderedDict((("wki1052_use_lock_read_unlock", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1052", pack=False, align=None),
    "WKSTA_INFO_1053": SimStruct(OrderedDict((("wki1053_utilize_nt_caching", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1053", pack=False, align=None),
    "WKSTA_INFO_1054": SimStruct(OrderedDict((("wki1054_use_raw_read", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1054", pack=False, align=None),
    "WKSTA_INFO_1055": SimStruct(OrderedDict((("wki1055_use_raw_write", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1055", pack=False, align=None),
    "WKSTA_INFO_1056": SimStruct(OrderedDict((("wki1056_use_write_raw_data", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1056", pack=False, align=None),
    "WKSTA_INFO_1057": SimStruct(OrderedDict((("wki1057_use_encryption", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1057", pack=False, align=None),
    "WKSTA_INFO_1058": SimStruct(OrderedDict((("wki1058_buf_files_deny_write", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1058", pack=False, align=None),
    "WKSTA_INFO_1059": SimStruct(OrderedDict((("wki1059_buf_read_only_files", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1059", pack=False, align=None),
    "WKSTA_INFO_1060": SimStruct(OrderedDict((("wki1060_force_core_create_mode", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1060", pack=False, align=None),
    "WKSTA_INFO_1061": SimStruct(OrderedDict((("wki1061_use_512_byte_max_transfer", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_1061", pack=False, align=None),
    "WKSTA_INFO_1062": SimStruct(OrderedDict((("wki1062_read_ahead_throughput", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_1062", pack=False, align=None),
    "WKSTA_INFO_302": SimStruct(OrderedDict((("wki302_char_wait", SimTypeInt(signed=False, label="UInt32")), ("wki302_collection_time", SimTypeInt(signed=False, label="UInt32")), ("wki302_maximum_collection_count", SimTypeInt(signed=False, label="UInt32")), ("wki302_keep_conn", SimTypeInt(signed=False, label="UInt32")), ("wki302_keep_search", SimTypeInt(signed=False, label="UInt32")), ("wki302_max_cmds", SimTypeInt(signed=False, label="UInt32")), ("wki302_num_work_buf", SimTypeInt(signed=False, label="UInt32")), ("wki302_siz_work_buf", SimTypeInt(signed=False, label="UInt32")), ("wki302_max_wrk_cache", SimTypeInt(signed=False, label="UInt32")), ("wki302_sess_timeout", SimTypeInt(signed=False, label="UInt32")), ("wki302_siz_error", SimTypeInt(signed=False, label="UInt32")), ("wki302_num_alerts", SimTypeInt(signed=False, label="UInt32")), ("wki302_num_services", SimTypeInt(signed=False, label="UInt32")), ("wki302_errlog_sz", SimTypeInt(signed=False, label="UInt32")), ("wki302_print_buf_time", SimTypeInt(signed=False, label="UInt32")), ("wki302_num_char_buf", SimTypeInt(signed=False, label="UInt32")), ("wki302_siz_char_buf", SimTypeInt(signed=False, label="UInt32")), ("wki302_wrk_heuristics", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki302_mailslots", SimTypeInt(signed=False, label="UInt32")), ("wki302_num_dgram_buf", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_302", pack=False, align=None),
    "WKSTA_INFO_402": SimStruct(OrderedDict((("wki402_char_wait", SimTypeInt(signed=False, label="UInt32")), ("wki402_collection_time", SimTypeInt(signed=False, label="UInt32")), ("wki402_maximum_collection_count", SimTypeInt(signed=False, label="UInt32")), ("wki402_keep_conn", SimTypeInt(signed=False, label="UInt32")), ("wki402_keep_search", SimTypeInt(signed=False, label="UInt32")), ("wki402_max_cmds", SimTypeInt(signed=False, label="UInt32")), ("wki402_num_work_buf", SimTypeInt(signed=False, label="UInt32")), ("wki402_siz_work_buf", SimTypeInt(signed=False, label="UInt32")), ("wki402_max_wrk_cache", SimTypeInt(signed=False, label="UInt32")), ("wki402_sess_timeout", SimTypeInt(signed=False, label="UInt32")), ("wki402_siz_error", SimTypeInt(signed=False, label="UInt32")), ("wki402_num_alerts", SimTypeInt(signed=False, label="UInt32")), ("wki402_num_services", SimTypeInt(signed=False, label="UInt32")), ("wki402_errlog_sz", SimTypeInt(signed=False, label="UInt32")), ("wki402_print_buf_time", SimTypeInt(signed=False, label="UInt32")), ("wki402_num_char_buf", SimTypeInt(signed=False, label="UInt32")), ("wki402_siz_char_buf", SimTypeInt(signed=False, label="UInt32")), ("wki402_wrk_heuristics", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wki402_mailslots", SimTypeInt(signed=False, label="UInt32")), ("wki402_num_dgram_buf", SimTypeInt(signed=False, label="UInt32")), ("wki402_max_threads", SimTypeInt(signed=False, label="UInt32")),)), name="WKSTA_INFO_402", pack=False, align=None),
    "WKSTA_INFO_502": SimStruct(OrderedDict((("wki502_char_wait", SimTypeInt(signed=False, label="UInt32")), ("wki502_collection_time", SimTypeInt(signed=False, label="UInt32")), ("wki502_maximum_collection_count", SimTypeInt(signed=False, label="UInt32")), ("wki502_keep_conn", SimTypeInt(signed=False, label="UInt32")), ("wki502_max_cmds", SimTypeInt(signed=False, label="UInt32")), ("wki502_sess_timeout", SimTypeInt(signed=False, label="UInt32")), ("wki502_siz_char_buf", SimTypeInt(signed=False, label="UInt32")), ("wki502_max_threads", SimTypeInt(signed=False, label="UInt32")), ("wki502_lock_quota", SimTypeInt(signed=False, label="UInt32")), ("wki502_lock_increment", SimTypeInt(signed=False, label="UInt32")), ("wki502_lock_maximum", SimTypeInt(signed=False, label="UInt32")), ("wki502_pipe_increment", SimTypeInt(signed=False, label="UInt32")), ("wki502_pipe_maximum", SimTypeInt(signed=False, label="UInt32")), ("wki502_cache_file_timeout", SimTypeInt(signed=False, label="UInt32")), ("wki502_dormant_file_limit", SimTypeInt(signed=False, label="UInt32")), ("wki502_read_ahead_throughput", SimTypeInt(signed=False, label="UInt32")), ("wki502_num_mailslot_buffers", SimTypeInt(signed=False, label="UInt32")), ("wki502_num_srv_announce_buffers", SimTypeInt(signed=False, label="UInt32")), ("wki502_max_illegal_datagram_events", SimTypeInt(signed=False, label="UInt32")), ("wki502_illegal_datagram_event_reset_frequency", SimTypeInt(signed=False, label="UInt32")), ("wki502_log_election_packets", SimTypeInt(signed=True, label="Int32")), ("wki502_use_opportunistic_locking", SimTypeInt(signed=True, label="Int32")), ("wki502_use_unlock_behind", SimTypeInt(signed=True, label="Int32")), ("wki502_use_close_behind", SimTypeInt(signed=True, label="Int32")), ("wki502_buf_named_pipes", SimTypeInt(signed=True, label="Int32")), ("wki502_use_lock_read_unlock", SimTypeInt(signed=True, label="Int32")), ("wki502_utilize_nt_caching", SimTypeInt(signed=True, label="Int32")), ("wki502_use_raw_read", SimTypeInt(signed=True, label="Int32")), ("wki502_use_raw_write", SimTypeInt(signed=True, label="Int32")), ("wki502_use_write_raw_data", SimTypeInt(signed=True, label="Int32")), ("wki502_use_encryption", SimTypeInt(signed=True, label="Int32")), ("wki502_buf_files_deny_write", SimTypeInt(signed=True, label="Int32")), ("wki502_buf_read_only_files", SimTypeInt(signed=True, label="Int32")), ("wki502_force_core_create_mode", SimTypeInt(signed=True, label="Int32")), ("wki502_use_512_byte_max_transfer", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_INFO_502", pack=False, align=None),
    "WKSTA_TRANSPORT_INFO_0": SimStruct(OrderedDict((("wkti0_quality_of_service", SimTypeInt(signed=False, label="UInt32")), ("wkti0_number_of_vcs", SimTypeInt(signed=False, label="UInt32")), ("wkti0_transport_name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wkti0_transport_address", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wkti0_wan_ish", SimTypeInt(signed=True, label="Int32")),)), name="WKSTA_TRANSPORT_INFO_0", pack=False, align=None),
    "WKSTA_USER_INFO_0": SimStruct(OrderedDict((("wkui0_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WKSTA_USER_INFO_0", pack=False, align=None),
    "WKSTA_USER_INFO_1": SimStruct(OrderedDict((("wkui1_username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wkui1_logon_domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wkui1_oth_domains", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wkui1_logon_server", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WKSTA_USER_INFO_1", pack=False, align=None),
    "WKSTA_USER_INFO_1101": SimStruct(OrderedDict((("wkui1101_oth_domains", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WKSTA_USER_INFO_1101", pack=False, align=None),
    "WLAN_ADHOC_NETWORK_STATE": SimTypeInt(signed=False, label="WLAN_ADHOC_NETWORK_STATE"),
    "WLAN_ASSOCIATION_ATTRIBUTES": SimStruct(OrderedDict((("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dot11Bssid", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("uDot11PhyIndex", SimTypeInt(signed=False, label="UInt32")), ("wlanSignalQuality", SimTypeInt(signed=False, label="UInt32")), ("ulRxRate", SimTypeInt(signed=False, label="UInt32")), ("ulTxRate", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_ASSOCIATION_ATTRIBUTES", pack=False, align=None),
    "WLAN_AUTH_CIPHER_PAIR_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("pAuthCipherPairList", SimTypePointer(SimTypeRef("DOT11_AUTH_CIPHER_PAIR", SimStruct), offset=0)),)), name="WLAN_AUTH_CIPHER_PAIR_LIST", pack=False, align=None),
    "WLAN_AUTOCONF_OPCODE": SimTypeInt(signed=False, label="WLAN_AUTOCONF_OPCODE"),
    "WLAN_AVAILABLE_NETWORK": SimStruct(OrderedDict((("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("uNumberOfBssids", SimTypeInt(signed=False, label="UInt32")), ("bNetworkConnectable", SimTypeInt(signed=True, label="Int32")), ("wlanNotConnectableReason", SimTypeInt(signed=False, label="UInt32")), ("uNumberOfPhyTypes", SimTypeInt(signed=False, label="UInt32")), ("dot11PhyTypes", SimTypeArray(SimTypeInt(signed=False, label="DOT11_PHY_TYPE"), 8)), ("bMorePhyTypes", SimTypeInt(signed=True, label="Int32")), ("wlanSignalQuality", SimTypeInt(signed=False, label="UInt32")), ("bSecurityEnabled", SimTypeInt(signed=True, label="Int32")), ("dot11DefaultAuthAlgorithm", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("dot11DefaultCipherAlgorithm", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_AVAILABLE_NETWORK", pack=False, align=None),
    "WLAN_AVAILABLE_NETWORK_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("Network", SimTypePointer(SimTypeRef("WLAN_AVAILABLE_NETWORK", SimStruct), offset=0)),)), name="WLAN_AVAILABLE_NETWORK_LIST", pack=False, align=None),
    "WLAN_AVAILABLE_NETWORK_LIST_V2": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("Network", SimTypePointer(SimTypeRef("WLAN_AVAILABLE_NETWORK_V2", SimStruct), offset=0)),)), name="WLAN_AVAILABLE_NETWORK_LIST_V2", pack=False, align=None),
    "WLAN_AVAILABLE_NETWORK_V2": SimStruct(OrderedDict((("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("uNumberOfBssids", SimTypeInt(signed=False, label="UInt32")), ("bNetworkConnectable", SimTypeInt(signed=True, label="Int32")), ("wlanNotConnectableReason", SimTypeInt(signed=False, label="UInt32")), ("uNumberOfPhyTypes", SimTypeInt(signed=False, label="UInt32")), ("dot11PhyTypes", SimTypeArray(SimTypeInt(signed=False, label="DOT11_PHY_TYPE"), 8)), ("bMorePhyTypes", SimTypeInt(signed=True, label="Int32")), ("wlanSignalQuality", SimTypeInt(signed=False, label="UInt32")), ("bSecurityEnabled", SimTypeInt(signed=True, label="Int32")), ("dot11DefaultAuthAlgorithm", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("dot11DefaultCipherAlgorithm", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("AccessNetworkOptions", SimTypeRef("DOT11_ACCESSNETWORKOPTIONS", SimStruct)), ("dot11HESSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("VenueInfo", SimTypeRef("DOT11_VENUEINFO", SimStruct)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_AVAILABLE_NETWORK_V2", pack=False, align=None),
    "WLAN_BSS_ENTRY": SimStruct(OrderedDict((("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("uPhyId", SimTypeInt(signed=False, label="UInt32")), ("dot11Bssid", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dot11BssPhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("lRssi", SimTypeInt(signed=True, label="Int32")), ("uLinkQuality", SimTypeInt(signed=False, label="UInt32")), ("bInRegDomain", SimTypeChar(label="Byte")), ("usBeaconPeriod", SimTypeShort(signed=False, label="UInt16")), ("ullTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("ullHostTimestamp", SimTypeLongLong(signed=False, label="UInt64")), ("usCapabilityInformation", SimTypeShort(signed=False, label="UInt16")), ("ulChCenterFrequency", SimTypeInt(signed=False, label="UInt32")), ("wlanRateSet", SimTypeRef("WLAN_RATE_SET", SimStruct)), ("ulIeOffset", SimTypeInt(signed=False, label="UInt32")), ("ulIeSize", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_BSS_ENTRY", pack=False, align=None),
    "WLAN_BSS_LIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("wlanBssEntries", SimTypePointer(SimTypeRef("WLAN_BSS_ENTRY", SimStruct), offset=0)),)), name="WLAN_BSS_LIST", pack=False, align=None),
    "WLAN_CONNECTION_ATTRIBUTES": SimStruct(OrderedDict((("isState", SimTypeInt(signed=False, label="WLAN_INTERFACE_STATE")), ("wlanConnectionMode", SimTypeInt(signed=False, label="WLAN_CONNECTION_MODE")), ("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("wlanAssociationAttributes", SimTypeRef("WLAN_ASSOCIATION_ATTRIBUTES", SimStruct)), ("wlanSecurityAttributes", SimTypeRef("WLAN_SECURITY_ATTRIBUTES", SimStruct)),)), name="WLAN_CONNECTION_ATTRIBUTES", pack=False, align=None),
    "WLAN_CONNECTION_MODE": SimTypeInt(signed=False, label="WLAN_CONNECTION_MODE"),
    "WLAN_CONNECTION_NOTIFICATION_DATA": SimStruct(OrderedDict((("wlanConnectionMode", SimTypeInt(signed=False, label="WLAN_CONNECTION_MODE")), ("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("bSecurityEnabled", SimTypeInt(signed=True, label="Int32")), ("wlanReasonCode", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="WLAN_CONNECTION_NOTIFICATION_FLAGS")), ("strProfileXml", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLAN_CONNECTION_NOTIFICATION_DATA", pack=False, align=None),
    "WLAN_CONNECTION_NOTIFICATION_FLAGS": SimTypeInt(signed=False, label="WLAN_CONNECTION_NOTIFICATION_FLAGS"),
    "WLAN_CONNECTION_PARAMETERS": SimStruct(OrderedDict((("wlanConnectionMode", SimTypeInt(signed=False, label="WLAN_CONNECTION_MODE")), ("strProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDot11Ssid", SimTypePointer(SimTypeRef("DOT11_SSID", SimStruct), offset=0)), ("pDesiredBssidList", SimTypePointer(SimTypeRef("DOT11_BSSID_LIST", SimStruct), offset=0)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_CONNECTION_PARAMETERS", pack=False, align=None),
    "WLAN_CONNECTION_PARAMETERS_V2": SimStruct(OrderedDict((("wlanConnectionMode", SimTypeInt(signed=False, label="WLAN_CONNECTION_MODE")), ("strProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDot11Ssid", SimTypePointer(SimTypeRef("DOT11_SSID", SimStruct), offset=0)), ("pDot11Hessid", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDesiredBssidList", SimTypePointer(SimTypeRef("DOT11_BSSID_LIST", SimStruct), offset=0)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pDot11AccessNetworkOptions", SimTypePointer(SimTypeRef("DOT11_ACCESSNETWORKOPTIONS", SimStruct), offset=0)),)), name="WLAN_CONNECTION_PARAMETERS_V2", pack=False, align=None),
    "WLAN_COUNTRY_OR_REGION_STRING_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("pCountryOrRegionStringList", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="WLAN_COUNTRY_OR_REGION_STRING_LIST", pack=False, align=None),
    "WLAN_DEVICE_SERVICE_GUID_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("DeviceService", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="WLAN_DEVICE_SERVICE_GUID_LIST", pack=False, align=None),
    "WLAN_DEVICE_SERVICE_NOTIFICATION_DATA": SimStruct(OrderedDict((("DeviceService", SimTypeBottom(label="Guid")), ("dwOpCode", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("DataBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WLAN_DEVICE_SERVICE_NOTIFICATION_DATA", pack=False, align=None),
    "WLAN_FILTER_LIST_TYPE": SimTypeInt(signed=False, label="WLAN_FILTER_LIST_TYPE"),
    "WLAN_HOSTED_NETWORK_CONNECTION_SETTINGS": SimStruct(OrderedDict((("hostedNetworkSSID", SimTypeRef("DOT11_SSID", SimStruct)), ("dwMaxNumberOfPeers", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_HOSTED_NETWORK_CONNECTION_SETTINGS", pack=False, align=None),
    "WLAN_HOSTED_NETWORK_DATA_PEER_STATE_CHANGE": SimStruct(OrderedDict((("OldState", SimTypeRef("WLAN_HOSTED_NETWORK_PEER_STATE", SimStruct)), ("NewState", SimTypeRef("WLAN_HOSTED_NETWORK_PEER_STATE", SimStruct)), ("PeerStateChangeReason", SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_REASON")),)), name="WLAN_HOSTED_NETWORK_DATA_PEER_STATE_CHANGE", pack=False, align=None),
    "WLAN_HOSTED_NETWORK_NOTIFICATION_CODE": SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_NOTIFICATION_CODE"),
    "WLAN_HOSTED_NETWORK_OPCODE": SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_OPCODE"),
    "WLAN_HOSTED_NETWORK_PEER_AUTH_STATE": SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_PEER_AUTH_STATE"),
    "WLAN_HOSTED_NETWORK_PEER_STATE": SimStruct(OrderedDict((("PeerMacAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("PeerAuthState", SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_PEER_AUTH_STATE")),)), name="WLAN_HOSTED_NETWORK_PEER_STATE", pack=False, align=None),
    "WLAN_HOSTED_NETWORK_RADIO_STATE": SimStruct(OrderedDict((("dot11SoftwareRadioState", SimTypeInt(signed=False, label="DOT11_RADIO_STATE")), ("dot11HardwareRadioState", SimTypeInt(signed=False, label="DOT11_RADIO_STATE")),)), name="WLAN_HOSTED_NETWORK_RADIO_STATE", pack=False, align=None),
    "WLAN_HOSTED_NETWORK_REASON": SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_REASON"),
    "WLAN_HOSTED_NETWORK_SECURITY_SETTINGS": SimStruct(OrderedDict((("dot11AuthAlgo", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("dot11CipherAlgo", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")),)), name="WLAN_HOSTED_NETWORK_SECURITY_SETTINGS", pack=False, align=None),
    "WLAN_HOSTED_NETWORK_STATE": SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_STATE"),
    "WLAN_HOSTED_NETWORK_STATE_CHANGE": SimStruct(OrderedDict((("OldState", SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_STATE")), ("NewState", SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_STATE")), ("StateChangeReason", SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_REASON")),)), name="WLAN_HOSTED_NETWORK_STATE_CHANGE", pack=False, align=None),
    "WLAN_HOSTED_NETWORK_STATUS": SimStruct(OrderedDict((("HostedNetworkState", SimTypeInt(signed=False, label="WLAN_HOSTED_NETWORK_STATE")), ("IPDeviceID", SimTypeBottom(label="Guid")), ("wlanHostedNetworkBSSID", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("dot11PhyType", SimTypeInt(signed=False, label="DOT11_PHY_TYPE")), ("ulChannelFrequency", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfPeers", SimTypeInt(signed=False, label="UInt32")), ("PeerList", SimTypePointer(SimTypeRef("WLAN_HOSTED_NETWORK_PEER_STATE", SimStruct), offset=0)),)), name="WLAN_HOSTED_NETWORK_STATUS", pack=False, align=None),
    "WLAN_IHV_CONTROL_TYPE": SimTypeInt(signed=False, label="WLAN_IHV_CONTROL_TYPE"),
    "WLAN_INTERFACE_CAPABILITY": SimStruct(OrderedDict((("interfaceType", SimTypeInt(signed=False, label="WLAN_INTERFACE_TYPE")), ("bDot11DSupported", SimTypeInt(signed=True, label="Int32")), ("dwMaxDesiredSsidListSize", SimTypeInt(signed=False, label="UInt32")), ("dwMaxDesiredBssidListSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfSupportedPhys", SimTypeInt(signed=False, label="UInt32")), ("dot11PhyTypes", SimTypeArray(SimTypeInt(signed=False, label="DOT11_PHY_TYPE"), 64)),)), name="WLAN_INTERFACE_CAPABILITY", pack=False, align=None),
    "WLAN_INTERFACE_INFO": SimStruct(OrderedDict((("InterfaceGuid", SimTypeBottom(label="Guid")), ("strInterfaceDescription", SimTypeArray(SimTypeChar(label="Char"), 256)), ("isState", SimTypeInt(signed=False, label="WLAN_INTERFACE_STATE")),)), name="WLAN_INTERFACE_INFO", pack=False, align=None),
    "WLAN_INTERFACE_INFO_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("InterfaceInfo", SimTypePointer(SimTypeRef("WLAN_INTERFACE_INFO", SimStruct), offset=0)),)), name="WLAN_INTERFACE_INFO_LIST", pack=False, align=None),
    "WLAN_INTERFACE_STATE": SimTypeInt(signed=False, label="WLAN_INTERFACE_STATE"),
    "WLAN_INTERFACE_TYPE": SimTypeInt(signed=False, label="WLAN_INTERFACE_TYPE"),
    "WLAN_INTF_OPCODE": SimTypeInt(signed=False, label="WLAN_INTF_OPCODE"),
    "WLAN_MAC_FRAME_STATISTICS": SimStruct(OrderedDict((("ullTransmittedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullWEPExcludedCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPLocalMICFailures", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPReplays", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPICVErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullCCMPReplays", SimTypeLongLong(signed=False, label="UInt64")), ("ullCCMPDecryptErrors", SimTypeLongLong(signed=False, label="UInt64")), ("ullWEPUndecryptableCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullWEPICVErrorCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullDecryptSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullDecryptFailureCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="WLAN_MAC_FRAME_STATISTICS", pack=False, align=None),
    "WLAN_MSM_NOTIFICATION_DATA": SimStruct(OrderedDict((("wlanConnectionMode", SimTypeInt(signed=False, label="WLAN_CONNECTION_MODE")), ("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dot11Ssid", SimTypeRef("DOT11_SSID", SimStruct)), ("dot11BssType", SimTypeInt(signed=False, label="DOT11_BSS_TYPE")), ("dot11MacAddr", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("bSecurityEnabled", SimTypeInt(signed=True, label="Int32")), ("bFirstPeer", SimTypeInt(signed=True, label="Int32")), ("bLastPeer", SimTypeInt(signed=True, label="Int32")), ("wlanReasonCode", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_MSM_NOTIFICATION_DATA", pack=False, align=None),
    "WLAN_NOTIFICATION_ACM": SimTypeInt(signed=False, label="WLAN_NOTIFICATION_ACM"),
    "WLAN_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("L2_NOTIFICATION_DATA", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0", "param1"]), offset=0),
    "WLAN_NOTIFICATION_MSM": SimTypeInt(signed=False, label="WLAN_NOTIFICATION_MSM"),
    "WLAN_NOTIFICATION_SECURITY": SimTypeInt(signed=False, label="WLAN_NOTIFICATION_SECURITY"),
    "WLAN_NOTIFICATION_SOURCES": SimTypeInt(signed=False, label="WLAN_NOTIFICATION_SOURCES"),
    "WLAN_OPCODE_VALUE_TYPE": SimTypeInt(signed=False, label="WLAN_OPCODE_VALUE_TYPE"),
    "WLAN_OPERATIONAL_STATE": SimTypeInt(signed=False, label="WLAN_OPERATIONAL_STATE"),
    "WLAN_PHY_FRAME_STATISTICS": SimStruct(OrderedDict((("ullTransmittedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMulticastTransmittedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullFailedCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRetryCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMultipleRetryCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMaxTXLifetimeExceededCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullTransmittedFragmentCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRTSSuccessCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullRTSFailureCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullACKFailureCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMulticastReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullPromiscuousReceivedFrameCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullMaxRXLifetimeExceededCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullFrameDuplicateCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullReceivedFragmentCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullPromiscuousReceivedFragmentCount", SimTypeLongLong(signed=False, label="UInt64")), ("ullFCSErrorCount", SimTypeLongLong(signed=False, label="UInt64")),)), name="WLAN_PHY_FRAME_STATISTICS", pack=False, align=None),
    "WLAN_PHY_RADIO_STATE": SimStruct(OrderedDict((("dwPhyIndex", SimTypeInt(signed=False, label="UInt32")), ("dot11SoftwareRadioState", SimTypeInt(signed=False, label="DOT11_RADIO_STATE")), ("dot11HardwareRadioState", SimTypeInt(signed=False, label="DOT11_RADIO_STATE")),)), name="WLAN_PHY_RADIO_STATE", pack=False, align=None),
    "WLAN_POWER_SETTING": SimTypeInt(signed=False, label="WLAN_POWER_SETTING"),
    "WLAN_PROFILE_INFO": SimStruct(OrderedDict((("strProfileName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WLAN_PROFILE_INFO", pack=False, align=None),
    "WLAN_PROFILE_INFO_LIST": SimStruct(OrderedDict((("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("dwIndex", SimTypeInt(signed=False, label="UInt32")), ("ProfileInfo", SimTypePointer(SimTypeRef("WLAN_PROFILE_INFO", SimStruct), offset=0)),)), name="WLAN_PROFILE_INFO_LIST", pack=False, align=None),
    "WLAN_RADIO_STATE": SimStruct(OrderedDict((("dwNumberOfPhys", SimTypeInt(signed=False, label="UInt32")), ("PhyRadioState", SimTypeArray(SimTypeRef("WLAN_PHY_RADIO_STATE", SimStruct), 64)),)), name="WLAN_RADIO_STATE", pack=False, align=None),
    "WLAN_RATE_SET": SimStruct(OrderedDict((("uRateSetLength", SimTypeInt(signed=False, label="UInt32")), ("usRateSet", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 126)),)), name="WLAN_RATE_SET", pack=False, align=None),
    "WLAN_RAW_DATA": SimStruct(OrderedDict((("dwDataSize", SimTypeInt(signed=False, label="UInt32")), ("DataBlob", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WLAN_RAW_DATA", pack=False, align=None),
    "WLAN_RAW_DATA_LIST": SimStruct(OrderedDict((("dwTotalSize", SimTypeInt(signed=False, label="UInt32")), ("dwNumberOfItems", SimTypeInt(signed=False, label="UInt32")), ("DataList", SimTypePointer(SimStruct(OrderedDict((("dwDataOffset", SimTypeInt(signed=False, label="UInt32")), ("dwDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), offset=0)),)), name="WLAN_RAW_DATA_LIST", pack=False, align=None),
    "WLAN_SECURABLE_OBJECT": SimTypeInt(signed=False, label="WLAN_SECURABLE_OBJECT"),
    "WLAN_SECURITY_ATTRIBUTES": SimStruct(OrderedDict((("bSecurityEnabled", SimTypeInt(signed=True, label="Int32")), ("bOneXEnabled", SimTypeInt(signed=True, label="Int32")), ("dot11AuthAlgorithm", SimTypeInt(signed=False, label="DOT11_AUTH_ALGORITHM")), ("dot11CipherAlgorithm", SimTypeInt(signed=False, label="DOT11_CIPHER_ALGORITHM")),)), name="WLAN_SECURITY_ATTRIBUTES", pack=False, align=None),
    "WLAN_SET_EAPHOST_FLAGS": SimTypeInt(signed=False, label="WLAN_SET_EAPHOST_FLAGS"),
    "WLAN_STATISTICS": SimStruct(OrderedDict((("ullFourWayHandshakeFailures", SimTypeLongLong(signed=False, label="UInt64")), ("ullTKIPCounterMeasuresInvoked", SimTypeLongLong(signed=False, label="UInt64")), ("ullReserved", SimTypeLongLong(signed=False, label="UInt64")), ("MacUcastCounters", SimTypeRef("WLAN_MAC_FRAME_STATISTICS", SimStruct)), ("MacMcastCounters", SimTypeRef("WLAN_MAC_FRAME_STATISTICS", SimStruct)), ("dwNumberOfPhys", SimTypeInt(signed=False, label="UInt32")), ("PhyCounters", SimTypePointer(SimTypeRef("WLAN_PHY_FRAME_STATISTICS", SimStruct), offset=0)),)), name="WLAN_STATISTICS", pack=False, align=None),
    "WLDP_DEVICE_SECURITY_INFORMATION": SimStruct(OrderedDict((("UnlockIdSize", SimTypeInt(signed=False, label="UInt32")), ("UnlockId", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ManufacturerIDLength", SimTypeInt(signed=False, label="UInt32")), ("ManufacturerID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLDP_DEVICE_SECURITY_INFORMATION", pack=False, align=None),
    "WLDP_EXECUTION_EVALUATION_OPTIONS": SimTypeInt(signed=False, label="WLDP_EXECUTION_EVALUATION_OPTIONS"),
    "WLDP_EXECUTION_POLICY": SimTypeInt(signed=False, label="WLDP_EXECUTION_POLICY"),
    "WLDP_HOST": SimTypeInt(signed=False, label="WLDP_HOST"),
    "WLDP_HOST_ID": SimTypeInt(signed=False, label="WLDP_HOST_ID"),
    "WLDP_HOST_INFORMATION": SimStruct(OrderedDict((("dwRevision", SimTypeInt(signed=False, label="UInt32")), ("dwHostId", SimTypeInt(signed=False, label="WLDP_HOST_ID")), ("szSource", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hSource", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WLDP_HOST_INFORMATION", pack=False, align=None),
    "WLDP_KEY": SimTypeInt(signed=False, label="WLDP_KEY"),
    "WLDP_POLICY_SETTING": SimTypeInt(signed=False, label="WLDP_POLICY_SETTING"),
    "WLDP_WINDOWS_LOCKDOWN_MODE": SimTypeInt(signed=False, label="WLDP_WINDOWS_LOCKDOWN_MODE"),
    "WLDP_WINDOWS_LOCKDOWN_RESTRICTION": SimTypeInt(signed=False, label="WLDP_WINDOWS_LOCKDOWN_RESTRICTION"),
    "WLOOP": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("ulType", SimTypeInt(signed=False, label="UInt32")), ("ulStart", SimTypeInt(signed=False, label="UInt32")), ("ulLength", SimTypeInt(signed=False, label="UInt32")),)), name="WLOOP", pack=False, align=None),
    "WLX_CLIENT_CREDENTIALS_INFO_V1_0": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fPromptForPassword", SimTypeInt(signed=True, label="Int32")),)), name="WLX_CLIENT_CREDENTIALS_INFO_V1_0", pack=False, align=None),
    "WLX_CLIENT_CREDENTIALS_INFO_V2_0": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fPromptForPassword", SimTypeInt(signed=True, label="Int32")), ("fDisconnectOnLogonFailure", SimTypeInt(signed=True, label="Int32")),)), name="WLX_CLIENT_CREDENTIALS_INFO_V2_0", pack=False, align=None),
    "WLX_CONSOLESWITCH_CREDENTIALS_INFO_V1_0": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("UserToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("LogonId", SimTypeRef("LUID", SimStruct)), ("Quotas", SimTypeRef("QUOTA_LIMITS", SimStruct)), ("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("SmartCardLogon", SimTypeInt(signed=True, label="Int32")), ("ProfileLength", SimTypeInt(signed=False, label="UInt32")), ("MessageType", SimTypeInt(signed=False, label="UInt32")), ("LogonCount", SimTypeShort(signed=False, label="UInt16")), ("BadPasswordCount", SimTypeShort(signed=False, label="UInt16")), ("ProfileLogonTime", SimTypeLongLong(signed=True, label="Int64")), ("LogoffTime", SimTypeLongLong(signed=True, label="Int64")), ("KickOffTime", SimTypeLongLong(signed=True, label="Int64")), ("PasswordLastSet", SimTypeLongLong(signed=True, label="Int64")), ("PasswordCanChange", SimTypeLongLong(signed=True, label="Int64")), ("PasswordMustChange", SimTypeLongLong(signed=True, label="Int64")), ("LogonScript", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HomeDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FullName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProfilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HomeDirectoryDrive", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("LogonServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("UserFlags", SimTypeInt(signed=False, label="UInt32")), ("PrivateDataLen", SimTypeInt(signed=False, label="UInt32")), ("PrivateData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WLX_CONSOLESWITCH_CREDENTIALS_INFO_V1_0", pack=False, align=None),
    "WLX_DESKTOP": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("hDesktop", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pszDesktopName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLX_DESKTOP", pack=False, align=None),
    "WLX_DISPATCH_VERSION_1_0": SimStruct(OrderedDict((("WlxUseCtrlAltDel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxSetContextPointer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx", "pWlxContext"]), offset=0)), ("WlxSasNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWlx", "dwSasType"]), offset=0)), ("WlxSetTimeout", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Timeout"]), offset=0)), ("WlxAssignShellProtection", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "hProcess", "hThread"]), offset=0)), ("WlxMessageBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hwndOwner", "lpszText", "lpszTitle", "fuStyle"]), offset=0)), ("WlxDialogBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxDialogBoxIndirect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxIndirectParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxSwitchDesktopToUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxSwitchDesktopToWinlogon", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxChangePasswordNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo"]), offset=0)),)), name="WLX_DISPATCH_VERSION_1_0", pack=False, align=None),
    "WLX_DISPATCH_VERSION_1_1": SimStruct(OrderedDict((("WlxUseCtrlAltDel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxSetContextPointer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx", "pWlxContext"]), offset=0)), ("WlxSasNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWlx", "dwSasType"]), offset=0)), ("WlxSetTimeout", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Timeout"]), offset=0)), ("WlxAssignShellProtection", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "hProcess", "hThread"]), offset=0)), ("WlxMessageBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hwndOwner", "lpszText", "lpszTitle", "fuStyle"]), offset=0)), ("WlxDialogBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxDialogBoxIndirect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxIndirectParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxSwitchDesktopToUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxSwitchDesktopToWinlogon", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxChangePasswordNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo"]), offset=0)), ("WlxGetSourceDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "ppDesktop"]), offset=0)), ("WlxSetReturnDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop"]), offset=0)), ("WlxCreateUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "Flags", "pszDesktopName", "ppDesktop"]), offset=0)), ("WlxChangePasswordNotifyEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo", "ProviderName", "Reserved"]), offset=0)),)), name="WLX_DISPATCH_VERSION_1_1", pack=False, align=None),
    "WLX_DISPATCH_VERSION_1_2": SimStruct(OrderedDict((("WlxUseCtrlAltDel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxSetContextPointer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx", "pWlxContext"]), offset=0)), ("WlxSasNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWlx", "dwSasType"]), offset=0)), ("WlxSetTimeout", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Timeout"]), offset=0)), ("WlxAssignShellProtection", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "hProcess", "hThread"]), offset=0)), ("WlxMessageBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hwndOwner", "lpszText", "lpszTitle", "fuStyle"]), offset=0)), ("WlxDialogBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxDialogBoxIndirect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxIndirectParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxSwitchDesktopToUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxSwitchDesktopToWinlogon", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxChangePasswordNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo"]), offset=0)), ("WlxGetSourceDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "ppDesktop"]), offset=0)), ("WlxSetReturnDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop"]), offset=0)), ("WlxCreateUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "Flags", "pszDesktopName", "ppDesktop"]), offset=0)), ("WlxChangePasswordNotifyEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo", "ProviderName", "Reserved"]), offset=0)), ("WlxCloseUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop", "hToken"]), offset=0)),)), name="WLX_DISPATCH_VERSION_1_2", pack=False, align=None),
    "WLX_DISPATCH_VERSION_1_3": SimStruct(OrderedDict((("WlxUseCtrlAltDel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxSetContextPointer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx", "pWlxContext"]), offset=0)), ("WlxSasNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWlx", "dwSasType"]), offset=0)), ("WlxSetTimeout", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Timeout"]), offset=0)), ("WlxAssignShellProtection", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "hProcess", "hThread"]), offset=0)), ("WlxMessageBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hwndOwner", "lpszText", "lpszTitle", "fuStyle"]), offset=0)), ("WlxDialogBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxDialogBoxIndirect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxIndirectParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxSwitchDesktopToUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxSwitchDesktopToWinlogon", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxChangePasswordNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo"]), offset=0)), ("WlxGetSourceDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "ppDesktop"]), offset=0)), ("WlxSetReturnDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop"]), offset=0)), ("WlxCreateUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "Flags", "pszDesktopName", "ppDesktop"]), offset=0)), ("WlxChangePasswordNotifyEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo", "ProviderName", "Reserved"]), offset=0)), ("WlxCloseUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop", "hToken"]), offset=0)), ("WlxSetOption", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Option", "Value", "OldValue"]), offset=0)), ("WlxGetOption", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Option", "Value"]), offset=0)), ("WlxWin31Migrate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxQueryClientCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0)), ("WlxQueryInetConnectorCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0)), ("WlxDisconnect", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("WlxQueryTerminalServicesData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_TERMINAL_SERVICES_DATA", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hWlx", "pTSData", "UserName", "Domain"]), offset=0)),)), name="WLX_DISPATCH_VERSION_1_3", pack=False, align=None),
    "WLX_DISPATCH_VERSION_1_4": SimStruct(OrderedDict((("WlxUseCtrlAltDel", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxSetContextPointer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx", "pWlxContext"]), offset=0)), ("WlxSasNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["hWlx", "dwSasType"]), offset=0)), ("WlxSetTimeout", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Timeout"]), offset=0)), ("WlxAssignShellProtection", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "hProcess", "hThread"]), offset=0)), ("WlxMessageBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hwndOwner", "lpszText", "lpszTitle", "fuStyle"]), offset=0)), ("WlxDialogBox", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "lpszTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxDialogBoxIndirect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc"]), offset=0)), ("WlxDialogBoxIndirectParam", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("DLGTEMPLATE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hInst", "hDialogTemplate", "hwndOwner", "dlgprc", "dwInitParam"]), offset=0)), ("WlxSwitchDesktopToUser", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxSwitchDesktopToWinlogon", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx"]), offset=0)), ("WlxChangePasswordNotify", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo"]), offset=0)), ("WlxGetSourceDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "ppDesktop"]), offset=0)), ("WlxSetReturnDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop"]), offset=0)), ("WlxCreateUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "hToken", "Flags", "pszDesktopName", "ppDesktop"]), offset=0)), ("WlxChangePasswordNotifyEx", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_MPR_NOTIFY_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pMprInfo", "dwChangeInfo", "ProviderName", "Reserved"]), offset=0)), ("WlxCloseUserDesktop", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_DESKTOP", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "pDesktop", "hToken"]), offset=0)), ("WlxSetOption", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Option", "Value", "OldValue"]), offset=0)), ("WlxGetOption", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWlx", "Option", "Value"]), offset=0)), ("WlxWin31Migrate", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hWlx"]), offset=0)), ("WlxQueryClientCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0)), ("WlxQueryInetConnectorCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0)), ("WlxDisconnect", SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0)), ("WlxQueryTerminalServicesData", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WLX_TERMINAL_SERVICES_DATA", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hWlx", "pTSData", "UserName", "Domain"]), offset=0)), ("WlxQueryConsoleSwitchCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CONSOLESWITCH_CREDENTIALS_INFO_V1_0", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pCred"]), offset=0)), ("WlxQueryTsLogonCredentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WLX_CLIENT_CREDENTIALS_INFO_V2_0", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCred"]), offset=0)),)), name="WLX_DISPATCH_VERSION_1_4", pack=False, align=None),
    "WLX_MPR_NOTIFY_INFO": SimStruct(OrderedDict((("pszUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszDomain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszOldPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLX_MPR_NOTIFY_INFO", pack=False, align=None),
    "WLX_NOTIFICATION_INFO": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("UserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Domain", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("WindowStation", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDesktop", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pStatusCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["bVerbose", "lpMessage"]), offset=0)),)), name="WLX_NOTIFICATION_INFO", pack=False, align=None),
    "WLX_PROFILE_V1_0": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("pszProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLX_PROFILE_V1_0", pack=False, align=None),
    "WLX_PROFILE_V2_0": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("pszProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszPolicy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszNetworkDefaultUserProfile", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszEnvironment", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLX_PROFILE_V2_0", pack=False, align=None),
    "WLX_SC_NOTIFICATION_INFO": SimStruct(OrderedDict((("pszCard", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszReader", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszContainer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pszCryptoProvider", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WLX_SC_NOTIFICATION_INFO", pack=False, align=None),
    "WLX_SHUTDOWN_TYPE": SimTypeInt(signed=False, label="WLX_SHUTDOWN_TYPE"),
    "WLX_TERMINAL_SERVICES_DATA": SimStruct(OrderedDict((("ProfilePath", SimTypeArray(SimTypeChar(label="Char"), 257)), ("HomeDir", SimTypeArray(SimTypeChar(label="Char"), 257)), ("HomeDirDrive", SimTypeArray(SimTypeChar(label="Char"), 4)),)), name="WLX_TERMINAL_SERVICES_DATA", pack=False, align=None),
    "WL_DISPLAY_PAGES": SimTypeInt(signed=False, label="WL_DISPLAY_PAGES"),
    "WMAUDIO2WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("dwSamplesPerBlock", SimTypeInt(signed=False, label="UInt32")), ("wEncodeOptions", SimTypeShort(signed=False, label="UInt16")), ("dwSuperBlockAlign", SimTypeInt(signed=False, label="UInt32")),)), name="WMAUDIO2WAVEFORMAT", pack=False, align=None),
    "WMAUDIO3WAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)), ("wValidBitsPerSample", SimTypeShort(signed=False, label="UInt16")), ("dwChannelMask", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("wEncodeOptions", SimTypeShort(signed=False, label="UInt16")), ("wReserved3", SimTypeShort(signed=False, label="UInt16")),)), name="WMAUDIO3WAVEFORMAT", pack=False, align=None),
    "WMDMDATETIME": SimStruct(OrderedDict((("wYear", SimTypeShort(signed=False, label="UInt16")), ("wMonth", SimTypeShort(signed=False, label="UInt16")), ("wDay", SimTypeShort(signed=False, label="UInt16")), ("wHour", SimTypeShort(signed=False, label="UInt16")), ("wMinute", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")),)), name="WMDMDATETIME", pack=False, align=None),
    "WMDMDetermineMaxPropStringLen": SimUnion({"sz001": SimTypeArray(SimTypeChar(label="Char"), 27), "sz002": SimTypeArray(SimTypeChar(label="Char"), 31), "sz003": SimTypeArray(SimTypeChar(label="Char"), 14), "sz004": SimTypeArray(SimTypeChar(label="Char"), 16), "sz005": SimTypeArray(SimTypeChar(label="Char"), 22), "sz006": SimTypeArray(SimTypeChar(label="Char"), 14), "sz007": SimTypeArray(SimTypeChar(label="Char"), 20), "sz008": SimTypeArray(SimTypeChar(label="Char"), 20), "sz009": SimTypeArray(SimTypeChar(label="Char"), 22), "sz010": SimTypeArray(SimTypeChar(label="Char"), 11), "sz011": SimTypeArray(SimTypeChar(label="Char"), 12), "sz012": SimTypeArray(SimTypeChar(label="Char"), 17), "sz013": SimTypeArray(SimTypeChar(label="Char"), 17), "sz014": SimTypeArray(SimTypeChar(label="Char"), 16), "sz015": SimTypeArray(SimTypeChar(label="Char"), 17), "sz016": SimTypeArray(SimTypeChar(label="Char"), 11), "sz017": SimTypeArray(SimTypeChar(label="Char"), 11), "sz018": SimTypeArray(SimTypeChar(label="Char"), 15), "sz019": SimTypeArray(SimTypeChar(label="Char"), 22), "sz020": SimTypeArray(SimTypeChar(label="Char"), 20), "sz021": SimTypeArray(SimTypeChar(label="Char"), 22), "sz022": SimTypeArray(SimTypeChar(label="Char"), 21), "sz023": SimTypeArray(SimTypeChar(label="Char"), 24), "sz024": SimTypeArray(SimTypeChar(label="Char"), 20), "sz025": SimTypeArray(SimTypeChar(label="Char"), 10), "sz026": SimTypeArray(SimTypeChar(label="Char"), 14), "sz027": SimTypeArray(SimTypeChar(label="Char"), 11), "sz028": SimTypeArray(SimTypeChar(label="Char"), 11), "sz029": SimTypeArray(SimTypeChar(label="Char"), 13), "sz030": SimTypeArray(SimTypeChar(label="Char"), 17), "sz031": SimTypeArray(SimTypeChar(label="Char"), 16), "sz032": SimTypeArray(SimTypeChar(label="Char"), 17), "sz033": SimTypeArray(SimTypeChar(label="Char"), 20), "sz034": SimTypeArray(SimTypeChar(label="Char"), 19), "sz035": SimTypeArray(SimTypeChar(label="Char"), 18), "sz036": SimTypeArray(SimTypeChar(label="Char"), 18), "sz037": SimTypeArray(SimTypeChar(label="Char"), 15), "sz041": SimTypeArray(SimTypeChar(label="Char"), 14), "sz043": SimTypeArray(SimTypeChar(label="Char"), 22), "sz044": SimTypeArray(SimTypeChar(label="Char"), 16), "sz045": SimTypeArray(SimTypeChar(label="Char"), 20), "sz046": SimTypeArray(SimTypeChar(label="Char"), 14), "sz047": SimTypeArray(SimTypeChar(label="Char"), 14), "sz048": SimTypeArray(SimTypeChar(label="Char"), 12), "sz049": SimTypeArray(SimTypeChar(label="Char"), 25), "sz050": SimTypeArray(SimTypeChar(label="Char"), 26), "sz051": SimTypeArray(SimTypeChar(label="Char"), 25), "sz052": SimTypeArray(SimTypeChar(label="Char"), 16), "sz053": SimTypeArray(SimTypeChar(label="Char"), 24), "sz054": SimTypeArray(SimTypeChar(label="Char"), 15), "sz055": SimTypeArray(SimTypeChar(label="Char"), 21), "sz056": SimTypeArray(SimTypeChar(label="Char"), 16), "sz057": SimTypeArray(SimTypeChar(label="Char"), 22), "sz058": SimTypeArray(SimTypeChar(label="Char"), 14), "sz059": SimTypeArray(SimTypeChar(label="Char"), 25), "sz060": SimTypeArray(SimTypeChar(label="Char"), 18), "sz061": SimTypeArray(SimTypeChar(label="Char"), 22), "sz062": SimTypeArray(SimTypeChar(label="Char"), 26), "sz063": SimTypeArray(SimTypeChar(label="Char"), 36), "sz064": SimTypeArray(SimTypeChar(label="Char"), 23), "sz065": SimTypeArray(SimTypeChar(label="Char"), 12), "sz066": SimTypeArray(SimTypeChar(label="Char"), 24), "sz067": SimTypeArray(SimTypeChar(label="Char"), 11), "sz068": SimTypeArray(SimTypeChar(label="Char"), 12), "sz069": SimTypeArray(SimTypeChar(label="Char"), 14), "sz070": SimTypeArray(SimTypeChar(label="Char"), 20), "sz071": SimTypeArray(SimTypeChar(label="Char"), 15), "sz072": SimTypeArray(SimTypeChar(label="Char"), 14), "sz073": SimTypeArray(SimTypeChar(label="Char"), 31), "sz074": SimTypeArray(SimTypeChar(label="Char"), 24), "sz075": SimTypeArray(SimTypeChar(label="Char"), 22), "sz076": SimTypeArray(SimTypeChar(label="Char"), 24), "sz077": SimTypeArray(SimTypeChar(label="Char"), 21), "sz078": SimTypeArray(SimTypeChar(label="Char"), 27), "sz079": SimTypeArray(SimTypeChar(label="Char"), 27), "sz080": SimTypeArray(SimTypeChar(label="Char"), 20), "sz081": SimTypeArray(SimTypeChar(label="Char"), 33), "sz082": SimTypeArray(SimTypeChar(label="Char"), 21), "sz083": SimTypeArray(SimTypeChar(label="Char"), 32), "sz084": SimTypeArray(SimTypeChar(label="Char"), 26), "sz085": SimTypeArray(SimTypeChar(label="Char"), 18), "sz086": SimTypeArray(SimTypeChar(label="Char"), 30)}, name="<anon>", label="None"),
    "WMDMID": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwVendorID", SimTypeInt(signed=False, label="UInt32")), ("pID", SimTypeArray(SimTypeChar(label="Byte"), 128)), ("SerialNumberLength", SimTypeInt(signed=False, label="UInt32")),)), name="WMDMID", pack=False, align=None),
    "WMDMMessage": SimTypeInt(signed=False, label="WMDMMessage"),
    "WMDMMetadataView": SimStruct(OrderedDict((("pwszViewName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("nDepth", SimTypeInt(signed=False, label="UInt32")), ("ppwszTags", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)),)), name="WMDMMetadataView", pack=False, align=None),
    "WMDMRIGHTS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwContentType", SimTypeInt(signed=False, label="UInt32")), ("fuFlags", SimTypeInt(signed=False, label="UInt32")), ("fuRights", SimTypeInt(signed=False, label="UInt32")), ("dwAppSec", SimTypeInt(signed=False, label="UInt32")), ("dwPlaybackCount", SimTypeInt(signed=False, label="UInt32")), ("ExpirationDate", SimTypeRef("WMDMDATETIME", SimStruct)),)), name="WMDMRIGHTS", pack=False, align=None),
    "WMDM_ENUM_PROP_VALID_VALUES_FORM": SimTypeInt(signed=False, label="WMDM_ENUM_PROP_VALID_VALUES_FORM"),
    "WMDM_FIND_SCOPE": SimTypeInt(signed=False, label="WMDM_FIND_SCOPE"),
    "WMDM_FORMATCODE": SimTypeInt(signed=False, label="WMDM_FORMATCODE"),
    "WMDM_FORMAT_CAPABILITY": SimStruct(OrderedDict((("nPropConfig", SimTypeInt(signed=False, label="UInt32")), ("pConfigs", SimTypePointer(SimTypeRef("WMDM_PROP_CONFIG", SimStruct), offset=0)),)), name="WMDM_FORMAT_CAPABILITY", pack=False, align=None),
    "WMDM_PROP_CONFIG": SimStruct(OrderedDict((("nPreference", SimTypeInt(signed=False, label="UInt32")), ("nPropDesc", SimTypeInt(signed=False, label="UInt32")), ("pPropDesc", SimTypePointer(SimTypeRef("WMDM_PROP_DESC", SimStruct), offset=0)),)), name="WMDM_PROP_CONFIG", pack=False, align=None),
    "WMDM_PROP_DESC": SimStruct(OrderedDict((("pwszPropName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ValidValuesForm", SimTypeInt(signed=False, label="WMDM_ENUM_PROP_VALID_VALUES_FORM")), ("ValidValues", SimUnion({"ValidValuesRange": SimTypeRef("WMDM_PROP_VALUES_RANGE", SimStruct), "EnumeratedValidValues": SimTypeRef("WMDM_PROP_VALUES_ENUM", SimStruct)}, name="<anon>", label="None")),)), name="WMDM_PROP_DESC", pack=False, align=None),
    "WMDM_PROP_VALUES_ENUM": SimStruct(OrderedDict((("cEnumValues", SimTypeInt(signed=False, label="UInt32")), ("pValues", SimTypePointer(SimTypeRef("PROPVARIANT", SimStruct), offset=0)),)), name="WMDM_PROP_VALUES_ENUM", pack=False, align=None),
    "WMDM_PROP_VALUES_RANGE": SimStruct(OrderedDict((("rangeMin", SimTypeRef("PROPVARIANT", SimStruct)), ("rangeMax", SimTypeRef("PROPVARIANT", SimStruct)), ("rangeStep", SimTypeRef("PROPVARIANT", SimStruct)),)), name="WMDM_PROP_VALUES_RANGE", pack=False, align=None),
    "WMDM_SESSION_TYPE": SimTypeInt(signed=False, label="WMDM_SESSION_TYPE"),
    "WMDM_STORAGE_ENUM_MODE": SimTypeInt(signed=False, label="WMDM_STORAGE_ENUM_MODE"),
    "WMDM_TAG_DATATYPE": SimTypeInt(signed=False, label="WMDM_TAG_DATATYPE"),
    "WMDRMProtectionInfo": SimStruct(OrderedDict((("wszKID", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 25)), ("qwCounter", SimTypeLongLong(signed=False, label="UInt64")), ("qwIndex", SimTypeLongLong(signed=False, label="UInt64")), ("bOffset", SimTypeChar(label="Byte")),)), name="WMDRMProtectionInfo", pack=False, align=None),
    "WMDRM_IMPORT_INIT_STRUCT": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("cbEncryptedSessionKeyMessage", SimTypeInt(signed=False, label="UInt32")), ("pbEncryptedSessionKeyMessage", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("cbEncryptedKeyMessage", SimTypeInt(signed=False, label="UInt32")), ("pbEncryptedKeyMessage", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WMDRM_IMPORT_INIT_STRUCT", pack=False, align=None),
    "WMFILECAPABILITIES": SimStruct(OrderedDict((("pwszMimeType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="WMFILECAPABILITIES", pack=False, align=None),
    "WMIDPREQUEST": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WMIDPREQUESTCODE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestCode", "RequestContext", "BufferSize", "Buffer"]), offset=0),
    "WMIDPREQUESTCODE": SimTypeInt(signed=False, label="WMIDPREQUESTCODE"),
    "WMIQ_ANALYSIS_TYPE": SimTypeInt(signed=False, label="WMIQ_ANALYSIS_TYPE"),
    "WMIQ_ASSOCQ_FLAGS": SimTypeInt(signed=False, label="WMIQ_ASSOCQ_FLAGS"),
    "WMIQ_LANGUAGE_FEATURES": SimTypeInt(signed=False, label="WMIQ_LANGUAGE_FEATURES"),
    "WMIQ_RPNF_FEATURE": SimTypeInt(signed=False, label="WMIQ_RPNF_FEATURE"),
    "WMIQ_RPN_TOKEN_FLAGS": SimTypeInt(signed=False, label="WMIQ_RPN_TOKEN_FLAGS"),
    "WMIREGGUIDW": SimStruct(OrderedDict((("Guid", SimTypeBottom(label="Guid")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"InstanceNameList": SimTypeInt(signed=False, label="UInt32"), "BaseNameOffset": SimTypeInt(signed=False, label="UInt32"), "Pdo": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "InstanceInfo": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None")),)), name="WMIREGGUIDW", pack=False, align=None),
    "WMIREGINFOW": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("NextWmiRegInfo", SimTypeInt(signed=False, label="UInt32")), ("RegistryPath", SimTypeInt(signed=False, label="UInt32")), ("MofResourceName", SimTypeInt(signed=False, label="UInt32")), ("GuidCount", SimTypeInt(signed=False, label="UInt32")), ("WmiRegGuid", SimTypePointer(SimTypeRef("WMIREGGUIDW", SimStruct), offset=0)),)), name="WMIREGINFOW", pack=False, align=None),
    "WMI_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "WMI_OBJ_TEXT": SimTypeInt(signed=False, label="WMI_OBJ_TEXT"),
    "WMMPEG2VIDEOINFO": SimStruct(OrderedDict((("hdr", SimTypeRef("WMVIDEOINFOHEADER2", SimStruct)), ("dwStartTimeCode", SimTypeInt(signed=False, label="UInt32")), ("cbSequenceHeader", SimTypeInt(signed=False, label="UInt32")), ("dwProfile", SimTypeInt(signed=False, label="UInt32")), ("dwLevel", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwSequenceHeader", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="WMMPEG2VIDEOINFO", pack=False, align=None),
    "WMPAccountType": SimTypeInt(signed=False, label="WMPAccountType"),
    "WMPBurnFormat": SimTypeInt(signed=False, label="WMPBurnFormat"),
    "WMPBurnState": SimTypeInt(signed=False, label="WMPBurnState"),
    "WMPCallbackNotification": SimTypeInt(signed=False, label="WMPCallbackNotification"),
    "WMPContextMenuInfo": SimStruct(OrderedDict((("dwID", SimTypeInt(signed=False, label="UInt32")), ("bstrMenuText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bstrHelpText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WMPContextMenuInfo", pack=False, align=None),
    "WMPDeviceStatus": SimTypeInt(signed=False, label="WMPDeviceStatus"),
    "WMPFolderScanState": SimTypeInt(signed=False, label="WMPFolderScanState"),
    "WMPLibraryType": SimTypeInt(signed=False, label="WMPLibraryType"),
    "WMPOpenState": SimTypeInt(signed=False, label="WMPOpenState"),
    "WMPPartnerNotification": SimTypeInt(signed=False, label="WMPPartnerNotification"),
    "WMPPlayState": SimTypeInt(signed=False, label="WMPPlayState"),
    "WMPPlaylistChangeEventType": SimTypeInt(signed=False, label="WMPPlaylistChangeEventType"),
    "WMPPlugin_Caps": SimTypeInt(signed=False, label="WMPPlugin_Caps"),
    "WMPRipState": SimTypeInt(signed=False, label="WMPRipState"),
    "WMPServices_StreamState": SimTypeInt(signed=False, label="WMPServices_StreamState"),
    "WMPStreamingType": SimTypeInt(signed=False, label="WMPStreamingType"),
    "WMPStringCollectionChangeEventType": SimTypeInt(signed=False, label="WMPStringCollectionChangeEventType"),
    "WMPSubscriptionDownloadState": SimTypeInt(signed=False, label="WMPSubscriptionDownloadState"),
    "WMPSubscriptionServiceEvent": SimTypeInt(signed=False, label="WMPSubscriptionServiceEvent"),
    "WMPSyncState": SimTypeInt(signed=False, label="WMPSyncState"),
    "WMPTaskType": SimTypeInt(signed=False, label="WMPTaskType"),
    "WMPTemplateSize": SimTypeInt(signed=False, label="WMPTemplateSize"),
    "WMPTransactionType": SimTypeInt(signed=False, label="WMPTransactionType"),
    "WMP_WMDM_METADATA_ROUND_TRIP_DEVICE2PC": SimStruct(OrderedDict((("dwCurrentTransactionID", SimTypeInt(signed=False, label="UInt32")), ("dwReturnedObjectCount", SimTypeInt(signed=False, label="UInt32")), ("dwUnretrievedObjectCount", SimTypeInt(signed=False, label="UInt32")), ("dwDeletedObjectStartingOffset", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("wsObjectPathnameList", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WMP_WMDM_METADATA_ROUND_TRIP_DEVICE2PC", pack=False, align=None),
    "WMP_WMDM_METADATA_ROUND_TRIP_PC2DEVICE": SimStruct(OrderedDict((("dwChangesSinceTransactionID", SimTypeInt(signed=False, label="UInt32")), ("dwResultSetStartingIndex", SimTypeInt(signed=False, label="UInt32")),)), name="WMP_WMDM_METADATA_ROUND_TRIP_PC2DEVICE", pack=False, align=None),
    "WMSCRIPTFORMAT": SimStruct(OrderedDict((("scriptType", SimTypeBottom(label="Guid")),)), name="WMSCRIPTFORMAT", pack=False, align=None),
    "WMT_ATTR_DATATYPE": SimTypeInt(signed=False, label="WMT_ATTR_DATATYPE"),
    "WMT_ATTR_IMAGETYPE": SimTypeInt(signed=False, label="WMT_ATTR_IMAGETYPE"),
    "WMT_BUFFER_SEGMENT": SimStruct(OrderedDict((("pBuffer", SimTypeBottom(label="INSSBuffer")), ("cbOffset", SimTypeInt(signed=False, label="UInt32")), ("cbLength", SimTypeInt(signed=False, label="UInt32")),)), name="WMT_BUFFER_SEGMENT", pack=False, align=None),
    "WMT_CODEC_INFO_TYPE": SimTypeInt(signed=False, label="WMT_CODEC_INFO_TYPE"),
    "WMT_COLORSPACEINFO_EXTENSION_DATA": SimStruct(OrderedDict((("ucColorPrimaries", SimTypeChar(label="Byte")), ("ucColorTransferChar", SimTypeChar(label="Byte")), ("ucColorMatrixCoef", SimTypeChar(label="Byte")),)), name="WMT_COLORSPACEINFO_EXTENSION_DATA", pack=False, align=None),
    "WMT_CREDENTIAL_FLAGS": SimTypeInt(signed=False, label="WMT_CREDENTIAL_FLAGS"),
    "WMT_DRMLA_TRUST": SimTypeInt(signed=False, label="WMT_DRMLA_TRUST"),
    "WMT_FILESINK_DATA_UNIT": SimStruct(OrderedDict((("packetHeaderBuffer", SimTypeRef("WMT_BUFFER_SEGMENT", SimStruct)), ("cPayloads", SimTypeInt(signed=False, label="UInt32")), ("pPayloadHeaderBuffers", SimTypePointer(SimTypeRef("WMT_BUFFER_SEGMENT", SimStruct), offset=0)), ("cPayloadDataFragments", SimTypeInt(signed=False, label="UInt32")), ("pPayloadDataFragments", SimTypePointer(SimTypeRef("WMT_PAYLOAD_FRAGMENT", SimStruct), offset=0)),)), name="WMT_FILESINK_DATA_UNIT", pack=False, align=None),
    "WMT_FILESINK_MODE": SimTypeInt(signed=False, label="WMT_FILESINK_MODE"),
    "WMT_IMAGE_TYPE": SimTypeInt(signed=False, label="WMT_IMAGE_TYPE"),
    "WMT_INDEXER_TYPE": SimTypeInt(signed=False, label="WMT_INDEXER_TYPE"),
    "WMT_INDEX_TYPE": SimTypeInt(signed=False, label="WMT_INDEX_TYPE"),
    "WMT_MUSICSPEECH_CLASS_MODE": SimTypeInt(signed=False, label="WMT_MUSICSPEECH_CLASS_MODE"),
    "WMT_NET_PROTOCOL": SimTypeInt(signed=False, label="WMT_NET_PROTOCOL"),
    "WMT_OFFSET_FORMAT": SimTypeInt(signed=False, label="WMT_OFFSET_FORMAT"),
    "WMT_PAYLOAD_FRAGMENT": SimStruct(OrderedDict((("dwPayloadIndex", SimTypeInt(signed=False, label="UInt32")), ("segmentData", SimTypeRef("WMT_BUFFER_SEGMENT", SimStruct)),)), name="WMT_PAYLOAD_FRAGMENT", pack=False, align=None),
    "WMT_PLAY_MODE": SimTypeInt(signed=False, label="WMT_PLAY_MODE"),
    "WMT_PROP_DATATYPE": SimTypeInt(signed=False, label="WMT_PROP_DATATYPE"),
    "WMT_PROXY_SETTINGS": SimTypeInt(signed=False, label="WMT_PROXY_SETTINGS"),
    "WMT_RIGHTS": SimTypeInt(signed=False, label="WMT_RIGHTS"),
    "WMT_STATUS": SimTypeInt(signed=False, label="WMT_STATUS"),
    "WMT_STORAGE_FORMAT": SimTypeInt(signed=False, label="WMT_STORAGE_FORMAT"),
    "WMT_STREAM_SELECTION": SimTypeInt(signed=False, label="WMT_STREAM_SELECTION"),
    "WMT_TIMECODE_EXTENSION_DATA": SimStruct(OrderedDict((("wRange", SimTypeShort(signed=False, label="UInt16")), ("dwTimecode", SimTypeInt(signed=False, label="UInt32")), ("dwUserbits", SimTypeInt(signed=False, label="UInt32")), ("dwAmFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WMT_TIMECODE_EXTENSION_DATA", pack=False, align=None),
    "WMT_TIMECODE_FRAMERATE": SimTypeInt(signed=False, label="WMT_TIMECODE_FRAMERATE"),
    "WMT_TRANSPORT_TYPE": SimTypeInt(signed=False, label="WMT_TRANSPORT_TYPE"),
    "WMT_VERSION": SimTypeInt(signed=False, label="WMT_VERSION"),
    "WMT_VIDEOIMAGE_SAMPLE": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwControlFlags", SimTypeInt(signed=False, label="UInt32")), ("dwInputFlagsCur", SimTypeInt(signed=False, label="UInt32")), ("lCurMotionXtoX", SimTypeInt(signed=True, label="Int32")), ("lCurMotionYtoX", SimTypeInt(signed=True, label="Int32")), ("lCurMotionXoffset", SimTypeInt(signed=True, label="Int32")), ("lCurMotionXtoY", SimTypeInt(signed=True, label="Int32")), ("lCurMotionYtoY", SimTypeInt(signed=True, label="Int32")), ("lCurMotionYoffset", SimTypeInt(signed=True, label="Int32")), ("lCurBlendCoef1", SimTypeInt(signed=True, label="Int32")), ("lCurBlendCoef2", SimTypeInt(signed=True, label="Int32")), ("dwInputFlagsPrev", SimTypeInt(signed=False, label="UInt32")), ("lPrevMotionXtoX", SimTypeInt(signed=True, label="Int32")), ("lPrevMotionYtoX", SimTypeInt(signed=True, label="Int32")), ("lPrevMotionXoffset", SimTypeInt(signed=True, label="Int32")), ("lPrevMotionXtoY", SimTypeInt(signed=True, label="Int32")), ("lPrevMotionYtoY", SimTypeInt(signed=True, label="Int32")), ("lPrevMotionYoffset", SimTypeInt(signed=True, label="Int32")), ("lPrevBlendCoef1", SimTypeInt(signed=True, label="Int32")), ("lPrevBlendCoef2", SimTypeInt(signed=True, label="Int32")),)), name="WMT_VIDEOIMAGE_SAMPLE", pack=False, align=None),
    "WMT_VIDEOIMAGE_SAMPLE2": SimStruct(OrderedDict((("dwMagic", SimTypeInt(signed=False, label="UInt32")), ("dwStructSize", SimTypeInt(signed=False, label="UInt32")), ("dwControlFlags", SimTypeInt(signed=False, label="UInt32")), ("dwViewportWidth", SimTypeInt(signed=False, label="UInt32")), ("dwViewportHeight", SimTypeInt(signed=False, label="UInt32")), ("dwCurrImageWidth", SimTypeInt(signed=False, label="UInt32")), ("dwCurrImageHeight", SimTypeInt(signed=False, label="UInt32")), ("fCurrRegionX0", SimTypeFloat(size=32)), ("fCurrRegionY0", SimTypeFloat(size=32)), ("fCurrRegionWidth", SimTypeFloat(size=32)), ("fCurrRegionHeight", SimTypeFloat(size=32)), ("fCurrBlendCoef", SimTypeFloat(size=32)), ("dwPrevImageWidth", SimTypeInt(signed=False, label="UInt32")), ("dwPrevImageHeight", SimTypeInt(signed=False, label="UInt32")), ("fPrevRegionX0", SimTypeFloat(size=32)), ("fPrevRegionY0", SimTypeFloat(size=32)), ("fPrevRegionWidth", SimTypeFloat(size=32)), ("fPrevRegionHeight", SimTypeFloat(size=32)), ("fPrevBlendCoef", SimTypeFloat(size=32)), ("dwEffectType", SimTypeInt(signed=False, label="UInt32")), ("dwNumEffectParas", SimTypeInt(signed=False, label="UInt32")), ("fEffectPara0", SimTypeFloat(size=32)), ("fEffectPara1", SimTypeFloat(size=32)), ("fEffectPara2", SimTypeFloat(size=32)), ("fEffectPara3", SimTypeFloat(size=32)), ("fEffectPara4", SimTypeFloat(size=32)), ("bKeepPrevImage", SimTypeInt(signed=True, label="Int32")),)), name="WMT_VIDEOIMAGE_SAMPLE2", pack=False, align=None),
    "WMT_WATERMARK_ENTRY": SimStruct(OrderedDict((("wmetType", SimTypeInt(signed=False, label="WMT_WATERMARK_ENTRY_TYPE")), ("clsid", SimTypeBottom(label="Guid")), ("cbDisplayName", SimTypeInt(signed=False, label="UInt32")), ("pwszDisplayName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WMT_WATERMARK_ENTRY", pack=False, align=None),
    "WMT_WATERMARK_ENTRY_TYPE": SimTypeInt(signed=False, label="WMT_WATERMARK_ENTRY_TYPE"),
    "WMT_WEBSTREAM_FORMAT": SimStruct(OrderedDict((("cbSize", SimTypeShort(signed=False, label="UInt16")), ("cbSampleHeaderFixedData", SimTypeShort(signed=False, label="UInt16")), ("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")),)), name="WMT_WEBSTREAM_FORMAT", pack=False, align=None),
    "WMT_WEBSTREAM_SAMPLE_HEADER": SimStruct(OrderedDict((("cbLength", SimTypeShort(signed=False, label="UInt16")), ("wPart", SimTypeShort(signed=False, label="UInt16")), ("cTotalParts", SimTypeShort(signed=False, label="UInt16")), ("wSampleType", SimTypeShort(signed=False, label="UInt16")), ("wszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WMT_WEBSTREAM_SAMPLE_HEADER", pack=False, align=None),
    "WMVIDEOINFOHEADER": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("bmiHeader", SimTypeRef("BITMAPINFOHEADER", SimStruct)),)), name="WMVIDEOINFOHEADER", pack=False, align=None),
    "WMVIDEOINFOHEADER2": SimStruct(OrderedDict((("rcSource", SimTypeRef("RECT", SimStruct)), ("rcTarget", SimTypeRef("RECT", SimStruct)), ("dwBitRate", SimTypeInt(signed=False, label="UInt32")), ("dwBitErrorRate", SimTypeInt(signed=False, label="UInt32")), ("AvgTimePerFrame", SimTypeLongLong(signed=True, label="Int64")), ("dwInterlaceFlags", SimTypeInt(signed=False, label="UInt32")), ("dwCopyProtectFlags", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioX", SimTypeInt(signed=False, label="UInt32")), ("dwPictAspectRatioY", SimTypeInt(signed=False, label="UInt32")), ("dwReserved1", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")), ("bmiHeader", SimTypeRef("BITMAPINFOHEADER", SimStruct)),)), name="WMVIDEOINFOHEADER2", pack=False, align=None),
    "WMV_DYNAMIC_FLAGS": SimTypeInt(signed=False, label="WMV_DYNAMIC_FLAGS"),
    "WM_ADDRESS_ACCESSENTRY": SimStruct(OrderedDict((("dwIPAddress", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")),)), name="WM_ADDRESS_ACCESSENTRY", pack=False, align=None),
    "WM_AETYPE": SimTypeInt(signed=False, label="WM_AETYPE"),
    "WM_CLIENT_PROPERTIES": SimStruct(OrderedDict((("dwIPAddress", SimTypeInt(signed=False, label="UInt32")), ("dwPort", SimTypeInt(signed=False, label="UInt32")),)), name="WM_CLIENT_PROPERTIES", pack=False, align=None),
    "WM_CLIENT_PROPERTIES_EX": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pwszIPAddress", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPort", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszDNSName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WM_CLIENT_PROPERTIES_EX", pack=False, align=None),
    "WM_DM_INTERLACED_TYPE": SimTypeInt(signed=False, label="WM_DM_INTERLACED_TYPE"),
    "WM_DM_IT_FIRST_FRAME_COHERENCY": SimTypeInt(signed=False, label="WM_DM_IT_FIRST_FRAME_COHERENCY"),
    "WM_LEAKY_BUCKET_PAIR": SimStruct(OrderedDict((("dwBitrate", SimTypeInt(signed=False, label="UInt32")), ("msBufferWindow", SimTypeInt(signed=False, label="UInt32")),)), name="WM_LEAKY_BUCKET_PAIR", pack=False, align=None),
    "WM_MEDIA_TYPE": SimStruct(OrderedDict((("majortype", SimTypeBottom(label="Guid")), ("subtype", SimTypeBottom(label="Guid")), ("bFixedSizeSamples", SimTypeInt(signed=True, label="Int32")), ("bTemporalCompression", SimTypeInt(signed=True, label="Int32")), ("lSampleSize", SimTypeInt(signed=False, label="UInt32")), ("formattype", SimTypeBottom(label="Guid")), ("pUnk", SimTypeBottom(label="IUnknown")), ("cbFormat", SimTypeInt(signed=False, label="UInt32")), ("pbFormat", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WM_MEDIA_TYPE", pack=False, align=None),
    "WM_PICTURE": SimStruct(OrderedDict((("pwszMIMEType", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("bPictureType", SimTypeChar(label="Byte")), ("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwDataLen", SimTypeInt(signed=False, label="UInt32")), ("pbData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WM_PICTURE", pack=False, align=None),
    "WM_PLAYBACK_DRC_LEVEL": SimTypeInt(signed=False, label="WM_PLAYBACK_DRC_LEVEL"),
    "WM_PORT_NUMBER_RANGE": SimStruct(OrderedDict((("wPortBegin", SimTypeShort(signed=False, label="UInt16")), ("wPortEnd", SimTypeShort(signed=False, label="UInt16")),)), name="WM_PORT_NUMBER_RANGE", pack=False, align=None),
    "WM_READER_CLIENTINFO": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("wszLang", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszBrowserUserAgent", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("wszBrowserWebPage", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("qwReserved", SimTypeLongLong(signed=False, label="UInt64")), ("pReserved", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("wszHostExe", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("qwHostVersion", SimTypeLongLong(signed=False, label="UInt64")), ("wszPlayerUserAgent", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WM_READER_CLIENTINFO", pack=False, align=None),
    "WM_READER_STATISTICS": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwBandwidth", SimTypeInt(signed=False, label="UInt32")), ("cPacketsReceived", SimTypeInt(signed=False, label="UInt32")), ("cPacketsRecovered", SimTypeInt(signed=False, label="UInt32")), ("cPacketsLost", SimTypeInt(signed=False, label="UInt32")), ("wQuality", SimTypeShort(signed=False, label="UInt16")),)), name="WM_READER_STATISTICS", pack=False, align=None),
    "WM_SFEX_TYPE": SimTypeInt(signed=False, label="WM_SFEX_TYPE"),
    "WM_SF_TYPE": SimTypeInt(signed=False, label="WM_SF_TYPE"),
    "WM_STREAM_PRIORITY_RECORD": SimStruct(OrderedDict((("wStreamNumber", SimTypeShort(signed=False, label="UInt16")), ("fMandatory", SimTypeInt(signed=True, label="Int32")),)), name="WM_STREAM_PRIORITY_RECORD", pack=False, align=None),
    "WM_STREAM_TYPE_INFO": SimStruct(OrderedDict((("guidMajorType", SimTypeBottom(label="Guid")), ("cbFormat", SimTypeInt(signed=False, label="UInt32")),)), name="WM_STREAM_TYPE_INFO", pack=False, align=None),
    "WM_SYNCHRONISED_LYRICS": SimStruct(OrderedDict((("bTimeStampFormat", SimTypeChar(label="Byte")), ("bContentType", SimTypeChar(label="Byte")), ("pwszContentDescriptor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwLyricsLen", SimTypeInt(signed=False, label="UInt32")), ("pbLyrics", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WM_SYNCHRONISED_LYRICS", pack=False, align=None),
    "WM_USER_TEXT": SimStruct(OrderedDict((("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszText", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WM_USER_TEXT", pack=False, align=None),
    "WM_USER_WEB_URL": SimStruct(OrderedDict((("pwszDescription", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WM_USER_WEB_URL", pack=False, align=None),
    "WM_WRITER_STATISTICS": SimStruct(OrderedDict((("qwSampleCount", SimTypeLongLong(signed=False, label="UInt64")), ("qwByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("qwDroppedSampleCount", SimTypeLongLong(signed=False, label="UInt64")), ("qwDroppedByteCount", SimTypeLongLong(signed=False, label="UInt64")), ("dwCurrentBitrate", SimTypeInt(signed=False, label="UInt32")), ("dwAverageBitrate", SimTypeInt(signed=False, label="UInt32")), ("dwExpectedBitrate", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentSampleRate", SimTypeInt(signed=False, label="UInt32")), ("dwAverageSampleRate", SimTypeInt(signed=False, label="UInt32")), ("dwExpectedSampleRate", SimTypeInt(signed=False, label="UInt32")),)), name="WM_WRITER_STATISTICS", pack=False, align=None),
    "WM_WRITER_STATISTICS_EX": SimStruct(OrderedDict((("dwBitratePlusOverhead", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentSampleDropRateInQueue", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentSampleDropRateInCodec", SimTypeInt(signed=False, label="UInt32")), ("dwCurrentSampleDropRateInMultiplexer", SimTypeInt(signed=False, label="UInt32")), ("dwTotalSampleDropsInQueue", SimTypeInt(signed=False, label="UInt32")), ("dwTotalSampleDropsInCodec", SimTypeInt(signed=False, label="UInt32")), ("dwTotalSampleDropsInMultiplexer", SimTypeInt(signed=False, label="UInt32")),)), name="WM_WRITER_STATISTICS_EX", pack=False, align=None),
    "WNDCLASSA": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="WNDCLASS_STYLES")), ("lpfnWndProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("cbClsExtra", SimTypeInt(signed=True, label="Int32")), ("cbWndExtra", SimTypeInt(signed=True, label="Int32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCursor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbrBackground", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszMenuName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszClassName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WNDCLASSA", pack=False, align=None),
    "WNDCLASSEXA": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("style", SimTypeInt(signed=False, label="WNDCLASS_STYLES")), ("lpfnWndProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("cbClsExtra", SimTypeInt(signed=True, label="Int32")), ("cbWndExtra", SimTypeInt(signed=True, label="Int32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCursor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbrBackground", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszMenuName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpszClassName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("hIconSm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WNDCLASSEXA", pack=False, align=None),
    "WNDCLASSEXW": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("style", SimTypeInt(signed=False, label="WNDCLASS_STYLES")), ("lpfnWndProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("cbClsExtra", SimTypeInt(signed=True, label="Int32")), ("cbWndExtra", SimTypeInt(signed=True, label="Int32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCursor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbrBackground", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszMenuName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("hIconSm", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="WNDCLASSEXW", pack=False, align=None),
    "WNDCLASSW": SimStruct(OrderedDict((("style", SimTypeInt(signed=False, label="WNDCLASS_STYLES")), ("lpfnWndProc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0)), ("cbClsExtra", SimTypeInt(signed=True, label="Int32")), ("cbWndExtra", SimTypeInt(signed=True, label="Int32")), ("hInstance", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hIcon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hCursor", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hbrBackground", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("lpszMenuName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpszClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WNDCLASSW", pack=False, align=None),
    "WNDCLASS_STYLES": SimTypeInt(signed=False, label="WNDCLASS_STYLES"),
    "WNDENUMPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1"]), offset=0),
    "WNDOBJ": SimStruct(OrderedDict((("coClient", SimTypeRef("CLIPOBJ", SimStruct)), ("pvConsumer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("rclClient", SimTypeRef("RECTL", SimStruct)), ("psoOwner", SimTypePointer(SimTypeRef("SURFOBJ", SimStruct), offset=0)),)), name="WNDOBJ", pack=False, align=None),
    "WNDOBJCHANGEPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WNDOBJ", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pwo", "fl"]), offset=0),
    "WNDPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "WNET_OPEN_ENUM_USAGE": SimTypeInt(signed=False, label="WNET_OPEN_ENUM_USAGE"),
    "WNF_KSCAMERA_STREAMSTATE_INFO": SimStruct(OrderedDict((("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("StreamState", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="WNF_KSCAMERA_STREAMSTATE_INFO", pack=False, align=None),
    "WNF_STATE_NAME": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)),)), name="WNF_STATE_NAME", pack=False, align=None),
    "WNODE_ALL_DATA": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)), ("DataBlockOffset", SimTypeInt(signed=False, label="UInt32")), ("InstanceCount", SimTypeInt(signed=False, label="UInt32")), ("OffsetInstanceNameOffsets", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"FixedInstanceSize": SimTypeInt(signed=False, label="UInt32"), "OffsetInstanceDataAndLength": SimTypePointer(SimTypeRef("OFFSETINSTANCEDATAANDLENGTH", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="WNODE_ALL_DATA", pack=False, align=None),
    "WNODE_EVENT_ITEM": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)),)), name="WNODE_EVENT_ITEM", pack=False, align=None),
    "WNODE_EVENT_REFERENCE": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)), ("TargetGuid", SimTypeBottom(label="Guid")), ("TargetDataBlockSize", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"TargetInstanceIndex": SimTypeInt(signed=False, label="UInt32"), "TargetInstanceName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="WNODE_EVENT_REFERENCE", pack=False, align=None),
    "WNODE_HEADER": SimStruct(OrderedDict((("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("ProviderId", SimTypeInt(signed=False, label="UInt32")), ("Anonymous1", SimUnion({"HistoricalContext": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Linkage", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"CountLost": SimTypeInt(signed=False, label="UInt32"), "KernelHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "TimeStamp": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")), ("Guid", SimTypeBottom(label="Guid")), ("ClientContext", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="WNODE_HEADER", pack=False, align=None),
    "WNODE_METHOD_ITEM": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)), ("OffsetInstanceName", SimTypeInt(signed=False, label="UInt32")), ("InstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("MethodId", SimTypeInt(signed=False, label="UInt32")), ("DataBlockOffset", SimTypeInt(signed=False, label="UInt32")), ("SizeDataBlock", SimTypeInt(signed=False, label="UInt32")), ("VariableData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WNODE_METHOD_ITEM", pack=False, align=None),
    "WNODE_SINGLE_INSTANCE": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)), ("OffsetInstanceName", SimTypeInt(signed=False, label="UInt32")), ("InstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("DataBlockOffset", SimTypeInt(signed=False, label="UInt32")), ("SizeDataBlock", SimTypeInt(signed=False, label="UInt32")), ("VariableData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WNODE_SINGLE_INSTANCE", pack=False, align=None),
    "WNODE_SINGLE_ITEM": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)), ("OffsetInstanceName", SimTypeInt(signed=False, label="UInt32")), ("InstanceIndex", SimTypeInt(signed=False, label="UInt32")), ("ItemId", SimTypeInt(signed=False, label="UInt32")), ("DataBlockOffset", SimTypeInt(signed=False, label="UInt32")), ("SizeDataItem", SimTypeInt(signed=False, label="UInt32")), ("VariableData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WNODE_SINGLE_ITEM", pack=False, align=None),
    "WNODE_TOO_SMALL": SimStruct(OrderedDict((("WnodeHeader", SimTypeRef("WNODE_HEADER", SimStruct)), ("SizeNeeded", SimTypeInt(signed=False, label="UInt32")),)), name="WNODE_TOO_SMALL", pack=False, align=None),
    "WNPERM_DLG": SimTypeInt(signed=False, label="WNPERM_DLG"),
    "WNV_CA_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="WNV_CA_NOTIFICATION_TYPE"),
    "WNV_CUSTOMER_ADDRESS_CHANGE_PARAM": SimStruct(OrderedDict((("MACAddress", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 6), "Anonymous": SimStruct(OrderedDict((("Oui", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Ei48", SimUnion({"Byte": SimTypeArray(SimTypeChar(label="Byte"), 3)}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("CAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("CA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)), ("VirtualSubnetId", SimTypeInt(signed=False, label="UInt32")), ("PAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("PA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)), ("NotificationReason", SimTypeInt(signed=False, label="WNV_CA_NOTIFICATION_TYPE")),)), name="WNV_CUSTOMER_ADDRESS_CHANGE_PARAM", pack=False, align=None),
    "WNV_IP_ADDRESS": SimStruct(OrderedDict((("IP", SimUnion({"v4": SimTypeRef("IN_ADDR", SimStruct), "v6": SimTypeRef("IN6_ADDR", SimStruct), "Addr": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")),)), name="WNV_IP_ADDRESS", pack=False, align=None),
    "WNV_NOTIFICATION_PARAM": SimStruct(OrderedDict((("Header", SimTypeRef("WNV_OBJECT_HEADER", SimStruct)), ("NotificationType", SimTypeInt(signed=False, label="WNV_NOTIFICATION_TYPE")), ("PendingNotifications", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WNV_NOTIFICATION_PARAM", pack=False, align=None),
    "WNV_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="WNV_NOTIFICATION_TYPE"),
    "WNV_OBJECT_CHANGE_PARAM": SimStruct(OrderedDict((("ObjectType", SimTypeInt(signed=False, label="WNV_OBJECT_TYPE")), ("ObjectParam", SimUnion({"ProviderAddressChange": SimTypeRef("WNV_PROVIDER_ADDRESS_CHANGE_PARAM", SimStruct), "CustomerAddressChange": SimTypeRef("WNV_CUSTOMER_ADDRESS_CHANGE_PARAM", SimStruct)}, name="<anon>", label="None")),)), name="WNV_OBJECT_CHANGE_PARAM", pack=False, align=None),
    "WNV_OBJECT_HEADER": SimStruct(OrderedDict((("MajorVersion", SimTypeChar(label="Byte")), ("MinorVersion", SimTypeChar(label="Byte")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="WNV_OBJECT_HEADER", pack=False, align=None),
    "WNV_OBJECT_TYPE": SimTypeInt(signed=False, label="WNV_OBJECT_TYPE"),
    "WNV_POLICY_MISMATCH_PARAM": SimStruct(OrderedDict((("CAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("PAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("VirtualSubnetId", SimTypeInt(signed=False, label="UInt32")), ("CA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)), ("PA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)),)), name="WNV_POLICY_MISMATCH_PARAM", pack=False, align=None),
    "WNV_PROVIDER_ADDRESS_CHANGE_PARAM": SimStruct(OrderedDict((("PAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("PA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)), ("AddressState", SimTypeInt(signed=False, label="NL_DAD_STATE")),)), name="WNV_PROVIDER_ADDRESS_CHANGE_PARAM", pack=False, align=None),
    "WNV_REDIRECT_PARAM": SimStruct(OrderedDict((("CAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("PAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("NewPAFamily", SimTypeInt(signed=False, label="ADDRESS_FAMILY")), ("VirtualSubnetId", SimTypeInt(signed=False, label="UInt32")), ("CA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)), ("PA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)), ("NewPA", SimTypeRef("WNV_IP_ADDRESS", SimStruct)),)), name="WNV_REDIRECT_PARAM", pack=False, align=None),
    "WOF_EXTERNAL_FILE_ID": SimStruct(OrderedDict((("FileId", SimTypeRef("FILE_ID_128", SimStruct)),)), name="WOF_EXTERNAL_FILE_ID", pack=False, align=None),
    "WOF_EXTERNAL_INFO": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Provider", SimTypeInt(signed=False, label="UInt32")),)), name="WOF_EXTERNAL_INFO", pack=False, align=None),
    "WOF_FILE_COMPRESSION_INFO_V0": SimStruct(OrderedDict((("Algorithm", SimTypeInt(signed=False, label="UInt32")),)), name="WOF_FILE_COMPRESSION_INFO_V0", pack=False, align=None),
    "WOF_FILE_COMPRESSION_INFO_V1": SimStruct(OrderedDict((("Algorithm", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="WOF_FILE_COMPRESSION_INFO_V1", pack=False, align=None),
    "WOF_VERSION_INFO": SimStruct(OrderedDict((("WofVersion", SimTypeInt(signed=False, label="UInt32")),)), name="WOF_VERSION_INFO", pack=False, align=None),
    "WORDLIST_TYPE": SimTypeInt(signed=False, label="WORDLIST_TYPE"),
    "WORDREP_BREAK_TYPE": SimTypeInt(signed=False, label="WORDREP_BREAK_TYPE"),
    "WORD_BLOB": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("asData", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="WORD_BLOB", pack=False, align=None),
    "WORD_BREAK_ACTION": SimTypeInt(signed=False, label="WORD_BREAK_ACTION"),
    "WORD_SIZEDARR": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("pData", SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)),)), name="WORD_SIZEDARR", pack=False, align=None),
    "WORD_WHEEL_OPEN_FLAGS": SimTypeInt(signed=False, label="WORD_WHEEL_OPEN_FLAGS"),
    "WORKERCALLBACKFUNC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "WORKERFUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "WORKER_THREAD_FLAGS": SimTypeInt(signed=False, label="WORKER_THREAD_FLAGS"),
    "WORKER_THREAD_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Parameter"]), offset=0),
    "WORK_QUEUE_ITEM": SimStruct(OrderedDict((("List", SimTypeRef("LIST_ENTRY", SimStruct)), ("WorkerRoutine", SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)), ("Parameter", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WORK_QUEUE_ITEM", pack=False, align=None),
    "WORK_QUEUE_TYPE": SimTypeInt(signed=False, label="WORK_QUEUE_TYPE"),
    "WOW64_CONTEXT": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=False, label="WOW64_CONTEXT_FLAGS")), ("Dr0", SimTypeInt(signed=False, label="UInt32")), ("Dr1", SimTypeInt(signed=False, label="UInt32")), ("Dr2", SimTypeInt(signed=False, label="UInt32")), ("Dr3", SimTypeInt(signed=False, label="UInt32")), ("Dr6", SimTypeInt(signed=False, label="UInt32")), ("Dr7", SimTypeInt(signed=False, label="UInt32")), ("FloatSave", SimTypeRef("WOW64_FLOATING_SAVE_AREA", SimStruct)), ("SegGs", SimTypeInt(signed=False, label="UInt32")), ("SegFs", SimTypeInt(signed=False, label="UInt32")), ("SegEs", SimTypeInt(signed=False, label="UInt32")), ("SegDs", SimTypeInt(signed=False, label="UInt32")), ("Edi", SimTypeInt(signed=False, label="UInt32")), ("Esi", SimTypeInt(signed=False, label="UInt32")), ("Ebx", SimTypeInt(signed=False, label="UInt32")), ("Edx", SimTypeInt(signed=False, label="UInt32")), ("Ecx", SimTypeInt(signed=False, label="UInt32")), ("Eax", SimTypeInt(signed=False, label="UInt32")), ("Ebp", SimTypeInt(signed=False, label="UInt32")), ("Eip", SimTypeInt(signed=False, label="UInt32")), ("SegCs", SimTypeInt(signed=False, label="UInt32")), ("EFlags", SimTypeInt(signed=False, label="UInt32")), ("Esp", SimTypeInt(signed=False, label="UInt32")), ("SegSs", SimTypeInt(signed=False, label="UInt32")), ("ExtendedRegisters", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="WOW64_CONTEXT", pack=False, align=None),
    "WOW64_CONTEXT_FLAGS": SimTypeInt(signed=False, label="WOW64_CONTEXT_FLAGS"),
    "WOW64_DESCRIPTOR_TABLE_ENTRY": SimStruct(OrderedDict((("Selector", SimTypeInt(signed=False, label="UInt32")), ("Descriptor", SimTypeRef("WOW64_LDT_ENTRY", SimStruct)),)), name="WOW64_DESCRIPTOR_TABLE_ENTRY", pack=False, align=None),
    "WOW64_FLOATING_SAVE_AREA": SimStruct(OrderedDict((("ControlWord", SimTypeInt(signed=False, label="UInt32")), ("StatusWord", SimTypeInt(signed=False, label="UInt32")), ("TagWord", SimTypeInt(signed=False, label="UInt32")), ("ErrorOffset", SimTypeInt(signed=False, label="UInt32")), ("ErrorSelector", SimTypeInt(signed=False, label="UInt32")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSelector", SimTypeInt(signed=False, label="UInt32")), ("RegisterArea", SimTypeArray(SimTypeChar(label="Byte"), 80)), ("Cr0NpxState", SimTypeInt(signed=False, label="UInt32")),)), name="WOW64_FLOATING_SAVE_AREA", pack=False, align=None),
    "WOW64_LDT_ENTRY": SimStruct(OrderedDict((("LimitLow", SimTypeShort(signed=False, label="UInt16")), ("BaseLow", SimTypeShort(signed=False, label="UInt16")), ("HighWord", SimUnion({"Bytes": SimTypeRef("_Bytes_e__Struct", SimStruct), "Bits": SimTypeRef("_Bits_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WOW64_LDT_ENTRY", pack=False, align=None),
    "WPAD_CACHE_DELETE": SimTypeInt(signed=False, label="WPAD_CACHE_DELETE"),
    "WPARAM": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0),
    "WPCFLAG_IM_FEATURE": SimTypeInt(signed=False, label="WPCFLAG_IM_FEATURE"),
    "WPCFLAG_IM_LEAVE": SimTypeInt(signed=False, label="WPCFLAG_IM_LEAVE"),
    "WPCFLAG_ISBLOCKED": SimTypeInt(signed=False, label="WPCFLAG_ISBLOCKED"),
    "WPCFLAG_LOGOFF_TYPE": SimTypeInt(signed=False, label="WPCFLAG_LOGOFF_TYPE"),
    "WPCFLAG_OVERRIDE": SimTypeInt(signed=False, label="WPCFLAG_OVERRIDE"),
    "WPCFLAG_RESTRICTION": SimTypeInt(signed=False, label="WPCFLAG_RESTRICTION"),
    "WPCFLAG_VISIBILITY": SimTypeInt(signed=False, label="WPCFLAG_VISIBILITY"),
    "WPCFLAG_WEB_SETTING": SimTypeInt(signed=False, label="WPCFLAG_WEB_SETTING"),
    "WPCSETTING": SimTypeInt(signed=False, label="WPCSETTING"),
    "WPC_ARGS_APPLICATIONEVENT": SimTypeInt(signed=False, label="WPC_ARGS_APPLICATIONEVENT"),
    "WPC_ARGS_APPOVERRIDEEVENT": SimTypeInt(signed=False, label="WPC_ARGS_APPOVERRIDEEVENT"),
    "WPC_ARGS_COMPUTERUSAGEEVENT": SimTypeInt(signed=False, label="WPC_ARGS_COMPUTERUSAGEEVENT"),
    "WPC_ARGS_CONTENTUSAGEEVENT": SimTypeInt(signed=False, label="WPC_ARGS_CONTENTUSAGEEVENT"),
    "WPC_ARGS_CONVERSATIONINITEVENT": SimTypeInt(signed=False, label="WPC_ARGS_CONVERSATIONINITEVENT"),
    "WPC_ARGS_CONVERSATIONJOINEVENT": SimTypeInt(signed=False, label="WPC_ARGS_CONVERSATIONJOINEVENT"),
    "WPC_ARGS_CONVERSATIONLEAVEEVENT": SimTypeInt(signed=False, label="WPC_ARGS_CONVERSATIONLEAVEEVENT"),
    "WPC_ARGS_CUSTOMEVENT": SimTypeInt(signed=False, label="WPC_ARGS_CUSTOMEVENT"),
    "WPC_ARGS_EMAILCONTACTEVENT": SimTypeInt(signed=False, label="WPC_ARGS_EMAILCONTACTEVENT"),
    "WPC_ARGS_EMAILRECEIEVEDEVENT": SimTypeInt(signed=False, label="WPC_ARGS_EMAILRECEIEVEDEVENT"),
    "WPC_ARGS_EMAILSENTEVENT": SimTypeInt(signed=False, label="WPC_ARGS_EMAILSENTEVENT"),
    "WPC_ARGS_FILEDOWNLOADEVENT": SimTypeInt(signed=False, label="WPC_ARGS_FILEDOWNLOADEVENT"),
    "WPC_ARGS_GAMESTARTEVENT": SimTypeInt(signed=False, label="WPC_ARGS_GAMESTARTEVENT"),
    "WPC_ARGS_IMCONTACTEVENT": SimTypeInt(signed=False, label="WPC_ARGS_IMCONTACTEVENT"),
    "WPC_ARGS_IMFEATUREEVENT": SimTypeInt(signed=False, label="WPC_ARGS_IMFEATUREEVENT"),
    "WPC_ARGS_MEDIADOWNLOADEVENT": SimTypeInt(signed=False, label="WPC_ARGS_MEDIADOWNLOADEVENT"),
    "WPC_ARGS_MEDIAPLAYBACKEVENT": SimTypeInt(signed=False, label="WPC_ARGS_MEDIAPLAYBACKEVENT"),
    "WPC_ARGS_SAFERAPPBLOCKED": SimTypeInt(signed=False, label="WPC_ARGS_SAFERAPPBLOCKED"),
    "WPC_ARGS_SETTINGSCHANGEEVENT": SimTypeInt(signed=False, label="WPC_ARGS_SETTINGSCHANGEEVENT"),
    "WPC_ARGS_URLVISITEVENT": SimTypeInt(signed=False, label="WPC_ARGS_URLVISITEVENT"),
    "WPC_ARGS_WEBOVERRIDEEVENT": SimTypeInt(signed=False, label="WPC_ARGS_WEBOVERRIDEEVENT"),
    "WPC_ARGS_WEBSITEVISITEVENT": SimTypeInt(signed=False, label="WPC_ARGS_WEBSITEVISITEVENT"),
    "WPC_MEDIA_EXPLICIT": SimTypeInt(signed=False, label="WPC_MEDIA_EXPLICIT"),
    "WPC_MEDIA_TYPE": SimTypeInt(signed=False, label="WPC_MEDIA_TYPE"),
    "WPC_SETTINGS": SimTypeInt(signed=False, label="WPC_SETTINGS"),
    "WPD_BITRATE_TYPES": SimTypeInt(signed=False, label="WPD_BITRATE_TYPES"),
    "WPD_CAPTURE_MODES": SimTypeInt(signed=False, label="WPD_CAPTURE_MODES"),
    "WPD_COLOR_CORRECTED_STATUS_VALUES": SimTypeInt(signed=False, label="WPD_COLOR_CORRECTED_STATUS_VALUES"),
    "WPD_COMMAND_ACCESS_LOOKUP_ENTRY": SimStruct(OrderedDict((("Command", SimTypeRef("PROPERTYKEY", SimStruct)), ("AccessType", SimTypeInt(signed=False, label="UInt32")), ("AccessProperty", SimTypeRef("PROPERTYKEY", SimStruct)),)), name="WPD_COMMAND_ACCESS_LOOKUP_ENTRY", pack=False, align=None),
    "WPD_COMMAND_ACCESS_TYPES": SimTypeInt(signed=False, label="WPD_COMMAND_ACCESS_TYPES"),
    "WPD_CROPPED_STATUS_VALUES": SimTypeInt(signed=False, label="WPD_CROPPED_STATUS_VALUES"),
    "WPD_DEVICE_TRANSPORTS": SimTypeInt(signed=False, label="WPD_DEVICE_TRANSPORTS"),
    "WPD_DEVICE_TYPES": SimTypeInt(signed=False, label="WPD_DEVICE_TYPES"),
    "WPD_EFFECT_MODES": SimTypeInt(signed=False, label="WPD_EFFECT_MODES"),
    "WPD_EXPOSURE_METERING_MODES": SimTypeInt(signed=False, label="WPD_EXPOSURE_METERING_MODES"),
    "WPD_EXPOSURE_PROGRAM_MODES": SimTypeInt(signed=False, label="WPD_EXPOSURE_PROGRAM_MODES"),
    "WPD_FLASH_MODES": SimTypeInt(signed=False, label="WPD_FLASH_MODES"),
    "WPD_FOCUS_METERING_MODES": SimTypeInt(signed=False, label="WPD_FOCUS_METERING_MODES"),
    "WPD_FOCUS_MODES": SimTypeInt(signed=False, label="WPD_FOCUS_MODES"),
    "WPD_META_GENRES": SimTypeInt(signed=False, label="WPD_META_GENRES"),
    "WPD_OPERATION_STATES": SimTypeInt(signed=False, label="WPD_OPERATION_STATES"),
    "WPD_PARAMETER_USAGE_TYPES": SimTypeInt(signed=False, label="WPD_PARAMETER_USAGE_TYPES"),
    "WPD_POWER_SOURCES": SimTypeInt(signed=False, label="WPD_POWER_SOURCES"),
    "WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPES": SimTypeInt(signed=False, label="WPD_RENDERING_INFORMATION_PROFILE_ENTRY_TYPES"),
    "WPD_SECTION_DATA_UNITS_VALUES": SimTypeInt(signed=False, label="WPD_SECTION_DATA_UNITS_VALUES"),
    "WPD_SERVICE_INHERITANCE_TYPES": SimTypeInt(signed=False, label="WPD_SERVICE_INHERITANCE_TYPES"),
    "WPD_SMS_ENCODING_TYPES": SimTypeInt(signed=False, label="WPD_SMS_ENCODING_TYPES"),
    "WPD_STORAGE_ACCESS_CAPABILITY_VALUES": SimTypeInt(signed=False, label="WPD_STORAGE_ACCESS_CAPABILITY_VALUES"),
    "WPD_STORAGE_TYPE_VALUES": SimTypeInt(signed=False, label="WPD_STORAGE_TYPE_VALUES"),
    "WPD_STREAM_UNITS": SimTypeInt(signed=False, label="WPD_STREAM_UNITS"),
    "WPD_VIDEO_SCAN_TYPES": SimTypeInt(signed=False, label="WPD_VIDEO_SCAN_TYPES"),
    "WPD_WHITE_BALANCE_SETTINGS": SimTypeInt(signed=False, label="WPD_WHITE_BALANCE_SETTINGS"),
    "WRDS_CONNECTION_SETTING": SimUnion({"WRdsConnectionSettings1": SimTypeRef("WRDS_CONNECTION_SETTINGS_1", SimStruct)}, name="<anon>", label="None"),
    "WRDS_CONNECTION_SETTINGS": SimStruct(OrderedDict((("WRdsConnectionSettingLevel", SimTypeInt(signed=False, label="WRDS_CONNECTION_SETTING_LEVEL")), ("WRdsConnectionSetting", SimUnion({"WRdsConnectionSettings1": SimTypeRef("WRDS_CONNECTION_SETTINGS_1", SimStruct)}, name="<anon>", label="None")),)), name="WRDS_CONNECTION_SETTINGS", pack=False, align=None),
    "WRDS_CONNECTION_SETTINGS_1": SimStruct(OrderedDict((("fInheritInitialProgram", SimTypeChar(label="Byte")), ("fInheritColorDepth", SimTypeChar(label="Byte")), ("fHideTitleBar", SimTypeChar(label="Byte")), ("fInheritAutoLogon", SimTypeChar(label="Byte")), ("fMaximizeShell", SimTypeChar(label="Byte")), ("fDisablePNP", SimTypeChar(label="Byte")), ("fPasswordIsScPin", SimTypeChar(label="Byte")), ("fPromptForPassword", SimTypeChar(label="Byte")), ("fDisableCpm", SimTypeChar(label="Byte")), ("fDisableCdm", SimTypeChar(label="Byte")), ("fDisableCcm", SimTypeChar(label="Byte")), ("fDisableLPT", SimTypeChar(label="Byte")), ("fDisableClip", SimTypeChar(label="Byte")), ("fResetBroken", SimTypeChar(label="Byte")), ("fDisableEncryption", SimTypeChar(label="Byte")), ("fDisableAutoReconnect", SimTypeChar(label="Byte")), ("fDisableCtrlAltDel", SimTypeChar(label="Byte")), ("fDoubleClickDetect", SimTypeChar(label="Byte")), ("fEnableWindowsKey", SimTypeChar(label="Byte")), ("fUsingSavedCreds", SimTypeChar(label="Byte")), ("fMouse", SimTypeChar(label="Byte")), ("fNoAudioPlayback", SimTypeChar(label="Byte")), ("fRemoteConsoleAudio", SimTypeChar(label="Byte")), ("EncryptionLevel", SimTypeChar(label="Byte")), ("ColorDepth", SimTypeShort(signed=False, label="UInt16")), ("ProtocolType", SimTypeShort(signed=False, label="UInt16")), ("HRes", SimTypeShort(signed=False, label="UInt16")), ("VRes", SimTypeShort(signed=False, label="UInt16")), ("ClientProductId", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountHost", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountClient", SimTypeShort(signed=False, label="UInt16")), ("OutBufLength", SimTypeShort(signed=False, label="UInt16")), ("KeyboardLayout", SimTypeInt(signed=False, label="UInt32")), ("MaxConnectionTime", SimTypeInt(signed=False, label="UInt32")), ("MaxDisconnectionTime", SimTypeInt(signed=False, label="UInt32")), ("MaxIdleTime", SimTypeInt(signed=False, label="UInt32")), ("PerformanceFlags", SimTypeInt(signed=False, label="UInt32")), ("KeyboardType", SimTypeInt(signed=False, label="UInt32")), ("KeyboardSubType", SimTypeInt(signed=False, label="UInt32")), ("KeyboardFunctionKey", SimTypeInt(signed=False, label="UInt32")), ("ActiveInputLocale", SimTypeInt(signed=False, label="UInt32")), ("SerialNumber", SimTypeInt(signed=False, label="UInt32")), ("ClientAddressFamily", SimTypeInt(signed=False, label="UInt32")), ("ClientBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("ClientSessionId", SimTypeInt(signed=False, label="UInt32")), ("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 257)), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 257)), ("UserName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Domain", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Password", SimTypeArray(SimTypeChar(label="Char"), 256)), ("ProtocolName", SimTypeArray(SimTypeChar(label="Char"), 9)), ("DisplayDriverName", SimTypeArray(SimTypeChar(label="Char"), 9)), ("DisplayDeviceName", SimTypeArray(SimTypeChar(label="Char"), 20)), ("imeFileName", SimTypeArray(SimTypeChar(label="Char"), 33)), ("AudioDriverName", SimTypeArray(SimTypeChar(label="Char"), 9)), ("ClientName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("ClientAddress", SimTypeArray(SimTypeChar(label="Char"), 31)), ("ClientDirectory", SimTypeArray(SimTypeChar(label="Char"), 257)), ("ClientDigProductId", SimTypeArray(SimTypeChar(label="Char"), 33)), ("ClientSockAddress", SimTypeRef("WTS_SOCKADDR", SimStruct)), ("ClientTimeZone", SimTypeRef("WTS_TIME_ZONE_INFORMATION", SimStruct)), ("WRdsListenerSettings", SimTypeRef("WRDS_LISTENER_SETTINGS", SimStruct)), ("EventLogActivityId", SimTypeBottom(label="Guid")), ("ContextSize", SimTypeInt(signed=False, label="UInt32")), ("ContextData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WRDS_CONNECTION_SETTINGS_1", pack=False, align=None),
    "WRDS_CONNECTION_SETTING_LEVEL": SimTypeInt(signed=False, label="WRDS_CONNECTION_SETTING_LEVEL"),
    "WRDS_DYNAMIC_TIME_ZONE_INFORMATION": SimStruct(OrderedDict((("Bias", SimTypeInt(signed=True, label="Int32")), ("StandardName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("StandardDate", SimTypeRef("WTS_SYSTEMTIME", SimStruct)), ("StandardBias", SimTypeInt(signed=True, label="Int32")), ("DaylightName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DaylightDate", SimTypeRef("WTS_SYSTEMTIME", SimStruct)), ("DaylightBias", SimTypeInt(signed=True, label="Int32")), ("TimeZoneKeyName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("DynamicDaylightTimeDisabled", SimTypeShort(signed=False, label="UInt16")),)), name="WRDS_DYNAMIC_TIME_ZONE_INFORMATION", pack=False, align=None),
    "WRDS_LISTENER_SETTING": SimUnion({"WRdsListenerSettings1": SimTypeRef("WRDS_LISTENER_SETTINGS_1", SimStruct)}, name="<anon>", label="None"),
    "WRDS_LISTENER_SETTINGS": SimStruct(OrderedDict((("WRdsListenerSettingLevel", SimTypeInt(signed=False, label="WRDS_LISTENER_SETTING_LEVEL")), ("WRdsListenerSetting", SimUnion({"WRdsListenerSettings1": SimTypeRef("WRDS_LISTENER_SETTINGS_1", SimStruct)}, name="<anon>", label="None")),)), name="WRDS_LISTENER_SETTINGS", pack=False, align=None),
    "WRDS_LISTENER_SETTINGS_1": SimStruct(OrderedDict((("MaxProtocolListenerConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptorSize", SimTypeInt(signed=False, label="UInt32")), ("pSecurityDescriptor", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WRDS_LISTENER_SETTINGS_1", pack=False, align=None),
    "WRDS_LISTENER_SETTING_LEVEL": SimTypeInt(signed=False, label="WRDS_LISTENER_SETTING_LEVEL"),
    "WRDS_SETTING": SimUnion({"WRdsSettings1": SimTypeRef("WRDS_SETTINGS_1", SimStruct)}, name="<anon>", label="None"),
    "WRDS_SETTINGS": SimStruct(OrderedDict((("WRdsSettingType", SimTypeInt(signed=False, label="WRDS_SETTING_TYPE")), ("WRdsSettingLevel", SimTypeInt(signed=False, label="WRDS_SETTING_LEVEL")), ("WRdsSetting", SimUnion({"WRdsSettings1": SimTypeRef("WRDS_SETTINGS_1", SimStruct)}, name="<anon>", label="None")),)), name="WRDS_SETTINGS", pack=False, align=None),
    "WRDS_SETTINGS_1": SimStruct(OrderedDict((("WRdsDisableClipStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableClipValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisableLPTStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableLPTValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisableCcmStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableCcmValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisableCdmStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableCdmValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisableCpmStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableCpmValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisablePnpStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisablePnpValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsEncryptionLevelStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsEncryptionValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsColorDepthStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsColorDepthValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisableAutoReconnecetStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableAutoReconnecetValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsDisableEncryptionStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsDisableEncryptionValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsResetBrokenStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsResetBrokenValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsMaxIdleTimeStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsMaxIdleTimeValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsMaxDisconnectTimeStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsMaxDisconnectTimeValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsMaxConnectTimeStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsMaxConnectTimeValue", SimTypeInt(signed=False, label="UInt32")), ("WRdsKeepAliveStatus", SimTypeInt(signed=False, label="WRDS_SETTING_STATUS")), ("WRdsKeepAliveStartValue", SimTypeChar(label="Byte")), ("WRdsKeepAliveIntervalValue", SimTypeInt(signed=False, label="UInt32")),)), name="WRDS_SETTINGS_1", pack=False, align=None),
    "WRDS_SETTING_LEVEL": SimTypeInt(signed=False, label="WRDS_SETTING_LEVEL"),
    "WRDS_SETTING_STATUS": SimTypeInt(signed=False, label="WRDS_SETTING_STATUS"),
    "WRDS_SETTING_TYPE": SimTypeInt(signed=False, label="WRDS_SETTING_TYPE"),
    "WRENCHSTATES": SimTypeInt(signed=False, label="WRENCHSTATES"),
    "WRITEEMBEDPROC": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "WRITE_CACHE_CHANGE": SimTypeInt(signed=False, label="WRITE_CACHE_CHANGE"),
    "WRITE_CACHE_ENABLE": SimTypeInt(signed=False, label="WRITE_CACHE_ENABLE"),
    "WRITE_CACHE_TYPE": SimTypeInt(signed=False, label="WRITE_CACHE_TYPE"),
    "WRITE_THROUGH": SimTypeInt(signed=False, label="WRITE_THROUGH"),
    "WRITE_USN_REASON_INPUT": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("UsnReasonToWrite", SimTypeInt(signed=False, label="UInt32")),)), name="WRITE_USN_REASON_INPUT", pack=False, align=None),
    "WRdsGraphicsChannelType": SimTypeInt(signed=False, label="WRdsGraphicsChannelType"),
    "WSABUF": SimStruct(OrderedDict((("len", SimTypeInt(signed=False, label="UInt32")), ("buf", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WSABUF", pack=False, align=None),
    "WSACOMPLETION": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="WSACOMPLETIONTYPE")), ("Parameters", SimUnion({"WindowMessage": SimTypeRef("_WindowMessage_e__Struct", SimStruct), "Event": SimTypeRef("_Event_e__Struct", SimStruct), "Apc": SimTypeRef("_Apc_e__Struct", SimStruct), "Port": SimTypeRef("_Port_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WSACOMPLETION", pack=False, align=None),
    "WSACOMPLETIONTYPE": SimTypeInt(signed=False, label="WSACOMPLETIONTYPE"),
    "WSADATA": SimStruct(OrderedDict((("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wHighVersion", SimTypeShort(signed=False, label="UInt16")), ("szDescription", SimTypeArray(SimTypeChar(label="SByte"), 257)), ("szSystemStatus", SimTypeArray(SimTypeChar(label="SByte"), 129)), ("iMaxSockets", SimTypeShort(signed=False, label="UInt16")), ("iMaxUdpDg", SimTypeShort(signed=False, label="UInt16")), ("lpVendorInfo", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WSADATA", pack=False, align=None),
    "WSAECOMPARATOR": SimTypeInt(signed=False, label="WSAECOMPARATOR"),
    "WSAESETSERVICEOP": SimTypeInt(signed=False, label="WSAESETSERVICEOP"),
    "WSAEVENT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WSAMSG": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0)), ("namelen", SimTypeInt(signed=True, label="Int32")), ("lpBuffers", SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0)), ("dwBufferCount", SimTypeInt(signed=False, label="UInt32")), ("Control", SimTypeRef("WSABUF", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WSAMSG", pack=False, align=None),
    "WSANAMESPACE_INFOA": SimStruct(OrderedDict((("NSProviderId", SimTypeBottom(label="Guid")), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("fActive", SimTypeInt(signed=True, label="Int32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WSANAMESPACE_INFOA", pack=False, align=None),
    "WSANAMESPACE_INFOEXA": SimStruct(OrderedDict((("NSProviderId", SimTypeBottom(label="Guid")), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("fActive", SimTypeInt(signed=True, label="Int32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszIdentifier", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ProviderSpecific", SimTypeRef("BLOB", SimStruct)),)), name="WSANAMESPACE_INFOEXA", pack=False, align=None),
    "WSANAMESPACE_INFOEXW": SimStruct(OrderedDict((("NSProviderId", SimTypeBottom(label="Guid")), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("fActive", SimTypeInt(signed=True, label="Int32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ProviderSpecific", SimTypeRef("BLOB", SimStruct)),)), name="WSANAMESPACE_INFOEXW", pack=False, align=None),
    "WSANAMESPACE_INFOW": SimStruct(OrderedDict((("NSProviderId", SimTypeBottom(label="Guid")), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("fActive", SimTypeInt(signed=True, label="Int32")), ("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("lpszIdentifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSANAMESPACE_INFOW", pack=False, align=None),
    "WSANETWORKEVENTS": SimStruct(OrderedDict((("lNetworkEvents", SimTypeInt(signed=True, label="Int32")), ("iErrorCode", SimTypeArray(SimTypeInt(signed=True, label="Int32"), 10)),)), name="WSANETWORKEVENTS", pack=False, align=None),
    "WSANSCLASSINFOA": SimStruct(OrderedDict((("lpszName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("dwValueSize", SimTypeInt(signed=False, label="UInt32")), ("lpValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WSANSCLASSINFOA", pack=False, align=None),
    "WSANSCLASSINFOW": SimStruct(OrderedDict((("lpszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("dwValueType", SimTypeInt(signed=False, label="UInt32")), ("dwValueSize", SimTypeInt(signed=False, label="UInt32")), ("lpValue", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WSANSCLASSINFOW", pack=False, align=None),
    "WSAPOLLDATA": SimStruct(OrderedDict((("result", SimTypeInt(signed=True, label="Int32")), ("fds", SimTypeInt(signed=False, label="UInt32")), ("timeout", SimTypeInt(signed=True, label="Int32")), ("fdArray", SimTypePointer(SimTypeRef("WSAPOLLFD", SimStruct), offset=0)),)), name="WSAPOLLDATA", pack=False, align=None),
    "WSAPOLLFD": SimStruct(OrderedDict((("fd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("events", SimTypeInt(signed=False, label="WSAPOLL_EVENT_FLAGS")), ("revents", SimTypeInt(signed=False, label="WSAPOLL_EVENT_FLAGS")),)), name="WSAPOLLFD", pack=False, align=None),
    "WSAPOLL_EVENT_FLAGS": SimTypeInt(signed=False, label="WSAPOLL_EVENT_FLAGS"),
    "WSAPROTOCOLCHAIN": SimStruct(OrderedDict((("ChainLen", SimTypeInt(signed=True, label="Int32")), ("ChainEntries", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 7)),)), name="WSAPROTOCOLCHAIN", pack=False, align=None),
    "WSAPROTOCOL_INFOA": SimStruct(OrderedDict((("dwServiceFlags1", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags2", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags3", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags4", SimTypeInt(signed=False, label="UInt32")), ("dwProviderFlags", SimTypeInt(signed=False, label="UInt32")), ("ProviderId", SimTypeBottom(label="Guid")), ("dwCatalogEntryId", SimTypeInt(signed=False, label="UInt32")), ("ProtocolChain", SimTypeRef("WSAPROTOCOLCHAIN", SimStruct)), ("iVersion", SimTypeInt(signed=True, label="Int32")), ("iAddressFamily", SimTypeInt(signed=True, label="Int32")), ("iMaxSockAddr", SimTypeInt(signed=True, label="Int32")), ("iMinSockAddr", SimTypeInt(signed=True, label="Int32")), ("iSocketType", SimTypeInt(signed=True, label="Int32")), ("iProtocol", SimTypeInt(signed=True, label="Int32")), ("iProtocolMaxOffset", SimTypeInt(signed=True, label="Int32")), ("iNetworkByteOrder", SimTypeInt(signed=True, label="Int32")), ("iSecurityScheme", SimTypeInt(signed=True, label="Int32")), ("dwMessageSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderReserved", SimTypeInt(signed=False, label="UInt32")), ("szProtocol", SimTypeArray(SimTypeChar(label="SByte"), 256)),)), name="WSAPROTOCOL_INFOA", pack=False, align=None),
    "WSAPROTOCOL_INFOW": SimStruct(OrderedDict((("dwServiceFlags1", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags2", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags3", SimTypeInt(signed=False, label="UInt32")), ("dwServiceFlags4", SimTypeInt(signed=False, label="UInt32")), ("dwProviderFlags", SimTypeInt(signed=False, label="UInt32")), ("ProviderId", SimTypeBottom(label="Guid")), ("dwCatalogEntryId", SimTypeInt(signed=False, label="UInt32")), ("ProtocolChain", SimTypeRef("WSAPROTOCOLCHAIN", SimStruct)), ("iVersion", SimTypeInt(signed=True, label="Int32")), ("iAddressFamily", SimTypeInt(signed=True, label="Int32")), ("iMaxSockAddr", SimTypeInt(signed=True, label="Int32")), ("iMinSockAddr", SimTypeInt(signed=True, label="Int32")), ("iSocketType", SimTypeInt(signed=True, label="Int32")), ("iProtocol", SimTypeInt(signed=True, label="Int32")), ("iProtocolMaxOffset", SimTypeInt(signed=True, label="Int32")), ("iNetworkByteOrder", SimTypeInt(signed=True, label="Int32")), ("iSecurityScheme", SimTypeInt(signed=True, label="Int32")), ("dwMessageSize", SimTypeInt(signed=False, label="UInt32")), ("dwProviderReserved", SimTypeInt(signed=False, label="UInt32")), ("szProtocol", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="WSAPROTOCOL_INFOW", pack=False, align=None),
    "WSAQUERYSET2A": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpszServiceInstanceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpVersion", SimTypePointer(SimTypeRef("WSAVERSION", SimStruct), offset=0)), ("lpszComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("lpNSProviderId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpszContext", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNumberOfProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpafpProtocols", SimTypePointer(SimTypeRef("AFPROTOCOLS", SimStruct), offset=0)), ("lpszQueryString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNumberOfCsAddrs", SimTypeInt(signed=False, label="UInt32")), ("lpcsaBuffer", SimTypePointer(SimTypeRef("CSADDR_INFO", SimStruct), offset=0)), ("dwOutputFlags", SimTypeInt(signed=False, label="UInt32")), ("lpBlob", SimTypePointer(SimTypeRef("BLOB", SimStruct), offset=0)),)), name="WSAQUERYSET2A", pack=False, align=None),
    "WSAQUERYSET2W": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpszServiceInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpVersion", SimTypePointer(SimTypeRef("WSAVERSION", SimStruct), offset=0)), ("lpszComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("lpNSProviderId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpszContext", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNumberOfProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpafpProtocols", SimTypePointer(SimTypeRef("AFPROTOCOLS", SimStruct), offset=0)), ("lpszQueryString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNumberOfCsAddrs", SimTypeInt(signed=False, label="UInt32")), ("lpcsaBuffer", SimTypePointer(SimTypeRef("CSADDR_INFO", SimStruct), offset=0)), ("dwOutputFlags", SimTypeInt(signed=False, label="UInt32")), ("lpBlob", SimTypePointer(SimTypeRef("BLOB", SimStruct), offset=0)),)), name="WSAQUERYSET2W", pack=False, align=None),
    "WSAQUERYSETA": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpszServiceInstanceName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("lpServiceClassId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpVersion", SimTypePointer(SimTypeRef("WSAVERSION", SimStruct), offset=0)), ("lpszComment", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("lpNSProviderId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpszContext", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNumberOfProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpafpProtocols", SimTypePointer(SimTypeRef("AFPROTOCOLS", SimStruct), offset=0)), ("lpszQueryString", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwNumberOfCsAddrs", SimTypeInt(signed=False, label="UInt32")), ("lpcsaBuffer", SimTypePointer(SimTypeRef("CSADDR_INFO", SimStruct), offset=0)), ("dwOutputFlags", SimTypeInt(signed=False, label="UInt32")), ("lpBlob", SimTypePointer(SimTypeRef("BLOB", SimStruct), offset=0)),)), name="WSAQUERYSETA", pack=False, align=None),
    "WSAQUERYSETW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("lpszServiceInstanceName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("lpServiceClassId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpVersion", SimTypePointer(SimTypeRef("WSAVERSION", SimStruct), offset=0)), ("lpszComment", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNameSpace", SimTypeInt(signed=False, label="UInt32")), ("lpNSProviderId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpszContext", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNumberOfProtocols", SimTypeInt(signed=False, label="UInt32")), ("lpafpProtocols", SimTypePointer(SimTypeRef("AFPROTOCOLS", SimStruct), offset=0)), ("lpszQueryString", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwNumberOfCsAddrs", SimTypeInt(signed=False, label="UInt32")), ("lpcsaBuffer", SimTypePointer(SimTypeRef("CSADDR_INFO", SimStruct), offset=0)), ("dwOutputFlags", SimTypeInt(signed=False, label="UInt32")), ("lpBlob", SimTypePointer(SimTypeRef("BLOB", SimStruct), offset=0)),)), name="WSAQUERYSETW", pack=False, align=None),
    "WSASENDMSG": SimStruct(OrderedDict((("lpMsg", SimTypePointer(SimTypeRef("WSAMSG", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("lpNumberOfBytesSent", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("lpOverlapped", SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)), ("lpCompletionRoutine", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)),)), name="WSASENDMSG", pack=False, align=None),
    "WSASERVICECLASSINFOA": SimStruct(OrderedDict((("lpServiceClassId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpszServiceClassName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("lpClassInfos", SimTypePointer(SimTypeRef("WSANSCLASSINFOA", SimStruct), offset=0)),)), name="WSASERVICECLASSINFOA", pack=False, align=None),
    "WSASERVICECLASSINFOW": SimStruct(OrderedDict((("lpServiceClassId", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("lpszServiceClassName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dwCount", SimTypeInt(signed=False, label="UInt32")), ("lpClassInfos", SimTypePointer(SimTypeRef("WSANSCLASSINFOW", SimStruct), offset=0)),)), name="WSASERVICECLASSINFOW", pack=False, align=None),
    "WSATHREADID": SimStruct(OrderedDict((("ThreadHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WSATHREADID", pack=False, align=None),
    "WSAVERSION": SimStruct(OrderedDict((("dwVersion", SimTypeInt(signed=False, label="UInt32")), ("ecHow", SimTypeInt(signed=False, label="WSAECOMPARATOR")),)), name="WSAVERSION", pack=False, align=None),
    "WSA_COMPATIBILITY_BEHAVIOR_ID": SimTypeInt(signed=False, label="WSA_COMPATIBILITY_BEHAVIOR_ID"),
    "WSA_COMPATIBILITY_MODE": SimStruct(OrderedDict((("BehaviorId", SimTypeInt(signed=False, label="WSA_COMPATIBILITY_BEHAVIOR_ID")), ("TargetOsVersion", SimTypeInt(signed=False, label="UInt32")),)), name="WSA_COMPATIBILITY_MODE", pack=False, align=None),
    "WSA_ERROR": SimTypeInt(signed=False, label="WSA_ERROR"),
    "WSB_OB_REGISTRATION_INFO": SimStruct(OrderedDict((("m_wszResourceDLL", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_guidSnapinId", SimTypeBottom(label="Guid")), ("m_dwProviderName", SimTypeInt(signed=False, label="UInt32")), ("m_dwProviderIcon", SimTypeInt(signed=False, label="UInt32")), ("m_bSupportsRemoting", SimTypeChar(label="Byte")),)), name="WSB_OB_REGISTRATION_INFO", pack=False, align=None),
    "WSB_OB_STATUS_ENTRY": SimStruct(OrderedDict((("m_dwIcon", SimTypeInt(signed=False, label="UInt32")), ("m_dwStatusEntryName", SimTypeInt(signed=False, label="UInt32")), ("m_dwStatusEntryValue", SimTypeInt(signed=False, label="UInt32")), ("m_cValueTypePair", SimTypeInt(signed=False, label="UInt32")), ("m_rgValueTypePair", SimTypePointer(SimTypeRef("WSB_OB_STATUS_ENTRY_VALUE_TYPE_PAIR", SimStruct), offset=0)),)), name="WSB_OB_STATUS_ENTRY", pack=False, align=None),
    "WSB_OB_STATUS_ENTRY_PAIR_TYPE": SimTypeInt(signed=False, label="WSB_OB_STATUS_ENTRY_PAIR_TYPE"),
    "WSB_OB_STATUS_ENTRY_VALUE_TYPE_PAIR": SimStruct(OrderedDict((("m_wszObStatusEntryPairValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("m_ObStatusEntryPairType", SimTypeInt(signed=False, label="WSB_OB_STATUS_ENTRY_PAIR_TYPE")),)), name="WSB_OB_STATUS_ENTRY_VALUE_TYPE_PAIR", pack=False, align=None),
    "WSB_OB_STATUS_INFO": SimStruct(OrderedDict((("m_guidSnapinId", SimTypeBottom(label="Guid")), ("m_cStatusEntry", SimTypeInt(signed=False, label="UInt32")), ("m_rgStatusEntry", SimTypePointer(SimTypeRef("WSB_OB_STATUS_ENTRY", SimStruct), offset=0)),)), name="WSB_OB_STATUS_INFO", pack=False, align=None),
    "WSB_PROP": SimTypeInt(signed=False, label="WSB_PROP"),
    "WSC_PROVIDER_AUDIT_INFO": SimStruct(OrderedDict((("RecordSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WSC_PROVIDER_AUDIT_INFO", pack=False, align=None),
    "WSC_PROVIDER_INFO_TYPE": SimTypeInt(signed=False, label="WSC_PROVIDER_INFO_TYPE"),
    "WSC_SECURITY_PRODUCT_STATE": SimTypeInt(signed=False, label="WSC_SECURITY_PRODUCT_STATE"),
    "WSC_SECURITY_PRODUCT_SUBSTATUS": SimTypeInt(signed=False, label="WSC_SECURITY_PRODUCT_SUBSTATUS"),
    "WSC_SECURITY_PROVIDER": SimTypeInt(signed=False, label="WSC_SECURITY_PROVIDER"),
    "WSC_SECURITY_PROVIDER_HEALTH": SimTypeInt(signed=False, label="WSC_SECURITY_PROVIDER_HEALTH"),
    "WSC_SECURITY_SIGNATURE_STATUS": SimTypeInt(signed=False, label="WSC_SECURITY_SIGNATURE_STATUS"),
    "WSDEventType": SimTypeInt(signed=False, label="WSDEventType"),
    "WSDUdpMessageType": SimTypeInt(signed=False, label="WSDUdpMessageType"),
    "WSDUdpRetransmitParams": SimStruct(OrderedDict((("ulSendDelay", SimTypeInt(signed=False, label="UInt32")), ("ulRepeat", SimTypeInt(signed=False, label="UInt32")), ("ulRepeatMinDelay", SimTypeInt(signed=False, label="UInt32")), ("ulRepeatMaxDelay", SimTypeInt(signed=False, label="UInt32")), ("ulRepeatUpperDelay", SimTypeInt(signed=False, label="UInt32")),)), name="WSDUdpRetransmitParams", pack=False, align=None),
    "WSDXML_ATTRIBUTE": SimStruct(OrderedDict((("Element", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)), ("Next", SimTypePointer(SimTypeRef("WSDXML_ATTRIBUTE", SimStruct), offset=0)), ("Name", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("Value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSDXML_ATTRIBUTE", pack=False, align=None),
    "WSDXML_ELEMENT": SimStruct(OrderedDict((("Node", SimTypeRef("WSDXML_NODE", SimStruct)), ("Name", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("FirstAttribute", SimTypePointer(SimTypeRef("WSDXML_ATTRIBUTE", SimStruct), offset=0)), ("FirstChild", SimTypePointer(SimTypeRef("WSDXML_NODE", SimStruct), offset=0)), ("PrefixMappings", SimTypePointer(SimTypeRef("WSDXML_PREFIX_MAPPING", SimStruct), offset=0)),)), name="WSDXML_ELEMENT", pack=False, align=None),
    "WSDXML_ELEMENT_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSDXML_ELEMENT_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSDXML_ELEMENT_LIST", pack=False, align=None),
    "WSDXML_NAME": SimStruct(OrderedDict((("Space", SimTypePointer(SimTypeRef("WSDXML_NAMESPACE", SimStruct), offset=0)), ("LocalName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSDXML_NAME", pack=False, align=None),
    "WSDXML_NAMESPACE": SimStruct(OrderedDict((("Uri", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PreferredPrefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Names", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("NamesCount", SimTypeShort(signed=False, label="UInt16")), ("Encoding", SimTypeShort(signed=False, label="UInt16")),)), name="WSDXML_NAMESPACE", pack=False, align=None),
    "WSDXML_NODE": SimStruct(OrderedDict((("Type", SimTypeInt(signed=True, label="Int32")), ("Parent", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)), ("Next", SimTypePointer(SimTypeRef("WSDXML_NODE", SimStruct), offset=0)),)), name="WSDXML_NODE", pack=False, align=None),
    "WSDXML_OP": SimTypeInt(signed=False, label="WSDXML_OP"),
    "WSDXML_PREFIX_MAPPING": SimStruct(OrderedDict((("Refs", SimTypeInt(signed=False, label="UInt32")), ("Next", SimTypePointer(SimTypeRef("WSDXML_PREFIX_MAPPING", SimStruct), offset=0)), ("Space", SimTypePointer(SimTypeRef("WSDXML_NAMESPACE", SimStruct), offset=0)), ("Prefix", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSDXML_PREFIX_MAPPING", pack=False, align=None),
    "WSDXML_TEXT": SimStruct(OrderedDict((("Node", SimTypeRef("WSDXML_NODE", SimStruct)), ("Text", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSDXML_TEXT", pack=False, align=None),
    "WSDXML_TYPE": SimStruct(OrderedDict((("Uri", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Table", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WSDXML_TYPE", pack=False, align=None),
    "WSD_APP_SEQUENCE": SimStruct(OrderedDict((("InstanceId", SimTypeLongLong(signed=False, label="UInt64")), ("SequenceId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MessageNumber", SimTypeLongLong(signed=False, label="UInt64")),)), name="WSD_APP_SEQUENCE", pack=False, align=None),
    "WSD_BYE": SimStruct(OrderedDict((("EndpointReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_BYE", pack=False, align=None),
    "WSD_CONFIG_ADDRESSES": SimStruct(OrderedDict((("addresses", SimTypePointer(SimTypeBottom(label="IWSDAddress"), offset=0)), ("dwAddressCount", SimTypeInt(signed=False, label="UInt32")),)), name="WSD_CONFIG_ADDRESSES", pack=False, align=None),
    "WSD_CONFIG_PARAM": SimStruct(OrderedDict((("configParamType", SimTypeInt(signed=False, label="WSD_CONFIG_PARAM_TYPE")), ("pConfigData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwConfigDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="WSD_CONFIG_PARAM", pack=False, align=None),
    "WSD_CONFIG_PARAM_TYPE": SimTypeInt(signed=False, label="WSD_CONFIG_PARAM_TYPE"),
    "WSD_DATETIME": SimStruct(OrderedDict((("isPositive", SimTypeInt(signed=True, label="Int32")), ("year", SimTypeInt(signed=False, label="UInt32")), ("month", SimTypeChar(label="Byte")), ("day", SimTypeChar(label="Byte")), ("hour", SimTypeChar(label="Byte")), ("minute", SimTypeChar(label="Byte")), ("second", SimTypeChar(label="Byte")), ("millisecond", SimTypeInt(signed=False, label="UInt32")), ("TZIsLocal", SimTypeInt(signed=True, label="Int32")), ("TZIsPositive", SimTypeInt(signed=True, label="Int32")), ("TZHour", SimTypeChar(label="Byte")), ("TZMinute", SimTypeChar(label="Byte")),)), name="WSD_DATETIME", pack=False, align=None),
    "WSD_DURATION": SimStruct(OrderedDict((("isPositive", SimTypeInt(signed=True, label="Int32")), ("year", SimTypeInt(signed=False, label="UInt32")), ("month", SimTypeInt(signed=False, label="UInt32")), ("day", SimTypeInt(signed=False, label="UInt32")), ("hour", SimTypeInt(signed=False, label="UInt32")), ("minute", SimTypeInt(signed=False, label="UInt32")), ("second", SimTypeInt(signed=False, label="UInt32")), ("millisecond", SimTypeInt(signed=False, label="UInt32")),)), name="WSD_DURATION", pack=False, align=None),
    "WSD_ENDPOINT_REFERENCE": SimStruct(OrderedDict((("Address", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReferenceProperties", SimTypeRef("WSD_REFERENCE_PROPERTIES", SimStruct)), ("ReferenceParameters", SimTypeRef("WSD_REFERENCE_PARAMETERS", SimStruct)), ("PortType", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("ServiceName", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_ENDPOINT_REFERENCE", pack=False, align=None),
    "WSD_ENDPOINT_REFERENCE_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)),)), name="WSD_ENDPOINT_REFERENCE_LIST", pack=False, align=None),
    "WSD_EVENT": SimStruct(OrderedDict((("Hr", SimTypeInt(signed=True, label="Int32")), ("EventType", SimTypeInt(signed=False, label="UInt32")), ("DispatchTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("HandlerContext", SimTypeRef("WSD_HANDLER_CONTEXT", SimStruct)), ("Soap", SimTypePointer(SimTypeRef("WSD_SOAP_MESSAGE", SimStruct), offset=0)), ("Operation", SimTypePointer(SimTypeRef("WSD_OPERATION", SimStruct), offset=0)), ("MessageParameters", SimTypeBottom(label="IWSDMessageParameters")),)), name="WSD_EVENT", pack=False, align=None),
    "WSD_EVENTING_DELIVERY_MODE": SimStruct(OrderedDict((("Mode", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Push", SimTypePointer(SimTypeRef("WSD_EVENTING_DELIVERY_MODE_PUSH", SimStruct), offset=0)), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WSD_EVENTING_DELIVERY_MODE", pack=False, align=None),
    "WSD_EVENTING_DELIVERY_MODE_PUSH": SimStruct(OrderedDict((("NotifyTo", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)),)), name="WSD_EVENTING_DELIVERY_MODE_PUSH", pack=False, align=None),
    "WSD_EVENTING_EXPIRES": SimStruct(OrderedDict((("Duration", SimTypePointer(SimTypeRef("WSD_DURATION", SimStruct), offset=0)), ("DateTime", SimTypePointer(SimTypeRef("WSD_DATETIME", SimStruct), offset=0)),)), name="WSD_EVENTING_EXPIRES", pack=False, align=None),
    "WSD_EVENTING_FILTER": SimStruct(OrderedDict((("Dialect", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("FilterAction", SimTypePointer(SimTypeRef("WSD_EVENTING_FILTER_ACTION", SimStruct), offset=0)), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WSD_EVENTING_FILTER", pack=False, align=None),
    "WSD_EVENTING_FILTER_ACTION": SimStruct(OrderedDict((("Actions", SimTypePointer(SimTypeRef("WSD_URI_LIST", SimStruct), offset=0)),)), name="WSD_EVENTING_FILTER_ACTION", pack=False, align=None),
    "WSD_HANDLER_CONTEXT": SimStruct(OrderedDict((("Handler", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeRef("WSD_EVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["thisUnknown", "event"]), offset=0)), ("PVoid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Unknown", SimTypeBottom(label="IUnknown")),)), name="WSD_HANDLER_CONTEXT", pack=False, align=None),
    "WSD_HEADER_RELATESTO": SimStruct(OrderedDict((("RelationshipType", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("MessageID", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSD_HEADER_RELATESTO", pack=False, align=None),
    "WSD_HELLO": SimStruct(OrderedDict((("EndpointReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Types", SimTypePointer(SimTypeRef("WSD_NAME_LIST", SimStruct), offset=0)), ("Scopes", SimTypePointer(SimTypeRef("WSD_SCOPES", SimStruct), offset=0)), ("XAddrs", SimTypePointer(SimTypeRef("WSD_URI_LIST", SimStruct), offset=0)), ("MetadataVersion", SimTypeLongLong(signed=False, label="UInt64")), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_HELLO", pack=False, align=None),
    "WSD_HOST_METADATA": SimStruct(OrderedDict((("Host", SimTypePointer(SimTypeRef("WSD_SERVICE_METADATA", SimStruct), offset=0)), ("Hosted", SimTypePointer(SimTypeRef("WSD_SERVICE_METADATA_LIST", SimStruct), offset=0)),)), name="WSD_HOST_METADATA", pack=False, align=None),
    "WSD_LOCALIZED_STRING": SimStruct(OrderedDict((("lang", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("String", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSD_LOCALIZED_STRING", pack=False, align=None),
    "WSD_LOCALIZED_STRING_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING", SimStruct), offset=0)),)), name="WSD_LOCALIZED_STRING_LIST", pack=False, align=None),
    "WSD_METADATA_SECTION": SimStruct(OrderedDict((("Dialect", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Identifier", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Data", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MetadataReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Location", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_METADATA_SECTION", pack=False, align=None),
    "WSD_METADATA_SECTION_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_METADATA_SECTION_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSD_METADATA_SECTION", SimStruct), offset=0)),)), name="WSD_METADATA_SECTION_LIST", pack=False, align=None),
    "WSD_NAME_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_NAME_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)),)), name="WSD_NAME_LIST", pack=False, align=None),
    "WSD_OPERATION": SimStruct(OrderedDict((("RequestType", SimTypePointer(SimTypeRef("WSDXML_TYPE", SimStruct), offset=0)), ("ResponseType", SimTypePointer(SimTypeRef("WSDXML_TYPE", SimStruct), offset=0)), ("RequestStubFunction", SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IWSDServiceMessaging"), SimTypePointer(SimTypeRef("WSD_EVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["server", "session", "event"]), offset=0)),)), name="WSD_OPERATION", pack=False, align=None),
    "WSD_PORT_TYPE": SimStruct(OrderedDict((("EncodedName", SimTypeInt(signed=False, label="UInt32")), ("OperationCount", SimTypeInt(signed=False, label="UInt32")), ("Operations", SimTypePointer(SimTypeRef("WSD_OPERATION", SimStruct), offset=0)), ("ProtocolType", SimTypeInt(signed=False, label="WSD_PROTOCOL_TYPE")),)), name="WSD_PORT_TYPE", pack=False, align=None),
    "WSD_PROBE": SimStruct(OrderedDict((("Types", SimTypePointer(SimTypeRef("WSD_NAME_LIST", SimStruct), offset=0)), ("Scopes", SimTypePointer(SimTypeRef("WSD_SCOPES", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_PROBE", pack=False, align=None),
    "WSD_PROBE_MATCH": SimStruct(OrderedDict((("EndpointReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Types", SimTypePointer(SimTypeRef("WSD_NAME_LIST", SimStruct), offset=0)), ("Scopes", SimTypePointer(SimTypeRef("WSD_SCOPES", SimStruct), offset=0)), ("XAddrs", SimTypePointer(SimTypeRef("WSD_URI_LIST", SimStruct), offset=0)), ("MetadataVersion", SimTypeLongLong(signed=False, label="UInt64")), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_PROBE_MATCH", pack=False, align=None),
    "WSD_PROBE_MATCHES": SimStruct(OrderedDict((("ProbeMatch", SimTypePointer(SimTypeRef("WSD_PROBE_MATCH_LIST", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_PROBE_MATCHES", pack=False, align=None),
    "WSD_PROBE_MATCH_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_PROBE_MATCH_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSD_PROBE_MATCH", SimStruct), offset=0)),)), name="WSD_PROBE_MATCH_LIST", pack=False, align=None),
    "WSD_PROTOCOL_TYPE": SimTypeInt(signed=False, label="WSD_PROTOCOL_TYPE"),
    "WSD_REFERENCE_PARAMETERS": SimStruct(OrderedDict((("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_REFERENCE_PARAMETERS", pack=False, align=None),
    "WSD_REFERENCE_PROPERTIES": SimStruct(OrderedDict((("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_REFERENCE_PROPERTIES", pack=False, align=None),
    "WSD_RELATIONSHIP_METADATA": SimStruct(OrderedDict((("Type", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Data", SimTypePointer(SimTypeRef("WSD_HOST_METADATA", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_RELATIONSHIP_METADATA", pack=False, align=None),
    "WSD_RESOLVE": SimStruct(OrderedDict((("EndpointReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_RESOLVE", pack=False, align=None),
    "WSD_RESOLVE_MATCH": SimStruct(OrderedDict((("EndpointReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("Types", SimTypePointer(SimTypeRef("WSD_NAME_LIST", SimStruct), offset=0)), ("Scopes", SimTypePointer(SimTypeRef("WSD_SCOPES", SimStruct), offset=0)), ("XAddrs", SimTypePointer(SimTypeRef("WSD_URI_LIST", SimStruct), offset=0)), ("MetadataVersion", SimTypeLongLong(signed=False, label="UInt64")), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_RESOLVE_MATCH", pack=False, align=None),
    "WSD_RESOLVE_MATCHES": SimStruct(OrderedDict((("ResolveMatch", SimTypePointer(SimTypeRef("WSD_RESOLVE_MATCH", SimStruct), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_RESOLVE_MATCHES", pack=False, align=None),
    "WSD_SCOPES": SimStruct(OrderedDict((("MatchBy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Scopes", SimTypePointer(SimTypeRef("WSD_URI_LIST", SimStruct), offset=0)),)), name="WSD_SCOPES", pack=False, align=None),
    "WSD_SECURITY_CERT_VALIDATION": SimStruct(OrderedDict((("certMatchArray", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("dwCertMatchArrayCount", SimTypeInt(signed=False, label="UInt32")), ("hCertMatchStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCertIssuerStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCertCheckOptions", SimTypeInt(signed=False, label="UInt32")), ("pszCNGHashAlgId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pbCertHash", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwCertHashSize", SimTypeInt(signed=False, label="UInt32")),)), name="WSD_SECURITY_CERT_VALIDATION", pack=False, align=None),
    "WSD_SECURITY_CERT_VALIDATION_V1": SimStruct(OrderedDict((("certMatchArray", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("dwCertMatchArrayCount", SimTypeInt(signed=False, label="UInt32")), ("hCertMatchStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("hCertIssuerStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwCertCheckOptions", SimTypeInt(signed=False, label="UInt32")),)), name="WSD_SECURITY_CERT_VALIDATION_V1", pack=False, align=None),
    "WSD_SECURITY_SIGNATURE_VALIDATION": SimStruct(OrderedDict((("signingCertArray", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("dwSigningCertArrayCount", SimTypeInt(signed=False, label="UInt32")), ("hSigningCertStore", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="WSD_SECURITY_SIGNATURE_VALIDATION", pack=False, align=None),
    "WSD_SERVICE_METADATA": SimStruct(OrderedDict((("EndpointReference", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE_LIST", SimStruct), offset=0)), ("Types", SimTypePointer(SimTypeRef("WSD_NAME_LIST", SimStruct), offset=0)), ("ServiceId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_SERVICE_METADATA", pack=False, align=None),
    "WSD_SERVICE_METADATA_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_SERVICE_METADATA_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeRef("WSD_SERVICE_METADATA", SimStruct), offset=0)),)), name="WSD_SERVICE_METADATA_LIST", pack=False, align=None),
    "WSD_SOAP_FAULT": SimStruct(OrderedDict((("Code", SimTypePointer(SimTypeRef("WSD_SOAP_FAULT_CODE", SimStruct), offset=0)), ("Reason", SimTypePointer(SimTypeRef("WSD_SOAP_FAULT_REASON", SimStruct), offset=0)), ("Node", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Role", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Detail", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_SOAP_FAULT", pack=False, align=None),
    "WSD_SOAP_FAULT_CODE": SimStruct(OrderedDict((("Value", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("Subcode", SimTypePointer(SimTypeRef("WSD_SOAP_FAULT_SUBCODE", SimStruct), offset=0)),)), name="WSD_SOAP_FAULT_CODE", pack=False, align=None),
    "WSD_SOAP_FAULT_REASON": SimStruct(OrderedDict((("Text", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING_LIST", SimStruct), offset=0)),)), name="WSD_SOAP_FAULT_REASON", pack=False, align=None),
    "WSD_SOAP_FAULT_SUBCODE": SimStruct(OrderedDict((("Value", SimTypePointer(SimTypeRef("WSDXML_NAME", SimStruct), offset=0)), ("Subcode", SimTypePointer(SimTypeRef("WSD_SOAP_FAULT_SUBCODE", SimStruct), offset=0)),)), name="WSD_SOAP_FAULT_SUBCODE", pack=False, align=None),
    "WSD_SOAP_HEADER": SimStruct(OrderedDict((("To", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Action", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MessageID", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("RelatesTo", SimTypeRef("WSD_HEADER_RELATESTO", SimStruct)), ("ReplyTo", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("From", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("FaultTo", SimTypePointer(SimTypeRef("WSD_ENDPOINT_REFERENCE", SimStruct), offset=0)), ("AppSequence", SimTypePointer(SimTypeRef("WSD_APP_SEQUENCE", SimStruct), offset=0)), ("AnyHeaders", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_SOAP_HEADER", pack=False, align=None),
    "WSD_SOAP_MESSAGE": SimStruct(OrderedDict((("Header", SimTypeRef("WSD_SOAP_HEADER", SimStruct)), ("Body", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BodyType", SimTypePointer(SimTypeRef("WSDXML_TYPE", SimStruct), offset=0)),)), name="WSD_SOAP_MESSAGE", pack=False, align=None),
    "WSD_STUB_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IWSDServiceMessaging"), SimTypePointer(SimTypeRef("WSD_EVENT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["server", "session", "event"]), offset=0),
    "WSD_SYNCHRONOUS_RESPONSE_CONTEXT": SimStruct(OrderedDict((("hr", SimTypeInt(signed=True, label="Int32")), ("eventHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("messageParameters", SimTypeBottom(label="IWSDMessageParameters")), ("results", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WSD_SYNCHRONOUS_RESPONSE_CONTEXT", pack=False, align=None),
    "WSD_THIS_DEVICE_METADATA": SimStruct(OrderedDict((("FriendlyName", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING_LIST", SimStruct), offset=0)), ("FirmwareVersion", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SerialNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_THIS_DEVICE_METADATA", pack=False, align=None),
    "WSD_THIS_MODEL_METADATA": SimStruct(OrderedDict((("Manufacturer", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING_LIST", SimStruct), offset=0)), ("ManufacturerUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ModelName", SimTypePointer(SimTypeRef("WSD_LOCALIZED_STRING_LIST", SimStruct), offset=0)), ("ModelNumber", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ModelUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PresentationUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_THIS_MODEL_METADATA", pack=False, align=None),
    "WSD_UNKNOWN_LOOKUP": SimStruct(OrderedDict((("Any", SimTypePointer(SimTypeRef("WSDXML_ELEMENT", SimStruct), offset=0)),)), name="WSD_UNKNOWN_LOOKUP", pack=False, align=None),
    "WSD_URI_LIST": SimStruct(OrderedDict((("Next", SimTypePointer(SimTypeRef("WSD_URI_LIST", SimStruct), offset=0)), ("Element", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSD_URI_LIST", pack=False, align=None),
    "WSL_DISTRIBUTION_FLAGS": SimTypeInt(signed=False, label="WSL_DISTRIBUTION_FLAGS"),
    "WSMAN_API_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WSMAN_AUTHENTICATION_CREDENTIALS": SimStruct(OrderedDict((("authenticationMechanism", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"userAccount": SimTypeRef("WSMAN_USERNAME_PASSWORD_CREDS", SimStruct), "certificateThumbprint": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="WSMAN_AUTHENTICATION_CREDENTIALS", pack=False, align=None),
    "WSMAN_AUTHZ_QUOTA": SimStruct(OrderedDict((("maxAllowedConcurrentShells", SimTypeInt(signed=False, label="UInt32")), ("maxAllowedConcurrentOperations", SimTypeInt(signed=False, label="UInt32")), ("timeslotSize", SimTypeInt(signed=False, label="UInt32")), ("maxAllowedOperationsPerTimeslot", SimTypeInt(signed=False, label="UInt32")),)), name="WSMAN_AUTHZ_QUOTA", pack=False, align=None),
    "WSMAN_CERTIFICATE_DETAILS": SimStruct(OrderedDict((("subject", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("issuerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("issuerThumbprint", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("subjectName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_CERTIFICATE_DETAILS", pack=False, align=None),
    "WSMAN_COMMAND_ARG_SET": SimStruct(OrderedDict((("argsCount", SimTypeInt(signed=False, label="UInt32")), ("args", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="WSMAN_COMMAND_ARG_SET", pack=False, align=None),
    "WSMAN_COMMAND_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WSMAN_CONNECT_DATA": SimStruct(OrderedDict((("data", SimTypeRef("WSMAN_DATA", SimStruct)),)), name="WSMAN_CONNECT_DATA", pack=False, align=None),
    "WSMAN_CREATE_SHELL_DATA": SimStruct(OrderedDict((("data", SimTypeRef("WSMAN_DATA", SimStruct)),)), name="WSMAN_CREATE_SHELL_DATA", pack=False, align=None),
    "WSMAN_DATA": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="WSManDataType")), ("Anonymous", SimUnion({"text": SimTypeRef("WSMAN_DATA_TEXT", SimStruct), "binaryData": SimTypeRef("WSMAN_DATA_BINARY", SimStruct), "number": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="WSMAN_DATA", pack=False, align=None),
    "WSMAN_DATA_BINARY": SimStruct(OrderedDict((("dataLength", SimTypeInt(signed=False, label="UInt32")), ("data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WSMAN_DATA_BINARY", pack=False, align=None),
    "WSMAN_DATA_TEXT": SimStruct(OrderedDict((("bufferLength", SimTypeInt(signed=False, label="UInt32")), ("buffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_DATA_TEXT", pack=False, align=None),
    "WSMAN_ENVIRONMENT_VARIABLE": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_ENVIRONMENT_VARIABLE", pack=False, align=None),
    "WSMAN_ENVIRONMENT_VARIABLE_SET": SimStruct(OrderedDict((("varsCount", SimTypeInt(signed=False, label="UInt32")), ("vars", SimTypePointer(SimTypeRef("WSMAN_ENVIRONMENT_VARIABLE", SimStruct), offset=0)),)), name="WSMAN_ENVIRONMENT_VARIABLE_SET", pack=False, align=None),
    "WSMAN_ERROR": SimStruct(OrderedDict((("code", SimTypeInt(signed=False, label="UInt32")), ("errorDetail", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("language", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("machineName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pluginName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_ERROR", pack=False, align=None),
    "WSMAN_FILTER": SimStruct(OrderedDict((("filter", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dialect", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_FILTER", pack=False, align=None),
    "WSMAN_FRAGMENT": SimStruct(OrderedDict((("path", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dialect", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_FRAGMENT", pack=False, align=None),
    "WSMAN_KEY": SimStruct(OrderedDict((("key", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("value", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_KEY", pack=False, align=None),
    "WSMAN_OPERATION_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WSMAN_OPERATION_INFO": SimStruct(OrderedDict((("fragment", SimTypeRef("WSMAN_FRAGMENT", SimStruct)), ("filter", SimTypeRef("WSMAN_FILTER", SimStruct)), ("selectorSet", SimTypeRef("WSMAN_SELECTOR_SET", SimStruct)), ("optionSet", SimTypeRef("WSMAN_OPTION_SET", SimStruct)), ("reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("version", SimTypeInt(signed=False, label="UInt32")),)), name="WSMAN_OPERATION_INFO", pack=False, align=None),
    "WSMAN_OPERATION_INFOEX": SimStruct(OrderedDict((("fragment", SimTypeRef("WSMAN_FRAGMENT", SimStruct)), ("filter", SimTypeRef("WSMAN_FILTER", SimStruct)), ("selectorSet", SimTypeRef("WSMAN_SELECTOR_SET", SimStruct)), ("optionSet", SimTypeRef("WSMAN_OPTION_SETEX", SimStruct)), ("version", SimTypeInt(signed=False, label="UInt32")), ("uiLocale", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("dataLocale", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_OPERATION_INFOEX", pack=False, align=None),
    "WSMAN_OPTION": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("value", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("mustComply", SimTypeInt(signed=True, label="Int32")),)), name="WSMAN_OPTION", pack=False, align=None),
    "WSMAN_OPTION_SET": SimStruct(OrderedDict((("optionsCount", SimTypeInt(signed=False, label="UInt32")), ("options", SimTypePointer(SimTypeRef("WSMAN_OPTION", SimStruct), offset=0)), ("optionsMustUnderstand", SimTypeInt(signed=True, label="Int32")),)), name="WSMAN_OPTION_SET", pack=False, align=None),
    "WSMAN_OPTION_SETEX": SimStruct(OrderedDict((("optionsCount", SimTypeInt(signed=False, label="UInt32")), ("options", SimTypePointer(SimTypeRef("WSMAN_OPTION", SimStruct), offset=0)), ("optionsMustUnderstand", SimTypeInt(signed=True, label="Int32")), ("optionTypes", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="WSMAN_OPTION_SETEX", pack=False, align=None),
    "WSMAN_PLUGIN_AUTHORIZE_OPERATION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pluginContext", "senderDetails", "flags", "operation", "action", "resourceUri"]), offset=0),
    "WSMAN_PLUGIN_AUTHORIZE_QUERY_QUOTA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pluginContext", "senderDetails", "flags"]), offset=0),
    "WSMAN_PLUGIN_AUTHORIZE_RELEASE_CONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["userAuthorizationContext"]), offset=0),
    "WSMAN_PLUGIN_AUTHORIZE_USER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pluginContext", "senderDetails", "flags"]), offset=0),
    "WSMAN_PLUGIN_COMMAND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_COMMAND_ARG_SET", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["requestDetails", "flags", "shellContext", "commandLine", "arguments"]), offset=0),
    "WSMAN_PLUGIN_CONNECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["requestDetails", "flags", "shellContext", "commandContext", "inboundConnectInformation"]), offset=0),
    "WSMAN_PLUGIN_RECEIVE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WSMAN_STREAM_ID_SET", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["requestDetails", "flags", "shellContext", "commandContext", "streamSet"]), offset=0),
    "WSMAN_PLUGIN_RELEASE_COMMAND_CONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["shellContext", "commandContext"]), offset=0),
    "WSMAN_PLUGIN_RELEASE_SHELL_CONTEXT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["shellContext"]), offset=0),
    "WSMAN_PLUGIN_REQUEST": SimStruct(OrderedDict((("senderDetails", SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0)), ("locale", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("resourceUri", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("operationInfo", SimTypePointer(SimTypeRef("WSMAN_OPERATION_INFO", SimStruct), offset=0)), ("shutdownNotification", SimTypeInt(signed=True, label="Int32")), ("shutdownNotificationHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dataLocale", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_PLUGIN_REQUEST", pack=False, align=None),
    "WSMAN_PLUGIN_SEND": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["requestDetails", "flags", "shellContext", "commandContext", "stream", "inboundData"]), offset=0),
    "WSMAN_PLUGIN_SHELL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_STARTUP_INFO_V11", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["pluginContext", "requestDetails", "flags", "startupInfo", "inboundShellInformation"]), offset=0),
    "WSMAN_PLUGIN_SHUTDOWN": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pluginContext", "flags", "reason"]), offset=0),
    "WSMAN_PLUGIN_SIGNAL": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["requestDetails", "flags", "shellContext", "commandContext", "code"]), offset=0),
    "WSMAN_PLUGIN_STARTUP": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["flags", "applicationIdentification", "extraInfo", "pluginContext"]), offset=0),
    "WSMAN_PROXY_INFO": SimStruct(OrderedDict((("accessType", SimTypeInt(signed=False, label="UInt32")), ("authenticationCredentials", SimTypeRef("WSMAN_AUTHENTICATION_CREDENTIALS", SimStruct)),)), name="WSMAN_PROXY_INFO", pack=False, align=None),
    "WSMAN_RECEIVE_DATA_RESULT": SimStruct(OrderedDict((("streamId", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("streamData", SimTypeRef("WSMAN_DATA", SimStruct)), ("commandState", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("exitCode", SimTypeInt(signed=False, label="UInt32")),)), name="WSMAN_RECEIVE_DATA_RESULT", pack=False, align=None),
    "WSMAN_RESPONSE_DATA": SimUnion({"receiveData": SimTypeRef("WSMAN_RECEIVE_DATA_RESULT", SimStruct), "connectData": SimTypeRef("WSMAN_CONNECT_DATA", SimStruct), "createData": SimTypeRef("WSMAN_CREATE_SHELL_DATA", SimStruct)}, name="<anon>", label="None"),
    "WSMAN_SELECTOR_SET": SimStruct(OrderedDict((("numberKeys", SimTypeInt(signed=False, label="UInt32")), ("keys", SimTypePointer(SimTypeRef("WSMAN_KEY", SimStruct), offset=0)),)), name="WSMAN_SELECTOR_SET", pack=False, align=None),
    "WSMAN_SENDER_DETAILS": SimStruct(OrderedDict((("senderName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("authenticationMechanism", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("certificateDetails", SimTypePointer(SimTypeRef("WSMAN_CERTIFICATE_DETAILS", SimStruct), offset=0)), ("clientToken", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("httpURL", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_SENDER_DETAILS", pack=False, align=None),
    "WSMAN_SESSION_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WSMAN_SHELL_ASYNC": SimStruct(OrderedDict((("operationContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("completionFunction", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_ERROR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimUnion({"receiveData": SimTypeRef("WSMAN_RECEIVE_DATA_RESULT", SimStruct), "connectData": SimTypeRef("WSMAN_CONNECT_DATA", SimStruct), "createData": SimTypeRef("WSMAN_CREATE_SHELL_DATA", SimStruct)}, name="<anon>", label="None"), offset=0)], SimTypeBottom(label="Void"), arg_names=["operationContext", "flags", "error", "shell", "command", "operationHandle", "data"]), offset=0)),)), name="WSMAN_SHELL_ASYNC", pack=False, align=None),
    "WSMAN_SHELL_COMPLETION_FUNCTION": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_ERROR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimUnion({"receiveData": SimTypeRef("WSMAN_RECEIVE_DATA_RESULT", SimStruct), "connectData": SimTypeRef("WSMAN_CONNECT_DATA", SimStruct), "createData": SimTypeRef("WSMAN_CREATE_SHELL_DATA", SimStruct)}, name="<anon>", label="None"), offset=0)], SimTypeBottom(label="Void"), arg_names=["operationContext", "flags", "error", "shell", "command", "operationHandle", "data"]), offset=0),
    "WSMAN_SHELL_DISCONNECT_INFO": SimStruct(OrderedDict((("idleTimeoutMs", SimTypeInt(signed=False, label="UInt32")),)), name="WSMAN_SHELL_DISCONNECT_INFO", pack=False, align=None),
    "WSMAN_SHELL_HANDLE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WSMAN_SHELL_STARTUP_INFO_V10": SimStruct(OrderedDict((("inputStreamSet", SimTypePointer(SimTypeRef("WSMAN_STREAM_ID_SET", SimStruct), offset=0)), ("outputStreamSet", SimTypePointer(SimTypeRef("WSMAN_STREAM_ID_SET", SimStruct), offset=0)), ("idleTimeoutMs", SimTypeInt(signed=False, label="UInt32")), ("workingDirectory", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("variableSet", SimTypePointer(SimTypeRef("WSMAN_ENVIRONMENT_VARIABLE_SET", SimStruct), offset=0)),)), name="WSMAN_SHELL_STARTUP_INFO_V10", pack=False, align=None),
    "WSMAN_SHELL_STARTUP_INFO_V11": SimStruct(OrderedDict((("Base", SimTypeRef("WSMAN_SHELL_STARTUP_INFO_V10", SimStruct)), ("name", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_SHELL_STARTUP_INFO_V11", pack=False, align=None),
    "WSMAN_STREAM_ID_SET": SimStruct(OrderedDict((("streamIDsCount", SimTypeInt(signed=False, label="UInt32")), ("streamIDs", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="WSMAN_STREAM_ID_SET", pack=False, align=None),
    "WSMAN_USERNAME_PASSWORD_CREDS": SimStruct(OrderedDict((("username", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("password", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WSMAN_USERNAME_PASSWORD_CREDS", pack=False, align=None),
    "WSMPL": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("usUnityNote", SimTypeShort(signed=False, label="UInt16")), ("sFineTune", SimTypeShort(signed=True, label="Int16")), ("lAttenuation", SimTypeInt(signed=True, label="Int32")), ("fulOptions", SimTypeInt(signed=False, label="UInt32")), ("cSampleLoops", SimTypeInt(signed=False, label="UInt32")),)), name="WSMPL", pack=False, align=None),
    "WSManAuthenticationFlags": SimTypeInt(signed=False, label="WSManAuthenticationFlags"),
    "WSManCallbackFlags": SimTypeInt(signed=False, label="WSManCallbackFlags"),
    "WSManDataType": SimTypeInt(signed=False, label="WSManDataType"),
    "WSManEnumFlags": SimTypeInt(signed=False, label="WSManEnumFlags"),
    "WSManProxyAccessType": SimTypeInt(signed=False, label="WSManProxyAccessType"),
    "WSManProxyAccessTypeFlags": SimTypeInt(signed=False, label="WSManProxyAccessTypeFlags"),
    "WSManProxyAuthenticationFlags": SimTypeInt(signed=False, label="WSManProxyAuthenticationFlags"),
    "WSManSessionFlags": SimTypeInt(signed=False, label="WSManSessionFlags"),
    "WSManSessionOption": SimTypeInt(signed=False, label="WSManSessionOption"),
    "WSManShellFlag": SimTypeInt(signed=False, label="WSManShellFlag"),
    "WSPDATA": SimStruct(OrderedDict((("wVersion", SimTypeShort(signed=False, label="UInt16")), ("wHighVersion", SimTypeShort(signed=False, label="UInt16")), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="WSPDATA", pack=False, align=None),
    "WSPPROC_TABLE": SimStruct(OrderedDict((("lpWSPAccept", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpCallerId", "lpCallerData", "lpSQOS", "lpGQOS", "lpCalleeId", "lpCalleeData", "g", "dwCallbackData"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["s", "addr", "addrlen", "lpfnCondition", "dwCallbackData", "lpErrno"]), offset=0)), ("lpWSPAddressToString", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpsaAddress", "dwAddressLength", "lpProtocolInfo", "lpszAddressString", "lpdwAddressStringLength", "lpErrno"]), offset=0)), ("lpWSPAsyncSelect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "hWnd", "wMsg", "lEvent", "lpErrno"]), offset=0)), ("lpWSPBind", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpErrno"]), offset=0)), ("lpWSPCancelBlockingCall", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpErrno"]), offset=0)), ("lpWSPCleanup", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpErrno"]), offset=0)), ("lpWSPCloseSocket", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpErrno"]), offset=0)), ("lpWSPConnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpCallerData", "lpCalleeData", "lpSQOS", "lpGQOS", "lpErrno"]), offset=0)), ("lpWSPDuplicateSocket", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "dwProcessId", "lpProtocolInfo", "lpErrno"]), offset=0)), ("lpWSPEnumNetworkEvents", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("WSANETWORKEVENTS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "hEventObject", "lpNetworkEvents", "lpErrno"]), offset=0)), ("lpWSPEventSelect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "hEventObject", "lNetworkEvents", "lpErrno"]), offset=0)), ("lpWSPGetOverlappedResult", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpOverlapped", "lpcbTransfer", "fWait", "lpdwFlags", "lpErrno"]), offset=0)), ("lpWSPGetPeerName", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpErrno"]), offset=0)), ("lpWSPGetSockName", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "name", "namelen", "lpErrno"]), offset=0)), ("lpWSPGetSockOpt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "level", "optname", "optval", "optlen", "lpErrno"]), offset=0)), ("lpWSPGetQOSByName", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpQOSName", "lpQOS", "lpErrno"]), offset=0)), ("lpWSPIoctl", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "dwIoControlCode", "lpvInBuffer", "cbInBuffer", "lpvOutBuffer", "cbOutBuffer", "lpcbBytesReturned", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0)), ("lpWSPJoinLeaf", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypePointer(SimTypeRef("QOS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["s", "name", "namelen", "lpCallerData", "lpCalleeData", "lpSQOS", "lpGQOS", "dwFlags", "lpErrno"]), offset=0)), ("lpWSPListen", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "backlog", "lpErrno"]), offset=0)), ("lpWSPRecv", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesRecvd", "lpFlags", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0)), ("lpWSPRecvDisconnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpInboundDisconnectData", "lpErrno"]), offset=0)), ("lpWSPRecvFrom", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesRecvd", "lpFlags", "lpFrom", "lpFromlen", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0)), ("lpWSPSelect", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("TIMEVAL", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["nfds", "readfds", "writefds", "exceptfds", "timeout", "lpErrno"]), offset=0)), ("lpWSPSend", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesSent", "dwFlags", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0)), ("lpWSPSendDisconnect", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpOutboundDisconnectData", "lpErrno"]), offset=0)), ("lpWSPSendTo", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("WSABUF", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0), SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpBuffers", "dwBufferCount", "lpNumberOfBytesSent", "dwFlags", "lpTo", "iTolen", "lpOverlapped", "lpCompletionRoutine", "lpThreadId", "lpErrno"]), offset=0)), ("lpWSPSetSockOpt", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "level", "optname", "optval", "optlen", "lpErrno"]), offset=0)), ("lpWSPShutdown", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "how", "lpErrno"]), offset=0)), ("lpWSPSocket", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["af", "type", "protocol", "lpProtocolInfo", "g", "dwFlags", "lpErrno"]), offset=0)), ("lpWSPStringToAddress", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSAPROTOCOL_INFOW", SimStruct), offset=0), SimTypePointer(SimTypeRef("SOCKADDR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["AddressString", "AddressFamily", "lpProtocolInfo", "lpAddress", "lpAddressLength", "lpErrno"]), offset=0)),)), name="WSPPROC_TABLE", pack=False, align=None),
    "WSPUPCALLTABLE": SimStruct(OrderedDict((("lpWPUCloseEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "lpErrno"]), offset=0)), ("lpWPUCloseSocketHandle", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpErrno"]), offset=0)), ("lpWPUCreateEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["lpErrno"]), offset=0)), ("lpWPUCreateSocketHandle", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["dwCatalogEntryId", "dwContext", "lpErrno"]), offset=0)), ("lpWPUFDIsSet", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeRef("FD_SET", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "fdset"]), offset=0)), ("lpWPUGetProviderPath", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpProviderId", "lpszProviderDllPath", "lpProviderDllPathLen", "lpErrno"]), offset=0)), ("lpWPUModifyIFSHandle", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["dwCatalogEntryId", "ProposedHandle", "lpErrno"]), offset=0)), ("lpWPUPostMessage", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "Msg", "wParam", "lParam"]), offset=0)), ("lpWPUQueryBlockingCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwContext"]), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCatalogEntryId", "lplpfnCallback", "lpdwContext", "lpErrno"]), offset=0)), ("lpWPUQuerySocketHandleContext", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["s", "lpContext", "lpErrno"]), offset=0)), ("lpWPUQueueApc", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["dwContext"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpThreadId", "lpfnUserApc", "dwContext", "lpErrno"]), offset=0)), ("lpWPUResetEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "lpErrno"]), offset=0)), ("lpWPUSetEvent", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "lpErrno"]), offset=0)), ("lpWPUOpenCurrentThread", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpThreadId", "lpErrno"]), offset=0)), ("lpWPUCloseThread", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WSATHREADID", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpThreadId", "lpErrno"]), offset=0)),)), name="WSPUPCALLTABLE", pack=False, align=None),
    "WST_BUFFER": SimStruct(OrderedDict((("ScanlinesRequested", SimTypeRef("VBICODECFILTERING_SCANLINES", SimStruct)), ("WstLines", SimTypeArray(SimTypeRef("WST_BUFFER_LINE", SimStruct), 17)),)), name="WST_BUFFER", pack=False, align=None),
    "WST_BUFFER_LINE": SimStruct(OrderedDict((("Confidence", SimTypeChar(label="Byte")), ("Bytes", SimTypeArray(SimTypeChar(label="Byte"), 42)),)), name="WST_BUFFER_LINE", pack=False, align=None),
    "WS_ABANDON_MESSAGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "error"]), offset=0),
    "WS_ABORT_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "error"]), offset=0),
    "WS_ABORT_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "error"]), offset=0),
    "WS_ACCEPT_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "channelInstance", "asyncContext", "error"]), offset=0),
    "WS_ADDRESSING_VERSION": SimTypeInt(signed=False, label="WS_ADDRESSING_VERSION"),
    "WS_ANY_ATTRIBUTE": SimStruct(OrderedDict((("localName", SimTypeRef("WS_XML_STRING", SimStruct)), ("ns", SimTypeRef("WS_XML_STRING", SimStruct)), ("value", SimTypePointer(SimTypeRef("WS_XML_TEXT", SimStruct), offset=0)),)), name="WS_ANY_ATTRIBUTE", pack=False, align=None),
    "WS_ANY_ATTRIBUTES": SimStruct(OrderedDict((("attributes", SimTypePointer(SimTypeRef("WS_ANY_ATTRIBUTE", SimStruct), offset=0)), ("attributeCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_ANY_ATTRIBUTES", pack=False, align=None),
    "WS_ASYNC_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="WS_CALLBACK_MODEL"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["errorCode", "callbackModel", "callbackState"]), offset=0),
    "WS_ASYNC_CONTEXT": SimStruct(OrderedDict((("callback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="WS_CALLBACK_MODEL"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["errorCode", "callbackModel", "callbackState"]), offset=0)), ("callbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_ASYNC_CONTEXT", pack=False, align=None),
    "WS_ASYNC_FUNCTION": SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="WS_CALLBACK_MODEL"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_OPERATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hr", "callbackModel", "callbackState", "next", "asyncContext", "error"]), offset=0),
    "WS_ASYNC_OPERATION": SimStruct(OrderedDict((("function", SimTypePointer(SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="WS_CALLBACK_MODEL"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_OPERATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hr", "callbackModel", "callbackState", "next", "asyncContext", "error"]), offset=0)),)), name="WS_ASYNC_OPERATION", pack=False, align=None),
    "WS_ASYNC_STATE": SimStruct(OrderedDict((("internal0", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("internal1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("internal2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("internal3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("internal4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_ASYNC_STATE", pack=False, align=None),
    "WS_ATTRIBUTE_DESCRIPTION": SimStruct(OrderedDict((("attributeLocalName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("attributeNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("type", SimTypeInt(signed=False, label="WS_TYPE")), ("typeDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_ATTRIBUTE_DESCRIPTION", pack=False, align=None),
    "WS_BINDING_TEMPLATE_TYPE": SimTypeInt(signed=False, label="WS_BINDING_TEMPLATE_TYPE"),
    "WS_BOOL_DESCRIPTION": SimStruct(OrderedDict((("value", SimTypeInt(signed=True, label="Int32")),)), name="WS_BOOL_DESCRIPTION", pack=False, align=None),
    "WS_BUFFERS": SimStruct(OrderedDict((("bufferCount", SimTypeInt(signed=False, label="UInt32")), ("buffers", SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), offset=0)),)), name="WS_BUFFERS", pack=False, align=None),
    "WS_BYTES": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("bytes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WS_BYTES", pack=False, align=None),
    "WS_BYTES_DESCRIPTION": SimStruct(OrderedDict((("minByteCount", SimTypeInt(signed=False, label="UInt32")), ("maxByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_BYTES_DESCRIPTION", pack=False, align=None),
    "WS_BYTE_ARRAY_DESCRIPTION": SimStruct(OrderedDict((("minByteCount", SimTypeInt(signed=False, label="UInt32")), ("maxByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_BYTE_ARRAY_DESCRIPTION", pack=False, align=None),
    "WS_CALLBACK_MODEL": SimTypeInt(signed=False, label="WS_CALLBACK_MODEL"),
    "WS_CALL_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_CALL_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_CALL_PROPERTY", pack=False, align=None),
    "WS_CALL_PROPERTY_ID": SimTypeInt(signed=False, label="WS_CALL_PROPERTY_ID"),
    "WS_CAPI_ASYMMETRIC_SECURITY_KEY_HANDLE": SimStruct(OrderedDict((("keyHandle", SimTypeRef("WS_SECURITY_KEY_HANDLE", SimStruct)), ("provider", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("keySpec", SimTypeInt(signed=False, label="UInt32")),)), name="WS_CAPI_ASYMMETRIC_SECURITY_KEY_HANDLE", pack=False, align=None),
    "WS_CERTIFICATE_VALIDATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["certContext", "state"]), offset=0),
    "WS_CERTIFICATE_VALIDATION_CALLBACK_CONTEXT": SimStruct(OrderedDict((("callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["certContext", "state"]), offset=0)), ("state", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_CERTIFICATE_VALIDATION_CALLBACK_CONTEXT", pack=False, align=None),
    "WS_CERT_CREDENTIAL": SimStruct(OrderedDict((("credentialType", SimTypeInt(signed=False, label="WS_CERT_CREDENTIAL_TYPE")),)), name="WS_CERT_CREDENTIAL", pack=False, align=None),
    "WS_CERT_CREDENTIAL_TYPE": SimTypeInt(signed=False, label="WS_CERT_CREDENTIAL_TYPE"),
    "WS_CERT_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identity", SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct)), ("rawCertificateData", SimTypeRef("WS_BYTES", SimStruct)),)), name="WS_CERT_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_CERT_ISSUER_LIST_NOTIFICATION_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecPkgContext_IssuerListInfoEx", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["certIssuerListNotificationCallbackState", "issuerList", "error"]), offset=0),
    "WS_CERT_MESSAGE_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")),)), name="WS_CERT_MESSAGE_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_CERT_SIGNED_SAML_AUTHENTICATOR": SimStruct(OrderedDict((("authenticator", SimTypeRef("WS_SAML_AUTHENTICATOR", SimStruct)), ("trustedIssuerCerts", SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0)), ("trustedIssuerCertCount", SimTypeInt(signed=False, label="UInt32")), ("decryptionCert", SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0)), ("samlValidator", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["samlValidatorCallbackState", "samlAssertion", "error"]), offset=0)), ("samlValidatorCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_CERT_SIGNED_SAML_AUTHENTICATOR", pack=False, align=None),
    "WS_CHANNEL": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_CHANNEL_BINDING": SimTypeInt(signed=False, label="WS_CHANNEL_BINDING"),
    "WS_CHANNEL_DECODER": SimStruct(OrderedDict((("createContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("createDecoderCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "bytes", "maxSize", "actualSize", "asyncContext", "error"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["createContext", "readCallback", "readContext", "decoderContext", "error"]), offset=0)), ("decoderGetContentTypeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["decoderContext", "contentType", "contentEncoding", "newContentType", "error"]), offset=0)), ("decoderStartCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0)), ("decoderDecodeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "buffer", "maxLength", "length", "asyncContext", "error"]), offset=0)), ("decoderEndCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0)), ("freeDecoderCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["decoderContext"]), offset=0)),)), name="WS_CHANNEL_DECODER", pack=False, align=None),
    "WS_CHANNEL_ENCODER": SimStruct(OrderedDict((("createContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("createEncoderCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "buffers", "count", "asyncContext", "error"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["createContext", "writeCallback", "writeContext", "encoderContext", "error"]), offset=0)), ("encoderGetContentTypeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "contentType", "newContentType", "contentEncoding", "error"]), offset=0)), ("encoderStartCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0)), ("encoderEncodeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "buffers", "count", "asyncContext", "error"]), offset=0)), ("encoderEndCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0)), ("freeEncoderCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["encoderContext"]), offset=0)),)), name="WS_CHANNEL_ENCODER", pack=False, align=None),
    "WS_CHANNEL_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_CHANNEL_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_CHANNEL_PROPERTIES", pack=False, align=None),
    "WS_CHANNEL_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_CHANNEL_PROPERTY", pack=False, align=None),
    "WS_CHANNEL_PROPERTY_CONSTRAINT": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID")), ("allowedValues", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("allowedValuesSize", SimTypeInt(signed=False, label="UInt32")), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_CHANNEL_PROPERTY_CONSTRAINT", pack=False, align=None),
    "WS_CHANNEL_PROPERTY_ID": SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID"),
    "WS_CHANNEL_STATE": SimTypeInt(signed=False, label="WS_CHANNEL_STATE"),
    "WS_CHANNEL_TYPE": SimTypeInt(signed=False, label="WS_CHANNEL_TYPE"),
    "WS_CHARSET": SimTypeInt(signed=False, label="WS_CHARSET"),
    "WS_CHAR_ARRAY_DESCRIPTION": SimStruct(OrderedDict((("minCharCount", SimTypeInt(signed=False, label="UInt32")), ("maxCharCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_CHAR_ARRAY_DESCRIPTION", pack=False, align=None),
    "WS_CLOSE_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "asyncContext", "error"]), offset=0),
    "WS_CLOSE_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "asyncContext", "error"]), offset=0),
    "WS_CONTRACT_DESCRIPTION": SimStruct(OrderedDict((("operationCount", SimTypeInt(signed=False, label="UInt32")), ("operations", SimTypePointer(SimTypePointer(SimTypeRef("WS_OPERATION_DESCRIPTION", SimStruct), offset=0), offset=0)),)), name="WS_CONTRACT_DESCRIPTION", pack=False, align=None),
    "WS_COOKIE_MODE": SimTypeInt(signed=False, label="WS_COOKIE_MODE"),
    "WS_CREATE_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WS_CHANNEL_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelType", "channelParameters", "channelParametersSize", "channelInstance", "error"]), offset=0),
    "WS_CREATE_CHANNEL_FOR_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "channelParameters", "channelParametersSize", "channelInstance", "error"]), offset=0),
    "WS_CREATE_DECODER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "bytes", "maxSize", "actualSize", "asyncContext", "error"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["createContext", "readCallback", "readContext", "decoderContext", "error"]), offset=0),
    "WS_CREATE_ENCODER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "buffers", "count", "asyncContext", "error"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["createContext", "writeCallback", "writeContext", "encoderContext", "error"]), offset=0),
    "WS_CREATE_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WS_CHANNEL_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelType", "listenerParameters", "listenerParametersSize", "listenerInstance", "error"]), offset=0),
    "WS_CUSTOM_CERT_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_CERT_CREDENTIAL", SimStruct)), ("getCertCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ENDPOINT_ADDRESS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["getCertCallbackState", "targetAddress", "viaUri", "cert", "error"]), offset=0)), ("getCertCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("certIssuerListNotificationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("SecPkgContext_IssuerListInfoEx", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["certIssuerListNotificationCallbackState", "issuerList", "error"]), offset=0)), ("certIssuerListNotificationCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_CUSTOM_CERT_CREDENTIAL", pack=False, align=None),
    "WS_CUSTOM_CHANNEL_CALLBACKS": SimStruct(OrderedDict((("createChannelCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WS_CHANNEL_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelType", "channelParameters", "channelParametersSize", "channelInstance", "error"]), offset=0)), ("freeChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["channelInstance"]), offset=0)), ("resetChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "error"]), offset=0)), ("openChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ENDPOINT_ADDRESS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "endpointAddress", "asyncContext", "error"]), offset=0)), ("closeChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "asyncContext", "error"]), offset=0)), ("abortChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "error"]), offset=0)), ("getChannelPropertyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "id", "value", "valueSize", "error"]), offset=0)), ("setChannelPropertyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "id", "value", "valueSize", "error"]), offset=0)), ("writeMessageStartCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0)), ("writeMessageEndCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0)), ("readMessageStartCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0)), ("readMessageEndCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0)), ("abandonMessageCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "error"]), offset=0)), ("shutdownSessionChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "asyncContext", "error"]), offset=0)),)), name="WS_CUSTOM_CHANNEL_CALLBACKS", pack=False, align=None),
    "WS_CUSTOM_HTTP_PROXY": SimStruct(OrderedDict((("servers", SimTypeRef("WS_STRING", SimStruct)), ("bypass", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_CUSTOM_HTTP_PROXY", pack=False, align=None),
    "WS_CUSTOM_LISTENER_CALLBACKS": SimStruct(OrderedDict((("createListenerCallback", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WS_CHANNEL_TYPE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelType", "listenerParameters", "listenerParametersSize", "listenerInstance", "error"]), offset=0)), ("freeListenerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["listenerInstance"]), offset=0)), ("resetListenerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "error"]), offset=0)), ("openListenerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "url", "asyncContext", "error"]), offset=0)), ("closeListenerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "asyncContext", "error"]), offset=0)), ("abortListenerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "error"]), offset=0)), ("getListenerPropertyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_LISTENER_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "id", "value", "valueSize", "error"]), offset=0)), ("setListenerPropertyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_LISTENER_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "id", "value", "valueSize", "error"]), offset=0)), ("createChannelForListenerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "channelParameters", "channelParametersSize", "channelInstance", "error"]), offset=0)), ("acceptChannelCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "channelInstance", "asyncContext", "error"]), offset=0)),)), name="WS_CUSTOM_LISTENER_CALLBACKS", pack=False, align=None),
    "WS_CUSTOM_TYPE_DESCRIPTION": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("alignment", SimTypeInt(signed=False, label="UInt32")), ("readCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="WS_TYPE_MAPPING"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["reader", "typeMapping", "descriptionData", "heap", "value", "valueSize", "error"]), offset=0)), ("writeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="WS_TYPE_MAPPING"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["writer", "typeMapping", "descriptionData", "value", "valueSize", "error"]), offset=0)), ("descriptionData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("isDefaultValueCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["descriptionData", "value", "defaultValue", "valueSize", "isDefault", "error"]), offset=0)),)), name="WS_CUSTOM_TYPE_DESCRIPTION", pack=False, align=None),
    "WS_DATETIME": SimStruct(OrderedDict((("ticks", SimTypeLongLong(signed=False, label="UInt64")), ("format", SimTypeInt(signed=False, label="WS_DATETIME_FORMAT")),)), name="WS_DATETIME", pack=False, align=None),
    "WS_DATETIME_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeRef("WS_DATETIME", SimStruct)), ("maxValue", SimTypeRef("WS_DATETIME", SimStruct)),)), name="WS_DATETIME_DESCRIPTION", pack=False, align=None),
    "WS_DATETIME_FORMAT": SimTypeInt(signed=False, label="WS_DATETIME_FORMAT"),
    "WS_DECIMAL_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeRef("DECIMAL", SimStruct)), ("maxValue", SimTypeRef("DECIMAL", SimStruct)),)), name="WS_DECIMAL_DESCRIPTION", pack=False, align=None),
    "WS_DECODER_DECODE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "buffer", "maxLength", "length", "asyncContext", "error"]), offset=0),
    "WS_DECODER_END_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0),
    "WS_DECODER_GET_CONTENT_TYPE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["decoderContext", "contentType", "contentEncoding", "newContentType", "error"]), offset=0),
    "WS_DECODER_START_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0),
    "WS_DEFAULT_VALUE": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_DEFAULT_VALUE", pack=False, align=None),
    "WS_DEFAULT_WINDOWS_INTEGRATED_AUTH_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct)),)), name="WS_DEFAULT_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", pack=False, align=None),
    "WS_DISALLOWED_USER_AGENT_SUBSTRINGS": SimStruct(OrderedDict((("subStringCount", SimTypeInt(signed=False, label="UInt32")), ("subStrings", SimTypePointer(SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), offset=0)),)), name="WS_DISALLOWED_USER_AGENT_SUBSTRINGS", pack=False, align=None),
    "WS_DNS_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identity", SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct)), ("dns", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_DNS_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_DOUBLE_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeFloat(size=64)), ("maxValue", SimTypeFloat(size=64)),)), name="WS_DOUBLE_DESCRIPTION", pack=False, align=None),
    "WS_DURATION": SimStruct(OrderedDict((("negative", SimTypeInt(signed=True, label="Int32")), ("years", SimTypeInt(signed=False, label="UInt32")), ("months", SimTypeInt(signed=False, label="UInt32")), ("days", SimTypeInt(signed=False, label="UInt32")), ("hours", SimTypeInt(signed=False, label="UInt32")), ("minutes", SimTypeInt(signed=False, label="UInt32")), ("seconds", SimTypeInt(signed=False, label="UInt32")), ("milliseconds", SimTypeInt(signed=False, label="UInt32")), ("ticks", SimTypeInt(signed=False, label="UInt32")),)), name="WS_DURATION", pack=False, align=None),
    "WS_DURATION_COMPARISON_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WS_DURATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_DURATION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["duration1", "duration2", "result", "error"]), offset=0),
    "WS_DURATION_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeRef("WS_DURATION", SimStruct)), ("maxValue", SimTypeRef("WS_DURATION", SimStruct)), ("comparer", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WS_DURATION", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_DURATION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["duration1", "duration2", "result", "error"]), offset=0)),)), name="WS_DURATION_DESCRIPTION", pack=False, align=None),
    "WS_DYNAMIC_STRING_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "string", "found", "id", "error"]), offset=0),
    "WS_ELEMENT_DESCRIPTION": SimStruct(OrderedDict((("elementLocalName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("elementNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("type", SimTypeInt(signed=False, label="WS_TYPE")), ("typeDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_ELEMENT_DESCRIPTION", pack=False, align=None),
    "WS_ENCODER_ENCODE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "buffers", "count", "asyncContext", "error"]), offset=0),
    "WS_ENCODER_END_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0),
    "WS_ENCODER_GET_CONTENT_TYPE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "contentType", "newContentType", "contentEncoding", "error"]), offset=0),
    "WS_ENCODER_START_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["encoderContext", "asyncContext", "error"]), offset=0),
    "WS_ENCODING": SimTypeInt(signed=False, label="WS_ENCODING"),
    "WS_ENDPOINT_ADDRESS": SimStruct(OrderedDict((("url", SimTypeRef("WS_STRING", SimStruct)), ("headers", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("extensions", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("identity", SimTypePointer(SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct), offset=0)),)), name="WS_ENDPOINT_ADDRESS", pack=False, align=None),
    "WS_ENDPOINT_ADDRESS_DESCRIPTION": SimStruct(OrderedDict((("addressingVersion", SimTypeInt(signed=False, label="WS_ADDRESSING_VERSION")),)), name="WS_ENDPOINT_ADDRESS_DESCRIPTION", pack=False, align=None),
    "WS_ENDPOINT_ADDRESS_EXTENSION_TYPE": SimTypeInt(signed=False, label="WS_ENDPOINT_ADDRESS_EXTENSION_TYPE"),
    "WS_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identityType", SimTypeInt(signed=False, label="WS_ENDPOINT_IDENTITY_TYPE")),)), name="WS_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_ENDPOINT_IDENTITY_TYPE": SimTypeInt(signed=False, label="WS_ENDPOINT_IDENTITY_TYPE"),
    "WS_ENDPOINT_POLICY_EXTENSION": SimStruct(OrderedDict((("policyExtension", SimTypeRef("WS_POLICY_EXTENSION", SimStruct)), ("assertionName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("assertionNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_ENDPOINT_POLICY_EXTENSION", pack=False, align=None),
    "WS_ENUM_DESCRIPTION": SimStruct(OrderedDict((("values", SimTypePointer(SimTypeRef("WS_ENUM_VALUE", SimStruct), offset=0)), ("valueCount", SimTypeInt(signed=False, label="UInt32")), ("maxByteCount", SimTypeInt(signed=False, label="UInt32")), ("nameIndices", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="WS_ENUM_DESCRIPTION", pack=False, align=None),
    "WS_ENUM_VALUE": SimStruct(OrderedDict((("value", SimTypeInt(signed=True, label="Int32")), ("name", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)),)), name="WS_ENUM_VALUE", pack=False, align=None),
    "WS_ENVELOPE_VERSION": SimTypeInt(signed=False, label="WS_ENVELOPE_VERSION"),
    "WS_ERROR": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_ERROR_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_ERROR_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_ERROR_PROPERTY", pack=False, align=None),
    "WS_ERROR_PROPERTY_ID": SimTypeInt(signed=False, label="WS_ERROR_PROPERTY_ID"),
    "WS_EXCEPTION_CODE": SimTypeInt(signed=False, label="WS_EXCEPTION_CODE"),
    "WS_EXTENDED_PROTECTION_POLICY": SimTypeInt(signed=False, label="WS_EXTENDED_PROTECTION_POLICY"),
    "WS_EXTENDED_PROTECTION_SCENARIO": SimTypeInt(signed=False, label="WS_EXTENDED_PROTECTION_SCENARIO"),
    "WS_FAULT": SimStruct(OrderedDict((("code", SimTypePointer(SimTypeRef("WS_FAULT_CODE", SimStruct), offset=0)), ("reasons", SimTypePointer(SimTypeRef("WS_FAULT_REASON", SimStruct), offset=0)), ("reasonCount", SimTypeInt(signed=False, label="UInt32")), ("actor", SimTypeRef("WS_STRING", SimStruct)), ("node", SimTypeRef("WS_STRING", SimStruct)), ("detail", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="WS_FAULT", pack=False, align=None),
    "WS_FAULT_CODE": SimStruct(OrderedDict((("value", SimTypeRef("WS_XML_QNAME", SimStruct)), ("subCode", SimTypePointer(SimTypeRef("WS_FAULT_CODE", SimStruct), offset=0)),)), name="WS_FAULT_CODE", pack=False, align=None),
    "WS_FAULT_DESCRIPTION": SimStruct(OrderedDict((("envelopeVersion", SimTypeInt(signed=False, label="WS_ENVELOPE_VERSION")),)), name="WS_FAULT_DESCRIPTION", pack=False, align=None),
    "WS_FAULT_DETAIL_DESCRIPTION": SimStruct(OrderedDict((("action", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("detailElementDescription", SimTypePointer(SimTypeRef("WS_ELEMENT_DESCRIPTION", SimStruct), offset=0)),)), name="WS_FAULT_DETAIL_DESCRIPTION", pack=False, align=None),
    "WS_FAULT_DISCLOSURE": SimTypeInt(signed=False, label="WS_FAULT_DISCLOSURE"),
    "WS_FAULT_ERROR_PROPERTY_ID": SimTypeInt(signed=False, label="WS_FAULT_ERROR_PROPERTY_ID"),
    "WS_FAULT_REASON": SimStruct(OrderedDict((("text", SimTypeRef("WS_STRING", SimStruct)), ("lang", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_FAULT_REASON", pack=False, align=None),
    "WS_FIELD_DESCRIPTION": SimStruct(OrderedDict((("mapping", SimTypeInt(signed=False, label="WS_FIELD_MAPPING")), ("localName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("ns", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("type", SimTypeInt(signed=False, label="WS_TYPE")), ("typeDescription", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("offset", SimTypeInt(signed=False, label="UInt32")), ("options", SimTypeInt(signed=False, label="UInt32")), ("defaultValue", SimTypePointer(SimTypeRef("WS_DEFAULT_VALUE", SimStruct), offset=0)), ("countOffset", SimTypeInt(signed=False, label="UInt32")), ("itemLocalName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("itemNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("itemRange", SimTypePointer(SimTypeRef("WS_ITEM_RANGE", SimStruct), offset=0)),)), name="WS_FIELD_DESCRIPTION", pack=False, align=None),
    "WS_FIELD_MAPPING": SimTypeInt(signed=False, label="WS_FIELD_MAPPING"),
    "WS_FLOAT_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeFloat(size=32)), ("maxValue", SimTypeFloat(size=32)),)), name="WS_FLOAT_DESCRIPTION", pack=False, align=None),
    "WS_FREE_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["channelInstance"]), offset=0),
    "WS_FREE_DECODER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["decoderContext"]), offset=0),
    "WS_FREE_ENCODER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["encoderContext"]), offset=0),
    "WS_FREE_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["listenerInstance"]), offset=0),
    "WS_GET_CERT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ENDPOINT_ADDRESS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("CERT_CONTEXT", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["getCertCallbackState", "targetAddress", "viaUri", "cert", "error"]), offset=0),
    "WS_GET_CHANNEL_PROPERTY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "id", "value", "valueSize", "error"]), offset=0),
    "WS_GET_LISTENER_PROPERTY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_LISTENER_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "id", "value", "valueSize", "error"]), offset=0),
    "WS_GUID_DESCRIPTION": SimStruct(OrderedDict((("value", SimTypeBottom(label="Guid")),)), name="WS_GUID_DESCRIPTION", pack=False, align=None),
    "WS_HEADER_TYPE": SimTypeInt(signed=False, label="WS_HEADER_TYPE"),
    "WS_HEAP": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_HEAP_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_HEAP_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_HEAP_PROPERTIES", pack=False, align=None),
    "WS_HEAP_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_HEAP_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_HEAP_PROPERTY", pack=False, align=None),
    "WS_HEAP_PROPERTY_ID": SimTypeInt(signed=False, label="WS_HEAP_PROPERTY_ID"),
    "WS_HOST_NAMES": SimStruct(OrderedDict((("hostNames", SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0)), ("hostNameCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_HOST_NAMES", pack=False, align=None),
    "WS_HTTPS_URL": SimStruct(OrderedDict((("url", SimTypeRef("WS_URL", SimStruct)), ("host", SimTypeRef("WS_STRING", SimStruct)), ("port", SimTypeShort(signed=False, label="UInt16")), ("portAsString", SimTypeRef("WS_STRING", SimStruct)), ("path", SimTypeRef("WS_STRING", SimStruct)), ("query", SimTypeRef("WS_STRING", SimStruct)), ("fragment", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_HTTPS_URL", pack=False, align=None),
    "WS_HTTP_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)),)), name="WS_HTTP_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("httpHeaderAuthSecurityBinding", SimTypeRef("WS_HTTP_HEADER_AUTH_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_HEADER_AUTH_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("httpHeaderAuthSecurityBinding", SimTypeRef("WS_HTTP_HEADER_AUTH_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_HEADER_AUTH_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_HTTP_HEADER_AUTH_SECURITY_BINDING", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)),)), name="WS_HTTP_HEADER_AUTH_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)),)), name="WS_HTTP_HEADER_AUTH_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_HTTP_HEADER_AUTH_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_HEADER_AUTH_TARGET": SimTypeInt(signed=False, label="WS_HTTP_HEADER_AUTH_TARGET"),
    "WS_HTTP_HEADER_MAPPING": SimStruct(OrderedDict((("headerName", SimTypeRef("WS_XML_STRING", SimStruct)), ("headerMappingOptions", SimTypeInt(signed=False, label="UInt32")),)), name="WS_HTTP_HEADER_MAPPING", pack=False, align=None),
    "WS_HTTP_MESSAGE_MAPPING": SimStruct(OrderedDict((("requestMappingOptions", SimTypeInt(signed=False, label="UInt32")), ("responseMappingOptions", SimTypeInt(signed=False, label="UInt32")), ("requestHeaderMappings", SimTypePointer(SimTypePointer(SimTypeRef("WS_HTTP_HEADER_MAPPING", SimStruct), offset=0), offset=0)), ("requestHeaderMappingCount", SimTypeInt(signed=False, label="UInt32")), ("responseHeaderMappings", SimTypePointer(SimTypePointer(SimTypeRef("WS_HTTP_HEADER_MAPPING", SimStruct), offset=0), offset=0)), ("responseHeaderMappingCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_HTTP_MESSAGE_MAPPING", pack=False, align=None),
    "WS_HTTP_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)),)), name="WS_HTTP_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_PROXY_SETTING_MODE": SimTypeInt(signed=False, label="WS_HTTP_PROXY_SETTING_MODE"),
    "WS_HTTP_REDIRECT_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["state", "originalUrl", "newUrl"]), offset=0),
    "WS_HTTP_REDIRECT_CALLBACK_CONTEXT": SimStruct(OrderedDict((("callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["state", "originalUrl", "newUrl"]), offset=0)), ("state", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_HTTP_REDIRECT_CALLBACK_CONTEXT", pack=False, align=None),
    "WS_HTTP_SSL_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_SSL_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_SSL_HEADER_AUTH_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("httpHeaderAuthSecurityBinding", SimTypeRef("WS_HTTP_HEADER_AUTH_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_SSL_HEADER_AUTH_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_SSL_HEADER_AUTH_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("httpHeaderAuthSecurityBinding", SimTypeRef("WS_HTTP_HEADER_AUTH_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_SSL_HEADER_AUTH_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_SSL_KERBEROS_APREQ_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_SSL_KERBEROS_APREQ_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_SSL_KERBEROS_APREQ_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_SSL_KERBEROS_APREQ_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_SSL_KERBEROS_APREQ_SECURITY_CONTEXT_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_SSL_KERBEROS_APREQ_SECURITY_CONTEXT_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_SSL_KERBEROS_APREQ_SECURITY_CONTEXT_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_SSL_KERBEROS_APREQ_SECURITY_CONTEXT_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_SSL_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_SSL_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_SSL_USERNAME_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_SSL_USERNAME_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_SSL_USERNAME_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_SSL_USERNAME_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_SSL_USERNAME_SECURITY_CONTEXT_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_HTTP_SSL_USERNAME_SECURITY_CONTEXT_BINDING_TEMPLATE", pack=False, align=None),
    "WS_HTTP_SSL_USERNAME_SECURITY_CONTEXT_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sslTransportSecurityBinding", SimTypeRef("WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_HTTP_SSL_USERNAME_SECURITY_CONTEXT_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_HTTP_URL": SimStruct(OrderedDict((("url", SimTypeRef("WS_URL", SimStruct)), ("host", SimTypeRef("WS_STRING", SimStruct)), ("port", SimTypeShort(signed=False, label="UInt16")), ("portAsString", SimTypeRef("WS_STRING", SimStruct)), ("path", SimTypeRef("WS_STRING", SimStruct)), ("query", SimTypeRef("WS_STRING", SimStruct)), ("fragment", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_HTTP_URL", pack=False, align=None),
    "WS_INT16_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeShort(signed=True, label="Int16")), ("maxValue", SimTypeShort(signed=True, label="Int16")),)), name="WS_INT16_DESCRIPTION", pack=False, align=None),
    "WS_INT32_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeInt(signed=True, label="Int32")), ("maxValue", SimTypeInt(signed=True, label="Int32")),)), name="WS_INT32_DESCRIPTION", pack=False, align=None),
    "WS_INT64_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeLongLong(signed=True, label="Int64")), ("maxValue", SimTypeLongLong(signed=True, label="Int64")),)), name="WS_INT64_DESCRIPTION", pack=False, align=None),
    "WS_INT8_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeChar(label="SByte")), ("maxValue", SimTypeChar(label="SByte")),)), name="WS_INT8_DESCRIPTION", pack=False, align=None),
    "WS_IP_VERSION": SimTypeInt(signed=False, label="WS_IP_VERSION"),
    "WS_ISSUED_TOKEN_MESSAGE_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("claimConstraints", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("claimConstraintCount", SimTypeInt(signed=False, label="UInt32")), ("requestSecurityTokenPropertyConstraints", SimTypePointer(SimTypeRef("WS_REQUEST_SECURITY_TOKEN_PROPERTY_CONSTRAINT", SimStruct), offset=0)), ("requestSecurityTokenPropertyConstraintCount", SimTypeInt(signed=False, label="UInt32")), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_ISSUED_TOKEN_MESSAGE_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_IS_DEFAULT_VALUE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["descriptionData", "value", "defaultValue", "valueSize", "isDefault", "error"]), offset=0),
    "WS_ITEM_RANGE": SimStruct(OrderedDict((("minItemCount", SimTypeInt(signed=False, label="UInt32")), ("maxItemCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_ITEM_RANGE", pack=False, align=None),
    "WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING", pack=False, align=None),
    "WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")),)), name="WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")),)), name="WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_LISTENER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_LISTENER_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_LISTENER_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_LISTENER_PROPERTIES", pack=False, align=None),
    "WS_LISTENER_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_LISTENER_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_LISTENER_PROPERTY", pack=False, align=None),
    "WS_LISTENER_PROPERTY_ID": SimTypeInt(signed=False, label="WS_LISTENER_PROPERTY_ID"),
    "WS_LISTENER_STATE": SimTypeInt(signed=False, label="WS_LISTENER_STATE"),
    "WS_MESSAGE": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_MESSAGE_DESCRIPTION": SimStruct(OrderedDict((("action", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("bodyElementDescription", SimTypePointer(SimTypeRef("WS_ELEMENT_DESCRIPTION", SimStruct), offset=0)),)), name="WS_MESSAGE_DESCRIPTION", pack=False, align=None),
    "WS_MESSAGE_DONE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["doneCallbackState"]), offset=0),
    "WS_MESSAGE_INITIALIZATION": SimTypeInt(signed=False, label="WS_MESSAGE_INITIALIZATION"),
    "WS_MESSAGE_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_MESSAGE_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_MESSAGE_PROPERTIES", pack=False, align=None),
    "WS_MESSAGE_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_MESSAGE_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_MESSAGE_PROPERTY", pack=False, align=None),
    "WS_MESSAGE_PROPERTY_ID": SimTypeInt(signed=False, label="WS_MESSAGE_PROPERTY_ID"),
    "WS_MESSAGE_SECURITY_USAGE": SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE"),
    "WS_MESSAGE_STATE": SimTypeInt(signed=False, label="WS_MESSAGE_STATE"),
    "WS_METADATA": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_METADATA_ENDPOINT": SimStruct(OrderedDict((("endpointAddress", SimTypeRef("WS_ENDPOINT_ADDRESS", SimStruct)), ("endpointPolicy", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("portName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("serviceName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("serviceNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("bindingName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("bindingNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("portTypeName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("portTypeNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)),)), name="WS_METADATA_ENDPOINT", pack=False, align=None),
    "WS_METADATA_ENDPOINTS": SimStruct(OrderedDict((("endpoints", SimTypePointer(SimTypeRef("WS_METADATA_ENDPOINT", SimStruct), offset=0)), ("endpointCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_METADATA_ENDPOINTS", pack=False, align=None),
    "WS_METADATA_EXCHANGE_TYPE": SimTypeInt(signed=False, label="WS_METADATA_EXCHANGE_TYPE"),
    "WS_METADATA_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_METADATA_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_METADATA_PROPERTY", pack=False, align=None),
    "WS_METADATA_PROPERTY_ID": SimTypeInt(signed=False, label="WS_METADATA_PROPERTY_ID"),
    "WS_METADATA_STATE": SimTypeInt(signed=False, label="WS_METADATA_STATE"),
    "WS_MOVE_TO": SimTypeInt(signed=False, label="WS_MOVE_TO"),
    "WS_NAMEDPIPE_SSPI_TRANSPORT_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_NAMEDPIPE_SSPI_TRANSPORT_SECURITY_BINDING", pack=False, align=None),
    "WS_NCRYPT_ASYMMETRIC_SECURITY_KEY_HANDLE": SimStruct(OrderedDict((("keyHandle", SimTypeRef("WS_SECURITY_KEY_HANDLE", SimStruct)), ("asymmetricKey", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="WS_NCRYPT_ASYMMETRIC_SECURITY_KEY_HANDLE", pack=False, align=None),
    "WS_NETPIPE_URL": SimStruct(OrderedDict((("url", SimTypeRef("WS_URL", SimStruct)), ("host", SimTypeRef("WS_STRING", SimStruct)), ("port", SimTypeShort(signed=False, label="UInt16")), ("portAsString", SimTypeRef("WS_STRING", SimStruct)), ("path", SimTypeRef("WS_STRING", SimStruct)), ("query", SimTypeRef("WS_STRING", SimStruct)), ("fragment", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_NETPIPE_URL", pack=False, align=None),
    "WS_NETTCP_URL": SimStruct(OrderedDict((("url", SimTypeRef("WS_URL", SimStruct)), ("host", SimTypeRef("WS_STRING", SimStruct)), ("port", SimTypeShort(signed=False, label="UInt16")), ("portAsString", SimTypeRef("WS_STRING", SimStruct)), ("path", SimTypeRef("WS_STRING", SimStruct)), ("query", SimTypeRef("WS_STRING", SimStruct)), ("fragment", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_NETTCP_URL", pack=False, align=None),
    "WS_OPAQUE_WINDOWS_INTEGRATED_AUTH_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct)), ("opaqueAuthIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_OPAQUE_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", pack=False, align=None),
    "WS_OPEN_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ENDPOINT_ADDRESS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "endpointAddress", "asyncContext", "error"]), offset=0),
    "WS_OPEN_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "url", "asyncContext", "error"]), offset=0),
    "WS_OPERATION_CANCEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="WS_SERVICE_CANCEL_REASON"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["reason", "state"]), offset=0),
    "WS_OPERATION_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_OPERATION_CONTEXT_PROPERTY_ID": SimTypeInt(signed=False, label="WS_OPERATION_CONTEXT_PROPERTY_ID"),
    "WS_OPERATION_DESCRIPTION": SimStruct(OrderedDict((("versionInfo", SimTypeInt(signed=False, label="UInt32")), ("inputMessageDescription", SimTypePointer(SimTypeRef("WS_MESSAGE_DESCRIPTION", SimStruct), offset=0)), ("outputMessageDescription", SimTypePointer(SimTypeRef("WS_MESSAGE_DESCRIPTION", SimStruct), offset=0)), ("inputMessageOptions", SimTypeInt(signed=False, label="UInt32")), ("outputMessageOptions", SimTypeInt(signed=False, label="UInt32")), ("parameterCount", SimTypeShort(signed=False, label="UInt16")), ("parameterDescription", SimTypePointer(SimTypeRef("WS_PARAMETER_DESCRIPTION", SimStruct), offset=0)), ("stubCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "frame", "callback", "asyncContext", "error"]), offset=0)), ("style", SimTypeInt(signed=False, label="WS_OPERATION_STYLE")),)), name="WS_OPERATION_DESCRIPTION", pack=False, align=None),
    "WS_OPERATION_FREE_STATE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["state"]), offset=0),
    "WS_OPERATION_STYLE": SimTypeInt(signed=False, label="WS_OPERATION_STYLE"),
    "WS_PARAMETER_DESCRIPTION": SimStruct(OrderedDict((("parameterType", SimTypeInt(signed=False, label="WS_PARAMETER_TYPE")), ("inputMessageIndex", SimTypeShort(signed=False, label="UInt16")), ("outputMessageIndex", SimTypeShort(signed=False, label="UInt16")),)), name="WS_PARAMETER_DESCRIPTION", pack=False, align=None),
    "WS_PARAMETER_TYPE": SimTypeInt(signed=False, label="WS_PARAMETER_TYPE"),
    "WS_POLICY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_POLICY_CONSTRAINTS": SimStruct(OrderedDict((("channelBinding", SimTypeInt(signed=False, label="WS_CHANNEL_BINDING")), ("channelPropertyConstraints", SimTypePointer(SimTypeRef("WS_CHANNEL_PROPERTY_CONSTRAINT", SimStruct), offset=0)), ("channelPropertyConstraintCount", SimTypeInt(signed=False, label="UInt32")), ("securityConstraints", SimTypePointer(SimTypeRef("WS_SECURITY_CONSTRAINTS", SimStruct), offset=0)), ("policyExtensions", SimTypePointer(SimTypePointer(SimTypeRef("WS_POLICY_EXTENSION", SimStruct), offset=0), offset=0)), ("policyExtensionCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_POLICY_CONSTRAINTS", pack=False, align=None),
    "WS_POLICY_EXTENSION": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="WS_POLICY_EXTENSION_TYPE")),)), name="WS_POLICY_EXTENSION", pack=False, align=None),
    "WS_POLICY_EXTENSION_TYPE": SimTypeInt(signed=False, label="WS_POLICY_EXTENSION_TYPE"),
    "WS_POLICY_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_POLICY_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_POLICY_PROPERTIES", pack=False, align=None),
    "WS_POLICY_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_POLICY_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_POLICY_PROPERTY", pack=False, align=None),
    "WS_POLICY_PROPERTY_ID": SimTypeInt(signed=False, label="WS_POLICY_PROPERTY_ID"),
    "WS_POLICY_STATE": SimTypeInt(signed=False, label="WS_POLICY_STATE"),
    "WS_PROTECTION_LEVEL": SimTypeInt(signed=False, label="WS_PROTECTION_LEVEL"),
    "WS_PROXY_MESSAGE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["message", "heap", "state", "error"]), offset=0),
    "WS_PROXY_MESSAGE_CALLBACK_CONTEXT": SimStruct(OrderedDict((("callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["message", "heap", "state", "error"]), offset=0)), ("state", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_PROXY_MESSAGE_CALLBACK_CONTEXT", pack=False, align=None),
    "WS_PROXY_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_PROXY_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_PROXY_PROPERTY", pack=False, align=None),
    "WS_PROXY_PROPERTY_ID": SimTypeInt(signed=False, label="WS_PROXY_PROPERTY_ID"),
    "WS_PULL_BYTES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "bytes", "maxSize", "actualSize", "asyncContext", "error"]), offset=0),
    "WS_PUSH_BYTES_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "buffers", "count", "asyncContext", "error"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "writeCallback", "writeCallbackState", "asyncContext", "error"]), offset=0),
    "WS_RAW_SYMMETRIC_SECURITY_KEY_HANDLE": SimStruct(OrderedDict((("keyHandle", SimTypeRef("WS_SECURITY_KEY_HANDLE", SimStruct)), ("rawKeyBytes", SimTypeRef("WS_BYTES", SimStruct)),)), name="WS_RAW_SYMMETRIC_SECURITY_KEY_HANDLE", pack=False, align=None),
    "WS_READ_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "bytes", "maxSize", "actualSize", "asyncContext", "error"]), offset=0),
    "WS_READ_MESSAGE_END_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0),
    "WS_READ_MESSAGE_START_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0),
    "WS_READ_OPTION": SimTypeInt(signed=False, label="WS_READ_OPTION"),
    "WS_READ_TYPE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="WS_TYPE_MAPPING"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["reader", "typeMapping", "descriptionData", "heap", "value", "valueSize", "error"]), offset=0),
    "WS_RECEIVE_OPTION": SimTypeInt(signed=False, label="WS_RECEIVE_OPTION"),
    "WS_REPEATING_HEADER_OPTION": SimTypeInt(signed=False, label="WS_REPEATING_HEADER_OPTION"),
    "WS_REQUEST_SECURITY_TOKEN_ACTION": SimTypeInt(signed=False, label="WS_REQUEST_SECURITY_TOKEN_ACTION"),
    "WS_REQUEST_SECURITY_TOKEN_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_REQUEST_SECURITY_TOKEN_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_REQUEST_SECURITY_TOKEN_PROPERTY", pack=False, align=None),
    "WS_REQUEST_SECURITY_TOKEN_PROPERTY_CONSTRAINT": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_REQUEST_SECURITY_TOKEN_PROPERTY_ID")), ("allowedValues", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("allowedValuesSize", SimTypeInt(signed=False, label="UInt32")), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_REQUEST_SECURITY_TOKEN_PROPERTY_CONSTRAINT", pack=False, align=None),
    "WS_REQUEST_SECURITY_TOKEN_PROPERTY_ID": SimTypeInt(signed=False, label="WS_REQUEST_SECURITY_TOKEN_PROPERTY_ID"),
    "WS_RESET_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "error"]), offset=0),
    "WS_RESET_LISTENER_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "error"]), offset=0),
    "WS_RSA_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identity", SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct)), ("modulus", SimTypeRef("WS_BYTES", SimStruct)), ("exponent", SimTypeRef("WS_BYTES", SimStruct)),)), name="WS_RSA_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_SAML_AUTHENTICATOR": SimStruct(OrderedDict((("authenticatorType", SimTypeInt(signed=False, label="WS_SAML_AUTHENTICATOR_TYPE")),)), name="WS_SAML_AUTHENTICATOR", pack=False, align=None),
    "WS_SAML_AUTHENTICATOR_TYPE": SimTypeInt(signed=False, label="WS_SAML_AUTHENTICATOR_TYPE"),
    "WS_SAML_MESSAGE_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("authenticator", SimTypePointer(SimTypeRef("WS_SAML_AUTHENTICATOR", SimStruct), offset=0)),)), name="WS_SAML_MESSAGE_SECURITY_BINDING", pack=False, align=None),
    "WS_SECURE_CONVERSATION_VERSION": SimTypeInt(signed=False, label="WS_SECURE_CONVERSATION_VERSION"),
    "WS_SECURE_PROTOCOL": SimTypeInt(signed=False, label="WS_SECURE_PROTOCOL"),
    "WS_SECURITY_ALGORITHM_ID": SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID"),
    "WS_SECURITY_ALGORITHM_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_ALGORITHM_PROPERTY", pack=False, align=None),
    "WS_SECURITY_ALGORITHM_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_PROPERTY_ID"),
    "WS_SECURITY_ALGORITHM_SUITE": SimStruct(OrderedDict((("canonicalizationAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("digestAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("symmetricSignatureAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("asymmetricSignatureAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("encryptionAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("keyDerivationAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("symmetricKeyWrapAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("asymmetricKeyWrapAlgorithm", SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_ID")), ("minSymmetricKeyLength", SimTypeInt(signed=False, label="UInt32")), ("maxSymmetricKeyLength", SimTypeInt(signed=False, label="UInt32")), ("minAsymmetricKeyLength", SimTypeInt(signed=False, label="UInt32")), ("maxAsymmetricKeyLength", SimTypeInt(signed=False, label="UInt32")), ("properties", SimTypePointer(SimTypeRef("WS_SECURITY_ALGORITHM_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_ALGORITHM_SUITE", pack=False, align=None),
    "WS_SECURITY_ALGORITHM_SUITE_NAME": SimTypeInt(signed=False, label="WS_SECURITY_ALGORITHM_SUITE_NAME"),
    "WS_SECURITY_BEARER_KEY_TYPE_VERSION": SimTypeInt(signed=False, label="WS_SECURITY_BEARER_KEY_TYPE_VERSION"),
    "WS_SECURITY_BINDING": SimStruct(OrderedDict((("bindingType", SimTypeInt(signed=False, label="WS_SECURITY_BINDING_TYPE")), ("properties", SimTypePointer(SimTypeRef("WS_SECURITY_BINDING_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_BINDING", pack=False, align=None),
    "WS_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="WS_SECURITY_BINDING_CONSTRAINT_TYPE")), ("propertyConstraints", SimTypePointer(SimTypeRef("WS_SECURITY_BINDING_PROPERTY_CONSTRAINT", SimStruct), offset=0)), ("propertyConstraintCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_SECURITY_BINDING_CONSTRAINT_TYPE": SimTypeInt(signed=False, label="WS_SECURITY_BINDING_CONSTRAINT_TYPE"),
    "WS_SECURITY_BINDING_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_SECURITY_BINDING_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_BINDING_PROPERTIES", pack=False, align=None),
    "WS_SECURITY_BINDING_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SECURITY_BINDING_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_BINDING_PROPERTY", pack=False, align=None),
    "WS_SECURITY_BINDING_PROPERTY_CONSTRAINT": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SECURITY_BINDING_PROPERTY_ID")), ("allowedValues", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("allowedValuesSize", SimTypeInt(signed=False, label="UInt32")), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_SECURITY_BINDING_PROPERTY_CONSTRAINT", pack=False, align=None),
    "WS_SECURITY_BINDING_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SECURITY_BINDING_PROPERTY_ID"),
    "WS_SECURITY_BINDING_TYPE": SimTypeInt(signed=False, label="WS_SECURITY_BINDING_TYPE"),
    "WS_SECURITY_CONSTRAINTS": SimStruct(OrderedDict((("securityPropertyConstraints", SimTypePointer(SimTypeRef("WS_SECURITY_PROPERTY_CONSTRAINT", SimStruct), offset=0)), ("securityPropertyConstraintCount", SimTypeInt(signed=False, label="UInt32")), ("securityBindingConstraints", SimTypePointer(SimTypePointer(SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct), offset=0), offset=0)), ("securityBindingConstraintCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_CONSTRAINTS", pack=False, align=None),
    "WS_SECURITY_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("bootstrapSecurityDescription", SimTypePointer(SimTypeRef("WS_SECURITY_DESCRIPTION", SimStruct), offset=0)),)), name="WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING", pack=False, align=None),
    "WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("bootstrapSecurityConstraint", SimTypePointer(SimTypeRef("WS_SECURITY_CONSTRAINTS", SimStruct), offset=0)),)), name="WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")),)), name="WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)),)), name="WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_SECURITY_CONTEXT_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SECURITY_CONTEXT_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_CONTEXT_PROPERTY", pack=False, align=None),
    "WS_SECURITY_CONTEXT_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SECURITY_CONTEXT_PROPERTY_ID"),
    "WS_SECURITY_CONTEXT_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityContextMessageSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)),)), name="WS_SECURITY_CONTEXT_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_SECURITY_CONTEXT_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityContextMessageSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)),)), name="WS_SECURITY_CONTEXT_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_SECURITY_DESCRIPTION": SimStruct(OrderedDict((("securityBindings", SimTypePointer(SimTypePointer(SimTypeRef("WS_SECURITY_BINDING", SimStruct), offset=0), offset=0)), ("securityBindingCount", SimTypeInt(signed=False, label="UInt32")), ("properties", SimTypePointer(SimTypeRef("WS_SECURITY_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_DESCRIPTION", pack=False, align=None),
    "WS_SECURITY_HEADER_LAYOUT": SimTypeInt(signed=False, label="WS_SECURITY_HEADER_LAYOUT"),
    "WS_SECURITY_HEADER_VERSION": SimTypeInt(signed=False, label="WS_SECURITY_HEADER_VERSION"),
    "WS_SECURITY_KEY_ENTROPY_MODE": SimTypeInt(signed=False, label="WS_SECURITY_KEY_ENTROPY_MODE"),
    "WS_SECURITY_KEY_HANDLE": SimStruct(OrderedDict((("keyHandleType", SimTypeInt(signed=False, label="WS_SECURITY_KEY_HANDLE_TYPE")),)), name="WS_SECURITY_KEY_HANDLE", pack=False, align=None),
    "WS_SECURITY_KEY_HANDLE_TYPE": SimTypeInt(signed=False, label="WS_SECURITY_KEY_HANDLE_TYPE"),
    "WS_SECURITY_KEY_TYPE": SimTypeInt(signed=False, label="WS_SECURITY_KEY_TYPE"),
    "WS_SECURITY_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_SECURITY_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_PROPERTIES", pack=False, align=None),
    "WS_SECURITY_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SECURITY_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SECURITY_PROPERTY", pack=False, align=None),
    "WS_SECURITY_PROPERTY_CONSTRAINT": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SECURITY_PROPERTY_ID")), ("allowedValues", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("allowedValuesSize", SimTypeInt(signed=False, label="UInt32")), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_SECURITY_PROPERTY_CONSTRAINT", pack=False, align=None),
    "WS_SECURITY_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SECURITY_PROPERTY_ID"),
    "WS_SECURITY_TIMESTAMP_USAGE": SimTypeInt(signed=False, label="WS_SECURITY_TIMESTAMP_USAGE"),
    "WS_SECURITY_TOKEN": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_SECURITY_TOKEN_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SECURITY_TOKEN_PROPERTY_ID"),
    "WS_SECURITY_TOKEN_REFERENCE_MODE": SimTypeInt(signed=False, label="WS_SECURITY_TOKEN_REFERENCE_MODE"),
    "WS_SERVICE_ACCEPT_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "channelState", "asyncContext", "error"]), offset=0),
    "WS_SERVICE_CANCEL_REASON": SimTypeInt(signed=False, label="WS_SERVICE_CANCEL_REASON"),
    "WS_SERVICE_CLOSE_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "asyncContext"]), offset=0),
    "WS_SERVICE_CONTRACT": SimStruct(OrderedDict((("contractDescription", SimTypePointer(SimTypeRef("WS_CONTRACT_DESCRIPTION", SimStruct), offset=0)), ("defaultMessageHandlerCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "asyncContext", "error"]), offset=0)), ("methodTable", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_SERVICE_CONTRACT", pack=False, align=None),
    "WS_SERVICE_ENDPOINT": SimStruct(OrderedDict((("address", SimTypeRef("WS_ENDPOINT_ADDRESS", SimStruct)), ("channelBinding", SimTypeInt(signed=False, label="WS_CHANNEL_BINDING")), ("channelType", SimTypeInt(signed=False, label="WS_CHANNEL_TYPE")), ("securityDescription", SimTypePointer(SimTypeRef("WS_SECURITY_DESCRIPTION", SimStruct), offset=0)), ("contract", SimTypePointer(SimTypeRef("WS_SERVICE_CONTRACT", SimStruct), offset=0)), ("authorizationCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "authorized", "error"]), offset=0)), ("properties", SimTypePointer(SimTypeRef("WS_SERVICE_ENDPOINT_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")), ("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)),)), name="WS_SERVICE_ENDPOINT", pack=False, align=None),
    "WS_SERVICE_ENDPOINT_METADATA": SimStruct(OrderedDict((("portName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("bindingName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("bindingNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)),)), name="WS_SERVICE_ENDPOINT_METADATA", pack=False, align=None),
    "WS_SERVICE_ENDPOINT_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SERVICE_ENDPOINT_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SERVICE_ENDPOINT_PROPERTY", pack=False, align=None),
    "WS_SERVICE_ENDPOINT_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SERVICE_ENDPOINT_PROPERTY_ID"),
    "WS_SERVICE_HOST": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_SERVICE_HOST_STATE": SimTypeInt(signed=False, label="WS_SERVICE_HOST_STATE"),
    "WS_SERVICE_MESSAGE_RECEIVE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "asyncContext", "error"]), offset=0),
    "WS_SERVICE_METADATA": SimStruct(OrderedDict((("documentCount", SimTypeInt(signed=False, label="UInt32")), ("documents", SimTypePointer(SimTypePointer(SimTypeRef("WS_SERVICE_METADATA_DOCUMENT", SimStruct), offset=0), offset=0)), ("serviceName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("serviceNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)),)), name="WS_SERVICE_METADATA", pack=False, align=None),
    "WS_SERVICE_METADATA_DOCUMENT": SimStruct(OrderedDict((("content", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("name", SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0)),)), name="WS_SERVICE_METADATA_DOCUMENT", pack=False, align=None),
    "WS_SERVICE_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_SERVICE_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SERVICE_PROPERTY", pack=False, align=None),
    "WS_SERVICE_PROPERTY_ACCEPT_CALLBACK": SimStruct(OrderedDict((("callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "channelState", "asyncContext", "error"]), offset=0)),)), name="WS_SERVICE_PROPERTY_ACCEPT_CALLBACK", pack=False, align=None),
    "WS_SERVICE_PROPERTY_CLOSE_CALLBACK": SimStruct(OrderedDict((("callback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "asyncContext"]), offset=0)),)), name="WS_SERVICE_PROPERTY_CLOSE_CALLBACK", pack=False, align=None),
    "WS_SERVICE_PROPERTY_ID": SimTypeInt(signed=False, label="WS_SERVICE_PROPERTY_ID"),
    "WS_SERVICE_PROXY": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_SERVICE_PROXY_STATE": SimTypeInt(signed=False, label="WS_SERVICE_PROXY_STATE"),
    "WS_SERVICE_SECURITY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "authorized", "error"]), offset=0),
    "WS_SERVICE_SECURITY_IDENTITIES": SimStruct(OrderedDict((("serviceIdentities", SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0)), ("serviceIdentityCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_SERVICE_SECURITY_IDENTITIES", pack=False, align=None),
    "WS_SERVICE_STUB_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "frame", "callback", "asyncContext", "error"]), offset=0),
    "WS_SET_CHANNEL_PROPERTY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_CHANNEL_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "id", "value", "valueSize", "error"]), offset=0),
    "WS_SET_LISTENER_PROPERTY_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="WS_LISTENER_PROPERTY_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["listenerInstance", "id", "value", "valueSize", "error"]), offset=0),
    "WS_SHUTDOWN_SESSION_CHANNEL_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "asyncContext", "error"]), offset=0),
    "WS_SOAPUDP_URL": SimStruct(OrderedDict((("url", SimTypeRef("WS_URL", SimStruct)), ("host", SimTypeRef("WS_STRING", SimStruct)), ("port", SimTypeShort(signed=False, label="UInt16")), ("portAsString", SimTypeRef("WS_STRING", SimStruct)), ("path", SimTypeRef("WS_STRING", SimStruct)), ("query", SimTypeRef("WS_STRING", SimStruct)), ("fragment", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_SOAPUDP_URL", pack=False, align=None),
    "WS_SPN_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identity", SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct)), ("spn", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_SPN_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_SSL_TRANSPORT_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("localCertCredential", SimTypePointer(SimTypeRef("WS_CERT_CREDENTIAL", SimStruct), offset=0)),)), name="WS_SSL_TRANSPORT_SECURITY_BINDING", pack=False, align=None),
    "WS_SSL_TRANSPORT_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)), ("out", SimTypeRef("_out_e__Struct", SimStruct)),)), name="WS_SSL_TRANSPORT_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)),)), name="WS_SSL_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("localCertCredential", SimTypePointer(SimTypeRef("WS_CERT_CREDENTIAL", SimStruct), offset=0)),)), name="WS_SSL_TRANSPORT_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)),)), name="WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_STRING": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("chars", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WS_STRING", pack=False, align=None),
    "WS_STRING_DESCRIPTION": SimStruct(OrderedDict((("minCharCount", SimTypeInt(signed=False, label="UInt32")), ("maxCharCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_STRING_DESCRIPTION", pack=False, align=None),
    "WS_STRING_USERNAME_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_USERNAME_CREDENTIAL", SimStruct)), ("username", SimTypeRef("WS_STRING", SimStruct)), ("password", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_STRING_USERNAME_CREDENTIAL", pack=False, align=None),
    "WS_STRING_WINDOWS_INTEGRATED_AUTH_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct)), ("username", SimTypeRef("WS_STRING", SimStruct)), ("password", SimTypeRef("WS_STRING", SimStruct)), ("domain", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_STRING_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", pack=False, align=None),
    "WS_STRUCT_DESCRIPTION": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("alignment", SimTypeInt(signed=False, label="UInt32")), ("fields", SimTypePointer(SimTypePointer(SimTypeRef("WS_FIELD_DESCRIPTION", SimStruct), offset=0), offset=0)), ("fieldCount", SimTypeInt(signed=False, label="UInt32")), ("typeLocalName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("typeNs", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("parentType", SimTypePointer(SimTypeRef("WS_STRUCT_DESCRIPTION", SimStruct), offset=0)), ("subTypes", SimTypePointer(SimTypePointer(SimTypeRef("WS_STRUCT_DESCRIPTION", SimStruct), offset=0), offset=0)), ("subTypeCount", SimTypeInt(signed=False, label="UInt32")), ("structOptions", SimTypeInt(signed=False, label="UInt32")),)), name="WS_STRUCT_DESCRIPTION", pack=False, align=None),
    "WS_SUBJECT_NAME_CERT_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_CERT_CREDENTIAL", SimStruct)), ("storeLocation", SimTypeInt(signed=False, label="UInt32")), ("storeName", SimTypeRef("WS_STRING", SimStruct)), ("subjectName", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_SUBJECT_NAME_CERT_CREDENTIAL", pack=False, align=None),
    "WS_TCP_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)),)), name="WS_TCP_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)),)), name="WS_TCP_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_TCP_SSPI_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_TCP_SSPI_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_SSPI_KERBEROS_APREQ_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_TCP_SSPI_KERBEROS_APREQ_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_SSPI_KERBEROS_APREQ_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_TCP_SSPI_KERBEROS_APREQ_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_TCP_SSPI_KERBEROS_APREQ_SECURITY_CONTEXT_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_TCP_SSPI_KERBEROS_APREQ_SECURITY_CONTEXT_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_SSPI_KERBEROS_APREQ_SECURITY_CONTEXT_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("kerberosApreqMessageSecurityBinding", SimTypeRef("WS_KERBEROS_APREQ_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_TCP_SSPI_KERBEROS_APREQ_SECURITY_CONTEXT_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_TCP_SSPI_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_TCP_SSPI_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING", pack=False, align=None),
    "WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)),)), name="WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", SimStruct), offset=0)),)), name="WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_SSPI_USERNAME_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_TCP_SSPI_USERNAME_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_SSPI_USERNAME_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_TCP_SSPI_USERNAME_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_TCP_SSPI_USERNAME_SECURITY_CONTEXT_BINDING_TEMPLATE": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_TCP_SSPI_TRANSPORT_SECURITY_BINDING_TEMPLATE", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_TEMPLATE", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_TEMPLATE", SimStruct)),)), name="WS_TCP_SSPI_USERNAME_SECURITY_CONTEXT_BINDING_TEMPLATE", pack=False, align=None),
    "WS_TCP_SSPI_USERNAME_SECURITY_CONTEXT_POLICY_DESCRIPTION": SimStruct(OrderedDict((("channelProperties", SimTypeRef("WS_CHANNEL_PROPERTIES", SimStruct)), ("securityProperties", SimTypeRef("WS_SECURITY_PROPERTIES", SimStruct)), ("sspiTransportSecurityBinding", SimTypeRef("WS_SSPI_TRANSPORT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("usernameMessageSecurityBinding", SimTypeRef("WS_USERNAME_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)), ("securityContextSecurityBinding", SimTypeRef("WS_SECURITY_CONTEXT_SECURITY_BINDING_POLICY_DESCRIPTION", SimStruct)),)), name="WS_TCP_SSPI_USERNAME_SECURITY_CONTEXT_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_THUMBPRINT_CERT_CREDENTIAL": SimStruct(OrderedDict((("credential", SimTypeRef("WS_CERT_CREDENTIAL", SimStruct)), ("storeLocation", SimTypeInt(signed=False, label="UInt32")), ("storeName", SimTypeRef("WS_STRING", SimStruct)), ("thumbprint", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_THUMBPRINT_CERT_CREDENTIAL", pack=False, align=None),
    "WS_TIMESPAN": SimStruct(OrderedDict((("ticks", SimTypeLongLong(signed=True, label="Int64")),)), name="WS_TIMESPAN", pack=False, align=None),
    "WS_TIMESPAN_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeRef("WS_TIMESPAN", SimStruct)), ("maxValue", SimTypeRef("WS_TIMESPAN", SimStruct)),)), name="WS_TIMESPAN_DESCRIPTION", pack=False, align=None),
    "WS_TRACE_API": SimTypeInt(signed=False, label="WS_TRACE_API"),
    "WS_TRANSFER_MODE": SimTypeInt(signed=False, label="WS_TRANSFER_MODE"),
    "WS_TRUST_VERSION": SimTypeInt(signed=False, label="WS_TRUST_VERSION"),
    "WS_TYPE": SimTypeInt(signed=False, label="WS_TYPE"),
    "WS_TYPE_MAPPING": SimTypeInt(signed=False, label="WS_TYPE_MAPPING"),
    "WS_UINT16_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeShort(signed=False, label="UInt16")), ("maxValue", SimTypeShort(signed=False, label="UInt16")),)), name="WS_UINT16_DESCRIPTION", pack=False, align=None),
    "WS_UINT32_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeInt(signed=False, label="UInt32")), ("maxValue", SimTypeInt(signed=False, label="UInt32")),)), name="WS_UINT32_DESCRIPTION", pack=False, align=None),
    "WS_UINT64_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeLongLong(signed=False, label="UInt64")), ("maxValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="WS_UINT64_DESCRIPTION", pack=False, align=None),
    "WS_UINT8_DESCRIPTION": SimStruct(OrderedDict((("minValue", SimTypeChar(label="Byte")), ("maxValue", SimTypeChar(label="Byte")),)), name="WS_UINT8_DESCRIPTION", pack=False, align=None),
    "WS_UNION_DESCRIPTION": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("alignment", SimTypeInt(signed=False, label="UInt32")), ("fields", SimTypePointer(SimTypePointer(SimTypeRef("WS_UNION_FIELD_DESCRIPTION", SimStruct), offset=0), offset=0)), ("fieldCount", SimTypeInt(signed=False, label="UInt32")), ("enumOffset", SimTypeInt(signed=False, label="UInt32")), ("noneEnumValue", SimTypeInt(signed=True, label="Int32")), ("valueIndices", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="WS_UNION_DESCRIPTION", pack=False, align=None),
    "WS_UNION_FIELD_DESCRIPTION": SimStruct(OrderedDict((("value", SimTypeInt(signed=True, label="Int32")), ("field", SimTypeRef("WS_FIELD_DESCRIPTION", SimStruct)),)), name="WS_UNION_FIELD_DESCRIPTION", pack=False, align=None),
    "WS_UNIQUE_ID": SimStruct(OrderedDict((("uri", SimTypeRef("WS_STRING", SimStruct)), ("guid", SimTypeBottom(label="Guid")),)), name="WS_UNIQUE_ID", pack=False, align=None),
    "WS_UNIQUE_ID_DESCRIPTION": SimStruct(OrderedDict((("minCharCount", SimTypeInt(signed=False, label="UInt32")), ("maxCharCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_UNIQUE_ID_DESCRIPTION", pack=False, align=None),
    "WS_UNKNOWN_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identity", SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct)), ("element", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="WS_UNKNOWN_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_UPN_ENDPOINT_IDENTITY": SimStruct(OrderedDict((("identity", SimTypeRef("WS_ENDPOINT_IDENTITY", SimStruct)), ("upn", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_UPN_ENDPOINT_IDENTITY", pack=False, align=None),
    "WS_URL": SimStruct(OrderedDict((("scheme", SimTypeInt(signed=False, label="WS_URL_SCHEME_TYPE")),)), name="WS_URL", pack=False, align=None),
    "WS_URL_SCHEME_TYPE": SimTypeInt(signed=False, label="WS_URL_SCHEME_TYPE"),
    "WS_USERNAME_CREDENTIAL": SimStruct(OrderedDict((("credentialType", SimTypeInt(signed=False, label="WS_USERNAME_CREDENTIAL_TYPE")),)), name="WS_USERNAME_CREDENTIAL", pack=False, align=None),
    "WS_USERNAME_CREDENTIAL_TYPE": SimTypeInt(signed=False, label="WS_USERNAME_CREDENTIAL_TYPE"),
    "WS_USERNAME_MESSAGE_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("clientCredential", SimTypePointer(SimTypeRef("WS_USERNAME_CREDENTIAL", SimStruct), offset=0)), ("passwordValidator", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["passwordValidatorCallbackState", "username", "password", "asyncContext", "error"]), offset=0)), ("passwordValidatorCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_USERNAME_MESSAGE_SECURITY_BINDING", pack=False, align=None),
    "WS_USERNAME_MESSAGE_SECURITY_BINDING_CONSTRAINT": SimStruct(OrderedDict((("bindingConstraint", SimTypeRef("WS_SECURITY_BINDING_CONSTRAINT", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")),)), name="WS_USERNAME_MESSAGE_SECURITY_BINDING_CONSTRAINT", pack=False, align=None),
    "WS_USERNAME_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")),)), name="WS_USERNAME_MESSAGE_SECURITY_BINDING_POLICY_DESCRIPTION", pack=False, align=None),
    "WS_USERNAME_MESSAGE_SECURITY_BINDING_TEMPLATE": SimStruct(OrderedDict((("securityBindingProperties", SimTypeRef("WS_SECURITY_BINDING_PROPERTIES", SimStruct)), ("clientCredential", SimTypePointer(SimTypeRef("WS_USERNAME_CREDENTIAL", SimStruct), offset=0)), ("passwordValidator", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["passwordValidatorCallbackState", "username", "password", "asyncContext", "error"]), offset=0)), ("passwordValidatorCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_USERNAME_MESSAGE_SECURITY_BINDING_TEMPLATE", pack=False, align=None),
    "WS_UTF8_ARRAY_DESCRIPTION": SimStruct(OrderedDict((("minByteCount", SimTypeInt(signed=False, label="UInt32")), ("maxByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_UTF8_ARRAY_DESCRIPTION", pack=False, align=None),
    "WS_VALIDATE_PASSWORD_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["passwordValidatorCallbackState", "username", "password", "asyncContext", "error"]), offset=0),
    "WS_VALIDATE_SAML_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["samlValidatorCallbackState", "samlAssertion", "error"]), offset=0),
    "WS_VALUE_TYPE": SimTypeInt(signed=False, label="WS_VALUE_TYPE"),
    "WS_VOID_DESCRIPTION": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")),)), name="WS_VOID_DESCRIPTION", pack=False, align=None),
    "WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL": SimStruct(OrderedDict((("credentialType", SimTypeInt(signed=False, label="WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL_TYPE")),)), name="WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL", pack=False, align=None),
    "WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL_TYPE": SimTypeInt(signed=False, label="WS_WINDOWS_INTEGRATED_AUTH_CREDENTIAL_TYPE"),
    "WS_WINDOWS_INTEGRATED_AUTH_PACKAGE": SimTypeInt(signed=False, label="WS_WINDOWS_INTEGRATED_AUTH_PACKAGE"),
    "WS_WRITE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "buffers", "count", "asyncContext", "error"]), offset=0),
    "WS_WRITE_MESSAGE_END_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0),
    "WS_WRITE_MESSAGE_START_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["channelInstance", "message", "asyncContext", "error"]), offset=0),
    "WS_WRITE_OPTION": SimTypeInt(signed=False, label="WS_WRITE_OPTION"),
    "WS_WRITE_TYPE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="WS_TYPE_MAPPING"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["writer", "typeMapping", "descriptionData", "value", "valueSize", "error"]), offset=0),
    "WS_WSZ_DESCRIPTION": SimStruct(OrderedDict((("minCharCount", SimTypeInt(signed=False, label="UInt32")), ("maxCharCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_WSZ_DESCRIPTION", pack=False, align=None),
    "WS_XML_ATTRIBUTE": SimStruct(OrderedDict((("singleQuote", SimTypeChar(label="Byte")), ("isXmlNs", SimTypeChar(label="Byte")), ("prefix", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("localName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("ns", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("value", SimTypePointer(SimTypeRef("WS_XML_TEXT", SimStruct), offset=0)),)), name="WS_XML_ATTRIBUTE", pack=False, align=None),
    "WS_XML_BASE64_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("bytes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_BASE64_TEXT", pack=False, align=None),
    "WS_XML_BOOL_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeInt(signed=True, label="Int32")),)), name="WS_XML_BOOL_TEXT", pack=False, align=None),
    "WS_XML_BUFFER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_XML_BUFFER_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_XML_BUFFER_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_BUFFER_PROPERTY", pack=False, align=None),
    "WS_XML_BUFFER_PROPERTY_ID": SimTypeInt(signed=False, label="WS_XML_BUFFER_PROPERTY_ID"),
    "WS_XML_CANONICALIZATION_ALGORITHM": SimTypeInt(signed=False, label="WS_XML_CANONICALIZATION_ALGORITHM"),
    "WS_XML_CANONICALIZATION_INCLUSIVE_PREFIXES": SimStruct(OrderedDict((("prefixCount", SimTypeInt(signed=False, label="UInt32")), ("prefixes", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)),)), name="WS_XML_CANONICALIZATION_INCLUSIVE_PREFIXES", pack=False, align=None),
    "WS_XML_CANONICALIZATION_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_XML_CANONICALIZATION_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_CANONICALIZATION_PROPERTY", pack=False, align=None),
    "WS_XML_CANONICALIZATION_PROPERTY_ID": SimTypeInt(signed=False, label="WS_XML_CANONICALIZATION_PROPERTY_ID"),
    "WS_XML_COMMENT_NODE": SimStruct(OrderedDict((("node", SimTypeRef("WS_XML_NODE", SimStruct)), ("value", SimTypeRef("WS_XML_STRING", SimStruct)),)), name="WS_XML_COMMENT_NODE", pack=False, align=None),
    "WS_XML_DATETIME_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeRef("WS_DATETIME", SimStruct)),)), name="WS_XML_DATETIME_TEXT", pack=False, align=None),
    "WS_XML_DECIMAL_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeRef("DECIMAL", SimStruct)),)), name="WS_XML_DECIMAL_TEXT", pack=False, align=None),
    "WS_XML_DICTIONARY": SimStruct(OrderedDict((("guid", SimTypeBottom(label="Guid")), ("strings", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("stringCount", SimTypeInt(signed=False, label="UInt32")), ("isConst", SimTypeInt(signed=True, label="Int32")),)), name="WS_XML_DICTIONARY", pack=False, align=None),
    "WS_XML_DOUBLE_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeFloat(size=64)),)), name="WS_XML_DOUBLE_TEXT", pack=False, align=None),
    "WS_XML_ELEMENT_NODE": SimStruct(OrderedDict((("node", SimTypeRef("WS_XML_NODE", SimStruct)), ("prefix", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("localName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("ns", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("attributeCount", SimTypeInt(signed=False, label="UInt32")), ("attributes", SimTypePointer(SimTypePointer(SimTypeRef("WS_XML_ATTRIBUTE", SimStruct), offset=0), offset=0)), ("isEmpty", SimTypeInt(signed=True, label="Int32")),)), name="WS_XML_ELEMENT_NODE", pack=False, align=None),
    "WS_XML_FLOAT_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeFloat(size=32)),)), name="WS_XML_FLOAT_TEXT", pack=False, align=None),
    "WS_XML_GUID_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeBottom(label="Guid")),)), name="WS_XML_GUID_TEXT", pack=False, align=None),
    "WS_XML_INT32_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeInt(signed=True, label="Int32")),)), name="WS_XML_INT32_TEXT", pack=False, align=None),
    "WS_XML_INT64_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeLongLong(signed=True, label="Int64")),)), name="WS_XML_INT64_TEXT", pack=False, align=None),
    "WS_XML_LIST_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("itemCount", SimTypeInt(signed=False, label="UInt32")), ("items", SimTypePointer(SimTypePointer(SimTypeRef("WS_XML_TEXT", SimStruct), offset=0), offset=0)),)), name="WS_XML_LIST_TEXT", pack=False, align=None),
    "WS_XML_NODE": SimStruct(OrderedDict((("nodeType", SimTypeInt(signed=False, label="WS_XML_NODE_TYPE")),)), name="WS_XML_NODE", pack=False, align=None),
    "WS_XML_NODE_POSITION": SimStruct(OrderedDict((("buffer", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("node", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_XML_NODE_POSITION", pack=False, align=None),
    "WS_XML_NODE_TYPE": SimTypeInt(signed=False, label="WS_XML_NODE_TYPE"),
    "WS_XML_QNAME": SimStruct(OrderedDict((("localName", SimTypeRef("WS_XML_STRING", SimStruct)), ("ns", SimTypeRef("WS_XML_STRING", SimStruct)),)), name="WS_XML_QNAME", pack=False, align=None),
    "WS_XML_QNAME_DESCRIPTION": SimStruct(OrderedDict((("minLocalNameByteCount", SimTypeInt(signed=False, label="UInt32")), ("maxLocalNameByteCount", SimTypeInt(signed=False, label="UInt32")), ("minNsByteCount", SimTypeInt(signed=False, label="UInt32")), ("maxNsByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_QNAME_DESCRIPTION", pack=False, align=None),
    "WS_XML_QNAME_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("prefix", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("localName", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)), ("ns", SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0)),)), name="WS_XML_QNAME_TEXT", pack=False, align=None),
    "WS_XML_READER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_XML_READER_BINARY_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_READER_ENCODING", SimStruct)), ("staticDictionary", SimTypePointer(SimTypeRef("WS_XML_DICTIONARY", SimStruct), offset=0)), ("dynamicDictionary", SimTypePointer(SimTypeRef("WS_XML_DICTIONARY", SimStruct), offset=0)),)), name="WS_XML_READER_BINARY_ENCODING", pack=False, align=None),
    "WS_XML_READER_BUFFER_INPUT": SimStruct(OrderedDict((("input", SimTypeRef("WS_XML_READER_INPUT", SimStruct)), ("encodedData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("encodedDataSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_READER_BUFFER_INPUT", pack=False, align=None),
    "WS_XML_READER_ENCODING": SimStruct(OrderedDict((("encodingType", SimTypeInt(signed=False, label="WS_XML_READER_ENCODING_TYPE")),)), name="WS_XML_READER_ENCODING", pack=False, align=None),
    "WS_XML_READER_ENCODING_TYPE": SimTypeInt(signed=False, label="WS_XML_READER_ENCODING_TYPE"),
    "WS_XML_READER_INPUT": SimStruct(OrderedDict((("inputType", SimTypeInt(signed=False, label="WS_XML_READER_INPUT_TYPE")),)), name="WS_XML_READER_INPUT", pack=False, align=None),
    "WS_XML_READER_INPUT_TYPE": SimTypeInt(signed=False, label="WS_XML_READER_INPUT_TYPE"),
    "WS_XML_READER_MTOM_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_READER_ENCODING", SimStruct)), ("textEncoding", SimTypePointer(SimTypeRef("WS_XML_READER_ENCODING", SimStruct), offset=0)), ("readMimeHeader", SimTypeInt(signed=True, label="Int32")), ("startInfo", SimTypeRef("WS_STRING", SimStruct)), ("boundary", SimTypeRef("WS_STRING", SimStruct)), ("startUri", SimTypeRef("WS_STRING", SimStruct)),)), name="WS_XML_READER_MTOM_ENCODING", pack=False, align=None),
    "WS_XML_READER_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_XML_READER_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_READER_PROPERTIES", pack=False, align=None),
    "WS_XML_READER_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_XML_READER_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_READER_PROPERTY", pack=False, align=None),
    "WS_XML_READER_PROPERTY_ID": SimTypeInt(signed=False, label="WS_XML_READER_PROPERTY_ID"),
    "WS_XML_READER_RAW_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_READER_ENCODING", SimStruct)),)), name="WS_XML_READER_RAW_ENCODING", pack=False, align=None),
    "WS_XML_READER_STREAM_INPUT": SimStruct(OrderedDict((("input", SimTypeRef("WS_XML_READER_INPUT", SimStruct)), ("readCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "bytes", "maxSize", "actualSize", "asyncContext", "error"]), offset=0)), ("readCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_XML_READER_STREAM_INPUT", pack=False, align=None),
    "WS_XML_READER_TEXT_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_READER_ENCODING", SimStruct)), ("charSet", SimTypeInt(signed=False, label="WS_CHARSET")),)), name="WS_XML_READER_TEXT_ENCODING", pack=False, align=None),
    "WS_XML_SECURITY_TOKEN_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_XML_SECURITY_TOKEN_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_SECURITY_TOKEN_PROPERTY", pack=False, align=None),
    "WS_XML_SECURITY_TOKEN_PROPERTY_ID": SimTypeInt(signed=False, label="WS_XML_SECURITY_TOKEN_PROPERTY_ID"),
    "WS_XML_STRING": SimStruct(OrderedDict((("length", SimTypeInt(signed=False, label="UInt32")), ("bytes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dictionary", SimTypePointer(SimTypeRef("WS_XML_DICTIONARY", SimStruct), offset=0)), ("id", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_STRING", pack=False, align=None),
    "WS_XML_STRING_DESCRIPTION": SimStruct(OrderedDict((("minByteCount", SimTypeInt(signed=False, label="UInt32")), ("maxByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_STRING_DESCRIPTION", pack=False, align=None),
    "WS_XML_TEXT": SimStruct(OrderedDict((("textType", SimTypeInt(signed=False, label="WS_XML_TEXT_TYPE")),)), name="WS_XML_TEXT", pack=False, align=None),
    "WS_XML_TEXT_NODE": SimStruct(OrderedDict((("node", SimTypeRef("WS_XML_NODE", SimStruct)), ("text", SimTypePointer(SimTypeRef("WS_XML_TEXT", SimStruct), offset=0)),)), name="WS_XML_TEXT_NODE", pack=False, align=None),
    "WS_XML_TEXT_TYPE": SimTypeInt(signed=False, label="WS_XML_TEXT_TYPE"),
    "WS_XML_TIMESPAN_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeRef("WS_TIMESPAN", SimStruct)),)), name="WS_XML_TIMESPAN_TEXT", pack=False, align=None),
    "WS_XML_TOKEN_MESSAGE_SECURITY_BINDING": SimStruct(OrderedDict((("binding", SimTypeRef("WS_SECURITY_BINDING", SimStruct)), ("bindingUsage", SimTypeInt(signed=False, label="WS_MESSAGE_SECURITY_USAGE")), ("xmlToken", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="WS_XML_TOKEN_MESSAGE_SECURITY_BINDING", pack=False, align=None),
    "WS_XML_UINT64_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeLongLong(signed=False, label="UInt64")),)), name="WS_XML_UINT64_TEXT", pack=False, align=None),
    "WS_XML_UNIQUE_ID_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeBottom(label="Guid")),)), name="WS_XML_UNIQUE_ID_TEXT", pack=False, align=None),
    "WS_XML_UTF16_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("bytes", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("byteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_UTF16_TEXT", pack=False, align=None),
    "WS_XML_UTF8_TEXT": SimStruct(OrderedDict((("text", SimTypeRef("WS_XML_TEXT", SimStruct)), ("value", SimTypeRef("WS_XML_STRING", SimStruct)),)), name="WS_XML_UTF8_TEXT", pack=False, align=None),
    "WS_XML_WRITER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "WS_XML_WRITER_BINARY_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_WRITER_ENCODING", SimStruct)), ("staticDictionary", SimTypePointer(SimTypeRef("WS_XML_DICTIONARY", SimStruct), offset=0)), ("dynamicStringCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_XML_STRING", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "string", "found", "id", "error"]), offset=0)), ("dynamicStringCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_XML_WRITER_BINARY_ENCODING", pack=False, align=None),
    "WS_XML_WRITER_BUFFER_OUTPUT": SimStruct(OrderedDict((("output", SimTypeRef("WS_XML_WRITER_OUTPUT", SimStruct)),)), name="WS_XML_WRITER_BUFFER_OUTPUT", pack=False, align=None),
    "WS_XML_WRITER_ENCODING": SimStruct(OrderedDict((("encodingType", SimTypeInt(signed=False, label="WS_XML_WRITER_ENCODING_TYPE")),)), name="WS_XML_WRITER_ENCODING", pack=False, align=None),
    "WS_XML_WRITER_ENCODING_TYPE": SimTypeInt(signed=False, label="WS_XML_WRITER_ENCODING_TYPE"),
    "WS_XML_WRITER_MTOM_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_WRITER_ENCODING", SimStruct)), ("textEncoding", SimTypePointer(SimTypeRef("WS_XML_WRITER_ENCODING", SimStruct), offset=0)), ("writeMimeHeader", SimTypeInt(signed=True, label="Int32")), ("boundary", SimTypeRef("WS_STRING", SimStruct)), ("startInfo", SimTypeRef("WS_STRING", SimStruct)), ("startUri", SimTypeRef("WS_STRING", SimStruct)), ("maxInlineByteCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_WRITER_MTOM_ENCODING", pack=False, align=None),
    "WS_XML_WRITER_OUTPUT": SimStruct(OrderedDict((("outputType", SimTypeInt(signed=False, label="WS_XML_WRITER_OUTPUT_TYPE")),)), name="WS_XML_WRITER_OUTPUT", pack=False, align=None),
    "WS_XML_WRITER_OUTPUT_TYPE": SimTypeInt(signed=False, label="WS_XML_WRITER_OUTPUT_TYPE"),
    "WS_XML_WRITER_PROPERTIES": SimStruct(OrderedDict((("properties", SimTypePointer(SimTypeRef("WS_XML_WRITER_PROPERTY", SimStruct), offset=0)), ("propertyCount", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_WRITER_PROPERTIES", pack=False, align=None),
    "WS_XML_WRITER_PROPERTY": SimStruct(OrderedDict((("id", SimTypeInt(signed=False, label="WS_XML_WRITER_PROPERTY_ID")), ("value", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("valueSize", SimTypeInt(signed=False, label="UInt32")),)), name="WS_XML_WRITER_PROPERTY", pack=False, align=None),
    "WS_XML_WRITER_PROPERTY_ID": SimTypeInt(signed=False, label="WS_XML_WRITER_PROPERTY_ID"),
    "WS_XML_WRITER_RAW_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_WRITER_ENCODING", SimStruct)),)), name="WS_XML_WRITER_RAW_ENCODING", pack=False, align=None),
    "WS_XML_WRITER_STREAM_OUTPUT": SimStruct(OrderedDict((("output", SimTypeRef("WS_XML_WRITER_OUTPUT", SimStruct)), ("writeCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("WS_BYTES", SimStruct), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WS_ASYNC_CONTEXT", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackState", "buffers", "count", "asyncContext", "error"]), offset=0)), ("writeCallbackState", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WS_XML_WRITER_STREAM_OUTPUT", pack=False, align=None),
    "WS_XML_WRITER_TEXT_ENCODING": SimStruct(OrderedDict((("encoding", SimTypeRef("WS_XML_WRITER_ENCODING", SimStruct)), ("charSet", SimTypeInt(signed=False, label="WS_CHARSET")),)), name="WS_XML_WRITER_TEXT_ENCODING", pack=False, align=None),
    "WTA_OPTIONS": SimStruct(OrderedDict((("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwMask", SimTypeInt(signed=False, label="UInt32")),)), name="WTA_OPTIONS", pack=False, align=None),
    "WTD_GENERIC_CHAIN_POLICY_CREATE_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "cbSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("hChainEngine", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("pChainPara", SimTypePointer(SimTypeRef("CERT_CHAIN_PARA", SimStruct), offset=0)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("pvReserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WTD_GENERIC_CHAIN_POLICY_CREATE_INFO", pack=False, align=None),
    "WTD_GENERIC_CHAIN_POLICY_DATA": SimStruct(OrderedDict((("Anonymous", SimUnion({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "cbSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("pSignerChainInfo", SimTypePointer(SimTypeRef("WTD_GENERIC_CHAIN_POLICY_CREATE_INFO", SimStruct), offset=0)), ("pCounterSignerChainInfo", SimTypePointer(SimTypeRef("WTD_GENERIC_CHAIN_POLICY_CREATE_INFO", SimStruct), offset=0)), ("pfnPolicyCallback", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CRYPT_PROVIDER_DATA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("WTD_GENERIC_CHAIN_POLICY_SIGNER_INFO", SimStruct), offset=0), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProvData", "dwStepError", "dwRegPolicySettings", "cSigner", "rgpSigner", "pvPolicyArg"]), offset=0)), ("pvPolicyArg", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WTD_GENERIC_CHAIN_POLICY_DATA", pack=False, align=None),
    "WTD_GENERIC_CHAIN_POLICY_SIGNER_INFO": SimStruct(OrderedDict((("Anonymous", SimUnion({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "cbSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("pChainContext", SimTypePointer(SimTypeRef("CERT_CHAIN_CONTEXT", SimStruct), offset=0)), ("dwSignerType", SimTypeInt(signed=False, label="UInt32")), ("pMsgSignerInfo", SimTypePointer(SimTypeRef("CMSG_SIGNER_INFO", SimStruct), offset=0)), ("dwError", SimTypeInt(signed=False, label="UInt32")), ("cCounterSigner", SimTypeInt(signed=False, label="UInt32")), ("rgpCounterSigner", SimTypePointer(SimTypePointer(SimTypeRef("WTD_GENERIC_CHAIN_POLICY_SIGNER_INFO", SimStruct), offset=0), offset=0)),)), name="WTD_GENERIC_CHAIN_POLICY_SIGNER_INFO", pack=False, align=None),
    "WTSCLIENTA": SimStruct(OrderedDict((("ClientName", SimTypeArray(SimTypeChar(label="SByte"), 21)), ("Domain", SimTypeArray(SimTypeChar(label="SByte"), 18)), ("UserName", SimTypeArray(SimTypeChar(label="SByte"), 21)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("InitialProgram", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("EncryptionLevel", SimTypeChar(label="Byte")), ("ClientAddressFamily", SimTypeInt(signed=False, label="UInt32")), ("ClientAddress", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 31)), ("HRes", SimTypeShort(signed=False, label="UInt16")), ("VRes", SimTypeShort(signed=False, label="UInt16")), ("ColorDepth", SimTypeShort(signed=False, label="UInt16")), ("ClientDirectory", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("ClientBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareId", SimTypeInt(signed=False, label="UInt32")), ("ClientProductId", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountHost", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountClient", SimTypeShort(signed=False, label="UInt16")), ("OutBufLength", SimTypeShort(signed=False, label="UInt16")), ("DeviceId", SimTypeArray(SimTypeChar(label="SByte"), 261)),)), name="WTSCLIENTA", pack=False, align=None),
    "WTSCLIENTW": SimStruct(OrderedDict((("ClientName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("Domain", SimTypeArray(SimTypeChar(label="Char"), 18)), ("UserName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 261)), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 261)), ("EncryptionLevel", SimTypeChar(label="Byte")), ("ClientAddressFamily", SimTypeInt(signed=False, label="UInt32")), ("ClientAddress", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 31)), ("HRes", SimTypeShort(signed=False, label="UInt16")), ("VRes", SimTypeShort(signed=False, label="UInt16")), ("ColorDepth", SimTypeShort(signed=False, label="UInt16")), ("ClientDirectory", SimTypeArray(SimTypeChar(label="Char"), 261)), ("ClientBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("ClientHardwareId", SimTypeInt(signed=False, label="UInt32")), ("ClientProductId", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountHost", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountClient", SimTypeShort(signed=False, label="UInt16")), ("OutBufLength", SimTypeShort(signed=False, label="UInt16")), ("DeviceId", SimTypeArray(SimTypeChar(label="Char"), 261)),)), name="WTSCLIENTW", pack=False, align=None),
    "WTSCONFIGINFOA": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("fConnectClientDrivesAtLogon", SimTypeInt(signed=False, label="UInt32")), ("fConnectPrinterAtLogon", SimTypeInt(signed=False, label="UInt32")), ("fDisablePrinterRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableDefaultMainClientPrinter", SimTypeInt(signed=False, label="UInt32")), ("ShadowSettings", SimTypeInt(signed=False, label="UInt32")), ("LogonUserName", SimTypeArray(SimTypeChar(label="SByte"), 21)), ("LogonDomain", SimTypeArray(SimTypeChar(label="SByte"), 18)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("InitialProgram", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("ApplicationName", SimTypeArray(SimTypeChar(label="SByte"), 261)),)), name="WTSCONFIGINFOA", pack=False, align=None),
    "WTSCONFIGINFOW": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("fConnectClientDrivesAtLogon", SimTypeInt(signed=False, label="UInt32")), ("fConnectPrinterAtLogon", SimTypeInt(signed=False, label="UInt32")), ("fDisablePrinterRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableDefaultMainClientPrinter", SimTypeInt(signed=False, label="UInt32")), ("ShadowSettings", SimTypeInt(signed=False, label="UInt32")), ("LogonUserName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("LogonDomain", SimTypeArray(SimTypeChar(label="Char"), 18)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 261)), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 261)), ("ApplicationName", SimTypeArray(SimTypeChar(label="Char"), 261)),)), name="WTSCONFIGINFOW", pack=False, align=None),
    "WTSINFOA": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("IncomingBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingBytes", SimTypeInt(signed=False, label="UInt32")), ("IncomingFrames", SimTypeInt(signed=False, label="UInt32")), ("OutgoingFrames", SimTypeInt(signed=False, label="UInt32")), ("IncomingCompressedBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingCompressedBy", SimTypeInt(signed=False, label="UInt32")), ("WinStationName", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("Domain", SimTypeArray(SimTypeChar(label="SByte"), 17)), ("UserName", SimTypeArray(SimTypeChar(label="SByte"), 21)), ("ConnectTime", SimTypeLongLong(signed=True, label="Int64")), ("DisconnectTime", SimTypeLongLong(signed=True, label="Int64")), ("LastInputTime", SimTypeLongLong(signed=True, label="Int64")), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("CurrentTime", SimTypeLongLong(signed=True, label="Int64")),)), name="WTSINFOA", pack=False, align=None),
    "WTSINFOEXA": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="UInt32")), ("Data", SimUnion({"WTSInfoExLevel1": SimTypeRef("WTSINFOEX_LEVEL1_A", SimStruct)}, name="<anon>", label="None")),)), name="WTSINFOEXA", pack=False, align=None),
    "WTSINFOEXW": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="UInt32")), ("Data", SimUnion({"WTSInfoExLevel1": SimTypeRef("WTSINFOEX_LEVEL1_W", SimStruct)}, name="<anon>", label="None")),)), name="WTSINFOEXW", pack=False, align=None),
    "WTSINFOEX_LEVEL1_A": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("SessionState", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")), ("SessionFlags", SimTypeInt(signed=True, label="Int32")), ("WinStationName", SimTypeArray(SimTypeChar(label="SByte"), 33)), ("UserName", SimTypeArray(SimTypeChar(label="SByte"), 21)), ("DomainName", SimTypeArray(SimTypeChar(label="SByte"), 18)), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("ConnectTime", SimTypeLongLong(signed=True, label="Int64")), ("DisconnectTime", SimTypeLongLong(signed=True, label="Int64")), ("LastInputTime", SimTypeLongLong(signed=True, label="Int64")), ("CurrentTime", SimTypeLongLong(signed=True, label="Int64")), ("IncomingBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingBytes", SimTypeInt(signed=False, label="UInt32")), ("IncomingFrames", SimTypeInt(signed=False, label="UInt32")), ("OutgoingFrames", SimTypeInt(signed=False, label="UInt32")), ("IncomingCompressedBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingCompressedBytes", SimTypeInt(signed=False, label="UInt32")),)), name="WTSINFOEX_LEVEL1_A", pack=False, align=None),
    "WTSINFOEX_LEVEL1_W": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("SessionState", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")), ("SessionFlags", SimTypeInt(signed=True, label="Int32")), ("WinStationName", SimTypeArray(SimTypeChar(label="Char"), 33)), ("UserName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("DomainName", SimTypeArray(SimTypeChar(label="Char"), 18)), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("ConnectTime", SimTypeLongLong(signed=True, label="Int64")), ("DisconnectTime", SimTypeLongLong(signed=True, label="Int64")), ("LastInputTime", SimTypeLongLong(signed=True, label="Int64")), ("CurrentTime", SimTypeLongLong(signed=True, label="Int64")), ("IncomingBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingBytes", SimTypeInt(signed=False, label="UInt32")), ("IncomingFrames", SimTypeInt(signed=False, label="UInt32")), ("OutgoingFrames", SimTypeInt(signed=False, label="UInt32")), ("IncomingCompressedBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingCompressedBytes", SimTypeInt(signed=False, label="UInt32")),)), name="WTSINFOEX_LEVEL1_W", pack=False, align=None),
    "WTSINFOEX_LEVEL_A": SimUnion({"WTSInfoExLevel1": SimTypeRef("WTSINFOEX_LEVEL1_A", SimStruct)}, name="<anon>", label="None"),
    "WTSINFOEX_LEVEL_W": SimUnion({"WTSInfoExLevel1": SimTypeRef("WTSINFOEX_LEVEL1_W", SimStruct)}, name="<anon>", label="None"),
    "WTSINFOW": SimStruct(OrderedDict((("State", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("IncomingBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingBytes", SimTypeInt(signed=False, label="UInt32")), ("IncomingFrames", SimTypeInt(signed=False, label="UInt32")), ("OutgoingFrames", SimTypeInt(signed=False, label="UInt32")), ("IncomingCompressedBytes", SimTypeInt(signed=False, label="UInt32")), ("OutgoingCompressedBytes", SimTypeInt(signed=False, label="UInt32")), ("WinStationName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("Domain", SimTypeArray(SimTypeChar(label="Char"), 17)), ("UserName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("ConnectTime", SimTypeLongLong(signed=True, label="Int64")), ("DisconnectTime", SimTypeLongLong(signed=True, label="Int64")), ("LastInputTime", SimTypeLongLong(signed=True, label="Int64")), ("LogonTime", SimTypeLongLong(signed=True, label="Int64")), ("CurrentTime", SimTypeLongLong(signed=True, label="Int64")),)), name="WTSINFOW", pack=False, align=None),
    "WTSLISTENERCONFIGA": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("fEnableListener", SimTypeInt(signed=False, label="UInt32")), ("MaxConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("fPromptForPassword", SimTypeInt(signed=False, label="UInt32")), ("fInheritColorDepth", SimTypeInt(signed=False, label="UInt32")), ("ColorDepth", SimTypeInt(signed=False, label="UInt32")), ("fInheritBrokenTimeoutSettings", SimTypeInt(signed=False, label="UInt32")), ("BrokenTimeoutSettings", SimTypeInt(signed=False, label="UInt32")), ("fDisablePrinterRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableDriveRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableComPortRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableLPTPortRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableClipboardRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableAudioRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisablePNPRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableDefaultMainClientPrinter", SimTypeInt(signed=False, label="UInt32")), ("LanAdapter", SimTypeInt(signed=False, label="UInt32")), ("PortNumber", SimTypeInt(signed=False, label="UInt32")), ("fInheritShadowSettings", SimTypeInt(signed=False, label="UInt32")), ("ShadowSettings", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsConnection", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsDisconnection", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsIdle", SimTypeInt(signed=False, label="UInt32")), ("SecurityLayer", SimTypeInt(signed=False, label="UInt32")), ("MinEncryptionLevel", SimTypeInt(signed=False, label="UInt32")), ("UserAuthentication", SimTypeInt(signed=False, label="UInt32")), ("Comment", SimTypeArray(SimTypeChar(label="SByte"), 61)), ("LogonUserName", SimTypeArray(SimTypeChar(label="SByte"), 21)), ("LogonDomain", SimTypeArray(SimTypeChar(label="SByte"), 18)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("InitialProgram", SimTypeArray(SimTypeChar(label="SByte"), 261)),)), name="WTSLISTENERCONFIGA", pack=False, align=None),
    "WTSLISTENERCONFIGW": SimStruct(OrderedDict((("version", SimTypeInt(signed=False, label="UInt32")), ("fEnableListener", SimTypeInt(signed=False, label="UInt32")), ("MaxConnectionCount", SimTypeInt(signed=False, label="UInt32")), ("fPromptForPassword", SimTypeInt(signed=False, label="UInt32")), ("fInheritColorDepth", SimTypeInt(signed=False, label="UInt32")), ("ColorDepth", SimTypeInt(signed=False, label="UInt32")), ("fInheritBrokenTimeoutSettings", SimTypeInt(signed=False, label="UInt32")), ("BrokenTimeoutSettings", SimTypeInt(signed=False, label="UInt32")), ("fDisablePrinterRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableDriveRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableComPortRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableLPTPortRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableClipboardRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableAudioRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisablePNPRedirection", SimTypeInt(signed=False, label="UInt32")), ("fDisableDefaultMainClientPrinter", SimTypeInt(signed=False, label="UInt32")), ("LanAdapter", SimTypeInt(signed=False, label="UInt32")), ("PortNumber", SimTypeInt(signed=False, label="UInt32")), ("fInheritShadowSettings", SimTypeInt(signed=False, label="UInt32")), ("ShadowSettings", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsConnection", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsDisconnection", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsIdle", SimTypeInt(signed=False, label="UInt32")), ("SecurityLayer", SimTypeInt(signed=False, label="UInt32")), ("MinEncryptionLevel", SimTypeInt(signed=False, label="UInt32")), ("UserAuthentication", SimTypeInt(signed=False, label="UInt32")), ("Comment", SimTypeArray(SimTypeChar(label="Char"), 61)), ("LogonUserName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("LogonDomain", SimTypeArray(SimTypeChar(label="Char"), 18)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 261)), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 261)),)), name="WTSLISTENERCONFIGW", pack=False, align=None),
    "WTSSBX_ADDRESS_FAMILY": SimTypeInt(signed=False, label="WTSSBX_ADDRESS_FAMILY"),
    "WTSSBX_IP_ADDRESS": SimStruct(OrderedDict((("AddressFamily", SimTypeInt(signed=False, label="WTSSBX_ADDRESS_FAMILY")), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("PortNumber", SimTypeShort(signed=False, label="UInt16")), ("dwScope", SimTypeInt(signed=False, label="UInt32")),)), name="WTSSBX_IP_ADDRESS", pack=False, align=None),
    "WTSSBX_MACHINE_CONNECT_INFO": SimStruct(OrderedDict((("wczMachineFQDN", SimTypeArray(SimTypeChar(label="Char"), 257)), ("wczMachineNetBiosName", SimTypeArray(SimTypeChar(label="Char"), 17)), ("dwNumOfIPAddr", SimTypeInt(signed=False, label="UInt32")), ("IPaddr", SimTypeArray(SimTypeRef("WTSSBX_IP_ADDRESS", SimStruct), 12)),)), name="WTSSBX_MACHINE_CONNECT_INFO", pack=False, align=None),
    "WTSSBX_MACHINE_DRAIN": SimTypeInt(signed=False, label="WTSSBX_MACHINE_DRAIN"),
    "WTSSBX_MACHINE_INFO": SimStruct(OrderedDict((("ClientConnectInfo", SimTypeRef("WTSSBX_MACHINE_CONNECT_INFO", SimStruct)), ("wczFarmName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("InternalIPAddress", SimTypeRef("WTSSBX_IP_ADDRESS", SimStruct)), ("dwMaxSessionsLimit", SimTypeInt(signed=False, label="UInt32")), ("ServerWeight", SimTypeInt(signed=False, label="UInt32")), ("SingleSessionMode", SimTypeInt(signed=False, label="WTSSBX_MACHINE_SESSION_MODE")), ("InDrain", SimTypeInt(signed=False, label="WTSSBX_MACHINE_DRAIN")), ("MachineState", SimTypeInt(signed=False, label="WTSSBX_MACHINE_STATE")),)), name="WTSSBX_MACHINE_INFO", pack=False, align=None),
    "WTSSBX_MACHINE_SESSION_MODE": SimTypeInt(signed=False, label="WTSSBX_MACHINE_SESSION_MODE"),
    "WTSSBX_MACHINE_STATE": SimTypeInt(signed=False, label="WTSSBX_MACHINE_STATE"),
    "WTSSBX_NOTIFICATION_TYPE": SimTypeInt(signed=False, label="WTSSBX_NOTIFICATION_TYPE"),
    "WTSSBX_SESSION_INFO": SimStruct(OrderedDict((("wszUserName", SimTypeArray(SimTypeChar(label="Char"), 105)), ("wszDomainName", SimTypeArray(SimTypeChar(label="Char"), 257)), ("ApplicationType", SimTypeArray(SimTypeChar(label="Char"), 257)), ("dwSessionId", SimTypeInt(signed=False, label="UInt32")), ("CreateTime", SimTypeRef("FILETIME", SimStruct)), ("DisconnectTime", SimTypeRef("FILETIME", SimStruct)), ("SessionState", SimTypeInt(signed=False, label="WTSSBX_SESSION_STATE")),)), name="WTSSBX_SESSION_INFO", pack=False, align=None),
    "WTSSBX_SESSION_STATE": SimTypeInt(signed=False, label="WTSSBX_SESSION_STATE"),
    "WTSSESSION_NOTIFICATION": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("dwSessionId", SimTypeInt(signed=False, label="UInt32")),)), name="WTSSESSION_NOTIFICATION", pack=False, align=None),
    "WTSUSERCONFIGA": SimStruct(OrderedDict((("Source", SimTypeInt(signed=False, label="UInt32")), ("InheritInitialProgram", SimTypeInt(signed=False, label="UInt32")), ("AllowLogonTerminalServer", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsConnections", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsDisconnections", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsIdle", SimTypeInt(signed=False, label="UInt32")), ("DeviceClientDrives", SimTypeInt(signed=False, label="UInt32")), ("DeviceClientPrinters", SimTypeInt(signed=False, label="UInt32")), ("ClientDefaultPrinter", SimTypeInt(signed=False, label="UInt32")), ("BrokenTimeoutSettings", SimTypeInt(signed=False, label="UInt32")), ("ReconnectSettings", SimTypeInt(signed=False, label="UInt32")), ("ShadowingSettings", SimTypeInt(signed=False, label="UInt32")), ("TerminalServerRemoteHomeDir", SimTypeInt(signed=False, label="UInt32")), ("InitialProgram", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("TerminalServerProfilePath", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("TerminalServerHomeDir", SimTypeArray(SimTypeChar(label="SByte"), 261)), ("TerminalServerHomeDirDrive", SimTypeArray(SimTypeChar(label="SByte"), 4)),)), name="WTSUSERCONFIGA", pack=False, align=None),
    "WTSUSERCONFIGW": SimStruct(OrderedDict((("Source", SimTypeInt(signed=False, label="UInt32")), ("InheritInitialProgram", SimTypeInt(signed=False, label="UInt32")), ("AllowLogonTerminalServer", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsConnections", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsDisconnections", SimTypeInt(signed=False, label="UInt32")), ("TimeoutSettingsIdle", SimTypeInt(signed=False, label="UInt32")), ("DeviceClientDrives", SimTypeInt(signed=False, label="UInt32")), ("DeviceClientPrinters", SimTypeInt(signed=False, label="UInt32")), ("ClientDefaultPrinter", SimTypeInt(signed=False, label="UInt32")), ("BrokenTimeoutSettings", SimTypeInt(signed=False, label="UInt32")), ("ReconnectSettings", SimTypeInt(signed=False, label="UInt32")), ("ShadowingSettings", SimTypeInt(signed=False, label="UInt32")), ("TerminalServerRemoteHomeDir", SimTypeInt(signed=False, label="UInt32")), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 261)), ("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 261)), ("TerminalServerProfilePath", SimTypeArray(SimTypeChar(label="Char"), 261)), ("TerminalServerHomeDir", SimTypeArray(SimTypeChar(label="Char"), 261)), ("TerminalServerHomeDirDrive", SimTypeArray(SimTypeChar(label="Char"), 4)),)), name="WTSUSERCONFIGW", pack=False, align=None),
    "WTS_ALPHATYPE": SimTypeInt(signed=False, label="WTS_ALPHATYPE"),
    "WTS_CACHEFLAGS": SimTypeInt(signed=False, label="WTS_CACHEFLAGS"),
    "WTS_CACHE_STATS": SimStruct(OrderedDict((("Specific", SimTypeInt(signed=False, label="UInt32")), ("Data", SimUnion({"ProtocolCache": SimTypeArray(SimTypeRef("WTS_PROTOCOL_CACHE", SimStruct), 4), "TShareCacheStats": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 20)}, name="<anon>", label="None")), ("ProtocolType", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeShort(signed=False, label="UInt16")),)), name="WTS_CACHE_STATS", pack=False, align=None),
    "WTS_CACHE_STATS_UN": SimUnion({"ProtocolCache": SimTypeArray(SimTypeRef("WTS_PROTOCOL_CACHE", SimStruct), 4), "TShareCacheStats": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 20)}, name="<anon>", label="None"),
    "WTS_CERT_TYPE": SimTypeInt(signed=False, label="WTS_CERT_TYPE"),
    "WTS_CLIENT_ADDRESS": SimStruct(OrderedDict((("AddressFamily", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="WTS_CLIENT_ADDRESS", pack=False, align=None),
    "WTS_CLIENT_DATA": SimStruct(OrderedDict((("fDisableCtrlAltDel", SimTypeChar(label="Byte")), ("fDoubleClickDetect", SimTypeChar(label="Byte")), ("fEnableWindowsKey", SimTypeChar(label="Byte")), ("fHideTitleBar", SimTypeChar(label="Byte")), ("fInheritAutoLogon", SimTypeInt(signed=True, label="Int32")), ("fPromptForPassword", SimTypeChar(label="Byte")), ("fUsingSavedCreds", SimTypeChar(label="Byte")), ("Domain", SimTypeArray(SimTypeChar(label="Char"), 256)), ("UserName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Password", SimTypeArray(SimTypeChar(label="Char"), 256)), ("fPasswordIsScPin", SimTypeChar(label="Byte")), ("fInheritInitialProgram", SimTypeInt(signed=True, label="Int32")), ("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 257)), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 257)), ("fMaximizeShell", SimTypeChar(label="Byte")), ("EncryptionLevel", SimTypeChar(label="Byte")), ("PerformanceFlags", SimTypeInt(signed=False, label="UInt32")), ("ProtocolName", SimTypeArray(SimTypeChar(label="Char"), 9)), ("ProtocolType", SimTypeShort(signed=False, label="UInt16")), ("fInheritColorDepth", SimTypeInt(signed=True, label="Int32")), ("HRes", SimTypeShort(signed=False, label="UInt16")), ("VRes", SimTypeShort(signed=False, label="UInt16")), ("ColorDepth", SimTypeShort(signed=False, label="UInt16")), ("DisplayDriverName", SimTypeArray(SimTypeChar(label="Char"), 9)), ("DisplayDeviceName", SimTypeArray(SimTypeChar(label="Char"), 20)), ("fMouse", SimTypeChar(label="Byte")), ("KeyboardLayout", SimTypeInt(signed=False, label="UInt32")), ("KeyboardType", SimTypeInt(signed=False, label="UInt32")), ("KeyboardSubType", SimTypeInt(signed=False, label="UInt32")), ("KeyboardFunctionKey", SimTypeInt(signed=False, label="UInt32")), ("imeFileName", SimTypeArray(SimTypeChar(label="Char"), 33)), ("ActiveInputLocale", SimTypeInt(signed=False, label="UInt32")), ("fNoAudioPlayback", SimTypeChar(label="Byte")), ("fRemoteConsoleAudio", SimTypeChar(label="Byte")), ("AudioDriverName", SimTypeArray(SimTypeChar(label="Char"), 9)), ("ClientTimeZone", SimTypeRef("WTS_TIME_ZONE_INFORMATION", SimStruct)), ("ClientName", SimTypeArray(SimTypeChar(label="Char"), 21)), ("SerialNumber", SimTypeInt(signed=False, label="UInt32")), ("ClientAddressFamily", SimTypeInt(signed=False, label="UInt32")), ("ClientAddress", SimTypeArray(SimTypeChar(label="Char"), 31)), ("ClientSockAddress", SimTypeRef("WTS_SOCKADDR", SimStruct)), ("ClientDirectory", SimTypeArray(SimTypeChar(label="Char"), 257)), ("ClientBuildNumber", SimTypeInt(signed=False, label="UInt32")), ("ClientProductId", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountHost", SimTypeShort(signed=False, label="UInt16")), ("OutBufCountClient", SimTypeShort(signed=False, label="UInt16")), ("OutBufLength", SimTypeShort(signed=False, label="UInt16")), ("ClientSessionId", SimTypeInt(signed=False, label="UInt32")), ("ClientDigProductId", SimTypeArray(SimTypeChar(label="Char"), 33)), ("fDisableCpm", SimTypeChar(label="Byte")), ("fDisableCdm", SimTypeChar(label="Byte")), ("fDisableCcm", SimTypeChar(label="Byte")), ("fDisableLPT", SimTypeChar(label="Byte")), ("fDisableClip", SimTypeChar(label="Byte")), ("fDisablePNP", SimTypeChar(label="Byte")),)), name="WTS_CLIENT_DATA", pack=False, align=None),
    "WTS_CLIENT_DISPLAY": SimStruct(OrderedDict((("HorizontalResolution", SimTypeInt(signed=False, label="UInt32")), ("VerticalResolution", SimTypeInt(signed=False, label="UInt32")), ("ColorDepth", SimTypeInt(signed=False, label="UInt32")),)), name="WTS_CLIENT_DISPLAY", pack=False, align=None),
    "WTS_CONFIG_CLASS": SimTypeInt(signed=False, label="WTS_CONFIG_CLASS"),
    "WTS_CONFIG_SOURCE": SimTypeInt(signed=False, label="WTS_CONFIG_SOURCE"),
    "WTS_CONNECTSTATE_CLASS": SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS"),
    "WTS_CONTEXTFLAGS": SimTypeInt(signed=False, label="WTS_CONTEXTFLAGS"),
    "WTS_DISPLAY_IOCTL": SimStruct(OrderedDict((("pDisplayIOCtlData", SimTypeArray(SimTypeChar(label="Byte"), 256)), ("cbDisplayIOCtlData", SimTypeInt(signed=False, label="UInt32")),)), name="WTS_DISPLAY_IOCTL", pack=False, align=None),
    "WTS_FLAGS": SimTypeInt(signed=False, label="WTS_FLAGS"),
    "WTS_INFO_CLASS": SimTypeInt(signed=False, label="WTS_INFO_CLASS"),
    "WTS_LICENSE_CAPABILITIES": SimStruct(OrderedDict((("KeyExchangeAlg", SimTypeInt(signed=False, label="UInt32")), ("ProtocolVer", SimTypeInt(signed=False, label="UInt32")), ("fAuthenticateServer", SimTypeInt(signed=True, label="Int32")), ("CertType", SimTypeInt(signed=False, label="WTS_CERT_TYPE")), ("cbClientName", SimTypeInt(signed=False, label="UInt32")), ("rgbClientName", SimTypeArray(SimTypeChar(label="Byte"), 42)),)), name="WTS_LICENSE_CAPABILITIES", pack=False, align=None),
    "WTS_LOGON_ERROR_REDIRECTOR_RESPONSE": SimTypeInt(signed=False, label="WTS_LOGON_ERROR_REDIRECTOR_RESPONSE"),
    "WTS_POLICY_DATA": SimStruct(OrderedDict((("fDisableEncryption", SimTypeChar(label="Byte")), ("fDisableAutoReconnect", SimTypeChar(label="Byte")), ("ColorDepth", SimTypeInt(signed=False, label="UInt32")), ("MinEncryptionLevel", SimTypeChar(label="Byte")), ("fDisableCpm", SimTypeChar(label="Byte")), ("fDisableCdm", SimTypeChar(label="Byte")), ("fDisableCcm", SimTypeChar(label="Byte")), ("fDisableLPT", SimTypeChar(label="Byte")), ("fDisableClip", SimTypeChar(label="Byte")), ("fDisablePNPRedir", SimTypeChar(label="Byte")),)), name="WTS_POLICY_DATA", pack=False, align=None),
    "WTS_PROCESS_INFOA": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("pProcessName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WTS_PROCESS_INFOA", pack=False, align=None),
    "WTS_PROCESS_INFOW": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("pProcessName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="WTS_PROCESS_INFOW", pack=False, align=None),
    "WTS_PROCESS_INFO_EXA": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("pProcessName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfThreads", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PagefileUsage", SimTypeInt(signed=False, label="UInt32")), ("PeakPagefileUsage", SimTypeInt(signed=False, label="UInt32")), ("WorkingSetSize", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypeInt(signed=False, label="UInt32")), ("UserTime", SimTypeLongLong(signed=True, label="Int64")), ("KernelTime", SimTypeLongLong(signed=True, label="Int64")),)), name="WTS_PROCESS_INFO_EXA", pack=False, align=None),
    "WTS_PROCESS_INFO_EXW": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("pProcessName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfThreads", SimTypeInt(signed=False, label="UInt32")), ("HandleCount", SimTypeInt(signed=False, label="UInt32")), ("PagefileUsage", SimTypeInt(signed=False, label="UInt32")), ("PeakPagefileUsage", SimTypeInt(signed=False, label="UInt32")), ("WorkingSetSize", SimTypeInt(signed=False, label="UInt32")), ("PeakWorkingSetSize", SimTypeInt(signed=False, label="UInt32")), ("UserTime", SimTypeLongLong(signed=True, label="Int64")), ("KernelTime", SimTypeLongLong(signed=True, label="Int64")),)), name="WTS_PROCESS_INFO_EXW", pack=False, align=None),
    "WTS_PROPERTY_VALUE": SimStruct(OrderedDict((("Type", SimTypeShort(signed=False, label="UInt16")), ("u", SimUnion({"ulVal": SimTypeInt(signed=False, label="UInt32"), "strVal": SimTypeRef("_strVal_e__Struct", SimStruct), "bVal": SimTypeRef("_bVal_e__Struct", SimStruct), "guidVal": SimTypeBottom(label="Guid")}, name="<anon>", label="None")),)), name="WTS_PROPERTY_VALUE", pack=False, align=None),
    "WTS_PROTOCOL_CACHE": SimStruct(OrderedDict((("CacheReads", SimTypeInt(signed=False, label="UInt32")), ("CacheHits", SimTypeInt(signed=False, label="UInt32")),)), name="WTS_PROTOCOL_CACHE", pack=False, align=None),
    "WTS_PROTOCOL_COUNTERS": SimStruct(OrderedDict((("WdBytes", SimTypeInt(signed=False, label="UInt32")), ("WdFrames", SimTypeInt(signed=False, label="UInt32")), ("WaitForOutBuf", SimTypeInt(signed=False, label="UInt32")), ("Frames", SimTypeInt(signed=False, label="UInt32")), ("Bytes", SimTypeInt(signed=False, label="UInt32")), ("CompressedBytes", SimTypeInt(signed=False, label="UInt32")), ("CompressFlushes", SimTypeInt(signed=False, label="UInt32")), ("Errors", SimTypeInt(signed=False, label="UInt32")), ("Timeouts", SimTypeInt(signed=False, label="UInt32")), ("AsyncFramingError", SimTypeInt(signed=False, label="UInt32")), ("AsyncOverrunError", SimTypeInt(signed=False, label="UInt32")), ("AsyncOverflowError", SimTypeInt(signed=False, label="UInt32")), ("AsyncParityError", SimTypeInt(signed=False, label="UInt32")), ("TdErrors", SimTypeInt(signed=False, label="UInt32")), ("ProtocolType", SimTypeShort(signed=False, label="UInt16")), ("Length", SimTypeShort(signed=False, label="UInt16")), ("Specific", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 100)),)), name="WTS_PROTOCOL_COUNTERS", pack=False, align=None),
    "WTS_PROTOCOL_STATUS": SimStruct(OrderedDict((("Output", SimTypeRef("WTS_PROTOCOL_COUNTERS", SimStruct)), ("Input", SimTypeRef("WTS_PROTOCOL_COUNTERS", SimStruct)), ("Cache", SimTypeRef("WTS_CACHE_STATS", SimStruct)), ("AsyncSignal", SimTypeInt(signed=False, label="UInt32")), ("AsyncSignalMask", SimTypeInt(signed=False, label="UInt32")), ("Counters", SimTypeArray(SimTypeLongLong(signed=True, label="Int64"), 100)),)), name="WTS_PROTOCOL_STATUS", pack=False, align=None),
    "WTS_RCM_DRAIN_STATE": SimTypeInt(signed=False, label="WTS_RCM_DRAIN_STATE"),
    "WTS_RCM_SERVICE_STATE": SimTypeInt(signed=False, label="WTS_RCM_SERVICE_STATE"),
    "WTS_SECURITY_FLAGS": SimTypeInt(signed=False, label="WTS_SECURITY_FLAGS"),
    "WTS_SERVER_INFOA": SimStruct(OrderedDict((("pServerName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WTS_SERVER_INFOA", pack=False, align=None),
    "WTS_SERVER_INFOW": SimStruct(OrderedDict((("pServerName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WTS_SERVER_INFOW", pack=False, align=None),
    "WTS_SERVICE_STATE": SimStruct(OrderedDict((("RcmServiceState", SimTypeInt(signed=False, label="WTS_RCM_SERVICE_STATE")), ("RcmDrainState", SimTypeInt(signed=False, label="WTS_RCM_DRAIN_STATE")),)), name="WTS_SERVICE_STATE", pack=False, align=None),
    "WTS_SESSION_ADDRESS": SimStruct(OrderedDict((("AddressFamily", SimTypeInt(signed=False, label="UInt32")), ("Address", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="WTS_SESSION_ADDRESS", pack=False, align=None),
    "WTS_SESSION_ID": SimStruct(OrderedDict((("SessionUniqueGuid", SimTypeBottom(label="Guid")), ("SessionId", SimTypeInt(signed=False, label="UInt32")),)), name="WTS_SESSION_ID", pack=False, align=None),
    "WTS_SESSION_INFOA": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("pWinStationName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("State", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")),)), name="WTS_SESSION_INFOA", pack=False, align=None),
    "WTS_SESSION_INFOW": SimStruct(OrderedDict((("SessionId", SimTypeInt(signed=False, label="UInt32")), ("pWinStationName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("State", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")),)), name="WTS_SESSION_INFOW", pack=False, align=None),
    "WTS_SESSION_INFO_1A": SimStruct(OrderedDict((("ExecEnvId", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("pSessionName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pHostName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pDomainName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pFarmName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="WTS_SESSION_INFO_1A", pack=False, align=None),
    "WTS_SESSION_INFO_1W": SimStruct(OrderedDict((("ExecEnvId", SimTypeInt(signed=False, label="UInt32")), ("State", SimTypeInt(signed=False, label="WTS_CONNECTSTATE_CLASS")), ("SessionId", SimTypeInt(signed=False, label="UInt32")), ("pSessionName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pHostName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pUserName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pDomainName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pFarmName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="WTS_SESSION_INFO_1W", pack=False, align=None),
    "WTS_SMALL_RECT": SimStruct(OrderedDict((("Left", SimTypeShort(signed=True, label="Int16")), ("Top", SimTypeShort(signed=True, label="Int16")), ("Right", SimTypeShort(signed=True, label="Int16")), ("Bottom", SimTypeShort(signed=True, label="Int16")),)), name="WTS_SMALL_RECT", pack=False, align=None),
    "WTS_SOCKADDR": SimStruct(OrderedDict((("sin_family", SimTypeShort(signed=False, label="UInt16")), ("u", SimUnion({"ipv4": SimTypeRef("_ipv4_e__Struct", SimStruct), "ipv6": SimTypeRef("_ipv6_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="WTS_SOCKADDR", pack=False, align=None),
    "WTS_SYSTEMTIME": SimStruct(OrderedDict((("wYear", SimTypeShort(signed=False, label="UInt16")), ("wMonth", SimTypeShort(signed=False, label="UInt16")), ("wDayOfWeek", SimTypeShort(signed=False, label="UInt16")), ("wDay", SimTypeShort(signed=False, label="UInt16")), ("wHour", SimTypeShort(signed=False, label="UInt16")), ("wMinute", SimTypeShort(signed=False, label="UInt16")), ("wSecond", SimTypeShort(signed=False, label="UInt16")), ("wMilliseconds", SimTypeShort(signed=False, label="UInt16")),)), name="WTS_SYSTEMTIME", pack=False, align=None),
    "WTS_THUMBNAILID": SimStruct(OrderedDict((("rgbKey", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="WTS_THUMBNAILID", pack=False, align=None),
    "WTS_TIME_ZONE_INFORMATION": SimStruct(OrderedDict((("Bias", SimTypeInt(signed=True, label="Int32")), ("StandardName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("StandardDate", SimTypeRef("WTS_SYSTEMTIME", SimStruct)), ("StandardBias", SimTypeInt(signed=True, label="Int32")), ("DaylightName", SimTypeArray(SimTypeChar(label="Char"), 32)), ("DaylightDate", SimTypeRef("WTS_SYSTEMTIME", SimStruct)), ("DaylightBias", SimTypeInt(signed=True, label="Int32")),)), name="WTS_TIME_ZONE_INFORMATION", pack=False, align=None),
    "WTS_TYPE_CLASS": SimTypeInt(signed=False, label="WTS_TYPE_CLASS"),
    "WTS_USER_CREDENTIAL": SimStruct(OrderedDict((("UserName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Password", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Domain", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="WTS_USER_CREDENTIAL", pack=False, align=None),
    "WTS_USER_DATA": SimStruct(OrderedDict((("WorkDirectory", SimTypeArray(SimTypeChar(label="Char"), 257)), ("InitialProgram", SimTypeArray(SimTypeChar(label="Char"), 257)), ("UserTimeZone", SimTypeRef("WTS_TIME_ZONE_INFORMATION", SimStruct)),)), name="WTS_USER_DATA", pack=False, align=None),
    "WTS_VALIDATION_INFORMATIONA": SimStruct(OrderedDict((("ProductInfo", SimTypeRef("PRODUCT_INFOA", SimStruct)), ("License", SimTypeArray(SimTypeChar(label="Byte"), 16384)), ("LicenseLength", SimTypeInt(signed=False, label="UInt32")), ("HardwareID", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("HardwareIDLength", SimTypeInt(signed=False, label="UInt32")),)), name="WTS_VALIDATION_INFORMATIONA", pack=False, align=None),
    "WTS_VALIDATION_INFORMATIONW": SimStruct(OrderedDict((("ProductInfo", SimTypeRef("PRODUCT_INFOW", SimStruct)), ("License", SimTypeArray(SimTypeChar(label="Byte"), 16384)), ("LicenseLength", SimTypeInt(signed=False, label="UInt32")), ("HardwareID", SimTypeArray(SimTypeChar(label="Byte"), 20)), ("HardwareIDLength", SimTypeInt(signed=False, label="UInt32")),)), name="WTS_VALIDATION_INFORMATIONW", pack=False, align=None),
    "WTS_VIRTUAL_CLASS": SimTypeInt(signed=False, label="WTS_VIRTUAL_CLASS"),
    "WWAEXT_SMS_CONSTANTS": SimTypeInt(signed=False, label="WWAEXT_SMS_CONSTANTS"),
    "W_CO_ACTIVATE_VC_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "W_CO_CREATE_VC_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "W_CO_DEACTIVATE_VC_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "W_CO_DELETE_VC_HANDLER": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "WarpMode": SimTypeInt(signed=False, label="WarpMode"),
    "WbemAuthenticationLevelEnum": SimTypeInt(signed=False, label="WbemAuthenticationLevelEnum"),
    "WbemChangeFlagEnum": SimTypeInt(signed=False, label="WbemChangeFlagEnum"),
    "WbemCimtypeEnum": SimTypeInt(signed=False, label="WbemCimtypeEnum"),
    "WbemComparisonFlagEnum": SimTypeInt(signed=False, label="WbemComparisonFlagEnum"),
    "WbemConnectOptionsEnum": SimTypeInt(signed=False, label="WbemConnectOptionsEnum"),
    "WbemErrorEnum": SimTypeInt(signed=False, label="WbemErrorEnum"),
    "WbemFlagEnum": SimTypeInt(signed=False, label="WbemFlagEnum"),
    "WbemImpersonationLevelEnum": SimTypeInt(signed=False, label="WbemImpersonationLevelEnum"),
    "WbemObjectTextFormatEnum": SimTypeInt(signed=False, label="WbemObjectTextFormatEnum"),
    "WbemPrivilegeEnum": SimTypeInt(signed=False, label="WbemPrivilegeEnum"),
    "WbemQueryFlagEnum": SimTypeInt(signed=False, label="WbemQueryFlagEnum"),
    "WbemTextFlagEnum": SimTypeInt(signed=False, label="WbemTextFlagEnum"),
    "WbemTimeout": SimTypeInt(signed=False, label="WbemTimeout"),
    "WcmDataType": SimTypeInt(signed=False, label="WcmDataType"),
    "WcmNamespaceAccess": SimTypeInt(signed=False, label="WcmNamespaceAccess"),
    "WcmNamespaceEnumerationFlags": SimTypeInt(signed=False, label="WcmNamespaceEnumerationFlags"),
    "WcmRestrictionFacets": SimTypeInt(signed=False, label="WcmRestrictionFacets"),
    "WcmSettingType": SimTypeInt(signed=False, label="WcmSettingType"),
    "WcmTargetMode": SimTypeInt(signed=False, label="WcmTargetMode"),
    "WcmUserStatus": SimTypeInt(signed=False, label="WcmUserStatus"),
    "WebEnrollmentFlags": SimTypeInt(signed=False, label="WebEnrollmentFlags"),
    "WebSecurityLevel": SimTypeInt(signed=False, label="WebSecurityLevel"),
    "WeekDays": SimTypeInt(signed=False, label="WeekDays"),
    "WiaTransferParams": SimStruct(OrderedDict((("lMessage", SimTypeInt(signed=True, label="Int32")), ("lPercentComplete", SimTypeInt(signed=True, label="Int32")), ("ulTransferredBytes", SimTypeLongLong(signed=False, label="UInt64")), ("hrErrorStatus", SimTypeInt(signed=True, label="Int32")),)), name="WiaTransferParams", pack=False, align=None),
    "WinHttpRequestAutoLogonPolicy": SimTypeInt(signed=False, label="WinHttpRequestAutoLogonPolicy"),
    "WinHttpRequestOption": SimTypeInt(signed=False, label="WinHttpRequestOption"),
    "WinHttpRequestSecureProtocols": SimTypeInt(signed=False, label="WinHttpRequestSecureProtocols"),
    "WinHttpRequestSslErrorFlags": SimTypeInt(signed=False, label="WinHttpRequestSslErrorFlags"),
    "WindowInteractionState": SimTypeInt(signed=False, label="WindowInteractionState"),
    "WindowVisualState": SimTypeInt(signed=False, label="WindowVisualState"),
    "WindowsMediaLibrarySharingDeviceAuthorizationStatus": SimTypeInt(signed=False, label="WindowsMediaLibrarySharingDeviceAuthorizationStatus"),
    "WitnessTagHelper": SimStruct(OrderedDict((("Version", SimTypeInt(signed=True, label="Int32")), ("paxosToValidate", SimTypeRef("PaxosTagCStruct", SimStruct)),)), name="WitnessTagHelper", pack=False, align=None),
    "WitnessTagUpdateHelper": SimStruct(OrderedDict((("Version", SimTypeInt(signed=True, label="Int32")), ("paxosToSet", SimTypeRef("PaxosTagCStruct", SimStruct)), ("paxosToValidate", SimTypeRef("PaxosTagCStruct", SimStruct)),)), name="WitnessTagUpdateHelper", pack=False, align=None),
    "WmfPlaceableFileHeader": SimStruct(OrderedDict((("Key", SimTypeInt(signed=False, label="UInt32")), ("Hmf", SimTypeShort(signed=True, label="Int16")), ("BoundingBox", SimTypeRef("PWMFRect16", SimStruct)), ("Inch", SimTypeShort(signed=True, label="Int16")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Checksum", SimTypeShort(signed=True, label="Int16")),)), name="WmfPlaceableFileHeader", pack=False, align=None),
    "WofEnumEntryProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["EntryInfo", "UserData"]), offset=0),
    "WofEnumFilesProc": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FilePath", "ExternalFileInfo", "UserData"]), offset=0),
    "WpdAttributeForm": SimTypeInt(signed=False, label="WpdAttributeForm"),
    "WpdParameterAttributeForm": SimTypeInt(signed=False, label="WpdParameterAttributeForm"),
    "WrapMode": SimTypeInt(signed=False, label="WrapMode"),
    "X500NameFlags": SimTypeInt(signed=False, label="X500NameFlags"),
    "X509Certificate": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("SerialNumber", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 4)), ("SignatureAlgorithm", SimTypeInt(signed=False, label="ALG_ID")), ("ValidFrom", SimTypeRef("FILETIME", SimStruct)), ("ValidUntil", SimTypeRef("FILETIME", SimStruct)), ("pszIssuer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pszSubject", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pPublicKey", SimTypePointer(SimTypeRef("PctPublicKey", SimStruct), offset=0)),)), name="X509Certificate", pack=False, align=None),
    "X509CertificateEnrollmentContext": SimTypeInt(signed=False, label="X509CertificateEnrollmentContext"),
    "X509CertificateTemplateEnrollmentFlag": SimTypeInt(signed=False, label="X509CertificateTemplateEnrollmentFlag"),
    "X509CertificateTemplateGeneralFlag": SimTypeInt(signed=False, label="X509CertificateTemplateGeneralFlag"),
    "X509CertificateTemplatePrivateKeyFlag": SimTypeInt(signed=False, label="X509CertificateTemplatePrivateKeyFlag"),
    "X509CertificateTemplateSubjectNameFlag": SimTypeInt(signed=False, label="X509CertificateTemplateSubjectNameFlag"),
    "X509EnrollmentAuthFlags": SimTypeInt(signed=False, label="X509EnrollmentAuthFlags"),
    "X509EnrollmentPolicyExportFlags": SimTypeInt(signed=False, label="X509EnrollmentPolicyExportFlags"),
    "X509EnrollmentPolicyLoadOption": SimTypeInt(signed=False, label="X509EnrollmentPolicyLoadOption"),
    "X509HardwareKeyUsageFlags": SimTypeInt(signed=False, label="X509HardwareKeyUsageFlags"),
    "X509KeyParametersExportType": SimTypeInt(signed=False, label="X509KeyParametersExportType"),
    "X509KeySpec": SimTypeInt(signed=False, label="X509KeySpec"),
    "X509KeyUsageFlags": SimTypeInt(signed=False, label="X509KeyUsageFlags"),
    "X509PrivateKeyExportFlags": SimTypeInt(signed=False, label="X509PrivateKeyExportFlags"),
    "X509PrivateKeyProtection": SimTypeInt(signed=False, label="X509PrivateKeyProtection"),
    "X509PrivateKeyUsageFlags": SimTypeInt(signed=False, label="X509PrivateKeyUsageFlags"),
    "X509PrivateKeyVerify": SimTypeInt(signed=False, label="X509PrivateKeyVerify"),
    "X509ProviderType": SimTypeInt(signed=False, label="X509ProviderType"),
    "X509RequestInheritOptions": SimTypeInt(signed=False, label="X509RequestInheritOptions"),
    "X509RequestType": SimTypeInt(signed=False, label="X509RequestType"),
    "X509SCEPDisposition": SimTypeInt(signed=False, label="X509SCEPDisposition"),
    "X509SCEPFailInfo": SimTypeInt(signed=False, label="X509SCEPFailInfo"),
    "X509SCEPMessageType": SimTypeInt(signed=False, label="X509SCEPMessageType"),
    "X509SCEPProcessMessageFlags": SimTypeInt(signed=False, label="X509SCEPProcessMessageFlags"),
    "XACTCONST": SimTypeInt(signed=False, label="XACTCONST"),
    "XACTHEURISTIC": SimTypeInt(signed=False, label="XACTHEURISTIC"),
    "XACTOPT": SimStruct(OrderedDict((("ulTimeout", SimTypeInt(signed=False, label="UInt32")), ("szDescription", SimTypeArray(SimTypeChar(label="Byte"), 40)),)), name="XACTOPT", pack=False, align=None),
    "XACTRM": SimTypeInt(signed=False, label="XACTRM"),
    "XACTSTAT": SimTypeInt(signed=False, label="XACTSTAT"),
    "XACTSTATS": SimStruct(OrderedDict((("cOpen", SimTypeInt(signed=False, label="UInt32")), ("cCommitting", SimTypeInt(signed=False, label="UInt32")), ("cCommitted", SimTypeInt(signed=False, label="UInt32")), ("cAborting", SimTypeInt(signed=False, label="UInt32")), ("cAborted", SimTypeInt(signed=False, label="UInt32")), ("cInDoubt", SimTypeInt(signed=False, label="UInt32")), ("cHeuristicDecision", SimTypeInt(signed=False, label="UInt32")), ("timeTransactionsUp", SimTypeRef("FILETIME", SimStruct)),)), name="XACTSTATS", pack=False, align=None),
    "XACTTC": SimTypeInt(signed=False, label="XACTTC"),
    "XACTTRANSINFO": SimStruct(OrderedDict((("uow", SimTypeRef("BOID", SimStruct)), ("isoLevel", SimTypeInt(signed=True, label="Int32")), ("isoFlags", SimTypeInt(signed=False, label="UInt32")), ("grfTCSupported", SimTypeInt(signed=False, label="UInt32")), ("grfRMSupported", SimTypeInt(signed=False, label="UInt32")), ("grfTCSupportedRetaining", SimTypeInt(signed=False, label="UInt32")), ("grfRMSupportedRetaining", SimTypeInt(signed=False, label="UInt32")),)), name="XACTTRANSINFO", pack=False, align=None),
    "XACT_DTC_CONSTANTS": SimTypeInt(signed=False, label="XACT_DTC_CONSTANTS"),
    "XACT_STATUS": SimTypeInt(signed=False, label="XACT_STATUS"),
    "XAPO_BUFFER_FLAGS": SimTypeInt(signed=False, label="XAPO_BUFFER_FLAGS"),
    "XAPO_LOCKFORPROCESS_PARAMETERS": SimStruct(OrderedDict((("pFormat", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("MaxFrameCount", SimTypeInt(signed=False, label="UInt32")),)), name="XAPO_LOCKFORPROCESS_PARAMETERS", pack=False, align=None),
    "XAPO_PROCESS_BUFFER_PARAMETERS": SimStruct(OrderedDict((("pBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferFlags", SimTypeInt(signed=False, label="XAPO_BUFFER_FLAGS")), ("ValidFrameCount", SimTypeInt(signed=False, label="UInt32")),)), name="XAPO_PROCESS_BUFFER_PARAMETERS", pack=False, align=None),
    "XAPO_REGISTRATION_PROPERTIES": SimStruct(OrderedDict((("clsid", SimTypeBottom(label="Guid")), ("FriendlyName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("CopyrightInfo", SimTypeArray(SimTypeChar(label="Char"), 256)), ("MajorVersion", SimTypeInt(signed=False, label="UInt32")), ("MinorVersion", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("MinInputBufferCount", SimTypeInt(signed=False, label="UInt32")), ("MaxInputBufferCount", SimTypeInt(signed=False, label="UInt32")), ("MinOutputBufferCount", SimTypeInt(signed=False, label="UInt32")), ("MaxOutputBufferCount", SimTypeInt(signed=False, label="UInt32")),)), name="XAPO_REGISTRATION_PROPERTIES", pack=False, align=None),
    "XAUDIO2FX_REVERB_I3DL2_PARAMETERS": SimStruct(OrderedDict((("WetDryMix", SimTypeFloat(size=32)), ("Room", SimTypeInt(signed=True, label="Int32")), ("RoomHF", SimTypeInt(signed=True, label="Int32")), ("RoomRolloffFactor", SimTypeFloat(size=32)), ("DecayTime", SimTypeFloat(size=32)), ("DecayHFRatio", SimTypeFloat(size=32)), ("Reflections", SimTypeInt(signed=True, label="Int32")), ("ReflectionsDelay", SimTypeFloat(size=32)), ("Reverb", SimTypeInt(signed=True, label="Int32")), ("ReverbDelay", SimTypeFloat(size=32)), ("Diffusion", SimTypeFloat(size=32)), ("Density", SimTypeFloat(size=32)), ("HFReference", SimTypeFloat(size=32)),)), name="XAUDIO2FX_REVERB_I3DL2_PARAMETERS", pack=False, align=None),
    "XAUDIO2FX_REVERB_PARAMETERS": SimStruct(OrderedDict((("WetDryMix", SimTypeFloat(size=32)), ("ReflectionsDelay", SimTypeInt(signed=False, label="UInt32")), ("ReverbDelay", SimTypeChar(label="Byte")), ("RearDelay", SimTypeChar(label="Byte")), ("SideDelay", SimTypeChar(label="Byte")), ("PositionLeft", SimTypeChar(label="Byte")), ("PositionRight", SimTypeChar(label="Byte")), ("PositionMatrixLeft", SimTypeChar(label="Byte")), ("PositionMatrixRight", SimTypeChar(label="Byte")), ("EarlyDiffusion", SimTypeChar(label="Byte")), ("LateDiffusion", SimTypeChar(label="Byte")), ("LowEQGain", SimTypeChar(label="Byte")), ("LowEQCutoff", SimTypeChar(label="Byte")), ("HighEQGain", SimTypeChar(label="Byte")), ("HighEQCutoff", SimTypeChar(label="Byte")), ("RoomFilterFreq", SimTypeFloat(size=32)), ("RoomFilterMain", SimTypeFloat(size=32)), ("RoomFilterHF", SimTypeFloat(size=32)), ("ReflectionsGain", SimTypeFloat(size=32)), ("ReverbGain", SimTypeFloat(size=32)), ("DecayTime", SimTypeFloat(size=32)), ("Density", SimTypeFloat(size=32)), ("RoomSize", SimTypeFloat(size=32)), ("DisableLateField", SimTypeInt(signed=True, label="Int32")),)), name="XAUDIO2FX_REVERB_PARAMETERS", pack=False, align=None),
    "XAUDIO2FX_VOLUMEMETER_LEVELS": SimStruct(OrderedDict((("pPeakLevels", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("pRMSLevels", SimTypePointer(SimTypeFloat(size=32), offset=0)), ("ChannelCount", SimTypeInt(signed=False, label="UInt32")),)), name="XAUDIO2FX_VOLUMEMETER_LEVELS", pack=False, align=None),
    "XAUDIO2_BUFFER": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("AudioBytes", SimTypeInt(signed=False, label="UInt32")), ("pAudioData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("PlayBegin", SimTypeInt(signed=False, label="UInt32")), ("PlayLength", SimTypeInt(signed=False, label="UInt32")), ("LoopBegin", SimTypeInt(signed=False, label="UInt32")), ("LoopLength", SimTypeInt(signed=False, label="UInt32")), ("LoopCount", SimTypeInt(signed=False, label="UInt32")), ("pContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="XAUDIO2_BUFFER", pack=False, align=None),
    "XAUDIO2_BUFFER_WMA": SimStruct(OrderedDict((("pDecodedPacketCumulativeBytes", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("PacketCount", SimTypeInt(signed=False, label="UInt32")),)), name="XAUDIO2_BUFFER_WMA", pack=False, align=None),
    "XAUDIO2_DEBUG_CONFIGURATION": SimStruct(OrderedDict((("TraceMask", SimTypeInt(signed=False, label="UInt32")), ("BreakMask", SimTypeInt(signed=False, label="UInt32")), ("LogThreadID", SimTypeInt(signed=True, label="Int32")), ("LogFileline", SimTypeInt(signed=True, label="Int32")), ("LogFunctionName", SimTypeInt(signed=True, label="Int32")), ("LogTiming", SimTypeInt(signed=True, label="Int32")),)), name="XAUDIO2_DEBUG_CONFIGURATION", pack=False, align=None),
    "XAUDIO2_EFFECT_CHAIN": SimStruct(OrderedDict((("EffectCount", SimTypeInt(signed=False, label="UInt32")), ("pEffectDescriptors", SimTypePointer(SimTypeRef("XAUDIO2_EFFECT_DESCRIPTOR", SimStruct), offset=0)),)), name="XAUDIO2_EFFECT_CHAIN", pack=False, align=None),
    "XAUDIO2_EFFECT_DESCRIPTOR": SimStruct(OrderedDict((("pEffect", SimTypeBottom(label="IUnknown")), ("InitialState", SimTypeInt(signed=True, label="Int32")), ("OutputChannels", SimTypeInt(signed=False, label="UInt32")),)), name="XAUDIO2_EFFECT_DESCRIPTOR", pack=False, align=None),
    "XAUDIO2_FILTER_PARAMETERS": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="XAUDIO2_FILTER_TYPE")), ("Frequency", SimTypeFloat(size=32)), ("OneOverQ", SimTypeFloat(size=32)),)), name="XAUDIO2_FILTER_PARAMETERS", pack=False, align=None),
    "XAUDIO2_FILTER_TYPE": SimTypeInt(signed=False, label="XAUDIO2_FILTER_TYPE"),
    "XAUDIO2_PERFORMANCE_DATA": SimStruct(OrderedDict((("AudioCyclesSinceLastQuery", SimTypeLongLong(signed=False, label="UInt64")), ("TotalCyclesSinceLastQuery", SimTypeLongLong(signed=False, label="UInt64")), ("MinimumCyclesPerQuantum", SimTypeInt(signed=False, label="UInt32")), ("MaximumCyclesPerQuantum", SimTypeInt(signed=False, label="UInt32")), ("MemoryUsageInBytes", SimTypeInt(signed=False, label="UInt32")), ("CurrentLatencyInSamples", SimTypeInt(signed=False, label="UInt32")), ("GlitchesSinceEngineStarted", SimTypeInt(signed=False, label="UInt32")), ("ActiveSourceVoiceCount", SimTypeInt(signed=False, label="UInt32")), ("TotalSourceVoiceCount", SimTypeInt(signed=False, label="UInt32")), ("ActiveSubmixVoiceCount", SimTypeInt(signed=False, label="UInt32")), ("ActiveResamplerCount", SimTypeInt(signed=False, label="UInt32")), ("ActiveMatrixMixCount", SimTypeInt(signed=False, label="UInt32")), ("ActiveXmaSourceVoices", SimTypeInt(signed=False, label="UInt32")), ("ActiveXmaStreams", SimTypeInt(signed=False, label="UInt32")),)), name="XAUDIO2_PERFORMANCE_DATA", pack=False, align=None),
    "XAUDIO2_SEND_DESCRIPTOR": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("pOutputVoice", SimTypeBottom(label="IXAudio2Voice")),)), name="XAUDIO2_SEND_DESCRIPTOR", pack=False, align=None),
    "XAUDIO2_VOICE_DETAILS": SimStruct(OrderedDict((("CreationFlags", SimTypeInt(signed=False, label="UInt32")), ("ActiveFlags", SimTypeInt(signed=False, label="UInt32")), ("InputChannels", SimTypeInt(signed=False, label="UInt32")), ("InputSampleRate", SimTypeInt(signed=False, label="UInt32")),)), name="XAUDIO2_VOICE_DETAILS", pack=False, align=None),
    "XAUDIO2_VOICE_SENDS": SimStruct(OrderedDict((("SendCount", SimTypeInt(signed=False, label="UInt32")), ("pSends", SimTypePointer(SimTypeRef("XAUDIO2_SEND_DESCRIPTOR", SimStruct), offset=0)),)), name="XAUDIO2_VOICE_SENDS", pack=False, align=None),
    "XAUDIO2_VOICE_STATE": SimStruct(OrderedDict((("pCurrentBufferContext", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BuffersQueued", SimTypeInt(signed=False, label="UInt32")), ("SamplesPlayed", SimTypeLongLong(signed=False, label="UInt64")),)), name="XAUDIO2_VOICE_STATE", pack=False, align=None),
    "XA_CLOSE_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_COMMIT_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_COMPLETE_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "XA_END_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_FORGET_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_OPEN_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_PREPARE_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_RECOVER_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0),
    "XA_ROLLBACK_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XA_START_EPT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("XID", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2"]), offset=0),
    "XBL_IDP_AUTH_TOKEN_STATUS": SimTypeInt(signed=False, label="XBL_IDP_AUTH_TOKEN_STATUS"),
    "XEKL_KEYSIZE": SimTypeInt(signed=False, label="XEKL_KEYSIZE"),
    "XEKL_KEYSPEC": SimTypeInt(signed=False, label="XEKL_KEYSPEC"),
    "XFORM": SimStruct(OrderedDict((("eM11", SimTypeFloat(size=32)), ("eM12", SimTypeFloat(size=32)), ("eM21", SimTypeFloat(size=32)), ("eM22", SimTypeFloat(size=32)), ("eDx", SimTypeFloat(size=32)), ("eDy", SimTypeFloat(size=32)),)), name="XFORM", pack=False, align=None),
    "XFORMCOORDS": SimTypeInt(signed=False, label="XFORMCOORDS"),
    "XFORML": SimStruct(OrderedDict((("eM11", SimTypeInt(signed=False, label="UInt32")), ("eM12", SimTypeInt(signed=False, label="UInt32")), ("eM21", SimTypeInt(signed=False, label="UInt32")), ("eM22", SimTypeInt(signed=False, label="UInt32")), ("eDx", SimTypeInt(signed=False, label="UInt32")), ("eDy", SimTypeInt(signed=False, label="UInt32")),)), name="XFORML", pack=False, align=None),
    "XFORMOBJ": SimStruct(OrderedDict((("ulReserved", SimTypeInt(signed=False, label="UInt32")),)), name="XFORMOBJ", pack=False, align=None),
    "XHR_AUTH": SimTypeInt(signed=False, label="XHR_AUTH"),
    "XHR_CERT": SimStruct(OrderedDict((("cbCert", SimTypeInt(signed=False, label="UInt32")), ("pbCert", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="XHR_CERT", pack=False, align=None),
    "XHR_CERT_ERROR_FLAG": SimTypeInt(signed=False, label="XHR_CERT_ERROR_FLAG"),
    "XHR_CERT_IGNORE_FLAG": SimTypeInt(signed=False, label="XHR_CERT_IGNORE_FLAG"),
    "XHR_COOKIE": SimStruct(OrderedDict((("pwszUrl", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszValue", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszP3PPolicy", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ftExpires", SimTypeRef("FILETIME", SimStruct)), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="XHR_COOKIE", pack=False, align=None),
    "XHR_COOKIE_FLAG": SimTypeInt(signed=False, label="XHR_COOKIE_FLAG"),
    "XHR_COOKIE_STATE": SimTypeInt(signed=False, label="XHR_COOKIE_STATE"),
    "XHR_CRED_PROMPT": SimTypeInt(signed=False, label="XHR_CRED_PROMPT"),
    "XHR_PROPERTY": SimTypeInt(signed=False, label="XHR_PROPERTY"),
    "XID": SimStruct(OrderedDict((("formatID", SimTypeInt(signed=True, label="Int32")), ("gtrid_length", SimTypeInt(signed=True, label="Int32")), ("bqual_length", SimTypeInt(signed=True, label="Int32")), ("data", SimTypeArray(SimTypeChar(label="SByte"), 128)),)), name="XID", pack=False, align=None),
    "XINPUT_BATTERY_INFORMATION": SimStruct(OrderedDict((("BatteryType", SimTypeInt(signed=False, label="BATTERY_TYPE")), ("BatteryLevel", SimTypeInt(signed=False, label="BATTERY_LEVEL")),)), name="XINPUT_BATTERY_INFORMATION", pack=False, align=None),
    "XINPUT_CAPABILITIES": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="XINPUT_DEVTYPE")), ("SubType", SimTypeInt(signed=False, label="XINPUT_DEVSUBTYPE")), ("Flags", SimTypeInt(signed=False, label="XINPUT_CAPABILITIES_FLAGS")), ("Gamepad", SimTypeRef("XINPUT_GAMEPAD", SimStruct)), ("Vibration", SimTypeRef("XINPUT_VIBRATION", SimStruct)),)), name="XINPUT_CAPABILITIES", pack=False, align=None),
    "XINPUT_CAPABILITIES_FLAGS": SimTypeInt(signed=False, label="XINPUT_CAPABILITIES_FLAGS"),
    "XINPUT_DEVSUBTYPE": SimTypeInt(signed=False, label="XINPUT_DEVSUBTYPE"),
    "XINPUT_DEVTYPE": SimTypeInt(signed=False, label="XINPUT_DEVTYPE"),
    "XINPUT_FLAG": SimTypeInt(signed=False, label="XINPUT_FLAG"),
    "XINPUT_GAMEPAD": SimStruct(OrderedDict((("wButtons", SimTypeInt(signed=False, label="XINPUT_GAMEPAD_BUTTON_FLAGS")), ("bLeftTrigger", SimTypeChar(label="Byte")), ("bRightTrigger", SimTypeChar(label="Byte")), ("sThumbLX", SimTypeShort(signed=True, label="Int16")), ("sThumbLY", SimTypeShort(signed=True, label="Int16")), ("sThumbRX", SimTypeShort(signed=True, label="Int16")), ("sThumbRY", SimTypeShort(signed=True, label="Int16")),)), name="XINPUT_GAMEPAD", pack=False, align=None),
    "XINPUT_GAMEPAD_BUTTON_FLAGS": SimTypeInt(signed=False, label="XINPUT_GAMEPAD_BUTTON_FLAGS"),
    "XINPUT_KEYSTROKE": SimStruct(OrderedDict((("VirtualKey", SimTypeInt(signed=False, label="XINPUT_VIRTUAL_KEY")), ("Unicode", SimTypeChar(label="Char")), ("Flags", SimTypeInt(signed=False, label="XINPUT_KEYSTROKE_FLAGS")), ("UserIndex", SimTypeChar(label="Byte")), ("HidCode", SimTypeChar(label="Byte")),)), name="XINPUT_KEYSTROKE", pack=False, align=None),
    "XINPUT_KEYSTROKE_FLAGS": SimTypeInt(signed=False, label="XINPUT_KEYSTROKE_FLAGS"),
    "XINPUT_STATE": SimStruct(OrderedDict((("dwPacketNumber", SimTypeInt(signed=False, label="UInt32")), ("Gamepad", SimTypeRef("XINPUT_GAMEPAD", SimStruct)),)), name="XINPUT_STATE", pack=False, align=None),
    "XINPUT_VIBRATION": SimStruct(OrderedDict((("wLeftMotorSpeed", SimTypeShort(signed=False, label="UInt16")), ("wRightMotorSpeed", SimTypeShort(signed=False, label="UInt16")),)), name="XINPUT_VIBRATION", pack=False, align=None),
    "XINPUT_VIRTUAL_KEY": SimTypeInt(signed=False, label="XINPUT_VIRTUAL_KEY"),
    "XLATEOBJ": SimStruct(OrderedDict((("iUniq", SimTypeInt(signed=False, label="UInt32")), ("flXlate", SimTypeInt(signed=False, label="UInt32")), ("iSrcType", SimTypeShort(signed=False, label="UInt16")), ("iDstType", SimTypeShort(signed=False, label="UInt16")), ("cEntries", SimTypeInt(signed=False, label="UInt32")), ("pulXlate", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="XLATEOBJ", pack=False, align=None),
    "XLAT_SIDE": SimTypeInt(signed=False, label="XLAT_SIDE"),
    "XMIT_HELPER_ROUTINE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0),
    "XMIT_ROUTINE_QUINTUPLE": SimStruct(OrderedDict((("pfnTranslateToXmit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("pfnTranslateFromXmit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("pfnFreeXmit", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)), ("pfnFreeInst", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("MIDL_STUB_MESSAGE", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["param0"]), offset=0)),)), name="XMIT_ROUTINE_QUINTUPLE", pack=False, align=None),
    "XMLDOMDocumentEvents": SimTypeBottom(label="XMLDOMDocumentEvents"),
    "XMLELEM_TYPE": SimTypeInt(signed=False, label="XMLELEM_TYPE"),
    "XML_DRIVER_NODE_INFO": SimStruct(OrderedDict((("FileName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("FileSize", SimTypeLongLong(signed=False, label="UInt64")), ("CreationDate", SimTypeLongLong(signed=False, label="UInt64")), ("Version", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("Manufacturer", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("ProductName", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Group", SimTypeArray(SimTypeChar(label="SByte"), 260)), ("Altitude", SimTypeArray(SimTypeChar(label="SByte"), 260)),)), name="XML_DRIVER_NODE_INFO", pack=False, align=None),
    "XML_ERROR": SimStruct(OrderedDict((("_nLine", SimTypeInt(signed=False, label="UInt32")), ("_pchBuf", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_cchBuf", SimTypeInt(signed=False, label="UInt32")), ("_ich", SimTypeInt(signed=False, label="UInt32")), ("_pszFound", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_pszExpected", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("_reserved1", SimTypeInt(signed=False, label="UInt32")), ("_reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="XML_ERROR", pack=False, align=None),
    "XPF_MCE_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "XPF_MC_BANK_FLAGS": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "XPF_RECOVERY_INFO": SimStruct(OrderedDict((("FailureReason", SimTypeRef("_FailureReason_e__Struct", SimStruct)), ("Action", SimTypeRef("_Action_e__Struct", SimStruct)), ("ActionRequired", SimTypeChar(label="Byte")), ("RecoverySucceeded", SimTypeChar(label="Byte")), ("RecoveryKernel", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeShort(signed=False, label="UInt16")), ("Reserved4", SimTypeInt(signed=False, label="UInt32")),)), name="XPF_RECOVERY_INFO", pack=False, align=None),
    "XPSRAS_BACKGROUND_COLOR": SimTypeInt(signed=False, label="XPSRAS_BACKGROUND_COLOR"),
    "XPSRAS_PIXEL_FORMAT": SimTypeInt(signed=False, label="XPSRAS_PIXEL_FORMAT"),
    "XPSRAS_RENDERING_MODE": SimTypeInt(signed=False, label="XPSRAS_RENDERING_MODE"),
    "XPS_COLOR": SimStruct(OrderedDict((("colorType", SimTypeInt(signed=False, label="XPS_COLOR_TYPE")), ("value", SimUnion({"sRGB": SimTypeRef("_sRGB_e__Struct", SimStruct), "scRGB": SimTypeRef("_scRGB_e__Struct", SimStruct), "context": SimTypeRef("_context_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="XPS_COLOR", pack=False, align=None),
    "XPS_COLOR_INTERPOLATION": SimTypeInt(signed=False, label="XPS_COLOR_INTERPOLATION"),
    "XPS_COLOR_TYPE": SimTypeInt(signed=False, label="XPS_COLOR_TYPE"),
    "XPS_COLOR_VALUE": SimUnion({"sRGB": SimTypeRef("_sRGB_e__Struct", SimStruct), "scRGB": SimTypeRef("_scRGB_e__Struct", SimStruct), "context": SimTypeRef("_context_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "XPS_DASH": SimStruct(OrderedDict((("length", SimTypeFloat(size=32)), ("gap", SimTypeFloat(size=32)),)), name="XPS_DASH", pack=False, align=None),
    "XPS_DASH_CAP": SimTypeInt(signed=False, label="XPS_DASH_CAP"),
    "XPS_DOCUMENT_TYPE": SimTypeInt(signed=False, label="XPS_DOCUMENT_TYPE"),
    "XPS_FILL_RULE": SimTypeInt(signed=False, label="XPS_FILL_RULE"),
    "XPS_FONT_EMBEDDING": SimTypeInt(signed=False, label="XPS_FONT_EMBEDDING"),
    "XPS_GLYPH_INDEX": SimStruct(OrderedDict((("index", SimTypeInt(signed=True, label="Int32")), ("advanceWidth", SimTypeFloat(size=32)), ("horizontalOffset", SimTypeFloat(size=32)), ("verticalOffset", SimTypeFloat(size=32)),)), name="XPS_GLYPH_INDEX", pack=False, align=None),
    "XPS_GLYPH_MAPPING": SimStruct(OrderedDict((("unicodeStringStart", SimTypeInt(signed=False, label="UInt32")), ("unicodeStringLength", SimTypeShort(signed=False, label="UInt16")), ("glyphIndicesStart", SimTypeInt(signed=False, label="UInt32")), ("glyphIndicesLength", SimTypeShort(signed=False, label="UInt16")),)), name="XPS_GLYPH_MAPPING", pack=False, align=None),
    "XPS_IMAGE_TYPE": SimTypeInt(signed=False, label="XPS_IMAGE_TYPE"),
    "XPS_INTERLEAVING": SimTypeInt(signed=False, label="XPS_INTERLEAVING"),
    "XPS_JOB_COMPLETION": SimTypeInt(signed=False, label="XPS_JOB_COMPLETION"),
    "XPS_JOB_STATUS": SimStruct(OrderedDict((("jobId", SimTypeInt(signed=False, label="UInt32")), ("currentDocument", SimTypeInt(signed=True, label="Int32")), ("currentPage", SimTypeInt(signed=True, label="Int32")), ("currentPageTotal", SimTypeInt(signed=True, label="Int32")), ("completion", SimTypeInt(signed=False, label="XPS_JOB_COMPLETION")), ("jobStatus", SimTypeInt(signed=True, label="Int32")),)), name="XPS_JOB_STATUS", pack=False, align=None),
    "XPS_LINE_CAP": SimTypeInt(signed=False, label="XPS_LINE_CAP"),
    "XPS_LINE_JOIN": SimTypeInt(signed=False, label="XPS_LINE_JOIN"),
    "XPS_MATRIX": SimStruct(OrderedDict((("m11", SimTypeFloat(size=32)), ("m12", SimTypeFloat(size=32)), ("m21", SimTypeFloat(size=32)), ("m22", SimTypeFloat(size=32)), ("m31", SimTypeFloat(size=32)), ("m32", SimTypeFloat(size=32)),)), name="XPS_MATRIX", pack=False, align=None),
    "XPS_OBJECT_TYPE": SimTypeInt(signed=False, label="XPS_OBJECT_TYPE"),
    "XPS_POINT": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)),)), name="XPS_POINT", pack=False, align=None),
    "XPS_RECT": SimStruct(OrderedDict((("x", SimTypeFloat(size=32)), ("y", SimTypeFloat(size=32)), ("width", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)),)), name="XPS_RECT", pack=False, align=None),
    "XPS_SEGMENT_STROKE_PATTERN": SimTypeInt(signed=False, label="XPS_SEGMENT_STROKE_PATTERN"),
    "XPS_SEGMENT_TYPE": SimTypeInt(signed=False, label="XPS_SEGMENT_TYPE"),
    "XPS_SIGNATURE_STATUS": SimTypeInt(signed=False, label="XPS_SIGNATURE_STATUS"),
    "XPS_SIGN_FLAGS": SimTypeInt(signed=False, label="XPS_SIGN_FLAGS"),
    "XPS_SIGN_POLICY": SimTypeInt(signed=False, label="XPS_SIGN_POLICY"),
    "XPS_SIZE": SimStruct(OrderedDict((("width", SimTypeFloat(size=32)), ("height", SimTypeFloat(size=32)),)), name="XPS_SIZE", pack=False, align=None),
    "XPS_SPREAD_METHOD": SimTypeInt(signed=False, label="XPS_SPREAD_METHOD"),
    "XPS_STYLE_SIMULATION": SimTypeInt(signed=False, label="XPS_STYLE_SIMULATION"),
    "XPS_THUMBNAIL_SIZE": SimTypeInt(signed=False, label="XPS_THUMBNAIL_SIZE"),
    "XPS_TILE_MODE": SimTypeInt(signed=False, label="XPS_TILE_MODE"),
    "XSAVE_AREA": SimStruct(OrderedDict((("LegacyState", SimTypeRef("XSAVE_FORMAT", SimStruct)), ("Header", SimTypeRef("XSAVE_AREA_HEADER", SimStruct)),)), name="XSAVE_AREA", pack=False, align=None),
    "XSAVE_AREA_HEADER": SimStruct(OrderedDict((("Mask", SimTypeLongLong(signed=False, label="UInt64")), ("CompactionMask", SimTypeLongLong(signed=False, label="UInt64")), ("Reserved2", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 6)),)), name="XSAVE_AREA_HEADER", pack=False, align=None),
    "XSAVE_CET_U_FORMAT": SimStruct(OrderedDict((("Ia32CetUMsr", SimTypeLongLong(signed=False, label="UInt64")), ("Ia32Pl3SspMsr", SimTypeLongLong(signed=False, label="UInt64")),)), name="XSAVE_CET_U_FORMAT", pack=False, align=None),
    "XSAVE_FORMAT": SimStruct(OrderedDict((("ControlWord", SimTypeShort(signed=False, label="UInt16")), ("StatusWord", SimTypeShort(signed=False, label="UInt16")), ("TagWord", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("ErrorOpcode", SimTypeShort(signed=False, label="UInt16")), ("ErrorOffset", SimTypeInt(signed=False, label="UInt32")), ("ErrorSelector", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("DataOffset", SimTypeInt(signed=False, label="UInt32")), ("DataSelector", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeShort(signed=False, label="UInt16")), ("MxCsr", SimTypeInt(signed=False, label="UInt32")), ("MxCsr_Mask", SimTypeInt(signed=False, label="UInt32")), ("FloatRegisters", SimTypeArray(SimTypeRef("M128A", SimStruct), 8)), ("XmmRegisters", SimTypeArray(SimTypeRef("M128A", SimStruct), 8)), ("Reserved4", SimTypeArray(SimTypeChar(label="Byte"), 224)),)), name="XSAVE_FORMAT", pack=False, align=None),
    "XSTATE_CONFIGURATION": SimStruct(OrderedDict((("EnabledFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("EnabledVolatileFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("Size", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"ControlFlags": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Features", SimTypeArray(SimTypeRef("XSTATE_FEATURE", SimStruct), 64)), ("EnabledSupervisorFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("AlignedFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("AllFeatureSize", SimTypeInt(signed=False, label="UInt32")), ("AllFeatures", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 64)), ("EnabledUserVisibleSupervisorFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("ExtendedFeatureDisableFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("AllNonLargeFeatureSize", SimTypeInt(signed=False, label="UInt32")), ("Spare", SimTypeInt(signed=False, label="UInt32")),)), name="XSTATE_CONFIGURATION", pack=False, align=None),
    "XSTATE_CONFIG_FEATURE_MSC_INFO": SimStruct(OrderedDict((("SizeOfInfo", SimTypeInt(signed=False, label="UInt32")), ("ContextSize", SimTypeInt(signed=False, label="UInt32")), ("EnabledFeatures", SimTypeLongLong(signed=False, label="UInt64")), ("Features", SimTypeArray(SimTypeRef("XSTATE_FEATURE", SimStruct), 64)),)), name="XSTATE_CONFIG_FEATURE_MSC_INFO", pack=False, align=None),
    "XSTATE_CONTEXT": SimStruct(OrderedDict((("Mask", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Area", SimTypePointer(SimTypeRef("XSAVE_AREA", SimStruct), offset=0)), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved3", SimTypeInt(signed=False, label="UInt32")),)), name="XSTATE_CONTEXT", pack=False, align=None),
    "XSTATE_FEATURE": SimStruct(OrderedDict((("Offset", SimTypeInt(signed=False, label="UInt32")), ("Size", SimTypeInt(signed=False, label="UInt32")),)), name="XSTATE_FEATURE", pack=False, align=None),
    "XSTATE_SAVE": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("Reserved1", SimTypeLongLong(signed=True, label="Int64")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Prev", SimTypePointer(SimTypeRef("XSTATE_SAVE", SimStruct), offset=0)), ("Reserved3", SimTypePointer(SimTypeRef("XSAVE_AREA", SimStruct), offset=0)), ("Thread", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)), ("Reserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Level", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XStateContext": SimTypeRef("XSTATE_CONTEXT", SimStruct)}, name="<anon>", label="None")),)), name="XSTATE_SAVE", pack=False, align=None),
    "XYZCOLOR": SimStruct(OrderedDict((("X", SimTypeShort(signed=False, label="UInt16")), ("Y", SimTypeShort(signed=False, label="UInt16")), ("Z", SimTypeShort(signed=False, label="UInt16")),)), name="XYZCOLOR", pack=False, align=None),
    "XYZColorF": SimStruct(OrderedDict((("X", SimTypeFloat(size=32)), ("Y", SimTypeFloat(size=32)), ("Z", SimTypeFloat(size=32)),)), name="XYZColorF", pack=False, align=None),
    "XmlConformanceLevel": SimTypeInt(signed=False, label="XmlConformanceLevel"),
    "XmlError": SimTypeInt(signed=False, label="XmlError"),
    "XmlNodeType": SimTypeInt(signed=False, label="XmlNodeType"),
    "XmlReadState": SimTypeInt(signed=False, label="XmlReadState"),
    "XmlReaderProperty": SimTypeInt(signed=False, label="XmlReaderProperty"),
    "XmlStandalone": SimTypeInt(signed=False, label="XmlStandalone"),
    "XmlWriterProperty": SimTypeInt(signed=False, label="XmlWriterProperty"),
    "YAMAHA_ADPCMWAVEFORMAT": SimStruct(OrderedDict((("wfx", SimTypeRef("WAVEFORMATEX", SimStruct)),)), name="YAMAHA_ADPCMWAVEFORMAT", pack=False, align=None),
    "YIELDPROC": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["mciId", "dwYieldData"]), offset=0),
    "YxyCOLOR": SimStruct(OrderedDict((("Y", SimTypeShort(signed=False, label="UInt16")), ("x", SimTypeShort(signed=False, label="UInt16")), ("y", SimTypeShort(signed=False, label="UInt16")),)), name="YxyCOLOR", pack=False, align=None),
    "ZAFLAGS": SimTypeInt(signed=False, label="ZAFLAGS"),
    "ZONEATTRIBUTES": SimStruct(OrderedDict((("cbSize", SimTypeInt(signed=False, label="UInt32")), ("szDisplayName", SimTypeArray(SimTypeChar(label="Char"), 260)), ("szDescription", SimTypeArray(SimTypeChar(label="Char"), 200)), ("szIconPath", SimTypeArray(SimTypeChar(label="Char"), 260)), ("dwTemplateMinLevel", SimTypeInt(signed=False, label="UInt32")), ("dwTemplateRecommended", SimTypeInt(signed=False, label="UInt32")), ("dwTemplateCurrentLevel", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")),)), name="ZONEATTRIBUTES", pack=False, align=None),
    "ZONE_HEADER": SimStruct(OrderedDict((("FreeList", SimTypeRef("SINGLE_LIST_ENTRY", SimStruct)), ("SegmentList", SimTypeRef("SINGLE_LIST_ENTRY", SimStruct)), ("BlockSize", SimTypeInt(signed=False, label="UInt32")), ("TotalSegmentSize", SimTypeInt(signed=False, label="UInt32")),)), name="ZONE_HEADER", pack=False, align=None),
    "ZONE_SEGMENT_HEADER": SimStruct(OrderedDict((("SegmentList", SimTypeRef("SINGLE_LIST_ENTRY", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="ZONE_SEGMENT_HEADER", pack=False, align=None),
    "ZONE_STATE": SimTypeInt(signed=False, label="ZONE_STATE"),
    "ZoomUnit": SimTypeInt(signed=False, label="ZoomUnit"),
    "_AALSpecificParameters_e__Union": SimUnion({"AAL5Parameters": SimTypeRef("AAL5_PARAMETERS", SimStruct), "AALUserParameters": SimTypeRef("AALUSER_PARAMETERS", SimStruct)}, name="<anon>", label="None"),
    "_ABORT_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_ABORT_e__Struct", pack=False, align=None),
    "_ADAPTER_OBJECT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "_AFI_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_AFI_e__Struct", pack=False, align=None),
    "_AMRESCTL_RESERVEFLAGS": SimTypeInt(signed=False, label="_AMRESCTL_RESERVEFLAGS"),
    "_AMSTREAMSELECTENABLEFLAGS": SimTypeInt(signed=False, label="_AMSTREAMSELECTENABLEFLAGS"),
    "_AMSTREAMSELECTINFOFLAGS": SimTypeInt(signed=False, label="_AMSTREAMSELECTINFOFLAGS"),
    "_AM_ASFWRITERCONFIG_PARAM": SimTypeInt(signed=False, label="_AM_ASFWRITERCONFIG_PARAM"),
    "_AM_AUDIO_RENDERER_STAT_PARAM": SimTypeInt(signed=False, label="_AM_AUDIO_RENDERER_STAT_PARAM"),
    "_AM_FILTER_MISC_FLAGS": SimTypeInt(signed=False, label="_AM_FILTER_MISC_FLAGS"),
    "_AM_INTF_SEARCH_FLAGS": SimTypeInt(signed=False, label="_AM_INTF_SEARCH_FLAGS"),
    "_AM_OVERLAY_NOTIFY_FLAGS": SimTypeInt(signed=False, label="_AM_OVERLAY_NOTIFY_FLAGS"),
    "_AM_PIN_FLOW_CONTROL_BLOCK_FLAGS": SimTypeInt(signed=False, label="_AM_PIN_FLOW_CONTROL_BLOCK_FLAGS"),
    "_AM_PUSHSOURCE_FLAGS": SimTypeInt(signed=False, label="_AM_PUSHSOURCE_FLAGS"),
    "_AM_RENSDEREXFLAGS": SimTypeInt(signed=False, label="_AM_RENSDEREXFLAGS"),
    "_ANACAP_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_ANACAP_e__Struct", pack=False, align=None),
    "_APC_e__Struct": SimStruct(OrderedDict((("NotificationRoutine", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("RPC_ASYNC_STATE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="RPC_ASYNC_EVENT")], SimTypeBottom(label="Void"), arg_names=["pAsync", "Context", "Event"]), offset=0)), ("hThread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_APC_e__Struct", pack=False, align=None),
    "_APSTA_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_APSTA_e__Struct", pack=False, align=None),
    "_ATSC_e__Struct": SimStruct(OrderedDict((("lProgNumber", SimTypeInt(signed=True, label="Int32")),)), name="_ATSC_e__Struct", pack=False, align=None),
    "_AUDCLNT_BUFFERFLAGS": SimTypeInt(signed=False, label="_AUDCLNT_BUFFERFLAGS"),
    "_AVSCC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_AVSCC_e__Struct", pack=False, align=None),
    "_AccountSid_e__Struct": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeArray(SimTypeChar(label="Byte"), 68)),)), name="_AccountSid_e__Struct", pack=False, align=None),
    "_AckData_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_ACK_DATA_FLAGS")), ("CompletionStatus", SimTypeInt(signed=True, label="Int32")), ("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="_AckData_e__Struct", pack=False, align=None),
    "_AckDehydrate_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_ACK_DEHYDRATE_FLAGS")), ("CompletionStatus", SimTypeInt(signed=True, label="Int32")), ("FileIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")),)), name="_AckDehydrate_e__Struct", pack=False, align=None),
    "_AckDelete_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_ACK_DELETE_FLAGS")), ("CompletionStatus", SimTypeInt(signed=True, label="Int32")),)), name="_AckDelete_e__Struct", pack=False, align=None),
    "_AckRename_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_ACK_RENAME_FLAGS")), ("CompletionStatus", SimTypeInt(signed=True, label="Int32")),)), name="_AckRename_e__Struct", pack=False, align=None),
    "_Acpi_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Acpi_e__Struct", pack=False, align=None),
    "_AcquireForModifiedPageWriter_e__Struct": SimStruct(OrderedDict((("EndingOffset", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("ResourceToRelease", SimTypePointer(SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0), offset=0)),)), name="_AcquireForModifiedPageWriter_e__Struct", pack=False, align=None),
    "_AcquireForSectionSynchronization_e__Struct": SimStruct(OrderedDict((("SyncType", SimTypeInt(signed=False, label="FS_FILTER_SECTION_SYNC_TYPE")), ("PageProtection", SimTypeInt(signed=False, label="UInt32")), ("OutputInformation", SimTypePointer(SimTypeRef("FS_FILTER_SECTION_SYNC_OUTPUT", SimStruct), offset=0)), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("AllocationAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="_AcquireForSectionSynchronization_e__Struct", pack=False, align=None),
    "_Action_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Action_e__Struct", pack=False, align=None),
    "_AdditionalSignalInfo_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_AdditionalSignalInfo_e__Struct", pack=False, align=None),
    "_AgentSpecific_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("dwAgentExtensionIDIndex", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("Params", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_AgentSpecific_e__Struct", pack=False, align=None),
    "_Alignment_e__Struct": SimStruct(OrderedDict((("Unused", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Alignment", SimTypeArray(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), 2)),)), name="_Alignment_e__Struct", pack=False, align=None),
    "_Allocate_e__Struct": SimStruct(OrderedDict((("Calls", SimTypeInt(signed=False, label="UInt32")), ("RunsReturned", SimTypeInt(signed=False, label="UInt32")), ("Hints", SimTypeInt(signed=False, label="UInt32")), ("HintsHonored", SimTypeInt(signed=False, label="UInt32")), ("Cache", SimTypeInt(signed=False, label="UInt32")), ("CacheMiss", SimTypeInt(signed=False, label="UInt32")), ("Clusters", SimTypeLongLong(signed=False, label="UInt64")), ("HintsClusters", SimTypeLongLong(signed=False, label="UInt64")), ("CacheClusters", SimTypeLongLong(signed=False, label="UInt64")), ("CacheMissClusters", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Allocate_e__Struct", pack=False, align=None),
    "_Anonymous1_e__Struct": SimStruct(OrderedDict((("PdlAutoNameInterval", SimTypeInt(signed=False, label="UInt32")), ("PdlAutoNameUnits", SimTypeInt(signed=False, label="UInt32")), ("PdlCommandFilename", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PdlCounterList", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PdlAutoNameFormat", SimTypeInt(signed=False, label="UInt32")), ("PdlSampleInterval", SimTypeInt(signed=False, label="UInt32")), ("PdlLogStartTime", SimTypeRef("FILETIME", SimStruct)), ("PdlLogEndTime", SimTypeRef("FILETIME", SimStruct)),)), name="_Anonymous1_e__Struct", pack=False, align=None),
    "_Anonymous1_e__Union": SimUnion({"dwControl": SimTypeInt(signed=False, label="UInt32"), "dwNotUsed": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_Anonymous2_e__Struct": SimStruct(OrderedDict((("TlNumberOfBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlMinimumBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlMaximumBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlFreeBuffers", SimTypeInt(signed=False, label="UInt32")), ("TlBufferSize", SimTypeInt(signed=False, label="UInt32")), ("TlEventsLost", SimTypeInt(signed=False, label="UInt32")), ("TlLoggerThreadId", SimTypeInt(signed=False, label="UInt32")), ("TlBuffersWritten", SimTypeInt(signed=False, label="UInt32")), ("TlLogHandle", SimTypeInt(signed=False, label="UInt32")), ("TlLogFileName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Anonymous2_e__Struct", pack=False, align=None),
    "_Anonymous2_e__Union": SimUnion({"pwszDscr": SimTypeArray(SimTypeChar(label="Char"), 31), "pwszNoUse": SimTypeArray(SimTypeChar(label="Char"), 31)}, name="<anon>", label="None"),
    "_Anonymous3_e__Struct": SimStruct(OrderedDict((("VmRegion", SimTypeRef("MINIDUMP_MEMORY_INFO", SimStruct)), ("Continue", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous3_e__Struct", pack=False, align=None),
    "_Anonymous3_e__Union": SimUnion({"pchReadIdxWDD": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pchCompIdxWDD": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="<anon>", label="None"),
    "_Anonymous4_e__Struct": SimStruct(OrderedDict((("VmQueryStatus", SimTypeInt(signed=True, label="Int32")), ("VmQueryResult", SimTypeRef("MINIDUMP_MEMORY_INFO", SimStruct)),)), name="_Anonymous4_e__Struct", pack=False, align=None),
    "_Anonymous4_e__Union": SimUnion({"hbmWatermark": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmWatermark": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_Anonymous5_e__Struct": SimStruct(OrderedDict((("VmReadStatus", SimTypeInt(signed=True, label="Int32")), ("VmReadBytesCompleted", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous5_e__Struct", pack=False, align=None),
    "_Anonymous5_e__Union": SimUnion({"hbmHeader": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszbmHeader": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_Anonymous6_e__Struct": SimStruct(OrderedDict((("ThreadType", SimTypeChar(label="Byte")), ("ThreadReserved", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"ThreadControlFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"DebugActive": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="_Anonymous6_e__Struct", pack=False, align=None),
    "_Anonymous6_e__Union": SimUnion({"nz": SimTypeFloat(size=32), "dvNZ": SimTypeFloat(size=32)}, name="<anon>", label="None"),
    "_Anonymous7_e__Struct": SimStruct(OrderedDict((("MutantType", SimTypeChar(label="Byte")), ("MutantSize", SimTypeChar(label="Byte")), ("DpcActive", SimTypeChar(label="Byte")), ("MutantReserved", SimTypeChar(label="Byte")),)), name="_Anonymous7_e__Struct", pack=False, align=None),
    "_Anonymous7_e__Union": SimUnion({"tu": SimTypeFloat(size=32), "dvTU": SimTypeFloat(size=32)}, name="<anon>", label="None"),
    "_Anonymous8_e__Union": SimUnion({"tv": SimTypeFloat(size=32), "dvTV": SimTypeFloat(size=32)}, name="<anon>", label="None"),
    "_Anonymous_e__Struct": SimStruct(OrderedDict((("_11", SimTypeFloat(size=32)), ("_12", SimTypeFloat(size=32)), ("_13", SimTypeFloat(size=32)), ("_14", SimTypeFloat(size=32)), ("_21", SimTypeFloat(size=32)), ("_22", SimTypeFloat(size=32)), ("_23", SimTypeFloat(size=32)), ("_24", SimTypeFloat(size=32)), ("_31", SimTypeFloat(size=32)), ("_32", SimTypeFloat(size=32)), ("_33", SimTypeFloat(size=32)), ("_34", SimTypeFloat(size=32)), ("_41", SimTypeFloat(size=32)), ("_42", SimTypeFloat(size=32)), ("_43", SimTypeFloat(size=32)), ("_44", SimTypeFloat(size=32)),)), name="_Anonymous_e__Struct", pack=False, align=None),
    "_Anonymous_e__Union": SimUnion({"Menu": SimTypeRef("_Menu_e__Struct", SimStruct), "MenuEx": SimTypeRef("_MenuEx_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Apc_e__Struct": SimStruct(OrderedDict((("lpOverlapped", SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)), ("lpfnCompletionProc", SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)),)), name="_Apc_e__Struct", pack=False, align=None),
    "_AppEvents": SimTypeBottom(label="_AppEvents"),
    "_AppName_e__Struct": SimStruct(OrderedDict((("Name", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("GPOId", SimTypeBottom(label="Guid")),)), name="_AppName_e__Struct", pack=False, align=None),
    "_Application": SimTypeBottom(label="_Application"),
    "_Array_e__Struct": SimStruct(OrderedDict((("Dimension", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 4)),)), name="_Array_e__Struct", pack=False, align=None),
    "_AssociatedIrp_e__Union": SimUnion({"MasterIrp": SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0), "IrpCount": SimTypeInt(signed=True, label="Int32"), "SystemBuffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"),
    "_AsynchronousParameters_e__Struct": SimStruct(OrderedDict((("Anonymous1", SimUnion({"UserApcRoutine": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("IO_STATUS_BLOCK", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ApcContext", "IoStatusBlock", "Reserved"]), offset=0), "IssuingProcess": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"UserApcContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "IoRing": SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)}, name="<anon>", label="None")),)), name="_AsynchronousParameters_e__Struct", pack=False, align=None),
    "_AtaPort_e__Struct": SimStruct(OrderedDict((("dwAtaPortSpecific", SimTypeInt(signed=False, label="UInt32")),)), name="_AtaPort_e__Struct", pack=False, align=None),
    "_AttributeType_e__Union": SimUnion({"Admin": SimTypeRef("GOPHER_ADMIN_ATTRIBUTE_TYPE", SimStruct), "ModDate": SimTypeRef("GOPHER_MOD_DATE_ATTRIBUTE_TYPE", SimStruct), "Ttl": SimTypeRef("GOPHER_TTL_ATTRIBUTE_TYPE", SimStruct), "Score": SimTypeRef("GOPHER_SCORE_ATTRIBUTE_TYPE", SimStruct), "ScoreRange": SimTypeRef("GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE", SimStruct), "Site": SimTypeRef("GOPHER_SITE_ATTRIBUTE_TYPE", SimStruct), "Organization": SimTypeRef("GOPHER_ORGANIZATION_ATTRIBUTE_TYPE", SimStruct), "Location": SimTypeRef("GOPHER_LOCATION_ATTRIBUTE_TYPE", SimStruct), "GeographicalLocation": SimTypeRef("GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE", SimStruct), "TimeZone": SimTypeRef("GOPHER_TIMEZONE_ATTRIBUTE_TYPE", SimStruct), "Provider": SimTypeRef("GOPHER_PROVIDER_ATTRIBUTE_TYPE", SimStruct), "Version": SimTypeRef("GOPHER_VERSION_ATTRIBUTE_TYPE", SimStruct), "Abstract": SimTypeRef("GOPHER_ABSTRACT_ATTRIBUTE_TYPE", SimStruct), "View": SimTypeRef("GOPHER_VIEW_ATTRIBUTE_TYPE", SimStruct), "Veronica": SimTypeRef("GOPHER_VERONICA_ATTRIBUTE_TYPE", SimStruct), "Ask": SimTypeRef("GOPHER_ASK_ATTRIBUTE_TYPE", SimStruct), "Unknown": SimTypeRef("GOPHER_UNKNOWN_ATTRIBUTE_TYPE", SimStruct)}, name="<anon>", label="None"),
    "_Attribute_e__Union": SimUnion({"pAttributeV1": SimTypePointer(SimTypeRef("CLAIM_SECURITY_ATTRIBUTE_V1", SimStruct), offset=0)}, name="<anon>", label="None"),
    "_Attributes_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Attributes_e__Struct", pack=False, align=None),
    "_Attributes_e__Union": SimUnion({"AllAttributes": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "_Authorization_e__Struct": SimStruct(OrderedDict((("Size", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="_Authorization_e__Struct", pack=False, align=None),
    "_AutoGrow_e__Struct": SimStruct(OrderedDict((("Enabled", SimTypeInt(signed=False, label="UInt32")),)), name="_AutoGrow_e__Struct", pack=False, align=None),
    "_AutoShrink_e__Struct": SimStruct(OrderedDict((("Percentage", SimTypeInt(signed=False, label="UInt32")),)), name="_AutoShrink_e__Struct", pack=False, align=None),
    "_BLOBType": SimStruct(OrderedDict((("dbobj", SimTypeRef("DBOBJECT", SimStruct)), ("pUnk", SimTypeBottom(label="IUnknown")),)), name="_BLOBType", pack=False, align=None),
    "_BM": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_BM", pack=False, align=None),
    "_BROWSERFRAMEOPTIONS": SimTypeInt(signed=False, label="_BROWSERFRAMEOPTIONS"),
    "_BadSystemNANDBlockCount_e__Struct": SimStruct(OrderedDict((("RawCount", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Normalized", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="_BadSystemNANDBlockCount_e__Struct", pack=False, align=None),
    "_BadUserNANDBlockCount_e__Struct": SimStruct(OrderedDict((("RawCount", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Normalized", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="_BadUserNANDBlockCount_e__Struct", pack=False, align=None),
    "_BasicInformation_e__Struct": SimStruct(OrderedDict((("CreationTime", SimTypeLongLong(signed=True, label="Int64")), ("LastAccessTime", SimTypeLongLong(signed=True, label="Int64")), ("LastWriteTime", SimTypeLongLong(signed=True, label="Int64")), ("ChangeTime", SimTypeLongLong(signed=True, label="Int64")), ("FileAttributes", SimTypeInt(signed=False, label="UInt32")),)), name="_BasicInformation_e__Struct", pack=False, align=None),
    "_BinaryVal": SimStruct(OrderedDict((("sActualLength", SimTypeShort(signed=True, label="Int16")), ("sMaxLength", SimTypeShort(signed=True, label="Int16")), ("prgbBinaryVal", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="_BinaryVal", pack=False, align=None),
    "_BitField": SimStruct(OrderedDict((("Position", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")),)), name="_BitField", pack=False, align=None),
    "_BitmapWritesUserLevel_e__Struct": SimStruct(OrderedDict((("Write", SimTypeInt(signed=False, label="UInt32")), ("Create", SimTypeInt(signed=False, label="UInt32")), ("SetInfo", SimTypeInt(signed=False, label="UInt32")), ("Flush", SimTypeInt(signed=False, label="UInt32")),)), name="_BitmapWritesUserLevel_e__Struct", pack=False, align=None),
    "_Bits_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Bits_e__Struct", pack=False, align=None),
    "_BlockPrefetch_e__Struct": SimStruct(OrderedDict((("Minimum", SimTypeShort(signed=False, label="UInt16")), ("Maximum", SimTypeShort(signed=False, label="UInt16")),)), name="_BlockPrefetch_e__Struct", pack=False, align=None),
    "_Block_e__Struct": SimStruct(OrderedDict((("hMem", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwReserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="_Block_e__Struct", pack=False, align=None),
    "_Bounds_e__Union": SimUnion({"Anonymous1": SimStruct(OrderedDict((("lMinimum", SimTypeInt(signed=True, label="Int32")), ("lMaximum", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("dwMinimum", SimTypeInt(signed=False, label="UInt32")), ("dwMaximum", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "dwReserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)}, name="<anon>", label="None"),
    "_BreakpointInformation_e__Struct": SimStruct(OrderedDict((("BreakpointId", SimTypeLongLong(signed=False, label="UInt64")),)), name="_BreakpointInformation_e__Struct", pack=False, align=None),
    "_BridgeStatus_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_BuddyAllocatorConfig_e__Struct": SimStruct(OrderedDict((("AddressWidth", SimTypeInt(signed=False, label="UInt32")),)), name="_BuddyAllocatorConfig_e__Struct", pack=False, align=None),
    "_Buffered_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("FsControlCode", SimTypeInt(signed=False, label="UInt32")), ("SystemBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Buffered_e__Struct", pack=False, align=None),
    "_Bulk_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Bulk_e__Struct", pack=False, align=None),
    "_BusNumber_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("MinBusNumber", SimTypeInt(signed=False, label="UInt32")), ("MaxBusNumber", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_BusNumber_e__Struct", pack=False, align=None),
    "_BusSpecificData_e__Union": SimUnion({"ScsiInformation": SimTypeRef("_ScsiInformation_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_ByName_e__Struct": SimStruct(OrderedDict((("pPackageName", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("PolicyId", SimTypeBottom(label="Guid")),)), name="_ByName_e__Struct", pack=False, align=None),
    "_ByObjectId_e__Struct": SimStruct(OrderedDict((("ObjectId", SimTypeBottom(label="Guid")), ("PolicyId", SimTypeBottom(label="Guid")),)), name="_ByObjectId_e__Struct", pack=False, align=None),
    "_Bytes_e__Struct": SimStruct(OrderedDict((("BaseMid", SimTypeChar(label="Byte")), ("Flags1", SimTypeChar(label="Byte")), ("Flags2", SimTypeChar(label="Byte")), ("BaseHi", SimTypeChar(label="Byte")),)), name="_Bytes_e__Struct", pack=False, align=None),
    "_CDBE_ACTIONS": SimTypeInt(signed=False, label="_CDBE_ACTIONS"),
    "_CMIC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_CMIC_e__Struct", pack=False, align=None),
    "_COMClass_e__Struct": SimStruct(OrderedDict((("Clsid", SimTypeBottom(label="Guid")), ("ClsCtx", SimTypeInt(signed=False, label="UInt32")),)), name="_COMClass_e__Struct", pack=False, align=None),
    "_CPSUICALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("CPSUICBPARAM", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCPSUICBParam"]), offset=0),
    "_CPUNotification_e__Struct": SimStruct(OrderedDict((("Event", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_CPUNotification_e__Struct", pack=False, align=None),
    "_CQES_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_CQES_e__Struct", pack=False, align=None),
    "_CRC_e__Struct": SimStruct(OrderedDict((("crc", SimTypeInt(signed=False, label="UInt32")), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 16)),)), name="_CRC_e__Struct", pack=False, align=None),
    "_CREATEIOCQ_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_CREATEIOCQ_e__Struct", pack=False, align=None),
    "_CREATEIOSQ_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_CREATEIOSQ_e__Struct", pack=False, align=None),
    "_CTRATT_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_CTRATT_e__Struct", pack=False, align=None),
    "_CacheFlags_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_CacheFlags_e__Struct", pack=False, align=None),
    "_Camera_e__Struct": SimStruct(OrderedDict((("Info", SimTypeRef("KSCAMERA_PROFILE_INFO", SimStruct)), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("ConcurrencyCount", SimTypeInt(signed=False, label="UInt32")), ("Concurrency", SimTypePointer(SimTypeRef("KSCAMERA_PROFILE_CONCURRENCYINFO", SimStruct), offset=0)),)), name="_Camera_e__Struct", pack=False, align=None),
    "_Cancel_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_CANCEL_FLAGS")), ("Anonymous", SimUnion({"FetchData": SimTypeRef("_FetchData_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="_Cancel_e__Struct", pack=False, align=None),
    "_Capabilities_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_CaptureSample_e__Struct": SimStruct(OrderedDict((("Sample", SimTypePointer(SimTypeRef("WINBIO_BIR", SimStruct), offset=0)), ("SampleSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")),)), name="_CaptureSample_e__Struct", pack=False, align=None),
    "_ChangeTrackingState_e__Struct": SimStruct(OrderedDict((("Enabled", SimTypeInt(signed=True, label="Int32")), ("NewerChanges", SimTypeInt(signed=True, label="Int32")), ("MostRecentId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_ChangeTrackingState_e__Struct", pack=False, align=None),
    "_CharVal": SimStruct(OrderedDict((("sActualLength", SimTypeShort(signed=True, label="Int16")), ("sMaxLength", SimTypeShort(signed=True, label="Int16")), ("pchCharVal", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pwchReserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_CharVal", pack=False, align=None),
    "_Char_e__Union": SimUnion({"UnicodeChar": SimTypeChar(label="Char"), "AsciiChar": SimTypeChar(label="SByte")}, name="<anon>", label="None"),
    "_CharacteristicExtendedProperties_e__Struct": SimStruct(OrderedDict((("IsReliableWriteEnabled", SimTypeChar(label="Byte")), ("IsAuxiliariesWritable", SimTypeChar(label="Byte")),)), name="_CharacteristicExtendedProperties_e__Struct", pack=False, align=None),
    "_CharacteristicFormat_e__Struct": SimStruct(OrderedDict((("Format", SimTypeChar(label="Byte")), ("Exponent", SimTypeChar(label="Byte")), ("Unit", SimTypeRef("BTH_LE_UUID", SimStruct)), ("NameSpace", SimTypeChar(label="Byte")), ("Description", SimTypeRef("BTH_LE_UUID", SimStruct)),)), name="_CharacteristicFormat_e__Struct", pack=False, align=None),
    "_CheckInfo_e__Union": SimUnion({"CacheCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfCacheCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "TlbCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfTLBCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "BusCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfBusCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "MsCheck": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Anonymous_e__Struct", pack=False, align=None), "XpfMsCheck": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "AsULONGLONG": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "_ChunkFinished_e__Struct": SimStruct(OrderedDict((("dwStreamNumber", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("hSourceFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDestinationFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uliChunkNumber", SimTypeLongLong(signed=False, label="UInt64")), ("uliChunkSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliStreamBytesTransferred", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalFileSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalBytesTransferred", SimTypeLongLong(signed=False, label="UInt64")),)), name="_ChunkFinished_e__Struct", pack=False, align=None),
    "_ChunkStarted_e__Struct": SimStruct(OrderedDict((("dwStreamNumber", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("hSourceFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDestinationFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uliChunkNumber", SimTypeLongLong(signed=False, label="UInt64")), ("uliChunkSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalFileSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="_ChunkStarted_e__Struct", pack=False, align=None),
    "_Class_e__Struct": SimStruct(OrderedDict((("Type", SimTypeChar(label="Byte")), ("Level", SimTypeChar(label="Byte")), ("Version", SimTypeShort(signed=False, label="UInt16")),)), name="_Class_e__Struct", pack=False, align=None),
    "_ClientCharacteristicConfiguration_e__Struct": SimStruct(OrderedDict((("IsSubscribeToNotification", SimTypeChar(label="Byte")), ("IsSubscribeToIndication", SimTypeChar(label="Byte")),)), name="_ClientCharacteristicConfiguration_e__Struct", pack=False, align=None),
    "_CloseCompletion_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_CLOSE_COMPLETION_FLAGS")),)), name="_CloseCompletion_e__Struct", pack=False, align=None),
    "_CloseStatus_e__Struct": SimStruct(OrderedDict((("pbReason", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulReasonLength", SimTypeInt(signed=False, label="UInt32")), ("usStatus", SimTypeShort(signed=False, label="UInt16")),)), name="_CloseStatus_e__Struct", pack=False, align=None),
    "_ClosedSource_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_ClosedSource_e__Struct", pack=False, align=None),
    "_ColumnSortOrder": SimTypeInt(signed=False, label="_ColumnSortOrder"),
    "_Command_e__Union": SimUnion({"bits": SimTypeRef("_bits_e__Struct", SimStruct), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_Common_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("FsControlCode", SimTypeInt(signed=False, label="UInt32")),)), name="_Common_e__Struct", pack=False, align=None),
    "_ConfigData_e__Struct": SimStruct(OrderedDict((("Priority", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_ConfigData_e__Struct", pack=False, align=None),
    "_ConnectionContext_e__Union": SimUnion({"Generic": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "InterruptObject": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "InterruptMessageTable": SimTypePointer(SimTypeRef("IO_INTERRUPT_MESSAGE_INFO", SimStruct), offset=0)}, name="<anon>", label="None"),
    "_Connection_e__Struct": SimStruct(OrderedDict((("Class", SimTypeChar(label="Byte")), ("Type", SimTypeChar(label="Byte")), ("Reserved1", SimTypeChar(label="Byte")), ("Reserved2", SimTypeChar(label="Byte")), ("IdLowPart", SimTypeInt(signed=False, label="UInt32")), ("IdHighPart", SimTypeInt(signed=False, label="UInt32")),)), name="_Connection_e__Struct", pack=False, align=None),
    "_ConstantBufferView_e__Struct": SimStruct(OrderedDict((("RootParameterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="_ConstantBufferView_e__Struct", pack=False, align=None),
    "_Constant_e__Struct": SimStruct(OrderedDict((("RootParameterIndex", SimTypeInt(signed=False, label="UInt32")), ("DestOffsetIn32BitValues", SimTypeInt(signed=False, label="UInt32")), ("Num32BitValuesToSet", SimTypeInt(signed=False, label="UInt32")),)), name="_Constant_e__Struct", pack=False, align=None),
    "_Constraint_e__Union": SimUnion({"ActiveSize": SimTypeRef("D3DKMDT_2DREGION", SimStruct), "MaxPixelRate": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="<anon>", label="None"),
    "_ContiguousRange_e__Struct": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=True, label="Int64")), ("Size", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_ContiguousRange_e__Struct", pack=False, align=None),
    "_ControlUnit_e__Struct": SimStruct(OrderedDict((("Component", SimTypeInt(signed=False, label="WINBIO_COMPONENT")), ("ControlCode", SimTypeInt(signed=False, label="UInt32")), ("OperationStatus", SimTypeInt(signed=False, label="UInt32")), ("SendBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("SendBufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ReceiveBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ReceiveBufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ReceiveDataSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_ControlUnit_e__Struct", pack=False, align=None),
    "_Control_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_Copy_e__Struct": SimStruct(OrderedDict((("SourceAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("DestAddress", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Copy_e__Struct", pack=False, align=None),
    "_CpuSet_e__Struct": SimStruct(OrderedDict((("Id", SimTypeInt(signed=False, label="UInt32")), ("Group", SimTypeShort(signed=False, label="UInt16")), ("LogicalProcessorIndex", SimTypeChar(label="Byte")), ("CoreIndex", SimTypeChar(label="Byte")), ("LastLevelCacheIndex", SimTypeChar(label="Byte")), ("NumaNodeIndex", SimTypeChar(label="Byte")), ("EfficiencyClass", SimTypeChar(label="Byte")), ("Anonymous1", SimUnion({"AllFlags": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Anonymous2", SimUnion({"Reserved": SimTypeInt(signed=False, label="UInt32"), "SchedulingClass": SimTypeChar(label="Byte")}, name="<anon>", label="None")), ("AllocationTag", SimTypeLongLong(signed=False, label="UInt64")),)), name="_CpuSet_e__Struct", pack=False, align=None),
    "_CreateAgentSession_e__Struct": SimStruct(OrderedDict((("hAgentSession", SimTypeInt(signed=False, label="UInt32")), ("dwAgentPINSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentPINOffset", SimTypeInt(signed=False, label="UInt32")), ("hAgent", SimTypeInt(signed=False, label="UInt32")), ("GroupID", SimTypeBottom(label="Guid")), ("dwWorkingAddressID", SimTypeInt(signed=False, label="UInt32")),)), name="_CreateAgentSession_e__Struct", pack=False, align=None),
    "_CreateAgent_e__Struct": SimStruct(OrderedDict((("hAgent", SimTypeInt(signed=False, label="UInt32")), ("dwAgentIDSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentIDOffset", SimTypeInt(signed=False, label="UInt32")), ("dwAgentPINSize", SimTypeInt(signed=False, label="UInt32")), ("dwAgentPINOffset", SimTypeInt(signed=False, label="UInt32")),)), name="_CreateAgent_e__Struct", pack=False, align=None),
    "_CreateMailslot_e__Struct": SimStruct(OrderedDict((("SecurityContext", SimTypePointer(SimTypeRef("IO_SECURITY_CONTEXT", SimStruct), offset=0)), ("Options", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("ShareAccess", SimTypeShort(signed=False, label="UInt16")), ("Parameters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_CreateMailslot_e__Struct", pack=False, align=None),
    "_CreatePipe_e__Struct": SimStruct(OrderedDict((("SecurityContext", SimTypePointer(SimTypeRef("IO_SECURITY_CONTEXT", SimStruct), offset=0)), ("Options", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeShort(signed=False, label="UInt16")), ("ShareAccess", SimTypeShort(signed=False, label="UInt16")), ("Parameters", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_CreatePipe_e__Struct", pack=False, align=None),
    "_Create_e__Struct": SimStruct(OrderedDict((("SecurityContext", SimTypePointer(SimTypeRef("IO_SECURITY_CONTEXT", SimStruct), offset=0)), ("Options", SimTypeInt(signed=False, label="UInt32")), ("FileAttributes", SimTypeShort(signed=False, label="UInt16")), ("ShareAccess", SimTypeShort(signed=False, label="UInt16")), ("EaLength", SimTypeInt(signed=False, label="UInt32")), ("EaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AllocationSize", SimTypeLongLong(signed=True, label="Int64")),)), name="_Create_e__Struct", pack=False, align=None),
    "_CriticalWarning_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_CurrentCompletion_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_CurrentCompletion_e__Struct", pack=False, align=None),
    "_CurrentOperation_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_CurrentOperation_e__Struct", pack=False, align=None),
    "_Current_e__Union": SimUnion({"lVal": SimTypeInt(signed=True, label="Int32"), "fltVal": SimTypeFloat(size=32), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "guidVal": SimTypeBottom(label="Guid")}, name="<anon>", label="None"),
    "_DATASETMANAGEMENT_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_DATASETMANAGEMENT_e__Struct", pack=False, align=None),
    "_DC_e__Struct": SimStruct(OrderedDict((("lProgNumber", SimTypeInt(signed=True, label="Int32")),)), name="_DC_e__Struct", pack=False, align=None),
    "_DEVICE_OBJECT_POWER_EXTENSION": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "_DEV_BROADCAST_HEADER": SimStruct(OrderedDict((("dbcd_size", SimTypeInt(signed=False, label="UInt32")), ("dbcd_devicetype", SimTypeInt(signed=False, label="UInt32")), ("dbcd_reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_DEV_BROADCAST_HEADER", pack=False, align=None),
    "_DEV_BROADCAST_USERDEFINED": SimStruct(OrderedDict((("dbud_dbh", SimTypeRef("DEV_BROADCAST_HDR", SimStruct)), ("dbud_szName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="_DEV_BROADCAST_USERDEFINED", pack=False, align=None),
    "_DEV_QUERY_RESULT_UPDATE_PAYLOAD": SimUnion({"State": SimTypeInt(signed=False, label="DEV_QUERY_STATE"), "DeviceObject": SimTypeRef("DEV_OBJECT", SimStruct)}, name="<anon>", label="None"),
    "_DHCP_CLIENT_SEARCH_UNION_V6": SimUnion({"ClientIpAddress": SimTypeRef("DHCP_IPV6_ADDRESS", SimStruct), "ClientDUID": SimTypeRef("DHCP_BINARY_DATA", SimStruct), "ClientName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_DHCP_OPTION_SCOPE_UNION": SimUnion({"DefaultScopeInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "GlobalScopeInfo": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "SubnetScopeInfo": SimTypeInt(signed=False, label="UInt32"), "ReservedScopeInfo": SimTypeRef("DHCP_RESERVED_SCOPE", SimStruct), "MScopeInfo": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_DHCP_PROPERTY_VALUE_UNION": SimUnion({"ByteValue": SimTypeChar(label="Byte"), "WordValue": SimTypeShort(signed=False, label="UInt16"), "DWordValue": SimTypeInt(signed=False, label="UInt32"), "StringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryValue": SimTypeRef("DHCP_BINARY_DATA", SimStruct)}, name="<anon>", label="None"),
    "_DHCP_SUBNET_ELEMENT_UNION_V5": SimUnion({"IpRange": SimTypePointer(SimTypeRef("DHCP_BOOTP_IP_RANGE", SimStruct), offset=0), "SecondaryHost": SimTypePointer(SimTypeRef("DHCP_HOST_INFO", SimStruct), offset=0), "ReservedIp": SimTypePointer(SimTypeRef("DHCP_IP_RESERVATION_V4", SimStruct), offset=0), "ExcludeIpRange": SimTypePointer(SimTypeRef("DHCP_IP_RANGE", SimStruct), offset=0), "IpUsedCluster": SimTypePointer(SimTypeRef("DHCP_IP_CLUSTER", SimStruct), offset=0)}, name="<anon>", label="None"),
    "_DIRECTIVERECEIVE_e__Struct": SimStruct(OrderedDict((("CDW10", SimTypeRef("NVME_CDW10_DIRECTIVE_RECEIVE", SimStruct)), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimUnion({"AllocateResources": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_DIRECTIVERECEIVE_e__Struct", pack=False, align=None),
    "_DIRECTIVESEND_e__Struct": SimStruct(OrderedDict((("CDW10", SimTypeRef("NVME_CDW10_DIRECTIVE_SEND", SimStruct)), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimUnion({"EnableDirective": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_DIRECTIVESEND_e__Struct", pack=False, align=None),
    "_DLFEAT_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_DLFEAT_e__Struct", pack=False, align=None),
    "_DMO_INPLACE_PROCESS_FLAGS": SimTypeInt(signed=False, label="_DMO_INPLACE_PROCESS_FLAGS"),
    "_DMO_INPUT_DATA_BUFFER_FLAGS": SimTypeInt(signed=False, label="_DMO_INPUT_DATA_BUFFER_FLAGS"),
    "_DMO_INPUT_STATUS_FLAGS": SimTypeInt(signed=False, label="_DMO_INPUT_STATUS_FLAGS"),
    "_DMO_INPUT_STREAM_INFO_FLAGS": SimTypeInt(signed=False, label="_DMO_INPUT_STREAM_INFO_FLAGS"),
    "_DMO_OUTPUT_DATA_BUFFER_FLAGS": SimTypeInt(signed=False, label="_DMO_OUTPUT_DATA_BUFFER_FLAGS"),
    "_DMO_OUTPUT_STREAM_INFO_FLAGS": SimTypeInt(signed=False, label="_DMO_OUTPUT_STREAM_INFO_FLAGS"),
    "_DMO_PROCESS_OUTPUT_FLAGS": SimTypeInt(signed=False, label="_DMO_PROCESS_OUTPUT_FLAGS"),
    "_DMO_QUALITY_STATUS_FLAGS": SimTypeInt(signed=False, label="_DMO_QUALITY_STATUS_FLAGS"),
    "_DMO_SET_TYPE_FLAGS": SimTypeInt(signed=False, label="_DMO_SET_TYPE_FLAGS"),
    "_DMO_VIDEO_OUTPUT_STREAM_FLAGS": SimTypeInt(signed=False, label="_DMO_VIDEO_OUTPUT_STREAM_FLAGS"),
    "_DMSMQEventEvents": SimTypeBottom(label="_DMSMQEventEvents"),
    "_DNS_CONNECTION_PROXY_INFO_CONFIG": SimStruct(OrderedDict((("pwszServer", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszUsername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszException", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszExtraInfo", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("Port", SimTypeShort(signed=False, label="UInt16")),)), name="_DNS_CONNECTION_PROXY_INFO_CONFIG", pack=False, align=None),
    "_DNS_CONNECTION_PROXY_INFO_SCRIPT": SimStruct(OrderedDict((("pwszScript", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszUsername", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pwszPassword", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_DNS_CONNECTION_PROXY_INFO_SCRIPT", pack=False, align=None),
    "_DPC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_DPC_e__Struct", pack=False, align=None),
    "_DPS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_DPS_e__Struct", pack=False, align=None),
    "_DSM_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_DSM_e__Struct", pack=False, align=None),
    "_DVB_e__Struct": SimStruct(OrderedDict((("lONID", SimTypeInt(signed=True, label="Int32")), ("lTSID", SimTypeInt(signed=True, label="Int32")), ("lSID", SimTypeInt(signed=True, label="Int32")),)), name="_DVB_e__Struct", pack=False, align=None),
    "_DVDECODERRESOLUTION": SimTypeInt(signed=False, label="_DVDECODERRESOLUTION"),
    "_DVENCODERFORMAT": SimTypeInt(signed=False, label="_DVENCODERFORMAT"),
    "_DVENCODERRESOLUTION": SimTypeInt(signed=False, label="_DVENCODERRESOLUTION"),
    "_DVENCODERVIDEOFORMAT": SimTypeInt(signed=False, label="_DVENCODERVIDEOFORMAT"),
    "_DVRESOLUTION": SimTypeInt(signed=False, label="_DVRESOLUTION"),
    "_DW2_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("SQHD", SimTypeShort(signed=False, label="UInt16")), ("SQID", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_DW3_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("CID", SimTypeShort(signed=False, label="UInt16")), ("Status", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_DataStream": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Vdl", SimTypeLongLong(signed=False, label="UInt64")),)), name="_DataStream", pack=False, align=None),
    "_Data_e__Struct": SimStruct(OrderedDict((("pbBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("ulBufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="_Data_e__Struct", pack=False, align=None),
    "_Data_e__Union": SimUnion({"OPT": SimTypeRef("DNS_OPT_DATA", SimStruct), "Opt": SimTypeRef("DNS_OPT_DATA", SimStruct)}, name="<anon>", label="None"),
    "_DatasetMgmtCommand_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_DebugLogStampUnits_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_Defrag_e__Struct": SimStruct(OrderedDict((("Operation", SimTypeInt(signed=False, label="D3DKMT_DEFRAG_ESCAPE_OPERATION")), ("SegmentId", SimTypeInt(signed=False, label="UInt32")), ("TotalCommitted", SimTypeLongLong(signed=False, label="UInt64")), ("TotalFree", SimTypeLongLong(signed=False, label="UInt64")), ("LargestGapBefore", SimTypeLongLong(signed=False, label="UInt64")), ("LargestGapAfter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Defrag_e__Struct", pack=False, align=None),
    "_DehydrateCompletion_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATE_COMPLETION_FLAGS")), ("Reason", SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATION_REASON")),)), name="_DehydrateCompletion_e__Struct", pack=False, align=None),
    "_Dehydrate_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATE_FLAGS")), ("Reason", SimTypeInt(signed=False, label="CF_CALLBACK_DEHYDRATION_REASON")),)), name="_Dehydrate_e__Struct", pack=False, align=None),
    "_DelayExecution_e__Struct": SimStruct(OrderedDict((("hPagingQueue", SimTypeInt(signed=False, label="UInt32")), ("PhysicalAdapterIndex", SimTypeInt(signed=False, label="UInt32")), ("Milliseconds", SimTypeInt(signed=False, label="UInt32")), ("PagingFenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="_DelayExecution_e__Struct", pack=False, align=None),
    "_DelayedEvictionConfig_e__Struct": SimStruct(OrderedDict((("TimerValue", SimTypeLongLong(signed=True, label="Int64")),)), name="_DelayedEvictionConfig_e__Struct", pack=False, align=None),
    "_DeleteCompletion_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_DELETE_COMPLETION_FLAGS")),)), name="_DeleteCompletion_e__Struct", pack=False, align=None),
    "_DeleteTemplate_e__Struct": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactor", SimTypeChar(label="Byte")),)), name="_DeleteTemplate_e__Struct", pack=False, align=None),
    "_Delete_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_DELETE_FLAGS")),)), name="_Delete_e__Struct", pack=False, align=None),
    "_Description_e__Union": SimUnion({"Name": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "Ordinal": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_DesiredStorageClass": SimStruct(OrderedDict((("Class", SimTypeInt(signed=False, label="FILE_STORAGE_TIER_CLASS")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_DesiredStorageClass", pack=False, align=None),
    "_Detailed_e__Struct": SimStruct(OrderedDict((("LocalizedReasonModule", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("LocalizedReasonId", SimTypeInt(signed=False, label="UInt32")), ("ReasonStringCount", SimTypeInt(signed=False, label="UInt32")), ("ReasonStrings", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="_Detailed_e__Struct", pack=False, align=None),
    "_DeviceCapabilities_e__Struct": SimStruct(OrderedDict((("Capabilities", SimTypePointer(SimTypeRef("DEVICE_CAPABILITIES", SimStruct), offset=0)),)), name="_DeviceCapabilities_e__Struct", pack=False, align=None),
    "_DeviceHandle_e__Struct": SimStruct(OrderedDict((("EventGuid", SimTypeBottom(label="Guid")), ("NameOffset", SimTypeInt(signed=True, label="Int32")), ("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Data", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_DeviceHandle_e__Struct", pack=False, align=None),
    "_DeviceInstance_e__Struct": SimStruct(OrderedDict((("InstanceId", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_DeviceInstance_e__Struct", pack=False, align=None),
    "_DeviceInterface_e__Struct": SimStruct(OrderedDict((("ClassGuid", SimTypeBottom(label="Guid")), ("SymbolicLink", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_DeviceInterface_e__Struct", pack=False, align=None),
    "_DeviceInterrupt_e__Struct": SimStruct(OrderedDict((("LogicalDeviceId", SimTypeLongLong(signed=False, label="UInt64")), ("MsiAddress", SimTypeLongLong(signed=False, label="UInt64")), ("MsiData", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_DeviceInterrupt_e__Struct", pack=False, align=None),
    "_DeviceIoControl_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("IoControlCode", SimTypeInt(signed=False, label="UInt32")), ("Type3InputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_DeviceIoControl_e__Struct", pack=False, align=None),
    "_DeviceIoControl_e__Union": SimUnion({"Common": SimTypeRef("_Common_e__Struct", SimStruct), "Neither": SimTypeRef("_Neither_e__Struct", SimStruct), "Buffered": SimTypeRef("_Buffered_e__Struct", SimStruct), "Direct": SimTypeRef("_Direct_e__Struct", SimStruct), "FastIo": SimTypeRef("_FastIo_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_DevicePrivate_e__Struct": SimStruct(OrderedDict((("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)),)), name="_DevicePrivate_e__Struct", pack=False, align=None),
    "_DeviceSpecificData_e__Struct": SimStruct(OrderedDict((("DataSize", SimTypeInt(signed=False, label="UInt32")), ("Reserved1", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_DeviceSpecificData_e__Struct", pack=False, align=None),
    "_DeviceSpecific_e__Union": SimUnion({"DiskInfo": SimTypeRef("_DiskInfo_e__Struct", SimStruct), "RemovableDiskInfo": SimTypeRef("_RemovableDiskInfo_e__Struct", SimStruct), "TapeInfo": SimTypeRef("_TapeInfo_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Direct_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("FsControlCode", SimTypeInt(signed=False, label="UInt32")), ("InputSystemBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputMdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_Direct_e__Struct", pack=False, align=None),
    "_DirectoryControl_e__Union": SimUnion({"QueryDirectory": SimTypeRef("_QueryDirectory_e__Struct", SimStruct), "NotifyDirectory": SimTypeRef("_NotifyDirectory_e__Struct", SimStruct), "NotifyDirectoryEx": SimTypeRef("_NotifyDirectoryEx_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Discrete_e__Struct": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("States", SimTypePointer(SimTypeRef("PO_FX_PERF_STATE", SimStruct), offset=0)),)), name="_Discrete_e__Struct", pack=False, align=None),
    "_DiskInfo_e__Struct": SimStruct(OrderedDict((("Cylinders", SimTypeLongLong(signed=True, label="Int64")), ("MediaType", SimTypeInt(signed=False, label="STORAGE_MEDIA_TYPE")), ("TracksPerCylinder", SimTypeInt(signed=False, label="UInt32")), ("SectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("NumberMediaSides", SimTypeInt(signed=False, label="UInt32")), ("MediaCharacteristics", SimTypeInt(signed=False, label="UInt32")),)), name="_DiskInfo_e__Struct", pack=False, align=None),
    "_DmaV3_e__Struct": SimStruct(OrderedDict((("RequestLine", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Channel", SimTypeInt(signed=False, label="UInt32")), ("TransferWidth", SimTypeInt(signed=False, label="UInt32")),)), name="_DmaV3_e__Struct", pack=False, align=None),
    "_Dma_e__Struct": SimStruct(OrderedDict((("MinimumChannel", SimTypeInt(signed=False, label="UInt32")), ("MaximumChannel", SimTypeInt(signed=False, label="UInt32")),)), name="_Dma_e__Struct", pack=False, align=None),
    "_DnsRecordOptA": SimStruct(OrderedDict((("pNext", SimTypePointer(SimTypeRef("DNS_RECORDA", SimStruct), offset=0)), ("pName", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("wType", SimTypeShort(signed=False, label="UInt16")), ("wDataLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimUnion({"DW": SimTypeInt(signed=False, label="UInt32"), "S": SimTypeRef("DNS_RECORD_FLAGS", SimStruct)}, name="<anon>", label="None")), ("ExtHeader", SimTypeRef("DNS_HEADER_EXT", SimStruct)), ("wPayloadSize", SimTypeShort(signed=False, label="UInt16")), ("wReserved", SimTypeShort(signed=False, label="UInt16")), ("Data", SimUnion({"OPT": SimTypeRef("DNS_OPT_DATA", SimStruct), "Opt": SimTypeRef("DNS_OPT_DATA", SimStruct)}, name="<anon>", label="None")),)), name="_DnsRecordOptA", pack=False, align=None),
    "_DocumentMode": SimTypeInt(signed=False, label="_DocumentMode"),
    "_DriverName_e__Struct": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("MaximumLength", SimTypeShort(signed=False, label="UInt16")), ("Buffer", SimTypeLongLong(signed=False, label="UInt64")),)), name="_DriverName_e__Struct", pack=False, align=None),
    "_ERROR_INFOW": SimStruct(OrderedDict((("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwGenericError", SimTypeInt(signed=False, label="UInt32")), ("dwVendorError", SimTypeInt(signed=False, label="UInt32")), ("szExtendedErrorText", SimTypeArray(SimTypeChar(label="Char"), 255)),)), name="_ERROR_INFOW", pack=False, align=None),
    "_EXPCMDFLAGS": SimTypeInt(signed=False, label="_EXPCMDFLAGS"),
    "_EXPCMDSTATE": SimTypeInt(signed=False, label="_EXPCMDSTATE"),
    "_EXPLORERPANESTATE": SimTypeInt(signed=False, label="_EXPLORERPANESTATE"),
    "_EXPPS": SimTypeInt(signed=False, label="_EXPPS"),
    "_EXT_SET_PARAMETERS_V0": SimStruct(OrderedDict((("Version", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("NoWakeTolerance", SimTypeLongLong(signed=True, label="Int64")),)), name="_EXT_SET_PARAMETERS_V0", pack=False, align=None),
    "_Ea": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("EaSize", SimTypeInt(signed=False, label="UInt32")), ("EaInformationOffset", SimTypeInt(signed=False, label="UInt32")),)), name="_Ea", pack=False, align=None),
    "_EaInformation_e__Struct": SimStruct(OrderedDict((("EaBufferSize", SimTypeInt(signed=False, label="UInt32")), ("OffsetToFirstEa", SimTypeInt(signed=False, label="UInt32")),)), name="_EaInformation_e__Struct", pack=False, align=None),
    "_EccControlStatus_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsULONG": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_EndToEndCorrectionCounts_e__Struct": SimStruct(OrderedDict((("DetectedCounts", SimTypeInt(signed=False, label="UInt32")), ("CorrectedCounts", SimTypeInt(signed=False, label="UInt32")),)), name="_EndToEndCorrectionCounts_e__Struct", pack=False, align=None),
    "_EnrollBegin_e__Struct": SimStruct(OrderedDict((("SubFactor", SimTypeChar(label="Byte")),)), name="_EnrollBegin_e__Struct", pack=False, align=None),
    "_EnrollCapture_e__Struct": SimStruct(OrderedDict((("RejectDetail", SimTypeInt(signed=False, label="UInt32")),)), name="_EnrollCapture_e__Struct", pack=False, align=None),
    "_EnrollCommit_e__Struct": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("IsNewTemplate", SimTypeChar(label="Byte")),)), name="_EnrollCommit_e__Struct", pack=False, align=None),
    "_EnrollSelect_e__Struct": SimStruct(OrderedDict((("SelectorValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="_EnrollSelect_e__Struct", pack=False, align=None),
    "_EnrollmentRequirements_e__Struct": SimStruct(OrderedDict((("Null", SimTypeInt(signed=False, label="UInt32")),)), name="_EnrollmentRequirements_e__Struct", pack=False, align=None),
    "_EnumBiometricUnits_e__Struct": SimStruct(OrderedDict((("UnitCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("UnitSchemaArray", SimTypePointer(SimTypeRef("WINBIO_UNIT_SCHEMA", SimStruct), offset=0)),)), name="_EnumBiometricUnits_e__Struct", pack=False, align=None),
    "_EnumDatabases_e__Struct": SimStruct(OrderedDict((("StorageCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("StorageSchemaArray", SimTypePointer(SimTypeRef("WINBIO_STORAGE_SCHEMA", SimStruct), offset=0)),)), name="_EnumDatabases_e__Struct", pack=False, align=None),
    "_EnumEnrollments_e__Struct": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactorCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("SubFactorArray", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_EnumEnrollments_e__Struct", pack=False, align=None),
    "_EnumServiceProviders_e__Struct": SimStruct(OrderedDict((("BspCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BspSchemaArray", SimTypePointer(SimTypeRef("WINBIO_BSP_SCHEMA", SimStruct), offset=0)),)), name="_EnumServiceProviders_e__Struct", pack=False, align=None),
    "_Enumeration_e__Struct": SimStruct(OrderedDict((("DirEntryBufferHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_Enumeration_e__Struct", pack=False, align=None),
    "_Error_e__Struct": SimStruct(OrderedDict((("CopyPhase", SimTypeInt(signed=False, label="COPYFILE2_COPY_PHASE")), ("dwStreamNumber", SimTypeInt(signed=False, label="UInt32")), ("hrFailure", SimTypeInt(signed=True, label="Int32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("uliChunkNumber", SimTypeLongLong(signed=False, label="UInt64")), ("uliStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliStreamBytesTransferred", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalFileSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalBytesTransferred", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Error_e__Struct", pack=False, align=None),
    "_EventConnector": SimTypeBottom(label="_EventConnector"),
    "_EventHandle_e__Struct": SimStruct(OrderedDict((("Event", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved", SimTypeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 2)),)), name="_EventHandle_e__Struct", pack=False, align=None),
    "_Event_e__Struct": SimStruct(OrderedDict((("ConnectionId", SimTypeInt(signed=False, label="UInt32")),)), name="_Event_e__Struct", pack=False, align=None),
    "_Event_e__Union": SimUnion({"KeyEvent": SimTypeRef("KEY_EVENT_RECORD", SimStruct), "MouseEvent": SimTypeRef("MOUSE_EVENT_RECORD", SimStruct), "WindowBufferSizeEvent": SimTypeRef("WINDOW_BUFFER_SIZE_RECORD", SimStruct), "MenuEvent": SimTypeRef("MENU_EVENT_RECORD", SimStruct), "FocusEvent": SimTypeRef("FOCUS_EVENT_RECORD", SimStruct)}, name="<anon>", label="None"),
    "_EvictByNtHandle_e__Struct": SimStruct(OrderedDict((("NtHandle", SimTypeLongLong(signed=False, label="UInt64")),)), name="_EvictByNtHandle_e__Struct", pack=False, align=None),
    "_Evict_e__Struct": SimStruct(OrderedDict((("ResourceHandle", SimTypeInt(signed=False, label="UInt32")), ("AllocationHandle", SimTypeInt(signed=False, label="UInt32")), ("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_Evict_e__Struct", pack=False, align=None),
    "_ExceptionInformation_e__Struct": SimStruct(OrderedDict((("IsUncaught", SimTypeChar(label="Byte")),)), name="_ExceptionInformation_e__Struct", pack=False, align=None),
    "_ExportListOptions": SimTypeInt(signed=False, label="_ExportListOptions"),
    "_ExtentInformation_e__Union": SimUnion({"RetrievalPointers": SimTypeRef("RETRIEVAL_POINTERS_BUFFER", SimStruct)}, name="<anon>", label="None"),
    "_ExternalStack_e__Struct": SimStruct(OrderedDict((("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="_ExternalStack_e__Struct", pack=False, align=None),
    "_F128Parts64_e__Struct": SimStruct(OrderedDict((("LowPart", SimTypeLongLong(signed=False, label="UInt64")), ("HighPart", SimTypeLongLong(signed=True, label="Int64")),)), name="_F128Parts64_e__Struct", pack=False, align=None),
    "_FHSS_e__Struct": SimStruct(OrderedDict((("uHopPattern", SimTypeInt(signed=False, label="UInt32")), ("uHopSet", SimTypeInt(signed=False, label="UInt32")), ("uDwellTime", SimTypeInt(signed=False, label="UInt32")),)), name="_FHSS_e__Struct", pack=False, align=None),
    "_FIRMWAREACTIVATE_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_FIRMWAREACTIVATE_e__Struct", pack=False, align=None),
    "_FIRMWAREDOWNLOAD_e__Struct": SimStruct(OrderedDict((("CDW10", SimTypeRef("NVME_CDW10_FIRMWARE_DOWNLOAD", SimStruct)), ("CDW11", SimTypeRef("NVME_CDW11_FIRMWARE_DOWNLOAD", SimStruct)), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_FIRMWAREDOWNLOAD_e__Struct", pack=False, align=None),
    "_FLBAS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_FLBAS_e__Struct", pack=False, align=None),
    "_FNA_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_FNA_e__Struct", pack=False, align=None),
    "_FORMATNVM_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_FORMATNVM_e__Struct", pack=False, align=None),
    "_FPI_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_FPI_e__Struct", pack=False, align=None),
    "_FRMW_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_FRMW_e__Struct", pack=False, align=None),
    "_FUSES_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_FUSES_e__Struct", pack=False, align=None),
    "_FacialFeatures_e__Struct": SimStruct(OrderedDict((("BoundingBox", SimTypeRef("RECT", SimStruct)), ("Distance", SimTypeInt(signed=True, label="Int32")), ("OpaqueEngineData", SimTypeRef("_OpaqueEngineData_e__Struct", SimStruct)),)), name="_FacialFeatures_e__Struct", pack=False, align=None),
    "_FailureReason_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_FailureReason_e__Struct", pack=False, align=None),
    "_FastIoCheckIfPossible_e__Struct": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("LockKey", SimTypeInt(signed=False, label="UInt32")), ("CheckForReadOperation", SimTypeChar(label="Byte")),)), name="_FastIoCheckIfPossible_e__Struct", pack=False, align=None),
    "_FastIo_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("IoControlCode", SimTypeInt(signed=False, label="UInt32")), ("InputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_FastIo_e__Struct", pack=False, align=None),
    "_FcnAry_e__Union": SimUnion({"Function": SimTypeRef("_Function_e__Struct", SimStruct), "Array": SimTypeRef("_Array_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Fence_e__Struct": SimStruct(OrderedDict((("FenceValue", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Fence_e__Struct", pack=False, align=None),
    "_FetchData_e__Struct": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="_FetchData_e__Struct", pack=False, align=None),
    "_FetchPlaceholders_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_FETCH_PLACEHOLDERS_FLAGS")), ("Pattern", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_FetchPlaceholders_e__Struct", pack=False, align=None),
    "_Fields_e__Union": SimUnion({"Others": SimTypeRef("_Others", SimStruct), "Value": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"),
    "_FileDeletedOnHandleClose_e__Struct": SimStruct(OrderedDict((("IsFileModified", SimTypeChar(label="Byte")),)), name="_FileDeletedOnHandleClose_e__Struct", pack=False, align=None),
    "_FileRenamed_e__Struct": SimStruct(OrderedDict((("NotificationMask", SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")),)), name="_FileRenamed_e__Struct", pack=False, align=None),
    "_FileSystemControl_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("FsControlCode", SimTypeInt(signed=False, label="UInt32")), ("Type3InputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_FileSystemControl_e__Struct", pack=False, align=None),
    "_FileSystemControl_e__Union": SimUnion({"VerifyVolume": SimTypeRef("_VerifyVolume_e__Struct", SimStruct), "Common": SimTypeRef("_Common_e__Struct", SimStruct), "Neither": SimTypeRef("_Neither_e__Struct", SimStruct), "Buffered": SimTypeRef("_Buffered_e__Struct", SimStruct), "Direct": SimTypeRef("_Direct_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_File_e__Struct": SimStruct(OrderedDict((("Name", SimTypeArray(SimTypeChar(label="Byte"), 20)),)), name="_File_e__Struct", pack=False, align=None),
    "_FilterParameter_e__Union": SimUnion({"Dummy": SimTypeInt(signed=True, label="Int32"), "Genre": SimTypeInt(signed=False, label="PM_APP_GENRE"), "AppHubType": SimTypeInt(signed=False, label="PM_APPLICATION_HUBTYPE"), "HubType": SimTypeInt(signed=False, label="PM_TILE_HUBTYPE"), "Tasktype": SimTypeInt(signed=False, label="PM_TASK_TYPE"), "TaskProductID": SimTypeBottom(label="Guid"), "TileProductID": SimTypeBottom(label="Guid"), "AppTaskType": SimTypeRef("PM_APPTASKTYPE", SimStruct), "Consumer": SimTypeRef("PM_EXTENSIONCONSUMER", SimStruct), "BSATask": SimTypeRef("PM_BSATASKID", SimStruct), "BSAProductID": SimTypeBottom(label="Guid"), "BWTask": SimTypeRef("PM_BWTASKID", SimStruct), "ProtocolName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "FileType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ContentType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "AppSupportedFileExtPID": SimTypeBottom(label="Guid"), "ShareTargetFileType": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_FilterResourceRequirements_e__Struct": SimStruct(OrderedDict((("IoResourceRequirementList", SimTypePointer(SimTypeRef("IO_RESOURCE_REQUIREMENTS_LIST", SimStruct), offset=0)),)), name="_FilterResourceRequirements_e__Struct", pack=False, align=None),
    "_Filter_e__Union": SimUnion({"ClusterRanges": SimTypePointer(SimTypeRef("CLUSTER_RANGE", SimStruct), offset=0), "FileReferenceRanges": SimTypePointer(SimTypeRef("FILE_REFERENCE_RANGE", SimStruct), offset=0), "StorageReserveIds": SimTypePointer(SimTypeInt(signed=False, label="STORAGE_RESERVE_ID"), offset=0)}, name="<anon>", label="None"),
    "_Fingerprint_e__Struct": SimStruct(OrderedDict((("GeneralSamples", SimTypeInt(signed=False, label="UInt32")), ("Center", SimTypeInt(signed=False, label="UInt32")), ("TopEdge", SimTypeInt(signed=False, label="UInt32")), ("BottomEdge", SimTypeInt(signed=False, label="UInt32")), ("LeftEdge", SimTypeInt(signed=False, label="UInt32")), ("RightEdge", SimTypeInt(signed=False, label="UInt32")),)), name="_Fingerprint_e__Struct", pack=False, align=None),
    "_FirmwareActivationCapability_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_FirmwareActivationCapability_e__Struct", pack=False, align=None),
    "_Flag_e__Struct": SimStruct(OrderedDict((("Nom", SimTypeInt(signed=True, label="Int32")), ("ValidBits", SimTypeInt(signed=True, label="Int32")),)), name="_Flag_e__Struct", pack=False, align=None),
    "_Flags_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Flags_e__Struct", pack=False, align=None),
    "_Flags_e__Union": SimUnion({"AsBYTE": SimTypeChar(label="Byte"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "_ForestTrustData_e__Union": SimUnion({"TopLevelName": SimTypeRef("LSA_UNICODE_STRING", SimStruct), "DomainInfo": SimTypeRef("LSA_FOREST_TRUST_DOMAIN_INFO", SimStruct), "BinaryData": SimTypeRef("LSA_FOREST_TRUST_BINARY_DATA", SimStruct), "ScannerInfo": SimTypeRef("LSA_FOREST_TRUST_SCANNER_INFO", SimStruct)}, name="<anon>", label="None"),
    "_Format_e__Union": SimUnion({"Graphics": SimTypeRef("D3DKMDT_GRAPHICS_RENDERING_FORMAT", SimStruct), "Text": SimTypeInt(signed=False, label="D3DKMDT_TEXT_RENDERING_FORMAT")}, name="<anon>", label="None"),
    "_FpControlStatus_e__Struct": SimStruct(OrderedDict((("FpControl", SimTypeShort(signed=False, label="UInt16")), ("FpStatus", SimTypeShort(signed=False, label="UInt16")), ("FpTag", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("LastFpOp", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"LastFpRip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpEip", SimTypeInt(signed=False, label="UInt32")), ("LastFpCs", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_FpControlStatus_e__Struct", pack=False, align=None),
    "_FromFileHandle_e__Struct": SimStruct(OrderedDict((("ByteRange", SimTypeRef("HTTP_BYTE_RANGE", SimStruct)), ("FileHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_FromFileHandle_e__Struct", pack=False, align=None),
    "_FromFragmentCacheEx_e__Struct": SimStruct(OrderedDict((("ByteRange", SimTypeRef("HTTP_BYTE_RANGE", SimStruct)), ("pFragmentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_FromFragmentCacheEx_e__Struct", pack=False, align=None),
    "_FromFragmentCache_e__Struct": SimStruct(OrderedDict((("FragmentNameLength", SimTypeShort(signed=False, label="UInt16")), ("pFragmentName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_FromFragmentCache_e__Struct", pack=False, align=None),
    "_FromMemory_e__Struct": SimStruct(OrderedDict((("pBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferLength", SimTypeInt(signed=False, label="UInt32")),)), name="_FromMemory_e__Struct", pack=False, align=None),
    "_Function_e__Struct": SimStruct(OrderedDict((("PointerToLinenumber", SimTypeInt(signed=False, label="UInt32")), ("PointerToNextFunction", SimTypeInt(signed=False, label="UInt32")),)), name="_Function_e__Struct", pack=False, align=None),
    "_FusedCommand_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_GENERAL_e__Struct": SimStruct(OrderedDict((("CDW10", SimTypeInt(signed=False, label="UInt32")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_GENERAL_e__Struct", pack=False, align=None),
    "_GETFEATURES_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"NumberOfQueues": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "InterruptCoalescing": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "InterruptVectorConfig": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "LbaRangeType": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Arbitration": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VolatileWriteCache": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsyncEventConfig": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PowerManagement": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AutoPowerStateTransition": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "TemperatureThreshold": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "ErrorRecovery": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "WriteAtomicityNormal": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "NonOperationalPowerState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "IoCommandSetProfile": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "ErrorInjection": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "HostIdentifier": SimTypeRef("NVME_CDW11_FEATURE_HOST_IDENTIFIER", SimStruct), "ReservationPersistence": SimTypeRef("NVME_CDW11_FEATURE_RESERVATION_PERSISTENCE", SimStruct), "ReservationNotificationMask": SimTypeRef("NVME_CDW11_FEATURE_RESERVATION_NOTIFICATION_MASK", SimStruct), "GetHostMetadata": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "SetHostMetadata": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_GETFEATURES_e__Struct", pack=False, align=None),
    "_GETLOGPAGE_e__Struct": SimStruct(OrderedDict((("Anonymous", SimUnion({"CDW10": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "CDW10_V13": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeRef("NVME_CDW12_GET_LOG_PAGE", SimStruct)), ("CDW13", SimTypeRef("NVME_CDW13_GET_LOG_PAGE", SimStruct)), ("CDW14", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_GETLOGPAGE_e__Struct", pack=False, align=None),
    "_GenericGUIDReparseBuffer_e__Struct": SimStruct(OrderedDict((("TagGuid", SimTypeBottom(label="Guid")), ("DataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_GenericGUIDReparseBuffer_e__Struct", pack=False, align=None),
    "_GenericReparseBuffer_e__Struct": SimStruct(OrderedDict((("DataBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_GenericReparseBuffer_e__Struct", pack=False, align=None),
    "_GenericReserved_e__Struct": SimStruct(OrderedDict((("Reserved", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 8)),)), name="_GenericReserved_e__Struct", pack=False, align=None),
    "_Generic_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Alignment", SimTypeInt(signed=False, label="UInt32")), ("MinimumAddress", SimTypeLongLong(signed=True, label="Int64")), ("MaximumAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="_Generic_e__Struct", pack=False, align=None),
    "_GetAgentActivityList_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("ActivityList", SimTypeRef("LINEAGENTACTIVITYLIST", SimStruct)),)), name="_GetAgentActivityList_e__Struct", pack=False, align=None),
    "_GetAgentCaps_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("AgentCaps", SimTypeRef("LINEAGENTCAPS", SimStruct)),)), name="_GetAgentCaps_e__Struct", pack=False, align=None),
    "_GetAgentGroupList_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("GroupList", SimTypeRef("LINEAGENTGROUPLIST", SimStruct)),)), name="_GetAgentGroupList_e__Struct", pack=False, align=None),
    "_GetAgentInfo_e__Struct": SimStruct(OrderedDict((("hAgent", SimTypeInt(signed=False, label="UInt32")), ("AgentInfo", SimTypeRef("LINEAGENTINFO", SimStruct)),)), name="_GetAgentInfo_e__Struct", pack=False, align=None),
    "_GetAgentSessionInfo_e__Struct": SimStruct(OrderedDict((("hAgentSession", SimTypeInt(signed=False, label="UInt32")), ("SessionInfo", SimTypeRef("LINEAGENTSESSIONINFO", SimStruct)),)), name="_GetAgentSessionInfo_e__Struct", pack=False, align=None),
    "_GetAgentSessionList_e__Struct": SimStruct(OrderedDict((("hAgent", SimTypeInt(signed=False, label="UInt32")), ("SessionList", SimTypeRef("LINEAGENTSESSIONLIST", SimStruct)),)), name="_GetAgentSessionList_e__Struct", pack=False, align=None),
    "_GetAgentStatus_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("AgentStatus", SimTypeRef("LINEAGENTSTATUS", SimStruct)),)), name="_GetAgentStatus_e__Struct", pack=False, align=None),
    "_GetBudget_e__Struct": SimStruct(OrderedDict((("NumBytesToTrim", SimTypeLongLong(signed=False, label="UInt64")),)), name="_GetBudget_e__Struct", pack=False, align=None),
    "_GetEvent_e__Struct": SimStruct(OrderedDict((("Event", SimTypeRef("WINBIO_EVENT", SimStruct)),)), name="_GetEvent_e__Struct", pack=False, align=None),
    "_GetGroupList_e__Struct": SimStruct(OrderedDict((("GroupList", SimTypeRef("LINEAGENTGROUPLIST", SimStruct)),)), name="_GetGroupList_e__Struct", pack=False, align=None),
    "_GetNumVads_e__Struct": SimStruct(OrderedDict((("NumVads", SimTypeInt(signed=False, label="UInt32")),)), name="_GetNumVads_e__Struct", pack=False, align=None),
    "_GetProperty_e__Struct": SimStruct(OrderedDict((("PropertyType", SimTypeInt(signed=False, label="UInt32")), ("PropertyId", SimTypeInt(signed=False, label="UInt32")), ("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactor", SimTypeChar(label="Byte")), ("PropertyBufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PropertyBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_GetProperty_e__Struct", pack=False, align=None),
    "_GetProtectionPolicy_e__Struct": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("Policy", SimTypeRef("WINBIO_PROTECTION_POLICY", SimStruct)),)), name="_GetProtectionPolicy_e__Struct", pack=False, align=None),
    "_GetQueueInfo_e__Struct": SimStruct(OrderedDict((("dwQueueID", SimTypeInt(signed=False, label="UInt32")), ("QueueInfo", SimTypeRef("LINEQUEUEINFO", SimStruct)),)), name="_GetQueueInfo_e__Struct", pack=False, align=None),
    "_GetQueueList_e__Struct": SimStruct(OrderedDict((("GroupID", SimTypeBottom(label="Guid")), ("QueueList", SimTypeRef("LINEQUEUELIST", SimStruct)),)), name="_GetQueueList_e__Struct", pack=False, align=None),
    "_GetVads_e__Struct": SimStruct(OrderedDict((("Anonymous", SimUnion({"GetNumVads": SimTypeRef("_GetNumVads_e__Struct", SimStruct), "GetVad": SimTypeRef("D3DKMT_VAD_DESC", SimStruct), "GetVadRange": SimTypeRef("D3DKMT_VA_RANGE_DESC", SimStruct), "GetGpuMmuCaps": SimTypeRef("D3DKMT_GET_GPUMMU_CAPS", SimStruct), "GetPte": SimTypeRef("D3DKMT_GET_PTE", SimStruct), "GetSegmentCaps": SimTypeRef("D3DKMT_GET_SEGMENT_CAPS", SimStruct)}, name="<anon>", label="None")), ("Command", SimTypeInt(signed=False, label="D3DKMT_VAD_ESCAPE_COMMAND")), ("Status", SimTypeInt(signed=True, label="Int32")),)), name="_GetVads_e__Struct", pack=False, align=None),
    "_GptPartInfo_e__Struct": SimStruct(OrderedDict((("partitionType", SimTypeBottom(label="Guid")),)), name="_GptPartInfo_e__Struct", pack=False, align=None),
    "_Gpt_e__Struct": SimStruct(OrderedDict((("DiskId", SimTypeBottom(label="Guid")),)), name="_Gpt_e__Struct", pack=False, align=None),
    "_GroupID_e__Struct": SimStruct(OrderedDict((("dwGroupID1", SimTypeInt(signed=False, label="UInt32")), ("dwGroupID2", SimTypeInt(signed=False, label="UInt32")), ("dwGroupID3", SimTypeInt(signed=False, label="UInt32")), ("dwGroupID4", SimTypeInt(signed=False, label="UInt32")),)), name="_GroupID_e__Struct", pack=False, align=None),
    "_GrowthRate_e__Struct": SimStruct(OrderedDict((("AbsoluteGrowthInContainers", SimTypeInt(signed=False, label="UInt32")), ("RelativeGrowthPercentage", SimTypeInt(signed=False, label="UInt32")),)), name="_GrowthRate_e__Struct", pack=False, align=None),
    "_Gsiv_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_Gsiv_e__Struct", pack=False, align=None),
    "_HCTMA_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_HCTMA_e__Struct", pack=False, align=None),
    "_HMAPPER": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "_Handles_e__Union": SimUnion({"hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "hCompletionPort": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"),
    "_HardwareInfo_e__Struct": SimStruct(OrderedDict((("ColorSensorId", SimTypeArray(SimTypeChar(label="Char"), 260)), ("InfraredSensorId", SimTypeArray(SimTypeChar(label="Char"), 260)), ("InfraredSensorRotationAngle", SimTypeInt(signed=False, label="UInt32")),)), name="_HardwareInfo_e__Struct", pack=False, align=None),
    "_HeaderArm64_e__Struct": SimStruct(OrderedDict((("_bitfield1", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_HeaderArm64_e__Struct", pack=False, align=None),
    "_HeaderX64_e__Struct": SimStruct(OrderedDict((("_bitfield1", SimTypeLongLong(signed=False, label="UInt64")), ("_bitfield2", SimTypeLongLong(signed=False, label="UInt64")),)), name="_HeaderX64_e__Struct", pack=False, align=None),
    "_Header_e__Union": SimUnion({"S": SimTypeRef("MPEG_HEADER_BITS_MIDL", SimStruct), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_HighWord_e__Union": SimUnion({"Bytes": SimTypeRef("_Bytes_e__Struct", SimStruct), "Bits": SimTypeRef("_Bits_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_I64Parts32_e__Struct": SimStruct(OrderedDict((("LowPart", SimTypeInt(signed=False, label="UInt32")), ("HighPart", SimTypeInt(signed=False, label="UInt32")),)), name="_I64Parts32_e__Struct", pack=False, align=None),
    "_ICS_e__Struct": SimStruct(OrderedDict((("remote", SimTypeRef("_remote_e__Struct", SimStruct)),)), name="_ICS_e__Struct", pack=False, align=None),
    "_ICivicAddressReportFactoryEvents": SimTypeBottom(label="_ICivicAddressReportFactoryEvents"),
    "_ICounterItemUnion": SimTypeBottom(label="_ICounterItemUnion"),
    "_IDENTIFY_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous1": SimStruct(OrderedDict((("NVMSETID", SimTypeShort(signed=False, label="UInt16")), ("Reserved", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous2_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_IDENTIFY_e__Struct", pack=False, align=None),
    "_IInkCollectorEvents": SimTypeBottom(label="_IInkCollectorEvents"),
    "_IInkEditEvents": SimTypeBottom(label="_IInkEditEvents"),
    "_IInkEvents": SimTypeBottom(label="_IInkEvents"),
    "_IInkOverlayEvents": SimTypeBottom(label="_IInkOverlayEvents"),
    "_IInkPictureEvents": SimTypeBottom(label="_IInkPictureEvents"),
    "_IInkRecognitionEvents": SimTypeBottom(label="_IInkRecognitionEvents"),
    "_IInkStrokesEvents": SimTypeBottom(label="_IInkStrokesEvents"),
    "_ILatLongReportFactoryEvents": SimTypeBottom(label="_ILatLongReportFactoryEvents"),
    "_IMPLICIT_HANDLE_INFO_e__Union": SimUnion({"pAutoHandle": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), "pPrimitiveHandle": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), "pGenericBindingInfo": SimTypePointer(SimTypeRef("GENERIC_BINDING_INFO", SimStruct), offset=0)}, name="<anon>", label="None"),
    "_IMSVidCtlEvents": SimTypeBottom(label="_IMSVidCtlEvents"),
    "_IManipulationEvents": SimTypeBottom(label="_IManipulationEvents"),
    "_IMathInputControlEvents": SimTypeBottom(label="_IMathInputControlEvents"),
    "_IOC_e__Struct": SimStruct(OrderedDict((("hIOPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("dwNumberOfBytesTransferred", SimTypeInt(signed=False, label="UInt32")), ("dwCompletionKey", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("lpOverlapped", SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)),)), name="_IOC_e__Struct", pack=False, align=None),
    "_IORING_OBJECT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "_IP_e__Union": SimUnion({"v4": SimTypeRef("IN_ADDR", SimStruct), "v6": SimTypeRef("IN6_ADDR", SimStruct), "Addr": SimTypeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"),
    "_IPenInputPanelEvents": SimTypeBottom(label="_IPenInputPanelEvents"),
    "_IPv4AH_e__Struct": SimStruct(OrderedDict((("Md5", SimTypeInt(signed=False, label="UInt32")), ("Sha_1", SimTypeInt(signed=False, label="UInt32")), ("Transport", SimTypeInt(signed=False, label="UInt32")), ("Tunnel", SimTypeInt(signed=False, label="UInt32")), ("Send", SimTypeInt(signed=False, label="UInt32")), ("Receive", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv4AH_e__Struct", pack=False, align=None),
    "_IPv4ESP_e__Struct": SimStruct(OrderedDict((("Des", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("TripleDes", SimTypeInt(signed=False, label="UInt32")), ("NullEsp", SimTypeInt(signed=False, label="UInt32")), ("Transport", SimTypeInt(signed=False, label="UInt32")), ("Tunnel", SimTypeInt(signed=False, label="UInt32")), ("Send", SimTypeInt(signed=False, label="UInt32")), ("Receive", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv4ESP_e__Struct", pack=False, align=None),
    "_IPv4Receive_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("IpOptionsSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpOptionsSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpChecksum", SimTypeInt(signed=False, label="UInt32")), ("UdpChecksum", SimTypeInt(signed=False, label="UInt32")), ("IpChecksum", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv4Receive_e__Struct", pack=False, align=None),
    "_IPv4Transmit_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("IpOptionsSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpOptionsSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpChecksum", SimTypeInt(signed=False, label="UInt32")), ("UdpChecksum", SimTypeInt(signed=False, label="UInt32")), ("IpChecksum", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv4Transmit_e__Struct", pack=False, align=None),
    "_IPv4_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("MaxOffLoadSize", SimTypeInt(signed=False, label="UInt32")), ("MinSegmentCount", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv4_e__Struct", pack=False, align=None),
    "_IPv6Receive_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("IpExtensionHeadersSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpOptionsSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpChecksum", SimTypeInt(signed=False, label="UInt32")), ("UdpChecksum", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv6Receive_e__Struct", pack=False, align=None),
    "_IPv6Transmit_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("IpExtensionHeadersSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpOptionsSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpChecksum", SimTypeInt(signed=False, label="UInt32")), ("UdpChecksum", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv6Transmit_e__Struct", pack=False, align=None),
    "_IPv6_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("MaxOffLoadSize", SimTypeInt(signed=False, label="UInt32")), ("MinSegmentCount", SimTypeInt(signed=False, label="UInt32")), ("IpExtensionHeadersSupported", SimTypeInt(signed=False, label="UInt32")), ("TcpOptionsSupported", SimTypeInt(signed=False, label="UInt32")),)), name="_IPv6_e__Struct", pack=False, align=None),
    "_IRDPSessionEvents": SimTypeBottom(label="_IRDPSessionEvents"),
    "_ISpPrivateEngineCall": SimTypeBottom(label="_ISpPrivateEngineCall"),
    "_ISpeechRecoContextEvents": SimTypeBottom(label="_ISpeechRecoContextEvents"),
    "_ISpeechVoiceEvents": SimTypeBottom(label="_ISpeechVoiceEvents"),
    "_ISystemMonitorUnion": SimTypeBottom(label="_ISystemMonitorUnion"),
    "_ITSWkspEvents": SimTypeBottom(label="_ITSWkspEvents"),
    "_IdentifyAndReleaseTicket_e__Struct": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactor", SimTypeChar(label="Byte")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")), ("Ticket", SimTypeLongLong(signed=False, label="UInt64")),)), name="_IdentifyAndReleaseTicket_e__Struct", pack=False, align=None),
    "_Identify_e__Struct": SimStruct(OrderedDict((("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactor", SimTypeChar(label="Byte")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")),)), name="_Identify_e__Struct", pack=False, align=None),
    "_Info_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Info_e__Struct", pack=False, align=None),
    "_Info_e__Union": SimUnion({"XpfMceDescriptor": SimTypeRef("WHEA_XPF_MCE_DESCRIPTOR", SimStruct), "XpfCmcDescriptor": SimTypeRef("WHEA_XPF_CMC_DESCRIPTOR", SimStruct), "XpfNmiDescriptor": SimTypeRef("WHEA_XPF_NMI_DESCRIPTOR", SimStruct), "IpfMcaDescriptor": SimTypeRef("WHEA_IPF_MCA_DESCRIPTOR", SimStruct), "IpfCmcDescriptor": SimTypeRef("WHEA_IPF_CMC_DESCRIPTOR", SimStruct), "IpfCpeDescriptor": SimTypeRef("WHEA_IPF_CPE_DESCRIPTOR", SimStruct), "AerRootportDescriptor": SimTypeRef("WHEA_AER_ROOTPORT_DESCRIPTOR", SimStruct), "AerEndpointDescriptor": SimTypeRef("WHEA_AER_ENDPOINT_DESCRIPTOR", SimStruct), "AerBridgeDescriptor": SimTypeRef("WHEA_AER_BRIDGE_DESCRIPTOR", SimStruct), "GenErrDescriptor": SimTypeRef("WHEA_GENERIC_ERROR_DESCRIPTOR", SimStruct), "GenErrDescriptorV2": SimTypeRef("WHEA_GENERIC_ERROR_DESCRIPTOR_V2", SimStruct), "DeviceDriverDescriptor": SimTypeRef("WHEA_DEVICE_DRIVER_DESCRIPTOR", SimStruct)}, name="<anon>", label="None"),
    "_IntPtr_e__Struct": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Msg", SimTypeInt(signed=False, label="UInt32")),)), name="_IntPtr_e__Struct", pack=False, align=None),
    "_InterfaceAndOperStatusFlags_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_InterfaceAndOperStatusFlags_e__Struct", pack=False, align=None),
    "_Interrupt_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_Interrupt_e__Struct", pack=False, align=None),
    "_Invalid_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Invalid_e__Struct", pack=False, align=None),
    "_Iocp_e__Struct": SimStruct(OrderedDict((("IocpHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("CompletionKey", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Overlapped", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Iocp_e__Struct", pack=False, align=None),
    "_Iris_e__Struct": SimStruct(OrderedDict((("EyeBoundingBox_1", SimTypeRef("RECT", SimStruct)), ("EyeBoundingBox_2", SimTypeRef("RECT", SimStruct)), ("PupilCenter_1", SimTypeRef("POINT", SimStruct)), ("PupilCenter_2", SimTypeRef("POINT", SimStruct)), ("Distance", SimTypeInt(signed=True, label="Int32")), ("GridPointCompletionPercent", SimTypeInt(signed=False, label="UInt32")), ("GridPointIndex", SimTypeShort(signed=False, label="UInt16")), ("Point3D", SimTypeRef("_Point3D_e__Struct", SimStruct)), ("StopCaptureAndShowCriticalFeedback", SimTypeInt(signed=True, label="Int32")),)), name="_Iris_e__Struct", pack=False, align=None),
    "_Isochronous_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Isochronous_e__Struct", pack=False, align=None),
    "_JobInfo_e__Union": SimUnion({"LogJobPrinted": SimTypeRef("BranchOfficeJobDataPrinted", SimStruct), "LogJobRendered": SimTypeRef("BranchOfficeJobDataRendered", SimStruct), "LogJobError": SimTypeRef("BranchOfficeJobDataError", SimStruct), "LogPipelineFailed": SimTypeRef("BranchOfficeJobDataPipelineFailed", SimStruct), "LogOfflineFileFull": SimTypeRef("BranchOfficeLogOfflineFileFull", SimStruct)}, name="<anon>", label="None"),
    "_KF_DEFINITION_FLAGS": SimTypeInt(signed=False, label="_KF_DEFINITION_FLAGS"),
    "_KF_REDIRECTION_CAPABILITIES": SimTypeInt(signed=False, label="_KF_REDIRECTION_CAPABILITIES"),
    "_KF_REDIRECT_FLAGS": SimTypeInt(signed=False, label="_KF_REDIRECT_FLAGS"),
    "_Kind_e__Union": SimUnion({"lID": SimTypeInt(signed=True, label="Int32"), "lpwstrName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_LCN_WEAK_REFERENCE_STATE": SimTypeInt(signed=False, label="_LCN_WEAK_REFERENCE_STATE"),
    "_LI_METRIC": SimTypeInt(signed=False, label="_LI_METRIC"),
    "_LPA_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_LPA_e__Struct", pack=False, align=None),
    "_LegacyFilter_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("AltitudeLength", SimTypeShort(signed=False, label="UInt16")), ("AltitudeBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("VolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("VolumeNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("FilterNameLength", SimTypeShort(signed=False, label="UInt16")), ("FilterNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("SupportedFeatures", SimTypeInt(signed=False, label="UInt32")),)), name="_LegacyFilter_e__Struct", pack=False, align=None),
    "_Linear_e__Struct": SimStruct(OrderedDict((("dwStartAlignment", SimTypeInt(signed=False, label="UInt32")), ("dwPitchAlignment", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_Linear_e__Struct", pack=False, align=None),
    "_ListBStr_e__Struct": SimStruct(OrderedDict((("cNumList", SimTypeInt(signed=True, label="Int32")), ("Nom", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pList", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="_ListBStr_e__Struct", pack=False, align=None),
    "_ListFloat_e__Struct": SimStruct(OrderedDict((("cNumList", SimTypeInt(signed=True, label="Int32")), ("Nom", SimTypeFloat(size=64)), ("pList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_ListFloat_e__Struct", pack=False, align=None),
    "_ListGuid_e__Struct": SimStruct(OrderedDict((("cNumList", SimTypeInt(signed=True, label="Int32")), ("Nom", SimTypeBottom(label="Guid")), ("pList", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)),)), name="_ListGuid_e__Struct", pack=False, align=None),
    "_ListViewMode": SimTypeInt(signed=False, label="_ListViewMode"),
    "_List_e__Struct": SimStruct(OrderedDict((("cNumList", SimTypeInt(signed=True, label="Int32")), ("Nom", SimTypeInt(signed=True, label="Int32")), ("pList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_List_e__Struct", pack=False, align=None),
    "_LnSz_e__Struct": SimStruct(OrderedDict((("Linenumber", SimTypeShort(signed=False, label="UInt16")), ("Size", SimTypeShort(signed=False, label="UInt16")),)), name="_LnSz_e__Struct", pack=False, align=None),
    "_LocalInterrupt_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_LocalInterrupt_e__Struct", pack=False, align=None),
    "_LockControl_e__Struct": SimStruct(OrderedDict((("Length", SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)), ("Key", SimTypeInt(signed=False, label="UInt32")), ("ByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("ProcessId", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("FailImmediately", SimTypeChar(label="Byte")), ("ExclusiveLock", SimTypeChar(label="Byte")),)), name="_LockControl_e__Struct", pack=False, align=None),
    "_LockObject_e__Struct": SimStruct(OrderedDict((("ObjectName", SimTypeArray(SimTypeChar(label="Char"), 128)), ("Timeout", SimTypeLongLong(signed=True, label="Int64")), ("Alertable", SimTypeInt(signed=True, label="Int32")),)), name="_LockObject_e__Struct", pack=False, align=None),
    "_LogTail_e__Struct": SimStruct(OrderedDict((("MinimumAvailablePercentage", SimTypeInt(signed=False, label="UInt32")), ("MinimumAvailableContainers", SimTypeInt(signed=False, label="UInt32")),)), name="_LogTail_e__Struct", pack=False, align=None),
    "_LogicalAddressLimits_e__Struct": SimStruct(OrderedDict((("MinimumAddress", SimTypeLongLong(signed=True, label="Int64")), ("MaximumAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="_LogicalAddressLimits_e__Struct", pack=False, align=None),
    "_MC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_MC_e__Struct", pack=False, align=None),
    "_MFP_CREDENTIAL_FLAGS": SimTypeInt(signed=False, label="_MFP_CREDENTIAL_FLAGS"),
    "_MFP_MEDIAITEM_CHARACTERISTICS": SimTypeInt(signed=False, label="_MFP_MEDIAITEM_CHARACTERISTICS"),
    "_MFT_INPUT_DATA_BUFFER_FLAGS": SimTypeInt(signed=False, label="_MFT_INPUT_DATA_BUFFER_FLAGS"),
    "_MFT_INPUT_STATUS_FLAGS": SimTypeInt(signed=False, label="_MFT_INPUT_STATUS_FLAGS"),
    "_MFT_INPUT_STREAM_INFO_FLAGS": SimTypeInt(signed=False, label="_MFT_INPUT_STREAM_INFO_FLAGS"),
    "_MFT_OUTPUT_DATA_BUFFER_FLAGS": SimTypeInt(signed=False, label="_MFT_OUTPUT_DATA_BUFFER_FLAGS"),
    "_MFT_OUTPUT_STATUS_FLAGS": SimTypeInt(signed=False, label="_MFT_OUTPUT_STATUS_FLAGS"),
    "_MFT_OUTPUT_STREAM_INFO_FLAGS": SimTypeInt(signed=False, label="_MFT_OUTPUT_STREAM_INFO_FLAGS"),
    "_MFT_PROCESS_OUTPUT_FLAGS": SimTypeInt(signed=False, label="_MFT_PROCESS_OUTPUT_FLAGS"),
    "_MFT_PROCESS_OUTPUT_STATUS": SimTypeInt(signed=False, label="_MFT_PROCESS_OUTPUT_STATUS"),
    "_MFT_SET_TYPE_FLAGS": SimTypeInt(signed=False, label="_MFT_SET_TYPE_FLAGS"),
    "_MP": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_MP", pack=False, align=None),
    "_MSP_ADDRESS_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="MSP_ADDRESS_EVENT")), ("pTerminal", SimTypeBottom(label="ITTerminal")),)), name="_MSP_ADDRESS_EVENT_INFO_e__Struct", pack=False, align=None),
    "_MSP_ASR_TERMINAL_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("pASRTerminal", SimTypeBottom(label="ITTerminal")), ("hrErrorCode", SimTypeInt(signed=True, label="Int32")),)), name="_MSP_ASR_TERMINAL_EVENT_INFO_e__Struct", pack=False, align=None),
    "_MSP_CALL_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="MSP_CALL_EVENT")), ("Cause", SimTypeInt(signed=False, label="MSP_CALL_EVENT_CAUSE")), ("pStream", SimTypeBottom(label="ITStream")), ("pTerminal", SimTypeBottom(label="ITTerminal")), ("hrError", SimTypeInt(signed=True, label="Int32")),)), name="_MSP_CALL_EVENT_INFO_e__Struct", pack=False, align=None),
    "_MSP_FILE_TERMINAL_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("pParentFileTerminal", SimTypeBottom(label="ITTerminal")), ("pFileTrack", SimTypeBottom(label="ITFileTrack")), ("TerminalMediaState", SimTypeInt(signed=False, label="TERMINAL_MEDIA_STATE")), ("ftecEventCause", SimTypeInt(signed=False, label="FT_STATE_EVENT_CAUSE")), ("hrErrorCode", SimTypeInt(signed=True, label="Int32")),)), name="_MSP_FILE_TERMINAL_EVENT_INFO_e__Struct", pack=False, align=None),
    "_MSP_PRIVATE_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("pEvent", SimTypeBottom(label="IDispatch")), ("lEventCode", SimTypeInt(signed=True, label="Int32")),)), name="_MSP_PRIVATE_EVENT_INFO_e__Struct", pack=False, align=None),
    "_MSP_TONE_TERMINAL_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("pToneTerminal", SimTypeBottom(label="ITTerminal")), ("hrErrorCode", SimTypeInt(signed=True, label="Int32")),)), name="_MSP_TONE_TERMINAL_EVENT_INFO_e__Struct", pack=False, align=None),
    "_MSP_TSP_DATA_e__Struct": SimStruct(OrderedDict((("dwBufferSize", SimTypeInt(signed=False, label="UInt32")), ("pBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_MSP_TSP_DATA_e__Struct", pack=False, align=None),
    "_MSP_TTS_TERMINAL_EVENT_INFO_e__Struct": SimStruct(OrderedDict((("pTTSTerminal", SimTypeBottom(label="ITTerminal")), ("hrErrorCode", SimTypeInt(signed=True, label="Int32")),)), name="_MSP_TTS_TERMINAL_EVENT_INFO_e__Struct", pack=False, align=None),
    "_ManufactureDate_e__Struct": SimStruct(OrderedDict((("Week", SimTypeInt(signed=False, label="UInt32")), ("Year", SimTypeInt(signed=False, label="UInt32")),)), name="_ManufactureDate_e__Struct", pack=False, align=None),
    "_MapProtect_e__Struct": SimStruct(OrderedDict((("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("AllocationOffsetInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Protection", SimTypeRef("D3DDDIGPUVIRTUALADDRESS_PROTECTION_TYPE", SimStruct)), ("DriverProtection", SimTypeLongLong(signed=False, label="UInt64")),)), name="_MapProtect_e__Struct", pack=False, align=None),
    "_Map_e__Struct": SimStruct(OrderedDict((("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("hAllocation", SimTypeInt(signed=False, label="UInt32")), ("AllocationOffsetInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("AllocationSizeInBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Map_e__Struct", pack=False, align=None),
    "_MaxFrameRate_e__Struct": SimStruct(OrderedDict((("Numerator", SimTypeInt(signed=False, label="UInt32")), ("Denominator", SimTypeInt(signed=False, label="UInt32")),)), name="_MaxFrameRate_e__Struct", pack=False, align=None),
    "_MaximumSize_e__Struct": SimStruct(OrderedDict((("Containers", SimTypeInt(signed=False, label="UInt32")),)), name="_MaximumSize_e__Struct", pack=False, align=None),
    "_MbrPartInfo_e__Struct": SimStruct(OrderedDict((("partitionType", SimTypeChar(label="Byte")),)), name="_MbrPartInfo_e__Struct", pack=False, align=None),
    "_Mbr_e__Struct": SimStruct(OrderedDict((("Signature", SimTypeInt(signed=False, label="UInt32")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")),)), name="_Mbr_e__Struct", pack=False, align=None),
    "_Mca_e__Struct": SimStruct(OrderedDict((("BankNumber", SimTypeChar(label="Byte")), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 7)), ("Status", SimUnion({"MciStats": SimTypeRef("_MciStats_e__Struct", SimStruct), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Address", SimUnion({"Anonymous": SimStruct(OrderedDict((("Address", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "QuadPart": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None")), ("Misc", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Mca_e__Struct", pack=False, align=None),
    "_Mce_e__Struct": SimStruct(OrderedDict((("Address", SimTypeLongLong(signed=False, label="UInt64")), ("Type", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Mce_e__Struct", pack=False, align=None),
    "_MciStats_e__Struct": SimStruct(OrderedDict((("McaCod", SimTypeShort(signed=False, label="UInt16")), ("MsCod", SimTypeShort(signed=False, label="UInt16")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_MciStats_e__Struct", pack=False, align=None),
    "_MdlReadComplete_e__Struct": SimStruct(OrderedDict((("MdlChain", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_MdlReadComplete_e__Struct", pack=False, align=None),
    "_MdlRead_e__Struct": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeInt(signed=False, label="UInt32")), ("MdlChain", SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0)),)), name="_MdlRead_e__Struct", pack=False, align=None),
    "_MdlWriteComplete_e__Struct": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("MdlChain", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_MdlWriteComplete_e__Struct", pack=False, align=None),
    "_Mdl_e__Struct": SimStruct(OrderedDict((("Mdl", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_Mdl_e__Struct", pack=False, align=None),
    "_Memory40_e__Struct": SimStruct(OrderedDict((("Length40", SimTypeInt(signed=False, label="UInt32")), ("Alignment40", SimTypeInt(signed=False, label="UInt32")), ("MinimumAddress", SimTypeLongLong(signed=True, label="Int64")), ("MaximumAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="_Memory40_e__Struct", pack=False, align=None),
    "_Memory48_e__Struct": SimStruct(OrderedDict((("Length48", SimTypeInt(signed=False, label="UInt32")), ("Alignment48", SimTypeInt(signed=False, label="UInt32")), ("MinimumAddress", SimTypeLongLong(signed=True, label="Int64")), ("MaximumAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="_Memory48_e__Struct", pack=False, align=None),
    "_Memory64_e__Struct": SimStruct(OrderedDict((("Length64", SimTypeInt(signed=False, label="UInt32")), ("Alignment64", SimTypeInt(signed=False, label="UInt32")), ("MinimumAddress", SimTypeLongLong(signed=True, label="Int64")), ("MaximumAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="_Memory64_e__Struct", pack=False, align=None),
    "_MemoryError_e__Struct": SimStruct(OrderedDict((("Address", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Consumed", SimTypeChar(label="Byte")), ("ErrorCode", SimTypeShort(signed=False, label="UInt16")), ("ErrorIpValid", SimTypeChar(label="Byte")), ("RestartIpValid", SimTypeChar(label="Byte")), ("ClearPoison", SimTypeChar(label="Byte")),)), name="_MemoryError_e__Struct", pack=False, align=None),
    "_Memory_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Alignment", SimTypeInt(signed=False, label="UInt32")), ("MinimumAddress", SimTypeLongLong(signed=True, label="Int64")), ("MaximumAddress", SimTypeLongLong(signed=True, label="Int64")),)), name="_Memory_e__Struct", pack=False, align=None),
    "_MenuEx_e__Struct": SimStruct(OrderedDict((("mexHeader", SimTypeRef("MENUEX_TEMPLATE_HEADER", SimStruct)), ("mexItem", SimTypePointer(SimTypeRef("MENUEX_TEMPLATE_ITEM", SimStruct), offset=0)),)), name="_MenuEx_e__Struct", pack=False, align=None),
    "_Menu_e__Struct": SimStruct(OrderedDict((("mitHeader", SimTypeRef("MENUITEMTEMPLATEHEADER", SimStruct)), ("miTemplate", SimTypePointer(SimTypeRef("MENUITEMTEMPLATE", SimStruct), offset=0)),)), name="_Menu_e__Struct", pack=False, align=None),
    "_MessageInterrupt_e__Struct": SimStruct(OrderedDict((("Anonymous", SimUnion({"Raw": SimTypeRef("_Raw_e__Struct", SimStruct), "Translated": SimTypeRef("_Translated_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="_MessageInterrupt_e__Struct", pack=False, align=None),
    "_Metrics_e__Union": SimUnion({"cSteps": SimTypeInt(signed=False, label="UInt32"), "cbCustomData": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)}, name="<anon>", label="None"),
    "_Mft2WritesUserLevel_e__Struct": SimStruct(OrderedDict((("Write", SimTypeInt(signed=False, label="UInt32")), ("Create", SimTypeInt(signed=False, label="UInt32")), ("SetInfo", SimTypeInt(signed=False, label="UInt32")), ("Flush", SimTypeInt(signed=False, label="UInt32")),)), name="_Mft2WritesUserLevel_e__Struct", pack=False, align=None),
    "_MftBitmapWritesUserLevel_e__Struct": SimStruct(OrderedDict((("Write", SimTypeInt(signed=False, label="UInt32")), ("Create", SimTypeInt(signed=False, label="UInt32")), ("SetInfo", SimTypeInt(signed=False, label="UInt32")), ("Flush", SimTypeInt(signed=False, label="UInt32")),)), name="_MftBitmapWritesUserLevel_e__Struct", pack=False, align=None),
    "_MftWritesUserLevel_e__Struct": SimStruct(OrderedDict((("Write", SimTypeInt(signed=False, label="UInt32")), ("Create", SimTypeInt(signed=False, label="UInt32")), ("SetInfo", SimTypeInt(signed=False, label="UInt32")), ("Flush", SimTypeInt(signed=False, label="UInt32")),)), name="_MftWritesUserLevel_e__Struct", pack=False, align=None),
    "_MiniFilter_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="UInt32")), ("FrameID", SimTypeInt(signed=False, label="UInt32")), ("VolumeFileSystemType", SimTypeInt(signed=False, label="FLT_FILESYSTEM_TYPE")), ("InstanceNameLength", SimTypeShort(signed=False, label="UInt16")), ("InstanceNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("AltitudeLength", SimTypeShort(signed=False, label="UInt16")), ("AltitudeBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("VolumeNameLength", SimTypeShort(signed=False, label="UInt16")), ("VolumeNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("FilterNameLength", SimTypeShort(signed=False, label="UInt16")), ("FilterNameBufferOffset", SimTypeShort(signed=False, label="UInt16")), ("SupportedFeatures", SimTypeInt(signed=False, label="UInt32")),)), name="_MiniFilter_e__Struct", pack=False, align=None),
    "_MinimumSize_e__Struct": SimStruct(OrderedDict((("Containers", SimTypeInt(signed=False, label="UInt32")),)), name="_MinimumSize_e__Struct", pack=False, align=None),
    "_MinorVersion_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("SubMinor", SimTypeChar(label="Byte")), ("Minor", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_Misc_e__Union": SimUnion({"LnSz": SimTypeRef("_LnSz_e__Struct", SimStruct), "TotalSize": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_MonitorPresence_e__Struct": SimStruct(OrderedDict((("ChangeType", SimTypeInt(signed=False, label="UInt32")), ("PresenceCount", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PresenceArray", SimTypePointer(SimTypeRef("WINBIO_PRESENCE", SimStruct), offset=0)),)), name="_MonitorPresence_e__Struct", pack=False, align=None),
    "_MonitoredFence_e__Struct": SimStruct(OrderedDict((("FenceValueCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FenceValueGPUVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")),)), name="_MonitoredFence_e__Struct", pack=False, align=None),
    "_MountPointReparseBuffer_e__Struct": SimStruct(OrderedDict((("SubstituteNameOffset", SimTypeShort(signed=False, label="UInt16")), ("SubstituteNameLength", SimTypeShort(signed=False, label="UInt16")), ("PrintNameOffset", SimTypeShort(signed=False, label="UInt16")), ("PrintNameLength", SimTypeShort(signed=False, label="UInt16")), ("PathBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_MountPointReparseBuffer_e__Struct", pack=False, align=None),
    "_MountVolume_e__Struct": SimStruct(OrderedDict((("DeviceType", SimTypeInt(signed=False, label="UInt32")),)), name="_MountVolume_e__Struct", pack=False, align=None),
    "_MultiSampleCaps_e__Struct": SimStruct(OrderedDict((("wFlipMSTypes", SimTypeShort(signed=False, label="UInt16")), ("wBltMSTypes", SimTypeShort(signed=False, label="UInt16")),)), name="_MultiSampleCaps_e__Struct", pack=False, align=None),
    "_Mutant_e__Struct": SimStruct(OrderedDict((("CurrentCount", SimTypeInt(signed=True, label="Int32")), ("Abandoned", SimTypeInt(signed=True, label="Int32")), ("OwnerProcessId", SimTypeInt(signed=False, label="UInt32")), ("OwnerThreadId", SimTypeInt(signed=False, label="UInt32")),)), name="_Mutant_e__Struct", pack=False, align=None),
    "_NCharVal": SimStruct(OrderedDict((("sActualLength", SimTypeShort(signed=True, label="Int16")), ("sMaxLength", SimTypeShort(signed=True, label="Int16")), ("pwchNCharVal", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 5)), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("pwchReserved", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_NCharVal", pack=False, align=None),
    "_NDR_PROC_CONTEXT": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "_NMCII_FLAGS": SimTypeInt(signed=False, label="_NMCII_FLAGS"),
    "_NMCSAEI_FLAGS": SimTypeInt(signed=False, label="_NMCSAEI_FLAGS"),
    "_NMIC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_NMIC_e__Struct", pack=False, align=None),
    "_NSATTR_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_NSATTR_e__Struct", pack=False, align=None),
    "_NSFEAT_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_NSFEAT_e__Struct", pack=False, align=None),
    "_NSTCECLICKTYPE": SimTypeInt(signed=False, label="_NSTCECLICKTYPE"),
    "_NSTCEHITTEST": SimTypeInt(signed=False, label="_NSTCEHITTEST"),
    "_NSTCITEMSTATE": SimTypeInt(signed=False, label="_NSTCITEMSTATE"),
    "_NSTCROOTSTYLE": SimTypeInt(signed=False, label="_NSTCROOTSTYLE"),
    "_NSTCSTYLE": SimTypeInt(signed=False, label="_NSTCSTYLE"),
    "_NT_D3DLINEPATTERN": SimStruct(OrderedDict((("wRepeatFactor", SimTypeShort(signed=False, label="UInt16")), ("wLinePattern", SimTypeShort(signed=False, label="UInt16")),)), name="_NT_D3DLINEPATTERN", pack=False, align=None),
    "_NVDIMM_N_e__Struct": SimStruct(OrderedDict((("CriticalHealth", SimTypeChar(label="Byte")), ("ModuleHealth", SimTypeArray(SimTypeChar(label="Byte"), 2)), ("ErrorThresholdStatus", SimTypeChar(label="Byte")),)), name="_NVDIMM_N_e__Struct", pack=False, align=None),
    "_NVSCC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_NVSCC_e__Struct", pack=False, align=None),
    "_NWPC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_NWPC_e__Struct", pack=False, align=None),
    "_N_e__Union": SimUnion({"ShortName": SimTypeArray(SimTypeChar(label="Byte"), 8), "Name": SimTypeRef("_Name_e__Struct", SimStruct), "LongName": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)}, name="<anon>", label="None"),
    "_Name_e__Struct": SimStruct(OrderedDict((("Short", SimTypeInt(signed=False, label="UInt32")), ("Long", SimTypeInt(signed=False, label="UInt32")),)), name="_Name_e__Struct", pack=False, align=None),
    "_NamedAddress_e__Struct": SimStruct(OrderedDict((("Address", SimTypeArray(SimTypeChar(label="Char"), 256)), ("Port", SimTypeArray(SimTypeChar(label="Char"), 6)),)), name="_NamedAddress_e__Struct", pack=False, align=None),
    "_Neither_e__Struct": SimStruct(OrderedDict((("OutputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("InputBufferLength", SimTypeInt(signed=False, label="UInt32")), ("FsControlCode", SimTypeInt(signed=False, label="UInt32")), ("InputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("OutputMdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_Neither_e__Struct", pack=False, align=None),
    "_NetworkQueryOpen_e__Struct": SimStruct(OrderedDict((("Irp", SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), ("NetworkInformation", SimTypePointer(SimTypeRef("FILE_NETWORK_OPEN_INFORMATION", SimStruct), offset=0)),)), name="_NetworkQueryOpen_e__Struct", pack=False, align=None),
    "_NewContainerExtension_e__Struct": SimStruct(OrderedDict((("ExtensionLengthInBytes", SimTypeShort(signed=False, label="UInt16")), ("ExtensionString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_NewContainerExtension_e__Struct", pack=False, align=None),
    "_NewContainerPrefix_e__Struct": SimStruct(OrderedDict((("PrefixLengthInBytes", SimTypeShort(signed=False, label="UInt16")), ("PrefixString", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_NewContainerPrefix_e__Struct", pack=False, align=None),
    "_NewContainerSize_e__Struct": SimStruct(OrderedDict((("SizeInBytes", SimTypeInt(signed=False, label="UInt32")),)), name="_NewContainerSize_e__Struct", pack=False, align=None),
    "_NewContainerSuffix_e__Struct": SimStruct(OrderedDict((("NextContainerSuffix", SimTypeLongLong(signed=False, label="UInt64")),)), name="_NewContainerSuffix_e__Struct", pack=False, align=None),
    "_Nmi_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_Nmi_e__Struct", pack=False, align=None),
    "_None_e__Struct": SimStruct(OrderedDict((("Dummy", SimTypeInt(signed=True, label="Int32")),)), name="_None_e__Struct", pack=False, align=None),
    "_NotRange_e__Struct": SimStruct(OrderedDict((("Usage", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("StringIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")), ("DesignatorIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeShort(signed=False, label="UInt16")), ("DataIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved4", SimTypeShort(signed=False, label="UInt16")),)), name="_NotRange_e__Struct", pack=False, align=None),
    "_Notification_e__Struct": SimStruct(OrderedDict((("NotificationMask", SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")),)), name="_Notification_e__Struct", pack=False, align=None),
    "_NotifyData_e__Union": SimUnion({"adwData": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2), "Data": SimTypeRef("_Data_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_NotifyDirectoryEx_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("CompletionFilter", SimTypeInt(signed=False, label="UInt32")), ("DirectoryNotifyInformationClass", SimTypeInt(signed=False, label="DIRECTORY_NOTIFY_INFORMATION_CLASS")), ("Spare2", SimTypeInt(signed=False, label="UInt32")), ("DirectoryBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_NotifyDirectoryEx_e__Struct", pack=False, align=None),
    "_NotifyDirectory_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("CompletionFilter", SimTypeInt(signed=False, label="UInt32")), ("Spare1", SimTypeInt(signed=False, label="UInt32")), ("Spare2", SimTypeInt(signed=False, label="UInt32")), ("DirectoryBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_NotifyDirectory_e__Struct", pack=False, align=None),
    "_NotifyUnitStatusChange_e__Struct": SimStruct(OrderedDict((("ExtendedStatus", SimTypeRef("WINBIO_EXTENDED_UNIT_STATUS", SimStruct)),)), name="_NotifyUnitStatusChange_e__Struct", pack=False, align=None),
    "_NumaNode_e__Struct": SimStruct(OrderedDict((("NodeNumber", SimTypeInt(signed=False, label="UInt32")),)), name="_NumaNode_e__Struct", pack=False, align=None),
    "_OACS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_OACS_e__Struct", pack=False, align=None),
    "_OAES_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_OAES_e__Struct", pack=False, align=None),
    "_ONCS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_ONCS_e__Struct", pack=False, align=None),
    "_OPPROGDLGF": SimTypeInt(signed=False, label="_OPPROGDLGF"),
    "_OZCS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_OZCS_e__Struct", pack=False, align=None),
    "_ObjectParam_e__Union": SimUnion({"ProviderAddressChange": SimTypeRef("WNV_PROVIDER_ADDRESS_CHANGE_PARAM", SimStruct), "CustomerAddressChange": SimTypeRef("WNV_CUSTOMER_ADDRESS_CHANGE_PARAM", SimStruct)}, name="<anon>", label="None"),
    "_Old_e__Union": SimUnion({"lVal": SimTypeInt(signed=True, label="Int32"), "fltVal": SimTypeFloat(size=32), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "guidVal": SimTypeBottom(label="Guid")}, name="<anon>", label="None"),
    "_OobMgmtSupport_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_OpaqueEngineData_e__Struct": SimStruct(OrderedDict((("AdapterId", SimTypeBottom(label="Guid")), ("Data", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 78)),)), name="_OpaqueEngineData_e__Struct", pack=False, align=None),
    "_OpenCompletion_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_OPEN_COMPLETION_FLAGS")),)), name="_OpenCompletion_e__Struct", pack=False, align=None),
    "_OpenSource_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_OpenSource_e__Struct", pack=False, align=None),
    "_Options_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Options_e__Struct", pack=False, align=None),
    "_OtherCpuInfo_e__Struct": SimStruct(OrderedDict((("ProcessorFeatures", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 2)),)), name="_OtherCpuInfo_e__Struct", pack=False, align=None),
    "_Others": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=True, label="Int64")),)), name="_Others", pack=False, align=None),
    "_Others_e__Struct": SimStruct(OrderedDict((("Argument1", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Argument2", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Argument3", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Argument4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Argument5", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Argument6", SimTypeLongLong(signed=True, label="Int64")),)), name="_Others_e__Struct", pack=False, align=None),
    "_Out_e__Struct": SimStruct(OrderedDict((("ReparseCount", SimTypeInt(signed=False, label="UInt32")), ("RemainingPathLength", SimTypeInt(signed=False, label="UInt32")),)), name="_Out_e__Struct", pack=False, align=None),
    "_Overlay_e__Struct": SimStruct(OrderedDict((("Anonymous1", SimUnion({"DeviceQueueEntry": SimTypeRef("KDEVICE_QUEUE_ENTRY", SimStruct), "Anonymous": SimStruct(OrderedDict((("DriverContext", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 4)),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("Thread", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("AuxiliaryBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("Anonymous2", SimStruct(OrderedDict((("ListEntry", SimTypeRef("LIST_ENTRY", SimStruct)), ("Anonymous", SimUnion({"CurrentStackLocation": SimTypePointer(SimTypeRef("IO_STACK_LOCATION", SimStruct), offset=0), "PacketType": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_Anonymous2_e__Struct", pack=False, align=None)), ("OriginalFileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)),)), name="_Overlay_e__Struct", pack=False, align=None),
    "_Overlay_e__Union": SimUnion({"AsynchronousParameters": SimTypeRef("_AsynchronousParameters_e__Struct", SimStruct), "AllocationSize": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"),
    "_PACKAGE_INFO_REFERENCE": SimStruct(OrderedDict((("reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_PACKAGE_INFO_REFERENCE", pack=False, align=None),
    "_PCI_AGP_COMMAND": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_PCI_AGP_COMMAND", pack=False, align=None),
    "_PCI_AGP_STATUS": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_PCI_AGP_STATUS", pack=False, align=None),
    "_PCI_HEADER_TYPE_0": SimStruct(OrderedDict((("BaseAddresses", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 6)), ("CIS", SimTypeInt(signed=False, label="UInt32")), ("SubVendorID", SimTypeShort(signed=False, label="UInt16")), ("SubSystemID", SimTypeShort(signed=False, label="UInt16")), ("ROMBaseAddress", SimTypeInt(signed=False, label="UInt32")), ("CapabilitiesPtr", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("InterruptLine", SimTypeChar(label="Byte")), ("InterruptPin", SimTypeChar(label="Byte")), ("MinimumGrant", SimTypeChar(label="Byte")), ("MaximumLatency", SimTypeChar(label="Byte")),)), name="_PCI_HEADER_TYPE_0", pack=False, align=None),
    "_PCI_HEADER_TYPE_1": SimStruct(OrderedDict((("BaseAddresses", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 2)), ("PrimaryBus", SimTypeChar(label="Byte")), ("SecondaryBus", SimTypeChar(label="Byte")), ("SubordinateBus", SimTypeChar(label="Byte")), ("SecondaryLatency", SimTypeChar(label="Byte")), ("IOBase", SimTypeChar(label="Byte")), ("IOLimit", SimTypeChar(label="Byte")), ("SecondaryStatus", SimTypeShort(signed=False, label="UInt16")), ("MemoryBase", SimTypeShort(signed=False, label="UInt16")), ("MemoryLimit", SimTypeShort(signed=False, label="UInt16")), ("PrefetchBase", SimTypeShort(signed=False, label="UInt16")), ("PrefetchLimit", SimTypeShort(signed=False, label="UInt16")), ("PrefetchBaseUpper32", SimTypeInt(signed=False, label="UInt32")), ("PrefetchLimitUpper32", SimTypeInt(signed=False, label="UInt32")), ("IOBaseUpper16", SimTypeShort(signed=False, label="UInt16")), ("IOLimitUpper16", SimTypeShort(signed=False, label="UInt16")), ("CapabilitiesPtr", SimTypeChar(label="Byte")), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("ROMBaseAddress", SimTypeInt(signed=False, label="UInt32")), ("InterruptLine", SimTypeChar(label="Byte")), ("InterruptPin", SimTypeChar(label="Byte")), ("BridgeControl", SimTypeShort(signed=False, label="UInt16")),)), name="_PCI_HEADER_TYPE_1", pack=False, align=None),
    "_PCI_HEADER_TYPE_2": SimStruct(OrderedDict((("SocketRegistersBaseAddress", SimTypeInt(signed=False, label="UInt32")), ("CapabilitiesPtr", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("SecondaryStatus", SimTypeShort(signed=False, label="UInt16")), ("PrimaryBus", SimTypeChar(label="Byte")), ("SecondaryBus", SimTypeChar(label="Byte")), ("SubordinateBus", SimTypeChar(label="Byte")), ("SecondaryLatency", SimTypeChar(label="Byte")), ("Range", SimTypeArray(SimStruct(OrderedDict((("Base", SimTypeInt(signed=False, label="UInt32")), ("Limit", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), 4)), ("InterruptLine", SimTypeChar(label="Byte")), ("InterruptPin", SimTypeChar(label="Byte")), ("BridgeControl", SimTypeShort(signed=False, label="UInt16")),)), name="_PCI_HEADER_TYPE_2", pack=False, align=None),
    "_PDMODE": SimTypeInt(signed=False, label="_PDMODE"),
    "_PERSIST_SPROPSTORE_FLAGS": SimTypeInt(signed=False, label="_PERSIST_SPROPSTORE_FLAGS"),
    "_PMCSR_BSE_e__Union": SimUnion({"BridgeSupport": SimTypeRef("PCI_PMCSR_BSE", SimStruct), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_PMCSR_e__Union": SimUnion({"ControlStatus": SimTypeRef("PCI_PMCSR", SimStruct), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_PMC_e__Union": SimUnion({"Capabilities": SimTypeRef("PCI_PMC", SimStruct), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_PM_SUPPORT": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_PM_SUPPORT", pack=False, align=None),
    "_PR_IN_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")), ("AllocationLength", SimTypeShort(signed=False, label="UInt16")),)), name="_PR_IN_e__Struct", pack=False, align=None),
    "_PR_OUT_e__Struct": SimStruct(OrderedDict((("_bitfield1", SimTypeChar(label="Byte")), ("_bitfield2", SimTypeChar(label="Byte")), ("ParameterList", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_PR_OUT_e__Struct", pack=False, align=None),
    "_ParameterData_e__Union": SimUnion({"IntegerData": SimTypeInt(signed=False, label="UInt32"), "StringData": SimTypeRef("UNICODE_STRING", SimStruct), "BinaryData": SimTypeRef("BINARY_DATA", SimStruct)}, name="<anon>", label="None"),
    "_ParameterErrorLocation_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_ParameterErrorLocation_e__Struct", pack=False, align=None),
    "_Parameters_e__Union": SimUnion({"WindowMessage": SimTypeRef("_WindowMessage_e__Struct", SimStruct), "Event": SimTypeRef("_Event_e__Struct", SimStruct), "Apc": SimTypeRef("_Apc_e__Struct", SimStruct), "Port": SimTypeRef("_Port_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_ParentLocation_e__Struct": SimStruct(OrderedDict((("ParentResolved", SimTypeInt(signed=True, label="Int32")), ("ParentLocationBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_ParentLocation_e__Struct", pack=False, align=None),
    "_ParentLocator_e__Struct": SimStruct(OrderedDict((("LinkageId", SimTypeBottom(label="Guid")), ("ParentFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_ParentLocator_e__Struct", pack=False, align=None),
    "_ParentPathWithDepthInfo_e__Struct": SimStruct(OrderedDict((("ChildDepth", SimTypeInt(signed=False, label="UInt32")), ("ParentFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_ParentPathWithDepthInfo_e__Struct", pack=False, align=None),
    "_Pci_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Pci_e__Struct", pack=False, align=None),
    "_PeriodicMonitoredFence_e__Struct": SimStruct(OrderedDict((("hAdapter", SimTypeInt(signed=False, label="UInt32")), ("VidPnTargetId", SimTypeInt(signed=False, label="UInt32")), ("Time", SimTypeLongLong(signed=False, label="UInt64")), ("FenceValueCPUVirtualAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FenceValueGPUVirtualAddress", SimTypeLongLong(signed=False, label="UInt64")), ("EngineAffinity", SimTypeInt(signed=False, label="UInt32")), ("Padding", SimTypeInt(signed=False, label="UInt32")),)), name="_PeriodicMonitoredFence_e__Struct", pack=False, align=None),
    "_PfnArray_e__Struct": SimStruct(OrderedDict((("PageFrame", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("NumberOfPages", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_PfnArray_e__Struct", pack=False, align=None),
    "_PhySpecificAttributes_e__Union": SimUnion({"HRDSSSAttributes": SimTypeRef("DOT11_HRDSSS_PHY_ATTRIBUTES", SimStruct), "OFDMAttributes": SimTypeRef("DOT11_OFDM_PHY_ATTRIBUTES", SimStruct), "ERPAttributes": SimTypeRef("DOT11_ERP_PHY_ATTRIBUTES", SimStruct)}, name="<anon>", label="None"),
    "_PhysicalDisk_e__Struct": SimStruct(OrderedDict((("LogicalSectorSize", SimTypeInt(signed=False, label="UInt32")), ("PhysicalSectorSize", SimTypeInt(signed=False, label="UInt32")), ("IsRemote", SimTypeInt(signed=True, label="Int32")),)), name="_PhysicalDisk_e__Struct", pack=False, align=None),
    "_PmemError_e__Struct": SimStruct(OrderedDict((("PmemErrInfo", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_PmemError_e__Struct", pack=False, align=None),
    "_Pnp_e__Union": SimUnion({"StartDevice": SimTypeRef("_StartDevice_e__Struct", SimStruct), "QueryDeviceRelations": SimTypeRef("_QueryDeviceRelations_e__Struct", SimStruct), "QueryInterface": SimTypeRef("_QueryInterface_e__Struct", SimStruct), "DeviceCapabilities": SimTypeRef("_DeviceCapabilities_e__Struct", SimStruct), "FilterResourceRequirements": SimTypeRef("_FilterResourceRequirements_e__Struct", SimStruct), "ReadWriteConfig": SimTypeRef("_ReadWriteConfig_e__Struct", SimStruct), "SetLock": SimTypeRef("_SetLock_e__Struct", SimStruct), "QueryId": SimTypeRef("_QueryId_e__Struct", SimStruct), "QueryDeviceText": SimTypeRef("_QueryDeviceText_e__Struct", SimStruct), "UsageNotification": SimTypeRef("_UsageNotification_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Point3D_e__Struct": SimStruct(OrderedDict((("X", SimTypeFloat(size=64)), ("Y", SimTypeFloat(size=64)), ("Z", SimTypeFloat(size=64)),)), name="_Point3D_e__Struct", pack=False, align=None),
    "_PolicyParameters_e__Union": SimUnion({"MaximumSize": SimTypeRef("_MaximumSize_e__Struct", SimStruct), "MinimumSize": SimTypeRef("_MinimumSize_e__Struct", SimStruct), "NewContainerSize": SimTypeRef("_NewContainerSize_e__Struct", SimStruct), "GrowthRate": SimTypeRef("_GrowthRate_e__Struct", SimStruct), "LogTail": SimTypeRef("_LogTail_e__Struct", SimStruct), "AutoShrink": SimTypeRef("_AutoShrink_e__Struct", SimStruct), "AutoGrow": SimTypeRef("_AutoGrow_e__Struct", SimStruct), "NewContainerPrefix": SimTypeRef("_NewContainerPrefix_e__Struct", SimStruct), "NewContainerSuffix": SimTypeRef("_NewContainerSuffix_e__Struct", SimStruct), "NewContainerExtension": SimTypeRef("_NewContainerExtension_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_PollContinue_e__Struct": SimStruct(OrderedDict((("dwReserved", SimTypeInt(signed=False, label="UInt32")),)), name="_PollContinue_e__Struct", pack=False, align=None),
    "_Polled_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")),)), name="_Polled_e__Struct", pack=False, align=None),
    "_Port_e__Struct": SimStruct(OrderedDict((("lpOverlapped", SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)), ("hPort", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Key", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Port_e__Struct", pack=False, align=None),
    "_PostCreate_e__Struct": SimStruct(OrderedDict((("NotificationMask", SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")),)), name="_PostCreate_e__Struct", pack=False, align=None),
    "_PowerFlags_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_PowerFlags_e__Struct", pack=False, align=None),
    "_PowerSequence_e__Struct": SimStruct(OrderedDict((("PowerSequence", SimTypePointer(SimTypeRef("POWER_SEQUENCE", SimStruct), offset=0)),)), name="_PowerSequence_e__Struct", pack=False, align=None),
    "_Power_e__Struct": SimStruct(OrderedDict((("Anonymous", SimUnion({"SystemContext": SimTypeInt(signed=False, label="UInt32"), "SystemPowerStateContext": SimTypeRef("SYSTEM_POWER_STATE_CONTEXT", SimStruct)}, name="<anon>", label="None")), ("Type", SimTypeInt(signed=False, label="POWER_STATE_TYPE")), ("State", SimUnion({"SystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DeviceState": SimTypeInt(signed=False, label="DEVICE_POWER_STATE")}, name="<anon>", label="None")), ("ShutdownType", SimTypeInt(signed=False, label="POWER_ACTION")),)), name="_Power_e__Struct", pack=False, align=None),
    "_PrepareMdlWrite_e__Struct": SimStruct(OrderedDict((("FileOffset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeInt(signed=False, label="UInt32")), ("MdlChain", SimTypePointer(SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0), offset=0)),)), name="_PrepareMdlWrite_e__Struct", pack=False, align=None),
    "_Priorities_e__Struct": SimStruct(OrderedDict((("PriorityLevelCount", SimTypeChar(label="Byte")), ("MaxPriorityBehavior", SimTypeChar(label="Byte")), ("OptimalWriteGranularity", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")), ("DirtyThresholdLow", SimTypeInt(signed=False, label="UInt32")), ("DirtyThresholdHigh", SimTypeInt(signed=False, label="UInt32")), ("SupportedCommands", SimTypeRef("_SupportedCommands_e__Struct", SimStruct)), ("Priority", SimTypePointer(SimTypeRef("NVCACHE_PRIORITY_LEVEL_DESCRIPTOR", SimStruct), offset=0)),)), name="_Priorities_e__Struct", pack=False, align=None),
    "_Privileges_e__Union": SimUnion({"InitialPrivilegeSet": SimTypeRef("INITIAL_PRIVILEGE_SET", SimStruct), "PrivilegeSet": SimTypeRef("PRIVILEGE_SET", SimStruct)}, name="<anon>", label="None"),
    "_Process_e__Struct": SimStruct(OrderedDict((("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("PebBaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ParentProcessId", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Process_e__Struct", pack=False, align=None),
    "_ProcessorCore_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeChar(label="Byte")),)), name="_ProcessorCore_e__Struct", pack=False, align=None),
    "_ProtocolSpecific_e__Union": SimUnion({"Smb2": SimTypeRef("_Smb2_e__Struct", SimStruct), "Reserved": SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 16)}, name="<anon>", label="None"),
    "_ProviderId_e__Struct": SimStruct(OrderedDict((("DeviceId", SimTypeBottom(label="Guid")), ("ClassId", SimTypeBottom(label="Guid")), ("InstanceId", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_ProviderId_e__Struct", pack=False, align=None),
    "_QueryDeviceRelations_e__Struct": SimStruct(OrderedDict((("Type", SimTypeInt(signed=False, label="DEVICE_RELATION_TYPE")),)), name="_QueryDeviceRelations_e__Struct", pack=False, align=None),
    "_QueryDeviceText_e__Struct": SimStruct(OrderedDict((("DeviceTextType", SimTypeInt(signed=False, label="DEVICE_TEXT_TYPE")), ("LocaleId", SimTypeInt(signed=False, label="UInt32")),)), name="_QueryDeviceText_e__Struct", pack=False, align=None),
    "_QueryDirectory_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FileName", SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0)), ("FileInformationClass", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")), ("FileIndex", SimTypeInt(signed=False, label="UInt32")), ("DirectoryBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_QueryDirectory_e__Struct", pack=False, align=None),
    "_QueryEa_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("EaList", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("EaListLength", SimTypeInt(signed=False, label="UInt32")), ("EaIndex", SimTypeInt(signed=False, label="UInt32")), ("EaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_QueryEa_e__Struct", pack=False, align=None),
    "_QueryFileInformation_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FileInformationClass", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")), ("InfoBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_QueryFileInformation_e__Struct", pack=False, align=None),
    "_QueryFile_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FileInformationClass", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")),)), name="_QueryFile_e__Struct", pack=False, align=None),
    "_QueryId_e__Struct": SimStruct(OrderedDict((("IdType", SimTypeInt(signed=False, label="BUS_QUERY_ID_TYPE")),)), name="_QueryId_e__Struct", pack=False, align=None),
    "_QueryInterface_e__Struct": SimStruct(OrderedDict((("InterfaceType", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("Size", SimTypeShort(signed=False, label="UInt16")), ("Version", SimTypeShort(signed=False, label="UInt16")), ("Interface", SimTypePointer(SimTypeRef("INTERFACE", SimStruct), offset=0)), ("InterfaceSpecificData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_QueryInterface_e__Struct", pack=False, align=None),
    "_QueryOpen_e__Struct": SimStruct(OrderedDict((("Irp", SimTypePointer(SimTypeRef("IRP", SimStruct), offset=0)), ("FileInformation", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Length", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)), ("FileInformationClass", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")),)), name="_QueryOpen_e__Struct", pack=False, align=None),
    "_QueryQuota_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("StartSid", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("SidList", SimTypePointer(SimTypeRef("FILE_GET_QUOTA_INFORMATION", SimStruct), offset=0)), ("SidListLength", SimTypeInt(signed=False, label="UInt32")), ("QuotaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_QueryQuota_e__Struct", pack=False, align=None),
    "_QuerySecurity_e__Struct": SimStruct(OrderedDict((("SecurityInformation", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")), ("SecurityBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_QuerySecurity_e__Struct", pack=False, align=None),
    "_QueryVolumeInformation_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FsInformationClass", SimTypeInt(signed=False, label="FS_INFORMATION_CLASS")), ("VolumeBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_QueryVolumeInformation_e__Struct", pack=False, align=None),
    "_QueryVolume_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FsInformationClass", SimTypeInt(signed=False, label="FS_INFORMATION_CLASS")),)), name="_QueryVolume_e__Struct", pack=False, align=None),
    "_Query_e__Struct": SimStruct(OrderedDict((("DriveMap", SimTypeInt(signed=False, label="UInt32")), ("DriveType", SimTypeArray(SimTypeChar(label="Byte"), 32)),)), name="_Query_e__Struct", pack=False, align=None),
    "_Queue_e__Union": SimUnion({"ListEntry": SimTypeRef("LIST_ENTRY", SimStruct), "Wcb": SimTypeRef("WAIT_CONTEXT_BLOCK", SimStruct)}, name="<anon>", label="None"),
    "_RCSTS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_RCSTS_e__Struct", pack=False, align=None),
    "_READWRITE_e__Struct": SimStruct(OrderedDict((("LBALOW", SimTypeInt(signed=False, label="UInt32")), ("LBAHIGH", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW13", SimUnion({"Anonymous": SimStruct(OrderedDict((("DSM", SimTypeRef("_DSM_e__Struct", SimStruct)), ("Reserved", SimTypeChar(label="Byte")), ("DSPEC", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_READWRITE_e__Struct", pack=False, align=None),
    "_REFS_STREAM_EXTENT_PROPERTIES": SimTypeInt(signed=False, label="_REFS_STREAM_EXTENT_PROPERTIES"),
    "_REM_FILTER_FLAGS": SimTypeInt(signed=False, label="_REM_FILTER_FLAGS"),
    "_RESERVATIONACQUIRE_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_RESERVATIONACQUIRE_e__Struct", pack=False, align=None),
    "_RESERVATIONREGISTER_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_RESERVATIONREGISTER_e__Struct", pack=False, align=None),
    "_RESERVATIONRELEASE_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeInt(signed=False, label="UInt32")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_RESERVATIONRELEASE_e__Struct", pack=False, align=None),
    "_RESERVATIONREPORT_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("NUMD", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_RESERVATIONREPORT_e__Struct", pack=False, align=None),
    "_RPMBS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_RPMBS_e__Struct", pack=False, align=None),
    "_RRLS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_RRLS_e__Struct", pack=False, align=None),
    "_RangeFloat_e__Struct": SimStruct(OrderedDict((("Min", SimTypeFloat(size=64)), ("Nom", SimTypeFloat(size=64)), ("Max", SimTypeFloat(size=64)), ("Inc", SimTypeFloat(size=64)),)), name="_RangeFloat_e__Struct", pack=False, align=None),
    "_Range_e__Struct": SimStruct(OrderedDict((("Min", SimTypeInt(signed=True, label="Int32")), ("Nom", SimTypeInt(signed=True, label="Int32")), ("Max", SimTypeInt(signed=True, label="Int32")), ("Inc", SimTypeInt(signed=True, label="Int32")),)), name="_Range_e__Struct", pack=False, align=None),
    "_RawAVC_e__Struct": SimStruct(OrderedDict((("PayloadSize", SimTypeInt(signed=False, label="UInt32")), ("Payload", SimTypeArray(SimTypeChar(label="Byte"), 512)),)), name="_RawAVC_e__Struct", pack=False, align=None),
    "_RawBytes_e__Union": SimUnion({"ScsiSenseKey": SimTypeRef("_ScsiSenseKey_e__Struct", SimStruct), "NVDIMM_N": SimTypeRef("_NVDIMM_N_e__Struct", SimStruct), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_Raw_e__Struct": SimStruct(OrderedDict((("Reserved", SimTypeShort(signed=False, label="UInt16")), ("MessageCount", SimTypeShort(signed=False, label="UInt16")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("Affinity", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Raw_e__Struct", pack=False, align=None),
    "_ReachabilityTime_e__Union": SimUnion({"LastReachable": SimTypeInt(signed=False, label="UInt32"), "LastUnreachable": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_ReadWriteConfig_e__Struct": SimStruct(OrderedDict((("WhichSpace", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Offset", SimTypeInt(signed=False, label="UInt32")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="_ReadWriteConfig_e__Struct", pack=False, align=None),
    "_Read_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeInt(signed=False, label="UInt32")), ("ByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("ReadBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_Read_e__Struct", pack=False, align=None),
    "_ReaderAndContainerParameter_e__Struct": SimStruct(OrderedDict((("cbReaderNameOffset", SimTypeInt(signed=False, label="UInt32")), ("cchReaderNameLength", SimTypeInt(signed=False, label="UInt32")), ("cbContainerNameOffset", SimTypeInt(signed=False, label="UInt32")), ("cchContainerNameLength", SimTypeInt(signed=False, label="UInt32")), ("dwDesiredCardModuleVersion", SimTypeInt(signed=False, label="UInt32")), ("dwCspFlags", SimTypeInt(signed=False, label="UInt32")),)), name="_ReaderAndContainerParameter_e__Struct", pack=False, align=None),
    "_Reason_e__Union": SimUnion({"Detailed": SimTypeRef("_Detailed_e__Struct", SimStruct), "SimpleReasonString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None"),
    "_Receive_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Receive_e__Struct", pack=False, align=None),
    "_Rectangular_e__Struct": SimStruct(OrderedDict((("dwXAlignment", SimTypeInt(signed=False, label="UInt32")), ("dwYAlignment", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwReserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_Rectangular_e__Struct", pack=False, align=None),
    "_Reg128_e__Struct": SimStruct(OrderedDict((("Low64", SimTypeLongLong(signed=False, label="UInt64")), ("High64", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Reg128_e__Struct", pack=False, align=None),
    "_Region_e__Struct": SimStruct(OrderedDict((("dwCommittedSize", SimTypeInt(signed=False, label="UInt32")), ("dwUnCommittedSize", SimTypeInt(signed=False, label="UInt32")), ("lpFirstBlock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("lpLastBlock", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Region_e__Struct", pack=False, align=None),
    "_ReleaseForModifiedPageWriter_e__Struct": SimStruct(OrderedDict((("ResourceToRelease", SimTypePointer(SimTypeRef("ERESOURCE", SimStruct), offset=0)),)), name="_ReleaseForModifiedPageWriter_e__Struct", pack=False, align=None),
    "_RemovableDiskInfo_e__Struct": SimStruct(OrderedDict((("Cylinders", SimTypeLongLong(signed=True, label="Int64")), ("MediaType", SimTypeInt(signed=False, label="STORAGE_MEDIA_TYPE")), ("TracksPerCylinder", SimTypeInt(signed=False, label="UInt32")), ("SectorsPerTrack", SimTypeInt(signed=False, label="UInt32")), ("BytesPerSector", SimTypeInt(signed=False, label="UInt32")), ("NumberMediaSides", SimTypeInt(signed=False, label="UInt32")), ("MediaCharacteristics", SimTypeInt(signed=False, label="UInt32")),)), name="_RemovableDiskInfo_e__Struct", pack=False, align=None),
    "_RenameCompletion_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_RENAME_COMPLETION_FLAGS")), ("SourcePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_RenameCompletion_e__Struct", pack=False, align=None),
    "_Rename_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_RENAME_FLAGS")), ("TargetPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Rename_e__Struct", pack=False, align=None),
    "_Reparse": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeShort(signed=False, label="UInt16")), ("ReparseDataSize", SimTypeInt(signed=False, label="UInt32")), ("ReparseDataOffset", SimTypeInt(signed=False, label="UInt32")),)), name="_Reparse", pack=False, align=None),
    "_ReprogramInterrupt_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_ReprogramInterrupt_e__Struct", pack=False, align=None),
    "_RequestType_e__Union": SimUnion({"ExeInfo": SimTypeRef("AMSI_UAC_REQUEST_EXE_INFO", SimStruct), "ComInfo": SimTypeRef("AMSI_UAC_REQUEST_COM_INFO", SimStruct), "MsiInfo": SimTypeRef("AMSI_UAC_REQUEST_MSI_INFO", SimStruct), "ActiveXInfo": SimTypeRef("AMSI_UAC_REQUEST_AX_INFO", SimStruct), "PackagedAppInfo": SimTypeRef("AMSI_UAC_REQUEST_PACKAGED_APP_INFO", SimStruct)}, name="<anon>", label="None"),
    "_Reserved_e__Struct": SimStruct(OrderedDict((("Reserved", SimTypeArray(SimTypeLongLong(signed=False, label="UInt64"), 8)),)), name="_Reserved_e__Struct", pack=False, align=None),
    "_Resolution_e__Struct": SimStruct(OrderedDict((("X", SimTypeInt(signed=False, label="UInt32")), ("Y", SimTypeInt(signed=False, label="UInt32")),)), name="_Resolution_e__Struct", pack=False, align=None),
    "_RestartHydration_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_RESTART_HYDRATION_FLAGS")), ("FsMetadata", SimTypePointer(SimTypeRef("CF_FS_METADATA", SimStruct), offset=0)), ("FileIdentity", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("FileIdentityLength", SimTypeInt(signed=False, label="UInt32")),)), name="_RestartHydration_e__Struct", pack=False, align=None),
    "_ResumeProcess_e__Struct": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_ResumeProcess_e__Struct", pack=False, align=None),
    "_RetrieveData_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_RETRIEVE_DATA_FLAGS")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")), ("ReturnedLength", SimTypeLongLong(signed=True, label="Int64")),)), name="_RetrieveData_e__Struct", pack=False, align=None),
    "_Revision_e__Union": SimUnion({"Info": SimTypeArray(SimTypeChar(label="Byte"), 8), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "_SANICAP_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_SANICAP_e__Struct", pack=False, align=None),
    "_SANITIZE_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"Anonymous": SimStruct(OrderedDict((("OVRPAT", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_SANITIZE_e__Struct", pack=False, align=None),
    "_SCSI_REQUEST_BLOCK": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "_SECURITYRECEIVE_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeRef("NVME_CDW11_SECURITY_RECEIVE", SimStruct)), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_SECURITYRECEIVE_e__Struct", pack=False, align=None),
    "_SECURITYSEND_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimTypeRef("NVME_CDW11_SECURITY_SEND", SimStruct)), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_SECURITYSEND_e__Struct", pack=False, align=None),
    "_SETFEATURES_e__Struct": SimStruct(OrderedDict((("CDW10", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW11", SimUnion({"NumberOfQueues": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "InterruptCoalescing": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "InterruptVectorConfig": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "LbaRangeType": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "Arbitration": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "VolatileWriteCache": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsyncEventConfig": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PowerManagement": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AutoPowerStateTransition": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "TemperatureThreshold": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "ErrorRecovery": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "WriteAtomicityNormal": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "NonOperationalPowerState": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "IoCommandSetProfile": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "ErrorInjection": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "HostIdentifier": SimTypeRef("NVME_CDW11_FEATURE_HOST_IDENTIFIER", SimStruct), "ReservationPersistence": SimTypeRef("NVME_CDW11_FEATURE_RESERVATION_PERSISTENCE", SimStruct), "ReservationNotificationMask": SimTypeRef("NVME_CDW11_FEATURE_RESERVATION_NOTIFICATION_MASK", SimStruct), "GetHostMetadata": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "SetHostMetadata": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW12", SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("HSIZE", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW13", SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW14", SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("HMDLUA", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW15", SimUnion({"HostMemoryBuffer": SimUnion({"Anonymous": SimStruct(OrderedDict((("HMDLEC", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_SETFEATURES_e__Struct", pack=False, align=None),
    "_SGLS_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_SGLS_e__Struct", pack=False, align=None),
    "_SHCONTF": SimTypeInt(signed=False, label="_SHCONTF"),
    "_SICHINTF": SimTypeInt(signed=False, label="_SICHINTF"),
    "_SPBEGINF": SimTypeInt(signed=False, label="_SPBEGINF"),
    "_SPINITF": SimTypeInt(signed=False, label="_SPINITF"),
    "_SPLCLIENT_INFO_2_V3": SimStruct(OrderedDict((("hSplPrinter", SimTypeLongLong(signed=False, label="UInt64")),)), name="_SPLCLIENT_INFO_2_V3", pack=False, align=None),
    "_SQES_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_SQES_e__Struct", pack=False, align=None),
    "_STGMEDIUM_UNION": SimStruct(OrderedDict((("tymed", SimTypeInt(signed=False, label="UInt32")), ("u", SimUnion({"hMetaFilePict": SimTypePointer(SimTypeRef("userHMETAFILEPICT", SimStruct), offset=0), "hHEnhMetaFile": SimTypePointer(SimTypeRef("userHENHMETAFILE", SimStruct), offset=0), "hGdiHandle": SimTypePointer(SimTypeRef("GDI_OBJECT", SimStruct), offset=0), "hGlobal": SimTypePointer(SimTypeRef("userHGLOBAL", SimStruct), offset=0), "lpszFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pstm": SimTypePointer(SimTypeRef("BYTE_BLOB", SimStruct), offset=0), "pstg": SimTypePointer(SimTypeRef("BYTE_BLOB", SimStruct), offset=0)}, name="<anon>", label="None")),)), name="_STGMEDIUM_UNION", pack=False, align=None),
    "_SUBTAGS": SimStruct(OrderedDict((("SubTag", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("MatchPattern", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReturnText", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("ReturnTextSize", SimTypeInt(signed=False, label="UInt32")), ("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("Reserved2", SimTypeInt(signed=False, label="UInt32")),)), name="_SUBTAGS", pack=False, align=None),
    "_SV3CVW3_FLAGS": SimTypeInt(signed=False, label="_SV3CVW3_FLAGS"),
    "_SVGIO": SimTypeInt(signed=False, label="_SVGIO"),
    "_SVSIF": SimTypeInt(signed=False, label="_SVSIF"),
    "_S_un_b_e__Struct": SimStruct(OrderedDict((("s_b1", SimTypeChar(label="Byte")), ("s_b2", SimTypeChar(label="Byte")), ("s_b3", SimTypeChar(label="Byte")), ("s_b4", SimTypeChar(label="Byte")),)), name="_S_un_b_e__Struct", pack=False, align=None),
    "_S_un_e__Union": SimUnion({"S_un_ports": SimTypeRef("_S_un_ports_e__Struct", SimStruct), "S_un_icmp": SimTypeRef("_S_un_icmp_e__Struct", SimStruct), "S_Spi": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_S_un_icmp_e__Struct": SimStruct(OrderedDict((("s_type", SimTypeChar(label="Byte")), ("s_code", SimTypeChar(label="Byte")), ("filler", SimTypeShort(signed=False, label="UInt16")),)), name="_S_un_icmp_e__Struct", pack=False, align=None),
    "_S_un_ports_e__Struct": SimStruct(OrderedDict((("s_srcport", SimTypeShort(signed=False, label="UInt16")), ("s_dstport", SimTypeShort(signed=False, label="UInt16")),)), name="_S_un_ports_e__Struct", pack=False, align=None),
    "_S_un_w_e__Struct": SimStruct(OrderedDict((("s_w1", SimTypeShort(signed=False, label="UInt16")), ("s_w2", SimTypeShort(signed=False, label="UInt16")),)), name="_S_un_w_e__Struct", pack=False, align=None),
    "_Samples_e__Union": SimUnion({"wValidBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "wSamplesPerBlock": SimTypeShort(signed=False, label="UInt16"), "wReserved": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_SanitizeCommand_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_SanitizeOption_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_SanitizeOption_e__Struct", pack=False, align=None),
    "_ScalarPrefetch_e__Struct": SimStruct(OrderedDict((("Minimum", SimTypeShort(signed=False, label="UInt16")), ("Maximum", SimTypeShort(signed=False, label="UInt16")), ("MaximumBlocks", SimTypeShort(signed=False, label="UInt16")),)), name="_ScalarPrefetch_e__Struct", pack=False, align=None),
    "_Sci_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_Sci_e__Struct", pack=False, align=None),
    "_ScsiInformation_e__Struct": SimStruct(OrderedDict((("MediumType", SimTypeChar(label="Byte")), ("DensityCode", SimTypeChar(label="Byte")),)), name="_ScsiInformation_e__Struct", pack=False, align=None),
    "_ScsiSenseKey_e__Struct": SimStruct(OrderedDict((("SenseKey", SimTypeChar(label="Byte")), ("ASC", SimTypeChar(label="Byte")), ("ASCQ", SimTypeChar(label="Byte")), ("Reserved", SimTypeChar(label="Byte")),)), name="_ScsiSenseKey_e__Struct", pack=False, align=None),
    "_Scsi_e__Struct": SimStruct(OrderedDict((("Srb", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="_Scsi_e__Struct", pack=False, align=None),
    "_Sea_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_Sea_e__Struct", pack=False, align=None),
    "_SecondaryStatus_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUSHORT": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_Section_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("NumberOfRelocations", SimTypeShort(signed=False, label="UInt16")), ("NumberOfLinenumbers", SimTypeShort(signed=False, label="UInt16")), ("CheckSum", SimTypeInt(signed=False, label="UInt32")), ("Number", SimTypeShort(signed=True, label="Int16")), ("Selection", SimTypeChar(label="Byte")), ("bReserved", SimTypeChar(label="Byte")), ("HighNumber", SimTypeShort(signed=True, label="Int16")), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="_Section_e__Struct", pack=False, align=None),
    "_SecurityInformation_e__Struct": SimStruct(OrderedDict((("SecurityBufferSize", SimTypeInt(signed=False, label="UInt32")), ("OffsetToSecurityDescriptor", SimTypeInt(signed=False, label="UInt32")),)), name="_SecurityInformation_e__Struct", pack=False, align=None),
    "_SegmentProperties_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeLongLong(signed=False, label="UInt64")),)), name="_SegmentProperties_e__Struct", pack=False, align=None),
    "_Segment_e__Struct": SimStruct(OrderedDict((("Base", SimTypeLongLong(signed=False, label="UInt64")), ("Limit", SimTypeInt(signed=False, label="UInt32")), ("Selector", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"Attributes": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")),)), name="_Segment_e__Struct", pack=False, align=None),
    "_Sei_e__Struct": SimStruct(OrderedDict((("PollInterval", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingThreshold", SimTypeInt(signed=False, label="UInt32")), ("SwitchToPollingWindow", SimTypeInt(signed=False, label="UInt32")), ("ErrorThreshold", SimTypeInt(signed=False, label="UInt32")), ("ErrorThresholdWindow", SimTypeInt(signed=False, label="UInt32")),)), name="_Sei_e__Struct", pack=False, align=None),
    "_SemaphoreHandle_e__Struct": SimStruct(OrderedDict((("Semaphore", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")), ("Adjustment", SimTypeInt(signed=True, label="Int32")),)), name="_SemaphoreHandle_e__Struct", pack=False, align=None),
    "_Semaphore_e__Struct": SimStruct(OrderedDict((("CurrentCount", SimTypeInt(signed=True, label="Int32")), ("MaximumCount", SimTypeInt(signed=True, label="Int32")),)), name="_Semaphore_e__Struct", pack=False, align=None),
    "_SequentialPreferredZone_e__Struct": SimStruct(OrderedDict((("OptimalOpenZoneCount", SimTypeInt(signed=False, label="UInt32")), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_SequentialPreferredZone_e__Struct", pack=False, align=None),
    "_SequentialRequiredZone_e__Struct": SimStruct(OrderedDict((("MaxOpenZoneCount", SimTypeInt(signed=False, label="UInt32")), ("UnrestrictedRead", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)),)), name="_SequentialRequiredZone_e__Struct", pack=False, align=None),
    "_SerialNumberParameter_e__Struct": SimStruct(OrderedDict((("cbSerialNumberOffset", SimTypeInt(signed=False, label="UInt32")), ("cbSerialNumberLength", SimTypeInt(signed=False, label="UInt32")), ("dwDesiredCardModuleVersion", SimTypeInt(signed=False, label="UInt32")),)), name="_SerialNumberParameter_e__Struct", pack=False, align=None),
    "_ServerCharacteristicConfiguration_e__Struct": SimStruct(OrderedDict((("IsBroadcast", SimTypeChar(label="Byte")),)), name="_ServerCharacteristicConfiguration_e__Struct", pack=False, align=None),
    "_Server_e__Struct": SimStruct(OrderedDict((("Capabilities", SimTypeInt(signed=False, label="UInt32")),)), name="_Server_e__Struct", pack=False, align=None),
    "_SetAgentActivity_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("dwActivityID", SimTypeInt(signed=False, label="UInt32")),)), name="_SetAgentActivity_e__Struct", pack=False, align=None),
    "_SetAgentGroup_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("GroupList", SimTypeRef("LINEAGENTGROUPLIST", SimStruct)),)), name="_SetAgentGroup_e__Struct", pack=False, align=None),
    "_SetAgentMeasurementPeriod_e__Struct": SimStruct(OrderedDict((("hAgent", SimTypeInt(signed=False, label="UInt32")), ("dwMeasurementPeriod", SimTypeInt(signed=False, label="UInt32")),)), name="_SetAgentMeasurementPeriod_e__Struct", pack=False, align=None),
    "_SetAgentSessionState_e__Struct": SimStruct(OrderedDict((("hAgentSession", SimTypeInt(signed=False, label="UInt32")), ("dwAgentSessionState", SimTypeInt(signed=False, label="UInt32")), ("dwNextAgentSessionState", SimTypeInt(signed=False, label="UInt32")),)), name="_SetAgentSessionState_e__Struct", pack=False, align=None),
    "_SetAgentStateEx_e__Struct": SimStruct(OrderedDict((("hAgent", SimTypeInt(signed=False, label="UInt32")), ("dwAgentState", SimTypeInt(signed=False, label="UInt32")), ("dwNextAgentState", SimTypeInt(signed=False, label="UInt32")),)), name="_SetAgentStateEx_e__Struct", pack=False, align=None),
    "_SetAgentState_e__Struct": SimStruct(OrderedDict((("dwAddressID", SimTypeInt(signed=False, label="UInt32")), ("dwAgentState", SimTypeInt(signed=False, label="UInt32")), ("dwNextAgentState", SimTypeInt(signed=False, label="UInt32")),)), name="_SetAgentState_e__Struct", pack=False, align=None),
    "_SetBudget_e__Struct": SimStruct(OrderedDict((("LocalMemoryBudget", SimTypeLongLong(signed=False, label="UInt64")), ("SystemMemoryBudget", SimTypeLongLong(signed=False, label="UInt64")),)), name="_SetBudget_e__Struct", pack=False, align=None),
    "_SetEa_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("EaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_SetEa_e__Struct", pack=False, align=None),
    "_SetFault_e__Struct": SimStruct(OrderedDict((("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_SetFault_e__Struct", pack=False, align=None),
    "_SetFileInformation_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FileInformationClass", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")), ("ParentOfTarget", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("ReplaceIfExists", SimTypeChar(label="Byte")), ("AdvanceOnly", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ClusterCount": SimTypeInt(signed=False, label="UInt32"), "DeleteHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")), ("InfoBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_SetFileInformation_e__Struct", pack=False, align=None),
    "_SetFile_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FileInformationClass", SimTypeInt(signed=False, label="FILE_INFORMATION_CLASS")), ("FileObject", SimTypePointer(SimTypeRef("FILE_OBJECT", SimStruct), offset=0)), ("Anonymous", SimUnion({"Anonymous": SimStruct(OrderedDict((("ReplaceIfExists", SimTypeChar(label="Byte")), ("AdvanceOnly", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "ClusterCount": SimTypeInt(signed=False, label="UInt32"), "DeleteHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None")),)), name="_SetFile_e__Struct", pack=False, align=None),
    "_SetLock_e__Struct": SimStruct(OrderedDict((("Lock", SimTypeChar(label="Byte")),)), name="_SetLock_e__Struct", pack=False, align=None),
    "_SetProperty_e__Struct": SimStruct(OrderedDict((("PropertyType", SimTypeInt(signed=False, label="UInt32")), ("PropertyId", SimTypeInt(signed=False, label="UInt32")), ("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactor", SimTypeChar(label="Byte")), ("PropertyBufferSize", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("PropertyBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_SetProperty_e__Struct", pack=False, align=None),
    "_SetQueueMeasurementPeriod_e__Struct": SimStruct(OrderedDict((("dwQueueID", SimTypeInt(signed=False, label="UInt32")), ("dwMeasurementPeriod", SimTypeInt(signed=False, label="UInt32")),)), name="_SetQueueMeasurementPeriod_e__Struct", pack=False, align=None),
    "_SetQuota_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("QuotaBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_SetQuota_e__Struct", pack=False, align=None),
    "_SetSecurity_e__Struct": SimStruct(OrderedDict((("SecurityInformation", SimTypeInt(signed=False, label="UInt32")), ("SecurityDescriptor", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_SetSecurity_e__Struct", pack=False, align=None),
    "_SetTrimIntervals_e__Struct": SimStruct(OrderedDict((("MinTrimInterval", SimTypeInt(signed=False, label="UInt32")), ("MaxTrimInterval", SimTypeInt(signed=False, label="UInt32")), ("IdleTrimInterval", SimTypeInt(signed=False, label="UInt32")),)), name="_SetTrimIntervals_e__Struct", pack=False, align=None),
    "_SetVolumeInformation_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FsInformationClass", SimTypeInt(signed=False, label="FS_INFORMATION_CLASS")), ("VolumeBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_SetVolumeInformation_e__Struct", pack=False, align=None),
    "_SetVolume_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("FsInformationClass", SimTypeInt(signed=False, label="FS_INFORMATION_CLASS")),)), name="_SetVolume_e__Struct", pack=False, align=None),
    "_Set_e__Struct": SimStruct(OrderedDict((("DirectoryHandle", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_Set_e__Struct", pack=False, align=None),
    "_SetupPacket_e__Struct": SimStruct(OrderedDict((("bmRequest", SimTypeChar(label="Byte")), ("bRequest", SimTypeChar(label="Byte")), ("wValue", SimTypeShort(signed=False, label="UInt16")), ("wIndex", SimTypeShort(signed=False, label="UInt16")), ("wLength", SimTypeShort(signed=False, label="UInt16")),)), name="_SetupPacket_e__Struct", pack=False, align=None),
    "_ShaderResourceView_e__Struct": SimStruct(OrderedDict((("RootParameterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="_ShaderResourceView_e__Struct", pack=False, align=None),
    "_Share_e__Struct": SimStruct(OrderedDict((("Capabilities", SimTypeInt(signed=False, label="UInt32")), ("ShareFlags", SimTypeInt(signed=False, label="UInt32")),)), name="_Share_e__Struct", pack=False, align=None),
    "_Size_e__Struct": SimStruct(OrderedDict((("VirtualSize", SimTypeLongLong(signed=False, label="UInt64")), ("PhysicalSize", SimTypeLongLong(signed=False, label="UInt64")), ("BlockSize", SimTypeInt(signed=False, label="UInt32")), ("SectorSize", SimTypeInt(signed=False, label="UInt32")),)), name="_Size_e__Struct", pack=False, align=None),
    "_Smb2_e__Struct": SimStruct(OrderedDict((("Server", SimTypeRef("_Server_e__Struct", SimStruct)), ("Share", SimTypeRef("_Share_e__Struct", SimStruct)),)), name="_Smb2_e__Struct", pack=False, align=None),
    "_SnapshotPath_e__Struct": SimStruct(OrderedDict((("SnapshotId", SimTypeBottom(label="Guid")), ("SnapshotFilePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_SnapshotPath_e__Struct", pack=False, align=None),
    "_Specific_e__Union": SimUnion({"Null": SimTypeInt(signed=False, label="UInt32"), "FacialFeatures": SimTypeRef("_FacialFeatures_e__Struct", SimStruct), "Fingerprint": SimTypeRef("_Fingerprint_e__Struct", SimStruct), "Iris": SimTypeRef("_Iris_e__Struct", SimStruct), "Voice": SimTypeRef("_Voice_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Src_e__Struct": SimStruct(OrderedDict((("NetworkAddress", SimTypeInt(signed=False, label="UInt32")), ("NodeAddress", SimTypeArray(SimTypeChar(label="Byte"), 6)), ("Socket", SimTypeShort(signed=False, label="UInt16")),)), name="_Src_e__Struct", pack=False, align=None),
    "_StackSpecific_e__Union": SimUnion({"ExternalStack": SimTypeRef("_ExternalStack_e__Struct", SimStruct), "AtaPort": SimTypeRef("_AtaPort_e__Struct", SimStruct), "StorPort": SimTypeRef("_StorPort_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_StartDevice_e__Struct": SimStruct(OrderedDict((("AllocatedResources", SimTypePointer(SimTypeRef("CM_RESOURCE_LIST", SimStruct), offset=0)), ("AllocatedResourcesTranslated", SimTypePointer(SimTypeRef("CM_RESOURCE_LIST", SimStruct), offset=0)),)), name="_StartDevice_e__Struct", pack=False, align=None),
    "_State_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUchar": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_StatusCodeType_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_StatusCodeType_e__Struct", pack=False, align=None),
    "_Status_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Status_e__Struct", pack=False, align=None),
    "_Status_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUCHAR": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_StorPort_e__Struct": SimStruct(OrderedDict((("SrbTag", SimTypeInt(signed=False, label="UInt32")),)), name="_StorPort_e__Struct", pack=False, align=None),
    "_StorageOffloadZeroDataToken_e__Struct": SimStruct(OrderedDict((("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 504)),)), name="_StorageOffloadZeroDataToken_e__Struct", pack=False, align=None),
    "_StreamFinished_e__Struct": SimStruct(OrderedDict((("dwStreamNumber", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("hSourceFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDestinationFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uliStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliStreamBytesTransferred", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalFileSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalBytesTransferred", SimTypeLongLong(signed=False, label="UInt64")),)), name="_StreamFinished_e__Struct", pack=False, align=None),
    "_StreamInformation": SimUnion({"DesiredStorageClass": SimTypeRef("_DesiredStorageClass", SimStruct), "DataStream": SimTypeRef("_DataStream", SimStruct), "Reparse": SimTypeRef("_Reparse", SimStruct), "Ea": SimTypeRef("_Ea", SimStruct)}, name="<anon>", label="None"),
    "_StreamStarted_e__Struct": SimStruct(OrderedDict((("dwStreamNumber", SimTypeInt(signed=False, label="UInt32")), ("dwReserved", SimTypeInt(signed=False, label="UInt32")), ("hSourceFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hDestinationFile", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uliStreamSize", SimTypeLongLong(signed=False, label="UInt64")), ("uliTotalFileSize", SimTypeLongLong(signed=False, label="UInt64")),)), name="_StreamStarted_e__Struct", pack=False, align=None),
    "_StreamsInformation_e__Struct": SimStruct(OrderedDict((("StreamsInfoBufferSize", SimTypeInt(signed=False, label="UInt32")), ("OffsetToFirstStreamInfo", SimTypeInt(signed=False, label="UInt32")),)), name="_StreamsInformation_e__Struct", pack=False, align=None),
    "_SubSection_e__Struct": SimStruct(OrderedDict((("Offset", SimTypeLongLong(signed=False, label="UInt64")), ("Length", SimTypeInt(signed=False, label="UInt32")),)), name="_SubSection_e__Struct", pack=False, align=None),
    "_SupportedCommands_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")), ("MaxEvictCommands", SimTypeInt(signed=False, label="UInt32")), ("MaxLbaRangeCountForEvict", SimTypeInt(signed=False, label="UInt32")), ("MaxLbaRangeCountForChangeLba", SimTypeInt(signed=False, label="UInt32")),)), name="_SupportedCommands_e__Struct", pack=False, align=None),
    "_Supported_e__Struct": SimStruct(OrderedDict((("Encapsulation", SimTypeInt(signed=False, label="UInt32")), ("AhEspCombined", SimTypeInt(signed=False, label="UInt32")), ("TransportTunnelCombined", SimTypeInt(signed=False, label="UInt32")), ("IPv4Options", SimTypeInt(signed=False, label="UInt32")), ("Flags", SimTypeInt(signed=False, label="UInt32")),)), name="_Supported_e__Struct", pack=False, align=None),
    "_SuspendProcess_e__Struct": SimStruct(OrderedDict((("hProcess", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("bAllowWakeOnSubmission", SimTypeInt(signed=True, label="Int32")),)), name="_SuspendProcess_e__Struct", pack=False, align=None),
    "_Sym_e__Struct": SimStruct(OrderedDict((("WeakDefaultSymIndex", SimTypeInt(signed=False, label="UInt32")), ("WeakSearchType", SimTypeInt(signed=False, label="UInt32")), ("rgbReserved", SimTypeArray(SimTypeChar(label="Byte"), 12)),)), name="_Sym_e__Struct", pack=False, align=None),
    "_SymbolicLinkReparseBuffer_e__Struct": SimStruct(OrderedDict((("SubstituteNameOffset", SimTypeShort(signed=False, label="UInt16")), ("SubstituteNameLength", SimTypeShort(signed=False, label="UInt16")), ("PrintNameOffset", SimTypeShort(signed=False, label="UInt16")), ("PrintNameLength", SimTypeShort(signed=False, label="UInt16")), ("Flags", SimTypeInt(signed=False, label="UInt32")), ("PathBuffer", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_SymbolicLinkReparseBuffer_e__Struct", pack=False, align=None),
    "_Symlink_e__Struct": SimStruct(OrderedDict((("TargetName", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Symlink_e__Struct", pack=False, align=None),
    "_SynchronizationMutex_e__Struct": SimStruct(OrderedDict((("InitialState", SimTypeInt(signed=True, label="Int32")),)), name="_SynchronizationMutex_e__Struct", pack=False, align=None),
    "_TDH_IN_TYPE": SimTypeInt(signed=False, label="_TDH_IN_TYPE"),
    "_TDH_OUT_TYPE": SimTypeInt(signed=False, label="_TDH_OUT_TYPE"),
    "_TRANSFER_ADVISE_STATE": SimTypeInt(signed=False, label="_TRANSFER_ADVISE_STATE"),
    "_TRANSFER_SOURCE_FLAGS": SimTypeInt(signed=False, label="_TRANSFER_SOURCE_FLAGS"),
    "_Table_e__Struct": SimStruct(OrderedDict((("Limit", SimTypeShort(signed=False, label="UInt16")), ("Base", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Table_e__Struct", pack=False, align=None),
    "_TagHeader_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_TagHeader_e__Struct", pack=False, align=None),
    "_Tail_e__Union": SimUnion({"Overlay": SimTypeRef("_Overlay_e__Struct", SimStruct), "Apc": SimTypeRef("KAPC", SimStruct), "CompletionKey": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"),
    "_TapeInfo_e__Struct": SimStruct(OrderedDict((("MediaType", SimTypeInt(signed=False, label="STORAGE_MEDIA_TYPE")), ("MediaCharacteristics", SimTypeInt(signed=False, label="UInt32")), ("CurrentBlockSize", SimTypeInt(signed=False, label="UInt32")), ("BusType", SimTypeInt(signed=False, label="STORAGE_BUS_TYPE")), ("BusSpecificData", SimUnion({"ScsiInformation": SimTypeRef("_ScsiInformation_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="_TapeInfo_e__Struct", pack=False, align=None),
    "_Target_e__Struct": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwDeviceID", SimTypeInt(signed=False, label="UInt32")), ("wMid", SimTypeShort(signed=False, label="UInt16")), ("wPid", SimTypeShort(signed=False, label="UInt16")), ("vDriverVersion", SimTypeInt(signed=False, label="UInt32")), ("szPname", SimTypeArray(SimTypeChar(label="Char"), 32)),)), name="_Target_e__Struct", pack=False, align=None),
    "_TdrControl2_e__Struct": SimStruct(OrderedDict((("TdrControl", SimTypeInt(signed=False, label="UInt32")), ("Anonymous", SimUnion({"NodeOrdinal": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_TdrControl2_e__Struct", pack=False, align=None),
    "_TdrLimit_e__Struct": SimStruct(OrderedDict((("Count", SimTypeInt(signed=False, label="UInt32")), ("Time", SimTypeInt(signed=False, label="UInt32")),)), name="_TdrLimit_e__Struct", pack=False, align=None),
    "_ThermalThrottling_e__Struct": SimStruct(OrderedDict((("EventCount", SimTypeChar(label="Byte")), ("Status", SimTypeChar(label="Byte")),)), name="_ThermalThrottling_e__Struct", pack=False, align=None),
    "_ThreadObject_e__Struct": SimStruct(OrderedDict((("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("WaitTime", SimTypeInt(signed=False, label="UInt32")), ("ContextSwitches", SimTypeInt(signed=False, label="UInt32")),)), name="_ThreadObject_e__Struct", pack=False, align=None),
    "_Thread_e__Struct": SimStruct(OrderedDict((("ExitStatus", SimTypeInt(signed=False, label="UInt32")), ("TebBaseAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("ProcessId", SimTypeInt(signed=False, label="UInt32")), ("ThreadId", SimTypeInt(signed=False, label="UInt32")), ("AffinityMask", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Priority", SimTypeInt(signed=True, label="Int32")), ("BasePriority", SimTypeInt(signed=True, label="Int32")), ("Win32StartAddress", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_Thread_e__Struct", pack=False, align=None),
    "_Timecode_e__Struct": SimStruct(OrderedDict((("frame", SimTypeChar(label="Byte")), ("second", SimTypeChar(label="Byte")), ("minute", SimTypeChar(label="Byte")), ("hour", SimTypeChar(label="Byte")),)), name="_Timecode_e__Struct", pack=False, align=None),
    "_Token_e__Union": SimUnion({"Flip": SimTypeRef("D3DKMT_FLIPMODEL_PRESENTHISTORYTOKEN", SimStruct), "Blt": SimTypeRef("D3DKMT_BLTMODEL_PRESENTHISTORYTOKEN", SimStruct), "VistaBlt": SimTypeLongLong(signed=False, label="UInt64"), "Gdi": SimTypeRef("D3DKMT_GDIMODEL_PRESENTHISTORYTOKEN", SimStruct), "Fence": SimTypeRef("D3DKMT_FENCE_PRESENTHISTORYTOKEN", SimStruct), "GdiSysMem": SimTypeRef("D3DKMT_GDIMODEL_SYSMEM_PRESENTHISTORYTOKEN", SimStruct), "Composition": SimTypeRef("D3DKMT_COMPOSITION_PRESENTHISTORYTOKEN", SimStruct), "FlipManager": SimTypeRef("D3DKMT_FLIPMANAGER_PRESENTHISTORYTOKEN", SimStruct), "SurfaceComplete": SimTypeRef("D3DKMT_SURFACECOMPLETE_PRESENTHISTORYTOKEN", SimStruct)}, name="<anon>", label="None"),
    "_Trailers_e__Struct": SimStruct(OrderedDict((("TrailerCount", SimTypeShort(signed=False, label="UInt16")), ("pTrailers", SimTypePointer(SimTypeRef("HTTP_UNKNOWN_HEADER", SimStruct), offset=0)),)), name="_Trailers_e__Struct", pack=False, align=None),
    "_TransferData_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_TRANSFER_DATA_FLAGS")), ("CompletionStatus", SimTypeInt(signed=True, label="Int32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Offset", SimTypeLongLong(signed=True, label="Int64")), ("Length", SimTypeLongLong(signed=True, label="Int64")),)), name="_TransferData_e__Struct", pack=False, align=None),
    "_TransferPlaceholders_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_OPERATION_TRANSFER_PLACEHOLDERS_FLAGS")), ("CompletionStatus", SimTypeInt(signed=True, label="Int32")), ("PlaceholderTotalCount", SimTypeLongLong(signed=True, label="Int64")), ("PlaceholderArray", SimTypePointer(SimTypeRef("CF_PLACEHOLDER_CREATE_INFO", SimStruct), offset=0)), ("PlaceholderCount", SimTypeInt(signed=False, label="UInt32")), ("EntriesProcessed", SimTypeInt(signed=False, label="UInt32")),)), name="_TransferPlaceholders_e__Struct", pack=False, align=None),
    "_Translated_e__Struct": SimStruct(OrderedDict((("Level", SimTypeInt(signed=False, label="UInt32")), ("Vector", SimTypeInt(signed=False, label="UInt32")), ("Affinity", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_Translated_e__Struct", pack=False, align=None),
    "_Transmit_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Transmit_e__Struct", pack=False, align=None),
    "_TxfFileId_e__Struct": SimStruct(OrderedDict((("LowPart", SimTypeLongLong(signed=True, label="Int64")), ("HighPart", SimTypeLongLong(signed=True, label="Int64")),)), name="_TxfFileId_e__Struct", pack=False, align=None),
    "_TypeSpecificInformation_e__Union": SimUnion({"Process": SimTypeRef("_Process_e__Struct", SimStruct), "Thread": SimTypeRef("_Thread_e__Struct", SimStruct), "Mutant": SimTypeRef("_Mutant_e__Struct", SimStruct), "Event": SimTypeRef("_Event_e__Struct", SimStruct), "Section": SimTypeRef("_Section_e__Struct", SimStruct), "Semaphore": SimTypeRef("_Semaphore_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_Type_e__Union": SimUnion({"SymbolTableIndex": SimTypeInt(signed=False, label="UInt32"), "VirtualAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_URB_BULK_OR_INTERRUPT_TRANSFER": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferFlags", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)),)), name="_URB_BULK_OR_INTERRUPT_TRANSFER", pack=False, align=None),
    "_URB_CONTROL_DESCRIPTOR_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("Reserved1", SimTypeShort(signed=False, label="UInt16")), ("Index", SimTypeChar(label="Byte")), ("DescriptorType", SimTypeChar(label="Byte")), ("LanguageId", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_URB_CONTROL_DESCRIPTOR_REQUEST", pack=False, align=None),
    "_URB_CONTROL_FEATURE_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved2", SimTypeInt(signed=False, label="UInt32")), ("Reserved3", SimTypeInt(signed=False, label="UInt32")), ("Reserved4", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved5", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("Reserved0", SimTypeShort(signed=False, label="UInt16")), ("FeatureSelector", SimTypeShort(signed=False, label="UInt16")), ("Index", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")),)), name="_URB_CONTROL_FEATURE_REQUEST", pack=False, align=None),
    "_URB_CONTROL_GET_CONFIGURATION_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="_URB_CONTROL_GET_CONFIGURATION_REQUEST", pack=False, align=None),
    "_URB_CONTROL_GET_INTERFACE_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Interface", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_URB_CONTROL_GET_INTERFACE_REQUEST", pack=False, align=None),
    "_URB_CONTROL_GET_STATUS_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 4)), ("Index", SimTypeShort(signed=False, label="UInt16")), ("Reserved2", SimTypeShort(signed=False, label="UInt16")),)), name="_URB_CONTROL_GET_STATUS_REQUEST", pack=False, align=None),
    "_URB_CONTROL_TRANSFER": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferFlags", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("SetupPacket", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="_URB_CONTROL_TRANSFER", pack=False, align=None),
    "_URB_CONTROL_TRANSFER_EX": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferFlags", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Timeout", SimTypeInt(signed=False, label="UInt32")), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("SetupPacket", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="_URB_CONTROL_TRANSFER_EX", pack=False, align=None),
    "_URB_CONTROL_VENDOR_OR_CLASS_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferFlags", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("RequestTypeReservedBits", SimTypeChar(label="Byte")), ("Request", SimTypeChar(label="Byte")), ("Value", SimTypeShort(signed=False, label="UInt16")), ("Index", SimTypeShort(signed=False, label="UInt16")), ("Reserved1", SimTypeShort(signed=False, label="UInt16")),)), name="_URB_CONTROL_VENDOR_OR_CLASS_REQUEST", pack=False, align=None),
    "_URB_FRAME_LENGTH_CONTROL": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)),)), name="_URB_FRAME_LENGTH_CONTROL", pack=False, align=None),
    "_URB_GET_CURRENT_FRAME_NUMBER": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("FrameNumber", SimTypeInt(signed=False, label="UInt32")),)), name="_URB_GET_CURRENT_FRAME_NUMBER", pack=False, align=None),
    "_URB_GET_FRAME_LENGTH": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("FrameLength", SimTypeInt(signed=False, label="UInt32")), ("FrameNumber", SimTypeInt(signed=False, label="UInt32")),)), name="_URB_GET_FRAME_LENGTH", pack=False, align=None),
    "_URB_GET_ISOCH_PIPE_TRANSFER_PATH_DELAYS": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MaximumSendPathDelayInMilliSeconds", SimTypeInt(signed=False, label="UInt32")), ("MaximumCompletionPathDelayInMilliSeconds", SimTypeInt(signed=False, label="UInt32")),)), name="_URB_GET_ISOCH_PIPE_TRANSFER_PATH_DELAYS", pack=False, align=None),
    "_URB_HCD_AREA": SimStruct(OrderedDict((("Reserved8", SimTypeArray(SimTypePointer(SimTypeBottom(label="Void"), offset=0), 8)),)), name="_URB_HCD_AREA", pack=False, align=None),
    "_URB_HEADER": SimStruct(OrderedDict((("Length", SimTypeShort(signed=False, label="UInt16")), ("Function", SimTypeShort(signed=False, label="UInt16")), ("Status", SimTypeInt(signed=True, label="Int32")), ("UsbdDeviceHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UsbdFlags", SimTypeInt(signed=False, label="UInt32")),)), name="_URB_HEADER", pack=False, align=None),
    "_URB_ISOCH_TRANSFER": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferFlags", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("StartFrame", SimTypeInt(signed=False, label="UInt32")), ("NumberOfPackets", SimTypeInt(signed=False, label="UInt32")), ("ErrorCount", SimTypeInt(signed=False, label="UInt32")), ("IsoPacket", SimTypePointer(SimTypeRef("USBD_ISO_PACKET_DESCRIPTOR", SimStruct), offset=0)),)), name="_URB_ISOCH_TRANSFER", pack=False, align=None),
    "_URB_OPEN_STATIC_STREAMS": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("NumberOfStreams", SimTypeInt(signed=False, label="UInt32")), ("StreamInfoVersion", SimTypeShort(signed=False, label="UInt16")), ("StreamInfoSize", SimTypeShort(signed=False, label="UInt16")), ("Streams", SimTypePointer(SimTypeRef("USBD_STREAM_INFORMATION", SimStruct), offset=0)),)), name="_URB_OPEN_STATIC_STREAMS", pack=False, align=None),
    "_URB_OS_FEATURE_DESCRIPTOR_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("Reserved", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved0", SimTypeInt(signed=False, label="UInt32")), ("TransferBufferLength", SimTypeInt(signed=False, label="UInt32")), ("TransferBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("TransferBufferMDL", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("UrbLink", SimTypePointer(SimTypeRef("URB", SimStruct), offset=0)), ("hca", SimTypeRef("_URB_HCD_AREA", SimStruct)), ("_bitfield", SimTypeChar(label="Byte")), ("Reserved2", SimTypeChar(label="Byte")), ("InterfaceNumber", SimTypeChar(label="Byte")), ("MS_PageIndex", SimTypeChar(label="Byte")), ("MS_FeatureDescriptorIndex", SimTypeShort(signed=False, label="UInt16")), ("Reserved3", SimTypeShort(signed=False, label="UInt16")),)), name="_URB_OS_FEATURE_DESCRIPTOR_REQUEST", pack=False, align=None),
    "_URB_PIPE_REQUEST": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("PipeHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_URB_PIPE_REQUEST", pack=False, align=None),
    "_URB_SELECT_CONFIGURATION": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("ConfigurationDescriptor", SimTypePointer(SimTypeRef("USB_CONFIGURATION_DESCRIPTOR", SimStruct), offset=0)), ("ConfigurationHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Interface", SimTypeRef("USBD_INTERFACE_INFORMATION", SimStruct)),)), name="_URB_SELECT_CONFIGURATION", pack=False, align=None),
    "_URB_SELECT_INTERFACE": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("ConfigurationHandle", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("Interface", SimTypeRef("USBD_INTERFACE_INFORMATION", SimStruct)),)), name="_URB_SELECT_INTERFACE", pack=False, align=None),
    "_URB_SET_FRAME_LENGTH": SimStruct(OrderedDict((("Hdr", SimTypeRef("_URB_HEADER", SimStruct)), ("FrameLengthDelta", SimTypeInt(signed=True, label="Int32")),)), name="_URB_SET_FRAME_LENGTH", pack=False, align=None),
    "_URes": SimUnion({"ar": SimTypeRef("NODERESTRICTION", SimStruct), "orRestriction": SimTypeRef("NODERESTRICTION", SimStruct), "pxr": SimTypeRef("NODERESTRICTION", SimStruct), "vr": SimTypeRef("VECTORRESTRICTION", SimStruct), "nr": SimTypeRef("NOTRESTRICTION", SimStruct), "cr": SimTypeRef("CONTENTRESTRICTION", SimStruct), "nlr": SimTypeRef("NATLANGUAGERESTRICTION", SimStruct), "pr": SimTypeRef("PROPERTYRESTRICTION", SimStruct)}, name="<anon>", label="None"),
    "_U_e__Union": SimUnion({"Demux": SimTypeRef("MPEG_BCS_DEMUX", SimStruct), "Winsock": SimTypeRef("MPEG_WINSOCK", SimStruct)}, name="<anon>", label="None"),
    "_UnclaimedIdentify_e__Struct": SimStruct(OrderedDict((("UnitId", SimTypeInt(signed=False, label="UInt32")), ("Identity", SimTypeRef("WINBIO_IDENTITY", SimStruct)), ("SubFactor", SimTypeChar(label="Byte")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")),)), name="_UnclaimedIdentify_e__Struct", pack=False, align=None),
    "_Unclaimed_e__Struct": SimStruct(OrderedDict((("UnitId", SimTypeInt(signed=False, label="UInt32")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")),)), name="_Unclaimed_e__Struct", pack=False, align=None),
    "_UnknownType": SimStruct(OrderedDict((("dwActualLength", SimTypeInt(signed=False, label="UInt32")), ("rgMetadata", SimTypeArray(SimTypeChar(label="Byte"), 16)), ("pUnknownData", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_UnknownType", pack=False, align=None),
    "_Unmap_e__Struct": SimStruct(OrderedDict((("BaseAddress", SimTypeLongLong(signed=False, label="UInt64")), ("SizeInBytes", SimTypeLongLong(signed=False, label="UInt64")), ("Protection", SimTypeRef("D3DDDIGPUVIRTUALADDRESS_PROTECTION_TYPE", SimStruct)),)), name="_Unmap_e__Struct", pack=False, align=None),
    "_UnorderedAccessView_e__Struct": SimStruct(OrderedDict((("RootParameterIndex", SimTypeInt(signed=False, label="UInt32")),)), name="_UnorderedAccessView_e__Struct", pack=False, align=None),
    "_UsageNotification_e__Struct": SimStruct(OrderedDict((("InPath", SimTypeChar(label="Byte")), ("Reserved", SimTypeArray(SimTypeChar(label="Byte"), 3)), ("Type", SimTypeInt(signed=False, label="DEVICE_USAGE_NOTIFICATION_TYPE")),)), name="_UsageNotification_e__Struct", pack=False, align=None),
    "_UserDataEraseCounts_e__Struct": SimStruct(OrderedDict((("MaximumCount", SimTypeInt(signed=False, label="UInt32")), ("MinimumCount", SimTypeInt(signed=False, label="UInt32")),)), name="_UserDataEraseCounts_e__Struct", pack=False, align=None),
    "_VWC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_VWC_e__Struct", pack=False, align=None),
    "_ValidDiagnostics_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_ValidDiagnostics_e__Struct", pack=False, align=None),
    "_ValidVal_e__Union": SimUnion({"Range": SimTypeRef("_Range_e__Struct", SimStruct), "RangeFloat": SimTypeRef("_RangeFloat_e__Struct", SimStruct), "List": SimTypeRef("_List_e__Struct", SimStruct), "ListFloat": SimTypeRef("_ListFloat_e__Struct", SimStruct), "ListGuid": SimTypeRef("_ListGuid_e__Struct", SimStruct), "ListBStr": SimTypeRef("_ListBStr_e__Struct", SimStruct), "Flag": SimTypeRef("_Flag_e__Struct", SimStruct), "None": SimTypeRef("_None_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_ValidValues_e__Union": SimUnion({"ValidValuesRange": SimTypeRef("WMDM_PROP_VALUES_RANGE", SimStruct), "EnumeratedValidValues": SimTypeRef("WMDM_PROP_VALUES_ENUM", SimStruct)}, name="<anon>", label="None"),
    "_ValidateData_e__Struct": SimStruct(OrderedDict((("Flags", SimTypeInt(signed=False, label="CF_CALLBACK_VALIDATE_DATA_FLAGS")), ("RequiredFileOffset", SimTypeLongLong(signed=True, label="Int64")), ("RequiredLength", SimTypeLongLong(signed=True, label="Int64")),)), name="_ValidateData_e__Struct", pack=False, align=None),
    "_ValidityPeriod_e__Struct": SimStruct(OrderedDict((("BeginDate", SimTypeLongLong(signed=True, label="Int64")), ("EndDate", SimTypeLongLong(signed=True, label="Int64")),)), name="_ValidityPeriod_e__Struct", pack=False, align=None),
    "_Value_e__Union": SimUnion({"ManufactureDate": SimTypeRef("_ManufactureDate_e__Struct", SimStruct), "AsUlonglong": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"),
    "_Values_e__Union": SimUnion({"pInt64": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pUint64": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "ppString": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pFqbn": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pOctetString": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="<anon>", label="None"),
    "_VconnPower_e__Union": SimUnion({"AsUshort": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "_VerifyAndReleaseTicket_e__Struct": SimStruct(OrderedDict((("Match", SimTypeChar(label="Byte")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")), ("Ticket", SimTypeLongLong(signed=False, label="UInt64")),)), name="_VerifyAndReleaseTicket_e__Struct", pack=False, align=None),
    "_VerifyIntegrity_e__Struct": SimStruct(OrderedDict((("SegmentId", SimTypeInt(signed=False, label="UInt32")),)), name="_VerifyIntegrity_e__Struct", pack=False, align=None),
    "_VerifyVolume_e__Struct": SimStruct(OrderedDict((("Vpb", SimTypePointer(SimTypeRef("VPB", SimStruct), offset=0)), ("DeviceObject", SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0)),)), name="_VerifyVolume_e__Struct", pack=False, align=None),
    "_Verify_e__Struct": SimStruct(OrderedDict((("Match", SimTypeChar(label="Byte")), ("RejectDetail", SimTypeInt(signed=False, label="UInt32")),)), name="_Verify_e__Struct", pack=False, align=None),
    "_Version1_e__Struct": SimStruct(OrderedDict((("ForkedVirtualDiskPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_Version1_e__Struct", pack=False, align=None),
    "_Version2_e__Struct": SimStruct(OrderedDict((("MergeSourceDepth", SimTypeInt(signed=False, label="UInt32")), ("MergeTargetDepth", SimTypeInt(signed=False, label="UInt32")),)), name="_Version2_e__Struct", pack=False, align=None),
    "_Version3_e__Struct": SimStruct(OrderedDict((("UniqueId", SimTypeBottom(label="Guid")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("BlockSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("SectorSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("PhysicalSectorSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("ParentPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourcePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("OpenFlags", SimTypeInt(signed=False, label="OPEN_VIRTUAL_DISK_FLAG")), ("ParentVirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("SourceVirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("ResiliencyGuid", SimTypeBottom(label="Guid")), ("SourceLimitPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackingStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)),)), name="_Version3_e__Struct", pack=False, align=None),
    "_Version4_e__Struct": SimStruct(OrderedDict((("UniqueId", SimTypeBottom(label="Guid")), ("MaximumSize", SimTypeLongLong(signed=False, label="UInt64")), ("BlockSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("SectorSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("PhysicalSectorSizeInBytes", SimTypeInt(signed=False, label="UInt32")), ("ParentPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("SourcePath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("OpenFlags", SimTypeInt(signed=False, label="OPEN_VIRTUAL_DISK_FLAG")), ("ParentVirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("SourceVirtualStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("ResiliencyGuid", SimTypeBottom(label="Guid")), ("SourceLimitPath", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("BackingStorageType", SimTypeRef("VIRTUAL_STORAGE_TYPE", SimStruct)), ("PmemAddressAbstractionType", SimTypeBottom(label="Guid")), ("DataAlignment", SimTypeLongLong(signed=False, label="UInt64")),)), name="_Version4_e__Struct", pack=False, align=None),
    "_VersionDetail_e__Struct": SimStruct(OrderedDict((("MajorVersion", SimTypeChar(label="Byte")), ("MinorVersion", SimTypeChar(label="Byte")), ("SubVersion", SimTypeChar(label="Byte")), ("SubMinorVersion", SimTypeChar(label="Byte")),)), name="_VersionDetail_e__Struct", pack=False, align=None),
    "_Version_e__Struct": SimStruct(OrderedDict((("Major", SimTypeInt(signed=False, label="UInt32")), ("Minor", SimTypeInt(signed=False, label="UInt32")),)), name="_Version_e__Struct", pack=False, align=None),
    "_Version_e__Union": SimUnion({"S": SimTypeRef("MPEG_HEADER_VERSION_BITS_MIDL", SimStruct), "B": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_VertexBuffer_e__Struct": SimStruct(OrderedDict((("Slot", SimTypeInt(signed=False, label="UInt32")),)), name="_VertexBuffer_e__Struct", pack=False, align=None),
    "_VidPnFromAllocation_e__Struct": SimStruct(OrderedDict((("hPrimaryAllocation", SimTypeInt(signed=False, label="UInt32")), ("VidPnSourceId", SimTypeInt(signed=False, label="UInt32")),)), name="_VidPnFromAllocation_e__Struct", pack=False, align=None),
    "_ViewOptions": SimTypeInt(signed=False, label="_ViewOptions"),
    "_Voice_e__Struct": SimStruct(OrderedDict((("Reserved", SimTypeInt(signed=False, label="UInt32")),)), name="_Voice_e__Struct", pack=False, align=None),
    "_WHEA_ERROR_SOURCE_CORRECT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ErrorSource", "MaximumSectionLength"]), offset=0),
    "_WHEA_ERROR_SOURCE_CREATE_RECORD": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_PACKET_V2", SimStruct), offset=0), SimTypePointer(SimTypeRef("WHEA_ERROR_RECORD", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ErrorSource", "ErrorPacket", "ErrorRecord", "BufferSize", "Context"]), offset=0),
    "_WHEA_ERROR_SOURCE_INITIALIZE": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WHEA_ERROR_SOURCE_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Phase", "ErrorSource", "Context"]), offset=0),
    "_WHEA_ERROR_SOURCE_RECOVER": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="WHEA_ERROR_SEVERITY"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RecoveryContext", "Severity"]), offset=0),
    "_WHEA_ERROR_SOURCE_UNINITIALIZE": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Context"]), offset=0),
    "_WHEA_SIGNAL_HANDLER_OVERRIDE_CALLBACK": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["Context"]), offset=0),
    "_WMI_e__Struct": SimStruct(OrderedDict((("ProviderId", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("DataPath", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("BufferSize", SimTypeInt(signed=False, label="UInt32")), ("Buffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_WMI_e__Struct", pack=False, align=None),
    "_WMPOCXEvents": SimTypeBottom(label="_WMPOCXEvents"),
    "_WaitWake_e__Struct": SimStruct(OrderedDict((("PowerState", SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")),)), name="_WaitWake_e__Struct", pack=False, align=None),
    "_Wake_e__Struct": SimStruct(OrderedDict((("bFlush", SimTypeInt(signed=True, label="Int32")),)), name="_Wake_e__Struct", pack=False, align=None),
    "_WindowMessage_e__Struct": SimStruct(OrderedDict((("hWnd", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("uMsg", SimTypeInt(signed=False, label="UInt32")), ("context", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="_WindowMessage_e__Struct", pack=False, align=None),
    "_WriteUncorrectableCommand_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_WriteZeroesCommand_e__Union": SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUshort": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_Write_e__Struct": SimStruct(OrderedDict((("Length", SimTypeInt(signed=False, label="UInt32")), ("Key", SimTypeInt(signed=False, label="UInt32")), ("ByteOffset", SimTypeLongLong(signed=True, label="Int64")), ("WriteBuffer", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("MdlAddress", SimTypePointer(SimTypeRef("MDL", SimStruct), offset=0)),)), name="_Write_e__Struct", pack=False, align=None),
    "_X64_e__Union": SimUnion({"Segment": SimTypeRef("_Segment_e__Struct", SimStruct), "Table": SimTypeRef("_Table_e__Struct", SimStruct), "FpControlStatus": SimTypeRef("_FpControlStatus_e__Struct", SimStruct), "XmmControlStatus": SimTypeRef("_XmmControlStatus_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_X86CpuInfo_e__Struct": SimStruct(OrderedDict((("VendorId", SimTypeArray(SimTypeInt(signed=False, label="UInt32"), 3)), ("VersionInformation", SimTypeInt(signed=False, label="UInt32")), ("FeatureInformation", SimTypeInt(signed=False, label="UInt32")), ("AMDExtendedCpuFeatures", SimTypeInt(signed=False, label="UInt32")),)), name="_X86CpuInfo_e__Struct", pack=False, align=None),
    "_XmmControlStatus_e__Struct": SimStruct(OrderedDict((("Anonymous", SimUnion({"LastFpRdp": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimStruct(OrderedDict((("LastFpDp", SimTypeInt(signed=False, label="UInt32")), ("LastFpDs", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")), ("XmmStatusControl", SimTypeInt(signed=False, label="UInt32")), ("XmmStatusControlMask", SimTypeInt(signed=False, label="UInt32")),)), name="_XmmControlStatus_e__Struct", pack=False, align=None),
    "_ZA_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeChar(label="Byte")),)), name="_ZA_e__Struct", pack=False, align=None),
    "_ZOC_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_ZOC_e__Struct", pack=False, align=None),
    "_ZONEAPPEND_e__Struct": SimStruct(OrderedDict((("CDW1011", SimTypeRef("NVME_CDW10_ZONE_APPEND", SimStruct)), ("CDW12", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW13", SimTypeInt(signed=False, label="UInt32")), ("ILBRT", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")),)), name="_ZONEAPPEND_e__Struct", pack=False, align=None),
    "_ZONEMANAGEMENTRECEIVE_e__Struct": SimStruct(OrderedDict((("CDW1011", SimTypeRef("NVME_CDW10_ZONE_MANAGEMENT_RECEIVE", SimStruct)), ("DWORDCOUNT", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_ZONEMANAGEMENTRECEIVE_e__Struct", pack=False, align=None),
    "_ZONEMANAGEMENTSEND_e__Struct": SimStruct(OrderedDict((("CDW1011", SimTypeRef("NVME_CDW10_ZONE_MANAGEMENT_SEND", SimStruct)), ("CDW12", SimTypeInt(signed=False, label="UInt32")), ("CDW13", SimUnion({"Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "AsUlong": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")), ("CDW14", SimTypeInt(signed=False, label="UInt32")), ("CDW15", SimTypeInt(signed=False, label="UInt32")),)), name="_ZONEMANAGEMENTSEND_e__Struct", pack=False, align=None),
    "_ZoneAttributes_e__Union": SimUnion({"SequentialRequiredZone": SimTypeRef("_SequentialRequiredZone_e__Struct", SimStruct), "SequentialPreferredZone": SimTypeRef("_SequentialPreferredZone_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "__DummyPinType__": SimStruct(OrderedDict((("pinType", SimTypeInt(signed=False, label="UInt32")),)), name="__DummyPinType__", pack=False, align=None),
    "__ReferenceRemainingTypes__": SimStruct(OrderedDict((("__ctrlLevel__", SimTypeInt(signed=False, label="CTRL_LEVEL")), ("__attendeeDisconnectReason__", SimTypeInt(signed=False, label="ATTENDEE_DISCONNECT_REASON")), ("__channelPriority__", SimTypeInt(signed=False, label="CHANNEL_PRIORITY")), ("__channelFlags__", SimTypeInt(signed=False, label="CHANNEL_FLAGS")), ("__channelAccessEnum__", SimTypeInt(signed=False, label="CHANNEL_ACCESS_ENUM")), ("__rdpencomapiAttendeeFlags__", SimTypeInt(signed=False, label="RDPENCOMAPI_ATTENDEE_FLAGS")), ("__rdpsrapiWndFlags__", SimTypeInt(signed=False, label="RDPSRAPI_WND_FLAGS")), ("__rdpsrapiAppFlags__", SimTypeInt(signed=False, label="RDPSRAPI_APP_FLAGS")),)), name="__ReferenceRemainingTypes__", pack=False, align=None),
    "__UPV": SimUnion({"i": SimTypeShort(signed=True, label="Int16"), "l": SimTypeInt(signed=True, label="Int32"), "ul": SimTypeInt(signed=False, label="UInt32"), "flt": SimTypeFloat(size=32), "dbl": SimTypeFloat(size=64), "b": SimTypeShort(signed=False, label="UInt16"), "cur": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), "at": SimTypeFloat(size=64), "ft": SimTypeRef("FILETIME", SimStruct), "lpszA": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "bin": SimTypeRef("SBinary", SimStruct), "lpszW": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lpguid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "li": SimTypeLongLong(signed=True, label="Int64"), "MVi": SimTypeRef("SShortArray", SimStruct), "MVl": SimTypeRef("SLongArray", SimStruct), "MVflt": SimTypeRef("SRealArray", SimStruct), "MVdbl": SimTypeRef("SDoubleArray", SimStruct), "MVcur": SimTypeRef("SCurrencyArray", SimStruct), "MVat": SimTypeRef("SAppTimeArray", SimStruct), "MVft": SimTypeRef("SDateTimeArray", SimStruct), "MVbin": SimTypeRef("SBinaryArray", SimStruct), "MVszA": SimTypeRef("SLPSTRArray", SimStruct), "MVszW": SimTypeRef("SWStringArray", SimStruct), "MVguid": SimTypeRef("SGuidArray", SimStruct), "MVli": SimTypeRef("SLargeIntegerArray", SimStruct), "err": SimTypeInt(signed=True, label="Int32"), "x": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"),
    "__VDS_PARTITION_STYLE": SimTypeInt(signed=False, label="__VDS_PARTITION_STYLE"),
    "__mbnapi_ReferenceRemainingTypes__": SimStruct(OrderedDict((("bandClass", SimTypeInt(signed=False, label="MBN_BAND_CLASS")), ("contextConstants", SimTypeInt(signed=False, label="MBN_CONTEXT_CONSTANTS")), ("ctrlCaps", SimTypeInt(signed=False, label="MBN_CTRL_CAPS")), ("dataClass", SimTypeInt(signed=False, label="MBN_DATA_CLASS")), ("interfaceCapsConstants", SimTypeInt(signed=False, label="MBN_INTERFACE_CAPS_CONSTANTS")), ("pinConstants", SimTypeInt(signed=False, label="MBN_PIN_CONSTANTS")), ("providerConstants", SimTypeInt(signed=False, label="MBN_PROVIDER_CONSTANTS")), ("providerState", SimTypeInt(signed=False, label="MBN_PROVIDER_STATE")), ("registrationConstants", SimTypeInt(signed=False, label="MBN_REGISTRATION_CONSTANTS")), ("signalConstants", SimTypeInt(signed=False, label="MBN_SIGNAL_CONSTANTS")), ("smsCaps", SimTypeInt(signed=False, label="MBN_SMS_CAPS")), ("smsConstants", SimTypeInt(signed=False, label="WWAEXT_SMS_CONSTANTS")), ("wwaextSmsConstants", SimTypeInt(signed=False, label="WWAEXT_SMS_CONSTANTS")), ("smsStatusFlag", SimTypeInt(signed=False, label="MBN_SMS_STATUS_FLAG")),)), name="__mbnapi_ReferenceRemainingTypes__", pack=False, align=None),
    "__msxml6_ReferenceRemainingTypes__": SimStruct(OrderedDict((("__tagDomNodeType__", SimTypeInt(signed=False, label="DOMNodeType")), ("__domNodeType__", SimTypeInt(signed=False, label="DOMNodeType")), ("__serverXmlHttpOptionEnum__", SimTypeInt(signed=False, label="SERVERXMLHTTP_OPTION")), ("__serverXmlHttpOption__", SimTypeInt(signed=False, label="SERVERXMLHTTP_OPTION")), ("__serverCertOptionEnum__", SimTypeInt(signed=False, label="SXH_SERVER_CERT_OPTION")), ("__serverCertOption__", SimTypeInt(signed=False, label="SXH_SERVER_CERT_OPTION")), ("__proxySettingEnum__", SimTypeInt(signed=False, label="SXH_PROXY_SETTING")), ("__proxySetting__", SimTypeInt(signed=False, label="SXH_PROXY_SETTING")), ("__somItemTypeEnum__", SimTypeInt(signed=False, label="SOMITEMTYPE")), ("__somItemType__", SimTypeInt(signed=False, label="SOMITEMTYPE")), ("__schemaUseEnum__", SimTypeInt(signed=False, label="SCHEMAUSE")), ("__schemaUse__", SimTypeInt(signed=False, label="SCHEMAUSE")), ("__schemaDerivationMethodEnum__", SimTypeInt(signed=False, label="SCHEMADERIVATIONMETHOD")), ("__schemaDerivationMethod__", SimTypeInt(signed=False, label="SCHEMADERIVATIONMETHOD")), ("__schemaContentTypeEnum__", SimTypeInt(signed=False, label="SCHEMACONTENTTYPE")), ("__schemaContentType__", SimTypeInt(signed=False, label="SCHEMACONTENTTYPE")), ("__schemaProcessContentsEnum__", SimTypeInt(signed=False, label="SCHEMAPROCESSCONTENTS")), ("__schemaProcessContents__", SimTypeInt(signed=False, label="SCHEMAPROCESSCONTENTS")), ("__schemaWhitespaceEnum__", SimTypeInt(signed=False, label="SCHEMAWHITESPACE")), ("__schemaWhitespace__", SimTypeInt(signed=False, label="SCHEMAWHITESPACE")), ("__schemaTypeVarietyEnum__", SimTypeInt(signed=False, label="SCHEMATYPEVARIETY")), ("__schemaTypeVariety__", SimTypeInt(signed=False, label="SCHEMATYPEVARIETY")),)), name="__msxml6_ReferenceRemainingTypes__", pack=False, align=None),
    "_address_e__Union": SimUnion({"alias": SimTypeRef("ADDRALIAS", SimStruct), "rgchInterNet": SimTypePointer(SimTypeChar(label="SByte"), offset=0)}, name="<anon>", label="None"),
    "_alternative_e__Struct": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="_alternative_e__Struct", pack=False, align=None),
    "_arguments_e__Union": SimUnion({"manipulation": SimTypeRef("INTERACTION_ARGUMENTS_MANIPULATION", SimStruct), "tap": SimTypeRef("INTERACTION_ARGUMENTS_TAP", SimStruct), "crossSlide": SimTypeRef("INTERACTION_ARGUMENTS_CROSS_SLIDE", SimStruct)}, name="<anon>", label="None"),
    "_asnValue_e__Union": SimUnion({"number": SimTypeInt(signed=True, label="Int32"), "unsigned32": SimTypeInt(signed=False, label="UInt32"), "counter64": SimTypeLongLong(signed=False, label="UInt64"), "string": SimTypeRef("AsnOctetString", SimStruct), "bits": SimTypeRef("AsnOctetString", SimStruct), "object": SimTypeRef("AsnObjectIdentifier", SimStruct), "sequence": SimTypeRef("AsnOctetString", SimStruct), "address": SimTypeRef("AsnOctetString", SimStruct), "counter": SimTypeInt(signed=False, label="UInt32"), "gauge": SimTypeInt(signed=False, label="UInt32"), "ticks": SimTypeInt(signed=False, label="UInt32"), "arbitrary": SimTypeRef("AsnOctetString", SimStruct)}, name="<anon>", label="None"),
    "_avifieldindex_entry": SimStruct(OrderedDict((("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwOffsetField2", SimTypeInt(signed=False, label="UInt32")),)), name="_avifieldindex_entry", pack=False, align=None),
    "_avioldindex_entry": SimStruct(OrderedDict((("dwChunkId", SimTypeInt(signed=False, label="UInt32")), ("dwFlags", SimTypeInt(signed=False, label="UInt32")), ("dwOffset", SimTypeInt(signed=False, label="UInt32")), ("dwSize", SimTypeInt(signed=False, label="UInt32")),)), name="_avioldindex_entry", pack=False, align=None),
    "_avisuperindex_entry": SimStruct(OrderedDict((("qwOffset", SimTypeLongLong(signed=False, label="UInt64")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("dwDuration", SimTypeInt(signed=False, label="UInt32")),)), name="_avisuperindex_entry", pack=False, align=None),
    "_bVal_e__Struct": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("pbVal", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_bVal_e__Struct", pack=False, align=None),
    "_bitmap_e__Union": SimUnion({"hdc": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pDDS": SimTypeBottom(label="IDirect3DSurface9")}, name="<anon>", label="None"),
    "_bits_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_bits_e__Struct", pack=False, align=None),
    "_bmAttributes_e__Union": SimUnion({"AsUchar": SimTypeChar(label="Byte"), "Bulk": SimTypeRef("_Bulk_e__Struct", SimStruct), "Isochronous": SimTypeRef("_Isochronous_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_bmCapabilities_e__Union": SimUnion({"AsUshort": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "_bmp_e__Struct": SimStruct(OrderedDict((("hbitmap", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("hpal", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_bmp_e__Struct", pack=False, align=None),
    "_bootstrapAddresses_e__Struct": SimStruct(OrderedDict((("cntAddress", SimTypeInt(signed=False, label="UInt32")), ("pAddresses", SimTypePointer(SimTypeRef("SOCKADDR_STORAGE", SimStruct), offset=0)),)), name="_bootstrapAddresses_e__Struct", pack=False, align=None),
    "_bvp": SimStruct(OrderedDict((("pVolumeUnk", SimTypeBottom(label="IUnknown")),)), name="_bvp", pack=False, align=None),
    "_cabinet_e__Struct": SimStruct(OrderedDict((("pHeaderReserve", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbHeaderReserve", SimTypeShort(signed=False, label="UInt16")), ("setID", SimTypeShort(signed=False, label="UInt16")), ("iCabinet", SimTypeInt(signed=True, label="Int32")),)), name="_cabinet_e__Struct", pack=False, align=None),
    "_cl": SimStruct(OrderedDict((("pLunUnk", SimTypeBottom(label="IUnknown")),)), name="_cl", pack=False, align=None),
    "_combined_e__Struct": SimStruct(OrderedDict((("pHandle", SimTypeInt(signed=False, label="UInt32")), ("fHandle", SimTypeInt(signed=False, label="UInt32")), ("uPayload", SimTypeLongLong(signed=False, label="UInt64")),)), name="_combined_e__Struct", pack=False, align=None),
    "_connectivity_e__Struct": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="NLA_CONNECTIVITY_TYPE")), ("internet", SimTypeInt(signed=False, label="NLA_INTERNET")),)), name="_connectivity_e__Struct", pack=False, align=None),
    "_context_e__Struct": SimStruct(OrderedDict((("channelCount", SimTypeChar(label="Byte")), ("channels", SimTypeArray(SimTypeFloat(size=32), 9)),)), name="_context_e__Struct", pack=False, align=None),
    "_cp": SimStruct(OrderedDict((("ullOffset", SimTypeLongLong(signed=False, label="UInt64")), ("volumeId", SimTypeBottom(label="Guid")),)), name="_cp", pack=False, align=None),
    "_cpg": SimStruct(OrderedDict((("pPortalGroupUnk", SimTypeBottom(label="IUnknown")),)), name="_cpg", pack=False, align=None),
    "_credentials_e__Union": SimUnion({"usernamePassword": SimTypeRef("MI_UsernamePasswordCreds", SimStruct), "certificateThumbprint": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="<anon>", label="None"),
    "_ct": SimStruct(OrderedDict((("pTargetUnk", SimTypeBottom(label="IUnknown")),)), name="_ct", pack=False, align=None),
    "_ctl_e__Union": SimUnion({"lpv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "lplabel": SimTypePointer(SimTypeRef("DTBLLABEL", SimStruct), offset=0), "lpedit": SimTypePointer(SimTypeRef("DTBLEDIT", SimStruct), offset=0), "lplbx": SimTypePointer(SimTypeRef("DTBLLBX", SimStruct), offset=0), "lpcombobox": SimTypePointer(SimTypeRef("DTBLCOMBOBOX", SimStruct), offset=0), "lpddlbx": SimTypePointer(SimTypeRef("DTBLDDLBX", SimStruct), offset=0), "lpcheckbox": SimTypePointer(SimTypeRef("DTBLCHECKBOX", SimStruct), offset=0), "lpgroupbox": SimTypePointer(SimTypeRef("DTBLGROUPBOX", SimStruct), offset=0), "lpbutton": SimTypePointer(SimTypeRef("DTBLBUTTON", SimStruct), offset=0), "lpradiobutton": SimTypePointer(SimTypeRef("DTBLRADIOBUTTON", SimStruct), offset=0), "lpmvlbx": SimTypePointer(SimTypeRef("DTBLMVLISTBOX", SimStruct), offset=0), "lpmvddlbx": SimTypePointer(SimTypeRef("DTBLMVDDLBX", SimStruct), offset=0), "lppage": SimTypePointer(SimTypeRef("DTBLPAGE", SimStruct), offset=0)}, name="<anon>", label="None"),
    "_customSchemaType": SimStruct(OrderedDict((("InType", SimTypeShort(signed=False, label="UInt16")), ("OutType", SimTypeShort(signed=False, label="UInt16")), ("CustomSchemaOffset", SimTypeInt(signed=False, label="UInt32")),)), name="_customSchemaType", pack=False, align=None),
    "_cv": SimStruct(OrderedDict((("pVolumeUnk", SimTypeBottom(label="IUnknown")),)), name="_cv", pack=False, align=None),
    "_cvd": SimStruct(OrderedDict((("pVDiskUnk", SimTypeBottom(label="IUnknown")),)), name="_cvd", pack=False, align=None),
    "_data_e__Union": SimUnion({"mouse": SimTypeRef("RAWMOUSE", SimStruct), "keyboard": SimTypeRef("RAWKEYBOARD", SimStruct), "hid": SimTypeRef("RAWHID", SimStruct)}, name="<anon>", label="None"),
    "_decorative_e__Struct": SimStruct(OrderedDict((("familyKind", SimTypeChar(label="Byte")), ("decorativeClass", SimTypeChar(label="Byte")), ("weight", SimTypeChar(label="Byte")), ("aspect", SimTypeChar(label="Byte")), ("contrast", SimTypeChar(label="Byte")), ("serifVariant", SimTypeChar(label="Byte")), ("fill", SimTypeChar(label="Byte")), ("lining", SimTypeChar(label="Byte")), ("decorativeTopology", SimTypeChar(label="Byte")), ("characterRange", SimTypeChar(label="Byte")),)), name="_decorative_e__Struct", pack=False, align=None),
    "_decrypt_e__Struct": SimStruct(OrderedDict((("pDataReserve", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbDataReserve", SimTypeShort(signed=False, label="UInt16")), ("pbData", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbData", SimTypeShort(signed=False, label="UInt16")), ("fSplit", SimTypeInt(signed=True, label="Int32")), ("cbPartial", SimTypeShort(signed=False, label="UInt16")),)), name="_decrypt_e__Struct", pack=False, align=None),
    "_emf_e__Struct": SimStruct(OrderedDict((("hemf", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_emf_e__Struct", pack=False, align=None),
    "_errs_u_e__Union": SimUnion({"errs_ipv4": SimTypeRef("Error_Spec_IPv4", SimStruct)}, name="<anon>", label="None"),
    "_filt_u_e__Union": SimUnion({"filt_ipv4": SimTypeRef("Filter_Spec_IPv4", SimStruct), "filt_ipv4gpi": SimTypeRef("Filter_Spec_IPv4GPI", SimStruct)}, name="<anon>", label="None"),
    "_folder_e__Struct": SimStruct(OrderedDict((("pFolderReserve", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("cbFolderReserve", SimTypeShort(signed=False, label="UInt16")), ("iFolder", SimTypeShort(signed=False, label="UInt16")),)), name="_folder_e__Struct", pack=False, align=None),
    "_header_e__Struct": SimStruct(OrderedDict((("type", SimTypeInt(signed=False, label="NLA_BLOB_DATA_TYPE")), ("dwSize", SimTypeInt(signed=False, label="UInt32")), ("nextOffset", SimTypeInt(signed=False, label="UInt32")),)), name="_header_e__Struct", pack=False, align=None),
    "_hop_u_e__Union": SimUnion({"hop_ipv4": SimTypeRef("Rsvp_Hop_IPv4", SimStruct)}, name="<anon>", label="None"),
    "_icon_e__Struct": SimStruct(OrderedDict((("hicon", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="_icon_e__Struct", pack=False, align=None),
    "_in_e__Struct": SimStruct(OrderedDict((("Location", SimTypeInt(signed=False, label="NETWORK_OPEN_LOCATION_QUALIFIER")), ("Integrity", SimTypeInt(signed=False, label="NETWORK_OPEN_INTEGRITY_QUALIFIER")),)), name="_in_e__Struct", pack=False, align=None),
    "_info_e__Union": SimUnion({"err": SimTypeRef("ERROR_NOTIFICATION", SimStruct), "newmail": SimTypeRef("NEWMAIL_NOTIFICATION", SimStruct), "obj": SimTypeRef("OBJECT_NOTIFICATION", SimStruct), "tab": SimTypeRef("TABLE_NOTIFICATION", SimStruct), "ext": SimTypeRef("EXTENDED_NOTIFICATION", SimStruct), "statobj": SimTypeRef("STATUS_OBJECT_NOTIFICATION", SimStruct)}, name="<anon>", label="None"),
    "_interfaceData_e__Struct": SimStruct(OrderedDict((("dwType", SimTypeInt(signed=False, label="UInt32")), ("dwSpeed", SimTypeInt(signed=False, label="UInt32")), ("adapterName", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="_interfaceData_e__Struct", pack=False, align=None),
    "_intval_e__Union": SimUnion({"year_month": SimTypeRef("SQL_YEAR_MONTH_STRUCT", SimStruct), "day_second": SimTypeRef("SQL_DAY_SECOND_STRUCT", SimStruct)}, name="<anon>", label="None"),
    "_ipv4_e__Struct": SimStruct(OrderedDict((("sin_port", SimTypeShort(signed=False, label="UInt16")), ("IN_ADDR", SimTypeInt(signed=False, label="UInt32")), ("sin_zero", SimTypeArray(SimTypeChar(label="Byte"), 8)),)), name="_ipv4_e__Struct", pack=False, align=None),
    "_ipv6_e__Struct": SimStruct(OrderedDict((("sin6_port", SimTypeShort(signed=False, label="UInt16")), ("sin6_flowinfo", SimTypeInt(signed=False, label="UInt32")), ("sin6_addr", SimTypeArray(SimTypeShort(signed=False, label="UInt16"), 8)), ("sin6_scope_id", SimTypeInt(signed=False, label="UInt32")),)), name="_ipv6_e__Struct", pack=False, align=None),
    "_irdaAttribOctetSeq_e__Struct": SimStruct(OrderedDict((("Len", SimTypeInt(signed=False, label="UInt32")), ("OctetSeq", SimTypeArray(SimTypeChar(label="Byte"), 1024)),)), name="_irdaAttribOctetSeq_e__Struct", pack=False, align=None),
    "_irdaAttribUsrStr_e__Struct": SimStruct(OrderedDict((("Len", SimTypeInt(signed=False, label="UInt32")), ("CharSet", SimTypeInt(signed=False, label="UInt32")), ("UsrStr", SimTypeArray(SimTypeChar(label="Byte"), 256)),)), name="_irdaAttribUsrStr_e__Struct", pack=False, align=None),
    "_irdaAttribute_e__Union": SimUnion({"irdaAttribInt": SimTypeInt(signed=True, label="Int32"), "irdaAttribOctetSeq": SimTypeRef("_irdaAttribOctetSeq_e__Struct", SimStruct), "irdaAttribUsrStr": SimTypeRef("_irdaAttribUsrStr_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_ld_sb_e__Struct": SimStruct(OrderedDict((("sb_sd", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved1", SimTypeArray(SimTypeChar(label="Byte"), 41)), ("sb_naddr", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("Reserved2", SimTypeArray(SimTypeChar(label="Byte"), 24)),)), name="_ld_sb_e__Struct", pack=False, align=None),
    "_leafsetKeyChange_e__Struct": SimStruct(OrderedDict((("change", SimTypeInt(signed=False, label="DRT_LEAFSET_KEY_CHANGE_TYPE")), ("localKey", SimTypeRef("DRT_DATA", SimStruct)), ("remoteKey", SimTypeRef("DRT_DATA", SimStruct)),)), name="_leafsetKeyChange_e__Struct", pack=False, align=None),
    "_lfFont_e__Union": SimUnion({"A": SimTypeRef("LOGFONTA", SimStruct), "W": SimTypeRef("LOGFONTW", SimStruct)}, name="<anon>", label="None"),
    "_locationData_e__Struct": SimStruct(OrderedDict((("information", SimTypePointer(SimTypeChar(label="SByte"), offset=0)),)), name="_locationData_e__Struct", pack=False, align=None),
    "_midi_e__Struct": SimStruct(OrderedDict((("songptrpos", SimTypeInt(signed=False, label="UInt32")),)), name="_midi_e__Struct", pack=False, align=None),
    "_mod_vals_e__Union": SimUnion({"modv_strvals": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), "modv_bvals": SimTypePointer(SimTypePointer(SimTypeRef("LDAP_BERVAL", SimStruct), offset=0), offset=0)}, name="<anon>", label="None"),
    "_nonStructType": SimStruct(OrderedDict((("InType", SimTypeShort(signed=False, label="UInt16")), ("OutType", SimTypeShort(signed=False, label="UInt16")), ("MapNameOffset", SimTypeInt(signed=False, label="UInt32")),)), name="_nonStructType", pack=False, align=None),
    "_out_e__Struct": SimStruct(OrderedDict((("assertionValue", SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)),)), name="_out_e__Struct", pack=False, align=None),
    "_positionInfo_e__Struct": SimStruct(OrderedDict((("numvSwitchFilterExtensions", SimTypeInt(signed=False, label="UInt32")), ("vSwitchFilterExtensions", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="_positionInfo_e__Struct", pack=False, align=None),
    "_propertyBlob_e__Struct": SimStruct(OrderedDict((("cbBuf", SimTypeInt(signed=False, label="UInt32")), ("pBuf", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="_propertyBlob_e__Struct", pack=False, align=None),
    "_rcFrame_e__Struct": SimStruct(OrderedDict((("left", SimTypeShort(signed=True, label="Int16")), ("top", SimTypeShort(signed=True, label="Int16")), ("right", SimTypeShort(signed=True, label="Int16")), ("bottom", SimTypeShort(signed=True, label="Int16")),)), name="_rcFrame_e__Struct", pack=False, align=None),
    "_registrationStateChange_e__Struct": SimStruct(OrderedDict((("state", SimTypeInt(signed=False, label="DRT_REGISTRATION_STATE")), ("localKey", SimTypeRef("DRT_DATA", SimStruct)),)), name="_registrationStateChange_e__Struct", pack=False, align=None),
    "_remote_e__Struct": SimStruct(OrderedDict((("speed", SimTypeInt(signed=False, label="UInt32")), ("type", SimTypeInt(signed=False, label="UInt32")), ("state", SimTypeInt(signed=False, label="UInt32")), ("machineName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("sharedAdapterName", SimTypeArray(SimTypeChar(label="Char"), 256)),)), name="_remote_e__Struct", pack=False, align=None),
    "_reorderInfo_e__Struct": SimStruct(OrderedDict((("inRequiredPosition", SimTypeInt(signed=True, label="Int32")), ("numvSwitchFilterExtensions", SimTypeInt(signed=False, label="UInt32")), ("vSwitchFilterExtensions", SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)),)), name="_reorderInfo_e__Struct", pack=False, align=None),
    "_res_e__Union": SimUnion({"resCompareProps": SimTypeRef("SComparePropsRestriction", SimStruct), "resAnd": SimTypeRef("SAndRestriction", SimStruct), "resOr": SimTypeRef("SOrRestriction", SimStruct), "resNot": SimTypeRef("SNotRestriction", SimStruct), "resContent": SimTypeRef("SContentRestriction", SimStruct), "resProperty": SimTypeRef("SPropertyRestriction", SimStruct), "resBitMask": SimTypeRef("SBitMaskRestriction", SimStruct), "resSize": SimTypeRef("SSizeRestriction", SimStruct), "resExist": SimTypeRef("SExistRestriction", SimStruct), "resSub": SimTypeRef("SSubRestriction", SimStruct), "resComment": SimTypeRef("SCommentRestriction", SimStruct)}, name="<anon>", label="None"),
    "_sRGB_e__Struct": SimStruct(OrderedDict((("alpha", SimTypeChar(label="Byte")), ("red", SimTypeChar(label="Byte")), ("green", SimTypeChar(label="Byte")), ("blue", SimTypeChar(label="Byte")),)), name="_sRGB_e__Struct", pack=False, align=None),
    "_s_e__Struct": SimStruct(OrderedDict((("_bitfield", SimTypeInt(signed=False, label="UInt32")),)), name="_s_e__Struct", pack=False, align=None),
    "_scRGB_e__Struct": SimStruct(OrderedDict((("alpha", SimTypeFloat(size=32)), ("red", SimTypeFloat(size=32)), ("green", SimTypeFloat(size=32)), ("blue", SimTypeFloat(size=32)),)), name="_scRGB_e__Struct", pack=False, align=None),
    "_scope_u_e__Union": SimUnion({"scopl_ipv4": SimTypeRef("Scope_list_ipv4", SimStruct)}, name="<anon>", label="None"),
    "_script_e__Struct": SimStruct(OrderedDict((("familyKind", SimTypeChar(label="Byte")), ("toolKind", SimTypeChar(label="Byte")), ("weight", SimTypeChar(label="Byte")), ("spacing", SimTypeChar(label="Byte")), ("aspectRatio", SimTypeChar(label="Byte")), ("contrast", SimTypeChar(label="Byte")), ("scriptTopology", SimTypeChar(label="Byte")), ("scriptForm", SimTypeChar(label="Byte")), ("finials", SimTypeChar(label="Byte")), ("xAscent", SimTypeChar(label="Byte")),)), name="_script_e__Struct", pack=False, align=None),
    "_sequence_e__Struct": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="_sequence_e__Struct", pack=False, align=None),
    "_sess_u_e__Union": SimUnion({"sess_ipv4": SimTypeRef("Session_IPv4", SimStruct)}, name="<anon>", label="None"),
    "_smpte_e__Struct": SimStruct(OrderedDict((("hour", SimTypeChar(label="Byte")), ("min", SimTypeChar(label="Byte")), ("sec", SimTypeChar(label="Byte")), ("frame", SimTypeChar(label="Byte")), ("fps", SimTypeChar(label="Byte")), ("dummy", SimTypeChar(label="Byte")), ("pad", SimTypeArray(SimTypeChar(label="Byte"), 2)),)), name="_smpte_e__Struct", pack=False, align=None),
    "_spec_u_e__Union": SimUnion({"CL_spec": SimTypeRef("CtrlLoadFlowspec", SimStruct), "G_spec": SimTypeRef("GuarFlowSpec", SimStruct), "Q_spec": SimTypeRef("QualAppFlowSpec", SimStruct)}, name="<anon>", label="None"),
    "_statusChange_e__Struct": SimStruct(OrderedDict((("status", SimTypeInt(signed=False, label="DRT_STATUS")), ("bootstrapAddresses", SimTypeRef("_bootstrapAddresses_e__Struct", SimStruct)),)), name="_statusChange_e__Struct", pack=False, align=None),
    "_strVal_e__Struct": SimStruct(OrderedDict((("size", SimTypeInt(signed=False, label="UInt32")), ("pstrVal", SimTypePointer(SimTypeChar(label="Char"), offset=0)),)), name="_strVal_e__Struct", pack=False, align=None),
    "_string_e__Struct": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="_string_e__Struct", pack=False, align=None),
    "_structType": SimStruct(OrderedDict((("StructStartIndex", SimTypeShort(signed=False, label="UInt16")), ("NumOfStructMembers", SimTypeShort(signed=False, label="UInt16")), ("padding", SimTypeInt(signed=False, label="UInt32")),)), name="_structType", pack=False, align=None),
    "_sv": SimStruct(OrderedDict((("ullReclaimedBytes", SimTypeLongLong(signed=False, label="UInt64")),)), name="_sv", pack=False, align=None),
    "_symbol_e__Struct": SimStruct(OrderedDict((("familyKind", SimTypeChar(label="Byte")), ("symbolKind", SimTypeChar(label="Byte")), ("weight", SimTypeChar(label="Byte")), ("spacing", SimTypeChar(label="Byte")), ("aspectRatioAndContrast", SimTypeChar(label="Byte")), ("aspectRatio94", SimTypeChar(label="Byte")), ("aspectRatio119", SimTypeChar(label="Byte")), ("aspectRatio157", SimTypeChar(label="Byte")), ("aspectRatio163", SimTypeChar(label="Byte")), ("aspectRatio211", SimTypeChar(label="Byte")),)), name="_symbol_e__Struct", pack=False, align=None),
    "_tagged_union_e__Struct": SimUnion({"clsid": SimTypeBottom(label="Guid"), "pFileExt": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pMimeType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pProgId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ByName": SimTypeRef("_ByName_e__Struct", SimStruct), "ByObjectId": SimTypeRef("_ByObjectId_e__Struct", SimStruct)}, name="<anon>", label="None"),
    "_text_e__Struct": SimStruct(OrderedDict((("familyKind", SimTypeChar(label="Byte")), ("serifStyle", SimTypeChar(label="Byte")), ("weight", SimTypeChar(label="Byte")), ("proportion", SimTypeChar(label="Byte")), ("contrast", SimTypeChar(label="Byte")), ("strokeVariation", SimTypeChar(label="Byte")), ("armStyle", SimTypeChar(label="Byte")), ("letterform", SimTypeChar(label="Byte")), ("midline", SimTypeChar(label="Byte")), ("xHeight", SimTypeChar(label="Byte")),)), name="_text_e__Struct", pack=False, align=None),
    "_tspec_u_e__Union": SimUnion({"gen_stspec": SimTypeRef("GenTspec", SimStruct), "qual_stspec": SimTypeRef("QualTspec", SimStruct)}, name="<anon>", label="None"),
    "_u1_e__Union": SimUnion({"ForwarderString": SimTypeInt(signed=False, label="UInt32"), "Function": SimTypeInt(signed=False, label="UInt32"), "Ordinal": SimTypeInt(signed=False, label="UInt32"), "AddressOfData": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_u2_e__Union": SimUnion({"stemp": SimTypePointer(SimTypeRef("FILTER_SPEC", SimStruct), offset=0), "fspec": SimTypePointer(SimTypeRef("FILTER_SPEC", SimStruct), offset=0)}, name="<anon>", label="None"),
    "_uChar_e__Union": SimUnion({"UnicodeChar": SimTypeChar(label="Char"), "AsciiChar": SimTypeChar(label="SByte")}, name="<anon>", label="None"),
    "_uCode_e__Union": SimUnion({"sCode": SimTypeShort(signed=True, label="Int16"), "ubCode": SimTypeChar(label="Byte"), "ubPairs": SimTypeArray(SimTypeChar(label="Byte"), 2)}, name="<anon>", label="None"),
    "_uGuid_e__Union": SimUnion({"guid": SimTypeBottom(label="Guid"), "pguid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="<anon>", label="None"),
    "_uName_e__Union": SimUnion({"pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ulPropid": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"),
    "_u_e__Struct": SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"),
    "_u_e__Union": SimUnion({"timestamp": SimTypeRef("MI_Timestamp", SimStruct), "interval": SimTypeRef("MI_Interval", SimStruct)}, name="<anon>", label="None"),
    "_url_e__Struct": SimStruct(OrderedDict((("value", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("length", SimTypeInt(signed=False, label="UInt32")),)), name="_url_e__Struct", pack=False, align=None),
    "_value_e__Union": SimUnion({"sNumber": SimTypeInt(signed=True, label="Int32"), "uNumber": SimTypeInt(signed=False, label="UInt32"), "hNumber": SimTypeRef("smiCNTR64", SimStruct), "string": SimTypeRef("smiOCTETS", SimStruct), "oid": SimTypeRef("smiOID", SimStruct), "empty": SimTypeChar(label="Byte")}, name="<anon>", label="None"),
    "_wFunctionalitySupport_e__Union": SimUnion({"AsUshort": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimStruct(OrderedDict((("_bitfield", SimTypeShort(signed=False, label="UInt16")),)), name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"),
    "_wIndex": SimUnion({"Anonymous": SimStruct(OrderedDict((("LowByte", SimTypeChar(label="Byte")), ("HiByte", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_wValue": SimUnion({"Anonymous": SimStruct(OrderedDict((("LowByte", SimTypeChar(label="Byte")), ("HiByte", SimTypeChar(label="Byte")),)), name="_Anonymous_e__Struct", pack=False, align=None), "W": SimTypeShort(signed=False, label="UInt16")}, name="<anon>", label="None"),
    "_wireBRECORD": SimStruct(OrderedDict((("fFlags", SimTypeInt(signed=False, label="UInt32")), ("clSize", SimTypeInt(signed=False, label="UInt32")), ("pRecInfo", SimTypeBottom(label="IRecordInfo")), ("pRecord", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="_wireBRECORD", pack=False, align=None),
    "_wireSAFEARRAY": SimStruct(OrderedDict((("cDims", SimTypeShort(signed=False, label="UInt16")), ("fFeatures", SimTypeShort(signed=False, label="UInt16")), ("cbElements", SimTypeInt(signed=False, label="UInt32")), ("cLocks", SimTypeInt(signed=False, label="UInt32")), ("uArrayStructs", SimTypeRef("SAFEARRAYUNION", SimStruct)), ("rgsabound", SimTypePointer(SimTypeRef("SAFEARRAYBOUND", SimStruct), offset=0)),)), name="_wireSAFEARRAY", pack=False, align=None),
    "_wireVARIANT": SimStruct(OrderedDict((("clSize", SimTypeInt(signed=False, label="UInt32")), ("rpcReserved", SimTypeInt(signed=False, label="UInt32")), ("vt", SimTypeShort(signed=False, label="UInt16")), ("wReserved1", SimTypeShort(signed=False, label="UInt16")), ("wReserved2", SimTypeShort(signed=False, label="UInt16")), ("wReserved3", SimTypeShort(signed=False, label="UInt16")), ("Anonymous", SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeRef("FLAGGED_WORD_BLOB", SimStruct), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimTypePointer(SimTypeRef("_wireSAFEARRAY", SimStruct), offset=0), offset=0), "brecVal": SimTypePointer(SimTypeRef("_wireBRECORD", SimStruct), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimUnion({"Anonymous": SimStruct(OrderedDict((("Lo", SimTypeInt(signed=False, label="UInt32")), ("Hi", SimTypeInt(signed=True, label="Int32")),)), name="_Anonymous_e__Struct", pack=False, align=None), "int64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeRef("FLAGGED_WORD_BLOB", SimStruct), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimTypePointer(SimTypeRef("_wireSAFEARRAY", SimStruct), offset=0), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypePointer(SimTypeRef("_wireVARIANT", SimStruct), offset=0), offset=0), "cVal": SimTypeChar(label="SByte"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "decVal": SimTypeRef("DECIMAL", SimStruct), "pdecVal": SimTypePointer(SimTypeRef("DECIMAL", SimStruct), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="<anon>", label="None")),)), name="_wireVARIANT", pack=False, align=None),
    "_wmf_e__Struct": SimStruct(OrderedDict((("hmeta", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xExt", SimTypeInt(signed=True, label="Int32")), ("yExt", SimTypeInt(signed=True, label="Int32")),)), name="_wmf_e__Struct", pack=False, align=None),
    "alljoyn_about_announced_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "busName", "version", "port", "objectDescriptionArg", "aboutDataArg"]), offset=0),
    "alljoyn_about_announceflag": SimTypeInt(signed=False, label="alljoyn_about_announceflag"),
    "alljoyn_aboutdata": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_aboutdatalistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_aboutdatalistener_callbacks": SimStruct(OrderedDict((("about_datalistener_getaboutdata", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "msgArg", "language"]), offset=0)), ("about_datalistener_getannouncedaboutdata", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "msgArg"]), offset=0)),)), name="alljoyn_aboutdatalistener_callbacks", pack=False, align=None),
    "alljoyn_aboutdatalistener_getaboutdata_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "msgArg", "language"]), offset=0),
    "alljoyn_aboutdatalistener_getannouncedaboutdata_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "msgArg"]), offset=0),
    "alljoyn_abouticon": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_abouticonobj": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_abouticonproxy": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_aboutlistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_aboutlistener_callback": SimStruct(OrderedDict((("about_listener_announced", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "busName", "version", "port", "objectDescriptionArg", "aboutDataArg"]), offset=0)),)), name="alljoyn_aboutlistener_callback", pack=False, align=None),
    "alljoyn_aboutobj": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_aboutobjectdescription": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_aboutproxy": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_applicationstate": SimTypeInt(signed=False, label="alljoyn_applicationstate"),
    "alljoyn_applicationstatelistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_applicationstatelistener_callbacks": SimStruct(OrderedDict((("state", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="alljoyn_applicationstate"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["busName", "publicKey", "applicationState", "context"]), offset=0)),)), name="alljoyn_applicationstatelistener_callbacks", pack=False, align=None),
    "alljoyn_applicationstatelistener_state_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0), SimTypeInt(signed=False, label="alljoyn_applicationstate"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["busName", "publicKey", "applicationState", "context"]), offset=0),
    "alljoyn_authlistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_authlistener_authenticationcomplete_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["context", "authMechanism", "peerName", "success"]), offset=0),
    "alljoyn_authlistener_callbacks": SimStruct(OrderedDict((("request_credentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "authMechanism", "peerName", "authCount", "userName", "credMask", "credentials"]), offset=0)), ("verify_credentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "authMechanism", "peerName", "credentials"]), offset=0)), ("security_violation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "status", "msg"]), offset=0)), ("authentication_complete", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["context", "authMechanism", "peerName", "success"]), offset=0)),)), name="alljoyn_authlistener_callbacks", pack=False, align=None),
    "alljoyn_authlistener_requestcredentials_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "authMechanism", "peerName", "authCount", "userName", "credMask", "credentials"]), offset=0),
    "alljoyn_authlistener_requestcredentialsasync_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "authMechanism", "peerName", "authCount", "userName", "credMask", "authContext"]), offset=0),
    "alljoyn_authlistener_securityviolation_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "status", "msg"]), offset=0),
    "alljoyn_authlistener_verifycredentials_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "authMechanism", "peerName", "credentials"]), offset=0),
    "alljoyn_authlistener_verifycredentialsasync_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "authMechanism", "peerName", "credentials", "authContext"]), offset=0),
    "alljoyn_authlistenerasync_callbacks": SimStruct(OrderedDict((("request_credentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "authMechanism", "peerName", "authCount", "userName", "credMask", "authContext"]), offset=0)), ("verify_credentials", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "authMechanism", "peerName", "credentials", "authContext"]), offset=0)), ("security_violation", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "status", "msg"]), offset=0)), ("authentication_complete", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["context", "authMechanism", "peerName", "success"]), offset=0)),)), name="alljoyn_authlistenerasync_callbacks", pack=False, align=None),
    "alljoyn_autopinger": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_autopinger_destination_found_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "group", "destination"]), offset=0),
    "alljoyn_autopinger_destination_lost_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "group", "destination"]), offset=0),
    "alljoyn_busattachment": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_busattachment_joinsessioncb_ptr": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="QStatus"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["status", "sessionId", "opts", "context"]), offset=0),
    "alljoyn_busattachment_setlinktimeoutcb_ptr": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="QStatus"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["status", "timeout", "context"]), offset=0),
    "alljoyn_buslistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_buslistener_bus_disconnected_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_buslistener_bus_prop_changed_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "prop_name", "prop_value"]), offset=0),
    "alljoyn_buslistener_bus_stopping_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_buslistener_callbacks": SimStruct(OrderedDict((("listener_registered", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "bus"]), offset=0)), ("listener_unregistered", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)), ("found_advertised_name", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "name", "transport", "namePrefix"]), offset=0)), ("lost_advertised_name", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "name", "transport", "namePrefix"]), offset=0)), ("name_owner_changed", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "busName", "previousOwner", "newOwner"]), offset=0)), ("bus_stopping", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)), ("bus_disconnected", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)), ("property_changed", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "prop_name", "prop_value"]), offset=0)),)), name="alljoyn_buslistener_callbacks", pack=False, align=None),
    "alljoyn_buslistener_found_advertised_name_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "name", "transport", "namePrefix"]), offset=0),
    "alljoyn_buslistener_listener_registered_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "bus"]), offset=0),
    "alljoyn_buslistener_listener_unregistered_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_buslistener_lost_advertised_name_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "name", "transport", "namePrefix"]), offset=0),
    "alljoyn_buslistener_name_owner_changed_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "busName", "previousOwner", "newOwner"]), offset=0),
    "alljoyn_busobject": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_busobject_callbacks": SimStruct(OrderedDict((("property_get", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "ifcName", "propName", "val"]), offset=0)), ("property_set", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "ifcName", "propName", "val"]), offset=0)), ("object_registered", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)), ("object_unregistered", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)),)), name="alljoyn_busobject_callbacks", pack=False, align=None),
    "alljoyn_busobject_methodentry": SimStruct(OrderedDict((("member", SimTypePointer(SimTypeRef("alljoyn_interfacedescription_member", SimStruct), offset=0)), ("method_handler", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("alljoyn_interfacedescription_member", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["bus", "member", "message"]), offset=0)),)), name="alljoyn_busobject_methodentry", pack=False, align=None),
    "alljoyn_busobject_object_registration_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_busobject_prop_get_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "ifcName", "propName", "val"]), offset=0),
    "alljoyn_busobject_prop_set_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "ifcName", "propName", "val"]), offset=0),
    "alljoyn_certificateid": SimStruct(OrderedDict((("serial", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("serialLen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("issuerPublicKey", SimTypePointer(SimTypeChar(label="SByte"), offset=0)), ("issuerAki", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("issuerAkiLen", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)),)), name="alljoyn_certificateid", pack=False, align=None),
    "alljoyn_certificateidarray": SimStruct(OrderedDict((("count", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("ids", SimTypePointer(SimTypeRef("alljoyn_certificateid", SimStruct), offset=0)),)), name="alljoyn_certificateidarray", pack=False, align=None),
    "alljoyn_claimcapability_masks": SimTypeInt(signed=False, label="alljoyn_claimcapability_masks"),
    "alljoyn_claimcapabilityadditionalinfo_masks": SimTypeInt(signed=False, label="alljoyn_claimcapabilityadditionalinfo_masks"),
    "alljoyn_credentials": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_interfacedescription": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_interfacedescription_member": SimStruct(OrderedDict((("iface", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("memberType", SimTypeInt(signed=False, label="alljoyn_messagetype")), ("name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("signature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("returnSignature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("argNames", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("internal_member", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="alljoyn_interfacedescription_member", pack=False, align=None),
    "alljoyn_interfacedescription_property": SimStruct(OrderedDict((("name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("signature", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("access", SimTypeChar(label="Byte")), ("internal_property", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="alljoyn_interfacedescription_property", pack=False, align=None),
    "alljoyn_interfacedescription_securitypolicy": SimTypeInt(signed=False, label="alljoyn_interfacedescription_securitypolicy"),
    "alljoyn_interfacedescription_translation_callback_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeChar(label="Byte"), offset=0), arg_names=["sourceLanguage", "targetLanguage", "sourceText"]), offset=0),
    "alljoyn_keystore": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_keystorelistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_keystorelistener_acquireexclusivelock_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener"]), offset=0),
    "alljoyn_keystorelistener_callbacks": SimStruct(OrderedDict((("load_request", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "keyStore"]), offset=0)), ("store_request", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "keyStore"]), offset=0)),)), name="alljoyn_keystorelistener_callbacks", pack=False, align=None),
    "alljoyn_keystorelistener_loadrequest_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "keyStore"]), offset=0),
    "alljoyn_keystorelistener_releaseexclusivelock_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "listener"]), offset=0),
    "alljoyn_keystorelistener_storerequest_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "keyStore"]), offset=0),
    "alljoyn_keystorelistener_with_synchronization_callbacks": SimStruct(OrderedDict((("load_request", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "keyStore"]), offset=0)), ("store_request", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener", "keyStore"]), offset=0)), ("acquire_exclusive_lock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context", "listener"]), offset=0)), ("release_exclusive_lock", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "listener"]), offset=0)),)), name="alljoyn_keystorelistener_with_synchronization_callbacks", pack=False, align=None),
    "alljoyn_manifestarray": SimStruct(OrderedDict((("count", SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)), ("xmls", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)),)), name="alljoyn_manifestarray", pack=False, align=None),
    "alljoyn_message": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_messagereceiver_methodhandler_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("alljoyn_interfacedescription_member", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["bus", "member", "message"]), offset=0),
    "alljoyn_messagereceiver_replyhandler_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["message", "context"]), offset=0),
    "alljoyn_messagereceiver_signalhandler_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("alljoyn_interfacedescription_member", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["member", "srcPath", "message"]), offset=0),
    "alljoyn_messagetype": SimTypeInt(signed=False, label="alljoyn_messagetype"),
    "alljoyn_msgarg": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_observer": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_observer_object_discovered_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "proxyref"]), offset=0),
    "alljoyn_observer_object_lost_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "proxyref"]), offset=0),
    "alljoyn_observerlistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_observerlistener_callback": SimStruct(OrderedDict((("object_discovered", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "proxyref"]), offset=0)), ("object_lost", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "proxyref"]), offset=0)),)), name="alljoyn_observerlistener_callback", pack=False, align=None),
    "alljoyn_permissionconfigurationlistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_permissionconfigurationlistener_callbacks": SimStruct(OrderedDict((("factory_reset", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context"]), offset=0)), ("policy_changed", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)), ("start_management", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)), ("end_management", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0)),)), name="alljoyn_permissionconfigurationlistener_callbacks", pack=False, align=None),
    "alljoyn_permissionconfigurationlistener_endmanagement_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_permissionconfigurationlistener_factoryreset_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="QStatus"), arg_names=["context"]), offset=0),
    "alljoyn_permissionconfigurationlistener_policychanged_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_permissionconfigurationlistener_startmanagement_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0),
    "alljoyn_permissionconfigurator": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_pinglistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_pinglistener_callback": SimStruct(OrderedDict((("destination_found", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "group", "destination"]), offset=0)), ("destination_lost", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "group", "destination"]), offset=0)),)), name="alljoyn_pinglistener_callback", pack=False, align=None),
    "alljoyn_proxybusobject": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_proxybusobject_listener_getallpropertiescb_ptr": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["status", "obj", "values", "context"]), offset=0),
    "alljoyn_proxybusobject_listener_getpropertycb_ptr": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["status", "obj", "value", "context"]), offset=0),
    "alljoyn_proxybusobject_listener_introspectcb_ptr": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["status", "obj", "context"]), offset=0),
    "alljoyn_proxybusobject_listener_propertieschanged_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["obj", "ifaceName", "changed", "invalidated", "context"]), offset=0),
    "alljoyn_proxybusobject_listener_setpropertycb_ptr": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="QStatus"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["status", "obj", "context"]), offset=0),
    "alljoyn_proxybusobject_ref": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_securityapplicationproxy": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_sessionlistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_sessionlistener_callbacks": SimStruct(OrderedDict((("session_lost", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="alljoyn_sessionlostreason")], SimTypeBottom(label="Void"), arg_names=["context", "sessionId", "reason"]), offset=0)), ("session_member_added", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sessionId", "uniqueName"]), offset=0)), ("session_member_removed", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sessionId", "uniqueName"]), offset=0)),)), name="alljoyn_sessionlistener_callbacks", pack=False, align=None),
    "alljoyn_sessionlistener_sessionlost_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="alljoyn_sessionlostreason")], SimTypeBottom(label="Void"), arg_names=["context", "sessionId", "reason"]), offset=0),
    "alljoyn_sessionlistener_sessionmemberadded_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sessionId", "uniqueName"]), offset=0),
    "alljoyn_sessionlistener_sessionmemberremoved_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sessionId", "uniqueName"]), offset=0),
    "alljoyn_sessionlostreason": SimTypeInt(signed=False, label="alljoyn_sessionlostreason"),
    "alljoyn_sessionopts": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_sessionportlistener": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0),
    "alljoyn_sessionportlistener_acceptsessionjoiner_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "sessionPort", "joiner", "opts"]), offset=0),
    "alljoyn_sessionportlistener_callbacks": SimStruct(OrderedDict((("accept_session_joiner", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["context", "sessionPort", "joiner", "opts"]), offset=0)), ("session_joined", SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sessionPort", "id", "joiner"]), offset=0)),)), name="alljoyn_sessionportlistener_callbacks", pack=False, align=None),
    "alljoyn_sessionportlistener_sessionjoined_ptr": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sessionPort", "id", "joiner"]), offset=0),
    "alljoyn_typeid": SimTypeInt(signed=False, label="alljoyn_typeid"),
    "bodyScroll": SimTypeInt(signed=False, label="bodyScroll"),
    "eAVAudioChannelConfig": SimTypeInt(signed=False, label="eAVAudioChannelConfig"),
    "eAVDDSurroundMode": SimTypeInt(signed=False, label="eAVDDSurroundMode"),
    "eAVDSPLoudnessEqualization": SimTypeInt(signed=False, label="eAVDSPLoudnessEqualization"),
    "eAVDSPSpeakerFill": SimTypeInt(signed=False, label="eAVDSPSpeakerFill"),
    "eAVDecAACDownmixMode": SimTypeInt(signed=False, label="eAVDecAACDownmixMode"),
    "eAVDecAudioDualMono": SimTypeInt(signed=False, label="eAVDecAudioDualMono"),
    "eAVDecAudioDualMonoReproMode": SimTypeInt(signed=False, label="eAVDecAudioDualMonoReproMode"),
    "eAVDecDDMatrixDecodingMode": SimTypeInt(signed=False, label="eAVDecDDMatrixDecodingMode"),
    "eAVDecDDOperationalMode": SimTypeInt(signed=False, label="eAVDecDDOperationalMode"),
    "eAVDecDDStereoDownMixMode": SimTypeInt(signed=False, label="eAVDecDDStereoDownMixMode"),
    "eAVDecHEAACDynamicRangeControl": SimTypeInt(signed=False, label="eAVDecHEAACDynamicRangeControl"),
    "eAVDecVideoCodecType": SimTypeInt(signed=False, label="eAVDecVideoCodecType"),
    "eAVDecVideoDXVABusEncryption": SimTypeInt(signed=False, label="eAVDecVideoDXVABusEncryption"),
    "eAVDecVideoDXVAMode": SimTypeInt(signed=False, label="eAVDecVideoDXVAMode"),
    "eAVDecVideoH264ErrorConcealment": SimTypeInt(signed=False, label="eAVDecVideoH264ErrorConcealment"),
    "eAVDecVideoInputScanType": SimTypeInt(signed=False, label="eAVDecVideoInputScanType"),
    "eAVDecVideoMPEG2ErrorConcealment": SimTypeInt(signed=False, label="eAVDecVideoMPEG2ErrorConcealment"),
    "eAVDecVideoSWPowerLevel": SimTypeInt(signed=False, label="eAVDecVideoSWPowerLevel"),
    "eAVDecVideoSoftwareDeinterlaceMode": SimTypeInt(signed=False, label="eAVDecVideoSoftwareDeinterlaceMode"),
    "eAVEncAV1PictureType": SimTypeInt(signed=False, label="eAVEncAV1PictureType"),
    "eAVEncAV1VLevel": SimTypeInt(signed=False, label="eAVEncAV1VLevel"),
    "eAVEncAV1VProfile": SimTypeInt(signed=False, label="eAVEncAV1VProfile"),
    "eAVEncAdaptiveMode": SimTypeInt(signed=False, label="eAVEncAdaptiveMode"),
    "eAVEncAudioDualMono": SimTypeInt(signed=False, label="eAVEncAudioDualMono"),
    "eAVEncAudioInputContent": SimTypeInt(signed=False, label="eAVEncAudioInputContent"),
    "eAVEncChromaEncodeMode": SimTypeInt(signed=False, label="eAVEncChromaEncodeMode"),
    "eAVEncCommonRateControlMode": SimTypeInt(signed=False, label="eAVEncCommonRateControlMode"),
    "eAVEncCommonStreamEndHandling": SimTypeInt(signed=False, label="eAVEncCommonStreamEndHandling"),
    "eAVEncDDAtoDConverterType": SimTypeInt(signed=False, label="eAVEncDDAtoDConverterType"),
    "eAVEncDDDynamicRangeCompressionControl": SimTypeInt(signed=False, label="eAVEncDDDynamicRangeCompressionControl"),
    "eAVEncDDHeadphoneMode": SimTypeInt(signed=False, label="eAVEncDDHeadphoneMode"),
    "eAVEncDDPreferredStereoDownMixMode": SimTypeInt(signed=False, label="eAVEncDDPreferredStereoDownMixMode"),
    "eAVEncDDProductionRoomType": SimTypeInt(signed=False, label="eAVEncDDProductionRoomType"),
    "eAVEncDDService": SimTypeInt(signed=False, label="eAVEncDDService"),
    "eAVEncDDSurroundExMode": SimTypeInt(signed=False, label="eAVEncDDSurroundExMode"),
    "eAVEncH263PictureType": SimTypeInt(signed=False, label="eAVEncH263PictureType"),
    "eAVEncH263VLevel": SimTypeInt(signed=False, label="eAVEncH263VLevel"),
    "eAVEncH263VProfile": SimTypeInt(signed=False, label="eAVEncH263VProfile"),
    "eAVEncH264PictureType": SimTypeInt(signed=False, label="eAVEncH264PictureType"),
    "eAVEncH264VLevel": SimTypeInt(signed=False, label="eAVEncH264VLevel"),
    "eAVEncH264VProfile": SimTypeInt(signed=False, label="eAVEncH264VProfile"),
    "eAVEncH265VLevel": SimTypeInt(signed=False, label="eAVEncH265VLevel"),
    "eAVEncH265VProfile": SimTypeInt(signed=False, label="eAVEncH265VProfile"),
    "eAVEncInputVideoSystem": SimTypeInt(signed=False, label="eAVEncInputVideoSystem"),
    "eAVEncMPACodingMode": SimTypeInt(signed=False, label="eAVEncMPACodingMode"),
    "eAVEncMPAEmphasisType": SimTypeInt(signed=False, label="eAVEncMPAEmphasisType"),
    "eAVEncMPALayer": SimTypeInt(signed=False, label="eAVEncMPALayer"),
    "eAVEncMPVFrameFieldMode": SimTypeInt(signed=False, label="eAVEncMPVFrameFieldMode"),
    "eAVEncMPVIntraVLCTable": SimTypeInt(signed=False, label="eAVEncMPVIntraVLCTable"),
    "eAVEncMPVLevel": SimTypeInt(signed=False, label="eAVEncMPVLevel"),
    "eAVEncMPVProfile": SimTypeInt(signed=False, label="eAVEncMPVProfile"),
    "eAVEncMPVQScaleType": SimTypeInt(signed=False, label="eAVEncMPVQScaleType"),
    "eAVEncMPVScanPattern": SimTypeInt(signed=False, label="eAVEncMPVScanPattern"),
    "eAVEncMPVSceneDetection": SimTypeInt(signed=False, label="eAVEncMPVSceneDetection"),
    "eAVEncMuxOutput": SimTypeInt(signed=False, label="eAVEncMuxOutput"),
    "eAVEncVP9VProfile": SimTypeInt(signed=False, label="eAVEncVP9VProfile"),
    "eAVEncVideoChromaResolution": SimTypeInt(signed=False, label="eAVEncVideoChromaResolution"),
    "eAVEncVideoChromaSubsampling": SimTypeInt(signed=False, label="eAVEncVideoChromaSubsampling"),
    "eAVEncVideoColorLighting": SimTypeInt(signed=False, label="eAVEncVideoColorLighting"),
    "eAVEncVideoColorNominalRange": SimTypeInt(signed=False, label="eAVEncVideoColorNominalRange"),
    "eAVEncVideoColorPrimaries": SimTypeInt(signed=False, label="eAVEncVideoColorPrimaries"),
    "eAVEncVideoColorTransferFunction": SimTypeInt(signed=False, label="eAVEncVideoColorTransferFunction"),
    "eAVEncVideoColorTransferMatrix": SimTypeInt(signed=False, label="eAVEncVideoColorTransferMatrix"),
    "eAVEncVideoContentType": SimTypeInt(signed=False, label="eAVEncVideoContentType"),
    "eAVEncVideoFilmContent": SimTypeInt(signed=False, label="eAVEncVideoFilmContent"),
    "eAVEncVideoOutputFrameRateConversion": SimTypeInt(signed=False, label="eAVEncVideoOutputFrameRateConversion"),
    "eAVEncVideoOutputScanType": SimTypeInt(signed=False, label="eAVEncVideoOutputScanType"),
    "eAVEncVideoSourceScanType": SimTypeInt(signed=False, label="eAVEncVideoSourceScanType"),
    "eAVFastDecodeMode": SimTypeInt(signed=False, label="eAVFastDecodeMode"),
    "eAVScenarioInfo": SimTypeInt(signed=False, label="eAVScenarioInfo"),
    "eAvrfResourceTypes": SimTypeInt(signed=False, label="eAvrfResourceTypes"),
    "eHANDLE_TRACE_OPERATIONS": SimTypeInt(signed=False, label="eHANDLE_TRACE_OPERATIONS"),
    "eHeapAllocationState": SimTypeInt(signed=False, label="eHeapAllocationState"),
    "eHeapEnumerationLevel": SimTypeInt(signed=False, label="eHeapEnumerationLevel"),
    "eTlsAlgorithmUsage": SimTypeInt(signed=False, label="eTlsAlgorithmUsage"),
    "eTlsHashAlgorithm": SimTypeInt(signed=False, label="eTlsHashAlgorithm"),
    "eTlsSignatureAlgorithm": SimTypeInt(signed=False, label="eTlsSignatureAlgorithm"),
    "eUserAllocationState": SimTypeInt(signed=False, label="eUserAllocationState"),
    "eVideoEncoderDisplayContentType": SimTypeInt(signed=False, label="eVideoEncoderDisplayContentType"),
    "eWINDOW_ADVANCE_METHOD": SimTypeInt(signed=False, label="eWINDOW_ADVANCE_METHOD"),
    "fnDebugFailureAnalysisCreateInstance": SimTypePointer(SimTypeFunction([SimTypeBottom(label="IDebugClient"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Client", "Args", "Flags", "rclsid", "riid", "ppv"]), offset=0),
    "fpCreateIFECommonInstanceType": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppvObj"]), offset=0),
    "fpCreateIFEDictionaryInstanceType": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppvObj"]), offset=0),
    "fpCreateIFELanguageInstanceType": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["clsid", "ppvObj"]), offset=0),
    "frameScrolling": SimTypeInt(signed=False, label="frameScrolling"),
    "htmlAdjacency": SimTypeInt(signed=False, label="htmlAdjacency"),
    "htmlApplyLocation": SimTypeInt(signed=False, label="htmlApplyLocation"),
    "htmlBlockAlign": SimTypeInt(signed=False, label="htmlBlockAlign"),
    "htmlCaptionAlign": SimTypeInt(signed=False, label="htmlCaptionAlign"),
    "htmlCaptionVAlign": SimTypeInt(signed=False, label="htmlCaptionVAlign"),
    "htmlCellAlign": SimTypeInt(signed=False, label="htmlCellAlign"),
    "htmlCellVAlign": SimTypeInt(signed=False, label="htmlCellVAlign"),
    "htmlClear": SimTypeInt(signed=False, label="htmlClear"),
    "htmlCompatMode": SimTypeInt(signed=False, label="htmlCompatMode"),
    "htmlComponent": SimTypeInt(signed=False, label="htmlComponent"),
    "htmlControlAlign": SimTypeInt(signed=False, label="htmlControlAlign"),
    "htmlDesignMode": SimTypeInt(signed=False, label="htmlDesignMode"),
    "htmlDir": SimTypeInt(signed=False, label="htmlDir"),
    "htmlDirection": SimTypeInt(signed=False, label="htmlDirection"),
    "htmlDraggable": SimTypeInt(signed=False, label="htmlDraggable"),
    "htmlDropEffect": SimTypeInt(signed=False, label="htmlDropEffect"),
    "htmlEditable": SimTypeInt(signed=False, label="htmlEditable"),
    "htmlEffectAllowed": SimTypeInt(signed=False, label="htmlEffectAllowed"),
    "htmlEncoding": SimTypeInt(signed=False, label="htmlEncoding"),
    "htmlEndPoints": SimTypeInt(signed=False, label="htmlEndPoints"),
    "htmlFrame": SimTypeInt(signed=False, label="htmlFrame"),
    "htmlGlyphMode": SimTypeInt(signed=False, label="htmlGlyphMode"),
    "htmlInput": SimTypeInt(signed=False, label="htmlInput"),
    "htmlListType": SimTypeInt(signed=False, label="htmlListType"),
    "htmlLoop": SimTypeInt(signed=False, label="htmlLoop"),
    "htmlMarqueeBehavior": SimTypeInt(signed=False, label="htmlMarqueeBehavior"),
    "htmlMarqueeDirection": SimTypeInt(signed=False, label="htmlMarqueeDirection"),
    "htmlMediaErr": SimTypeInt(signed=False, label="htmlMediaErr"),
    "htmlMediaNetworkState": SimTypeInt(signed=False, label="htmlMediaNetworkState"),
    "htmlMediaReadyState": SimTypeInt(signed=False, label="htmlMediaReadyState"),
    "htmlMethod": SimTypeInt(signed=False, label="htmlMethod"),
    "htmlPersistState": SimTypeInt(signed=False, label="htmlPersistState"),
    "htmlReadyState": SimTypeInt(signed=False, label="htmlReadyState"),
    "htmlRules": SimTypeInt(signed=False, label="htmlRules"),
    "htmlSelectExFlag": SimTypeInt(signed=False, label="htmlSelectExFlag"),
    "htmlSelectType": SimTypeInt(signed=False, label="htmlSelectType"),
    "htmlSelection": SimTypeInt(signed=False, label="htmlSelection"),
    "htmlSpellCheck": SimTypeInt(signed=False, label="htmlSpellCheck"),
    "htmlStart": SimTypeInt(signed=False, label="htmlStart"),
    "htmlTabIndex": SimTypeInt(signed=False, label="htmlTabIndex"),
    "htmlUnit": SimTypeInt(signed=False, label="htmlUnit"),
    "htmlWrap": SimTypeInt(signed=False, label="htmlWrap"),
    "htmlZOrder": SimTypeInt(signed=False, label="htmlZOrder"),
    "int_serv_wkp": SimTypeInt(signed=False, label="int_serv_wkp"),
    "lengthAdjust": SimTypeInt(signed=False, label="lengthAdjust"),
    "mediaType": SimTypeInt(signed=False, label="mediaType"),
    "msidbAssemblyAttributes": SimTypeInt(signed=False, label="msidbAssemblyAttributes"),
    "msidbClassAttributes": SimTypeInt(signed=False, label="msidbClassAttributes"),
    "msidbComponentAttributes": SimTypeInt(signed=False, label="msidbComponentAttributes"),
    "msidbControlAttributes": SimTypeInt(signed=False, label="msidbControlAttributes"),
    "msidbCustomActionType": SimTypeInt(signed=False, label="msidbCustomActionType"),
    "msidbDialogAttributes": SimTypeInt(signed=False, label="msidbDialogAttributes"),
    "msidbEmbeddedUIAttributes": SimTypeInt(signed=False, label="msidbEmbeddedUIAttributes"),
    "msidbFeatureAttributes": SimTypeInt(signed=False, label="msidbFeatureAttributes"),
    "msidbFileAttributes": SimTypeInt(signed=False, label="msidbFileAttributes"),
    "msidbIniFileAction": SimTypeInt(signed=False, label="msidbIniFileAction"),
    "msidbLocatorType": SimTypeInt(signed=False, label="msidbLocatorType"),
    "msidbMoveFileOptions": SimTypeInt(signed=False, label="msidbMoveFileOptions"),
    "msidbODBCDataSourceRegistration": SimTypeInt(signed=False, label="msidbODBCDataSourceRegistration"),
    "msidbPatchAttributes": SimTypeInt(signed=False, label="msidbPatchAttributes"),
    "msidbRegistryRoot": SimTypeInt(signed=False, label="msidbRegistryRoot"),
    "msidbRemoveFileInstallMode": SimTypeInt(signed=False, label="msidbRemoveFileInstallMode"),
    "msidbServiceConfigEvent": SimTypeInt(signed=False, label="msidbServiceConfigEvent"),
    "msidbServiceControlEvent": SimTypeInt(signed=False, label="msidbServiceControlEvent"),
    "msidbServiceInstallErrorControl": SimTypeInt(signed=False, label="msidbServiceInstallErrorControl"),
    "msidbSumInfoSourceType": SimTypeInt(signed=False, label="msidbSumInfoSourceType"),
    "msidbTextStyleStyleBits": SimTypeInt(signed=False, label="msidbTextStyleStyleBits"),
    "msidbUpgradeAttributes": SimTypeInt(signed=False, label="msidbUpgradeAttributes"),
    "msifiFastInstallBits": SimTypeInt(signed=False, label="msifiFastInstallBits"),
    "msirbRebootReason": SimTypeInt(signed=False, label="msirbRebootReason"),
    "msirbRebootType": SimTypeInt(signed=False, label="msirbRebootType"),
    "msmErrorType": SimTypeInt(signed=False, label="msmErrorType"),
    "netent": SimStruct(OrderedDict((("n_name", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("n_aliases", SimTypePointer(SimTypePointer(SimTypeChar(label="SByte"), offset=0), offset=0)), ("n_addrtype", SimTypeShort(signed=True, label="Int16")), ("n_net", SimTypeInt(signed=False, label="UInt32")),)), name="netent", pack=False, align=None),
    "pCryptSIPCreateIndirectData": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pcbIndirectData", "pIndirectData"]), offset=0),
    "pCryptSIPGetCaps": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("SIP_CAP_SET_V3", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjInfo", "pCaps"]), offset=0),
    "pCryptSIPGetSealedDigest": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pSig", "dwSig", "pbDigest", "pcbDigest"]), offset=0),
    "pCryptSIPGetSignedDataMsg": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pdwEncodingType", "dwIndex", "pcbSignedDataMsg", "pbSignedDataMsg"]), offset=0),
    "pCryptSIPPutSignedDataMsg": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "dwEncodingType", "pdwIndex", "cbSignedDataMsg", "pbSignedDataMsg"]), offset=0),
    "pCryptSIPRemoveSignedDataMsg": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "dwIndex"]), offset=0),
    "pCryptSIPVerifyIndirectData": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("SIP_SUBJECTINFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("SIP_INDIRECT_DATA", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSubjectInfo", "pIndirectData"]), offset=0),
    "pD3DCompile": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("D3D_SHADER_MACRO", SimStruct), offset=0), SimTypeBottom(label="ID3DInclude"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSize", "pFileName", "pDefines", "pInclude", "pEntrypoint", "pTarget", "Flags1", "Flags2", "ppCode", "ppErrorMsgs"]), offset=0),
    "pD3DDisassemble": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSize", "Flags", "szComments", "ppDisassembly"]), offset=0),
    "pD3DPreprocess": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("D3D_SHADER_MACRO", SimStruct), offset=0), SimTypeBottom(label="ID3DInclude"), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0), SimTypePointer(SimTypeBottom(label="ID3DBlob"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrcData", "SrcDataSize", "pFileName", "pDefines", "pInclude", "ppCodeText", "ppErrorMsgs"]), offset=0),
    "pHalAssignSlotResources": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("UNICODE_STRING", SimStruct), offset=0), SimTypePointer(SimTypeRef("DRIVER_OBJECT", SimStruct), offset=0), SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeRef("CM_RESOURCE_LIST", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RegistryPath", "DriverClassName", "DriverObject", "DeviceObject", "BusType", "BusNumber", "SlotNumber", "AllocatedResources"]), offset=0),
    "pHalEndMirroring": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["PassNumber"]), offset=0),
    "pHalEndOfBoot": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "pHalExamineMBR": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["DeviceObject", "SectorSize", "MBRTypeIdentifier", "Buffer"]), offset=0),
    "pHalFindBusAddressTranslation": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["BusAddress", "AddressSpace", "TranslatedAddress", "Context", "NextBus"]), offset=0),
    "pHalGetAcpiTable": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["Signature", "OemId", "OemTableId"]), offset=0),
    "pHalGetDmaAdapter": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEVICE_DESCRIPTION", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypePointer(SimTypeRef("DMA_ADAPTER", SimStruct), offset=0), arg_names=["PhysicalDeviceObject", "DeviceDescriptor", "NumberOfMapRegisters"]), offset=0),
    "pHalGetInterruptTranslator": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("TRANSLATOR_INTERFACE", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ParentInterfaceType", "ParentBusNumber", "BridgeInterfaceType", "Size", "Version", "Translator", "BridgeBusNumber"]), offset=0),
    "pHalGetPrmCache": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("LIST_ENTRY", SimStruct), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["FirmwareList", "UpdateList"]), offset=0),
    "pHalHaltSystem": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "pHalHandlerForBus": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["InterfaceType", "BusNumber"]), offset=0),
    "pHalInitPnpDriver": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "pHalInitPowerManagement": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("PM_DISPATCH_TABLE", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeRef("PM_DISPATCH_TABLE", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PmDriverDispatchTable", "PmHalDispatchTable"]), offset=0),
    "pHalIoReadPartitionTable": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimTypePointer(SimTypeRef("DRIVE_LAYOUT_INFORMATION", SimStruct), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "SectorSize", "ReturnRecognizedPartitions", "PartitionBuffer"]), offset=0),
    "pHalIoSetPartitionInformation": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "SectorSize", "PartitionNumber", "PartitionType"]), offset=0),
    "pHalIoWritePartitionTable": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEVICE_OBJECT", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("DRIVE_LAYOUT_INFORMATION", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceObject", "SectorSize", "SectorsPerTrack", "NumberOfHeads", "PartitionBuffer"]), offset=0),
    "pHalMirrorPhysicalMemory": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PhysicalAddress", "NumberOfBytes"]), offset=0),
    "pHalMirrorVerify": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeLongLong(signed=True, label="Int64")], SimTypeInt(signed=True, label="Int32"), arg_names=["PhysicalAddress", "NumberOfBytes"]), offset=0),
    "pHalQueryBusSlots": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["BusHandler", "BufferSize", "SlotNumbers", "ReturnedLength"]), offset=0),
    "pHalQuerySystemInformation": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="HAL_QUERY_INFORMATION_CLASS"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InformationClass", "BufferSize", "Buffer", "ReturnedLength"]), offset=0),
    "pHalReferenceBusHandler": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["BusHandler"]), offset=0),
    "pHalResetDisplay": SimTypePointer(SimTypeFunction([], SimTypeChar(label="Byte")), offset=0),
    "pHalSetPciErrorHandlerCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0)], SimTypeBottom(label="Void"), arg_names=["Callback"]), offset=0),
    "pHalSetSystemInformation": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="HAL_SET_INFORMATION_CLASS"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["InformationClass", "BufferSize", "Buffer"]), offset=0),
    "pHalStartMirroring": SimTypePointer(SimTypeFunction([], SimTypeInt(signed=True, label="Int32")), offset=0),
    "pHalTranslateBusAddress": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="INTERFACE_TYPE"), SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0)], SimTypeChar(label="Byte"), arg_names=["InterfaceType", "BusNumber", "BusAddress", "AddressSpace", "TranslatedAddress"]), offset=0),
    "pHalVectorToIDTEntry": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["Vector"]), offset=0),
    "pKdCheckPowerButton": SimTypePointer(SimTypeFunction([], SimTypeBottom(label="Void")), offset=0),
    "pKdEnumerateDebuggingDevices": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=False, label="KD_CALLBACK_ACTION"), arg_names=["Device"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LoaderBlock", "Device", "Callback"]), offset=0),
    "pKdGetAcpiTablePhase0": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["LoaderBlock", "Signature"]), offset=0),
    "pKdGetPciDataByOffset": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["BusNumber", "SlotNumber", "Buffer", "Offset", "Length"]), offset=0),
    "pKdMapPhysicalMemory64": SimTypePointer(SimTypeFunction([SimTypeLongLong(signed=True, label="Int64"), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["PhysicalAddress", "NumberPages", "FlushCurrentTLB"]), offset=0),
    "pKdReleaseIntegratedDeviceForDebugging": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["IntegratedDevice"]), offset=0),
    "pKdReleasePciDeviceForDebugging": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PciDevice"]), offset=0),
    "pKdSetPciDataByOffset": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["BusNumber", "SlotNumber", "Buffer", "Offset", "Length"]), offset=0),
    "pKdSetupIntegratedDeviceForDebugging": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LoaderBlock", "IntegratedDevice"]), offset=0),
    "pKdSetupPciDeviceForDebugging": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeRef("DEBUG_DEVICE_DESCRIPTOR", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LoaderBlock", "PciDevice"]), offset=0),
    "pKdUnmapVirtualAddress": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte")], SimTypeBottom(label="Void"), arg_names=["VirtualAddress", "NumberPages", "FlushCurrentTLB"]), offset=0),
    "pfnInternetDeInitializeAutoProxyDll": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszMime", "dwReserved"]), offset=0),
    "pfnInternetGetProxyInfo": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszUrl", "dwUrlLength", "lpszUrlHostName", "dwUrlHostNameLength", "lplpszProxyHostName", "lpdwProxyHostNameLength"]), offset=0),
    "pfnInternetInitializeAutoProxyDll": SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("AutoProxyHelperFunctions", SimStruct), offset=0), SimTypePointer(SimTypeRef("AUTO_PROXY_SCRIPT_BUFFER", SimStruct), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwVersion", "lpszDownloadedTempFile", "lpszMime", "lpAutoProxyCallbacks", "lpAutoProxyScriptBuffer"]), offset=0),
    "pfnIsFileSupported": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hFile", "pgSubject"]), offset=0),
    "pfnIsFileSupportedName": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszFileName", "pgSubject"]), offset=0),
    "pfn_ADDEREXCLUDEDAPPLICATIONA": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="EFaultRepRetVal"), arg_names=["param0"]), offset=0),
    "pfn_ADDEREXCLUDEDAPPLICATIONW": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="EFaultRepRetVal"), arg_names=["param0"]), offset=0),
    "pfn_REPORTFAULT": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeRef("EXCEPTION_POINTERS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="EFaultRepRetVal"), arg_names=["param0", "param1"]), offset=0),
    "pluginResource": SimStruct(OrderedDict((("alias", SimTypeArray(SimTypeChar(label="Char"), 256)), ("name", SimTypeArray(SimTypeChar(label="Char"), 256)), ("resourceFileContents", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("fileExtension", SimTypeArray(SimTypeChar(label="Char"), 256)), ("resourcePluginType", SimTypeArray(SimTypeChar(label="Char"), 256)), ("isDiscoverable", SimTypeChar(label="Byte")), ("resourceType", SimTypeInt(signed=True, label="Int32")), ("pceIconSize", SimTypeInt(signed=False, label="UInt32")), ("iconContents", SimTypePointer(SimTypeChar(label="Byte"), offset=0)), ("pcePluginBlobSize", SimTypeInt(signed=False, label="UInt32")), ("blobContents", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="pluginResource", pack=False, align=None),
    "pluginResource2": SimStruct(OrderedDict((("resourceV1", SimTypeRef("pluginResource", SimStruct)), ("pceFileAssocListSize", SimTypeInt(signed=False, label="UInt32")), ("fileAssocList", SimTypePointer(SimTypeRef("pluginResource2FileAssociation", SimStruct), offset=0)), ("securityDescriptor", SimTypePointer(SimTypeChar(label="Char"), offset=0)), ("pceFolderListSize", SimTypeInt(signed=False, label="UInt32")), ("folderList", SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)),)), name="pluginResource2", pack=False, align=None),
    "pluginResource2FileAssociation": SimStruct(OrderedDict((("extName", SimTypeArray(SimTypeChar(label="Char"), 256)), ("primaryHandler", SimTypeChar(label="Byte")), ("pceIconSize", SimTypeInt(signed=False, label="UInt32")), ("iconContents", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="pluginResource2FileAssociation", pack=False, align=None),
    "remoteMETAFILEPICT": SimStruct(OrderedDict((("mm", SimTypeInt(signed=True, label="Int32")), ("xExt", SimTypeInt(signed=True, label="Int32")), ("yExt", SimTypeInt(signed=True, label="Int32")), ("hMF", SimTypePointer(SimTypeRef("userHMETAFILE", SimStruct), offset=0)),)), name="remoteMETAFILEPICT", pack=False, align=None),
    "s_RIFFWAVE_inst": SimStruct(OrderedDict((("bUnshiftedNote", SimTypeChar(label="Byte")), ("chFineTune", SimTypeChar(label="SByte")), ("chGain", SimTypeChar(label="SByte")), ("bLowNote", SimTypeChar(label="Byte")), ("bHighNote", SimTypeChar(label="Byte")), ("bLowVelocity", SimTypeChar(label="Byte")), ("bHighVelocity", SimTypeChar(label="Byte")),)), name="s_RIFFWAVE_inst", pack=False, align=None),
    "sandboxAllow": SimTypeInt(signed=False, label="sandboxAllow"),
    "smiCNTR64": SimStruct(OrderedDict((("hipart", SimTypeInt(signed=False, label="UInt32")), ("lopart", SimTypeInt(signed=False, label="UInt32")),)), name="smiCNTR64", pack=False, align=None),
    "smiOCTETS": SimStruct(OrderedDict((("len", SimTypeInt(signed=False, label="UInt32")), ("ptr", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="smiOCTETS", pack=False, align=None),
    "smiOID": SimStruct(OrderedDict((("len", SimTypeInt(signed=False, label="UInt32")), ("ptr", SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)),)), name="smiOID", pack=False, align=None),
    "smiVALUE": SimStruct(OrderedDict((("syntax", SimTypeInt(signed=False, label="UInt32")), ("value", SimUnion({"sNumber": SimTypeInt(signed=True, label="Int32"), "uNumber": SimTypeInt(signed=False, label="UInt32"), "hNumber": SimTypeRef("smiCNTR64", SimStruct), "string": SimTypeRef("smiOCTETS", SimStruct), "oid": SimTypeRef("smiOID", SimStruct), "empty": SimTypeChar(label="Byte")}, name="<anon>", label="None")),)), name="smiVALUE", pack=False, align=None),
    "smiVENDORINFO": SimStruct(OrderedDict((("vendorName", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("vendorContact", SimTypeArray(SimTypeChar(label="SByte"), 64)), ("vendorVersionId", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("vendorVersionDate", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("vendorEnterprise", SimTypeInt(signed=False, label="UInt32")),)), name="smiVENDORINFO", pack=False, align=None),
    "sockaddr_gen": SimUnion({"Address": SimTypeRef("SOCKADDR", SimStruct), "AddressIn": SimTypeRef("SOCKADDR_IN", SimStruct), "AddressIn6": SimTypeRef("sockaddr_in6_old", SimStruct)}, name="<anon>", label="None"),
    "sockaddr_in6_old": SimStruct(OrderedDict((("sin6_family", SimTypeShort(signed=True, label="Int16")), ("sin6_port", SimTypeShort(signed=False, label="UInt16")), ("sin6_flowinfo", SimTypeInt(signed=False, label="UInt32")), ("sin6_addr", SimTypeRef("IN6_ADDR", SimStruct)),)), name="sockaddr_in6_old", pack=False, align=None),
    "socklen_t": SimTypeInt(signed=True, label="Int32"),
    "sockproto": SimStruct(OrderedDict((("sp_family", SimTypeShort(signed=False, label="UInt16")), ("sp_protocol", SimTypeShort(signed=False, label="UInt16")),)), name="sockproto", pack=False, align=None),
    "styleAccelerator": SimTypeInt(signed=False, label="styleAccelerator"),
    "styleAlignContent": SimTypeInt(signed=False, label="styleAlignContent"),
    "styleAlignItems": SimTypeInt(signed=False, label="styleAlignItems"),
    "styleAlignSelf": SimTypeInt(signed=False, label="styleAlignSelf"),
    "styleAlignmentBaseline": SimTypeInt(signed=False, label="styleAlignmentBaseline"),
    "styleAttrType": SimTypeInt(signed=False, label="styleAttrType"),
    "styleAuto": SimTypeInt(signed=False, label="styleAuto"),
    "styleBackfaceVisibility": SimTypeInt(signed=False, label="styleBackfaceVisibility"),
    "styleBackgroundAttachment": SimTypeInt(signed=False, label="styleBackgroundAttachment"),
    "styleBackgroundAttachment3": SimTypeInt(signed=False, label="styleBackgroundAttachment3"),
    "styleBackgroundClip": SimTypeInt(signed=False, label="styleBackgroundClip"),
    "styleBackgroundOrigin": SimTypeInt(signed=False, label="styleBackgroundOrigin"),
    "styleBackgroundRepeat": SimTypeInt(signed=False, label="styleBackgroundRepeat"),
    "styleBaselineShift": SimTypeInt(signed=False, label="styleBaselineShift"),
    "styleBidi": SimTypeInt(signed=False, label="styleBidi"),
    "styleBlockProgression": SimTypeInt(signed=False, label="styleBlockProgression"),
    "styleBool": SimTypeInt(signed=False, label="styleBool"),
    "styleBorderCollapse": SimTypeInt(signed=False, label="styleBorderCollapse"),
    "styleBorderImageRepeat": SimTypeInt(signed=False, label="styleBorderImageRepeat"),
    "styleBorderImageSliceFill": SimTypeInt(signed=False, label="styleBorderImageSliceFill"),
    "styleBorderStyle": SimTypeInt(signed=False, label="styleBorderStyle"),
    "styleBorderWidth": SimTypeInt(signed=False, label="styleBorderWidth"),
    "styleBoxSizing": SimTypeInt(signed=False, label="styleBoxSizing"),
    "styleBreak": SimTypeInt(signed=False, label="styleBreak"),
    "styleBreakInside": SimTypeInt(signed=False, label="styleBreakInside"),
    "styleCaptionSide": SimTypeInt(signed=False, label="styleCaptionSide"),
    "styleClipRule": SimTypeInt(signed=False, label="styleClipRule"),
    "styleColorInterpolationFilters": SimTypeInt(signed=False, label="styleColorInterpolationFilters"),
    "styleColumnFill": SimTypeInt(signed=False, label="styleColumnFill"),
    "styleColumnSpan": SimTypeInt(signed=False, label="styleColumnSpan"),
    "styleCursor": SimTypeInt(signed=False, label="styleCursor"),
    "styleDataRepeat": SimTypeInt(signed=False, label="styleDataRepeat"),
    "styleDefaultTextSelection": SimTypeInt(signed=False, label="styleDefaultTextSelection"),
    "styleDir": SimTypeInt(signed=False, label="styleDir"),
    "styleDisplay": SimTypeInt(signed=False, label="styleDisplay"),
    "styleDominantBaseline": SimTypeInt(signed=False, label="styleDominantBaseline"),
    "styleEmptyCells": SimTypeInt(signed=False, label="styleEmptyCells"),
    "styleEnableBackground": SimTypeInt(signed=False, label="styleEnableBackground"),
    "styleFillRule": SimTypeInt(signed=False, label="styleFillRule"),
    "styleFlex": SimTypeInt(signed=False, label="styleFlex"),
    "styleFlexBasis": SimTypeInt(signed=False, label="styleFlexBasis"),
    "styleFlexDirection": SimTypeInt(signed=False, label="styleFlexDirection"),
    "styleFlexWrap": SimTypeInt(signed=False, label="styleFlexWrap"),
    "styleFontSize": SimTypeInt(signed=False, label="styleFontSize"),
    "styleFontStretch": SimTypeInt(signed=False, label="styleFontStretch"),
    "styleFontStyle": SimTypeInt(signed=False, label="styleFontStyle"),
    "styleFontVariant": SimTypeInt(signed=False, label="styleFontVariant"),
    "styleFontWeight": SimTypeInt(signed=False, label="styleFontWeight"),
    "styleGridColumn": SimTypeInt(signed=False, label="styleGridColumn"),
    "styleGridColumnAlign": SimTypeInt(signed=False, label="styleGridColumnAlign"),
    "styleGridColumnSpan": SimTypeInt(signed=False, label="styleGridColumnSpan"),
    "styleGridRow": SimTypeInt(signed=False, label="styleGridRow"),
    "styleGridRowAlign": SimTypeInt(signed=False, label="styleGridRowAlign"),
    "styleGridRowSpan": SimTypeInt(signed=False, label="styleGridRowSpan"),
    "styleHyphenateLimitLines": SimTypeInt(signed=False, label="styleHyphenateLimitLines"),
    "styleHyphens": SimTypeInt(signed=False, label="styleHyphens"),
    "styleImeMode": SimTypeInt(signed=False, label="styleImeMode"),
    "styleInitialColor": SimTypeInt(signed=False, label="styleInitialColor"),
    "styleInitialString": SimTypeInt(signed=False, label="styleInitialString"),
    "styleInterpolation": SimTypeInt(signed=False, label="styleInterpolation"),
    "styleJustifyContent": SimTypeInt(signed=False, label="styleJustifyContent"),
    "styleLayoutFlow": SimTypeInt(signed=False, label="styleLayoutFlow"),
    "styleLayoutGridChar": SimTypeInt(signed=False, label="styleLayoutGridChar"),
    "styleLayoutGridLine": SimTypeInt(signed=False, label="styleLayoutGridLine"),
    "styleLayoutGridMode": SimTypeInt(signed=False, label="styleLayoutGridMode"),
    "styleLayoutGridType": SimTypeInt(signed=False, label="styleLayoutGridType"),
    "styleLineBreak": SimTypeInt(signed=False, label="styleLineBreak"),
    "styleListStylePosition": SimTypeInt(signed=False, label="styleListStylePosition"),
    "styleListStyleType": SimTypeInt(signed=False, label="styleListStyleType"),
    "styleMsAnimationDirection": SimTypeInt(signed=False, label="styleMsAnimationDirection"),
    "styleMsAnimationFillMode": SimTypeInt(signed=False, label="styleMsAnimationFillMode"),
    "styleMsAnimationPlayState": SimTypeInt(signed=False, label="styleMsAnimationPlayState"),
    "styleMsContentZoomChaining": SimTypeInt(signed=False, label="styleMsContentZoomChaining"),
    "styleMsContentZoomSnapType": SimTypeInt(signed=False, label="styleMsContentZoomSnapType"),
    "styleMsContentZooming": SimTypeInt(signed=False, label="styleMsContentZooming"),
    "styleMsFlexAlign": SimTypeInt(signed=False, label="styleMsFlexAlign"),
    "styleMsFlexItemAlign": SimTypeInt(signed=False, label="styleMsFlexItemAlign"),
    "styleMsFlexLinePack": SimTypeInt(signed=False, label="styleMsFlexLinePack"),
    "styleMsFlexPack": SimTypeInt(signed=False, label="styleMsFlexPack"),
    "styleMsHighContrastAdjust": SimTypeInt(signed=False, label="styleMsHighContrastAdjust"),
    "styleMsImeAlign": SimTypeInt(signed=False, label="styleMsImeAlign"),
    "styleMsOverflowStyle": SimTypeInt(signed=False, label="styleMsOverflowStyle"),
    "styleMsScrollChaining": SimTypeInt(signed=False, label="styleMsScrollChaining"),
    "styleMsScrollRails": SimTypeInt(signed=False, label="styleMsScrollRails"),
    "styleMsScrollSnapType": SimTypeInt(signed=False, label="styleMsScrollSnapType"),
    "styleMsScrollTranslation": SimTypeInt(signed=False, label="styleMsScrollTranslation"),
    "styleMsTextCombineHorizontal": SimTypeInt(signed=False, label="styleMsTextCombineHorizontal"),
    "styleMsTouchAction": SimTypeInt(signed=False, label="styleMsTouchAction"),
    "styleMsTouchSelect": SimTypeInt(signed=False, label="styleMsTouchSelect"),
    "styleMsUserSelect": SimTypeInt(signed=False, label="styleMsUserSelect"),
    "styleNone": SimTypeInt(signed=False, label="styleNone"),
    "styleNormal": SimTypeInt(signed=False, label="styleNormal"),
    "styleOutlineStyle": SimTypeInt(signed=False, label="styleOutlineStyle"),
    "styleOverflow": SimTypeInt(signed=False, label="styleOverflow"),
    "stylePageBreak": SimTypeInt(signed=False, label="stylePageBreak"),
    "stylePageBreakInside": SimTypeInt(signed=False, label="stylePageBreakInside"),
    "stylePerspectiveOriginX": SimTypeInt(signed=False, label="stylePerspectiveOriginX"),
    "stylePerspectiveOriginY": SimTypeInt(signed=False, label="stylePerspectiveOriginY"),
    "stylePointerEvents": SimTypeInt(signed=False, label="stylePointerEvents"),
    "stylePosition": SimTypeInt(signed=False, label="stylePosition"),
    "styleRubyAlign": SimTypeInt(signed=False, label="styleRubyAlign"),
    "styleRubyOverhang": SimTypeInt(signed=False, label="styleRubyOverhang"),
    "styleRubyPosition": SimTypeInt(signed=False, label="styleRubyPosition"),
    "styleStrokeLinecap": SimTypeInt(signed=False, label="styleStrokeLinecap"),
    "styleStrokeLinejoin": SimTypeInt(signed=False, label="styleStrokeLinejoin"),
    "styleStyleFloat": SimTypeInt(signed=False, label="styleStyleFloat"),
    "styleTableLayout": SimTypeInt(signed=False, label="styleTableLayout"),
    "styleTextAlignLast": SimTypeInt(signed=False, label="styleTextAlignLast"),
    "styleTextAnchor": SimTypeInt(signed=False, label="styleTextAnchor"),
    "styleTextDecoration": SimTypeInt(signed=False, label="styleTextDecoration"),
    "styleTextEffect": SimTypeInt(signed=False, label="styleTextEffect"),
    "styleTextJustify": SimTypeInt(signed=False, label="styleTextJustify"),
    "styleTextJustifyTrim": SimTypeInt(signed=False, label="styleTextJustifyTrim"),
    "styleTextLineThroughStyle": SimTypeInt(signed=False, label="styleTextLineThroughStyle"),
    "styleTextOverflow": SimTypeInt(signed=False, label="styleTextOverflow"),
    "styleTextSizeAdjust": SimTypeInt(signed=False, label="styleTextSizeAdjust"),
    "styleTextTransform": SimTypeInt(signed=False, label="styleTextTransform"),
    "styleTextUnderlinePosition": SimTypeInt(signed=False, label="styleTextUnderlinePosition"),
    "styleTextUnderlineStyle": SimTypeInt(signed=False, label="styleTextUnderlineStyle"),
    "styleTransformOriginX": SimTypeInt(signed=False, label="styleTransformOriginX"),
    "styleTransformOriginY": SimTypeInt(signed=False, label="styleTransformOriginY"),
    "styleTransformStyle": SimTypeInt(signed=False, label="styleTransformStyle"),
    "styleUserZoom": SimTypeInt(signed=False, label="styleUserZoom"),
    "styleVerticalAlign": SimTypeInt(signed=False, label="styleVerticalAlign"),
    "styleViewportSize": SimTypeInt(signed=False, label="styleViewportSize"),
    "styleVisibility": SimTypeInt(signed=False, label="styleVisibility"),
    "styleWebkitAppearance": SimTypeInt(signed=False, label="styleWebkitAppearance"),
    "styleWebkitBoxDirection": SimTypeInt(signed=False, label="styleWebkitBoxDirection"),
    "styleWebkitBoxOrient": SimTypeInt(signed=False, label="styleWebkitBoxOrient"),
    "styleWebkitBoxPack": SimTypeInt(signed=False, label="styleWebkitBoxPack"),
    "styleWhiteSpace": SimTypeInt(signed=False, label="styleWhiteSpace"),
    "styleWidowsOrphans": SimTypeInt(signed=False, label="styleWidowsOrphans"),
    "styleWordBreak": SimTypeInt(signed=False, label="styleWordBreak"),
    "styleWordWrap": SimTypeInt(signed=False, label="styleWordWrap"),
    "styleWrapFlow": SimTypeInt(signed=False, label="styleWrapFlow"),
    "styleWrapThrough": SimTypeInt(signed=False, label="styleWrapThrough"),
    "styleWritingMode": SimTypeInt(signed=False, label="styleWritingMode"),
    "styleZIndex": SimTypeInt(signed=False, label="styleZIndex"),
    "svgAngleType": SimTypeInt(signed=False, label="svgAngleType"),
    "svgChannel": SimTypeInt(signed=False, label="svgChannel"),
    "svgEdgemode": SimTypeInt(signed=False, label="svgEdgemode"),
    "svgExternalResourcesRequired": SimTypeInt(signed=False, label="svgExternalResourcesRequired"),
    "svgFeblendMode": SimTypeInt(signed=False, label="svgFeblendMode"),
    "svgFecolormatrixType": SimTypeInt(signed=False, label="svgFecolormatrixType"),
    "svgFecomponenttransferType": SimTypeInt(signed=False, label="svgFecomponenttransferType"),
    "svgFecompositeOperator": SimTypeInt(signed=False, label="svgFecompositeOperator"),
    "svgFocusable": SimTypeInt(signed=False, label="svgFocusable"),
    "svgLengthType": SimTypeInt(signed=False, label="svgLengthType"),
    "svgMarkerOrient": SimTypeInt(signed=False, label="svgMarkerOrient"),
    "svgMarkerOrientAttribute": SimTypeInt(signed=False, label="svgMarkerOrientAttribute"),
    "svgMarkerUnits": SimTypeInt(signed=False, label="svgMarkerUnits"),
    "svgMorphologyOperator": SimTypeInt(signed=False, label="svgMorphologyOperator"),
    "svgPathSegType": SimTypeInt(signed=False, label="svgPathSegType"),
    "svgPreserveAlpha": SimTypeInt(signed=False, label="svgPreserveAlpha"),
    "svgPreserveAspectMeetOrSliceType": SimTypeInt(signed=False, label="svgPreserveAspectMeetOrSliceType"),
    "svgPreserveAspectRatioAlignType": SimTypeInt(signed=False, label="svgPreserveAspectRatioAlignType"),
    "svgSpreadMethod": SimTypeInt(signed=False, label="svgSpreadMethod"),
    "svgStitchtype": SimTypeInt(signed=False, label="svgStitchtype"),
    "svgTransformType": SimTypeInt(signed=False, label="svgTransformType"),
    "svgTurbulenceType": SimTypeInt(signed=False, label="svgTurbulenceType"),
    "svgUnitTypes": SimTypeInt(signed=False, label="svgUnitTypes"),
    "system_handle_t": SimTypeInt(signed=False, label="system_handle_t"),
    "tACMFORMATDETAILSW": SimStruct(OrderedDict((("cbStruct", SimTypeInt(signed=False, label="UInt32")), ("dwFormatIndex", SimTypeInt(signed=False, label="UInt32")), ("dwFormatTag", SimTypeInt(signed=False, label="UInt32")), ("fdwSupport", SimTypeInt(signed=False, label="UInt32")), ("pwfx", SimTypePointer(SimTypeRef("WAVEFORMATEX", SimStruct), offset=0)), ("cbwfx", SimTypeInt(signed=False, label="UInt32")), ("szFormat", SimTypeArray(SimTypeChar(label="Char"), 128)),)), name="tACMFORMATDETAILSW", pack=False, align=None),
    "tagIMMPID_GUIDLIST_ITEM": SimStruct(OrderedDict((("pguid", SimTypePointer(SimTypeBottom(label="Guid"), offset=0)), ("dwStart", SimTypeInt(signed=False, label="UInt32")), ("dwLast", SimTypeInt(signed=False, label="UInt32")),)), name="tagIMMPID_GUIDLIST_ITEM", pack=False, align=None),
    "tcp_keepalive": SimStruct(OrderedDict((("onoff", SimTypeInt(signed=False, label="UInt32")), ("keepalivetime", SimTypeInt(signed=False, label="UInt32")), ("keepaliveinterval", SimTypeInt(signed=False, label="UInt32")),)), name="tcp_keepalive", pack=False, align=None),
    "tcp_opt_sack_block": SimStruct(OrderedDict((("Left", SimTypeInt(signed=False, label="UInt32")), ("Right", SimTypeInt(signed=False, label="UInt32")),)), name="tcp_opt_sack_block", pack=False, align=None),
    "textDecoration": SimTypeInt(signed=False, label="textDecoration"),
    "textpathMethodtype": SimTypeInt(signed=False, label="textpathMethodtype"),
    "textpathSpacingtype": SimTypeInt(signed=False, label="textpathSpacingtype"),
    "tomConstants": SimTypeInt(signed=False, label="tomConstants"),
    "uCLSSPEC": SimStruct(OrderedDict((("tyspec", SimTypeInt(signed=False, label="UInt32")), ("tagged_union", SimUnion({"clsid": SimTypeBottom(label="Guid"), "pFileExt": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pMimeType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pProgId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ByName": SimTypeRef("_ByName_e__Struct", SimStruct), "ByObjectId": SimTypeRef("_ByObjectId_e__Struct", SimStruct)}, name="<anon>", label="None")),)), name="uCLSSPEC", pack=False, align=None),
    "userBITMAP": SimStruct(OrderedDict((("bmType", SimTypeInt(signed=True, label="Int32")), ("bmWidth", SimTypeInt(signed=True, label="Int32")), ("bmHeight", SimTypeInt(signed=True, label="Int32")), ("bmWidthBytes", SimTypeInt(signed=True, label="Int32")), ("bmPlanes", SimTypeShort(signed=False, label="UInt16")), ("bmBitsPixel", SimTypeShort(signed=False, label="UInt16")), ("cbSize", SimTypeInt(signed=False, label="UInt32")), ("pBuffer", SimTypePointer(SimTypeChar(label="Byte"), offset=0)),)), name="userBITMAP", pack=False, align=None),
    "userCLIPFORMAT": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"dwValue": SimTypeInt(signed=False, label="UInt32"), "pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")),)), name="userCLIPFORMAT", pack=False, align=None),
    "userFLAG_STGMEDIUM": SimStruct(OrderedDict((("ContextFlags", SimTypeInt(signed=True, label="Int32")), ("fPassOwnership", SimTypeInt(signed=True, label="Int32")), ("Stgmed", SimTypeRef("userSTGMEDIUM", SimStruct)),)), name="userFLAG_STGMEDIUM", pack=False, align=None),
    "userHBITMAP": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("userBITMAP", SimStruct), offset=0), "hInproc64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="userHBITMAP", pack=False, align=None),
    "userHENHMETAFILE": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("BYTE_BLOB", SimStruct), offset=0), "hInproc64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="userHENHMETAFILE", pack=False, align=None),
    "userHGLOBAL": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("FLAGGED_BYTE_BLOB", SimStruct), offset=0), "hInproc64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="userHGLOBAL", pack=False, align=None),
    "userHMETAFILE": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("BYTE_BLOB", SimStruct), offset=0), "hInproc64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="userHMETAFILE", pack=False, align=None),
    "userHMETAFILEPICT": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("remoteMETAFILEPICT", SimStruct), offset=0), "hInproc64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="userHMETAFILEPICT", pack=False, align=None),
    "userHPALETTE": SimStruct(OrderedDict((("fContext", SimTypeInt(signed=True, label="Int32")), ("u", SimUnion({"hInproc": SimTypeInt(signed=True, label="Int32"), "hRemote": SimTypePointer(SimTypeRef("LOGPALETTE", SimStruct), offset=0), "hInproc64": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")),)), name="userHPALETTE", pack=False, align=None),
    "userSTGMEDIUM": SimStruct(OrderedDict((("u", SimTypeRef("_STGMEDIUM_UNION", SimStruct)), ("pUnkForRelease", SimTypeBottom(label="IUnknown")),)), name="userSTGMEDIUM", pack=False, align=None),
    "val_context": SimStruct(OrderedDict((("valuelen", SimTypeInt(signed=True, label="Int32")), ("value_context", SimTypePointer(SimTypeBottom(label="Void"), offset=0)), ("val_buff_ptr", SimTypePointer(SimTypeBottom(label="Void"), offset=0)),)), name="val_context", pack=False, align=None),
    "xa_switch_t": SimStruct(OrderedDict((("name", SimTypeArray(SimTypeChar(label="SByte"), 32)), ("flags", SimTypeInt(signed=True, label="Int32")), ("version", SimTypeInt(signed=True, label="Int32")), ("xa_open_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_close_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_start_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_end_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_rollback_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_prepare_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_commit_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_recover_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_forget_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)), ("xa_complete_entry", SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)),)), name="xa_switch_t", pack=False, align=None),
}